/*
** ###################################################################
**     Processors:          MIMXRT1182CVP2B
**                          MIMXRT1182XVP2B
**
**     Compilers:           GNU C Compiler
**                          IAR ANSI C/C++ Compiler for ARM
**                          Keil ARM C/C++ Compiler
**                          MCUXpresso Compiler
**
**     Reference manual:    IMXRT1180RM, Rev 5, 01/2024
**     Version:             rev. 2.0, 2024-01-18
**     Build:               b240417
**
**     Abstract:
**         CMSIS Peripheral Access Layer for MIMXRT1182
**
**     Copyright 1997-2016 Freescale Semiconductor, Inc.
**     Copyright 2016-2024 NXP
**     SPDX-License-Identifier: BSD-3-Clause
**
**     http:                 www.nxp.com
**     mail:                 support@nxp.com
**
**     Revisions:
**     - rev. 1.0 (2021-03-09)
**         Initial version.
**     - rev. 2.0 (2024-01-18)
**         Header RFP.
**
** ###################################################################
*/

/*!
 * @file MIMXRT1182.h
 * @version 2.0
 * @date 2024-01-18
 * @brief CMSIS Peripheral Access Layer for MIMXRT1182
 *
 * CMSIS Peripheral Access Layer for MIMXRT1182
 */

#if !defined(MIMXRT1182_H_)
#define MIMXRT1182_H_                            /**< Symbol preventing repeated inclusion */

/** Memory map major version (memory maps with equal major version number are
 * compatible) */
#define MCU_MEM_MAP_VERSION 0x0200U
/** Memory map minor version */
#define MCU_MEM_MAP_VERSION_MINOR 0x0000U

/* ----------------------------------------------------------------------------
   --
   ---------------------------------------------------------------------------- */

/* XBAR related definitions */
typedef enum
{
    kXBAR_DSC1 = 1,
    kXBAR_DSC2 = 2,
    kXBAR_DSC3 = 3
} xbar_instance_t;
#define XBAR_INFO \
    { \
        { \
            (volatile uint16_t *)XBAR1_BASE, 0x0U, 111U, 0xDEU, 4U \
        }, \
        { \
            (volatile uint16_t *)XBAR2_BASE, 0x0U, 16U, 0x20U, 1U \
        }, \
        { \
            (volatile uint16_t *)XBAR3_BASE, 0x0U, 16U, 0x20U, 1U \
        } \
    }



/* ----------------------------------------------------------------------------
   -- Interrupt vector numbers
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Interrupt_vector_numbers Interrupt vector numbers
 * @{
 */

/** Interrupt Number Definitions */
#define NUMBER_OF_INT_VECTORS 255                /**< Number of interrupts in the Vector table */

typedef enum IRQn {
  /* Auxiliary constants */
  NotAvail_IRQn                = -128,             /**< Not available device specific interrupt */

  /* Core interrupts */
  NonMaskableInt_IRQn          = -14,              /**< Non Maskable Interrupt */
  HardFault_IRQn               = -13,              /**< Cortex-M33 SV Hard Fault Interrupt */
  MemoryManagement_IRQn        = -12,              /**< Cortex-M33 Memory Management Interrupt */
  BusFault_IRQn                = -11,              /**< Cortex-M33 Bus Fault Interrupt */
  UsageFault_IRQn              = -10,              /**< Cortex-M33 Usage Fault Interrupt */
  SecureFault_IRQn             = -9,               /**< Cortex-M33 Secure Fault Interrupt */
  SVCall_IRQn                  = -5,               /**< Cortex-M33 SV Call Interrupt */
  DebugMonitor_IRQn            = -4,               /**< Cortex-M33 Debug Monitor Interrupt */
  PendSV_IRQn                  = -2,               /**< Cortex-M33 Pend SV Interrupt */
  SysTick_IRQn                 = -1,               /**< Cortex-M33 System Tick Interrupt */

  /* Device specific interrupts */
  TMR1_IRQn                    = 0,                /**< TMR1 interrupt */
  DAP_IRQn                     = 1,                /**< DAP interrupt */
  M7_CTI_TRIGGER_OUTPUT_IRQn   = 2,                /**< CTI trigger outputs from CM7 */
  M33_CTI_TRIGGER_OUTPUT_IRQn  = 3,                /**< CTI trigger outputs from CM33 */
  Reserved20_IRQn              = 4,                /**< Reserved interrupt */
  Reserved21_IRQn              = 5,                /**< Reserved interrupt */
  Reserved22_IRQn              = 6,                /**< Reserved interrupt */
  Reserved23_IRQn              = 7,                /**< Reserved interrupt */
  CAN1_IRQn                    = 8,                /**< CAN1 interrupt */
  CAN1_ERROR_IRQn              = 9,                /**< CAN1 error interrupt */
  GPIO1_0_IRQn                 = 10,               /**< GPIO1 interrupt 0 */
  GPIO1_1_IRQn                 = 11,               /**< GPIO1 interrupt 1 */
  I3C1_IRQn                    = 12,               /**< I3C1 interrupt */
  LPI2C1_IRQn                  = 13,               /**< LPI2C1 interrupt */
  LPI2C2_IRQn                  = 14,               /**< LPI2C2 interrupt */
  LPIT1_IRQn                   = 15,               /**< LPIT1 interrupt */
  LPSPI1_IRQn                  = 16,               /**< LPSPI1 interrupt */
  LPSPI2_IRQn                  = 17,               /**< LPSPI2 interrupt */
  LPTMR1_IRQn                  = 18,               /**< LPTMR1 interrupt */
  LPUART1_IRQn                 = 19,               /**< LPUART1 interrupt */
  LPUART2_IRQn                 = 20,               /**< LPUART2 interrupt */
  MU1_IRQn                     = 21,               /**< MU1 interrupt */
  MU2_IRQn                     = 22,               /**< MU2 interrupt */
  PWM1_FAULT_IRQn              = 23,               /**< PWM1 fault or reload error interrupt */
  PWM1_0_IRQn                  = 24,               /**< PWM1 capture 0, compare 0, or reload 0 interrupt */
  PWM1_1_IRQn                  = 25,               /**< PWM1 capture 1, compare 1, or reload 1 interrupt */
  PWM1_2_IRQn                  = 26,               /**< PWM1 capture 2, compare 2, or reload 2 interrupt */
  PWM1_3_IRQn                  = 27,               /**< PWM1 capture 3, compare 3, or reload 3 interrupt */
  EDGELOCK_TRUST_MUA_RX_FULL_IRQn = 28,            /**< Edgelock Trust MUA RX full interrupt */
  EDGELOCK_TRUST_MUA_TX_EMPTY_IRQn = 29,           /**< Edgelock Trust MUA TX empty interrupt */
  EDGELOCK_APPS_CORE_MUA_RX_FULL_IRQn = 30,        /**< Edgelock Apps Core MUA RX full interrupt */
  EDGELOCK_APPS_CORE_MUA_TX_EMPTY_IRQn = 31,       /**< Edgelock Apps Core MUA TX empty interrupt */
  EDGELOCK_REALTIME_CORE_MUA_RX_FULL_IRQn = 32,    /**< Edgelock Realtime Core MUA RX full interrupt */
  EDGELOCK_REALTIME_CORE_MUA_TX_EMPTY_IRQn = 33,   /**< Edgelock Realtime Core MUA TX empty interrupt */
  EDGELOCK_SECURE_IRQn         = 34,               /**< Edgelock secure interrupt */
  EDGELOCK_NONSECURE_IRQn      = 35,               /**< Edgelock non-secure interrupt */
  TPM1_IRQn                    = 36,               /**< TPM1 interrupt */
  TPM2_IRQn                    = 37,               /**< TPM2 interrupt */
  RTWDOG1_IRQn                 = 38,               /**< RTWDOG1 interrupt */
  RTWDOG2_IRQn                 = 39,               /**< RTWDOG2 interrupt */
  TRDC_MGR_AON_IRQn            = 40,               /**< AONMIX TRDC transfer error interrupt */
  PDM_HWVAD_EVENT_IRQn         = 41,               /**< HWVAD event interrupt */
  PDM_HWVAD_ERROR_IRQn         = 42,               /**< HWVAD error interrupt */
  PDM_EVENT_IRQn               = 43,               /**< PDM event interrupt */
  PDM_ERROR_IRQn               = 44,               /**< PDM error interrupt */
  SAI1_IRQn                    = 45,               /**< SAI interrupt */
  CM33_PS_IRQn                 = 46,               /**< M33 PS Tag/Data Parity Error */
  CM33_TCM_ECC_IRQn            = 47,               /**< M33 TCM ECC interrupt */
  CM33_TCM_ERROR_IRQn          = 48,               /**< M33 TCM Error interrupt */
  CM7_TCM_ECC_IRQn             = 49,               /**< M7 TCM ECC interrupt */
  CM7_TCM_ERROR_IRQn           = 50,               /**< M7 TCM Error interrupt */
  CAN2_IRQn                    = 51,               /**< CAN2 interrupt */
  CAN2_ERROR_IRQn              = 52,               /**< CAN2 error interrupt */
  FLEXIO1_IRQn                 = 53,               /**< FLEXIO1 interrupt */
  FLEXIO2_IRQn                 = 54,               /**< FLEXIO2 interrupt */
  FLEXSPI1_IRQn                = 55,               /**< FLEXSPI1 interrupt */
  FLEXSPI2_IRQn                = 56,               /**< FLEXSPI2 interrupt */
  GPIO2_0_IRQn                 = 57,               /**< GPIO2 interrupt 0 */
  GPIO2_1_IRQn                 = 58,               /**< GPIO2 interrupt 1 */
  GPIO3_0_IRQn                 = 59,               /**< GPIO3 interrupt 0 */
  GPIO3_1_IRQn                 = 60,               /**< GPIO3 interrupt 1 */
  I3C2_IRQn                    = 61,               /**< I3C2 interrupt */
  LPI2C3_IRQn                  = 62,               /**< LPI2C3 interrupt */
  Reserved79_IRQn              = 63,               /**< Reserved interrupt */
  LPIT2_IRQn                   = 64,               /**< LPIT2 interrupt */
  LPSPI3_IRQn                  = 65,               /**< LPSPI3 interrupt */
  LPSPI4_IRQn                  = 66,               /**< LPSPI4 interrupt */
  Reserved83_IRQn              = 67,               /**< Reserved interrupt */
  LPUART3_IRQn                 = 68,               /**< LPUART3 interrupt */
  LPUART4_IRQn                 = 69,               /**< LPUART4 interrupt */
  LPUART5_IRQn                 = 70,               /**< LPUART5 interrupt */
  LPUART6_IRQn                 = 71,               /**< LPUART6 interrupt */
  Reserved88_IRQn              = 72,               /**< Reserved interrupt 88 */
  BBNSM_IRQn                   = 73,               /**< BBNSM iterrupt */
  SYS_CTR1_IRQn                = 74,               /**< System Counter compare interrupt 0 and 1 */
  TPM3_IRQn                    = 75,               /**< TPM3 interrupt */
  Reserved92_IRQn              = 76,               /**< Reserved interrupt */
  Reserved93_IRQn              = 77,               /**< Reserved interrupt */
  Reserved94_IRQn              = 78,               /**< Reserved interrupt */
  RTWDOG3_IRQn                 = 79,               /**< RTWDOG3 interrupt */
  RTWDOG4_IRQn                 = 80,               /**< RTWDOG4 interrupt */
  RTWDOG5_IRQn                 = 81,               /**< RTWDOG5 interrupt */
  TRDC_MGR_WKUP_IRQn           = 82,               /**< WAKEUPMIX TRDC transfer error interrupt */
  TMPSNS_INT_IRQn              = 83,               /**< Temperature alarm interrupt */
  BBSM_IRQn                    = 84,               /**< BBSM wakeup alarm interrupt */
  LDO_AON_ANA_IRQn             = 85,               /**< Brown out interrupt */
  USDHC1_IRQn                  = 86,               /**< USDHC1 */
  Reserved103_IRQn             = 87,               /**< Reserved interrupt */
  TRDC_MGR_MEGA_IRQn           = 88,               /**< MEGAMIX TRDC transfer error interrupt */
  SFA_IRQn                     = 89,               /**< Signal Frequency Analyzer interrupt */
  LDO_AON_DIG_IRQn             = 90,               /**< Brown out interrupt */
  MECC1_IRQn                   = 91,               /**< MECC1 interrupt */
  MECC2_IRQn                   = 92,               /**< MECC2 interrupt */
  ADC1_IRQn                    = 93,               /**< ADC1 interrupt */
  DMA_ERROR_IRQn               = 94,               /**< AON Domain eDMA error interrupt */
  DMA3_CH0_IRQn                = 95,               /**< AON Domain eDMA channel 0 interrupt */
  DMA3_CH1_IRQn                = 96,               /**< AON Domain eDMA channel 1 interrupt */
  DMA3_CH2_IRQn                = 97,               /**< AON Domain eDMA channel 2 interrupt */
  DMA3_CH3_IRQn                = 98,               /**< AON Domain eDMA channel 3 interrupt */
  DMA3_CH4_IRQn                = 99,               /**< AON Domain eDMA channel 4 interrupt */
  DMA3_CH5_IRQn                = 100,              /**< AON Domain eDMA channel 5 interrupt */
  DMA3_CH6_IRQn                = 101,              /**< AON Domain eDMA channel 6 interrupt */
  DMA3_CH7_IRQn                = 102,              /**< AON Domain eDMA channel 7 interrupt */
  DMA3_CH8_IRQn                = 103,              /**< AON Domain eDMA channel 8 interrupt */
  DMA3_CH9_IRQn                = 104,              /**< AON Domain eDMA channel 9 interrupt */
  DMA3_CH10_IRQn               = 105,              /**< AON Domain eDMA channel 10 interrupt */
  DMA3_CH11_IRQn               = 106,              /**< AON Domain eDMA channel 11 interrupt */
  DMA3_CH12_IRQn               = 107,              /**< AON Domain eDMA channel 12 interrupt */
  DMA3_CH13_IRQn               = 108,              /**< AON Domain eDMA channel 13 interrupt */
  DMA3_CH14_IRQn               = 109,              /**< AON Domain eDMA channel 14 interrupt */
  DMA3_CH15_IRQn               = 110,              /**< AON Domain eDMA channel 15 interrupt */
  DMA3_CH16_IRQn               = 111,              /**< AON Domain eDMA channel 16 interrupt */
  DMA3_CH17_IRQn               = 112,              /**< AON Domain eDMA channel 17 interrupt */
  DMA3_CH18_IRQn               = 113,              /**< AON Domain eDMA channel 18 interrupt */
  DMA3_CH19_IRQn               = 114,              /**< AON Domain eDMA channel 19 interrupt */
  DMA3_CH20_IRQn               = 115,              /**< AON Domain eDMA channel 20 interrupt */
  DMA3_CH21_IRQn               = 116,              /**< AON Domain eDMA channel 21 interrupt */
  DMA3_CH22_IRQn               = 117,              /**< AON Domain eDMA channel 22 interrupt */
  DMA3_CH23_IRQn               = 118,              /**< AON Domain eDMA channel 23 interrupt */
  DMA3_CH24_IRQn               = 119,              /**< AON Domain eDMA channel 24 interrupt */
  DMA3_CH25_IRQn               = 120,              /**< AON Domain eDMA channel 25 interrupt */
  DMA3_CH26_IRQn               = 121,              /**< AON Domain eDMA channel 26 interrupt */
  DMA3_CH27_IRQn               = 122,              /**< AON Domain eDMA channel 27 interrupt */
  DMA3_CH28_IRQn               = 123,              /**< AON Domain eDMA channel 28 interrupt */
  DMA3_CH29_IRQn               = 124,              /**< AON Domain eDMA channel 29 interrupt */
  DMA3_CH30_IRQn               = 125,              /**< AON Domain eDMA channel 30 interrupt */
  DMA3_CH31_IRQn               = 126,              /**< AON Domain eDMA channel 31 interrupt */
  DMA4_ERROR_IRQn              = 127,              /**< WAKEUP Domain eDMA error interrupt */
  DMA4_CH0_CH1_CH32_CH33_IRQn  = 128,              /**< WAKEUP Domain eDMA channel 0/1/32/33 interrupt */
  DMA4_CH2_CH3_CH34_CH35_IRQn  = 129,              /**< WAKEUP Domain eDMA channel 2/3/34/35 interrupt */
  DMA4_CH4_CH5_CH36_CH37_IRQn  = 130,              /**< WAKEUP Domain eDMA channel 4/5/36/37 interrupt */
  DMA4_CH6_CH7_CH38_CH39_IRQn  = 131,              /**< WAKEUP Domain eDMA channel 6/7/38/39 interrupt */
  DMA4_CH8_CH9_CH40_CH41_IRQn  = 132,              /**< WAKEUP Domain eDMA channel 8/9/40/41 interrupt */
  DMA4_CH10_CH11_CH42_CH43_IRQn = 133,             /**< WAKEUP Domain eDMA channel 10/11/42/43 interrupt */
  DMA4_CH12_CH13_CH44_CH45_IRQn = 134,             /**< WAKEUP Domain eDMA channel 12/13/44/45 interrupt */
  DMA4_CH14_CH15_CH46_CH47_IRQn = 135,             /**< WAKEUP Domain eDMA channel 14/15/46/47 interrupt */
  DMA4_CH16_CH17_CH48_CH49_IRQn = 136,             /**< WAKEUP Domain eDMA channel 16/17/48/49 interrupt */
  DMA4_CH18_CH19_CH50_CH51_IRQn = 137,             /**< WAKEUP Domain eDMA channel 18/19/50/51 interrupt */
  DMA4_CH20_CH21_CH52_CH53_IRQn = 138,             /**< WAKEUP Domain eDMA channel 20/21/52/53 interrupt */
  DMA4_CH22_CH23_CH54_CH55_IRQn = 139,             /**< WAKEUP Domain eDMA channel 22/23/54/55 interrupt */
  DMA4_CH24_CH25_CH56_CH57_IRQn = 140,             /**< WAKEUP Domain eDMA channel 24/25/56/57 interrupt */
  DMA4_CH26_CH27_CH58_CH59_IRQn = 141,             /**< WAKEUP Domain eDMA channel 26/27/58/59 interrupt */
  DMA4_CH28_CH29_CH60_CH61_IRQn = 142,             /**< WAKEUP Domain eDMA channel 28/29/60/61 interrupt */
  DMA4_CH30_CH31_CH62_CH63_IRQn = 143,             /**< WAKEUP Domain eDMA channel 30/31/62/63 interrupt */
  XBAR1_CH0_CH1_IRQn           = 144,              /**< XBAR1 channel 0/1 interrupt */
  XBAR1_CH2_CH3_IRQn           = 145,              /**< XBAR1 channel 2/3 interrupt */
  Reserved162_IRQn             = 146,              /**< Reserved interrupt */
  EWM_IRQn                     = 147,              /**< EWM reset out interrupt */
  Reserved164_IRQn             = 148,              /**< Reserved interrupt */
  LPIT3_IRQn                   = 149,              /**< LPIT3 interrupt */
  Reserved166_IRQn             = 150,              /**< Reserved interrupt */
  TMR4_IRQn                    = 151,              /**< TMR4 interrupt */
  Reserved168_IRQn             = 152,              /**< Reserved interrupt */
  Reserved169_IRQn             = 153,              /**< Reserved interrupt */
  SAI4_IRQn                    = 154,              /**< SAI4 interrupt */
  SPDIF_IRQn                   = 155,              /**< SPDIF interrupt */
  Reserved172_IRQn             = 156,              /**< Reserved interrupt */
  Reserved173_IRQn             = 157,              /**< Reserved interrupt */
  Reserved174_IRQn             = 158,              /**< Reserved interrupt */
  Reserved175_IRQn             = 159,              /**< Reserved interrupt */
  INTG_BOOTROM_DEBUG_CTRL_IRQn = 160,              /**< CM33, CM7, DAP access IRQ */
  EDGELOCK_REQ1_IRQn           = 161,              /**< Edgelock reuqest 1 interrupt */
  EDGELOCK_REQ2_IRQn           = 162,              /**< Edgelock reuqest 2 interrupt */
  EDGELOCK_REQ3_IRQn           = 163,              /**< Edgelock reuqest 3 interrupt */
  TMR3_IRQn                    = 164,              /**< TMR3 interrupt */
  JTAGC_IRQn                   = 165,              /**< JTAGC SRC reset source */
  M33_SYSRESET_REQ_IRQn        = 166,              /**< CM33 SYSREQRST SRC reset source */
  M33_LOCKUP_IRQn              = 167,              /**< CM33 LOCKUP SRC reset source */
  M7_SYSRESET_REQ_IRQn         = 168,              /**< CM33 SYSREQRST SRC reset source */
  M7_LOCKUP_IRQn               = 169,              /**< CM33 LOCKUP SRC reset source */
  PWM2_FAULT_IRQn              = 170,              /**< PWM2 fault or reload error interrupt */
  PWM2_0_IRQn                  = 171,              /**< PWM2 capture 0, compare 0, or reload 0 interrupt */
  PWM2_1_IRQn                  = 172,              /**< PWM2 capture 1, compare 1, or reload 1 interrupt */
  PWM2_2_IRQn                  = 173,              /**< PWM2 capture 2, compare 2, or reload 2 interrupt */
  PWM2_3_IRQn                  = 174,              /**< PWM2 capture 3, compare 3, or reload 3 interrupt */
  Reserved191_IRQn             = 175,              /**< Reserved interrupt */
  Reserved192_IRQn             = 176,              /**< Reserved interrupt */
  Reserved193_IRQn             = 177,              /**< Reserved interrupt */
  Reserved194_IRQn             = 178,              /**< Reserved interrupt */
  Reserved195_IRQn             = 179,              /**< Reserved interrupt */
  Reserved196_IRQn             = 180,              /**< Reserved interrupt */
  Reserved197_IRQn             = 181,              /**< Reserved interrupt */
  Reserved198_IRQn             = 182,              /**< Reserved interrupt */
  Reserved199_IRQn             = 183,              /**< Reserved interrupt */
  Reserved200_IRQn             = 184,              /**< Reserved interrupt */
  EQDC1_IRQn                   = 185,              /**< EQDC1 interrupt */
  EQDC2_IRQn                   = 186,              /**< EQDC2 interrupt */
  Reserved203_IRQn             = 187,              /**< Reserved interrupt */
  Reserved204_IRQn             = 188,              /**< Reserved interrupt */
  Reserved205_IRQn             = 189,              /**< Reserved interrupt */
  DCDC_IRQn                    = 190,              /**< DCDC brown out interrupt */
  Reserved207_IRQn             = 191,              /**< Reserved interrupt */
  Reserved208_IRQn             = 192,              /**< Reserved interrupt */
  DAC_IRQn                     = 193,              /**< DAC interrupt */
  Reserved210_IRQn             = 194,              /**< Reserved interrupt */
  Reserved211_IRQn             = 195,              /**< Reserved interrupt */
  LPUART7_IRQn                 = 196,              /**< LPUART7 interrupt */
  LPUART8_IRQn                 = 197,              /**< LPUART8 interrupt */
  SAI2_IRQn                    = 198,              /**< SAI2 interrupt */
  SAI3_IRQn                    = 199,              /**< SAI3 interrupt */
  ACMP1_IRQn                   = 200,              /**< CMP1 interrupt */
  ACMP2_IRQn                   = 201,              /**< CMP2 interrupt */
  ACMP3_IRQn                   = 202,              /**< CMP3 interrupt */
  ACMP4_IRQn                   = 203,              /**< CMP4 interrupt */
  CM7_PS_IRQn                  = 204,              /**< M7 PS Tag/Data Parity Error */
  CM7_MCM_IRQn                 = 205,              /**< M7 MCM interrupt */
  CM33_MCM_IRQn                = 206,              /**< M33 MCM interrupt */
  ECAT_INT_IRQn                = 207,              /**< EtherCAT interrupt */
  SAFETY_CLK_MON_IRQn          = 208,              /**< Safety clock monitor interrupt */
  GPT1_IRQn                    = 209,              /**< GPT1 interrupt */
  GPT2_IRQn                    = 210,              /**< GPT2 interrupt */
  KPP_IRQn                     = 211,              /**< KPP interrupt */
  Reserved228_IRQn             = 212,              /**< Reserved interrupt */
  Reserved229_IRQn             = 213,              /**< Reserved interrupt */
  Reserved230_IRQn             = 214,              /**< Reserved interrupt */
  Reserved231_IRQn             = 215,              /**< Reserved interrupt */
  FLEXSPI_SLV_IRQn             = 216,              /**< FLEXSPI follower interrupt */
  NETC_IRQn                    = 217,              /**< NETC interrupt */
  MSGINTR1_IRQn                = 218,              /**< MSGINTR1 interrupt */
  MSGINTR2_IRQn                = 219,              /**< MSGINTR2 interrupt */
  MSGINTR3_IRQn                = 220,              /**< MSGINTR3 interrupt */
  MSGINTR4_IRQn                = 221,              /**< MSGINTR4 interrupt */
  MSGINTR5_IRQn                = 222,              /**< MSGINTR5 interrupt */
  MSGINTR6_IRQn                = 223,              /**< MSGINTR6 interrupt */
  Reserved240_IRQn             = 224,              /**< Reserved interrupt */
  Reserved241_IRQn             = 225,              /**< Reserved interrupt */
  Reserved242_IRQn             = 226,              /**< Reserved interrupt */
  Reserved243_IRQn             = 227,              /**< Reserved interrupt */
  Reserved244_IRQn             = 228,              /**< Reserved interrupt */
  Reserved245_IRQn             = 229,              /**< Reserved interrupt */
  Reserved246_IRQn             = 230,              /**< Reserved interrupt */
  Reserved247_IRQn             = 231,              /**< Reserved interrupt */
  GPIO4_IRQn                   = 232,              /**< GPIO4 interrupt */
  TMR2_IRQn                    = 233,              /**< TMR2 interrupt */
  GPIO5_IRQn                   = 234,              /**< GPIO5 interrupt */
  ASRC_IRQn                    = 235,              /**< ASRC interrupt */
  GPIO6_IRQn                   = 236,              /**< GPIO6 interrupt */
  DBG_TRACE_IRQn               = 237,              /**< JTAGSW DAP MDM-AP SRC reset source */
  ECAT_RST_OUT_IRQn            = 238               /**< ECAT reset out interrupt */
} IRQn_Type;

/*!
 * @}
 */ /* end of group Interrupt_vector_numbers */


/* ----------------------------------------------------------------------------
   -- Cortex M33 Core Configuration
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Cortex_Core_Configuration Cortex M33 Core Configuration
 * @{
 */

#define __MPU_PRESENT                  1         /**< Defines if an MPU is present or not */
#define __NVIC_PRIO_BITS               3         /**< Number of priority bits implemented in the NVIC */
#define __Vendor_SysTickConfig         0         /**< Vendor specific implementation of SysTickConfig is defined */
#define __FPU_PRESENT                  1         /**< Defines if an FPU is present or not */
#define __DSP_PRESENT                  1         /**< Defines if Armv8-M Mainline core supports DSP instructions */
#define __SAUREGION_PRESENT            1         /**< Defines if an SAU is present or not */

#include "core_cm33.h"                 /* Core Peripheral Access Layer */
#include "system_MIMXRT1182.h"         /* Device specific configuration file */

/*!
 * @}
 */ /* end of group Cortex_Core_Configuration */


/* ----------------------------------------------------------------------------
   -- Mapping Information
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Mapping_Information Mapping Information
 * @{
 */

/** Mapping Information */
/*!
 * @addtogroup edma_request
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*!
 * @brief Structure for the DMA hardware request
 *
 * Defines the structure for the DMA hardware request collections.
 */
typedef enum _dma_request_source
{
    kDma3RequestMuxReserved0        = 0|0x100U,    /**< Reserved 0 */
    kDma3RequestMuxCAN1             = 1|0x100U,    /**< CAN1 */
    kDma3RequestMuxReserved2        = 2|0x100U,    /**< Reserved 2 */
    kDma3RequestMuxGPIO1Request0    = 3|0x100U,    /**< GPIO1 channel 0 */
    kDma3RequestMuxGPIO1Request1    = 4|0x100U,    /**< GPIO1 channel 1 */
    kDma3RequestMuxI3C1ToBusRequest = 5|0x100U,    /**< I3C1 To-bus Request */
    kDma3RequestMuxI3C1FromBusRequest = 6|0x100U,  /**< I3C1 From-bus Request */
    kDma3RequestMuxLPI2C1Tx         = 7|0x100U,    /**< LPI2C1 TX */
    kDma3RequestMuxLPI2C1Rx         = 8|0x100U,    /**< LPI2C1 RX */
    kDma3RequestMuxLPI2C2Tx         = 9|0x100U,    /**< LPI2C2 TX */
    kDma3RequestMuxLPI2C2Rx         = 10|0x100U,   /**< LPI2C2 RX */
    kDma3RequestMuxLPSPI1Tx         = 11|0x100U,   /**< LPSPI1 TX */
    kDma3RequestMuxLPSPI1Rx         = 12|0x100U,   /**< LPSPI1 RX */
    kDma3RequestMuxLPSPI2Tx         = 13|0x100U,   /**< LPSPI2 TX */
    kDma3RequestMuxLPSPI2Rx         = 14|0x100U,   /**< LPSPI2 RX */
    kDma3RequestMuxLPTMR1Request    = 15|0x100U,   /**< LPTMR1 */
    kDma3RequestMuxLPUART1Tx        = 16|0x100U,   /**< LPUART1 TX */
    kDma3RequestMuxLPUART1Rx        = 17|0x100U,   /**< LPUART1 RX */
    kDma3RequestMuxLPUART2Tx        = 18|0x100U,   /**< LPUART2 TX */
    kDma3RequestMuxLPUART2Rx        = 19|0x100U,   /**< LPUART2 RX */
    kDma3RequestMuxEdgelockRequest  = 20|0x100U,   /**< Edgelock enclave DMA Request */
    kDma3RequestMuxSai1Tx           = 21|0x100U,   /**< SAI1 TX */
    kDma3RequestMuxSai1Rx           = 22|0x100U,   /**< SAI1 RX */
    kDma3RequestMuxTPM1Request0Request2 = 23|0x100U, /**< TPM1 request 0 and request 2 */
    kDma3RequestMuxTPM1Request1Request3 = 24|0x100U, /**< TPM1 request 1 and request 3 */
    kDma3RequestMuxTPM1OverflowRequest = 25|0x100U, /**< TPM1 Overflow request */
    kDma3RequestMuxTPM2Request0Request2 = 26|0x100U, /**< TPM2 request 0 and request 2 */
    kDma3RequestMuxTPM2Request1Request3 = 27|0x100U, /**< TPM2 request 1 and request 3 */
    kDma3RequestMuxTPM2OverflowRequest = 28|0x100U, /**< TPM2 Overflow request */
    kDma3RequestMuxLPUART7Tx        = 29|0x100U,   /**< LPUART7 TX */
    kDma3RequestMuxLPUART7Rx        = 30|0x100U,   /**< LPUART7 RX */
    kDma3RequestMuxFlexSPI2Tx       = 33|0x100U,   /**< FlexSPI2 TX */
    kDma3RequestMuxFlexSPI2Rx       = 34|0x100U,   /**< FlexSPI2 RX */
    kDma3RequestMuxReserved35       = 35|0x100U,   /**< Reserved 35 */
    kDma3RequestMuxReserved36       = 36|0x100U,   /**< Reserved 36 */
    kDma3RequestMuxReserved37       = 37|0x100U,   /**< Reserved 37 */
    kDma4RequestMuxReserved0        = 0|0x200U,    /**< Reserved 0 */
    kDma4RequestMuxCAN2             = 1|0x200U,    /**< CAN2 */
    kDma4RequestMuxGPIO2Request0    = 2|0x200U,    /**< GPIO2 channel 0 */
    kDma4RequestMuxGPIO2Request1    = 3|0x200U,    /**< GPIO2 channel 1 */
    kDma4RequestMuxGPIO3Request0    = 4|0x200U,    /**< GPIO3 channel 0 */
    kDma4RequestMuxGPIO3Request1    = 5|0x200U,    /**< GPIO3 channel 1 */
    kDma4RequestMuxI3C2ToBusRequest = 6|0x200U,    /**< I3C2 To-bus Request */
    kDma4RequestMuxI3C2FromBusRequest = 7|0x200U,  /**< I3C2 From-bus Request */
    kDma4RequestMuxLPI2C3Tx         = 8|0x200U,    /**< LPI2C3 TX */
    kDma4RequestMuxLPI2C3Rx         = 9|0x200U,    /**< LPI2C3 RX */
    kDma4RequestMuxReserved10       = 10|0x200U,   /**< Reserved 10 */
    kDma4RequestMuxReserved11       = 11|0x200U,   /**< Reserved 11 */
    kDma4RequestMuxLPSPI3Tx         = 12|0x200U,   /**< LPSPI3 TX */
    kDma4RequestMuxLPSPI3Rx         = 13|0x200U,   /**< LPSPI3 RX */
    kDma4RequestMuxLPSPI4Tx         = 14|0x200U,   /**< LPSPI4 TX */
    kDma4RequestMuxLPSPI4Rx         = 15|0x200U,   /**< LPSPI4 RX */
    kDma4RequestMuxLPUART3Tx        = 17|0x200U,   /**< LPUART3 TX */
    kDma4RequestMuxLPUART3Rx        = 18|0x200U,   /**< LPUART3 RX */
    kDma4RequestMuxLPUART4Tx        = 19|0x200U,   /**< LPUART4 TX */
    kDma4RequestMuxLPUART4Rx        = 20|0x200U,   /**< LPUART4 RX */
    kDma4RequestMuxLPUART5Tx        = 21|0x200U,   /**< LPUART5 TX */
    kDma4RequestMuxLPUART5Rx        = 22|0x200U,   /**< LPUART5 RX */
    kDma4RequestMuxLPUART6Tx        = 23|0x200U,   /**< LPUART6 TX */
    kDma4RequestMuxLPUART6Rx        = 24|0x200U,   /**< LPUART6 RX */
    kDma4RequestMuxTPM3Request0Request2 = 25|0x200U, /**< TPM3 request 0 and request 2 */
    kDma4RequestMuxTPM3Request1Request3 = 26|0x200U, /**< TPM3 request 1 and request 3 */
    kDma4RequestMuxTPM3OverflowRequest = 27|0x200U, /**< TPM3 Overflow request */
    kDma4RequestMuxFlexIO1Request0  = 37|0x200U,   /**< FlexIO1 Request0 */
    kDma4RequestMuxFlexIO1Request1  = 38|0x200U,   /**< FlexIO1 Request1 */
    kDma4RequestMuxFlexIO1Request2  = 39|0x200U,   /**< FlexIO1 Request2 */
    kDma4RequestMuxFlexIO1Request3  = 40|0x200U,   /**< FlexIO1 Request3 */
    kDma4RequestMuxFlexIO1Request4  = 41|0x200U,   /**< FlexIO1 Request4 */
    kDma4RequestMuxFlexIO1Request5  = 42|0x200U,   /**< FlexIO1 Request5 */
    kDma4RequestMuxFlexIO1Request6  = 43|0x200U,   /**< FlexIO1 Request6 */
    kDma4RequestMuxFlexIO1Request7  = 44|0x200U,   /**< FlexIO1 Request7 */
    kDma4RequestMuxFlexIO2Request0  = 45|0x200U,   /**< FlexIO2 Request0 */
    kDma4RequestMuxFlexIO2Request1  = 46|0x200U,   /**< FlexIO2 Request1 */
    kDma4RequestMuxFlexIO2Request2  = 47|0x200U,   /**< FlexIO2 Request2 */
    kDma4RequestMuxFlexIO2Request3  = 48|0x200U,   /**< FlexIO2 Request3 */
    kDma4RequestMuxFlexIO2Request4  = 49|0x200U,   /**< FlexIO2 Request4 */
    kDma4RequestMuxFlexIO2Request5  = 50|0x200U,   /**< FlexIO2 Request5 */
    kDma4RequestMuxFlexIO2Request6  = 51|0x200U,   /**< FlexIO2 Request6 */
    kDma4RequestMuxFlexIO2Request7  = 52|0x200U,   /**< FlexIO2 Request7 */
    kDma4RequestMuxFlexSPI1Tx       = 53|0x200U,   /**< FlexSPI1 TX */
    kDma4RequestMuxFlexSPI1Rx       = 54|0x200U,   /**< FlexSPI1 RX */
    kDma4RequestMuxReserved55       = 55|0x200U,   /**< Reserved 55 */
    kDma4RequestMuxReserved56       = 56|0x200U,   /**< Reserved 56 */
    kDma4RequestMuxADC1Request0     = 57|0x200U,   /**< ADC1 Request 0 */
    kDma4RequestMuxFlexPWM1CaptureSub0 = 58|0x200U, /**< FlexPWM1 Capture sub-module0 */
    kDma4RequestMuxFlexPWM1CaptureSub1 = 59|0x200U, /**< FlexPWM1 Capture sub-module1 */
    kDma4RequestMuxFlexPWM1CaptureSub2 = 60|0x200U, /**< FlexPWM1 Capture sub-module2 */
    kDma4RequestMuxFlexPWM1CaptureSub3 = 61|0x200U, /**< FlexPWM1 Capture sub-module3 */
    kDma4RequestMuxFlexPWM1ValueSub0 = 62|0x200U,  /**< FlexPWM1 Value sub-module 0 */
    kDma4RequestMuxFlexPWM1ValueSub1 = 63|0x200U,  /**< FlexPWM1 Value sub-module 1 */
    kDma4RequestMuxFlexPWM1ValueSub2 = 64|0x200U,  /**< FlexPWM1 Value sub-module 2 */
    kDma4RequestMuxFlexPWM1ValueSub3 = 65|0x200U,  /**< FlexPWM1 Value sub-module 3 */
    kDma4RequestMuxFlexPWM2CaptureSub0 = 66|0x200U, /**< FlexPWM2 Capture sub-module0 */
    kDma4RequestMuxFlexPWM2CaptureSub1 = 67|0x200U, /**< FlexPWM2 Capture sub-module1 */
    kDma4RequestMuxFlexPWM2CaptureSub2 = 68|0x200U, /**< FlexPWM2 Capture sub-module2 */
    kDma4RequestMuxFlexPWM2CaptureSub3 = 69|0x200U, /**< FlexPWM2 Capture sub-module3 */
    kDma4RequestMuxFlexPWM2ValueSub0 = 70|0x200U,  /**< FlexPWM2 Value sub-module 0 */
    kDma4RequestMuxFlexPWM2ValueSub1 = 71|0x200U,  /**< FlexPWM2 Value sub-module 1 */
    kDma4RequestMuxFlexPWM2ValueSub2 = 72|0x200U,  /**< FlexPWM2 Value sub-module 2 */
    kDma4RequestMuxFlexPWM2ValueSub3 = 73|0x200U,  /**< FlexPWM2 Value sub-module 3 */
    kDma4RequestMuxQTIMER1CaptTimer0 = 90|0x200U,  /**< TMR1 Capture timer 0 */
    kDma4RequestMuxQTIMER1CaptTimer1 = 91|0x200U,  /**< TMR1 Capture timer 1 */
    kDma4RequestMuxQTIMER1CaptTimer2 = 92|0x200U,  /**< TMR1 Capture timer 2 */
    kDma4RequestMuxQTIMER1CaptTimer3 = 93|0x200U,  /**< TMR1 Capture timer 3 */
    kDma4RequestMuxQTIMER1Cmpld1Timer0Cmpld2Timer1 = 94|0x200U, /**< TMR1 cmpld1 in timer 0 or cmpld2 in timer 1 */
    kDma4RequestMuxQTIMER1Cmpld1Timer1Cmpld2Timer0 = 95|0x200U, /**< TMR1 cmpld1 in timer 1 or cmpld2 in timer 0 */
    kDma4RequestMuxQTIMER1Cmpld1Timer2Cmpld2Timer3 = 96|0x200U, /**< TMR1 cmpld1 in timer 2 or cmpld2 in timer 3 */
    kDma4RequestMuxQTIMER1Cmpld1Timer3Cmpld2Timer2 = 97|0x200U, /**< TMR1 cmpld1 in timer 3 or cmpld2 in timer 2 */
    kDma4RequestMuxQTIMER2CaptTimer0 = 98|0x200U,  /**< TMR2 Capture timer 0 */
    kDma4RequestMuxQTIMER2CaptTimer1 = 99|0x200U,  /**< TMR2 Capture timer 1 */
    kDma4RequestMuxQTIMER2CaptTimer2 = 100|0x200U, /**< TMR2 Capture timer 2 */
    kDma4RequestMuxQTIMER2CaptTimer3 = 101|0x200U, /**< TMR2 Capture timer 3 */
    kDma4RequestMuxQTIMER2Cmpld1Timer0Cmpld2Timer1 = 102|0x200U, /**< TMR2 cmpld1 in timer 0 or cmpld2 in timer 1 */
    kDma4RequestMuxQTIMER2Cmpld1Timer1Cmpld2Timer0 = 103|0x200U, /**< TMR2 cmpld1 in timer 1 or cmpld2 in timer 0 */
    kDma4RequestMuxQTIMER2Cmpld1Timer2Cmpld2Timer3 = 104|0x200U, /**< TMR2 cmpld1 in timer 2 or cmpld2 in timer 3 */
    kDma4RequestMuxQTIMER2Cmpld1Timer3Cmpld2Timer2 = 105|0x200U, /**< TMR2 cmpld1 in timer 3 or cmpld2 in timer 2 */
    kDma4RequestMuxQTIMER3CaptTimer0 = 106|0x200U, /**< TMR3 Capture timer 0 */
    kDma4RequestMuxQTIMER3CaptTimer1 = 107|0x200U, /**< TMR3 Capture timer 1 */
    kDma4RequestMuxQTIMER3CaptTimer2 = 108|0x200U, /**< TMR3 Capture timer 2 */
    kDma4RequestMuxQTIMER3CaptTimer3 = 109|0x200U, /**< TMR3 Capture timer 3 */
    kDma4RequestMuxQTIMER3Cmpld1Timer0Cmpld2Timer1 = 110|0x200U, /**< TMR3 cmpld1 in timer 0 or cmpld2 in timer 1 */
    kDma4RequestMuxQTIMER3Cmpld1Timer1Cmpld2Timer0 = 111|0x200U, /**< TMR3 cmpld1 in timer 1 or cmpld2 in timer 0 */
    kDma4RequestMuxQTIMER3Cmpld1Timer2Cmpld2Timer3 = 112|0x200U, /**< TMR3 cmpld1 in timer 2 or cmpld2 in timer 3 */
    kDma4RequestMuxQTIMER3Cmpld1Timer3Cmpld2Timer2 = 113|0x200U, /**< TMR3 cmpld1 in timer 3 or cmpld2 in timer 2 */
    kDma4RequestMuxQTIMER4CaptTimer0 = 114|0x200U, /**< TMR4 Capture timer 0 */
    kDma4RequestMuxQTIMER4CaptTimer1 = 115|0x200U, /**< TMR4 Capture timer 1 */
    kDma4RequestMuxQTIMER4CaptTimer2 = 116|0x200U, /**< TMR4 Capture timer 2 */
    kDma4RequestMuxQTIMER4CaptTimer3 = 117|0x200U, /**< TMR4 Capture timer 3 */
    kDma4RequestMuxQTIMER4Cmpld1Timer0Cmpld2Timer1 = 118|0x200U, /**< TMR4 cmpld1 in timer 0 or cmpld2 in timer 1 */
    kDma4RequestMuxQTIMER4Cmpld1Timer1Cmpld2Timer0 = 119|0x200U, /**< TMR4 cmpld1 in timer 1 or cmpld2 in timer 0 */
    kDma4RequestMuxQTIMER4Cmpld1Timer2Cmpld2Timer3 = 120|0x200U, /**< TMR4 cmpld1 in timer 2 or cmpld2 in timer 3 */
    kDma4RequestMuxQTIMER4Cmpld1Timer3Cmpld2Timer2 = 121|0x200U, /**< TMR4 cmpld1 in timer 3 or cmpld2 in timer 2 */
    kDma4RequestMuxXBAR1Request0    = 154|0x200U,  /**< XBAR1 Request 0 */
    kDma4RequestMuxXBAR1Request1    = 155|0x200U,  /**< XBAR1 Request 1 */
    kDma4RequestMuxXBAR1Request2    = 156|0x200U,  /**< XBAR1 Request 2 */
    kDma4RequestMuxXBAR1Request3    = 157|0x200U,  /**< XBAR1 Request 3 */
    kDma4RequestMuxEQDC1            = 159|0x200U,  /**< EQDC1 */
    kDma4RequestMuxEQDC2            = 160|0x200U,  /**< EQDC2 */
    kDma4RequestMuxReserved163      = 163|0x200U,  /**< Reserved 163 */
    kDma4RequestMuxReserved164      = 164|0x200U,  /**< Reserved 164 */
    kDma4RequestMuxLPUART8Tx        = 178|0x200U,  /**< LPUART8 TX */
    kDma4RequestMuxLPUART8Rx        = 179|0x200U,  /**< LPUART8 RX */
    kDma4RequestMuxSai2Tx           = 180|0x200U,  /**< SAI2 TX */
    kDma4RequestMuxSai2Rx           = 181|0x200U,  /**< SAI2 RX */
    kDma4RequestMuxSai3Tx           = 182|0x200U,  /**< SAI3 TX */
    kDma4RequestMuxSai3Rx           = 183|0x200U,  /**< SAI3 RX */
    kDma4RequestMuxSai4Tx           = 184|0x200U,  /**< SAI4 TX */
    kDma4RequestMuxSai4Rx           = 185|0x200U,  /**< SAI4 RX */
    kDma4RequestMuxDAC              = 186|0x200U,  /**< DAC */
    kDma4RequestMuxCMP1             = 187|0x200U,  /**< CMP1 */
    kDma4RequestMuxCMP2             = 188|0x200U,  /**< CMP2 */
    kDma4RequestMuxCMP3             = 189|0x200U,  /**< CMP3 */
    kDma4RequestMuxCMP4             = 190|0x200U,  /**< CMP4 */
    kDma4RequestMuxASRCRequest1     = 191|0x200U,  /**< ASRC request 1 pair A input request */
    kDma4RequestMuxASRCRequest2     = 192|0x200U,  /**< ASRC request 2 pair B input request */
    kDma4RequestMuxASRCRequest3     = 193|0x200U,  /**< ASRC request 3 pair C input request */
    kDma4RequestMuxASRCRequest4     = 194|0x200U,  /**< ASRC request 4 pair A output request */
    kDma4RequestMuxASRCRequest5     = 195|0x200U,  /**< ASRC request 5 pair B output request */
    kDma4RequestMuxASRCRequest6     = 196|0x200U,  /**< ASRC request 6 pair C output request */
    kDma4RequestMuxSpdifRx          = 197|0x200U,  /**< SPDIF RX */
    kDma4RequestMuxSpdifTx          = 198|0x200U,  /**< SPDIF TX */
    kDma4RequestMuxPdmRx            = 199|0x200U,  /**< PDM */
    kDma4RequestMuxGPIO4Request0    = 200|0x200U,  /**< GPIO4 channel 0 */
    kDma4RequestMuxGPIO4Request1    = 201|0x200U,  /**< GPIO4 channel 1 */
    kDma4RequestMuxGPIO5Request0    = 202|0x200U,  /**< GPIO5 channel 0 */
    kDma4RequestMuxGPIO5Request1    = 203|0x200U,  /**< GPIO5 channel 1 */
    kDma4RequestMuxGPIO6Request0    = 204|0x200U,  /**< GPIO6 channel 0 */
    kDma4RequestMuxGPIO6Request1    = 205|0x200U,  /**< GPIO6 channel 1 */
    kDma4RequestMuxReserved206      = 206|0x200U,  /**< Reserved 206 */
    kDma4RequestMuxReserved207      = 207|0x200U,  /**< Reserved 207 */
    kDma4RequestMuxADC1Request1     = 220|0x200U,  /**< ADC1 Request 1 */
} dma_request_source_t;

/* @} */

/*!
 * @addtogroup trdc1_master_mapping
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*!
 * @brief Enumeration for trdc master 1 mapping
 * Defines the enumeration for trdc master 1 resource collections.
 */
typedef enum _trdc1_master
{
    kTRDC1_MasterReserved0          = 0U,          /**< Reserved */
    kTRDC1_MasterCM33               = 1U,          /**< CM33 */
    kTRDC1_MasterDMA3               = 2U,          /**< DMA3 */
    kTRDC1_MasterReserved1          = 3U,          /**< Reserved */
} trdc1_master_t;

/* @} */

/*!
 * @addtogroup trdc2_master_mapping
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*!
 * @brief Enumeration for trdc master 2 mapping
 * Defines the enumeration for trdc master 2 resource collections.
 */
typedef enum _trdc2_master
{
    kTRDC2_MasterCM7AHBP            = 0U,          /**< CM7 AHBP */
    kTRDC2_MasterCM7AXI             = 1U,          /**< CM7 AXI */
    kTRDC2_MasterDAP                = 2U,          /**< DAP AHB_AP_SYS */
    kTRDC2_MasterCoreSight          = 3U,          /**< CoreSight */
    kTRDC2_MasterDMA4               = 4U,          /**< DMA4 */
    kTRDC2_MasterNETC               = 5U,          /**< NETC */
} trdc2_master_t;

/* @} */

/*!
 * @addtogroup trdc3_master_mapping
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*!
 * @brief Enumeration for trdc master 3 mapping
 * Defines the enumeration for trdc master 3 resource collections.
 */
typedef enum _trdc3_master
{
    kTRDC3_MasterUSDHC1             = 0U,          /**< uSDHC1 */
    kTRDC3_MasterUSDHC2             = 1U,          /**< uSDHC2 */
    kTRDC3_MasterReserved0          = 2U,          /**< Reserved0 */
    kTRDC3_MasterUsb                = 3U,          /**< USB */
    kTRDC3_MasterFlexspiFlr         = 4U,          /**< FlexSPI_FLR */
} trdc3_master_t;

/* @} */

/*!
 * @addtogroup iomuxc_pads
 * @{ */

/*******************************************************************************
 * Definitions
*******************************************************************************/

/*!
 * @brief Enumeration for the IOMUXC SW_MUX_CTL_PAD
 *
 * Defines the enumeration for the IOMUXC SW_MUX_CTL_PAD collections.
 */
typedef enum _iomuxc_sw_mux_ctl_pad
{
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_00 = 0U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_01 = 1U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_02 = 2U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_03 = 3U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_04 = 4U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_05 = 5U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_06 = 6U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_07 = 7U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_08 = 8U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_09 = 9U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_10 = 10U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_11 = 11U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_12 = 12U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_13 = 13U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_14 = 14U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_15 = 15U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_16 = 16U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_17 = 17U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_18 = 18U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_19 = 19U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_20 = 20U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_21 = 21U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_22 = 22U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_23 = 23U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_24 = 24U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_25 = 25U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_26 = 26U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_27 = 27U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_28 = 28U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_29 = 29U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_30 = 30U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_31 = 31U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_32 = 32U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_33 = 33U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_34 = 34U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_35 = 35U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_36 = 36U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_37 = 37U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_38 = 38U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_39 = 39U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_40 = 40U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B1_41 = 41U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_00 = 42U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_01 = 43U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_02 = 44U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_03 = 45U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_04 = 46U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_05 = 47U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_06 = 48U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_07 = 49U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_08 = 50U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_09 = 51U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_10 = 52U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_11 = 53U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_12 = 54U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_13 = 55U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_14 = 56U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_15 = 57U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_16 = 58U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_17 = 59U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_18 = 60U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_19 = 61U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_EMC_B2_20 = 62U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_00 = 63U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_01 = 64U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_02 = 65U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_03 = 66U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_04 = 67U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_05 = 68U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_06 = 69U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_07 = 70U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_08 = 71U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_09 = 72U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_10 = 73U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_11 = 74U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_12 = 75U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_13 = 76U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_14 = 77U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_15 = 78U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_16 = 79U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_17 = 80U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_18 = 81U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_19 = 82U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_20 = 83U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_21 = 84U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_22 = 85U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_23 = 86U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_24 = 87U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_25 = 88U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_26 = 89U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_27 = 90U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_28 = 91U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_29 = 92U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_30 = 93U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_31 = 94U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_32 = 95U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_33 = 96U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_34 = 97U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_AD_35 = 98U,       /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B1_00 = 99U,    /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B1_01 = 100U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B1_02 = 101U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B1_03 = 102U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B1_04 = 103U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B1_05 = 104U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_00 = 105U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_01 = 106U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_02 = 107U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_03 = 108U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_04 = 109U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_05 = 110U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_06 = 111U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_07 = 112U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_08 = 113U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_09 = 114U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_10 = 115U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_11 = 116U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B2_12_DUMMY = 117U, /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_00 = 118U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_01 = 119U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_02 = 120U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_03 = 121U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_04 = 122U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_05 = 123U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_06 = 124U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_07 = 125U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_08 = 126U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_09 = 127U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_10 = 128U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_11 = 129U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_12 = 130U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B1_13 = 131U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_00 = 132U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_01 = 133U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_02 = 134U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_03 = 135U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_04 = 136U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_05 = 137U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_06 = 138U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_07 = 139U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_08 = 140U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_09 = 141U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_10 = 142U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_11 = 143U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_12 = 144U,      /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_SW_MUX_CTL_PAD_GPIO_B2_13 = 145U,      /**< IOMUXC SW_MUX_CTL_PAD index */
} iomuxc_sw_mux_ctl_pad_t;

/* @} */

/*!
 * @addtogroup iomuxc_pads
 * @{ */

/*******************************************************************************
 * Definitions
*******************************************************************************/

/*!
 * @brief Enumeration for the IOMUXC SW_PAD_CTL_PAD
 *
 * Defines the enumeration for the IOMUXC SW_PAD_CTL_PAD collections.
 */
typedef enum _iomuxc_sw_pad_ctl_pad
{
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_00 = 0U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_01 = 1U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_02 = 2U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_03 = 3U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_04 = 4U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_05 = 5U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_06 = 6U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_07 = 7U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_08 = 8U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_09 = 9U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_10 = 10U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_11 = 11U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_12 = 12U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_13 = 13U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_14 = 14U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_15 = 15U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_16 = 16U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_17 = 17U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_18 = 18U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_19 = 19U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_20 = 20U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_21 = 21U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_22 = 22U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_23 = 23U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_24 = 24U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_25 = 25U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_26 = 26U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_27 = 27U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_28 = 28U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_29 = 29U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_30 = 30U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_31 = 31U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_32 = 32U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_33 = 33U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_34 = 34U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_35 = 35U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_36 = 36U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_37 = 37U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_38 = 38U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_39 = 39U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_40 = 40U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B1_41 = 41U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_00 = 42U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_01 = 43U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_02 = 44U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_03 = 45U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_04 = 46U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_05 = 47U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_06 = 48U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_07 = 49U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_08 = 50U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_09 = 51U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_10 = 52U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_11 = 53U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_12 = 54U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_13 = 55U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_14 = 56U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_15 = 57U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_16 = 58U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_17 = 59U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_18 = 60U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_19 = 61U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_EMC_B2_20 = 62U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_00 = 63U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_01 = 64U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_02 = 65U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_03 = 66U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_04 = 67U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_05 = 68U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_06 = 69U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_07 = 70U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_08 = 71U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_09 = 72U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_10 = 73U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_11 = 74U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_12 = 75U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_13 = 76U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_14 = 77U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_15 = 78U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_16 = 79U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_17 = 80U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_18 = 81U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_19 = 82U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_20 = 83U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_21 = 84U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_22 = 85U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_23 = 86U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_24 = 87U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_25 = 88U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_26 = 89U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_27 = 90U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_28 = 91U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_29 = 92U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_30 = 93U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_31 = 94U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_32 = 95U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_33 = 96U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_34 = 97U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_AD_35 = 98U,       /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B1_00 = 99U,    /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B1_01 = 100U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B1_02 = 101U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B1_03 = 102U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B1_04 = 103U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B1_05 = 104U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_00 = 105U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_01 = 106U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_02 = 107U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_03 = 108U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_04 = 109U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_05 = 110U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_06 = 111U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_07 = 112U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_08 = 113U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_09 = 114U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_10 = 115U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_11 = 116U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B2_12_DUMMY = 117U, /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_00 = 118U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_01 = 119U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_02 = 120U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_03 = 121U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_04 = 122U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_05 = 123U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_06 = 124U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_07 = 125U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_08 = 126U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_09 = 127U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_10 = 128U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_11 = 129U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_12 = 130U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B1_13 = 131U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_00 = 132U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_01 = 133U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_02 = 134U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_03 = 135U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_04 = 136U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_05 = 137U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_06 = 138U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_07 = 139U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_08 = 140U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_09 = 141U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_10 = 142U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_11 = 143U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_12 = 144U,      /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_SW_PAD_CTL_PAD_GPIO_B2_13 = 145U,      /**< IOMUXC SW_PAD_CTL_PAD index */
} iomuxc_sw_pad_ctl_pad_t;

/* @} */

/*!
 * @brief Enumeration for the IOMUXC select input
 *
 * Defines the enumeration for the IOMUXC select input collections.
 */
typedef enum _iomuxc_select_input
{
    kIOMUXC_CAN1_IPP_IND_CANRX_SELECT_INPUT = 0U,  /**< IOMUXC select input index */
    kIOMUXC_CAN2_IPP_IND_CANRX_SELECT_INPUT = 1U,  /**< IOMUXC select input index */
    kIOMUXC_CAN3_IPP_IND_CANRX_SELECT_INPUT = 2U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_CLK_0_SELECT_INPUT = 3U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_CLK_1_SELECT_INPUT = 4U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA0_0_SELECT_INPUT = 5U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA0_1_SELECT_INPUT = 6U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA1_0_SELECT_INPUT = 7U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA1_1_SELECT_INPUT = 8U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA2_0_SELECT_INPUT = 9U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA2_1_SELECT_INPUT = 10U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA3_0_SELECT_INPUT = 11U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DATA3_1_SELECT_INPUT = 12U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DV_0_SELECT_INPUT = 13U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_DV_1_SELECT_INPUT = 14U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_ER_0_SELECT_INPUT = 15U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_RX_ER_1_SELECT_INPUT = 16U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_TX_CLK_0_SELECT_INPUT = 17U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_ECAT_TX_CLK_1_SELECT_INPUT = 18U, /**< IOMUXC select input index */
    kIOMUXC_ECAT_MDIO_DATA_IN_SELECT_INPUT = 19U,  /**< IOMUXC select input index */
    kIOMUXC_ECAT_PROM_DATA_IN_SELECT_INPUT = 20U,  /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM1_IPP_IND_PWMA_SELECT_INPUT_0 = 21U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM1_IPP_IND_PWMA_SELECT_INPUT_1 = 22U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM1_IPP_IND_PWMA_SELECT_INPUT_2 = 23U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM1_IPP_IND_PWMB_SELECT_INPUT_0 = 24U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM1_IPP_IND_PWMB_SELECT_INPUT_1 = 25U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM1_IPP_IND_PWMB_SELECT_INPUT_2 = 26U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM2_IPP_IND_PWMA_SELECT_INPUT_0 = 27U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM2_IPP_IND_PWMA_SELECT_INPUT_1 = 28U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM2_IPP_IND_PWMA_SELECT_INPUT_2 = 29U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM2_IPP_IND_PWMB_SELECT_INPUT_0 = 30U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM2_IPP_IND_PWMB_SELECT_INPUT_1 = 31U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM2_IPP_IND_PWMB_SELECT_INPUT_2 = 32U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMA_SELECT_INPUT_0 = 33U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMA_SELECT_INPUT_1 = 34U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMA_SELECT_INPUT_2 = 35U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMA_SELECT_INPUT_3 = 36U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMB_SELECT_INPUT_0 = 37U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMB_SELECT_INPUT_1 = 38U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMB_SELECT_INPUT_2 = 39U, /**< IOMUXC select input index */
    kIOMUXC_FLEXPWM3_IPP_IND_PWMB_SELECT_INPUT_3 = 40U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_DQS_FA_SELECT_INPUT = 41U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_DQS_FB_SELECT_INPUT = 42U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT0_SELECT_INPUT = 43U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT1_SELECT_INPUT = 44U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT2_SELECT_INPUT = 45U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT3_SELECT_INPUT = 46U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT4_SELECT_INPUT = 47U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT5_SELECT_INPUT = 48U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT6_SELECT_INPUT = 49U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_IO_FB_BIT7_SELECT_INPUT = 50U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI1_BUS2BIT_IPP_IND_SCK_FB_SELECT_INPUT = 51U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_DQS_FA_SELECT_INPUT = 52U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_DQS_FB_SELECT_INPUT = 53U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FA_BIT0_SELECT_INPUT = 54U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FA_BIT1_SELECT_INPUT = 55U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FA_BIT2_SELECT_INPUT = 56U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FA_BIT3_SELECT_INPUT = 57U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FB_BIT0_SELECT_INPUT = 58U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FB_BIT1_SELECT_INPUT = 59U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FB_BIT2_SELECT_INPUT = 60U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_IO_FB_BIT3_SELECT_INPUT = 61U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_SCK_FA_SELECT_INPUT = 62U, /**< IOMUXC select input index */
    kIOMUXC_FLEXSPI2_BUS2BIT_IPP_IND_SCK_FB_SELECT_INPUT = 63U, /**< IOMUXC select input index */
    kIOMUXC_I3C2_PIN_SCL_IN_SELECT_INPUT = 64U,    /**< IOMUXC select input index */
    kIOMUXC_I3C2_PIN_SDA_IN_SELECT_INPUT = 65U,    /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_0 = 66U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_1 = 67U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_2 = 68U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_3 = 69U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_4 = 70U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_5 = 71U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_6 = 72U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_COL_SELECT_INPUT_7 = 73U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_0 = 74U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_1 = 75U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_2 = 76U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_3 = 77U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_4 = 78U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_5 = 79U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_6 = 80U,  /**< IOMUXC select input index */
    kIOMUXC_KPP_IPP_IND_ROW_SELECT_INPUT_7 = 81U,  /**< IOMUXC select input index */
    kIOMUXC_LPI2C3_IPP_IND_LPI2C_SCL_SELECT_INPUT = 82U, /**< IOMUXC select input index */
    kIOMUXC_LPI2C3_IPP_IND_LPI2C_SDA_SELECT_INPUT = 83U, /**< IOMUXC select input index */
    kIOMUXC_LPI2C4_IPP_IND_LPI2C_SCL_SELECT_INPUT = 84U, /**< IOMUXC select input index */
    kIOMUXC_LPI2C4_IPP_IND_LPI2C_SDA_SELECT_INPUT = 85U, /**< IOMUXC select input index */
    kIOMUXC_LPI2C5_IPP_IND_LPI2C_SCL_SELECT_INPUT = 86U, /**< IOMUXC select input index */
    kIOMUXC_LPI2C5_IPP_IND_LPI2C_SDA_SELECT_INPUT = 87U, /**< IOMUXC select input index */
    kIOMUXC_LPI2C6_IPP_IND_LPI2C_SCL_SELECT_INPUT = 88U, /**< IOMUXC select input index */
    kIOMUXC_LPI2C6_IPP_IND_LPI2C_SDA_SELECT_INPUT = 89U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI3_IPP_IND_LPSPI_PCS_SELECT_INPUT_0 = 90U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI3_IPP_IND_LPSPI_PCS_SELECT_INPUT_1 = 91U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI3_IPP_IND_LPSPI_PCS_SELECT_INPUT_2 = 92U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI3_IPP_IND_LPSPI_PCS_SELECT_INPUT_3 = 93U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI3_IPP_IND_LPSPI_SCK_SELECT_INPUT = 94U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI3_IPP_IND_LPSPI_SDI_SELECT_INPUT = 95U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI3_IPP_IND_LPSPI_SDO_SELECT_INPUT = 96U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI4_IPP_IND_LPSPI_PCS_SELECT_INPUT_0 = 97U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI4_IPP_IND_LPSPI_SCK_SELECT_INPUT = 98U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI4_IPP_IND_LPSPI_SDI_SELECT_INPUT = 99U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI4_IPP_IND_LPSPI_SDO_SELECT_INPUT = 100U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI5_IPP_IND_LPSPI_PCS_SELECT_INPUT_0 = 101U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI5_IPP_IND_LPSPI_PCS_SELECT_INPUT_1 = 102U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI5_IPP_IND_LPSPI_PCS_SELECT_INPUT_2 = 103U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI5_IPP_IND_LPSPI_PCS_SELECT_INPUT_3 = 104U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI5_IPP_IND_LPSPI_SCK_SELECT_INPUT = 105U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI5_IPP_IND_LPSPI_SDI_SELECT_INPUT = 106U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI5_IPP_IND_LPSPI_SDO_SELECT_INPUT = 107U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI6_IPP_IND_LPSPI_PCS_SELECT_INPUT_0 = 108U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI6_IPP_IND_LPSPI_PCS_SELECT_INPUT_1 = 109U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI6_IPP_IND_LPSPI_PCS_SELECT_INPUT_2 = 110U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI6_IPP_IND_LPSPI_PCS_SELECT_INPUT_3 = 111U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI6_IPP_IND_LPSPI_SCK_SELECT_INPUT = 112U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI6_IPP_IND_LPSPI_SDI_SELECT_INPUT = 113U, /**< IOMUXC select input index */
    kIOMUXC_LPSPI6_IPP_IND_LPSPI_SDO_SELECT_INPUT = 114U, /**< IOMUXC select input index */
    kIOMUXC_LPUART10_IPP_IND_LPUART_RXD_SELECT_INPUT = 115U, /**< IOMUXC select input index */
    kIOMUXC_LPUART10_IPP_IND_LPUART_TXD_SELECT_INPUT = 116U, /**< IOMUXC select input index */
    kIOMUXC_LPUART11_IPP_IND_LPUART_RXD_SELECT_INPUT = 117U, /**< IOMUXC select input index */
    kIOMUXC_LPUART11_IPP_IND_LPUART_TXD_SELECT_INPUT = 118U, /**< IOMUXC select input index */
    kIOMUXC_LPUART3_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 119U, /**< IOMUXC select input index */
    kIOMUXC_LPUART3_IPP_IND_LPUART_RXD_SELECT_INPUT = 120U, /**< IOMUXC select input index */
    kIOMUXC_LPUART3_IPP_IND_LPUART_TXD_SELECT_INPUT = 121U, /**< IOMUXC select input index */
    kIOMUXC_LPUART4_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 122U, /**< IOMUXC select input index */
    kIOMUXC_LPUART5_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 123U, /**< IOMUXC select input index */
    kIOMUXC_LPUART5_IPP_IND_LPUART_DCD_N_SELECT_INPUT = 124U, /**< IOMUXC select input index */
    kIOMUXC_LPUART5_IPP_IND_LPUART_DSR_N_SELECT_INPUT = 125U, /**< IOMUXC select input index */
    kIOMUXC_LPUART5_IPP_IND_LPUART_RI_N_SELECT_INPUT = 126U, /**< IOMUXC select input index */
    kIOMUXC_LPUART5_IPP_IND_LPUART_RXD_SELECT_INPUT = 127U, /**< IOMUXC select input index */
    kIOMUXC_LPUART5_IPP_IND_LPUART_TXD_SELECT_INPUT = 128U, /**< IOMUXC select input index */
    kIOMUXC_LPUART6_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 129U, /**< IOMUXC select input index */
    kIOMUXC_LPUART6_IPP_IND_LPUART_DCD_N_SELECT_INPUT = 130U, /**< IOMUXC select input index */
    kIOMUXC_LPUART6_IPP_IND_LPUART_DSR_N_SELECT_INPUT = 131U, /**< IOMUXC select input index */
    kIOMUXC_LPUART6_IPP_IND_LPUART_RI_N_SELECT_INPUT = 132U, /**< IOMUXC select input index */
    kIOMUXC_LPUART6_IPP_IND_LPUART_RXD_SELECT_INPUT = 133U, /**< IOMUXC select input index */
    kIOMUXC_LPUART6_IPP_IND_LPUART_TXD_SELECT_INPUT = 134U, /**< IOMUXC select input index */
    kIOMUXC_LPUART8_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 135U, /**< IOMUXC select input index */
    kIOMUXC_LPUART8_IPP_IND_LPUART_RXD_SELECT_INPUT = 136U, /**< IOMUXC select input index */
    kIOMUXC_LPUART8_IPP_IND_LPUART_TXD_SELECT_INPUT = 137U, /**< IOMUXC select input index */
    kIOMUXC_LPUART9_IPP_IND_LPUART_RXD_SELECT_INPUT = 138U, /**< IOMUXC select input index */
    kIOMUXC_LPUART9_IPP_IND_LPUART_TXD_SELECT_INPUT = 139U, /**< IOMUXC select input index */
    kIOMUXC_MIC_IPP_IND_MIC_PDM_BITSTREAM_SELECT_INPUT_0 = 140U, /**< IOMUXC select input index */
    kIOMUXC_MIC_IPP_IND_MIC_PDM_BITSTREAM_SELECT_INPUT_1 = 141U, /**< IOMUXC select input index */
    kIOMUXC_MIC_IPP_IND_MIC_PDM_BITSTREAM_SELECT_INPUT_2 = 142U, /**< IOMUXC select input index */
    kIOMUXC_MIC_IPP_IND_MIC_PDM_BITSTREAM_SELECT_INPUT_3 = 143U, /**< IOMUXC select input index */
    kIOMUXC_NETC_EMDIO_IN_SELECT_INPUT = 190U,     /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH2_COL_SELECT_INPUT = 191U,     /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH2_CRS_SELECT_INPUT = 192U,     /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH2_SLV_MDC_IN_SELECT_INPUT = 193U, /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH2_SLV_MDIO_IN_SELECT_INPUT = 194U, /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH3_COL_SELECT_INPUT = 195U,     /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH3_CRS_SELECT_INPUT = 196U,     /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH3_SLV_MDC_IN_SELECT_INPUT = 197U, /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH3_SLV_MDIO_IN_SELECT_INPUT = 198U, /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH4_COL_SELECT_INPUT = 199U,     /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH4_CRS_SELECT_INPUT = 200U,     /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH4_SLV_MDC_IN_SELECT_INPUT = 201U, /**< IOMUXC select input index */
    kIOMUXC_NETC_ETH4_SLV_MDIO_IN_SELECT_INPUT = 202U, /**< IOMUXC select input index */
    kIOMUXC_NETC_TMR_TRIG1_SELECT_INPUT = 203U,    /**< IOMUXC select input index */
    kIOMUXC_NETC_TMR_TRIG2_SELECT_INPUT = 204U,    /**< IOMUXC select input index */
    kIOMUXC_NETC_CLKGEN_IPP_TMR_CLK_SELECT_INPUT = 205U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_RX_CLK_SELECT_INPUT = 206U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_RX_DV_SELECT_INPUT = 207U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_RX_ER_SELECT_INPUT = 208U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_RXD_SELECT_INPUT_0 = 209U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_RXD_SELECT_INPUT_1 = 210U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_RXD_SELECT_INPUT_2 = 211U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_RXD_SELECT_INPUT_3 = 212U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH0_TX_CLK_SELECT_INPUT = 213U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_RX_CLK_SELECT_INPUT = 214U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_RX_DV_SELECT_INPUT = 215U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_RX_ER_SELECT_INPUT = 216U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_RXD_SELECT_INPUT_0 = 217U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_RXD_SELECT_INPUT_1 = 218U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_RXD_SELECT_INPUT_2 = 219U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_RXD_SELECT_INPUT_3 = 220U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH2_TX_CLK_SELECT_INPUT = 221U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_RX_CLK_SELECT_INPUT = 222U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_RX_DV_SELECT_INPUT = 223U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_RX_ER_SELECT_INPUT = 224U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_RXD_SELECT_INPUT_0 = 225U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_RXD_SELECT_INPUT_1 = 226U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_RXD_SELECT_INPUT_2 = 227U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_RXD_SELECT_INPUT_3 = 228U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH3_TX_CLK_SELECT_INPUT = 229U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_RX_CLK_SELECT_INPUT = 230U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_RX_DV_SELECT_INPUT = 231U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_RX_ER_SELECT_INPUT = 232U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_RXD_SELECT_INPUT_0 = 233U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_RXD_SELECT_INPUT_1 = 234U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_RXD_SELECT_INPUT_2 = 235U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_RXD_SELECT_INPUT_3 = 236U, /**< IOMUXC select input index */
    kIOMUXC_NETC_PINMUX_IPP_IND_ETH4_TX_CLK_SELECT_INPUT = 237U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER1_TMR0_INPUT_SELECT_INPUT = 238U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER1_TMR1_INPUT_SELECT_INPUT = 239U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER1_TMR2_INPUT_SELECT_INPUT = 240U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER2_TMR0_INPUT_SELECT_INPUT = 241U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER2_TMR1_INPUT_SELECT_INPUT = 242U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER2_TMR2_INPUT_SELECT_INPUT = 243U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER3_TMR0_INPUT_SELECT_INPUT = 244U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER3_TMR1_INPUT_SELECT_INPUT = 245U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER3_TMR2_INPUT_SELECT_INPUT = 246U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER4_TMR0_INPUT_SELECT_INPUT = 247U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER4_TMR1_INPUT_SELECT_INPUT = 248U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER4_TMR2_INPUT_SELECT_INPUT = 249U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER5_TMR0_INPUT_SELECT_INPUT = 250U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER5_TMR1_INPUT_SELECT_INPUT = 251U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER5_TMR2_INPUT_SELECT_INPUT = 252U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER6_TMR0_INPUT_SELECT_INPUT = 253U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER6_TMR1_INPUT_SELECT_INPUT = 254U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER6_TMR2_INPUT_SELECT_INPUT = 255U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER7_TMR0_INPUT_SELECT_INPUT = 256U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER7_TMR1_INPUT_SELECT_INPUT = 257U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER8_TMR0_INPUT_SELECT_INPUT = 258U, /**< IOMUXC select input index */
    kIOMUXC_QTIMER8_TMR1_INPUT_SELECT_INPUT = 259U, /**< IOMUXC select input index */
    kIOMUXC_SAI4_IPG_CLK_SAI_MCLK_SELECT_INPUT = 260U, /**< IOMUXC select input index */
    kIOMUXC_SAI4_IPP_IND_SAI_RXBCLK_SELECT_INPUT = 261U, /**< IOMUXC select input index */
    kIOMUXC_SAI4_IPP_IND_SAI_RXDATA_SELECT_INPUT_0 = 262U, /**< IOMUXC select input index */
    kIOMUXC_SAI4_IPP_IND_SAI_RXDATA_SELECT_INPUT_1 = 263U, /**< IOMUXC select input index */
    kIOMUXC_SAI4_IPP_IND_SAI_RXSYNC_SELECT_INPUT = 264U, /**< IOMUXC select input index */
    kIOMUXC_SAI4_IPP_IND_SAI_TXBCLK_SELECT_INPUT = 265U, /**< IOMUXC select input index */
    kIOMUXC_SAI4_IPP_IND_SAI_TXSYNC_SELECT_INPUT = 266U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMBIT_SELECT_INPUT_0 = 269U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMBIT_SELECT_INPUT_1 = 270U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMBIT_SELECT_INPUT_2 = 271U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMBIT_SELECT_INPUT_3 = 272U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMCLK_SELECT_INPUT_0 = 273U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMCLK_SELECT_INPUT_1 = 274U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMCLK_SELECT_INPUT_2 = 275U, /**< IOMUXC select input index */
    kIOMUXC_SINC1_IPP_IND_EMCLK_SELECT_INPUT_3 = 276U, /**< IOMUXC select input index */
    kIOMUXC_SINC2_IPP_IND_EMBIT_SELECT_INPUT_2 = 277U, /**< IOMUXC select input index */
    kIOMUXC_SINC2_IPP_IND_EMBIT_SELECT_INPUT_3 = 278U, /**< IOMUXC select input index */
    kIOMUXC_SINC2_IPP_IND_EMCLK_SELECT_INPUT_0 = 279U, /**< IOMUXC select input index */
    kIOMUXC_SINC2_IPP_IND_EMCLK_SELECT_INPUT_2 = 280U, /**< IOMUXC select input index */
    kIOMUXC_SINC2_IPP_IND_EMCLK_SELECT_INPUT_3 = 281U, /**< IOMUXC select input index */
    kIOMUXC_SPDIF_SPDIF_IN1_SELECT_INPUT = 282U,   /**< IOMUXC select input index */
    kIOMUXC_USB_IPP_IND_OTG2_OC_SELECT_INPUT = 285U, /**< IOMUXC select input index */
    kIOMUXC_USB_IPP_IND_OTG_OC_SELECT_INPUT = 286U, /**< IOMUXC select input index */
    kIOMUXC_USBPHY1_USB_ID_SELECT_INPUT = 287U,    /**< IOMUXC select input index */
    kIOMUXC_USBPHY2_USB_ID_SELECT_INPUT = 288U,    /**< IOMUXC select input index */
    kIOMUXC_USDHC1_IPP_CARD_DET_SELECT_INPUT = 289U, /**< IOMUXC select input index */
    kIOMUXC_USDHC1_IPP_WP_ON_SELECT_INPUT = 290U,  /**< IOMUXC select input index */
    kIOMUXC_USDHC2_IPP_CARD_DET_SELECT_INPUT = 291U, /**< IOMUXC select input index */
    kIOMUXC_USDHC2_IPP_WP_ON_SELECT_INPUT = 292U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_14 = 293U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_15 = 294U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_17 = 295U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_18 = 296U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_19 = 297U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_20 = 298U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_21 = 299U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_22 = 300U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_23 = 301U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_24 = 302U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_25 = 303U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_26 = 304U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_27 = 305U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_28 = 306U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_29 = 307U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_30 = 308U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_31 = 309U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_32 = 310U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_33 = 311U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_34 = 312U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_35 = 313U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_36 = 314U,  /**< IOMUXC select input index */
    kIOMUXC_XBAR1_XBAR_IN_SELECT_INPUT_37 = 315U,  /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_CS_SELECT_INPUT = 344U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_DQS_SELECT_INPUT = 345U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_0 = 346U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_1 = 347U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_2 = 348U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_3 = 349U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_4 = 350U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_5 = 351U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_6 = 352U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_IO_SELECT_INPUT_7 = 353U, /**< IOMUXC select input index */
    kIOMUXC_XSPI_SLV_IPP_IND_SCK_SELECT_INPUT = 354U, /**< IOMUXC select input index */
} iomuxc_select_input_t;

/*!
 * @addtogroup iomuxc_aon_pads
 * @{ */

/*******************************************************************************
 * Definitions
*******************************************************************************/

/*!
 * @brief Enumeration for the IOMUXC_AON SW_MUX_CTL_PAD
 *
 * Defines the enumeration for the IOMUXC_AON SW_MUX_CTL_PAD collections.
 */
typedef enum _iomuxc_aon_sw_mux_ctl_pad
{
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_00 = 0U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_01 = 1U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_02 = 2U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_03 = 3U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_04 = 4U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_05 = 5U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_06 = 6U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_07 = 7U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_08 = 8U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_09 = 9U,   /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_10 = 10U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_11 = 11U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_12 = 12U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_13 = 13U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_14 = 14U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_15 = 15U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_16 = 16U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_17 = 17U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_18 = 18U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_19 = 19U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_20 = 20U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_21 = 21U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_22 = 22U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_23 = 23U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_24 = 24U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_25 = 25U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_26 = 26U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_27 = 27U,  /**< IOMUXC SW_MUX_CTL_PAD index */
    kIOMUXC_AON_SW_MUX_CTL_PAD_GPIO_AON_28_DUMMY = 28U, /**< IOMUXC SW_MUX_CTL_PAD index */
} iomuxc_aon_sw_mux_ctl_pad_t;

/* @} */

/*!
 * @addtogroup iomuxc_aon_pads
 * @{ */

/*******************************************************************************
 * Definitions
*******************************************************************************/

/*!
 * @brief Enumeration for the IOMUXC_AON SW_PAD_CTL_PAD
 *
 * Defines the enumeration for the IOMUXC_AON SW_PAD_CTL_PAD collections.
 */
typedef enum _iomuxc_aon_sw_pad_ctl_pad
{
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_00 = 0U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_01 = 1U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_02 = 2U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_03 = 3U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_04 = 4U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_05 = 5U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_06 = 6U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_07 = 7U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_08 = 8U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_09 = 9U,   /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_10 = 10U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_11 = 11U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_12 = 12U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_13 = 13U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_14 = 14U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_15 = 15U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_16 = 16U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_17 = 17U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_18 = 18U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_19 = 19U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_20 = 20U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_21 = 21U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_22 = 22U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_23 = 23U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_24 = 24U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_25 = 25U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_26 = 26U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_27 = 27U,  /**< IOMUXC SW_PAD_CTL_PAD index */
    kIOMUXC_AON_SW_PAD_CTL_PAD_GPIO_AON_28_DUMMY = 28U, /**< IOMUXC SW_PAD_CTL_PAD index */
} iomuxc_aon_sw_pad_ctl_pad_t;

/* @} */

/*!
 * @brief Enumeration for the IOMUXC_AON select input
 *
 * Defines the enumeration for the IOMUXC_AON select input collections.
 */
typedef enum _iomuxc_aon_select_input
{
    kIOMUXC_AON_I3C1_PIN_SCL_IN_SELECT_INPUT = 0U, /**< IOMUXC select input index */
    kIOMUXC_AON_I3C1_PIN_SDA_IN_SELECT_INPUT = 1U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPI2C1_IPP_IND_LPI2C_SCL_SELECT_INPUT = 2U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPI2C1_IPP_IND_LPI2C_SDA_SELECT_INPUT = 3U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPI2C2_IPP_IND_LPI2C_SCL_SELECT_INPUT = 4U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPI2C2_IPP_IND_LPI2C_SDA_SELECT_INPUT = 5U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI1_IPP_IND_LPSPI_PCS_SELECT_INPUT_0 = 6U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI1_IPP_IND_LPSPI_PCS_SELECT_INPUT_1 = 7U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI1_IPP_IND_LPSPI_SCK_SELECT_INPUT = 8U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI1_IPP_IND_LPSPI_SDI_SELECT_INPUT = 9U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI1_IPP_IND_LPSPI_SDO_SELECT_INPUT = 10U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI2_IPP_IND_LPSPI_PCS_SELECT_INPUT_0 = 11U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI2_IPP_IND_LPSPI_PCS_SELECT_INPUT_1 = 12U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI2_IPP_IND_LPSPI_PCS_SELECT_INPUT_3 = 13U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI2_IPP_IND_LPSPI_SCK_SELECT_INPUT = 14U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI2_IPP_IND_LPSPI_SDI_SELECT_INPUT = 15U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPSPI2_IPP_IND_LPSPI_SDO_SELECT_INPUT = 16U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPTMR1_IPP_IND_LPTIMER_SELECT_INPUT_1 = 17U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPTMR1_IPP_IND_LPTIMER_SELECT_INPUT_2 = 18U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPTMR1_IPP_IND_LPTIMER_SELECT_INPUT_3 = 19U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART1_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 20U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART1_IPP_IND_LPUART_DCD_N_SELECT_INPUT = 21U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART1_IPP_IND_LPUART_DSR_N_SELECT_INPUT = 22U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART12_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 23U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART12_IPP_IND_LPUART_RXD_SELECT_INPUT = 24U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART12_IPP_IND_LPUART_TXD_SELECT_INPUT = 25U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART2_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 26U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART2_IPP_IND_LPUART_RXD_SELECT_INPUT = 27U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART2_IPP_IND_LPUART_TXD_SELECT_INPUT = 28U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART7_IPP_IND_LPUART_CTS_N_SELECT_INPUT = 29U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART7_IPP_IND_LPUART_RXD_SELECT_INPUT = 30U, /**< IOMUXC select input index */
    kIOMUXC_AON_LPUART7_IPP_IND_LPUART_TXD_SELECT_INPUT = 31U, /**< IOMUXC select input index */
    kIOMUXC_AON_SAI1_IPG_CLK_SAI_MCLK_SELECT_INPUT = 32U, /**< IOMUXC select input index */
    kIOMUXC_AON_SAI1_IPP_IND_SAI_RXBCLK_SELECT_INPUT = 33U, /**< IOMUXC select input index */
    kIOMUXC_AON_SAI1_IPP_IND_SAI_RXDATA_SELECT_INPUT_0 = 34U, /**< IOMUXC select input index */
    kIOMUXC_AON_SAI1_IPP_IND_SAI_RXDATA_SELECT_INPUT_1 = 35U, /**< IOMUXC select input index */
    kIOMUXC_AON_SAI1_IPP_IND_SAI_RXSYNC_SELECT_INPUT = 36U, /**< IOMUXC select input index */
    kIOMUXC_AON_SAI1_IPP_IND_SAI_TXBCLK_SELECT_INPUT = 37U, /**< IOMUXC select input index */
    kIOMUXC_AON_SAI1_IPP_IND_SAI_TXSYNC_SELECT_INPUT = 38U, /**< IOMUXC select input index */
} iomuxc_aon_select_input_t;

/*!
 * @addtogroup asrc_clock_source_mapping
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*!
 * @brief The ASRC clock source
 */
typedef enum _asrc_clock_source
{
    kASRC_ClockSourceNotAvalible    = -1,          /**< not avalible */
    kASRC_ClockSourceBitClock0_SAI1_TX = 0U,       /**< SAI1 TX */
    kASRC_ClockSourceBitClock1_SAI1_RX = 1U,       /**< SAI1 RX */
    kASRC_ClockSourceBitClock2_SAI2_TX = 2U,       /**< SAI2 TX */
    kASRC_ClockSourceBitClock3_SAI2_RX = 3U,       /**< SAI2 RX */
    kASRC_ClockSourceBitClock4_SAI3_TX = 4U,       /**< SAI3 TX */
    kASRC_ClockSourceBitClock5_SAI3_RX = 5U,       /**< SAI3 RX */
    kASRC_ClockSourceBitClock6_SAI4_TX = 6U,       /**< SAI4 TX */
    kASRC_ClockSourceBitClock7_SAI4_RX = 7U,       /**< SAI4 RX */
    kASRC_ClockSourceBitClock8_SPDIF_TX = 8U,      /**< SPDIF TX */
    kASRC_ClockSourceBitClock9_SPDIF_RX = 9U,      /**< SPDIF RX */
    kASRC_ClockSourceBitClocka_SAI2_CLOCK_ROOT = 10U, /**< SAI2 CLOCK ROOT */
    kASRC_ClockSourceBitClockb_SAI3_CLOCK_ROOT = 11U, /**< SAI3 CLOCK ROOT */
    kASRC_ClockSourceBitClockc_SAI4_CLOCK_ROOT = 12U, /**< SAI4 CLOCK ROOT */
    kASRC_ClockSourceBitClockd_MIC_CLOCK_ROOT = 13U, /**< MIC CLOCK ROOT */
    kASRC_ClockSourceBitClocke_MQS_CLOCK_ROOT = 14U, /**< MQS CLOCK ROOT */
} asrc_clock_source_t;

/* @} */

typedef enum _xbar_input_signal
{
    kXBAR1_InputLogicLow            = 0|0x100U,    /**< LOGIC_LOW output assigned to XBAR1_IN0 input. */
    kXBAR1_InputLogicHigh           = 1|0x100U,    /**< LOGIC_HIGH output assigned to XBAR1_IN1 input. */
    kXBAR1_InputLogicLow1           = 2|0x100U,    /**< LOGIC_LOW1 output assigned to XBAR1_IN2 input. */
    kXBAR1_InputLogicHigh1          = 3|0x100U,    /**< LOGIC_HIGH1 output assigned to XBAR1_IN3 input. */
    kXBAR1_InputIomuxXbarInout04    = 4|0x100U,    /**< IOMUX_XBAR_INOUT04 output assigned to XBAR1_IN4 input. */
    kXBAR1_InputIomuxXbarInout05    = 5|0x100U,    /**< IOMUX_XBAR_INOUT05 output assigned to XBAR1_IN5 input. */
    kXBAR1_InputIomuxXbarInout06    = 6|0x100U,    /**< IOMUX_XBAR_INOUT06 output assigned to XBAR1_IN6 input. */
    kXBAR1_InputIomuxXbarInout07    = 7|0x100U,    /**< IOMUX_XBAR_INOUT07 output assigned to XBAR1_IN7 input. */
    kXBAR1_InputIomuxXbarInout08    = 8|0x100U,    /**< IOMUX_XBAR_INOUT08 output assigned to XBAR1_IN8 input. */
    kXBAR1_InputIomuxXbarInout09    = 9|0x100U,    /**< IOMUX_XBAR_INOUT09 output assigned to XBAR1_IN9 input. */
    kXBAR1_InputIomuxXbarInout10    = 10|0x100U,   /**< IOMUX_XBAR_INOUT10 output assigned to XBAR1_IN10 input. */
    kXBAR1_InputIomuxXbarInout11    = 11|0x100U,   /**< IOMUX_XBAR_INOUT11 output assigned to XBAR1_IN11 input. */
    kXBAR1_InputIomuxXbarInout12    = 12|0x100U,   /**< IOMUX_XBAR_INOUT12 output assigned to XBAR1_IN12 input. */
    kXBAR1_InputIomuxXbarInout13    = 13|0x100U,   /**< IOMUX_XBAR_INOUT13 output assigned to XBAR1_IN13 input. */
    kXBAR1_InputIomuxXbarInout14    = 14|0x100U,   /**< IOMUX_XBAR_INOUT14 output assigned to XBAR1_IN14 input. */
    kXBAR1_InputIomuxXbarInout15    = 15|0x100U,   /**< IOMUX_XBAR_INOUT15 output assigned to XBAR1_IN15 input. */
    kXBAR1_InputIomuxXbarInout16    = 16|0x100U,   /**< IOMUX_XBAR_INOUT16 output assigned to XBAR1_IN16 input. */
    kXBAR1_InputIomuxXbarInout17    = 17|0x100U,   /**< IOMUX_XBAR_INOUT17 output assigned to XBAR1_IN17 input. */
    kXBAR1_InputIomuxXbarInout18    = 18|0x100U,   /**< IOMUX_XBAR_INOUT18 output assigned to XBAR1_IN18 input. */
    kXBAR1_InputIomuxXbarInout19    = 19|0x100U,   /**< IOMUX_XBAR_INOUT19 output assigned to XBAR1_IN19 input. */
    kXBAR1_InputIomuxXbarInout20    = 20|0x100U,   /**< IOMUX_XBAR_INOUT20 output assigned to XBAR1_IN20 input. */
    kXBAR1_InputIomuxXbarInout21    = 21|0x100U,   /**< IOMUX_XBAR_INOUT21 output assigned to XBAR1_IN21 input. */
    kXBAR1_InputIomuxXbarInout22    = 22|0x100U,   /**< IOMUX_XBAR_INOUT22 output assigned to XBAR1_IN22 input. */
    kXBAR1_InputIomuxXbarInout23    = 23|0x100U,   /**< IOMUX_XBAR_INOUT23 output assigned to XBAR1_IN23 input. */
    kXBAR1_InputIomuxXbarInout24    = 24|0x100U,   /**< IOMUX_XBAR_INOUT24 output assigned to XBAR1_IN24 input. */
    kXBAR1_InputIomuxXbarInout25    = 25|0x100U,   /**< IOMUX_XBAR_INOUT25 output assigned to XBAR1_IN25 input. */
    kXBAR1_InputIomuxXbarInout26    = 26|0x100U,   /**< IOMUX_XBAR_INOUT26 output assigned to XBAR1_IN26 input. */
    kXBAR1_InputIomuxXbarInout27    = 27|0x100U,   /**< IOMUX_XBAR_INOUT27 output assigned to XBAR1_IN27 input. */
    kXBAR1_InputIomuxXbarInout28    = 28|0x100U,   /**< IOMUX_XBAR_INOUT28 output assigned to XBAR1_IN28 input. */
    kXBAR1_InputIomuxXbarInout29    = 29|0x100U,   /**< IOMUX_XBAR_INOUT29 output assigned to XBAR1_IN29 input. */
    kXBAR1_InputIomuxXbarInout30    = 30|0x100U,   /**< IOMUX_XBAR_INOUT30 output assigned to XBAR1_IN30 input. */
    kXBAR1_InputIomuxXbarInout31    = 31|0x100U,   /**< IOMUX_XBAR_INOUT31 output assigned to XBAR1_IN31 input. */
    kXBAR1_InputIomuxXbarInout32    = 32|0x100U,   /**< IOMUX_XBAR_INOUT32 output assigned to XBAR1_IN32 input. */
    kXBAR1_InputIomuxXbarInout33    = 33|0x100U,   /**< IOMUX_XBAR_INOUT33 output assigned to XBAR1_IN33 input. */
    kXBAR1_InputIomuxXbarInout34    = 34|0x100U,   /**< IOMUX_XBAR_INOUT34 output assigned to XBAR1_IN34 input. */
    kXBAR1_InputIomuxXbarInout35    = 35|0x100U,   /**< IOMUX_XBAR_INOUT35 output assigned to XBAR1_IN35 input. */
    kXBAR1_InputIomuxXbarInout36    = 36|0x100U,   /**< IOMUX_XBAR_INOUT36 output assigned to XBAR1_IN36 input. */
    kXBAR1_InputIomuxXbarInout37    = 37|0x100U,   /**< IOMUX_XBAR_INOUT37 output assigned to XBAR1_IN37 input. */
    kXBAR1_InputAcmp1Out            = 38|0x100U,   /**< ACMP1_OUT output assigned to XBAR1_IN38 input. */
    kXBAR1_InputAcmp2Out            = 39|0x100U,   /**< ACMP2_OUT output assigned to XBAR1_IN39 input. */
    kXBAR1_InputAcmp3Out            = 40|0x100U,   /**< ACMP3_OUT output assigned to XBAR1_IN40 input. */
    kXBAR1_InputAcmp4Out            = 41|0x100U,   /**< ACMP4_OUT output assigned to XBAR1_IN41 input. */
    kXBAR1_InputQtimer1Timer0       = 42|0x100U,   /**< QTIMER1_TIMER0 output assigned to XBAR1_IN42 input. */
    kXBAR1_InputQtimer1Timer1       = 43|0x100U,   /**< QTIMER1_TIMER1 output assigned to XBAR1_IN43 input. */
    kXBAR1_InputQtimer1Timer2       = 44|0x100U,   /**< QTIMER1_TIMER2 output assigned to XBAR1_IN44 input. */
    kXBAR1_InputQtimer1Timer3       = 45|0x100U,   /**< QTIMER1_TIMER3 output assigned to XBAR1_IN45 input. */
    kXBAR1_InputQtimer2Timer0       = 46|0x100U,   /**< QTIMER2_TIMER0 output assigned to XBAR1_IN46 input. */
    kXBAR1_InputQtimer2Timer1       = 47|0x100U,   /**< QTIMER2_TIMER1 output assigned to XBAR1_IN47 input. */
    kXBAR1_InputQtimer2Timer2       = 48|0x100U,   /**< QTIMER2_TIMER2 output assigned to XBAR1_IN48 input. */
    kXBAR1_InputQtimer2Timer3       = 49|0x100U,   /**< QTIMER2_TIMER3 output assigned to XBAR1_IN49 input. */
    kXBAR1_InputQtimer3Timer0       = 50|0x100U,   /**< QTIMER3_TIMER0 output assigned to XBAR1_IN50 input. */
    kXBAR1_InputQtimer3Timer1       = 51|0x100U,   /**< QTIMER3_TIMER1 output assigned to XBAR1_IN51 input. */
    kXBAR1_InputQtimer3Timer2       = 52|0x100U,   /**< QTIMER3_TIMER2 output assigned to XBAR1_IN52 input. */
    kXBAR1_InputQtimer3Timer3       = 53|0x100U,   /**< QTIMER3_TIMER3 output assigned to XBAR1_IN53 input. */
    kXBAR1_InputQtimer4Timer0       = 54|0x100U,   /**< QTIMER4_TIMER0 output assigned to XBAR1_IN54 input. */
    kXBAR1_InputQtimer4Timer1       = 55|0x100U,   /**< QTIMER4_TIMER1 output assigned to XBAR1_IN55 input. */
    kXBAR1_InputQtimer4Timer2       = 56|0x100U,   /**< QTIMER4_TIMER2 output assigned to XBAR1_IN56 input. */
    kXBAR1_InputQtimer4Timer3       = 57|0x100U,   /**< QTIMER4_TIMER3 output assigned to XBAR1_IN57 input. */
    kXBAR1_InputRESERVED58          = 58|0x100U,   /**< XBAR1_IN58 input is reserved. */
    kXBAR1_InputRESERVED59          = 59|0x100U,   /**< XBAR1_IN59 input is reserved. */
    kXBAR1_InputRESERVED60          = 60|0x100U,   /**< XBAR1_IN60 input is reserved. */
    kXBAR1_InputRESERVED61          = 61|0x100U,   /**< XBAR1_IN61 input is reserved. */
    kXBAR1_InputRESERVED62          = 62|0x100U,   /**< XBAR1_IN62 input is reserved. */
    kXBAR1_InputRESERVED63          = 63|0x100U,   /**< XBAR1_IN63 input is reserved. */
    kXBAR1_InputRESERVED64          = 64|0x100U,   /**< XBAR1_IN64 input is reserved. */
    kXBAR1_InputRESERVED65          = 65|0x100U,   /**< XBAR1_IN65 input is reserved. */
    kXBAR1_InputRESERVED66          = 66|0x100U,   /**< XBAR1_IN66 input is reserved. */
    kXBAR1_InputRESERVED67          = 67|0x100U,   /**< XBAR1_IN67 input is reserved. */
    kXBAR1_InputRESERVED68          = 68|0x100U,   /**< XBAR1_IN68 input is reserved. */
    kXBAR1_InputRESERVED69          = 69|0x100U,   /**< XBAR1_IN69 input is reserved. */
    kXBAR1_InputRESERVED70          = 70|0x100U,   /**< XBAR1_IN70 input is reserved. */
    kXBAR1_InputRESERVED71          = 71|0x100U,   /**< XBAR1_IN71 input is reserved. */
    kXBAR1_InputRESERVED72          = 72|0x100U,   /**< XBAR1_IN72 input is reserved. */
    kXBAR1_InputRESERVED73          = 73|0x100U,   /**< XBAR1_IN73 input is reserved. */
    kXBAR1_InputFlexpwm1Pwm0OutTrig0 = 74|0x100U,  /**< FLEXPWM1_PWM0_OUT_TRIG0 output assigned to XBAR1_IN74 input. */
    kXBAR1_InputFlexpwm1Pwm0OutTrig1 = 75|0x100U,  /**< FLEXPWM1_PWM0_OUT_TRIG1 output assigned to XBAR1_IN75 input. */
    kXBAR1_InputFlexpwm1Pwm1OutTrig0 = 76|0x100U,  /**< FLEXPWM1_PWM1_OUT_TRIG0 output assigned to XBAR1_IN76 input. */
    kXBAR1_InputFlexpwm1Pwm1OutTrig1 = 77|0x100U,  /**< FLEXPWM1_PWM1_OUT_TRIG1 output assigned to XBAR1_IN77 input. */
    kXBAR1_InputFlexpwm1Pwm2OutTrig0 = 78|0x100U,  /**< FLEXPWM1_PWM2_OUT_TRIG0 output assigned to XBAR1_IN78 input. */
    kXBAR1_InputFlexpwm1Pwm2OutTrig1 = 79|0x100U,  /**< FLEXPWM1_PWM2_OUT_TRIG1 output assigned to XBAR1_IN79 input. */
    kXBAR1_InputFlexpwm1Pwm3OutTrig0 = 80|0x100U,  /**< FLEXPWM1_PWM3_OUT_TRIG0 output assigned to XBAR1_IN80 input. */
    kXBAR1_InputFlexpwm1Pwm3OutTrig1 = 81|0x100U,  /**< FLEXPWM1_PWM3_OUT_TRIG1 output assigned to XBAR1_IN81 input. */
    kXBAR1_InputFlexpwm2Pwm0OutTrig01 = 82|0x100U, /**< FLEXPWM2_PWM0_OUT_TRIG0_1 output assigned to XBAR1_IN82 input. */
    kXBAR1_InputFlexpwm2Pwm1OutTrig01 = 83|0x100U, /**< FLEXPWM2_PWM1_OUT_TRIG0_1 output assigned to XBAR1_IN83 input. */
    kXBAR1_InputFlexpwm2Pwm2OutTrig01 = 84|0x100U, /**< FLEXPWM2_PWM2_OUT_TRIG0_1 output assigned to XBAR1_IN84 input. */
    kXBAR1_InputFlexpwm2Pwm3OutTrig01 = 85|0x100U, /**< FLEXPWM2_PWM3_OUT_TRIG0_1 output assigned to XBAR1_IN85 input. */
    kXBAR1_InputRESERVED86          = 86|0x100U,   /**< XBAR1_IN86 input is reserved. */
    kXBAR1_InputRESERVED87          = 87|0x100U,   /**< XBAR1_IN87 input is reserved. */
    kXBAR1_InputRESERVED88          = 88|0x100U,   /**< XBAR1_IN88 input is reserved. */
    kXBAR1_InputRESERVED89          = 89|0x100U,   /**< XBAR1_IN89 input is reserved. */
    kXBAR1_InputRESERVED90          = 90|0x100U,   /**< XBAR1_IN90 input is reserved. */
    kXBAR1_InputRESERVED91          = 91|0x100U,   /**< XBAR1_IN91 input is reserved. */
    kXBAR1_InputRESERVED92          = 92|0x100U,   /**< XBAR1_IN92 input is reserved. */
    kXBAR1_InputRESERVED93          = 93|0x100U,   /**< XBAR1_IN93 input is reserved. */
    kXBAR1_InputPit1Trigger0        = 94|0x100U,   /**< PIT1_TRIGGER0 output assigned to XBAR1_IN94 input. */
    kXBAR1_InputPit1Trigger1        = 95|0x100U,   /**< PIT1_TRIGGER1 output assigned to XBAR1_IN95 input. */
    kXBAR1_InputPit1Trigger2        = 96|0x100U,   /**< PIT1_TRIGGER2 output assigned to XBAR1_IN96 input. */
    kXBAR1_InputPit1Trigger3        = 97|0x100U,   /**< PIT1_TRIGGER3 output assigned to XBAR1_IN97 input. */
    kXBAR1_InputPit2Trigger0        = 98|0x100U,   /**< PIT2_TRIGGER0 output assigned to XBAR1_IN98 input. */
    kXBAR1_InputPit2Trigger1        = 99|0x100U,   /**< PIT2_TRIGGER1 output assigned to XBAR1_IN99 input. */
    kXBAR1_InputPit2Trigger2        = 100|0x100U,  /**< PIT2_TRIGGER2 output assigned to XBAR1_IN100 input. */
    kXBAR1_InputPit2Trigger3        = 101|0x100U,  /**< PIT2_TRIGGER3 output assigned to XBAR1_IN101 input. */
    kXBAR1_InputPit3Trigger0        = 102|0x100U,  /**< PIT3_TRIGGER0 output assigned to XBAR1_IN102 input. */
    kXBAR1_InputPit3Trigger1        = 103|0x100U,  /**< PIT3_TRIGGER1 output assigned to XBAR1_IN103 input. */
    kXBAR1_InputPit3Trigger2        = 104|0x100U,  /**< PIT3_TRIGGER2 output assigned to XBAR1_IN104 input. */
    kXBAR1_InputPit3Trigger3        = 105|0x100U,  /**< PIT3_TRIGGER3 output assigned to XBAR1_IN105 input. */
    kXBAR1_InputTriggerSyncOut0     = 106|0x100U,  /**< TRIGGER_SYNC_OUT0 output assigned to XBAR1_IN106 input. */
    kXBAR1_InputTriggerSyncOut1     = 107|0x100U,  /**< TRIGGER_SYNC_OUT1 output assigned to XBAR1_IN107 input. */
    kXBAR1_InputTriggerSyncOut2     = 108|0x100U,  /**< TRIGGER_SYNC_OUT2 output assigned to XBAR1_IN108 input. */
    kXBAR1_InputTriggerSyncOut3     = 109|0x100U,  /**< TRIGGER_SYNC_OUT3 output assigned to XBAR1_IN109 input. */
    kXBAR1_InputDma4TriggerOut0     = 110|0x100U,  /**< DMA4_TRIGGER_OUT0 output assigned to XBAR1_IN110 input. */
    kXBAR1_InputDma4TriggerOut1     = 111|0x100U,  /**< DMA4_TRIGGER_OUT1 output assigned to XBAR1_IN111 input. */
    kXBAR1_InputDma4TriggerOut2     = 112|0x100U,  /**< DMA4_TRIGGER_OUT2 output assigned to XBAR1_IN112 input. */
    kXBAR1_InputDma4TriggerOut3     = 113|0x100U,  /**< DMA4_TRIGGER_OUT3 output assigned to XBAR1_IN113 input. */
    kXBAR1_InputDma4TriggerOut4     = 114|0x100U,  /**< DMA4_TRIGGER_OUT4 output assigned to XBAR1_IN114 input. */
    kXBAR1_InputDma4TriggerOut5     = 115|0x100U,  /**< DMA4_TRIGGER_OUT5 output assigned to XBAR1_IN115 input. */
    kXBAR1_InputDma4TriggerOut6     = 116|0x100U,  /**< DMA4_TRIGGER_OUT6 output assigned to XBAR1_IN116 input. */
    kXBAR1_InputDma4TriggerOut7     = 117|0x100U,  /**< DMA4_TRIGGER_OUT7 output assigned to XBAR1_IN117 input. */
    kXBAR1_InputDma3TriggerOut0     = 118|0x100U,  /**< DMA3_TRIGGER_OUT0 output assigned to XBAR1_IN118 input. */
    kXBAR1_InputDma3TriggerOut1     = 119|0x100U,  /**< DMA3_TRIGGER_OUT1 output assigned to XBAR1_IN119 input. */
    kXBAR1_InputDma3TriggerOut2     = 120|0x100U,  /**< DMA3_TRIGGER_OUT2 output assigned to XBAR1_IN120 input. */
    kXBAR1_InputDma3TriggerOut3     = 121|0x100U,  /**< DMA3_TRIGGER_OUT3 output assigned to XBAR1_IN121 input. */
    kXBAR1_InputDma3TriggerOut4     = 122|0x100U,  /**< DMA3_TRIGGER_OUT4 output assigned to XBAR1_IN122 input. */
    kXBAR1_InputDma3TriggerOut5     = 123|0x100U,  /**< DMA3_TRIGGER_OUT5 output assigned to XBAR1_IN123 input. */
    kXBAR1_InputDma3TriggerOut6     = 124|0x100U,  /**< DMA3_TRIGGER_OUT6 output assigned to XBAR1_IN124 input. */
    kXBAR1_InputDma3TriggerOut7     = 125|0x100U,  /**< DMA3_TRIGGER_OUT7 output assigned to XBAR1_IN125 input. */
    kXBAR1_InputAdc1TcompPulse0     = 126|0x100U,  /**< ADC1_TCOMP_PULSE0 output assigned to XBAR1_IN126 input. */
    kXBAR1_InputAdc1TcompPulse1     = 127|0x100U,  /**< ADC1_TCOMP_PULSE1 output assigned to XBAR1_IN127 input. */
    kXBAR1_InputAdc1TcompPulse2     = 128|0x100U,  /**< ADC1_TCOMP_PULSE2 output assigned to XBAR1_IN128 input. */
    kXBAR1_InputAdc1TcompPulse3     = 129|0x100U,  /**< ADC1_TCOMP_PULSE3 output assigned to XBAR1_IN129 input. */
    kXBAR1_InputAdc1TcompPulse4     = 130|0x100U,  /**< ADC1_TCOMP_PULSE4 output assigned to XBAR1_IN130 input. */
    kXBAR1_InputAdc1TcompPulse5     = 131|0x100U,  /**< ADC1_TCOMP_PULSE5 output assigned to XBAR1_IN131 input. */
    kXBAR1_InputAdc1TcompPulse6     = 132|0x100U,  /**< ADC1_TCOMP_PULSE6 output assigned to XBAR1_IN132 input. */
    kXBAR1_InputAdc1TcompPulse7     = 133|0x100U,  /**< ADC1_TCOMP_PULSE7 output assigned to XBAR1_IN133 input. */
    kXBAR1_InputRESERVED134         = 134|0x100U,  /**< XBAR1_IN134 input is reserved. */
    kXBAR1_InputRESERVED135         = 135|0x100U,  /**< XBAR1_IN135 input is reserved. */
    kXBAR1_InputRESERVED136         = 136|0x100U,  /**< XBAR1_IN136 input is reserved. */
    kXBAR1_InputRESERVED137         = 137|0x100U,  /**< XBAR1_IN137 input is reserved. */
    kXBAR1_InputRESERVED138         = 138|0x100U,  /**< XBAR1_IN138 input is reserved. */
    kXBAR1_InputRESERVED139         = 139|0x100U,  /**< XBAR1_IN139 input is reserved. */
    kXBAR1_InputRESERVED140         = 140|0x100U,  /**< XBAR1_IN140 input is reserved. */
    kXBAR1_InputRESERVED141         = 141|0x100U,  /**< XBAR1_IN141 input is reserved. */
    kXBAR1_InputTpm1ChTrigger0      = 142|0x100U,  /**< TPM1_CH_TRIGGER0 output assigned to XBAR1_IN142 input. */
    kXBAR1_InputTpm1ChTrigger1      = 143|0x100U,  /**< TPM1_CH_TRIGGER1 output assigned to XBAR1_IN143 input. */
    kXBAR1_InputTpm1ChTrigger2      = 144|0x100U,  /**< TPM1_CH_TRIGGER2 output assigned to XBAR1_IN144 input. */
    kXBAR1_InputTpm1ChTrigger3      = 145|0x100U,  /**< TPM1_CH_TRIGGER3 output assigned to XBAR1_IN145 input. */
    kXBAR1_InputTpm1Trigger         = 146|0x100U,  /**< TPM1_TRIGGER output assigned to XBAR1_IN146 input. */
    kXBAR1_InputTpm2ChTrigger0      = 147|0x100U,  /**< TPM2_CH_TRIGGER0 output assigned to XBAR1_IN147 input. */
    kXBAR1_InputTpm2ChTrigger1      = 148|0x100U,  /**< TPM2_CH_TRIGGER1 output assigned to XBAR1_IN148 input. */
    kXBAR1_InputTpm2ChTrigger2      = 149|0x100U,  /**< TPM2_CH_TRIGGER2 output assigned to XBAR1_IN149 input. */
    kXBAR1_InputTpm2ChTrigger3      = 150|0x100U,  /**< TPM2_CH_TRIGGER3 output assigned to XBAR1_IN150 input. */
    kXBAR1_InputTpm2Trigger         = 151|0x100U,  /**< TPM2_TRIGGER output assigned to XBAR1_IN151 input. */
    kXBAR1_InputTpm3ChTrigger0      = 152|0x100U,  /**< TPM3_CH_TRIGGER0 output assigned to XBAR1_IN152 input. */
    kXBAR1_InputTpm3ChTrigger1      = 153|0x100U,  /**< TPM3_CH_TRIGGER1 output assigned to XBAR1_IN153 input. */
    kXBAR1_InputTpm3ChTrigger2      = 154|0x100U,  /**< TPM3_CH_TRIGGER2 output assigned to XBAR1_IN154 input. */
    kXBAR1_InputTpm3ChTrigger3      = 155|0x100U,  /**< TPM3_CH_TRIGGER3 output assigned to XBAR1_IN155 input. */
    kXBAR1_InputTpm3Trigger         = 156|0x100U,  /**< TPM3_TRIGGER output assigned to XBAR1_IN156 input. */
    kXBAR1_InputRESERVED157         = 157|0x100U,  /**< XBAR1_IN157 input is reserved. */
    kXBAR1_InputRESERVED158         = 158|0x100U,  /**< XBAR1_IN158 input is reserved. */
    kXBAR1_InputRESERVED159         = 159|0x100U,  /**< XBAR1_IN159 input is reserved. */
    kXBAR1_InputRESERVED160         = 160|0x100U,  /**< XBAR1_IN160 input is reserved. */
    kXBAR1_InputRESERVED161         = 161|0x100U,  /**< XBAR1_IN161 input is reserved. */
    kXBAR1_InputRESERVED162         = 162|0x100U,  /**< XBAR1_IN162 input is reserved. */
    kXBAR1_InputRESERVED163         = 163|0x100U,  /**< XBAR1_IN163 input is reserved. */
    kXBAR1_InputRESERVED164         = 164|0x100U,  /**< XBAR1_IN164 input is reserved. */
    kXBAR1_InputRESERVED165         = 165|0x100U,  /**< XBAR1_IN165 input is reserved. */
    kXBAR1_InputRESERVED166         = 166|0x100U,  /**< XBAR1_IN166 input is reserved. */
    kXBAR1_InputRESERVED167         = 167|0x100U,  /**< XBAR1_IN167 input is reserved. */
    kXBAR1_InputRESERVED168         = 168|0x100U,  /**< XBAR1_IN168 input is reserved. */
    kXBAR1_InputRESERVED169         = 169|0x100U,  /**< XBAR1_IN169 input is reserved. */
    kXBAR1_InputRESERVED170         = 170|0x100U,  /**< XBAR1_IN170 input is reserved. */
    kXBAR1_InputRESERVED171         = 171|0x100U,  /**< XBAR1_IN171 input is reserved. */
    kXBAR1_InputLptmr1TriggerDelay  = 172|0x100U,  /**< LPTMR1_TRIGGER_DELAY output assigned to XBAR1_IN172 input. */
    kXBAR1_InputRESERVED173         = 173|0x100U,  /**< XBAR1_IN173 input is reserved. */
    kXBAR1_InputRESERVED174         = 174|0x100U,  /**< XBAR1_IN174 input is reserved. */
    kXBAR1_InputNetcTmrPp1          = 175|0x100U,  /**< NETC_TMR_PP1 output assigned to XBAR1_IN175 input. */
    kXBAR1_InputNetcTmrPp2          = 176|0x100U,  /**< NETC_TMR_PP2 output assigned to XBAR1_IN176 input. */
    kXBAR1_InputNetcTmrPp3          = 177|0x100U,  /**< NETC_TMR_PP3 output assigned to XBAR1_IN177 input. */
    kXBAR1_InputRESERVED178         = 178|0x100U,  /**< XBAR1_IN178 input is reserved. */
    kXBAR1_InputRESERVED179         = 179|0x100U,  /**< XBAR1_IN179 input is reserved. */
    kXBAR1_InputRESERVED180         = 180|0x100U,  /**< XBAR1_IN180 input is reserved. */
    kXBAR1_InputRESERVED181         = 181|0x100U,  /**< XBAR1_IN181 input is reserved. */
    kXBAR1_InputRESERVED182         = 182|0x100U,  /**< XBAR1_IN182 input is reserved. */
    kXBAR1_InputRESERVED183         = 183|0x100U,  /**< XBAR1_IN183 input is reserved. */
    kXBAR1_InputRESERVED184         = 184|0x100U,  /**< XBAR1_IN184 input is reserved. */
    kXBAR1_InputRESERVED185         = 185|0x100U,  /**< XBAR1_IN185 input is reserved. */
    kXBAR1_InputRESERVED186         = 186|0x100U,  /**< XBAR1_IN186 input is reserved. */
    kXBAR1_InputRESERVED187         = 187|0x100U,  /**< XBAR1_IN187 input is reserved. */
    kXBAR1_InputRESERVED188         = 188|0x100U,  /**< XBAR1_IN188 input is reserved. */
    kXBAR1_InputRESERVED189         = 189|0x100U,  /**< XBAR1_IN189 input is reserved. */
    kXBAR1_InputAoi1Out0            = 190|0x100U,  /**< AOI1_OUT0 output assigned to XBAR1_IN190 input. */
    kXBAR1_InputAoi1Out1            = 191|0x100U,  /**< AOI1_OUT1 output assigned to XBAR1_IN191 input. */
    kXBAR1_InputAoi1Out2            = 192|0x100U,  /**< AOI1_OUT2 output assigned to XBAR1_IN192 input. */
    kXBAR1_InputAoi1Out3            = 193|0x100U,  /**< AOI1_OUT3 output assigned to XBAR1_IN193 input. */
    kXBAR1_InputAoi2Out0            = 194|0x100U,  /**< AOI2_OUT0 output assigned to XBAR1_IN194 input. */
    kXBAR1_InputAoi2Out1            = 195|0x100U,  /**< AOI2_OUT1 output assigned to XBAR1_IN195 input. */
    kXBAR1_InputAoi2Out2            = 196|0x100U,  /**< AOI2_OUT2 output assigned to XBAR1_IN196 input. */
    kXBAR1_InputAoi2Out3            = 197|0x100U,  /**< AOI2_OUT3 output assigned to XBAR1_IN197 input. */
    kXBAR1_InputTriggerSyncOut4     = 198|0x100U,  /**< TRIGGER_SYNC_OUT4 output assigned to XBAR1_IN198 input. */
    kXBAR1_InputTriggerSyncOut5     = 199|0x100U,  /**< TRIGGER_SYNC_OUT5 output assigned to XBAR1_IN199 input. */
    kXBAR1_InputTriggerSyncOut6     = 200|0x100U,  /**< TRIGGER_SYNC_OUT6 output assigned to XBAR1_IN200 input. */
    kXBAR1_InputTriggerSyncOut7     = 201|0x100U,  /**< TRIGGER_SYNC_OUT7 output assigned to XBAR1_IN201 input. */
    kXBAR1_InputRESERVED202         = 202|0x100U,  /**< XBAR1_IN202 input is reserved. */
    kXBAR1_InputRESERVED203         = 203|0x100U,  /**< XBAR1_IN203 input is reserved. */
    kXBAR1_InputRESERVED204         = 204|0x100U,  /**< XBAR1_IN204 input is reserved. */
    kXBAR1_InputRESERVED205         = 205|0x100U,  /**< XBAR1_IN205 input is reserved. */
    kXBAR1_InputAoi3Out0            = 206|0x100U,  /**< AOI3_OUT0 output assigned to XBAR1_IN206 input. */
    kXBAR1_InputAoi3Out1            = 207|0x100U,  /**< AOI3_OUT1 output assigned to XBAR1_IN207 input. */
    kXBAR1_InputAoi3Out2            = 208|0x100U,  /**< AOI3_OUT2 output assigned to XBAR1_IN208 input. */
    kXBAR1_InputAoi3Out3            = 209|0x100U,  /**< AOI3_OUT3 output assigned to XBAR1_IN209 input. */
    kXBAR1_InputAoi4Out0            = 210|0x100U,  /**< AOI4_OUT0 output assigned to XBAR1_IN210 input. */
    kXBAR1_InputAoi4Out1            = 211|0x100U,  /**< AOI4_OUT1 output assigned to XBAR1_IN211 input. */
    kXBAR1_InputAoi4Out2            = 212|0x100U,  /**< AOI4_OUT2 output assigned to XBAR1_IN212 input. */
    kXBAR1_InputAoi4Out3            = 213|0x100U,  /**< AOI4_OUT3 output assigned to XBAR1_IN213 input. */
    kXBAR1_InputEcatSyncOut0        = 214|0x100U,  /**< ECAT_SYNC_OUT0 output assigned to XBAR1_IN214 input. */
    kXBAR1_InputEcatSyncOut1        = 215|0x100U,  /**< ECAT_SYNC_OUT1 output assigned to XBAR1_IN215 input. */
    kXBAR2_InputLogicLow            = 0|0x200U,    /**< LOGIC_LOW output assigned to XBAR2_IN0 input. */
    kXBAR2_InputLogicHigh           = 1|0x200U,    /**< LOGIC_HIGH output assigned to XBAR2_IN1 input. */
    kXBAR2_InputLogicLow1           = 2|0x200U,    /**< LOGIC_LOW1 output assigned to XBAR2_IN2 input. */
    kXBAR2_InputLogicHigh1          = 3|0x200U,    /**< LOGIC_HIGH1 output assigned to XBAR2_IN3 input. */
    kXBAR2_InputAcmp1Out            = 4|0x200U,    /**< ACMP1_OUT output assigned to XBAR2_IN4 input. */
    kXBAR2_InputAcmp2Out            = 5|0x200U,    /**< ACMP2_OUT output assigned to XBAR2_IN5 input. */
    kXBAR2_InputAcmp3Out            = 6|0x200U,    /**< ACMP3_OUT output assigned to XBAR2_IN6 input. */
    kXBAR2_InputAcmp4Out            = 7|0x200U,    /**< ACMP4_OUT output assigned to XBAR2_IN7 input. */
    kXBAR2_InputQtimer1Timer0       = 8|0x200U,    /**< QTIMER1_TIMER0 output assigned to XBAR2_IN8 input. */
    kXBAR2_InputQtimer1Timer1       = 9|0x200U,    /**< QTIMER1_TIMER1 output assigned to XBAR2_IN9 input. */
    kXBAR2_InputQtimer1Timer2       = 10|0x200U,   /**< QTIMER1_TIMER2 output assigned to XBAR2_IN10 input. */
    kXBAR2_InputQtimer1Timer3       = 11|0x200U,   /**< QTIMER1_TIMER3 output assigned to XBAR2_IN11 input. */
    kXBAR2_InputQtimer2Timer0       = 12|0x200U,   /**< QTIMER2_TIMER0 output assigned to XBAR2_IN12 input. */
    kXBAR2_InputQtimer2Timer1       = 13|0x200U,   /**< QTIMER2_TIMER1 output assigned to XBAR2_IN13 input. */
    kXBAR2_InputQtimer2Timer2       = 14|0x200U,   /**< QTIMER2_TIMER2 output assigned to XBAR2_IN14 input. */
    kXBAR2_InputQtimer2Timer3       = 15|0x200U,   /**< QTIMER2_TIMER3 output assigned to XBAR2_IN15 input. */
    kXBAR2_InputQtimer3Timer0       = 16|0x200U,   /**< QTIMER3_TIMER0 output assigned to XBAR2_IN16 input. */
    kXBAR2_InputQtimer3Timer1       = 17|0x200U,   /**< QTIMER3_TIMER1 output assigned to XBAR2_IN17 input. */
    kXBAR2_InputQtimer3Timer2       = 18|0x200U,   /**< QTIMER3_TIMER2 output assigned to XBAR2_IN18 input. */
    kXBAR2_InputQtimer3Timer3       = 19|0x200U,   /**< QTIMER3_TIMER3 output assigned to XBAR2_IN19 input. */
    kXBAR2_InputQtimer4Timer0       = 20|0x200U,   /**< QTIMER4_TIMER0 output assigned to XBAR2_IN20 input. */
    kXBAR2_InputQtimer4Timer1       = 21|0x200U,   /**< QTIMER4_TIMER1 output assigned to XBAR2_IN21 input. */
    kXBAR2_InputQtimer4Timer2       = 22|0x200U,   /**< QTIMER4_TIMER2 output assigned to XBAR2_IN22 input. */
    kXBAR2_InputQtimer4Timer3       = 23|0x200U,   /**< QTIMER4_TIMER3 output assigned to XBAR2_IN23 input. */
    kXBAR2_InputRESERVED24          = 24|0x200U,   /**< XBAR2_IN24 input is reserved. */
    kXBAR2_InputRESERVED25          = 25|0x200U,   /**< XBAR2_IN25 input is reserved. */
    kXBAR2_InputRESERVED26          = 26|0x200U,   /**< XBAR2_IN26 input is reserved. */
    kXBAR2_InputRESERVED27          = 27|0x200U,   /**< XBAR2_IN27 input is reserved. */
    kXBAR2_InputRESERVED28          = 28|0x200U,   /**< XBAR2_IN28 input is reserved. */
    kXBAR2_InputRESERVED29          = 29|0x200U,   /**< XBAR2_IN29 input is reserved. */
    kXBAR2_InputRESERVED30          = 30|0x200U,   /**< XBAR2_IN30 input is reserved. */
    kXBAR2_InputRESERVED31          = 31|0x200U,   /**< XBAR2_IN31 input is reserved. */
    kXBAR2_InputRESERVED32          = 32|0x200U,   /**< XBAR2_IN32 input is reserved. */
    kXBAR2_InputRESERVED33          = 33|0x200U,   /**< XBAR2_IN33 input is reserved. */
    kXBAR2_InputRESERVED34          = 34|0x200U,   /**< XBAR2_IN34 input is reserved. */
    kXBAR2_InputRESERVED35          = 35|0x200U,   /**< XBAR2_IN35 input is reserved. */
    kXBAR2_InputRESERVED36          = 36|0x200U,   /**< XBAR2_IN36 input is reserved. */
    kXBAR2_InputRESERVED37          = 37|0x200U,   /**< XBAR2_IN37 input is reserved. */
    kXBAR2_InputRESERVED38          = 38|0x200U,   /**< XBAR2_IN38 input is reserved. */
    kXBAR2_InputRESERVED39          = 39|0x200U,   /**< XBAR2_IN39 input is reserved. */
    kXBAR2_InputFlexpwm1Pwm0OutTrig01 = 40|0x200U, /**< FLEXPWM1_PWM0_OUT_TRIG0_1 output assigned to XBAR2_IN40 input. */
    kXBAR2_InputFlexpwm1Pwm1OutTrig01 = 41|0x200U, /**< FLEXPWM1_PWM1_OUT_TRIG0_1 output assigned to XBAR2_IN41 input. */
    kXBAR2_InputFlexpwm1Pwm2OutTrig01 = 42|0x200U, /**< FLEXPWM1_PWM2_OUT_TRIG0_1 output assigned to XBAR2_IN42 input. */
    kXBAR2_InputFlexpwm1Pwm3OutTrig01 = 43|0x200U, /**< FLEXPWM1_PWM3_OUT_TRIG0_1 output assigned to XBAR2_IN43 input. */
    kXBAR2_InputFlexpwm2Pwm0OutTrig01 = 44|0x200U, /**< FLEXPWM2_PWM0_OUT_TRIG0_1 output assigned to XBAR2_IN44 input. */
    kXBAR2_InputFlexpwm2Pwm1OutTrig01 = 45|0x200U, /**< FLEXPWM2_PWM1_OUT_TRIG0_1 output assigned to XBAR2_IN45 input. */
    kXBAR2_InputFlexpwm2Pwm2OutTrig01 = 46|0x200U, /**< FLEXPWM2_PWM2_OUT_TRIG0_1 output assigned to XBAR2_IN46 input. */
    kXBAR2_InputFlexpwm2Pwm3OutTrig01 = 47|0x200U, /**< FLEXPWM2_PWM3_OUT_TRIG0_1 output assigned to XBAR2_IN47 input. */
    kXBAR2_InputRESERVED48          = 48|0x200U,   /**< XBAR2_IN48 input is reserved. */
    kXBAR2_InputRESERVED49          = 49|0x200U,   /**< XBAR2_IN49 input is reserved. */
    kXBAR2_InputRESERVED50          = 50|0x200U,   /**< XBAR2_IN50 input is reserved. */
    kXBAR2_InputRESERVED51          = 51|0x200U,   /**< XBAR2_IN51 input is reserved. */
    kXBAR2_InputRESERVED52          = 52|0x200U,   /**< XBAR2_IN52 input is reserved. */
    kXBAR2_InputRESERVED53          = 53|0x200U,   /**< XBAR2_IN53 input is reserved. */
    kXBAR2_InputRESERVED54          = 54|0x200U,   /**< XBAR2_IN54 input is reserved. */
    kXBAR2_InputRESERVED55          = 55|0x200U,   /**< XBAR2_IN55 input is reserved. */
    kXBAR2_InputPit1Trigger0        = 56|0x200U,   /**< PIT1_TRIGGER0 output assigned to XBAR2_IN56 input. */
    kXBAR2_InputPit1Trigger1        = 57|0x200U,   /**< PIT1_TRIGGER1 output assigned to XBAR2_IN57 input. */
    kXBAR2_InputPit1Trigger2        = 58|0x200U,   /**< PIT1_TRIGGER2 output assigned to XBAR2_IN58 input. */
    kXBAR2_InputPit1Trigger3        = 59|0x200U,   /**< PIT1_TRIGGER3 output assigned to XBAR2_IN59 input. */
    kXBAR2_InputPit2Trigger0        = 60|0x200U,   /**< PIT2_TRIGGER0 output assigned to XBAR2_IN60 input. */
    kXBAR2_InputPit2Trigger1        = 61|0x200U,   /**< PIT2_TRIGGER1 output assigned to XBAR2_IN61 input. */
    kXBAR2_InputPit2Trigger2        = 62|0x200U,   /**< PIT2_TRIGGER2 output assigned to XBAR2_IN62 input. */
    kXBAR2_InputPit2Trigger3        = 63|0x200U,   /**< PIT2_TRIGGER3 output assigned to XBAR2_IN63 input. */
    kXBAR2_InputPit3Trigger0        = 64|0x200U,   /**< PIT3_TRIGGER0 output assigned to XBAR2_IN64 input. */
    kXBAR2_InputPit3Trigger1        = 65|0x200U,   /**< PIT3_TRIGGER1 output assigned to XBAR2_IN65 input. */
    kXBAR2_InputPit3Trigger2        = 66|0x200U,   /**< PIT3_TRIGGER2 output assigned to XBAR2_IN66 input. */
    kXBAR2_InputPit3Trigger3        = 67|0x200U,   /**< PIT3_TRIGGER3 output assigned to XBAR2_IN67 input. */
    kXBAR2_InputRESERVED68          = 68|0x200U,   /**< XBAR2_IN68 input is reserved. */
    kXBAR2_InputRESERVED69          = 69|0x200U,   /**< XBAR2_IN69 input is reserved. */
    kXBAR2_InputRESERVED70          = 70|0x200U,   /**< XBAR2_IN70 input is reserved. */
    kXBAR2_InputRESERVED71          = 71|0x200U,   /**< XBAR2_IN71 input is reserved. */
    kXBAR2_InputDma4TriggerOut0     = 72|0x200U,   /**< DMA4_TRIGGER_OUT0 output assigned to XBAR2_IN72 input. */
    kXBAR2_InputDma4TriggerOut1     = 73|0x200U,   /**< DMA4_TRIGGER_OUT1 output assigned to XBAR2_IN73 input. */
    kXBAR2_InputDma4TriggerOut2     = 74|0x200U,   /**< DMA4_TRIGGER_OUT2 output assigned to XBAR2_IN74 input. */
    kXBAR2_InputDma4TriggerOut3     = 75|0x200U,   /**< DMA4_TRIGGER_OUT3 output assigned to XBAR2_IN75 input. */
    kXBAR2_InputDma4TriggerOut4     = 76|0x200U,   /**< DMA4_TRIGGER_OUT4 output assigned to XBAR2_IN76 input. */
    kXBAR2_InputDma4TriggerOut5     = 77|0x200U,   /**< DMA4_TRIGGER_OUT5 output assigned to XBAR2_IN77 input. */
    kXBAR2_InputDma4TriggerOut6     = 78|0x200U,   /**< DMA4_TRIGGER_OUT6 output assigned to XBAR2_IN78 input. */
    kXBAR2_InputDma4TriggerOut7     = 79|0x200U,   /**< DMA4_TRIGGER_OUT7 output assigned to XBAR2_IN79 input. */
    kXBAR2_InputDma3TriggerOut0     = 80|0x200U,   /**< DMA3_TRIGGER_OUT0 output assigned to XBAR2_IN80 input. */
    kXBAR2_InputDma3TriggerOut1     = 81|0x200U,   /**< DMA3_TRIGGER_OUT1 output assigned to XBAR2_IN81 input. */
    kXBAR2_InputDma3TriggerOut2     = 82|0x200U,   /**< DMA3_TRIGGER_OUT2 output assigned to XBAR2_IN82 input. */
    kXBAR2_InputDma3TriggerOut3     = 83|0x200U,   /**< DMA3_TRIGGER_OUT3 output assigned to XBAR2_IN83 input. */
    kXBAR2_InputDma3TriggerOut4     = 84|0x200U,   /**< DMA3_TRIGGER_OUT4 output assigned to XBAR2_IN84 input. */
    kXBAR2_InputDma3TriggerOut5     = 85|0x200U,   /**< DMA3_TRIGGER_OUT5 output assigned to XBAR2_IN85 input. */
    kXBAR2_InputDma3TriggerOut6     = 86|0x200U,   /**< DMA3_TRIGGER_OUT6 output assigned to XBAR2_IN86 input. */
    kXBAR2_InputDma3TriggerOut7     = 87|0x200U,   /**< DMA3_TRIGGER_OUT7 output assigned to XBAR2_IN87 input. */
    kXBAR2_InputAdc1TcompPulse0     = 88|0x200U,   /**< ADC1_TCOMP_PULSE0 output assigned to XBAR2_IN88 input. */
    kXBAR2_InputAdc1TcompPulse1     = 89|0x200U,   /**< ADC1_TCOMP_PULSE1 output assigned to XBAR2_IN89 input. */
    kXBAR2_InputAdc1TcompPulse2     = 90|0x200U,   /**< ADC1_TCOMP_PULSE2 output assigned to XBAR2_IN90 input. */
    kXBAR2_InputAdc1TcompPulse3     = 91|0x200U,   /**< ADC1_TCOMP_PULSE3 output assigned to XBAR2_IN91 input. */
    kXBAR2_InputAdc1TcompPulse4     = 92|0x200U,   /**< ADC1_TCOMP_PULSE4 output assigned to XBAR2_IN92 input. */
    kXBAR2_InputAdc1TcompPulse5     = 93|0x200U,   /**< ADC1_TCOMP_PULSE5 output assigned to XBAR2_IN93 input. */
    kXBAR2_InputAdc1TcompPulse6     = 94|0x200U,   /**< ADC1_TCOMP_PULSE6 output assigned to XBAR2_IN94 input. */
    kXBAR2_InputAdc1TcompPulse7     = 95|0x200U,   /**< ADC1_TCOMP_PULSE7 output assigned to XBAR2_IN95 input. */
    kXBAR2_InputRESERVED96          = 96|0x200U,   /**< XBAR2_IN96 input is reserved. */
    kXBAR2_InputRESERVED97          = 97|0x200U,   /**< XBAR2_IN97 input is reserved. */
    kXBAR2_InputRESERVED98          = 98|0x200U,   /**< XBAR2_IN98 input is reserved. */
    kXBAR2_InputRESERVED99          = 99|0x200U,   /**< XBAR2_IN99 input is reserved. */
    kXBAR2_InputRESERVED100         = 100|0x200U,  /**< XBAR2_IN100 input is reserved. */
    kXBAR2_InputRESERVED101         = 101|0x200U,  /**< XBAR2_IN101 input is reserved. */
    kXBAR2_InputRESERVED102         = 102|0x200U,  /**< XBAR2_IN102 input is reserved. */
    kXBAR2_InputRESERVED103         = 103|0x200U,  /**< XBAR2_IN103 input is reserved. */
    kXBAR2_InputTpm1ChTrigger0      = 104|0x200U,  /**< TPM1_CH_TRIGGER0 output assigned to XBAR2_IN104 input. */
    kXBAR2_InputTpm1ChTrigger1      = 105|0x200U,  /**< TPM1_CH_TRIGGER1 output assigned to XBAR2_IN105 input. */
    kXBAR2_InputTpm1ChTrigger2      = 106|0x200U,  /**< TPM1_CH_TRIGGER2 output assigned to XBAR2_IN106 input. */
    kXBAR2_InputTpm1ChTrigger3      = 107|0x200U,  /**< TPM1_CH_TRIGGER3 output assigned to XBAR2_IN107 input. */
    kXBAR2_InputTpm1Trigger         = 108|0x200U,  /**< TPM1_TRIGGER output assigned to XBAR2_IN108 input. */
    kXBAR2_InputTpm2ChTrigger0      = 109|0x200U,  /**< TPM2_CH_TRIGGER0 output assigned to XBAR2_IN109 input. */
    kXBAR2_InputTpm2ChTrigger1      = 110|0x200U,  /**< TPM2_CH_TRIGGER1 output assigned to XBAR2_IN110 input. */
    kXBAR2_InputTpm2ChTrigger2      = 111|0x200U,  /**< TPM2_CH_TRIGGER2 output assigned to XBAR2_IN111 input. */
    kXBAR2_InputTpm2ChTrigger3      = 112|0x200U,  /**< TPM2_CH_TRIGGER3 output assigned to XBAR2_IN112 input. */
    kXBAR2_InputTpm2Trigger         = 113|0x200U,  /**< TPM2_TRIGGER output assigned to XBAR2_IN113 input. */
    kXBAR2_InputTpm3ChTrigger0      = 114|0x200U,  /**< TPM3_CH_TRIGGER0 output assigned to XBAR2_IN114 input. */
    kXBAR2_InputTpm3ChTrigger1      = 115|0x200U,  /**< TPM3_CH_TRIGGER1 output assigned to XBAR2_IN115 input. */
    kXBAR2_InputTpm3ChTrigger2      = 116|0x200U,  /**< TPM3_CH_TRIGGER2 output assigned to XBAR2_IN116 input. */
    kXBAR2_InputTpm3ChTrigger3      = 117|0x200U,  /**< TPM3_CH_TRIGGER3 output assigned to XBAR2_IN117 input. */
    kXBAR2_InputTpm3Trigger         = 118|0x200U,  /**< TPM3_TRIGGER output assigned to XBAR2_IN118 input. */
    kXBAR2_InputRESERVED119         = 119|0x200U,  /**< XBAR2_IN119 input is reserved. */
    kXBAR2_InputRESERVED120         = 120|0x200U,  /**< XBAR2_IN120 input is reserved. */
    kXBAR2_InputRESERVED121         = 121|0x200U,  /**< XBAR2_IN121 input is reserved. */
    kXBAR2_InputRESERVED122         = 122|0x200U,  /**< XBAR2_IN122 input is reserved. */
    kXBAR2_InputRESERVED123         = 123|0x200U,  /**< XBAR2_IN123 input is reserved. */
    kXBAR2_InputRESERVED124         = 124|0x200U,  /**< XBAR2_IN124 input is reserved. */
    kXBAR2_InputRESERVED125         = 125|0x200U,  /**< XBAR2_IN125 input is reserved. */
    kXBAR2_InputRESERVED126         = 126|0x200U,  /**< XBAR2_IN126 input is reserved. */
    kXBAR2_InputRESERVED127         = 127|0x200U,  /**< XBAR2_IN127 input is reserved. */
    kXBAR2_InputRESERVED128         = 128|0x200U,  /**< XBAR2_IN128 input is reserved. */
    kXBAR2_InputRESERVED129         = 129|0x200U,  /**< XBAR2_IN129 input is reserved. */
    kXBAR2_InputRESERVED130         = 130|0x200U,  /**< XBAR2_IN130 input is reserved. */
    kXBAR2_InputRESERVED131         = 131|0x200U,  /**< XBAR2_IN131 input is reserved. */
    kXBAR2_InputRESERVED132         = 132|0x200U,  /**< XBAR2_IN132 input is reserved. */
    kXBAR2_InputRESERVED133         = 133|0x200U,  /**< XBAR2_IN133 input is reserved. */
    kXBAR2_InputLptmr1TriggerDelay  = 134|0x200U,  /**< LPTMR1_TRIGGER_DELAY output assigned to XBAR2_IN134 input. */
    kXBAR2_InputRESERVED135         = 135|0x200U,  /**< XBAR2_IN135 input is reserved. */
    kXBAR2_InputRESERVED136         = 136|0x200U,  /**< XBAR2_IN136 input is reserved. */
    kXBAR2_InputNetcTmrPp1          = 137|0x200U,  /**< NETC_TMR_PP1 output assigned to XBAR2_IN137 input. */
    kXBAR2_InputNetcTmrPp2          = 138|0x200U,  /**< NETC_TMR_PP2 output assigned to XBAR2_IN138 input. */
    kXBAR2_InputNetcTmrPp3          = 139|0x200U,  /**< NETC_TMR_PP3 output assigned to XBAR2_IN139 input. */
    kXBAR2_InputRESERVED140         = 140|0x200U,  /**< XBAR2_IN140 input is reserved. */
    kXBAR2_InputRESERVED141         = 141|0x200U,  /**< XBAR2_IN141 input is reserved. */
    kXBAR2_InputRESERVED142         = 142|0x200U,  /**< XBAR2_IN142 input is reserved. */
    kXBAR2_InputRESERVED143         = 143|0x200U,  /**< XBAR2_IN143 input is reserved. */
    kXBAR2_InputRESERVED144         = 144|0x200U,  /**< XBAR2_IN144 input is reserved. */
    kXBAR2_InputRESERVED145         = 145|0x200U,  /**< XBAR2_IN145 input is reserved. */
    kXBAR2_InputRESERVED146         = 146|0x200U,  /**< XBAR2_IN146 input is reserved. */
    kXBAR2_InputRESERVED147         = 147|0x200U,  /**< XBAR2_IN147 input is reserved. */
    kXBAR2_InputRESERVED148         = 148|0x200U,  /**< XBAR2_IN148 input is reserved. */
    kXBAR2_InputRESERVED149         = 149|0x200U,  /**< XBAR2_IN149 input is reserved. */
    kXBAR2_InputRESERVED150         = 150|0x200U,  /**< XBAR2_IN150 input is reserved. */
    kXBAR2_InputRESERVED151         = 151|0x200U,  /**< XBAR2_IN151 input is reserved. */
    kXBAR2_InputRESERVED152         = 152|0x200U,  /**< XBAR2_IN152 input is reserved. */
    kXBAR2_InputRESERVED153         = 153|0x200U,  /**< XBAR2_IN153 input is reserved. */
    kXBAR2_InputRESERVED154         = 154|0x200U,  /**< XBAR2_IN154 input is reserved. */
    kXBAR2_InputRESERVED155         = 155|0x200U,  /**< XBAR2_IN155 input is reserved. */
    kXBAR2_InputEqdc1PosMatch0      = 156|0x200U,  /**< EQDC1_POS_MATCH0 output assigned to XBAR2_IN156 input. */
    kXBAR2_InputEqdc1PosMatch1      = 157|0x200U,  /**< EQDC1_POS_MATCH1 output assigned to XBAR2_IN157 input. */
    kXBAR2_InputEqdc1PosMatch2      = 158|0x200U,  /**< EQDC1_POS_MATCH2 output assigned to XBAR2_IN158 input. */
    kXBAR2_InputEqdc1PosMatch3      = 159|0x200U,  /**< EQDC1_POS_MATCH3 output assigned to XBAR2_IN159 input. */
    kXBAR2_InputEqdc1CompFlg0       = 160|0x200U,  /**< EQDC1_COMP_FLG0 output assigned to XBAR2_IN160 input. */
    kXBAR2_InputEqdc1CompFlg1       = 161|0x200U,  /**< EQDC1_COMP_FLG1 output assigned to XBAR2_IN161 input. */
    kXBAR2_InputEqdc1CompFlg2       = 162|0x200U,  /**< EQDC1_COMP_FLG2 output assigned to XBAR2_IN162 input. */
    kXBAR2_InputEqdc1CompFlg3       = 163|0x200U,  /**< EQDC1_COMP_FLG3 output assigned to XBAR2_IN163 input. */
    kXBAR2_InputEqdc1CntDn          = 164|0x200U,  /**< EQDC1_CNT_DN output assigned to XBAR2_IN164 input. */
    kXBAR2_InputEqdc1CntUp          = 165|0x200U,  /**< EQDC1_CNT_UP output assigned to XBAR2_IN165 input. */
    kXBAR2_InputEqdc1CntDir         = 166|0x200U,  /**< EQDC1_CNT_DIR output assigned to XBAR2_IN166 input. */
    kXBAR2_InputRESERVED167         = 167|0x200U,  /**< XBAR2_IN167 input is reserved. */
    kXBAR2_InputRESERVED168         = 168|0x200U,  /**< XBAR2_IN168 input is reserved. */
    kXBAR2_InputRESERVED169         = 169|0x200U,  /**< XBAR2_IN169 input is reserved. */
    kXBAR2_InputRESERVED170         = 170|0x200U,  /**< XBAR2_IN170 input is reserved. */
    kXBAR2_InputRESERVED171         = 171|0x200U,  /**< XBAR2_IN171 input is reserved. */
    kXBAR2_InputRESERVED172         = 172|0x200U,  /**< XBAR2_IN172 input is reserved. */
    kXBAR2_InputRESERVED173         = 173|0x200U,  /**< XBAR2_IN173 input is reserved. */
    kXBAR2_InputRESERVED174         = 174|0x200U,  /**< XBAR2_IN174 input is reserved. */
    kXBAR2_InputRESERVED175         = 175|0x200U,  /**< XBAR2_IN175 input is reserved. */
    kXBAR2_InputRESERVED176         = 176|0x200U,  /**< XBAR2_IN176 input is reserved. */
    kXBAR2_InputRESERVED177         = 177|0x200U,  /**< XBAR2_IN177 input is reserved. */
    kXBAR3_InputLogicLow            = 0|0x300U,    /**< LOGIC_LOW output assigned to XBAR3_IN0 input. */
    kXBAR3_InputLogicHigh           = 1|0x300U,    /**< LOGIC_HIGH output assigned to XBAR3_IN1 input. */
    kXBAR3_InputLogicLow1           = 2|0x300U,    /**< LOGIC_LOW1 output assigned to XBAR3_IN2 input. */
    kXBAR3_InputLogicHigh1          = 3|0x300U,    /**< LOGIC_HIGH1 output assigned to XBAR3_IN3 input. */
    kXBAR3_InputAcmp1Out            = 4|0x300U,    /**< ACMP1_OUT output assigned to XBAR3_IN4 input. */
    kXBAR3_InputAcmp2Out            = 5|0x300U,    /**< ACMP2_OUT output assigned to XBAR3_IN5 input. */
    kXBAR3_InputAcmp3Out            = 6|0x300U,    /**< ACMP3_OUT output assigned to XBAR3_IN6 input. */
    kXBAR3_InputAcmp4Out            = 7|0x300U,    /**< ACMP4_OUT output assigned to XBAR3_IN7 input. */
    kXBAR3_InputQtimer1Timer0       = 8|0x300U,    /**< QTIMER1_TIMER0 output assigned to XBAR3_IN8 input. */
    kXBAR3_InputQtimer1Timer1       = 9|0x300U,    /**< QTIMER1_TIMER1 output assigned to XBAR3_IN9 input. */
    kXBAR3_InputQtimer1Timer2       = 10|0x300U,   /**< QTIMER1_TIMER2 output assigned to XBAR3_IN10 input. */
    kXBAR3_InputQtimer1Timer3       = 11|0x300U,   /**< QTIMER1_TIMER3 output assigned to XBAR3_IN11 input. */
    kXBAR3_InputQtimer2Timer0       = 12|0x300U,   /**< QTIMER2_TIMER0 output assigned to XBAR3_IN12 input. */
    kXBAR3_InputQtimer2Timer1       = 13|0x300U,   /**< QTIMER2_TIMER1 output assigned to XBAR3_IN13 input. */
    kXBAR3_InputQtimer2Timer2       = 14|0x300U,   /**< QTIMER2_TIMER2 output assigned to XBAR3_IN14 input. */
    kXBAR3_InputQtimer2Timer3       = 15|0x300U,   /**< QTIMER2_TIMER3 output assigned to XBAR3_IN15 input. */
    kXBAR3_InputQtimer3Timer0       = 16|0x300U,   /**< QTIMER3_TIMER0 output assigned to XBAR3_IN16 input. */
    kXBAR3_InputQtimer3Timer1       = 17|0x300U,   /**< QTIMER3_TIMER1 output assigned to XBAR3_IN17 input. */
    kXBAR3_InputQtimer3Timer2       = 18|0x300U,   /**< QTIMER3_TIMER2 output assigned to XBAR3_IN18 input. */
    kXBAR3_InputQtimer3Timer3       = 19|0x300U,   /**< QTIMER3_TIMER3 output assigned to XBAR3_IN19 input. */
    kXBAR3_InputQtimer4Timer0       = 20|0x300U,   /**< QTIMER4_TIMER0 output assigned to XBAR3_IN20 input. */
    kXBAR3_InputQtimer4Timer1       = 21|0x300U,   /**< QTIMER4_TIMER1 output assigned to XBAR3_IN21 input. */
    kXBAR3_InputQtimer4Timer2       = 22|0x300U,   /**< QTIMER4_TIMER2 output assigned to XBAR3_IN22 input. */
    kXBAR3_InputQtimer4Timer3       = 23|0x300U,   /**< QTIMER4_TIMER3 output assigned to XBAR3_IN23 input. */
    kXBAR3_InputRESERVED24          = 24|0x300U,   /**< XBAR3_IN24 input is reserved. */
    kXBAR3_InputRESERVED25          = 25|0x300U,   /**< XBAR3_IN25 input is reserved. */
    kXBAR3_InputRESERVED26          = 26|0x300U,   /**< XBAR3_IN26 input is reserved. */
    kXBAR3_InputRESERVED27          = 27|0x300U,   /**< XBAR3_IN27 input is reserved. */
    kXBAR3_InputRESERVED28          = 28|0x300U,   /**< XBAR3_IN28 input is reserved. */
    kXBAR3_InputRESERVED29          = 29|0x300U,   /**< XBAR3_IN29 input is reserved. */
    kXBAR3_InputRESERVED30          = 30|0x300U,   /**< XBAR3_IN30 input is reserved. */
    kXBAR3_InputRESERVED31          = 31|0x300U,   /**< XBAR3_IN31 input is reserved. */
    kXBAR3_InputRESERVED32          = 32|0x300U,   /**< XBAR3_IN32 input is reserved. */
    kXBAR3_InputRESERVED33          = 33|0x300U,   /**< XBAR3_IN33 input is reserved. */
    kXBAR3_InputRESERVED34          = 34|0x300U,   /**< XBAR3_IN34 input is reserved. */
    kXBAR3_InputRESERVED35          = 35|0x300U,   /**< XBAR3_IN35 input is reserved. */
    kXBAR3_InputRESERVED36          = 36|0x300U,   /**< XBAR3_IN36 input is reserved. */
    kXBAR3_InputRESERVED37          = 37|0x300U,   /**< XBAR3_IN37 input is reserved. */
    kXBAR3_InputRESERVED38          = 38|0x300U,   /**< XBAR3_IN38 input is reserved. */
    kXBAR3_InputRESERVED39          = 39|0x300U,   /**< XBAR3_IN39 input is reserved. */
    kXBAR3_InputFlexpwm1Pwm0OutTrig01 = 40|0x300U, /**< FLEXPWM1_PWM0_OUT_TRIG0_1 output assigned to XBAR3_IN40 input. */
    kXBAR3_InputFlexpwm1Pwm1OutTrig01 = 41|0x300U, /**< FLEXPWM1_PWM1_OUT_TRIG0_1 output assigned to XBAR3_IN41 input. */
    kXBAR3_InputFlexpwm1Pwm2OutTrig01 = 42|0x300U, /**< FLEXPWM1_PWM2_OUT_TRIG0_1 output assigned to XBAR3_IN42 input. */
    kXBAR3_InputFlexpwm1Pwm3OutTrig01 = 43|0x300U, /**< FLEXPWM1_PWM3_OUT_TRIG0_1 output assigned to XBAR3_IN43 input. */
    kXBAR3_InputFlexpwm2Pwm0OutTrig01 = 44|0x300U, /**< FLEXPWM2_PWM0_OUT_TRIG0_1 output assigned to XBAR3_IN44 input. */
    kXBAR3_InputFlexpwm2Pwm1OutTrig01 = 45|0x300U, /**< FLEXPWM2_PWM1_OUT_TRIG0_1 output assigned to XBAR3_IN45 input. */
    kXBAR3_InputFlexpwm2Pwm2OutTrig01 = 46|0x300U, /**< FLEXPWM2_PWM2_OUT_TRIG0_1 output assigned to XBAR3_IN46 input. */
    kXBAR3_InputFlexpwm2Pwm3OutTrig01 = 47|0x300U, /**< FLEXPWM2_PWM3_OUT_TRIG0_1 output assigned to XBAR3_IN47 input. */
    kXBAR3_InputRESERVED48          = 48|0x300U,   /**< XBAR3_IN48 input is reserved. */
    kXBAR3_InputRESERVED49          = 49|0x300U,   /**< XBAR3_IN49 input is reserved. */
    kXBAR3_InputRESERVED50          = 50|0x300U,   /**< XBAR3_IN50 input is reserved. */
    kXBAR3_InputRESERVED51          = 51|0x300U,   /**< XBAR3_IN51 input is reserved. */
    kXBAR3_InputRESERVED52          = 52|0x300U,   /**< XBAR3_IN52 input is reserved. */
    kXBAR3_InputRESERVED53          = 53|0x300U,   /**< XBAR3_IN53 input is reserved. */
    kXBAR3_InputRESERVED54          = 54|0x300U,   /**< XBAR3_IN54 input is reserved. */
    kXBAR3_InputRESERVED55          = 55|0x300U,   /**< XBAR3_IN55 input is reserved. */
    kXBAR3_InputPit1Trigger0        = 56|0x300U,   /**< PIT1_TRIGGER0 output assigned to XBAR3_IN56 input. */
    kXBAR3_InputPit1Trigger1        = 57|0x300U,   /**< PIT1_TRIGGER1 output assigned to XBAR3_IN57 input. */
    kXBAR3_InputPit1Trigger2        = 58|0x300U,   /**< PIT1_TRIGGER2 output assigned to XBAR3_IN58 input. */
    kXBAR3_InputPit1Trigger3        = 59|0x300U,   /**< PIT1_TRIGGER3 output assigned to XBAR3_IN59 input. */
    kXBAR3_InputPit2Trigger0        = 60|0x300U,   /**< PIT2_TRIGGER0 output assigned to XBAR3_IN60 input. */
    kXBAR3_InputPit2Trigger1        = 61|0x300U,   /**< PIT2_TRIGGER1 output assigned to XBAR3_IN61 input. */
    kXBAR3_InputPit2Trigger2        = 62|0x300U,   /**< PIT2_TRIGGER2 output assigned to XBAR3_IN62 input. */
    kXBAR3_InputPit2Trigger3        = 63|0x300U,   /**< PIT2_TRIGGER3 output assigned to XBAR3_IN63 input. */
    kXBAR3_InputPit3Trigger0        = 64|0x300U,   /**< PIT3_TRIGGER0 output assigned to XBAR3_IN64 input. */
    kXBAR3_InputPit3Trigger1        = 65|0x300U,   /**< PIT3_TRIGGER1 output assigned to XBAR3_IN65 input. */
    kXBAR3_InputPit3Trigger2        = 66|0x300U,   /**< PIT3_TRIGGER2 output assigned to XBAR3_IN66 input. */
    kXBAR3_InputPit3Trigger3        = 67|0x300U,   /**< PIT3_TRIGGER3 output assigned to XBAR3_IN67 input. */
    kXBAR3_InputRESERVED68          = 68|0x300U,   /**< XBAR3_IN68 input is reserved. */
    kXBAR3_InputRESERVED69          = 69|0x300U,   /**< XBAR3_IN69 input is reserved. */
    kXBAR3_InputRESERVED70          = 70|0x300U,   /**< XBAR3_IN70 input is reserved. */
    kXBAR3_InputRESERVED71          = 71|0x300U,   /**< XBAR3_IN71 input is reserved. */
    kXBAR3_InputDma4TriggerOut0     = 72|0x300U,   /**< DMA4_TRIGGER_OUT0 output assigned to XBAR3_IN72 input. */
    kXBAR3_InputDma4TriggerOut1     = 73|0x300U,   /**< DMA4_TRIGGER_OUT1 output assigned to XBAR3_IN73 input. */
    kXBAR3_InputDma4TriggerOut2     = 74|0x300U,   /**< DMA4_TRIGGER_OUT2 output assigned to XBAR3_IN74 input. */
    kXBAR3_InputDma4TriggerOut3     = 75|0x300U,   /**< DMA4_TRIGGER_OUT3 output assigned to XBAR3_IN75 input. */
    kXBAR3_InputDma4TriggerOut4     = 76|0x300U,   /**< DMA4_TRIGGER_OUT4 output assigned to XBAR3_IN76 input. */
    kXBAR3_InputDma4TriggerOut5     = 77|0x300U,   /**< DMA4_TRIGGER_OUT5 output assigned to XBAR3_IN77 input. */
    kXBAR3_InputDma4TriggerOut6     = 78|0x300U,   /**< DMA4_TRIGGER_OUT6 output assigned to XBAR3_IN78 input. */
    kXBAR3_InputDma4TriggerOut7     = 79|0x300U,   /**< DMA4_TRIGGER_OUT7 output assigned to XBAR3_IN79 input. */
    kXBAR3_InputDma3TriggerOut0     = 80|0x300U,   /**< DMA3_TRIGGER_OUT0 output assigned to XBAR3_IN80 input. */
    kXBAR3_InputDma3TriggerOut1     = 81|0x300U,   /**< DMA3_TRIGGER_OUT1 output assigned to XBAR3_IN81 input. */
    kXBAR3_InputDma3TriggerOut2     = 82|0x300U,   /**< DMA3_TRIGGER_OUT2 output assigned to XBAR3_IN82 input. */
    kXBAR3_InputDma3TriggerOut3     = 83|0x300U,   /**< DMA3_TRIGGER_OUT3 output assigned to XBAR3_IN83 input. */
    kXBAR3_InputDma3TriggerOut4     = 84|0x300U,   /**< DMA3_TRIGGER_OUT4 output assigned to XBAR3_IN84 input. */
    kXBAR3_InputDma3TriggerOut5     = 85|0x300U,   /**< DMA3_TRIGGER_OUT5 output assigned to XBAR3_IN85 input. */
    kXBAR3_InputDma3TriggerOut6     = 86|0x300U,   /**< DMA3_TRIGGER_OUT6 output assigned to XBAR3_IN86 input. */
    kXBAR3_InputDma3TriggerOut7     = 87|0x300U,   /**< DMA3_TRIGGER_OUT7 output assigned to XBAR3_IN87 input. */
    kXBAR3_InputAdc1TcompPulse0     = 88|0x300U,   /**< ADC1_TCOMP_PULSE0 output assigned to XBAR3_IN88 input. */
    kXBAR3_InputAdc1TcompPulse1     = 89|0x300U,   /**< ADC1_TCOMP_PULSE1 output assigned to XBAR3_IN89 input. */
    kXBAR3_InputAdc1TcompPulse2     = 90|0x300U,   /**< ADC1_TCOMP_PULSE2 output assigned to XBAR3_IN90 input. */
    kXBAR3_InputAdc1TcompPulse3     = 91|0x300U,   /**< ADC1_TCOMP_PULSE3 output assigned to XBAR3_IN91 input. */
    kXBAR3_InputAdc1TcompPulse4     = 92|0x300U,   /**< ADC1_TCOMP_PULSE4 output assigned to XBAR3_IN92 input. */
    kXBAR3_InputAdc1TcompPulse5     = 93|0x300U,   /**< ADC1_TCOMP_PULSE5 output assigned to XBAR3_IN93 input. */
    kXBAR3_InputAdc1TcompPulse6     = 94|0x300U,   /**< ADC1_TCOMP_PULSE6 output assigned to XBAR3_IN94 input. */
    kXBAR3_InputAdc1TcompPulse7     = 95|0x300U,   /**< ADC1_TCOMP_PULSE7 output assigned to XBAR3_IN95 input. */
    kXBAR3_InputRESERVED96          = 96|0x300U,   /**< XBAR3_IN96 input is reserved. */
    kXBAR3_InputRESERVED97          = 97|0x300U,   /**< XBAR3_IN97 input is reserved. */
    kXBAR3_InputRESERVED98          = 98|0x300U,   /**< XBAR3_IN98 input is reserved. */
    kXBAR3_InputRESERVED99          = 99|0x300U,   /**< XBAR3_IN99 input is reserved. */
    kXBAR3_InputRESERVED100         = 100|0x300U,  /**< XBAR3_IN100 input is reserved. */
    kXBAR3_InputRESERVED101         = 101|0x300U,  /**< XBAR3_IN101 input is reserved. */
    kXBAR3_InputRESERVED102         = 102|0x300U,  /**< XBAR3_IN102 input is reserved. */
    kXBAR3_InputRESERVED103         = 103|0x300U,  /**< XBAR3_IN103 input is reserved. */
    kXBAR3_InputTpm1ChTrigger0      = 104|0x300U,  /**< TPM1_CH_TRIGGER0 output assigned to XBAR3_IN104 input. */
    kXBAR3_InputTpm1ChTrigger1      = 105|0x300U,  /**< TPM1_CH_TRIGGER1 output assigned to XBAR3_IN105 input. */
    kXBAR3_InputTpm1ChTrigger2      = 106|0x300U,  /**< TPM1_CH_TRIGGER2 output assigned to XBAR3_IN106 input. */
    kXBAR3_InputTpm1ChTrigger3      = 107|0x300U,  /**< TPM1_CH_TRIGGER3 output assigned to XBAR3_IN107 input. */
    kXBAR3_InputTpm1Trigger         = 108|0x300U,  /**< TPM1_TRIGGER output assigned to XBAR3_IN108 input. */
    kXBAR3_InputTpm2ChTrigger0      = 109|0x300U,  /**< TPM2_CH_TRIGGER0 output assigned to XBAR3_IN109 input. */
    kXBAR3_InputTpm2ChTrigger1      = 110|0x300U,  /**< TPM2_CH_TRIGGER1 output assigned to XBAR3_IN110 input. */
    kXBAR3_InputTpm2ChTrigger2      = 111|0x300U,  /**< TPM2_CH_TRIGGER2 output assigned to XBAR3_IN111 input. */
    kXBAR3_InputTpm2ChTrigger3      = 112|0x300U,  /**< TPM2_CH_TRIGGER3 output assigned to XBAR3_IN112 input. */
    kXBAR3_InputTpm2Trigger         = 113|0x300U,  /**< TPM2_TRIGGER output assigned to XBAR3_IN113 input. */
    kXBAR3_InputTpm3ChTrigger0      = 114|0x300U,  /**< TPM3_CH_TRIGGER0 output assigned to XBAR3_IN114 input. */
    kXBAR3_InputTpm3ChTrigger1      = 115|0x300U,  /**< TPM3_CH_TRIGGER1 output assigned to XBAR3_IN115 input. */
    kXBAR3_InputTpm3ChTrigger2      = 116|0x300U,  /**< TPM3_CH_TRIGGER2 output assigned to XBAR3_IN116 input. */
    kXBAR3_InputTpm3ChTrigger3      = 117|0x300U,  /**< TPM3_CH_TRIGGER3 output assigned to XBAR3_IN117 input. */
    kXBAR3_InputTpm3Trigger         = 118|0x300U,  /**< TPM3_TRIGGER output assigned to XBAR3_IN118 input. */
    kXBAR3_InputRESERVED119         = 119|0x300U,  /**< XBAR3_IN119 input is reserved. */
    kXBAR3_InputRESERVED120         = 120|0x300U,  /**< XBAR3_IN120 input is reserved. */
    kXBAR3_InputRESERVED121         = 121|0x300U,  /**< XBAR3_IN121 input is reserved. */
    kXBAR3_InputRESERVED122         = 122|0x300U,  /**< XBAR3_IN122 input is reserved. */
    kXBAR3_InputRESERVED123         = 123|0x300U,  /**< XBAR3_IN123 input is reserved. */
    kXBAR3_InputRESERVED124         = 124|0x300U,  /**< XBAR3_IN124 input is reserved. */
    kXBAR3_InputRESERVED125         = 125|0x300U,  /**< XBAR3_IN125 input is reserved. */
    kXBAR3_InputRESERVED126         = 126|0x300U,  /**< XBAR3_IN126 input is reserved. */
    kXBAR3_InputRESERVED127         = 127|0x300U,  /**< XBAR3_IN127 input is reserved. */
    kXBAR3_InputRESERVED128         = 128|0x300U,  /**< XBAR3_IN128 input is reserved. */
    kXBAR3_InputRESERVED129         = 129|0x300U,  /**< XBAR3_IN129 input is reserved. */
    kXBAR3_InputRESERVED130         = 130|0x300U,  /**< XBAR3_IN130 input is reserved. */
    kXBAR3_InputRESERVED131         = 131|0x300U,  /**< XBAR3_IN131 input is reserved. */
    kXBAR3_InputRESERVED132         = 132|0x300U,  /**< XBAR3_IN132 input is reserved. */
    kXBAR3_InputRESERVED133         = 133|0x300U,  /**< XBAR3_IN133 input is reserved. */
    kXBAR3_InputLptmr1TriggerDelay  = 134|0x300U,  /**< LPTMR1_TRIGGER_DELAY output assigned to XBAR3_IN134 input. */
    kXBAR3_InputRESERVED135         = 135|0x300U,  /**< XBAR3_IN135 input is reserved. */
    kXBAR3_InputRESERVED136         = 136|0x300U,  /**< XBAR3_IN136 input is reserved. */
    kXBAR3_InputNetcTmrPp1          = 137|0x300U,  /**< NETC_TMR_PP1 output assigned to XBAR3_IN137 input. */
    kXBAR3_InputNetcTmrPp2          = 138|0x300U,  /**< NETC_TMR_PP2 output assigned to XBAR3_IN138 input. */
    kXBAR3_InputNetcTmrPp3          = 139|0x300U,  /**< NETC_TMR_PP3 output assigned to XBAR3_IN139 input. */
    kXBAR3_InputRESERVED140         = 140|0x300U,  /**< XBAR3_IN140 input is reserved. */
    kXBAR3_InputRESERVED141         = 141|0x300U,  /**< XBAR3_IN141 input is reserved. */
    kXBAR3_InputRESERVED142         = 142|0x300U,  /**< XBAR3_IN142 input is reserved. */
    kXBAR3_InputRESERVED143         = 143|0x300U,  /**< XBAR3_IN143 input is reserved. */
    kXBAR3_InputRESERVED144         = 144|0x300U,  /**< XBAR3_IN144 input is reserved. */
    kXBAR3_InputRESERVED145         = 145|0x300U,  /**< XBAR3_IN145 input is reserved. */
    kXBAR3_InputRESERVED146         = 146|0x300U,  /**< XBAR3_IN146 input is reserved. */
    kXBAR3_InputRESERVED147         = 147|0x300U,  /**< XBAR3_IN147 input is reserved. */
    kXBAR3_InputRESERVED148         = 148|0x300U,  /**< XBAR3_IN148 input is reserved. */
    kXBAR3_InputRESERVED149         = 149|0x300U,  /**< XBAR3_IN149 input is reserved. */
    kXBAR3_InputRESERVED150         = 150|0x300U,  /**< XBAR3_IN150 input is reserved. */
    kXBAR3_InputRESERVED151         = 151|0x300U,  /**< XBAR3_IN151 input is reserved. */
    kXBAR3_InputRESERVED152         = 152|0x300U,  /**< XBAR3_IN152 input is reserved. */
    kXBAR3_InputRESERVED153         = 153|0x300U,  /**< XBAR3_IN153 input is reserved. */
    kXBAR3_InputRESERVED154         = 154|0x300U,  /**< XBAR3_IN154 input is reserved. */
    kXBAR3_InputRESERVED155         = 155|0x300U,  /**< XBAR3_IN155 input is reserved. */
    kXBAR3_InputEqdc2PosMatch0      = 156|0x300U,  /**< EQDC2_POS_MATCH0 output assigned to XBAR3_IN156 input. */
    kXBAR3_InputEqdc2PosMatch1      = 157|0x300U,  /**< EQDC2_POS_MATCH1 output assigned to XBAR3_IN157 input. */
    kXBAR3_InputEqdc2PosMatch2      = 158|0x300U,  /**< EQDC2_POS_MATCH2 output assigned to XBAR3_IN158 input. */
    kXBAR3_InputEqdc2PosMatch3      = 159|0x300U,  /**< EQDC2_POS_MATCH3 output assigned to XBAR3_IN159 input. */
    kXBAR3_InputEqdc2CompFlg0       = 160|0x300U,  /**< EQDC2_COMP_FLG0 output assigned to XBAR3_IN160 input. */
    kXBAR3_InputEqdc2CompFlg1       = 161|0x300U,  /**< EQDC2_COMP_FLG1 output assigned to XBAR3_IN161 input. */
    kXBAR3_InputEqdc2CompFlg2       = 162|0x300U,  /**< EQDC2_COMP_FLG2 output assigned to XBAR3_IN162 input. */
    kXBAR3_InputEqdc2CompFlg3       = 163|0x300U,  /**< EQDC2_COMP_FLG3 output assigned to XBAR3_IN163 input. */
    kXBAR3_InputEqdc2CntDn          = 164|0x300U,  /**< EQDC2_CNT_DN output assigned to XBAR3_IN164 input. */
    kXBAR3_InputEqdc2CntUp          = 165|0x300U,  /**< EQDC2_CNT_UP output assigned to XBAR3_IN165 input. */
    kXBAR3_InputEqdc2CntDir         = 166|0x300U,  /**< EQDC2_CNT_DIR output assigned to XBAR3_IN166 input. */
    kXBAR3_InputRESERVED167         = 167|0x300U,  /**< XBAR3_IN167 input is reserved. */
    kXBAR3_InputRESERVED168         = 168|0x300U,  /**< XBAR3_IN168 input is reserved. */
    kXBAR3_InputRESERVED169         = 169|0x300U,  /**< XBAR3_IN169 input is reserved. */
    kXBAR3_InputRESERVED170         = 170|0x300U,  /**< XBAR3_IN170 input is reserved. */
    kXBAR3_InputRESERVED171         = 171|0x300U,  /**< XBAR3_IN171 input is reserved. */
    kXBAR3_InputRESERVED172         = 172|0x300U,  /**< XBAR3_IN172 input is reserved. */
    kXBAR3_InputRESERVED173         = 173|0x300U,  /**< XBAR3_IN173 input is reserved. */
    kXBAR3_InputRESERVED174         = 174|0x300U,  /**< XBAR3_IN174 input is reserved. */
    kXBAR3_InputRESERVED175         = 175|0x300U,  /**< XBAR3_IN175 input is reserved. */
    kXBAR3_InputRESERVED176         = 176|0x300U,  /**< XBAR3_IN176 input is reserved. */
    kXBAR3_InputRESERVED177         = 177|0x300U,  /**< XBAR3_IN177 input is reserved. */
} xbar_input_signal_t;

typedef enum _xbar_output_signal
{
    kXBAR1_OutputDma4MuxReq154      = 0|0x100U,    /**< XBAR1_OUT0 output assigned to DMA4_MUX_REQ154 */
    kXBAR1_OutputDma4MuxReq155      = 1|0x100U,    /**< XBAR1_OUT1 output assigned to DMA4_MUX_REQ155 */
    kXBAR1_OutputDma4MuxReq156      = 2|0x100U,    /**< XBAR1_OUT2 output assigned to DMA4_MUX_REQ156 */
    kXBAR1_OutputDma4MuxReq157      = 3|0x100U,    /**< XBAR1_OUT3 output assigned to DMA4_MUX_REQ157 */
    kXBAR1_OutputIomuxXbarInout04   = 4|0x100U,    /**< XBAR1_OUT4 output assigned to IOMUX_XBAR_INOUT04 */
    kXBAR1_OutputIomuxXbarInout05   = 5|0x100U,    /**< XBAR1_OUT5 output assigned to IOMUX_XBAR_INOUT05 */
    kXBAR1_OutputIomuxXbarInout06   = 6|0x100U,    /**< XBAR1_OUT6 output assigned to IOMUX_XBAR_INOUT06 */
    kXBAR1_OutputIomuxXbarInout07   = 7|0x100U,    /**< XBAR1_OUT7 output assigned to IOMUX_XBAR_INOUT07 */
    kXBAR1_OutputIomuxXbarInout08   = 8|0x100U,    /**< XBAR1_OUT8 output assigned to IOMUX_XBAR_INOUT08 */
    kXBAR1_OutputIomuxXbarInout09   = 9|0x100U,    /**< XBAR1_OUT9 output assigned to IOMUX_XBAR_INOUT09 */
    kXBAR1_OutputIomuxXbarInout10   = 10|0x100U,   /**< XBAR1_OUT10 output assigned to IOMUX_XBAR_INOUT10 */
    kXBAR1_OutputIomuxXbarInout11   = 11|0x100U,   /**< XBAR1_OUT11 output assigned to IOMUX_XBAR_INOUT11 */
    kXBAR1_OutputIomuxXbarInout12   = 12|0x100U,   /**< XBAR1_OUT12 output assigned to IOMUX_XBAR_INOUT12 */
    kXBAR1_OutputIomuxXbarInout13   = 13|0x100U,   /**< XBAR1_OUT13 output assigned to IOMUX_XBAR_INOUT13 */
    kXBAR1_OutputIomuxXbarInout14   = 14|0x100U,   /**< XBAR1_OUT14 output assigned to IOMUX_XBAR_INOUT14 */
    kXBAR1_OutputIomuxXbarInout15   = 15|0x100U,   /**< XBAR1_OUT15 output assigned to IOMUX_XBAR_INOUT15 */
    kXBAR1_OutputIomuxXbarInout16   = 16|0x100U,   /**< XBAR1_OUT16 output assigned to IOMUX_XBAR_INOUT16 */
    kXBAR1_OutputIomuxXbarInout17   = 17|0x100U,   /**< XBAR1_OUT17 output assigned to IOMUX_XBAR_INOUT17 */
    kXBAR1_OutputIomuxXbarInout18   = 18|0x100U,   /**< XBAR1_OUT18 output assigned to IOMUX_XBAR_INOUT18 */
    kXBAR1_OutputIomuxXbarInout19   = 19|0x100U,   /**< XBAR1_OUT19 output assigned to IOMUX_XBAR_INOUT19 */
    kXBAR1_OutputIomuxXbarInout20   = 20|0x100U,   /**< XBAR1_OUT20 output assigned to IOMUX_XBAR_INOUT20 */
    kXBAR1_OutputIomuxXbarInout21   = 21|0x100U,   /**< XBAR1_OUT21 output assigned to IOMUX_XBAR_INOUT21 */
    kXBAR1_OutputIomuxXbarInout22   = 22|0x100U,   /**< XBAR1_OUT22 output assigned to IOMUX_XBAR_INOUT22 */
    kXBAR1_OutputIomuxXbarInout23   = 23|0x100U,   /**< XBAR1_OUT23 output assigned to IOMUX_XBAR_INOUT23 */
    kXBAR1_OutputIomuxXbarInout24   = 24|0x100U,   /**< XBAR1_OUT24 output assigned to IOMUX_XBAR_INOUT24 */
    kXBAR1_OutputIomuxXbarInout25   = 25|0x100U,   /**< XBAR1_OUT25 output assigned to IOMUX_XBAR_INOUT25 */
    kXBAR1_OutputIomuxXbarInout26   = 26|0x100U,   /**< XBAR1_OUT26 output assigned to IOMUX_XBAR_INOUT26 */
    kXBAR1_OutputIomuxXbarInout27   = 27|0x100U,   /**< XBAR1_OUT27 output assigned to IOMUX_XBAR_INOUT27 */
    kXBAR1_OutputIomuxXbarInout28   = 28|0x100U,   /**< XBAR1_OUT28 output assigned to IOMUX_XBAR_INOUT28 */
    kXBAR1_OutputIomuxXbarInout29   = 29|0x100U,   /**< XBAR1_OUT29 output assigned to IOMUX_XBAR_INOUT29 */
    kXBAR1_OutputTriggerSyncIn0     = 30|0x100U,   /**< XBAR1_OUT30 output assigned to TRIGGER_SYNC_IN0 */
    kXBAR1_OutputTriggerSyncIn1     = 31|0x100U,   /**< XBAR1_OUT31 output assigned to TRIGGER_SYNC_IN1 */
    kXBAR1_OutputTriggerSyncIn2     = 32|0x100U,   /**< XBAR1_OUT32 output assigned to TRIGGER_SYNC_IN2 */
    kXBAR1_OutputTriggerSyncIn3     = 33|0x100U,   /**< XBAR1_OUT33 output assigned to TRIGGER_SYNC_IN3 */
    kXBAR1_OutputTriggerSyncIn4     = 34|0x100U,   /**< XBAR1_OUT34 output assigned to TRIGGER_SYNC_IN4 */
    kXBAR1_OutputTriggerSyncIn5     = 35|0x100U,   /**< XBAR1_OUT35 output assigned to TRIGGER_SYNC_IN5 */
    kXBAR1_OutputTriggerSyncIn6     = 36|0x100U,   /**< XBAR1_OUT36 output assigned to TRIGGER_SYNC_IN6 */
    kXBAR1_OutputTriggerSyncIn7     = 37|0x100U,   /**< XBAR1_OUT37 output assigned to TRIGGER_SYNC_IN7 */
    kXBAR1_OutputAcmp1Sample        = 38|0x100U,   /**< XBAR1_OUT38 output assigned to ACMP1_SAMPLE */
    kXBAR1_OutputAcmp2Sample        = 39|0x100U,   /**< XBAR1_OUT39 output assigned to ACMP2_SAMPLE */
    kXBAR1_OutputAcmp3Sample        = 40|0x100U,   /**< XBAR1_OUT40 output assigned to ACMP3_SAMPLE */
    kXBAR1_OutputAcmp4Sample        = 41|0x100U,   /**< XBAR1_OUT41 output assigned to ACMP4_SAMPLE */
    kXBAR1_OutputFlexpwm1Exta0      = 42|0x100U,   /**< XBAR1_OUT42 output assigned to FLEXPWM1_EXTA0 */
    kXBAR1_OutputFlexpwm1Exta1      = 43|0x100U,   /**< XBAR1_OUT43 output assigned to FLEXPWM1_EXTA1 */
    kXBAR1_OutputFlexpwm1Exta2      = 44|0x100U,   /**< XBAR1_OUT44 output assigned to FLEXPWM1_EXTA2 */
    kXBAR1_OutputFlexpwm1Exta3      = 45|0x100U,   /**< XBAR1_OUT45 output assigned to FLEXPWM1_EXTA3 */
    kXBAR1_OutputFlexpwm1ExtSync0   = 46|0x100U,   /**< XBAR1_OUT46 output assigned to FLEXPWM1_EXT_SYNC0 */
    kXBAR1_OutputFlexpwm1ExtSync1   = 47|0x100U,   /**< XBAR1_OUT47 output assigned to FLEXPWM1_EXT_SYNC1 */
    kXBAR1_OutputFlexpwm1ExtSync2   = 48|0x100U,   /**< XBAR1_OUT48 output assigned to FLEXPWM1_EXT_SYNC2 */
    kXBAR1_OutputFlexpwm1ExtSync3   = 49|0x100U,   /**< XBAR1_OUT49 output assigned to FLEXPWM1_EXT_SYNC3 */
    kXBAR1_OutputFlexpwm1ExtClk     = 50|0x100U,   /**< XBAR1_OUT50 output assigned to FLEXPWM1_EXT_CLK */
    kXBAR1_OutputFlexpwm1Fault0     = 51|0x100U,   /**< XBAR1_OUT51 output assigned to FLEXPWM1_FAULT0 */
    kXBAR1_OutputFlexpwm1Fault1     = 52|0x100U,   /**< XBAR1_OUT52 output assigned to FLEXPWM1_FAULT1 */
    kXBAR1_OutputFlexpwm1234Fault2  = 53|0x100U,   /**< XBAR1_OUT53 output assigned to FLEXPWM1_2_3_4_FAULT2 */
    kXBAR1_OutputFlexpwm1234Fault3  = 54|0x100U,   /**< XBAR1_OUT54 output assigned to FLEXPWM1_2_3_4_FAULT3 */
    kXBAR1_OutputFlexpwm1ExtForce   = 55|0x100U,   /**< XBAR1_OUT55 output assigned to FLEXPWM1_EXT_FORCE */
    kXBAR1_OutputFlexpwm2Exta0      = 56|0x100U,   /**< XBAR1_OUT56 output assigned to FLEXPWM2_EXTA0 */
    kXBAR1_OutputFlexpwm2Exta1      = 57|0x100U,   /**< XBAR1_OUT57 output assigned to FLEXPWM2_EXTA1 */
    kXBAR1_OutputFlexpwm2Exta2      = 58|0x100U,   /**< XBAR1_OUT58 output assigned to FLEXPWM2_EXTA2 */
    kXBAR1_OutputFlexpwm2Exta3      = 59|0x100U,   /**< XBAR1_OUT59 output assigned to FLEXPWM2_EXTA3 */
    kXBAR1_OutputFlexpwm2ExtSync0   = 60|0x100U,   /**< XBAR1_OUT60 output assigned to FLEXPWM2_EXT_SYNC0 */
    kXBAR1_OutputFlexpwm2ExtSync1   = 61|0x100U,   /**< XBAR1_OUT61 output assigned to FLEXPWM2_EXT_SYNC1 */
    kXBAR1_OutputFlexpwm2ExtSync2   = 62|0x100U,   /**< XBAR1_OUT62 output assigned to FLEXPWM2_EXT_SYNC2 */
    kXBAR1_OutputFlexpwm2ExtSync3   = 63|0x100U,   /**< XBAR1_OUT63 output assigned to FLEXPWM2_EXT_SYNC3 */
    kXBAR1_OutputFlexpwm2ExtClk     = 64|0x100U,   /**< XBAR1_OUT64 output assigned to FLEXPWM2_EXT_CLK */
    kXBAR1_OutputFlexpwm2Fault0     = 65|0x100U,   /**< XBAR1_OUT65 output assigned to FLEXPWM2_FAULT0 */
    kXBAR1_OutputFlexpwm2Fault1     = 66|0x100U,   /**< XBAR1_OUT66 output assigned to FLEXPWM2_FAULT1 */
    kXBAR1_OutputFlexpwm2ExtForce   = 67|0x100U,   /**< XBAR1_OUT67 output assigned to FLEXPWM2_EXT_FORCE */
    kXBAR1_OutputRESERVED68         = 68|0x100U,   /**< XBAR1_OUT68 output is reserved. */
    kXBAR1_OutputRESERVED69         = 69|0x100U,   /**< XBAR1_OUT69 output is reserved. */
    kXBAR1_OutputRESERVED70         = 70|0x100U,   /**< XBAR1_OUT70 output is reserved. */
    kXBAR1_OutputRESERVED71         = 71|0x100U,   /**< XBAR1_OUT71 output is reserved. */
    kXBAR1_OutputRESERVED72         = 72|0x100U,   /**< XBAR1_OUT72 output is reserved. */
    kXBAR1_OutputRESERVED73         = 73|0x100U,   /**< XBAR1_OUT73 output is reserved. */
    kXBAR1_OutputRESERVED74         = 74|0x100U,   /**< XBAR1_OUT74 output is reserved. */
    kXBAR1_OutputRESERVED75         = 75|0x100U,   /**< XBAR1_OUT75 output is reserved. */
    kXBAR1_OutputRESERVED76         = 76|0x100U,   /**< XBAR1_OUT76 output is reserved. */
    kXBAR1_OutputRESERVED77         = 77|0x100U,   /**< XBAR1_OUT77 output is reserved. */
    kXBAR1_OutputRESERVED78         = 78|0x100U,   /**< XBAR1_OUT78 output is reserved. */
    kXBAR1_OutputRESERVED79         = 79|0x100U,   /**< XBAR1_OUT79 output is reserved. */
    kXBAR1_OutputRESERVED80         = 80|0x100U,   /**< XBAR1_OUT80 output is reserved. */
    kXBAR1_OutputRESERVED81         = 81|0x100U,   /**< XBAR1_OUT81 output is reserved. */
    kXBAR1_OutputRESERVED82         = 82|0x100U,   /**< XBAR1_OUT82 output is reserved. */
    kXBAR1_OutputRESERVED83         = 83|0x100U,   /**< XBAR1_OUT83 output is reserved. */
    kXBAR1_OutputRESERVED84         = 84|0x100U,   /**< XBAR1_OUT84 output is reserved. */
    kXBAR1_OutputRESERVED85         = 85|0x100U,   /**< XBAR1_OUT85 output is reserved. */
    kXBAR1_OutputRESERVED86         = 86|0x100U,   /**< XBAR1_OUT86 output is reserved. */
    kXBAR1_OutputEqdc1Phasea        = 87|0x100U,   /**< XBAR1_OUT87 output assigned to EQDC1_PHASEA */
    kXBAR1_OutputEqdc1Phaseb        = 88|0x100U,   /**< XBAR1_OUT88 output assigned to EQDC1_PHASEB */
    kXBAR1_OutputEqdc1Index         = 89|0x100U,   /**< XBAR1_OUT89 output assigned to EQDC1_INDEX */
    kXBAR1_OutputEqdc1Home          = 90|0x100U,   /**< XBAR1_OUT90 output assigned to EQDC1_HOME */
    kXBAR1_OutputEqdc1Trigger       = 91|0x100U,   /**< XBAR1_OUT91 output assigned to EQDC1_TRIGGER */
    kXBAR1_OutputEqdc2Phasea        = 92|0x100U,   /**< XBAR1_OUT92 output assigned to EQDC2_PHASEA */
    kXBAR1_OutputEqdc2Phaseb        = 93|0x100U,   /**< XBAR1_OUT93 output assigned to EQDC2_PHASEB */
    kXBAR1_OutputEqdc2Index         = 94|0x100U,   /**< XBAR1_OUT94 output assigned to EQDC2_INDEX */
    kXBAR1_OutputEqdc2Home          = 95|0x100U,   /**< XBAR1_OUT95 output assigned to EQDC2_HOME */
    kXBAR1_OutputEqdc2Trigger       = 96|0x100U,   /**< XBAR1_OUT96 output assigned to EQDC2_TRIGGER */
    kXBAR1_OutputRESERVED97         = 97|0x100U,   /**< XBAR1_OUT97 output is reserved. */
    kXBAR1_OutputRESERVED98         = 98|0x100U,   /**< XBAR1_OUT98 output is reserved. */
    kXBAR1_OutputRESERVED99         = 99|0x100U,   /**< XBAR1_OUT99 output is reserved. */
    kXBAR1_OutputRESERVED100        = 100|0x100U,  /**< XBAR1_OUT100 output is reserved. */
    kXBAR1_OutputRESERVED101        = 101|0x100U,  /**< XBAR1_OUT101 output is reserved. */
    kXBAR1_OutputRESERVED102        = 102|0x100U,  /**< XBAR1_OUT102 output is reserved. */
    kXBAR1_OutputRESERVED103        = 103|0x100U,  /**< XBAR1_OUT103 output is reserved. */
    kXBAR1_OutputRESERVED104        = 104|0x100U,  /**< XBAR1_OUT104 output is reserved. */
    kXBAR1_OutputRESERVED105        = 105|0x100U,  /**< XBAR1_OUT105 output is reserved. */
    kXBAR1_OutputRESERVED106        = 106|0x100U,  /**< XBAR1_OUT106 output is reserved. */
    kXBAR1_OutputQtimer1Timer0      = 107|0x100U,  /**< XBAR1_OUT107 output assigned to QTIMER1_TIMER0 */
    kXBAR1_OutputQtimer1Timer1      = 108|0x100U,  /**< XBAR1_OUT108 output assigned to QTIMER1_TIMER1 */
    kXBAR1_OutputQtimer1Timer2      = 109|0x100U,  /**< XBAR1_OUT109 output assigned to QTIMER1_TIMER2 */
    kXBAR1_OutputQtimer1Timer3      = 110|0x100U,  /**< XBAR1_OUT110 output assigned to QTIMER1_TIMER3 */
    kXBAR1_OutputQtimer2Timer0      = 111|0x100U,  /**< XBAR1_OUT111 output assigned to QTIMER2_TIMER0 */
    kXBAR1_OutputQtimer2Timer1      = 112|0x100U,  /**< XBAR1_OUT112 output assigned to QTIMER2_TIMER1 */
    kXBAR1_OutputQtimer2Timer2      = 113|0x100U,  /**< XBAR1_OUT113 output assigned to QTIMER2_TIMER2 */
    kXBAR1_OutputQtimer2Timer3      = 114|0x100U,  /**< XBAR1_OUT114 output assigned to QTIMER2_TIMER3 */
    kXBAR1_OutputQtimer3Timer0      = 115|0x100U,  /**< XBAR1_OUT115 output assigned to QTIMER3_TIMER0 */
    kXBAR1_OutputQtimer3Timer1      = 116|0x100U,  /**< XBAR1_OUT116 output assigned to QTIMER3_TIMER1 */
    kXBAR1_OutputQtimer3Timer2      = 117|0x100U,  /**< XBAR1_OUT117 output assigned to QTIMER3_TIMER2 */
    kXBAR1_OutputQtimer3Timer3      = 118|0x100U,  /**< XBAR1_OUT118 output assigned to QTIMER3_TIMER3 */
    kXBAR1_OutputQtimer4Timer0      = 119|0x100U,  /**< XBAR1_OUT119 output assigned to QTIMER4_TIMER0 */
    kXBAR1_OutputQtimer4Timer1      = 120|0x100U,  /**< XBAR1_OUT120 output assigned to QTIMER4_TIMER1 */
    kXBAR1_OutputQtimer4Timer2      = 121|0x100U,  /**< XBAR1_OUT121 output assigned to QTIMER4_TIMER2 */
    kXBAR1_OutputQtimer4Timer3      = 122|0x100U,  /**< XBAR1_OUT122 output assigned to QTIMER4_TIMER3 */
    kXBAR1_OutputRESERVED123        = 123|0x100U,  /**< XBAR1_OUT123 output is reserved. */
    kXBAR1_OutputRESERVED124        = 124|0x100U,  /**< XBAR1_OUT124 output is reserved. */
    kXBAR1_OutputRESERVED125        = 125|0x100U,  /**< XBAR1_OUT125 output is reserved. */
    kXBAR1_OutputRESERVED126        = 126|0x100U,  /**< XBAR1_OUT126 output is reserved. */
    kXBAR1_OutputRESERVED127        = 127|0x100U,  /**< XBAR1_OUT127 output is reserved. */
    kXBAR1_OutputRESERVED128        = 128|0x100U,  /**< XBAR1_OUT128 output is reserved. */
    kXBAR1_OutputRESERVED129        = 129|0x100U,  /**< XBAR1_OUT129 output is reserved. */
    kXBAR1_OutputRESERVED130        = 130|0x100U,  /**< XBAR1_OUT130 output is reserved. */
    kXBAR1_OutputRESERVED131        = 131|0x100U,  /**< XBAR1_OUT131 output is reserved. */
    kXBAR1_OutputRESERVED132        = 132|0x100U,  /**< XBAR1_OUT132 output is reserved. */
    kXBAR1_OutputRESERVED133        = 133|0x100U,  /**< XBAR1_OUT133 output is reserved. */
    kXBAR1_OutputRESERVED134        = 134|0x100U,  /**< XBAR1_OUT134 output is reserved. */
    kXBAR1_OutputRESERVED135        = 135|0x100U,  /**< XBAR1_OUT135 output is reserved. */
    kXBAR1_OutputRESERVED136        = 136|0x100U,  /**< XBAR1_OUT136 output is reserved. */
    kXBAR1_OutputRESERVED137        = 137|0x100U,  /**< XBAR1_OUT137 output is reserved. */
    kXBAR1_OutputRESERVED138        = 138|0x100U,  /**< XBAR1_OUT138 output is reserved. */
    kXBAR1_OutputEwmIn              = 139|0x100U,  /**< XBAR1_OUT139 output assigned to EWM_IN */
    kXBAR1_OutputAdc12HwTrig0       = 140|0x100U,  /**< XBAR1_OUT140 output assigned to ADC1_2_HW_TRIG0 */
    kXBAR1_OutputAdc12HwTrig1       = 141|0x100U,  /**< XBAR1_OUT141 output assigned to ADC1_2_HW_TRIG1 */
    kXBAR1_OutputAdc12HwTrig2       = 142|0x100U,  /**< XBAR1_OUT142 output assigned to ADC1_2_HW_TRIG2 */
    kXBAR1_OutputAdc12HwTrig3       = 143|0x100U,  /**< XBAR1_OUT143 output assigned to ADC1_2_HW_TRIG3 */
    kXBAR1_OutputAdc12HwTrig4       = 144|0x100U,  /**< XBAR1_OUT144 output assigned to ADC1_2_HW_TRIG4 */
    kXBAR1_OutputAdc12HwTrig5       = 145|0x100U,  /**< XBAR1_OUT145 output assigned to ADC1_2_HW_TRIG5 */
    kXBAR1_OutputAdc12HwTrig6       = 146|0x100U,  /**< XBAR1_OUT146 output assigned to ADC1_2_HW_TRIG6 */
    kXBAR1_OutputAdc12HwTrig7       = 147|0x100U,  /**< XBAR1_OUT147 output assigned to ADC1_2_HW_TRIG7 */
    kXBAR1_OutputRESERVED148        = 148|0x100U,  /**< XBAR1_OUT148 output is reserved. */
    kXBAR1_OutputRESERVED149        = 149|0x100U,  /**< XBAR1_OUT149 output is reserved. */
    kXBAR1_OutputRESERVED150        = 150|0x100U,  /**< XBAR1_OUT150 output is reserved. */
    kXBAR1_OutputRESERVED151        = 151|0x100U,  /**< XBAR1_OUT151 output is reserved. */
    kXBAR1_OutputFlexio1TrigIn0     = 152|0x100U,  /**< XBAR1_OUT152 output assigned to FLEXIO1_TRIG_IN0 */
    kXBAR1_OutputFlexio1TrigIn1     = 153|0x100U,  /**< XBAR1_OUT153 output assigned to FLEXIO1_TRIG_IN1 */
    kXBAR1_OutputFlexio2TrigIn0     = 154|0x100U,  /**< XBAR1_OUT154 output assigned to FLEXIO2_TRIG_IN0 */
    kXBAR1_OutputFlexio2TrigIn1     = 155|0x100U,  /**< XBAR1_OUT155 output assigned to FLEXIO2_TRIG_IN1 */
    kXBAR1_OutputLpi2c1TrigIn       = 156|0x100U,  /**< XBAR1_OUT156 output assigned to LPI2C1_TRIG_IN */
    kXBAR1_OutputLpi2c2TrigIn       = 157|0x100U,  /**< XBAR1_OUT157 output assigned to LPI2C2_TRIG_IN */
    kXBAR1_OutputLpi2c3TrigIn       = 158|0x100U,  /**< XBAR1_OUT158 output assigned to LPI2C3_TRIG_IN */
    kXBAR1_OutputRESERVED159        = 159|0x100U,  /**< XBAR1_OUT159 output is reserved. */
    kXBAR1_OutputRESERVED160        = 160|0x100U,  /**< XBAR1_OUT160 output is reserved. */
    kXBAR1_OutputRESERVED161        = 161|0x100U,  /**< XBAR1_OUT161 output is reserved. */
    kXBAR1_OutputLpspi1TrigIn       = 162|0x100U,  /**< XBAR1_OUT162 output assigned to LPSPI1_TRIG_IN */
    kXBAR1_OutputLpspi2TrigIn       = 163|0x100U,  /**< XBAR1_OUT163 output assigned to LPSPI2_TRIG_IN */
    kXBAR1_OutputLpspi3TrigIn       = 164|0x100U,  /**< XBAR1_OUT164 output assigned to LPSPI3_TRIG_IN */
    kXBAR1_OutputLpspi4TrigIn       = 165|0x100U,  /**< XBAR1_OUT165 output assigned to LPSPI4_TRIG_IN */
    kXBAR1_OutputRESERVED166        = 166|0x100U,  /**< XBAR1_OUT166 output is reserved. */
    kXBAR1_OutputRESERVED167        = 167|0x100U,  /**< XBAR1_OUT167 output is reserved. */
    kXBAR1_OutputLpuart1TrigIn      = 168|0x100U,  /**< XBAR1_OUT168 output assigned to LPUART1_TRIG_IN */
    kXBAR1_OutputLpuart2TrigIn      = 169|0x100U,  /**< XBAR1_OUT169 output assigned to LPUART2_TRIG_IN */
    kXBAR1_OutputLpuart3TrigIn      = 170|0x100U,  /**< XBAR1_OUT170 output assigned to LPUART3_TRIG_IN */
    kXBAR1_OutputLpuart4TrigIn      = 171|0x100U,  /**< XBAR1_OUT171 output assigned to LPUART4_TRIG_IN */
    kXBAR1_OutputLpuart5TrigIn      = 172|0x100U,  /**< XBAR1_OUT172 output assigned to LPUART5_TRIG_IN */
    kXBAR1_OutputLpuart6TrigIn      = 173|0x100U,  /**< XBAR1_OUT173 output assigned to LPUART6_TRIG_IN */
    kXBAR1_OutputLpuart7TrigIn      = 174|0x100U,  /**< XBAR1_OUT174 output assigned to LPUART7_TRIG_IN */
    kXBAR1_OutputLpuart8TrigIn      = 175|0x100U,  /**< XBAR1_OUT175 output assigned to LPUART8_TRIG_IN */
    kXBAR1_OutputRESERVED176        = 176|0x100U,  /**< XBAR1_OUT176 output is reserved. */
    kXBAR1_OutputRESERVED177        = 177|0x100U,  /**< XBAR1_OUT177 output is reserved. */
    kXBAR1_OutputRESERVED178        = 178|0x100U,  /**< XBAR1_OUT178 output is reserved. */
    kXBAR1_OutputRESERVED179        = 179|0x100U,  /**< XBAR1_OUT179 output is reserved. */
    kXBAR1_OutputLpit123TrigIn0     = 180|0x100U,  /**< XBAR1_OUT180 output assigned to LPIT1_2_3_TRIG_IN0 */
    kXBAR1_OutputLpit123TrigIn1     = 181|0x100U,  /**< XBAR1_OUT181 output assigned to LPIT1_2_3_TRIG_IN1 */
    kXBAR1_OutputLpit123TrigIn2     = 182|0x100U,  /**< XBAR1_OUT182 output assigned to LPIT1_2_3_TRIG_IN2 */
    kXBAR1_OutputLpit123TrigIn3     = 183|0x100U,  /**< XBAR1_OUT183 output assigned to LPIT1_2_3_TRIG_IN3 */
    kXBAR1_OutputTpm123TrigIn0      = 184|0x100U,  /**< XBAR1_OUT184 output assigned to TPM1_2_3_TRIG_IN0 */
    kXBAR1_OutputTpm1TrigIn1        = 185|0x100U,  /**< XBAR1_OUT185 output assigned to TPM1_TRIG_IN1 */
    kXBAR1_OutputTpm2TrigIn1        = 186|0x100U,  /**< XBAR1_OUT186 output assigned to TPM2_TRIG_IN1 */
    kXBAR1_OutputTpm3TrigIn1        = 187|0x100U,  /**< XBAR1_OUT187 output assigned to TPM3_TRIG_IN1 */
    kXBAR1_OutputTpm123TrigIn2      = 188|0x100U,  /**< XBAR1_OUT188 output assigned to TPM1_2_3_TRIG_IN2 */
    kXBAR1_OutputTpm1TrigIn3        = 189|0x100U,  /**< XBAR1_OUT189 output assigned to TPM1_TRIG_IN3 */
    kXBAR1_OutputTpm2TrigIn3        = 190|0x100U,  /**< XBAR1_OUT190 output assigned to TPM2_TRIG_IN3 */
    kXBAR1_OutputTpm3TrigIn3        = 191|0x100U,  /**< XBAR1_OUT191 output assigned to TPM3_TRIG_IN3 */
    kXBAR1_OutputRESERVED192        = 192|0x100U,  /**< XBAR1_OUT192 output is reserved. */
    kXBAR1_OutputRESERVED193        = 193|0x100U,  /**< XBAR1_OUT193 output is reserved. */
    kXBAR1_OutputRESERVED194        = 194|0x100U,  /**< XBAR1_OUT194 output is reserved. */
    kXBAR1_OutputRESERVED195        = 195|0x100U,  /**< XBAR1_OUT195 output is reserved. */
    kXBAR1_OutputRESERVED196        = 196|0x100U,  /**< XBAR1_OUT196 output is reserved. */
    kXBAR1_OutputRESERVED197        = 197|0x100U,  /**< XBAR1_OUT197 output is reserved. */
    kXBAR1_OutputRESERVED198        = 198|0x100U,  /**< XBAR1_OUT198 output is reserved. */
    kXBAR1_OutputRESERVED199        = 199|0x100U,  /**< XBAR1_OUT199 output is reserved. */
    kXBAR1_OutputNetcTmrTrig1       = 200|0x100U,  /**< XBAR1_OUT200 output assigned to NETC_TMR_TRIG1 */
    kXBAR1_OutputNetcTmrTrig2       = 201|0x100U,  /**< XBAR1_OUT201 output assigned to NETC_TMR_TRIG2 */
    kXBAR1_OutputRESERVED202        = 202|0x100U,  /**< XBAR1_OUT202 output is reserved. */
    kXBAR1_OutputRESERVED203        = 203|0x100U,  /**< XBAR1_OUT203 output is reserved. */
    kXBAR1_OutputRESERVED204        = 204|0x100U,  /**< XBAR1_OUT204 output is reserved. */
    kXBAR1_OutputRESERVED205        = 205|0x100U,  /**< XBAR1_OUT205 output is reserved. */
    kXBAR1_OutputEqdc1Icap1         = 206|0x100U,  /**< XBAR1_OUT206 output assigned to EQDC1_ICAP1 */
    kXBAR1_OutputEqdc1Icap2         = 207|0x100U,  /**< XBAR1_OUT207 output assigned to EQDC1_ICAP2 */
    kXBAR1_OutputEqdc1Icap3         = 208|0x100U,  /**< XBAR1_OUT208 output assigned to EQDC1_ICAP3 */
    kXBAR1_OutputEqdc2Icap1         = 209|0x100U,  /**< XBAR1_OUT209 output assigned to EQDC2_ICAP1 */
    kXBAR1_OutputEqdc2Icap2         = 210|0x100U,  /**< XBAR1_OUT210 output assigned to EQDC2_ICAP2 */
    kXBAR1_OutputEqdc2Icap3         = 211|0x100U,  /**< XBAR1_OUT211 output assigned to EQDC2_ICAP3 */
    kXBAR1_OutputRESERVED212        = 212|0x100U,  /**< XBAR1_OUT212 output is reserved. */
    kXBAR1_OutputRESERVED213        = 213|0x100U,  /**< XBAR1_OUT213 output is reserved. */
    kXBAR1_OutputRESERVED214        = 214|0x100U,  /**< XBAR1_OUT214 output is reserved. */
    kXBAR1_OutputRESERVED215        = 215|0x100U,  /**< XBAR1_OUT215 output is reserved. */
    kXBAR1_OutputRESERVED216        = 216|0x100U,  /**< XBAR1_OUT216 output is reserved. */
    kXBAR1_OutputRESERVED217        = 217|0x100U,  /**< XBAR1_OUT217 output is reserved. */
    kXBAR1_OutputEcatLatchIn0       = 218|0x100U,  /**< XBAR1_OUT218 output assigned to ECAT_LATCH_IN0 */
    kXBAR1_OutputEcatLatchIn1       = 219|0x100U,  /**< XBAR1_OUT219 output assigned to ECAT_LATCH_IN1 */
    kXBAR1_OutputDutClk             = 220|0x100U,  /**< XBAR1_OUT220 output assigned to DUT_CLK */
    kXBAR2_OutputAoi1In00           = 0|0x200U,    /**< XBAR2_OUT0 output assigned to AOI1_IN00 */
    kXBAR2_OutputAoi1In01           = 1|0x200U,    /**< XBAR2_OUT1 output assigned to AOI1_IN01 */
    kXBAR2_OutputAoi1In02           = 2|0x200U,    /**< XBAR2_OUT2 output assigned to AOI1_IN02 */
    kXBAR2_OutputAoi1In03           = 3|0x200U,    /**< XBAR2_OUT3 output assigned to AOI1_IN03 */
    kXBAR2_OutputAoi1In04           = 4|0x200U,    /**< XBAR2_OUT4 output assigned to AOI1_IN04 */
    kXBAR2_OutputAoi1In05           = 5|0x200U,    /**< XBAR2_OUT5 output assigned to AOI1_IN05 */
    kXBAR2_OutputAoi1In06           = 6|0x200U,    /**< XBAR2_OUT6 output assigned to AOI1_IN06 */
    kXBAR2_OutputAoi1In07           = 7|0x200U,    /**< XBAR2_OUT7 output assigned to AOI1_IN07 */
    kXBAR2_OutputAoi1In08           = 8|0x200U,    /**< XBAR2_OUT8 output assigned to AOI1_IN08 */
    kXBAR2_OutputAoi1In09           = 9|0x200U,    /**< XBAR2_OUT9 output assigned to AOI1_IN09 */
    kXBAR2_OutputAoi1In10           = 10|0x200U,   /**< XBAR2_OUT10 output assigned to AOI1_IN10 */
    kXBAR2_OutputAoi1In11           = 11|0x200U,   /**< XBAR2_OUT11 output assigned to AOI1_IN11 */
    kXBAR2_OutputAoi1In12           = 12|0x200U,   /**< XBAR2_OUT12 output assigned to AOI1_IN12 */
    kXBAR2_OutputAoi1In13           = 13|0x200U,   /**< XBAR2_OUT13 output assigned to AOI1_IN13 */
    kXBAR2_OutputAoi1In14           = 14|0x200U,   /**< XBAR2_OUT14 output assigned to AOI1_IN14 */
    kXBAR2_OutputAoi1In15           = 15|0x200U,   /**< XBAR2_OUT15 output assigned to AOI1_IN15 */
    kXBAR2_OutputAoi3In00           = 16|0x200U,   /**< XBAR2_OUT16 output assigned to AOI3_IN00 */
    kXBAR2_OutputAoi3In01           = 17|0x200U,   /**< XBAR2_OUT17 output assigned to AOI3_IN01 */
    kXBAR2_OutputAoi3In02           = 18|0x200U,   /**< XBAR2_OUT18 output assigned to AOI3_IN02 */
    kXBAR2_OutputAoi3In03           = 19|0x200U,   /**< XBAR2_OUT19 output assigned to AOI3_IN03 */
    kXBAR2_OutputAoi3In04           = 20|0x200U,   /**< XBAR2_OUT20 output assigned to AOI3_IN04 */
    kXBAR2_OutputAoi3In05           = 21|0x200U,   /**< XBAR2_OUT21 output assigned to AOI3_IN05 */
    kXBAR2_OutputAoi3In06           = 22|0x200U,   /**< XBAR2_OUT22 output assigned to AOI3_IN06 */
    kXBAR2_OutputAoi3In07           = 23|0x200U,   /**< XBAR2_OUT23 output assigned to AOI3_IN07 */
    kXBAR2_OutputAoi3In08           = 24|0x200U,   /**< XBAR2_OUT24 output assigned to AOI3_IN08 */
    kXBAR2_OutputAoi3In09           = 25|0x200U,   /**< XBAR2_OUT25 output assigned to AOI3_IN09 */
    kXBAR2_OutputAoi3In10           = 26|0x200U,   /**< XBAR2_OUT26 output assigned to AOI3_IN10 */
    kXBAR2_OutputAoi3In11           = 27|0x200U,   /**< XBAR2_OUT27 output assigned to AOI3_IN11 */
    kXBAR2_OutputAoi3In12           = 28|0x200U,   /**< XBAR2_OUT28 output assigned to AOI3_IN12 */
    kXBAR2_OutputAoi3In13           = 29|0x200U,   /**< XBAR2_OUT29 output assigned to AOI3_IN13 */
    kXBAR2_OutputAoi3In14           = 30|0x200U,   /**< XBAR2_OUT30 output assigned to AOI3_IN14 */
    kXBAR2_OutputAoi3In15           = 31|0x200U,   /**< XBAR2_OUT31 output assigned to AOI3_IN15 */
    kXBAR3_OutputAoi2In00           = 0|0x300U,    /**< XBAR3_OUT0 output assigned to AOI2_IN00 */
    kXBAR3_OutputAoi2In01           = 1|0x300U,    /**< XBAR3_OUT1 output assigned to AOI2_IN01 */
    kXBAR3_OutputAoi2In02           = 2|0x300U,    /**< XBAR3_OUT2 output assigned to AOI2_IN02 */
    kXBAR3_OutputAoi2In03           = 3|0x300U,    /**< XBAR3_OUT3 output assigned to AOI2_IN03 */
    kXBAR3_OutputAoi2In04           = 4|0x300U,    /**< XBAR3_OUT4 output assigned to AOI2_IN04 */
    kXBAR3_OutputAoi2In05           = 5|0x300U,    /**< XBAR3_OUT5 output assigned to AOI2_IN05 */
    kXBAR3_OutputAoi2In06           = 6|0x300U,    /**< XBAR3_OUT6 output assigned to AOI2_IN06 */
    kXBAR3_OutputAoi2In07           = 7|0x300U,    /**< XBAR3_OUT7 output assigned to AOI2_IN07 */
    kXBAR3_OutputAoi2In08           = 8|0x300U,    /**< XBAR3_OUT8 output assigned to AOI2_IN08 */
    kXBAR3_OutputAoi2In09           = 9|0x300U,    /**< XBAR3_OUT9 output assigned to AOI2_IN09 */
    kXBAR3_OutputAoi2In10           = 10|0x300U,   /**< XBAR3_OUT10 output assigned to AOI2_IN10 */
    kXBAR3_OutputAoi2In11           = 11|0x300U,   /**< XBAR3_OUT11 output assigned to AOI2_IN11 */
    kXBAR3_OutputAoi2In12           = 12|0x300U,   /**< XBAR3_OUT12 output assigned to AOI2_IN12 */
    kXBAR3_OutputAoi2In13           = 13|0x300U,   /**< XBAR3_OUT13 output assigned to AOI2_IN13 */
    kXBAR3_OutputAoi2In14           = 14|0x300U,   /**< XBAR3_OUT14 output assigned to AOI2_IN14 */
    kXBAR3_OutputAoi2In15           = 15|0x300U,   /**< XBAR3_OUT15 output assigned to AOI2_IN15 */
    kXBAR3_OutputAoi4In00           = 16|0x300U,   /**< XBAR3_OUT16 output assigned to AOI4_IN00 */
    kXBAR3_OutputAoi4In01           = 17|0x300U,   /**< XBAR3_OUT17 output assigned to AOI4_IN01 */
    kXBAR3_OutputAoi4In02           = 18|0x300U,   /**< XBAR3_OUT18 output assigned to AOI4_IN02 */
    kXBAR3_OutputAoi4In03           = 19|0x300U,   /**< XBAR3_OUT19 output assigned to AOI4_IN03 */
    kXBAR3_OutputAoi4In04           = 20|0x300U,   /**< XBAR3_OUT20 output assigned to AOI4_IN04 */
    kXBAR3_OutputAoi4In05           = 21|0x300U,   /**< XBAR3_OUT21 output assigned to AOI4_IN05 */
    kXBAR3_OutputAoi4In06           = 22|0x300U,   /**< XBAR3_OUT22 output assigned to AOI4_IN06 */
    kXBAR3_OutputAoi4In07           = 23|0x300U,   /**< XBAR3_OUT23 output assigned to AOI4_IN07 */
    kXBAR3_OutputAoi4In08           = 24|0x300U,   /**< XBAR3_OUT24 output assigned to AOI4_IN08 */
    kXBAR3_OutputAoi4In09           = 25|0x300U,   /**< XBAR3_OUT25 output assigned to AOI4_IN09 */
    kXBAR3_OutputAoi4In10           = 26|0x300U,   /**< XBAR3_OUT26 output assigned to AOI4_IN10 */
    kXBAR3_OutputAoi4In11           = 27|0x300U,   /**< XBAR3_OUT27 output assigned to AOI4_IN11 */
    kXBAR3_OutputAoi4In12           = 28|0x300U,   /**< XBAR3_OUT28 output assigned to AOI4_IN12 */
    kXBAR3_OutputAoi4In13           = 29|0x300U,   /**< XBAR3_OUT29 output assigned to AOI4_IN13 */
    kXBAR3_OutputAoi4In14           = 30|0x300U,   /**< XBAR3_OUT30 output assigned to AOI4_IN14 */
    kXBAR3_OutputAoi4In15           = 31|0x300U,   /**< XBAR3_OUT31 output assigned to AOI4_IN15 */
} xbar_output_signal_t;


/*!
 * @}
 */ /* end of group Mapping_Information */


/* ----------------------------------------------------------------------------
   -- Device Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Peripheral_access_layer Device Peripheral Access Layer
 * @{
 */


/*
** Start of section using anonymous unions
*/

#if defined(__ARMCC_VERSION)
  #if (__ARMCC_VERSION >= 6010050)
    #pragma clang diagnostic push
  #else
    #pragma push
    #pragma anon_unions
  #endif
#elif defined(__GNUC__)
  /* anonymous unions are enabled by default */
#elif defined(__IAR_SYSTEMS_ICC__)
  #pragma language=extended
#else
  #error Not supported compiler type
#endif

/* ----------------------------------------------------------------------------
   -- ADC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ADC_Peripheral_Access_Layer ADC Peripheral Access Layer
 * @{
 */

/** ADC - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID Register, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter Register, offset: 0x4 */
       uint8_t RESERVED_0[8];
  __IO uint32_t CTRL;                              /**< Control Register, offset: 0x10 */
  __IO uint32_t STAT;                              /**< Status Register, offset: 0x14 */
  __IO uint32_t IE;                                /**< Interrupt Enable Register, offset: 0x18 */
  __IO uint32_t DE;                                /**< DMA Enable Register, offset: 0x1C */
  __IO uint32_t CFG;                               /**< Configuration Register, offset: 0x20 */
  __IO uint32_t PAUSE;                             /**< Pause Register, offset: 0x24 */
       uint8_t RESERVED_1[12];
  __O  uint32_t SWTRIG;                            /**< Software Trigger Register, offset: 0x34 */
  __IO uint32_t TSTAT;                             /**< Trigger Status Register, offset: 0x38 */
       uint8_t RESERVED_2[4];
  __IO uint32_t OFSTRIM16;                         /**< Offset Trim 16 bit Register, offset: 0x40 */
  __IO uint32_t OFSTRIM12;                         /**< Offset Trim 12 bit Register, offset: 0x44 */
       uint8_t RESERVED_3[88];
  __IO uint32_t TCTRL[8];                          /**< Trigger Control Register, array offset: 0xA0, array step: 0x4 */
       uint8_t RESERVED_4[32];
  __IO uint32_t FCTRL[2];                          /**< FIFO Control Register, array offset: 0xE0, array step: 0x4 */
       uint8_t RESERVED_5[8];
  __I  uint32_t GCC[2];                            /**< Gain Calibration Control, array offset: 0xF0, array step: 0x4 */
  __IO uint32_t GCR[2];                            /**< Gain Calculation Result, array offset: 0xF8, array step: 0x4 */
  struct {                                         /* offset: 0x100, array step: 0x8 */
    __IO uint32_t CMDL;                              /**< Command Low Buffer Register, array offset: 0x100, array step: 0x8 */
    __IO uint32_t CMDH;                              /**< Command High Buffer Register, array offset: 0x104, array step: 0x8 */
  } CMD[15];
       uint8_t RESERVED_6[136];
  __IO uint32_t CV[4];                             /**< Compare Value Register, array offset: 0x200, array step: 0x4 */
       uint8_t RESERVED_7[240];
  __I  uint32_t RESFIFO[2];                        /**< Data Result FIFO Register, array offset: 0x300, array step: 0x4 */
       uint8_t RESERVED_8[248];
  __IO uint32_t CAL_GAR[33];                       /**< Calibration General A-Side Registers, array offset: 0x400, array step: 0x4 */
       uint8_t RESERVED_9[124];
  __IO uint32_t CAL_GBR[33];                       /**< Calibration General B-Side Registers, array offset: 0x500, array step: 0x4 */
       uint8_t RESERVED_10[2676];
  __IO uint32_t CFG2;                              /**< Configuration 2 Register, offset: 0xFF8 */
} ADC_Type;

/* ----------------------------------------------------------------------------
   -- ADC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ADC_Register_Masks ADC Register Masks
 * @{
 */

/*! @name VERID - Version ID Register */
/*! @{ */

#define ADC_VERID_RES_MASK                       (0x1U)
#define ADC_VERID_RES_SHIFT                      (0U)
/*! RES - Resolution
 *  0b0..Up to 13-bit differential/12-bit single ended resolution supported.
 *  0b1..Up to 16-bit differential/16-bit single ended resolution supported.
 */
#define ADC_VERID_RES(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_VERID_RES_SHIFT)) & ADC_VERID_RES_MASK)

#define ADC_VERID_DIFFEN_MASK                    (0x2U)
#define ADC_VERID_DIFFEN_SHIFT                   (1U)
/*! DIFFEN - Differential Supported
 *  0b0..Differential operation not supported.
 *  0b1..Differential operation supported. CMDLa[CTYPE] controls fields implemented.
 */
#define ADC_VERID_DIFFEN(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_VERID_DIFFEN_SHIFT)) & ADC_VERID_DIFFEN_MASK)

#define ADC_VERID_MVI_MASK                       (0x8U)
#define ADC_VERID_MVI_SHIFT                      (3U)
/*! MVI - Multi Vref Implemented
 *  0b0..Single voltage reference high (VREFH) input supported.
 *  0b1..Multiple voltage reference high (VREFH) inputs supported.
 */
#define ADC_VERID_MVI(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_VERID_MVI_SHIFT)) & ADC_VERID_MVI_MASK)

#define ADC_VERID_CSW_MASK                       (0x70U)
#define ADC_VERID_CSW_SHIFT                      (4U)
/*! CSW - Channel Scale Width
 *  0b000..Channel scaling not supported. CSCALE control field not implemented.
 *  0b001..Channel scaling supported. 1-bit CSCALE control field.
 *  0b110..Channel scaling supported. 6-bit CSCALE control field.
 */
#define ADC_VERID_CSW(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_VERID_CSW_SHIFT)) & ADC_VERID_CSW_MASK)

#define ADC_VERID_VR1RNGI_MASK                   (0x100U)
#define ADC_VERID_VR1RNGI_SHIFT                  (8U)
/*! VR1RNGI - Voltage Reference 1 Range Control Bit Implemented
 *  0b0..Range control not required. CFG[VREF1RNG] is not implemented.
 *  0b1..Range control required. CFG[VREF1RNG] is implemented.
 */
#define ADC_VERID_VR1RNGI(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_VERID_VR1RNGI_SHIFT)) & ADC_VERID_VR1RNGI_MASK)

#define ADC_VERID_IADCKI_MASK                    (0x200U)
#define ADC_VERID_IADCKI_SHIFT                   (9U)
/*! IADCKI - Internal ADC Clock Implemented
 *  0b0..Internal clock source not implemented.
 *  0b1..Internal clock source (and CFG[ADCKEN]) implemented.
 */
#define ADC_VERID_IADCKI(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_VERID_IADCKI_SHIFT)) & ADC_VERID_IADCKI_MASK)

#define ADC_VERID_CALOFSI_MASK                   (0x400U)
#define ADC_VERID_CALOFSI_SHIFT                  (10U)
/*! CALOFSI - Calibration Function Implemented
 *  0b0..Calibration Not Implemented.
 *  0b1..Calibration Implemented.
 */
#define ADC_VERID_CALOFSI(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_VERID_CALOFSI_SHIFT)) & ADC_VERID_CALOFSI_MASK)

#define ADC_VERID_NUM_SEC_MASK                   (0x800U)
#define ADC_VERID_NUM_SEC_SHIFT                  (11U)
/*! NUM_SEC - Number of Single Ended Outputs Supported
 *  0b0..This design supports one single ended conversion at a time.
 *  0b1..This design supports two simultaneous single ended conversions.
 */
#define ADC_VERID_NUM_SEC(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_VERID_NUM_SEC_SHIFT)) & ADC_VERID_NUM_SEC_MASK)

#define ADC_VERID_NUM_FIFO_MASK                  (0x7000U)
#define ADC_VERID_NUM_FIFO_SHIFT                 (12U)
/*! NUM_FIFO - Number of FIFOs
 *  0b000..N/A
 *  0b001..This design supports one result FIFO.
 *  0b010..This design supports two result FIFOs.
 *  0b011..This design supports three result FIFOs.
 *  0b100..This design supports four result FIFOs.
 */
#define ADC_VERID_NUM_FIFO(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_VERID_NUM_FIFO_SHIFT)) & ADC_VERID_NUM_FIFO_MASK)

#define ADC_VERID_MINOR_MASK                     (0xFF0000U)
#define ADC_VERID_MINOR_SHIFT                    (16U)
/*! MINOR - Minor Version Number */
#define ADC_VERID_MINOR(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_VERID_MINOR_SHIFT)) & ADC_VERID_MINOR_MASK)

#define ADC_VERID_MAJOR_MASK                     (0xFF000000U)
#define ADC_VERID_MAJOR_SHIFT                    (24U)
/*! MAJOR - Major Version Number */
#define ADC_VERID_MAJOR(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_VERID_MAJOR_SHIFT)) & ADC_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter Register */
/*! @{ */

#define ADC_PARAM_TRIG_NUM_MASK                  (0xFFU)
#define ADC_PARAM_TRIG_NUM_SHIFT                 (0U)
/*! TRIG_NUM - Trigger Number */
#define ADC_PARAM_TRIG_NUM(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_PARAM_TRIG_NUM_SHIFT)) & ADC_PARAM_TRIG_NUM_MASK)

#define ADC_PARAM_FIFOSIZE_MASK                  (0xFF00U)
#define ADC_PARAM_FIFOSIZE_SHIFT                 (8U)
/*! FIFOSIZE - Result FIFO Depth
 *  0b00000001..Result FIFO depth = 2 dataword.
 *  0b00000100..Result FIFO depth = 4 datawords.
 *  0b00001000..Result FIFO depth = 8 datawords.
 *  0b00010000..Result FIFO depth = 16 datawords.
 *  0b00100000..Result FIFO depth = 32 datawords.
 *  0b01000000..Result FIFO depth = 64 datawords.
 */
#define ADC_PARAM_FIFOSIZE(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_PARAM_FIFOSIZE_SHIFT)) & ADC_PARAM_FIFOSIZE_MASK)

#define ADC_PARAM_CV_NUM_MASK                    (0xFF0000U)
#define ADC_PARAM_CV_NUM_SHIFT                   (16U)
/*! CV_NUM - Compare Value Number */
#define ADC_PARAM_CV_NUM(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_PARAM_CV_NUM_SHIFT)) & ADC_PARAM_CV_NUM_MASK)

#define ADC_PARAM_CMD_NUM_MASK                   (0xFF000000U)
#define ADC_PARAM_CMD_NUM_SHIFT                  (24U)
/*! CMD_NUM - Command Buffer Number */
#define ADC_PARAM_CMD_NUM(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_PARAM_CMD_NUM_SHIFT)) & ADC_PARAM_CMD_NUM_MASK)
/*! @} */

/*! @name CTRL - Control Register */
/*! @{ */

#define ADC_CTRL_ADCEN_MASK                      (0x1U)
#define ADC_CTRL_ADCEN_SHIFT                     (0U)
/*! ADCEN - ADC Enable
 *  0b0..ADC is disabled.
 *  0b1..ADC is enabled.
 */
#define ADC_CTRL_ADCEN(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_ADCEN_SHIFT)) & ADC_CTRL_ADCEN_MASK)

#define ADC_CTRL_RST_MASK                        (0x2U)
#define ADC_CTRL_RST_SHIFT                       (1U)
/*! RST - Software Reset
 *  0b0..ADC logic is not reset.
 *  0b1..ADC logic is reset.
 */
#define ADC_CTRL_RST(x)                          (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_RST_SHIFT)) & ADC_CTRL_RST_MASK)

#define ADC_CTRL_DOZEN_MASK                      (0x4U)
#define ADC_CTRL_DOZEN_SHIFT                     (2U)
/*! DOZEN - Doze Enable
 *  0b0..ADC is enabled in low power mode.
 *  0b1..ADC is disabled in low power mode.
 */
#define ADC_CTRL_DOZEN(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_DOZEN_SHIFT)) & ADC_CTRL_DOZEN_MASK)

#define ADC_CTRL_CAL_REQ_MASK                    (0x8U)
#define ADC_CTRL_CAL_REQ_SHIFT                   (3U)
/*! CAL_REQ - Auto-Calibration Request
 *  0b0..No request for hardware calibration has been made.
 *  0b1..A request for hardware calibration has been made
 */
#define ADC_CTRL_CAL_REQ(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_CAL_REQ_SHIFT)) & ADC_CTRL_CAL_REQ_MASK)

#define ADC_CTRL_CALOFS_MASK                     (0x10U)
#define ADC_CTRL_CALOFS_SHIFT                    (4U)
/*! CALOFS - Offset Calibration Request
 *  0b0..Calibration function disabled
 *  0b1..Request for offset calibration function
 */
#define ADC_CTRL_CALOFS(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_CALOFS_SHIFT)) & ADC_CTRL_CALOFS_MASK)

#define ADC_CTRL_CALOFSMODE_MASK                 (0x20U)
#define ADC_CTRL_CALOFSMODE_SHIFT                (5U)
/*! CALOFSMODE - Configure Mode for Offset Calibration Function
 *  0b0..Configure offset calibration for 12-bit mode.
 *  0b1..Configure offset calibration for 16-bit mode.
 */
#define ADC_CTRL_CALOFSMODE(x)                   (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_CALOFSMODE_SHIFT)) & ADC_CTRL_CALOFSMODE_MASK)

#define ADC_CTRL_RSTFIFO0_MASK                   (0x100U)
#define ADC_CTRL_RSTFIFO0_SHIFT                  (8U)
/*! RSTFIFO0 - Reset FIFO 0
 *  0b0..No effect.
 *  0b1..FIFO 0 is reset.
 */
#define ADC_CTRL_RSTFIFO0(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_RSTFIFO0_SHIFT)) & ADC_CTRL_RSTFIFO0_MASK)

#define ADC_CTRL_RSTFIFO1_MASK                   (0x200U)
#define ADC_CTRL_RSTFIFO1_SHIFT                  (9U)
/*! RSTFIFO1 - Reset FIFO 1
 *  0b0..No effect.
 *  0b1..FIFO 1 is reset.
 */
#define ADC_CTRL_RSTFIFO1(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_RSTFIFO1_SHIFT)) & ADC_CTRL_RSTFIFO1_MASK)

#define ADC_CTRL_CAL_AVGS_MASK                   (0xF0000U)
#define ADC_CTRL_CAL_AVGS_SHIFT                  (16U)
/*! CAL_AVGS - Auto-Calibration Averages
 *  0b0000..Single conversion.
 *  0b0001..2 conversions averaged.
 *  0b0010..4 conversions averaged.
 *  0b0011..8 conversions averaged.
 *  0b0100..16 conversions averaged.
 *  0b0101..32 conversions averaged.
 *  0b0110..64 conversions averaged.
 *  0b0111..128 conversions averaged.
 *  0b1000..256 conversions averaged.
 *  0b1001..512 conversions averaged.
 *  0b1010..1024 conversions averaged.
 */
#define ADC_CTRL_CAL_AVGS(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_CTRL_CAL_AVGS_SHIFT)) & ADC_CTRL_CAL_AVGS_MASK)
/*! @} */

/*! @name STAT - Status Register */
/*! @{ */

#define ADC_STAT_RDY0_MASK                       (0x1U)
#define ADC_STAT_RDY0_SHIFT                      (0U)
/*! RDY0 - Result FIFO 0 Ready Flag
 *  0b0..Result FIFO 0 data level not above watermark level.
 *  0b1..Result FIFO 0 holding data above watermark level.
 */
#define ADC_STAT_RDY0(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_STAT_RDY0_SHIFT)) & ADC_STAT_RDY0_MASK)

#define ADC_STAT_FOF0_MASK                       (0x2U)
#define ADC_STAT_FOF0_SHIFT                      (1U)
/*! FOF0 - Result FIFO 0 Overflow Flag
 *  0b0..No result FIFO 0 overflow has occurred since the last time the flag was cleared.
 *  0b1..At least one result FIFO 0 overflow has occurred since the last time the flag was cleared.
 */
#define ADC_STAT_FOF0(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_STAT_FOF0_SHIFT)) & ADC_STAT_FOF0_MASK)

#define ADC_STAT_RDY1_MASK                       (0x4U)
#define ADC_STAT_RDY1_SHIFT                      (2U)
/*! RDY1 - Result FIFO1 Ready Flag
 *  0b0..Result FIFO1 data level not above watermark level.
 *  0b1..Result FIFO1 holding data above watermark level.
 */
#define ADC_STAT_RDY1(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_STAT_RDY1_SHIFT)) & ADC_STAT_RDY1_MASK)

#define ADC_STAT_FOF1_MASK                       (0x8U)
#define ADC_STAT_FOF1_SHIFT                      (3U)
/*! FOF1 - Result FIFO1 Overflow Flag
 *  0b0..No result FIFO1 overflow has occurred since the last time the flag was cleared.
 *  0b1..At least one result FIFO1 overflow has occurred since the last time the flag was cleared.
 */
#define ADC_STAT_FOF1(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_STAT_FOF1_SHIFT)) & ADC_STAT_FOF1_MASK)

#define ADC_STAT_TEXC_INT_MASK                   (0x100U)
#define ADC_STAT_TEXC_INT_SHIFT                  (8U)
/*! TEXC_INT - Interrupt Flag For High Priority Trigger Exception
 *  0b0..No trigger exceptions have occurred.
 *  0b1..A trigger exception has occurred and is pending acknowledgement.
 */
#define ADC_STAT_TEXC_INT(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_STAT_TEXC_INT_SHIFT)) & ADC_STAT_TEXC_INT_MASK)

#define ADC_STAT_TCOMP_INT_MASK                  (0x200U)
#define ADC_STAT_TCOMP_INT_SHIFT                 (9U)
/*! TCOMP_INT - Interrupt Flag For Trigger Completion
 *  0b0..Either IE[TCOMP_IE] is set to 0, or no trigger sequences have run to completion.
 *  0b1..Trigger sequence has been completed and all data is stored in the associated FIFO.
 */
#define ADC_STAT_TCOMP_INT(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_STAT_TCOMP_INT_SHIFT)) & ADC_STAT_TCOMP_INT_MASK)

#define ADC_STAT_CAL_RDY_MASK                    (0x400U)
#define ADC_STAT_CAL_RDY_SHIFT                   (10U)
/*! CAL_RDY - Calibration Ready
 *  0b0..Calibration is incomplete or hasn't been ran.
 *  0b1..The ADC is calibrated.
 */
#define ADC_STAT_CAL_RDY(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_STAT_CAL_RDY_SHIFT)) & ADC_STAT_CAL_RDY_MASK)

#define ADC_STAT_ADC_ACTIVE_MASK                 (0x800U)
#define ADC_STAT_ADC_ACTIVE_SHIFT                (11U)
/*! ADC_ACTIVE - ADC Active
 *  0b0..The ADC is IDLE. There are no pending triggers to service and no active commands are being processed.
 *  0b1..The ADC is processing a conversion, running through the power up delay, or servicing a trigger.
 */
#define ADC_STAT_ADC_ACTIVE(x)                   (((uint32_t)(((uint32_t)(x)) << ADC_STAT_ADC_ACTIVE_SHIFT)) & ADC_STAT_ADC_ACTIVE_MASK)

#define ADC_STAT_TRGACT_MASK                     (0x70000U)
#define ADC_STAT_TRGACT_SHIFT                    (16U)
/*! TRGACT - Trigger Active
 *  0b000..Command (sequence) associated with Trigger 0 currently being executed.
 *  0b001..Command (sequence) associated with Trigger 1 currently being executed.
 *  0b010..Command (sequence) associated with Trigger 2 currently being executed.
 *  0b011-0b111..Command (sequence) from the associated Trigger number is currently being executed.
 */
#define ADC_STAT_TRGACT(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_STAT_TRGACT_SHIFT)) & ADC_STAT_TRGACT_MASK)

#define ADC_STAT_CMDACT_MASK                     (0xF000000U)
#define ADC_STAT_CMDACT_SHIFT                    (24U)
/*! CMDACT - Command Active
 *  0b0000..No command is currently in progress.
 *  0b0001..Command 1 currently being executed.
 *  0b0010..Command 2 currently being executed.
 *  0b0011-0b1111..Associated command number is currently being executed.
 */
#define ADC_STAT_CMDACT(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_STAT_CMDACT_SHIFT)) & ADC_STAT_CMDACT_MASK)
/*! @} */

/*! @name IE - Interrupt Enable Register */
/*! @{ */

#define ADC_IE_FWMIE0_MASK                       (0x1U)
#define ADC_IE_FWMIE0_SHIFT                      (0U)
/*! FWMIE0 - FIFO 0 Watermark Interrupt Enable
 *  0b0..FIFO 0 watermark interrupts are not enabled.
 *  0b1..FIFO 0 watermark interrupts are enabled.
 */
#define ADC_IE_FWMIE0(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_IE_FWMIE0_SHIFT)) & ADC_IE_FWMIE0_MASK)

#define ADC_IE_FOFIE0_MASK                       (0x2U)
#define ADC_IE_FOFIE0_SHIFT                      (1U)
/*! FOFIE0 - Result FIFO 0 Overflow Interrupt Enable
 *  0b0..FIFO 0 overflow interrupts are not enabled.
 *  0b1..FIFO 0 overflow interrupts are enabled.
 */
#define ADC_IE_FOFIE0(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_IE_FOFIE0_SHIFT)) & ADC_IE_FOFIE0_MASK)

#define ADC_IE_FWMIE1_MASK                       (0x4U)
#define ADC_IE_FWMIE1_SHIFT                      (2U)
/*! FWMIE1 - FIFO1 Watermark Interrupt Enable
 *  0b0..FIFO1 watermark interrupts are not enabled.
 *  0b1..FIFO1 watermark interrupts are enabled.
 */
#define ADC_IE_FWMIE1(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_IE_FWMIE1_SHIFT)) & ADC_IE_FWMIE1_MASK)

#define ADC_IE_FOFIE1_MASK                       (0x8U)
#define ADC_IE_FOFIE1_SHIFT                      (3U)
/*! FOFIE1 - Result FIFO1 Overflow Interrupt Enable
 *  0b0..No result FIFO1 overflow has occurred since the last time the flag was cleared.
 *  0b1..At least one result FIFO1 overflow has occurred since the last time the flag was cleared.
 */
#define ADC_IE_FOFIE1(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_IE_FOFIE1_SHIFT)) & ADC_IE_FOFIE1_MASK)

#define ADC_IE_TEXC_IE_MASK                      (0x100U)
#define ADC_IE_TEXC_IE_SHIFT                     (8U)
/*! TEXC_IE - Trigger Exception Interrupt Enable
 *  0b0..Trigger exception interrupts are disabled.
 *  0b1..Trigger exception interrupts are enabled.
 */
#define ADC_IE_TEXC_IE(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_IE_TEXC_IE_SHIFT)) & ADC_IE_TEXC_IE_MASK)

#define ADC_IE_TCOMP_IE_MASK                     (0xFF0000U)
#define ADC_IE_TCOMP_IE_SHIFT                    (16U)
/*! TCOMP_IE - Trigger Completion Interrupt Enable
 *  0b00000000..Trigger completion interrupts are disabled.
 *  0b00000001..Trigger completion interrupts are enabled for trigger source 0 only.
 *  0b00000010..Trigger completion interrupts are enabled for trigger source 1 only.
 *  0b00000011-0b11111110..Associated trigger completion interrupts are enabled.
 *  0b11111111..Trigger completion interrupts are enabled for every trigger source.
 */
#define ADC_IE_TCOMP_IE(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_IE_TCOMP_IE_SHIFT)) & ADC_IE_TCOMP_IE_MASK)
/*! @} */

/*! @name DE - DMA Enable Register */
/*! @{ */

#define ADC_DE_FWMDE0_MASK                       (0x1U)
#define ADC_DE_FWMDE0_SHIFT                      (0U)
/*! FWMDE0 - FIFO 0 Watermark DMA Enable
 *  0b0..DMA request disabled.
 *  0b1..DMA request enabled.
 */
#define ADC_DE_FWMDE0(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_DE_FWMDE0_SHIFT)) & ADC_DE_FWMDE0_MASK)

#define ADC_DE_FWMDE1_MASK                       (0x2U)
#define ADC_DE_FWMDE1_SHIFT                      (1U)
/*! FWMDE1 - FIFO1 Watermark DMA Enable
 *  0b0..DMA request disabled.
 *  0b1..DMA request enabled.
 */
#define ADC_DE_FWMDE1(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_DE_FWMDE1_SHIFT)) & ADC_DE_FWMDE1_MASK)
/*! @} */

/*! @name CFG - Configuration Register */
/*! @{ */

#define ADC_CFG_TPRICTRL_MASK                    (0x3U)
#define ADC_CFG_TPRICTRL_SHIFT                   (0U)
/*! TPRICTRL - ADC Trigger Priority Control
 *  0b00..If a higher priority trigger is detected during command processing, the current conversion is aborted
 *        and the new command specified by the trigger is started.
 *  0b01..If a higher priority trigger is received during command processing, the current command is stopped after
 *        completing the current conversion. If averaging is enabled, the averaging loop will be completed.
 *        However, CMDHa[LOOP] will be ignored and the higher priority trigger will be serviced.
 *  0b10..If a higher priority trigger is received during command processing, the current command will be
 *        completed (averaging, looping, compare) before servicing the higher priority trigger.
 *  0b11..RESERVED
 */
#define ADC_CFG_TPRICTRL(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_CFG_TPRICTRL_SHIFT)) & ADC_CFG_TPRICTRL_MASK)

#define ADC_CFG_REFSEL_MASK                      (0xC0U)
#define ADC_CFG_REFSEL_SHIFT                     (6U)
/*! REFSEL - Voltage Reference Selection
 *  0b00..(Default) Option 1 setting.
 *  0b01..Option 2 setting.
 *  0b10..Option 3 setting.
 *  0b11..Reserved
 */
#define ADC_CFG_REFSEL(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_CFG_REFSEL_SHIFT)) & ADC_CFG_REFSEL_MASK)

#define ADC_CFG_TRES_MASK                        (0x100U)
#define ADC_CFG_TRES_SHIFT                       (8U)
/*! TRES - Trigger Resume Enable
 *  0b0..Trigger sequences interrupted by a high priority trigger exception are not automatically resumed or restarted.
 *  0b1..Trigger sequences interrupted by a high priority trigger exception are automatically resumed or restarted.
 */
#define ADC_CFG_TRES(x)                          (((uint32_t)(((uint32_t)(x)) << ADC_CFG_TRES_SHIFT)) & ADC_CFG_TRES_MASK)

#define ADC_CFG_TCMDRES_MASK                     (0x200U)
#define ADC_CFG_TCMDRES_SHIFT                    (9U)
/*! TCMDRES - Trigger Command Resume
 *  0b0..Trigger sequences interrupted by a high priority trigger exception is automatically restarted.
 *  0b1..Trigger sequences interrupted by a high priority trigger exception is resumed from the command executing before the exception.
 */
#define ADC_CFG_TCMDRES(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_CFG_TCMDRES_SHIFT)) & ADC_CFG_TCMDRES_MASK)

#define ADC_CFG_HPT_EXDI_MASK                    (0x400U)
#define ADC_CFG_HPT_EXDI_SHIFT                   (10U)
/*! HPT_EXDI - High Priority Trigger Exception Disable
 *  0b0..High priority trigger exceptions are enabled.
 *  0b1..High priority trigger exceptions are disabled.
 */
#define ADC_CFG_HPT_EXDI(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_CFG_HPT_EXDI_SHIFT)) & ADC_CFG_HPT_EXDI_MASK)

#define ADC_CFG_PUDLY_MASK                       (0xFF0000U)
#define ADC_CFG_PUDLY_SHIFT                      (16U)
/*! PUDLY - Power Up Delay */
#define ADC_CFG_PUDLY(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_CFG_PUDLY_SHIFT)) & ADC_CFG_PUDLY_MASK)

#define ADC_CFG_PWREN_MASK                       (0x10000000U)
#define ADC_CFG_PWREN_SHIFT                      (28U)
/*! PWREN - ADC Analog Pre-Enable
 *  0b0..ADC analog circuits are only enabled while conversions are active. Performance is affected due to analog startup delays.
 *  0b1..ADC analog circuits are pre-enabled and ready to execute conversions without startup delays (at the cost
 *       of higher DC current consumption). Note that a single power up delay (CFG[PUDLY]) is executed immediately
 *       once PWREN is set, and any detected trigger does not begin ADC operation until the power up delay time has
 *       passed. After this initial delay expires the analog remains pre-enabled and no additional delays are
 *       executed.
 */
#define ADC_CFG_PWREN(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_CFG_PWREN_SHIFT)) & ADC_CFG_PWREN_MASK)
/*! @} */

/*! @name PAUSE - Pause Register */
/*! @{ */

#define ADC_PAUSE_PAUSEDLY_MASK                  (0x1FFU)
#define ADC_PAUSE_PAUSEDLY_SHIFT                 (0U)
/*! PAUSEDLY - Pause Delay */
#define ADC_PAUSE_PAUSEDLY(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_PAUSE_PAUSEDLY_SHIFT)) & ADC_PAUSE_PAUSEDLY_MASK)

#define ADC_PAUSE_PAUSEEN_MASK                   (0x80000000U)
#define ADC_PAUSE_PAUSEEN_SHIFT                  (31U)
/*! PAUSEEN - PAUSE Option Enable
 *  0b0..Pause operation disabled
 *  0b1..Pause operation enabled
 */
#define ADC_PAUSE_PAUSEEN(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_PAUSE_PAUSEEN_SHIFT)) & ADC_PAUSE_PAUSEEN_MASK)
/*! @} */

/*! @name SWTRIG - Software Trigger Register */
/*! @{ */

#define ADC_SWTRIG_SWT0_MASK                     (0x1U)
#define ADC_SWTRIG_SWT0_SHIFT                    (0U)
/*! SWT0 - Software Trigger 0 Event
 *  0b0..No trigger 0 event generated.
 *  0b1..Trigger 0 event generated.
 */
#define ADC_SWTRIG_SWT0(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT0_SHIFT)) & ADC_SWTRIG_SWT0_MASK)

#define ADC_SWTRIG_SWT1_MASK                     (0x2U)
#define ADC_SWTRIG_SWT1_SHIFT                    (1U)
/*! SWT1 - Software Trigger 1 Event
 *  0b0..No trigger 1 event generated.
 *  0b1..Trigger 1 event generated.
 */
#define ADC_SWTRIG_SWT1(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT1_SHIFT)) & ADC_SWTRIG_SWT1_MASK)

#define ADC_SWTRIG_SWT2_MASK                     (0x4U)
#define ADC_SWTRIG_SWT2_SHIFT                    (2U)
/*! SWT2 - Software Trigger 2 Event
 *  0b0..No trigger 2 event generated.
 *  0b1..Trigger 2 event generated.
 */
#define ADC_SWTRIG_SWT2(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT2_SHIFT)) & ADC_SWTRIG_SWT2_MASK)

#define ADC_SWTRIG_SWT3_MASK                     (0x8U)
#define ADC_SWTRIG_SWT3_SHIFT                    (3U)
/*! SWT3 - Software Trigger 3 Event
 *  0b0..No trigger 3 event generated.
 *  0b1..Trigger 3 event generated.
 */
#define ADC_SWTRIG_SWT3(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT3_SHIFT)) & ADC_SWTRIG_SWT3_MASK)

#define ADC_SWTRIG_SWT4_MASK                     (0x10U)
#define ADC_SWTRIG_SWT4_SHIFT                    (4U)
/*! SWT4 - Software trigger 4 event
 *  0b0..No trigger 4 event generated.
 *  0b1..Trigger 4 event generated.
 */
#define ADC_SWTRIG_SWT4(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT4_SHIFT)) & ADC_SWTRIG_SWT4_MASK)

#define ADC_SWTRIG_SWT5_MASK                     (0x20U)
#define ADC_SWTRIG_SWT5_SHIFT                    (5U)
/*! SWT5 - Software trigger 5 event
 *  0b0..No trigger 5 event generated.
 *  0b1..Trigger 5 event generated.
 */
#define ADC_SWTRIG_SWT5(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT5_SHIFT)) & ADC_SWTRIG_SWT5_MASK)

#define ADC_SWTRIG_SWT6_MASK                     (0x40U)
#define ADC_SWTRIG_SWT6_SHIFT                    (6U)
/*! SWT6 - Software trigger 6 event
 *  0b0..No trigger 6 event generated.
 *  0b1..Trigger 6 event generated.
 */
#define ADC_SWTRIG_SWT6(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT6_SHIFT)) & ADC_SWTRIG_SWT6_MASK)

#define ADC_SWTRIG_SWT7_MASK                     (0x80U)
#define ADC_SWTRIG_SWT7_SHIFT                    (7U)
/*! SWT7 - Software trigger 7 event
 *  0b0..No trigger 7 event generated.
 *  0b1..Trigger 7 event generated.
 */
#define ADC_SWTRIG_SWT7(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_SWTRIG_SWT7_SHIFT)) & ADC_SWTRIG_SWT7_MASK)
/*! @} */

/*! @name TSTAT - Trigger Status Register */
/*! @{ */

#define ADC_TSTAT_TEXC_NUM_MASK                  (0xFFU)
#define ADC_TSTAT_TEXC_NUM_SHIFT                 (0U)
/*! TEXC_NUM - Trigger Exception Number
 *  0b00000000..No triggers have been interrupted by a high priority exception. Or CFG[TRES] = 1.
 *  0b00000001..Trigger 0 has been interrupted by a high priority exception.
 *  0b00000010..Trigger 1 has been interrupted by a high priority exception.
 *  0b00000011-0b11111110..Associated trigger sequence has interrupted by a high priority exception.
 *  0b11111111..Every trigger sequence has been interrupted by a high priority exception.
 */
#define ADC_TSTAT_TEXC_NUM(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_TSTAT_TEXC_NUM_SHIFT)) & ADC_TSTAT_TEXC_NUM_MASK)

#define ADC_TSTAT_TCOMP_FLAG_MASK                (0xFF0000U)
#define ADC_TSTAT_TCOMP_FLAG_SHIFT               (16U)
/*! TCOMP_FLAG - Trigger Completion Flag
 *  0b00000000..No triggers have been completed. Trigger completion interrupts are disabled.
 *  0b00000001..Trigger 0 has been completed and trigger 0 has enabled completion interrupts.
 *  0b00000010..Trigger 1 has been completed and trigger 1 has enabled completion interrupts.
 *  0b00000011-0b11111110..Associated trigger sequence has completed and has enabled completion interrupts.
 *  0b11111111..Every trigger sequence has been completed and every trigger has enabled completion interrupts.
 */
#define ADC_TSTAT_TCOMP_FLAG(x)                  (((uint32_t)(((uint32_t)(x)) << ADC_TSTAT_TCOMP_FLAG_SHIFT)) & ADC_TSTAT_TCOMP_FLAG_MASK)
/*! @} */

/*! @name OFSTRIM16 - Offset Trim 16 bit Register */
/*! @{ */

#define ADC_OFSTRIM16_OFSTRIM_A_MASK             (0x3FFU)
#define ADC_OFSTRIM16_OFSTRIM_A_SHIFT            (0U)
/*! OFSTRIM_A - Trim for Offset in A-side Converter for 16-bit Conversions */
#define ADC_OFSTRIM16_OFSTRIM_A(x)               (((uint32_t)(((uint32_t)(x)) << ADC_OFSTRIM16_OFSTRIM_A_SHIFT)) & ADC_OFSTRIM16_OFSTRIM_A_MASK)

#define ADC_OFSTRIM16_OFSTRIM_B_MASK             (0x3FF0000U)
#define ADC_OFSTRIM16_OFSTRIM_B_SHIFT            (16U)
/*! OFSTRIM_B - Trim for Offset in B-side Converter for 16-bit Conversions */
#define ADC_OFSTRIM16_OFSTRIM_B(x)               (((uint32_t)(((uint32_t)(x)) << ADC_OFSTRIM16_OFSTRIM_B_SHIFT)) & ADC_OFSTRIM16_OFSTRIM_B_MASK)
/*! @} */

/*! @name OFSTRIM12 - Offset Trim 12 bit Register */
/*! @{ */

#define ADC_OFSTRIM12_OFSTRIM_A_MASK             (0x3FFU)
#define ADC_OFSTRIM12_OFSTRIM_A_SHIFT            (0U)
/*! OFSTRIM_A - Trim for Offset in A-side Converter for 12-bit Conversions */
#define ADC_OFSTRIM12_OFSTRIM_A(x)               (((uint32_t)(((uint32_t)(x)) << ADC_OFSTRIM12_OFSTRIM_A_SHIFT)) & ADC_OFSTRIM12_OFSTRIM_A_MASK)

#define ADC_OFSTRIM12_OFSTRIM_B_MASK             (0x3FF0000U)
#define ADC_OFSTRIM12_OFSTRIM_B_SHIFT            (16U)
/*! OFSTRIM_B - Trim for Offset in B-side Converter for 12-bit Conversions */
#define ADC_OFSTRIM12_OFSTRIM_B(x)               (((uint32_t)(((uint32_t)(x)) << ADC_OFSTRIM12_OFSTRIM_B_SHIFT)) & ADC_OFSTRIM12_OFSTRIM_B_MASK)
/*! @} */

/*! @name TCTRL - Trigger Control Register */
/*! @{ */

#define ADC_TCTRL_HTEN_MASK                      (0x1U)
#define ADC_TCTRL_HTEN_SHIFT                     (0U)
/*! HTEN - Trigger Enable
 *  0b0..Hardware trigger source disabled
 *  0b1..Hardware trigger source enabled
 */
#define ADC_TCTRL_HTEN(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_TCTRL_HTEN_SHIFT)) & ADC_TCTRL_HTEN_MASK)

#define ADC_TCTRL_FIFO_SEL_A_MASK                (0x2U)
#define ADC_TCTRL_FIFO_SEL_A_SHIFT               (1U)
/*! FIFO_SEL_A - SAR Result Destination for Channel A
 *  0b0..Result written to FIFO 0
 *  0b1..Result written to FIFO 1
 */
#define ADC_TCTRL_FIFO_SEL_A(x)                  (((uint32_t)(((uint32_t)(x)) << ADC_TCTRL_FIFO_SEL_A_SHIFT)) & ADC_TCTRL_FIFO_SEL_A_MASK)

#define ADC_TCTRL_FIFO_SEL_B_MASK                (0x4U)
#define ADC_TCTRL_FIFO_SEL_B_SHIFT               (2U)
/*! FIFO_SEL_B - SAR Result Destination for Channel B
 *  0b0..Result written to FIFO 0
 *  0b1..Result written to FIFO 1
 */
#define ADC_TCTRL_FIFO_SEL_B(x)                  (((uint32_t)(((uint32_t)(x)) << ADC_TCTRL_FIFO_SEL_B_SHIFT)) & ADC_TCTRL_FIFO_SEL_B_MASK)

#define ADC_TCTRL_TPRI_MASK                      (0x700U)
#define ADC_TCTRL_TPRI_SHIFT                     (8U)
/*! TPRI - Trigger Priority Setting
 *  0b000..Set to highest priority, Level 1
 *  0b001-0b110..Set to corresponding priority level
 *  0b111..Set to lowest priority, Level 8
 */
#define ADC_TCTRL_TPRI(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_TCTRL_TPRI_SHIFT)) & ADC_TCTRL_TPRI_MASK)

#define ADC_TCTRL_RSYNC_MASK                     (0x8000U)
#define ADC_TCTRL_RSYNC_SHIFT                    (15U)
/*! RSYNC - Trigger Resync */
#define ADC_TCTRL_RSYNC(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_TCTRL_RSYNC_SHIFT)) & ADC_TCTRL_RSYNC_MASK)

#define ADC_TCTRL_TDLY_MASK                      (0xF0000U)
#define ADC_TCTRL_TDLY_SHIFT                     (16U)
/*! TDLY - Trigger Delay Select */
#define ADC_TCTRL_TDLY(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_TCTRL_TDLY_SHIFT)) & ADC_TCTRL_TDLY_MASK)

#define ADC_TCTRL_TCMD_MASK                      (0xF000000U)
#define ADC_TCTRL_TCMD_SHIFT                     (24U)
/*! TCMD - Trigger Command Select
 *  0b0000..Not a valid selection from the command buffer. Trigger event is ignored.
 *  0b0001..CMD1 is executed
 *  0b0010-0b1110..Corresponding CMD is executed
 *  0b1111..CMD15 is executed
 */
#define ADC_TCTRL_TCMD(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_TCTRL_TCMD_SHIFT)) & ADC_TCTRL_TCMD_MASK)
/*! @} */

/* The count of ADC_TCTRL */
#define ADC_TCTRL_COUNT                          (8U)

/*! @name FCTRL - FIFO Control Register */
/*! @{ */

#define ADC_FCTRL_FCOUNT_MASK                    (0x1FU)
#define ADC_FCTRL_FCOUNT_SHIFT                   (0U)
/*! FCOUNT - Result FIFO Counter */
#define ADC_FCTRL_FCOUNT(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_FCTRL_FCOUNT_SHIFT)) & ADC_FCTRL_FCOUNT_MASK)

#define ADC_FCTRL_FWMARK_MASK                    (0xF0000U)
#define ADC_FCTRL_FWMARK_SHIFT                   (16U)
/*! FWMARK - Watermark Level Selection */
#define ADC_FCTRL_FWMARK(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_FCTRL_FWMARK_SHIFT)) & ADC_FCTRL_FWMARK_MASK)
/*! @} */

/* The count of ADC_FCTRL */
#define ADC_FCTRL_COUNT                          (2U)

/*! @name GCC - Gain Calibration Control */
/*! @{ */

#define ADC_GCC_GAIN_CAL_MASK                    (0xFFFFU)
#define ADC_GCC_GAIN_CAL_SHIFT                   (0U)
/*! GAIN_CAL - Gain Calibration Value */
#define ADC_GCC_GAIN_CAL(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_GCC_GAIN_CAL_SHIFT)) & ADC_GCC_GAIN_CAL_MASK)

#define ADC_GCC_RDY_MASK                         (0x1000000U)
#define ADC_GCC_RDY_SHIFT                        (24U)
/*! RDY - Hardware Calculated GAIN_CAL Value Ready
 *  0b0..The GAIN_CAL value is invalid. Run the hardware calibration routine for this value to be set.
 *  0b1..The GAIN_CAL value is valid. GAIN_CAL should be used by software to derive GCRa[GCALR].
 */
#define ADC_GCC_RDY(x)                           (((uint32_t)(((uint32_t)(x)) << ADC_GCC_RDY_SHIFT)) & ADC_GCC_RDY_MASK)
/*! @} */

/* The count of ADC_GCC */
#define ADC_GCC_COUNT                            (2U)

/*! @name GCR - Gain Calculation Result */
/*! @{ */

#define ADC_GCR_GCALR_MASK                       (0x1FFFFU)
#define ADC_GCR_GCALR_SHIFT                      (0U)
/*! GCALR - Gain Calculation Result */
#define ADC_GCR_GCALR(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_GCR_GCALR_SHIFT)) & ADC_GCR_GCALR_MASK)

#define ADC_GCR_RDY_MASK                         (0x1000000U)
#define ADC_GCR_RDY_SHIFT                        (24U)
/*! RDY - Gain Calculation Ready
 *  0b0..The GCALR value is invalid.
 *  0b1..The GCALR value is valid.
 */
#define ADC_GCR_RDY(x)                           (((uint32_t)(((uint32_t)(x)) << ADC_GCR_RDY_SHIFT)) & ADC_GCR_RDY_MASK)
/*! @} */

/* The count of ADC_GCR */
#define ADC_GCR_COUNT                            (2U)

/*! @name CMDL - Command Low Buffer Register */
/*! @{ */

#define ADC_CMDL_ADCH_MASK                       (0x1FU)
#define ADC_CMDL_ADCH_SHIFT                      (0U)
/*! ADCH - Input Channel Select
 *  0b00000..Select CH0A or CH0B or CH0A/CH0B pair.
 *  0b00001..Select CH1A or CH1B or CH1A/CH1B pair.
 *  0b00010..Select CH2A or CH2B or CH2A/CH2B pair.
 *  0b00011..Select CH3A or CH3B or CH3A/CH3B pair.
 *  0b00100-0b11101..Select corresponding channel CHnA or CHnB or CHnA/CHnB pair.
 *  0b11110..Select CH30A or CH30B or CH30A/CH30B pair.
 *  0b11111..Select CH31A or CH31B or CH31A/CH31B pair.
 */
#define ADC_CMDL_ADCH(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_CMDL_ADCH_SHIFT)) & ADC_CMDL_ADCH_MASK)

#define ADC_CMDL_CTYPE_MASK                      (0x60U)
#define ADC_CMDL_CTYPE_SHIFT                     (5U)
/*! CTYPE - Conversion Type
 *  0b00..Single-Ended Mode. Only A side channel is converted.
 *  0b01..Single-Ended Mode. Only B side channel is converted.
 *  0b10..Differential Mode. A-B.
 *  0b11..Dual-Single-Ended Mode. Both A side and B side channels are converted independently.
 */
#define ADC_CMDL_CTYPE(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_CMDL_CTYPE_SHIFT)) & ADC_CMDL_CTYPE_MASK)

#define ADC_CMDL_MODE_MASK                       (0x80U)
#define ADC_CMDL_MODE_SHIFT                      (7U)
/*! MODE - Select Resolution of Conversions
 *  0b0..Standard resolution. Single-ended 12-bit conversion; Differential 13-bit conversion with 2's complement output.
 *  0b1..High resolution. Single-ended 16-bit conversion; Differential 16-bit conversion with 2's complement output.
 */
#define ADC_CMDL_MODE(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_CMDL_MODE_SHIFT)) & ADC_CMDL_MODE_MASK)

#define ADC_CMDL_CSCALE_MASK                     (0x2000U)
#define ADC_CMDL_CSCALE_SHIFT                    (13U)
/*! CSCALE - Channel Scale
 *  0b0..Scale selected analog channel (Factor of 1/2)
 *  0b1..(Default) Full scale (Factor of 1)
 */
#define ADC_CMDL_CSCALE(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_CMDL_CSCALE_SHIFT)) & ADC_CMDL_CSCALE_MASK)

#define ADC_CMDL_ALTB_ADCH_MASK                  (0x1F0000U)
#define ADC_CMDL_ALTB_ADCH_SHIFT                 (16U)
/*! ALTB_ADCH - Alternate Channel B Input Channel Select
 *  0b00000..Select CH0B
 *  0b00001..Select CH1B
 *  0b00010..Select CH2B
 *  0b00011..Select CH3B
 *  0b00100-0b11101..Select corresponding channel CHnB
 *  0b11110..Select CH30B
 *  0b11111..Select CH31B
 */
#define ADC_CMDL_ALTB_ADCH(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_CMDL_ALTB_ADCH_SHIFT)) & ADC_CMDL_ALTB_ADCH_MASK)

#define ADC_CMDL_ALTBEN_MASK                     (0x200000U)
#define ADC_CMDL_ALTBEN_SHIFT                    (21U)
/*! ALTBEN - Alternate Channel B Select Enable
 *  0b0..ALTBEN_ADCH disabled. Channel A and Channel B inputs are selected based on ADCH settings.
 *  0b1..ALTBEN_ADCH enabled. Channel A inputs selected by ADCH setting and Channel B inputs selected by ALTB_ADCH setting.
 */
#define ADC_CMDL_ALTBEN(x)                       (((uint32_t)(((uint32_t)(x)) << ADC_CMDL_ALTBEN_SHIFT)) & ADC_CMDL_ALTBEN_MASK)

#define ADC_CMDL_ALTB_CSCALE_MASK                (0x800000U)
#define ADC_CMDL_ALTB_CSCALE_SHIFT               (23U)
/*! ALTB_CSCALE - Alt Channel B Scale
 *  0b0..Scale selected analog channel (Factor of 1/2)
 *  0b1..(Default) Full scale (Factor of 1)
 */
#define ADC_CMDL_ALTB_CSCALE(x)                  (((uint32_t)(((uint32_t)(x)) << ADC_CMDL_ALTB_CSCALE_SHIFT)) & ADC_CMDL_ALTB_CSCALE_MASK)
/*! @} */

/* The count of ADC_CMDL */
#define ADC_CMDL_COUNT                           (15U)

/*! @name CMDH - Command High Buffer Register */
/*! @{ */

#define ADC_CMDH_CMPEN_MASK                      (0x3U)
#define ADC_CMDH_CMPEN_SHIFT                     (0U)
/*! CMPEN - Compare Function Enable
 *  0b00..Compare disabled.
 *  0b01..Reserved
 *  0b10..Compare enabled. Store on true.
 *  0b11..Compare enabled. Repeat channel acquisition (sample/convert/compare) until true.
 */
#define ADC_CMDH_CMPEN(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_CMDH_CMPEN_SHIFT)) & ADC_CMDH_CMPEN_MASK)

#define ADC_CMDH_WAIT_TRIG_MASK                  (0x4U)
#define ADC_CMDH_WAIT_TRIG_SHIFT                 (2U)
/*! WAIT_TRIG - Wait for Trigger Assertion before Execution.
 *  0b0..This command will be automatically executed.
 *  0b1..The active trigger must be asserted again before executing this command.
 */
#define ADC_CMDH_WAIT_TRIG(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_CMDH_WAIT_TRIG_SHIFT)) & ADC_CMDH_WAIT_TRIG_MASK)

#define ADC_CMDH_LWI_MASK                        (0x80U)
#define ADC_CMDH_LWI_SHIFT                       (7U)
/*! LWI - Loop with Increment
 *  0b0..Auto channel increment disabled
 *  0b1..Auto channel increment enabled
 */
#define ADC_CMDH_LWI(x)                          (((uint32_t)(((uint32_t)(x)) << ADC_CMDH_LWI_SHIFT)) & ADC_CMDH_LWI_MASK)

#define ADC_CMDH_STS_MASK                        (0x700U)
#define ADC_CMDH_STS_SHIFT                       (8U)
/*! STS - Sample Time Select
 *  0b000..Minimum sample time of 3.5 ADCK cycles.
 *  0b001..3.5 + 21 ADCK cycles; 5.5 ADCK cycles total sample time.
 *  0b010..3.5 + 22 ADCK cycles; 7.5 ADCK cycles total sample time.
 *  0b011..3.5 + 23 ADCK cycles; 11.5 ADCK cycles total sample time.
 *  0b100..3.5 + 24 ADCK cycles; 19.5 ADCK cycles total sample time.
 *  0b101..3.5 + 25 ADCK cycles; 35.5 ADCK cycles total sample time.
 *  0b110..3.5 + 26 ADCK cycles; 67.5 ADCK cycles total sample time.
 *  0b111..3.5 + 27 ADCK cycles; 131.5 ADCK cycles total sample time.
 */
#define ADC_CMDH_STS(x)                          (((uint32_t)(((uint32_t)(x)) << ADC_CMDH_STS_SHIFT)) & ADC_CMDH_STS_MASK)

#define ADC_CMDH_AVGS_MASK                       (0xF000U)
#define ADC_CMDH_AVGS_SHIFT                      (12U)
/*! AVGS - Hardware Average Select
 *  0b0000..Single conversion.
 *  0b0001..2 conversions averaged.
 *  0b0010..4 conversions averaged.
 *  0b0011..8 conversions averaged.
 *  0b0100..16 conversions averaged.
 *  0b0101..32 conversions averaged.
 *  0b0110..64 conversions averaged.
 *  0b0111..128 conversions averaged.
 *  0b1000..256 conversions averaged.
 *  0b1001..512 conversions averaged.
 *  0b1010..1024 conversions averaged.
 */
#define ADC_CMDH_AVGS(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_CMDH_AVGS_SHIFT)) & ADC_CMDH_AVGS_MASK)

#define ADC_CMDH_LOOP_MASK                       (0xF0000U)
#define ADC_CMDH_LOOP_SHIFT                      (16U)
/*! LOOP - Loop Count Select
 *  0b0000..Looping not enabled. Command executes 1 time.
 *  0b0001..Loop 1 time. Command executes 2 times.
 *  0b0010..Loop 2 times. Command executes 3 times.
 *  0b0011-0b1110..Loop corresponding number of times. Command executes LOOP+1 times.
 *  0b1111..Loop 15 times. Command executes 16 times.
 */
#define ADC_CMDH_LOOP(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_CMDH_LOOP_SHIFT)) & ADC_CMDH_LOOP_MASK)

#define ADC_CMDH_NEXT_MASK                       (0xF000000U)
#define ADC_CMDH_NEXT_SHIFT                      (24U)
/*! NEXT - Next Command Select
 *  0b0000..No next command defined. Terminate conversions at completion of current command. If lower priority
 *          trigger pending, begin command associated with lower priority trigger.
 *  0b0001..Select CMD1 command buffer register as next command.
 *  0b0010-0b1110..Select corresponding CMD command buffer register as next command
 *  0b1111..Select CMD15 command buffer register as next command.
 */
#define ADC_CMDH_NEXT(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_CMDH_NEXT_SHIFT)) & ADC_CMDH_NEXT_MASK)
/*! @} */

/* The count of ADC_CMDH */
#define ADC_CMDH_COUNT                           (15U)

/*! @name CV - Compare Value Register */
/*! @{ */

#define ADC_CV_CVL_MASK                          (0xFFFFU)
#define ADC_CV_CVL_SHIFT                         (0U)
/*! CVL - Compare Value Low */
#define ADC_CV_CVL(x)                            (((uint32_t)(((uint32_t)(x)) << ADC_CV_CVL_SHIFT)) & ADC_CV_CVL_MASK)

#define ADC_CV_CVH_MASK                          (0xFFFF0000U)
#define ADC_CV_CVH_SHIFT                         (16U)
/*! CVH - Compare Value High */
#define ADC_CV_CVH(x)                            (((uint32_t)(((uint32_t)(x)) << ADC_CV_CVH_SHIFT)) & ADC_CV_CVH_MASK)
/*! @} */

/* The count of ADC_CV */
#define ADC_CV_COUNT                             (4U)

/*! @name RESFIFO - Data Result FIFO Register */
/*! @{ */

#define ADC_RESFIFO_D_MASK                       (0xFFFFU)
#define ADC_RESFIFO_D_SHIFT                      (0U)
/*! D - Data Result */
#define ADC_RESFIFO_D(x)                         (((uint32_t)(((uint32_t)(x)) << ADC_RESFIFO_D_SHIFT)) & ADC_RESFIFO_D_MASK)

#define ADC_RESFIFO_TSRC_MASK                    (0x70000U)
#define ADC_RESFIFO_TSRC_SHIFT                   (16U)
/*! TSRC - Trigger Source
 *  0b000..Trigger source 0 initiated this conversion.
 *  0b001..Trigger source 1 initiated this conversion.
 *  0b010-0b110..Corresponding trigger source initiated this conversion.
 *  0b111..Trigger source 7 initiated this conversion.
 */
#define ADC_RESFIFO_TSRC(x)                      (((uint32_t)(((uint32_t)(x)) << ADC_RESFIFO_TSRC_SHIFT)) & ADC_RESFIFO_TSRC_MASK)

#define ADC_RESFIFO_LOOPCNT_MASK                 (0xF00000U)
#define ADC_RESFIFO_LOOPCNT_SHIFT                (20U)
/*! LOOPCNT - Loop Count Value
 *  0b0000..Result is from initial conversion in command.
 *  0b0001..Result is from second conversion in command.
 *  0b0010-0b1110..Result is from LOOPCNT+1 conversion in command.
 *  0b1111..Result is from 16th conversion in command.
 */
#define ADC_RESFIFO_LOOPCNT(x)                   (((uint32_t)(((uint32_t)(x)) << ADC_RESFIFO_LOOPCNT_SHIFT)) & ADC_RESFIFO_LOOPCNT_MASK)

#define ADC_RESFIFO_CMDSRC_MASK                  (0xF000000U)
#define ADC_RESFIFO_CMDSRC_SHIFT                 (24U)
/*! CMDSRC - Command Buffer Source
 *  0b0000..Not a valid value CMDSRC value for a dataword in RESFIFO. 0x0 is only found in initial FIFO state
 *          prior to an ADC conversion result dataword being stored to a RESFIFO buffer.
 *  0b0001..CMD1 buffer used as control settings for this conversion.
 *  0b0010-0b1110..Corresponding command buffer used as control settings for this conversion.
 *  0b1111..CMD15 buffer used as control settings for this conversion.
 */
#define ADC_RESFIFO_CMDSRC(x)                    (((uint32_t)(((uint32_t)(x)) << ADC_RESFIFO_CMDSRC_SHIFT)) & ADC_RESFIFO_CMDSRC_MASK)

#define ADC_RESFIFO_VALID_MASK                   (0x80000000U)
#define ADC_RESFIFO_VALID_SHIFT                  (31U)
/*! VALID - FIFO Entry is Valid
 *  0b0..FIFO is empty. Discard any read from RESFIFO.
 *  0b1..FIFO record read from RESFIFO is valid.
 */
#define ADC_RESFIFO_VALID(x)                     (((uint32_t)(((uint32_t)(x)) << ADC_RESFIFO_VALID_SHIFT)) & ADC_RESFIFO_VALID_MASK)
/*! @} */

/* The count of ADC_RESFIFO */
#define ADC_RESFIFO_COUNT                        (2U)

/*! @name CAL_GAR - Calibration General A-Side Registers */
/*! @{ */

#define ADC_CAL_GAR_CAL_GAR_VAL_MASK             (0xFFFFU)
#define ADC_CAL_GAR_CAL_GAR_VAL_SHIFT            (0U)
/*! CAL_GAR_VAL - Calibration General A Side Register Element */
#define ADC_CAL_GAR_CAL_GAR_VAL(x)               (((uint32_t)(((uint32_t)(x)) << ADC_CAL_GAR_CAL_GAR_VAL_SHIFT)) & ADC_CAL_GAR_CAL_GAR_VAL_MASK)
/*! @} */

/* The count of ADC_CAL_GAR */
#define ADC_CAL_GAR_COUNT                        (33U)

/*! @name CAL_GBR - Calibration General B-Side Registers */
/*! @{ */

#define ADC_CAL_GBR_CAL_GBR_VAL_MASK             (0xFFFFU)
#define ADC_CAL_GBR_CAL_GBR_VAL_SHIFT            (0U)
/*! CAL_GBR_VAL - Calibration General B Side Register Element */
#define ADC_CAL_GBR_CAL_GBR_VAL(x)               (((uint32_t)(((uint32_t)(x)) << ADC_CAL_GBR_CAL_GBR_VAL_SHIFT)) & ADC_CAL_GBR_CAL_GBR_VAL_MASK)
/*! @} */

/* The count of ADC_CAL_GBR */
#define ADC_CAL_GBR_COUNT                        (33U)

/*! @name CFG2 - Configuration 2 Register */
/*! @{ */

#define ADC_CFG2_JLEFT_MASK                      (0x100U)
#define ADC_CFG2_JLEFT_SHIFT                     (8U)
/*! JLEFT - Justified Left Enable register
 *  0b0..For 12-bit single-ended conversions, RESFIFO data format is in standard format with data presented in bits RESFIFOa[D][14:3].
 *  0b1..For 12-bit single-ended conversions, RESFIFO data format is left-justified.
 */
#define ADC_CFG2_JLEFT(x)                        (((uint32_t)(((uint32_t)(x)) << ADC_CFG2_JLEFT_SHIFT)) & ADC_CFG2_JLEFT_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ADC_Register_Masks */


/* ADC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ADC1 base address */
  #define ADC1_BASE                                (0x52600000u)
  /** Peripheral ADC1 base address */
  #define ADC1_BASE_NS                             (0x42600000u)
  /** Peripheral ADC1 base pointer */
  #define ADC1                                     ((ADC_Type *)ADC1_BASE)
  /** Peripheral ADC1 base pointer */
  #define ADC1_NS                                  ((ADC_Type *)ADC1_BASE_NS)
  /** Array initializer of ADC peripheral base addresses */
  #define ADC_BASE_ADDRS                           { 0u, ADC1_BASE }
  /** Array initializer of ADC peripheral base pointers */
  #define ADC_BASE_PTRS                            { (ADC_Type *)0u, ADC1 }
  /** Array initializer of ADC peripheral base addresses */
  #define ADC_BASE_ADDRS_NS                        { 0u, ADC1_BASE_NS }
  /** Array initializer of ADC peripheral base pointers */
  #define ADC_BASE_PTRS_NS                         { (ADC_Type *)0u, ADC1_NS }
#else
  /** Peripheral ADC1 base address */
  #define ADC1_BASE                                (0x42600000u)
  /** Peripheral ADC1 base pointer */
  #define ADC1                                     ((ADC_Type *)ADC1_BASE)
  /** Array initializer of ADC peripheral base addresses */
  #define ADC_BASE_ADDRS                           { 0u, ADC1_BASE }
  /** Array initializer of ADC peripheral base pointers */
  #define ADC_BASE_PTRS                            { (ADC_Type *)0u, ADC1 }
#endif
/** Interrupt vectors for the ADC peripheral type */
#define ADC_IRQS                                 { NotAvail_IRQn, ADC1_IRQn }

/*!
 * @}
 */ /* end of group ADC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ANADIG Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_Peripheral_Access_Layer ANADIG Peripheral Access Layer
 * @{
 */

/** ANADIG - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[19456];
  __IO uint32_t SLOT_CTRL[35];                     /**< Slot Control Register, array offset: 0x4C00, array step: 0x4 */
} ANADIG_Type;

/* ----------------------------------------------------------------------------
   -- ANADIG Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_Register_Masks ANADIG Register Masks
 * @{
 */

/*! @name SLOT_CTRL - Slot Control Register */
/*! @{ */

#define ANADIG_SLOT_CTRL_LOCKED_DOMAIN_ID_MASK   (0xFU)
#define ANADIG_SLOT_CTRL_LOCKED_DOMAIN_ID_SHIFT  (0U)
/*! LOCKED_DOMAIN_ID - Domain ID of the slot to be locked */
#define ANADIG_SLOT_CTRL_LOCKED_DOMAIN_ID(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_SLOT_CTRL_LOCKED_DOMAIN_ID_SHIFT)) & ANADIG_SLOT_CTRL_LOCKED_DOMAIN_ID_MASK)

#define ANADIG_SLOT_CTRL_DOMAIN_LOCK_MASK        (0x8000U)
#define ANADIG_SLOT_CTRL_DOMAIN_LOCK_SHIFT       (15U)
/*! DOMAIN_LOCK - Lock domain ID of this slot
 *  0b0..Do not lock the domain ID
 *  0b1..Lock the domain ID
 */
#define ANADIG_SLOT_CTRL_DOMAIN_LOCK(x)          (((uint32_t)(((uint32_t)(x)) << ANADIG_SLOT_CTRL_DOMAIN_LOCK_SHIFT)) & ANADIG_SLOT_CTRL_DOMAIN_LOCK_MASK)

#define ANADIG_SLOT_CTRL_ALLOW_NONSECURE_MASK    (0x10000U)
#define ANADIG_SLOT_CTRL_ALLOW_NONSECURE_SHIFT   (16U)
/*! ALLOW_NONSECURE - Allow non-secure write access to this domain control register or domain register
 *  0b0..Do not allow non-secure write access
 *  0b1..Allow non-secure write access
 */
#define ANADIG_SLOT_CTRL_ALLOW_NONSECURE(x)      (((uint32_t)(((uint32_t)(x)) << ANADIG_SLOT_CTRL_ALLOW_NONSECURE_SHIFT)) & ANADIG_SLOT_CTRL_ALLOW_NONSECURE_MASK)

#define ANADIG_SLOT_CTRL_ALLOW_USER_MASK         (0x20000U)
#define ANADIG_SLOT_CTRL_ALLOW_USER_SHIFT        (17U)
/*! ALLOW_USER - Allow user write access to this domain control register or domain register
 *  0b0..Do not allow user write access
 *  0b1..Allow user write access
 */
#define ANADIG_SLOT_CTRL_ALLOW_USER(x)           (((uint32_t)(((uint32_t)(x)) << ANADIG_SLOT_CTRL_ALLOW_USER_SHIFT)) & ANADIG_SLOT_CTRL_ALLOW_USER_MASK)

#define ANADIG_SLOT_CTRL_LOCK_CONTROL_MASK       (0x80000000U)
#define ANADIG_SLOT_CTRL_LOCK_CONTROL_SHIFT      (31U)
/*! LOCK_CONTROL - Lock control of this slot
 *  0b0..Do not lock the control register of this slot
 *  0b1..Lock the control register of this slot
 */
#define ANADIG_SLOT_CTRL_LOCK_CONTROL(x)         (((uint32_t)(((uint32_t)(x)) << ANADIG_SLOT_CTRL_LOCK_CONTROL_SHIFT)) & ANADIG_SLOT_CTRL_LOCK_CONTROL_MASK)
/*! @} */

/* The count of ANADIG_SLOT_CTRL */
#define ANADIG_SLOT_CTRL_COUNT                   (35U)


/*!
 * @}
 */ /* end of group ANADIG_Register_Masks */


/* ANADIG - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ANADIG_SLOTS base address */
  #define ANADIG_SLOTS_BASE                        (0x54480000u)
  /** Peripheral ANADIG_SLOTS base address */
  #define ANADIG_SLOTS_BASE_NS                     (0x44480000u)
  /** Peripheral ANADIG_SLOTS base pointer */
  #define ANADIG_SLOTS                             ((ANADIG_Type *)ANADIG_SLOTS_BASE)
  /** Peripheral ANADIG_SLOTS base pointer */
  #define ANADIG_SLOTS_NS                          ((ANADIG_Type *)ANADIG_SLOTS_BASE_NS)
  /** Array initializer of ANADIG peripheral base addresses */
  #define ANADIG_BASE_ADDRS                        { ANADIG_SLOTS_BASE }
  /** Array initializer of ANADIG peripheral base pointers */
  #define ANADIG_BASE_PTRS                         { ANADIG_SLOTS }
  /** Array initializer of ANADIG peripheral base addresses */
  #define ANADIG_BASE_ADDRS_NS                     { ANADIG_SLOTS_BASE_NS }
  /** Array initializer of ANADIG peripheral base pointers */
  #define ANADIG_BASE_PTRS_NS                      { ANADIG_SLOTS_NS }
#else
  /** Peripheral ANADIG_SLOTS base address */
  #define ANADIG_SLOTS_BASE                        (0x44480000u)
  /** Peripheral ANADIG_SLOTS base pointer */
  #define ANADIG_SLOTS                             ((ANADIG_Type *)ANADIG_SLOTS_BASE)
  /** Array initializer of ANADIG peripheral base addresses */
  #define ANADIG_BASE_ADDRS                        { ANADIG_SLOTS_BASE }
  /** Array initializer of ANADIG peripheral base pointers */
  #define ANADIG_BASE_PTRS                         { ANADIG_SLOTS }
#endif

/*!
 * @}
 */ /* end of group ANADIG_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ANADIG_LDO_BBSM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_LDO_BBSM_Peripheral_Access_Layer ANADIG_LDO_BBSM Peripheral Access Layer
 * @{
 */

/** ANADIG_LDO_BBSM - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[18240];
  __IO uint32_t PMU_LDO_AON_ANA;                   /**< PMU_LDO_AON_ANA_REGISTER, offset: 0x4740 */
       uint8_t RESERVED_1[28];
  __IO uint32_t PMU_LDO_AON_DIG;                   /**< PMU_LDO_AON_DIG_REGISTER, offset: 0x4760 */
} ANADIG_LDO_BBSM_Type;

/* ----------------------------------------------------------------------------
   -- ANADIG_LDO_BBSM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_LDO_BBSM_Register_Masks ANADIG_LDO_BBSM Register Masks
 * @{
 */

/*! @name PMU_LDO_AON_ANA - PMU_LDO_AON_ANA_REGISTER */
/*! @{ */

#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_LP_EN_MASK (0x1U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_LP_EN_SHIFT (0U)
/*! REG_LP_EN - reg_lp_en
 *  0b1..Disable
 *  0b0..Enable
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_LP_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_LP_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_LP_EN_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_DISABLE_MASK (0x4U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_DISABLE_SHIFT (2U)
/*! REG_DISABLE - reg_disable
 *  0b1..Disable
 *  0b0..Enable
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_DISABLE_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_REG_DISABLE_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_2MA_EN_MASK (0x8U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_2MA_EN_SHIFT (3U)
/*! PULL_DOWN_2MA_EN - pull_down_2ma_en
 *  0b0..Disable
 *  0b1..Enable
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_2MA_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_2MA_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_2MA_EN_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_STANDBY_EN_MASK (0x40U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_STANDBY_EN_SHIFT (6U)
/*! STANDBY_EN - standby_en
 *  0b0..Standby mode disable
 *  0b1..Standby mode enable
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_STANDBY_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_STANDBY_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_STANDBY_EN_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_ALWAYS_4MA_PULLDOWN_EN_MASK (0x100U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_ALWAYS_4MA_PULLDOWN_EN_SHIFT (8U)
/*! ALWAYS_4MA_PULLDOWN_EN - always_4ma_pulldown_en */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_ALWAYS_4MA_PULLDOWN_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_ALWAYS_4MA_PULLDOWN_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_ALWAYS_4MA_PULLDOWN_EN_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_TRACK_MODE_EN_MASK (0x80000U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_TRACK_MODE_EN_SHIFT (19U)
/*! TRACK_MODE_EN - Track Mode Enable
 *  0b0..Normal use
 *  0b1..Switch preparation
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_TRACK_MODE_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_TRACK_MODE_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_TRACK_MODE_EN_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_20UA_EN_MASK (0x100000U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_20UA_EN_SHIFT (20U)
/*! PULL_DOWN_20UA_EN - pull_down_20ua_en
 *  0b0..Disable 20uA loading
 *  0b1..Enable 20uA loading
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_20UA_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_20UA_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_ANA_PULL_DOWN_20UA_EN_MASK)
/*! @} */

/*! @name PMU_LDO_AON_DIG - PMU_LDO_AON_DIG_REGISTER */
/*! @{ */

#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_REG_EN_MASK (0x4U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_REG_EN_SHIFT (2U)
/*! REG_EN - ENABLE_ILIMIT
 *  0b0..LDO_AON_DIG disable
 *  0b1..LDO_AON_DIG enable
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_REG_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_REG_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_REG_EN_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_STANDBY_EN_MASK (0x40U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_STANDBY_EN_SHIFT (6U)
/*! STANDBY_EN - standby_en
 *  0b0..Standby disable
 *  0b1..Standby enable
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_STANDBY_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_STANDBY_EN_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_STANDBY_EN_MASK)

#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_VOLTAGE_SELECT_MASK (0x1F00000U)
#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_VOLTAGE_SELECT_SHIFT (20U)
/*! VOLTAGE_SELECT - VOLTAGE_SELECT
 *  0b00000..Stable Voltage (range)
 *  0b00001..Stable Voltage (range)
 *  0b00010..Stable Voltage (range)
 *  0b00011..Stable Voltage (range)
 *  0b00100..Stable Voltage (range)
 *  0b00101..Stable Voltage (range)
 *  0b00110..Stable Voltage (range)
 *  0b00111..Stable Voltage (range)
 *  0b01000..Stable Voltage (range)
 *  0b01001..Stable Voltage (range)
 *  0b01010..Stable Voltage (range)
 *  0b01011..Stable Voltage (range)
 *  0b01100..Stable Voltage (range)
 *  0b01101..Stable Voltage (range)
 *  0b01110..Stable Voltage (range)
 *  0b01111..Stable Voltage (range)
 *  0b10000..Stable Voltage (range)
 *  0b10001..Stable Voltage (range)
 *  0b10010..Stable Voltage (range)
 *  0b10011..Stable Voltage (range)
 *  0b10100..Stable Voltage (range)
 *  0b10101..Stable Voltage (range)
 *  0b10110..Stable Voltage (range)
 *  0b10111..Stable Voltage (range)
 *  0b11000..Stable Voltage (range)
 *  0b11001..Stable Voltage (range)
 *  0b11010..Stable Voltage (range)
 *  0b11011..Stable Voltage (range)
 *  0b11100..Stable Voltage (range)
 *  0b11101..Stable Voltage (range)
 *  0b11110..Stable Voltage (range)
 *  0b11111..Stable Voltage (range)
 */
#define ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_VOLTAGE_SELECT(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_VOLTAGE_SELECT_SHIFT)) & ANADIG_LDO_BBSM_PMU_LDO_AON_DIG_VOLTAGE_SELECT_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ANADIG_LDO_BBSM_Register_Masks */


/* ANADIG_LDO_BBSM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ANADIG_LDO_BBSM base address */
  #define ANADIG_LDO_BBSM_BASE                     (0x54480000u)
  /** Peripheral ANADIG_LDO_BBSM base address */
  #define ANADIG_LDO_BBSM_BASE_NS                  (0x44480000u)
  /** Peripheral ANADIG_LDO_BBSM base pointer */
  #define ANADIG_LDO_BBSM                          ((ANADIG_LDO_BBSM_Type *)ANADIG_LDO_BBSM_BASE)
  /** Peripheral ANADIG_LDO_BBSM base pointer */
  #define ANADIG_LDO_BBSM_NS                       ((ANADIG_LDO_BBSM_Type *)ANADIG_LDO_BBSM_BASE_NS)
  /** Array initializer of ANADIG_LDO_BBSM peripheral base addresses */
  #define ANADIG_LDO_BBSM_BASE_ADDRS               { ANADIG_LDO_BBSM_BASE }
  /** Array initializer of ANADIG_LDO_BBSM peripheral base pointers */
  #define ANADIG_LDO_BBSM_BASE_PTRS                { ANADIG_LDO_BBSM }
  /** Array initializer of ANADIG_LDO_BBSM peripheral base addresses */
  #define ANADIG_LDO_BBSM_BASE_ADDRS_NS            { ANADIG_LDO_BBSM_BASE_NS }
  /** Array initializer of ANADIG_LDO_BBSM peripheral base pointers */
  #define ANADIG_LDO_BBSM_BASE_PTRS_NS             { ANADIG_LDO_BBSM_NS }
#else
  /** Peripheral ANADIG_LDO_BBSM base address */
  #define ANADIG_LDO_BBSM_BASE                     (0x44480000u)
  /** Peripheral ANADIG_LDO_BBSM base pointer */
  #define ANADIG_LDO_BBSM                          ((ANADIG_LDO_BBSM_Type *)ANADIG_LDO_BBSM_BASE)
  /** Array initializer of ANADIG_LDO_BBSM peripheral base addresses */
  #define ANADIG_LDO_BBSM_BASE_ADDRS               { ANADIG_LDO_BBSM_BASE }
  /** Array initializer of ANADIG_LDO_BBSM peripheral base pointers */
  #define ANADIG_LDO_BBSM_BASE_PTRS                { ANADIG_LDO_BBSM }
#endif

/*!
 * @}
 */ /* end of group ANADIG_LDO_BBSM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ANADIG_MISC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_MISC_Peripheral_Access_Layer ANADIG_MISC Peripheral Access Layer
 * @{
 */

/** ANADIG_MISC - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[18432];
  __I  uint32_t MISC_DIFPROG;                      /**< Chip Silicon Version Register, offset: 0x4800 */
} ANADIG_MISC_Type;

/* ----------------------------------------------------------------------------
   -- ANADIG_MISC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_MISC_Register_Masks ANADIG_MISC Register Masks
 * @{
 */

/*! @name MISC_DIFPROG - Chip Silicon Version Register */
/*! @{ */

#define ANADIG_MISC_MISC_DIFPROG_CHIPID_MASK     (0xFFFFFFFFU)
#define ANADIG_MISC_MISC_DIFPROG_CHIPID_SHIFT    (0U)
/*! CHIPID - Chip ID */
#define ANADIG_MISC_MISC_DIFPROG_CHIPID(x)       (((uint32_t)(((uint32_t)(x)) << ANADIG_MISC_MISC_DIFPROG_CHIPID_SHIFT)) & ANADIG_MISC_MISC_DIFPROG_CHIPID_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ANADIG_MISC_Register_Masks */


/* ANADIG_MISC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ANADIG_MISC base address */
  #define ANADIG_MISC_BASE                         (0x54480000u)
  /** Peripheral ANADIG_MISC base address */
  #define ANADIG_MISC_BASE_NS                      (0x44480000u)
  /** Peripheral ANADIG_MISC base pointer */
  #define ANADIG_MISC                              ((ANADIG_MISC_Type *)ANADIG_MISC_BASE)
  /** Peripheral ANADIG_MISC base pointer */
  #define ANADIG_MISC_NS                           ((ANADIG_MISC_Type *)ANADIG_MISC_BASE_NS)
  /** Array initializer of ANADIG_MISC peripheral base addresses */
  #define ANADIG_MISC_BASE_ADDRS                   { ANADIG_MISC_BASE }
  /** Array initializer of ANADIG_MISC peripheral base pointers */
  #define ANADIG_MISC_BASE_PTRS                    { ANADIG_MISC }
  /** Array initializer of ANADIG_MISC peripheral base addresses */
  #define ANADIG_MISC_BASE_ADDRS_NS                { ANADIG_MISC_BASE_NS }
  /** Array initializer of ANADIG_MISC peripheral base pointers */
  #define ANADIG_MISC_BASE_PTRS_NS                 { ANADIG_MISC_NS }
#else
  /** Peripheral ANADIG_MISC base address */
  #define ANADIG_MISC_BASE                         (0x44480000u)
  /** Peripheral ANADIG_MISC base pointer */
  #define ANADIG_MISC                              ((ANADIG_MISC_Type *)ANADIG_MISC_BASE)
  /** Array initializer of ANADIG_MISC peripheral base addresses */
  #define ANADIG_MISC_BASE_ADDRS                   { ANADIG_MISC_BASE }
  /** Array initializer of ANADIG_MISC peripheral base pointers */
  #define ANADIG_MISC_BASE_PTRS                    { ANADIG_MISC }
#endif

/*!
 * @}
 */ /* end of group ANADIG_MISC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ANADIG_OSC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_OSC_Peripheral_Access_Layer ANADIG_OSC Peripheral Access Layer
 * @{
 */

/** ANADIG_OSC - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[17168];
  __IO uint32_t OSC_RC24M_CTRL;                    /**< 24MHz RCOSC Control Register, offset: 0x4310 */
       uint8_t RESERVED_1[12];
  __IO uint32_t OSC_24M_CTRL;                      /**< 24MHz OSC Control Register, offset: 0x4320 */
       uint8_t RESERVED_2[28];
  __I  uint32_t OSC_400M_CTRL0;                    /**< 400MHz RCOSC Control0 Register, offset: 0x4340 */
       uint8_t RESERVED_3[12];
  __IO uint32_t OSC_400M_CTRL1;                    /**< 400MHz RCOSC Control1 Register, offset: 0x4350 */
} ANADIG_OSC_Type;

/* ----------------------------------------------------------------------------
   -- ANADIG_OSC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_OSC_Register_Masks ANADIG_OSC Register Masks
 * @{
 */

/*! @name OSC_RC24M_CTRL - 24MHz RCOSC Control Register */
/*! @{ */

#define ANADIG_OSC_OSC_RC24M_CTRL_TEN_MASK       (0x2U)
#define ANADIG_OSC_OSC_RC24M_CTRL_TEN_SHIFT      (1U)
/*! TEN - RC24M Enable
 *  0b0..Power down
 *  0b1..Power up
 */
#define ANADIG_OSC_OSC_RC24M_CTRL_TEN(x)         (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_RC24M_CTRL_TEN_SHIFT)) & ANADIG_OSC_OSC_RC24M_CTRL_TEN_MASK)

#define ANADIG_OSC_OSC_RC24M_CTRL_SOURCE_SEL_24M_MASK (0x2000000U)
#define ANADIG_OSC_OSC_RC24M_CTRL_SOURCE_SEL_24M_SHIFT (25U)
/*! SOURCE_SEL_24M - source_sel_24M
 *  0b0..OSC_RC24M
 *  0b1..OSC24M
 */
#define ANADIG_OSC_OSC_RC24M_CTRL_SOURCE_SEL_24M(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_RC24M_CTRL_SOURCE_SEL_24M_SHIFT)) & ANADIG_OSC_OSC_RC24M_CTRL_SOURCE_SEL_24M_MASK)

#define ANADIG_OSC_OSC_RC24M_CTRL_RC_24M_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_OSC_OSC_RC24M_CTRL_RC_24M_CONTROL_MODE_SHIFT (31U)
/*! RC_24M_CONTROL_MODE - RCOSC Control Mode
 *  0b0..Software mode (default)
 *  0b1..GPC mode
 */
#define ANADIG_OSC_OSC_RC24M_CTRL_RC_24M_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_RC24M_CTRL_RC_24M_CONTROL_MODE_SHIFT)) & ANADIG_OSC_OSC_RC24M_CTRL_RC_24M_CONTROL_MODE_MASK)
/*! @} */

/*! @name OSC_24M_CTRL - 24MHz OSC Control Register */
/*! @{ */

#define ANADIG_OSC_OSC_24M_CTRL_BYPASS_EN_MASK   (0x2U)
#define ANADIG_OSC_OSC_24M_CTRL_BYPASS_EN_SHIFT  (1U)
/*! BYPASS_EN - 24MHz OSC Bypass Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define ANADIG_OSC_OSC_24M_CTRL_BYPASS_EN(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_24M_CTRL_BYPASS_EN_SHIFT)) & ANADIG_OSC_OSC_24M_CTRL_BYPASS_EN_MASK)

#define ANADIG_OSC_OSC_24M_CTRL_LP_EN_MASK       (0x4U)
#define ANADIG_OSC_OSC_24M_CTRL_LP_EN_SHIFT      (2U)
/*! LP_EN - 24MHz OSC Low-Power Mode Enable
 *  0b0..High Gain mode (HP)
 *  0b1..Low-power mode (LP)
 */
#define ANADIG_OSC_OSC_24M_CTRL_LP_EN(x)         (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_24M_CTRL_LP_EN_SHIFT)) & ANADIG_OSC_OSC_24M_CTRL_LP_EN_MASK)

#define ANADIG_OSC_OSC_24M_CTRL_OSC_COMP_MODE_MASK (0x8U)
#define ANADIG_OSC_OSC_24M_CTRL_OSC_COMP_MODE_SHIFT (3U)
/*! OSC_COMP_MODE - 24MHz OSC Comparator Mode
 *  0b0..Single-ended mode (default)
 *  0b1..Differential mode (test mode)
 */
#define ANADIG_OSC_OSC_24M_CTRL_OSC_COMP_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_24M_CTRL_OSC_COMP_MODE_SHIFT)) & ANADIG_OSC_OSC_24M_CTRL_OSC_COMP_MODE_MASK)

#define ANADIG_OSC_OSC_24M_CTRL_OSC_EN_MASK      (0x10U)
#define ANADIG_OSC_OSC_24M_CTRL_OSC_EN_SHIFT     (4U)
/*! OSC_EN - 24MHz OSC Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define ANADIG_OSC_OSC_24M_CTRL_OSC_EN(x)        (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_24M_CTRL_OSC_EN_SHIFT)) & ANADIG_OSC_OSC_24M_CTRL_OSC_EN_MASK)

#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_GATE_MASK (0x80U)
#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_GATE_SHIFT (7U)
/*! OSC_24M_GATE - 24MHz OSC Gate Control
 *  0b0..Not Gated
 *  0b1..Gated
 */
#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_GATE(x)  (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_24M_CTRL_OSC_24M_GATE_SHIFT)) & ANADIG_OSC_OSC_24M_CTRL_OSC_24M_GATE_MASK)

#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_STABLE_MASK (0x40000000U)
#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_STABLE_SHIFT (30U)
/*! OSC_24M_STABLE - 24MHz OSC Stable
 *  0b0..Not Stable
 *  0b1..Stable
 */
#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_STABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_24M_CTRL_OSC_24M_STABLE_SHIFT)) & ANADIG_OSC_OSC_24M_CTRL_OSC_24M_STABLE_MASK)

#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_CONTROL_MODE_SHIFT (31U)
/*! OSC_24M_CONTROL_MODE - 24MHz OSC Control Mode
 *  0b0..Software mode (default)
 *  0b1..GPC mode
 */
#define ANADIG_OSC_OSC_24M_CTRL_OSC_24M_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_24M_CTRL_OSC_24M_CONTROL_MODE_SHIFT)) & ANADIG_OSC_OSC_24M_CTRL_OSC_24M_CONTROL_MODE_MASK)
/*! @} */

/*! @name OSC_400M_CTRL0 - 400MHz RCOSC Control0 Register */
/*! @{ */

#define ANADIG_OSC_OSC_400M_CTRL0_OSC400M_AI_BUSY_MASK (0x80000000U)
#define ANADIG_OSC_OSC_400M_CTRL0_OSC400M_AI_BUSY_SHIFT (31U)
/*! OSC400M_AI_BUSY - 400MHz OSC AI BUSY */
#define ANADIG_OSC_OSC_400M_CTRL0_OSC400M_AI_BUSY(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_400M_CTRL0_OSC400M_AI_BUSY_SHIFT)) & ANADIG_OSC_OSC_400M_CTRL0_OSC400M_AI_BUSY_MASK)
/*! @} */

/*! @name OSC_400M_CTRL1 - 400MHz RCOSC Control1 Register */
/*! @{ */

#define ANADIG_OSC_OSC_400M_CTRL1_PWD_MASK       (0x1U)
#define ANADIG_OSC_OSC_400M_CTRL1_PWD_SHIFT      (0U)
/*! PWD - Power down control for 400MHz RCOSC
 *  0b0..No Power down
 *  0b1..Power down
 */
#define ANADIG_OSC_OSC_400M_CTRL1_PWD(x)         (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_400M_CTRL1_PWD_SHIFT)) & ANADIG_OSC_OSC_400M_CTRL1_PWD_MASK)

#define ANADIG_OSC_OSC_400M_CTRL1_CLKGATE_400MEG_MASK (0x2U)
#define ANADIG_OSC_OSC_400M_CTRL1_CLKGATE_400MEG_SHIFT (1U)
/*! CLKGATE_400MEG - Clock gate control for 400MHz RCOSC
 *  0b0..Not Gated
 *  0b1..Gated
 */
#define ANADIG_OSC_OSC_400M_CTRL1_CLKGATE_400MEG(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_400M_CTRL1_CLKGATE_400MEG_SHIFT)) & ANADIG_OSC_OSC_400M_CTRL1_CLKGATE_400MEG_MASK)

#define ANADIG_OSC_OSC_400M_CTRL1_RC_400M_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_OSC_OSC_400M_CTRL1_RC_400M_CONTROL_MODE_SHIFT (31U)
/*! RC_400M_CONTROL_MODE - 400MHz RCOSC Control mode
 *  0b0..Software mode (default)
 *  0b1..GPC mode
 */
#define ANADIG_OSC_OSC_400M_CTRL1_RC_400M_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_OSC_OSC_400M_CTRL1_RC_400M_CONTROL_MODE_SHIFT)) & ANADIG_OSC_OSC_400M_CTRL1_RC_400M_CONTROL_MODE_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ANADIG_OSC_Register_Masks */


/* ANADIG_OSC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ANADIG_OSC base address */
  #define ANADIG_OSC_BASE                          (0x54480000u)
  /** Peripheral ANADIG_OSC base address */
  #define ANADIG_OSC_BASE_NS                       (0x44480000u)
  /** Peripheral ANADIG_OSC base pointer */
  #define ANADIG_OSC                               ((ANADIG_OSC_Type *)ANADIG_OSC_BASE)
  /** Peripheral ANADIG_OSC base pointer */
  #define ANADIG_OSC_NS                            ((ANADIG_OSC_Type *)ANADIG_OSC_BASE_NS)
  /** Array initializer of ANADIG_OSC peripheral base addresses */
  #define ANADIG_OSC_BASE_ADDRS                    { ANADIG_OSC_BASE }
  /** Array initializer of ANADIG_OSC peripheral base pointers */
  #define ANADIG_OSC_BASE_PTRS                     { ANADIG_OSC }
  /** Array initializer of ANADIG_OSC peripheral base addresses */
  #define ANADIG_OSC_BASE_ADDRS_NS                 { ANADIG_OSC_BASE_NS }
  /** Array initializer of ANADIG_OSC peripheral base pointers */
  #define ANADIG_OSC_BASE_PTRS_NS                  { ANADIG_OSC_NS }
#else
  /** Peripheral ANADIG_OSC base address */
  #define ANADIG_OSC_BASE                          (0x44480000u)
  /** Peripheral ANADIG_OSC base pointer */
  #define ANADIG_OSC                               ((ANADIG_OSC_Type *)ANADIG_OSC_BASE)
  /** Array initializer of ANADIG_OSC peripheral base addresses */
  #define ANADIG_OSC_BASE_ADDRS                    { ANADIG_OSC_BASE }
  /** Array initializer of ANADIG_OSC peripheral base pointers */
  #define ANADIG_OSC_BASE_PTRS                     { ANADIG_OSC }
#endif

/*!
 * @}
 */ /* end of group ANADIG_OSC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ANADIG_PLL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_PLL_Peripheral_Access_Layer ANADIG_PLL Peripheral Access Layer
 * @{
 */

/** ANADIG_PLL - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[16384];
  __IO uint32_t ARM_PLL_CTRL;                      /**< ARM_PLL_CTRL_REGISTER, offset: 0x4000 */
       uint8_t RESERVED_1[12];
  __IO uint32_t SYS_PLL3_CTRL;                     /**< SYS_PLL3_CTRL_REGISTER, offset: 0x4010 */
       uint8_t RESERVED_2[12];
  __IO uint32_t SYS_PLL3_UPDATE;                   /**< SYS_PLL3_UPDATE_REGISTER, offset: 0x4020 */
       uint8_t RESERVED_3[12];
  __IO uint32_t SYS_PLL3_PFD;                      /**< SYS_PLL3_PFD_REGISTER, offset: 0x4030 */
       uint8_t RESERVED_4[12];
  __IO uint32_t SYS_PLL2_CTRL;                     /**< SYS_PLL2_CTRL_REGISTER, offset: 0x4040 */
       uint8_t RESERVED_5[12];
  __IO uint32_t SYS_PLL2_UPDATE;                   /**< SYS_PLL2_UPDATE_REGISTER, offset: 0x4050 */
       uint8_t RESERVED_6[12];
  __IO uint32_t SYS_PLL2_SS;                       /**< SYS_PLL2_SS_REGISTER, offset: 0x4060 */
       uint8_t RESERVED_7[12];
  __IO uint32_t SYS_PLL2_PFD;                      /**< SYS_PLL2_PFD_REGISTER, offset: 0x4070 */
       uint8_t RESERVED_8[12];
  __IO uint32_t SYS_PLL2_MFN;                      /**< SYS_PLL2_MFN_REGISTER, offset: 0x4080 */
       uint8_t RESERVED_9[12];
  __IO uint32_t SYS_PLL2_MFI;                      /**< SYS_PLL2_MFI_REGISTER, offset: 0x4090 */
       uint8_t RESERVED_10[12];
  __IO uint32_t SYS_PLL2_MFD;                      /**< SYS_PLL2_MFD_REGISTER, offset: 0x40A0 */
       uint8_t RESERVED_11[92];
  __IO uint32_t SYS_PLL1_CTRL;                     /**< SYS_PLL1_CTRL_REGISTER, offset: 0x4100 */
       uint8_t RESERVED_12[252];
  __IO uint32_t PLL_AUDIO_CTRL;                    /**< PLL_AUDIO_CTRL_REGISTER, offset: 0x4200 */
} ANADIG_PLL_Type;

/* ----------------------------------------------------------------------------
   -- ANADIG_PLL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_PLL_Register_Masks ANADIG_PLL Register Masks
 * @{
 */

/*! @name ARM_PLL_CTRL - ARM_PLL_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PLL_ARM_PLL_CTRL_DIV_SELECT_MASK  (0xFFU)
#define ANADIG_PLL_ARM_PLL_CTRL_DIV_SELECT_SHIFT (0U)
/*! DIV_SELECT - DIV_SELECT */
#define ANADIG_PLL_ARM_PLL_CTRL_DIV_SELECT(x)    (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_DIV_SELECT_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_DIV_SELECT_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_HOLD_RING_OFF_MASK (0x1000U)
#define ANADIG_PLL_ARM_PLL_CTRL_HOLD_RING_OFF_SHIFT (12U)
/*! HOLD_RING_OFF - PLL Start up initialization
 *  0b0..Normal operation
 *  0b1..Initialize PLL start up
 */
#define ANADIG_PLL_ARM_PLL_CTRL_HOLD_RING_OFF(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_HOLD_RING_OFF_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_HOLD_RING_OFF_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_POWERUP_MASK     (0x2000U)
#define ANADIG_PLL_ARM_PLL_CTRL_POWERUP_SHIFT    (13U)
/*! POWERUP - Power up the PLL
 *  0b1..Power Up the PLL
 *  0b0..Power down the PLL
 */
#define ANADIG_PLL_ARM_PLL_CTRL_POWERUP(x)       (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_POWERUP_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_POWERUP_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_ENABLE_CLK_MASK  (0x4000U)
#define ANADIG_PLL_ARM_PLL_CTRL_ENABLE_CLK_SHIFT (14U)
/*! ENABLE_CLK - Enable the clock output.
 *  0b0..Disable the clock
 *  0b1..Enable the clock
 */
#define ANADIG_PLL_ARM_PLL_CTRL_ENABLE_CLK(x)    (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_ENABLE_CLK_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_ENABLE_CLK_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_POST_DIV_SEL_MASK (0x18000U)
#define ANADIG_PLL_ARM_PLL_CTRL_POST_DIV_SEL_SHIFT (15U)
/*! POST_DIV_SEL - POST_DIV_SEL
 *  0b00..post_div=2
 *  0b01..post_div=4
 *  0b10..post_div=8
 *  0b11..post_div=1
 */
#define ANADIG_PLL_ARM_PLL_CTRL_POST_DIV_SEL(x)  (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_POST_DIV_SEL_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_POST_DIV_SEL_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_BYPASS_MASK      (0x20000U)
#define ANADIG_PLL_ARM_PLL_CTRL_BYPASS_SHIFT     (17U)
/*! BYPASS - Bypass the pll.
 *  0b1..Bypass Mode
 *  0b0..Function mode
 */
#define ANADIG_PLL_ARM_PLL_CTRL_BYPASS(x)        (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_BYPASS_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_BYPASS_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_STABLE_MASK (0x20000000U)
#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_STABLE_SHIFT (29U)
/*! ARM_PLL_STABLE - ARM_PLL_STABLE
 *  0b1..ARM PLL is stable
 *  0b0..ARM PLL is not stable
 */
#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_STABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_STABLE_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_STABLE_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_GATE_MASK (0x40000000U)
#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_GATE_SHIFT (30U)
/*! ARM_PLL_GATE - ARM_PLL_GATE
 *  0b1..Clock is gated
 *  0b0..Clock is not gated
 */
#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_GATE(x)  (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_GATE_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_GATE_MASK)

#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_CONTROL_MODE_SHIFT (31U)
/*! ARM_PLL_CONTROL_MODE - pll_arm_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_CONTROL_MODE_SHIFT)) & ANADIG_PLL_ARM_PLL_CTRL_ARM_PLL_CONTROL_MODE_MASK)
/*! @} */

/*! @name SYS_PLL3_CTRL - SYS_PLL3_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL3_CTRL_DIV_SELECT_MASK (0x7U)
#define ANADIG_PLL_SYS_PLL3_CTRL_DIV_SELECT_SHIFT (0U)
/*! DIV_SELECT - DIV_SELECT
 *  0b000..div_select=130x1: div_select=150x2
 *  0b001..div_select=160x3: div_select=200x4
 *  0b010..div_select=220x5: div_select=250x6
 *  0b011..div_select=300x7: div_select=240
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_DIV_SELECT(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_DIV_SELECT_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_DIV_SELECT_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_MASK (0x8U)
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_SHIFT (3U)
/*! SYS_PLL3_DIV2 - SYS PLL3 DIV2 gate */
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_PLL_REG_EN_MASK (0x10U)
#define ANADIG_PLL_SYS_PLL3_CTRL_PLL_REG_EN_SHIFT (4U)
/*! PLL_REG_EN - Enable Internal PLL Regulator */
#define ANADIG_PLL_SYS_PLL3_CTRL_PLL_REG_EN(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_PLL_REG_EN_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_PLL_REG_EN_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_HOLD_RING_OFF_MASK (0x800U)
#define ANADIG_PLL_SYS_PLL3_CTRL_HOLD_RING_OFF_SHIFT (11U)
/*! HOLD_RING_OFF - PLL Start up initialization
 *  0b0..Normal operation
 *  0b1..Initialize PLL start up
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_HOLD_RING_OFF(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_HOLD_RING_OFF_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_HOLD_RING_OFF_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_ENABLE_CLK_MASK (0x2000U)
#define ANADIG_PLL_SYS_PLL3_CTRL_ENABLE_CLK_SHIFT (13U)
/*! ENABLE_CLK - Enable the clock output.
 *  0b0..Disable the clock
 *  0b1..Enable the clock
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_ENABLE_CLK(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_ENABLE_CLK_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_ENABLE_CLK_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_BYPASS_MASK     (0x10000U)
#define ANADIG_PLL_SYS_PLL3_CTRL_BYPASS_SHIFT    (16U)
/*! BYPASS - BYPASS
 *  0b1..Bypass Mode
 *  0b0..Function mode
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_BYPASS(x)       (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_BYPASS_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_BYPASS_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_POWERUP_MASK    (0x200000U)
#define ANADIG_PLL_SYS_PLL3_CTRL_POWERUP_SHIFT   (21U)
/*! POWERUP - Power up the PLL
 *  0b1..Power Up the PLL
 *  0b0..Power down the PLL
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_POWERUP(x)      (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_POWERUP_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_POWERUP_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_CONTROL_MODE_MASK (0x10000000U)
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_CONTROL_MODE_SHIFT (28U)
/*! SYS_PLL3_DIV2_CONTROL_MODE - SYS_PLL3_DIV2_CONTROL_MODE
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_DIV2_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_STABLE_MASK (0x20000000U)
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_STABLE_SHIFT (29U)
/*! SYS_PLL3_STABLE - SYS_PLL3_STABLE
 *  0b0..Not Stable
 *  0b1..Stable
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_STABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_GATE_MASK (0x40000000U)
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_GATE_SHIFT (30U)
/*! SYS_PLL3_GATE - SYS_PLL3_GATE
 *  0b1..Clock is gated
 *  0b0..Clock is not gated
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_GATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_GATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_GATE_MASK)

#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_CONTROL_MODE_SHIFT (31U)
/*! SYS_PLL3_CONTROL_MODE - SYS_PLL3_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL3_CTRL_SYS_PLL3_CONTROL_MODE_MASK)
/*! @} */

/*! @name SYS_PLL3_UPDATE - SYS_PLL3_UPDATE_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_UPDATE_MASK (0x2U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_UPDATE_SHIFT (1U)
/*! PFD0_UPDATE - PFD0_OVERRIDE */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_UPDATE_MASK (0x4U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_UPDATE_SHIFT (2U)
/*! PFD1_UPDATE - PFD1_OVERRIDE */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_UPDATE_MASK (0x8U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_UPDATE_SHIFT (3U)
/*! PFD2_UPDATE - PFD2_OVERRIDE */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_UPDATE_MASK (0x10U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_UPDATE_SHIFT (4U)
/*! PFD3_UPDATE - PFD3_UPDATE */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_CONTROL_MODE_MASK (0x20U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_CONTROL_MODE_SHIFT (5U)
/*! PFD0_CONTROL_MODE - pfd0_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD0_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_CONTROL_MODE_MASK (0x40U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_CONTROL_MODE_SHIFT (6U)
/*! PFD1_CONTROL_MODE - pfd1_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD1_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_CONTROL_MODE_MASK (0x80U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_CONTROL_MODE_SHIFT (7U)
/*! PFD2_CONTROL_MODE - pdf2_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD2_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_CONTROL_MODE_MASK (0x100U)
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_CONTROL_MODE_SHIFT (8U)
/*! PFD3_CONTROL_MODE - pfd3_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL3_UPDATE_PFD3_CONTROL_MODE_MASK)
/*! @} */

/*! @name SYS_PLL3_PFD - SYS_PLL3_PFD_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_FRAC_MASK   (0x3FU)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_FRAC_SHIFT  (0U)
/*! PFD0_FRAC - PFD0_FRAC */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD0_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD0_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_STABLE_MASK (0x40U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_STABLE_SHIFT (6U)
/*! PFD0_STABLE - PFD0_STABLE */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD0_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD0_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_DIV1_CLKGATE_MASK (0x80U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_DIV1_CLKGATE_SHIFT (7U)
/*! PFD0_DIV1_CLKGATE - PFD0_CLKGATE
 *  0b1..Fractional divider clock (reference ref_pfd0) is off (power savings
 *  0b0..PFD0 fractional divider clock is enabled
 */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD0_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD0_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD0_DIV1_CLKGATE_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_FRAC_MASK   (0x3F00U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_FRAC_SHIFT  (8U)
/*! PFD1_FRAC - PFD1_FRAC */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD1_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD1_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_STABLE_MASK (0x4000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_STABLE_SHIFT (14U)
/*! PFD1_STABLE - PFD1_STABLE */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD1_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD1_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_DIV1_CLKGATE_MASK (0x8000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_DIV1_CLKGATE_SHIFT (15U)
/*! PFD1_DIV1_CLKGATE - PFD1_CLKGATE
 *  0b1..Fractional divider clock (reference PFD1) is off (power savings)
 *  0b0..PFD1 fractional divider clock is enabled
 */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD1_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD1_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD1_DIV1_CLKGATE_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_FRAC_MASK   (0x3F0000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_FRAC_SHIFT  (16U)
/*! PFD2_FRAC - PFD2_FRAC */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD2_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD2_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_STABLE_MASK (0x400000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_STABLE_SHIFT (22U)
/*! PFD2_STABLE - PFD2_STABLE */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD2_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD2_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_DIV1_CLKGATE_MASK (0x800000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_DIV1_CLKGATE_SHIFT (23U)
/*! PFD2_DIV1_CLKGATE - PFD2_CLKGATE
 *  0b1..Fractional divider clock (reference PFD2) is off (power savings)
 *  0b0..PFD2 fractional divider clock is enabled
 */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD2_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD2_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD2_DIV1_CLKGATE_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_FRAC_MASK   (0x3F000000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_FRAC_SHIFT  (24U)
/*! PFD3_FRAC - PFD3_FRAC */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD3_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD3_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_STABLE_MASK (0x40000000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_STABLE_SHIFT (30U)
/*! PFD3_STABLE - PFD3_STABLE */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD3_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD3_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_DIV1_CLKGATE_MASK (0x80000000U)
#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_DIV1_CLKGATE_SHIFT (31U)
/*! PFD3_DIV1_CLKGATE - PFD3_CLKGATE
 *  0b1..Fractional divider clock (reference PFD3) is off (power savings)
 *  0b0..PFD3 fractional divider clock is enabled
 */
#define ANADIG_PLL_SYS_PLL3_PFD_PFD3_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL3_PFD_PFD3_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL3_PFD_PFD3_DIV1_CLKGATE_MASK)
/*! @} */

/*! @name SYS_PLL2_CTRL - SYS_PLL2_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL2_CTRL_PLL_REG_EN_MASK (0x8U)
#define ANADIG_PLL_SYS_PLL2_CTRL_PLL_REG_EN_SHIFT (3U)
/*! PLL_REG_EN - Enable Internal PLL Regulator
 *  0b0..Disable
 *  0b1..Enable
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_PLL_REG_EN(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_PLL_REG_EN_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_PLL_REG_EN_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_HOLD_RING_OFF_MASK (0x800U)
#define ANADIG_PLL_SYS_PLL2_CTRL_HOLD_RING_OFF_SHIFT (11U)
/*! HOLD_RING_OFF - PLL Start up initialization
 *  0b0..Normal operation
 *  0b1..Initialize PLL start up
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_HOLD_RING_OFF(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_HOLD_RING_OFF_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_HOLD_RING_OFF_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_ENABLE_CLK_MASK (0x2000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_ENABLE_CLK_SHIFT (13U)
/*! ENABLE_CLK - Enable the clock output.
 *  0b0..Disable the clock
 *  0b1..Enable the clock
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_ENABLE_CLK(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_ENABLE_CLK_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_ENABLE_CLK_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_BYPASS_MASK     (0x10000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_BYPASS_SHIFT    (16U)
/*! BYPASS - Bypass the pll.
 *  0b1..Bypass Mode
 *  0b0..Function mode
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_BYPASS(x)       (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_BYPASS_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_BYPASS_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_DITHER_ENABLE_MASK (0x20000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_DITHER_ENABLE_SHIFT (17U)
/*! DITHER_ENABLE - DITHER_ENABLE
 *  0b0..Disable Dither
 *  0b1..Enable Dither
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_DITHER_ENABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_DITHER_ENABLE_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_DITHER_ENABLE_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_PFD_OFFSET_EN_MASK (0x40000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_PFD_OFFSET_EN_SHIFT (18U)
/*! PFD_OFFSET_EN - PFD_OFFSET_EN */
#define ANADIG_PLL_SYS_PLL2_CTRL_PFD_OFFSET_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_PFD_OFFSET_EN_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_PFD_OFFSET_EN_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_PLL_DDR_OVERRIDE_MASK (0x80000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_PLL_DDR_OVERRIDE_SHIFT (19U)
/*! PLL_DDR_OVERRIDE - PLL_DDR_OVERRIDE */
#define ANADIG_PLL_SYS_PLL2_CTRL_PLL_DDR_OVERRIDE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_PLL_DDR_OVERRIDE_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_PLL_DDR_OVERRIDE_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_POWERUP_MASK    (0x800000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_POWERUP_SHIFT   (23U)
/*! POWERUP - Power up the PLL
 *  0b1..Power Up the PLL
 *  0b0..Power down the PLL
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_POWERUP(x)      (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_POWERUP_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_POWERUP_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_STABLE_MASK (0x20000000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_STABLE_SHIFT (29U)
/*! SYS_PLL2_STABLE - SYS_PLL2_STABLE */
#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_STABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_GATE_MASK (0x40000000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_GATE_SHIFT (30U)
/*! SYS_PLL2_GATE - SYS_PLL2_GATE
 *  0b1..Clock is gated
 *  0b0..Clock is not gated
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_GATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_GATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_GATE_MASK)

#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_CONTROL_MODE_SHIFT (31U)
/*! SYS_PLL2_CONTROL_MODE - SYS_PLL2_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL2_CTRL_SYS_PLL2_CONTROL_MODE_MASK)
/*! @} */

/*! @name SYS_PLL2_UPDATE - SYS_PLL2_UPDATE_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_UPDATE_MASK (0x2U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_UPDATE_SHIFT (1U)
/*! PFD0_UPDATE - PFD0_UPDATE */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_UPDATE_MASK (0x4U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_UPDATE_SHIFT (2U)
/*! PFD1_UPDATE - PFD1_UPDATE */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_UPDATE_MASK (0x8U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_UPDATE_SHIFT (3U)
/*! PFD2_UPDATE - PFD2_UPDATE */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_UPDATE_MASK (0x10U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_UPDATE_SHIFT (4U)
/*! PFD3_UPDATE - PFD3_UPDATE */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_UPDATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_UPDATE_MASK)

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_CONTROL_MODE_MASK (0x20U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_CONTROL_MODE_SHIFT (5U)
/*! PFD0_CONTROL_MODE - pfd0_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD0_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_CONTROL_MODE_MASK (0x40U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_CONTROL_MODE_SHIFT (6U)
/*! PFD1_CONTROL_MODE - pfd1_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD1_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_CONTROL_MODE_MASK (0x80U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_CONTROL_MODE_SHIFT (7U)
/*! PFD2_CONTROL_MODE - pfd2_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD2_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_CONTROL_MODE_MASK (0x100U)
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_CONTROL_MODE_SHIFT (8U)
/*! PFD3_CONTROL_MODE - pfd3_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL2_UPDATE_PFD3_CONTROL_MODE_MASK)
/*! @} */

/*! @name SYS_PLL2_SS - SYS_PLL2_SS_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL2_SS_STEP_MASK         (0x7FFFU)
#define ANADIG_PLL_SYS_PLL2_SS_STEP_SHIFT        (0U)
/*! STEP - STEP */
#define ANADIG_PLL_SYS_PLL2_SS_STEP(x)           (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_SS_STEP_SHIFT)) & ANADIG_PLL_SYS_PLL2_SS_STEP_MASK)

#define ANADIG_PLL_SYS_PLL2_SS_ENABLE_MASK       (0x8000U)
#define ANADIG_PLL_SYS_PLL2_SS_ENABLE_SHIFT      (15U)
/*! ENABLE - ENABLE
 *  0b1..Enable Spread Spectrum
 *  0b0..Disable Spread Spectrum
 */
#define ANADIG_PLL_SYS_PLL2_SS_ENABLE(x)         (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_SS_ENABLE_SHIFT)) & ANADIG_PLL_SYS_PLL2_SS_ENABLE_MASK)

#define ANADIG_PLL_SYS_PLL2_SS_STOP_MASK         (0xFFFF0000U)
#define ANADIG_PLL_SYS_PLL2_SS_STOP_SHIFT        (16U)
/*! STOP - STOP */
#define ANADIG_PLL_SYS_PLL2_SS_STOP(x)           (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_SS_STOP_SHIFT)) & ANADIG_PLL_SYS_PLL2_SS_STOP_MASK)
/*! @} */

/*! @name SYS_PLL2_PFD - SYS_PLL2_PFD_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_FRAC_MASK   (0x3FU)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_FRAC_SHIFT  (0U)
/*! PFD0_FRAC - PFD0_FRAC */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD0_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD0_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_STABLE_MASK (0x40U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_STABLE_SHIFT (6U)
/*! PFD0_STABLE - PFD0_STABLE */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD0_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD0_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_DIV1_CLKGATE_MASK (0x80U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_DIV1_CLKGATE_SHIFT (7U)
/*! PFD0_DIV1_CLKGATE - PFD0_CLKGATE
 *  0b0..PFD0 fractional divider clock is enabled.
 *  0b1..Fractional divider clock (reference PFD0) is off (power savings)
 */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD0_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD0_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD0_DIV1_CLKGATE_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_FRAC_MASK   (0x3F00U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_FRAC_SHIFT  (8U)
/*! PFD1_FRAC - PFD1_FRAC */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD1_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD1_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_STABLE_MASK (0x4000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_STABLE_SHIFT (14U)
/*! PFD1_STABLE - PFD1_STABLE */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD1_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD1_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_DIV1_CLKGATE_MASK (0x8000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_DIV1_CLKGATE_SHIFT (15U)
/*! PFD1_DIV1_CLKGATE - PFD1_CLKGATE
 *  0b0..PFD1 fractional divider clock is enabled.
 *  0b1..Fractional divider clock (reference PFD1) is off (power savings)
 */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD1_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD1_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD1_DIV1_CLKGATE_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_FRAC_MASK   (0x3F0000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_FRAC_SHIFT  (16U)
/*! PFD2_FRAC - PFD2_FRAC */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD2_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD2_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_STABLE_MASK (0x400000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_STABLE_SHIFT (22U)
/*! PFD2_STABLE - PFD2_STABLE */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD2_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD2_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_DIV1_CLKGATE_MASK (0x800000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_DIV1_CLKGATE_SHIFT (23U)
/*! PFD2_DIV1_CLKGATE - PFD2_CLKGATE
 *  0b0..PFD2 fractional divider clock is enabled.
 *  0b1..Fractional divider clock (reference PFD2) is off (power savings)
 */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD2_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD2_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD2_DIV1_CLKGATE_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_FRAC_MASK   (0x3F000000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_FRAC_SHIFT  (24U)
/*! PFD3_FRAC - PFD3_FRAC */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_FRAC(x)     (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD3_FRAC_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD3_FRAC_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_STABLE_MASK (0x40000000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_STABLE_SHIFT (30U)
/*! PFD3_STABLE - PFD3_STABLE */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_STABLE(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD3_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD3_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_DIV1_CLKGATE_MASK (0x80000000U)
#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_DIV1_CLKGATE_SHIFT (31U)
/*! PFD3_DIV1_CLKGATE - PFD3_CLKGATE
 *  0b0..PFD3 fractional divider clock is enabled.
 *  0b1..Fractional divider clock (reference PFD3) is off (power savings)
 */
#define ANADIG_PLL_SYS_PLL2_PFD_PFD3_DIV1_CLKGATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_PFD_PFD3_DIV1_CLKGATE_SHIFT)) & ANADIG_PLL_SYS_PLL2_PFD_PFD3_DIV1_CLKGATE_MASK)
/*! @} */

/*! @name SYS_PLL2_MFN - SYS_PLL2_MFN_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL2_MFN_MFN_MASK         (0x3FFFFFFFU)
#define ANADIG_PLL_SYS_PLL2_MFN_MFN_SHIFT        (0U)
/*! MFN - MFN */
#define ANADIG_PLL_SYS_PLL2_MFN_MFN(x)           (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_MFN_MFN_SHIFT)) & ANADIG_PLL_SYS_PLL2_MFN_MFN_MASK)
/*! @} */

/*! @name SYS_PLL2_MFI - SYS_PLL2_MFI_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL2_MFI_MFI_MASK         (0x7FU)
#define ANADIG_PLL_SYS_PLL2_MFI_MFI_SHIFT        (0U)
/*! MFI - MFI */
#define ANADIG_PLL_SYS_PLL2_MFI_MFI(x)           (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_MFI_MFI_SHIFT)) & ANADIG_PLL_SYS_PLL2_MFI_MFI_MASK)
/*! @} */

/*! @name SYS_PLL2_MFD - SYS_PLL2_MFD_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL2_MFD_MFD_MASK         (0x3FFFFFFFU)
#define ANADIG_PLL_SYS_PLL2_MFD_MFD_SHIFT        (0U)
/*! MFD - Denominator */
#define ANADIG_PLL_SYS_PLL2_MFD_MFD(x)           (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL2_MFD_MFD_SHIFT)) & ANADIG_PLL_SYS_PLL2_MFD_MFD_MASK)
/*! @} */

/*! @name SYS_PLL1_CTRL - SYS_PLL1_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PLL_SYS_PLL1_CTRL_ENABLE_CLK_MASK (0x2000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_ENABLE_CLK_SHIFT (13U)
/*! ENABLE_CLK - ENABLE_CLK
 *  0b1..Disable
 *  0b0..Enable
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_ENABLE_CLK(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_ENABLE_CLK_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_ENABLE_CLK_MASK)

#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_GATE_MASK (0x4000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_GATE_SHIFT (14U)
/*! SYS_PLL1_GATE - SYS_PLL1_GATE
 *  0b1..Gate the output
 *  0b0..No gate
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_GATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_GATE_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_GATE_MASK)

#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_MASK (0x2000000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_SHIFT (25U)
/*! SYS_PLL1_DIV2 - SYS_PLL1_DIV2
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_MASK)

#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_MASK (0x4000000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_SHIFT (26U)
/*! SYS_PLL1_DIV5 - SYS_PLL1_DIV5
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_MASK)

#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_CONTROL_MODE_MASK (0x8000000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_CONTROL_MODE_SHIFT (27U)
/*! SYS_PLL1_DIV5_CONTROL_MODE - SYS_PLL1_DIV5_CONTROL_MODE
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV5_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_CONTROL_MODE_MASK (0x10000000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_CONTROL_MODE_SHIFT (28U)
/*! SYS_PLL1_DIV2_CONTROL_MODE - SYS_PLL1_DIV2_CONTROL_MODE
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_DIV2_CONTROL_MODE_MASK)

#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_STABLE_MASK (0x20000000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_STABLE_SHIFT (29U)
/*! SYS_PLL1_STABLE - SYS_PLL1_STABLE
 *  0b0..Not Stable
 *  0b1..Stable
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_STABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_STABLE_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_STABLE_MASK)

#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_CONTROL_MODE_SHIFT (31U)
/*! SYS_PLL1_CONTROL_MODE - SYS_PLL1_CONTROL_MODE
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_CONTROL_MODE_SHIFT)) & ANADIG_PLL_SYS_PLL1_CTRL_SYS_PLL1_CONTROL_MODE_MASK)
/*! @} */

/*! @name PLL_AUDIO_CTRL - PLL_AUDIO_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PLL_PLL_AUDIO_CTRL_ENABLE_CLK_MASK (0x2000U)
#define ANADIG_PLL_PLL_AUDIO_CTRL_ENABLE_CLK_SHIFT (13U)
/*! ENABLE_CLK - ENABLE_CLK
 *  0b0..Disable
 *  0b1..Enable
 */
#define ANADIG_PLL_PLL_AUDIO_CTRL_ENABLE_CLK(x)  (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_PLL_AUDIO_CTRL_ENABLE_CLK_SHIFT)) & ANADIG_PLL_PLL_AUDIO_CTRL_ENABLE_CLK_MASK)

#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_GATE_MASK (0x4000U)
#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_GATE_SHIFT (14U)
/*! PLL_AUDIO_GATE - PLL_AUDIO_GATE
 *  0b1..Gate the output
 *  0b0..No gate
 */
#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_GATE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_GATE_SHIFT)) & ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_GATE_MASK)

#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_STABLE_MASK (0x20000000U)
#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_STABLE_SHIFT (29U)
/*! PLL_AUDIO_STABLE - PLL_AUDIO_STABLE */
#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_STABLE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_STABLE_SHIFT)) & ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_STABLE_MASK)

#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_CONTROL_MODE_MASK (0x80000000U)
#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_CONTROL_MODE_SHIFT (31U)
/*! PLL_AUDIO_CONTROL_MODE - pll_audio_control_mode
 *  0b0..Software Mode (Default)
 *  0b1..GPC Mode
 */
#define ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_CONTROL_MODE_SHIFT)) & ANADIG_PLL_PLL_AUDIO_CTRL_PLL_AUDIO_CONTROL_MODE_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ANADIG_PLL_Register_Masks */


/* ANADIG_PLL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ANADIG_PLL base address */
  #define ANADIG_PLL_BASE                          (0x54480000u)
  /** Peripheral ANADIG_PLL base address */
  #define ANADIG_PLL_BASE_NS                       (0x44480000u)
  /** Peripheral ANADIG_PLL base pointer */
  #define ANADIG_PLL                               ((ANADIG_PLL_Type *)ANADIG_PLL_BASE)
  /** Peripheral ANADIG_PLL base pointer */
  #define ANADIG_PLL_NS                            ((ANADIG_PLL_Type *)ANADIG_PLL_BASE_NS)
  /** Array initializer of ANADIG_PLL peripheral base addresses */
  #define ANADIG_PLL_BASE_ADDRS                    { ANADIG_PLL_BASE }
  /** Array initializer of ANADIG_PLL peripheral base pointers */
  #define ANADIG_PLL_BASE_PTRS                     { ANADIG_PLL }
  /** Array initializer of ANADIG_PLL peripheral base addresses */
  #define ANADIG_PLL_BASE_ADDRS_NS                 { ANADIG_PLL_BASE_NS }
  /** Array initializer of ANADIG_PLL peripheral base pointers */
  #define ANADIG_PLL_BASE_PTRS_NS                  { ANADIG_PLL_NS }
#else
  /** Peripheral ANADIG_PLL base address */
  #define ANADIG_PLL_BASE                          (0x44480000u)
  /** Peripheral ANADIG_PLL base pointer */
  #define ANADIG_PLL                               ((ANADIG_PLL_Type *)ANADIG_PLL_BASE)
  /** Array initializer of ANADIG_PLL peripheral base addresses */
  #define ANADIG_PLL_BASE_ADDRS                    { ANADIG_PLL_BASE }
  /** Array initializer of ANADIG_PLL peripheral base pointers */
  #define ANADIG_PLL_BASE_PTRS                     { ANADIG_PLL }
#endif

/*!
 * @}
 */ /* end of group ANADIG_PLL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ANADIG_PMU Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_PMU_Peripheral_Access_Layer ANADIG_PMU Peripheral Access Layer
 * @{
 */

/** ANADIG_PMU - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[17920];
  __IO uint32_t PMU_BIAS_CTRL;                     /**< PMU_BIAS_CTRL_REGISTER, offset: 0x4600 */
       uint8_t RESERVED_1[12];
  __IO uint32_t PMU_BIAS_CTRL2;                    /**< PMU_BIAS_CTRL2_REGISTER, offset: 0x4610 */
       uint8_t RESERVED_2[44];
  __IO uint32_t PMU_LDO_PLL;                       /**< PMU_LDO_PLL_REGISTER, offset: 0x4640 */
       uint8_t RESERVED_3[188];
  __IO uint32_t PMU_POWER_DETECT_CTRL;             /**< PMU_POWER_DETECT_CTRL_REGISTER, offset: 0x4700 */
       uint8_t RESERVED_4[12];
  __IO uint32_t PMU_REF_CTRL;                      /**< PMU_REF_CTRL_REGISTER, offset: 0x4710 */
} ANADIG_PMU_Type;

/* ----------------------------------------------------------------------------
   -- ANADIG_PMU Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_PMU_Register_Masks ANADIG_PMU Register Masks
 * @{
 */

/*! @name PMU_BIAS_CTRL - PMU_BIAS_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PMU_PMU_BIAS_CTRL_WB_CFG_1P8_MASK (0x1FFFU)
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_CFG_1P8_SHIFT (0U)
/*! WB_CFG_1P8 - wb_cfg_1p8 */
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_CFG_1P8(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL_WB_CFG_1P8_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL_WB_CFG_1P8_MASK)

#define ANADIG_PMU_PMU_BIAS_CTRL_WB_VDD_SEL_1P8_MASK (0x4000U)
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_VDD_SEL_1P8_SHIFT (14U)
/*! WB_VDD_SEL_1P8 - wb_vdd_sel_1p8
 *  0b0..VDD_LV1
 *  0b1..VDD_LV2
 */
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_VDD_SEL_1P8(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL_WB_VDD_SEL_1P8_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL_WB_VDD_SEL_1P8_MASK)

#define ANADIG_PMU_PMU_BIAS_CTRL_FBB_M7_STBY_EN_MASK (0x8000U)
#define ANADIG_PMU_PMU_BIAS_CTRL_FBB_M7_STBY_EN_SHIFT (15U)
/*! FBB_M7_STBY_EN - standby enable bit of fbb m7
 *  0b0..FBB_M7 will be still on when gpc give standby request. After the mode is switched to gpc mode, keep this bit as it is.
 *  0b1..FBB_M7 will standby when gpc give standby request.
 */
#define ANADIG_PMU_PMU_BIAS_CTRL_FBB_M7_STBY_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL_FBB_M7_STBY_EN_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL_FBB_M7_STBY_EN_MASK)

#define ANADIG_PMU_PMU_BIAS_CTRL_WB_PW_LVL_1P8_MASK (0xF000000U)
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_PW_LVL_1P8_SHIFT (24U)
/*! WB_PW_LVL_1P8 - wb_pw_lvl_1p8 */
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_PW_LVL_1P8(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL_WB_PW_LVL_1P8_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL_WB_PW_LVL_1P8_MASK)

#define ANADIG_PMU_PMU_BIAS_CTRL_WB_NW_LVL_1P8_MASK (0xF0000000U)
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_NW_LVL_1P8_SHIFT (28U)
/*! WB_NW_LVL_1P8 - wb_nw_lvl_1p8 */
#define ANADIG_PMU_PMU_BIAS_CTRL_WB_NW_LVL_1P8(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL_WB_NW_LVL_1P8_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL_WB_NW_LVL_1P8_MASK)
/*! @} */

/*! @name PMU_BIAS_CTRL2 - PMU_BIAS_CTRL2_REGISTER */
/*! @{ */

#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_PWR_SW_EN_1P8_MASK (0x1000U)
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_PWR_SW_EN_1P8_SHIFT (12U)
/*! WB_PWR_SW_EN_1P8 - MODSEL_wb_tst_md_1p8
 *  0b1..WELL connected to back biasing generators.
 *  0b0..WELL connected to no back-biasing power supplies
 */
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_PWR_SW_EN_1P8(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL2_WB_PWR_SW_EN_1P8_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL2_WB_PWR_SW_EN_1P8_MASK)

#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_ADJ_1P8_MASK (0x1FE000U)
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_ADJ_1P8_SHIFT (13U)
/*! WB_ADJ_1P8 - wb_adj_1p8
 *  0b00000000..Cref= 0fF Cspl= 0fF DeltaC= 0fF
 *  0b00000001..Cref= 0fF Cspl= 30fF DeltaC= -30fF
 *  0b00000010..Cref= 0fF Cspl= 43fF DeltaC= -43fF
 *  0b00000011..Cref= 0fF Cspl= 62fF DeltaC=-62fF
 *  0b00000100..Cref= 0fF Cspl=105fF DeltaC=-105fF
 *  0b00000101..Cref= 30fF Cspl= 0fF DeltaC= 30fF
 *  0b00000110..Cref= 30fF Cspl= 43fF DeltaC= -12fF
 *  0b00000111..Cref= 30fF Cspl=105fF DeltaC= -75fF
 *  0b00001000..Cref= 43fF Cspl= 0fF DeltaC= 43fF
 *  0b00001001..Cref= 43fF Cspl= 30fF DeltaC= 13fF
 *  0b00001010..Cref= 43fF Cspl= 62fF DeltaC= -19fF
 *  0b00001011..Cref= 62fF Cspl= 0fF DeltaC= 62fF
 *  0b00001100..Cref= 62fF Cspl= 43fF DeltaC= 19fF
 *  0b00001101..Cref=105fF Cspl= 0fF DeltaC= 105fF
 *  0b00001110..Cref=105fF Cspl=30fF DeltaC= 75fF
 *  0b00001111..Cref=0fF Cspl=0fF DeltaC= 0fF
 */
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_ADJ_1P8(x)  (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL2_WB_ADJ_1P8_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL2_WB_ADJ_1P8_MASK)

#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_EN_MASK     (0x1000000U)
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_EN_SHIFT    (24U)
/*! WB_EN - wb_en
 *  0b0..disable
 *  0b1..Enable
 */
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_EN(x)       (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL2_WB_EN_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL2_WB_EN_MASK)

#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_OK_MASK     (0x4000000U)
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_OK_SHIFT    (26U)
/*! WB_OK - Digital Output pin.
 *  0b0..Regulator is unstable.
 *  0b1..Regulator is stable.
 */
#define ANADIG_PMU_PMU_BIAS_CTRL2_WB_OK(x)       (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_BIAS_CTRL2_WB_OK_SHIFT)) & ANADIG_PMU_PMU_BIAS_CTRL2_WB_OK_MASK)
/*! @} */

/*! @name PMU_LDO_PLL - PMU_LDO_PLL_REGISTER */
/*! @{ */

#define ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_CONTROL_MODE_MASK (0x2U)
#define ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_CONTROL_MODE_SHIFT (1U)
/*! LDO_PLL_CONTROL_MODE - LDO_PLL_CONTROL_MODE
 *  0b0..SW Control
 *  0b1..HW Control
 */
#define ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_CONTROL_MODE_SHIFT)) & ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_CONTROL_MODE_MASK)

#define ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_STBY_EN_MASK (0x4U)
#define ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_STBY_EN_SHIFT (2U)
/*! LDO_PLL_STBY_EN - standby enable bit of ldopll
 *  0b0..phy_ldo will be still on when gpc gives standby request. After the mode is switched to gpc mode, keep this bit as it is.
 *  0b1..phy_ldo will standby when gpc give standby request
 */
#define ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_STBY_EN(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_STBY_EN_SHIFT)) & ANADIG_PMU_PMU_LDO_PLL_LDO_PLL_STBY_EN_MASK)
/*! @} */

/*! @name PMU_POWER_DETECT_CTRL - PMU_POWER_DETECT_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PMU_PMU_POWER_DETECT_CTRL_CKGB_AON1P0_MASK (0x100U)
#define ANADIG_PMU_PMU_POWER_DETECT_CTRL_CKGB_AON1P0_SHIFT (8U)
/*! CKGB_AON1P0 - ckgb_aon1p0
 *  0b0..To disable lpsr_1p0, ckgb need to be disabled first.
 *  0b1..After lpsr_1p0 start up, wait for 1ms and then set ckgb bit.
 */
#define ANADIG_PMU_PMU_POWER_DETECT_CTRL_CKGB_AON1P0(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_POWER_DETECT_CTRL_CKGB_AON1P0_SHIFT)) & ANADIG_PMU_PMU_POWER_DETECT_CTRL_CKGB_AON1P0_MASK)
/*! @} */

/*! @name PMU_REF_CTRL - PMU_REF_CTRL_REGISTER */
/*! @{ */

#define ANADIG_PMU_PMU_REF_CTRL_REF_CONTROL_MODE_MASK (0x8U)
#define ANADIG_PMU_PMU_REF_CTRL_REF_CONTROL_MODE_SHIFT (3U)
/*! REF_CONTROL_MODE - REF_CONTROL_MODE
 *  0b0..SW Control
 *  0b1..HW Control
 */
#define ANADIG_PMU_PMU_REF_CTRL_REF_CONTROL_MODE(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_REF_CTRL_REF_CONTROL_MODE_SHIFT)) & ANADIG_PMU_PMU_REF_CTRL_REF_CONTROL_MODE_MASK)

#define ANADIG_PMU_PMU_REF_CTRL_EN_PLL_VOL_REF_BUFFER_MASK (0x10U)
#define ANADIG_PMU_PMU_REF_CTRL_EN_PLL_VOL_REF_BUFFER_SHIFT (4U)
/*! EN_PLL_VOL_REF_BUFFER - en_pll_vol_ref_buffer */
#define ANADIG_PMU_PMU_REF_CTRL_EN_PLL_VOL_REF_BUFFER(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_REF_CTRL_EN_PLL_VOL_REF_BUFFER_SHIFT)) & ANADIG_PMU_PMU_REF_CTRL_EN_PLL_VOL_REF_BUFFER_MASK)

#define ANADIG_PMU_PMU_REF_CTRL_REF_STBY_EN_MASK (0x40U)
#define ANADIG_PMU_PMU_REF_CTRL_REF_STBY_EN_SHIFT (6U)
/*! REF_STBY_EN - standby enable bit of reftop */
#define ANADIG_PMU_PMU_REF_CTRL_REF_STBY_EN(x)   (((uint32_t)(((uint32_t)(x)) << ANADIG_PMU_PMU_REF_CTRL_REF_STBY_EN_SHIFT)) & ANADIG_PMU_PMU_REF_CTRL_REF_STBY_EN_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ANADIG_PMU_Register_Masks */


/* ANADIG_PMU - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ANADIG_PMU base address */
  #define ANADIG_PMU_BASE                          (0x54480000u)
  /** Peripheral ANADIG_PMU base address */
  #define ANADIG_PMU_BASE_NS                       (0x44480000u)
  /** Peripheral ANADIG_PMU base pointer */
  #define ANADIG_PMU                               ((ANADIG_PMU_Type *)ANADIG_PMU_BASE)
  /** Peripheral ANADIG_PMU base pointer */
  #define ANADIG_PMU_NS                            ((ANADIG_PMU_Type *)ANADIG_PMU_BASE_NS)
  /** Array initializer of ANADIG_PMU peripheral base addresses */
  #define ANADIG_PMU_BASE_ADDRS                    { ANADIG_PMU_BASE }
  /** Array initializer of ANADIG_PMU peripheral base pointers */
  #define ANADIG_PMU_BASE_PTRS                     { ANADIG_PMU }
  /** Array initializer of ANADIG_PMU peripheral base addresses */
  #define ANADIG_PMU_BASE_ADDRS_NS                 { ANADIG_PMU_BASE_NS }
  /** Array initializer of ANADIG_PMU peripheral base pointers */
  #define ANADIG_PMU_BASE_PTRS_NS                  { ANADIG_PMU_NS }
#else
  /** Peripheral ANADIG_PMU base address */
  #define ANADIG_PMU_BASE                          (0x44480000u)
  /** Peripheral ANADIG_PMU base pointer */
  #define ANADIG_PMU                               ((ANADIG_PMU_Type *)ANADIG_PMU_BASE)
  /** Array initializer of ANADIG_PMU peripheral base addresses */
  #define ANADIG_PMU_BASE_ADDRS                    { ANADIG_PMU_BASE }
  /** Array initializer of ANADIG_PMU peripheral base pointers */
  #define ANADIG_PMU_BASE_PTRS                     { ANADIG_PMU }
#endif

/*!
 * @}
 */ /* end of group ANADIG_PMU_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ANADIG_TEMPSENSOR Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_TEMPSENSOR_Peripheral_Access_Layer ANADIG_TEMPSENSOR Peripheral Access Layer
 * @{
 */

/** ANADIG_TEMPSENSOR - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[17712];
  __I  uint32_t TEMPSNS_OTP_TRIM_VALUE;            /**< TEMPSNS_OTP_TRIM_VALUE_REGISTER, offset: 0x4530 */
} ANADIG_TEMPSENSOR_Type;

/* ----------------------------------------------------------------------------
   -- ANADIG_TEMPSENSOR Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ANADIG_TEMPSENSOR_Register_Masks ANADIG_TEMPSENSOR Register Masks
 * @{
 */

/*! @name TEMPSNS_OTP_TRIM_VALUE - TEMPSNS_OTP_TRIM_VALUE_REGISTER */
/*! @{ */

#define ANADIG_TEMPSENSOR_TEMPSNS_OTP_TRIM_VALUE_TEMPSNS_TEMP_VAL_MASK (0x3FFC00U)
#define ANADIG_TEMPSENSOR_TEMPSNS_OTP_TRIM_VALUE_TEMPSNS_TEMP_VAL_SHIFT (10U)
/*! TEMPSNS_TEMP_VAL - Temperature Value at 25C */
#define ANADIG_TEMPSENSOR_TEMPSNS_OTP_TRIM_VALUE_TEMPSNS_TEMP_VAL(x) (((uint32_t)(((uint32_t)(x)) << ANADIG_TEMPSENSOR_TEMPSNS_OTP_TRIM_VALUE_TEMPSNS_TEMP_VAL_SHIFT)) & ANADIG_TEMPSENSOR_TEMPSNS_OTP_TRIM_VALUE_TEMPSNS_TEMP_VAL_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ANADIG_TEMPSENSOR_Register_Masks */


/* ANADIG_TEMPSENSOR - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ANADIG_TEMPSENSOR base address */
  #define ANADIG_TEMPSENSOR_BASE                   (0x54480000u)
  /** Peripheral ANADIG_TEMPSENSOR base address */
  #define ANADIG_TEMPSENSOR_BASE_NS                (0x44480000u)
  /** Peripheral ANADIG_TEMPSENSOR base pointer */
  #define ANADIG_TEMPSENSOR                        ((ANADIG_TEMPSENSOR_Type *)ANADIG_TEMPSENSOR_BASE)
  /** Peripheral ANADIG_TEMPSENSOR base pointer */
  #define ANADIG_TEMPSENSOR_NS                     ((ANADIG_TEMPSENSOR_Type *)ANADIG_TEMPSENSOR_BASE_NS)
  /** Array initializer of ANADIG_TEMPSENSOR peripheral base addresses */
  #define ANADIG_TEMPSENSOR_BASE_ADDRS             { ANADIG_TEMPSENSOR_BASE }
  /** Array initializer of ANADIG_TEMPSENSOR peripheral base pointers */
  #define ANADIG_TEMPSENSOR_BASE_PTRS              { ANADIG_TEMPSENSOR }
  /** Array initializer of ANADIG_TEMPSENSOR peripheral base addresses */
  #define ANADIG_TEMPSENSOR_BASE_ADDRS_NS          { ANADIG_TEMPSENSOR_BASE_NS }
  /** Array initializer of ANADIG_TEMPSENSOR peripheral base pointers */
  #define ANADIG_TEMPSENSOR_BASE_PTRS_NS           { ANADIG_TEMPSENSOR_NS }
#else
  /** Peripheral ANADIG_TEMPSENSOR base address */
  #define ANADIG_TEMPSENSOR_BASE                   (0x44480000u)
  /** Peripheral ANADIG_TEMPSENSOR base pointer */
  #define ANADIG_TEMPSENSOR                        ((ANADIG_TEMPSENSOR_Type *)ANADIG_TEMPSENSOR_BASE)
  /** Array initializer of ANADIG_TEMPSENSOR peripheral base addresses */
  #define ANADIG_TEMPSENSOR_BASE_ADDRS             { ANADIG_TEMPSENSOR_BASE }
  /** Array initializer of ANADIG_TEMPSENSOR peripheral base pointers */
  #define ANADIG_TEMPSENSOR_BASE_PTRS              { ANADIG_TEMPSENSOR }
#endif

/*!
 * @}
 */ /* end of group ANADIG_TEMPSENSOR_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- AOI Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup AOI_Peripheral_Access_Layer AOI Peripheral Access Layer
 * @{
 */

/** AOI - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x4 */
    __IO uint16_t BFCRT01;                           /**< Boolean Function Term 0 and 1 Configuration for EVENT0..Boolean Function Term 0 and 1 Configuration for EVENT3, array offset: 0x0, array step: 0x4 */
    __IO uint16_t BFCRT23;                           /**< Boolean Function Term 2 and 3 Configuration for EVENT0..Boolean Function Term 2 and 3 Configuration for EVENT3, array offset: 0x2, array step: 0x4 */
  } BFCRT[4];
} AOI_Type;

/* ----------------------------------------------------------------------------
   -- AOI Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup AOI_Register_Masks AOI Register Masks
 * @{
 */

/*! @name BFCRT01 - Boolean Function Term 0 and 1 Configuration for EVENT0..Boolean Function Term 0 and 1 Configuration for EVENT3 */
/*! @{ */

#define AOI_BFCRT01_PT1_DC_MASK                  (0x3U)
#define AOI_BFCRT01_PT1_DC_SHIFT                 (0U)
/*! PT1_DC - Product Term 1, Input D Configuration
 *  0b00..Force input D to become 0
 *  0b01..Pass input D
 *  0b10..Complement input D
 *  0b11..Force input D to become 1
 */
#define AOI_BFCRT01_PT1_DC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT1_DC_SHIFT)) & AOI_BFCRT01_PT1_DC_MASK)

#define AOI_BFCRT01_PT1_CC_MASK                  (0xCU)
#define AOI_BFCRT01_PT1_CC_SHIFT                 (2U)
/*! PT1_CC - Product Term 1, Input C Configuration
 *  0b00..Force input C to become 0
 *  0b01..Pass input C
 *  0b10..Complement input C
 *  0b11..Force input C to become 1
 */
#define AOI_BFCRT01_PT1_CC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT1_CC_SHIFT)) & AOI_BFCRT01_PT1_CC_MASK)

#define AOI_BFCRT01_PT1_BC_MASK                  (0x30U)
#define AOI_BFCRT01_PT1_BC_SHIFT                 (4U)
/*! PT1_BC - Product Term 1, Input B Configuration
 *  0b00..Force input B to become 0
 *  0b01..Pass input B
 *  0b10..Complement input B
 *  0b11..Force input B to become 1
 */
#define AOI_BFCRT01_PT1_BC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT1_BC_SHIFT)) & AOI_BFCRT01_PT1_BC_MASK)

#define AOI_BFCRT01_PT1_AC_MASK                  (0xC0U)
#define AOI_BFCRT01_PT1_AC_SHIFT                 (6U)
/*! PT1_AC - Product Term 1, Input A Configuration
 *  0b00..Force input A to become 0
 *  0b01..Pass input A
 *  0b10..Complement input A
 *  0b11..Force input A to become 1
 */
#define AOI_BFCRT01_PT1_AC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT1_AC_SHIFT)) & AOI_BFCRT01_PT1_AC_MASK)

#define AOI_BFCRT01_PT0_DC_MASK                  (0x300U)
#define AOI_BFCRT01_PT0_DC_SHIFT                 (8U)
/*! PT0_DC - Product Term 0, Input D Configuration
 *  0b00..Force input D to become 0
 *  0b01..Pass input D
 *  0b10..Complement input D
 *  0b11..Force input D to become 1
 */
#define AOI_BFCRT01_PT0_DC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT0_DC_SHIFT)) & AOI_BFCRT01_PT0_DC_MASK)

#define AOI_BFCRT01_PT0_CC_MASK                  (0xC00U)
#define AOI_BFCRT01_PT0_CC_SHIFT                 (10U)
/*! PT0_CC - Product Term 0, Input C Configuration
 *  0b00..Force input C to become 0
 *  0b01..Pass input C
 *  0b10..Complement input C
 *  0b11..Force input C to become 1
 */
#define AOI_BFCRT01_PT0_CC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT0_CC_SHIFT)) & AOI_BFCRT01_PT0_CC_MASK)

#define AOI_BFCRT01_PT0_BC_MASK                  (0x3000U)
#define AOI_BFCRT01_PT0_BC_SHIFT                 (12U)
/*! PT0_BC - Product Term 0, Input B Configuration
 *  0b00..Force input B to become 0
 *  0b01..Pass input B
 *  0b10..Complement input B
 *  0b11..Force input B to become 1
 */
#define AOI_BFCRT01_PT0_BC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT0_BC_SHIFT)) & AOI_BFCRT01_PT0_BC_MASK)

#define AOI_BFCRT01_PT0_AC_MASK                  (0xC000U)
#define AOI_BFCRT01_PT0_AC_SHIFT                 (14U)
/*! PT0_AC - Product Term 0, Input A Configuration
 *  0b00..Force input A to become 0
 *  0b01..Pass input A
 *  0b10..Complement input A
 *  0b11..Force input A to become 1
 */
#define AOI_BFCRT01_PT0_AC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT01_PT0_AC_SHIFT)) & AOI_BFCRT01_PT0_AC_MASK)
/*! @} */

/* The count of AOI_BFCRT01 */
#define AOI_BFCRT01_COUNT                        (4U)

/*! @name BFCRT23 - Boolean Function Term 2 and 3 Configuration for EVENT0..Boolean Function Term 2 and 3 Configuration for EVENT3 */
/*! @{ */

#define AOI_BFCRT23_PT3_DC_MASK                  (0x3U)
#define AOI_BFCRT23_PT3_DC_SHIFT                 (0U)
/*! PT3_DC - Product Term 3, Input D Configuration
 *  0b00..Force input D to become 0
 *  0b01..Pass input D
 *  0b10..Complement input D
 *  0b11..Force input D to become 1
 */
#define AOI_BFCRT23_PT3_DC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT3_DC_SHIFT)) & AOI_BFCRT23_PT3_DC_MASK)

#define AOI_BFCRT23_PT3_CC_MASK                  (0xCU)
#define AOI_BFCRT23_PT3_CC_SHIFT                 (2U)
/*! PT3_CC - Product Term 3, Input C Configuration
 *  0b00..Force input C to become 0
 *  0b01..Pass input C
 *  0b10..Complement input C
 *  0b11..Force input C to become 1
 */
#define AOI_BFCRT23_PT3_CC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT3_CC_SHIFT)) & AOI_BFCRT23_PT3_CC_MASK)

#define AOI_BFCRT23_PT3_BC_MASK                  (0x30U)
#define AOI_BFCRT23_PT3_BC_SHIFT                 (4U)
/*! PT3_BC - Product Term 3, Input B Configuration
 *  0b00..Force input B to become 0
 *  0b01..Pass input B
 *  0b10..Complement input B
 *  0b11..Force input B to become 1
 */
#define AOI_BFCRT23_PT3_BC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT3_BC_SHIFT)) & AOI_BFCRT23_PT3_BC_MASK)

#define AOI_BFCRT23_PT3_AC_MASK                  (0xC0U)
#define AOI_BFCRT23_PT3_AC_SHIFT                 (6U)
/*! PT3_AC - Product Term 3, Input A Configuration
 *  0b00..Force input A to become 0
 *  0b01..Pass input A
 *  0b10..Complement input A
 *  0b11..Force input to become 1
 */
#define AOI_BFCRT23_PT3_AC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT3_AC_SHIFT)) & AOI_BFCRT23_PT3_AC_MASK)

#define AOI_BFCRT23_PT2_DC_MASK                  (0x300U)
#define AOI_BFCRT23_PT2_DC_SHIFT                 (8U)
/*! PT2_DC - Product Term 2, Input D Configuration
 *  0b00..Force input D to become 0
 *  0b01..Pass input D
 *  0b10..Complement input D
 *  0b11..Force input D to become 1
 */
#define AOI_BFCRT23_PT2_DC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT2_DC_SHIFT)) & AOI_BFCRT23_PT2_DC_MASK)

#define AOI_BFCRT23_PT2_CC_MASK                  (0xC00U)
#define AOI_BFCRT23_PT2_CC_SHIFT                 (10U)
/*! PT2_CC - Product Term 2, Input C Configuration
 *  0b00..Force input C to become 0
 *  0b01..Pass input C
 *  0b10..Complement input C
 *  0b11..Force input C to become 1
 */
#define AOI_BFCRT23_PT2_CC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT2_CC_SHIFT)) & AOI_BFCRT23_PT2_CC_MASK)

#define AOI_BFCRT23_PT2_BC_MASK                  (0x3000U)
#define AOI_BFCRT23_PT2_BC_SHIFT                 (12U)
/*! PT2_BC - Product Term 2, Input B Configuration
 *  0b00..Force input B to become 0
 *  0b01..Pass input B
 *  0b10..Complement input B
 *  0b11..Force input B to become 1
 */
#define AOI_BFCRT23_PT2_BC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT2_BC_SHIFT)) & AOI_BFCRT23_PT2_BC_MASK)

#define AOI_BFCRT23_PT2_AC_MASK                  (0xC000U)
#define AOI_BFCRT23_PT2_AC_SHIFT                 (14U)
/*! PT2_AC - Product Term 2, Input A Configuration
 *  0b00..Force input A to become 0
 *  0b01..Pass input A
 *  0b10..Complement input A
 *  0b11..Force input A to become 1
 */
#define AOI_BFCRT23_PT2_AC(x)                    (((uint16_t)(((uint16_t)(x)) << AOI_BFCRT23_PT2_AC_SHIFT)) & AOI_BFCRT23_PT2_AC_MASK)
/*! @} */

/* The count of AOI_BFCRT23 */
#define AOI_BFCRT23_COUNT                        (4U)


/*!
 * @}
 */ /* end of group AOI_Register_Masks */


/* AOI - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral AOI1 base address */
  #define AOI1_BASE                                (0x52780000u)
  /** Peripheral AOI1 base address */
  #define AOI1_BASE_NS                             (0x42780000u)
  /** Peripheral AOI1 base pointer */
  #define AOI1                                     ((AOI_Type *)AOI1_BASE)
  /** Peripheral AOI1 base pointer */
  #define AOI1_NS                                  ((AOI_Type *)AOI1_BASE_NS)
  /** Peripheral AOI2 base address */
  #define AOI2_BASE                                (0x52790000u)
  /** Peripheral AOI2 base address */
  #define AOI2_BASE_NS                             (0x42790000u)
  /** Peripheral AOI2 base pointer */
  #define AOI2                                     ((AOI_Type *)AOI2_BASE)
  /** Peripheral AOI2 base pointer */
  #define AOI2_NS                                  ((AOI_Type *)AOI2_BASE_NS)
  /** Peripheral AOI3 base address */
  #define AOI3_BASE                                (0x527E0000u)
  /** Peripheral AOI3 base address */
  #define AOI3_BASE_NS                             (0x427E0000u)
  /** Peripheral AOI3 base pointer */
  #define AOI3                                     ((AOI_Type *)AOI3_BASE)
  /** Peripheral AOI3 base pointer */
  #define AOI3_NS                                  ((AOI_Type *)AOI3_BASE_NS)
  /** Peripheral AOI4 base address */
  #define AOI4_BASE                                (0x527F0000u)
  /** Peripheral AOI4 base address */
  #define AOI4_BASE_NS                             (0x427F0000u)
  /** Peripheral AOI4 base pointer */
  #define AOI4                                     ((AOI_Type *)AOI4_BASE)
  /** Peripheral AOI4 base pointer */
  #define AOI4_NS                                  ((AOI_Type *)AOI4_BASE_NS)
  /** Array initializer of AOI peripheral base addresses */
  #define AOI_BASE_ADDRS                           { 0u, AOI1_BASE, AOI2_BASE, AOI3_BASE, AOI4_BASE }
  /** Array initializer of AOI peripheral base pointers */
  #define AOI_BASE_PTRS                            { (AOI_Type *)0u, AOI1, AOI2, AOI3, AOI4 }
  /** Array initializer of AOI peripheral base addresses */
  #define AOI_BASE_ADDRS_NS                        { 0u, AOI1_BASE_NS, AOI2_BASE_NS, AOI3_BASE_NS, AOI4_BASE_NS }
  /** Array initializer of AOI peripheral base pointers */
  #define AOI_BASE_PTRS_NS                         { (AOI_Type *)0u, AOI1_NS, AOI2_NS, AOI3_NS, AOI4_NS }
#else
  /** Peripheral AOI1 base address */
  #define AOI1_BASE                                (0x42780000u)
  /** Peripheral AOI1 base pointer */
  #define AOI1                                     ((AOI_Type *)AOI1_BASE)
  /** Peripheral AOI2 base address */
  #define AOI2_BASE                                (0x42790000u)
  /** Peripheral AOI2 base pointer */
  #define AOI2                                     ((AOI_Type *)AOI2_BASE)
  /** Peripheral AOI3 base address */
  #define AOI3_BASE                                (0x427E0000u)
  /** Peripheral AOI3 base pointer */
  #define AOI3                                     ((AOI_Type *)AOI3_BASE)
  /** Peripheral AOI4 base address */
  #define AOI4_BASE                                (0x427F0000u)
  /** Peripheral AOI4 base pointer */
  #define AOI4                                     ((AOI_Type *)AOI4_BASE)
  /** Array initializer of AOI peripheral base addresses */
  #define AOI_BASE_ADDRS                           { 0u, AOI1_BASE, AOI2_BASE, AOI3_BASE, AOI4_BASE }
  /** Array initializer of AOI peripheral base pointers */
  #define AOI_BASE_PTRS                            { (AOI_Type *)0u, AOI1, AOI2, AOI3, AOI4 }
#endif

/*!
 * @}
 */ /* end of group AOI_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ASRC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ASRC_Peripheral_Access_Layer ASRC Peripheral Access Layer
 * @{
 */

/** ASRC - Register Layout Typedef */
typedef struct {
  __IO uint32_t ASRCTR;                            /**< ASRC Control, offset: 0x0 */
  __IO uint32_t ASRIER;                            /**< ASRC Interrupt Enable, offset: 0x4 */
       uint8_t RESERVED_0[4];
  __IO uint32_t ASRCNCR;                           /**< ASRC Channel Number Configuration, offset: 0xC */
  __IO uint32_t ASRCFG;                            /**< ASRC Filter Configuration Status, offset: 0x10 */
  __IO uint32_t ASRCSR;                            /**< ASRC Clock Source, offset: 0x14 */
  __IO uint32_t ASRCDR1;                           /**< ASRC Clock Divider 1, offset: 0x18 */
  __IO uint32_t ASRCDR2;                           /**< ASRC Clock Divider 2, offset: 0x1C */
  __I  uint32_t ASRSTR;                            /**< ASRC Status, offset: 0x20 */
       uint8_t RESERVED_1[28];
  __IO uint32_t ASRPM[5];                          /**< ASRC Parameter x, array offset: 0x40, array step: 0x4 */
  __IO uint32_t ASRTFR1;                           /**< ASRC Task Queue FIFO 1, offset: 0x54 */
       uint8_t RESERVED_2[4];
  __IO uint32_t ASRCCR;                            /**< ASRC Channel Counter, offset: 0x5C */
  __O  uint32_t ASRDIA;                            /**< ASRC Data Input for Pair x, offset: 0x60 */
  __I  uint32_t ASRDOA;                            /**< ASRC Data Output for Pair x, offset: 0x64 */
  __O  uint32_t ASRDIB;                            /**< ASRC Data Input for Pair x, offset: 0x68 */
  __I  uint32_t ASRDOB;                            /**< ASRC Data Output for Pair x, offset: 0x6C */
  __O  uint32_t ASRDIC;                            /**< ASRC Data Input for Pair x, offset: 0x70 */
  __I  uint32_t ASRDOC;                            /**< ASRC Data Output for Pair x, offset: 0x74 */
       uint8_t RESERVED_3[8];
  __IO uint32_t ASRIDRHA;                          /**< ASRC Ideal Ratio for Pair A-High Part, offset: 0x80 */
  __IO uint32_t ASRIDRLA;                          /**< ASRC Ideal Ratio for Pair A -Low Part, offset: 0x84 */
  __IO uint32_t ASRIDRHB;                          /**< ASRC Ideal Ratio for Pair B-High Part, offset: 0x88 */
  __IO uint32_t ASRIDRLB;                          /**< ASRC Ideal Ratio for Pair B-Low Part, offset: 0x8C */
  __IO uint32_t ASRIDRHC;                          /**< ASRC Ideal Ratio for Pair C-High Part, offset: 0x90 */
  __IO uint32_t ASRIDRLC;                          /**< ASRC Ideal Ratio for Pair C-Low Part, offset: 0x94 */
  __IO uint32_t ASR76K;                            /**< ASRC 76 kHz Period, offset: 0x98 */
  __IO uint32_t ASR56K;                            /**< ASRC 56 kHz Period, offset: 0x9C */
  __IO uint32_t ASRMCRA;                           /**< ASRC Misc Control for Pair A, offset: 0xA0 */
  __I  uint32_t ASRFSTA;                           /**< ASRC FIFO Status for Pair A, offset: 0xA4 */
  __IO uint32_t ASRMCRB;                           /**< ASRC Misc Control for Pair B, offset: 0xA8 */
  __I  uint32_t ASRFSTB;                           /**< ASRC FIFO Status for Pair B, offset: 0xAC */
  __IO uint32_t ASRMCRC;                           /**< ASRC Misc Control for Pair C, offset: 0xB0 */
  __I  uint32_t ASRFSTC;                           /**< ASRC FIFO Status for Pair C, offset: 0xB4 */
       uint8_t RESERVED_4[8];
  __IO uint32_t ASRMCR1[3];                        /**< ASRC Misc Control 1 for Pair X, array offset: 0xC0, array step: 0x4 */
} ASRC_Type;

/* ----------------------------------------------------------------------------
   -- ASRC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ASRC_Register_Masks ASRC Register Masks
 * @{
 */

/*! @name ASRCTR - ASRC Control */
/*! @{ */

#define ASRC_ASRCTR_ASRCEN_MASK                  (0x1U)
#define ASRC_ASRCTR_ASRCEN_SHIFT                 (0U)
/*! ASRCEN - ASRC Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ASRC_ASRCTR_ASRCEN(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_ASRCEN_SHIFT)) & ASRC_ASRCTR_ASRCEN_MASK)

#define ASRC_ASRCTR_ASREA_MASK                   (0x2U)
#define ASRC_ASRCTR_ASREA_SHIFT                  (1U)
/*! ASREA - ASRC Enable A
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ASRC_ASRCTR_ASREA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_ASREA_SHIFT)) & ASRC_ASRCTR_ASREA_MASK)

#define ASRC_ASRCTR_ASREB_MASK                   (0x4U)
#define ASRC_ASRCTR_ASREB_SHIFT                  (2U)
/*! ASREB - ASRC Enable B
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ASRC_ASRCTR_ASREB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_ASREB_SHIFT)) & ASRC_ASRCTR_ASREB_MASK)

#define ASRC_ASRCTR_ASREC_MASK                   (0x8U)
#define ASRC_ASRCTR_ASREC_SHIFT                  (3U)
/*! ASREC - ASRC Enable C
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ASRC_ASRCTR_ASREC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_ASREC_SHIFT)) & ASRC_ASRCTR_ASREC_MASK)

#define ASRC_ASRCTR_SRST_MASK                    (0x10U)
#define ASRC_ASRCTR_SRST_SHIFT                   (4U)
/*! SRST - Software Reset
 *  0b0..ASRC Software reset cleared
 *  0b1..ASRC Software reset generated. NOTE: This is a self-clear bit
 */
#define ASRC_ASRCTR_SRST(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_SRST_SHIFT)) & ASRC_ASRCTR_SRST_MASK)

#define ASRC_ASRCTR_IDRA_MASK                    (0x2000U)
#define ASRC_ASRCTR_IDRA_SHIFT                   (13U)
/*! IDRA - Use Ideal Ratio for Pair A
 *  0b0..ASRC internal measured ratio is used
 *  0b1..Ideal ratio from the interface register ASRIDRHA, ASRIDRLA is used
 */
#define ASRC_ASRCTR_IDRA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_IDRA_SHIFT)) & ASRC_ASRCTR_IDRA_MASK)

#define ASRC_ASRCTR_USRA_MASK                    (0x4000U)
#define ASRC_ASRCTR_USRA_SHIFT                   (14U)
/*! USRA - Use Ratio for Pair A
 *  0b1..Use ratio as the input to ASRC for pair A
 *  0b0..Do not use ratio as the input to ASRC for pair A
 */
#define ASRC_ASRCTR_USRA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_USRA_SHIFT)) & ASRC_ASRCTR_USRA_MASK)

#define ASRC_ASRCTR_IDRB_MASK                    (0x8000U)
#define ASRC_ASRCTR_IDRB_SHIFT                   (15U)
/*! IDRB - Use Ideal Ratio for Pair B
 *  0b0..ASRC internal measured ratio is used
 *  0b1..Ideal ratio from the interface register ASRIDRHB, ASRIDRLB is used
 */
#define ASRC_ASRCTR_IDRB(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_IDRB_SHIFT)) & ASRC_ASRCTR_IDRB_MASK)

#define ASRC_ASRCTR_USRB_MASK                    (0x10000U)
#define ASRC_ASRCTR_USRB_SHIFT                   (16U)
/*! USRB - Use Ratio for Pair B
 *  0b1..Use ratio as the input to ASRC for pair B
 *  0b0..Do not use ratio as the input to ASRC for pair B
 */
#define ASRC_ASRCTR_USRB(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_USRB_SHIFT)) & ASRC_ASRCTR_USRB_MASK)

#define ASRC_ASRCTR_IDRC_MASK                    (0x20000U)
#define ASRC_ASRCTR_IDRC_SHIFT                   (17U)
/*! IDRC - Use Ideal Ratio for Pair C
 *  0b0..ASRC internal measured ratio is used
 *  0b1..Ideal ratio from the interface register ASRIDRHC, ASRIDRLC is used
 */
#define ASRC_ASRCTR_IDRC(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_IDRC_SHIFT)) & ASRC_ASRCTR_IDRC_MASK)

#define ASRC_ASRCTR_USRC_MASK                    (0x40000U)
#define ASRC_ASRCTR_USRC_SHIFT                   (18U)
/*! USRC - Use Ratio for Pair C
 *  0b1..Use ratio as the input to ASRC for pair C
 *  0b0..Do not use ratio as the input to ASRC for pair C
 */
#define ASRC_ASRCTR_USRC(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_USRC_SHIFT)) & ASRC_ASRCTR_USRC_MASK)

#define ASRC_ASRCTR_ATSA_MASK                    (0x100000U)
#define ASRC_ASRCTR_ATSA_SHIFT                   (20U)
/*! ATSA - ASRC Pair A Automatic Selection For Processing Options
 *  0b1..Pair A automatically updates its pre-processing and post-processing options
 *  0b0..Pair A does not automatically update its pre-processing and post-processing options
 */
#define ASRC_ASRCTR_ATSA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_ATSA_SHIFT)) & ASRC_ASRCTR_ATSA_MASK)

#define ASRC_ASRCTR_ATSB_MASK                    (0x200000U)
#define ASRC_ASRCTR_ATSB_SHIFT                   (21U)
/*! ATSB - ASRC Pair B Automatic Selection For Processing Options
 *  0b1..Pair B automatically updates its pre-processing and post-processing options
 *  0b0..Pair B does not automatically update its pre-processing and post-processing options
 */
#define ASRC_ASRCTR_ATSB(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_ATSB_SHIFT)) & ASRC_ASRCTR_ATSB_MASK)

#define ASRC_ASRCTR_ATSC_MASK                    (0x400000U)
#define ASRC_ASRCTR_ATSC_SHIFT                   (22U)
/*! ATSC - ASRC Pair C Automatic Selection For Processing Options
 *  0b1..Pair C automatically updates its pre-processing and post-processing options
 *  0b0..Pair C does not automatically update its pre-processing and post-processing options
 */
#define ASRC_ASRCTR_ATSC(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCTR_ATSC_SHIFT)) & ASRC_ASRCTR_ATSC_MASK)
/*! @} */

/*! @name ASRIER - ASRC Interrupt Enable */
/*! @{ */

#define ASRC_ASRIER_ADIEA_MASK                   (0x1U)
#define ASRC_ASRIER_ADIEA_SHIFT                  (0U)
/*! ADIEA - Pair A Data Input Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_ADIEA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_ADIEA_SHIFT)) & ASRC_ASRIER_ADIEA_MASK)

#define ASRC_ASRIER_ADIEB_MASK                   (0x2U)
#define ASRC_ASRIER_ADIEB_SHIFT                  (1U)
/*! ADIEB - Pair B Data Input Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_ADIEB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_ADIEB_SHIFT)) & ASRC_ASRIER_ADIEB_MASK)

#define ASRC_ASRIER_ADIEC_MASK                   (0x4U)
#define ASRC_ASRIER_ADIEC_SHIFT                  (2U)
/*! ADIEC - Pair C Data Input Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_ADIEC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_ADIEC_SHIFT)) & ASRC_ASRIER_ADIEC_MASK)

#define ASRC_ASRIER_ADOEA_MASK                   (0x8U)
#define ASRC_ASRIER_ADOEA_SHIFT                  (3U)
/*! ADOEA - Pair A Data Output Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_ADOEA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_ADOEA_SHIFT)) & ASRC_ASRIER_ADOEA_MASK)

#define ASRC_ASRIER_ADOEB_MASK                   (0x10U)
#define ASRC_ASRIER_ADOEB_SHIFT                  (4U)
/*! ADOEB - Pair B Data Output Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_ADOEB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_ADOEB_SHIFT)) & ASRC_ASRIER_ADOEB_MASK)

#define ASRC_ASRIER_ADOEC_MASK                   (0x20U)
#define ASRC_ASRIER_ADOEC_SHIFT                  (5U)
/*! ADOEC - Pair C Data Output Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_ADOEC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_ADOEC_SHIFT)) & ASRC_ASRIER_ADOEC_MASK)

#define ASRC_ASRIER_AOLIE_MASK                   (0x40U)
#define ASRC_ASRIER_AOLIE_SHIFT                  (6U)
/*! AOLIE - Overload Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_AOLIE(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_AOLIE_SHIFT)) & ASRC_ASRIER_AOLIE_MASK)

#define ASRC_ASRIER_AFPWE_MASK                   (0x80U)
#define ASRC_ASRIER_AFPWE_SHIFT                  (7U)
/*! AFPWE - FP in Wait State Interrupt Enable
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define ASRC_ASRIER_AFPWE(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIER_AFPWE_SHIFT)) & ASRC_ASRIER_AFPWE_MASK)
/*! @} */

/*! @name ASRCNCR - ASRC Channel Number Configuration */
/*! @{ */

#define ASRC_ASRCNCR_ANCA_MASK                   (0xFU)
#define ASRC_ASRCNCR_ANCA_SHIFT                  (0U)
/*! ANCA - Number of A Channels
 *  0b0000..0 channels in A (Pair A is disabled)
 *  0b0001..1 channel in A
 *  0b0010..2 channels in A
 *  0b0011..3 channels in A
 *  0b0100..4 channels in A
 *  0b0101..5 channels in A
 *  0b0110..6 channels in A
 *  0b0111..7 channels in A
 *  0b1000..8 channels in A
 *  0b1001..9 channels in A
 *  0b1010..10 channels in A
 *  0b1011-0b1111..Should not be used.
 */
#define ASRC_ASRCNCR_ANCA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCNCR_ANCA_SHIFT)) & ASRC_ASRCNCR_ANCA_MASK)

#define ASRC_ASRCNCR_ANCB_MASK                   (0xF0U)
#define ASRC_ASRCNCR_ANCB_SHIFT                  (4U)
/*! ANCB - Number of B Channels
 *  0b0000..0 channels in B (Pair B is disabled)
 *  0b0001..1 channel in B
 *  0b0010..2 channels in B
 *  0b0011..3 channels in B
 *  0b0100..4 channels in B
 *  0b0101..5 channels in B
 *  0b0110..6 channels in B
 *  0b0111..7 channels in B
 *  0b1000..8 channels in B
 *  0b1001..9 channels in B
 *  0b1010..10 channels in B
 *  0b1011-0b1111..Should not be used.
 */
#define ASRC_ASRCNCR_ANCB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCNCR_ANCB_SHIFT)) & ASRC_ASRCNCR_ANCB_MASK)

#define ASRC_ASRCNCR_ANCC_MASK                   (0xF00U)
#define ASRC_ASRCNCR_ANCC_SHIFT                  (8U)
/*! ANCC - Number of C Channels
 *  0b0000..0 channels in C (Pair C is disabled)
 *  0b0001..1 channel in C
 *  0b0010..2 channels in C
 *  0b0011..3 channels in C
 *  0b0100..4 channels in C
 *  0b0101..5 channels in C
 *  0b0110..6 channels in C
 *  0b0111..7 channels in C
 *  0b1000..8 channels in C
 *  0b1001..9 channels in C
 *  0b1010..10 channels in C
 *  0b1011-0b1111..Should not be used.
 */
#define ASRC_ASRCNCR_ANCC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCNCR_ANCC_SHIFT)) & ASRC_ASRCNCR_ANCC_MASK)
/*! @} */

/*! @name ASRCFG - ASRC Filter Configuration Status */
/*! @{ */

#define ASRC_ASRCFG_PREMODA_MASK                 (0xC0U)
#define ASRC_ASRCFG_PREMODA_SHIFT                (6U)
/*! PREMODA - Pre-Processing Configuration for Conversion Pair A
 *  0b00..Select Upsampling-by-2
 *  0b01..Select Direct-Connection
 *  0b10..Select Downsampling-by-2
 *  0b11..Select passthrough mode. In this case, POSTMODA[1:0] have no use.
 */
#define ASRC_ASRCFG_PREMODA(x)                   (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_PREMODA_SHIFT)) & ASRC_ASRCFG_PREMODA_MASK)

#define ASRC_ASRCFG_POSTMODA_MASK                (0x300U)
#define ASRC_ASRCFG_POSTMODA_SHIFT               (8U)
/*! POSTMODA - Post-Processing Configuration for Conversion Pair A
 *  0b00..Select Upsampling-by-2
 *  0b01..Select Direct-Connection
 *  0b10..Select Downsampling-by-2
 *  0b11..Reserved.
 */
#define ASRC_ASRCFG_POSTMODA(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_POSTMODA_SHIFT)) & ASRC_ASRCFG_POSTMODA_MASK)

#define ASRC_ASRCFG_PREMODB_MASK                 (0xC00U)
#define ASRC_ASRCFG_PREMODB_SHIFT                (10U)
/*! PREMODB - Pre-Processing Configuration for Conversion Pair B
 *  0b00..Select Upsampling-by-2
 *  0b01..Select Direct-Connection
 *  0b10..Select Downsampling-by-2
 *  0b11..Select passthrough mode. In this case, POSTMODB[1:0] have no use.
 */
#define ASRC_ASRCFG_PREMODB(x)                   (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_PREMODB_SHIFT)) & ASRC_ASRCFG_PREMODB_MASK)

#define ASRC_ASRCFG_POSTMODB_MASK                (0x3000U)
#define ASRC_ASRCFG_POSTMODB_SHIFT               (12U)
/*! POSTMODB - Post-Processing Configuration for Conversion Pair B
 *  0b00..Select Upsampling-by-2
 *  0b01..Select Direct-Connection
 *  0b10..Select Downsampling-by-2
 *  0b11..Reserved.
 */
#define ASRC_ASRCFG_POSTMODB(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_POSTMODB_SHIFT)) & ASRC_ASRCFG_POSTMODB_MASK)

#define ASRC_ASRCFG_PREMODC_MASK                 (0xC000U)
#define ASRC_ASRCFG_PREMODC_SHIFT                (14U)
/*! PREMODC - Pre-Processing Configuration for Conversion Pair C
 *  0b00..Select Upsampling-by-2
 *  0b01..Select Direct-Connection
 *  0b10..Select Downsampling-by-2
 *  0b11..Select passthrough mode. In this case, POSTMODC[1:0] have no use.
 */
#define ASRC_ASRCFG_PREMODC(x)                   (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_PREMODC_SHIFT)) & ASRC_ASRCFG_PREMODC_MASK)

#define ASRC_ASRCFG_POSTMODC_MASK                (0x30000U)
#define ASRC_ASRCFG_POSTMODC_SHIFT               (16U)
/*! POSTMODC - Post-Processing Configuration for Conversion Pair C
 *  0b00..Select Upsampling-by-2 as defined in Signal Processing Flow.
 *  0b01..Select Direct-Connection as defined in Signal Processing Flow.
 *  0b10..Select Downsampling-by-2 as defined in Signal Processing Flow.
 *  0b11..Reserved.
 */
#define ASRC_ASRCFG_POSTMODC(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_POSTMODC_SHIFT)) & ASRC_ASRCFG_POSTMODC_MASK)

#define ASRC_ASRCFG_NDPRA_MASK                   (0x40000U)
#define ASRC_ASRCFG_NDPRA_SHIFT                  (18U)
/*! NDPRA - Not Use Default Parameters for RAM-stored Parameters For Conversion Pair A
 *  0b0..Use default parameters for RAM-stored parameters. Override any parameters already in RAM.
 *  0b1..Don't use default parameters for RAM-stored parameters. Use the parameters already stored in RAM.
 */
#define ASRC_ASRCFG_NDPRA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_NDPRA_SHIFT)) & ASRC_ASRCFG_NDPRA_MASK)

#define ASRC_ASRCFG_NDPRB_MASK                   (0x80000U)
#define ASRC_ASRCFG_NDPRB_SHIFT                  (19U)
/*! NDPRB - Not Use Default Parameters for RAM-Stored Parameters For Conversion Pair B
 *  0b0..Use default parameters for RAM-stored parameters. Override any parameters already in RAM.
 *  0b1..Don't use default parameters for RAM-stored parameter. Use the parameters already stored in RAM.
 */
#define ASRC_ASRCFG_NDPRB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_NDPRB_SHIFT)) & ASRC_ASRCFG_NDPRB_MASK)

#define ASRC_ASRCFG_NDPRC_MASK                   (0x100000U)
#define ASRC_ASRCFG_NDPRC_SHIFT                  (20U)
/*! NDPRC - Not Use Default Parameters for RAM-Stored Parameters For Conversion Pair C
 *  0b0..Use default parameters for RAM-stored parameters. Override any parameters already in RAM.
 *  0b1..Don't use default parameters for RAM-stored parameters. Use the parameters already stored in RAM.
 */
#define ASRC_ASRCFG_NDPRC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_NDPRC_SHIFT)) & ASRC_ASRCFG_NDPRC_MASK)

#define ASRC_ASRCFG_INIRQA_MASK                  (0x200000U)
#define ASRC_ASRCFG_INIRQA_SHIFT                 (21U)
/*! INIRQA - Initialization for Conversion Pair A is served
 *  0b0..Initialization for Conversion Pair A not served
 *  0b1..Initialization for Conversion Pair A served
 */
#define ASRC_ASRCFG_INIRQA(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_INIRQA_SHIFT)) & ASRC_ASRCFG_INIRQA_MASK)

#define ASRC_ASRCFG_INIRQB_MASK                  (0x400000U)
#define ASRC_ASRCFG_INIRQB_SHIFT                 (22U)
/*! INIRQB - Initialization for Conversion Pair B is Served
 *  0b0..Initialization for Conversion Pair B not served
 *  0b1..Initialization for Conversion Pair B served
 */
#define ASRC_ASRCFG_INIRQB(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_INIRQB_SHIFT)) & ASRC_ASRCFG_INIRQB_MASK)

#define ASRC_ASRCFG_INIRQC_MASK                  (0x800000U)
#define ASRC_ASRCFG_INIRQC_SHIFT                 (23U)
/*! INIRQC - Initialization for Conversion Pair C is Served
 *  0b0..Initialization for Conversion Pair C not served
 *  0b1..Initialization for Conversion Pair C served
 */
#define ASRC_ASRCFG_INIRQC(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCFG_INIRQC_SHIFT)) & ASRC_ASRCFG_INIRQC_MASK)
/*! @} */

/*! @name ASRCSR - ASRC Clock Source */
/*! @{ */

#define ASRC_ASRCSR_AICSA_MASK                   (0xFU)
#define ASRC_ASRCSR_AICSA_SHIFT                  (0U)
/*! AICSA - Input Clock Source A
 *  0b0000..Bit clock 0
 *  0b0001..Bit clock 1
 *  0b0010..Bit clock 2
 *  0b0011..Bit clock 3
 *  0b0100..Bit clock 4
 *  0b0101..Bit clock 5
 *  0b0110..Bit clock 6
 *  0b0111..Bit clock 7
 *  0b1000..Bit clock 8
 *  0b1001..Bit clock 9
 *  0b1010..Bit clock A
 *  0b1011..Bit clock B
 *  0b1100..Bit clock C
 *  0b1101..Bit clock D
 *  0b1110..Bit clock E
 *  0b1111..Clock disabled, connected to zero
 */
#define ASRC_ASRCSR_AICSA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCSR_AICSA_SHIFT)) & ASRC_ASRCSR_AICSA_MASK)

#define ASRC_ASRCSR_AICSB_MASK                   (0xF0U)
#define ASRC_ASRCSR_AICSB_SHIFT                  (4U)
/*! AICSB - Input Clock Source B
 *  0b0000..Bit clock 0
 *  0b0001..Bit clock 1
 *  0b0010..Bit clock 2
 *  0b0011..Bit clock 3
 *  0b0100..Bit clock 4
 *  0b0101..Bit clock 5
 *  0b0110..Bit clock 6
 *  0b0111..Bit clock 7
 *  0b1000..Bit clock 8
 *  0b1001..Bit clock 9
 *  0b1010..Bit clock A
 *  0b1011..Bit clock B
 *  0b1100..Bit clock C
 *  0b1101..Bit clock D
 *  0b1110..Bit clock E
 *  0b1111..Clock disabled, connected to zero
 */
#define ASRC_ASRCSR_AICSB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCSR_AICSB_SHIFT)) & ASRC_ASRCSR_AICSB_MASK)

#define ASRC_ASRCSR_AICSC_MASK                   (0xF00U)
#define ASRC_ASRCSR_AICSC_SHIFT                  (8U)
/*! AICSC - Input Clock Source C
 *  0b0000..Bit clock 0
 *  0b0001..Bit clock 1
 *  0b0010..Bit clock 2
 *  0b0011..Bit clock 3
 *  0b0100..Bit clock 4
 *  0b0101..Bit clock 5
 *  0b0110..Bit clock 6
 *  0b0111..Bit clock 7
 *  0b1000..Bit clock 8
 *  0b1001..Bit clock 9
 *  0b1010..Bit clock A
 *  0b1011..Bit clock B
 *  0b1100..Bit clock C
 *  0b1101..Bit clock D
 *  0b1110..Bit clock E
 *  0b1111..Clock disabled, connected to zero
 */
#define ASRC_ASRCSR_AICSC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCSR_AICSC_SHIFT)) & ASRC_ASRCSR_AICSC_MASK)

#define ASRC_ASRCSR_AOCSA_MASK                   (0xF000U)
#define ASRC_ASRCSR_AOCSA_SHIFT                  (12U)
/*! AOCSA - Output Clock Source A
 *  0b0000..Bit clock 0
 *  0b0001..Bit clock 1
 *  0b0010..Bit clock 2
 *  0b0011..Bit clock 3
 *  0b0100..Bit clock 4
 *  0b0101..Bit clock 5
 *  0b0110..Bit clock 6
 *  0b0111..Bit clock 7
 *  0b1000..Bit clock 8
 *  0b1001..Bit clock 9
 *  0b1010..Bit clock A
 *  0b1011..Bit clock B
 *  0b1100..Bit clock C
 *  0b1101..Bit clock D
 *  0b1110..Bit clock E
 *  0b1111..Clock disabled, connected to zero
 */
#define ASRC_ASRCSR_AOCSA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCSR_AOCSA_SHIFT)) & ASRC_ASRCSR_AOCSA_MASK)

#define ASRC_ASRCSR_AOCSB_MASK                   (0xF0000U)
#define ASRC_ASRCSR_AOCSB_SHIFT                  (16U)
/*! AOCSB - Output Clock Source B
 *  0b0000..Bit clock 0
 *  0b0001..Bit clock 1
 *  0b0010..Bit clock 2
 *  0b0011..Bit clock 3
 *  0b0100..Bit clock 4
 *  0b0101..Bit clock 5
 *  0b0110..Bit clock 6
 *  0b0111..Bit clock 7
 *  0b1000..Bit clock 8
 *  0b1001..Bit clock 9
 *  0b1010..Bit clock A
 *  0b1011..Bit clock B
 *  0b1100..Bit clock C
 *  0b1101..Bit clock D
 *  0b1110..Bit clock E
 *  0b1111..Clock disabled, connected to zero
 */
#define ASRC_ASRCSR_AOCSB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCSR_AOCSB_SHIFT)) & ASRC_ASRCSR_AOCSB_MASK)

#define ASRC_ASRCSR_AOCSC_MASK                   (0xF00000U)
#define ASRC_ASRCSR_AOCSC_SHIFT                  (20U)
/*! AOCSC - Output Clock Source C
 *  0b0000..Bit clock 0
 *  0b0001..Bit clock 1
 *  0b0010..Bit clock 2
 *  0b0011..Bit clock 3
 *  0b0100..Bit clock 4
 *  0b0101..Bit clock 5
 *  0b0110..Bit clock 6
 *  0b0111..Bit clock 7
 *  0b1000..Bit clock 8
 *  0b1001..Bit clock 9
 *  0b1010..Bit clock A
 *  0b1011..Bit clock B
 *  0b1100..Bit clock C
 *  0b1101..Bit clock D
 *  0b1110..Bit clock E
 *  0b1111..Clock disabled, connected to zero
 */
#define ASRC_ASRCSR_AOCSC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCSR_AOCSC_SHIFT)) & ASRC_ASRCSR_AOCSC_MASK)
/*! @} */

/*! @name ASRCDR1 - ASRC Clock Divider 1 */
/*! @{ */

#define ASRC_ASRCDR1_AICPA_MASK                  (0x7U)
#define ASRC_ASRCDR1_AICPA_SHIFT                 (0U)
/*! AICPA - Input Clock Prescaler A */
#define ASRC_ASRCDR1_AICPA(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AICPA_SHIFT)) & ASRC_ASRCDR1_AICPA_MASK)

#define ASRC_ASRCDR1_AICDA_MASK                  (0x38U)
#define ASRC_ASRCDR1_AICDA_SHIFT                 (3U)
/*! AICDA - Input Clock Divider A */
#define ASRC_ASRCDR1_AICDA(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AICDA_SHIFT)) & ASRC_ASRCDR1_AICDA_MASK)

#define ASRC_ASRCDR1_AICPB_MASK                  (0x1C0U)
#define ASRC_ASRCDR1_AICPB_SHIFT                 (6U)
/*! AICPB - Input Clock Prescaler B */
#define ASRC_ASRCDR1_AICPB(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AICPB_SHIFT)) & ASRC_ASRCDR1_AICPB_MASK)

#define ASRC_ASRCDR1_AICDB_MASK                  (0xE00U)
#define ASRC_ASRCDR1_AICDB_SHIFT                 (9U)
/*! AICDB - Input Clock Divider B */
#define ASRC_ASRCDR1_AICDB(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AICDB_SHIFT)) & ASRC_ASRCDR1_AICDB_MASK)

#define ASRC_ASRCDR1_AOCPA_MASK                  (0x7000U)
#define ASRC_ASRCDR1_AOCPA_SHIFT                 (12U)
/*! AOCPA - Output Clock Prescaler A */
#define ASRC_ASRCDR1_AOCPA(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AOCPA_SHIFT)) & ASRC_ASRCDR1_AOCPA_MASK)

#define ASRC_ASRCDR1_AOCDA_MASK                  (0x38000U)
#define ASRC_ASRCDR1_AOCDA_SHIFT                 (15U)
/*! AOCDA - Output Clock Divider A */
#define ASRC_ASRCDR1_AOCDA(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AOCDA_SHIFT)) & ASRC_ASRCDR1_AOCDA_MASK)

#define ASRC_ASRCDR1_AOCPB_MASK                  (0x1C0000U)
#define ASRC_ASRCDR1_AOCPB_SHIFT                 (18U)
/*! AOCPB - Output Clock Prescaler B */
#define ASRC_ASRCDR1_AOCPB(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AOCPB_SHIFT)) & ASRC_ASRCDR1_AOCPB_MASK)

#define ASRC_ASRCDR1_AOCDB_MASK                  (0xE00000U)
#define ASRC_ASRCDR1_AOCDB_SHIFT                 (21U)
/*! AOCDB - Output Clock Divider B */
#define ASRC_ASRCDR1_AOCDB(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR1_AOCDB_SHIFT)) & ASRC_ASRCDR1_AOCDB_MASK)
/*! @} */

/*! @name ASRCDR2 - ASRC Clock Divider 2 */
/*! @{ */

#define ASRC_ASRCDR2_AICPC_MASK                  (0x7U)
#define ASRC_ASRCDR2_AICPC_SHIFT                 (0U)
/*! AICPC - Input Clock Prescaler C */
#define ASRC_ASRCDR2_AICPC(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR2_AICPC_SHIFT)) & ASRC_ASRCDR2_AICPC_MASK)

#define ASRC_ASRCDR2_AICDC_MASK                  (0x38U)
#define ASRC_ASRCDR2_AICDC_SHIFT                 (3U)
/*! AICDC - Input Clock Divider C */
#define ASRC_ASRCDR2_AICDC(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR2_AICDC_SHIFT)) & ASRC_ASRCDR2_AICDC_MASK)

#define ASRC_ASRCDR2_AOCPC_MASK                  (0x1C0U)
#define ASRC_ASRCDR2_AOCPC_SHIFT                 (6U)
/*! AOCPC - Output Clock Prescaler C */
#define ASRC_ASRCDR2_AOCPC(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR2_AOCPC_SHIFT)) & ASRC_ASRCDR2_AOCPC_MASK)

#define ASRC_ASRCDR2_AOCDC_MASK                  (0xE00U)
#define ASRC_ASRCDR2_AOCDC_SHIFT                 (9U)
/*! AOCDC - Output Clock Divider C */
#define ASRC_ASRCDR2_AOCDC(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCDR2_AOCDC_SHIFT)) & ASRC_ASRCDR2_AOCDC_MASK)
/*! @} */

/*! @name ASRSTR - ASRC Status */
/*! @{ */

#define ASRC_ASRSTR_AIDEA_MASK                   (0x1U)
#define ASRC_ASRSTR_AIDEA_SHIFT                  (0U)
/*! AIDEA - Number of Data in Input Data Buffer A is Less than Threshold
 *  0b1..When AIDEA is set, the ASRC generates data input A interrupt request to the processor if ASRIER[AIDEA] = 1
 *  0b0..The threshold has been met and no data input A interrupt is generated
 */
#define ASRC_ASRSTR_AIDEA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIDEA_SHIFT)) & ASRC_ASRSTR_AIDEA_MASK)

#define ASRC_ASRSTR_AIDEB_MASK                   (0x2U)
#define ASRC_ASRSTR_AIDEB_SHIFT                  (1U)
/*! AIDEB - Number of Data in Input Data Buffer B is Less than Threshold
 *  0b1..When AIDEB is set, the ASRC generates data input B interrupt request to the processor if ASRIER[AIDEB] = 1
 *  0b0..The threshold has been met and no data input B interrupt is generated
 */
#define ASRC_ASRSTR_AIDEB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIDEB_SHIFT)) & ASRC_ASRSTR_AIDEB_MASK)

#define ASRC_ASRSTR_AIDEC_MASK                   (0x4U)
#define ASRC_ASRSTR_AIDEC_SHIFT                  (2U)
/*! AIDEC - Number of Data in Input Data Buffer C is Less than Threshold
 *  0b1..When AIDEC is set, the ASRC generates data input C interrupt request to the processor if ASRIER[AIDEC] = 1
 *  0b0..The threshold has been met and no data input C interrupt is generated
 */
#define ASRC_ASRSTR_AIDEC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIDEC_SHIFT)) & ASRC_ASRSTR_AIDEC_MASK)

#define ASRC_ASRSTR_AODFA_MASK                   (0x8U)
#define ASRC_ASRSTR_AODFA_SHIFT                  (3U)
/*! AODFA - Number of Data in Output Data Buffer A is Greater than Threshold
 *  0b1..When AODFA is set, the ASRC generates data output A interrupt request to the processor if ASRIER[ADOEA] = 1
 *  0b0..The threshold has not yet been met and no data output A interrupt is generated
 */
#define ASRC_ASRSTR_AODFA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AODFA_SHIFT)) & ASRC_ASRSTR_AODFA_MASK)

#define ASRC_ASRSTR_AODFB_MASK                   (0x10U)
#define ASRC_ASRSTR_AODFB_SHIFT                  (4U)
/*! AODFB - Number of data in Output Data Buffer B is Greater than Threshold
 *  0b1..When AODFB is set, the ASRC generates data output B interrupt request to the processor if ASRIER[ADOEB] = 1
 *  0b0..The threshold has not yet been met and no data output B interrupt is generated
 */
#define ASRC_ASRSTR_AODFB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AODFB_SHIFT)) & ASRC_ASRSTR_AODFB_MASK)

#define ASRC_ASRSTR_AODFC_MASK                   (0x20U)
#define ASRC_ASRSTR_AODFC_SHIFT                  (5U)
/*! AODFC - Number of data in Output Data Buffer C is Greater than Threshold
 *  0b1..When AODFC is set, the ASRC generates data output C interrupt request to the processor if ASRIER[ADOEC] = 1
 *  0b0..The threshold has not yet been met and no data output C interrupt is generated
 */
#define ASRC_ASRSTR_AODFC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AODFC_SHIFT)) & ASRC_ASRSTR_AODFC_MASK)

#define ASRC_ASRSTR_AOLE_MASK                    (0x40U)
#define ASRC_ASRSTR_AOLE_SHIFT                   (6U)
/*! AOLE - Overload Error Flag
 *  0b1..Task rate is too high
 *  0b0..No overload
 */
#define ASRC_ASRSTR_AOLE(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AOLE_SHIFT)) & ASRC_ASRSTR_AOLE_MASK)

#define ASRC_ASRSTR_FPWT_MASK                    (0x80U)
#define ASRC_ASRSTR_FPWT_SHIFT                   (7U)
/*! FPWT - FP is in Wait States
 *  0b0..ASRC is not in wait state
 *  0b1..ASRC is in wait state
 */
#define ASRC_ASRSTR_FPWT(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_FPWT_SHIFT)) & ASRC_ASRSTR_FPWT_MASK)

#define ASRC_ASRSTR_AIDUA_MASK                   (0x100U)
#define ASRC_ASRSTR_AIDUA_SHIFT                  (8U)
/*! AIDUA - Input Data Buffer A has Underflowed
 *  0b0..No Underflow in Input data buffer A
 *  0b1..Underflow in Input data buffer A
 */
#define ASRC_ASRSTR_AIDUA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIDUA_SHIFT)) & ASRC_ASRSTR_AIDUA_MASK)

#define ASRC_ASRSTR_AIDUB_MASK                   (0x200U)
#define ASRC_ASRSTR_AIDUB_SHIFT                  (9U)
/*! AIDUB - Input Data Buffer B has Underflowed
 *  0b0..No Underflow in Input data buffer B
 *  0b1..Underflow in Input data buffer B
 */
#define ASRC_ASRSTR_AIDUB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIDUB_SHIFT)) & ASRC_ASRSTR_AIDUB_MASK)

#define ASRC_ASRSTR_AIDUC_MASK                   (0x400U)
#define ASRC_ASRSTR_AIDUC_SHIFT                  (10U)
/*! AIDUC - Input Data Buffer C has Underflowed
 *  0b0..No Underflow in Input data buffer C
 *  0b1..Underflow in Input data buffer C
 */
#define ASRC_ASRSTR_AIDUC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIDUC_SHIFT)) & ASRC_ASRSTR_AIDUC_MASK)

#define ASRC_ASRSTR_AODOA_MASK                   (0x800U)
#define ASRC_ASRSTR_AODOA_SHIFT                  (11U)
/*! AODOA - Output Data Buffer A has Overflowed
 *  0b0..No Overflow in Output data buffer A
 *  0b1..Overflow in Output data buffer A
 */
#define ASRC_ASRSTR_AODOA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AODOA_SHIFT)) & ASRC_ASRSTR_AODOA_MASK)

#define ASRC_ASRSTR_AODOB_MASK                   (0x1000U)
#define ASRC_ASRSTR_AODOB_SHIFT                  (12U)
/*! AODOB - Output Data Buffer B has Overflowed
 *  0b0..No Overflow in Output data buffer B
 *  0b1..Overflow in Output data buffer B
 */
#define ASRC_ASRSTR_AODOB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AODOB_SHIFT)) & ASRC_ASRSTR_AODOB_MASK)

#define ASRC_ASRSTR_AODOC_MASK                   (0x2000U)
#define ASRC_ASRSTR_AODOC_SHIFT                  (13U)
/*! AODOC - Output Data Buffer C has Overflowed
 *  0b0..No Overflow in Output data buffer C
 *  0b1..Overflow in Output data buffer C
 */
#define ASRC_ASRSTR_AODOC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AODOC_SHIFT)) & ASRC_ASRSTR_AODOC_MASK)

#define ASRC_ASRSTR_AIOLA_MASK                   (0x4000U)
#define ASRC_ASRSTR_AIOLA_SHIFT                  (14U)
/*! AIOLA - Pair A Input Task Overload
 *  0b0..Pair A input task is not overloaded
 *  0b1..Pair A input task is overloaded
 */
#define ASRC_ASRSTR_AIOLA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIOLA_SHIFT)) & ASRC_ASRSTR_AIOLA_MASK)

#define ASRC_ASRSTR_AIOLB_MASK                   (0x8000U)
#define ASRC_ASRSTR_AIOLB_SHIFT                  (15U)
/*! AIOLB - Pair B Input Task Overload
 *  0b0..Pair B input task is not overloaded
 *  0b1..Pair B input task is overloaded
 */
#define ASRC_ASRSTR_AIOLB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIOLB_SHIFT)) & ASRC_ASRSTR_AIOLB_MASK)

#define ASRC_ASRSTR_AIOLC_MASK                   (0x10000U)
#define ASRC_ASRSTR_AIOLC_SHIFT                  (16U)
/*! AIOLC - Pair C Input Task Overload
 *  0b0..Pair C input task is not overloaded
 *  0b1..Pair C input task is overloaded
 */
#define ASRC_ASRSTR_AIOLC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AIOLC_SHIFT)) & ASRC_ASRSTR_AIOLC_MASK)

#define ASRC_ASRSTR_AOOLA_MASK                   (0x20000U)
#define ASRC_ASRSTR_AOOLA_SHIFT                  (17U)
/*! AOOLA - Pair A Output Task Overload
 *  0b0..Pair A output task is not overloaded
 *  0b1..Pair A output task is overloaded
 */
#define ASRC_ASRSTR_AOOLA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AOOLA_SHIFT)) & ASRC_ASRSTR_AOOLA_MASK)

#define ASRC_ASRSTR_AOOLB_MASK                   (0x40000U)
#define ASRC_ASRSTR_AOOLB_SHIFT                  (18U)
/*! AOOLB - Pair B Output Task Overload
 *  0b0..Pair B output task is not overloaded
 *  0b1..Pair B output task is overloaded
 */
#define ASRC_ASRSTR_AOOLB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AOOLB_SHIFT)) & ASRC_ASRSTR_AOOLB_MASK)

#define ASRC_ASRSTR_AOOLC_MASK                   (0x80000U)
#define ASRC_ASRSTR_AOOLC_SHIFT                  (19U)
/*! AOOLC - Pair C Output Task Overload
 *  0b0..Pair C output task is not overloaded
 *  0b1..Pair C output task is overloaded
 */
#define ASRC_ASRSTR_AOOLC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_AOOLC_SHIFT)) & ASRC_ASRSTR_AOOLC_MASK)

#define ASRC_ASRSTR_ATQOL_MASK                   (0x100000U)
#define ASRC_ASRSTR_ATQOL_SHIFT                  (20U)
/*! ATQOL - Task Queue FIFO overload
 *  0b0..Task queue FIFO logic is not overloaded
 *  0b1..Task queue FIFO logic is overloaded
 */
#define ASRC_ASRSTR_ATQOL(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_ATQOL_SHIFT)) & ASRC_ASRSTR_ATQOL_MASK)

#define ASRC_ASRSTR_DSLCNT_MASK                  (0x200000U)
#define ASRC_ASRSTR_DSLCNT_SHIFT                 (21U)
/*! DSLCNT - Digital Servo Loop (DSL) Counter Input to FIFO Ready
 *  0b0..New DSL counter information is in the process of storage into the internal ASRC FIFO
 *  0b1..New DSL counter information is stored in the internal ASRC FIFO
 */
#define ASRC_ASRSTR_DSLCNT(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASRSTR_DSLCNT_SHIFT)) & ASRC_ASRSTR_DSLCNT_MASK)
/*! @} */

/*! @name ASRPM - ASRC Parameter x */
/*! @{ */

#define ASRC_ASRPM_PARAMETER_VALUE_MASK          (0xFFFFFFU)
#define ASRC_ASRPM_PARAMETER_VALUE_SHIFT         (0U)
/*! PARAMETER_VALUE - Parameter Value */
#define ASRC_ASRPM_PARAMETER_VALUE(x)            (((uint32_t)(((uint32_t)(x)) << ASRC_ASRPM_PARAMETER_VALUE_SHIFT)) & ASRC_ASRPM_PARAMETER_VALUE_MASK)
/*! @} */

/* The count of ASRC_ASRPM */
#define ASRC_ASRPM_COUNT                         (5U)

/*! @name ASRTFR1 - ASRC Task Queue FIFO 1 */
/*! @{ */

#define ASRC_ASRTFR1_TF_BASE_MASK                (0x1FC0U)
#define ASRC_ASRTFR1_TF_BASE_SHIFT               (6U)
/*! TF_BASE - Base Address for Task Queue FIFO */
#define ASRC_ASRTFR1_TF_BASE(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRTFR1_TF_BASE_SHIFT)) & ASRC_ASRTFR1_TF_BASE_MASK)

#define ASRC_ASRTFR1_TF_FILL_MASK                (0xFE000U)
#define ASRC_ASRTFR1_TF_FILL_SHIFT               (13U)
/*! TF_FILL - Current Number of Entries in Task Queue FIFO */
#define ASRC_ASRTFR1_TF_FILL(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRTFR1_TF_FILL_SHIFT)) & ASRC_ASRTFR1_TF_FILL_MASK)
/*! @} */

/*! @name ASRCCR - ASRC Channel Counter */
/*! @{ */

#define ASRC_ASRCCR_ACIA_MASK                    (0xFU)
#define ASRC_ASRCCR_ACIA_SHIFT                   (0U)
/*! ACIA - Channel Counter for Pair A's Input FIFO */
#define ASRC_ASRCCR_ACIA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCCR_ACIA_SHIFT)) & ASRC_ASRCCR_ACIA_MASK)

#define ASRC_ASRCCR_ACIB_MASK                    (0xF0U)
#define ASRC_ASRCCR_ACIB_SHIFT                   (4U)
/*! ACIB - Channel Counter for Pair B's Input FIFO */
#define ASRC_ASRCCR_ACIB(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCCR_ACIB_SHIFT)) & ASRC_ASRCCR_ACIB_MASK)

#define ASRC_ASRCCR_ACIC_MASK                    (0xF00U)
#define ASRC_ASRCCR_ACIC_SHIFT                   (8U)
/*! ACIC - Channel Counter for Pair C's Input FIFO */
#define ASRC_ASRCCR_ACIC(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCCR_ACIC_SHIFT)) & ASRC_ASRCCR_ACIC_MASK)

#define ASRC_ASRCCR_ACOA_MASK                    (0xF000U)
#define ASRC_ASRCCR_ACOA_SHIFT                   (12U)
/*! ACOA - Channel Counter for Pair A's Output FIFO */
#define ASRC_ASRCCR_ACOA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCCR_ACOA_SHIFT)) & ASRC_ASRCCR_ACOA_MASK)

#define ASRC_ASRCCR_ACOB_MASK                    (0xF0000U)
#define ASRC_ASRCCR_ACOB_SHIFT                   (16U)
/*! ACOB - Channel Counter for Pair B's Output FIFO */
#define ASRC_ASRCCR_ACOB(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCCR_ACOB_SHIFT)) & ASRC_ASRCCR_ACOB_MASK)

#define ASRC_ASRCCR_ACOC_MASK                    (0xF00000U)
#define ASRC_ASRCCR_ACOC_SHIFT                   (20U)
/*! ACOC - Channel Counter for Pair C's Output FIFO */
#define ASRC_ASRCCR_ACOC(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRCCR_ACOC_SHIFT)) & ASRC_ASRCCR_ACOC_MASK)
/*! @} */

/*! @name ASRDIA - ASRC Data Input for Pair x */
/*! @{ */

#define ASRC_ASRDIA_DATA_MASK                    (0xFFFFFFU)
#define ASRC_ASRDIA_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define ASRC_ASRDIA_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRDIA_DATA_SHIFT)) & ASRC_ASRDIA_DATA_MASK)
/*! @} */

/*! @name ASRDOA - ASRC Data Output for Pair x */
/*! @{ */

#define ASRC_ASRDOA_DATA_MASK                    (0xFFFFFFU)
#define ASRC_ASRDOA_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define ASRC_ASRDOA_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRDOA_DATA_SHIFT)) & ASRC_ASRDOA_DATA_MASK)
/*! @} */

/*! @name ASRDIB - ASRC Data Input for Pair x */
/*! @{ */

#define ASRC_ASRDIB_DATA_MASK                    (0xFFFFFFU)
#define ASRC_ASRDIB_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define ASRC_ASRDIB_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRDIB_DATA_SHIFT)) & ASRC_ASRDIB_DATA_MASK)
/*! @} */

/*! @name ASRDOB - ASRC Data Output for Pair x */
/*! @{ */

#define ASRC_ASRDOB_DATA_MASK                    (0xFFFFFFU)
#define ASRC_ASRDOB_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define ASRC_ASRDOB_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRDOB_DATA_SHIFT)) & ASRC_ASRDOB_DATA_MASK)
/*! @} */

/*! @name ASRDIC - ASRC Data Input for Pair x */
/*! @{ */

#define ASRC_ASRDIC_DATA_MASK                    (0xFFFFFFU)
#define ASRC_ASRDIC_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define ASRC_ASRDIC_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRDIC_DATA_SHIFT)) & ASRC_ASRDIC_DATA_MASK)
/*! @} */

/*! @name ASRDOC - ASRC Data Output for Pair x */
/*! @{ */

#define ASRC_ASRDOC_DATA_MASK                    (0xFFFFFFU)
#define ASRC_ASRDOC_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define ASRC_ASRDOC_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRDOC_DATA_SHIFT)) & ASRC_ASRDOC_DATA_MASK)
/*! @} */

/*! @name ASRIDRHA - ASRC Ideal Ratio for Pair A-High Part */
/*! @{ */

#define ASRC_ASRIDRHA_IDRATIOA_H_MASK            (0xFFU)
#define ASRC_ASRIDRHA_IDRATIOA_H_SHIFT           (0U)
/*! IDRATIOA_H - Ideal Ratio A High */
#define ASRC_ASRIDRHA_IDRATIOA_H(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIDRHA_IDRATIOA_H_SHIFT)) & ASRC_ASRIDRHA_IDRATIOA_H_MASK)
/*! @} */

/*! @name ASRIDRLA - ASRC Ideal Ratio for Pair A -Low Part */
/*! @{ */

#define ASRC_ASRIDRLA_IDRATIOA_L_MASK            (0xFFFFFFU)
#define ASRC_ASRIDRLA_IDRATIOA_L_SHIFT           (0U)
/*! IDRATIOA_L - Ideal Ratio A Low */
#define ASRC_ASRIDRLA_IDRATIOA_L(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIDRLA_IDRATIOA_L_SHIFT)) & ASRC_ASRIDRLA_IDRATIOA_L_MASK)
/*! @} */

/*! @name ASRIDRHB - ASRC Ideal Ratio for Pair B-High Part */
/*! @{ */

#define ASRC_ASRIDRHB_IDRATIOB_H_MASK            (0xFFU)
#define ASRC_ASRIDRHB_IDRATIOB_H_SHIFT           (0U)
/*! IDRATIOB_H - Ideal Ratio B High */
#define ASRC_ASRIDRHB_IDRATIOB_H(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIDRHB_IDRATIOB_H_SHIFT)) & ASRC_ASRIDRHB_IDRATIOB_H_MASK)
/*! @} */

/*! @name ASRIDRLB - ASRC Ideal Ratio for Pair B-Low Part */
/*! @{ */

#define ASRC_ASRIDRLB_IDRATIOB_L_MASK            (0xFFFFFFU)
#define ASRC_ASRIDRLB_IDRATIOB_L_SHIFT           (0U)
/*! IDRATIOB_L - Ideal Ratio B Low */
#define ASRC_ASRIDRLB_IDRATIOB_L(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIDRLB_IDRATIOB_L_SHIFT)) & ASRC_ASRIDRLB_IDRATIOB_L_MASK)
/*! @} */

/*! @name ASRIDRHC - ASRC Ideal Ratio for Pair C-High Part */
/*! @{ */

#define ASRC_ASRIDRHC_IDRATIOC_H_MASK            (0xFFU)
#define ASRC_ASRIDRHC_IDRATIOC_H_SHIFT           (0U)
/*! IDRATIOC_H - Ideal Ratio C High */
#define ASRC_ASRIDRHC_IDRATIOC_H(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIDRHC_IDRATIOC_H_SHIFT)) & ASRC_ASRIDRHC_IDRATIOC_H_MASK)
/*! @} */

/*! @name ASRIDRLC - ASRC Ideal Ratio for Pair C-Low Part */
/*! @{ */

#define ASRC_ASRIDRLC_IDRATIOC_L_MASK            (0xFFFFFFU)
#define ASRC_ASRIDRLC_IDRATIOC_L_SHIFT           (0U)
/*! IDRATIOC_L - Ideal Ratio C Low */
#define ASRC_ASRIDRLC_IDRATIOC_L(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRIDRLC_IDRATIOC_L_SHIFT)) & ASRC_ASRIDRLC_IDRATIOC_L_MASK)
/*! @} */

/*! @name ASR76K - ASRC 76 kHz Period */
/*! @{ */

#define ASRC_ASR76K_ASR76K_MASK                  (0x1FFFFU)
#define ASRC_ASR76K_ASR76K_SHIFT                 (0U)
/*! ASR76K - Value for the Period of the 76 kHz Sampling Clock */
#define ASRC_ASR76K_ASR76K(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASR76K_ASR76K_SHIFT)) & ASRC_ASR76K_ASR76K_MASK)
/*! @} */

/*! @name ASR56K - ASRC 56 kHz Period */
/*! @{ */

#define ASRC_ASR56K_ASR56K_MASK                  (0x1FFFFU)
#define ASRC_ASR56K_ASR56K_SHIFT                 (0U)
/*! ASR56K - Value for the Period of the 56 kHz Sampling Clock */
#define ASRC_ASR56K_ASR56K(x)                    (((uint32_t)(((uint32_t)(x)) << ASRC_ASR56K_ASR56K_SHIFT)) & ASRC_ASR56K_ASR56K_MASK)
/*! @} */

/*! @name ASRMCRA - ASRC Misc Control for Pair A */
/*! @{ */

#define ASRC_ASRMCRA_INFIFO_THRESHOLDA_MASK      (0x3FU)
#define ASRC_ASRMCRA_INFIFO_THRESHOLDA_SHIFT     (0U)
/*! INFIFO_THRESHOLDA - Threshold for Pair A's Input FIFO per Channel */
#define ASRC_ASRMCRA_INFIFO_THRESHOLDA(x)        (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_INFIFO_THRESHOLDA_SHIFT)) & ASRC_ASRMCRA_INFIFO_THRESHOLDA_MASK)

#define ASRC_ASRMCRA_RSYNOFA_MASK                (0x400U)
#define ASRC_ASRMCRA_RSYNOFA_SHIFT               (10U)
/*! RSYNOFA - Re-sync Output FIFO Channel Counter
 *  0b1..Force ASRCCR[ACOA]=0
 *  0b0..Do not touch ASRCCR[ACOA]
 */
#define ASRC_ASRMCRA_RSYNOFA(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_RSYNOFA_SHIFT)) & ASRC_ASRMCRA_RSYNOFA_MASK)

#define ASRC_ASRMCRA_RSYNIFA_MASK                (0x800U)
#define ASRC_ASRMCRA_RSYNIFA_SHIFT               (11U)
/*! RSYNIFA - Re-sync Input FIFO Channel Counter
 *  0b1..Force ASRCCR[ACIA]=0
 *  0b0..Do not touch ASRCCR[ACIA]
 */
#define ASRC_ASRMCRA_RSYNIFA(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_RSYNIFA_SHIFT)) & ASRC_ASRMCRA_RSYNIFA_MASK)

#define ASRC_ASRMCRA_OUTFIFO_THRESHOLDA_MASK     (0x3F000U)
#define ASRC_ASRMCRA_OUTFIFO_THRESHOLDA_SHIFT    (12U)
/*! OUTFIFO_THRESHOLDA - Threshold for Pair A's Output FIFO per Channel */
#define ASRC_ASRMCRA_OUTFIFO_THRESHOLDA(x)       (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_OUTFIFO_THRESHOLDA_SHIFT)) & ASRC_ASRMCRA_OUTFIFO_THRESHOLDA_MASK)

#define ASRC_ASRMCRA_BYPASSPOLYA_MASK            (0x100000U)
#define ASRC_ASRMCRA_BYPASSPOLYA_SHIFT           (20U)
/*! BYPASSPOLYA - Bypass Polyphase Filtering for Pair A
 *  0b1..Bypass polyphase filtering.
 *  0b0..Don't bypass polyphase filtering.
 */
#define ASRC_ASRMCRA_BYPASSPOLYA(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_BYPASSPOLYA_SHIFT)) & ASRC_ASRMCRA_BYPASSPOLYA_MASK)

#define ASRC_ASRMCRA_BUFSTALLA_MASK              (0x200000U)
#define ASRC_ASRMCRA_BUFSTALLA_SHIFT             (21U)
/*! BUFSTALLA - Stall Pair A Conversion in Case of Buffer Near Empty/Full Condition
 *  0b1..Stall Pair A conversion in case of near empty/full FIFO conditions.
 *  0b0..Don't stall Pair A conversion even in case of near empty/full FIFO conditions.
 */
#define ASRC_ASRMCRA_BUFSTALLA(x)                (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_BUFSTALLA_SHIFT)) & ASRC_ASRMCRA_BUFSTALLA_MASK)

#define ASRC_ASRMCRA_EXTTHRSHA_MASK              (0x400000U)
#define ASRC_ASRMCRA_EXTTHRSHA_SHIFT             (22U)
/*! EXTTHRSHA - Use External Thresholds for FIFO Control of Pair A
 *  0b1..Use external defined thresholds.
 *  0b0..Use default thresholds.
 */
#define ASRC_ASRMCRA_EXTTHRSHA(x)                (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_EXTTHRSHA_SHIFT)) & ASRC_ASRMCRA_EXTTHRSHA_MASK)

#define ASRC_ASRMCRA_ZEROBUFA_MASK               (0x800000U)
#define ASRC_ASRMCRA_ZEROBUFA_SHIFT              (23U)
/*! ZEROBUFA - Zero Buffer A
 *  0b1..Don't zeroize the buffer
 *  0b0..Zeroize the buffer
 */
#define ASRC_ASRMCRA_ZEROBUFA(x)                 (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRA_ZEROBUFA_SHIFT)) & ASRC_ASRMCRA_ZEROBUFA_MASK)
/*! @} */

/*! @name ASRFSTA - ASRC FIFO Status for Pair A */
/*! @{ */

#define ASRC_ASRFSTA_INFIFO_FILLA_MASK           (0x7FU)
#define ASRC_ASRFSTA_INFIFO_FILLA_SHIFT          (0U)
/*! INFIFO_FILLA - Fillings for Pair A's Input FIFO per Channel */
#define ASRC_ASRFSTA_INFIFO_FILLA(x)             (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTA_INFIFO_FILLA_SHIFT)) & ASRC_ASRFSTA_INFIFO_FILLA_MASK)

#define ASRC_ASRFSTA_IAEA_MASK                   (0x800U)
#define ASRC_ASRFSTA_IAEA_SHIFT                  (11U)
/*! IAEA - Input FIFO is Near Empty for Pair A
 *  0b1..Input FIFO is near empty for Pair A
 *  0b0..Input FIFO is not near empty for Pair A
 */
#define ASRC_ASRFSTA_IAEA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTA_IAEA_SHIFT)) & ASRC_ASRFSTA_IAEA_MASK)

#define ASRC_ASRFSTA_OUTFIFO_FILLA_MASK          (0x7F000U)
#define ASRC_ASRFSTA_OUTFIFO_FILLA_SHIFT         (12U)
/*! OUTFIFO_FILLA - Fillings for Pair A's Output FIFO per Channel */
#define ASRC_ASRFSTA_OUTFIFO_FILLA(x)            (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTA_OUTFIFO_FILLA_SHIFT)) & ASRC_ASRFSTA_OUTFIFO_FILLA_MASK)

#define ASRC_ASRFSTA_OAFA_MASK                   (0x800000U)
#define ASRC_ASRFSTA_OAFA_SHIFT                  (23U)
/*! OAFA - Output FIFO is Near Full for Pair A
 *  0b1..Output FIFO is near full for Pair A
 *  0b0..Output FIFO is not near full for Pair A
 */
#define ASRC_ASRFSTA_OAFA(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTA_OAFA_SHIFT)) & ASRC_ASRFSTA_OAFA_MASK)
/*! @} */

/*! @name ASRMCRB - ASRC Misc Control for Pair B */
/*! @{ */

#define ASRC_ASRMCRB_INFIFO_THRESHOLDB_MASK      (0x3FU)
#define ASRC_ASRMCRB_INFIFO_THRESHOLDB_SHIFT     (0U)
/*! INFIFO_THRESHOLDB - Threshold for Pair B's Input FIFO per Channel */
#define ASRC_ASRMCRB_INFIFO_THRESHOLDB(x)        (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_INFIFO_THRESHOLDB_SHIFT)) & ASRC_ASRMCRB_INFIFO_THRESHOLDB_MASK)

#define ASRC_ASRMCRB_RSYNOFB_MASK                (0x400U)
#define ASRC_ASRMCRB_RSYNOFB_SHIFT               (10U)
/*! RSYNOFB - Re-sync Output FIFO Channel Counter
 *  0b1..Force ASRCCR[ACOB]=0
 *  0b0..Do not touch ASRCCR[ACOB]
 */
#define ASRC_ASRMCRB_RSYNOFB(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_RSYNOFB_SHIFT)) & ASRC_ASRMCRB_RSYNOFB_MASK)

#define ASRC_ASRMCRB_RSYNIFB_MASK                (0x800U)
#define ASRC_ASRMCRB_RSYNIFB_SHIFT               (11U)
/*! RSYNIFB - Re-sync Input FIFO Channel Counter
 *  0b1..Force ASRCCR[ACIB]=0
 *  0b0..Do not touch ASRCCR[ACIB]
 */
#define ASRC_ASRMCRB_RSYNIFB(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_RSYNIFB_SHIFT)) & ASRC_ASRMCRB_RSYNIFB_MASK)

#define ASRC_ASRMCRB_OUTFIFO_THRESHOLDB_MASK     (0x3F000U)
#define ASRC_ASRMCRB_OUTFIFO_THRESHOLDB_SHIFT    (12U)
/*! OUTFIFO_THRESHOLDB - Threshold for Pair B's Output FIFO per Channel */
#define ASRC_ASRMCRB_OUTFIFO_THRESHOLDB(x)       (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_OUTFIFO_THRESHOLDB_SHIFT)) & ASRC_ASRMCRB_OUTFIFO_THRESHOLDB_MASK)

#define ASRC_ASRMCRB_BYPASSPOLYB_MASK            (0x100000U)
#define ASRC_ASRMCRB_BYPASSPOLYB_SHIFT           (20U)
/*! BYPASSPOLYB - Bypass Polyphase Filtering for Pair B
 *  0b1..Bypass polyphase filtering.
 *  0b0..Don't bypass polyphase filtering.
 */
#define ASRC_ASRMCRB_BYPASSPOLYB(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_BYPASSPOLYB_SHIFT)) & ASRC_ASRMCRB_BYPASSPOLYB_MASK)

#define ASRC_ASRMCRB_BUFSTALLB_MASK              (0x200000U)
#define ASRC_ASRMCRB_BUFSTALLB_SHIFT             (21U)
/*! BUFSTALLB - Stall Pair B Conversion in Case of Buffer Near Empty/Full Condition
 *  0b1..Stall Pair B conversion in case of near empty/full FIFO conditions.
 *  0b0..Don't stall Pair B conversion even in case of near empty/full FIFO conditions.
 */
#define ASRC_ASRMCRB_BUFSTALLB(x)                (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_BUFSTALLB_SHIFT)) & ASRC_ASRMCRB_BUFSTALLB_MASK)

#define ASRC_ASRMCRB_EXTTHRSHB_MASK              (0x400000U)
#define ASRC_ASRMCRB_EXTTHRSHB_SHIFT             (22U)
/*! EXTTHRSHB - Use External Thresholds for FIFO Control of Pair B
 *  0b1..Use external defined thresholds.
 *  0b0..Use default thresholds.
 */
#define ASRC_ASRMCRB_EXTTHRSHB(x)                (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_EXTTHRSHB_SHIFT)) & ASRC_ASRMCRB_EXTTHRSHB_MASK)

#define ASRC_ASRMCRB_ZEROBUFB_MASK               (0x800000U)
#define ASRC_ASRMCRB_ZEROBUFB_SHIFT              (23U)
/*! ZEROBUFB - Zero Buffer B
 *  0b1..Don't zeroize the buffer
 *  0b0..Zeroize the buffer
 */
#define ASRC_ASRMCRB_ZEROBUFB(x)                 (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRB_ZEROBUFB_SHIFT)) & ASRC_ASRMCRB_ZEROBUFB_MASK)
/*! @} */

/*! @name ASRFSTB - ASRC FIFO Status for Pair B */
/*! @{ */

#define ASRC_ASRFSTB_INFIFO_FILLB_MASK           (0x7FU)
#define ASRC_ASRFSTB_INFIFO_FILLB_SHIFT          (0U)
/*! INFIFO_FILLB - Fillings for Pair B's Input FIFO per Channel */
#define ASRC_ASRFSTB_INFIFO_FILLB(x)             (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTB_INFIFO_FILLB_SHIFT)) & ASRC_ASRFSTB_INFIFO_FILLB_MASK)

#define ASRC_ASRFSTB_IAEB_MASK                   (0x800U)
#define ASRC_ASRFSTB_IAEB_SHIFT                  (11U)
/*! IAEB - Input FIFO is Near Empty for Pair B
 *  0b1..Input FIFO is near empty for Pair B
 *  0b0..Input FIFO is not near empty for Pair B
 */
#define ASRC_ASRFSTB_IAEB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTB_IAEB_SHIFT)) & ASRC_ASRFSTB_IAEB_MASK)

#define ASRC_ASRFSTB_OUTFIFO_FILLB_MASK          (0x7F000U)
#define ASRC_ASRFSTB_OUTFIFO_FILLB_SHIFT         (12U)
/*! OUTFIFO_FILLB - Fillings for Pair B's Output FIFO per Channel */
#define ASRC_ASRFSTB_OUTFIFO_FILLB(x)            (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTB_OUTFIFO_FILLB_SHIFT)) & ASRC_ASRFSTB_OUTFIFO_FILLB_MASK)

#define ASRC_ASRFSTB_OAFB_MASK                   (0x800000U)
#define ASRC_ASRFSTB_OAFB_SHIFT                  (23U)
/*! OAFB - Output FIFO is Near Full for Pair B
 *  0b1..Output FIFO is near full for Pair B
 *  0b0..Output FIFO is not near full for Pair B
 */
#define ASRC_ASRFSTB_OAFB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTB_OAFB_SHIFT)) & ASRC_ASRFSTB_OAFB_MASK)
/*! @} */

/*! @name ASRMCRC - ASRC Misc Control for Pair C */
/*! @{ */

#define ASRC_ASRMCRC_INFIFO_THRESHOLDC_MASK      (0x3FU)
#define ASRC_ASRMCRC_INFIFO_THRESHOLDC_SHIFT     (0U)
/*! INFIFO_THRESHOLDC - Threshold for Pair C's Input FIFO per Channel */
#define ASRC_ASRMCRC_INFIFO_THRESHOLDC(x)        (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_INFIFO_THRESHOLDC_SHIFT)) & ASRC_ASRMCRC_INFIFO_THRESHOLDC_MASK)

#define ASRC_ASRMCRC_RSYNOFC_MASK                (0x400U)
#define ASRC_ASRMCRC_RSYNOFC_SHIFT               (10U)
/*! RSYNOFC - Re-sync Output FIFO Channel Counter
 *  0b1..Force ASRCCR[ACOC]=0
 *  0b0..Do not touch ASRCCR[ACOC]
 */
#define ASRC_ASRMCRC_RSYNOFC(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_RSYNOFC_SHIFT)) & ASRC_ASRMCRC_RSYNOFC_MASK)

#define ASRC_ASRMCRC_RSYNIFC_MASK                (0x800U)
#define ASRC_ASRMCRC_RSYNIFC_SHIFT               (11U)
/*! RSYNIFC - Re-sync Input FIFO Channel Counter
 *  0b1..Force ASRCCR[ACIC]=0
 *  0b0..Do not touch ASRCCR[ACIC]
 */
#define ASRC_ASRMCRC_RSYNIFC(x)                  (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_RSYNIFC_SHIFT)) & ASRC_ASRMCRC_RSYNIFC_MASK)

#define ASRC_ASRMCRC_OUTFIFO_THRESHOLDC_MASK     (0x3F000U)
#define ASRC_ASRMCRC_OUTFIFO_THRESHOLDC_SHIFT    (12U)
/*! OUTFIFO_THRESHOLDC - Threshold for Pair C's Output FIFO per Channel */
#define ASRC_ASRMCRC_OUTFIFO_THRESHOLDC(x)       (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_OUTFIFO_THRESHOLDC_SHIFT)) & ASRC_ASRMCRC_OUTFIFO_THRESHOLDC_MASK)

#define ASRC_ASRMCRC_BYPASSPOLYC_MASK            (0x100000U)
#define ASRC_ASRMCRC_BYPASSPOLYC_SHIFT           (20U)
/*! BYPASSPOLYC - Bypass Polyphase Filtering for Pair C
 *  0b1..Bypass polyphase filtering.
 *  0b0..Don't bypass polyphase filtering.
 */
#define ASRC_ASRMCRC_BYPASSPOLYC(x)              (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_BYPASSPOLYC_SHIFT)) & ASRC_ASRMCRC_BYPASSPOLYC_MASK)

#define ASRC_ASRMCRC_BUFSTALLC_MASK              (0x200000U)
#define ASRC_ASRMCRC_BUFSTALLC_SHIFT             (21U)
/*! BUFSTALLC - Stall Pair C Conversion in Case of Buffer Near Empty/Full Condition
 *  0b1..Stall Pair C conversion in case of near empty/full FIFO conditions.
 *  0b0..Don't stall Pair C conversion even in case of near empty/full FIFO conditions.
 */
#define ASRC_ASRMCRC_BUFSTALLC(x)                (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_BUFSTALLC_SHIFT)) & ASRC_ASRMCRC_BUFSTALLC_MASK)

#define ASRC_ASRMCRC_EXTTHRSHC_MASK              (0x400000U)
#define ASRC_ASRMCRC_EXTTHRSHC_SHIFT             (22U)
/*! EXTTHRSHC - Use External Thresholds for FIFO Control of Pair C
 *  0b1..Use external defined thresholds.
 *  0b0..Use default thresholds.
 */
#define ASRC_ASRMCRC_EXTTHRSHC(x)                (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_EXTTHRSHC_SHIFT)) & ASRC_ASRMCRC_EXTTHRSHC_MASK)

#define ASRC_ASRMCRC_ZEROBUFC_MASK               (0x800000U)
#define ASRC_ASRMCRC_ZEROBUFC_SHIFT              (23U)
/*! ZEROBUFC - Zero Buffer C
 *  0b1..Don't zeroize the buffer
 *  0b0..Zeroize the buffer
 */
#define ASRC_ASRMCRC_ZEROBUFC(x)                 (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCRC_ZEROBUFC_SHIFT)) & ASRC_ASRMCRC_ZEROBUFC_MASK)
/*! @} */

/*! @name ASRFSTC - ASRC FIFO Status for Pair C */
/*! @{ */

#define ASRC_ASRFSTC_INFIFO_FILLC_MASK           (0x7FU)
#define ASRC_ASRFSTC_INFIFO_FILLC_SHIFT          (0U)
/*! INFIFO_FILLC - Fillings for Pair C's Input FIFO per Channel */
#define ASRC_ASRFSTC_INFIFO_FILLC(x)             (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTC_INFIFO_FILLC_SHIFT)) & ASRC_ASRFSTC_INFIFO_FILLC_MASK)

#define ASRC_ASRFSTC_IAEC_MASK                   (0x800U)
#define ASRC_ASRFSTC_IAEC_SHIFT                  (11U)
/*! IAEC - Input FIFO is Near Empty for Pair C
 *  0b1..Input FIFO is near empty for Pair C
 *  0b0..Input FIFO is not near empty for Pair C
 */
#define ASRC_ASRFSTC_IAEC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTC_IAEC_SHIFT)) & ASRC_ASRFSTC_IAEC_MASK)

#define ASRC_ASRFSTC_OUTFIFO_FILLC_MASK          (0x7F000U)
#define ASRC_ASRFSTC_OUTFIFO_FILLC_SHIFT         (12U)
/*! OUTFIFO_FILLC - Fillings for Pair C's Output FIFO per Channel */
#define ASRC_ASRFSTC_OUTFIFO_FILLC(x)            (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTC_OUTFIFO_FILLC_SHIFT)) & ASRC_ASRFSTC_OUTFIFO_FILLC_MASK)

#define ASRC_ASRFSTC_OAFC_MASK                   (0x800000U)
#define ASRC_ASRFSTC_OAFC_SHIFT                  (23U)
/*! OAFC - Output FIFO is Near Full for Pair C
 *  0b1..Output FIFO is near full for Pair C
 *  0b0..Output FIFO is not near full for Pair C
 */
#define ASRC_ASRFSTC_OAFC(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRFSTC_OAFC_SHIFT)) & ASRC_ASRFSTC_OAFC_MASK)
/*! @} */

/*! @name ASRMCR1 - ASRC Misc Control 1 for Pair X */
/*! @{ */

#define ASRC_ASRMCR1_OW16_MASK                   (0x1U)
#define ASRC_ASRMCR1_OW16_SHIFT                  (0U)
/*! OW16 - Bit Width Option of the Output FIFO
 *  0b1..16-bit output data
 *  0b0..24-bit output data
 */
#define ASRC_ASRMCR1_OW16(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCR1_OW16_SHIFT)) & ASRC_ASRMCR1_OW16_MASK)

#define ASRC_ASRMCR1_OSGN_MASK                   (0x2U)
#define ASRC_ASRMCR1_OSGN_SHIFT                  (1U)
/*! OSGN - Sign Extension Option of the Output FIFO
 *  0b1..Sign extension
 *  0b0..No sign extension
 */
#define ASRC_ASRMCR1_OSGN(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCR1_OSGN_SHIFT)) & ASRC_ASRMCR1_OSGN_MASK)

#define ASRC_ASRMCR1_OMSB_MASK                   (0x4U)
#define ASRC_ASRMCR1_OMSB_SHIFT                  (2U)
/*! OMSB - Data Alignment of the Output FIFO
 *  0b1..MSB aligned
 *  0b0..LSB aligned
 */
#define ASRC_ASRMCR1_OMSB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCR1_OMSB_SHIFT)) & ASRC_ASRMCR1_OMSB_MASK)

#define ASRC_ASRMCR1_IMSB_MASK                   (0x100U)
#define ASRC_ASRMCR1_IMSB_SHIFT                  (8U)
/*! IMSB - Data Alignment of the Input FIFO
 *  0b1..MSB aligned
 *  0b0..LSB aligned
 */
#define ASRC_ASRMCR1_IMSB(x)                     (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCR1_IMSB_SHIFT)) & ASRC_ASRMCR1_IMSB_MASK)

#define ASRC_ASRMCR1_IWD_MASK                    (0x600U)
#define ASRC_ASRMCR1_IWD_SHIFT                   (9U)
/*! IWD - Data Width of the Input FIFO
 *  0b00..24-bit audio data.
 *  0b01..16-bit audio data.
 *  0b10..8-bit audio data.
 *  0b11..Reserved.
 */
#define ASRC_ASRMCR1_IWD(x)                      (((uint32_t)(((uint32_t)(x)) << ASRC_ASRMCR1_IWD_SHIFT)) & ASRC_ASRMCR1_IWD_MASK)
/*! @} */

/* The count of ASRC_ASRMCR1 */
#define ASRC_ASRMCR1_COUNT                       (3U)


/*!
 * @}
 */ /* end of group ASRC_Register_Masks */


/* ASRC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ASRC base address */
  #define ASRC_BASE                                (0x529A0000u)
  /** Peripheral ASRC base address */
  #define ASRC_BASE_NS                             (0x429A0000u)
  /** Peripheral ASRC base pointer */
  #define ASRC                                     ((ASRC_Type *)ASRC_BASE)
  /** Peripheral ASRC base pointer */
  #define ASRC_NS                                  ((ASRC_Type *)ASRC_BASE_NS)
  /** Array initializer of ASRC peripheral base addresses */
  #define ASRC_BASE_ADDRS                          { ASRC_BASE }
  /** Array initializer of ASRC peripheral base pointers */
  #define ASRC_BASE_PTRS                           { ASRC }
  /** Array initializer of ASRC peripheral base addresses */
  #define ASRC_BASE_ADDRS_NS                       { ASRC_BASE_NS }
  /** Array initializer of ASRC peripheral base pointers */
  #define ASRC_BASE_PTRS_NS                        { ASRC_NS }
#else
  /** Peripheral ASRC base address */
  #define ASRC_BASE                                (0x429A0000u)
  /** Peripheral ASRC base pointer */
  #define ASRC                                     ((ASRC_Type *)ASRC_BASE)
  /** Array initializer of ASRC peripheral base addresses */
  #define ASRC_BASE_ADDRS                          { ASRC_BASE }
  /** Array initializer of ASRC peripheral base pointers */
  #define ASRC_BASE_PTRS                           { ASRC }
#endif
/** Interrupt vectors for the ASRC peripheral type */
#define ASRC_IRQS                                { ASRC_IRQn }

/*!
 * @}
 */ /* end of group ASRC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- AXBS Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup AXBS_Peripheral_Access_Layer AXBS Peripheral Access Layer
 * @{
 */

/** AXBS - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x100 */
    __IO uint32_t PRS;                               /**< Priority Slave Registers, array offset: 0x0, array step: 0x100 */
         uint8_t RESERVED_0[12];
    __IO uint32_t CRS;                               /**< Control Register, array offset: 0x10, array step: 0x100 */
         uint8_t RESERVED_1[236];
  } SLAVE[8];
} AXBS_Type;

/* ----------------------------------------------------------------------------
   -- AXBS Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup AXBS_Register_Masks AXBS Register Masks
 * @{
 */

/*! @name PRS - Priority Slave Registers */
/*! @{ */

#define AXBS_PRS_M0_MASK                         (0x7U)
#define AXBS_PRS_M0_SHIFT                        (0U)
/*! M0 - Master 0 Priority
 *  0b000..This master has level 1 or highest priority when accessing the slave port.
 *  0b001..This master has level 2 priority when accessing the slave port.
 *  0b010..This master has level 3 priority when accessing the slave port.
 *  0b011..This master has level 4 priority when accessing the slave port.
 *  0b100..This master has level 5 priority when accessing the slave port.
 *  0b101..This master has level 6 priority when accessing the slave port.
 *  0b110..This master has level 7 priority when accessing the slave port.
 *  0b111..This master has level 8 or the lowest priority when accessing the slave port.
 */
#define AXBS_PRS_M0(x)                           (((uint32_t)(((uint32_t)(x)) << AXBS_PRS_M0_SHIFT)) & AXBS_PRS_M0_MASK)

#define AXBS_PRS_M1_MASK                         (0x70U)
#define AXBS_PRS_M1_SHIFT                        (4U)
/*! M1 - Master 1 Priority
 *  0b000..This master has level 1 or highest priority when accessing the slave port.
 *  0b001..This master has level 2 priority when accessing the slave port.
 *  0b010..This master has level 3 priority when accessing the slave port.
 *  0b011..This master has level 4 priority when accessing the slave port.
 *  0b100..This master has level 5 priority when accessing the slave port.
 *  0b101..This master has level 6 priority when accessing the slave port.
 *  0b110..This master has level 7 priority when accessing the slave port.
 *  0b111..This master has level 8 or lowest priority when accessing the slave port.
 */
#define AXBS_PRS_M1(x)                           (((uint32_t)(((uint32_t)(x)) << AXBS_PRS_M1_SHIFT)) & AXBS_PRS_M1_MASK)

#define AXBS_PRS_M2_MASK                         (0x700U)
#define AXBS_PRS_M2_SHIFT                        (8U)
/*! M2 - Master 2 Priority
 *  0b000..This master has level 1 or highest priority when accessing the slave port.
 *  0b001..This master has level 2 priority when accessing the slave port.
 *  0b010..This master has level 3 priority when accessing the slave port.
 *  0b011..This master has level 4 priority when accessing the slave port.
 *  0b100..This master has level 5 priority when accessing the slave port.
 *  0b101..This master has level 6 priority when accessing the slave port.
 *  0b110..This master has level 7 priority when accessing the slave port.
 *  0b111..This master has level 8the or lowest priority when accessing the slave port.
 */
#define AXBS_PRS_M2(x)                           (((uint32_t)(((uint32_t)(x)) << AXBS_PRS_M2_SHIFT)) & AXBS_PRS_M2_MASK)

#define AXBS_PRS_M3_MASK                         (0x7000U)
#define AXBS_PRS_M3_SHIFT                        (12U)
/*! M3 - Master 3 Priority
 *  0b000..This master has level 1 or highest priority when accessing the slave port.
 *  0b001..This master has level 2 priority when accessing the slave port.
 *  0b010..This master has level 3 priority when accessing the slave port.
 *  0b011..This master has level 4 priority when accessing the slave port.
 *  0b100..This master has level 5 priority when accessing the slave port.
 *  0b101..This master has level 6 priority when accessing the slave port.
 *  0b110..This master has level 7 priority when accessing the slave port.
 *  0b111..This master has level 8the or lowest priority when accessing the slave port.
 */
#define AXBS_PRS_M3(x)                           (((uint32_t)(((uint32_t)(x)) << AXBS_PRS_M3_SHIFT)) & AXBS_PRS_M3_MASK)

#define AXBS_PRS_M4_MASK                         (0x70000U)
#define AXBS_PRS_M4_SHIFT                        (16U)
/*! M4 - Master 4 Priority
 *  0b000..This master has level 1 or highest priority when accessing the slave port.
 *  0b001..This master has level 2 priority when accessing the slave port.
 *  0b010..This master has level 3 priority when accessing the slave port.
 *  0b011..This master has level 4 priority when accessing the slave port.
 *  0b100..This master has level 5 priority when accessing the slave port.
 *  0b101..This master has level 6 priority when accessing the slave port.
 *  0b110..This master has level 7 priority when accessing the slave port.
 *  0b111..This master has level 8 or lowest priority when accessing the slave port.
 */
#define AXBS_PRS_M4(x)                           (((uint32_t)(((uint32_t)(x)) << AXBS_PRS_M4_SHIFT)) & AXBS_PRS_M4_MASK)

#define AXBS_PRS_M5_MASK                         (0x700000U)
#define AXBS_PRS_M5_SHIFT                        (20U)
/*! M5 - Master 5 Priority
 *  0b000..This master has level 1 or highest priority when accessing the slave port.
 *  0b001..This master has level 2 priority when accessing the slave port.
 *  0b010..This master has level 3 priority when accessing the slave port.
 *  0b011..This master has level 4 priority when accessing the slave port.
 *  0b100..This master has level 5 priority when accessing the slave port.
 *  0b101..This master has level 6 priority when accessing the slave port.
 *  0b110..This master has level 7 priority when accessing the slave port.
 *  0b111..This master has level 8 or lowest priority when accessing the slave port.
 */
#define AXBS_PRS_M5(x)                           (((uint32_t)(((uint32_t)(x)) << AXBS_PRS_M5_SHIFT)) & AXBS_PRS_M5_MASK)
/*! @} */

/* The count of AXBS_PRS */
#define AXBS_PRS_COUNT                           (8U)

/*! @name CRS - Control Register */
/*! @{ */

#define AXBS_CRS_PARK_MASK                       (0x7U)
#define AXBS_CRS_PARK_SHIFT                      (0U)
/*! PARK - Park
 *  0b000..Park on master port M0
 *  0b001..Park on master port M1
 *  0b010..Park on master port M2
 *  0b011..Park on master port M3
 *  0b100..Park on master port M4
 *  0b101..Park on master port M5
 *  0b110..Park on master port M6
 *  0b111..Park on master port M7
 */
#define AXBS_CRS_PARK(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_PARK_SHIFT)) & AXBS_CRS_PARK_MASK)

#define AXBS_CRS_PCTL_MASK                       (0x30U)
#define AXBS_CRS_PCTL_SHIFT                      (4U)
/*! PCTL - Parking Control
 *  0b00..When no master makes a request, the arbiter parks the slave port on the master port defined by the PARK bit field.
 *  0b01..When no master makes a request, the arbiter parks the slave port on the last master to be in control of the slave port.
 *  0b10..Low-power park. When no master makes a request, the slave port is not parked on a master and the arbiter
 *        drives all outputs to a constant safe state.
 *  0b11..Reserved
 */
#define AXBS_CRS_PCTL(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_PCTL_SHIFT)) & AXBS_CRS_PCTL_MASK)

#define AXBS_CRS_ARB_MASK                        (0x300U)
#define AXBS_CRS_ARB_SHIFT                       (8U)
/*! ARB - Arbitration Mode
 *  0b00..Fixed priority
 *  0b01..Round-robin (rotating) priority
 *  0b10..Reserved
 *  0b11..Reserved
 */
#define AXBS_CRS_ARB(x)                          (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_ARB_SHIFT)) & AXBS_CRS_ARB_MASK)

#define AXBS_CRS_HPE0_MASK                       (0x10000U)
#define AXBS_CRS_HPE0_SHIFT                      (16U)
/*! HPE0 - High Priority Elevation 0
 *  0b0..Master high-priority elevation for master 0. is disabled on this slave port.
 *  0b1..Master high-priority elevation for master 0. is enabled on this slave port.
 */
#define AXBS_CRS_HPE0(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_HPE0_SHIFT)) & AXBS_CRS_HPE0_MASK)

#define AXBS_CRS_HPE1_MASK                       (0x20000U)
#define AXBS_CRS_HPE1_SHIFT                      (17U)
/*! HPE1 - High Priority Elevation 1
 *  0b0..Master high-priority elevation for master 1. is disabled on this slave port.
 *  0b1..Master high-priority elevation for master 1. is enabled on this slave port.
 */
#define AXBS_CRS_HPE1(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_HPE1_SHIFT)) & AXBS_CRS_HPE1_MASK)

#define AXBS_CRS_HPE2_MASK                       (0x40000U)
#define AXBS_CRS_HPE2_SHIFT                      (18U)
/*! HPE2 - High Priority Elevation 2
 *  0b0..Master high-priority elevation for master 2. is disabled on this slave port.
 *  0b1..Master high-priority elevation for master 2. is enabled on this slave port.
 */
#define AXBS_CRS_HPE2(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_HPE2_SHIFT)) & AXBS_CRS_HPE2_MASK)

#define AXBS_CRS_HPE3_MASK                       (0x80000U)
#define AXBS_CRS_HPE3_SHIFT                      (19U)
/*! HPE3 - High Priority Elevation 3
 *  0b0..Master high-priority elevation for master 3. is disabled on this slave port.
 *  0b1..Master high-priority elevation for master 3. is enabled on this slave port.
 */
#define AXBS_CRS_HPE3(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_HPE3_SHIFT)) & AXBS_CRS_HPE3_MASK)

#define AXBS_CRS_HPE4_MASK                       (0x100000U)
#define AXBS_CRS_HPE4_SHIFT                      (20U)
/*! HPE4 - High Priority Elevation 4
 *  0b0..Master high-priority elevation for master 4. is disabled on this slave port.
 *  0b1..Master high-priority elevation for master 4. is enabled on this slave port.
 */
#define AXBS_CRS_HPE4(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_HPE4_SHIFT)) & AXBS_CRS_HPE4_MASK)

#define AXBS_CRS_HPE5_MASK                       (0x200000U)
#define AXBS_CRS_HPE5_SHIFT                      (21U)
/*! HPE5 - High Priority Elevation 5
 *  0b0..Master high-priority elevation for master 5. is disabled on this slave port.
 *  0b1..Master high-priority elevation for master 5. is enabled on this slave port.
 */
#define AXBS_CRS_HPE5(x)                         (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_HPE5_SHIFT)) & AXBS_CRS_HPE5_MASK)

#define AXBS_CRS_RO_MASK                         (0x80000000U)
#define AXBS_CRS_RO_SHIFT                        (31U)
/*! RO - Read Only
 *  0b0..The CRSn and PRSn registers are writeable
 *  0b1..The CRSn and PRSn registers are read-only and cannot be written (attempted writes have no effect on the
 *       registers and result in a bus error response).
 */
#define AXBS_CRS_RO(x)                           (((uint32_t)(((uint32_t)(x)) << AXBS_CRS_RO_SHIFT)) & AXBS_CRS_RO_MASK)
/*! @} */

/* The count of AXBS_CRS */
#define AXBS_CRS_COUNT                           (8U)


/*!
 * @}
 */ /* end of group AXBS_Register_Masks */


/* AXBS - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral AXBS base address */
  #define AXBS_BASE                                (0x54510000u)
  /** Peripheral AXBS base address */
  #define AXBS_BASE_NS                             (0x44510000u)
  /** Peripheral AXBS base pointer */
  #define AXBS                                     ((AXBS_Type *)AXBS_BASE)
  /** Peripheral AXBS base pointer */
  #define AXBS_NS                                  ((AXBS_Type *)AXBS_BASE_NS)
  /** Array initializer of AXBS peripheral base addresses */
  #define AXBS_BASE_ADDRS                          { AXBS_BASE }
  /** Array initializer of AXBS peripheral base pointers */
  #define AXBS_BASE_PTRS                           { AXBS }
  /** Array initializer of AXBS peripheral base addresses */
  #define AXBS_BASE_ADDRS_NS                       { AXBS_BASE_NS }
  /** Array initializer of AXBS peripheral base pointers */
  #define AXBS_BASE_PTRS_NS                        { AXBS_NS }
#else
  /** Peripheral AXBS base address */
  #define AXBS_BASE                                (0x44510000u)
  /** Peripheral AXBS base pointer */
  #define AXBS                                     ((AXBS_Type *)AXBS_BASE)
  /** Array initializer of AXBS peripheral base addresses */
  #define AXBS_BASE_ADDRS                          { AXBS_BASE }
  /** Array initializer of AXBS peripheral base pointers */
  #define AXBS_BASE_PTRS                           { AXBS }
#endif

/*!
 * @}
 */ /* end of group AXBS_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- BBNSM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BBNSM_Peripheral_Access_Layer BBNSM Peripheral Access Layer
 * @{
 */

/** BBNSM - Register Layout Typedef */
typedef struct {
  __I  uint32_t BBNSM_VID;                         /**< BBNSM Version ID Register, offset: 0x0 */
  __I  uint32_t BBNSM_FEATURES;                    /**< BBNSM Features Register, offset: 0x4 */
  __IO uint32_t BBNSM_CTRL;                        /**< BBNSM Control Register, offset: 0x8 */
       uint8_t RESERVED_0[4];
  __IO uint32_t BBNSM_INT_EN;                      /**< BBNSM Interrupt Enable Register, offset: 0x10 */
  __IO uint32_t BBNSM_EVENTS;                      /**< BBNSM Events Register, offset: 0x14 */
       uint8_t RESERVED_1[12];
  __IO uint32_t BBNSM_PAD_CTRL;                    /**< BBNSM External Pad Control Register, offset: 0x24 */
       uint8_t RESERVED_2[24];
  __IO uint32_t BBNSM_RTC_LS;                      /**< BBNSM Real-Time Counter LS Register, offset: 0x40 */
  __IO uint32_t BBNSM_RTC_MS;                      /**< BBNSM Real-Time Counter MS Register, offset: 0x44 */
       uint8_t RESERVED_3[8];
  __IO uint32_t BBNSM_TA;                          /**< BBNSM Time Alarm Register, offset: 0x50 */
       uint8_t RESERVED_4[684];
  __IO uint32_t GPR[8];                            /**< General Purpose Register Word 0..General Purpose Register Word 7, array offset: 0x300, array step: 0x4 */
} BBNSM_Type;

/* ----------------------------------------------------------------------------
   -- BBNSM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BBNSM_Register_Masks BBNSM Register Masks
 * @{
 */

/*! @name BBNSM_VID - BBNSM Version ID Register */
/*! @{ */

#define BBNSM_BBNSM_VID_BBNSM_IPID_MASK          (0xFFU)
#define BBNSM_BBNSM_VID_BBNSM_IPID_SHIFT         (0U)
/*! BBNSM_IPID - BBNSM IP ID */
#define BBNSM_BBNSM_VID_BBNSM_IPID(x)            (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_VID_BBNSM_IPID_SHIFT)) & BBNSM_BBNSM_VID_BBNSM_IPID_MASK)

#define BBNSM_BBNSM_VID_BBNSM_REV_MASK           (0xFF00U)
#define BBNSM_BBNSM_VID_BBNSM_REV_SHIFT          (8U)
/*! BBNSM_REV - BBNSM Revision */
#define BBNSM_BBNSM_VID_BBNSM_REV(x)             (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_VID_BBNSM_REV_SHIFT)) & BBNSM_BBNSM_VID_BBNSM_REV_MASK)

#define BBNSM_BBNSM_VID_BBNSM_VID_MASK           (0xFF0000U)
#define BBNSM_BBNSM_VID_BBNSM_VID_SHIFT          (16U)
/*! BBNSM_VID - BBNSM Version ID */
#define BBNSM_BBNSM_VID_BBNSM_VID(x)             (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_VID_BBNSM_VID_SHIFT)) & BBNSM_BBNSM_VID_BBNSM_VID_MASK)
/*! @} */

/*! @name BBNSM_FEATURES - BBNSM Features Register */
/*! @{ */

#define BBNSM_BBNSM_FEATURES_GPR_SZ_MASK         (0xFCU)
#define BBNSM_BBNSM_FEATURES_GPR_SZ_SHIFT        (2U)
/*! GPR_SZ - GPR Register Array Size
 *  0b000000..This version of BBNSM does not implement a general-purpose register array.
 *  *..The number of 32-bit words implemented in the general-purpose register array.
 */
#define BBNSM_BBNSM_FEATURES_GPR_SZ(x)           (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_FEATURES_GPR_SZ_SHIFT)) & BBNSM_BBNSM_FEATURES_GPR_SZ_MASK)
/*! @} */

/*! @name BBNSM_CTRL - BBNSM Control Register */
/*! @{ */

#define BBNSM_BBNSM_CTRL_RTC_EN_MASK             (0x3U)
#define BBNSM_BBNSM_CTRL_RTC_EN_SHIFT            (0U)
/*! RTC_EN - Real-Time Counter Enable
 *  0b01..Disable the real-time counter.
 *  0b10..Enable the real-time counter.
 */
#define BBNSM_BBNSM_CTRL_RTC_EN(x)               (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_RTC_EN_SHIFT)) & BBNSM_BBNSM_CTRL_RTC_EN_MASK)

#define BBNSM_BBNSM_CTRL_TA_EN_MASK              (0xCU)
#define BBNSM_BBNSM_CTRL_TA_EN_SHIFT             (2U)
/*! TA_EN - Time Alarm Enable
 *  0b01..Disable the time alarm.
 *  0b10..Enable the time alarm. A time alarm event occurs if the value in the real-time counter register is equal
 *        to the value in the time alarm register.
 */
#define BBNSM_BBNSM_CTRL_TA_EN(x)                (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_TA_EN_SHIFT)) & BBNSM_BBNSM_CTRL_TA_EN_MASK)

#define BBNSM_BBNSM_CTRL_CAL_EN_MASK             (0x10U)
#define BBNSM_BBNSM_CTRL_CAL_EN_SHIFT            (4U)
/*! CAL_EN - Calibration Enable
 *  0b0..RTC Time calibration is disabled.
 *  0b1..RTC Time calibration is enabled.
 */
#define BBNSM_BBNSM_CTRL_CAL_EN(x)               (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_CAL_EN_SHIFT)) & BBNSM_BBNSM_CTRL_CAL_EN_MASK)

#define BBNSM_BBNSM_CTRL_CAL_VAL_MASK            (0x1F00U)
#define BBNSM_BBNSM_CTRL_CAL_VAL_SHIFT           (8U)
/*! CAL_VAL - Calibration Value
 *  0b01111..+15 counts per each 32768 ticks of the counter clock.
 *  0b00010..+2 counts per each 32768 ticks of the counter clock.
 *  0b00001..+1 counts per each 32768 ticks of the counter clock.
 *  0b00000..+0 counts per each 32768 ticks of the counter clock.
 *  0b11111..-1 counts per each 32768 ticks of the counter clock.
 *  0b11110..-2 counts per each 32768 ticks of the counter clock.
 *  0b10001..-15 counts per each 32768 ticks of the counter clock.
 *  0b10000..-16 counts per each 32768 ticks of the counter clock.
 */
#define BBNSM_BBNSM_CTRL_CAL_VAL(x)              (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_CAL_VAL_SHIFT)) & BBNSM_BBNSM_CTRL_CAL_VAL_MASK)

#define BBNSM_BBNSM_CTRL_BTN_TIMEOUT_MASK        (0x30000U)
#define BBNSM_BBNSM_CTRL_BTN_TIMEOUT_SHIFT       (16U)
/*! BTN_TIMEOUT - Button Press Timeout
 *  0b00..5 seconds.
 *  0b01..10 seconds.
 *  0b10..15 seconds.
 *  0b11..Timeout disabled. Long button presses will not request a power down.
 */
#define BBNSM_BBNSM_CTRL_BTN_TIMEOUT(x)          (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_BTN_TIMEOUT_SHIFT)) & BBNSM_BBNSM_CTRL_BTN_TIMEOUT_MASK)

#define BBNSM_BBNSM_CTRL_DEBOUNCE_MASK           (0xC0000U)
#define BBNSM_BBNSM_CTRL_DEBOUNCE_SHIFT          (18U)
/*! DEBOUNCE - Debounce Time
 *  0b00..50 milliseconds.
 *  0b01..100 milliseconds.
 *  0b10..500 milliseconds.
 *  0b11..0 milliseconds.
 */
#define BBNSM_BBNSM_CTRL_DEBOUNCE(x)             (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_DEBOUNCE_SHIFT)) & BBNSM_BBNSM_CTRL_DEBOUNCE_MASK)

#define BBNSM_BBNSM_CTRL_TURN_ON_TIME_MASK       (0x300000U)
#define BBNSM_BBNSM_CTRL_TURN_ON_TIME_SHIFT      (20U)
/*! TURN_ON_TIME - Turn-On Time
 *  0b00..500 milliseconds.
 *  0b01..50 milliseconds.
 *  0b10..100 milliseconds.
 *  0b11..0 milliseconds.
 */
#define BBNSM_BBNSM_CTRL_TURN_ON_TIME(x)         (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_TURN_ON_TIME_SHIFT)) & BBNSM_BBNSM_CTRL_TURN_ON_TIME_MASK)

#define BBNSM_BBNSM_CTRL_PK_EN_MASK              (0x400000U)
#define BBNSM_BBNSM_CTRL_PK_EN_SHIFT             (22U)
/*! PK_EN - PMIC On Request Enable
 *  0b0..PMIC On Request is disabled.
 *  0b1..PMIC On Request is enabled.
 */
#define BBNSM_BBNSM_CTRL_PK_EN(x)                (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_PK_EN_SHIFT)) & BBNSM_BBNSM_CTRL_PK_EN_MASK)

#define BBNSM_BBNSM_CTRL_PK_OVR_MASK             (0x800000U)
#define BBNSM_BBNSM_CTRL_PK_OVR_SHIFT            (23U)
/*! PK_OVR - PMIC On Request Override
 *  0b0..PMIC On Request Override is disabled.
 *  0b1..PMIC On Request Override is enabled.
 */
#define BBNSM_BBNSM_CTRL_PK_OVR(x)               (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_PK_OVR_SHIFT)) & BBNSM_BBNSM_CTRL_PK_OVR_MASK)

#define BBNSM_BBNSM_CTRL_DP_EN_MASK              (0x1000000U)
#define BBNSM_BBNSM_CTRL_DP_EN_SHIFT             (24U)
/*! DP_EN - Dumb PMIC Enable
 *  0b0..Smart PMIC is enabled.
 *  0b1..Dumb PMIC is enabled.
 */
#define BBNSM_BBNSM_CTRL_DP_EN(x)                (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_DP_EN_SHIFT)) & BBNSM_BBNSM_CTRL_DP_EN_MASK)

#define BBNSM_BBNSM_CTRL_TOSP_MASK               (0x2000000U)
#define BBNSM_BBNSM_CTRL_TOSP_SHIFT              (25U)
/*! TOSP - Turn Off System Power
 *  0b0..Leave system power on.
 *  0b1..Turn off system power when Dumb PMIC is enabled.
 */
#define BBNSM_BBNSM_CTRL_TOSP(x)                 (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_CTRL_TOSP_SHIFT)) & BBNSM_BBNSM_CTRL_TOSP_MASK)
/*! @} */

/*! @name BBNSM_INT_EN - BBNSM Interrupt Enable Register */
/*! @{ */

#define BBNSM_BBNSM_INT_EN_RTC_INT_EN_MASK       (0x3U)
#define BBNSM_BBNSM_INT_EN_RTC_INT_EN_SHIFT      (0U)
/*! RTC_INT_EN - Real-Time Counter Rollover Interrupt Enable
 *  0b01..Do not issue an interrupt when RTC has rolled over. The interrupt is cleared when this value is written.
 *  0b10..Issue an interrupt when RTC has rolled over.
 */
#define BBNSM_BBNSM_INT_EN_RTC_INT_EN(x)         (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_INT_EN_RTC_INT_EN_SHIFT)) & BBNSM_BBNSM_INT_EN_RTC_INT_EN_MASK)

#define BBNSM_BBNSM_INT_EN_TA_INT_EN_MASK        (0xCU)
#define BBNSM_BBNSM_INT_EN_TA_INT_EN_SHIFT       (2U)
/*! TA_INT_EN - Time Alarm Interrupt Enable
 *  0b01..Do not issue an interrupt when RTC has reached alarm time. The interrupt is cleared when this value is written.
 *  0b10..Issue an interrupt when RTC has reached alarm time.
 */
#define BBNSM_BBNSM_INT_EN_TA_INT_EN(x)          (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_INT_EN_TA_INT_EN_SHIFT)) & BBNSM_BBNSM_INT_EN_TA_INT_EN_MASK)
/*! @} */

/*! @name BBNSM_EVENTS - BBNSM Events Register */
/*! @{ */

#define BBNSM_BBNSM_EVENTS_RTC_ROLL_MASK         (0x3U)
#define BBNSM_BBNSM_EVENTS_RTC_ROLL_SHIFT        (0U)
/*! RTC_ROLL - Real-Time Counter Rollover Event
 *  0b01..The real-time counter has not rolled over.
 *  0b10..The real-time counter has rolled over.
 */
#define BBNSM_BBNSM_EVENTS_RTC_ROLL(x)           (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_EVENTS_RTC_ROLL_SHIFT)) & BBNSM_BBNSM_EVENTS_RTC_ROLL_MASK)

#define BBNSM_BBNSM_EVENTS_TA_MASK               (0xCU)
#define BBNSM_BBNSM_EVENTS_TA_SHIFT              (2U)
/*! TA - Time Alarm Event
 *  0b01..The real-time counter has not reached the alarm time.
 *  0b10..The real-time counter has reached the alarm time.
 */
#define BBNSM_BBNSM_EVENTS_TA(x)                 (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_EVENTS_TA_SHIFT)) & BBNSM_BBNSM_EVENTS_TA_MASK)

#define BBNSM_BBNSM_EVENTS_EMG_OFF_MASK          (0x10U)
#define BBNSM_BBNSM_EVENTS_EMG_OFF_SHIFT         (4U)
/*! EMG_OFF - Emergency Off Event
 *  0b0..An emergency power off has not been requested.
 *  0b1..An emergency power off has been requested.
 */
#define BBNSM_BBNSM_EVENTS_EMG_OFF(x)            (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_EVENTS_EMG_OFF_SHIFT)) & BBNSM_BBNSM_EVENTS_EMG_OFF_MASK)

#define BBNSM_BBNSM_EVENTS_PWR_OFF_MASK          (0x20U)
#define BBNSM_BBNSM_EVENTS_PWR_OFF_SHIFT         (5U)
/*! PWR_OFF - Set Power Off Event
 *  0b0..The power off interrupt has not been requested.
 *  0b1..The power off interrupt has been requested.
 */
#define BBNSM_BBNSM_EVENTS_PWR_OFF(x)            (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_EVENTS_PWR_OFF_SHIFT)) & BBNSM_BBNSM_EVENTS_PWR_OFF_MASK)

#define BBNSM_BBNSM_EVENTS_PWR_ON_MASK           (0x40U)
#define BBNSM_BBNSM_EVENTS_PWR_ON_SHIFT          (6U)
/*! PWR_ON - Set Power On Event
 *  0b0..The power on interrupt has not been requested.
 *  0b1..The power on interrupt has been requested.
 */
#define BBNSM_BBNSM_EVENTS_PWR_ON(x)             (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_EVENTS_PWR_ON_SHIFT)) & BBNSM_BBNSM_EVENTS_PWR_ON_MASK)
/*! @} */

/*! @name BBNSM_PAD_CTRL - BBNSM External Pad Control Register */
/*! @{ */

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL0_MASK      (0x1U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL0_SHIFT     (0U)
/*! PAD_CTRL0 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL0(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL0_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL0_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL1_MASK      (0x2U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL1_SHIFT     (1U)
/*! PAD_CTRL1 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL1(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL1_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL1_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL2_MASK      (0x4U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL2_SHIFT     (2U)
/*! PAD_CTRL2 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL2(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL2_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL2_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL3_MASK      (0x8U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL3_SHIFT     (3U)
/*! PAD_CTRL3 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL3(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL3_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL3_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL4_MASK      (0x10U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL4_SHIFT     (4U)
/*! PAD_CTRL4 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL4(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL4_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL4_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL5_MASK      (0x20U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL5_SHIFT     (5U)
/*! PAD_CTRL5 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL5(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL5_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL5_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL6_MASK      (0x40U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL6_SHIFT     (6U)
/*! PAD_CTRL6 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL6(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL6_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL6_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL7_MASK      (0x80U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL7_SHIFT     (7U)
/*! PAD_CTRL7 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL7(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL7_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL7_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL8_MASK      (0x100U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL8_SHIFT     (8U)
/*! PAD_CTRL8 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL8(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL8_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL8_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL9_MASK      (0x200U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL9_SHIFT     (9U)
/*! PAD_CTRL9 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL9(x)        (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL9_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL9_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL10_MASK     (0x400U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL10_SHIFT    (10U)
/*! PAD_CTRL10 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL10(x)       (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL10_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL10_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL11_MASK     (0x800U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL11_SHIFT    (11U)
/*! PAD_CTRL11 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL11(x)       (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL11_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL11_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL12_MASK     (0x1000U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL12_SHIFT    (12U)
/*! PAD_CTRL12 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL12(x)       (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL12_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL12_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL13_MASK     (0x2000U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL13_SHIFT    (13U)
/*! PAD_CTRL13 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL13(x)       (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL13_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL13_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL14_MASK     (0x4000U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL14_SHIFT    (14U)
/*! PAD_CTRL14 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL14(x)       (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL14_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL14_MASK)

#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL15_MASK     (0x8000U)
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL15_SHIFT    (15U)
/*! PAD_CTRL15 - Control I/O Pads
 *  0b0..Deasserts bit n in bbnsm_pad_ctrl[n]
 *  0b1..Assert bit n in bbnsm_pad_ctrl[n]
 */
#define BBNSM_BBNSM_PAD_CTRL_PAD_CTRL15(x)       (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_PAD_CTRL_PAD_CTRL15_SHIFT)) & BBNSM_BBNSM_PAD_CTRL_PAD_CTRL15_MASK)
/*! @} */

/*! @name BBNSM_RTC_LS - BBNSM Real-Time Counter LS Register */
/*! @{ */

#define BBNSM_BBNSM_RTC_LS_RTC_MASK              (0xFFFFFFFFU)
#define BBNSM_BBNSM_RTC_LS_RTC_SHIFT             (0U)
/*! RTC - Real-time Counter */
#define BBNSM_BBNSM_RTC_LS_RTC(x)                (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_RTC_LS_RTC_SHIFT)) & BBNSM_BBNSM_RTC_LS_RTC_MASK)
/*! @} */

/*! @name BBNSM_RTC_MS - BBNSM Real-Time Counter MS Register */
/*! @{ */

#define BBNSM_BBNSM_RTC_MS_RTC_MASK              (0x7FFFU)
#define BBNSM_BBNSM_RTC_MS_RTC_SHIFT             (0U)
/*! RTC - Real-Time Counter */
#define BBNSM_BBNSM_RTC_MS_RTC(x)                (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_RTC_MS_RTC_SHIFT)) & BBNSM_BBNSM_RTC_MS_RTC_MASK)
/*! @} */

/*! @name BBNSM_TA - BBNSM Time Alarm Register */
/*! @{ */

#define BBNSM_BBNSM_TA_TA_MASK                   (0xFFFFFFFFU)
#define BBNSM_BBNSM_TA_TA_SHIFT                  (0U)
/*! TA - Time Alarm Value */
#define BBNSM_BBNSM_TA_TA(x)                     (((uint32_t)(((uint32_t)(x)) << BBNSM_BBNSM_TA_TA_SHIFT)) & BBNSM_BBNSM_TA_TA_MASK)
/*! @} */

/*! @name GPR - General Purpose Register Word 0..General Purpose Register Word 7 */
/*! @{ */

#define BBNSM_GPR_GPR_MASK                       (0xFFFFFFFFU)
#define BBNSM_GPR_GPR_SHIFT                      (0U)
/*! GPR - 32 bits of the GPR. */
#define BBNSM_GPR_GPR(x)                         (((uint32_t)(((uint32_t)(x)) << BBNSM_GPR_GPR_SHIFT)) & BBNSM_GPR_GPR_MASK)
/*! @} */

/* The count of BBNSM_GPR */
#define BBNSM_GPR_COUNT                          (8U)


/*!
 * @}
 */ /* end of group BBNSM_Register_Masks */


/* BBNSM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral BBNSM base address */
  #define BBNSM_BASE                               (0x54440000u)
  /** Peripheral BBNSM base address */
  #define BBNSM_BASE_NS                            (0x44440000u)
  /** Peripheral BBNSM base pointer */
  #define BBNSM                                    ((BBNSM_Type *)BBNSM_BASE)
  /** Peripheral BBNSM base pointer */
  #define BBNSM_NS                                 ((BBNSM_Type *)BBNSM_BASE_NS)
  /** Array initializer of BBNSM peripheral base addresses */
  #define BBNSM_BASE_ADDRS                         { BBNSM_BASE }
  /** Array initializer of BBNSM peripheral base pointers */
  #define BBNSM_BASE_PTRS                          { BBNSM }
  /** Array initializer of BBNSM peripheral base addresses */
  #define BBNSM_BASE_ADDRS_NS                      { BBNSM_BASE_NS }
  /** Array initializer of BBNSM peripheral base pointers */
  #define BBNSM_BASE_PTRS_NS                       { BBNSM_NS }
#else
  /** Peripheral BBNSM base address */
  #define BBNSM_BASE                               (0x44440000u)
  /** Peripheral BBNSM base pointer */
  #define BBNSM                                    ((BBNSM_Type *)BBNSM_BASE)
  /** Array initializer of BBNSM peripheral base addresses */
  #define BBNSM_BASE_ADDRS                         { BBNSM_BASE }
  /** Array initializer of BBNSM peripheral base pointers */
  #define BBNSM_BASE_PTRS                          { BBNSM }
#endif
/** Interrupt vectors for the BBNSM peripheral type */
#define BBNSM_IRQS                               { BBNSM_IRQn }

/*!
 * @}
 */ /* end of group BBNSM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- BLK_CTRL_BBSMMIX Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_BBSMMIX_Peripheral_Access_Layer BLK_CTRL_BBSMMIX Peripheral Access Layer
 * @{
 */

/** BLK_CTRL_BBSMMIX - Register Layout Typedef */
typedef struct {
  __IO uint32_t BBSM_MISC;                         /**< BBSM miscellaneous register, offset: 0x0 */
  __IO uint32_t BBSM_TRIM;                         /**< BBSM TRIM register, offset: 0x4 */
} BLK_CTRL_BBSMMIX_Type;

/* ----------------------------------------------------------------------------
   -- BLK_CTRL_BBSMMIX Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_BBSMMIX_Register_Masks BLK_CTRL_BBSMMIX Register Masks
 * @{
 */

/*! @name BBSM_MISC - BBSM miscellaneous register */
/*! @{ */

#define BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_BYPASS_EN_MASK (0x4U)
#define BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_BYPASS_EN_SHIFT (2U)
/*! BBSM_BYPASS_EN - LDO_BBSM_ANA bypass enable
 *  0b1..Enable bypass
 *  0b0..Disable bypass
 */
#define BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_BYPASS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_BYPASS_EN_SHIFT)) & BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_BYPASS_EN_MASK)

#define BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_XTAL_CLK_OK_MASK (0x100000U)
#define BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_XTAL_CLK_OK_SHIFT (20U)
/*! BBSM_XTAL_CLK_OK - 32K OSC ok flag
 *  0b1..32K oscillator is stable into normal operation
 *  0b0..32K oscillator is NOT stable into normal operation
 */
#define BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_XTAL_CLK_OK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_XTAL_CLK_OK_SHIFT)) & BLK_CTRL_BBSMMIX_BBSM_MISC_BBSM_XTAL_CLK_OK_MASK)
/*! @} */

/*! @name BBSM_TRIM - BBSM TRIM register */
/*! @{ */

#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_SEL_MASK (0x2U)
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_SEL_SHIFT (1U)
/*! BBSM_CORE_VOLT_DET_TRIM_SEL - BBSM core voltage detect trim select
 *  0b0..The trimming codes are selected from eFuse
 *  0b1..The trimming codes of core voltage detectors used to change the voltage falling trip point are selected from BBSM_CORE_VOLT_DET_TRIM
 */
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_SEL_SHIFT)) & BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_SEL_MASK)

#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_MASK (0xCU)
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_SHIFT (2U)
/*! BBSM_CORE_VOLT_DET_TRIM - BBSM core voltage detect trim */
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_SHIFT)) & BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CORE_VOLT_DET_TRIM_MASK)

#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CAP_TRIM_SEL_MASK (0x800000U)
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CAP_TRIM_SEL_SHIFT (23U)
/*! BBSM_CAP_TRIM_SEL - BBSM OSC load capacitor trim select
 *  0b0..The trimming codes are selected from eFuse
 *  0b1..The trimming codes are used from BBSM_OSC_CAP_TRIM (osc32k's load capacitor)
 */
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CAP_TRIM_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CAP_TRIM_SEL_SHIFT)) & BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_CAP_TRIM_SEL_MASK)

#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_OSC_CAP_TRIM_MASK (0xF000000U)
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_OSC_CAP_TRIM_SHIFT (24U)
/*! BBSM_OSC_CAP_TRIM - BBSM OSC load capacitor trim */
#define BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_OSC_CAP_TRIM(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_OSC_CAP_TRIM_SHIFT)) & BLK_CTRL_BBSMMIX_BBSM_TRIM_BBSM_OSC_CAP_TRIM_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group BLK_CTRL_BBSMMIX_Register_Masks */


/* BLK_CTRL_BBSMMIX - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral BLK_CTRL_BBSMMIX base address */
  #define BLK_CTRL_BBSMMIX_BASE                    (0x54410000u)
  /** Peripheral BLK_CTRL_BBSMMIX base address */
  #define BLK_CTRL_BBSMMIX_BASE_NS                 (0x44410000u)
  /** Peripheral BLK_CTRL_BBSMMIX base pointer */
  #define BLK_CTRL_BBSMMIX                         ((BLK_CTRL_BBSMMIX_Type *)BLK_CTRL_BBSMMIX_BASE)
  /** Peripheral BLK_CTRL_BBSMMIX base pointer */
  #define BLK_CTRL_BBSMMIX_NS                      ((BLK_CTRL_BBSMMIX_Type *)BLK_CTRL_BBSMMIX_BASE_NS)
  /** Array initializer of BLK_CTRL_BBSMMIX peripheral base addresses */
  #define BLK_CTRL_BBSMMIX_BASE_ADDRS              { BLK_CTRL_BBSMMIX_BASE }
  /** Array initializer of BLK_CTRL_BBSMMIX peripheral base pointers */
  #define BLK_CTRL_BBSMMIX_BASE_PTRS               { BLK_CTRL_BBSMMIX }
  /** Array initializer of BLK_CTRL_BBSMMIX peripheral base addresses */
  #define BLK_CTRL_BBSMMIX_BASE_ADDRS_NS           { BLK_CTRL_BBSMMIX_BASE_NS }
  /** Array initializer of BLK_CTRL_BBSMMIX peripheral base pointers */
  #define BLK_CTRL_BBSMMIX_BASE_PTRS_NS            { BLK_CTRL_BBSMMIX_NS }
#else
  /** Peripheral BLK_CTRL_BBSMMIX base address */
  #define BLK_CTRL_BBSMMIX_BASE                    (0x44410000u)
  /** Peripheral BLK_CTRL_BBSMMIX base pointer */
  #define BLK_CTRL_BBSMMIX                         ((BLK_CTRL_BBSMMIX_Type *)BLK_CTRL_BBSMMIX_BASE)
  /** Array initializer of BLK_CTRL_BBSMMIX peripheral base addresses */
  #define BLK_CTRL_BBSMMIX_BASE_ADDRS              { BLK_CTRL_BBSMMIX_BASE }
  /** Array initializer of BLK_CTRL_BBSMMIX peripheral base pointers */
  #define BLK_CTRL_BBSMMIX_BASE_PTRS               { BLK_CTRL_BBSMMIX }
#endif

/*!
 * @}
 */ /* end of group BLK_CTRL_BBSMMIX_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- BLK_CTRL_NS_AONMIX Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_NS_AONMIX_Peripheral_Access_Layer BLK_CTRL_NS_AONMIX Peripheral Access Layer
 * @{
 */

/** BLK_CTRL_NS_AONMIX - Register Layout Typedef */
typedef struct {
  __IO uint32_t GPC_CFG;                           /**< GPC CORE SLEEP Request Select, offset: 0x0 */
       uint8_t RESERVED_0[4];
  __IO uint32_t IPG_DEBUG;                         /**< IPG Debug mask, offset: 0x8 */
       uint8_t RESERVED_1[16];
  __IO uint32_t SSI;                               /**< offset: 0x1C */
  __IO uint32_t SAI1_MCLK_CTRL;                    /**< SAI1 MCLK control register, offset: 0x20 */
  __IO uint32_t DCDC_STATUS;                       /**< DCDC status register, offset: 0x24 */
  __I  uint32_t FUSE_ACC_DIS;                      /**< Fuse access disable register, offset: 0x28 */
  __IO uint32_t M33_NMI_CLR;                       /**< M33 NMI interrupt clear register, offset: 0x2C */
  __IO uint32_t I3C1_ASYNC_WAKEUP_CTRL;            /**< I3C1 async wakeup control register, offset: 0x30 */
  __IO uint32_t MISC_IO_CTRL;                      /**< Miscellaneous control register of IO, offset: 0x34 */
} BLK_CTRL_NS_AONMIX_Type;

/* ----------------------------------------------------------------------------
   -- BLK_CTRL_NS_AONMIX Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_NS_AONMIX_Register_Masks BLK_CTRL_NS_AONMIX Register Masks
 * @{
 */

/*! @name GPC_CFG - GPC CORE SLEEP Request Select */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_GPC_CFG_M33_SLEEP_SEL_MASK (0x1U)
#define BLK_CTRL_NS_AONMIX_GPC_CFG_M33_SLEEP_SEL_SHIFT (0U)
/*! M33_SLEEP_SEL - M33 SLEEP Request Select
 *  0b0..Select SLEEPING as request source
 *  0b1..Select SLEEPDEEP as request source
 */
#define BLK_CTRL_NS_AONMIX_GPC_CFG_M33_SLEEP_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_GPC_CFG_M33_SLEEP_SEL_SHIFT)) & BLK_CTRL_NS_AONMIX_GPC_CFG_M33_SLEEP_SEL_MASK)

#define BLK_CTRL_NS_AONMIX_GPC_CFG_M7_SLEEP_SEL_MASK (0x2U)
#define BLK_CTRL_NS_AONMIX_GPC_CFG_M7_SLEEP_SEL_SHIFT (1U)
/*! M7_SLEEP_SEL - M7 SLEEP Request Select
 *  0b0..Select SLEEPING as request source
 *  0b1..Select SLEEPDEEP as request source
 */
#define BLK_CTRL_NS_AONMIX_GPC_CFG_M7_SLEEP_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_GPC_CFG_M7_SLEEP_SEL_SHIFT)) & BLK_CTRL_NS_AONMIX_GPC_CFG_M7_SLEEP_SEL_MASK)
/*! @} */

/*! @name IPG_DEBUG - IPG Debug mask */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN1_MASK (0x1U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN1_SHIFT (0U)
/*! M33_CAN1 - Mask bit for CAN1 debug halted mode with M33 core
 *  0b1..CAN1 enters debug halted mode when CM33 is debug halted
 *  0b0..CAN1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_EDMA3_MASK (0x2U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_EDMA3_SHIFT (1U)
/*! M33_EDMA3 - Mask bit for EDMA3 debug halted mode with M33 core
 *  0b1..EDMA3 enters debug halted mode when CM33 is debug halted
 *  0b0..EDMA3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_EDMA3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_EDMA3_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_EDMA3_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C1_MASK (0x4U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C1_SHIFT (2U)
/*! M33_LPI2C1 - Mask bit for LPI2C1 debug halted mode with M33 core
 *  0b1..LPI2C1 enters debug halted mode when CM33 is debug halted
 *  0b0..LPI2C1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C2_MASK (0x8U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C2_SHIFT (3U)
/*! M33_LPI2C2 - Mask bit for LPI2C2 debug halted mode with M33 core
 *  0b1..LPI2C2 enters debug halted mode when CM33 is debug halted
 *  0b0..LPI2C2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPI2C2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPIT1_MASK (0x10U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPIT1_SHIFT (4U)
/*! M33_LPIT1 - Mask bit for LPIT1 debug halted mode with M33 core
 *  0b1..LPIT1 enters debug halted mode when CM33 is debug halted
 *  0b0..LPIT1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPIT1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPIT1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPIT1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI1_MASK (0x20U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI1_SHIFT (5U)
/*! M33_LPSPI1 - Mask bit for LPSPI1 debug halted mode with M33 core
 *  0b1..LPSPI1 enters debug halted mode when CM33 is debug halted
 *  0b0..LPSPI1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI2_MASK (0x40U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI2_SHIFT (6U)
/*! M33_LPSPI2 - Mask bit for LPSPI2 debug halted mode with M33 core
 *  0b1..LPSPI2 enters debug halted mode when CM33 is debug halted
 *  0b0..LPSPI2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPSPI2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPTMR1_MASK (0x80U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPTMR1_SHIFT (7U)
/*! M33_LPTMR1 - Mask bit for LPTMR1 debug halted mode with M33 core
 *  0b1..LPTMR1 enters debug halted mode when CM33 is debug halted
 *  0b0..LPTMR1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPTMR1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPTMR1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_LPTMR1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_SAI1_MASK (0x100U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_SAI1_SHIFT (8U)
/*! M33_SAI1 - Mask bit for SAI1 debug halted mode with M33 core
 *  0b1..SAI1 enters debug halted mode when CM33 is debug halted
 *  0b0..SAI1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_SAI1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_SAI1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_SAI1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM1_MASK (0x200U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM1_SHIFT (9U)
/*! M33_TPM1 - Mask bit for TPM1 debug halted mode with M33 core
 *  0b1..TPM1 enters debug halted mode when CM33 is debug halted
 *  0b0..TPM1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM2_MASK (0x400U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM2_SHIFT (10U)
/*! M33_TPM2 - Mask bit for TPM2 debug halted mode with M33 core
 *  0b1..TPM2 enters debug halted mode when CM33 is debug halted
 *  0b0..TPM2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_TPM2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG1_MASK (0x800U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG1_SHIFT (11U)
/*! M33_WDOG1 - Mask bit for WDOG1 debug halted mode with M33 core
 *  0b1..WDOG1 enters debug halted mode when CM33 is debug halted
 *  0b0..WDOG1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG2_MASK (0x1000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG2_SHIFT (12U)
/*! M33_WDOG2 - Mask bit for WDOG2 debug halted mode with M33 core
 *  0b1..WDOG2 enters debug halted mode when CM33 is debug halted
 *  0b0..WDOG2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_WDOG2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_GPT1_MASK (0x2000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_GPT1_SHIFT (13U)
/*! M33_GPT1 - Mask bit for GPT1 debug halted mode with M33 core
 *  0b1..GPT1 enters debug halted mode when CM33 is debug halted
 *  0b0..GPT1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_GPT1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_GPT1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_GPT1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN3_MASK (0x4000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN3_SHIFT (14U)
/*! M33_CAN3 - Mask bit for CAN3 debug halted mode with M33 core
 *  0b1..CAN3 enters debug halted mode when CM33 is debug halted
 *  0b0..CAN3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN3_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_CAN3_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_I3C1_MASK (0x8000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_I3C1_SHIFT (15U)
/*! M33_I3C1 - Mask bit for I3C1 debug halted mode with M33 core
 *  0b1..I3C1 enters debug halted mode when CM33 is debug halted
 *  0b0..I3C1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_I3C1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_I3C1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M33_I3C1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN1_MASK (0x10000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN1_SHIFT (16U)
/*! M7_CAN1 - Mask bit for CAN1 debug halted mode with M7 core
 *  0b1..CAN1 enters debug halted mode when CM7 is debug halted
 *  0b0..CAN1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN1(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_EDMA3_MASK (0x20000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_EDMA3_SHIFT (17U)
/*! M7_EDMA3 - Mask bit for EDMA3 debug halted mode with M7 core
 *  0b1..EDMA3 enters debug halted mode when CM7 is debug halted
 *  0b0..EDMA3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_EDMA3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_EDMA3_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_EDMA3_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C1_MASK (0x40000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C1_SHIFT (18U)
/*! M7_LPI2C1 - Mask bit for LPI2C1 debug halted mode with M7 core
 *  0b1..LPI2C1 enters debug halted mode when CM7 is debug halted
 *  0b0..LPI2C1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C2_MASK (0x80000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C2_SHIFT (19U)
/*! M7_LPI2C2 - Mask bit for LPI2C2 debug halted mode with M7 core
 *  0b1..LPI2C2 enters debug halted mode when CM7 is debug halted
 *  0b0..LPI2C2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPI2C2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPIT1_MASK (0x100000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPIT1_SHIFT (20U)
/*! M7_LPIT1 - Mask bit for LPIT1 debug halted mode with M7 core
 *  0b1..LPIT1 enters debug halted mode when CM7 is debug halted
 *  0b0..LPIT1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPIT1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPIT1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPIT1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI1_MASK (0x200000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI1_SHIFT (21U)
/*! M7_LPSPI1 - Mask bit for LPSPI1 debug halted mode with M7 core
 *  0b1..LPSPI1 enters debug halted mode when CM7 is debug halted
 *  0b0..LPSPI1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI2_MASK (0x400000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI2_SHIFT (22U)
/*! M7_LPSPI2 - Mask bit for LPSPI2 debug halted mode with M7 core
 *  0b1..LPSPI2 enters debug halted mode when CM7 is debug halted
 *  0b0..LPSPI2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPSPI2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPTMR1_MASK (0x800000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPTMR1_SHIFT (23U)
/*! M7_LPTMR1 - Mask bit for LPTMR1 debug halted mode with M7 core
 *  0b1..LPTMR1 enters debug halted mode when CM7 is debug halted
 *  0b0..LPTMR1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPTMR1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPTMR1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_LPTMR1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_SAI1_MASK (0x1000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_SAI1_SHIFT (24U)
/*! M7_SAI1 - Mask bit for SAI1 debug halted mode with M7 core
 *  0b1..SAI1 enters debug halted mode when CM7 is debug halted
 *  0b0..SAI1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_SAI1(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_SAI1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_SAI1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM1_MASK (0x2000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM1_SHIFT (25U)
/*! M7_TPM1 - Mask bit for TPM1 debug halted mode with M7 core
 *  0b1..TPM1 enters debug halted mode when CM7 is debug halted
 *  0b0..TPM1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM1(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM2_MASK (0x4000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM2_SHIFT (26U)
/*! M7_TPM2 - Mask bit for TPM2 debug halted mode with M7 core
 *  0b1..TPM2 enters debug halted mode when CM7 is debug halted
 *  0b0..TPM2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM2(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_TPM2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG1_MASK (0x8000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG1_SHIFT (27U)
/*! M7_WDOG1 - Mask bit for WDOG1 debug halted mode with M7 core
 *  0b1..WDOG1 enters debug halted mode when CM7 is debug halted
 *  0b0..WDOG1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG2_MASK (0x10000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG2_SHIFT (28U)
/*! M7_WDOG2 - Mask bit for WDOG2 debug halted mode with M7 core
 *  0b1..WDOG2 enters debug halted mode when CM7 is debug halted
 *  0b0..WDOG2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG2_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_WDOG2_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_GPT1_MASK (0x20000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_GPT1_SHIFT (29U)
/*! M7_GPT1 - Mask bit for GPT1 debug halted mode with M7 core
 *  0b1..GPT1 enters debug halted mode when CM7 is debug halted
 *  0b0..GPT1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_GPT1(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_GPT1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_GPT1_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN3_MASK (0x40000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN3_SHIFT (30U)
/*! M7_CAN3 - Mask bit for CAN3 debug halted mode with M7 core
 *  0b1..CAN3 enters debug halted mode when CM7 is debug halted
 *  0b0..CAN3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN3(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN3_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_CAN3_MASK)

#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_I3C1_MASK (0x80000000U)
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_I3C1_SHIFT (31U)
/*! M7_I3C1 - Mask bit for I3C1 debug halted mode with M7 core
 *  0b1..I3C1 enters debug halted mode when CM7 is debug halted
 *  0b0..I3C1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_I3C1(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_I3C1_SHIFT)) & BLK_CTRL_NS_AONMIX_IPG_DEBUG_M7_I3C1_MASK)
/*! @} */

/*! @name SSI -  */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_SSI_PAUSE_MODE_MASK   (0x1U)
#define BLK_CTRL_NS_AONMIX_SSI_PAUSE_MODE_SHIFT  (0U)
/*! PAUSE_MODE - AON Domain SSI master pause mode
 *  0b0..AON Domain SSI master is not in pause mode
 *  0b1..AON Domain SSI master is in pause mode
 */
#define BLK_CTRL_NS_AONMIX_SSI_PAUSE_MODE(x)     (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_SSI_PAUSE_MODE_SHIFT)) & BLK_CTRL_NS_AONMIX_SSI_PAUSE_MODE_MASK)

#define BLK_CTRL_NS_AONMIX_SSI_BLKHOLE_MODE_B_MASK (0x2U)
#define BLK_CTRL_NS_AONMIX_SSI_BLKHOLE_MODE_B_SHIFT (1U)
/*! BLKHOLE_MODE_B - AON Domain SSI master blackhole mode
 *  0b0..AON Domain SSI master will enter into blackhole mode
 *  0b1..AON Domain SSI master will exit from blackhole mode
 */
#define BLK_CTRL_NS_AONMIX_SSI_BLKHOLE_MODE_B(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_SSI_BLKHOLE_MODE_B_SHIFT)) & BLK_CTRL_NS_AONMIX_SSI_BLKHOLE_MODE_B_MASK)
/*! @} */

/*! @name SAI1_MCLK_CTRL - SAI1 MCLK control register */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_SAI1_MCLK_CTRL_SAI1_MCLK_DIR_MASK (0x100U)
#define BLK_CTRL_NS_AONMIX_SAI1_MCLK_CTRL_SAI1_MCLK_DIR_SHIFT (8U)
/*! SAI1_MCLK_DIR - SAI1_MCLK IO direction control. IOMUX need select SAI1 MCLK function
 *  0b0..SAI1_MCLK is input signal
 *  0b1..SAI1_MCLK is output signal
 */
#define BLK_CTRL_NS_AONMIX_SAI1_MCLK_CTRL_SAI1_MCLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_SAI1_MCLK_CTRL_SAI1_MCLK_DIR_SHIFT)) & BLK_CTRL_NS_AONMIX_SAI1_MCLK_CTRL_SAI1_MCLK_DIR_MASK)
/*! @} */

/*! @name DCDC_STATUS - DCDC status register */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STATUS_CAPT_CLR_MASK (0x2U)
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STATUS_CAPT_CLR_SHIFT (1U)
/*! DCDC_STATUS_CAPT_CLR - DCDC captured status clear
 *  0b0..No change
 *  0b1..Clear the 3 bits of DCDC captured status: DCDC_OVER_VOL, DCDC_OVER_CUR, and DCDC_IN_LOW_VOL
 */
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STATUS_CAPT_CLR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STATUS_CAPT_CLR_SHIFT)) & BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STATUS_CAPT_CLR_MASK)

#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_IN_LOW_VOL_MASK (0x10000U)
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_IN_LOW_VOL_SHIFT (16U)
/*! DCDC_IN_LOW_VOL - DCDC_IN low voltage detect
 *  0b1..Voltage on DCDC_IN is lower than 2.6V
 *  0b0..Voltage on DCDC_IN is higher than 2.6V
 */
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_IN_LOW_VOL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_IN_LOW_VOL_SHIFT)) & BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_IN_LOW_VOL_MASK)

#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_CUR_MASK (0x20000U)
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_CUR_SHIFT (17U)
/*! DCDC_OVER_CUR - DCDC output over current alert
 *  0b1..Overcurrent on DCDC output
 *  0b0..No Overcurrent on DCDC output
 */
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_CUR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_CUR_SHIFT)) & BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_CUR_MASK)

#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_VOL_MASK (0x40000U)
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_VOL_SHIFT (18U)
/*! DCDC_OVER_VOL - DCDC output over voltage alert
 *  0b1..Overvoltage on DCDC VDDLP0 or VDDLP8 output
 *  0b0..No Overvoltage on DCDC VDDLP0 or VDDLP8 output
 */
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_VOL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_VOL_SHIFT)) & BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_OVER_VOL_MASK)

#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STS_DC_OK_MASK (0x80000U)
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STS_DC_OK_SHIFT (19U)
/*! DCDC_STS_DC_OK - DCDC status OK
 *  0b0..DCDC is settling
 *  0b1..DCDC already settled
 */
#define BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STS_DC_OK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STS_DC_OK_SHIFT)) & BLK_CTRL_NS_AONMIX_DCDC_STATUS_DCDC_STS_DC_OK_MASK)
/*! @} */

/*! @name FUSE_ACC_DIS - Fuse access disable register */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OSCCA_FUSE_READ_DIS_MASK (0x1U)
#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OSCCA_FUSE_READ_DIS_SHIFT (0U)
/*! OSCCA_FUSE_READ_DIS - Fuse read disable flag
 *  0b1..Read is not allowed
 *  0b0..Read is allowed
 */
#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OSCCA_FUSE_READ_DIS(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OSCCA_FUSE_READ_DIS_SHIFT)) & BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OSCCA_FUSE_READ_DIS_MASK)

#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_CALIBRATED_MASK (0x2U)
#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_CALIBRATED_SHIFT (1U)
/*! OCOTP_CALIBRATED - Fuse calibrate flag
 *  0b1..OCOTP is calibrated
 *  0b0..OCOTP is not calibrated
 */
#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_CALIBRATED(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_CALIBRATED_SHIFT)) & BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_CALIBRATED_MASK)

#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_BUSY_MASK (0x4U)
#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_BUSY_SHIFT (2U)
/*! OCOTP_BUSY - OCOTP busy flag
 *  0b1..OCOTP is busy
 *  0b0..OCOTP is not busy
 */
#define BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_BUSY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_BUSY_SHIFT)) & BLK_CTRL_NS_AONMIX_FUSE_ACC_DIS_OCOTP_BUSY_MASK)
/*! @} */

/*! @name M33_NMI_CLR - M33 NMI interrupt clear register */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_M33_NMI_CLR_M33_NMI_CLEAR_MASK (0x1U)
#define BLK_CTRL_NS_AONMIX_M33_NMI_CLR_M33_NMI_CLEAR_SHIFT (0U)
/*! M33_NMI_CLEAR - Clear CM33 NMI holding register */
#define BLK_CTRL_NS_AONMIX_M33_NMI_CLR_M33_NMI_CLEAR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_M33_NMI_CLR_M33_NMI_CLEAR_SHIFT)) & BLK_CTRL_NS_AONMIX_M33_NMI_CLR_M33_NMI_CLEAR_MASK)
/*! @} */

/*! @name I3C1_ASYNC_WAKEUP_CTRL - I3C1 async wakeup control register */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_CLR_MASK (0x1U)
#define BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_CLR_SHIFT (0U)
/*! IRQ_CLR - Async wakeup interrupt clear */
#define BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_CLR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_CLR_SHIFT)) & BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_CLR_MASK)

#define BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_STATUS_MASK (0x10000U)
#define BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_STATUS_SHIFT (16U)
/*! IRQ_STATUS - Async wakeup interrupt status
 *  0b0..Interrupt not asserted
 *  0b1..Interrupt asserted
 */
#define BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_STATUS(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_STATUS_SHIFT)) & BLK_CTRL_NS_AONMIX_I3C1_ASYNC_WAKEUP_CTRL_IRQ_STATUS_MASK)
/*! @} */

/*! @name MISC_IO_CTRL - Miscellaneous control register of IO */
/*! @{ */

#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_MASK (0x1U)
#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_SHIFT (0U)
/*! I3C_ON_CHIP_STRONG_PULL_DIS - Disable I3C on-chip strong pull for I3C1
 *  0b0..On-chip strong pull is enabled
 *  0b1..On-chip strong pull is disabled
 */
#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_SHIFT)) & BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_MASK)

#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_HIGH_RANGE_MASK (0x2U)
#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_HIGH_RANGE_SHIFT (1U)
/*! GPIO_AON_HIGH_RANGE - GPIO_AON IO bank supply voltage range selection */
#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_HIGH_RANGE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_HIGH_RANGE_SHIFT)) & BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_HIGH_RANGE_MASK)

#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_LOW_RANGE_MASK (0x4U)
#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_LOW_RANGE_SHIFT (2U)
/*! GPIO_AON_LOW_RANGE - GPIO_AON IO bank supply voltage range selection */
#define BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_LOW_RANGE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_LOW_RANGE_SHIFT)) & BLK_CTRL_NS_AONMIX_MISC_IO_CTRL_GPIO_AON_LOW_RANGE_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group BLK_CTRL_NS_AONMIX_Register_Masks */


/* BLK_CTRL_NS_AONMIX - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral BLK_CTRL_NS_AONMIX base address */
  #define BLK_CTRL_NS_AONMIX_BASE                  (0x54210000u)
  /** Peripheral BLK_CTRL_NS_AONMIX base address */
  #define BLK_CTRL_NS_AONMIX_BASE_NS               (0x44210000u)
  /** Peripheral BLK_CTRL_NS_AONMIX base pointer */
  #define BLK_CTRL_NS_AONMIX                       ((BLK_CTRL_NS_AONMIX_Type *)BLK_CTRL_NS_AONMIX_BASE)
  /** Peripheral BLK_CTRL_NS_AONMIX base pointer */
  #define BLK_CTRL_NS_AONMIX_NS                    ((BLK_CTRL_NS_AONMIX_Type *)BLK_CTRL_NS_AONMIX_BASE_NS)
  /** Array initializer of BLK_CTRL_NS_AONMIX peripheral base addresses */
  #define BLK_CTRL_NS_AONMIX_BASE_ADDRS            { BLK_CTRL_NS_AONMIX_BASE }
  /** Array initializer of BLK_CTRL_NS_AONMIX peripheral base pointers */
  #define BLK_CTRL_NS_AONMIX_BASE_PTRS             { BLK_CTRL_NS_AONMIX }
  /** Array initializer of BLK_CTRL_NS_AONMIX peripheral base addresses */
  #define BLK_CTRL_NS_AONMIX_BASE_ADDRS_NS         { BLK_CTRL_NS_AONMIX_BASE_NS }
  /** Array initializer of BLK_CTRL_NS_AONMIX peripheral base pointers */
  #define BLK_CTRL_NS_AONMIX_BASE_PTRS_NS          { BLK_CTRL_NS_AONMIX_NS }
#else
  /** Peripheral BLK_CTRL_NS_AONMIX base address */
  #define BLK_CTRL_NS_AONMIX_BASE                  (0x44210000u)
  /** Peripheral BLK_CTRL_NS_AONMIX base pointer */
  #define BLK_CTRL_NS_AONMIX                       ((BLK_CTRL_NS_AONMIX_Type *)BLK_CTRL_NS_AONMIX_BASE)
  /** Array initializer of BLK_CTRL_NS_AONMIX peripheral base addresses */
  #define BLK_CTRL_NS_AONMIX_BASE_ADDRS            { BLK_CTRL_NS_AONMIX_BASE }
  /** Array initializer of BLK_CTRL_NS_AONMIX peripheral base pointers */
  #define BLK_CTRL_NS_AONMIX_BASE_PTRS             { BLK_CTRL_NS_AONMIX }
#endif

/*!
 * @}
 */ /* end of group BLK_CTRL_NS_AONMIX_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- BLK_CTRL_S_AONMIX Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_S_AONMIX_Peripheral_Access_Layer BLK_CTRL_S_AONMIX Peripheral Access Layer
 * @{
 */

/** BLK_CTRL_S_AONMIX - Register Layout Typedef */
typedef struct {
  __IO uint32_t CM33_IRQ_MASK[8];                  /**< CM33_IRQ_MASK0..CM33_IRQ_MASK7, array offset: 0x0, array step: 0x4 */
  __IO uint32_t CM7_IRQ_MASK[8];                   /**< CM7_IRQ_MASK0..CM7_IRQ_MASK7, array offset: 0x20, array step: 0x4 */
       uint8_t RESERVED_0[24];
  __IO uint32_t EDGELOCK_RESET_REQ_MASK;           /**< EdgeLock reset request mask, offset: 0x58 */
  __IO uint32_t EDGELOCK_IRQ_MASK;                 /**< EdgeLock IRQ request mask, offset: 0x5C */
  __IO uint32_t M33_CFG;                           /**< M33 Configuration, offset: 0x60 */
  __IO uint32_t M33_INITSVTOR;                     /**< M33 INITSVTOR, offset: 0x64 */
  __IO uint32_t M33_INITNSVTOR;                    /**< M33 INITNSVTOR, offset: 0x68 */
       uint8_t RESERVED_1[20];
  __IO uint32_t M7_CFG;                            /**< M7 Configuration, offset: 0x80 */
       uint8_t RESERVED_2[12];
  __IO uint32_t AXBS_AON_CTRL;                     /**< AXBS_AON_CTRL, offset: 0x90 */
       uint8_t RESERVED_3[108];
  __IO uint32_t DAP_ACCESS_STKYBIT;                /**< DAP Access Sticky Bit, offset: 0x100 */
       uint8_t RESERVED_4[12];
  __IO uint32_t LP_HANDSHAKE;                      /**< Low power handshake enable, offset: 0x110 */
  __IO uint32_t EDGELOCK_HALT_ST;                  /**< EdgeLock halt status, offset: 0x114 */
       uint8_t RESERVED_5[8];
  __I  uint32_t ECC_MEM_INIT;                      /**< ECC memory hardware initialization, offset: 0x120 */
       uint8_t RESERVED_6[36];
  __IO uint32_t IOMUXC_DOMAIN_CFG;                 /**< IOMUXC domain configure, offset: 0x148 */
  __IO uint32_t IOMUXC_AON_DOMAIN_CFG;             /**< IOMUXC_AON domain configure, offset: 0x14C */
       uint8_t RESERVED_7[4];
  __IO uint32_t NMI_CTRL;                          /**< NMI control, offset: 0x154 */
  __IO uint32_t S401_NOCLK_CLEAR_CTRL;             /**< s401_ipi_noclk_ref1 clear control, offset: 0x158 */
} BLK_CTRL_S_AONMIX_Type;

/* ----------------------------------------------------------------------------
   -- BLK_CTRL_S_AONMIX Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_S_AONMIX_Register_Masks BLK_CTRL_S_AONMIX Register Masks
 * @{
 */

/*! @name CM33_IRQ_MASK - CM33_IRQ_MASK0..CM33_IRQ_MASK7 */
/*! @{ */

#define BLK_CTRL_S_AONMIX_CM33_IRQ_MASK_M_MASK   (0xFFFFFFFFU)
#define BLK_CTRL_S_AONMIX_CM33_IRQ_MASK_M_SHIFT  (0U)
/*! M - CM33 IRQ MASK
 *  0b00000000000000000000000000000001..No Mask IRQ
 *  0b00000000000000000000000000000000..Mask IRQ
 */
#define BLK_CTRL_S_AONMIX_CM33_IRQ_MASK_M(x)     (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_CM33_IRQ_MASK_M_SHIFT)) & BLK_CTRL_S_AONMIX_CM33_IRQ_MASK_M_MASK)
/*! @} */

/* The count of BLK_CTRL_S_AONMIX_CM33_IRQ_MASK */
#define BLK_CTRL_S_AONMIX_CM33_IRQ_MASK_COUNT    (8U)

/*! @name CM7_IRQ_MASK - CM7_IRQ_MASK0..CM7_IRQ_MASK7 */
/*! @{ */

#define BLK_CTRL_S_AONMIX_CM7_IRQ_MASK_M_MASK    (0xFFFFFFFFU)
#define BLK_CTRL_S_AONMIX_CM7_IRQ_MASK_M_SHIFT   (0U)
/*! M - CM7 IRQ MASK
 *  0b00000000000000000000000000000001..No Mask IRQ
 *  0b00000000000000000000000000000000..Mask IRQ
 */
#define BLK_CTRL_S_AONMIX_CM7_IRQ_MASK_M(x)      (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_CM7_IRQ_MASK_M_SHIFT)) & BLK_CTRL_S_AONMIX_CM7_IRQ_MASK_M_MASK)
/*! @} */

/* The count of BLK_CTRL_S_AONMIX_CM7_IRQ_MASK */
#define BLK_CTRL_S_AONMIX_CM7_IRQ_MASK_COUNT     (8U)

/*! @name EDGELOCK_RESET_REQ_MASK - EdgeLock reset request mask */
/*! @{ */

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_WDG_RESET_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_WDG_RESET_SHIFT (0U)
/*! WDG_RESET - EdgeLock Wdog reset mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_WDG_RESET(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_WDG_RESET_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_WDG_RESET_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_PUF_RESET_MASK (0x2U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_PUF_RESET_SHIFT (1U)
/*! PUF_RESET - EdgeLock PUF reset mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_PUF_RESET(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_PUF_RESET_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_PUF_RESET_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LC_BRICKED_MASK (0x4U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LC_BRICKED_SHIFT (2U)
/*! LC_BRICKED - EdgeLock LMDA life cycle bricked reset mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LC_BRICKED(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LC_BRICKED_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LC_BRICKED_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_SYS_FAIL_MASK (0x8U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_SYS_FAIL_SHIFT (3U)
/*! LMDA_SYS_FAIL - EdgeLock system failure reset mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_SYS_FAIL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_SYS_FAIL_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_SYS_FAIL_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_32K_MASK (0x10U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_32K_SHIFT (4U)
/*! NOCLK_32K - EdgeLock 32k clock loss reset mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_32K(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_32K_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_32K_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_RESET_REQ_MASK (0x20U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_RESET_REQ_SHIFT (5U)
/*! LMDA_RESET_REQ - EdgeLock LMDA reset request mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_RESET_REQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_RESET_REQ_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_RESET_REQ_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_32K_RESET_REQ_MASK (0x40U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_32K_RESET_REQ_SHIFT (6U)
/*! LMDA_32K_RESET_REQ - EdgeLock LMDA reset request from 32k clock domain mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_32K_RESET_REQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_32K_RESET_REQ_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_LMDA_32K_RESET_REQ_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF1_MASK (0x80U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF1_SHIFT (7U)
/*! NOCLK_REF1 - EdgeLock CM33 root clock loss reset mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF1_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF1_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF2_MASK (0x100U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF2_SHIFT (8U)
/*! NOCLK_REF2 - EdgeLock OSC 24Mhz clock loss reset mask
 *  0b1..Mask reset
 *  0b0..Unmask reset
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF2_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_RESET_REQ_MASK_NOCLK_REF2_MASK)
/*! @} */

/*! @name EDGELOCK_IRQ_MASK - EdgeLock IRQ request mask */
/*! @{ */

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_WDG_RESET_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_WDG_RESET_SHIFT (0U)
/*! WDG_RESET - EdgeLock Wdog reset interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_WDG_RESET(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_WDG_RESET_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_WDG_RESET_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_PUF_RESET_MASK (0x2U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_PUF_RESET_SHIFT (1U)
/*! PUF_RESET - EdgeLock PUF reset interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_PUF_RESET(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_PUF_RESET_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_PUF_RESET_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LC_BRICKED_MASK (0x4U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LC_BRICKED_SHIFT (2U)
/*! LC_BRICKED - EdgeLock LMDA life cycle bricked interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LC_BRICKED(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LC_BRICKED_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LC_BRICKED_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_SYS_FAIL_MASK (0x8U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_SYS_FAIL_SHIFT (3U)
/*! LMDA_SYS_FAIL - EdgeLock system failure interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_SYS_FAIL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_SYS_FAIL_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_SYS_FAIL_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_32K_MASK (0x10U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_32K_SHIFT (4U)
/*! NOCLK_32K - EdgeLock 32k clock loss interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_32K(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_32K_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_32K_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_RESET_REQ_MASK (0x20U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_RESET_REQ_SHIFT (5U)
/*! LMDA_RESET_REQ - EdgeLock LMDA reset request interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_RESET_REQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_RESET_REQ_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_RESET_REQ_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_32K_RESET_REQ_MASK (0x40U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_32K_RESET_REQ_SHIFT (6U)
/*! LMDA_32K_RESET_REQ - EdgeLock LMDA reset request from 32k clock domain interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_32K_RESET_REQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_32K_RESET_REQ_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_LMDA_32K_RESET_REQ_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF1_MASK (0x80U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF1_SHIFT (7U)
/*! NOCLK_REF1 - EdgeLock cm33 root clock loss interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF1_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF1_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF2_MASK (0x100U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF2_SHIFT (8U)
/*! NOCLK_REF2 - EdgeLock OSC 24Mhz clock loss interrupt mask
 *  0b0..Unmask interrupt
 *  0b1..Mask interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF2_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_IRQ_MASK_NOCLK_REF2_MASK)
/*! @} */

/*! @name M33_CFG - M33 Configuration */
/*! @{ */

#define BLK_CTRL_S_AONMIX_M33_CFG_WAIT_MASK      (0x4U)
#define BLK_CTRL_S_AONMIX_M33_CFG_WAIT_SHIFT     (2U)
/*! WAIT - M33 CPU WAIT */
#define BLK_CTRL_S_AONMIX_M33_CFG_WAIT(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M33_CFG_WAIT_SHIFT)) & BLK_CTRL_S_AONMIX_M33_CFG_WAIT_MASK)

#define BLK_CTRL_S_AONMIX_M33_CFG_TCM_SIZE_MASK  (0x18U)
#define BLK_CTRL_S_AONMIX_M33_CFG_TCM_SIZE_SHIFT (3U)
/*! TCM_SIZE - M33 TCM SIZE
 *  0b11..Reserved
 *  0b10..Double Sys TCM, 256KB Sys TCM
 *  0b01..Double Code TCM, 256KB Code TCM
 *  0b00..Regular TCM, 128KB Code TCM and 128KB Sys TCM
 */
#define BLK_CTRL_S_AONMIX_M33_CFG_TCM_SIZE(x)    (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M33_CFG_TCM_SIZE_SHIFT)) & BLK_CTRL_S_AONMIX_M33_CFG_TCM_SIZE_MASK)

#define BLK_CTRL_S_AONMIX_M33_CFG_CORECLK_FORCE_ON_MASK (0x20U)
#define BLK_CTRL_S_AONMIX_M33_CFG_CORECLK_FORCE_ON_SHIFT (5U)
/*! CORECLK_FORCE_ON - Force CM33 core clock on in WAIT mode
 *  0b0..CM33 core clock is off in WAIT mode
 *  0b1..CM33 core clock is on in WAIT mode
 */
#define BLK_CTRL_S_AONMIX_M33_CFG_CORECLK_FORCE_ON(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M33_CFG_CORECLK_FORCE_ON_SHIFT)) & BLK_CTRL_S_AONMIX_M33_CFG_CORECLK_FORCE_ON_MASK)
/*! @} */

/*! @name M33_INITSVTOR - M33 INITSVTOR */
/*! @{ */

#define BLK_CTRL_S_AONMIX_M33_INITSVTOR_INITSVTOR_MASK (0x1FFFFFFU)
#define BLK_CTRL_S_AONMIX_M33_INITSVTOR_INITSVTOR_SHIFT (0U)
/*! INITSVTOR - M33 INITSVTOR */
#define BLK_CTRL_S_AONMIX_M33_INITSVTOR_INITSVTOR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M33_INITSVTOR_INITSVTOR_SHIFT)) & BLK_CTRL_S_AONMIX_M33_INITSVTOR_INITSVTOR_MASK)
/*! @} */

/*! @name M33_INITNSVTOR - M33 INITNSVTOR */
/*! @{ */

#define BLK_CTRL_S_AONMIX_M33_INITNSVTOR_INITNSVTOR_MASK (0x1FFFFFFU)
#define BLK_CTRL_S_AONMIX_M33_INITNSVTOR_INITNSVTOR_SHIFT (0U)
/*! INITNSVTOR - M33 INITNSVTOR */
#define BLK_CTRL_S_AONMIX_M33_INITNSVTOR_INITNSVTOR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M33_INITNSVTOR_INITNSVTOR_SHIFT)) & BLK_CTRL_S_AONMIX_M33_INITNSVTOR_INITNSVTOR_MASK)
/*! @} */

/*! @name M7_CFG - M7 Configuration */
/*! @{ */

#define BLK_CTRL_S_AONMIX_M7_CFG_TCM_SIZE_MASK   (0x7U)
#define BLK_CTRL_S_AONMIX_M7_CFG_TCM_SIZE_SHIFT  (0U)
/*! TCM_SIZE - M7 TCM SIZE
 *  0b111..Reserved
 *  0b110..Reserved
 *  0b101..HALF DTCM, 384KB ITCM and 128KB DTCM
 *  0b100..HALF ITCM, 128KB ITCM and 384KB DTCM
 *  0b010..Double DTCM, 512KB DTCM
 *  0b001..Double ITCM, 512KB ITCM
 *  0b000..Regular TCM, 256KB ITCM and 256KB DTCM
 */
#define BLK_CTRL_S_AONMIX_M7_CFG_TCM_SIZE(x)     (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M7_CFG_TCM_SIZE_SHIFT)) & BLK_CTRL_S_AONMIX_M7_CFG_TCM_SIZE_MASK)

#define BLK_CTRL_S_AONMIX_M7_CFG_WAIT_MASK       (0x10U)
#define BLK_CTRL_S_AONMIX_M7_CFG_WAIT_SHIFT      (4U)
/*! WAIT - M7 CPUWAIT */
#define BLK_CTRL_S_AONMIX_M7_CFG_WAIT(x)         (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M7_CFG_WAIT_SHIFT)) & BLK_CTRL_S_AONMIX_M7_CFG_WAIT_MASK)

#define BLK_CTRL_S_AONMIX_M7_CFG_CORECLK_FORCE_ON_MASK (0x20U)
#define BLK_CTRL_S_AONMIX_M7_CFG_CORECLK_FORCE_ON_SHIFT (5U)
/*! CORECLK_FORCE_ON - Force CM7 core clock on in WAIT mode
 *  0b0..CM7 core clock is off in WAIT mode
 *  0b1..CM7 core clock is on in WAIT mode
 */
#define BLK_CTRL_S_AONMIX_M7_CFG_CORECLK_FORCE_ON(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M7_CFG_CORECLK_FORCE_ON_SHIFT)) & BLK_CTRL_S_AONMIX_M7_CFG_CORECLK_FORCE_ON_MASK)

#define BLK_CTRL_S_AONMIX_M7_CFG_HCLK_FORCE_ON_MASK (0x40U)
#define BLK_CTRL_S_AONMIX_M7_CFG_HCLK_FORCE_ON_SHIFT (6U)
/*! HCLK_FORCE_ON - CM7 platform AHB clock enable */
#define BLK_CTRL_S_AONMIX_M7_CFG_HCLK_FORCE_ON(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M7_CFG_HCLK_FORCE_ON_SHIFT)) & BLK_CTRL_S_AONMIX_M7_CFG_HCLK_FORCE_ON_MASK)

#define BLK_CTRL_S_AONMIX_M7_CFG_INITVTOR_MASK   (0xFFFFFF80U)
#define BLK_CTRL_S_AONMIX_M7_CFG_INITVTOR_SHIFT  (7U)
#define BLK_CTRL_S_AONMIX_M7_CFG_INITVTOR(x)     (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_M7_CFG_INITVTOR_SHIFT)) & BLK_CTRL_S_AONMIX_M7_CFG_INITVTOR_MASK)
/*! @} */

/*! @name AXBS_AON_CTRL - AXBS_AON_CTRL */
/*! @{ */

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_FORCE_ROUND_ROBIN_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_FORCE_ROUND_ROBIN_SHIFT (0U)
/*! FORCE_ROUND_ROBIN - AXBS_AON Force Round Robin
 *  0b0..Enable force round robin(default)
 *  0b1..Disable force round robin
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_FORCE_ROUND_ROBIN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_FORCE_ROUND_ROBIN_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_FORCE_ROUND_ROBIN_MASK)

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M0_HIGH_PRIORITY_MASK (0x2U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M0_HIGH_PRIORITY_SHIFT (1U)
/*! M0_HIGH_PRIORITY - M0 High Priority Control Bit
 *  0b0..Default Priority
 *  0b1..High Priority
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M0_HIGH_PRIORITY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M0_HIGH_PRIORITY_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M0_HIGH_PRIORITY_MASK)

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M1_HIGH_PRIORITY_MASK (0x4U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M1_HIGH_PRIORITY_SHIFT (2U)
/*! M1_HIGH_PRIORITY - M1 High Priority Control Bit
 *  0b0..Default Priority
 *  0b1..High Priority
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M1_HIGH_PRIORITY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M1_HIGH_PRIORITY_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M1_HIGH_PRIORITY_MASK)

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M2_HIGH_PRIORITY_MASK (0x8U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M2_HIGH_PRIORITY_SHIFT (3U)
/*! M2_HIGH_PRIORITY - M2 High Priority Control Bit
 *  0b0..Default Priority
 *  0b1..High Priority
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M2_HIGH_PRIORITY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M2_HIGH_PRIORITY_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M2_HIGH_PRIORITY_MASK)

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M3_HIGH_PRIORITY_MASK (0x10U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M3_HIGH_PRIORITY_SHIFT (4U)
/*! M3_HIGH_PRIORITY - M3 High Priority Control Bit
 *  0b0..Default Priority
 *  0b1..High Priority
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M3_HIGH_PRIORITY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M3_HIGH_PRIORITY_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M3_HIGH_PRIORITY_MASK)

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M4_HIGH_PRIORITY_MASK (0x20U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M4_HIGH_PRIORITY_SHIFT (5U)
/*! M4_HIGH_PRIORITY - M4 High Priority Control Bit
 *  0b0..Default Priority
 *  0b1..High Priority
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M4_HIGH_PRIORITY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M4_HIGH_PRIORITY_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M4_HIGH_PRIORITY_MASK)

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M5_HIGH_PRIORITY_MASK (0x40U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M5_HIGH_PRIORITY_SHIFT (6U)
/*! M5_HIGH_PRIORITY - M5 High Priority Control Bit
 *  0b0..Default Priority
 *  0b1..High Priority
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M5_HIGH_PRIORITY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M5_HIGH_PRIORITY_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M5_HIGH_PRIORITY_MASK)

#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M6_HIGH_PRIORITY_MASK (0x80U)
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M6_HIGH_PRIORITY_SHIFT (7U)
/*! M6_HIGH_PRIORITY - M6 High Priority Control Bit
 *  0b0..Default Priority
 *  0b1..High Priority
 */
#define BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M6_HIGH_PRIORITY(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M6_HIGH_PRIORITY_SHIFT)) & BLK_CTRL_S_AONMIX_AXBS_AON_CTRL_M6_HIGH_PRIORITY_MASK)
/*! @} */

/*! @name DAP_ACCESS_STKYBIT - DAP Access Sticky Bit */
/*! @{ */

#define BLK_CTRL_S_AONMIX_DAP_ACCESS_STKYBIT_DAP_CTR_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_DAP_ACCESS_STKYBIT_DAP_CTR_SHIFT (0U)
/*! DAP_CTR - DAP access grant bit controlled by Cortex-M33 ROM, once set "1" will kept "1" unless there is a reset.
 *  0b0..DAP access is not granted by ROM
 *  0b1..DAP access is granted by ROM
 */
#define BLK_CTRL_S_AONMIX_DAP_ACCESS_STKYBIT_DAP_CTR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_DAP_ACCESS_STKYBIT_DAP_CTR_SHIFT)) & BLK_CTRL_S_AONMIX_DAP_ACCESS_STKYBIT_DAP_CTR_MASK)
/*! @} */

/*! @name LP_HANDSHAKE - Low power handshake enable */
/*! @{ */

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_RESET_HS_EN_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_RESET_HS_EN_SHIFT (0U)
/*! CM33_RESET_HS_EN - CM33 reset handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_RESET_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_RESET_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_RESET_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_RESET_HS_EN_MASK (0x2U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_RESET_HS_EN_SHIFT (1U)
/*! CM7_RESET_HS_EN - CM7 reset handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_RESET_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_RESET_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_RESET_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_SUSPEND_HS_EN_MASK (0x4U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_SUSPEND_HS_EN_SHIFT (2U)
/*! CM7_SUSPEND_HS_EN - CM7 suspend exit reset handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_SUSPEND_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_SUSPEND_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_SUSPEND_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_AONMIX_RESET_HS_EN_MASK (0x8U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_AONMIX_RESET_HS_EN_SHIFT (3U)
/*! AONMIX_RESET_HS_EN - AONMIX reset handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_AONMIX_RESET_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_AONMIX_RESET_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_AONMIX_RESET_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_WAKEUPMIX_RESET_HS_EN_MASK (0x10U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_WAKEUPMIX_RESET_HS_EN_SHIFT (4U)
/*! WAKEUPMIX_RESET_HS_EN - Wakeupmix reset handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_WAKEUPMIX_RESET_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_WAKEUPMIX_RESET_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_WAKEUPMIX_RESET_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_RESET_HS_EN_MASK (0x20U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_RESET_HS_EN_SHIFT (5U)
/*! MEGAMIX_RESET_HS_EN - Megamix reset handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_RESET_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_RESET_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_RESET_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_LPM_HS_EN_MASK (0x40U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_LPM_HS_EN_SHIFT (6U)
/*! MEGAMIX_LPM_HS_EN - Megamix low power mode exit reset handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_LPM_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_LPM_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_MEGAMIX_LPM_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_OFF_HS_EN_MASK (0x80U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_OFF_HS_EN_SHIFT (7U)
/*! EDGELOCK_CLK_OFF_HS_EN - EDGELOCK clock off handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_OFF_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_OFF_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_OFF_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_OFF_HS_EN_MASK (0x100U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_OFF_HS_EN_SHIFT (8U)
/*! CM33_CLK_OFF_HS_EN - CM33 clock off handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_OFF_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_OFF_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_OFF_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_OFF_HS_EN_MASK (0x200U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_OFF_HS_EN_SHIFT (9U)
/*! CM7_CLK_OFF_HS_EN - CM7 clock off handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_OFF_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_OFF_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_OFF_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_OFF_HS_EN_MASK (0x400U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_OFF_HS_EN_SHIFT (10U)
/*! TRDC_CLK_OFF_HS_EN - TRDC clock off handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_OFF_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_OFF_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_OFF_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_OFF_HS_EN_MASK (0x800U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_OFF_HS_EN_SHIFT (11U)
/*! IEE_CLK_OFF_HS_EN - IEE clock off handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_OFF_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_OFF_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_OFF_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_OFF_HS_EN_MASK (0x1000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_OFF_HS_EN_SHIFT (12U)
/*! OTFAD1_CLK_OFF_HS_EN - OTFAD1 clock off handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_OFF_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_OFF_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_OFF_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_OFF_HS_EN_MASK (0x2000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_OFF_HS_EN_SHIFT (13U)
/*! OTFAD2_CLK_OFF_HS_EN - OTFAD2 clock off handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_OFF_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_OFF_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_OFF_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_ON_HS_EN_MASK (0x4000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_ON_HS_EN_SHIFT (14U)
/*! EDGELOCK_CLK_ON_HS_EN - EDGELOCK clock on handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_ON_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_ON_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_EDGELOCK_CLK_ON_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_ON_HS_EN_MASK (0x8000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_ON_HS_EN_SHIFT (15U)
/*! CM33_CLK_ON_HS_EN - CM33 clock on handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_ON_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_ON_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM33_CLK_ON_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_ON_HS_EN_MASK (0x10000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_ON_HS_EN_SHIFT (16U)
/*! CM7_CLK_ON_HS_EN - CM7 clock on handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_ON_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_ON_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_CM7_CLK_ON_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_ON_HS_EN_MASK (0x20000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_ON_HS_EN_SHIFT (17U)
/*! TRDC_CLK_ON_HS_EN - TRDC clock on handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_ON_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_ON_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_TRDC_CLK_ON_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_ON_HS_EN_MASK (0x40000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_ON_HS_EN_SHIFT (18U)
/*! IEE_CLK_ON_HS_EN - IEE clock on handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_ON_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_ON_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_IEE_CLK_ON_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_ON_HS_EN_MASK (0x80000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_ON_HS_EN_SHIFT (19U)
/*! OTFAD1_CLK_ON_HS_EN - OTFAD1 clock on handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_ON_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_ON_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD1_CLK_ON_HS_EN_MASK)

#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_ON_HS_EN_MASK (0x100000U)
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_ON_HS_EN_SHIFT (20U)
/*! OTFAD2_CLK_ON_HS_EN - OTFAD2 clock on handshake enable
 *  0b1..Handshake is enabled
 *  0b0..Handshake is not enabled
 */
#define BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_ON_HS_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_ON_HS_EN_SHIFT)) & BLK_CTRL_S_AONMIX_LP_HANDSHAKE_OTFAD2_CLK_ON_HS_EN_MASK)
/*! @} */

/*! @name EDGELOCK_HALT_ST - EdgeLock halt status */
/*! @{ */

#define BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_ACK_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_ACK_SHIFT (0U)
/*! EDGELOCK_HALT_ACK - EdgeLock halt and clock status
 *  0b0..EdgeLock is not fully halted and its clocks must be enabled
 *  0b1..EdgeLock is fully halted indicating clocks may be removed
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_ACK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_ACK_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_ACK_MASK)

#define BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_EXIT_IRQ_CLR_MASK (0x100U)
#define BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_EXIT_IRQ_CLR_SHIFT (8U)
/*! EDGELOCK_HALT_EXIT_IRQ_CLR - EdgeLock halt exit interrupt clear
 *  0b0..Remove the clear signal. This bit is not self-clearing and need SW to clear.
 *  0b1..Clear EdgeLock halt exit interrupt
 */
#define BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_EXIT_IRQ_CLR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_EXIT_IRQ_CLR_SHIFT)) & BLK_CTRL_S_AONMIX_EDGELOCK_HALT_ST_EDGELOCK_HALT_EXIT_IRQ_CLR_MASK)
/*! @} */

/*! @name ECC_MEM_INIT - ECC memory hardware initialization */
/*! @{ */

#define BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM1_INIT_DONE_MASK (0x10000U)
#define BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM1_INIT_DONE_SHIFT (16U)
/*! OCRAM1_INIT_DONE - OCRAM1 initialization status
 *  0b0..OCRAM1 memory is under initialization
 *  0b1..OCRAM1 memory initialization is complete
 */
#define BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM1_INIT_DONE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM1_INIT_DONE_SHIFT)) & BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM1_INIT_DONE_MASK)

#define BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM2_INIT_DONE_MASK (0x1000000U)
#define BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM2_INIT_DONE_SHIFT (24U)
/*! OCRAM2_INIT_DONE - OCRAM2 initialization status
 *  0b0..OCRAM2 memory is under initialization
 *  0b1..OCRAM2 memory initialization is complete
 */
#define BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM2_INIT_DONE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM2_INIT_DONE_SHIFT)) & BLK_CTRL_S_AONMIX_ECC_MEM_INIT_OCRAM2_INIT_DONE_MASK)
/*! @} */

/*! @name IOMUXC_DOMAIN_CFG - IOMUXC domain configure */
/*! @{ */

#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID0_MASK (0xFU)
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID0_SHIFT (0U)
/*! DID0 - Domain ID 0 */
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID0(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID0_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID0_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID1_MASK (0xF0U)
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID1_SHIFT (4U)
/*! DID1 - Domain ID 1 */
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID1_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID1_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID2_MASK (0xF00U)
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID2_SHIFT (8U)
/*! DID2 - Domain ID 2 */
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID2_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID2_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID3_MASK (0xF000U)
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID3_SHIFT (12U)
/*! DID3 - Domain ID 3 */
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID3_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_DID3_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_LOCK_MASK (0x80000000U)
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_LOCK_SHIFT (31U)
/*! LOCK - Lock bit */
#define BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_LOCK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_LOCK_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_DOMAIN_CFG_LOCK_MASK)
/*! @} */

/*! @name IOMUXC_AON_DOMAIN_CFG - IOMUXC_AON domain configure */
/*! @{ */

#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID0_MASK (0xFU)
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID0_SHIFT (0U)
/*! DID0 - Domain ID 0 */
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID0(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID0_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID0_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID1_MASK (0xF0U)
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID1_SHIFT (4U)
/*! DID1 - Domain ID 1 */
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID1_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID1_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID2_MASK (0xF00U)
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID2_SHIFT (8U)
/*! DID2 - Domain ID 2 */
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID2_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID2_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID3_MASK (0xF000U)
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID3_SHIFT (12U)
/*! DID3 - Domain ID 3 */
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID3_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_DID3_MASK)

#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_LOCK_MASK (0x80000000U)
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_LOCK_SHIFT (31U)
/*! LOCK - Lock bit */
#define BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_LOCK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_LOCK_SHIFT)) & BLK_CTRL_S_AONMIX_IOMUXC_AON_DOMAIN_CFG_LOCK_MASK)
/*! @} */

/*! @name NMI_CTRL - NMI control */
/*! @{ */

#define BLK_CTRL_S_AONMIX_NMI_CTRL_M7_NMI_MASK_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_NMI_CTRL_M7_NMI_MASK_SHIFT (0U)
/*! M7_NMI_MASK - Mask CM7 NMI pin input
 *  0b0..NMI input from IO to CM7 is not blocked
 *  0b1..NMI input from IO to CM7 is blocked
 */
#define BLK_CTRL_S_AONMIX_NMI_CTRL_M7_NMI_MASK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_NMI_CTRL_M7_NMI_MASK_SHIFT)) & BLK_CTRL_S_AONMIX_NMI_CTRL_M7_NMI_MASK_MASK)

#define BLK_CTRL_S_AONMIX_NMI_CTRL_M33_NMI_MASK_MASK (0x2U)
#define BLK_CTRL_S_AONMIX_NMI_CTRL_M33_NMI_MASK_SHIFT (1U)
/*! M33_NMI_MASK - Mask CM33 NMI pin input
 *  0b0..NMI input from IO to CM33 is not blocked
 *  0b1..NMI input from IO to CM33 is blocked
 */
#define BLK_CTRL_S_AONMIX_NMI_CTRL_M33_NMI_MASK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_NMI_CTRL_M33_NMI_MASK_SHIFT)) & BLK_CTRL_S_AONMIX_NMI_CTRL_M33_NMI_MASK_MASK)
/*! @} */

/*! @name S401_NOCLK_CLEAR_CTRL - s401_ipi_noclk_ref1 clear control */
/*! @{ */

#define BLK_CTRL_S_AONMIX_S401_NOCLK_CLEAR_CTRL_ref1_slow_clear_MASK (0x1U)
#define BLK_CTRL_S_AONMIX_S401_NOCLK_CLEAR_CTRL_ref1_slow_clear_SHIFT (0U)
/*! ref1_slow_clear - clear the interrupt or reset source */
#define BLK_CTRL_S_AONMIX_S401_NOCLK_CLEAR_CTRL_ref1_slow_clear(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_S_AONMIX_S401_NOCLK_CLEAR_CTRL_ref1_slow_clear_SHIFT)) & BLK_CTRL_S_AONMIX_S401_NOCLK_CLEAR_CTRL_ref1_slow_clear_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group BLK_CTRL_S_AONMIX_Register_Masks */


/* BLK_CTRL_S_AONMIX - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral BLK_CTRL_S_AONMIX base address */
  #define BLK_CTRL_S_AONMIX_BASE                   (0x544F0000u)
  /** Peripheral BLK_CTRL_S_AONMIX base address */
  #define BLK_CTRL_S_AONMIX_BASE_NS                (0x444F0000u)
  /** Peripheral BLK_CTRL_S_AONMIX base pointer */
  #define BLK_CTRL_S_AONMIX                        ((BLK_CTRL_S_AONMIX_Type *)BLK_CTRL_S_AONMIX_BASE)
  /** Peripheral BLK_CTRL_S_AONMIX base pointer */
  #define BLK_CTRL_S_AONMIX_NS                     ((BLK_CTRL_S_AONMIX_Type *)BLK_CTRL_S_AONMIX_BASE_NS)
  /** Array initializer of BLK_CTRL_S_AONMIX peripheral base addresses */
  #define BLK_CTRL_S_AONMIX_BASE_ADDRS             { BLK_CTRL_S_AONMIX_BASE }
  /** Array initializer of BLK_CTRL_S_AONMIX peripheral base pointers */
  #define BLK_CTRL_S_AONMIX_BASE_PTRS              { BLK_CTRL_S_AONMIX }
  /** Array initializer of BLK_CTRL_S_AONMIX peripheral base addresses */
  #define BLK_CTRL_S_AONMIX_BASE_ADDRS_NS          { BLK_CTRL_S_AONMIX_BASE_NS }
  /** Array initializer of BLK_CTRL_S_AONMIX peripheral base pointers */
  #define BLK_CTRL_S_AONMIX_BASE_PTRS_NS           { BLK_CTRL_S_AONMIX_NS }
#else
  /** Peripheral BLK_CTRL_S_AONMIX base address */
  #define BLK_CTRL_S_AONMIX_BASE                   (0x444F0000u)
  /** Peripheral BLK_CTRL_S_AONMIX base pointer */
  #define BLK_CTRL_S_AONMIX                        ((BLK_CTRL_S_AONMIX_Type *)BLK_CTRL_S_AONMIX_BASE)
  /** Array initializer of BLK_CTRL_S_AONMIX peripheral base addresses */
  #define BLK_CTRL_S_AONMIX_BASE_ADDRS             { BLK_CTRL_S_AONMIX_BASE }
  /** Array initializer of BLK_CTRL_S_AONMIX peripheral base pointers */
  #define BLK_CTRL_S_AONMIX_BASE_PTRS              { BLK_CTRL_S_AONMIX }
#endif

/*!
 * @}
 */ /* end of group BLK_CTRL_S_AONMIX_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- BLK_CTRL_WAKEUPMIX Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_WAKEUPMIX_Peripheral_Access_Layer BLK_CTRL_WAKEUPMIX Peripheral Access Layer
 * @{
 */

/** BLK_CTRL_WAKEUPMIX - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t IPG_DEBUG1;                        /**< IPG DEBUG mask bit, offset: 0x4 */
  __IO uint32_t IPG_DEBUG2;                        /**< IPG DEBUG mask bit, offset: 0x8 */
  __IO uint32_t IPG_DEBUG3;                        /**< IPG DEBUG mask bit, offset: 0xC */
       uint8_t RESERVED_1[4];
  __IO uint32_t SSI;                               /**< SSI master low power mode control, offset: 0x14 */
  __IO uint32_t ECAT_MISC_CFG;                     /**< EtherCAT miscellaneous configuration, offset: 0x18 */
  __IO uint32_t DEXSC_ERR;                         /**< DEXSC error response configuration, offset: 0x1C */
  __IO uint32_t USBPHY_MISC_CTRL;                  /**< USBPHY miscellaneous control, offset: 0x20 */
  __IO uint32_t NETC_PORT_MISC_CFG;                /**< NETC Port miscellaneous configuration, offset: 0x24 */
  __IO uint32_t M7_NMI_CLR;                        /**< M7 NMI interrupt clear register, offset: 0x28 */
       uint8_t RESERVED_2[4];
  __IO uint32_t QTIMER_CTRL1;                      /**< Qtimer miscellaneous control register 1, offset: 0x30 */
  __IO uint32_t QTIMER_CTRL2;                      /**< Qtimer miscellaneous control register 2, offset: 0x34 */
  __IO uint32_t SAI2_MCLK_CTRL;                    /**< SAI2 MCLK control register, offset: 0x38 */
  __IO uint32_t SAI3_MCLK_CTRL;                    /**< SAI3 MCLK control register, offset: 0x3C */
  __IO uint32_t SAI4_MCLK_CTRL;                    /**< SAI4 MCLK control register, offset: 0x40 */
  __IO uint32_t XBAR_DIR_CTRL1;                    /**< XBAR IO direction control register, offset: 0x44 */
  __IO uint32_t XBAR_DIR_CTRL2;                    /**< XBAR IO direction control register, offset: 0x48 */
  __IO uint32_t LPIT_TRIG_SEL;                     /**< LPIT trigger input select register, offset: 0x4C */
  __IO uint32_t AXI_ATTR_CFG;                      /**< AXI bus attribute configuration register, offset: 0x50 */
  __IO uint32_t SRAMCR0;                           /**< SRAM Control Register 0, offset: 0x54 */
  __IO uint32_t SRAMCR1;                           /**< SRAM Control Register 1, offset: 0x58 */
       uint8_t RESERVED_3[4];
  __IO uint32_t SLAVE_STOP_MODE_CFG;               /**< Slave stop mode configure register, offset: 0x60 */
       uint8_t RESERVED_4[16];
  __IO uint32_t I3C2_ASYNC_WAKEUP_CTRL;            /**< I3C2 async wakeup control register, offset: 0x74 */
  __IO uint32_t XBAR_AOI_WE;                       /**< XBAR and AOI write protect register, offset: 0x78 */
  __IO uint32_t XBAR_TRIG_SYNC_CTRL1;              /**< XBAR trigger synchronizer control register1, offset: 0x7C */
  __IO uint32_t XBAR_TRIG_SYNC_CTRL2;              /**< XBAR trigger synchronizer control register2, offset: 0x80 */
       uint8_t RESERVED_5[124];
  __IO uint32_t NETC_LINK_CFG[5];                  /**< NETC link configuration for port0..NETC link configuration for port4, array offset: 0x100, array step: 0x4 */
  __IO uint32_t NETC_REVMII_DLL[5];                /**< NETC RevMII RGMII delay line configuration for port0..NETC RevMII RGMII delay line configuration for port4, array offset: 0x114, array step: 0x4 */
       uint8_t RESERVED_6[8];
  __IO uint32_t SAFETY_CLK_MON_CS;                 /**< Safety clock monitor control and status register, offset: 0x130 */
  __IO uint32_t SAFETY_CLK_MON_TH;                 /**< Safety clock monitor threshold register, offset: 0x134 */
       uint8_t RESERVED_7[8];
  __IO uint32_t EMC_B1_IO_CTRL;                    /**< GPIO_EMC_B1 bank IO control, offset: 0x140 */
  __IO uint32_t EMC_B2_IO_CTRL;                    /**< GPIO_EMC_B2 bank IO control, offset: 0x144 */
  __IO uint32_t SD_B1_IO_CTRL;                     /**< GPIO_SD_B1 bank IO control, offset: 0x148 */
  __IO uint32_t SD_B2_IO_CTRL;                     /**< GPIO_SD_B2 bank IO control, offset: 0x14C */
  __IO uint32_t GPIO_B1_IO_CTRL;                   /**< GPIO_B1 bank IO control, offset: 0x150 */
  __IO uint32_t GPIO_B2_IO_CTRL;                   /**< GPIO_B2 bank IO control, offset: 0x154 */
  __IO uint32_t MISC_IO_CTRL;                      /**< Miscellaneous control register of IO, offset: 0x158 */
} BLK_CTRL_WAKEUPMIX_Type;

/* ----------------------------------------------------------------------------
   -- BLK_CTRL_WAKEUPMIX Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup BLK_CTRL_WAKEUPMIX_Register_Masks BLK_CTRL_WAKEUPMIX Register Masks
 * @{
 */

/*! @name IPG_DEBUG1 - IPG DEBUG mask bit */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_CAN2_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_CAN2_SHIFT (0U)
/*! M33_CAN2 - CAN2 debug halted mode with M7
 *  0b1..CAN2 enters debug halted mode when CM33 is debug halted
 *  0b0..CAN2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_CAN2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_CAN2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_CAN2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_EDMA4_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_EDMA4_SHIFT (1U)
/*! M33_EDMA4 - EDMA4 debug halted mode with M33
 *  0b1..EDMA4 enters debug halted mode when CM33 is debug halted
 *  0b0..EDMA4 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_EDMA4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_EDMA4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_EDMA4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO1_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO1_SHIFT (2U)
/*! M33_FLEXIO1 - FLEXIO1 debug halted mode with M33
 *  0b1..FLEXIO1 enters debug halted mode when CM33 is debug halted
 *  0b0..FLEXIO1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO2_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO2_SHIFT (3U)
/*! M33_FLEXIO2 - FLEXIO2 debug halted mode with M33
 *  0b1..FLEXIO2 enters debug halted mode when CM33 is debug halted
 *  0b0..FLEXIO2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_FLEXIO2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C3_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C3_SHIFT (4U)
/*! M33_LPI2C3 - LPI2C3 debug halted mode with M33
 *  0b1..enters debug halted mode when CM33 is debug halted
 *  0b0..LPI2C3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C4_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C4_SHIFT (5U)
/*! M33_LPI2C4 - LPI2C4 debug halted mode with M33
 *  0b1..LPI2C4 enters debug halted mode when CM33 is debug halted
 *  0b0..LPI2C4 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPI2C4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPIT2_MASK (0x40U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPIT2_SHIFT (6U)
/*! M33_LPIT2 - LPIT2 debug halted mode with M33
 *  0b1..LPIT2 enters debug halted mode when CM33 is debug halted
 *  0b0..LPIT2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPIT2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPIT2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPIT2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI3_MASK (0x80U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI3_SHIFT (7U)
/*! M33_LPSPI3 - LPSPI3 debug halted mode with M33
 *  0b1..LPSPI3 enters debug halted mode when CM33 is debug halted
 *  0b0..LPSPI3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI4_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI4_SHIFT (8U)
/*! M33_LPSPI4 - LPSPI4 debug halted mode with M33
 *  0b1..LPSPI4 enters debug halted mode when CM33 is debug halted
 *  0b0..LPSPI4 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPSPI4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPTMR2_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPTMR2_SHIFT (9U)
/*! M33_LPTMR2 - LPTMR2 debug halted mode with M33
 *  0b1..LPTMR2 enters debug halted mode when CM33 is debug halted
 *  0b0..LPTMR2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPTMR2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPTMR2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_LPTMR2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM3_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM3_SHIFT (10U)
/*! M33_TPM3 - debug halted mode with M33
 *  0b1..TPM3 enters debug halted mode when CM33 is debug halted
 *  0b0..TPM3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM4_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM4_SHIFT (11U)
/*! M33_TPM4 - TPM3 debug halted mode with M33
 *  0b1..enters debug halted mode when CM33 is debug halted
 *  0b0..does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM5_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM5_SHIFT (12U)
/*! M33_TPM5 - TPM5 debug halted mode with M33
 *  0b1..TPM5 enters debug halted mode when CM33 is debug halted
 *  0b0..TPM5 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM6_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM6_SHIFT (13U)
/*! M33_TPM6 - TPM6 debug halted mode with M33
 *  0b1..TPM6 enters debug halted mode when CM33 is debug halted
 *  0b0..TPM6 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_TPM6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG3_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG3_SHIFT (14U)
/*! M33_WDOG3 - WDOG3 debug halted mode with M33
 *  0b1..WDOG3 enters debug halted mode when CM33 is debug halted
 *  0b0..WDOG3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG4_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG4_SHIFT (15U)
/*! M33_WDOG4 - WDOG4 debug halted mode with M33
 *  0b1..WDOG4 enters debug halted mode when CM33 is debug halted
 *  0b0..WDOG4 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M33_WDOG4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_CAN2_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_CAN2_SHIFT (16U)
/*! M7_CAN2 - CAN2 debug halted mode with M7
 *  0b1..CAN2 enters debug halted mode when CM7 is debug halted
 *  0b0..CAN2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_CAN2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_CAN2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_CAN2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_EDMA4_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_EDMA4_SHIFT (17U)
/*! M7_EDMA4 - EDMA4 debug halted mode with M7
 *  0b1..EDMA4 enters debug halted mode when CM7 is debug halted
 *  0b0..EDMA4 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_EDMA4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_EDMA4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_EDMA4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO1_MASK (0x40000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO1_SHIFT (18U)
/*! M7_FLEXIO1 - FLEXIO1 debug halted mode with M7
 *  0b1..FLEXIO1 enters debug halted mode when CM7 is debug halted
 *  0b0..FLEXIO1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO2_MASK (0x80000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO2_SHIFT (19U)
/*! M7_FLEXIO2 - FLEXIO2 debug halted mode with M7
 *  0b1..FLEXIO2 enters debug halted mode when CM7 is debug halted
 *  0b0..FLEXIO2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_FLEXIO2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C3_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C3_SHIFT (20U)
/*! M7_LPI2C3 - LPI2C3 debug halted mode with M7
 *  0b1..LPI2C3 enters debug halted mode when CM7 is debug halted
 *  0b0..LPI2C3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C4_MASK (0x200000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C4_SHIFT (21U)
/*! M7_LPI2C4 - LPI2C4 debug halted mode with M7
 *  0b1..LPI2C4 enters debug halted mode when CM7 is debug halted
 *  0b0..LPI2C4 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPI2C4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPIT2_MASK (0x400000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPIT2_SHIFT (22U)
/*! M7_LPIT2 - LPIT2 debug halted mode with M7
 *  0b1..LPIT2 enters debug halted mode when CM7 is debug halted
 *  0b0..LPIT2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPIT2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPIT2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPIT2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI3_MASK (0x800000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI3_SHIFT (23U)
/*! M7_LPSPI3 - WDOG3 debug halted mode with M7
 *  0b1..WDOG3 enters debug halted mode when CM7 is debug halted
 *  0b0..WDOD3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI4_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI4_SHIFT (24U)
/*! M7_LPSPI4 - LPSPI4 debug halted mode with M7
 *  0b1..LPSPI4 enters debug halted mode when CM7 is debug halted
 *  0b0..LPSPI4 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPSPI4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPTMR2_MASK (0x2000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPTMR2_SHIFT (25U)
/*! M7_LPTMR2 - LPTMR2 debug halted mode with M7
 *  0b1..LPTMR2 enters debug halted mode when CM7 is debug halted
 *  0b0..LPTMR2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPTMR2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPTMR2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_LPTMR2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM3_MASK (0x4000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM3_SHIFT (26U)
/*! M7_TPM3 - TPM3 debug halted mode with M7
 *  0b1..TPM3 enters debug halted mode when CM7 is debug halted
 *  0b0..TPM3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM4_MASK (0x8000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM4_SHIFT (27U)
/*! M7_TPM4 - TPM4 debug halted mode with M7
 *  0b1..TPM4 enters debug halted mode when CM7 is debug halted
 *  0b0..TPM4 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM5_MASK (0x10000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM5_SHIFT (28U)
/*! M7_TPM5 - TPM5 debug halted mode with M7
 *  0b1..TPM5 enters debug halted mode when CM7 is debug halted
 *  0b0..TPM5 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM6_MASK (0x20000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM6_SHIFT (29U)
/*! M7_TPM6 - TPM6 debug halted mode with M7
 *  0b1..TPM5 enters debug halted mode when CM7 is debug halted
 *  0b0..TPM5 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_TPM6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG3_MASK (0x40000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG3_SHIFT (30U)
/*! M7_WDOG3 - WDOG3 debug halted mode with M7 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG4_MASK (0x80000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG4_SHIFT (31U)
/*! M7_WDOG4 - WDOG4 debug halted mode with M7 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG1_M7_WDOG4_MASK)
/*! @} */

/*! @name IPG_DEBUG2 - IPG DEBUG mask bit */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_WDOG5_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_WDOG5_SHIFT (0U)
/*! M33_WDOG5 - WDOG5 debug halted mode with M7
 *  0b1..WDOG5 enters debug halted mode when CM33 is debug halted
 *  0b0..WDOG5 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_WDOG5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_WDOG5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_WDOG5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPTMR3_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPTMR3_SHIFT (1U)
/*! M33_LPTMR3 - LPTMR3 debug halted mode with M33
 *  0b1..LPTMR3 enters debug halted mode when CM33 is debug halted
 *  0b0..LPTMR3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPTMR3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPTMR3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPTMR3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI5_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI5_SHIFT (2U)
/*! M33_LPSPI5 - LPSPI5 debug halted mode with M33
 *  0b1..LPSPI5 enters debug halted mode when CM33 is debug halted
 *  0b0..LPSPI5 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI6_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI6_SHIFT (3U)
/*! M33_LPSPI6 - LPSPI6 debug halted mode with M33
 *  0b1..LPSPI6 enters debug halted mode when CM33 is debug halted
 *  0b0..LPSPI6 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPSPI6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPIT3_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPIT3_SHIFT (4U)
/*! M33_LPIT3 - LPIT3 debug halted mode with M33
 *  0b1..LPIT3 enters debug halted mode when CM33 is debug halted
 *  0b0..LPIT3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPIT3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPIT3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPIT3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C5_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C5_SHIFT (5U)
/*! M33_LPI2C5 - LPI2C5 debug halted mode with M33
 *  0b1..LPI2C5 enters debug halted mode when CM33 is debug halted
 *  0b0..LPI2C5 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C6_MASK (0x40U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C6_SHIFT (6U)
/*! M33_LPI2C6 - LPI2C6 debug halted mode with M33
 *  0b1..LPI2C6 enters debug halted mode when CM33 is debug halted
 *  0b0..LPI2C6 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_LPI2C6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_GPT2_MASK (0x80U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_GPT2_SHIFT (7U)
/*! M33_GPT2 - GPT2 debug halted mode with M33
 *  0b1..GPT2 enters debug halted mode when CM33 is debug halted
 *  0b0..GPT2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_GPT2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_GPT2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_GPT2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM1_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM1_SHIFT (8U)
/*! M33_FLEXPWM1 - FLEXPWM1 debug halted mode with M33
 *  0b1..FLEXPWM1 enters debug halted mode when CM33 is debug halted
 *  0b0..FLEXPWM1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM2_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM2_SHIFT (9U)
/*! M33_FLEXPWM2 - FLEXPWM2 debug halted mode with M33
 *  0b1..FLEXPWM2 enters debug halted mode when CM33 is debug halted
 *  0b0..FLEXPWM2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM3_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM3_SHIFT (10U)
/*! M33_FLEXPWM3 - FLEXPWM3 debug halted mode with M33
 *  0b1..FLEXPWM3 enters debug halted mode when CM33 is debug halted
 *  0b0..FLEXPWM3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM4_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM4_SHIFT (11U)
/*! M33_FLEXPWM4 - FLEXPWM4 debug halted mode with M33
 *  0b1..FLEXPWM4 enters debug halted mode when CM33 is debug halted
 *  0b0..FLEXPWM4 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_FLEXPWM4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_MIC_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_MIC_SHIFT (12U)
/*! M33_MIC - MIC debug halted mode with M33
 *  0b1..MIC enters debug halted mode when CM33 is debug halted
 *  0b0..MIC does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_MIC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_MIC_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_MIC_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI2_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI2_SHIFT (13U)
/*! M33_SAI2 - SAI2 debug halted mode with M33
 *  0b1..SAI2 enters debug halted mode when CM33 is debug halted
 *  0b0..SAI2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI3_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI3_SHIFT (14U)
/*! M33_SAI3 - SAI3 debug halted mode with M33
 *  0b1..SAI3 enters debug halted mode when CM33 is debug halted
 *  0b0..SAI3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI4_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI4_SHIFT (15U)
/*! M33_SAI4 - SAI4 debug halted mode with M33
 *  0b1..SAI4 enters debug halted mode when CM33 is debug halted
 *  0b0..SAI4 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M33_SAI4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_WDOG5_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_WDOG5_SHIFT (16U)
/*! M7_WDOG5 - WDOG5 debug halted mode with M7
 *  0b1..WDOG5 enters debug halted mode when CM7 is debug halted
 *  0b0..WDOG5 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_WDOG5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_WDOG5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_WDOG5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPTMR3_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPTMR3_SHIFT (17U)
/*! M7_LPTMR3 - LPTMR3 debug halted mode with M7
 *  0b1..LPTMR3 enters debug halted mode when CM7 is debug halted
 *  0b0..LPTMR3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPTMR3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPTMR3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPTMR3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI5_MASK (0x40000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI5_SHIFT (18U)
/*! M7_LPSPI5 - LPTMR3 debug halted mode with M7
 *  0b1..enters debug halted mode when CM7 is debug halted
 *  0b0..LPTMR3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI6_MASK (0x80000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI6_SHIFT (19U)
/*! M7_LPSPI6 - LPSPI6 debug halted mode with M7
 *  0b1..LPSPI6 enters debug halted mode when CM7 is debug halted
 *  0b0..LPSPI6 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPSPI6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPIT3_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPIT3_SHIFT (20U)
/*! M7_LPIT3 - LPIT3 debug halted mode with M7
 *  0b1..LPIT3 enters debug halted mode when CM7 is debug halted
 *  0b0..LPIT3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPIT3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPIT3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPIT3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C5_MASK (0x200000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C5_SHIFT (21U)
/*! M7_LPI2C5 - LPI2C5 debug halted mode with M7
 *  0b1..LPI2C5 enters debug halted mode when CM7 is debug halted
 *  0b0..LPI2C5 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C6_MASK (0x400000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C6_SHIFT (22U)
/*! M7_LPI2C6 - LPI2C6" debug halted mode with M7
 *  0b1..LPI2C6" enters debug halted mode when CM7 is debug halted
 *  0b0..LPI2C6" does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_LPI2C6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_GPT2_MASK (0x800000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_GPT2_SHIFT (23U)
/*! M7_GPT2 - GPT2 debug halted mode with M7
 *  0b1..GPT2 enters debug halted mode when CM7 is debug halted
 *  0b0..GPT2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_GPT2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_GPT2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_GPT2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM1_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM1_SHIFT (24U)
/*! M7_FLEXPWM1 - FLEXPWM1 debug halted mode with M7
 *  0b1..FLEXPWM1 enters debug halted mode when CM7 is debug halted
 *  0b0..FLEXPWM1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM2_MASK (0x2000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM2_SHIFT (25U)
/*! M7_FLEXPWM2 - FLEXPWM2 debug halted mode with M7
 *  0b1..FLEXPWM2 enters debug halted mode when CM7 is debug halted
 *  0b0..FLEXPWM2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM3_MASK (0x4000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM3_SHIFT (26U)
/*! M7_FLEXPWM3 - FLEXPWM3 debug halted mode with M7
 *  0b1..FLEXPWM3 enters debug halted mode when CM7 is debug halted
 *  0b0..FLEXPWM3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM4_MASK (0x8000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM4_SHIFT (27U)
/*! M7_FLEXPWM4 - FLEXPWM4 debug halted mode with M7
 *  0b1..FLEXPWM4 enters debug halted mode when CM7 is debug halted
 *  0b0..FLEXPWM4 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_FLEXPWM4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_MIC_MASK (0x10000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_MIC_SHIFT (28U)
/*! M7_MIC - MIC debug halted mode with M7
 *  0b1..MIC enters debug halted mode when CM7 is debug halted
 *  0b0..MIC does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_MIC(x)  (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_MIC_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_MIC_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI2_MASK (0x20000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI2_SHIFT (29U)
/*! M7_SAI2 - SAI2 debug halted mode with M7
 *  0b1..SAI2 enters debug halted mode when CM7 is debug halted
 *  0b0..SAI2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI3_MASK (0x40000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI3_SHIFT (30U)
/*! M7_SAI3 - SAI3 debug halted mode with M7
 *  0b1..SAI3 enters debug halted mode when CM7 is debug halted
 *  0b0..SAI3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI4_MASK (0x80000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI4_SHIFT (31U)
/*! M7_SAI4 - SAI4 debug halted mode with M7
 *  0b1..SAI4 enters debug halted mode when CM7 is debug halted
 *  0b0..SAI4 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG2_M7_SAI4_MASK)
/*! @} */

/*! @name IPG_DEBUG3 - IPG DEBUG mask bit */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC1_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC1_SHIFT (0U)
/*! M33_SINC1 - I3C2 debug halted mode with M33
 *  0b1..I3C2 enters debug halted mode when CM33 is debug halted
 *  0b0..I3C2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC2_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC2_SHIFT (1U)
/*! M33_SINC2 - SINC2 debug halted mode with M33
 *  0b1..SINC2 enters debug halted mode when CM33 is debug halted
 *  0b0..SINC2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC3_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC3_SHIFT (2U)
/*! M33_SINC3 - SINC3 debug halted mode with M33
 *  0b1..SINC3 enters debug halted mode when CM33 is debug halted
 *  0b0..SINC3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_SINC3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER1_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER1_SHIFT (3U)
/*! M33_QTIMER1 - QTIMER1 debug halted mode with M33
 *  0b1..QTIMER1 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER1 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER2_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER2_SHIFT (4U)
/*! M33_QTIMER2 - QTIMER2 debug halted mode with M33
 *  0b1..QTIMER2 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER3_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER3_SHIFT (5U)
/*! M33_QTIMER3 - QTIMER3 debug halted mode with M33
 *  0b1..QTIMER3 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER3 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER4_MASK (0x40U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER4_SHIFT (6U)
/*! M33_QTIMER4 - QTIMER4 debug halted mode with M33
 *  0b1..QTIMER4 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER4 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER5_MASK (0x80U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER5_SHIFT (7U)
/*! M33_QTIMER5 - QTIMER5 debug halted mode with M33
 *  0b1..QTIMER5 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER5 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER6_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER6_SHIFT (8U)
/*! M33_QTIMER6 - QTIMER6 debug halted mode with M33
 *  0b1..QTIMER6 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER6 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER7_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER7_SHIFT (9U)
/*! M33_QTIMER7 - QTIMER7 debug halted mode with M33
 *  0b1..QTIMER7 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER7 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER7(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER7_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER7_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER8_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER8_SHIFT (10U)
/*! M33_QTIMER8 - QTIMER8 debug halted mode with M33
 *  0b1..QTIMER8 enters debug halted mode when CM33 is debug halted
 *  0b0..QTIMER8 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER8(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER8_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_QTIMER8_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_I3C2_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_I3C2_SHIFT (11U)
/*! M33_I3C2 - I3C2 debug halted mode with M33
 *  0b1..I3C2 enters debug halted mode when CM33 is debug halted
 *  0b0..I3C2 does not enter debug halted mode with CM33
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_I3C2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_I3C2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M33_I3C2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC1_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC1_SHIFT (16U)
/*! M7_SINC1 - SINC1 debug halted mode with M7
 *  0b1..SINC1 enters debug halted mode when CM7 is debug halted
 *  0b0..SINC1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC2_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC2_SHIFT (17U)
/*! M7_SINC2 - SINC2 debug halted mode with M7
 *  0b1..SINC2 enters debug halted mode when CM7 is debug halted
 *  0b0..SINC2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC3_MASK (0x40000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC3_SHIFT (18U)
/*! M7_SINC3 - SINC3 debug halted mode with M7
 *  0b1..SINC3 enters debug halted mode when CM7 is debug halted
 *  0b0..SINC3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_SINC3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER1_MASK (0x80000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER1_SHIFT (19U)
/*! M7_QTIMER1 - QTIMER1 debug halted mode with M7
 *  0b1..QTIMER1 enters debug halted mode when CM7 is debug halted
 *  0b0..QTIMER1 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER1_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER1_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER2_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER2_SHIFT (20U)
/*! M7_QTIMER2 - QTIMER2 debug halted mode with M7
 *  0b1..QTIMER2 enters debug halted mode when CM7 is debug halted
 *  0b0..QTIMER2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER2_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER3_MASK (0x200000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER3_SHIFT (21U)
/*! M7_QTIMER3 - QTIMER3 debug halted mode with M7
 *  0b1..QTIMER3 enters debug halted mode when CM7 is debug halted
 *  0b0..QTIMER3 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER3_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER3_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER4_MASK (0x400000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER4_SHIFT (22U)
/*! M7_QTIMER4 - QTIMER4 debug halted mode with M7
 *  0b1..QTIMER4 enters debug halted mode when CM7 is debug halted
 *  0b0..QTIMER4 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER4_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER4_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER5_MASK (0x800000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER5_SHIFT (23U)
/*! M7_QTIMER5 - QTIMER5 debug halted mode with M7
 *  0b1..QTIMER5 enters debug halted mode when CM7 is debug halted
 *  0b0..QTIMER5 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER5_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER5_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER6_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER6_SHIFT (24U)
/*! M7_QTIMER6 - debug halted mode with M7
 *  0b1..enters debug halted mode when CM7 is debug halted
 *  0b0..does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER6_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER6_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER7_MASK (0x2000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER7_SHIFT (25U)
/*! M7_QTIMER7 - QTIMER7 debug halted mode with M7
 *  0b1..QTIMER7 enters debug halted mode when CM7 is debug halted
 *  0b0..QTIMER7 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER7(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER7_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER7_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER8_MASK (0x4000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER8_SHIFT (26U)
/*! M7_QTIMER8 - QTIMER8 debug halted mode with M7
 *  0b1..QTIMER8 enters debug halted mode when CM7 is debug halted
 *  0b0..QTIMER8 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER8(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER8_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_QTIMER8_MASK)

#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_I3C2_MASK (0x8000000U)
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_I3C2_SHIFT (27U)
/*! M7_I3C2 - debug halted mode with M7
 *  0b1..I3C2 enters debug halted mode when CM7 is debug halted
 *  0b0..I3C2 does not enter debug halted mode with CM7
 */
#define BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_I3C2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_I3C2_SHIFT)) & BLK_CTRL_WAKEUPMIX_IPG_DEBUG3_M7_I3C2_MASK)
/*! @} */

/*! @name SSI - SSI master low power mode control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SSI_SSI_IDLE_MASK     (0x1U)
#define BLK_CTRL_WAKEUPMIX_SSI_SSI_IDLE_SHIFT    (0U)
/*! SSI_IDLE - WAKEUP Domain to M7 SSI master idle
 *  0b1..WAKEUP Domain to M7 SSI master is idle
 *  0b0..WAKEUP Domain to M7 SSI master is not idle
 */
#define BLK_CTRL_WAKEUPMIX_SSI_SSI_IDLE(x)       (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SSI_SSI_IDLE_SHIFT)) & BLK_CTRL_WAKEUPMIX_SSI_SSI_IDLE_MASK)

#define BLK_CTRL_WAKEUPMIX_SSI_BLKHOLE_MODE_B_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_SSI_BLKHOLE_MODE_B_SHIFT (1U)
/*! BLKHOLE_MODE_B - WAKEUP Domain to M7 SSI master blackhole mode
 *  0b1..WAKEUP Domain to M7 SSI master will enter blackhole mode
 *  0b0..WAKEUP Domain to M7 SSI master will exit blackhole mode
 */
#define BLK_CTRL_WAKEUPMIX_SSI_BLKHOLE_MODE_B(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SSI_BLKHOLE_MODE_B_SHIFT)) & BLK_CTRL_WAKEUPMIX_SSI_BLKHOLE_MODE_B_MASK)

#define BLK_CTRL_WAKEUPMIX_SSI_PAUSE_MODE_MASK   (0x4U)
#define BLK_CTRL_WAKEUPMIX_SSI_PAUSE_MODE_SHIFT  (2U)
/*! PAUSE_MODE - WAKEUP Domain to M7 SSI master pause mode
 *  0b1..WAKEUP Domain to M7 SSI master will exit pause mode
 *  0b0..WAKEUP Domain to M7 SSI master will enter pause mode
 */
#define BLK_CTRL_WAKEUPMIX_SSI_PAUSE_MODE(x)     (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SSI_PAUSE_MODE_SHIFT)) & BLK_CTRL_WAKEUPMIX_SSI_PAUSE_MODE_MASK)
/*! @} */

/*! @name ECAT_MISC_CFG - EtherCAT miscellaneous configuration */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL0_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL0_SHIFT (0U)
/*! RMII_SEL0 - RMII mode selection for EtherCAT port 0
 *  0b0..EtherCAT port0 is in MII mode
 *  0b1..EtherCAT port0 is in RMII mode
 */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL0(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL0_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL0_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL1_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL1_SHIFT (1U)
/*! RMII_SEL1 - RMII mode selection for EtherCAT port 1
 *  0b0..EtherCAT port1 is in MII mode
 *  0b1..EtherCAT port1 is in RMII mode
 */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL1_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_SEL1_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_EN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_EN_SHIFT (2U)
/*! GLB_EN - Global enable of EtherCAT
 *  0b1..EtherCAT is on
 *  0b0..EtherCAT is off
 */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_RST_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_RST_SHIFT (3U)
/*! GLB_RST - Global reset of EtherCAT
 *  0b0..EtherCAT is out of reset
 *  0b1..EtherCAT is held in reset
 */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_RST(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_RST_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_GLB_RST_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR0_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR0_SHIFT (8U)
/*! RMII_REF_CLK_DIR0 - RMII Port0 REF_CLK direction control
 *  0b0..RMII REF_CLK is input
 *  0b1..RMII REF_CLK is output driven by ECAT_CLK_ROOT/2
 */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR0(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR0_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR0_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR1_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR1_SHIFT (9U)
/*! RMII_REF_CLK_DIR1 - RMII Port1 REF_CLK direction control
 *  0b0..RMII REF_CLK is input
 *  0b1..RMII REF_CLK is output driven by ECAT_CLK_ROOT/2
 */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR1_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_RMII_REF_CLK_DIR1_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_SHIFT (16U)
/*! PHY_OFFSET - EtherCAT PHY_OFFSET */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_VEC_MASK (0x3E0000U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_VEC_SHIFT (17U)
/*! PHY_OFFSET_VEC - EtherCAT PHY_OFFSET_VEC */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_VEC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_VEC_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_PHY_OFFSET_VEC_MASK)

#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_EEPROM_SIZE_OPTION_MASK (0x400000U)
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_EEPROM_SIZE_OPTION_SHIFT (22U)
/*! EEPROM_SIZE_OPTION - EtherCAT EEPROM SIZE OPTION */
#define BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_EEPROM_SIZE_OPTION(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_EEPROM_SIZE_OPTION_SHIFT)) & BLK_CTRL_WAKEUPMIX_ECAT_MISC_CFG_EEPROM_SIZE_OPTION_MASK)
/*! @} */

/*! @name DEXSC_ERR - DEXSC error response configuration */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_DEXSC_ERR_EXC_ERR_RESP_EN_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_DEXSC_ERR_EXC_ERR_RESP_EN_SHIFT (0U)
/*! EXC_ERR_RESP_EN - Exclusive error response enable
 *  0b0..OKAY response
 *  0b1..SLVError response
 */
#define BLK_CTRL_WAKEUPMIX_DEXSC_ERR_EXC_ERR_RESP_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_DEXSC_ERR_EXC_ERR_RESP_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_DEXSC_ERR_EXC_ERR_RESP_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_DEXSC_ERR_LOCK_EXC_ERR_RESP_EN_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_DEXSC_ERR_LOCK_EXC_ERR_RESP_EN_SHIFT (1U)
/*! LOCK_EXC_ERR_RESP_EN - Lock bit of EXC_ERR_RESP_EN */
#define BLK_CTRL_WAKEUPMIX_DEXSC_ERR_LOCK_EXC_ERR_RESP_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_DEXSC_ERR_LOCK_EXC_ERR_RESP_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_DEXSC_ERR_LOCK_EXC_ERR_RESP_EN_MASK)
/*! @} */

/*! @name USBPHY_MISC_CTRL - USBPHY miscellaneous control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_IPG_CLK_ACTIVE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_IPG_CLK_ACTIVE_SHIFT (0U)
/*! USBPHY1_IPG_CLK_ACTIVE - USBPHY1 register access clock enable */
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_IPG_CLK_ACTIVE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_IPG_CLK_ACTIVE_SHIFT)) & BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_IPG_CLK_ACTIVE_MASK)

#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_IPG_CLK_ACTIVE_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_IPG_CLK_ACTIVE_SHIFT (8U)
/*! USBPHY2_IPG_CLK_ACTIVE - USBPHY2 register access clock enable */
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_IPG_CLK_ACTIVE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_IPG_CLK_ACTIVE_SHIFT)) & BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_IPG_CLK_ACTIVE_MASK)

#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_WAKEUP_IRQ_CLEAR_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_WAKEUP_IRQ_CLEAR_SHIFT (16U)
/*! USBPHY1_WAKEUP_IRQ_CLEAR - Clear USBPHY1 wakeup interrupt holding register */
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_WAKEUP_IRQ_CLEAR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_WAKEUP_IRQ_CLEAR_SHIFT)) & BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY1_WAKEUP_IRQ_CLEAR_MASK)

#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_WAKEUP_IRQ_CLEAR_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_WAKEUP_IRQ_CLEAR_SHIFT (24U)
/*! USBPHY2_WAKEUP_IRQ_CLEAR - Clear USBPHY2 wakeup interrupt holding register */
#define BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_WAKEUP_IRQ_CLEAR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_WAKEUP_IRQ_CLEAR_SHIFT)) & BLK_CTRL_WAKEUPMIX_USBPHY_MISC_CTRL_USBPHY2_WAKEUP_IRQ_CLEAR_MASK)
/*! @} */

/*! @name NETC_PORT_MISC_CFG - NETC Port miscellaneous configuration */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT0_RMII_REF_CLK_DIR_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT0_RMII_REF_CLK_DIR_SHIFT (0U)
/*! PORT0_RMII_REF_CLK_DIR - Port0 RMII Reference clock direction control */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT0_RMII_REF_CLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT0_RMII_REF_CLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT0_RMII_REF_CLK_DIR_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT1_RMII_REF_CLK_DIR_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT1_RMII_REF_CLK_DIR_SHIFT (1U)
/*! PORT1_RMII_REF_CLK_DIR - Port1 RMII Reference clock direction control
 *  0b0..Port1 RMII Reference clock is input
 *  0b1..Port1 RMII Reference clock is output
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT1_RMII_REF_CLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT1_RMII_REF_CLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT1_RMII_REF_CLK_DIR_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT2_RMII_REF_CLK_DIR_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT2_RMII_REF_CLK_DIR_SHIFT (2U)
/*! PORT2_RMII_REF_CLK_DIR - Port2 RMII Reference clock direction control
 *  0b0..Port2 RMII Reference clock is input
 *  0b1..Port2 RMII Reference clock is output
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT2_RMII_REF_CLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT2_RMII_REF_CLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT2_RMII_REF_CLK_DIR_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT3_RMII_REF_CLK_DIR_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT3_RMII_REF_CLK_DIR_SHIFT (3U)
/*! PORT3_RMII_REF_CLK_DIR - Port3 RMII Reference clock direction control
 *  0b0..Port3 RMII Reference clock is input
 *  0b1..Port3 RMII Reference clock is output
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT3_RMII_REF_CLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT3_RMII_REF_CLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT3_RMII_REF_CLK_DIR_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT4_RMII_REF_CLK_DIR_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT4_RMII_REF_CLK_DIR_SHIFT (4U)
/*! PORT4_RMII_REF_CLK_DIR - Port4 RMII Reference clock direction control
 *  0b0..Port4 RMII Reference clock is input
 *  0b1..Port4 RMII Reference clock is output
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT4_RMII_REF_CLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT4_RMII_REF_CLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_PORT4_RMII_REF_CLK_DIR_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_CFG_IERB_LOCK_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_CFG_IERB_LOCK_SHIFT (8U)
/*! CFG_IERB_LOCK - Default value for IERB NETCRR[LOCK] bit. Determines write accessibility of IERB registers after power-on-reset
 *  0b0..Unlocked after power-on-reset. Normal read/write access to all IERB registers
 *  0b1..Locked after power-on-reset. Write access inhibited to all IERB registers, except NETCRR
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_CFG_IERB_LOCK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_CFG_IERB_LOCK_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_CFG_IERB_LOCK_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_EXT_CLK_SEL_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_EXT_CLK_SEL_SHIFT (24U)
/*! TMR_EXT_CLK_SEL - 1588 timer external clock selection
 *  0b0..CCM tmr_1588_clk_root is selected
 *  0b1..External pin is selected
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_EXT_CLK_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_EXT_CLK_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_EXT_CLK_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG1_SEL_MASK (0x2000000U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG1_SEL_SHIFT (25U)
/*! TMR_TRIG1_SEL - 1588 timer trigger1 input selection
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG1_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG1_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG1_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG2_SEL_MASK (0x4000000U)
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG2_SEL_SHIFT (26U)
/*! TMR_TRIG2_SEL - 1588 timer trigger2 input selection
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG2_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG2_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_PORT_MISC_CFG_TMR_TRIG2_SEL_MASK)
/*! @} */

/*! @name M7_NMI_CLR - M7 NMI interrupt clear register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_M7_NMI_CLR_M7_NMI_CLEAR_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_M7_NMI_CLR_M7_NMI_CLEAR_SHIFT (0U)
/*! M7_NMI_CLEAR - Clear CM7 NMI holding register */
#define BLK_CTRL_WAKEUPMIX_M7_NMI_CLR_M7_NMI_CLEAR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_M7_NMI_CLR_M7_NMI_CLEAR_SHIFT)) & BLK_CTRL_WAKEUPMIX_M7_NMI_CLR_M7_NMI_CLEAR_MASK)
/*! @} */

/*! @name QTIMER_CTRL1 - Qtimer miscellaneous control register 1 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR_CNTS_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR_CNTS_FREEZE_SHIFT (0U)
/*! QTIMER1_TMR_CNTS_FREEZE - QTIMER1 timer counter freeze
 *  0b0..Timer counter works normally
 *  0b1..Reset counter and output flags
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR0_INPUT_SEL_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR0_INPUT_SEL_SHIFT (1U)
/*! QTIMER1_TMR0_INPUT_SEL - QTIMER1 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR1_INPUT_SEL_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR1_INPUT_SEL_SHIFT (2U)
/*! QTIMER1_TMR1_INPUT_SEL - QTIMER1 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR2_INPUT_SEL_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR2_INPUT_SEL_SHIFT (3U)
/*! QTIMER1_TMR2_INPUT_SEL - QTIMER1 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR3_INPUT_SEL_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR3_INPUT_SEL_SHIFT (4U)
/*! QTIMER1_TMR3_INPUT_SEL - QTIMER1 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER1_TMR3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR_CNTS_FREEZE_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR_CNTS_FREEZE_SHIFT (8U)
/*! QTIMER2_TMR_CNTS_FREEZE - QTIMER2 timer counter freeze
 *  0b0..Timer counter works normally
 *  0b1..Reset counter and output flags
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR0_INPUT_SEL_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR0_INPUT_SEL_SHIFT (9U)
/*! QTIMER2_TMR0_INPUT_SEL - QTIMER2 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR1_INPUT_SEL_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR1_INPUT_SEL_SHIFT (10U)
/*! QTIMER2_TMR1_INPUT_SEL - QTIMER2 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR2_INPUT_SEL_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR2_INPUT_SEL_SHIFT (11U)
/*! QTIMER2_TMR2_INPUT_SEL - QTIMER2 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR3_INPUT_SEL_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR3_INPUT_SEL_SHIFT (12U)
/*! QTIMER2_TMR3_INPUT_SEL - QTIMER2 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER2_TMR3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR_CNTS_FREEZE_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR_CNTS_FREEZE_SHIFT (16U)
/*! QTIMER3_TMR_CNTS_FREEZE - QTIMER3 timer counter freeze
 *  0b0..Timer counter works normally
 *  0b1..Reset counter and ouput flags
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR0_INPUT_SEL_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR0_INPUT_SEL_SHIFT (17U)
/*! QTIMER3_TMR0_INPUT_SEL - QTIMER3 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR1_INPUT_SEL_MASK (0x40000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR1_INPUT_SEL_SHIFT (18U)
/*! QTIMER3_TMR1_INPUT_SEL - QTIMER3 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR2_INPUT_SEL_MASK (0x80000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR2_INPUT_SEL_SHIFT (19U)
/*! QTIMER3_TMR2_INPUT_SEL - QTIMER3 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR3_INPUT_SEL_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR3_INPUT_SEL_SHIFT (20U)
/*! QTIMER3_TMR3_INPUT_SEL - QTIMER3 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER3_TMR3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR_CNTS_FREEZE_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR_CNTS_FREEZE_SHIFT (24U)
/*! QTIMER4_TMR_CNTS_FREEZE - QTIMER4 timer counter freeze
 *  0b0..Timer counter works normally
 *  0b1..Reset counter and output flags
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR0_INPUT_SEL_MASK (0x2000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR0_INPUT_SEL_SHIFT (25U)
/*! QTIMER4_TMR0_INPUT_SEL - QTIMER4 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR1_INPUT_SEL_MASK (0x4000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR1_INPUT_SEL_SHIFT (26U)
/*! QTIMER4_TMR1_INPUT_SEL - QTIMER4 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR2_INPUT_SEL_MASK (0x8000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR2_INPUT_SEL_SHIFT (27U)
/*! QTIMER4_TMR2_INPUT_SEL - QTIMER4 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR3_INPUT_SEL_MASK (0x10000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR3_INPUT_SEL_SHIFT (28U)
/*! QTIMER4_TMR3_INPUT_SEL - QTIMER4 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL1_QTIMER4_TMR3_INPUT_SEL_MASK)
/*! @} */

/*! @name QTIMER_CTRL2 - Qtimer miscellaneous control register 2 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR_CNTS_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR_CNTS_FREEZE_SHIFT (0U)
/*! QTIMER5_TMR_CNTS_FREEZE - QTIMER5 timer counter freeze
 *  0b0..Timer counter works normally
 *  0b1..Reset counter and output flags
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR0_INPUT_SEL_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR0_INPUT_SEL_SHIFT (1U)
/*! QTIMER5_TMR0_INPUT_SEL - QTIMER5 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR1_INPUT_SEL_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR1_INPUT_SEL_SHIFT (2U)
/*! QTIMER5_TMR1_INPUT_SEL - QTIMER5 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR2_INPUT_SEL_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR2_INPUT_SEL_SHIFT (3U)
/*! QTIMER5_TMR2_INPUT_SEL - QTIMER5 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR3_INPUT_SEL_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR3_INPUT_SEL_SHIFT (4U)
/*! QTIMER5_TMR3_INPUT_SEL - QTIMER5 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER5_TMR3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR_CNTS_FREEZE_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR_CNTS_FREEZE_SHIFT (8U)
/*! QTIMER6_TMR_CNTS_FREEZE - QTIMER6 timer counter freeze */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR0_INPUT_SEL_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR0_INPUT_SEL_SHIFT (9U)
/*! QTIMER6_TMR0_INPUT_SEL - QTIMER6 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR1_INPUT_SEL_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR1_INPUT_SEL_SHIFT (10U)
/*! QTIMER6_TMR1_INPUT_SEL - QTIMER6 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR2_INPUT_SEL_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR2_INPUT_SEL_SHIFT (11U)
/*! QTIMER6_TMR2_INPUT_SEL - QTIMER6 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR3_INPUT_SEL_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR3_INPUT_SEL_SHIFT (12U)
/*! QTIMER6_TMR3_INPUT_SEL - QTIMER6 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER6_TMR3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR_CNTS_FREEZE_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR_CNTS_FREEZE_SHIFT (16U)
/*! QTIMER7_TMR_CNTS_FREEZE - QTIMER7 timer counter freeze
 *  0b0..Timer counter works normally
 *  0b1..Reset counter and output flags
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR0_INPUT_SEL_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR0_INPUT_SEL_SHIFT (17U)
/*! QTIMER7_TMR0_INPUT_SEL - QTIMER7 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR1_INPUT_SEL_MASK (0x40000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR1_INPUT_SEL_SHIFT (18U)
/*! QTIMER7_TMR1_INPUT_SEL - QTIMER7 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR2_INPUT_SEL_MASK (0x80000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR2_INPUT_SEL_SHIFT (19U)
/*! QTIMER7_TMR2_INPUT_SEL - QTIMER7 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR3_INPUT_SEL_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR3_INPUT_SEL_SHIFT (20U)
/*! QTIMER7_TMR3_INPUT_SEL - QTIMER7 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER7_TMR3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR_CNTS_FREEZE_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR_CNTS_FREEZE_SHIFT (24U)
/*! QTIMER8_TMR_CNTS_FREEZE - QTIMER8 timer counter freeze
 *  0b0..Timer counter works normally
 *  0b1..Reset counter and output flags
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR_CNTS_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR_CNTS_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR_CNTS_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR0_INPUT_SEL_MASK (0x2000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR0_INPUT_SEL_SHIFT (25U)
/*! QTIMER8_TMR0_INPUT_SEL - QTIMER8 TMR0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR1_INPUT_SEL_MASK (0x4000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR1_INPUT_SEL_SHIFT (26U)
/*! QTIMER8_TMR1_INPUT_SEL - QTIMER8 TMR1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR2_INPUT_SEL_MASK (0x8000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR2_INPUT_SEL_SHIFT (27U)
/*! QTIMER8_TMR2_INPUT_SEL - QTIMER8 TMR2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR3_INPUT_SEL_MASK (0x10000000U)
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR3_INPUT_SEL_SHIFT (28U)
/*! QTIMER8_TMR3_INPUT_SEL - QTIMER8 TMR3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_QTIMER_CTRL2_QTIMER8_TMR3_INPUT_SEL_MASK)
/*! @} */

/*! @name SAI2_MCLK_CTRL - SAI2 MCLK control register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK3_SEL_MASK (0x3U)
#define BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK3_SEL_SHIFT (0U)
/*! SAI2_MCLK3_SEL - SAI2 MCLK3 source select
 *  0b00..SPDIF_CLK_ROOT
 *  0b01..spdif_tx_clk2
 *  0b10..spdif_srclk
 *  0b11..spdif_outclock
 */
#define BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK3_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK3_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK3_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK_DIR_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK_DIR_SHIFT (8U)
/*! SAI2_MCLK_DIR - SAI2_MCLK IO direction control. IOMUX need select SAI2 MCLK function.
 *  0b0..SAI2_MCLK is input signal
 *  0b1..SAI2_MCLK is output signal
 */
#define BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI2_MCLK_CTRL_SAI2_MCLK_DIR_MASK)
/*! @} */

/*! @name SAI3_MCLK_CTRL - SAI3 MCLK control register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK3_SEL_MASK (0x3U)
#define BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK3_SEL_SHIFT (0U)
/*! SAI3_MCLK3_SEL - SAI3 MCLK3 source select
 *  0b00..SPDIF_CLK_ROOT
 *  0b01..spdif_tx_clk2
 *  0b10..spdif_srclk
 *  0b11..spdif_outclock
 */
#define BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK3_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK3_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK3_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK_DIR_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK_DIR_SHIFT (8U)
/*! SAI3_MCLK_DIR - SAI3_MCLK IO direction control. IOMUX need select SAI3 MCLK function.
 *  0b0..SAI3_MCLK is input signal
 *  0b1..SAI3_MCLK is output signal
 */
#define BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI3_MCLK_CTRL_SAI3_MCLK_DIR_MASK)
/*! @} */

/*! @name SAI4_MCLK_CTRL - SAI4 MCLK control register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK1_SEL_MASK (0x7U)
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK1_SEL_SHIFT (0U)
/*! SAI4_MCLK1_SEL - SAI4 MCLK1 source select
 *  0b000..SAI4_CLK_ROOT
 *  0b001..SAI2_CLK_ROOT
 *  0b011..SAI3_CLK_ROOT
 *  0b100..SAI4 MCLK IO pin
 *  0b101..SAI2 MCLK IO pin
 *  0b110..SAI3 MCLK IO pin
 *  0b111..Reserved
 */
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK1_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK1_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK1_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK2_SEL_MASK (0x38U)
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK2_SEL_SHIFT (3U)
/*! SAI4_MCLK2_SEL - SAI4 MCLK2 source select
 *  0b000..SAI4_CLK_ROOT
 *  0b001..SAI2_CLK_ROOT
 *  0b011..SAI3_CLK_ROOT
 *  0b100..SAI4 MCLK IO pin
 *  0b101..SAI2 MCLK IO pin
 *  0b110..SAI3 MCLK IO pin
 *  0b111..Reserved
 */
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK2_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK2_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK2_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK3_SEL_MASK (0xC0U)
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK3_SEL_SHIFT (6U)
/*! SAI4_MCLK3_SEL - SAI4 MCLK3 source select
 *  0b00..SPDIF_CLK_ROOT
 *  0b01..spdif_tx_clk2
 *  0b10..spdif_srclk
 *  0b11..spdif_outclock
 */
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK3_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK3_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK3_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK_DIR_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK_DIR_SHIFT (8U)
/*! SAI4_MCLK_DIR - SAI4_MCLK IO direction control. IOMUX need select SAI4 MCLK function. */
#define BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK_DIR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK_DIR_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAI4_MCLK_CTRL_SAI4_MCLK_DIR_MASK)
/*! @} */

/*! @name XBAR_DIR_CTRL1 - XBAR IO direction control register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_4_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_4_SHIFT (4U)
/*! IOMUXC_XBAR_DIR_SEL_4 - IOMUXC XBAR_INOUT4 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_4_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_4_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_5_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_5_SHIFT (5U)
/*! IOMUXC_XBAR_DIR_SEL_5 - IOMUXC XBAR_INOUT5 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_5_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_5_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_6_MASK (0x40U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_6_SHIFT (6U)
/*! IOMUXC_XBAR_DIR_SEL_6 - IOMUXC XBAR_INOUT6 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_6_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_6_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_7_MASK (0x80U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_7_SHIFT (7U)
/*! IOMUXC_XBAR_DIR_SEL_7 - IOMUXC XBAR_INOUT7 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_7(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_7_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_7_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_8_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_8_SHIFT (8U)
/*! IOMUXC_XBAR_DIR_SEL_8 - IOMUXC XBAR_INOUT8 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_8(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_8_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_8_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_9_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_9_SHIFT (9U)
/*! IOMUXC_XBAR_DIR_SEL_9 - IOMUXC XBAR_INOUT9 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_9(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_9_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_9_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_10_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_10_SHIFT (10U)
/*! IOMUXC_XBAR_DIR_SEL_10 - IOMUXC XBAR_INOUT10 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_10(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_10_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_10_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_11_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_11_SHIFT (11U)
/*! IOMUXC_XBAR_DIR_SEL_11 - IOMUXC XBAR_INOUT11 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_11(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_11_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_11_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_12_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_12_SHIFT (12U)
/*! IOMUXC_XBAR_DIR_SEL_12 - IOMUXC XBAR_INOUT12 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_12(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_12_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_12_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_13_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_13_SHIFT (13U)
/*! IOMUXC_XBAR_DIR_SEL_13 - IOMUXC XBAR_INOUT13 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_13(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_13_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_13_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_14_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_14_SHIFT (14U)
/*! IOMUXC_XBAR_DIR_SEL_14 - IOMUXC XBAR_INOUT14 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_14(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_14_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_14_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_15_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_15_SHIFT (15U)
/*! IOMUXC_XBAR_DIR_SEL_15 - IOMUXC XBAR_INOUT15 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_15(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_15_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_15_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_16_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_16_SHIFT (16U)
/*! IOMUXC_XBAR_DIR_SEL_16 - IOMUXC XBAR_INOUT16 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_16(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_16_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_16_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_17_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_17_SHIFT (17U)
/*! IOMUXC_XBAR_DIR_SEL_17 - IOMUXC XBAR_INOUT17 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_17(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_17_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_17_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_18_MASK (0x40000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_18_SHIFT (18U)
/*! IOMUXC_XBAR_DIR_SEL_18 - IOMUXC XBAR_INOUT18 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_18(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_18_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_18_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_19_MASK (0x80000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_19_SHIFT (19U)
/*! IOMUXC_XBAR_DIR_SEL_19 - IOMUXC XBAR_INOUT19 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_19(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_19_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_19_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_20_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_20_SHIFT (20U)
/*! IOMUXC_XBAR_DIR_SEL_20 - IOMUXC XBAR_INOUT20 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_20(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_20_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_20_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_21_MASK (0x200000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_21_SHIFT (21U)
/*! IOMUXC_XBAR_DIR_SEL_21 - IOMUXC XBAR_INOUT21 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_21(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_21_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_21_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_22_MASK (0x400000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_22_SHIFT (22U)
/*! IOMUXC_XBAR_DIR_SEL_22 - IOMUXC XBAR_INOUT22 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_22(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_22_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_22_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_23_MASK (0x800000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_23_SHIFT (23U)
/*! IOMUXC_XBAR_DIR_SEL_23 - IOMUXC XBAR_INOUT23 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_23(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_23_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_23_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_24_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_24_SHIFT (24U)
/*! IOMUXC_XBAR_DIR_SEL_24 - IOMUXC XBAR_INOUT24 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_24(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_24_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_24_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_25_MASK (0x2000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_25_SHIFT (25U)
/*! IOMUXC_XBAR_DIR_SEL_25 - IOMUXC XBAR_INOUT25 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_25(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_25_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_25_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_26_MASK (0x4000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_26_SHIFT (26U)
/*! IOMUXC_XBAR_DIR_SEL_26 - IOMUXC XBAR_INOUT26 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_26(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_26_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_26_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_27_MASK (0x8000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_27_SHIFT (27U)
/*! IOMUXC_XBAR_DIR_SEL_27 - IOMUXC XBAR_INOUT27 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_27(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_27_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_27_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_28_MASK (0x10000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_28_SHIFT (28U)
/*! IOMUXC_XBAR_DIR_SEL_28 - IOMUXC XBAR_INOUT28 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_28(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_28_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_28_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_29_MASK (0x20000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_29_SHIFT (29U)
/*! IOMUXC_XBAR_DIR_SEL_29 - IOMUXC XBAR_INOUT29 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_29(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_29_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_29_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_30_MASK (0x40000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_30_SHIFT (30U)
/*! IOMUXC_XBAR_DIR_SEL_30 - IOMUXC XBAR_INOUT30 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_30(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_30_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_30_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_31_MASK (0x80000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_31_SHIFT (31U)
/*! IOMUXC_XBAR_DIR_SEL_31 - IOMUXC XBAR_INOUT31 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_31(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_31_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL1_IOMUXC_XBAR_DIR_SEL_31_MASK)
/*! @} */

/*! @name XBAR_DIR_CTRL2 - XBAR IO direction control register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_32_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_32_SHIFT (0U)
/*! IOMUXC_XBAR_DIR_SEL_32 - IOMUXC XBAR_INOUT32 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_32(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_32_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_32_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_33_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_33_SHIFT (1U)
/*! IOMUXC_XBAR_DIR_SEL_33 - IOMUXC XBAR_INOUT33 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_33(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_33_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_33_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_34_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_34_SHIFT (2U)
/*! IOMUXC_XBAR_DIR_SEL_34 - IOMUXC XBAR_INOUT34 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_34(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_34_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_34_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_35_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_35_SHIFT (3U)
/*! IOMUXC_XBAR_DIR_SEL_35 - IOMUXC XBAR_INOUT35 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_35(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_35_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_35_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_36_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_36_SHIFT (4U)
/*! IOMUXC_XBAR_DIR_SEL_36 - IOMUXC XBAR_INOUT36 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_36(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_36_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_36_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_37_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_37_SHIFT (5U)
/*! IOMUXC_XBAR_DIR_SEL_37 - IOMUXC XBAR_INOUT37 function direction select
 *  0b0..XBAR_INOUT as input
 *  0b1..XBAR_INOUT as output
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_37(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_37_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_DIR_CTRL2_IOMUXC_XBAR_DIR_SEL_37_MASK)
/*! @} */

/*! @name LPIT_TRIG_SEL - LPIT trigger input select register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG0_INPUT_SEL_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG0_INPUT_SEL_SHIFT (0U)
/*! LPIT1_TRIG0_INPUT_SEL - LPIT1 TRIG0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG1_INPUT_SEL_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG1_INPUT_SEL_SHIFT (1U)
/*! LPIT1_TRIG1_INPUT_SEL - LPIT1 TRIG1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG2_INPUT_SEL_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG2_INPUT_SEL_SHIFT (2U)
/*! LPIT1_TRIG2_INPUT_SEL - LPIT1 TRIG2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG3_INPUT_SEL_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG3_INPUT_SEL_SHIFT (3U)
/*! LPIT1_TRIG3_INPUT_SEL - LPIT1 TRIG3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT1_TRIG3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG0_INPUT_SEL_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG0_INPUT_SEL_SHIFT (8U)
/*! LPIT2_TRIG0_INPUT_SEL - LPIT2 TRIG0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG1_INPUT_SEL_MASK (0x200U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG1_INPUT_SEL_SHIFT (9U)
/*! LPIT2_TRIG1_INPUT_SEL - LPIT2 TRIG1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG2_INPUT_SEL_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG2_INPUT_SEL_SHIFT (10U)
/*! LPIT2_TRIG2_INPUT_SEL - LPIT2 TRIG2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG3_INPUT_SEL_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG3_INPUT_SEL_SHIFT (11U)
/*! LPIT2_TRIG3_INPUT_SEL - LPIT2 TRIG3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT2_TRIG3_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG0_INPUT_SEL_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG0_INPUT_SEL_SHIFT (16U)
/*! LPIT3_TRIG0_INPUT_SEL - LPIT3 TRIG0 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG0_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG0_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG0_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG1_INPUT_SEL_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG1_INPUT_SEL_SHIFT (17U)
/*! LPIT3_TRIG1_INPUT_SEL - LPIT3 TRIG1 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG1_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG1_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG1_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG2_INPUT_SEL_MASK (0x40000U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG2_INPUT_SEL_SHIFT (18U)
/*! LPIT3_TRIG2_INPUT_SEL - LPIT3 TRIG2 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG2_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG2_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG2_INPUT_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG3_INPUT_SEL_MASK (0x80000U)
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG3_INPUT_SEL_SHIFT (19U)
/*! LPIT3_TRIG3_INPUT_SEL - LPIT3 TRIG3 input select
 *  0b0..Input from IOMUX
 *  0b1..Input from XBAR
 */
#define BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG3_INPUT_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG3_INPUT_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_LPIT_TRIG_SEL_LPIT3_TRIG3_INPUT_SEL_MASK)
/*! @} */

/*! @name AXI_ATTR_CFG - AXI bus attribute configuration register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC1_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC1_SHIFT (0U)
/*! ARCACHE_USDHC1 - uSDHC1 block cacheable attribute value of AXI read transactions
 *  0b0..Cacheable attribute is off for read transactions
 *  0b1..Cacheable attribute is on for read transactions
 */
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC1_SHIFT)) & BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC1_MASK)

#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC1_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC1_SHIFT (1U)
/*! AWCACHE_USDHC1 - uSDHC1 block cacheable attribute value of AXI write transactions
 *  0b0..Cacheable attribute is off for write transactions
 *  0b1..Cacheable attribute is on for write transactions
 */
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC1_SHIFT)) & BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC1_MASK)

#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC2_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC2_SHIFT (2U)
/*! ARCACHE_USDHC2 - uSDHC2 block cacheable attribute value of AXI read transactions
 *  0b0..Cacheable attribute is off for read transactions
 *  0b1..Cacheable attribute is on for read transactions
 */
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC2_SHIFT)) & BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USDHC2_MASK)

#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC2_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC2_SHIFT (3U)
/*! AWCACHE_USDHC2 - uSDHC2 block cacheable attribute value of AXI write transactions
 *  0b0..Cacheable attribute is off for write transactions
 *  0b1..Cacheable attribute is on for write transactions
 */
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC2_SHIFT)) & BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USDHC2_MASK)

#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USB_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USB_SHIFT (4U)
/*! ARCACHE_USB - USB block cacheable attribute value of AXI read transactions
 *  0b0..Cacheable attribute is off for read transactions
 *  0b1..Cacheable attribute is on for read transactions
 */
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USB(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USB_SHIFT)) & BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_ARCACHE_USB_MASK)

#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USB_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USB_SHIFT (5U)
/*! AWCACHE_USB - USB block cacheable attribute value of AXI write transactions
 *  0b0..Cacheable attribute is off for write transactions
 *  0b1..Cacheable attribute is on for write transactions
 */
#define BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USB(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USB_SHIFT)) & BLK_CTRL_WAKEUPMIX_AXI_ATTR_CFG_AWCACHE_USB_MASK)
/*! @} */

/*! @name SRAMCR0 - SRAM Control Register 0 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_BTO_MASK      (0xFFU)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_BTO_SHIFT     (0U)
/*! BTO - AHB Bus Timeout Wait Cycle */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_BTO(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_BTO_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_BTO_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_BTOEN_MASK    (0x100U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_BTOEN_SHIFT   (8U)
/*! BTOEN - AHB Bus Timeout Enable
 *  0b0..AHB bus timeout counter is not enabled.
 *  0b1..AHB bus timeout counter is enabled.
 */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_BTOEN(x)      (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_BTOEN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_BTOEN_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_PS_MASK       (0x200U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_PS_SHIFT      (9U)
/*! PS - Port Size
 *  0b0..8bit
 *  0b1..16bit
 */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_PS(x)         (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_PS_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_PS_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AM_MASK       (0x400U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AM_SHIFT      (10U)
/*! AM - Address Mode
 *  0b0..Address/Data MUX mode (ADMUX)
 *  0b1..Address/Data non-MUX mode (Non-ADMUX)
 */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AM(x)         (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_AM_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_AM_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_ADVP_MASK     (0x800U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_ADVP_SHIFT    (11U)
/*! ADVP - ADV# polarity
 *  0b0..ADV# is active low.
 *  0b1..ADV# is active high.
 */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_ADVP(x)       (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_ADVP_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_ADVP_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_CES_MASK      (0xF000U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_CES_SHIFT     (12U)
/*! CES - CE setup time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_CES(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_CES_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_CES_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_CEH_MASK      (0xF0000U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_CEH_SHIFT     (16U)
/*! CEH - CE hold time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_CEH(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_CEH_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_CEH_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AS_MASK       (0xF00000U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AS_SHIFT      (20U)
/*! AS - Address setup time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AS(x)         (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_AS_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_AS_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AH_MASK       (0xF000000U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AH_SHIFT      (24U)
/*! AH - Address hold time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_AH(x)         (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_AH_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_AH_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR0_TA_MASK       (0xF0000000U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_TA_SHIFT      (28U)
/*! TA - Turnaround time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR0_TA(x)         (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR0_TA_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR0_TA_MASK)
/*! @} */

/*! @name SRAMCR1 - SRAM Control Register 1 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SRAMCR1_WEL_MASK      (0x3FU)
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_WEL_SHIFT     (0U)
/*! WEL - WE low time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_WEL(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR1_WEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR1_WEL_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR1_WEH_MASK      (0x3C0U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_WEH_SHIFT     (6U)
/*! WEH - WE high time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_WEH(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR1_WEH_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR1_WEH_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR1_REL_MASK      (0xFC00U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_REL_SHIFT     (10U)
/*! REL - RE low time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_REL(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR1_REL_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR1_REL_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR1_REH_MASK      (0xF0000U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_REH_SHIFT     (16U)
/*! REH - RE high time */
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_REH(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR1_REH_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR1_REH_MASK)

#define BLK_CTRL_WAKEUPMIX_SRAMCR1_PRE_MASK      (0x300000U)
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_PRE_SHIFT     (20U)
/*! PRE - Prescaler timer
 *  0b00..Time granularity is 1 clock cycle.
 *  0b01..Time granularity is 2 clock cycles.
 *  0b10..Time granularity is 3 clock cycles.
 *  0b11..Time granularity is 4 clock cycles.
 */
#define BLK_CTRL_WAKEUPMIX_SRAMCR1_PRE(x)        (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SRAMCR1_PRE_SHIFT)) & BLK_CTRL_WAKEUPMIX_SRAMCR1_PRE_MASK)
/*! @} */

/*! @name SLAVE_STOP_MODE_CFG - Slave stop mode configure register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC1_IPG_STOP_MODE_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC1_IPG_STOP_MODE_SHIFT (4U)
/*! ADC1_IPG_STOP_MODE - ADC1 stop mode selection.
 *  0b0..This module is functional in Stop Mode
 *  0b1..This module is not functional in Stop Mode
 */
#define BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC1_IPG_STOP_MODE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC1_IPG_STOP_MODE_SHIFT)) & BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC1_IPG_STOP_MODE_MASK)

#define BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC2_IPG_STOP_MODE_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC2_IPG_STOP_MODE_SHIFT (5U)
/*! ADC2_IPG_STOP_MODE - ADC2 stop mode selection.
 *  0b0..This module is functional in Stop Mode
 *  0b1..This module is not functional in Stop Mode
 */
#define BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC2_IPG_STOP_MODE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC2_IPG_STOP_MODE_SHIFT)) & BLK_CTRL_WAKEUPMIX_SLAVE_STOP_MODE_CFG_ADC2_IPG_STOP_MODE_MASK)
/*! @} */

/*! @name I3C2_ASYNC_WAKEUP_CTRL - I3C2 async wakeup control register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_CLR_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_CLR_SHIFT (0U)
/*! IRQ_CLR - Async wakeup interrupt clear */
#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_CLR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_CLR_SHIFT)) & BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_CLR_MASK)

#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_STATUS_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_STATUS_SHIFT (16U)
/*! IRQ_STATUS - Async wakeup interrupt status
 *  0b0..Interrupt not asserted
 *  0b1..Interrupt asserted
 */
#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_STATUS(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_STATUS_SHIFT)) & BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_STATUS_MASK)

#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_EN_MASK (0x80000000U)
#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_EN_SHIFT (31U)
/*! IRQ_EN - Master mode async wakeup interrupt enable
 *  0b0..Interrupt disabled
 *  0b1..Interrupt enabled
 */
#define BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_I3C2_ASYNC_WAKEUP_CTRL_IRQ_EN_MASK)
/*! @} */

/*! @name XBAR_AOI_WE - XBAR and AOI write protect register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_XBAR_AOI_WE_WE_MASK   (0x1U)
#define BLK_CTRL_WAKEUPMIX_XBAR_AOI_WE_WE_SHIFT  (0U)
/*! WE - Write Enable to XBAR and AOI
 *  0b0..Write is disabled
 *  0b1..Write is enabled
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_AOI_WE_WE(x)     (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_AOI_WE_WE_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_AOI_WE_WE_MASK)
/*! @} */

/*! @name XBAR_TRIG_SYNC_CTRL1 - XBAR trigger synchronizer control register1 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_POL_SEL_MASK (0xFFU)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_POL_SEL_SHIFT (0U)
/*! POL_SEL - Trigger out polarity select
 *  0b00000000..Same as trigger in
 *  0b00000001..Invert trigger in
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_POL_SEL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_POL_SEL_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_POL_SEL_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_ASYNC_EN_MASK (0xFF00U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_ASYNC_EN_SHIFT (8U)
/*! ASYNC_EN - Asynchronous trigger in enable
 *  0b00000000..Trigger in is synchronous
 *  0b00000001..Trigger in is asynchronous
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_ASYNC_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_ASYNC_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_ASYNC_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_SYNC_ENABLE_MASK (0xFF0000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_SYNC_ENABLE_SHIFT (16U)
/*! SYNC_ENABLE - Trigger out synchronizer enable
 *  0b00000000..Channel is disabled
 *  0b00000001..Channel is enabled
 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_SYNC_ENABLE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_SYNC_ENABLE_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL1_SYNC_ENABLE_MASK)
/*! @} */

/*! @name XBAR_TRIG_SYNC_CTRL2 - XBAR trigger synchronizer control register2 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH0_MASK (0xFU)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH0_SHIFT (0U)
/*! PULSE_WIDTH0 - Pulse width control register of channel0 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH0(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH0_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH0_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH1_MASK (0xF0U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH1_SHIFT (4U)
/*! PULSE_WIDTH1 - Pulse width control register of channel1 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH1(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH1_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH1_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH2_MASK (0xF00U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH2_SHIFT (8U)
/*! PULSE_WIDTH2 - Pulse width control register of channel2 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH2(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH2_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH2_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH3_MASK (0xF000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH3_SHIFT (12U)
/*! PULSE_WIDTH3 - Pulse width control register of channel3 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH3(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH3_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH3_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH4_MASK (0xF0000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH4_SHIFT (16U)
/*! PULSE_WIDTH4 - Pulse width control register of channel4 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH4(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH4_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH4_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH5_MASK (0xF00000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH5_SHIFT (20U)
/*! PULSE_WIDTH5 - Pulse width control register of channel5 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH5(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH5_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH5_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH6_MASK (0xF000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH6_SHIFT (24U)
/*! PULSE_WIDTH6 - Pulse width control register of channel6 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH6(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH6_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH6_MASK)

#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH7_MASK (0xF0000000U)
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH7_SHIFT (28U)
/*! PULSE_WIDTH7 - Pulse width control register of channel7 */
#define BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH7(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH7_SHIFT)) & BLK_CTRL_WAKEUPMIX_XBAR_TRIG_SYNC_CTRL2_PULSE_WIDTH7_MASK)
/*! @} */

/*! @name NETC_LINK_CFG - NETC link configuration for port0..NETC link configuration for port4 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_MII_PROT_MASK (0xFU)
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_MII_PROT_SHIFT (0U)
/*! MII_PROT - MII protocol selection
 *  0b0000..MII
 *  0b0001..RMII
 *  0b0010..RGMII
 *  0b0100-0b1111..Reserved
 */
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_MII_PROT(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_MII_PROT_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_MII_PROT_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_IO_VAR_MASK (0xF0000U)
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_IO_VAR_SHIFT (16U)
/*! IO_VAR - IO variant selection
 *  0b0000..None
 *  0b0001-0b1110..Reserved
 */
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_IO_VAR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_IO_VAR_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_IO_VAR_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_RATE_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_RATE_SHIFT (24U)
/*! REVMII_RATE - When REVMII=1 and MII_PROT=MII, this bit configures RevMII rates, otherwise this field has no meaning.
 *  0b0..MII interface is operating at 100Mbps
 *  0b1..MII interface is operating at 10Mbps
 */
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_RATE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_RATE_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_RATE_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_MASK (0x80000000U)
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_SHIFT (31U)
/*! REVMII - RevMII selection
 *  0b0..RevMII not selected
 *  0b1..RevMII selected
 */
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_REVMII_MASK)
/*! @} */

/* The count of BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG */
#define BLK_CTRL_WAKEUPMIX_NETC_LINK_CFG_COUNT   (5U)

/*! @name NETC_REVMII_DLL - NETC RevMII RGMII delay line configuration for port0..NETC RevMII RGMII delay line configuration for port4 */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_DLY_TARGET_MASK (0xFU)
#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_DLY_TARGET_SHIFT (0U)
/*! DLY_TARGET - Delay target of slave delay line */
#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_DLY_TARGET(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_DLY_TARGET_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_DLY_TARGET_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_REF_LOCK_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_REF_LOCK_SHIFT (16U)
/*! REF_LOCK - Reference delay line lock flag
 *  0b0..Reference delay line is not locked
 *  0b1..Reference delay line is locked
 */
#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_REF_LOCK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_REF_LOCK_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_REF_LOCK_MASK)

#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_SLV_LOCK_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_SLV_LOCK_SHIFT (17U)
/*! SLV_LOCK - Slave delay line lock flag
 *  0b0..Slave delay line is not locked
 *  0b1..Slave delay line is locked
 */
#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_SLV_LOCK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_SLV_LOCK_SHIFT)) & BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_SLV_LOCK_MASK)
/*! @} */

/* The count of BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL */
#define BLK_CTRL_WAKEUPMIX_NETC_REVMII_DLL_COUNT (5U)

/*! @name SAFETY_CLK_MON_CS - Safety clock monitor control and status register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_MON_EN_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_MON_EN_SHIFT (0U)
/*! MON_EN - Monitor enable bit
 *  0b0..The monitor is off
 *  0b1..The monitor is on
 */
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_MON_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_MON_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_MON_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_IRQ_EN_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_IRQ_EN_SHIFT (1U)
/*! IRQ_EN - Interrupt enable
 *  0b0..Clock failure will not assert interrupt
 *  0b1..Clock failure will assert interrupt
 */
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_IRQ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_IRQ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_IRQ_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_FAST_RST_EN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_FAST_RST_EN_SHIFT (2U)
/*! FAST_RST_EN - Reset out enable
 *  0b0..Clock failure will not assert EWM_OUT
 *  0b1..Clock failure will assert EWM_OUT_b immediately regardless EWM state
 */
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_FAST_RST_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_FAST_RST_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_FAST_RST_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_CLR_MASK (0x100U)
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_CLR_SHIFT (8U)
/*! STAT_CLR - Status clear
 *  0b0..No effect to clock failure status bit
 *  0b1..Clear clock failure status bit
 */
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_CLR(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_CLR_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_CLR_MASK)

#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_MASK (0x1000000U)
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_SHIFT (24U)
/*! STAT - XBAR_OUT220 clock failure status
 *  0b0..No failure detected by the monitor
 *  0b1..Clock failure has been detected by the monitor
 */
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_CS_STAT_MASK)
/*! @} */

/*! @name SAFETY_CLK_MON_TH - Safety clock monitor threshold register */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_LOW_MASK (0xFFFFU)
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_LOW_SHIFT (0U)
/*! TH_LOW - Threshold low value */
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_LOW(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_LOW_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_LOW_MASK)

#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_HIGH_MASK (0xFFFF0000U)
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_HIGH_SHIFT (16U)
/*! TH_HIGH - Threshold high value */
#define BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_HIGH(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_HIGH_SHIFT)) & BLK_CTRL_WAKEUPMIX_SAFETY_CLK_MON_TH_TH_HIGH_MASK)
/*! @} */

/*! @name EMC_B1_IO_CTRL - GPIO_EMC_B1 bank IO control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FREEZE_SHIFT (0U)
/*! GPIO_EMC1_FREEZE - Compensation code freeze */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPTQ_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPTQ_SHIFT (1U)
/*! GPIO_EMC1_COMPTQ - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPTQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPTQ_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPTQ_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPEN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPEN_SHIFT (2U)
/*! GPIO_EMC1_COMPEN - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPEN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPEN_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPEN_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_EN_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_EN_SHIFT (3U)
/*! GPIO_EMC1_FASTFRZ_EN - Compensation code fast freeze enable */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCP_MASK (0xF0U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCP_SHIFT (4U)
/*! GPIO_EMC1_RASRCP - GPIO_EMC_B1 IO bank's 4-bit PMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCP_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCP_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCN_MASK (0xF00U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCN_SHIFT (8U)
/*! GPIO_EMC1_RASRCN - GPIO_EMC_B1 IO bank's 4-bit NMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCN_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_RASRCN_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SELECT_NASRC_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SELECT_NASRC_SHIFT (12U)
/*! GPIO_EMC1_SELECT_NASRC - GPIO_EMC1_NASRC selection */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SELECT_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SELECT_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SELECT_NASRC_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_REFGEN_SLEEP_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_REFGEN_SLEEP_SHIFT (13U)
/*! GPIO_EMC1_REFGEN_SLEEP - GPIO_EMC_B1 IO bank reference voltage generator cell sleep enable */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_REFGEN_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_REFGEN_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_REFGEN_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SUPLYDET_LATCH_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SUPLYDET_LATCH_SHIFT (14U)
/*! GPIO_EMC1_SUPLYDET_LATCH - GPIO_EMC_B1 IO bank power supply mode latch enable */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SUPLYDET_LATCH(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SUPLYDET_LATCH_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_SUPLYDET_LATCH_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_SHIFT (15U)
/*! GPIO_EMC1_FASTFRZ - Compensation code fast-freeze */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_FASTFRZ_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPOK_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPOK_SHIFT (20U)
/*! GPIO_EMC1_COMPOK - GPIO_EMC_B1 IO bank compensation OK flag */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPOK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPOK_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_COMPOK_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_NASRC_MASK (0x1E00000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_NASRC_SHIFT (21U)
/*! GPIO_EMC1_NASRC - GPIO_EMC_B1 IO bank compensation codes */
#define BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B1_IO_CTRL_GPIO_EMC1_NASRC_MASK)
/*! @} */

/*! @name EMC_B2_IO_CTRL - GPIO_EMC_B2 bank IO control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FREEZE_SHIFT (0U)
/*! GPIO_EMC2_FREEZE - Compensation code freeze */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPTQ_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPTQ_SHIFT (1U)
/*! GPIO_EMC2_COMPTQ - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPTQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPTQ_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPTQ_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPEN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPEN_SHIFT (2U)
/*! GPIO_EMC2_COMPEN - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPEN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPEN_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPEN_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_EN_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_EN_SHIFT (3U)
/*! GPIO_EMC2_FASTFRZ_EN - Compensation code fast freeze enable */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCP_MASK (0xF0U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCP_SHIFT (4U)
/*! GPIO_EMC2_RASRCP - GPIO_EMC_B2 IO bank's 4-bit PMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCP_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCP_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCN_MASK (0xF00U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCN_SHIFT (8U)
/*! GPIO_EMC2_RASRCN - GPIO_EMC_B2 IO bank's 4-bit NMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCN_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_RASRCN_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SELECT_NASRC_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SELECT_NASRC_SHIFT (12U)
/*! GPIO_EMC2_SELECT_NASRC - GPIO_EMC2_NASRC selection
 *  0b0..Show the 4-bit PMOS compensation codes in GPIO_EMC2_NASRC field
 *  0b1..Show the 4-bit NMOS compensation codes in GPIO_EMC2_NASRC field
 */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SELECT_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SELECT_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SELECT_NASRC_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_REFGEN_SLEEP_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_REFGEN_SLEEP_SHIFT (13U)
/*! GPIO_EMC2_REFGEN_SLEEP - GPIO_EMC_B2 IO bank reference voltage generator cell sleep enable */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_REFGEN_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_REFGEN_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_REFGEN_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SUPLYDET_LATCH_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SUPLYDET_LATCH_SHIFT (14U)
/*! GPIO_EMC2_SUPLYDET_LATCH - GPIO_EMC_B2 IO bank power supply mode latch enable */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SUPLYDET_LATCH(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SUPLYDET_LATCH_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_SUPLYDET_LATCH_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_SHIFT (15U)
/*! GPIO_EMC2_FASTFRZ - Compensation code fast-freeze */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_FASTFRZ_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPOK_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPOK_SHIFT (20U)
/*! GPIO_EMC2_COMPOK - GPIO_EMC_B2 IO bank compensation OK flag */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPOK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPOK_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_COMPOK_MASK)

#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_NASRC_MASK (0x1E00000U)
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_NASRC_SHIFT (21U)
/*! GPIO_EMC2_NASRC - GPIO_EMC_B2 IO bank compensation codes */
#define BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_EMC_B2_IO_CTRL_GPIO_EMC2_NASRC_MASK)
/*! @} */

/*! @name SD_B1_IO_CTRL - GPIO_SD_B1 bank IO control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FREEZE_SHIFT (0U)
/*! GPIO_SD1_FREEZE - Compensation code freeze */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPTQ_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPTQ_SHIFT (1U)
/*! GPIO_SD1_COMPTQ - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPTQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPTQ_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPTQ_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPEN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPEN_SHIFT (2U)
/*! GPIO_SD1_COMPEN - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPEN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPEN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPEN_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_EN_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_EN_SHIFT (3U)
/*! GPIO_SD1_FASTFRZ_EN - Compensation code fast freeze enable */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCP_MASK (0xF0U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCP_SHIFT (4U)
/*! GPIO_SD1_RASRCP - GPIO_SD_B1 IO bank's 4-bit PMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCP_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCP_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCN_MASK (0xF00U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCN_SHIFT (8U)
/*! GPIO_SD1_RASRCN - GPIO_SD_B1 IO bank's 4-bit NMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_RASRCN_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SELECT_NASRC_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SELECT_NASRC_SHIFT (12U)
/*! GPIO_SD1_SELECT_NASRC - GPIO_SD1_NASRC selection */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SELECT_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SELECT_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SELECT_NASRC_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_REFGEN_SLEEP_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_REFGEN_SLEEP_SHIFT (13U)
/*! GPIO_SD1_REFGEN_SLEEP - GPIO_SD_B1 IO bank reference voltage generator cell sleep enable */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_REFGEN_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_REFGEN_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_REFGEN_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SUPLYDET_LATCH_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SUPLYDET_LATCH_SHIFT (14U)
/*! GPIO_SD1_SUPLYDET_LATCH - GPIO_SD_B1 IO bank power supply mode latch enable */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SUPLYDET_LATCH(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SUPLYDET_LATCH_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_SUPLYDET_LATCH_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_SHIFT (15U)
/*! GPIO_SD1_FASTFRZ - Compensation code fast-freeze */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_FASTFRZ_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPOK_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPOK_SHIFT (20U)
/*! GPIO_SD1_COMPOK - GPIO_SD_B1 IO bank compensation OK flag */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPOK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPOK_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_COMPOK_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_NASRC_MASK (0x1E00000U)
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_NASRC_SHIFT (21U)
/*! GPIO_SD1_NASRC - GPIO_SD_B1 IO bank compensation codes */
#define BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B1_IO_CTRL_GPIO_SD1_NASRC_MASK)
/*! @} */

/*! @name SD_B2_IO_CTRL - GPIO_SD_B2 bank IO control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FREEZE_SHIFT (0U)
/*! GPIO_SD2_FREEZE - Compensation code freeze */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPTQ_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPTQ_SHIFT (1U)
/*! GPIO_SD2_COMPTQ - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPTQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPTQ_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPTQ_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPEN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPEN_SHIFT (2U)
/*! GPIO_SD2_COMPEN - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPEN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPEN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPEN_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_EN_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_EN_SHIFT (3U)
/*! GPIO_SD2_FASTFRZ_EN - Compensation code fast freeze enable */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCP_MASK (0xF0U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCP_SHIFT (4U)
/*! GPIO_SD2_RASRCP - GPIO_SD_B2 IO bank's 4-bit PMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCP_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCP_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCN_MASK (0xF00U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCN_SHIFT (8U)
/*! GPIO_SD2_RASRCN - GPIO_SD_B2 IO bank's 4-bit NMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCN_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_RASRCN_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SELECT_NASRC_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SELECT_NASRC_SHIFT (12U)
/*! GPIO_SD2_SELECT_NASRC - GPIO_SD2_NASRC selection */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SELECT_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SELECT_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SELECT_NASRC_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_REFGEN_SLEEP_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_REFGEN_SLEEP_SHIFT (13U)
/*! GPIO_SD2_REFGEN_SLEEP - GPIO_SD_B2 IO bank reference voltage generator cell sleep enable */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_REFGEN_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_REFGEN_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_REFGEN_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SUPLYDET_LATCH_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SUPLYDET_LATCH_SHIFT (14U)
/*! GPIO_SD2_SUPLYDET_LATCH - GPIO_SD_B2 IO bank power supply mode latch enable */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SUPLYDET_LATCH(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SUPLYDET_LATCH_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_SUPLYDET_LATCH_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_SHIFT (15U)
/*! GPIO_SD2_FASTFRZ - Compensation code fast-freeze */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_FASTFRZ_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPOK_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPOK_SHIFT (20U)
/*! GPIO_SD2_COMPOK - GPIO_SD_B2 IO bank compensation OK flag */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPOK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPOK_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_COMPOK_MASK)

#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_NASRC_MASK (0x1E00000U)
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_NASRC_SHIFT (21U)
/*! GPIO_SD2_NASRC - GPIO_SD_B2 IO bank compensation codes */
#define BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_SD_B2_IO_CTRL_GPIO_SD2_NASRC_MASK)
/*! @} */

/*! @name GPIO_B1_IO_CTRL - GPIO_B1 bank IO control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FREEZE_SHIFT (0U)
/*! GPIO_B1_FREEZE - Compensation code freeze */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPTQ_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPTQ_SHIFT (1U)
/*! GPIO_B1_COMPTQ - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPTQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPTQ_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPTQ_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPEN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPEN_SHIFT (2U)
/*! GPIO_B1_COMPEN - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPEN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPEN_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPEN_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_EN_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_EN_SHIFT (3U)
/*! GPIO_B1_FASTFRZ_EN - Compensation code fast freeze enable */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCP_MASK (0xF0U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCP_SHIFT (4U)
/*! GPIO_B1_RASRCP - GPIO_B1 IO bank's 4-bit PMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCP_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCP_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCN_MASK (0xF00U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCN_SHIFT (8U)
/*! GPIO_B1_RASRCN - GPIO_B1 IO bank's 4-bit NMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCN_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_RASRCN_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SELECT_NASRC_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SELECT_NASRC_SHIFT (12U)
/*! GPIO_B1_SELECT_NASRC - GPIO_B1_NASRC selection
 *  0b0..Show the 4-bit PMOS compensation codes in GPIO_B1_NASRC field
 *  0b1..Show the 4-bit NMOS compensation codes in GPIO_B1_NASRC field
 */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SELECT_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SELECT_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SELECT_NASRC_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_REFGEN_SLEEP_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_REFGEN_SLEEP_SHIFT (13U)
/*! GPIO_B1_REFGEN_SLEEP - GPIO_B1 IO bank reference voltage generator cell sleep enable */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_REFGEN_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_REFGEN_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_REFGEN_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SUPLYDET_LATCH_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SUPLYDET_LATCH_SHIFT (14U)
/*! GPIO_B1_SUPLYDET_LATCH - GPIO_B1 IO bank power supply mode latch enable */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SUPLYDET_LATCH(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SUPLYDET_LATCH_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_SUPLYDET_LATCH_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_SHIFT (15U)
/*! GPIO_B1_FASTFRZ - Compensation code fast-freeze */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_FASTFRZ_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPOK_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPOK_SHIFT (20U)
/*! GPIO_B1_COMPOK - GPIO_B1 IO bank compensation OK flag */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPOK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPOK_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_COMPOK_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_NASRC_MASK (0x1E00000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_NASRC_SHIFT (21U)
/*! GPIO_B1_NASRC - GPIO_B1 IO bank compensation codes */
#define BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B1_IO_CTRL_GPIO_B1_NASRC_MASK)
/*! @} */

/*! @name GPIO_B2_IO_CTRL - GPIO_B2 bank IO control */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FREEZE_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FREEZE_SHIFT (0U)
/*! GPIO_B2_FREEZE - Compensation code freeze */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FREEZE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FREEZE_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FREEZE_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPTQ_MASK (0x2U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPTQ_SHIFT (1U)
/*! GPIO_B2_COMPTQ - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPTQ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPTQ_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPTQ_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPEN_MASK (0x4U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPEN_SHIFT (2U)
/*! GPIO_B2_COMPEN - COMPEN and COMPTQ control the operating modes of the compensation cell */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPEN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPEN_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPEN_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_EN_MASK (0x8U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_EN_SHIFT (3U)
/*! GPIO_B2_FASTFRZ_EN - Compensation code fast freeze enable */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_EN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_EN_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_EN_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCP_MASK (0xF0U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCP_SHIFT (4U)
/*! GPIO_B2_RASRCP - GPIO_B2 IO bank's 4-bit PMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCP_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCP_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCN_MASK (0xF00U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCN_SHIFT (8U)
/*! GPIO_B2_RASRCN - GPIO_B2 IO bank's 4-bit NMOS compensation codes from core */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCN(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCN_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_RASRCN_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SELECT_NASRC_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SELECT_NASRC_SHIFT (12U)
/*! GPIO_B2_SELECT_NASRC - GPIO_B2_NASRC selection */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SELECT_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SELECT_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SELECT_NASRC_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_REFGEN_SLEEP_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_REFGEN_SLEEP_SHIFT (13U)
/*! GPIO_B2_REFGEN_SLEEP - GPIO_B2 IO bank reference voltage generator cell sleep enable */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_REFGEN_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_REFGEN_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_REFGEN_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SUPLYDET_LATCH_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SUPLYDET_LATCH_SHIFT (14U)
/*! GPIO_B2_SUPLYDET_LATCH - GPIO_B2 IO bank power supply mode latch enable */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SUPLYDET_LATCH(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SUPLYDET_LATCH_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_SUPLYDET_LATCH_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_SHIFT (15U)
/*! GPIO_B2_FASTFRZ - Compensation code fast-freeze */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_FASTFRZ_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPOK_MASK (0x100000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPOK_SHIFT (20U)
/*! GPIO_B2_COMPOK - GPIO_B2 IO bank compensation OK flag */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPOK(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPOK_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_COMPOK_MASK)

#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_NASRC_MASK (0x1E00000U)
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_NASRC_SHIFT (21U)
/*! GPIO_B2_NASRC - GPIO_B2 IO bank compensation codes */
#define BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_NASRC(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_NASRC_SHIFT)) & BLK_CTRL_WAKEUPMIX_GPIO_B2_IO_CTRL_GPIO_B2_NASRC_MASK)
/*! @} */

/*! @name MISC_IO_CTRL - Miscellaneous control register of IO */
/*! @{ */

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_MASK (0x1U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_SHIFT (0U)
/*! I3C_ON_CHIP_STRONG_PULL_DIS - Disable I3C on-chip strong pull for I3C2
 *  0b0..On-chip strong pull is enabled
 *  0b1..On-chip strong pull is disabled
 */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_I3C_ON_CHIP_STRONG_PULL_DIS_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_HIGH_RANGE_MASK (0x10U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_HIGH_RANGE_SHIFT (4U)
/*! GPIO_AD_HIGH_RANGE - GPIO_AD IO bank supply voltage range selection for GPIO_AD_00 to GPIO_AD_17 */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_HIGH_RANGE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_HIGH_RANGE_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_HIGH_RANGE_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_LOW_RANGE_MASK (0x20U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_LOW_RANGE_SHIFT (5U)
/*! GPIO_AD_LOW_RANGE - GPIO_AD IO bank supply voltage range selection for GPIO_AD_00 to GPIO_AD_17 */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_LOW_RANGE(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_LOW_RANGE_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_GPIO_AD_LOW_RANGE_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC1_SLEEP_MASK (0x400U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC1_SLEEP_SHIFT (10U)
/*! SUPLYDET_EMC1_SLEEP - GPIO_EMC_B1 IO bank supply voltage detector sleep mode enable */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC1_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC1_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC1_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC2_SLEEP_MASK (0x800U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC2_SLEEP_SHIFT (11U)
/*! SUPLYDET_EMC2_SLEEP - GPIO_EMC_B2 IO bank supply voltage detector sleep mode enable */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC2_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC2_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_EMC2_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD1_SLEEP_MASK (0x1000U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD1_SLEEP_SHIFT (12U)
/*! SUPLYDET_SD1_SLEEP - GPIO_SD_B1 IO bank supply voltage detector sleep mode enable */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD1_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD1_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD1_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD2_SLEEP_MASK (0x2000U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD2_SLEEP_SHIFT (13U)
/*! SUPLYDET_SD2_SLEEP - GPIO_SD_B2 IO bank supply voltage detector sleep mode enable */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD2_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD2_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_SD2_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B1_SLEEP_MASK (0x4000U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B1_SLEEP_SHIFT (14U)
/*! SUPLYDET_GPIO_B1_SLEEP - GPIO_GPIO_B1 IO bank supply voltage detector sleep mode enable */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B1_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B1_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B1_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B2_SLEEP_MASK (0x8000U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B2_SLEEP_SHIFT (15U)
/*! SUPLYDET_GPIO_B2_SLEEP - GPIO_GPIO_B1 IO bank supply voltage detector sleep mode enable */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B2_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B2_SLEEP_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_SUPLYDET_GPIO_B2_SLEEP_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT0_POL_MASK (0x10000U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT0_POL_SHIFT (16U)
/*! ECAT_LINK_ACT0_POL - ECAT_LINK_ACT[0] polarity control */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT0_POL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT0_POL_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT0_POL_MASK)

#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT1_POL_MASK (0x20000U)
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT1_POL_SHIFT (17U)
/*! ECAT_LINK_ACT1_POL - ECAT_LINK_ACT[1] polarity control */
#define BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT1_POL(x) (((uint32_t)(((uint32_t)(x)) << BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT1_POL_SHIFT)) & BLK_CTRL_WAKEUPMIX_MISC_IO_CTRL_ECAT_LINK_ACT1_POL_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group BLK_CTRL_WAKEUPMIX_Register_Masks */


/* BLK_CTRL_WAKEUPMIX - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral BLK_CTRL_WAKEUPMIX base address */
  #define BLK_CTRL_WAKEUPMIX_BASE                  (0x52420000u)
  /** Peripheral BLK_CTRL_WAKEUPMIX base address */
  #define BLK_CTRL_WAKEUPMIX_BASE_NS               (0x42420000u)
  /** Peripheral BLK_CTRL_WAKEUPMIX base pointer */
  #define BLK_CTRL_WAKEUPMIX                       ((BLK_CTRL_WAKEUPMIX_Type *)BLK_CTRL_WAKEUPMIX_BASE)
  /** Peripheral BLK_CTRL_WAKEUPMIX base pointer */
  #define BLK_CTRL_WAKEUPMIX_NS                    ((BLK_CTRL_WAKEUPMIX_Type *)BLK_CTRL_WAKEUPMIX_BASE_NS)
  /** Array initializer of BLK_CTRL_WAKEUPMIX peripheral base addresses */
  #define BLK_CTRL_WAKEUPMIX_BASE_ADDRS            { BLK_CTRL_WAKEUPMIX_BASE }
  /** Array initializer of BLK_CTRL_WAKEUPMIX peripheral base pointers */
  #define BLK_CTRL_WAKEUPMIX_BASE_PTRS             { BLK_CTRL_WAKEUPMIX }
  /** Array initializer of BLK_CTRL_WAKEUPMIX peripheral base addresses */
  #define BLK_CTRL_WAKEUPMIX_BASE_ADDRS_NS         { BLK_CTRL_WAKEUPMIX_BASE_NS }
  /** Array initializer of BLK_CTRL_WAKEUPMIX peripheral base pointers */
  #define BLK_CTRL_WAKEUPMIX_BASE_PTRS_NS          { BLK_CTRL_WAKEUPMIX_NS }
#else
  /** Peripheral BLK_CTRL_WAKEUPMIX base address */
  #define BLK_CTRL_WAKEUPMIX_BASE                  (0x42420000u)
  /** Peripheral BLK_CTRL_WAKEUPMIX base pointer */
  #define BLK_CTRL_WAKEUPMIX                       ((BLK_CTRL_WAKEUPMIX_Type *)BLK_CTRL_WAKEUPMIX_BASE)
  /** Array initializer of BLK_CTRL_WAKEUPMIX peripheral base addresses */
  #define BLK_CTRL_WAKEUPMIX_BASE_ADDRS            { BLK_CTRL_WAKEUPMIX_BASE }
  /** Array initializer of BLK_CTRL_WAKEUPMIX peripheral base pointers */
  #define BLK_CTRL_WAKEUPMIX_BASE_PTRS             { BLK_CTRL_WAKEUPMIX }
#endif

/*!
 * @}
 */ /* end of group BLK_CTRL_WAKEUPMIX_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- CACHE_ECC_MCM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CACHE_ECC_MCM_Peripheral_Access_Layer CACHE_ECC_MCM Peripheral Access Layer
 * @{
 */

/** CACHE_ECC_MCM - Register Layout Typedef */
typedef struct {
  __IO uint32_t CACHE_ECCR;                        /**< CACHE ECC Control, offset: 0x0 */
       uint8_t RESERVED_0[28];
  __IO uint32_t INT_STATUS;                        /**< Interrupt Status, offset: 0x20 */
  __IO uint32_t INT_STAT_EN;                       /**< Interrupt Status Enable, offset: 0x24 */
  __IO uint32_t INT_SIG_EN;                        /**< Interrupt Enable, offset: 0x28 */
       uint8_t RESERVED_1[48];
  __I  uint32_t CODE_CACHE_ECC_SINGLE_ERROR_INFO;  /**< Code Cache Single-Bit ECC Error Information, offset: 0x5C */
  __I  uint32_t CODE_CACHE_ECC_SINGLE_ERROR_ADDR;  /**< Code Cache Single-Bit ECC Error Address, offset: 0x60 */
       uint8_t RESERVED_2[4];
  __I  uint32_t CODE_CACHE_ECC_MULTI_ERROR_INFO;   /**< Code Cache Multibit ECC Error Information, offset: 0x68 */
       uint8_t RESERVED_3[8];
  __I  uint32_t SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO; /**< System Cache Single-Bit ECC Error Information, offset: 0x74 */
  __I  uint32_t SYSTEM_CACHE_ECC_SINGLE_ERROR_ADDR; /**< System Cache Single-Bit ECC Error Address, offset: 0x78 */
       uint8_t RESERVED_4[4];
  __I  uint32_t SYSTEM_CACHE_ECC_MULTI_ERROR_INFO; /**< System Cache Multibit ECC Error Information, offset: 0x80 */
  __I  uint32_t SYSTEM_CACHE_ECC_MULTI_ERROR_DATA; /**< System Cache Multibit ECC Error Data, offset: 0x84 */
       uint8_t RESERVED_5[4];
  __IO uint32_t CODE_CACHE_TAG0_ECC_ERROR_INJEC;   /**< Code Cache TAG0 ECC Error Injection, offset: 0x8C */
  __IO uint32_t CODE_CACHE_TAG1_ECC_ERROR_INJEC;   /**< Code Cache TAG1 ECC Error Injection, offset: 0x90 */
  __IO uint32_t CODE_CACHE_DATA0_ECC_ERROR_INJEC;  /**< Code Cache DATA0 ECC Error Injection, offset: 0x94 */
  __IO uint32_t CODE_CACHE_DATA1_ECC_ERROR_INJEC;  /**< Code Cache DATA1 ECC Error Injection, offset: 0x98 */
  __IO uint32_t SYTEM_CACHE_TAG0_ECC_ERROR_INJEC;  /**< System Cache TAG0 ECC Error Injection, offset: 0x9C */
  __IO uint32_t SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC; /**< System Cache TAG1 ECC Error Injection, offset: 0xA0 */
  __IO uint32_t SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC; /**< System Cache DATA0 ECC Error Injection, offset: 0xA4 */
  __IO uint32_t STSTEM_CACHE_DATA1_ECC_ERROR_INJEC; /**< System Cache DATA1 ECC Error Injection, offset: 0xA8 */
} CACHE_ECC_MCM_Type;

/* ----------------------------------------------------------------------------
   -- CACHE_ECC_MCM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CACHE_ECC_MCM_Register_Masks CACHE_ECC_MCM Register Masks
 * @{
 */

/*! @name CACHE_ECCR - CACHE ECC Control */
/*! @{ */

#define CACHE_ECC_MCM_CACHE_ECCR_WECC_DIS_MASK   (0x1U)
#define CACHE_ECC_MCM_CACHE_ECCR_WECC_DIS_SHIFT  (0U)
/*! WECC_DIS - Disable CACHE ECC Write Generation
 *  0b1..Disable
 *  0b0..Enable
 */
#define CACHE_ECC_MCM_CACHE_ECCR_WECC_DIS(x)     (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CACHE_ECCR_WECC_DIS_SHIFT)) & CACHE_ECC_MCM_CACHE_ECCR_WECC_DIS_MASK)

#define CACHE_ECC_MCM_CACHE_ECCR_RECC_DIS_MASK   (0x2U)
#define CACHE_ECC_MCM_CACHE_ECCR_RECC_DIS_SHIFT  (1U)
/*! RECC_DIS - Disable Cache ECC Read Check
 *  0b1..Disable
 *  0b0..Enable
 */
#define CACHE_ECC_MCM_CACHE_ECCR_RECC_DIS(x)     (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CACHE_ECCR_RECC_DIS_SHIFT)) & CACHE_ECC_MCM_CACHE_ECCR_RECC_DIS_MASK)
/*! @} */

/*! @name INT_STATUS - Interrupt Status */
/*! @{ */

#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_INT_MASK (0x100U)
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_INT_SHIFT (8U)
/*! CODE_CACHE_ECC_ERRM_INT - Code Cache Access Multibit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_INT_MASK)

#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_INT_MASK (0x200U)
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_INT_SHIFT (9U)
/*! CODE_CACHE_ECC_ERRS_INT - Code Cache Access Single-Bit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_INT_MASK)

#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_OVER_INT_MASK (0x400U)
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_OVER_INT_SHIFT (10U)
/*! CODE_CACHE_ECC_ERRM_OVER_INT - Code Cache Access Multiple Multibit ECC Error Interrupt Status
 *  0b0..Not more than one error
 *  0b1..Multiple errors
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_OVER_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_OVER_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRM_OVER_INT_MASK)

#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_OVER_INT_MASK (0x800U)
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_OVER_INT_SHIFT (11U)
/*! CODE_CACHE_ECC_ERRS_OVER_INT - Code Cache Access Multiple Single-Bit ECC Error Interrupt Status
 *  0b0..Not more than one error
 *  0b1..Multiple errors
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_OVER_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_OVER_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_CODE_CACHE_ECC_ERRS_OVER_INT_MASK)

#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_INT_MASK (0x1000U)
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_INT_SHIFT (12U)
/*! SYSTEM_CACHE_ECC_ERRM_INT - System Cache Access Multibit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_INT_MASK)

#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_INT_MASK (0x2000U)
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_INT_SHIFT (13U)
/*! SYSTEM_CACHE_ECC_ERRS_INT - System Cache Access Single-Bit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_INT_MASK)

#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_OVER_INT_MASK (0x4000U)
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_OVER_INT_SHIFT (14U)
/*! SYSTEM_CACHE_ECC_ERRM_OVER_INT - System Cache Access Multiple Multibit ECC Error Interrupt Status
 *  0b0..Not more than one error
 *  0b1..Multiple errors
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_OVER_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_OVER_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRM_OVER_INT_MASK)

#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_OVER_INT_MASK (0x8000U)
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_OVER_INT_SHIFT (15U)
/*! SYSTEM_CACHE_ECC_ERRS_OVER_INT - System Cache Access Multiple Single-Bit ECC Error Interrupt Status
 *  0b0..Not more than one error
 *  0b1..Multiple errors
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_OVER_INT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_OVER_INT_SHIFT)) & CACHE_ECC_MCM_INT_STATUS_SYSTEM_CACHE_ECC_ERRS_OVER_INT_MASK)
/*! @} */

/*! @name INT_STAT_EN - Interrupt Status Enable */
/*! @{ */

#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_INT_EN_MASK (0x100U)
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_INT_EN_SHIFT (8U)
/*! CODE_CACHE_ERRM_INT_EN - Code Cache Access Multibit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_INT_EN_MASK)

#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_INT_EN_MASK (0x200U)
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_INT_EN_SHIFT (9U)
/*! CODE_CACHE_ERRS_INT_EN - Code Cache Access Single-Bit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_INT_EN_MASK)

#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_OVER_INT_EN_MASK (0x400U)
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_OVER_INT_EN_SHIFT (10U)
/*! CODE_CACHE_ERRM_OVER_INT_EN - Code Cache Access Multiple Multibit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_OVER_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_OVER_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRM_OVER_INT_EN_MASK)

#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_OVER_INT_EN_MASK (0x800U)
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_OVER_INT_EN_SHIFT (11U)
/*! CODE_CACHE_ERRS_OVER_INT_EN - Code Cache Access Multiple Single-Bit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_OVER_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_OVER_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_CODE_CACHE_ERRS_OVER_INT_EN_MASK)

#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_INT_EN_MASK (0x1000U)
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_INT_EN_SHIFT (12U)
/*! SYSTEM_CACHE_ECC_ERRM_INT_EN - System Cache Access Multibit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_INT_EN_MASK)

#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_INT_EN_MASK (0x2000U)
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_INT_EN_SHIFT (13U)
/*! SYSTEM_CACHE_ECC_ERRS_INT_EN - System Cache Access Single-Bit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_INT_EN_MASK)

#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_OVER_INT_EN_MASK (0x4000U)
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_OVER_INT_EN_SHIFT (14U)
/*! SYSTEM_CACHE_ECC_ERRM_OVER_INT_EN - System Cache Access Multiple Multibit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_OVER_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_OVER_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRM_OVER_INT_EN_MASK)

#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_OVER_INT_EN_MASK (0x8000U)
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_OVER_INT_EN_SHIFT (15U)
/*! SYSTEM_CACHE_ECC_ERRS_OVER_INT_EN - System Cache Access Multiple Single-Bit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_OVER_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_OVER_INT_EN_SHIFT)) & CACHE_ECC_MCM_INT_STAT_EN_SYSTEM_CACHE_ECC_ERRS_OVER_INT_EN_MASK)
/*! @} */

/*! @name INT_SIG_EN - Interrupt Enable */
/*! @{ */

#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_INT_SIG_EN_MASK (0x100U)
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_INT_SIG_EN_SHIFT (8U)
/*! CODE_CACHE_ERRM_INT_SIG_EN - Code Cache Access Multibit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_INT_SIG_EN_MASK)

#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_INT_SIG_EN_MASK (0x200U)
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_INT_SIG_EN_SHIFT (9U)
/*! CODE_CACHE_ERRS_INT_SIG_EN - Code Cache Access Single-Bit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_INT_SIG_EN_MASK)

#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_OVER_INT_SIG_EN_MASK (0x400U)
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_OVER_INT_SIG_EN_SHIFT (10U)
/*! CODE_CACHE_ERRM_OVER_INT_SIG_EN - Code Cache Access Multiple Multibit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_OVER_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_OVER_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRM_OVER_INT_SIG_EN_MASK)

#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_OVER_INT_SIG_EN_MASK (0x800U)
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_OVER_INT_SIG_EN_SHIFT (11U)
/*! CODE_CACHE_ERRS_OVER_INT_SIG_EN - Code Cache Access Multiple Single-Bit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_OVER_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_OVER_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_CODE_CACHE_ERRS_OVER_INT_SIG_EN_MASK)

#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_INT_SIG_EN_MASK (0x1000U)
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_INT_SIG_EN_SHIFT (12U)
/*! SYSTEM_CACHE_ERRM_INT_SIG_EN - System Cache Access Multibit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_INT_SIG_EN_MASK)

#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_INT_SIG_EN_MASK (0x2000U)
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_INT_SIG_EN_SHIFT (13U)
/*! SYSTEM_CACHE_ERRS_INT_SIG_EN - System Cache Access Single-Bit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_INT_SIG_EN_MASK)

#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_OVER_INT_SIG_EN_MASK (0x4000U)
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_OVER_INT_SIG_EN_SHIFT (14U)
/*! SYSTEM_CACHE_ERRM_OVER_INT_SIG_EN - System Cache Access Multiple Multibit ECC Error Interrupt Signal Enable
 *  0b0..Masked
 *  0b1..Enabled
 */
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_OVER_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_OVER_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRM_OVER_INT_SIG_EN_MASK)

#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_OVER_INT_SIG_EN_MASK (0x8000U)
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_OVER_INT_SIG_EN_SHIFT (15U)
/*! SYSTEM_CACHE_ERRS_OVER_INT_SIG_EN - System Cache Access Multiple Single-Bit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_OVER_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_OVER_INT_SIG_EN_SHIFT)) & CACHE_ECC_MCM_INT_SIG_EN_SYSTEM_CACHE_ERRS_OVER_INT_SIG_EN_MASK)
/*! @} */

/*! @name CODE_CACHE_ECC_SINGLE_ERROR_INFO - Code Cache Single-Bit ECC Error Information */
/*! @{ */

#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_TAG_MASK (0x1U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_TAG_SHIFT (0U)
/*! CODE_CACHE_ECCS_TAG - Code Cache Single-Bit ECC Error
 *  0b1..Tag
 *  0b0..Data
 */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_TAG(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_TAG_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_TAG_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_CMD_MASK (0x2U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_CMD_SHIFT (1U)
/*! CODE_CACHE_ECCS_CMD - Code Cache Single-Bit ECC Error on Cache Command
 *  0b0..No error
 *  0b1..Error
 */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_CMD(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_CMD_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_CMD_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFMST_MASK (0xF0U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFMST_SHIFT (4U)
/*! CODE_CACHE_ECCS_EFMST - Code Cache Single-Bit ECC Error Master Number */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFMST(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFMST_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFMST_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFPRT_MASK (0x3F00U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFPRT_SHIFT (8U)
/*! CODE_CACHE_ECCS_EFPRT - Code Cache Single-Bit ECC Error Protection */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFPRT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFPRT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFSYN_MASK (0x7F0000U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFSYN_SHIFT (16U)
/*! CODE_CACHE_ECCS_EFSYN - Code Cache Single-Bit ECC Error Corresponding Syndrome */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFSYN_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_INFO_CODE_CACHE_ECCS_EFSYN_MASK)
/*! @} */

/*! @name CODE_CACHE_ECC_SINGLE_ERROR_ADDR - Code Cache Single-Bit ECC Error Address */
/*! @{ */

#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_ADDR_CODE_CACHE_ECCS_ERRED_ADDR_MASK (0xFFFFFFFFU)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_ADDR_CODE_CACHE_ECCS_ERRED_ADDR_SHIFT (0U)
/*! CODE_CACHE_ECCS_ERRED_ADDR - Code Cache Single-Bit ECC Error Address */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_ADDR_CODE_CACHE_ECCS_ERRED_ADDR(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_ADDR_CODE_CACHE_ECCS_ERRED_ADDR_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_SINGLE_ERROR_ADDR_CODE_CACHE_ECCS_ERRED_ADDR_MASK)
/*! @} */

/*! @name CODE_CACHE_ECC_MULTI_ERROR_INFO - Code Cache Multibit ECC Error Information */
/*! @{ */

#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_TAG_MASK (0x1U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_TAG_SHIFT (0U)
/*! CODE_CACHE_ECCM_TAG - Code Cache Multibit ECC Error
 *  0b1..Tag
 *  0b0..Data
 */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_TAG(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_TAG_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_TAG_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_CMD_MASK (0x2U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_CMD_SHIFT (1U)
/*! CODE_CACHE_ECCM_CMD - Code Cache Multibit ECC Error on Code Cache Command
 *  0b0..No error
 *  0b1..Error
 */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_CMD(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_CMD_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_CMD_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFMST_MASK (0xF0U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFMST_SHIFT (4U)
/*! CODE_CACHE_ECCM_EFMST - Code Cache Multibit ECC Error Master Number */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFMST(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFMST_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFMST_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFPRT_MASK (0x3F00U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFPRT_SHIFT (8U)
/*! CODE_CACHE_ECCM_EFPRT - Code Cache Multibit ECC Error Protection */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFPRT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFPRT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFSYN_MASK (0x7F0000U)
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFSYN_SHIFT (16U)
/*! CODE_CACHE_ECCM_EFSYN - Code Cache Multibit ECC Error Corresponding Syndrome */
#define CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFSYN_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_ECC_MULTI_ERROR_INFO_CODE_CACHE_ECCM_EFSYN_MASK)
/*! @} */

/*! @name SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO - System Cache Single-Bit ECC Error Information */
/*! @{ */

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_TAG_MASK (0x1U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_TAG_SHIFT (0U)
/*! SYSTEM_CACHE_ECCS_TAG - System Cache Single-Bit ECC Error
 *  0b1..Tag
 *  0b0..Data
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_TAG(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_TAG_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_TAG_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_CMD_MASK (0x2U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_CMD_SHIFT (1U)
/*! SYSTEM_CACHE_ECCS_CMD - System Cache Single-Bit ECC Error on Cache Command
 *  0b0..No error
 *  0b1..Error
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_CMD(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_CMD_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_CMD_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFMST_MASK (0xF0U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFMST_SHIFT (4U)
/*! SYSTEM_CACHE_ECCS_EFMST - System Cache Single-Bit ECC Error Master Number */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFMST(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFMST_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFMST_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFPRT_MASK (0x3F00U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFPRT_SHIFT (8U)
/*! SYSTEM_CACHE_ECCS_EFPRT - System Cache Single-Bit ECC Error Protection */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFPRT_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFPRT_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFSYN_MASK (0x7F0000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFSYN_SHIFT (16U)
/*! SYSTEM_CACHE_ECCS_EFSYN - System Cache Single-Bit ECC Error Corresponding Syndrome */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFSYN_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_INFO_SYSTEM_CACHE_ECCS_EFSYN_MASK)
/*! @} */

/*! @name SYSTEM_CACHE_ECC_SINGLE_ERROR_ADDR - System Cache Single-Bit ECC Error Address */
/*! @{ */

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_ADDR_SYSTEM_CACHE_ECCS_ERRED_ADDR_MASK (0xFFFFFFFFU)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_ADDR_SYSTEM_CACHE_ECCS_ERRED_ADDR_SHIFT (0U)
/*! SYSTEM_CACHE_ECCS_ERRED_ADDR - System Cache Single-Bit ECC Error Address */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_ADDR_SYSTEM_CACHE_ECCS_ERRED_ADDR(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_ADDR_SYSTEM_CACHE_ECCS_ERRED_ADDR_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_SINGLE_ERROR_ADDR_SYSTEM_CACHE_ECCS_ERRED_ADDR_MASK)
/*! @} */

/*! @name SYSTEM_CACHE_ECC_MULTI_ERROR_INFO - System Cache Multibit ECC Error Information */
/*! @{ */

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_TAG_MASK (0x1U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_TAG_SHIFT (0U)
/*! SYSTEM_CACHE_ECCM_TAG - System Cache Multibit ECC Error
 *  0b1..Tag
 *  0b0..Data
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_TAG(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_TAG_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_TAG_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_CMD_MASK (0x2U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_CMD_SHIFT (1U)
/*! SYSTEM_CACHE_ECCM_CMD - System Cache Multibit ECC Error on System Cache Command
 *  0b0..No error
 *  0b1..Error
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_CMD(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_CMD_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_CMD_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFMST_MASK (0xF0U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFMST_SHIFT (4U)
/*! SYSTEM_CACHE_ECCM_EFMST - System Cache Multibit ECC Error Master Number */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFMST(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFMST_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFMST_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFPRT_MASK (0x3F00U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFPRT_SHIFT (8U)
/*! SYSTEM_CACHE_ECCM_EFPRT - System Cache Multibit ECC Error Protection */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFPRT_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFPRT_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFSYN_MASK (0x7F0000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFSYN_SHIFT (16U)
/*! SYSTEM_CACHE_ECCM_EFSYN - System Cache Multibit ECC Error Corresponding Syndrome */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFSYN_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_INFO_SYSTEM_CACHE_ECCM_EFSYN_MASK)
/*! @} */

/*! @name SYSTEM_CACHE_ECC_MULTI_ERROR_DATA - System Cache Multibit ECC Error Data */
/*! @{ */

#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_DATA_SYSTEM_CACHE_ECCM_ERRED_DATA_MASK (0xFFFFFFFFU)
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_DATA_SYSTEM_CACHE_ECCM_ERRED_DATA_SHIFT (0U)
/*! SYSTEM_CACHE_ECCM_ERRED_DATA - System Cache Multibit ECC Error Data */
#define CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_DATA_SYSTEM_CACHE_ECCM_ERRED_DATA(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_DATA_SYSTEM_CACHE_ECCM_ERRED_DATA_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_ECC_MULTI_ERROR_DATA_SYSTEM_CACHE_ECCM_ERRED_DATA_MASK)
/*! @} */

/*! @name CODE_CACHE_TAG0_ECC_ERROR_INJEC - Code Cache TAG0 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR1BIT_SHIFT (0U)
/*! CODE_CACHE_TAG0_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR1BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR2BIT_SHIFT (8U)
/*! CODE_CACHE_TAG0_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_ERR2BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR11BI_SHIFT (16U)
/*! CODE_CACHE_TAG0_FR11BI - Force One 1-Bit Data Inversion on Code Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR11BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR11BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR1NCI_SHIFT (17U)
/*! CODE_CACHE_TAG0_FR1NCI - Force One Noncorrectable Data Inversion on Code Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR1NCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FR1NCI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRC1BI_SHIFT (18U)
/*! CODE_CACHE_TAG0_FRC1BI - Force Continuous 1-Bit Data Inversions on Code Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRC1BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRC1BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRCNCI_SHIFT (19U)
/*! CODE_CACHE_TAG0_FRCNCI - Force Continuous Noncorrectable Data Inversions on Code Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRCNCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG0_ECC_ERROR_INJEC_CODE_CACHE_TAG0_FRCNCI_MASK)
/*! @} */

/*! @name CODE_CACHE_TAG1_ECC_ERROR_INJEC - Code Cache TAG1 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR1BIT_SHIFT (0U)
/*! CODE_CACHE_TAG1_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR1BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR2BIT_SHIFT (8U)
/*! CODE_CACHE_TAG1_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_ERR2BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR11BI_SHIFT (16U)
/*! CODE_CACHE_TAG1_FR11BI - Force One 1-Bit Data Inversion on Code Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR11BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR11BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR1NCI_SHIFT (17U)
/*! CODE_CACHE_TAG1_FR1NCI - Force One Noncorrectable Data Inversion on Code Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR1NCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FR1NCI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRC1BI_SHIFT (18U)
/*! CODE_CACHE_TAG1_FRC1BI - Force Continuous 1-Bit Data Inversions on Code Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRC1BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRC1BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRCNCI_SHIFT (19U)
/*! CODE_CACHE_TAG1_FRCNCI - Force Continuous Noncorrectable Data Inversions on Code Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRCNCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_TAG1_ECC_ERROR_INJEC_CODE_CACHE_TAG1_FRCNCI_MASK)
/*! @} */

/*! @name CODE_CACHE_DATA0_ECC_ERROR_INJEC - Code Cache DATA0 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR1BIT_SHIFT (0U)
/*! CODE_CACHE_DATA0_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR1BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR2BIT_SHIFT (8U)
/*! CODE_CACHE_DATA0_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_ERR2BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR11BI_SHIFT (16U)
/*! CODE_CACHE_DATA0_FR11BI - Force One 1-Bit Data Inversion on Code Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR11BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR11BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR1NCI_SHIFT (17U)
/*! CODE_CACHE_DATA0_FR1NCI - Force One Noncorrectable Data Inversion on Code Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR1NCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FR1NCI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRC1BI_SHIFT (18U)
/*! CODE_CACHE_DATA0_FRC1BI - Force Continuous 1-Bit Data Inversions on Code Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRC1BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRC1BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRCNCI_SHIFT (19U)
/*! CODE_CACHE_DATA0_FRCNCI - Force Continuous Noncorrectable Data Inversions on Code Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRCNCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA0_ECC_ERROR_INJEC_CODE_CACHE_DATA0_FRCNCI_MASK)
/*! @} */

/*! @name CODE_CACHE_DATA1_ECC_ERROR_INJEC - Code Cache DATA1 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR1BIT_SHIFT (0U)
/*! CODE_CACHE_DATA1_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR1BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR2BIT_SHIFT (8U)
/*! CODE_CACHE_DATA1_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_ERR2BIT_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR11BI_SHIFT (16U)
/*! CODE_CACHE_DATA1_FR11BI - Force One 1-Bit Data Inversion on Code Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR11BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR11BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR1NCI_SHIFT (17U)
/*! CODE_CACHE_DATA1_FR1NCI - Force One Noncorrectable Data Inversion on Code Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR1NCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FR1NCI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRC1BI_SHIFT (18U)
/*! CODE_CACHE_DATA1_FRC1BI - Force Continuous 1-Bit Data Inversions on Code Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRC1BI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRC1BI_MASK)

#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRCNCI_SHIFT (19U)
/*! CODE_CACHE_DATA1_FRCNCI - Force Continuous Noncorrectable Data Inversions on Code Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRCNCI_SHIFT)) & CACHE_ECC_MCM_CODE_CACHE_DATA1_ECC_ERROR_INJEC_CODE_CACHE_DATA1_FRCNCI_MASK)
/*! @} */

/*! @name SYTEM_CACHE_TAG0_ECC_ERROR_INJEC - System Cache TAG0 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR1BIT_SHIFT (0U)
/*! SYSTEM_CACHE_TAG0_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR1BIT_MASK)

#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR2BIT_SHIFT (8U)
/*! SYSTEM_CACHE_TAG0_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_ERR2BIT_MASK)

#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR11BI_SHIFT (16U)
/*! SYSTEM_CACHE_TAG0_FR11BI - Force One 1-Bit Data Inversion on System Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR11BI_SHIFT)) & CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR11BI_MASK)

#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR1NCI_SHIFT (17U)
/*! SYSTEM_CACHE_TAG0_FR1NCI - Force One Noncorrectable Data Inversion on System Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR1NCI_SHIFT)) & CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FR1NCI_MASK)

#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRC1BI_SHIFT (18U)
/*! SYSTEM_CACHE_TAG0_FRC1BI - Force Continuous 1-Bit Data Inversions on System Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRC1BI_SHIFT)) & CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRC1BI_MASK)

#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRCNCI_SHIFT (19U)
/*! SYSTEM_CACHE_TAG0_FRCNCI - Force Continuous Noncorrectable Data Inversions on System Cache TAG0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRCNCI_SHIFT)) & CACHE_ECC_MCM_SYTEM_CACHE_TAG0_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG0_FRCNCI_MASK)
/*! @} */

/*! @name SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC - System Cache TAG1 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_ERR1BIT_SHIFT (0U)
/*! SYSTEM_CACHE_TAG1_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_ERR1BIT_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEMCACHE_TAG1_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEMCACHE_TAG1_ERR2BIT_SHIFT (8U)
/*! SYSTEMCACHE_TAG1_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEMCACHE_TAG1_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEMCACHE_TAG1_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEMCACHE_TAG1_ERR2BIT_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR11BI_SHIFT (16U)
/*! SYSTEM_CACHE_TAG1_FR11BI - Force One 1-Bit Data Inversion on System Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR11BI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR11BI_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR1NCI_SHIFT (17U)
/*! SYSTEM_CACHE_TAG1_FR1NCI - Force One Noncorrectable Data Inversion on System Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR1NCI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FR1NCI_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRC1BI_SHIFT (18U)
/*! SYSTEM_CACHE_TAG1_FRC1BI - Force Continuous 1-Bit Data Inversions on System Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRC1BI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRC1BI_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRCNCI_SHIFT (19U)
/*! SYSTEM_CACHE_TAG1_FRCNCI - Force Continuous Noncorrectable Data Inversions on System Cache TAG1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRCNCI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_TAG1_ECC_ERROR_INJEC_SYSTEM_CACHE_TAG1_FRCNCI_MASK)
/*! @} */

/*! @name SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC - System Cache DATA0 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR1BIT_SHIFT (0U)
/*! SYSTEM_CACHE_DATA0_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR1BIT_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR2BIT_SHIFT (8U)
/*! SYSTEM_CACHE_DATA0_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_ERR2BIT_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR11BI_SHIFT (16U)
/*! SYSTEM_CACHE_DATA0_FR11BI - Force One 1-Bit Data Inversion on System Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR11BI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR11BI_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR1NCI_SHIFT (17U)
/*! SYSTEM_CACHE_DATA0_FR1NCI - Force One Noncorrectable Data Inversion on System Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR1NCI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FR1NCI_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRC1BI_SHIFT (18U)
/*! SYSTEM_CACHE_DATA0_FRC1BI - Force Continuous 1-Bit Data Inversions on System Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRC1BI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRC1BI_MASK)

#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRCNCI_SHIFT (19U)
/*! SYSTEM_CACHE_DATA0_FRCNCI - Force Continuous Noncorrectable Data Inversions on System Cache DATA0 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRCNCI_SHIFT)) & CACHE_ECC_MCM_SYSTEM_CACHE_DATA0_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA0_FRCNCI_MASK)
/*! @} */

/*! @name STSTEM_CACHE_DATA1_ECC_ERROR_INJEC - System Cache DATA1 ECC Error Injection */
/*! @{ */

#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR1BIT_MASK (0x7FU)
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR1BIT_SHIFT (0U)
/*! SYSTEM_CACHE_DATA1_ERR1BIT - Position of First Bit to Inject ECC Error */
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR1BIT_SHIFT)) & CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR1BIT_MASK)

#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR2BIT_MASK (0x7F00U)
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR2BIT_SHIFT (8U)
/*! SYSTEM_CACHE_DATA1_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR2BIT_SHIFT)) & CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_ERR2BIT_MASK)

#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR11BI_MASK (0x10000U)
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR11BI_SHIFT (16U)
/*! SYSTEM_CACHE_DATA1_FR11BI - Force One 1-Bit Data Inversion on System Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR11BI_SHIFT)) & CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR11BI_MASK)

#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR1NCI_MASK (0x20000U)
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR1NCI_SHIFT (17U)
/*! SYSTEM_CACHE_DATA1_FR1NCI - Force One Noncorrectable Data Inversion on System Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR1NCI_SHIFT)) & CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FR1NCI_MASK)

#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRC1BI_MASK (0x40000U)
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRC1BI_SHIFT (18U)
/*! SYSTEM_CACHE_DATA1_FRC1BI - Force Continuous 1-Bit Data Inversions on System Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRC1BI_SHIFT)) & CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRC1BI_MASK)

#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRCNCI_MASK (0x80000U)
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRCNCI_SHIFT (19U)
/*! SYSTEM_CACHE_DATA1_FRCNCI - Force Continuous Noncorrectable Data Inversions on System Cache DATA1 Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRCNCI_SHIFT)) & CACHE_ECC_MCM_STSTEM_CACHE_DATA1_ECC_ERROR_INJEC_SYSTEM_CACHE_DATA1_FRCNCI_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group CACHE_ECC_MCM_Register_Masks */


/* CACHE_ECC_MCM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM base address */
  #define CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE (0x54401000u)
  /** Peripheral CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM base address */
  #define CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE_NS (0x44401000u)
  /** Peripheral CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM base pointer */
  #define CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM      ((CACHE_ECC_MCM_Type *)CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE)
  /** Peripheral CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM base pointer */
  #define CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_NS   ((CACHE_ECC_MCM_Type *)CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE_NS)
  /** Array initializer of CACHE_ECC_MCM peripheral base addresses */
  #define CACHE_ECC_MCM_BASE_ADDRS                 { CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE }
  /** Array initializer of CACHE_ECC_MCM peripheral base pointers */
  #define CACHE_ECC_MCM_BASE_PTRS                  { CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM }
  /** Array initializer of CACHE_ECC_MCM peripheral base addresses */
  #define CACHE_ECC_MCM_BASE_ADDRS_NS              { CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE_NS }
  /** Array initializer of CACHE_ECC_MCM peripheral base pointers */
  #define CACHE_ECC_MCM_BASE_PTRS_NS               { CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_NS }
#else
  /** Peripheral CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM base address */
  #define CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE (0x44401000u)
  /** Peripheral CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM base pointer */
  #define CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM      ((CACHE_ECC_MCM_Type *)CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE)
  /** Array initializer of CACHE_ECC_MCM peripheral base addresses */
  #define CACHE_ECC_MCM_BASE_ADDRS                 { CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM_BASE }
  /** Array initializer of CACHE_ECC_MCM peripheral base pointers */
  #define CACHE_ECC_MCM_BASE_PTRS                  { CP_CM33_IMX9RTC__CM33_CACHE_ECC_MCM }
#endif

/*!
 * @}
 */ /* end of group CACHE_ECC_MCM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- CAN Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CAN_Peripheral_Access_Layer CAN Peripheral Access Layer
 * @{
 */

/** CAN - Register Layout Typedef */
typedef struct {
  __IO uint32_t MCR;                               /**< Module Configuration, offset: 0x0 */
  __IO uint32_t CTRL1;                             /**< Control 1, offset: 0x4 */
  __IO uint32_t TIMER;                             /**< Free-Running Timer, offset: 0x8 */
       uint8_t RESERVED_0[4];
  __IO uint32_t RXMGMASK;                          /**< RX Message Buffers Global Mask, offset: 0x10 */
  __IO uint32_t RX14MASK;                          /**< Receive 14 Mask, offset: 0x14 */
  __IO uint32_t RX15MASK;                          /**< Receive 15 Mask, offset: 0x18 */
  __IO uint32_t ECR;                               /**< Error Counter, offset: 0x1C */
  __IO uint32_t ESR1;                              /**< Error and Status 1, offset: 0x20 */
  __IO uint32_t IMASK2;                            /**< Interrupt Masks 2, offset: 0x24 */
  __IO uint32_t IMASK1;                            /**< Interrupt Masks 1, offset: 0x28 */
  __IO uint32_t IFLAG2;                            /**< Interrupt Flags 2, offset: 0x2C */
  __IO uint32_t IFLAG1;                            /**< Interrupt Flags 1, offset: 0x30 */
  __IO uint32_t CTRL2;                             /**< Control 2, offset: 0x34 */
  __I  uint32_t ESR2;                              /**< Error and Status 2, offset: 0x38 */
       uint8_t RESERVED_1[8];
  __I  uint32_t CRCR;                              /**< Cyclic Redundancy Check, offset: 0x44 */
  __IO uint32_t RXFGMASK;                          /**< Legacy RX FIFO Global Mask, offset: 0x48 */
  __I  uint32_t RXFIR;                             /**< Legacy RX FIFO Information, offset: 0x4C */
  __IO uint32_t CBT;                               /**< CAN Bit Timing, offset: 0x50 */
       uint8_t RESERVED_2[24];
  __IO uint32_t IMASK3;                            /**< Interrupt Masks 3, offset: 0x6C */
       uint8_t RESERVED_3[4];
  __IO uint32_t IFLAG3;                            /**< Interrupt Flags 3, offset: 0x74 */
       uint8_t RESERVED_4[8];
  union {                                          /* offset: 0x80 */
    struct {                                         /* offset: 0x80, array step: 0x10 */
      __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 95 CS Register, array offset: 0x80, array step: 0x10 */
      __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 95 ID Register, array offset: 0x84, array step: 0x10 */
      __IO uint32_t WORD[2];                           /**< Message Buffer 0 WORD_8B Register..Message Buffer 95 WORD_8B Register, array offset: 0x88, array step: index*0x10, index2*0x4 */
    } MB_8B[96];
    struct {                                         /* offset: 0x80 */
      struct {                                         /* offset: 0x80, array step: 0x18 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 20 CS Register, array offset: 0x80, array step: 0x18 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 20 ID Register, array offset: 0x84, array step: 0x18 */
        __IO uint32_t WORD[4];                           /**< Message Buffer 0 WORD_16B Register..Message Buffer 20 WORD_16B Register, array offset: 0x88, array step: index*0x18, index2*0x4 */
      } MB_16B_L[21];
           uint8_t RESERVED_0[8];
      struct {                                         /* offset: 0x280, array step: 0x18 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 20 CS Register, array offset: 0x280, array step: 0x18 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 20 ID Register, array offset: 0x284, array step: 0x18 */
        __IO uint32_t WORD[4];                           /**< Message Buffer 0 WORD_16B Register..Message Buffer 20 WORD_16B Register, array offset: 0x288, array step: index*0x18, index2*0x4 */
      } MB_16B_M[21];
           uint8_t RESERVED_1[8];
      struct {                                         /* offset: 0x480, array step: 0x18 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 20 CS Register, array offset: 0x480, array step: 0x18 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 20 ID Register, array offset: 0x484, array step: 0x18 */
        __IO uint32_t WORD[4];                           /**< Message Buffer 0 WORD_16B Register..Message Buffer 20 WORD_16B Register, array offset: 0x488, array step: index*0x18, index2*0x4 */
      } MB_16B_H[21];
    } MB_16B;
    struct {                                         /* offset: 0x80 */
      struct {                                         /* offset: 0x80, array step: 0x28 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 11 CS Register, array offset: 0x80, array step: 0x28 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 11 ID Register, array offset: 0x84, array step: 0x28 */
        __IO uint32_t WORD[8];                           /**< Message Buffer 0 WORD_32B Register..Message Buffer 11 WORD_32B Register, array offset: 0x88, array step: index*0x28, index2*0x4 */
      } MB_32B_L[12];
           uint8_t RESERVED_0[32];
      struct {                                         /* offset: 0x280, array step: 0x28 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 11 CS Register, array offset: 0x280, array step: 0x28 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 11 ID Register, array offset: 0x284, array step: 0x28 */
        __IO uint32_t WORD[8];                           /**< Message Buffer 0 WORD_32B Register..Message Buffer 11 WORD_32B Register, array offset: 0x288, array step: index*0x28, index2*0x4 */
      } MB_32B_M[12];
           uint8_t RESERVED_1[32];
      struct {                                         /* offset: 0x480, array step: 0x28 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 11 CS Register, array offset: 0x480, array step: 0x28 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 11 ID Register, array offset: 0x484, array step: 0x28 */
        __IO uint32_t WORD[8];                           /**< Message Buffer 0 WORD_32B Register..Message Buffer 11 WORD_32B Register, array offset: 0x488, array step: index*0x28, index2*0x4 */
      } MB_32B_H[12];
    } MB_32B;
    struct {                                         /* offset: 0x80 */
      struct {                                         /* offset: 0x80, array step: 0x48 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 6 CS Register, array offset: 0x80, array step: 0x48 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 6 ID Register, array offset: 0x84, array step: 0x48 */
        __IO uint32_t WORD[16];                          /**< Message Buffer 0 WORD_64B Register..Message Buffer 6 WORD_64B Register, array offset: 0x88, array step: index*0x48, index2*0x4 */
      } MB_64B_L[7];
           uint8_t RESERVED_0[8];
      struct {                                         /* offset: 0x280, array step: 0x48 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 6 CS Register, array offset: 0x280, array step: 0x48 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 6 ID Register, array offset: 0x284, array step: 0x48 */
        __IO uint32_t WORD[16];                          /**< Message Buffer 0 WORD_64B Register..Message Buffer 6 WORD_64B Register, array offset: 0x288, array step: index*0x48, index2*0x4 */
      } MB_64B_M[7];
           uint8_t RESERVED_1[8];
      struct {                                         /* offset: 0x480, array step: 0x48 */
        __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 6 CS Register, array offset: 0x480, array step: 0x48 */
        __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 6 ID Register, array offset: 0x484, array step: 0x48 */
        __IO uint32_t WORD[16];                          /**< Message Buffer 0 WORD_64B Register..Message Buffer 6 WORD_64B Register, array offset: 0x488, array step: index*0x48, index2*0x4 */
      } MB_64B_H[7];
    } MB_64B;
    struct {                                         /* offset: 0x80, array step: 0x10 */
      __IO uint32_t CS;                                /**< Message Buffer 0 CS Register..Message Buffer 95 CS Register, array offset: 0x80, array step: 0x10 */
      __IO uint32_t ID;                                /**< Message Buffer 0 ID Register..Message Buffer 95 ID Register, array offset: 0x84, array step: 0x10 */
      __IO uint32_t WORD0;                             /**< Message Buffer 0 WORD0 Register..Message Buffer 95 WORD0 Register, array offset: 0x88, array step: 0x10 */
      __IO uint32_t WORD1;                             /**< Message Buffer 0 WORD1 Register..Message Buffer 95 WORD1 Register, array offset: 0x8C, array step: 0x10 */
    } MB[96];
  };
       uint8_t RESERVED_5[512];
  __IO uint32_t RXIMR[96];                         /**< Receive Individual Mask, array offset: 0x880, array step: 0x4 */
       uint8_t RESERVED_6[224];
  __IO uint32_t MECR;                              /**< Memory Error Control, offset: 0xAE0 */
  __IO uint32_t ERRIAR;                            /**< Error Injection Address, offset: 0xAE4 */
  __IO uint32_t ERRIDPR;                           /**< Error Injection Data Pattern, offset: 0xAE8 */
  __IO uint32_t ERRIPPR;                           /**< Error Injection Parity Pattern, offset: 0xAEC */
  __I  uint32_t RERRAR;                            /**< Error Report Address, offset: 0xAF0 */
  __I  uint32_t RERRDR;                            /**< Error Report Data, offset: 0xAF4 */
  __I  uint32_t RERRSYNR;                          /**< Error Report Syndrome, offset: 0xAF8 */
  __IO uint32_t ERRSR;                             /**< Error Status, offset: 0xAFC */
       uint8_t RESERVED_7[240];
  __IO uint32_t EPRS;                              /**< Enhanced CAN Bit Timing Prescalers, offset: 0xBF0 */
  __IO uint32_t ENCBT;                             /**< Enhanced Nominal CAN Bit Timing, offset: 0xBF4 */
  __IO uint32_t EDCBT;                             /**< Enhanced Data Phase CAN Bit Timing, offset: 0xBF8 */
  __IO uint32_t ETDC;                              /**< Enhanced Transceiver Delay Compensation, offset: 0xBFC */
  __IO uint32_t FDCTRL;                            /**< CAN FD Control, offset: 0xC00 */
  __IO uint32_t FDCBT;                             /**< CAN FD Bit Timing, offset: 0xC04 */
  __I  uint32_t FDCRC;                             /**< CAN FD CRC, offset: 0xC08 */
  __IO uint32_t ERFCR;                             /**< Enhanced RX FIFO Control, offset: 0xC0C */
  __IO uint32_t ERFIER;                            /**< Enhanced RX FIFO Interrupt Enable, offset: 0xC10 */
  __IO uint32_t ERFSR;                             /**< Enhanced RX FIFO Status, offset: 0xC14 */
       uint8_t RESERVED_8[24];
  __IO uint32_t HR_TIME_STAMP[96];                 /**< High-Resolution Timestamp, array offset: 0xC30, array step: 0x4 */
       uint8_t RESERVED_9[8784];
  __IO uint32_t ERFFEL[128];                       /**< Enhanced RX FIFO Filter Element, array offset: 0x3000, array step: 0x4 */
} CAN_Type;

/* ----------------------------------------------------------------------------
   -- CAN Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CAN_Register_Masks CAN Register Masks
 * @{
 */

/*! @name MCR - Module Configuration */
/*! @{ */

#define CAN_MCR_MAXMB_MASK                       (0x7FU)
#define CAN_MCR_MAXMB_SHIFT                      (0U)
/*! MAXMB - Number of the Last Message Buffer */
#define CAN_MCR_MAXMB(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_MCR_MAXMB_SHIFT)) & CAN_MCR_MAXMB_MASK)

#define CAN_MCR_IDAM_MASK                        (0x300U)
#define CAN_MCR_IDAM_SHIFT                       (8U)
/*! IDAM - ID Acceptance Mode
 *  0b00..Format A: One full ID (standard and extended) per ID filter table element.
 *  0b01..Format B: Two full standard IDs or two partial 14-bit (standard and extended) IDs per ID filter table element.
 *  0b10..Format C: Four partial 8-bit standard IDs per ID filter table element.
 *  0b11..Format D: All frames rejected.
 */
#define CAN_MCR_IDAM(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_IDAM_SHIFT)) & CAN_MCR_IDAM_MASK)

#define CAN_MCR_FDEN_MASK                        (0x800U)
#define CAN_MCR_FDEN_SHIFT                       (11U)
/*! FDEN - CAN FD Operation Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define CAN_MCR_FDEN(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_FDEN_SHIFT)) & CAN_MCR_FDEN_MASK)

#define CAN_MCR_AEN_MASK                         (0x1000U)
#define CAN_MCR_AEN_SHIFT                        (12U)
/*! AEN - Abort Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define CAN_MCR_AEN(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_MCR_AEN_SHIFT)) & CAN_MCR_AEN_MASK)

#define CAN_MCR_LPRIOEN_MASK                     (0x2000U)
#define CAN_MCR_LPRIOEN_SHIFT                    (13U)
/*! LPRIOEN - Local Priority Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_LPRIOEN(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_MCR_LPRIOEN_SHIFT)) & CAN_MCR_LPRIOEN_MASK)

#define CAN_MCR_DMA_MASK                         (0x8000U)
#define CAN_MCR_DMA_SHIFT                        (15U)
/*! DMA - DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_DMA(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_MCR_DMA_SHIFT)) & CAN_MCR_DMA_MASK)

#define CAN_MCR_IRMQ_MASK                        (0x10000U)
#define CAN_MCR_IRMQ_SHIFT                       (16U)
/*! IRMQ - Individual RX Masking and Queue Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_IRMQ(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_IRMQ_SHIFT)) & CAN_MCR_IRMQ_MASK)

#define CAN_MCR_SRXDIS_MASK                      (0x20000U)
#define CAN_MCR_SRXDIS_SHIFT                     (17U)
/*! SRXDIS - Self-Reception Disable
 *  0b0..Enable
 *  0b1..Disable
 */
#define CAN_MCR_SRXDIS(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_MCR_SRXDIS_SHIFT)) & CAN_MCR_SRXDIS_MASK)

#define CAN_MCR_DOZE_MASK                        (0x40000U)
#define CAN_MCR_DOZE_SHIFT                       (18U)
/*! DOZE - Doze Mode Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_DOZE(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_DOZE_SHIFT)) & CAN_MCR_DOZE_MASK)

#define CAN_MCR_WAKSRC_MASK                      (0x80000U)
#define CAN_MCR_WAKSRC_SHIFT                     (19U)
/*! WAKSRC - Wake-Up Source
 *  0b0..No filter applied
 *  0b1..Filter applied
 */
#define CAN_MCR_WAKSRC(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_MCR_WAKSRC_SHIFT)) & CAN_MCR_WAKSRC_MASK)

#define CAN_MCR_LPMACK_MASK                      (0x100000U)
#define CAN_MCR_LPMACK_SHIFT                     (20U)
/*! LPMACK - Low-Power Mode Acknowledge
 *  0b0..Not in a low-power mode
 *  0b1..In a low-power mode
 */
#define CAN_MCR_LPMACK(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_MCR_LPMACK_SHIFT)) & CAN_MCR_LPMACK_MASK)

#define CAN_MCR_WRNEN_MASK                       (0x200000U)
#define CAN_MCR_WRNEN_SHIFT                      (21U)
/*! WRNEN - Warning Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_WRNEN(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_MCR_WRNEN_SHIFT)) & CAN_MCR_WRNEN_MASK)

#define CAN_MCR_SLFWAK_MASK                      (0x400000U)
#define CAN_MCR_SLFWAK_SHIFT                     (22U)
/*! SLFWAK - Self Wake-up
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_SLFWAK(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_MCR_SLFWAK_SHIFT)) & CAN_MCR_SLFWAK_MASK)

#define CAN_MCR_SUPV_MASK                        (0x800000U)
#define CAN_MCR_SUPV_SHIFT                       (23U)
/*! SUPV - Supervisor Mode
 *  0b0..User mode
 *  0b1..Supervisor mode
 */
#define CAN_MCR_SUPV(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_SUPV_SHIFT)) & CAN_MCR_SUPV_MASK)

#define CAN_MCR_FRZACK_MASK                      (0x1000000U)
#define CAN_MCR_FRZACK_SHIFT                     (24U)
/*! FRZACK - Freeze Mode Acknowledge
 *  0b0..Not in Freeze mode, prescaler running.
 *  0b1..In Freeze mode, prescaler stopped.
 */
#define CAN_MCR_FRZACK(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_MCR_FRZACK_SHIFT)) & CAN_MCR_FRZACK_MASK)

#define CAN_MCR_SOFTRST_MASK                     (0x2000000U)
#define CAN_MCR_SOFTRST_SHIFT                    (25U)
/*! SOFTRST - Soft Reset
 *  0b0..No reset
 *  0b1..Soft reset affects reset registers
 */
#define CAN_MCR_SOFTRST(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_MCR_SOFTRST_SHIFT)) & CAN_MCR_SOFTRST_MASK)

#define CAN_MCR_WAKMSK_MASK                      (0x4000000U)
#define CAN_MCR_WAKMSK_SHIFT                     (26U)
/*! WAKMSK - Wake-up Interrupt Mask
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define CAN_MCR_WAKMSK(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_MCR_WAKMSK_SHIFT)) & CAN_MCR_WAKMSK_MASK)

#define CAN_MCR_NOTRDY_MASK                      (0x8000000U)
#define CAN_MCR_NOTRDY_SHIFT                     (27U)
/*! NOTRDY - FlexCAN Not Ready
 *  0b0..FlexCAN is in Normal mode, Listen-Only mode, or Loopback mode.
 *  0b1..FlexCAN is in Disable mode, Doze mode, Stop mode, or Freeze mode.
 */
#define CAN_MCR_NOTRDY(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_MCR_NOTRDY_SHIFT)) & CAN_MCR_NOTRDY_MASK)

#define CAN_MCR_HALT_MASK                        (0x10000000U)
#define CAN_MCR_HALT_SHIFT                       (28U)
/*! HALT - Halt FlexCAN
 *  0b0..No request
 *  0b1..Enter Freeze mode, if MCR[FRZ] = 1.
 */
#define CAN_MCR_HALT(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_HALT_SHIFT)) & CAN_MCR_HALT_MASK)

#define CAN_MCR_RFEN_MASK                        (0x20000000U)
#define CAN_MCR_RFEN_SHIFT                       (29U)
/*! RFEN - Legacy RX FIFO Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_RFEN(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_RFEN_SHIFT)) & CAN_MCR_RFEN_MASK)

#define CAN_MCR_FRZ_MASK                         (0x40000000U)
#define CAN_MCR_FRZ_SHIFT                        (30U)
/*! FRZ - Freeze Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MCR_FRZ(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_MCR_FRZ_SHIFT)) & CAN_MCR_FRZ_MASK)

#define CAN_MCR_MDIS_MASK                        (0x80000000U)
#define CAN_MCR_MDIS_SHIFT                       (31U)
/*! MDIS - Module Disable
 *  0b0..Enable
 *  0b1..Disable
 */
#define CAN_MCR_MDIS(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_MCR_MDIS_SHIFT)) & CAN_MCR_MDIS_MASK)
/*! @} */

/*! @name CTRL1 - Control 1 */
/*! @{ */

#define CAN_CTRL1_PROPSEG_MASK                   (0x7U)
#define CAN_CTRL1_PROPSEG_SHIFT                  (0U)
/*! PROPSEG - Propagation Segment */
#define CAN_CTRL1_PROPSEG(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_PROPSEG_SHIFT)) & CAN_CTRL1_PROPSEG_MASK)

#define CAN_CTRL1_LOM_MASK                       (0x8U)
#define CAN_CTRL1_LOM_SHIFT                      (3U)
/*! LOM - Listen-Only Mode
 *  0b0..Listen-Only mode is deactivated.
 *  0b1..FlexCAN module operates in Listen-Only mode.
 */
#define CAN_CTRL1_LOM(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_LOM_SHIFT)) & CAN_CTRL1_LOM_MASK)

#define CAN_CTRL1_LBUF_MASK                      (0x10U)
#define CAN_CTRL1_LBUF_SHIFT                     (4U)
/*! LBUF - Lowest Buffer Transmitted First
 *  0b0..Buffer with highest priority is transmitted first.
 *  0b1..Lowest number buffer is transmitted first.
 */
#define CAN_CTRL1_LBUF(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_LBUF_SHIFT)) & CAN_CTRL1_LBUF_MASK)

#define CAN_CTRL1_TSYN_MASK                      (0x20U)
#define CAN_CTRL1_TSYN_SHIFT                     (5U)
/*! TSYN - Timer Sync
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL1_TSYN(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_TSYN_SHIFT)) & CAN_CTRL1_TSYN_MASK)

#define CAN_CTRL1_BOFFREC_MASK                   (0x40U)
#define CAN_CTRL1_BOFFREC_SHIFT                  (6U)
/*! BOFFREC - Bus Off Recovery
 *  0b0..Enabled
 *  0b1..Disabled
 */
#define CAN_CTRL1_BOFFREC(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_BOFFREC_SHIFT)) & CAN_CTRL1_BOFFREC_MASK)

#define CAN_CTRL1_SMP_MASK                       (0x80U)
#define CAN_CTRL1_SMP_SHIFT                      (7U)
/*! SMP - CAN Bit Sampling
 *  0b0..One sample is used to determine the bit value.
 *  0b1..Three samples are used to determine the value of the received bit: the regular one (sample point) and two
 *       preceding samples. A majority rule is used.
 */
#define CAN_CTRL1_SMP(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_SMP_SHIFT)) & CAN_CTRL1_SMP_MASK)

#define CAN_CTRL1_RWRNMSK_MASK                   (0x400U)
#define CAN_CTRL1_RWRNMSK_SHIFT                  (10U)
/*! RWRNMSK - RX Warning Interrupt Mask
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define CAN_CTRL1_RWRNMSK(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_RWRNMSK_SHIFT)) & CAN_CTRL1_RWRNMSK_MASK)

#define CAN_CTRL1_TWRNMSK_MASK                   (0x800U)
#define CAN_CTRL1_TWRNMSK_SHIFT                  (11U)
/*! TWRNMSK - TX Warning Interrupt Mask
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define CAN_CTRL1_TWRNMSK(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_TWRNMSK_SHIFT)) & CAN_CTRL1_TWRNMSK_MASK)

#define CAN_CTRL1_LPB_MASK                       (0x1000U)
#define CAN_CTRL1_LPB_SHIFT                      (12U)
/*! LPB - Loopback Mode
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define CAN_CTRL1_LPB(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_LPB_SHIFT)) & CAN_CTRL1_LPB_MASK)

#define CAN_CTRL1_CLKSRC_MASK                    (0x2000U)
#define CAN_CTRL1_CLKSRC_SHIFT                   (13U)
/*! CLKSRC - CAN Engine Clock Source
 *  0b0..Peripheral clock
 *  0b1..Bus clock
 */
#define CAN_CTRL1_CLKSRC(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_CLKSRC_SHIFT)) & CAN_CTRL1_CLKSRC_MASK)

#define CAN_CTRL1_ERRMSK_MASK                    (0x4000U)
#define CAN_CTRL1_ERRMSK_SHIFT                   (14U)
/*! ERRMSK - Error Interrupt Mask
 *  0b0..Interrupt disabled
 *  0b1..Interrupt enabled
 */
#define CAN_CTRL1_ERRMSK(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_ERRMSK_SHIFT)) & CAN_CTRL1_ERRMSK_MASK)

#define CAN_CTRL1_BOFFMSK_MASK                   (0x8000U)
#define CAN_CTRL1_BOFFMSK_SHIFT                  (15U)
/*! BOFFMSK - Bus Off Interrupt Mask
 *  0b0..Interrupt disabled
 *  0b1..Interrupt enabled
 */
#define CAN_CTRL1_BOFFMSK(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_BOFFMSK_SHIFT)) & CAN_CTRL1_BOFFMSK_MASK)

#define CAN_CTRL1_PSEG2_MASK                     (0x70000U)
#define CAN_CTRL1_PSEG2_SHIFT                    (16U)
/*! PSEG2 - Phase Segment 2 */
#define CAN_CTRL1_PSEG2(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_PSEG2_SHIFT)) & CAN_CTRL1_PSEG2_MASK)

#define CAN_CTRL1_PSEG1_MASK                     (0x380000U)
#define CAN_CTRL1_PSEG1_SHIFT                    (19U)
/*! PSEG1 - Phase Segment 1 */
#define CAN_CTRL1_PSEG1(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_PSEG1_SHIFT)) & CAN_CTRL1_PSEG1_MASK)

#define CAN_CTRL1_RJW_MASK                       (0xC00000U)
#define CAN_CTRL1_RJW_SHIFT                      (22U)
/*! RJW - Resync Jump Width */
#define CAN_CTRL1_RJW(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_RJW_SHIFT)) & CAN_CTRL1_RJW_MASK)

#define CAN_CTRL1_PRESDIV_MASK                   (0xFF000000U)
#define CAN_CTRL1_PRESDIV_SHIFT                  (24U)
/*! PRESDIV - Prescaler Division Factor */
#define CAN_CTRL1_PRESDIV(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CTRL1_PRESDIV_SHIFT)) & CAN_CTRL1_PRESDIV_MASK)
/*! @} */

/*! @name TIMER - Free-Running Timer */
/*! @{ */

#define CAN_TIMER_TIMER_MASK                     (0xFFFFU)
#define CAN_TIMER_TIMER_SHIFT                    (0U)
/*! TIMER - Timer Value */
#define CAN_TIMER_TIMER(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_TIMER_TIMER_SHIFT)) & CAN_TIMER_TIMER_MASK)
/*! @} */

/*! @name RXMGMASK - RX Message Buffers Global Mask */
/*! @{ */

#define CAN_RXMGMASK_MG_MASK                     (0xFFFFFFFFU)
#define CAN_RXMGMASK_MG_SHIFT                    (0U)
/*! MG - Global Mask for RX Message Buffers */
#define CAN_RXMGMASK_MG(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_RXMGMASK_MG_SHIFT)) & CAN_RXMGMASK_MG_MASK)
/*! @} */

/*! @name RX14MASK - Receive 14 Mask */
/*! @{ */

#define CAN_RX14MASK_RX14M_MASK                  (0xFFFFFFFFU)
#define CAN_RX14MASK_RX14M_SHIFT                 (0U)
/*! RX14M - RX Buffer 14 Mask Bits */
#define CAN_RX14MASK_RX14M(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_RX14MASK_RX14M_SHIFT)) & CAN_RX14MASK_RX14M_MASK)
/*! @} */

/*! @name RX15MASK - Receive 15 Mask */
/*! @{ */

#define CAN_RX15MASK_RX15M_MASK                  (0xFFFFFFFFU)
#define CAN_RX15MASK_RX15M_SHIFT                 (0U)
/*! RX15M - RX Buffer 15 Mask Bits */
#define CAN_RX15MASK_RX15M(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_RX15MASK_RX15M_SHIFT)) & CAN_RX15MASK_RX15M_MASK)
/*! @} */

/*! @name ECR - Error Counter */
/*! @{ */

#define CAN_ECR_TXERRCNT_MASK                    (0xFFU)
#define CAN_ECR_TXERRCNT_SHIFT                   (0U)
/*! TXERRCNT - Transmit Error Counter */
#define CAN_ECR_TXERRCNT(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ECR_TXERRCNT_SHIFT)) & CAN_ECR_TXERRCNT_MASK)

#define CAN_ECR_RXERRCNT_MASK                    (0xFF00U)
#define CAN_ECR_RXERRCNT_SHIFT                   (8U)
/*! RXERRCNT - Receive Error Counter */
#define CAN_ECR_RXERRCNT(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ECR_RXERRCNT_SHIFT)) & CAN_ECR_RXERRCNT_MASK)

#define CAN_ECR_TXERRCNT_FAST_MASK               (0xFF0000U)
#define CAN_ECR_TXERRCNT_FAST_SHIFT              (16U)
/*! TXERRCNT_FAST - Transmit Error Counter for Fast Bits */
#define CAN_ECR_TXERRCNT_FAST(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_ECR_TXERRCNT_FAST_SHIFT)) & CAN_ECR_TXERRCNT_FAST_MASK)

#define CAN_ECR_RXERRCNT_FAST_MASK               (0xFF000000U)
#define CAN_ECR_RXERRCNT_FAST_SHIFT              (24U)
/*! RXERRCNT_FAST - Receive Error Counter for Fast Bits */
#define CAN_ECR_RXERRCNT_FAST(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_ECR_RXERRCNT_FAST_SHIFT)) & CAN_ECR_RXERRCNT_FAST_MASK)
/*! @} */

/*! @name ESR1 - Error and Status 1 */
/*! @{ */

#define CAN_ESR1_WAKINT_MASK                     (0x1U)
#define CAN_ESR1_WAKINT_SHIFT                    (0U)
/*! WAKINT - Wake-up Interrupt Flag
 *  0b0..No such occurrence.
 *  0b1..Indicates that a recessive-to-dominant transition was received on the CAN bus.
 */
#define CAN_ESR1_WAKINT(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_WAKINT_SHIFT)) & CAN_ESR1_WAKINT_MASK)

#define CAN_ESR1_ERRINT_MASK                     (0x2U)
#define CAN_ESR1_ERRINT_SHIFT                    (1U)
/*! ERRINT - Error Interrupt Flag
 *  0b0..No such occurrence.
 *  0b1..Indicates setting of any error flag in the Error and Status register.
 */
#define CAN_ESR1_ERRINT(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_ERRINT_SHIFT)) & CAN_ESR1_ERRINT_MASK)

#define CAN_ESR1_BOFFINT_MASK                    (0x4U)
#define CAN_ESR1_BOFFINT_SHIFT                   (2U)
/*! BOFFINT - Bus Off Interrupt Flag
 *  0b0..No such occurrence.
 *  0b1..FlexCAN module entered Bus Off state.
 */
#define CAN_ESR1_BOFFINT(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_BOFFINT_SHIFT)) & CAN_ESR1_BOFFINT_MASK)

#define CAN_ESR1_RX_MASK                         (0x8U)
#define CAN_ESR1_RX_SHIFT                        (3U)
/*! RX - FlexCAN in Reception Flag
 *  0b0..Not receiving
 *  0b1..Receiving
 */
#define CAN_ESR1_RX(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_RX_SHIFT)) & CAN_ESR1_RX_MASK)

#define CAN_ESR1_FLTCONF_MASK                    (0x30U)
#define CAN_ESR1_FLTCONF_SHIFT                   (4U)
/*! FLTCONF - Fault Confinement State
 *  0b00..Error Active
 *  0b01..Error Passive
 *  0b1x..Bus Off
 */
#define CAN_ESR1_FLTCONF(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_FLTCONF_SHIFT)) & CAN_ESR1_FLTCONF_MASK)

#define CAN_ESR1_TX_MASK                         (0x40U)
#define CAN_ESR1_TX_SHIFT                        (6U)
/*! TX - FlexCAN In Transmission
 *  0b0..Not transmitting
 *  0b1..Transmitting
 */
#define CAN_ESR1_TX(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_TX_SHIFT)) & CAN_ESR1_TX_MASK)

#define CAN_ESR1_IDLE_MASK                       (0x80U)
#define CAN_ESR1_IDLE_SHIFT                      (7U)
/*! IDLE - Idle
 *  0b0..Not IDLE
 *  0b1..IDLE
 */
#define CAN_ESR1_IDLE(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_IDLE_SHIFT)) & CAN_ESR1_IDLE_MASK)

#define CAN_ESR1_RXWRN_MASK                      (0x100U)
#define CAN_ESR1_RXWRN_SHIFT                     (8U)
/*! RXWRN - RX Error Warning Flag
 *  0b0..No such occurrence.
 *  0b1..RXERRCNT is greater than or equal to 96.
 */
#define CAN_ESR1_RXWRN(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_RXWRN_SHIFT)) & CAN_ESR1_RXWRN_MASK)

#define CAN_ESR1_TXWRN_MASK                      (0x200U)
#define CAN_ESR1_TXWRN_SHIFT                     (9U)
/*! TXWRN - TX Error Warning Flag
 *  0b0..No such occurrence.
 *  0b1..TXERRCNT is 96 or greater.
 */
#define CAN_ESR1_TXWRN(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_TXWRN_SHIFT)) & CAN_ESR1_TXWRN_MASK)

#define CAN_ESR1_STFERR_MASK                     (0x400U)
#define CAN_ESR1_STFERR_SHIFT                    (10U)
/*! STFERR - Stuffing Error Flag
 *  0b0..No error
 *  0b1..Error occurred since last read of this register.
 */
#define CAN_ESR1_STFERR(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_STFERR_SHIFT)) & CAN_ESR1_STFERR_MASK)

#define CAN_ESR1_FRMERR_MASK                     (0x800U)
#define CAN_ESR1_FRMERR_SHIFT                    (11U)
/*! FRMERR - Form Error Flag
 *  0b0..No error
 *  0b1..Error occurred since last read of this register.
 */
#define CAN_ESR1_FRMERR(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_FRMERR_SHIFT)) & CAN_ESR1_FRMERR_MASK)

#define CAN_ESR1_CRCERR_MASK                     (0x1000U)
#define CAN_ESR1_CRCERR_SHIFT                    (12U)
/*! CRCERR - Cyclic Redundancy Check Error Flag
 *  0b0..No error
 *  0b1..Error occurred since last read of this register.
 */
#define CAN_ESR1_CRCERR(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_CRCERR_SHIFT)) & CAN_ESR1_CRCERR_MASK)

#define CAN_ESR1_ACKERR_MASK                     (0x2000U)
#define CAN_ESR1_ACKERR_SHIFT                    (13U)
/*! ACKERR - Acknowledge Error Flag
 *  0b0..No error
 *  0b1..Error occurred since last read of this register.
 */
#define CAN_ESR1_ACKERR(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_ACKERR_SHIFT)) & CAN_ESR1_ACKERR_MASK)

#define CAN_ESR1_BIT0ERR_MASK                    (0x4000U)
#define CAN_ESR1_BIT0ERR_SHIFT                   (14U)
/*! BIT0ERR - Bit0 Error Flag
 *  0b0..No such occurrence.
 *  0b1..At least one bit sent as dominant is received as recessive.
 */
#define CAN_ESR1_BIT0ERR(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_BIT0ERR_SHIFT)) & CAN_ESR1_BIT0ERR_MASK)

#define CAN_ESR1_BIT1ERR_MASK                    (0x8000U)
#define CAN_ESR1_BIT1ERR_SHIFT                   (15U)
/*! BIT1ERR - Bit1 Error Flag
 *  0b0..No such occurrence.
 *  0b1..At least one bit sent as recessive is received as dominant.
 */
#define CAN_ESR1_BIT1ERR(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_BIT1ERR_SHIFT)) & CAN_ESR1_BIT1ERR_MASK)

#define CAN_ESR1_RWRNINT_MASK                    (0x10000U)
#define CAN_ESR1_RWRNINT_SHIFT                   (16U)
/*! RWRNINT - RX Warning Interrupt Flag
 *  0b0..No such occurrence
 *  0b1..RX error counter changed from less than 96 to greater than or equal to 96.
 */
#define CAN_ESR1_RWRNINT(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_RWRNINT_SHIFT)) & CAN_ESR1_RWRNINT_MASK)

#define CAN_ESR1_TWRNINT_MASK                    (0x20000U)
#define CAN_ESR1_TWRNINT_SHIFT                   (17U)
/*! TWRNINT - TX Warning Interrupt Flag
 *  0b0..No such occurrence
 *  0b1..TX error counter changed from less than 96 to greater than or equal to 96.
 */
#define CAN_ESR1_TWRNINT(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_TWRNINT_SHIFT)) & CAN_ESR1_TWRNINT_MASK)

#define CAN_ESR1_SYNCH_MASK                      (0x40000U)
#define CAN_ESR1_SYNCH_SHIFT                     (18U)
/*! SYNCH - CAN Synchronization Status Flag
 *  0b0..Not synchronized
 *  0b1..Synchronized
 */
#define CAN_ESR1_SYNCH(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_SYNCH_SHIFT)) & CAN_ESR1_SYNCH_MASK)

#define CAN_ESR1_BOFFDONEINT_MASK                (0x80000U)
#define CAN_ESR1_BOFFDONEINT_SHIFT               (19U)
/*! BOFFDONEINT - Bus Off Done Interrupt Flag
 *  0b0..No such occurrence
 *  0b1..FlexCAN module has completed Bus Off process.
 */
#define CAN_ESR1_BOFFDONEINT(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_BOFFDONEINT_SHIFT)) & CAN_ESR1_BOFFDONEINT_MASK)

#define CAN_ESR1_ERRINT_FAST_MASK                (0x100000U)
#define CAN_ESR1_ERRINT_FAST_SHIFT               (20U)
/*! ERRINT_FAST - Fast Error Interrupt Flag
 *  0b0..No such occurrence.
 *  0b1..Error flag set in the data phase of CAN FD frames that have BRS = 1.
 */
#define CAN_ESR1_ERRINT_FAST(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_ERRINT_FAST_SHIFT)) & CAN_ESR1_ERRINT_FAST_MASK)

#define CAN_ESR1_ERROVR_MASK                     (0x200000U)
#define CAN_ESR1_ERROVR_SHIFT                    (21U)
/*! ERROVR - Error Overrun Flag
 *  0b0..No overrun
 *  0b1..Overrun
 */
#define CAN_ESR1_ERROVR(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_ERROVR_SHIFT)) & CAN_ESR1_ERROVR_MASK)

#define CAN_ESR1_STFERR_FAST_MASK                (0x4000000U)
#define CAN_ESR1_STFERR_FAST_SHIFT               (26U)
/*! STFERR_FAST - Fast Stuffing Error Flag
 *  0b0..No such occurrence.
 *  0b1..A stuffing error occurred since last read of this register.
 */
#define CAN_ESR1_STFERR_FAST(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_STFERR_FAST_SHIFT)) & CAN_ESR1_STFERR_FAST_MASK)

#define CAN_ESR1_FRMERR_FAST_MASK                (0x8000000U)
#define CAN_ESR1_FRMERR_FAST_SHIFT               (27U)
/*! FRMERR_FAST - Fast Form Error Flag
 *  0b0..No such occurrence.
 *  0b1..A form error occurred since last read of this register.
 */
#define CAN_ESR1_FRMERR_FAST(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_FRMERR_FAST_SHIFT)) & CAN_ESR1_FRMERR_FAST_MASK)

#define CAN_ESR1_CRCERR_FAST_MASK                (0x10000000U)
#define CAN_ESR1_CRCERR_FAST_SHIFT               (28U)
/*! CRCERR_FAST - Fast Cyclic Redundancy Check Error Flag
 *  0b0..No such occurrence.
 *  0b1..A CRC error occurred since last read of this register.
 */
#define CAN_ESR1_CRCERR_FAST(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_CRCERR_FAST_SHIFT)) & CAN_ESR1_CRCERR_FAST_MASK)

#define CAN_ESR1_BIT0ERR_FAST_MASK               (0x40000000U)
#define CAN_ESR1_BIT0ERR_FAST_SHIFT              (30U)
/*! BIT0ERR_FAST - Fast Bit0 Error Flag
 *  0b0..No such occurrence.
 *  0b1..At least one bit transmitted as dominant is received as recessive.
 */
#define CAN_ESR1_BIT0ERR_FAST(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_BIT0ERR_FAST_SHIFT)) & CAN_ESR1_BIT0ERR_FAST_MASK)

#define CAN_ESR1_BIT1ERR_FAST_MASK               (0x80000000U)
#define CAN_ESR1_BIT1ERR_FAST_SHIFT              (31U)
/*! BIT1ERR_FAST - Fast Bit1 Error Flag
 *  0b0..No such occurrence.
 *  0b1..At least one bit transmitted as recessive is received as dominant.
 */
#define CAN_ESR1_BIT1ERR_FAST(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_ESR1_BIT1ERR_FAST_SHIFT)) & CAN_ESR1_BIT1ERR_FAST_MASK)
/*! @} */

/*! @name IMASK2 - Interrupt Masks 2 */
/*! @{ */

#define CAN_IMASK2_BUF63TO32M_MASK               (0xFFFFFFFFU)
#define CAN_IMASK2_BUF63TO32M_SHIFT              (0U)
/*! BUF63TO32M - Buffer MBi Mask */
#define CAN_IMASK2_BUF63TO32M(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_IMASK2_BUF63TO32M_SHIFT)) & CAN_IMASK2_BUF63TO32M_MASK)
/*! @} */

/*! @name IMASK1 - Interrupt Masks 1 */
/*! @{ */

#define CAN_IMASK1_BUF31TO0M_MASK                (0xFFFFFFFFU)
#define CAN_IMASK1_BUF31TO0M_SHIFT               (0U)
/*! BUF31TO0M - Buffer MBi Mask */
#define CAN_IMASK1_BUF31TO0M(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_IMASK1_BUF31TO0M_SHIFT)) & CAN_IMASK1_BUF31TO0M_MASK)
/*! @} */

/*! @name IFLAG2 - Interrupt Flags 2 */
/*! @{ */

#define CAN_IFLAG2_BUF63TO32I_MASK               (0xFFFFFFFFU)
#define CAN_IFLAG2_BUF63TO32I_SHIFT              (0U)
/*! BUF63TO32I - Buffer MBi Interrupt */
#define CAN_IFLAG2_BUF63TO32I(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG2_BUF63TO32I_SHIFT)) & CAN_IFLAG2_BUF63TO32I_MASK)
/*! @} */

/*! @name IFLAG1 - Interrupt Flags 1 */
/*! @{ */

#define CAN_IFLAG1_BUF0I_MASK                    (0x1U)
#define CAN_IFLAG1_BUF0I_SHIFT                   (0U)
/*! BUF0I - Buffer MB0 Interrupt or Clear Legacy FIFO bit
 *  0b0..MB0 has no occurrence of successfully completed transmission or reception.
 *  0b1..MB0 has successfully completed transmission or reception.
 */
#define CAN_IFLAG1_BUF0I(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG1_BUF0I_SHIFT)) & CAN_IFLAG1_BUF0I_MASK)

#define CAN_IFLAG1_BUF4TO1I_MASK                 (0x1EU)
#define CAN_IFLAG1_BUF4TO1I_SHIFT                (1U)
/*! BUF4TO1I - Buffer MBi Interrupt or Reserved */
#define CAN_IFLAG1_BUF4TO1I(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG1_BUF4TO1I_SHIFT)) & CAN_IFLAG1_BUF4TO1I_MASK)

#define CAN_IFLAG1_BUF5I_MASK                    (0x20U)
#define CAN_IFLAG1_BUF5I_SHIFT                   (5U)
/*! BUF5I - Buffer MB5 Interrupt or Frames available in Legacy RX FIFO
 *  0b0..No occurrence of completed transmission or reception, or no frames available
 *  0b1..MB5 completed transmission or reception, or frames available
 */
#define CAN_IFLAG1_BUF5I(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG1_BUF5I_SHIFT)) & CAN_IFLAG1_BUF5I_MASK)

#define CAN_IFLAG1_BUF6I_MASK                    (0x40U)
#define CAN_IFLAG1_BUF6I_SHIFT                   (6U)
/*! BUF6I - Buffer MB6 Interrupt or Legacy RX FIFO Warning
 *  0b0..No occurrence of MB6 completing transmission or reception, or FIFO not almost full.
 *  0b1..MB6 completed transmission or reception, or FIFO almost full.
 */
#define CAN_IFLAG1_BUF6I(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG1_BUF6I_SHIFT)) & CAN_IFLAG1_BUF6I_MASK)

#define CAN_IFLAG1_BUF7I_MASK                    (0x80U)
#define CAN_IFLAG1_BUF7I_SHIFT                   (7U)
/*! BUF7I - Buffer MB7 Interrupt or Legacy RX FIFO Overflow
 *  0b0..No occurrence of MB7 completing transmission or reception, or no FIFO overflow.
 *  0b1..MB7 completed transmission or reception, or FIFO overflow.
 */
#define CAN_IFLAG1_BUF7I(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG1_BUF7I_SHIFT)) & CAN_IFLAG1_BUF7I_MASK)

#define CAN_IFLAG1_BUF31TO8I_MASK                (0xFFFFFF00U)
#define CAN_IFLAG1_BUF31TO8I_SHIFT               (8U)
/*! BUF31TO8I - Buffer MBi Interrupt */
#define CAN_IFLAG1_BUF31TO8I(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG1_BUF31TO8I_SHIFT)) & CAN_IFLAG1_BUF31TO8I_MASK)
/*! @} */

/*! @name CTRL2 - Control 2 */
/*! @{ */

#define CAN_CTRL2_TSTAMPCAP_MASK                 (0xC0U)
#define CAN_CTRL2_TSTAMPCAP_SHIFT                (6U)
/*! TSTAMPCAP - Timestamp Capture Point
 *  0b00..Disabled
 *  0b01..End of the CAN frame
 *  0b10..Start of the CAN frame
 *  0b11..Start of frame for classical CAN frames; res bit for CAN FD frames
 */
#define CAN_CTRL2_TSTAMPCAP(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_TSTAMPCAP_SHIFT)) & CAN_CTRL2_TSTAMPCAP_MASK)

#define CAN_CTRL2_MBTSBASE_MASK                  (0x300U)
#define CAN_CTRL2_MBTSBASE_SHIFT                 (8U)
/*! MBTSBASE - Message Buffer Timestamp Base
 *  0b00..TIMER
 *  0b01..Lower 16 bits of high-resolution timer
 *  0b10..Upper 16 bits of high-resolution timer
 *  0b11..Reserved
 */
#define CAN_CTRL2_MBTSBASE(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_MBTSBASE_SHIFT)) & CAN_CTRL2_MBTSBASE_MASK)

#define CAN_CTRL2_EDFLTDIS_MASK                  (0x800U)
#define CAN_CTRL2_EDFLTDIS_SHIFT                 (11U)
/*! EDFLTDIS - Edge Filter Disable
 *  0b0..Enabled
 *  0b1..Disabled
 */
#define CAN_CTRL2_EDFLTDIS(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_EDFLTDIS_SHIFT)) & CAN_CTRL2_EDFLTDIS_MASK)

#define CAN_CTRL2_ISOCANFDEN_MASK                (0x1000U)
#define CAN_CTRL2_ISOCANFDEN_SHIFT               (12U)
/*! ISOCANFDEN - ISO CAN FD Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL2_ISOCANFDEN(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_ISOCANFDEN_SHIFT)) & CAN_CTRL2_ISOCANFDEN_MASK)

#define CAN_CTRL2_BTE_MASK                       (0x2000U)
#define CAN_CTRL2_BTE_SHIFT                      (13U)
/*! BTE - Bit Timing Expansion Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL2_BTE(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_BTE_SHIFT)) & CAN_CTRL2_BTE_MASK)

#define CAN_CTRL2_PREXCEN_MASK                   (0x4000U)
#define CAN_CTRL2_PREXCEN_SHIFT                  (14U)
/*! PREXCEN - Protocol Exception Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define CAN_CTRL2_PREXCEN(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_PREXCEN_SHIFT)) & CAN_CTRL2_PREXCEN_MASK)

#define CAN_CTRL2_TIMER_SRC_MASK                 (0x8000U)
#define CAN_CTRL2_TIMER_SRC_SHIFT                (15U)
/*! TIMER_SRC - Timer Source
 *  0b0..CAN bit clock
 *  0b1..External time tick
 */
#define CAN_CTRL2_TIMER_SRC(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_TIMER_SRC_SHIFT)) & CAN_CTRL2_TIMER_SRC_MASK)

#define CAN_CTRL2_EACEN_MASK                     (0x10000U)
#define CAN_CTRL2_EACEN_SHIFT                    (16U)
/*! EACEN - Entire Frame Arbitration Field Comparison Enable for RX Message Buffers
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL2_EACEN(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_EACEN_SHIFT)) & CAN_CTRL2_EACEN_MASK)

#define CAN_CTRL2_RRS_MASK                       (0x20000U)
#define CAN_CTRL2_RRS_SHIFT                      (17U)
/*! RRS - Remote Request Storing
 *  0b0..Generated
 *  0b1..Stored
 */
#define CAN_CTRL2_RRS(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_RRS_SHIFT)) & CAN_CTRL2_RRS_MASK)

#define CAN_CTRL2_MRP_MASK                       (0x40000U)
#define CAN_CTRL2_MRP_SHIFT                      (18U)
/*! MRP - Message Buffers Reception Priority
 *  0b0..Matching starts from Legacy RX FIFO or Enhanced RX FIFO and continues on message buffers.
 *  0b1..Matching starts from message buffers and continues on Legacy RX FIFO or Enhanced RX FIFO.
 */
#define CAN_CTRL2_MRP(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_MRP_SHIFT)) & CAN_CTRL2_MRP_MASK)

#define CAN_CTRL2_TASD_MASK                      (0xF80000U)
#define CAN_CTRL2_TASD_SHIFT                     (19U)
/*! TASD - Transmission Arbitration Start Delay */
#define CAN_CTRL2_TASD(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_TASD_SHIFT)) & CAN_CTRL2_TASD_MASK)

#define CAN_CTRL2_RFFN_MASK                      (0xF000000U)
#define CAN_CTRL2_RFFN_SHIFT                     (24U)
/*! RFFN - Number of Legacy Receive FIFO Filters */
#define CAN_CTRL2_RFFN(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_RFFN_SHIFT)) & CAN_CTRL2_RFFN_MASK)

#define CAN_CTRL2_WRMFRZ_MASK                    (0x10000000U)
#define CAN_CTRL2_WRMFRZ_SHIFT                   (28U)
/*! WRMFRZ - Write Access to Memory in Freeze Mode
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL2_WRMFRZ(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_WRMFRZ_SHIFT)) & CAN_CTRL2_WRMFRZ_MASK)

#define CAN_CTRL2_ECRWRE_MASK                    (0x20000000U)
#define CAN_CTRL2_ECRWRE_SHIFT                   (29U)
/*! ECRWRE - Error Correction Configuration Register Write Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL2_ECRWRE(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_ECRWRE_SHIFT)) & CAN_CTRL2_ECRWRE_MASK)

#define CAN_CTRL2_BOFFDONEMSK_MASK               (0x40000000U)
#define CAN_CTRL2_BOFFDONEMSK_SHIFT              (30U)
/*! BOFFDONEMSK - Bus Off Done Interrupt Mask
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL2_BOFFDONEMSK(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_BOFFDONEMSK_SHIFT)) & CAN_CTRL2_BOFFDONEMSK_MASK)

#define CAN_CTRL2_ERRMSK_FAST_MASK               (0x80000000U)
#define CAN_CTRL2_ERRMSK_FAST_SHIFT              (31U)
/*! ERRMSK_FAST - Error Interrupt Mask for Errors Detected in the Data Phase of Fast CAN FD Frames
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CTRL2_ERRMSK_FAST(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_CTRL2_ERRMSK_FAST_SHIFT)) & CAN_CTRL2_ERRMSK_FAST_MASK)
/*! @} */

/*! @name ESR2 - Error and Status 2 */
/*! @{ */

#define CAN_ESR2_IMB_MASK                        (0x2000U)
#define CAN_ESR2_IMB_SHIFT                       (13U)
/*! IMB - Inactive Message Buffer
 *  0b0..Message buffer indicated by ESR2[LPTM] is not inactive.
 *  0b1..At least one message buffer is inactive.
 */
#define CAN_ESR2_IMB(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_ESR2_IMB_SHIFT)) & CAN_ESR2_IMB_MASK)

#define CAN_ESR2_VPS_MASK                        (0x4000U)
#define CAN_ESR2_VPS_SHIFT                       (14U)
/*! VPS - Valid Priority Status
 *  0b0..Invalid
 *  0b1..Valid
 */
#define CAN_ESR2_VPS(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_ESR2_VPS_SHIFT)) & CAN_ESR2_VPS_MASK)

#define CAN_ESR2_LPTM_MASK                       (0x7F0000U)
#define CAN_ESR2_LPTM_SHIFT                      (16U)
/*! LPTM - Lowest Priority TX Message Buffer */
#define CAN_ESR2_LPTM(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_ESR2_LPTM_SHIFT)) & CAN_ESR2_LPTM_MASK)
/*! @} */

/*! @name CRCR - Cyclic Redundancy Check */
/*! @{ */

#define CAN_CRCR_TXCRC_MASK                      (0x7FFFU)
#define CAN_CRCR_TXCRC_SHIFT                     (0U)
/*! TXCRC - Transmitted CRC value */
#define CAN_CRCR_TXCRC(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CRCR_TXCRC_SHIFT)) & CAN_CRCR_TXCRC_MASK)

#define CAN_CRCR_MBCRC_MASK                      (0x7F0000U)
#define CAN_CRCR_MBCRC_SHIFT                     (16U)
/*! MBCRC - CRC Message Buffer */
#define CAN_CRCR_MBCRC(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CRCR_MBCRC_SHIFT)) & CAN_CRCR_MBCRC_MASK)
/*! @} */

/*! @name RXFGMASK - Legacy RX FIFO Global Mask */
/*! @{ */

#define CAN_RXFGMASK_FGM_MASK                    (0xFFFFFFFFU)
#define CAN_RXFGMASK_FGM_SHIFT                   (0U)
/*! FGM - Legacy RX FIFO Global Mask Bits */
#define CAN_RXFGMASK_FGM(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_RXFGMASK_FGM_SHIFT)) & CAN_RXFGMASK_FGM_MASK)
/*! @} */

/*! @name RXFIR - Legacy RX FIFO Information */
/*! @{ */

#define CAN_RXFIR_IDHIT_MASK                     (0x1FFU)
#define CAN_RXFIR_IDHIT_SHIFT                    (0U)
/*! IDHIT - Identifier Acceptance Filter Hit Indicator */
#define CAN_RXFIR_IDHIT(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_RXFIR_IDHIT_SHIFT)) & CAN_RXFIR_IDHIT_MASK)
/*! @} */

/*! @name CBT - CAN Bit Timing */
/*! @{ */

#define CAN_CBT_EPSEG2_MASK                      (0x1FU)
#define CAN_CBT_EPSEG2_SHIFT                     (0U)
/*! EPSEG2 - Extended Phase Segment 2 */
#define CAN_CBT_EPSEG2(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CBT_EPSEG2_SHIFT)) & CAN_CBT_EPSEG2_MASK)

#define CAN_CBT_EPSEG1_MASK                      (0x3E0U)
#define CAN_CBT_EPSEG1_SHIFT                     (5U)
/*! EPSEG1 - Extended Phase Segment 1 */
#define CAN_CBT_EPSEG1(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_CBT_EPSEG1_SHIFT)) & CAN_CBT_EPSEG1_MASK)

#define CAN_CBT_EPROPSEG_MASK                    (0xFC00U)
#define CAN_CBT_EPROPSEG_SHIFT                   (10U)
/*! EPROPSEG - Extended Propagation Segment */
#define CAN_CBT_EPROPSEG(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_CBT_EPROPSEG_SHIFT)) & CAN_CBT_EPROPSEG_MASK)

#define CAN_CBT_ERJW_MASK                        (0x1F0000U)
#define CAN_CBT_ERJW_SHIFT                       (16U)
/*! ERJW - Extended Resync Jump Width */
#define CAN_CBT_ERJW(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_CBT_ERJW_SHIFT)) & CAN_CBT_ERJW_MASK)

#define CAN_CBT_EPRESDIV_MASK                    (0x7FE00000U)
#define CAN_CBT_EPRESDIV_SHIFT                   (21U)
/*! EPRESDIV - Extended Prescaler Division Factor */
#define CAN_CBT_EPRESDIV(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_CBT_EPRESDIV_SHIFT)) & CAN_CBT_EPRESDIV_MASK)

#define CAN_CBT_BTF_MASK                         (0x80000000U)
#define CAN_CBT_BTF_SHIFT                        (31U)
/*! BTF - Bit Timing Format Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_CBT_BTF(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_CBT_BTF_SHIFT)) & CAN_CBT_BTF_MASK)
/*! @} */

/*! @name IMASK3 - Interrupt Masks 3 */
/*! @{ */

#define CAN_IMASK3_BUF95TO64M_MASK               (0xFFFFFFFFU)
#define CAN_IMASK3_BUF95TO64M_SHIFT              (0U)
/*! BUF95TO64M - Buffer MBi Mask */
#define CAN_IMASK3_BUF95TO64M(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_IMASK3_BUF95TO64M_SHIFT)) & CAN_IMASK3_BUF95TO64M_MASK)
/*! @} */

/*! @name IFLAG3 - Interrupt Flags 3 */
/*! @{ */

#define CAN_IFLAG3_BUF95TO64_MASK                (0xFFFFFFFFU)
#define CAN_IFLAG3_BUF95TO64_SHIFT               (0U)
/*! BUF95TO64 - Buffer MBi Interrupt */
#define CAN_IFLAG3_BUF95TO64(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_IFLAG3_BUF95TO64_SHIFT)) & CAN_IFLAG3_BUF95TO64_MASK)
/*! @} */

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB8B                        (96U)

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB8B                        (96U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB8B                      (96U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB8B2                     (2U)

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB16B_L                     (21U)

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB16B_L                     (21U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB16B_L                   (21U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB16B_L2                  (4U)

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB16B_M                     (21U)

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB16B_M                     (21U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB16B_M                   (21U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB16B_M2                  (4U)

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB16B_H                     (21U)

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB16B_H                     (21U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB16B_H                   (21U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB16B_H2                  (4U)

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB32B_L                     (12U)

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB32B_L                     (12U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB32B_L                   (12U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB32B_L2                  (8U)

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB32B_M                     (12U)

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB32B_M                     (12U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB32B_M                   (12U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB32B_M2                  (8U)

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB32B_H                     (12U)

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB32B_H                     (12U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB32B_H                   (12U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB32B_H2                  (8U)

/*! @name CS - Message Buffer 0 CS Register..Message Buffer 6 CS Register */
/*! @{ */

#define CAN_CS_TIME_STAMP_MASK                   (0xFFFFU)
#define CAN_CS_TIME_STAMP_SHIFT                  (0U)
/*! TIME_STAMP - Free-Running Counter Time stamp. This 16-bit field is a copy of the Free-Running
 *    Timer, captured for Tx and Rx frames at the time when the beginning of the Identifier field
 *    appears on the CAN bus.
 */
#define CAN_CS_TIME_STAMP(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CS_TIME_STAMP_SHIFT)) & CAN_CS_TIME_STAMP_MASK)

#define CAN_CS_DLC_MASK                          (0xF0000U)
#define CAN_CS_DLC_SHIFT                         (16U)
/*! DLC - Length of the data to be stored/transmitted. */
#define CAN_CS_DLC(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_DLC_SHIFT)) & CAN_CS_DLC_MASK)

#define CAN_CS_RTR_MASK                          (0x100000U)
#define CAN_CS_RTR_SHIFT                         (20U)
/*! RTR - Remote Transmission Request. One/zero for remote/data frame. */
#define CAN_CS_RTR(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_RTR_SHIFT)) & CAN_CS_RTR_MASK)

#define CAN_CS_IDE_MASK                          (0x200000U)
#define CAN_CS_IDE_SHIFT                         (21U)
/*! IDE - ID Extended. One/zero for extended/standard format frame. */
#define CAN_CS_IDE(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_IDE_SHIFT)) & CAN_CS_IDE_MASK)

#define CAN_CS_SRR_MASK                          (0x400000U)
#define CAN_CS_SRR_SHIFT                         (22U)
/*! SRR - Substitute Remote Request. Contains a fixed recessive bit. */
#define CAN_CS_SRR(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_SRR_SHIFT)) & CAN_CS_SRR_MASK)

#define CAN_CS_CODE_MASK                         (0xF000000U)
#define CAN_CS_CODE_SHIFT                        (24U)
/*! CODE - Message Buffer Code. This 4-bit field can be accessed (read or write) by the CPU and by
 *    the FlexCAN module itself, as part of the message buffer matching and arbitration process.
 */
#define CAN_CS_CODE(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_CS_CODE_SHIFT)) & CAN_CS_CODE_MASK)

#define CAN_CS_ESI_MASK                          (0x20000000U)
#define CAN_CS_ESI_SHIFT                         (29U)
/*! ESI - Error State Indicator. This bit indicates if the transmitting node is error active or error passive. */
#define CAN_CS_ESI(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_ESI_SHIFT)) & CAN_CS_ESI_MASK)

#define CAN_CS_BRS_MASK                          (0x40000000U)
#define CAN_CS_BRS_SHIFT                         (30U)
/*! BRS - Bit Rate Switch. This bit defines whether the bit rate is switched inside a CAN FD format frame. */
#define CAN_CS_BRS(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_BRS_SHIFT)) & CAN_CS_BRS_MASK)

#define CAN_CS_EDL_MASK                          (0x80000000U)
#define CAN_CS_EDL_SHIFT                         (31U)
/*! EDL - Extended Data Length. This bit distinguishes between CAN format and CAN FD format frames.
 *    The EDL bit must not be set for Message Buffers configured to RANSWER with code field 0b1010.
 */
#define CAN_CS_EDL(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_EDL_SHIFT)) & CAN_CS_EDL_MASK)
/*! @} */

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB64B_L                     (7U)

/*! @name ID - Message Buffer 0 ID Register..Message Buffer 6 ID Register */
/*! @{ */

#define CAN_ID_EXT_MASK                          (0x3FFFFU)
#define CAN_ID_EXT_SHIFT                         (0U)
/*! EXT - Contains extended (LOW word) identifier of message buffer. */
#define CAN_ID_EXT(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_ID_EXT_SHIFT)) & CAN_ID_EXT_MASK)

#define CAN_ID_STD_MASK                          (0x1FFC0000U)
#define CAN_ID_STD_SHIFT                         (18U)
/*! STD - Contains standard/extended (HIGH word) identifier of message buffer. */
#define CAN_ID_STD(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_ID_STD_SHIFT)) & CAN_ID_STD_MASK)

#define CAN_ID_PRIO_MASK                         (0xE0000000U)
#define CAN_ID_PRIO_SHIFT                        (29U)
/*! PRIO - Local priority. This 3-bit fieldis only used when LPRIO_EN bit is set in MCR and it only
 *    makes sense for Tx buffers. These bits are not transmitted. They are appended to the regular
 *    ID to define the transmission priority.
 */
#define CAN_ID_PRIO(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_ID_PRIO_SHIFT)) & CAN_ID_PRIO_MASK)
/*! @} */

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB64B_L                     (7U)

/*! @name WORD - Message Buffer 0 WORD_64B Register..Message Buffer 6 WORD_64B Register */
/*! @{ */

#define CAN_WORD_DATA_BYTE_3_MASK                (0xFFU)
#define CAN_WORD_DATA_BYTE_3_SHIFT               (0U)
/*! DATA_BYTE_3 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_3(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_3_SHIFT)) & CAN_WORD_DATA_BYTE_3_MASK)

#define CAN_WORD_DATA_BYTE_7_MASK                (0xFFU)
#define CAN_WORD_DATA_BYTE_7_SHIFT               (0U)
/*! DATA_BYTE_7 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_7(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_7_SHIFT)) & CAN_WORD_DATA_BYTE_7_MASK)

#define CAN_WORD_DATA_BYTE_11_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_11_SHIFT              (0U)
/*! DATA_BYTE_11 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_11(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_11_SHIFT)) & CAN_WORD_DATA_BYTE_11_MASK)

#define CAN_WORD_DATA_BYTE_15_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_15_SHIFT              (0U)
/*! DATA_BYTE_15 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_15(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_15_SHIFT)) & CAN_WORD_DATA_BYTE_15_MASK)

#define CAN_WORD_DATA_BYTE_19_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_19_SHIFT              (0U)
/*! DATA_BYTE_19 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_19(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_19_SHIFT)) & CAN_WORD_DATA_BYTE_19_MASK)

#define CAN_WORD_DATA_BYTE_23_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_23_SHIFT              (0U)
/*! DATA_BYTE_23 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_23(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_23_SHIFT)) & CAN_WORD_DATA_BYTE_23_MASK)

#define CAN_WORD_DATA_BYTE_27_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_27_SHIFT              (0U)
/*! DATA_BYTE_27 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_27(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_27_SHIFT)) & CAN_WORD_DATA_BYTE_27_MASK)

#define CAN_WORD_DATA_BYTE_31_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_31_SHIFT              (0U)
/*! DATA_BYTE_31 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_31(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_31_SHIFT)) & CAN_WORD_DATA_BYTE_31_MASK)

#define CAN_WORD_DATA_BYTE_35_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_35_SHIFT              (0U)
/*! DATA_BYTE_35 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_35(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_35_SHIFT)) & CAN_WORD_DATA_BYTE_35_MASK)

#define CAN_WORD_DATA_BYTE_39_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_39_SHIFT              (0U)
/*! DATA_BYTE_39 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_39(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_39_SHIFT)) & CAN_WORD_DATA_BYTE_39_MASK)

#define CAN_WORD_DATA_BYTE_43_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_43_SHIFT              (0U)
/*! DATA_BYTE_43 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_43(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_43_SHIFT)) & CAN_WORD_DATA_BYTE_43_MASK)

#define CAN_WORD_DATA_BYTE_47_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_47_SHIFT              (0U)
/*! DATA_BYTE_47 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_47(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_47_SHIFT)) & CAN_WORD_DATA_BYTE_47_MASK)

#define CAN_WORD_DATA_BYTE_51_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_51_SHIFT              (0U)
/*! DATA_BYTE_51 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_51(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_51_SHIFT)) & CAN_WORD_DATA_BYTE_51_MASK)

#define CAN_WORD_DATA_BYTE_55_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_55_SHIFT              (0U)
/*! DATA_BYTE_55 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_55(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_55_SHIFT)) & CAN_WORD_DATA_BYTE_55_MASK)

#define CAN_WORD_DATA_BYTE_59_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_59_SHIFT              (0U)
/*! DATA_BYTE_59 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_59(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_59_SHIFT)) & CAN_WORD_DATA_BYTE_59_MASK)

#define CAN_WORD_DATA_BYTE_63_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_63_SHIFT              (0U)
/*! DATA_BYTE_63 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_63(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_63_SHIFT)) & CAN_WORD_DATA_BYTE_63_MASK)

#define CAN_WORD_DATA_BYTE_2_MASK                (0xFF00U)
#define CAN_WORD_DATA_BYTE_2_SHIFT               (8U)
/*! DATA_BYTE_2 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_2(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_2_SHIFT)) & CAN_WORD_DATA_BYTE_2_MASK)

#define CAN_WORD_DATA_BYTE_6_MASK                (0xFF00U)
#define CAN_WORD_DATA_BYTE_6_SHIFT               (8U)
/*! DATA_BYTE_6 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_6(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_6_SHIFT)) & CAN_WORD_DATA_BYTE_6_MASK)

#define CAN_WORD_DATA_BYTE_10_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_10_SHIFT              (8U)
/*! DATA_BYTE_10 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_10(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_10_SHIFT)) & CAN_WORD_DATA_BYTE_10_MASK)

#define CAN_WORD_DATA_BYTE_14_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_14_SHIFT              (8U)
/*! DATA_BYTE_14 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_14(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_14_SHIFT)) & CAN_WORD_DATA_BYTE_14_MASK)

#define CAN_WORD_DATA_BYTE_18_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_18_SHIFT              (8U)
/*! DATA_BYTE_18 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_18(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_18_SHIFT)) & CAN_WORD_DATA_BYTE_18_MASK)

#define CAN_WORD_DATA_BYTE_22_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_22_SHIFT              (8U)
/*! DATA_BYTE_22 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_22(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_22_SHIFT)) & CAN_WORD_DATA_BYTE_22_MASK)

#define CAN_WORD_DATA_BYTE_26_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_26_SHIFT              (8U)
/*! DATA_BYTE_26 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_26(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_26_SHIFT)) & CAN_WORD_DATA_BYTE_26_MASK)

#define CAN_WORD_DATA_BYTE_30_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_30_SHIFT              (8U)
/*! DATA_BYTE_30 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_30(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_30_SHIFT)) & CAN_WORD_DATA_BYTE_30_MASK)

#define CAN_WORD_DATA_BYTE_34_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_34_SHIFT              (8U)
/*! DATA_BYTE_34 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_34(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_34_SHIFT)) & CAN_WORD_DATA_BYTE_34_MASK)

#define CAN_WORD_DATA_BYTE_38_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_38_SHIFT              (8U)
/*! DATA_BYTE_38 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_38(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_38_SHIFT)) & CAN_WORD_DATA_BYTE_38_MASK)

#define CAN_WORD_DATA_BYTE_42_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_42_SHIFT              (8U)
/*! DATA_BYTE_42 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_42(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_42_SHIFT)) & CAN_WORD_DATA_BYTE_42_MASK)

#define CAN_WORD_DATA_BYTE_46_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_46_SHIFT              (8U)
/*! DATA_BYTE_46 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_46(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_46_SHIFT)) & CAN_WORD_DATA_BYTE_46_MASK)

#define CAN_WORD_DATA_BYTE_50_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_50_SHIFT              (8U)
/*! DATA_BYTE_50 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_50(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_50_SHIFT)) & CAN_WORD_DATA_BYTE_50_MASK)

#define CAN_WORD_DATA_BYTE_54_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_54_SHIFT              (8U)
/*! DATA_BYTE_54 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_54(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_54_SHIFT)) & CAN_WORD_DATA_BYTE_54_MASK)

#define CAN_WORD_DATA_BYTE_58_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_58_SHIFT              (8U)
/*! DATA_BYTE_58 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_58(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_58_SHIFT)) & CAN_WORD_DATA_BYTE_58_MASK)

#define CAN_WORD_DATA_BYTE_62_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_62_SHIFT              (8U)
/*! DATA_BYTE_62 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_62(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_62_SHIFT)) & CAN_WORD_DATA_BYTE_62_MASK)

#define CAN_WORD_DATA_BYTE_1_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_1_SHIFT               (16U)
/*! DATA_BYTE_1 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_1(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_1_SHIFT)) & CAN_WORD_DATA_BYTE_1_MASK)

#define CAN_WORD_DATA_BYTE_5_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_5_SHIFT               (16U)
/*! DATA_BYTE_5 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_5(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_5_SHIFT)) & CAN_WORD_DATA_BYTE_5_MASK)

#define CAN_WORD_DATA_BYTE_9_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_9_SHIFT               (16U)
/*! DATA_BYTE_9 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_9(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_9_SHIFT)) & CAN_WORD_DATA_BYTE_9_MASK)

#define CAN_WORD_DATA_BYTE_13_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_13_SHIFT              (16U)
/*! DATA_BYTE_13 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_13(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_13_SHIFT)) & CAN_WORD_DATA_BYTE_13_MASK)

#define CAN_WORD_DATA_BYTE_17_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_17_SHIFT              (16U)
/*! DATA_BYTE_17 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_17(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_17_SHIFT)) & CAN_WORD_DATA_BYTE_17_MASK)

#define CAN_WORD_DATA_BYTE_21_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_21_SHIFT              (16U)
/*! DATA_BYTE_21 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_21(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_21_SHIFT)) & CAN_WORD_DATA_BYTE_21_MASK)

#define CAN_WORD_DATA_BYTE_25_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_25_SHIFT              (16U)
/*! DATA_BYTE_25 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_25(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_25_SHIFT)) & CAN_WORD_DATA_BYTE_25_MASK)

#define CAN_WORD_DATA_BYTE_29_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_29_SHIFT              (16U)
/*! DATA_BYTE_29 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_29(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_29_SHIFT)) & CAN_WORD_DATA_BYTE_29_MASK)

#define CAN_WORD_DATA_BYTE_33_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_33_SHIFT              (16U)
/*! DATA_BYTE_33 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_33(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_33_SHIFT)) & CAN_WORD_DATA_BYTE_33_MASK)

#define CAN_WORD_DATA_BYTE_37_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_37_SHIFT              (16U)
/*! DATA_BYTE_37 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_37(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_37_SHIFT)) & CAN_WORD_DATA_BYTE_37_MASK)

#define CAN_WORD_DATA_BYTE_41_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_41_SHIFT              (16U)
/*! DATA_BYTE_41 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_41(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_41_SHIFT)) & CAN_WORD_DATA_BYTE_41_MASK)

#define CAN_WORD_DATA_BYTE_45_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_45_SHIFT              (16U)
/*! DATA_BYTE_45 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_45(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_45_SHIFT)) & CAN_WORD_DATA_BYTE_45_MASK)

#define CAN_WORD_DATA_BYTE_49_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_49_SHIFT              (16U)
/*! DATA_BYTE_49 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_49(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_49_SHIFT)) & CAN_WORD_DATA_BYTE_49_MASK)

#define CAN_WORD_DATA_BYTE_53_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_53_SHIFT              (16U)
/*! DATA_BYTE_53 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_53(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_53_SHIFT)) & CAN_WORD_DATA_BYTE_53_MASK)

#define CAN_WORD_DATA_BYTE_57_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_57_SHIFT              (16U)
/*! DATA_BYTE_57 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_57(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_57_SHIFT)) & CAN_WORD_DATA_BYTE_57_MASK)

#define CAN_WORD_DATA_BYTE_61_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_61_SHIFT              (16U)
/*! DATA_BYTE_61 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_61(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_61_SHIFT)) & CAN_WORD_DATA_BYTE_61_MASK)

#define CAN_WORD_DATA_BYTE_0_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_0_SHIFT               (24U)
/*! DATA_BYTE_0 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_0(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_0_SHIFT)) & CAN_WORD_DATA_BYTE_0_MASK)

#define CAN_WORD_DATA_BYTE_4_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_4_SHIFT               (24U)
/*! DATA_BYTE_4 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_4(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_4_SHIFT)) & CAN_WORD_DATA_BYTE_4_MASK)

#define CAN_WORD_DATA_BYTE_8_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_8_SHIFT               (24U)
/*! DATA_BYTE_8 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_8(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_8_SHIFT)) & CAN_WORD_DATA_BYTE_8_MASK)

#define CAN_WORD_DATA_BYTE_12_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_12_SHIFT              (24U)
/*! DATA_BYTE_12 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_12(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_12_SHIFT)) & CAN_WORD_DATA_BYTE_12_MASK)

#define CAN_WORD_DATA_BYTE_16_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_16_SHIFT              (24U)
/*! DATA_BYTE_16 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_16(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_16_SHIFT)) & CAN_WORD_DATA_BYTE_16_MASK)

#define CAN_WORD_DATA_BYTE_20_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_20_SHIFT              (24U)
/*! DATA_BYTE_20 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_20(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_20_SHIFT)) & CAN_WORD_DATA_BYTE_20_MASK)

#define CAN_WORD_DATA_BYTE_24_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_24_SHIFT              (24U)
/*! DATA_BYTE_24 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_24(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_24_SHIFT)) & CAN_WORD_DATA_BYTE_24_MASK)

#define CAN_WORD_DATA_BYTE_28_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_28_SHIFT              (24U)
/*! DATA_BYTE_28 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_28(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_28_SHIFT)) & CAN_WORD_DATA_BYTE_28_MASK)

#define CAN_WORD_DATA_BYTE_32_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_32_SHIFT              (24U)
/*! DATA_BYTE_32 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_32(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_32_SHIFT)) & CAN_WORD_DATA_BYTE_32_MASK)

#define CAN_WORD_DATA_BYTE_36_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_36_SHIFT              (24U)
/*! DATA_BYTE_36 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_36(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_36_SHIFT)) & CAN_WORD_DATA_BYTE_36_MASK)

#define CAN_WORD_DATA_BYTE_40_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_40_SHIFT              (24U)
/*! DATA_BYTE_40 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_40(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_40_SHIFT)) & CAN_WORD_DATA_BYTE_40_MASK)

#define CAN_WORD_DATA_BYTE_44_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_44_SHIFT              (24U)
/*! DATA_BYTE_44 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_44(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_44_SHIFT)) & CAN_WORD_DATA_BYTE_44_MASK)

#define CAN_WORD_DATA_BYTE_48_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_48_SHIFT              (24U)
/*! DATA_BYTE_48 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_48(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_48_SHIFT)) & CAN_WORD_DATA_BYTE_48_MASK)

#define CAN_WORD_DATA_BYTE_52_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_52_SHIFT              (24U)
/*! DATA_BYTE_52 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_52(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_52_SHIFT)) & CAN_WORD_DATA_BYTE_52_MASK)

#define CAN_WORD_DATA_BYTE_56_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_56_SHIFT              (24U)
/*! DATA_BYTE_56 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_56(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_56_SHIFT)) & CAN_WORD_DATA_BYTE_56_MASK)

#define CAN_WORD_DATA_BYTE_60_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_60_SHIFT              (24U)
/*! DATA_BYTE_60 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_60(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_60_SHIFT)) & CAN_WORD_DATA_BYTE_60_MASK)
/*! @} */

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB64B_L                   (7U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB64B_L2                  (16U)

/*! @name CS - Message Buffer 0 CS Register..Message Buffer 6 CS Register */
/*! @{ */

#define CAN_CS_TIME_STAMP_MASK                   (0xFFFFU)
#define CAN_CS_TIME_STAMP_SHIFT                  (0U)
/*! TIME_STAMP - Free-Running Counter Time stamp. This 16-bit field is a copy of the Free-Running
 *    Timer, captured for Tx and Rx frames at the time when the beginning of the Identifier field
 *    appears on the CAN bus.
 */
#define CAN_CS_TIME_STAMP(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CS_TIME_STAMP_SHIFT)) & CAN_CS_TIME_STAMP_MASK)

#define CAN_CS_DLC_MASK                          (0xF0000U)
#define CAN_CS_DLC_SHIFT                         (16U)
/*! DLC - Length of the data to be stored/transmitted. */
#define CAN_CS_DLC(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_DLC_SHIFT)) & CAN_CS_DLC_MASK)

#define CAN_CS_RTR_MASK                          (0x100000U)
#define CAN_CS_RTR_SHIFT                         (20U)
/*! RTR - Remote Transmission Request. One/zero for remote/data frame. */
#define CAN_CS_RTR(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_RTR_SHIFT)) & CAN_CS_RTR_MASK)

#define CAN_CS_IDE_MASK                          (0x200000U)
#define CAN_CS_IDE_SHIFT                         (21U)
/*! IDE - ID Extended. One/zero for extended/standard format frame. */
#define CAN_CS_IDE(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_IDE_SHIFT)) & CAN_CS_IDE_MASK)

#define CAN_CS_SRR_MASK                          (0x400000U)
#define CAN_CS_SRR_SHIFT                         (22U)
/*! SRR - Substitute Remote Request. Contains a fixed recessive bit. */
#define CAN_CS_SRR(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_SRR_SHIFT)) & CAN_CS_SRR_MASK)

#define CAN_CS_CODE_MASK                         (0xF000000U)
#define CAN_CS_CODE_SHIFT                        (24U)
/*! CODE - Message Buffer Code. This 4-bit field can be accessed (read or write) by the CPU and by
 *    the FlexCAN module itself, as part of the message buffer matching and arbitration process.
 */
#define CAN_CS_CODE(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_CS_CODE_SHIFT)) & CAN_CS_CODE_MASK)

#define CAN_CS_ESI_MASK                          (0x20000000U)
#define CAN_CS_ESI_SHIFT                         (29U)
/*! ESI - Error State Indicator. This bit indicates if the transmitting node is error active or error passive. */
#define CAN_CS_ESI(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_ESI_SHIFT)) & CAN_CS_ESI_MASK)

#define CAN_CS_BRS_MASK                          (0x40000000U)
#define CAN_CS_BRS_SHIFT                         (30U)
/*! BRS - Bit Rate Switch. This bit defines whether the bit rate is switched inside a CAN FD format frame. */
#define CAN_CS_BRS(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_BRS_SHIFT)) & CAN_CS_BRS_MASK)

#define CAN_CS_EDL_MASK                          (0x80000000U)
#define CAN_CS_EDL_SHIFT                         (31U)
/*! EDL - Extended Data Length. This bit distinguishes between CAN format and CAN FD format frames.
 *    The EDL bit must not be set for Message Buffers configured to RANSWER with code field 0b1010.
 */
#define CAN_CS_EDL(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_EDL_SHIFT)) & CAN_CS_EDL_MASK)
/*! @} */

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB64B_M                     (7U)

/*! @name ID - Message Buffer 0 ID Register..Message Buffer 6 ID Register */
/*! @{ */

#define CAN_ID_EXT_MASK                          (0x3FFFFU)
#define CAN_ID_EXT_SHIFT                         (0U)
/*! EXT - Contains extended (LOW word) identifier of message buffer. */
#define CAN_ID_EXT(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_ID_EXT_SHIFT)) & CAN_ID_EXT_MASK)

#define CAN_ID_STD_MASK                          (0x1FFC0000U)
#define CAN_ID_STD_SHIFT                         (18U)
/*! STD - Contains standard/extended (HIGH word) identifier of message buffer. */
#define CAN_ID_STD(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_ID_STD_SHIFT)) & CAN_ID_STD_MASK)

#define CAN_ID_PRIO_MASK                         (0xE0000000U)
#define CAN_ID_PRIO_SHIFT                        (29U)
/*! PRIO - Local priority. This 3-bit fieldis only used when LPRIO_EN bit is set in MCR and it only
 *    makes sense for Tx buffers. These bits are not transmitted. They are appended to the regular
 *    ID to define the transmission priority.
 */
#define CAN_ID_PRIO(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_ID_PRIO_SHIFT)) & CAN_ID_PRIO_MASK)
/*! @} */

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB64B_M                     (7U)

/*! @name WORD - Message Buffer 0 WORD_64B Register..Message Buffer 6 WORD_64B Register */
/*! @{ */

#define CAN_WORD_DATA_BYTE_3_MASK                (0xFFU)
#define CAN_WORD_DATA_BYTE_3_SHIFT               (0U)
/*! DATA_BYTE_3 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_3(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_3_SHIFT)) & CAN_WORD_DATA_BYTE_3_MASK)

#define CAN_WORD_DATA_BYTE_7_MASK                (0xFFU)
#define CAN_WORD_DATA_BYTE_7_SHIFT               (0U)
/*! DATA_BYTE_7 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_7(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_7_SHIFT)) & CAN_WORD_DATA_BYTE_7_MASK)

#define CAN_WORD_DATA_BYTE_11_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_11_SHIFT              (0U)
/*! DATA_BYTE_11 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_11(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_11_SHIFT)) & CAN_WORD_DATA_BYTE_11_MASK)

#define CAN_WORD_DATA_BYTE_15_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_15_SHIFT              (0U)
/*! DATA_BYTE_15 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_15(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_15_SHIFT)) & CAN_WORD_DATA_BYTE_15_MASK)

#define CAN_WORD_DATA_BYTE_19_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_19_SHIFT              (0U)
/*! DATA_BYTE_19 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_19(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_19_SHIFT)) & CAN_WORD_DATA_BYTE_19_MASK)

#define CAN_WORD_DATA_BYTE_23_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_23_SHIFT              (0U)
/*! DATA_BYTE_23 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_23(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_23_SHIFT)) & CAN_WORD_DATA_BYTE_23_MASK)

#define CAN_WORD_DATA_BYTE_27_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_27_SHIFT              (0U)
/*! DATA_BYTE_27 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_27(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_27_SHIFT)) & CAN_WORD_DATA_BYTE_27_MASK)

#define CAN_WORD_DATA_BYTE_31_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_31_SHIFT              (0U)
/*! DATA_BYTE_31 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_31(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_31_SHIFT)) & CAN_WORD_DATA_BYTE_31_MASK)

#define CAN_WORD_DATA_BYTE_35_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_35_SHIFT              (0U)
/*! DATA_BYTE_35 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_35(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_35_SHIFT)) & CAN_WORD_DATA_BYTE_35_MASK)

#define CAN_WORD_DATA_BYTE_39_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_39_SHIFT              (0U)
/*! DATA_BYTE_39 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_39(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_39_SHIFT)) & CAN_WORD_DATA_BYTE_39_MASK)

#define CAN_WORD_DATA_BYTE_43_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_43_SHIFT              (0U)
/*! DATA_BYTE_43 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_43(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_43_SHIFT)) & CAN_WORD_DATA_BYTE_43_MASK)

#define CAN_WORD_DATA_BYTE_47_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_47_SHIFT              (0U)
/*! DATA_BYTE_47 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_47(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_47_SHIFT)) & CAN_WORD_DATA_BYTE_47_MASK)

#define CAN_WORD_DATA_BYTE_51_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_51_SHIFT              (0U)
/*! DATA_BYTE_51 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_51(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_51_SHIFT)) & CAN_WORD_DATA_BYTE_51_MASK)

#define CAN_WORD_DATA_BYTE_55_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_55_SHIFT              (0U)
/*! DATA_BYTE_55 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_55(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_55_SHIFT)) & CAN_WORD_DATA_BYTE_55_MASK)

#define CAN_WORD_DATA_BYTE_59_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_59_SHIFT              (0U)
/*! DATA_BYTE_59 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_59(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_59_SHIFT)) & CAN_WORD_DATA_BYTE_59_MASK)

#define CAN_WORD_DATA_BYTE_63_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_63_SHIFT              (0U)
/*! DATA_BYTE_63 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_63(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_63_SHIFT)) & CAN_WORD_DATA_BYTE_63_MASK)

#define CAN_WORD_DATA_BYTE_2_MASK                (0xFF00U)
#define CAN_WORD_DATA_BYTE_2_SHIFT               (8U)
/*! DATA_BYTE_2 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_2(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_2_SHIFT)) & CAN_WORD_DATA_BYTE_2_MASK)

#define CAN_WORD_DATA_BYTE_6_MASK                (0xFF00U)
#define CAN_WORD_DATA_BYTE_6_SHIFT               (8U)
/*! DATA_BYTE_6 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_6(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_6_SHIFT)) & CAN_WORD_DATA_BYTE_6_MASK)

#define CAN_WORD_DATA_BYTE_10_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_10_SHIFT              (8U)
/*! DATA_BYTE_10 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_10(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_10_SHIFT)) & CAN_WORD_DATA_BYTE_10_MASK)

#define CAN_WORD_DATA_BYTE_14_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_14_SHIFT              (8U)
/*! DATA_BYTE_14 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_14(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_14_SHIFT)) & CAN_WORD_DATA_BYTE_14_MASK)

#define CAN_WORD_DATA_BYTE_18_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_18_SHIFT              (8U)
/*! DATA_BYTE_18 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_18(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_18_SHIFT)) & CAN_WORD_DATA_BYTE_18_MASK)

#define CAN_WORD_DATA_BYTE_22_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_22_SHIFT              (8U)
/*! DATA_BYTE_22 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_22(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_22_SHIFT)) & CAN_WORD_DATA_BYTE_22_MASK)

#define CAN_WORD_DATA_BYTE_26_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_26_SHIFT              (8U)
/*! DATA_BYTE_26 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_26(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_26_SHIFT)) & CAN_WORD_DATA_BYTE_26_MASK)

#define CAN_WORD_DATA_BYTE_30_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_30_SHIFT              (8U)
/*! DATA_BYTE_30 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_30(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_30_SHIFT)) & CAN_WORD_DATA_BYTE_30_MASK)

#define CAN_WORD_DATA_BYTE_34_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_34_SHIFT              (8U)
/*! DATA_BYTE_34 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_34(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_34_SHIFT)) & CAN_WORD_DATA_BYTE_34_MASK)

#define CAN_WORD_DATA_BYTE_38_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_38_SHIFT              (8U)
/*! DATA_BYTE_38 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_38(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_38_SHIFT)) & CAN_WORD_DATA_BYTE_38_MASK)

#define CAN_WORD_DATA_BYTE_42_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_42_SHIFT              (8U)
/*! DATA_BYTE_42 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_42(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_42_SHIFT)) & CAN_WORD_DATA_BYTE_42_MASK)

#define CAN_WORD_DATA_BYTE_46_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_46_SHIFT              (8U)
/*! DATA_BYTE_46 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_46(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_46_SHIFT)) & CAN_WORD_DATA_BYTE_46_MASK)

#define CAN_WORD_DATA_BYTE_50_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_50_SHIFT              (8U)
/*! DATA_BYTE_50 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_50(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_50_SHIFT)) & CAN_WORD_DATA_BYTE_50_MASK)

#define CAN_WORD_DATA_BYTE_54_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_54_SHIFT              (8U)
/*! DATA_BYTE_54 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_54(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_54_SHIFT)) & CAN_WORD_DATA_BYTE_54_MASK)

#define CAN_WORD_DATA_BYTE_58_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_58_SHIFT              (8U)
/*! DATA_BYTE_58 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_58(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_58_SHIFT)) & CAN_WORD_DATA_BYTE_58_MASK)

#define CAN_WORD_DATA_BYTE_62_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_62_SHIFT              (8U)
/*! DATA_BYTE_62 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_62(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_62_SHIFT)) & CAN_WORD_DATA_BYTE_62_MASK)

#define CAN_WORD_DATA_BYTE_1_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_1_SHIFT               (16U)
/*! DATA_BYTE_1 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_1(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_1_SHIFT)) & CAN_WORD_DATA_BYTE_1_MASK)

#define CAN_WORD_DATA_BYTE_5_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_5_SHIFT               (16U)
/*! DATA_BYTE_5 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_5(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_5_SHIFT)) & CAN_WORD_DATA_BYTE_5_MASK)

#define CAN_WORD_DATA_BYTE_9_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_9_SHIFT               (16U)
/*! DATA_BYTE_9 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_9(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_9_SHIFT)) & CAN_WORD_DATA_BYTE_9_MASK)

#define CAN_WORD_DATA_BYTE_13_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_13_SHIFT              (16U)
/*! DATA_BYTE_13 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_13(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_13_SHIFT)) & CAN_WORD_DATA_BYTE_13_MASK)

#define CAN_WORD_DATA_BYTE_17_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_17_SHIFT              (16U)
/*! DATA_BYTE_17 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_17(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_17_SHIFT)) & CAN_WORD_DATA_BYTE_17_MASK)

#define CAN_WORD_DATA_BYTE_21_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_21_SHIFT              (16U)
/*! DATA_BYTE_21 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_21(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_21_SHIFT)) & CAN_WORD_DATA_BYTE_21_MASK)

#define CAN_WORD_DATA_BYTE_25_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_25_SHIFT              (16U)
/*! DATA_BYTE_25 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_25(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_25_SHIFT)) & CAN_WORD_DATA_BYTE_25_MASK)

#define CAN_WORD_DATA_BYTE_29_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_29_SHIFT              (16U)
/*! DATA_BYTE_29 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_29(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_29_SHIFT)) & CAN_WORD_DATA_BYTE_29_MASK)

#define CAN_WORD_DATA_BYTE_33_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_33_SHIFT              (16U)
/*! DATA_BYTE_33 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_33(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_33_SHIFT)) & CAN_WORD_DATA_BYTE_33_MASK)

#define CAN_WORD_DATA_BYTE_37_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_37_SHIFT              (16U)
/*! DATA_BYTE_37 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_37(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_37_SHIFT)) & CAN_WORD_DATA_BYTE_37_MASK)

#define CAN_WORD_DATA_BYTE_41_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_41_SHIFT              (16U)
/*! DATA_BYTE_41 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_41(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_41_SHIFT)) & CAN_WORD_DATA_BYTE_41_MASK)

#define CAN_WORD_DATA_BYTE_45_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_45_SHIFT              (16U)
/*! DATA_BYTE_45 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_45(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_45_SHIFT)) & CAN_WORD_DATA_BYTE_45_MASK)

#define CAN_WORD_DATA_BYTE_49_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_49_SHIFT              (16U)
/*! DATA_BYTE_49 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_49(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_49_SHIFT)) & CAN_WORD_DATA_BYTE_49_MASK)

#define CAN_WORD_DATA_BYTE_53_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_53_SHIFT              (16U)
/*! DATA_BYTE_53 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_53(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_53_SHIFT)) & CAN_WORD_DATA_BYTE_53_MASK)

#define CAN_WORD_DATA_BYTE_57_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_57_SHIFT              (16U)
/*! DATA_BYTE_57 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_57(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_57_SHIFT)) & CAN_WORD_DATA_BYTE_57_MASK)

#define CAN_WORD_DATA_BYTE_61_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_61_SHIFT              (16U)
/*! DATA_BYTE_61 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_61(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_61_SHIFT)) & CAN_WORD_DATA_BYTE_61_MASK)

#define CAN_WORD_DATA_BYTE_0_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_0_SHIFT               (24U)
/*! DATA_BYTE_0 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_0(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_0_SHIFT)) & CAN_WORD_DATA_BYTE_0_MASK)

#define CAN_WORD_DATA_BYTE_4_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_4_SHIFT               (24U)
/*! DATA_BYTE_4 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_4(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_4_SHIFT)) & CAN_WORD_DATA_BYTE_4_MASK)

#define CAN_WORD_DATA_BYTE_8_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_8_SHIFT               (24U)
/*! DATA_BYTE_8 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_8(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_8_SHIFT)) & CAN_WORD_DATA_BYTE_8_MASK)

#define CAN_WORD_DATA_BYTE_12_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_12_SHIFT              (24U)
/*! DATA_BYTE_12 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_12(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_12_SHIFT)) & CAN_WORD_DATA_BYTE_12_MASK)

#define CAN_WORD_DATA_BYTE_16_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_16_SHIFT              (24U)
/*! DATA_BYTE_16 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_16(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_16_SHIFT)) & CAN_WORD_DATA_BYTE_16_MASK)

#define CAN_WORD_DATA_BYTE_20_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_20_SHIFT              (24U)
/*! DATA_BYTE_20 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_20(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_20_SHIFT)) & CAN_WORD_DATA_BYTE_20_MASK)

#define CAN_WORD_DATA_BYTE_24_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_24_SHIFT              (24U)
/*! DATA_BYTE_24 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_24(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_24_SHIFT)) & CAN_WORD_DATA_BYTE_24_MASK)

#define CAN_WORD_DATA_BYTE_28_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_28_SHIFT              (24U)
/*! DATA_BYTE_28 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_28(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_28_SHIFT)) & CAN_WORD_DATA_BYTE_28_MASK)

#define CAN_WORD_DATA_BYTE_32_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_32_SHIFT              (24U)
/*! DATA_BYTE_32 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_32(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_32_SHIFT)) & CAN_WORD_DATA_BYTE_32_MASK)

#define CAN_WORD_DATA_BYTE_36_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_36_SHIFT              (24U)
/*! DATA_BYTE_36 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_36(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_36_SHIFT)) & CAN_WORD_DATA_BYTE_36_MASK)

#define CAN_WORD_DATA_BYTE_40_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_40_SHIFT              (24U)
/*! DATA_BYTE_40 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_40(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_40_SHIFT)) & CAN_WORD_DATA_BYTE_40_MASK)

#define CAN_WORD_DATA_BYTE_44_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_44_SHIFT              (24U)
/*! DATA_BYTE_44 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_44(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_44_SHIFT)) & CAN_WORD_DATA_BYTE_44_MASK)

#define CAN_WORD_DATA_BYTE_48_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_48_SHIFT              (24U)
/*! DATA_BYTE_48 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_48(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_48_SHIFT)) & CAN_WORD_DATA_BYTE_48_MASK)

#define CAN_WORD_DATA_BYTE_52_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_52_SHIFT              (24U)
/*! DATA_BYTE_52 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_52(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_52_SHIFT)) & CAN_WORD_DATA_BYTE_52_MASK)

#define CAN_WORD_DATA_BYTE_56_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_56_SHIFT              (24U)
/*! DATA_BYTE_56 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_56(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_56_SHIFT)) & CAN_WORD_DATA_BYTE_56_MASK)

#define CAN_WORD_DATA_BYTE_60_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_60_SHIFT              (24U)
/*! DATA_BYTE_60 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_60(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_60_SHIFT)) & CAN_WORD_DATA_BYTE_60_MASK)
/*! @} */

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB64B_M                   (7U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB64B_M2                  (16U)

/*! @name CS - Message Buffer 0 CS Register..Message Buffer 6 CS Register */
/*! @{ */

#define CAN_CS_TIME_STAMP_MASK                   (0xFFFFU)
#define CAN_CS_TIME_STAMP_SHIFT                  (0U)
/*! TIME_STAMP - Free-Running Counter Time stamp. This 16-bit field is a copy of the Free-Running
 *    Timer, captured for Tx and Rx frames at the time when the beginning of the Identifier field
 *    appears on the CAN bus.
 */
#define CAN_CS_TIME_STAMP(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_CS_TIME_STAMP_SHIFT)) & CAN_CS_TIME_STAMP_MASK)

#define CAN_CS_DLC_MASK                          (0xF0000U)
#define CAN_CS_DLC_SHIFT                         (16U)
/*! DLC - Length of the data to be stored/transmitted. */
#define CAN_CS_DLC(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_DLC_SHIFT)) & CAN_CS_DLC_MASK)

#define CAN_CS_RTR_MASK                          (0x100000U)
#define CAN_CS_RTR_SHIFT                         (20U)
/*! RTR - Remote Transmission Request. One/zero for remote/data frame. */
#define CAN_CS_RTR(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_RTR_SHIFT)) & CAN_CS_RTR_MASK)

#define CAN_CS_IDE_MASK                          (0x200000U)
#define CAN_CS_IDE_SHIFT                         (21U)
/*! IDE - ID Extended. One/zero for extended/standard format frame. */
#define CAN_CS_IDE(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_IDE_SHIFT)) & CAN_CS_IDE_MASK)

#define CAN_CS_SRR_MASK                          (0x400000U)
#define CAN_CS_SRR_SHIFT                         (22U)
/*! SRR - Substitute Remote Request. Contains a fixed recessive bit. */
#define CAN_CS_SRR(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_SRR_SHIFT)) & CAN_CS_SRR_MASK)

#define CAN_CS_CODE_MASK                         (0xF000000U)
#define CAN_CS_CODE_SHIFT                        (24U)
/*! CODE - Message Buffer Code. This 4-bit field can be accessed (read or write) by the CPU and by
 *    the FlexCAN module itself, as part of the message buffer matching and arbitration process.
 */
#define CAN_CS_CODE(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_CS_CODE_SHIFT)) & CAN_CS_CODE_MASK)

#define CAN_CS_ESI_MASK                          (0x20000000U)
#define CAN_CS_ESI_SHIFT                         (29U)
/*! ESI - Error State Indicator. This bit indicates if the transmitting node is error active or error passive. */
#define CAN_CS_ESI(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_ESI_SHIFT)) & CAN_CS_ESI_MASK)

#define CAN_CS_BRS_MASK                          (0x40000000U)
#define CAN_CS_BRS_SHIFT                         (30U)
/*! BRS - Bit Rate Switch. This bit defines whether the bit rate is switched inside a CAN FD format frame. */
#define CAN_CS_BRS(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_BRS_SHIFT)) & CAN_CS_BRS_MASK)

#define CAN_CS_EDL_MASK                          (0x80000000U)
#define CAN_CS_EDL_SHIFT                         (31U)
/*! EDL - Extended Data Length. This bit distinguishes between CAN format and CAN FD format frames.
 *    The EDL bit must not be set for Message Buffers configured to RANSWER with code field 0b1010.
 */
#define CAN_CS_EDL(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_CS_EDL_SHIFT)) & CAN_CS_EDL_MASK)
/*! @} */

/* The count of CAN_CS */
#define CAN_CS_COUNT_MB64B_H                     (7U)

/*! @name ID - Message Buffer 0 ID Register..Message Buffer 6 ID Register */
/*! @{ */

#define CAN_ID_EXT_MASK                          (0x3FFFFU)
#define CAN_ID_EXT_SHIFT                         (0U)
/*! EXT - Contains extended (LOW word) identifier of message buffer. */
#define CAN_ID_EXT(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_ID_EXT_SHIFT)) & CAN_ID_EXT_MASK)

#define CAN_ID_STD_MASK                          (0x1FFC0000U)
#define CAN_ID_STD_SHIFT                         (18U)
/*! STD - Contains standard/extended (HIGH word) identifier of message buffer. */
#define CAN_ID_STD(x)                            (((uint32_t)(((uint32_t)(x)) << CAN_ID_STD_SHIFT)) & CAN_ID_STD_MASK)

#define CAN_ID_PRIO_MASK                         (0xE0000000U)
#define CAN_ID_PRIO_SHIFT                        (29U)
/*! PRIO - Local priority. This 3-bit fieldis only used when LPRIO_EN bit is set in MCR and it only
 *    makes sense for Tx buffers. These bits are not transmitted. They are appended to the regular
 *    ID to define the transmission priority.
 */
#define CAN_ID_PRIO(x)                           (((uint32_t)(((uint32_t)(x)) << CAN_ID_PRIO_SHIFT)) & CAN_ID_PRIO_MASK)
/*! @} */

/* The count of CAN_ID */
#define CAN_ID_COUNT_MB64B_H                     (7U)

/*! @name WORD - Message Buffer 0 WORD_64B Register..Message Buffer 6 WORD_64B Register */
/*! @{ */

#define CAN_WORD_DATA_BYTE_3_MASK                (0xFFU)
#define CAN_WORD_DATA_BYTE_3_SHIFT               (0U)
/*! DATA_BYTE_3 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_3(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_3_SHIFT)) & CAN_WORD_DATA_BYTE_3_MASK)

#define CAN_WORD_DATA_BYTE_7_MASK                (0xFFU)
#define CAN_WORD_DATA_BYTE_7_SHIFT               (0U)
/*! DATA_BYTE_7 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_7(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_7_SHIFT)) & CAN_WORD_DATA_BYTE_7_MASK)

#define CAN_WORD_DATA_BYTE_11_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_11_SHIFT              (0U)
/*! DATA_BYTE_11 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_11(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_11_SHIFT)) & CAN_WORD_DATA_BYTE_11_MASK)

#define CAN_WORD_DATA_BYTE_15_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_15_SHIFT              (0U)
/*! DATA_BYTE_15 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_15(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_15_SHIFT)) & CAN_WORD_DATA_BYTE_15_MASK)

#define CAN_WORD_DATA_BYTE_19_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_19_SHIFT              (0U)
/*! DATA_BYTE_19 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_19(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_19_SHIFT)) & CAN_WORD_DATA_BYTE_19_MASK)

#define CAN_WORD_DATA_BYTE_23_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_23_SHIFT              (0U)
/*! DATA_BYTE_23 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_23(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_23_SHIFT)) & CAN_WORD_DATA_BYTE_23_MASK)

#define CAN_WORD_DATA_BYTE_27_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_27_SHIFT              (0U)
/*! DATA_BYTE_27 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_27(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_27_SHIFT)) & CAN_WORD_DATA_BYTE_27_MASK)

#define CAN_WORD_DATA_BYTE_31_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_31_SHIFT              (0U)
/*! DATA_BYTE_31 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_31(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_31_SHIFT)) & CAN_WORD_DATA_BYTE_31_MASK)

#define CAN_WORD_DATA_BYTE_35_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_35_SHIFT              (0U)
/*! DATA_BYTE_35 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_35(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_35_SHIFT)) & CAN_WORD_DATA_BYTE_35_MASK)

#define CAN_WORD_DATA_BYTE_39_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_39_SHIFT              (0U)
/*! DATA_BYTE_39 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_39(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_39_SHIFT)) & CAN_WORD_DATA_BYTE_39_MASK)

#define CAN_WORD_DATA_BYTE_43_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_43_SHIFT              (0U)
/*! DATA_BYTE_43 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_43(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_43_SHIFT)) & CAN_WORD_DATA_BYTE_43_MASK)

#define CAN_WORD_DATA_BYTE_47_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_47_SHIFT              (0U)
/*! DATA_BYTE_47 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_47(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_47_SHIFT)) & CAN_WORD_DATA_BYTE_47_MASK)

#define CAN_WORD_DATA_BYTE_51_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_51_SHIFT              (0U)
/*! DATA_BYTE_51 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_51(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_51_SHIFT)) & CAN_WORD_DATA_BYTE_51_MASK)

#define CAN_WORD_DATA_BYTE_55_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_55_SHIFT              (0U)
/*! DATA_BYTE_55 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_55(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_55_SHIFT)) & CAN_WORD_DATA_BYTE_55_MASK)

#define CAN_WORD_DATA_BYTE_59_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_59_SHIFT              (0U)
/*! DATA_BYTE_59 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_59(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_59_SHIFT)) & CAN_WORD_DATA_BYTE_59_MASK)

#define CAN_WORD_DATA_BYTE_63_MASK               (0xFFU)
#define CAN_WORD_DATA_BYTE_63_SHIFT              (0U)
/*! DATA_BYTE_63 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_63(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_63_SHIFT)) & CAN_WORD_DATA_BYTE_63_MASK)

#define CAN_WORD_DATA_BYTE_2_MASK                (0xFF00U)
#define CAN_WORD_DATA_BYTE_2_SHIFT               (8U)
/*! DATA_BYTE_2 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_2(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_2_SHIFT)) & CAN_WORD_DATA_BYTE_2_MASK)

#define CAN_WORD_DATA_BYTE_6_MASK                (0xFF00U)
#define CAN_WORD_DATA_BYTE_6_SHIFT               (8U)
/*! DATA_BYTE_6 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_6(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_6_SHIFT)) & CAN_WORD_DATA_BYTE_6_MASK)

#define CAN_WORD_DATA_BYTE_10_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_10_SHIFT              (8U)
/*! DATA_BYTE_10 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_10(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_10_SHIFT)) & CAN_WORD_DATA_BYTE_10_MASK)

#define CAN_WORD_DATA_BYTE_14_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_14_SHIFT              (8U)
/*! DATA_BYTE_14 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_14(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_14_SHIFT)) & CAN_WORD_DATA_BYTE_14_MASK)

#define CAN_WORD_DATA_BYTE_18_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_18_SHIFT              (8U)
/*! DATA_BYTE_18 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_18(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_18_SHIFT)) & CAN_WORD_DATA_BYTE_18_MASK)

#define CAN_WORD_DATA_BYTE_22_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_22_SHIFT              (8U)
/*! DATA_BYTE_22 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_22(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_22_SHIFT)) & CAN_WORD_DATA_BYTE_22_MASK)

#define CAN_WORD_DATA_BYTE_26_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_26_SHIFT              (8U)
/*! DATA_BYTE_26 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_26(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_26_SHIFT)) & CAN_WORD_DATA_BYTE_26_MASK)

#define CAN_WORD_DATA_BYTE_30_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_30_SHIFT              (8U)
/*! DATA_BYTE_30 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_30(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_30_SHIFT)) & CAN_WORD_DATA_BYTE_30_MASK)

#define CAN_WORD_DATA_BYTE_34_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_34_SHIFT              (8U)
/*! DATA_BYTE_34 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_34(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_34_SHIFT)) & CAN_WORD_DATA_BYTE_34_MASK)

#define CAN_WORD_DATA_BYTE_38_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_38_SHIFT              (8U)
/*! DATA_BYTE_38 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_38(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_38_SHIFT)) & CAN_WORD_DATA_BYTE_38_MASK)

#define CAN_WORD_DATA_BYTE_42_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_42_SHIFT              (8U)
/*! DATA_BYTE_42 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_42(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_42_SHIFT)) & CAN_WORD_DATA_BYTE_42_MASK)

#define CAN_WORD_DATA_BYTE_46_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_46_SHIFT              (8U)
/*! DATA_BYTE_46 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_46(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_46_SHIFT)) & CAN_WORD_DATA_BYTE_46_MASK)

#define CAN_WORD_DATA_BYTE_50_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_50_SHIFT              (8U)
/*! DATA_BYTE_50 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_50(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_50_SHIFT)) & CAN_WORD_DATA_BYTE_50_MASK)

#define CAN_WORD_DATA_BYTE_54_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_54_SHIFT              (8U)
/*! DATA_BYTE_54 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_54(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_54_SHIFT)) & CAN_WORD_DATA_BYTE_54_MASK)

#define CAN_WORD_DATA_BYTE_58_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_58_SHIFT              (8U)
/*! DATA_BYTE_58 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_58(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_58_SHIFT)) & CAN_WORD_DATA_BYTE_58_MASK)

#define CAN_WORD_DATA_BYTE_62_MASK               (0xFF00U)
#define CAN_WORD_DATA_BYTE_62_SHIFT              (8U)
/*! DATA_BYTE_62 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_62(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_62_SHIFT)) & CAN_WORD_DATA_BYTE_62_MASK)

#define CAN_WORD_DATA_BYTE_1_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_1_SHIFT               (16U)
/*! DATA_BYTE_1 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_1(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_1_SHIFT)) & CAN_WORD_DATA_BYTE_1_MASK)

#define CAN_WORD_DATA_BYTE_5_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_5_SHIFT               (16U)
/*! DATA_BYTE_5 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_5(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_5_SHIFT)) & CAN_WORD_DATA_BYTE_5_MASK)

#define CAN_WORD_DATA_BYTE_9_MASK                (0xFF0000U)
#define CAN_WORD_DATA_BYTE_9_SHIFT               (16U)
/*! DATA_BYTE_9 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_9(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_9_SHIFT)) & CAN_WORD_DATA_BYTE_9_MASK)

#define CAN_WORD_DATA_BYTE_13_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_13_SHIFT              (16U)
/*! DATA_BYTE_13 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_13(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_13_SHIFT)) & CAN_WORD_DATA_BYTE_13_MASK)

#define CAN_WORD_DATA_BYTE_17_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_17_SHIFT              (16U)
/*! DATA_BYTE_17 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_17(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_17_SHIFT)) & CAN_WORD_DATA_BYTE_17_MASK)

#define CAN_WORD_DATA_BYTE_21_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_21_SHIFT              (16U)
/*! DATA_BYTE_21 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_21(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_21_SHIFT)) & CAN_WORD_DATA_BYTE_21_MASK)

#define CAN_WORD_DATA_BYTE_25_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_25_SHIFT              (16U)
/*! DATA_BYTE_25 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_25(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_25_SHIFT)) & CAN_WORD_DATA_BYTE_25_MASK)

#define CAN_WORD_DATA_BYTE_29_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_29_SHIFT              (16U)
/*! DATA_BYTE_29 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_29(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_29_SHIFT)) & CAN_WORD_DATA_BYTE_29_MASK)

#define CAN_WORD_DATA_BYTE_33_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_33_SHIFT              (16U)
/*! DATA_BYTE_33 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_33(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_33_SHIFT)) & CAN_WORD_DATA_BYTE_33_MASK)

#define CAN_WORD_DATA_BYTE_37_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_37_SHIFT              (16U)
/*! DATA_BYTE_37 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_37(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_37_SHIFT)) & CAN_WORD_DATA_BYTE_37_MASK)

#define CAN_WORD_DATA_BYTE_41_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_41_SHIFT              (16U)
/*! DATA_BYTE_41 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_41(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_41_SHIFT)) & CAN_WORD_DATA_BYTE_41_MASK)

#define CAN_WORD_DATA_BYTE_45_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_45_SHIFT              (16U)
/*! DATA_BYTE_45 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_45(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_45_SHIFT)) & CAN_WORD_DATA_BYTE_45_MASK)

#define CAN_WORD_DATA_BYTE_49_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_49_SHIFT              (16U)
/*! DATA_BYTE_49 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_49(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_49_SHIFT)) & CAN_WORD_DATA_BYTE_49_MASK)

#define CAN_WORD_DATA_BYTE_53_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_53_SHIFT              (16U)
/*! DATA_BYTE_53 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_53(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_53_SHIFT)) & CAN_WORD_DATA_BYTE_53_MASK)

#define CAN_WORD_DATA_BYTE_57_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_57_SHIFT              (16U)
/*! DATA_BYTE_57 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_57(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_57_SHIFT)) & CAN_WORD_DATA_BYTE_57_MASK)

#define CAN_WORD_DATA_BYTE_61_MASK               (0xFF0000U)
#define CAN_WORD_DATA_BYTE_61_SHIFT              (16U)
/*! DATA_BYTE_61 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_61(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_61_SHIFT)) & CAN_WORD_DATA_BYTE_61_MASK)

#define CAN_WORD_DATA_BYTE_0_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_0_SHIFT               (24U)
/*! DATA_BYTE_0 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_0(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_0_SHIFT)) & CAN_WORD_DATA_BYTE_0_MASK)

#define CAN_WORD_DATA_BYTE_4_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_4_SHIFT               (24U)
/*! DATA_BYTE_4 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_4(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_4_SHIFT)) & CAN_WORD_DATA_BYTE_4_MASK)

#define CAN_WORD_DATA_BYTE_8_MASK                (0xFF000000U)
#define CAN_WORD_DATA_BYTE_8_SHIFT               (24U)
/*! DATA_BYTE_8 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_8(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_8_SHIFT)) & CAN_WORD_DATA_BYTE_8_MASK)

#define CAN_WORD_DATA_BYTE_12_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_12_SHIFT              (24U)
/*! DATA_BYTE_12 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_12(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_12_SHIFT)) & CAN_WORD_DATA_BYTE_12_MASK)

#define CAN_WORD_DATA_BYTE_16_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_16_SHIFT              (24U)
/*! DATA_BYTE_16 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_16(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_16_SHIFT)) & CAN_WORD_DATA_BYTE_16_MASK)

#define CAN_WORD_DATA_BYTE_20_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_20_SHIFT              (24U)
/*! DATA_BYTE_20 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_20(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_20_SHIFT)) & CAN_WORD_DATA_BYTE_20_MASK)

#define CAN_WORD_DATA_BYTE_24_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_24_SHIFT              (24U)
/*! DATA_BYTE_24 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_24(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_24_SHIFT)) & CAN_WORD_DATA_BYTE_24_MASK)

#define CAN_WORD_DATA_BYTE_28_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_28_SHIFT              (24U)
/*! DATA_BYTE_28 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_28(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_28_SHIFT)) & CAN_WORD_DATA_BYTE_28_MASK)

#define CAN_WORD_DATA_BYTE_32_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_32_SHIFT              (24U)
/*! DATA_BYTE_32 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_32(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_32_SHIFT)) & CAN_WORD_DATA_BYTE_32_MASK)

#define CAN_WORD_DATA_BYTE_36_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_36_SHIFT              (24U)
/*! DATA_BYTE_36 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_36(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_36_SHIFT)) & CAN_WORD_DATA_BYTE_36_MASK)

#define CAN_WORD_DATA_BYTE_40_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_40_SHIFT              (24U)
/*! DATA_BYTE_40 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_40(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_40_SHIFT)) & CAN_WORD_DATA_BYTE_40_MASK)

#define CAN_WORD_DATA_BYTE_44_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_44_SHIFT              (24U)
/*! DATA_BYTE_44 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_44(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_44_SHIFT)) & CAN_WORD_DATA_BYTE_44_MASK)

#define CAN_WORD_DATA_BYTE_48_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_48_SHIFT              (24U)
/*! DATA_BYTE_48 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_48(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_48_SHIFT)) & CAN_WORD_DATA_BYTE_48_MASK)

#define CAN_WORD_DATA_BYTE_52_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_52_SHIFT              (24U)
/*! DATA_BYTE_52 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_52(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_52_SHIFT)) & CAN_WORD_DATA_BYTE_52_MASK)

#define CAN_WORD_DATA_BYTE_56_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_56_SHIFT              (24U)
/*! DATA_BYTE_56 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_56(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_56_SHIFT)) & CAN_WORD_DATA_BYTE_56_MASK)

#define CAN_WORD_DATA_BYTE_60_MASK               (0xFF000000U)
#define CAN_WORD_DATA_BYTE_60_SHIFT              (24U)
/*! DATA_BYTE_60 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD_DATA_BYTE_60(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD_DATA_BYTE_60_SHIFT)) & CAN_WORD_DATA_BYTE_60_MASK)
/*! @} */

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB64B_H                   (7U)

/* The count of CAN_WORD */
#define CAN_WORD_COUNT_MB64B_H2                  (16U)

/* The count of CAN_CS */
#define CAN_CS_COUNT                             (96U)

/* The count of CAN_ID */
#define CAN_ID_COUNT                             (96U)

/*! @name WORD0 - Message Buffer 0 WORD0 Register..Message Buffer 95 WORD0 Register */
/*! @{ */

#define CAN_WORD0_DATA_BYTE_3_MASK               (0xFFU)
#define CAN_WORD0_DATA_BYTE_3_SHIFT              (0U)
/*! DATA_BYTE_3 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD0_DATA_BYTE_3(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD0_DATA_BYTE_3_SHIFT)) & CAN_WORD0_DATA_BYTE_3_MASK)

#define CAN_WORD0_DATA_BYTE_2_MASK               (0xFF00U)
#define CAN_WORD0_DATA_BYTE_2_SHIFT              (8U)
/*! DATA_BYTE_2 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD0_DATA_BYTE_2(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD0_DATA_BYTE_2_SHIFT)) & CAN_WORD0_DATA_BYTE_2_MASK)

#define CAN_WORD0_DATA_BYTE_1_MASK               (0xFF0000U)
#define CAN_WORD0_DATA_BYTE_1_SHIFT              (16U)
/*! DATA_BYTE_1 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD0_DATA_BYTE_1(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD0_DATA_BYTE_1_SHIFT)) & CAN_WORD0_DATA_BYTE_1_MASK)

#define CAN_WORD0_DATA_BYTE_0_MASK               (0xFF000000U)
#define CAN_WORD0_DATA_BYTE_0_SHIFT              (24U)
/*! DATA_BYTE_0 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD0_DATA_BYTE_0(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD0_DATA_BYTE_0_SHIFT)) & CAN_WORD0_DATA_BYTE_0_MASK)
/*! @} */

/* The count of CAN_WORD0 */
#define CAN_WORD0_COUNT                          (96U)

/*! @name WORD1 - Message Buffer 0 WORD1 Register..Message Buffer 95 WORD1 Register */
/*! @{ */

#define CAN_WORD1_DATA_BYTE_7_MASK               (0xFFU)
#define CAN_WORD1_DATA_BYTE_7_SHIFT              (0U)
/*! DATA_BYTE_7 - Data byte 0 of Rx/Tx frame. */
#define CAN_WORD1_DATA_BYTE_7(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD1_DATA_BYTE_7_SHIFT)) & CAN_WORD1_DATA_BYTE_7_MASK)

#define CAN_WORD1_DATA_BYTE_6_MASK               (0xFF00U)
#define CAN_WORD1_DATA_BYTE_6_SHIFT              (8U)
/*! DATA_BYTE_6 - Data byte 1 of Rx/Tx frame. */
#define CAN_WORD1_DATA_BYTE_6(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD1_DATA_BYTE_6_SHIFT)) & CAN_WORD1_DATA_BYTE_6_MASK)

#define CAN_WORD1_DATA_BYTE_5_MASK               (0xFF0000U)
#define CAN_WORD1_DATA_BYTE_5_SHIFT              (16U)
/*! DATA_BYTE_5 - Data byte 2 of Rx/Tx frame. */
#define CAN_WORD1_DATA_BYTE_5(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD1_DATA_BYTE_5_SHIFT)) & CAN_WORD1_DATA_BYTE_5_MASK)

#define CAN_WORD1_DATA_BYTE_4_MASK               (0xFF000000U)
#define CAN_WORD1_DATA_BYTE_4_SHIFT              (24U)
/*! DATA_BYTE_4 - Data byte 3 of Rx/Tx frame. */
#define CAN_WORD1_DATA_BYTE_4(x)                 (((uint32_t)(((uint32_t)(x)) << CAN_WORD1_DATA_BYTE_4_SHIFT)) & CAN_WORD1_DATA_BYTE_4_MASK)
/*! @} */

/* The count of CAN_WORD1 */
#define CAN_WORD1_COUNT                          (96U)

/*! @name RXIMR - Receive Individual Mask */
/*! @{ */

#define CAN_RXIMR_MI_MASK                        (0xFFFFFFFFU)
#define CAN_RXIMR_MI_SHIFT                       (0U)
/*! MI - Individual Mask Bits */
#define CAN_RXIMR_MI(x)                          (((uint32_t)(((uint32_t)(x)) << CAN_RXIMR_MI_SHIFT)) & CAN_RXIMR_MI_MASK)
/*! @} */

/* The count of CAN_RXIMR */
#define CAN_RXIMR_COUNT                          (96U)

/*! @name MECR - Memory Error Control */
/*! @{ */

#define CAN_MECR_NCEFAFRZ_MASK                   (0x80U)
#define CAN_MECR_NCEFAFRZ_SHIFT                  (7U)
/*! NCEFAFRZ - Noncorrectable Errors in FlexCAN Access Put Chip in Freeze Mode
 *  0b0..Normal operation
 *  0b1..Freeze mode
 */
#define CAN_MECR_NCEFAFRZ(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_MECR_NCEFAFRZ_SHIFT)) & CAN_MECR_NCEFAFRZ_MASK)

#define CAN_MECR_ECCDIS_MASK                     (0x100U)
#define CAN_MECR_ECCDIS_SHIFT                    (8U)
/*! ECCDIS - Error Correction Disable
 *  0b0..Enable
 *  0b1..Disable
 */
#define CAN_MECR_ECCDIS(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_MECR_ECCDIS_SHIFT)) & CAN_MECR_ECCDIS_MASK)

#define CAN_MECR_RERRDIS_MASK                    (0x200U)
#define CAN_MECR_RERRDIS_SHIFT                   (9U)
/*! RERRDIS - Error Report Disable
 *  0b0..Enable
 *  0b1..Disable
 */
#define CAN_MECR_RERRDIS(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_MECR_RERRDIS_SHIFT)) & CAN_MECR_RERRDIS_MASK)

#define CAN_MECR_EXTERRIE_MASK                   (0x2000U)
#define CAN_MECR_EXTERRIE_SHIFT                  (13U)
/*! EXTERRIE - Extended Error Injection Enable
 *  0b0..Disable. Apply error injection only to the 32-bit word.
 *  0b1..Enable. Apply error injection to the 64-bit word.
 */
#define CAN_MECR_EXTERRIE(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_MECR_EXTERRIE_SHIFT)) & CAN_MECR_EXTERRIE_MASK)

#define CAN_MECR_FAERRIE_MASK                    (0x4000U)
#define CAN_MECR_FAERRIE_SHIFT                   (14U)
/*! FAERRIE - FlexCAN Access Error Injection Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MECR_FAERRIE(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_MECR_FAERRIE_SHIFT)) & CAN_MECR_FAERRIE_MASK)

#define CAN_MECR_HAERRIE_MASK                    (0x8000U)
#define CAN_MECR_HAERRIE_SHIFT                   (15U)
/*! HAERRIE - Host Access Error Injection Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MECR_HAERRIE(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_MECR_HAERRIE_SHIFT)) & CAN_MECR_HAERRIE_MASK)

#define CAN_MECR_CEI_MSK_MASK                    (0x10000U)
#define CAN_MECR_CEI_MSK_SHIFT                   (16U)
/*! CEI_MSK - Correctable Errors Interrupt Mask
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MECR_CEI_MSK(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_MECR_CEI_MSK_SHIFT)) & CAN_MECR_CEI_MSK_MASK)

#define CAN_MECR_FANCEI_MSK_MASK                 (0x40000U)
#define CAN_MECR_FANCEI_MSK_SHIFT                (18U)
/*! FANCEI_MSK - FlexCAN Access with Noncorrectable Errors Interrupt Mask
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MECR_FANCEI_MSK(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_MECR_FANCEI_MSK_SHIFT)) & CAN_MECR_FANCEI_MSK_MASK)

#define CAN_MECR_HANCEI_MSK_MASK                 (0x80000U)
#define CAN_MECR_HANCEI_MSK_SHIFT                (19U)
/*! HANCEI_MSK - Host Access with Noncorrectable Errors Interrupt Mask
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_MECR_HANCEI_MSK(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_MECR_HANCEI_MSK_SHIFT)) & CAN_MECR_HANCEI_MSK_MASK)

#define CAN_MECR_ECRWRDIS_MASK                   (0x80000000U)
#define CAN_MECR_ECRWRDIS_SHIFT                  (31U)
/*! ECRWRDIS - Error Configuration Register Write Disable
 *  0b0..Enable
 *  0b1..Disable
 */
#define CAN_MECR_ECRWRDIS(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_MECR_ECRWRDIS_SHIFT)) & CAN_MECR_ECRWRDIS_MASK)
/*! @} */

/*! @name ERRIAR - Error Injection Address */
/*! @{ */

#define CAN_ERRIAR_INJADDR_L_MASK                (0x3U)
#define CAN_ERRIAR_INJADDR_L_SHIFT               (0U)
/*! INJADDR_L - Error Injection Address Low */
#define CAN_ERRIAR_INJADDR_L(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_ERRIAR_INJADDR_L_SHIFT)) & CAN_ERRIAR_INJADDR_L_MASK)

#define CAN_ERRIAR_INJADDR_H_MASK                (0x3FFCU)
#define CAN_ERRIAR_INJADDR_H_SHIFT               (2U)
/*! INJADDR_H - Error Injection Address High */
#define CAN_ERRIAR_INJADDR_H(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_ERRIAR_INJADDR_H_SHIFT)) & CAN_ERRIAR_INJADDR_H_MASK)
/*! @} */

/*! @name ERRIDPR - Error Injection Data Pattern */
/*! @{ */

#define CAN_ERRIDPR_DFLIP_MASK                   (0xFFFFFFFFU)
#define CAN_ERRIDPR_DFLIP_SHIFT                  (0U)
/*! DFLIP - Data Flip Pattern */
#define CAN_ERRIDPR_DFLIP(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_ERRIDPR_DFLIP_SHIFT)) & CAN_ERRIDPR_DFLIP_MASK)
/*! @} */

/*! @name ERRIPPR - Error Injection Parity Pattern */
/*! @{ */

#define CAN_ERRIPPR_PFLIP0_MASK                  (0x1FU)
#define CAN_ERRIPPR_PFLIP0_SHIFT                 (0U)
/*! PFLIP0 - Parity Flip Pattern for Byte 0 (Least Significant) */
#define CAN_ERRIPPR_PFLIP0(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_ERRIPPR_PFLIP0_SHIFT)) & CAN_ERRIPPR_PFLIP0_MASK)

#define CAN_ERRIPPR_PFLIP1_MASK                  (0x1F00U)
#define CAN_ERRIPPR_PFLIP1_SHIFT                 (8U)
/*! PFLIP1 - Parity Flip Pattern for Byte 1 */
#define CAN_ERRIPPR_PFLIP1(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_ERRIPPR_PFLIP1_SHIFT)) & CAN_ERRIPPR_PFLIP1_MASK)

#define CAN_ERRIPPR_PFLIP2_MASK                  (0x1F0000U)
#define CAN_ERRIPPR_PFLIP2_SHIFT                 (16U)
/*! PFLIP2 - Parity Flip Pattern for Byte 2 */
#define CAN_ERRIPPR_PFLIP2(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_ERRIPPR_PFLIP2_SHIFT)) & CAN_ERRIPPR_PFLIP2_MASK)

#define CAN_ERRIPPR_PFLIP3_MASK                  (0x1F000000U)
#define CAN_ERRIPPR_PFLIP3_SHIFT                 (24U)
/*! PFLIP3 - Parity Flip Pattern for Byte 3 (Most Significant) */
#define CAN_ERRIPPR_PFLIP3(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_ERRIPPR_PFLIP3_SHIFT)) & CAN_ERRIPPR_PFLIP3_MASK)
/*! @} */

/*! @name RERRAR - Error Report Address */
/*! @{ */

#define CAN_RERRAR_ERRADDR_MASK                  (0x3FFFU)
#define CAN_RERRAR_ERRADDR_SHIFT                 (0U)
/*! ERRADDR - Address Where Error Detected */
#define CAN_RERRAR_ERRADDR(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_RERRAR_ERRADDR_SHIFT)) & CAN_RERRAR_ERRADDR_MASK)

#define CAN_RERRAR_SAID_MASK                     (0x70000U)
#define CAN_RERRAR_SAID_SHIFT                    (16U)
/*! SAID - SAID */
#define CAN_RERRAR_SAID(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_RERRAR_SAID_SHIFT)) & CAN_RERRAR_SAID_MASK)

#define CAN_RERRAR_NCE_MASK                      (0x1000000U)
#define CAN_RERRAR_NCE_SHIFT                     (24U)
/*! NCE - Noncorrectable Error
 *  0b0..Reporting a correctable error
 *  0b1..Reporting a noncorrectable error
 */
#define CAN_RERRAR_NCE(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_RERRAR_NCE_SHIFT)) & CAN_RERRAR_NCE_MASK)
/*! @} */

/*! @name RERRDR - Error Report Data */
/*! @{ */

#define CAN_RERRDR_RDATA_MASK                    (0xFFFFFFFFU)
#define CAN_RERRDR_RDATA_SHIFT                   (0U)
/*! RDATA - Raw Data Word Read from Memory with Error */
#define CAN_RERRDR_RDATA(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_RERRDR_RDATA_SHIFT)) & CAN_RERRDR_RDATA_MASK)
/*! @} */

/*! @name RERRSYNR - Error Report Syndrome */
/*! @{ */

#define CAN_RERRSYNR_SYND0_MASK                  (0x1FU)
#define CAN_RERRSYNR_SYND0_SHIFT                 (0U)
/*! SYND0 - Error Syndrome for Byte 0 (Least Significant) */
#define CAN_RERRSYNR_SYND0(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_SYND0_SHIFT)) & CAN_RERRSYNR_SYND0_MASK)

#define CAN_RERRSYNR_BE0_MASK                    (0x80U)
#define CAN_RERRSYNR_BE0_SHIFT                   (7U)
/*! BE0 - Byte Enabled for Byte 0 (Least Significant)
 *  0b0..Byte was not read.
 *  0b1..Byte was read.
 */
#define CAN_RERRSYNR_BE0(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_BE0_SHIFT)) & CAN_RERRSYNR_BE0_MASK)

#define CAN_RERRSYNR_SYND1_MASK                  (0x1F00U)
#define CAN_RERRSYNR_SYND1_SHIFT                 (8U)
/*! SYND1 - Error Syndrome for Byte 1 */
#define CAN_RERRSYNR_SYND1(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_SYND1_SHIFT)) & CAN_RERRSYNR_SYND1_MASK)

#define CAN_RERRSYNR_BE1_MASK                    (0x8000U)
#define CAN_RERRSYNR_BE1_SHIFT                   (15U)
/*! BE1 - Byte Enabled for Byte 1
 *  0b0..Byte was not read.
 *  0b1..Byte was read.
 */
#define CAN_RERRSYNR_BE1(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_BE1_SHIFT)) & CAN_RERRSYNR_BE1_MASK)

#define CAN_RERRSYNR_SYND2_MASK                  (0x1F0000U)
#define CAN_RERRSYNR_SYND2_SHIFT                 (16U)
/*! SYND2 - Error Syndrome for Byte 2 */
#define CAN_RERRSYNR_SYND2(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_SYND2_SHIFT)) & CAN_RERRSYNR_SYND2_MASK)

#define CAN_RERRSYNR_BE2_MASK                    (0x800000U)
#define CAN_RERRSYNR_BE2_SHIFT                   (23U)
/*! BE2 - Byte Enabled for Byte 2
 *  0b0..Byte was not read.
 *  0b1..Byte was read.
 */
#define CAN_RERRSYNR_BE2(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_BE2_SHIFT)) & CAN_RERRSYNR_BE2_MASK)

#define CAN_RERRSYNR_SYND3_MASK                  (0x1F000000U)
#define CAN_RERRSYNR_SYND3_SHIFT                 (24U)
/*! SYND3 - Error Syndrome for Byte 3 (Most Significant) */
#define CAN_RERRSYNR_SYND3(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_SYND3_SHIFT)) & CAN_RERRSYNR_SYND3_MASK)

#define CAN_RERRSYNR_BE3_MASK                    (0x80000000U)
#define CAN_RERRSYNR_BE3_SHIFT                   (31U)
/*! BE3 - Byte Enabled for Byte 3 (Most Significant)
 *  0b0..Byte was not read.
 *  0b1..Byte was read.
 */
#define CAN_RERRSYNR_BE3(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_RERRSYNR_BE3_SHIFT)) & CAN_RERRSYNR_BE3_MASK)
/*! @} */

/*! @name ERRSR - Error Status */
/*! @{ */

#define CAN_ERRSR_CEIOF_MASK                     (0x1U)
#define CAN_ERRSR_CEIOF_SHIFT                    (0U)
/*! CEIOF - Correctable Error Interrupt Overrun Flag
 *  0b0..No errors detected
 *  0b1..Error detected
 */
#define CAN_ERRSR_CEIOF(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ERRSR_CEIOF_SHIFT)) & CAN_ERRSR_CEIOF_MASK)

#define CAN_ERRSR_FANCEIOF_MASK                  (0x4U)
#define CAN_ERRSR_FANCEIOF_SHIFT                 (2U)
/*! FANCEIOF - FlexCAN Access with Noncorrectable Error Interrupt Overrun Flag
 *  0b0..No errors detected
 *  0b1..Error detected
 */
#define CAN_ERRSR_FANCEIOF(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_ERRSR_FANCEIOF_SHIFT)) & CAN_ERRSR_FANCEIOF_MASK)

#define CAN_ERRSR_HANCEIOF_MASK                  (0x8U)
#define CAN_ERRSR_HANCEIOF_SHIFT                 (3U)
/*! HANCEIOF - Host Access With Noncorrectable Error Interrupt Overrun Flag
 *  0b0..No errors detected
 *  0b1..Error detected
 */
#define CAN_ERRSR_HANCEIOF(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_ERRSR_HANCEIOF_SHIFT)) & CAN_ERRSR_HANCEIOF_MASK)

#define CAN_ERRSR_CEIF_MASK                      (0x10000U)
#define CAN_ERRSR_CEIF_SHIFT                     (16U)
/*! CEIF - Correctable Error Interrupt Flag
 *  0b0..No errors detected
 *  0b1..Error detected
 */
#define CAN_ERRSR_CEIF(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ERRSR_CEIF_SHIFT)) & CAN_ERRSR_CEIF_MASK)

#define CAN_ERRSR_FANCEIF_MASK                   (0x40000U)
#define CAN_ERRSR_FANCEIF_SHIFT                  (18U)
/*! FANCEIF - FlexCAN Access with Noncorrectable Error Interrupt Flag
 *  0b0..No errors detected
 *  0b1..Error detected
 */
#define CAN_ERRSR_FANCEIF(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_ERRSR_FANCEIF_SHIFT)) & CAN_ERRSR_FANCEIF_MASK)

#define CAN_ERRSR_HANCEIF_MASK                   (0x80000U)
#define CAN_ERRSR_HANCEIF_SHIFT                  (19U)
/*! HANCEIF - Host Access with Noncorrectable Error Interrupt Flag
 *  0b0..No errors detected
 *  0b1..Error detected
 */
#define CAN_ERRSR_HANCEIF(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_ERRSR_HANCEIF_SHIFT)) & CAN_ERRSR_HANCEIF_MASK)
/*! @} */

/*! @name EPRS - Enhanced CAN Bit Timing Prescalers */
/*! @{ */

#define CAN_EPRS_ENPRESDIV_MASK                  (0x3FFU)
#define CAN_EPRS_ENPRESDIV_SHIFT                 (0U)
/*! ENPRESDIV - Extended Nominal Prescaler Division Factor */
#define CAN_EPRS_ENPRESDIV(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_EPRS_ENPRESDIV_SHIFT)) & CAN_EPRS_ENPRESDIV_MASK)

#define CAN_EPRS_EDPRESDIV_MASK                  (0x3FF0000U)
#define CAN_EPRS_EDPRESDIV_SHIFT                 (16U)
/*! EDPRESDIV - Extended Data Phase Prescaler Division Factor */
#define CAN_EPRS_EDPRESDIV(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_EPRS_EDPRESDIV_SHIFT)) & CAN_EPRS_EDPRESDIV_MASK)
/*! @} */

/*! @name ENCBT - Enhanced Nominal CAN Bit Timing */
/*! @{ */

#define CAN_ENCBT_NTSEG1_MASK                    (0xFFU)
#define CAN_ENCBT_NTSEG1_SHIFT                   (0U)
/*! NTSEG1 - Nominal Time Segment 1 */
#define CAN_ENCBT_NTSEG1(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ENCBT_NTSEG1_SHIFT)) & CAN_ENCBT_NTSEG1_MASK)

#define CAN_ENCBT_NTSEG2_MASK                    (0x7F000U)
#define CAN_ENCBT_NTSEG2_SHIFT                   (12U)
/*! NTSEG2 - Nominal Time Segment 2 */
#define CAN_ENCBT_NTSEG2(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ENCBT_NTSEG2_SHIFT)) & CAN_ENCBT_NTSEG2_MASK)

#define CAN_ENCBT_NRJW_MASK                      (0x1FC00000U)
#define CAN_ENCBT_NRJW_SHIFT                     (22U)
/*! NRJW - Nominal Resynchronization Jump Width */
#define CAN_ENCBT_NRJW(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ENCBT_NRJW_SHIFT)) & CAN_ENCBT_NRJW_MASK)
/*! @} */

/*! @name EDCBT - Enhanced Data Phase CAN Bit Timing */
/*! @{ */

#define CAN_EDCBT_DTSEG1_MASK                    (0x1FU)
#define CAN_EDCBT_DTSEG1_SHIFT                   (0U)
/*! DTSEG1 - Data Phase Segment 1 */
#define CAN_EDCBT_DTSEG1(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_EDCBT_DTSEG1_SHIFT)) & CAN_EDCBT_DTSEG1_MASK)

#define CAN_EDCBT_DTSEG2_MASK                    (0xF000U)
#define CAN_EDCBT_DTSEG2_SHIFT                   (12U)
/*! DTSEG2 - Data Phase Time Segment 2 */
#define CAN_EDCBT_DTSEG2(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_EDCBT_DTSEG2_SHIFT)) & CAN_EDCBT_DTSEG2_MASK)

#define CAN_EDCBT_DRJW_MASK                      (0x3C00000U)
#define CAN_EDCBT_DRJW_SHIFT                     (22U)
/*! DRJW - Data Phase Resynchronization Jump Width */
#define CAN_EDCBT_DRJW(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_EDCBT_DRJW_SHIFT)) & CAN_EDCBT_DRJW_MASK)
/*! @} */

/*! @name ETDC - Enhanced Transceiver Delay Compensation */
/*! @{ */

#define CAN_ETDC_ETDCVAL_MASK                    (0xFFU)
#define CAN_ETDC_ETDCVAL_SHIFT                   (0U)
/*! ETDCVAL - Enhanced Transceiver Delay Compensation Value */
#define CAN_ETDC_ETDCVAL(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ETDC_ETDCVAL_SHIFT)) & CAN_ETDC_ETDCVAL_MASK)

#define CAN_ETDC_ETDCFAIL_MASK                   (0x8000U)
#define CAN_ETDC_ETDCFAIL_SHIFT                  (15U)
/*! ETDCFAIL - Transceiver Delay Compensation Fail
 *  0b0..In range
 *  0b1..Out of range
 */
#define CAN_ETDC_ETDCFAIL(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_ETDC_ETDCFAIL_SHIFT)) & CAN_ETDC_ETDCFAIL_MASK)

#define CAN_ETDC_ETDCOFF_MASK                    (0x7F0000U)
#define CAN_ETDC_ETDCOFF_SHIFT                   (16U)
/*! ETDCOFF - Enhanced Transceiver Delay Compensation Offset */
#define CAN_ETDC_ETDCOFF(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ETDC_ETDCOFF_SHIFT)) & CAN_ETDC_ETDCOFF_MASK)

#define CAN_ETDC_TDMDIS_MASK                     (0x40000000U)
#define CAN_ETDC_TDMDIS_SHIFT                    (30U)
/*! TDMDIS - Transceiver Delay Measurement Disable
 *  0b0..Enable
 *  0b1..Disable
 */
#define CAN_ETDC_TDMDIS(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ETDC_TDMDIS_SHIFT)) & CAN_ETDC_TDMDIS_MASK)

#define CAN_ETDC_ETDCEN_MASK                     (0x80000000U)
#define CAN_ETDC_ETDCEN_SHIFT                    (31U)
/*! ETDCEN - Transceiver Delay Compensation Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_ETDC_ETDCEN(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ETDC_ETDCEN_SHIFT)) & CAN_ETDC_ETDCEN_MASK)
/*! @} */

/*! @name FDCTRL - CAN FD Control */
/*! @{ */

#define CAN_FDCTRL_TDCVAL_MASK                   (0x3FU)
#define CAN_FDCTRL_TDCVAL_SHIFT                  (0U)
/*! TDCVAL - Transceiver Delay Compensation Value */
#define CAN_FDCTRL_TDCVAL(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_TDCVAL_SHIFT)) & CAN_FDCTRL_TDCVAL_MASK)

#define CAN_FDCTRL_TDCOFF_MASK                   (0x1F00U)
#define CAN_FDCTRL_TDCOFF_SHIFT                  (8U)
/*! TDCOFF - Transceiver Delay Compensation Offset */
#define CAN_FDCTRL_TDCOFF(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_TDCOFF_SHIFT)) & CAN_FDCTRL_TDCOFF_MASK)

#define CAN_FDCTRL_TDCFAIL_MASK                  (0x4000U)
#define CAN_FDCTRL_TDCFAIL_SHIFT                 (14U)
/*! TDCFAIL - Transceiver Delay Compensation Fail
 *  0b0..In range
 *  0b1..Out of range
 */
#define CAN_FDCTRL_TDCFAIL(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_TDCFAIL_SHIFT)) & CAN_FDCTRL_TDCFAIL_MASK)

#define CAN_FDCTRL_TDCEN_MASK                    (0x8000U)
#define CAN_FDCTRL_TDCEN_SHIFT                   (15U)
/*! TDCEN - Transceiver Delay Compensation Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_FDCTRL_TDCEN(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_TDCEN_SHIFT)) & CAN_FDCTRL_TDCEN_MASK)

#define CAN_FDCTRL_MBDSR0_MASK                   (0x30000U)
#define CAN_FDCTRL_MBDSR0_SHIFT                  (16U)
/*! MBDSR0 - Message Buffer Data Size for Region 0
 *  0b00..8 bytes
 *  0b01..16 bytes
 *  0b10..32 bytes
 *  0b11..64 bytes
 */
#define CAN_FDCTRL_MBDSR0(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_MBDSR0_SHIFT)) & CAN_FDCTRL_MBDSR0_MASK)

#define CAN_FDCTRL_MBDSR1_MASK                   (0x180000U)
#define CAN_FDCTRL_MBDSR1_SHIFT                  (19U)
/*! MBDSR1 - Message Buffer Data Size for Region 1
 *  0b00..8 bytes
 *  0b01..16 bytes
 *  0b10..32 bytes
 *  0b11..64 bytes
 */
#define CAN_FDCTRL_MBDSR1(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_MBDSR1_SHIFT)) & CAN_FDCTRL_MBDSR1_MASK)

#define CAN_FDCTRL_MBDSR2_MASK                   (0xC00000U)
#define CAN_FDCTRL_MBDSR2_SHIFT                  (22U)
/*! MBDSR2 - Message Buffer Data Size for Region 2
 *  0b00..8 bytes
 *  0b01..16 bytes
 *  0b10..32 bytes
 *  0b11..64 bytes
 */
#define CAN_FDCTRL_MBDSR2(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_MBDSR2_SHIFT)) & CAN_FDCTRL_MBDSR2_MASK)

#define CAN_FDCTRL_FDRATE_MASK                   (0x80000000U)
#define CAN_FDCTRL_FDRATE_SHIFT                  (31U)
/*! FDRATE - Bit Rate Switch Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_FDCTRL_FDRATE(x)                     (((uint32_t)(((uint32_t)(x)) << CAN_FDCTRL_FDRATE_SHIFT)) & CAN_FDCTRL_FDRATE_MASK)
/*! @} */

/*! @name FDCBT - CAN FD Bit Timing */
/*! @{ */

#define CAN_FDCBT_FPSEG2_MASK                    (0x7U)
#define CAN_FDCBT_FPSEG2_SHIFT                   (0U)
/*! FPSEG2 - Fast Phase Segment 2 */
#define CAN_FDCBT_FPSEG2(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_FDCBT_FPSEG2_SHIFT)) & CAN_FDCBT_FPSEG2_MASK)

#define CAN_FDCBT_FPSEG1_MASK                    (0xE0U)
#define CAN_FDCBT_FPSEG1_SHIFT                   (5U)
/*! FPSEG1 - Fast Phase Segment 1 */
#define CAN_FDCBT_FPSEG1(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_FDCBT_FPSEG1_SHIFT)) & CAN_FDCBT_FPSEG1_MASK)

#define CAN_FDCBT_FPROPSEG_MASK                  (0x7C00U)
#define CAN_FDCBT_FPROPSEG_SHIFT                 (10U)
/*! FPROPSEG - Fast Propagation Segment */
#define CAN_FDCBT_FPROPSEG(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_FDCBT_FPROPSEG_SHIFT)) & CAN_FDCBT_FPROPSEG_MASK)

#define CAN_FDCBT_FRJW_MASK                      (0x70000U)
#define CAN_FDCBT_FRJW_SHIFT                     (16U)
/*! FRJW - Fast Resync Jump Width */
#define CAN_FDCBT_FRJW(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_FDCBT_FRJW_SHIFT)) & CAN_FDCBT_FRJW_MASK)

#define CAN_FDCBT_FPRESDIV_MASK                  (0x3FF00000U)
#define CAN_FDCBT_FPRESDIV_SHIFT                 (20U)
/*! FPRESDIV - Fast Prescaler Division Factor */
#define CAN_FDCBT_FPRESDIV(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_FDCBT_FPRESDIV_SHIFT)) & CAN_FDCBT_FPRESDIV_MASK)
/*! @} */

/*! @name FDCRC - CAN FD CRC */
/*! @{ */

#define CAN_FDCRC_FD_TXCRC_MASK                  (0x1FFFFFU)
#define CAN_FDCRC_FD_TXCRC_SHIFT                 (0U)
/*! FD_TXCRC - Extended Transmitted CRC value */
#define CAN_FDCRC_FD_TXCRC(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_FDCRC_FD_TXCRC_SHIFT)) & CAN_FDCRC_FD_TXCRC_MASK)

#define CAN_FDCRC_FD_MBCRC_MASK                  (0x7F000000U)
#define CAN_FDCRC_FD_MBCRC_SHIFT                 (24U)
/*! FD_MBCRC - CRC Message Buffer Number for FD_TXCRC */
#define CAN_FDCRC_FD_MBCRC(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_FDCRC_FD_MBCRC_SHIFT)) & CAN_FDCRC_FD_MBCRC_MASK)
/*! @} */

/*! @name ERFCR - Enhanced RX FIFO Control */
/*! @{ */

#define CAN_ERFCR_ERFWM_MASK                     (0x1FU)
#define CAN_ERFCR_ERFWM_SHIFT                    (0U)
/*! ERFWM - Enhanced RX FIFO Watermark */
#define CAN_ERFCR_ERFWM(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ERFCR_ERFWM_SHIFT)) & CAN_ERFCR_ERFWM_MASK)

#define CAN_ERFCR_NFE_MASK                       (0x3F00U)
#define CAN_ERFCR_NFE_SHIFT                      (8U)
/*! NFE - Number of Enhanced RX FIFO Filter Elements */
#define CAN_ERFCR_NFE(x)                         (((uint32_t)(((uint32_t)(x)) << CAN_ERFCR_NFE_SHIFT)) & CAN_ERFCR_NFE_MASK)

#define CAN_ERFCR_NEXIF_MASK                     (0x7F0000U)
#define CAN_ERFCR_NEXIF_SHIFT                    (16U)
/*! NEXIF - Number of Extended ID Filter Elements */
#define CAN_ERFCR_NEXIF(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ERFCR_NEXIF_SHIFT)) & CAN_ERFCR_NEXIF_MASK)

#define CAN_ERFCR_DMALW_MASK                     (0x7C000000U)
#define CAN_ERFCR_DMALW_SHIFT                    (26U)
/*! DMALW - DMA Last Word */
#define CAN_ERFCR_DMALW(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ERFCR_DMALW_SHIFT)) & CAN_ERFCR_DMALW_MASK)

#define CAN_ERFCR_ERFEN_MASK                     (0x80000000U)
#define CAN_ERFCR_ERFEN_SHIFT                    (31U)
/*! ERFEN - Enhanced RX FIFO enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_ERFCR_ERFEN(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ERFCR_ERFEN_SHIFT)) & CAN_ERFCR_ERFEN_MASK)
/*! @} */

/*! @name ERFIER - Enhanced RX FIFO Interrupt Enable */
/*! @{ */

#define CAN_ERFIER_ERFDAIE_MASK                  (0x10000000U)
#define CAN_ERFIER_ERFDAIE_SHIFT                 (28U)
/*! ERFDAIE - Enhanced RX FIFO Data Available Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_ERFIER_ERFDAIE(x)                    (((uint32_t)(((uint32_t)(x)) << CAN_ERFIER_ERFDAIE_SHIFT)) & CAN_ERFIER_ERFDAIE_MASK)

#define CAN_ERFIER_ERFWMIIE_MASK                 (0x20000000U)
#define CAN_ERFIER_ERFWMIIE_SHIFT                (29U)
/*! ERFWMIIE - Enhanced RX FIFO Watermark Indication Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_ERFIER_ERFWMIIE(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_ERFIER_ERFWMIIE_SHIFT)) & CAN_ERFIER_ERFWMIIE_MASK)

#define CAN_ERFIER_ERFOVFIE_MASK                 (0x40000000U)
#define CAN_ERFIER_ERFOVFIE_SHIFT                (30U)
/*! ERFOVFIE - Enhanced RX FIFO Overflow Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_ERFIER_ERFOVFIE(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_ERFIER_ERFOVFIE_SHIFT)) & CAN_ERFIER_ERFOVFIE_MASK)

#define CAN_ERFIER_ERFUFWIE_MASK                 (0x80000000U)
#define CAN_ERFIER_ERFUFWIE_SHIFT                (31U)
/*! ERFUFWIE - Enhanced RX FIFO Underflow Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CAN_ERFIER_ERFUFWIE(x)                   (((uint32_t)(((uint32_t)(x)) << CAN_ERFIER_ERFUFWIE_SHIFT)) & CAN_ERFIER_ERFUFWIE_MASK)
/*! @} */

/*! @name ERFSR - Enhanced RX FIFO Status */
/*! @{ */

#define CAN_ERFSR_ERFEL_MASK                     (0x3FU)
#define CAN_ERFSR_ERFEL_SHIFT                    (0U)
/*! ERFEL - Enhanced RX FIFO Elements */
#define CAN_ERFSR_ERFEL(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFEL_SHIFT)) & CAN_ERFSR_ERFEL_MASK)

#define CAN_ERFSR_ERFF_MASK                      (0x10000U)
#define CAN_ERFSR_ERFF_SHIFT                     (16U)
/*! ERFF - Enhanced RX FIFO Full Flag
 *  0b0..Not full
 *  0b1..Full
 */
#define CAN_ERFSR_ERFF(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFF_SHIFT)) & CAN_ERFSR_ERFF_MASK)

#define CAN_ERFSR_ERFE_MASK                      (0x20000U)
#define CAN_ERFSR_ERFE_SHIFT                     (17U)
/*! ERFE - Enhanced RX FIFO Empty Flag
 *  0b0..Not empty
 *  0b1..Empty
 */
#define CAN_ERFSR_ERFE(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFE_SHIFT)) & CAN_ERFSR_ERFE_MASK)

#define CAN_ERFSR_ERFCLR_MASK                    (0x8000000U)
#define CAN_ERFSR_ERFCLR_SHIFT                   (27U)
/*! ERFCLR - Enhanced RX FIFO Clear
 *  0b0..No effect
 *  0b1..Clear enhanced RX FIFO content
 */
#define CAN_ERFSR_ERFCLR(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFCLR_SHIFT)) & CAN_ERFSR_ERFCLR_MASK)

#define CAN_ERFSR_ERFDA_MASK                     (0x10000000U)
#define CAN_ERFSR_ERFDA_SHIFT                    (28U)
/*! ERFDA - Enhanced RX FIFO Data Available Flag
 *  0b0..No such occurrence
 *  0b1..At least one message stored in Enhanced RX FIFO
 */
#define CAN_ERFSR_ERFDA(x)                       (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFDA_SHIFT)) & CAN_ERFSR_ERFDA_MASK)

#define CAN_ERFSR_ERFWMI_MASK                    (0x20000000U)
#define CAN_ERFSR_ERFWMI_SHIFT                   (29U)
/*! ERFWMI - Enhanced RX FIFO Watermark Indication Flag
 *  0b0..No such occurrence
 *  0b1..Number of messages in FIFO is greater than the watermark
 */
#define CAN_ERFSR_ERFWMI(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFWMI_SHIFT)) & CAN_ERFSR_ERFWMI_MASK)

#define CAN_ERFSR_ERFOVF_MASK                    (0x40000000U)
#define CAN_ERFSR_ERFOVF_SHIFT                   (30U)
/*! ERFOVF - Enhanced RX FIFO Overflow Flag
 *  0b0..No such occurrence
 *  0b1..Overflow
 */
#define CAN_ERFSR_ERFOVF(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFOVF_SHIFT)) & CAN_ERFSR_ERFOVF_MASK)

#define CAN_ERFSR_ERFUFW_MASK                    (0x80000000U)
#define CAN_ERFSR_ERFUFW_SHIFT                   (31U)
/*! ERFUFW - Enhanced RX FIFO Underflow Flag
 *  0b0..No such occurrence
 *  0b1..Underflow
 */
#define CAN_ERFSR_ERFUFW(x)                      (((uint32_t)(((uint32_t)(x)) << CAN_ERFSR_ERFUFW_SHIFT)) & CAN_ERFSR_ERFUFW_MASK)
/*! @} */

/*! @name HR_TIME_STAMP - High-Resolution Timestamp */
/*! @{ */

#define CAN_HR_TIME_STAMP_TS_MASK                (0xFFFFFFFFU)
#define CAN_HR_TIME_STAMP_TS_SHIFT               (0U)
/*! TS - High-Resolution Timestamp */
#define CAN_HR_TIME_STAMP_TS(x)                  (((uint32_t)(((uint32_t)(x)) << CAN_HR_TIME_STAMP_TS_SHIFT)) & CAN_HR_TIME_STAMP_TS_MASK)
/*! @} */

/* The count of CAN_HR_TIME_STAMP */
#define CAN_HR_TIME_STAMP_COUNT                  (96U)

/*! @name ERFFEL - Enhanced RX FIFO Filter Element */
/*! @{ */

#define CAN_ERFFEL_FEL_MASK                      (0xFFFFFFFFU)
#define CAN_ERFFEL_FEL_SHIFT                     (0U)
/*! FEL - Filter Element Bits */
#define CAN_ERFFEL_FEL(x)                        (((uint32_t)(((uint32_t)(x)) << CAN_ERFFEL_FEL_SHIFT)) & CAN_ERFFEL_FEL_MASK)
/*! @} */

/* The count of CAN_ERFFEL */
#define CAN_ERFFEL_COUNT                         (128U)


/*!
 * @}
 */ /* end of group CAN_Register_Masks */


/* CAN - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral CAN1 base address */
  #define CAN1_BASE                                (0x543A0000u)
  /** Peripheral CAN1 base address */
  #define CAN1_BASE_NS                             (0x443A0000u)
  /** Peripheral CAN1 base pointer */
  #define CAN1                                     ((CAN_Type *)CAN1_BASE)
  /** Peripheral CAN1 base pointer */
  #define CAN1_NS                                  ((CAN_Type *)CAN1_BASE_NS)
  /** Peripheral CAN2 base address */
  #define CAN2_BASE                                (0x525B0000u)
  /** Peripheral CAN2 base address */
  #define CAN2_BASE_NS                             (0x425B0000u)
  /** Peripheral CAN2 base pointer */
  #define CAN2                                     ((CAN_Type *)CAN2_BASE)
  /** Peripheral CAN2 base pointer */
  #define CAN2_NS                                  ((CAN_Type *)CAN2_BASE_NS)
  /** Array initializer of CAN peripheral base addresses */
  #define CAN_BASE_ADDRS                           { 0u, CAN1_BASE, CAN2_BASE }
  /** Array initializer of CAN peripheral base pointers */
  #define CAN_BASE_PTRS                            { (CAN_Type *)0u, CAN1, CAN2 }
  /** Array initializer of CAN peripheral base addresses */
  #define CAN_BASE_ADDRS_NS                        { 0u, CAN1_BASE_NS, CAN2_BASE_NS }
  /** Array initializer of CAN peripheral base pointers */
  #define CAN_BASE_PTRS_NS                         { (CAN_Type *)0u, CAN1_NS, CAN2_NS }
#else
  /** Peripheral CAN1 base address */
  #define CAN1_BASE                                (0x443A0000u)
  /** Peripheral CAN1 base pointer */
  #define CAN1                                     ((CAN_Type *)CAN1_BASE)
  /** Peripheral CAN2 base address */
  #define CAN2_BASE                                (0x425B0000u)
  /** Peripheral CAN2 base pointer */
  #define CAN2                                     ((CAN_Type *)CAN2_BASE)
  /** Array initializer of CAN peripheral base addresses */
  #define CAN_BASE_ADDRS                           { 0u, CAN1_BASE, CAN2_BASE }
  /** Array initializer of CAN peripheral base pointers */
  #define CAN_BASE_PTRS                            { (CAN_Type *)0u, CAN1, CAN2 }
#endif
/** Interrupt vectors for the CAN peripheral type */
#define CAN_Rx_Warning_IRQS                      { NotAvail_IRQn, CAN1_IRQn, CAN2_IRQn }
#define CAN_Tx_Warning_IRQS                      { NotAvail_IRQn, CAN1_IRQn, CAN2_IRQn }
#define CAN_Wake_Up_IRQS                         { NotAvail_IRQn, CAN1_IRQn, CAN2_IRQn }
#define CAN_Error_IRQS                           { NotAvail_IRQn, CAN1_IRQn, CAN2_IRQn }
#define CAN_Bus_Off_IRQS                         { NotAvail_IRQn, CAN1_IRQn, CAN2_IRQn }
#define CAN_ORed_Message_buffer_IRQS             { NotAvail_IRQn, CAN1_IRQn, CAN2_IRQn }

/*!
 * @}
 */ /* end of group CAN_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- CCM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CCM_Peripheral_Access_Layer CCM Peripheral Access Layer
 * @{
 */

/** CCM - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x80 */
    __IO uint32_t CONTROL;                           /**< Clock Root Control Register, array offset: 0x0, array step: 0x80 */
    __IO uint32_t CONTROL_SET;                       /**< Clock Root Control Register, array offset: 0x4, array step: 0x80 */
    __IO uint32_t CONTROL_CLR;                       /**< Clock Root Control Register, array offset: 0x8, array step: 0x80 */
    __IO uint32_t CONTROL_TOG;                       /**< Clock Root Control Register, array offset: 0xC, array step: 0x80 */
         uint8_t RESERVED_0[16];
    __I  uint32_t STATUS0;                           /**< Clock root working status, array offset: 0x20, array step: 0x80 */
         uint8_t RESERVED_1[12];
    __IO uint32_t AUTHEN;                            /**< Clock root access control, array offset: 0x30, array step: 0x80 */
         uint8_t RESERVED_2[76];
  } CLOCK_ROOT[74];
       uint8_t RESERVED_0[7936];
  struct {                                         /* offset: 0x4400, array step: 0x80 */
    __IO uint32_t CONTROL;                           /**< Observe control, array offset: 0x4400, array step: 0x80 */
    __IO uint32_t CONTROL_SET;                       /**< Observe control, array offset: 0x4404, array step: 0x80 */
    __IO uint32_t CONTROL_CLR;                       /**< Observe control, array offset: 0x4408, array step: 0x80 */
    __IO uint32_t CONTROL_TOG;                       /**< Observe control, array offset: 0x440C, array step: 0x80 */
         uint8_t RESERVED_0[16];
    __I  uint32_t STATUS;                            /**< Observe status, array offset: 0x4420, array step: 0x80 */
         uint8_t RESERVED_1[12];
    __IO uint32_t AUTHEN;                            /**< Observe access control, array offset: 0x4430, array step: 0x80 */
    __IO uint32_t AUTHEN_SET;                        /**< Observe access control, array offset: 0x4434, array step: 0x80 */
    __IO uint32_t AUTHEN_CLR;                        /**< Observe access control, array offset: 0x4438, array step: 0x80 */
    __IO uint32_t AUTHEN_TOG;                        /**< Observe access control, array offset: 0x443C, array step: 0x80 */
    __I  uint32_t FREQUENCY_CURRENT;                 /**< Current frequency detected, array offset: 0x4440, array step: 0x80 */
    __I  uint32_t FREQUENCY_MIN;                     /**< Minimum frequency detected, array offset: 0x4444, array step: 0x80 */
    __I  uint32_t FREQUENCY_MAX;                     /**< Maximum frequency detected, array offset: 0x4448, array step: 0x80 */
         uint8_t RESERVED_2[4];
    __I  uint32_t PERIOD_CURRENT;                    /**< Current period time detected, array offset: 0x4450, array step: 0x80 */
    __I  uint32_t PERIOD_MIN;                        /**< Minimum period time detected, array offset: 0x4454, array step: 0x80 */
    __I  uint32_t PERIOD_MAX;                        /**< Maximum period time detected, array offset: 0x4458, array step: 0x80 */
         uint8_t RESERVED_3[4];
    __I  uint32_t HIGH_CURRENT;                      /**< Current high level time detected, array offset: 0x4460, array step: 0x80 */
    __I  uint32_t HIGH_MIN;                          /**< Minimum high level time detected, array offset: 0x4464, array step: 0x80 */
    __I  uint32_t HIGH_MAX;                          /**< Maximum high level time detected, array offset: 0x4468, array step: 0x80 */
         uint8_t RESERVED_4[4];
    __I  uint32_t LOW_CURRENT;                       /**< Current high level time detected, array offset: 0x4470, array step: 0x80 */
    __I  uint32_t LOW_MIN;                           /**< Minimum high level time detected, array offset: 0x4474, array step: 0x80 */
    __I  uint32_t LOW_MAX;                           /**< Maximum high level time detected, array offset: 0x4478, array step: 0x80 */
         uint8_t RESERVED_5[4];
  } OBSERVE[2];
       uint8_t RESERVED_1[768];
       uint32_t GPR_SHARED0;                       /**< General Purpose Register, offset: 0x4800 */
       uint32_t GPR_SHARED0_SET;                   /**< General Purpose Register, offset: 0x4804 */
       uint32_t GPR_SHARED0_CLR;                   /**< General Purpose Register, offset: 0x4808 */
       uint32_t GPR_SHARED0_TOG;                   /**< General Purpose Register, offset: 0x480C */
  __IO uint32_t GPR_SHARED0_AUTHEN;                /**< GPR access control, offset: 0x4810 */
  __IO uint32_t GPR_SHARED0_AUTHEN_SET;            /**< GPR access control, offset: 0x4814 */
  __IO uint32_t GPR_SHARED0_AUTHEN_CLR;            /**< GPR access control, offset: 0x4818 */
  __IO uint32_t GPR_SHARED0_AUTHEN_TOG;            /**< GPR access control, offset: 0x481C */
       uint32_t GPR_SHARED1;                       /**< General Purpose Register, offset: 0x4820 */
       uint32_t GPR_SHARED1_SET;                   /**< General Purpose Register, offset: 0x4824 */
       uint32_t GPR_SHARED1_CLR;                   /**< General Purpose Register, offset: 0x4828 */
       uint32_t GPR_SHARED1_TOG;                   /**< General Purpose Register, offset: 0x482C */
  __IO uint32_t GPR_SHARED1_AUTHEN;                /**< GPR access control, offset: 0x4830 */
  __IO uint32_t GPR_SHARED1_AUTHEN_SET;            /**< GPR access control, offset: 0x4834 */
  __IO uint32_t GPR_SHARED1_AUTHEN_CLR;            /**< GPR access control, offset: 0x4838 */
  __IO uint32_t GPR_SHARED1_AUTHEN_TOG;            /**< GPR access control, offset: 0x483C */
  __IO uint32_t GPR_SHARED2;                       /**< General Purpose Register, offset: 0x4840 */
  __IO uint32_t GPR_SHARED2_SET;                   /**< General Purpose Register, offset: 0x4844 */
  __IO uint32_t GPR_SHARED2_CLR;                   /**< General Purpose Register, offset: 0x4848 */
  __IO uint32_t GPR_SHARED2_TOG;                   /**< General Purpose Register, offset: 0x484C */
  __IO uint32_t GPR_SHARED2_AUTHEN;                /**< GPR access control, offset: 0x4850 */
  __IO uint32_t GPR_SHARED2_AUTHEN_SET;            /**< GPR access control, offset: 0x4854 */
  __IO uint32_t GPR_SHARED2_AUTHEN_CLR;            /**< GPR access control, offset: 0x4858 */
  __IO uint32_t GPR_SHARED2_AUTHEN_TOG;            /**< GPR access control, offset: 0x485C */
  __IO uint32_t GPR_SHARED3;                       /**< General Purpose Register, offset: 0x4860 */
  __IO uint32_t GPR_SHARED3_SET;                   /**< General Purpose Register, offset: 0x4864 */
  __IO uint32_t GPR_SHARED3_CLR;                   /**< General Purpose Register, offset: 0x4868 */
  __IO uint32_t GPR_SHARED3_TOG;                   /**< General Purpose Register, offset: 0x486C */
  __IO uint32_t GPR_SHARED3_AUTHEN;                /**< GPR access control, offset: 0x4870 */
  __IO uint32_t GPR_SHARED3_AUTHEN_SET;            /**< GPR access control, offset: 0x4874 */
  __IO uint32_t GPR_SHARED3_AUTHEN_CLR;            /**< GPR access control, offset: 0x4878 */
  __IO uint32_t GPR_SHARED3_AUTHEN_TOG;            /**< GPR access control, offset: 0x487C */
  __IO uint32_t GPR_SHARED4;                       /**< General Purpose Register, offset: 0x4880 */
  __IO uint32_t GPR_SHARED4_SET;                   /**< General Purpose Register, offset: 0x4884 */
  __IO uint32_t GPR_SHARED4_CLR;                   /**< General Purpose Register, offset: 0x4888 */
  __IO uint32_t GPR_SHARED4_TOG;                   /**< General Purpose Register, offset: 0x488C */
  __IO uint32_t GPR_SHARED4_AUTHEN;                /**< GPR access control, offset: 0x4890 */
  __IO uint32_t GPR_SHARED4_AUTHEN_SET;            /**< GPR access control, offset: 0x4894 */
  __IO uint32_t GPR_SHARED4_AUTHEN_CLR;            /**< GPR access control, offset: 0x4898 */
  __IO uint32_t GPR_SHARED4_AUTHEN_TOG;            /**< GPR access control, offset: 0x489C */
  __IO uint32_t GPR_SHARED5;                       /**< General Purpose Register, offset: 0x48A0 */
  __IO uint32_t GPR_SHARED5_SET;                   /**< General Purpose Register, offset: 0x48A4 */
  __IO uint32_t GPR_SHARED5_CLR;                   /**< General Purpose Register, offset: 0x48A8 */
  __IO uint32_t GPR_SHARED5_TOG;                   /**< General Purpose Register, offset: 0x48AC */
  __IO uint32_t GPR_SHARED5_AUTHEN;                /**< GPR access control, offset: 0x48B0 */
  __IO uint32_t GPR_SHARED5_AUTHEN_SET;            /**< GPR access control, offset: 0x48B4 */
  __IO uint32_t GPR_SHARED5_AUTHEN_CLR;            /**< GPR access control, offset: 0x48B8 */
  __IO uint32_t GPR_SHARED5_AUTHEN_TOG;            /**< GPR access control, offset: 0x48BC */
  __IO uint32_t GPR_SHARED6;                       /**< General Purpose Register, offset: 0x48C0 */
  __IO uint32_t GPR_SHARED6_SET;                   /**< General Purpose Register, offset: 0x48C4 */
  __IO uint32_t GPR_SHARED6_CLR;                   /**< General Purpose Register, offset: 0x48C8 */
  __IO uint32_t GPR_SHARED6_TOG;                   /**< General Purpose Register, offset: 0x48CC */
  __IO uint32_t GPR_SHARED6_AUTHEN;                /**< GPR access control, offset: 0x48D0 */
  __IO uint32_t GPR_SHARED6_AUTHEN_SET;            /**< GPR access control, offset: 0x48D4 */
  __IO uint32_t GPR_SHARED6_AUTHEN_CLR;            /**< GPR access control, offset: 0x48D8 */
  __IO uint32_t GPR_SHARED6_AUTHEN_TOG;            /**< GPR access control, offset: 0x48DC */
  __IO uint32_t GPR_SHARED7;                       /**< General Purpose Register, offset: 0x48E0 */
  __IO uint32_t GPR_SHARED7_SET;                   /**< General Purpose Register, offset: 0x48E4 */
  __IO uint32_t GPR_SHARED7_CLR;                   /**< General Purpose Register, offset: 0x48E8 */
  __IO uint32_t GPR_SHARED7_TOG;                   /**< General Purpose Register, offset: 0x48EC */
  __IO uint32_t GPR_SHARED7_AUTHEN;                /**< GPR access control, offset: 0x48F0 */
  __IO uint32_t GPR_SHARED7_AUTHEN_SET;            /**< GPR access control, offset: 0x48F4 */
  __IO uint32_t GPR_SHARED7_AUTHEN_CLR;            /**< GPR access control, offset: 0x48F8 */
  __IO uint32_t GPR_SHARED7_AUTHEN_TOG;            /**< GPR access control, offset: 0x48FC */
  __IO uint32_t GPR_SHARED8;                       /**< General Purpose Register, offset: 0x4900 */
  __IO uint32_t GPR_SHARED8_SET;                   /**< General Purpose Register, offset: 0x4904 */
  __IO uint32_t GPR_SHARED8_CLR;                   /**< General Purpose Register, offset: 0x4908 */
  __IO uint32_t GPR_SHARED8_TOG;                   /**< General Purpose Register, offset: 0x490C */
  __IO uint32_t GPR_SHARED8_AUTHEN;                /**< GPR access control, offset: 0x4910 */
  __IO uint32_t GPR_SHARED8_AUTHEN_SET;            /**< GPR access control, offset: 0x4914 */
  __IO uint32_t GPR_SHARED8_AUTHEN_CLR;            /**< GPR access control, offset: 0x4918 */
  __IO uint32_t GPR_SHARED8_AUTHEN_TOG;            /**< GPR access control, offset: 0x491C */
  __IO uint32_t GPR_SHARED9;                       /**< General Purpose Register, offset: 0x4920 */
  __IO uint32_t GPR_SHARED9_SET;                   /**< General Purpose Register, offset: 0x4924 */
  __IO uint32_t GPR_SHARED9_CLR;                   /**< General Purpose Register, offset: 0x4928 */
  __IO uint32_t GPR_SHARED9_TOG;                   /**< General Purpose Register, offset: 0x492C */
  __IO uint32_t GPR_SHARED9_AUTHEN;                /**< GPR access control, offset: 0x4930 */
  __IO uint32_t GPR_SHARED9_AUTHEN_SET;            /**< GPR access control, offset: 0x4934 */
  __IO uint32_t GPR_SHARED9_AUTHEN_CLR;            /**< GPR access control, offset: 0x4938 */
  __IO uint32_t GPR_SHARED9_AUTHEN_TOG;            /**< GPR access control, offset: 0x493C */
  __IO uint32_t GPR_SHARED10;                      /**< General Purpose Register, offset: 0x4940 */
  __IO uint32_t GPR_SHARED10_SET;                  /**< General Purpose Register, offset: 0x4944 */
  __IO uint32_t GPR_SHARED10_CLR;                  /**< General Purpose Register, offset: 0x4948 */
  __IO uint32_t GPR_SHARED10_TOG;                  /**< General Purpose Register, offset: 0x494C */
  __IO uint32_t GPR_SHARED10_AUTHEN;               /**< GPR access control, offset: 0x4950 */
  __IO uint32_t GPR_SHARED10_AUTHEN_SET;           /**< GPR access control, offset: 0x4954 */
  __IO uint32_t GPR_SHARED10_AUTHEN_CLR;           /**< GPR access control, offset: 0x4958 */
  __IO uint32_t GPR_SHARED10_AUTHEN_TOG;           /**< GPR access control, offset: 0x495C */
  __IO uint32_t GPR_SHARED11;                      /**< General Purpose Register, offset: 0x4960 */
  __IO uint32_t GPR_SHARED11_SET;                  /**< General Purpose Register, offset: 0x4964 */
  __IO uint32_t GPR_SHARED11_CLR;                  /**< General Purpose Register, offset: 0x4968 */
  __IO uint32_t GPR_SHARED11_TOG;                  /**< General Purpose Register, offset: 0x496C */
  __IO uint32_t GPR_SHARED11_AUTHEN;               /**< GPR access control, offset: 0x4970 */
  __IO uint32_t GPR_SHARED11_AUTHEN_SET;           /**< GPR access control, offset: 0x4974 */
  __IO uint32_t GPR_SHARED11_AUTHEN_CLR;           /**< GPR access control, offset: 0x4978 */
  __IO uint32_t GPR_SHARED11_AUTHEN_TOG;           /**< GPR access control, offset: 0x497C */
  __IO uint32_t GPR_SHARED12;                      /**< General Purpose Register, offset: 0x4980 */
  __IO uint32_t GPR_SHARED12_SET;                  /**< General Purpose Register, offset: 0x4984 */
  __IO uint32_t GPR_SHARED12_CLR;                  /**< General Purpose Register, offset: 0x4988 */
  __IO uint32_t GPR_SHARED12_TOG;                  /**< General Purpose Register, offset: 0x498C */
  __IO uint32_t GPR_SHARED12_AUTHEN;               /**< GPR access control, offset: 0x4990 */
  __IO uint32_t GPR_SHARED12_AUTHEN_SET;           /**< GPR access control, offset: 0x4994 */
  __IO uint32_t GPR_SHARED12_AUTHEN_CLR;           /**< GPR access control, offset: 0x4998 */
  __IO uint32_t GPR_SHARED12_AUTHEN_TOG;           /**< GPR access control, offset: 0x499C */
  __IO uint32_t GPR_SHARED13;                      /**< General Purpose Register, offset: 0x49A0 */
  __IO uint32_t GPR_SHARED13_SET;                  /**< General Purpose Register, offset: 0x49A4 */
  __IO uint32_t GPR_SHARED13_CLR;                  /**< General Purpose Register, offset: 0x49A8 */
  __IO uint32_t GPR_SHARED13_TOG;                  /**< General Purpose Register, offset: 0x49AC */
  __IO uint32_t GPR_SHARED13_AUTHEN;               /**< GPR access control, offset: 0x49B0 */
  __IO uint32_t GPR_SHARED13_AUTHEN_SET;           /**< GPR access control, offset: 0x49B4 */
  __IO uint32_t GPR_SHARED13_AUTHEN_CLR;           /**< GPR access control, offset: 0x49B8 */
  __IO uint32_t GPR_SHARED13_AUTHEN_TOG;           /**< GPR access control, offset: 0x49BC */
  __IO uint32_t GPR_SHARED14;                      /**< General Purpose Register, offset: 0x49C0 */
  __IO uint32_t GPR_SHARED14_SET;                  /**< General Purpose Register, offset: 0x49C4 */
  __IO uint32_t GPR_SHARED14_CLR;                  /**< General Purpose Register, offset: 0x49C8 */
  __IO uint32_t GPR_SHARED14_TOG;                  /**< General Purpose Register, offset: 0x49CC */
  __IO uint32_t GPR_SHARED14_AUTHEN;               /**< GPR access control, offset: 0x49D0 */
  __IO uint32_t GPR_SHARED14_AUTHEN_SET;           /**< GPR access control, offset: 0x49D4 */
  __IO uint32_t GPR_SHARED14_AUTHEN_CLR;           /**< GPR access control, offset: 0x49D8 */
  __IO uint32_t GPR_SHARED14_AUTHEN_TOG;           /**< GPR access control, offset: 0x49DC */
  __IO uint32_t GPR_SHARED15;                      /**< General Purpose Register, offset: 0x49E0 */
  __IO uint32_t GPR_SHARED15_SET;                  /**< General Purpose Register, offset: 0x49E4 */
  __IO uint32_t GPR_SHARED15_CLR;                  /**< General Purpose Register, offset: 0x49E8 */
  __IO uint32_t GPR_SHARED15_TOG;                  /**< General Purpose Register, offset: 0x49EC */
  __IO uint32_t GPR_SHARED15_AUTHEN;               /**< GPR access control, offset: 0x49F0 */
  __IO uint32_t GPR_SHARED15_AUTHEN_SET;           /**< GPR access control, offset: 0x49F4 */
  __IO uint32_t GPR_SHARED15_AUTHEN_CLR;           /**< GPR access control, offset: 0x49F8 */
  __IO uint32_t GPR_SHARED15_AUTHEN_TOG;           /**< GPR access control, offset: 0x49FC */
  __I  uint32_t GPR_SHARED_STATUS[8];              /**< General purpose status register for CM33..General purpose status register for CM7, array offset: 0x4A00, array step: 0x4 */
       uint8_t RESERVED_2[480];
  struct {                                         /* offset: 0x4C00, array step: 0x20 */
    __IO uint32_t GPR_PRIVATE;                       /**< General purpose register, array offset: 0x4C00, array step: 0x20 */
    __IO uint32_t SET;                               /**< General purpose register, array offset: 0x4C04, array step: 0x20 */
    __IO uint32_t CLR;                               /**< General purpose register, array offset: 0x4C08, array step: 0x20 */
    __IO uint32_t TOG;                               /**< General purpose register, array offset: 0x4C0C, array step: 0x20 */
    __IO uint32_t AUTHEN;                            /**< GPR access control, array offset: 0x4C10, array step: 0x20 */
    __IO uint32_t AUTHEN_SET;                        /**< GPR access control, array offset: 0x4C14, array step: 0x20 */
    __IO uint32_t AUTHEN_CLR;                        /**< GPR access control, array offset: 0x4C18, array step: 0x20 */
    __IO uint32_t AUTHEN_TOG;                        /**< GPR access control, array offset: 0x4C1C, array step: 0x20 */
  } GPR_PRIVATE[4];
       uint8_t RESERVED_3[896];
  struct {                                         /* offset: 0x5000, array step: 0x40 */
    __IO uint32_t DIRECT;                            /**< Clock source direct control, array offset: 0x5000, array step: 0x40 */
         uint8_t RESERVED_0[12];
    __IO uint32_t LPM0;                              /**< Clock source low power mode setting, array offset: 0x5010, array step: 0x40 */
    __IO uint32_t LPM1;                              /**< clock source low power mode setting, array offset: 0x5014, array step: 0x40 */
         uint8_t RESERVED_1[4];
    __IO uint32_t LPM_CUR;                           /**< LPM setting of current CPU domain, array offset: 0x501C, array step: 0x40 */
    __I  uint32_t STATUS0;                           /**< Clock source working status, array offset: 0x5020, array step: 0x40 */
    __I  uint32_t STATUS1;                           /**< Clock source domain status, array offset: 0x5024, array step: 0x40 */
         uint8_t RESERVED_2[8];
    __IO uint32_t AUTHEN;                            /**< Clock Source access control, array offset: 0x5030, array step: 0x40 */
         uint8_t RESERVED_3[12];
  } OSCPLL[25];
       uint8_t RESERVED_4[10688];
  struct {                                         /* offset: 0x8000, array step: 0x40 */
    __IO uint32_t DIRECT;                            /**< LPCG direct control, array offset: 0x8000, array step: 0x40 */
         uint8_t RESERVED_0[12];
    __IO uint32_t LPM0;                              /**< Clock source low power mode setting, array offset: 0x8010, array step: 0x40 */
    __IO uint32_t LPM1;                              /**< clock source low power mode setting, array offset: 0x8014, array step: 0x40 */
         uint8_t RESERVED_1[4];
    __IO uint32_t LPM_CUR;                           /**< LPM setting of current CPU domain, array offset: 0x801C, array step: 0x40 */
    __I  uint32_t STATUS0;                           /**< LPCG working status, array offset: 0x8020, array step: 0x40 */
    __I  uint32_t STATUS1;                           /**< LPCG domain status, array offset: 0x8024, array step: 0x40 */
         uint8_t RESERVED_2[8];
    __IO uint32_t AUTHEN;                            /**< LPCG access control, array offset: 0x8030, array step: 0x40 */
         uint8_t RESERVED_3[12];
  } LPCG[149];
} CCM_Type;

/* ----------------------------------------------------------------------------
   -- CCM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CCM_Register_Masks CCM Register Masks
 * @{
 */

/*! @name CLOCK_ROOT_CONTROL - Clock Root Control Register */
/*! @{ */

#define CCM_CLOCK_ROOT_CONTROL_DIV_MASK          (0xFFU)
#define CCM_CLOCK_ROOT_CONTROL_DIV_SHIFT         (0U)
/*! DIV - Clock division fraction. */
#define CCM_CLOCK_ROOT_CONTROL_DIV(x)            (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_DIV_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_DIV_MASK)

#define CCM_CLOCK_ROOT_CONTROL_MUX_MASK          (0x300U)
#define CCM_CLOCK_ROOT_CONTROL_MUX_SHIFT         (8U)
/*! MUX - Clock multiplexer.
 *  0b10..Select clock source 2
 *  0b00..Select clock source 0
 *  0b01..Select clock source 1
 *  0b11..Select clock source 3
 */
#define CCM_CLOCK_ROOT_CONTROL_MUX(x)            (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_MUX_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_MUX_MASK)

#define CCM_CLOCK_ROOT_CONTROL_OFF_MASK          (0x1000000U)
#define CCM_CLOCK_ROOT_CONTROL_OFF_SHIFT         (24U)
/*! OFF - Shutdown clock root.
 *  0b0..Clock root is enabled
 *  0b1..Clock root is disabled
 */
#define CCM_CLOCK_ROOT_CONTROL_OFF(x)            (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_OFF_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_OFF_MASK)
/*! @} */

/* The count of CCM_CLOCK_ROOT_CONTROL */
#define CCM_CLOCK_ROOT_CONTROL_COUNT             (74U)

/*! @name CLOCK_ROOT_CONTROL_SET - Clock Root Control Register */
/*! @{ */

#define CCM_CLOCK_ROOT_CONTROL_SET_DIV_MASK      (0xFFU)
#define CCM_CLOCK_ROOT_CONTROL_SET_DIV_SHIFT     (0U)
/*! DIV - Clock division fraction. */
#define CCM_CLOCK_ROOT_CONTROL_SET_DIV(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_SET_DIV_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_SET_DIV_MASK)

#define CCM_CLOCK_ROOT_CONTROL_SET_MUX_MASK      (0x300U)
#define CCM_CLOCK_ROOT_CONTROL_SET_MUX_SHIFT     (8U)
/*! MUX - Clock multiplexer. */
#define CCM_CLOCK_ROOT_CONTROL_SET_MUX(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_SET_MUX_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_SET_MUX_MASK)

#define CCM_CLOCK_ROOT_CONTROL_SET_OFF_MASK      (0x1000000U)
#define CCM_CLOCK_ROOT_CONTROL_SET_OFF_SHIFT     (24U)
/*! OFF - Shutdown clock root. */
#define CCM_CLOCK_ROOT_CONTROL_SET_OFF(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_SET_OFF_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_SET_OFF_MASK)
/*! @} */

/* The count of CCM_CLOCK_ROOT_CONTROL_SET */
#define CCM_CLOCK_ROOT_CONTROL_SET_COUNT         (74U)

/*! @name CLOCK_ROOT_CONTROL_CLR - Clock Root Control Register */
/*! @{ */

#define CCM_CLOCK_ROOT_CONTROL_CLR_DIV_MASK      (0xFFU)
#define CCM_CLOCK_ROOT_CONTROL_CLR_DIV_SHIFT     (0U)
/*! DIV - Clock division fraction. */
#define CCM_CLOCK_ROOT_CONTROL_CLR_DIV(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_CLR_DIV_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_CLR_DIV_MASK)

#define CCM_CLOCK_ROOT_CONTROL_CLR_MUX_MASK      (0x300U)
#define CCM_CLOCK_ROOT_CONTROL_CLR_MUX_SHIFT     (8U)
/*! MUX - Clock multiplexer. */
#define CCM_CLOCK_ROOT_CONTROL_CLR_MUX(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_CLR_MUX_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_CLR_MUX_MASK)

#define CCM_CLOCK_ROOT_CONTROL_CLR_OFF_MASK      (0x1000000U)
#define CCM_CLOCK_ROOT_CONTROL_CLR_OFF_SHIFT     (24U)
/*! OFF - Shutdown clock root. */
#define CCM_CLOCK_ROOT_CONTROL_CLR_OFF(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_CLR_OFF_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_CLR_OFF_MASK)
/*! @} */

/* The count of CCM_CLOCK_ROOT_CONTROL_CLR */
#define CCM_CLOCK_ROOT_CONTROL_CLR_COUNT         (74U)

/*! @name CLOCK_ROOT_CONTROL_TOG - Clock Root Control Register */
/*! @{ */

#define CCM_CLOCK_ROOT_CONTROL_TOG_DIV_MASK      (0xFFU)
#define CCM_CLOCK_ROOT_CONTROL_TOG_DIV_SHIFT     (0U)
/*! DIV - Clock division fraction. */
#define CCM_CLOCK_ROOT_CONTROL_TOG_DIV(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_TOG_DIV_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_TOG_DIV_MASK)

#define CCM_CLOCK_ROOT_CONTROL_TOG_MUX_MASK      (0x300U)
#define CCM_CLOCK_ROOT_CONTROL_TOG_MUX_SHIFT     (8U)
/*! MUX - Clock multiplexer. */
#define CCM_CLOCK_ROOT_CONTROL_TOG_MUX(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_TOG_MUX_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_TOG_MUX_MASK)

#define CCM_CLOCK_ROOT_CONTROL_TOG_OFF_MASK      (0x1000000U)
#define CCM_CLOCK_ROOT_CONTROL_TOG_OFF_SHIFT     (24U)
/*! OFF - Shutdown clock root. */
#define CCM_CLOCK_ROOT_CONTROL_TOG_OFF(x)        (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_CONTROL_TOG_OFF_SHIFT)) & CCM_CLOCK_ROOT_CONTROL_TOG_OFF_MASK)
/*! @} */

/* The count of CCM_CLOCK_ROOT_CONTROL_TOG */
#define CCM_CLOCK_ROOT_CONTROL_TOG_COUNT         (74U)

/*! @name CLOCK_ROOT_STATUS0 - Clock root working status */
/*! @{ */

#define CCM_CLOCK_ROOT_STATUS0_DIV_MASK          (0xFFU)
#define CCM_CLOCK_ROOT_STATUS0_DIV_SHIFT         (0U)
/*! DIV - Current clock root DIV setting */
#define CCM_CLOCK_ROOT_STATUS0_DIV(x)            (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_STATUS0_DIV_SHIFT)) & CCM_CLOCK_ROOT_STATUS0_DIV_MASK)

#define CCM_CLOCK_ROOT_STATUS0_MUX_MASK          (0x300U)
#define CCM_CLOCK_ROOT_STATUS0_MUX_SHIFT         (8U)
/*! MUX - Current clock root MUX setting */
#define CCM_CLOCK_ROOT_STATUS0_MUX(x)            (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_STATUS0_MUX_SHIFT)) & CCM_CLOCK_ROOT_STATUS0_MUX_MASK)

#define CCM_CLOCK_ROOT_STATUS0_OFF_MASK          (0x1000000U)
#define CCM_CLOCK_ROOT_STATUS0_OFF_SHIFT         (24U)
/*! OFF - Current clock root OFF setting
 *  0b0..Clock root is enabled
 *  0b1..Clock root is disabled
 */
#define CCM_CLOCK_ROOT_STATUS0_OFF(x)            (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_STATUS0_OFF_SHIFT)) & CCM_CLOCK_ROOT_STATUS0_OFF_MASK)

#define CCM_CLOCK_ROOT_STATUS0_SLICE_BUSY_MASK   (0x10000000U)
#define CCM_CLOCK_ROOT_STATUS0_SLICE_BUSY_SHIFT  (28U)
/*! SLICE_BUSY - Internal updating in generation logic Indication for clock generation logic is applying new setting.
 *  0b0..Clock generation logic is not busy
 *  0b1..Clock generation logic is applying the new setting
 */
#define CCM_CLOCK_ROOT_STATUS0_SLICE_BUSY(x)     (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_STATUS0_SLICE_BUSY_SHIFT)) & CCM_CLOCK_ROOT_STATUS0_SLICE_BUSY_MASK)

#define CCM_CLOCK_ROOT_STATUS0_UPDATE_FORWARD_MASK (0x20000000U)
#define CCM_CLOCK_ROOT_STATUS0_UPDATE_FORWARD_SHIFT (29U)
/*! UPDATE_FORWARD - Internal status synchronization to clock generation logic Indication for clock status is synchronizing for clock root.
 *  0b0..Synchronization not in process
 *  0b1..Synchronization in process
 */
#define CCM_CLOCK_ROOT_STATUS0_UPDATE_FORWARD(x) (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_STATUS0_UPDATE_FORWARD_SHIFT)) & CCM_CLOCK_ROOT_STATUS0_UPDATE_FORWARD_MASK)

#define CCM_CLOCK_ROOT_STATUS0_CHANGING_MASK     (0x80000000U)
#define CCM_CLOCK_ROOT_STATUS0_CHANGING_SHIFT    (31U)
/*! CHANGING - Indication for clock root internal logic is updating. This status is a combination of UPDATE_FORWARD and SLICE_BUSY.
 *  0b0..Clock Status is not updating currently
 *  0b1..Clock generation logic is currently updating
 */
#define CCM_CLOCK_ROOT_STATUS0_CHANGING(x)       (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_STATUS0_CHANGING_SHIFT)) & CCM_CLOCK_ROOT_STATUS0_CHANGING_MASK)
/*! @} */

/* The count of CCM_CLOCK_ROOT_STATUS0 */
#define CCM_CLOCK_ROOT_STATUS0_COUNT             (74U)

/*! @name CLOCK_ROOT_AUTHEN - Clock root access control */
/*! @{ */

#define CCM_CLOCK_ROOT_AUTHEN_TZ_USER_MASK       (0x100U)
#define CCM_CLOCK_ROOT_AUTHEN_TZ_USER_SHIFT      (8U)
/*! TZ_USER - User access permission
 *  0b1..Clock Root settings can be changed in user mode.
 *  0b0..Clock Root settings cannot be changed in user mode.
 */
#define CCM_CLOCK_ROOT_AUTHEN_TZ_USER(x)         (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_AUTHEN_TZ_USER_SHIFT)) & CCM_CLOCK_ROOT_AUTHEN_TZ_USER_MASK)

#define CCM_CLOCK_ROOT_AUTHEN_TZ_NS_MASK         (0x200U)
#define CCM_CLOCK_ROOT_AUTHEN_TZ_NS_SHIFT        (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_CLOCK_ROOT_AUTHEN_TZ_NS(x)           (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_AUTHEN_TZ_NS_SHIFT)) & CCM_CLOCK_ROOT_AUTHEN_TZ_NS_MASK)

#define CCM_CLOCK_ROOT_AUTHEN_LOCK_TZ_MASK       (0x800U)
#define CCM_CLOCK_ROOT_AUTHEN_LOCK_TZ_SHIFT      (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_CLOCK_ROOT_AUTHEN_LOCK_TZ(x)         (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_AUTHEN_LOCK_TZ_SHIFT)) & CCM_CLOCK_ROOT_AUTHEN_LOCK_TZ_MASK)

#define CCM_CLOCK_ROOT_AUTHEN_LOCK_LIST_MASK     (0x8000U)
#define CCM_CLOCK_ROOT_AUTHEN_LOCK_LIST_SHIFT    (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_CLOCK_ROOT_AUTHEN_LOCK_LIST(x)       (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_AUTHEN_LOCK_LIST_SHIFT)) & CCM_CLOCK_ROOT_AUTHEN_LOCK_LIST_MASK)

#define CCM_CLOCK_ROOT_AUTHEN_WHITE_LIST_MASK    (0xFFFF0000U)
#define CCM_CLOCK_ROOT_AUTHEN_WHITE_LIST_SHIFT   (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_CLOCK_ROOT_AUTHEN_WHITE_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_CLOCK_ROOT_AUTHEN_WHITE_LIST_SHIFT)) & CCM_CLOCK_ROOT_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_CLOCK_ROOT_AUTHEN */
#define CCM_CLOCK_ROOT_AUTHEN_COUNT              (74U)

/*! @name OBSERVE_CONTROL - Observe control */
/*! @{ */

#define CCM_OBSERVE_CONTROL_SELECT_MASK          (0x1FFU)
#define CCM_OBSERVE_CONTROL_SELECT_SHIFT         (0U)
/*! SELECT - Observe signal selector */
#define CCM_OBSERVE_CONTROL_SELECT(x)            (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_SELECT_SHIFT)) & CCM_OBSERVE_CONTROL_SELECT_MASK)

#define CCM_OBSERVE_CONTROL_RAW_MASK             (0x1000U)
#define CCM_OBSERVE_CONTROL_RAW_SHIFT            (12U)
/*! RAW - Observe raw signal
 *  0b0..Select divided signal.
 *  0b1..Select raw signal.
 */
#define CCM_OBSERVE_CONTROL_RAW(x)               (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_RAW_SHIFT)) & CCM_OBSERVE_CONTROL_RAW_MASK)

#define CCM_OBSERVE_CONTROL_INV_MASK             (0x2000U)
#define CCM_OBSERVE_CONTROL_INV_SHIFT            (13U)
/*! INV
 *  0b0..Clock phase remain same.
 *  0b1..Invert clock phase before measurement or send to IO.
 */
#define CCM_OBSERVE_CONTROL_INV(x)               (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_INV_SHIFT)) & CCM_OBSERVE_CONTROL_INV_MASK)

#define CCM_OBSERVE_CONTROL_RESET_MASK           (0x8000U)
#define CCM_OBSERVE_CONTROL_RESET_SHIFT          (15U)
/*! RESET - Reset observe divider
 *  0b0..Reset deasserts
 *  0b1..Reset asserts
 */
#define CCM_OBSERVE_CONTROL_RESET(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_RESET_SHIFT)) & CCM_OBSERVE_CONTROL_RESET_MASK)

#define CCM_OBSERVE_CONTROL_DIVIDE_MASK          (0xFF0000U)
#define CCM_OBSERVE_CONTROL_DIVIDE_SHIFT         (16U)
/*! DIVIDE - Division factor of the divider for observed signal */
#define CCM_OBSERVE_CONTROL_DIVIDE(x)            (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_DIVIDE_SHIFT)) & CCM_OBSERVE_CONTROL_DIVIDE_MASK)

#define CCM_OBSERVE_CONTROL_OFF_MASK             (0x1000000U)
#define CCM_OBSERVE_CONTROL_OFF_SHIFT            (24U)
/*! OFF - Turn off
 *  0b0..observe slice is on
 *  0b1..observe slice is off
 */
#define CCM_OBSERVE_CONTROL_OFF(x)               (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_OFF_SHIFT)) & CCM_OBSERVE_CONTROL_OFF_MASK)
/*! @} */

/* The count of CCM_OBSERVE_CONTROL */
#define CCM_OBSERVE_CONTROL_COUNT                (2U)

/*! @name OBSERVE_CONTROL_SET - Observe control */
/*! @{ */

#define CCM_OBSERVE_CONTROL_SET_SELECT_MASK      (0x1FFU)
#define CCM_OBSERVE_CONTROL_SET_SELECT_SHIFT     (0U)
/*! SELECT - Observe signal selector */
#define CCM_OBSERVE_CONTROL_SET_SELECT(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_SET_SELECT_SHIFT)) & CCM_OBSERVE_CONTROL_SET_SELECT_MASK)

#define CCM_OBSERVE_CONTROL_SET_RAW_MASK         (0x1000U)
#define CCM_OBSERVE_CONTROL_SET_RAW_SHIFT        (12U)
/*! RAW - Observe raw signal */
#define CCM_OBSERVE_CONTROL_SET_RAW(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_SET_RAW_SHIFT)) & CCM_OBSERVE_CONTROL_SET_RAW_MASK)

#define CCM_OBSERVE_CONTROL_SET_INV_MASK         (0x2000U)
#define CCM_OBSERVE_CONTROL_SET_INV_SHIFT        (13U)
#define CCM_OBSERVE_CONTROL_SET_INV(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_SET_INV_SHIFT)) & CCM_OBSERVE_CONTROL_SET_INV_MASK)

#define CCM_OBSERVE_CONTROL_SET_RESET_MASK       (0x8000U)
#define CCM_OBSERVE_CONTROL_SET_RESET_SHIFT      (15U)
/*! RESET - Reset observe divider */
#define CCM_OBSERVE_CONTROL_SET_RESET(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_SET_RESET_SHIFT)) & CCM_OBSERVE_CONTROL_SET_RESET_MASK)

#define CCM_OBSERVE_CONTROL_SET_DIVIDE_MASK      (0xFF0000U)
#define CCM_OBSERVE_CONTROL_SET_DIVIDE_SHIFT     (16U)
/*! DIVIDE - Division factor of the divider for observed signal */
#define CCM_OBSERVE_CONTROL_SET_DIVIDE(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_SET_DIVIDE_SHIFT)) & CCM_OBSERVE_CONTROL_SET_DIVIDE_MASK)

#define CCM_OBSERVE_CONTROL_SET_OFF_MASK         (0x1000000U)
#define CCM_OBSERVE_CONTROL_SET_OFF_SHIFT        (24U)
/*! OFF - Turn off */
#define CCM_OBSERVE_CONTROL_SET_OFF(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_SET_OFF_SHIFT)) & CCM_OBSERVE_CONTROL_SET_OFF_MASK)
/*! @} */

/* The count of CCM_OBSERVE_CONTROL_SET */
#define CCM_OBSERVE_CONTROL_SET_COUNT            (2U)

/*! @name OBSERVE_CONTROL_CLR - Observe control */
/*! @{ */

#define CCM_OBSERVE_CONTROL_CLR_SELECT_MASK      (0x1FFU)
#define CCM_OBSERVE_CONTROL_CLR_SELECT_SHIFT     (0U)
/*! SELECT - Observe signal selector */
#define CCM_OBSERVE_CONTROL_CLR_SELECT(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_CLR_SELECT_SHIFT)) & CCM_OBSERVE_CONTROL_CLR_SELECT_MASK)

#define CCM_OBSERVE_CONTROL_CLR_RAW_MASK         (0x1000U)
#define CCM_OBSERVE_CONTROL_CLR_RAW_SHIFT        (12U)
/*! RAW - Observe raw signal */
#define CCM_OBSERVE_CONTROL_CLR_RAW(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_CLR_RAW_SHIFT)) & CCM_OBSERVE_CONTROL_CLR_RAW_MASK)

#define CCM_OBSERVE_CONTROL_CLR_INV_MASK         (0x2000U)
#define CCM_OBSERVE_CONTROL_CLR_INV_SHIFT        (13U)
#define CCM_OBSERVE_CONTROL_CLR_INV(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_CLR_INV_SHIFT)) & CCM_OBSERVE_CONTROL_CLR_INV_MASK)

#define CCM_OBSERVE_CONTROL_CLR_RESET_MASK       (0x8000U)
#define CCM_OBSERVE_CONTROL_CLR_RESET_SHIFT      (15U)
/*! RESET - Reset observe divider */
#define CCM_OBSERVE_CONTROL_CLR_RESET(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_CLR_RESET_SHIFT)) & CCM_OBSERVE_CONTROL_CLR_RESET_MASK)

#define CCM_OBSERVE_CONTROL_CLR_DIVIDE_MASK      (0xFF0000U)
#define CCM_OBSERVE_CONTROL_CLR_DIVIDE_SHIFT     (16U)
/*! DIVIDE - Division factor of the divider for observed signal */
#define CCM_OBSERVE_CONTROL_CLR_DIVIDE(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_CLR_DIVIDE_SHIFT)) & CCM_OBSERVE_CONTROL_CLR_DIVIDE_MASK)

#define CCM_OBSERVE_CONTROL_CLR_OFF_MASK         (0x1000000U)
#define CCM_OBSERVE_CONTROL_CLR_OFF_SHIFT        (24U)
/*! OFF - Turn off */
#define CCM_OBSERVE_CONTROL_CLR_OFF(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_CLR_OFF_SHIFT)) & CCM_OBSERVE_CONTROL_CLR_OFF_MASK)
/*! @} */

/* The count of CCM_OBSERVE_CONTROL_CLR */
#define CCM_OBSERVE_CONTROL_CLR_COUNT            (2U)

/*! @name OBSERVE_CONTROL_TOG - Observe control */
/*! @{ */

#define CCM_OBSERVE_CONTROL_TOG_SELECT_MASK      (0x1FFU)
#define CCM_OBSERVE_CONTROL_TOG_SELECT_SHIFT     (0U)
/*! SELECT - Observe signal selector */
#define CCM_OBSERVE_CONTROL_TOG_SELECT(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_TOG_SELECT_SHIFT)) & CCM_OBSERVE_CONTROL_TOG_SELECT_MASK)

#define CCM_OBSERVE_CONTROL_TOG_RAW_MASK         (0x1000U)
#define CCM_OBSERVE_CONTROL_TOG_RAW_SHIFT        (12U)
/*! RAW - Observe raw signal */
#define CCM_OBSERVE_CONTROL_TOG_RAW(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_TOG_RAW_SHIFT)) & CCM_OBSERVE_CONTROL_TOG_RAW_MASK)

#define CCM_OBSERVE_CONTROL_TOG_INV_MASK         (0x2000U)
#define CCM_OBSERVE_CONTROL_TOG_INV_SHIFT        (13U)
#define CCM_OBSERVE_CONTROL_TOG_INV(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_TOG_INV_SHIFT)) & CCM_OBSERVE_CONTROL_TOG_INV_MASK)

#define CCM_OBSERVE_CONTROL_TOG_RESET_MASK       (0x8000U)
#define CCM_OBSERVE_CONTROL_TOG_RESET_SHIFT      (15U)
/*! RESET - Reset observe divider */
#define CCM_OBSERVE_CONTROL_TOG_RESET(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_TOG_RESET_SHIFT)) & CCM_OBSERVE_CONTROL_TOG_RESET_MASK)

#define CCM_OBSERVE_CONTROL_TOG_DIVIDE_MASK      (0xFF0000U)
#define CCM_OBSERVE_CONTROL_TOG_DIVIDE_SHIFT     (16U)
/*! DIVIDE - Division factor of the divider for observed signal */
#define CCM_OBSERVE_CONTROL_TOG_DIVIDE(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_TOG_DIVIDE_SHIFT)) & CCM_OBSERVE_CONTROL_TOG_DIVIDE_MASK)

#define CCM_OBSERVE_CONTROL_TOG_OFF_MASK         (0x1000000U)
#define CCM_OBSERVE_CONTROL_TOG_OFF_SHIFT        (24U)
/*! OFF - Turn off */
#define CCM_OBSERVE_CONTROL_TOG_OFF(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_CONTROL_TOG_OFF_SHIFT)) & CCM_OBSERVE_CONTROL_TOG_OFF_MASK)
/*! @} */

/* The count of CCM_OBSERVE_CONTROL_TOG */
#define CCM_OBSERVE_CONTROL_TOG_COUNT            (2U)

/*! @name OBSERVE_STATUS - Observe status */
/*! @{ */

#define CCM_OBSERVE_STATUS_SELECT_MASK           (0x1FFU)
#define CCM_OBSERVE_STATUS_SELECT_SHIFT          (0U)
/*! SELECT - Observe signal selector */
#define CCM_OBSERVE_STATUS_SELECT(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_SELECT_SHIFT)) & CCM_OBSERVE_STATUS_SELECT_MASK)

#define CCM_OBSERVE_STATUS_RAW_MASK              (0x1000U)
#define CCM_OBSERVE_STATUS_RAW_SHIFT             (12U)
/*! RAW - Observe raw signal
 *  0b0..Select divided signal.
 *  0b1..Select raw signal.
 */
#define CCM_OBSERVE_STATUS_RAW(x)                (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_RAW_SHIFT)) & CCM_OBSERVE_STATUS_RAW_MASK)

#define CCM_OBSERVE_STATUS_INV_MASK              (0x2000U)
#define CCM_OBSERVE_STATUS_INV_SHIFT             (13U)
/*! INV - Invert
 *  0b0..Clock phase remain same.
 *  0b1..Invert clock phase before measurement or send to IO.
 */
#define CCM_OBSERVE_STATUS_INV(x)                (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_INV_SHIFT)) & CCM_OBSERVE_STATUS_INV_MASK)

#define CCM_OBSERVE_STATUS_RESET_MASK            (0x8000U)
#define CCM_OBSERVE_STATUS_RESET_SHIFT           (15U)
/*! RESET - Reset state
 *  0b1..Observe divider is in reset state
 *  0b0..Observe divider is not in reset state
 */
#define CCM_OBSERVE_STATUS_RESET(x)              (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_RESET_SHIFT)) & CCM_OBSERVE_STATUS_RESET_MASK)

#define CCM_OBSERVE_STATUS_DIVIDE_MASK           (0xFF0000U)
#define CCM_OBSERVE_STATUS_DIVIDE_SHIFT          (16U)
/*! DIVIDE - Divider for observe signal */
#define CCM_OBSERVE_STATUS_DIVIDE(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_DIVIDE_SHIFT)) & CCM_OBSERVE_STATUS_DIVIDE_MASK)

#define CCM_OBSERVE_STATUS_OFF_MASK              (0x1000000U)
#define CCM_OBSERVE_STATUS_OFF_SHIFT             (24U)
/*! OFF - Turn off slice
 *  0b0..observe slice is on
 *  0b1..observe slice is off
 */
#define CCM_OBSERVE_STATUS_OFF(x)                (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_OFF_SHIFT)) & CCM_OBSERVE_STATUS_OFF_MASK)

#define CCM_OBSERVE_STATUS_FREQ_MEASURE_DONE_MASK (0x2000000U)
#define CCM_OBSERVE_STATUS_FREQ_MEASURE_DONE_SHIFT (25U)
/*! FREQ_MEASURE_DONE - frequency measurement done flag
 *  0b0..Frequency measurement is on-going or not started
 *  0b1..Frequency measurement is done.
 */
#define CCM_OBSERVE_STATUS_FREQ_MEASURE_DONE(x)  (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_FREQ_MEASURE_DONE_SHIFT)) & CCM_OBSERVE_STATUS_FREQ_MEASURE_DONE_MASK)

#define CCM_OBSERVE_STATUS_BUSY_MASK             (0x10000000U)
#define CCM_OBSERVE_STATUS_BUSY_SHIFT            (28U)
/*! BUSY - Busy
 *  0b1..Current observe is busy
 *  0b0..Current observe is not busy
 */
#define CCM_OBSERVE_STATUS_BUSY(x)               (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_BUSY_SHIFT)) & CCM_OBSERVE_STATUS_BUSY_MASK)

#define CCM_OBSERVE_STATUS_UPDATED_FORWARD_MASK  (0x20000000U)
#define CCM_OBSERVE_STATUS_UPDATED_FORWARD_SHIFT (29U)
/*! UPDATED_FORWARD - Update Forward */
#define CCM_OBSERVE_STATUS_UPDATED_FORWARD(x)    (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_UPDATED_FORWARD_SHIFT)) & CCM_OBSERVE_STATUS_UPDATED_FORWARD_MASK)

#define CCM_OBSERVE_STATUS_CHANGING_MASK         (0x80000000U)
#define CCM_OBSERVE_STATUS_CHANGING_SHIFT        (31U)
/*! CHANGING - Busy */
#define CCM_OBSERVE_STATUS_CHANGING(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_STATUS_CHANGING_SHIFT)) & CCM_OBSERVE_STATUS_CHANGING_MASK)
/*! @} */

/* The count of CCM_OBSERVE_STATUS */
#define CCM_OBSERVE_STATUS_COUNT                 (2U)

/*! @name OBSERVE_AUTHEN - Observe access control */
/*! @{ */

#define CCM_OBSERVE_AUTHEN_TZ_USER_MASK          (0x100U)
#define CCM_OBSERVE_AUTHEN_TZ_USER_SHIFT         (8U)
/*! TZ_USER - User access permission
 *  0b1..Observe slice settings can be changed in user mode.
 *  0b0..Observe slice settings cannot be changed in user mode.
 */
#define CCM_OBSERVE_AUTHEN_TZ_USER(x)            (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_TZ_USER_SHIFT)) & CCM_OBSERVE_AUTHEN_TZ_USER_MASK)

#define CCM_OBSERVE_AUTHEN_TZ_NS_MASK            (0x200U)
#define CCM_OBSERVE_AUTHEN_TZ_NS_SHIFT           (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in non-secure mode.
 *  0b1..Can be changed in non-secure mode.
 */
#define CCM_OBSERVE_AUTHEN_TZ_NS(x)              (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_TZ_NS_SHIFT)) & CCM_OBSERVE_AUTHEN_TZ_NS_MASK)

#define CCM_OBSERVE_AUTHEN_LOCK_TZ_MASK          (0x800U)
#define CCM_OBSERVE_AUTHEN_LOCK_TZ_SHIFT         (11U)
/*! LOCK_TZ - Lock TrustZone setting
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_OBSERVE_AUTHEN_LOCK_TZ(x)            (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_LOCK_TZ_SHIFT)) & CCM_OBSERVE_AUTHEN_LOCK_TZ_MASK)

#define CCM_OBSERVE_AUTHEN_LOCK_LIST_MASK        (0x8000U)
#define CCM_OBSERVE_AUTHEN_LOCK_LIST_SHIFT       (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..White list is not locked.
 *  0b1..White list is locked.
 */
#define CCM_OBSERVE_AUTHEN_LOCK_LIST(x)          (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_LOCK_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_LOCK_LIST_MASK)

#define CCM_OBSERVE_AUTHEN_WHITE_LIST_MASK       (0xFFFF0000U)
#define CCM_OBSERVE_AUTHEN_WHITE_LIST_SHIFT      (16U)
/*! WHITE_LIST - Whitelist settings
 *  0b0000000000001111..All domain can change.
 *  0b0000000000000010..Domain 1 can change.
 *  0b0000000000000011..Domain 0 and domain 1 can change.
 *  0b0000000000000000..No domain can change.
 *  0b0000000000000100..Domain 2 can change.
 *  0b0000000000000001..Domain 0 can change.
 */
#define CCM_OBSERVE_AUTHEN_WHITE_LIST(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_WHITE_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_OBSERVE_AUTHEN */
#define CCM_OBSERVE_AUTHEN_COUNT                 (2U)

/*! @name OBSERVE_AUTHEN_SET - Observe access control */
/*! @{ */

#define CCM_OBSERVE_AUTHEN_SET_TZ_USER_MASK      (0x100U)
#define CCM_OBSERVE_AUTHEN_SET_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission */
#define CCM_OBSERVE_AUTHEN_SET_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_OBSERVE_AUTHEN_SET_TZ_USER_MASK)

#define CCM_OBSERVE_AUTHEN_SET_TZ_NS_MASK        (0x200U)
#define CCM_OBSERVE_AUTHEN_SET_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_OBSERVE_AUTHEN_SET_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_OBSERVE_AUTHEN_SET_TZ_NS_MASK)

#define CCM_OBSERVE_AUTHEN_SET_LOCK_TZ_MASK      (0x800U)
#define CCM_OBSERVE_AUTHEN_SET_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone setting */
#define CCM_OBSERVE_AUTHEN_SET_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_OBSERVE_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_OBSERVE_AUTHEN_SET_LOCK_LIST_MASK    (0x8000U)
#define CCM_OBSERVE_AUTHEN_SET_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_OBSERVE_AUTHEN_SET_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_OBSERVE_AUTHEN_SET_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_OBSERVE_AUTHEN_SET_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_OBSERVE_AUTHEN_SET_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_OBSERVE_AUTHEN_SET */
#define CCM_OBSERVE_AUTHEN_SET_COUNT             (2U)

/*! @name OBSERVE_AUTHEN_CLR - Observe access control */
/*! @{ */

#define CCM_OBSERVE_AUTHEN_CLR_TZ_USER_MASK      (0x100U)
#define CCM_OBSERVE_AUTHEN_CLR_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission */
#define CCM_OBSERVE_AUTHEN_CLR_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_OBSERVE_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_OBSERVE_AUTHEN_CLR_TZ_NS_MASK        (0x200U)
#define CCM_OBSERVE_AUTHEN_CLR_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_OBSERVE_AUTHEN_CLR_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_OBSERVE_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_OBSERVE_AUTHEN_CLR_LOCK_TZ_MASK      (0x800U)
#define CCM_OBSERVE_AUTHEN_CLR_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone setting */
#define CCM_OBSERVE_AUTHEN_CLR_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_OBSERVE_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_OBSERVE_AUTHEN_CLR_LOCK_LIST_MASK    (0x8000U)
#define CCM_OBSERVE_AUTHEN_CLR_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_OBSERVE_AUTHEN_CLR_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_OBSERVE_AUTHEN_CLR_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_OBSERVE_AUTHEN_CLR_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_OBSERVE_AUTHEN_CLR_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_OBSERVE_AUTHEN_CLR */
#define CCM_OBSERVE_AUTHEN_CLR_COUNT             (2U)

/*! @name OBSERVE_AUTHEN_TOG - Observe access control */
/*! @{ */

#define CCM_OBSERVE_AUTHEN_TOG_TZ_USER_MASK      (0x100U)
#define CCM_OBSERVE_AUTHEN_TOG_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission */
#define CCM_OBSERVE_AUTHEN_TOG_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_OBSERVE_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_OBSERVE_AUTHEN_TOG_TZ_NS_MASK        (0x200U)
#define CCM_OBSERVE_AUTHEN_TOG_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_OBSERVE_AUTHEN_TOG_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_OBSERVE_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_OBSERVE_AUTHEN_TOG_LOCK_TZ_MASK      (0x800U)
#define CCM_OBSERVE_AUTHEN_TOG_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone setting */
#define CCM_OBSERVE_AUTHEN_TOG_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_OBSERVE_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_OBSERVE_AUTHEN_TOG_LOCK_LIST_MASK    (0x8000U)
#define CCM_OBSERVE_AUTHEN_TOG_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_OBSERVE_AUTHEN_TOG_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_OBSERVE_AUTHEN_TOG_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_OBSERVE_AUTHEN_TOG_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_OBSERVE_AUTHEN_TOG_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_OBSERVE_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_OBSERVE_AUTHEN_TOG */
#define CCM_OBSERVE_AUTHEN_TOG_COUNT             (2U)

/*! @name OBSERVE_FREQUENCY_CURRENT - Current frequency detected */
/*! @{ */

#define CCM_OBSERVE_FREQUENCY_CURRENT_FREQUENCY_MASK (0xFFFFFFFFU)
#define CCM_OBSERVE_FREQUENCY_CURRENT_FREQUENCY_SHIFT (0U)
/*! FREQUENCY - Frequency */
#define CCM_OBSERVE_FREQUENCY_CURRENT_FREQUENCY(x) (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_FREQUENCY_CURRENT_FREQUENCY_SHIFT)) & CCM_OBSERVE_FREQUENCY_CURRENT_FREQUENCY_MASK)
/*! @} */

/* The count of CCM_OBSERVE_FREQUENCY_CURRENT */
#define CCM_OBSERVE_FREQUENCY_CURRENT_COUNT      (2U)

/*! @name OBSERVE_FREQUENCY_MIN - Minimum frequency detected */
/*! @{ */

#define CCM_OBSERVE_FREQUENCY_MIN_FREQUENCY_MASK (0xFFFFFFFFU)
#define CCM_OBSERVE_FREQUENCY_MIN_FREQUENCY_SHIFT (0U)
/*! FREQUENCY - Frequency */
#define CCM_OBSERVE_FREQUENCY_MIN_FREQUENCY(x)   (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_FREQUENCY_MIN_FREQUENCY_SHIFT)) & CCM_OBSERVE_FREQUENCY_MIN_FREQUENCY_MASK)
/*! @} */

/* The count of CCM_OBSERVE_FREQUENCY_MIN */
#define CCM_OBSERVE_FREQUENCY_MIN_COUNT          (2U)

/*! @name OBSERVE_FREQUENCY_MAX - Maximum frequency detected */
/*! @{ */

#define CCM_OBSERVE_FREQUENCY_MAX_FREQUENCY_MASK (0xFFFFFFFFU)
#define CCM_OBSERVE_FREQUENCY_MAX_FREQUENCY_SHIFT (0U)
/*! FREQUENCY - Frequency */
#define CCM_OBSERVE_FREQUENCY_MAX_FREQUENCY(x)   (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_FREQUENCY_MAX_FREQUENCY_SHIFT)) & CCM_OBSERVE_FREQUENCY_MAX_FREQUENCY_MASK)
/*! @} */

/* The count of CCM_OBSERVE_FREQUENCY_MAX */
#define CCM_OBSERVE_FREQUENCY_MAX_COUNT          (2U)

/*! @name OBSERVE_PERIOD_CURRENT - Current period time detected */
/*! @{ */

#define CCM_OBSERVE_PERIOD_CURRENT_PERIOD_MASK   (0xFFFFFFFFU)
#define CCM_OBSERVE_PERIOD_CURRENT_PERIOD_SHIFT  (0U)
/*! PERIOD - Period time */
#define CCM_OBSERVE_PERIOD_CURRENT_PERIOD(x)     (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_PERIOD_CURRENT_PERIOD_SHIFT)) & CCM_OBSERVE_PERIOD_CURRENT_PERIOD_MASK)
/*! @} */

/* The count of CCM_OBSERVE_PERIOD_CURRENT */
#define CCM_OBSERVE_PERIOD_CURRENT_COUNT         (2U)

/*! @name OBSERVE_PERIOD_MIN - Minimum period time detected */
/*! @{ */

#define CCM_OBSERVE_PERIOD_MIN_PERIOD_MASK       (0xFFFFFFFFU)
#define CCM_OBSERVE_PERIOD_MIN_PERIOD_SHIFT      (0U)
/*! PERIOD - Period time */
#define CCM_OBSERVE_PERIOD_MIN_PERIOD(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_PERIOD_MIN_PERIOD_SHIFT)) & CCM_OBSERVE_PERIOD_MIN_PERIOD_MASK)
/*! @} */

/* The count of CCM_OBSERVE_PERIOD_MIN */
#define CCM_OBSERVE_PERIOD_MIN_COUNT             (2U)

/*! @name OBSERVE_PERIOD_MAX - Maximum period time detected */
/*! @{ */

#define CCM_OBSERVE_PERIOD_MAX_PERIOD_MASK       (0xFFFFFFFFU)
#define CCM_OBSERVE_PERIOD_MAX_PERIOD_SHIFT      (0U)
/*! PERIOD - Period time */
#define CCM_OBSERVE_PERIOD_MAX_PERIOD(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_PERIOD_MAX_PERIOD_SHIFT)) & CCM_OBSERVE_PERIOD_MAX_PERIOD_MASK)
/*! @} */

/* The count of CCM_OBSERVE_PERIOD_MAX */
#define CCM_OBSERVE_PERIOD_MAX_COUNT             (2U)

/*! @name OBSERVE_HIGH_CURRENT - Current high level time detected */
/*! @{ */

#define CCM_OBSERVE_HIGH_CURRENT_HIGH_MASK       (0xFFFFFFFFU)
#define CCM_OBSERVE_HIGH_CURRENT_HIGH_SHIFT      (0U)
/*! HIGH - High level time */
#define CCM_OBSERVE_HIGH_CURRENT_HIGH(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_HIGH_CURRENT_HIGH_SHIFT)) & CCM_OBSERVE_HIGH_CURRENT_HIGH_MASK)
/*! @} */

/* The count of CCM_OBSERVE_HIGH_CURRENT */
#define CCM_OBSERVE_HIGH_CURRENT_COUNT           (2U)

/*! @name OBSERVE_HIGH_MIN - Minimum high level time detected */
/*! @{ */

#define CCM_OBSERVE_HIGH_MIN_HIGH_MASK           (0xFFFFFFFFU)
#define CCM_OBSERVE_HIGH_MIN_HIGH_SHIFT          (0U)
/*! HIGH - High level time */
#define CCM_OBSERVE_HIGH_MIN_HIGH(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_HIGH_MIN_HIGH_SHIFT)) & CCM_OBSERVE_HIGH_MIN_HIGH_MASK)
/*! @} */

/* The count of CCM_OBSERVE_HIGH_MIN */
#define CCM_OBSERVE_HIGH_MIN_COUNT               (2U)

/*! @name OBSERVE_HIGH_MAX - Maximum high level time detected */
/*! @{ */

#define CCM_OBSERVE_HIGH_MAX_HIGH_MASK           (0xFFFFFFFFU)
#define CCM_OBSERVE_HIGH_MAX_HIGH_SHIFT          (0U)
/*! HIGH - High level time */
#define CCM_OBSERVE_HIGH_MAX_HIGH(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_HIGH_MAX_HIGH_SHIFT)) & CCM_OBSERVE_HIGH_MAX_HIGH_MASK)
/*! @} */

/* The count of CCM_OBSERVE_HIGH_MAX */
#define CCM_OBSERVE_HIGH_MAX_COUNT               (2U)

/*! @name OBSERVE_LOW_CURRENT - Current high level time detected */
/*! @{ */

#define CCM_OBSERVE_LOW_CURRENT_LOW_MASK         (0xFFFFFFFFU)
#define CCM_OBSERVE_LOW_CURRENT_LOW_SHIFT        (0U)
/*! LOW - High level time */
#define CCM_OBSERVE_LOW_CURRENT_LOW(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_LOW_CURRENT_LOW_SHIFT)) & CCM_OBSERVE_LOW_CURRENT_LOW_MASK)
/*! @} */

/* The count of CCM_OBSERVE_LOW_CURRENT */
#define CCM_OBSERVE_LOW_CURRENT_COUNT            (2U)

/*! @name OBSERVE_LOW_MIN - Minimum high level time detected */
/*! @{ */

#define CCM_OBSERVE_LOW_MIN_LOW_MASK             (0xFFFFFFFFU)
#define CCM_OBSERVE_LOW_MIN_LOW_SHIFT            (0U)
/*! LOW - High level time */
#define CCM_OBSERVE_LOW_MIN_LOW(x)               (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_LOW_MIN_LOW_SHIFT)) & CCM_OBSERVE_LOW_MIN_LOW_MASK)
/*! @} */

/* The count of CCM_OBSERVE_LOW_MIN */
#define CCM_OBSERVE_LOW_MIN_COUNT                (2U)

/*! @name OBSERVE_LOW_MAX - Maximum high level time detected */
/*! @{ */

#define CCM_OBSERVE_LOW_MAX_LOW_MASK             (0xFFFFFFFFU)
#define CCM_OBSERVE_LOW_MAX_LOW_SHIFT            (0U)
/*! LOW - High level time */
#define CCM_OBSERVE_LOW_MAX_LOW(x)               (((uint32_t)(((uint32_t)(x)) << CCM_OBSERVE_LOW_MAX_LOW_SHIFT)) & CCM_OBSERVE_LOW_MAX_LOW_MASK)
/*! @} */

/* The count of CCM_OBSERVE_LOW_MAX */
#define CCM_OBSERVE_LOW_MAX_COUNT                (2U)

/*! @name GPR_SHARED0_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED0_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED0_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED0_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED0_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED0_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED0_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED0_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED0_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED0_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED0_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED0_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED0_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED0_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED0_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED0_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED0_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED0_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED0_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED0_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED0_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED0_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED0_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED0_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED0_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED0_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED0_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED0_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED0_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED0_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED0_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED0_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED0_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED0_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED0_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED0_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED0_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED0_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED0_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED0_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED0_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED0_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED0_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED0_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED0_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED0_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED0_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED0_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED0_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED0_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED0_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED0_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED0_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED0_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED1_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED1_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED1_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED1_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED1_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED1_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED1_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED1_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED1_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED1_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED1_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED1_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED1_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED1_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED1_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED1_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED1_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED1_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED1_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED1_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED1_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED1_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED1_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED1_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED1_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED1_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED1_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED1_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED1_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED1_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED1_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED1_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED1_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED1_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED1_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED1_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED1_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED1_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED1_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED1_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED1_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED1_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED1_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED1_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED1_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED1_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED1_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED1_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED1_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED1_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED1_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED1_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED1_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED1_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED2 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED2_m33_mask_cm7_MASK        (0x1U)
#define CCM_GPR_SHARED2_m33_mask_cm7_SHIFT       (0U)
/*! m33_mask_cm7 - m33_mask_cm7 */
#define CCM_GPR_SHARED2_m33_mask_cm7(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_cm7_SHIFT)) & CCM_GPR_SHARED2_m33_mask_cm7_MASK)

#define CCM_GPR_SHARED2_m33_mask_cm33_MASK       (0x2U)
#define CCM_GPR_SHARED2_m33_mask_cm33_SHIFT      (1U)
/*! m33_mask_cm33 - m33_mask_cm33 */
#define CCM_GPR_SHARED2_m33_mask_cm33(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_cm33_SHIFT)) & CCM_GPR_SHARED2_m33_mask_cm33_MASK)

#define CCM_GPR_SHARED2_m33_mask_edma3_MASK      (0x4U)
#define CCM_GPR_SHARED2_m33_mask_edma3_SHIFT     (2U)
/*! m33_mask_edma3 - m33_mask_edma3 */
#define CCM_GPR_SHARED2_m33_mask_edma3(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_edma3_SHIFT)) & CCM_GPR_SHARED2_m33_mask_edma3_MASK)

#define CCM_GPR_SHARED2_m33_mask_edma4_MASK      (0x8U)
#define CCM_GPR_SHARED2_m33_mask_edma4_SHIFT     (3U)
/*! m33_mask_edma4 - m33_mask_edma4 */
#define CCM_GPR_SHARED2_m33_mask_edma4(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_edma4_SHIFT)) & CCM_GPR_SHARED2_m33_mask_edma4_MASK)

#define CCM_GPR_SHARED2_m33_mask_netc_MASK       (0x10U)
#define CCM_GPR_SHARED2_m33_mask_netc_SHIFT      (4U)
/*! m33_mask_netc - m33_mask_netc */
#define CCM_GPR_SHARED2_m33_mask_netc(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_netc_SHIFT)) & CCM_GPR_SHARED2_m33_mask_netc_MASK)

#define CCM_GPR_SHARED2_m33_mask_sim_aon_MASK    (0x100U)
#define CCM_GPR_SHARED2_m33_mask_sim_aon_SHIFT   (8U)
/*! m33_mask_sim_aon - m33_mask_sim_aon */
#define CCM_GPR_SHARED2_m33_mask_sim_aon(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED2_m33_mask_sim_aon_MASK)

#define CCM_GPR_SHARED2_m33_mask_adc1_MASK       (0x200U)
#define CCM_GPR_SHARED2_m33_mask_adc1_SHIFT      (9U)
/*! m33_mask_adc1 - m33_mask_adc1 */
#define CCM_GPR_SHARED2_m33_mask_adc1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_adc1_SHIFT)) & CCM_GPR_SHARED2_m33_mask_adc1_MASK)

#define CCM_GPR_SHARED2_m33_mask_adc2_MASK       (0x400U)
#define CCM_GPR_SHARED2_m33_mask_adc2_SHIFT      (10U)
/*! m33_mask_adc2 - m33_mask_adc2 */
#define CCM_GPR_SHARED2_m33_mask_adc2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_adc2_SHIFT)) & CCM_GPR_SHARED2_m33_mask_adc2_MASK)

#define CCM_GPR_SHARED2_m33_mask_flexspi1_MASK   (0x800U)
#define CCM_GPR_SHARED2_m33_mask_flexspi1_SHIFT  (11U)
/*! m33_mask_flexspi1 - m33_mask_flexspi1 */
#define CCM_GPR_SHARED2_m33_mask_flexspi1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED2_m33_mask_flexspi1_MASK)

#define CCM_GPR_SHARED2_m33_mask_flexspi2_MASK   (0x1000U)
#define CCM_GPR_SHARED2_m33_mask_flexspi2_SHIFT  (12U)
/*! m33_mask_flexspi2 - m33_mask_flexspi2 */
#define CCM_GPR_SHARED2_m33_mask_flexspi2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED2_m33_mask_flexspi2_MASK)

#define CCM_GPR_SHARED2_m33_mask_trdc_MASK       (0x2000U)
#define CCM_GPR_SHARED2_m33_mask_trdc_SHIFT      (13U)
/*! m33_mask_trdc - m33_mask_trdc */
#define CCM_GPR_SHARED2_m33_mask_trdc(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_trdc_SHIFT)) & CCM_GPR_SHARED2_m33_mask_trdc_MASK)

#define CCM_GPR_SHARED2_m33_mask_semc_MASK       (0x4000U)
#define CCM_GPR_SHARED2_m33_mask_semc_SHIFT      (14U)
/*! m33_mask_semc - m33_mask_semc */
#define CCM_GPR_SHARED2_m33_mask_semc(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_semc_SHIFT)) & CCM_GPR_SHARED2_m33_mask_semc_MASK)

#define CCM_GPR_SHARED2_m33_mask_iee_MASK        (0x8000U)
#define CCM_GPR_SHARED2_m33_mask_iee_SHIFT       (15U)
/*! m33_mask_iee - m33_mask_iee */
#define CCM_GPR_SHARED2_m33_mask_iee(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_iee_SHIFT)) & CCM_GPR_SHARED2_m33_mask_iee_MASK)

#define CCM_GPR_SHARED2_m33_mask_gpio1_MASK      (0x10000U)
#define CCM_GPR_SHARED2_m33_mask_gpio1_SHIFT     (16U)
/*! m33_mask_gpio1 - m33_mask_gpio1 */
#define CCM_GPR_SHARED2_m33_mask_gpio1(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_gpio1_SHIFT)) & CCM_GPR_SHARED2_m33_mask_gpio1_MASK)

#define CCM_GPR_SHARED2_m33_mask_gpio2_MASK      (0x20000U)
#define CCM_GPR_SHARED2_m33_mask_gpio2_SHIFT     (17U)
/*! m33_mask_gpio2 - m33_mask_gpio2 */
#define CCM_GPR_SHARED2_m33_mask_gpio2(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_gpio2_SHIFT)) & CCM_GPR_SHARED2_m33_mask_gpio2_MASK)

#define CCM_GPR_SHARED2_m33_mask_gpio3_MASK      (0x40000U)
#define CCM_GPR_SHARED2_m33_mask_gpio3_SHIFT     (18U)
/*! m33_mask_gpio3 - m33_mask_gpio3 */
#define CCM_GPR_SHARED2_m33_mask_gpio3(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_gpio3_SHIFT)) & CCM_GPR_SHARED2_m33_mask_gpio3_MASK)

#define CCM_GPR_SHARED2_m33_mask_gpio4_MASK      (0x80000U)
#define CCM_GPR_SHARED2_m33_mask_gpio4_SHIFT     (19U)
/*! m33_mask_gpio4 - m33_mask_gpio4 */
#define CCM_GPR_SHARED2_m33_mask_gpio4(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_gpio4_SHIFT)) & CCM_GPR_SHARED2_m33_mask_gpio4_MASK)

#define CCM_GPR_SHARED2_m33_mask_gpio5_MASK      (0x100000U)
#define CCM_GPR_SHARED2_m33_mask_gpio5_SHIFT     (20U)
/*! m33_mask_gpio5 - m33_mask_gpio5 */
#define CCM_GPR_SHARED2_m33_mask_gpio5(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_gpio5_SHIFT)) & CCM_GPR_SHARED2_m33_mask_gpio5_MASK)

#define CCM_GPR_SHARED2_m33_mask_gpio6_MASK      (0x200000U)
#define CCM_GPR_SHARED2_m33_mask_gpio6_SHIFT     (21U)
/*! m33_mask_gpio6 - m33_mask_gpio6 */
#define CCM_GPR_SHARED2_m33_mask_gpio6(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_gpio6_SHIFT)) & CCM_GPR_SHARED2_m33_mask_gpio6_MASK)

#define CCM_GPR_SHARED2_m33_mask_flexio1_MASK    (0x400000U)
#define CCM_GPR_SHARED2_m33_mask_flexio1_SHIFT   (22U)
/*! m33_mask_flexio1 - m33_mask_flexio1 */
#define CCM_GPR_SHARED2_m33_mask_flexio1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_flexio1_SHIFT)) & CCM_GPR_SHARED2_m33_mask_flexio1_MASK)

#define CCM_GPR_SHARED2_m33_mask_flexio2_MASK    (0x800000U)
#define CCM_GPR_SHARED2_m33_mask_flexio2_SHIFT   (23U)
/*! m33_mask_flexio2 - m33_mask_flexio2 */
#define CCM_GPR_SHARED2_m33_mask_flexio2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_flexio2_SHIFT)) & CCM_GPR_SHARED2_m33_mask_flexio2_MASK)

#define CCM_GPR_SHARED2_m33_mask_lpit1_MASK      (0x1000000U)
#define CCM_GPR_SHARED2_m33_mask_lpit1_SHIFT     (24U)
/*! m33_mask_lpit1 - m33_mask_lpit1 */
#define CCM_GPR_SHARED2_m33_mask_lpit1(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_lpit1_SHIFT)) & CCM_GPR_SHARED2_m33_mask_lpit1_MASK)

#define CCM_GPR_SHARED2_m33_mask_lpit2_MASK      (0x2000000U)
#define CCM_GPR_SHARED2_m33_mask_lpit2_SHIFT     (25U)
/*! m33_mask_lpit2 - m33_mask_lpit2 */
#define CCM_GPR_SHARED2_m33_mask_lpit2(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_lpit2_SHIFT)) & CCM_GPR_SHARED2_m33_mask_lpit2_MASK)

#define CCM_GPR_SHARED2_m33_mask_lpit3_MASK      (0x4000000U)
#define CCM_GPR_SHARED2_m33_mask_lpit3_SHIFT     (26U)
/*! m33_mask_lpit3 - m33_mask_lpit3 */
#define CCM_GPR_SHARED2_m33_mask_lpit3(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_lpit3_SHIFT)) & CCM_GPR_SHARED2_m33_mask_lpit3_MASK)

#define CCM_GPR_SHARED2_m33_mask_tpm1_MASK       (0x8000000U)
#define CCM_GPR_SHARED2_m33_mask_tpm1_SHIFT      (27U)
/*! m33_mask_tpm1 - m33_mask_tpm1 */
#define CCM_GPR_SHARED2_m33_mask_tpm1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_tpm1_SHIFT)) & CCM_GPR_SHARED2_m33_mask_tpm1_MASK)

#define CCM_GPR_SHARED2_m33_mask_tpm2_MASK       (0x10000000U)
#define CCM_GPR_SHARED2_m33_mask_tpm2_SHIFT      (28U)
/*! m33_mask_tpm2 - m33_mask_tpm2 */
#define CCM_GPR_SHARED2_m33_mask_tpm2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_tpm2_SHIFT)) & CCM_GPR_SHARED2_m33_mask_tpm2_MASK)

#define CCM_GPR_SHARED2_m33_mask_tpm3_MASK       (0x20000000U)
#define CCM_GPR_SHARED2_m33_mask_tpm3_SHIFT      (29U)
/*! m33_mask_tpm3 - m33_mask_tpm3 */
#define CCM_GPR_SHARED2_m33_mask_tpm3(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_tpm3_SHIFT)) & CCM_GPR_SHARED2_m33_mask_tpm3_MASK)

#define CCM_GPR_SHARED2_m33_mask_tpm4_MASK       (0x40000000U)
#define CCM_GPR_SHARED2_m33_mask_tpm4_SHIFT      (30U)
/*! m33_mask_tpm4 - m33_mask_tpm4 */
#define CCM_GPR_SHARED2_m33_mask_tpm4(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_tpm4_SHIFT)) & CCM_GPR_SHARED2_m33_mask_tpm4_MASK)

#define CCM_GPR_SHARED2_m33_mask_tpm5_MASK       (0x80000000U)
#define CCM_GPR_SHARED2_m33_mask_tpm5_SHIFT      (31U)
/*! m33_mask_tpm5 - m33_mask_tpm5 */
#define CCM_GPR_SHARED2_m33_mask_tpm5(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_m33_mask_tpm5_SHIFT)) & CCM_GPR_SHARED2_m33_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED2_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED2_SET_m33_mask_cm7_MASK    (0x1U)
#define CCM_GPR_SHARED2_SET_m33_mask_cm7_SHIFT   (0U)
/*! m33_mask_cm7 - m33_mask_cm7 */
#define CCM_GPR_SHARED2_SET_m33_mask_cm7(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_cm7_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_cm7_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_cm33_MASK   (0x2U)
#define CCM_GPR_SHARED2_SET_m33_mask_cm33_SHIFT  (1U)
/*! m33_mask_cm33 - m33_mask_cm33 */
#define CCM_GPR_SHARED2_SET_m33_mask_cm33(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_cm33_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_cm33_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_edma3_MASK  (0x4U)
#define CCM_GPR_SHARED2_SET_m33_mask_edma3_SHIFT (2U)
/*! m33_mask_edma3 - m33_mask_edma3 */
#define CCM_GPR_SHARED2_SET_m33_mask_edma3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_edma3_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_edma3_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_edma4_MASK  (0x8U)
#define CCM_GPR_SHARED2_SET_m33_mask_edma4_SHIFT (3U)
/*! m33_mask_edma4 - m33_mask_edma4 */
#define CCM_GPR_SHARED2_SET_m33_mask_edma4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_edma4_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_edma4_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_netc_MASK   (0x10U)
#define CCM_GPR_SHARED2_SET_m33_mask_netc_SHIFT  (4U)
/*! m33_mask_netc - m33_mask_netc */
#define CCM_GPR_SHARED2_SET_m33_mask_netc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_netc_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_netc_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_sim_aon_MASK (0x100U)
#define CCM_GPR_SHARED2_SET_m33_mask_sim_aon_SHIFT (8U)
/*! m33_mask_sim_aon - m33_mask_sim_aon */
#define CCM_GPR_SHARED2_SET_m33_mask_sim_aon(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_sim_aon_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_adc1_MASK   (0x200U)
#define CCM_GPR_SHARED2_SET_m33_mask_adc1_SHIFT  (9U)
/*! m33_mask_adc1 - m33_mask_adc1 */
#define CCM_GPR_SHARED2_SET_m33_mask_adc1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_adc1_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_adc1_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_adc2_MASK   (0x400U)
#define CCM_GPR_SHARED2_SET_m33_mask_adc2_SHIFT  (10U)
/*! m33_mask_adc2 - m33_mask_adc2 */
#define CCM_GPR_SHARED2_SET_m33_mask_adc2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_adc2_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_adc2_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_flexspi1_MASK (0x800U)
#define CCM_GPR_SHARED2_SET_m33_mask_flexspi1_SHIFT (11U)
/*! m33_mask_flexspi1 - m33_mask_flexspi1 */
#define CCM_GPR_SHARED2_SET_m33_mask_flexspi1(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_flexspi1_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_flexspi2_MASK (0x1000U)
#define CCM_GPR_SHARED2_SET_m33_mask_flexspi2_SHIFT (12U)
/*! m33_mask_flexspi2 - m33_mask_flexspi2 */
#define CCM_GPR_SHARED2_SET_m33_mask_flexspi2(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_flexspi2_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_trdc_MASK   (0x2000U)
#define CCM_GPR_SHARED2_SET_m33_mask_trdc_SHIFT  (13U)
/*! m33_mask_trdc - m33_mask_trdc */
#define CCM_GPR_SHARED2_SET_m33_mask_trdc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_trdc_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_trdc_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_semc_MASK   (0x4000U)
#define CCM_GPR_SHARED2_SET_m33_mask_semc_SHIFT  (14U)
/*! m33_mask_semc - m33_mask_semc */
#define CCM_GPR_SHARED2_SET_m33_mask_semc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_semc_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_semc_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_iee_MASK    (0x8000U)
#define CCM_GPR_SHARED2_SET_m33_mask_iee_SHIFT   (15U)
/*! m33_mask_iee - m33_mask_iee */
#define CCM_GPR_SHARED2_SET_m33_mask_iee(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_iee_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_iee_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_gpio1_MASK  (0x10000U)
#define CCM_GPR_SHARED2_SET_m33_mask_gpio1_SHIFT (16U)
/*! m33_mask_gpio1 - m33_mask_gpio1 */
#define CCM_GPR_SHARED2_SET_m33_mask_gpio1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_gpio1_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_gpio1_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_gpio2_MASK  (0x20000U)
#define CCM_GPR_SHARED2_SET_m33_mask_gpio2_SHIFT (17U)
/*! m33_mask_gpio2 - m33_mask_gpio2 */
#define CCM_GPR_SHARED2_SET_m33_mask_gpio2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_gpio2_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_gpio2_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_gpio3_MASK  (0x40000U)
#define CCM_GPR_SHARED2_SET_m33_mask_gpio3_SHIFT (18U)
/*! m33_mask_gpio3 - m33_mask_gpio3 */
#define CCM_GPR_SHARED2_SET_m33_mask_gpio3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_gpio3_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_gpio3_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_gpio4_MASK  (0x80000U)
#define CCM_GPR_SHARED2_SET_m33_mask_gpio4_SHIFT (19U)
/*! m33_mask_gpio4 - m33_mask_gpio4 */
#define CCM_GPR_SHARED2_SET_m33_mask_gpio4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_gpio4_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_gpio4_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_gpio5_MASK  (0x100000U)
#define CCM_GPR_SHARED2_SET_m33_mask_gpio5_SHIFT (20U)
/*! m33_mask_gpio5 - m33_mask_gpio5 */
#define CCM_GPR_SHARED2_SET_m33_mask_gpio5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_gpio5_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_gpio5_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_gpio6_MASK  (0x200000U)
#define CCM_GPR_SHARED2_SET_m33_mask_gpio6_SHIFT (21U)
/*! m33_mask_gpio6 - m33_mask_gpio6 */
#define CCM_GPR_SHARED2_SET_m33_mask_gpio6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_gpio6_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_gpio6_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_flexio1_MASK (0x400000U)
#define CCM_GPR_SHARED2_SET_m33_mask_flexio1_SHIFT (22U)
/*! m33_mask_flexio1 - m33_mask_flexio1 */
#define CCM_GPR_SHARED2_SET_m33_mask_flexio1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_flexio1_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_flexio1_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_flexio2_MASK (0x800000U)
#define CCM_GPR_SHARED2_SET_m33_mask_flexio2_SHIFT (23U)
/*! m33_mask_flexio2 - m33_mask_flexio2 */
#define CCM_GPR_SHARED2_SET_m33_mask_flexio2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_flexio2_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_flexio2_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_lpit1_MASK  (0x1000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_lpit1_SHIFT (24U)
/*! m33_mask_lpit1 - m33_mask_lpit1 */
#define CCM_GPR_SHARED2_SET_m33_mask_lpit1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_lpit1_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_lpit1_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_lpit2_MASK  (0x2000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_lpit2_SHIFT (25U)
/*! m33_mask_lpit2 - m33_mask_lpit2 */
#define CCM_GPR_SHARED2_SET_m33_mask_lpit2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_lpit2_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_lpit2_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_lpit3_MASK  (0x4000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_lpit3_SHIFT (26U)
/*! m33_mask_lpit3 - m33_mask_lpit3 */
#define CCM_GPR_SHARED2_SET_m33_mask_lpit3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_lpit3_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_lpit3_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_tpm1_MASK   (0x8000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_tpm1_SHIFT  (27U)
/*! m33_mask_tpm1 - m33_mask_tpm1 */
#define CCM_GPR_SHARED2_SET_m33_mask_tpm1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_tpm1_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_tpm1_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_tpm2_MASK   (0x10000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_tpm2_SHIFT  (28U)
/*! m33_mask_tpm2 - m33_mask_tpm2 */
#define CCM_GPR_SHARED2_SET_m33_mask_tpm2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_tpm2_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_tpm2_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_tpm3_MASK   (0x20000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_tpm3_SHIFT  (29U)
/*! m33_mask_tpm3 - m33_mask_tpm3 */
#define CCM_GPR_SHARED2_SET_m33_mask_tpm3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_tpm3_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_tpm3_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_tpm4_MASK   (0x40000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_tpm4_SHIFT  (30U)
/*! m33_mask_tpm4 - m33_mask_tpm4 */
#define CCM_GPR_SHARED2_SET_m33_mask_tpm4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_tpm4_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_tpm4_MASK)

#define CCM_GPR_SHARED2_SET_m33_mask_tpm5_MASK   (0x80000000U)
#define CCM_GPR_SHARED2_SET_m33_mask_tpm5_SHIFT  (31U)
/*! m33_mask_tpm5 - m33_mask_tpm5 */
#define CCM_GPR_SHARED2_SET_m33_mask_tpm5(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_SET_m33_mask_tpm5_SHIFT)) & CCM_GPR_SHARED2_SET_m33_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED2_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED2_CLR_m33_mask_cm7_MASK    (0x1U)
#define CCM_GPR_SHARED2_CLR_m33_mask_cm7_SHIFT   (0U)
/*! m33_mask_cm7 - m33_mask_cm7 */
#define CCM_GPR_SHARED2_CLR_m33_mask_cm7(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_cm7_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_cm7_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_cm33_MASK   (0x2U)
#define CCM_GPR_SHARED2_CLR_m33_mask_cm33_SHIFT  (1U)
/*! m33_mask_cm33 - m33_mask_cm33 */
#define CCM_GPR_SHARED2_CLR_m33_mask_cm33(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_cm33_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_cm33_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_edma3_MASK  (0x4U)
#define CCM_GPR_SHARED2_CLR_m33_mask_edma3_SHIFT (2U)
/*! m33_mask_edma3 - m33_mask_edma3 */
#define CCM_GPR_SHARED2_CLR_m33_mask_edma3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_edma3_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_edma3_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_edma4_MASK  (0x8U)
#define CCM_GPR_SHARED2_CLR_m33_mask_edma4_SHIFT (3U)
/*! m33_mask_edma4 - m33_mask_edma4 */
#define CCM_GPR_SHARED2_CLR_m33_mask_edma4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_edma4_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_edma4_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_netc_MASK   (0x10U)
#define CCM_GPR_SHARED2_CLR_m33_mask_netc_SHIFT  (4U)
/*! m33_mask_netc - m33_mask_netc */
#define CCM_GPR_SHARED2_CLR_m33_mask_netc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_netc_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_netc_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_sim_aon_MASK (0x100U)
#define CCM_GPR_SHARED2_CLR_m33_mask_sim_aon_SHIFT (8U)
/*! m33_mask_sim_aon - m33_mask_sim_aon */
#define CCM_GPR_SHARED2_CLR_m33_mask_sim_aon(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_sim_aon_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_adc1_MASK   (0x200U)
#define CCM_GPR_SHARED2_CLR_m33_mask_adc1_SHIFT  (9U)
/*! m33_mask_adc1 - m33_mask_adc1 */
#define CCM_GPR_SHARED2_CLR_m33_mask_adc1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_adc1_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_adc1_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_adc2_MASK   (0x400U)
#define CCM_GPR_SHARED2_CLR_m33_mask_adc2_SHIFT  (10U)
/*! m33_mask_adc2 - m33_mask_adc2 */
#define CCM_GPR_SHARED2_CLR_m33_mask_adc2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_adc2_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_adc2_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_flexspi1_MASK (0x800U)
#define CCM_GPR_SHARED2_CLR_m33_mask_flexspi1_SHIFT (11U)
/*! m33_mask_flexspi1 - m33_mask_flexspi1 */
#define CCM_GPR_SHARED2_CLR_m33_mask_flexspi1(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_flexspi1_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_flexspi2_MASK (0x1000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_flexspi2_SHIFT (12U)
/*! m33_mask_flexspi2 - m33_mask_flexspi2 */
#define CCM_GPR_SHARED2_CLR_m33_mask_flexspi2(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_flexspi2_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_trdc_MASK   (0x2000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_trdc_SHIFT  (13U)
/*! m33_mask_trdc - m33_mask_trdc */
#define CCM_GPR_SHARED2_CLR_m33_mask_trdc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_trdc_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_trdc_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_semc_MASK   (0x4000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_semc_SHIFT  (14U)
/*! m33_mask_semc - m33_mask_semc */
#define CCM_GPR_SHARED2_CLR_m33_mask_semc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_semc_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_semc_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_iee_MASK    (0x8000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_iee_SHIFT   (15U)
/*! m33_mask_iee - m33_mask_iee */
#define CCM_GPR_SHARED2_CLR_m33_mask_iee(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_iee_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_iee_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_gpio1_MASK  (0x10000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio1_SHIFT (16U)
/*! m33_mask_gpio1 - m33_mask_gpio1 */
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_gpio1_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_gpio1_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_gpio2_MASK  (0x20000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio2_SHIFT (17U)
/*! m33_mask_gpio2 - m33_mask_gpio2 */
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_gpio2_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_gpio2_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_gpio3_MASK  (0x40000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio3_SHIFT (18U)
/*! m33_mask_gpio3 - m33_mask_gpio3 */
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_gpio3_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_gpio3_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_gpio4_MASK  (0x80000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio4_SHIFT (19U)
/*! m33_mask_gpio4 - m33_mask_gpio4 */
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_gpio4_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_gpio4_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_gpio5_MASK  (0x100000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio5_SHIFT (20U)
/*! m33_mask_gpio5 - m33_mask_gpio5 */
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_gpio5_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_gpio5_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_gpio6_MASK  (0x200000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio6_SHIFT (21U)
/*! m33_mask_gpio6 - m33_mask_gpio6 */
#define CCM_GPR_SHARED2_CLR_m33_mask_gpio6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_gpio6_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_gpio6_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_flexio1_MASK (0x400000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_flexio1_SHIFT (22U)
/*! m33_mask_flexio1 - m33_mask_flexio1 */
#define CCM_GPR_SHARED2_CLR_m33_mask_flexio1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_flexio1_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_flexio1_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_flexio2_MASK (0x800000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_flexio2_SHIFT (23U)
/*! m33_mask_flexio2 - m33_mask_flexio2 */
#define CCM_GPR_SHARED2_CLR_m33_mask_flexio2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_flexio2_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_flexio2_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_lpit1_MASK  (0x1000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_lpit1_SHIFT (24U)
/*! m33_mask_lpit1 - m33_mask_lpit1 */
#define CCM_GPR_SHARED2_CLR_m33_mask_lpit1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_lpit1_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_lpit1_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_lpit2_MASK  (0x2000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_lpit2_SHIFT (25U)
/*! m33_mask_lpit2 - m33_mask_lpit2 */
#define CCM_GPR_SHARED2_CLR_m33_mask_lpit2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_lpit2_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_lpit2_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_lpit3_MASK  (0x4000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_lpit3_SHIFT (26U)
/*! m33_mask_lpit3 - m33_mask_lpit3 */
#define CCM_GPR_SHARED2_CLR_m33_mask_lpit3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_lpit3_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_lpit3_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_tpm1_MASK   (0x8000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm1_SHIFT  (27U)
/*! m33_mask_tpm1 - m33_mask_tpm1 */
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_tpm1_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_tpm1_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_tpm2_MASK   (0x10000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm2_SHIFT  (28U)
/*! m33_mask_tpm2 - m33_mask_tpm2 */
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_tpm2_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_tpm2_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_tpm3_MASK   (0x20000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm3_SHIFT  (29U)
/*! m33_mask_tpm3 - m33_mask_tpm3 */
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_tpm3_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_tpm3_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_tpm4_MASK   (0x40000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm4_SHIFT  (30U)
/*! m33_mask_tpm4 - m33_mask_tpm4 */
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_tpm4_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_tpm4_MASK)

#define CCM_GPR_SHARED2_CLR_m33_mask_tpm5_MASK   (0x80000000U)
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm5_SHIFT  (31U)
/*! m33_mask_tpm5 - m33_mask_tpm5 */
#define CCM_GPR_SHARED2_CLR_m33_mask_tpm5(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_CLR_m33_mask_tpm5_SHIFT)) & CCM_GPR_SHARED2_CLR_m33_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED2_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED2_TOG_m33_mask_cm7_MASK    (0x1U)
#define CCM_GPR_SHARED2_TOG_m33_mask_cm7_SHIFT   (0U)
/*! m33_mask_cm7 - m33_mask_cm7 */
#define CCM_GPR_SHARED2_TOG_m33_mask_cm7(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_cm7_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_cm7_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_cm33_MASK   (0x2U)
#define CCM_GPR_SHARED2_TOG_m33_mask_cm33_SHIFT  (1U)
/*! m33_mask_cm33 - m33_mask_cm33 */
#define CCM_GPR_SHARED2_TOG_m33_mask_cm33(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_cm33_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_cm33_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_edma3_MASK  (0x4U)
#define CCM_GPR_SHARED2_TOG_m33_mask_edma3_SHIFT (2U)
/*! m33_mask_edma3 - m33_mask_edma3 */
#define CCM_GPR_SHARED2_TOG_m33_mask_edma3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_edma3_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_edma3_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_edma4_MASK  (0x8U)
#define CCM_GPR_SHARED2_TOG_m33_mask_edma4_SHIFT (3U)
/*! m33_mask_edma4 - m33_mask_edma4 */
#define CCM_GPR_SHARED2_TOG_m33_mask_edma4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_edma4_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_edma4_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_netc_MASK   (0x10U)
#define CCM_GPR_SHARED2_TOG_m33_mask_netc_SHIFT  (4U)
/*! m33_mask_netc - m33_mask_netc */
#define CCM_GPR_SHARED2_TOG_m33_mask_netc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_netc_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_netc_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_sim_aon_MASK (0x100U)
#define CCM_GPR_SHARED2_TOG_m33_mask_sim_aon_SHIFT (8U)
/*! m33_mask_sim_aon - m33_mask_sim_aon */
#define CCM_GPR_SHARED2_TOG_m33_mask_sim_aon(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_sim_aon_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_adc1_MASK   (0x200U)
#define CCM_GPR_SHARED2_TOG_m33_mask_adc1_SHIFT  (9U)
/*! m33_mask_adc1 - m33_mask_adc1 */
#define CCM_GPR_SHARED2_TOG_m33_mask_adc1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_adc1_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_adc1_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_adc2_MASK   (0x400U)
#define CCM_GPR_SHARED2_TOG_m33_mask_adc2_SHIFT  (10U)
/*! m33_mask_adc2 - m33_mask_adc2 */
#define CCM_GPR_SHARED2_TOG_m33_mask_adc2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_adc2_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_adc2_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_flexspi1_MASK (0x800U)
#define CCM_GPR_SHARED2_TOG_m33_mask_flexspi1_SHIFT (11U)
/*! m33_mask_flexspi1 - m33_mask_flexspi1 */
#define CCM_GPR_SHARED2_TOG_m33_mask_flexspi1(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_flexspi1_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_flexspi2_MASK (0x1000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_flexspi2_SHIFT (12U)
/*! m33_mask_flexspi2 - m33_mask_flexspi2 */
#define CCM_GPR_SHARED2_TOG_m33_mask_flexspi2(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_flexspi2_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_trdc_MASK   (0x2000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_trdc_SHIFT  (13U)
/*! m33_mask_trdc - m33_mask_trdc */
#define CCM_GPR_SHARED2_TOG_m33_mask_trdc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_trdc_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_trdc_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_semc_MASK   (0x4000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_semc_SHIFT  (14U)
/*! m33_mask_semc - m33_mask_semc */
#define CCM_GPR_SHARED2_TOG_m33_mask_semc(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_semc_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_semc_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_iee_MASK    (0x8000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_iee_SHIFT   (15U)
/*! m33_mask_iee - m33_mask_iee */
#define CCM_GPR_SHARED2_TOG_m33_mask_iee(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_iee_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_iee_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_gpio1_MASK  (0x10000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio1_SHIFT (16U)
/*! m33_mask_gpio1 - m33_mask_gpio1 */
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_gpio1_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_gpio1_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_gpio2_MASK  (0x20000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio2_SHIFT (17U)
/*! m33_mask_gpio2 - m33_mask_gpio2 */
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_gpio2_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_gpio2_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_gpio3_MASK  (0x40000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio3_SHIFT (18U)
/*! m33_mask_gpio3 - m33_mask_gpio3 */
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_gpio3_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_gpio3_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_gpio4_MASK  (0x80000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio4_SHIFT (19U)
/*! m33_mask_gpio4 - m33_mask_gpio4 */
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_gpio4_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_gpio4_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_gpio5_MASK  (0x100000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio5_SHIFT (20U)
/*! m33_mask_gpio5 - m33_mask_gpio5 */
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_gpio5_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_gpio5_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_gpio6_MASK  (0x200000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio6_SHIFT (21U)
/*! m33_mask_gpio6 - m33_mask_gpio6 */
#define CCM_GPR_SHARED2_TOG_m33_mask_gpio6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_gpio6_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_gpio6_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_flexio1_MASK (0x400000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_flexio1_SHIFT (22U)
/*! m33_mask_flexio1 - m33_mask_flexio1 */
#define CCM_GPR_SHARED2_TOG_m33_mask_flexio1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_flexio1_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_flexio1_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_flexio2_MASK (0x800000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_flexio2_SHIFT (23U)
/*! m33_mask_flexio2 - m33_mask_flexio2 */
#define CCM_GPR_SHARED2_TOG_m33_mask_flexio2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_flexio2_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_flexio2_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_lpit1_MASK  (0x1000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_lpit1_SHIFT (24U)
/*! m33_mask_lpit1 - m33_mask_lpit1 */
#define CCM_GPR_SHARED2_TOG_m33_mask_lpit1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_lpit1_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_lpit1_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_lpit2_MASK  (0x2000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_lpit2_SHIFT (25U)
/*! m33_mask_lpit2 - m33_mask_lpit2 */
#define CCM_GPR_SHARED2_TOG_m33_mask_lpit2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_lpit2_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_lpit2_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_lpit3_MASK  (0x4000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_lpit3_SHIFT (26U)
/*! m33_mask_lpit3 - m33_mask_lpit3 */
#define CCM_GPR_SHARED2_TOG_m33_mask_lpit3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_lpit3_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_lpit3_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_tpm1_MASK   (0x8000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm1_SHIFT  (27U)
/*! m33_mask_tpm1 - m33_mask_tpm1 */
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_tpm1_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_tpm1_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_tpm2_MASK   (0x10000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm2_SHIFT  (28U)
/*! m33_mask_tpm2 - m33_mask_tpm2 */
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_tpm2_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_tpm2_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_tpm3_MASK   (0x20000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm3_SHIFT  (29U)
/*! m33_mask_tpm3 - m33_mask_tpm3 */
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_tpm3_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_tpm3_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_tpm4_MASK   (0x40000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm4_SHIFT  (30U)
/*! m33_mask_tpm4 - m33_mask_tpm4 */
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_tpm4_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_tpm4_MASK)

#define CCM_GPR_SHARED2_TOG_m33_mask_tpm5_MASK   (0x80000000U)
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm5_SHIFT  (31U)
/*! m33_mask_tpm5 - m33_mask_tpm5 */
#define CCM_GPR_SHARED2_TOG_m33_mask_tpm5(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_TOG_m33_mask_tpm5_SHIFT)) & CCM_GPR_SHARED2_TOG_m33_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED2_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED2_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED2_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED2_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED2_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED2_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED2_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED2_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED2_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED2_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED2_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED2_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED2_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED2_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED2_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED2_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED2_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED2_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED2_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED2_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED2_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED2_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED2_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED2_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED2_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED2_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED2_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED2_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED2_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED2_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED2_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED2_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED2_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED2_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED2_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED2_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED2_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED2_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED2_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED2_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED2_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED2_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED2_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED2_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED2_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED2_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED2_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED2_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED2_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED2_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED2_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED2_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED2_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED2_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED3 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED3_m33_mask_tpm6_MASK       (0x1U)
#define CCM_GPR_SHARED3_m33_mask_tpm6_SHIFT      (0U)
/*! m33_mask_tpm6 - m33_mask_tpm6 */
#define CCM_GPR_SHARED3_m33_mask_tpm6(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_tpm6_SHIFT)) & CCM_GPR_SHARED3_m33_mask_tpm6_MASK)

#define CCM_GPR_SHARED3_m33_mask_gpt1_MASK       (0x2U)
#define CCM_GPR_SHARED3_m33_mask_gpt1_SHIFT      (1U)
/*! m33_mask_gpt1 - m33_mask_gpt1 */
#define CCM_GPR_SHARED3_m33_mask_gpt1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_gpt1_SHIFT)) & CCM_GPR_SHARED3_m33_mask_gpt1_MASK)

#define CCM_GPR_SHARED3_m33_mask_gpt2_MASK       (0x4U)
#define CCM_GPR_SHARED3_m33_mask_gpt2_SHIFT      (2U)
/*! m33_mask_gpt2 - m33_mask_gpt2 */
#define CCM_GPR_SHARED3_m33_mask_gpt2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_gpt2_SHIFT)) & CCM_GPR_SHARED3_m33_mask_gpt2_MASK)

#define CCM_GPR_SHARED3_m33_mask_can1_MASK       (0x8U)
#define CCM_GPR_SHARED3_m33_mask_can1_SHIFT      (3U)
/*! m33_mask_can1 - m33_mask_can1 */
#define CCM_GPR_SHARED3_m33_mask_can1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_can1_SHIFT)) & CCM_GPR_SHARED3_m33_mask_can1_MASK)

#define CCM_GPR_SHARED3_m33_mask_can2_MASK       (0x10U)
#define CCM_GPR_SHARED3_m33_mask_can2_SHIFT      (4U)
/*! m33_mask_can2 - m33_mask_can2 */
#define CCM_GPR_SHARED3_m33_mask_can2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_can2_SHIFT)) & CCM_GPR_SHARED3_m33_mask_can2_MASK)

#define CCM_GPR_SHARED3_m33_mask_can3_MASK       (0x20U)
#define CCM_GPR_SHARED3_m33_mask_can3_SHIFT      (5U)
/*! m33_mask_can3 - m33_mask_can3 */
#define CCM_GPR_SHARED3_m33_mask_can3(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_can3_SHIFT)) & CCM_GPR_SHARED3_m33_mask_can3_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart1_MASK    (0x40U)
#define CCM_GPR_SHARED3_m33_mask_lpuart1_SHIFT   (6U)
/*! m33_mask_lpuart1 - m33_mask_lpuart1 */
#define CCM_GPR_SHARED3_m33_mask_lpuart1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart1_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart2_MASK    (0x80U)
#define CCM_GPR_SHARED3_m33_mask_lpuart2_SHIFT   (7U)
/*! m33_mask_lpuart2 - m33_mask_lpuart2 */
#define CCM_GPR_SHARED3_m33_mask_lpuart2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart2_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart3_MASK    (0x100U)
#define CCM_GPR_SHARED3_m33_mask_lpuart3_SHIFT   (8U)
/*! m33_mask_lpuart3 - m33_mask_lpuart3 */
#define CCM_GPR_SHARED3_m33_mask_lpuart3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart3_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart4_MASK    (0x200U)
#define CCM_GPR_SHARED3_m33_mask_lpuart4_SHIFT   (9U)
/*! m33_mask_lpuart4 - m33_mask_lpuart4 */
#define CCM_GPR_SHARED3_m33_mask_lpuart4(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart4_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart5_MASK    (0x400U)
#define CCM_GPR_SHARED3_m33_mask_lpuart5_SHIFT   (10U)
/*! m33_mask_lpuart5 - m33_mask_lpuart5 */
#define CCM_GPR_SHARED3_m33_mask_lpuart5(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart5_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart6_MASK    (0x800U)
#define CCM_GPR_SHARED3_m33_mask_lpuart6_SHIFT   (11U)
/*! m33_mask_lpuart6 - m33_mask_lpuart6 */
#define CCM_GPR_SHARED3_m33_mask_lpuart6(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart6_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart7_MASK    (0x1000U)
#define CCM_GPR_SHARED3_m33_mask_lpuart7_SHIFT   (12U)
/*! m33_mask_lpuart7 - m33_mask_lpuart7 */
#define CCM_GPR_SHARED3_m33_mask_lpuart7(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart7_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart8_MASK    (0x2000U)
#define CCM_GPR_SHARED3_m33_mask_lpuart8_SHIFT   (13U)
/*! m33_mask_lpuart8 - m33_mask_lpuart8 */
#define CCM_GPR_SHARED3_m33_mask_lpuart8(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart8_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart9_MASK    (0x4000U)
#define CCM_GPR_SHARED3_m33_mask_lpuart9_SHIFT   (14U)
/*! m33_mask_lpuart9 - m33_mask_lpuart9 */
#define CCM_GPR_SHARED3_m33_mask_lpuart9(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart9_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart10_MASK   (0x8000U)
#define CCM_GPR_SHARED3_m33_mask_lpuart10_SHIFT  (15U)
/*! m33_mask_lpuart10 - m33_mask_lpuart10 */
#define CCM_GPR_SHARED3_m33_mask_lpuart10(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart10_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart11_MASK   (0x10000U)
#define CCM_GPR_SHARED3_m33_mask_lpuart11_SHIFT  (16U)
/*! m33_mask_lpuart11 - m33_mask_lpuart11 */
#define CCM_GPR_SHARED3_m33_mask_lpuart11(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart11_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpuart12_MASK   (0x20000U)
#define CCM_GPR_SHARED3_m33_mask_lpuart12_SHIFT  (17U)
/*! m33_mask_lpuart12 - m33_mask_lpuart12 */
#define CCM_GPR_SHARED3_m33_mask_lpuart12(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpuart12_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpi2c1_MASK     (0x40000U)
#define CCM_GPR_SHARED3_m33_mask_lpi2c1_SHIFT    (18U)
/*! m33_mask_lpi2c1 - m33_mask_lpi2c1 */
#define CCM_GPR_SHARED3_m33_mask_lpi2c1(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpi2c2_MASK     (0x80000U)
#define CCM_GPR_SHARED3_m33_mask_lpi2c2_SHIFT    (19U)
/*! m33_mask_lpi2c2 - m33_mask_lpi2c2 */
#define CCM_GPR_SHARED3_m33_mask_lpi2c2(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpi2c3_MASK     (0x100000U)
#define CCM_GPR_SHARED3_m33_mask_lpi2c3_SHIFT    (20U)
/*! m33_mask_lpi2c3 - m33_mask_lpi2c3 */
#define CCM_GPR_SHARED3_m33_mask_lpi2c3(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpi2c4_MASK     (0x200000U)
#define CCM_GPR_SHARED3_m33_mask_lpi2c4_SHIFT    (21U)
/*! m33_mask_lpi2c4 - m33_mask_lpi2c4 */
#define CCM_GPR_SHARED3_m33_mask_lpi2c4(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpi2c5_MASK     (0x400000U)
#define CCM_GPR_SHARED3_m33_mask_lpi2c5_SHIFT    (22U)
/*! m33_mask_lpi2c5 - m33_mask_lpi2c5 */
#define CCM_GPR_SHARED3_m33_mask_lpi2c5(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpi2c6_MASK     (0x800000U)
#define CCM_GPR_SHARED3_m33_mask_lpi2c6_SHIFT    (23U)
/*! m33_mask_lpi2c6 - m33_mask_lpi2c6 */
#define CCM_GPR_SHARED3_m33_mask_lpi2c6(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpspi1_MASK     (0x1000000U)
#define CCM_GPR_SHARED3_m33_mask_lpspi1_SHIFT    (24U)
/*! m33_mask_lpspi1 - m33_mask_lpspi1 */
#define CCM_GPR_SHARED3_m33_mask_lpspi1(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpspi1_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpspi2_MASK     (0x2000000U)
#define CCM_GPR_SHARED3_m33_mask_lpspi2_SHIFT    (25U)
/*! m33_mask_lpspi2 - m33_mask_lpspi2 */
#define CCM_GPR_SHARED3_m33_mask_lpspi2(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpspi2_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpspi3_MASK     (0x4000000U)
#define CCM_GPR_SHARED3_m33_mask_lpspi3_SHIFT    (26U)
/*! m33_mask_lpspi3 - m33_mask_lpspi3 */
#define CCM_GPR_SHARED3_m33_mask_lpspi3(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpspi3_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpspi4_MASK     (0x8000000U)
#define CCM_GPR_SHARED3_m33_mask_lpspi4_SHIFT    (27U)
/*! m33_mask_lpspi4 - m33_mask_lpspi4 */
#define CCM_GPR_SHARED3_m33_mask_lpspi4(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpspi4_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpspi5_MASK     (0x10000000U)
#define CCM_GPR_SHARED3_m33_mask_lpspi5_SHIFT    (28U)
/*! m33_mask_lpspi5 - m33_mask_lpspi5 */
#define CCM_GPR_SHARED3_m33_mask_lpspi5(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpspi5_MASK)

#define CCM_GPR_SHARED3_m33_mask_lpspi6_MASK     (0x20000000U)
#define CCM_GPR_SHARED3_m33_mask_lpspi6_SHIFT    (29U)
/*! m33_mask_lpspi6 - m33_mask_lpspi6 */
#define CCM_GPR_SHARED3_m33_mask_lpspi6(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED3_m33_mask_lpspi6_MASK)

#define CCM_GPR_SHARED3_m33_mask_sinc1_MASK      (0x40000000U)
#define CCM_GPR_SHARED3_m33_mask_sinc1_SHIFT     (30U)
/*! m33_mask_sinc1 - m33_mask_sinc1 */
#define CCM_GPR_SHARED3_m33_mask_sinc1(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_sinc1_SHIFT)) & CCM_GPR_SHARED3_m33_mask_sinc1_MASK)

#define CCM_GPR_SHARED3_m33_mask_sinc2_MASK      (0x80000000U)
#define CCM_GPR_SHARED3_m33_mask_sinc2_SHIFT     (31U)
/*! m33_mask_sinc2 - m33_mask_sinc2 */
#define CCM_GPR_SHARED3_m33_mask_sinc2(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_m33_mask_sinc2_SHIFT)) & CCM_GPR_SHARED3_m33_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED3_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED3_SET_m33_mask_tpm6_MASK   (0x1U)
#define CCM_GPR_SHARED3_SET_m33_mask_tpm6_SHIFT  (0U)
/*! m33_mask_tpm6 - m33_mask_tpm6 */
#define CCM_GPR_SHARED3_SET_m33_mask_tpm6(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_tpm6_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_tpm6_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_gpt1_MASK   (0x2U)
#define CCM_GPR_SHARED3_SET_m33_mask_gpt1_SHIFT  (1U)
/*! m33_mask_gpt1 - m33_mask_gpt1 */
#define CCM_GPR_SHARED3_SET_m33_mask_gpt1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_gpt1_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_gpt1_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_gpt2_MASK   (0x4U)
#define CCM_GPR_SHARED3_SET_m33_mask_gpt2_SHIFT  (2U)
/*! m33_mask_gpt2 - m33_mask_gpt2 */
#define CCM_GPR_SHARED3_SET_m33_mask_gpt2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_gpt2_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_gpt2_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_can1_MASK   (0x8U)
#define CCM_GPR_SHARED3_SET_m33_mask_can1_SHIFT  (3U)
/*! m33_mask_can1 - m33_mask_can1 */
#define CCM_GPR_SHARED3_SET_m33_mask_can1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_can1_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_can1_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_can2_MASK   (0x10U)
#define CCM_GPR_SHARED3_SET_m33_mask_can2_SHIFT  (4U)
/*! m33_mask_can2 - m33_mask_can2 */
#define CCM_GPR_SHARED3_SET_m33_mask_can2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_can2_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_can2_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_can3_MASK   (0x20U)
#define CCM_GPR_SHARED3_SET_m33_mask_can3_SHIFT  (5U)
/*! m33_mask_can3 - m33_mask_can3 */
#define CCM_GPR_SHARED3_SET_m33_mask_can3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_can3_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_can3_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart1_MASK (0x40U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart1_SHIFT (6U)
/*! m33_mask_lpuart1 - m33_mask_lpuart1 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart1_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart2_MASK (0x80U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart2_SHIFT (7U)
/*! m33_mask_lpuart2 - m33_mask_lpuart2 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart2_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart3_MASK (0x100U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart3_SHIFT (8U)
/*! m33_mask_lpuart3 - m33_mask_lpuart3 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart3(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart3_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart4_MASK (0x200U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart4_SHIFT (9U)
/*! m33_mask_lpuart4 - m33_mask_lpuart4 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart4(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart4_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart5_MASK (0x400U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart5_SHIFT (10U)
/*! m33_mask_lpuart5 - m33_mask_lpuart5 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart5(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart5_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart6_MASK (0x800U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart6_SHIFT (11U)
/*! m33_mask_lpuart6 - m33_mask_lpuart6 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart6(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart6_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart7_MASK (0x1000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart7_SHIFT (12U)
/*! m33_mask_lpuart7 - m33_mask_lpuart7 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart7(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart7_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart8_MASK (0x2000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart8_SHIFT (13U)
/*! m33_mask_lpuart8 - m33_mask_lpuart8 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart8(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart8_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart9_MASK (0x4000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart9_SHIFT (14U)
/*! m33_mask_lpuart9 - m33_mask_lpuart9 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart9(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart9_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart10_MASK (0x8000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart10_SHIFT (15U)
/*! m33_mask_lpuart10 - m33_mask_lpuart10 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart10(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart10_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart11_MASK (0x10000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart11_SHIFT (16U)
/*! m33_mask_lpuart11 - m33_mask_lpuart11 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart11(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart11_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpuart12_MASK (0x20000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart12_SHIFT (17U)
/*! m33_mask_lpuart12 - m33_mask_lpuart12 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpuart12(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpuart12_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c1_MASK (0x40000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c1_SHIFT (18U)
/*! m33_mask_lpi2c1 - m33_mask_lpi2c1 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c2_MASK (0x80000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c2_SHIFT (19U)
/*! m33_mask_lpi2c2 - m33_mask_lpi2c2 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c3_MASK (0x100000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c3_SHIFT (20U)
/*! m33_mask_lpi2c3 - m33_mask_lpi2c3 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c4_MASK (0x200000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c4_SHIFT (21U)
/*! m33_mask_lpi2c4 - m33_mask_lpi2c4 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c5_MASK (0x400000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c5_SHIFT (22U)
/*! m33_mask_lpi2c5 - m33_mask_lpi2c5 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c6_MASK (0x800000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c6_SHIFT (23U)
/*! m33_mask_lpi2c6 - m33_mask_lpi2c6 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpi2c6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpspi1_MASK (0x1000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi1_SHIFT (24U)
/*! m33_mask_lpspi1 - m33_mask_lpspi1 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpspi1_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpspi2_MASK (0x2000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi2_SHIFT (25U)
/*! m33_mask_lpspi2 - m33_mask_lpspi2 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpspi2_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpspi3_MASK (0x4000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi3_SHIFT (26U)
/*! m33_mask_lpspi3 - m33_mask_lpspi3 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpspi3_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpspi4_MASK (0x8000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi4_SHIFT (27U)
/*! m33_mask_lpspi4 - m33_mask_lpspi4 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpspi4_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpspi5_MASK (0x10000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi5_SHIFT (28U)
/*! m33_mask_lpspi5 - m33_mask_lpspi5 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpspi5_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_lpspi6_MASK (0x20000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi6_SHIFT (29U)
/*! m33_mask_lpspi6 - m33_mask_lpspi6 */
#define CCM_GPR_SHARED3_SET_m33_mask_lpspi6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_lpspi6_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_sinc1_MASK  (0x40000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_sinc1_SHIFT (30U)
/*! m33_mask_sinc1 - m33_mask_sinc1 */
#define CCM_GPR_SHARED3_SET_m33_mask_sinc1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_sinc1_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_sinc1_MASK)

#define CCM_GPR_SHARED3_SET_m33_mask_sinc2_MASK  (0x80000000U)
#define CCM_GPR_SHARED3_SET_m33_mask_sinc2_SHIFT (31U)
/*! m33_mask_sinc2 - m33_mask_sinc2 */
#define CCM_GPR_SHARED3_SET_m33_mask_sinc2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_SET_m33_mask_sinc2_SHIFT)) & CCM_GPR_SHARED3_SET_m33_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED3_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED3_CLR_m33_mask_tpm6_MASK   (0x1U)
#define CCM_GPR_SHARED3_CLR_m33_mask_tpm6_SHIFT  (0U)
/*! m33_mask_tpm6 - m33_mask_tpm6 */
#define CCM_GPR_SHARED3_CLR_m33_mask_tpm6(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_tpm6_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_tpm6_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_gpt1_MASK   (0x2U)
#define CCM_GPR_SHARED3_CLR_m33_mask_gpt1_SHIFT  (1U)
/*! m33_mask_gpt1 - m33_mask_gpt1 */
#define CCM_GPR_SHARED3_CLR_m33_mask_gpt1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_gpt1_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_gpt1_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_gpt2_MASK   (0x4U)
#define CCM_GPR_SHARED3_CLR_m33_mask_gpt2_SHIFT  (2U)
/*! m33_mask_gpt2 - m33_mask_gpt2 */
#define CCM_GPR_SHARED3_CLR_m33_mask_gpt2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_gpt2_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_gpt2_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_can1_MASK   (0x8U)
#define CCM_GPR_SHARED3_CLR_m33_mask_can1_SHIFT  (3U)
/*! m33_mask_can1 - m33_mask_can1 */
#define CCM_GPR_SHARED3_CLR_m33_mask_can1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_can1_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_can1_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_can2_MASK   (0x10U)
#define CCM_GPR_SHARED3_CLR_m33_mask_can2_SHIFT  (4U)
/*! m33_mask_can2 - m33_mask_can2 */
#define CCM_GPR_SHARED3_CLR_m33_mask_can2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_can2_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_can2_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_can3_MASK   (0x20U)
#define CCM_GPR_SHARED3_CLR_m33_mask_can3_SHIFT  (5U)
/*! m33_mask_can3 - m33_mask_can3 */
#define CCM_GPR_SHARED3_CLR_m33_mask_can3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_can3_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_can3_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart1_MASK (0x40U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart1_SHIFT (6U)
/*! m33_mask_lpuart1 - m33_mask_lpuart1 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart1_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart2_MASK (0x80U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart2_SHIFT (7U)
/*! m33_mask_lpuart2 - m33_mask_lpuart2 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart2_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart3_MASK (0x100U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart3_SHIFT (8U)
/*! m33_mask_lpuart3 - m33_mask_lpuart3 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart3(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart3_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart4_MASK (0x200U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart4_SHIFT (9U)
/*! m33_mask_lpuart4 - m33_mask_lpuart4 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart4(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart4_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart5_MASK (0x400U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart5_SHIFT (10U)
/*! m33_mask_lpuart5 - m33_mask_lpuart5 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart5(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart5_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart6_MASK (0x800U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart6_SHIFT (11U)
/*! m33_mask_lpuart6 - m33_mask_lpuart6 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart6(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart6_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart7_MASK (0x1000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart7_SHIFT (12U)
/*! m33_mask_lpuart7 - m33_mask_lpuart7 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart7(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart7_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart8_MASK (0x2000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart8_SHIFT (13U)
/*! m33_mask_lpuart8 - m33_mask_lpuart8 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart8(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart8_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart9_MASK (0x4000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart9_SHIFT (14U)
/*! m33_mask_lpuart9 - m33_mask_lpuart9 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart9(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart9_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart10_MASK (0x8000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart10_SHIFT (15U)
/*! m33_mask_lpuart10 - m33_mask_lpuart10 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart10(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart10_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart11_MASK (0x10000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart11_SHIFT (16U)
/*! m33_mask_lpuart11 - m33_mask_lpuart11 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart11(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart11_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart12_MASK (0x20000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart12_SHIFT (17U)
/*! m33_mask_lpuart12 - m33_mask_lpuart12 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpuart12(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpuart12_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c1_MASK (0x40000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c1_SHIFT (18U)
/*! m33_mask_lpi2c1 - m33_mask_lpi2c1 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c2_MASK (0x80000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c2_SHIFT (19U)
/*! m33_mask_lpi2c2 - m33_mask_lpi2c2 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c3_MASK (0x100000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c3_SHIFT (20U)
/*! m33_mask_lpi2c3 - m33_mask_lpi2c3 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c4_MASK (0x200000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c4_SHIFT (21U)
/*! m33_mask_lpi2c4 - m33_mask_lpi2c4 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c5_MASK (0x400000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c5_SHIFT (22U)
/*! m33_mask_lpi2c5 - m33_mask_lpi2c5 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c6_MASK (0x800000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c6_SHIFT (23U)
/*! m33_mask_lpi2c6 - m33_mask_lpi2c6 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpi2c6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi1_MASK (0x1000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi1_SHIFT (24U)
/*! m33_mask_lpspi1 - m33_mask_lpspi1 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpspi1_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi2_MASK (0x2000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi2_SHIFT (25U)
/*! m33_mask_lpspi2 - m33_mask_lpspi2 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpspi2_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi3_MASK (0x4000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi3_SHIFT (26U)
/*! m33_mask_lpspi3 - m33_mask_lpspi3 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpspi3_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi4_MASK (0x8000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi4_SHIFT (27U)
/*! m33_mask_lpspi4 - m33_mask_lpspi4 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpspi4_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi5_MASK (0x10000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi5_SHIFT (28U)
/*! m33_mask_lpspi5 - m33_mask_lpspi5 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpspi5_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi6_MASK (0x20000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi6_SHIFT (29U)
/*! m33_mask_lpspi6 - m33_mask_lpspi6 */
#define CCM_GPR_SHARED3_CLR_m33_mask_lpspi6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_lpspi6_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_sinc1_MASK  (0x40000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_sinc1_SHIFT (30U)
/*! m33_mask_sinc1 - m33_mask_sinc1 */
#define CCM_GPR_SHARED3_CLR_m33_mask_sinc1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_sinc1_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_sinc1_MASK)

#define CCM_GPR_SHARED3_CLR_m33_mask_sinc2_MASK  (0x80000000U)
#define CCM_GPR_SHARED3_CLR_m33_mask_sinc2_SHIFT (31U)
/*! m33_mask_sinc2 - m33_mask_sinc2 */
#define CCM_GPR_SHARED3_CLR_m33_mask_sinc2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_CLR_m33_mask_sinc2_SHIFT)) & CCM_GPR_SHARED3_CLR_m33_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED3_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED3_TOG_m33_mask_tpm6_MASK   (0x1U)
#define CCM_GPR_SHARED3_TOG_m33_mask_tpm6_SHIFT  (0U)
/*! m33_mask_tpm6 - m33_mask_tpm6 */
#define CCM_GPR_SHARED3_TOG_m33_mask_tpm6(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_tpm6_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_tpm6_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_gpt1_MASK   (0x2U)
#define CCM_GPR_SHARED3_TOG_m33_mask_gpt1_SHIFT  (1U)
/*! m33_mask_gpt1 - m33_mask_gpt1 */
#define CCM_GPR_SHARED3_TOG_m33_mask_gpt1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_gpt1_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_gpt1_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_gpt2_MASK   (0x4U)
#define CCM_GPR_SHARED3_TOG_m33_mask_gpt2_SHIFT  (2U)
/*! m33_mask_gpt2 - m33_mask_gpt2 */
#define CCM_GPR_SHARED3_TOG_m33_mask_gpt2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_gpt2_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_gpt2_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_can1_MASK   (0x8U)
#define CCM_GPR_SHARED3_TOG_m33_mask_can1_SHIFT  (3U)
/*! m33_mask_can1 - m33_mask_can1 */
#define CCM_GPR_SHARED3_TOG_m33_mask_can1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_can1_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_can1_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_can2_MASK   (0x10U)
#define CCM_GPR_SHARED3_TOG_m33_mask_can2_SHIFT  (4U)
/*! m33_mask_can2 - m33_mask_can2 */
#define CCM_GPR_SHARED3_TOG_m33_mask_can2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_can2_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_can2_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_can3_MASK   (0x20U)
#define CCM_GPR_SHARED3_TOG_m33_mask_can3_SHIFT  (5U)
/*! m33_mask_can3 - m33_mask_can3 */
#define CCM_GPR_SHARED3_TOG_m33_mask_can3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_can3_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_can3_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart1_MASK (0x40U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart1_SHIFT (6U)
/*! m33_mask_lpuart1 - m33_mask_lpuart1 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart1_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart2_MASK (0x80U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart2_SHIFT (7U)
/*! m33_mask_lpuart2 - m33_mask_lpuart2 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart2_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart3_MASK (0x100U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart3_SHIFT (8U)
/*! m33_mask_lpuart3 - m33_mask_lpuart3 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart3(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart3_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart4_MASK (0x200U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart4_SHIFT (9U)
/*! m33_mask_lpuart4 - m33_mask_lpuart4 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart4(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart4_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart5_MASK (0x400U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart5_SHIFT (10U)
/*! m33_mask_lpuart5 - m33_mask_lpuart5 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart5(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart5_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart6_MASK (0x800U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart6_SHIFT (11U)
/*! m33_mask_lpuart6 - m33_mask_lpuart6 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart6(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart6_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart7_MASK (0x1000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart7_SHIFT (12U)
/*! m33_mask_lpuart7 - m33_mask_lpuart7 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart7(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart7_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart8_MASK (0x2000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart8_SHIFT (13U)
/*! m33_mask_lpuart8 - m33_mask_lpuart8 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart8(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart8_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart9_MASK (0x4000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart9_SHIFT (14U)
/*! m33_mask_lpuart9 - m33_mask_lpuart9 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart9(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart9_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart10_MASK (0x8000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart10_SHIFT (15U)
/*! m33_mask_lpuart10 - m33_mask_lpuart10 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart10(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart10_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart11_MASK (0x10000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart11_SHIFT (16U)
/*! m33_mask_lpuart11 - m33_mask_lpuart11 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart11(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart11_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart12_MASK (0x20000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart12_SHIFT (17U)
/*! m33_mask_lpuart12 - m33_mask_lpuart12 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpuart12(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpuart12_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c1_MASK (0x40000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c1_SHIFT (18U)
/*! m33_mask_lpi2c1 - m33_mask_lpi2c1 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c2_MASK (0x80000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c2_SHIFT (19U)
/*! m33_mask_lpi2c2 - m33_mask_lpi2c2 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c3_MASK (0x100000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c3_SHIFT (20U)
/*! m33_mask_lpi2c3 - m33_mask_lpi2c3 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c4_MASK (0x200000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c4_SHIFT (21U)
/*! m33_mask_lpi2c4 - m33_mask_lpi2c4 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c5_MASK (0x400000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c5_SHIFT (22U)
/*! m33_mask_lpi2c5 - m33_mask_lpi2c5 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c6_MASK (0x800000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c6_SHIFT (23U)
/*! m33_mask_lpi2c6 - m33_mask_lpi2c6 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpi2c6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi1_MASK (0x1000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi1_SHIFT (24U)
/*! m33_mask_lpspi1 - m33_mask_lpspi1 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpspi1_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi2_MASK (0x2000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi2_SHIFT (25U)
/*! m33_mask_lpspi2 - m33_mask_lpspi2 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpspi2_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi3_MASK (0x4000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi3_SHIFT (26U)
/*! m33_mask_lpspi3 - m33_mask_lpspi3 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpspi3_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi4_MASK (0x8000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi4_SHIFT (27U)
/*! m33_mask_lpspi4 - m33_mask_lpspi4 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpspi4_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi5_MASK (0x10000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi5_SHIFT (28U)
/*! m33_mask_lpspi5 - m33_mask_lpspi5 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpspi5_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi6_MASK (0x20000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi6_SHIFT (29U)
/*! m33_mask_lpspi6 - m33_mask_lpspi6 */
#define CCM_GPR_SHARED3_TOG_m33_mask_lpspi6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_lpspi6_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_sinc1_MASK  (0x40000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_sinc1_SHIFT (30U)
/*! m33_mask_sinc1 - m33_mask_sinc1 */
#define CCM_GPR_SHARED3_TOG_m33_mask_sinc1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_sinc1_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_sinc1_MASK)

#define CCM_GPR_SHARED3_TOG_m33_mask_sinc2_MASK  (0x80000000U)
#define CCM_GPR_SHARED3_TOG_m33_mask_sinc2_SHIFT (31U)
/*! m33_mask_sinc2 - m33_mask_sinc2 */
#define CCM_GPR_SHARED3_TOG_m33_mask_sinc2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_TOG_m33_mask_sinc2_SHIFT)) & CCM_GPR_SHARED3_TOG_m33_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED3_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED3_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED3_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED3_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED3_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED3_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED3_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED3_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED3_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED3_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED3_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED3_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED3_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED3_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED3_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED3_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED3_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED3_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED3_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED3_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED3_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED3_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED3_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED3_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED3_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED3_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED3_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED3_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED3_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED3_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED3_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED3_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED3_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED3_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED3_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED3_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED3_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED3_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED3_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED3_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED3_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED3_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED3_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED3_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED3_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED3_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED3_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED3_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED3_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED3_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED3_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED3_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED3_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED3_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED4 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED4_m33_mask_sinc3_MASK      (0x1U)
#define CCM_GPR_SHARED4_m33_mask_sinc3_SHIFT     (0U)
/*! m33_mask_sinc3 - m33_mask_sinc3 */
#define CCM_GPR_SHARED4_m33_mask_sinc3(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_m33_mask_sinc3_SHIFT)) & CCM_GPR_SHARED4_m33_mask_sinc3_MASK)

#define CCM_GPR_SHARED4_m33_mask_sai1_MASK       (0x2U)
#define CCM_GPR_SHARED4_m33_mask_sai1_SHIFT      (1U)
/*! m33_mask_sai1 - m33_mask_sai1 */
#define CCM_GPR_SHARED4_m33_mask_sai1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_m33_mask_sai1_SHIFT)) & CCM_GPR_SHARED4_m33_mask_sai1_MASK)

#define CCM_GPR_SHARED4_m33_mask_sai2_MASK       (0x4U)
#define CCM_GPR_SHARED4_m33_mask_sai2_SHIFT      (2U)
/*! m33_mask_sai2 - m33_mask_sai2 */
#define CCM_GPR_SHARED4_m33_mask_sai2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_m33_mask_sai2_SHIFT)) & CCM_GPR_SHARED4_m33_mask_sai2_MASK)

#define CCM_GPR_SHARED4_m33_mask_sai3_MASK       (0x8U)
#define CCM_GPR_SHARED4_m33_mask_sai3_SHIFT      (3U)
/*! m33_mask_sai3 - m33_mask_sai3 */
#define CCM_GPR_SHARED4_m33_mask_sai3(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_m33_mask_sai3_SHIFT)) & CCM_GPR_SHARED4_m33_mask_sai3_MASK)

#define CCM_GPR_SHARED4_m33_mask_sai4_MASK       (0x10U)
#define CCM_GPR_SHARED4_m33_mask_sai4_SHIFT      (4U)
/*! m33_mask_sai4 - m33_mask_sai4 */
#define CCM_GPR_SHARED4_m33_mask_sai4(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_m33_mask_sai4_SHIFT)) & CCM_GPR_SHARED4_m33_mask_sai4_MASK)

#define CCM_GPR_SHARED4_m33_mask_mic_MASK        (0x20U)
#define CCM_GPR_SHARED4_m33_mask_mic_SHIFT       (5U)
/*! m33_mask_mic - m33_mask_mic */
#define CCM_GPR_SHARED4_m33_mask_mic(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_m33_mask_mic_SHIFT)) & CCM_GPR_SHARED4_m33_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED4_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED4_SET_m33_mask_sinc3_MASK  (0x1U)
#define CCM_GPR_SHARED4_SET_m33_mask_sinc3_SHIFT (0U)
/*! m33_mask_sinc3 - m33_mask_sinc3 */
#define CCM_GPR_SHARED4_SET_m33_mask_sinc3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_SET_m33_mask_sinc3_SHIFT)) & CCM_GPR_SHARED4_SET_m33_mask_sinc3_MASK)

#define CCM_GPR_SHARED4_SET_m33_mask_sai1_MASK   (0x2U)
#define CCM_GPR_SHARED4_SET_m33_mask_sai1_SHIFT  (1U)
/*! m33_mask_sai1 - m33_mask_sai1 */
#define CCM_GPR_SHARED4_SET_m33_mask_sai1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_SET_m33_mask_sai1_SHIFT)) & CCM_GPR_SHARED4_SET_m33_mask_sai1_MASK)

#define CCM_GPR_SHARED4_SET_m33_mask_sai2_MASK   (0x4U)
#define CCM_GPR_SHARED4_SET_m33_mask_sai2_SHIFT  (2U)
/*! m33_mask_sai2 - m33_mask_sai2 */
#define CCM_GPR_SHARED4_SET_m33_mask_sai2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_SET_m33_mask_sai2_SHIFT)) & CCM_GPR_SHARED4_SET_m33_mask_sai2_MASK)

#define CCM_GPR_SHARED4_SET_m33_mask_sai3_MASK   (0x8U)
#define CCM_GPR_SHARED4_SET_m33_mask_sai3_SHIFT  (3U)
/*! m33_mask_sai3 - m33_mask_sai3 */
#define CCM_GPR_SHARED4_SET_m33_mask_sai3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_SET_m33_mask_sai3_SHIFT)) & CCM_GPR_SHARED4_SET_m33_mask_sai3_MASK)

#define CCM_GPR_SHARED4_SET_m33_mask_sai4_MASK   (0x10U)
#define CCM_GPR_SHARED4_SET_m33_mask_sai4_SHIFT  (4U)
/*! m33_mask_sai4 - m33_mask_sai4 */
#define CCM_GPR_SHARED4_SET_m33_mask_sai4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_SET_m33_mask_sai4_SHIFT)) & CCM_GPR_SHARED4_SET_m33_mask_sai4_MASK)

#define CCM_GPR_SHARED4_SET_m33_mask_mic_MASK    (0x20U)
#define CCM_GPR_SHARED4_SET_m33_mask_mic_SHIFT   (5U)
/*! m33_mask_mic - m33_mask_mic */
#define CCM_GPR_SHARED4_SET_m33_mask_mic(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_SET_m33_mask_mic_SHIFT)) & CCM_GPR_SHARED4_SET_m33_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED4_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED4_CLR_m33_mask_sinc3_MASK  (0x1U)
#define CCM_GPR_SHARED4_CLR_m33_mask_sinc3_SHIFT (0U)
/*! m33_mask_sinc3 - m33_mask_sinc3 */
#define CCM_GPR_SHARED4_CLR_m33_mask_sinc3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_CLR_m33_mask_sinc3_SHIFT)) & CCM_GPR_SHARED4_CLR_m33_mask_sinc3_MASK)

#define CCM_GPR_SHARED4_CLR_m33_mask_sai1_MASK   (0x2U)
#define CCM_GPR_SHARED4_CLR_m33_mask_sai1_SHIFT  (1U)
/*! m33_mask_sai1 - m33_mask_sai1 */
#define CCM_GPR_SHARED4_CLR_m33_mask_sai1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_CLR_m33_mask_sai1_SHIFT)) & CCM_GPR_SHARED4_CLR_m33_mask_sai1_MASK)

#define CCM_GPR_SHARED4_CLR_m33_mask_sai2_MASK   (0x4U)
#define CCM_GPR_SHARED4_CLR_m33_mask_sai2_SHIFT  (2U)
/*! m33_mask_sai2 - m33_mask_sai2 */
#define CCM_GPR_SHARED4_CLR_m33_mask_sai2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_CLR_m33_mask_sai2_SHIFT)) & CCM_GPR_SHARED4_CLR_m33_mask_sai2_MASK)

#define CCM_GPR_SHARED4_CLR_m33_mask_sai3_MASK   (0x8U)
#define CCM_GPR_SHARED4_CLR_m33_mask_sai3_SHIFT  (3U)
/*! m33_mask_sai3 - m33_mask_sai3 */
#define CCM_GPR_SHARED4_CLR_m33_mask_sai3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_CLR_m33_mask_sai3_SHIFT)) & CCM_GPR_SHARED4_CLR_m33_mask_sai3_MASK)

#define CCM_GPR_SHARED4_CLR_m33_mask_sai4_MASK   (0x10U)
#define CCM_GPR_SHARED4_CLR_m33_mask_sai4_SHIFT  (4U)
/*! m33_mask_sai4 - m33_mask_sai4 */
#define CCM_GPR_SHARED4_CLR_m33_mask_sai4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_CLR_m33_mask_sai4_SHIFT)) & CCM_GPR_SHARED4_CLR_m33_mask_sai4_MASK)

#define CCM_GPR_SHARED4_CLR_m33_mask_mic_MASK    (0x20U)
#define CCM_GPR_SHARED4_CLR_m33_mask_mic_SHIFT   (5U)
/*! m33_mask_mic - m33_mask_mic */
#define CCM_GPR_SHARED4_CLR_m33_mask_mic(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_CLR_m33_mask_mic_SHIFT)) & CCM_GPR_SHARED4_CLR_m33_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED4_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED4_TOG_m33_mask_sinc3_MASK  (0x1U)
#define CCM_GPR_SHARED4_TOG_m33_mask_sinc3_SHIFT (0U)
/*! m33_mask_sinc3 - m33_mask_sinc3 */
#define CCM_GPR_SHARED4_TOG_m33_mask_sinc3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_TOG_m33_mask_sinc3_SHIFT)) & CCM_GPR_SHARED4_TOG_m33_mask_sinc3_MASK)

#define CCM_GPR_SHARED4_TOG_m33_mask_sai1_MASK   (0x2U)
#define CCM_GPR_SHARED4_TOG_m33_mask_sai1_SHIFT  (1U)
/*! m33_mask_sai1 - m33_mask_sai1 */
#define CCM_GPR_SHARED4_TOG_m33_mask_sai1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_TOG_m33_mask_sai1_SHIFT)) & CCM_GPR_SHARED4_TOG_m33_mask_sai1_MASK)

#define CCM_GPR_SHARED4_TOG_m33_mask_sai2_MASK   (0x4U)
#define CCM_GPR_SHARED4_TOG_m33_mask_sai2_SHIFT  (2U)
/*! m33_mask_sai2 - m33_mask_sai2 */
#define CCM_GPR_SHARED4_TOG_m33_mask_sai2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_TOG_m33_mask_sai2_SHIFT)) & CCM_GPR_SHARED4_TOG_m33_mask_sai2_MASK)

#define CCM_GPR_SHARED4_TOG_m33_mask_sai3_MASK   (0x8U)
#define CCM_GPR_SHARED4_TOG_m33_mask_sai3_SHIFT  (3U)
/*! m33_mask_sai3 - m33_mask_sai3 */
#define CCM_GPR_SHARED4_TOG_m33_mask_sai3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_TOG_m33_mask_sai3_SHIFT)) & CCM_GPR_SHARED4_TOG_m33_mask_sai3_MASK)

#define CCM_GPR_SHARED4_TOG_m33_mask_sai4_MASK   (0x10U)
#define CCM_GPR_SHARED4_TOG_m33_mask_sai4_SHIFT  (4U)
/*! m33_mask_sai4 - m33_mask_sai4 */
#define CCM_GPR_SHARED4_TOG_m33_mask_sai4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_TOG_m33_mask_sai4_SHIFT)) & CCM_GPR_SHARED4_TOG_m33_mask_sai4_MASK)

#define CCM_GPR_SHARED4_TOG_m33_mask_mic_MASK    (0x20U)
#define CCM_GPR_SHARED4_TOG_m33_mask_mic_SHIFT   (5U)
/*! m33_mask_mic - m33_mask_mic */
#define CCM_GPR_SHARED4_TOG_m33_mask_mic(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_TOG_m33_mask_mic_SHIFT)) & CCM_GPR_SHARED4_TOG_m33_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED4_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED4_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED4_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED4_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED4_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED4_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED4_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED4_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED4_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED4_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED4_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED4_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED4_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED4_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED4_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED4_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED4_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED4_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED4_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED4_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED4_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED4_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED4_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED4_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED4_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED4_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED4_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED4_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED4_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED4_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED4_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED4_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED4_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED4_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED4_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED4_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED4_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED4_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED4_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED4_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED4_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED4_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED4_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED4_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED4_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED4_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED4_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED4_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED4_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED4_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED4_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED4_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED4_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED4_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED5 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED5_m7_mask_cm7_MASK         (0x1U)
#define CCM_GPR_SHARED5_m7_mask_cm7_SHIFT        (0U)
/*! m7_mask_cm7 - m7_mask_cm7 */
#define CCM_GPR_SHARED5_m7_mask_cm7(x)           (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_cm7_SHIFT)) & CCM_GPR_SHARED5_m7_mask_cm7_MASK)

#define CCM_GPR_SHARED5_m7_mask_cm33_MASK        (0x2U)
#define CCM_GPR_SHARED5_m7_mask_cm33_SHIFT       (1U)
/*! m7_mask_cm33 - m7_mask_cm33 */
#define CCM_GPR_SHARED5_m7_mask_cm33(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_cm33_SHIFT)) & CCM_GPR_SHARED5_m7_mask_cm33_MASK)

#define CCM_GPR_SHARED5_m7_mask_edma3_MASK       (0x4U)
#define CCM_GPR_SHARED5_m7_mask_edma3_SHIFT      (2U)
/*! m7_mask_edma3 - m7_mask_edma3 */
#define CCM_GPR_SHARED5_m7_mask_edma3(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_edma3_SHIFT)) & CCM_GPR_SHARED5_m7_mask_edma3_MASK)

#define CCM_GPR_SHARED5_m7_mask_edma4_MASK       (0x8U)
#define CCM_GPR_SHARED5_m7_mask_edma4_SHIFT      (3U)
/*! m7_mask_edma4 - m7_mask_edma4 */
#define CCM_GPR_SHARED5_m7_mask_edma4(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_edma4_SHIFT)) & CCM_GPR_SHARED5_m7_mask_edma4_MASK)

#define CCM_GPR_SHARED5_m7_mask_netc_MASK        (0x10U)
#define CCM_GPR_SHARED5_m7_mask_netc_SHIFT       (4U)
/*! m7_mask_netc - m7_mask_netc */
#define CCM_GPR_SHARED5_m7_mask_netc(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_netc_SHIFT)) & CCM_GPR_SHARED5_m7_mask_netc_MASK)

#define CCM_GPR_SHARED5_m7_mask_sim_aon_MASK     (0x100U)
#define CCM_GPR_SHARED5_m7_mask_sim_aon_SHIFT    (8U)
/*! m7_mask_sim_aon - m7_mask_sim_aon */
#define CCM_GPR_SHARED5_m7_mask_sim_aon(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED5_m7_mask_sim_aon_MASK)

#define CCM_GPR_SHARED5_m7_mask_adc1_MASK        (0x200U)
#define CCM_GPR_SHARED5_m7_mask_adc1_SHIFT       (9U)
/*! m7_mask_adc1 - m7_mask_adc1 */
#define CCM_GPR_SHARED5_m7_mask_adc1(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_adc1_SHIFT)) & CCM_GPR_SHARED5_m7_mask_adc1_MASK)

#define CCM_GPR_SHARED5_m7_mask_adc2_MASK        (0x400U)
#define CCM_GPR_SHARED5_m7_mask_adc2_SHIFT       (10U)
/*! m7_mask_adc2 - m7_mask_adc2 */
#define CCM_GPR_SHARED5_m7_mask_adc2(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_adc2_SHIFT)) & CCM_GPR_SHARED5_m7_mask_adc2_MASK)

#define CCM_GPR_SHARED5_m7_mask_flexspi1_MASK    (0x800U)
#define CCM_GPR_SHARED5_m7_mask_flexspi1_SHIFT   (11U)
/*! m7_mask_flexspi1 - m7_mask_flexspi1 */
#define CCM_GPR_SHARED5_m7_mask_flexspi1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED5_m7_mask_flexspi1_MASK)

#define CCM_GPR_SHARED5_m7_mask_flexspi2_MASK    (0x1000U)
#define CCM_GPR_SHARED5_m7_mask_flexspi2_SHIFT   (12U)
/*! m7_mask_flexspi2 - m7_mask_flexspi2 */
#define CCM_GPR_SHARED5_m7_mask_flexspi2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED5_m7_mask_flexspi2_MASK)

#define CCM_GPR_SHARED5_m7_mask_trdc_MASK        (0x2000U)
#define CCM_GPR_SHARED5_m7_mask_trdc_SHIFT       (13U)
/*! m7_mask_trdc - m7_mask_trdc */
#define CCM_GPR_SHARED5_m7_mask_trdc(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_trdc_SHIFT)) & CCM_GPR_SHARED5_m7_mask_trdc_MASK)

#define CCM_GPR_SHARED5_m7_mask_semc_MASK        (0x4000U)
#define CCM_GPR_SHARED5_m7_mask_semc_SHIFT       (14U)
/*! m7_mask_semc - m7_mask_semc */
#define CCM_GPR_SHARED5_m7_mask_semc(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_semc_SHIFT)) & CCM_GPR_SHARED5_m7_mask_semc_MASK)

#define CCM_GPR_SHARED5_m7_mask_iee_MASK         (0x8000U)
#define CCM_GPR_SHARED5_m7_mask_iee_SHIFT        (15U)
/*! m7_mask_iee - m7_mask_iee */
#define CCM_GPR_SHARED5_m7_mask_iee(x)           (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_iee_SHIFT)) & CCM_GPR_SHARED5_m7_mask_iee_MASK)

#define CCM_GPR_SHARED5_m7_mask_gpio1_MASK       (0x10000U)
#define CCM_GPR_SHARED5_m7_mask_gpio1_SHIFT      (16U)
/*! m7_mask_gpio1 - m7_mask_gpio1 */
#define CCM_GPR_SHARED5_m7_mask_gpio1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_gpio1_SHIFT)) & CCM_GPR_SHARED5_m7_mask_gpio1_MASK)

#define CCM_GPR_SHARED5_m7_mask_gpio2_MASK       (0x20000U)
#define CCM_GPR_SHARED5_m7_mask_gpio2_SHIFT      (17U)
/*! m7_mask_gpio2 - m7_mask_gpio2 */
#define CCM_GPR_SHARED5_m7_mask_gpio2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_gpio2_SHIFT)) & CCM_GPR_SHARED5_m7_mask_gpio2_MASK)

#define CCM_GPR_SHARED5_m7_mask_gpio3_MASK       (0x40000U)
#define CCM_GPR_SHARED5_m7_mask_gpio3_SHIFT      (18U)
/*! m7_mask_gpio3 - m7_mask_gpio3 */
#define CCM_GPR_SHARED5_m7_mask_gpio3(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_gpio3_SHIFT)) & CCM_GPR_SHARED5_m7_mask_gpio3_MASK)

#define CCM_GPR_SHARED5_m7_mask_gpio4_MASK       (0x80000U)
#define CCM_GPR_SHARED5_m7_mask_gpio4_SHIFT      (19U)
/*! m7_mask_gpio4 - m7_mask_gpio4 */
#define CCM_GPR_SHARED5_m7_mask_gpio4(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_gpio4_SHIFT)) & CCM_GPR_SHARED5_m7_mask_gpio4_MASK)

#define CCM_GPR_SHARED5_m7_mask_gpio5_MASK       (0x100000U)
#define CCM_GPR_SHARED5_m7_mask_gpio5_SHIFT      (20U)
/*! m7_mask_gpio5 - m7_mask_gpio5 */
#define CCM_GPR_SHARED5_m7_mask_gpio5(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_gpio5_SHIFT)) & CCM_GPR_SHARED5_m7_mask_gpio5_MASK)

#define CCM_GPR_SHARED5_m7_mask_gpio6_MASK       (0x200000U)
#define CCM_GPR_SHARED5_m7_mask_gpio6_SHIFT      (21U)
/*! m7_mask_gpio6 - m7_mask_gpio6 */
#define CCM_GPR_SHARED5_m7_mask_gpio6(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_gpio6_SHIFT)) & CCM_GPR_SHARED5_m7_mask_gpio6_MASK)

#define CCM_GPR_SHARED5_m7_mask_flexio1_MASK     (0x400000U)
#define CCM_GPR_SHARED5_m7_mask_flexio1_SHIFT    (22U)
/*! m7_mask_flexio1 - m7_mask_flexio1 */
#define CCM_GPR_SHARED5_m7_mask_flexio1(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_flexio1_SHIFT)) & CCM_GPR_SHARED5_m7_mask_flexio1_MASK)

#define CCM_GPR_SHARED5_m7_mask_flexio2_MASK     (0x800000U)
#define CCM_GPR_SHARED5_m7_mask_flexio2_SHIFT    (23U)
/*! m7_mask_flexio2 - m7_mask_flexio2 */
#define CCM_GPR_SHARED5_m7_mask_flexio2(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_flexio2_SHIFT)) & CCM_GPR_SHARED5_m7_mask_flexio2_MASK)

#define CCM_GPR_SHARED5_m7_mask_lpit1_MASK       (0x1000000U)
#define CCM_GPR_SHARED5_m7_mask_lpit1_SHIFT      (24U)
/*! m7_mask_lpit1 - m7_mask_lpit1 */
#define CCM_GPR_SHARED5_m7_mask_lpit1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_lpit1_SHIFT)) & CCM_GPR_SHARED5_m7_mask_lpit1_MASK)

#define CCM_GPR_SHARED5_m7_mask_lpit2_MASK       (0x2000000U)
#define CCM_GPR_SHARED5_m7_mask_lpit2_SHIFT      (25U)
/*! m7_mask_lpit2 - m7_mask_lpit2 */
#define CCM_GPR_SHARED5_m7_mask_lpit2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_lpit2_SHIFT)) & CCM_GPR_SHARED5_m7_mask_lpit2_MASK)

#define CCM_GPR_SHARED5_m7_mask_lpit3_MASK       (0x4000000U)
#define CCM_GPR_SHARED5_m7_mask_lpit3_SHIFT      (26U)
/*! m7_mask_lpit3 - m7_mask_lpit3 */
#define CCM_GPR_SHARED5_m7_mask_lpit3(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_lpit3_SHIFT)) & CCM_GPR_SHARED5_m7_mask_lpit3_MASK)

#define CCM_GPR_SHARED5_m7_mask_tpm1_MASK        (0x8000000U)
#define CCM_GPR_SHARED5_m7_mask_tpm1_SHIFT       (27U)
/*! m7_mask_tpm1 - m7_mask_tpm1 */
#define CCM_GPR_SHARED5_m7_mask_tpm1(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_tpm1_SHIFT)) & CCM_GPR_SHARED5_m7_mask_tpm1_MASK)

#define CCM_GPR_SHARED5_m7_mask_tpm2_MASK        (0x10000000U)
#define CCM_GPR_SHARED5_m7_mask_tpm2_SHIFT       (28U)
/*! m7_mask_tpm2 - m7_mask_tpm2 */
#define CCM_GPR_SHARED5_m7_mask_tpm2(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_tpm2_SHIFT)) & CCM_GPR_SHARED5_m7_mask_tpm2_MASK)

#define CCM_GPR_SHARED5_m7_mask_tpm3_MASK        (0x20000000U)
#define CCM_GPR_SHARED5_m7_mask_tpm3_SHIFT       (29U)
/*! m7_mask_tpm3 - m7_mask_tpm3 */
#define CCM_GPR_SHARED5_m7_mask_tpm3(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_tpm3_SHIFT)) & CCM_GPR_SHARED5_m7_mask_tpm3_MASK)

#define CCM_GPR_SHARED5_m7_mask_tpm4_MASK        (0x40000000U)
#define CCM_GPR_SHARED5_m7_mask_tpm4_SHIFT       (30U)
/*! m7_mask_tpm4 - m7_mask_tpm4 */
#define CCM_GPR_SHARED5_m7_mask_tpm4(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_tpm4_SHIFT)) & CCM_GPR_SHARED5_m7_mask_tpm4_MASK)

#define CCM_GPR_SHARED5_m7_mask_tpm5_MASK        (0x80000000U)
#define CCM_GPR_SHARED5_m7_mask_tpm5_SHIFT       (31U)
/*! m7_mask_tpm5 - m7_mask_tpm5 */
#define CCM_GPR_SHARED5_m7_mask_tpm5(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_m7_mask_tpm5_SHIFT)) & CCM_GPR_SHARED5_m7_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED5_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED5_SET_m7_mask_cm7_MASK     (0x1U)
#define CCM_GPR_SHARED5_SET_m7_mask_cm7_SHIFT    (0U)
/*! m7_mask_cm7 - m7_mask_cm7 */
#define CCM_GPR_SHARED5_SET_m7_mask_cm7(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_cm7_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_cm7_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_cm33_MASK    (0x2U)
#define CCM_GPR_SHARED5_SET_m7_mask_cm33_SHIFT   (1U)
/*! m7_mask_cm33 - m7_mask_cm33 */
#define CCM_GPR_SHARED5_SET_m7_mask_cm33(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_cm33_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_cm33_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_edma3_MASK   (0x4U)
#define CCM_GPR_SHARED5_SET_m7_mask_edma3_SHIFT  (2U)
/*! m7_mask_edma3 - m7_mask_edma3 */
#define CCM_GPR_SHARED5_SET_m7_mask_edma3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_edma3_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_edma3_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_edma4_MASK   (0x8U)
#define CCM_GPR_SHARED5_SET_m7_mask_edma4_SHIFT  (3U)
/*! m7_mask_edma4 - m7_mask_edma4 */
#define CCM_GPR_SHARED5_SET_m7_mask_edma4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_edma4_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_edma4_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_netc_MASK    (0x10U)
#define CCM_GPR_SHARED5_SET_m7_mask_netc_SHIFT   (4U)
/*! m7_mask_netc - m7_mask_netc */
#define CCM_GPR_SHARED5_SET_m7_mask_netc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_netc_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_netc_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_sim_aon_MASK (0x100U)
#define CCM_GPR_SHARED5_SET_m7_mask_sim_aon_SHIFT (8U)
/*! m7_mask_sim_aon - m7_mask_sim_aon */
#define CCM_GPR_SHARED5_SET_m7_mask_sim_aon(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_sim_aon_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_adc1_MASK    (0x200U)
#define CCM_GPR_SHARED5_SET_m7_mask_adc1_SHIFT   (9U)
/*! m7_mask_adc1 - m7_mask_adc1 */
#define CCM_GPR_SHARED5_SET_m7_mask_adc1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_adc1_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_adc1_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_adc2_MASK    (0x400U)
#define CCM_GPR_SHARED5_SET_m7_mask_adc2_SHIFT   (10U)
/*! m7_mask_adc2 - m7_mask_adc2 */
#define CCM_GPR_SHARED5_SET_m7_mask_adc2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_adc2_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_adc2_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_flexspi1_MASK (0x800U)
#define CCM_GPR_SHARED5_SET_m7_mask_flexspi1_SHIFT (11U)
/*! m7_mask_flexspi1 - m7_mask_flexspi1 */
#define CCM_GPR_SHARED5_SET_m7_mask_flexspi1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_flexspi1_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_flexspi2_MASK (0x1000U)
#define CCM_GPR_SHARED5_SET_m7_mask_flexspi2_SHIFT (12U)
/*! m7_mask_flexspi2 - m7_mask_flexspi2 */
#define CCM_GPR_SHARED5_SET_m7_mask_flexspi2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_flexspi2_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_trdc_MASK    (0x2000U)
#define CCM_GPR_SHARED5_SET_m7_mask_trdc_SHIFT   (13U)
/*! m7_mask_trdc - m7_mask_trdc */
#define CCM_GPR_SHARED5_SET_m7_mask_trdc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_trdc_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_trdc_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_semc_MASK    (0x4000U)
#define CCM_GPR_SHARED5_SET_m7_mask_semc_SHIFT   (14U)
/*! m7_mask_semc - m7_mask_semc */
#define CCM_GPR_SHARED5_SET_m7_mask_semc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_semc_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_semc_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_iee_MASK     (0x8000U)
#define CCM_GPR_SHARED5_SET_m7_mask_iee_SHIFT    (15U)
/*! m7_mask_iee - m7_mask_iee */
#define CCM_GPR_SHARED5_SET_m7_mask_iee(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_iee_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_iee_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_gpio1_MASK   (0x10000U)
#define CCM_GPR_SHARED5_SET_m7_mask_gpio1_SHIFT  (16U)
/*! m7_mask_gpio1 - m7_mask_gpio1 */
#define CCM_GPR_SHARED5_SET_m7_mask_gpio1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_gpio1_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_gpio1_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_gpio2_MASK   (0x20000U)
#define CCM_GPR_SHARED5_SET_m7_mask_gpio2_SHIFT  (17U)
/*! m7_mask_gpio2 - m7_mask_gpio2 */
#define CCM_GPR_SHARED5_SET_m7_mask_gpio2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_gpio2_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_gpio2_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_gpio3_MASK   (0x40000U)
#define CCM_GPR_SHARED5_SET_m7_mask_gpio3_SHIFT  (18U)
/*! m7_mask_gpio3 - m7_mask_gpio3 */
#define CCM_GPR_SHARED5_SET_m7_mask_gpio3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_gpio3_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_gpio3_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_gpio4_MASK   (0x80000U)
#define CCM_GPR_SHARED5_SET_m7_mask_gpio4_SHIFT  (19U)
/*! m7_mask_gpio4 - m7_mask_gpio4 */
#define CCM_GPR_SHARED5_SET_m7_mask_gpio4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_gpio4_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_gpio4_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_gpio5_MASK   (0x100000U)
#define CCM_GPR_SHARED5_SET_m7_mask_gpio5_SHIFT  (20U)
/*! m7_mask_gpio5 - m7_mask_gpio5 */
#define CCM_GPR_SHARED5_SET_m7_mask_gpio5(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_gpio5_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_gpio5_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_gpio6_MASK   (0x200000U)
#define CCM_GPR_SHARED5_SET_m7_mask_gpio6_SHIFT  (21U)
/*! m7_mask_gpio6 - m7_mask_gpio6 */
#define CCM_GPR_SHARED5_SET_m7_mask_gpio6(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_gpio6_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_gpio6_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_flexio1_MASK (0x400000U)
#define CCM_GPR_SHARED5_SET_m7_mask_flexio1_SHIFT (22U)
/*! m7_mask_flexio1 - m7_mask_flexio1 */
#define CCM_GPR_SHARED5_SET_m7_mask_flexio1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_flexio1_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_flexio1_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_flexio2_MASK (0x800000U)
#define CCM_GPR_SHARED5_SET_m7_mask_flexio2_SHIFT (23U)
/*! m7_mask_flexio2 - m7_mask_flexio2 */
#define CCM_GPR_SHARED5_SET_m7_mask_flexio2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_flexio2_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_flexio2_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_lpit1_MASK   (0x1000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_lpit1_SHIFT  (24U)
/*! m7_mask_lpit1 - m7_mask_lpit1 */
#define CCM_GPR_SHARED5_SET_m7_mask_lpit1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_lpit1_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_lpit1_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_lpit2_MASK   (0x2000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_lpit2_SHIFT  (25U)
/*! m7_mask_lpit2 - m7_mask_lpit2 */
#define CCM_GPR_SHARED5_SET_m7_mask_lpit2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_lpit2_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_lpit2_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_lpit3_MASK   (0x4000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_lpit3_SHIFT  (26U)
/*! m7_mask_lpit3 - m7_mask_lpit3 */
#define CCM_GPR_SHARED5_SET_m7_mask_lpit3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_lpit3_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_lpit3_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_tpm1_MASK    (0x8000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_tpm1_SHIFT   (27U)
/*! m7_mask_tpm1 - m7_mask_tpm1 */
#define CCM_GPR_SHARED5_SET_m7_mask_tpm1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_tpm1_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_tpm1_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_tpm2_MASK    (0x10000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_tpm2_SHIFT   (28U)
/*! m7_mask_tpm2 - m7_mask_tpm2 */
#define CCM_GPR_SHARED5_SET_m7_mask_tpm2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_tpm2_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_tpm2_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_tpm3_MASK    (0x20000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_tpm3_SHIFT   (29U)
/*! m7_mask_tpm3 - m7_mask_tpm3 */
#define CCM_GPR_SHARED5_SET_m7_mask_tpm3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_tpm3_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_tpm3_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_tpm4_MASK    (0x40000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_tpm4_SHIFT   (30U)
/*! m7_mask_tpm4 - m7_mask_tpm4 */
#define CCM_GPR_SHARED5_SET_m7_mask_tpm4(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_tpm4_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_tpm4_MASK)

#define CCM_GPR_SHARED5_SET_m7_mask_tpm5_MASK    (0x80000000U)
#define CCM_GPR_SHARED5_SET_m7_mask_tpm5_SHIFT   (31U)
/*! m7_mask_tpm5 - m7_mask_tpm5 */
#define CCM_GPR_SHARED5_SET_m7_mask_tpm5(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_SET_m7_mask_tpm5_SHIFT)) & CCM_GPR_SHARED5_SET_m7_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED5_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED5_CLR_m7_mask_cm7_MASK     (0x1U)
#define CCM_GPR_SHARED5_CLR_m7_mask_cm7_SHIFT    (0U)
/*! m7_mask_cm7 - m7_mask_cm7 */
#define CCM_GPR_SHARED5_CLR_m7_mask_cm7(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_cm7_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_cm7_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_cm33_MASK    (0x2U)
#define CCM_GPR_SHARED5_CLR_m7_mask_cm33_SHIFT   (1U)
/*! m7_mask_cm33 - m7_mask_cm33 */
#define CCM_GPR_SHARED5_CLR_m7_mask_cm33(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_cm33_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_cm33_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_edma3_MASK   (0x4U)
#define CCM_GPR_SHARED5_CLR_m7_mask_edma3_SHIFT  (2U)
/*! m7_mask_edma3 - m7_mask_edma3 */
#define CCM_GPR_SHARED5_CLR_m7_mask_edma3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_edma3_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_edma3_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_edma4_MASK   (0x8U)
#define CCM_GPR_SHARED5_CLR_m7_mask_edma4_SHIFT  (3U)
/*! m7_mask_edma4 - m7_mask_edma4 */
#define CCM_GPR_SHARED5_CLR_m7_mask_edma4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_edma4_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_edma4_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_netc_MASK    (0x10U)
#define CCM_GPR_SHARED5_CLR_m7_mask_netc_SHIFT   (4U)
/*! m7_mask_netc - m7_mask_netc */
#define CCM_GPR_SHARED5_CLR_m7_mask_netc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_netc_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_netc_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_sim_aon_MASK (0x100U)
#define CCM_GPR_SHARED5_CLR_m7_mask_sim_aon_SHIFT (8U)
/*! m7_mask_sim_aon - m7_mask_sim_aon */
#define CCM_GPR_SHARED5_CLR_m7_mask_sim_aon(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_sim_aon_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_adc1_MASK    (0x200U)
#define CCM_GPR_SHARED5_CLR_m7_mask_adc1_SHIFT   (9U)
/*! m7_mask_adc1 - m7_mask_adc1 */
#define CCM_GPR_SHARED5_CLR_m7_mask_adc1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_adc1_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_adc1_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_adc2_MASK    (0x400U)
#define CCM_GPR_SHARED5_CLR_m7_mask_adc2_SHIFT   (10U)
/*! m7_mask_adc2 - m7_mask_adc2 */
#define CCM_GPR_SHARED5_CLR_m7_mask_adc2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_adc2_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_adc2_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_flexspi1_MASK (0x800U)
#define CCM_GPR_SHARED5_CLR_m7_mask_flexspi1_SHIFT (11U)
/*! m7_mask_flexspi1 - m7_mask_flexspi1 */
#define CCM_GPR_SHARED5_CLR_m7_mask_flexspi1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_flexspi1_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_flexspi2_MASK (0x1000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_flexspi2_SHIFT (12U)
/*! m7_mask_flexspi2 - m7_mask_flexspi2 */
#define CCM_GPR_SHARED5_CLR_m7_mask_flexspi2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_flexspi2_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_trdc_MASK    (0x2000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_trdc_SHIFT   (13U)
/*! m7_mask_trdc - m7_mask_trdc */
#define CCM_GPR_SHARED5_CLR_m7_mask_trdc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_trdc_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_trdc_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_semc_MASK    (0x4000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_semc_SHIFT   (14U)
/*! m7_mask_semc - m7_mask_semc */
#define CCM_GPR_SHARED5_CLR_m7_mask_semc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_semc_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_semc_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_iee_MASK     (0x8000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_iee_SHIFT    (15U)
/*! m7_mask_iee - m7_mask_iee */
#define CCM_GPR_SHARED5_CLR_m7_mask_iee(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_iee_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_iee_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_gpio1_MASK   (0x10000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio1_SHIFT  (16U)
/*! m7_mask_gpio1 - m7_mask_gpio1 */
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_gpio1_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_gpio1_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_gpio2_MASK   (0x20000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio2_SHIFT  (17U)
/*! m7_mask_gpio2 - m7_mask_gpio2 */
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_gpio2_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_gpio2_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_gpio3_MASK   (0x40000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio3_SHIFT  (18U)
/*! m7_mask_gpio3 - m7_mask_gpio3 */
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_gpio3_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_gpio3_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_gpio4_MASK   (0x80000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio4_SHIFT  (19U)
/*! m7_mask_gpio4 - m7_mask_gpio4 */
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_gpio4_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_gpio4_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_gpio5_MASK   (0x100000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio5_SHIFT  (20U)
/*! m7_mask_gpio5 - m7_mask_gpio5 */
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio5(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_gpio5_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_gpio5_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_gpio6_MASK   (0x200000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio6_SHIFT  (21U)
/*! m7_mask_gpio6 - m7_mask_gpio6 */
#define CCM_GPR_SHARED5_CLR_m7_mask_gpio6(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_gpio6_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_gpio6_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_flexio1_MASK (0x400000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_flexio1_SHIFT (22U)
/*! m7_mask_flexio1 - m7_mask_flexio1 */
#define CCM_GPR_SHARED5_CLR_m7_mask_flexio1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_flexio1_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_flexio1_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_flexio2_MASK (0x800000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_flexio2_SHIFT (23U)
/*! m7_mask_flexio2 - m7_mask_flexio2 */
#define CCM_GPR_SHARED5_CLR_m7_mask_flexio2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_flexio2_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_flexio2_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_lpit1_MASK   (0x1000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_lpit1_SHIFT  (24U)
/*! m7_mask_lpit1 - m7_mask_lpit1 */
#define CCM_GPR_SHARED5_CLR_m7_mask_lpit1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_lpit1_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_lpit1_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_lpit2_MASK   (0x2000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_lpit2_SHIFT  (25U)
/*! m7_mask_lpit2 - m7_mask_lpit2 */
#define CCM_GPR_SHARED5_CLR_m7_mask_lpit2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_lpit2_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_lpit2_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_lpit3_MASK   (0x4000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_lpit3_SHIFT  (26U)
/*! m7_mask_lpit3 - m7_mask_lpit3 */
#define CCM_GPR_SHARED5_CLR_m7_mask_lpit3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_lpit3_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_lpit3_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_tpm1_MASK    (0x8000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm1_SHIFT   (27U)
/*! m7_mask_tpm1 - m7_mask_tpm1 */
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_tpm1_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_tpm1_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_tpm2_MASK    (0x10000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm2_SHIFT   (28U)
/*! m7_mask_tpm2 - m7_mask_tpm2 */
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_tpm2_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_tpm2_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_tpm3_MASK    (0x20000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm3_SHIFT   (29U)
/*! m7_mask_tpm3 - m7_mask_tpm3 */
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_tpm3_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_tpm3_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_tpm4_MASK    (0x40000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm4_SHIFT   (30U)
/*! m7_mask_tpm4 - m7_mask_tpm4 */
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm4(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_tpm4_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_tpm4_MASK)

#define CCM_GPR_SHARED5_CLR_m7_mask_tpm5_MASK    (0x80000000U)
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm5_SHIFT   (31U)
/*! m7_mask_tpm5 - m7_mask_tpm5 */
#define CCM_GPR_SHARED5_CLR_m7_mask_tpm5(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_CLR_m7_mask_tpm5_SHIFT)) & CCM_GPR_SHARED5_CLR_m7_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED5_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED5_TOG_m7_mask_cm7_MASK     (0x1U)
#define CCM_GPR_SHARED5_TOG_m7_mask_cm7_SHIFT    (0U)
/*! m7_mask_cm7 - m7_mask_cm7 */
#define CCM_GPR_SHARED5_TOG_m7_mask_cm7(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_cm7_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_cm7_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_cm33_MASK    (0x2U)
#define CCM_GPR_SHARED5_TOG_m7_mask_cm33_SHIFT   (1U)
/*! m7_mask_cm33 - m7_mask_cm33 */
#define CCM_GPR_SHARED5_TOG_m7_mask_cm33(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_cm33_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_cm33_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_edma3_MASK   (0x4U)
#define CCM_GPR_SHARED5_TOG_m7_mask_edma3_SHIFT  (2U)
/*! m7_mask_edma3 - m7_mask_edma3 */
#define CCM_GPR_SHARED5_TOG_m7_mask_edma3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_edma3_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_edma3_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_edma4_MASK   (0x8U)
#define CCM_GPR_SHARED5_TOG_m7_mask_edma4_SHIFT  (3U)
/*! m7_mask_edma4 - m7_mask_edma4 */
#define CCM_GPR_SHARED5_TOG_m7_mask_edma4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_edma4_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_edma4_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_netc_MASK    (0x10U)
#define CCM_GPR_SHARED5_TOG_m7_mask_netc_SHIFT   (4U)
/*! m7_mask_netc - m7_mask_netc */
#define CCM_GPR_SHARED5_TOG_m7_mask_netc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_netc_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_netc_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_sim_aon_MASK (0x100U)
#define CCM_GPR_SHARED5_TOG_m7_mask_sim_aon_SHIFT (8U)
/*! m7_mask_sim_aon - m7_mask_sim_aon */
#define CCM_GPR_SHARED5_TOG_m7_mask_sim_aon(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_sim_aon_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_sim_aon_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_adc1_MASK    (0x200U)
#define CCM_GPR_SHARED5_TOG_m7_mask_adc1_SHIFT   (9U)
/*! m7_mask_adc1 - m7_mask_adc1 */
#define CCM_GPR_SHARED5_TOG_m7_mask_adc1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_adc1_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_adc1_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_adc2_MASK    (0x400U)
#define CCM_GPR_SHARED5_TOG_m7_mask_adc2_SHIFT   (10U)
/*! m7_mask_adc2 - m7_mask_adc2 */
#define CCM_GPR_SHARED5_TOG_m7_mask_adc2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_adc2_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_adc2_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_flexspi1_MASK (0x800U)
#define CCM_GPR_SHARED5_TOG_m7_mask_flexspi1_SHIFT (11U)
/*! m7_mask_flexspi1 - m7_mask_flexspi1 */
#define CCM_GPR_SHARED5_TOG_m7_mask_flexspi1(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_flexspi1_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_flexspi1_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_flexspi2_MASK (0x1000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_flexspi2_SHIFT (12U)
/*! m7_mask_flexspi2 - m7_mask_flexspi2 */
#define CCM_GPR_SHARED5_TOG_m7_mask_flexspi2(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_flexspi2_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_flexspi2_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_trdc_MASK    (0x2000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_trdc_SHIFT   (13U)
/*! m7_mask_trdc - m7_mask_trdc */
#define CCM_GPR_SHARED5_TOG_m7_mask_trdc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_trdc_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_trdc_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_semc_MASK    (0x4000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_semc_SHIFT   (14U)
/*! m7_mask_semc - m7_mask_semc */
#define CCM_GPR_SHARED5_TOG_m7_mask_semc(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_semc_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_semc_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_iee_MASK     (0x8000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_iee_SHIFT    (15U)
/*! m7_mask_iee - m7_mask_iee */
#define CCM_GPR_SHARED5_TOG_m7_mask_iee(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_iee_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_iee_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_gpio1_MASK   (0x10000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio1_SHIFT  (16U)
/*! m7_mask_gpio1 - m7_mask_gpio1 */
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_gpio1_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_gpio1_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_gpio2_MASK   (0x20000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio2_SHIFT  (17U)
/*! m7_mask_gpio2 - m7_mask_gpio2 */
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_gpio2_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_gpio2_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_gpio3_MASK   (0x40000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio3_SHIFT  (18U)
/*! m7_mask_gpio3 - m7_mask_gpio3 */
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_gpio3_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_gpio3_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_gpio4_MASK   (0x80000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio4_SHIFT  (19U)
/*! m7_mask_gpio4 - m7_mask_gpio4 */
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio4(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_gpio4_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_gpio4_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_gpio5_MASK   (0x100000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio5_SHIFT  (20U)
/*! m7_mask_gpio5 - m7_mask_gpio5 */
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio5(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_gpio5_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_gpio5_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_gpio6_MASK   (0x200000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio6_SHIFT  (21U)
/*! m7_mask_gpio6 - m7_mask_gpio6 */
#define CCM_GPR_SHARED5_TOG_m7_mask_gpio6(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_gpio6_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_gpio6_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_flexio1_MASK (0x400000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_flexio1_SHIFT (22U)
/*! m7_mask_flexio1 - m7_mask_flexio1 */
#define CCM_GPR_SHARED5_TOG_m7_mask_flexio1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_flexio1_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_flexio1_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_flexio2_MASK (0x800000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_flexio2_SHIFT (23U)
/*! m7_mask_flexio2 - m7_mask_flexio2 */
#define CCM_GPR_SHARED5_TOG_m7_mask_flexio2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_flexio2_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_flexio2_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_lpit1_MASK   (0x1000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_lpit1_SHIFT  (24U)
/*! m7_mask_lpit1 - m7_mask_lpit1 */
#define CCM_GPR_SHARED5_TOG_m7_mask_lpit1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_lpit1_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_lpit1_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_lpit2_MASK   (0x2000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_lpit2_SHIFT  (25U)
/*! m7_mask_lpit2 - m7_mask_lpit2 */
#define CCM_GPR_SHARED5_TOG_m7_mask_lpit2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_lpit2_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_lpit2_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_lpit3_MASK   (0x4000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_lpit3_SHIFT  (26U)
/*! m7_mask_lpit3 - m7_mask_lpit3 */
#define CCM_GPR_SHARED5_TOG_m7_mask_lpit3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_lpit3_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_lpit3_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_tpm1_MASK    (0x8000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm1_SHIFT   (27U)
/*! m7_mask_tpm1 - m7_mask_tpm1 */
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_tpm1_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_tpm1_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_tpm2_MASK    (0x10000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm2_SHIFT   (28U)
/*! m7_mask_tpm2 - m7_mask_tpm2 */
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_tpm2_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_tpm2_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_tpm3_MASK    (0x20000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm3_SHIFT   (29U)
/*! m7_mask_tpm3 - m7_mask_tpm3 */
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_tpm3_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_tpm3_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_tpm4_MASK    (0x40000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm4_SHIFT   (30U)
/*! m7_mask_tpm4 - m7_mask_tpm4 */
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm4(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_tpm4_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_tpm4_MASK)

#define CCM_GPR_SHARED5_TOG_m7_mask_tpm5_MASK    (0x80000000U)
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm5_SHIFT   (31U)
/*! m7_mask_tpm5 - m7_mask_tpm5 */
#define CCM_GPR_SHARED5_TOG_m7_mask_tpm5(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_TOG_m7_mask_tpm5_SHIFT)) & CCM_GPR_SHARED5_TOG_m7_mask_tpm5_MASK)
/*! @} */

/*! @name GPR_SHARED5_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED5_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED5_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED5_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED5_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED5_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED5_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED5_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED5_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED5_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED5_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED5_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED5_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED5_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED5_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED5_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED5_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED5_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED5_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED5_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED5_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED5_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED5_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED5_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED5_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED5_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED5_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED5_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED5_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED5_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED5_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED5_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED5_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED5_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED5_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED5_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED5_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED5_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED5_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED5_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED5_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED5_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED5_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED5_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED5_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED5_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED5_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED5_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED5_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED5_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED5_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED5_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED5_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED5_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED6 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED6_m7_mask_tpm6_MASK        (0x1U)
#define CCM_GPR_SHARED6_m7_mask_tpm6_SHIFT       (0U)
/*! m7_mask_tpm6 - m7_mask_tpm6 */
#define CCM_GPR_SHARED6_m7_mask_tpm6(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_tpm6_SHIFT)) & CCM_GPR_SHARED6_m7_mask_tpm6_MASK)

#define CCM_GPR_SHARED6_m7_mask_gpt1_MASK        (0x2U)
#define CCM_GPR_SHARED6_m7_mask_gpt1_SHIFT       (1U)
/*! m7_mask_gpt1 - m7_mask_gpt1 */
#define CCM_GPR_SHARED6_m7_mask_gpt1(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_gpt1_SHIFT)) & CCM_GPR_SHARED6_m7_mask_gpt1_MASK)

#define CCM_GPR_SHARED6_m7_mask_gpt2_MASK        (0x4U)
#define CCM_GPR_SHARED6_m7_mask_gpt2_SHIFT       (2U)
/*! m7_mask_gpt2 - m7_mask_gpt2 */
#define CCM_GPR_SHARED6_m7_mask_gpt2(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_gpt2_SHIFT)) & CCM_GPR_SHARED6_m7_mask_gpt2_MASK)

#define CCM_GPR_SHARED6_m7_mask_can1_MASK        (0x8U)
#define CCM_GPR_SHARED6_m7_mask_can1_SHIFT       (3U)
/*! m7_mask_can1 - m7_mask_can1 */
#define CCM_GPR_SHARED6_m7_mask_can1(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_can1_SHIFT)) & CCM_GPR_SHARED6_m7_mask_can1_MASK)

#define CCM_GPR_SHARED6_m7_mask_can2_MASK        (0x10U)
#define CCM_GPR_SHARED6_m7_mask_can2_SHIFT       (4U)
/*! m7_mask_can2 - m7_mask_can2 */
#define CCM_GPR_SHARED6_m7_mask_can2(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_can2_SHIFT)) & CCM_GPR_SHARED6_m7_mask_can2_MASK)

#define CCM_GPR_SHARED6_m7_mask_can3_MASK        (0x20U)
#define CCM_GPR_SHARED6_m7_mask_can3_SHIFT       (5U)
/*! m7_mask_can3 - m7_mask_can3 */
#define CCM_GPR_SHARED6_m7_mask_can3(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_can3_SHIFT)) & CCM_GPR_SHARED6_m7_mask_can3_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart1_MASK     (0x40U)
#define CCM_GPR_SHARED6_m7_mask_lpuart1_SHIFT    (6U)
/*! m7_mask_lpuart1 - m7_mask_lpuart1 */
#define CCM_GPR_SHARED6_m7_mask_lpuart1(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart1_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart2_MASK     (0x80U)
#define CCM_GPR_SHARED6_m7_mask_lpuart2_SHIFT    (7U)
/*! m7_mask_lpuart2 - m7_mask_lpuart2 */
#define CCM_GPR_SHARED6_m7_mask_lpuart2(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart2_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart3_MASK     (0x100U)
#define CCM_GPR_SHARED6_m7_mask_lpuart3_SHIFT    (8U)
/*! m7_mask_lpuart3 - m7_mask_lpuart3 */
#define CCM_GPR_SHARED6_m7_mask_lpuart3(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart3_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart4_MASK     (0x200U)
#define CCM_GPR_SHARED6_m7_mask_lpuart4_SHIFT    (9U)
/*! m7_mask_lpuart4 - m7_mask_lpuart4 */
#define CCM_GPR_SHARED6_m7_mask_lpuart4(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart4_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart5_MASK     (0x400U)
#define CCM_GPR_SHARED6_m7_mask_lpuart5_SHIFT    (10U)
/*! m7_mask_lpuart5 - m7_mask_lpuart5 */
#define CCM_GPR_SHARED6_m7_mask_lpuart5(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart5_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart6_MASK     (0x800U)
#define CCM_GPR_SHARED6_m7_mask_lpuart6_SHIFT    (11U)
/*! m7_mask_lpuart6 - m7_mask_lpuart6 */
#define CCM_GPR_SHARED6_m7_mask_lpuart6(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart6_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart7_MASK     (0x1000U)
#define CCM_GPR_SHARED6_m7_mask_lpuart7_SHIFT    (12U)
/*! m7_mask_lpuart7 - m7_mask_lpuart7 */
#define CCM_GPR_SHARED6_m7_mask_lpuart7(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart7_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart8_MASK     (0x2000U)
#define CCM_GPR_SHARED6_m7_mask_lpuart8_SHIFT    (13U)
/*! m7_mask_lpuart8 - m7_mask_lpuart8 */
#define CCM_GPR_SHARED6_m7_mask_lpuart8(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart8_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart9_MASK     (0x4000U)
#define CCM_GPR_SHARED6_m7_mask_lpuart9_SHIFT    (14U)
/*! m7_mask_lpuart9 - m7_mask_lpuart9 */
#define CCM_GPR_SHARED6_m7_mask_lpuart9(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart9_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart10_MASK    (0x8000U)
#define CCM_GPR_SHARED6_m7_mask_lpuart10_SHIFT   (15U)
/*! m7_mask_lpuart10 - m7_mask_lpuart10 */
#define CCM_GPR_SHARED6_m7_mask_lpuart10(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart10_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart11_MASK    (0x10000U)
#define CCM_GPR_SHARED6_m7_mask_lpuart11_SHIFT   (16U)
/*! m7_mask_lpuart11 - m7_mask_lpuart11 */
#define CCM_GPR_SHARED6_m7_mask_lpuart11(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart11_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpuart12_MASK    (0x20000U)
#define CCM_GPR_SHARED6_m7_mask_lpuart12_SHIFT   (17U)
/*! m7_mask_lpuart12 - m7_mask_lpuart12 */
#define CCM_GPR_SHARED6_m7_mask_lpuart12(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpuart12_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpi2c1_MASK      (0x40000U)
#define CCM_GPR_SHARED6_m7_mask_lpi2c1_SHIFT     (18U)
/*! m7_mask_lpi2c1 - m7_mask_lpi2c1 */
#define CCM_GPR_SHARED6_m7_mask_lpi2c1(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpi2c2_MASK      (0x80000U)
#define CCM_GPR_SHARED6_m7_mask_lpi2c2_SHIFT     (19U)
/*! m7_mask_lpi2c2 - m7_mask_lpi2c2 */
#define CCM_GPR_SHARED6_m7_mask_lpi2c2(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpi2c3_MASK      (0x100000U)
#define CCM_GPR_SHARED6_m7_mask_lpi2c3_SHIFT     (20U)
/*! m7_mask_lpi2c3 - m7_mask_lpi2c3 */
#define CCM_GPR_SHARED6_m7_mask_lpi2c3(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpi2c4_MASK      (0x200000U)
#define CCM_GPR_SHARED6_m7_mask_lpi2c4_SHIFT     (21U)
/*! m7_mask_lpi2c4 - m7_mask_lpi2c4 */
#define CCM_GPR_SHARED6_m7_mask_lpi2c4(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpi2c5_MASK      (0x400000U)
#define CCM_GPR_SHARED6_m7_mask_lpi2c5_SHIFT     (22U)
/*! m7_mask_lpi2c5 - m7_mask_lpi2c5 */
#define CCM_GPR_SHARED6_m7_mask_lpi2c5(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpi2c6_MASK      (0x800000U)
#define CCM_GPR_SHARED6_m7_mask_lpi2c6_SHIFT     (23U)
/*! m7_mask_lpi2c6 - m7_mask_lpi2c6 */
#define CCM_GPR_SHARED6_m7_mask_lpi2c6(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpspi1_MASK      (0x1000000U)
#define CCM_GPR_SHARED6_m7_mask_lpspi1_SHIFT     (24U)
/*! m7_mask_lpspi1 - m7_mask_lpspi1 */
#define CCM_GPR_SHARED6_m7_mask_lpspi1(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpspi1_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpspi2_MASK      (0x2000000U)
#define CCM_GPR_SHARED6_m7_mask_lpspi2_SHIFT     (25U)
/*! m7_mask_lpspi2 - m7_mask_lpspi2 */
#define CCM_GPR_SHARED6_m7_mask_lpspi2(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpspi2_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpspi3_MASK      (0x4000000U)
#define CCM_GPR_SHARED6_m7_mask_lpspi3_SHIFT     (26U)
/*! m7_mask_lpspi3 - m7_mask_lpspi3 */
#define CCM_GPR_SHARED6_m7_mask_lpspi3(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpspi3_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpspi4_MASK      (0x8000000U)
#define CCM_GPR_SHARED6_m7_mask_lpspi4_SHIFT     (27U)
/*! m7_mask_lpspi4 - m7_mask_lpspi4 */
#define CCM_GPR_SHARED6_m7_mask_lpspi4(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpspi4_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpspi5_MASK      (0x10000000U)
#define CCM_GPR_SHARED6_m7_mask_lpspi5_SHIFT     (28U)
/*! m7_mask_lpspi5 - m7_mask_lpspi5 */
#define CCM_GPR_SHARED6_m7_mask_lpspi5(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpspi5_MASK)

#define CCM_GPR_SHARED6_m7_mask_lpspi6_MASK      (0x20000000U)
#define CCM_GPR_SHARED6_m7_mask_lpspi6_SHIFT     (29U)
/*! m7_mask_lpspi6 - m7_mask_lpspi6 */
#define CCM_GPR_SHARED6_m7_mask_lpspi6(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED6_m7_mask_lpspi6_MASK)

#define CCM_GPR_SHARED6_m7_mask_sinc1_MASK       (0x40000000U)
#define CCM_GPR_SHARED6_m7_mask_sinc1_SHIFT      (30U)
/*! m7_mask_sinc1 - m7_mask_sinc1 */
#define CCM_GPR_SHARED6_m7_mask_sinc1(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_sinc1_SHIFT)) & CCM_GPR_SHARED6_m7_mask_sinc1_MASK)

#define CCM_GPR_SHARED6_m7_mask_sinc2_MASK       (0x80000000U)
#define CCM_GPR_SHARED6_m7_mask_sinc2_SHIFT      (31U)
/*! m7_mask_sinc2 - m7_mask_sinc2 */
#define CCM_GPR_SHARED6_m7_mask_sinc2(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_m7_mask_sinc2_SHIFT)) & CCM_GPR_SHARED6_m7_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED6_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED6_SET_m7_mask_tpm6_MASK    (0x1U)
#define CCM_GPR_SHARED6_SET_m7_mask_tpm6_SHIFT   (0U)
/*! m7_mask_tpm6 - m7_mask_tpm6 */
#define CCM_GPR_SHARED6_SET_m7_mask_tpm6(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_tpm6_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_tpm6_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_gpt1_MASK    (0x2U)
#define CCM_GPR_SHARED6_SET_m7_mask_gpt1_SHIFT   (1U)
/*! m7_mask_gpt1 - m7_mask_gpt1 */
#define CCM_GPR_SHARED6_SET_m7_mask_gpt1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_gpt1_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_gpt1_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_gpt2_MASK    (0x4U)
#define CCM_GPR_SHARED6_SET_m7_mask_gpt2_SHIFT   (2U)
/*! m7_mask_gpt2 - m7_mask_gpt2 */
#define CCM_GPR_SHARED6_SET_m7_mask_gpt2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_gpt2_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_gpt2_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_can1_MASK    (0x8U)
#define CCM_GPR_SHARED6_SET_m7_mask_can1_SHIFT   (3U)
/*! m7_mask_can1 - m7_mask_can1 */
#define CCM_GPR_SHARED6_SET_m7_mask_can1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_can1_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_can1_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_can2_MASK    (0x10U)
#define CCM_GPR_SHARED6_SET_m7_mask_can2_SHIFT   (4U)
/*! m7_mask_can2 - m7_mask_can2 */
#define CCM_GPR_SHARED6_SET_m7_mask_can2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_can2_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_can2_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_can3_MASK    (0x20U)
#define CCM_GPR_SHARED6_SET_m7_mask_can3_SHIFT   (5U)
/*! m7_mask_can3 - m7_mask_can3 */
#define CCM_GPR_SHARED6_SET_m7_mask_can3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_can3_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_can3_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart1_MASK (0x40U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart1_SHIFT (6U)
/*! m7_mask_lpuart1 - m7_mask_lpuart1 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart1_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart2_MASK (0x80U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart2_SHIFT (7U)
/*! m7_mask_lpuart2 - m7_mask_lpuart2 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart2_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart3_MASK (0x100U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart3_SHIFT (8U)
/*! m7_mask_lpuart3 - m7_mask_lpuart3 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart3_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart4_MASK (0x200U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart4_SHIFT (9U)
/*! m7_mask_lpuart4 - m7_mask_lpuart4 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart4_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart5_MASK (0x400U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart5_SHIFT (10U)
/*! m7_mask_lpuart5 - m7_mask_lpuart5 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart5_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart6_MASK (0x800U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart6_SHIFT (11U)
/*! m7_mask_lpuart6 - m7_mask_lpuart6 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart6_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart7_MASK (0x1000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart7_SHIFT (12U)
/*! m7_mask_lpuart7 - m7_mask_lpuart7 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart7(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart7_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart8_MASK (0x2000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart8_SHIFT (13U)
/*! m7_mask_lpuart8 - m7_mask_lpuart8 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart8(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart8_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart9_MASK (0x4000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart9_SHIFT (14U)
/*! m7_mask_lpuart9 - m7_mask_lpuart9 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart9(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart9_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart10_MASK (0x8000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart10_SHIFT (15U)
/*! m7_mask_lpuart10 - m7_mask_lpuart10 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart10(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart10_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart11_MASK (0x10000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart11_SHIFT (16U)
/*! m7_mask_lpuart11 - m7_mask_lpuart11 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart11(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart11_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpuart12_MASK (0x20000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart12_SHIFT (17U)
/*! m7_mask_lpuart12 - m7_mask_lpuart12 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpuart12(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpuart12_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c1_MASK  (0x40000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c1_SHIFT (18U)
/*! m7_mask_lpi2c1 - m7_mask_lpi2c1 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c2_MASK  (0x80000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c2_SHIFT (19U)
/*! m7_mask_lpi2c2 - m7_mask_lpi2c2 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c3_MASK  (0x100000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c3_SHIFT (20U)
/*! m7_mask_lpi2c3 - m7_mask_lpi2c3 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c4_MASK  (0x200000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c4_SHIFT (21U)
/*! m7_mask_lpi2c4 - m7_mask_lpi2c4 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c5_MASK  (0x400000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c5_SHIFT (22U)
/*! m7_mask_lpi2c5 - m7_mask_lpi2c5 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c6_MASK  (0x800000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c6_SHIFT (23U)
/*! m7_mask_lpi2c6 - m7_mask_lpi2c6 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpi2c6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpspi1_MASK  (0x1000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi1_SHIFT (24U)
/*! m7_mask_lpspi1 - m7_mask_lpspi1 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpspi1_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpspi2_MASK  (0x2000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi2_SHIFT (25U)
/*! m7_mask_lpspi2 - m7_mask_lpspi2 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpspi2_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpspi3_MASK  (0x4000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi3_SHIFT (26U)
/*! m7_mask_lpspi3 - m7_mask_lpspi3 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpspi3_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpspi4_MASK  (0x8000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi4_SHIFT (27U)
/*! m7_mask_lpspi4 - m7_mask_lpspi4 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpspi4_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpspi5_MASK  (0x10000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi5_SHIFT (28U)
/*! m7_mask_lpspi5 - m7_mask_lpspi5 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpspi5_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_lpspi6_MASK  (0x20000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi6_SHIFT (29U)
/*! m7_mask_lpspi6 - m7_mask_lpspi6 */
#define CCM_GPR_SHARED6_SET_m7_mask_lpspi6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_lpspi6_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_sinc1_MASK   (0x40000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_sinc1_SHIFT  (30U)
/*! m7_mask_sinc1 - m7_mask_sinc1 */
#define CCM_GPR_SHARED6_SET_m7_mask_sinc1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_sinc1_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_sinc1_MASK)

#define CCM_GPR_SHARED6_SET_m7_mask_sinc2_MASK   (0x80000000U)
#define CCM_GPR_SHARED6_SET_m7_mask_sinc2_SHIFT  (31U)
/*! m7_mask_sinc2 - m7_mask_sinc2 */
#define CCM_GPR_SHARED6_SET_m7_mask_sinc2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_SET_m7_mask_sinc2_SHIFT)) & CCM_GPR_SHARED6_SET_m7_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED6_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED6_CLR_m7_mask_tpm6_MASK    (0x1U)
#define CCM_GPR_SHARED6_CLR_m7_mask_tpm6_SHIFT   (0U)
/*! m7_mask_tpm6 - m7_mask_tpm6 */
#define CCM_GPR_SHARED6_CLR_m7_mask_tpm6(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_tpm6_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_tpm6_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_gpt1_MASK    (0x2U)
#define CCM_GPR_SHARED6_CLR_m7_mask_gpt1_SHIFT   (1U)
/*! m7_mask_gpt1 - m7_mask_gpt1 */
#define CCM_GPR_SHARED6_CLR_m7_mask_gpt1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_gpt1_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_gpt1_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_gpt2_MASK    (0x4U)
#define CCM_GPR_SHARED6_CLR_m7_mask_gpt2_SHIFT   (2U)
/*! m7_mask_gpt2 - m7_mask_gpt2 */
#define CCM_GPR_SHARED6_CLR_m7_mask_gpt2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_gpt2_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_gpt2_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_can1_MASK    (0x8U)
#define CCM_GPR_SHARED6_CLR_m7_mask_can1_SHIFT   (3U)
/*! m7_mask_can1 - m7_mask_can1 */
#define CCM_GPR_SHARED6_CLR_m7_mask_can1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_can1_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_can1_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_can2_MASK    (0x10U)
#define CCM_GPR_SHARED6_CLR_m7_mask_can2_SHIFT   (4U)
/*! m7_mask_can2 - m7_mask_can2 */
#define CCM_GPR_SHARED6_CLR_m7_mask_can2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_can2_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_can2_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_can3_MASK    (0x20U)
#define CCM_GPR_SHARED6_CLR_m7_mask_can3_SHIFT   (5U)
/*! m7_mask_can3 - m7_mask_can3 */
#define CCM_GPR_SHARED6_CLR_m7_mask_can3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_can3_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_can3_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart1_MASK (0x40U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart1_SHIFT (6U)
/*! m7_mask_lpuart1 - m7_mask_lpuart1 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart1_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart2_MASK (0x80U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart2_SHIFT (7U)
/*! m7_mask_lpuart2 - m7_mask_lpuart2 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart2_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart3_MASK (0x100U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart3_SHIFT (8U)
/*! m7_mask_lpuart3 - m7_mask_lpuart3 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart3_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart4_MASK (0x200U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart4_SHIFT (9U)
/*! m7_mask_lpuart4 - m7_mask_lpuart4 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart4_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart5_MASK (0x400U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart5_SHIFT (10U)
/*! m7_mask_lpuart5 - m7_mask_lpuart5 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart5_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart6_MASK (0x800U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart6_SHIFT (11U)
/*! m7_mask_lpuart6 - m7_mask_lpuart6 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart6_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart7_MASK (0x1000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart7_SHIFT (12U)
/*! m7_mask_lpuart7 - m7_mask_lpuart7 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart7(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart7_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart8_MASK (0x2000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart8_SHIFT (13U)
/*! m7_mask_lpuart8 - m7_mask_lpuart8 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart8(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart8_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart9_MASK (0x4000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart9_SHIFT (14U)
/*! m7_mask_lpuart9 - m7_mask_lpuart9 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart9(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart9_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart10_MASK (0x8000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart10_SHIFT (15U)
/*! m7_mask_lpuart10 - m7_mask_lpuart10 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart10(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart10_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart11_MASK (0x10000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart11_SHIFT (16U)
/*! m7_mask_lpuart11 - m7_mask_lpuart11 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart11(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart11_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart12_MASK (0x20000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart12_SHIFT (17U)
/*! m7_mask_lpuart12 - m7_mask_lpuart12 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpuart12(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpuart12_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c1_MASK  (0x40000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c1_SHIFT (18U)
/*! m7_mask_lpi2c1 - m7_mask_lpi2c1 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c2_MASK  (0x80000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c2_SHIFT (19U)
/*! m7_mask_lpi2c2 - m7_mask_lpi2c2 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c3_MASK  (0x100000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c3_SHIFT (20U)
/*! m7_mask_lpi2c3 - m7_mask_lpi2c3 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c4_MASK  (0x200000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c4_SHIFT (21U)
/*! m7_mask_lpi2c4 - m7_mask_lpi2c4 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c5_MASK  (0x400000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c5_SHIFT (22U)
/*! m7_mask_lpi2c5 - m7_mask_lpi2c5 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c6_MASK  (0x800000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c6_SHIFT (23U)
/*! m7_mask_lpi2c6 - m7_mask_lpi2c6 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpi2c6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi1_MASK  (0x1000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi1_SHIFT (24U)
/*! m7_mask_lpspi1 - m7_mask_lpspi1 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpspi1_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi2_MASK  (0x2000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi2_SHIFT (25U)
/*! m7_mask_lpspi2 - m7_mask_lpspi2 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpspi2_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi3_MASK  (0x4000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi3_SHIFT (26U)
/*! m7_mask_lpspi3 - m7_mask_lpspi3 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpspi3_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi4_MASK  (0x8000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi4_SHIFT (27U)
/*! m7_mask_lpspi4 - m7_mask_lpspi4 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpspi4_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi5_MASK  (0x10000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi5_SHIFT (28U)
/*! m7_mask_lpspi5 - m7_mask_lpspi5 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpspi5_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi6_MASK  (0x20000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi6_SHIFT (29U)
/*! m7_mask_lpspi6 - m7_mask_lpspi6 */
#define CCM_GPR_SHARED6_CLR_m7_mask_lpspi6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_lpspi6_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_sinc1_MASK   (0x40000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_sinc1_SHIFT  (30U)
/*! m7_mask_sinc1 - m7_mask_sinc1 */
#define CCM_GPR_SHARED6_CLR_m7_mask_sinc1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_sinc1_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_sinc1_MASK)

#define CCM_GPR_SHARED6_CLR_m7_mask_sinc2_MASK   (0x80000000U)
#define CCM_GPR_SHARED6_CLR_m7_mask_sinc2_SHIFT  (31U)
/*! m7_mask_sinc2 - m7_mask_sinc2 */
#define CCM_GPR_SHARED6_CLR_m7_mask_sinc2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_CLR_m7_mask_sinc2_SHIFT)) & CCM_GPR_SHARED6_CLR_m7_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED6_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED6_TOG_m7_mask_tpm6_MASK    (0x1U)
#define CCM_GPR_SHARED6_TOG_m7_mask_tpm6_SHIFT   (0U)
/*! m7_mask_tpm6 - m7_mask_tpm6 */
#define CCM_GPR_SHARED6_TOG_m7_mask_tpm6(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_tpm6_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_tpm6_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_gpt1_MASK    (0x2U)
#define CCM_GPR_SHARED6_TOG_m7_mask_gpt1_SHIFT   (1U)
/*! m7_mask_gpt1 - m7_mask_gpt1 */
#define CCM_GPR_SHARED6_TOG_m7_mask_gpt1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_gpt1_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_gpt1_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_gpt2_MASK    (0x4U)
#define CCM_GPR_SHARED6_TOG_m7_mask_gpt2_SHIFT   (2U)
/*! m7_mask_gpt2 - m7_mask_gpt2 */
#define CCM_GPR_SHARED6_TOG_m7_mask_gpt2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_gpt2_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_gpt2_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_can1_MASK    (0x8U)
#define CCM_GPR_SHARED6_TOG_m7_mask_can1_SHIFT   (3U)
/*! m7_mask_can1 - m7_mask_can1 */
#define CCM_GPR_SHARED6_TOG_m7_mask_can1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_can1_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_can1_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_can2_MASK    (0x10U)
#define CCM_GPR_SHARED6_TOG_m7_mask_can2_SHIFT   (4U)
/*! m7_mask_can2 - m7_mask_can2 */
#define CCM_GPR_SHARED6_TOG_m7_mask_can2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_can2_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_can2_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_can3_MASK    (0x20U)
#define CCM_GPR_SHARED6_TOG_m7_mask_can3_SHIFT   (5U)
/*! m7_mask_can3 - m7_mask_can3 */
#define CCM_GPR_SHARED6_TOG_m7_mask_can3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_can3_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_can3_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart1_MASK (0x40U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart1_SHIFT (6U)
/*! m7_mask_lpuart1 - m7_mask_lpuart1 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart1(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart1_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart1_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart2_MASK (0x80U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart2_SHIFT (7U)
/*! m7_mask_lpuart2 - m7_mask_lpuart2 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart2(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart2_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart2_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart3_MASK (0x100U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart3_SHIFT (8U)
/*! m7_mask_lpuart3 - m7_mask_lpuart3 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart3(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart3_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart3_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart4_MASK (0x200U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart4_SHIFT (9U)
/*! m7_mask_lpuart4 - m7_mask_lpuart4 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart4(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart4_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart4_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart5_MASK (0x400U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart5_SHIFT (10U)
/*! m7_mask_lpuart5 - m7_mask_lpuart5 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart5(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart5_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart5_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart6_MASK (0x800U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart6_SHIFT (11U)
/*! m7_mask_lpuart6 - m7_mask_lpuart6 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart6(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart6_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart6_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart7_MASK (0x1000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart7_SHIFT (12U)
/*! m7_mask_lpuart7 - m7_mask_lpuart7 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart7(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart7_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart7_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart8_MASK (0x2000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart8_SHIFT (13U)
/*! m7_mask_lpuart8 - m7_mask_lpuart8 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart8(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart8_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart8_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart9_MASK (0x4000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart9_SHIFT (14U)
/*! m7_mask_lpuart9 - m7_mask_lpuart9 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart9(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart9_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart9_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart10_MASK (0x8000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart10_SHIFT (15U)
/*! m7_mask_lpuart10 - m7_mask_lpuart10 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart10(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart10_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart10_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart11_MASK (0x10000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart11_SHIFT (16U)
/*! m7_mask_lpuart11 - m7_mask_lpuart11 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart11(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart11_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart11_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart12_MASK (0x20000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart12_SHIFT (17U)
/*! m7_mask_lpuart12 - m7_mask_lpuart12 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpuart12(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpuart12_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpuart12_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c1_MASK  (0x40000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c1_SHIFT (18U)
/*! m7_mask_lpi2c1 - m7_mask_lpi2c1 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpi2c1_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpi2c1_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c2_MASK  (0x80000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c2_SHIFT (19U)
/*! m7_mask_lpi2c2 - m7_mask_lpi2c2 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpi2c2_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpi2c2_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c3_MASK  (0x100000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c3_SHIFT (20U)
/*! m7_mask_lpi2c3 - m7_mask_lpi2c3 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpi2c3_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpi2c3_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c4_MASK  (0x200000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c4_SHIFT (21U)
/*! m7_mask_lpi2c4 - m7_mask_lpi2c4 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpi2c4_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpi2c4_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c5_MASK  (0x400000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c5_SHIFT (22U)
/*! m7_mask_lpi2c5 - m7_mask_lpi2c5 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpi2c5_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpi2c5_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c6_MASK  (0x800000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c6_SHIFT (23U)
/*! m7_mask_lpi2c6 - m7_mask_lpi2c6 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpi2c6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpi2c6_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpi2c6_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi1_MASK  (0x1000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi1_SHIFT (24U)
/*! m7_mask_lpspi1 - m7_mask_lpspi1 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi1(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpspi1_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpspi1_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi2_MASK  (0x2000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi2_SHIFT (25U)
/*! m7_mask_lpspi2 - m7_mask_lpspi2 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi2(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpspi2_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpspi2_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi3_MASK  (0x4000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi3_SHIFT (26U)
/*! m7_mask_lpspi3 - m7_mask_lpspi3 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi3(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpspi3_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpspi3_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi4_MASK  (0x8000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi4_SHIFT (27U)
/*! m7_mask_lpspi4 - m7_mask_lpspi4 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi4(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpspi4_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpspi4_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi5_MASK  (0x10000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi5_SHIFT (28U)
/*! m7_mask_lpspi5 - m7_mask_lpspi5 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi5(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpspi5_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpspi5_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi6_MASK  (0x20000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi6_SHIFT (29U)
/*! m7_mask_lpspi6 - m7_mask_lpspi6 */
#define CCM_GPR_SHARED6_TOG_m7_mask_lpspi6(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_lpspi6_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_lpspi6_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_sinc1_MASK   (0x40000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_sinc1_SHIFT  (30U)
/*! m7_mask_sinc1 - m7_mask_sinc1 */
#define CCM_GPR_SHARED6_TOG_m7_mask_sinc1(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_sinc1_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_sinc1_MASK)

#define CCM_GPR_SHARED6_TOG_m7_mask_sinc2_MASK   (0x80000000U)
#define CCM_GPR_SHARED6_TOG_m7_mask_sinc2_SHIFT  (31U)
/*! m7_mask_sinc2 - m7_mask_sinc2 */
#define CCM_GPR_SHARED6_TOG_m7_mask_sinc2(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_TOG_m7_mask_sinc2_SHIFT)) & CCM_GPR_SHARED6_TOG_m7_mask_sinc2_MASK)
/*! @} */

/*! @name GPR_SHARED6_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED6_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED6_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED6_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED6_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED6_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED6_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED6_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED6_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED6_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED6_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED6_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED6_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED6_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED6_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED6_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED6_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED6_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED6_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED6_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED6_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED6_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED6_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED6_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED6_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED6_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED6_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED6_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED6_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED6_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED6_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED6_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED6_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED6_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED6_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED6_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED6_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED6_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED6_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED6_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED6_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED6_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED6_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED6_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED6_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED6_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED6_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED6_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED6_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED6_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED6_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED6_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED6_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED6_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED7 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED7_m7_mask_sinc3_MASK       (0x1U)
#define CCM_GPR_SHARED7_m7_mask_sinc3_SHIFT      (0U)
/*! m7_mask_sinc3 - m7_mask_sinc3 */
#define CCM_GPR_SHARED7_m7_mask_sinc3(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_m7_mask_sinc3_SHIFT)) & CCM_GPR_SHARED7_m7_mask_sinc3_MASK)

#define CCM_GPR_SHARED7_m7_mask_sai1_MASK        (0x2U)
#define CCM_GPR_SHARED7_m7_mask_sai1_SHIFT       (1U)
/*! m7_mask_sai1 - m7_mask_sai1 */
#define CCM_GPR_SHARED7_m7_mask_sai1(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_m7_mask_sai1_SHIFT)) & CCM_GPR_SHARED7_m7_mask_sai1_MASK)

#define CCM_GPR_SHARED7_m7_mask_sai2_MASK        (0x4U)
#define CCM_GPR_SHARED7_m7_mask_sai2_SHIFT       (2U)
/*! m7_mask_sai2 - m7_mask_sai2 */
#define CCM_GPR_SHARED7_m7_mask_sai2(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_m7_mask_sai2_SHIFT)) & CCM_GPR_SHARED7_m7_mask_sai2_MASK)

#define CCM_GPR_SHARED7_m7_mask_sai3_MASK        (0x8U)
#define CCM_GPR_SHARED7_m7_mask_sai3_SHIFT       (3U)
/*! m7_mask_sai3 - m7_mask_sai3 */
#define CCM_GPR_SHARED7_m7_mask_sai3(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_m7_mask_sai3_SHIFT)) & CCM_GPR_SHARED7_m7_mask_sai3_MASK)

#define CCM_GPR_SHARED7_m7_mask_sai4_MASK        (0x10U)
#define CCM_GPR_SHARED7_m7_mask_sai4_SHIFT       (4U)
/*! m7_mask_sai4 - m7_mask_sai4 */
#define CCM_GPR_SHARED7_m7_mask_sai4(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_m7_mask_sai4_SHIFT)) & CCM_GPR_SHARED7_m7_mask_sai4_MASK)

#define CCM_GPR_SHARED7_m7_mask_mic_MASK         (0x20U)
#define CCM_GPR_SHARED7_m7_mask_mic_SHIFT        (5U)
/*! m7_mask_mic - m7_mask_mic */
#define CCM_GPR_SHARED7_m7_mask_mic(x)           (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_m7_mask_mic_SHIFT)) & CCM_GPR_SHARED7_m7_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED7_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED7_SET_m7_mask_sinc3_MASK   (0x1U)
#define CCM_GPR_SHARED7_SET_m7_mask_sinc3_SHIFT  (0U)
/*! m7_mask_sinc3 - m7_mask_sinc3 */
#define CCM_GPR_SHARED7_SET_m7_mask_sinc3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_SET_m7_mask_sinc3_SHIFT)) & CCM_GPR_SHARED7_SET_m7_mask_sinc3_MASK)

#define CCM_GPR_SHARED7_SET_m7_mask_sai1_MASK    (0x2U)
#define CCM_GPR_SHARED7_SET_m7_mask_sai1_SHIFT   (1U)
/*! m7_mask_sai1 - m7_mask_sai1 */
#define CCM_GPR_SHARED7_SET_m7_mask_sai1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_SET_m7_mask_sai1_SHIFT)) & CCM_GPR_SHARED7_SET_m7_mask_sai1_MASK)

#define CCM_GPR_SHARED7_SET_m7_mask_sai2_MASK    (0x4U)
#define CCM_GPR_SHARED7_SET_m7_mask_sai2_SHIFT   (2U)
/*! m7_mask_sai2 - m7_mask_sai2 */
#define CCM_GPR_SHARED7_SET_m7_mask_sai2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_SET_m7_mask_sai2_SHIFT)) & CCM_GPR_SHARED7_SET_m7_mask_sai2_MASK)

#define CCM_GPR_SHARED7_SET_m7_mask_sai3_MASK    (0x8U)
#define CCM_GPR_SHARED7_SET_m7_mask_sai3_SHIFT   (3U)
/*! m7_mask_sai3 - m7_mask_sai3 */
#define CCM_GPR_SHARED7_SET_m7_mask_sai3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_SET_m7_mask_sai3_SHIFT)) & CCM_GPR_SHARED7_SET_m7_mask_sai3_MASK)

#define CCM_GPR_SHARED7_SET_m7_mask_sai4_MASK    (0x10U)
#define CCM_GPR_SHARED7_SET_m7_mask_sai4_SHIFT   (4U)
/*! m7_mask_sai4 - m7_mask_sai4 */
#define CCM_GPR_SHARED7_SET_m7_mask_sai4(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_SET_m7_mask_sai4_SHIFT)) & CCM_GPR_SHARED7_SET_m7_mask_sai4_MASK)

#define CCM_GPR_SHARED7_SET_m7_mask_mic_MASK     (0x20U)
#define CCM_GPR_SHARED7_SET_m7_mask_mic_SHIFT    (5U)
/*! m7_mask_mic - m7_mask_mic */
#define CCM_GPR_SHARED7_SET_m7_mask_mic(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_SET_m7_mask_mic_SHIFT)) & CCM_GPR_SHARED7_SET_m7_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED7_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED7_CLR_m7_mask_sinc3_MASK   (0x1U)
#define CCM_GPR_SHARED7_CLR_m7_mask_sinc3_SHIFT  (0U)
/*! m7_mask_sinc3 - m7_mask_sinc3 */
#define CCM_GPR_SHARED7_CLR_m7_mask_sinc3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_CLR_m7_mask_sinc3_SHIFT)) & CCM_GPR_SHARED7_CLR_m7_mask_sinc3_MASK)

#define CCM_GPR_SHARED7_CLR_m7_mask_sai1_MASK    (0x2U)
#define CCM_GPR_SHARED7_CLR_m7_mask_sai1_SHIFT   (1U)
/*! m7_mask_sai1 - m7_mask_sai1 */
#define CCM_GPR_SHARED7_CLR_m7_mask_sai1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_CLR_m7_mask_sai1_SHIFT)) & CCM_GPR_SHARED7_CLR_m7_mask_sai1_MASK)

#define CCM_GPR_SHARED7_CLR_m7_mask_sai2_MASK    (0x4U)
#define CCM_GPR_SHARED7_CLR_m7_mask_sai2_SHIFT   (2U)
/*! m7_mask_sai2 - m7_mask_sai2 */
#define CCM_GPR_SHARED7_CLR_m7_mask_sai2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_CLR_m7_mask_sai2_SHIFT)) & CCM_GPR_SHARED7_CLR_m7_mask_sai2_MASK)

#define CCM_GPR_SHARED7_CLR_m7_mask_sai3_MASK    (0x8U)
#define CCM_GPR_SHARED7_CLR_m7_mask_sai3_SHIFT   (3U)
/*! m7_mask_sai3 - m7_mask_sai3 */
#define CCM_GPR_SHARED7_CLR_m7_mask_sai3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_CLR_m7_mask_sai3_SHIFT)) & CCM_GPR_SHARED7_CLR_m7_mask_sai3_MASK)

#define CCM_GPR_SHARED7_CLR_m7_mask_sai4_MASK    (0x10U)
#define CCM_GPR_SHARED7_CLR_m7_mask_sai4_SHIFT   (4U)
/*! m7_mask_sai4 - m7_mask_sai4 */
#define CCM_GPR_SHARED7_CLR_m7_mask_sai4(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_CLR_m7_mask_sai4_SHIFT)) & CCM_GPR_SHARED7_CLR_m7_mask_sai4_MASK)

#define CCM_GPR_SHARED7_CLR_m7_mask_mic_MASK     (0x20U)
#define CCM_GPR_SHARED7_CLR_m7_mask_mic_SHIFT    (5U)
/*! m7_mask_mic - m7_mask_mic */
#define CCM_GPR_SHARED7_CLR_m7_mask_mic(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_CLR_m7_mask_mic_SHIFT)) & CCM_GPR_SHARED7_CLR_m7_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED7_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED7_TOG_m7_mask_sinc3_MASK   (0x1U)
#define CCM_GPR_SHARED7_TOG_m7_mask_sinc3_SHIFT  (0U)
/*! m7_mask_sinc3 - m7_mask_sinc3 */
#define CCM_GPR_SHARED7_TOG_m7_mask_sinc3(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_TOG_m7_mask_sinc3_SHIFT)) & CCM_GPR_SHARED7_TOG_m7_mask_sinc3_MASK)

#define CCM_GPR_SHARED7_TOG_m7_mask_sai1_MASK    (0x2U)
#define CCM_GPR_SHARED7_TOG_m7_mask_sai1_SHIFT   (1U)
/*! m7_mask_sai1 - m7_mask_sai1 */
#define CCM_GPR_SHARED7_TOG_m7_mask_sai1(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_TOG_m7_mask_sai1_SHIFT)) & CCM_GPR_SHARED7_TOG_m7_mask_sai1_MASK)

#define CCM_GPR_SHARED7_TOG_m7_mask_sai2_MASK    (0x4U)
#define CCM_GPR_SHARED7_TOG_m7_mask_sai2_SHIFT   (2U)
/*! m7_mask_sai2 - m7_mask_sai2 */
#define CCM_GPR_SHARED7_TOG_m7_mask_sai2(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_TOG_m7_mask_sai2_SHIFT)) & CCM_GPR_SHARED7_TOG_m7_mask_sai2_MASK)

#define CCM_GPR_SHARED7_TOG_m7_mask_sai3_MASK    (0x8U)
#define CCM_GPR_SHARED7_TOG_m7_mask_sai3_SHIFT   (3U)
/*! m7_mask_sai3 - m7_mask_sai3 */
#define CCM_GPR_SHARED7_TOG_m7_mask_sai3(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_TOG_m7_mask_sai3_SHIFT)) & CCM_GPR_SHARED7_TOG_m7_mask_sai3_MASK)

#define CCM_GPR_SHARED7_TOG_m7_mask_sai4_MASK    (0x10U)
#define CCM_GPR_SHARED7_TOG_m7_mask_sai4_SHIFT   (4U)
/*! m7_mask_sai4 - m7_mask_sai4 */
#define CCM_GPR_SHARED7_TOG_m7_mask_sai4(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_TOG_m7_mask_sai4_SHIFT)) & CCM_GPR_SHARED7_TOG_m7_mask_sai4_MASK)

#define CCM_GPR_SHARED7_TOG_m7_mask_mic_MASK     (0x20U)
#define CCM_GPR_SHARED7_TOG_m7_mask_mic_SHIFT    (5U)
/*! m7_mask_mic - m7_mask_mic */
#define CCM_GPR_SHARED7_TOG_m7_mask_mic(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_TOG_m7_mask_mic_SHIFT)) & CCM_GPR_SHARED7_TOG_m7_mask_mic_MASK)
/*! @} */

/*! @name GPR_SHARED7_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED7_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED7_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED7_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED7_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED7_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED7_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED7_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED7_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED7_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED7_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED7_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED7_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED7_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED7_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED7_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED7_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED7_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED7_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED7_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED7_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED7_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED7_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED7_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED7_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED7_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED7_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED7_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED7_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED7_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED7_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED7_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED7_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED7_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED7_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED7_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED7_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED7_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED7_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED7_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED7_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED7_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED7_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED7_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED7_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED7_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED7_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED7_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED7_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED7_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED7_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED7_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED7_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED7_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED8 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED8_m33_cm7_ipg_stop_MASK    (0x1U)
#define CCM_GPR_SHARED8_m33_cm7_ipg_stop_SHIFT   (0U)
/*! m33_cm7_ipg_stop - m33_cm7_ipg_stop */
#define CCM_GPR_SHARED8_m33_cm7_ipg_stop(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_cm33_ipg_stop_MASK   (0x2U)
#define CCM_GPR_SHARED8_m33_cm33_ipg_stop_SHIFT  (1U)
/*! m33_cm33_ipg_stop - m33_cm33_ipg_stop */
#define CCM_GPR_SHARED8_m33_cm33_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_edma3_ipg_stop_MASK  (0x4U)
#define CCM_GPR_SHARED8_m33_edma3_ipg_stop_SHIFT (2U)
/*! m33_edma3_ipg_stop - m33_edma3_ipg_stop */
#define CCM_GPR_SHARED8_m33_edma3_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_edma4_ipg_stop_MASK  (0x8U)
#define CCM_GPR_SHARED8_m33_edma4_ipg_stop_SHIFT (3U)
/*! m33_edma4_ipg_stop - m33_edma4_ipg_stop */
#define CCM_GPR_SHARED8_m33_edma4_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_netc_ipg_stop_MASK   (0x10U)
#define CCM_GPR_SHARED8_m33_netc_ipg_stop_SHIFT  (4U)
/*! m33_netc_ipg_stop - m33_netc_ipg_stop */
#define CCM_GPR_SHARED8_m33_netc_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED8_m33_sim_aon_ipg_stop_SHIFT (8U)
/*! m33_sim_aon_ipg_stop - m33_sim_aon_ipg_stop */
#define CCM_GPR_SHARED8_m33_sim_aon_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_adc1_ipg_stop_MASK   (0x200U)
#define CCM_GPR_SHARED8_m33_adc1_ipg_stop_SHIFT  (9U)
/*! m33_adc1_ipg_stop - m33_adc1_ipg_stop */
#define CCM_GPR_SHARED8_m33_adc1_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_adc2_ipg_stop_MASK   (0x400U)
#define CCM_GPR_SHARED8_m33_adc2_ipg_stop_SHIFT  (10U)
/*! m33_adc2_ipg_stop - m33_adc2_ipg_stop */
#define CCM_GPR_SHARED8_m33_adc2_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED8_m33_flexspi1_ipg_stop_SHIFT (11U)
/*! m33_flexspi1_ipg_stop - m33_flexspi1_ipg_stop */
#define CCM_GPR_SHARED8_m33_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED8_m33_flexspi2_ipg_stop_SHIFT (12U)
/*! m33_flexspi2_ipg_stop - m33_flexspi2_ipg_stop */
#define CCM_GPR_SHARED8_m33_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_trdc_ipg_stop_MASK   (0x2000U)
#define CCM_GPR_SHARED8_m33_trdc_ipg_stop_SHIFT  (13U)
/*! m33_trdc_ipg_stop - m33_trdc_ipg_stop */
#define CCM_GPR_SHARED8_m33_trdc_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_semc_ipg_stop_MASK   (0x4000U)
#define CCM_GPR_SHARED8_m33_semc_ipg_stop_SHIFT  (14U)
/*! m33_semc_ipg_stop - m33_semc_ipg_stop */
#define CCM_GPR_SHARED8_m33_semc_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_iee_ipg_stop_MASK    (0x8000U)
#define CCM_GPR_SHARED8_m33_iee_ipg_stop_SHIFT   (15U)
/*! m33_iee_ipg_stop - m33_iee_ipg_stop */
#define CCM_GPR_SHARED8_m33_iee_ipg_stop(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_gpio1_ipg_stop_MASK  (0x10000U)
#define CCM_GPR_SHARED8_m33_gpio1_ipg_stop_SHIFT (16U)
/*! m33_gpio1_ipg_stop - m33_gpio1_ipg_stop */
#define CCM_GPR_SHARED8_m33_gpio1_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_gpio2_ipg_stop_MASK  (0x20000U)
#define CCM_GPR_SHARED8_m33_gpio2_ipg_stop_SHIFT (17U)
/*! m33_gpio2_ipg_stop - m33_gpio2_ipg_stop */
#define CCM_GPR_SHARED8_m33_gpio2_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_gpio3_ipg_stop_MASK  (0x40000U)
#define CCM_GPR_SHARED8_m33_gpio3_ipg_stop_SHIFT (18U)
/*! m33_gpio3_ipg_stop - m33_gpio3_ipg_stop */
#define CCM_GPR_SHARED8_m33_gpio3_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_gpio4_ipg_stop_MASK  (0x80000U)
#define CCM_GPR_SHARED8_m33_gpio4_ipg_stop_SHIFT (19U)
/*! m33_gpio4_ipg_stop - m33_gpio4_ipg_stop */
#define CCM_GPR_SHARED8_m33_gpio4_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_gpio5_ipg_stop_MASK  (0x100000U)
#define CCM_GPR_SHARED8_m33_gpio5_ipg_stop_SHIFT (20U)
/*! m33_gpio5_ipg_stop - m33_gpio5_ipg_stop */
#define CCM_GPR_SHARED8_m33_gpio5_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_gpio6_ipg_stop_MASK  (0x200000U)
#define CCM_GPR_SHARED8_m33_gpio6_ipg_stop_SHIFT (21U)
/*! m33_gpio6_ipg_stop - m33_gpio6_ipg_stop */
#define CCM_GPR_SHARED8_m33_gpio6_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED8_m33_flexio1_ipg_stop_SHIFT (22U)
/*! m33_flexio1_ipg_stop - m33_flexio1_ipg_stop */
#define CCM_GPR_SHARED8_m33_flexio1_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED8_m33_flexio2_ipg_stop_SHIFT (23U)
/*! m33_flexio2_ipg_stop - m33_flexio2_ipg_stop */
#define CCM_GPR_SHARED8_m33_flexio2_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_can1_ipg_stop_MASK   (0x1000000U)
#define CCM_GPR_SHARED8_m33_can1_ipg_stop_SHIFT  (24U)
/*! m33_can1_ipg_stop - m33_can1_ipg_stop */
#define CCM_GPR_SHARED8_m33_can1_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_can2_ipg_stop_MASK   (0x2000000U)
#define CCM_GPR_SHARED8_m33_can2_ipg_stop_SHIFT  (25U)
/*! m33_can2_ipg_stop - m33_can2_ipg_stop */
#define CCM_GPR_SHARED8_m33_can2_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_can3_ipg_stop_MASK   (0x4000000U)
#define CCM_GPR_SHARED8_m33_can3_ipg_stop_SHIFT  (26U)
/*! m33_can3_ipg_stop - m33_can3_ipg_stop */
#define CCM_GPR_SHARED8_m33_can3_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED8_m33_lpuart1_ipg_stop_SHIFT (27U)
/*! m33_lpuart1_ipg_stop - m33_lpuart1_ipg_stop */
#define CCM_GPR_SHARED8_m33_lpuart1_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED8_m33_lpuart2_ipg_stop_SHIFT (28U)
/*! m33_lpuart2_ipg_stop - m33_lpuart2_ipg_stop */
#define CCM_GPR_SHARED8_m33_lpuart2_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED8_m33_lpuart3_ipg_stop_SHIFT (29U)
/*! m33_lpuart3_ipg_stop - m33_lpuart3_ipg_stop */
#define CCM_GPR_SHARED8_m33_lpuart3_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED8_m33_lpuart4_ipg_stop_SHIFT (30U)
/*! m33_lpuart4_ipg_stop - m33_lpuart4_ipg_stop */
#define CCM_GPR_SHARED8_m33_lpuart4_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_m33_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED8_m33_lpuart5_ipg_stop_SHIFT (31U)
/*! m33_lpuart5_ipg_stop - m33_lpuart5_ipg_stop */
#define CCM_GPR_SHARED8_m33_lpuart5_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_m33_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_m33_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED8_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED8_SET_m33_cm7_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED8_SET_m33_cm7_ipg_stop_SHIFT (0U)
/*! m33_cm7_ipg_stop - m33_cm7_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_cm7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_cm33_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED8_SET_m33_cm33_ipg_stop_SHIFT (1U)
/*! m33_cm33_ipg_stop - m33_cm33_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_cm33_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_edma3_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED8_SET_m33_edma3_ipg_stop_SHIFT (2U)
/*! m33_edma3_ipg_stop - m33_edma3_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_edma3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_edma4_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED8_SET_m33_edma4_ipg_stop_SHIFT (3U)
/*! m33_edma4_ipg_stop - m33_edma4_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_edma4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_netc_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED8_SET_m33_netc_ipg_stop_SHIFT (4U)
/*! m33_netc_ipg_stop - m33_netc_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_netc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED8_SET_m33_sim_aon_ipg_stop_SHIFT (8U)
/*! m33_sim_aon_ipg_stop - m33_sim_aon_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_sim_aon_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_adc1_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED8_SET_m33_adc1_ipg_stop_SHIFT (9U)
/*! m33_adc1_ipg_stop - m33_adc1_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_adc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_adc2_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED8_SET_m33_adc2_ipg_stop_SHIFT (10U)
/*! m33_adc2_ipg_stop - m33_adc2_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_adc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED8_SET_m33_flexspi1_ipg_stop_SHIFT (11U)
/*! m33_flexspi1_ipg_stop - m33_flexspi1_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED8_SET_m33_flexspi2_ipg_stop_SHIFT (12U)
/*! m33_flexspi2_ipg_stop - m33_flexspi2_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_trdc_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED8_SET_m33_trdc_ipg_stop_SHIFT (13U)
/*! m33_trdc_ipg_stop - m33_trdc_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_trdc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_semc_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED8_SET_m33_semc_ipg_stop_SHIFT (14U)
/*! m33_semc_ipg_stop - m33_semc_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_semc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_iee_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED8_SET_m33_iee_ipg_stop_SHIFT (15U)
/*! m33_iee_ipg_stop - m33_iee_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_iee_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_gpio1_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED8_SET_m33_gpio1_ipg_stop_SHIFT (16U)
/*! m33_gpio1_ipg_stop - m33_gpio1_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_gpio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_gpio2_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED8_SET_m33_gpio2_ipg_stop_SHIFT (17U)
/*! m33_gpio2_ipg_stop - m33_gpio2_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_gpio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_gpio3_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED8_SET_m33_gpio3_ipg_stop_SHIFT (18U)
/*! m33_gpio3_ipg_stop - m33_gpio3_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_gpio3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_gpio4_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED8_SET_m33_gpio4_ipg_stop_SHIFT (19U)
/*! m33_gpio4_ipg_stop - m33_gpio4_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_gpio4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_gpio5_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED8_SET_m33_gpio5_ipg_stop_SHIFT (20U)
/*! m33_gpio5_ipg_stop - m33_gpio5_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_gpio5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_gpio6_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED8_SET_m33_gpio6_ipg_stop_SHIFT (21U)
/*! m33_gpio6_ipg_stop - m33_gpio6_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_gpio6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED8_SET_m33_flexio1_ipg_stop_SHIFT (22U)
/*! m33_flexio1_ipg_stop - m33_flexio1_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_flexio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED8_SET_m33_flexio2_ipg_stop_SHIFT (23U)
/*! m33_flexio2_ipg_stop - m33_flexio2_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_flexio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_can1_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED8_SET_m33_can1_ipg_stop_SHIFT (24U)
/*! m33_can1_ipg_stop - m33_can1_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_can1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_can2_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED8_SET_m33_can2_ipg_stop_SHIFT (25U)
/*! m33_can2_ipg_stop - m33_can2_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_can2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_can3_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED8_SET_m33_can3_ipg_stop_SHIFT (26U)
/*! m33_can3_ipg_stop - m33_can3_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_can3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED8_SET_m33_lpuart1_ipg_stop_SHIFT (27U)
/*! m33_lpuart1_ipg_stop - m33_lpuart1_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_lpuart1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED8_SET_m33_lpuart2_ipg_stop_SHIFT (28U)
/*! m33_lpuart2_ipg_stop - m33_lpuart2_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_lpuart2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED8_SET_m33_lpuart3_ipg_stop_SHIFT (29U)
/*! m33_lpuart3_ipg_stop - m33_lpuart3_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_lpuart3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED8_SET_m33_lpuart4_ipg_stop_SHIFT (30U)
/*! m33_lpuart4_ipg_stop - m33_lpuart4_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_lpuart4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_SET_m33_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED8_SET_m33_lpuart5_ipg_stop_SHIFT (31U)
/*! m33_lpuart5_ipg_stop - m33_lpuart5_ipg_stop */
#define CCM_GPR_SHARED8_SET_m33_lpuart5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_SET_m33_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_SET_m33_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED8_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED8_CLR_m33_cm7_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED8_CLR_m33_cm7_ipg_stop_SHIFT (0U)
/*! m33_cm7_ipg_stop - m33_cm7_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_cm7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_cm33_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED8_CLR_m33_cm33_ipg_stop_SHIFT (1U)
/*! m33_cm33_ipg_stop - m33_cm33_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_cm33_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_edma3_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED8_CLR_m33_edma3_ipg_stop_SHIFT (2U)
/*! m33_edma3_ipg_stop - m33_edma3_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_edma3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_edma4_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED8_CLR_m33_edma4_ipg_stop_SHIFT (3U)
/*! m33_edma4_ipg_stop - m33_edma4_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_edma4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_netc_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED8_CLR_m33_netc_ipg_stop_SHIFT (4U)
/*! m33_netc_ipg_stop - m33_netc_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_netc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED8_CLR_m33_sim_aon_ipg_stop_SHIFT (8U)
/*! m33_sim_aon_ipg_stop - m33_sim_aon_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_sim_aon_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_adc1_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED8_CLR_m33_adc1_ipg_stop_SHIFT (9U)
/*! m33_adc1_ipg_stop - m33_adc1_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_adc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_adc2_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED8_CLR_m33_adc2_ipg_stop_SHIFT (10U)
/*! m33_adc2_ipg_stop - m33_adc2_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_adc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED8_CLR_m33_flexspi1_ipg_stop_SHIFT (11U)
/*! m33_flexspi1_ipg_stop - m33_flexspi1_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED8_CLR_m33_flexspi2_ipg_stop_SHIFT (12U)
/*! m33_flexspi2_ipg_stop - m33_flexspi2_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_trdc_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED8_CLR_m33_trdc_ipg_stop_SHIFT (13U)
/*! m33_trdc_ipg_stop - m33_trdc_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_trdc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_semc_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED8_CLR_m33_semc_ipg_stop_SHIFT (14U)
/*! m33_semc_ipg_stop - m33_semc_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_semc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_iee_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED8_CLR_m33_iee_ipg_stop_SHIFT (15U)
/*! m33_iee_ipg_stop - m33_iee_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_iee_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_gpio1_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED8_CLR_m33_gpio1_ipg_stop_SHIFT (16U)
/*! m33_gpio1_ipg_stop - m33_gpio1_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_gpio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_gpio2_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED8_CLR_m33_gpio2_ipg_stop_SHIFT (17U)
/*! m33_gpio2_ipg_stop - m33_gpio2_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_gpio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_gpio3_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED8_CLR_m33_gpio3_ipg_stop_SHIFT (18U)
/*! m33_gpio3_ipg_stop - m33_gpio3_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_gpio3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_gpio4_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED8_CLR_m33_gpio4_ipg_stop_SHIFT (19U)
/*! m33_gpio4_ipg_stop - m33_gpio4_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_gpio4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_gpio5_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED8_CLR_m33_gpio5_ipg_stop_SHIFT (20U)
/*! m33_gpio5_ipg_stop - m33_gpio5_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_gpio5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_gpio6_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED8_CLR_m33_gpio6_ipg_stop_SHIFT (21U)
/*! m33_gpio6_ipg_stop - m33_gpio6_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_gpio6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED8_CLR_m33_flexio1_ipg_stop_SHIFT (22U)
/*! m33_flexio1_ipg_stop - m33_flexio1_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_flexio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED8_CLR_m33_flexio2_ipg_stop_SHIFT (23U)
/*! m33_flexio2_ipg_stop - m33_flexio2_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_flexio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_can1_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED8_CLR_m33_can1_ipg_stop_SHIFT (24U)
/*! m33_can1_ipg_stop - m33_can1_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_can1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_can2_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED8_CLR_m33_can2_ipg_stop_SHIFT (25U)
/*! m33_can2_ipg_stop - m33_can2_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_can2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_can3_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED8_CLR_m33_can3_ipg_stop_SHIFT (26U)
/*! m33_can3_ipg_stop - m33_can3_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_can3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED8_CLR_m33_lpuart1_ipg_stop_SHIFT (27U)
/*! m33_lpuart1_ipg_stop - m33_lpuart1_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_lpuart1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED8_CLR_m33_lpuart2_ipg_stop_SHIFT (28U)
/*! m33_lpuart2_ipg_stop - m33_lpuart2_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_lpuart2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED8_CLR_m33_lpuart3_ipg_stop_SHIFT (29U)
/*! m33_lpuart3_ipg_stop - m33_lpuart3_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_lpuart3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED8_CLR_m33_lpuart4_ipg_stop_SHIFT (30U)
/*! m33_lpuart4_ipg_stop - m33_lpuart4_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_lpuart4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_CLR_m33_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED8_CLR_m33_lpuart5_ipg_stop_SHIFT (31U)
/*! m33_lpuart5_ipg_stop - m33_lpuart5_ipg_stop */
#define CCM_GPR_SHARED8_CLR_m33_lpuart5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_CLR_m33_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_CLR_m33_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED8_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED8_TOG_m33_cm7_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED8_TOG_m33_cm7_ipg_stop_SHIFT (0U)
/*! m33_cm7_ipg_stop - m33_cm7_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_cm7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_cm33_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED8_TOG_m33_cm33_ipg_stop_SHIFT (1U)
/*! m33_cm33_ipg_stop - m33_cm33_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_cm33_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_edma3_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED8_TOG_m33_edma3_ipg_stop_SHIFT (2U)
/*! m33_edma3_ipg_stop - m33_edma3_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_edma3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_edma4_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED8_TOG_m33_edma4_ipg_stop_SHIFT (3U)
/*! m33_edma4_ipg_stop - m33_edma4_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_edma4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_netc_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED8_TOG_m33_netc_ipg_stop_SHIFT (4U)
/*! m33_netc_ipg_stop - m33_netc_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_netc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED8_TOG_m33_sim_aon_ipg_stop_SHIFT (8U)
/*! m33_sim_aon_ipg_stop - m33_sim_aon_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_sim_aon_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_adc1_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED8_TOG_m33_adc1_ipg_stop_SHIFT (9U)
/*! m33_adc1_ipg_stop - m33_adc1_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_adc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_adc2_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED8_TOG_m33_adc2_ipg_stop_SHIFT (10U)
/*! m33_adc2_ipg_stop - m33_adc2_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_adc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED8_TOG_m33_flexspi1_ipg_stop_SHIFT (11U)
/*! m33_flexspi1_ipg_stop - m33_flexspi1_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED8_TOG_m33_flexspi2_ipg_stop_SHIFT (12U)
/*! m33_flexspi2_ipg_stop - m33_flexspi2_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_trdc_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED8_TOG_m33_trdc_ipg_stop_SHIFT (13U)
/*! m33_trdc_ipg_stop - m33_trdc_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_trdc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_semc_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED8_TOG_m33_semc_ipg_stop_SHIFT (14U)
/*! m33_semc_ipg_stop - m33_semc_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_semc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_iee_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED8_TOG_m33_iee_ipg_stop_SHIFT (15U)
/*! m33_iee_ipg_stop - m33_iee_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_iee_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_gpio1_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED8_TOG_m33_gpio1_ipg_stop_SHIFT (16U)
/*! m33_gpio1_ipg_stop - m33_gpio1_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_gpio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_gpio2_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED8_TOG_m33_gpio2_ipg_stop_SHIFT (17U)
/*! m33_gpio2_ipg_stop - m33_gpio2_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_gpio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_gpio3_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED8_TOG_m33_gpio3_ipg_stop_SHIFT (18U)
/*! m33_gpio3_ipg_stop - m33_gpio3_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_gpio3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_gpio4_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED8_TOG_m33_gpio4_ipg_stop_SHIFT (19U)
/*! m33_gpio4_ipg_stop - m33_gpio4_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_gpio4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_gpio5_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED8_TOG_m33_gpio5_ipg_stop_SHIFT (20U)
/*! m33_gpio5_ipg_stop - m33_gpio5_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_gpio5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_gpio6_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED8_TOG_m33_gpio6_ipg_stop_SHIFT (21U)
/*! m33_gpio6_ipg_stop - m33_gpio6_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_gpio6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED8_TOG_m33_flexio1_ipg_stop_SHIFT (22U)
/*! m33_flexio1_ipg_stop - m33_flexio1_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_flexio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED8_TOG_m33_flexio2_ipg_stop_SHIFT (23U)
/*! m33_flexio2_ipg_stop - m33_flexio2_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_flexio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_can1_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED8_TOG_m33_can1_ipg_stop_SHIFT (24U)
/*! m33_can1_ipg_stop - m33_can1_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_can1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_can2_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED8_TOG_m33_can2_ipg_stop_SHIFT (25U)
/*! m33_can2_ipg_stop - m33_can2_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_can2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_can3_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED8_TOG_m33_can3_ipg_stop_SHIFT (26U)
/*! m33_can3_ipg_stop - m33_can3_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_can3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED8_TOG_m33_lpuart1_ipg_stop_SHIFT (27U)
/*! m33_lpuart1_ipg_stop - m33_lpuart1_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_lpuart1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED8_TOG_m33_lpuart2_ipg_stop_SHIFT (28U)
/*! m33_lpuart2_ipg_stop - m33_lpuart2_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_lpuart2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED8_TOG_m33_lpuart3_ipg_stop_SHIFT (29U)
/*! m33_lpuart3_ipg_stop - m33_lpuart3_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_lpuart3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED8_TOG_m33_lpuart4_ipg_stop_SHIFT (30U)
/*! m33_lpuart4_ipg_stop - m33_lpuart4_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_lpuart4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED8_TOG_m33_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED8_TOG_m33_lpuart5_ipg_stop_SHIFT (31U)
/*! m33_lpuart5_ipg_stop - m33_lpuart5_ipg_stop */
#define CCM_GPR_SHARED8_TOG_m33_lpuart5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_TOG_m33_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED8_TOG_m33_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED8_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED8_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED8_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED8_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED8_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED8_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED8_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED8_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED8_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED8_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED8_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED8_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED8_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED8_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED8_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED8_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED8_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED8_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED8_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED8_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED8_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED8_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED8_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED8_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED8_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED8_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED8_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED8_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED8_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED8_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED8_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED8_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED8_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED8_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED8_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED8_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED8_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED8_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED8_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED8_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED8_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED8_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED8_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED8_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED8_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED8_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED8_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED8_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED8_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED8_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED8_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED8_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED8_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED8_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED9 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED9_m33_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED9_m33_lpuart6_ipg_stop_SHIFT (0U)
/*! m33_lpuart6_ipg_stop - m33_lpuart6_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpuart6_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED9_m33_lpuart7_ipg_stop_SHIFT (1U)
/*! m33_lpuart7_ipg_stop - m33_lpuart7_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpuart7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED9_m33_lpuart8_ipg_stop_SHIFT (2U)
/*! m33_lpuart8_ipg_stop - m33_lpuart8_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpuart8_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED9_m33_lpuart9_ipg_stop_SHIFT (3U)
/*! m33_lpuart9_ipg_stop - m33_lpuart9_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpuart9_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED9_m33_lpuart10_ipg_stop_SHIFT (4U)
/*! m33_lpuart10_ipg_stop - m33_lpuart10_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED9_m33_lpuart11_ipg_stop_SHIFT (5U)
/*! m33_lpuart11_ipg_stop - m33_lpuart11_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED9_m33_lpuart12_ipg_stop_SHIFT (6U)
/*! m33_lpuart12_ipg_stop - m33_lpuart12_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED9_m33_lpi2c1_ipg_stop_SHIFT (7U)
/*! m33_lpi2c1_ipg_stop - m33_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpi2c1_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED9_m33_lpi2c2_ipg_stop_SHIFT (8U)
/*! m33_lpi2c2_ipg_stop - m33_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpi2c2_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED9_m33_lpi2c3_ipg_stop_SHIFT (9U)
/*! m33_lpi2c3_ipg_stop - m33_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpi2c3_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED9_m33_lpi2c4_ipg_stop_SHIFT (10U)
/*! m33_lpi2c4_ipg_stop - m33_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpi2c4_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED9_m33_lpi2c5_ipg_stop_SHIFT (11U)
/*! m33_lpi2c5_ipg_stop - m33_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpi2c5_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED9_m33_lpi2c6_ipg_stop_SHIFT (12U)
/*! m33_lpi2c6_ipg_stop - m33_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpi2c6_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED9_m33_lpspi1_ipg_stop_SHIFT (13U)
/*! m33_lpspi1_ipg_stop - m33_lpspi1_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpspi1_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED9_m33_lpspi2_ipg_stop_SHIFT (14U)
/*! m33_lpspi2_ipg_stop - m33_lpspi2_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpspi2_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED9_m33_lpspi3_ipg_stop_SHIFT (15U)
/*! m33_lpspi3_ipg_stop - m33_lpspi3_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpspi3_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED9_m33_lpspi4_ipg_stop_SHIFT (16U)
/*! m33_lpspi4_ipg_stop - m33_lpspi4_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpspi4_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED9_m33_lpspi5_ipg_stop_SHIFT (17U)
/*! m33_lpspi5_ipg_stop - m33_lpspi5_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpspi5_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED9_m33_lpspi6_ipg_stop_SHIFT (18U)
/*! m33_lpspi6_ipg_stop - m33_lpspi6_ipg_stop */
#define CCM_GPR_SHARED9_m33_lpspi6_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_sinc1_ipg_stop_MASK  (0x80000U)
#define CCM_GPR_SHARED9_m33_sinc1_ipg_stop_SHIFT (19U)
/*! m33_sinc1_ipg_stop - m33_sinc1_ipg_stop */
#define CCM_GPR_SHARED9_m33_sinc1_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_sinc2_ipg_stop_MASK  (0x100000U)
#define CCM_GPR_SHARED9_m33_sinc2_ipg_stop_SHIFT (20U)
/*! m33_sinc2_ipg_stop - m33_sinc2_ipg_stop */
#define CCM_GPR_SHARED9_m33_sinc2_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_sinc3_ipg_stop_MASK  (0x200000U)
#define CCM_GPR_SHARED9_m33_sinc3_ipg_stop_SHIFT (21U)
/*! m33_sinc3_ipg_stop - m33_sinc3_ipg_stop */
#define CCM_GPR_SHARED9_m33_sinc3_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_sai1_ipg_stop_MASK   (0x400000U)
#define CCM_GPR_SHARED9_m33_sai1_ipg_stop_SHIFT  (22U)
/*! m33_sai1_ipg_stop - m33_sai1_ipg_stop */
#define CCM_GPR_SHARED9_m33_sai1_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_sai2_ipg_stop_MASK   (0x800000U)
#define CCM_GPR_SHARED9_m33_sai2_ipg_stop_SHIFT  (23U)
/*! m33_sai2_ipg_stop - m33_sai2_ipg_stop */
#define CCM_GPR_SHARED9_m33_sai2_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_sai3_ipg_stop_MASK   (0x1000000U)
#define CCM_GPR_SHARED9_m33_sai3_ipg_stop_SHIFT  (24U)
/*! m33_sai3_ipg_stop - m33_sai3_ipg_stop */
#define CCM_GPR_SHARED9_m33_sai3_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_sai4_ipg_stop_MASK   (0x2000000U)
#define CCM_GPR_SHARED9_m33_sai4_ipg_stop_SHIFT  (25U)
/*! m33_sai4_ipg_stop - m33_sai4_ipg_stop */
#define CCM_GPR_SHARED9_m33_sai4_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_m33_mic_ipg_stop_MASK    (0x4000000U)
#define CCM_GPR_SHARED9_m33_mic_ipg_stop_SHIFT   (26U)
/*! m33_mic_ipg_stop - m33_mic_ipg_stop */
#define CCM_GPR_SHARED9_m33_mic_ipg_stop(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_m33_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_m33_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED9_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED9_SET_m33_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED9_SET_m33_lpuart6_ipg_stop_SHIFT (0U)
/*! m33_lpuart6_ipg_stop - m33_lpuart6_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpuart6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED9_SET_m33_lpuart7_ipg_stop_SHIFT (1U)
/*! m33_lpuart7_ipg_stop - m33_lpuart7_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpuart7_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED9_SET_m33_lpuart8_ipg_stop_SHIFT (2U)
/*! m33_lpuart8_ipg_stop - m33_lpuart8_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpuart8_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED9_SET_m33_lpuart9_ipg_stop_SHIFT (3U)
/*! m33_lpuart9_ipg_stop - m33_lpuart9_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpuart9_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED9_SET_m33_lpuart10_ipg_stop_SHIFT (4U)
/*! m33_lpuart10_ipg_stop - m33_lpuart10_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED9_SET_m33_lpuart11_ipg_stop_SHIFT (5U)
/*! m33_lpuart11_ipg_stop - m33_lpuart11_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED9_SET_m33_lpuart12_ipg_stop_SHIFT (6U)
/*! m33_lpuart12_ipg_stop - m33_lpuart12_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED9_SET_m33_lpi2c1_ipg_stop_SHIFT (7U)
/*! m33_lpi2c1_ipg_stop - m33_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpi2c1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED9_SET_m33_lpi2c2_ipg_stop_SHIFT (8U)
/*! m33_lpi2c2_ipg_stop - m33_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpi2c2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED9_SET_m33_lpi2c3_ipg_stop_SHIFT (9U)
/*! m33_lpi2c3_ipg_stop - m33_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpi2c3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED9_SET_m33_lpi2c4_ipg_stop_SHIFT (10U)
/*! m33_lpi2c4_ipg_stop - m33_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpi2c4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED9_SET_m33_lpi2c5_ipg_stop_SHIFT (11U)
/*! m33_lpi2c5_ipg_stop - m33_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpi2c5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED9_SET_m33_lpi2c6_ipg_stop_SHIFT (12U)
/*! m33_lpi2c6_ipg_stop - m33_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpi2c6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED9_SET_m33_lpspi1_ipg_stop_SHIFT (13U)
/*! m33_lpspi1_ipg_stop - m33_lpspi1_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED9_SET_m33_lpspi2_ipg_stop_SHIFT (14U)
/*! m33_lpspi2_ipg_stop - m33_lpspi2_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED9_SET_m33_lpspi3_ipg_stop_SHIFT (15U)
/*! m33_lpspi3_ipg_stop - m33_lpspi3_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpspi3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED9_SET_m33_lpspi4_ipg_stop_SHIFT (16U)
/*! m33_lpspi4_ipg_stop - m33_lpspi4_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpspi4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED9_SET_m33_lpspi5_ipg_stop_SHIFT (17U)
/*! m33_lpspi5_ipg_stop - m33_lpspi5_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpspi5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED9_SET_m33_lpspi6_ipg_stop_SHIFT (18U)
/*! m33_lpspi6_ipg_stop - m33_lpspi6_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_lpspi6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_sinc1_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED9_SET_m33_sinc1_ipg_stop_SHIFT (19U)
/*! m33_sinc1_ipg_stop - m33_sinc1_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_sinc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_sinc2_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED9_SET_m33_sinc2_ipg_stop_SHIFT (20U)
/*! m33_sinc2_ipg_stop - m33_sinc2_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_sinc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_sinc3_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED9_SET_m33_sinc3_ipg_stop_SHIFT (21U)
/*! m33_sinc3_ipg_stop - m33_sinc3_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_sinc3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_sai1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED9_SET_m33_sai1_ipg_stop_SHIFT (22U)
/*! m33_sai1_ipg_stop - m33_sai1_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_sai1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_sai2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED9_SET_m33_sai2_ipg_stop_SHIFT (23U)
/*! m33_sai2_ipg_stop - m33_sai2_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_sai2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_sai3_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED9_SET_m33_sai3_ipg_stop_SHIFT (24U)
/*! m33_sai3_ipg_stop - m33_sai3_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_sai3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_sai4_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED9_SET_m33_sai4_ipg_stop_SHIFT (25U)
/*! m33_sai4_ipg_stop - m33_sai4_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_sai4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_SET_m33_mic_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED9_SET_m33_mic_ipg_stop_SHIFT (26U)
/*! m33_mic_ipg_stop - m33_mic_ipg_stop */
#define CCM_GPR_SHARED9_SET_m33_mic_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_SET_m33_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_SET_m33_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED9_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED9_CLR_m33_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED9_CLR_m33_lpuart6_ipg_stop_SHIFT (0U)
/*! m33_lpuart6_ipg_stop - m33_lpuart6_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpuart6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED9_CLR_m33_lpuart7_ipg_stop_SHIFT (1U)
/*! m33_lpuart7_ipg_stop - m33_lpuart7_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpuart7_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED9_CLR_m33_lpuart8_ipg_stop_SHIFT (2U)
/*! m33_lpuart8_ipg_stop - m33_lpuart8_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpuart8_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED9_CLR_m33_lpuart9_ipg_stop_SHIFT (3U)
/*! m33_lpuart9_ipg_stop - m33_lpuart9_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpuart9_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED9_CLR_m33_lpuart10_ipg_stop_SHIFT (4U)
/*! m33_lpuart10_ipg_stop - m33_lpuart10_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED9_CLR_m33_lpuart11_ipg_stop_SHIFT (5U)
/*! m33_lpuart11_ipg_stop - m33_lpuart11_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED9_CLR_m33_lpuart12_ipg_stop_SHIFT (6U)
/*! m33_lpuart12_ipg_stop - m33_lpuart12_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED9_CLR_m33_lpi2c1_ipg_stop_SHIFT (7U)
/*! m33_lpi2c1_ipg_stop - m33_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpi2c1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED9_CLR_m33_lpi2c2_ipg_stop_SHIFT (8U)
/*! m33_lpi2c2_ipg_stop - m33_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpi2c2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED9_CLR_m33_lpi2c3_ipg_stop_SHIFT (9U)
/*! m33_lpi2c3_ipg_stop - m33_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpi2c3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED9_CLR_m33_lpi2c4_ipg_stop_SHIFT (10U)
/*! m33_lpi2c4_ipg_stop - m33_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpi2c4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED9_CLR_m33_lpi2c5_ipg_stop_SHIFT (11U)
/*! m33_lpi2c5_ipg_stop - m33_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpi2c5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED9_CLR_m33_lpi2c6_ipg_stop_SHIFT (12U)
/*! m33_lpi2c6_ipg_stop - m33_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpi2c6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED9_CLR_m33_lpspi1_ipg_stop_SHIFT (13U)
/*! m33_lpspi1_ipg_stop - m33_lpspi1_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED9_CLR_m33_lpspi2_ipg_stop_SHIFT (14U)
/*! m33_lpspi2_ipg_stop - m33_lpspi2_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED9_CLR_m33_lpspi3_ipg_stop_SHIFT (15U)
/*! m33_lpspi3_ipg_stop - m33_lpspi3_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpspi3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED9_CLR_m33_lpspi4_ipg_stop_SHIFT (16U)
/*! m33_lpspi4_ipg_stop - m33_lpspi4_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpspi4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED9_CLR_m33_lpspi5_ipg_stop_SHIFT (17U)
/*! m33_lpspi5_ipg_stop - m33_lpspi5_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpspi5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED9_CLR_m33_lpspi6_ipg_stop_SHIFT (18U)
/*! m33_lpspi6_ipg_stop - m33_lpspi6_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_lpspi6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_sinc1_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED9_CLR_m33_sinc1_ipg_stop_SHIFT (19U)
/*! m33_sinc1_ipg_stop - m33_sinc1_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_sinc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_sinc2_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED9_CLR_m33_sinc2_ipg_stop_SHIFT (20U)
/*! m33_sinc2_ipg_stop - m33_sinc2_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_sinc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_sinc3_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED9_CLR_m33_sinc3_ipg_stop_SHIFT (21U)
/*! m33_sinc3_ipg_stop - m33_sinc3_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_sinc3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_sai1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED9_CLR_m33_sai1_ipg_stop_SHIFT (22U)
/*! m33_sai1_ipg_stop - m33_sai1_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_sai1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_sai2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED9_CLR_m33_sai2_ipg_stop_SHIFT (23U)
/*! m33_sai2_ipg_stop - m33_sai2_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_sai2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_sai3_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED9_CLR_m33_sai3_ipg_stop_SHIFT (24U)
/*! m33_sai3_ipg_stop - m33_sai3_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_sai3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_sai4_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED9_CLR_m33_sai4_ipg_stop_SHIFT (25U)
/*! m33_sai4_ipg_stop - m33_sai4_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_sai4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_CLR_m33_mic_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED9_CLR_m33_mic_ipg_stop_SHIFT (26U)
/*! m33_mic_ipg_stop - m33_mic_ipg_stop */
#define CCM_GPR_SHARED9_CLR_m33_mic_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_CLR_m33_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_CLR_m33_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED9_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED9_TOG_m33_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED9_TOG_m33_lpuart6_ipg_stop_SHIFT (0U)
/*! m33_lpuart6_ipg_stop - m33_lpuart6_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpuart6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED9_TOG_m33_lpuart7_ipg_stop_SHIFT (1U)
/*! m33_lpuart7_ipg_stop - m33_lpuart7_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpuart7_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED9_TOG_m33_lpuart8_ipg_stop_SHIFT (2U)
/*! m33_lpuart8_ipg_stop - m33_lpuart8_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpuart8_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED9_TOG_m33_lpuart9_ipg_stop_SHIFT (3U)
/*! m33_lpuart9_ipg_stop - m33_lpuart9_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpuart9_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED9_TOG_m33_lpuart10_ipg_stop_SHIFT (4U)
/*! m33_lpuart10_ipg_stop - m33_lpuart10_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED9_TOG_m33_lpuart11_ipg_stop_SHIFT (5U)
/*! m33_lpuart11_ipg_stop - m33_lpuart11_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED9_TOG_m33_lpuart12_ipg_stop_SHIFT (6U)
/*! m33_lpuart12_ipg_stop - m33_lpuart12_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED9_TOG_m33_lpi2c1_ipg_stop_SHIFT (7U)
/*! m33_lpi2c1_ipg_stop - m33_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpi2c1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED9_TOG_m33_lpi2c2_ipg_stop_SHIFT (8U)
/*! m33_lpi2c2_ipg_stop - m33_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpi2c2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED9_TOG_m33_lpi2c3_ipg_stop_SHIFT (9U)
/*! m33_lpi2c3_ipg_stop - m33_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpi2c3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED9_TOG_m33_lpi2c4_ipg_stop_SHIFT (10U)
/*! m33_lpi2c4_ipg_stop - m33_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpi2c4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED9_TOG_m33_lpi2c5_ipg_stop_SHIFT (11U)
/*! m33_lpi2c5_ipg_stop - m33_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpi2c5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED9_TOG_m33_lpi2c6_ipg_stop_SHIFT (12U)
/*! m33_lpi2c6_ipg_stop - m33_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpi2c6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED9_TOG_m33_lpspi1_ipg_stop_SHIFT (13U)
/*! m33_lpspi1_ipg_stop - m33_lpspi1_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED9_TOG_m33_lpspi2_ipg_stop_SHIFT (14U)
/*! m33_lpspi2_ipg_stop - m33_lpspi2_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED9_TOG_m33_lpspi3_ipg_stop_SHIFT (15U)
/*! m33_lpspi3_ipg_stop - m33_lpspi3_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpspi3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED9_TOG_m33_lpspi4_ipg_stop_SHIFT (16U)
/*! m33_lpspi4_ipg_stop - m33_lpspi4_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpspi4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED9_TOG_m33_lpspi5_ipg_stop_SHIFT (17U)
/*! m33_lpspi5_ipg_stop - m33_lpspi5_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpspi5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED9_TOG_m33_lpspi6_ipg_stop_SHIFT (18U)
/*! m33_lpspi6_ipg_stop - m33_lpspi6_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_lpspi6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_sinc1_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED9_TOG_m33_sinc1_ipg_stop_SHIFT (19U)
/*! m33_sinc1_ipg_stop - m33_sinc1_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_sinc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_sinc2_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED9_TOG_m33_sinc2_ipg_stop_SHIFT (20U)
/*! m33_sinc2_ipg_stop - m33_sinc2_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_sinc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_sinc3_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED9_TOG_m33_sinc3_ipg_stop_SHIFT (21U)
/*! m33_sinc3_ipg_stop - m33_sinc3_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_sinc3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_sai1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED9_TOG_m33_sai1_ipg_stop_SHIFT (22U)
/*! m33_sai1_ipg_stop - m33_sai1_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_sai1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_sai2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED9_TOG_m33_sai2_ipg_stop_SHIFT (23U)
/*! m33_sai2_ipg_stop - m33_sai2_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_sai2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_sai3_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED9_TOG_m33_sai3_ipg_stop_SHIFT (24U)
/*! m33_sai3_ipg_stop - m33_sai3_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_sai3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_sai4_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED9_TOG_m33_sai4_ipg_stop_SHIFT (25U)
/*! m33_sai4_ipg_stop - m33_sai4_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_sai4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED9_TOG_m33_mic_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED9_TOG_m33_mic_ipg_stop_SHIFT (26U)
/*! m33_mic_ipg_stop - m33_mic_ipg_stop */
#define CCM_GPR_SHARED9_TOG_m33_mic_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_TOG_m33_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED9_TOG_m33_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED9_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED9_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_SHARED9_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED9_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED9_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_SHARED9_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED9_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED9_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_SHARED9_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED9_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED9_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_SHARED9_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED9_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED9_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_SHARED9_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED9_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED9_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED9_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED9_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED9_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED9_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED9_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED9_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED9_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED9_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED9_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED9_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED9_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED9_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED9_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED9_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED9_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED9_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED9_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED9_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED9_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED9_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED9_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED9_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED9_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED9_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED9_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED9_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED9_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_SHARED9_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED9_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED9_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_SHARED9_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED9_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED9_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED9_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED9_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED9_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED9_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED10 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED10_m33_adc1_ipg_doze_MASK  (0x1U)
#define CCM_GPR_SHARED10_m33_adc1_ipg_doze_SHIFT (0U)
/*! m33_adc1_ipg_doze - m33_adc1_ipg_doze */
#define CCM_GPR_SHARED10_m33_adc1_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_adc2_ipg_doze_MASK  (0x2U)
#define CCM_GPR_SHARED10_m33_adc2_ipg_doze_SHIFT (1U)
/*! m33_adc2_ipg_doze - m33_adc2_ipg_doze */
#define CCM_GPR_SHARED10_m33_adc2_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED10_m33_flexspi1_ipg_doze_SHIFT (2U)
/*! m33_flexspi1_ipg_doze - m33_flexspi1_ipg_doze */
#define CCM_GPR_SHARED10_m33_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED10_m33_flexspi2_ipg_doze_SHIFT (3U)
/*! m33_flexspi2_ipg_doze - m33_flexspi2_ipg_doze */
#define CCM_GPR_SHARED10_m33_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED10_m33_flexio1_ipg_doze_SHIFT (4U)
/*! m33_flexio1_ipg_doze - m33_flexio1_ipg_doze */
#define CCM_GPR_SHARED10_m33_flexio1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED10_m33_flexio2_ipg_doze_SHIFT (5U)
/*! m33_flexio2_ipg_doze - m33_flexio2_ipg_doze */
#define CCM_GPR_SHARED10_m33_flexio2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpit1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED10_m33_lpit1_ipg_doze_SHIFT (6U)
/*! m33_lpit1_ipg_doze - m33_lpit1_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpit1_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpit2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED10_m33_lpit2_ipg_doze_SHIFT (7U)
/*! m33_lpit2_ipg_doze - m33_lpit2_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpit2_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpit3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED10_m33_lpit3_ipg_doze_SHIFT (8U)
/*! m33_lpit3_ipg_doze - m33_lpit3_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpit3_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_tpm1_ipg_doze_MASK  (0x200U)
#define CCM_GPR_SHARED10_m33_tpm1_ipg_doze_SHIFT (9U)
/*! m33_tpm1_ipg_doze - m33_tpm1_ipg_doze */
#define CCM_GPR_SHARED10_m33_tpm1_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_tpm2_ipg_doze_MASK  (0x400U)
#define CCM_GPR_SHARED10_m33_tpm2_ipg_doze_SHIFT (10U)
/*! m33_tpm2_ipg_doze - m33_tpm2_ipg_doze */
#define CCM_GPR_SHARED10_m33_tpm2_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_tpm3_ipg_doze_MASK  (0x800U)
#define CCM_GPR_SHARED10_m33_tpm3_ipg_doze_SHIFT (11U)
/*! m33_tpm3_ipg_doze - m33_tpm3_ipg_doze */
#define CCM_GPR_SHARED10_m33_tpm3_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_tpm4_ipg_doze_MASK  (0x1000U)
#define CCM_GPR_SHARED10_m33_tpm4_ipg_doze_SHIFT (12U)
/*! m33_tpm4_ipg_doze - m33_tpm4_ipg_doze */
#define CCM_GPR_SHARED10_m33_tpm4_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_tpm5_ipg_doze_MASK  (0x2000U)
#define CCM_GPR_SHARED10_m33_tpm5_ipg_doze_SHIFT (13U)
/*! m33_tpm5_ipg_doze - m33_tpm5_ipg_doze */
#define CCM_GPR_SHARED10_m33_tpm5_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_tpm6_ipg_doze_MASK  (0x4000U)
#define CCM_GPR_SHARED10_m33_tpm6_ipg_doze_SHIFT (14U)
/*! m33_tpm6_ipg_doze - m33_tpm6_ipg_doze */
#define CCM_GPR_SHARED10_m33_tpm6_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_gpt1_ipg_doze_MASK  (0x8000U)
#define CCM_GPR_SHARED10_m33_gpt1_ipg_doze_SHIFT (15U)
/*! m33_gpt1_ipg_doze - m33_gpt1_ipg_doze */
#define CCM_GPR_SHARED10_m33_gpt1_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_gpt2_ipg_doze_MASK  (0x10000U)
#define CCM_GPR_SHARED10_m33_gpt2_ipg_doze_SHIFT (16U)
/*! m33_gpt2_ipg_doze - m33_gpt2_ipg_doze */
#define CCM_GPR_SHARED10_m33_gpt2_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_can1_ipg_doze_MASK  (0x20000U)
#define CCM_GPR_SHARED10_m33_can1_ipg_doze_SHIFT (17U)
/*! m33_can1_ipg_doze - m33_can1_ipg_doze */
#define CCM_GPR_SHARED10_m33_can1_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_can2_ipg_doze_MASK  (0x40000U)
#define CCM_GPR_SHARED10_m33_can2_ipg_doze_SHIFT (18U)
/*! m33_can2_ipg_doze - m33_can2_ipg_doze */
#define CCM_GPR_SHARED10_m33_can2_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_can3_ipg_doze_MASK  (0x80000U)
#define CCM_GPR_SHARED10_m33_can3_ipg_doze_SHIFT (19U)
/*! m33_can3_ipg_doze - m33_can3_ipg_doze */
#define CCM_GPR_SHARED10_m33_can3_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED10_m33_lpuart1_ipg_doze_SHIFT (20U)
/*! m33_lpuart1_ipg_doze - m33_lpuart1_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED10_m33_lpuart2_ipg_doze_SHIFT (21U)
/*! m33_lpuart2_ipg_doze - m33_lpuart2_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED10_m33_lpuart3_ipg_doze_SHIFT (22U)
/*! m33_lpuart3_ipg_doze - m33_lpuart3_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED10_m33_lpuart4_ipg_doze_SHIFT (23U)
/*! m33_lpuart4_ipg_doze - m33_lpuart4_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED10_m33_lpuart5_ipg_doze_SHIFT (24U)
/*! m33_lpuart5_ipg_doze - m33_lpuart5_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED10_m33_lpuart6_ipg_doze_SHIFT (25U)
/*! m33_lpuart6_ipg_doze - m33_lpuart6_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED10_m33_lpuart7_ipg_doze_SHIFT (26U)
/*! m33_lpuart7_ipg_doze - m33_lpuart7_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart7_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED10_m33_lpuart8_ipg_doze_SHIFT (27U)
/*! m33_lpuart8_ipg_doze - m33_lpuart8_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart8_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED10_m33_lpuart9_ipg_doze_SHIFT (28U)
/*! m33_lpuart9_ipg_doze - m33_lpuart9_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart9_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED10_m33_lpuart10_ipg_doze_SHIFT (29U)
/*! m33_lpuart10_ipg_doze - m33_lpuart10_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED10_m33_lpuart11_ipg_doze_SHIFT (30U)
/*! m33_lpuart11_ipg_doze - m33_lpuart11_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED10_m33_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED10_m33_lpuart12_ipg_doze_SHIFT (31U)
/*! m33_lpuart12_ipg_doze - m33_lpuart12_ipg_doze */
#define CCM_GPR_SHARED10_m33_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_m33_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_m33_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED10_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED10_SET_m33_adc1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED10_SET_m33_adc1_ipg_doze_SHIFT (0U)
/*! m33_adc1_ipg_doze - m33_adc1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_adc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_adc2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED10_SET_m33_adc2_ipg_doze_SHIFT (1U)
/*! m33_adc2_ipg_doze - m33_adc2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_adc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED10_SET_m33_flexspi1_ipg_doze_SHIFT (2U)
/*! m33_flexspi1_ipg_doze - m33_flexspi1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED10_SET_m33_flexspi2_ipg_doze_SHIFT (3U)
/*! m33_flexspi2_ipg_doze - m33_flexspi2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED10_SET_m33_flexio1_ipg_doze_SHIFT (4U)
/*! m33_flexio1_ipg_doze - m33_flexio1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_flexio1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED10_SET_m33_flexio2_ipg_doze_SHIFT (5U)
/*! m33_flexio2_ipg_doze - m33_flexio2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_flexio2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpit1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED10_SET_m33_lpit1_ipg_doze_SHIFT (6U)
/*! m33_lpit1_ipg_doze - m33_lpit1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpit1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpit2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED10_SET_m33_lpit2_ipg_doze_SHIFT (7U)
/*! m33_lpit2_ipg_doze - m33_lpit2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpit2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpit3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED10_SET_m33_lpit3_ipg_doze_SHIFT (8U)
/*! m33_lpit3_ipg_doze - m33_lpit3_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpit3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_tpm1_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED10_SET_m33_tpm1_ipg_doze_SHIFT (9U)
/*! m33_tpm1_ipg_doze - m33_tpm1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_tpm1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_tpm2_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED10_SET_m33_tpm2_ipg_doze_SHIFT (10U)
/*! m33_tpm2_ipg_doze - m33_tpm2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_tpm2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_tpm3_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED10_SET_m33_tpm3_ipg_doze_SHIFT (11U)
/*! m33_tpm3_ipg_doze - m33_tpm3_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_tpm3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_tpm4_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED10_SET_m33_tpm4_ipg_doze_SHIFT (12U)
/*! m33_tpm4_ipg_doze - m33_tpm4_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_tpm4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_tpm5_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED10_SET_m33_tpm5_ipg_doze_SHIFT (13U)
/*! m33_tpm5_ipg_doze - m33_tpm5_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_tpm5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_tpm6_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED10_SET_m33_tpm6_ipg_doze_SHIFT (14U)
/*! m33_tpm6_ipg_doze - m33_tpm6_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_tpm6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_gpt1_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED10_SET_m33_gpt1_ipg_doze_SHIFT (15U)
/*! m33_gpt1_ipg_doze - m33_gpt1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_gpt1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_gpt2_ipg_doze_MASK (0x10000U)
#define CCM_GPR_SHARED10_SET_m33_gpt2_ipg_doze_SHIFT (16U)
/*! m33_gpt2_ipg_doze - m33_gpt2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_gpt2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_can1_ipg_doze_MASK (0x20000U)
#define CCM_GPR_SHARED10_SET_m33_can1_ipg_doze_SHIFT (17U)
/*! m33_can1_ipg_doze - m33_can1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_can1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_can2_ipg_doze_MASK (0x40000U)
#define CCM_GPR_SHARED10_SET_m33_can2_ipg_doze_SHIFT (18U)
/*! m33_can2_ipg_doze - m33_can2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_can2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_can3_ipg_doze_MASK (0x80000U)
#define CCM_GPR_SHARED10_SET_m33_can3_ipg_doze_SHIFT (19U)
/*! m33_can3_ipg_doze - m33_can3_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_can3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart1_ipg_doze_SHIFT (20U)
/*! m33_lpuart1_ipg_doze - m33_lpuart1_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart2_ipg_doze_SHIFT (21U)
/*! m33_lpuart2_ipg_doze - m33_lpuart2_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart3_ipg_doze_SHIFT (22U)
/*! m33_lpuart3_ipg_doze - m33_lpuart3_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart4_ipg_doze_SHIFT (23U)
/*! m33_lpuart4_ipg_doze - m33_lpuart4_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart5_ipg_doze_SHIFT (24U)
/*! m33_lpuart5_ipg_doze - m33_lpuart5_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart6_ipg_doze_SHIFT (25U)
/*! m33_lpuart6_ipg_doze - m33_lpuart6_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart7_ipg_doze_SHIFT (26U)
/*! m33_lpuart7_ipg_doze - m33_lpuart7_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart7_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart8_ipg_doze_SHIFT (27U)
/*! m33_lpuart8_ipg_doze - m33_lpuart8_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart8_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart9_ipg_doze_SHIFT (28U)
/*! m33_lpuart9_ipg_doze - m33_lpuart9_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart9_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart10_ipg_doze_SHIFT (29U)
/*! m33_lpuart10_ipg_doze - m33_lpuart10_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart11_ipg_doze_SHIFT (30U)
/*! m33_lpuart11_ipg_doze - m33_lpuart11_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED10_SET_m33_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED10_SET_m33_lpuart12_ipg_doze_SHIFT (31U)
/*! m33_lpuart12_ipg_doze - m33_lpuart12_ipg_doze */
#define CCM_GPR_SHARED10_SET_m33_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_SET_m33_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_SET_m33_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED10_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED10_CLR_m33_adc1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED10_CLR_m33_adc1_ipg_doze_SHIFT (0U)
/*! m33_adc1_ipg_doze - m33_adc1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_adc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_adc2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED10_CLR_m33_adc2_ipg_doze_SHIFT (1U)
/*! m33_adc2_ipg_doze - m33_adc2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_adc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED10_CLR_m33_flexspi1_ipg_doze_SHIFT (2U)
/*! m33_flexspi1_ipg_doze - m33_flexspi1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED10_CLR_m33_flexspi2_ipg_doze_SHIFT (3U)
/*! m33_flexspi2_ipg_doze - m33_flexspi2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED10_CLR_m33_flexio1_ipg_doze_SHIFT (4U)
/*! m33_flexio1_ipg_doze - m33_flexio1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_flexio1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED10_CLR_m33_flexio2_ipg_doze_SHIFT (5U)
/*! m33_flexio2_ipg_doze - m33_flexio2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_flexio2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpit1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED10_CLR_m33_lpit1_ipg_doze_SHIFT (6U)
/*! m33_lpit1_ipg_doze - m33_lpit1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpit1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpit2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED10_CLR_m33_lpit2_ipg_doze_SHIFT (7U)
/*! m33_lpit2_ipg_doze - m33_lpit2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpit2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpit3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED10_CLR_m33_lpit3_ipg_doze_SHIFT (8U)
/*! m33_lpit3_ipg_doze - m33_lpit3_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpit3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_tpm1_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED10_CLR_m33_tpm1_ipg_doze_SHIFT (9U)
/*! m33_tpm1_ipg_doze - m33_tpm1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_tpm1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_tpm2_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED10_CLR_m33_tpm2_ipg_doze_SHIFT (10U)
/*! m33_tpm2_ipg_doze - m33_tpm2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_tpm2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_tpm3_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED10_CLR_m33_tpm3_ipg_doze_SHIFT (11U)
/*! m33_tpm3_ipg_doze - m33_tpm3_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_tpm3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_tpm4_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED10_CLR_m33_tpm4_ipg_doze_SHIFT (12U)
/*! m33_tpm4_ipg_doze - m33_tpm4_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_tpm4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_tpm5_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED10_CLR_m33_tpm5_ipg_doze_SHIFT (13U)
/*! m33_tpm5_ipg_doze - m33_tpm5_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_tpm5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_tpm6_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED10_CLR_m33_tpm6_ipg_doze_SHIFT (14U)
/*! m33_tpm6_ipg_doze - m33_tpm6_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_tpm6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_gpt1_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED10_CLR_m33_gpt1_ipg_doze_SHIFT (15U)
/*! m33_gpt1_ipg_doze - m33_gpt1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_gpt1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_gpt2_ipg_doze_MASK (0x10000U)
#define CCM_GPR_SHARED10_CLR_m33_gpt2_ipg_doze_SHIFT (16U)
/*! m33_gpt2_ipg_doze - m33_gpt2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_gpt2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_can1_ipg_doze_MASK (0x20000U)
#define CCM_GPR_SHARED10_CLR_m33_can1_ipg_doze_SHIFT (17U)
/*! m33_can1_ipg_doze - m33_can1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_can1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_can2_ipg_doze_MASK (0x40000U)
#define CCM_GPR_SHARED10_CLR_m33_can2_ipg_doze_SHIFT (18U)
/*! m33_can2_ipg_doze - m33_can2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_can2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_can3_ipg_doze_MASK (0x80000U)
#define CCM_GPR_SHARED10_CLR_m33_can3_ipg_doze_SHIFT (19U)
/*! m33_can3_ipg_doze - m33_can3_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_can3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart1_ipg_doze_SHIFT (20U)
/*! m33_lpuart1_ipg_doze - m33_lpuart1_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart2_ipg_doze_SHIFT (21U)
/*! m33_lpuart2_ipg_doze - m33_lpuart2_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart3_ipg_doze_SHIFT (22U)
/*! m33_lpuart3_ipg_doze - m33_lpuart3_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart4_ipg_doze_SHIFT (23U)
/*! m33_lpuart4_ipg_doze - m33_lpuart4_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart5_ipg_doze_SHIFT (24U)
/*! m33_lpuart5_ipg_doze - m33_lpuart5_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart6_ipg_doze_SHIFT (25U)
/*! m33_lpuart6_ipg_doze - m33_lpuart6_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart7_ipg_doze_SHIFT (26U)
/*! m33_lpuart7_ipg_doze - m33_lpuart7_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart7_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart8_ipg_doze_SHIFT (27U)
/*! m33_lpuart8_ipg_doze - m33_lpuart8_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart8_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart9_ipg_doze_SHIFT (28U)
/*! m33_lpuart9_ipg_doze - m33_lpuart9_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart9_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart10_ipg_doze_SHIFT (29U)
/*! m33_lpuart10_ipg_doze - m33_lpuart10_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart11_ipg_doze_SHIFT (30U)
/*! m33_lpuart11_ipg_doze - m33_lpuart11_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED10_CLR_m33_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED10_CLR_m33_lpuart12_ipg_doze_SHIFT (31U)
/*! m33_lpuart12_ipg_doze - m33_lpuart12_ipg_doze */
#define CCM_GPR_SHARED10_CLR_m33_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_CLR_m33_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_CLR_m33_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED10_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED10_TOG_m33_adc1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED10_TOG_m33_adc1_ipg_doze_SHIFT (0U)
/*! m33_adc1_ipg_doze - m33_adc1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_adc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_adc2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED10_TOG_m33_adc2_ipg_doze_SHIFT (1U)
/*! m33_adc2_ipg_doze - m33_adc2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_adc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED10_TOG_m33_flexspi1_ipg_doze_SHIFT (2U)
/*! m33_flexspi1_ipg_doze - m33_flexspi1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED10_TOG_m33_flexspi2_ipg_doze_SHIFT (3U)
/*! m33_flexspi2_ipg_doze - m33_flexspi2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED10_TOG_m33_flexio1_ipg_doze_SHIFT (4U)
/*! m33_flexio1_ipg_doze - m33_flexio1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_flexio1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED10_TOG_m33_flexio2_ipg_doze_SHIFT (5U)
/*! m33_flexio2_ipg_doze - m33_flexio2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_flexio2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpit1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED10_TOG_m33_lpit1_ipg_doze_SHIFT (6U)
/*! m33_lpit1_ipg_doze - m33_lpit1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpit1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpit2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED10_TOG_m33_lpit2_ipg_doze_SHIFT (7U)
/*! m33_lpit2_ipg_doze - m33_lpit2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpit2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpit3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED10_TOG_m33_lpit3_ipg_doze_SHIFT (8U)
/*! m33_lpit3_ipg_doze - m33_lpit3_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpit3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_tpm1_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED10_TOG_m33_tpm1_ipg_doze_SHIFT (9U)
/*! m33_tpm1_ipg_doze - m33_tpm1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_tpm1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_tpm2_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED10_TOG_m33_tpm2_ipg_doze_SHIFT (10U)
/*! m33_tpm2_ipg_doze - m33_tpm2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_tpm2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_tpm3_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED10_TOG_m33_tpm3_ipg_doze_SHIFT (11U)
/*! m33_tpm3_ipg_doze - m33_tpm3_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_tpm3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_tpm4_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED10_TOG_m33_tpm4_ipg_doze_SHIFT (12U)
/*! m33_tpm4_ipg_doze - m33_tpm4_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_tpm4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_tpm5_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED10_TOG_m33_tpm5_ipg_doze_SHIFT (13U)
/*! m33_tpm5_ipg_doze - m33_tpm5_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_tpm5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_tpm6_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED10_TOG_m33_tpm6_ipg_doze_SHIFT (14U)
/*! m33_tpm6_ipg_doze - m33_tpm6_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_tpm6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_gpt1_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED10_TOG_m33_gpt1_ipg_doze_SHIFT (15U)
/*! m33_gpt1_ipg_doze - m33_gpt1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_gpt1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_gpt2_ipg_doze_MASK (0x10000U)
#define CCM_GPR_SHARED10_TOG_m33_gpt2_ipg_doze_SHIFT (16U)
/*! m33_gpt2_ipg_doze - m33_gpt2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_gpt2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_can1_ipg_doze_MASK (0x20000U)
#define CCM_GPR_SHARED10_TOG_m33_can1_ipg_doze_SHIFT (17U)
/*! m33_can1_ipg_doze - m33_can1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_can1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_can2_ipg_doze_MASK (0x40000U)
#define CCM_GPR_SHARED10_TOG_m33_can2_ipg_doze_SHIFT (18U)
/*! m33_can2_ipg_doze - m33_can2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_can2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_can3_ipg_doze_MASK (0x80000U)
#define CCM_GPR_SHARED10_TOG_m33_can3_ipg_doze_SHIFT (19U)
/*! m33_can3_ipg_doze - m33_can3_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_can3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart1_ipg_doze_SHIFT (20U)
/*! m33_lpuart1_ipg_doze - m33_lpuart1_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart2_ipg_doze_SHIFT (21U)
/*! m33_lpuart2_ipg_doze - m33_lpuart2_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart3_ipg_doze_SHIFT (22U)
/*! m33_lpuart3_ipg_doze - m33_lpuart3_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart4_ipg_doze_SHIFT (23U)
/*! m33_lpuart4_ipg_doze - m33_lpuart4_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart5_ipg_doze_SHIFT (24U)
/*! m33_lpuart5_ipg_doze - m33_lpuart5_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart6_ipg_doze_SHIFT (25U)
/*! m33_lpuart6_ipg_doze - m33_lpuart6_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart7_ipg_doze_SHIFT (26U)
/*! m33_lpuart7_ipg_doze - m33_lpuart7_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart7_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart8_ipg_doze_SHIFT (27U)
/*! m33_lpuart8_ipg_doze - m33_lpuart8_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart8_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart9_ipg_doze_SHIFT (28U)
/*! m33_lpuart9_ipg_doze - m33_lpuart9_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart9_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart10_ipg_doze_SHIFT (29U)
/*! m33_lpuart10_ipg_doze - m33_lpuart10_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart11_ipg_doze_SHIFT (30U)
/*! m33_lpuart11_ipg_doze - m33_lpuart11_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED10_TOG_m33_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED10_TOG_m33_lpuart12_ipg_doze_SHIFT (31U)
/*! m33_lpuart12_ipg_doze - m33_lpuart12_ipg_doze */
#define CCM_GPR_SHARED10_TOG_m33_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_TOG_m33_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED10_TOG_m33_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED10_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED10_AUTHEN_TZ_USER_MASK     (0x100U)
#define CCM_GPR_SHARED10_AUTHEN_TZ_USER_SHIFT    (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED10_AUTHEN_TZ_USER(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED10_AUTHEN_TZ_NS_MASK       (0x200U)
#define CCM_GPR_SHARED10_AUTHEN_TZ_NS_SHIFT      (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED10_AUTHEN_TZ_NS(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED10_AUTHEN_LOCK_TZ_MASK     (0x800U)
#define CCM_GPR_SHARED10_AUTHEN_LOCK_TZ_SHIFT    (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED10_AUTHEN_LOCK_TZ(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED10_AUTHEN_LOCK_LIST_MASK   (0x8000U)
#define CCM_GPR_SHARED10_AUTHEN_LOCK_LIST_SHIFT  (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED10_AUTHEN_LOCK_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED10_AUTHEN_WHITE_LIST_MASK  (0xFFFF0000U)
#define CCM_GPR_SHARED10_AUTHEN_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED10_AUTHEN_WHITE_LIST(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED10_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED10_AUTHEN_SET_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED10_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED10_AUTHEN_SET_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED10_AUTHEN_SET_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED10_AUTHEN_SET_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED10_AUTHEN_SET_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED10_AUTHEN_SET_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED10_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED10_AUTHEN_SET_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED10_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED10_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED10_AUTHEN_SET_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED10_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED10_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED10_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED10_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED10_AUTHEN_CLR_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED10_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED10_AUTHEN_CLR_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED10_AUTHEN_CLR_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED10_AUTHEN_CLR_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED10_AUTHEN_CLR_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED10_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED10_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED10_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED10_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED10_AUTHEN_TOG_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED10_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED10_AUTHEN_TOG_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED10_AUTHEN_TOG_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED10_AUTHEN_TOG_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED10_AUTHEN_TOG_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED10_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED10_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED10_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED10_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED10_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED11 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED11_m33_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED11_m33_lpi2c1_ipg_doze_SHIFT (0U)
/*! m33_lpi2c1_ipg_doze - m33_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpi2c1_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED11_m33_lpi2c2_ipg_doze_SHIFT (1U)
/*! m33_lpi2c2_ipg_doze - m33_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpi2c2_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED11_m33_lpi2c3_ipg_doze_SHIFT (2U)
/*! m33_lpi2c3_ipg_doze - m33_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpi2c3_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED11_m33_lpi2c4_ipg_doze_SHIFT (3U)
/*! m33_lpi2c4_ipg_doze - m33_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpi2c4_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED11_m33_lpi2c5_ipg_doze_SHIFT (4U)
/*! m33_lpi2c5_ipg_doze - m33_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpi2c5_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED11_m33_lpi2c6_ipg_doze_SHIFT (5U)
/*! m33_lpi2c6_ipg_doze - m33_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpi2c6_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED11_m33_lpspi1_ipg_doze_SHIFT (6U)
/*! m33_lpspi1_ipg_doze - m33_lpspi1_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpspi1_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED11_m33_lpspi2_ipg_doze_SHIFT (7U)
/*! m33_lpspi2_ipg_doze - m33_lpspi2_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpspi2_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED11_m33_lpspi3_ipg_doze_SHIFT (8U)
/*! m33_lpspi3_ipg_doze - m33_lpspi3_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpspi3_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED11_m33_lpspi4_ipg_doze_SHIFT (9U)
/*! m33_lpspi4_ipg_doze - m33_lpspi4_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpspi4_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED11_m33_lpspi5_ipg_doze_SHIFT (10U)
/*! m33_lpspi5_ipg_doze - m33_lpspi5_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpspi5_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED11_m33_lpspi6_ipg_doze_SHIFT (11U)
/*! m33_lpspi6_ipg_doze - m33_lpspi6_ipg_doze */
#define CCM_GPR_SHARED11_m33_lpspi6_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_sinc1_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED11_m33_sinc1_ipg_doze_SHIFT (12U)
/*! m33_sinc1_ipg_doze - m33_sinc1_ipg_doze */
#define CCM_GPR_SHARED11_m33_sinc1_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_sinc2_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED11_m33_sinc2_ipg_doze_SHIFT (13U)
/*! m33_sinc2_ipg_doze - m33_sinc2_ipg_doze */
#define CCM_GPR_SHARED11_m33_sinc2_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_sinc3_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED11_m33_sinc3_ipg_doze_SHIFT (14U)
/*! m33_sinc3_ipg_doze - m33_sinc3_ipg_doze */
#define CCM_GPR_SHARED11_m33_sinc3_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_m33_mic_ipg_doze_MASK   (0x8000U)
#define CCM_GPR_SHARED11_m33_mic_ipg_doze_SHIFT  (15U)
/*! m33_mic_ipg_doze - m33_mic_ipg_doze */
#define CCM_GPR_SHARED11_m33_mic_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_m33_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_m33_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED11_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED11_SET_m33_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED11_SET_m33_lpi2c1_ipg_doze_SHIFT (0U)
/*! m33_lpi2c1_ipg_doze - m33_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpi2c1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED11_SET_m33_lpi2c2_ipg_doze_SHIFT (1U)
/*! m33_lpi2c2_ipg_doze - m33_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpi2c2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED11_SET_m33_lpi2c3_ipg_doze_SHIFT (2U)
/*! m33_lpi2c3_ipg_doze - m33_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpi2c3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED11_SET_m33_lpi2c4_ipg_doze_SHIFT (3U)
/*! m33_lpi2c4_ipg_doze - m33_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpi2c4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED11_SET_m33_lpi2c5_ipg_doze_SHIFT (4U)
/*! m33_lpi2c5_ipg_doze - m33_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpi2c5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED11_SET_m33_lpi2c6_ipg_doze_SHIFT (5U)
/*! m33_lpi2c6_ipg_doze - m33_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpi2c6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED11_SET_m33_lpspi1_ipg_doze_SHIFT (6U)
/*! m33_lpspi1_ipg_doze - m33_lpspi1_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED11_SET_m33_lpspi2_ipg_doze_SHIFT (7U)
/*! m33_lpspi2_ipg_doze - m33_lpspi2_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED11_SET_m33_lpspi3_ipg_doze_SHIFT (8U)
/*! m33_lpspi3_ipg_doze - m33_lpspi3_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpspi3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED11_SET_m33_lpspi4_ipg_doze_SHIFT (9U)
/*! m33_lpspi4_ipg_doze - m33_lpspi4_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpspi4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED11_SET_m33_lpspi5_ipg_doze_SHIFT (10U)
/*! m33_lpspi5_ipg_doze - m33_lpspi5_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpspi5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED11_SET_m33_lpspi6_ipg_doze_SHIFT (11U)
/*! m33_lpspi6_ipg_doze - m33_lpspi6_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_lpspi6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_sinc1_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED11_SET_m33_sinc1_ipg_doze_SHIFT (12U)
/*! m33_sinc1_ipg_doze - m33_sinc1_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_sinc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_sinc2_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED11_SET_m33_sinc2_ipg_doze_SHIFT (13U)
/*! m33_sinc2_ipg_doze - m33_sinc2_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_sinc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_sinc3_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED11_SET_m33_sinc3_ipg_doze_SHIFT (14U)
/*! m33_sinc3_ipg_doze - m33_sinc3_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_sinc3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_SET_m33_mic_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED11_SET_m33_mic_ipg_doze_SHIFT (15U)
/*! m33_mic_ipg_doze - m33_mic_ipg_doze */
#define CCM_GPR_SHARED11_SET_m33_mic_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_SET_m33_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_SET_m33_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED11_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED11_CLR_m33_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED11_CLR_m33_lpi2c1_ipg_doze_SHIFT (0U)
/*! m33_lpi2c1_ipg_doze - m33_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpi2c1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED11_CLR_m33_lpi2c2_ipg_doze_SHIFT (1U)
/*! m33_lpi2c2_ipg_doze - m33_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpi2c2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED11_CLR_m33_lpi2c3_ipg_doze_SHIFT (2U)
/*! m33_lpi2c3_ipg_doze - m33_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpi2c3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED11_CLR_m33_lpi2c4_ipg_doze_SHIFT (3U)
/*! m33_lpi2c4_ipg_doze - m33_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpi2c4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED11_CLR_m33_lpi2c5_ipg_doze_SHIFT (4U)
/*! m33_lpi2c5_ipg_doze - m33_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpi2c5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED11_CLR_m33_lpi2c6_ipg_doze_SHIFT (5U)
/*! m33_lpi2c6_ipg_doze - m33_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpi2c6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED11_CLR_m33_lpspi1_ipg_doze_SHIFT (6U)
/*! m33_lpspi1_ipg_doze - m33_lpspi1_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED11_CLR_m33_lpspi2_ipg_doze_SHIFT (7U)
/*! m33_lpspi2_ipg_doze - m33_lpspi2_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED11_CLR_m33_lpspi3_ipg_doze_SHIFT (8U)
/*! m33_lpspi3_ipg_doze - m33_lpspi3_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpspi3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED11_CLR_m33_lpspi4_ipg_doze_SHIFT (9U)
/*! m33_lpspi4_ipg_doze - m33_lpspi4_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpspi4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED11_CLR_m33_lpspi5_ipg_doze_SHIFT (10U)
/*! m33_lpspi5_ipg_doze - m33_lpspi5_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpspi5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED11_CLR_m33_lpspi6_ipg_doze_SHIFT (11U)
/*! m33_lpspi6_ipg_doze - m33_lpspi6_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_lpspi6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_sinc1_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED11_CLR_m33_sinc1_ipg_doze_SHIFT (12U)
/*! m33_sinc1_ipg_doze - m33_sinc1_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_sinc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_sinc2_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED11_CLR_m33_sinc2_ipg_doze_SHIFT (13U)
/*! m33_sinc2_ipg_doze - m33_sinc2_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_sinc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_sinc3_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED11_CLR_m33_sinc3_ipg_doze_SHIFT (14U)
/*! m33_sinc3_ipg_doze - m33_sinc3_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_sinc3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_CLR_m33_mic_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED11_CLR_m33_mic_ipg_doze_SHIFT (15U)
/*! m33_mic_ipg_doze - m33_mic_ipg_doze */
#define CCM_GPR_SHARED11_CLR_m33_mic_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_CLR_m33_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_CLR_m33_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED11_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED11_TOG_m33_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED11_TOG_m33_lpi2c1_ipg_doze_SHIFT (0U)
/*! m33_lpi2c1_ipg_doze - m33_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpi2c1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED11_TOG_m33_lpi2c2_ipg_doze_SHIFT (1U)
/*! m33_lpi2c2_ipg_doze - m33_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpi2c2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED11_TOG_m33_lpi2c3_ipg_doze_SHIFT (2U)
/*! m33_lpi2c3_ipg_doze - m33_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpi2c3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED11_TOG_m33_lpi2c4_ipg_doze_SHIFT (3U)
/*! m33_lpi2c4_ipg_doze - m33_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpi2c4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED11_TOG_m33_lpi2c5_ipg_doze_SHIFT (4U)
/*! m33_lpi2c5_ipg_doze - m33_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpi2c5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED11_TOG_m33_lpi2c6_ipg_doze_SHIFT (5U)
/*! m33_lpi2c6_ipg_doze - m33_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpi2c6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED11_TOG_m33_lpspi1_ipg_doze_SHIFT (6U)
/*! m33_lpspi1_ipg_doze - m33_lpspi1_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED11_TOG_m33_lpspi2_ipg_doze_SHIFT (7U)
/*! m33_lpspi2_ipg_doze - m33_lpspi2_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED11_TOG_m33_lpspi3_ipg_doze_SHIFT (8U)
/*! m33_lpspi3_ipg_doze - m33_lpspi3_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpspi3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED11_TOG_m33_lpspi4_ipg_doze_SHIFT (9U)
/*! m33_lpspi4_ipg_doze - m33_lpspi4_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpspi4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED11_TOG_m33_lpspi5_ipg_doze_SHIFT (10U)
/*! m33_lpspi5_ipg_doze - m33_lpspi5_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpspi5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED11_TOG_m33_lpspi6_ipg_doze_SHIFT (11U)
/*! m33_lpspi6_ipg_doze - m33_lpspi6_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_lpspi6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_sinc1_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED11_TOG_m33_sinc1_ipg_doze_SHIFT (12U)
/*! m33_sinc1_ipg_doze - m33_sinc1_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_sinc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_sinc2_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED11_TOG_m33_sinc2_ipg_doze_SHIFT (13U)
/*! m33_sinc2_ipg_doze - m33_sinc2_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_sinc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_sinc3_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED11_TOG_m33_sinc3_ipg_doze_SHIFT (14U)
/*! m33_sinc3_ipg_doze - m33_sinc3_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_sinc3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED11_TOG_m33_mic_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED11_TOG_m33_mic_ipg_doze_SHIFT (15U)
/*! m33_mic_ipg_doze - m33_mic_ipg_doze */
#define CCM_GPR_SHARED11_TOG_m33_mic_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_TOG_m33_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED11_TOG_m33_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED11_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED11_AUTHEN_TZ_USER_MASK     (0x100U)
#define CCM_GPR_SHARED11_AUTHEN_TZ_USER_SHIFT    (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED11_AUTHEN_TZ_USER(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED11_AUTHEN_TZ_NS_MASK       (0x200U)
#define CCM_GPR_SHARED11_AUTHEN_TZ_NS_SHIFT      (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED11_AUTHEN_TZ_NS(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED11_AUTHEN_LOCK_TZ_MASK     (0x800U)
#define CCM_GPR_SHARED11_AUTHEN_LOCK_TZ_SHIFT    (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED11_AUTHEN_LOCK_TZ(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED11_AUTHEN_LOCK_LIST_MASK   (0x8000U)
#define CCM_GPR_SHARED11_AUTHEN_LOCK_LIST_SHIFT  (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED11_AUTHEN_LOCK_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED11_AUTHEN_WHITE_LIST_MASK  (0xFFFF0000U)
#define CCM_GPR_SHARED11_AUTHEN_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED11_AUTHEN_WHITE_LIST(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED11_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED11_AUTHEN_SET_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED11_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED11_AUTHEN_SET_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED11_AUTHEN_SET_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED11_AUTHEN_SET_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED11_AUTHEN_SET_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED11_AUTHEN_SET_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED11_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED11_AUTHEN_SET_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED11_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED11_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED11_AUTHEN_SET_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED11_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED11_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED11_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED11_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED11_AUTHEN_CLR_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED11_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED11_AUTHEN_CLR_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED11_AUTHEN_CLR_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED11_AUTHEN_CLR_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED11_AUTHEN_CLR_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED11_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED11_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED11_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED11_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED11_AUTHEN_TOG_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED11_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED11_AUTHEN_TOG_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED11_AUTHEN_TOG_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED11_AUTHEN_TOG_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED11_AUTHEN_TOG_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED11_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED11_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED11_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED11_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED11_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED12 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED12_m7_cm7_ipg_stop_MASK    (0x1U)
#define CCM_GPR_SHARED12_m7_cm7_ipg_stop_SHIFT   (0U)
/*! m7_cm7_ipg_stop - m7_cm7_ipg_stop */
#define CCM_GPR_SHARED12_m7_cm7_ipg_stop(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_cm33_ipg_stop_MASK   (0x2U)
#define CCM_GPR_SHARED12_m7_cm33_ipg_stop_SHIFT  (1U)
/*! m7_cm33_ipg_stop - m7_cm33_ipg_stop */
#define CCM_GPR_SHARED12_m7_cm33_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_edma3_ipg_stop_MASK  (0x4U)
#define CCM_GPR_SHARED12_m7_edma3_ipg_stop_SHIFT (2U)
/*! m7_edma3_ipg_stop - m7_edma3_ipg_stop */
#define CCM_GPR_SHARED12_m7_edma3_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_edma4_ipg_stop_MASK  (0x8U)
#define CCM_GPR_SHARED12_m7_edma4_ipg_stop_SHIFT (3U)
/*! m7_edma4_ipg_stop - m7_edma4_ipg_stop */
#define CCM_GPR_SHARED12_m7_edma4_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_netc_ipg_stop_MASK   (0x10U)
#define CCM_GPR_SHARED12_m7_netc_ipg_stop_SHIFT  (4U)
/*! m7_netc_ipg_stop - m7_netc_ipg_stop */
#define CCM_GPR_SHARED12_m7_netc_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED12_m7_sim_aon_ipg_stop_SHIFT (8U)
/*! m7_sim_aon_ipg_stop - m7_sim_aon_ipg_stop */
#define CCM_GPR_SHARED12_m7_sim_aon_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_adc1_ipg_stop_MASK   (0x200U)
#define CCM_GPR_SHARED12_m7_adc1_ipg_stop_SHIFT  (9U)
/*! m7_adc1_ipg_stop - m7_adc1_ipg_stop */
#define CCM_GPR_SHARED12_m7_adc1_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_adc2_ipg_stop_MASK   (0x400U)
#define CCM_GPR_SHARED12_m7_adc2_ipg_stop_SHIFT  (10U)
/*! m7_adc2_ipg_stop - m7_adc2_ipg_stop */
#define CCM_GPR_SHARED12_m7_adc2_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED12_m7_flexspi1_ipg_stop_SHIFT (11U)
/*! m7_flexspi1_ipg_stop - m7_flexspi1_ipg_stop */
#define CCM_GPR_SHARED12_m7_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED12_m7_flexspi2_ipg_stop_SHIFT (12U)
/*! m7_flexspi2_ipg_stop - m7_flexspi2_ipg_stop */
#define CCM_GPR_SHARED12_m7_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_trdc_ipg_stop_MASK   (0x2000U)
#define CCM_GPR_SHARED12_m7_trdc_ipg_stop_SHIFT  (13U)
/*! m7_trdc_ipg_stop - m7_trdc_ipg_stop */
#define CCM_GPR_SHARED12_m7_trdc_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_semc_ipg_stop_MASK   (0x4000U)
#define CCM_GPR_SHARED12_m7_semc_ipg_stop_SHIFT  (14U)
/*! m7_semc_ipg_stop - m7_semc_ipg_stop */
#define CCM_GPR_SHARED12_m7_semc_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_iee_ipg_stop_MASK    (0x8000U)
#define CCM_GPR_SHARED12_m7_iee_ipg_stop_SHIFT   (15U)
/*! m7_iee_ipg_stop - m7_iee_ipg_stop */
#define CCM_GPR_SHARED12_m7_iee_ipg_stop(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_gpio1_ipg_stop_MASK  (0x10000U)
#define CCM_GPR_SHARED12_m7_gpio1_ipg_stop_SHIFT (16U)
/*! m7_gpio1_ipg_stop - m7_gpio1_ipg_stop */
#define CCM_GPR_SHARED12_m7_gpio1_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_gpio2_ipg_stop_MASK  (0x20000U)
#define CCM_GPR_SHARED12_m7_gpio2_ipg_stop_SHIFT (17U)
/*! m7_gpio2_ipg_stop - m7_gpio2_ipg_stop */
#define CCM_GPR_SHARED12_m7_gpio2_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_gpio3_ipg_stop_MASK  (0x40000U)
#define CCM_GPR_SHARED12_m7_gpio3_ipg_stop_SHIFT (18U)
/*! m7_gpio3_ipg_stop - m7_gpio3_ipg_stop */
#define CCM_GPR_SHARED12_m7_gpio3_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_gpio4_ipg_stop_MASK  (0x80000U)
#define CCM_GPR_SHARED12_m7_gpio4_ipg_stop_SHIFT (19U)
/*! m7_gpio4_ipg_stop - m7_gpio4_ipg_stop */
#define CCM_GPR_SHARED12_m7_gpio4_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_gpio5_ipg_stop_MASK  (0x100000U)
#define CCM_GPR_SHARED12_m7_gpio5_ipg_stop_SHIFT (20U)
/*! m7_gpio5_ipg_stop - m7_gpio5_ipg_stop */
#define CCM_GPR_SHARED12_m7_gpio5_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_gpio6_ipg_stop_MASK  (0x200000U)
#define CCM_GPR_SHARED12_m7_gpio6_ipg_stop_SHIFT (21U)
/*! m7_gpio6_ipg_stop - m7_gpio6_ipg_stop */
#define CCM_GPR_SHARED12_m7_gpio6_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED12_m7_flexio1_ipg_stop_SHIFT (22U)
/*! m7_flexio1_ipg_stop - m7_flexio1_ipg_stop */
#define CCM_GPR_SHARED12_m7_flexio1_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED12_m7_flexio2_ipg_stop_SHIFT (23U)
/*! m7_flexio2_ipg_stop - m7_flexio2_ipg_stop */
#define CCM_GPR_SHARED12_m7_flexio2_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_can1_ipg_stop_MASK   (0x1000000U)
#define CCM_GPR_SHARED12_m7_can1_ipg_stop_SHIFT  (24U)
/*! m7_can1_ipg_stop - m7_can1_ipg_stop */
#define CCM_GPR_SHARED12_m7_can1_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_can2_ipg_stop_MASK   (0x2000000U)
#define CCM_GPR_SHARED12_m7_can2_ipg_stop_SHIFT  (25U)
/*! m7_can2_ipg_stop - m7_can2_ipg_stop */
#define CCM_GPR_SHARED12_m7_can2_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_can3_ipg_stop_MASK   (0x4000000U)
#define CCM_GPR_SHARED12_m7_can3_ipg_stop_SHIFT  (26U)
/*! m7_can3_ipg_stop - m7_can3_ipg_stop */
#define CCM_GPR_SHARED12_m7_can3_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED12_m7_lpuart1_ipg_stop_SHIFT (27U)
/*! m7_lpuart1_ipg_stop - m7_lpuart1_ipg_stop */
#define CCM_GPR_SHARED12_m7_lpuart1_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED12_m7_lpuart2_ipg_stop_SHIFT (28U)
/*! m7_lpuart2_ipg_stop - m7_lpuart2_ipg_stop */
#define CCM_GPR_SHARED12_m7_lpuart2_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED12_m7_lpuart3_ipg_stop_SHIFT (29U)
/*! m7_lpuart3_ipg_stop - m7_lpuart3_ipg_stop */
#define CCM_GPR_SHARED12_m7_lpuart3_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED12_m7_lpuart4_ipg_stop_SHIFT (30U)
/*! m7_lpuart4_ipg_stop - m7_lpuart4_ipg_stop */
#define CCM_GPR_SHARED12_m7_lpuart4_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_m7_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED12_m7_lpuart5_ipg_stop_SHIFT (31U)
/*! m7_lpuart5_ipg_stop - m7_lpuart5_ipg_stop */
#define CCM_GPR_SHARED12_m7_lpuart5_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_m7_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_m7_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED12_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED12_SET_m7_cm7_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED12_SET_m7_cm7_ipg_stop_SHIFT (0U)
/*! m7_cm7_ipg_stop - m7_cm7_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_cm7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_cm33_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED12_SET_m7_cm33_ipg_stop_SHIFT (1U)
/*! m7_cm33_ipg_stop - m7_cm33_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_cm33_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_edma3_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED12_SET_m7_edma3_ipg_stop_SHIFT (2U)
/*! m7_edma3_ipg_stop - m7_edma3_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_edma3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_edma4_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED12_SET_m7_edma4_ipg_stop_SHIFT (3U)
/*! m7_edma4_ipg_stop - m7_edma4_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_edma4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_netc_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED12_SET_m7_netc_ipg_stop_SHIFT (4U)
/*! m7_netc_ipg_stop - m7_netc_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_netc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED12_SET_m7_sim_aon_ipg_stop_SHIFT (8U)
/*! m7_sim_aon_ipg_stop - m7_sim_aon_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_sim_aon_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_adc1_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED12_SET_m7_adc1_ipg_stop_SHIFT (9U)
/*! m7_adc1_ipg_stop - m7_adc1_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_adc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_adc2_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED12_SET_m7_adc2_ipg_stop_SHIFT (10U)
/*! m7_adc2_ipg_stop - m7_adc2_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_adc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED12_SET_m7_flexspi1_ipg_stop_SHIFT (11U)
/*! m7_flexspi1_ipg_stop - m7_flexspi1_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED12_SET_m7_flexspi2_ipg_stop_SHIFT (12U)
/*! m7_flexspi2_ipg_stop - m7_flexspi2_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_trdc_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED12_SET_m7_trdc_ipg_stop_SHIFT (13U)
/*! m7_trdc_ipg_stop - m7_trdc_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_trdc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_semc_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED12_SET_m7_semc_ipg_stop_SHIFT (14U)
/*! m7_semc_ipg_stop - m7_semc_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_semc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_iee_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED12_SET_m7_iee_ipg_stop_SHIFT (15U)
/*! m7_iee_ipg_stop - m7_iee_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_iee_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_gpio1_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED12_SET_m7_gpio1_ipg_stop_SHIFT (16U)
/*! m7_gpio1_ipg_stop - m7_gpio1_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_gpio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_gpio2_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED12_SET_m7_gpio2_ipg_stop_SHIFT (17U)
/*! m7_gpio2_ipg_stop - m7_gpio2_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_gpio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_gpio3_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED12_SET_m7_gpio3_ipg_stop_SHIFT (18U)
/*! m7_gpio3_ipg_stop - m7_gpio3_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_gpio3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_gpio4_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED12_SET_m7_gpio4_ipg_stop_SHIFT (19U)
/*! m7_gpio4_ipg_stop - m7_gpio4_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_gpio4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_gpio5_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED12_SET_m7_gpio5_ipg_stop_SHIFT (20U)
/*! m7_gpio5_ipg_stop - m7_gpio5_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_gpio5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_gpio6_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED12_SET_m7_gpio6_ipg_stop_SHIFT (21U)
/*! m7_gpio6_ipg_stop - m7_gpio6_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_gpio6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED12_SET_m7_flexio1_ipg_stop_SHIFT (22U)
/*! m7_flexio1_ipg_stop - m7_flexio1_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_flexio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED12_SET_m7_flexio2_ipg_stop_SHIFT (23U)
/*! m7_flexio2_ipg_stop - m7_flexio2_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_flexio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_can1_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED12_SET_m7_can1_ipg_stop_SHIFT (24U)
/*! m7_can1_ipg_stop - m7_can1_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_can1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_can2_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED12_SET_m7_can2_ipg_stop_SHIFT (25U)
/*! m7_can2_ipg_stop - m7_can2_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_can2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_can3_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED12_SET_m7_can3_ipg_stop_SHIFT (26U)
/*! m7_can3_ipg_stop - m7_can3_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_can3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED12_SET_m7_lpuart1_ipg_stop_SHIFT (27U)
/*! m7_lpuart1_ipg_stop - m7_lpuart1_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_lpuart1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED12_SET_m7_lpuart2_ipg_stop_SHIFT (28U)
/*! m7_lpuart2_ipg_stop - m7_lpuart2_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_lpuart2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED12_SET_m7_lpuart3_ipg_stop_SHIFT (29U)
/*! m7_lpuart3_ipg_stop - m7_lpuart3_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_lpuart3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED12_SET_m7_lpuart4_ipg_stop_SHIFT (30U)
/*! m7_lpuart4_ipg_stop - m7_lpuart4_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_lpuart4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_SET_m7_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED12_SET_m7_lpuart5_ipg_stop_SHIFT (31U)
/*! m7_lpuart5_ipg_stop - m7_lpuart5_ipg_stop */
#define CCM_GPR_SHARED12_SET_m7_lpuart5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_SET_m7_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_SET_m7_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED12_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED12_CLR_m7_cm7_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED12_CLR_m7_cm7_ipg_stop_SHIFT (0U)
/*! m7_cm7_ipg_stop - m7_cm7_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_cm7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_cm33_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED12_CLR_m7_cm33_ipg_stop_SHIFT (1U)
/*! m7_cm33_ipg_stop - m7_cm33_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_cm33_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_edma3_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED12_CLR_m7_edma3_ipg_stop_SHIFT (2U)
/*! m7_edma3_ipg_stop - m7_edma3_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_edma3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_edma4_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED12_CLR_m7_edma4_ipg_stop_SHIFT (3U)
/*! m7_edma4_ipg_stop - m7_edma4_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_edma4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_netc_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED12_CLR_m7_netc_ipg_stop_SHIFT (4U)
/*! m7_netc_ipg_stop - m7_netc_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_netc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED12_CLR_m7_sim_aon_ipg_stop_SHIFT (8U)
/*! m7_sim_aon_ipg_stop - m7_sim_aon_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_sim_aon_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_adc1_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED12_CLR_m7_adc1_ipg_stop_SHIFT (9U)
/*! m7_adc1_ipg_stop - m7_adc1_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_adc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_adc2_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED12_CLR_m7_adc2_ipg_stop_SHIFT (10U)
/*! m7_adc2_ipg_stop - m7_adc2_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_adc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED12_CLR_m7_flexspi1_ipg_stop_SHIFT (11U)
/*! m7_flexspi1_ipg_stop - m7_flexspi1_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED12_CLR_m7_flexspi2_ipg_stop_SHIFT (12U)
/*! m7_flexspi2_ipg_stop - m7_flexspi2_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_trdc_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED12_CLR_m7_trdc_ipg_stop_SHIFT (13U)
/*! m7_trdc_ipg_stop - m7_trdc_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_trdc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_semc_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED12_CLR_m7_semc_ipg_stop_SHIFT (14U)
/*! m7_semc_ipg_stop - m7_semc_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_semc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_iee_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED12_CLR_m7_iee_ipg_stop_SHIFT (15U)
/*! m7_iee_ipg_stop - m7_iee_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_iee_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_gpio1_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED12_CLR_m7_gpio1_ipg_stop_SHIFT (16U)
/*! m7_gpio1_ipg_stop - m7_gpio1_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_gpio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_gpio2_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED12_CLR_m7_gpio2_ipg_stop_SHIFT (17U)
/*! m7_gpio2_ipg_stop - m7_gpio2_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_gpio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_gpio3_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED12_CLR_m7_gpio3_ipg_stop_SHIFT (18U)
/*! m7_gpio3_ipg_stop - m7_gpio3_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_gpio3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_gpio4_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED12_CLR_m7_gpio4_ipg_stop_SHIFT (19U)
/*! m7_gpio4_ipg_stop - m7_gpio4_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_gpio4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_gpio5_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED12_CLR_m7_gpio5_ipg_stop_SHIFT (20U)
/*! m7_gpio5_ipg_stop - m7_gpio5_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_gpio5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_gpio6_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED12_CLR_m7_gpio6_ipg_stop_SHIFT (21U)
/*! m7_gpio6_ipg_stop - m7_gpio6_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_gpio6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED12_CLR_m7_flexio1_ipg_stop_SHIFT (22U)
/*! m7_flexio1_ipg_stop - m7_flexio1_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_flexio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED12_CLR_m7_flexio2_ipg_stop_SHIFT (23U)
/*! m7_flexio2_ipg_stop - m7_flexio2_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_flexio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_can1_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED12_CLR_m7_can1_ipg_stop_SHIFT (24U)
/*! m7_can1_ipg_stop - m7_can1_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_can1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_can2_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED12_CLR_m7_can2_ipg_stop_SHIFT (25U)
/*! m7_can2_ipg_stop - m7_can2_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_can2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_can3_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED12_CLR_m7_can3_ipg_stop_SHIFT (26U)
/*! m7_can3_ipg_stop - m7_can3_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_can3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED12_CLR_m7_lpuart1_ipg_stop_SHIFT (27U)
/*! m7_lpuart1_ipg_stop - m7_lpuart1_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_lpuart1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED12_CLR_m7_lpuart2_ipg_stop_SHIFT (28U)
/*! m7_lpuart2_ipg_stop - m7_lpuart2_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_lpuart2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED12_CLR_m7_lpuart3_ipg_stop_SHIFT (29U)
/*! m7_lpuart3_ipg_stop - m7_lpuart3_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_lpuart3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED12_CLR_m7_lpuart4_ipg_stop_SHIFT (30U)
/*! m7_lpuart4_ipg_stop - m7_lpuart4_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_lpuart4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_CLR_m7_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED12_CLR_m7_lpuart5_ipg_stop_SHIFT (31U)
/*! m7_lpuart5_ipg_stop - m7_lpuart5_ipg_stop */
#define CCM_GPR_SHARED12_CLR_m7_lpuart5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_CLR_m7_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_CLR_m7_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED12_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED12_TOG_m7_cm7_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED12_TOG_m7_cm7_ipg_stop_SHIFT (0U)
/*! m7_cm7_ipg_stop - m7_cm7_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_cm7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_cm7_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_cm7_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_cm33_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED12_TOG_m7_cm33_ipg_stop_SHIFT (1U)
/*! m7_cm33_ipg_stop - m7_cm33_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_cm33_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_cm33_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_cm33_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_edma3_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED12_TOG_m7_edma3_ipg_stop_SHIFT (2U)
/*! m7_edma3_ipg_stop - m7_edma3_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_edma3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_edma3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_edma3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_edma4_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED12_TOG_m7_edma4_ipg_stop_SHIFT (3U)
/*! m7_edma4_ipg_stop - m7_edma4_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_edma4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_edma4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_edma4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_netc_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED12_TOG_m7_netc_ipg_stop_SHIFT (4U)
/*! m7_netc_ipg_stop - m7_netc_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_netc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_netc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_netc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_sim_aon_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED12_TOG_m7_sim_aon_ipg_stop_SHIFT (8U)
/*! m7_sim_aon_ipg_stop - m7_sim_aon_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_sim_aon_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_sim_aon_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_sim_aon_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_adc1_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED12_TOG_m7_adc1_ipg_stop_SHIFT (9U)
/*! m7_adc1_ipg_stop - m7_adc1_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_adc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_adc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_adc1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_adc2_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED12_TOG_m7_adc2_ipg_stop_SHIFT (10U)
/*! m7_adc2_ipg_stop - m7_adc2_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_adc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_adc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_adc2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_flexspi1_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED12_TOG_m7_flexspi1_ipg_stop_SHIFT (11U)
/*! m7_flexspi1_ipg_stop - m7_flexspi1_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_flexspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_flexspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_flexspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_flexspi2_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED12_TOG_m7_flexspi2_ipg_stop_SHIFT (12U)
/*! m7_flexspi2_ipg_stop - m7_flexspi2_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_flexspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_flexspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_flexspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_trdc_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED12_TOG_m7_trdc_ipg_stop_SHIFT (13U)
/*! m7_trdc_ipg_stop - m7_trdc_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_trdc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_trdc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_trdc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_semc_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED12_TOG_m7_semc_ipg_stop_SHIFT (14U)
/*! m7_semc_ipg_stop - m7_semc_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_semc_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_semc_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_semc_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_iee_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED12_TOG_m7_iee_ipg_stop_SHIFT (15U)
/*! m7_iee_ipg_stop - m7_iee_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_iee_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_iee_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_iee_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_gpio1_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED12_TOG_m7_gpio1_ipg_stop_SHIFT (16U)
/*! m7_gpio1_ipg_stop - m7_gpio1_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_gpio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_gpio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_gpio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_gpio2_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED12_TOG_m7_gpio2_ipg_stop_SHIFT (17U)
/*! m7_gpio2_ipg_stop - m7_gpio2_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_gpio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_gpio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_gpio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_gpio3_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED12_TOG_m7_gpio3_ipg_stop_SHIFT (18U)
/*! m7_gpio3_ipg_stop - m7_gpio3_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_gpio3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_gpio3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_gpio3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_gpio4_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED12_TOG_m7_gpio4_ipg_stop_SHIFT (19U)
/*! m7_gpio4_ipg_stop - m7_gpio4_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_gpio4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_gpio4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_gpio4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_gpio5_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED12_TOG_m7_gpio5_ipg_stop_SHIFT (20U)
/*! m7_gpio5_ipg_stop - m7_gpio5_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_gpio5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_gpio5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_gpio5_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_gpio6_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED12_TOG_m7_gpio6_ipg_stop_SHIFT (21U)
/*! m7_gpio6_ipg_stop - m7_gpio6_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_gpio6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_gpio6_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_gpio6_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_flexio1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED12_TOG_m7_flexio1_ipg_stop_SHIFT (22U)
/*! m7_flexio1_ipg_stop - m7_flexio1_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_flexio1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_flexio1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_flexio1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_flexio2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED12_TOG_m7_flexio2_ipg_stop_SHIFT (23U)
/*! m7_flexio2_ipg_stop - m7_flexio2_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_flexio2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_flexio2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_flexio2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_can1_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED12_TOG_m7_can1_ipg_stop_SHIFT (24U)
/*! m7_can1_ipg_stop - m7_can1_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_can1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_can1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_can1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_can2_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED12_TOG_m7_can2_ipg_stop_SHIFT (25U)
/*! m7_can2_ipg_stop - m7_can2_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_can2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_can2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_can2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_can3_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED12_TOG_m7_can3_ipg_stop_SHIFT (26U)
/*! m7_can3_ipg_stop - m7_can3_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_can3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_can3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_can3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_lpuart1_ipg_stop_MASK (0x8000000U)
#define CCM_GPR_SHARED12_TOG_m7_lpuart1_ipg_stop_SHIFT (27U)
/*! m7_lpuart1_ipg_stop - m7_lpuart1_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_lpuart1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_lpuart1_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_lpuart1_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_lpuart2_ipg_stop_MASK (0x10000000U)
#define CCM_GPR_SHARED12_TOG_m7_lpuart2_ipg_stop_SHIFT (28U)
/*! m7_lpuart2_ipg_stop - m7_lpuart2_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_lpuart2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_lpuart2_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_lpuart2_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_lpuart3_ipg_stop_MASK (0x20000000U)
#define CCM_GPR_SHARED12_TOG_m7_lpuart3_ipg_stop_SHIFT (29U)
/*! m7_lpuart3_ipg_stop - m7_lpuart3_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_lpuart3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_lpuart3_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_lpuart3_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_lpuart4_ipg_stop_MASK (0x40000000U)
#define CCM_GPR_SHARED12_TOG_m7_lpuart4_ipg_stop_SHIFT (30U)
/*! m7_lpuart4_ipg_stop - m7_lpuart4_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_lpuart4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_lpuart4_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_lpuart4_ipg_stop_MASK)

#define CCM_GPR_SHARED12_TOG_m7_lpuart5_ipg_stop_MASK (0x80000000U)
#define CCM_GPR_SHARED12_TOG_m7_lpuart5_ipg_stop_SHIFT (31U)
/*! m7_lpuart5_ipg_stop - m7_lpuart5_ipg_stop */
#define CCM_GPR_SHARED12_TOG_m7_lpuart5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_TOG_m7_lpuart5_ipg_stop_SHIFT)) & CCM_GPR_SHARED12_TOG_m7_lpuart5_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED12_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED12_AUTHEN_TZ_USER_MASK     (0x100U)
#define CCM_GPR_SHARED12_AUTHEN_TZ_USER_SHIFT    (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED12_AUTHEN_TZ_USER(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED12_AUTHEN_TZ_NS_MASK       (0x200U)
#define CCM_GPR_SHARED12_AUTHEN_TZ_NS_SHIFT      (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED12_AUTHEN_TZ_NS(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED12_AUTHEN_LOCK_TZ_MASK     (0x800U)
#define CCM_GPR_SHARED12_AUTHEN_LOCK_TZ_SHIFT    (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED12_AUTHEN_LOCK_TZ(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED12_AUTHEN_LOCK_LIST_MASK   (0x8000U)
#define CCM_GPR_SHARED12_AUTHEN_LOCK_LIST_SHIFT  (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED12_AUTHEN_LOCK_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED12_AUTHEN_WHITE_LIST_MASK  (0xFFFF0000U)
#define CCM_GPR_SHARED12_AUTHEN_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED12_AUTHEN_WHITE_LIST(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED12_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED12_AUTHEN_SET_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED12_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED12_AUTHEN_SET_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED12_AUTHEN_SET_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED12_AUTHEN_SET_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED12_AUTHEN_SET_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED12_AUTHEN_SET_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED12_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED12_AUTHEN_SET_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED12_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED12_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED12_AUTHEN_SET_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED12_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED12_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED12_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED12_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED12_AUTHEN_CLR_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED12_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED12_AUTHEN_CLR_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED12_AUTHEN_CLR_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED12_AUTHEN_CLR_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED12_AUTHEN_CLR_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED12_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED12_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED12_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED12_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED12_AUTHEN_TOG_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED12_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED12_AUTHEN_TOG_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED12_AUTHEN_TOG_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED12_AUTHEN_TOG_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED12_AUTHEN_TOG_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED12_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED12_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED12_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED12_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED12_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED13 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED13_m7_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED13_m7_lpuart6_ipg_stop_SHIFT (0U)
/*! m7_lpuart6_ipg_stop - m7_lpuart6_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpuart6_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED13_m7_lpuart7_ipg_stop_SHIFT (1U)
/*! m7_lpuart7_ipg_stop - m7_lpuart7_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpuart7_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED13_m7_lpuart8_ipg_stop_SHIFT (2U)
/*! m7_lpuart8_ipg_stop - m7_lpuart8_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpuart8_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED13_m7_lpuart9_ipg_stop_SHIFT (3U)
/*! m7_lpuart9_ipg_stop - m7_lpuart9_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpuart9_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED13_m7_lpuart10_ipg_stop_SHIFT (4U)
/*! m7_lpuart10_ipg_stop - m7_lpuart10_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED13_m7_lpuart11_ipg_stop_SHIFT (5U)
/*! m7_lpuart11_ipg_stop - m7_lpuart11_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED13_m7_lpuart12_ipg_stop_SHIFT (6U)
/*! m7_lpuart12_ipg_stop - m7_lpuart12_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED13_m7_lpi2c1_ipg_stop_SHIFT (7U)
/*! m7_lpi2c1_ipg_stop - m7_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpi2c1_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED13_m7_lpi2c2_ipg_stop_SHIFT (8U)
/*! m7_lpi2c2_ipg_stop - m7_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpi2c2_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED13_m7_lpi2c3_ipg_stop_SHIFT (9U)
/*! m7_lpi2c3_ipg_stop - m7_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpi2c3_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED13_m7_lpi2c4_ipg_stop_SHIFT (10U)
/*! m7_lpi2c4_ipg_stop - m7_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpi2c4_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED13_m7_lpi2c5_ipg_stop_SHIFT (11U)
/*! m7_lpi2c5_ipg_stop - m7_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpi2c5_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED13_m7_lpi2c6_ipg_stop_SHIFT (12U)
/*! m7_lpi2c6_ipg_stop - m7_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpi2c6_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED13_m7_lpspi1_ipg_stop_SHIFT (13U)
/*! m7_lpspi1_ipg_stop - m7_lpspi1_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpspi1_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED13_m7_lpspi2_ipg_stop_SHIFT (14U)
/*! m7_lpspi2_ipg_stop - m7_lpspi2_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpspi2_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED13_m7_lpspi3_ipg_stop_SHIFT (15U)
/*! m7_lpspi3_ipg_stop - m7_lpspi3_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpspi3_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED13_m7_lpspi4_ipg_stop_SHIFT (16U)
/*! m7_lpspi4_ipg_stop - m7_lpspi4_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpspi4_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED13_m7_lpspi5_ipg_stop_SHIFT (17U)
/*! m7_lpspi5_ipg_stop - m7_lpspi5_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpspi5_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED13_m7_lpspi6_ipg_stop_SHIFT (18U)
/*! m7_lpspi6_ipg_stop - m7_lpspi6_ipg_stop */
#define CCM_GPR_SHARED13_m7_lpspi6_ipg_stop(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_sinc1_ipg_stop_MASK  (0x80000U)
#define CCM_GPR_SHARED13_m7_sinc1_ipg_stop_SHIFT (19U)
/*! m7_sinc1_ipg_stop - m7_sinc1_ipg_stop */
#define CCM_GPR_SHARED13_m7_sinc1_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_sinc2_ipg_stop_MASK  (0x100000U)
#define CCM_GPR_SHARED13_m7_sinc2_ipg_stop_SHIFT (20U)
/*! m7_sinc2_ipg_stop - m7_sinc2_ipg_stop */
#define CCM_GPR_SHARED13_m7_sinc2_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_sinc3_ipg_stop_MASK  (0x200000U)
#define CCM_GPR_SHARED13_m7_sinc3_ipg_stop_SHIFT (21U)
/*! m7_sinc3_ipg_stop - m7_sinc3_ipg_stop */
#define CCM_GPR_SHARED13_m7_sinc3_ipg_stop(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_sai1_ipg_stop_MASK   (0x400000U)
#define CCM_GPR_SHARED13_m7_sai1_ipg_stop_SHIFT  (22U)
/*! m7_sai1_ipg_stop - m7_sai1_ipg_stop */
#define CCM_GPR_SHARED13_m7_sai1_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_sai2_ipg_stop_MASK   (0x800000U)
#define CCM_GPR_SHARED13_m7_sai2_ipg_stop_SHIFT  (23U)
/*! m7_sai2_ipg_stop - m7_sai2_ipg_stop */
#define CCM_GPR_SHARED13_m7_sai2_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_sai3_ipg_stop_MASK   (0x1000000U)
#define CCM_GPR_SHARED13_m7_sai3_ipg_stop_SHIFT  (24U)
/*! m7_sai3_ipg_stop - m7_sai3_ipg_stop */
#define CCM_GPR_SHARED13_m7_sai3_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_sai4_ipg_stop_MASK   (0x2000000U)
#define CCM_GPR_SHARED13_m7_sai4_ipg_stop_SHIFT  (25U)
/*! m7_sai4_ipg_stop - m7_sai4_ipg_stop */
#define CCM_GPR_SHARED13_m7_sai4_ipg_stop(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_m7_mic_ipg_stop_MASK    (0x4000000U)
#define CCM_GPR_SHARED13_m7_mic_ipg_stop_SHIFT   (26U)
/*! m7_mic_ipg_stop - m7_mic_ipg_stop */
#define CCM_GPR_SHARED13_m7_mic_ipg_stop(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_m7_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_m7_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED13_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED13_SET_m7_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED13_SET_m7_lpuart6_ipg_stop_SHIFT (0U)
/*! m7_lpuart6_ipg_stop - m7_lpuart6_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpuart6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED13_SET_m7_lpuart7_ipg_stop_SHIFT (1U)
/*! m7_lpuart7_ipg_stop - m7_lpuart7_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpuart7_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED13_SET_m7_lpuart8_ipg_stop_SHIFT (2U)
/*! m7_lpuart8_ipg_stop - m7_lpuart8_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpuart8_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED13_SET_m7_lpuart9_ipg_stop_SHIFT (3U)
/*! m7_lpuart9_ipg_stop - m7_lpuart9_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpuart9_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED13_SET_m7_lpuart10_ipg_stop_SHIFT (4U)
/*! m7_lpuart10_ipg_stop - m7_lpuart10_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED13_SET_m7_lpuart11_ipg_stop_SHIFT (5U)
/*! m7_lpuart11_ipg_stop - m7_lpuart11_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED13_SET_m7_lpuart12_ipg_stop_SHIFT (6U)
/*! m7_lpuart12_ipg_stop - m7_lpuart12_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED13_SET_m7_lpi2c1_ipg_stop_SHIFT (7U)
/*! m7_lpi2c1_ipg_stop - m7_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpi2c1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED13_SET_m7_lpi2c2_ipg_stop_SHIFT (8U)
/*! m7_lpi2c2_ipg_stop - m7_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpi2c2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED13_SET_m7_lpi2c3_ipg_stop_SHIFT (9U)
/*! m7_lpi2c3_ipg_stop - m7_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpi2c3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED13_SET_m7_lpi2c4_ipg_stop_SHIFT (10U)
/*! m7_lpi2c4_ipg_stop - m7_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpi2c4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED13_SET_m7_lpi2c5_ipg_stop_SHIFT (11U)
/*! m7_lpi2c5_ipg_stop - m7_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpi2c5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED13_SET_m7_lpi2c6_ipg_stop_SHIFT (12U)
/*! m7_lpi2c6_ipg_stop - m7_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpi2c6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED13_SET_m7_lpspi1_ipg_stop_SHIFT (13U)
/*! m7_lpspi1_ipg_stop - m7_lpspi1_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED13_SET_m7_lpspi2_ipg_stop_SHIFT (14U)
/*! m7_lpspi2_ipg_stop - m7_lpspi2_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED13_SET_m7_lpspi3_ipg_stop_SHIFT (15U)
/*! m7_lpspi3_ipg_stop - m7_lpspi3_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpspi3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED13_SET_m7_lpspi4_ipg_stop_SHIFT (16U)
/*! m7_lpspi4_ipg_stop - m7_lpspi4_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpspi4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED13_SET_m7_lpspi5_ipg_stop_SHIFT (17U)
/*! m7_lpspi5_ipg_stop - m7_lpspi5_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpspi5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED13_SET_m7_lpspi6_ipg_stop_SHIFT (18U)
/*! m7_lpspi6_ipg_stop - m7_lpspi6_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_lpspi6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_sinc1_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED13_SET_m7_sinc1_ipg_stop_SHIFT (19U)
/*! m7_sinc1_ipg_stop - m7_sinc1_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_sinc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_sinc2_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED13_SET_m7_sinc2_ipg_stop_SHIFT (20U)
/*! m7_sinc2_ipg_stop - m7_sinc2_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_sinc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_sinc3_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED13_SET_m7_sinc3_ipg_stop_SHIFT (21U)
/*! m7_sinc3_ipg_stop - m7_sinc3_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_sinc3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_sai1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED13_SET_m7_sai1_ipg_stop_SHIFT (22U)
/*! m7_sai1_ipg_stop - m7_sai1_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_sai1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_sai2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED13_SET_m7_sai2_ipg_stop_SHIFT (23U)
/*! m7_sai2_ipg_stop - m7_sai2_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_sai2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_sai3_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED13_SET_m7_sai3_ipg_stop_SHIFT (24U)
/*! m7_sai3_ipg_stop - m7_sai3_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_sai3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_sai4_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED13_SET_m7_sai4_ipg_stop_SHIFT (25U)
/*! m7_sai4_ipg_stop - m7_sai4_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_sai4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_SET_m7_mic_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED13_SET_m7_mic_ipg_stop_SHIFT (26U)
/*! m7_mic_ipg_stop - m7_mic_ipg_stop */
#define CCM_GPR_SHARED13_SET_m7_mic_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_SET_m7_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_SET_m7_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED13_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED13_CLR_m7_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED13_CLR_m7_lpuart6_ipg_stop_SHIFT (0U)
/*! m7_lpuart6_ipg_stop - m7_lpuart6_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpuart6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED13_CLR_m7_lpuart7_ipg_stop_SHIFT (1U)
/*! m7_lpuart7_ipg_stop - m7_lpuart7_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpuart7_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED13_CLR_m7_lpuart8_ipg_stop_SHIFT (2U)
/*! m7_lpuart8_ipg_stop - m7_lpuart8_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpuart8_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED13_CLR_m7_lpuart9_ipg_stop_SHIFT (3U)
/*! m7_lpuart9_ipg_stop - m7_lpuart9_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpuart9_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED13_CLR_m7_lpuart10_ipg_stop_SHIFT (4U)
/*! m7_lpuart10_ipg_stop - m7_lpuart10_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED13_CLR_m7_lpuart11_ipg_stop_SHIFT (5U)
/*! m7_lpuart11_ipg_stop - m7_lpuart11_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED13_CLR_m7_lpuart12_ipg_stop_SHIFT (6U)
/*! m7_lpuart12_ipg_stop - m7_lpuart12_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED13_CLR_m7_lpi2c1_ipg_stop_SHIFT (7U)
/*! m7_lpi2c1_ipg_stop - m7_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpi2c1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED13_CLR_m7_lpi2c2_ipg_stop_SHIFT (8U)
/*! m7_lpi2c2_ipg_stop - m7_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpi2c2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED13_CLR_m7_lpi2c3_ipg_stop_SHIFT (9U)
/*! m7_lpi2c3_ipg_stop - m7_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpi2c3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED13_CLR_m7_lpi2c4_ipg_stop_SHIFT (10U)
/*! m7_lpi2c4_ipg_stop - m7_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpi2c4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED13_CLR_m7_lpi2c5_ipg_stop_SHIFT (11U)
/*! m7_lpi2c5_ipg_stop - m7_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpi2c5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED13_CLR_m7_lpi2c6_ipg_stop_SHIFT (12U)
/*! m7_lpi2c6_ipg_stop - m7_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpi2c6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED13_CLR_m7_lpspi1_ipg_stop_SHIFT (13U)
/*! m7_lpspi1_ipg_stop - m7_lpspi1_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED13_CLR_m7_lpspi2_ipg_stop_SHIFT (14U)
/*! m7_lpspi2_ipg_stop - m7_lpspi2_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED13_CLR_m7_lpspi3_ipg_stop_SHIFT (15U)
/*! m7_lpspi3_ipg_stop - m7_lpspi3_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpspi3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED13_CLR_m7_lpspi4_ipg_stop_SHIFT (16U)
/*! m7_lpspi4_ipg_stop - m7_lpspi4_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpspi4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED13_CLR_m7_lpspi5_ipg_stop_SHIFT (17U)
/*! m7_lpspi5_ipg_stop - m7_lpspi5_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpspi5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED13_CLR_m7_lpspi6_ipg_stop_SHIFT (18U)
/*! m7_lpspi6_ipg_stop - m7_lpspi6_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_lpspi6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_sinc1_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED13_CLR_m7_sinc1_ipg_stop_SHIFT (19U)
/*! m7_sinc1_ipg_stop - m7_sinc1_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_sinc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_sinc2_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED13_CLR_m7_sinc2_ipg_stop_SHIFT (20U)
/*! m7_sinc2_ipg_stop - m7_sinc2_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_sinc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_sinc3_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED13_CLR_m7_sinc3_ipg_stop_SHIFT (21U)
/*! m7_sinc3_ipg_stop - m7_sinc3_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_sinc3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_sai1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED13_CLR_m7_sai1_ipg_stop_SHIFT (22U)
/*! m7_sai1_ipg_stop - m7_sai1_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_sai1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_sai2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED13_CLR_m7_sai2_ipg_stop_SHIFT (23U)
/*! m7_sai2_ipg_stop - m7_sai2_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_sai2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_sai3_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED13_CLR_m7_sai3_ipg_stop_SHIFT (24U)
/*! m7_sai3_ipg_stop - m7_sai3_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_sai3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_sai4_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED13_CLR_m7_sai4_ipg_stop_SHIFT (25U)
/*! m7_sai4_ipg_stop - m7_sai4_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_sai4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_CLR_m7_mic_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED13_CLR_m7_mic_ipg_stop_SHIFT (26U)
/*! m7_mic_ipg_stop - m7_mic_ipg_stop */
#define CCM_GPR_SHARED13_CLR_m7_mic_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_CLR_m7_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_CLR_m7_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED13_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED13_TOG_m7_lpuart6_ipg_stop_MASK (0x1U)
#define CCM_GPR_SHARED13_TOG_m7_lpuart6_ipg_stop_SHIFT (0U)
/*! m7_lpuart6_ipg_stop - m7_lpuart6_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpuart6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpuart6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpuart6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpuart7_ipg_stop_MASK (0x2U)
#define CCM_GPR_SHARED13_TOG_m7_lpuart7_ipg_stop_SHIFT (1U)
/*! m7_lpuart7_ipg_stop - m7_lpuart7_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpuart7_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpuart7_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpuart7_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpuart8_ipg_stop_MASK (0x4U)
#define CCM_GPR_SHARED13_TOG_m7_lpuart8_ipg_stop_SHIFT (2U)
/*! m7_lpuart8_ipg_stop - m7_lpuart8_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpuart8_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpuart8_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpuart8_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpuart9_ipg_stop_MASK (0x8U)
#define CCM_GPR_SHARED13_TOG_m7_lpuart9_ipg_stop_SHIFT (3U)
/*! m7_lpuart9_ipg_stop - m7_lpuart9_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpuart9_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpuart9_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpuart9_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpuart10_ipg_stop_MASK (0x10U)
#define CCM_GPR_SHARED13_TOG_m7_lpuart10_ipg_stop_SHIFT (4U)
/*! m7_lpuart10_ipg_stop - m7_lpuart10_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpuart10_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpuart10_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpuart10_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpuart11_ipg_stop_MASK (0x20U)
#define CCM_GPR_SHARED13_TOG_m7_lpuart11_ipg_stop_SHIFT (5U)
/*! m7_lpuart11_ipg_stop - m7_lpuart11_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpuart11_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpuart11_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpuart11_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpuart12_ipg_stop_MASK (0x40U)
#define CCM_GPR_SHARED13_TOG_m7_lpuart12_ipg_stop_SHIFT (6U)
/*! m7_lpuart12_ipg_stop - m7_lpuart12_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpuart12_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpuart12_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpuart12_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpi2c1_ipg_stop_MASK (0x80U)
#define CCM_GPR_SHARED13_TOG_m7_lpi2c1_ipg_stop_SHIFT (7U)
/*! m7_lpi2c1_ipg_stop - m7_lpi2c1_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpi2c1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpi2c1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpi2c1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpi2c2_ipg_stop_MASK (0x100U)
#define CCM_GPR_SHARED13_TOG_m7_lpi2c2_ipg_stop_SHIFT (8U)
/*! m7_lpi2c2_ipg_stop - m7_lpi2c2_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpi2c2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpi2c2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpi2c2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpi2c3_ipg_stop_MASK (0x200U)
#define CCM_GPR_SHARED13_TOG_m7_lpi2c3_ipg_stop_SHIFT (9U)
/*! m7_lpi2c3_ipg_stop - m7_lpi2c3_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpi2c3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpi2c3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpi2c3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpi2c4_ipg_stop_MASK (0x400U)
#define CCM_GPR_SHARED13_TOG_m7_lpi2c4_ipg_stop_SHIFT (10U)
/*! m7_lpi2c4_ipg_stop - m7_lpi2c4_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpi2c4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpi2c4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpi2c4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpi2c5_ipg_stop_MASK (0x800U)
#define CCM_GPR_SHARED13_TOG_m7_lpi2c5_ipg_stop_SHIFT (11U)
/*! m7_lpi2c5_ipg_stop - m7_lpi2c5_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpi2c5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpi2c5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpi2c5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpi2c6_ipg_stop_MASK (0x1000U)
#define CCM_GPR_SHARED13_TOG_m7_lpi2c6_ipg_stop_SHIFT (12U)
/*! m7_lpi2c6_ipg_stop - m7_lpi2c6_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpi2c6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpi2c6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpi2c6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpspi1_ipg_stop_MASK (0x2000U)
#define CCM_GPR_SHARED13_TOG_m7_lpspi1_ipg_stop_SHIFT (13U)
/*! m7_lpspi1_ipg_stop - m7_lpspi1_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpspi1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpspi1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpspi1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpspi2_ipg_stop_MASK (0x4000U)
#define CCM_GPR_SHARED13_TOG_m7_lpspi2_ipg_stop_SHIFT (14U)
/*! m7_lpspi2_ipg_stop - m7_lpspi2_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpspi2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpspi2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpspi2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpspi3_ipg_stop_MASK (0x8000U)
#define CCM_GPR_SHARED13_TOG_m7_lpspi3_ipg_stop_SHIFT (15U)
/*! m7_lpspi3_ipg_stop - m7_lpspi3_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpspi3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpspi3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpspi3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpspi4_ipg_stop_MASK (0x10000U)
#define CCM_GPR_SHARED13_TOG_m7_lpspi4_ipg_stop_SHIFT (16U)
/*! m7_lpspi4_ipg_stop - m7_lpspi4_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpspi4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpspi4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpspi4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpspi5_ipg_stop_MASK (0x20000U)
#define CCM_GPR_SHARED13_TOG_m7_lpspi5_ipg_stop_SHIFT (17U)
/*! m7_lpspi5_ipg_stop - m7_lpspi5_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpspi5_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpspi5_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpspi5_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_lpspi6_ipg_stop_MASK (0x40000U)
#define CCM_GPR_SHARED13_TOG_m7_lpspi6_ipg_stop_SHIFT (18U)
/*! m7_lpspi6_ipg_stop - m7_lpspi6_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_lpspi6_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_lpspi6_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_lpspi6_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_sinc1_ipg_stop_MASK (0x80000U)
#define CCM_GPR_SHARED13_TOG_m7_sinc1_ipg_stop_SHIFT (19U)
/*! m7_sinc1_ipg_stop - m7_sinc1_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_sinc1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_sinc1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_sinc1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_sinc2_ipg_stop_MASK (0x100000U)
#define CCM_GPR_SHARED13_TOG_m7_sinc2_ipg_stop_SHIFT (20U)
/*! m7_sinc2_ipg_stop - m7_sinc2_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_sinc2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_sinc2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_sinc2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_sinc3_ipg_stop_MASK (0x200000U)
#define CCM_GPR_SHARED13_TOG_m7_sinc3_ipg_stop_SHIFT (21U)
/*! m7_sinc3_ipg_stop - m7_sinc3_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_sinc3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_sinc3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_sinc3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_sai1_ipg_stop_MASK (0x400000U)
#define CCM_GPR_SHARED13_TOG_m7_sai1_ipg_stop_SHIFT (22U)
/*! m7_sai1_ipg_stop - m7_sai1_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_sai1_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_sai1_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_sai1_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_sai2_ipg_stop_MASK (0x800000U)
#define CCM_GPR_SHARED13_TOG_m7_sai2_ipg_stop_SHIFT (23U)
/*! m7_sai2_ipg_stop - m7_sai2_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_sai2_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_sai2_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_sai2_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_sai3_ipg_stop_MASK (0x1000000U)
#define CCM_GPR_SHARED13_TOG_m7_sai3_ipg_stop_SHIFT (24U)
/*! m7_sai3_ipg_stop - m7_sai3_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_sai3_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_sai3_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_sai3_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_sai4_ipg_stop_MASK (0x2000000U)
#define CCM_GPR_SHARED13_TOG_m7_sai4_ipg_stop_SHIFT (25U)
/*! m7_sai4_ipg_stop - m7_sai4_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_sai4_ipg_stop(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_sai4_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_sai4_ipg_stop_MASK)

#define CCM_GPR_SHARED13_TOG_m7_mic_ipg_stop_MASK (0x4000000U)
#define CCM_GPR_SHARED13_TOG_m7_mic_ipg_stop_SHIFT (26U)
/*! m7_mic_ipg_stop - m7_mic_ipg_stop */
#define CCM_GPR_SHARED13_TOG_m7_mic_ipg_stop(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_TOG_m7_mic_ipg_stop_SHIFT)) & CCM_GPR_SHARED13_TOG_m7_mic_ipg_stop_MASK)
/*! @} */

/*! @name GPR_SHARED13_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED13_AUTHEN_TZ_USER_MASK     (0x100U)
#define CCM_GPR_SHARED13_AUTHEN_TZ_USER_SHIFT    (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED13_AUTHEN_TZ_USER(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED13_AUTHEN_TZ_NS_MASK       (0x200U)
#define CCM_GPR_SHARED13_AUTHEN_TZ_NS_SHIFT      (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED13_AUTHEN_TZ_NS(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED13_AUTHEN_LOCK_TZ_MASK     (0x800U)
#define CCM_GPR_SHARED13_AUTHEN_LOCK_TZ_SHIFT    (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED13_AUTHEN_LOCK_TZ(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED13_AUTHEN_LOCK_LIST_MASK   (0x8000U)
#define CCM_GPR_SHARED13_AUTHEN_LOCK_LIST_SHIFT  (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED13_AUTHEN_LOCK_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED13_AUTHEN_WHITE_LIST_MASK  (0xFFFF0000U)
#define CCM_GPR_SHARED13_AUTHEN_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED13_AUTHEN_WHITE_LIST(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED13_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED13_AUTHEN_SET_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED13_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED13_AUTHEN_SET_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED13_AUTHEN_SET_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED13_AUTHEN_SET_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED13_AUTHEN_SET_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED13_AUTHEN_SET_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED13_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED13_AUTHEN_SET_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED13_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED13_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED13_AUTHEN_SET_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED13_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED13_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED13_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED13_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED13_AUTHEN_CLR_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED13_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED13_AUTHEN_CLR_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED13_AUTHEN_CLR_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED13_AUTHEN_CLR_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED13_AUTHEN_CLR_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED13_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED13_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED13_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED13_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED13_AUTHEN_TOG_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED13_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED13_AUTHEN_TOG_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED13_AUTHEN_TOG_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED13_AUTHEN_TOG_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED13_AUTHEN_TOG_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED13_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED13_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED13_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED13_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED13_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED14 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED14_m7_adc1_ipg_doze_MASK   (0x1U)
#define CCM_GPR_SHARED14_m7_adc1_ipg_doze_SHIFT  (0U)
/*! m7_adc1_ipg_doze - m7_adc1_ipg_doze */
#define CCM_GPR_SHARED14_m7_adc1_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_adc2_ipg_doze_MASK   (0x2U)
#define CCM_GPR_SHARED14_m7_adc2_ipg_doze_SHIFT  (1U)
/*! m7_adc2_ipg_doze - m7_adc2_ipg_doze */
#define CCM_GPR_SHARED14_m7_adc2_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED14_m7_flexspi1_ipg_doze_SHIFT (2U)
/*! m7_flexspi1_ipg_doze - m7_flexspi1_ipg_doze */
#define CCM_GPR_SHARED14_m7_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED14_m7_flexspi2_ipg_doze_SHIFT (3U)
/*! m7_flexspi2_ipg_doze - m7_flexspi2_ipg_doze */
#define CCM_GPR_SHARED14_m7_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED14_m7_flexio1_ipg_doze_SHIFT (4U)
/*! m7_flexio1_ipg_doze - m7_flexio1_ipg_doze */
#define CCM_GPR_SHARED14_m7_flexio1_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED14_m7_flexio2_ipg_doze_SHIFT (5U)
/*! m7_flexio2_ipg_doze - m7_flexio2_ipg_doze */
#define CCM_GPR_SHARED14_m7_flexio2_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpit1_ipg_doze_MASK  (0x40U)
#define CCM_GPR_SHARED14_m7_lpit1_ipg_doze_SHIFT (6U)
/*! m7_lpit1_ipg_doze - m7_lpit1_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpit1_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpit2_ipg_doze_MASK  (0x80U)
#define CCM_GPR_SHARED14_m7_lpit2_ipg_doze_SHIFT (7U)
/*! m7_lpit2_ipg_doze - m7_lpit2_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpit2_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpit3_ipg_doze_MASK  (0x100U)
#define CCM_GPR_SHARED14_m7_lpit3_ipg_doze_SHIFT (8U)
/*! m7_lpit3_ipg_doze - m7_lpit3_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpit3_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_tpm1_ipg_doze_MASK   (0x200U)
#define CCM_GPR_SHARED14_m7_tpm1_ipg_doze_SHIFT  (9U)
/*! m7_tpm1_ipg_doze - m7_tpm1_ipg_doze */
#define CCM_GPR_SHARED14_m7_tpm1_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_tpm2_ipg_doze_MASK   (0x400U)
#define CCM_GPR_SHARED14_m7_tpm2_ipg_doze_SHIFT  (10U)
/*! m7_tpm2_ipg_doze - m7_tpm2_ipg_doze */
#define CCM_GPR_SHARED14_m7_tpm2_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_tpm3_ipg_doze_MASK   (0x800U)
#define CCM_GPR_SHARED14_m7_tpm3_ipg_doze_SHIFT  (11U)
/*! m7_tpm3_ipg_doze - m7_tpm3_ipg_doze */
#define CCM_GPR_SHARED14_m7_tpm3_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_tpm4_ipg_doze_MASK   (0x1000U)
#define CCM_GPR_SHARED14_m7_tpm4_ipg_doze_SHIFT  (12U)
/*! m7_tpm4_ipg_doze - m7_tpm4_ipg_doze */
#define CCM_GPR_SHARED14_m7_tpm4_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_tpm5_ipg_doze_MASK   (0x2000U)
#define CCM_GPR_SHARED14_m7_tpm5_ipg_doze_SHIFT  (13U)
/*! m7_tpm5_ipg_doze - m7_tpm5_ipg_doze */
#define CCM_GPR_SHARED14_m7_tpm5_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_tpm6_ipg_doze_MASK   (0x4000U)
#define CCM_GPR_SHARED14_m7_tpm6_ipg_doze_SHIFT  (14U)
/*! m7_tpm6_ipg_doze - m7_tpm6_ipg_doze */
#define CCM_GPR_SHARED14_m7_tpm6_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_gpt1_ipg_doze_MASK   (0x8000U)
#define CCM_GPR_SHARED14_m7_gpt1_ipg_doze_SHIFT  (15U)
/*! m7_gpt1_ipg_doze - m7_gpt1_ipg_doze */
#define CCM_GPR_SHARED14_m7_gpt1_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_gpt2_ipg_doze_MASK   (0x10000U)
#define CCM_GPR_SHARED14_m7_gpt2_ipg_doze_SHIFT  (16U)
/*! m7_gpt2_ipg_doze - m7_gpt2_ipg_doze */
#define CCM_GPR_SHARED14_m7_gpt2_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_can1_ipg_doze_MASK   (0x20000U)
#define CCM_GPR_SHARED14_m7_can1_ipg_doze_SHIFT  (17U)
/*! m7_can1_ipg_doze - m7_can1_ipg_doze */
#define CCM_GPR_SHARED14_m7_can1_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_can2_ipg_doze_MASK   (0x40000U)
#define CCM_GPR_SHARED14_m7_can2_ipg_doze_SHIFT  (18U)
/*! m7_can2_ipg_doze - m7_can2_ipg_doze */
#define CCM_GPR_SHARED14_m7_can2_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_can3_ipg_doze_MASK   (0x80000U)
#define CCM_GPR_SHARED14_m7_can3_ipg_doze_SHIFT  (19U)
/*! m7_can3_ipg_doze - m7_can3_ipg_doze */
#define CCM_GPR_SHARED14_m7_can3_ipg_doze(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED14_m7_lpuart1_ipg_doze_SHIFT (20U)
/*! m7_lpuart1_ipg_doze - m7_lpuart1_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart1_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED14_m7_lpuart2_ipg_doze_SHIFT (21U)
/*! m7_lpuart2_ipg_doze - m7_lpuart2_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart2_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED14_m7_lpuart3_ipg_doze_SHIFT (22U)
/*! m7_lpuart3_ipg_doze - m7_lpuart3_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart3_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED14_m7_lpuart4_ipg_doze_SHIFT (23U)
/*! m7_lpuart4_ipg_doze - m7_lpuart4_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart4_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED14_m7_lpuart5_ipg_doze_SHIFT (24U)
/*! m7_lpuart5_ipg_doze - m7_lpuart5_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart5_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED14_m7_lpuart6_ipg_doze_SHIFT (25U)
/*! m7_lpuart6_ipg_doze - m7_lpuart6_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart6_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED14_m7_lpuart7_ipg_doze_SHIFT (26U)
/*! m7_lpuart7_ipg_doze - m7_lpuart7_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart7_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED14_m7_lpuart8_ipg_doze_SHIFT (27U)
/*! m7_lpuart8_ipg_doze - m7_lpuart8_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart8_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED14_m7_lpuart9_ipg_doze_SHIFT (28U)
/*! m7_lpuart9_ipg_doze - m7_lpuart9_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart9_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED14_m7_lpuart10_ipg_doze_SHIFT (29U)
/*! m7_lpuart10_ipg_doze - m7_lpuart10_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED14_m7_lpuart11_ipg_doze_SHIFT (30U)
/*! m7_lpuart11_ipg_doze - m7_lpuart11_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED14_m7_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED14_m7_lpuart12_ipg_doze_SHIFT (31U)
/*! m7_lpuart12_ipg_doze - m7_lpuart12_ipg_doze */
#define CCM_GPR_SHARED14_m7_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_m7_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_m7_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED14_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED14_SET_m7_adc1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED14_SET_m7_adc1_ipg_doze_SHIFT (0U)
/*! m7_adc1_ipg_doze - m7_adc1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_adc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_adc2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED14_SET_m7_adc2_ipg_doze_SHIFT (1U)
/*! m7_adc2_ipg_doze - m7_adc2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_adc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED14_SET_m7_flexspi1_ipg_doze_SHIFT (2U)
/*! m7_flexspi1_ipg_doze - m7_flexspi1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED14_SET_m7_flexspi2_ipg_doze_SHIFT (3U)
/*! m7_flexspi2_ipg_doze - m7_flexspi2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED14_SET_m7_flexio1_ipg_doze_SHIFT (4U)
/*! m7_flexio1_ipg_doze - m7_flexio1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_flexio1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED14_SET_m7_flexio2_ipg_doze_SHIFT (5U)
/*! m7_flexio2_ipg_doze - m7_flexio2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_flexio2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpit1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED14_SET_m7_lpit1_ipg_doze_SHIFT (6U)
/*! m7_lpit1_ipg_doze - m7_lpit1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpit1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpit2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED14_SET_m7_lpit2_ipg_doze_SHIFT (7U)
/*! m7_lpit2_ipg_doze - m7_lpit2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpit2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpit3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED14_SET_m7_lpit3_ipg_doze_SHIFT (8U)
/*! m7_lpit3_ipg_doze - m7_lpit3_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpit3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_tpm1_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED14_SET_m7_tpm1_ipg_doze_SHIFT (9U)
/*! m7_tpm1_ipg_doze - m7_tpm1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_tpm1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_tpm2_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED14_SET_m7_tpm2_ipg_doze_SHIFT (10U)
/*! m7_tpm2_ipg_doze - m7_tpm2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_tpm2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_tpm3_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED14_SET_m7_tpm3_ipg_doze_SHIFT (11U)
/*! m7_tpm3_ipg_doze - m7_tpm3_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_tpm3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_tpm4_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED14_SET_m7_tpm4_ipg_doze_SHIFT (12U)
/*! m7_tpm4_ipg_doze - m7_tpm4_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_tpm4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_tpm5_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED14_SET_m7_tpm5_ipg_doze_SHIFT (13U)
/*! m7_tpm5_ipg_doze - m7_tpm5_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_tpm5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_tpm6_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED14_SET_m7_tpm6_ipg_doze_SHIFT (14U)
/*! m7_tpm6_ipg_doze - m7_tpm6_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_tpm6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_gpt1_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED14_SET_m7_gpt1_ipg_doze_SHIFT (15U)
/*! m7_gpt1_ipg_doze - m7_gpt1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_gpt1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_gpt2_ipg_doze_MASK (0x10000U)
#define CCM_GPR_SHARED14_SET_m7_gpt2_ipg_doze_SHIFT (16U)
/*! m7_gpt2_ipg_doze - m7_gpt2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_gpt2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_can1_ipg_doze_MASK (0x20000U)
#define CCM_GPR_SHARED14_SET_m7_can1_ipg_doze_SHIFT (17U)
/*! m7_can1_ipg_doze - m7_can1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_can1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_can2_ipg_doze_MASK (0x40000U)
#define CCM_GPR_SHARED14_SET_m7_can2_ipg_doze_SHIFT (18U)
/*! m7_can2_ipg_doze - m7_can2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_can2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_can3_ipg_doze_MASK (0x80000U)
#define CCM_GPR_SHARED14_SET_m7_can3_ipg_doze_SHIFT (19U)
/*! m7_can3_ipg_doze - m7_can3_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_can3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart1_ipg_doze_SHIFT (20U)
/*! m7_lpuart1_ipg_doze - m7_lpuart1_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart2_ipg_doze_SHIFT (21U)
/*! m7_lpuart2_ipg_doze - m7_lpuart2_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart3_ipg_doze_SHIFT (22U)
/*! m7_lpuart3_ipg_doze - m7_lpuart3_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart4_ipg_doze_SHIFT (23U)
/*! m7_lpuart4_ipg_doze - m7_lpuart4_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart5_ipg_doze_SHIFT (24U)
/*! m7_lpuart5_ipg_doze - m7_lpuart5_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart6_ipg_doze_SHIFT (25U)
/*! m7_lpuart6_ipg_doze - m7_lpuart6_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart7_ipg_doze_SHIFT (26U)
/*! m7_lpuart7_ipg_doze - m7_lpuart7_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart7_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart8_ipg_doze_SHIFT (27U)
/*! m7_lpuart8_ipg_doze - m7_lpuart8_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart8_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart9_ipg_doze_SHIFT (28U)
/*! m7_lpuart9_ipg_doze - m7_lpuart9_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart9_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart10_ipg_doze_SHIFT (29U)
/*! m7_lpuart10_ipg_doze - m7_lpuart10_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart11_ipg_doze_SHIFT (30U)
/*! m7_lpuart11_ipg_doze - m7_lpuart11_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED14_SET_m7_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED14_SET_m7_lpuart12_ipg_doze_SHIFT (31U)
/*! m7_lpuart12_ipg_doze - m7_lpuart12_ipg_doze */
#define CCM_GPR_SHARED14_SET_m7_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_SET_m7_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_SET_m7_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED14_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED14_CLR_m7_adc1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED14_CLR_m7_adc1_ipg_doze_SHIFT (0U)
/*! m7_adc1_ipg_doze - m7_adc1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_adc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_adc2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED14_CLR_m7_adc2_ipg_doze_SHIFT (1U)
/*! m7_adc2_ipg_doze - m7_adc2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_adc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED14_CLR_m7_flexspi1_ipg_doze_SHIFT (2U)
/*! m7_flexspi1_ipg_doze - m7_flexspi1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED14_CLR_m7_flexspi2_ipg_doze_SHIFT (3U)
/*! m7_flexspi2_ipg_doze - m7_flexspi2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED14_CLR_m7_flexio1_ipg_doze_SHIFT (4U)
/*! m7_flexio1_ipg_doze - m7_flexio1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_flexio1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED14_CLR_m7_flexio2_ipg_doze_SHIFT (5U)
/*! m7_flexio2_ipg_doze - m7_flexio2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_flexio2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpit1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED14_CLR_m7_lpit1_ipg_doze_SHIFT (6U)
/*! m7_lpit1_ipg_doze - m7_lpit1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpit1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpit2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED14_CLR_m7_lpit2_ipg_doze_SHIFT (7U)
/*! m7_lpit2_ipg_doze - m7_lpit2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpit2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpit3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED14_CLR_m7_lpit3_ipg_doze_SHIFT (8U)
/*! m7_lpit3_ipg_doze - m7_lpit3_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpit3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_tpm1_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED14_CLR_m7_tpm1_ipg_doze_SHIFT (9U)
/*! m7_tpm1_ipg_doze - m7_tpm1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_tpm1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_tpm2_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED14_CLR_m7_tpm2_ipg_doze_SHIFT (10U)
/*! m7_tpm2_ipg_doze - m7_tpm2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_tpm2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_tpm3_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED14_CLR_m7_tpm3_ipg_doze_SHIFT (11U)
/*! m7_tpm3_ipg_doze - m7_tpm3_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_tpm3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_tpm4_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED14_CLR_m7_tpm4_ipg_doze_SHIFT (12U)
/*! m7_tpm4_ipg_doze - m7_tpm4_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_tpm4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_tpm5_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED14_CLR_m7_tpm5_ipg_doze_SHIFT (13U)
/*! m7_tpm5_ipg_doze - m7_tpm5_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_tpm5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_tpm6_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED14_CLR_m7_tpm6_ipg_doze_SHIFT (14U)
/*! m7_tpm6_ipg_doze - m7_tpm6_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_tpm6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_gpt1_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED14_CLR_m7_gpt1_ipg_doze_SHIFT (15U)
/*! m7_gpt1_ipg_doze - m7_gpt1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_gpt1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_gpt2_ipg_doze_MASK (0x10000U)
#define CCM_GPR_SHARED14_CLR_m7_gpt2_ipg_doze_SHIFT (16U)
/*! m7_gpt2_ipg_doze - m7_gpt2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_gpt2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_can1_ipg_doze_MASK (0x20000U)
#define CCM_GPR_SHARED14_CLR_m7_can1_ipg_doze_SHIFT (17U)
/*! m7_can1_ipg_doze - m7_can1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_can1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_can2_ipg_doze_MASK (0x40000U)
#define CCM_GPR_SHARED14_CLR_m7_can2_ipg_doze_SHIFT (18U)
/*! m7_can2_ipg_doze - m7_can2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_can2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_can3_ipg_doze_MASK (0x80000U)
#define CCM_GPR_SHARED14_CLR_m7_can3_ipg_doze_SHIFT (19U)
/*! m7_can3_ipg_doze - m7_can3_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_can3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart1_ipg_doze_SHIFT (20U)
/*! m7_lpuart1_ipg_doze - m7_lpuart1_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart2_ipg_doze_SHIFT (21U)
/*! m7_lpuart2_ipg_doze - m7_lpuart2_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart3_ipg_doze_SHIFT (22U)
/*! m7_lpuart3_ipg_doze - m7_lpuart3_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart4_ipg_doze_SHIFT (23U)
/*! m7_lpuart4_ipg_doze - m7_lpuart4_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart5_ipg_doze_SHIFT (24U)
/*! m7_lpuart5_ipg_doze - m7_lpuart5_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart6_ipg_doze_SHIFT (25U)
/*! m7_lpuart6_ipg_doze - m7_lpuart6_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart7_ipg_doze_SHIFT (26U)
/*! m7_lpuart7_ipg_doze - m7_lpuart7_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart7_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart8_ipg_doze_SHIFT (27U)
/*! m7_lpuart8_ipg_doze - m7_lpuart8_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart8_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart9_ipg_doze_SHIFT (28U)
/*! m7_lpuart9_ipg_doze - m7_lpuart9_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart9_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart10_ipg_doze_SHIFT (29U)
/*! m7_lpuart10_ipg_doze - m7_lpuart10_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart11_ipg_doze_SHIFT (30U)
/*! m7_lpuart11_ipg_doze - m7_lpuart11_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED14_CLR_m7_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED14_CLR_m7_lpuart12_ipg_doze_SHIFT (31U)
/*! m7_lpuart12_ipg_doze - m7_lpuart12_ipg_doze */
#define CCM_GPR_SHARED14_CLR_m7_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_CLR_m7_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_CLR_m7_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED14_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED14_TOG_m7_adc1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED14_TOG_m7_adc1_ipg_doze_SHIFT (0U)
/*! m7_adc1_ipg_doze - m7_adc1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_adc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_adc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_adc1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_adc2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED14_TOG_m7_adc2_ipg_doze_SHIFT (1U)
/*! m7_adc2_ipg_doze - m7_adc2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_adc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_adc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_adc2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_flexspi1_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED14_TOG_m7_flexspi1_ipg_doze_SHIFT (2U)
/*! m7_flexspi1_ipg_doze - m7_flexspi1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_flexspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_flexspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_flexspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_flexspi2_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED14_TOG_m7_flexspi2_ipg_doze_SHIFT (3U)
/*! m7_flexspi2_ipg_doze - m7_flexspi2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_flexspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_flexspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_flexspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_flexio1_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED14_TOG_m7_flexio1_ipg_doze_SHIFT (4U)
/*! m7_flexio1_ipg_doze - m7_flexio1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_flexio1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_flexio1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_flexio1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_flexio2_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED14_TOG_m7_flexio2_ipg_doze_SHIFT (5U)
/*! m7_flexio2_ipg_doze - m7_flexio2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_flexio2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_flexio2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_flexio2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpit1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED14_TOG_m7_lpit1_ipg_doze_SHIFT (6U)
/*! m7_lpit1_ipg_doze - m7_lpit1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpit1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpit1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpit1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpit2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED14_TOG_m7_lpit2_ipg_doze_SHIFT (7U)
/*! m7_lpit2_ipg_doze - m7_lpit2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpit2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpit2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpit2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpit3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED14_TOG_m7_lpit3_ipg_doze_SHIFT (8U)
/*! m7_lpit3_ipg_doze - m7_lpit3_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpit3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpit3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpit3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_tpm1_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED14_TOG_m7_tpm1_ipg_doze_SHIFT (9U)
/*! m7_tpm1_ipg_doze - m7_tpm1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_tpm1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_tpm1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_tpm1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_tpm2_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED14_TOG_m7_tpm2_ipg_doze_SHIFT (10U)
/*! m7_tpm2_ipg_doze - m7_tpm2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_tpm2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_tpm2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_tpm2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_tpm3_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED14_TOG_m7_tpm3_ipg_doze_SHIFT (11U)
/*! m7_tpm3_ipg_doze - m7_tpm3_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_tpm3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_tpm3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_tpm3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_tpm4_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED14_TOG_m7_tpm4_ipg_doze_SHIFT (12U)
/*! m7_tpm4_ipg_doze - m7_tpm4_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_tpm4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_tpm4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_tpm4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_tpm5_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED14_TOG_m7_tpm5_ipg_doze_SHIFT (13U)
/*! m7_tpm5_ipg_doze - m7_tpm5_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_tpm5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_tpm5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_tpm5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_tpm6_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED14_TOG_m7_tpm6_ipg_doze_SHIFT (14U)
/*! m7_tpm6_ipg_doze - m7_tpm6_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_tpm6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_tpm6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_tpm6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_gpt1_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED14_TOG_m7_gpt1_ipg_doze_SHIFT (15U)
/*! m7_gpt1_ipg_doze - m7_gpt1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_gpt1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_gpt1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_gpt1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_gpt2_ipg_doze_MASK (0x10000U)
#define CCM_GPR_SHARED14_TOG_m7_gpt2_ipg_doze_SHIFT (16U)
/*! m7_gpt2_ipg_doze - m7_gpt2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_gpt2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_gpt2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_gpt2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_can1_ipg_doze_MASK (0x20000U)
#define CCM_GPR_SHARED14_TOG_m7_can1_ipg_doze_SHIFT (17U)
/*! m7_can1_ipg_doze - m7_can1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_can1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_can1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_can1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_can2_ipg_doze_MASK (0x40000U)
#define CCM_GPR_SHARED14_TOG_m7_can2_ipg_doze_SHIFT (18U)
/*! m7_can2_ipg_doze - m7_can2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_can2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_can2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_can2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_can3_ipg_doze_MASK (0x80000U)
#define CCM_GPR_SHARED14_TOG_m7_can3_ipg_doze_SHIFT (19U)
/*! m7_can3_ipg_doze - m7_can3_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_can3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_can3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_can3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart1_ipg_doze_MASK (0x100000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart1_ipg_doze_SHIFT (20U)
/*! m7_lpuart1_ipg_doze - m7_lpuart1_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart1_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart1_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart2_ipg_doze_MASK (0x200000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart2_ipg_doze_SHIFT (21U)
/*! m7_lpuart2_ipg_doze - m7_lpuart2_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart2_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart2_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart3_ipg_doze_MASK (0x400000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart3_ipg_doze_SHIFT (22U)
/*! m7_lpuart3_ipg_doze - m7_lpuart3_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart3_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart3_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart4_ipg_doze_MASK (0x800000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart4_ipg_doze_SHIFT (23U)
/*! m7_lpuart4_ipg_doze - m7_lpuart4_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart4_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart4_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart5_ipg_doze_MASK (0x1000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart5_ipg_doze_SHIFT (24U)
/*! m7_lpuart5_ipg_doze - m7_lpuart5_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart5_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart5_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart6_ipg_doze_MASK (0x2000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart6_ipg_doze_SHIFT (25U)
/*! m7_lpuart6_ipg_doze - m7_lpuart6_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart6_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart6_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart7_ipg_doze_MASK (0x4000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart7_ipg_doze_SHIFT (26U)
/*! m7_lpuart7_ipg_doze - m7_lpuart7_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart7_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart7_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart7_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart8_ipg_doze_MASK (0x8000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart8_ipg_doze_SHIFT (27U)
/*! m7_lpuart8_ipg_doze - m7_lpuart8_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart8_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart8_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart8_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart9_ipg_doze_MASK (0x10000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart9_ipg_doze_SHIFT (28U)
/*! m7_lpuart9_ipg_doze - m7_lpuart9_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart9_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart9_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart9_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart10_ipg_doze_MASK (0x20000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart10_ipg_doze_SHIFT (29U)
/*! m7_lpuart10_ipg_doze - m7_lpuart10_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart10_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart10_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart10_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart11_ipg_doze_MASK (0x40000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart11_ipg_doze_SHIFT (30U)
/*! m7_lpuart11_ipg_doze - m7_lpuart11_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart11_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart11_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart11_ipg_doze_MASK)

#define CCM_GPR_SHARED14_TOG_m7_lpuart12_ipg_doze_MASK (0x80000000U)
#define CCM_GPR_SHARED14_TOG_m7_lpuart12_ipg_doze_SHIFT (31U)
/*! m7_lpuart12_ipg_doze - m7_lpuart12_ipg_doze */
#define CCM_GPR_SHARED14_TOG_m7_lpuart12_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_TOG_m7_lpuart12_ipg_doze_SHIFT)) & CCM_GPR_SHARED14_TOG_m7_lpuart12_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED14_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED14_AUTHEN_TZ_USER_MASK     (0x100U)
#define CCM_GPR_SHARED14_AUTHEN_TZ_USER_SHIFT    (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED14_AUTHEN_TZ_USER(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED14_AUTHEN_TZ_NS_MASK       (0x200U)
#define CCM_GPR_SHARED14_AUTHEN_TZ_NS_SHIFT      (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED14_AUTHEN_TZ_NS(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED14_AUTHEN_LOCK_TZ_MASK     (0x800U)
#define CCM_GPR_SHARED14_AUTHEN_LOCK_TZ_SHIFT    (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED14_AUTHEN_LOCK_TZ(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED14_AUTHEN_LOCK_LIST_MASK   (0x8000U)
#define CCM_GPR_SHARED14_AUTHEN_LOCK_LIST_SHIFT  (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED14_AUTHEN_LOCK_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED14_AUTHEN_WHITE_LIST_MASK  (0xFFFF0000U)
#define CCM_GPR_SHARED14_AUTHEN_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED14_AUTHEN_WHITE_LIST(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED14_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED14_AUTHEN_SET_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED14_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED14_AUTHEN_SET_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED14_AUTHEN_SET_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED14_AUTHEN_SET_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED14_AUTHEN_SET_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED14_AUTHEN_SET_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED14_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED14_AUTHEN_SET_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED14_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED14_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED14_AUTHEN_SET_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED14_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED14_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED14_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED14_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED14_AUTHEN_CLR_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED14_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED14_AUTHEN_CLR_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED14_AUTHEN_CLR_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED14_AUTHEN_CLR_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED14_AUTHEN_CLR_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED14_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED14_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED14_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED14_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED14_AUTHEN_TOG_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED14_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED14_AUTHEN_TOG_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED14_AUTHEN_TOG_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED14_AUTHEN_TOG_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED14_AUTHEN_TOG_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED14_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED14_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED14_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED14_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED14_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED15 - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED15_m7_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED15_m7_lpi2c1_ipg_doze_SHIFT (0U)
/*! m7_lpi2c1_ipg_doze - m7_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpi2c1_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED15_m7_lpi2c2_ipg_doze_SHIFT (1U)
/*! m7_lpi2c2_ipg_doze - m7_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpi2c2_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED15_m7_lpi2c3_ipg_doze_SHIFT (2U)
/*! m7_lpi2c3_ipg_doze - m7_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpi2c3_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED15_m7_lpi2c4_ipg_doze_SHIFT (3U)
/*! m7_lpi2c4_ipg_doze - m7_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpi2c4_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED15_m7_lpi2c5_ipg_doze_SHIFT (4U)
/*! m7_lpi2c5_ipg_doze - m7_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpi2c5_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED15_m7_lpi2c6_ipg_doze_SHIFT (5U)
/*! m7_lpi2c6_ipg_doze - m7_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpi2c6_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED15_m7_lpspi1_ipg_doze_SHIFT (6U)
/*! m7_lpspi1_ipg_doze - m7_lpspi1_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpspi1_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED15_m7_lpspi2_ipg_doze_SHIFT (7U)
/*! m7_lpspi2_ipg_doze - m7_lpspi2_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpspi2_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED15_m7_lpspi3_ipg_doze_SHIFT (8U)
/*! m7_lpspi3_ipg_doze - m7_lpspi3_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpspi3_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED15_m7_lpspi4_ipg_doze_SHIFT (9U)
/*! m7_lpspi4_ipg_doze - m7_lpspi4_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpspi4_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED15_m7_lpspi5_ipg_doze_SHIFT (10U)
/*! m7_lpspi5_ipg_doze - m7_lpspi5_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpspi5_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED15_m7_lpspi6_ipg_doze_SHIFT (11U)
/*! m7_lpspi6_ipg_doze - m7_lpspi6_ipg_doze */
#define CCM_GPR_SHARED15_m7_lpspi6_ipg_doze(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_sinc1_ipg_doze_MASK  (0x1000U)
#define CCM_GPR_SHARED15_m7_sinc1_ipg_doze_SHIFT (12U)
/*! m7_sinc1_ipg_doze - m7_sinc1_ipg_doze */
#define CCM_GPR_SHARED15_m7_sinc1_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_sinc2_ipg_doze_MASK  (0x2000U)
#define CCM_GPR_SHARED15_m7_sinc2_ipg_doze_SHIFT (13U)
/*! m7_sinc2_ipg_doze - m7_sinc2_ipg_doze */
#define CCM_GPR_SHARED15_m7_sinc2_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_sinc3_ipg_doze_MASK  (0x4000U)
#define CCM_GPR_SHARED15_m7_sinc3_ipg_doze_SHIFT (14U)
/*! m7_sinc3_ipg_doze - m7_sinc3_ipg_doze */
#define CCM_GPR_SHARED15_m7_sinc3_ipg_doze(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_m7_mic_ipg_doze_MASK    (0x8000U)
#define CCM_GPR_SHARED15_m7_mic_ipg_doze_SHIFT   (15U)
/*! m7_mic_ipg_doze - m7_mic_ipg_doze */
#define CCM_GPR_SHARED15_m7_mic_ipg_doze(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_m7_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_m7_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED15_SET - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED15_SET_m7_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED15_SET_m7_lpi2c1_ipg_doze_SHIFT (0U)
/*! m7_lpi2c1_ipg_doze - m7_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpi2c1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED15_SET_m7_lpi2c2_ipg_doze_SHIFT (1U)
/*! m7_lpi2c2_ipg_doze - m7_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpi2c2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED15_SET_m7_lpi2c3_ipg_doze_SHIFT (2U)
/*! m7_lpi2c3_ipg_doze - m7_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpi2c3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED15_SET_m7_lpi2c4_ipg_doze_SHIFT (3U)
/*! m7_lpi2c4_ipg_doze - m7_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpi2c4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED15_SET_m7_lpi2c5_ipg_doze_SHIFT (4U)
/*! m7_lpi2c5_ipg_doze - m7_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpi2c5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED15_SET_m7_lpi2c6_ipg_doze_SHIFT (5U)
/*! m7_lpi2c6_ipg_doze - m7_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpi2c6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED15_SET_m7_lpspi1_ipg_doze_SHIFT (6U)
/*! m7_lpspi1_ipg_doze - m7_lpspi1_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED15_SET_m7_lpspi2_ipg_doze_SHIFT (7U)
/*! m7_lpspi2_ipg_doze - m7_lpspi2_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED15_SET_m7_lpspi3_ipg_doze_SHIFT (8U)
/*! m7_lpspi3_ipg_doze - m7_lpspi3_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpspi3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED15_SET_m7_lpspi4_ipg_doze_SHIFT (9U)
/*! m7_lpspi4_ipg_doze - m7_lpspi4_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpspi4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED15_SET_m7_lpspi5_ipg_doze_SHIFT (10U)
/*! m7_lpspi5_ipg_doze - m7_lpspi5_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpspi5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED15_SET_m7_lpspi6_ipg_doze_SHIFT (11U)
/*! m7_lpspi6_ipg_doze - m7_lpspi6_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_lpspi6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_sinc1_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED15_SET_m7_sinc1_ipg_doze_SHIFT (12U)
/*! m7_sinc1_ipg_doze - m7_sinc1_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_sinc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_sinc2_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED15_SET_m7_sinc2_ipg_doze_SHIFT (13U)
/*! m7_sinc2_ipg_doze - m7_sinc2_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_sinc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_sinc3_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED15_SET_m7_sinc3_ipg_doze_SHIFT (14U)
/*! m7_sinc3_ipg_doze - m7_sinc3_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_sinc3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_SET_m7_mic_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED15_SET_m7_mic_ipg_doze_SHIFT (15U)
/*! m7_mic_ipg_doze - m7_mic_ipg_doze */
#define CCM_GPR_SHARED15_SET_m7_mic_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_SET_m7_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_SET_m7_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED15_CLR - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED15_CLR_m7_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED15_CLR_m7_lpi2c1_ipg_doze_SHIFT (0U)
/*! m7_lpi2c1_ipg_doze - m7_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpi2c1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED15_CLR_m7_lpi2c2_ipg_doze_SHIFT (1U)
/*! m7_lpi2c2_ipg_doze - m7_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpi2c2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED15_CLR_m7_lpi2c3_ipg_doze_SHIFT (2U)
/*! m7_lpi2c3_ipg_doze - m7_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpi2c3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED15_CLR_m7_lpi2c4_ipg_doze_SHIFT (3U)
/*! m7_lpi2c4_ipg_doze - m7_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpi2c4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED15_CLR_m7_lpi2c5_ipg_doze_SHIFT (4U)
/*! m7_lpi2c5_ipg_doze - m7_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpi2c5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED15_CLR_m7_lpi2c6_ipg_doze_SHIFT (5U)
/*! m7_lpi2c6_ipg_doze - m7_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpi2c6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED15_CLR_m7_lpspi1_ipg_doze_SHIFT (6U)
/*! m7_lpspi1_ipg_doze - m7_lpspi1_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED15_CLR_m7_lpspi2_ipg_doze_SHIFT (7U)
/*! m7_lpspi2_ipg_doze - m7_lpspi2_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED15_CLR_m7_lpspi3_ipg_doze_SHIFT (8U)
/*! m7_lpspi3_ipg_doze - m7_lpspi3_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpspi3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED15_CLR_m7_lpspi4_ipg_doze_SHIFT (9U)
/*! m7_lpspi4_ipg_doze - m7_lpspi4_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpspi4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED15_CLR_m7_lpspi5_ipg_doze_SHIFT (10U)
/*! m7_lpspi5_ipg_doze - m7_lpspi5_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpspi5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED15_CLR_m7_lpspi6_ipg_doze_SHIFT (11U)
/*! m7_lpspi6_ipg_doze - m7_lpspi6_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_lpspi6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_sinc1_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED15_CLR_m7_sinc1_ipg_doze_SHIFT (12U)
/*! m7_sinc1_ipg_doze - m7_sinc1_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_sinc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_sinc2_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED15_CLR_m7_sinc2_ipg_doze_SHIFT (13U)
/*! m7_sinc2_ipg_doze - m7_sinc2_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_sinc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_sinc3_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED15_CLR_m7_sinc3_ipg_doze_SHIFT (14U)
/*! m7_sinc3_ipg_doze - m7_sinc3_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_sinc3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_CLR_m7_mic_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED15_CLR_m7_mic_ipg_doze_SHIFT (15U)
/*! m7_mic_ipg_doze - m7_mic_ipg_doze */
#define CCM_GPR_SHARED15_CLR_m7_mic_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_CLR_m7_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_CLR_m7_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED15_TOG - General Purpose Register */
/*! @{ */

#define CCM_GPR_SHARED15_TOG_m7_lpi2c1_ipg_doze_MASK (0x1U)
#define CCM_GPR_SHARED15_TOG_m7_lpi2c1_ipg_doze_SHIFT (0U)
/*! m7_lpi2c1_ipg_doze - m7_lpi2c1_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpi2c1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpi2c1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpi2c1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpi2c2_ipg_doze_MASK (0x2U)
#define CCM_GPR_SHARED15_TOG_m7_lpi2c2_ipg_doze_SHIFT (1U)
/*! m7_lpi2c2_ipg_doze - m7_lpi2c2_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpi2c2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpi2c2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpi2c2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpi2c3_ipg_doze_MASK (0x4U)
#define CCM_GPR_SHARED15_TOG_m7_lpi2c3_ipg_doze_SHIFT (2U)
/*! m7_lpi2c3_ipg_doze - m7_lpi2c3_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpi2c3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpi2c3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpi2c3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpi2c4_ipg_doze_MASK (0x8U)
#define CCM_GPR_SHARED15_TOG_m7_lpi2c4_ipg_doze_SHIFT (3U)
/*! m7_lpi2c4_ipg_doze - m7_lpi2c4_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpi2c4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpi2c4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpi2c4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpi2c5_ipg_doze_MASK (0x10U)
#define CCM_GPR_SHARED15_TOG_m7_lpi2c5_ipg_doze_SHIFT (4U)
/*! m7_lpi2c5_ipg_doze - m7_lpi2c5_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpi2c5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpi2c5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpi2c5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpi2c6_ipg_doze_MASK (0x20U)
#define CCM_GPR_SHARED15_TOG_m7_lpi2c6_ipg_doze_SHIFT (5U)
/*! m7_lpi2c6_ipg_doze - m7_lpi2c6_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpi2c6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpi2c6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpi2c6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpspi1_ipg_doze_MASK (0x40U)
#define CCM_GPR_SHARED15_TOG_m7_lpspi1_ipg_doze_SHIFT (6U)
/*! m7_lpspi1_ipg_doze - m7_lpspi1_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpspi1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpspi1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpspi1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpspi2_ipg_doze_MASK (0x80U)
#define CCM_GPR_SHARED15_TOG_m7_lpspi2_ipg_doze_SHIFT (7U)
/*! m7_lpspi2_ipg_doze - m7_lpspi2_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpspi2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpspi2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpspi2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpspi3_ipg_doze_MASK (0x100U)
#define CCM_GPR_SHARED15_TOG_m7_lpspi3_ipg_doze_SHIFT (8U)
/*! m7_lpspi3_ipg_doze - m7_lpspi3_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpspi3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpspi3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpspi3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpspi4_ipg_doze_MASK (0x200U)
#define CCM_GPR_SHARED15_TOG_m7_lpspi4_ipg_doze_SHIFT (9U)
/*! m7_lpspi4_ipg_doze - m7_lpspi4_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpspi4_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpspi4_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpspi4_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpspi5_ipg_doze_MASK (0x400U)
#define CCM_GPR_SHARED15_TOG_m7_lpspi5_ipg_doze_SHIFT (10U)
/*! m7_lpspi5_ipg_doze - m7_lpspi5_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpspi5_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpspi5_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpspi5_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_lpspi6_ipg_doze_MASK (0x800U)
#define CCM_GPR_SHARED15_TOG_m7_lpspi6_ipg_doze_SHIFT (11U)
/*! m7_lpspi6_ipg_doze - m7_lpspi6_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_lpspi6_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_lpspi6_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_lpspi6_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_sinc1_ipg_doze_MASK (0x1000U)
#define CCM_GPR_SHARED15_TOG_m7_sinc1_ipg_doze_SHIFT (12U)
/*! m7_sinc1_ipg_doze - m7_sinc1_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_sinc1_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_sinc1_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_sinc1_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_sinc2_ipg_doze_MASK (0x2000U)
#define CCM_GPR_SHARED15_TOG_m7_sinc2_ipg_doze_SHIFT (13U)
/*! m7_sinc2_ipg_doze - m7_sinc2_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_sinc2_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_sinc2_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_sinc2_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_sinc3_ipg_doze_MASK (0x4000U)
#define CCM_GPR_SHARED15_TOG_m7_sinc3_ipg_doze_SHIFT (14U)
/*! m7_sinc3_ipg_doze - m7_sinc3_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_sinc3_ipg_doze(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_sinc3_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_sinc3_ipg_doze_MASK)

#define CCM_GPR_SHARED15_TOG_m7_mic_ipg_doze_MASK (0x8000U)
#define CCM_GPR_SHARED15_TOG_m7_mic_ipg_doze_SHIFT (15U)
/*! m7_mic_ipg_doze - m7_mic_ipg_doze */
#define CCM_GPR_SHARED15_TOG_m7_mic_ipg_doze(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_TOG_m7_mic_ipg_doze_SHIFT)) & CCM_GPR_SHARED15_TOG_m7_mic_ipg_doze_MASK)
/*! @} */

/*! @name GPR_SHARED15_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED15_AUTHEN_TZ_USER_MASK     (0x100U)
#define CCM_GPR_SHARED15_AUTHEN_TZ_USER_SHIFT    (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of shared GPR slice can be changed in user mode.
 *  0b0..Registers of shared GPR slice cannot be changed in user mode.
 */
#define CCM_GPR_SHARED15_AUTHEN_TZ_USER(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_SHARED15_AUTHEN_TZ_NS_MASK       (0x200U)
#define CCM_GPR_SHARED15_AUTHEN_TZ_NS_SHIFT      (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_SHARED15_AUTHEN_TZ_NS(x)         (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_SHARED15_AUTHEN_LOCK_TZ_MASK     (0x800U)
#define CCM_GPR_SHARED15_AUTHEN_LOCK_TZ_SHIFT    (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_SHARED15_AUTHEN_LOCK_TZ(x)       (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_SHARED15_AUTHEN_LOCK_LIST_MASK   (0x8000U)
#define CCM_GPR_SHARED15_AUTHEN_LOCK_LIST_SHIFT  (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_SHARED15_AUTHEN_LOCK_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_SHARED15_AUTHEN_WHITE_LIST_MASK  (0xFFFF0000U)
#define CCM_GPR_SHARED15_AUTHEN_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED15_AUTHEN_WHITE_LIST(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED15_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED15_AUTHEN_SET_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED15_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED15_AUTHEN_SET_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_SHARED15_AUTHEN_SET_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED15_AUTHEN_SET_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED15_AUTHEN_SET_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_SHARED15_AUTHEN_SET_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED15_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED15_AUTHEN_SET_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_SHARED15_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED15_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED15_AUTHEN_SET_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_SHARED15_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED15_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED15_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED15_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED15_AUTHEN_CLR_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED15_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED15_AUTHEN_CLR_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_SHARED15_AUTHEN_CLR_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED15_AUTHEN_CLR_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED15_AUTHEN_CLR_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_SHARED15_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED15_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED15_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED15_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_SHARED15_AUTHEN_TOG_TZ_USER_MASK (0x100U)
#define CCM_GPR_SHARED15_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_SHARED15_AUTHEN_TOG_TZ_USER(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_SHARED15_AUTHEN_TOG_TZ_NS_MASK   (0x200U)
#define CCM_GPR_SHARED15_AUTHEN_TOG_TZ_NS_SHIFT  (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_SHARED15_AUTHEN_TOG_TZ_NS(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_TZ_MASK (0x800U)
#define CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_TZ(x)   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_SHARED15_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_SHARED15_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_SHARED15_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED15_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_SHARED15_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/*! @name GPR_SHARED_STATUS - General purpose status register for CM33..General purpose status register for CM7 */
/*! @{ */

#define CCM_GPR_SHARED_STATUS_GPR_STATUS_MASK    (0xFFFFFFFFU)
#define CCM_GPR_SHARED_STATUS_GPR_STATUS_SHIFT   (0U)
/*! GPR_STATUS - Acknowledge indicators for low power handshake */
#define CCM_GPR_SHARED_STATUS_GPR_STATUS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_SHARED_STATUS_GPR_STATUS_SHIFT)) & CCM_GPR_SHARED_STATUS_GPR_STATUS_MASK)
/*! @} */

/* The count of CCM_GPR_SHARED_STATUS */
#define CCM_GPR_SHARED_STATUS_COUNT              (8U)

/*! @name GPR_PRIVATE - General purpose register */
/*! @{ */

#define CCM_GPR_PRIVATE_GPR_MASK                 (0xFFFFFFFFU)
#define CCM_GPR_PRIVATE_GPR_SHIFT                (0U)
/*! GPR - GP register */
#define CCM_GPR_PRIVATE_GPR(x)                   (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_GPR_SHIFT)) & CCM_GPR_PRIVATE_GPR_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE */
#define CCM_GPR_PRIVATE_COUNT                    (4U)

/*! @name GPR_PRIVATE_SET - General purpose register */
/*! @{ */

#define CCM_GPR_PRIVATE_SET_GPR_MASK             (0xFFFFFFFFU)
#define CCM_GPR_PRIVATE_SET_GPR_SHIFT            (0U)
/*! GPR - GP register */
#define CCM_GPR_PRIVATE_SET_GPR(x)               (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_SET_GPR_SHIFT)) & CCM_GPR_PRIVATE_SET_GPR_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE_SET */
#define CCM_GPR_PRIVATE_SET_COUNT                (4U)

/*! @name GPR_PRIVATE_CLR - General purpose register */
/*! @{ */

#define CCM_GPR_PRIVATE_CLR_GPR_MASK             (0xFFFFFFFFU)
#define CCM_GPR_PRIVATE_CLR_GPR_SHIFT            (0U)
/*! GPR - GP register */
#define CCM_GPR_PRIVATE_CLR_GPR(x)               (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_CLR_GPR_SHIFT)) & CCM_GPR_PRIVATE_CLR_GPR_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE_CLR */
#define CCM_GPR_PRIVATE_CLR_COUNT                (4U)

/*! @name GPR_PRIVATE_TOG - General purpose register */
/*! @{ */

#define CCM_GPR_PRIVATE_TOG_GPR_MASK             (0xFFFFFFFFU)
#define CCM_GPR_PRIVATE_TOG_GPR_SHIFT            (0U)
/*! GPR - GP register */
#define CCM_GPR_PRIVATE_TOG_GPR(x)               (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_TOG_GPR_SHIFT)) & CCM_GPR_PRIVATE_TOG_GPR_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE_TOG */
#define CCM_GPR_PRIVATE_TOG_COUNT                (4U)

/*! @name GPR_PRIVATE_AUTHEN - GPR access control */
/*! @{ */

#define CCM_GPR_PRIVATE_AUTHEN_TZ_USER_MASK      (0x100U)
#define CCM_GPR_PRIVATE_AUTHEN_TZ_USER_SHIFT     (8U)
/*! TZ_USER - User access permission
 *  0b1..Registers of private GPR can be changed in user mode.
 *  0b0..Registers of private GPR cannot be changed in user mode.
 */
#define CCM_GPR_PRIVATE_AUTHEN_TZ_USER(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_TZ_USER_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_TZ_USER_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_TZ_NS_MASK        (0x200U)
#define CCM_GPR_PRIVATE_AUTHEN_TZ_NS_SHIFT       (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_GPR_PRIVATE_AUTHEN_TZ_NS(x)          (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_TZ_NS_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_TZ_NS_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_LOCK_TZ_MASK      (0x800U)
#define CCM_GPR_PRIVATE_AUTHEN_LOCK_TZ_SHIFT     (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_GPR_PRIVATE_AUTHEN_LOCK_TZ(x)        (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_LOCK_TZ_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_LOCK_TZ_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_LOCK_LIST_MASK    (0x8000U)
#define CCM_GPR_PRIVATE_AUTHEN_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_GPR_PRIVATE_AUTHEN_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_LOCK_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_LOCK_LIST_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_WHITE_LIST_MASK   (0xFFFF0000U)
#define CCM_GPR_PRIVATE_AUTHEN_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_PRIVATE_AUTHEN_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_WHITE_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE_AUTHEN */
#define CCM_GPR_PRIVATE_AUTHEN_COUNT             (4U)

/*! @name GPR_PRIVATE_AUTHEN_SET - GPR access control */
/*! @{ */

#define CCM_GPR_PRIVATE_AUTHEN_SET_TZ_USER_MASK  (0x100U)
#define CCM_GPR_PRIVATE_AUTHEN_SET_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_PRIVATE_AUTHEN_SET_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_SET_TZ_USER_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_SET_TZ_USER_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_SET_TZ_NS_MASK    (0x200U)
#define CCM_GPR_PRIVATE_AUTHEN_SET_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_PRIVATE_AUTHEN_SET_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_SET_TZ_NS_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_SET_TZ_NS_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_TZ_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_TZ_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_SET_LOCK_LIST_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_SET_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_PRIVATE_AUTHEN_SET_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_PRIVATE_AUTHEN_SET_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_SET_WHITE_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_SET_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE_AUTHEN_SET */
#define CCM_GPR_PRIVATE_AUTHEN_SET_COUNT         (4U)

/*! @name GPR_PRIVATE_AUTHEN_CLR - GPR access control */
/*! @{ */

#define CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_USER_MASK  (0x100U)
#define CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_USER_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_USER_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_NS_MASK    (0x200U)
#define CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_NS_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_CLR_TZ_NS_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_TZ_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_TZ_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_CLR_LOCK_LIST_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_CLR_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_PRIVATE_AUTHEN_CLR_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_PRIVATE_AUTHEN_CLR_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_CLR_WHITE_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_CLR_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE_AUTHEN_CLR */
#define CCM_GPR_PRIVATE_AUTHEN_CLR_COUNT         (4U)

/*! @name GPR_PRIVATE_AUTHEN_TOG - GPR access control */
/*! @{ */

#define CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_USER_MASK  (0x100U)
#define CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_USER_SHIFT (8U)
/*! TZ_USER - User access permission */
#define CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_USER(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_USER_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_USER_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_NS_MASK    (0x200U)
#define CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_NS_SHIFT   (9U)
/*! TZ_NS - Non-secure access permission */
#define CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_NS(x)      (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_NS_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_TOG_TZ_NS_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_TZ_MASK  (0x800U)
#define CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_TZ_SHIFT (11U)
/*! LOCK_TZ - Lock TrustZone settings */
#define CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_TZ(x)    (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_TZ_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_TZ_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_LIST_MASK (0x8000U)
#define CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - Lock white list */
#define CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_LIST(x)  (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_TOG_LOCK_LIST_MASK)

#define CCM_GPR_PRIVATE_AUTHEN_TOG_WHITE_LIST_MASK (0xFFFF0000U)
#define CCM_GPR_PRIVATE_AUTHEN_TOG_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Whitelist settings */
#define CCM_GPR_PRIVATE_AUTHEN_TOG_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << CCM_GPR_PRIVATE_AUTHEN_TOG_WHITE_LIST_SHIFT)) & CCM_GPR_PRIVATE_AUTHEN_TOG_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_GPR_PRIVATE_AUTHEN_TOG */
#define CCM_GPR_PRIVATE_AUTHEN_TOG_COUNT         (4U)

/*! @name OSCPLL_DIRECT - Clock source direct control */
/*! @{ */

#define CCM_OSCPLL_DIRECT_ON_MASK                (0x1U)
#define CCM_OSCPLL_DIRECT_ON_SHIFT               (0U)
/*! ON - Turn on clock source
 *  0b0..Clock source is OFF.
 *  0b1..Clock source is ON.
 */
#define CCM_OSCPLL_DIRECT_ON(x)                  (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_DIRECT_ON_SHIFT)) & CCM_OSCPLL_DIRECT_ON_MASK)
/*! @} */

/* The count of CCM_OSCPLL_DIRECT */
#define CCM_OSCPLL_DIRECT_COUNT                  (25U)

/*! @name OSCPLL_LPM0 - Clock source low power mode setting */
/*! @{ */

#define CCM_OSCPLL_LPM0_LPM_SETTING_D0_MASK      (0x7U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D0_SHIFT     (0U)
/*! LPM_SETTING_D0 - Clock Source LPM in DOMAIN0
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D0(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D0_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D0_MASK)

#define CCM_OSCPLL_LPM0_LPM_SETTING_D1_MASK      (0x70U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D1_SHIFT     (4U)
/*! LPM_SETTING_D1 - Clock Source LPM in DOMAIN1
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D1(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D1_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D1_MASK)

#define CCM_OSCPLL_LPM0_LPM_SETTING_D2_MASK      (0x700U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D2_SHIFT     (8U)
/*! LPM_SETTING_D2 - Clock Source LPM in DOMAIN2
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D2(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D2_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D2_MASK)

#define CCM_OSCPLL_LPM0_LPM_SETTING_D3_MASK      (0x7000U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D3_SHIFT     (12U)
/*! LPM_SETTING_D3 - Clock Source LPM in DOMAIN3
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D3(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D3_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D3_MASK)

#define CCM_OSCPLL_LPM0_LPM_SETTING_D4_MASK      (0x70000U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D4_SHIFT     (16U)
/*! LPM_SETTING_D4 - Clock Source LPM in DOMAIN4
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D4(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D4_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D4_MASK)

#define CCM_OSCPLL_LPM0_LPM_SETTING_D5_MASK      (0x700000U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D5_SHIFT     (20U)
/*! LPM_SETTING_D5 - Clock Source LPM in DOMAIN5
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D5(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D5_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D5_MASK)

#define CCM_OSCPLL_LPM0_LPM_SETTING_D6_MASK      (0x7000000U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D6_SHIFT     (24U)
/*! LPM_SETTING_D6 - Clock Source LPM in DOMAIN6
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D6(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D6_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D6_MASK)

#define CCM_OSCPLL_LPM0_LPM_SETTING_D7_MASK      (0x70000000U)
#define CCM_OSCPLL_LPM0_LPM_SETTING_D7_SHIFT     (28U)
/*! LPM_SETTING_D7 - Clock Source LPM in DOMAIN7
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM0_LPM_SETTING_D7(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM0_LPM_SETTING_D7_SHIFT)) & CCM_OSCPLL_LPM0_LPM_SETTING_D7_MASK)
/*! @} */

/* The count of CCM_OSCPLL_LPM0 */
#define CCM_OSCPLL_LPM0_COUNT                    (25U)

/*! @name OSCPLL_LPM1 - clock source low power mode setting */
/*! @{ */

#define CCM_OSCPLL_LPM1_LPM_SETTING_D8_MASK      (0x7U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D8_SHIFT     (0U)
/*! LPM_SETTING_D8 - Clock Source LPM in DOMAIN8
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D8(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D8_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D8_MASK)

#define CCM_OSCPLL_LPM1_LPM_SETTING_D9_MASK      (0x70U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D9_SHIFT     (4U)
/*! LPM_SETTING_D9 - Clock Source LPM in DOMAIN9
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D9(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D9_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D9_MASK)

#define CCM_OSCPLL_LPM1_LPM_SETTING_D10_MASK     (0x700U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D10_SHIFT    (8U)
/*! LPM_SETTING_D10 - Clock Source LPM in DOMAIN10
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D10(x)       (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D10_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D10_MASK)

#define CCM_OSCPLL_LPM1_LPM_SETTING_D11_MASK     (0x7000U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D11_SHIFT    (12U)
/*! LPM_SETTING_D11 - Clock Source LPM in DOMAIN11
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D11(x)       (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D11_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D11_MASK)

#define CCM_OSCPLL_LPM1_LPM_SETTING_D12_MASK     (0x70000U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D12_SHIFT    (16U)
/*! LPM_SETTING_D12 - Clock Source LPM in DOMAIN12
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D12(x)       (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D12_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D12_MASK)

#define CCM_OSCPLL_LPM1_LPM_SETTING_D13_MASK     (0x700000U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D13_SHIFT    (20U)
/*! LPM_SETTING_D13 - Clock Source LPM in DOMAIN13
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D13(x)       (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D13_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D13_MASK)

#define CCM_OSCPLL_LPM1_LPM_SETTING_D14_MASK     (0x7000000U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D14_SHIFT    (24U)
/*! LPM_SETTING_D14 - Clock Source LPM in DOMAIN14
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D14(x)       (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D14_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D14_MASK)

#define CCM_OSCPLL_LPM1_LPM_SETTING_D15_MASK     (0x70000000U)
#define CCM_OSCPLL_LPM1_LPM_SETTING_D15_SHIFT    (28U)
/*! LPM_SETTING_D15 - Clock Source LPM in DOMAIN15
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM1_LPM_SETTING_D15(x)       (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM1_LPM_SETTING_D15_SHIFT)) & CCM_OSCPLL_LPM1_LPM_SETTING_D15_MASK)
/*! @} */

/* The count of CCM_OSCPLL_LPM1 */
#define CCM_OSCPLL_LPM1_COUNT                    (25U)

/*! @name OSCPLL_LPM_CUR - LPM setting of current CPU domain */
/*! @{ */

#define CCM_OSCPLL_LPM_CUR_LPM_SETTING_CUR_MASK  (0x7U)
#define CCM_OSCPLL_LPM_CUR_LPM_SETTING_CUR_SHIFT (0U)
/*! LPM_SETTING_CUR - LPM settings value for current CPU domain that is reading this register.
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_OSCPLL_LPM_CUR_LPM_SETTING_CUR(x)    (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_LPM_CUR_LPM_SETTING_CUR_SHIFT)) & CCM_OSCPLL_LPM_CUR_LPM_SETTING_CUR_MASK)
/*! @} */

/* The count of CCM_OSCPLL_LPM_CUR */
#define CCM_OSCPLL_LPM_CUR_COUNT                 (25U)

/*! @name OSCPLL_STATUS0 - Clock source working status */
/*! @{ */

#define CCM_OSCPLL_STATUS0_ON_MASK               (0x1U)
#define CCM_OSCPLL_STATUS0_ON_SHIFT              (0U)
/*! ON - Clock source current state
 *  0b0..Clock source is OFF.
 *  0b1..Clock source is ON.
 */
#define CCM_OSCPLL_STATUS0_ON(x)                 (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_STATUS0_ON_SHIFT)) & CCM_OSCPLL_STATUS0_ON_MASK)

#define CCM_OSCPLL_STATUS0_STATUS_EARLY_MASK     (0x10U)
#define CCM_OSCPLL_STATUS0_STATUS_EARLY_SHIFT    (4U)
/*! STATUS_EARLY - Clock source active
 *  0b0..Clock source is not active
 *  0b1..Clock source is active
 */
#define CCM_OSCPLL_STATUS0_STATUS_EARLY(x)       (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_STATUS0_STATUS_EARLY_SHIFT)) & CCM_OSCPLL_STATUS0_STATUS_EARLY_MASK)

#define CCM_OSCPLL_STATUS0_STATUS_LATE_MASK      (0x20U)
#define CCM_OSCPLL_STATUS0_STATUS_LATE_SHIFT     (5U)
/*! STATUS_LATE - Clock source ready
 *  0b0..Clock source is not ready to use
 *  0b1..Clock source is ready to use
 */
#define CCM_OSCPLL_STATUS0_STATUS_LATE(x)        (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_STATUS0_STATUS_LATE_SHIFT)) & CCM_OSCPLL_STATUS0_STATUS_LATE_MASK)

#define CCM_OSCPLL_STATUS0_IN_USE_MASK           (0x1000U)
#define CCM_OSCPLL_STATUS0_IN_USE_SHIFT          (12U)
/*! IN_USE - This Clock Source is being used or not.
 *  0b0..Clock Source is not being used.
 *  0b1..Clock Source is being used.
 */
#define CCM_OSCPLL_STATUS0_IN_USE(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_STATUS0_IN_USE_SHIFT)) & CCM_OSCPLL_STATUS0_IN_USE_MASK)
/*! @} */

/* The count of CCM_OSCPLL_STATUS0 */
#define CCM_OSCPLL_STATUS0_COUNT                 (25U)

/*! @name OSCPLL_STATUS1 - Clock source domain status */
/*! @{ */

#define CCM_OSCPLL_STATUS1_DOMAIN_ACTIVE_MASK    (0xFFFFU)
#define CCM_OSCPLL_STATUS1_DOMAIN_ACTIVE_SHIFT   (0U)
/*! DOMAIN_ACTIVE - Domain active */
#define CCM_OSCPLL_STATUS1_DOMAIN_ACTIVE(x)      (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_STATUS1_DOMAIN_ACTIVE_SHIFT)) & CCM_OSCPLL_STATUS1_DOMAIN_ACTIVE_MASK)

#define CCM_OSCPLL_STATUS1_DOMAIN_ENABLE_MASK    (0xFFFF0000U)
#define CCM_OSCPLL_STATUS1_DOMAIN_ENABLE_SHIFT   (16U)
/*! DOMAIN_ENABLE - Domain enable */
#define CCM_OSCPLL_STATUS1_DOMAIN_ENABLE(x)      (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_STATUS1_DOMAIN_ENABLE_SHIFT)) & CCM_OSCPLL_STATUS1_DOMAIN_ENABLE_MASK)
/*! @} */

/* The count of CCM_OSCPLL_STATUS1 */
#define CCM_OSCPLL_STATUS1_COUNT                 (25U)

/*! @name OSCPLL_AUTHEN - Clock Source access control */
/*! @{ */

#define CCM_OSCPLL_AUTHEN_CPULPM_MODE_MASK       (0x4U)
#define CCM_OSCPLL_AUTHEN_CPULPM_MODE_SHIFT      (2U)
/*! CPULPM_MODE - CPULPM mode enable
 *  0b0..Disable CPULPM mode.
 *  0b1..Enable CPULPM mode.
 */
#define CCM_OSCPLL_AUTHEN_CPULPM_MODE(x)         (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_AUTHEN_CPULPM_MODE_SHIFT)) & CCM_OSCPLL_AUTHEN_CPULPM_MODE_MASK)

#define CCM_OSCPLL_AUTHEN_AUTO_CTRL_MASK         (0x8U)
#define CCM_OSCPLL_AUTHEN_AUTO_CTRL_SHIFT        (3U)
/*! AUTO_CTRL - Auto mode enable
 *  0b0..Disable Auto mode
 *  0b1..Enable Auto mode
 */
#define CCM_OSCPLL_AUTHEN_AUTO_CTRL(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_AUTHEN_AUTO_CTRL_SHIFT)) & CCM_OSCPLL_AUTHEN_AUTO_CTRL_MASK)

#define CCM_OSCPLL_AUTHEN_TZ_USER_MASK           (0x100U)
#define CCM_OSCPLL_AUTHEN_TZ_USER_SHIFT          (8U)
/*! TZ_USER - User access permission
 *  0b1..Clock Source settings can be changed in user mode.
 *  0b0..Clock Source settings cannot be changed in user mode.
 */
#define CCM_OSCPLL_AUTHEN_TZ_USER(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_AUTHEN_TZ_USER_SHIFT)) & CCM_OSCPLL_AUTHEN_TZ_USER_MASK)

#define CCM_OSCPLL_AUTHEN_TZ_NS_MASK             (0x200U)
#define CCM_OSCPLL_AUTHEN_TZ_NS_SHIFT            (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_OSCPLL_AUTHEN_TZ_NS(x)               (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_AUTHEN_TZ_NS_SHIFT)) & CCM_OSCPLL_AUTHEN_TZ_NS_MASK)

#define CCM_OSCPLL_AUTHEN_LOCK_TZ_MASK           (0x800U)
#define CCM_OSCPLL_AUTHEN_LOCK_TZ_SHIFT          (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_OSCPLL_AUTHEN_LOCK_TZ(x)             (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_AUTHEN_LOCK_TZ_SHIFT)) & CCM_OSCPLL_AUTHEN_LOCK_TZ_MASK)

#define CCM_OSCPLL_AUTHEN_LOCK_LIST_MASK         (0x8000U)
#define CCM_OSCPLL_AUTHEN_LOCK_LIST_SHIFT        (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_OSCPLL_AUTHEN_LOCK_LIST(x)           (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_AUTHEN_LOCK_LIST_SHIFT)) & CCM_OSCPLL_AUTHEN_LOCK_LIST_MASK)

#define CCM_OSCPLL_AUTHEN_WHITE_LIST_MASK        (0xFFFF0000U)
#define CCM_OSCPLL_AUTHEN_WHITE_LIST_SHIFT       (16U)
/*! WHITE_LIST - Whitelist */
#define CCM_OSCPLL_AUTHEN_WHITE_LIST(x)          (((uint32_t)(((uint32_t)(x)) << CCM_OSCPLL_AUTHEN_WHITE_LIST_SHIFT)) & CCM_OSCPLL_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_OSCPLL_AUTHEN */
#define CCM_OSCPLL_AUTHEN_COUNT                  (25U)

/*! @name LPCG_DIRECT - LPCG direct control */
/*! @{ */

#define CCM_LPCG_DIRECT_ON_MASK                  (0x1U)
#define CCM_LPCG_DIRECT_ON_SHIFT                 (0U)
/*! ON - Turn on LPCG
 *  0b0..LPCG is OFF.
 *  0b1..LPCG is ON.
 */
#define CCM_LPCG_DIRECT_ON(x)                    (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_DIRECT_ON_SHIFT)) & CCM_LPCG_DIRECT_ON_MASK)

#define CCM_LPCG_DIRECT_CLKOFF_ACK_TIMEOUT_EN_MASK (0x4U)
#define CCM_LPCG_DIRECT_CLKOFF_ACK_TIMEOUT_EN_SHIFT (2U)
/*! CLKOFF_ACK_TIMEOUT_EN - Clock off handshake timeout enable
 *  0b0..disable
 *  0b1..enable
 */
#define CCM_LPCG_DIRECT_CLKOFF_ACK_TIMEOUT_EN(x) (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_DIRECT_CLKOFF_ACK_TIMEOUT_EN_SHIFT)) & CCM_LPCG_DIRECT_CLKOFF_ACK_TIMEOUT_EN_MASK)
/*! @} */

/* The count of CCM_LPCG_DIRECT */
#define CCM_LPCG_DIRECT_COUNT                    (149U)

/*! @name LPCG_LPM0 - Clock source low power mode setting */
/*! @{ */

#define CCM_LPCG_LPM0_LPM_SETTING_D0_MASK        (0x7U)
#define CCM_LPCG_LPM0_LPM_SETTING_D0_SHIFT       (0U)
/*! LPM_SETTING_D0 - Clock Source LPM in DOMAIN0
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D0(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D0_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D0_MASK)

#define CCM_LPCG_LPM0_LPM_SETTING_D1_MASK        (0x70U)
#define CCM_LPCG_LPM0_LPM_SETTING_D1_SHIFT       (4U)
/*! LPM_SETTING_D1 - Clock Source LPM in DOMAIN1
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D1(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D1_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D1_MASK)

#define CCM_LPCG_LPM0_LPM_SETTING_D2_MASK        (0x700U)
#define CCM_LPCG_LPM0_LPM_SETTING_D2_SHIFT       (8U)
/*! LPM_SETTING_D2 - Clock Source LPM in DOMAIN2
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D2(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D2_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D2_MASK)

#define CCM_LPCG_LPM0_LPM_SETTING_D3_MASK        (0x7000U)
#define CCM_LPCG_LPM0_LPM_SETTING_D3_SHIFT       (12U)
/*! LPM_SETTING_D3 - Clock Source LPM in DOMAIN3
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D3(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D3_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D3_MASK)

#define CCM_LPCG_LPM0_LPM_SETTING_D4_MASK        (0x70000U)
#define CCM_LPCG_LPM0_LPM_SETTING_D4_SHIFT       (16U)
/*! LPM_SETTING_D4 - Clock Source LPM in DOMAIN4
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D4(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D4_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D4_MASK)

#define CCM_LPCG_LPM0_LPM_SETTING_D5_MASK        (0x700000U)
#define CCM_LPCG_LPM0_LPM_SETTING_D5_SHIFT       (20U)
/*! LPM_SETTING_D5 - Clock Source LPM in DOMAIN5
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D5(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D5_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D5_MASK)

#define CCM_LPCG_LPM0_LPM_SETTING_D6_MASK        (0x7000000U)
#define CCM_LPCG_LPM0_LPM_SETTING_D6_SHIFT       (24U)
/*! LPM_SETTING_D6 - Clock Source LPM in DOMAIN6
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D6(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D6_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D6_MASK)

#define CCM_LPCG_LPM0_LPM_SETTING_D7_MASK        (0x70000000U)
#define CCM_LPCG_LPM0_LPM_SETTING_D7_SHIFT       (28U)
/*! LPM_SETTING_D7 - Clock Source LPM in DOMAIN7
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM0_LPM_SETTING_D7(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM0_LPM_SETTING_D7_SHIFT)) & CCM_LPCG_LPM0_LPM_SETTING_D7_MASK)
/*! @} */

/* The count of CCM_LPCG_LPM0 */
#define CCM_LPCG_LPM0_COUNT                      (149U)

/*! @name LPCG_LPM1 - clock source low power mode setting */
/*! @{ */

#define CCM_LPCG_LPM1_LPM_SETTING_D8_MASK        (0x7U)
#define CCM_LPCG_LPM1_LPM_SETTING_D8_SHIFT       (0U)
/*! LPM_SETTING_D8 - Clock Source LPM in DOMAIN8
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D8(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D8_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D8_MASK)

#define CCM_LPCG_LPM1_LPM_SETTING_D9_MASK        (0x70U)
#define CCM_LPCG_LPM1_LPM_SETTING_D9_SHIFT       (4U)
/*! LPM_SETTING_D9 - Clock Source LPM in DOMAIN9
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D9(x)          (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D9_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D9_MASK)

#define CCM_LPCG_LPM1_LPM_SETTING_D10_MASK       (0x700U)
#define CCM_LPCG_LPM1_LPM_SETTING_D10_SHIFT      (8U)
/*! LPM_SETTING_D10 - Clock Source LPM in DOMAIN10
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D10(x)         (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D10_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D10_MASK)

#define CCM_LPCG_LPM1_LPM_SETTING_D11_MASK       (0x7000U)
#define CCM_LPCG_LPM1_LPM_SETTING_D11_SHIFT      (12U)
/*! LPM_SETTING_D11 - Clock Source LPM in DOMAIN11
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D11(x)         (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D11_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D11_MASK)

#define CCM_LPCG_LPM1_LPM_SETTING_D12_MASK       (0x70000U)
#define CCM_LPCG_LPM1_LPM_SETTING_D12_SHIFT      (16U)
/*! LPM_SETTING_D12 - Clock Source LPM in DOMAIN12
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D12(x)         (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D12_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D12_MASK)

#define CCM_LPCG_LPM1_LPM_SETTING_D13_MASK       (0x700000U)
#define CCM_LPCG_LPM1_LPM_SETTING_D13_SHIFT      (20U)
/*! LPM_SETTING_D13 - Clock Source LPM in DOMAIN13
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D13(x)         (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D13_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D13_MASK)

#define CCM_LPCG_LPM1_LPM_SETTING_D14_MASK       (0x7000000U)
#define CCM_LPCG_LPM1_LPM_SETTING_D14_SHIFT      (24U)
/*! LPM_SETTING_D14 - Clock Source LPM in DOMAIN14
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D14(x)         (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D14_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D14_MASK)

#define CCM_LPCG_LPM1_LPM_SETTING_D15_MASK       (0x70000000U)
#define CCM_LPCG_LPM1_LPM_SETTING_D15_SHIFT      (28U)
/*! LPM_SETTING_D15 - Clock Source LPM in DOMAIN15
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM1_LPM_SETTING_D15(x)         (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM1_LPM_SETTING_D15_SHIFT)) & CCM_LPCG_LPM1_LPM_SETTING_D15_MASK)
/*! @} */

/* The count of CCM_LPCG_LPM1 */
#define CCM_LPCG_LPM1_COUNT                      (149U)

/*! @name LPCG_LPM_CUR - LPM setting of current CPU domain */
/*! @{ */

#define CCM_LPCG_LPM_CUR_LPM_SETTING_CUR_MASK    (0x7U)
#define CCM_LPCG_LPM_CUR_LPM_SETTING_CUR_SHIFT   (0U)
/*! LPM_SETTING_CUR - LPM settings value for current CPU domain that is reading this register.
 *  0b000..Clock Source will be OFF in any CPU mode.
 *  0b001..Clock Source will be ON in RUN mode, OFF in WAIT/STOP/SUSPEND mode.
 *  0b010..Clock Source will be ON in RUN/WAIT mode, OFF in STOP/SUSPEND mode.
 *  0b011..Clock Source will be ON in RUN/WAIT/STOP mode, OFF in SUSPEND mode.
 */
#define CCM_LPCG_LPM_CUR_LPM_SETTING_CUR(x)      (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_LPM_CUR_LPM_SETTING_CUR_SHIFT)) & CCM_LPCG_LPM_CUR_LPM_SETTING_CUR_MASK)
/*! @} */

/* The count of CCM_LPCG_LPM_CUR */
#define CCM_LPCG_LPM_CUR_COUNT                   (149U)

/*! @name LPCG_STATUS0 - LPCG working status */
/*! @{ */

#define CCM_LPCG_STATUS0_ON_MASK                 (0x1U)
#define CCM_LPCG_STATUS0_ON_SHIFT                (0U)
/*! ON - LPCG work status
 *  0b0..LPCG is OFF.
 *  0b1..LPCG is ON.
 */
#define CCM_LPCG_STATUS0_ON(x)                   (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_STATUS0_ON_SHIFT)) & CCM_LPCG_STATUS0_ON_MASK)
/*! @} */

/* The count of CCM_LPCG_STATUS0 */
#define CCM_LPCG_STATUS0_COUNT                   (149U)

/*! @name LPCG_STATUS1 - LPCG domain status */
/*! @{ */

#define CCM_LPCG_STATUS1_DOMAIN_ACTIVE_MASK      (0xFFFFU)
#define CCM_LPCG_STATUS1_DOMAIN_ACTIVE_SHIFT     (0U)
/*! DOMAIN_ACTIVE - Domain active */
#define CCM_LPCG_STATUS1_DOMAIN_ACTIVE(x)        (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_STATUS1_DOMAIN_ACTIVE_SHIFT)) & CCM_LPCG_STATUS1_DOMAIN_ACTIVE_MASK)

#define CCM_LPCG_STATUS1_DOMAIN_ENABLE_MASK      (0xFFFF0000U)
#define CCM_LPCG_STATUS1_DOMAIN_ENABLE_SHIFT     (16U)
/*! DOMAIN_ENABLE - Domain enable */
#define CCM_LPCG_STATUS1_DOMAIN_ENABLE(x)        (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_STATUS1_DOMAIN_ENABLE_SHIFT)) & CCM_LPCG_STATUS1_DOMAIN_ENABLE_MASK)
/*! @} */

/* The count of CCM_LPCG_STATUS1 */
#define CCM_LPCG_STATUS1_COUNT                   (149U)

/*! @name LPCG_AUTHEN - LPCG access control */
/*! @{ */

#define CCM_LPCG_AUTHEN_CPULPM_MODE_MASK         (0x4U)
#define CCM_LPCG_AUTHEN_CPULPM_MODE_SHIFT        (2U)
/*! CPULPM_MODE - CPULPM mode enable
 *  0b0..Disable CPULPM mode, this LPCG is in Direct Control mode.
 *  0b1..Enable CPULPM mode, this LPCG is in CPULPM mode.
 */
#define CCM_LPCG_AUTHEN_CPULPM_MODE(x)           (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_AUTHEN_CPULPM_MODE_SHIFT)) & CCM_LPCG_AUTHEN_CPULPM_MODE_MASK)

#define CCM_LPCG_AUTHEN_ACK_MODE_MASK            (0x10U)
#define CCM_LPCG_AUTHEN_ACK_MODE_SHIFT           (4U)
/*! ACK_MODE - CPULPM mode enable
 *  0b0..Disable ACK mode.
 *  0b1..Enable ACK mode.
 */
#define CCM_LPCG_AUTHEN_ACK_MODE(x)              (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_AUTHEN_ACK_MODE_SHIFT)) & CCM_LPCG_AUTHEN_ACK_MODE_MASK)

#define CCM_LPCG_AUTHEN_TZ_USER_MASK             (0x100U)
#define CCM_LPCG_AUTHEN_TZ_USER_SHIFT            (8U)
/*! TZ_USER - User access permission
 *  0b1..LPCG settings can be changed in user mode.
 *  0b0..LPCG settings cannot be changed in user mode.
 */
#define CCM_LPCG_AUTHEN_TZ_USER(x)               (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_AUTHEN_TZ_USER_SHIFT)) & CCM_LPCG_AUTHEN_TZ_USER_MASK)

#define CCM_LPCG_AUTHEN_TZ_NS_MASK               (0x200U)
#define CCM_LPCG_AUTHEN_TZ_NS_SHIFT              (9U)
/*! TZ_NS - Non-secure access permission
 *  0b0..Cannot be changed in Non-secure mode.
 *  0b1..Can be changed in Non-secure mode.
 */
#define CCM_LPCG_AUTHEN_TZ_NS(x)                 (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_AUTHEN_TZ_NS_SHIFT)) & CCM_LPCG_AUTHEN_TZ_NS_MASK)

#define CCM_LPCG_AUTHEN_LOCK_TZ_MASK             (0x800U)
#define CCM_LPCG_AUTHEN_LOCK_TZ_SHIFT            (11U)
/*! LOCK_TZ - Lock TrustZone settings
 *  0b0..TrustZone settings is not locked.
 *  0b1..TrustZone settings is locked.
 */
#define CCM_LPCG_AUTHEN_LOCK_TZ(x)               (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_AUTHEN_LOCK_TZ_SHIFT)) & CCM_LPCG_AUTHEN_LOCK_TZ_MASK)

#define CCM_LPCG_AUTHEN_LOCK_LIST_MASK           (0x8000U)
#define CCM_LPCG_AUTHEN_LOCK_LIST_SHIFT          (15U)
/*! LOCK_LIST - Lock white list
 *  0b0..Whitelist is not locked.
 *  0b1..Whitelist is locked.
 */
#define CCM_LPCG_AUTHEN_LOCK_LIST(x)             (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_AUTHEN_LOCK_LIST_SHIFT)) & CCM_LPCG_AUTHEN_LOCK_LIST_MASK)

#define CCM_LPCG_AUTHEN_WHITE_LIST_MASK          (0xFFFF0000U)
#define CCM_LPCG_AUTHEN_WHITE_LIST_SHIFT         (16U)
/*! WHITE_LIST - Whitelist */
#define CCM_LPCG_AUTHEN_WHITE_LIST(x)            (((uint32_t)(((uint32_t)(x)) << CCM_LPCG_AUTHEN_WHITE_LIST_SHIFT)) & CCM_LPCG_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/* The count of CCM_LPCG_AUTHEN */
#define CCM_LPCG_AUTHEN_COUNT                    (149U)


/*!
 * @}
 */ /* end of group CCM_Register_Masks */


/* CCM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral CCM base address */
  #define CCM_BASE                                 (0x54450000u)
  /** Peripheral CCM base address */
  #define CCM_BASE_NS                              (0x44450000u)
  /** Peripheral CCM base pointer */
  #define CCM                                      ((CCM_Type *)CCM_BASE)
  /** Peripheral CCM base pointer */
  #define CCM_NS                                   ((CCM_Type *)CCM_BASE_NS)
  /** Array initializer of CCM peripheral base addresses */
  #define CCM_BASE_ADDRS                           { CCM_BASE }
  /** Array initializer of CCM peripheral base pointers */
  #define CCM_BASE_PTRS                            { CCM }
  /** Array initializer of CCM peripheral base addresses */
  #define CCM_BASE_ADDRS_NS                        { CCM_BASE_NS }
  /** Array initializer of CCM peripheral base pointers */
  #define CCM_BASE_PTRS_NS                         { CCM_NS }
#else
  /** Peripheral CCM base address */
  #define CCM_BASE                                 (0x44450000u)
  /** Peripheral CCM base pointer */
  #define CCM                                      ((CCM_Type *)CCM_BASE)
  /** Array initializer of CCM peripheral base addresses */
  #define CCM_BASE_ADDRS                           { CCM_BASE }
  /** Array initializer of CCM peripheral base pointers */
  #define CCM_BASE_PTRS                            { CCM }
#endif

/*!
 * @}
 */ /* end of group CCM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- CMP Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CMP_Peripheral_Access_Layer CMP Peripheral Access Layer
 * @{
 */

/** CMP - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID Register, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter Register, offset: 0x4 */
  __IO uint32_t C0;                                /**< CMP Control 0, offset: 0x8 */
  __IO uint32_t C1;                                /**< CMP Control 1, offset: 0xC */
  __IO uint32_t C2;                                /**< CMP Control 2, offset: 0x10 */
  __IO uint32_t C3;                                /**< CMP Control 3, offset: 0x14 */
} CMP_Type;

/* ----------------------------------------------------------------------------
   -- CMP Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup CMP_Register_Masks CMP Register Masks
 * @{
 */

/*! @name VERID - Version ID Register */
/*! @{ */

#define CMP_VERID_FEATURE_MASK                   (0xFFFFU)
#define CMP_VERID_FEATURE_SHIFT                  (0U)
/*! FEATURE - Feature Specification Number */
#define CMP_VERID_FEATURE(x)                     (((uint32_t)(((uint32_t)(x)) << CMP_VERID_FEATURE_SHIFT)) & CMP_VERID_FEATURE_MASK)

#define CMP_VERID_MINOR_MASK                     (0xFF0000U)
#define CMP_VERID_MINOR_SHIFT                    (16U)
/*! MINOR - Minor Version Number */
#define CMP_VERID_MINOR(x)                       (((uint32_t)(((uint32_t)(x)) << CMP_VERID_MINOR_SHIFT)) & CMP_VERID_MINOR_MASK)

#define CMP_VERID_MAJOR_MASK                     (0xFF000000U)
#define CMP_VERID_MAJOR_SHIFT                    (24U)
/*! MAJOR - Major Version Number */
#define CMP_VERID_MAJOR(x)                       (((uint32_t)(((uint32_t)(x)) << CMP_VERID_MAJOR_SHIFT)) & CMP_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter Register */
/*! @{ */

#define CMP_PARAM_PARAM_MASK                     (0xFFFFFFFFU)
#define CMP_PARAM_PARAM_SHIFT                    (0U)
/*! PARAM - Parameters */
#define CMP_PARAM_PARAM(x)                       (((uint32_t)(((uint32_t)(x)) << CMP_PARAM_PARAM_SHIFT)) & CMP_PARAM_PARAM_MASK)
/*! @} */

/*! @name C0 - CMP Control 0 */
/*! @{ */

#define CMP_C0_FILTER_CNT_MASK                   (0x70U)
#define CMP_C0_FILTER_CNT_SHIFT                  (4U)
/*! FILTER_CNT - Filter Sample Count
 *  0b000..Filter is disabled (if C0[SE] = 1, then COUT is a logic zero (this is not a legal state, and is not
 *         recommended); if C0[SE] = 0, COUT = COUTA)
 *  0b001..One consecutive sample (comparator output is simply sampled)
 *  0b010..Two consecutive samples
 *  0b011..Three consecutive samples
 *  0b100..Four consecutive samples
 *  0b101..Five consecutive samples
 *  0b110..Six consecutive samples
 *  0b111..Seven consecutive samples
 */
#define CMP_C0_FILTER_CNT(x)                     (((uint32_t)(((uint32_t)(x)) << CMP_C0_FILTER_CNT_SHIFT)) & CMP_C0_FILTER_CNT_MASK)

#define CMP_C0_EN_MASK                           (0x100U)
#define CMP_C0_EN_SHIFT                          (8U)
/*! EN - Analog Comparator Module Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_EN(x)                             (((uint32_t)(((uint32_t)(x)) << CMP_C0_EN_SHIFT)) & CMP_C0_EN_MASK)

#define CMP_C0_OPE_MASK                          (0x200U)
#define CMP_C0_OPE_SHIFT                         (9U)
/*! OPE - Comparator Output Pin Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_OPE(x)                            (((uint32_t)(((uint32_t)(x)) << CMP_C0_OPE_SHIFT)) & CMP_C0_OPE_MASK)

#define CMP_C0_COS_MASK                          (0x400U)
#define CMP_C0_COS_SHIFT                         (10U)
/*! COS - Comparator Output Select
 *  0b0..COUT
 *  0b1..COUTA
 */
#define CMP_C0_COS(x)                            (((uint32_t)(((uint32_t)(x)) << CMP_C0_COS_SHIFT)) & CMP_C0_COS_MASK)

#define CMP_C0_INVT_MASK                         (0x800U)
#define CMP_C0_INVT_SHIFT                        (11U)
/*! INVT - Comparator Invert
 *  0b0..Do not invert
 *  0b1..Invert
 */
#define CMP_C0_INVT(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C0_INVT_SHIFT)) & CMP_C0_INVT_MASK)

#define CMP_C0_PMODE_MASK                        (0x1000U)
#define CMP_C0_PMODE_SHIFT                       (12U)
/*! PMODE - Power Mode Select
 *  0b0..Low-speed (LS)
 *  0b1..High-speed (HS)
 */
#define CMP_C0_PMODE(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C0_PMODE_SHIFT)) & CMP_C0_PMODE_MASK)

#define CMP_C0_WE_MASK                           (0x4000U)
#define CMP_C0_WE_SHIFT                          (14U)
/*! WE - Windowing Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_WE(x)                             (((uint32_t)(((uint32_t)(x)) << CMP_C0_WE_SHIFT)) & CMP_C0_WE_MASK)

#define CMP_C0_SE_MASK                           (0x8000U)
#define CMP_C0_SE_SHIFT                          (15U)
/*! SE - Sample Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_SE(x)                             (((uint32_t)(((uint32_t)(x)) << CMP_C0_SE_SHIFT)) & CMP_C0_SE_MASK)

#define CMP_C0_FPR_MASK                          (0xFF0000U)
#define CMP_C0_FPR_SHIFT                         (16U)
/*! FPR - Filter Sample Period */
#define CMP_C0_FPR(x)                            (((uint32_t)(((uint32_t)(x)) << CMP_C0_FPR_SHIFT)) & CMP_C0_FPR_MASK)

#define CMP_C0_COUT_MASK                         (0x1000000U)
#define CMP_C0_COUT_SHIFT                        (24U)
/*! COUT - Analog Comparator Output */
#define CMP_C0_COUT(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C0_COUT_SHIFT)) & CMP_C0_COUT_MASK)

#define CMP_C0_CFF_MASK                          (0x2000000U)
#define CMP_C0_CFF_SHIFT                         (25U)
/*! CFF - Analog Comparator Flag Falling
 *  0b0..Not detected
 *  0b1..Detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CMP_C0_CFF(x)                            (((uint32_t)(((uint32_t)(x)) << CMP_C0_CFF_SHIFT)) & CMP_C0_CFF_MASK)

#define CMP_C0_CFR_MASK                          (0x4000000U)
#define CMP_C0_CFR_SHIFT                         (26U)
/*! CFR - Analog Comparator Flag Rising
 *  0b0..Not detected
 *  0b1..Detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define CMP_C0_CFR(x)                            (((uint32_t)(((uint32_t)(x)) << CMP_C0_CFR_SHIFT)) & CMP_C0_CFR_MASK)

#define CMP_C0_IEF_MASK                          (0x8000000U)
#define CMP_C0_IEF_SHIFT                         (27U)
/*! IEF - Comparator Interrupt Enable Falling
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_IEF(x)                            (((uint32_t)(((uint32_t)(x)) << CMP_C0_IEF_SHIFT)) & CMP_C0_IEF_MASK)

#define CMP_C0_IER_MASK                          (0x10000000U)
#define CMP_C0_IER_SHIFT                         (28U)
/*! IER - Comparator Interrupt Enable Rising
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_IER(x)                            (((uint32_t)(((uint32_t)(x)) << CMP_C0_IER_SHIFT)) & CMP_C0_IER_MASK)

#define CMP_C0_DMAEN_MASK                        (0x40000000U)
#define CMP_C0_DMAEN_SHIFT                       (30U)
/*! DMAEN - DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_DMAEN(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C0_DMAEN_SHIFT)) & CMP_C0_DMAEN_MASK)

#define CMP_C0_LINKEN_MASK                       (0x80000000U)
#define CMP_C0_LINKEN_SHIFT                      (31U)
/*! LINKEN - CMP to DAC Link Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C0_LINKEN(x)                         (((uint32_t)(((uint32_t)(x)) << CMP_C0_LINKEN_SHIFT)) & CMP_C0_LINKEN_MASK)
/*! @} */

/*! @name C1 - CMP Control 1 */
/*! @{ */

#define CMP_C1_VOSEL_MASK                        (0xFFU)
#define CMP_C1_VOSEL_SHIFT                       (0U)
/*! VOSEL - DAC Output Voltage Select */
#define CMP_C1_VOSEL(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C1_VOSEL_SHIFT)) & CMP_C1_VOSEL_MASK)

#define CMP_C1_DMODE_MASK                        (0x100U)
#define CMP_C1_DMODE_SHIFT                       (8U)
/*! DMODE - DAC Mode Select
 *  0b0..Low-Speed and Low-Power mode
 *  0b1..High-Speed and High-Power mode
 */
#define CMP_C1_DMODE(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C1_DMODE_SHIFT)) & CMP_C1_DMODE_MASK)

#define CMP_C1_VRSEL_MASK                        (0x200U)
#define CMP_C1_VRSEL_SHIFT                       (9U)
/*! VRSEL - Supply Voltage Reference Source Select
 *  0b0..Vin1
 *  0b1..Vin2
 */
#define CMP_C1_VRSEL(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C1_VRSEL_SHIFT)) & CMP_C1_VRSEL_MASK)

#define CMP_C1_DACEN_MASK                        (0x400U)
#define CMP_C1_DACEN_SHIFT                       (10U)
/*! DACEN - DAC Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C1_DACEN(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C1_DACEN_SHIFT)) & CMP_C1_DACEN_MASK)

#define CMP_C1_CHN0_MASK                         (0x10000U)
#define CMP_C1_CHN0_SHIFT                        (16U)
/*! CHN0 - Channel 0 Input Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C1_CHN0(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_CHN0_SHIFT)) & CMP_C1_CHN0_MASK)

#define CMP_C1_CHN1_MASK                         (0x20000U)
#define CMP_C1_CHN1_SHIFT                        (17U)
/*! CHN1 - Channel 1 Input Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C1_CHN1(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_CHN1_SHIFT)) & CMP_C1_CHN1_MASK)

#define CMP_C1_CHN2_MASK                         (0x40000U)
#define CMP_C1_CHN2_SHIFT                        (18U)
/*! CHN2 - Channel 2 Input Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C1_CHN2(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_CHN2_SHIFT)) & CMP_C1_CHN2_MASK)

#define CMP_C1_CHN3_MASK                         (0x80000U)
#define CMP_C1_CHN3_SHIFT                        (19U)
/*! CHN3 - Channel 3 Input Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C1_CHN3(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_CHN3_SHIFT)) & CMP_C1_CHN3_MASK)

#define CMP_C1_CHN4_MASK                         (0x100000U)
#define CMP_C1_CHN4_SHIFT                        (20U)
/*! CHN4 - Channel 4 Input Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C1_CHN4(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_CHN4_SHIFT)) & CMP_C1_CHN4_MASK)

#define CMP_C1_CHN5_MASK                         (0x200000U)
#define CMP_C1_CHN5_SHIFT                        (21U)
/*! CHN5 - Channel 5 Input Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C1_CHN5(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_CHN5_SHIFT)) & CMP_C1_CHN5_MASK)

#define CMP_C1_MSEL_MASK                         (0x7000000U)
#define CMP_C1_MSEL_SHIFT                        (24U)
/*! MSEL - Minus Input MUX Control
 *  0b000..Internal negative input 0 for minus channel (internal minus input)
 *  0b001..External input 1 for minus channel (reference input 0)
 *  0b010..External input 2 for minus channel (reference input 1)
 *  0b011..External input 3 for minus channel (reference input 2)
 *  0b100..External input 4 for minus channel (reference input 3)
 *  0b101..Reserved
 *  0b110..Reserved
 *  0b111..Internal 8-bit DAC output
 */
#define CMP_C1_MSEL(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_MSEL_SHIFT)) & CMP_C1_MSEL_MASK)

#define CMP_C1_PSEL_MASK                         (0x70000000U)
#define CMP_C1_PSEL_SHIFT                        (28U)
/*! PSEL - Plus Input MUX Control
 *  0b000..Internal positive input 0 for plus channel (internal plus input)
 *  0b001..External input 1 for plus Channel (reference input 0)
 *  0b010..External input 2 for plus channel (reference input 1)
 *  0b011..External input 3 for plus channel (reference input 2)
 *  0b100..External input 4 for plus channel (reference input 3)
 *  0b101..Reserved
 *  0b110..Reserved
 *  0b111..Internal 8-bit DAC output
 */
#define CMP_C1_PSEL(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C1_PSEL_SHIFT)) & CMP_C1_PSEL_MASK)
/*! @} */

/*! @name C2 - CMP Control 2 */
/*! @{ */

#define CMP_C2_ACOn_MASK                         (0x3FU)
#define CMP_C2_ACOn_SHIFT                        (0U)
/*! ACOn - ACOn */
#define CMP_C2_ACOn(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_ACOn_SHIFT)) & CMP_C2_ACOn_MASK)

#define CMP_C2_INITMOD_MASK                      (0x3F00U)
#define CMP_C2_INITMOD_SHIFT                     (8U)
/*! INITMOD - Comparator and DAC Initialization Delay Modulus */
#define CMP_C2_INITMOD(x)                        (((uint32_t)(((uint32_t)(x)) << CMP_C2_INITMOD_SHIFT)) & CMP_C2_INITMOD_MASK)

#define CMP_C2_NSAM_MASK                         (0xC000U)
#define CMP_C2_NSAM_SHIFT                        (14U)
/*! NSAM - Number of Sample Clocks
 *  0b00..As soon as the active channel is scanned in one round-robin clock
 *  0b01..After one round-robin clock cycle
 *  0b10..After two round-robin clock cycles
 *  0b11..After three round-robin clock cycles
 */
#define CMP_C2_NSAM(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_NSAM_SHIFT)) & CMP_C2_NSAM_MASK)

#define CMP_C2_CH0F_MASK                         (0x10000U)
#define CMP_C2_CH0F_SHIFT                        (16U)
/*! CH0F - External Channel 0 Input Changed Flag */
#define CMP_C2_CH0F(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_CH0F_SHIFT)) & CMP_C2_CH0F_MASK)

#define CMP_C2_CH1F_MASK                         (0x20000U)
#define CMP_C2_CH1F_SHIFT                        (17U)
/*! CH1F - External Channel 1 Input Changed Flag */
#define CMP_C2_CH1F(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_CH1F_SHIFT)) & CMP_C2_CH1F_MASK)

#define CMP_C2_CH2F_MASK                         (0x40000U)
#define CMP_C2_CH2F_SHIFT                        (18U)
/*! CH2F - External Channel 2 Input Changed Flag */
#define CMP_C2_CH2F(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_CH2F_SHIFT)) & CMP_C2_CH2F_MASK)

#define CMP_C2_CH3F_MASK                         (0x80000U)
#define CMP_C2_CH3F_SHIFT                        (19U)
/*! CH3F - External Channel 3 Input Changed Flag */
#define CMP_C2_CH3F(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_CH3F_SHIFT)) & CMP_C2_CH3F_MASK)

#define CMP_C2_CH4F_MASK                         (0x100000U)
#define CMP_C2_CH4F_SHIFT                        (20U)
/*! CH4F - External Channel 4 Input Changed Flag */
#define CMP_C2_CH4F(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_CH4F_SHIFT)) & CMP_C2_CH4F_MASK)

#define CMP_C2_CH5F_MASK                         (0x200000U)
#define CMP_C2_CH5F_SHIFT                        (21U)
/*! CH5F - External Channel 5 Input Changed Flag */
#define CMP_C2_CH5F(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_CH5F_SHIFT)) & CMP_C2_CH5F_MASK)

#define CMP_C2_FXMXCH_MASK                       (0xE000000U)
#define CMP_C2_FXMXCH_SHIFT                      (25U)
/*! FXMXCH - Fixed Channel Select
 *  0b000..External reference input 0
 *  0b001..External reference input 1
 *  0b010..External reference input 2
 *  0b011..External reference input 3
 *  0b100..External reference input 4
 *  0b101..External reference input 5
 *  0b110..Reserved
 *  0b111..8-bit DAC
 */
#define CMP_C2_FXMXCH(x)                         (((uint32_t)(((uint32_t)(x)) << CMP_C2_FXMXCH_SHIFT)) & CMP_C2_FXMXCH_MASK)

#define CMP_C2_FXMP_MASK                         (0x20000000U)
#define CMP_C2_FXMP_SHIFT                        (29U)
/*! FXMP - Fixed MUX Port
 *  0b0..Fix plus port
 *  0b1..Fix minus port
 */
#define CMP_C2_FXMP(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_FXMP_SHIFT)) & CMP_C2_FXMP_MASK)

#define CMP_C2_RRIE_MASK                         (0x40000000U)
#define CMP_C2_RRIE_SHIFT                        (30U)
/*! RRIE - Round-Robin Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define CMP_C2_RRIE(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C2_RRIE_SHIFT)) & CMP_C2_RRIE_MASK)
/*! @} */

/*! @name C3 - CMP Control 3 */
/*! @{ */

#define CMP_C3_ACPH2TC_MASK                      (0x70U)
#define CMP_C3_ACPH2TC_SHIFT                     (4U)
/*! ACPH2TC - Analog Comparator Phase 2 Timing Control
 *  0b000..Phase 2 active time in one sampling period equals to T
 *  0b001..Phase 2 active time in one sampling period equals to 2 * T
 *  0b010..Phase 2 active time in one sampling period equals to 4 * T
 *  0b011..Phase 2 active time in one sampling period equals to 8 * T
 *  0b100..Phase 2 active time in one sampling period equals to 16 * T
 *  0b101..Phase 2 active time in one sampling period equals to 32 * T
 *  0b110..Phase 2 active time in one sampling period equals to 64 * T
 *  0b111..Phase 2 active time in one sampling period equals to 16 * T
 */
#define CMP_C3_ACPH2TC(x)                        (((uint32_t)(((uint32_t)(x)) << CMP_C3_ACPH2TC_SHIFT)) & CMP_C3_ACPH2TC_MASK)

#define CMP_C3_ACPH1TC_MASK                      (0x700U)
#define CMP_C3_ACPH1TC_SHIFT                     (8U)
/*! ACPH1TC - Analog Comparator Phase 1 Timing Control
 *  0b000..Phase 1 active time in one sampling period equals to T
 *  0b001..Phase 1 active time in one sampling period equals to 2 * T
 *  0b010..Phase 1 active time in one sampling period equals to 4 * T
 *  0b011..Phase 1 active time in one sampling period equals to 8 * T
 *  0b100..Phase 1 active time in one sampling period equals to T
 *  0b101..Phase 1 active time in one sampling period equals to T
 *  0b110..Phase 1 active time in one sampling period equals to T
 *  0b111..Phase 1 active time in one sampling period equals to 0
 */
#define CMP_C3_ACPH1TC(x)                        (((uint32_t)(((uint32_t)(x)) << CMP_C3_ACPH1TC_SHIFT)) & CMP_C3_ACPH1TC_MASK)

#define CMP_C3_ACSAT_MASK                        (0x7000U)
#define CMP_C3_ACSAT_SHIFT                       (12U)
/*! ACSAT - Analog Comparator Sampling Time Control
 *  0b000..The sampling time equals to T
 *  0b001..The sampling time equals to 2 * T
 *  0b010..The sampling time equals to 4 * T
 *  0b011..The sampling time equals to 8 * T
 *  0b100..The sampling time equals to 16 * T
 *  0b101..The sampling time equals to 32 * T
 *  0b110..The sampling time equals to 64 * T
 *  0b111..The sampling time equals to 256 * T
 */
#define CMP_C3_ACSAT(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C3_ACSAT_SHIFT)) & CMP_C3_ACSAT_MASK)

#define CMP_C3_DMCS_MASK                         (0x10000U)
#define CMP_C3_DMCS_SHIFT                        (16U)
/*! DMCS - Discrete Mode Clock Select
 *  0b0..Slow clock is selected for the timing generation.
 *  0b1..Fast clock is selected for the timing generation.
 */
#define CMP_C3_DMCS(x)                           (((uint32_t)(((uint32_t)(x)) << CMP_C3_DMCS_SHIFT)) & CMP_C3_DMCS_MASK)

#define CMP_C3_RDIVE_MASK                        (0x100000U)
#define CMP_C3_RDIVE_SHIFT                       (20U)
/*! RDIVE - Resistor Divider Enable
 *  0b0..Disable even when either NCHEN or PCHEN is set to1 but the actual input is in the range of 0 to 1.8v.
 *  0b1..Enable because the inputs are above 1.8v.
 */
#define CMP_C3_RDIVE(x)                          (((uint32_t)(((uint32_t)(x)) << CMP_C3_RDIVE_SHIFT)) & CMP_C3_RDIVE_MASK)

#define CMP_C3_NCHCTEN_MASK                      (0x1000000U)
#define CMP_C3_NCHCTEN_SHIFT                     (24U)
/*! NCHCTEN - Negative Channel Continuous Mode Enable
 *  0b0..in Discrete Mode and special timing needs to be configured.
 *  0b1..in Continuous Mode and no special timing is required.
 */
#define CMP_C3_NCHCTEN(x)                        (((uint32_t)(((uint32_t)(x)) << CMP_C3_NCHCTEN_SHIFT)) & CMP_C3_NCHCTEN_MASK)

#define CMP_C3_PCHCTEN_MASK                      (0x10000000U)
#define CMP_C3_PCHCTEN_SHIFT                     (28U)
/*! PCHCTEN - Positive Channel Continuous Mode Enable
 *  0b0..in Discrete Mode and special timing needs to be configured
 *  0b1..in Continuous Mode and no special timing is required
 */
#define CMP_C3_PCHCTEN(x)                        (((uint32_t)(((uint32_t)(x)) << CMP_C3_PCHCTEN_SHIFT)) & CMP_C3_PCHCTEN_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group CMP_Register_Masks */


/* CMP - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral CMP1 base address */
  #define CMP1_BASE                                (0x52DC0000u)
  /** Peripheral CMP1 base address */
  #define CMP1_BASE_NS                             (0x42DC0000u)
  /** Peripheral CMP1 base pointer */
  #define CMP1                                     ((CMP_Type *)CMP1_BASE)
  /** Peripheral CMP1 base pointer */
  #define CMP1_NS                                  ((CMP_Type *)CMP1_BASE_NS)
  /** Peripheral CMP2 base address */
  #define CMP2_BASE                                (0x52DD0000u)
  /** Peripheral CMP2 base address */
  #define CMP2_BASE_NS                             (0x42DD0000u)
  /** Peripheral CMP2 base pointer */
  #define CMP2                                     ((CMP_Type *)CMP2_BASE)
  /** Peripheral CMP2 base pointer */
  #define CMP2_NS                                  ((CMP_Type *)CMP2_BASE_NS)
  /** Peripheral CMP3 base address */
  #define CMP3_BASE                                (0x52DE0000u)
  /** Peripheral CMP3 base address */
  #define CMP3_BASE_NS                             (0x42DE0000u)
  /** Peripheral CMP3 base pointer */
  #define CMP3                                     ((CMP_Type *)CMP3_BASE)
  /** Peripheral CMP3 base pointer */
  #define CMP3_NS                                  ((CMP_Type *)CMP3_BASE_NS)
  /** Peripheral CMP4 base address */
  #define CMP4_BASE                                (0x52DF0000u)
  /** Peripheral CMP4 base address */
  #define CMP4_BASE_NS                             (0x42DF0000u)
  /** Peripheral CMP4 base pointer */
  #define CMP4                                     ((CMP_Type *)CMP4_BASE)
  /** Peripheral CMP4 base pointer */
  #define CMP4_NS                                  ((CMP_Type *)CMP4_BASE_NS)
  /** Array initializer of CMP peripheral base addresses */
  #define CMP_BASE_ADDRS                           { 0u, CMP1_BASE, CMP2_BASE, CMP3_BASE, CMP4_BASE }
  /** Array initializer of CMP peripheral base pointers */
  #define CMP_BASE_PTRS                            { (CMP_Type *)0u, CMP1, CMP2, CMP3, CMP4 }
  /** Array initializer of CMP peripheral base addresses */
  #define CMP_BASE_ADDRS_NS                        { 0u, CMP1_BASE_NS, CMP2_BASE_NS, CMP3_BASE_NS, CMP4_BASE_NS }
  /** Array initializer of CMP peripheral base pointers */
  #define CMP_BASE_PTRS_NS                         { (CMP_Type *)0u, CMP1_NS, CMP2_NS, CMP3_NS, CMP4_NS }
#else
  /** Peripheral CMP1 base address */
  #define CMP1_BASE                                (0x42DC0000u)
  /** Peripheral CMP1 base pointer */
  #define CMP1                                     ((CMP_Type *)CMP1_BASE)
  /** Peripheral CMP2 base address */
  #define CMP2_BASE                                (0x42DD0000u)
  /** Peripheral CMP2 base pointer */
  #define CMP2                                     ((CMP_Type *)CMP2_BASE)
  /** Peripheral CMP3 base address */
  #define CMP3_BASE                                (0x42DE0000u)
  /** Peripheral CMP3 base pointer */
  #define CMP3                                     ((CMP_Type *)CMP3_BASE)
  /** Peripheral CMP4 base address */
  #define CMP4_BASE                                (0x42DF0000u)
  /** Peripheral CMP4 base pointer */
  #define CMP4                                     ((CMP_Type *)CMP4_BASE)
  /** Array initializer of CMP peripheral base addresses */
  #define CMP_BASE_ADDRS                           { 0u, CMP1_BASE, CMP2_BASE, CMP3_BASE, CMP4_BASE }
  /** Array initializer of CMP peripheral base pointers */
  #define CMP_BASE_PTRS                            { (CMP_Type *)0u, CMP1, CMP2, CMP3, CMP4 }
#endif
/** Interrupt vectors for the CMP peripheral type */
#define CMP_IRQS                                 { NotAvail_IRQn, ACMP1_IRQn, ACMP2_IRQn, ACMP3_IRQn, ACMP4_IRQn }

/*!
 * @}
 */ /* end of group CMP_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- DAC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DAC_Peripheral_Access_Layer DAC Peripheral Access Layer
 * @{
 */

/** DAC - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version Identifier Register, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter Register, offset: 0x4 */
  __O  uint32_t DATA;                              /**< DAC Data Register, offset: 0x8 */
  __IO uint32_t CR;                                /**< DAC Status and Control Register, offset: 0xC */
  __I  uint32_t PTR;                               /**< DAC FIFO Pointer Register, offset: 0x10 */
  __IO uint32_t CR2;                               /**< DAC Status and Control Register 2, offset: 0x14 */
} DAC_Type;

/* ----------------------------------------------------------------------------
   -- DAC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DAC_Register_Masks DAC Register Masks
 * @{
 */

/*! @name VERID - Version Identifier Register */
/*! @{ */

#define DAC_VERID_FEATURE_MASK                   (0xFFFFU)
#define DAC_VERID_FEATURE_SHIFT                  (0U)
/*! FEATURE - Feature Identification Number
 *  0b0000000000000000..Standard feature set
 *  0b0000000000000001..C40 feature set
 *  0b0000000000000010..5V DAC feature set
 *  0b0000000000000100..ADC BIST feature set
 */
#define DAC_VERID_FEATURE(x)                     (((uint32_t)(((uint32_t)(x)) << DAC_VERID_FEATURE_SHIFT)) & DAC_VERID_FEATURE_MASK)

#define DAC_VERID_MINOR_MASK                     (0xFF0000U)
#define DAC_VERID_MINOR_SHIFT                    (16U)
/*! MINOR - Minor version number */
#define DAC_VERID_MINOR(x)                       (((uint32_t)(((uint32_t)(x)) << DAC_VERID_MINOR_SHIFT)) & DAC_VERID_MINOR_MASK)

#define DAC_VERID_MAJOR_MASK                     (0xFF000000U)
#define DAC_VERID_MAJOR_SHIFT                    (24U)
/*! MAJOR - Major version number */
#define DAC_VERID_MAJOR(x)                       (((uint32_t)(((uint32_t)(x)) << DAC_VERID_MAJOR_SHIFT)) & DAC_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter Register */
/*! @{ */

#define DAC_PARAM_FIFOSZ_MASK                    (0x7U)
#define DAC_PARAM_FIFOSZ_SHIFT                   (0U)
/*! FIFOSZ - FIFO size
 *  0b000..FIFO depth is 2
 *  0b001..FIFO depth is 4
 *  0b010..FIFO depth is 8
 *  0b011..FIFO depth is 16
 *  0b100..FIFO depth is 32
 *  0b101..FIFO depth is 64
 *  0b110..FIFO depth is 128
 *  0b111..FIFO depth is 256
 */
#define DAC_PARAM_FIFOSZ(x)                      (((uint32_t)(((uint32_t)(x)) << DAC_PARAM_FIFOSZ_SHIFT)) & DAC_PARAM_FIFOSZ_MASK)
/*! @} */

/*! @name DATA - DAC Data Register */
/*! @{ */

#define DAC_DATA_DATA0_MASK                      (0xFFFU)
#define DAC_DATA_DATA0_SHIFT                     (0U)
/*! DATA0 - FIFO DATA0 */
#define DAC_DATA_DATA0(x)                        (((uint32_t)(((uint32_t)(x)) << DAC_DATA_DATA0_SHIFT)) & DAC_DATA_DATA0_MASK)
/*! @} */

/*! @name CR - DAC Status and Control Register */
/*! @{ */

#define DAC_CR_FULLF_MASK                        (0x1U)
#define DAC_CR_FULLF_SHIFT                       (0U)
/*! FULLF - Full Flag
 *  0b0..FIFO is not full.
 *  0b1..FIFO is full.
 */
#define DAC_CR_FULLF(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR_FULLF_SHIFT)) & DAC_CR_FULLF_MASK)

#define DAC_CR_NEMPTF_MASK                       (0x2U)
#define DAC_CR_NEMPTF_SHIFT                      (1U)
/*! NEMPTF - Nearly Empty Flag
 *  0b0..More than one data is available in the FIFO.
 *  0b1..One data is available in the FIFO.
 */
#define DAC_CR_NEMPTF(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR_NEMPTF_SHIFT)) & DAC_CR_NEMPTF_MASK)

#define DAC_CR_WMF_MASK                          (0x4U)
#define DAC_CR_WMF_SHIFT                         (2U)
/*! WMF - FIFO Watermark Status Flag
 *  0b0..The DAC buffer read pointer has not reached the watermark level.
 *  0b1..The DAC buffer read pointer has reached the watermark level.
 */
#define DAC_CR_WMF(x)                            (((uint32_t)(((uint32_t)(x)) << DAC_CR_WMF_SHIFT)) & DAC_CR_WMF_MASK)

#define DAC_CR_UDFF_MASK                         (0x8U)
#define DAC_CR_UDFF_SHIFT                        (3U)
/*! UDFF - Underflow Flag
 *  0b0..No underflow has occurred since the last time the flag was cleared.
 *  0b1..At least one trigger underflow has occurred since the last time the flag was cleared.
 */
#define DAC_CR_UDFF(x)                           (((uint32_t)(((uint32_t)(x)) << DAC_CR_UDFF_SHIFT)) & DAC_CR_UDFF_MASK)

#define DAC_CR_OVFF_MASK                         (0x10U)
#define DAC_CR_OVFF_SHIFT                        (4U)
/*! OVFF - Overflow Flag
 *  0b0..No overflow has occurred since the last time the flag was cleared.
 *  0b1..At least one FIFO overflow has occurred since the last time the flag was cleared.
 */
#define DAC_CR_OVFF(x)                           (((uint32_t)(((uint32_t)(x)) << DAC_CR_OVFF_SHIFT)) & DAC_CR_OVFF_MASK)

#define DAC_CR_FULLIE_MASK                       (0x100U)
#define DAC_CR_FULLIE_SHIFT                      (8U)
/*! FULLIE - Full Interrupt Enable
 *  0b0..FIFO Full interrupt is disabled.
 *  0b1..FIFO Full interrupt is enabled.
 */
#define DAC_CR_FULLIE(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR_FULLIE_SHIFT)) & DAC_CR_FULLIE_MASK)

#define DAC_CR_EMPTIE_MASK                       (0x200U)
#define DAC_CR_EMPTIE_SHIFT                      (9U)
/*! EMPTIE - Nearly Empty Interrupt Enable
 *  0b0..FIFO Nearly Empty interrupt is disabled.
 *  0b1..FIFO Nearly Empty interrupt is enabled.
 */
#define DAC_CR_EMPTIE(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR_EMPTIE_SHIFT)) & DAC_CR_EMPTIE_MASK)

#define DAC_CR_WTMIE_MASK                        (0x400U)
#define DAC_CR_WTMIE_SHIFT                       (10U)
/*! WTMIE - Watermark Interrupt Enable
 *  0b0..Watermark interrupt is disabled.
 *  0b1..Watermark interrupt is enabled.
 */
#define DAC_CR_WTMIE(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR_WTMIE_SHIFT)) & DAC_CR_WTMIE_MASK)

#define DAC_CR_SWTRG_MASK                        (0x1000U)
#define DAC_CR_SWTRG_SHIFT                       (12U)
/*! SWTRG - DAC Software Trigger
 *  0b0..The DAC soft trigger is not valid.
 *  0b1..The DAC soft trigger is valid.
 */
#define DAC_CR_SWTRG(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR_SWTRG_SHIFT)) & DAC_CR_SWTRG_MASK)

#define DAC_CR_TRGSEL_MASK                       (0x2000U)
#define DAC_CR_TRGSEL_SHIFT                      (13U)
/*! TRGSEL - DAC Trigger Select
 *  0b0..The DAC hardware trigger is selected.
 *  0b1..The DAC software trigger is selected.
 */
#define DAC_CR_TRGSEL(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR_TRGSEL_SHIFT)) & DAC_CR_TRGSEL_MASK)

#define DAC_CR_DACRFS_MASK                       (0x4000U)
#define DAC_CR_DACRFS_SHIFT                      (14U)
/*! DACRFS - DAC Reference Select
 *  0b0..The DAC selects DACREF_1 as the reference voltage.
 *  0b1..The DAC selects DACREF_2 as the reference voltage.
 */
#define DAC_CR_DACRFS(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR_DACRFS_SHIFT)) & DAC_CR_DACRFS_MASK)

#define DAC_CR_DACEN_MASK                        (0x8000U)
#define DAC_CR_DACEN_SHIFT                       (15U)
/*! DACEN - DAC Enable
 *  0b0..The DAC system is disabled.
 *  0b1..The DAC system is enabled.
 */
#define DAC_CR_DACEN(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR_DACEN_SHIFT)) & DAC_CR_DACEN_MASK)

#define DAC_CR_FIFOEN_MASK                       (0x10000U)
#define DAC_CR_FIFOEN_SHIFT                      (16U)
/*! FIFOEN - FIFO Enable
 *  0b0..FIFO is disabled and only one level buffer is enabled. Any data written from this buffer goes to conversion.
 *  0b1..FIFO is enabled. Data will first read from FIFO to buffer then go to conversion.
 */
#define DAC_CR_FIFOEN(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR_FIFOEN_SHIFT)) & DAC_CR_FIFOEN_MASK)

#define DAC_CR_SWMD_MASK                         (0x20000U)
#define DAC_CR_SWMD_SHIFT                        (17U)
/*! SWMD - DAC FIFO Mode Select
 *  0b0..Normal mode
 *  0b1..Swing back mode
 */
#define DAC_CR_SWMD(x)                           (((uint32_t)(((uint32_t)(x)) << DAC_CR_SWMD_SHIFT)) & DAC_CR_SWMD_MASK)

#define DAC_CR_UVIE_MASK                         (0x40000U)
#define DAC_CR_UVIE_SHIFT                        (18U)
/*! UVIE - Underflow and overflow interrupt enable
 *  0b0..Underflow and overflow interrupt is disabled.
 *  0b1..Underflow and overflow interrupt is enabled.
 */
#define DAC_CR_UVIE(x)                           (((uint32_t)(((uint32_t)(x)) << DAC_CR_UVIE_SHIFT)) & DAC_CR_UVIE_MASK)

#define DAC_CR_FIFORST_MASK                      (0x200000U)
#define DAC_CR_FIFORST_SHIFT                     (21U)
/*! FIFORST - FIFO Reset
 *  0b0..No effect
 *  0b1..FIFO reset
 */
#define DAC_CR_FIFORST(x)                        (((uint32_t)(((uint32_t)(x)) << DAC_CR_FIFORST_SHIFT)) & DAC_CR_FIFORST_MASK)

#define DAC_CR_SWRST_MASK                        (0x400000U)
#define DAC_CR_SWRST_SHIFT                       (22U)
/*! SWRST - Software reset */
#define DAC_CR_SWRST(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR_SWRST_SHIFT)) & DAC_CR_SWRST_MASK)

#define DAC_CR_DMAEN_MASK                        (0x800000U)
#define DAC_CR_DMAEN_SHIFT                       (23U)
/*! DMAEN - DMA Enable Select
 *  0b0..DMA is disabled.
 *  0b1..DMA is enabled. When DMA is enabled, the DMA request will be generated by original interrupts. The
 *       interrupts will not be presented on this module at the same time.
 */
#define DAC_CR_DMAEN(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR_DMAEN_SHIFT)) & DAC_CR_DMAEN_MASK)

#define DAC_CR_WML_MASK                          (0xFF000000U)
#define DAC_CR_WML_SHIFT                         (24U)
/*! WML - Watermark Level Select */
#define DAC_CR_WML(x)                            (((uint32_t)(((uint32_t)(x)) << DAC_CR_WML_SHIFT)) & DAC_CR_WML_MASK)
/*! @} */

/*! @name PTR - DAC FIFO Pointer Register */
/*! @{ */

#define DAC_PTR_DACWFP_MASK                      (0xFFU)
#define DAC_PTR_DACWFP_SHIFT                     (0U)
/*! DACWFP - DACWFP */
#define DAC_PTR_DACWFP(x)                        (((uint32_t)(((uint32_t)(x)) << DAC_PTR_DACWFP_SHIFT)) & DAC_PTR_DACWFP_MASK)

#define DAC_PTR_DACRFP_MASK                      (0xFF0000U)
#define DAC_PTR_DACRFP_SHIFT                     (16U)
/*! DACRFP - DACRFP */
#define DAC_PTR_DACRFP(x)                        (((uint32_t)(((uint32_t)(x)) << DAC_PTR_DACRFP_SHIFT)) & DAC_PTR_DACRFP_MASK)
/*! @} */

/*! @name CR2 - DAC Status and Control Register 2 */
/*! @{ */

#define DAC_CR2_BFEN_MASK                        (0x1U)
#define DAC_CR2_BFEN_SHIFT                       (0U)
/*! BFEN - Buffer Enable
 *  0b0..Opamp is not used as buffer
 *  0b1..Opamp is used as buffer
 */
#define DAC_CR2_BFEN(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR2_BFEN_SHIFT)) & DAC_CR2_BFEN_MASK)

#define DAC_CR2_OEN_MASK                         (0x2U)
#define DAC_CR2_OEN_SHIFT                        (1U)
/*! OEN - Optional Enable
 *  0b0..Output buffer is not bypassed
 *  0b1..Output buffer is bypassed
 */
#define DAC_CR2_OEN(x)                           (((uint32_t)(((uint32_t)(x)) << DAC_CR2_OEN_SHIFT)) & DAC_CR2_OEN_MASK)

#define DAC_CR2_BFMS_MASK                        (0x4U)
#define DAC_CR2_BFMS_SHIFT                       (2U)
/*! BFMS - Buffer Middle Speed Select
 *  0b0..Buffer middle speed not selected
 *  0b1..Buffer middle speed selected
 */
#define DAC_CR2_BFMS(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR2_BFMS_SHIFT)) & DAC_CR2_BFMS_MASK)

#define DAC_CR2_BFHS_MASK                        (0x8U)
#define DAC_CR2_BFHS_SHIFT                       (3U)
/*! BFHS - Buffer High Speed Select
 *  0b0..Buffer high speed not selected
 *  0b1..Buffer high speed selected
 */
#define DAC_CR2_BFHS(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR2_BFHS_SHIFT)) & DAC_CR2_BFHS_MASK)

#define DAC_CR2_IREF2_MASK                       (0x10U)
#define DAC_CR2_IREF2_SHIFT                      (4U)
/*! IREF2 - Internal PTAT (Proportional To Absolute Temperature) Current Reference Select
 *  0b0..Internal PTAT Current Reference not selected
 *  0b1..Internal PTAT Current Reference selected
 */
#define DAC_CR2_IREF2(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR2_IREF2_SHIFT)) & DAC_CR2_IREF2_MASK)

#define DAC_CR2_IREF1_MASK                       (0x20U)
#define DAC_CR2_IREF1_SHIFT                      (5U)
/*! IREF1 - Internal ZTC (Zero Temperature Coefficient) Current Reference Select
 *  0b0..Internal ZTC Current Reference not selected
 *  0b1..Internal ZTC Current Reference selected
 */
#define DAC_CR2_IREF1(x)                         (((uint32_t)(((uint32_t)(x)) << DAC_CR2_IREF1_SHIFT)) & DAC_CR2_IREF1_MASK)

#define DAC_CR2_IREF_MASK                        (0x40U)
#define DAC_CR2_IREF_SHIFT                       (6U)
/*! IREF - Internal Current Reference Select
 *  0b0..Internal Current Reference not selected
 *  0b1..Internal Current Reference selected
 */
#define DAC_CR2_IREF(x)                          (((uint32_t)(((uint32_t)(x)) << DAC_CR2_IREF_SHIFT)) & DAC_CR2_IREF_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group DAC_Register_Masks */


/* DAC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral DAC base address */
  #define DAC_BASE                                 (0x52E20000u)
  /** Peripheral DAC base address */
  #define DAC_BASE_NS                              (0x42E20000u)
  /** Peripheral DAC base pointer */
  #define DAC                                      ((DAC_Type *)DAC_BASE)
  /** Peripheral DAC base pointer */
  #define DAC_NS                                   ((DAC_Type *)DAC_BASE_NS)
  /** Array initializer of DAC peripheral base addresses */
  #define DAC_BASE_ADDRS                           { DAC_BASE }
  /** Array initializer of DAC peripheral base pointers */
  #define DAC_BASE_PTRS                            { DAC }
  /** Array initializer of DAC peripheral base addresses */
  #define DAC_BASE_ADDRS_NS                        { DAC_BASE_NS }
  /** Array initializer of DAC peripheral base pointers */
  #define DAC_BASE_PTRS_NS                         { DAC_NS }
#else
  /** Peripheral DAC base address */
  #define DAC_BASE                                 (0x42E20000u)
  /** Peripheral DAC base pointer */
  #define DAC                                      ((DAC_Type *)DAC_BASE)
  /** Array initializer of DAC peripheral base addresses */
  #define DAC_BASE_ADDRS                           { DAC_BASE }
  /** Array initializer of DAC peripheral base pointers */
  #define DAC_BASE_PTRS                            { DAC }
#endif
/** Interrupt vectors for the DAC peripheral type */
#define DAC_IRQS                                 { DAC_IRQn }

/*!
 * @}
 */ /* end of group DAC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- DCDC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DCDC_Peripheral_Access_Layer DCDC Peripheral Access Layer
 * @{
 */

/** DCDC - Register Layout Typedef */
typedef struct {
  __IO uint32_t REG0;                              /**< DCDC Register 0, offset: 0x0 */
  __IO uint32_t REG1;                              /**< DCDC Register 1, offset: 0x4 */
  __IO uint32_t REG2;                              /**< DCDC Register 2, offset: 0x8 */
  __IO uint32_t REG3;                              /**< DCDC Register 3, offset: 0xC */
  __IO uint32_t CTRL0;                             /**< DCDC Control Register 0, offset: 0x10 */
  __IO uint32_t OK_CNT;                            /**< OK CNT, offset: 0x14 */
  __IO uint32_t CURRENT_TRG;                       /**< CURRENT TARGET VALUE for DCDC ANALOG, offset: 0x18 */
  __IO uint32_t FILTER_CNT;                        /**< FILTER CNT, offset: 0x1C */
  __IO uint32_t TRG_0_AUTHEN;                      /**< TRG_0 Authentication Control, offset: 0x20 */
  __IO uint32_t TRG_SW_0;                          /**< Target SW Control for CORE 0, offset: 0x24 */
  __IO uint32_t TRG_GPC_0;                         /**< Target GPC Control for CORE 0, offset: 0x28 */
       uint8_t RESERVED_0[4];
  __IO uint32_t TRG_1_AUTHEN;                      /**< TRG_1 Authentication Control, offset: 0x30 */
  __IO uint32_t TRG_SW_1;                          /**< Target SW Control for CORE 1, offset: 0x34 */
  __IO uint32_t TRG_GPC_1;                         /**< Target GPC Control for CORE 1, offset: 0x38 */
} DCDC_Type;

/* ----------------------------------------------------------------------------
   -- DCDC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DCDC_Register_Masks DCDC Register Masks
 * @{
 */

/*! @name REG0 - DCDC Register 0 */
/*! @{ */

#define DCDC_REG0_PWD_ZCD_MASK                   (0x1U)
#define DCDC_REG0_PWD_ZCD_SHIFT                  (0U)
/*! PWD_ZCD - Power Down Zero Cross Detection
 *  0b0..Zero cross detection function powered up
 *  0b1..Zero cross detection function powered down
 */
#define DCDC_REG0_PWD_ZCD(x)                     (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_PWD_ZCD_SHIFT)) & DCDC_REG0_PWD_ZCD_MASK)

#define DCDC_REG0_DISABLE_AUTO_CLK_SWITCH_MASK   (0x2U)
#define DCDC_REG0_DISABLE_AUTO_CLK_SWITCH_SHIFT  (1U)
/*! DISABLE_AUTO_CLK_SWITCH - Disable Auto Clock Switch
 *  0b0..If DISABLE_AUTO_CLK_SWITCH is set to 0 and 24M xtal is OK, the clock source will switch from internal ring OSC to 24M xtal automatically
 *  0b1..If DISABLE_AUTO_CLK_SWITCH is set to 1, SEL_CLK will determine which clock source the DCDC uses
 */
#define DCDC_REG0_DISABLE_AUTO_CLK_SWITCH(x)     (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_DISABLE_AUTO_CLK_SWITCH_SHIFT)) & DCDC_REG0_DISABLE_AUTO_CLK_SWITCH_MASK)

#define DCDC_REG0_SEL_CLK_MASK                   (0x4U)
#define DCDC_REG0_SEL_CLK_SHIFT                  (2U)
/*! SEL_CLK - Select Clock
 *  0b0..DCDC uses internal ring oscillator
 *  0b1..DCDC uses 24M xtal
 */
#define DCDC_REG0_SEL_CLK(x)                     (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_SEL_CLK_SHIFT)) & DCDC_REG0_SEL_CLK_MASK)

#define DCDC_REG0_PWD_OSC_INT_MASK               (0x8U)
#define DCDC_REG0_PWD_OSC_INT_SHIFT              (3U)
/*! PWD_OSC_INT - Power down internal osc
 *  0b0..Internal oscillator powered up
 *  0b1..Internal oscillator powered down
 */
#define DCDC_REG0_PWD_OSC_INT(x)                 (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_PWD_OSC_INT_SHIFT)) & DCDC_REG0_PWD_OSC_INT_MASK)

#define DCDC_REG0_PWD_OVERCUR_DET_MASK           (0x100U)
#define DCDC_REG0_PWD_OVERCUR_DET_SHIFT          (8U)
/*! PWD_OVERCUR_DET - Power down overcurrent detection comparator
 *  0b0..Overcurrent detection comparator is enabled
 *  0b1..Overcurrent detection comparator is disabled
 */
#define DCDC_REG0_PWD_OVERCUR_DET(x)             (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_PWD_OVERCUR_DET_SHIFT)) & DCDC_REG0_PWD_OVERCUR_DET_MASK)

#define DCDC_REG0_OVERCUR_TRIG_ADJ_MASK          (0x600U)
#define DCDC_REG0_OVERCUR_TRIG_ADJ_SHIFT         (9U)
/*! OVERCUR_TRIG_ADJ - Overcurrent Trigger Adjust
 *  0b00..In Run Mode, 1.5 A. In LP Mode, 150 mA
 *  0b01..In Run Mode, 1.5 A. In LP Mode, 130 mA
 *  0b10..In Run Mode, 2 A. In LP Mode, 150 mA
 *  0b11..In Run Mode, 2 A. In LP Mode, 130 mA
 */
#define DCDC_REG0_OVERCUR_TRIG_ADJ(x)            (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_OVERCUR_TRIG_ADJ_SHIFT)) & DCDC_REG0_OVERCUR_TRIG_ADJ_MASK)

#define DCDC_REG0_PWD_CMP_DCDC_IN_DET_MASK       (0x800U)
#define DCDC_REG0_PWD_CMP_DCDC_IN_DET_SHIFT      (11U)
/*! PWD_CMP_DCDC_IN_DET
 *  0b0..Low voltage detection comparator is enabled
 *  0b1..Low voltage detection comparator is disabled
 */
#define DCDC_REG0_PWD_CMP_DCDC_IN_DET(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_PWD_CMP_DCDC_IN_DET_SHIFT)) & DCDC_REG0_PWD_CMP_DCDC_IN_DET_MASK)

#define DCDC_REG0_PWD_HIGH_VDD1P8_DET_MASK       (0x10000U)
#define DCDC_REG0_PWD_HIGH_VDD1P8_DET_SHIFT      (16U)
/*! PWD_HIGH_VDD1P8_DET - Power Down High Voltage Detection for VDD1P8
 *  0b0..Overvoltage detection comparator for the VDD1P8 output is enabled
 *  0b1..Overvoltage detection comparator for the VDD1P8 output is disabled
 */
#define DCDC_REG0_PWD_HIGH_VDD1P8_DET(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_PWD_HIGH_VDD1P8_DET_SHIFT)) & DCDC_REG0_PWD_HIGH_VDD1P8_DET_MASK)

#define DCDC_REG0_PWD_HIGH_VDD1P0_DET_MASK       (0x20000U)
#define DCDC_REG0_PWD_HIGH_VDD1P0_DET_SHIFT      (17U)
/*! PWD_HIGH_VDD1P0_DET - Power Down High Voltage Detection for VDD1P0
 *  0b0..Overvoltage detection comparator for the VDD1P0 output is enabled
 *  0b1..Overvoltage detection comparator for the VDD1P0 output is disabled
 */
#define DCDC_REG0_PWD_HIGH_VDD1P0_DET(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_PWD_HIGH_VDD1P0_DET_SHIFT)) & DCDC_REG0_PWD_HIGH_VDD1P0_DET_MASK)

#define DCDC_REG0_PWD_CMP_OFFSET_MASK            (0x4000000U)
#define DCDC_REG0_PWD_CMP_OFFSET_SHIFT           (26U)
/*! PWD_CMP_OFFSET - power down the out-of-range detection comparator
 *  0b0..Out-of-range comparator powered up
 *  0b1..Out-of-range comparator powered down
 */
#define DCDC_REG0_PWD_CMP_OFFSET(x)              (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_PWD_CMP_OFFSET_SHIFT)) & DCDC_REG0_PWD_CMP_OFFSET_MASK)

#define DCDC_REG0_XTALOK_DISABLE_MASK            (0x8000000U)
#define DCDC_REG0_XTALOK_DISABLE_SHIFT           (27U)
/*! XTALOK_DISABLE - Disable xtalok detection circuit
 *  0b0..Enable xtalok detection circuit
 *  0b1..Disable xtalok detection circuit and always outputs OK signal "1"
 */
#define DCDC_REG0_XTALOK_DISABLE(x)              (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_XTALOK_DISABLE_SHIFT)) & DCDC_REG0_XTALOK_DISABLE_MASK)

#define DCDC_REG0_XTAL_24M_OK_MASK               (0x20000000U)
#define DCDC_REG0_XTAL_24M_OK_SHIFT              (29U)
/*! XTAL_24M_OK - 24M XTAL OK
 *  0b0..DCDC uses internal ring OSC
 *  0b1..DCDC uses xtal 24M
 */
#define DCDC_REG0_XTAL_24M_OK(x)                 (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_XTAL_24M_OK_SHIFT)) & DCDC_REG0_XTAL_24M_OK_MASK)

#define DCDC_REG0_STS_DC_OK_MASK                 (0x80000000U)
#define DCDC_REG0_STS_DC_OK_SHIFT                (31U)
/*! STS_DC_OK - DCDC Output OK
 *  0b0..DCDC is settling
 *  0b1..DCDC already settled
 */
#define DCDC_REG0_STS_DC_OK(x)                   (((uint32_t)(((uint32_t)(x)) << DCDC_REG0_STS_DC_OK_SHIFT)) & DCDC_REG0_STS_DC_OK_MASK)
/*! @} */

/*! @name REG1 - DCDC Register 1 */
/*! @{ */

#define DCDC_REG1_RLOAD_REG_EN_MASK              (0x10U)
#define DCDC_REG1_RLOAD_REG_EN_SHIFT             (4U)
/*! RLOAD_REG_EN - Resistor Load of Regulator Enable
 *  0b0..Resistor load disconnected
 *  0b1..Resistor load connected
 */
#define DCDC_REG1_RLOAD_REG_EN(x)                (((uint32_t)(((uint32_t)(x)) << DCDC_REG1_RLOAD_REG_EN_SHIFT)) & DCDC_REG1_RLOAD_REG_EN_MASK)

#define DCDC_REG1_VBG_TRIM_MASK                  (0x7C0U)
#define DCDC_REG1_VBG_TRIM_SHIFT                 (6U)
#define DCDC_REG1_VBG_TRIM(x)                    (((uint32_t)(((uint32_t)(x)) << DCDC_REG1_VBG_TRIM_SHIFT)) & DCDC_REG1_VBG_TRIM_MASK)

#define DCDC_REG1_NEGLIMIT_IN_MASK               (0xFE000U)
#define DCDC_REG1_NEGLIMIT_IN_SHIFT              (13U)
#define DCDC_REG1_NEGLIMIT_IN(x)                 (((uint32_t)(((uint32_t)(x)) << DCDC_REG1_NEGLIMIT_IN_SHIFT)) & DCDC_REG1_NEGLIMIT_IN_MASK)

#define DCDC_REG1_LOOPCTRL_CM_HST_THRESH_MASK    (0x8000000U)
#define DCDC_REG1_LOOPCTRL_CM_HST_THRESH_SHIFT   (27U)
/*! LOOPCTRL_CM_HST_THRESH - Increase Threshold Detection
 *  0b0..Disable increase the threshold detection for common mode analog comparators.
 *  0b1..Enable increase the threshold detection for common mode analog comparators.
 */
#define DCDC_REG1_LOOPCTRL_CM_HST_THRESH(x)      (((uint32_t)(((uint32_t)(x)) << DCDC_REG1_LOOPCTRL_CM_HST_THRESH_SHIFT)) & DCDC_REG1_LOOPCTRL_CM_HST_THRESH_MASK)

#define DCDC_REG1_LOOPCTRL_DF_HST_THRESH_MASK    (0x10000000U)
#define DCDC_REG1_LOOPCTRL_DF_HST_THRESH_SHIFT   (28U)
/*! LOOPCTRL_DF_HST_THRESH - Increase Threshold Detection
 *  0b0..Disable increase the threshold detection for differential mode analog comparators.
 *  0b1..Enable increase the threshold detection for differential mode analog comparators.
 */
#define DCDC_REG1_LOOPCTRL_DF_HST_THRESH(x)      (((uint32_t)(((uint32_t)(x)) << DCDC_REG1_LOOPCTRL_DF_HST_THRESH_SHIFT)) & DCDC_REG1_LOOPCTRL_DF_HST_THRESH_MASK)

#define DCDC_REG1_LOOPCTRL_EN_CM_HYST_MASK       (0x20000000U)
#define DCDC_REG1_LOOPCTRL_EN_CM_HYST_SHIFT      (29U)
/*! LOOPCTRL_EN_CM_HYST
 *  0b0..Disable hysteresis in switching converter common mode analog comparators
 *  0b1..Enable hysteresis in switching converter common mode analog comparators
 */
#define DCDC_REG1_LOOPCTRL_EN_CM_HYST(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG1_LOOPCTRL_EN_CM_HYST_SHIFT)) & DCDC_REG1_LOOPCTRL_EN_CM_HYST_MASK)

#define DCDC_REG1_LOOPCTRL_EN_DF_HYST_MASK       (0x40000000U)
#define DCDC_REG1_LOOPCTRL_EN_DF_HYST_SHIFT      (30U)
/*! LOOPCTRL_EN_DF_HYST
 *  0b0..Disable hysteresis in switching converter differential mode analog comparators
 *  0b1..Enable hysteresis in switching converter differential mode analog comparators
 */
#define DCDC_REG1_LOOPCTRL_EN_DF_HYST(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG1_LOOPCTRL_EN_DF_HYST_SHIFT)) & DCDC_REG1_LOOPCTRL_EN_DF_HYST_MASK)
/*! @} */

/*! @name REG2 - DCDC Register 2 */
/*! @{ */

#define DCDC_REG2_LOOPCTRL_DC_C_MASK             (0x3U)
#define DCDC_REG2_LOOPCTRL_DC_C_SHIFT            (0U)
#define DCDC_REG2_LOOPCTRL_DC_C(x)               (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_LOOPCTRL_DC_C_SHIFT)) & DCDC_REG2_LOOPCTRL_DC_C_MASK)

#define DCDC_REG2_LOOPCTRL_DC_R_MASK             (0x3CU)
#define DCDC_REG2_LOOPCTRL_DC_R_SHIFT            (2U)
#define DCDC_REG2_LOOPCTRL_DC_R(x)               (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_LOOPCTRL_DC_R_SHIFT)) & DCDC_REG2_LOOPCTRL_DC_R_MASK)

#define DCDC_REG2_LOOPCTRL_DC_FF_MASK            (0x1C0U)
#define DCDC_REG2_LOOPCTRL_DC_FF_SHIFT           (6U)
#define DCDC_REG2_LOOPCTRL_DC_FF(x)              (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_LOOPCTRL_DC_FF_SHIFT)) & DCDC_REG2_LOOPCTRL_DC_FF_MASK)

#define DCDC_REG2_LOOPCTRL_EN_RCSCALE_MASK       (0xE00U)
#define DCDC_REG2_LOOPCTRL_EN_RCSCALE_SHIFT      (9U)
/*! LOOPCTRL_EN_RCSCALE - Enable RC Scale */
#define DCDC_REG2_LOOPCTRL_EN_RCSCALE(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_LOOPCTRL_EN_RCSCALE_SHIFT)) & DCDC_REG2_LOOPCTRL_EN_RCSCALE_MASK)

#define DCDC_REG2_LOOPCTRL_RCSCALE_THRSH_MASK    (0x1000U)
#define DCDC_REG2_LOOPCTRL_RCSCALE_THRSH_SHIFT   (12U)
/*! LOOPCTRL_RCSCALE_THRSH
 *  0b0..Disable increasing the threshold detection function.
 *  0b1..Enable increasing the threshold detection function.
 */
#define DCDC_REG2_LOOPCTRL_RCSCALE_THRSH(x)      (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_LOOPCTRL_RCSCALE_THRSH_SHIFT)) & DCDC_REG2_LOOPCTRL_RCSCALE_THRSH_MASK)

#define DCDC_REG2_LOOPCTRL_HYST_SIGN_MASK        (0x2000U)
#define DCDC_REG2_LOOPCTRL_HYST_SIGN_SHIFT       (13U)
/*! LOOPCTRL_HYST_SIGN
 *  0b0..Disable the invert function.
 *  0b1..Enable the invert function.
 */
#define DCDC_REG2_LOOPCTRL_HYST_SIGN(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_LOOPCTRL_HYST_SIGN_SHIFT)) & DCDC_REG2_LOOPCTRL_HYST_SIGN_MASK)

#define DCDC_REG2_BATTMONITOR_EN_BATADJ_MASK     (0x8000U)
#define DCDC_REG2_BATTMONITOR_EN_BATADJ_SHIFT    (15U)
/*! BATTMONITOR_EN_BATADJ
 *  0b0..Disable the improvement function.
 *  0b1..Enable the improvement function.
 */
#define DCDC_REG2_BATTMONITOR_EN_BATADJ(x)       (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_BATTMONITOR_EN_BATADJ_SHIFT)) & DCDC_REG2_BATTMONITOR_EN_BATADJ_MASK)

#define DCDC_REG2_BATTMONITOR_BATT_VAL_MASK      (0x3FF0000U)
#define DCDC_REG2_BATTMONITOR_BATT_VAL_SHIFT     (16U)
#define DCDC_REG2_BATTMONITOR_BATT_VAL(x)        (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_BATTMONITOR_BATT_VAL_SHIFT)) & DCDC_REG2_BATTMONITOR_BATT_VAL_MASK)

#define DCDC_REG2_LOOPCTRL_TOGGLE_DIF_MASK       (0x40000000U)
#define DCDC_REG2_LOOPCTRL_TOGGLE_DIF_SHIFT      (30U)
/*! LOOPCTRL_TOGGLE_DIF
 *  0b0..Disable supply stepping to change.
 *  0b1..Enable supply stepping to change.
 */
#define DCDC_REG2_LOOPCTRL_TOGGLE_DIF(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG2_LOOPCTRL_TOGGLE_DIF_SHIFT)) & DCDC_REG2_LOOPCTRL_TOGGLE_DIF_MASK)
/*! @} */

/*! @name REG3 - DCDC Register 3 */
/*! @{ */

#define DCDC_REG3_IN_BROWNOUT_WARN_MASK          (0x200U)
#define DCDC_REG3_IN_BROWNOUT_WARN_SHIFT         (9U)
/*! IN_BROWNOUT_WARN
 *  0b0..The voltage on DCDC_IN raises up to 2.8V.
 *  0b1..The voltage on DCDC_IN is lower than 2.8V. Once this bit sets, the bit must be cleared by software write
 *       one clear action while the voltage on DCDC_IN raises up to 2.8V. Writing "0" to this bit has no effect.
 *       Writing "1" to this bit has no effect while it's "0".
 */
#define DCDC_REG3_IN_BROWNOUT_WARN(x)            (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_IN_BROWNOUT_WARN_SHIFT)) & DCDC_REG3_IN_BROWNOUT_WARN_MASK)

#define DCDC_REG3_ENABLE_FF_MASK                 (0x40000U)
#define DCDC_REG3_ENABLE_FF_SHIFT                (18U)
/*! ENABLE_FF
 *  0b0..Enable the FF function.
 *  0b1..Disable the FF function.
 */
#define DCDC_REG3_ENABLE_FF(x)                   (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_ENABLE_FF_SHIFT)) & DCDC_REG3_ENABLE_FF_MASK)

#define DCDC_REG3_DISABLE_PULSE_SKIP_MASK        (0x80000U)
#define DCDC_REG3_DISABLE_PULSE_SKIP_SHIFT       (19U)
/*! DISABLE_PULSE_SKIP - Disable Pulse Skip */
#define DCDC_REG3_DISABLE_PULSE_SKIP(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_DISABLE_PULSE_SKIP_SHIFT)) & DCDC_REG3_DISABLE_PULSE_SKIP_MASK)

#define DCDC_REG3_DISABLE_IDLE_SKIP_MASK         (0x100000U)
#define DCDC_REG3_DISABLE_IDLE_SKIP_SHIFT        (20U)
/*! DISABLE_IDLE_SKIP
 *  0b0..Enable the idle skip function.
 *  0b1..Disable the idle skip function.
 */
#define DCDC_REG3_DISABLE_IDLE_SKIP(x)           (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_DISABLE_IDLE_SKIP_SHIFT)) & DCDC_REG3_DISABLE_IDLE_SKIP_MASK)

#define DCDC_REG3_DOUBLE_IBIAS_CMP_LP_MASK       (0x200000U)
#define DCDC_REG3_DOUBLE_IBIAS_CMP_LP_SHIFT      (21U)
/*! DOUBLE_IBIAS_CMP_LP
 *  0b0..Disable the function.
 *  0b1..Enable the function.
 */
#define DCDC_REG3_DOUBLE_IBIAS_CMP_LP(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_DOUBLE_IBIAS_CMP_LP_SHIFT)) & DCDC_REG3_DOUBLE_IBIAS_CMP_LP_MASK)

#define DCDC_REG3_REG_FBK_SEL_MASK               (0xC00000U)
#define DCDC_REG3_REG_FBK_SEL_SHIFT              (22U)
#define DCDC_REG3_REG_FBK_SEL(x)                 (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_REG_FBK_SEL_SHIFT)) & DCDC_REG3_REG_FBK_SEL_MASK)

#define DCDC_REG3_MINPWR_HALF_FETS_MASK          (0x4000000U)
#define DCDC_REG3_MINPWR_HALF_FETS_SHIFT         (26U)
/*! MINPWR_HALF_FETS
 *  0b0..Donot use half switch FET.
 *  0b1..Use half switch FET.
 */
#define DCDC_REG3_MINPWR_HALF_FETS(x)            (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_MINPWR_HALF_FETS_SHIFT)) & DCDC_REG3_MINPWR_HALF_FETS_MASK)

#define DCDC_REG3_MISC_DELAY_TIMING_MASK         (0x8000000U)
#define DCDC_REG3_MISC_DELAY_TIMING_SHIFT        (27U)
/*! MISC_DELAY_TIMING - Miscellaneous Delay Timing
 *  0b0..Disable the function.
 *  0b1..Enable the function.
 */
#define DCDC_REG3_MISC_DELAY_TIMING(x)           (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_MISC_DELAY_TIMING_SHIFT)) & DCDC_REG3_MISC_DELAY_TIMING_MASK)

#define DCDC_REG3_VDD1P0CTRL_DISABLE_STEP_MASK   (0x20000000U)
#define DCDC_REG3_VDD1P0CTRL_DISABLE_STEP_SHIFT  (29U)
/*! VDD1P0CTRL_DISABLE_STEP - Disable Step for VDD1P0
 *  0b0..Enable stepping for VDD1P0
 *  0b1..Disable stepping for VDD1P0
 */
#define DCDC_REG3_VDD1P0CTRL_DISABLE_STEP(x)     (((uint32_t)(((uint32_t)(x)) << DCDC_REG3_VDD1P0CTRL_DISABLE_STEP_SHIFT)) & DCDC_REG3_VDD1P0CTRL_DISABLE_STEP_MASK)
/*! @} */

/*! @name CTRL0 - DCDC Control Register 0 */
/*! @{ */

#define DCDC_CTRL0_ENABLE_OK_CNT_MASK            (0x20U)
#define DCDC_CTRL0_ENABLE_OK_CNT_SHIFT           (5U)
/*! ENABLE_OK_CNT - Enable internal count for DCDC_OK timeout
 *  0b0..Wait DCDC_OK for ACK
 *  0b1..Enable internal count for DCDC_OK timeout
 */
#define DCDC_CTRL0_ENABLE_OK_CNT(x)              (((uint32_t)(((uint32_t)(x)) << DCDC_CTRL0_ENABLE_OK_CNT_SHIFT)) & DCDC_CTRL0_ENABLE_OK_CNT_MASK)

#define DCDC_CTRL0_TRIM_HOLD_MASK                (0x40U)
#define DCDC_CTRL0_TRIM_HOLD_SHIFT               (6U)
/*! TRIM_HOLD - Hold trim input
 *  0b0..Sample trim value from FUSE or value from REG1[VBG_TRIM] depending on FUSE select bit.
 *  0b1..Use value from REG1[VBG_TRIM] as trim value.
 */
#define DCDC_CTRL0_TRIM_HOLD(x)                  (((uint32_t)(((uint32_t)(x)) << DCDC_CTRL0_TRIM_HOLD_SHIFT)) & DCDC_CTRL0_TRIM_HOLD_MASK)

#define DCDC_CTRL0_IN_BROWNOUT_WARN_EN_MASK      (0x200U)
#define DCDC_CTRL0_IN_BROWNOUT_WARN_EN_SHIFT     (9U)
/*! IN_BROWNOUT_WARN_EN - IN_BROWNOUT_WARN_EN
 *  0b0..Disable IN_BROWNOUT_WARN int flag bit output to CORE as an interrupt resource.
 *  0b1..Enable IN_BROWNOUT_WARN int flag bit output to CORE as an interrupt resource.
 */
#define DCDC_CTRL0_IN_BROWNOUT_WARN_EN(x)        (((uint32_t)(((uint32_t)(x)) << DCDC_CTRL0_IN_BROWNOUT_WARN_EN_SHIFT)) & DCDC_CTRL0_IN_BROWNOUT_WARN_EN_MASK)

#define DCDC_CTRL0_DEBUG_BITS_MASK               (0x7FF80000U)
#define DCDC_CTRL0_DEBUG_BITS_SHIFT              (19U)
#define DCDC_CTRL0_DEBUG_BITS(x)                 (((uint32_t)(((uint32_t)(x)) << DCDC_CTRL0_DEBUG_BITS_SHIFT)) & DCDC_CTRL0_DEBUG_BITS_MASK)

#define DCDC_CTRL0_TRG_GPC_EN_MASK               (0x80000000U)
#define DCDC_CTRL0_TRG_GPC_EN_SHIFT              (31U)
/*! TRG_GPC_EN - TRG_GPC_EN: used to enable TRG_GPC_* value or not.
 *  0b0..No matter there is GPC stby request or not, value in TRG_SW_* register will always be used as DCDC analog target value.
 *  0b1..When there is a GPC stby request, value in TRG_GPC_* register will be used as DCDC analog target value instead of TRG_SW_*'s
 */
#define DCDC_CTRL0_TRG_GPC_EN(x)                 (((uint32_t)(((uint32_t)(x)) << DCDC_CTRL0_TRG_GPC_EN_SHIFT)) & DCDC_CTRL0_TRG_GPC_EN_MASK)
/*! @} */

/*! @name OK_CNT - OK CNT */
/*! @{ */

#define DCDC_OK_CNT_OK_COUNT_MASK                (0xFFFFFFFFU)
#define DCDC_OK_CNT_OK_COUNT_SHIFT               (0U)
/*! OK_COUNT - OK_COUNT */
#define DCDC_OK_CNT_OK_COUNT(x)                  (((uint32_t)(((uint32_t)(x)) << DCDC_OK_CNT_OK_COUNT_SHIFT)) & DCDC_OK_CNT_OK_COUNT_MASK)
/*! @} */

/*! @name CURRENT_TRG - CURRENT TARGET VALUE for DCDC ANALOG */
/*! @{ */

#define DCDC_CURRENT_TRG_VDD1P0CTRL_TRG_MASK     (0x1FU)
#define DCDC_CURRENT_TRG_VDD1P0CTRL_TRG_SHIFT    (0U)
#define DCDC_CURRENT_TRG_VDD1P0CTRL_TRG(x)       (((uint32_t)(((uint32_t)(x)) << DCDC_CURRENT_TRG_VDD1P0CTRL_TRG_SHIFT)) & DCDC_CURRENT_TRG_VDD1P0CTRL_TRG_MASK)

#define DCDC_CURRENT_TRG_VDD1P8CTRL_TRG_MASK     (0x1F00U)
#define DCDC_CURRENT_TRG_VDD1P8CTRL_TRG_SHIFT    (8U)
#define DCDC_CURRENT_TRG_VDD1P8CTRL_TRG(x)       (((uint32_t)(((uint32_t)(x)) << DCDC_CURRENT_TRG_VDD1P8CTRL_TRG_SHIFT)) & DCDC_CURRENT_TRG_VDD1P8CTRL_TRG_MASK)

#define DCDC_CURRENT_TRG_DCDC_UPDATING_MASK      (0x8000U)
#define DCDC_CURRENT_TRG_DCDC_UPDATING_SHIFT     (15U)
/*! DCDC_UPDATING
 *  0b0..Last DCDC change has been done. New value can be written to TRG register to trigger a new change of DCDC voltage.
 *  0b1..Last DCDC change is still not done. New value should not be written to TRG register to trigger a new change of DCDC voltage.
 */
#define DCDC_CURRENT_TRG_DCDC_UPDATING(x)        (((uint32_t)(((uint32_t)(x)) << DCDC_CURRENT_TRG_DCDC_UPDATING_SHIFT)) & DCDC_CURRENT_TRG_DCDC_UPDATING_MASK)

#define DCDC_CURRENT_TRG_VDD1P0CTRL_LP_TRG_MASK  (0x1F0000U)
#define DCDC_CURRENT_TRG_VDD1P0CTRL_LP_TRG_SHIFT (16U)
#define DCDC_CURRENT_TRG_VDD1P0CTRL_LP_TRG(x)    (((uint32_t)(((uint32_t)(x)) << DCDC_CURRENT_TRG_VDD1P0CTRL_LP_TRG_SHIFT)) & DCDC_CURRENT_TRG_VDD1P0CTRL_LP_TRG_MASK)

#define DCDC_CURRENT_TRG_LP_EN_1P0_MASK          (0x80000000U)
#define DCDC_CURRENT_TRG_LP_EN_1P0_SHIFT         (31U)
/*! LP_EN_1P0 - This value comes from the smaller one between TRG_SW_0 and TRG_SW_1. This bit only
 *    controls 1P0. 1P8 is always controlled by VDD1P8CTRL_TRG
 *  0b0..DCDC 1P0 works in run mode. Its output voltage is controlled by VDD1P0CTRL_TRG.
 *  0b1..DCDC 1P0 works in low power mode. Its output voltage is controlled by VDD1P0CTRL_LP_TRG and its output current is less than 50mA.
 */
#define DCDC_CURRENT_TRG_LP_EN_1P0(x)            (((uint32_t)(((uint32_t)(x)) << DCDC_CURRENT_TRG_LP_EN_1P0_SHIFT)) & DCDC_CURRENT_TRG_LP_EN_1P0_MASK)
/*! @} */

/*! @name FILTER_CNT - FILTER CNT */
/*! @{ */

#define DCDC_FILTER_CNT_FILTER_CNT_CFG_MASK      (0xFFFFFFFFU)
#define DCDC_FILTER_CNT_FILTER_CNT_CFG_SHIFT     (0U)
/*! FILTER_CNT_CFG - FILTER_CNT_CFG */
#define DCDC_FILTER_CNT_FILTER_CNT_CFG(x)        (((uint32_t)(((uint32_t)(x)) << DCDC_FILTER_CNT_FILTER_CNT_CFG_SHIFT)) & DCDC_FILTER_CNT_FILTER_CNT_CFG_MASK)
/*! @} */

/*! @name TRG_0_AUTHEN - TRG_0 Authentication Control */
/*! @{ */

#define DCDC_TRG_0_AUTHEN_TZ_USER_MASK           (0x100U)
#define DCDC_TRG_0_AUTHEN_TZ_USER_SHIFT          (8U)
/*! TZ_USER - Allow user mode write
 *  0b0..TRG_0 registers can only be written in privilege mode.
 *  0b1..TRG_0 registers can be written either in privilege mode or user mode.
 */
#define DCDC_TRG_0_AUTHEN_TZ_USER(x)             (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_0_AUTHEN_TZ_USER_SHIFT)) & DCDC_TRG_0_AUTHEN_TZ_USER_MASK)

#define DCDC_TRG_0_AUTHEN_TZ_NS_MASK             (0x200U)
#define DCDC_TRG_0_AUTHEN_TZ_NS_SHIFT            (9U)
/*! TZ_NS - Allow non-secure mode access
 *  0b0..TRG_0 registers can only be written in secure mode.
 *  0b1..TRG_0 registers can be written either in secure mode or non-secure mode.
 */
#define DCDC_TRG_0_AUTHEN_TZ_NS(x)               (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_0_AUTHEN_TZ_NS_SHIFT)) & DCDC_TRG_0_AUTHEN_TZ_NS_MASK)

#define DCDC_TRG_0_AUTHEN_LOCK_TZ_MASK           (0x800U)
#define DCDC_TRG_0_AUTHEN_LOCK_TZ_SHIFT          (11U)
/*! LOCK_TZ - Lock TZ_NS and TZ_USER
 *  0b0..TZ_NS and TZ_USER value can be changed.
 *  0b1..LOCK_TZ, TZ_NS and TZ_USER value cannot be changed.
 */
#define DCDC_TRG_0_AUTHEN_LOCK_TZ(x)             (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_0_AUTHEN_LOCK_TZ_SHIFT)) & DCDC_TRG_0_AUTHEN_LOCK_TZ_MASK)

#define DCDC_TRG_0_AUTHEN_LOCK_LIST_MASK         (0x8000U)
#define DCDC_TRG_0_AUTHEN_LOCK_LIST_SHIFT        (15U)
/*! LOCK_LIST - White list lock
 *  0b0..WHITE_LIST value can be changed.
 *  0b1..LOCK_LIST and WHITE_LIST value cannot be changed.
 */
#define DCDC_TRG_0_AUTHEN_LOCK_LIST(x)           (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_0_AUTHEN_LOCK_LIST_SHIFT)) & DCDC_TRG_0_AUTHEN_LOCK_LIST_MASK)

#define DCDC_TRG_0_AUTHEN_WHITE_LIST_MASK        (0xFFFF0000U)
#define DCDC_TRG_0_AUTHEN_WHITE_LIST_SHIFT       (16U)
/*! WHITE_LIST - Domain ID white list
 *  0b0000000000000001..Core with domain ID=0 can write TRG_0 registers.
 *  0b0000000000000010..Core with domain ID=1 can write TRG_0 registers.
 *  0b0000000000000100..Core with domain ID=2 can write TRG_0 registers.
 *  0b0000000000001000..Core with domain ID=3 can write TRG_0 registers.
 *  0b0000000000010000..Core with domain ID=4 can write TRG_0 registers.
 *  0b0000000000100000..Core with domain ID=5 can write TRG_0 registers.
 *  0b0000000001000000..Core with domain ID=6 can write TRG_0 registers.
 *  0b0000000010000000..Core with domain ID=7 can write TRG_0 registers.
 *  0b0000000100000000..Core with domain ID=8 can write TRG_0 registers.
 *  0b0000001000000000..Core with domain ID=9 can write TRG_0 registers.
 *  0b0000010000000000..Core with domain ID=10 can write TRG_0 registers.
 *  0b0000100000000000..Core with domain ID=11 can write TRG_0 registers.
 *  0b0001000000000000..Core with domain ID=12 can write TRG_0 registers.
 *  0b0010000000000000..Core with domain ID=13 can write TRG_0 registers.
 *  0b0100000000000000..Core with domain ID=14 can write TRG_0 registers.
 *  0b1000000000000000..Core with domain ID=15 can write TRG_0 registers.
 */
#define DCDC_TRG_0_AUTHEN_WHITE_LIST(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_0_AUTHEN_WHITE_LIST_SHIFT)) & DCDC_TRG_0_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name TRG_SW_0 - Target SW Control for CORE 0 */
/*! @{ */

#define DCDC_TRG_SW_0_VDD1P0CTRL_TRG_MASK        (0x1FU)
#define DCDC_TRG_SW_0_VDD1P0CTRL_TRG_SHIFT       (0U)
#define DCDC_TRG_SW_0_VDD1P0CTRL_TRG(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_0_VDD1P0CTRL_TRG_SHIFT)) & DCDC_TRG_SW_0_VDD1P0CTRL_TRG_MASK)

#define DCDC_TRG_SW_0_VDD1P8CTRL_TRG_MASK        (0x1F00U)
#define DCDC_TRG_SW_0_VDD1P8CTRL_TRG_SHIFT       (8U)
#define DCDC_TRG_SW_0_VDD1P8CTRL_TRG(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_0_VDD1P8CTRL_TRG_SHIFT)) & DCDC_TRG_SW_0_VDD1P8CTRL_TRG_MASK)

#define DCDC_TRG_SW_0_VDD1P0CTRL_LP_TRG_MASK     (0x1F0000U)
#define DCDC_TRG_SW_0_VDD1P0CTRL_LP_TRG_SHIFT    (16U)
#define DCDC_TRG_SW_0_VDD1P0CTRL_LP_TRG(x)       (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_0_VDD1P0CTRL_LP_TRG_SHIFT)) & DCDC_TRG_SW_0_VDD1P0CTRL_LP_TRG_MASK)

#define DCDC_TRG_SW_0_LP_EN_1P0_MASK             (0x80000000U)
#define DCDC_TRG_SW_0_LP_EN_1P0_SHIFT            (31U)
/*! LP_EN_1P0 - LP_EN_1P0 only controls 1P0. 1P8 is always controlled by VDD1P8CTRL_TRG */
#define DCDC_TRG_SW_0_LP_EN_1P0(x)               (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_0_LP_EN_1P0_SHIFT)) & DCDC_TRG_SW_0_LP_EN_1P0_MASK)
/*! @} */

/*! @name TRG_GPC_0 - Target GPC Control for CORE 0 */
/*! @{ */

#define DCDC_TRG_GPC_0_VDD1P0CTRL_TRG_MASK       (0x1FU)
#define DCDC_TRG_GPC_0_VDD1P0CTRL_TRG_SHIFT      (0U)
#define DCDC_TRG_GPC_0_VDD1P0CTRL_TRG(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_0_VDD1P0CTRL_TRG_SHIFT)) & DCDC_TRG_GPC_0_VDD1P0CTRL_TRG_MASK)

#define DCDC_TRG_GPC_0_VDD1P8CTRL_TRG_MASK       (0x1F00U)
#define DCDC_TRG_GPC_0_VDD1P8CTRL_TRG_SHIFT      (8U)
#define DCDC_TRG_GPC_0_VDD1P8CTRL_TRG(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_0_VDD1P8CTRL_TRG_SHIFT)) & DCDC_TRG_GPC_0_VDD1P8CTRL_TRG_MASK)

#define DCDC_TRG_GPC_0_VDD1P0CTRL_LP_TRG_MASK    (0x1F0000U)
#define DCDC_TRG_GPC_0_VDD1P0CTRL_LP_TRG_SHIFT   (16U)
#define DCDC_TRG_GPC_0_VDD1P0CTRL_LP_TRG(x)      (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_0_VDD1P0CTRL_LP_TRG_SHIFT)) & DCDC_TRG_GPC_0_VDD1P0CTRL_LP_TRG_MASK)

#define DCDC_TRG_GPC_0_LP_EN_1P0_MASK            (0x80000000U)
#define DCDC_TRG_GPC_0_LP_EN_1P0_SHIFT           (31U)
/*! LP_EN_1P0 - LP_EN_1P0 only controls 1P0. 1P8 is always controlled by VDD1P8CTRL_TRG */
#define DCDC_TRG_GPC_0_LP_EN_1P0(x)              (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_0_LP_EN_1P0_SHIFT)) & DCDC_TRG_GPC_0_LP_EN_1P0_MASK)
/*! @} */

/*! @name TRG_1_AUTHEN - TRG_1 Authentication Control */
/*! @{ */

#define DCDC_TRG_1_AUTHEN_TZ_USER_MASK           (0x100U)
#define DCDC_TRG_1_AUTHEN_TZ_USER_SHIFT          (8U)
/*! TZ_USER - Allow user mode write
 *  0b0..TRG_1 registers can only be written in privilege mode.
 *  0b1..TRG_1 registers can be written either in privilege mode or user mode.
 */
#define DCDC_TRG_1_AUTHEN_TZ_USER(x)             (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_1_AUTHEN_TZ_USER_SHIFT)) & DCDC_TRG_1_AUTHEN_TZ_USER_MASK)

#define DCDC_TRG_1_AUTHEN_TZ_NS_MASK             (0x200U)
#define DCDC_TRG_1_AUTHEN_TZ_NS_SHIFT            (9U)
/*! TZ_NS - Allow non-secure mode access
 *  0b0..TRG_1 registers can only be written in secure mode.
 *  0b1..TRG_1 registers can be written either in secure mode or non-secure mode.
 */
#define DCDC_TRG_1_AUTHEN_TZ_NS(x)               (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_1_AUTHEN_TZ_NS_SHIFT)) & DCDC_TRG_1_AUTHEN_TZ_NS_MASK)

#define DCDC_TRG_1_AUTHEN_LOCK_TZ_MASK           (0x800U)
#define DCDC_TRG_1_AUTHEN_LOCK_TZ_SHIFT          (11U)
/*! LOCK_TZ - Lock TZ_NS and TZ_USER
 *  0b0..TZ_NS and TZ_USER value can be changed.
 *  0b1..LOCK_TZ, TZ_NS and TZ_USER value cannot be changed.
 */
#define DCDC_TRG_1_AUTHEN_LOCK_TZ(x)             (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_1_AUTHEN_LOCK_TZ_SHIFT)) & DCDC_TRG_1_AUTHEN_LOCK_TZ_MASK)

#define DCDC_TRG_1_AUTHEN_LOCK_LIST_MASK         (0x8000U)
#define DCDC_TRG_1_AUTHEN_LOCK_LIST_SHIFT        (15U)
/*! LOCK_LIST - White list lock
 *  0b0..WHITE_LIST value can be changed.
 *  0b1..LOCK_LIST and WHITE_LIST value cannot be changed.
 */
#define DCDC_TRG_1_AUTHEN_LOCK_LIST(x)           (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_1_AUTHEN_LOCK_LIST_SHIFT)) & DCDC_TRG_1_AUTHEN_LOCK_LIST_MASK)

#define DCDC_TRG_1_AUTHEN_WHITE_LIST_MASK        (0xFFFF0000U)
#define DCDC_TRG_1_AUTHEN_WHITE_LIST_SHIFT       (16U)
/*! WHITE_LIST - Domain ID white list
 *  0b0000000000000001..Core with domain ID=0 can write TRG_1 registers.
 *  0b0000000000000010..Core with domain ID=1 can write TRG_1 registers.
 *  0b0000000000000100..Core with domain ID=2 can write TRG_1 registers.
 *  0b0000000000001000..Core with domain ID=3 can write TRG_1 registers.
 *  0b0000000000010000..Core with domain ID=4 can write TRG_1 registers.
 *  0b0000000000100000..Core with domain ID=5 can write TRG_1 registers.
 *  0b0000000001000000..Core with domain ID=6 can write TRG_1 registers.
 *  0b0000000010000000..Core with domain ID=7 can write TRG_1 registers.
 *  0b0000000100000000..Core with domain ID=8 can write TRG_1 registers.
 *  0b0000001000000000..Core with domain ID=9 can write TRG_1 registers.
 *  0b0000010000000000..Core with domain ID=10 can write TRG_1 registers.
 *  0b0000100000000000..Core with domain ID=11 can write TRG_1 registers.
 *  0b0001000000000000..Core with domain ID=12 can write TRG_1 registers.
 *  0b0010000000000000..Core with domain ID=13 can write TRG_1 registers.
 *  0b0100000000000000..Core with domain ID=14 can write TRG_1 registers.
 *  0b1000000000000000..Core with domain ID=15 can write TRG_1 registers.
 */
#define DCDC_TRG_1_AUTHEN_WHITE_LIST(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_1_AUTHEN_WHITE_LIST_SHIFT)) & DCDC_TRG_1_AUTHEN_WHITE_LIST_MASK)
/*! @} */

/*! @name TRG_SW_1 - Target SW Control for CORE 1 */
/*! @{ */

#define DCDC_TRG_SW_1_VDD1P0CTRL_TRG_MASK        (0x1FU)
#define DCDC_TRG_SW_1_VDD1P0CTRL_TRG_SHIFT       (0U)
#define DCDC_TRG_SW_1_VDD1P0CTRL_TRG(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_1_VDD1P0CTRL_TRG_SHIFT)) & DCDC_TRG_SW_1_VDD1P0CTRL_TRG_MASK)

#define DCDC_TRG_SW_1_VDD1P8CTRL_TRG_MASK        (0x1F00U)
#define DCDC_TRG_SW_1_VDD1P8CTRL_TRG_SHIFT       (8U)
#define DCDC_TRG_SW_1_VDD1P8CTRL_TRG(x)          (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_1_VDD1P8CTRL_TRG_SHIFT)) & DCDC_TRG_SW_1_VDD1P8CTRL_TRG_MASK)

#define DCDC_TRG_SW_1_VDD1P0CTRL_LP_TRG_MASK     (0x1F0000U)
#define DCDC_TRG_SW_1_VDD1P0CTRL_LP_TRG_SHIFT    (16U)
#define DCDC_TRG_SW_1_VDD1P0CTRL_LP_TRG(x)       (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_1_VDD1P0CTRL_LP_TRG_SHIFT)) & DCDC_TRG_SW_1_VDD1P0CTRL_LP_TRG_MASK)

#define DCDC_TRG_SW_1_LP_EN_1P0_MASK             (0x80000000U)
#define DCDC_TRG_SW_1_LP_EN_1P0_SHIFT            (31U)
/*! LP_EN_1P0 - LP_EN_1P0 only controls 1P0. 1P8 is always controlled by VDD1P8CTRL_TRG */
#define DCDC_TRG_SW_1_LP_EN_1P0(x)               (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_SW_1_LP_EN_1P0_SHIFT)) & DCDC_TRG_SW_1_LP_EN_1P0_MASK)
/*! @} */

/*! @name TRG_GPC_1 - Target GPC Control for CORE 1 */
/*! @{ */

#define DCDC_TRG_GPC_1_VDD1P0CTRL_TRG_MASK       (0x1FU)
#define DCDC_TRG_GPC_1_VDD1P0CTRL_TRG_SHIFT      (0U)
#define DCDC_TRG_GPC_1_VDD1P0CTRL_TRG(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_1_VDD1P0CTRL_TRG_SHIFT)) & DCDC_TRG_GPC_1_VDD1P0CTRL_TRG_MASK)

#define DCDC_TRG_GPC_1_VDD1P8CTRL_TRG_MASK       (0x1F00U)
#define DCDC_TRG_GPC_1_VDD1P8CTRL_TRG_SHIFT      (8U)
#define DCDC_TRG_GPC_1_VDD1P8CTRL_TRG(x)         (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_1_VDD1P8CTRL_TRG_SHIFT)) & DCDC_TRG_GPC_1_VDD1P8CTRL_TRG_MASK)

#define DCDC_TRG_GPC_1_VDD1P0CTRL_LP_TRG_MASK    (0x1F0000U)
#define DCDC_TRG_GPC_1_VDD1P0CTRL_LP_TRG_SHIFT   (16U)
#define DCDC_TRG_GPC_1_VDD1P0CTRL_LP_TRG(x)      (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_1_VDD1P0CTRL_LP_TRG_SHIFT)) & DCDC_TRG_GPC_1_VDD1P0CTRL_LP_TRG_MASK)

#define DCDC_TRG_GPC_1_LP_EN_1P0_MASK            (0x80000000U)
#define DCDC_TRG_GPC_1_LP_EN_1P0_SHIFT           (31U)
/*! LP_EN_1P0 - LP_EN_1P0 only controls 1P0. 1P8 is always controlled by VDD1P8CTRL_TRG */
#define DCDC_TRG_GPC_1_LP_EN_1P0(x)              (((uint32_t)(((uint32_t)(x)) << DCDC_TRG_GPC_1_LP_EN_1P0_SHIFT)) & DCDC_TRG_GPC_1_LP_EN_1P0_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group DCDC_Register_Masks */


/* DCDC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral DCDC base address */
  #define DCDC_BASE                                (0x54520000u)
  /** Peripheral DCDC base address */
  #define DCDC_BASE_NS                             (0x44520000u)
  /** Peripheral DCDC base pointer */
  #define DCDC                                     ((DCDC_Type *)DCDC_BASE)
  /** Peripheral DCDC base pointer */
  #define DCDC_NS                                  ((DCDC_Type *)DCDC_BASE_NS)
  /** Array initializer of DCDC peripheral base addresses */
  #define DCDC_BASE_ADDRS                          { DCDC_BASE }
  /** Array initializer of DCDC peripheral base pointers */
  #define DCDC_BASE_PTRS                           { DCDC }
  /** Array initializer of DCDC peripheral base addresses */
  #define DCDC_BASE_ADDRS_NS                       { DCDC_BASE_NS }
  /** Array initializer of DCDC peripheral base pointers */
  #define DCDC_BASE_PTRS_NS                        { DCDC_NS }
#else
  /** Peripheral DCDC base address */
  #define DCDC_BASE                                (0x44520000u)
  /** Peripheral DCDC base pointer */
  #define DCDC                                     ((DCDC_Type *)DCDC_BASE)
  /** Array initializer of DCDC peripheral base addresses */
  #define DCDC_BASE_ADDRS                          { DCDC_BASE }
  /** Array initializer of DCDC peripheral base pointers */
  #define DCDC_BASE_PTRS                           { DCDC }
#endif
/** Interrupt vectors for the DCDC peripheral type */
#define DCDC_IRQS                                { DCDC_IRQn }

/*!
 * @}
 */ /* end of group DCDC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- DMA Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DMA_Peripheral_Access_Layer DMA Peripheral Access Layer
 * @{
 */

/** DMA - Register Layout Typedef */
typedef struct {
  __IO uint32_t MP_CSR;                            /**< Management Page Control, offset: 0x0 */
  __I  uint32_t MP_ES;                             /**< Management Page Error Status, offset: 0x4 */
  __I  uint32_t MP_INT;                            /**< Management Page Interrupt Request Status, offset: 0x8 */
  __I  uint32_t MP_HRS;                            /**< Management Page Hardware Request Status, offset: 0xC */
       uint8_t RESERVED_0[240];
  __IO uint32_t CH_GRPRI[32];                      /**< Channel Arbitration Group, array offset: 0x100, array step: 0x4 */
       uint8_t RESERVED_1[65152];
  struct {                                         /* offset: 0x10000, array step: 0x10000 */
    __IO uint32_t CH_CSR;                            /**< Channel Control and Status, array offset: 0x10000, array step: 0x10000 */
    __IO uint32_t CH_ES;                             /**< Channel Error Status, array offset: 0x10004, array step: 0x10000 */
    __IO uint32_t CH_INT;                            /**< Channel Interrupt Status, array offset: 0x10008, array step: 0x10000 */
    __IO uint32_t CH_SBR;                            /**< Channel System Bus, array offset: 0x1000C, array step: 0x10000 */
    __IO uint32_t CH_PRI;                            /**< Channel Priority, array offset: 0x10010, array step: 0x10000 */
    __IO uint32_t CH_MUX;                            /**< Channel Multiplexor Configuration, array offset: 0x10014, array step: 0x10000 */
         uint8_t RESERVED_0[8];
    __IO uint32_t TCD_SADDR;                         /**< TCD Source Address, array offset: 0x10020, array step: 0x10000 */
    __IO uint16_t TCD_SOFF;                          /**< TCD Signed Source Address Offset, array offset: 0x10024, array step: 0x10000 */
    __IO uint16_t TCD_ATTR;                          /**< TCD Transfer Attributes, array offset: 0x10026, array step: 0x10000 */
    union {                                          /* offset: 0x10028, array step: 0x10000 */
      __IO uint32_t TCD_NBYTES_MLOFFNO;                /**< TCD Transfer Size Without Minor Loop Offsets, array offset: 0x10028, array step: 0x10000 */
      __IO uint32_t TCD_NBYTES_MLOFFYES;               /**< TCD Transfer Size with Minor Loop Offsets, array offset: 0x10028, array step: 0x10000 */
    };
    __IO uint32_t TCD_SLAST_SDA;                     /**< TCD Last Source Address Adjustment / Store DADDR Address, array offset: 0x1002C, array step: 0x10000 */
    __IO uint32_t TCD_DADDR;                         /**< TCD Destination Address, array offset: 0x10030, array step: 0x10000 */
    __IO uint16_t TCD_DOFF;                          /**< TCD Signed Destination Address Offset, array offset: 0x10034, array step: 0x10000 */
    union {                                          /* offset: 0x10036, array step: 0x10000 */
      __IO uint16_t TCD_CITER_ELINKNO;                 /**< TCD Current Major Loop Count (Minor Loop Channel Linking Disabled), array offset: 0x10036, array step: 0x10000 */
      __IO uint16_t TCD_CITER_ELINKYES;                /**< TCD Current Major Loop Count (Minor Loop Channel Linking Enabled), array offset: 0x10036, array step: 0x10000 */
    };
    __IO uint32_t TCD_DLAST_SGA;                     /**< TCD Last Destination Address Adjustment / Scatter Gather Address, array offset: 0x10038, array step: 0x10000 */
    __IO uint16_t TCD_CSR;                           /**< TCD Control and Status, array offset: 0x1003C, array step: 0x10000 */
    union {                                          /* offset: 0x1003E, array step: 0x10000 */
      __IO uint16_t TCD_BITER_ELINKNO;                 /**< TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled), array offset: 0x1003E, array step: 0x10000 */
      __IO uint16_t TCD_BITER_ELINKYES;                /**< TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled), array offset: 0x1003E, array step: 0x10000 */
    };
         uint8_t RESERVED_1[65472];
  } CH[32];
} DMA_Type;

/* ----------------------------------------------------------------------------
   -- DMA Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DMA_Register_Masks DMA Register Masks
 * @{
 */

/*! @name MP_CSR - Management Page Control */
/*! @{ */

#define DMA_MP_CSR_EDBG_MASK                     (0x2U)
#define DMA_MP_CSR_EDBG_SHIFT                    (1U)
/*! EDBG - Enable Debug
 *  0b0..Debug mode disabled
 *  0b1..Debug mode is enabled.
 */
#define DMA_MP_CSR_EDBG(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_EDBG_SHIFT)) & DMA_MP_CSR_EDBG_MASK)

#define DMA_MP_CSR_ERCA_MASK                     (0x4U)
#define DMA_MP_CSR_ERCA_SHIFT                    (2U)
/*! ERCA - Enable Round Robin Channel Arbitration
 *  0b0..Round-robin channel arbitration disabled
 *  0b1..Round-robin channel arbitration enabled
 */
#define DMA_MP_CSR_ERCA(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_ERCA_SHIFT)) & DMA_MP_CSR_ERCA_MASK)

#define DMA_MP_CSR_HAE_MASK                      (0x10U)
#define DMA_MP_CSR_HAE_SHIFT                     (4U)
/*! HAE - Halt After Error
 *  0b0..Normal operation
 *  0b1..Any error causes the HALT field to be set to 1
 */
#define DMA_MP_CSR_HAE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_HAE_SHIFT)) & DMA_MP_CSR_HAE_MASK)

#define DMA_MP_CSR_HALT_MASK                     (0x20U)
#define DMA_MP_CSR_HALT_SHIFT                    (5U)
/*! HALT - Halt DMA Operations
 *  0b0..Normal operation
 *  0b1..Stall the start of any new channels
 */
#define DMA_MP_CSR_HALT(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_HALT_SHIFT)) & DMA_MP_CSR_HALT_MASK)

#define DMA_MP_CSR_GCLC_MASK                     (0x40U)
#define DMA_MP_CSR_GCLC_SHIFT                    (6U)
/*! GCLC - Global Channel Linking Control
 *  0b0..Channel linking disabled for all channels
 *  0b1..Channel linking available and controlled by each channel's link settings
 */
#define DMA_MP_CSR_GCLC(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_GCLC_SHIFT)) & DMA_MP_CSR_GCLC_MASK)

#define DMA_MP_CSR_GMRC_MASK                     (0x80U)
#define DMA_MP_CSR_GMRC_SHIFT                    (7U)
/*! GMRC - Global Master ID Replication Control
 *  0b0..Master ID replication disabled for all channels
 *  0b1..Master ID replication available and controlled by each channel's CHn_SBR[EMI] setting
 */
#define DMA_MP_CSR_GMRC(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_GMRC_SHIFT)) & DMA_MP_CSR_GMRC_MASK)

#define DMA_MP_CSR_ECX_MASK                      (0x100U)
#define DMA_MP_CSR_ECX_SHIFT                     (8U)
/*! ECX - Cancel Transfer With Error
 *  0b0..Normal operation
 *  0b1..Cancel the remaining data transfer
 */
#define DMA_MP_CSR_ECX(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_ECX_SHIFT)) & DMA_MP_CSR_ECX_MASK)

#define DMA_MP_CSR_CX_MASK                       (0x200U)
#define DMA_MP_CSR_CX_SHIFT                      (9U)
/*! CX - Cancel Transfer
 *  0b0..Normal operation
 *  0b1..Cancel the remaining data transfer
 */
#define DMA_MP_CSR_CX(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_CX_SHIFT)) & DMA_MP_CSR_CX_MASK)

#define DMA_MP_CSR_ACTIVE_ID_MASK                (0x1F000000U)
#define DMA_MP_CSR_ACTIVE_ID_SHIFT               (24U)
/*! ACTIVE_ID - Active Channel ID */
#define DMA_MP_CSR_ACTIVE_ID(x)                  (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_ACTIVE_ID_SHIFT)) & DMA_MP_CSR_ACTIVE_ID_MASK)

#define DMA_MP_CSR_ACTIVE_MASK                   (0x80000000U)
#define DMA_MP_CSR_ACTIVE_SHIFT                  (31U)
/*! ACTIVE - DMA Active Status
 *  0b0..eDMA is idle
 *  0b1..eDMA is executing a channel
 */
#define DMA_MP_CSR_ACTIVE(x)                     (((uint32_t)(((uint32_t)(x)) << DMA_MP_CSR_ACTIVE_SHIFT)) & DMA_MP_CSR_ACTIVE_MASK)
/*! @} */

/*! @name MP_ES - Management Page Error Status */
/*! @{ */

#define DMA_MP_ES_DBE_MASK                       (0x1U)
#define DMA_MP_ES_DBE_SHIFT                      (0U)
/*! DBE - Destination Bus Error
 *  0b0..No destination bus error
 *  0b1..Last recorded error was a bus error on a destination write
 */
#define DMA_MP_ES_DBE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_DBE_SHIFT)) & DMA_MP_ES_DBE_MASK)

#define DMA_MP_ES_SBE_MASK                       (0x2U)
#define DMA_MP_ES_SBE_SHIFT                      (1U)
/*! SBE - Source Bus Error
 *  0b0..No source bus error
 *  0b1..Last recorded error was a bus error on a source read
 */
#define DMA_MP_ES_SBE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_SBE_SHIFT)) & DMA_MP_ES_SBE_MASK)

#define DMA_MP_ES_SGE_MASK                       (0x4U)
#define DMA_MP_ES_SGE_SHIFT                      (2U)
/*! SGE - Scatter/Gather Configuration Error
 *  0b0..No scatter/gather configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_DLAST_SGA field
 */
#define DMA_MP_ES_SGE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_SGE_SHIFT)) & DMA_MP_ES_SGE_MASK)

#define DMA_MP_ES_NCE_MASK                       (0x8U)
#define DMA_MP_ES_NCE_SHIFT                      (3U)
/*! NCE - NBYTES/CITER Configuration Error
 *  0b0..No NBYTES/CITER configuration error
 *  0b1..The last recorded error was NBYTES equal to zero or a CITER not equal to BITER error
 */
#define DMA_MP_ES_NCE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_NCE_SHIFT)) & DMA_MP_ES_NCE_MASK)

#define DMA_MP_ES_DOE_MASK                       (0x10U)
#define DMA_MP_ES_DOE_SHIFT                      (4U)
/*! DOE - Destination Offset Error
 *  0b0..No destination offset configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_DOFF field
 */
#define DMA_MP_ES_DOE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_DOE_SHIFT)) & DMA_MP_ES_DOE_MASK)

#define DMA_MP_ES_DAE_MASK                       (0x20U)
#define DMA_MP_ES_DAE_SHIFT                      (5U)
/*! DAE - Destination Address Error
 *  0b0..No destination address configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_DADDR field
 */
#define DMA_MP_ES_DAE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_DAE_SHIFT)) & DMA_MP_ES_DAE_MASK)

#define DMA_MP_ES_SOE_MASK                       (0x40U)
#define DMA_MP_ES_SOE_SHIFT                      (6U)
/*! SOE - Source Offset Error
 *  0b0..No source offset configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_SOFF field
 */
#define DMA_MP_ES_SOE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_SOE_SHIFT)) & DMA_MP_ES_SOE_MASK)

#define DMA_MP_ES_SAE_MASK                       (0x80U)
#define DMA_MP_ES_SAE_SHIFT                      (7U)
/*! SAE - Source Address Error
 *  0b0..No source address configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_SADDR field
 */
#define DMA_MP_ES_SAE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_SAE_SHIFT)) & DMA_MP_ES_SAE_MASK)

#define DMA_MP_ES_ECX_MASK                       (0x100U)
#define DMA_MP_ES_ECX_SHIFT                      (8U)
/*! ECX - Transfer Canceled
 *  0b0..No canceled transfers
 *  0b1..Last recorded entry was a canceled transfer by the error cancel transfer input
 */
#define DMA_MP_ES_ECX(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_ECX_SHIFT)) & DMA_MP_ES_ECX_MASK)

#define DMA_MP_ES_ERRCHN_MASK                    (0x1F000000U)
#define DMA_MP_ES_ERRCHN_SHIFT                   (24U)
/*! ERRCHN - Error Channel Number or Canceled Channel Number */
#define DMA_MP_ES_ERRCHN(x)                      (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_ERRCHN_SHIFT)) & DMA_MP_ES_ERRCHN_MASK)

#define DMA_MP_ES_VLD_MASK                       (0x80000000U)
#define DMA_MP_ES_VLD_SHIFT                      (31U)
/*! VLD - Valid
 *  0b0..No CHn_ES[ERR] fields are set to 1
 *  0b1..At least one CHn_ES[ERR] field is set to 1, indicating a valid error exists that software has not cleared
 */
#define DMA_MP_ES_VLD(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_MP_ES_VLD_SHIFT)) & DMA_MP_ES_VLD_MASK)
/*! @} */

/*! @name MP_INT - Management Page Interrupt Request Status */
/*! @{ */

#define DMA_MP_INT_INT_MASK                      (0xFFFFFFFFU)
#define DMA_MP_INT_INT_SHIFT                     (0U)
/*! INT - Interrupt Request Status */
#define DMA_MP_INT_INT(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_MP_INT_INT_SHIFT)) & DMA_MP_INT_INT_MASK)
/*! @} */

/*! @name MP_HRS - Management Page Hardware Request Status */
/*! @{ */

#define DMA_MP_HRS_HRS_MASK                      (0xFFFFFFFFU)
#define DMA_MP_HRS_HRS_SHIFT                     (0U)
/*! HRS - Hardware Request Status */
#define DMA_MP_HRS_HRS(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_MP_HRS_HRS_SHIFT)) & DMA_MP_HRS_HRS_MASK)
/*! @} */

/*! @name CH_GRPRI - Channel Arbitration Group */
/*! @{ */

#define DMA_CH_GRPRI_GRPRI_MASK                  (0x1FU)
#define DMA_CH_GRPRI_GRPRI_SHIFT                 (0U)
/*! GRPRI - Arbitration Group For Channel n */
#define DMA_CH_GRPRI_GRPRI(x)                    (((uint32_t)(((uint32_t)(x)) << DMA_CH_GRPRI_GRPRI_SHIFT)) & DMA_CH_GRPRI_GRPRI_MASK)
/*! @} */

/* The count of DMA_CH_GRPRI */
#define DMA_CH_GRPRI_COUNT                       (32U)

/*! @name CH_CSR - Channel Control and Status */
/*! @{ */

#define DMA_CH_CSR_ERQ_MASK                      (0x1U)
#define DMA_CH_CSR_ERQ_SHIFT                     (0U)
/*! ERQ - Enable DMA Request
 *  0b0..DMA hardware request signal for corresponding channel disabled
 *  0b1..DMA hardware request signal for corresponding channel enabled
 */
#define DMA_CH_CSR_ERQ(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_CSR_ERQ_SHIFT)) & DMA_CH_CSR_ERQ_MASK)

#define DMA_CH_CSR_EARQ_MASK                     (0x2U)
#define DMA_CH_CSR_EARQ_SHIFT                    (1U)
/*! EARQ - Enable Asynchronous DMA Request
 *  0b0..Disable asynchronous DMA request for the channel
 *  0b1..Enable asynchronous DMA request for the channel
 */
#define DMA_CH_CSR_EARQ(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_CH_CSR_EARQ_SHIFT)) & DMA_CH_CSR_EARQ_MASK)

#define DMA_CH_CSR_EEI_MASK                      (0x4U)
#define DMA_CH_CSR_EEI_SHIFT                     (2U)
/*! EEI - Enable Error Interrupt
 *  0b0..Error signal for corresponding channel does not generate error interrupt
 *  0b1..Assertion of error signal for corresponding channel generates error interrupt request
 */
#define DMA_CH_CSR_EEI(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_CSR_EEI_SHIFT)) & DMA_CH_CSR_EEI_MASK)

#define DMA_CH_CSR_EBW_MASK                      (0x8U)
#define DMA_CH_CSR_EBW_SHIFT                     (3U)
/*! EBW - Enable Buffered Writes
 *  0b0..Buffered writes on system bus disabled
 *  0b1..Buffered writes on system bus enabled
 */
#define DMA_CH_CSR_EBW(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_CSR_EBW_SHIFT)) & DMA_CH_CSR_EBW_MASK)

#define DMA_CH_CSR_DONE_MASK                     (0x40000000U)
#define DMA_CH_CSR_DONE_SHIFT                    (30U)
/*! DONE - Channel Done */
#define DMA_CH_CSR_DONE(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_CH_CSR_DONE_SHIFT)) & DMA_CH_CSR_DONE_MASK)

#define DMA_CH_CSR_ACTIVE_MASK                   (0x80000000U)
#define DMA_CH_CSR_ACTIVE_SHIFT                  (31U)
/*! ACTIVE - Channel Active */
#define DMA_CH_CSR_ACTIVE(x)                     (((uint32_t)(((uint32_t)(x)) << DMA_CH_CSR_ACTIVE_SHIFT)) & DMA_CH_CSR_ACTIVE_MASK)
/*! @} */

/* The count of DMA_CH_CSR */
#define DMA_CH_CSR_COUNT                         (32U)

/*! @name CH_ES - Channel Error Status */
/*! @{ */

#define DMA_CH_ES_DBE_MASK                       (0x1U)
#define DMA_CH_ES_DBE_SHIFT                      (0U)
/*! DBE - Destination Bus Error
 *  0b0..No destination bus error
 *  0b1..Last recorded error was bus error on destination write
 */
#define DMA_CH_ES_DBE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_DBE_SHIFT)) & DMA_CH_ES_DBE_MASK)

#define DMA_CH_ES_SBE_MASK                       (0x2U)
#define DMA_CH_ES_SBE_SHIFT                      (1U)
/*! SBE - Source Bus Error
 *  0b0..No source bus error
 *  0b1..Last recorded error was bus error on source read
 */
#define DMA_CH_ES_SBE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_SBE_SHIFT)) & DMA_CH_ES_SBE_MASK)

#define DMA_CH_ES_SGE_MASK                       (0x4U)
#define DMA_CH_ES_SGE_SHIFT                      (2U)
/*! SGE - Scatter/Gather Configuration Error
 *  0b0..No scatter/gather configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_DLAST_SGA field
 */
#define DMA_CH_ES_SGE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_SGE_SHIFT)) & DMA_CH_ES_SGE_MASK)

#define DMA_CH_ES_NCE_MASK                       (0x8U)
#define DMA_CH_ES_NCE_SHIFT                      (3U)
/*! NCE - NBYTES/CITER Configuration Error
 *  0b0..No NBYTES/CITER configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_NBYTES or TCDn_CITER fields
 */
#define DMA_CH_ES_NCE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_NCE_SHIFT)) & DMA_CH_ES_NCE_MASK)

#define DMA_CH_ES_DOE_MASK                       (0x10U)
#define DMA_CH_ES_DOE_SHIFT                      (4U)
/*! DOE - Destination Offset Error
 *  0b0..No destination offset configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_DOFF field
 */
#define DMA_CH_ES_DOE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_DOE_SHIFT)) & DMA_CH_ES_DOE_MASK)

#define DMA_CH_ES_DAE_MASK                       (0x20U)
#define DMA_CH_ES_DAE_SHIFT                      (5U)
/*! DAE - Destination Address Error
 *  0b0..No destination address configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_DADDR field
 */
#define DMA_CH_ES_DAE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_DAE_SHIFT)) & DMA_CH_ES_DAE_MASK)

#define DMA_CH_ES_SOE_MASK                       (0x40U)
#define DMA_CH_ES_SOE_SHIFT                      (6U)
/*! SOE - Source Offset Error
 *  0b0..No source offset configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_SOFF field
 */
#define DMA_CH_ES_SOE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_SOE_SHIFT)) & DMA_CH_ES_SOE_MASK)

#define DMA_CH_ES_SAE_MASK                       (0x80U)
#define DMA_CH_ES_SAE_SHIFT                      (7U)
/*! SAE - Source Address Error
 *  0b0..No source address configuration error
 *  0b1..Last recorded error was a configuration error detected in the TCDn_SADDR field
 */
#define DMA_CH_ES_SAE(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_SAE_SHIFT)) & DMA_CH_ES_SAE_MASK)

#define DMA_CH_ES_ERR_MASK                       (0x80000000U)
#define DMA_CH_ES_ERR_SHIFT                      (31U)
/*! ERR - Error In Channel
 *  0b0..An error in this channel has not occurred
 *  0b1..An error in this channel has occurred
 */
#define DMA_CH_ES_ERR(x)                         (((uint32_t)(((uint32_t)(x)) << DMA_CH_ES_ERR_SHIFT)) & DMA_CH_ES_ERR_MASK)
/*! @} */

/* The count of DMA_CH_ES */
#define DMA_CH_ES_COUNT                          (32U)

/*! @name CH_INT - Channel Interrupt Status */
/*! @{ */

#define DMA_CH_INT_INT_MASK                      (0x1U)
#define DMA_CH_INT_INT_SHIFT                     (0U)
/*! INT - Interrupt Request
 *  0b0..Interrupt request for corresponding channel cleared
 *  0b1..Interrupt request for corresponding channel active
 */
#define DMA_CH_INT_INT(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_INT_INT_SHIFT)) & DMA_CH_INT_INT_MASK)
/*! @} */

/* The count of DMA_CH_INT */
#define DMA_CH_INT_COUNT                         (32U)

/*! @name CH_SBR - Channel System Bus */
/*! @{ */

#define DMA_CH_SBR_MID_MASK                      (0xFU)
#define DMA_CH_SBR_MID_SHIFT                     (0U)
/*! MID - Master ID */
#define DMA_CH_SBR_MID(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_SBR_MID_SHIFT)) & DMA_CH_SBR_MID_MASK)

#define DMA_CH_SBR_SEC_MASK                      (0x4000U)
#define DMA_CH_SBR_SEC_SHIFT                     (14U)
/*! SEC - Security Level
 *  0b0..Nonsecure protection level for DMA transfers
 *  0b1..Secure protection level for DMA transfers
 */
#define DMA_CH_SBR_SEC(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_SBR_SEC_SHIFT)) & DMA_CH_SBR_SEC_MASK)

#define DMA_CH_SBR_PAL_MASK                      (0x8000U)
#define DMA_CH_SBR_PAL_SHIFT                     (15U)
/*! PAL - Privileged Access Level
 *  0b0..User protection level for DMA transfers
 *  0b1..Privileged protection level for DMA transfers
 */
#define DMA_CH_SBR_PAL(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_SBR_PAL_SHIFT)) & DMA_CH_SBR_PAL_MASK)

#define DMA_CH_SBR_EMI_MASK                      (0x10000U)
#define DMA_CH_SBR_EMI_SHIFT                     (16U)
/*! EMI - Enable Master ID Replication
 *  0b0..Master ID replication is disabled
 *  0b1..Master ID replication is enabled
 */
#define DMA_CH_SBR_EMI(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_SBR_EMI_SHIFT)) & DMA_CH_SBR_EMI_MASK)

#define DMA_CH_SBR_ATTR_MASK                     (0x7E0000U)
#define DMA_CH_SBR_ATTR_SHIFT                    (17U)
/*! ATTR - Attribute Output */
#define DMA_CH_SBR_ATTR(x)                       (((uint32_t)(((uint32_t)(x)) << DMA_CH_SBR_ATTR_SHIFT)) & DMA_CH_SBR_ATTR_MASK)
/*! @} */

/* The count of DMA_CH_SBR */
#define DMA_CH_SBR_COUNT                         (32U)

/*! @name CH_PRI - Channel Priority */
/*! @{ */

#define DMA_CH_PRI_APL_MASK                      (0x7U)
#define DMA_CH_PRI_APL_SHIFT                     (0U)
/*! APL - Arbitration Priority Level */
#define DMA_CH_PRI_APL(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_PRI_APL_SHIFT)) & DMA_CH_PRI_APL_MASK)

#define DMA_CH_PRI_DPA_MASK                      (0x40000000U)
#define DMA_CH_PRI_DPA_SHIFT                     (30U)
/*! DPA - Disable Preempt Ability
 *  0b0..Channel can suspend a lower-priority channel
 *  0b1..Channel cannot suspend any other channel, regardless of channel priority
 */
#define DMA_CH_PRI_DPA(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_PRI_DPA_SHIFT)) & DMA_CH_PRI_DPA_MASK)

#define DMA_CH_PRI_ECP_MASK                      (0x80000000U)
#define DMA_CH_PRI_ECP_SHIFT                     (31U)
/*! ECP - Enable Channel Preemption
 *  0b0..Channel cannot be suspended by a higher-priority channel's service request
 *  0b1..Channel can be temporarily suspended by a higher-priority channel's service request
 */
#define DMA_CH_PRI_ECP(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_PRI_ECP_SHIFT)) & DMA_CH_PRI_ECP_MASK)
/*! @} */

/* The count of DMA_CH_PRI */
#define DMA_CH_PRI_COUNT                         (32U)

/*! @name CH_MUX - Channel Multiplexor Configuration */
/*! @{ */

#define DMA_CH_MUX_SRC_MASK                      (0x3FU)
#define DMA_CH_MUX_SRC_SHIFT                     (0U)
/*! SRC - Service Request Source */
#define DMA_CH_MUX_SRC(x)                        (((uint32_t)(((uint32_t)(x)) << DMA_CH_MUX_SRC_SHIFT)) & DMA_CH_MUX_SRC_MASK)
/*! @} */

/* The count of DMA_CH_MUX */
#define DMA_CH_MUX_COUNT                         (32U)

/*! @name TCD_SADDR - TCD Source Address */
/*! @{ */

#define DMA_TCD_SADDR_SADDR_MASK                 (0xFFFFFFFFU)
#define DMA_TCD_SADDR_SADDR_SHIFT                (0U)
/*! SADDR - Source Address */
#define DMA_TCD_SADDR_SADDR(x)                   (((uint32_t)(((uint32_t)(x)) << DMA_TCD_SADDR_SADDR_SHIFT)) & DMA_TCD_SADDR_SADDR_MASK)
/*! @} */

/* The count of DMA_TCD_SADDR */
#define DMA_TCD_SADDR_COUNT                      (32U)

/*! @name TCD_SOFF - TCD Signed Source Address Offset */
/*! @{ */

#define DMA_TCD_SOFF_SOFF_MASK                   (0xFFFFU)
#define DMA_TCD_SOFF_SOFF_SHIFT                  (0U)
/*! SOFF - Source Address Signed Offset */
#define DMA_TCD_SOFF_SOFF(x)                     (((uint16_t)(((uint16_t)(x)) << DMA_TCD_SOFF_SOFF_SHIFT)) & DMA_TCD_SOFF_SOFF_MASK)
/*! @} */

/* The count of DMA_TCD_SOFF */
#define DMA_TCD_SOFF_COUNT                       (32U)

/*! @name TCD_ATTR - TCD Transfer Attributes */
/*! @{ */

#define DMA_TCD_ATTR_DSIZE_MASK                  (0x7U)
#define DMA_TCD_ATTR_DSIZE_SHIFT                 (0U)
/*! DSIZE - Destination Data Transfer Size */
#define DMA_TCD_ATTR_DSIZE(x)                    (((uint16_t)(((uint16_t)(x)) << DMA_TCD_ATTR_DSIZE_SHIFT)) & DMA_TCD_ATTR_DSIZE_MASK)

#define DMA_TCD_ATTR_DMOD_MASK                   (0xF8U)
#define DMA_TCD_ATTR_DMOD_SHIFT                  (3U)
/*! DMOD - Destination Address Modulo */
#define DMA_TCD_ATTR_DMOD(x)                     (((uint16_t)(((uint16_t)(x)) << DMA_TCD_ATTR_DMOD_SHIFT)) & DMA_TCD_ATTR_DMOD_MASK)

#define DMA_TCD_ATTR_SSIZE_MASK                  (0x700U)
#define DMA_TCD_ATTR_SSIZE_SHIFT                 (8U)
/*! SSIZE - Source Data Transfer Size
 *  0b000..8-bit
 *  0b001..16-bit
 *  0b010..32-bit
 *  0b011..64-bit
 *  0b100..16-byte
 *  0b101..32-byte
 *  0b110..64-byte
 *  0b111..
 */
#define DMA_TCD_ATTR_SSIZE(x)                    (((uint16_t)(((uint16_t)(x)) << DMA_TCD_ATTR_SSIZE_SHIFT)) & DMA_TCD_ATTR_SSIZE_MASK)

#define DMA_TCD_ATTR_SMOD_MASK                   (0xF800U)
#define DMA_TCD_ATTR_SMOD_SHIFT                  (11U)
/*! SMOD - Source Address Modulo
 *  0b00000..Source address modulo feature disabled
 *  0b00001..Source address modulo feature enabled for any non-zero value [1-31]
 */
#define DMA_TCD_ATTR_SMOD(x)                     (((uint16_t)(((uint16_t)(x)) << DMA_TCD_ATTR_SMOD_SHIFT)) & DMA_TCD_ATTR_SMOD_MASK)
/*! @} */

/* The count of DMA_TCD_ATTR */
#define DMA_TCD_ATTR_COUNT                       (32U)

/*! @name TCD_NBYTES_MLOFFNO - TCD Transfer Size Without Minor Loop Offsets */
/*! @{ */

#define DMA_TCD_NBYTES_MLOFFNO_NBYTES_MASK       (0x3FFFFFFFU)
#define DMA_TCD_NBYTES_MLOFFNO_NBYTES_SHIFT      (0U)
/*! NBYTES - Number of Bytes To Transfer Per Service Request */
#define DMA_TCD_NBYTES_MLOFFNO_NBYTES(x)         (((uint32_t)(((uint32_t)(x)) << DMA_TCD_NBYTES_MLOFFNO_NBYTES_SHIFT)) & DMA_TCD_NBYTES_MLOFFNO_NBYTES_MASK)

#define DMA_TCD_NBYTES_MLOFFNO_DMLOE_MASK        (0x40000000U)
#define DMA_TCD_NBYTES_MLOFFNO_DMLOE_SHIFT       (30U)
/*! DMLOE - Destination Minor Loop Offset Enable
 *  0b0..Minor loop offset not applied to DADDR
 *  0b1..Minor loop offset applied to DADDR
 */
#define DMA_TCD_NBYTES_MLOFFNO_DMLOE(x)          (((uint32_t)(((uint32_t)(x)) << DMA_TCD_NBYTES_MLOFFNO_DMLOE_SHIFT)) & DMA_TCD_NBYTES_MLOFFNO_DMLOE_MASK)

#define DMA_TCD_NBYTES_MLOFFNO_SMLOE_MASK        (0x80000000U)
#define DMA_TCD_NBYTES_MLOFFNO_SMLOE_SHIFT       (31U)
/*! SMLOE - Source Minor Loop Offset Enable
 *  0b0..Minor loop offset not applied to SADDR
 *  0b1..Minor loop offset applied to SADDR
 */
#define DMA_TCD_NBYTES_MLOFFNO_SMLOE(x)          (((uint32_t)(((uint32_t)(x)) << DMA_TCD_NBYTES_MLOFFNO_SMLOE_SHIFT)) & DMA_TCD_NBYTES_MLOFFNO_SMLOE_MASK)
/*! @} */

/* The count of DMA_TCD_NBYTES_MLOFFNO */
#define DMA_TCD_NBYTES_MLOFFNO_COUNT             (32U)

/*! @name TCD_NBYTES_MLOFFYES - TCD Transfer Size with Minor Loop Offsets */
/*! @{ */

#define DMA_TCD_NBYTES_MLOFFYES_NBYTES_MASK      (0x3FFU)
#define DMA_TCD_NBYTES_MLOFFYES_NBYTES_SHIFT     (0U)
/*! NBYTES - Number of Bytes To Transfer Per Service Request */
#define DMA_TCD_NBYTES_MLOFFYES_NBYTES(x)        (((uint32_t)(((uint32_t)(x)) << DMA_TCD_NBYTES_MLOFFYES_NBYTES_SHIFT)) & DMA_TCD_NBYTES_MLOFFYES_NBYTES_MASK)

#define DMA_TCD_NBYTES_MLOFFYES_MLOFF_MASK       (0x3FFFFC00U)
#define DMA_TCD_NBYTES_MLOFFYES_MLOFF_SHIFT      (10U)
/*! MLOFF - Minor Loop Offset */
#define DMA_TCD_NBYTES_MLOFFYES_MLOFF(x)         (((uint32_t)(((uint32_t)(x)) << DMA_TCD_NBYTES_MLOFFYES_MLOFF_SHIFT)) & DMA_TCD_NBYTES_MLOFFYES_MLOFF_MASK)

#define DMA_TCD_NBYTES_MLOFFYES_DMLOE_MASK       (0x40000000U)
#define DMA_TCD_NBYTES_MLOFFYES_DMLOE_SHIFT      (30U)
/*! DMLOE - Destination Minor Loop Offset Enable
 *  0b0..Minor loop offset not applied to DADDR
 *  0b1..Minor loop offset applied to DADDR
 */
#define DMA_TCD_NBYTES_MLOFFYES_DMLOE(x)         (((uint32_t)(((uint32_t)(x)) << DMA_TCD_NBYTES_MLOFFYES_DMLOE_SHIFT)) & DMA_TCD_NBYTES_MLOFFYES_DMLOE_MASK)

#define DMA_TCD_NBYTES_MLOFFYES_SMLOE_MASK       (0x80000000U)
#define DMA_TCD_NBYTES_MLOFFYES_SMLOE_SHIFT      (31U)
/*! SMLOE - Source Minor Loop Offset Enable
 *  0b0..Minor loop offset not applied to SADDR
 *  0b1..Minor loop offset applied to SADDR
 */
#define DMA_TCD_NBYTES_MLOFFYES_SMLOE(x)         (((uint32_t)(((uint32_t)(x)) << DMA_TCD_NBYTES_MLOFFYES_SMLOE_SHIFT)) & DMA_TCD_NBYTES_MLOFFYES_SMLOE_MASK)
/*! @} */

/* The count of DMA_TCD_NBYTES_MLOFFYES */
#define DMA_TCD_NBYTES_MLOFFYES_COUNT            (32U)

/*! @name TCD_SLAST_SDA - TCD Last Source Address Adjustment / Store DADDR Address */
/*! @{ */

#define DMA_TCD_SLAST_SDA_SLAST_SDA_MASK         (0xFFFFFFFFU)
#define DMA_TCD_SLAST_SDA_SLAST_SDA_SHIFT        (0U)
/*! SLAST_SDA - Last Source Address Adjustment / Store DADDR Address */
#define DMA_TCD_SLAST_SDA_SLAST_SDA(x)           (((uint32_t)(((uint32_t)(x)) << DMA_TCD_SLAST_SDA_SLAST_SDA_SHIFT)) & DMA_TCD_SLAST_SDA_SLAST_SDA_MASK)
/*! @} */

/* The count of DMA_TCD_SLAST_SDA */
#define DMA_TCD_SLAST_SDA_COUNT                  (32U)

/*! @name TCD_DADDR - TCD Destination Address */
/*! @{ */

#define DMA_TCD_DADDR_DADDR_MASK                 (0xFFFFFFFFU)
#define DMA_TCD_DADDR_DADDR_SHIFT                (0U)
/*! DADDR - Destination Address */
#define DMA_TCD_DADDR_DADDR(x)                   (((uint32_t)(((uint32_t)(x)) << DMA_TCD_DADDR_DADDR_SHIFT)) & DMA_TCD_DADDR_DADDR_MASK)
/*! @} */

/* The count of DMA_TCD_DADDR */
#define DMA_TCD_DADDR_COUNT                      (32U)

/*! @name TCD_DOFF - TCD Signed Destination Address Offset */
/*! @{ */

#define DMA_TCD_DOFF_DOFF_MASK                   (0xFFFFU)
#define DMA_TCD_DOFF_DOFF_SHIFT                  (0U)
/*! DOFF - Destination Address Signed Offset */
#define DMA_TCD_DOFF_DOFF(x)                     (((uint16_t)(((uint16_t)(x)) << DMA_TCD_DOFF_DOFF_SHIFT)) & DMA_TCD_DOFF_DOFF_MASK)
/*! @} */

/* The count of DMA_TCD_DOFF */
#define DMA_TCD_DOFF_COUNT                       (32U)

/*! @name TCD_CITER_ELINKNO - TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) */
/*! @{ */

#define DMA_TCD_CITER_ELINKNO_CITER_MASK         (0x7FFFU)
#define DMA_TCD_CITER_ELINKNO_CITER_SHIFT        (0U)
/*! CITER - Current Major Iteration Count */
#define DMA_TCD_CITER_ELINKNO_CITER(x)           (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CITER_ELINKNO_CITER_SHIFT)) & DMA_TCD_CITER_ELINKNO_CITER_MASK)

#define DMA_TCD_CITER_ELINKNO_ELINK_MASK         (0x8000U)
#define DMA_TCD_CITER_ELINKNO_ELINK_SHIFT        (15U)
/*! ELINK - Enable Link
 *  0b0..Channel-to-channel linking disabled
 *  0b1..Channel-to-channel linking enabled
 */
#define DMA_TCD_CITER_ELINKNO_ELINK(x)           (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CITER_ELINKNO_ELINK_SHIFT)) & DMA_TCD_CITER_ELINKNO_ELINK_MASK)
/*! @} */

/* The count of DMA_TCD_CITER_ELINKNO */
#define DMA_TCD_CITER_ELINKNO_COUNT              (32U)

/*! @name TCD_CITER_ELINKYES - TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) */
/*! @{ */

#define DMA_TCD_CITER_ELINKYES_CITER_MASK        (0x1FFU)
#define DMA_TCD_CITER_ELINKYES_CITER_SHIFT       (0U)
/*! CITER - Current Major Iteration Count */
#define DMA_TCD_CITER_ELINKYES_CITER(x)          (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CITER_ELINKYES_CITER_SHIFT)) & DMA_TCD_CITER_ELINKYES_CITER_MASK)

#define DMA_TCD_CITER_ELINKYES_LINKCH_MASK       (0x3E00U)
#define DMA_TCD_CITER_ELINKYES_LINKCH_SHIFT      (9U)
/*! LINKCH - Minor Loop Link Channel Number */
#define DMA_TCD_CITER_ELINKYES_LINKCH(x)         (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CITER_ELINKYES_LINKCH_SHIFT)) & DMA_TCD_CITER_ELINKYES_LINKCH_MASK)

#define DMA_TCD_CITER_ELINKYES_ELINK_MASK        (0x8000U)
#define DMA_TCD_CITER_ELINKYES_ELINK_SHIFT       (15U)
/*! ELINK - Enable Link
 *  0b0..Channel-to-channel linking disabled
 *  0b1..Channel-to-channel linking enabled
 */
#define DMA_TCD_CITER_ELINKYES_ELINK(x)          (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CITER_ELINKYES_ELINK_SHIFT)) & DMA_TCD_CITER_ELINKYES_ELINK_MASK)
/*! @} */

/* The count of DMA_TCD_CITER_ELINKYES */
#define DMA_TCD_CITER_ELINKYES_COUNT             (32U)

/*! @name TCD_DLAST_SGA - TCD Last Destination Address Adjustment / Scatter Gather Address */
/*! @{ */

#define DMA_TCD_DLAST_SGA_DLAST_SGA_MASK         (0xFFFFFFFFU)
#define DMA_TCD_DLAST_SGA_DLAST_SGA_SHIFT        (0U)
/*! DLAST_SGA - Last Destination Address Adjustment / Scatter Gather Address */
#define DMA_TCD_DLAST_SGA_DLAST_SGA(x)           (((uint32_t)(((uint32_t)(x)) << DMA_TCD_DLAST_SGA_DLAST_SGA_SHIFT)) & DMA_TCD_DLAST_SGA_DLAST_SGA_MASK)
/*! @} */

/* The count of DMA_TCD_DLAST_SGA */
#define DMA_TCD_DLAST_SGA_COUNT                  (32U)

/*! @name TCD_CSR - TCD Control and Status */
/*! @{ */

#define DMA_TCD_CSR_START_MASK                   (0x1U)
#define DMA_TCD_CSR_START_SHIFT                  (0U)
/*! START - Channel Start
 *  0b0..Channel not explicitly started
 *  0b1..Channel explicitly started via a software-initiated service request
 */
#define DMA_TCD_CSR_START(x)                     (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_START_SHIFT)) & DMA_TCD_CSR_START_MASK)

#define DMA_TCD_CSR_INTMAJOR_MASK                (0x2U)
#define DMA_TCD_CSR_INTMAJOR_SHIFT               (1U)
/*! INTMAJOR - Enable Interrupt If Major count complete
 *  0b0..End-of-major loop interrupt disabled
 *  0b1..End-of-major loop interrupt enabled
 */
#define DMA_TCD_CSR_INTMAJOR(x)                  (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_INTMAJOR_SHIFT)) & DMA_TCD_CSR_INTMAJOR_MASK)

#define DMA_TCD_CSR_INTHALF_MASK                 (0x4U)
#define DMA_TCD_CSR_INTHALF_SHIFT                (2U)
/*! INTHALF - Enable Interrupt If Major Counter Half-complete
 *  0b0..Halfway point interrupt disabled
 *  0b1..Halfway point interrupt enabled
 */
#define DMA_TCD_CSR_INTHALF(x)                   (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_INTHALF_SHIFT)) & DMA_TCD_CSR_INTHALF_MASK)

#define DMA_TCD_CSR_DREQ_MASK                    (0x8U)
#define DMA_TCD_CSR_DREQ_SHIFT                   (3U)
/*! DREQ - Disable Request
 *  0b0..No operation
 *  0b1..Clear the ERQ field to 0 upon major loop completion, thus disabling hardware service requests
 */
#define DMA_TCD_CSR_DREQ(x)                      (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_DREQ_SHIFT)) & DMA_TCD_CSR_DREQ_MASK)

#define DMA_TCD_CSR_ESG_MASK                     (0x10U)
#define DMA_TCD_CSR_ESG_SHIFT                    (4U)
/*! ESG - Enable Scatter/Gather Processing
 *  0b0..Current channel's TCD is normal format
 *  0b1..Current channel's TCD specifies scatter/gather format.
 */
#define DMA_TCD_CSR_ESG(x)                       (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_ESG_SHIFT)) & DMA_TCD_CSR_ESG_MASK)

#define DMA_TCD_CSR_MAJORELINK_MASK              (0x20U)
#define DMA_TCD_CSR_MAJORELINK_SHIFT             (5U)
/*! MAJORELINK - Enable Link When Major Loop Complete
 *  0b0..Channel-to-channel linking disabled
 *  0b1..Channel-to-channel linking enabled
 */
#define DMA_TCD_CSR_MAJORELINK(x)                (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_MAJORELINK_SHIFT)) & DMA_TCD_CSR_MAJORELINK_MASK)

#define DMA_TCD_CSR_EEOP_MASK                    (0x40U)
#define DMA_TCD_CSR_EEOP_SHIFT                   (6U)
/*! EEOP - Enable End-Of-Packet Processing
 *  0b0..End-of-packet operation disabled
 *  0b1..End-of-packet hardware input signal enabled
 */
#define DMA_TCD_CSR_EEOP(x)                      (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_EEOP_SHIFT)) & DMA_TCD_CSR_EEOP_MASK)

#define DMA_TCD_CSR_ESDA_MASK                    (0x80U)
#define DMA_TCD_CSR_ESDA_SHIFT                   (7U)
/*! ESDA - Enable Store Destination Address
 *  0b0..Ability to store destination address to system memory disabled
 *  0b1..Ability to store destination address to system memory enabled
 */
#define DMA_TCD_CSR_ESDA(x)                      (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_ESDA_SHIFT)) & DMA_TCD_CSR_ESDA_MASK)

#define DMA_TCD_CSR_MAJORLINKCH_MASK             (0x1F00U)
#define DMA_TCD_CSR_MAJORLINKCH_SHIFT            (8U)
/*! MAJORLINKCH - Major Loop Link Channel Number */
#define DMA_TCD_CSR_MAJORLINKCH(x)               (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_MAJORLINKCH_SHIFT)) & DMA_TCD_CSR_MAJORLINKCH_MASK)

#define DMA_TCD_CSR_BWC_MASK                     (0xC000U)
#define DMA_TCD_CSR_BWC_SHIFT                    (14U)
/*! BWC - Bandwidth Control
 *  0b00..No eDMA engine stalls
 *  0b01..
 *  0b10..eDMA engine stalls for 4 cycles after each R/W
 *  0b11..eDMA engine stalls for 8 cycles after each R/W
 */
#define DMA_TCD_CSR_BWC(x)                       (((uint16_t)(((uint16_t)(x)) << DMA_TCD_CSR_BWC_SHIFT)) & DMA_TCD_CSR_BWC_MASK)
/*! @} */

/* The count of DMA_TCD_CSR */
#define DMA_TCD_CSR_COUNT                        (32U)

/*! @name TCD_BITER_ELINKNO - TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) */
/*! @{ */

#define DMA_TCD_BITER_ELINKNO_BITER_MASK         (0x7FFFU)
#define DMA_TCD_BITER_ELINKNO_BITER_SHIFT        (0U)
/*! BITER - Starting Major Iteration Count */
#define DMA_TCD_BITER_ELINKNO_BITER(x)           (((uint16_t)(((uint16_t)(x)) << DMA_TCD_BITER_ELINKNO_BITER_SHIFT)) & DMA_TCD_BITER_ELINKNO_BITER_MASK)

#define DMA_TCD_BITER_ELINKNO_ELINK_MASK         (0x8000U)
#define DMA_TCD_BITER_ELINKNO_ELINK_SHIFT        (15U)
/*! ELINK - Enables Link
 *  0b0..Channel-to-channel linking disabled
 *  0b1..Channel-to-channel linking enabled
 */
#define DMA_TCD_BITER_ELINKNO_ELINK(x)           (((uint16_t)(((uint16_t)(x)) << DMA_TCD_BITER_ELINKNO_ELINK_SHIFT)) & DMA_TCD_BITER_ELINKNO_ELINK_MASK)
/*! @} */

/* The count of DMA_TCD_BITER_ELINKNO */
#define DMA_TCD_BITER_ELINKNO_COUNT              (32U)

/*! @name TCD_BITER_ELINKYES - TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) */
/*! @{ */

#define DMA_TCD_BITER_ELINKYES_BITER_MASK        (0x1FFU)
#define DMA_TCD_BITER_ELINKYES_BITER_SHIFT       (0U)
/*! BITER - Starting Major Iteration Count */
#define DMA_TCD_BITER_ELINKYES_BITER(x)          (((uint16_t)(((uint16_t)(x)) << DMA_TCD_BITER_ELINKYES_BITER_SHIFT)) & DMA_TCD_BITER_ELINKYES_BITER_MASK)

#define DMA_TCD_BITER_ELINKYES_LINKCH_MASK       (0x3E00U)
#define DMA_TCD_BITER_ELINKYES_LINKCH_SHIFT      (9U)
/*! LINKCH - Link Channel Number */
#define DMA_TCD_BITER_ELINKYES_LINKCH(x)         (((uint16_t)(((uint16_t)(x)) << DMA_TCD_BITER_ELINKYES_LINKCH_SHIFT)) & DMA_TCD_BITER_ELINKYES_LINKCH_MASK)

#define DMA_TCD_BITER_ELINKYES_ELINK_MASK        (0x8000U)
#define DMA_TCD_BITER_ELINKYES_ELINK_SHIFT       (15U)
/*! ELINK - Enable Link
 *  0b0..Channel-to-channel linking disabled
 *  0b1..Channel-to-channel linking enabled
 */
#define DMA_TCD_BITER_ELINKYES_ELINK(x)          (((uint16_t)(((uint16_t)(x)) << DMA_TCD_BITER_ELINKYES_ELINK_SHIFT)) & DMA_TCD_BITER_ELINKYES_ELINK_MASK)
/*! @} */

/* The count of DMA_TCD_BITER_ELINKYES */
#define DMA_TCD_BITER_ELINKYES_COUNT             (32U)


/*!
 * @}
 */ /* end of group DMA_Register_Masks */


/* DMA - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral DMA3 base address */
  #define DMA3_BASE                                (0x54000000u)
  /** Peripheral DMA3 base address */
  #define DMA3_BASE_NS                             (0x44000000u)
  /** Peripheral DMA3 base pointer */
  #define DMA3                                     ((DMA_Type *)DMA3_BASE)
  /** Peripheral DMA3 base pointer */
  #define DMA3_NS                                  ((DMA_Type *)DMA3_BASE_NS)
  /** Array initializer of DMA peripheral base addresses */
  #define DMA_BASE_ADDRS                           { DMA3_BASE }
  /** Array initializer of DMA peripheral base pointers */
  #define DMA_BASE_PTRS                            { DMA3 }
  /** Array initializer of DMA peripheral base addresses */
  #define DMA_BASE_ADDRS_NS                        { DMA3_BASE_NS }
  /** Array initializer of DMA peripheral base pointers */
  #define DMA_BASE_PTRS_NS                         { DMA3_NS }
#else
  /** Peripheral DMA3 base address */
  #define DMA3_BASE                                (0x44000000u)
  /** Peripheral DMA3 base pointer */
  #define DMA3                                     ((DMA_Type *)DMA3_BASE)
  /** Array initializer of DMA peripheral base addresses */
  #define DMA_BASE_ADDRS                           { DMA3_BASE }
  /** Array initializer of DMA peripheral base pointers */
  #define DMA_BASE_PTRS                            { DMA3 }
#endif
/** Interrupt vectors for the DMA peripheral type */
#define DMA_IRQS                                 { { DMA3_CH0_IRQn, DMA3_CH1_IRQn, DMA3_CH2_IRQn, DMA3_CH3_IRQn, DMA3_CH4_IRQn, DMA3_CH5_IRQn, DMA3_CH6_IRQn, DMA3_CH7_IRQn, DMA3_CH8_IRQn, DMA3_CH9_IRQn, DMA3_CH10_IRQn, DMA3_CH11_IRQn, DMA3_CH12_IRQn, DMA3_CH13_IRQn, DMA3_CH14_IRQn, DMA3_CH15_IRQn, DMA3_CH16_IRQn, DMA3_CH17_IRQn, DMA3_CH18_IRQn, DMA3_CH19_IRQn, DMA3_CH20_IRQn, DMA3_CH21_IRQn, DMA3_CH22_IRQn, DMA3_CH23_IRQn, DMA3_CH24_IRQn, DMA3_CH25_IRQn, DMA3_CH26_IRQn, DMA3_CH27_IRQn, DMA3_CH28_IRQn, DMA3_CH29_IRQn, DMA3_CH30_IRQn, DMA3_CH31_IRQn } }
#define DMA_ERROR_IRQS                           { { DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn, DMA_ERROR_IRQn } }

/*!
 * @}
 */ /* end of group DMA_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- DMA4 Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DMA4_Peripheral_Access_Layer DMA4 Peripheral Access Layer
 * @{
 */

/** DMA4 - Register Layout Typedef */
typedef struct {
  __IO uint32_t MP_CSR;                            /**< Management Page Control Register, offset: 0x0 */
  __I  uint32_t MP_ES;                             /**< Management Page Error Status Register, offset: 0x4 */
  __I  uint32_t MP_INT_LOW;                        /**< Management Page Interrupt Request Status Register - Low, offset: 0x8 */
  __I  uint32_t MP_INT_HIGH;                       /**< Management Page Interrupt Request Status Register- High, offset: 0xC */
  __I  uint32_t MP_HRS_LOW;                        /**< Management Page Hardware Request Status Register - Low, offset: 0x10 */
  __I  uint32_t MP_HRS_HIGH;                       /**< Management Page Hardware Request Status Register - High, offset: 0x14 */
       uint8_t RESERVED_0[232];
  __IO uint32_t CH_GRPRI[64];                      /**< Channel Arbitration Group Register, array offset: 0x100, array step: 0x4 */
       uint8_t RESERVED_1[65024];
  struct {                                         /* offset: 0x10000, array step: 0x8000 */
    __IO uint32_t CH_CSR;                            /**< Channel Control and Status Register, array offset: 0x10000, array step: 0x8000 */
    __IO uint32_t CH_ES;                             /**< Channel Error Status Register, array offset: 0x10004, array step: 0x8000 */
    __IO uint32_t CH_INT;                            /**< Channel Interrupt Status Register, array offset: 0x10008, array step: 0x8000 */
    __IO uint32_t CH_SBR;                            /**< Channel System Bus Register, array offset: 0x1000C, array step: 0x8000 */
    __IO uint32_t CH_PRI;                            /**< Channel Priority Register, array offset: 0x10010, array step: 0x8000 */
    __IO uint32_t CH_MUX;                            /**< Channel Multiplexor Configuration, array offset: 0x10014, array step: 0x8000 */
    __IO uint16_t CH_MATTR;                          /**< Memory Attributes Register, array offset: 0x10018, array step: 0x8000 */
         uint8_t RESERVED_0[6];
    __IO uint32_t SADDR;                             /**< TCD Source Address Register, array offset: 0x10020, array step: 0x8000 */
    __IO uint16_t SOFF;                              /**< TCD Signed Source Address Offset Register, array offset: 0x10024, array step: 0x8000 */
    __IO uint16_t ATTR;                              /**< TCD Transfer Attributes Register, array offset: 0x10026, array step: 0x8000 */
    union {                                          /* offset: 0x10028, array step: 0x8000 */
      __IO uint32_t NBYTES_MLOFFNO;                    /**< TCD Transfer Size without Minor Loop Offsets Register, array offset: 0x10028, array step: 0x8000 */
      __IO uint32_t NBYTES_MLOFFYES;                   /**< TCD Transfer Size with Minor Loop Offsets Register, array offset: 0x10028, array step: 0x8000 */
    };
    __IO uint32_t SLAST_SDA;                         /**< TCD Last Source Address Adjustment / Store DADDR Address Register, array offset: 0x1002C, array step: 0x8000 */
    __IO uint32_t DADDR;                             /**< TCD Destination Address Register, array offset: 0x10030, array step: 0x8000 */
    __IO uint16_t DOFF;                              /**< TCD Signed Destination Address Offset Register, array offset: 0x10034, array step: 0x8000 */
    union {                                          /* offset: 0x10036, array step: 0x8000 */
      __IO uint16_t CITER_ELINKNO;                     /**< TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register, array offset: 0x10036, array step: 0x8000 */
      __IO uint16_t CITER_ELINKYES;                    /**< TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register, array offset: 0x10036, array step: 0x8000 */
    };
    __IO uint32_t DLAST_SGA;                         /**< TCD Last Destination Address Adjustment / Scatter Gather Address Register, array offset: 0x10038, array step: 0x8000 */
    __IO uint16_t CSR;                               /**< TCD Control and Status Register, array offset: 0x1003C, array step: 0x8000 */
    union {                                          /* offset: 0x1003E, array step: 0x8000 */
      __IO uint16_t BITER_ELINKNO;                     /**< TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register, array offset: 0x1003E, array step: 0x8000 */
      __IO uint16_t BITER_ELINKYES;                    /**< TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register, array offset: 0x1003E, array step: 0x8000 */
    };
         uint8_t RESERVED_1[32704];
  } TCD[64];
} DMA4_Type;

/* ----------------------------------------------------------------------------
   -- DMA4 Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup DMA4_Register_Masks DMA4 Register Masks
 * @{
 */

/*! @name MP_CSR - Management Page Control Register */
/*! @{ */

#define DMA4_MP_CSR_EDBG_MASK                    (0x2U)
#define DMA4_MP_CSR_EDBG_SHIFT                   (1U)
/*! EDBG - Enable Debug
 *  0b0..Debug mode is disabled.
 *  0b1..Debug mode is enabled.
 */
#define DMA4_MP_CSR_EDBG(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_EDBG_SHIFT)) & DMA4_MP_CSR_EDBG_MASK)

#define DMA4_MP_CSR_ERCA_MASK                    (0x4U)
#define DMA4_MP_CSR_ERCA_SHIFT                   (2U)
/*! ERCA - Enable Round Robin Channel Arbitration
 *  0b0..Round robin channel arbitration is disabled.
 *  0b1..Round robin channel arbitration is enabled.
 */
#define DMA4_MP_CSR_ERCA(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_ERCA_SHIFT)) & DMA4_MP_CSR_ERCA_MASK)

#define DMA4_MP_CSR_HAE_MASK                     (0x10U)
#define DMA4_MP_CSR_HAE_SHIFT                    (4U)
/*! HAE - Halt After Error
 *  0b0..Normal operation
 *  0b1..Any error causes the HALT bit to set. Subsequently, all service requests are ignored until the HALT bit is cleared.
 */
#define DMA4_MP_CSR_HAE(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_HAE_SHIFT)) & DMA4_MP_CSR_HAE_MASK)

#define DMA4_MP_CSR_HALT_MASK                    (0x20U)
#define DMA4_MP_CSR_HALT_SHIFT                   (5U)
/*! HALT - Halt DMA Operations
 *  0b0..Normal operation
 *  0b1..Stall the start of any new channels. Executing channels are allowed to complete. Channel execution resumes when this bit is cleared.
 */
#define DMA4_MP_CSR_HALT(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_HALT_SHIFT)) & DMA4_MP_CSR_HALT_MASK)

#define DMA4_MP_CSR_GCLC_MASK                    (0x40U)
#define DMA4_MP_CSR_GCLC_SHIFT                   (6U)
/*! GCLC - Global Channel Linking Control
 *  0b0..Channel linking is disabled for all channels.
 *  0b1..Channel linking is available and controlled by each channel's link settings.
 */
#define DMA4_MP_CSR_GCLC(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_GCLC_SHIFT)) & DMA4_MP_CSR_GCLC_MASK)

#define DMA4_MP_CSR_GMRC_MASK                    (0x80U)
#define DMA4_MP_CSR_GMRC_SHIFT                   (7U)
/*! GMRC - Global Master ID Replication Control
 *  0b0..Master ID replication is disabled for all channels.
 *  0b1..Master ID replication is available and is controlled by each channel's CHn_SBR[EMI] setting.
 */
#define DMA4_MP_CSR_GMRC(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_GMRC_SHIFT)) & DMA4_MP_CSR_GMRC_MASK)

#define DMA4_MP_CSR_VER_MASK                     (0xFF0000U)
#define DMA4_MP_CSR_VER_SHIFT                    (16U)
/*! VER - eDMA version */
#define DMA4_MP_CSR_VER(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_VER_SHIFT)) & DMA4_MP_CSR_VER_MASK)

#define DMA4_MP_CSR_ACTIVE_ID_MASK               (0x3F000000U)
#define DMA4_MP_CSR_ACTIVE_ID_SHIFT              (24U)
/*! ACTIVE_ID - Active channel ID */
#define DMA4_MP_CSR_ACTIVE_ID(x)                 (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_ACTIVE_ID_SHIFT)) & DMA4_MP_CSR_ACTIVE_ID_MASK)

#define DMA4_MP_CSR_ACTIVE_MASK                  (0x80000000U)
#define DMA4_MP_CSR_ACTIVE_SHIFT                 (31U)
/*! ACTIVE - DMA Active Status
 *  0b0..eDMA is idle.
 *  0b1..eDMA is executing a channel.
 */
#define DMA4_MP_CSR_ACTIVE(x)                    (((uint32_t)(((uint32_t)(x)) << DMA4_MP_CSR_ACTIVE_SHIFT)) & DMA4_MP_CSR_ACTIVE_MASK)
/*! @} */

/*! @name MP_ES - Management Page Error Status Register */
/*! @{ */

#define DMA4_MP_ES_DBE_MASK                      (0x1U)
#define DMA4_MP_ES_DBE_SHIFT                     (0U)
/*! DBE - Destination Bus Error
 *  0b0..No destination bus error
 *  0b1..The last recorded error was a bus error on a destination write
 */
#define DMA4_MP_ES_DBE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_DBE_SHIFT)) & DMA4_MP_ES_DBE_MASK)

#define DMA4_MP_ES_SBE_MASK                      (0x2U)
#define DMA4_MP_ES_SBE_SHIFT                     (1U)
/*! SBE - Source Bus Error
 *  0b0..No source bus error
 *  0b1..The last recorded error was a bus error on a source read
 */
#define DMA4_MP_ES_SBE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_SBE_SHIFT)) & DMA4_MP_ES_SBE_MASK)

#define DMA4_MP_ES_SGE_MASK                      (0x4U)
#define DMA4_MP_ES_SGE_SHIFT                     (2U)
/*! SGE - Scatter/Gather Configuration Error
 *  0b0..No scatter/gather configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_DLASTSGA field. This field is
 *       checked at the beginning of a scatter/gather operation after major loop completion if TCDn_CSR[ESG] is
 *       enabled. TCDn_DLASTSGA is not on a 32 byte boundary.
 */
#define DMA4_MP_ES_SGE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_SGE_SHIFT)) & DMA4_MP_ES_SGE_MASK)

#define DMA4_MP_ES_NCE_MASK                      (0x8U)
#define DMA4_MP_ES_NCE_SHIFT                     (3U)
/*! NCE - NBYTES/CITER Configuration Error
 *  0b0..No NBYTES/CITER configuration error
 *  0b1..The last recorded error was NBYTES equal to zero or a CITER not equal to BITER error
 */
#define DMA4_MP_ES_NCE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_NCE_SHIFT)) & DMA4_MP_ES_NCE_MASK)

#define DMA4_MP_ES_DOE_MASK                      (0x10U)
#define DMA4_MP_ES_DOE_SHIFT                     (4U)
/*! DOE - Destination Offset Error
 *  0b0..No destination offset configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_DOFF field. TCDn_DOFF is inconsistent with TCDn_ATTR[DSIZE].
 */
#define DMA4_MP_ES_DOE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_DOE_SHIFT)) & DMA4_MP_ES_DOE_MASK)

#define DMA4_MP_ES_DAE_MASK                      (0x20U)
#define DMA4_MP_ES_DAE_SHIFT                     (5U)
/*! DAE - Destination Address Error
 *  0b0..No destination address configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_DADDR field. TCDn_DADDR is inconsistent with TCDn_ATTR[DSIZE].
 */
#define DMA4_MP_ES_DAE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_DAE_SHIFT)) & DMA4_MP_ES_DAE_MASK)

#define DMA4_MP_ES_SOE_MASK                      (0x40U)
#define DMA4_MP_ES_SOE_SHIFT                     (6U)
/*! SOE - Source Offset Error
 *  0b0..No source offset configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_SOFF field. TCDn_SOFF is inconsistent with TCDn_ATTR[SSIZE].
 */
#define DMA4_MP_ES_SOE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_SOE_SHIFT)) & DMA4_MP_ES_SOE_MASK)

#define DMA4_MP_ES_SAE_MASK                      (0x80U)
#define DMA4_MP_ES_SAE_SHIFT                     (7U)
/*! SAE - Source Address Error
 *  0b0..No source address configuration error.
 *  0b1..The last recorded error was a configuration error detected in the TCDn_SADDR field. TCDn_SADDR is inconsistent with TCDn_ATTR[SSIZE].
 */
#define DMA4_MP_ES_SAE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_SAE_SHIFT)) & DMA4_MP_ES_SAE_MASK)

#define DMA4_MP_ES_ECX_MASK                      (0x100U)
#define DMA4_MP_ES_ECX_SHIFT                     (8U)
/*! ECX - Transfer Canceled
 *  0b0..No canceled transfers
 *  0b1..The last recorded entry was a canceled transfer by the error cancel transfer input.
 */
#define DMA4_MP_ES_ECX(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_ECX_SHIFT)) & DMA4_MP_ES_ECX_MASK)

#define DMA4_MP_ES_ERRCHN_MASK                   (0x3F000000U)
#define DMA4_MP_ES_ERRCHN_SHIFT                  (24U)
/*! ERRCHN - Error Channel Number or Canceled Channel Number */
#define DMA4_MP_ES_ERRCHN(x)                     (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_ERRCHN_SHIFT)) & DMA4_MP_ES_ERRCHN_MASK)

#define DMA4_MP_ES_VLD_MASK                      (0x80000000U)
#define DMA4_MP_ES_VLD_SHIFT                     (31U)
/*! VLD - Valid
 *  0b0..No ERR bits are set.
 *  0b1..At least one ERR bit is set indicating a valid error exists that has not been cleared.
 */
#define DMA4_MP_ES_VLD(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_MP_ES_VLD_SHIFT)) & DMA4_MP_ES_VLD_MASK)
/*! @} */

/*! @name MP_INT_LOW - Management Page Interrupt Request Status Register - Low */
/*! @{ */

#define DMA4_MP_INT_LOW_INT_MASK                 (0xFFFFFFFFU)
#define DMA4_MP_INT_LOW_INT_SHIFT                (0U)
/*! INT - Interrupt Request Status for channels 31 - 0 */
#define DMA4_MP_INT_LOW_INT(x)                   (((uint32_t)(((uint32_t)(x)) << DMA4_MP_INT_LOW_INT_SHIFT)) & DMA4_MP_INT_LOW_INT_MASK)
/*! @} */

/*! @name MP_INT_HIGH - Management Page Interrupt Request Status Register- High */
/*! @{ */

#define DMA4_MP_INT_HIGH_INT_MASK                (0xFFFFFFFFU)
#define DMA4_MP_INT_HIGH_INT_SHIFT               (0U)
/*! INT - Interrupt Request Status for channels 63-32 */
#define DMA4_MP_INT_HIGH_INT(x)                  (((uint32_t)(((uint32_t)(x)) << DMA4_MP_INT_HIGH_INT_SHIFT)) & DMA4_MP_INT_HIGH_INT_MASK)
/*! @} */

/*! @name MP_HRS_LOW - Management Page Hardware Request Status Register - Low */
/*! @{ */

#define DMA4_MP_HRS_LOW_HRS_MASK                 (0xFFFFFFFFU)
#define DMA4_MP_HRS_LOW_HRS_SHIFT                (0U)
/*! HRS - Hardware Request Status for channels 31 - 0
 *  0b00000000000000000000000000000000..A hardware service request for the channel is not present
 *  0b00000000000000000000000000000001..A hardware service request for channel 0 is present
 */
#define DMA4_MP_HRS_LOW_HRS(x)                   (((uint32_t)(((uint32_t)(x)) << DMA4_MP_HRS_LOW_HRS_SHIFT)) & DMA4_MP_HRS_LOW_HRS_MASK)
/*! @} */

/*! @name MP_HRS_HIGH - Management Page Hardware Request Status Register - High */
/*! @{ */

#define DMA4_MP_HRS_HIGH_HRS_MASK                (0xFFFFFFFFU)
#define DMA4_MP_HRS_HIGH_HRS_SHIFT               (0U)
/*! HRS - Hardware Request Status for channels 63-32
 *  0b00000000000000000000000000000000..A hardware service request for the channel is not present
 *  0b00000000000000000000000000000001..A hardware service request for channel 0 is present
 */
#define DMA4_MP_HRS_HIGH_HRS(x)                  (((uint32_t)(((uint32_t)(x)) << DMA4_MP_HRS_HIGH_HRS_SHIFT)) & DMA4_MP_HRS_HIGH_HRS_MASK)
/*! @} */

/*! @name CH_GRPRI - Channel Arbitration Group Register */
/*! @{ */

#define DMA4_CH_GRPRI_GRPRI_MASK                 (0x3FU)
#define DMA4_CH_GRPRI_GRPRI_SHIFT                (0U)
/*! GRPRI - Arbitration group per channel. */
#define DMA4_CH_GRPRI_GRPRI(x)                   (((uint32_t)(((uint32_t)(x)) << DMA4_CH_GRPRI_GRPRI_SHIFT)) & DMA4_CH_GRPRI_GRPRI_MASK)
/*! @} */

/* The count of DMA4_CH_GRPRI */
#define DMA4_CH_GRPRI_COUNT                      (64U)

/*! @name CH_CSR - Channel Control and Status Register */
/*! @{ */

#define DMA4_CH_CSR_ERQ_MASK                     (0x1U)
#define DMA4_CH_CSR_ERQ_SHIFT                    (0U)
/*! ERQ - Enable DMA Request
 *  0b0..The DMA hardware request signal for the corresponding channel is disabled.
 *  0b1..The DMA hardware request signal for the corresponding channel is enabled.
 */
#define DMA4_CH_CSR_ERQ(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_CSR_ERQ_SHIFT)) & DMA4_CH_CSR_ERQ_MASK)

#define DMA4_CH_CSR_EARQ_MASK                    (0x2U)
#define DMA4_CH_CSR_EARQ_SHIFT                   (1U)
/*! EARQ - Enable Asynchronous DMA Request
 *  0b0..Disable asynchronous DMA request for the channel.
 *  0b1..Enable asynchronous DMA request for the channel.
 */
#define DMA4_CH_CSR_EARQ(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_CH_CSR_EARQ_SHIFT)) & DMA4_CH_CSR_EARQ_MASK)

#define DMA4_CH_CSR_EEI_MASK                     (0x4U)
#define DMA4_CH_CSR_EEI_SHIFT                    (2U)
/*! EEI - Enable Error Interrupt
 *  0b0..The error signal for corresponding channel does not generate an error interrupt
 *  0b1..The assertion of the error signal for corresponding channel generates an error interrupt request
 */
#define DMA4_CH_CSR_EEI(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_CSR_EEI_SHIFT)) & DMA4_CH_CSR_EEI_MASK)

#define DMA4_CH_CSR_SWAP_MASK                    (0xF000U)
#define DMA4_CH_CSR_SWAP_SHIFT                   (12U)
/*! SWAP - Swap size
 *  0b0000..disabled
 *  0b0001..read with 8-bit swap
 *  0b0010..read with 16-bit swap
 *  0b0011..read with 32-bit swap
 *  0b0100-0b1000..reserved
 *  0b1001..write with 8-bit swap
 *  0b1010..write with 16-bit swap
 *  0b1011..write with 32-bit swap
 *  0b1100-0b1111..reserved
 */
#define DMA4_CH_CSR_SWAP(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_CH_CSR_SWAP_SHIFT)) & DMA4_CH_CSR_SWAP_MASK)

#define DMA4_CH_CSR_SIGNEXT_MASK                 (0x3F0000U)
#define DMA4_CH_CSR_SIGNEXT_SHIFT                (16U)
/*! SIGNEXT - Sign Extension
 *  0b000000..disabled
 *  0b000001..A non-zero value specifying the sign extend bit position
 */
#define DMA4_CH_CSR_SIGNEXT(x)                   (((uint32_t)(((uint32_t)(x)) << DMA4_CH_CSR_SIGNEXT_SHIFT)) & DMA4_CH_CSR_SIGNEXT_MASK)

#define DMA4_CH_CSR_DONE_MASK                    (0x40000000U)
#define DMA4_CH_CSR_DONE_SHIFT                   (30U)
/*! DONE - Channel Done */
#define DMA4_CH_CSR_DONE(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_CH_CSR_DONE_SHIFT)) & DMA4_CH_CSR_DONE_MASK)

#define DMA4_CH_CSR_ACTIVE_MASK                  (0x80000000U)
#define DMA4_CH_CSR_ACTIVE_SHIFT                 (31U)
/*! ACTIVE - Channel Active */
#define DMA4_CH_CSR_ACTIVE(x)                    (((uint32_t)(((uint32_t)(x)) << DMA4_CH_CSR_ACTIVE_SHIFT)) & DMA4_CH_CSR_ACTIVE_MASK)
/*! @} */

/* The count of DMA4_CH_CSR */
#define DMA4_CH_CSR_COUNT                        (64U)

/*! @name CH_ES - Channel Error Status Register */
/*! @{ */

#define DMA4_CH_ES_DBE_MASK                      (0x1U)
#define DMA4_CH_ES_DBE_SHIFT                     (0U)
/*! DBE - Destination Bus Error
 *  0b0..No destination bus error
 *  0b1..The last recorded error was a bus error on a destination write
 */
#define DMA4_CH_ES_DBE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_DBE_SHIFT)) & DMA4_CH_ES_DBE_MASK)

#define DMA4_CH_ES_SBE_MASK                      (0x2U)
#define DMA4_CH_ES_SBE_SHIFT                     (1U)
/*! SBE - Source Bus Error
 *  0b0..No source bus error
 *  0b1..The last recorded error was a bus error on a source read
 */
#define DMA4_CH_ES_SBE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_SBE_SHIFT)) & DMA4_CH_ES_SBE_MASK)

#define DMA4_CH_ES_SGE_MASK                      (0x4U)
#define DMA4_CH_ES_SGE_SHIFT                     (2U)
/*! SGE - Scatter/Gather Configuration Error
 *  0b0..No scatter/gather configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_DLASTSGA field. This field is
 *       checked at the beginning of a scatter/gather operation after major loop completion if TCDn_CSR[ESG] is
 *       enabled. TCDn_DLASTSGA is not on a 32 byte boundary.
 */
#define DMA4_CH_ES_SGE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_SGE_SHIFT)) & DMA4_CH_ES_SGE_MASK)

#define DMA4_CH_ES_NCE_MASK                      (0x8U)
#define DMA4_CH_ES_NCE_SHIFT                     (3U)
/*! NCE - NBYTES/CITER Configuration Error
 *  0b0..No NBYTES/CITER configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_NBYTES or TCDn_CITER fields.
 *       TCDn_NBYTES is not a multiple of TCDn_ATTR[SSIZE] and TCDn_ATTR[DSIZE], or TCDn_CITER[CITER] is equal to zero,
 *       or TCDn_CITER[ELINK] is not equal to TCDn_BITER[ELINK]
 */
#define DMA4_CH_ES_NCE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_NCE_SHIFT)) & DMA4_CH_ES_NCE_MASK)

#define DMA4_CH_ES_DOE_MASK                      (0x10U)
#define DMA4_CH_ES_DOE_SHIFT                     (4U)
/*! DOE - Destination Offset Error
 *  0b0..No destination offset configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_DOFF field. TCDn_DOFF is inconsistent with TCDn_ATTR[DSIZE].
 */
#define DMA4_CH_ES_DOE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_DOE_SHIFT)) & DMA4_CH_ES_DOE_MASK)

#define DMA4_CH_ES_DAE_MASK                      (0x20U)
#define DMA4_CH_ES_DAE_SHIFT                     (5U)
/*! DAE - Destination Address Error
 *  0b0..No destination address configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_DADDR field. TCDn_DADDR is inconsistent with TCDn_ATTR[DSIZE].
 */
#define DMA4_CH_ES_DAE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_DAE_SHIFT)) & DMA4_CH_ES_DAE_MASK)

#define DMA4_CH_ES_SOE_MASK                      (0x40U)
#define DMA4_CH_ES_SOE_SHIFT                     (6U)
/*! SOE - Source Offset Error
 *  0b0..No source offset configuration error
 *  0b1..The last recorded error was a configuration error detected in the TCDn_SOFF field. TCDn_SOFF is inconsistent with TCDn_ATTR[SSIZE].
 */
#define DMA4_CH_ES_SOE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_SOE_SHIFT)) & DMA4_CH_ES_SOE_MASK)

#define DMA4_CH_ES_SAE_MASK                      (0x80U)
#define DMA4_CH_ES_SAE_SHIFT                     (7U)
/*! SAE - Source Address Error
 *  0b0..No source address configuration error.
 *  0b1..The last recorded error was a configuration error detected in the TCDn_SADDR field. TCDn_SADDR is inconsistent with TCDn_ATTR[SSIZE].
 */
#define DMA4_CH_ES_SAE(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_SAE_SHIFT)) & DMA4_CH_ES_SAE_MASK)

#define DMA4_CH_ES_ERR_MASK                      (0x80000000U)
#define DMA4_CH_ES_ERR_SHIFT                     (31U)
/*! ERR - Error In Channel
 *  0b0..An error in this channel has not occurred
 *  0b1..An error in this channel has occurred
 */
#define DMA4_CH_ES_ERR(x)                        (((uint32_t)(((uint32_t)(x)) << DMA4_CH_ES_ERR_SHIFT)) & DMA4_CH_ES_ERR_MASK)
/*! @} */

/* The count of DMA4_CH_ES */
#define DMA4_CH_ES_COUNT                         (64U)

/*! @name CH_INT - Channel Interrupt Status Register */
/*! @{ */

#define DMA4_CH_INT_INT_MASK                     (0x1U)
#define DMA4_CH_INT_INT_SHIFT                    (0U)
/*! INT - Interrupt Request
 *  0b0..The interrupt request for corresponding channel is cleared
 *  0b1..The interrupt request for corresponding channel is active
 */
#define DMA4_CH_INT_INT(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_INT_INT_SHIFT)) & DMA4_CH_INT_INT_MASK)
/*! @} */

/* The count of DMA4_CH_INT */
#define DMA4_CH_INT_COUNT                        (64U)

/*! @name CH_SBR - Channel System Bus Register */
/*! @{ */

#define DMA4_CH_SBR_MID_MASK                     (0xFU)
#define DMA4_CH_SBR_MID_SHIFT                    (0U)
/*! MID - Master ID */
#define DMA4_CH_SBR_MID(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_SBR_MID_SHIFT)) & DMA4_CH_SBR_MID_MASK)

#define DMA4_CH_SBR_INSTR_MASK                   (0x2000U)
#define DMA4_CH_SBR_INSTR_SHIFT                  (13U)
/*! INSTR - Instruction/Data Access
 *  0b0..Data access for DMA transfers
 *  0b1..Instruction access for DMA transfers
 */
#define DMA4_CH_SBR_INSTR(x)                     (((uint32_t)(((uint32_t)(x)) << DMA4_CH_SBR_INSTR_SHIFT)) & DMA4_CH_SBR_INSTR_MASK)

#define DMA4_CH_SBR_SEC_MASK                     (0x4000U)
#define DMA4_CH_SBR_SEC_SHIFT                    (14U)
/*! SEC - Security Level
 *  0b0..Nonsecure protection level for DMA transfers
 *  0b1..Secure protection level for DMA transfers
 */
#define DMA4_CH_SBR_SEC(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_SBR_SEC_SHIFT)) & DMA4_CH_SBR_SEC_MASK)

#define DMA4_CH_SBR_PAL_MASK                     (0x8000U)
#define DMA4_CH_SBR_PAL_SHIFT                    (15U)
/*! PAL - Privileged Access Level
 *  0b0..User protection level for DMA transfers
 *  0b1..Privileged protection level for DMA transfers
 */
#define DMA4_CH_SBR_PAL(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_SBR_PAL_SHIFT)) & DMA4_CH_SBR_PAL_MASK)

#define DMA4_CH_SBR_EMI_MASK                     (0x10000U)
#define DMA4_CH_SBR_EMI_SHIFT                    (16U)
/*! EMI - Enable Master ID replication
 *  0b0..Master ID replication is disabled
 *  0b1..Master ID replication is enabled
 */
#define DMA4_CH_SBR_EMI(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_SBR_EMI_SHIFT)) & DMA4_CH_SBR_EMI_MASK)

#define DMA4_CH_SBR_ATTR_MASK                    (0x7E0000U)
#define DMA4_CH_SBR_ATTR_SHIFT                   (17U)
/*! ATTR - Attribute Output */
#define DMA4_CH_SBR_ATTR(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_CH_SBR_ATTR_SHIFT)) & DMA4_CH_SBR_ATTR_MASK)
/*! @} */

/* The count of DMA4_CH_SBR */
#define DMA4_CH_SBR_COUNT                        (64U)

/*! @name CH_PRI - Channel Priority Register */
/*! @{ */

#define DMA4_CH_PRI_APL_MASK                     (0x7U)
#define DMA4_CH_PRI_APL_SHIFT                    (0U)
/*! APL - Arbitration Priority Level */
#define DMA4_CH_PRI_APL(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_PRI_APL_SHIFT)) & DMA4_CH_PRI_APL_MASK)

#define DMA4_CH_PRI_DPA_MASK                     (0x40000000U)
#define DMA4_CH_PRI_DPA_SHIFT                    (30U)
/*! DPA - Disable Preempt Ability.
 *  0b0..The channel can suspend a lower priority channel.
 *  0b1..The channel cannot suspend any other channel, regardless of channel priority.
 */
#define DMA4_CH_PRI_DPA(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_PRI_DPA_SHIFT)) & DMA4_CH_PRI_DPA_MASK)

#define DMA4_CH_PRI_ECP_MASK                     (0x80000000U)
#define DMA4_CH_PRI_ECP_SHIFT                    (31U)
/*! ECP - Enable Channel Preemption.
 *  0b0..The channel cannot be suspended by a higher priority channel's service request.
 *  0b1..The channel can be temporarily suspended by the service request of a higher priority channel.
 */
#define DMA4_CH_PRI_ECP(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_PRI_ECP_SHIFT)) & DMA4_CH_PRI_ECP_MASK)
/*! @} */

/* The count of DMA4_CH_PRI */
#define DMA4_CH_PRI_COUNT                        (64U)

/*! @name CH_MUX - Channel Multiplexor Configuration */
/*! @{ */

#define DMA4_CH_MUX_SRC_MASK                     (0xFFU)
#define DMA4_CH_MUX_SRC_SHIFT                    (0U)
/*! SRC - Service Request Source */
#define DMA4_CH_MUX_SRC(x)                       (((uint32_t)(((uint32_t)(x)) << DMA4_CH_MUX_SRC_SHIFT)) & DMA4_CH_MUX_SRC_MASK)
/*! @} */

/* The count of DMA4_CH_MUX */
#define DMA4_CH_MUX_COUNT                        (64U)

/*! @name CH_MATTR - Memory Attributes Register */
/*! @{ */

#define DMA4_CH_MATTR_RCACHE_MASK                (0xFU)
#define DMA4_CH_MATTR_RCACHE_SHIFT               (0U)
/*! RCACHE - Read Cache Attributes */
#define DMA4_CH_MATTR_RCACHE(x)                  (((uint16_t)(((uint16_t)(x)) << DMA4_CH_MATTR_RCACHE_SHIFT)) & DMA4_CH_MATTR_RCACHE_MASK)

#define DMA4_CH_MATTR_WCACHE_MASK                (0xF0U)
#define DMA4_CH_MATTR_WCACHE_SHIFT               (4U)
/*! WCACHE - Write Cache Attributes */
#define DMA4_CH_MATTR_WCACHE(x)                  (((uint16_t)(((uint16_t)(x)) << DMA4_CH_MATTR_WCACHE_SHIFT)) & DMA4_CH_MATTR_WCACHE_MASK)
/*! @} */

/* The count of DMA4_CH_MATTR */
#define DMA4_CH_MATTR_COUNT                      (64U)

/*! @name SADDR - TCD Source Address Register */
/*! @{ */

#define DMA4_SADDR_SADDR_MASK                    (0xFFFFFFFFU)
#define DMA4_SADDR_SADDR_SHIFT                   (0U)
/*! SADDR - Source Address */
#define DMA4_SADDR_SADDR(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_SADDR_SADDR_SHIFT)) & DMA4_SADDR_SADDR_MASK)
/*! @} */

/* The count of DMA4_SADDR */
#define DMA4_SADDR_COUNT                         (64U)

/*! @name SOFF - TCD Signed Source Address Offset Register */
/*! @{ */

#define DMA4_SOFF_SOFF_MASK                      (0xFFFFU)
#define DMA4_SOFF_SOFF_SHIFT                     (0U)
/*! SOFF - Source address signed offset */
#define DMA4_SOFF_SOFF(x)                        (((uint16_t)(((uint16_t)(x)) << DMA4_SOFF_SOFF_SHIFT)) & DMA4_SOFF_SOFF_MASK)
/*! @} */

/* The count of DMA4_SOFF */
#define DMA4_SOFF_COUNT                          (64U)

/*! @name ATTR - TCD Transfer Attributes Register */
/*! @{ */

#define DMA4_ATTR_DSIZE_MASK                     (0x7U)
#define DMA4_ATTR_DSIZE_SHIFT                    (0U)
/*! DSIZE - Destination data transfer size
 *  0b000..8-bit
 *  0b001..16-bit
 *  0b010..32-bit
 *  0b011..64-bit
 *  0b100..16-byte
 *  0b101..32-byte
 *  0b110..64-byte
 *  0b111..128-byte
 */
#define DMA4_ATTR_DSIZE(x)                       (((uint16_t)(((uint16_t)(x)) << DMA4_ATTR_DSIZE_SHIFT)) & DMA4_ATTR_DSIZE_MASK)

#define DMA4_ATTR_DMOD_MASK                      (0xF8U)
#define DMA4_ATTR_DMOD_SHIFT                     (3U)
/*! DMOD - Destination address modulo */
#define DMA4_ATTR_DMOD(x)                        (((uint16_t)(((uint16_t)(x)) << DMA4_ATTR_DMOD_SHIFT)) & DMA4_ATTR_DMOD_MASK)

#define DMA4_ATTR_SSIZE_MASK                     (0x700U)
#define DMA4_ATTR_SSIZE_SHIFT                    (8U)
/*! SSIZE - Source data transfer size
 *  0b000..8-bit
 *  0b001..16-bit
 *  0b010..32-bit
 *  0b011..64-bit
 *  0b100..16-byte
 *  0b101..32-byte
 *  0b110..64-byte
 *  0b111..128-byte
 */
#define DMA4_ATTR_SSIZE(x)                       (((uint16_t)(((uint16_t)(x)) << DMA4_ATTR_SSIZE_SHIFT)) & DMA4_ATTR_SSIZE_MASK)

#define DMA4_ATTR_SMOD_MASK                      (0xF800U)
#define DMA4_ATTR_SMOD_SHIFT                     (11U)
/*! SMOD - Source address modulo
 *  0b00000..Source address modulo feature is disabled
 *  0b00001..Source address modulo feature is enabled for any non-zero value [1-31]
 */
#define DMA4_ATTR_SMOD(x)                        (((uint16_t)(((uint16_t)(x)) << DMA4_ATTR_SMOD_SHIFT)) & DMA4_ATTR_SMOD_MASK)
/*! @} */

/* The count of DMA4_ATTR */
#define DMA4_ATTR_COUNT                          (64U)

/*! @name NBYTES_MLOFFNO - TCD Transfer Size without Minor Loop Offsets Register */
/*! @{ */

#define DMA4_NBYTES_MLOFFNO_NBYTES_MASK          (0x3FFFFFFFU)
#define DMA4_NBYTES_MLOFFNO_NBYTES_SHIFT         (0U)
/*! NBYTES - Number of Bytes to transfer per service request */
#define DMA4_NBYTES_MLOFFNO_NBYTES(x)            (((uint32_t)(((uint32_t)(x)) << DMA4_NBYTES_MLOFFNO_NBYTES_SHIFT)) & DMA4_NBYTES_MLOFFNO_NBYTES_MASK)

#define DMA4_NBYTES_MLOFFNO_DMLOE_MASK           (0x40000000U)
#define DMA4_NBYTES_MLOFFNO_DMLOE_SHIFT          (30U)
/*! DMLOE - Destination Minor Loop Offset Enable
 *  0b0..The minor loop offset is not applied to the DADDR
 *  0b1..The minor loop offset is applied to the DADDR
 */
#define DMA4_NBYTES_MLOFFNO_DMLOE(x)             (((uint32_t)(((uint32_t)(x)) << DMA4_NBYTES_MLOFFNO_DMLOE_SHIFT)) & DMA4_NBYTES_MLOFFNO_DMLOE_MASK)

#define DMA4_NBYTES_MLOFFNO_SMLOE_MASK           (0x80000000U)
#define DMA4_NBYTES_MLOFFNO_SMLOE_SHIFT          (31U)
/*! SMLOE - Source Minor Loop Offset Enable
 *  0b0..The minor loop offset is not applied to the SADDR
 *  0b1..The minor loop offset is applied to the SADDR
 */
#define DMA4_NBYTES_MLOFFNO_SMLOE(x)             (((uint32_t)(((uint32_t)(x)) << DMA4_NBYTES_MLOFFNO_SMLOE_SHIFT)) & DMA4_NBYTES_MLOFFNO_SMLOE_MASK)
/*! @} */

/* The count of DMA4_NBYTES_MLOFFNO */
#define DMA4_NBYTES_MLOFFNO_COUNT                (64U)

/*! @name NBYTES_MLOFFYES - TCD Transfer Size with Minor Loop Offsets Register */
/*! @{ */

#define DMA4_NBYTES_MLOFFYES_NBYTES_MASK         (0x3FFU)
#define DMA4_NBYTES_MLOFFYES_NBYTES_SHIFT        (0U)
/*! NBYTES - Number of Bytes to transfer per service request */
#define DMA4_NBYTES_MLOFFYES_NBYTES(x)           (((uint32_t)(((uint32_t)(x)) << DMA4_NBYTES_MLOFFYES_NBYTES_SHIFT)) & DMA4_NBYTES_MLOFFYES_NBYTES_MASK)

#define DMA4_NBYTES_MLOFFYES_MLOFF_MASK          (0x3FFFFC00U)
#define DMA4_NBYTES_MLOFFYES_MLOFF_SHIFT         (10U)
/*! MLOFF - Minor Loop Offset */
#define DMA4_NBYTES_MLOFFYES_MLOFF(x)            (((uint32_t)(((uint32_t)(x)) << DMA4_NBYTES_MLOFFYES_MLOFF_SHIFT)) & DMA4_NBYTES_MLOFFYES_MLOFF_MASK)

#define DMA4_NBYTES_MLOFFYES_DMLOE_MASK          (0x40000000U)
#define DMA4_NBYTES_MLOFFYES_DMLOE_SHIFT         (30U)
/*! DMLOE - Destination Minor Loop Offset Enable
 *  0b0..The minor loop offset is not applied to the DADDR
 *  0b1..The minor loop offset is applied to the DADDR
 */
#define DMA4_NBYTES_MLOFFYES_DMLOE(x)            (((uint32_t)(((uint32_t)(x)) << DMA4_NBYTES_MLOFFYES_DMLOE_SHIFT)) & DMA4_NBYTES_MLOFFYES_DMLOE_MASK)

#define DMA4_NBYTES_MLOFFYES_SMLOE_MASK          (0x80000000U)
#define DMA4_NBYTES_MLOFFYES_SMLOE_SHIFT         (31U)
/*! SMLOE - Source Minor Loop Offset Enable
 *  0b0..The minor loop offset is not applied to the SADDR
 *  0b1..The minor loop offset is applied to the SADDR
 */
#define DMA4_NBYTES_MLOFFYES_SMLOE(x)            (((uint32_t)(((uint32_t)(x)) << DMA4_NBYTES_MLOFFYES_SMLOE_SHIFT)) & DMA4_NBYTES_MLOFFYES_SMLOE_MASK)
/*! @} */

/* The count of DMA4_NBYTES_MLOFFYES */
#define DMA4_NBYTES_MLOFFYES_COUNT               (64U)

/*! @name SLAST_SDA - TCD Last Source Address Adjustment / Store DADDR Address Register */
/*! @{ */

#define DMA4_SLAST_SDA_SLAST_SDA_MASK            (0xFFFFFFFFU)
#define DMA4_SLAST_SDA_SLAST_SDA_SHIFT           (0U)
/*! SLAST_SDA - Last Source Address Adjustment / Store DADDR Address */
#define DMA4_SLAST_SDA_SLAST_SDA(x)              (((uint32_t)(((uint32_t)(x)) << DMA4_SLAST_SDA_SLAST_SDA_SHIFT)) & DMA4_SLAST_SDA_SLAST_SDA_MASK)
/*! @} */

/* The count of DMA4_SLAST_SDA */
#define DMA4_SLAST_SDA_COUNT                     (64U)

/*! @name DADDR - TCD Destination Address Register */
/*! @{ */

#define DMA4_DADDR_DADDR_MASK                    (0xFFFFFFFFU)
#define DMA4_DADDR_DADDR_SHIFT                   (0U)
/*! DADDR - Destination Address */
#define DMA4_DADDR_DADDR(x)                      (((uint32_t)(((uint32_t)(x)) << DMA4_DADDR_DADDR_SHIFT)) & DMA4_DADDR_DADDR_MASK)
/*! @} */

/* The count of DMA4_DADDR */
#define DMA4_DADDR_COUNT                         (64U)

/*! @name DOFF - TCD Signed Destination Address Offset Register */
/*! @{ */

#define DMA4_DOFF_DOFF_MASK                      (0xFFFFU)
#define DMA4_DOFF_DOFF_SHIFT                     (0U)
/*! DOFF - Destination Address Signed Offset */
#define DMA4_DOFF_DOFF(x)                        (((uint16_t)(((uint16_t)(x)) << DMA4_DOFF_DOFF_SHIFT)) & DMA4_DOFF_DOFF_MASK)
/*! @} */

/* The count of DMA4_DOFF */
#define DMA4_DOFF_COUNT                          (64U)

/*! @name CITER_ELINKNO - TCD Current Major Loop Count (Minor Loop Channel Linking Disabled) Register */
/*! @{ */

#define DMA4_CITER_ELINKNO_CITER_MASK            (0x7FFFU)
#define DMA4_CITER_ELINKNO_CITER_SHIFT           (0U)
/*! CITER - Current Major Iteration Count */
#define DMA4_CITER_ELINKNO_CITER(x)              (((uint16_t)(((uint16_t)(x)) << DMA4_CITER_ELINKNO_CITER_SHIFT)) & DMA4_CITER_ELINKNO_CITER_MASK)

#define DMA4_CITER_ELINKNO_ELINK_MASK            (0x8000U)
#define DMA4_CITER_ELINKNO_ELINK_SHIFT           (15U)
/*! ELINK - Enable channel-to-channel linking on minor-loop complete
 *  0b0..The channel-to-channel linking is disabled
 *  0b1..The channel-to-channel linking is enabled
 */
#define DMA4_CITER_ELINKNO_ELINK(x)              (((uint16_t)(((uint16_t)(x)) << DMA4_CITER_ELINKNO_ELINK_SHIFT)) & DMA4_CITER_ELINKNO_ELINK_MASK)
/*! @} */

/* The count of DMA4_CITER_ELINKNO */
#define DMA4_CITER_ELINKNO_COUNT                 (64U)

/*! @name CITER_ELINKYES - TCD Current Major Loop Count (Minor Loop Channel Linking Enabled) Register */
/*! @{ */

#define DMA4_CITER_ELINKYES_CITER_MASK           (0x1FFU)
#define DMA4_CITER_ELINKYES_CITER_SHIFT          (0U)
/*! CITER - Current Major Iteration Count */
#define DMA4_CITER_ELINKYES_CITER(x)             (((uint16_t)(((uint16_t)(x)) << DMA4_CITER_ELINKYES_CITER_SHIFT)) & DMA4_CITER_ELINKYES_CITER_MASK)

#define DMA4_CITER_ELINKYES_LINKCH_MASK          (0x7E00U)
#define DMA4_CITER_ELINKYES_LINKCH_SHIFT         (9U)
/*! LINKCH - Minor Loop Link Channel Number */
#define DMA4_CITER_ELINKYES_LINKCH(x)            (((uint16_t)(((uint16_t)(x)) << DMA4_CITER_ELINKYES_LINKCH_SHIFT)) & DMA4_CITER_ELINKYES_LINKCH_MASK)

#define DMA4_CITER_ELINKYES_ELINK_MASK           (0x8000U)
#define DMA4_CITER_ELINKYES_ELINK_SHIFT          (15U)
/*! ELINK - Enable channel-to-channel linking on minor-loop complete
 *  0b0..The channel-to-channel linking is disabled
 *  0b1..The channel-to-channel linking is enabled
 */
#define DMA4_CITER_ELINKYES_ELINK(x)             (((uint16_t)(((uint16_t)(x)) << DMA4_CITER_ELINKYES_ELINK_SHIFT)) & DMA4_CITER_ELINKYES_ELINK_MASK)
/*! @} */

/* The count of DMA4_CITER_ELINKYES */
#define DMA4_CITER_ELINKYES_COUNT                (64U)

/*! @name DLAST_SGA - TCD Last Destination Address Adjustment / Scatter Gather Address Register */
/*! @{ */

#define DMA4_DLAST_SGA_DLAST_SGA_MASK            (0xFFFFFFFFU)
#define DMA4_DLAST_SGA_DLAST_SGA_SHIFT           (0U)
/*! DLAST_SGA - Final Destination Address Adjustment / Scatter Gather Address */
#define DMA4_DLAST_SGA_DLAST_SGA(x)              (((uint32_t)(((uint32_t)(x)) << DMA4_DLAST_SGA_DLAST_SGA_SHIFT)) & DMA4_DLAST_SGA_DLAST_SGA_MASK)
/*! @} */

/* The count of DMA4_DLAST_SGA */
#define DMA4_DLAST_SGA_COUNT                     (64U)

/*! @name CSR - TCD Control and Status Register */
/*! @{ */

#define DMA4_CSR_START_MASK                      (0x1U)
#define DMA4_CSR_START_SHIFT                     (0U)
/*! START - Channel Start
 *  0b0..The channel is not explicitly started.
 *  0b1..The channel is explicitly started via a software initiated service request.
 */
#define DMA4_CSR_START(x)                        (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_START_SHIFT)) & DMA4_CSR_START_MASK)

#define DMA4_CSR_INTMAJOR_MASK                   (0x2U)
#define DMA4_CSR_INTMAJOR_SHIFT                  (1U)
/*! INTMAJOR - Enable an interrupt when major iteration count completes.
 *  0b0..The end-of-major loop interrupt is disabled.
 *  0b1..The end-of-major loop interrupt is enabled.
 */
#define DMA4_CSR_INTMAJOR(x)                     (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_INTMAJOR_SHIFT)) & DMA4_CSR_INTMAJOR_MASK)

#define DMA4_CSR_INTHALF_MASK                    (0x4U)
#define DMA4_CSR_INTHALF_SHIFT                   (2U)
/*! INTHALF - Enable an interrupt when major counter is half complete.
 *  0b0..The half-point interrupt is disabled.
 *  0b1..The half-point interrupt is enabled.
 */
#define DMA4_CSR_INTHALF(x)                      (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_INTHALF_SHIFT)) & DMA4_CSR_INTHALF_MASK)

#define DMA4_CSR_DREQ_MASK                       (0x8U)
#define DMA4_CSR_DREQ_SHIFT                      (3U)
/*! DREQ - Disable request
 *  0b0..No operation
 *  0b1..Clear the ERQ bit upon major loop completion, thus disabling hardware service requests.
 */
#define DMA4_CSR_DREQ(x)                         (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_DREQ_SHIFT)) & DMA4_CSR_DREQ_MASK)

#define DMA4_CSR_ESG_MASK                        (0x10U)
#define DMA4_CSR_ESG_SHIFT                       (4U)
/*! ESG - Enable Scatter/Gather processing
 *  0b0..The current channel's TCD is normal format.
 *  0b1..The current channel's TCD specifies a scatter gather format. The DLASTSGA field provides a memory pointer
 *       to the next TCD to be loaded into this channel after the major loop completes its execution.
 */
#define DMA4_CSR_ESG(x)                          (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_ESG_SHIFT)) & DMA4_CSR_ESG_MASK)

#define DMA4_CSR_MAJORELINK_MASK                 (0x20U)
#define DMA4_CSR_MAJORELINK_SHIFT                (5U)
/*! MAJORELINK - Enable channel-to-channel linking on major loop complete
 *  0b0..The channel-to-channel linking is disabled.
 *  0b1..The channel-to-channel linking is enabled.
 */
#define DMA4_CSR_MAJORELINK(x)                   (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_MAJORELINK_SHIFT)) & DMA4_CSR_MAJORELINK_MASK)

#define DMA4_CSR_EEOP_MASK                       (0x40U)
#define DMA4_CSR_EEOP_SHIFT                      (6U)
/*! EEOP - Enable end-of-packet processing
 *  0b0..The end-of-packet operation is disabled.
 *  0b1..The end-of-packet hardware input signal is enabled.
 */
#define DMA4_CSR_EEOP(x)                         (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_EEOP_SHIFT)) & DMA4_CSR_EEOP_MASK)

#define DMA4_CSR_ESDA_MASK                       (0x80U)
#define DMA4_CSR_ESDA_SHIFT                      (7U)
/*! ESDA - Enable store destination address
 *  0b0..The store destination address to system memory operation is disabled.
 *  0b1..The store destination address to system memory operation is enabled.
 */
#define DMA4_CSR_ESDA(x)                         (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_ESDA_SHIFT)) & DMA4_CSR_ESDA_MASK)

#define DMA4_CSR_MAJORLINKCH_MASK                (0x3F00U)
#define DMA4_CSR_MAJORLINKCH_SHIFT               (8U)
/*! MAJORLINKCH - Major loop link channel number */
#define DMA4_CSR_MAJORLINKCH(x)                  (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_MAJORLINKCH_SHIFT)) & DMA4_CSR_MAJORLINKCH_MASK)

#define DMA4_CSR_TMC_MASK                        (0xC000U)
#define DMA4_CSR_TMC_SHIFT                       (14U)
/*! TMC - Transfer Mode Control
 *  0b00..Read/Write
 *  0b01..Read Only
 *  0b10..Write Only
 *  0b11..Reserved
 */
#define DMA4_CSR_TMC(x)                          (((uint16_t)(((uint16_t)(x)) << DMA4_CSR_TMC_SHIFT)) & DMA4_CSR_TMC_MASK)
/*! @} */

/* The count of DMA4_CSR */
#define DMA4_CSR_COUNT                           (64U)

/*! @name BITER_ELINKNO - TCD Beginning Major Loop Count (Minor Loop Channel Linking Disabled) Register */
/*! @{ */

#define DMA4_BITER_ELINKNO_BITER_MASK            (0x7FFFU)
#define DMA4_BITER_ELINKNO_BITER_SHIFT           (0U)
/*! BITER - Starting Major Iteration Count */
#define DMA4_BITER_ELINKNO_BITER(x)              (((uint16_t)(((uint16_t)(x)) << DMA4_BITER_ELINKNO_BITER_SHIFT)) & DMA4_BITER_ELINKNO_BITER_MASK)

#define DMA4_BITER_ELINKNO_ELINK_MASK            (0x8000U)
#define DMA4_BITER_ELINKNO_ELINK_SHIFT           (15U)
/*! ELINK - Enables channel-to-channel linking on minor loop complete
 *  0b0..The channel-to-channel linking is disabled
 *  0b1..The channel-to-channel linking is enabled
 */
#define DMA4_BITER_ELINKNO_ELINK(x)              (((uint16_t)(((uint16_t)(x)) << DMA4_BITER_ELINKNO_ELINK_SHIFT)) & DMA4_BITER_ELINKNO_ELINK_MASK)
/*! @} */

/* The count of DMA4_BITER_ELINKNO */
#define DMA4_BITER_ELINKNO_COUNT                 (64U)

/*! @name BITER_ELINKYES - TCD Beginning Major Loop Count (Minor Loop Channel Linking Enabled) Register */
/*! @{ */

#define DMA4_BITER_ELINKYES_BITER_MASK           (0x1FFU)
#define DMA4_BITER_ELINKYES_BITER_SHIFT          (0U)
/*! BITER - Starting major iteration count */
#define DMA4_BITER_ELINKYES_BITER(x)             (((uint16_t)(((uint16_t)(x)) << DMA4_BITER_ELINKYES_BITER_SHIFT)) & DMA4_BITER_ELINKYES_BITER_MASK)

#define DMA4_BITER_ELINKYES_LINKCH_MASK          (0x7E00U)
#define DMA4_BITER_ELINKYES_LINKCH_SHIFT         (9U)
/*! LINKCH - Link Channel Number */
#define DMA4_BITER_ELINKYES_LINKCH(x)            (((uint16_t)(((uint16_t)(x)) << DMA4_BITER_ELINKYES_LINKCH_SHIFT)) & DMA4_BITER_ELINKYES_LINKCH_MASK)

#define DMA4_BITER_ELINKYES_ELINK_MASK           (0x8000U)
#define DMA4_BITER_ELINKYES_ELINK_SHIFT          (15U)
/*! ELINK - Enables channel-to-channel linking on minor loop complete
 *  0b0..The channel-to-channel linking is disabled
 *  0b1..The channel-to-channel linking is enabled
 */
#define DMA4_BITER_ELINKYES_ELINK(x)             (((uint16_t)(((uint16_t)(x)) << DMA4_BITER_ELINKYES_ELINK_SHIFT)) & DMA4_BITER_ELINKYES_ELINK_MASK)
/*! @} */

/* The count of DMA4_BITER_ELINKYES */
#define DMA4_BITER_ELINKYES_COUNT                (64U)


/*!
 * @}
 */ /* end of group DMA4_Register_Masks */


/* DMA4 - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral DMA4 base address */
  #define DMA4_BASE                                (0x52000000u)
  /** Peripheral DMA4 base address */
  #define DMA4_BASE_NS                             (0x42000000u)
  /** Peripheral DMA4 base pointer */
  #define DMA4                                     ((DMA4_Type *)DMA4_BASE)
  /** Peripheral DMA4 base pointer */
  #define DMA4_NS                                  ((DMA4_Type *)DMA4_BASE_NS)
  /** Array initializer of DMA4 peripheral base addresses */
  #define DMA4_BASE_ADDRS                          { DMA4_BASE }
  /** Array initializer of DMA4 peripheral base pointers */
  #define DMA4_BASE_PTRS                           { DMA4 }
  /** Array initializer of DMA4 peripheral base addresses */
  #define DMA4_BASE_ADDRS_NS                       { DMA4_BASE_NS }
  /** Array initializer of DMA4 peripheral base pointers */
  #define DMA4_BASE_PTRS_NS                        { DMA4_NS }
#else
  /** Peripheral DMA4 base address */
  #define DMA4_BASE                                (0x42000000u)
  /** Peripheral DMA4 base pointer */
  #define DMA4                                     ((DMA4_Type *)DMA4_BASE)
  /** Array initializer of DMA4 peripheral base addresses */
  #define DMA4_BASE_ADDRS                          { DMA4_BASE }
  /** Array initializer of DMA4 peripheral base pointers */
  #define DMA4_BASE_PTRS                           { DMA4 }
#endif
/** Interrupt vectors for the DMA4 peripheral type */
#define DMA4_IRQS                                { { DMA4_CH0_CH1_CH32_CH33_IRQn, DMA4_CH0_CH1_CH32_CH33_IRQn, DMA4_CH2_CH3_CH34_CH35_IRQn, DMA4_CH2_CH3_CH34_CH35_IRQn, DMA4_CH4_CH5_CH36_CH37_IRQn, DMA4_CH4_CH5_CH36_CH37_IRQn, DMA4_CH6_CH7_CH38_CH39_IRQn, DMA4_CH6_CH7_CH38_CH39_IRQn, DMA4_CH8_CH9_CH40_CH41_IRQn, DMA4_CH8_CH9_CH40_CH41_IRQn, DMA4_CH10_CH11_CH42_CH43_IRQn, DMA4_CH10_CH11_CH42_CH43_IRQn, DMA4_CH12_CH13_CH44_CH45_IRQn, DMA4_CH12_CH13_CH44_CH45_IRQn, DMA4_CH14_CH15_CH46_CH47_IRQn, DMA4_CH14_CH15_CH46_CH47_IRQn, DMA4_CH16_CH17_CH48_CH49_IRQn, DMA4_CH16_CH17_CH48_CH49_IRQn, DMA4_CH18_CH19_CH50_CH51_IRQn, DMA4_CH18_CH19_CH50_CH51_IRQn, DMA4_CH20_CH21_CH52_CH53_IRQn, DMA4_CH20_CH21_CH52_CH53_IRQn, DMA4_CH22_CH23_CH54_CH55_IRQn, DMA4_CH22_CH23_CH54_CH55_IRQn, DMA4_CH24_CH25_CH56_CH57_IRQn, DMA4_CH24_CH25_CH56_CH57_IRQn, DMA4_CH26_CH27_CH58_CH59_IRQn, DMA4_CH26_CH27_CH58_CH59_IRQn, DMA4_CH28_CH29_CH60_CH61_IRQn, DMA4_CH28_CH29_CH60_CH61_IRQn, DMA4_CH30_CH31_CH62_CH63_IRQn, DMA4_CH30_CH31_CH62_CH63_IRQn, DMA4_CH0_CH1_CH32_CH33_IRQn, DMA4_CH0_CH1_CH32_CH33_IRQn, DMA4_CH2_CH3_CH34_CH35_IRQn, DMA4_CH2_CH3_CH34_CH35_IRQn, DMA4_CH4_CH5_CH36_CH37_IRQn, DMA4_CH4_CH5_CH36_CH37_IRQn, DMA4_CH6_CH7_CH38_CH39_IRQn, DMA4_CH6_CH7_CH38_CH39_IRQn, DMA4_CH8_CH9_CH40_CH41_IRQn, DMA4_CH8_CH9_CH40_CH41_IRQn, DMA4_CH10_CH11_CH42_CH43_IRQn, DMA4_CH10_CH11_CH42_CH43_IRQn, DMA4_CH12_CH13_CH44_CH45_IRQn, DMA4_CH12_CH13_CH44_CH45_IRQn, DMA4_CH14_CH15_CH46_CH47_IRQn, DMA4_CH14_CH15_CH46_CH47_IRQn, DMA4_CH16_CH17_CH48_CH49_IRQn, DMA4_CH16_CH17_CH48_CH49_IRQn, DMA4_CH18_CH19_CH50_CH51_IRQn, DMA4_CH18_CH19_CH50_CH51_IRQn, DMA4_CH20_CH21_CH52_CH53_IRQn, DMA4_CH20_CH21_CH52_CH53_IRQn, DMA4_CH22_CH23_CH54_CH55_IRQn, DMA4_CH22_CH23_CH54_CH55_IRQn, DMA4_CH24_CH25_CH56_CH57_IRQn, DMA4_CH24_CH25_CH56_CH57_IRQn, DMA4_CH26_CH27_CH58_CH59_IRQn, DMA4_CH26_CH27_CH58_CH59_IRQn, DMA4_CH28_CH29_CH60_CH61_IRQn, DMA4_CH28_CH29_CH60_CH61_IRQn, DMA4_CH30_CH31_CH62_CH63_IRQn, DMA4_CH30_CH31_CH62_CH63_IRQn } }
#define DMA4_ERROR_IRQS                          { { DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn, DMA4_ERROR_IRQn } }

/*!
 * @}
 */ /* end of group DMA4_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ECAT Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ECAT_Peripheral_Access_Layer ECAT Peripheral Access Layer
 * @{
 */

/** ECAT - Register Layout Typedef */
typedef struct {
  __I  uint8_t TYPE;                               /**< Type, offset: 0x0 */
  __I  uint8_t REVISION;                           /**< Revision, offset: 0x1 */
  __I  uint16_t BUILD;                             /**< Build, offset: 0x2 */
  __I  uint8_t FMMUS_SUPPORTED;                    /**< FMMUs supported, offset: 0x4 */
  __I  uint8_t SYNCMANAGERS_SUPPORTED;             /**< SyncManagers supported, offset: 0x5 */
  __I  uint8_t RAM_SIZE;                           /**< RAM Size, offset: 0x6 */
  __I  uint8_t PORT_DESCRIPTOR;                    /**< Port configuration, offset: 0x7 */
  __I  uint16_t ESC_FEATURES_SUPPORTED;            /**< Register ESC Features supported, offset: 0x8 */
       uint8_t RESERVED_0[6];
  union {                                          /* offset: 0x10 */
    __IO uint16_t CONFIGURED_STATION_ADDRESS;        /**< Configured Station Address, offset: 0x10 */
    __I  uint16_t CONFIGURED_STATION_ADDRESS_PDI;    /**< Configured Station Address, offset: 0x10 */
  };
  union {                                          /* offset: 0x12 */
    __I  uint16_t CONFIGURED_STATION_ALIAS;          /**< Configured Station Alias, offset: 0x12 */
    __IO uint16_t CONFIGURED_STATION_ALIAS_PDI;      /**< Configured Station Alias, offset: 0x12 */
  };
       uint8_t RESERVED_1[12];
  union {                                          /* offset: 0x20 */
    __IO uint8_t REGISTER_WRITE_ENABLE;              /**< Register Write Enable, offset: 0x20 */
    __I  uint8_t REGISTER_WRITE_ENABLE_PDI;          /**< Register Write Enable, offset: 0x20 */
  };
  union {                                          /* offset: 0x21 */
    __IO uint8_t REGISTER_WRITE_PROTECTION;          /**< Register Write Protection, offset: 0x21 */
    __I  uint8_t REGISTER_WRITE_PROTECTION_PDI;      /**< Register Write Protection, offset: 0x21 */
  };
       uint8_t RESERVED_2[14];
  union {                                          /* offset: 0x30 */
    __IO uint8_t ESC_WRITE_ENABLE;                   /**< ESC Write Enable, offset: 0x30 */
    __I  uint8_t ESC_WRITE_ENABLE_PDI;               /**< ESC Write Enable, offset: 0x30 */
  };
  union {                                          /* offset: 0x31 */
    __IO uint8_t ESC_WRITE_PROTECTION;               /**< ESC Write Protection, offset: 0x31 */
    __I  uint8_t ESC_WRITE_PROTECTION_PDI;           /**< ESC Write Protection, offset: 0x31 */
  };
       uint8_t RESERVED_3[14];
  union {                                          /* offset: 0x40 */
    __IO uint8_t ESC_RESET_ECAT_WRITE;               /**< ESC Reset ECAT WRITE, offset: 0x40 */
    __I  uint8_t ESC_RESET_ECAT_WRITE_PDI;           /**< ESC Reset ECAT WRITE, offset: 0x40 */
  };
  union {                                          /* offset: 0x41 */
    __I  uint8_t ESC_RESET_PDI_WRITE;                /**< ESC Reset PDI WRITE, offset: 0x41 */
    __IO uint8_t ESC_RESET_PDI_WRITE_PDI;            /**< ESC Reset PDI WRITE, offset: 0x41 */
  };
       uint8_t RESERVED_4[190];
  union {                                          /* offset: 0x100 */
    __IO uint32_t ESC_DL_CONTROL;                    /**< ESC DL Control, offset: 0x100 */
    __I  uint32_t ESC_DL_CONTROL_PDI;                /**< ESC DL Control, offset: 0x100 */
  };
       uint8_t RESERVED_5[4];
  union {                                          /* offset: 0x108 */
    __IO uint16_t PHYSICAL_READ_WRITE_OFFSET;        /**< Physical Read Write Offset, offset: 0x108 */
    __I  uint16_t PHYSICAL_READ_WRITE_OFFSET_PDI;    /**< Physical Read Write Offset, offset: 0x108 */
  };
       uint8_t RESERVED_6[6];
  __I  uint16_t ESC_DL_STATUS;                     /**< ESC DL Status, offset: 0x110 */
       uint8_t RESERVED_7[14];
  union {                                          /* offset: 0x120 */
    __IO uint16_t AL_CONTROL;                        /**< AL Control, offset: 0x120 */
    __I  uint16_t AL_CONTROL_PDI;                    /**< AL Control, offset: 0x120 */
  };
       uint8_t RESERVED_8[14];
  union {                                          /* offset: 0x130 */
    __I  uint16_t AL_STATUS;                         /**< AL Status, offset: 0x130 */
    __IO uint16_t AL_STATUS_PDI;                     /**< AL Status, offset: 0x130 */
  };
       uint8_t RESERVED_9[2];
  union {                                          /* offset: 0x134 */
    __I  uint16_t AL_STATUS_CODE;                    /**< AL Status Code, offset: 0x134 */
    __IO uint16_t AL_STATUS_CODE_PDI;                /**< AL Status Code, offset: 0x134 */
  };
       uint8_t RESERVED_10[2];
  __IO uint8_t RUN_LED_OVERRIDE;                   /**< RUN LED Override, offset: 0x138 */
  __IO uint8_t ERR_LED_OVERRIDE;                   /**< ERR LED Override, offset: 0x139 */
       uint8_t RESERVED_11[6];
  __I  uint8_t PDI_CONTROL;                        /**< PDI Control, offset: 0x140 */
  __I  uint8_t ESC_CONFIGURATION;                  /**< ESC Configuration, offset: 0x141 */
       uint8_t RESERVED_12[12];
  __I  uint16_t PDI_INFORMATION;                   /**< PDI Information, offset: 0x14E */
  __I  uint8_t PDI_ON_CHIP_BUS_CONFIGURATION;      /**< Register PDI On-chip bus configuration, offset: 0x150 */
  __I  uint8_t SYNC_LATCH_1_AND_0_PDI_CONFIGURATION; /**< PDI Configuration Sync Latch 1 and 0 PDI Configuration, offset: 0x151 */
  __I  uint16_t PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION; /**< Register PDI On-chip bus extended configuration., offset: 0x152 */
       uint8_t RESERVED_13[172];
  union {                                          /* offset: 0x200 */
    __IO uint16_t ECAT_EVENT_MASK;                   /**< ECAT Event Mask, offset: 0x200 */
    __I  uint16_t ECAT_EVENT_MASK_PDI;               /**< ECAT Event Mask, offset: 0x200 */
  };
       uint8_t RESERVED_14[2];
  union {                                          /* offset: 0x204 */
    __I  uint32_t PDI_AL_EVENT_MASK;                 /**< PDI AL Event Mask, offset: 0x204 */
    __IO uint32_t PDI_AL_EVENT_MASK_PDI;             /**< PDI AL Event Mask, offset: 0x204 */
  };
       uint8_t RESERVED_15[8];
  __I  uint16_t ECAT_EVENT_REQUEST;                /**< ECAT Event Request, offset: 0x210 */
       uint8_t RESERVED_16[14];
  __I  uint32_t AL_EVENT_REQUEST;                  /**< AL Event request, offset: 0x220 */
       uint8_t RESERVED_17[220];
  union {                                          /* offset: 0x300, array step: 0x2 */
    __IO uint16_t RX_ERROR_COUNTER_PORT[2];          /**< RX Error Counter, array offset: 0x300, array step: 0x2 */
    __I  uint16_t RX_ERROR_COUNTER_PORT_PDI[2];      /**< RX Error Counter, array offset: 0x300, array step: 0x2 */
  };
       uint8_t RESERVED_18[4];
  union {                                          /* offset: 0x308, array step: 0x1 */
    __IO uint8_t FORWARDED_RX_ERROR_COUNTER_PORT[2];   /**< Forwarded RX Error Counter, array offset: 0x308, array step: 0x1 */
    __I  uint8_t FORWARDED_RX_ERROR_COUNTER_PORT_PDI[2];   /**< Forwarded RX Error Counter, array offset: 0x308, array step: 0x1 */
  };
       uint8_t RESERVED_19[2];
  union {                                          /* offset: 0x30C */
    __IO uint8_t ECAT_PROCESSING_UNIT_ERROR_COUNTER;   /**< ECAT Processing Unit Error Counter, offset: 0x30C */
    __I  uint8_t ECAT_PROCESSING_UNIT_ERROR_COUNTER_PDI;   /**< ECAT Processing Unit Error Counter, offset: 0x30C */
  };
  union {                                          /* offset: 0x30D */
    __IO uint8_t PDI_ERROR_COUNTER;                  /**< PDI Error counter, offset: 0x30D */
    __I  uint8_t PDI_ERROR_COUNTER_PDI;              /**< PDI Error counter, offset: 0x30D */
  };
  __I  uint8_t ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER; /**< ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_PDI_ERROR_CODE., offset: 0x30E */
       uint8_t RESERVED_20[1];
  union {                                          /* offset: 0x310, array step: 0x1 */
    __IO uint8_t LOST_LINK_COUNTER_PORT[2];          /**< Lost Link Counter, array offset: 0x310, array step: 0x1 */
    __I  uint8_t LOST_LINK_COUNTER_PORT_PDI[2];      /**< Lost Link Counter, array offset: 0x310, array step: 0x1 */
  };
       uint8_t RESERVED_21[238];
  union {                                          /* offset: 0x400 */
    __IO uint16_t WATCHDOG_DIVIDER;                  /**< Watchdog Divider, offset: 0x400 */
    __I  uint16_t WATCHDOG_DIVIDER_PDI;              /**< Watchdog Divider, offset: 0x400 */
  };
       uint8_t RESERVED_22[14];
  union {                                          /* offset: 0x410 */
    __IO uint16_t WATCHDOG_TIME_PDI;                 /**< Register Watchdog Time PDI, offset: 0x410 */
    __I  uint16_t WATCHDOG_TIME_PDI_PDI;             /**< Register Watchdog Time PDI, offset: 0x410 */
  };
       uint8_t RESERVED_23[14];
  union {                                          /* offset: 0x420 */
    __IO uint16_t WATCHDOG_TIME_PROCESS_DATA;        /**< Regsister Watchdog Time Process Data, offset: 0x420 */
    __I  uint16_t WATCHDOG_TIME_PROCESS_DATA_PDI;    /**< Regsister Watchdog Time Process Data, offset: 0x420 */
  };
       uint8_t RESERVED_24[30];
  __I  uint16_t WATCHDOG_STATUS_PROCESS_DATA;      /**< Watchdog Status Process Data, offset: 0x440 */
  union {                                          /* offset: 0x442 */
    __IO uint8_t WATCHDOG_COUNTER_PROCESS_DATA;      /**< Watchdog Counter Process Data, offset: 0x442 */
    __I  uint8_t WATCHDOG_COUNTER_PROCESS_DATA_PDI;   /**< Watchdog Counter Process Data, offset: 0x442 */
  };
  union {                                          /* offset: 0x443 */
    __IO uint8_t WATCHDOG_COUNTER_PDI;               /**< Watchdog Counter PDI, offset: 0x443 */
    __I  uint8_t WATCHDOG_COUNTER_PDI_PDI;           /**< Watchdog Counter PDI, offset: 0x443 */
  };
       uint8_t RESERVED_25[188];
  union {                                          /* offset: 0x500 */
    __IO uint8_t EEPROM_CONFIGURATION;               /**< EEPROM Configuration, offset: 0x500 */
    __I  uint8_t EEPROM_CONFIGURATION_PDI;           /**< EEPROM Configuration, offset: 0x500 */
  };
  union {                                          /* offset: 0x501 */
    __I  uint8_t REGISTER_EEPROM_PDI_ACCESS_STATE;   /**< EEPROM PDI Access State, offset: 0x501 */
    __IO uint8_t REGISTER_EEPROM_PDI_ACCESS_STATE_PDI;   /**< EEPROM PDI Access State, offset: 0x501 */
  };
  union {                                          /* offset: 0x502 */
    __IO uint16_t EEPROM_CONTROL_STATUS;             /**< Register EEPROM Control/Status, offset: 0x502 */
    __IO uint16_t EEPROM_CONTROL_STATUS_PDI;         /**< Register EEPROM Control/Status, offset: 0x502 */
  };
  __IO uint32_t EEPROM_ADDRESS;                    /**< EEPROM Address, offset: 0x504 */
  __IO uint64_t EEPROM_DATA;                       /**< EEPROM Data, offset: 0x508 */
  union {                                          /* offset: 0x510 */
    __IO uint16_t MII_MANAGEMENT_CONTROL_OR_STATUS;   /**< MII Management Control/Status, offset: 0x510 */
    __IO uint16_t MII_MANAGEMENT_CONTROL_OR_STATUS_PDI;   /**< MII Management Control/Status, offset: 0x510 */
  };
  __IO uint8_t PHY_ADDRESS;                        /**< PHY Address, offset: 0x512 */
  __IO uint8_t PHY_REGISTER_ADDRESS;               /**< PHY Register Address, offset: 0x513 */
  __IO uint16_t PHY_DATA;                          /**< PHY Data, offset: 0x514 */
  union {                                          /* offset: 0x516 */
    __IO uint8_t MII_MANAGEMENT_ECAT_ACCESS_STATE;   /**< MII Management ECAT Access State, offset: 0x516 */
    __I  uint8_t MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI;   /**< MII Management ECAT Access State, offset: 0x516 */
  };
  union {                                          /* offset: 0x517 */
    __IO uint8_t MII_MANAGEMENT_PDI_ACCESS_STATE;    /**< MII Management PDI Access State, offset: 0x517 */
    __IO uint8_t MII_MANAGEMENT_PDI_ACCESS_STATE_PDI;   /**< MII Management PDI Access State, offset: 0x517 */
  };
  __IO uint8_t PHY_PORT_STATUS[2];                 /**< PHY Port, array offset: 0x518, array step: 0x1 */
       uint8_t RESERVED_26[230];
  struct {                                         /* offset: 0x600, array step: 0x10 */
    union {                                          /* offset: 0x600, array step: 0x10 */
      __IO uint32_t LOGICAL_START_ADDRESS;             /**< Register Logical Start address FMMU, array offset: 0x600, array step: 0x10 */
      __I  uint32_t LOGICAL_START_ADDRESS_PDI;         /**< Register Logical Start address FMMU, array offset: 0x600, array step: 0x10 */
    };
    union {                                          /* offset: 0x604, array step: 0x10 */
      __IO uint16_t LENGTH;                            /**< Register Length FMMU, array offset: 0x604, array step: 0x10 */
      __I  uint16_t LENGTH_PDI;                        /**< Register Length FMMU, array offset: 0x604, array step: 0x10 */
    };
    union {                                          /* offset: 0x606, array step: 0x10 */
      __IO uint8_t LOGICAL_START_BIT;                  /**< Register Start bit FMMU y in logical address space, array offset: 0x606, array step: 0x10 */
      __I  uint8_t LOGICAL_START_BIT_PDI;              /**< Register Start bit FMMU y in logical address space, array offset: 0x606, array step: 0x10 */
    };
    union {                                          /* offset: 0x607, array step: 0x10 */
      __IO uint8_t LOGICAL_STOP_BIT;                   /**< Register Stop bit FMMU y in logical address space, array offset: 0x607, array step: 0x10 */
      __I  uint8_t LOGICAL_STOP_BIT_PDI;               /**< Register Stop bit FMMU y in logical address space, array offset: 0x607, array step: 0x10 */
    };
    union {                                          /* offset: 0x608, array step: 0x10 */
      __IO uint16_t PHYSICAL_START_ADDRESS;            /**< Register Physical Start address FMMU, array offset: 0x608, array step: 0x10 */
      __I  uint16_t PHYSICAL_START_ADDRESS_PDI;        /**< Register Physical Start address FMMU, array offset: 0x608, array step: 0x10 */
    };
    union {                                          /* offset: 0x60A, array step: 0x10 */
      __IO uint8_t PHYSICAL_START_BIT;                 /**< Register Physical Start bit FMMU, array offset: 0x60A, array step: 0x10 */
      __I  uint8_t PHYSICAL_START_BIT_PDI;             /**< Register Physical Start bit FMMU, array offset: 0x60A, array step: 0x10 */
    };
    union {                                          /* offset: 0x60B, array step: 0x10 */
      __IO uint8_t TYPE;                               /**< Register Type FMMU y, array offset: 0x60B, array step: 0x10 */
      __I  uint8_t TYPE_PDI;                           /**< Register Type FMMU y, array offset: 0x60B, array step: 0x10 */
    };
    union {                                          /* offset: 0x60C, array step: 0x10 */
      __IO uint8_t ACTIVATE;                           /**< Register Activate FMMU, array offset: 0x60C, array step: 0x10 */
      __I  uint8_t ACTIVATE_PDI;                       /**< Register Activate FMMU, array offset: 0x60C, array step: 0x10 */
    };
         uint8_t RESERVED_0[3];
  } FMMU[8];
       uint8_t RESERVED_27[384];
  struct {                                         /* offset: 0x800, array step: 0x8 */
    union {                                          /* offset: 0x800, array step: 0x8 */
      __IO uint16_t PHYSICAL_START_ADDRESS;            /**< Register physical Start Address SyncManager, array offset: 0x800, array step: 0x8 */
      __I  uint16_t PHYSICAL_START_ADDRESS_PDI;        /**< Register physical Start Address SyncManager, array offset: 0x800, array step: 0x8 */
    };
    union {                                          /* offset: 0x802, array step: 0x8 */
      __IO uint16_t LENGTH;                            /**< Register Length SyncManager, array offset: 0x802, array step: 0x8 */
      __I  uint16_t LENGTH_PDI;                        /**< Register Length SyncManager, array offset: 0x802, array step: 0x8 */
    };
    union {                                          /* offset: 0x804, array step: 0x8 */
      __IO uint8_t CONTROL_REGISTER;                   /**< Register Control Register SyncManager, array offset: 0x804, array step: 0x8 */
      __I  uint8_t CONTROL_REGISTER_PDI;               /**< Register Control Register SyncManager, array offset: 0x804, array step: 0x8 */
    };
    __I  uint8_t STATUS;                             /**< Register Status Register SyncManager, array offset: 0x805, array step: 0x8 */
    union {                                          /* offset: 0x806, array step: 0x8 */
      __IO uint8_t ACTIVATE;                           /**< Register Activate SyncManager, array offset: 0x806, array step: 0x8 */
      __I  uint8_t ACTIVATE_PDI;                       /**< Register Activate SyncManager, array offset: 0x806, array step: 0x8 */
    };
    union {                                          /* offset: 0x807, array step: 0x8 */
      __I  uint8_t PDI_CONTROL;                        /**< Register PDI Control SyncManager, array offset: 0x807, array step: 0x8 */
      __IO uint8_t PDI_CONTROL_PDI;                    /**< Register PDI Control SyncManager, array offset: 0x807, array step: 0x8 */
    };
  } SYNCMANAGER[16];
       uint8_t RESERVED_28[128];
  union {                                          /* offset: 0x900 */
    __IO uint32_t RECEIVE_TIMES;                     /**< Distributed Clocks Receive Times, offset: 0x900 */
    __I  uint32_t RECEIVE_TIMES_PDI;                 /**< Distributed Clocks Receive Times, offset: 0x900 */
  };
  __I  uint32_t RECEIVE_TIME_PORT_1;               /**< Distributed Clocks Receive Time Port 1, offset: 0x904 */
       uint8_t RESERVED_29[8];
  union {                                          /* offset: 0x910 */
    __IO uint64_t SYSTEM_TIME;                       /**< Register System Time, offset: 0x910 */
    __IO uint64_t SYSTEM_TIME_PDI;                   /**< Register System Time, offset: 0x910 */
  };
  __I  uint64_t RECEIVE_TIME_ECAT_PROCESSING_UNIT; /**< Distributed Clocks Register Receive Time ECAT Processing Unit, offset: 0x918 */
  __IO uint64_t SYSTEM_TIME_OFFSET;                /**< Register System Time Offset, offset: 0x920 */
  __IO uint32_t SYSTEM_TIME_DELAY;                 /**< Register System Time Delay, offset: 0x928 */
  __I  uint32_t SYSTEM_TIME_DIFFERENCE;            /**< Register System Time Difference, offset: 0x92C */
  __IO uint16_t SPEED_COUNTER_START;               /**< Register Speed Counter Start, offset: 0x930 */
  __I  uint16_t SPEED_COUNTER_DIFF;                /**< Register Speed Counter Diff, offset: 0x932 */
  __IO uint8_t SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH; /**< Register System Time Difference Filter Depth, offset: 0x934 */
  __IO uint8_t SPEED_COUNTER_FILTER_DEPTH;         /**< Register Speed Counter Filter Depth, offset: 0x935 */
       uint8_t RESERVED_30[74];
  union {                                          /* offset: 0x980 */
    __IO uint8_t CYCLIC_UNIT_CONTROL;                /**< Register Cyclic Unit Control, offset: 0x980 */
    __I  uint8_t CYCLIC_UNIT_CONTROL_PDI;            /**< Register Cyclic Unit Control, offset: 0x980 */
  };
  __IO uint8_t UNIT_ACTIVATION_REGISTER;           /**< Register Activation register, offset: 0x981 */
  __I  uint16_t UNI_PULSE_LENGTH_OF_SYNCSIGNALS;   /**< Register Pulse Length of SyncSignals, offset: 0x982 */
  __I  uint8_t UNIT_ACTIVATION_STATUS;             /**< Register Activation Status, offset: 0x984 */
       uint8_t RESERVED_31[9];
  __I  uint8_t UNIT_SYNC0_STATUS;                  /**< Register SYNC0 Status, offset: 0x98E */
  __I  uint8_t UNIT_SYNC1_STATUS;                  /**< Register SYNC1 Status, offset: 0x98F */
  __IO uint64_t UNIT_START_TIME_CYCLIC_OPERATION;  /**< Register Start Time Cyclic Operation, offset: 0x990 */
  __I  uint64_t UNIT_NEXT_SYNC1_PULSE;             /**< Register Next SYNC1 Pulse, offset: 0x998 */
  __IO uint32_t UNIT_SYNC0_CYCLE_TIME;             /**< Register SYNC0 Cycle Time, offset: 0x9A0 */
  __IO uint32_t UNIT_SYNC1_CYCLE_TIME;             /**< Register SYNC1 Cycle Time, offset: 0x9A4 */
  __IO uint8_t LATCH0_CONTROL;                     /**< Register Latch0 Control, offset: 0x9A8 */
  __IO uint8_t LATCH1_CONTROL;                     /**< Register Latch1 Control, offset: 0x9A9 */
       uint8_t RESERVED_32[4];
  __I  uint8_t LATCH0_STATUS;                      /**< Register Latch0 Status, offset: 0x9AE */
  __I  uint8_t LATCH1_STATUS;                      /**< Register Latch1 Status, offset: 0x9AF */
  __I  uint64_t LATCH0_TIME_POSITIVE_EDGE;         /**< Register Latch0 Time Positive Edge, offset: 0x9B0 */
  __I  uint64_t LATCH0_TIME_NEGATIVE_EDGE;         /**< Register Latch0 Time Negative Edge, offset: 0x9B8 */
  __I  uint64_t LATCH1_TIME_POSITIVE_EDGE;         /**< Register Latch1 Time Positive Edge, offset: 0x9C0 */
  __I  uint64_t LATCH1_TIME_NEGATIVE_EDGE;         /**< Register Latch1 Time Negative Edge, offset: 0x9C8 */
       uint8_t RESERVED_33[32];
  __I  uint32_t ETHERCAT_BUFFER_CHANGE_EVENT_TIME; /**< Register EtherCAT Buffer Change Event Time, offset: 0x9F0 */
       uint8_t RESERVED_34[4];
  __I  uint32_t PDI_BUFFER_START_EVENT_TIME;       /**< Register PDI Buffer Start Event Time, offset: 0x9F8 */
  __I  uint32_t PDI_BUFFER_CHANGE_EVENT_TIME;      /**< Register PDI Buffer Change Event Time, offset: 0x9FC */
       uint8_t RESERVED_35[1024];
  __I  uint64_t PRODUCT_ID_IP_CORE;                /**< Register Product ID IP Core, offset: 0xE00 */
  __I  uint64_t VENDOR_ID_IP_CORE;                 /**< Register Vendor ID IP Core, offset: 0xE08 */
       uint8_t RESERVED_36[256];
  __IO uint16_t GENERAL_PURPOSE_OUTPUTS;           /**< Register General Purpose Outputs, offset: 0xF10 */
       uint8_t RESERVED_37[6];
  __I  uint16_t GENERAL_PURPOSE_INPUTS;            /**< Register General Purpose Inputs, offset: 0xF18 */
} ECAT_Type;

/* ----------------------------------------------------------------------------
   -- ECAT Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ECAT_Register_Masks ECAT Register Masks
 * @{
 */

/*! @name TYPE - Type */
/*! @{ */

#define ECAT_TYPE_BF0_MASK                       (0xFFU)
#define ECAT_TYPE_BF0_SHIFT                      (0U)
/*! BF0 - Type of EtherCAT controller */
#define ECAT_TYPE_BF0(x)                         (((uint8_t)(((uint8_t)(x)) << ECAT_TYPE_BF0_SHIFT)) & ECAT_TYPE_BF0_MASK)
/*! @} */

/*! @name REVISION - Revision */
/*! @{ */

#define ECAT_REVISION_BF0_MASK                   (0xFFU)
#define ECAT_REVISION_BF0_SHIFT                  (0U)
/*! BF0 - Revision of EtherCAT controller. */
#define ECAT_REVISION_BF0(x)                     (((uint8_t)(((uint8_t)(x)) << ECAT_REVISION_BF0_SHIFT)) & ECAT_REVISION_BF0_MASK)
/*! @} */

/*! @name BUILD - Build */
/*! @{ */

#define ECAT_BUILD_BF0_MASK                      (0xFFFFU)
#define ECAT_BUILD_BF0_SHIFT                     (0U)
/*! BF0 - Build of EtherCAT controller */
#define ECAT_BUILD_BF0(x)                        (((uint16_t)(((uint16_t)(x)) << ECAT_BUILD_BF0_SHIFT)) & ECAT_BUILD_BF0_MASK)
/*! @} */

/*! @name FMMUS_SUPPORTED - FMMUs supported */
/*! @{ */

#define ECAT_FMMUS_SUPPORTED_BF0_MASK            (0xFFU)
#define ECAT_FMMUS_SUPPORTED_BF0_SHIFT           (0U)
/*! BF0 - Number of supported FMMU channels (or entities) */
#define ECAT_FMMUS_SUPPORTED_BF0(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_FMMUS_SUPPORTED_BF0_SHIFT)) & ECAT_FMMUS_SUPPORTED_BF0_MASK)
/*! @} */

/*! @name SYNCMANAGERS_SUPPORTED - SyncManagers supported */
/*! @{ */

#define ECAT_SYNCMANAGERS_SUPPORTED_BF0_MASK     (0xFFU)
#define ECAT_SYNCMANAGERS_SUPPORTED_BF0_SHIFT    (0U)
/*! BF0 - Number of supported SyncManager channels (or entities) */
#define ECAT_SYNCMANAGERS_SUPPORTED_BF0(x)       (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGERS_SUPPORTED_BF0_SHIFT)) & ECAT_SYNCMANAGERS_SUPPORTED_BF0_MASK)
/*! @} */

/*! @name RAM_SIZE - RAM Size */
/*! @{ */

#define ECAT_RAM_SIZE_BF0_MASK                   (0xFFU)
#define ECAT_RAM_SIZE_BF0_SHIFT                  (0U)
/*! BF0 - Process Data RAM size supported in KByte */
#define ECAT_RAM_SIZE_BF0(x)                     (((uint8_t)(((uint8_t)(x)) << ECAT_RAM_SIZE_BF0_SHIFT)) & ECAT_RAM_SIZE_BF0_MASK)
/*! @} */

/*! @name PORT_DESCRIPTOR - Port configuration */
/*! @{ */

#define ECAT_PORT_DESCRIPTOR_BF0_MASK            (0x3U)
#define ECAT_PORT_DESCRIPTOR_BF0_SHIFT           (0U)
/*! BF0 - Port 0 */
#define ECAT_PORT_DESCRIPTOR_BF0(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PORT_DESCRIPTOR_BF0_SHIFT)) & ECAT_PORT_DESCRIPTOR_BF0_MASK)

#define ECAT_PORT_DESCRIPTOR_BF2_MASK            (0xCU)
#define ECAT_PORT_DESCRIPTOR_BF2_SHIFT           (2U)
/*! BF2 - Port 1 */
#define ECAT_PORT_DESCRIPTOR_BF2(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PORT_DESCRIPTOR_BF2_SHIFT)) & ECAT_PORT_DESCRIPTOR_BF2_MASK)

#define ECAT_PORT_DESCRIPTOR_BF4_MASK            (0x30U)
#define ECAT_PORT_DESCRIPTOR_BF4_SHIFT           (4U)
/*! BF4 - Reserved */
#define ECAT_PORT_DESCRIPTOR_BF4(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PORT_DESCRIPTOR_BF4_SHIFT)) & ECAT_PORT_DESCRIPTOR_BF4_MASK)

#define ECAT_PORT_DESCRIPTOR_BF6_MASK            (0xC0U)
#define ECAT_PORT_DESCRIPTOR_BF6_SHIFT           (6U)
/*! BF6 - Reserved */
#define ECAT_PORT_DESCRIPTOR_BF6(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PORT_DESCRIPTOR_BF6_SHIFT)) & ECAT_PORT_DESCRIPTOR_BF6_MASK)
/*! @} */

/*! @name ESC_FEATURES_SUPPORTED - Register ESC Features supported */
/*! @{ */

#define ECAT_ESC_FEATURES_SUPPORTED_BF0_MASK     (0x1U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF0_SHIFT    (0U)
/*! BF0 - FMMU Operation:
 *  0b0..Bit oriented
 *  0b1..Byte oriented
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF0(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF0_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF0_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF1_MASK     (0x2U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF1_SHIFT    (1U)
/*! BF1 - Unused register access:
 *  0b0..allowed
 *  0b1..not supported
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF1(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF1_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF1_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF2_MASK     (0x4U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF2_SHIFT    (2U)
/*! BF2 - Distributed Clocks:
 *  0b0..Not available
 *  0b1..Available
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF2(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF2_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF2_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF3_MASK     (0x8U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF3_SHIFT    (3U)
/*! BF3 - Distributed Clocks (width):
 *  0b0..32 bit
 *  0b1..64 bit
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF3(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF3_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF3_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF6_MASK     (0x40U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF6_SHIFT    (6U)
/*! BF6 - Enhanced Link Detection MII:
 *  0b0..Not available
 *  0b1..Available
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF6(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF6_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF6_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF7_MASK     (0x80U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF7_SHIFT    (7U)
/*! BF7 - Separate Handling of FCS Errors:
 *  0b0..Not supported
 *  0b1..Supported, frames with wrong FCS and additional nibble will be counted separately in Forwarded RX Error Counter
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF7(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF7_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF7_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF8_MASK     (0x100U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF8_SHIFT    (8U)
/*! BF8 - Enhanced DC SYNC Activation
 *  0b0..Not available
 *  0b1..Available
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF8(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF8_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF8_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF9_MASK     (0x200U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF9_SHIFT    (9U)
/*! BF9 - EtherCAT LRW command support:
 *  0b0..Supported
 *  0b1..Not supported
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF9(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF9_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF9_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF10_MASK    (0x400U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF10_SHIFT   (10U)
/*! BF10 - EtherCAT read/write command support (BRW, APRW, FPRW):
 *  0b0..Supported
 *  0b1..Not supported
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF10(x)      (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF10_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF10_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF11_MASK    (0x800U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF11_SHIFT   (11U)
/*! BF11 - Fixed FMMU/SyncManager configuration
 *  0b0..Variable configuration
 *  0b1..Fixed configuration (refer to documentation of supporting ESCs)
 */
#define ECAT_ESC_FEATURES_SUPPORTED_BF11(x)      (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF11_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF11_MASK)

#define ECAT_ESC_FEATURES_SUPPORTED_BF12_MASK    (0xF000U)
#define ECAT_ESC_FEATURES_SUPPORTED_BF12_SHIFT   (12U)
/*! BF12 - Reserved */
#define ECAT_ESC_FEATURES_SUPPORTED_BF12(x)      (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_FEATURES_SUPPORTED_BF12_SHIFT)) & ECAT_ESC_FEATURES_SUPPORTED_BF12_MASK)
/*! @} */

/*! @name CONFIGURED_STATION_ADDRESS - Configured Station Address */
/*! @{ */

#define ECAT_CONFIGURED_STATION_ADDRESS_BF0_MASK (0xFFFFU)
#define ECAT_CONFIGURED_STATION_ADDRESS_BF0_SHIFT (0U)
/*! BF0 - Address used for node addressing (FPRD/FPWR/FPRW/FRMW commands). */
#define ECAT_CONFIGURED_STATION_ADDRESS_BF0(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_CONFIGURED_STATION_ADDRESS_BF0_SHIFT)) & ECAT_CONFIGURED_STATION_ADDRESS_BF0_MASK)
/*! @} */

/*! @name CONFIGURED_STATION_ADDRESS_PDI - Configured Station Address */
/*! @{ */

#define ECAT_CONFIGURED_STATION_ADDRESS_PDI_BF0_MASK (0xFFFFU)
#define ECAT_CONFIGURED_STATION_ADDRESS_PDI_BF0_SHIFT (0U)
/*! BF0 - Address used for node addressing (FPRD/FPWR/FPRW/FRMW commands). */
#define ECAT_CONFIGURED_STATION_ADDRESS_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_CONFIGURED_STATION_ADDRESS_PDI_BF0_SHIFT)) & ECAT_CONFIGURED_STATION_ADDRESS_PDI_BF0_MASK)
/*! @} */

/*! @name CONFIGURED_STATION_ALIAS - Configured Station Alias */
/*! @{ */

#define ECAT_CONFIGURED_STATION_ALIAS_BF0_MASK   (0xFFFFU)
#define ECAT_CONFIGURED_STATION_ALIAS_BF0_SHIFT  (0U)
/*! BF0 - Alias Address used for node addressing (FPRD/FPWR/FPRW/FRMW commands). */
#define ECAT_CONFIGURED_STATION_ALIAS_BF0(x)     (((uint16_t)(((uint16_t)(x)) << ECAT_CONFIGURED_STATION_ALIAS_BF0_SHIFT)) & ECAT_CONFIGURED_STATION_ALIAS_BF0_MASK)
/*! @} */

/*! @name CONFIGURED_STATION_ALIAS_PDI - Configured Station Alias */
/*! @{ */

#define ECAT_CONFIGURED_STATION_ALIAS_PDI_BF0_MASK (0xFFFFU)
#define ECAT_CONFIGURED_STATION_ALIAS_PDI_BF0_SHIFT (0U)
/*! BF0 - Alias Address used for node addressing (FPRD/FPWR/FPRW/FRMW commands). */
#define ECAT_CONFIGURED_STATION_ALIAS_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_CONFIGURED_STATION_ALIAS_PDI_BF0_SHIFT)) & ECAT_CONFIGURED_STATION_ALIAS_PDI_BF0_MASK)
/*! @} */

/*! @name REGISTER_WRITE_ENABLE - Register Write Enable */
/*! @{ */

#define ECAT_REGISTER_WRITE_ENABLE_BF0_MASK      (0x1U)
#define ECAT_REGISTER_WRITE_ENABLE_BF0_SHIFT     (0U)
/*! BF0 - Register Write Enable. */
#define ECAT_REGISTER_WRITE_ENABLE_BF0(x)        (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_ENABLE_BF0_SHIFT)) & ECAT_REGISTER_WRITE_ENABLE_BF0_MASK)

#define ECAT_REGISTER_WRITE_ENABLE_BF1_MASK      (0xFEU)
#define ECAT_REGISTER_WRITE_ENABLE_BF1_SHIFT     (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_REGISTER_WRITE_ENABLE_BF1(x)        (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_ENABLE_BF1_SHIFT)) & ECAT_REGISTER_WRITE_ENABLE_BF1_MASK)
/*! @} */

/*! @name REGISTER_WRITE_ENABLE_PDI - Register Write Enable */
/*! @{ */

#define ECAT_REGISTER_WRITE_ENABLE_PDI_BF0_MASK  (0x1U)
#define ECAT_REGISTER_WRITE_ENABLE_PDI_BF0_SHIFT (0U)
/*! BF0 - Register Write Enable. */
#define ECAT_REGISTER_WRITE_ENABLE_PDI_BF0(x)    (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_ENABLE_PDI_BF0_SHIFT)) & ECAT_REGISTER_WRITE_ENABLE_PDI_BF0_MASK)

#define ECAT_REGISTER_WRITE_ENABLE_PDI_BF1_MASK  (0xFEU)
#define ECAT_REGISTER_WRITE_ENABLE_PDI_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_REGISTER_WRITE_ENABLE_PDI_BF1(x)    (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_ENABLE_PDI_BF1_SHIFT)) & ECAT_REGISTER_WRITE_ENABLE_PDI_BF1_MASK)
/*! @} */

/*! @name REGISTER_WRITE_PROTECTION - Register Write Protection */
/*! @{ */

#define ECAT_REGISTER_WRITE_PROTECTION_BF0_MASK  (0x1U)
#define ECAT_REGISTER_WRITE_PROTECTION_BF0_SHIFT (0U)
/*! BF0 - Register write protection.
 *  0b0..Protection disabled
 *  0b1..Protection enabled
 */
#define ECAT_REGISTER_WRITE_PROTECTION_BF0(x)    (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_PROTECTION_BF0_SHIFT)) & ECAT_REGISTER_WRITE_PROTECTION_BF0_MASK)

#define ECAT_REGISTER_WRITE_PROTECTION_BF1_MASK  (0xFEU)
#define ECAT_REGISTER_WRITE_PROTECTION_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_REGISTER_WRITE_PROTECTION_BF1(x)    (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_PROTECTION_BF1_SHIFT)) & ECAT_REGISTER_WRITE_PROTECTION_BF1_MASK)
/*! @} */

/*! @name REGISTER_WRITE_PROTECTION_PDI - Register Write Protection */
/*! @{ */

#define ECAT_REGISTER_WRITE_PROTECTION_PDI_BF0_MASK (0x1U)
#define ECAT_REGISTER_WRITE_PROTECTION_PDI_BF0_SHIFT (0U)
/*! BF0 - Register write protection.
 *  0b0..Protection disabled
 *  0b1..Protection enabled
 */
#define ECAT_REGISTER_WRITE_PROTECTION_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_PROTECTION_PDI_BF0_SHIFT)) & ECAT_REGISTER_WRITE_PROTECTION_PDI_BF0_MASK)

#define ECAT_REGISTER_WRITE_PROTECTION_PDI_BF1_MASK (0xFEU)
#define ECAT_REGISTER_WRITE_PROTECTION_PDI_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_REGISTER_WRITE_PROTECTION_PDI_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_WRITE_PROTECTION_PDI_BF1_SHIFT)) & ECAT_REGISTER_WRITE_PROTECTION_PDI_BF1_MASK)
/*! @} */

/*! @name ESC_WRITE_ENABLE - ESC Write Enable */
/*! @{ */

#define ECAT_ESC_WRITE_ENABLE_BF0_MASK           (0x1U)
#define ECAT_ESC_WRITE_ENABLE_BF0_SHIFT          (0U)
/*! BF0 - ESC Write Enable */
#define ECAT_ESC_WRITE_ENABLE_BF0(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_ENABLE_BF0_SHIFT)) & ECAT_ESC_WRITE_ENABLE_BF0_MASK)

#define ECAT_ESC_WRITE_ENABLE_BF1_MASK           (0xFEU)
#define ECAT_ESC_WRITE_ENABLE_BF1_SHIFT          (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_ESC_WRITE_ENABLE_BF1(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_ENABLE_BF1_SHIFT)) & ECAT_ESC_WRITE_ENABLE_BF1_MASK)
/*! @} */

/*! @name ESC_WRITE_ENABLE_PDI - ESC Write Enable */
/*! @{ */

#define ECAT_ESC_WRITE_ENABLE_PDI_BF0_MASK       (0x1U)
#define ECAT_ESC_WRITE_ENABLE_PDI_BF0_SHIFT      (0U)
/*! BF0 - ESC Write Enable */
#define ECAT_ESC_WRITE_ENABLE_PDI_BF0(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_ENABLE_PDI_BF0_SHIFT)) & ECAT_ESC_WRITE_ENABLE_PDI_BF0_MASK)

#define ECAT_ESC_WRITE_ENABLE_PDI_BF1_MASK       (0xFEU)
#define ECAT_ESC_WRITE_ENABLE_PDI_BF1_SHIFT      (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_ESC_WRITE_ENABLE_PDI_BF1(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_ENABLE_PDI_BF1_SHIFT)) & ECAT_ESC_WRITE_ENABLE_PDI_BF1_MASK)
/*! @} */

/*! @name ESC_WRITE_PROTECTION - ESC Write Protection */
/*! @{ */

#define ECAT_ESC_WRITE_PROTECTION_BF0_MASK       (0x1U)
#define ECAT_ESC_WRITE_PROTECTION_BF0_SHIFT      (0U)
/*! BF0 - Write protect:
 *  0b0..Protection disabled
 *  0b1..Protection enabled
 */
#define ECAT_ESC_WRITE_PROTECTION_BF0(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_PROTECTION_BF0_SHIFT)) & ECAT_ESC_WRITE_PROTECTION_BF0_MASK)

#define ECAT_ESC_WRITE_PROTECTION_BF1_MASK       (0xFEU)
#define ECAT_ESC_WRITE_PROTECTION_BF1_SHIFT      (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_ESC_WRITE_PROTECTION_BF1(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_PROTECTION_BF1_SHIFT)) & ECAT_ESC_WRITE_PROTECTION_BF1_MASK)
/*! @} */

/*! @name ESC_WRITE_PROTECTION_PDI - ESC Write Protection */
/*! @{ */

#define ECAT_ESC_WRITE_PROTECTION_PDI_BF0_MASK   (0x1U)
#define ECAT_ESC_WRITE_PROTECTION_PDI_BF0_SHIFT  (0U)
/*! BF0 - Write protect:
 *  0b0..Protection disabled
 *  0b1..Protection enabled
 */
#define ECAT_ESC_WRITE_PROTECTION_PDI_BF0(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_PROTECTION_PDI_BF0_SHIFT)) & ECAT_ESC_WRITE_PROTECTION_PDI_BF0_MASK)

#define ECAT_ESC_WRITE_PROTECTION_PDI_BF1_MASK   (0xFEU)
#define ECAT_ESC_WRITE_PROTECTION_PDI_BF1_SHIFT  (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_ESC_WRITE_PROTECTION_PDI_BF1(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_WRITE_PROTECTION_PDI_BF1_SHIFT)) & ECAT_ESC_WRITE_PROTECTION_PDI_BF1_MASK)
/*! @} */

/*! @name ESC_RESET_ECAT_WRITE - ESC Reset ECAT WRITE */
/*! @{ */

#define ECAT_ESC_RESET_ECAT_WRITE_BF0_MASK       (0xFFU)
#define ECAT_ESC_RESET_ECAT_WRITE_BF0_SHIFT      (0U)
/*! BF0 - A reset is asserted after writing 0x52 ('R'), 0x45 ('E') and 0x53 ('S') in this register with 3 consecutive frames. */
#define ECAT_ESC_RESET_ECAT_WRITE_BF0(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_RESET_ECAT_WRITE_BF0_SHIFT)) & ECAT_ESC_RESET_ECAT_WRITE_BF0_MASK)
/*! @} */

/*! @name ESC_RESET_ECAT_WRITE_PDI - ESC Reset ECAT WRITE */
/*! @{ */

#define ECAT_ESC_RESET_ECAT_WRITE_PDI_BF0_MASK   (0xFFU)
#define ECAT_ESC_RESET_ECAT_WRITE_PDI_BF0_SHIFT  (0U)
/*! BF0 - A reset is asserted after writing 0x52 ('R'), 0x45 ('E') and 0x53 ('S') in this register with 3 consecutive frames. */
#define ECAT_ESC_RESET_ECAT_WRITE_PDI_BF0(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_RESET_ECAT_WRITE_PDI_BF0_SHIFT)) & ECAT_ESC_RESET_ECAT_WRITE_PDI_BF0_MASK)
/*! @} */

/*! @name ESC_RESET_PDI_WRITE - ESC Reset PDI WRITE */
/*! @{ */

#define ECAT_ESC_RESET_PDI_WRITE_BF0_MASK        (0xFFU)
#define ECAT_ESC_RESET_PDI_WRITE_BF0_SHIFT       (0U)
/*! BF0 - A reset is asserted after writing 0x52 ('R'), 0x45 ('E') and 0x53 ('S') in this register with 3 consecutive commands. */
#define ECAT_ESC_RESET_PDI_WRITE_BF0(x)          (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_RESET_PDI_WRITE_BF0_SHIFT)) & ECAT_ESC_RESET_PDI_WRITE_BF0_MASK)
/*! @} */

/*! @name ESC_RESET_PDI_WRITE_PDI - ESC Reset PDI WRITE */
/*! @{ */

#define ECAT_ESC_RESET_PDI_WRITE_PDI_BF0_MASK    (0xFFU)
#define ECAT_ESC_RESET_PDI_WRITE_PDI_BF0_SHIFT   (0U)
/*! BF0 - A reset is asserted after writing 0x52 ('R'), 0x45 ('E') and 0x53 ('S') in this register with 3 consecutive commands. */
#define ECAT_ESC_RESET_PDI_WRITE_PDI_BF0(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_RESET_PDI_WRITE_PDI_BF0_SHIFT)) & ECAT_ESC_RESET_PDI_WRITE_PDI_BF0_MASK)
/*! @} */

/*! @name ESC_DL_CONTROL - ESC DL Control */
/*! @{ */

#define ECAT_ESC_DL_CONTROL_BF0_MASK             (0x1U)
#define ECAT_ESC_DL_CONTROL_BF0_SHIFT            (0U)
/*! BF0 - Forwarding Rule
 *  0b0..EtherCAT frames are processed, non-EtherCAT frames are forwarded without processing or modification.
 *  0b1..EtherCAT frames are processed, non-EtherCAT frames are destroyed.
 */
#define ECAT_ESC_DL_CONTROL_BF0(x)               (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF0_SHIFT)) & ECAT_ESC_DL_CONTROL_BF0_MASK)

#define ECAT_ESC_DL_CONTROL_BF1_MASK             (0x2U)
#define ECAT_ESC_DL_CONTROL_BF1_SHIFT            (1U)
/*! BF1 - Temporary use of settings in 0x0100:0x0103[8:15]:
 *  0b0..permanent use
 *  0b1..use for about 1 second, then revert to previous settings
 */
#define ECAT_ESC_DL_CONTROL_BF1(x)               (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF1_SHIFT)) & ECAT_ESC_DL_CONTROL_BF1_MASK)

#define ECAT_ESC_DL_CONTROL_BF2_MASK             (0xFCU)
#define ECAT_ESC_DL_CONTROL_BF2_SHIFT            (2U)
/*! BF2 - Reserved, Write 0 */
#define ECAT_ESC_DL_CONTROL_BF2(x)               (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF2_SHIFT)) & ECAT_ESC_DL_CONTROL_BF2_MASK)

#define ECAT_ESC_DL_CONTROL_BF8_MASK             (0x300U)
#define ECAT_ESC_DL_CONTROL_BF8_SHIFT            (8U)
/*! BF8 - Loop Port 0:
 *  0b00..Auto
 *  0b01..Auto Close
 *  0b10..Open
 *  0b11..Closed
 */
#define ECAT_ESC_DL_CONTROL_BF8(x)               (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF8_SHIFT)) & ECAT_ESC_DL_CONTROL_BF8_MASK)

#define ECAT_ESC_DL_CONTROL_BF10_MASK            (0xC00U)
#define ECAT_ESC_DL_CONTROL_BF10_SHIFT           (10U)
/*! BF10 - Loop Port 1:
 *  0b00..Auto
 *  0b01..Auto Close
 *  0b10..Open
 *  0b11..Closed
 */
#define ECAT_ESC_DL_CONTROL_BF10(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF10_SHIFT)) & ECAT_ESC_DL_CONTROL_BF10_MASK)

#define ECAT_ESC_DL_CONTROL_BF12_MASK            (0x3000U)
#define ECAT_ESC_DL_CONTROL_BF12_SHIFT           (12U)
/*! BF12 - Reserved */
#define ECAT_ESC_DL_CONTROL_BF12(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF12_SHIFT)) & ECAT_ESC_DL_CONTROL_BF12_MASK)

#define ECAT_ESC_DL_CONTROL_BF14_MASK            (0xC000U)
#define ECAT_ESC_DL_CONTROL_BF14_SHIFT           (14U)
/*! BF14 - Reserved */
#define ECAT_ESC_DL_CONTROL_BF14(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF14_SHIFT)) & ECAT_ESC_DL_CONTROL_BF14_MASK)

#define ECAT_ESC_DL_CONTROL_BF16_MASK            (0x70000U)
#define ECAT_ESC_DL_CONTROL_BF16_SHIFT           (16U)
/*! BF16 - RX FIFO Size (ESC delays start of forwarding until FIFO is at least half full). */
#define ECAT_ESC_DL_CONTROL_BF16(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF16_SHIFT)) & ECAT_ESC_DL_CONTROL_BF16_MASK)

#define ECAT_ESC_DL_CONTROL_BF20_MASK            (0x300000U)
#define ECAT_ESC_DL_CONTROL_BF20_SHIFT           (20U)
/*! BF20 - Reserved, write 0 */
#define ECAT_ESC_DL_CONTROL_BF20(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF20_SHIFT)) & ECAT_ESC_DL_CONTROL_BF20_MASK)

#define ECAT_ESC_DL_CONTROL_BF23_MASK            (0x800000U)
#define ECAT_ESC_DL_CONTROL_BF23_SHIFT           (23U)
/*! BF23 - Reserved, write 0 */
#define ECAT_ESC_DL_CONTROL_BF23(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF23_SHIFT)) & ECAT_ESC_DL_CONTROL_BF23_MASK)

#define ECAT_ESC_DL_CONTROL_BF24_MASK            (0x1000000U)
#define ECAT_ESC_DL_CONTROL_BF24_SHIFT           (24U)
/*! BF24 - Station alias:
 *  0b0..Ignore Station Alias
 *  0b1..Alias can be used for all configured address command types (FPRD, FPWR, ...)
 */
#define ECAT_ESC_DL_CONTROL_BF24(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF24_SHIFT)) & ECAT_ESC_DL_CONTROL_BF24_MASK)

#define ECAT_ESC_DL_CONTROL_BF25_MASK            (0xFE000000U)
#define ECAT_ESC_DL_CONTROL_BF25_SHIFT           (25U)
/*! BF25 - Reserved, write 0 */
#define ECAT_ESC_DL_CONTROL_BF25(x)              (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_BF25_SHIFT)) & ECAT_ESC_DL_CONTROL_BF25_MASK)
/*! @} */

/*! @name ESC_DL_CONTROL_PDI - ESC DL Control */
/*! @{ */

#define ECAT_ESC_DL_CONTROL_PDI_BF0_MASK         (0x1U)
#define ECAT_ESC_DL_CONTROL_PDI_BF0_SHIFT        (0U)
/*! BF0 - Forwarding Rule
 *  0b0..EtherCAT frames are processed, non-EtherCAT frames are forwarded without processing or modification.
 *  0b1..EtherCAT frames are processed, non-EtherCAT frames are destroyed.
 */
#define ECAT_ESC_DL_CONTROL_PDI_BF0(x)           (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF0_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF0_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF1_MASK         (0x2U)
#define ECAT_ESC_DL_CONTROL_PDI_BF1_SHIFT        (1U)
/*! BF1 - Temporary use of settings in 0x0100:0x0103[8:15]:
 *  0b0..permanent use
 *  0b1..use for about 1 second, then revert to previous settings
 */
#define ECAT_ESC_DL_CONTROL_PDI_BF1(x)           (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF1_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF1_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF2_MASK         (0xFCU)
#define ECAT_ESC_DL_CONTROL_PDI_BF2_SHIFT        (2U)
/*! BF2 - Reserved, Write 0 */
#define ECAT_ESC_DL_CONTROL_PDI_BF2(x)           (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF2_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF2_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF8_MASK         (0x300U)
#define ECAT_ESC_DL_CONTROL_PDI_BF8_SHIFT        (8U)
/*! BF8 - Loop Port 0:
 *  0b00..Auto
 *  0b01..Auto Close
 *  0b10..Open
 *  0b11..Closed
 */
#define ECAT_ESC_DL_CONTROL_PDI_BF8(x)           (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF8_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF8_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF10_MASK        (0xC00U)
#define ECAT_ESC_DL_CONTROL_PDI_BF10_SHIFT       (10U)
/*! BF10 - Loop Port 1:
 *  0b00..Auto
 *  0b01..Auto Close
 *  0b10..Open
 *  0b11..Closed
 */
#define ECAT_ESC_DL_CONTROL_PDI_BF10(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF10_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF10_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF12_MASK        (0x3000U)
#define ECAT_ESC_DL_CONTROL_PDI_BF12_SHIFT       (12U)
/*! BF12 - Reserved */
#define ECAT_ESC_DL_CONTROL_PDI_BF12(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF12_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF12_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF14_MASK        (0xC000U)
#define ECAT_ESC_DL_CONTROL_PDI_BF14_SHIFT       (14U)
/*! BF14 - Reserved */
#define ECAT_ESC_DL_CONTROL_PDI_BF14(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF14_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF14_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF16_MASK        (0x70000U)
#define ECAT_ESC_DL_CONTROL_PDI_BF16_SHIFT       (16U)
/*! BF16 - RX FIFO Size (ESC delays start of forwarding until FIFO is at least half full). */
#define ECAT_ESC_DL_CONTROL_PDI_BF16(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF16_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF16_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF20_MASK        (0x300000U)
#define ECAT_ESC_DL_CONTROL_PDI_BF20_SHIFT       (20U)
/*! BF20 - Reserved, write 0 */
#define ECAT_ESC_DL_CONTROL_PDI_BF20(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF20_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF20_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF23_MASK        (0x800000U)
#define ECAT_ESC_DL_CONTROL_PDI_BF23_SHIFT       (23U)
/*! BF23 - Reserved, write 0 */
#define ECAT_ESC_DL_CONTROL_PDI_BF23(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF23_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF23_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF24_MASK        (0x1000000U)
#define ECAT_ESC_DL_CONTROL_PDI_BF24_SHIFT       (24U)
/*! BF24 - Station alias:
 *  0b0..Ignore Station Alias
 *  0b1..Alias can be used for all configured address command types (FPRD, FPWR, ...)
 */
#define ECAT_ESC_DL_CONTROL_PDI_BF24(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF24_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF24_MASK)

#define ECAT_ESC_DL_CONTROL_PDI_BF25_MASK        (0xFE000000U)
#define ECAT_ESC_DL_CONTROL_PDI_BF25_SHIFT       (25U)
/*! BF25 - Reserved, write 0 */
#define ECAT_ESC_DL_CONTROL_PDI_BF25(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_ESC_DL_CONTROL_PDI_BF25_SHIFT)) & ECAT_ESC_DL_CONTROL_PDI_BF25_MASK)
/*! @} */

/*! @name PHYSICAL_READ_WRITE_OFFSET - Physical Read Write Offset */
/*! @{ */

#define ECAT_PHYSICAL_READ_WRITE_OFFSET_BF0_MASK (0xFFFFU)
#define ECAT_PHYSICAL_READ_WRITE_OFFSET_BF0_SHIFT (0U)
/*! BF0 - This register is used for ReadWrite commands in Device Addressing mode (FPRW, APRW, BRW). */
#define ECAT_PHYSICAL_READ_WRITE_OFFSET_BF0(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_PHYSICAL_READ_WRITE_OFFSET_BF0_SHIFT)) & ECAT_PHYSICAL_READ_WRITE_OFFSET_BF0_MASK)
/*! @} */

/*! @name PHYSICAL_READ_WRITE_OFFSET_PDI - Physical Read Write Offset */
/*! @{ */

#define ECAT_PHYSICAL_READ_WRITE_OFFSET_PDI_BF0_MASK (0xFFFFU)
#define ECAT_PHYSICAL_READ_WRITE_OFFSET_PDI_BF0_SHIFT (0U)
/*! BF0 - This register is used for ReadWrite commands in Device Addressing mode (FPRW, APRW, BRW). */
#define ECAT_PHYSICAL_READ_WRITE_OFFSET_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_PHYSICAL_READ_WRITE_OFFSET_PDI_BF0_SHIFT)) & ECAT_PHYSICAL_READ_WRITE_OFFSET_PDI_BF0_MASK)
/*! @} */

/*! @name ESC_DL_STATUS - ESC DL Status */
/*! @{ */

#define ECAT_ESC_DL_STATUS_BF0_MASK              (0x1U)
#define ECAT_ESC_DL_STATUS_BF0_SHIFT             (0U)
/*! BF0 - Register ESC DL Status
 *  0b0..PDI operational/EEPROM loaded correctly:
 *  0b1..EEPROM loaded correctly, PDI operational (access to Process Data RAM)
 */
#define ECAT_ESC_DL_STATUS_BF0(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF0_SHIFT)) & ECAT_ESC_DL_STATUS_BF0_MASK)

#define ECAT_ESC_DL_STATUS_BF1_MASK              (0x2U)
#define ECAT_ESC_DL_STATUS_BF1_SHIFT             (1U)
/*! BF1 - PDI Watchdog Status:
 *  0b0..Watchdog expired
 *  0b1..Watchdog reloaded
 */
#define ECAT_ESC_DL_STATUS_BF1(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF1_SHIFT)) & ECAT_ESC_DL_STATUS_BF1_MASK)

#define ECAT_ESC_DL_STATUS_BF2_MASK              (0x4U)
#define ECAT_ESC_DL_STATUS_BF2_SHIFT             (2U)
/*! BF2 - Enhanced Link detection:
 *  0b0..Deactivated for all ports
 *  0b1..Activated for at least one port
 */
#define ECAT_ESC_DL_STATUS_BF2(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF2_SHIFT)) & ECAT_ESC_DL_STATUS_BF2_MASK)

#define ECAT_ESC_DL_STATUS_BF3_MASK              (0x8U)
#define ECAT_ESC_DL_STATUS_BF3_SHIFT             (3U)
/*! BF3 - Reserved */
#define ECAT_ESC_DL_STATUS_BF3(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF3_SHIFT)) & ECAT_ESC_DL_STATUS_BF3_MASK)

#define ECAT_ESC_DL_STATUS_BF4_MASK              (0x10U)
#define ECAT_ESC_DL_STATUS_BF4_SHIFT             (4U)
/*! BF4 - Physical link on Port 0:
 *  0b0..No link
 *  0b1..Link detected
 */
#define ECAT_ESC_DL_STATUS_BF4(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF4_SHIFT)) & ECAT_ESC_DL_STATUS_BF4_MASK)

#define ECAT_ESC_DL_STATUS_BF5_MASK              (0x20U)
#define ECAT_ESC_DL_STATUS_BF5_SHIFT             (5U)
/*! BF5 - Physical link on Port 1:
 *  0b0..No link
 *  0b1..Link detected
 */
#define ECAT_ESC_DL_STATUS_BF5(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF5_SHIFT)) & ECAT_ESC_DL_STATUS_BF5_MASK)

#define ECAT_ESC_DL_STATUS_BF6_MASK              (0x40U)
#define ECAT_ESC_DL_STATUS_BF6_SHIFT             (6U)
/*! BF6 - Reserved */
#define ECAT_ESC_DL_STATUS_BF6(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF6_SHIFT)) & ECAT_ESC_DL_STATUS_BF6_MASK)

#define ECAT_ESC_DL_STATUS_BF7_MASK              (0x80U)
#define ECAT_ESC_DL_STATUS_BF7_SHIFT             (7U)
/*! BF7 - Reserved */
#define ECAT_ESC_DL_STATUS_BF7(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF7_SHIFT)) & ECAT_ESC_DL_STATUS_BF7_MASK)

#define ECAT_ESC_DL_STATUS_BF8_MASK              (0x100U)
#define ECAT_ESC_DL_STATUS_BF8_SHIFT             (8U)
/*! BF8 - Loop Port 0:
 *  0b0..Open
 *  0b1..Closed
 */
#define ECAT_ESC_DL_STATUS_BF8(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF8_SHIFT)) & ECAT_ESC_DL_STATUS_BF8_MASK)

#define ECAT_ESC_DL_STATUS_BF9_MASK              (0x200U)
#define ECAT_ESC_DL_STATUS_BF9_SHIFT             (9U)
/*! BF9 - Communication on Port 0:
 *  0b0..No stable communication
 *  0b1..Communication established
 */
#define ECAT_ESC_DL_STATUS_BF9(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF9_SHIFT)) & ECAT_ESC_DL_STATUS_BF9_MASK)

#define ECAT_ESC_DL_STATUS_BF10_MASK             (0x400U)
#define ECAT_ESC_DL_STATUS_BF10_SHIFT            (10U)
/*! BF10 - Loop Port 1
 *  0b0..Open
 *  0b1..Closed
 */
#define ECAT_ESC_DL_STATUS_BF10(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF10_SHIFT)) & ECAT_ESC_DL_STATUS_BF10_MASK)

#define ECAT_ESC_DL_STATUS_BF11_MASK             (0x800U)
#define ECAT_ESC_DL_STATUS_BF11_SHIFT            (11U)
/*! BF11 - Communication on Port 1:
 *  0b0..No stable communication
 *  0b1..Communication established
 */
#define ECAT_ESC_DL_STATUS_BF11(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF11_SHIFT)) & ECAT_ESC_DL_STATUS_BF11_MASK)

#define ECAT_ESC_DL_STATUS_BF12_MASK             (0x1000U)
#define ECAT_ESC_DL_STATUS_BF12_SHIFT            (12U)
/*! BF12 - Reserved */
#define ECAT_ESC_DL_STATUS_BF12(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF12_SHIFT)) & ECAT_ESC_DL_STATUS_BF12_MASK)

#define ECAT_ESC_DL_STATUS_BF13_MASK             (0x2000U)
#define ECAT_ESC_DL_STATUS_BF13_SHIFT            (13U)
/*! BF13 - Reserved */
#define ECAT_ESC_DL_STATUS_BF13(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF13_SHIFT)) & ECAT_ESC_DL_STATUS_BF13_MASK)

#define ECAT_ESC_DL_STATUS_BF14_MASK             (0x4000U)
#define ECAT_ESC_DL_STATUS_BF14_SHIFT            (14U)
/*! BF14 - Reserved */
#define ECAT_ESC_DL_STATUS_BF14(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF14_SHIFT)) & ECAT_ESC_DL_STATUS_BF14_MASK)

#define ECAT_ESC_DL_STATUS_BF15_MASK             (0x8000U)
#define ECAT_ESC_DL_STATUS_BF15_SHIFT            (15U)
/*! BF15 - Reserved */
#define ECAT_ESC_DL_STATUS_BF15(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_ESC_DL_STATUS_BF15_SHIFT)) & ECAT_ESC_DL_STATUS_BF15_MASK)
/*! @} */

/*! @name AL_CONTROL - AL Control */
/*! @{ */

#define ECAT_AL_CONTROL_BF0_MASK                 (0xFU)
#define ECAT_AL_CONTROL_BF0_SHIFT                (0U)
/*! BF0 - Initiate State Transition of the Device State Machine: */
#define ECAT_AL_CONTROL_BF0(x)                   (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_BF0_SHIFT)) & ECAT_AL_CONTROL_BF0_MASK)

#define ECAT_AL_CONTROL_BF4_MASK                 (0x10U)
#define ECAT_AL_CONTROL_BF4_SHIFT                (4U)
/*! BF4 - Error Ind Ack
 *  0b0..No Ack of Error Ind in AL status register
 *  0b1..Ack of Error Ind in AL status register
 */
#define ECAT_AL_CONTROL_BF4(x)                   (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_BF4_SHIFT)) & ECAT_AL_CONTROL_BF4_MASK)

#define ECAT_AL_CONTROL_BF5_MASK                 (0x20U)
#define ECAT_AL_CONTROL_BF5_SHIFT                (5U)
/*! BF5 - Device Identification:
 *  0b0..No request
 *  0b1..Device Identification request
 */
#define ECAT_AL_CONTROL_BF5(x)                   (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_BF5_SHIFT)) & ECAT_AL_CONTROL_BF5_MASK)

#define ECAT_AL_CONTROL_BF6_MASK                 (0xFFC0U)
#define ECAT_AL_CONTROL_BF6_SHIFT                (6U)
/*! BF6 - Reserved, write 0 */
#define ECAT_AL_CONTROL_BF6(x)                   (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_BF6_SHIFT)) & ECAT_AL_CONTROL_BF6_MASK)
/*! @} */

/*! @name AL_CONTROL_PDI - AL Control */
/*! @{ */

#define ECAT_AL_CONTROL_PDI_BF0_MASK             (0xFU)
#define ECAT_AL_CONTROL_PDI_BF0_SHIFT            (0U)
/*! BF0 - Initiate State Transition of the Device State Machine: */
#define ECAT_AL_CONTROL_PDI_BF0(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_PDI_BF0_SHIFT)) & ECAT_AL_CONTROL_PDI_BF0_MASK)

#define ECAT_AL_CONTROL_PDI_BF4_MASK             (0x10U)
#define ECAT_AL_CONTROL_PDI_BF4_SHIFT            (4U)
/*! BF4 - Error Ind Ack
 *  0b0..No Ack of Error Ind in AL status register
 *  0b1..Ack of Error Ind in AL status register
 */
#define ECAT_AL_CONTROL_PDI_BF4(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_PDI_BF4_SHIFT)) & ECAT_AL_CONTROL_PDI_BF4_MASK)

#define ECAT_AL_CONTROL_PDI_BF5_MASK             (0x20U)
#define ECAT_AL_CONTROL_PDI_BF5_SHIFT            (5U)
/*! BF5 - Device Identification:
 *  0b0..No request
 *  0b1..Device Identification request
 */
#define ECAT_AL_CONTROL_PDI_BF5(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_PDI_BF5_SHIFT)) & ECAT_AL_CONTROL_PDI_BF5_MASK)

#define ECAT_AL_CONTROL_PDI_BF6_MASK             (0xFFC0U)
#define ECAT_AL_CONTROL_PDI_BF6_SHIFT            (6U)
/*! BF6 - Reserved, write 0 */
#define ECAT_AL_CONTROL_PDI_BF6(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_AL_CONTROL_PDI_BF6_SHIFT)) & ECAT_AL_CONTROL_PDI_BF6_MASK)
/*! @} */

/*! @name AL_STATUS - AL Status */
/*! @{ */

#define ECAT_AL_STATUS_BF0_MASK                  (0xFU)
#define ECAT_AL_STATUS_BF0_SHIFT                 (0U)
/*! BF0 - Actual State of the Device State Machine: */
#define ECAT_AL_STATUS_BF0(x)                    (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_BF0_SHIFT)) & ECAT_AL_STATUS_BF0_MASK)

#define ECAT_AL_STATUS_BF4_MASK                  (0x10U)
#define ECAT_AL_STATUS_BF4_SHIFT                 (4U)
/*! BF4 - Error Ind:
 *  0b0..Device is in State as requested or Flag cleared by command
 *  0b1..Device has not entered requested State or changed State as result of a local action
 */
#define ECAT_AL_STATUS_BF4(x)                    (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_BF4_SHIFT)) & ECAT_AL_STATUS_BF4_MASK)

#define ECAT_AL_STATUS_BF5_MASK                  (0x20U)
#define ECAT_AL_STATUS_BF5_SHIFT                 (5U)
/*! BF5 - Device Identification:
 *  0b0..Device Identification not valid
 *  0b1..Device Identification loaded
 */
#define ECAT_AL_STATUS_BF5(x)                    (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_BF5_SHIFT)) & ECAT_AL_STATUS_BF5_MASK)

#define ECAT_AL_STATUS_BF6_MASK                  (0xFFC0U)
#define ECAT_AL_STATUS_BF6_SHIFT                 (6U)
/*! BF6 - Reserved, write 0 */
#define ECAT_AL_STATUS_BF6(x)                    (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_BF6_SHIFT)) & ECAT_AL_STATUS_BF6_MASK)
/*! @} */

/*! @name AL_STATUS_PDI - AL Status */
/*! @{ */

#define ECAT_AL_STATUS_PDI_BF0_MASK              (0xFU)
#define ECAT_AL_STATUS_PDI_BF0_SHIFT             (0U)
/*! BF0 - Actual State of the Device State Machine: */
#define ECAT_AL_STATUS_PDI_BF0(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_PDI_BF0_SHIFT)) & ECAT_AL_STATUS_PDI_BF0_MASK)

#define ECAT_AL_STATUS_PDI_BF4_MASK              (0x10U)
#define ECAT_AL_STATUS_PDI_BF4_SHIFT             (4U)
/*! BF4 - Error Ind:
 *  0b0..Device is in State as requested or Flag cleared by command
 *  0b1..Device has not entered requested State or changed State as result of a local action
 */
#define ECAT_AL_STATUS_PDI_BF4(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_PDI_BF4_SHIFT)) & ECAT_AL_STATUS_PDI_BF4_MASK)

#define ECAT_AL_STATUS_PDI_BF5_MASK              (0x20U)
#define ECAT_AL_STATUS_PDI_BF5_SHIFT             (5U)
/*! BF5 - Device Identification:
 *  0b0..Device Identification not valid
 *  0b1..Device Identification loaded
 */
#define ECAT_AL_STATUS_PDI_BF5(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_PDI_BF5_SHIFT)) & ECAT_AL_STATUS_PDI_BF5_MASK)

#define ECAT_AL_STATUS_PDI_BF6_MASK              (0xFFC0U)
#define ECAT_AL_STATUS_PDI_BF6_SHIFT             (6U)
/*! BF6 - Reserved, write 0 */
#define ECAT_AL_STATUS_PDI_BF6(x)                (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_PDI_BF6_SHIFT)) & ECAT_AL_STATUS_PDI_BF6_MASK)
/*! @} */

/*! @name AL_STATUS_CODE - AL Status Code */
/*! @{ */

#define ECAT_AL_STATUS_CODE_BF0_MASK             (0xFFFFU)
#define ECAT_AL_STATUS_CODE_BF0_SHIFT            (0U)
/*! BF0 - AL Status Code */
#define ECAT_AL_STATUS_CODE_BF0(x)               (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_CODE_BF0_SHIFT)) & ECAT_AL_STATUS_CODE_BF0_MASK)
/*! @} */

/*! @name AL_STATUS_CODE_PDI - AL Status Code */
/*! @{ */

#define ECAT_AL_STATUS_CODE_PDI_BF0_MASK         (0xFFFFU)
#define ECAT_AL_STATUS_CODE_PDI_BF0_SHIFT        (0U)
/*! BF0 - AL Status Code */
#define ECAT_AL_STATUS_CODE_PDI_BF0(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_AL_STATUS_CODE_PDI_BF0_SHIFT)) & ECAT_AL_STATUS_CODE_PDI_BF0_MASK)
/*! @} */

/*! @name RUN_LED_OVERRIDE - RUN LED Override */
/*! @{ */

#define ECAT_RUN_LED_OVERRIDE_BF0_MASK           (0xFU)
#define ECAT_RUN_LED_OVERRIDE_BF0_SHIFT          (0U)
/*! BF0 - LED code and AL Status */
#define ECAT_RUN_LED_OVERRIDE_BF0(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_RUN_LED_OVERRIDE_BF0_SHIFT)) & ECAT_RUN_LED_OVERRIDE_BF0_MASK)

#define ECAT_RUN_LED_OVERRIDE_BF4_MASK           (0x10U)
#define ECAT_RUN_LED_OVERRIDE_BF4_SHIFT          (4U)
/*! BF4 - Enable Override:
 *  0b0..Override disabled
 *  0b1..Override enabled
 */
#define ECAT_RUN_LED_OVERRIDE_BF4(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_RUN_LED_OVERRIDE_BF4_SHIFT)) & ECAT_RUN_LED_OVERRIDE_BF4_MASK)

#define ECAT_RUN_LED_OVERRIDE_BF5_MASK           (0xE0U)
#define ECAT_RUN_LED_OVERRIDE_BF5_SHIFT          (5U)
/*! BF5 - Reserved, write 0 */
#define ECAT_RUN_LED_OVERRIDE_BF5(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_RUN_LED_OVERRIDE_BF5_SHIFT)) & ECAT_RUN_LED_OVERRIDE_BF5_MASK)
/*! @} */

/*! @name ERR_LED_OVERRIDE - ERR LED Override */
/*! @{ */

#define ECAT_ERR_LED_OVERRIDE_BF0_MASK           (0xFU)
#define ECAT_ERR_LED_OVERRIDE_BF0_SHIFT          (0U)
/*! BF0 - LED code */
#define ECAT_ERR_LED_OVERRIDE_BF0(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_ERR_LED_OVERRIDE_BF0_SHIFT)) & ECAT_ERR_LED_OVERRIDE_BF0_MASK)

#define ECAT_ERR_LED_OVERRIDE_BF4_MASK           (0x10U)
#define ECAT_ERR_LED_OVERRIDE_BF4_SHIFT          (4U)
/*! BF4 - Enable Override:
 *  0b0..Override disabled
 *  0b1..Override enabled
 */
#define ECAT_ERR_LED_OVERRIDE_BF4(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_ERR_LED_OVERRIDE_BF4_SHIFT)) & ECAT_ERR_LED_OVERRIDE_BF4_MASK)

#define ECAT_ERR_LED_OVERRIDE_BF5_MASK           (0xE0U)
#define ECAT_ERR_LED_OVERRIDE_BF5_SHIFT          (5U)
/*! BF5 - Reserved, write 0 */
#define ECAT_ERR_LED_OVERRIDE_BF5(x)             (((uint8_t)(((uint8_t)(x)) << ECAT_ERR_LED_OVERRIDE_BF5_SHIFT)) & ECAT_ERR_LED_OVERRIDE_BF5_MASK)
/*! @} */

/*! @name PDI_CONTROL - PDI Control */
/*! @{ */

#define ECAT_PDI_CONTROL_BF0_MASK                (0xFFU)
#define ECAT_PDI_CONTROL_BF0_SHIFT               (0U)
/*! BF0 - Process data interface:
 *  0b00000000..Interface deactivated (no PDI)
 *  0b00000001..4 Digital Input
 *  0b00000010..4 Digital Output
 *  0b00000011..2 Digital Input and 2 Digital Output
 *  0b00000100..Digital I/O
 *  0b00000101..SPI Slave
 *  0b00000110..Oversampling I/O
 *  0b00000111..Reserved
 *  0b00001000..16 Bit asynchronous Microcontroller interface
 *  0b00001001..8 Bit asynchronous Microcontroller interface
 *  0b00001010..16 Bit synchronous Microcontroller interface
 *  0b00001011..8 Bit synchronous Microcontroller interface
 *  0b00010000..32 Digital Input and 0 Digital Output
 *  0b00010001..24 Digital Input and 8 Digital Output
 *  0b00010010..16 Digital Input and 16 Digital Output
 *  0b00010011..8 Digital Input and 24 Digital Output
 *  0b00010100..0 Digital Input and 32 Digital Output
 *  0b10000000..On-chip bus.
 */
#define ECAT_PDI_CONTROL_BF0(x)                  (((uint8_t)(((uint8_t)(x)) << ECAT_PDI_CONTROL_BF0_SHIFT)) & ECAT_PDI_CONTROL_BF0_MASK)
/*! @} */

/*! @name ESC_CONFIGURATION - ESC Configuration */
/*! @{ */

#define ECAT_ESC_CONFIGURATION_BF0_MASK          (0x1U)
#define ECAT_ESC_CONFIGURATION_BF0_SHIFT         (0U)
/*! BF0 - Device emulation (control of AL status):
 *  0b0..AL status register has to be set by PDI
 *  0b1..AL status register will be set to value written to AL control register
 */
#define ECAT_ESC_CONFIGURATION_BF0(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF0_SHIFT)) & ECAT_ESC_CONFIGURATION_BF0_MASK)

#define ECAT_ESC_CONFIGURATION_BF1_MASK          (0x2U)
#define ECAT_ESC_CONFIGURATION_BF1_SHIFT         (1U)
/*! BF1 - Enhanced Link detection all ports:
 *  0b0..disabled (if bits [7:4]=0)
 *  0b1..enabled at all ports (overrides bits [7:4])
 */
#define ECAT_ESC_CONFIGURATION_BF1(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF1_SHIFT)) & ECAT_ESC_CONFIGURATION_BF1_MASK)

#define ECAT_ESC_CONFIGURATION_BF2_MASK          (0x4U)
#define ECAT_ESC_CONFIGURATION_BF2_SHIFT         (2U)
/*! BF2 - Distributed Clocks SYNC Out Unit:
 *  0b0..disabled (power saving)
 *  0b1..enabled
 */
#define ECAT_ESC_CONFIGURATION_BF2(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF2_SHIFT)) & ECAT_ESC_CONFIGURATION_BF2_MASK)

#define ECAT_ESC_CONFIGURATION_BF3_MASK          (0x8U)
#define ECAT_ESC_CONFIGURATION_BF3_SHIFT         (3U)
/*! BF3 - Distributed Clocks Latch In Unit:
 *  0b0..disabled (power saving)
 *  0b1..enabled
 */
#define ECAT_ESC_CONFIGURATION_BF3(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF3_SHIFT)) & ECAT_ESC_CONFIGURATION_BF3_MASK)

#define ECAT_ESC_CONFIGURATION_BF4_MASK          (0x10U)
#define ECAT_ESC_CONFIGURATION_BF4_SHIFT         (4U)
/*! BF4 - Enhanced Link port 0:
 *  0b0..disabled (if bit 1=0)
 *  0b1..enabled
 */
#define ECAT_ESC_CONFIGURATION_BF4(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF4_SHIFT)) & ECAT_ESC_CONFIGURATION_BF4_MASK)

#define ECAT_ESC_CONFIGURATION_BF5_MASK          (0x20U)
#define ECAT_ESC_CONFIGURATION_BF5_SHIFT         (5U)
/*! BF5 - Enhanced Link port 1:
 *  0b0..disabled (if bit 1=0)
 *  0b1..enabled
 */
#define ECAT_ESC_CONFIGURATION_BF5(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF5_SHIFT)) & ECAT_ESC_CONFIGURATION_BF5_MASK)

#define ECAT_ESC_CONFIGURATION_BF6_MASK          (0x40U)
#define ECAT_ESC_CONFIGURATION_BF6_SHIFT         (6U)
/*! BF6 - Reserved */
#define ECAT_ESC_CONFIGURATION_BF6(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF6_SHIFT)) & ECAT_ESC_CONFIGURATION_BF6_MASK)

#define ECAT_ESC_CONFIGURATION_BF7_MASK          (0x80U)
#define ECAT_ESC_CONFIGURATION_BF7_SHIFT         (7U)
/*! BF7 - Reserved */
#define ECAT_ESC_CONFIGURATION_BF7(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_ESC_CONFIGURATION_BF7_SHIFT)) & ECAT_ESC_CONFIGURATION_BF7_MASK)
/*! @} */

/*! @name PDI_INFORMATION - PDI Information */
/*! @{ */

#define ECAT_PDI_INFORMATION_BF0_MASK            (0x1U)
#define ECAT_PDI_INFORMATION_BF0_SHIFT           (0U)
/*! BF0 - PDI function
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_PDI_INFORMATION_BF0(x)              (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_INFORMATION_BF0_SHIFT)) & ECAT_PDI_INFORMATION_BF0_MASK)

#define ECAT_PDI_INFORMATION_BF1_MASK            (0x2U)
#define ECAT_PDI_INFORMATION_BF1_SHIFT           (1U)
/*! BF1 - ESC configuration area loaded from EEPROM:
 *  0b0..not loaded
 *  0b1..loaded
 */
#define ECAT_PDI_INFORMATION_BF1(x)              (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_INFORMATION_BF1_SHIFT)) & ECAT_PDI_INFORMATION_BF1_MASK)

#define ECAT_PDI_INFORMATION_BF2_MASK            (0x4U)
#define ECAT_PDI_INFORMATION_BF2_SHIFT           (2U)
/*! BF2 - PDI active:
 *  0b0..PDI not active
 *  0b1..PDI active
 */
#define ECAT_PDI_INFORMATION_BF2(x)              (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_INFORMATION_BF2_SHIFT)) & ECAT_PDI_INFORMATION_BF2_MASK)

#define ECAT_PDI_INFORMATION_BF3_MASK            (0x8U)
#define ECAT_PDI_INFORMATION_BF3_SHIFT           (3U)
/*! BF3 - PDI configuration invalid:
 *  0b0..PDI configuration ok
 *  0b1..PDI configuration invalid
 */
#define ECAT_PDI_INFORMATION_BF3(x)              (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_INFORMATION_BF3_SHIFT)) & ECAT_PDI_INFORMATION_BF3_MASK)

#define ECAT_PDI_INFORMATION_BF4_MASK            (0xFFF0U)
#define ECAT_PDI_INFORMATION_BF4_SHIFT           (4U)
/*! BF4 - Reserved */
#define ECAT_PDI_INFORMATION_BF4(x)              (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_INFORMATION_BF4_SHIFT)) & ECAT_PDI_INFORMATION_BF4_MASK)
/*! @} */

/*! @name PDI_ON_CHIP_BUS_CONFIGURATION - Register PDI On-chip bus configuration */
/*! @{ */

#define ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF0_MASK (0x1FU)
#define ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF0_SHIFT (0U)
/*! BF0 - On-chip bus clock:
 *  0b00000..asynchronous
 *  0b00001-0b11111..synchronous multiplication factor (N * 25 MHz)
 */
#define ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF0_SHIFT)) & ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF0_MASK)

#define ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF5_MASK (0xE0U)
#define ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF5_SHIFT (5U)
/*! BF5 - On-chip bus
 *  0b000..Intel® Avalon®
 *  0b001..AXI® 010: Xilinx® PLB v4.6
 *  0b100..Xilinx OPB
 */
#define ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF5(x) (((uint8_t)(((uint8_t)(x)) << ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF5_SHIFT)) & ECAT_PDI_ON_CHIP_BUS_CONFIGURATION_BF5_MASK)
/*! @} */

/*! @name SYNC_LATCH_1_AND_0_PDI_CONFIGURATION - PDI Configuration Sync Latch 1 and 0 PDI Configuration */
/*! @{ */

#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF0_MASK (0x3U)
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF0_SHIFT (0U)
/*! BF0 - SYNC0 output driver/polarity:
 *  0b00..Push-Pull active low
 *  0b01..Open Drain (active low)
 *  0b10..Push-Pull active high
 *  0b11..Open Source (active high)
 */
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF0_SHIFT)) & ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF0_MASK)

#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF2_MASK (0x4U)
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF2_SHIFT (2U)
/*! BF2 - SYNC0/LATCH0 configuration*:
 *  0b0..LATCH0 Input
 *  0b1..SYNC0 Output
 */
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF2(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF2_SHIFT)) & ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF2_MASK)

#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF3_MASK (0x8U)
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF3_SHIFT (3U)
/*! BF3 - SYNC0 mapped to AL Event Request register 0x0220[2]:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF3(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF3_SHIFT)) & ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF3_MASK)

#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF4_MASK (0x30U)
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF4_SHIFT (4U)
/*! BF4 - SYNC1 output driver/polarity:
 *  0b00..Push-Pull active low
 *  0b01..Open Drain (active low)
 *  0b10..Push-Pull active high
 *  0b11..Open Source (active high)
 */
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF4(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF4_SHIFT)) & ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF4_MASK)

#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF6_MASK (0x40U)
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF6_SHIFT (6U)
/*! BF6 - SYNC1/LATCH1 configuration*
 *  0b0..LATCH1 input
 *  0b1..SYNC1 output
 */
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF6(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF6_SHIFT)) & ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF6_MASK)

#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF7_MASK (0x80U)
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF7_SHIFT (7U)
/*! BF7 - SYNC1 mapped to AL Event Request register 0x0220[3]:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF7(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF7_SHIFT)) & ECAT_SYNC_LATCH_1_AND_0_PDI_CONFIGURATION_BF7_MASK)
/*! @} */

/*! @name PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION - Register PDI On-chip bus extended configuration. */
/*! @{ */

#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF0_MASK (0x3U)
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF0_SHIFT (0U)
/*! BF0 - Read prefetch size (in cycles of PDI width):
 *  0b00..4 cycles
 *  0b01..1 cycle (typical)
 *  0b10..2 cycles
 *  0b11..Reserved
 */
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF0_SHIFT)) & ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF0_MASK)

#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF2_MASK (0xFCU)
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF2_SHIFT (2U)
/*! BF2 - Reserved */
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF2(x) (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF2_SHIFT)) & ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF2_MASK)

#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF8_MASK (0x700U)
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF8_SHIFT (8U)
/*! BF8 - On-chip bus sub-type for AXI:
 *  0b000..AXI3
 *  0b001..AXI4
 *  0b010..AXI4 LITE
 */
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF8(x) (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF8_SHIFT)) & ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF8_MASK)

#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF11_MASK (0xF800U)
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF11_SHIFT (11U)
/*! BF11 - Reserved */
#define ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF11(x) (((uint16_t)(((uint16_t)(x)) << ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF11_SHIFT)) & ECAT_PDI_ON_CHIP_BUS_EXTENDED_CONFIGURATION_BF11_MASK)
/*! @} */

/*! @name ECAT_EVENT_MASK - ECAT Event Mask */
/*! @{ */

#define ECAT_ECAT_EVENT_MASK_BF0_MASK            (0xFFFFU)
#define ECAT_ECAT_EVENT_MASK_BF0_SHIFT           (0U)
/*! BF0 - ECAT Event masking of the ECAT Event Request Events for mapping into ECAT event field of EtherCAT frames:
 *  0b0000000000000000..Corresponding ECAT Event Request register bit is not mapped
 *  0b0000000000000001..Corresponding ECAT Event Request register bit is mapped
 */
#define ECAT_ECAT_EVENT_MASK_BF0(x)              (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_MASK_BF0_SHIFT)) & ECAT_ECAT_EVENT_MASK_BF0_MASK)
/*! @} */

/*! @name ECAT_EVENT_MASK_PDI - ECAT Event Mask */
/*! @{ */

#define ECAT_ECAT_EVENT_MASK_PDI_BF0_MASK        (0xFFFFU)
#define ECAT_ECAT_EVENT_MASK_PDI_BF0_SHIFT       (0U)
/*! BF0 - ECAT Event masking of the ECAT Event Request Events for mapping into ECAT event field of EtherCAT frames:
 *  0b0000000000000000..Corresponding ECAT Event Request register bit is not mapped
 *  0b0000000000000001..Corresponding ECAT Event Request register bit is mapped
 */
#define ECAT_ECAT_EVENT_MASK_PDI_BF0(x)          (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_MASK_PDI_BF0_SHIFT)) & ECAT_ECAT_EVENT_MASK_PDI_BF0_MASK)
/*! @} */

/*! @name PDI_AL_EVENT_MASK - PDI AL Event Mask */
/*! @{ */

#define ECAT_PDI_AL_EVENT_MASK_BF0_MASK          (0xFFFFFFFFU)
#define ECAT_PDI_AL_EVENT_MASK_BF0_SHIFT         (0U)
/*! BF0 - AL Event masking of the AL Event Request register Events for mapping to PDI IRQ signal:
 *  0b00000000000000000000000000000000..Corresponding AL Event Request register bit is not mapped
 *  0b00000000000000000000000000000001..Corresponding AL Event Request register bit is mapped
 */
#define ECAT_PDI_AL_EVENT_MASK_BF0(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_PDI_AL_EVENT_MASK_BF0_SHIFT)) & ECAT_PDI_AL_EVENT_MASK_BF0_MASK)
/*! @} */

/*! @name PDI_AL_EVENT_MASK_PDI - PDI AL Event Mask */
/*! @{ */

#define ECAT_PDI_AL_EVENT_MASK_PDI_BF0_MASK      (0xFFFFFFFFU)
#define ECAT_PDI_AL_EVENT_MASK_PDI_BF0_SHIFT     (0U)
/*! BF0 - AL Event masking of the AL Event Request register Events for mapping to PDI IRQ signal:
 *  0b00000000000000000000000000000000..Corresponding AL Event Request register bit is not mapped
 *  0b00000000000000000000000000000001..Corresponding AL Event Request register bit is mapped
 */
#define ECAT_PDI_AL_EVENT_MASK_PDI_BF0(x)        (((uint32_t)(((uint32_t)(x)) << ECAT_PDI_AL_EVENT_MASK_PDI_BF0_SHIFT)) & ECAT_PDI_AL_EVENT_MASK_PDI_BF0_MASK)
/*! @} */

/*! @name ECAT_EVENT_REQUEST - ECAT Event Request */
/*! @{ */

#define ECAT_ECAT_EVENT_REQUEST_BF0_MASK         (0x1U)
#define ECAT_ECAT_EVENT_REQUEST_BF0_SHIFT        (0U)
/*! BF0 - DC Latch event:
 *  0b0..No change on DC Latch Inputs
 *  0b1..At least one change on DC Latch Inputs
 */
#define ECAT_ECAT_EVENT_REQUEST_BF0(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF0_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF0_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF1_MASK         (0x2U)
#define ECAT_ECAT_EVENT_REQUEST_BF1_SHIFT        (1U)
/*! BF1 - Reserved */
#define ECAT_ECAT_EVENT_REQUEST_BF1(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF1_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF1_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF2_MASK         (0x4U)
#define ECAT_ECAT_EVENT_REQUEST_BF2_SHIFT        (2U)
/*! BF2 - DL Status event:
 *  0b0..No change in DL Status
 *  0b1..DL Status change
 */
#define ECAT_ECAT_EVENT_REQUEST_BF2(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF2_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF2_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF3_MASK         (0x8U)
#define ECAT_ECAT_EVENT_REQUEST_BF3_SHIFT        (3U)
/*! BF3 - AL Status event:
 *  0b0..No change in AL Status
 *  0b1..AL Status change
 */
#define ECAT_ECAT_EVENT_REQUEST_BF3(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF3_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF3_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF4_MASK         (0x10U)
#define ECAT_ECAT_EVENT_REQUEST_BF4_SHIFT        (4U)
/*! BF4 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 0 event
 *  0b1..Sync Channel 0 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF4(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF4_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF4_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF5_MASK         (0x20U)
#define ECAT_ECAT_EVENT_REQUEST_BF5_SHIFT        (5U)
/*! BF5 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 1 event
 *  0b1..Sync Channel 1 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF5(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF5_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF5_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF6_MASK         (0x40U)
#define ECAT_ECAT_EVENT_REQUEST_BF6_SHIFT        (6U)
/*! BF6 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 2 event
 *  0b1..Sync Channel 2 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF6(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF6_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF6_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF7_MASK         (0x80U)
#define ECAT_ECAT_EVENT_REQUEST_BF7_SHIFT        (7U)
/*! BF7 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 3 event
 *  0b1..Sync Channel 3 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF7(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF7_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF7_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF8_MASK         (0x100U)
#define ECAT_ECAT_EVENT_REQUEST_BF8_SHIFT        (8U)
/*! BF8 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 4 event
 *  0b1..Sync Channel 4 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF8(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF8_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF8_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF9_MASK         (0x200U)
#define ECAT_ECAT_EVENT_REQUEST_BF9_SHIFT        (9U)
/*! BF9 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 5 event
 *  0b1..Sync Channel 5 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF9(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF9_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF9_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF10_MASK        (0x400U)
#define ECAT_ECAT_EVENT_REQUEST_BF10_SHIFT       (10U)
/*! BF10 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 6 event
 *  0b1..Sync Channel 6 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF10(x)          (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF10_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF10_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF11_MASK        (0x800U)
#define ECAT_ECAT_EVENT_REQUEST_BF11_SHIFT       (11U)
/*! BF11 - Mirrors values of each SyncManager Status:
 *  0b0..No Sync Channel 7 event
 *  0b1..Sync Channel 7 event pending
 */
#define ECAT_ECAT_EVENT_REQUEST_BF11(x)          (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF11_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF11_MASK)

#define ECAT_ECAT_EVENT_REQUEST_BF12_MASK        (0xF000U)
#define ECAT_ECAT_EVENT_REQUEST_BF12_SHIFT       (12U)
/*! BF12 - Reserved */
#define ECAT_ECAT_EVENT_REQUEST_BF12(x)          (((uint16_t)(((uint16_t)(x)) << ECAT_ECAT_EVENT_REQUEST_BF12_SHIFT)) & ECAT_ECAT_EVENT_REQUEST_BF12_MASK)
/*! @} */

/*! @name AL_EVENT_REQUEST - AL Event request */
/*! @{ */

#define ECAT_AL_EVENT_REQUEST_BF0_MASK           (0x1U)
#define ECAT_AL_EVENT_REQUEST_BF0_SHIFT          (0U)
/*! BF0 - AL Control event:
 *  0b0..No AL Control Register change
 *  0b1..AL Control Register has been written3
 */
#define ECAT_AL_EVENT_REQUEST_BF0(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF0_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF0_MASK)

#define ECAT_AL_EVENT_REQUEST_BF1_MASK           (0x2U)
#define ECAT_AL_EVENT_REQUEST_BF1_SHIFT          (1U)
/*! BF1 - DC Latch event:
 *  0b0..No change on DC Latch Inputs
 *  0b1..At least one change on DC Latch Inputs
 */
#define ECAT_AL_EVENT_REQUEST_BF1(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF1_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF1_MASK)

#define ECAT_AL_EVENT_REQUEST_BF2_MASK           (0x4U)
#define ECAT_AL_EVENT_REQUEST_BF2_SHIFT          (2U)
/*! BF2 - State of DC SYNC0 (if register 0x0151[3]=1): */
#define ECAT_AL_EVENT_REQUEST_BF2(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF2_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF2_MASK)

#define ECAT_AL_EVENT_REQUEST_BF3_MASK           (0x8U)
#define ECAT_AL_EVENT_REQUEST_BF3_SHIFT          (3U)
/*! BF3 - State of DC SYNC1 (if register 0x0151[7]=1): */
#define ECAT_AL_EVENT_REQUEST_BF3(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF3_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF3_MASK)

#define ECAT_AL_EVENT_REQUEST_BF4_MASK           (0x10U)
#define ECAT_AL_EVENT_REQUEST_BF4_SHIFT          (4U)
/*! BF4 - SyncManager activation register (SyncManager register offset 0x6) changed:
 *  0b0..No change in any SyncManager
 *  0b1..At least one SyncManager changed
 */
#define ECAT_AL_EVENT_REQUEST_BF4(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF4_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF4_MASK)

#define ECAT_AL_EVENT_REQUEST_BF5_MASK           (0x20U)
#define ECAT_AL_EVENT_REQUEST_BF5_SHIFT          (5U)
/*! BF5 - EEPROM Emulation:
 *  0b0..No command pending
 *  0b1..EEPROM command pending
 */
#define ECAT_AL_EVENT_REQUEST_BF5(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF5_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF5_MASK)

#define ECAT_AL_EVENT_REQUEST_BF6_MASK           (0x40U)
#define ECAT_AL_EVENT_REQUEST_BF6_SHIFT          (6U)
/*! BF6 - Watchdog Process Data:
 *  0b0..Has not expired
 *  0b1..Has expired
 */
#define ECAT_AL_EVENT_REQUEST_BF6(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF6_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF6_MASK)

#define ECAT_AL_EVENT_REQUEST_BF7_MASK           (0x80U)
#define ECAT_AL_EVENT_REQUEST_BF7_SHIFT          (7U)
/*! BF7 - Reserved */
#define ECAT_AL_EVENT_REQUEST_BF7(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF7_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF7_MASK)

#define ECAT_AL_EVENT_REQUEST_BF8_MASK           (0x100U)
#define ECAT_AL_EVENT_REQUEST_BF8_SHIFT          (8U)
/*! BF8 - SyncManager interrupts (SyncManager register offset 0x5, bit [0] or [1]):
 *  0b0..No SyncManager 0 interrupt
 *  0b1..SyncManager 0 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF8(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF8_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF8_MASK)

#define ECAT_AL_EVENT_REQUEST_BF9_MASK           (0x200U)
#define ECAT_AL_EVENT_REQUEST_BF9_SHIFT          (9U)
/*! BF9 - Bit field access for ECAT: r/-
 *  0b0..No SyncManager 1 interrupt
 *  0b1..SyncManager 1 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF9(x)             (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF9_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF9_MASK)

#define ECAT_AL_EVENT_REQUEST_BF10_MASK          (0x400U)
#define ECAT_AL_EVENT_REQUEST_BF10_SHIFT         (10U)
/*! BF10 - Bit field access for ECAT: r/-
 *  0b0..No SyncManager2 interrupt
 *  0b1..SyncManager 2 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF10(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF10_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF10_MASK)

#define ECAT_AL_EVENT_REQUEST_BF11_MASK          (0x800U)
#define ECAT_AL_EVENT_REQUEST_BF11_SHIFT         (11U)
/*! BF11 - Bit field access for ECAT: r/-
 *  0b0..No SyncManager 3 interrupt
 *  0b1..SyncManager 3 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF11(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF11_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF11_MASK)

#define ECAT_AL_EVENT_REQUEST_BF12_MASK          (0x1000U)
#define ECAT_AL_EVENT_REQUEST_BF12_SHIFT         (12U)
/*! BF12 - Bit field access for ECAT: r/-
 *  0b0..No SyncManager4 interrupt
 *  0b1..SyncManager 4 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF12(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF12_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF12_MASK)

#define ECAT_AL_EVENT_REQUEST_BF13_MASK          (0x2000U)
#define ECAT_AL_EVENT_REQUEST_BF13_SHIFT         (13U)
/*! BF13 - Bit field access for ECAT: r/-
 *  0b0..No SyncManager 5 interrupt
 *  0b1..SyncManager 5 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF13(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF13_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF13_MASK)

#define ECAT_AL_EVENT_REQUEST_BF14_MASK          (0x4000U)
#define ECAT_AL_EVENT_REQUEST_BF14_SHIFT         (14U)
/*! BF14 - Bit field access for ECAT: r/-
 *  0b0..No SyncManager 6 interrupt
 *  0b1..SyncManager6 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF14(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF14_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF14_MASK)

#define ECAT_AL_EVENT_REQUEST_BF15_MASK          (0x8000U)
#define ECAT_AL_EVENT_REQUEST_BF15_SHIFT         (15U)
/*! BF15 - Bit field access for ECAT: r/-
 *  0b0..No SyncManager 7 interrupt
 *  0b1..SyncManager 7 interrupt pending
 */
#define ECAT_AL_EVENT_REQUEST_BF15(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF15_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF15_MASK)

#define ECAT_AL_EVENT_REQUEST_BF16_MASK          (0xFFFF0000U)
#define ECAT_AL_EVENT_REQUEST_BF16_SHIFT         (16U)
/*! BF16 - Reserved */
#define ECAT_AL_EVENT_REQUEST_BF16(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_AL_EVENT_REQUEST_BF16_SHIFT)) & ECAT_AL_EVENT_REQUEST_BF16_MASK)
/*! @} */

/*! @name RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT - RX Error Counter */
/*! @{ */

#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF0_MASK (0xFFU)
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF0_SHIFT (0U)
/*! BF0 - Invalid frame counter of Port y (counting is stopped when 0xFF is reached). */
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF0_SHIFT)) & ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF0_MASK)

#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF8_MASK (0xFF00U)
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF8_SHIFT (8U)
/*! BF8 - RX Error counter of Port y (counting is stopped when 0xFF is reached). */
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF8(x) (((uint16_t)(((uint16_t)(x)) << ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF8_SHIFT)) & ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_BF8_MASK)
/*! @} */

/* The count of ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT */
#define ECAT_RX_ERROR_COUNT_E                    (2U)

/*! @name RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI - RX Error Counter */
/*! @{ */

#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF0_MASK (0xFFU)
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF0_SHIFT (0U)
/*! BF0 - Invalid frame counter of Port y (counting is stopped when 0xFF is reached). */
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF0_SHIFT)) & ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF0_MASK)

#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF8_MASK (0xFF00U)
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF8_SHIFT (8U)
/*! BF8 - RX Error counter of Port y (counting is stopped when 0xFF is reached). */
#define ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF8(x) (((uint16_t)(((uint16_t)(x)) << ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF8_SHIFT)) & ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI_BF8_MASK)
/*! @} */

/* The count of ECAT_RX_ERROR_CNTR_RX_ERROR_COUNTER_PORT_PDI */
#define ECAT_RX_ERROR_COUNT_P                    (2U)

/*! @name FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT - Forwarded RX Error Counter */
/*! @{ */

#define ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_BF0_MASK (0xFFU)
#define ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_BF0_SHIFT (0U)
/*! BF0 - Forwarded error counter of Port y (counting is stopped when 0xFF is reached). */
#define ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_BF0_SHIFT)) & ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_BF0_MASK)
/*! @} */

/* The count of ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT */
#define ECAT_FORWARDED_RX_ERROR_COUNT_E          (2U)

/*! @name FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_PDI - Forwarded RX Error Counter */
/*! @{ */

#define ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_PDI_BF0_MASK (0xFFU)
#define ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_PDI_BF0_SHIFT (0U)
/*! BF0 - Forwarded error counter of Port y (counting is stopped when 0xFF is reached). */
#define ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_PDI_BF0_SHIFT)) & ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_PDI_BF0_MASK)
/*! @} */

/* The count of ECAT_FORWARDED_RX_ERROR_CNTR_FORWARDED_RX_ERROR_COUNTER_PORT_PDI */
#define ECAT_FORWARDED_RX_ERROR_COUNT_P          (2U)

/*! @name ECAT_PROCESSING_UNIT_ERROR_COUNTER - ECAT Processing Unit Error Counter */
/*! @{ */

#define ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_BF0_MASK (0xFFU)
#define ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_BF0_SHIFT (0U)
/*! BF0 - ECAT Processing Unit error counter (counting is stopped when 0xFF is reached). Counts
 *    errors of frames passing the Processing Unit.
 */
#define ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_BF0_SHIFT)) & ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_BF0_MASK)
/*! @} */

/*! @name ECAT_PROCESSING_UNIT_ERROR_COUNTER_PDI - ECAT Processing Unit Error Counter */
/*! @{ */

#define ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_PDI_BF0_MASK (0xFFU)
#define ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_PDI_BF0_SHIFT (0U)
/*! BF0 - ECAT Processing Unit error counter (counting is stopped when 0xFF is reached). Counts
 *    errors of frames passing the Processing Unit.
 */
#define ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_PDI_BF0_SHIFT)) & ECAT_ECAT_PROCESSING_UNIT_ERROR_COUNTER_PDI_BF0_MASK)
/*! @} */

/*! @name PDI_ERROR_COUNTER - PDI Error counter */
/*! @{ */

#define ECAT_PDI_ERROR_COUNTER_BF0_MASK          (0xFFU)
#define ECAT_PDI_ERROR_COUNTER_BF0_SHIFT         (0U)
/*! BF0 - PDI Error counter (counting is stopped when 0xFF is reached). Counts if a PDI access has an interface error. */
#define ECAT_PDI_ERROR_COUNTER_BF0(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_PDI_ERROR_COUNTER_BF0_SHIFT)) & ECAT_PDI_ERROR_COUNTER_BF0_MASK)
/*! @} */

/*! @name PDI_ERROR_COUNTER_PDI - PDI Error counter */
/*! @{ */

#define ECAT_PDI_ERROR_COUNTER_PDI_BF0_MASK      (0xFFU)
#define ECAT_PDI_ERROR_COUNTER_PDI_BF0_SHIFT     (0U)
/*! BF0 - PDI Error counter (counting is stopped when 0xFF is reached). Counts if a PDI access has an interface error. */
#define ECAT_PDI_ERROR_COUNTER_PDI_BF0(x)        (((uint8_t)(((uint8_t)(x)) << ECAT_PDI_ERROR_COUNTER_PDI_BF0_SHIFT)) & ECAT_PDI_ERROR_COUNTER_PDI_BF0_MASK)
/*! @} */

/*! @name ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER - ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_PDI_ERROR_CODE. */
/*! @{ */

#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF0_MASK (0x1U)
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF0_SHIFT (0U)
/*! BF0 - Busy violation during read access
 *  0b0..no error
 *  0b1..error detected
 */
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF0_SHIFT)) & ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF0_MASK)

#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF1_MASK (0x2U)
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF1_SHIFT (1U)
/*! BF1 - Busy violation during write access
 *  0b0..no error
 *  0b1..error detected
 */
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF1_SHIFT)) & ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF1_MASK)

#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF2_MASK (0x4U)
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF2_SHIFT (2U)
/*! BF2 - Addressing error for a read access (odd address without BHE)
 *  0b0..no error
 *  0b1..error detected
 */
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF2(x) (((uint8_t)(((uint8_t)(x)) << ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF2_SHIFT)) & ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF2_MASK)

#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF3_MASK (0x8U)
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF3_SHIFT (3U)
/*! BF3 - Addressing error for a write access (odd address without BHE)
 *  0b0..no error
 *  0b1..error detected
 */
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF3(x) (((uint8_t)(((uint8_t)(x)) << ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF3_SHIFT)) & ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF3_MASK)

#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF4_MASK (0xF0U)
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF4_SHIFT (4U)
/*! BF4 - reserved */
#define ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF4(x) (((uint8_t)(((uint8_t)(x)) << ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF4_SHIFT)) & ECAT_ASYNCHRONOUS_SYNCHRONOUS_MICROCONTROLLER_BF4_MASK)
/*! @} */

/*! @name LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT - Lost Link Counter */
/*! @{ */

#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_BF0_MASK (0xFFU)
#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_BF0_SHIFT (0U)
/*! BF0 - Lost Link counter of Port y (counting is stopped when 0xff is reached). */
#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_BF0_SHIFT)) & ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_BF0_MASK)
/*! @} */

/* The count of ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT */
#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_COUNT (2U)

/*! @name LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI - Lost Link Counter */
/*! @{ */

#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI_BF0_MASK (0xFFU)
#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI_BF0_SHIFT (0U)
/*! BF0 - Lost Link counter of Port y (counting is stopped when 0xff is reached). */
#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI_BF0_SHIFT)) & ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI_BF0_MASK)
/*! @} */

/* The count of ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI */
#define ECAT_LOST_LINK_CNTR_LOST_LINK_COUNTER_PORT_PDI_COUNT (2U)

/*! @name WATCHDOG_DIVIDER - Watchdog Divider */
/*! @{ */

#define ECAT_WATCHDOG_DIVIDER_BF0_MASK           (0xFFFFU)
#define ECAT_WATCHDOG_DIVIDER_BF0_SHIFT          (0U)
/*! BF0 - Watchdog divider: Number of 25 MHz tics (minus 2) that represent the basic watchdog
 *    increment. (Default value is 100us = 2498)
 */
#define ECAT_WATCHDOG_DIVIDER_BF0(x)             (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_DIVIDER_BF0_SHIFT)) & ECAT_WATCHDOG_DIVIDER_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_DIVIDER_PDI - Watchdog Divider */
/*! @{ */

#define ECAT_WATCHDOG_DIVIDER_PDI_BF0_MASK       (0xFFFFU)
#define ECAT_WATCHDOG_DIVIDER_PDI_BF0_SHIFT      (0U)
/*! BF0 - Watchdog divider: Number of 25 MHz tics (minus 2) that represent the basic watchdog
 *    increment. (Default value is 100us = 2498)
 */
#define ECAT_WATCHDOG_DIVIDER_PDI_BF0(x)         (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_DIVIDER_PDI_BF0_SHIFT)) & ECAT_WATCHDOG_DIVIDER_PDI_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_TIME_PDI - Register Watchdog Time PDI */
/*! @{ */

#define ECAT_WATCHDOG_TIME_PDI_BF0_MASK          (0xFFFFU)
#define ECAT_WATCHDOG_TIME_PDI_BF0_SHIFT         (0U)
/*! BF0 - Watchdog Time PDI: number or basic watchdog increments (Default value with Watchdog divider 100us means 100ms Watchdog) */
#define ECAT_WATCHDOG_TIME_PDI_BF0(x)            (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_TIME_PDI_BF0_SHIFT)) & ECAT_WATCHDOG_TIME_PDI_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_TIME_PDI_PDI - Register Watchdog Time PDI */
/*! @{ */

#define ECAT_WATCHDOG_TIME_PDI_PDI_BF0_MASK      (0xFFFFU)
#define ECAT_WATCHDOG_TIME_PDI_PDI_BF0_SHIFT     (0U)
/*! BF0 - Watchdog Time PDI: number or basic watchdog increments (Default value with Watchdog divider 100us means 100ms Watchdog) */
#define ECAT_WATCHDOG_TIME_PDI_PDI_BF0(x)        (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_TIME_PDI_PDI_BF0_SHIFT)) & ECAT_WATCHDOG_TIME_PDI_PDI_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_TIME_PROCESS_DATA - Regsister Watchdog Time Process Data */
/*! @{ */

#define ECAT_WATCHDOG_TIME_PROCESS_DATA_BF0_MASK (0xFFFFU)
#define ECAT_WATCHDOG_TIME_PROCESS_DATA_BF0_SHIFT (0U)
/*! BF0 - Watchdog Time Process Data */
#define ECAT_WATCHDOG_TIME_PROCESS_DATA_BF0(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_TIME_PROCESS_DATA_BF0_SHIFT)) & ECAT_WATCHDOG_TIME_PROCESS_DATA_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_TIME_PROCESS_DATA_PDI - Regsister Watchdog Time Process Data */
/*! @{ */

#define ECAT_WATCHDOG_TIME_PROCESS_DATA_PDI_BF0_MASK (0xFFFFU)
#define ECAT_WATCHDOG_TIME_PROCESS_DATA_PDI_BF0_SHIFT (0U)
/*! BF0 - Watchdog Time Process Data */
#define ECAT_WATCHDOG_TIME_PROCESS_DATA_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_TIME_PROCESS_DATA_PDI_BF0_SHIFT)) & ECAT_WATCHDOG_TIME_PROCESS_DATA_PDI_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_STATUS_PROCESS_DATA - Watchdog Status Process Data */
/*! @{ */

#define ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF0_MASK (0x1U)
#define ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF0_SHIFT (0U)
/*! BF0 - Watchdog Status of Process Data (triggered by SyncManagers)
 *  0b0..Watchdog Process Data expired
 *  0b1..Watchdog Process Data is active or disabled
 */
#define ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF0_SHIFT)) & ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF0_MASK)

#define ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF1_MASK (0xFFFEU)
#define ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF1_SHIFT (1U)
/*! BF1 - Reserved */
#define ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF1(x) (((uint16_t)(((uint16_t)(x)) << ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF1_SHIFT)) & ECAT_WATCHDOG_STATUS_PROCESS_DATA_BF1_MASK)
/*! @} */

/*! @name WATCHDOG_COUNTER_PROCESS_DATA - Watchdog Counter Process Data */
/*! @{ */

#define ECAT_WATCHDOG_COUNTER_PROCESS_DATA_BF0_MASK (0xFFU)
#define ECAT_WATCHDOG_COUNTER_PROCESS_DATA_BF0_SHIFT (0U)
/*! BF0 - Watchdog Counter Process Data (counting is stopped when 0xFF is reached). Counts if Process Data Watchdog expires. */
#define ECAT_WATCHDOG_COUNTER_PROCESS_DATA_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_WATCHDOG_COUNTER_PROCESS_DATA_BF0_SHIFT)) & ECAT_WATCHDOG_COUNTER_PROCESS_DATA_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_COUNTER_PROCESS_DATA_PDI - Watchdog Counter Process Data */
/*! @{ */

#define ECAT_WATCHDOG_COUNTER_PROCESS_DATA_PDI_BF0_MASK (0xFFU)
#define ECAT_WATCHDOG_COUNTER_PROCESS_DATA_PDI_BF0_SHIFT (0U)
/*! BF0 - Watchdog Counter Process Data (counting is stopped when 0xFF is reached). Counts if Process Data Watchdog expires. */
#define ECAT_WATCHDOG_COUNTER_PROCESS_DATA_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_WATCHDOG_COUNTER_PROCESS_DATA_PDI_BF0_SHIFT)) & ECAT_WATCHDOG_COUNTER_PROCESS_DATA_PDI_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_COUNTER_PDI - Watchdog Counter PDI */
/*! @{ */

#define ECAT_WATCHDOG_COUNTER_PDI_BF0_MASK       (0xFFU)
#define ECAT_WATCHDOG_COUNTER_PDI_BF0_SHIFT      (0U)
/*! BF0 - Watchdog PDI counter (counting is stopped when 0xFF is reached). */
#define ECAT_WATCHDOG_COUNTER_PDI_BF0(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_WATCHDOG_COUNTER_PDI_BF0_SHIFT)) & ECAT_WATCHDOG_COUNTER_PDI_BF0_MASK)
/*! @} */

/*! @name WATCHDOG_COUNTER_PDI_PDI - Watchdog Counter PDI */
/*! @{ */

#define ECAT_WATCHDOG_COUNTER_PDI_PDI_BF0_MASK   (0xFFU)
#define ECAT_WATCHDOG_COUNTER_PDI_PDI_BF0_SHIFT  (0U)
/*! BF0 - Watchdog PDI counter (counting is stopped when 0xFF is reached). */
#define ECAT_WATCHDOG_COUNTER_PDI_PDI_BF0(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_WATCHDOG_COUNTER_PDI_PDI_BF0_SHIFT)) & ECAT_WATCHDOG_COUNTER_PDI_PDI_BF0_MASK)
/*! @} */

/*! @name EEPROM_CONFIGURATION - EEPROM Configuration */
/*! @{ */

#define ECAT_EEPROM_CONFIGURATION_BF0_MASK       (0x1U)
#define ECAT_EEPROM_CONFIGURATION_BF0_SHIFT      (0U)
/*! BF0 - EEPROM control is offered to PDI
 *  0b0..no
 *  0b1..yes (PDI has EEPROM control)
 */
#define ECAT_EEPROM_CONFIGURATION_BF0(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_EEPROM_CONFIGURATION_BF0_SHIFT)) & ECAT_EEPROM_CONFIGURATION_BF0_MASK)

#define ECAT_EEPROM_CONFIGURATION_BF1_MASK       (0x2U)
#define ECAT_EEPROM_CONFIGURATION_BF1_SHIFT      (1U)
/*! BF1 - Force ECAT access
 *  0b0..Do not change Bit 0x0501[0]
 *  0b1..Reset Bit 0x0501[0] to 0
 */
#define ECAT_EEPROM_CONFIGURATION_BF1(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_EEPROM_CONFIGURATION_BF1_SHIFT)) & ECAT_EEPROM_CONFIGURATION_BF1_MASK)

#define ECAT_EEPROM_CONFIGURATION_BF2_MASK       (0xFCU)
#define ECAT_EEPROM_CONFIGURATION_BF2_SHIFT      (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_EEPROM_CONFIGURATION_BF2(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_EEPROM_CONFIGURATION_BF2_SHIFT)) & ECAT_EEPROM_CONFIGURATION_BF2_MASK)
/*! @} */

/*! @name EEPROM_CONFIGURATION_PDI - EEPROM Configuration */
/*! @{ */

#define ECAT_EEPROM_CONFIGURATION_PDI_BF0_MASK   (0x1U)
#define ECAT_EEPROM_CONFIGURATION_PDI_BF0_SHIFT  (0U)
/*! BF0 - EEPROM control is offered to PDI
 *  0b0..no
 *  0b1..yes (PDI has EEPROM control)
 */
#define ECAT_EEPROM_CONFIGURATION_PDI_BF0(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_EEPROM_CONFIGURATION_PDI_BF0_SHIFT)) & ECAT_EEPROM_CONFIGURATION_PDI_BF0_MASK)

#define ECAT_EEPROM_CONFIGURATION_PDI_BF1_MASK   (0x2U)
#define ECAT_EEPROM_CONFIGURATION_PDI_BF1_SHIFT  (1U)
/*! BF1 - Force ECAT access
 *  0b0..Do not change Bit 0x0501[0]
 *  0b1..Reset Bit 0x0501[0] to 0
 */
#define ECAT_EEPROM_CONFIGURATION_PDI_BF1(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_EEPROM_CONFIGURATION_PDI_BF1_SHIFT)) & ECAT_EEPROM_CONFIGURATION_PDI_BF1_MASK)

#define ECAT_EEPROM_CONFIGURATION_PDI_BF2_MASK   (0xFCU)
#define ECAT_EEPROM_CONFIGURATION_PDI_BF2_SHIFT  (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_EEPROM_CONFIGURATION_PDI_BF2(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_EEPROM_CONFIGURATION_PDI_BF2_SHIFT)) & ECAT_EEPROM_CONFIGURATION_PDI_BF2_MASK)
/*! @} */

/*! @name REGISTER_EEPROM_PDI_ACCESS_STATE - EEPROM PDI Access State */
/*! @{ */

#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF0_MASK (0x1U)
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF0_SHIFT (0U)
/*! BF0 - Access to EEPROM:
 *  0b0..PDI releases EEPROM access
 *  0b1..PDI takes EEPROM access (PDI has EEPROM control)
 */
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF0_SHIFT)) & ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF0_MASK)

#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF1_MASK (0xFEU)
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF1_SHIFT)) & ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_BF1_MASK)
/*! @} */

/*! @name REGISTER_EEPROM_PDI_ACCESS_STATE_PDI - EEPROM PDI Access State */
/*! @{ */

#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF0_MASK (0x1U)
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF0_SHIFT (0U)
/*! BF0 - Access to EEPROM:
 *  0b0..PDI releases EEPROM access
 *  0b1..PDI takes EEPROM access (PDI has EEPROM control)
 */
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF0_SHIFT)) & ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF0_MASK)

#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF1_MASK (0xFEU)
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF1_SHIFT)) & ECAT_REGISTER_EEPROM_PDI_ACCESS_STATE_PDI_BF1_MASK)
/*! @} */

/*! @name EEPROM_CONTROL_STATUS - Register EEPROM Control/Status */
/*! @{ */

#define ECAT_EEPROM_CONTROL_STATUS_BF0_MASK      (0x1U)
#define ECAT_EEPROM_CONTROL_STATUS_BF0_SHIFT     (0U)
/*! BF0 - ECAT write enable2
 *  0b0..Write requests are disabled
 *  0b1..Write requests are enabled
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF0(x)        (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF0_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF0_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF1_MASK      (0x1EU)
#define ECAT_EEPROM_CONTROL_STATUS_BF1_SHIFT     (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_EEPROM_CONTROL_STATUS_BF1(x)        (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF1_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF1_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF5_MASK      (0x20U)
#define ECAT_EEPROM_CONTROL_STATUS_BF5_SHIFT     (5U)
/*! BF5 - EEPROM emulation:
 *  0b0..Normal operation (I2C interface used)
 *  0b1..PDI emulates EEPROM (I2C not used)
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF5(x)        (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF5_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF5_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF6_MASK      (0x40U)
#define ECAT_EEPROM_CONTROL_STATUS_BF6_SHIFT     (6U)
/*! BF6 - Supported number of EEPROM read bytes:
 *  0b0..4 Bytes
 *  0b1..8 Bytes
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF6(x)        (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF6_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF6_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF7_MASK      (0x80U)
#define ECAT_EEPROM_CONTROL_STATUS_BF7_SHIFT     (7U)
/*! BF7 - Selected EEPROM Algorithm:
 *  0b0..1 address byte (1Kbit to 16Kbit EEPROMs)
 *  0b1..2 address bytes (32Kbit to 4 Mbit EEPROMs)
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF7(x)        (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF7_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF7_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF8_MASK      (0x700U)
#define ECAT_EEPROM_CONTROL_STATUS_BF8_SHIFT     (8U)
/*! BF8 - Command register
 *  0b000..No command/EEPROM idle (clear error bits)
 *  0b001..Read
 *  0b010..Write
 *  0b100..Reload
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF8(x)        (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF8_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF8_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF11_MASK     (0x800U)
#define ECAT_EEPROM_CONTROL_STATUS_BF11_SHIFT    (11U)
/*! BF11 - Checksum Error in ESC Configuration Area:
 *  0b0..Checksum ok
 *  0b1..Checksum error
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF11(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF11_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF11_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF12_MASK     (0x1000U)
#define ECAT_EEPROM_CONTROL_STATUS_BF12_SHIFT    (12U)
/*! BF12 - EEPROM loading status:
 *  0b0..EEPROM loaded, device information ok
 *  0b1..EEPROM not loaded, device information not available (EEPROM loading in progress or finished with a failure)
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF12(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF12_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF12_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF13_MASK     (0x2000U)
#define ECAT_EEPROM_CONTROL_STATUS_BF13_SHIFT    (13U)
/*! BF13 - Error Acknowledge/Command3:
 *  0b0..No error
 *  0b1..Missing EEPROM acknowledge or invalid command
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF13(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF13_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF13_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF14_MASK     (0x4000U)
#define ECAT_EEPROM_CONTROL_STATUS_BF14_SHIFT    (14U)
/*! BF14 - Error Write Enable3:
 *  0b0..No error
 *  0b1..Write Command without Write enable
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF14(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF14_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF14_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_BF15_MASK     (0x8000U)
#define ECAT_EEPROM_CONTROL_STATUS_BF15_SHIFT    (15U)
/*! BF15 - Busy:
 *  0b0..EEPROM Interface is idle
 *  0b1..EEPROM Interface is busy
 */
#define ECAT_EEPROM_CONTROL_STATUS_BF15(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_BF15_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_BF15_MASK)
/*! @} */

/*! @name EEPROM_CONTROL_STATUS_PDI - Register EEPROM Control/Status */
/*! @{ */

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF0_MASK  (0x1U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF0_SHIFT (0U)
/*! BF0 - ECAT write enable2
 *  0b0..Write requests are disabled
 *  0b1..Write requests are enabled
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF0(x)    (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF0_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF0_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF1_MASK  (0x1EU)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF1(x)    (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF1_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF1_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF5_MASK  (0x20U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF5_SHIFT (5U)
/*! BF5 - EEPROM emulation:
 *  0b0..Normal operation (I2C interface used)
 *  0b1..PDI emulates EEPROM (I2C not used)
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF5(x)    (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF5_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF5_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF6_MASK  (0x40U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF6_SHIFT (6U)
/*! BF6 - Supported number of EEPROM read bytes:
 *  0b0..4 Bytes
 *  0b1..8 Bytes
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF6(x)    (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF6_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF6_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF7_MASK  (0x80U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF7_SHIFT (7U)
/*! BF7 - Selected EEPROM Algorithm:
 *  0b0..1 address byte (1Kbit to 16Kbit EEPROMs)
 *  0b1..2 address bytes (32Kbit to 4 Mbit EEPROMs)
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF7(x)    (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF7_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF7_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF8_MASK  (0x700U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF8_SHIFT (8U)
/*! BF8 - Command register
 *  0b000..No command/EEPROM idle (clear error bits)
 *  0b001..Read
 *  0b010..Write
 *  0b100..Reload
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF8(x)    (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF8_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF8_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF11_MASK (0x800U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF11_SHIFT (11U)
/*! BF11 - Checksum Error in ESC Configuration Area:
 *  0b0..Checksum ok
 *  0b1..Checksum error
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF11(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF11_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF11_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF12_MASK (0x1000U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF12_SHIFT (12U)
/*! BF12 - EEPROM loading status:
 *  0b0..EEPROM loaded, device information ok
 *  0b1..EEPROM not loaded, device information not available (EEPROM loading in progress or finished with a failure)
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF12(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF12_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF12_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF13_MASK (0x2000U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF13_SHIFT (13U)
/*! BF13 - Error Acknowledge/Command3:
 *  0b0..No error
 *  0b1..Missing EEPROM acknowledge or invalid command
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF13(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF13_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF13_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF14_MASK (0x4000U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF14_SHIFT (14U)
/*! BF14 - Error Write Enable3:
 *  0b0..No error
 *  0b1..Write Command without Write enable
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF14(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF14_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF14_MASK)

#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF15_MASK (0x8000U)
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF15_SHIFT (15U)
/*! BF15 - Busy:
 *  0b0..EEPROM Interface is idle
 *  0b1..EEPROM Interface is busy
 */
#define ECAT_EEPROM_CONTROL_STATUS_PDI_BF15(x)   (((uint16_t)(((uint16_t)(x)) << ECAT_EEPROM_CONTROL_STATUS_PDI_BF15_SHIFT)) & ECAT_EEPROM_CONTROL_STATUS_PDI_BF15_MASK)
/*! @} */

/*! @name EEPROM_ADDRESS - EEPROM Address */
/*! @{ */

#define ECAT_EEPROM_ADDRESS_BF0_MASK             (0xFFFFFFFFU)
#define ECAT_EEPROM_ADDRESS_BF0_SHIFT            (0U)
/*! BF0 - EEPROM Address */
#define ECAT_EEPROM_ADDRESS_BF0(x)               (((uint32_t)(((uint32_t)(x)) << ECAT_EEPROM_ADDRESS_BF0_SHIFT)) & ECAT_EEPROM_ADDRESS_BF0_MASK)
/*! @} */

/*! @name EEPROM_DATA - EEPROM Data */
/*! @{ */

#define ECAT_EEPROM_DATA_BF0_MASK                (0xFFFFU)
#define ECAT_EEPROM_DATA_BF0_SHIFT               (0U)
/*! BF0 - EEPROM Write data (data to be written to EEPROM) or EEPROM Read data (data read from EEPROM,. lower bytes) */
#define ECAT_EEPROM_DATA_BF0(x)                  (((uint64_t)(((uint64_t)(x)) << ECAT_EEPROM_DATA_BF0_SHIFT)) & ECAT_EEPROM_DATA_BF0_MASK)

#define ECAT_EEPROM_DATA_BF16_MASK               (0xFFFFFFFFFFFF0000U)
#define ECAT_EEPROM_DATA_BF16_SHIFT              (16U)
/*! BF16 - EEPROM Read data (data read from EEPROM, higher bytes) */
#define ECAT_EEPROM_DATA_BF16(x)                 (((uint64_t)(((uint64_t)(x)) << ECAT_EEPROM_DATA_BF16_SHIFT)) & ECAT_EEPROM_DATA_BF16_MASK)
/*! @} */

/*! @name MII_MANAGEMENT_CONTROL_OR_STATUS - MII Management Control/Status */
/*! @{ */

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF0_MASK (0x1U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF0_SHIFT (0U)
/*! BF0 - Write enable*:
 *  0b0..Write disabled
 *  0b1..Write enabled
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF0_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF0_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF1_MASK (0x2U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF1_SHIFT (1U)
/*! BF1 - Management Interface can be controlled by PDI (registers 0x0516-0x0517):
 *  0b0..Only ECAT control
 *  0b1..PDI control possible
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF1(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF1_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF1_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF2_MASK (0x4U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF2_SHIFT (2U)
/*! BF2 - MI link detection and configuration:
 *  0b0..Disabled for all ports
 *  0b1..Enabled for at least one MII port, refer to PHY Port Status (0x0518 ff.) for details
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF2(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF2_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF2_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF3_MASK (0xF8U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF3_SHIFT (3U)
/*! BF3 - PHY address of port 0 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF3(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF3_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF3_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF8_MASK (0x300U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF8_SHIFT (8U)
/*! BF8 - Command register*:
 *  0b00..No command/MI idle (clear error bits)
 *  0b01..Read
 *  0b10..Write
 *  0b11..Reserved/invalid command (do not issue)
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF8(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF8_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF8_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF10_MASK (0x1C00U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF10_SHIFT (10U)
/*! BF10 - Reserved, write 0 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF10(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF10_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF10_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF13_MASK (0x2000U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF13_SHIFT (13U)
/*! BF13 - Read error:
 *  0b0..No read error
 *  0b1..Read error occurred (PHY or register not available)
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF13(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF13_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF13_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF14_MASK (0x4000U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF14_SHIFT (14U)
/*! BF14 - Command error:
 *  0b0..Last Command was successful
 *  0b1..Invalid command or write command without Write Enable
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF14(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF14_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF14_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF15_MASK (0x8000U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF15_SHIFT (15U)
/*! BF15 - Busy:
 *  0b0..MII Management Interface is idle
 *  0b1..MII Management Interface is busy
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF15(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF15_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_BF15_MASK)
/*! @} */

/*! @name MII_MANAGEMENT_CONTROL_OR_STATUS_PDI - MII Management Control/Status */
/*! @{ */

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF0_MASK (0x1U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF0_SHIFT (0U)
/*! BF0 - Write enable*:
 *  0b0..Write disabled
 *  0b1..Write enabled
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF0_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF0_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF1_MASK (0x2U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF1_SHIFT (1U)
/*! BF1 - Management Interface can be controlled by PDI (registers 0x0516-0x0517):
 *  0b0..Only ECAT control
 *  0b1..PDI control possible
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF1(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF1_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF1_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF2_MASK (0x4U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF2_SHIFT (2U)
/*! BF2 - MI link detection and configuration:
 *  0b0..Disabled for all ports
 *  0b1..Enabled for at least one MII port, refer to PHY Port Status (0x0518 ff.) for details
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF2(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF2_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF2_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF3_MASK (0xF8U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF3_SHIFT (3U)
/*! BF3 - PHY address of port 0 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF3(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF3_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF3_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF8_MASK (0x300U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF8_SHIFT (8U)
/*! BF8 - Command register*:
 *  0b00..No command/MI idle (clear error bits)
 *  0b01..Read
 *  0b10..Write
 *  0b11..Reserved/invalid command (do not issue)
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF8(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF8_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF8_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF10_MASK (0x1C00U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF10_SHIFT (10U)
/*! BF10 - Reserved, write 0 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF10(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF10_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF10_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF13_MASK (0x2000U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF13_SHIFT (13U)
/*! BF13 - Read error:
 *  0b0..No read error
 *  0b1..Read error occurred (PHY or register not available)
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF13(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF13_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF13_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF14_MASK (0x4000U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF14_SHIFT (14U)
/*! BF14 - Command error:
 *  0b0..Last Command was successful
 *  0b1..Invalid command or write command without Write Enable
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF14(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF14_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF14_MASK)

#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF15_MASK (0x8000U)
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF15_SHIFT (15U)
/*! BF15 - Busy:
 *  0b0..MII Management Interface is idle
 *  0b1..MII Management Interface is busy
 */
#define ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF15(x) (((uint16_t)(((uint16_t)(x)) << ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF15_SHIFT)) & ECAT_MII_MANAGEMENT_CONTROL_OR_STATUS_PDI_BF15_MASK)
/*! @} */

/*! @name PHY_ADDRESS - PHY Address */
/*! @{ */

#define ECAT_PHY_ADDRESS_BF0_MASK                (0x1FU)
#define ECAT_PHY_ADDRESS_BF0_SHIFT               (0U)
/*! BF0 - PHY Address */
#define ECAT_PHY_ADDRESS_BF0(x)                  (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_ADDRESS_BF0_SHIFT)) & ECAT_PHY_ADDRESS_BF0_MASK)

#define ECAT_PHY_ADDRESS_BF5_MASK                (0x60U)
#define ECAT_PHY_ADDRESS_BF5_SHIFT               (5U)
/*! BF5 - Reserved, write 0 */
#define ECAT_PHY_ADDRESS_BF5(x)                  (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_ADDRESS_BF5_SHIFT)) & ECAT_PHY_ADDRESS_BF5_MASK)

#define ECAT_PHY_ADDRESS_BF7_MASK                (0x80U)
#define ECAT_PHY_ADDRESS_BF7_SHIFT               (7U)
/*! BF7 - Show configured PHY address of port 0-3 in register 0x0510[7:3]. This is used if the PHY addresses are not consecutive.
 *  0b0..Register 0x0510[7:3] shows PHY address of port 0 (this is also the PHY address offset, if the PHY addresses are consecutive)
 *  0b1..Register 0x0510[7:3] shows PHY address of port 0x0512[4:0] (valid values 0-3)
 */
#define ECAT_PHY_ADDRESS_BF7(x)                  (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_ADDRESS_BF7_SHIFT)) & ECAT_PHY_ADDRESS_BF7_MASK)
/*! @} */

/*! @name PHY_REGISTER_ADDRESS - PHY Register Address */
/*! @{ */

#define ECAT_PHY_REGISTER_ADDRESS_BF0_MASK       (0x1FU)
#define ECAT_PHY_REGISTER_ADDRESS_BF0_SHIFT      (0U)
/*! BF0 - Address of PHY Register that shall be read/written */
#define ECAT_PHY_REGISTER_ADDRESS_BF0(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_REGISTER_ADDRESS_BF0_SHIFT)) & ECAT_PHY_REGISTER_ADDRESS_BF0_MASK)

#define ECAT_PHY_REGISTER_ADDRESS_BF5_MASK       (0xE0U)
#define ECAT_PHY_REGISTER_ADDRESS_BF5_SHIFT      (5U)
/*! BF5 - Reserved, write 0 */
#define ECAT_PHY_REGISTER_ADDRESS_BF5(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_REGISTER_ADDRESS_BF5_SHIFT)) & ECAT_PHY_REGISTER_ADDRESS_BF5_MASK)
/*! @} */

/*! @name PHY_DATA - PHY Data */
/*! @{ */

#define ECAT_PHY_DATA_BF0_MASK                   (0xFFFFU)
#define ECAT_PHY_DATA_BF0_SHIFT                  (0U)
/*! BF0 - PHY Read/Write Data */
#define ECAT_PHY_DATA_BF0(x)                     (((uint16_t)(((uint16_t)(x)) << ECAT_PHY_DATA_BF0_SHIFT)) & ECAT_PHY_DATA_BF0_MASK)
/*! @} */

/*! @name MII_MANAGEMENT_ECAT_ACCESS_STATE - MII Management ECAT Access State */
/*! @{ */

#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF0_MASK (0x1U)
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF0_SHIFT (0U)
/*! BF0 - Access to MII management:
 *  0b0..ECAT enables PDI takeover of MII management interface
 *  0b1..ECAT claims exclusive access to MII management interface
 */
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF0_SHIFT)) & ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF0_MASK)

#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF1_MASK (0xFEU)
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF1_SHIFT)) & ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_BF1_MASK)
/*! @} */

/*! @name MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI - MII Management ECAT Access State */
/*! @{ */

#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF0_MASK (0x1U)
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF0_SHIFT (0U)
/*! BF0 - Access to MII management:
 *  0b0..ECAT enables PDI takeover of MII management interface
 *  0b1..ECAT claims exclusive access to MII management interface
 */
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF0_SHIFT)) & ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF0_MASK)

#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF1_MASK (0xFEU)
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF1_SHIFT (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF1_SHIFT)) & ECAT_MII_MANAGEMENT_ECAT_ACCESS_STATE_PDI_BF1_MASK)
/*! @} */

/*! @name MII_MANAGEMENT_PDI_ACCESS_STATE - MII Management PDI Access State */
/*! @{ */

#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF0_MASK (0x1U)
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF0_SHIFT (0U)
/*! BF0 - Access to MII management:
 *  0b0..ECAT has access to MII management
 *  0b1..PDI has access to MII management
 */
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF0_SHIFT)) & ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF0_MASK)

#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF1_MASK (0x2U)
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF1_SHIFT (1U)
/*! BF1 - Force PDI Access State:
 *  0b0..Do not change Bit 0x0517[0]
 *  0b1..Reset Bit 0x0517[0] to 0
 */
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF1_SHIFT)) & ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF1_MASK)

#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF2_MASK (0xFCU)
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF2_SHIFT (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF2(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF2_SHIFT)) & ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_BF2_MASK)
/*! @} */

/*! @name MII_MANAGEMENT_PDI_ACCESS_STATE_PDI - MII Management PDI Access State */
/*! @{ */

#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF0_MASK (0x1U)
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF0_SHIFT (0U)
/*! BF0 - Access to MII management:
 *  0b0..ECAT has access to MII management
 *  0b1..PDI has access to MII management
 */
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF0_SHIFT)) & ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF0_MASK)

#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF1_MASK (0x2U)
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF1_SHIFT (1U)
/*! BF1 - Force PDI Access State:
 *  0b0..Do not change Bit 0x0517[0]
 *  0b1..Reset Bit 0x0517[0] to 0
 */
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF1(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF1_SHIFT)) & ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF1_MASK)

#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF2_MASK (0xFCU)
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF2_SHIFT (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF2(x) (((uint8_t)(((uint8_t)(x)) << ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF2_SHIFT)) & ECAT_MII_MANAGEMENT_PDI_ACCESS_STATE_PDI_BF2_MASK)
/*! @} */

/*! @name PHY_PORT_STATUS - PHY Port */
/*! @{ */

#define ECAT_PHY_PORT_STATUS_BF0_MASK            (0x1U)
#define ECAT_PHY_PORT_STATUS_BF0_SHIFT           (0U)
/*! BF0 - Physical link status (PHY status register 1.2):
 *  0b0..No physical link
 *  0b1..Physical link detected
 */
#define ECAT_PHY_PORT_STATUS_BF0(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_PORT_STATUS_BF0_SHIFT)) & ECAT_PHY_PORT_STATUS_BF0_MASK)

#define ECAT_PHY_PORT_STATUS_BF1_MASK            (0x2U)
#define ECAT_PHY_PORT_STATUS_BF1_SHIFT           (1U)
/*! BF1 - Link status (100 Mbit/s, Full Duplex, Auto negotiation):
 *  0b0..No link
 *  0b1..Link detected
 */
#define ECAT_PHY_PORT_STATUS_BF1(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_PORT_STATUS_BF1_SHIFT)) & ECAT_PHY_PORT_STATUS_BF1_MASK)

#define ECAT_PHY_PORT_STATUS_BF2_MASK            (0x4U)
#define ECAT_PHY_PORT_STATUS_BF2_SHIFT           (2U)
/*! BF2 - Link status error:
 *  0b0..No error
 *  0b1..Link error, link inhibited
 */
#define ECAT_PHY_PORT_STATUS_BF2(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_PORT_STATUS_BF2_SHIFT)) & ECAT_PHY_PORT_STATUS_BF2_MASK)

#define ECAT_PHY_PORT_STATUS_BF3_MASK            (0x8U)
#define ECAT_PHY_PORT_STATUS_BF3_SHIFT           (3U)
/*! BF3 - Read error:
 *  0b0..No read error occurred
 *  0b1..A read error has occurred
 */
#define ECAT_PHY_PORT_STATUS_BF3(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_PORT_STATUS_BF3_SHIFT)) & ECAT_PHY_PORT_STATUS_BF3_MASK)

#define ECAT_PHY_PORT_STATUS_BF4_MASK            (0x10U)
#define ECAT_PHY_PORT_STATUS_BF4_SHIFT           (4U)
/*! BF4 - Link partner error:
 *  0b0..No error detected
 *  0b1..Link partner error
 */
#define ECAT_PHY_PORT_STATUS_BF4(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_PORT_STATUS_BF4_SHIFT)) & ECAT_PHY_PORT_STATUS_BF4_MASK)

#define ECAT_PHY_PORT_STATUS_BF5_MASK            (0x20U)
#define ECAT_PHY_PORT_STATUS_BF5_SHIFT           (5U)
/*! BF5 - PHY configuration updated
 *  0b0..No update
 *  0b1..PHY configuration was updated
 */
#define ECAT_PHY_PORT_STATUS_BF5(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_PORT_STATUS_BF5_SHIFT)) & ECAT_PHY_PORT_STATUS_BF5_MASK)

#define ECAT_PHY_PORT_STATUS_BF6_MASK            (0xC0U)
#define ECAT_PHY_PORT_STATUS_BF6_SHIFT           (6U)
/*! BF6 - Reserved */
#define ECAT_PHY_PORT_STATUS_BF6(x)              (((uint8_t)(((uint8_t)(x)) << ECAT_PHY_PORT_STATUS_BF6_SHIFT)) & ECAT_PHY_PORT_STATUS_BF6_MASK)
/*! @} */

/* The count of ECAT_PHY_PORT_STATUS */
#define ECAT_PHY_PORT_STATUS_COUNT               (2U)

/*! @name FMMU_LOGICAL_START_ADDRESS - Register Logical Start address FMMU */
/*! @{ */

#define ECAT_FMMU_LOGICAL_START_ADDRESS_BF0_MASK (0xFFFFFFFFU)
#define ECAT_FMMU_LOGICAL_START_ADDRESS_BF0_SHIFT (0U)
/*! BF0 - Logical start address within the EtherCAT Address Space. */
#define ECAT_FMMU_LOGICAL_START_ADDRESS_BF0(x)   (((uint32_t)(((uint32_t)(x)) << ECAT_FMMU_LOGICAL_START_ADDRESS_BF0_SHIFT)) & ECAT_FMMU_LOGICAL_START_ADDRESS_BF0_MASK)
/*! @} */

/* The count of ECAT_FMMU_LOGICAL_START_ADDRESS */
#define ECAT_FMMU_LOGICAL_START_ADDRESS_COUNT    (8U)

/*! @name FMMU_LOGICAL_START_ADDRESS_PDI - Register Logical Start address FMMU */
/*! @{ */

#define ECAT_FMMU_LOGICAL_START_ADDRESS_PDI_BF0_MASK (0xFFFFFFFFU)
#define ECAT_FMMU_LOGICAL_START_ADDRESS_PDI_BF0_SHIFT (0U)
/*! BF0 - Logical start address within the EtherCAT Address Space. */
#define ECAT_FMMU_LOGICAL_START_ADDRESS_PDI_BF0(x) (((uint32_t)(((uint32_t)(x)) << ECAT_FMMU_LOGICAL_START_ADDRESS_PDI_BF0_SHIFT)) & ECAT_FMMU_LOGICAL_START_ADDRESS_PDI_BF0_MASK)
/*! @} */

/* The count of ECAT_FMMU_LOGICAL_START_ADDRESS_PDI */
#define ECAT_FMMU_LOGICAL_START_ADDRESS_PDI_COUNT (8U)

/*! @name FMMU_LENGTH - Register Length FMMU */
/*! @{ */

#define ECAT_FMMU_LENGTH_BF0_MASK                (0xFFFFU)
#define ECAT_FMMU_LENGTH_BF0_SHIFT               (0U)
/*! BF0 - Offset from the first logical FMMU byte to the last FMMU byte + 1 */
#define ECAT_FMMU_LENGTH_BF0(x)                  (((uint16_t)(((uint16_t)(x)) << ECAT_FMMU_LENGTH_BF0_SHIFT)) & ECAT_FMMU_LENGTH_BF0_MASK)
/*! @} */

/* The count of ECAT_FMMU_LENGTH */
#define ECAT_FMMU_LENGTH_COUNT                   (8U)

/*! @name FMMU_LENGTH_PDI - Register Length FMMU */
/*! @{ */

#define ECAT_FMMU_LENGTH_PDI_BF0_MASK            (0xFFFFU)
#define ECAT_FMMU_LENGTH_PDI_BF0_SHIFT           (0U)
/*! BF0 - Offset from the first logical FMMU byte to the last FMMU byte + 1 */
#define ECAT_FMMU_LENGTH_PDI_BF0(x)              (((uint16_t)(((uint16_t)(x)) << ECAT_FMMU_LENGTH_PDI_BF0_SHIFT)) & ECAT_FMMU_LENGTH_PDI_BF0_MASK)
/*! @} */

/* The count of ECAT_FMMU_LENGTH_PDI */
#define ECAT_FMMU_LENGTH_PDI_COUNT               (8U)

/*! @name FMMU_LOGICAL_START_BIT - Register Start bit FMMU y in logical address space */
/*! @{ */

#define ECAT_FMMU_LOGICAL_START_BIT_BF0_MASK     (0x7U)
#define ECAT_FMMU_LOGICAL_START_BIT_BF0_SHIFT    (0U)
/*! BF0 - Logical starting bit that shall be mapped (bits are counted from least significant bit 0 to most significant bit 7) */
#define ECAT_FMMU_LOGICAL_START_BIT_BF0(x)       (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_START_BIT_BF0_SHIFT)) & ECAT_FMMU_LOGICAL_START_BIT_BF0_MASK)

#define ECAT_FMMU_LOGICAL_START_BIT_BF3_MASK     (0xF8U)
#define ECAT_FMMU_LOGICAL_START_BIT_BF3_SHIFT    (3U)
/*! BF3 - Reserved, write 0 */
#define ECAT_FMMU_LOGICAL_START_BIT_BF3(x)       (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_START_BIT_BF3_SHIFT)) & ECAT_FMMU_LOGICAL_START_BIT_BF3_MASK)
/*! @} */

/* The count of ECAT_FMMU_LOGICAL_START_BIT */
#define ECAT_FMMU_LOGICAL_START_BIT_COUNT        (8U)

/*! @name FMMU_LOGICAL_START_BIT_PDI - Register Start bit FMMU y in logical address space */
/*! @{ */

#define ECAT_FMMU_LOGICAL_START_BIT_PDI_BF0_MASK (0x7U)
#define ECAT_FMMU_LOGICAL_START_BIT_PDI_BF0_SHIFT (0U)
/*! BF0 - Logical starting bit that shall be mapped (bits are counted from least significant bit 0 to most significant bit 7) */
#define ECAT_FMMU_LOGICAL_START_BIT_PDI_BF0(x)   (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_START_BIT_PDI_BF0_SHIFT)) & ECAT_FMMU_LOGICAL_START_BIT_PDI_BF0_MASK)

#define ECAT_FMMU_LOGICAL_START_BIT_PDI_BF3_MASK (0xF8U)
#define ECAT_FMMU_LOGICAL_START_BIT_PDI_BF3_SHIFT (3U)
/*! BF3 - Reserved, write 0 */
#define ECAT_FMMU_LOGICAL_START_BIT_PDI_BF3(x)   (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_START_BIT_PDI_BF3_SHIFT)) & ECAT_FMMU_LOGICAL_START_BIT_PDI_BF3_MASK)
/*! @} */

/* The count of ECAT_FMMU_LOGICAL_START_BIT_PDI */
#define ECAT_FMMU_LOGICAL_START_BIT_PDI_COUNT    (8U)

/*! @name FMMU_LOGICAL_STOP_BIT - Register Stop bit FMMU y in logical address space */
/*! @{ */

#define ECAT_FMMU_LOGICAL_STOP_BIT_BF0_MASK      (0x7U)
#define ECAT_FMMU_LOGICAL_STOP_BIT_BF0_SHIFT     (0U)
/*! BF0 - Last logical bit that shall be mapped (bits are counted from least significant bit 0 to most significant bit 7) */
#define ECAT_FMMU_LOGICAL_STOP_BIT_BF0(x)        (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_STOP_BIT_BF0_SHIFT)) & ECAT_FMMU_LOGICAL_STOP_BIT_BF0_MASK)

#define ECAT_FMMU_LOGICAL_STOP_BIT_BF3_MASK      (0xF8U)
#define ECAT_FMMU_LOGICAL_STOP_BIT_BF3_SHIFT     (3U)
/*! BF3 - Reserved, write 0 */
#define ECAT_FMMU_LOGICAL_STOP_BIT_BF3(x)        (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_STOP_BIT_BF3_SHIFT)) & ECAT_FMMU_LOGICAL_STOP_BIT_BF3_MASK)
/*! @} */

/* The count of ECAT_FMMU_LOGICAL_STOP_BIT */
#define ECAT_FMMU_LOGICAL_STOP_BIT_COUNT         (8U)

/*! @name FMMU_LOGICAL_STOP_BIT_PDI - Register Stop bit FMMU y in logical address space */
/*! @{ */

#define ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF0_MASK  (0x7U)
#define ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF0_SHIFT (0U)
/*! BF0 - Last logical bit that shall be mapped (bits are counted from least significant bit 0 to most significant bit 7) */
#define ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF0(x)    (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF0_SHIFT)) & ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF0_MASK)

#define ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF3_MASK  (0xF8U)
#define ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF3_SHIFT (3U)
/*! BF3 - Reserved, write 0 */
#define ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF3(x)    (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF3_SHIFT)) & ECAT_FMMU_LOGICAL_STOP_BIT_PDI_BF3_MASK)
/*! @} */

/* The count of ECAT_FMMU_LOGICAL_STOP_BIT_PDI */
#define ECAT_FMMU_LOGICAL_STOP_BIT_PDI_COUNT     (8U)

/*! @name FMMU_PHYSICAL_START_ADDRESS - Register Physical Start address FMMU */
/*! @{ */

#define ECAT_FMMU_PHYSICAL_START_ADDRESS_BF0_MASK (0xFFFFU)
#define ECAT_FMMU_PHYSICAL_START_ADDRESS_BF0_SHIFT (0U)
/*! BF0 - Physical Start Address (mapped to logical Start address) */
#define ECAT_FMMU_PHYSICAL_START_ADDRESS_BF0(x)  (((uint16_t)(((uint16_t)(x)) << ECAT_FMMU_PHYSICAL_START_ADDRESS_BF0_SHIFT)) & ECAT_FMMU_PHYSICAL_START_ADDRESS_BF0_MASK)
/*! @} */

/* The count of ECAT_FMMU_PHYSICAL_START_ADDRESS */
#define ECAT_FMMU_PHYSICAL_START_ADDRESS_COUNT   (8U)

/*! @name FMMU_PHYSICAL_START_ADDRESS_PDI - Register Physical Start address FMMU */
/*! @{ */

#define ECAT_FMMU_PHYSICAL_START_ADDRESS_PDI_BF0_MASK (0xFFFFU)
#define ECAT_FMMU_PHYSICAL_START_ADDRESS_PDI_BF0_SHIFT (0U)
/*! BF0 - Physical Start Address (mapped to logical Start address) */
#define ECAT_FMMU_PHYSICAL_START_ADDRESS_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_FMMU_PHYSICAL_START_ADDRESS_PDI_BF0_SHIFT)) & ECAT_FMMU_PHYSICAL_START_ADDRESS_PDI_BF0_MASK)
/*! @} */

/* The count of ECAT_FMMU_PHYSICAL_START_ADDRESS_PDI */
#define ECAT_FMMU_PHYSICAL_START_ADDRESS_PDI_COUNT (8U)

/*! @name FMMU_PHYSICAL_START_BIT - Register Physical Start bit FMMU */
/*! @{ */

#define ECAT_FMMU_PHYSICAL_START_BIT_BF0_MASK    (0x7U)
#define ECAT_FMMU_PHYSICAL_START_BIT_BF0_SHIFT   (0U)
/*! BF0 - Physical starting bit as target of logical start bit mapping (bits are counted from least
 *    significant bit 0 to most significant bit 7)
 */
#define ECAT_FMMU_PHYSICAL_START_BIT_BF0(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_PHYSICAL_START_BIT_BF0_SHIFT)) & ECAT_FMMU_PHYSICAL_START_BIT_BF0_MASK)

#define ECAT_FMMU_PHYSICAL_START_BIT_BF3_MASK    (0xF8U)
#define ECAT_FMMU_PHYSICAL_START_BIT_BF3_SHIFT   (3U)
/*! BF3 - Reserved, write 0 */
#define ECAT_FMMU_PHYSICAL_START_BIT_BF3(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_PHYSICAL_START_BIT_BF3_SHIFT)) & ECAT_FMMU_PHYSICAL_START_BIT_BF3_MASK)
/*! @} */

/* The count of ECAT_FMMU_PHYSICAL_START_BIT */
#define ECAT_FMMU_PHYSICAL_START_BIT_COUNT       (8U)

/*! @name FMMU_PHYSICAL_START_BIT_PDI - Register Physical Start bit FMMU */
/*! @{ */

#define ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF0_MASK (0x7U)
#define ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF0_SHIFT (0U)
/*! BF0 - Physical starting bit as target of logical start bit mapping (bits are counted from least
 *    significant bit 0 to most significant bit 7)
 */
#define ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF0(x)  (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF0_SHIFT)) & ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF0_MASK)

#define ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF3_MASK (0xF8U)
#define ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF3_SHIFT (3U)
/*! BF3 - Reserved, write 0 */
#define ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF3(x)  (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF3_SHIFT)) & ECAT_FMMU_PHYSICAL_START_BIT_PDI_BF3_MASK)
/*! @} */

/* The count of ECAT_FMMU_PHYSICAL_START_BIT_PDI */
#define ECAT_FMMU_PHYSICAL_START_BIT_PDI_COUNT   (8U)

/*! @name FMMU_TYPE - Register Type FMMU y */
/*! @{ */

#define ECAT_FMMU_TYPE_BF0_MASK                  (0x1U)
#define ECAT_FMMU_TYPE_BF0_SHIFT                 (0U)
/*! BF0 - Bit field access for ECAT: r/w
 *  0b0..Ignore mapping for read accesses
 *  0b1..Use mapping for read accesses
 */
#define ECAT_FMMU_TYPE_BF0(x)                    (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_TYPE_BF0_SHIFT)) & ECAT_FMMU_TYPE_BF0_MASK)

#define ECAT_FMMU_TYPE_BF1_MASK                  (0x2U)
#define ECAT_FMMU_TYPE_BF1_SHIFT                 (1U)
/*! BF1 - Bit field access for ECAT: r/w
 *  0b0..Ignore mapping for write accesses
 *  0b1..Use mapping for write accesses
 */
#define ECAT_FMMU_TYPE_BF1(x)                    (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_TYPE_BF1_SHIFT)) & ECAT_FMMU_TYPE_BF1_MASK)

#define ECAT_FMMU_TYPE_BF2_MASK                  (0xFCU)
#define ECAT_FMMU_TYPE_BF2_SHIFT                 (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_FMMU_TYPE_BF2(x)                    (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_TYPE_BF2_SHIFT)) & ECAT_FMMU_TYPE_BF2_MASK)
/*! @} */

/* The count of ECAT_FMMU_TYPE */
#define ECAT_FMMU_TYPE_COUNT                     (8U)

/*! @name FMMU_TYPE_PDI - Register Type FMMU y */
/*! @{ */

#define ECAT_FMMU_TYPE_PDI_BF0_MASK              (0x1U)
#define ECAT_FMMU_TYPE_PDI_BF0_SHIFT             (0U)
/*! BF0
 *  0b0..Ignore mapping for read accesses
 *  0b1..Use mapping for read accesses
 */
#define ECAT_FMMU_TYPE_PDI_BF0(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_TYPE_PDI_BF0_SHIFT)) & ECAT_FMMU_TYPE_PDI_BF0_MASK)

#define ECAT_FMMU_TYPE_PDI_BF1_MASK              (0x2U)
#define ECAT_FMMU_TYPE_PDI_BF1_SHIFT             (1U)
/*! BF1
 *  0b0..Ignore mapping for write accesses
 *  0b1..Use mapping for write accesses
 */
#define ECAT_FMMU_TYPE_PDI_BF1(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_TYPE_PDI_BF1_SHIFT)) & ECAT_FMMU_TYPE_PDI_BF1_MASK)

#define ECAT_FMMU_TYPE_PDI_BF2_MASK              (0xFCU)
#define ECAT_FMMU_TYPE_PDI_BF2_SHIFT             (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_FMMU_TYPE_PDI_BF2(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_TYPE_PDI_BF2_SHIFT)) & ECAT_FMMU_TYPE_PDI_BF2_MASK)
/*! @} */

/* The count of ECAT_FMMU_TYPE_PDI */
#define ECAT_FMMU_TYPE_PDI_COUNT                 (8U)

/*! @name FMMU_ACTIVATE - Register Activate FMMU */
/*! @{ */

#define ECAT_FMMU_ACTIVATE_BF0_MASK              (0x1U)
#define ECAT_FMMU_ACTIVATE_BF0_SHIFT             (0U)
/*! BF0 - Bit field access for ECAT: r/w
 *  0b0..FMMU deactivated
 *  0b1..FMMU activated. FMMU checks logically addressed blocks to be mapped according to configured mapping
 */
#define ECAT_FMMU_ACTIVATE_BF0(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_ACTIVATE_BF0_SHIFT)) & ECAT_FMMU_ACTIVATE_BF0_MASK)

#define ECAT_FMMU_ACTIVATE_BF1_MASK              (0xFEU)
#define ECAT_FMMU_ACTIVATE_BF1_SHIFT             (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_FMMU_ACTIVATE_BF1(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_ACTIVATE_BF1_SHIFT)) & ECAT_FMMU_ACTIVATE_BF1_MASK)
/*! @} */

/* The count of ECAT_FMMU_ACTIVATE */
#define ECAT_FMMU_ACTIVATE_COUNT                 (8U)

/*! @name FMMU_ACTIVATE_PDI - Register Activate FMMU */
/*! @{ */

#define ECAT_FMMU_ACTIVATE_PDI_BF0_MASK          (0x1U)
#define ECAT_FMMU_ACTIVATE_PDI_BF0_SHIFT         (0U)
/*! BF0
 *  0b0..FMMU deactivated
 *  0b1..FMMU activated. FMMU checks logically addressed blocks to be mapped according to configured mapping
 */
#define ECAT_FMMU_ACTIVATE_PDI_BF0(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_ACTIVATE_PDI_BF0_SHIFT)) & ECAT_FMMU_ACTIVATE_PDI_BF0_MASK)

#define ECAT_FMMU_ACTIVATE_PDI_BF1_MASK          (0xFEU)
#define ECAT_FMMU_ACTIVATE_PDI_BF1_SHIFT         (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_FMMU_ACTIVATE_PDI_BF1(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_FMMU_ACTIVATE_PDI_BF1_SHIFT)) & ECAT_FMMU_ACTIVATE_PDI_BF1_MASK)
/*! @} */

/* The count of ECAT_FMMU_ACTIVATE_PDI */
#define ECAT_FMMU_ACTIVATE_PDI_COUNT             (8U)

/*! @name SYNCMANAGER_PHYSICAL_START_ADDRESS - Register physical Start Address SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_BF0_MASK (0xFFFFU)
#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_BF0_SHIFT (0U)
/*! BF0 - First byte that will be handled by SyncManager */
#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_BF0_SHIFT)) & ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_BF0_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS */
#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_COUNT (16U)

/*! @name SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI - Register physical Start Address SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI_BF0_MASK (0xFFFFU)
#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI_BF0_SHIFT (0U)
/*! BF0 - First byte that will be handled by SyncManager */
#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI_BF0_SHIFT)) & ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI_BF0_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI */
#define ECAT_SYNCMANAGER_PHYSICAL_START_ADDRESS_PDI_COUNT (16U)

/*! @name SYNCMANAGER_LENGTH - Register Length SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_LENGTH_BF0_MASK         (0xFFFFU)
#define ECAT_SYNCMANAGER_LENGTH_BF0_SHIFT        (0U)
/*! BF0 - Number of bytes assigned to SyncManager (shall be greater than 1 */
#define ECAT_SYNCMANAGER_LENGTH_BF0(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_SYNCMANAGER_LENGTH_BF0_SHIFT)) & ECAT_SYNCMANAGER_LENGTH_BF0_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_LENGTH */
#define ECAT_SYNCMANAGER_LENGTH_COUNT            (16U)

/*! @name SYNCMANAGER_LENGTH_PDI - Register Length SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_LENGTH_PDI_BF0_MASK     (0xFFFFU)
#define ECAT_SYNCMANAGER_LENGTH_PDI_BF0_SHIFT    (0U)
/*! BF0 - Number of bytes assigned to SyncManager (shall be greater than 1 */
#define ECAT_SYNCMANAGER_LENGTH_PDI_BF0(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_SYNCMANAGER_LENGTH_PDI_BF0_SHIFT)) & ECAT_SYNCMANAGER_LENGTH_PDI_BF0_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_LENGTH_PDI */
#define ECAT_SYNCMANAGER_LENGTH_PDI_COUNT        (16U)

/*! @name SYNCMANAGER_CONTROL_REGISTER - Register Control Register SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF0_MASK (0x3U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF0_SHIFT (0U)
/*! BF0 - Operation Mode:
 *  0b00..Buffered (3 buffer mode)
 *  0b01..Reserved
 *  0b10..Mailbox (Single buffer mode)
 *  0b11..Reserved
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_BF0_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_BF0_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF2_MASK (0xCU)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF2_SHIFT (2U)
/*! BF2 - Direction:
 *  0b00..Read: ECAT read access, PDI write access.
 *  0b01..Write: ECAT write access, PDI read access.
 *  0b10..Reserved
 *  0b11..Reserved
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF2(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_BF2_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_BF2_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF4_MASK (0x10U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF4_SHIFT (4U)
/*! BF4 - Interrupt in ECAT Event Request Register:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF4(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_BF4_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_BF4_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF5_MASK (0x20U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF5_SHIFT (5U)
/*! BF5 - Interrupt in AL Event Request Register:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF5(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_BF5_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_BF5_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF6_MASK (0x40U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF6_SHIFT (6U)
/*! BF6 - Watchdog Trigger Enable:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF6(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_BF6_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_BF6_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF7_MASK (0x80U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF7_SHIFT (7U)
/*! BF7 - Reserved, write 0 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_BF7(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_BF7_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_BF7_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_CONTROL_REGISTER */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_COUNT  (16U)

/*! @name SYNCMANAGER_CONTROL_REGISTER_PDI - Register Control Register SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF0_MASK (0x3U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF0_SHIFT (0U)
/*! BF0 - Operation Mode:
 *  0b00..Buffered (3 buffer mode)
 *  0b01..Reserved
 *  0b10..Mailbox (Single buffer mode)
 *  0b11..Reserved
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF0_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF0_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF2_MASK (0xCU)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF2_SHIFT (2U)
/*! BF2 - Direction:
 *  0b00..Read: ECAT read access, PDI write access.
 *  0b01..Write: ECAT write access, PDI read access.
 *  0b10..Reserved
 *  0b11..Reserved
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF2(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF2_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF2_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF4_MASK (0x10U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF4_SHIFT (4U)
/*! BF4 - Interrupt in ECAT Event Request Register:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF4(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF4_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF4_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF5_MASK (0x20U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF5_SHIFT (5U)
/*! BF5 - Interrupt in AL Event Request Register:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF5(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF5_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF5_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF6_MASK (0x40U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF6_SHIFT (6U)
/*! BF6 - Watchdog Trigger Enable:
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF6(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF6_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF6_MASK)

#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF7_MASK (0x80U)
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF7_SHIFT (7U)
/*! BF7 - Reserved, write 0 */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF7(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF7_SHIFT)) & ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_BF7_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI */
#define ECAT_SYNCMANAGER_CONTROL_REGISTER_PDI_COUNT (16U)

/*! @name SYNCMANAGER_STATUS - Register Status Register SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_STATUS_BF0_MASK         (0x1U)
#define ECAT_SYNCMANAGER_STATUS_BF0_SHIFT        (0U)
/*! BF0 - Interrupt Write:
 *  0b1..Interrupt after buffer was completely and successfully written
 *  0b0..Interrupt cleared after first byte of buffer was read
 */
#define ECAT_SYNCMANAGER_STATUS_BF0(x)           (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_STATUS_BF0_SHIFT)) & ECAT_SYNCMANAGER_STATUS_BF0_MASK)

#define ECAT_SYNCMANAGER_STATUS_BF1_MASK         (0x2U)
#define ECAT_SYNCMANAGER_STATUS_BF1_SHIFT        (1U)
/*! BF1 - Interrupt Read:
 *  0b1..Interrupt after buffer was completely and successfully read
 *  0b0..Interrupt cleared after first byte of buffer was written
 */
#define ECAT_SYNCMANAGER_STATUS_BF1(x)           (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_STATUS_BF1_SHIFT)) & ECAT_SYNCMANAGER_STATUS_BF1_MASK)

#define ECAT_SYNCMANAGER_STATUS_BF2_MASK         (0x4U)
#define ECAT_SYNCMANAGER_STATUS_BF2_SHIFT        (2U)
/*! BF2 - Reserved */
#define ECAT_SYNCMANAGER_STATUS_BF2(x)           (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_STATUS_BF2_SHIFT)) & ECAT_SYNCMANAGER_STATUS_BF2_MASK)

#define ECAT_SYNCMANAGER_STATUS_BF3_MASK         (0x8U)
#define ECAT_SYNCMANAGER_STATUS_BF3_SHIFT        (3U)
/*! BF3 - Mailbox mode: mailbox status:
 *  0b0..Mailbox empty
 *  0b1..Mailbox full
 */
#define ECAT_SYNCMANAGER_STATUS_BF3(x)           (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_STATUS_BF3_SHIFT)) & ECAT_SYNCMANAGER_STATUS_BF3_MASK)

#define ECAT_SYNCMANAGER_STATUS_BF4_MASK         (0x30U)
#define ECAT_SYNCMANAGER_STATUS_BF4_SHIFT        (4U)
/*! BF4 - Buffered mode: buffer status (last written buffer):
 *  0b00..1st buffer
 *  0b01..2nd buffer
 *  0b10..3rd buffer
 *  0b11..(no buffer written)
 */
#define ECAT_SYNCMANAGER_STATUS_BF4(x)           (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_STATUS_BF4_SHIFT)) & ECAT_SYNCMANAGER_STATUS_BF4_MASK)

#define ECAT_SYNCMANAGER_STATUS_BF6_MASK         (0x40U)
#define ECAT_SYNCMANAGER_STATUS_BF6_SHIFT        (6U)
/*! BF6 - Read buffer in use (opened) */
#define ECAT_SYNCMANAGER_STATUS_BF6(x)           (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_STATUS_BF6_SHIFT)) & ECAT_SYNCMANAGER_STATUS_BF6_MASK)

#define ECAT_SYNCMANAGER_STATUS_BF7_MASK         (0x80U)
#define ECAT_SYNCMANAGER_STATUS_BF7_SHIFT        (7U)
/*! BF7 - Write buffer in use (opened) */
#define ECAT_SYNCMANAGER_STATUS_BF7(x)           (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_STATUS_BF7_SHIFT)) & ECAT_SYNCMANAGER_STATUS_BF7_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_STATUS */
#define ECAT_SYNCMANAGER_STATUS_COUNT            (16U)

/*! @name SYNCMANAGER_ACTIVATE - Register Activate SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_ACTIVATE_BF0_MASK       (0x1U)
#define ECAT_SYNCMANAGER_ACTIVATE_BF0_SHIFT      (0U)
/*! BF0 - SyncManager Enable/Disable:
 *  0b0..Disable: Access to Memory without SyncManager control
 *  0b1..Enable: SyncManager is active and controls Memory area set in configuration
 */
#define ECAT_SYNCMANAGER_ACTIVATE_BF0(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_BF0_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_BF0_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_BF1_MASK       (0x2U)
#define ECAT_SYNCMANAGER_ACTIVATE_BF1_SHIFT      (1U)
/*! BF1 - Repeat Request: */
#define ECAT_SYNCMANAGER_ACTIVATE_BF1(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_BF1_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_BF1_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_BF2_MASK       (0x3CU)
#define ECAT_SYNCMANAGER_ACTIVATE_BF2_SHIFT      (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_SYNCMANAGER_ACTIVATE_BF2(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_BF2_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_BF2_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_BF6_MASK       (0x40U)
#define ECAT_SYNCMANAGER_ACTIVATE_BF6_SHIFT      (6U)
/*! BF6 - Latch Event ECAT:
 *  0b0..No
 *  0b1..Generate Latch event when EtherCAT master issues a buffer exchange
 */
#define ECAT_SYNCMANAGER_ACTIVATE_BF6(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_BF6_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_BF6_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_BF7_MASK       (0x80U)
#define ECAT_SYNCMANAGER_ACTIVATE_BF7_SHIFT      (7U)
/*! BF7 - Latch Event PDI:
 *  0b0..No
 *  0b1..Generate Latch events when PDI issues a buffer exchange or when PDI accesses buffer start address
 */
#define ECAT_SYNCMANAGER_ACTIVATE_BF7(x)         (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_BF7_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_BF7_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_ACTIVATE */
#define ECAT_SYNCMANAGER_ACTIVATE_COUNT          (16U)

/*! @name SYNCMANAGER_ACTIVATE_PDI - Register Activate SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF0_MASK   (0x1U)
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF0_SHIFT  (0U)
/*! BF0 - SyncManager Enable/Disable:
 *  0b0..Disable: Access to Memory without SyncManager control
 *  0b1..Enable: SyncManager is active and controls Memory area set in configuration
 */
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF0(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_PDI_BF0_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_PDI_BF0_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF1_MASK   (0x2U)
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF1_SHIFT  (1U)
/*! BF1 - Repeat Request: */
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF1(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_PDI_BF1_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_PDI_BF1_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF2_MASK   (0x3CU)
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF2_SHIFT  (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF2(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_PDI_BF2_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_PDI_BF2_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF6_MASK   (0x40U)
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF6_SHIFT  (6U)
/*! BF6 - Latch Event ECAT:
 *  0b0..No
 *  0b1..Generate Latch event when EtherCAT master issues a buffer exchange
 */
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF6(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_PDI_BF6_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_PDI_BF6_MASK)

#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF7_MASK   (0x80U)
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF7_SHIFT  (7U)
/*! BF7 - Latch Event PDI:
 *  0b0..No
 *  0b1..Generate Latch events when PDI issues a buffer exchange or when PDI accesses buffer start address
 */
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_BF7(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_ACTIVATE_PDI_BF7_SHIFT)) & ECAT_SYNCMANAGER_ACTIVATE_PDI_BF7_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_ACTIVATE_PDI */
#define ECAT_SYNCMANAGER_ACTIVATE_PDI_COUNT      (16U)

/*! @name SYNCMANAGER_PDI_CONTROL - Register PDI Control SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_PDI_CONTROL_BF0_MASK    (0x1U)
#define ECAT_SYNCMANAGER_PDI_CONTROL_BF0_SHIFT   (0U)
/*! BF0 - Deactivate SyncManager: */
#define ECAT_SYNCMANAGER_PDI_CONTROL_BF0(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_PDI_CONTROL_BF0_SHIFT)) & ECAT_SYNCMANAGER_PDI_CONTROL_BF0_MASK)

#define ECAT_SYNCMANAGER_PDI_CONTROL_BF1_MASK    (0x2U)
#define ECAT_SYNCMANAGER_PDI_CONTROL_BF1_SHIFT   (1U)
/*! BF1 - Repeat Ack: */
#define ECAT_SYNCMANAGER_PDI_CONTROL_BF1(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_PDI_CONTROL_BF1_SHIFT)) & ECAT_SYNCMANAGER_PDI_CONTROL_BF1_MASK)

#define ECAT_SYNCMANAGER_PDI_CONTROL_BF2_MASK    (0xFCU)
#define ECAT_SYNCMANAGER_PDI_CONTROL_BF2_SHIFT   (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_SYNCMANAGER_PDI_CONTROL_BF2(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_PDI_CONTROL_BF2_SHIFT)) & ECAT_SYNCMANAGER_PDI_CONTROL_BF2_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_PDI_CONTROL */
#define ECAT_SYNCMANAGER_PDI_CONTROL_COUNT       (16U)

/*! @name SYNCMANAGER_PDI_CONTROL_PDI - Register PDI Control SyncManager */
/*! @{ */

#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF0_MASK (0x1U)
#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF0_SHIFT (0U)
/*! BF0 - Deactivate SyncManager: */
#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF0(x)  (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF0_SHIFT)) & ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF0_MASK)

#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF1_MASK (0x2U)
#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF1_SHIFT (1U)
/*! BF1 - Repeat Ack: */
#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF1(x)  (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF1_SHIFT)) & ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF1_MASK)

#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF2_MASK (0xFCU)
#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF2_SHIFT (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF2(x)  (((uint8_t)(((uint8_t)(x)) << ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF2_SHIFT)) & ECAT_SYNCMANAGER_PDI_CONTROL_PDI_BF2_MASK)
/*! @} */

/* The count of ECAT_SYNCMANAGER_PDI_CONTROL_PDI */
#define ECAT_SYNCMANAGER_PDI_CONTROL_PDI_COUNT   (16U)

/*! @name RECEIVE_TIMES - Distributed Clocks Receive Times */
/*! @{ */

#define ECAT_RECEIVE_TIMES_BF0_MASK              (0xFFU)
#define ECAT_RECEIVE_TIMES_BF0_SHIFT             (0U)
/*! BF0 - Write */
#define ECAT_RECEIVE_TIMES_BF0(x)                (((uint32_t)(((uint32_t)(x)) << ECAT_RECEIVE_TIMES_BF0_SHIFT)) & ECAT_RECEIVE_TIMES_BF0_MASK)

#define ECAT_RECEIVE_TIMES_BF8_MASK              (0xFFFFFF00U)
#define ECAT_RECEIVE_TIMES_BF8_SHIFT             (8U)
/*! BF8 - Local time at the beginning of the last receive frame containing a write access to register 0x0900. */
#define ECAT_RECEIVE_TIMES_BF8(x)                (((uint32_t)(((uint32_t)(x)) << ECAT_RECEIVE_TIMES_BF8_SHIFT)) & ECAT_RECEIVE_TIMES_BF8_MASK)
/*! @} */

/*! @name RECEIVE_TIMES_PDI - Distributed Clocks Receive Times */
/*! @{ */

#define ECAT_RECEIVE_TIMES_PDI_BF0_MASK          (0xFFU)
#define ECAT_RECEIVE_TIMES_PDI_BF0_SHIFT         (0U)
/*! BF0 - Write */
#define ECAT_RECEIVE_TIMES_PDI_BF0(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_RECEIVE_TIMES_PDI_BF0_SHIFT)) & ECAT_RECEIVE_TIMES_PDI_BF0_MASK)

#define ECAT_RECEIVE_TIMES_PDI_BF8_MASK          (0xFFFFFF00U)
#define ECAT_RECEIVE_TIMES_PDI_BF8_SHIFT         (8U)
/*! BF8 - Local time at the beginning of the last receive frame containing a write access to register 0x0900. */
#define ECAT_RECEIVE_TIMES_PDI_BF8(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_RECEIVE_TIMES_PDI_BF8_SHIFT)) & ECAT_RECEIVE_TIMES_PDI_BF8_MASK)
/*! @} */

/*! @name RECEIVE_TIME_PORT_1 - Distributed Clocks Receive Time Port 1 */
/*! @{ */

#define ECAT_RECEIVE_TIME_PORT_1_BF0_MASK        (0xFFFFFFFFU)
#define ECAT_RECEIVE_TIME_PORT_1_BF0_SHIFT       (0U)
/*! BF0 - Local time at the beginning of a frame (start first bit of preamble) received at port 1
 *    containing a BWR or FPWR to register 0x0900.
 */
#define ECAT_RECEIVE_TIME_PORT_1_BF0(x)          (((uint32_t)(((uint32_t)(x)) << ECAT_RECEIVE_TIME_PORT_1_BF0_SHIFT)) & ECAT_RECEIVE_TIME_PORT_1_BF0_MASK)
/*! @} */

/*! @name SYSTEM_TIME - Register System Time */
/*! @{ */

#define ECAT_SYSTEM_TIME_BF0ECAT_MASK            (0xFFFFFFFFFFFFFFFFU)
#define ECAT_SYSTEM_TIME_BF0ECAT_SHIFT           (0U)
/*! BF0ECAT - ECAT read access */
#define ECAT_SYSTEM_TIME_BF0ECAT(x)              (((uint64_t)(((uint64_t)(x)) << ECAT_SYSTEM_TIME_BF0ECAT_SHIFT)) & ECAT_SYSTEM_TIME_BF0ECAT_MASK)
/*! @} */

/*! @name SYSTEM_TIME_PDI - Register System Time */
/*! @{ */

#define ECAT_SYSTEM_TIME_PDI_BF0ECAT_MASK        (0xFFFFFFFFFFFFFFFFU)
#define ECAT_SYSTEM_TIME_PDI_BF0ECAT_SHIFT       (0U)
/*! BF0ECAT - PDI read access */
#define ECAT_SYSTEM_TIME_PDI_BF0ECAT(x)          (((uint64_t)(((uint64_t)(x)) << ECAT_SYSTEM_TIME_PDI_BF0ECAT_SHIFT)) & ECAT_SYSTEM_TIME_PDI_BF0ECAT_MASK)
/*! @} */

/*! @name RECEIVE_TIME_ECAT_PROCESSING_UNIT - Distributed Clocks Register Receive Time ECAT Processing Unit */
/*! @{ */

#define ECAT_RECEIVE_TIME_ECAT_PROCESSING_UNIT_BF0_MASK (0xFFFFFFFFFFFFFFFFU)
#define ECAT_RECEIVE_TIME_ECAT_PROCESSING_UNIT_BF0_SHIFT (0U)
/*! BF0 - Local time at the beginning of a frame (start first bit of preamble) received at the ECAT
 *    Processing Unit containing a write access to register 0x0900
 */
#define ECAT_RECEIVE_TIME_ECAT_PROCESSING_UNIT_BF0(x) (((uint64_t)(((uint64_t)(x)) << ECAT_RECEIVE_TIME_ECAT_PROCESSING_UNIT_BF0_SHIFT)) & ECAT_RECEIVE_TIME_ECAT_PROCESSING_UNIT_BF0_MASK)
/*! @} */

/*! @name SYSTEM_TIME_OFFSET - Register System Time Offset */
/*! @{ */

#define ECAT_SYSTEM_TIME_OFFSET_BF0_MASK         (0xFFFFFFFFFFFFFFFFU)
#define ECAT_SYSTEM_TIME_OFFSET_BF0_SHIFT        (0U)
/*! BF0 - Difference between local time and System Time. Offset is added to the local time. */
#define ECAT_SYSTEM_TIME_OFFSET_BF0(x)           (((uint64_t)(((uint64_t)(x)) << ECAT_SYSTEM_TIME_OFFSET_BF0_SHIFT)) & ECAT_SYSTEM_TIME_OFFSET_BF0_MASK)
/*! @} */

/*! @name SYSTEM_TIME_DELAY - Register System Time Delay */
/*! @{ */

#define ECAT_SYSTEM_TIME_DELAY_BF0_MASK          (0xFFFFFFFFU)
#define ECAT_SYSTEM_TIME_DELAY_BF0_SHIFT         (0U)
/*! BF0 - Delay between Reference Clock and the eCAT */
#define ECAT_SYSTEM_TIME_DELAY_BF0(x)            (((uint32_t)(((uint32_t)(x)) << ECAT_SYSTEM_TIME_DELAY_BF0_SHIFT)) & ECAT_SYSTEM_TIME_DELAY_BF0_MASK)
/*! @} */

/*! @name SYSTEM_TIME_DIFFERENCE - Register System Time Difference */
/*! @{ */

#define ECAT_SYSTEM_TIME_DIFFERENCE_BF0_MASK     (0x7FFFFFFFU)
#define ECAT_SYSTEM_TIME_DIFFERENCE_BF0_SHIFT    (0U)
/*! BF0 - Mean difference between local copy of system Time and received System Time values */
#define ECAT_SYSTEM_TIME_DIFFERENCE_BF0(x)       (((uint32_t)(((uint32_t)(x)) << ECAT_SYSTEM_TIME_DIFFERENCE_BF0_SHIFT)) & ECAT_SYSTEM_TIME_DIFFERENCE_BF0_MASK)

#define ECAT_SYSTEM_TIME_DIFFERENCE_BF31_MASK    (0x80000000U)
#define ECAT_SYSTEM_TIME_DIFFERENCE_BF31_SHIFT   (31U)
/*! BF31 - Bit field access for ECAT: r/-
 *  0b0..Local copy of System Time less than received System Time
 *  0b1..Local copy of System Time greater than or equal to received System Time
 */
#define ECAT_SYSTEM_TIME_DIFFERENCE_BF31(x)      (((uint32_t)(((uint32_t)(x)) << ECAT_SYSTEM_TIME_DIFFERENCE_BF31_SHIFT)) & ECAT_SYSTEM_TIME_DIFFERENCE_BF31_MASK)
/*! @} */

/*! @name SPEED_COUNTER_START - Register Speed Counter Start */
/*! @{ */

#define ECAT_SPEED_COUNTER_START_BF0_MASK        (0x7FFFU)
#define ECAT_SPEED_COUNTER_START_BF0_SHIFT       (0U)
/*! BF0 - Bandwidth for adjustment of local copy of system Time (larger values -> smaller bandwidth and smoother adjustment) */
#define ECAT_SPEED_COUNTER_START_BF0(x)          (((uint16_t)(((uint16_t)(x)) << ECAT_SPEED_COUNTER_START_BF0_SHIFT)) & ECAT_SPEED_COUNTER_START_BF0_MASK)

#define ECAT_SPEED_COUNTER_START_BF15_MASK       (0x8000U)
#define ECAT_SPEED_COUNTER_START_BF15_SHIFT      (15U)
/*! BF15 - Reserved, write 0 */
#define ECAT_SPEED_COUNTER_START_BF15(x)         (((uint16_t)(((uint16_t)(x)) << ECAT_SPEED_COUNTER_START_BF15_SHIFT)) & ECAT_SPEED_COUNTER_START_BF15_MASK)
/*! @} */

/*! @name SPEED_COUNTER_DIFF - Register Speed Counter Diff */
/*! @{ */

#define ECAT_SPEED_COUNTER_DIFF_BF0_MASK         (0xFFFFU)
#define ECAT_SPEED_COUNTER_DIFF_BF0_SHIFT        (0U)
/*! BF0 - Representation of the deviation between local clock period and Reference Clock's clock
 *    period (representation: two's complement)
 */
#define ECAT_SPEED_COUNTER_DIFF_BF0(x)           (((uint16_t)(((uint16_t)(x)) << ECAT_SPEED_COUNTER_DIFF_BF0_SHIFT)) & ECAT_SPEED_COUNTER_DIFF_BF0_MASK)
/*! @} */

/*! @name SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH - Register System Time Difference Filter Depth */
/*! @{ */

#define ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF0_MASK (0xFU)
#define ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF0_SHIFT (0U)
/*! BF0 - Filter depth for averaging the received System Time deviation */
#define ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF0(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF0_SHIFT)) & ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF0_MASK)

#define ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF4_MASK (0xF0U)
#define ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF4_SHIFT (4U)
/*! BF4 - Reserved, write 0 */
#define ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF4(x) (((uint8_t)(((uint8_t)(x)) << ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF4_SHIFT)) & ECAT_SYSTEM_TIME_DIFFERENCE_FILTER_DEPTH_BF4_MASK)
/*! @} */

/*! @name SPEED_COUNTER_FILTER_DEPTH - Register Speed Counter Filter Depth */
/*! @{ */

#define ECAT_SPEED_COUNTER_FILTER_DEPTH_BF0_MASK (0xFU)
#define ECAT_SPEED_COUNTER_FILTER_DEPTH_BF0_SHIFT (0U)
/*! BF0 - Filter depth for averaging the clock period deviation */
#define ECAT_SPEED_COUNTER_FILTER_DEPTH_BF0(x)   (((uint8_t)(((uint8_t)(x)) << ECAT_SPEED_COUNTER_FILTER_DEPTH_BF0_SHIFT)) & ECAT_SPEED_COUNTER_FILTER_DEPTH_BF0_MASK)

#define ECAT_SPEED_COUNTER_FILTER_DEPTH_BF4_MASK (0xF0U)
#define ECAT_SPEED_COUNTER_FILTER_DEPTH_BF4_SHIFT (4U)
/*! BF4 - Reserved, write 0 */
#define ECAT_SPEED_COUNTER_FILTER_DEPTH_BF4(x)   (((uint8_t)(((uint8_t)(x)) << ECAT_SPEED_COUNTER_FILTER_DEPTH_BF4_SHIFT)) & ECAT_SPEED_COUNTER_FILTER_DEPTH_BF4_MASK)
/*! @} */

/*! @name CYCLIC_UNIT_CONTROL - Register Cyclic Unit Control */
/*! @{ */

#define ECAT_CYCLIC_UNIT_CONTROL_BF0_MASK        (0x1U)
#define ECAT_CYCLIC_UNIT_CONTROL_BF0_SHIFT       (0U)
/*! BF0 - SYNC out unit control:
 *  0b0..ECAT-controlled
 *  0b1..PDI-controlled
 */
#define ECAT_CYCLIC_UNIT_CONTROL_BF0(x)          (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_BF0_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_BF0_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_BF1_MASK        (0xEU)
#define ECAT_CYCLIC_UNIT_CONTROL_BF1_SHIFT       (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_CYCLIC_UNIT_CONTROL_BF1(x)          (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_BF1_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_BF1_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_BF4_MASK        (0x10U)
#define ECAT_CYCLIC_UNIT_CONTROL_BF4_SHIFT       (4U)
/*! BF4 - Latch In unit 0:
 *  0b0..ECAT-controlled
 *  0b1..PDI-controlled
 */
#define ECAT_CYCLIC_UNIT_CONTROL_BF4(x)          (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_BF4_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_BF4_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_BF5_MASK        (0x20U)
#define ECAT_CYCLIC_UNIT_CONTROL_BF5_SHIFT       (5U)
/*! BF5 - Latch In unit 1:
 *  0b0..ECAT-controlled
 *  0b1..PDI-controlled
 */
#define ECAT_CYCLIC_UNIT_CONTROL_BF5(x)          (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_BF5_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_BF5_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_BF6_MASK        (0xC0U)
#define ECAT_CYCLIC_UNIT_CONTROL_BF6_SHIFT       (6U)
/*! BF6 - Reserved, write 0 */
#define ECAT_CYCLIC_UNIT_CONTROL_BF6(x)          (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_BF6_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_BF6_MASK)
/*! @} */

/*! @name CYCLIC_UNIT_CONTROL_PDI - Register Cyclic Unit Control */
/*! @{ */

#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF0_MASK    (0x1U)
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF0_SHIFT   (0U)
/*! BF0 - SYNC out unit control:
 *  0b0..ECAT-controlled
 *  0b1..PDI-controlled
 */
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF0(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_PDI_BF0_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_PDI_BF0_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF1_MASK    (0xEU)
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF1_SHIFT   (1U)
/*! BF1 - Reserved, write 0 */
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF1(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_PDI_BF1_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_PDI_BF1_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF4_MASK    (0x10U)
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF4_SHIFT   (4U)
/*! BF4 - Latch In unit 0:
 *  0b0..ECAT-controlled
 *  0b1..PDI-controlled
 */
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF4(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_PDI_BF4_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_PDI_BF4_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF5_MASK    (0x20U)
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF5_SHIFT   (5U)
/*! BF5 - Latch In unit 1:
 *  0b0..ECAT-controlled
 *  0b1..PDI-controlled
 */
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF5(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_PDI_BF5_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_PDI_BF5_MASK)

#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF6_MASK    (0xC0U)
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF6_SHIFT   (6U)
/*! BF6 - Reserved, write 0 */
#define ECAT_CYCLIC_UNIT_CONTROL_PDI_BF6(x)      (((uint8_t)(((uint8_t)(x)) << ECAT_CYCLIC_UNIT_CONTROL_PDI_BF6_SHIFT)) & ECAT_CYCLIC_UNIT_CONTROL_PDI_BF6_MASK)
/*! @} */

/*! @name UNIT_ACTIVATION_REGISTER - Register Activation register */
/*! @{ */

#define ECAT_UNIT_ACTIVATION_REGISTER_BF0_MASK   (0x1U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF0_SHIFT  (0U)
/*! BF0 - Sync Out Unit activation:
 *  0b0..Deactivated
 *  0b1..Activated
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF0(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF0_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF0_MASK)

#define ECAT_UNIT_ACTIVATION_REGISTER_BF1_MASK   (0x2U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF1_SHIFT  (1U)
/*! BF1 - SYNC0 generation:
 *  0b0..Deactivated
 *  0b1..SYNC0 pulse is generated
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF1(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF1_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF1_MASK)

#define ECAT_UNIT_ACTIVATION_REGISTER_BF2_MASK   (0x4U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF2_SHIFT  (2U)
/*! BF2 - SYNC1 generation:
 *  0b0..Deactivated
 *  0b1..SYNC1 pulse is generated
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF2(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF2_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF2_MASK)

#define ECAT_UNIT_ACTIVATION_REGISTER_BF3_MASK   (0x8U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF3_SHIFT  (3U)
/*! BF3 - Auto-activation by writing Start Time Cyclic Operation (0x0990:0x0997):
 *  0b0..Disabled
 *  0b1..Auto-activation enabled. 0x0981[0] is set automatically after Start Time is written
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF3(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF3_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF3_MASK)

#define ECAT_UNIT_ACTIVATION_REGISTER_BF4_MASK   (0x10U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF4_SHIFT  (4U)
/*! BF4 - Extension of Start Time Cyclic Operation (0x0990:0x0993):
 *  0b0..No extension
 *  0b1..Extend 32 bit written Start Time to 64 bit
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF4(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF4_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF4_MASK)

#define ECAT_UNIT_ACTIVATION_REGISTER_BF5_MASK   (0x20U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF5_SHIFT  (5U)
/*! BF5 - Start Time plausibility check:
 *  0b0..Disabled. SyncSignal generation if Start Time is reached.
 *  0b1..Immediate SyncSignal generation if Start Time is outside near future (see 0x0981[6])
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF5(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF5_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF5_MASK)

#define ECAT_UNIT_ACTIVATION_REGISTER_BF6_MASK   (0x40U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF6_SHIFT  (6U)
/*! BF6 - Near future configuration (approx.):
 *  0b0..1/2 DC width future (2^31ns or 2^63ns)
 *  0b1..~2.1 sec. future (2^31ns)
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF6(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF6_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF6_MASK)

#define ECAT_UNIT_ACTIVATION_REGISTER_BF7_MASK   (0x80U)
#define ECAT_UNIT_ACTIVATION_REGISTER_BF7_SHIFT  (7U)
/*! BF7 - SyncSignal debug pulse (Vasily bit):
 *  0b0..Deactivated
 *  0b1..Immediately generate one ping only on SYNC0-1 according to 0x0981[2:1] for debugging.
 */
#define ECAT_UNIT_ACTIVATION_REGISTER_BF7(x)     (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_REGISTER_BF7_SHIFT)) & ECAT_UNIT_ACTIVATION_REGISTER_BF7_MASK)
/*! @} */

/*! @name UNI_PULSE_LENGTH_OF_SYNCSIGNALS - Register Pulse Length of SyncSignals */
/*! @{ */

#define ECAT_UNI_PULSE_LENGTH_OF_SYNCSIGNALS_BF0_MASK (0xFFFFU)
#define ECAT_UNI_PULSE_LENGTH_OF_SYNCSIGNALS_BF0_SHIFT (0U)
/*! BF0 - Pulse length of SyncSignals (in Units of 10ns)
 *  0b0000000000000000..Acknowledge mode: SyncSignal will be cleared by reading SYNC[1:0] Status register
 */
#define ECAT_UNI_PULSE_LENGTH_OF_SYNCSIGNALS_BF0(x) (((uint16_t)(((uint16_t)(x)) << ECAT_UNI_PULSE_LENGTH_OF_SYNCSIGNALS_BF0_SHIFT)) & ECAT_UNI_PULSE_LENGTH_OF_SYNCSIGNALS_BF0_MASK)
/*! @} */

/*! @name UNIT_ACTIVATION_STATUS - Register Activation Status */
/*! @{ */

#define ECAT_UNIT_ACTIVATION_STATUS_BF0_MASK     (0x1U)
#define ECAT_UNIT_ACTIVATION_STATUS_BF0_SHIFT    (0U)
/*! BF0 - SYNC0 activation state:
 *  0b0..First SYNC0 pulse is not pending
 *  0b1..First SYNC0 pulse is pending
 */
#define ECAT_UNIT_ACTIVATION_STATUS_BF0(x)       (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_STATUS_BF0_SHIFT)) & ECAT_UNIT_ACTIVATION_STATUS_BF0_MASK)

#define ECAT_UNIT_ACTIVATION_STATUS_BF1_MASK     (0x2U)
#define ECAT_UNIT_ACTIVATION_STATUS_BF1_SHIFT    (1U)
/*! BF1 - SYNC1 activation state:
 *  0b0..First SYNC1 pulse is not pending
 *  0b1..First SYNC1 pulse is pending
 */
#define ECAT_UNIT_ACTIVATION_STATUS_BF1(x)       (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_STATUS_BF1_SHIFT)) & ECAT_UNIT_ACTIVATION_STATUS_BF1_MASK)

#define ECAT_UNIT_ACTIVATION_STATUS_BF2_MASK     (0x4U)
#define ECAT_UNIT_ACTIVATION_STATUS_BF2_SHIFT    (2U)
/*! BF2 - Start Time Cyclic Operation (0x0990:0x0997) plausibility check result when Sync Out Unit was activated:
 *  0b0..Start Time was within near future
 *  0b1..Start Time was out of near future (0x0981[6])
 */
#define ECAT_UNIT_ACTIVATION_STATUS_BF2(x)       (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_STATUS_BF2_SHIFT)) & ECAT_UNIT_ACTIVATION_STATUS_BF2_MASK)

#define ECAT_UNIT_ACTIVATION_STATUS_BF3_MASK     (0xF8U)
#define ECAT_UNIT_ACTIVATION_STATUS_BF3_SHIFT    (3U)
/*! BF3 - Reserved */
#define ECAT_UNIT_ACTIVATION_STATUS_BF3(x)       (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_ACTIVATION_STATUS_BF3_SHIFT)) & ECAT_UNIT_ACTIVATION_STATUS_BF3_MASK)
/*! @} */

/*! @name UNIT_SYNC0_STATUS - Register SYNC0 Status */
/*! @{ */

#define ECAT_UNIT_SYNC0_STATUS_BF0_MASK          (0x1U)
#define ECAT_UNIT_SYNC0_STATUS_BF0_SHIFT         (0U)
/*! BF0 - SYNC0 state for Acknowledge mode. */
#define ECAT_UNIT_SYNC0_STATUS_BF0(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_SYNC0_STATUS_BF0_SHIFT)) & ECAT_UNIT_SYNC0_STATUS_BF0_MASK)

#define ECAT_UNIT_SYNC0_STATUS_BF1_MASK          (0xFEU)
#define ECAT_UNIT_SYNC0_STATUS_BF1_SHIFT         (1U)
/*! BF1 - Reserved */
#define ECAT_UNIT_SYNC0_STATUS_BF1(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_SYNC0_STATUS_BF1_SHIFT)) & ECAT_UNIT_SYNC0_STATUS_BF1_MASK)
/*! @} */

/*! @name UNIT_SYNC1_STATUS - Register SYNC1 Status */
/*! @{ */

#define ECAT_UNIT_SYNC1_STATUS_BF0_MASK          (0x1U)
#define ECAT_UNIT_SYNC1_STATUS_BF0_SHIFT         (0U)
/*! BF0 - SYNC1 state for Acknowledge mode. */
#define ECAT_UNIT_SYNC1_STATUS_BF0(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_SYNC1_STATUS_BF0_SHIFT)) & ECAT_UNIT_SYNC1_STATUS_BF0_MASK)

#define ECAT_UNIT_SYNC1_STATUS_BF1_MASK          (0xFEU)
#define ECAT_UNIT_SYNC1_STATUS_BF1_SHIFT         (1U)
/*! BF1 - Reserved */
#define ECAT_UNIT_SYNC1_STATUS_BF1(x)            (((uint8_t)(((uint8_t)(x)) << ECAT_UNIT_SYNC1_STATUS_BF1_SHIFT)) & ECAT_UNIT_SYNC1_STATUS_BF1_MASK)
/*! @} */

/*! @name UNIT_START_TIME_CYCLIC_OPERATION - Register Start Time Cyclic Operation */
/*! @{ */

#define ECAT_UNIT_START_TIME_CYCLIC_OPERATION_BF0_MASK (0xFFFFFFFFFFFFFFFFU)
#define ECAT_UNIT_START_TIME_CYCLIC_OPERATION_BF0_SHIFT (0U)
/*! BF0 - Write: Start time (System time) of cyclic operation in ns */
#define ECAT_UNIT_START_TIME_CYCLIC_OPERATION_BF0(x) (((uint64_t)(((uint64_t)(x)) << ECAT_UNIT_START_TIME_CYCLIC_OPERATION_BF0_SHIFT)) & ECAT_UNIT_START_TIME_CYCLIC_OPERATION_BF0_MASK)
/*! @} */

/*! @name UNIT_NEXT_SYNC1_PULSE - Register Next SYNC1 Pulse */
/*! @{ */

#define ECAT_UNIT_NEXT_SYNC1_PULSE_BF0_MASK      (0xFFFFFFFFFFFFFFFFU)
#define ECAT_UNIT_NEXT_SYNC1_PULSE_BF0_SHIFT     (0U)
/*! BF0 - System time of next SYNC1 pulse in ns */
#define ECAT_UNIT_NEXT_SYNC1_PULSE_BF0(x)        (((uint64_t)(((uint64_t)(x)) << ECAT_UNIT_NEXT_SYNC1_PULSE_BF0_SHIFT)) & ECAT_UNIT_NEXT_SYNC1_PULSE_BF0_MASK)
/*! @} */

/*! @name UNIT_SYNC0_CYCLE_TIME - Register SYNC0 Cycle Time */
/*! @{ */

#define ECAT_UNIT_SYNC0_CYCLE_TIME_BF0_MASK      (0xFFFFFFFFU)
#define ECAT_UNIT_SYNC0_CYCLE_TIME_BF0_SHIFT     (0U)
/*! BF0 - Time between two consecutive SYNC0 pulses in ns.
 *  0b00000000000000000000000000000000..Single shot mode, generate only one SYNC0 pulse.
 */
#define ECAT_UNIT_SYNC0_CYCLE_TIME_BF0(x)        (((uint32_t)(((uint32_t)(x)) << ECAT_UNIT_SYNC0_CYCLE_TIME_BF0_SHIFT)) & ECAT_UNIT_SYNC0_CYCLE_TIME_BF0_MASK)
/*! @} */

/*! @name UNIT_SYNC1_CYCLE_TIME - Register SYNC1 Cycle Time */
/*! @{ */

#define ECAT_UNIT_SYNC1_CYCLE_TIME_BF0_MASK      (0xFFFFFFFFU)
#define ECAT_UNIT_SYNC1_CYCLE_TIME_BF0_SHIFT     (0U)
/*! BF0 - Time between SYNC0 pulse and SYNC1 pulse in ns */
#define ECAT_UNIT_SYNC1_CYCLE_TIME_BF0(x)        (((uint32_t)(((uint32_t)(x)) << ECAT_UNIT_SYNC1_CYCLE_TIME_BF0_SHIFT)) & ECAT_UNIT_SYNC1_CYCLE_TIME_BF0_MASK)
/*! @} */

/*! @name LATCH0_CONTROL - Register Latch0 Control */
/*! @{ */

#define ECAT_LATCH0_CONTROL_BF0_MASK             (0x1U)
#define ECAT_LATCH0_CONTROL_BF0_SHIFT            (0U)
/*! BF0 - Latch0 positive edge:
 *  0b0..Continuous Latch active
 *  0b1..Single event (only first event active)
 */
#define ECAT_LATCH0_CONTROL_BF0(x)               (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH0_CONTROL_BF0_SHIFT)) & ECAT_LATCH0_CONTROL_BF0_MASK)

#define ECAT_LATCH0_CONTROL_BF1_MASK             (0x2U)
#define ECAT_LATCH0_CONTROL_BF1_SHIFT            (1U)
/*! BF1 - Latch0 negative edge:
 *  0b0..Continuous Latch active
 *  0b1..Single event (only first event active)
 */
#define ECAT_LATCH0_CONTROL_BF1(x)               (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH0_CONTROL_BF1_SHIFT)) & ECAT_LATCH0_CONTROL_BF1_MASK)

#define ECAT_LATCH0_CONTROL_BF2_MASK             (0xFCU)
#define ECAT_LATCH0_CONTROL_BF2_SHIFT            (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_LATCH0_CONTROL_BF2(x)               (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH0_CONTROL_BF2_SHIFT)) & ECAT_LATCH0_CONTROL_BF2_MASK)
/*! @} */

/*! @name LATCH1_CONTROL - Register Latch1 Control */
/*! @{ */

#define ECAT_LATCH1_CONTROL_BF0_MASK             (0x1U)
#define ECAT_LATCH1_CONTROL_BF0_SHIFT            (0U)
/*! BF0 - Latch1 positive edge:
 *  0b0..Continuous Latch active
 *  0b1..Single event (only first event active)
 */
#define ECAT_LATCH1_CONTROL_BF0(x)               (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH1_CONTROL_BF0_SHIFT)) & ECAT_LATCH1_CONTROL_BF0_MASK)

#define ECAT_LATCH1_CONTROL_BF1_MASK             (0x2U)
#define ECAT_LATCH1_CONTROL_BF1_SHIFT            (1U)
/*! BF1 - Latch1 negative edge:
 *  0b0..Continuous Latch active
 *  0b1..Single event (only first event active)
 */
#define ECAT_LATCH1_CONTROL_BF1(x)               (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH1_CONTROL_BF1_SHIFT)) & ECAT_LATCH1_CONTROL_BF1_MASK)

#define ECAT_LATCH1_CONTROL_BF2_MASK             (0xFCU)
#define ECAT_LATCH1_CONTROL_BF2_SHIFT            (2U)
/*! BF2 - Reserved, write 0 */
#define ECAT_LATCH1_CONTROL_BF2(x)               (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH1_CONTROL_BF2_SHIFT)) & ECAT_LATCH1_CONTROL_BF2_MASK)
/*! @} */

/*! @name LATCH0_STATUS - Register Latch0 Status */
/*! @{ */

#define ECAT_LATCH0_STATUS_BF0_MASK              (0x1U)
#define ECAT_LATCH0_STATUS_BF0_SHIFT             (0U)
/*! BF0 - Event Latch0 positive edge.
 *  0b0..Positive edge not detected or continuous mode
 *  0b1..Positive edge detected in single event mode only.
 */
#define ECAT_LATCH0_STATUS_BF0(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH0_STATUS_BF0_SHIFT)) & ECAT_LATCH0_STATUS_BF0_MASK)

#define ECAT_LATCH0_STATUS_BF1_MASK              (0x2U)
#define ECAT_LATCH0_STATUS_BF1_SHIFT             (1U)
/*! BF1 - Event Latch0 negative edge.
 *  0b0..Negative edge not detected or continuous mode
 *  0b1..Negative edge detected in single event mode only
 */
#define ECAT_LATCH0_STATUS_BF1(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH0_STATUS_BF1_SHIFT)) & ECAT_LATCH0_STATUS_BF1_MASK)

#define ECAT_LATCH0_STATUS_BF2_MASK              (0x4U)
#define ECAT_LATCH0_STATUS_BF2_SHIFT             (2U)
/*! BF2 - Latch0 pin state */
#define ECAT_LATCH0_STATUS_BF2(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH0_STATUS_BF2_SHIFT)) & ECAT_LATCH0_STATUS_BF2_MASK)

#define ECAT_LATCH0_STATUS_BF3_MASK              (0xF8U)
#define ECAT_LATCH0_STATUS_BF3_SHIFT             (3U)
/*! BF3 - Reserved */
#define ECAT_LATCH0_STATUS_BF3(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH0_STATUS_BF3_SHIFT)) & ECAT_LATCH0_STATUS_BF3_MASK)
/*! @} */

/*! @name LATCH1_STATUS - Register Latch1 Status */
/*! @{ */

#define ECAT_LATCH1_STATUS_BF0_MASK              (0x1U)
#define ECAT_LATCH1_STATUS_BF0_SHIFT             (0U)
/*! BF0 - Event Latch1 positive edge.
 *  0b0..Positive edge not detected or continuous mode
 *  0b1..Positive edge detected in single event mode only.
 */
#define ECAT_LATCH1_STATUS_BF0(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH1_STATUS_BF0_SHIFT)) & ECAT_LATCH1_STATUS_BF0_MASK)

#define ECAT_LATCH1_STATUS_BF1_MASK              (0x2U)
#define ECAT_LATCH1_STATUS_BF1_SHIFT             (1U)
/*! BF1 - Event Latch1 negative edge.
 *  0b0..Negative edge not detected or continuous mode
 *  0b1..Negative edge detected in single event mode only.
 */
#define ECAT_LATCH1_STATUS_BF1(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH1_STATUS_BF1_SHIFT)) & ECAT_LATCH1_STATUS_BF1_MASK)

#define ECAT_LATCH1_STATUS_BF2_MASK              (0x4U)
#define ECAT_LATCH1_STATUS_BF2_SHIFT             (2U)
/*! BF2 - Latch1 pin state */
#define ECAT_LATCH1_STATUS_BF2(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH1_STATUS_BF2_SHIFT)) & ECAT_LATCH1_STATUS_BF2_MASK)

#define ECAT_LATCH1_STATUS_BF3_MASK              (0xF8U)
#define ECAT_LATCH1_STATUS_BF3_SHIFT             (3U)
/*! BF3 - Reserved */
#define ECAT_LATCH1_STATUS_BF3(x)                (((uint8_t)(((uint8_t)(x)) << ECAT_LATCH1_STATUS_BF3_SHIFT)) & ECAT_LATCH1_STATUS_BF3_MASK)
/*! @} */

/*! @name LATCH0_TIME_POSITIVE_EDGE - Register Latch0 Time Positive Edge */
/*! @{ */

#define ECAT_LATCH0_TIME_POSITIVE_EDGE_BF0_MASK  (0xFFFFFFFFFFFFFFFFU)
#define ECAT_LATCH0_TIME_POSITIVE_EDGE_BF0_SHIFT (0U)
/*! BF0 - System time at the positive edge of the Latch0 signal. */
#define ECAT_LATCH0_TIME_POSITIVE_EDGE_BF0(x)    (((uint64_t)(((uint64_t)(x)) << ECAT_LATCH0_TIME_POSITIVE_EDGE_BF0_SHIFT)) & ECAT_LATCH0_TIME_POSITIVE_EDGE_BF0_MASK)
/*! @} */

/*! @name LATCH0_TIME_NEGATIVE_EDGE - Register Latch0 Time Negative Edge */
/*! @{ */

#define ECAT_LATCH0_TIME_NEGATIVE_EDGE_BF0_MASK  (0xFFFFFFFFFFFFFFFFU)
#define ECAT_LATCH0_TIME_NEGATIVE_EDGE_BF0_SHIFT (0U)
/*! BF0 - System time at the negative edge of the Latch0 signal. */
#define ECAT_LATCH0_TIME_NEGATIVE_EDGE_BF0(x)    (((uint64_t)(((uint64_t)(x)) << ECAT_LATCH0_TIME_NEGATIVE_EDGE_BF0_SHIFT)) & ECAT_LATCH0_TIME_NEGATIVE_EDGE_BF0_MASK)
/*! @} */

/*! @name LATCH1_TIME_POSITIVE_EDGE - Register Latch1 Time Positive Edge */
/*! @{ */

#define ECAT_LATCH1_TIME_POSITIVE_EDGE_BF0_MASK  (0xFFFFFFFFFFFFFFFFU)
#define ECAT_LATCH1_TIME_POSITIVE_EDGE_BF0_SHIFT (0U)
/*! BF0 - System time at the positive edge of the Latch1 signal. */
#define ECAT_LATCH1_TIME_POSITIVE_EDGE_BF0(x)    (((uint64_t)(((uint64_t)(x)) << ECAT_LATCH1_TIME_POSITIVE_EDGE_BF0_SHIFT)) & ECAT_LATCH1_TIME_POSITIVE_EDGE_BF0_MASK)
/*! @} */

/*! @name LATCH1_TIME_NEGATIVE_EDGE - Register Latch1 Time Negative Edge */
/*! @{ */

#define ECAT_LATCH1_TIME_NEGATIVE_EDGE_BF0_MASK  (0xFFFFFFFFFFFFFFFFU)
#define ECAT_LATCH1_TIME_NEGATIVE_EDGE_BF0_SHIFT (0U)
/*! BF0 - System time at the negative edge of the Latch1 signal. */
#define ECAT_LATCH1_TIME_NEGATIVE_EDGE_BF0(x)    (((uint64_t)(((uint64_t)(x)) << ECAT_LATCH1_TIME_NEGATIVE_EDGE_BF0_SHIFT)) & ECAT_LATCH1_TIME_NEGATIVE_EDGE_BF0_MASK)
/*! @} */

/*! @name ETHERCAT_BUFFER_CHANGE_EVENT_TIME - Register EtherCAT Buffer Change Event Time */
/*! @{ */

#define ECAT_ETHERCAT_BUFFER_CHANGE_EVENT_TIME_BF0_MASK (0xFFFFFFFFU)
#define ECAT_ETHERCAT_BUFFER_CHANGE_EVENT_TIME_BF0_SHIFT (0U)
/*! BF0 - Local time at the beginning of the frame which causes at least one SyncManager to assert an ECAT event */
#define ECAT_ETHERCAT_BUFFER_CHANGE_EVENT_TIME_BF0(x) (((uint32_t)(((uint32_t)(x)) << ECAT_ETHERCAT_BUFFER_CHANGE_EVENT_TIME_BF0_SHIFT)) & ECAT_ETHERCAT_BUFFER_CHANGE_EVENT_TIME_BF0_MASK)
/*! @} */

/*! @name PDI_BUFFER_START_EVENT_TIME - Register PDI Buffer Start Event Time */
/*! @{ */

#define ECAT_PDI_BUFFER_START_EVENT_TIME_BF0_MASK (0xFFFFFFFFU)
#define ECAT_PDI_BUFFER_START_EVENT_TIME_BF0_SHIFT (0U)
/*! BF0 - Local time when at least one SyncManager asserts a PDI buffer start event */
#define ECAT_PDI_BUFFER_START_EVENT_TIME_BF0(x)  (((uint32_t)(((uint32_t)(x)) << ECAT_PDI_BUFFER_START_EVENT_TIME_BF0_SHIFT)) & ECAT_PDI_BUFFER_START_EVENT_TIME_BF0_MASK)
/*! @} */

/*! @name PDI_BUFFER_CHANGE_EVENT_TIME - Register PDI Buffer Change Event Time */
/*! @{ */

#define ECAT_PDI_BUFFER_CHANGE_EVENT_TIME_BF0_MASK (0xFFFFFFFFU)
#define ECAT_PDI_BUFFER_CHANGE_EVENT_TIME_BF0_SHIFT (0U)
/*! BF0 - Local time when at least one SyncManager asserts a PDI buffer change event */
#define ECAT_PDI_BUFFER_CHANGE_EVENT_TIME_BF0(x) (((uint32_t)(((uint32_t)(x)) << ECAT_PDI_BUFFER_CHANGE_EVENT_TIME_BF0_SHIFT)) & ECAT_PDI_BUFFER_CHANGE_EVENT_TIME_BF0_MASK)
/*! @} */

/*! @name PRODUCT_ID_IP_CORE - Register Product ID IP Core */
/*! @{ */

#define ECAT_PRODUCT_ID_IP_CORE_BF0_MASK         (0xFFFFFFFFFFFFFFFFU)
#define ECAT_PRODUCT_ID_IP_CORE_BF0_SHIFT        (0U)
/*! BF0 - Product ID */
#define ECAT_PRODUCT_ID_IP_CORE_BF0(x)           (((uint64_t)(((uint64_t)(x)) << ECAT_PRODUCT_ID_IP_CORE_BF0_SHIFT)) & ECAT_PRODUCT_ID_IP_CORE_BF0_MASK)
/*! @} */

/*! @name VENDOR_ID_IP_CORE - Register Vendor ID IP Core */
/*! @{ */

#define ECAT_VENDOR_ID_IP_CORE_BF0_MASK          (0xFFFFFFFFU)
#define ECAT_VENDOR_ID_IP_CORE_BF0_SHIFT         (0U)
/*! BF0 - Vendor ID */
#define ECAT_VENDOR_ID_IP_CORE_BF0(x)            (((uint64_t)(((uint64_t)(x)) << ECAT_VENDOR_ID_IP_CORE_BF0_SHIFT)) & ECAT_VENDOR_ID_IP_CORE_BF0_MASK)

#define ECAT_VENDOR_ID_IP_CORE_BF32_MASK         (0xFFFFFFFF00000000U)
#define ECAT_VENDOR_ID_IP_CORE_BF32_SHIFT        (32U)
/*! BF32 - Reserved */
#define ECAT_VENDOR_ID_IP_CORE_BF32(x)           (((uint64_t)(((uint64_t)(x)) << ECAT_VENDOR_ID_IP_CORE_BF32_SHIFT)) & ECAT_VENDOR_ID_IP_CORE_BF32_MASK)
/*! @} */

/*! @name GENERAL_PURPOSE_OUTPUTS - Register General Purpose Outputs */
/*! @{ */

#define ECAT_GENERAL_PURPOSE_OUTPUTS_BF0_MASK    (0xFFFFU)
#define ECAT_GENERAL_PURPOSE_OUTPUTS_BF0_SHIFT   (0U)
/*! BF0 - General Purpose Output Data */
#define ECAT_GENERAL_PURPOSE_OUTPUTS_BF0(x)      (((uint16_t)(((uint16_t)(x)) << ECAT_GENERAL_PURPOSE_OUTPUTS_BF0_SHIFT)) & ECAT_GENERAL_PURPOSE_OUTPUTS_BF0_MASK)
/*! @} */

/*! @name GENERAL_PURPOSE_INPUTS - Register General Purpose Inputs */
/*! @{ */

#define ECAT_GENERAL_PURPOSE_INPUTS_BF0_MASK     (0xFFFFU)
#define ECAT_GENERAL_PURPOSE_INPUTS_BF0_SHIFT    (0U)
/*! BF0 - General Purpose Input Data */
#define ECAT_GENERAL_PURPOSE_INPUTS_BF0(x)       (((uint16_t)(((uint16_t)(x)) << ECAT_GENERAL_PURPOSE_INPUTS_BF0_SHIFT)) & ECAT_GENERAL_PURPOSE_INPUTS_BF0_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ECAT_Register_Masks */


/* ECAT - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ECAT base address */
  #define ECAT_BASE                                (0x52A80000u)
  /** Peripheral ECAT base address */
  #define ECAT_BASE_NS                             (0x42A80000u)
  /** Peripheral ECAT base pointer */
  #define ECAT                                     ((ECAT_Type *)ECAT_BASE)
  /** Peripheral ECAT base pointer */
  #define ECAT_NS                                  ((ECAT_Type *)ECAT_BASE_NS)
  /** Array initializer of ECAT peripheral base addresses */
  #define ECAT_BASE_ADDRS                          { ECAT_BASE }
  /** Array initializer of ECAT peripheral base pointers */
  #define ECAT_BASE_PTRS                           { ECAT }
  /** Array initializer of ECAT peripheral base addresses */
  #define ECAT_BASE_ADDRS_NS                       { ECAT_BASE_NS }
  /** Array initializer of ECAT peripheral base pointers */
  #define ECAT_BASE_PTRS_NS                        { ECAT_NS }
#else
  /** Peripheral ECAT base address */
  #define ECAT_BASE                                (0x42A80000u)
  /** Peripheral ECAT base pointer */
  #define ECAT                                     ((ECAT_Type *)ECAT_BASE)
  /** Array initializer of ECAT peripheral base addresses */
  #define ECAT_BASE_ADDRS                          { ECAT_BASE }
  /** Array initializer of ECAT peripheral base pointers */
  #define ECAT_BASE_PTRS                           { ECAT }
#endif

/*!
 * @}
 */ /* end of group ECAT_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- EIM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup EIM_Peripheral_Access_Layer EIM Peripheral Access Layer
 * @{
 */

/** EIM - Register Layout Typedef */
typedef struct {
  __IO uint32_t EIMCR;                             /**< Error Injection Module Configuration Register, offset: 0x0 */
  __IO uint32_t EICHEN;                            /**< Error Injection Channel Enable register, offset: 0x4 */
       uint8_t RESERVED_0[248];
  __IO uint32_t EICHD0_WORD0;                      /**< Error Injection Channel Descriptor 0, Word0, offset: 0x100 */
  __IO uint32_t EICHD0_WORD1;                      /**< Error Injection Channel Descriptor 0, Word1, offset: 0x104 */
  __IO uint32_t EICHD0_WORD2;                      /**< Error Injection Channel Descriptor 0, Word2, offset: 0x108 */
       uint8_t RESERVED_1[52];
  __IO uint32_t EICHD1_WORD0;                      /**< Error Injection Channel Descriptor 1, Word0, offset: 0x140 */
  __IO uint32_t EICHD1_WORD1;                      /**< Error Injection Channel Descriptor 1, Word1, offset: 0x144 */
  __IO uint32_t EICHD1_WORD2;                      /**< Error Injection Channel Descriptor 1, Word2, offset: 0x148 */
  __IO uint32_t EICHD1_WORD3;                      /**< Error Injection Channel Descriptor 1, Word3, offset: 0x14C */
  __IO uint32_t EICHD1_WORD4;                      /**< Error Injection Channel Descriptor 1, Word4, offset: 0x150 */
       uint8_t RESERVED_2[44];
  __IO uint32_t EICHD2_WORD0;                      /**< Error Injection Channel Descriptor 2, Word0, offset: 0x180 */
  __IO uint32_t EICHD2_WORD1;                      /**< Error Injection Channel Descriptor 2, Word1, offset: 0x184 */
  __IO uint32_t EICHD2_WORD2;                      /**< Error Injection Channel Descriptor 2, Word2, offset: 0x188 */
  __IO uint32_t EICHD2_WORD3;                      /**< Error Injection Channel Descriptor 2, Word3, offset: 0x18C */
  __IO uint32_t EICHD2_WORD4;                      /**< Error Injection Channel Descriptor 2, Word4, offset: 0x190 */
       uint8_t RESERVED_3[44];
  __IO uint32_t EICHD3_WORD0;                      /**< Error Injection Channel Descriptor 3, Word0, offset: 0x1C0 */
  __IO uint32_t EICHD3_WORD1;                      /**< Error Injection Channel Descriptor 3, Word1, offset: 0x1C4 */
  __IO uint32_t EICHD3_WORD2;                      /**< Error Injection Channel Descriptor 3, Word2, offset: 0x1C8 */
  __IO uint32_t EICHD3_WORD3;                      /**< Error Injection Channel Descriptor 3, Word3, offset: 0x1CC */
  __IO uint32_t EICHD3_WORD4;                      /**< Error Injection Channel Descriptor 3, Word4, offset: 0x1D0 */
       uint8_t RESERVED_4[44];
  __IO uint32_t EICHD4_WORD0;                      /**< Error Injection Channel Descriptor 4, Word0, offset: 0x200 */
  __IO uint32_t EICHD4_WORD1;                      /**< Error Injection Channel Descriptor 4, Word1, offset: 0x204 */
  __IO uint32_t EICHD4_WORD2;                      /**< Error Injection Channel Descriptor 4, Word2, offset: 0x208 */
  __IO uint32_t EICHD4_WORD3;                      /**< Error Injection Channel Descriptor 4, Word3, offset: 0x20C */
  __IO uint32_t EICHD4_WORD4;                      /**< Error Injection Channel Descriptor 4, Word4, offset: 0x210 */
} EIM_Type;

/* ----------------------------------------------------------------------------
   -- EIM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup EIM_Register_Masks EIM Register Masks
 * @{
 */

/*! @name EIMCR - Error Injection Module Configuration Register */
/*! @{ */

#define EIM_EIMCR_GEIEN_MASK                     (0x1U)
#define EIM_EIMCR_GEIEN_SHIFT                    (0U)
/*! GEIEN - Global Error Injection Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EIM_EIMCR_GEIEN(x)                       (((uint32_t)(((uint32_t)(x)) << EIM_EIMCR_GEIEN_SHIFT)) & EIM_EIMCR_GEIEN_MASK)
/*! @} */

/*! @name EICHEN - Error Injection Channel Enable register */
/*! @{ */

#define EIM_EICHEN_EICH4EN_MASK                  (0x8000000U)
#define EIM_EICHEN_EICH4EN_SHIFT                 (27U)
/*! EICH4EN - Error Injection Channel 4 Enable
 *  0b0..Error injection is disabled on Error Injection Channel 4
 *  0b1..Error injection is enabled on Error Injection Channel 4
 */
#define EIM_EICHEN_EICH4EN(x)                    (((uint32_t)(((uint32_t)(x)) << EIM_EICHEN_EICH4EN_SHIFT)) & EIM_EICHEN_EICH4EN_MASK)

#define EIM_EICHEN_EICH3EN_MASK                  (0x10000000U)
#define EIM_EICHEN_EICH3EN_SHIFT                 (28U)
/*! EICH3EN - Error Injection Channel 3 Enable
 *  0b0..Error injection is disabled on Error Injection Channel 3
 *  0b1..Error injection is enabled on Error Injection Channel 3
 */
#define EIM_EICHEN_EICH3EN(x)                    (((uint32_t)(((uint32_t)(x)) << EIM_EICHEN_EICH3EN_SHIFT)) & EIM_EICHEN_EICH3EN_MASK)

#define EIM_EICHEN_EICH2EN_MASK                  (0x20000000U)
#define EIM_EICHEN_EICH2EN_SHIFT                 (29U)
/*! EICH2EN - Error Injection Channel 2 Enable
 *  0b0..Error injection is disabled on Error Injection Channel 2
 *  0b1..Error injection is enabled on Error Injection Channel 2
 */
#define EIM_EICHEN_EICH2EN(x)                    (((uint32_t)(((uint32_t)(x)) << EIM_EICHEN_EICH2EN_SHIFT)) & EIM_EICHEN_EICH2EN_MASK)

#define EIM_EICHEN_EICH1EN_MASK                  (0x40000000U)
#define EIM_EICHEN_EICH1EN_SHIFT                 (30U)
/*! EICH1EN - Error Injection Channel 1 Enable
 *  0b0..Error injection is disabled on Error Injection Channel 1
 *  0b1..Error injection is enabled on Error Injection Channel 1
 */
#define EIM_EICHEN_EICH1EN(x)                    (((uint32_t)(((uint32_t)(x)) << EIM_EICHEN_EICH1EN_SHIFT)) & EIM_EICHEN_EICH1EN_MASK)

#define EIM_EICHEN_EICH0EN_MASK                  (0x80000000U)
#define EIM_EICHEN_EICH0EN_SHIFT                 (31U)
/*! EICH0EN - Error Injection Channel 0 Enable
 *  0b0..Error injection is disabled on Error Injection Channel 0
 *  0b1..Error injection is enabled on Error Injection Channel 0
 */
#define EIM_EICHEN_EICH0EN(x)                    (((uint32_t)(((uint32_t)(x)) << EIM_EICHEN_EICH0EN_SHIFT)) & EIM_EICHEN_EICH0EN_MASK)
/*! @} */

/*! @name EICHD0_WORD0 - Error Injection Channel Descriptor 0, Word0 */
/*! @{ */

#define EIM_EICHD0_WORD0_CHKBIT_MASK_MASK        (0xFFFC0000U)
#define EIM_EICHD0_WORD0_CHKBIT_MASK_SHIFT       (18U)
/*! CHKBIT_MASK - Checkbit Mask */
#define EIM_EICHD0_WORD0_CHKBIT_MASK(x)          (((uint32_t)(((uint32_t)(x)) << EIM_EICHD0_WORD0_CHKBIT_MASK_SHIFT)) & EIM_EICHD0_WORD0_CHKBIT_MASK_MASK)
/*! @} */

/*! @name EICHD0_WORD1 - Error Injection Channel Descriptor 0, Word1 */
/*! @{ */

#define EIM_EICHD0_WORD1_B0_3DATA_MASK_MASK      (0xFFFU)
#define EIM_EICHD0_WORD1_B0_3DATA_MASK_SHIFT     (0U)
/*! B0_3DATA_MASK - Data Mask Bytes 0-3 */
#define EIM_EICHD0_WORD1_B0_3DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD0_WORD1_B0_3DATA_MASK_SHIFT)) & EIM_EICHD0_WORD1_B0_3DATA_MASK_MASK)
/*! @} */

/*! @name EICHD0_WORD2 - Error Injection Channel Descriptor 0, Word2 */
/*! @{ */

#define EIM_EICHD0_WORD2_B4_7DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD0_WORD2_B4_7DATA_MASK_SHIFT     (0U)
/*! B4_7DATA_MASK - Data Mask Bytes 4-7 */
#define EIM_EICHD0_WORD2_B4_7DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD0_WORD2_B4_7DATA_MASK_SHIFT)) & EIM_EICHD0_WORD2_B4_7DATA_MASK_MASK)
/*! @} */

/*! @name EICHD1_WORD0 - Error Injection Channel Descriptor 1, Word0 */
/*! @{ */

#define EIM_EICHD1_WORD0_CHKBIT_MASK_MASK        (0xFFFF0000U)
#define EIM_EICHD1_WORD0_CHKBIT_MASK_SHIFT       (16U)
/*! CHKBIT_MASK - Checkbit Mask */
#define EIM_EICHD1_WORD0_CHKBIT_MASK(x)          (((uint32_t)(((uint32_t)(x)) << EIM_EICHD1_WORD0_CHKBIT_MASK_SHIFT)) & EIM_EICHD1_WORD0_CHKBIT_MASK_MASK)
/*! @} */

/*! @name EICHD1_WORD1 - Error Injection Channel Descriptor 1, Word1 */
/*! @{ */

#define EIM_EICHD1_WORD1_B0_3DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD1_WORD1_B0_3DATA_MASK_SHIFT     (0U)
/*! B0_3DATA_MASK - Data Mask Bytes 0-3 */
#define EIM_EICHD1_WORD1_B0_3DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD1_WORD1_B0_3DATA_MASK_SHIFT)) & EIM_EICHD1_WORD1_B0_3DATA_MASK_MASK)
/*! @} */

/*! @name EICHD1_WORD2 - Error Injection Channel Descriptor 1, Word2 */
/*! @{ */

#define EIM_EICHD1_WORD2_B4_7DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD1_WORD2_B4_7DATA_MASK_SHIFT     (0U)
/*! B4_7DATA_MASK - Data Mask Bytes 4-7 */
#define EIM_EICHD1_WORD2_B4_7DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD1_WORD2_B4_7DATA_MASK_SHIFT)) & EIM_EICHD1_WORD2_B4_7DATA_MASK_MASK)
/*! @} */

/*! @name EICHD1_WORD3 - Error Injection Channel Descriptor 1, Word3 */
/*! @{ */

#define EIM_EICHD1_WORD3_B8_11DATA_MASK_MASK     (0xFFFFFFFFU)
#define EIM_EICHD1_WORD3_B8_11DATA_MASK_SHIFT    (0U)
/*! B8_11DATA_MASK - Data Mask Bytes 8-11 */
#define EIM_EICHD1_WORD3_B8_11DATA_MASK(x)       (((uint32_t)(((uint32_t)(x)) << EIM_EICHD1_WORD3_B8_11DATA_MASK_SHIFT)) & EIM_EICHD1_WORD3_B8_11DATA_MASK_MASK)
/*! @} */

/*! @name EICHD1_WORD4 - Error Injection Channel Descriptor 1, Word4 */
/*! @{ */

#define EIM_EICHD1_WORD4_B12_15DATA_MASK_MASK    (0xFFFFFFFFU)
#define EIM_EICHD1_WORD4_B12_15DATA_MASK_SHIFT   (0U)
/*! B12_15DATA_MASK - Data Mask Bytes 12-15 */
#define EIM_EICHD1_WORD4_B12_15DATA_MASK(x)      (((uint32_t)(((uint32_t)(x)) << EIM_EICHD1_WORD4_B12_15DATA_MASK_SHIFT)) & EIM_EICHD1_WORD4_B12_15DATA_MASK_MASK)
/*! @} */

/*! @name EICHD2_WORD0 - Error Injection Channel Descriptor 2, Word0 */
/*! @{ */

#define EIM_EICHD2_WORD0_CHKBIT_MASK_MASK        (0xFFFFFFF0U)
#define EIM_EICHD2_WORD0_CHKBIT_MASK_SHIFT       (4U)
/*! CHKBIT_MASK - Checkbit Mask */
#define EIM_EICHD2_WORD0_CHKBIT_MASK(x)          (((uint32_t)(((uint32_t)(x)) << EIM_EICHD2_WORD0_CHKBIT_MASK_SHIFT)) & EIM_EICHD2_WORD0_CHKBIT_MASK_MASK)
/*! @} */

/*! @name EICHD2_WORD1 - Error Injection Channel Descriptor 2, Word1 */
/*! @{ */

#define EIM_EICHD2_WORD1_B0_3DATA_MASK_MASK      (0xFFU)
#define EIM_EICHD2_WORD1_B0_3DATA_MASK_SHIFT     (0U)
/*! B0_3DATA_MASK - Data Mask Bytes 0-3 */
#define EIM_EICHD2_WORD1_B0_3DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD2_WORD1_B0_3DATA_MASK_SHIFT)) & EIM_EICHD2_WORD1_B0_3DATA_MASK_MASK)
/*! @} */

/*! @name EICHD2_WORD2 - Error Injection Channel Descriptor 2, Word2 */
/*! @{ */

#define EIM_EICHD2_WORD2_B4_7DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD2_WORD2_B4_7DATA_MASK_SHIFT     (0U)
/*! B4_7DATA_MASK - Data Mask Bytes 4-7 */
#define EIM_EICHD2_WORD2_B4_7DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD2_WORD2_B4_7DATA_MASK_SHIFT)) & EIM_EICHD2_WORD2_B4_7DATA_MASK_MASK)
/*! @} */

/*! @name EICHD2_WORD3 - Error Injection Channel Descriptor 2, Word3 */
/*! @{ */

#define EIM_EICHD2_WORD3_B8_11DATA_MASK_MASK     (0xFFFFFFFFU)
#define EIM_EICHD2_WORD3_B8_11DATA_MASK_SHIFT    (0U)
/*! B8_11DATA_MASK - Data Mask Bytes 8-11 */
#define EIM_EICHD2_WORD3_B8_11DATA_MASK(x)       (((uint32_t)(((uint32_t)(x)) << EIM_EICHD2_WORD3_B8_11DATA_MASK_SHIFT)) & EIM_EICHD2_WORD3_B8_11DATA_MASK_MASK)
/*! @} */

/*! @name EICHD2_WORD4 - Error Injection Channel Descriptor 2, Word4 */
/*! @{ */

#define EIM_EICHD2_WORD4_B12_15DATA_MASK_MASK    (0xFFFFFFFFU)
#define EIM_EICHD2_WORD4_B12_15DATA_MASK_SHIFT   (0U)
/*! B12_15DATA_MASK - Data Mask Bytes 12-15 */
#define EIM_EICHD2_WORD4_B12_15DATA_MASK(x)      (((uint32_t)(((uint32_t)(x)) << EIM_EICHD2_WORD4_B12_15DATA_MASK_SHIFT)) & EIM_EICHD2_WORD4_B12_15DATA_MASK_MASK)
/*! @} */

/*! @name EICHD3_WORD0 - Error Injection Channel Descriptor 3, Word0 */
/*! @{ */

#define EIM_EICHD3_WORD0_CHKBIT_MASK_MASK        (0xFFFFFFF0U)
#define EIM_EICHD3_WORD0_CHKBIT_MASK_SHIFT       (4U)
/*! CHKBIT_MASK - Checkbit Mask */
#define EIM_EICHD3_WORD0_CHKBIT_MASK(x)          (((uint32_t)(((uint32_t)(x)) << EIM_EICHD3_WORD0_CHKBIT_MASK_SHIFT)) & EIM_EICHD3_WORD0_CHKBIT_MASK_MASK)
/*! @} */

/*! @name EICHD3_WORD1 - Error Injection Channel Descriptor 3, Word1 */
/*! @{ */

#define EIM_EICHD3_WORD1_B0_3DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD3_WORD1_B0_3DATA_MASK_SHIFT     (0U)
/*! B0_3DATA_MASK - Data Mask Bytes 0-3 */
#define EIM_EICHD3_WORD1_B0_3DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD3_WORD1_B0_3DATA_MASK_SHIFT)) & EIM_EICHD3_WORD1_B0_3DATA_MASK_MASK)
/*! @} */

/*! @name EICHD3_WORD2 - Error Injection Channel Descriptor 3, Word2 */
/*! @{ */

#define EIM_EICHD3_WORD2_B4_7DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD3_WORD2_B4_7DATA_MASK_SHIFT     (0U)
/*! B4_7DATA_MASK - Data Mask Bytes 4-7 */
#define EIM_EICHD3_WORD2_B4_7DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD3_WORD2_B4_7DATA_MASK_SHIFT)) & EIM_EICHD3_WORD2_B4_7DATA_MASK_MASK)
/*! @} */

/*! @name EICHD3_WORD3 - Error Injection Channel Descriptor 3, Word3 */
/*! @{ */

#define EIM_EICHD3_WORD3_B8_11DATA_MASK_MASK     (0xFFFFFFFFU)
#define EIM_EICHD3_WORD3_B8_11DATA_MASK_SHIFT    (0U)
/*! B8_11DATA_MASK - Data Mask Bytes 8-11 */
#define EIM_EICHD3_WORD3_B8_11DATA_MASK(x)       (((uint32_t)(((uint32_t)(x)) << EIM_EICHD3_WORD3_B8_11DATA_MASK_SHIFT)) & EIM_EICHD3_WORD3_B8_11DATA_MASK_MASK)
/*! @} */

/*! @name EICHD3_WORD4 - Error Injection Channel Descriptor 3, Word4 */
/*! @{ */

#define EIM_EICHD3_WORD4_B12_15DATA_MASK_MASK    (0xFFFFFFFFU)
#define EIM_EICHD3_WORD4_B12_15DATA_MASK_SHIFT   (0U)
/*! B12_15DATA_MASK - Data Mask Bytes 12-15 */
#define EIM_EICHD3_WORD4_B12_15DATA_MASK(x)      (((uint32_t)(((uint32_t)(x)) << EIM_EICHD3_WORD4_B12_15DATA_MASK_SHIFT)) & EIM_EICHD3_WORD4_B12_15DATA_MASK_MASK)
/*! @} */

/*! @name EICHD4_WORD0 - Error Injection Channel Descriptor 4, Word0 */
/*! @{ */

#define EIM_EICHD4_WORD0_CHKBIT_MASK_MASK        (0xFFFFFFF0U)
#define EIM_EICHD4_WORD0_CHKBIT_MASK_SHIFT       (4U)
/*! CHKBIT_MASK - Checkbit Mask */
#define EIM_EICHD4_WORD0_CHKBIT_MASK(x)          (((uint32_t)(((uint32_t)(x)) << EIM_EICHD4_WORD0_CHKBIT_MASK_SHIFT)) & EIM_EICHD4_WORD0_CHKBIT_MASK_MASK)
/*! @} */

/*! @name EICHD4_WORD1 - Error Injection Channel Descriptor 4, Word1 */
/*! @{ */

#define EIM_EICHD4_WORD1_B0_3DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD4_WORD1_B0_3DATA_MASK_SHIFT     (0U)
/*! B0_3DATA_MASK - Data Mask Bytes 0-3 */
#define EIM_EICHD4_WORD1_B0_3DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD4_WORD1_B0_3DATA_MASK_SHIFT)) & EIM_EICHD4_WORD1_B0_3DATA_MASK_MASK)
/*! @} */

/*! @name EICHD4_WORD2 - Error Injection Channel Descriptor 4, Word2 */
/*! @{ */

#define EIM_EICHD4_WORD2_B4_7DATA_MASK_MASK      (0xFFFFFFFFU)
#define EIM_EICHD4_WORD2_B4_7DATA_MASK_SHIFT     (0U)
/*! B4_7DATA_MASK - Data Mask Bytes 4-7 */
#define EIM_EICHD4_WORD2_B4_7DATA_MASK(x)        (((uint32_t)(((uint32_t)(x)) << EIM_EICHD4_WORD2_B4_7DATA_MASK_SHIFT)) & EIM_EICHD4_WORD2_B4_7DATA_MASK_MASK)
/*! @} */

/*! @name EICHD4_WORD3 - Error Injection Channel Descriptor 4, Word3 */
/*! @{ */

#define EIM_EICHD4_WORD3_B8_11DATA_MASK_MASK     (0xFFFFFFFFU)
#define EIM_EICHD4_WORD3_B8_11DATA_MASK_SHIFT    (0U)
/*! B8_11DATA_MASK - Data Mask Bytes 8-11 */
#define EIM_EICHD4_WORD3_B8_11DATA_MASK(x)       (((uint32_t)(((uint32_t)(x)) << EIM_EICHD4_WORD3_B8_11DATA_MASK_SHIFT)) & EIM_EICHD4_WORD3_B8_11DATA_MASK_MASK)
/*! @} */

/*! @name EICHD4_WORD4 - Error Injection Channel Descriptor 4, Word4 */
/*! @{ */

#define EIM_EICHD4_WORD4_B12_15DATA_MASK_MASK    (0xFFFFFFFFU)
#define EIM_EICHD4_WORD4_B12_15DATA_MASK_SHIFT   (0U)
/*! B12_15DATA_MASK - Data Mask Bytes 12-15 */
#define EIM_EICHD4_WORD4_B12_15DATA_MASK(x)      (((uint32_t)(((uint32_t)(x)) << EIM_EICHD4_WORD4_B12_15DATA_MASK_SHIFT)) & EIM_EICHD4_WORD4_B12_15DATA_MASK_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group EIM_Register_Masks */


/* EIM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral EIM base address */
  #define EIM_BASE                                 (0x5B860000u)
  /** Peripheral EIM base address */
  #define EIM_BASE_NS                              (0x4B860000u)
  /** Peripheral EIM base pointer */
  #define EIM                                      ((EIM_Type *)EIM_BASE)
  /** Peripheral EIM base pointer */
  #define EIM_NS                                   ((EIM_Type *)EIM_BASE_NS)
  /** Array initializer of EIM peripheral base addresses */
  #define EIM_BASE_ADDRS                           { EIM_BASE }
  /** Array initializer of EIM peripheral base pointers */
  #define EIM_BASE_PTRS                            { EIM }
  /** Array initializer of EIM peripheral base addresses */
  #define EIM_BASE_ADDRS_NS                        { EIM_BASE_NS }
  /** Array initializer of EIM peripheral base pointers */
  #define EIM_BASE_PTRS_NS                         { EIM_NS }
#else
  /** Peripheral EIM base address */
  #define EIM_BASE                                 (0x4B860000u)
  /** Peripheral EIM base pointer */
  #define EIM                                      ((EIM_Type *)EIM_BASE)
  /** Array initializer of EIM peripheral base addresses */
  #define EIM_BASE_ADDRS                           { EIM_BASE }
  /** Array initializer of EIM peripheral base pointers */
  #define EIM_BASE_PTRS                            { EIM }
#endif

/*!
 * @}
 */ /* end of group EIM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ENETC_GLOBAL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_GLOBAL_Peripheral_Access_Layer ENETC_GLOBAL Peripheral Access Layer
 * @{
 */

/** ENETC_GLOBAL - Register Layout Typedef */
typedef struct {
  __I  uint32_t SMCAPR;                            /**< Shared memory capability register, offset: 0x0 */
  __I  uint32_t SMDTR;                             /**< Shared memory depletion threshold register, offset: 0x4 */
  __I  uint32_t SMACR;                             /**< Shared memory available count register, offset: 0x8 */
       uint8_t RESERVED_0[4];
  __I  uint32_t SMCLWMR;                           /**< Shared memory count low watermark register, offset: 0x10 */
  __I  uint32_t SMBUCR;                            /**< Shared memory buffer unassigned count register, offset: 0x14 */
  __I  uint32_t SMBUCHWMR;                         /**< Shared memory buffer unassigned count high watermark register, offset: 0x18 */
  __I  uint32_t SMLCR;                             /**< Shared memory loss count register, offset: 0x1C */
  __I  uint32_t HBTCAPR;                           /**< Hash bucket table capability register, offset: 0x20 */
  __I  uint32_t HBTOR0;                            /**< Hash bucket table operational register 0, offset: 0x24 */
       uint8_t RESERVED_1[4];
  __I  uint32_t HBTOR2;                            /**< Hash bucket table operational register 2, offset: 0x2C */
       uint8_t RESERVED_2[16];
  __I  uint32_t SMERBCAPR;                         /**< Shared memory ENETC receive buffer capability register, offset: 0x40 */
  __I  uint32_t SMERBOR0;                          /**< Shared memory ENETC receive buffer operational register 0, offset: 0x44 */
  __I  uint32_t SMERBOR1;                          /**< Shared memory ENETC receive buffer operational 1, offset: 0x48 */
       uint8_t RESERVED_3[180];
  struct {                                         /* offset: 0x100, array step: 0x8 */
    __I  uint32_t PCEOR;                             /**< PCE 0 operational register, array offset: 0x100, array step: 0x8 */
    __I  uint32_t RFEOR;                             /**< Replication Forwarding Engine 0 operational register, array offset: 0x104, array step: 0x8 */
  } PCE_SL[1];
       uint8_t RESERVED_4[92];
  __I  uint32_t NETCCLKR;                          /**< NETC clock register, offset: 0x164 */
       uint8_t RESERVED_5[152];
  struct {                                         /* offset: 0x200, array step: 0x28 */
    __I  uint32_t HTACAPR;                           /**< HTA 0 capability register, array offset: 0x200, array step: 0x28 */
    __I  uint32_t HTARFCOR;                          /**< HTA 0 receive frame count operational register, array offset: 0x204, array step: 0x28 */
    __I  uint32_t HTAHPBCOR;                         /**< HTA 0 high priority byte count operational register, array offset: 0x208, array step: 0x28 */
    __I  uint32_t HTALPBCOR;                         /**< HTA 0 low priority byte count operational register, array offset: 0x20C, array step: 0x28 */
         uint8_t RESERVED_0[20];
    __I  uint32_t HTATFCOR;                          /**< HTA 0 transmit frame count operational register, array offset: 0x224, array step: 0x28 */
  } HTA_LOOP[1];
       uint8_t RESERVED_6[216];
  struct {                                         /* offset: 0x300, array step: 0x10 */
    __IO uint32_t RCSBRLAR;                          /**< Root complex 0 system bus read latency average register, array offset: 0x300, array step: 0x10 */
    __I  uint32_t RCSBRLHWMR;                        /**< Root complex 0 system bus read latency high watermark register, array offset: 0x304, array step: 0x10 */
    __IO uint32_t RCSBWLAR;                          /**< Root complex 0 system bus write latency average register, array offset: 0x308, array step: 0x10 */
    __I  uint32_t RCSBWLHWMR;                        /**< Root complex 0 system bus write latency high watermark register, array offset: 0x30C, array step: 0x10 */
  } ARRAY_NUM_RC[1];
       uint8_t RESERVED_7[2280];
  __I  uint32_t IPBRR0;                            /**< IP block revision register 0, offset: 0xBF8 */
  __I  uint32_t IPBRR1;                            /**< IP block revision register 1, offset: 0xBFC */
       uint8_t RESERVED_8[256];
  __I  uint32_t FBLPR[2];                          /**< Function boot loader parameter register 0..Function boot loader parameter register 1, array offset: 0xD00, array step: 0x4 */
       uint8_t RESERVED_9[280];
  union {                                          /* offset: 0xE20 */
    struct {                                         /* offset: 0xE20 */
      __IO uint32_t EMDIOUFSBECR;                      /**< EMDIO uncorrectable fatal system bus error configuration register, offset: 0xE20, available only on: EMDIO_GLOBAL (missing on ENETC0_GLOBAL, ENETC1_GLOBAL, SW0_GLOBAL, TMR0_GLOBAL) */
      __IO uint32_t EMDIOUFSBESR;                      /**< EMDIO uncorrectable fatal system bus error status register, offset: 0xE24, available only on: EMDIO_GLOBAL (missing on ENETC0_GLOBAL, ENETC1_GLOBAL, SW0_GLOBAL, TMR0_GLOBAL) */
    } EMDIO;
    struct {                                         /* offset: 0xE20 */
      __IO uint32_t TUFSBECR;                          /**< Timer uncorrectable fatal system bus error configuration register, offset: 0xE20, available only on: TMR0_GLOBAL (missing on EMDIO_GLOBAL, ENETC0_GLOBAL, ENETC1_GLOBAL, SW0_GLOBAL) */
      __IO uint32_t TUFSBESR;                          /**< Timer uncorrectable fatal system bus error status register, offset: 0xE24, available only on: TMR0_GLOBAL (missing on EMDIO_GLOBAL, ENETC0_GLOBAL, ENETC1_GLOBAL, SW0_GLOBAL) */
    } TIMER;
  };
} ENETC_GLOBAL_Type;

/* ----------------------------------------------------------------------------
   -- ENETC_GLOBAL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_GLOBAL_Register_Masks ENETC_GLOBAL Register Masks
 * @{
 */

/*! @name SMCAPR - Shared memory capability register */
/*! @{ */

#define ENETC_GLOBAL_SMCAPR_NUM_WORDS_MASK       (0xFFFFFFU)
#define ENETC_GLOBAL_SMCAPR_NUM_WORDS_SHIFT      (0U)
#define ENETC_GLOBAL_SMCAPR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMCAPR_NUM_WORDS_SHIFT)) & ENETC_GLOBAL_SMCAPR_NUM_WORDS_MASK)
/*! @} */

/*! @name SMDTR - Shared memory depletion threshold register */
/*! @{ */

#define ENETC_GLOBAL_SMDTR_THRESH_MASK           (0xFFFFFFU)
#define ENETC_GLOBAL_SMDTR_THRESH_SHIFT          (0U)
#define ENETC_GLOBAL_SMDTR_THRESH(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMDTR_THRESH_SHIFT)) & ENETC_GLOBAL_SMDTR_THRESH_MASK)
/*! @} */

/*! @name SMACR - Shared memory available count register */
/*! @{ */

#define ENETC_GLOBAL_SMACR_COUNT_MASK            (0xFFFFFFU)
#define ENETC_GLOBAL_SMACR_COUNT_SHIFT           (0U)
#define ENETC_GLOBAL_SMACR_COUNT(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMACR_COUNT_SHIFT)) & ENETC_GLOBAL_SMACR_COUNT_MASK)
/*! @} */

/*! @name SMCLWMR - Shared memory count low watermark register */
/*! @{ */

#define ENETC_GLOBAL_SMCLWMR_WATERMARK_MASK      (0xFFFFFFU)
#define ENETC_GLOBAL_SMCLWMR_WATERMARK_SHIFT     (0U)
#define ENETC_GLOBAL_SMCLWMR_WATERMARK(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMCLWMR_WATERMARK_SHIFT)) & ENETC_GLOBAL_SMCLWMR_WATERMARK_MASK)
/*! @} */

/*! @name SMBUCR - Shared memory buffer unassigned count register */
/*! @{ */

#define ENETC_GLOBAL_SMBUCR_COUNT_MASK           (0xFFFFFFU)
#define ENETC_GLOBAL_SMBUCR_COUNT_SHIFT          (0U)
#define ENETC_GLOBAL_SMBUCR_COUNT(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMBUCR_COUNT_SHIFT)) & ENETC_GLOBAL_SMBUCR_COUNT_MASK)
/*! @} */

/*! @name SMBUCHWMR - Shared memory buffer unassigned count high watermark register */
/*! @{ */

#define ENETC_GLOBAL_SMBUCHWMR_WATERMARK_MASK    (0xFFFFFFU)
#define ENETC_GLOBAL_SMBUCHWMR_WATERMARK_SHIFT   (0U)
#define ENETC_GLOBAL_SMBUCHWMR_WATERMARK(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMBUCHWMR_WATERMARK_SHIFT)) & ENETC_GLOBAL_SMBUCHWMR_WATERMARK_MASK)
/*! @} */

/*! @name SMLCR - Shared memory loss count register */
/*! @{ */

#define ENETC_GLOBAL_SMLCR_COUNT_MASK            (0xFFFFFFU)
#define ENETC_GLOBAL_SMLCR_COUNT_SHIFT           (0U)
#define ENETC_GLOBAL_SMLCR_COUNT(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMLCR_COUNT_SHIFT)) & ENETC_GLOBAL_SMLCR_COUNT_MASK)

#define ENETC_GLOBAL_SMLCR_IFLC_MASK             (0x40000000U)
#define ENETC_GLOBAL_SMLCR_IFLC_SHIFT            (30U)
#define ENETC_GLOBAL_SMLCR_IFLC(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMLCR_IFLC_SHIFT)) & ENETC_GLOBAL_SMLCR_IFLC_MASK)

#define ENETC_GLOBAL_SMLCR_IFDC_MASK             (0x80000000U)
#define ENETC_GLOBAL_SMLCR_IFDC_SHIFT            (31U)
#define ENETC_GLOBAL_SMLCR_IFDC(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMLCR_IFDC_SHIFT)) & ENETC_GLOBAL_SMLCR_IFDC_MASK)
/*! @} */

/*! @name HBTCAPR - Hash bucket table capability register */
/*! @{ */

#define ENETC_GLOBAL_HBTCAPR_NUM_ENTRIES_MASK    (0xFFFFU)
#define ENETC_GLOBAL_HBTCAPR_NUM_ENTRIES_SHIFT   (0U)
#define ENETC_GLOBAL_HBTCAPR_NUM_ENTRIES(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTCAPR_NUM_ENTRIES_SHIFT)) & ENETC_GLOBAL_HBTCAPR_NUM_ENTRIES_MASK)

#define ENETC_GLOBAL_HBTCAPR_MAX_COL_MASK        (0x7000000U)
#define ENETC_GLOBAL_HBTCAPR_MAX_COL_SHIFT       (24U)
#define ENETC_GLOBAL_HBTCAPR_MAX_COL(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTCAPR_MAX_COL_SHIFT)) & ENETC_GLOBAL_HBTCAPR_MAX_COL_MASK)

#define ENETC_GLOBAL_HBTCAPR_MAX_VISITS_MASK     (0xF0000000U)
#define ENETC_GLOBAL_HBTCAPR_MAX_VISITS_SHIFT    (28U)
#define ENETC_GLOBAL_HBTCAPR_MAX_VISITS(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTCAPR_MAX_VISITS_SHIFT)) & ENETC_GLOBAL_HBTCAPR_MAX_VISITS_MASK)
/*! @} */

/*! @name HBTOR0 - Hash bucket table operational register 0 */
/*! @{ */

#define ENETC_GLOBAL_HBTOR0_NUM_ENTRIES_MASK     (0xFFFFU)
#define ENETC_GLOBAL_HBTOR0_NUM_ENTRIES_SHIFT    (0U)
#define ENETC_GLOBAL_HBTOR0_NUM_ENTRIES(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTOR0_NUM_ENTRIES_SHIFT)) & ENETC_GLOBAL_HBTOR0_NUM_ENTRIES_MASK)

#define ENETC_GLOBAL_HBTOR0_HWM_ENTRIES_MASK     (0xFFFF0000U)
#define ENETC_GLOBAL_HBTOR0_HWM_ENTRIES_SHIFT    (16U)
#define ENETC_GLOBAL_HBTOR0_HWM_ENTRIES(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTOR0_HWM_ENTRIES_SHIFT)) & ENETC_GLOBAL_HBTOR0_HWM_ENTRIES_MASK)
/*! @} */

/*! @name HBTOR2 - Hash bucket table operational register 2 */
/*! @{ */

#define ENETC_GLOBAL_HBTOR2_RUN_AVG_FRACT_MASK   (0xFFU)
#define ENETC_GLOBAL_HBTOR2_RUN_AVG_FRACT_SHIFT  (0U)
#define ENETC_GLOBAL_HBTOR2_RUN_AVG_FRACT(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTOR2_RUN_AVG_FRACT_SHIFT)) & ENETC_GLOBAL_HBTOR2_RUN_AVG_FRACT_MASK)

#define ENETC_GLOBAL_HBTOR2_RUN_AVG_INT_MASK     (0xFF00U)
#define ENETC_GLOBAL_HBTOR2_RUN_AVG_INT_SHIFT    (8U)
#define ENETC_GLOBAL_HBTOR2_RUN_AVG_INT(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTOR2_RUN_AVG_INT_SHIFT)) & ENETC_GLOBAL_HBTOR2_RUN_AVG_INT_MASK)

#define ENETC_GLOBAL_HBTOR2_HWM_COL_MASK         (0xF0000U)
#define ENETC_GLOBAL_HBTOR2_HWM_COL_SHIFT        (16U)
#define ENETC_GLOBAL_HBTOR2_HWM_COL(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HBTOR2_HWM_COL_SHIFT)) & ENETC_GLOBAL_HBTOR2_HWM_COL_MASK)
/*! @} */

/*! @name SMERBCAPR - Shared memory ENETC receive buffer capability register */
/*! @{ */

#define ENETC_GLOBAL_SMERBCAPR_THRESH_MASK       (0xFFFFFFU)
#define ENETC_GLOBAL_SMERBCAPR_THRESH_SHIFT      (0U)
#define ENETC_GLOBAL_SMERBCAPR_THRESH(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMERBCAPR_THRESH_SHIFT)) & ENETC_GLOBAL_SMERBCAPR_THRESH_MASK)

#define ENETC_GLOBAL_SMERBCAPR_WORD_SIZE_MASK    (0x30000000U)
#define ENETC_GLOBAL_SMERBCAPR_WORD_SIZE_SHIFT   (28U)
#define ENETC_GLOBAL_SMERBCAPR_WORD_SIZE(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMERBCAPR_WORD_SIZE_SHIFT)) & ENETC_GLOBAL_SMERBCAPR_WORD_SIZE_MASK)

#define ENETC_GLOBAL_SMERBCAPR_MLOC_MASK         (0xC0000000U)
#define ENETC_GLOBAL_SMERBCAPR_MLOC_SHIFT        (30U)
#define ENETC_GLOBAL_SMERBCAPR_MLOC(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMERBCAPR_MLOC_SHIFT)) & ENETC_GLOBAL_SMERBCAPR_MLOC_MASK)
/*! @} */

/*! @name SMERBOR0 - Shared memory ENETC receive buffer operational register 0 */
/*! @{ */

#define ENETC_GLOBAL_SMERBOR0_AMOUNT_MASK        (0xFFFFFFU)
#define ENETC_GLOBAL_SMERBOR0_AMOUNT_SHIFT       (0U)
#define ENETC_GLOBAL_SMERBOR0_AMOUNT(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMERBOR0_AMOUNT_SHIFT)) & ENETC_GLOBAL_SMERBOR0_AMOUNT_MASK)
/*! @} */

/*! @name SMERBOR1 - Shared memory ENETC receive buffer operational 1 */
/*! @{ */

#define ENETC_GLOBAL_SMERBOR1_WATERMARK_MASK     (0xFFFFFFU)
#define ENETC_GLOBAL_SMERBOR1_WATERMARK_SHIFT    (0U)
#define ENETC_GLOBAL_SMERBOR1_WATERMARK(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_SMERBOR1_WATERMARK_SHIFT)) & ENETC_GLOBAL_SMERBOR1_WATERMARK_MASK)
/*! @} */

/*! @name PCEOR - PCE 0 operational register */
/*! @{ */

#define ENETC_GLOBAL_PCEOR_NUM_FRAMES_MASK       (0x3FU)
#define ENETC_GLOBAL_PCEOR_NUM_FRAMES_SHIFT      (0U)
#define ENETC_GLOBAL_PCEOR_NUM_FRAMES(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_PCEOR_NUM_FRAMES_SHIFT)) & ENETC_GLOBAL_PCEOR_NUM_FRAMES_MASK)

#define ENETC_GLOBAL_PCEOR_HWM_FRAMES_MASK       (0x3F00U)
#define ENETC_GLOBAL_PCEOR_HWM_FRAMES_SHIFT      (8U)
#define ENETC_GLOBAL_PCEOR_HWM_FRAMES(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_PCEOR_HWM_FRAMES_SHIFT)) & ENETC_GLOBAL_PCEOR_HWM_FRAMES_MASK)

#define ENETC_GLOBAL_PCEOR_MAX_FRAMES_MASK       (0x3F0000U)
#define ENETC_GLOBAL_PCEOR_MAX_FRAMES_SHIFT      (16U)
#define ENETC_GLOBAL_PCEOR_MAX_FRAMES(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_PCEOR_MAX_FRAMES_SHIFT)) & ENETC_GLOBAL_PCEOR_MAX_FRAMES_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_PCEOR */
#define ENETC_GLOBAL_PCEOR_COUNT                 (1U)

/*! @name RFEOR - Replication Forwarding Engine 0 operational register */
/*! @{ */

#define ENETC_GLOBAL_RFEOR_NUM_FRAMES_MASK       (0x3FU)
#define ENETC_GLOBAL_RFEOR_NUM_FRAMES_SHIFT      (0U)
#define ENETC_GLOBAL_RFEOR_NUM_FRAMES(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RFEOR_NUM_FRAMES_SHIFT)) & ENETC_GLOBAL_RFEOR_NUM_FRAMES_MASK)

#define ENETC_GLOBAL_RFEOR_HWM_FRAMES_MASK       (0x3F00U)
#define ENETC_GLOBAL_RFEOR_HWM_FRAMES_SHIFT      (8U)
#define ENETC_GLOBAL_RFEOR_HWM_FRAMES(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RFEOR_HWM_FRAMES_SHIFT)) & ENETC_GLOBAL_RFEOR_HWM_FRAMES_MASK)

#define ENETC_GLOBAL_RFEOR_MAX_FRAMES_MASK       (0x3F0000U)
#define ENETC_GLOBAL_RFEOR_MAX_FRAMES_SHIFT      (16U)
#define ENETC_GLOBAL_RFEOR_MAX_FRAMES(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RFEOR_MAX_FRAMES_SHIFT)) & ENETC_GLOBAL_RFEOR_MAX_FRAMES_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_RFEOR */
#define ENETC_GLOBAL_RFEOR_COUNT                 (1U)

/*! @name NETCCLKR - NETC clock register */
/*! @{ */

#define ENETC_GLOBAL_NETCCLKR_FREQ_MASK          (0x7FFU)
#define ENETC_GLOBAL_NETCCLKR_FREQ_SHIFT         (0U)
/*! FREQ - Frequency */
#define ENETC_GLOBAL_NETCCLKR_FREQ(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_NETCCLKR_FREQ_SHIFT)) & ENETC_GLOBAL_NETCCLKR_FREQ_MASK)
/*! @} */

/*! @name HTACAPR - HTA 0 capability register */
/*! @{ */

#define ENETC_GLOBAL_HTACAPR_MAX_RX_FRAMES_MASK  (0xFFU)
#define ENETC_GLOBAL_HTACAPR_MAX_RX_FRAMES_SHIFT (0U)
#define ENETC_GLOBAL_HTACAPR_MAX_RX_FRAMES(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTACAPR_MAX_RX_FRAMES_SHIFT)) & ENETC_GLOBAL_HTACAPR_MAX_RX_FRAMES_MASK)

#define ENETC_GLOBAL_HTACAPR_MAX_TX_FRAMES_MASK  (0xFF00U)
#define ENETC_GLOBAL_HTACAPR_MAX_TX_FRAMES_SHIFT (8U)
#define ENETC_GLOBAL_HTACAPR_MAX_TX_FRAMES(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTACAPR_MAX_TX_FRAMES_SHIFT)) & ENETC_GLOBAL_HTACAPR_MAX_TX_FRAMES_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_HTACAPR */
#define ENETC_GLOBAL_HTACAPR_COUNT               (1U)

/*! @name HTARFCOR - HTA 0 receive frame count operational register */
/*! @{ */

#define ENETC_GLOBAL_HTARFCOR_HP_COUNT_MASK      (0xFFU)
#define ENETC_GLOBAL_HTARFCOR_HP_COUNT_SHIFT     (0U)
#define ENETC_GLOBAL_HTARFCOR_HP_COUNT(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTARFCOR_HP_COUNT_SHIFT)) & ENETC_GLOBAL_HTARFCOR_HP_COUNT_MASK)

#define ENETC_GLOBAL_HTARFCOR_HP_HWM_MASK        (0xFF00U)
#define ENETC_GLOBAL_HTARFCOR_HP_HWM_SHIFT       (8U)
#define ENETC_GLOBAL_HTARFCOR_HP_HWM(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTARFCOR_HP_HWM_SHIFT)) & ENETC_GLOBAL_HTARFCOR_HP_HWM_MASK)

#define ENETC_GLOBAL_HTARFCOR_LP_COUNT_MASK      (0xFF0000U)
#define ENETC_GLOBAL_HTARFCOR_LP_COUNT_SHIFT     (16U)
#define ENETC_GLOBAL_HTARFCOR_LP_COUNT(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTARFCOR_LP_COUNT_SHIFT)) & ENETC_GLOBAL_HTARFCOR_LP_COUNT_MASK)

#define ENETC_GLOBAL_HTARFCOR_LP_HWM_MASK        (0xFF000000U)
#define ENETC_GLOBAL_HTARFCOR_LP_HWM_SHIFT       (24U)
#define ENETC_GLOBAL_HTARFCOR_LP_HWM(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTARFCOR_LP_HWM_SHIFT)) & ENETC_GLOBAL_HTARFCOR_LP_HWM_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_HTARFCOR */
#define ENETC_GLOBAL_HTARFCOR_COUNT              (1U)

/*! @name HTAHPBCOR - HTA 0 high priority byte count operational register */
/*! @{ */

#define ENETC_GLOBAL_HTAHPBCOR_HP_COUNT_MASK     (0xFFFFU)
#define ENETC_GLOBAL_HTAHPBCOR_HP_COUNT_SHIFT    (0U)
#define ENETC_GLOBAL_HTAHPBCOR_HP_COUNT(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTAHPBCOR_HP_COUNT_SHIFT)) & ENETC_GLOBAL_HTAHPBCOR_HP_COUNT_MASK)

#define ENETC_GLOBAL_HTAHPBCOR_HWM_MASK          (0xFFFF0000U)
#define ENETC_GLOBAL_HTAHPBCOR_HWM_SHIFT         (16U)
#define ENETC_GLOBAL_HTAHPBCOR_HWM(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTAHPBCOR_HWM_SHIFT)) & ENETC_GLOBAL_HTAHPBCOR_HWM_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_HTAHPBCOR */
#define ENETC_GLOBAL_HTAHPBCOR_COUNT             (1U)

/*! @name HTALPBCOR - HTA 0 low priority byte count operational register */
/*! @{ */

#define ENETC_GLOBAL_HTALPBCOR_LP_COUNT_MASK     (0xFFFFU)
#define ENETC_GLOBAL_HTALPBCOR_LP_COUNT_SHIFT    (0U)
#define ENETC_GLOBAL_HTALPBCOR_LP_COUNT(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTALPBCOR_LP_COUNT_SHIFT)) & ENETC_GLOBAL_HTALPBCOR_LP_COUNT_MASK)

#define ENETC_GLOBAL_HTALPBCOR_HWM_MASK          (0xFFFF0000U)
#define ENETC_GLOBAL_HTALPBCOR_HWM_SHIFT         (16U)
#define ENETC_GLOBAL_HTALPBCOR_HWM(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTALPBCOR_HWM_SHIFT)) & ENETC_GLOBAL_HTALPBCOR_HWM_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_HTALPBCOR */
#define ENETC_GLOBAL_HTALPBCOR_COUNT             (1U)

/*! @name HTATFCOR - HTA 0 transmit frame count operational register */
/*! @{ */

#define ENETC_GLOBAL_HTATFCOR_HP_COUNT_MASK      (0xFFU)
#define ENETC_GLOBAL_HTATFCOR_HP_COUNT_SHIFT     (0U)
#define ENETC_GLOBAL_HTATFCOR_HP_COUNT(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTATFCOR_HP_COUNT_SHIFT)) & ENETC_GLOBAL_HTATFCOR_HP_COUNT_MASK)

#define ENETC_GLOBAL_HTATFCOR_HP_HWM_MASK        (0xFF00U)
#define ENETC_GLOBAL_HTATFCOR_HP_HWM_SHIFT       (8U)
#define ENETC_GLOBAL_HTATFCOR_HP_HWM(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTATFCOR_HP_HWM_SHIFT)) & ENETC_GLOBAL_HTATFCOR_HP_HWM_MASK)

#define ENETC_GLOBAL_HTATFCOR_LP_COUNT_MASK      (0xFF0000U)
#define ENETC_GLOBAL_HTATFCOR_LP_COUNT_SHIFT     (16U)
#define ENETC_GLOBAL_HTATFCOR_LP_COUNT(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTATFCOR_LP_COUNT_SHIFT)) & ENETC_GLOBAL_HTATFCOR_LP_COUNT_MASK)

#define ENETC_GLOBAL_HTATFCOR_LP_HWM_MASK        (0xFF000000U)
#define ENETC_GLOBAL_HTATFCOR_LP_HWM_SHIFT       (24U)
#define ENETC_GLOBAL_HTATFCOR_LP_HWM(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_HTATFCOR_LP_HWM_SHIFT)) & ENETC_GLOBAL_HTATFCOR_LP_HWM_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_HTATFCOR */
#define ENETC_GLOBAL_HTATFCOR_COUNT              (1U)

/*! @name RCSBRLAR - Root complex 0 system bus read latency average register */
/*! @{ */

#define ENETC_GLOBAL_RCSBRLAR_FRACT_MASK         (0xFFU)
#define ENETC_GLOBAL_RCSBRLAR_FRACT_SHIFT        (0U)
#define ENETC_GLOBAL_RCSBRLAR_FRACT(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBRLAR_FRACT_SHIFT)) & ENETC_GLOBAL_RCSBRLAR_FRACT_MASK)

#define ENETC_GLOBAL_RCSBRLAR_INT_MASK           (0xFFF00U)
#define ENETC_GLOBAL_RCSBRLAR_INT_SHIFT          (8U)
#define ENETC_GLOBAL_RCSBRLAR_INT(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBRLAR_INT_SHIFT)) & ENETC_GLOBAL_RCSBRLAR_INT_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_RCSBRLAR */
#define ENETC_GLOBAL_RCSBRLAR_COUNT              (1U)

/*! @name RCSBRLHWMR - Root complex 0 system bus read latency high watermark register */
/*! @{ */

#define ENETC_GLOBAL_RCSBRLHWMR_FRACT_MASK       (0xFFU)
#define ENETC_GLOBAL_RCSBRLHWMR_FRACT_SHIFT      (0U)
#define ENETC_GLOBAL_RCSBRLHWMR_FRACT(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBRLHWMR_FRACT_SHIFT)) & ENETC_GLOBAL_RCSBRLHWMR_FRACT_MASK)

#define ENETC_GLOBAL_RCSBRLHWMR_INT_MASK         (0xFFF00U)
#define ENETC_GLOBAL_RCSBRLHWMR_INT_SHIFT        (8U)
#define ENETC_GLOBAL_RCSBRLHWMR_INT(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBRLHWMR_INT_SHIFT)) & ENETC_GLOBAL_RCSBRLHWMR_INT_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_RCSBRLHWMR */
#define ENETC_GLOBAL_RCSBRLHWMR_COUNT            (1U)

/*! @name RCSBWLAR - Root complex 0 system bus write latency average register */
/*! @{ */

#define ENETC_GLOBAL_RCSBWLAR_FRACT_MASK         (0xFFU)
#define ENETC_GLOBAL_RCSBWLAR_FRACT_SHIFT        (0U)
#define ENETC_GLOBAL_RCSBWLAR_FRACT(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBWLAR_FRACT_SHIFT)) & ENETC_GLOBAL_RCSBWLAR_FRACT_MASK)

#define ENETC_GLOBAL_RCSBWLAR_INT_MASK           (0xFFF00U)
#define ENETC_GLOBAL_RCSBWLAR_INT_SHIFT          (8U)
#define ENETC_GLOBAL_RCSBWLAR_INT(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBWLAR_INT_SHIFT)) & ENETC_GLOBAL_RCSBWLAR_INT_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_RCSBWLAR */
#define ENETC_GLOBAL_RCSBWLAR_COUNT              (1U)

/*! @name RCSBWLHWMR - Root complex 0 system bus write latency high watermark register */
/*! @{ */

#define ENETC_GLOBAL_RCSBWLHWMR_FRACT_MASK       (0xFFU)
#define ENETC_GLOBAL_RCSBWLHWMR_FRACT_SHIFT      (0U)
#define ENETC_GLOBAL_RCSBWLHWMR_FRACT(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBWLHWMR_FRACT_SHIFT)) & ENETC_GLOBAL_RCSBWLHWMR_FRACT_MASK)

#define ENETC_GLOBAL_RCSBWLHWMR_INT_MASK         (0xFFF00U)
#define ENETC_GLOBAL_RCSBWLHWMR_INT_SHIFT        (8U)
#define ENETC_GLOBAL_RCSBWLHWMR_INT(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_RCSBWLHWMR_INT_SHIFT)) & ENETC_GLOBAL_RCSBWLHWMR_INT_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_RCSBWLHWMR */
#define ENETC_GLOBAL_RCSBWLHWMR_COUNT            (1U)

/*! @name IPBRR0 - IP block revision register 0 */
/*! @{ */

#define ENETC_GLOBAL_IPBRR0_IP_MN_MASK           (0xFFU)
#define ENETC_GLOBAL_IPBRR0_IP_MN_SHIFT          (0U)
#define ENETC_GLOBAL_IPBRR0_IP_MN(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_IPBRR0_IP_MN_SHIFT)) & ENETC_GLOBAL_IPBRR0_IP_MN_MASK)

#define ENETC_GLOBAL_IPBRR0_IP_MJ_MASK           (0xFF00U)
#define ENETC_GLOBAL_IPBRR0_IP_MJ_SHIFT          (8U)
#define ENETC_GLOBAL_IPBRR0_IP_MJ(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_IPBRR0_IP_MJ_SHIFT)) & ENETC_GLOBAL_IPBRR0_IP_MJ_MASK)

#define ENETC_GLOBAL_IPBRR0_IP_ID_MASK           (0xFFFF0000U)
#define ENETC_GLOBAL_IPBRR0_IP_ID_SHIFT          (16U)
#define ENETC_GLOBAL_IPBRR0_IP_ID(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_IPBRR0_IP_ID_SHIFT)) & ENETC_GLOBAL_IPBRR0_IP_ID_MASK)
/*! @} */

/*! @name IPBRR1 - IP block revision register 1 */
/*! @{ */

#define ENETC_GLOBAL_IPBRR1_IP_CFG_MASK          (0xFFU)
#define ENETC_GLOBAL_IPBRR1_IP_CFG_SHIFT         (0U)
#define ENETC_GLOBAL_IPBRR1_IP_CFG(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_IPBRR1_IP_CFG_SHIFT)) & ENETC_GLOBAL_IPBRR1_IP_CFG_MASK)

#define ENETC_GLOBAL_IPBRR1_IP_MNT_MASK          (0xFF00U)
#define ENETC_GLOBAL_IPBRR1_IP_MNT_SHIFT         (8U)
#define ENETC_GLOBAL_IPBRR1_IP_MNT(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_IPBRR1_IP_MNT_SHIFT)) & ENETC_GLOBAL_IPBRR1_IP_MNT_MASK)

#define ENETC_GLOBAL_IPBRR1_IP_INT_MASK          (0xFF0000U)
#define ENETC_GLOBAL_IPBRR1_IP_INT_SHIFT         (16U)
#define ENETC_GLOBAL_IPBRR1_IP_INT(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_IPBRR1_IP_INT_SHIFT)) & ENETC_GLOBAL_IPBRR1_IP_INT_MASK)
/*! @} */

/*! @name FBLPR - Function boot loader parameter register 0..Function boot loader parameter register 1 */
/*! @{ */

#define ENETC_GLOBAL_FBLPR_PARAM_VAL_MASK        (0xFFFFFFFFU)
#define ENETC_GLOBAL_FBLPR_PARAM_VAL_SHIFT       (0U)
#define ENETC_GLOBAL_FBLPR_PARAM_VAL(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_FBLPR_PARAM_VAL_SHIFT)) & ENETC_GLOBAL_FBLPR_PARAM_VAL_MASK)
/*! @} */

/* The count of ENETC_GLOBAL_FBLPR */
#define ENETC_GLOBAL_FBLPR_COUNT                 (2U)

/*! @name EMDIOUFSBECR - EMDIO uncorrectable fatal system bus error configuration register */
/*! @{ */

#define ENETC_GLOBAL_EMDIOUFSBECR_RD_MASK        (0x80000000U)
#define ENETC_GLOBAL_EMDIOUFSBECR_RD_SHIFT       (31U)
/*! RD - Report disable */
#define ENETC_GLOBAL_EMDIOUFSBECR_RD(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_EMDIOUFSBECR_RD_SHIFT)) & ENETC_GLOBAL_EMDIOUFSBECR_RD_MASK)
/*! @} */

/*! @name EMDIOUFSBESR - EMDIO uncorrectable fatal system bus error status register */
/*! @{ */

#define ENETC_GLOBAL_EMDIOUFSBESR_SB_ID_MASK     (0xFU)
#define ENETC_GLOBAL_EMDIOUFSBESR_SB_ID_SHIFT    (0U)
/*! SB_ID - System Bus ID */
#define ENETC_GLOBAL_EMDIOUFSBESR_SB_ID(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_EMDIOUFSBESR_SB_ID_SHIFT)) & ENETC_GLOBAL_EMDIOUFSBESR_SB_ID_MASK)

#define ENETC_GLOBAL_EMDIOUFSBESR_M_MASK         (0x40000000U)
#define ENETC_GLOBAL_EMDIOUFSBESR_M_SHIFT        (30U)
/*! M - Multiple */
#define ENETC_GLOBAL_EMDIOUFSBESR_M(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_EMDIOUFSBESR_M_SHIFT)) & ENETC_GLOBAL_EMDIOUFSBESR_M_MASK)

#define ENETC_GLOBAL_EMDIOUFSBESR_SBE_MASK       (0x80000000U)
#define ENETC_GLOBAL_EMDIOUFSBESR_SBE_SHIFT      (31U)
/*! SBE - System bus error */
#define ENETC_GLOBAL_EMDIOUFSBESR_SBE(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_EMDIOUFSBESR_SBE_SHIFT)) & ENETC_GLOBAL_EMDIOUFSBESR_SBE_MASK)
/*! @} */

/*! @name TUFSBECR - Timer uncorrectable fatal system bus error configuration register */
/*! @{ */

#define ENETC_GLOBAL_TUFSBECR_RD_MASK            (0x80000000U)
#define ENETC_GLOBAL_TUFSBECR_RD_SHIFT           (31U)
/*! RD - Report disable */
#define ENETC_GLOBAL_TUFSBECR_RD(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_TUFSBECR_RD_SHIFT)) & ENETC_GLOBAL_TUFSBECR_RD_MASK)
/*! @} */

/*! @name TUFSBESR - Timer uncorrectable fatal system bus error status register */
/*! @{ */

#define ENETC_GLOBAL_TUFSBESR_SB_ID_MASK         (0xFU)
#define ENETC_GLOBAL_TUFSBESR_SB_ID_SHIFT        (0U)
/*! SB_ID - System Bus ID */
#define ENETC_GLOBAL_TUFSBESR_SB_ID(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_TUFSBESR_SB_ID_SHIFT)) & ENETC_GLOBAL_TUFSBESR_SB_ID_MASK)

#define ENETC_GLOBAL_TUFSBESR_M_MASK             (0x40000000U)
#define ENETC_GLOBAL_TUFSBESR_M_SHIFT            (30U)
/*! M - Multiple */
#define ENETC_GLOBAL_TUFSBESR_M(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_TUFSBESR_M_SHIFT)) & ENETC_GLOBAL_TUFSBESR_M_MASK)

#define ENETC_GLOBAL_TUFSBESR_SBE_MASK           (0x80000000U)
#define ENETC_GLOBAL_TUFSBESR_SBE_SHIFT          (31U)
/*! SBE - System bus error */
#define ENETC_GLOBAL_TUFSBESR_SBE(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_GLOBAL_TUFSBESR_SBE_SHIFT)) & ENETC_GLOBAL_TUFSBESR_SBE_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ENETC_GLOBAL_Register_Masks */


/* ENETC_GLOBAL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral EMDIO_GLOBAL base address */
  #define EMDIO_GLOBAL_BASE                        (0x70BB0000u)
  /** Peripheral EMDIO_GLOBAL base address */
  #define EMDIO_GLOBAL_BASE_NS                     (0x60BB0000u)
  /** Peripheral EMDIO_GLOBAL base pointer */
  #define EMDIO_GLOBAL                             ((ENETC_GLOBAL_Type *)EMDIO_GLOBAL_BASE)
  /** Peripheral EMDIO_GLOBAL base pointer */
  #define EMDIO_GLOBAL_NS                          ((ENETC_GLOBAL_Type *)EMDIO_GLOBAL_BASE_NS)
  /** Peripheral ENETC0_GLOBAL base address */
  #define ENETC0_GLOBAL_BASE                       (0x70B20000u)
  /** Peripheral ENETC0_GLOBAL base address */
  #define ENETC0_GLOBAL_BASE_NS                    (0x60B20000u)
  /** Peripheral ENETC0_GLOBAL base pointer */
  #define ENETC0_GLOBAL                            ((ENETC_GLOBAL_Type *)ENETC0_GLOBAL_BASE)
  /** Peripheral ENETC0_GLOBAL base pointer */
  #define ENETC0_GLOBAL_NS                         ((ENETC_GLOBAL_Type *)ENETC0_GLOBAL_BASE_NS)
  /** Peripheral ENETC1_GLOBAL base address */
  #define ENETC1_GLOBAL_BASE                       (0x70B60000u)
  /** Peripheral ENETC1_GLOBAL base address */
  #define ENETC1_GLOBAL_BASE_NS                    (0x60B60000u)
  /** Peripheral ENETC1_GLOBAL base pointer */
  #define ENETC1_GLOBAL                            ((ENETC_GLOBAL_Type *)ENETC1_GLOBAL_BASE)
  /** Peripheral ENETC1_GLOBAL base pointer */
  #define ENETC1_GLOBAL_NS                         ((ENETC_GLOBAL_Type *)ENETC1_GLOBAL_BASE_NS)
  /** Peripheral SW0_GLOBAL base address */
  #define SW0_GLOBAL_BASE                          (0x70A80000u)
  /** Peripheral SW0_GLOBAL base address */
  #define SW0_GLOBAL_BASE_NS                       (0x60A80000u)
  /** Peripheral SW0_GLOBAL base pointer */
  #define SW0_GLOBAL                               ((ENETC_GLOBAL_Type *)SW0_GLOBAL_BASE)
  /** Peripheral SW0_GLOBAL base pointer */
  #define SW0_GLOBAL_NS                            ((ENETC_GLOBAL_Type *)SW0_GLOBAL_BASE_NS)
  /** Peripheral TMR0_GLOBAL base address */
  #define TMR0_GLOBAL_BASE                         (0x70B90000u)
  /** Peripheral TMR0_GLOBAL base address */
  #define TMR0_GLOBAL_BASE_NS                      (0x60B90000u)
  /** Peripheral TMR0_GLOBAL base pointer */
  #define TMR0_GLOBAL                              ((ENETC_GLOBAL_Type *)TMR0_GLOBAL_BASE)
  /** Peripheral TMR0_GLOBAL base pointer */
  #define TMR0_GLOBAL_NS                           ((ENETC_GLOBAL_Type *)TMR0_GLOBAL_BASE_NS)
  /** Array initializer of ENETC_GLOBAL peripheral base addresses */
  #define ENETC_GLOBAL_BASE_ADDRS                  { EMDIO_GLOBAL_BASE, ENETC0_GLOBAL_BASE, ENETC1_GLOBAL_BASE, SW0_GLOBAL_BASE, TMR0_GLOBAL_BASE }
  /** Array initializer of ENETC_GLOBAL peripheral base pointers */
  #define ENETC_GLOBAL_BASE_PTRS                   { EMDIO_GLOBAL, ENETC0_GLOBAL, ENETC1_GLOBAL, SW0_GLOBAL, TMR0_GLOBAL }
  /** Array initializer of ENETC_GLOBAL peripheral base addresses */
  #define ENETC_GLOBAL_BASE_ADDRS_NS               { EMDIO_GLOBAL_BASE_NS, ENETC0_GLOBAL_BASE_NS, ENETC1_GLOBAL_BASE_NS, SW0_GLOBAL_BASE_NS, TMR0_GLOBAL_BASE_NS }
  /** Array initializer of ENETC_GLOBAL peripheral base pointers */
  #define ENETC_GLOBAL_BASE_PTRS_NS                { EMDIO_GLOBAL_NS, ENETC0_GLOBAL_NS, ENETC1_GLOBAL_NS, SW0_GLOBAL_NS, TMR0_GLOBAL_NS }
#else
  /** Peripheral EMDIO_GLOBAL base address */
  #define EMDIO_GLOBAL_BASE                        (0x60BB0000u)
  /** Peripheral EMDIO_GLOBAL base pointer */
  #define EMDIO_GLOBAL                             ((ENETC_GLOBAL_Type *)EMDIO_GLOBAL_BASE)
  /** Peripheral ENETC0_GLOBAL base address */
  #define ENETC0_GLOBAL_BASE                       (0x60B20000u)
  /** Peripheral ENETC0_GLOBAL base pointer */
  #define ENETC0_GLOBAL                            ((ENETC_GLOBAL_Type *)ENETC0_GLOBAL_BASE)
  /** Peripheral ENETC1_GLOBAL base address */
  #define ENETC1_GLOBAL_BASE                       (0x60B60000u)
  /** Peripheral ENETC1_GLOBAL base pointer */
  #define ENETC1_GLOBAL                            ((ENETC_GLOBAL_Type *)ENETC1_GLOBAL_BASE)
  /** Peripheral SW0_GLOBAL base address */
  #define SW0_GLOBAL_BASE                          (0x60A80000u)
  /** Peripheral SW0_GLOBAL base pointer */
  #define SW0_GLOBAL                               ((ENETC_GLOBAL_Type *)SW0_GLOBAL_BASE)
  /** Peripheral TMR0_GLOBAL base address */
  #define TMR0_GLOBAL_BASE                         (0x60B90000u)
  /** Peripheral TMR0_GLOBAL base pointer */
  #define TMR0_GLOBAL                              ((ENETC_GLOBAL_Type *)TMR0_GLOBAL_BASE)
  /** Array initializer of ENETC_GLOBAL peripheral base addresses */
  #define ENETC_GLOBAL_BASE_ADDRS                  { EMDIO_GLOBAL_BASE, ENETC0_GLOBAL_BASE, ENETC1_GLOBAL_BASE, SW0_GLOBAL_BASE, TMR0_GLOBAL_BASE }
  /** Array initializer of ENETC_GLOBAL peripheral base pointers */
  #define ENETC_GLOBAL_BASE_PTRS                   { EMDIO_GLOBAL, ENETC0_GLOBAL, ENETC1_GLOBAL, SW0_GLOBAL, TMR0_GLOBAL }
#endif

/*!
 * @}
 */ /* end of group ENETC_GLOBAL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ENETC_PCI_TYPE0 Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_PCI_TYPE0_Peripheral_Access_Layer ENETC_PCI_TYPE0 Peripheral Access Layer
 * @{
 */

/** ENETC_PCI_TYPE0 - Register Layout Typedef */
typedef struct {
  __I  uint32_t PCI_CFH_DID_VID;                   /**< PCI device ID and vendor ID register, offset: 0x0 */
  __IO uint16_t PCI_CFH_CMD;                       /**< PCI command register, offset: 0x4 */
  __I  uint16_t PCI_CFH_STAT;                      /**< PCI status register, offset: 0x6 */
  __I  uint32_t PCI_CFH_REVID_CLASSCODE;           /**< PCI revision ID and classcode register, offset: 0x8 */
  __IO uint8_t PCI_CFH_CL_SIZE;                    /**< PCI cache line size register, offset: 0xC */
       uint8_t RESERVED_0[1];
  __I  uint8_t PCI_CFH_HDR_TYPE;                   /**< PCI header type register, offset: 0xE */
       uint8_t RESERVED_1[1];
  __I  uint32_t PCI_CFH_BAR0;                      /**< PCI base address register 0, offset: 0x10 */
  __I  uint32_t PCI_CFH_BAR1;                      /**< PCI base address register 1, offset: 0x14 */
  __I  uint32_t PCI_CFH_BAR2;                      /**< PCI base address register 2, offset: 0x18 */
  __I  uint32_t PCI_CFH_BAR3;                      /**< PCI base address register 3, offset: 0x1C */
  __I  uint32_t PCI_CFH_BAR4;                      /**< PCI base address register 4, offset: 0x20 */
  __I  uint32_t PCI_CFH_BAR5;                      /**< PCI base address register 5, offset: 0x24 */
       uint8_t RESERVED_2[4];
  __I  uint16_t PCI_CFH_SUBSYS_VID;                /**< PCI subsystem vendor ID register, offset: 0x2C */
  __I  uint16_t PCI_CFH_SUBSYS_ID;                 /**< PCI subsystem ID register, offset: 0x2E */
       uint8_t RESERVED_3[4];
  __I  uint8_t PCI_CFH_CAP_PTR;                    /**< PCI capabilities pointer register, offset: 0x34 */
       uint8_t RESERVED_4[11];
  __I  uint16_t PCI_CFC_PCIE_CAP_LIST;             /**< PCI PCIe capabilities list register, offset: 0x40 */
  __I  uint16_t PCI_CFC_PCIE_CAP;                  /**< PCI PCIe capabilities register, offset: 0x42 */
  __I  uint32_t PCI_CFC_PCIE_DEV_CAP;              /**< PCI PCIe device capabilities register, offset: 0x44 */
  __IO uint16_t PCI_CFC_PCIE_DEV_CTL;              /**< PCI PCIe device control register, offset: 0x48 */
  __I  uint16_t PCI_CFC_PCIE_DEV_STAT;             /**< PCI PCIe device status register, offset: 0x4A */
       uint8_t RESERVED_5[24];
  __I  uint32_t PCI_CFC_PCIE_DEV_CAP2;             /**< PCI PCIe device capabilities 2 register, offset: 0x64 */
  __I  uint16_t PCI_CFC_PCIE_DEV_CTL2;             /**< PCI PCIe device control 2 register, offset: 0x68 */
       uint8_t RESERVED_6[22];
  __I  uint16_t PCI_CFC_MSIX_CAP_LIST;             /**< PCI MSI-X capabilities list register, offset: 0x80 */
  __IO uint16_t PCI_CFC_MSIX_MSG_CTL;              /**< PCI MSI-X message control register, offset: 0x82 */
  __I  uint32_t PCI_CFC_MSIX_TABLE_OFF_BIR;        /**< PCI MSI-X table offset/BIR register, offset: 0x84 */
  __I  uint32_t PCI_CFC_MSIX_PBA_OFF_BIR;          /**< PCI MSI-X PBA offset/BIR register, offset: 0x88 */
       uint8_t RESERVED_7[4];
  __I  uint16_t PCI_CFC_PCIPM_CAP_LIST;            /**< PCI PCI-PM capabilities list register, offset: 0x90 */
  __I  uint16_t PCI_CFC_PCIPM_CAP;                 /**< PCI PCI-PM capabilities register, offset: 0x92 */
  __IO uint16_t PCI_CFC_PCIPM_CTL_STAT;            /**< PCI PCI-PM control and status register, offset: 0x94 */
       uint8_t RESERVED_8[1];
       uint8_t PCI_CFC_PCIPM_DATA;                 /**< PCI PCI-PM capabilities data register, offset: 0x97 */
       uint8_t RESERVED_9[4];
  __I  uint16_t PCI_CFC_EA_CAP_LIST;               /**< PCI EA capabilities list register, offset: 0x9C */
  __I  uint16_t PCI_CFC_EA_CAP;                    /**< PCI EA capabilities register, offset: 0x9E */
  struct {                                         /* offset: 0xA0, array step: 0x10 */
    __I  uint32_t PCI_CFC_EA_PE_FMT;                 /**< PCI EA per-entry 0 format register..PCI EA per-entry 3 format register, array offset: 0xA0, array step: 0x10, irregular array, not all indices are valid */
    __I  uint32_t PCI_CFC_EA_PE_BASE;                /**< PCI EA per-entry 0 base register..PCI EA per-entry 3 base register, array offset: 0xA4, array step: 0x10, irregular array, not all indices are valid */
    __I  uint32_t PCI_CFC_EA_PE_MAXOFF;              /**< PCI EA per-entry 0 max offset register..PCI EA per-entry 3 max offset register, array offset: 0xA8, array step: 0x10, irregular array, not all indices are valid */
    __I  uint32_t PCI_CFC_EA_PE_EXT_BASE;            /**< PCI EA per-entry 0 extended base register..PCI EA per-entry 3 extended base register, array offset: 0xAC, array step: 0x10, irregular array, not all indices are valid */
  } NUM_EA[4];
       uint8_t RESERVED_10[32];
  __I  uint32_t PCIE_CFC_AER_EXT_CAP_HDR;          /**< PCIe AER extended capability header, offset: 0x100 */
  __IO uint32_t PCIE_CFC_AER_UCORR_ERR_STAT;       /**< PCIe AER uncorrectable error status register, offset: 0x104 */
  __IO uint32_t PCIE_CFC_AER_UCORR_ERR_MASK;       /**< PCIe AER uncorrectable error mask register, offset: 0x108 */
  __IO uint32_t PCIE_CFC_AER_UCORR_ERR_SEV;        /**< PCIe AER uncorrectable error severity register, offset: 0x10C */
  __IO uint32_t PCIE_CFC_AER_CORR_ERR_STAT;        /**< PCIe AER correctable error status register, offset: 0x110 */
  __IO uint32_t PCIE_CFC_AER_CORR_ERR_MASK;        /**< PCIe AER correctable error mask register, offset: 0x114 */
  __I  uint32_t PCIE_CFC_AER_CAP_CTL;              /**< PCIe AER capabilities and control register, offset: 0x118 */
       uint8_t RESERVED_11[20];
  __I  uint32_t PCIE_CFC_ACS_CAP_HDR;              /**< PCIe ACS capability header, offset: 0x130 */
  __I  uint16_t PCIE_CFC_ACS_CAP;                  /**< PCIe ACS capability register, offset: 0x134 */
  __I  uint16_t PCIE_CFC_ACS_CTL;                  /**< PCIe ACS control register, offset: 0x136 */
       uint8_t RESERVED_12[8];
  __I  uint32_t PCIE_CFC_RTR_CAP_HDR;              /**< PCIe readiness time reporting capability header, offset: 0x140 */
  __I  uint32_t PCIE_CFC_RTR_RTR1;                 /**< PCIe RTR readiness time reporting 1 register, offset: 0x144 */
  __I  uint32_t PCIE_CFC_RTR_RTR2;                 /**< PCIe RTR readiness time reporting 2 register, offset: 0x148 */
       uint8_t RESERVED_13[4];
  __I  uint32_t PCIE_CFC_SRIOV_CAP_HDR;            /**< PCIe SR-IOV capability header, offset: 0x150, not available in all instances (available on 2 out of 10) */
  __I  uint32_t PCIE_CFC_SRIOV_CAP;                /**< PCIe SR-IOV capability register, offset: 0x154, not available in all instances (available on 2 out of 10) */
  __IO uint16_t PCIE_CFC_SRIOV_CTL;                /**< PCIe SR-IOV control register, offset: 0x158, not available in all instances (available on 2 out of 10) */
  __I  uint16_t PCIE_CFC_SRIOV_STAT;               /**< PCIe SR-IOV status register, offset: 0x15A, not available in all instances (available on 2 out of 10) */
  __I  uint16_t PCIE_CFC_SRIOV_INIT_VFS;           /**< PCIe SR-IOV initial VFs register, offset: 0x15C, not available in all instances (available on 2 out of 10) */
  __I  uint16_t PCIE_CFC_SRIOV_TOTAL_VFS;          /**< PCIe SR-IOV total VFs register, offset: 0x15E, not available in all instances (available on 2 out of 10) */
  __IO uint16_t PCIE_CFC_SRIOV_NUM_VFS;            /**< PCIe SR-IOV num VFs register, offset: 0x160, not available in all instances (available on 2 out of 10) */
  __I  uint16_t PCIE_CFC_SRIOV_FUNC_DEP_LIST;      /**< PCIe SR-IOV function dependency list register, offset: 0x162, not available in all instances (available on 2 out of 10) */
  __I  uint16_t PCIE_CFC_SRIOV_FIRST_VF_OFF;       /**< PCIe SR-IOV first VF offset register, offset: 0x164, not available in all instances (available on 2 out of 10) */
  __I  uint16_t PCIE_CFC_SRIOV_VF_STRIDE;          /**< PCIe SR-IOV VF stride register, offset: 0x166, not available in all instances (available on 2 out of 10) */
       uint8_t RESERVED_14[2];
  __I  uint16_t PCIE_CFC_SRIOV_VF_DEV_ID;          /**< PCIe SR-IOV VF device ID register, offset: 0x16A, not available in all instances (available on 2 out of 10) */
  __I  uint32_t PCIE_CFC_SRIOV_SUP_PAGE_SIZES;     /**< PCIe SR-IOV supported page sizes ID register, offset: 0x16C, not available in all instances (available on 2 out of 10) */
  __I  uint32_t PCIE_CFC_SRIOV_SYS_PAGE_SIZE;      /**< PCIe SR-IOV system page size ID register, offset: 0x170, not available in all instances (available on 2 out of 10) */
  __I  uint32_t PCIE_CFC_VF_BAR[6];                /**< PCIe SR-IOV VF base address register 0..PCIe SR-IOV VF base address register 5, array offset: 0x174, array step: 0x4, not available in all instances (available on 2 out of 10) */
  __I  uint32_t PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF; /**< PCIe SR-IOV VF migration state array offset register, offset: 0x18C, not available in all instances (available on 2 out of 10) */
} ENETC_PCI_TYPE0_Type;

/* ----------------------------------------------------------------------------
   -- ENETC_PCI_TYPE0 Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_PCI_TYPE0_Register_Masks ENETC_PCI_TYPE0 Register Masks
 * @{
 */

/*! @name PCI_CFH_DID_VID - PCI device ID and vendor ID register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_MASK (0xFFFF0000U)
#define ENETC_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_SHIFT (16U)
#define ENETC_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_MASK)
/*! @} */

/*! @name PCI_CFH_CMD - PCI command register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_CMD_MEM_ACCESS_MASK (0x2U)
#define ENETC_PCI_TYPE0_PCI_CFH_CMD_MEM_ACCESS_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFH_CMD_MEM_ACCESS(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_CMD_MEM_ACCESS_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_CMD_MEM_ACCESS_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_MASK (0x4U)
#define ENETC_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_SHIFT (2U)
#define ENETC_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_MASK)
/*! @} */

/*! @name PCI_CFH_STAT - PCI status register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_MASK (0x10U)
#define ENETC_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_SHIFT (4U)
#define ENETC_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_MASK)
/*! @} */

/*! @name PCI_CFH_REVID_CLASSCODE - PCI revision ID and classcode register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_MASK (0xFFU)
#define ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_MASK (0xFFFFFF00U)
#define ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_SHIFT (8U)
#define ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_MASK)
/*! @} */

/*! @name PCI_CFH_CL_SIZE - PCI cache line size register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_MASK (0xFFU)
#define ENETC_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE(x) (((uint8_t)(((uint8_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_MASK)
/*! @} */

/*! @name PCI_CFH_HDR_TYPE - PCI header type register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_MASK (0x7FU)
#define ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE(x) (((uint8_t)(((uint8_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_MASK (0x80U)
#define ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_SHIFT (7U)
#define ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV(x) (((uint8_t)(((uint8_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_MASK)
/*! @} */

/*! @name PCI_CFH_BAR0 - PCI base address register 0 */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_MASK (0x6U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_ADDR_MASK   (0xFFFFFFF0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_ADDR_SHIFT  (4U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR0_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR0_ADDR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR0_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR1 - PCI base address register 1 */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_MASK (0x6U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_ADDR_MASK   (0xFFFFFFF0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_ADDR_SHIFT  (4U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR1_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR1_ADDR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR1_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR2 - PCI base address register 2 */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_MASK (0x6U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_ADDR_MASK   (0xFFFFFFF0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_ADDR_SHIFT  (4U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR2_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR2_ADDR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR2_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR3 - PCI base address register 3 */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_MASK (0x6U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_ADDR_MASK   (0xFFFFFFF0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_ADDR_SHIFT  (4U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR3_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR3_ADDR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR3_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR4 - PCI base address register 4 */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_MASK (0x6U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_ADDR_MASK   (0xFFFFFFF0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_ADDR_SHIFT  (4U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR4_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR4_ADDR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR4_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR5 - PCI base address register 5 */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_MASK (0x6U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_MASK)

#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_ADDR_MASK   (0xFFFFFFF0U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_ADDR_SHIFT  (4U)
#define ENETC_PCI_TYPE0_PCI_CFH_BAR5_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_BAR5_ADDR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_BAR5_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_SUBSYS_VID - PCI subsystem vendor ID register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_MASK)
/*! @} */

/*! @name PCI_CFH_SUBSYS_ID - PCI subsystem ID register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_MASK)
/*! @} */

/*! @name PCI_CFH_CAP_PTR - PCI capabilities pointer register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_MASK (0xFFU)
#define ENETC_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR(x) (((uint8_t)(((uint8_t)(x)) << ENETC_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_CAP_LIST - PCI PCIe capabilities list register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_MASK (0xFFU)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_MASK (0xFF00U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_SHIFT (8U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_CAP - PCI PCIe capabilities register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_MASK (0xFU)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_MASK (0xF0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_SHIFT (4U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_MASK (0x3E00U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_SHIFT (9U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CAP - PCI PCIe device capabilities register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_MASK (0x10000000U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_SHIFT (28U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CTL - PCI PCIe device control register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_MASK (0x8000U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_SHIFT (15U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_STAT - PCI PCIe device status register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_MASK (0x20U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_SHIFT (5U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CAP2 - PCI PCIe device capabilities 2 register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_MASK (0xFU)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_MASK (0x10U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_SHIFT (4U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CTL2 - PCI PCIe device control 2 register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_MASK (0xFU)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_MASK (0x10U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_SHIFT (4U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_CAP_LIST - PCI MSI-X capabilities list register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_MASK (0xFFU)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_MASK (0xFF00U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_SHIFT (8U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_MSG_CTL - PCI MSI-X message control register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_MASK (0x7FFU)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_MASK (0x4000U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_SHIFT (14U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_MASK (0x8000U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_SHIFT (15U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_TABLE_OFF_BIR - PCI MSI-X table offset/BIR register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_MASK (0x7U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_MASK (0xFFFFFFF8U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_PBA_OFF_BIR - PCI MSI-X PBA offset/BIR register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_MASK (0x7U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_MASK (0xFFFFFFF8U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_MASK)
/*! @} */

/*! @name PCI_CFC_PCIPM_CAP_LIST - PCI PCI-PM capabilities list register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_MASK (0xFFU)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_PTR_MASK (0xFF00U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_PTR_SHIFT (8U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_PTR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_PTR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_PCIPM_CAP - PCI PCI-PM capabilities register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_VERSION_MASK (0x7U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_VERSION_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_VERSION(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_VERSION_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_VERSION_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_PME_SUPPORT_MASK (0xF800U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_PME_SUPPORT_SHIFT (11U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_PME_SUPPORT(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_PME_SUPPORT_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CAP_PME_SUPPORT_MASK)
/*! @} */

/*! @name PCI_CFC_PCIPM_CTL_STAT - PCI PCI-PM control and status register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_MASK (0x3U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_EN_MASK (0x100U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_EN_SHIFT (8U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_EN_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_EN_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_STATUS_MASK (0x8000U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_STATUS_SHIFT (15U)
#define ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_STATUS(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_STATUS_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_PCIPM_CTL_STAT_PME_STATUS_MASK)
/*! @} */

/*! @name PCI_CFC_EA_CAP_LIST - PCI EA capabilities list register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_CAP_ID_MASK (0xFFU)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_NEXT_CAP_PTR_MASK (0xFF00U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_NEXT_CAP_PTR_SHIFT (8U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_NEXT_CAP_PTR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_NEXT_CAP_PTR_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_LIST_NEXT_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_EA_CAP - PCI EA capabilities register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_NUM_ENTRIES_MASK (0x3FU)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_NUM_ENTRIES_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_NUM_ENTRIES(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_NUM_ENTRIES_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_CAP_NUM_ENTRIES_MASK)
/*! @} */

/*! @name PCI_CFC_EA_PE_FMT - PCI EA per-entry 0 format register..PCI EA per-entry 3 format register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENTRY_SIZE_MASK (0x7U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENTRY_SIZE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENTRY_SIZE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENTRY_SIZE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENTRY_SIZE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_BEI_MASK (0xF0U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_BEI_SHIFT (4U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_BEI(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_BEI_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_BEI_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_PRIM_PROP_MASK (0xFF00U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_PRIM_PROP_SHIFT (8U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_PRIM_PROP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_PRIM_PROP_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_PRIM_PROP_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_SEC_PROP_MASK (0xFF0000U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_SEC_PROP_SHIFT (16U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_SEC_PROP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_SEC_PROP_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_SEC_PROP_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_WRITABLE_MASK (0x40000000U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_WRITABLE_SHIFT (30U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_WRITABLE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_WRITABLE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_WRITABLE_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENABLE_MASK (0x80000000U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENABLE_SHIFT (31U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENABLE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENABLE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_ENABLE_MASK)
/*! @} */

/* The count of ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT */
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_FMT_COUNT  (4U)

/*! @name PCI_CFC_EA_PE_BASE - PCI EA per-entry 0 base register..PCI EA per-entry 3 base register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_S_MASK (0x2U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_S_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_S(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_S_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_S_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_BASE_MASK (0xFFFFFFFCU)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_BASE_SHIFT (2U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_BASE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_BASE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_BASE_MASK)
/*! @} */

/* The count of ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE */
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_BASE_COUNT (4U)

/*! @name PCI_CFC_EA_PE_MAXOFF - PCI EA per-entry 0 max offset register..PCI EA per-entry 3 max offset register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_S_MASK (0x2U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_S_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_S(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_S_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_S_MASK)

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_MAX_OFFSET_MASK (0xFFFFFFFCU)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_MAX_OFFSET_SHIFT (2U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_MAX_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_MAX_OFFSET_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_MAX_OFFSET_MASK)
/*! @} */

/* The count of ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF */
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_MAXOFF_COUNT (4U)

/*! @name PCI_CFC_EA_PE_EXT_BASE - PCI EA per-entry 0 extended base register..PCI EA per-entry 3 extended base register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_EXT_BASE_BASE_MASK (0xFFFFFFFFU)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_EXT_BASE_BASE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_EXT_BASE_BASE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCI_CFC_EA_PE_EXT_BASE_BASE_SHIFT)) & ENETC_PCI_TYPE0_PCI_CFC_EA_PE_EXT_BASE_BASE_MASK)
/*! @} */

/* The count of ENETC_PCI_TYPE0_PCI_CFC_EA_PE_EXT_BASE */
#define ENETC_PCI_TYPE0_PCI_CFC_EA_PE_EXT_BASE_COUNT (4U)

/*! @name PCIE_CFC_AER_EXT_CAP_HDR - PCIe AER extended capability header */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_SHIFT (16U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_UCORR_ERR_STAT - PCIe AER uncorrectable error status register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_ACS_VIOLATION_STAT_MASK (0x200000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_ACS_VIOLATION_STAT_SHIFT (21U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_ACS_VIOLATION_STAT(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_ACS_VIOLATION_STAT_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_ACS_VIOLATION_STAT_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_MASK (0x400000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_SHIFT (22U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_UCORR_ERR_MASK - PCIe AER uncorrectable error mask register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_MASK (0x400000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_SHIFT (22U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_UCORR_ERR_SEV - PCIe AER uncorrectable error severity register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_MASK (0x400000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_SHIFT (22U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_CORR_ERR_STAT - PCIe AER correctable error status register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_MASK (0x4000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_SHIFT (14U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_CORR_ERR_MASK - PCIe AER correctable error mask register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_MASK (0x4000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_SHIFT (14U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_CAP_CTL - PCIe AER capabilities and control register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_MASK (0x1FU)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_MASK)
/*! @} */

/*! @name PCIE_CFC_ACS_CAP_HDR - PCIe ACS capability header */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_SHIFT (16U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_ACS_CAP - PCIe ACS capability register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_MASK (0x2U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_MASK (0x4U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_SHIFT (2U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_MASK (0x40U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_SHIFT (6U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_MASK)
/*! @} */

/*! @name PCIE_CFC_ACS_CTL - PCIe ACS control register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_MASK (0x2U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_MASK (0x4U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_SHIFT (2U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_MASK (0x40U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_SHIFT (6U)
#define ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_MASK)
/*! @} */

/*! @name PCIE_CFC_RTR_CAP_HDR - PCIe readiness time reporting capability header */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_SHIFT (16U)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_RTR_RTR1 - PCIe RTR readiness time reporting 1 register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_MASK (0xFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_SHIFT (0U)
/*! RESET_TIME - Reset Time */
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_MASK (0x80000000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_SHIFT (31U)
/*! VALID - Valid */
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_MASK)
/*! @} */

/*! @name PCIE_CFC_RTR_RTR2 - PCIe RTR readiness time reporting 2 register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_MASK (0xFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_SHIFT (0U)
/*! FLR_TIME - FLR Time */
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_MASK (0xFFF000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_SHIFT (12U)
/*! D3HOT_D0_TIME - D3 hot to D0 time */
#define ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_CAP_HDR - PCIe SR-IOV capability header */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_CAP_VER_SHIFT (16U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_CAP_VER_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_CAP_VER_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_CAP - PCIe SR-IOV capability register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_CAP_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_CAP_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_CAP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_CAP_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_CAP_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_ARI_CAP_HIER_PRSV_MASK (0x2U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_ARI_CAP_HIER_PRSV_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_ARI_CAP_HIER_PRSV(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_ARI_CAP_HIER_PRSV_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_ARI_CAP_HIER_PRSV_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_ING_MSG_NUM_MASK (0xFFE00000U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_ING_MSG_NUM_SHIFT (21U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_ING_MSG_NUM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_ING_MSG_NUM_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CAP_VF_MIGRATION_ING_MSG_NUM_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_CTL - PCIe SR-IOV control register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_ENABLE_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_ENABLE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_ENABLE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_ENABLE_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_ENABLE_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_ENABLE_MASK (0x2U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_ENABLE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_ENABLE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_ENABLE_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_ENABLE_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_INT_ENABLE_MASK (0x4U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_INT_ENABLE_SHIFT (2U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_INT_ENABLE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_INT_ENABLE_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MIGRATION_INT_ENABLE_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MSE_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MSE_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MSE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MSE_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_VF_MSE_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_ARI_CAP_HIERARCHY_MASK (0x10U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_ARI_CAP_HIERARCHY_SHIFT (4U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_ARI_CAP_HIERARCHY(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_ARI_CAP_HIERARCHY_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_ARI_CAP_HIERARCHY_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_D2_SUPPORT_MASK (0x400U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_D2_SUPPORT_SHIFT (10U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_D2_SUPPORT(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_D2_SUPPORT_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_D2_SUPPORT_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_PME_SUPPORT_MASK (0xF800U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_PME_SUPPORT_SHIFT (11U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_PME_SUPPORT(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_PME_SUPPORT_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_CTL_PME_SUPPORT_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_STAT - PCIe SR-IOV status register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_STAT_VF_MIGRATION_STATUS_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_STAT_VF_MIGRATION_STATUS_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_STAT_VF_MIGRATION_STATUS(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_STAT_VF_MIGRATION_STATUS_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_STAT_VF_MIGRATION_STATUS_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_INIT_VFS - PCIe SR-IOV initial VFs register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_INITIAL_VFS_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_INITIAL_VFS_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_INITIAL_VFS(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_INITIAL_VFS_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_INIT_VFS_INITIAL_VFS_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_TOTAL_VFS - PCIe SR-IOV total VFs register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_TOTAL_VFS_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_TOTAL_VFS_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_TOTAL_VFS(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_TOTAL_VFS_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_TOTAL_VFS_TOTAL_VFS_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_NUM_VFS - PCIe SR-IOV num VFs register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_NUM_VFS_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_NUM_VFS_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_NUM_VFS(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_NUM_VFS_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_NUM_VFS_NUM_VFS_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_FUNC_DEP_LIST - PCIe SR-IOV function dependency list register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_FUNC_DEP_LIST_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_FUNC_DEP_LIST_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_FUNC_DEP_LIST(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_FUNC_DEP_LIST_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FUNC_DEP_LIST_FUNC_DEP_LIST_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_FIRST_VF_OFF - PCIe SR-IOV first VF offset register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_FIRST_VF_OFFSET_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_FIRST_VF_OFFSET_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_FIRST_VF_OFFSET(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_FIRST_VF_OFFSET_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_FIRST_VF_OFF_FIRST_VF_OFFSET_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_VF_STRIDE - PCIe SR-IOV VF stride register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_VF_STRIDE_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_VF_STRIDE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_VF_STRIDE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_VF_STRIDE_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_STRIDE_VF_STRIDE_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_VF_DEV_ID - PCIe SR-IOV VF device ID register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_VF_DEVICE_ID_MASK (0xFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_VF_DEVICE_ID_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_VF_DEVICE_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_VF_DEVICE_ID_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_DEV_ID_VF_DEVICE_ID_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_SUP_PAGE_SIZES - PCIe SR-IOV supported page sizes ID register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_SUP_PAGE_SIZES_MASK (0xFFFFFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_SUP_PAGE_SIZES_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_SUP_PAGE_SIZES(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_SUP_PAGE_SIZES_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SUP_PAGE_SIZES_SUP_PAGE_SIZES_MASK)
/*! @} */

/*! @name PCIE_CFC_SRIOV_SYS_PAGE_SIZE - PCIe SR-IOV system page size ID register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_SYS_PAGE_SIZE_MASK (0xFFFFFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_SYS_PAGE_SIZE_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_SYS_PAGE_SIZE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_SYS_PAGE_SIZE_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_SYS_PAGE_SIZE_SYS_PAGE_SIZE_MASK)
/*! @} */

/*! @name PCIE_CFC_VF_BAR - PCIe SR-IOV VF base address register 0..PCIe SR-IOV VF base address register 5 */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_IO_IND_MASK (0x1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_IO_IND_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_IO_IND_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_IO_IND_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_TYPE_MASK (0x6U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_TYPE_SHIFT (1U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_TYPE_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_MEM_TYPE_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_PF_MEM_MASK (0x8U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_PF_MEM_SHIFT (3U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_PF_MEM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_PF_MEM_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_PF_MEM_MASK)

#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_ADDR_MASK (0xFFFFFFF0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_ADDR_SHIFT (4U)
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_ADDR_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_ADDR_MASK)
/*! @} */

/* The count of ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR */
#define ENETC_PCI_TYPE0_PCIE_CFC_VF_BAR_COUNT    (6U)

/*! @name PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF - PCIe SR-IOV VF migration state array offset register */
/*! @{ */

#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_VF_MIG_STATE_ARR_OFF_MASK (0xFFFFFFFFU)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_VF_MIG_STATE_ARR_OFF_SHIFT (0U)
#define ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_VF_MIG_STATE_ARR_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_VF_MIG_STATE_ARR_OFF_SHIFT)) & ENETC_PCI_TYPE0_PCIE_CFC_SRIOV_VF_MIG_STATE_ARR_OFF_VF_MIG_STATE_ARR_OFF_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ENETC_PCI_TYPE0_Register_Masks */


/* ENETC_PCI_TYPE0 - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral NETC_F0_PCI_HDR_TYPE0 base address */
  #define NETC_F0_PCI_HDR_TYPE0_BASE               (0x70000000u)
  /** Peripheral NETC_F0_PCI_HDR_TYPE0 base address */
  #define NETC_F0_PCI_HDR_TYPE0_BASE_NS            (0x60000000u)
  /** Peripheral NETC_F0_PCI_HDR_TYPE0 base pointer */
  #define NETC_F0_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F0_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F0_PCI_HDR_TYPE0 base pointer */
  #define NETC_F0_PCI_HDR_TYPE0_NS                 ((ENETC_PCI_TYPE0_Type *)NETC_F0_PCI_HDR_TYPE0_BASE_NS)
  /** Peripheral NETC_F1_PCI_HDR_TYPE0 base address */
  #define NETC_F1_PCI_HDR_TYPE0_BASE               (0x70001000u)
  /** Peripheral NETC_F1_PCI_HDR_TYPE0 base address */
  #define NETC_F1_PCI_HDR_TYPE0_BASE_NS            (0x60001000u)
  /** Peripheral NETC_F1_PCI_HDR_TYPE0 base pointer */
  #define NETC_F1_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F1_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F1_PCI_HDR_TYPE0 base pointer */
  #define NETC_F1_PCI_HDR_TYPE0_NS                 ((ENETC_PCI_TYPE0_Type *)NETC_F1_PCI_HDR_TYPE0_BASE_NS)
  /** Peripheral NETC_F2_PCI_HDR_TYPE0 base address */
  #define NETC_F2_PCI_HDR_TYPE0_BASE               (0x70002000u)
  /** Peripheral NETC_F2_PCI_HDR_TYPE0 base address */
  #define NETC_F2_PCI_HDR_TYPE0_BASE_NS            (0x60002000u)
  /** Peripheral NETC_F2_PCI_HDR_TYPE0 base pointer */
  #define NETC_F2_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F2_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F2_PCI_HDR_TYPE0 base pointer */
  #define NETC_F2_PCI_HDR_TYPE0_NS                 ((ENETC_PCI_TYPE0_Type *)NETC_F2_PCI_HDR_TYPE0_BASE_NS)
  /** Peripheral NETC_F3_PCI_HDR_TYPE0 base address */
  #define NETC_F3_PCI_HDR_TYPE0_BASE               (0x70003000u)
  /** Peripheral NETC_F3_PCI_HDR_TYPE0 base address */
  #define NETC_F3_PCI_HDR_TYPE0_BASE_NS            (0x60003000u)
  /** Peripheral NETC_F3_PCI_HDR_TYPE0 base pointer */
  #define NETC_F3_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F3_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F3_PCI_HDR_TYPE0 base pointer */
  #define NETC_F3_PCI_HDR_TYPE0_NS                 ((ENETC_PCI_TYPE0_Type *)NETC_F3_PCI_HDR_TYPE0_BASE_NS)
  /** Peripheral NETC_F4_PCI_HDR_TYPE0 base address */
  #define NETC_F4_PCI_HDR_TYPE0_BASE               (0x70004000u)
  /** Peripheral NETC_F4_PCI_HDR_TYPE0 base address */
  #define NETC_F4_PCI_HDR_TYPE0_BASE_NS            (0x60004000u)
  /** Peripheral NETC_F4_PCI_HDR_TYPE0 base pointer */
  #define NETC_F4_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F4_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F4_PCI_HDR_TYPE0 base pointer */
  #define NETC_F4_PCI_HDR_TYPE0_NS                 ((ENETC_PCI_TYPE0_Type *)NETC_F4_PCI_HDR_TYPE0_BASE_NS)
  /** Array initializer of ENETC_PCI_TYPE0 peripheral base addresses */
  #define ENETC_PCI_TYPE0_BASE_ADDRS               { NETC_F0_PCI_HDR_TYPE0_BASE, NETC_F1_PCI_HDR_TYPE0_BASE, NETC_F2_PCI_HDR_TYPE0_BASE, NETC_F3_PCI_HDR_TYPE0_BASE, NETC_F4_PCI_HDR_TYPE0_BASE }
  /** Array initializer of ENETC_PCI_TYPE0 peripheral base pointers */
  #define ENETC_PCI_TYPE0_BASE_PTRS                { NETC_F0_PCI_HDR_TYPE0, NETC_F1_PCI_HDR_TYPE0, NETC_F2_PCI_HDR_TYPE0, NETC_F3_PCI_HDR_TYPE0, NETC_F4_PCI_HDR_TYPE0 }
  /** Array initializer of ENETC_PCI_TYPE0 peripheral base addresses */
  #define ENETC_PCI_TYPE0_BASE_ADDRS_NS            { NETC_F0_PCI_HDR_TYPE0_BASE_NS, NETC_F1_PCI_HDR_TYPE0_BASE_NS, NETC_F2_PCI_HDR_TYPE0_BASE_NS, NETC_F3_PCI_HDR_TYPE0_BASE_NS, NETC_F4_PCI_HDR_TYPE0_BASE_NS }
  /** Array initializer of ENETC_PCI_TYPE0 peripheral base pointers */
  #define ENETC_PCI_TYPE0_BASE_PTRS_NS             { NETC_F0_PCI_HDR_TYPE0_NS, NETC_F1_PCI_HDR_TYPE0_NS, NETC_F2_PCI_HDR_TYPE0_NS, NETC_F3_PCI_HDR_TYPE0_NS, NETC_F4_PCI_HDR_TYPE0_NS }
#else
  /** Peripheral NETC_F0_PCI_HDR_TYPE0 base address */
  #define NETC_F0_PCI_HDR_TYPE0_BASE               (0x60000000u)
  /** Peripheral NETC_F0_PCI_HDR_TYPE0 base pointer */
  #define NETC_F0_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F0_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F1_PCI_HDR_TYPE0 base address */
  #define NETC_F1_PCI_HDR_TYPE0_BASE               (0x60001000u)
  /** Peripheral NETC_F1_PCI_HDR_TYPE0 base pointer */
  #define NETC_F1_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F1_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F2_PCI_HDR_TYPE0 base address */
  #define NETC_F2_PCI_HDR_TYPE0_BASE               (0x60002000u)
  /** Peripheral NETC_F2_PCI_HDR_TYPE0 base pointer */
  #define NETC_F2_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F2_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F3_PCI_HDR_TYPE0 base address */
  #define NETC_F3_PCI_HDR_TYPE0_BASE               (0x60003000u)
  /** Peripheral NETC_F3_PCI_HDR_TYPE0 base pointer */
  #define NETC_F3_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F3_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_F4_PCI_HDR_TYPE0 base address */
  #define NETC_F4_PCI_HDR_TYPE0_BASE               (0x60004000u)
  /** Peripheral NETC_F4_PCI_HDR_TYPE0 base pointer */
  #define NETC_F4_PCI_HDR_TYPE0                    ((ENETC_PCI_TYPE0_Type *)NETC_F4_PCI_HDR_TYPE0_BASE)
  /** Array initializer of ENETC_PCI_TYPE0 peripheral base addresses */
  #define ENETC_PCI_TYPE0_BASE_ADDRS               { NETC_F0_PCI_HDR_TYPE0_BASE, NETC_F1_PCI_HDR_TYPE0_BASE, NETC_F2_PCI_HDR_TYPE0_BASE, NETC_F3_PCI_HDR_TYPE0_BASE, NETC_F4_PCI_HDR_TYPE0_BASE }
  /** Array initializer of ENETC_PCI_TYPE0 peripheral base pointers */
  #define ENETC_PCI_TYPE0_BASE_PTRS                { NETC_F0_PCI_HDR_TYPE0, NETC_F1_PCI_HDR_TYPE0, NETC_F2_PCI_HDR_TYPE0, NETC_F3_PCI_HDR_TYPE0, NETC_F4_PCI_HDR_TYPE0 }
#endif

/*!
 * @}
 */ /* end of group ENETC_PCI_TYPE0_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ENETC_PF_EMDIO Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_PF_EMDIO_Peripheral_Access_Layer ENETC_PF_EMDIO Peripheral Access Layer
 * @{
 */

/** ENETC_PF_EMDIO - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[7168];
  __IO uint32_t EMDIO_CFG;                         /**< External MDIO configuration register, offset: 0x1C00 */
  __IO uint32_t EMDIO_CTL;                         /**< External MDIO interface control register, offset: 0x1C04 */
  __IO uint32_t EMDIO_DATA;                        /**< External MDIO interface data register, offset: 0x1C08 */
  __IO uint32_t EMDIO_ADDR;                        /**< External MDIO register address register, offset: 0x1C0C */
  __I  uint32_t EMDIO_STAT;                        /**< External MDIO status register, offset: 0x1C10 */
       uint8_t RESERVED_1[12];
  __IO uint32_t PHY_STATUS_CFG;                    /**< PHY status configuration register, offset: 0x1C20 */
  __IO uint32_t PHY_STATUS_CTL;                    /**< PHY status control register, offset: 0x1C24 */
  __I  uint32_t PHY_STATUS_DATA;                   /**< PHY status data register, offset: 0x1C28 */
  __IO uint32_t PHY_STATUS_ADDR;                   /**< PHY status register address register, offset: 0x1C2C */
  __IO uint32_t PHY_STATUS_EVENT;                  /**< PHY status event register, offset: 0x1C30 */
  __IO uint32_t PHY_STATUS_MASK;                   /**< PHY status mask register, offset: 0x1C34 */
       uint8_t RESERVED_2[8];
  __I  uint32_t MDIO_CFG;                          /**< MDIO configuration register, offset: 0x1C40 */
} ENETC_PF_EMDIO_Type;

/* ----------------------------------------------------------------------------
   -- ENETC_PF_EMDIO Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_PF_EMDIO_Register_Masks ENETC_PF_EMDIO Register Masks
 * @{
 */

/*! @name EMDIO_CFG - External MDIO configuration register */
/*! @{ */

#define ENETC_PF_EMDIO_EMDIO_CFG_BSY2_MASK       (0x1U)
#define ENETC_PF_EMDIO_EMDIO_CFG_BSY2_SHIFT      (0U)
/*! BSY2 - Busy 2 (same as bit 31)
 *  0b0..An MDIO transaction is not occurring; software may access other MDIO registers.
 *  0b1..An MDIO transaction is occurring.
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_BSY2(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_BSY2_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_BSY2_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_RD_ER_MASK (0x2U)
#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_RD_ER_SHIFT (1U)
/*! MDIO_RD_ER
 *  0b0..No error
 *  0b1..The last read transaction received no response from a PHY; any data read should be considered invalid
 *       (for example, the PHY address does not match any PHY available on the MDIO bus).
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_RD_ER(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_MDIO_RD_ER_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_MDIO_RD_ER_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_HOLD_MASK  (0x1CU)
#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_HOLD_SHIFT (2U)
/*! MDIO_HOLD
 *  0b000..1 NETC cycle
 *  0b001..3 NETC cycles
 *  0b010..5 NETC cycles (default - recommended value)
 *  0b011..7 NETC cycles
 *  0b100..9 NETC cycles
 *  0b101..11 NETC cycles
 *  0b110..13 NETC cycles
 *  0b111..15 NETC cycles
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_HOLD(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_MDIO_HOLD_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_MDIO_HOLD_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_PRE_DIS_MASK    (0x20U)
#define ENETC_PF_EMDIO_EMDIO_CFG_PRE_DIS_SHIFT   (5U)
/*! PRE_DIS
 *  0b0..Generation of MDIO preamble is enabled (default operation).
 *  0b1..Generation of MDIO preamble is disabled
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_PRE_DIS(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_PRE_DIS_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_PRE_DIS_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_ENC45_MASK      (0x40U)
#define ENETC_PF_EMDIO_EMDIO_CFG_ENC45_SHIFT     (6U)
/*! ENC45
 *  0b0..Clause 22 transactions are used.
 *  0b1..Clause 45 transactions are used.
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_ENC45(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_ENC45_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_ENC45_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_CLK_DIV_MASK (0xFF80U)
#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_CLK_DIV_SHIFT (7U)
/*! MDIO_CLK_DIV - MDIO Clock Divisor */
#define ENETC_PF_EMDIO_EMDIO_CFG_MDIO_CLK_DIV(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_MDIO_CLK_DIV_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_MDIO_CLK_DIV_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_WHOAMI_MASK     (0x70000U)
#define ENETC_PF_EMDIO_EMDIO_CFG_WHOAMI_SHIFT    (16U)
/*! WHOAMI - Returns the virtual port ID. */
#define ENETC_PF_EMDIO_EMDIO_CFG_WHOAMI(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_WHOAMI_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_WHOAMI_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_EHOLD_MASK      (0x400000U)
#define ENETC_PF_EMDIO_EMDIO_CFG_EHOLD_SHIFT     (22U)
/*! EHOLD - Extended HOLD
 *  0b0..Normal operation. MDIO hold time is specified in .
 *  0b1..Extended Operation
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_EHOLD(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_EHOLD_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_EHOLD_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_NEG_MASK        (0x800000U)
#define ENETC_PF_EMDIO_EMDIO_CFG_NEG_SHIFT       (23U)
/*! NEG
 *  0b0..Normal operation - positive edge
 *  0b1..MDIO is driven by master on MDC negative edge (default for external MDIOs)
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_NEG(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_NEG_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_NEG_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_ADDR_ERR_MASK   (0x10000000U)
#define ENETC_PF_EMDIO_EMDIO_CFG_ADDR_ERR_SHIFT  (28U)
/*! ADDR_ERR
 *  0b0..Normal
 *  0b1..Error. An access control violation has occurred. The request address used does not match the MDIO PHY's
 *       address (clause 22) or MDIO port address (clause 45) assigned.
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_ADDR_ERR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_ADDR_ERR_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_ADDR_ERR_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_CIM_MASK        (0x20000000U)
#define ENETC_PF_EMDIO_EMDIO_CFG_CIM_SHIFT       (29U)
/*! CIM
 *  0b0..Masked
 *  0b1..Enabled
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_CIM(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_CIM_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_CIM_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_CMP_MASK        (0x40000000U)
#define ENETC_PF_EMDIO_EMDIO_CFG_CMP_SHIFT       (30U)
/*! CMP - MDIO Command Completion
 *  0b0..An MDIO command completion did not occur.
 *  0b1..An MDIO command completion occurred.
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_CMP(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_CMP_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_CMP_MASK)

#define ENETC_PF_EMDIO_EMDIO_CFG_BSY1_MASK       (0x80000000U)
#define ENETC_PF_EMDIO_EMDIO_CFG_BSY1_SHIFT      (31U)
/*! BSY1 - Busy 1
 *  0b0..An MDIO transaction is not occurring; software may access other MDIO registers.
 *  0b1..An MDIO transaction is occurring.
 */
#define ENETC_PF_EMDIO_EMDIO_CFG_BSY1(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CFG_BSY1_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CFG_BSY1_MASK)
/*! @} */

/*! @name EMDIO_CTL - External MDIO interface control register */
/*! @{ */

#define ENETC_PF_EMDIO_EMDIO_CTL_DEV_ADDR_MASK   (0x1FU)
#define ENETC_PF_EMDIO_EMDIO_CTL_DEV_ADDR_SHIFT  (0U)
/*! DEV_ADDR - 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
#define ENETC_PF_EMDIO_EMDIO_CTL_DEV_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CTL_DEV_ADDR_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CTL_DEV_ADDR_MASK)

#define ENETC_PF_EMDIO_EMDIO_CTL_PORT_ADDR_MASK  (0x3E0U)
#define ENETC_PF_EMDIO_EMDIO_CTL_PORT_ADDR_SHIFT (5U)
/*! PORT_ADDR - 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
#define ENETC_PF_EMDIO_EMDIO_CTL_PORT_ADDR(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CTL_PORT_ADDR_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CTL_PORT_ADDR_MASK)

#define ENETC_PF_EMDIO_EMDIO_CTL_POST_INC_MASK   (0x4000U)
#define ENETC_PF_EMDIO_EMDIO_CTL_POST_INC_SHIFT  (14U)
/*! POST_INC - MDIO read with address post-increment initiation. Self-clearing once transaction is complete. */
#define ENETC_PF_EMDIO_EMDIO_CTL_POST_INC(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CTL_POST_INC_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CTL_POST_INC_MASK)

#define ENETC_PF_EMDIO_EMDIO_CTL_READ_MASK       (0x8000U)
#define ENETC_PF_EMDIO_EMDIO_CTL_READ_SHIFT      (15U)
/*! READ - MDIO read initiation. */
#define ENETC_PF_EMDIO_EMDIO_CTL_READ(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CTL_READ_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CTL_READ_MASK)

#define ENETC_PF_EMDIO_EMDIO_CTL_BSY_MASK        (0x80000000U)
#define ENETC_PF_EMDIO_EMDIO_CTL_BSY_SHIFT       (31U)
/*! BSY - MDIO busy */
#define ENETC_PF_EMDIO_EMDIO_CTL_BSY(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_CTL_BSY_SHIFT)) & ENETC_PF_EMDIO_EMDIO_CTL_BSY_MASK)
/*! @} */

/*! @name EMDIO_DATA - External MDIO interface data register */
/*! @{ */

#define ENETC_PF_EMDIO_EMDIO_DATA_MDIO_DATA_MASK (0xFFFFU)
#define ENETC_PF_EMDIO_EMDIO_DATA_MDIO_DATA_SHIFT (0U)
/*! MDIO_DATA - 16-bit MDIO data. */
#define ENETC_PF_EMDIO_EMDIO_DATA_MDIO_DATA(x)   (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_DATA_MDIO_DATA_SHIFT)) & ENETC_PF_EMDIO_EMDIO_DATA_MDIO_DATA_MASK)
/*! @} */

/*! @name EMDIO_ADDR - External MDIO register address register */
/*! @{ */

#define ENETC_PF_EMDIO_EMDIO_ADDR_REGADDR_MASK   (0xFFFFU)
#define ENETC_PF_EMDIO_EMDIO_ADDR_REGADDR_SHIFT  (0U)
/*! REGADDR - MDIO PHY register address. */
#define ENETC_PF_EMDIO_EMDIO_ADDR_REGADDR(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_ADDR_REGADDR_SHIFT)) & ENETC_PF_EMDIO_EMDIO_ADDR_REGADDR_MASK)
/*! @} */

/*! @name EMDIO_STAT - External MDIO status register */
/*! @{ */

#define ENETC_PF_EMDIO_EMDIO_STAT_BSY_MASK       (0x1U)
#define ENETC_PF_EMDIO_EMDIO_STAT_BSY_SHIFT      (0U)
/*! BSY - Global MDIO busy */
#define ENETC_PF_EMDIO_EMDIO_STAT_BSY(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_STAT_BSY_SHIFT)) & ENETC_PF_EMDIO_EMDIO_STAT_BSY_MASK)

#define ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_MASK  (0x1F00U)
#define ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_SHIFT (8U)
/*! WHT_LIST - PHY white list */
#define ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_SHIFT)) & ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_MASK)

#define ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_ENA_MASK (0x8000U)
#define ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_ENA_SHIFT (15U)
/*! WHT_LIST_ENA - PHY white list enable */
#define ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_ENA(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_ENA_SHIFT)) & ENETC_PF_EMDIO_EMDIO_STAT_WHT_LIST_ENA_MASK)

#define ENETC_PF_EMDIO_EMDIO_STAT_PORT_ID_MASK   (0x70000U)
#define ENETC_PF_EMDIO_EMDIO_STAT_PORT_ID_SHIFT  (16U)
/*! PORT_ID - Port ID */
#define ENETC_PF_EMDIO_EMDIO_STAT_PORT_ID(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_STAT_PORT_ID_SHIFT)) & ENETC_PF_EMDIO_EMDIO_STAT_PORT_ID_MASK)

#define ENETC_PF_EMDIO_EMDIO_STAT_REQ_TYPE_MASK  (0x80000U)
#define ENETC_PF_EMDIO_EMDIO_STAT_REQ_TYPE_SHIFT (19U)
/*! REQ_TYPE - Port ID */
#define ENETC_PF_EMDIO_EMDIO_STAT_REQ_TYPE(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_EMDIO_STAT_REQ_TYPE_SHIFT)) & ENETC_PF_EMDIO_EMDIO_STAT_REQ_TYPE_MASK)
/*! @} */

/*! @name PHY_STATUS_CFG - PHY status configuration register */
/*! @{ */

#define ENETC_PF_EMDIO_PHY_STATUS_CFG_BSY_MASK   (0x1U)
#define ENETC_PF_EMDIO_PHY_STATUS_CFG_BSY_SHIFT  (0U)
/*! BSY - MDIO busy */
#define ENETC_PF_EMDIO_PHY_STATUS_CFG_BSY(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_CFG_BSY_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_CFG_BSY_MASK)

#define ENETC_PF_EMDIO_PHY_STATUS_CFG_MDIO_RD_ER_MASK (0x2U)
#define ENETC_PF_EMDIO_PHY_STATUS_CFG_MDIO_RD_ER_SHIFT (1U)
/*! MDIO_RD_ER - MDIO read error */
#define ENETC_PF_EMDIO_PHY_STATUS_CFG_MDIO_RD_ER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_CFG_MDIO_RD_ER_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_CFG_MDIO_RD_ER_MASK)

#define ENETC_PF_EMDIO_PHY_STATUS_CFG_STATUS_INTERVAL_MASK (0xFFFF0000U)
#define ENETC_PF_EMDIO_PHY_STATUS_CFG_STATUS_INTERVAL_SHIFT (16U)
/*! STATUS_INTERVAL - PHY status read interval */
#define ENETC_PF_EMDIO_PHY_STATUS_CFG_STATUS_INTERVAL(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_CFG_STATUS_INTERVAL_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_CFG_STATUS_INTERVAL_MASK)
/*! @} */

/*! @name PHY_STATUS_CTL - PHY status control register */
/*! @{ */

#define ENETC_PF_EMDIO_PHY_STATUS_CTL_DEV_ADDR_MASK (0x1FU)
#define ENETC_PF_EMDIO_PHY_STATUS_CTL_DEV_ADDR_SHIFT (0U)
/*! DEV_ADDR - 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
#define ENETC_PF_EMDIO_PHY_STATUS_CTL_DEV_ADDR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_CTL_DEV_ADDR_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_CTL_DEV_ADDR_MASK)

#define ENETC_PF_EMDIO_PHY_STATUS_CTL_PORT_ADDR_MASK (0x3E0U)
#define ENETC_PF_EMDIO_PHY_STATUS_CTL_PORT_ADDR_SHIFT (5U)
/*! PORT_ADDR - 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
#define ENETC_PF_EMDIO_PHY_STATUS_CTL_PORT_ADDR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_CTL_PORT_ADDR_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_CTL_PORT_ADDR_MASK)
/*! @} */

/*! @name PHY_STATUS_DATA - PHY status data register */
/*! @{ */

#define ENETC_PF_EMDIO_PHY_STATUS_DATA_MDIO_DATA_MASK (0xFFFFU)
#define ENETC_PF_EMDIO_PHY_STATUS_DATA_MDIO_DATA_SHIFT (0U)
/*! MDIO_DATA - 16-bit MDIO data */
#define ENETC_PF_EMDIO_PHY_STATUS_DATA_MDIO_DATA(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_DATA_MDIO_DATA_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_DATA_MDIO_DATA_MASK)

#define ENETC_PF_EMDIO_PHY_STATUS_DATA_CURR_CNT_MASK (0xFFFF0000U)
#define ENETC_PF_EMDIO_PHY_STATUS_DATA_CURR_CNT_SHIFT (16U)
/*! CURR_CNT - Current count */
#define ENETC_PF_EMDIO_PHY_STATUS_DATA_CURR_CNT(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_DATA_CURR_CNT_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_DATA_CURR_CNT_MASK)
/*! @} */

/*! @name PHY_STATUS_ADDR - PHY status register address register */
/*! @{ */

#define ENETC_PF_EMDIO_PHY_STATUS_ADDR_REGADDR_MASK (0xFFFFU)
#define ENETC_PF_EMDIO_PHY_STATUS_ADDR_REGADDR_SHIFT (0U)
/*! REGADDR - MDIO PHY register address. Address of the register within the Clause 45 PHY device from which data is to be read. */
#define ENETC_PF_EMDIO_PHY_STATUS_ADDR_REGADDR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_ADDR_REGADDR_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_ADDR_REGADDR_MASK)
/*! @} */

/*! @name PHY_STATUS_EVENT - PHY status event register */
/*! @{ */

#define ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_HL_MASK (0xFFFFU)
#define ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_HL_SHIFT (0U)
/*! STATUS_EVENT_HL - Status event high-to-low. Set to 1 if a 1->0 transition on a corresponding data bit has occurred. Write 1 to clear. */
#define ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_HL(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_HL_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_HL_MASK)

#define ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_LH_MASK (0xFFFF0000U)
#define ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_LH_SHIFT (16U)
/*! STATUS_EVENT_LH - Status event low-to-high. Set to 1 if a 0->1 transition on a corresponding data bit has occurred. Write 1 to clear. */
#define ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_LH(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_LH_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_EVENT_STATUS_EVENT_LH_MASK)
/*! @} */

/*! @name PHY_STATUS_MASK - PHY status mask register */
/*! @{ */

#define ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_HL_MASK (0xFFFFU)
#define ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_HL_SHIFT (0U)
/*! STATUS_MASK_HL - Status high-to-low mask. If set to 1, assert an interrupt if the corresponding event bit is set. */
#define ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_HL(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_HL_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_HL_MASK)

#define ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_LH_MASK (0xFFFF0000U)
#define ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_LH_SHIFT (16U)
/*! STATUS_MASK_LH - Status mask low-to-high. If set to 1, assert an interrupt if the corresponding event bit is set. */
#define ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_LH(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_LH_SHIFT)) & ENETC_PF_EMDIO_PHY_STATUS_MASK_STATUS_MASK_LH_MASK)
/*! @} */

/*! @name MDIO_CFG - MDIO configuration register */
/*! @{ */

#define ENETC_PF_EMDIO_MDIO_CFG_MDIO_MODE_MASK   (0x10U)
#define ENETC_PF_EMDIO_MDIO_CFG_MDIO_MODE_SHIFT  (4U)
/*! MDIO_MODE - MDIO pin mode */
#define ENETC_PF_EMDIO_MDIO_CFG_MDIO_MODE(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_MDIO_CFG_MDIO_MODE_SHIFT)) & ENETC_PF_EMDIO_MDIO_CFG_MDIO_MODE_MASK)

#define ENETC_PF_EMDIO_MDIO_CFG_MDC_MODE_MASK    (0x20U)
#define ENETC_PF_EMDIO_MDIO_CFG_MDC_MODE_SHIFT   (5U)
/*! MDC_MODE - MDC pin mode */
#define ENETC_PF_EMDIO_MDIO_CFG_MDC_MODE(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_EMDIO_MDIO_CFG_MDC_MODE_SHIFT)) & ENETC_PF_EMDIO_MDIO_CFG_MDC_MODE_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ENETC_PF_EMDIO_Register_Masks */


/* ENETC_PF_EMDIO - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral EMDIO_BASE base address */
  #define EMDIO_BASE_BASE                          (0x70BA0000u)
  /** Peripheral EMDIO_BASE base address */
  #define EMDIO_BASE_BASE_NS                       (0x60BA0000u)
  /** Peripheral EMDIO_BASE base pointer */
  #define EMDIO_BASE                               ((ENETC_PF_EMDIO_Type *)EMDIO_BASE_BASE)
  /** Peripheral EMDIO_BASE base pointer */
  #define EMDIO_BASE_NS                            ((ENETC_PF_EMDIO_Type *)EMDIO_BASE_BASE_NS)
  /** Array initializer of ENETC_PF_EMDIO peripheral base addresses */
  #define ENETC_PF_EMDIO_BASE_ADDRS                { EMDIO_BASE_BASE }
  /** Array initializer of ENETC_PF_EMDIO peripheral base pointers */
  #define ENETC_PF_EMDIO_BASE_PTRS                 { EMDIO_BASE }
  /** Array initializer of ENETC_PF_EMDIO peripheral base addresses */
  #define ENETC_PF_EMDIO_BASE_ADDRS_NS             { EMDIO_BASE_BASE_NS }
  /** Array initializer of ENETC_PF_EMDIO peripheral base pointers */
  #define ENETC_PF_EMDIO_BASE_PTRS_NS              { EMDIO_BASE_NS }
#else
  /** Peripheral EMDIO_BASE base address */
  #define EMDIO_BASE_BASE                          (0x60BA0000u)
  /** Peripheral EMDIO_BASE base pointer */
  #define EMDIO_BASE                               ((ENETC_PF_EMDIO_Type *)EMDIO_BASE_BASE)
  /** Array initializer of ENETC_PF_EMDIO peripheral base addresses */
  #define ENETC_PF_EMDIO_BASE_ADDRS                { EMDIO_BASE_BASE }
  /** Array initializer of ENETC_PF_EMDIO peripheral base pointers */
  #define ENETC_PF_EMDIO_BASE_PTRS                 { EMDIO_BASE }
#endif

/*!
 * @}
 */ /* end of group ENETC_PF_EMDIO_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ENETC_PF_TMR Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_PF_TMR_Peripheral_Access_Layer ENETC_PF_TMR Peripheral Access Layer
 * @{
 */

/** ENETC_PF_TMR - Register Layout Typedef */
typedef struct {
  __I  uint32_t TMR_ID;                            /**< Module ID Register, offset: 0x0 */
       uint8_t RESERVED_0[4];
  __I  uint32_t TMR_CAPR;                          /**< Timer Capability Register, offset: 0x8 */
       uint8_t RESERVED_1[20];
  __I  uint32_t TMR_FRT_L;                         /**< Timer free running time low register, offset: 0x20 */
  __I  uint32_t TMR_FRT_H;                         /**< Timer free running time high register, offset: 0x24 */
  __I  uint32_t TMR_SRT_L;                         /**< Timer synchronous time low register, offset: 0x28 */
  __I  uint32_t TMR_SRT_H;                         /**< Timer synchronous time high register., offset: 0x2C */
  __I  uint32_t TMR_DEF_CNT_L;                     /**< Default ns timer counter low register, offset: 0x30 */
  __I  uint32_t TMR_DEF_CNT_H;                     /**< Default ns timer counter high register, offset: 0x34 */
       uint8_t RESERVED_2[72];
  __IO uint32_t TMR_CTRL;                          /**< Timer Control Register, offset: 0x80 */
  __IO uint32_t TMR_TEVENT;                        /**< Timer Event Register, offset: 0x84 */
  __IO uint32_t TMR_TEMASK;                        /**< Timer event mask register, offset: 0x88 */
       uint8_t RESERVED_3[8];
  __I  uint32_t TMR_STAT;                          /**< Timer status register, offset: 0x94 */
  __IO uint32_t TMR_CNT_L;                         /**< Timer counter low register, offset: 0x98 */
  __IO uint32_t TMR_CNT_H;                         /**< Timer counter high register, offset: 0x9C */
  __IO uint32_t TMR_ADD;                           /**< Timer addend register, offset: 0xA0 */
  __I  uint32_t TMR_ACC;                           /**< Timer accumulator register, offset: 0xA4 */
  __IO uint32_t TMR_PRSC;                          /**< Timer prescale register, offset: 0xA8 */
  __IO uint32_t TMR_ECTRL;                         /**< Extended timer control register, offset: 0xAC */
  __IO uint32_t TMROFF_L;                          /**< Timer offset low register, offset: 0xB0 */
  __IO uint32_t TMROFF_H;                          /**< Timer offset high register, offset: 0xB4 */
  struct {                                         /* offset: 0xB8, array step: 0x8 */
    __IO uint32_t TMR_ALARM_L;                       /**< Alarm 1 time comparator low register..Alarm 2 time comparator low register, array offset: 0xB8, array step: 0x8 */
    __IO uint32_t TMR_ALARM_H;                       /**< Alarm 1 time comparator high register..Alarm 2 time comparator high register, array offset: 0xBC, array step: 0x8 */
  } TMR_ALARMM[2];
       uint8_t RESERVED_4[4];
  __IO uint32_t TMR_ALARM_CTRL;                    /**< Timer Alarm Control Register, offset: 0xCC */
  __IO uint32_t TMR_FIPER[3];                      /**< Timer 1 fixed interval period register..Timer 3 fixed interval period register, array offset: 0xD0, array step: 0x4 */
  __IO uint32_t TMR_FIPER_CTRL;                    /**< Timer FIPER Control Register, offset: 0xDC */
  struct {                                         /* offset: 0xE0, array step: 0x8 */
    __I  uint32_t TMR_ETTS_L;                        /**< External trigger stamp register, array offset: 0xE0, array step: 0x8 */
    __I  uint32_t TMR_ETTS_H;                        /**< External trigger stamp register, array offset: 0xE4, array step: 0x8 */
  } TMR_ETTSN[2];
  __I  uint32_t TMR_CUR_TIME_L;                    /**< Timer current time low register, offset: 0xF0 */
  __I  uint32_t TMR_CUR_TIME_H;                    /**< Timer current time high register, offset: 0xF4 */
  __IO uint32_t TMR_PARAM;                         /**< Timer parameter register, offset: 0xF8 */
} ENETC_PF_TMR_Type;

/* ----------------------------------------------------------------------------
   -- ENETC_PF_TMR Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_PF_TMR_Register_Masks ENETC_PF_TMR Register Masks
 * @{
 */

/*! @name TMR_ID - Module ID Register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ID_REV_MN_MASK          (0xFFU)
#define ENETC_PF_TMR_TMR_ID_REV_MN_SHIFT         (0U)
/*! REV_MN - Minor revision */
#define ENETC_PF_TMR_TMR_ID_REV_MN(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ID_REV_MN_SHIFT)) & ENETC_PF_TMR_TMR_ID_REV_MN_MASK)

#define ENETC_PF_TMR_TMR_ID_REV_MJ_MASK          (0xFF00U)
#define ENETC_PF_TMR_TMR_ID_REV_MJ_SHIFT         (8U)
/*! REV_MJ - Major revision */
#define ENETC_PF_TMR_TMR_ID_REV_MJ(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ID_REV_MJ_SHIFT)) & ENETC_PF_TMR_TMR_ID_REV_MJ_MASK)

#define ENETC_PF_TMR_TMR_ID_TMR_ID_MASK          (0xFFFF0000U)
#define ENETC_PF_TMR_TMR_ID_TMR_ID_SHIFT         (16U)
/*! TMR_ID - Timer IP ID */
#define ENETC_PF_TMR_TMR_ID_TMR_ID(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ID_TMR_ID_SHIFT)) & ENETC_PF_TMR_TMR_ID_TMR_ID_MASK)
/*! @} */

/*! @name TMR_CAPR - Timer Capability Register */
/*! @{ */

#define ENETC_PF_TMR_TMR_CAPR_IEEE_1722_MASK     (0x1U)
#define ENETC_PF_TMR_TMR_CAPR_IEEE_1722_SHIFT    (0U)
#define ENETC_PF_TMR_TMR_CAPR_IEEE_1722(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CAPR_IEEE_1722_SHIFT)) & ENETC_PF_TMR_TMR_CAPR_IEEE_1722_MASK)

#define ENETC_PF_TMR_TMR_CAPR_ECADJ_MASK         (0x2U)
#define ENETC_PF_TMR_TMR_CAPR_ECADJ_SHIFT        (1U)
#define ENETC_PF_TMR_TMR_CAPR_ECADJ(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CAPR_ECADJ_SHIFT)) & ENETC_PF_TMR_TMR_CAPR_ECADJ_MASK)

#define ENETC_PF_TMR_TMR_CAPR_IEEE_8021AS_REV_MASK (0x4U)
#define ENETC_PF_TMR_TMR_CAPR_IEEE_8021AS_REV_SHIFT (2U)
#define ENETC_PF_TMR_TMR_CAPR_IEEE_8021AS_REV(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CAPR_IEEE_8021AS_REV_SHIFT)) & ENETC_PF_TMR_TMR_CAPR_IEEE_8021AS_REV_MASK)

#define ENETC_PF_TMR_TMR_CAPR_NUM_MSIX_MASK      (0x10000U)
#define ENETC_PF_TMR_TMR_CAPR_NUM_MSIX_SHIFT     (16U)
#define ENETC_PF_TMR_TMR_CAPR_NUM_MSIX(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CAPR_NUM_MSIX_SHIFT)) & ENETC_PF_TMR_TMR_CAPR_NUM_MSIX_MASK)
/*! @} */

/*! @name TMR_FRT_L - Timer free running time low register */
/*! @{ */

#define ENETC_PF_TMR_TMR_FRT_L_TMR_FRT_L_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_FRT_L_TMR_FRT_L_SHIFT   (0U)
#define ENETC_PF_TMR_TMR_FRT_L_TMR_FRT_L(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FRT_L_TMR_FRT_L_SHIFT)) & ENETC_PF_TMR_TMR_FRT_L_TMR_FRT_L_MASK)
/*! @} */

/*! @name TMR_FRT_H - Timer free running time high register */
/*! @{ */

#define ENETC_PF_TMR_TMR_FRT_H_TMR_FRT_H_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_FRT_H_TMR_FRT_H_SHIFT   (0U)
#define ENETC_PF_TMR_TMR_FRT_H_TMR_FRT_H(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FRT_H_TMR_FRT_H_SHIFT)) & ENETC_PF_TMR_TMR_FRT_H_TMR_FRT_H_MASK)
/*! @} */

/*! @name TMR_SRT_L - Timer synchronous time low register */
/*! @{ */

#define ENETC_PF_TMR_TMR_SRT_L_TMR_SRT_L_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_SRT_L_TMR_SRT_L_SHIFT   (0U)
#define ENETC_PF_TMR_TMR_SRT_L_TMR_SRT_L(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_SRT_L_TMR_SRT_L_SHIFT)) & ENETC_PF_TMR_TMR_SRT_L_TMR_SRT_L_MASK)
/*! @} */

/*! @name TMR_SRT_H - Timer synchronous time high register. */
/*! @{ */

#define ENETC_PF_TMR_TMR_SRT_H_TMR_SRT_H_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_SRT_H_TMR_SRT_H_SHIFT   (0U)
#define ENETC_PF_TMR_TMR_SRT_H_TMR_SRT_H(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_SRT_H_TMR_SRT_H_SHIFT)) & ENETC_PF_TMR_TMR_SRT_H_TMR_SRT_H_MASK)
/*! @} */

/*! @name TMR_DEF_CNT_L - Default ns timer counter low register */
/*! @{ */

#define ENETC_PF_TMR_TMR_DEF_CNT_L_TMR_DEF_CNT_L_MASK (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_DEF_CNT_L_TMR_DEF_CNT_L_SHIFT (0U)
#define ENETC_PF_TMR_TMR_DEF_CNT_L_TMR_DEF_CNT_L(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_DEF_CNT_L_TMR_DEF_CNT_L_SHIFT)) & ENETC_PF_TMR_TMR_DEF_CNT_L_TMR_DEF_CNT_L_MASK)
/*! @} */

/*! @name TMR_DEF_CNT_H - Default ns timer counter high register */
/*! @{ */

#define ENETC_PF_TMR_TMR_DEF_CNT_H_TMR_DEF_CNT_H_MASK (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_DEF_CNT_H_TMR_DEF_CNT_H_SHIFT (0U)
#define ENETC_PF_TMR_TMR_DEF_CNT_H_TMR_DEF_CNT_H(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_DEF_CNT_H_TMR_DEF_CNT_H_SHIFT)) & ENETC_PF_TMR_TMR_DEF_CNT_H_TMR_DEF_CNT_H_MASK)
/*! @} */

/*! @name TMR_CTRL - Timer Control Register */
/*! @{ */

#define ENETC_PF_TMR_TMR_CTRL_CK_SEL_MASK        (0x3U)
#define ENETC_PF_TMR_TMR_CTRL_CK_SEL_SHIFT       (0U)
/*! CK_SEL - 1588 timer reference clock source select */
#define ENETC_PF_TMR_TMR_CTRL_CK_SEL(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_CK_SEL_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_CK_SEL_MASK)

#define ENETC_PF_TMR_TMR_CTRL_TE_MASK            (0x4U)
#define ENETC_PF_TMR_TMR_CTRL_TE_SHIFT           (2U)
/*! TE - 1588 timer enable */
#define ENETC_PF_TMR_TMR_CTRL_TE(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_TE_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_TE_MASK)

#define ENETC_PF_TMR_TMR_CTRL_CIPH_MASK          (0x40U)
#define ENETC_PF_TMR_TMR_CTRL_CIPH_SHIFT         (6U)
/*! CIPH - External oscillator input clock phase */
#define ENETC_PF_TMR_TMR_CTRL_CIPH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_CIPH_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_CIPH_MASK)

#define ENETC_PF_TMR_TMR_CTRL_COPH_MASK          (0x80U)
#define ENETC_PF_TMR_TMR_CTRL_COPH_SHIFT         (7U)
/*! COPH - Generated clock (TMR_GCLK) output phase. */
#define ENETC_PF_TMR_TMR_CTRL_COPH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_COPH_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_COPH_MASK)

#define ENETC_PF_TMR_TMR_CTRL_ETEP1_MASK         (0x100U)
#define ENETC_PF_TMR_TMR_CTRL_ETEP1_SHIFT        (8U)
/*! ETEP1 - External trigger 1 edge polarity */
#define ENETC_PF_TMR_TMR_CTRL_ETEP1(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_ETEP1_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_ETEP1_MASK)

#define ENETC_PF_TMR_TMR_CTRL_ETEP2_MASK         (0x200U)
#define ENETC_PF_TMR_TMR_CTRL_ETEP2_SHIFT        (9U)
/*! ETEP2 - External trigger 2 edge polarity */
#define ENETC_PF_TMR_TMR_CTRL_ETEP2(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_ETEP2_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_ETEP2_MASK)

#define ENETC_PF_TMR_TMR_CTRL_COMP_MODE_MASK     (0x8000U)
#define ENETC_PF_TMR_TMR_CTRL_COMP_MODE_SHIFT    (15U)
/*! COMP_MODE - Mode bit to allow atomic writes to TCLK_PERIOD and TMR_ADD */
#define ENETC_PF_TMR_TMR_CTRL_COMP_MODE(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_COMP_MODE_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_COMP_MODE_MASK)

#define ENETC_PF_TMR_TMR_CTRL_TCLK_PERIOD_MASK   (0x3FF0000U)
#define ENETC_PF_TMR_TMR_CTRL_TCLK_PERIOD_SHIFT  (16U)
/*! TCLK_PERIOD - 1588 timer reference clock period */
#define ENETC_PF_TMR_TMR_CTRL_TCLK_PERIOD(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_TCLK_PERIOD_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_TCLK_PERIOD_MASK)

#define ENETC_PF_TMR_TMR_CTRL_PP2L_MASK          (0x4000000U)
#define ENETC_PF_TMR_TMR_CTRL_PP2L_SHIFT         (26U)
/*! PP2L - Fiper2 pulse loop back mode enabled */
#define ENETC_PF_TMR_TMR_CTRL_PP2L(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_PP2L_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_PP2L_MASK)

#define ENETC_PF_TMR_TMR_CTRL_PP1L_MASK          (0x8000000U)
#define ENETC_PF_TMR_TMR_CTRL_PP1L_SHIFT         (27U)
/*! PP1L - Fiper1 pulse loop back mode enabled */
#define ENETC_PF_TMR_TMR_CTRL_PP1L(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_PP1L_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_PP1L_MASK)

#define ENETC_PF_TMR_TMR_CTRL_FS_MASK            (0x10000000U)
#define ENETC_PF_TMR_TMR_CTRL_FS_SHIFT           (28U)
/*! FS - FIPER start indication */
#define ENETC_PF_TMR_TMR_CTRL_FS(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_FS_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_FS_MASK)

#define ENETC_PF_TMR_TMR_CTRL_SHADOW_DIS_MASK    (0x20000000U)
#define ENETC_PF_TMR_TMR_CTRL_SHADOW_DIS_SHIFT   (29U)
/*! SHADOW_DIS - shadow Register disable */
#define ENETC_PF_TMR_TMR_CTRL_SHADOW_DIS(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_SHADOW_DIS_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_SHADOW_DIS_MASK)

#define ENETC_PF_TMR_TMR_CTRL_ALM2P_MASK         (0x40000000U)
#define ENETC_PF_TMR_TMR_CTRL_ALM2P_SHIFT        (30U)
/*! ALM2P - Alarm2 output polarity */
#define ENETC_PF_TMR_TMR_CTRL_ALM2P(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_ALM2P_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_ALM2P_MASK)

#define ENETC_PF_TMR_TMR_CTRL_ALM1P_MASK         (0x80000000U)
#define ENETC_PF_TMR_TMR_CTRL_ALM1P_SHIFT        (31U)
/*! ALM1P - Alarm1 output polarity */
#define ENETC_PF_TMR_TMR_CTRL_ALM1P(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CTRL_ALM1P_SHIFT)) & ENETC_PF_TMR_TMR_CTRL_ALM1P_MASK)
/*! @} */

/*! @name TMR_TEVENT - Timer Event Register */
/*! @{ */

#define ENETC_PF_TMR_TMR_TEVENT_PP3EN_MASK       (0x20U)
#define ENETC_PF_TMR_TMR_TEVENT_PP3EN_SHIFT      (5U)
#define ENETC_PF_TMR_TMR_TEVENT_PP3EN(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_PP3EN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_PP3EN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_PP2EN_MASK       (0x40U)
#define ENETC_PF_TMR_TMR_TEVENT_PP2EN_SHIFT      (6U)
#define ENETC_PF_TMR_TMR_TEVENT_PP2EN(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_PP2EN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_PP2EN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_PP1EN_MASK       (0x80U)
#define ENETC_PF_TMR_TMR_TEVENT_PP1EN_SHIFT      (7U)
#define ENETC_PF_TMR_TMR_TEVENT_PP1EN(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_PP1EN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_PP1EN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ALM1EN_MASK      (0x10000U)
#define ENETC_PF_TMR_TMR_TEVENT_ALM1EN_SHIFT     (16U)
#define ENETC_PF_TMR_TMR_TEVENT_ALM1EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ALM1EN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ALM1EN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ALM2EN_MASK      (0x20000U)
#define ENETC_PF_TMR_TMR_TEVENT_ALM2EN_SHIFT     (17U)
#define ENETC_PF_TMR_TMR_TEVENT_ALM2EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ALM2EN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ALM2EN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ETS1_THREN_MASK  (0x100000U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS1_THREN_SHIFT (20U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS1_THREN(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ETS1_THREN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ETS1_THREN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ETS2_THREN_MASK  (0x200000U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS2_THREN_SHIFT (21U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS2_THREN(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ETS2_THREN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ETS2_THREN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ETS1EN_MASK      (0x1000000U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS1EN_SHIFT     (24U)
/*! ETS1EN - External trigger 1 new timestamp sample event available */
#define ENETC_PF_TMR_TMR_TEVENT_ETS1EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ETS1EN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ETS1EN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ETS2EN_MASK      (0x2000000U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS2EN_SHIFT     (25U)
/*! ETS2EN - External trigger 2 new timestamp sample event available */
#define ENETC_PF_TMR_TMR_TEVENT_ETS2EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ETS2EN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ETS2EN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ETS1_OVEN_MASK   (0x10000000U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS1_OVEN_SHIFT  (28U)
/*! ETS1_OVEN - External trigger 1 timestamp FIFO Overflow event occurred */
#define ENETC_PF_TMR_TMR_TEVENT_ETS1_OVEN(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ETS1_OVEN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ETS1_OVEN_MASK)

#define ENETC_PF_TMR_TMR_TEVENT_ETS2_OVEN_MASK   (0x20000000U)
#define ENETC_PF_TMR_TMR_TEVENT_ETS2_OVEN_SHIFT  (29U)
/*! ETS2_OVEN - External trigger 2 timestamp FIFO Overflow event occurred */
#define ENETC_PF_TMR_TMR_TEVENT_ETS2_OVEN(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEVENT_ETS2_OVEN_SHIFT)) & ENETC_PF_TMR_TMR_TEVENT_ETS2_OVEN_MASK)
/*! @} */

/*! @name TMR_TEMASK - Timer event mask register */
/*! @{ */

#define ENETC_PF_TMR_TMR_TEMASK_PP3EN_MASK       (0x20U)
#define ENETC_PF_TMR_TMR_TEMASK_PP3EN_SHIFT      (5U)
#define ENETC_PF_TMR_TMR_TEMASK_PP3EN(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_PP3EN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_PP3EN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_PP2EN_MASK       (0x40U)
#define ENETC_PF_TMR_TMR_TEMASK_PP2EN_SHIFT      (6U)
#define ENETC_PF_TMR_TMR_TEMASK_PP2EN(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_PP2EN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_PP2EN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_PP1EN_MASK       (0x80U)
#define ENETC_PF_TMR_TMR_TEMASK_PP1EN_SHIFT      (7U)
#define ENETC_PF_TMR_TMR_TEMASK_PP1EN(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_PP1EN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_PP1EN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ALM1EN_MASK      (0x10000U)
#define ENETC_PF_TMR_TMR_TEMASK_ALM1EN_SHIFT     (16U)
#define ENETC_PF_TMR_TMR_TEMASK_ALM1EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ALM1EN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ALM1EN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ALM2EN_MASK      (0x20000U)
#define ENETC_PF_TMR_TMR_TEMASK_ALM2EN_SHIFT     (17U)
#define ENETC_PF_TMR_TMR_TEMASK_ALM2EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ALM2EN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ALM2EN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ETS1_THREN_MASK  (0x100000U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS1_THREN_SHIFT (20U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS1_THREN(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ETS1_THREN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ETS1_THREN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ETS2_THREN_MASK  (0x200000U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS2_THREN_SHIFT (21U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS2_THREN(x)    (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ETS2_THREN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ETS2_THREN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ETS1EN_MASK      (0x1000000U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS1EN_SHIFT     (24U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS1EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ETS1EN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ETS1EN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ETS2EN_MASK      (0x2000000U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS2EN_SHIFT     (25U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS2EN(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ETS2EN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ETS2EN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ETS1_OVEN_MASK   (0x10000000U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS1_OVEN_SHIFT  (28U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS1_OVEN(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ETS1_OVEN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ETS1_OVEN_MASK)

#define ENETC_PF_TMR_TMR_TEMASK_ETS2_OVEN_MASK   (0x20000000U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS2_OVEN_SHIFT  (29U)
#define ENETC_PF_TMR_TMR_TEMASK_ETS2_OVEN(x)     (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_TEMASK_ETS2_OVEN_SHIFT)) & ENETC_PF_TMR_TMR_TEMASK_ETS2_OVEN_MASK)
/*! @} */

/*! @name TMR_STAT - Timer status register */
/*! @{ */

#define ENETC_PF_TMR_TMR_STAT_ETS1_VLD_MASK      (0x1000000U)
#define ENETC_PF_TMR_TMR_STAT_ETS1_VLD_SHIFT     (24U)
#define ENETC_PF_TMR_TMR_STAT_ETS1_VLD(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_STAT_ETS1_VLD_SHIFT)) & ENETC_PF_TMR_TMR_STAT_ETS1_VLD_MASK)

#define ENETC_PF_TMR_TMR_STAT_ETS2_VLD_MASK      (0x2000000U)
#define ENETC_PF_TMR_TMR_STAT_ETS2_VLD_SHIFT     (25U)
#define ENETC_PF_TMR_TMR_STAT_ETS2_VLD(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_STAT_ETS2_VLD_SHIFT)) & ENETC_PF_TMR_TMR_STAT_ETS2_VLD_MASK)

#define ENETC_PF_TMR_TMR_STAT_RCD_MASK           (0x80000000U)
#define ENETC_PF_TMR_TMR_STAT_RCD_SHIFT          (31U)
/*! RCD - Timer Reference Clock Detected
 *  0b0..Reference Clock has not been detected as active. Registers in timer clock domain are not allowed to be
 *       accessed; reads return 0, writes are ignored.
 *  0b1..Reference Clock has been detected as active. Registers in timer clock domain are allowed to be accessed.
 */
#define ENETC_PF_TMR_TMR_STAT_RCD(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_STAT_RCD_SHIFT)) & ENETC_PF_TMR_TMR_STAT_RCD_MASK)
/*! @} */

/*! @name TMR_CNT_L - Timer counter low register */
/*! @{ */

#define ENETC_PF_TMR_TMR_CNT_L_TMR_CNT_L_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_CNT_L_TMR_CNT_L_SHIFT   (0U)
/*! TMR_CNT_L - Timer counter register. */
#define ENETC_PF_TMR_TMR_CNT_L_TMR_CNT_L(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CNT_L_TMR_CNT_L_SHIFT)) & ENETC_PF_TMR_TMR_CNT_L_TMR_CNT_L_MASK)
/*! @} */

/*! @name TMR_CNT_H - Timer counter high register */
/*! @{ */

#define ENETC_PF_TMR_TMR_CNT_H_TMR_CNT_H_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_CNT_H_TMR_CNT_H_SHIFT   (0U)
/*! TMR_CNT_H - Timer counter register. */
#define ENETC_PF_TMR_TMR_CNT_H_TMR_CNT_H(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CNT_H_TMR_CNT_H_SHIFT)) & ENETC_PF_TMR_TMR_CNT_H_TMR_CNT_H_MASK)
/*! @} */

/*! @name TMR_ADD - Timer addend register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ADD_ADDEND_MASK         (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_ADD_ADDEND_SHIFT        (0U)
/*! ADDEND - Timer addend. */
#define ENETC_PF_TMR_TMR_ADD_ADDEND(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ADD_ADDEND_SHIFT)) & ENETC_PF_TMR_TMR_ADD_ADDEND_MASK)
/*! @} */

/*! @name TMR_ACC - Timer accumulator register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ACC_TMR_ACC_MASK        (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_ACC_TMR_ACC_SHIFT       (0U)
/*! TMR_ACC - 32-bit timer accumulator register */
#define ENETC_PF_TMR_TMR_ACC_TMR_ACC(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ACC_TMR_ACC_SHIFT)) & ENETC_PF_TMR_TMR_ACC_TMR_ACC_MASK)
/*! @} */

/*! @name TMR_PRSC - Timer prescale register */
/*! @{ */

#define ENETC_PF_TMR_TMR_PRSC_PRSC_OCK_MASK      (0xFFFFU)
#define ENETC_PF_TMR_TMR_PRSC_PRSC_OCK_SHIFT     (0U)
/*! PRSC_OCK - Output clock division prescale factor. */
#define ENETC_PF_TMR_TMR_PRSC_PRSC_OCK(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_PRSC_PRSC_OCK_SHIFT)) & ENETC_PF_TMR_TMR_PRSC_PRSC_OCK_MASK)
/*! @} */

/*! @name TMR_ECTRL - Extended timer control register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ECTRL_ETFF_THR_MASK     (0xFU)
#define ENETC_PF_TMR_TMR_ECTRL_ETFF_THR_SHIFT    (0U)
/*! ETFF_THR - External trigger FIFO threshold. */
#define ENETC_PF_TMR_TMR_ECTRL_ETFF_THR(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ECTRL_ETFF_THR_SHIFT)) & ENETC_PF_TMR_TMR_ECTRL_ETFF_THR_MASK)
/*! @} */

/*! @name TMROFF_L - Timer offset low register */
/*! @{ */

#define ENETC_PF_TMR_TMROFF_L_TMROFF_L_MASK      (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMROFF_L_TMROFF_L_SHIFT     (0U)
/*! TMROFF_L - Offset value of the clock counter. */
#define ENETC_PF_TMR_TMROFF_L_TMROFF_L(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMROFF_L_TMROFF_L_SHIFT)) & ENETC_PF_TMR_TMROFF_L_TMROFF_L_MASK)
/*! @} */

/*! @name TMROFF_H - Timer offset high register */
/*! @{ */

#define ENETC_PF_TMR_TMROFF_H_TMROFF_H_MASK      (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMROFF_H_TMROFF_H_SHIFT     (0U)
/*! TMROFF_H - Offset value of the clock counter. */
#define ENETC_PF_TMR_TMROFF_H_TMROFF_H(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMROFF_H_TMROFF_H_SHIFT)) & ENETC_PF_TMR_TMROFF_H_TMROFF_H_MASK)
/*! @} */

/*! @name TMR_ALARM_L - Alarm 1 time comparator low register..Alarm 2 time comparator low register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ALARM_L_ALARM_L_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_ALARM_L_ALARM_L_SHIFT   (0U)
/*! ALARM_L - Alarm time comparator register. */
#define ENETC_PF_TMR_TMR_ALARM_L_ALARM_L(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ALARM_L_ALARM_L_SHIFT)) & ENETC_PF_TMR_TMR_ALARM_L_ALARM_L_MASK)
/*! @} */

/* The count of ENETC_PF_TMR_TMR_ALARM_L */
#define ENETC_PF_TMR_TMR_ALARM_L_COUNT           (2U)

/*! @name TMR_ALARM_H - Alarm 1 time comparator high register..Alarm 2 time comparator high register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ALARM_H_ALARM_H_MASK    (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_ALARM_H_ALARM_H_SHIFT   (0U)
/*! ALARM_H - Alarm time comparator register. */
#define ENETC_PF_TMR_TMR_ALARM_H_ALARM_H(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ALARM_H_ALARM_H_SHIFT)) & ENETC_PF_TMR_TMR_ALARM_H_ALARM_H_MASK)
/*! @} */

/* The count of ENETC_PF_TMR_TMR_ALARM_H */
#define ENETC_PF_TMR_TMR_ALARM_H_COUNT           (2U)

/*! @name TMR_ALARM_CTRL - Timer Alarm Control Register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM1_PW_MASK (0x1FU)
#define ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM1_PW_SHIFT (0U)
/*! ALARM1_PW - ALARM 1 pulse width selector */
#define ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM1_PW(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM1_PW_SHIFT)) & ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM1_PW_MASK)

#define ENETC_PF_TMR_TMR_ALARM_CTRL_PG1_MASK     (0x80U)
#define ENETC_PF_TMR_TMR_ALARM_CTRL_PG1_SHIFT    (7U)
/*! PG1 - Alarm1 pulse generation time */
#define ENETC_PF_TMR_TMR_ALARM_CTRL_PG1(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ALARM_CTRL_PG1_SHIFT)) & ENETC_PF_TMR_TMR_ALARM_CTRL_PG1_MASK)

#define ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM2_PW_MASK (0x1F00U)
#define ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM2_PW_SHIFT (8U)
/*! ALARM2_PW - ALARM 2 pulse width selector */
#define ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM2_PW(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM2_PW_SHIFT)) & ENETC_PF_TMR_TMR_ALARM_CTRL_ALARM2_PW_MASK)

#define ENETC_PF_TMR_TMR_ALARM_CTRL_PG2_MASK     (0x8000U)
#define ENETC_PF_TMR_TMR_ALARM_CTRL_PG2_SHIFT    (15U)
/*! PG2 - Alarm2 pulse generation time */
#define ENETC_PF_TMR_TMR_ALARM_CTRL_PG2(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ALARM_CTRL_PG2_SHIFT)) & ENETC_PF_TMR_TMR_ALARM_CTRL_PG2_MASK)
/*! @} */

/*! @name TMR_FIPER - Timer 1 fixed interval period register..Timer 3 fixed interval period register */
/*! @{ */

#define ENETC_PF_TMR_TMR_FIPER_FIPER_MASK        (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_FIPER_FIPER_SHIFT       (0U)
/*! FIPER - Fixed Interval Pulse Period */
#define ENETC_PF_TMR_TMR_FIPER_FIPER(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_FIPER_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_FIPER_MASK)
/*! @} */

/* The count of ENETC_PF_TMR_TMR_FIPER */
#define ENETC_PF_TMR_TMR_FIPER_COUNT             (3U)

/*! @name TMR_FIPER_CTRL - Timer FIPER Control Register */
/*! @{ */

#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_PW_MASK (0x1FU)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_PW_SHIFT (0U)
/*! FIPER1_PW - FIPER 1 pulse width selection */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_PW(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_PW_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_PW_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG1_MASK     (0x40U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG1_SHIFT    (6U)
/*! PG1 - FIPER1 pulse generation select */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG1(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_PG1_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_PG1_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_DIS_MASK (0x80U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_DIS_SHIFT (7U)
/*! FIPER1_DIS - FIPER1 disable */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_DIS(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_DIS_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER1_DIS_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_PW_MASK (0x1F00U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_PW_SHIFT (8U)
/*! FIPER2_PW - FIPER 2 pulse width selection */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_PW(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_PW_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_PW_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG2_MASK     (0x4000U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG2_SHIFT    (14U)
/*! PG2 - FIPER2 pulse generation time */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG2(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_PG2_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_PG2_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_DIS_MASK (0x8000U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_DIS_SHIFT (15U)
/*! FIPER2_DIS - FIPER2 disable */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_DIS(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_DIS_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER2_DIS_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_PW_MASK (0x1F0000U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_PW_SHIFT (16U)
/*! FIPER3_PW - FIPER 3 pulse width selection */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_PW(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_PW_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_PW_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG3_MASK     (0x400000U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG3_SHIFT    (22U)
/*! PG3 - FIPER3 pulse generation time */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_PG3(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_PG3_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_PG3_MASK)

#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_DIS_MASK (0x800000U)
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_DIS_SHIFT (23U)
/*! FIPER3_DIS - FIPER3 disable */
#define ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_DIS(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_DIS_SHIFT)) & ENETC_PF_TMR_TMR_FIPER_CTRL_FIPER3_DIS_MASK)
/*! @} */

/*! @name TMR_ETTS_L - External trigger stamp register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ETTS_L_ETTS_L_MASK      (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_ETTS_L_ETTS_L_SHIFT     (0U)
#define ENETC_PF_TMR_TMR_ETTS_L_ETTS_L(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ETTS_L_ETTS_L_SHIFT)) & ENETC_PF_TMR_TMR_ETTS_L_ETTS_L_MASK)
/*! @} */

/* The count of ENETC_PF_TMR_TMR_ETTS_L */
#define ENETC_PF_TMR_TMR_ETTS_L_COUNT            (2U)

/*! @name TMR_ETTS_H - External trigger stamp register */
/*! @{ */

#define ENETC_PF_TMR_TMR_ETTS_H_ETTS_H_MASK      (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_ETTS_H_ETTS_H_SHIFT     (0U)
#define ENETC_PF_TMR_TMR_ETTS_H_ETTS_H(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_ETTS_H_ETTS_H_SHIFT)) & ENETC_PF_TMR_TMR_ETTS_H_ETTS_H_MASK)
/*! @} */

/* The count of ENETC_PF_TMR_TMR_ETTS_H */
#define ENETC_PF_TMR_TMR_ETTS_H_COUNT            (2U)

/*! @name TMR_CUR_TIME_L - Timer current time low register */
/*! @{ */

#define ENETC_PF_TMR_TMR_CUR_TIME_L_TMR_CUR_TIME_L_MASK (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_CUR_TIME_L_TMR_CUR_TIME_L_SHIFT (0U)
#define ENETC_PF_TMR_TMR_CUR_TIME_L_TMR_CUR_TIME_L(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CUR_TIME_L_TMR_CUR_TIME_L_SHIFT)) & ENETC_PF_TMR_TMR_CUR_TIME_L_TMR_CUR_TIME_L_MASK)
/*! @} */

/*! @name TMR_CUR_TIME_H - Timer current time high register */
/*! @{ */

#define ENETC_PF_TMR_TMR_CUR_TIME_H_TMR_CUR_TIME_H_MASK (0xFFFFFFFFU)
#define ENETC_PF_TMR_TMR_CUR_TIME_H_TMR_CUR_TIME_H_SHIFT (0U)
#define ENETC_PF_TMR_TMR_CUR_TIME_H_TMR_CUR_TIME_H(x) (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_CUR_TIME_H_TMR_CUR_TIME_H_SHIFT)) & ENETC_PF_TMR_TMR_CUR_TIME_H_TMR_CUR_TIME_H_MASK)
/*! @} */

/*! @name TMR_PARAM - Timer parameter register */
/*! @{ */

#define ENETC_PF_TMR_TMR_PARAM_SYNC_MASK         (0x1U)
#define ENETC_PF_TMR_TMR_PARAM_SYNC_SHIFT        (0U)
/*! SYNC - Timer synchronization */
#define ENETC_PF_TMR_TMR_PARAM_SYNC(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_PARAM_SYNC_SHIFT)) & ENETC_PF_TMR_TMR_PARAM_SYNC_MASK)

#define ENETC_PF_TMR_TMR_PARAM_PARAM_VAL_MASK    (0xFFFFFFFEU)
#define ENETC_PF_TMR_TMR_PARAM_PARAM_VAL_SHIFT   (1U)
/*! PARAM_VAL - User specific parameter values */
#define ENETC_PF_TMR_TMR_PARAM_PARAM_VAL(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_PF_TMR_TMR_PARAM_PARAM_VAL_SHIFT)) & ENETC_PF_TMR_TMR_PARAM_PARAM_VAL_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ENETC_PF_TMR_Register_Masks */


/* ENETC_PF_TMR - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral TMR0_BASE base address */
  #define TMR0_BASE_BASE                           (0x70B80000u)
  /** Peripheral TMR0_BASE base address */
  #define TMR0_BASE_BASE_NS                        (0x60B80000u)
  /** Peripheral TMR0_BASE base pointer */
  #define TMR0_BASE                                ((ENETC_PF_TMR_Type *)TMR0_BASE_BASE)
  /** Peripheral TMR0_BASE base pointer */
  #define TMR0_BASE_NS                             ((ENETC_PF_TMR_Type *)TMR0_BASE_BASE_NS)
  /** Array initializer of ENETC_PF_TMR peripheral base addresses */
  #define ENETC_PF_TMR_BASE_ADDRS                  { TMR0_BASE_BASE }
  /** Array initializer of ENETC_PF_TMR peripheral base pointers */
  #define ENETC_PF_TMR_BASE_PTRS                   { TMR0_BASE }
  /** Array initializer of ENETC_PF_TMR peripheral base addresses */
  #define ENETC_PF_TMR_BASE_ADDRS_NS               { TMR0_BASE_BASE_NS }
  /** Array initializer of ENETC_PF_TMR peripheral base pointers */
  #define ENETC_PF_TMR_BASE_PTRS_NS                { TMR0_BASE_NS }
#else
  /** Peripheral TMR0_BASE base address */
  #define TMR0_BASE_BASE                           (0x60B80000u)
  /** Peripheral TMR0_BASE base pointer */
  #define TMR0_BASE                                ((ENETC_PF_TMR_Type *)TMR0_BASE_BASE)
  /** Array initializer of ENETC_PF_TMR peripheral base addresses */
  #define ENETC_PF_TMR_BASE_ADDRS                  { TMR0_BASE_BASE }
  /** Array initializer of ENETC_PF_TMR peripheral base pointers */
  #define ENETC_PF_TMR_BASE_PTRS                   { TMR0_BASE }
#endif

/*!
 * @}
 */ /* end of group ENETC_PF_TMR_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ENETC_SI Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_SI_Peripheral_Access_Layer ENETC_SI Peripheral Access Layer
 * @{
 */

/** ENETC_SI - Register Layout Typedef */
typedef struct {
  __IO uint32_t SIMR;                              /**< Station interface mode register, offset: 0x0 */
  __I  uint32_t SISR;                              /**< Station interface status register, offset: 0x4 */
       uint8_t RESERVED_0[16];
  __I  uint32_t SICTR[2];                          /**< Station interface current time register 0..Station interface current time register 1, array offset: 0x18, array step: 0x4 */
  __I  uint32_t SIPCAPR0;                          /**< Station interface port capability register 0, offset: 0x20 */
  __I  uint32_t SIPCAPR1;                          /**< Station interface port capability register 1, offset: 0x24 */
       uint8_t RESERVED_1[8];
  __I  uint32_t SITSR;                             /**< Station interface timer status register, offset: 0x30, available only on: ENETC0_SI0, ENETC1_SI0 (missing on ENETC1_SI1) */
       uint8_t RESERVED_2[4];
  __IO uint32_t SIRBGCR;                           /**< Station interface receive BDR group control register, offset: 0x38 */
       uint8_t RESERVED_3[4];
  __IO uint32_t SIBCAR;                            /**< Station interface buffer cache attribute register, offset: 0x40 */
  __IO uint32_t SIMCAR;                            /**< Station interface message cache attribute register, offset: 0x44, available only on: ENETC1_SI0 (missing on ENETC0_SI0, ENETC1_SI1) */
  __IO uint32_t SICCAR;                            /**< Station interface command cache attribute register, offset: 0x48 */
       uint8_t RESERVED_4[52];
  __I  uint32_t SIPMAR0;                           /**< Station interface primary MAC address register 0, offset: 0x80 */
  __I  uint32_t SIPMAR1;                           /**< Station interface primary MAC address register 1, offset: 0x84 */
       uint8_t RESERVED_5[8];
  __I  uint32_t SICVLANR1;                         /**< Station interface custom VLAN register 1, offset: 0x90 */
  __I  uint32_t SICVLANR2;                         /**< Station interface custom VLAN register 2, offset: 0x94 */
       uint8_t RESERVED_6[104];
  __IO uint32_t SIVLANIPVMR0;                      /**< Station interface VLAN to IPV mapping register 0, offset: 0x100 */
  __IO uint32_t SIVLANIPVMR1;                      /**< Station interface VLAN to IPV mapping register 1, offset: 0x104 */
       uint8_t RESERVED_7[72];
  __IO uint32_t SIIPVBDRMR0;                       /**< Station interface IPV to ring mapping register, offset: 0x150 */
       uint8_t RESERVED_8[176];
  union {                                          /* offset: 0x204 */
    struct {                                         /* offset: 0x204 */
      __IO uint32_t PSIMSGRR;                          /**< Physical station interface message receive register, offset: 0x204, available only on: ENETC1_SI0 (missing on ENETC0_SI0, ENETC1_SI1) */
      __IO uint32_t PSIMSGSR;                          /**< Physical station interface message send register, offset: 0x208, available only on: ENETC1_SI0 (missing on ENETC0_SI0, ENETC1_SI1) */
           uint8_t RESERVED_0[4];
      struct {                                         /* offset: 0x210, array step: 0x8 */
        __IO uint32_t PSIVMSGRCVAR0;                     /**< PSI VSI 1 message receive address register 0, array offset: 0x210, array step: 0x8, available only on: ENETC1_SI0 (missing on ENETC0_SI0, ENETC1_SI1) */
        __IO uint32_t PSIVMSGRCVAR1;                     /**< PSI VSI 1 message receive address register 1, array offset: 0x214, array step: 0x8, available only on: ENETC1_SI0 (missing on ENETC0_SI0, ENETC1_SI1) */
      } VSI_NUM[1];
    } PSI_A;
    struct {                                         /* offset: 0x204 */
      __I  uint32_t VSIMSGSR;                          /**< Virtual station interface message send register, offset: 0x204, available only on: ENETC1_SI1 (missing on ENETC0_SI0, ENETC1_SI0) */
      __I  uint32_t VSIMSGRR;                          /**< Virtual station interface message receive register, offset: 0x208, available only on: ENETC1_SI1 (missing on ENETC0_SI0, ENETC1_SI0) */
           uint8_t RESERVED_0[4];
      __IO uint32_t VSIMSGSNDAR0;                      /**< Virtual station interface message send register 0, offset: 0x210, available only on: ENETC1_SI1 (missing on ENETC0_SI0, ENETC1_SI0) */
      __IO uint32_t VSIMSGSNDAR1;                      /**< Virtual station interface message send address register 1, offset: 0x214, available only on: ENETC1_SI1 (missing on ENETC0_SI0, ENETC1_SI0) */
    } VSI_A;
  };
       uint8_t RESERVED_9[232];
  __I  uint32_t SIROCT0;                           /**< Station interface receive octets counter (ifInOctets) 0, offset: 0x300 */
  __I  uint32_t SIROCT1;                           /**< Station interface receive octets counter (ifInOctets) 1, offset: 0x304 */
  __I  uint32_t SIRFRM0;                           /**< Station interface receive frame counter (aFrameReceivedOK) 0, offset: 0x308 */
  __I  uint32_t SIRFRM1;                           /**< Station interface receive frame counter (aFrameReceivedOK) 1, offset: 0x30C */
  __I  uint32_t SIRUCA0;                           /**< Station interface receive unicast frame counter (ifInUcastPkts) 0, offset: 0x310 */
  __I  uint32_t SIRUCA1;                           /**< Station interface receive unicast frame counter (ifInUcastPkts) 1, offset: 0x314 */
  __I  uint32_t SIRMCA0;                           /**< Station interface receive multicast frame counter (ifInMulticastPkts) 0, offset: 0x318 */
  __I  uint32_t SIRMCA1;                           /**< Station interface receive multicast frame counter (ifInMulticastPkts) 1, offset: 0x31C */
  __I  uint32_t SITOCT0;                           /**< Station interface transmit octets counter (ifOutOctets) 0, offset: 0x320 */
  __I  uint32_t SITOCT1;                           /**< Station interface transmit octets counter (ifOutOctets) 1, offset: 0x324 */
  __I  uint32_t SITFRM0;                           /**< Station interface transmit frame counter (aFrameTransmittedOK) 0, offset: 0x328 */
  __I  uint32_t SITFRM1;                           /**< Station interface transmit frame counter (aFrameTransmittedOK) 1, offset: 0x32C */
  __I  uint32_t SITUCA0;                           /**< Station interface transmit unicast frame counter (ifOutUcastPkts) 0, offset: 0x330 */
  __I  uint32_t SITUCA1;                           /**< Station interface transmit unicast frame counter (ifOutUcastPkts) 1, offset: 0x334 */
  __I  uint32_t SITMCA0;                           /**< Station interface transmit multicast frame counter (ifOutMulticastPkts) 0, offset: 0x338 */
  __I  uint32_t SITMCA1;                           /**< Station interface transmit multicast frame counter (ifOutMulticastPkts) 1, offset: 0x33C */
       uint8_t RESERVED_10[176];
  __I  uint32_t SIBLPR[2];                         /**< Station interface boot loader parameter register 0..Station interface boot loader parameter register 1, array offset: 0x3F0, array step: 0x4, available only on: ENETC1_SI1 (missing on ENETC0_SI0, ENETC1_SI0) */
       uint8_t RESERVED_11[1032];
  __IO uint32_t SICBDRMR;                          /**< Station interface command BDR mode register, offset: 0x800 */
  __I  uint32_t SICBDRSR;                          /**< Station interface command BDR status register, offset: 0x804 */
       uint8_t RESERVED_12[8];
  __IO uint32_t SICBDRBAR0;                        /**< Station interface command BDR base address register 0, offset: 0x810 */
  __IO uint32_t SICBDRBAR1;                        /**< Station interface command BDR base address register 1, offset: 0x814 */
  __IO uint32_t SICBDRPIR;                         /**< Station interface command BDR producer index register, offset: 0x818 */
  __IO uint32_t SICBDRCIR;                         /**< Station interface command BDR consumer index register, offset: 0x81C */
  __IO uint32_t SICBDRLENR;                        /**< Station interface command BDR length register, offset: 0x820 */
       uint8_t RESERVED_13[124];
  __IO uint32_t SICBDRIER;                         /**< Station interface command BDR interrupt enable register, offset: 0x8A0 */
  __IO uint32_t SICBDRIDR;                         /**< Station interface command BDR interrupt detect register, offset: 0x8A4 */
       uint8_t RESERVED_14[88];
  __I  uint32_t SICAPR0;                           /**< Station interface capability register 0, offset: 0x900 */
  __I  uint32_t SICAPR1;                           /**< Station interface capability register 1, offset: 0x904 */
  __I  uint32_t SICAPR2;                           /**< Station interface capability register 2, offset: 0x908 */
       uint8_t RESERVED_15[244];
  union {                                          /* offset: 0xA00 */
    struct {                                         /* offset: 0xA00 */
      __IO uint32_t PSIIER;                            /**< Physical station interface interrupt enable register, offset: 0xA00, available only on: ENETC1_SI0 (missing on ENETC0_SI0, ENETC1_SI1) */
           uint8_t RESERVED_0[4];
      __IO uint32_t PSIIDR;                            /**< Physical station interface interrupt detect register, offset: 0xA08, available only on: ENETC1_SI0 (missing on ENETC0_SI0, ENETC1_SI1) */
    } PSI;
    struct {                                         /* offset: 0xA00 */
      __IO uint32_t VSIIER;                            /**< Virtual station interface interrupt enable register, offset: 0xA00, available only on: ENETC1_SI1 (missing on ENETC0_SI0, ENETC1_SI0) */
           uint8_t RESERVED_0[4];
      __IO uint32_t VSIIDR;                            /**< Virtual station interface interrupt detect register, offset: 0xA08, available only on: ENETC1_SI1 (missing on ENETC0_SI0, ENETC1_SI0) */
    } VSI;
  };
       uint8_t RESERVED_16[12];
  __IO uint32_t SITXIDR0;                          /**< Station interface transmit interrupt detect register 0, offset: 0xA18 */
       uint8_t RESERVED_17[12];
  __IO uint32_t SIRXIDR0;                          /**< Station interface receive interrupt detect register 0, offset: 0xA28 */
       uint8_t RESERVED_18[4];
  __IO uint32_t SIMSIVR;                           /**< Station interface MSI-X vector register, offset: 0xA30, available only on: ENETC1_SI0, ENETC1_SI1 (missing on ENETC0_SI0) */
  __IO uint32_t SICMSIVR;                          /**< Station interface command MSI-X vector register, offset: 0xA34 */
       uint8_t RESERVED_19[8];
  __IO uint32_t SITMRIER;                          /**< Station interface timer interrupt enable register, offset: 0xA40 */
  __IO uint32_t SITMRIDR;                          /**< Station interface timer interrupt detect register, offset: 0xA44 */
       uint8_t RESERVED_20[4];
  __IO uint32_t SITMRMSIVR;                        /**< Station interface timer MSI-X vector register, offset: 0xA4C */
       uint8_t RESERVED_21[176];
  __IO uint32_t SIMSITRVR[10];                     /**< Station interface MSI-X transmit ring 0 vector register..Station interface MSI-X transmit ring 9 vector register, array offset: 0xB00, array step: 0x4, irregular array, not all indices are valid */
       uint8_t RESERVED_22[88];
  __IO uint32_t SIMSIRRVR[10];                     /**< Station interface MSI-X receive ring 0 vector register..Station interface MSI-X receive ring 9 vector register, array offset: 0xB80, array step: 0x4, irregular array, not all indices are valid */
       uint8_t RESERVED_23[600];
  __IO uint32_t SICMECR;                           /**< Station interface correctable memory error configuration register, offset: 0xE00 */
  __IO uint32_t SICMESR;                           /**< Station interface correctable memory error status register, offset: 0xE04 */
       uint8_t RESERVED_24[4];
  __I  uint32_t SICMECTR;                          /**< Station interface correctable memory error count register, offset: 0xE0C */
  __IO uint32_t SIUPECR;                           /**< Station interface uncorrectable programming error configuration register, offset: 0xE10 */
  __IO uint32_t SIUPESR;                           /**< Station interface uncorrectable programming error status register, offset: 0xE14 */
       uint8_t RESERVED_25[4];
  __I  uint32_t SIUPECTR;                          /**< Station interface uncorrectable programming error count register, offset: 0xE1C */
  __IO uint32_t SIUNSBECR;                         /**< Station interface uncorrectable non-fatal system bus error configuration register, offset: 0xE20 */
  __IO uint32_t SIUNSBESR;                         /**< Station interface uncorrectable non-fatal system bus error status register, offset: 0xE24 */
       uint8_t RESERVED_26[4];
  __I  uint32_t SIUNSBECTR;                        /**< Station interface uncorrectable non-fatal system bus error count register, offset: 0xE2C */
  __IO uint32_t SIUFSBECR;                         /**< Station interface uncorrectable fatal system bus error configuration register, offset: 0xE30 */
  __IO uint32_t SIUFSBESR;                         /**< Station interface uncorrectable fatal system bus error status register, offset: 0xE34 */
       uint8_t RESERVED_27[8];
  __IO uint32_t SIUNMECR;                          /**< Station interface uncorrectable non-fatal memory error configuration register, offset: 0xE40 */
  __IO uint32_t SIUNMESR0;                         /**< Station interface uncorrectable non-fatal memory error status register 0, offset: 0xE44 */
  __I  uint32_t SIUNMESR1;                         /**< Station interface uncorrectable non-fatal memory error status register 1, offset: 0xE48 */
  __I  uint32_t SIUNMECTR;                         /**< Station interface uncorrectable non-fatal memory error count register, offset: 0xE4C */
  __IO uint32_t SIUFMECR;                          /**< Station interface uncorrectable fatal memory error configuration register, offset: 0xE50 */
  __IO uint32_t SIUFMESR0;                         /**< Station interface uncorrectable fatal memory error status register 0, offset: 0xE54 */
  __I  uint32_t SIUFMESR1;                         /**< Station interface uncorrectable fatal memory error status register 1, offset: 0xE58 */
       uint8_t RESERVED_28[420];
  __I  uint32_t SIMAFTCAPR;                        /**< Station interface MAC address filter table capability register, offset: 0x1000 */
       uint8_t RESERVED_29[252];
  __I  uint32_t SIVFTCAPR;                         /**< Station interface VLAN filter table capability register, offset: 0x1100 */
       uint8_t RESERVED_30[28412];
  struct {                                         /* offset: 0x8000, array step: 0x200 */
    __IO uint32_t TBMR;                              /**< Tx BDR 0 mode register..Tx BDR 9 mode register, array offset: 0x8000, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t TBSR;                              /**< Tx BDR 0 status register..Tx BDR 9 status register, array offset: 0x8004, array step: 0x200, irregular array, not all indices are valid */
         uint8_t RESERVED_0[8];
    __IO uint32_t TBBAR0;                            /**< Tx BDR 0 base address register 0..Tx BDR 9 base address register 0, array offset: 0x8010, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t TBBAR1;                            /**< Tx BDR 0 base address register 1..Tx BDR 9 base address register 1, array offset: 0x8014, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t TBPIR;                             /**< Tx BDR 0 producer index register..Tx BDR 9 producer index register, array offset: 0x8018, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t TBCIR;                             /**< Tx BDR 0 consumer index register..Tx BDR 9 consumer index register, array offset: 0x801C, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t TBLENR;                            /**< Tx BDR 0 length register..Tx BDR 9 length register, array offset: 0x8020, array step: 0x200, irregular array, not all indices are valid */
         uint8_t RESERVED_1[124];
    __IO uint32_t TBIER;                             /**< Tx BDR 0 interrupt enable register..Tx BDR 9 interrupt enable register, array offset: 0x80A0, array step: 0x200, irregular array, not all indices are valid */
    __I  uint32_t TBIDR;                             /**< Tx BDR 0 interrupt detect register..Tx BDR 9 interrupt detect register, array offset: 0x80A4, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t TBICR0;                            /**< Tx BDR 0 interrupt coalescing register 0..Tx BDR 9 interrupt coalescing register 0, array offset: 0x80A8, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t TBICR1;                            /**< Tx BDR 0 interrupt coalescing register 1..Tx BDR 9 interrupt coalescing register 1, array offset: 0x80AC, array step: 0x200, irregular array, not all indices are valid */
         uint8_t RESERVED_2[80];
    __IO uint32_t RBMR;                              /**< Rx BDR 0 mode register..Rx BDR 9 mode register, array offset: 0x8100, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBSR;                              /**< Rx BDR 0 status register..Rx BDR 9 status register, array offset: 0x8104, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBBSR;                             /**< Rx BDR 0 buffer size register..Rx BDR 9 buffer size register, array offset: 0x8108, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBCIR;                             /**< Rx BDR 0 consumer index register..Rx BDR 9 consumer index register, array offset: 0x810C, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBBAR0;                            /**< Rx BDR 0 base address register 0..Rx BDR 9 base address register 0, array offset: 0x8110, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBBAR1;                            /**< Rx BDR 0 base address register 1..Rx BDR 9 base address register 1, array offset: 0x8114, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBPIR;                             /**< Rx BDR 0 producer index register..Rx BDR 9 producer index register, array offset: 0x8118, array step: 0x200, irregular array, not all indices are valid */
         uint8_t RESERVED_3[4];
    __IO uint32_t RBLENR;                            /**< Rx BDR 0 length register..Rx BDR 9 length register, array offset: 0x8120, array step: 0x200, irregular array, not all indices are valid */
         uint8_t RESERVED_4[92];
    __I  uint32_t RBDCR;                             /**< Rx BDR 0 drop count register..Rx BDR 9 drop count register, array offset: 0x8180, array step: 0x200, irregular array, not all indices are valid */
         uint8_t RESERVED_5[28];
    __IO uint32_t RBIER;                             /**< Rx BDR 0 interrupt enable register..Rx BDR 9 interrupt enable register, array offset: 0x81A0, array step: 0x200, irregular array, not all indices are valid */
    __I  uint32_t RBIDR;                             /**< Rx BDR 0 interrupt detect register..Rx BDR 9 interrupt detect register, array offset: 0x81A4, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBICR0;                            /**< Rx BDR 0 interrupt coalescing register 0..Rx BDR 9 interrupt coalescing register 0, array offset: 0x81A8, array step: 0x200, irregular array, not all indices are valid */
    __IO uint32_t RBICR1;                            /**< Rx BDR 0 interrupt coalescing register 1..Rx BDR 9 interrupt coalescing register 1, array offset: 0x81AC, array step: 0x200, irregular array, not all indices are valid */
         uint8_t RESERVED_6[80];
  } BDR[10];
} ENETC_SI_Type;

/* ----------------------------------------------------------------------------
   -- ENETC_SI Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_SI_Register_Masks ENETC_SI Register Masks
 * @{
 */

/*! @name SIMR - Station interface mode register */
/*! @{ */

#define ENETC_SI_SIMR_RSSE_MASK                  (0x1U)
#define ENETC_SI_SIMR_RSSE_SHIFT                 (0U)
#define ENETC_SI_SIMR_RSSE(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMR_RSSE_SHIFT)) & ENETC_SI_SIMR_RSSE_MASK)

#define ENETC_SI_SIMR_RNUM_MASK                  (0x2U)
#define ENETC_SI_SIMR_RNUM_SHIFT                 (1U)
#define ENETC_SI_SIMR_RNUM(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMR_RNUM_SHIFT)) & ENETC_SI_SIMR_RNUM_MASK)

#define ENETC_SI_SIMR_RNMM_MASK                  (0x4U)
#define ENETC_SI_SIMR_RNMM_SHIFT                 (2U)
#define ENETC_SI_SIMR_RNMM(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMR_RNMM_SHIFT)) & ENETC_SI_SIMR_RNMM_MASK)

#define ENETC_SI_SIMR_RNBM_MASK                  (0x8U)
#define ENETC_SI_SIMR_RNBM_SHIFT                 (3U)
#define ENETC_SI_SIMR_RNBM(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMR_RNBM_SHIFT)) & ENETC_SI_SIMR_RNBM_MASK)

#define ENETC_SI_SIMR_V2IPVE_MASK                (0x10U)
#define ENETC_SI_SIMR_V2IPVE_SHIFT               (4U)
#define ENETC_SI_SIMR_V2IPVE(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMR_V2IPVE_SHIFT)) & ENETC_SI_SIMR_V2IPVE_MASK)

#define ENETC_SI_SIMR_DEFAULT_RX_GROUP_MASK      (0x10000U)
#define ENETC_SI_SIMR_DEFAULT_RX_GROUP_SHIFT     (16U)
#define ENETC_SI_SIMR_DEFAULT_RX_GROUP(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMR_DEFAULT_RX_GROUP_SHIFT)) & ENETC_SI_SIMR_DEFAULT_RX_GROUP_MASK)

#define ENETC_SI_SIMR_EN_MASK                    (0x80000000U)
#define ENETC_SI_SIMR_EN_SHIFT                   (31U)
#define ENETC_SI_SIMR_EN(x)                      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMR_EN_SHIFT)) & ENETC_SI_SIMR_EN_MASK)
/*! @} */

/*! @name SISR - Station interface status register */
/*! @{ */

#define ENETC_SI_SISR_TX_BUSY_MASK               (0x1U)
#define ENETC_SI_SISR_TX_BUSY_SHIFT              (0U)
#define ENETC_SI_SISR_TX_BUSY(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SISR_TX_BUSY_SHIFT)) & ENETC_SI_SISR_TX_BUSY_MASK)

#define ENETC_SI_SISR_MAC_UP_MASK                (0x2U)
#define ENETC_SI_SISR_MAC_UP_SHIFT               (1U)
#define ENETC_SI_SISR_MAC_UP(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SISR_MAC_UP_SHIFT)) & ENETC_SI_SISR_MAC_UP_MASK)

#define ENETC_SI_SISR_MAC_MP_MASK                (0x4U)
#define ENETC_SI_SISR_MAC_MP_SHIFT               (2U)
#define ENETC_SI_SISR_MAC_MP(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SISR_MAC_MP_SHIFT)) & ENETC_SI_SISR_MAC_MP_MASK)

#define ENETC_SI_SISR_VLAN_P_MASK                (0x8U)
#define ENETC_SI_SISR_VLAN_P_SHIFT               (3U)
#define ENETC_SI_SISR_VLAN_P(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SISR_VLAN_P_SHIFT)) & ENETC_SI_SISR_VLAN_P_MASK)

#define ENETC_SI_SISR_VLAN_UTA_MASK              (0x10U)
#define ENETC_SI_SISR_VLAN_UTA_SHIFT             (4U)
#define ENETC_SI_SISR_VLAN_UTA(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SISR_VLAN_UTA_SHIFT)) & ENETC_SI_SISR_VLAN_UTA_MASK)
/*! @} */

/*! @name SICTR - Station interface current time register 0..Station interface current time register 1 */
/*! @{ */

#define ENETC_SI_SICTR_CURR_TIME_MASK            (0xFFFFFFFFU)
#define ENETC_SI_SICTR_CURR_TIME_SHIFT           (0U)
#define ENETC_SI_SICTR_CURR_TIME(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICTR_CURR_TIME_SHIFT)) & ENETC_SI_SICTR_CURR_TIME_MASK)
/*! @} */

/* The count of ENETC_SI_SICTR */
#define ENETC_SI_SICTR_COUNT                     (2U)

/*! @name SIPCAPR0 - Station interface port capability register 0 */
/*! @{ */

#define ENETC_SI_SIPCAPR0_RFS_MASK               (0x4U)
#define ENETC_SI_SIPCAPR0_RFS_SHIFT              (2U)
/*! RFS - Receive Flow Steering
 *  0b0..Not supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_RFS(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_RFS_SHIFT)) & ENETC_SI_SIPCAPR0_RFS_MASK)

#define ENETC_SI_SIPCAPR0_FP_MASK                (0x8U)
#define ENETC_SI_SIPCAPR0_FP_SHIFT               (3U)
/*! FP - Frame Preemption
 *  0b0..Not Supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_FP(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_FP_SHIFT)) & ENETC_SI_SIPCAPR0_FP_MASK)

#define ENETC_SI_SIPCAPR0_TGS_MASK               (0x10U)
#define ENETC_SI_SIPCAPR0_TGS_SHIFT              (4U)
/*! TGS - Time Gate Scheduling
 *  0b0..Not supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_TGS(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_TGS_SHIFT)) & ENETC_SI_SIPCAPR0_TGS_MASK)

#define ENETC_SI_SIPCAPR0_TSD_MASK               (0x20U)
#define ENETC_SI_SIPCAPR0_TSD_SHIFT              (5U)
/*! TSD - Time Specific Departure
 *  0b0..Not Supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_TSD(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_TSD_SHIFT)) & ENETC_SI_SIPCAPR0_TSD_MASK)

#define ENETC_SI_SIPCAPR0_CBS_MASK               (0x40U)
#define ENETC_SI_SIPCAPR0_CBS_SHIFT              (6U)
/*! CBS - Credit Based Shaping (CBS)
 *  0b0..Not Supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_CBS(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_CBS_SHIFT)) & ENETC_SI_SIPCAPR0_CBS_MASK)

#define ENETC_SI_SIPCAPR0_RSS_MASK               (0x100U)
#define ENETC_SI_SIPCAPR0_RSS_SHIFT              (8U)
/*! RSS - Receive Side Scaling
 *  0b0..Not supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_RSS(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_RSS_SHIFT)) & ENETC_SI_SIPCAPR0_RSS_MASK)

#define ENETC_SI_SIPCAPR0_PSFP_MASK              (0x200U)
#define ENETC_SI_SIPCAPR0_PSFP_SHIFT             (9U)
/*! PSFP - Per-Stream Filtering and Policing (IEEE 802.1Qci)
 *  0b0..Not Supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_PSFP(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_PSFP_SHIFT)) & ENETC_SI_SIPCAPR0_PSFP_MASK)

#define ENETC_SI_SIPCAPR0_IPFLT_MASK             (0x400U)
#define ENETC_SI_SIPCAPR0_IPFLT_SHIFT            (10U)
/*! IPFLT - Ingress Port Filtering
 *  0b0..Not supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_IPFLT(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_IPFLT_SHIFT)) & ENETC_SI_SIPCAPR0_IPFLT_MASK)

#define ENETC_SI_SIPCAPR0_RP_MASK                (0x800U)
#define ENETC_SI_SIPCAPR0_RP_SHIFT               (11U)
/*! RP - Rate Policing
 *  0b0..Not Supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_RP(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_RP_SHIFT)) & ENETC_SI_SIPCAPR0_RP_MASK)

#define ENETC_SI_SIPCAPR0_WO_MASK                (0x2000U)
#define ENETC_SI_SIPCAPR0_WO_SHIFT               (13U)
/*! WO - Wake-on-LAN
 *  0b0..Not supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_WO(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_WO_SHIFT)) & ENETC_SI_SIPCAPR0_WO_MASK)

#define ENETC_SI_SIPCAPR0_FS_MASK                (0x10000U)
#define ENETC_SI_SIPCAPR0_FS_SHIFT               (16U)
/*! FS - Functional Safety
 *  0b0..Not supported
 *  0b1..Supported
 */
#define ENETC_SI_SIPCAPR0_FS(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR0_FS_SHIFT)) & ENETC_SI_SIPCAPR0_FS_MASK)
/*! @} */

/*! @name SIPCAPR1 - Station interface port capability register 1 */
/*! @{ */

#define ENETC_SI_SIPCAPR1_NUM_TCS_MASK           (0x70U)
#define ENETC_SI_SIPCAPR1_NUM_TCS_SHIFT          (4U)
#define ENETC_SI_SIPCAPR1_NUM_TCS(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR1_NUM_TCS_SHIFT)) & ENETC_SI_SIPCAPR1_NUM_TCS_MASK)

#define ENETC_SI_SIPCAPR1_NUM_MCH_MASK           (0x300U)
#define ENETC_SI_SIPCAPR1_NUM_MCH_SHIFT          (8U)
#define ENETC_SI_SIPCAPR1_NUM_MCH(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR1_NUM_MCH_SHIFT)) & ENETC_SI_SIPCAPR1_NUM_MCH_MASK)

#define ENETC_SI_SIPCAPR1_NUM_UCH_MASK           (0xC00U)
#define ENETC_SI_SIPCAPR1_NUM_UCH_SHIFT          (10U)
#define ENETC_SI_SIPCAPR1_NUM_UCH(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR1_NUM_UCH_SHIFT)) & ENETC_SI_SIPCAPR1_NUM_UCH_MASK)

#define ENETC_SI_SIPCAPR1_NUM_MSIX_MASK          (0x3F000U)
#define ENETC_SI_SIPCAPR1_NUM_MSIX_SHIFT         (12U)
#define ENETC_SI_SIPCAPR1_NUM_MSIX(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR1_NUM_MSIX_SHIFT)) & ENETC_SI_SIPCAPR1_NUM_MSIX_MASK)

#define ENETC_SI_SIPCAPR1_NUM_IPV_MASK           (0x80000000U)
#define ENETC_SI_SIPCAPR1_NUM_IPV_SHIFT          (31U)
#define ENETC_SI_SIPCAPR1_NUM_IPV(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPCAPR1_NUM_IPV_SHIFT)) & ENETC_SI_SIPCAPR1_NUM_IPV_MASK)
/*! @} */

/*! @name SITSR - Station interface timer status register */
/*! @{ */

#define ENETC_SI_SITSR_SYNC_MASK                 (0x1U)
#define ENETC_SI_SITSR_SYNC_SHIFT                (0U)
/*! SYNC - Timer synchronization */
#define ENETC_SI_SITSR_SYNC(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITSR_SYNC_SHIFT)) & ENETC_SI_SITSR_SYNC_MASK)

#define ENETC_SI_SITSR_PARAM_VAL_MASK            (0xFFFFFFFEU)
#define ENETC_SI_SITSR_PARAM_VAL_SHIFT           (1U)
/*! PARAM_VAL - User specific parameter values */
#define ENETC_SI_SITSR_PARAM_VAL(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITSR_PARAM_VAL_SHIFT)) & ENETC_SI_SITSR_PARAM_VAL_MASK)
/*! @} */

/*! @name SIRBGCR - Station interface receive BDR group control register */
/*! @{ */

#define ENETC_SI_SIRBGCR_NUM_GROUPS_MASK         (0x3U)
#define ENETC_SI_SIRBGCR_NUM_GROUPS_SHIFT        (0U)
#define ENETC_SI_SIRBGCR_NUM_GROUPS(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRBGCR_NUM_GROUPS_SHIFT)) & ENETC_SI_SIRBGCR_NUM_GROUPS_MASK)

#define ENETC_SI_SIRBGCR_RINGS_PER_GROUP_MASK    (0x70000U)
#define ENETC_SI_SIRBGCR_RINGS_PER_GROUP_SHIFT   (16U)
#define ENETC_SI_SIRBGCR_RINGS_PER_GROUP(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRBGCR_RINGS_PER_GROUP_SHIFT)) & ENETC_SI_SIRBGCR_RINGS_PER_GROUP_MASK)
/*! @} */

/*! @name SIBCAR - Station interface buffer cache attribute register */
/*! @{ */

#define ENETC_SI_SIBCAR_BD_WRCACHE_MASK          (0xFU)
#define ENETC_SI_SIBCAR_BD_WRCACHE_SHIFT         (0U)
#define ENETC_SI_SIBCAR_BD_WRCACHE(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_BD_WRCACHE_SHIFT)) & ENETC_SI_SIBCAR_BD_WRCACHE_MASK)

#define ENETC_SI_SIBCAR_BD_WRDOMAIN_MASK         (0x30U)
#define ENETC_SI_SIBCAR_BD_WRDOMAIN_SHIFT        (4U)
#define ENETC_SI_SIBCAR_BD_WRDOMAIN(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_BD_WRDOMAIN_SHIFT)) & ENETC_SI_SIBCAR_BD_WRDOMAIN_MASK)

#define ENETC_SI_SIBCAR_BD_WRSNP_MASK            (0x40U)
#define ENETC_SI_SIBCAR_BD_WRSNP_SHIFT           (6U)
#define ENETC_SI_SIBCAR_BD_WRSNP(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_BD_WRSNP_SHIFT)) & ENETC_SI_SIBCAR_BD_WRSNP_MASK)

#define ENETC_SI_SIBCAR_WRCACHE_MASK             (0xF00U)
#define ENETC_SI_SIBCAR_WRCACHE_SHIFT            (8U)
#define ENETC_SI_SIBCAR_WRCACHE(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_WRCACHE_SHIFT)) & ENETC_SI_SIBCAR_WRCACHE_MASK)

#define ENETC_SI_SIBCAR_WRDOMAIN_MASK            (0x3000U)
#define ENETC_SI_SIBCAR_WRDOMAIN_SHIFT           (12U)
#define ENETC_SI_SIBCAR_WRDOMAIN(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_WRDOMAIN_SHIFT)) & ENETC_SI_SIBCAR_WRDOMAIN_MASK)

#define ENETC_SI_SIBCAR_WRSNP_MASK               (0x4000U)
#define ENETC_SI_SIBCAR_WRSNP_SHIFT              (14U)
#define ENETC_SI_SIBCAR_WRSNP(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_WRSNP_SHIFT)) & ENETC_SI_SIBCAR_WRSNP_MASK)

#define ENETC_SI_SIBCAR_BD_RDCACHE_MASK          (0xF0000U)
#define ENETC_SI_SIBCAR_BD_RDCACHE_SHIFT         (16U)
#define ENETC_SI_SIBCAR_BD_RDCACHE(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_BD_RDCACHE_SHIFT)) & ENETC_SI_SIBCAR_BD_RDCACHE_MASK)

#define ENETC_SI_SIBCAR_BD_RDDOMAIN_MASK         (0x300000U)
#define ENETC_SI_SIBCAR_BD_RDDOMAIN_SHIFT        (20U)
#define ENETC_SI_SIBCAR_BD_RDDOMAIN(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_BD_RDDOMAIN_SHIFT)) & ENETC_SI_SIBCAR_BD_RDDOMAIN_MASK)

#define ENETC_SI_SIBCAR_BD_RDSNP_MASK            (0x400000U)
#define ENETC_SI_SIBCAR_BD_RDSNP_SHIFT           (22U)
#define ENETC_SI_SIBCAR_BD_RDSNP(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_BD_RDSNP_SHIFT)) & ENETC_SI_SIBCAR_BD_RDSNP_MASK)

#define ENETC_SI_SIBCAR_RDCACHE_MASK             (0xF000000U)
#define ENETC_SI_SIBCAR_RDCACHE_SHIFT            (24U)
#define ENETC_SI_SIBCAR_RDCACHE(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_RDCACHE_SHIFT)) & ENETC_SI_SIBCAR_RDCACHE_MASK)

#define ENETC_SI_SIBCAR_RDDOMAIN_MASK            (0x30000000U)
#define ENETC_SI_SIBCAR_RDDOMAIN_SHIFT           (28U)
#define ENETC_SI_SIBCAR_RDDOMAIN(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_RDDOMAIN_SHIFT)) & ENETC_SI_SIBCAR_RDDOMAIN_MASK)

#define ENETC_SI_SIBCAR_RDSNP_MASK               (0x40000000U)
#define ENETC_SI_SIBCAR_RDSNP_SHIFT              (30U)
#define ENETC_SI_SIBCAR_RDSNP(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBCAR_RDSNP_SHIFT)) & ENETC_SI_SIBCAR_RDSNP_MASK)
/*! @} */

/*! @name SIMCAR - Station interface message cache attribute register */
/*! @{ */

#define ENETC_SI_SIMCAR_MSG_WRCACHE_MASK         (0xFU)
#define ENETC_SI_SIMCAR_MSG_WRCACHE_SHIFT        (0U)
#define ENETC_SI_SIMCAR_MSG_WRCACHE(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMCAR_MSG_WRCACHE_SHIFT)) & ENETC_SI_SIMCAR_MSG_WRCACHE_MASK)

#define ENETC_SI_SIMCAR_MSG_WRDOMAIN_MASK        (0x30U)
#define ENETC_SI_SIMCAR_MSG_WRDOMAIN_SHIFT       (4U)
#define ENETC_SI_SIMCAR_MSG_WRDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMCAR_MSG_WRDOMAIN_SHIFT)) & ENETC_SI_SIMCAR_MSG_WRDOMAIN_MASK)

#define ENETC_SI_SIMCAR_MSG_WRSNP_MASK           (0x40U)
#define ENETC_SI_SIMCAR_MSG_WRSNP_SHIFT          (6U)
#define ENETC_SI_SIMCAR_MSG_WRSNP(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMCAR_MSG_WRSNP_SHIFT)) & ENETC_SI_SIMCAR_MSG_WRSNP_MASK)

#define ENETC_SI_SIMCAR_MSG_RDCACHE_MASK         (0xF0000U)
#define ENETC_SI_SIMCAR_MSG_RDCACHE_SHIFT        (16U)
#define ENETC_SI_SIMCAR_MSG_RDCACHE(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMCAR_MSG_RDCACHE_SHIFT)) & ENETC_SI_SIMCAR_MSG_RDCACHE_MASK)

#define ENETC_SI_SIMCAR_MSG_RDDOMAIN_MASK        (0x300000U)
#define ENETC_SI_SIMCAR_MSG_RDDOMAIN_SHIFT       (20U)
#define ENETC_SI_SIMCAR_MSG_RDDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMCAR_MSG_RDDOMAIN_SHIFT)) & ENETC_SI_SIMCAR_MSG_RDDOMAIN_MASK)

#define ENETC_SI_SIMCAR_MSG_RDSNP_MASK           (0x400000U)
#define ENETC_SI_SIMCAR_MSG_RDSNP_SHIFT          (22U)
#define ENETC_SI_SIMCAR_MSG_RDSNP(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMCAR_MSG_RDSNP_SHIFT)) & ENETC_SI_SIMCAR_MSG_RDSNP_MASK)
/*! @} */

/*! @name SICCAR - Station interface command cache attribute register */
/*! @{ */

#define ENETC_SI_SICCAR_CBD_WRCACHE_MASK         (0xFU)
#define ENETC_SI_SICCAR_CBD_WRCACHE_SHIFT        (0U)
#define ENETC_SI_SICCAR_CBD_WRCACHE(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CBD_WRCACHE_SHIFT)) & ENETC_SI_SICCAR_CBD_WRCACHE_MASK)

#define ENETC_SI_SICCAR_CBD_WRDOMAIN_MASK        (0x30U)
#define ENETC_SI_SICCAR_CBD_WRDOMAIN_SHIFT       (4U)
#define ENETC_SI_SICCAR_CBD_WRDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CBD_WRDOMAIN_SHIFT)) & ENETC_SI_SICCAR_CBD_WRDOMAIN_MASK)

#define ENETC_SI_SICCAR_CBD_WRSNP_MASK           (0x40U)
#define ENETC_SI_SICCAR_CBD_WRSNP_SHIFT          (6U)
#define ENETC_SI_SICCAR_CBD_WRSNP(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CBD_WRSNP_SHIFT)) & ENETC_SI_SICCAR_CBD_WRSNP_MASK)

#define ENETC_SI_SICCAR_CWRCACHE_MASK            (0xF00U)
#define ENETC_SI_SICCAR_CWRCACHE_SHIFT           (8U)
#define ENETC_SI_SICCAR_CWRCACHE(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CWRCACHE_SHIFT)) & ENETC_SI_SICCAR_CWRCACHE_MASK)

#define ENETC_SI_SICCAR_CWRDOMAIN_MASK           (0x3000U)
#define ENETC_SI_SICCAR_CWRDOMAIN_SHIFT          (12U)
#define ENETC_SI_SICCAR_CWRDOMAIN(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CWRDOMAIN_SHIFT)) & ENETC_SI_SICCAR_CWRDOMAIN_MASK)

#define ENETC_SI_SICCAR_CWRSNP_MASK              (0x4000U)
#define ENETC_SI_SICCAR_CWRSNP_SHIFT             (14U)
#define ENETC_SI_SICCAR_CWRSNP(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CWRSNP_SHIFT)) & ENETC_SI_SICCAR_CWRSNP_MASK)

#define ENETC_SI_SICCAR_CBD_RDCACHE_MASK         (0xF0000U)
#define ENETC_SI_SICCAR_CBD_RDCACHE_SHIFT        (16U)
#define ENETC_SI_SICCAR_CBD_RDCACHE(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CBD_RDCACHE_SHIFT)) & ENETC_SI_SICCAR_CBD_RDCACHE_MASK)

#define ENETC_SI_SICCAR_CBD_RDDOMAIN_MASK        (0x300000U)
#define ENETC_SI_SICCAR_CBD_RDDOMAIN_SHIFT       (20U)
#define ENETC_SI_SICCAR_CBD_RDDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CBD_RDDOMAIN_SHIFT)) & ENETC_SI_SICCAR_CBD_RDDOMAIN_MASK)

#define ENETC_SI_SICCAR_CBD_RDSNP_MASK           (0x400000U)
#define ENETC_SI_SICCAR_CBD_RDSNP_SHIFT          (22U)
#define ENETC_SI_SICCAR_CBD_RDSNP(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CBD_RDSNP_SHIFT)) & ENETC_SI_SICCAR_CBD_RDSNP_MASK)

#define ENETC_SI_SICCAR_CRDCACHE_MASK            (0xF000000U)
#define ENETC_SI_SICCAR_CRDCACHE_SHIFT           (24U)
#define ENETC_SI_SICCAR_CRDCACHE(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CRDCACHE_SHIFT)) & ENETC_SI_SICCAR_CRDCACHE_MASK)

#define ENETC_SI_SICCAR_CRDDOMAIN_MASK           (0x30000000U)
#define ENETC_SI_SICCAR_CRDDOMAIN_SHIFT          (28U)
#define ENETC_SI_SICCAR_CRDDOMAIN(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CRDDOMAIN_SHIFT)) & ENETC_SI_SICCAR_CRDDOMAIN_MASK)

#define ENETC_SI_SICCAR_CRDSNP_MASK              (0x40000000U)
#define ENETC_SI_SICCAR_CRDSNP_SHIFT             (30U)
#define ENETC_SI_SICCAR_CRDSNP(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICCAR_CRDSNP_SHIFT)) & ENETC_SI_SICCAR_CRDSNP_MASK)
/*! @} */

/*! @name SIPMAR0 - Station interface primary MAC address register 0 */
/*! @{ */

#define ENETC_SI_SIPMAR0_PRIM_MAC_ADDR_MASK      (0xFFFFFFFFU)
#define ENETC_SI_SIPMAR0_PRIM_MAC_ADDR_SHIFT     (0U)
#define ENETC_SI_SIPMAR0_PRIM_MAC_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPMAR0_PRIM_MAC_ADDR_SHIFT)) & ENETC_SI_SIPMAR0_PRIM_MAC_ADDR_MASK)
/*! @} */

/*! @name SIPMAR1 - Station interface primary MAC address register 1 */
/*! @{ */

#define ENETC_SI_SIPMAR1_PRIM_MAC_ADDR_MASK      (0xFFFFU)
#define ENETC_SI_SIPMAR1_PRIM_MAC_ADDR_SHIFT     (0U)
#define ENETC_SI_SIPMAR1_PRIM_MAC_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIPMAR1_PRIM_MAC_ADDR_SHIFT)) & ENETC_SI_SIPMAR1_PRIM_MAC_ADDR_MASK)
/*! @} */

/*! @name SICVLANR1 - Station interface custom VLAN register 1 */
/*! @{ */

#define ENETC_SI_SICVLANR1_ETYPE_MASK            (0xFFFFU)
#define ENETC_SI_SICVLANR1_ETYPE_SHIFT           (0U)
#define ENETC_SI_SICVLANR1_ETYPE(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICVLANR1_ETYPE_SHIFT)) & ENETC_SI_SICVLANR1_ETYPE_MASK)

#define ENETC_SI_SICVLANR1_V_MASK                (0x80000000U)
#define ENETC_SI_SICVLANR1_V_SHIFT               (31U)
#define ENETC_SI_SICVLANR1_V(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICVLANR1_V_SHIFT)) & ENETC_SI_SICVLANR1_V_MASK)
/*! @} */

/*! @name SICVLANR2 - Station interface custom VLAN register 2 */
/*! @{ */

#define ENETC_SI_SICVLANR2_ETYPE_MASK            (0xFFFFU)
#define ENETC_SI_SICVLANR2_ETYPE_SHIFT           (0U)
#define ENETC_SI_SICVLANR2_ETYPE(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICVLANR2_ETYPE_SHIFT)) & ENETC_SI_SICVLANR2_ETYPE_MASK)

#define ENETC_SI_SICVLANR2_V_MASK                (0x80000000U)
#define ENETC_SI_SICVLANR2_V_SHIFT               (31U)
#define ENETC_SI_SICVLANR2_V(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICVLANR2_V_SHIFT)) & ENETC_SI_SICVLANR2_V_MASK)
/*! @} */

/*! @name SIVLANIPVMR0 - Station interface VLAN to IPV mapping register 0 */
/*! @{ */

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_0_MASK     (0xFU)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_0_SHIFT    (0U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_0(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_0_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_0_MASK)

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_1_MASK     (0xF0U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_1_SHIFT    (4U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_1(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_1_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_1_MASK)

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_2_MASK     (0xF00U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_2_SHIFT    (8U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_2(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_2_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_2_MASK)

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_3_MASK     (0xF000U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_3_SHIFT    (12U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_3(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_3_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_3_MASK)

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_4_MASK     (0xF0000U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_4_SHIFT    (16U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_4(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_4_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_4_MASK)

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_5_MASK     (0xF00000U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_5_SHIFT    (20U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_5(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_5_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_5_MASK)

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_6_MASK     (0xF000000U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_6_SHIFT    (24U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_6(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_6_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_6_MASK)

#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_7_MASK     (0xF0000000U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_7_SHIFT    (28U)
#define ENETC_SI_SIVLANIPVMR0_PCP_DEI_7(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR0_PCP_DEI_7_SHIFT)) & ENETC_SI_SIVLANIPVMR0_PCP_DEI_7_MASK)
/*! @} */

/*! @name SIVLANIPVMR1 - Station interface VLAN to IPV mapping register 1 */
/*! @{ */

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_8_MASK     (0xFU)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_8_SHIFT    (0U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_8(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_8_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_8_MASK)

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_9_MASK     (0xF0U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_9_SHIFT    (4U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_9(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_9_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_9_MASK)

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_10_MASK    (0xF00U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_10_SHIFT   (8U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_10(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_10_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_10_MASK)

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_11_MASK    (0xF000U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_11_SHIFT   (12U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_11(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_11_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_11_MASK)

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_12_MASK    (0xF0000U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_12_SHIFT   (16U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_12(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_12_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_12_MASK)

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_13_MASK    (0xF00000U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_13_SHIFT   (20U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_13(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_13_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_13_MASK)

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_14_MASK    (0xF000000U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_14_SHIFT   (24U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_14(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_14_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_14_MASK)

#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_15_MASK    (0xF0000000U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_15_SHIFT   (28U)
#define ENETC_SI_SIVLANIPVMR1_PCP_DEI_15(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVLANIPVMR1_PCP_DEI_15_SHIFT)) & ENETC_SI_SIVLANIPVMR1_PCP_DEI_15_MASK)
/*! @} */

/*! @name SIIPVBDRMR0 - Station interface IPV to ring mapping register */
/*! @{ */

#define ENETC_SI_SIIPVBDRMR0_IPV0BDR_MASK        (0x7U)
#define ENETC_SI_SIIPVBDRMR0_IPV0BDR_SHIFT       (0U)
#define ENETC_SI_SIIPVBDRMR0_IPV0BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV0BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV0BDR_MASK)

#define ENETC_SI_SIIPVBDRMR0_IPV1BDR_MASK        (0x70U)
#define ENETC_SI_SIIPVBDRMR0_IPV1BDR_SHIFT       (4U)
#define ENETC_SI_SIIPVBDRMR0_IPV1BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV1BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV1BDR_MASK)

#define ENETC_SI_SIIPVBDRMR0_IPV2BDR_MASK        (0x700U)
#define ENETC_SI_SIIPVBDRMR0_IPV2BDR_SHIFT       (8U)
#define ENETC_SI_SIIPVBDRMR0_IPV2BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV2BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV2BDR_MASK)

#define ENETC_SI_SIIPVBDRMR0_IPV3BDR_MASK        (0x7000U)
#define ENETC_SI_SIIPVBDRMR0_IPV3BDR_SHIFT       (12U)
#define ENETC_SI_SIIPVBDRMR0_IPV3BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV3BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV3BDR_MASK)

#define ENETC_SI_SIIPVBDRMR0_IPV4BDR_MASK        (0x70000U)
#define ENETC_SI_SIIPVBDRMR0_IPV4BDR_SHIFT       (16U)
#define ENETC_SI_SIIPVBDRMR0_IPV4BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV4BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV4BDR_MASK)

#define ENETC_SI_SIIPVBDRMR0_IPV5BDR_MASK        (0x700000U)
#define ENETC_SI_SIIPVBDRMR0_IPV5BDR_SHIFT       (20U)
#define ENETC_SI_SIIPVBDRMR0_IPV5BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV5BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV5BDR_MASK)

#define ENETC_SI_SIIPVBDRMR0_IPV6BDR_MASK        (0x7000000U)
#define ENETC_SI_SIIPVBDRMR0_IPV6BDR_SHIFT       (24U)
#define ENETC_SI_SIIPVBDRMR0_IPV6BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV6BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV6BDR_MASK)

#define ENETC_SI_SIIPVBDRMR0_IPV7BDR_MASK        (0x70000000U)
#define ENETC_SI_SIIPVBDRMR0_IPV7BDR_SHIFT       (28U)
#define ENETC_SI_SIIPVBDRMR0_IPV7BDR(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIIPVBDRMR0_IPV7BDR_SHIFT)) & ENETC_SI_SIIPVBDRMR0_IPV7BDR_MASK)
/*! @} */

/*! @name PSIMSGRR - Physical station interface message receive register */
/*! @{ */

#define ENETC_SI_PSIMSGRR_MR1_MASK               (0x2U)
#define ENETC_SI_PSIMSGRR_MR1_SHIFT              (1U)
#define ENETC_SI_PSIMSGRR_MR1(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIMSGRR_MR1_SHIFT)) & ENETC_SI_PSIMSGRR_MR1_MASK)

#define ENETC_SI_PSIMSGRR_MC_MASK                (0xFFFF0000U)
#define ENETC_SI_PSIMSGRR_MC_SHIFT               (16U)
/*! MC - Message code */
#define ENETC_SI_PSIMSGRR_MC(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIMSGRR_MC_SHIFT)) & ENETC_SI_PSIMSGRR_MC_MASK)
/*! @} */

/*! @name PSIMSGSR - Physical station interface message send register */
/*! @{ */

#define ENETC_SI_PSIMSGSR_MS1_MASK               (0x2U)
#define ENETC_SI_PSIMSGSR_MS1_SHIFT              (1U)
#define ENETC_SI_PSIMSGSR_MS1(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIMSGSR_MS1_SHIFT)) & ENETC_SI_PSIMSGSR_MS1_MASK)

#define ENETC_SI_PSIMSGSR_MC_MASK                (0xFFFF0000U)
#define ENETC_SI_PSIMSGSR_MC_SHIFT               (16U)
#define ENETC_SI_PSIMSGSR_MC(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIMSGSR_MC_SHIFT)) & ENETC_SI_PSIMSGSR_MC_MASK)
/*! @} */

/*! @name PSIVMSGRCVAR0 - PSI VSI 1 message receive address register 0 */
/*! @{ */

#define ENETC_SI_PSIVMSGRCVAR0_MSIZE_MASK        (0x1FU)
#define ENETC_SI_PSIVMSGRCVAR0_MSIZE_SHIFT       (0U)
#define ENETC_SI_PSIVMSGRCVAR0_MSIZE(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIVMSGRCVAR0_MSIZE_SHIFT)) & ENETC_SI_PSIVMSGRCVAR0_MSIZE_MASK)

#define ENETC_SI_PSIVMSGRCVAR0_ADDRL_MASK        (0xFFFFFFC0U)
#define ENETC_SI_PSIVMSGRCVAR0_ADDRL_SHIFT       (6U)
#define ENETC_SI_PSIVMSGRCVAR0_ADDRL(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIVMSGRCVAR0_ADDRL_SHIFT)) & ENETC_SI_PSIVMSGRCVAR0_ADDRL_MASK)
/*! @} */

/* The count of ENETC_SI_PSIVMSGRCVAR0 */
#define ENETC_SI_PSIVMSGRCVAR0_COUNT             (1U)

/*! @name PSIVMSGRCVAR1 - PSI VSI 1 message receive address register 1 */
/*! @{ */

#define ENETC_SI_PSIVMSGRCVAR1_ADDRH_MASK        (0xFFFFFFFFU)
#define ENETC_SI_PSIVMSGRCVAR1_ADDRH_SHIFT       (0U)
#define ENETC_SI_PSIVMSGRCVAR1_ADDRH(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIVMSGRCVAR1_ADDRH_SHIFT)) & ENETC_SI_PSIVMSGRCVAR1_ADDRH_MASK)
/*! @} */

/* The count of ENETC_SI_PSIVMSGRCVAR1 */
#define ENETC_SI_PSIVMSGRCVAR1_COUNT             (1U)

/*! @name VSIMSGSR - Virtual station interface message send register */
/*! @{ */

#define ENETC_SI_VSIMSGSR_MB_MASK                (0x1U)
#define ENETC_SI_VSIMSGSR_MB_SHIFT               (0U)
#define ENETC_SI_VSIMSGSR_MB(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGSR_MB_SHIFT)) & ENETC_SI_VSIMSGSR_MB_MASK)

#define ENETC_SI_VSIMSGSR_MS_MASK                (0x2U)
#define ENETC_SI_VSIMSGSR_MS_SHIFT               (1U)
#define ENETC_SI_VSIMSGSR_MS(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGSR_MS_SHIFT)) & ENETC_SI_VSIMSGSR_MS_MASK)

#define ENETC_SI_VSIMSGSR_MC_MASK                (0xFFFF0000U)
#define ENETC_SI_VSIMSGSR_MC_SHIFT               (16U)
#define ENETC_SI_VSIMSGSR_MC(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGSR_MC_SHIFT)) & ENETC_SI_VSIMSGSR_MC_MASK)
/*! @} */

/*! @name VSIMSGRR - Virtual station interface message receive register */
/*! @{ */

#define ENETC_SI_VSIMSGRR_MR_MASK                (0x1U)
#define ENETC_SI_VSIMSGRR_MR_SHIFT               (0U)
#define ENETC_SI_VSIMSGRR_MR(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGRR_MR_SHIFT)) & ENETC_SI_VSIMSGRR_MR_MASK)

#define ENETC_SI_VSIMSGRR_MC_MASK                (0xFFFF0000U)
#define ENETC_SI_VSIMSGRR_MC_SHIFT               (16U)
#define ENETC_SI_VSIMSGRR_MC(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGRR_MC_SHIFT)) & ENETC_SI_VSIMSGRR_MC_MASK)
/*! @} */

/*! @name VSIMSGSNDAR0 - Virtual station interface message send register 0 */
/*! @{ */

#define ENETC_SI_VSIMSGSNDAR0_MSIZE_MASK         (0x1FU)
#define ENETC_SI_VSIMSGSNDAR0_MSIZE_SHIFT        (0U)
#define ENETC_SI_VSIMSGSNDAR0_MSIZE(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGSNDAR0_MSIZE_SHIFT)) & ENETC_SI_VSIMSGSNDAR0_MSIZE_MASK)

#define ENETC_SI_VSIMSGSNDAR0_ADDRL_MASK         (0xFFFFFFC0U)
#define ENETC_SI_VSIMSGSNDAR0_ADDRL_SHIFT        (6U)
#define ENETC_SI_VSIMSGSNDAR0_ADDRL(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGSNDAR0_ADDRL_SHIFT)) & ENETC_SI_VSIMSGSNDAR0_ADDRL_MASK)
/*! @} */

/*! @name VSIMSGSNDAR1 - Virtual station interface message send address register 1 */
/*! @{ */

#define ENETC_SI_VSIMSGSNDAR1_ADDRH_MASK         (0xFFFFFFFFU)
#define ENETC_SI_VSIMSGSNDAR1_ADDRH_SHIFT        (0U)
#define ENETC_SI_VSIMSGSNDAR1_ADDRH(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIMSGSNDAR1_ADDRH_SHIFT)) & ENETC_SI_VSIMSGSNDAR1_ADDRH_MASK)
/*! @} */

/*! @name SIROCT0 - Station interface receive octets counter (ifInOctets) 0 */
/*! @{ */

#define ENETC_SI_SIROCT0_ROCT_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SIROCT0_ROCT_LOW_SHIFT          (0U)
#define ENETC_SI_SIROCT0_ROCT_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIROCT0_ROCT_LOW_SHIFT)) & ENETC_SI_SIROCT0_ROCT_LOW_MASK)
/*! @} */

/*! @name SIROCT1 - Station interface receive octets counter (ifInOctets) 1 */
/*! @{ */

#define ENETC_SI_SIROCT1_ROCT_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SIROCT1_ROCT_HIGH_SHIFT         (0U)
#define ENETC_SI_SIROCT1_ROCT_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIROCT1_ROCT_HIGH_SHIFT)) & ENETC_SI_SIROCT1_ROCT_HIGH_MASK)
/*! @} */

/*! @name SIRFRM0 - Station interface receive frame counter (aFrameReceivedOK) 0 */
/*! @{ */

#define ENETC_SI_SIRFRM0_RFRM_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SIRFRM0_RFRM_LOW_SHIFT          (0U)
#define ENETC_SI_SIRFRM0_RFRM_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRFRM0_RFRM_LOW_SHIFT)) & ENETC_SI_SIRFRM0_RFRM_LOW_MASK)
/*! @} */

/*! @name SIRFRM1 - Station interface receive frame counter (aFrameReceivedOK) 1 */
/*! @{ */

#define ENETC_SI_SIRFRM1_RFRM_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SIRFRM1_RFRM_HIGH_SHIFT         (0U)
#define ENETC_SI_SIRFRM1_RFRM_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRFRM1_RFRM_HIGH_SHIFT)) & ENETC_SI_SIRFRM1_RFRM_HIGH_MASK)
/*! @} */

/*! @name SIRUCA0 - Station interface receive unicast frame counter (ifInUcastPkts) 0 */
/*! @{ */

#define ENETC_SI_SIRUCA0_RUCA_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SIRUCA0_RUCA_LOW_SHIFT          (0U)
#define ENETC_SI_SIRUCA0_RUCA_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRUCA0_RUCA_LOW_SHIFT)) & ENETC_SI_SIRUCA0_RUCA_LOW_MASK)
/*! @} */

/*! @name SIRUCA1 - Station interface receive unicast frame counter (ifInUcastPkts) 1 */
/*! @{ */

#define ENETC_SI_SIRUCA1_RUCA_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SIRUCA1_RUCA_HIGH_SHIFT         (0U)
#define ENETC_SI_SIRUCA1_RUCA_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRUCA1_RUCA_HIGH_SHIFT)) & ENETC_SI_SIRUCA1_RUCA_HIGH_MASK)
/*! @} */

/*! @name SIRMCA0 - Station interface receive multicast frame counter (ifInMulticastPkts) 0 */
/*! @{ */

#define ENETC_SI_SIRMCA0_RMCA_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SIRMCA0_RMCA_LOW_SHIFT          (0U)
#define ENETC_SI_SIRMCA0_RMCA_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRMCA0_RMCA_LOW_SHIFT)) & ENETC_SI_SIRMCA0_RMCA_LOW_MASK)
/*! @} */

/*! @name SIRMCA1 - Station interface receive multicast frame counter (ifInMulticastPkts) 1 */
/*! @{ */

#define ENETC_SI_SIRMCA1_RMCA_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SIRMCA1_RMCA_HIGH_SHIFT         (0U)
#define ENETC_SI_SIRMCA1_RMCA_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRMCA1_RMCA_HIGH_SHIFT)) & ENETC_SI_SIRMCA1_RMCA_HIGH_MASK)
/*! @} */

/*! @name SITOCT0 - Station interface transmit octets counter (ifOutOctets) 0 */
/*! @{ */

#define ENETC_SI_SITOCT0_TOCT_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SITOCT0_TOCT_LOW_SHIFT          (0U)
#define ENETC_SI_SITOCT0_TOCT_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITOCT0_TOCT_LOW_SHIFT)) & ENETC_SI_SITOCT0_TOCT_LOW_MASK)
/*! @} */

/*! @name SITOCT1 - Station interface transmit octets counter (ifOutOctets) 1 */
/*! @{ */

#define ENETC_SI_SITOCT1_TOCT_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SITOCT1_TOCT_HIGH_SHIFT         (0U)
#define ENETC_SI_SITOCT1_TOCT_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITOCT1_TOCT_HIGH_SHIFT)) & ENETC_SI_SITOCT1_TOCT_HIGH_MASK)
/*! @} */

/*! @name SITFRM0 - Station interface transmit frame counter (aFrameTransmittedOK) 0 */
/*! @{ */

#define ENETC_SI_SITFRM0_TFRM_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SITFRM0_TFRM_LOW_SHIFT          (0U)
#define ENETC_SI_SITFRM0_TFRM_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITFRM0_TFRM_LOW_SHIFT)) & ENETC_SI_SITFRM0_TFRM_LOW_MASK)
/*! @} */

/*! @name SITFRM1 - Station interface transmit frame counter (aFrameTransmittedOK) 1 */
/*! @{ */

#define ENETC_SI_SITFRM1_TFRM_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SITFRM1_TFRM_HIGH_SHIFT         (0U)
#define ENETC_SI_SITFRM1_TFRM_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITFRM1_TFRM_HIGH_SHIFT)) & ENETC_SI_SITFRM1_TFRM_HIGH_MASK)
/*! @} */

/*! @name SITUCA0 - Station interface transmit unicast frame counter (ifOutUcastPkts) 0 */
/*! @{ */

#define ENETC_SI_SITUCA0_TUCA_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SITUCA0_TUCA_LOW_SHIFT          (0U)
#define ENETC_SI_SITUCA0_TUCA_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITUCA0_TUCA_LOW_SHIFT)) & ENETC_SI_SITUCA0_TUCA_LOW_MASK)
/*! @} */

/*! @name SITUCA1 - Station interface transmit unicast frame counter (ifOutUcastPkts) 1 */
/*! @{ */

#define ENETC_SI_SITUCA1_TUCA_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SITUCA1_TUCA_HIGH_SHIFT         (0U)
#define ENETC_SI_SITUCA1_TUCA_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITUCA1_TUCA_HIGH_SHIFT)) & ENETC_SI_SITUCA1_TUCA_HIGH_MASK)
/*! @} */

/*! @name SITMCA0 - Station interface transmit multicast frame counter (ifOutMulticastPkts) 0 */
/*! @{ */

#define ENETC_SI_SITMCA0_TMCA_LOW_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SITMCA0_TMCA_LOW_SHIFT          (0U)
#define ENETC_SI_SITMCA0_TMCA_LOW(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITMCA0_TMCA_LOW_SHIFT)) & ENETC_SI_SITMCA0_TMCA_LOW_MASK)
/*! @} */

/*! @name SITMCA1 - Station interface transmit multicast frame counter (ifOutMulticastPkts) 1 */
/*! @{ */

#define ENETC_SI_SITMCA1_TMCA_HIGH_MASK          (0xFFFFFFFFU)
#define ENETC_SI_SITMCA1_TMCA_HIGH_SHIFT         (0U)
#define ENETC_SI_SITMCA1_TMCA_HIGH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITMCA1_TMCA_HIGH_SHIFT)) & ENETC_SI_SITMCA1_TMCA_HIGH_MASK)
/*! @} */

/*! @name SIBLPR - Station interface boot loader parameter register 0..Station interface boot loader parameter register 1 */
/*! @{ */

#define ENETC_SI_SIBLPR_PARAM_VAL_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SIBLPR_PARAM_VAL_SHIFT          (0U)
#define ENETC_SI_SIBLPR_PARAM_VAL(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIBLPR_PARAM_VAL_SHIFT)) & ENETC_SI_SIBLPR_PARAM_VAL_MASK)
/*! @} */

/* The count of ENETC_SI_SIBLPR */
#define ENETC_SI_SIBLPR_COUNT                    (2U)

/*! @name SICBDRMR - Station interface command BDR mode register */
/*! @{ */

#define ENETC_SI_SICBDRMR_EN_MASK                (0x80000000U)
#define ENETC_SI_SICBDRMR_EN_SHIFT               (31U)
#define ENETC_SI_SICBDRMR_EN(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRMR_EN_SHIFT)) & ENETC_SI_SICBDRMR_EN_MASK)
/*! @} */

/*! @name SICBDRSR - Station interface command BDR status register */
/*! @{ */

#define ENETC_SI_SICBDRSR_BUSY_MASK              (0x1U)
#define ENETC_SI_SICBDRSR_BUSY_SHIFT             (0U)
#define ENETC_SI_SICBDRSR_BUSY(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRSR_BUSY_SHIFT)) & ENETC_SI_SICBDRSR_BUSY_MASK)
/*! @} */

/*! @name SICBDRBAR0 - Station interface command BDR base address register 0 */
/*! @{ */

#define ENETC_SI_SICBDRBAR0_ADDRL_MASK           (0xFFFFFF80U)
#define ENETC_SI_SICBDRBAR0_ADDRL_SHIFT          (7U)
#define ENETC_SI_SICBDRBAR0_ADDRL(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRBAR0_ADDRL_SHIFT)) & ENETC_SI_SICBDRBAR0_ADDRL_MASK)
/*! @} */

/*! @name SICBDRBAR1 - Station interface command BDR base address register 1 */
/*! @{ */

#define ENETC_SI_SICBDRBAR1_ADDRH_MASK           (0xFFFFFFFFU)
#define ENETC_SI_SICBDRBAR1_ADDRH_SHIFT          (0U)
#define ENETC_SI_SICBDRBAR1_ADDRH(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRBAR1_ADDRH_SHIFT)) & ENETC_SI_SICBDRBAR1_ADDRH_MASK)
/*! @} */

/*! @name SICBDRPIR - Station interface command BDR producer index register */
/*! @{ */

#define ENETC_SI_SICBDRPIR_BDR_INDEX_MASK        (0x3FFU)
#define ENETC_SI_SICBDRPIR_BDR_INDEX_SHIFT       (0U)
#define ENETC_SI_SICBDRPIR_BDR_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRPIR_BDR_INDEX_SHIFT)) & ENETC_SI_SICBDRPIR_BDR_INDEX_MASK)
/*! @} */

/*! @name SICBDRCIR - Station interface command BDR consumer index register */
/*! @{ */

#define ENETC_SI_SICBDRCIR_BDR_INDEX_MASK        (0x3FFU)
#define ENETC_SI_SICBDRCIR_BDR_INDEX_SHIFT       (0U)
#define ENETC_SI_SICBDRCIR_BDR_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRCIR_BDR_INDEX_SHIFT)) & ENETC_SI_SICBDRCIR_BDR_INDEX_MASK)
/*! @} */

/*! @name SICBDRLENR - Station interface command BDR length register */
/*! @{ */

#define ENETC_SI_SICBDRLENR_LENGTH_MASK          (0x7F8U)
#define ENETC_SI_SICBDRLENR_LENGTH_SHIFT         (3U)
#define ENETC_SI_SICBDRLENR_LENGTH(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRLENR_LENGTH_SHIFT)) & ENETC_SI_SICBDRLENR_LENGTH_MASK)
/*! @} */

/*! @name SICBDRIER - Station interface command BDR interrupt enable register */
/*! @{ */

#define ENETC_SI_SICBDRIER_CBDCIE_MASK           (0x1U)
#define ENETC_SI_SICBDRIER_CBDCIE_SHIFT          (0U)
#define ENETC_SI_SICBDRIER_CBDCIE(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRIER_CBDCIE_SHIFT)) & ENETC_SI_SICBDRIER_CBDCIE_MASK)
/*! @} */

/*! @name SICBDRIDR - Station interface command BDR interrupt detect register */
/*! @{ */

#define ENETC_SI_SICBDRIDR_CBDC_MASK             (0x1U)
#define ENETC_SI_SICBDRIDR_CBDC_SHIFT            (0U)
#define ENETC_SI_SICBDRIDR_CBDC(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICBDRIDR_CBDC_SHIFT)) & ENETC_SI_SICBDRIDR_CBDC_MASK)
/*! @} */

/*! @name SICAPR0 - Station interface capability register 0 */
/*! @{ */

#define ENETC_SI_SICAPR0_NUM_TX_BDR_MASK         (0xFFU)
#define ENETC_SI_SICAPR0_NUM_TX_BDR_SHIFT        (0U)
#define ENETC_SI_SICAPR0_NUM_TX_BDR(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICAPR0_NUM_TX_BDR_SHIFT)) & ENETC_SI_SICAPR0_NUM_TX_BDR_MASK)

#define ENETC_SI_SICAPR0_NUM_RX_BDR_MASK         (0xFF0000U)
#define ENETC_SI_SICAPR0_NUM_RX_BDR_SHIFT        (16U)
#define ENETC_SI_SICAPR0_NUM_RX_BDR(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICAPR0_NUM_RX_BDR_SHIFT)) & ENETC_SI_SICAPR0_NUM_RX_BDR_MASK)

#define ENETC_SI_SICAPR0_NUM_MAC_ADDR_MASK       (0xF0000000U)
#define ENETC_SI_SICAPR0_NUM_MAC_ADDR_SHIFT      (28U)
#define ENETC_SI_SICAPR0_NUM_MAC_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICAPR0_NUM_MAC_ADDR_SHIFT)) & ENETC_SI_SICAPR0_NUM_MAC_ADDR_MASK)
/*! @} */

/*! @name SICAPR1 - Station interface capability register 1 */
/*! @{ */

#define ENETC_SI_SICAPR1_NUM_RX_GRP_MASK         (0xFF0000U)
#define ENETC_SI_SICAPR1_NUM_RX_GRP_SHIFT        (16U)
#define ENETC_SI_SICAPR1_NUM_RX_GRP(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICAPR1_NUM_RX_GRP_SHIFT)) & ENETC_SI_SICAPR1_NUM_RX_GRP_MASK)
/*! @} */

/*! @name SICAPR2 - Station interface capability register 2 */
/*! @{ */

#define ENETC_SI_SICAPR2_VTP_MASK                (0xFU)
#define ENETC_SI_SICAPR2_VTP_SHIFT               (0U)
#define ENETC_SI_SICAPR2_VTP(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICAPR2_VTP_SHIFT)) & ENETC_SI_SICAPR2_VTP_MASK)
/*! @} */

/*! @name PSIIER - Physical station interface interrupt enable register */
/*! @{ */

#define ENETC_SI_PSIIER_MR1IE_MASK               (0x2U)
#define ENETC_SI_PSIIER_MR1IE_SHIFT              (1U)
#define ENETC_SI_PSIIER_MR1IE(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIIER_MR1IE_SHIFT)) & ENETC_SI_PSIIER_MR1IE_MASK)

#define ENETC_SI_PSIIER_FLR1IE_MASK              (0x20000U)
#define ENETC_SI_PSIIER_FLR1IE_SHIFT             (17U)
#define ENETC_SI_PSIIER_FLR1IE(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIIER_FLR1IE_SHIFT)) & ENETC_SI_PSIIER_FLR1IE_MASK)
/*! @} */

/*! @name PSIIDR - Physical station interface interrupt detect register */
/*! @{ */

#define ENETC_SI_PSIIDR_TXR_MASK                 (0x1U)
#define ENETC_SI_PSIIDR_TXR_SHIFT                (0U)
#define ENETC_SI_PSIIDR_TXR(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIIDR_TXR_SHIFT)) & ENETC_SI_PSIIDR_TXR_MASK)

#define ENETC_SI_PSIIDR_MR1_MASK                 (0x2U)
#define ENETC_SI_PSIIDR_MR1_SHIFT                (1U)
#define ENETC_SI_PSIIDR_MR1(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIIDR_MR1_SHIFT)) & ENETC_SI_PSIIDR_MR1_MASK)

#define ENETC_SI_PSIIDR_RXR_MASK                 (0x10000U)
#define ENETC_SI_PSIIDR_RXR_SHIFT                (16U)
#define ENETC_SI_PSIIDR_RXR(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIIDR_RXR_SHIFT)) & ENETC_SI_PSIIDR_RXR_MASK)

#define ENETC_SI_PSIIDR_FLR1_MASK                (0x20000U)
#define ENETC_SI_PSIIDR_FLR1_SHIFT               (17U)
#define ENETC_SI_PSIIDR_FLR1(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_PSIIDR_FLR1_SHIFT)) & ENETC_SI_PSIIDR_FLR1_MASK)
/*! @} */

/*! @name VSIIER - Virtual station interface interrupt enable register */
/*! @{ */

#define ENETC_SI_VSIIER_MSIE_MASK                (0x100U)
#define ENETC_SI_VSIIER_MSIE_SHIFT               (8U)
#define ENETC_SI_VSIIER_MSIE(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIIER_MSIE_SHIFT)) & ENETC_SI_VSIIER_MSIE_MASK)

#define ENETC_SI_VSIIER_MRIE_MASK                (0x200U)
#define ENETC_SI_VSIIER_MRIE_SHIFT               (9U)
#define ENETC_SI_VSIIER_MRIE(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIIER_MRIE_SHIFT)) & ENETC_SI_VSIIER_MRIE_MASK)
/*! @} */

/*! @name VSIIDR - Virtual station interface interrupt detect register */
/*! @{ */

#define ENETC_SI_VSIIDR_TXR_MASK                 (0x1U)
#define ENETC_SI_VSIIDR_TXR_SHIFT                (0U)
#define ENETC_SI_VSIIDR_TXR(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIIDR_TXR_SHIFT)) & ENETC_SI_VSIIDR_TXR_MASK)

#define ENETC_SI_VSIIDR_MS_MASK                  (0x100U)
#define ENETC_SI_VSIIDR_MS_SHIFT                 (8U)
#define ENETC_SI_VSIIDR_MS(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIIDR_MS_SHIFT)) & ENETC_SI_VSIIDR_MS_MASK)

#define ENETC_SI_VSIIDR_MR_MASK                  (0x200U)
#define ENETC_SI_VSIIDR_MR_SHIFT                 (9U)
#define ENETC_SI_VSIIDR_MR(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIIDR_MR_SHIFT)) & ENETC_SI_VSIIDR_MR_MASK)

#define ENETC_SI_VSIIDR_RXR_MASK                 (0x10000U)
#define ENETC_SI_VSIIDR_RXR_SHIFT                (16U)
#define ENETC_SI_VSIIDR_RXR(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_VSIIDR_RXR_SHIFT)) & ENETC_SI_VSIIDR_RXR_MASK)
/*! @} */

/*! @name SITXIDR0 - Station interface transmit interrupt detect register 0 */
/*! @{ */

#define ENETC_SI_SITXIDR0_TXT0_MASK              (0x1U)
#define ENETC_SI_SITXIDR0_TXT0_SHIFT             (0U)
#define ENETC_SI_SITXIDR0_TXT0(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT0_SHIFT)) & ENETC_SI_SITXIDR0_TXT0_MASK)

#define ENETC_SI_SITXIDR0_TXT1_MASK              (0x2U)
#define ENETC_SI_SITXIDR0_TXT1_SHIFT             (1U)
#define ENETC_SI_SITXIDR0_TXT1(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT1_SHIFT)) & ENETC_SI_SITXIDR0_TXT1_MASK)

#define ENETC_SI_SITXIDR0_TXT2_MASK              (0x4U)
#define ENETC_SI_SITXIDR0_TXT2_SHIFT             (2U)
#define ENETC_SI_SITXIDR0_TXT2(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT2_SHIFT)) & ENETC_SI_SITXIDR0_TXT2_MASK)

#define ENETC_SI_SITXIDR0_TXT3_MASK              (0x8U)
#define ENETC_SI_SITXIDR0_TXT3_SHIFT             (3U)
#define ENETC_SI_SITXIDR0_TXT3(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT3_SHIFT)) & ENETC_SI_SITXIDR0_TXT3_MASK)

#define ENETC_SI_SITXIDR0_TXT4_MASK              (0x10U)
#define ENETC_SI_SITXIDR0_TXT4_SHIFT             (4U)
#define ENETC_SI_SITXIDR0_TXT4(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT4_SHIFT)) & ENETC_SI_SITXIDR0_TXT4_MASK)

#define ENETC_SI_SITXIDR0_TXT5_MASK              (0x20U)
#define ENETC_SI_SITXIDR0_TXT5_SHIFT             (5U)
#define ENETC_SI_SITXIDR0_TXT5(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT5_SHIFT)) & ENETC_SI_SITXIDR0_TXT5_MASK)

#define ENETC_SI_SITXIDR0_TXT6_MASK              (0x40U)
#define ENETC_SI_SITXIDR0_TXT6_SHIFT             (6U)
#define ENETC_SI_SITXIDR0_TXT6(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT6_SHIFT)) & ENETC_SI_SITXIDR0_TXT6_MASK)

#define ENETC_SI_SITXIDR0_TXT7_MASK              (0x80U)
#define ENETC_SI_SITXIDR0_TXT7_SHIFT             (7U)
#define ENETC_SI_SITXIDR0_TXT7(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT7_SHIFT)) & ENETC_SI_SITXIDR0_TXT7_MASK)

#define ENETC_SI_SITXIDR0_TXT8_MASK              (0x100U)
#define ENETC_SI_SITXIDR0_TXT8_SHIFT             (8U)
#define ENETC_SI_SITXIDR0_TXT8(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT8_SHIFT)) & ENETC_SI_SITXIDR0_TXT8_MASK)

#define ENETC_SI_SITXIDR0_TXT9_MASK              (0x200U)
#define ENETC_SI_SITXIDR0_TXT9_SHIFT             (9U)
#define ENETC_SI_SITXIDR0_TXT9(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT9_SHIFT)) & ENETC_SI_SITXIDR0_TXT9_MASK)

#define ENETC_SI_SITXIDR0_TXT10_MASK             (0x400U)
#define ENETC_SI_SITXIDR0_TXT10_SHIFT            (10U)
#define ENETC_SI_SITXIDR0_TXT10(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT10_SHIFT)) & ENETC_SI_SITXIDR0_TXT10_MASK)

#define ENETC_SI_SITXIDR0_TXT11_MASK             (0x800U)
#define ENETC_SI_SITXIDR0_TXT11_SHIFT            (11U)
#define ENETC_SI_SITXIDR0_TXT11(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT11_SHIFT)) & ENETC_SI_SITXIDR0_TXT11_MASK)

#define ENETC_SI_SITXIDR0_TXT12_MASK             (0x1000U)
#define ENETC_SI_SITXIDR0_TXT12_SHIFT            (12U)
#define ENETC_SI_SITXIDR0_TXT12(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT12_SHIFT)) & ENETC_SI_SITXIDR0_TXT12_MASK)

#define ENETC_SI_SITXIDR0_TXT13_MASK             (0x2000U)
#define ENETC_SI_SITXIDR0_TXT13_SHIFT            (13U)
#define ENETC_SI_SITXIDR0_TXT13(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXT13_SHIFT)) & ENETC_SI_SITXIDR0_TXT13_MASK)

#define ENETC_SI_SITXIDR0_TXF0_MASK              (0x10000U)
#define ENETC_SI_SITXIDR0_TXF0_SHIFT             (16U)
#define ENETC_SI_SITXIDR0_TXF0(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF0_SHIFT)) & ENETC_SI_SITXIDR0_TXF0_MASK)

#define ENETC_SI_SITXIDR0_TXF1_MASK              (0x20000U)
#define ENETC_SI_SITXIDR0_TXF1_SHIFT             (17U)
#define ENETC_SI_SITXIDR0_TXF1(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF1_SHIFT)) & ENETC_SI_SITXIDR0_TXF1_MASK)

#define ENETC_SI_SITXIDR0_TXF2_MASK              (0x40000U)
#define ENETC_SI_SITXIDR0_TXF2_SHIFT             (18U)
#define ENETC_SI_SITXIDR0_TXF2(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF2_SHIFT)) & ENETC_SI_SITXIDR0_TXF2_MASK)

#define ENETC_SI_SITXIDR0_TXF3_MASK              (0x80000U)
#define ENETC_SI_SITXIDR0_TXF3_SHIFT             (19U)
#define ENETC_SI_SITXIDR0_TXF3(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF3_SHIFT)) & ENETC_SI_SITXIDR0_TXF3_MASK)

#define ENETC_SI_SITXIDR0_TXF4_MASK              (0x100000U)
#define ENETC_SI_SITXIDR0_TXF4_SHIFT             (20U)
#define ENETC_SI_SITXIDR0_TXF4(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF4_SHIFT)) & ENETC_SI_SITXIDR0_TXF4_MASK)

#define ENETC_SI_SITXIDR0_TXF5_MASK              (0x200000U)
#define ENETC_SI_SITXIDR0_TXF5_SHIFT             (21U)
#define ENETC_SI_SITXIDR0_TXF5(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF5_SHIFT)) & ENETC_SI_SITXIDR0_TXF5_MASK)

#define ENETC_SI_SITXIDR0_TXF6_MASK              (0x400000U)
#define ENETC_SI_SITXIDR0_TXF6_SHIFT             (22U)
#define ENETC_SI_SITXIDR0_TXF6(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF6_SHIFT)) & ENETC_SI_SITXIDR0_TXF6_MASK)

#define ENETC_SI_SITXIDR0_TXF7_MASK              (0x800000U)
#define ENETC_SI_SITXIDR0_TXF7_SHIFT             (23U)
#define ENETC_SI_SITXIDR0_TXF7(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF7_SHIFT)) & ENETC_SI_SITXIDR0_TXF7_MASK)

#define ENETC_SI_SITXIDR0_TXF8_MASK              (0x1000000U)
#define ENETC_SI_SITXIDR0_TXF8_SHIFT             (24U)
#define ENETC_SI_SITXIDR0_TXF8(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF8_SHIFT)) & ENETC_SI_SITXIDR0_TXF8_MASK)

#define ENETC_SI_SITXIDR0_TXF9_MASK              (0x2000000U)
#define ENETC_SI_SITXIDR0_TXF9_SHIFT             (25U)
#define ENETC_SI_SITXIDR0_TXF9(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF9_SHIFT)) & ENETC_SI_SITXIDR0_TXF9_MASK)

#define ENETC_SI_SITXIDR0_TXF10_MASK             (0x4000000U)
#define ENETC_SI_SITXIDR0_TXF10_SHIFT            (26U)
#define ENETC_SI_SITXIDR0_TXF10(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF10_SHIFT)) & ENETC_SI_SITXIDR0_TXF10_MASK)

#define ENETC_SI_SITXIDR0_TXF11_MASK             (0x8000000U)
#define ENETC_SI_SITXIDR0_TXF11_SHIFT            (27U)
#define ENETC_SI_SITXIDR0_TXF11(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF11_SHIFT)) & ENETC_SI_SITXIDR0_TXF11_MASK)

#define ENETC_SI_SITXIDR0_TXF12_MASK             (0x10000000U)
#define ENETC_SI_SITXIDR0_TXF12_SHIFT            (28U)
#define ENETC_SI_SITXIDR0_TXF12(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF12_SHIFT)) & ENETC_SI_SITXIDR0_TXF12_MASK)

#define ENETC_SI_SITXIDR0_TXF13_MASK             (0x20000000U)
#define ENETC_SI_SITXIDR0_TXF13_SHIFT            (29U)
#define ENETC_SI_SITXIDR0_TXF13(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITXIDR0_TXF13_SHIFT)) & ENETC_SI_SITXIDR0_TXF13_MASK)
/*! @} */

/*! @name SIRXIDR0 - Station interface receive interrupt detect register 0 */
/*! @{ */

#define ENETC_SI_SIRXIDR0_RX0_MASK               (0x1U)
#define ENETC_SI_SIRXIDR0_RX0_SHIFT              (0U)
#define ENETC_SI_SIRXIDR0_RX0(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX0_SHIFT)) & ENETC_SI_SIRXIDR0_RX0_MASK)

#define ENETC_SI_SIRXIDR0_RX1_MASK               (0x2U)
#define ENETC_SI_SIRXIDR0_RX1_SHIFT              (1U)
#define ENETC_SI_SIRXIDR0_RX1(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX1_SHIFT)) & ENETC_SI_SIRXIDR0_RX1_MASK)

#define ENETC_SI_SIRXIDR0_RX2_MASK               (0x4U)
#define ENETC_SI_SIRXIDR0_RX2_SHIFT              (2U)
#define ENETC_SI_SIRXIDR0_RX2(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX2_SHIFT)) & ENETC_SI_SIRXIDR0_RX2_MASK)

#define ENETC_SI_SIRXIDR0_RX3_MASK               (0x8U)
#define ENETC_SI_SIRXIDR0_RX3_SHIFT              (3U)
#define ENETC_SI_SIRXIDR0_RX3(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX3_SHIFT)) & ENETC_SI_SIRXIDR0_RX3_MASK)

#define ENETC_SI_SIRXIDR0_RX4_MASK               (0x10U)
#define ENETC_SI_SIRXIDR0_RX4_SHIFT              (4U)
#define ENETC_SI_SIRXIDR0_RX4(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX4_SHIFT)) & ENETC_SI_SIRXIDR0_RX4_MASK)

#define ENETC_SI_SIRXIDR0_RX5_MASK               (0x20U)
#define ENETC_SI_SIRXIDR0_RX5_SHIFT              (5U)
#define ENETC_SI_SIRXIDR0_RX5(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX5_SHIFT)) & ENETC_SI_SIRXIDR0_RX5_MASK)

#define ENETC_SI_SIRXIDR0_RX6_MASK               (0x40U)
#define ENETC_SI_SIRXIDR0_RX6_SHIFT              (6U)
#define ENETC_SI_SIRXIDR0_RX6(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX6_SHIFT)) & ENETC_SI_SIRXIDR0_RX6_MASK)

#define ENETC_SI_SIRXIDR0_RX7_MASK               (0x80U)
#define ENETC_SI_SIRXIDR0_RX7_SHIFT              (7U)
#define ENETC_SI_SIRXIDR0_RX7(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX7_SHIFT)) & ENETC_SI_SIRXIDR0_RX7_MASK)

#define ENETC_SI_SIRXIDR0_RX8_MASK               (0x100U)
#define ENETC_SI_SIRXIDR0_RX8_SHIFT              (8U)
#define ENETC_SI_SIRXIDR0_RX8(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX8_SHIFT)) & ENETC_SI_SIRXIDR0_RX8_MASK)

#define ENETC_SI_SIRXIDR0_RX9_MASK               (0x200U)
#define ENETC_SI_SIRXIDR0_RX9_SHIFT              (9U)
#define ENETC_SI_SIRXIDR0_RX9(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX9_SHIFT)) & ENETC_SI_SIRXIDR0_RX9_MASK)

#define ENETC_SI_SIRXIDR0_RX10_MASK              (0x400U)
#define ENETC_SI_SIRXIDR0_RX10_SHIFT             (10U)
#define ENETC_SI_SIRXIDR0_RX10(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX10_SHIFT)) & ENETC_SI_SIRXIDR0_RX10_MASK)

#define ENETC_SI_SIRXIDR0_RX11_MASK              (0x800U)
#define ENETC_SI_SIRXIDR0_RX11_SHIFT             (11U)
#define ENETC_SI_SIRXIDR0_RX11(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX11_SHIFT)) & ENETC_SI_SIRXIDR0_RX11_MASK)

#define ENETC_SI_SIRXIDR0_RX12_MASK              (0x1000U)
#define ENETC_SI_SIRXIDR0_RX12_SHIFT             (12U)
#define ENETC_SI_SIRXIDR0_RX12(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX12_SHIFT)) & ENETC_SI_SIRXIDR0_RX12_MASK)

#define ENETC_SI_SIRXIDR0_RX13_MASK              (0x2000U)
#define ENETC_SI_SIRXIDR0_RX13_SHIFT             (13U)
#define ENETC_SI_SIRXIDR0_RX13(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIRXIDR0_RX13_SHIFT)) & ENETC_SI_SIRXIDR0_RX13_MASK)
/*! @} */

/*! @name SIMSIVR - Station interface MSI-X vector register */
/*! @{ */

#define ENETC_SI_SIMSIVR_VECTOR_MASK             (0x3FU)
#define ENETC_SI_SIMSIVR_VECTOR_SHIFT            (0U)
#define ENETC_SI_SIMSIVR_VECTOR(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMSIVR_VECTOR_SHIFT)) & ENETC_SI_SIMSIVR_VECTOR_MASK)
/*! @} */

/*! @name SICMSIVR - Station interface command MSI-X vector register */
/*! @{ */

#define ENETC_SI_SICMSIVR_VECTOR_MASK            (0x3FU)
#define ENETC_SI_SICMSIVR_VECTOR_SHIFT           (0U)
#define ENETC_SI_SICMSIVR_VECTOR(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICMSIVR_VECTOR_SHIFT)) & ENETC_SI_SICMSIVR_VECTOR_MASK)
/*! @} */

/*! @name SITMRIER - Station interface timer interrupt enable register */
/*! @{ */

#define ENETC_SI_SITMRIER_SYNCE_MASK             (0x1U)
#define ENETC_SI_SITMRIER_SYNCE_SHIFT            (0U)
#define ENETC_SI_SITMRIER_SYNCE(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITMRIER_SYNCE_SHIFT)) & ENETC_SI_SITMRIER_SYNCE_MASK)
/*! @} */

/*! @name SITMRIDR - Station interface timer interrupt detect register */
/*! @{ */

#define ENETC_SI_SITMRIDR_SYNC_MASK              (0x1U)
#define ENETC_SI_SITMRIDR_SYNC_SHIFT             (0U)
#define ENETC_SI_SITMRIDR_SYNC(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITMRIDR_SYNC_SHIFT)) & ENETC_SI_SITMRIDR_SYNC_MASK)
/*! @} */

/*! @name SITMRMSIVR - Station interface timer MSI-X vector register */
/*! @{ */

#define ENETC_SI_SITMRMSIVR_VECTOR_MASK          (0x3FU)
#define ENETC_SI_SITMRMSIVR_VECTOR_SHIFT         (0U)
#define ENETC_SI_SITMRMSIVR_VECTOR(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SITMRMSIVR_VECTOR_SHIFT)) & ENETC_SI_SITMRMSIVR_VECTOR_MASK)
/*! @} */

/*! @name SIMSITRVR - Station interface MSI-X transmit ring 0 vector register..Station interface MSI-X transmit ring 9 vector register */
/*! @{ */

#define ENETC_SI_SIMSITRVR_VECTOR_MASK           (0x3FU)
#define ENETC_SI_SIMSITRVR_VECTOR_SHIFT          (0U)
#define ENETC_SI_SIMSITRVR_VECTOR(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMSITRVR_VECTOR_SHIFT)) & ENETC_SI_SIMSITRVR_VECTOR_MASK)
/*! @} */

/* The count of ENETC_SI_SIMSITRVR */
#define ENETC_SI_SIMSITRVR_COUNT                 (10U)

/*! @name SIMSIRRVR - Station interface MSI-X receive ring 0 vector register..Station interface MSI-X receive ring 9 vector register */
/*! @{ */

#define ENETC_SI_SIMSIRRVR_VECTOR_MASK           (0x3FU)
#define ENETC_SI_SIMSIRRVR_VECTOR_SHIFT          (0U)
#define ENETC_SI_SIMSIRRVR_VECTOR(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMSIRRVR_VECTOR_SHIFT)) & ENETC_SI_SIMSIRRVR_VECTOR_MASK)
/*! @} */

/* The count of ENETC_SI_SIMSIRRVR */
#define ENETC_SI_SIMSIRRVR_COUNT                 (10U)

/*! @name SICMECR - Station interface correctable memory error configuration register */
/*! @{ */

#define ENETC_SI_SICMECR_THRESHOLD_MASK          (0xFFU)
#define ENETC_SI_SICMECR_THRESHOLD_SHIFT         (0U)
/*! THRESHOLD - Threshold */
#define ENETC_SI_SICMECR_THRESHOLD(x)            (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICMECR_THRESHOLD_SHIFT)) & ENETC_SI_SICMECR_THRESHOLD_MASK)
/*! @} */

/*! @name SICMESR - Station interface correctable memory error status register */
/*! @{ */

#define ENETC_SI_SICMESR_MEM_ID_MASK             (0x1F0000U)
#define ENETC_SI_SICMESR_MEM_ID_SHIFT            (16U)
/*! MEM_ID - Memory ID */
#define ENETC_SI_SICMESR_MEM_ID(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICMESR_MEM_ID_SHIFT)) & ENETC_SI_SICMESR_MEM_ID_MASK)

#define ENETC_SI_SICMESR_SBEE_MASK               (0x80000000U)
#define ENETC_SI_SICMESR_SBEE_SHIFT              (31U)
/*! SBEE - Single-bit ECC error */
#define ENETC_SI_SICMESR_SBEE(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICMESR_SBEE_SHIFT)) & ENETC_SI_SICMESR_SBEE_MASK)
/*! @} */

/*! @name SICMECTR - Station interface correctable memory error count register */
/*! @{ */

#define ENETC_SI_SICMECTR_COUNT_MASK             (0xFFU)
#define ENETC_SI_SICMECTR_COUNT_SHIFT            (0U)
/*! COUNT - Count */
#define ENETC_SI_SICMECTR_COUNT(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SICMECTR_COUNT_SHIFT)) & ENETC_SI_SICMECTR_COUNT_MASK)
/*! @} */

/*! @name SIUPECR - Station interface uncorrectable programming error configuration register */
/*! @{ */

#define ENETC_SI_SIUPECR_RD_MASK                 (0x80000000U)
#define ENETC_SI_SIUPECR_RD_SHIFT                (31U)
/*! RD - Report disable */
#define ENETC_SI_SIUPECR_RD(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUPECR_RD_SHIFT)) & ENETC_SI_SIUPECR_RD_MASK)
/*! @} */

/*! @name SIUPESR - Station interface uncorrectable programming error status register */
/*! @{ */

#define ENETC_SI_SIUPESR_DROP_SI_EN_MASK         (0x1U)
#define ENETC_SI_SIUPESR_DROP_SI_EN_SHIFT        (0U)
#define ENETC_SI_SIUPESR_DROP_SI_EN(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUPESR_DROP_SI_EN_SHIFT)) & ENETC_SI_SIUPESR_DROP_SI_EN_MASK)

#define ENETC_SI_SIUPESR_DROP_RING_EN_MASK       (0x2U)
#define ENETC_SI_SIUPESR_DROP_RING_EN_SHIFT      (1U)
#define ENETC_SI_SIUPESR_DROP_RING_EN(x)         (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUPESR_DROP_RING_EN_SHIFT)) & ENETC_SI_SIUPESR_DROP_RING_EN_MASK)

#define ENETC_SI_SIUPESR_DROP_RING_SEL_MASK      (0x8U)
#define ENETC_SI_SIUPESR_DROP_RING_SEL_SHIFT     (3U)
#define ENETC_SI_SIUPESR_DROP_RING_SEL(x)        (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUPESR_DROP_RING_SEL_SHIFT)) & ENETC_SI_SIUPESR_DROP_RING_SEL_MASK)

#define ENETC_SI_SIUPESR_M_MASK                  (0x40000000U)
#define ENETC_SI_SIUPESR_M_SHIFT                 (30U)
/*! M - Multiple */
#define ENETC_SI_SIUPESR_M(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUPESR_M_SHIFT)) & ENETC_SI_SIUPESR_M_MASK)

#define ENETC_SI_SIUPESR_PE_MASK                 (0x80000000U)
#define ENETC_SI_SIUPESR_PE_SHIFT                (31U)
/*! PE - Programming error */
#define ENETC_SI_SIUPESR_PE(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUPESR_PE_SHIFT)) & ENETC_SI_SIUPESR_PE_MASK)
/*! @} */

/*! @name SIUPECTR - Station interface uncorrectable programming error count register */
/*! @{ */

#define ENETC_SI_SIUPECTR_COUNT_MASK             (0xFFFFFFFFU)
#define ENETC_SI_SIUPECTR_COUNT_SHIFT            (0U)
/*! COUNT - Count */
#define ENETC_SI_SIUPECTR_COUNT(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUPECTR_COUNT_SHIFT)) & ENETC_SI_SIUPECTR_COUNT_MASK)
/*! @} */

/*! @name SIUNSBECR - Station interface uncorrectable non-fatal system bus error configuration register */
/*! @{ */

#define ENETC_SI_SIUNSBECR_THRESHOLD_MASK        (0xFFU)
#define ENETC_SI_SIUNSBECR_THRESHOLD_SHIFT       (0U)
/*! THRESHOLD - Threshold */
#define ENETC_SI_SIUNSBECR_THRESHOLD(x)          (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNSBECR_THRESHOLD_SHIFT)) & ENETC_SI_SIUNSBECR_THRESHOLD_MASK)
/*! @} */

/*! @name SIUNSBESR - Station interface uncorrectable non-fatal system bus error status register */
/*! @{ */

#define ENETC_SI_SIUNSBESR_SB_ID_MASK            (0xFU)
#define ENETC_SI_SIUNSBESR_SB_ID_SHIFT           (0U)
/*! SB_ID - System Bus ID */
#define ENETC_SI_SIUNSBESR_SB_ID(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNSBESR_SB_ID_SHIFT)) & ENETC_SI_SIUNSBESR_SB_ID_MASK)

#define ENETC_SI_SIUNSBESR_SBE_MASK              (0x80000000U)
#define ENETC_SI_SIUNSBESR_SBE_SHIFT             (31U)
/*! SBE - System bus error */
#define ENETC_SI_SIUNSBESR_SBE(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNSBESR_SBE_SHIFT)) & ENETC_SI_SIUNSBESR_SBE_MASK)
/*! @} */

/*! @name SIUNSBECTR - Station interface uncorrectable non-fatal system bus error count register */
/*! @{ */

#define ENETC_SI_SIUNSBECTR_COUNT_MASK           (0xFFU)
#define ENETC_SI_SIUNSBECTR_COUNT_SHIFT          (0U)
/*! COUNT - Count */
#define ENETC_SI_SIUNSBECTR_COUNT(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNSBECTR_COUNT_SHIFT)) & ENETC_SI_SIUNSBECTR_COUNT_MASK)
/*! @} */

/*! @name SIUFSBECR - Station interface uncorrectable fatal system bus error configuration register */
/*! @{ */

#define ENETC_SI_SIUFSBECR_RD_MASK               (0x80000000U)
#define ENETC_SI_SIUFSBECR_RD_SHIFT              (31U)
/*! RD - Report disable */
#define ENETC_SI_SIUFSBECR_RD(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFSBECR_RD_SHIFT)) & ENETC_SI_SIUFSBECR_RD_MASK)
/*! @} */

/*! @name SIUFSBESR - Station interface uncorrectable fatal system bus error status register */
/*! @{ */

#define ENETC_SI_SIUFSBESR_SB_ID_MASK            (0xFU)
#define ENETC_SI_SIUFSBESR_SB_ID_SHIFT           (0U)
/*! SB_ID - System Bus ID */
#define ENETC_SI_SIUFSBESR_SB_ID(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFSBESR_SB_ID_SHIFT)) & ENETC_SI_SIUFSBESR_SB_ID_MASK)

#define ENETC_SI_SIUFSBESR_M_MASK                (0x40000000U)
#define ENETC_SI_SIUFSBESR_M_SHIFT               (30U)
/*! M - Multiple */
#define ENETC_SI_SIUFSBESR_M(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFSBESR_M_SHIFT)) & ENETC_SI_SIUFSBESR_M_MASK)

#define ENETC_SI_SIUFSBESR_SBE_MASK              (0x80000000U)
#define ENETC_SI_SIUFSBESR_SBE_SHIFT             (31U)
/*! SBE - System bus error */
#define ENETC_SI_SIUFSBESR_SBE(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFSBESR_SBE_SHIFT)) & ENETC_SI_SIUFSBESR_SBE_MASK)
/*! @} */

/*! @name SIUNMECR - Station interface uncorrectable non-fatal memory error configuration register */
/*! @{ */

#define ENETC_SI_SIUNMECR_THRESHOLD_MASK         (0xFFU)
#define ENETC_SI_SIUNMECR_THRESHOLD_SHIFT        (0U)
/*! THRESHOLD - Threshold */
#define ENETC_SI_SIUNMECR_THRESHOLD(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNMECR_THRESHOLD_SHIFT)) & ENETC_SI_SIUNMECR_THRESHOLD_MASK)

#define ENETC_SI_SIUNMECR_RD_MASK                (0x80000000U)
#define ENETC_SI_SIUNMECR_RD_SHIFT               (31U)
/*! RD - Report disable */
#define ENETC_SI_SIUNMECR_RD(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNMECR_RD_SHIFT)) & ENETC_SI_SIUNMECR_RD_MASK)
/*! @} */

/*! @name SIUNMESR0 - Station interface uncorrectable non-fatal memory error status register 0 */
/*! @{ */

#define ENETC_SI_SIUNMESR0_SYNDROME_MASK         (0x7FFU)
#define ENETC_SI_SIUNMESR0_SYNDROME_SHIFT        (0U)
/*! SYNDROME - Syndrome */
#define ENETC_SI_SIUNMESR0_SYNDROME(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNMESR0_SYNDROME_SHIFT)) & ENETC_SI_SIUNMESR0_SYNDROME_MASK)

#define ENETC_SI_SIUNMESR0_MEM_ID_MASK           (0x1F0000U)
#define ENETC_SI_SIUNMESR0_MEM_ID_SHIFT          (16U)
/*! MEM_ID - Memory ID */
#define ENETC_SI_SIUNMESR0_MEM_ID(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNMESR0_MEM_ID_SHIFT)) & ENETC_SI_SIUNMESR0_MEM_ID_MASK)

#define ENETC_SI_SIUNMESR0_MBEE_MASK             (0x80000000U)
#define ENETC_SI_SIUNMESR0_MBEE_SHIFT            (31U)
/*! MBEE - Multi-bit ECC error */
#define ENETC_SI_SIUNMESR0_MBEE(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNMESR0_MBEE_SHIFT)) & ENETC_SI_SIUNMESR0_MBEE_MASK)
/*! @} */

/*! @name SIUNMESR1 - Station interface uncorrectable non-fatal memory error status register 1 */
/*! @{ */

#define ENETC_SI_SIUNMESR1_ADDR_MASK             (0xFFFFFFFFU)
#define ENETC_SI_SIUNMESR1_ADDR_SHIFT            (0U)
/*! ADDR - Address */
#define ENETC_SI_SIUNMESR1_ADDR(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNMESR1_ADDR_SHIFT)) & ENETC_SI_SIUNMESR1_ADDR_MASK)
/*! @} */

/*! @name SIUNMECTR - Station interface uncorrectable non-fatal memory error count register */
/*! @{ */

#define ENETC_SI_SIUNMECTR_COUNT_MASK            (0xFFU)
#define ENETC_SI_SIUNMECTR_COUNT_SHIFT           (0U)
/*! COUNT - Count */
#define ENETC_SI_SIUNMECTR_COUNT(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUNMECTR_COUNT_SHIFT)) & ENETC_SI_SIUNMECTR_COUNT_MASK)
/*! @} */

/*! @name SIUFMECR - Station interface uncorrectable fatal memory error configuration register */
/*! @{ */

#define ENETC_SI_SIUFMECR_RD_MASK                (0x80000000U)
#define ENETC_SI_SIUFMECR_RD_SHIFT               (31U)
/*! RD - Report disable */
#define ENETC_SI_SIUFMECR_RD(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFMECR_RD_SHIFT)) & ENETC_SI_SIUFMECR_RD_MASK)
/*! @} */

/*! @name SIUFMESR0 - Station interface uncorrectable fatal memory error status register 0 */
/*! @{ */

#define ENETC_SI_SIUFMESR0_SYNDROME_MASK         (0x7FFU)
#define ENETC_SI_SIUFMESR0_SYNDROME_SHIFT        (0U)
/*! SYNDROME - Syndrome */
#define ENETC_SI_SIUFMESR0_SYNDROME(x)           (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFMESR0_SYNDROME_SHIFT)) & ENETC_SI_SIUFMESR0_SYNDROME_MASK)

#define ENETC_SI_SIUFMESR0_MEM_ID_MASK           (0x1F0000U)
#define ENETC_SI_SIUFMESR0_MEM_ID_SHIFT          (16U)
/*! MEM_ID - Memory ID */
#define ENETC_SI_SIUFMESR0_MEM_ID(x)             (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFMESR0_MEM_ID_SHIFT)) & ENETC_SI_SIUFMESR0_MEM_ID_MASK)

#define ENETC_SI_SIUFMESR0_M_MASK                (0x40000000U)
#define ENETC_SI_SIUFMESR0_M_SHIFT               (30U)
/*! M - Multiple */
#define ENETC_SI_SIUFMESR0_M(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFMESR0_M_SHIFT)) & ENETC_SI_SIUFMESR0_M_MASK)

#define ENETC_SI_SIUFMESR0_MBEE_MASK             (0x80000000U)
#define ENETC_SI_SIUFMESR0_MBEE_SHIFT            (31U)
/*! MBEE - Multi-bit ECC error */
#define ENETC_SI_SIUFMESR0_MBEE(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFMESR0_MBEE_SHIFT)) & ENETC_SI_SIUFMESR0_MBEE_MASK)
/*! @} */

/*! @name SIUFMESR1 - Station interface uncorrectable fatal memory error status register 1 */
/*! @{ */

#define ENETC_SI_SIUFMESR1_ADDR_MASK             (0xFFFFFFFFU)
#define ENETC_SI_SIUFMESR1_ADDR_SHIFT            (0U)
/*! ADDR - Address */
#define ENETC_SI_SIUFMESR1_ADDR(x)               (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIUFMESR1_ADDR_SHIFT)) & ENETC_SI_SIUFMESR1_ADDR_MASK)
/*! @} */

/*! @name SIMAFTCAPR - Station interface MAC address filter table capability register */
/*! @{ */

#define ENETC_SI_SIMAFTCAPR_NUM_MAC_AFTE_MASK    (0xFFU)
#define ENETC_SI_SIMAFTCAPR_NUM_MAC_AFTE_SHIFT   (0U)
#define ENETC_SI_SIMAFTCAPR_NUM_MAC_AFTE(x)      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIMAFTCAPR_NUM_MAC_AFTE_SHIFT)) & ENETC_SI_SIMAFTCAPR_NUM_MAC_AFTE_MASK)
/*! @} */

/*! @name SIVFTCAPR - Station interface VLAN filter table capability register */
/*! @{ */

#define ENETC_SI_SIVFTCAPR_NUM_VLAN_FTE_MASK     (0xFFU)
#define ENETC_SI_SIVFTCAPR_NUM_VLAN_FTE_SHIFT    (0U)
#define ENETC_SI_SIVFTCAPR_NUM_VLAN_FTE(x)       (((uint32_t)(((uint32_t)(x)) << ENETC_SI_SIVFTCAPR_NUM_VLAN_FTE_SHIFT)) & ENETC_SI_SIVFTCAPR_NUM_VLAN_FTE_MASK)
/*! @} */

/*! @name TBMR - Tx BDR 0 mode register..Tx BDR 9 mode register */
/*! @{ */

#define ENETC_SI_TBMR_PRIO_MASK                  (0x7U)
#define ENETC_SI_TBMR_PRIO_SHIFT                 (0U)
#define ENETC_SI_TBMR_PRIO(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBMR_PRIO_SHIFT)) & ENETC_SI_TBMR_PRIO_MASK)

#define ENETC_SI_TBMR_WRR_MASK                   (0x70U)
#define ENETC_SI_TBMR_WRR_SHIFT                  (4U)
#define ENETC_SI_TBMR_WRR(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBMR_WRR_SHIFT)) & ENETC_SI_TBMR_WRR_MASK)

#define ENETC_SI_TBMR_CRC_MASK                   (0x100U)
#define ENETC_SI_TBMR_CRC_SHIFT                  (8U)
#define ENETC_SI_TBMR_CRC(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBMR_CRC_SHIFT)) & ENETC_SI_TBMR_CRC_MASK)

#define ENETC_SI_TBMR_VIH_MASK                   (0x200U)
#define ENETC_SI_TBMR_VIH_SHIFT                  (9U)
#define ENETC_SI_TBMR_VIH(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBMR_VIH_SHIFT)) & ENETC_SI_TBMR_VIH_MASK)

#define ENETC_SI_TBMR_EN_MASK                    (0x80000000U)
#define ENETC_SI_TBMR_EN_SHIFT                   (31U)
#define ENETC_SI_TBMR_EN(x)                      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBMR_EN_SHIFT)) & ENETC_SI_TBMR_EN_MASK)
/*! @} */

/* The count of ENETC_SI_TBMR */
#define ENETC_SI_TBMR_COUNT                      (10U)

/*! @name TBSR - Tx BDR 0 status register..Tx BDR 9 status register */
/*! @{ */

#define ENETC_SI_TBSR_BUSY_MASK                  (0x1U)
#define ENETC_SI_TBSR_BUSY_SHIFT                 (0U)
#define ENETC_SI_TBSR_BUSY(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBSR_BUSY_SHIFT)) & ENETC_SI_TBSR_BUSY_MASK)

#define ENETC_SI_TBSR_SBE_MASK                   (0x10000U)
#define ENETC_SI_TBSR_SBE_SHIFT                  (16U)
#define ENETC_SI_TBSR_SBE(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBSR_SBE_SHIFT)) & ENETC_SI_TBSR_SBE_MASK)
/*! @} */

/* The count of ENETC_SI_TBSR */
#define ENETC_SI_TBSR_COUNT                      (10U)

/*! @name TBBAR0 - Tx BDR 0 base address register 0..Tx BDR 9 base address register 0 */
/*! @{ */

#define ENETC_SI_TBBAR0_ADDRL_MASK               (0xFFFFFF80U)
#define ENETC_SI_TBBAR0_ADDRL_SHIFT              (7U)
#define ENETC_SI_TBBAR0_ADDRL(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBBAR0_ADDRL_SHIFT)) & ENETC_SI_TBBAR0_ADDRL_MASK)
/*! @} */

/* The count of ENETC_SI_TBBAR0 */
#define ENETC_SI_TBBAR0_COUNT                    (10U)

/*! @name TBBAR1 - Tx BDR 0 base address register 1..Tx BDR 9 base address register 1 */
/*! @{ */

#define ENETC_SI_TBBAR1_ADDRH_MASK               (0xFFFFFFFFU)
#define ENETC_SI_TBBAR1_ADDRH_SHIFT              (0U)
#define ENETC_SI_TBBAR1_ADDRH(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBBAR1_ADDRH_SHIFT)) & ENETC_SI_TBBAR1_ADDRH_MASK)
/*! @} */

/* The count of ENETC_SI_TBBAR1 */
#define ENETC_SI_TBBAR1_COUNT                    (10U)

/*! @name TBPIR - Tx BDR 0 producer index register..Tx BDR 9 producer index register */
/*! @{ */

#define ENETC_SI_TBPIR_BDR_INDEX_MASK            (0xFFFFU)
#define ENETC_SI_TBPIR_BDR_INDEX_SHIFT           (0U)
#define ENETC_SI_TBPIR_BDR_INDEX(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBPIR_BDR_INDEX_SHIFT)) & ENETC_SI_TBPIR_BDR_INDEX_MASK)
/*! @} */

/* The count of ENETC_SI_TBPIR */
#define ENETC_SI_TBPIR_COUNT                     (10U)

/*! @name TBCIR - Tx BDR 0 consumer index register..Tx BDR 9 consumer index register */
/*! @{ */

#define ENETC_SI_TBCIR_BDR_INDEX_MASK            (0xFFFFU)
#define ENETC_SI_TBCIR_BDR_INDEX_SHIFT           (0U)
#define ENETC_SI_TBCIR_BDR_INDEX(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBCIR_BDR_INDEX_SHIFT)) & ENETC_SI_TBCIR_BDR_INDEX_MASK)

#define ENETC_SI_TBCIR_STAT_ID_MASK              (0xFFFF0000U)
#define ENETC_SI_TBCIR_STAT_ID_SHIFT             (16U)
#define ENETC_SI_TBCIR_STAT_ID(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBCIR_STAT_ID_SHIFT)) & ENETC_SI_TBCIR_STAT_ID_MASK)
/*! @} */

/* The count of ENETC_SI_TBCIR */
#define ENETC_SI_TBCIR_COUNT                     (10U)

/*! @name TBLENR - Tx BDR 0 length register..Tx BDR 9 length register */
/*! @{ */

#define ENETC_SI_TBLENR_LENGTH_MASK              (0x1FFF8U)
#define ENETC_SI_TBLENR_LENGTH_SHIFT             (3U)
#define ENETC_SI_TBLENR_LENGTH(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBLENR_LENGTH_SHIFT)) & ENETC_SI_TBLENR_LENGTH_MASK)
/*! @} */

/* The count of ENETC_SI_TBLENR */
#define ENETC_SI_TBLENR_COUNT                    (10U)

/*! @name TBIER - Tx BDR 0 interrupt enable register..Tx BDR 9 interrupt enable register */
/*! @{ */

#define ENETC_SI_TBIER_TXTIE_MASK                (0x1U)
#define ENETC_SI_TBIER_TXTIE_SHIFT               (0U)
#define ENETC_SI_TBIER_TXTIE(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBIER_TXTIE_SHIFT)) & ENETC_SI_TBIER_TXTIE_MASK)

#define ENETC_SI_TBIER_TXFIE_MASK                (0x2U)
#define ENETC_SI_TBIER_TXFIE_SHIFT               (1U)
#define ENETC_SI_TBIER_TXFIE(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBIER_TXFIE_SHIFT)) & ENETC_SI_TBIER_TXFIE_MASK)
/*! @} */

/* The count of ENETC_SI_TBIER */
#define ENETC_SI_TBIER_COUNT                     (10U)

/*! @name TBIDR - Tx BDR 0 interrupt detect register..Tx BDR 9 interrupt detect register */
/*! @{ */

#define ENETC_SI_TBIDR_TXT_MASK                  (0x1U)
#define ENETC_SI_TBIDR_TXT_SHIFT                 (0U)
#define ENETC_SI_TBIDR_TXT(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBIDR_TXT_SHIFT)) & ENETC_SI_TBIDR_TXT_MASK)

#define ENETC_SI_TBIDR_TXF_MASK                  (0x2U)
#define ENETC_SI_TBIDR_TXF_SHIFT                 (1U)
#define ENETC_SI_TBIDR_TXF(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBIDR_TXF_SHIFT)) & ENETC_SI_TBIDR_TXF_MASK)
/*! @} */

/* The count of ENETC_SI_TBIDR */
#define ENETC_SI_TBIDR_COUNT                     (10U)

/*! @name TBICR0 - Tx BDR 0 interrupt coalescing register 0..Tx BDR 9 interrupt coalescing register 0 */
/*! @{ */

#define ENETC_SI_TBICR0_ICPT_MASK                (0xFU)
#define ENETC_SI_TBICR0_ICPT_SHIFT               (0U)
#define ENETC_SI_TBICR0_ICPT(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBICR0_ICPT_SHIFT)) & ENETC_SI_TBICR0_ICPT_MASK)

#define ENETC_SI_TBICR0_ICEN_MASK                (0x80000000U)
#define ENETC_SI_TBICR0_ICEN_SHIFT               (31U)
#define ENETC_SI_TBICR0_ICEN(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBICR0_ICEN_SHIFT)) & ENETC_SI_TBICR0_ICEN_MASK)
/*! @} */

/* The count of ENETC_SI_TBICR0 */
#define ENETC_SI_TBICR0_COUNT                    (10U)

/*! @name TBICR1 - Tx BDR 0 interrupt coalescing register 1..Tx BDR 9 interrupt coalescing register 1 */
/*! @{ */

#define ENETC_SI_TBICR1_ICTT_MASK                (0xFFFFFFFFU)
#define ENETC_SI_TBICR1_ICTT_SHIFT               (0U)
#define ENETC_SI_TBICR1_ICTT(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_TBICR1_ICTT_SHIFT)) & ENETC_SI_TBICR1_ICTT_MASK)
/*! @} */

/* The count of ENETC_SI_TBICR1 */
#define ENETC_SI_TBICR1_COUNT                    (10U)

/*! @name RBMR - Rx BDR 0 mode register..Rx BDR 9 mode register */
/*! @{ */

#define ENETC_SI_RBMR_AL_MASK                    (0x1U)
#define ENETC_SI_RBMR_AL_SHIFT                   (0U)
#define ENETC_SI_RBMR_AL(x)                      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBMR_AL_SHIFT)) & ENETC_SI_RBMR_AL_MASK)

#define ENETC_SI_RBMR_BDS_MASK                   (0x4U)
#define ENETC_SI_RBMR_BDS_SHIFT                  (2U)
#define ENETC_SI_RBMR_BDS(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBMR_BDS_SHIFT)) & ENETC_SI_RBMR_BDS_MASK)

#define ENETC_SI_RBMR_CM_MASK                    (0x10U)
#define ENETC_SI_RBMR_CM_SHIFT                   (4U)
#define ENETC_SI_RBMR_CM(x)                      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBMR_CM_SHIFT)) & ENETC_SI_RBMR_CM_MASK)

#define ENETC_SI_RBMR_VTE_MASK                   (0x20U)
#define ENETC_SI_RBMR_VTE_SHIFT                  (5U)
#define ENETC_SI_RBMR_VTE(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBMR_VTE_SHIFT)) & ENETC_SI_RBMR_VTE_MASK)

#define ENETC_SI_RBMR_VTPD_MASK                  (0x40U)
#define ENETC_SI_RBMR_VTPD_SHIFT                 (6U)
#define ENETC_SI_RBMR_VTPD(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBMR_VTPD_SHIFT)) & ENETC_SI_RBMR_VTPD_MASK)

#define ENETC_SI_RBMR_CRC_MASK                   (0x100U)
#define ENETC_SI_RBMR_CRC_SHIFT                  (8U)
#define ENETC_SI_RBMR_CRC(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBMR_CRC_SHIFT)) & ENETC_SI_RBMR_CRC_MASK)

#define ENETC_SI_RBMR_EN_MASK                    (0x80000000U)
#define ENETC_SI_RBMR_EN_SHIFT                   (31U)
#define ENETC_SI_RBMR_EN(x)                      (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBMR_EN_SHIFT)) & ENETC_SI_RBMR_EN_MASK)
/*! @} */

/* The count of ENETC_SI_RBMR */
#define ENETC_SI_RBMR_COUNT                      (10U)

/*! @name RBSR - Rx BDR 0 status register..Rx BDR 9 status register */
/*! @{ */

#define ENETC_SI_RBSR_EMPTY_MASK                 (0x1U)
#define ENETC_SI_RBSR_EMPTY_SHIFT                (0U)
#define ENETC_SI_RBSR_EMPTY(x)                   (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBSR_EMPTY_SHIFT)) & ENETC_SI_RBSR_EMPTY_MASK)

#define ENETC_SI_RBSR_SBE_MASK                   (0x10000U)
#define ENETC_SI_RBSR_SBE_SHIFT                  (16U)
#define ENETC_SI_RBSR_SBE(x)                     (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBSR_SBE_SHIFT)) & ENETC_SI_RBSR_SBE_MASK)
/*! @} */

/* The count of ENETC_SI_RBSR */
#define ENETC_SI_RBSR_COUNT                      (10U)

/*! @name RBBSR - Rx BDR 0 buffer size register..Rx BDR 9 buffer size register */
/*! @{ */

#define ENETC_SI_RBBSR_BSIZE_MASK                (0xFFFFU)
#define ENETC_SI_RBBSR_BSIZE_SHIFT               (0U)
#define ENETC_SI_RBBSR_BSIZE(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBBSR_BSIZE_SHIFT)) & ENETC_SI_RBBSR_BSIZE_MASK)
/*! @} */

/* The count of ENETC_SI_RBBSR */
#define ENETC_SI_RBBSR_COUNT                     (10U)

/*! @name RBCIR - Rx BDR 0 consumer index register..Rx BDR 9 consumer index register */
/*! @{ */

#define ENETC_SI_RBCIR_BDR_INDEX_MASK            (0xFFFFU)
#define ENETC_SI_RBCIR_BDR_INDEX_SHIFT           (0U)
#define ENETC_SI_RBCIR_BDR_INDEX(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBCIR_BDR_INDEX_SHIFT)) & ENETC_SI_RBCIR_BDR_INDEX_MASK)
/*! @} */

/* The count of ENETC_SI_RBCIR */
#define ENETC_SI_RBCIR_COUNT                     (10U)

/*! @name RBBAR0 - Rx BDR 0 base address register 0..Rx BDR 9 base address register 0 */
/*! @{ */

#define ENETC_SI_RBBAR0_ADDRL_MASK               (0xFFFFFF80U)
#define ENETC_SI_RBBAR0_ADDRL_SHIFT              (7U)
#define ENETC_SI_RBBAR0_ADDRL(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBBAR0_ADDRL_SHIFT)) & ENETC_SI_RBBAR0_ADDRL_MASK)
/*! @} */

/* The count of ENETC_SI_RBBAR0 */
#define ENETC_SI_RBBAR0_COUNT                    (10U)

/*! @name RBBAR1 - Rx BDR 0 base address register 1..Rx BDR 9 base address register 1 */
/*! @{ */

#define ENETC_SI_RBBAR1_ADDRH_MASK               (0xFFFFFFFFU)
#define ENETC_SI_RBBAR1_ADDRH_SHIFT              (0U)
#define ENETC_SI_RBBAR1_ADDRH(x)                 (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBBAR1_ADDRH_SHIFT)) & ENETC_SI_RBBAR1_ADDRH_MASK)
/*! @} */

/* The count of ENETC_SI_RBBAR1 */
#define ENETC_SI_RBBAR1_COUNT                    (10U)

/*! @name RBPIR - Rx BDR 0 producer index register..Rx BDR 9 producer index register */
/*! @{ */

#define ENETC_SI_RBPIR_BDR_INDEX_MASK            (0xFFFFU)
#define ENETC_SI_RBPIR_BDR_INDEX_SHIFT           (0U)
#define ENETC_SI_RBPIR_BDR_INDEX(x)              (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBPIR_BDR_INDEX_SHIFT)) & ENETC_SI_RBPIR_BDR_INDEX_MASK)
/*! @} */

/* The count of ENETC_SI_RBPIR */
#define ENETC_SI_RBPIR_COUNT                     (10U)

/*! @name RBLENR - Rx BDR 0 length register..Rx BDR 9 length register */
/*! @{ */

#define ENETC_SI_RBLENR_LENGTH_MASK              (0x1FFF8U)
#define ENETC_SI_RBLENR_LENGTH_SHIFT             (3U)
#define ENETC_SI_RBLENR_LENGTH(x)                (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBLENR_LENGTH_SHIFT)) & ENETC_SI_RBLENR_LENGTH_MASK)
/*! @} */

/* The count of ENETC_SI_RBLENR */
#define ENETC_SI_RBLENR_COUNT                    (10U)

/*! @name RBDCR - Rx BDR 0 drop count register..Rx BDR 9 drop count register */
/*! @{ */

#define ENETC_SI_RBDCR_COUNT_MASK                (0xFFFFFFFFU)
#define ENETC_SI_RBDCR_COUNT_SHIFT               (0U)
#define ENETC_SI_RBDCR_COUNT(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBDCR_COUNT_SHIFT)) & ENETC_SI_RBDCR_COUNT_MASK)
/*! @} */

/* The count of ENETC_SI_RBDCR */
#define ENETC_SI_BDR_RBDCR_COUNT                 (10U)

/*! @name RBIER - Rx BDR 0 interrupt enable register..Rx BDR 9 interrupt enable register */
/*! @{ */

#define ENETC_SI_RBIER_RXTIE_MASK                (0x1U)
#define ENETC_SI_RBIER_RXTIE_SHIFT               (0U)
#define ENETC_SI_RBIER_RXTIE(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBIER_RXTIE_SHIFT)) & ENETC_SI_RBIER_RXTIE_MASK)
/*! @} */

/* The count of ENETC_SI_RBIER */
#define ENETC_SI_RBIER_COUNT                     (10U)

/*! @name RBIDR - Rx BDR 0 interrupt detect register..Rx BDR 9 interrupt detect register */
/*! @{ */

#define ENETC_SI_RBIDR_RXT_MASK                  (0x1U)
#define ENETC_SI_RBIDR_RXT_SHIFT                 (0U)
#define ENETC_SI_RBIDR_RXT(x)                    (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBIDR_RXT_SHIFT)) & ENETC_SI_RBIDR_RXT_MASK)
/*! @} */

/* The count of ENETC_SI_RBIDR */
#define ENETC_SI_RBIDR_COUNT                     (10U)

/*! @name RBICR0 - Rx BDR 0 interrupt coalescing register 0..Rx BDR 9 interrupt coalescing register 0 */
/*! @{ */

#define ENETC_SI_RBICR0_ICPT_MASK                (0x1FFU)
#define ENETC_SI_RBICR0_ICPT_SHIFT               (0U)
#define ENETC_SI_RBICR0_ICPT(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBICR0_ICPT_SHIFT)) & ENETC_SI_RBICR0_ICPT_MASK)

#define ENETC_SI_RBICR0_ICEN_MASK                (0x80000000U)
#define ENETC_SI_RBICR0_ICEN_SHIFT               (31U)
#define ENETC_SI_RBICR0_ICEN(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBICR0_ICEN_SHIFT)) & ENETC_SI_RBICR0_ICEN_MASK)
/*! @} */

/* The count of ENETC_SI_RBICR0 */
#define ENETC_SI_RBICR0_COUNT                    (10U)

/*! @name RBICR1 - Rx BDR 0 interrupt coalescing register 1..Rx BDR 9 interrupt coalescing register 1 */
/*! @{ */

#define ENETC_SI_RBICR1_ICTT_MASK                (0xFFFFFFFFU)
#define ENETC_SI_RBICR1_ICTT_SHIFT               (0U)
#define ENETC_SI_RBICR1_ICTT(x)                  (((uint32_t)(((uint32_t)(x)) << ENETC_SI_RBICR1_ICTT_SHIFT)) & ENETC_SI_RBICR1_ICTT_MASK)
/*! @} */

/* The count of ENETC_SI_RBICR1 */
#define ENETC_SI_RBICR1_COUNT                    (10U)


/*!
 * @}
 */ /* end of group ENETC_SI_Register_Masks */


/* ENETC_SI - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ENETC0_SI0 base address */
  #define ENETC0_SI0_BASE                          (0x70B00000u)
  /** Peripheral ENETC0_SI0 base address */
  #define ENETC0_SI0_BASE_NS                       (0x60B00000u)
  /** Peripheral ENETC0_SI0 base pointer */
  #define ENETC0_SI0                               ((ENETC_SI_Type *)ENETC0_SI0_BASE)
  /** Peripheral ENETC0_SI0 base pointer */
  #define ENETC0_SI0_NS                            ((ENETC_SI_Type *)ENETC0_SI0_BASE_NS)
  /** Peripheral ENETC1_SI0 base address */
  #define ENETC1_SI0_BASE                          (0x70B40000u)
  /** Peripheral ENETC1_SI0 base address */
  #define ENETC1_SI0_BASE_NS                       (0x60B40000u)
  /** Peripheral ENETC1_SI0 base pointer */
  #define ENETC1_SI0                               ((ENETC_SI_Type *)ENETC1_SI0_BASE)
  /** Peripheral ENETC1_SI0 base pointer */
  #define ENETC1_SI0_NS                            ((ENETC_SI_Type *)ENETC1_SI0_BASE_NS)
  /** Peripheral ENETC1_SI1 base address */
  #define ENETC1_SI1_BASE                          (0x70C10000u)
  /** Peripheral ENETC1_SI1 base address */
  #define ENETC1_SI1_BASE_NS                       (0x60C10000u)
  /** Peripheral ENETC1_SI1 base pointer */
  #define ENETC1_SI1                               ((ENETC_SI_Type *)ENETC1_SI1_BASE)
  /** Peripheral ENETC1_SI1 base pointer */
  #define ENETC1_SI1_NS                            ((ENETC_SI_Type *)ENETC1_SI1_BASE_NS)
  /** Array initializer of ENETC_SI peripheral base addresses */
  #define ENETC_SI_BASE_ADDRS                      { ENETC0_SI0_BASE, ENETC1_SI0_BASE, ENETC1_SI1_BASE }
  /** Array initializer of ENETC_SI peripheral base pointers */
  #define ENETC_SI_BASE_PTRS                       { ENETC0_SI0, ENETC1_SI0, ENETC1_SI1 }
  /** Array initializer of ENETC_SI peripheral base addresses */
  #define ENETC_SI_BASE_ADDRS_NS                   { ENETC0_SI0_BASE_NS, ENETC1_SI0_BASE_NS, ENETC1_SI1_BASE_NS }
  /** Array initializer of ENETC_SI peripheral base pointers */
  #define ENETC_SI_BASE_PTRS_NS                    { ENETC0_SI0_NS, ENETC1_SI0_NS, ENETC1_SI1_NS }
#else
  /** Peripheral ENETC0_SI0 base address */
  #define ENETC0_SI0_BASE                          (0x60B00000u)
  /** Peripheral ENETC0_SI0 base pointer */
  #define ENETC0_SI0                               ((ENETC_SI_Type *)ENETC0_SI0_BASE)
  /** Peripheral ENETC1_SI0 base address */
  #define ENETC1_SI0_BASE                          (0x60B40000u)
  /** Peripheral ENETC1_SI0 base pointer */
  #define ENETC1_SI0                               ((ENETC_SI_Type *)ENETC1_SI0_BASE)
  /** Peripheral ENETC1_SI1 base address */
  #define ENETC1_SI1_BASE                          (0x60C10000u)
  /** Peripheral ENETC1_SI1 base pointer */
  #define ENETC1_SI1                               ((ENETC_SI_Type *)ENETC1_SI1_BASE)
  /** Array initializer of ENETC_SI peripheral base addresses */
  #define ENETC_SI_BASE_ADDRS                      { ENETC0_SI0_BASE, ENETC1_SI0_BASE, ENETC1_SI1_BASE }
  /** Array initializer of ENETC_SI peripheral base pointers */
  #define ENETC_SI_BASE_PTRS                       { ENETC0_SI0, ENETC1_SI0, ENETC1_SI1 }
#endif

/*!
 * @}
 */ /* end of group ENETC_SI_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ENETC_VF_PCI_TYPE0 Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_VF_PCI_TYPE0_Peripheral_Access_Layer ENETC_VF_PCI_TYPE0 Peripheral Access Layer
 * @{
 */

/** ENETC_VF_PCI_TYPE0 - Register Layout Typedef */
typedef struct {
  __I  uint32_t PCI_CFH_DID_VID;                   /**< PCI device ID and vendor ID register, offset: 0x0 */
  __IO uint16_t PCI_CFH_CMD;                       /**< PCI command register, offset: 0x4 */
  __I  uint16_t PCI_CFH_STAT;                      /**< PCI status register, offset: 0x6 */
  __I  uint32_t PCI_CFH_REVID_CLASSCODE;           /**< PCI revision ID and classcode register, offset: 0x8 */
  __I  uint8_t PCI_CFH_CL_SIZE;                    /**< PCI cache line size register, offset: 0xC */
  __I  uint8_t PCI_CFH_LAT_TIMER;                  /**< PCI latency timer register, offset: 0xD */
  __I  uint8_t PCI_CFH_HDR_TYPE;                   /**< PCI header type register, offset: 0xE */
  __I  uint8_t PCI_CFH_BIST;                       /**< PCI BIST register, offset: 0xF */
  __I  uint32_t PCI_CFH_BAR0;                      /**< PCI base address register 0, offset: 0x10 */
  __I  uint32_t PCI_CFH_BAR1;                      /**< PCI base address register 1, offset: 0x14 */
  __I  uint32_t PCI_CFH_BAR2;                      /**< PCI base address register 2, offset: 0x18 */
  __I  uint32_t PCI_CFH_BAR3;                      /**< PCI base address register 3, offset: 0x1C */
  __I  uint32_t PCI_CFH_BAR4;                      /**< PCI base address register 4, offset: 0x20 */
  __I  uint32_t PCI_CFH_BAR5;                      /**< PCI base address register 5, offset: 0x24 */
  __I  uint32_t PCI_CFH_CARDBUS_CIS;               /**< PCI cardbus CIS register, offset: 0x28 */
  __I  uint16_t PCI_CFH_SUBSYS_VID;                /**< PCI subsystem vendor ID register, offset: 0x2C */
  __I  uint16_t PCI_CFH_SUBSYS_ID;                 /**< PCI subsystem ID register, offset: 0x2E */
  __I  uint32_t PCI_CFH_EXP_ROM_BA;                /**< PCI expansion ROM base address register, offset: 0x30 */
  __I  uint8_t PCI_CFH_CAP_PTR;                    /**< PCI capabilities pointer register, offset: 0x34 */
       uint8_t RESERVED_0[11];
  __I  uint16_t PCI_CFC_PCIE_CAP_LIST;             /**< PCI PCIe capabilities list register, offset: 0x40 */
  __I  uint16_t PCI_CFC_PCIE_CAP;                  /**< PCI PCIe capabilities register, offset: 0x42 */
  __I  uint32_t PCI_CFC_PCIE_DEV_CAP;              /**< PCI PCIe device capabilities register, offset: 0x44 */
  __IO uint16_t PCI_CFC_PCIE_DEV_CTL;              /**< PCI PCIe device control register, offset: 0x48 */
  __I  uint16_t PCI_CFC_PCIE_DEV_STAT;             /**< PCI PCIe device status register, offset: 0x4A */
       uint8_t RESERVED_1[24];
  __I  uint32_t PCI_CFC_PCIE_DEV_CAP2;             /**< PCI PCIe device capabilities 2 register, offset: 0x64 */
  __I  uint16_t PCI_CFC_PCIE_DEV_CTL2;             /**< PCI PCIe device control 2 register, offset: 0x68 */
       uint8_t RESERVED_2[22];
  __I  uint16_t PCI_CFC_MSIX_CAP_LIST;             /**< PCI MSI-X capabilities list register, offset: 0x80 */
  __IO uint16_t PCI_CFC_MSIX_MSG_CTL;              /**< PCI MSI-X message control register, offset: 0x82 */
  __I  uint32_t PCI_CFC_MSIX_TABLE_OFF_BIR;        /**< PCI MSI-X table offset/BIR register, offset: 0x84 */
  __I  uint32_t PCI_CFC_MSIX_PBA_OFF_BIR;          /**< PCI MSI-X PBA offset/BIR register, offset: 0x88 */
       uint8_t RESERVED_3[116];
  __I  uint32_t PCIE_CFC_AER_EXT_CAP_HDR;          /**< PCIe AER extended capability header, offset: 0x100 */
  __IO uint32_t PCIE_CFC_AER_UCORR_ERR_STAT;       /**< PCIe AER uncorrectable error status register, offset: 0x104 */
  __IO uint32_t PCIE_CFC_AER_UCORR_ERR_MASK;       /**< PCIe AER uncorrectable error mask register, offset: 0x108 */
  __IO uint32_t PCIE_CFC_AER_UCORR_ERR_SEV;        /**< PCIe AER uncorrectable error severity register, offset: 0x10C */
  __IO uint32_t PCIE_CFC_AER_CORR_ERR_STAT;        /**< PCIe AER correctable error status register, offset: 0x110 */
  __IO uint32_t PCIE_CFC_AER_CORR_ERR_MASK;        /**< PCIe AER correctable error mask register, offset: 0x114 */
  __I  uint32_t PCIE_CFC_AER_CAP_CTL;              /**< PCIe AER capabilities and control register, offset: 0x118 */
       uint8_t RESERVED_4[20];
  __I  uint32_t PCIE_CFC_ACS_CAP_HDR;              /**< PCIe ACS capability header, offset: 0x130 */
  __I  uint16_t PCIE_CFC_ACS_CAP;                  /**< PCIe ACS capability register, offset: 0x134 */
  __I  uint16_t PCIE_CFC_ACS_CTL;                  /**< PCIe ACS control register, offset: 0x136 */
       uint8_t RESERVED_5[8];
  __I  uint32_t PCIE_CFC_RTR_CAP_HDR;              /**< PCIe readiness time reporting capability header, offset: 0x140 */
  __I  uint32_t PCIE_CFC_RTR_RTR1;                 /**< PCIe RTR readiness time reporting 1 register, offset: 0x144 */
  __I  uint32_t PCIE_CFC_RTR_RTR2;                 /**< PCIe RTR readiness time reporting 2 register, offset: 0x148 */
} ENETC_VF_PCI_TYPE0_Type;

/* ----------------------------------------------------------------------------
   -- ENETC_VF_PCI_TYPE0 Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ENETC_VF_PCI_TYPE0_Register_Masks ENETC_VF_PCI_TYPE0 Register Masks
 * @{
 */

/*! @name PCI_CFH_DID_VID - PCI device ID and vendor ID register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_MASK (0xFFFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_VENDOR_ID_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_MASK (0xFFFF0000U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_SHIFT (16U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_DID_VID_DEVICE_ID_MASK)
/*! @} */

/*! @name PCI_CFH_CMD - PCI command register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_MASK (0x4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_SHIFT (2U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_CMD_BUS_MASTER_EN_MASK)
/*! @} */

/*! @name PCI_CFH_STAT - PCI status register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_MASK (0x10U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_STAT_CAP_LIST_MASK)
/*! @} */

/*! @name PCI_CFH_REVID_CLASSCODE - PCI revision ID and classcode register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_MASK (0xFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_REV_ID_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_MASK (0xFFFFFF00U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_SHIFT (8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_MASK)
/*! @} */

/*! @name PCI_CFH_CL_SIZE - PCI cache line size register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_MASK (0xFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE(x) (((uint8_t)(((uint8_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_MASK)
/*! @} */

/*! @name PCI_CFH_LAT_TIMER - PCI latency timer register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_LAT_TIMER_LATENCY_TIMER_MASK (0xFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_LAT_TIMER_LATENCY_TIMER_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_LAT_TIMER_LATENCY_TIMER(x) (((uint8_t)(((uint8_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_LAT_TIMER_LATENCY_TIMER_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_LAT_TIMER_LATENCY_TIMER_MASK)
/*! @} */

/*! @name PCI_CFH_HDR_TYPE - PCI header type register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_MASK (0x7FU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE(x) (((uint8_t)(((uint8_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_HDR_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_MASK (0x80U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_SHIFT (7U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV(x) (((uint8_t)(((uint8_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_MASK)
/*! @} */

/*! @name PCI_CFH_BIST - PCI BIST register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BIST_BIST_MASK (0xFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BIST_BIST_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BIST_BIST(x)  (((uint8_t)(((uint8_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BIST_BIST_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BIST_BIST_MASK)
/*! @} */

/*! @name PCI_CFH_BAR0 - PCI base address register 0 */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_MASK (0x1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_IO_IND_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_MASK (0x6U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_MEM_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_MASK (0x8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_PF_MEM_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_ADDR_MASK (0xFFFFFFF0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_ADDR_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_ADDR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR0_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR1 - PCI base address register 1 */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_MASK (0x1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_IO_IND_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_MASK (0x6U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_MEM_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_MASK (0x8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_PF_MEM_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_ADDR_MASK (0xFFFFFFF0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_ADDR_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_ADDR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR1_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR2 - PCI base address register 2 */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_MASK (0x1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_IO_IND_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_MASK (0x6U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_MEM_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_MASK (0x8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_PF_MEM_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_ADDR_MASK (0xFFFFFFF0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_ADDR_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_ADDR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR2_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR3 - PCI base address register 3 */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_MASK (0x1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_IO_IND_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_MASK (0x6U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_MEM_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_MASK (0x8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_PF_MEM_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_ADDR_MASK (0xFFFFFFF0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_ADDR_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_ADDR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR3_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR4 - PCI base address register 4 */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_MASK (0x1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_IO_IND_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_MASK (0x6U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_MEM_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_MASK (0x8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_PF_MEM_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_ADDR_MASK (0xFFFFFFF0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_ADDR_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_ADDR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR4_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_BAR5 - PCI base address register 5 */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_MASK (0x1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_IO_IND_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_MASK (0x6U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_MEM_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_MASK (0x8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_PF_MEM_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_ADDR_MASK (0xFFFFFFF0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_ADDR_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_ADDR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_BAR5_ADDR_MASK)
/*! @} */

/*! @name PCI_CFH_CARDBUS_CIS - PCI cardbus CIS register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_CARDBUS_CIS_CARDBUS_CIS_PTR_MASK (0xFFFFFFFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CARDBUS_CIS_CARDBUS_CIS_PTR_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CARDBUS_CIS_CARDBUS_CIS_PTR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_CARDBUS_CIS_CARDBUS_CIS_PTR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_CARDBUS_CIS_CARDBUS_CIS_PTR_MASK)
/*! @} */

/*! @name PCI_CFH_SUBSYS_VID - PCI subsystem vendor ID register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SYBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SYBSYSTEM_VENDOR_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SYBSYSTEM_VENDOR_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SYBSYSTEM_VENDOR_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_VID_SYBSYSTEM_VENDOR_ID_MASK)
/*! @} */

/*! @name PCI_CFH_SUBSYS_ID - PCI subsystem ID register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SYBSYSTEM_ID_MASK (0xFFFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SYBSYSTEM_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SYBSYSTEM_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SYBSYSTEM_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_SUBSYS_ID_SYBSYSTEM_ID_MASK)
/*! @} */

/*! @name PCI_CFH_EXP_ROM_BA - PCI expansion ROM base address register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_EXP_ROM_BA_EXP_ROM_BA_MASK (0xFFFFFFFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_EXP_ROM_BA_EXP_ROM_BA_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_EXP_ROM_BA_EXP_ROM_BA(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_EXP_ROM_BA_EXP_ROM_BA_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_EXP_ROM_BA_EXP_ROM_BA_MASK)
/*! @} */

/*! @name PCI_CFH_CAP_PTR - PCI capabilities pointer register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_MASK (0xFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR(x) (((uint8_t)(((uint8_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFH_CAP_PTR_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_CAP_LIST - PCI PCIe capabilities list register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_MASK (0xFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_CAP_ID_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_MASK (0xFF00U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_SHIFT (8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_CAP - PCI PCIe capabilities register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_MASK (0xFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_CAP_VER_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_MASK (0xF0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_MASK (0x3E00U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_SHIFT (9U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_CAP_INT_MSG_NUM_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CAP - PCI PCIe device capabilities register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_MASK (0x10000000U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_SHIFT (28U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CTL - PCI PCIe device control register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_MASK (0x8000U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_SHIFT (15U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL_INIT_FLR_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_STAT - PCI PCIe device status register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_MASK (0x20U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_SHIFT (5U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CAP2 - PCI PCIe device capabilities 2 register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_MASK (0xFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_RNG_SUPP_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_MASK (0x10U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CAP2_CMPL_TO_DIS_SUPP_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CTL2 - PCI PCIe device control 2 register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_MASK (0xFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_VALUE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_MASK (0x10U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_SHIFT (4U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_PCIE_DEV_CTL2_CMPL_TO_EN_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_CAP_LIST - PCI MSI-X capabilities list register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_MASK (0xFFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_CAP_ID_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_MASK (0xFF00U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_SHIFT (8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_CAP_LIST_NEXT_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_MSG_CTL - PCI MSI-X message control register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_MASK (0x7FFU)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_TABLE_SIZE_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_MASK (0x4000U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_SHIFT (14U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_FUNC_MASK_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_MASK (0x8000U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_SHIFT (15U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_MSG_CTL_MSIX_EN_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_TABLE_OFF_BIR - PCI MSI-X table offset/BIR register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_MASK (0x7U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_BIR_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_MASK (0xFFFFFFF8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_TABLE_OFF_BIR_TABLE_OFFSET_MASK)
/*! @} */

/*! @name PCI_CFC_MSIX_PBA_OFF_BIR - PCI MSI-X PBA offset/BIR register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_MASK (0x7U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_BIR_MASK)

#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_MASK (0xFFFFFFF8U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_SHIFT (3U)
#define ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_SHIFT)) & ENETC_VF_PCI_TYPE0_PCI_CFC_MSIX_PBA_OFF_BIR_PBA_OFFSET_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_EXT_CAP_HDR - PCIe AER extended capability header */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_SHIFT (16U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_UCORR_ERR_STAT - PCIe AER uncorrectable error status register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_MASK (0x400000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_SHIFT (22U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_STAT_UCORR_INT_ERR_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_UCORR_ERR_MASK - PCIe AER uncorrectable error mask register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_MASK (0x400000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_SHIFT (22U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_MASK_UCORR_INT_ERR_MASK_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_UCORR_ERR_SEV - PCIe AER uncorrectable error severity register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_MASK (0x400000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_SHIFT (22U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_UCORR_ERR_SEV_UCORR_INT_SEV_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_CORR_ERR_STAT - PCIe AER correctable error status register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_MASK (0x4000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_SHIFT (14U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_STAT_CORR_INT_ERR_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_CORR_ERR_MASK - PCIe AER correctable error mask register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_MASK (0x4000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_SHIFT (14U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CORR_ERR_MASK_CORR_INT_MASK_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_CAP_CTL - PCIe AER capabilities and control register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_MASK (0x1FU)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_AER_CAP_CTL_FIRST_ERR_PTR_MASK)
/*! @} */

/*! @name PCIE_CFC_ACS_CAP_HDR - PCIe ACS capability header */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_SHIFT (16U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_CAP_VER_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_ACS_CAP - PCIe ACS capability register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_MASK (0x2U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_TRANS_BLOCK_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_MASK (0x4U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_SHIFT (2U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_P2P_REQ_REDIR_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_MASK (0x40U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_SHIFT (6U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CAP_ACS_DIR_TRANS_P2P_MASK)
/*! @} */

/*! @name PCIE_CFC_ACS_CTL - PCIe ACS control register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_MASK (0x2U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_SHIFT (1U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_TRANS_BLOCK_EN_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_MASK (0x4U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_SHIFT (2U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_P2P_REQ_REDIR_EN_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_MASK (0x40U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_SHIFT (6U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN(x) (((uint16_t)(((uint16_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_ACS_CTL_ACS_DIR_TRANS_P2P_EN_MASK)
/*! @} */

/*! @name PCIE_CFC_RTR_CAP_HDR - PCIe readiness time reporting capability header */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_SHIFT (16U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_CAP_VER_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_RTR_RTR1 - PCIe RTR readiness time reporting 1 register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_MASK (0xFFFU)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_SHIFT (0U)
/*! RESET_TIME - Reset Time */
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_RESET_TIME_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_MASK (0x80000000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_SHIFT (31U)
/*! VALID - Valid */
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR1_VALID_MASK)
/*! @} */

/*! @name PCIE_CFC_RTR_RTR2 - PCIe RTR readiness time reporting 2 register */
/*! @{ */

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_MASK (0xFFFU)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_SHIFT (0U)
/*! FLR_TIME - FLR Time */
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_FLR_TIME_MASK)

#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_MASK (0xFFF000U)
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_SHIFT (12U)
/*! D3HOT_D0_TIME - D3 hot to D0 time */
#define ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME(x) (((uint32_t)(((uint32_t)(x)) << ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_SHIFT)) & ENETC_VF_PCI_TYPE0_PCIE_CFC_RTR_RTR2_D3HOT_D0_TIME_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group ENETC_VF_PCI_TYPE0_Register_Masks */


/* ENETC_VF_PCI_TYPE0 - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral NETC_VF1_PCI_HDR_TYPE0 base address */
  #define NETC_VF1_PCI_HDR_TYPE0_BASE              (0x70100000u)
  /** Peripheral NETC_VF1_PCI_HDR_TYPE0 base address */
  #define NETC_VF1_PCI_HDR_TYPE0_BASE_NS           (0x60100000u)
  /** Peripheral NETC_VF1_PCI_HDR_TYPE0 base pointer */
  #define NETC_VF1_PCI_HDR_TYPE0                   ((ENETC_VF_PCI_TYPE0_Type *)NETC_VF1_PCI_HDR_TYPE0_BASE)
  /** Peripheral NETC_VF1_PCI_HDR_TYPE0 base pointer */
  #define NETC_VF1_PCI_HDR_TYPE0_NS                ((ENETC_VF_PCI_TYPE0_Type *)NETC_VF1_PCI_HDR_TYPE0_BASE_NS)
  /** Array initializer of ENETC_VF_PCI_TYPE0 peripheral base addresses */
  #define ENETC_VF_PCI_TYPE0_BASE_ADDRS            { NETC_VF1_PCI_HDR_TYPE0_BASE }
  /** Array initializer of ENETC_VF_PCI_TYPE0 peripheral base pointers */
  #define ENETC_VF_PCI_TYPE0_BASE_PTRS             { NETC_VF1_PCI_HDR_TYPE0 }
  /** Array initializer of ENETC_VF_PCI_TYPE0 peripheral base addresses */
  #define ENETC_VF_PCI_TYPE0_BASE_ADDRS_NS         { NETC_VF1_PCI_HDR_TYPE0_BASE_NS }
  /** Array initializer of ENETC_VF_PCI_TYPE0 peripheral base pointers */
  #define ENETC_VF_PCI_TYPE0_BASE_PTRS_NS          { NETC_VF1_PCI_HDR_TYPE0_NS }
#else
  /** Peripheral NETC_VF1_PCI_HDR_TYPE0 base address */
  #define NETC_VF1_PCI_HDR_TYPE0_BASE              (0x60100000u)
  /** Peripheral NETC_VF1_PCI_HDR_TYPE0 base pointer */
  #define NETC_VF1_PCI_HDR_TYPE0                   ((ENETC_VF_PCI_TYPE0_Type *)NETC_VF1_PCI_HDR_TYPE0_BASE)
  /** Array initializer of ENETC_VF_PCI_TYPE0 peripheral base addresses */
  #define ENETC_VF_PCI_TYPE0_BASE_ADDRS            { NETC_VF1_PCI_HDR_TYPE0_BASE }
  /** Array initializer of ENETC_VF_PCI_TYPE0 peripheral base pointers */
  #define ENETC_VF_PCI_TYPE0_BASE_PTRS             { NETC_VF1_PCI_HDR_TYPE0 }
#endif

/*!
 * @}
 */ /* end of group ENETC_VF_PCI_TYPE0_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- EQDC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup EQDC_Peripheral_Access_Layer EQDC Peripheral Access Layer
 * @{
 */

/** EQDC - Register Layout Typedef */
typedef struct {
  __IO uint16_t CTRL;                              /**< Control Register, offset: 0x0 */
  __IO uint16_t CTRL2;                             /**< Control 2 Register, offset: 0x2 */
  __IO uint16_t FILT;                              /**< Input Filter Register, offset: 0x4 */
  __I  uint16_t LASTEDGE;                          /**< Last Edge Time Register, offset: 0x6 */
  __I  uint16_t POSDPER;                           /**< Position Difference Period Counter Register, offset: 0x8 */
  __I  uint16_t POSDPERBFR;                        /**< Position Difference Period Buffer Register, offset: 0xA */
  __IO uint16_t UPOS;                              /**< Upper Position Counter Register, offset: 0xC */
  __IO uint16_t LPOS;                              /**< Lower Position Counter Register, offset: 0xE */
  __IO uint16_t POSD;                              /**< Position Difference Counter Register, offset: 0x10 */
  __I  uint16_t POSDH;                             /**< Position Difference Hold Register, offset: 0x12 */
  __I  uint16_t UPOSH;                             /**< Upper Position Hold Register, offset: 0x14 */
  __I  uint16_t LPOSH;                             /**< Lower Position Hold Register, offset: 0x16 */
  __I  uint16_t LASTEDGEH;                         /**< Last Edge Time Hold Register, offset: 0x18 */
  __I  uint16_t POSDPERH;                          /**< Position Difference Period Hold Register, offset: 0x1A */
  __I  uint16_t REVH;                              /**< Revolution Hold Register, offset: 0x1C */
  __IO uint16_t REV;                               /**< Revolution Counter Register, offset: 0x1E */
  __IO uint16_t UINIT;                             /**< Upper Initialization Register, offset: 0x20 */
  __IO uint16_t LINIT;                             /**< Lower Initialization Register, offset: 0x22 */
  __IO uint16_t UMOD;                              /**< Upper Modulus Register, offset: 0x24 */
  __IO uint16_t LMOD;                              /**< Lower Modulus Register, offset: 0x26 */
  __IO uint16_t UCOMP0;                            /**< Upper Position Compare Register 0, offset: 0x28 */
  __IO uint16_t LCOMP0;                            /**< Lower Position Compare Register 0, offset: 0x2A */
  union {                                          /* offset: 0x2C */
    __O  uint16_t UCOMP1;                            /**< Upper Position Compare 1, offset: 0x2C */
    __I  uint16_t UPOSH1;                            /**< Upper Position Holder Register 1, offset: 0x2C */
  };
  union {                                          /* offset: 0x2E */
    __O  uint16_t LCOMP1;                            /**< Lower Position Compare 1, offset: 0x2E */
    __I  uint16_t LPOSH1;                            /**< Lower Position Holder Register 1, offset: 0x2E */
  };
  union {                                          /* offset: 0x30 */
    __O  uint16_t UCOMP2;                            /**< Upper Position Compare 2, offset: 0x30 */
    __I  uint16_t UPOSH2;                            /**< Upper Position Holder Register 3, offset: 0x30 */
  };
  union {                                          /* offset: 0x32 */
    __O  uint16_t LCOMP2;                            /**< Lower Position Compare 2, offset: 0x32 */
    __I  uint16_t LPOSH2;                            /**< Lower Position Holder Register 2, offset: 0x32 */
  };
  union {                                          /* offset: 0x34 */
    __O  uint16_t UCOMP3;                            /**< Upper Position Compare 3, offset: 0x34 */
    __I  uint16_t UPOSH3;                            /**< Upper Position Holder Register 3, offset: 0x34 */
  };
  union {                                          /* offset: 0x36 */
    __O  uint16_t LCOMP3;                            /**< Lower Position Compare 3, offset: 0x36 */
    __I  uint16_t LPOSH3;                            /**< Lower Position Holder Register 3, offset: 0x36 */
  };
  __IO uint16_t INTCTRL;                           /**< Interrupt Control Register, offset: 0x38 */
  __IO uint16_t WTR;                               /**< Watchdog Timeout Register, offset: 0x3A */
  __IO uint16_t IMR;                               /**< Input Monitor Register, offset: 0x3C */
  __IO uint16_t TST;                               /**< Test Register, offset: 0x3E */
       uint8_t RESERVED_0[16];
  __I  uint16_t UVERID;                            /**< Upper VERID, offset: 0x50 */
  __I  uint16_t LVERID;                            /**< Lower VERID, offset: 0x52 */
} EQDC_Type;

/* ----------------------------------------------------------------------------
   -- EQDC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup EQDC_Register_Masks EQDC Register Masks
 * @{
 */

/*! @name CTRL - Control Register */
/*! @{ */

#define EQDC_CTRL_LDOK_MASK                      (0x1U)
#define EQDC_CTRL_LDOK_SHIFT                     (0U)
/*! LDOK - Load Okay
 *  0b0..No loading action taken. Users can write new values to buffered registers (writing into outer-set of these buffered registers)
 *  0b1..Outer-set values are ready to be loaded into inner-set and take effect. The loading time point depends on CTRL2[LDMOD].
 */
#define EQDC_CTRL_LDOK(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_LDOK_SHIFT)) & EQDC_CTRL_LDOK_MASK)

#define EQDC_CTRL_DMAEN_MASK                     (0x2U)
#define EQDC_CTRL_DMAEN_SHIFT                    (1U)
/*! DMAEN - DMA Enable
 *  0b0..DMA is disabled
 *  0b1..DMA is enabled. DMA request asserts automatically when the values in the outer-set of buffered compare
 *       registers (UCOMP0/LCOMP0;UCOMP1/LCOMP1;UCOMP2/LCOMP2;UCOMP3/LCOMP3), initial registers(UINIT/LINIT) and
 *       modulus registers (UMOD/LMOD) are loaded into the inner-set of buffer and then LDOK is cleared automatically.
 *       After the completion of this DMA transfer, LDOK is set automatically, it ensures outer-set values can be
 *       loaded into inner-set which in turn triggers DMA again.
 */
#define EQDC_CTRL_DMAEN(x)                       (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_DMAEN_SHIFT)) & EQDC_CTRL_DMAEN_MASK)

#define EQDC_CTRL_WDE_MASK                       (0x4U)
#define EQDC_CTRL_WDE_SHIFT                      (2U)
/*! WDE - Watchdog Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_CTRL_WDE(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_WDE_SHIFT)) & EQDC_CTRL_WDE_MASK)

#define EQDC_CTRL_WDIE_MASK                      (0x8U)
#define EQDC_CTRL_WDIE_SHIFT                     (3U)
/*! WDIE - Watchdog Timeout Interrupt Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_CTRL_WDIE(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_WDIE_SHIFT)) & EQDC_CTRL_WDIE_MASK)

#define EQDC_CTRL_WDIRQ_MASK                     (0x10U)
#define EQDC_CTRL_WDIRQ_SHIFT                    (4U)
/*! WDIRQ - Watchdog Timeout Interrupt Request
 *  0b0..No Watchdog timeout interrupt has occurred
 *  0b1..Watchdog timeout interrupt has occurred
 */
#define EQDC_CTRL_WDIRQ(x)                       (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_WDIRQ_SHIFT)) & EQDC_CTRL_WDIRQ_MASK)

#define EQDC_CTRL_XNE_MASK                       (0x20U)
#define EQDC_CTRL_XNE_SHIFT                      (5U)
/*! XNE - Select Positive/Negative Edge of INDEX/PRESET Pulse
 *  0b0..Use positive edge of INDEX/PRESET pulse
 *  0b1..Use negative edge of INDEX/PRESET pulse
 */
#define EQDC_CTRL_XNE(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_XNE_SHIFT)) & EQDC_CTRL_XNE_MASK)

#define EQDC_CTRL_XIP_MASK                       (0x40U)
#define EQDC_CTRL_XIP_SHIFT                      (6U)
/*! XIP - INDEX Triggered Initialization of Position Counters UPOS and LPOS
 *  0b0..INDEX pulse does not initialize the position counter
 *  0b1..INDEX pulse initializes the position counter
 */
#define EQDC_CTRL_XIP(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_XIP_SHIFT)) & EQDC_CTRL_XIP_MASK)

#define EQDC_CTRL_XIE_MASK                       (0x80U)
#define EQDC_CTRL_XIE_SHIFT                      (7U)
/*! XIE - INDEX/PRESET Pulse Interrupt Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_CTRL_XIE(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_XIE_SHIFT)) & EQDC_CTRL_XIE_MASK)

#define EQDC_CTRL_XIRQ_MASK                      (0x100U)
#define EQDC_CTRL_XIRQ_SHIFT                     (8U)
/*! XIRQ - INDEX/PRESET Pulse Interrupt Request
 *  0b0..INDEX/PRESET pulse has not occurred
 *  0b1..INDEX/PRESET pulse has occurred
 */
#define EQDC_CTRL_XIRQ(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_XIRQ_SHIFT)) & EQDC_CTRL_XIRQ_MASK)

#define EQDC_CTRL_PH1_MASK                       (0x200U)
#define EQDC_CTRL_PH1_SHIFT                      (9U)
/*! PH1 - Enable Single Phase Mode
 *  0b0..Standard quadrature decoder, where PHASEA and PHASEB represent a two-phase quadrature signal.
 *  0b1..Single phase mode, bypass the quadrature decoder, refer to CTRL2[CMODE] description
 */
#define EQDC_CTRL_PH1(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_PH1_SHIFT)) & EQDC_CTRL_PH1_MASK)

#define EQDC_CTRL_REV_MASK                       (0x400U)
#define EQDC_CTRL_REV_SHIFT                      (10U)
/*! REV - Enable Reverse Direction Counting
 *  0b0..Count normally and the position counter initialization uses upper/lower initialization register UINIT/LINIT
 *  0b1..Count in the reverse direction and the position counter initialization uses upper/lower modulus register UMOD/LMOD
 */
#define EQDC_CTRL_REV(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_REV_SHIFT)) & EQDC_CTRL_REV_MASK)

#define EQDC_CTRL_SWIP_MASK                      (0x800U)
#define EQDC_CTRL_SWIP_SHIFT                     (11U)
/*! SWIP - Software-Triggered Initialization of Position Counters UPOS and LPOS
 *  0b0..No action
 *  0b1..Initialize position counter
 */
#define EQDC_CTRL_SWIP(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_SWIP_SHIFT)) & EQDC_CTRL_SWIP_MASK)

#define EQDC_CTRL_HNE_MASK                       (0x1000U)
#define EQDC_CTRL_HNE_SHIFT                      (12U)
/*! HNE - Use Negative Edge of HOME/ENABLE Input
 *  0b0..When CTRL[OPMODE] = 0,use HOME positive edge to trigger initialization of position counters. When
 *       CTRL[OPMODE] = 1,use ENABLE high level to enable POS/POSD/WDG/REV counters
 *  0b1..When CTRL[OPMODE] = 0,use HOME negative edge to trigger initialization of position counters. When
 *       CTRL[OPMODE] = 1,use ENABLE low level to enable POS/POSD/WDG/REV counters
 */
#define EQDC_CTRL_HNE(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_HNE_SHIFT)) & EQDC_CTRL_HNE_MASK)

#define EQDC_CTRL_HIP_MASK                       (0x2000U)
#define EQDC_CTRL_HIP_SHIFT                      (13U)
/*! HIP - Enable HOME to Initialize Position Counter UPOS/LPOS
 *  0b0..No action
 *  0b1..HOME signal initializes the position counter
 */
#define EQDC_CTRL_HIP(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_HIP_SHIFT)) & EQDC_CTRL_HIP_MASK)

#define EQDC_CTRL_HIE_MASK                       (0x4000U)
#define EQDC_CTRL_HIE_SHIFT                      (14U)
/*! HIE - HOME/ENABLE Interrupt Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_CTRL_HIE(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_HIE_SHIFT)) & EQDC_CTRL_HIE_MASK)

#define EQDC_CTRL_HIRQ_MASK                      (0x8000U)
#define EQDC_CTRL_HIRQ_SHIFT                     (15U)
/*! HIRQ - HOME/ENABLE Signal Transition Interrupt Request
 *  0b0..No transition on the HOME/ENABLE signal has occurred
 *  0b1..A transition on the HOME/ENABLE signal has occurred
 */
#define EQDC_CTRL_HIRQ(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL_HIRQ_SHIFT)) & EQDC_CTRL_HIRQ_MASK)
/*! @} */

/*! @name CTRL2 - Control 2 Register */
/*! @{ */

#define EQDC_CTRL2_UPDHLD_MASK                   (0x1U)
#define EQDC_CTRL2_UPDHLD_SHIFT                  (0U)
/*! UPDHLD - Update Hold Registers */
#define EQDC_CTRL2_UPDHLD(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_UPDHLD_SHIFT)) & EQDC_CTRL2_UPDHLD_MASK)

#define EQDC_CTRL2_UPDPOS_MASK                   (0x2U)
#define EQDC_CTRL2_UPDPOS_SHIFT                  (1U)
/*! UPDPOS - Update Position Registers */
#define EQDC_CTRL2_UPDPOS(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_UPDPOS_SHIFT)) & EQDC_CTRL2_UPDPOS_MASK)

#define EQDC_CTRL2_OPMODE_MASK                   (0x4U)
#define EQDC_CTRL2_OPMODE_SHIFT                  (2U)
/*! OPMODE - Operation Mode Select
 *  0b0..Decode Mode: Input nodes INDEX/PRESET and HOME/ENABLE are assigned to function of INDEX and HOME.
 *  0b1..Count Mode: Input nodes INDEX/PRESET and HOME/ENABLE are assigned to functions of PRESET and ENABLE. In
 *       this mode: (1)only when ENABLE=1, all counters (position/position difference/revolution/watchdog) can run,
 *       when ENABLE=0, all counters (position/position difference/revolution/watchdog) can't run. (2) the rising
 *       edge of PRESET input can initialize position/revolution/watchdog counters (position counter initialization
 *       also need referring to bit CTRL[REV]).
 */
#define EQDC_CTRL2_OPMODE(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_OPMODE_SHIFT)) & EQDC_CTRL2_OPMODE_MASK)

#define EQDC_CTRL2_LDMOD_MASK                    (0x8U)
#define EQDC_CTRL2_LDMOD_SHIFT                   (3U)
/*! LDMOD - Buffered Register Load (Update) Mode Select
 *  0b0..Buffered registers are loaded and take effect immediately upon CTRL[LDOK] is set.
 *  0b1..Buffered registers are loaded and take effect at the next roll-over or roll-under if CTRL[LDOK] is set.
 */
#define EQDC_CTRL2_LDMOD(x)                      (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_LDMOD_SHIFT)) & EQDC_CTRL2_LDMOD_MASK)

#define EQDC_CTRL2_REVMOD_MASK                   (0x100U)
#define EQDC_CTRL2_REVMOD_SHIFT                  (8U)
/*! REVMOD - Revolution Counter Modulus Enable
 *  0b0..Use INDEX pulse to increment/decrement revolution counter (REV)
 *  0b1..Use modulus counting roll-over/under to increment/decrement revolution counter (REV)
 */
#define EQDC_CTRL2_REVMOD(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_REVMOD_SHIFT)) & EQDC_CTRL2_REVMOD_MASK)

#define EQDC_CTRL2_OUTCTL_MASK                   (0x200U)
#define EQDC_CTRL2_OUTCTL_SHIFT                  (9U)
/*! OUTCTL - Output Control
 *  0b0..POS_MATCH[x](x range is 0-3) is asserted when the Position Counter is equal to according compare value
 *       (UCOMPx/LCOMPx)(x range is 0-3), and de-asserted when the Position Counter not equal to the compare value
 *       (UCOMPx/LCOMPx)(x range is 0-3)
 *  0b1..All POS_MATCH[x](x range is 0-3) are asserted a pulse, when the UPOS, LPOS, REV, or POSD registers are read
 */
#define EQDC_CTRL2_OUTCTL(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_OUTCTL_SHIFT)) & EQDC_CTRL2_OUTCTL_MASK)

#define EQDC_CTRL2_PMEN_MASK                     (0x400U)
#define EQDC_CTRL2_PMEN_SHIFT                    (10U)
/*! PMEN - Period measurement function enable
 *  0b0..Period measurement functions are not used. POSD is loaded to POSDH and then cleared whenever POSD, UPOS, LPOS or REV is read.
 *  0b1..Period measurement functions are used. POSD is loaded into POSDH and then cleared only when POSD is read.
 */
#define EQDC_CTRL2_PMEN(x)                       (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_PMEN_SHIFT)) & EQDC_CTRL2_PMEN_MASK)

#define EQDC_CTRL2_INITPOS_MASK                  (0x1000U)
#define EQDC_CTRL2_INITPOS_SHIFT                 (12U)
/*! INITPOS - Initial Position Register
 *  0b0..Don't initialize position counter on rising edge of TRIGGER
 *  0b1..Initialize position counter on rising edge of TRIGGER
 */
#define EQDC_CTRL2_INITPOS(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_INITPOS_SHIFT)) & EQDC_CTRL2_INITPOS_MASK)

#define EQDC_CTRL2_ONCE_MASK                     (0x2000U)
#define EQDC_CTRL2_ONCE_SHIFT                    (13U)
/*! ONCE - Count Once
 *  0b0..Position counter counts repeatedly
 *  0b1..Position counter counts until roll-over or roll-under, then stop.
 */
#define EQDC_CTRL2_ONCE(x)                       (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_ONCE_SHIFT)) & EQDC_CTRL2_ONCE_MASK)

#define EQDC_CTRL2_CMODE_MASK                    (0xC000U)
#define EQDC_CTRL2_CMODE_SHIFT                   (14U)
/*! CMODE - Counting Mode */
#define EQDC_CTRL2_CMODE(x)                      (((uint16_t)(((uint16_t)(x)) << EQDC_CTRL2_CMODE_SHIFT)) & EQDC_CTRL2_CMODE_MASK)
/*! @} */

/*! @name FILT - Input Filter Register */
/*! @{ */

#define EQDC_FILT_FILT_PER_MASK                  (0xFFU)
#define EQDC_FILT_FILT_PER_SHIFT                 (0U)
/*! FILT_PER - Input Filter Sample Period */
#define EQDC_FILT_FILT_PER(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_FILT_FILT_PER_SHIFT)) & EQDC_FILT_FILT_PER_MASK)

#define EQDC_FILT_FILT_CNT_MASK                  (0x700U)
#define EQDC_FILT_FILT_CNT_SHIFT                 (8U)
/*! FILT_CNT - Input Filter Sample Count */
#define EQDC_FILT_FILT_CNT(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_FILT_FILT_CNT_SHIFT)) & EQDC_FILT_FILT_CNT_MASK)

#define EQDC_FILT_FILT_CS_MASK                   (0x800U)
#define EQDC_FILT_FILT_CS_SHIFT                  (11U)
/*! FILT_CS - Filter Clock Source selection
 *  0b0..Peripheral Clock
 *  0b1..Prescaled peripheral clock by PRSC
 */
#define EQDC_FILT_FILT_CS(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_FILT_FILT_CS_SHIFT)) & EQDC_FILT_FILT_CS_MASK)

#define EQDC_FILT_PRSC_MASK                      (0xF000U)
#define EQDC_FILT_PRSC_SHIFT                     (12U)
/*! PRSC - Prescaler */
#define EQDC_FILT_PRSC(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_FILT_PRSC_SHIFT)) & EQDC_FILT_PRSC_MASK)
/*! @} */

/*! @name LASTEDGE - Last Edge Time Register */
/*! @{ */

#define EQDC_LASTEDGE_LASTEDGE_MASK              (0xFFFFU)
#define EQDC_LASTEDGE_LASTEDGE_SHIFT             (0U)
/*! LASTEDGE - Last Edge Time Counter */
#define EQDC_LASTEDGE_LASTEDGE(x)                (((uint16_t)(((uint16_t)(x)) << EQDC_LASTEDGE_LASTEDGE_SHIFT)) & EQDC_LASTEDGE_LASTEDGE_MASK)
/*! @} */

/*! @name POSDPER - Position Difference Period Counter Register */
/*! @{ */

#define EQDC_POSDPER_POSDPER_MASK                (0xFFFFU)
#define EQDC_POSDPER_POSDPER_SHIFT               (0U)
/*! POSDPER - Position difference period */
#define EQDC_POSDPER_POSDPER(x)                  (((uint16_t)(((uint16_t)(x)) << EQDC_POSDPER_POSDPER_SHIFT)) & EQDC_POSDPER_POSDPER_MASK)
/*! @} */

/*! @name POSDPERBFR - Position Difference Period Buffer Register */
/*! @{ */

#define EQDC_POSDPERBFR_POSDPERBFR_MASK          (0xFFFFU)
#define EQDC_POSDPERBFR_POSDPERBFR_SHIFT         (0U)
/*! POSDPERBFR - Position difference period buffer */
#define EQDC_POSDPERBFR_POSDPERBFR(x)            (((uint16_t)(((uint16_t)(x)) << EQDC_POSDPERBFR_POSDPERBFR_SHIFT)) & EQDC_POSDPERBFR_POSDPERBFR_MASK)
/*! @} */

/*! @name UPOS - Upper Position Counter Register */
/*! @{ */

#define EQDC_UPOS_POS_MASK                       (0xFFFFU)
#define EQDC_UPOS_POS_SHIFT                      (0U)
/*! POS - POS */
#define EQDC_UPOS_POS(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_UPOS_POS_SHIFT)) & EQDC_UPOS_POS_MASK)
/*! @} */

/*! @name LPOS - Lower Position Counter Register */
/*! @{ */

#define EQDC_LPOS_POS_MASK                       (0xFFFFU)
#define EQDC_LPOS_POS_SHIFT                      (0U)
/*! POS - POS */
#define EQDC_LPOS_POS(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_LPOS_POS_SHIFT)) & EQDC_LPOS_POS_MASK)
/*! @} */

/*! @name POSD - Position Difference Counter Register */
/*! @{ */

#define EQDC_POSD_POSD_MASK                      (0xFFFFU)
#define EQDC_POSD_POSD_SHIFT                     (0U)
/*! POSD - POSD */
#define EQDC_POSD_POSD(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_POSD_POSD_SHIFT)) & EQDC_POSD_POSD_MASK)
/*! @} */

/*! @name POSDH - Position Difference Hold Register */
/*! @{ */

#define EQDC_POSDH_POSDH_MASK                    (0xFFFFU)
#define EQDC_POSDH_POSDH_SHIFT                   (0U)
/*! POSDH - POSDH */
#define EQDC_POSDH_POSDH(x)                      (((uint16_t)(((uint16_t)(x)) << EQDC_POSDH_POSDH_SHIFT)) & EQDC_POSDH_POSDH_MASK)
/*! @} */

/*! @name UPOSH - Upper Position Hold Register */
/*! @{ */

#define EQDC_UPOSH_POSH_MASK                     (0xFFFFU)
#define EQDC_UPOSH_POSH_SHIFT                    (0U)
/*! POSH - POSH */
#define EQDC_UPOSH_POSH(x)                       (((uint16_t)(((uint16_t)(x)) << EQDC_UPOSH_POSH_SHIFT)) & EQDC_UPOSH_POSH_MASK)
/*! @} */

/*! @name LPOSH - Lower Position Hold Register */
/*! @{ */

#define EQDC_LPOSH_LPOSH_MASK                    (0xFFFFU)
#define EQDC_LPOSH_LPOSH_SHIFT                   (0U)
/*! LPOSH - POSH */
#define EQDC_LPOSH_LPOSH(x)                      (((uint16_t)(((uint16_t)(x)) << EQDC_LPOSH_LPOSH_SHIFT)) & EQDC_LPOSH_LPOSH_MASK)
/*! @} */

/*! @name LASTEDGEH - Last Edge Time Hold Register */
/*! @{ */

#define EQDC_LASTEDGEH_LASTEDGEH_MASK            (0xFFFFU)
#define EQDC_LASTEDGEH_LASTEDGEH_SHIFT           (0U)
/*! LASTEDGEH - Last Edge Time Hold */
#define EQDC_LASTEDGEH_LASTEDGEH(x)              (((uint16_t)(((uint16_t)(x)) << EQDC_LASTEDGEH_LASTEDGEH_SHIFT)) & EQDC_LASTEDGEH_LASTEDGEH_MASK)
/*! @} */

/*! @name POSDPERH - Position Difference Period Hold Register */
/*! @{ */

#define EQDC_POSDPERH_POSDPERH_MASK              (0xFFFFU)
#define EQDC_POSDPERH_POSDPERH_SHIFT             (0U)
/*! POSDPERH - Position difference period hold */
#define EQDC_POSDPERH_POSDPERH(x)                (((uint16_t)(((uint16_t)(x)) << EQDC_POSDPERH_POSDPERH_SHIFT)) & EQDC_POSDPERH_POSDPERH_MASK)
/*! @} */

/*! @name REVH - Revolution Hold Register */
/*! @{ */

#define EQDC_REVH_REVH_MASK                      (0xFFFFU)
#define EQDC_REVH_REVH_SHIFT                     (0U)
/*! REVH - REVH */
#define EQDC_REVH_REVH(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_REVH_REVH_SHIFT)) & EQDC_REVH_REVH_MASK)
/*! @} */

/*! @name REV - Revolution Counter Register */
/*! @{ */

#define EQDC_REV_REV_MASK                        (0xFFFFU)
#define EQDC_REV_REV_SHIFT                       (0U)
/*! REV - REV */
#define EQDC_REV_REV(x)                          (((uint16_t)(((uint16_t)(x)) << EQDC_REV_REV_SHIFT)) & EQDC_REV_REV_MASK)
/*! @} */

/*! @name UINIT - Upper Initialization Register */
/*! @{ */

#define EQDC_UINIT_INIT_MASK                     (0xFFFFU)
#define EQDC_UINIT_INIT_SHIFT                    (0U)
/*! INIT - INIT */
#define EQDC_UINIT_INIT(x)                       (((uint16_t)(((uint16_t)(x)) << EQDC_UINIT_INIT_SHIFT)) & EQDC_UINIT_INIT_MASK)
/*! @} */

/*! @name LINIT - Lower Initialization Register */
/*! @{ */

#define EQDC_LINIT_INIT_MASK                     (0xFFFFU)
#define EQDC_LINIT_INIT_SHIFT                    (0U)
/*! INIT - INIT */
#define EQDC_LINIT_INIT(x)                       (((uint16_t)(((uint16_t)(x)) << EQDC_LINIT_INIT_SHIFT)) & EQDC_LINIT_INIT_MASK)
/*! @} */

/*! @name UMOD - Upper Modulus Register */
/*! @{ */

#define EQDC_UMOD_MOD_MASK                       (0xFFFFU)
#define EQDC_UMOD_MOD_SHIFT                      (0U)
/*! MOD - MOD */
#define EQDC_UMOD_MOD(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_UMOD_MOD_SHIFT)) & EQDC_UMOD_MOD_MASK)
/*! @} */

/*! @name LMOD - Lower Modulus Register */
/*! @{ */

#define EQDC_LMOD_MOD_MASK                       (0xFFFFU)
#define EQDC_LMOD_MOD_SHIFT                      (0U)
/*! MOD - MOD */
#define EQDC_LMOD_MOD(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_LMOD_MOD_SHIFT)) & EQDC_LMOD_MOD_MASK)
/*! @} */

/*! @name UCOMP0 - Upper Position Compare Register 0 */
/*! @{ */

#define EQDC_UCOMP0_UCOMP0_MASK                  (0xFFFFU)
#define EQDC_UCOMP0_UCOMP0_SHIFT                 (0U)
/*! UCOMP0 - UCOMP0 */
#define EQDC_UCOMP0_UCOMP0(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UCOMP0_UCOMP0_SHIFT)) & EQDC_UCOMP0_UCOMP0_MASK)
/*! @} */

/*! @name LCOMP0 - Lower Position Compare Register 0 */
/*! @{ */

#define EQDC_LCOMP0_LCOMP0_MASK                  (0xFFFFU)
#define EQDC_LCOMP0_LCOMP0_SHIFT                 (0U)
/*! LCOMP0 - LCOMP0 */
#define EQDC_LCOMP0_LCOMP0(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LCOMP0_LCOMP0_SHIFT)) & EQDC_LCOMP0_LCOMP0_MASK)
/*! @} */

/*! @name UCOMP1 - Upper Position Compare 1 */
/*! @{ */

#define EQDC_UCOMP1_UCOMP1_MASK                  (0xFFFFU)
#define EQDC_UCOMP1_UCOMP1_SHIFT                 (0U)
/*! UCOMP1 - UCOMP1 */
#define EQDC_UCOMP1_UCOMP1(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UCOMP1_UCOMP1_SHIFT)) & EQDC_UCOMP1_UCOMP1_MASK)
/*! @} */

/*! @name UPOSH1 - Upper Position Holder Register 1 */
/*! @{ */

#define EQDC_UPOSH1_UPOSH1_MASK                  (0xFFFFU)
#define EQDC_UPOSH1_UPOSH1_SHIFT                 (0U)
/*! UPOSH1 - UPOSH1 */
#define EQDC_UPOSH1_UPOSH1(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UPOSH1_UPOSH1_SHIFT)) & EQDC_UPOSH1_UPOSH1_MASK)
/*! @} */

/*! @name LCOMP1 - Lower Position Compare 1 */
/*! @{ */

#define EQDC_LCOMP1_LCOMP1_MASK                  (0xFFFFU)
#define EQDC_LCOMP1_LCOMP1_SHIFT                 (0U)
/*! LCOMP1 - LCOMP1 */
#define EQDC_LCOMP1_LCOMP1(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LCOMP1_LCOMP1_SHIFT)) & EQDC_LCOMP1_LCOMP1_MASK)
/*! @} */

/*! @name LPOSH1 - Lower Position Holder Register 1 */
/*! @{ */

#define EQDC_LPOSH1_LPOSH1_MASK                  (0xFFFFU)
#define EQDC_LPOSH1_LPOSH1_SHIFT                 (0U)
/*! LPOSH1 - LPOSH1 */
#define EQDC_LPOSH1_LPOSH1(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LPOSH1_LPOSH1_SHIFT)) & EQDC_LPOSH1_LPOSH1_MASK)
/*! @} */

/*! @name UCOMP2 - Upper Position Compare 2 */
/*! @{ */

#define EQDC_UCOMP2_UCOMP2_MASK                  (0xFFFFU)
#define EQDC_UCOMP2_UCOMP2_SHIFT                 (0U)
/*! UCOMP2 - UCOMP2 */
#define EQDC_UCOMP2_UCOMP2(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UCOMP2_UCOMP2_SHIFT)) & EQDC_UCOMP2_UCOMP2_MASK)
/*! @} */

/*! @name UPOSH2 - Upper Position Holder Register 3 */
/*! @{ */

#define EQDC_UPOSH2_UPOSH2_MASK                  (0xFFFFU)
#define EQDC_UPOSH2_UPOSH2_SHIFT                 (0U)
/*! UPOSH2 - UPOSH2 */
#define EQDC_UPOSH2_UPOSH2(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UPOSH2_UPOSH2_SHIFT)) & EQDC_UPOSH2_UPOSH2_MASK)
/*! @} */

/*! @name LCOMP2 - Lower Position Compare 2 */
/*! @{ */

#define EQDC_LCOMP2_LCOMP2_MASK                  (0xFFFFU)
#define EQDC_LCOMP2_LCOMP2_SHIFT                 (0U)
/*! LCOMP2 - LCOMP2 */
#define EQDC_LCOMP2_LCOMP2(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LCOMP2_LCOMP2_SHIFT)) & EQDC_LCOMP2_LCOMP2_MASK)
/*! @} */

/*! @name LPOSH2 - Lower Position Holder Register 2 */
/*! @{ */

#define EQDC_LPOSH2_LPOSH2_MASK                  (0xFFFFU)
#define EQDC_LPOSH2_LPOSH2_SHIFT                 (0U)
/*! LPOSH2 - LPOSH2 */
#define EQDC_LPOSH2_LPOSH2(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LPOSH2_LPOSH2_SHIFT)) & EQDC_LPOSH2_LPOSH2_MASK)
/*! @} */

/*! @name UCOMP3 - Upper Position Compare 3 */
/*! @{ */

#define EQDC_UCOMP3_UCOMP3_MASK                  (0xFFFFU)
#define EQDC_UCOMP3_UCOMP3_SHIFT                 (0U)
/*! UCOMP3 - UCOMP3 */
#define EQDC_UCOMP3_UCOMP3(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UCOMP3_UCOMP3_SHIFT)) & EQDC_UCOMP3_UCOMP3_MASK)
/*! @} */

/*! @name UPOSH3 - Upper Position Holder Register 3 */
/*! @{ */

#define EQDC_UPOSH3_UPOSH3_MASK                  (0xFFFFU)
#define EQDC_UPOSH3_UPOSH3_SHIFT                 (0U)
/*! UPOSH3 - UPOSH3 */
#define EQDC_UPOSH3_UPOSH3(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UPOSH3_UPOSH3_SHIFT)) & EQDC_UPOSH3_UPOSH3_MASK)
/*! @} */

/*! @name LCOMP3 - Lower Position Compare 3 */
/*! @{ */

#define EQDC_LCOMP3_LCOMP3_MASK                  (0xFFFFU)
#define EQDC_LCOMP3_LCOMP3_SHIFT                 (0U)
/*! LCOMP3 - LCOMP3 */
#define EQDC_LCOMP3_LCOMP3(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LCOMP3_LCOMP3_SHIFT)) & EQDC_LCOMP3_LCOMP3_MASK)
/*! @} */

/*! @name LPOSH3 - Lower Position Holder Register 3 */
/*! @{ */

#define EQDC_LPOSH3_LPOSH3_MASK                  (0xFFFFU)
#define EQDC_LPOSH3_LPOSH3_SHIFT                 (0U)
/*! LPOSH3 - LPOSH3 */
#define EQDC_LPOSH3_LPOSH3(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LPOSH3_LPOSH3_SHIFT)) & EQDC_LPOSH3_LPOSH3_MASK)
/*! @} */

/*! @name INTCTRL - Interrupt Control Register */
/*! @{ */

#define EQDC_INTCTRL_SABIE_MASK                  (0x1U)
#define EQDC_INTCTRL_SABIE_SHIFT                 (0U)
/*! SABIE - Simultaneous PHASEA and PHASEB Change Interrupt Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_INTCTRL_SABIE(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_SABIE_SHIFT)) & EQDC_INTCTRL_SABIE_MASK)

#define EQDC_INTCTRL_SABIRQ_MASK                 (0x2U)
#define EQDC_INTCTRL_SABIRQ_SHIFT                (1U)
/*! SABIRQ - Simultaneous PHASEA and PHASEB Change Interrupt Request
 *  0b0..No simultaneous change of PHASEA and PHASEB has occurred
 *  0b1..A simultaneous change of PHASEA and PHASEB has occurred
 */
#define EQDC_INTCTRL_SABIRQ(x)                   (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_SABIRQ_SHIFT)) & EQDC_INTCTRL_SABIRQ_MASK)

#define EQDC_INTCTRL_DIRIE_MASK                  (0x4U)
#define EQDC_INTCTRL_DIRIE_SHIFT                 (2U)
/*! DIRIE - Count direction change interrupt enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_INTCTRL_DIRIE(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_DIRIE_SHIFT)) & EQDC_INTCTRL_DIRIE_MASK)

#define EQDC_INTCTRL_DIRIRQ_MASK                 (0x8U)
#define EQDC_INTCTRL_DIRIRQ_SHIFT                (3U)
/*! DIRIRQ - Count direction change interrupt
 *  0b0..Count direction unchanged
 *  0b1..Count direction changed
 */
#define EQDC_INTCTRL_DIRIRQ(x)                   (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_DIRIRQ_SHIFT)) & EQDC_INTCTRL_DIRIRQ_MASK)

#define EQDC_INTCTRL_RUIE_MASK                   (0x10U)
#define EQDC_INTCTRL_RUIE_SHIFT                  (4U)
/*! RUIE - Roll-under Interrupt Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_INTCTRL_RUIE(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_RUIE_SHIFT)) & EQDC_INTCTRL_RUIE_MASK)

#define EQDC_INTCTRL_RUIRQ_MASK                  (0x20U)
#define EQDC_INTCTRL_RUIRQ_SHIFT                 (5U)
/*! RUIRQ - Roll-under Interrupt Request
 *  0b0..No roll-under has occurred
 *  0b1..Roll-under has occurred
 */
#define EQDC_INTCTRL_RUIRQ(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_RUIRQ_SHIFT)) & EQDC_INTCTRL_RUIRQ_MASK)

#define EQDC_INTCTRL_ROIE_MASK                   (0x40U)
#define EQDC_INTCTRL_ROIE_SHIFT                  (6U)
/*! ROIE - Roll-over Interrupt Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_INTCTRL_ROIE(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_ROIE_SHIFT)) & EQDC_INTCTRL_ROIE_MASK)

#define EQDC_INTCTRL_ROIRQ_MASK                  (0x80U)
#define EQDC_INTCTRL_ROIRQ_SHIFT                 (7U)
/*! ROIRQ - Roll-over Interrupt Request
 *  0b0..No roll-over has occurred
 *  0b1..Roll-over has occurred
 */
#define EQDC_INTCTRL_ROIRQ(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_ROIRQ_SHIFT)) & EQDC_INTCTRL_ROIRQ_MASK)

#define EQDC_INTCTRL_CMP0IRQ_MASK                (0x200U)
#define EQDC_INTCTRL_CMP0IRQ_SHIFT               (9U)
/*! CMP0IRQ - Compare 0 Interrupt Request
 *  0b0..No match has occurred (the position counter does not match the COMP0 value)
 *  0b1..COMP match has occurred (the position counter matches the COMP0 value)
 */
#define EQDC_INTCTRL_CMP0IRQ(x)                  (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_CMP0IRQ_SHIFT)) & EQDC_INTCTRL_CMP0IRQ_MASK)

#define EQDC_INTCTRL_CMP1IRQ_MASK                (0x800U)
#define EQDC_INTCTRL_CMP1IRQ_SHIFT               (11U)
/*! CMP1IRQ - Compare1 Interrupt Request
 *  0b0..No match has occurred (the position counter does not match the COMP1 value)
 *  0b1..COMP1 match has occurred (the position counter matches the COMP1 value)
 */
#define EQDC_INTCTRL_CMP1IRQ(x)                  (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_CMP1IRQ_SHIFT)) & EQDC_INTCTRL_CMP1IRQ_MASK)

#define EQDC_INTCTRL_CMP2IRQ_MASK                (0x2000U)
#define EQDC_INTCTRL_CMP2IRQ_SHIFT               (13U)
/*! CMP2IRQ - Compare2 Interrupt Request
 *  0b0..No match has occurred (the position counter does not match the COMP2 value)
 *  0b1..COMP2 match has occurred (the position counter matches the COMP2 value)
 */
#define EQDC_INTCTRL_CMP2IRQ(x)                  (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_CMP2IRQ_SHIFT)) & EQDC_INTCTRL_CMP2IRQ_MASK)

#define EQDC_INTCTRL_CMP3IRQ_MASK                (0x8000U)
#define EQDC_INTCTRL_CMP3IRQ_SHIFT               (15U)
/*! CMP3IRQ - Compare3 Interrupt Request
 *  0b0..No match has occurred (the position counter does not match the COMP3 value)
 *  0b1..COMP3 match has occurred (the position counter matches the COMP3 value)
 */
#define EQDC_INTCTRL_CMP3IRQ(x)                  (((uint16_t)(((uint16_t)(x)) << EQDC_INTCTRL_CMP3IRQ_SHIFT)) & EQDC_INTCTRL_CMP3IRQ_MASK)
/*! @} */

/*! @name WTR - Watchdog Timeout Register */
/*! @{ */

#define EQDC_WTR_WDOG_MASK                       (0xFFFFU)
#define EQDC_WTR_WDOG_SHIFT                      (0U)
/*! WDOG - WDOG */
#define EQDC_WTR_WDOG(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_WTR_WDOG_SHIFT)) & EQDC_WTR_WDOG_MASK)
/*! @} */

/*! @name IMR - Input Monitor Register */
/*! @{ */

#define EQDC_IMR_HOME_ENABLE_MASK                (0x1U)
#define EQDC_IMR_HOME_ENABLE_SHIFT               (0U)
/*! HOME_ENABLE - HOME_ENABLE */
#define EQDC_IMR_HOME_ENABLE(x)                  (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_HOME_ENABLE_SHIFT)) & EQDC_IMR_HOME_ENABLE_MASK)

#define EQDC_IMR_INDEX_PRESET_MASK               (0x2U)
#define EQDC_IMR_INDEX_PRESET_SHIFT              (1U)
/*! INDEX_PRESET - INDEX_PRESET */
#define EQDC_IMR_INDEX_PRESET(x)                 (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_INDEX_PRESET_SHIFT)) & EQDC_IMR_INDEX_PRESET_MASK)

#define EQDC_IMR_PHB_MASK                        (0x4U)
#define EQDC_IMR_PHB_SHIFT                       (2U)
/*! PHB - PHB */
#define EQDC_IMR_PHB(x)                          (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_PHB_SHIFT)) & EQDC_IMR_PHB_MASK)

#define EQDC_IMR_PHA_MASK                        (0x8U)
#define EQDC_IMR_PHA_SHIFT                       (3U)
/*! PHA - PHA */
#define EQDC_IMR_PHA(x)                          (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_PHA_SHIFT)) & EQDC_IMR_PHA_MASK)

#define EQDC_IMR_FHOM_ENA_MASK                   (0x10U)
#define EQDC_IMR_FHOM_ENA_SHIFT                  (4U)
/*! FHOM_ENA - filter operation on HOME/ENABLE input */
#define EQDC_IMR_FHOM_ENA(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_FHOM_ENA_SHIFT)) & EQDC_IMR_FHOM_ENA_MASK)

#define EQDC_IMR_FIND_PRE_MASK                   (0x20U)
#define EQDC_IMR_FIND_PRE_SHIFT                  (5U)
/*! FIND_PRE - filter operation on INDEX/PRESET input */
#define EQDC_IMR_FIND_PRE(x)                     (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_FIND_PRE_SHIFT)) & EQDC_IMR_FIND_PRE_MASK)

#define EQDC_IMR_FPHB_MASK                       (0x40U)
#define EQDC_IMR_FPHB_SHIFT                      (6U)
/*! FPHB - filter operation on PHASEB input */
#define EQDC_IMR_FPHB(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_FPHB_SHIFT)) & EQDC_IMR_FPHB_MASK)

#define EQDC_IMR_FPHA_MASK                       (0x80U)
#define EQDC_IMR_FPHA_SHIFT                      (7U)
/*! FPHA - filter operation on PHASEA input */
#define EQDC_IMR_FPHA(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_FPHA_SHIFT)) & EQDC_IMR_FPHA_MASK)

#define EQDC_IMR_CMPF0_MASK                      (0x100U)
#define EQDC_IMR_CMPF0_SHIFT                     (8U)
/*! CMPF0 - Position Compare 0 Flag Output
 *  0b0..When the position counter is less than value of COMP0 register
 *  0b1..When the position counter is greater or equal than value of COMP0 register
 */
#define EQDC_IMR_CMPF0(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_CMPF0_SHIFT)) & EQDC_IMR_CMPF0_MASK)

#define EQDC_IMR_CMP1F_MASK                      (0x200U)
#define EQDC_IMR_CMP1F_SHIFT                     (9U)
/*! CMP1F - Position Compare1 Flag Output
 *  0b0..When the position counter is less than value of COMP1 register
 *  0b1..When the position counter is greater or equal than value of COMP1 register
 */
#define EQDC_IMR_CMP1F(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_CMP1F_SHIFT)) & EQDC_IMR_CMP1F_MASK)

#define EQDC_IMR_CMP2F_MASK                      (0x400U)
#define EQDC_IMR_CMP2F_SHIFT                     (10U)
/*! CMP2F - Position Compare2 Flag Output
 *  0b0..When the position counter is less than value of COMP2 register
 *  0b1..When the position counter is greater or equal than value of COMP2 register
 */
#define EQDC_IMR_CMP2F(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_CMP2F_SHIFT)) & EQDC_IMR_CMP2F_MASK)

#define EQDC_IMR_CMP3F_MASK                      (0x800U)
#define EQDC_IMR_CMP3F_SHIFT                     (11U)
/*! CMP3F - Position Compare3 Flag Output
 *  0b0..When the position counter value is less than value of COMP3 register
 *  0b1..When the position counter is greater or equal than value of COMP3 register
 */
#define EQDC_IMR_CMP3F(x)                        (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_CMP3F_SHIFT)) & EQDC_IMR_CMP3F_MASK)

#define EQDC_IMR_DIRH_MASK                       (0x4000U)
#define EQDC_IMR_DIRH_SHIFT                      (14U)
/*! DIRH - Count Direction Flag Hold */
#define EQDC_IMR_DIRH(x)                         (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_DIRH_SHIFT)) & EQDC_IMR_DIRH_MASK)

#define EQDC_IMR_DIR_MASK                        (0x8000U)
#define EQDC_IMR_DIR_SHIFT                       (15U)
/*! DIR - Count Direction Flag Output
 *  0b0..Current count was in the down direction
 *  0b1..Current count was in the up direction
 */
#define EQDC_IMR_DIR(x)                          (((uint16_t)(((uint16_t)(x)) << EQDC_IMR_DIR_SHIFT)) & EQDC_IMR_DIR_MASK)
/*! @} */

/*! @name TST - Test Register */
/*! @{ */

#define EQDC_TST_TEST_COUNT_MASK                 (0xFFU)
#define EQDC_TST_TEST_COUNT_SHIFT                (0U)
/*! TEST_COUNT - TEST_COUNT */
#define EQDC_TST_TEST_COUNT(x)                   (((uint16_t)(((uint16_t)(x)) << EQDC_TST_TEST_COUNT_SHIFT)) & EQDC_TST_TEST_COUNT_MASK)

#define EQDC_TST_TEST_PERIOD_MASK                (0x1F00U)
#define EQDC_TST_TEST_PERIOD_SHIFT               (8U)
/*! TEST_PERIOD - TEST_PERIOD */
#define EQDC_TST_TEST_PERIOD(x)                  (((uint16_t)(((uint16_t)(x)) << EQDC_TST_TEST_PERIOD_SHIFT)) & EQDC_TST_TEST_PERIOD_MASK)

#define EQDC_TST_QDN_MASK                        (0x2000U)
#define EQDC_TST_QDN_SHIFT                       (13U)
/*! QDN - Quadrature Decoder Negative Signal
 *  0b0..Generates a positive quadrature decoder signal
 *  0b1..Generates a negative quadrature decoder signal
 */
#define EQDC_TST_QDN(x)                          (((uint16_t)(((uint16_t)(x)) << EQDC_TST_QDN_SHIFT)) & EQDC_TST_QDN_MASK)

#define EQDC_TST_TCE_MASK                        (0x4000U)
#define EQDC_TST_TCE_SHIFT                       (14U)
/*! TCE - Test Counter Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_TST_TCE(x)                          (((uint16_t)(((uint16_t)(x)) << EQDC_TST_TCE_SHIFT)) & EQDC_TST_TCE_MASK)

#define EQDC_TST_TEN_MASK                        (0x8000U)
#define EQDC_TST_TEN_SHIFT                       (15U)
/*! TEN - Test Mode Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define EQDC_TST_TEN(x)                          (((uint16_t)(((uint16_t)(x)) << EQDC_TST_TEN_SHIFT)) & EQDC_TST_TEN_MASK)
/*! @} */

/*! @name UVERID - Upper VERID */
/*! @{ */

#define EQDC_UVERID_UVERID_MASK                  (0xFFFFU)
#define EQDC_UVERID_UVERID_SHIFT                 (0U)
/*! UVERID - UVERID */
#define EQDC_UVERID_UVERID(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_UVERID_UVERID_SHIFT)) & EQDC_UVERID_UVERID_MASK)
/*! @} */

/*! @name LVERID - Lower VERID */
/*! @{ */

#define EQDC_LVERID_LVERID_MASK                  (0xFFFFU)
#define EQDC_LVERID_LVERID_SHIFT                 (0U)
/*! LVERID - LVERID */
#define EQDC_LVERID_LVERID(x)                    (((uint16_t)(((uint16_t)(x)) << EQDC_LVERID_LVERID_SHIFT)) & EQDC_LVERID_LVERID_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group EQDC_Register_Masks */


/* EQDC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral EQDC1 base address */
  #define EQDC1_BASE                               (0x52710000u)
  /** Peripheral EQDC1 base address */
  #define EQDC1_BASE_NS                            (0x42710000u)
  /** Peripheral EQDC1 base pointer */
  #define EQDC1                                    ((EQDC_Type *)EQDC1_BASE)
  /** Peripheral EQDC1 base pointer */
  #define EQDC1_NS                                 ((EQDC_Type *)EQDC1_BASE_NS)
  /** Peripheral EQDC2 base address */
  #define EQDC2_BASE                               (0x52720000u)
  /** Peripheral EQDC2 base address */
  #define EQDC2_BASE_NS                            (0x42720000u)
  /** Peripheral EQDC2 base pointer */
  #define EQDC2                                    ((EQDC_Type *)EQDC2_BASE)
  /** Peripheral EQDC2 base pointer */
  #define EQDC2_NS                                 ((EQDC_Type *)EQDC2_BASE_NS)
  /** Array initializer of EQDC peripheral base addresses */
  #define EQDC_BASE_ADDRS                          { 0u, EQDC1_BASE, EQDC2_BASE }
  /** Array initializer of EQDC peripheral base pointers */
  #define EQDC_BASE_PTRS                           { (EQDC_Type *)0u, EQDC1, EQDC2 }
  /** Array initializer of EQDC peripheral base addresses */
  #define EQDC_BASE_ADDRS_NS                       { 0u, EQDC1_BASE_NS, EQDC2_BASE_NS }
  /** Array initializer of EQDC peripheral base pointers */
  #define EQDC_BASE_PTRS_NS                        { (EQDC_Type *)0u, EQDC1_NS, EQDC2_NS }
#else
  /** Peripheral EQDC1 base address */
  #define EQDC1_BASE                               (0x42710000u)
  /** Peripheral EQDC1 base pointer */
  #define EQDC1                                    ((EQDC_Type *)EQDC1_BASE)
  /** Peripheral EQDC2 base address */
  #define EQDC2_BASE                               (0x42720000u)
  /** Peripheral EQDC2 base pointer */
  #define EQDC2                                    ((EQDC_Type *)EQDC2_BASE)
  /** Array initializer of EQDC peripheral base addresses */
  #define EQDC_BASE_ADDRS                          { 0u, EQDC1_BASE, EQDC2_BASE }
  /** Array initializer of EQDC peripheral base pointers */
  #define EQDC_BASE_PTRS                           { (EQDC_Type *)0u, EQDC1, EQDC2 }
#endif
/** Interrupt vectors for the EQDC peripheral type */
#define EQDC_COMPARE_IRQS                        { NotAvail_IRQn, EQDC1_IRQn, EQDC2_IRQn }
#define EQDC_HOME_IRQS                           { NotAvail_IRQn, EQDC1_IRQn, EQDC2_IRQn }
#define EQDC_WDOG_IRQS                           { NotAvail_IRQn, EQDC1_IRQn, EQDC2_IRQn }
#define EQDC_INDEX_IRQS                          { NotAvail_IRQn, EQDC1_IRQn, EQDC2_IRQn }
#define EQDC_INPUT_SWITCH_IRQS                   { NotAvail_IRQn, EQDC1_IRQn, EQDC2_IRQn }

/*!
 * @}
 */ /* end of group EQDC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- ERM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ERM_Peripheral_Access_Layer ERM Peripheral Access Layer
 * @{
 */

/** ERM - Register Layout Typedef */
typedef struct {
  __IO uint32_t CR0;                               /**< ERM Configuration Register 0, offset: 0x0 */
       uint8_t RESERVED_0[12];
  __IO uint32_t SR0;                               /**< ERM Status Register 0, offset: 0x10 */
       uint8_t RESERVED_1[244];
  struct {                                         /* offset: 0x108, array step: 0x10 */
    __IO uint32_t CORR_ERR;                          /**< ERM Memory 0 Correctable Error Count Register..ERM Memory 3 Correctable Error Count Register, array offset: 0x108, array step: 0x10 */
         uint8_t RESERVED_0[12];
  } CORR_ERR_CNT[4];
} ERM_Type;

/* ----------------------------------------------------------------------------
   -- ERM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup ERM_Register_Masks ERM Register Masks
 * @{
 */

/*! @name CR0 - ERM Configuration Register 0 */
/*! @{ */

#define ERM_CR0_ENCIE3_MASK                      (0x40000U)
#define ERM_CR0_ENCIE3_SHIFT                     (18U)
/*! ENCIE3 - ENCIE3
 *  0b0..Interrupt notification of Memory 3 non-correctable error events is disabled.
 *  0b1..Interrupt notification of Memory 3 non-correctable error events is enabled.
 */
#define ERM_CR0_ENCIE3(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ENCIE3_SHIFT)) & ERM_CR0_ENCIE3_MASK)

#define ERM_CR0_ESCIE3_MASK                      (0x80000U)
#define ERM_CR0_ESCIE3_SHIFT                     (19U)
/*! ESCIE3 - ESCIE3
 *  0b0..Interrupt notification of Memory 3 single-bit correction events is disabled.
 *  0b1..Interrupt notification of Memory 3 single-bit correction events is enabled.
 */
#define ERM_CR0_ESCIE3(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ESCIE3_SHIFT)) & ERM_CR0_ESCIE3_MASK)

#define ERM_CR0_ENCIE2_MASK                      (0x400000U)
#define ERM_CR0_ENCIE2_SHIFT                     (22U)
/*! ENCIE2 - ENCIE2
 *  0b0..Interrupt notification of Memory 2 non-correctable error events is disabled.
 *  0b1..Interrupt notification of Memory 2 non-correctable error events is enabled.
 */
#define ERM_CR0_ENCIE2(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ENCIE2_SHIFT)) & ERM_CR0_ENCIE2_MASK)

#define ERM_CR0_ESCIE2_MASK                      (0x800000U)
#define ERM_CR0_ESCIE2_SHIFT                     (23U)
/*! ESCIE2 - ESCIE2
 *  0b0..Interrupt notification of Memory 2 single-bit correction events is disabled.
 *  0b1..Interrupt notification of Memory 2 single-bit correction events is enabled.
 */
#define ERM_CR0_ESCIE2(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ESCIE2_SHIFT)) & ERM_CR0_ESCIE2_MASK)

#define ERM_CR0_ENCIE1_MASK                      (0x4000000U)
#define ERM_CR0_ENCIE1_SHIFT                     (26U)
/*! ENCIE1 - ENCIE1
 *  0b0..Interrupt notification of Memory 1 non-correctable error events is disabled.
 *  0b1..Interrupt notification of Memory 1 non-correctable error events is enabled.
 */
#define ERM_CR0_ENCIE1(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ENCIE1_SHIFT)) & ERM_CR0_ENCIE1_MASK)

#define ERM_CR0_ESCIE1_MASK                      (0x8000000U)
#define ERM_CR0_ESCIE1_SHIFT                     (27U)
/*! ESCIE1 - ESCIE1
 *  0b0..Interrupt notification of Memory 1 single-bit correction events is disabled.
 *  0b1..Interrupt notification of Memory 1 single-bit correction events is enabled.
 */
#define ERM_CR0_ESCIE1(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ESCIE1_SHIFT)) & ERM_CR0_ESCIE1_MASK)

#define ERM_CR0_ENCIE0_MASK                      (0x40000000U)
#define ERM_CR0_ENCIE0_SHIFT                     (30U)
/*! ENCIE0 - ENCIE0
 *  0b0..Interrupt notification of Memory 0 non-correctable error events is disabled.
 *  0b1..Interrupt notification of Memory 0 non-correctable error events is enabled.
 */
#define ERM_CR0_ENCIE0(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ENCIE0_SHIFT)) & ERM_CR0_ENCIE0_MASK)

#define ERM_CR0_ESCIE0_MASK                      (0x80000000U)
#define ERM_CR0_ESCIE0_SHIFT                     (31U)
/*! ESCIE0 - ESCIE0
 *  0b0..Interrupt notification of Memory 0 single-bit correction events is disabled.
 *  0b1..Interrupt notification of Memory 0 single-bit correction events is enabled.
 */
#define ERM_CR0_ESCIE0(x)                        (((uint32_t)(((uint32_t)(x)) << ERM_CR0_ESCIE0_SHIFT)) & ERM_CR0_ESCIE0_MASK)
/*! @} */

/*! @name SR0 - ERM Status Register 0 */
/*! @{ */

#define ERM_SR0_NCE3_MASK                        (0x40000U)
#define ERM_SR0_NCE3_SHIFT                       (18U)
/*! NCE3 - NCE3
 *  0b0..No non-correctable error event on Memory 3 detected.
 *  0b1..Non-correctable error event on Memory 3 detected.
 */
#define ERM_SR0_NCE3(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_NCE3_SHIFT)) & ERM_SR0_NCE3_MASK)

#define ERM_SR0_SBC3_MASK                        (0x80000U)
#define ERM_SR0_SBC3_SHIFT                       (19U)
/*! SBC3 - SBC3
 *  0b0..No single-bit correction event on Memory 3 detected.
 *  0b1..Single-bit correction event on Memory 3 detected.
 */
#define ERM_SR0_SBC3(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_SBC3_SHIFT)) & ERM_SR0_SBC3_MASK)

#define ERM_SR0_NCE2_MASK                        (0x400000U)
#define ERM_SR0_NCE2_SHIFT                       (22U)
/*! NCE2 - NCE2
 *  0b0..No non-correctable error event on Memory 2 detected.
 *  0b1..Non-correctable error event on Memory 2 detected.
 */
#define ERM_SR0_NCE2(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_NCE2_SHIFT)) & ERM_SR0_NCE2_MASK)

#define ERM_SR0_SBC2_MASK                        (0x800000U)
#define ERM_SR0_SBC2_SHIFT                       (23U)
/*! SBC2 - SBC2
 *  0b0..No single-bit correction event on Memory 2 detected.
 *  0b1..Single-bit correction event on Memory 2 detected.
 */
#define ERM_SR0_SBC2(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_SBC2_SHIFT)) & ERM_SR0_SBC2_MASK)

#define ERM_SR0_NCE1_MASK                        (0x4000000U)
#define ERM_SR0_NCE1_SHIFT                       (26U)
/*! NCE1 - NCE1
 *  0b0..No non-correctable error event on Memory 1 detected.
 *  0b1..Non-correctable error event on Memory 1 detected.
 */
#define ERM_SR0_NCE1(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_NCE1_SHIFT)) & ERM_SR0_NCE1_MASK)

#define ERM_SR0_SBC1_MASK                        (0x8000000U)
#define ERM_SR0_SBC1_SHIFT                       (27U)
/*! SBC1 - SBC1
 *  0b0..No single-bit correction event on Memory 1 detected.
 *  0b1..Single-bit correction event on Memory 1 detected.
 */
#define ERM_SR0_SBC1(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_SBC1_SHIFT)) & ERM_SR0_SBC1_MASK)

#define ERM_SR0_NCE0_MASK                        (0x40000000U)
#define ERM_SR0_NCE0_SHIFT                       (30U)
/*! NCE0 - NCE0
 *  0b0..No non-correctable error event on Memory 0 detected.
 *  0b1..Non-correctable error event on Memory 0 detected.
 */
#define ERM_SR0_NCE0(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_NCE0_SHIFT)) & ERM_SR0_NCE0_MASK)

#define ERM_SR0_SBC0_MASK                        (0x80000000U)
#define ERM_SR0_SBC0_SHIFT                       (31U)
/*! SBC0 - SBC0
 *  0b0..No single-bit correction event on Memory 0 detected.
 *  0b1..Single-bit correction event on Memory 0 detected.
 */
#define ERM_SR0_SBC0(x)                          (((uint32_t)(((uint32_t)(x)) << ERM_SR0_SBC0_SHIFT)) & ERM_SR0_SBC0_MASK)
/*! @} */

/*! @name CORR_ERR - ERM Memory 0 Correctable Error Count Register..ERM Memory 3 Correctable Error Count Register */
/*! @{ */

#define ERM_CORR_ERR_COUNT_MASK                  (0xFFU)
#define ERM_CORR_ERR_COUNT_SHIFT                 (0U)
/*! COUNT - Memory n Correctable Error Count */
#define ERM_CORR_ERR_COUNT(x)                    (((uint32_t)(((uint32_t)(x)) << ERM_CORR_ERR_COUNT_SHIFT)) & ERM_CORR_ERR_COUNT_MASK)
/*! @} */

/* The count of ERM_CORR_ERR */
#define ERM_CORR_ERR_CNT_CORR_ERR_COUNT          (4U)


/*!
 * @}
 */ /* end of group ERM_Register_Masks */


/* ERM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ERM base address */
  #define ERM_BASE                                 (0x5B864000u)
  /** Peripheral ERM base address */
  #define ERM_BASE_NS                              (0x4B864000u)
  /** Peripheral ERM base pointer */
  #define ERM                                      ((ERM_Type *)ERM_BASE)
  /** Peripheral ERM base pointer */
  #define ERM_NS                                   ((ERM_Type *)ERM_BASE_NS)
  /** Array initializer of ERM peripheral base addresses */
  #define ERM_BASE_ADDRS                           { ERM_BASE }
  /** Array initializer of ERM peripheral base pointers */
  #define ERM_BASE_PTRS                            { ERM }
  /** Array initializer of ERM peripheral base addresses */
  #define ERM_BASE_ADDRS_NS                        { ERM_BASE_NS }
  /** Array initializer of ERM peripheral base pointers */
  #define ERM_BASE_PTRS_NS                         { ERM_NS }
#else
  /** Peripheral ERM base address */
  #define ERM_BASE                                 (0x4B864000u)
  /** Peripheral ERM base pointer */
  #define ERM                                      ((ERM_Type *)ERM_BASE)
  /** Array initializer of ERM peripheral base addresses */
  #define ERM_BASE_ADDRS                           { ERM_BASE }
  /** Array initializer of ERM peripheral base pointers */
  #define ERM_BASE_PTRS                            { ERM }
#endif

/*!
 * @}
 */ /* end of group ERM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- EWM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup EWM_Peripheral_Access_Layer EWM Peripheral Access Layer
 * @{
 */

/** EWM - Register Layout Typedef */
typedef struct {
  __IO uint8_t CTRL;                               /**< Control, offset: 0x0 */
  __O  uint8_t SERV;                               /**< Service, offset: 0x1 */
  __IO uint8_t CMPL;                               /**< Compare Low, offset: 0x2 */
  __IO uint8_t CMPH;                               /**< Compare High, offset: 0x3 */
  __IO uint8_t CLKCTRL;                            /**< Clock Control, offset: 0x4 */
  __IO uint8_t CLKPRESCALER;                       /**< Clock Prescaler, offset: 0x5 */
} EWM_Type;

/* ----------------------------------------------------------------------------
   -- EWM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup EWM_Register_Masks EWM Register Masks
 * @{
 */

/*! @name CTRL - Control */
/*! @{ */

#define EWM_CTRL_EWMEN_MASK                      (0x1U)
#define EWM_CTRL_EWMEN_SHIFT                     (0U)
/*! EWMEN - EWM Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define EWM_CTRL_EWMEN(x)                        (((uint8_t)(((uint8_t)(x)) << EWM_CTRL_EWMEN_SHIFT)) & EWM_CTRL_EWMEN_MASK)

#define EWM_CTRL_ASSIN_MASK                      (0x2U)
#define EWM_CTRL_ASSIN_SHIFT                     (1U)
/*! ASSIN - Assertion State Select
 *  0b0..Logic 0
 *  0b1..Logic 1
 */
#define EWM_CTRL_ASSIN(x)                        (((uint8_t)(((uint8_t)(x)) << EWM_CTRL_ASSIN_SHIFT)) & EWM_CTRL_ASSIN_MASK)

#define EWM_CTRL_INEN_MASK                       (0x4U)
#define EWM_CTRL_INEN_SHIFT                      (2U)
/*! INEN - Input Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define EWM_CTRL_INEN(x)                         (((uint8_t)(((uint8_t)(x)) << EWM_CTRL_INEN_SHIFT)) & EWM_CTRL_INEN_MASK)

#define EWM_CTRL_INTEN_MASK                      (0x8U)
#define EWM_CTRL_INTEN_SHIFT                     (3U)
/*! INTEN - Interrupt Enable
 *  0b1..Generates interrupt requests
 *  0b0..Deasserts interrupt requests
 */
#define EWM_CTRL_INTEN(x)                        (((uint8_t)(((uint8_t)(x)) << EWM_CTRL_INTEN_SHIFT)) & EWM_CTRL_INTEN_MASK)
/*! @} */

/*! @name SERV - Service */
/*! @{ */

#define EWM_SERV_SERVICE_MASK                    (0xFFU)
#define EWM_SERV_SERVICE_SHIFT                   (0U)
/*! SERVICE - Service */
#define EWM_SERV_SERVICE(x)                      (((uint8_t)(((uint8_t)(x)) << EWM_SERV_SERVICE_SHIFT)) & EWM_SERV_SERVICE_MASK)
/*! @} */

/*! @name CMPL - Compare Low */
/*! @{ */

#define EWM_CMPL_COMPAREL_MASK                   (0xFFU)
#define EWM_CMPL_COMPAREL_SHIFT                  (0U)
/*! COMPAREL - Compare Low */
#define EWM_CMPL_COMPAREL(x)                     (((uint8_t)(((uint8_t)(x)) << EWM_CMPL_COMPAREL_SHIFT)) & EWM_CMPL_COMPAREL_MASK)
/*! @} */

/*! @name CMPH - Compare High */
/*! @{ */

#define EWM_CMPH_COMPAREH_MASK                   (0xFFU)
#define EWM_CMPH_COMPAREH_SHIFT                  (0U)
/*! COMPAREH - Compare High */
#define EWM_CMPH_COMPAREH(x)                     (((uint8_t)(((uint8_t)(x)) << EWM_CMPH_COMPAREH_SHIFT)) & EWM_CMPH_COMPAREH_MASK)
/*! @} */

/*! @name CLKCTRL - Clock Control */
/*! @{ */

#define EWM_CLKCTRL_CLKSEL_MASK                  (0x3U)
#define EWM_CLKCTRL_CLKSEL_SHIFT                 (0U)
/*! CLKSEL - Clock Select */
#define EWM_CLKCTRL_CLKSEL(x)                    (((uint8_t)(((uint8_t)(x)) << EWM_CLKCTRL_CLKSEL_SHIFT)) & EWM_CLKCTRL_CLKSEL_MASK)
/*! @} */

/*! @name CLKPRESCALER - Clock Prescaler */
/*! @{ */

#define EWM_CLKPRESCALER_CLK_DIV_MASK            (0xFFU)
#define EWM_CLKPRESCALER_CLK_DIV_SHIFT           (0U)
/*! CLK_DIV - Clock Divider */
#define EWM_CLKPRESCALER_CLK_DIV(x)              (((uint8_t)(((uint8_t)(x)) << EWM_CLKPRESCALER_CLK_DIV_SHIFT)) & EWM_CLKPRESCALER_CLK_DIV_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group EWM_Register_Masks */


/* EWM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral EWM base address */
  #define EWM_BASE                                 (0x527B0000u)
  /** Peripheral EWM base address */
  #define EWM_BASE_NS                              (0x427B0000u)
  /** Peripheral EWM base pointer */
  #define EWM                                      ((EWM_Type *)EWM_BASE)
  /** Peripheral EWM base pointer */
  #define EWM_NS                                   ((EWM_Type *)EWM_BASE_NS)
  /** Array initializer of EWM peripheral base addresses */
  #define EWM_BASE_ADDRS                           { EWM_BASE }
  /** Array initializer of EWM peripheral base pointers */
  #define EWM_BASE_PTRS                            { EWM }
  /** Array initializer of EWM peripheral base addresses */
  #define EWM_BASE_ADDRS_NS                        { EWM_BASE_NS }
  /** Array initializer of EWM peripheral base pointers */
  #define EWM_BASE_PTRS_NS                         { EWM_NS }
#else
  /** Peripheral EWM base address */
  #define EWM_BASE                                 (0x427B0000u)
  /** Peripheral EWM base pointer */
  #define EWM                                      ((EWM_Type *)EWM_BASE)
  /** Array initializer of EWM peripheral base addresses */
  #define EWM_BASE_ADDRS                           { EWM_BASE }
  /** Array initializer of EWM peripheral base pointers */
  #define EWM_BASE_PTRS                            { EWM }
#endif
/** Interrupt vectors for the EWM peripheral type */
#define EWM_IRQS                                 { EWM_IRQn }

/*!
 * @}
 */ /* end of group EWM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- FLEXIO Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup FLEXIO_Peripheral_Access_Layer FLEXIO Peripheral Access Layer
 * @{
 */

/** FLEXIO - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
  __IO uint32_t CTRL;                              /**< FLEXIO Control, offset: 0x8 */
  __I  uint32_t PIN;                               /**< Pin State, offset: 0xC */
  __IO uint32_t SHIFTSTAT;                         /**< Shifter Status, offset: 0x10 */
  __IO uint32_t SHIFTERR;                          /**< Shifter Error, offset: 0x14 */
  __IO uint32_t TIMSTAT;                           /**< Timer Status, offset: 0x18 */
       uint8_t RESERVED_0[4];
  __IO uint32_t SHIFTSIEN;                         /**< Shifter Status Interrupt Enable, offset: 0x20 */
  __IO uint32_t SHIFTEIEN;                         /**< Shifter Error Interrupt Enable, offset: 0x24 */
  __IO uint32_t TIMIEN;                            /**< Timer Interrupt Enable, offset: 0x28 */
       uint8_t RESERVED_1[4];
  __IO uint32_t SHIFTSDEN;                         /**< Shifter Status DMA Enable, offset: 0x30 */
       uint8_t RESERVED_2[4];
  __IO uint32_t TIMERSDEN;                         /**< Timer Status DMA Enable, offset: 0x38 */
       uint8_t RESERVED_3[4];
  __IO uint32_t SHIFTSTATE;                        /**< Shifter State, offset: 0x40 */
       uint8_t RESERVED_4[4];
  __IO uint32_t TRGSTAT;                           /**< Trigger Status, offset: 0x48 */
  __IO uint32_t TRIGIEN;                           /**< External Trigger Interrupt Enable, offset: 0x4C */
  __IO uint32_t PINSTAT;                           /**< Pin Status, offset: 0x50 */
  __IO uint32_t PINIEN;                            /**< Pin Interrupt Enable, offset: 0x54 */
  __IO uint32_t PINREN;                            /**< Pin Rising Edge Enable, offset: 0x58 */
  __IO uint32_t PINFEN;                            /**< Pin Falling Edge Enable, offset: 0x5C */
  __IO uint32_t PINOUTD;                           /**< Pin Output Data, offset: 0x60 */
  __IO uint32_t PINOUTE;                           /**< Pin Output Enable, offset: 0x64 */
  __O  uint32_t PINOUTDIS;                         /**< Pin Output Disable, offset: 0x68 */
  __O  uint32_t PINOUTCLR;                         /**< Pin Output Clear, offset: 0x6C */
  __O  uint32_t PINOUTSET;                         /**< Pin Output Set, offset: 0x70 */
  __O  uint32_t PINOUTTOG;                         /**< Pin Output Toggle, offset: 0x74 */
       uint8_t RESERVED_5[8];
  __IO uint32_t SHIFTCTL[8];                       /**< Shifter Control N, array offset: 0x80, array step: 0x4 */
       uint8_t RESERVED_6[96];
  __IO uint32_t SHIFTCFG[8];                       /**< Shifter Configuration N, array offset: 0x100, array step: 0x4 */
       uint8_t RESERVED_7[224];
  __IO uint32_t SHIFTBUF[8];                       /**< Shifter Buffer N, array offset: 0x200, array step: 0x4 */
       uint8_t RESERVED_8[96];
  __IO uint32_t SHIFTBUFBIS[8];                    /**< Shifter Buffer N Bit Swapped, array offset: 0x280, array step: 0x4 */
       uint8_t RESERVED_9[96];
  __IO uint32_t SHIFTBUFBYS[8];                    /**< Shifter Buffer N Byte Swapped, array offset: 0x300, array step: 0x4 */
       uint8_t RESERVED_10[96];
  __IO uint32_t SHIFTBUFBBS[8];                    /**< Shifter Buffer N Bit Byte Swapped, array offset: 0x380, array step: 0x4 */
       uint8_t RESERVED_11[96];
  __IO uint32_t TIMCTL[8];                         /**< Timer Control N, array offset: 0x400, array step: 0x4 */
       uint8_t RESERVED_12[96];
  __IO uint32_t TIMCFG[8];                         /**< Timer Configuration N, array offset: 0x480, array step: 0x4 */
       uint8_t RESERVED_13[96];
  __IO uint32_t TIMCMP[8];                         /**< Timer Compare N, array offset: 0x500, array step: 0x4 */
       uint8_t RESERVED_14[352];
  __IO uint32_t SHIFTBUFNBS[8];                    /**< Shifter Buffer N Nibble Byte Swapped, array offset: 0x680, array step: 0x4 */
       uint8_t RESERVED_15[96];
  __IO uint32_t SHIFTBUFHWS[8];                    /**< Shifter Buffer N Halfword Swapped, array offset: 0x700, array step: 0x4 */
       uint8_t RESERVED_16[96];
  __IO uint32_t SHIFTBUFNIS[8];                    /**< Shifter Buffer N Nibble Swapped, array offset: 0x780, array step: 0x4 */
       uint8_t RESERVED_17[96];
  __IO uint32_t SHIFTBUFOES[8];                    /**< Shifter Buffer N Odd Even Swapped, array offset: 0x800, array step: 0x4 */
       uint8_t RESERVED_18[96];
  __IO uint32_t SHIFTBUFEOS[8];                    /**< Shifter Buffer N Even Odd Swapped, array offset: 0x880, array step: 0x4 */
       uint8_t RESERVED_19[96];
  __IO uint32_t SHIFTBUFHBS[8];                    /**< Shifter Buffer N Halfword Byte Swapped, array offset: 0x900, array step: 0x4 */
} FLEXIO_Type;

/* ----------------------------------------------------------------------------
   -- FLEXIO Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup FLEXIO_Register_Masks FLEXIO Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define FLEXIO_VERID_FEATURE_MASK                (0xFFFFU)
#define FLEXIO_VERID_FEATURE_SHIFT               (0U)
/*! FEATURE - Feature Specification Number
 *  0b0000000000000000..Standard features implemented.
 *  0b0000000000000001..Supports state, logic, and parallel modes.
 *  0b0000000000000010..Supports pin control registers.
 *  0b0000000000000011..Supports state, logic, and parallel modes, plus pin control registers.
 */
#define FLEXIO_VERID_FEATURE(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_VERID_FEATURE_SHIFT)) & FLEXIO_VERID_FEATURE_MASK)

#define FLEXIO_VERID_MINOR_MASK                  (0xFF0000U)
#define FLEXIO_VERID_MINOR_SHIFT                 (16U)
/*! MINOR - Minor Version Number */
#define FLEXIO_VERID_MINOR(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_VERID_MINOR_SHIFT)) & FLEXIO_VERID_MINOR_MASK)

#define FLEXIO_VERID_MAJOR_MASK                  (0xFF000000U)
#define FLEXIO_VERID_MAJOR_SHIFT                 (24U)
/*! MAJOR - Major Version Number */
#define FLEXIO_VERID_MAJOR(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_VERID_MAJOR_SHIFT)) & FLEXIO_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define FLEXIO_PARAM_SHIFTER_MASK                (0xFFU)
#define FLEXIO_PARAM_SHIFTER_SHIFT               (0U)
/*! SHIFTER - Shifter Number */
#define FLEXIO_PARAM_SHIFTER(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_PARAM_SHIFTER_SHIFT)) & FLEXIO_PARAM_SHIFTER_MASK)

#define FLEXIO_PARAM_TIMER_MASK                  (0xFF00U)
#define FLEXIO_PARAM_TIMER_SHIFT                 (8U)
/*! TIMER - Timer Number */
#define FLEXIO_PARAM_TIMER(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_PARAM_TIMER_SHIFT)) & FLEXIO_PARAM_TIMER_MASK)

#define FLEXIO_PARAM_PIN_MASK                    (0xFF0000U)
#define FLEXIO_PARAM_PIN_SHIFT                   (16U)
/*! PIN - Pin Number */
#define FLEXIO_PARAM_PIN(x)                      (((uint32_t)(((uint32_t)(x)) << FLEXIO_PARAM_PIN_SHIFT)) & FLEXIO_PARAM_PIN_MASK)

#define FLEXIO_PARAM_TRIGGER_MASK                (0xFF000000U)
#define FLEXIO_PARAM_TRIGGER_SHIFT               (24U)
/*! TRIGGER - Trigger Number */
#define FLEXIO_PARAM_TRIGGER(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_PARAM_TRIGGER_SHIFT)) & FLEXIO_PARAM_TRIGGER_MASK)
/*! @} */

/*! @name CTRL - FLEXIO Control */
/*! @{ */

#define FLEXIO_CTRL_FLEXEN_MASK                  (0x1U)
#define FLEXIO_CTRL_FLEXEN_SHIFT                 (0U)
/*! FLEXEN - FLEXIO Enable
 *  0b0..FLEXIO module is disabled.
 *  0b1..FLEXIO module is enabled.
 */
#define FLEXIO_CTRL_FLEXEN(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_CTRL_FLEXEN_SHIFT)) & FLEXIO_CTRL_FLEXEN_MASK)

#define FLEXIO_CTRL_SWRST_MASK                   (0x2U)
#define FLEXIO_CTRL_SWRST_SHIFT                  (1U)
/*! SWRST - Software Reset
 *  0b0..Software reset is disabled
 *  0b1..Software reset is enabled. All FLEXIO registers except the Control Register are reset.
 */
#define FLEXIO_CTRL_SWRST(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXIO_CTRL_SWRST_SHIFT)) & FLEXIO_CTRL_SWRST_MASK)

#define FLEXIO_CTRL_FASTACC_MASK                 (0x4U)
#define FLEXIO_CTRL_FASTACC_SHIFT                (2U)
/*! FASTACC - Fast Access
 *  0b0..Configures for normal register accesses to FLEXIO
 *  0b1..Configures for fast register accesses to FLEXIO
 */
#define FLEXIO_CTRL_FASTACC(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_CTRL_FASTACC_SHIFT)) & FLEXIO_CTRL_FASTACC_MASK)

#define FLEXIO_CTRL_DBGE_MASK                    (0x40000000U)
#define FLEXIO_CTRL_DBGE_SHIFT                   (30U)
/*! DBGE - Debug Enable
 *  0b0..FLEXIO is disabled in Debug modes.
 *  0b1..FLEXIO is enabled in Debug modes.
 */
#define FLEXIO_CTRL_DBGE(x)                      (((uint32_t)(((uint32_t)(x)) << FLEXIO_CTRL_DBGE_SHIFT)) & FLEXIO_CTRL_DBGE_MASK)

#define FLEXIO_CTRL_DOZEN_MASK                   (0x80000000U)
#define FLEXIO_CTRL_DOZEN_SHIFT                  (31U)
/*! DOZEN - Doze Enable
 *  0b0..FLEXIO enabled in Doze modes.
 *  0b1..FLEXIO disabled in Doze modes.
 */
#define FLEXIO_CTRL_DOZEN(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXIO_CTRL_DOZEN_SHIFT)) & FLEXIO_CTRL_DOZEN_MASK)
/*! @} */

/*! @name PIN - Pin State */
/*! @{ */

#define FLEXIO_PIN_PDI_MASK                      (0xFFFFFFFFU)
#define FLEXIO_PIN_PDI_SHIFT                     (0U)
/*! PDI - Pin Data Input */
#define FLEXIO_PIN_PDI(x)                        (((uint32_t)(((uint32_t)(x)) << FLEXIO_PIN_PDI_SHIFT)) & FLEXIO_PIN_PDI_MASK)
/*! @} */

/*! @name SHIFTSTAT - Shifter Status */
/*! @{ */

#define FLEXIO_SHIFTSTAT_SSF_MASK                (0xFFU)
#define FLEXIO_SHIFTSTAT_SSF_SHIFT               (0U)
/*! SSF - Shifter Status Flag */
#define FLEXIO_SHIFTSTAT_SSF(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTSTAT_SSF_SHIFT)) & FLEXIO_SHIFTSTAT_SSF_MASK)
/*! @} */

/*! @name SHIFTERR - Shifter Error */
/*! @{ */

#define FLEXIO_SHIFTERR_SEF_MASK                 (0xFFU)
#define FLEXIO_SHIFTERR_SEF_SHIFT                (0U)
/*! SEF - Shifter Error Flags */
#define FLEXIO_SHIFTERR_SEF(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTERR_SEF_SHIFT)) & FLEXIO_SHIFTERR_SEF_MASK)
/*! @} */

/*! @name TIMSTAT - Timer Status */
/*! @{ */

#define FLEXIO_TIMSTAT_TSF_MASK                  (0xFFU)
#define FLEXIO_TIMSTAT_TSF_SHIFT                 (0U)
/*! TSF - Timer Status Flags */
#define FLEXIO_TIMSTAT_TSF(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMSTAT_TSF_SHIFT)) & FLEXIO_TIMSTAT_TSF_MASK)
/*! @} */

/*! @name SHIFTSIEN - Shifter Status Interrupt Enable */
/*! @{ */

#define FLEXIO_SHIFTSIEN_SSIE_MASK               (0xFFU)
#define FLEXIO_SHIFTSIEN_SSIE_SHIFT              (0U)
/*! SSIE - Shifter Status Interrupt Enable */
#define FLEXIO_SHIFTSIEN_SSIE(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTSIEN_SSIE_SHIFT)) & FLEXIO_SHIFTSIEN_SSIE_MASK)
/*! @} */

/*! @name SHIFTEIEN - Shifter Error Interrupt Enable */
/*! @{ */

#define FLEXIO_SHIFTEIEN_SEIE_MASK               (0xFFU)
#define FLEXIO_SHIFTEIEN_SEIE_SHIFT              (0U)
/*! SEIE - Shifter Error Interrupt Enable */
#define FLEXIO_SHIFTEIEN_SEIE(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTEIEN_SEIE_SHIFT)) & FLEXIO_SHIFTEIEN_SEIE_MASK)
/*! @} */

/*! @name TIMIEN - Timer Interrupt Enable */
/*! @{ */

#define FLEXIO_TIMIEN_TEIE_MASK                  (0xFFU)
#define FLEXIO_TIMIEN_TEIE_SHIFT                 (0U)
/*! TEIE - Timer Status Interrupt Enable */
#define FLEXIO_TIMIEN_TEIE(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMIEN_TEIE_SHIFT)) & FLEXIO_TIMIEN_TEIE_MASK)
/*! @} */

/*! @name SHIFTSDEN - Shifter Status DMA Enable */
/*! @{ */

#define FLEXIO_SHIFTSDEN_SSDE_MASK               (0xFFU)
#define FLEXIO_SHIFTSDEN_SSDE_SHIFT              (0U)
/*! SSDE - Shifter Status DMA Enable */
#define FLEXIO_SHIFTSDEN_SSDE(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTSDEN_SSDE_SHIFT)) & FLEXIO_SHIFTSDEN_SSDE_MASK)
/*! @} */

/*! @name TIMERSDEN - Timer Status DMA Enable */
/*! @{ */

#define FLEXIO_TIMERSDEN_TSDE_MASK               (0xFFU)
#define FLEXIO_TIMERSDEN_TSDE_SHIFT              (0U)
/*! TSDE - Timer Status DMA Enable */
#define FLEXIO_TIMERSDEN_TSDE(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMERSDEN_TSDE_SHIFT)) & FLEXIO_TIMERSDEN_TSDE_MASK)
/*! @} */

/*! @name SHIFTSTATE - Shifter State */
/*! @{ */

#define FLEXIO_SHIFTSTATE_STATE_MASK             (0x7U)
#define FLEXIO_SHIFTSTATE_STATE_SHIFT            (0U)
/*! STATE - Current State Pointer */
#define FLEXIO_SHIFTSTATE_STATE(x)               (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTSTATE_STATE_SHIFT)) & FLEXIO_SHIFTSTATE_STATE_MASK)
/*! @} */

/*! @name TRGSTAT - Trigger Status */
/*! @{ */

#define FLEXIO_TRGSTAT_ETSF_MASK                 (0xFU)
#define FLEXIO_TRGSTAT_ETSF_SHIFT                (0U)
/*! ETSF - External Trigger Status Flags */
#define FLEXIO_TRGSTAT_ETSF(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_TRGSTAT_ETSF_SHIFT)) & FLEXIO_TRGSTAT_ETSF_MASK)
/*! @} */

/*! @name TRIGIEN - External Trigger Interrupt Enable */
/*! @{ */

#define FLEXIO_TRIGIEN_TRIE_MASK                 (0xFU)
#define FLEXIO_TRIGIEN_TRIE_SHIFT                (0U)
/*! TRIE - External Trigger Interrupt Enable */
#define FLEXIO_TRIGIEN_TRIE(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_TRIGIEN_TRIE_SHIFT)) & FLEXIO_TRIGIEN_TRIE_MASK)
/*! @} */

/*! @name PINSTAT - Pin Status */
/*! @{ */

#define FLEXIO_PINSTAT_PSF_MASK                  (0xFFFFFFFFU)
#define FLEXIO_PINSTAT_PSF_SHIFT                 (0U)
/*! PSF - Pin Status Flags */
#define FLEXIO_PINSTAT_PSF(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINSTAT_PSF_SHIFT)) & FLEXIO_PINSTAT_PSF_MASK)
/*! @} */

/*! @name PINIEN - Pin Interrupt Enable */
/*! @{ */

#define FLEXIO_PINIEN_PSIE_MASK                  (0xFFFFFFFFU)
#define FLEXIO_PINIEN_PSIE_SHIFT                 (0U)
/*! PSIE - Pin Status Interrupt Enable */
#define FLEXIO_PINIEN_PSIE(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINIEN_PSIE_SHIFT)) & FLEXIO_PINIEN_PSIE_MASK)
/*! @} */

/*! @name PINREN - Pin Rising Edge Enable */
/*! @{ */

#define FLEXIO_PINREN_PRE_MASK                   (0xFFFFFFFFU)
#define FLEXIO_PINREN_PRE_SHIFT                  (0U)
/*! PRE - Pin Rising Edge */
#define FLEXIO_PINREN_PRE(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINREN_PRE_SHIFT)) & FLEXIO_PINREN_PRE_MASK)
/*! @} */

/*! @name PINFEN - Pin Falling Edge Enable */
/*! @{ */

#define FLEXIO_PINFEN_PFE_MASK                   (0xFFFFFFFFU)
#define FLEXIO_PINFEN_PFE_SHIFT                  (0U)
/*! PFE - Pin Falling Edge */
#define FLEXIO_PINFEN_PFE(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINFEN_PFE_SHIFT)) & FLEXIO_PINFEN_PFE_MASK)
/*! @} */

/*! @name PINOUTD - Pin Output Data */
/*! @{ */

#define FLEXIO_PINOUTD_OUTD_MASK                 (0xFFFFFFFFU)
#define FLEXIO_PINOUTD_OUTD_SHIFT                (0U)
/*! OUTD - Output Data */
#define FLEXIO_PINOUTD_OUTD(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINOUTD_OUTD_SHIFT)) & FLEXIO_PINOUTD_OUTD_MASK)
/*! @} */

/*! @name PINOUTE - Pin Output Enable */
/*! @{ */

#define FLEXIO_PINOUTE_OUTE_MASK                 (0xFFFFFFFFU)
#define FLEXIO_PINOUTE_OUTE_SHIFT                (0U)
/*! OUTE - Output Enable */
#define FLEXIO_PINOUTE_OUTE(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINOUTE_OUTE_SHIFT)) & FLEXIO_PINOUTE_OUTE_MASK)
/*! @} */

/*! @name PINOUTDIS - Pin Output Disable */
/*! @{ */

#define FLEXIO_PINOUTDIS_OUTDIS_MASK             (0xFFFFFFFFU)
#define FLEXIO_PINOUTDIS_OUTDIS_SHIFT            (0U)
/*! OUTDIS - Output Disable */
#define FLEXIO_PINOUTDIS_OUTDIS(x)               (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINOUTDIS_OUTDIS_SHIFT)) & FLEXIO_PINOUTDIS_OUTDIS_MASK)
/*! @} */

/*! @name PINOUTCLR - Pin Output Clear */
/*! @{ */

#define FLEXIO_PINOUTCLR_OUTCLR_MASK             (0xFFFFFFFFU)
#define FLEXIO_PINOUTCLR_OUTCLR_SHIFT            (0U)
/*! OUTCLR - Output Clear */
#define FLEXIO_PINOUTCLR_OUTCLR(x)               (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINOUTCLR_OUTCLR_SHIFT)) & FLEXIO_PINOUTCLR_OUTCLR_MASK)
/*! @} */

/*! @name PINOUTSET - Pin Output Set */
/*! @{ */

#define FLEXIO_PINOUTSET_OUTSET_MASK             (0xFFFFFFFFU)
#define FLEXIO_PINOUTSET_OUTSET_SHIFT            (0U)
/*! OUTSET - Output Set */
#define FLEXIO_PINOUTSET_OUTSET(x)               (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINOUTSET_OUTSET_SHIFT)) & FLEXIO_PINOUTSET_OUTSET_MASK)
/*! @} */

/*! @name PINOUTTOG - Pin Output Toggle */
/*! @{ */

#define FLEXIO_PINOUTTOG_OUTTOG_MASK             (0xFFFFFFFFU)
#define FLEXIO_PINOUTTOG_OUTTOG_SHIFT            (0U)
/*! OUTTOG - Output Toggle */
#define FLEXIO_PINOUTTOG_OUTTOG(x)               (((uint32_t)(((uint32_t)(x)) << FLEXIO_PINOUTTOG_OUTTOG_SHIFT)) & FLEXIO_PINOUTTOG_OUTTOG_MASK)
/*! @} */

/*! @name SHIFTCTL - Shifter Control N */
/*! @{ */

#define FLEXIO_SHIFTCTL_SMOD_MASK                (0x7U)
#define FLEXIO_SHIFTCTL_SMOD_SHIFT               (0U)
/*! SMOD - Shifter Mode
 *  0b000..Disabled.
 *  0b001..Receive mode. Captures the current shifter content into the SHIFTBUF on expiration of the timer.
 *  0b010..Transmit mode. Load SHIFTBUF contents into the shifter on expiration of the timer.
 *  0b011..Reserved.
 *  0b100..Match Store mode. Shifter data is compared to SHIFTBUF content on expiration of the timer.
 *  0b101..Match Continuous mode. Shifter data is continuously compared to SHIFTBUF contents.
 *  0b110..State mode. SHIFTBUF contents are used for storing programmable state attributes.
 *  0b111..Logic mode. SHIFTBUF contents are used for implementing programmable logic lookup table.
 */
#define FLEXIO_SHIFTCTL_SMOD(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCTL_SMOD_SHIFT)) & FLEXIO_SHIFTCTL_SMOD_MASK)

#define FLEXIO_SHIFTCTL_PINPOL_MASK              (0x80U)
#define FLEXIO_SHIFTCTL_PINPOL_SHIFT             (7U)
/*! PINPOL - Shifter Pin Polarity
 *  0b0..Pin is active high
 *  0b1..Pin is active low
 */
#define FLEXIO_SHIFTCTL_PINPOL(x)                (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCTL_PINPOL_SHIFT)) & FLEXIO_SHIFTCTL_PINPOL_MASK)

#define FLEXIO_SHIFTCTL_PINSEL_MASK              (0x1F00U)
#define FLEXIO_SHIFTCTL_PINSEL_SHIFT             (8U)
/*! PINSEL - Shifter Pin Select */
#define FLEXIO_SHIFTCTL_PINSEL(x)                (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCTL_PINSEL_SHIFT)) & FLEXIO_SHIFTCTL_PINSEL_MASK)

#define FLEXIO_SHIFTCTL_PINCFG_MASK              (0x30000U)
#define FLEXIO_SHIFTCTL_PINCFG_SHIFT             (16U)
/*! PINCFG - Shifter Pin Configuration
 *  0b00..Shifter pin output disabled
 *  0b01..Shifter pin open-drain or bidirectional output enable
 *  0b10..Shifter pin bidirectional output data
 *  0b11..Shifter pin output
 */
#define FLEXIO_SHIFTCTL_PINCFG(x)                (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCTL_PINCFG_SHIFT)) & FLEXIO_SHIFTCTL_PINCFG_MASK)

#define FLEXIO_SHIFTCTL_TIMPOL_MASK              (0x800000U)
#define FLEXIO_SHIFTCTL_TIMPOL_SHIFT             (23U)
/*! TIMPOL - Timer Polarity
 *  0b0..Shift on posedge of shift clock
 *  0b1..Shift on negedge of shift clock
 */
#define FLEXIO_SHIFTCTL_TIMPOL(x)                (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCTL_TIMPOL_SHIFT)) & FLEXIO_SHIFTCTL_TIMPOL_MASK)

#define FLEXIO_SHIFTCTL_TIMSEL_MASK              (0x7000000U)
#define FLEXIO_SHIFTCTL_TIMSEL_SHIFT             (24U)
/*! TIMSEL - Timer Select */
#define FLEXIO_SHIFTCTL_TIMSEL(x)                (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCTL_TIMSEL_SHIFT)) & FLEXIO_SHIFTCTL_TIMSEL_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTCTL */
#define FLEXIO_SHIFTCTL_COUNT                    (8U)

/*! @name SHIFTCFG - Shifter Configuration N */
/*! @{ */

#define FLEXIO_SHIFTCFG_SSTART_MASK              (0x3U)
#define FLEXIO_SHIFTCFG_SSTART_SHIFT             (0U)
/*! SSTART - Shifter Start Bit
 *  0b00..Start bit disabled for transmitter/receiver/match store. Transmitter loads data on enable.
 *  0b01..Start bit disabled for transmitter/receiver/match store. Transmitter loads data on first shift.
 *  0b10..Transmitter outputs start bit value 0 before loading data on first shift. If start bit is not 0, receiver/match store sets error flag.
 *  0b11..Transmitter outputs start bit value 1 before loading data on first shift. If start bit is not 1, receiver/match store sets error flag.
 */
#define FLEXIO_SHIFTCFG_SSTART(x)                (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCFG_SSTART_SHIFT)) & FLEXIO_SHIFTCFG_SSTART_MASK)

#define FLEXIO_SHIFTCFG_SSTOP_MASK               (0x30U)
#define FLEXIO_SHIFTCFG_SSTOP_SHIFT              (4U)
/*! SSTOP - Shifter Stop bit
 *  0b00..Stop bit disabled for transmitter/receiver/match store
 *  0b01..Stop bit disabled for transmitter/receiver/match store. When timer is in stop condition, receiver/match
 *        store stores receive data on the configured shift edge.
 *  0b10..Transmitter outputs stop bit value 0 on store. If stop bit is not 0, receiver/match store sets error
 *        flag. When timer is in stop condition, receiver/match stores also store receive data on the configured shift
 *        edge.
 *  0b11..Transmitter outputs stop bit value 1 on store. If stop bit is not 1, receiver/match store sets error
 *        flag. When timer is in stop condition, receiver/match store also stores receive data on the configured shift
 *        edge.
 */
#define FLEXIO_SHIFTCFG_SSTOP(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCFG_SSTOP_SHIFT)) & FLEXIO_SHIFTCFG_SSTOP_MASK)

#define FLEXIO_SHIFTCFG_INSRC_MASK               (0x100U)
#define FLEXIO_SHIFTCFG_INSRC_SHIFT              (8U)
/*! INSRC - Input Source
 *  0b0..Pin
 *  0b1..Shifter N+1 Output
 */
#define FLEXIO_SHIFTCFG_INSRC(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCFG_INSRC_SHIFT)) & FLEXIO_SHIFTCFG_INSRC_MASK)

#define FLEXIO_SHIFTCFG_LATST_MASK               (0x200U)
#define FLEXIO_SHIFTCFG_LATST_SHIFT              (9U)
/*! LATST - Late Store
 *  0b0..Shift register stores the pre-shift register state.
 *  0b1..Shift register stores the post-shift register state.
 */
#define FLEXIO_SHIFTCFG_LATST(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCFG_LATST_SHIFT)) & FLEXIO_SHIFTCFG_LATST_MASK)

#define FLEXIO_SHIFTCFG_SSIZE_MASK               (0x1000U)
#define FLEXIO_SHIFTCFG_SSIZE_SHIFT              (12U)
/*! SSIZE - Shifter Size
 *  0b0..Shift register is 32-bit.
 *  0b1..Shift register is 24-bit.
 */
#define FLEXIO_SHIFTCFG_SSIZE(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCFG_SSIZE_SHIFT)) & FLEXIO_SHIFTCFG_SSIZE_MASK)

#define FLEXIO_SHIFTCFG_PWIDTH_MASK              (0x1F0000U)
#define FLEXIO_SHIFTCFG_PWIDTH_SHIFT             (16U)
/*! PWIDTH - Parallel Width */
#define FLEXIO_SHIFTCFG_PWIDTH(x)                (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTCFG_PWIDTH_SHIFT)) & FLEXIO_SHIFTCFG_PWIDTH_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTCFG */
#define FLEXIO_SHIFTCFG_COUNT                    (8U)

/*! @name SHIFTBUF - Shifter Buffer N */
/*! @{ */

#define FLEXIO_SHIFTBUF_SHIFTBUF_MASK            (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUF_SHIFTBUF_SHIFT           (0U)
/*! SHIFTBUF - Shift Buffer */
#define FLEXIO_SHIFTBUF_SHIFTBUF(x)              (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUF_SHIFTBUF_SHIFT)) & FLEXIO_SHIFTBUF_SHIFTBUF_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUF */
#define FLEXIO_SHIFTBUF_COUNT                    (8U)

/*! @name SHIFTBUFBIS - Shifter Buffer N Bit Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFBIS_SHIFTBUFBIS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFBIS_SHIFTBUFBIS_SHIFT     (0U)
/*! SHIFTBUFBIS - Shift Buffer */
#define FLEXIO_SHIFTBUFBIS_SHIFTBUFBIS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFBIS_SHIFTBUFBIS_SHIFT)) & FLEXIO_SHIFTBUFBIS_SHIFTBUFBIS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFBIS */
#define FLEXIO_SHIFTBUFBIS_COUNT                 (8U)

/*! @name SHIFTBUFBYS - Shifter Buffer N Byte Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFBYS_SHIFTBUFBYS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFBYS_SHIFTBUFBYS_SHIFT     (0U)
/*! SHIFTBUFBYS - Shift Buffer */
#define FLEXIO_SHIFTBUFBYS_SHIFTBUFBYS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFBYS_SHIFTBUFBYS_SHIFT)) & FLEXIO_SHIFTBUFBYS_SHIFTBUFBYS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFBYS */
#define FLEXIO_SHIFTBUFBYS_COUNT                 (8U)

/*! @name SHIFTBUFBBS - Shifter Buffer N Bit Byte Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFBBS_SHIFTBUFBBS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFBBS_SHIFTBUFBBS_SHIFT     (0U)
/*! SHIFTBUFBBS - Shift Buffer */
#define FLEXIO_SHIFTBUFBBS_SHIFTBUFBBS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFBBS_SHIFTBUFBBS_SHIFT)) & FLEXIO_SHIFTBUFBBS_SHIFTBUFBBS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFBBS */
#define FLEXIO_SHIFTBUFBBS_COUNT                 (8U)

/*! @name TIMCTL - Timer Control N */
/*! @{ */

#define FLEXIO_TIMCTL_TIMOD_MASK                 (0x7U)
#define FLEXIO_TIMCTL_TIMOD_SHIFT                (0U)
/*! TIMOD - Timer Mode
 *  0b000..Timer disabled.
 *  0b001..Dual 8-bit counters baud mode.
 *  0b010..Dual 8-bit counters PWM high mode.
 *  0b011..Single 16-bit counter mode.
 *  0b100..Single 16-bit counter disable mode.
 *  0b101..Dual 8-bit counters word mode.
 *  0b110..Dual 8-bit counters PWM low mode.
 *  0b111..Single 16-bit input capture mode.
 */
#define FLEXIO_TIMCTL_TIMOD(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_TIMOD_SHIFT)) & FLEXIO_TIMCTL_TIMOD_MASK)

#define FLEXIO_TIMCTL_ONETIM_MASK                (0x20U)
#define FLEXIO_TIMCTL_ONETIM_SHIFT               (5U)
/*! ONETIM - Timer One Time Operation
 *  0b0..The timer enable event is generated as normal.
 *  0b1..The timer enable event is blocked unless timer status flag is clear.
 */
#define FLEXIO_TIMCTL_ONETIM(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_ONETIM_SHIFT)) & FLEXIO_TIMCTL_ONETIM_MASK)

#define FLEXIO_TIMCTL_PININS_MASK                (0x40U)
#define FLEXIO_TIMCTL_PININS_SHIFT               (6U)
/*! PININS - Timer Pin Input Select
 *  0b0..Timer pin input and output are selected by PINSEL.
 *  0b1..Timer pin input is selected by PINSEL+1. Timer pin output remains selected by PINSEL.
 */
#define FLEXIO_TIMCTL_PININS(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_PININS_SHIFT)) & FLEXIO_TIMCTL_PININS_MASK)

#define FLEXIO_TIMCTL_PINPOL_MASK                (0x80U)
#define FLEXIO_TIMCTL_PINPOL_SHIFT               (7U)
/*! PINPOL - Timer Pin Polarity
 *  0b0..Pin is active high
 *  0b1..Pin is active low
 */
#define FLEXIO_TIMCTL_PINPOL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_PINPOL_SHIFT)) & FLEXIO_TIMCTL_PINPOL_MASK)

#define FLEXIO_TIMCTL_PINSEL_MASK                (0x1F00U)
#define FLEXIO_TIMCTL_PINSEL_SHIFT               (8U)
/*! PINSEL - Timer Pin Select */
#define FLEXIO_TIMCTL_PINSEL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_PINSEL_SHIFT)) & FLEXIO_TIMCTL_PINSEL_MASK)

#define FLEXIO_TIMCTL_PINCFG_MASK                (0x30000U)
#define FLEXIO_TIMCTL_PINCFG_SHIFT               (16U)
/*! PINCFG - Timer Pin Configuration
 *  0b00..Timer pin output disabled
 *  0b01..Timer pin open-drain or bidirectional output enable
 *  0b10..Timer pin bidirectional output data
 *  0b11..Timer pin output
 */
#define FLEXIO_TIMCTL_PINCFG(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_PINCFG_SHIFT)) & FLEXIO_TIMCTL_PINCFG_MASK)

#define FLEXIO_TIMCTL_TRGSRC_MASK                (0x400000U)
#define FLEXIO_TIMCTL_TRGSRC_SHIFT               (22U)
/*! TRGSRC - Trigger Source
 *  0b0..External trigger selected
 *  0b1..Internal trigger selected
 */
#define FLEXIO_TIMCTL_TRGSRC(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_TRGSRC_SHIFT)) & FLEXIO_TIMCTL_TRGSRC_MASK)

#define FLEXIO_TIMCTL_TRGPOL_MASK                (0x800000U)
#define FLEXIO_TIMCTL_TRGPOL_SHIFT               (23U)
/*! TRGPOL - Trigger Polarity
 *  0b0..Trigger active high
 *  0b1..Trigger active low
 */
#define FLEXIO_TIMCTL_TRGPOL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_TRGPOL_SHIFT)) & FLEXIO_TIMCTL_TRGPOL_MASK)

#define FLEXIO_TIMCTL_TRGSEL_MASK                (0x3F000000U)
#define FLEXIO_TIMCTL_TRGSEL_SHIFT               (24U)
/*! TRGSEL - Trigger Select */
#define FLEXIO_TIMCTL_TRGSEL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCTL_TRGSEL_SHIFT)) & FLEXIO_TIMCTL_TRGSEL_MASK)
/*! @} */

/* The count of FLEXIO_TIMCTL */
#define FLEXIO_TIMCTL_COUNT                      (8U)

/*! @name TIMCFG - Timer Configuration N */
/*! @{ */

#define FLEXIO_TIMCFG_TSTART_MASK                (0x2U)
#define FLEXIO_TIMCFG_TSTART_SHIFT               (1U)
/*! TSTART - Timer Start Bit
 *  0b0..Start bit disabled
 *  0b1..Start bit enabled
 */
#define FLEXIO_TIMCFG_TSTART(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCFG_TSTART_SHIFT)) & FLEXIO_TIMCFG_TSTART_MASK)

#define FLEXIO_TIMCFG_TSTOP_MASK                 (0x30U)
#define FLEXIO_TIMCFG_TSTOP_SHIFT                (4U)
/*! TSTOP - Timer Stop Bit
 *  0b00..Stop bit disabled
 *  0b01..Stop bit is enabled on timer compare
 *  0b10..Stop bit is enabled on timer disable
 *  0b11..Stop bit is enabled on timer compare and timer disable
 */
#define FLEXIO_TIMCFG_TSTOP(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCFG_TSTOP_SHIFT)) & FLEXIO_TIMCFG_TSTOP_MASK)

#define FLEXIO_TIMCFG_TIMENA_MASK                (0x700U)
#define FLEXIO_TIMCFG_TIMENA_SHIFT               (8U)
/*! TIMENA - Timer Enable
 *  0b000..Timer always enabled
 *  0b001..Timer enabled on Timer N-1 enable
 *  0b010..Timer enabled on Trigger high
 *  0b011..Timer enabled on Trigger high and Pin high
 *  0b100..Timer enabled on Pin rising edge
 *  0b101..Timer enabled on Pin rising edge and Trigger high
 *  0b110..Timer enabled on Trigger rising edge
 *  0b111..Timer enabled on Trigger rising or falling edge
 */
#define FLEXIO_TIMCFG_TIMENA(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCFG_TIMENA_SHIFT)) & FLEXIO_TIMCFG_TIMENA_MASK)

#define FLEXIO_TIMCFG_TIMDIS_MASK                (0x7000U)
#define FLEXIO_TIMCFG_TIMDIS_SHIFT               (12U)
/*! TIMDIS - Timer Disable
 *  0b000..Timer never disabled
 *  0b001..Timer disabled on Timer N-1 disable
 *  0b010..Timer disabled on timer compare (upper 8-bits match and decrement)
 *  0b011..Timer disabled on timer compare (upper 8-bits match and decrement) and trigger low
 *  0b100..Timer disabled on pin rising or falling edge
 *  0b101..Timer disabled on pin rising or falling edge provided trigger is high
 *  0b110..Timer disabled on trigger falling edge
 *  0b111..Reserved
 */
#define FLEXIO_TIMCFG_TIMDIS(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCFG_TIMDIS_SHIFT)) & FLEXIO_TIMCFG_TIMDIS_MASK)

#define FLEXIO_TIMCFG_TIMRST_MASK                (0x70000U)
#define FLEXIO_TIMCFG_TIMRST_SHIFT               (16U)
/*! TIMRST - Timer Reset
 *  0b000..Timer never reset
 *  0b001..Timer reset on timer output high.
 *  0b010..Timer reset on timer pin equal to timer output
 *  0b011..Timer reset on timer trigger equal to timer output
 *  0b100..Timer reset on timer pin rising edge
 *  0b101..Reserved
 *  0b110..Timer reset on trigger rising edge
 *  0b111..Timer reset on trigger rising or falling edge
 */
#define FLEXIO_TIMCFG_TIMRST(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCFG_TIMRST_SHIFT)) & FLEXIO_TIMCFG_TIMRST_MASK)

#define FLEXIO_TIMCFG_TIMDEC_MASK                (0x700000U)
#define FLEXIO_TIMCFG_TIMDEC_SHIFT               (20U)
/*! TIMDEC - Timer Decrement
 *  0b000..Decrement counter on FLEXIO clock. Shift clock equals timer output.
 *  0b001..Decrement counter on trigger input (both edges). Shift clock equals timer output.
 *  0b010..Decrement counter on pin input (both edges). Shift clock equals pin input.
 *  0b011..Decrement counter on trigger input (both edges). Shift clock equals trigger input.
 *  0b100..Decrement counter on FLEXIO clock divided by 16. Shift clock equals timer output.
 *  0b101..Decrement counter on FLEXIO clock divided by 256. Shift clock equals timer output.
 *  0b110..Decrement counter on pin input (rising edge). Shift clock equals pin input.
 *  0b111..Decrement counter on trigger input (rising edge). Shift clock equals trigger input.
 */
#define FLEXIO_TIMCFG_TIMDEC(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCFG_TIMDEC_SHIFT)) & FLEXIO_TIMCFG_TIMDEC_MASK)

#define FLEXIO_TIMCFG_TIMOUT_MASK                (0x3000000U)
#define FLEXIO_TIMCFG_TIMOUT_SHIFT               (24U)
/*! TIMOUT - Timer Output
 *  0b00..Timer output is logic one when enabled and is not affected by timer reset
 *  0b01..Timer output is logic zero when enabled and is not affected by timer reset
 *  0b10..Timer output is logic one when enabled and on timer reset
 *  0b11..Timer output is logic zero when enabled and on timer reset
 */
#define FLEXIO_TIMCFG_TIMOUT(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCFG_TIMOUT_SHIFT)) & FLEXIO_TIMCFG_TIMOUT_MASK)
/*! @} */

/* The count of FLEXIO_TIMCFG */
#define FLEXIO_TIMCFG_COUNT                      (8U)

/*! @name TIMCMP - Timer Compare N */
/*! @{ */

#define FLEXIO_TIMCMP_CMP_MASK                   (0xFFFFU)
#define FLEXIO_TIMCMP_CMP_SHIFT                  (0U)
/*! CMP - Timer Compare Value */
#define FLEXIO_TIMCMP_CMP(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXIO_TIMCMP_CMP_SHIFT)) & FLEXIO_TIMCMP_CMP_MASK)
/*! @} */

/* The count of FLEXIO_TIMCMP */
#define FLEXIO_TIMCMP_COUNT                      (8U)

/*! @name SHIFTBUFNBS - Shifter Buffer N Nibble Byte Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFNBS_SHIFTBUFNBS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFNBS_SHIFTBUFNBS_SHIFT     (0U)
/*! SHIFTBUFNBS - Shift Buffer */
#define FLEXIO_SHIFTBUFNBS_SHIFTBUFNBS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFNBS_SHIFTBUFNBS_SHIFT)) & FLEXIO_SHIFTBUFNBS_SHIFTBUFNBS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFNBS */
#define FLEXIO_SHIFTBUFNBS_COUNT                 (8U)

/*! @name SHIFTBUFHWS - Shifter Buffer N Halfword Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFHWS_SHIFTBUFHWS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFHWS_SHIFTBUFHWS_SHIFT     (0U)
/*! SHIFTBUFHWS - Shift Buffer */
#define FLEXIO_SHIFTBUFHWS_SHIFTBUFHWS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFHWS_SHIFTBUFHWS_SHIFT)) & FLEXIO_SHIFTBUFHWS_SHIFTBUFHWS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFHWS */
#define FLEXIO_SHIFTBUFHWS_COUNT                 (8U)

/*! @name SHIFTBUFNIS - Shifter Buffer N Nibble Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFNIS_SHIFTBUFNIS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFNIS_SHIFTBUFNIS_SHIFT     (0U)
/*! SHIFTBUFNIS - Shift Buffer */
#define FLEXIO_SHIFTBUFNIS_SHIFTBUFNIS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFNIS_SHIFTBUFNIS_SHIFT)) & FLEXIO_SHIFTBUFNIS_SHIFTBUFNIS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFNIS */
#define FLEXIO_SHIFTBUFNIS_COUNT                 (8U)

/*! @name SHIFTBUFOES - Shifter Buffer N Odd Even Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFOES_SHIFTBUFOES_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFOES_SHIFTBUFOES_SHIFT     (0U)
/*! SHIFTBUFOES - Shift Buffer */
#define FLEXIO_SHIFTBUFOES_SHIFTBUFOES(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFOES_SHIFTBUFOES_SHIFT)) & FLEXIO_SHIFTBUFOES_SHIFTBUFOES_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFOES */
#define FLEXIO_SHIFTBUFOES_COUNT                 (8U)

/*! @name SHIFTBUFEOS - Shifter Buffer N Even Odd Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFEOS_SHIFTBUFEOS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFEOS_SHIFTBUFEOS_SHIFT     (0U)
/*! SHIFTBUFEOS - Shift Buffer */
#define FLEXIO_SHIFTBUFEOS_SHIFTBUFEOS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFEOS_SHIFTBUFEOS_SHIFT)) & FLEXIO_SHIFTBUFEOS_SHIFTBUFEOS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFEOS */
#define FLEXIO_SHIFTBUFEOS_COUNT                 (8U)

/*! @name SHIFTBUFHBS - Shifter Buffer N Halfword Byte Swapped */
/*! @{ */

#define FLEXIO_SHIFTBUFHBS_SHIFTBUFHBS_MASK      (0xFFFFFFFFU)
#define FLEXIO_SHIFTBUFHBS_SHIFTBUFHBS_SHIFT     (0U)
/*! SHIFTBUFHBS - Shift Buffer */
#define FLEXIO_SHIFTBUFHBS_SHIFTBUFHBS(x)        (((uint32_t)(((uint32_t)(x)) << FLEXIO_SHIFTBUFHBS_SHIFTBUFHBS_SHIFT)) & FLEXIO_SHIFTBUFHBS_SHIFTBUFHBS_MASK)
/*! @} */

/* The count of FLEXIO_SHIFTBUFHBS */
#define FLEXIO_SHIFTBUFHBS_COUNT                 (8U)


/*!
 * @}
 */ /* end of group FLEXIO_Register_Masks */


/* FLEXIO - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral FLEXIO1 base address */
  #define FLEXIO1_BASE                             (0x525C0000u)
  /** Peripheral FLEXIO1 base address */
  #define FLEXIO1_BASE_NS                          (0x425C0000u)
  /** Peripheral FLEXIO1 base pointer */
  #define FLEXIO1                                  ((FLEXIO_Type *)FLEXIO1_BASE)
  /** Peripheral FLEXIO1 base pointer */
  #define FLEXIO1_NS                               ((FLEXIO_Type *)FLEXIO1_BASE_NS)
  /** Peripheral FLEXIO2 base address */
  #define FLEXIO2_BASE                             (0x525D0000u)
  /** Peripheral FLEXIO2 base address */
  #define FLEXIO2_BASE_NS                          (0x425D0000u)
  /** Peripheral FLEXIO2 base pointer */
  #define FLEXIO2                                  ((FLEXIO_Type *)FLEXIO2_BASE)
  /** Peripheral FLEXIO2 base pointer */
  #define FLEXIO2_NS                               ((FLEXIO_Type *)FLEXIO2_BASE_NS)
  /** Array initializer of FLEXIO peripheral base addresses */
  #define FLEXIO_BASE_ADDRS                        { 0u, FLEXIO1_BASE, FLEXIO2_BASE }
  /** Array initializer of FLEXIO peripheral base pointers */
  #define FLEXIO_BASE_PTRS                         { (FLEXIO_Type *)0u, FLEXIO1, FLEXIO2 }
  /** Array initializer of FLEXIO peripheral base addresses */
  #define FLEXIO_BASE_ADDRS_NS                     { 0u, FLEXIO1_BASE_NS, FLEXIO2_BASE_NS }
  /** Array initializer of FLEXIO peripheral base pointers */
  #define FLEXIO_BASE_PTRS_NS                      { (FLEXIO_Type *)0u, FLEXIO1_NS, FLEXIO2_NS }
#else
  /** Peripheral FLEXIO1 base address */
  #define FLEXIO1_BASE                             (0x425C0000u)
  /** Peripheral FLEXIO1 base pointer */
  #define FLEXIO1                                  ((FLEXIO_Type *)FLEXIO1_BASE)
  /** Peripheral FLEXIO2 base address */
  #define FLEXIO2_BASE                             (0x425D0000u)
  /** Peripheral FLEXIO2 base pointer */
  #define FLEXIO2                                  ((FLEXIO_Type *)FLEXIO2_BASE)
  /** Array initializer of FLEXIO peripheral base addresses */
  #define FLEXIO_BASE_ADDRS                        { 0u, FLEXIO1_BASE, FLEXIO2_BASE }
  /** Array initializer of FLEXIO peripheral base pointers */
  #define FLEXIO_BASE_PTRS                         { (FLEXIO_Type *)0u, FLEXIO1, FLEXIO2 }
#endif
/** Interrupt vectors for the FLEXIO peripheral type */
#define FLEXIO_IRQS                              { NotAvail_IRQn, FLEXIO1_IRQn, FLEXIO2_IRQn }

/*!
 * @}
 */ /* end of group FLEXIO_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- FLEXSPI Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup FLEXSPI_Peripheral_Access_Layer FLEXSPI Peripheral Access Layer
 * @{
 */

/** FLEXSPI - Register Layout Typedef */
typedef struct {
  __IO uint32_t MCR0;                              /**< Module Control 0, offset: 0x0 */
  __IO uint32_t MCR1;                              /**< Module Control 1, offset: 0x4 */
  __IO uint32_t MCR2;                              /**< Module Control 2, offset: 0x8 */
  __IO uint32_t AHBCR;                             /**< AHB Bus Control, offset: 0xC */
  __IO uint32_t INTEN;                             /**< Interrupt Enable, offset: 0x10 */
  __IO uint32_t INTR;                              /**< Interrupt, offset: 0x14 */
  __IO uint32_t LUTKEY;                            /**< LUT Key, offset: 0x18 */
  __IO uint32_t LUTCR;                             /**< LUT Control, offset: 0x1C */
  __IO uint32_t AHBRXBUFCR0[8];                    /**< AHB Receive Buffer 0 Control 0..AHB Receive Buffer 7 Control 0, array offset: 0x20, array step: 0x4 */
       uint8_t RESERVED_0[32];
  __IO uint32_t FLSHCR0[4];                        /**< Flash Control 0, array offset: 0x60, array step: 0x4 */
  __IO uint32_t FLSHCR1[4];                        /**< Flash Control 1, array offset: 0x70, array step: 0x4 */
  __IO uint32_t FLSHCR2[4];                        /**< Flash Control 2, array offset: 0x80, array step: 0x4 */
       uint8_t RESERVED_1[4];
  __IO uint32_t FLSHCR4;                           /**< Flash Control 4, offset: 0x94 */
       uint8_t RESERVED_2[8];
  __IO uint32_t IPCR0;                             /**< IP Control 0, offset: 0xA0 */
  __IO uint32_t IPCR1;                             /**< IP Control 1, offset: 0xA4 */
       uint8_t RESERVED_3[8];
  __O  uint32_t IPCMD;                             /**< IP Command, offset: 0xB0 */
       uint8_t RESERVED_4[4];
  __IO uint32_t IPRXFCR;                           /**< IP Receive FIFO Control, offset: 0xB8 */
  __IO uint32_t IPTXFCR;                           /**< IP Transmit FIFO Control, offset: 0xBC */
  __IO uint32_t DLLCR[2];                          /**< DLL Control 0, array offset: 0xC0, array step: 0x4 */
       uint8_t RESERVED_5[24];
  __I  uint32_t STS0;                              /**< Status 0, offset: 0xE0 */
  __I  uint32_t STS1;                              /**< Status 1, offset: 0xE4 */
  __I  uint32_t STS2;                              /**< Status 2, offset: 0xE8 */
  __I  uint32_t AHBSPNDSTS;                        /**< AHB Suspend Status, offset: 0xEC */
  __I  uint32_t IPRXFSTS;                          /**< IP Receive FIFO Status, offset: 0xF0 */
  __I  uint32_t IPTXFSTS;                          /**< IP Transmit FIFO Status, offset: 0xF4 */
       uint8_t RESERVED_6[8];
  __I  uint32_t RFDR[32];                          /**< IP Receive FIFO Data 0..IP Receive FIFO Data 31, array offset: 0x100, array step: 0x4 */
  __O  uint32_t TFDR[32];                          /**< IP TX FIFO Data 0..IP TX FIFO Data 31, array offset: 0x180, array step: 0x4 */
  __IO uint32_t LUT[128];                          /**< Lookup Table 0..Lookup Table 127, array offset: 0x200, array step: 0x4 */
       uint8_t RESERVED_7[64];
  __IO uint32_t AHBBUFREGIONSTART0;                /**< Receive Buffer Start Address of Region 0, offset: 0x440 */
  __IO uint32_t AHBBUFREGIONEND0;                  /**< Receive Buffer Region 0 End Address, offset: 0x444 */
  __IO uint32_t AHBBUFREGIONSTART1;                /**< Receive Buffer Start Address of Region 1, offset: 0x448 */
  __IO uint32_t AHBBUFREGIONEND1;                  /**< Receive Buffer Region 1 End Address, offset: 0x44C */
  __IO uint32_t AHBBUFREGIONSTART2;                /**< Receive Buffer Start Address of Region 2, offset: 0x450 */
  __IO uint32_t AHBBUFREGIONEND2;                  /**< Receive Buffer Region 2 End Address, offset: 0x454 */
  __IO uint32_t AHBBUFREGIONSTART3;                /**< Receive Buffer Start Address of Region 3, offset: 0x458 */
  __IO uint32_t AHBBUFREGIONEND3;                  /**< Receive Buffer Region 3 End Address, offset: 0x45C */
} FLEXSPI_Type;

/* ----------------------------------------------------------------------------
   -- FLEXSPI Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup FLEXSPI_Register_Masks FLEXSPI Register Masks
 * @{
 */

/*! @name MCR0 - Module Control 0 */
/*! @{ */

#define FLEXSPI_MCR0_SWRESET_MASK                (0x1U)
#define FLEXSPI_MCR0_SWRESET_SHIFT               (0U)
/*! SWRESET - Software Reset
 *  0b0..No impact
 *  0b1..Software reset
 */
#define FLEXSPI_MCR0_SWRESET(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_SWRESET_SHIFT)) & FLEXSPI_MCR0_SWRESET_MASK)

#define FLEXSPI_MCR0_MDIS_MASK                   (0x2U)
#define FLEXSPI_MCR0_MDIS_SHIFT                  (1U)
/*! MDIS - Module Disable
 *  0b0..No impact
 *  0b1..Module disable
 */
#define FLEXSPI_MCR0_MDIS(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_MDIS_SHIFT)) & FLEXSPI_MCR0_MDIS_MASK)

#define FLEXSPI_MCR0_RXCLKSRC_MASK               (0x30U)
#define FLEXSPI_MCR0_RXCLKSRC_SHIFT              (4U)
/*! RXCLKSRC - Sample Clock Source for Flash Reading
 *  0b00..Dummy Read strobe that FlexSPI generates, looped back internally
 *  0b01..Dummy Read strobe that FlexSPI generates, looped back from DQS pad
 *  0b10..Reserved
 *  0b11..Flash-memory-provided read strobe and input from DQS pad
 */
#define FLEXSPI_MCR0_RXCLKSRC(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_RXCLKSRC_SHIFT)) & FLEXSPI_MCR0_RXCLKSRC_MASK)

#define FLEXSPI_MCR0_ARDFEN_MASK                 (0x40U)
#define FLEXSPI_MCR0_ARDFEN_SHIFT                (6U)
/*! ARDFEN - AHB Read Access to IP Receive FIFO Enable
 *  0b0..AHB read access disabled. IP bus reads IP receive FIFO. AHB Bus read access to IP receive FIFO memory space produces bus error.
 *  0b1..AHB read access enabled. AHB bus reads IP receive FIFO. IP Bus read access to IP receive FIFO memory
 *       space returns data zero and causes no bus error.
 */
#define FLEXSPI_MCR0_ARDFEN(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_ARDFEN_SHIFT)) & FLEXSPI_MCR0_ARDFEN_MASK)

#define FLEXSPI_MCR0_ATDFEN_MASK                 (0x80U)
#define FLEXSPI_MCR0_ATDFEN_SHIFT                (7U)
/*! ATDFEN - AHB Write Access to IP Transmit FIFO Enable
 *  0b0..AHB write access disabled. IP bus writes to IP transmit FIFO. AHB bus write access to IP transmit FIFO memory space produces bus error.
 *  0b1..AHB write access enabled. AHB bus writes to IP transmit FIFO. IP Bus write access to IP transmit FIFO
 *       memory space is ignored and causes no bus error.
 */
#define FLEXSPI_MCR0_ATDFEN(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_ATDFEN_SHIFT)) & FLEXSPI_MCR0_ATDFEN_MASK)

#define FLEXSPI_MCR0_SERCLKDIV_MASK              (0x700U)
#define FLEXSPI_MCR0_SERCLKDIV_SHIFT             (8U)
/*! SERCLKDIV - Serial Root Clock Divider
 *  0b000..Divided by 1
 *  0b001..Divided by 2
 *  0b010..Divided by 3
 *  0b011..Divided by 4
 *  0b100..Divided by 5
 *  0b101..Divided by 6
 *  0b110..Divided by 7
 *  0b111..Divided by 8
 */
#define FLEXSPI_MCR0_SERCLKDIV(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_SERCLKDIV_SHIFT)) & FLEXSPI_MCR0_SERCLKDIV_MASK)

#define FLEXSPI_MCR0_HSEN_MASK                   (0x800U)
#define FLEXSPI_MCR0_HSEN_SHIFT                  (11U)
/*! HSEN - Half Speed Serial Flash Memory Access Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define FLEXSPI_MCR0_HSEN(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_HSEN_SHIFT)) & FLEXSPI_MCR0_HSEN_MASK)

#define FLEXSPI_MCR0_DOZEEN_MASK                 (0x1000U)
#define FLEXSPI_MCR0_DOZEEN_SHIFT                (12U)
/*! DOZEEN - Doze Mode Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define FLEXSPI_MCR0_DOZEEN(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_DOZEEN_SHIFT)) & FLEXSPI_MCR0_DOZEEN_MASK)

#define FLEXSPI_MCR0_COMBINATIONEN_MASK          (0x2000U)
#define FLEXSPI_MCR0_COMBINATIONEN_SHIFT         (13U)
/*! COMBINATIONEN - Combination Mode Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define FLEXSPI_MCR0_COMBINATIONEN(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_COMBINATIONEN_SHIFT)) & FLEXSPI_MCR0_COMBINATIONEN_MASK)

#define FLEXSPI_MCR0_SCKFREERUNEN_MASK           (0x4000U)
#define FLEXSPI_MCR0_SCKFREERUNEN_SHIFT          (14U)
/*! SCKFREERUNEN - SCLK Free-running Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define FLEXSPI_MCR0_SCKFREERUNEN(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_SCKFREERUNEN_SHIFT)) & FLEXSPI_MCR0_SCKFREERUNEN_MASK)

#define FLEXSPI_MCR0_IPGRANTWAIT_MASK            (0xFF0000U)
#define FLEXSPI_MCR0_IPGRANTWAIT_SHIFT           (16U)
/*! IPGRANTWAIT - Timeout Wait Cycle for IP Command Grant */
#define FLEXSPI_MCR0_IPGRANTWAIT(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_IPGRANTWAIT_SHIFT)) & FLEXSPI_MCR0_IPGRANTWAIT_MASK)

#define FLEXSPI_MCR0_AHBGRANTWAIT_MASK           (0xFF000000U)
#define FLEXSPI_MCR0_AHBGRANTWAIT_SHIFT          (24U)
/*! AHBGRANTWAIT - Timeouts Wait Cycle for AHB command Grant */
#define FLEXSPI_MCR0_AHBGRANTWAIT(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR0_AHBGRANTWAIT_SHIFT)) & FLEXSPI_MCR0_AHBGRANTWAIT_MASK)
/*! @} */

/*! @name MCR1 - Module Control 1 */
/*! @{ */

#define FLEXSPI_MCR1_AHBBUSWAIT_MASK             (0xFFFFU)
#define FLEXSPI_MCR1_AHBBUSWAIT_SHIFT            (0U)
/*! AHBBUSWAIT - AHB Bus Wait */
#define FLEXSPI_MCR1_AHBBUSWAIT(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR1_AHBBUSWAIT_SHIFT)) & FLEXSPI_MCR1_AHBBUSWAIT_MASK)

#define FLEXSPI_MCR1_SEQWAIT_MASK                (0xFFFF0000U)
#define FLEXSPI_MCR1_SEQWAIT_SHIFT               (16U)
/*! SEQWAIT - Command Sequence Wait */
#define FLEXSPI_MCR1_SEQWAIT(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR1_SEQWAIT_SHIFT)) & FLEXSPI_MCR1_SEQWAIT_MASK)
/*! @} */

/*! @name MCR2 - Module Control 2 */
/*! @{ */

#define FLEXSPI_MCR2_CLRAHBBUFOPT_MASK           (0x800U)
#define FLEXSPI_MCR2_CLRAHBBUFOPT_SHIFT          (11U)
/*! CLRAHBBUFOPT - Clear AHB Buffer
 *  0b0..Not cleared automatically
 *  0b1..Cleared automatically
 */
#define FLEXSPI_MCR2_CLRAHBBUFOPT(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR2_CLRAHBBUFOPT_SHIFT)) & FLEXSPI_MCR2_CLRAHBBUFOPT_MASK)

#define FLEXSPI_MCR2_SAMEDEVICEEN_MASK           (0x8000U)
#define FLEXSPI_MCR2_SAMEDEVICEEN_SHIFT          (15U)
/*! SAMEDEVICEEN - Same Device Enable
 *  0b0..In Individual mode, FLSHA1CRx and FLSHA2CRx, FLSHB1CRx and FLSHB2CRx settings are applied to Flash A1,
 *       A2, B1, B2 separately. In Parallel mode, FLSHA1CRx register setting is applied to Flash A1 and B1, FLSHA2CRx
 *       register setting is applied to Flash A2 and B2. FLSHB1CRx and FLSHB2CRx register settings are ignored.
 *  0b1..FLSHA1CR0, FLSHA1CR1, and FLSHA1CR2 register settings are applied to Flash A1, A2, B1, B2. FLSHA2CRx,
 *       FLSHB1CRx, and FLSHB2CRx settings are ignored.
 */
#define FLEXSPI_MCR2_SAMEDEVICEEN(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR2_SAMEDEVICEEN_SHIFT)) & FLEXSPI_MCR2_SAMEDEVICEEN_MASK)

#define FLEXSPI_MCR2_SCKBDIFFOPT_MASK            (0x80000U)
#define FLEXSPI_MCR2_SCKBDIFFOPT_SHIFT           (19U)
/*! SCKBDIFFOPT - SCLK Port B Differential Output
 *  0b1..Use B_SCLK pad as port A SCLK inverted clock output (Differential clock to A_SCLK). Port B flash memory access is not available.
 *  0b0..Use B_SCLK pad as port B SCLK clock output. Port B flash memory access is available.
 */
#define FLEXSPI_MCR2_SCKBDIFFOPT(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR2_SCKBDIFFOPT_SHIFT)) & FLEXSPI_MCR2_SCKBDIFFOPT_MASK)

#define FLEXSPI_MCR2_RXCLKSRC_B_MASK             (0x600000U)
#define FLEXSPI_MCR2_RXCLKSRC_B_SHIFT            (21U)
/*! RXCLKSRC_B - Port B Receiver Clock Source
 *  0b00..Dummy read strobe that FlexSPI generates, looped back internally.
 *  0b01..Dummy read strobe that FlexSPI generates, looped back from DQS pad.
 *  0b10..SCLK output clock and looped back from SCLK padReserved
 *  0b11..Flash-memory-provided read strobe and input from DQS pad
 */
#define FLEXSPI_MCR2_RXCLKSRC_B(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR2_RXCLKSRC_B_SHIFT)) & FLEXSPI_MCR2_RXCLKSRC_B_MASK)

#define FLEXSPI_MCR2_RX_CLK_SRC_DIFF_MASK        (0x800000U)
#define FLEXSPI_MCR2_RX_CLK_SRC_DIFF_SHIFT       (23U)
/*! RX_CLK_SRC_DIFF - Sample Clock Source Different
 *  0b0..Use MCR0[RXCLKSRC] for Port A and Port B. MCR2[RXCLKSRC_B] is ignored and MCR0[RXCLKSRC] selects the
 *       Sample Clock source for Flash Reading of both ports A and B.
 *  0b1..Use MCR0[RXCLKSRC] for Port A, and MCR2[RXCLKSRC_B] for Port B. MCR0[RXCLKSRC] selects the Sample Clock
 *       source for Flash Reading of port A (A_SCLK) and MCR2[RXCLKSRC_B] selects the Sample Clock source for Flash
 *       Reading of port B (B_SCLK).
 */
#define FLEXSPI_MCR2_RX_CLK_SRC_DIFF(x)          (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR2_RX_CLK_SRC_DIFF_SHIFT)) & FLEXSPI_MCR2_RX_CLK_SRC_DIFF_MASK)

#define FLEXSPI_MCR2_RESUMEWAIT_MASK             (0xFF000000U)
#define FLEXSPI_MCR2_RESUMEWAIT_SHIFT            (24U)
/*! RESUMEWAIT - Resume Wait Duration */
#define FLEXSPI_MCR2_RESUMEWAIT(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_MCR2_RESUMEWAIT_SHIFT)) & FLEXSPI_MCR2_RESUMEWAIT_MASK)
/*! @} */

/*! @name AHBCR - AHB Bus Control */
/*! @{ */

#define FLEXSPI_AHBCR_APAREN_MASK                (0x1U)
#define FLEXSPI_AHBCR_APAREN_SHIFT               (0U)
/*! APAREN - AHB Parallel Mode Enable
 *  0b0..Flash is accessed in Individual mode.
 *  0b1..Flash is accessed in Parallel mode.
 */
#define FLEXSPI_AHBCR_APAREN(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_APAREN_SHIFT)) & FLEXSPI_AHBCR_APAREN_MASK)

#define FLEXSPI_AHBCR_CLRAHBTXBUF_MASK           (0x4U)
#define FLEXSPI_AHBCR_CLRAHBTXBUF_SHIFT          (2U)
/*! CLRAHBTXBUF - Clear AHB Transmit Buffer
 *  0b0..No impact.
 *  0b1..Enable clear operation.
 */
#define FLEXSPI_AHBCR_CLRAHBTXBUF(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_CLRAHBTXBUF_SHIFT)) & FLEXSPI_AHBCR_CLRAHBTXBUF_MASK)

#define FLEXSPI_AHBCR_CACHABLEEN_MASK            (0x8U)
#define FLEXSPI_AHBCR_CACHABLEEN_SHIFT           (3U)
/*! CACHABLEEN - Cacheable Read Access Enable
 *  0b0..Disabled. When an AHB bus cacheable read access occurs, FlexSPI does not check whether it hit the AHB transmit buffer.
 *  0b1..Enabled. When an AHB bus cacheable read access occurs, FlexSPI first checks whether the access hit the AHB transmit buffer.
 */
#define FLEXSPI_AHBCR_CACHABLEEN(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_CACHABLEEN_SHIFT)) & FLEXSPI_AHBCR_CACHABLEEN_MASK)

#define FLEXSPI_AHBCR_BUFFERABLEEN_MASK          (0x10U)
#define FLEXSPI_AHBCR_BUFFERABLEEN_SHIFT         (4U)
/*! BUFFERABLEEN - Bufferable Write Access Enable
 *  0b0..Disabled. For all AHB write accesses (bufferable or nonbufferable), FlexSPI returns AHB Bus Ready after
 *       transmitting all data and finishing command.
 *  0b1..Enabled. For AHB bufferable write access, FlexSPI returns AHB Bus Ready when the arbitrator grants the
 *       AHB command. FlexSPI does not wait for the AHB command to finish.
 */
#define FLEXSPI_AHBCR_BUFFERABLEEN(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_BUFFERABLEEN_SHIFT)) & FLEXSPI_AHBCR_BUFFERABLEEN_MASK)

#define FLEXSPI_AHBCR_PREFETCHEN_MASK            (0x20U)
#define FLEXSPI_AHBCR_PREFETCHEN_SHIFT           (5U)
/*! PREFETCHEN - AHB Read Prefetch Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define FLEXSPI_AHBCR_PREFETCHEN(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_PREFETCHEN_SHIFT)) & FLEXSPI_AHBCR_PREFETCHEN_MASK)

#define FLEXSPI_AHBCR_READADDROPT_MASK           (0x40U)
#define FLEXSPI_AHBCR_READADDROPT_SHIFT          (6U)
/*! READADDROPT - AHB Read Address Option
 *  0b0..AHB read burst start address alignment is limited when flash memory is accessed in parallel mode or flash is word-addressable.
 *  0b1..AHB read burst start address alignment is not limited. FlexSPI fetches more data than the AHB burst requires for address alignment.
 */
#define FLEXSPI_AHBCR_READADDROPT(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_READADDROPT_SHIFT)) & FLEXSPI_AHBCR_READADDROPT_MASK)

#define FLEXSPI_AHBCR_RESUMEDISABLE_MASK         (0x80U)
#define FLEXSPI_AHBCR_RESUMEDISABLE_SHIFT        (7U)
/*! RESUMEDISABLE - AHB Read Resume Disable
 *  0b0..Suspended AHB read prefetch resumes when AHB is IDLE.
 *  0b1..Suspended AHB read prefetch does not resume once aborted.
 */
#define FLEXSPI_AHBCR_RESUMEDISABLE(x)           (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_RESUMEDISABLE_SHIFT)) & FLEXSPI_AHBCR_RESUMEDISABLE_MASK)

#define FLEXSPI_AHBCR_READSZALIGN_MASK           (0x400U)
#define FLEXSPI_AHBCR_READSZALIGN_SHIFT          (10U)
/*! READSZALIGN - AHB Read Size Alignment
 *  0b0..Register settings such as PREFETCH_EN and OTFAD_EN determine AHB read size.
 *  0b1..AHB read size to up size to 8 bytes aligned, no prefetching
 */
#define FLEXSPI_AHBCR_READSZALIGN(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_READSZALIGN_SHIFT)) & FLEXSPI_AHBCR_READSZALIGN_MASK)

#define FLEXSPI_AHBCR_ALIGNMENT_MASK             (0x300000U)
#define FLEXSPI_AHBCR_ALIGNMENT_SHIFT            (20U)
/*! ALIGNMENT - AHB Boundary Alignment
 *  0b00..No limit
 *  0b01..1 KB
 *  0b10..512 bytes
 *  0b11..256 bytes
 */
#define FLEXSPI_AHBCR_ALIGNMENT(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_ALIGNMENT_SHIFT)) & FLEXSPI_AHBCR_ALIGNMENT_MASK)

#define FLEXSPI_AHBCR_AFLASHBASE_MASK            (0xFC000000U)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */
#define FLEXSPI_AHBCR_AFLASHBASE_SHIFT           (26U)
/*! AFLASHBASE - AHB Memory-Mapped Flash Base Address */
#define FLEXSPI_AHBCR_AFLASHBASE(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBCR_AFLASHBASE_SHIFT)) & FLEXSPI_AHBCR_AFLASHBASE_MASK)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */
/*! @} */

/*! @name INTEN - Interrupt Enable */
/*! @{ */

#define FLEXSPI_INTEN_IPCMDDONEEN_MASK           (0x1U)
#define FLEXSPI_INTEN_IPCMDDONEEN_SHIFT          (0U)
/*! IPCMDDONEEN - IP-Triggered Command Sequences Execution Finished Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_IPCMDDONEEN(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_IPCMDDONEEN_SHIFT)) & FLEXSPI_INTEN_IPCMDDONEEN_MASK)

#define FLEXSPI_INTEN_IPCMDGEEN_MASK             (0x2U)
#define FLEXSPI_INTEN_IPCMDGEEN_SHIFT            (1U)
/*! IPCMDGEEN - IP-Triggered Command Sequences Grant Timeout Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_IPCMDGEEN(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_IPCMDGEEN_SHIFT)) & FLEXSPI_INTEN_IPCMDGEEN_MASK)

#define FLEXSPI_INTEN_AHBCMDGEEN_MASK            (0x4U)
#define FLEXSPI_INTEN_AHBCMDGEEN_SHIFT           (2U)
/*! AHBCMDGEEN - AHB-Triggered Command Sequences Grant Timeout Interrupt Enable.
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_AHBCMDGEEN(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_AHBCMDGEEN_SHIFT)) & FLEXSPI_INTEN_AHBCMDGEEN_MASK)

#define FLEXSPI_INTEN_IPCMDERREN_MASK            (0x8U)
#define FLEXSPI_INTEN_IPCMDERREN_SHIFT           (3U)
/*! IPCMDERREN - IP-Triggered Command Sequences Error Detected Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_IPCMDERREN(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_IPCMDERREN_SHIFT)) & FLEXSPI_INTEN_IPCMDERREN_MASK)

#define FLEXSPI_INTEN_AHBCMDERREN_MASK           (0x10U)
#define FLEXSPI_INTEN_AHBCMDERREN_SHIFT          (4U)
/*! AHBCMDERREN - AHB-Triggered Command Sequences Error Detected Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_AHBCMDERREN(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_AHBCMDERREN_SHIFT)) & FLEXSPI_INTEN_AHBCMDERREN_MASK)

#define FLEXSPI_INTEN_IPRXWAEN_MASK              (0x20U)
#define FLEXSPI_INTEN_IPRXWAEN_SHIFT             (5U)
/*! IPRXWAEN - IP Receive FIFO Watermark Available Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_IPRXWAEN(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_IPRXWAEN_SHIFT)) & FLEXSPI_INTEN_IPRXWAEN_MASK)

#define FLEXSPI_INTEN_IPTXWEEN_MASK              (0x40U)
#define FLEXSPI_INTEN_IPTXWEEN_SHIFT             (6U)
/*! IPTXWEEN - IP Transmit FIFO Watermark Empty Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_IPTXWEEN(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_IPTXWEEN_SHIFT)) & FLEXSPI_INTEN_IPTXWEEN_MASK)

#define FLEXSPI_INTEN_SCKSTOPBYRDEN_MASK         (0x100U)
#define FLEXSPI_INTEN_SCKSTOPBYRDEN_SHIFT        (8U)
/*! SCKSTOPBYRDEN - SCLK Stopped By Read Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_SCKSTOPBYRDEN(x)           (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_SCKSTOPBYRDEN_SHIFT)) & FLEXSPI_INTEN_SCKSTOPBYRDEN_MASK)

#define FLEXSPI_INTEN_SCKSTOPBYWREN_MASK         (0x200U)
#define FLEXSPI_INTEN_SCKSTOPBYWREN_SHIFT        (9U)
/*! SCKSTOPBYWREN - SCLK Stopped By Write Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_SCKSTOPBYWREN(x)           (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_SCKSTOPBYWREN_SHIFT)) & FLEXSPI_INTEN_SCKSTOPBYWREN_MASK)

#define FLEXSPI_INTEN_AHBBUSERROREN_MASK         (0x400U)
#define FLEXSPI_INTEN_AHBBUSERROREN_SHIFT        (10U)
/*! AHBBUSERROREN - AHB Bus Error Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_AHBBUSERROREN(x)           (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_AHBBUSERROREN_SHIFT)) & FLEXSPI_INTEN_AHBBUSERROREN_MASK)

#define FLEXSPI_INTEN_SEQTIMEOUTEN_MASK          (0x800U)
#define FLEXSPI_INTEN_SEQTIMEOUTEN_SHIFT         (11U)
/*! SEQTIMEOUTEN - Sequence execution Timeout Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_SEQTIMEOUTEN(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_SEQTIMEOUTEN_SHIFT)) & FLEXSPI_INTEN_SEQTIMEOUTEN_MASK)

#define FLEXSPI_INTEN_KEYDONEEN_MASK             (0x1000U)
#define FLEXSPI_INTEN_KEYDONEEN_SHIFT            (12U)
/*! KEYDONEEN - OTFAD Key Blob Processing Done Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_KEYDONEEN(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_KEYDONEEN_SHIFT)) & FLEXSPI_INTEN_KEYDONEEN_MASK)

#define FLEXSPI_INTEN_KEYERROREN_MASK            (0x2000U)
#define FLEXSPI_INTEN_KEYERROREN_SHIFT           (13U)
/*! KEYERROREN - OTFAD Key Blob Processing Error Interrupt Enable
 *  0b0..Disable interrupt or no impact
 *  0b1..Enable interrupt
 */
#define FLEXSPI_INTEN_KEYERROREN(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTEN_KEYERROREN_SHIFT)) & FLEXSPI_INTEN_KEYERROREN_MASK)
/*! @} */

/*! @name INTR - Interrupt */
/*! @{ */

#define FLEXSPI_INTR_IPCMDDONE_MASK              (0x1U)
#define FLEXSPI_INTR_IPCMDDONE_SHIFT             (0U)
/*! IPCMDDONE - IP-Triggered Command Sequences Execution Finished
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_IPCMDDONE(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_IPCMDDONE_SHIFT)) & FLEXSPI_INTR_IPCMDDONE_MASK)

#define FLEXSPI_INTR_IPCMDGE_MASK                (0x2U)
#define FLEXSPI_INTR_IPCMDGE_SHIFT               (1U)
/*! IPCMDGE - IP-Triggered Command Sequences Grant Timeout
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_IPCMDGE(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_IPCMDGE_SHIFT)) & FLEXSPI_INTR_IPCMDGE_MASK)

#define FLEXSPI_INTR_AHBCMDGE_MASK               (0x4U)
#define FLEXSPI_INTR_AHBCMDGE_SHIFT              (2U)
/*! AHBCMDGE - AHB-Triggered Command Sequences Grant Timeout
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_AHBCMDGE(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_AHBCMDGE_SHIFT)) & FLEXSPI_INTR_AHBCMDGE_MASK)

#define FLEXSPI_INTR_IPCMDERR_MASK               (0x8U)
#define FLEXSPI_INTR_IPCMDERR_SHIFT              (3U)
/*! IPCMDERR - IP-Triggered Command Sequences Error
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_IPCMDERR(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_IPCMDERR_SHIFT)) & FLEXSPI_INTR_IPCMDERR_MASK)

#define FLEXSPI_INTR_AHBCMDERR_MASK              (0x10U)
#define FLEXSPI_INTR_AHBCMDERR_SHIFT             (4U)
/*! AHBCMDERR - AHB-Triggered Command Sequences Error
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_AHBCMDERR(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_AHBCMDERR_SHIFT)) & FLEXSPI_INTR_AHBCMDERR_MASK)

#define FLEXSPI_INTR_IPRXWA_MASK                 (0x20U)
#define FLEXSPI_INTR_IPRXWA_SHIFT                (5U)
/*! IPRXWA - IP Receive FIFO Watermark Available
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_IPRXWA(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_IPRXWA_SHIFT)) & FLEXSPI_INTR_IPRXWA_MASK)

#define FLEXSPI_INTR_IPTXWE_MASK                 (0x40U)
#define FLEXSPI_INTR_IPTXWE_SHIFT                (6U)
/*! IPTXWE - IP Transmit FIFO Watermark Empty
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_IPTXWE(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_IPTXWE_SHIFT)) & FLEXSPI_INTR_IPTXWE_MASK)

#define FLEXSPI_INTR_SCKSTOPBYRD_MASK            (0x100U)
#define FLEXSPI_INTR_SCKSTOPBYRD_SHIFT           (8U)
/*! SCKSTOPBYRD - SCLK Stopped Due To Full Receive FIFO
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_SCKSTOPBYRD(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_SCKSTOPBYRD_SHIFT)) & FLEXSPI_INTR_SCKSTOPBYRD_MASK)

#define FLEXSPI_INTR_SCKSTOPBYWR_MASK            (0x200U)
#define FLEXSPI_INTR_SCKSTOPBYWR_SHIFT           (9U)
/*! SCKSTOPBYWR - SCLK Stopped Due To Empty Transmit FIFO
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_SCKSTOPBYWR(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_SCKSTOPBYWR_SHIFT)) & FLEXSPI_INTR_SCKSTOPBYWR_MASK)

#define FLEXSPI_INTR_AHBBUSERROR_MASK            (0x400U)
#define FLEXSPI_INTR_AHBBUSERROR_SHIFT           (10U)
/*! AHBBUSERROR - AHB Bus Error
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_AHBBUSERROR(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_AHBBUSERROR_SHIFT)) & FLEXSPI_INTR_AHBBUSERROR_MASK)

#define FLEXSPI_INTR_SEQTIMEOUT_MASK             (0x800U)
#define FLEXSPI_INTR_SEQTIMEOUT_SHIFT            (11U)
/*! SEQTIMEOUT - Sequence Execution Timeout
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_SEQTIMEOUT(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_SEQTIMEOUT_SHIFT)) & FLEXSPI_INTR_SEQTIMEOUT_MASK)

#define FLEXSPI_INTR_KEYDONE_MASK                (0x1000U)
#define FLEXSPI_INTR_KEYDONE_SHIFT               (12U)
/*! KEYDONE - OTFAD key blob processing done interrupt. */
#define FLEXSPI_INTR_KEYDONE(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_KEYDONE_SHIFT)) & FLEXSPI_INTR_KEYDONE_MASK)

#define FLEXSPI_INTR_KEYERROR_MASK               (0x2000U)
#define FLEXSPI_INTR_KEYERROR_SHIFT              (13U)
/*! KEYERROR - OTFAD Key Blob Processing Error
 *  0b0..Interrupt condition has not occurred
 *  0b1..Interrupt condition has occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define FLEXSPI_INTR_KEYERROR(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_INTR_KEYERROR_SHIFT)) & FLEXSPI_INTR_KEYERROR_MASK)
/*! @} */

/*! @name LUTKEY - LUT Key */
/*! @{ */

#define FLEXSPI_LUTKEY_KEY_MASK                  (0xFFFFFFFFU)
#define FLEXSPI_LUTKEY_KEY_SHIFT                 (0U)
/*! KEY - LUT Key */
#define FLEXSPI_LUTKEY_KEY(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUTKEY_KEY_SHIFT)) & FLEXSPI_LUTKEY_KEY_MASK)
/*! @} */

/*! @name LUTCR - LUT Control */
/*! @{ */

#define FLEXSPI_LUTCR_LOCK_MASK                  (0x1U)
#define FLEXSPI_LUTCR_LOCK_SHIFT                 (0U)
/*! LOCK - Lock LUT
 *  0b0..LUT is unlocked (LUTCR[UNLOCK] must be 1)
 *  0b1..LUT is locked and cannot be written
 */
#define FLEXSPI_LUTCR_LOCK(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUTCR_LOCK_SHIFT)) & FLEXSPI_LUTCR_LOCK_MASK)

#define FLEXSPI_LUTCR_UNLOCK_MASK                (0x2U)
#define FLEXSPI_LUTCR_UNLOCK_SHIFT               (1U)
/*! UNLOCK - Unlock LUT
 *  0b0..LUT is locked (LUTCR[LOCK] must be 1)
 *  0b1..LUT is unlocked and can be written
 */
#define FLEXSPI_LUTCR_UNLOCK(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUTCR_UNLOCK_SHIFT)) & FLEXSPI_LUTCR_UNLOCK_MASK)
/*! @} */

/*! @name AHBRXBUFCR0 - AHB Receive Buffer 0 Control 0..AHB Receive Buffer 7 Control 0 */
/*! @{ */

#define FLEXSPI_AHBRXBUFCR0_BUFSZ_MASK           (0x3FFU)
#define FLEXSPI_AHBRXBUFCR0_BUFSZ_SHIFT          (0U)
/*! BUFSZ - AHB Receive Buffer Size */
#define FLEXSPI_AHBRXBUFCR0_BUFSZ(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBRXBUFCR0_BUFSZ_SHIFT)) & FLEXSPI_AHBRXBUFCR0_BUFSZ_MASK)

#define FLEXSPI_AHBRXBUFCR0_MSTRID_MASK          (0xF0000U)
#define FLEXSPI_AHBRXBUFCR0_MSTRID_SHIFT         (16U)
/*! MSTRID - AHB Controller ID */
#define FLEXSPI_AHBRXBUFCR0_MSTRID(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBRXBUFCR0_MSTRID_SHIFT)) & FLEXSPI_AHBRXBUFCR0_MSTRID_MASK)

#define FLEXSPI_AHBRXBUFCR0_PRIORITY_MASK        (0x7000000U)
#define FLEXSPI_AHBRXBUFCR0_PRIORITY_SHIFT       (24U)
/*! PRIORITY - AHB Controller Read Priority */
#define FLEXSPI_AHBRXBUFCR0_PRIORITY(x)          (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBRXBUFCR0_PRIORITY_SHIFT)) & FLEXSPI_AHBRXBUFCR0_PRIORITY_MASK)

#define FLEXSPI_AHBRXBUFCR0_REGIONEN_MASK        (0x40000000U)
#define FLEXSPI_AHBRXBUFCR0_REGIONEN_SHIFT       (30U)
/*! REGIONEN - AHB Receive Buffer Address Region Enable
 *  0b0..Disabled. The buffer hit is based on the value of MSTRID only.
 *  0b1..Enabled. The buffer hit is based on the value of MSTRID and the address within AHBBUFREGIONSTARTn and AHBREGIONENDn.
 */
#define FLEXSPI_AHBRXBUFCR0_REGIONEN(x)          (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBRXBUFCR0_REGIONEN_SHIFT)) & FLEXSPI_AHBRXBUFCR0_REGIONEN_MASK)

#define FLEXSPI_AHBRXBUFCR0_PREFETCHEN_MASK      (0x80000000U)
#define FLEXSPI_AHBRXBUFCR0_PREFETCHEN_SHIFT     (31U)
/*! PREFETCHEN - AHB Read Prefetch Enable
 *  0b0..Disabled
 *  0b1..Enabled when is enabled.
 */
#define FLEXSPI_AHBRXBUFCR0_PREFETCHEN(x)        (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBRXBUFCR0_PREFETCHEN_SHIFT)) & FLEXSPI_AHBRXBUFCR0_PREFETCHEN_MASK)
/*! @} */

/* The count of FLEXSPI_AHBRXBUFCR0 */
#define FLEXSPI_AHBRXBUFCR0_COUNT                (8U)

/*! @name FLSHCR0 - Flash Control 0 */
/*! @{ */

#define FLEXSPI_FLSHCR0_FLSHSZ_MASK              (0x7FFFFFU)
#define FLEXSPI_FLSHCR0_FLSHSZ_SHIFT             (0U)
/*! FLSHSZ - Flash Size in KB */
#define FLEXSPI_FLSHCR0_FLSHSZ(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR0_FLSHSZ_SHIFT)) & FLEXSPI_FLSHCR0_FLSHSZ_MASK)

#define FLEXSPI_FLSHCR0_ADDRSHIFT_MASK           (0x20000000U)
#define FLEXSPI_FLSHCR0_ADDRSHIFT_SHIFT          (29U)
/*! ADDRSHIFT - AHB Address Shift Function control
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define FLEXSPI_FLSHCR0_ADDRSHIFT(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR0_ADDRSHIFT_SHIFT)) & FLEXSPI_FLSHCR0_ADDRSHIFT_MASK)
/*! @} */

/* The count of FLEXSPI_FLSHCR0 */
#define FLEXSPI_FLSHCR0_COUNT                    (4U)

/*! @name FLSHCR1 - Flash Control 1 */
/*! @{ */

#define FLEXSPI_FLSHCR1_TCSS_MASK                (0x1FU)
#define FLEXSPI_FLSHCR1_TCSS_SHIFT               (0U)
/*! TCSS - Serial Flash CS Setup Time */
#define FLEXSPI_FLSHCR1_TCSS(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR1_TCSS_SHIFT)) & FLEXSPI_FLSHCR1_TCSS_MASK)

#define FLEXSPI_FLSHCR1_TCSH_MASK                (0x3E0U)
#define FLEXSPI_FLSHCR1_TCSH_SHIFT               (5U)
/*! TCSH - Serial Flash CS Hold Time */
#define FLEXSPI_FLSHCR1_TCSH(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR1_TCSH_SHIFT)) & FLEXSPI_FLSHCR1_TCSH_MASK)

#define FLEXSPI_FLSHCR1_WA_MASK                  (0x400U)
#define FLEXSPI_FLSHCR1_WA_SHIFT                 (10U)
/*! WA - Word-Addressable
 *  0b0..Byte-addressable
 *  0b1..Word-addressable
 */
#define FLEXSPI_FLSHCR1_WA(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR1_WA_SHIFT)) & FLEXSPI_FLSHCR1_WA_MASK)

#define FLEXSPI_FLSHCR1_CAS_MASK                 (0x7800U)
#define FLEXSPI_FLSHCR1_CAS_SHIFT                (11U)
/*! CAS - Column Address Size */
#define FLEXSPI_FLSHCR1_CAS(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR1_CAS_SHIFT)) & FLEXSPI_FLSHCR1_CAS_MASK)

#define FLEXSPI_FLSHCR1_CSINTERVALUNIT_MASK      (0x8000U)
#define FLEXSPI_FLSHCR1_CSINTERVALUNIT_SHIFT     (15U)
/*! CSINTERVALUNIT - Chip Select Interval Unit
 *  0b0..1 serial clock cycle
 *  0b1..256 serial clock cycles
 */
#define FLEXSPI_FLSHCR1_CSINTERVALUNIT(x)        (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR1_CSINTERVALUNIT_SHIFT)) & FLEXSPI_FLSHCR1_CSINTERVALUNIT_MASK)

#define FLEXSPI_FLSHCR1_CSINTERVAL_MASK          (0xFFFF0000U)
#define FLEXSPI_FLSHCR1_CSINTERVAL_SHIFT         (16U)
/*! CSINTERVAL - Chip Select Interval */
#define FLEXSPI_FLSHCR1_CSINTERVAL(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR1_CSINTERVAL_SHIFT)) & FLEXSPI_FLSHCR1_CSINTERVAL_MASK)
/*! @} */

/* The count of FLEXSPI_FLSHCR1 */
#define FLEXSPI_FLSHCR1_COUNT                    (4U)

/*! @name FLSHCR2 - Flash Control 2 */
/*! @{ */

#define FLEXSPI_FLSHCR2_ARDSEQID_MASK            (0x1FU)
#define FLEXSPI_FLSHCR2_ARDSEQID_SHIFT           (0U)
/*! ARDSEQID - Sequence Index for AHB Read-Triggered Command in LUT */
#define FLEXSPI_FLSHCR2_ARDSEQID(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR2_ARDSEQID_SHIFT)) & FLEXSPI_FLSHCR2_ARDSEQID_MASK)

#define FLEXSPI_FLSHCR2_ARDSEQNUM_MASK           (0xE0U)
#define FLEXSPI_FLSHCR2_ARDSEQNUM_SHIFT          (5U)
/*! ARDSEQNUM - Sequence Number for AHB Read-Triggered Command */
#define FLEXSPI_FLSHCR2_ARDSEQNUM(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR2_ARDSEQNUM_SHIFT)) & FLEXSPI_FLSHCR2_ARDSEQNUM_MASK)

#define FLEXSPI_FLSHCR2_AWRSEQID_MASK            (0x1F00U)
#define FLEXSPI_FLSHCR2_AWRSEQID_SHIFT           (8U)
/*! AWRSEQID - Sequence Index for AHB Write-Triggered Command */
#define FLEXSPI_FLSHCR2_AWRSEQID(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR2_AWRSEQID_SHIFT)) & FLEXSPI_FLSHCR2_AWRSEQID_MASK)

#define FLEXSPI_FLSHCR2_AWRSEQNUM_MASK           (0xE000U)
#define FLEXSPI_FLSHCR2_AWRSEQNUM_SHIFT          (13U)
/*! AWRSEQNUM - Sequence Number for AHB Write-Triggered Command */
#define FLEXSPI_FLSHCR2_AWRSEQNUM(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR2_AWRSEQNUM_SHIFT)) & FLEXSPI_FLSHCR2_AWRSEQNUM_MASK)

#define FLEXSPI_FLSHCR2_AWRWAIT_MASK             (0xFFF0000U)
#define FLEXSPI_FLSHCR2_AWRWAIT_SHIFT            (16U)
/*! AWRWAIT - AHB Write Wait */
#define FLEXSPI_FLSHCR2_AWRWAIT(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR2_AWRWAIT_SHIFT)) & FLEXSPI_FLSHCR2_AWRWAIT_MASK)

#define FLEXSPI_FLSHCR2_AWRWAITUNIT_MASK         (0x70000000U)
#define FLEXSPI_FLSHCR2_AWRWAITUNIT_SHIFT        (28U)
/*! AWRWAITUNIT - AWRWAIT Unit
 *  0b000..2
 *  0b001..8
 *  0b010..32
 *  0b011..128
 *  0b100..512
 *  0b101..2048
 *  0b110..8192
 *  0b111..32768
 */
#define FLEXSPI_FLSHCR2_AWRWAITUNIT(x)           (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR2_AWRWAITUNIT_SHIFT)) & FLEXSPI_FLSHCR2_AWRWAITUNIT_MASK)

#define FLEXSPI_FLSHCR2_CLRINSTRPTR_MASK         (0x80000000U)
#define FLEXSPI_FLSHCR2_CLRINSTRPTR_SHIFT        (31U)
/*! CLRINSTRPTR - Clear Instruction Pointer */
#define FLEXSPI_FLSHCR2_CLRINSTRPTR(x)           (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR2_CLRINSTRPTR_SHIFT)) & FLEXSPI_FLSHCR2_CLRINSTRPTR_MASK)
/*! @} */

/* The count of FLEXSPI_FLSHCR2 */
#define FLEXSPI_FLSHCR2_COUNT                    (4U)

/*! @name FLSHCR4 - Flash Control 4 */
/*! @{ */

#define FLEXSPI_FLSHCR4_WMOPT1_MASK              (0x1U)
#define FLEXSPI_FLSHCR4_WMOPT1_SHIFT             (0U)
/*! WMOPT1 - Write Mask Option 1
 *  0b0..When writing to an external device, DQS pin is used as write mask. When flash memory is accessed in
 *       individual mode, AHB or IP write burst start address alignment is not limited.
 *  0b1..When writing to an external device, DQS pin is not used as write mask. When flash memory is accessed in
 *       individual mode, AHB or IP write burst start address alignment is limited.
 */
#define FLEXSPI_FLSHCR4_WMOPT1(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR4_WMOPT1_SHIFT)) & FLEXSPI_FLSHCR4_WMOPT1_MASK)

#define FLEXSPI_FLSHCR4_WMOPT2_MASK              (0x2U)
#define FLEXSPI_FLSHCR4_WMOPT2_SHIFT             (1U)
/*! WMOPT2 - Write Mask Option 2
 *  0b0..When writing to an external device, DQS pin is used as write mask. When flash memory is accessed in
 *       individual mode, AHB or IP write burst length is not limited.
 *  0b1..When writing to an external device, DQS pin is not used as write mask. When flash memory is accessed in
 *       individual mode, AHB or IP write burst length is limited. The minimum write burst length should be 4.
 */
#define FLEXSPI_FLSHCR4_WMOPT2(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR4_WMOPT2_SHIFT)) & FLEXSPI_FLSHCR4_WMOPT2_MASK)

#define FLEXSPI_FLSHCR4_WMENA_MASK               (0x4U)
#define FLEXSPI_FLSHCR4_WMENA_SHIFT              (2U)
/*! WMENA - Write Mask Enable for Port A
 *  0b0..Disabled. When writing to external device, DQS(RWDS) pin is not driven.
 *  0b1..Enabled. When writing to external device, FlexSPI drives DQS(RWDS) pin as write mask output.
 */
#define FLEXSPI_FLSHCR4_WMENA(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR4_WMENA_SHIFT)) & FLEXSPI_FLSHCR4_WMENA_MASK)

#define FLEXSPI_FLSHCR4_WMENB_MASK               (0x8U)
#define FLEXSPI_FLSHCR4_WMENB_SHIFT              (3U)
/*! WMENB - Write Mask Enable for Port B
 *  0b0..Disabled. When writing to external device, DQS(RWDS) pin is not driven.
 *  0b1..Enabled. When writing to external device, FlexSPI drives DQS(RWDS) pin as write mask output.
 */
#define FLEXSPI_FLSHCR4_WMENB(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_FLSHCR4_WMENB_SHIFT)) & FLEXSPI_FLSHCR4_WMENB_MASK)
/*! @} */

/*! @name IPCR0 - IP Control 0 */
/*! @{ */

#define FLEXSPI_IPCR0_SFAR_MASK                  (0xFFFFFFFFU)
#define FLEXSPI_IPCR0_SFAR_SHIFT                 (0U)
/*! SFAR - Serial Flash Address */
#define FLEXSPI_IPCR0_SFAR(x)                    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPCR0_SFAR_SHIFT)) & FLEXSPI_IPCR0_SFAR_MASK)
/*! @} */

/*! @name IPCR1 - IP Control 1 */
/*! @{ */

#define FLEXSPI_IPCR1_IDATSZ_MASK                (0xFFFFU)
#define FLEXSPI_IPCR1_IDATSZ_SHIFT               (0U)
/*! IDATSZ - Flash Read/Program Data Size (in bytes) for IP command. */
#define FLEXSPI_IPCR1_IDATSZ(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPCR1_IDATSZ_SHIFT)) & FLEXSPI_IPCR1_IDATSZ_MASK)

#define FLEXSPI_IPCR1_ISEQID_MASK                (0x1F0000U)
#define FLEXSPI_IPCR1_ISEQID_SHIFT               (16U)
/*! ISEQID - Sequence Index in LUT for IP command. */
#define FLEXSPI_IPCR1_ISEQID(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPCR1_ISEQID_SHIFT)) & FLEXSPI_IPCR1_ISEQID_MASK)

#define FLEXSPI_IPCR1_ISEQNUM_MASK               (0x7000000U)
#define FLEXSPI_IPCR1_ISEQNUM_SHIFT              (24U)
/*! ISEQNUM - Sequence Number for IP command: ISEQNUM+1. */
#define FLEXSPI_IPCR1_ISEQNUM(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPCR1_ISEQNUM_SHIFT)) & FLEXSPI_IPCR1_ISEQNUM_MASK)

#define FLEXSPI_IPCR1_IPAREN_MASK                (0x80000000U)
#define FLEXSPI_IPCR1_IPAREN_SHIFT               (31U)
/*! IPAREN - Parallel Mode Enable for IP Commands
 *  0b0..Disabled. Flash memory is accessed in Individual mode.
 *  0b1..Enabled. Flash memory is accessed in Parallel mode.
 */
#define FLEXSPI_IPCR1_IPAREN(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPCR1_IPAREN_SHIFT)) & FLEXSPI_IPCR1_IPAREN_MASK)
/*! @} */

/*! @name IPCMD - IP Command */
/*! @{ */

#define FLEXSPI_IPCMD_TRG_MASK                   (0x1U)
#define FLEXSPI_IPCMD_TRG_SHIFT                  (0U)
/*! TRG - Command Trigger
 *  0b0..No action
 *  0b1..Start the IP command that the IPCR0 and IPCR1 registers define.
 */
#define FLEXSPI_IPCMD_TRG(x)                     (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPCMD_TRG_SHIFT)) & FLEXSPI_IPCMD_TRG_MASK)
/*! @} */

/*! @name IPRXFCR - IP Receive FIFO Control */
/*! @{ */

#define FLEXSPI_IPRXFCR_CLRIPRXF_MASK            (0x1U)
#define FLEXSPI_IPRXFCR_CLRIPRXF_SHIFT           (0U)
/*! CLRIPRXF - Clear IP Receive FIFO
 *  0b0..No function
 *  0b1..A clock cycle pulse clears all valid data entries in IP receive FIFO.
 */
#define FLEXSPI_IPRXFCR_CLRIPRXF(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPRXFCR_CLRIPRXF_SHIFT)) & FLEXSPI_IPRXFCR_CLRIPRXF_MASK)

#define FLEXSPI_IPRXFCR_RXDMAEN_MASK             (0x2U)
#define FLEXSPI_IPRXFCR_RXDMAEN_SHIFT            (1U)
/*! RXDMAEN - IP Receive FIFO Reading by DMA Enable
 *  0b0..Disabled. The processor reads the FIFO.
 *  0b1..Enabled. DMA reads the FIFO.
 */
#define FLEXSPI_IPRXFCR_RXDMAEN(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPRXFCR_RXDMAEN_SHIFT)) & FLEXSPI_IPRXFCR_RXDMAEN_MASK)

#define FLEXSPI_IPRXFCR_RXWMRK_MASK              (0x7CU)
#define FLEXSPI_IPRXFCR_RXWMRK_SHIFT             (2U)
/*! RXWMRK - IP Receive FIFO Watermark Level */
#define FLEXSPI_IPRXFCR_RXWMRK(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPRXFCR_RXWMRK_SHIFT)) & FLEXSPI_IPRXFCR_RXWMRK_MASK)
/*! @} */

/*! @name IPTXFCR - IP Transmit FIFO Control */
/*! @{ */

#define FLEXSPI_IPTXFCR_CLRIPTXF_MASK            (0x1U)
#define FLEXSPI_IPTXFCR_CLRIPTXF_SHIFT           (0U)
/*! CLRIPTXF - Clear IP Transmit FIFO
 *  0b0..No function
 *  0b1..A clock cycle pulse clears all valid data entries in the IP transmit FIFO.
 */
#define FLEXSPI_IPTXFCR_CLRIPTXF(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPTXFCR_CLRIPTXF_SHIFT)) & FLEXSPI_IPTXFCR_CLRIPTXF_MASK)

#define FLEXSPI_IPTXFCR_TXDMAEN_MASK             (0x2U)
#define FLEXSPI_IPTXFCR_TXDMAEN_SHIFT            (1U)
/*! TXDMAEN - Transmit FIFO DMA Enable
 *  0b0..Processor
 *  0b1..DMA
 */
#define FLEXSPI_IPTXFCR_TXDMAEN(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPTXFCR_TXDMAEN_SHIFT)) & FLEXSPI_IPTXFCR_TXDMAEN_MASK)

#define FLEXSPI_IPTXFCR_TXWMRK_MASK              (0x1FCU)
#define FLEXSPI_IPTXFCR_TXWMRK_SHIFT             (2U)
/*! TXWMRK - Transmit Watermark Level */
#define FLEXSPI_IPTXFCR_TXWMRK(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPTXFCR_TXWMRK_SHIFT)) & FLEXSPI_IPTXFCR_TXWMRK_MASK)
/*! @} */

/*! @name DLLCR - DLL Control 0 */
/*! @{ */

#define FLEXSPI_DLLCR_DLLEN_MASK                 (0x1U)
#define FLEXSPI_DLLCR_DLLEN_SHIFT                (0U)
/*! DLLEN - DLL Calibration Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define FLEXSPI_DLLCR_DLLEN(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_DLLCR_DLLEN_SHIFT)) & FLEXSPI_DLLCR_DLLEN_MASK)

#define FLEXSPI_DLLCR_DLLRESET_MASK              (0x2U)
#define FLEXSPI_DLLCR_DLLRESET_SHIFT             (1U)
/*! DLLRESET - DLL reset
 *  0b0..No function
 *  0b1..Force DLL reset.
 */
#define FLEXSPI_DLLCR_DLLRESET(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_DLLCR_DLLRESET_SHIFT)) & FLEXSPI_DLLCR_DLLRESET_MASK)

#define FLEXSPI_DLLCR_SLVDLYTARGET_MASK          (0x78U)
#define FLEXSPI_DLLCR_SLVDLYTARGET_SHIFT         (3U)
/*! SLVDLYTARGET - Target Delay Line */
#define FLEXSPI_DLLCR_SLVDLYTARGET(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_DLLCR_SLVDLYTARGET_SHIFT)) & FLEXSPI_DLLCR_SLVDLYTARGET_MASK)

#define FLEXSPI_DLLCR_OVRDEN_MASK                (0x100U)
#define FLEXSPI_DLLCR_OVRDEN_SHIFT               (8U)
/*! OVRDEN - Target Clock Delay Line Override Value Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define FLEXSPI_DLLCR_OVRDEN(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_DLLCR_OVRDEN_SHIFT)) & FLEXSPI_DLLCR_OVRDEN_MASK)

#define FLEXSPI_DLLCR_OVRDVAL_MASK               (0x7E00U)
#define FLEXSPI_DLLCR_OVRDVAL_SHIFT              (9U)
/*! OVRDVAL - Target Clock Delay Line Override Value */
#define FLEXSPI_DLLCR_OVRDVAL(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_DLLCR_OVRDVAL_SHIFT)) & FLEXSPI_DLLCR_OVRDVAL_MASK)

#define FLEXSPI_DLLCR_REFPHASEGAP_MASK           (0x18000U)
#define FLEXSPI_DLLCR_REFPHASEGAP_SHIFT          (15U)
/*! REFPHASEGAP - Reference Clock Delay Line Phase Adjust Gap. REFPHASEGAP setting of 2h is recommended if DLLEN is set. */
#define FLEXSPI_DLLCR_REFPHASEGAP(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_DLLCR_REFPHASEGAP_SHIFT)) & FLEXSPI_DLLCR_REFPHASEGAP_MASK)
/*! @} */

/* The count of FLEXSPI_DLLCR */
#define FLEXSPI_DLLCR_COUNT                      (2U)

/*! @name STS0 - Status 0 */
/*! @{ */

#define FLEXSPI_STS0_SEQIDLE_MASK                (0x1U)
#define FLEXSPI_STS0_SEQIDLE_SHIFT               (0U)
/*! SEQIDLE - SEQ_CTL State Machine Idle
 *  0b0..Not idle
 *  0b1..Idle
 */
#define FLEXSPI_STS0_SEQIDLE(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS0_SEQIDLE_SHIFT)) & FLEXSPI_STS0_SEQIDLE_MASK)

#define FLEXSPI_STS0_ARBIDLE_MASK                (0x2U)
#define FLEXSPI_STS0_ARBIDLE_SHIFT               (1U)
/*! ARBIDLE - ARB_CTL State Machine Idle
 *  0b0..Not idle
 *  0b1..Idle
 */
#define FLEXSPI_STS0_ARBIDLE(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS0_ARBIDLE_SHIFT)) & FLEXSPI_STS0_ARBIDLE_MASK)

#define FLEXSPI_STS0_ARBCMDSRC_MASK              (0xCU)
#define FLEXSPI_STS0_ARBCMDSRC_SHIFT             (2U)
/*! ARBCMDSRC - ARB Command Source
 *  0b00..Trigger source is AHB read command.
 *  0b01..Trigger source is AHB write command.
 *  0b10..Trigger source is IP command (by writing 1 to IPCMD[TRG]).
 *  0b11..Trigger source is a suspended command that has resumed.
 */
#define FLEXSPI_STS0_ARBCMDSRC(x)                (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS0_ARBCMDSRC_SHIFT)) & FLEXSPI_STS0_ARBCMDSRC_MASK)
/*! @} */

/*! @name STS1 - Status 1 */
/*! @{ */

#define FLEXSPI_STS1_AHBCMDERRID_MASK            (0x1FU)
#define FLEXSPI_STS1_AHBCMDERRID_SHIFT           (0U)
/*! AHBCMDERRID - AHB Command Error ID */
#define FLEXSPI_STS1_AHBCMDERRID(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS1_AHBCMDERRID_SHIFT)) & FLEXSPI_STS1_AHBCMDERRID_MASK)

#define FLEXSPI_STS1_AHBCMDERRCODE_MASK          (0xF00U)
#define FLEXSPI_STS1_AHBCMDERRCODE_SHIFT         (8U)
/*! AHBCMDERRCODE - AHB Command Error Code
 *  0b0000..No error
 *  0b0010..AHB Write command with JMP_ON_CS instruction used in the sequence
 *  0b0011..Unknown instruction opcode in the sequence
 *  0b0100..DUMMY_SDR or DUMMY_RWDS_SDR instruction used in DDR sequence
 *  0b0101..DUMMY_DDR or DUMMY_RWDS_DDR instruction used in SDR sequence
 *  0b1110..Sequence execution timeout
 */
#define FLEXSPI_STS1_AHBCMDERRCODE(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS1_AHBCMDERRCODE_SHIFT)) & FLEXSPI_STS1_AHBCMDERRCODE_MASK)

#define FLEXSPI_STS1_IPCMDERRID_MASK             (0x1F0000U)
#define FLEXSPI_STS1_IPCMDERRID_SHIFT            (16U)
/*! IPCMDERRID - IP Command Error ID */
#define FLEXSPI_STS1_IPCMDERRID(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS1_IPCMDERRID_SHIFT)) & FLEXSPI_STS1_IPCMDERRID_MASK)

#define FLEXSPI_STS1_IPCMDERRCODE_MASK           (0xF000000U)
#define FLEXSPI_STS1_IPCMDERRCODE_SHIFT          (24U)
/*! IPCMDERRCODE - IP Command Error Code
 *  0b0000..No error
 *  0b0010..IP command with JMP_ON_CS instruction used in the sequence
 *  0b0011..Unknown instruction opcode in the sequence
 *  0b0100..DUMMY_SDR or DUMMY_RWDS_SDR instruction used in DDR sequence
 *  0b0101..DUMMY_DDR or DUMMY_RWDS_DDR instruction used in SDR sequence
 *  0b0110..Flash memory access start address exceeds entire flash address range (A1, A2, B1, and B2)
 *  0b1110..Sequence execution timeout
 *  0b1111..Flash boundary crossed
 */
#define FLEXSPI_STS1_IPCMDERRCODE(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS1_IPCMDERRCODE_SHIFT)) & FLEXSPI_STS1_IPCMDERRCODE_MASK)
/*! @} */

/*! @name STS2 - Status 2 */
/*! @{ */

#define FLEXSPI_STS2_ASLVLOCK_MASK               (0x1U)
#define FLEXSPI_STS2_ASLVLOCK_SHIFT              (0U)
/*! ASLVLOCK - Flash A Sample Target Delay Line Locked
 *  0b0..Not locked
 *  0b1..Locked
 */
#define FLEXSPI_STS2_ASLVLOCK(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_ASLVLOCK_SHIFT)) & FLEXSPI_STS2_ASLVLOCK_MASK)

#define FLEXSPI_STS2_AREFLOCK_MASK               (0x2U)
#define FLEXSPI_STS2_AREFLOCK_SHIFT              (1U)
/*! AREFLOCK - Flash A Sample Clock Reference Delay Line Locked
 *  0b0..Not locked
 *  0b1..Locked
 */
#define FLEXSPI_STS2_AREFLOCK(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_AREFLOCK_SHIFT)) & FLEXSPI_STS2_AREFLOCK_MASK)

#define FLEXSPI_STS2_ASLVSEL_MASK                (0xFCU)
#define FLEXSPI_STS2_ASLVSEL_SHIFT               (2U)
/*! ASLVSEL - Flash A Sample Clock Target Delay Line Delay Cell Number */
#define FLEXSPI_STS2_ASLVSEL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_ASLVSEL_SHIFT)) & FLEXSPI_STS2_ASLVSEL_MASK)

#define FLEXSPI_STS2_AREFSEL_MASK                (0x3F00U)
#define FLEXSPI_STS2_AREFSEL_SHIFT               (8U)
/*! AREFSEL - Flash A Sample Clock Reference Delay Line Delay Cell Number */
#define FLEXSPI_STS2_AREFSEL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_AREFSEL_SHIFT)) & FLEXSPI_STS2_AREFSEL_MASK)

#define FLEXSPI_STS2_BSLVLOCK_MASK               (0x10000U)
#define FLEXSPI_STS2_BSLVLOCK_SHIFT              (16U)
/*! BSLVLOCK - Flash B Sample Target Reference Delay Line Locked
 *  0b0..Not locked
 *  0b1..Locked
 */
#define FLEXSPI_STS2_BSLVLOCK(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_BSLVLOCK_SHIFT)) & FLEXSPI_STS2_BSLVLOCK_MASK)

#define FLEXSPI_STS2_BREFLOCK_MASK               (0x20000U)
#define FLEXSPI_STS2_BREFLOCK_SHIFT              (17U)
/*! BREFLOCK - Flash B Sample Clock Reference Delay Line Locked
 *  0b0..Not locked
 *  0b1..Locked
 */
#define FLEXSPI_STS2_BREFLOCK(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_BREFLOCK_SHIFT)) & FLEXSPI_STS2_BREFLOCK_MASK)

#define FLEXSPI_STS2_BSLVSEL_MASK                (0xFC0000U)
#define FLEXSPI_STS2_BSLVSEL_SHIFT               (18U)
/*! BSLVSEL - Flash B Sample Clock Target Delay Line Delay Cell Number */
#define FLEXSPI_STS2_BSLVSEL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_BSLVSEL_SHIFT)) & FLEXSPI_STS2_BSLVSEL_MASK)

#define FLEXSPI_STS2_BREFSEL_MASK                (0x3F000000U)
#define FLEXSPI_STS2_BREFSEL_SHIFT               (24U)
/*! BREFSEL - Flash B Sample Clock Reference Delay Line Delay Cell Number */
#define FLEXSPI_STS2_BREFSEL(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_STS2_BREFSEL_SHIFT)) & FLEXSPI_STS2_BREFSEL_MASK)
/*! @} */

/*! @name AHBSPNDSTS - AHB Suspend Status */
/*! @{ */

#define FLEXSPI_AHBSPNDSTS_ACTIVE_MASK           (0x1U)
#define FLEXSPI_AHBSPNDSTS_ACTIVE_SHIFT          (0U)
/*! ACTIVE - Active AHB Read Prefetch Suspended
 *  0b0..No suspended AHB read prefetch command.
 *  0b1..An AHB read prefetch command sequence has been suspended.
 */
#define FLEXSPI_AHBSPNDSTS_ACTIVE(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBSPNDSTS_ACTIVE_SHIFT)) & FLEXSPI_AHBSPNDSTS_ACTIVE_MASK)

#define FLEXSPI_AHBSPNDSTS_BUFID_MASK            (0xEU)
#define FLEXSPI_AHBSPNDSTS_BUFID_SHIFT           (1U)
/*! BUFID - AHB Receive Buffer ID for Suspended Command Sequence */
#define FLEXSPI_AHBSPNDSTS_BUFID(x)              (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBSPNDSTS_BUFID_SHIFT)) & FLEXSPI_AHBSPNDSTS_BUFID_MASK)

#define FLEXSPI_AHBSPNDSTS_DATLFT_MASK           (0xFFFF0000U)
#define FLEXSPI_AHBSPNDSTS_DATLFT_SHIFT          (16U)
/*! DATLFT - Data Left */
#define FLEXSPI_AHBSPNDSTS_DATLFT(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBSPNDSTS_DATLFT_SHIFT)) & FLEXSPI_AHBSPNDSTS_DATLFT_MASK)
/*! @} */

/*! @name IPRXFSTS - IP Receive FIFO Status */
/*! @{ */

#define FLEXSPI_IPRXFSTS_FILL_MASK               (0xFFU)
#define FLEXSPI_IPRXFSTS_FILL_SHIFT              (0U)
/*! FILL - Fill Level of IP Receive FIFO */
#define FLEXSPI_IPRXFSTS_FILL(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPRXFSTS_FILL_SHIFT)) & FLEXSPI_IPRXFSTS_FILL_MASK)

#define FLEXSPI_IPRXFSTS_RDCNTR_MASK             (0xFFFF0000U)
#define FLEXSPI_IPRXFSTS_RDCNTR_SHIFT            (16U)
/*! RDCNTR - Read Data Counter */
#define FLEXSPI_IPRXFSTS_RDCNTR(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPRXFSTS_RDCNTR_SHIFT)) & FLEXSPI_IPRXFSTS_RDCNTR_MASK)
/*! @} */

/*! @name IPTXFSTS - IP Transmit FIFO Status */
/*! @{ */

#define FLEXSPI_IPTXFSTS_FILL_MASK               (0xFFU)
#define FLEXSPI_IPTXFSTS_FILL_SHIFT              (0U)
/*! FILL - Fill Level of IP Transmit FIFO */
#define FLEXSPI_IPTXFSTS_FILL(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPTXFSTS_FILL_SHIFT)) & FLEXSPI_IPTXFSTS_FILL_MASK)

#define FLEXSPI_IPTXFSTS_WRCNTR_MASK             (0xFFFF0000U)
#define FLEXSPI_IPTXFSTS_WRCNTR_SHIFT            (16U)
/*! WRCNTR - Write Data Counter */
#define FLEXSPI_IPTXFSTS_WRCNTR(x)               (((uint32_t)(((uint32_t)(x)) << FLEXSPI_IPTXFSTS_WRCNTR_SHIFT)) & FLEXSPI_IPTXFSTS_WRCNTR_MASK)
/*! @} */

/*! @name RFDR - IP Receive FIFO Data 0..IP Receive FIFO Data 31 */
/*! @{ */

#define FLEXSPI_RFDR_RXDATA_MASK                 (0xFFFFFFFFU)
#define FLEXSPI_RFDR_RXDATA_SHIFT                (0U)
/*! RXDATA - Receive Data */
#define FLEXSPI_RFDR_RXDATA(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_RFDR_RXDATA_SHIFT)) & FLEXSPI_RFDR_RXDATA_MASK)
/*! @} */

/* The count of FLEXSPI_RFDR */
#define FLEXSPI_RFDR_COUNT                       (32U)

/*! @name TFDR - IP TX FIFO Data 0..IP TX FIFO Data 31 */
/*! @{ */

#define FLEXSPI_TFDR_TXDATA_MASK                 (0xFFFFFFFFU)
#define FLEXSPI_TFDR_TXDATA_SHIFT                (0U)
/*! TXDATA - Transmit Data */
#define FLEXSPI_TFDR_TXDATA(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_TFDR_TXDATA_SHIFT)) & FLEXSPI_TFDR_TXDATA_MASK)
/*! @} */

/* The count of FLEXSPI_TFDR */
#define FLEXSPI_TFDR_COUNT                       (32U)

/*! @name LUT - Lookup Table 0..Lookup Table 127 */
/*! @{ */

#define FLEXSPI_LUT_OPERAND0_MASK                (0xFFU)
#define FLEXSPI_LUT_OPERAND0_SHIFT               (0U)
/*! OPERAND0 - OPERAND0 */
#define FLEXSPI_LUT_OPERAND0(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUT_OPERAND0_SHIFT)) & FLEXSPI_LUT_OPERAND0_MASK)

#define FLEXSPI_LUT_NUM_PADS0_MASK               (0x300U)
#define FLEXSPI_LUT_NUM_PADS0_SHIFT              (8U)
/*! NUM_PADS0 - NUM_PADS0 */
#define FLEXSPI_LUT_NUM_PADS0(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUT_NUM_PADS0_SHIFT)) & FLEXSPI_LUT_NUM_PADS0_MASK)

#define FLEXSPI_LUT_OPCODE0_MASK                 (0xFC00U)
#define FLEXSPI_LUT_OPCODE0_SHIFT                (10U)
/*! OPCODE0 - OPCODE */
#define FLEXSPI_LUT_OPCODE0(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUT_OPCODE0_SHIFT)) & FLEXSPI_LUT_OPCODE0_MASK)

#define FLEXSPI_LUT_OPERAND1_MASK                (0xFF0000U)
#define FLEXSPI_LUT_OPERAND1_SHIFT               (16U)
/*! OPERAND1 - OPERAND1 */
#define FLEXSPI_LUT_OPERAND1(x)                  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUT_OPERAND1_SHIFT)) & FLEXSPI_LUT_OPERAND1_MASK)

#define FLEXSPI_LUT_NUM_PADS1_MASK               (0x3000000U)
#define FLEXSPI_LUT_NUM_PADS1_SHIFT              (24U)
/*! NUM_PADS1 - NUM_PADS1 */
#define FLEXSPI_LUT_NUM_PADS1(x)                 (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUT_NUM_PADS1_SHIFT)) & FLEXSPI_LUT_NUM_PADS1_MASK)

#define FLEXSPI_LUT_OPCODE1_MASK                 (0xFC000000U)
#define FLEXSPI_LUT_OPCODE1_SHIFT                (26U)
/*! OPCODE1 - OPCODE1 */
#define FLEXSPI_LUT_OPCODE1(x)                   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_LUT_OPCODE1_SHIFT)) & FLEXSPI_LUT_OPCODE1_MASK)
/*! @} */

/* The count of FLEXSPI_LUT */
#define FLEXSPI_LUT_COUNT                        (128U)

/*! @name AHBBUFREGIONSTART0 - Receive Buffer Start Address of Region 0 */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONSTART0_START_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONSTART0_START_ADDRESS_SHIFT (12U)
/*! START_ADDRESS - Start Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONSTART0_START_ADDRESS(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONSTART0_START_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONSTART0_START_ADDRESS_MASK)
/*! @} */

/*! @name AHBBUFREGIONEND0 - Receive Buffer Region 0 End Address */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONEND0_END_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONEND0_END_ADDRESS_SHIFT (12U)
/*! END_ADDRESS - End Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONEND0_END_ADDRESS(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONEND0_END_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONEND0_END_ADDRESS_MASK)
/*! @} */

/*! @name AHBBUFREGIONSTART1 - Receive Buffer Start Address of Region 1 */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONSTART1_START_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONSTART1_START_ADDRESS_SHIFT (12U)
/*! START_ADDRESS - Start Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONSTART1_START_ADDRESS(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONSTART1_START_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONSTART1_START_ADDRESS_MASK)
/*! @} */

/*! @name AHBBUFREGIONEND1 - Receive Buffer Region 1 End Address */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONEND1_END_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONEND1_END_ADDRESS_SHIFT (12U)
/*! END_ADDRESS - End Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONEND1_END_ADDRESS(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONEND1_END_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONEND1_END_ADDRESS_MASK)
/*! @} */

/*! @name AHBBUFREGIONSTART2 - Receive Buffer Start Address of Region 2 */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONSTART2_START_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONSTART2_START_ADDRESS_SHIFT (12U)
/*! START_ADDRESS - Start Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONSTART2_START_ADDRESS(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONSTART2_START_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONSTART2_START_ADDRESS_MASK)
/*! @} */

/*! @name AHBBUFREGIONEND2 - Receive Buffer Region 2 End Address */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONEND2_END_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONEND2_END_ADDRESS_SHIFT (12U)
/*! END_ADDRESS - End Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONEND2_END_ADDRESS(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONEND2_END_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONEND2_END_ADDRESS_MASK)
/*! @} */

/*! @name AHBBUFREGIONSTART3 - Receive Buffer Start Address of Region 3 */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONSTART3_START_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONSTART3_START_ADDRESS_SHIFT (12U)
/*! START_ADDRESS - Start Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONSTART3_START_ADDRESS(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONSTART3_START_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONSTART3_START_ADDRESS_MASK)
/*! @} */

/*! @name AHBBUFREGIONEND3 - Receive Buffer Region 3 End Address */
/*! @{ */

#define FLEXSPI_AHBBUFREGIONEND3_END_ADDRESS_MASK (0xFFFFF000U)
#define FLEXSPI_AHBBUFREGIONEND3_END_ADDRESS_SHIFT (12U)
/*! END_ADDRESS - End Address of Prefetch Sub-Buffer Region */
#define FLEXSPI_AHBBUFREGIONEND3_END_ADDRESS(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_AHBBUFREGIONEND3_END_ADDRESS_SHIFT)) & FLEXSPI_AHBBUFREGIONEND3_END_ADDRESS_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group FLEXSPI_Register_Masks */


/* FLEXSPI - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral FLEXSPI1 base address */
  #define FLEXSPI1_BASE                            (0x525E0000u)
  /** Peripheral FLEXSPI1 base address */
  #define FLEXSPI1_BASE_NS                         (0x425E0000u)
  /** Peripheral FLEXSPI1 base pointer */
  #define FLEXSPI1                                 ((FLEXSPI_Type *)FLEXSPI1_BASE)
  /** Peripheral FLEXSPI1 base pointer */
  #define FLEXSPI1_NS                              ((FLEXSPI_Type *)FLEXSPI1_BASE_NS)
  /** Peripheral FLEXSPI2 base address */
  #define FLEXSPI2_BASE                            (0x545E0000u)
  /** Peripheral FLEXSPI2 base address */
  #define FLEXSPI2_BASE_NS                         (0x445E0000u)
  /** Peripheral FLEXSPI2 base pointer */
  #define FLEXSPI2                                 ((FLEXSPI_Type *)FLEXSPI2_BASE)
  /** Peripheral FLEXSPI2 base pointer */
  #define FLEXSPI2_NS                              ((FLEXSPI_Type *)FLEXSPI2_BASE_NS)
  /** Array initializer of FLEXSPI peripheral base addresses */
  #define FLEXSPI_BASE_ADDRS                       { 0u, FLEXSPI1_BASE, FLEXSPI2_BASE }
  /** Array initializer of FLEXSPI peripheral base pointers */
  #define FLEXSPI_BASE_PTRS                        { (FLEXSPI_Type *)0u, FLEXSPI1, FLEXSPI2 }
  /** Array initializer of FLEXSPI peripheral base addresses */
  #define FLEXSPI_BASE_ADDRS_NS                    { 0u, FLEXSPI1_BASE_NS, FLEXSPI2_BASE_NS }
  /** Array initializer of FLEXSPI peripheral base pointers */
  #define FLEXSPI_BASE_PTRS_NS                     { (FLEXSPI_Type *)0u, FLEXSPI1_NS, FLEXSPI2_NS }
#else
  /** Peripheral FLEXSPI1 base address */
  #define FLEXSPI1_BASE                            (0x425E0000u)
  /** Peripheral FLEXSPI1 base pointer */
  #define FLEXSPI1                                 ((FLEXSPI_Type *)FLEXSPI1_BASE)
  /** Peripheral FLEXSPI2 base address */
  #define FLEXSPI2_BASE                            (0x445E0000u)
  /** Peripheral FLEXSPI2 base pointer */
  #define FLEXSPI2                                 ((FLEXSPI_Type *)FLEXSPI2_BASE)
  /** Array initializer of FLEXSPI peripheral base addresses */
  #define FLEXSPI_BASE_ADDRS                       { 0u, FLEXSPI1_BASE, FLEXSPI2_BASE }
  /** Array initializer of FLEXSPI peripheral base pointers */
  #define FLEXSPI_BASE_PTRS                        { (FLEXSPI_Type *)0u, FLEXSPI1, FLEXSPI2 }
#endif
/** Interrupt vectors for the FLEXSPI peripheral type */
#define FLEXSPI_IRQS                             { NotAvail_IRQn, FLEXSPI1_IRQn, FLEXSPI2_IRQn }
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /* FlexSPI1 AMBA base address. */
  #define FlexSPI1_AMBA_BASE                       (0x38000000U)
  /* FlexSPI1 AMBA end address. */
  #define FlexSPI1_AMBA_END                        (0x3FFFFFFFU)
  /* FlexSPI1 ASFM address. */
  #define FlexSPI1_ASFM_BASE                       (0x38000000U)
  /* Base Address of AHB address space mapped to IP RX FIFO. */
  #define FlexSPI1_ARDF_BASE                       (0x57420000U)
  /* Base Address of AHB address space mapped to IP TX FIFO. */
  #define FlexSPI1_ATDF_BASE                       (0x57430000U)
  /* FlexSPI1 alias base address. */
  #define FlexSPI1_ALIAS_BASE                      (0x12000000U)
  /* FlexSPI2 AMBA base address. */
  #define FlexSPI2_AMBA_BASE                       (0x14000000U)
  /* FlexSPI2 AMBA end address. */
  #define FlexSPI2_AMBA_END                        (0x17FFFFFFU)
  /* FlexSPI ASFM address. */
  #define FlexSPI2_ASFM_BASE                       (0x14000000U)
  /* Base Address of AHB address space mapped to IP RX FIFO. */
  #define FlexSPI2_ARDF_BASE                       (0x5DE00000U)
  /* Base Address of AHB address space mapped to IP TX FIFO. */
  #define FlexSPI2_ATDF_BASE                       (0x5DE10000U)
  /* FlexSPI2 alias base address. */
  #define FlexSPI2_ALIAS_BASE                      (0x22000000U)
#else
  /* FlexSPI1 AMBA base address. */
  #define FlexSPI1_AMBA_BASE                       (0x38000000U)
  #define FlexSPI1_AMBA_BASE_NS                    (0x28000000U)
  /* FlexSPI1 AMBA end address. */
  #define FlexSPI1_AMBA_END                        (0x3FFFFFFFU)
  #define FlexSPI1_AMBA_END_NS                     (0x2FFFFFFFU)
  /* FlexSPI1 ASFM address. */
  #define FlexSPI1_ASFM_BASE                       (0x38000000U)
  #define FlexSPI1_ASFM_BASE_NS                    (0x28000000U)
  /* Base Address of AHB address space mapped to IP RX FIFO. */
  #define FlexSPI1_ARDF_BASE                       (0x57420000U)
  #define FlexSPI1_ARDF_BASE_NS                    (0x47420000U)
  /* Base Address of AHB address space mapped to IP TX FIFO. */
  #define FlexSPI1_ATDF_BASE                       (0x57430000U)
  #define FlexSPI1_ATDF_BASE_NS                    (0x47430000U)
  /* FlexSPI1 alias base address. */
  #define FlexSPI1_ALIAS_BASE                      (0x12000000U)
  /* FlexSPI2 AMBA base address. */
  #define FlexSPI2_AMBA_BASE                       (0x14000000U)
  #define FlexSPI2_AMBA_BASE_NS                    (0x4000000U)
  /* FlexSPI2 AMBA end address. */
  #define FlexSPI2_AMBA_END                        (0x17FFFFFFU)
  #define FlexSPI2_AMBA_END_NS                     (0x7FFFFFFU)
  /* FlexSPI ASFM address. */
  #define FlexSPI2_ASFM_BASE                       (0x14000000U)
  #define FlexSPI2_ASFM_BASE_NS                    (0x4000000U)
  /* Base Address of AHB address space mapped to IP RX FIFO. */
  #define FlexSPI2_ARDF_BASE                       (0x5DE00000U)
  #define FlexSPI2_ARDF_BASE_NS                    (0x4DE00000U)
  /* Base Address of AHB address space mapped to IP TX FIFO. */
  #define FlexSPI2_ATDF_BASE                       (0x5DE10000U)
  #define FlexSPI2_ATDF_BASE_NS                    (0x4DE10000U)
  /* FlexSPI2 alias base address. */
  #define FlexSPI2_ALIAS_BASE                      (0x22000000U)
#endif


/*!
 * @}
 */ /* end of group FLEXSPI_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- FLEXSPI_SLV Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup FLEXSPI_SLV_Peripheral_Access_Layer FLEXSPI_SLV Peripheral Access Layer
 * @{
 */

/** FLEXSPI_SLV - Register Layout Typedef */
typedef struct {
  __IO uint32_t MODULE_CONTROL;                    /**< Module Control, offset: 0x0 */
  __IO uint32_t READ_COMMAND_CONTROL;              /**< Read Command Control, offset: 0x4 */
  __IO uint32_t READ_REGISTER_COMMAND0;            /**< Read Register Command Setting, offset: 0x8 */
  __IO uint32_t READ_COMMAND[2];                   /**< Read Command 1 setting..Read Command 2 setting, array offset: 0xC, array step: 0x4 */
  __IO uint32_t WRITE_COMMAND_CONTROL;             /**< Write Command Control, offset: 0x14 */
  __IO uint32_t WRITE_REGISTER_COMMAND0;           /**< Write Register Command 0 Setting, offset: 0x18 */
  __IO uint32_t WRITE_COMMAND[2];                  /**< Write Command 1 Setting..Write Command 2 Setting, array offset: 0x1C, array step: 0x4 */
  __IO uint32_t RW_COMMAND_BASE;                   /**< Read Write Command Address Base, offset: 0x24 */
  __IO uint32_t CMD_RANGE[2];                      /**< Command Suit 1 Range..Command Suit 2 Range, array offset: 0x28, array step: 0x4 */
  __I  uint32_t MODULE_STATUS;                     /**< Module Status, offset: 0x30 */
  __IO uint32_t MODULE_INT;                        /**< SPI FLR interrupt, offset: 0x34 */
  __IO uint32_t MODULE_INTEN;                      /**< SPI FLR Interrupt Enable, offset: 0x38 */
  __IO uint32_t SPI_MAIL_CTRL;                     /**< SPI Mailbox control, offset: 0x3C */
  __I  uint32_t SPIMAIL[9];                        /**< SPI Mail Interrupt, array offset: 0x40, array step: 0x4 */
} FLEXSPI_SLV_Type;

/* ----------------------------------------------------------------------------
   -- FLEXSPI_SLV Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup FLEXSPI_SLV_Register_Masks FLEXSPI_SLV Register Masks
 * @{
 */

/*! @name MODULE_CONTROL - Module Control */
/*! @{ */

#define FLEXSPI_SLV_MODULE_CONTROL_SWRESET_MASK  (0x1U)
#define FLEXSPI_SLV_MODULE_CONTROL_SWRESET_SHIFT (0U)
/*! SWRESET - Software Reset
 *  0b1..Initiate
 *  0b0..Finished
 */
#define FLEXSPI_SLV_MODULE_CONTROL_SWRESET(x)    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_SWRESET_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_SWRESET_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_IOMODE_MASK   (0x6U)
#define FLEXSPI_SLV_MODULE_CONTROL_IOMODE_SHIFT  (1U)
/*! IOMODE - SPI IO Mode Control
 *  0b00..SDR*4
 *  0b01..SDR*8
 *  0b10..DDR*4
 *  0b11..DDR*8
 */
#define FLEXSPI_SLV_MODULE_CONTROL_IOMODE(x)     (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_IOMODE_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_IOMODE_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_DQSSTOP_MASK  (0x8U)
#define FLEXSPI_SLV_MODULE_CONTROL_DQSSTOP_SHIFT (3U)
/*! DQSSTOP - DQS Stop Feature
 *  0b1..Enable
 *  0b0..Disable
 */
#define FLEXSPI_SLV_MODULE_CONTROL_DQSSTOP(x)    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_DQSSTOP_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_DQSSTOP_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_CSMASK_MASK   (0x10U)
#define FLEXSPI_SLV_MODULE_CONTROL_CSMASK_SHIFT  (4U)
/*! CSMASK - Chip Select Mask
 *  0b1..Masked
 *  0b0..Not masked
 */
#define FLEXSPI_SLV_MODULE_CONTROL_CSMASK(x)     (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_CSMASK_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_CSMASK_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_BLKREAD_MASK  (0x20U)
#define FLEXSPI_SLV_MODULE_CONTROL_BLKREAD_SHIFT (5U)
/*! BLKREAD - Block Read
 *  0b1..Blocked
 *  0b0..Allowed
 */
#define FLEXSPI_SLV_MODULE_CONTROL_BLKREAD(x)    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_BLKREAD_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_BLKREAD_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_BLKWRITE_MASK (0x40U)
#define FLEXSPI_SLV_MODULE_CONTROL_BLKWRITE_SHIFT (6U)
/*! BLKWRITE - Block Write
 *  0b1..Blocked
 *  0b0..Allowed
 */
#define FLEXSPI_SLV_MODULE_CONTROL_BLKWRITE(x)   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_BLKWRITE_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_BLKWRITE_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_BLKNXTWR_MASK (0x80U)
#define FLEXSPI_SLV_MODULE_CONTROL_BLKNXTWR_SHIFT (7U)
/*! BLKNXTWR - Block Next Write Command
 *  0b1..Blocked
 *  0b0..Allowed
 */
#define FLEXSPI_SLV_MODULE_CONTROL_BLKNXTWR(x)   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_BLKNXTWR_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_BLKNXTWR_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_BLKNXTRD_MASK (0x100U)
#define FLEXSPI_SLV_MODULE_CONTROL_BLKNXTRD_SHIFT (8U)
/*! BLKNXTRD - Block Next Read
 *  0b1..Blocked
 *  0b0..Allowed
 */
#define FLEXSPI_SLV_MODULE_CONTROL_BLKNXTRD(x)   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_BLKNXTRD_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_BLKNXTRD_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_ALLOWONEWR_MASK (0x200U)
#define FLEXSPI_SLV_MODULE_CONTROL_ALLOWONEWR_SHIFT (9U)
/*! ALLOWONEWR - Allow One More Write
 *  0b1..Allowed
 *  0b0..Not allowed
 */
#define FLEXSPI_SLV_MODULE_CONTROL_ALLOWONEWR(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_ALLOWONEWR_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_ALLOWONEWR_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_ALLOWONERD_MASK (0x400U)
#define FLEXSPI_SLV_MODULE_CONTROL_ALLOWONERD_SHIFT (10U)
/*! ALLOWONERD - Allow One More Read
 *  0b1..Allowed
 *  0b0..Not allowed
 */
#define FLEXSPI_SLV_MODULE_CONTROL_ALLOWONERD(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_ALLOWONERD_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_ALLOWONERD_MASK)

#define FLEXSPI_SLV_MODULE_CONTROL_CMDRANGEBASEUPDATE_MASK (0x800U)
#define FLEXSPI_SLV_MODULE_CONTROL_CMDRANGEBASEUPDATE_SHIFT (11U)
/*! CMDRANGEBASEUPDATE - AXI Command Range Base Update
 *  0b1..Updated
 *  0b0..Not updated
 */
#define FLEXSPI_SLV_MODULE_CONTROL_CMDRANGEBASEUPDATE(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_CONTROL_CMDRANGEBASEUPDATE_SHIFT)) & FLEXSPI_SLV_MODULE_CONTROL_CMDRANGEBASEUPDATE_MASK)
/*! @} */

/*! @name READ_COMMAND_CONTROL - Read Command Control */
/*! @{ */

#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDFETCHSIZE_MASK (0x3U)
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDFETCHSIZE_SHIFT (0U)
/*! RDFETCHSIZE - Read Fetch Size
 *  0b00..256 bytes
 *  0b01..512 bytes
 *  0b10..1 KB
 *  0b11..2 KB
 */
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDFETCHSIZE(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_COMMAND_CONTROL_RDFETCHSIZE_SHIFT)) & FLEXSPI_SLV_READ_COMMAND_CONTROL_RDFETCHSIZE_MASK)

#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDWM_MASK (0xFCU)
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDWM_SHIFT (2U)
/*! RDWM - Read Watermark Level */
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDWM(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_COMMAND_CONTROL_RDWM_SHIFT)) & FLEXSPI_SLV_READ_COMMAND_CONTROL_RDWM_MASK)

#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDOT_MASK (0x100U)
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDOT_SHIFT (8U)
/*! RDOT - Read Outstanding
 *  0b1..Send requests outstandingly
 *  0b0..Send requests after previous leaders finish
 */
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_RDOT(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_COMMAND_CONTROL_RDOT_SHIFT)) & FLEXSPI_SLV_READ_COMMAND_CONTROL_RDOT_MASK)

#define FLEXSPI_SLV_READ_COMMAND_CONTROL_WMEN_MASK (0x200U)
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_WMEN_SHIFT (9U)
/*! WMEN - Read Water Mark Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define FLEXSPI_SLV_READ_COMMAND_CONTROL_WMEN(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_COMMAND_CONTROL_WMEN_SHIFT)) & FLEXSPI_SLV_READ_COMMAND_CONTROL_WMEN_MASK)
/*! @} */

/*! @name READ_REGISTER_COMMAND0 - Read Register Command Setting */
/*! @{ */

#define FLEXSPI_SLV_READ_REGISTER_COMMAND0_DUMMYCYCLES_MASK (0xFFFFU)
#define FLEXSPI_SLV_READ_REGISTER_COMMAND0_DUMMYCYCLES_SHIFT (0U)
/*! DUMMYCYCLES - Read Register Dummy Cycles */
#define FLEXSPI_SLV_READ_REGISTER_COMMAND0_DUMMYCYCLES(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_REGISTER_COMMAND0_DUMMYCYCLES_SHIFT)) & FLEXSPI_SLV_READ_REGISTER_COMMAND0_DUMMYCYCLES_MASK)

#define FLEXSPI_SLV_READ_REGISTER_COMMAND0_COMMANDSET_MASK (0xFFFF0000U)
#define FLEXSPI_SLV_READ_REGISTER_COMMAND0_COMMANDSET_SHIFT (16U)
/*! COMMANDSET - Read Register Command Setting */
#define FLEXSPI_SLV_READ_REGISTER_COMMAND0_COMMANDSET(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_REGISTER_COMMAND0_COMMANDSET_SHIFT)) & FLEXSPI_SLV_READ_REGISTER_COMMAND0_COMMANDSET_MASK)
/*! @} */

/*! @name READ_COMMAND - Read Command 1 setting..Read Command 2 setting */
/*! @{ */

#define FLEXSPI_SLV_READ_COMMAND_DUMMYCYCLES_MASK (0xFFFFU)
#define FLEXSPI_SLV_READ_COMMAND_DUMMYCYCLES_SHIFT (0U)
/*! DUMMYCYCLES - Read Command 2 Dummy Cycles */
#define FLEXSPI_SLV_READ_COMMAND_DUMMYCYCLES(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_COMMAND_DUMMYCYCLES_SHIFT)) & FLEXSPI_SLV_READ_COMMAND_DUMMYCYCLES_MASK)

#define FLEXSPI_SLV_READ_COMMAND_COMMANDSET_MASK (0xFFFF0000U)
#define FLEXSPI_SLV_READ_COMMAND_COMMANDSET_SHIFT (16U)
/*! COMMANDSET - Read Command 2 Setting */
#define FLEXSPI_SLV_READ_COMMAND_COMMANDSET(x)   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_READ_COMMAND_COMMANDSET_SHIFT)) & FLEXSPI_SLV_READ_COMMAND_COMMANDSET_MASK)
/*! @} */

/* The count of FLEXSPI_SLV_READ_COMMAND */
#define FLEXSPI_SLV_READ_COMMAND_COUNT           (2U)

/*! @name WRITE_COMMAND_CONTROL - Write Command Control */
/*! @{ */

#define FLEXSPI_SLV_WRITE_COMMAND_CONTROL_WRWM_MASK (0x3U)
#define FLEXSPI_SLV_WRITE_COMMAND_CONTROL_WRWM_SHIFT (0U)
/*! WRWM - Write Watermark
 *  0b00..32 bytes
 *  0b01..64 bytes
 *  0b10..128 bytes
 *  0b11..256 bytes
 */
#define FLEXSPI_SLV_WRITE_COMMAND_CONTROL_WRWM(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_WRITE_COMMAND_CONTROL_WRWM_SHIFT)) & FLEXSPI_SLV_WRITE_COMMAND_CONTROL_WRWM_MASK)
/*! @} */

/*! @name WRITE_REGISTER_COMMAND0 - Write Register Command 0 Setting */
/*! @{ */

#define FLEXSPI_SLV_WRITE_REGISTER_COMMAND0_COMMANDSET_MASK (0xFFFF0000U)
#define FLEXSPI_SLV_WRITE_REGISTER_COMMAND0_COMMANDSET_SHIFT (16U)
/*! COMMANDSET - Write Register Command Setting */
#define FLEXSPI_SLV_WRITE_REGISTER_COMMAND0_COMMANDSET(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_WRITE_REGISTER_COMMAND0_COMMANDSET_SHIFT)) & FLEXSPI_SLV_WRITE_REGISTER_COMMAND0_COMMANDSET_MASK)
/*! @} */

/*! @name WRITE_COMMAND - Write Command 1 Setting..Write Command 2 Setting */
/*! @{ */

#define FLEXSPI_SLV_WRITE_COMMAND_COMMANDSET_MASK (0xFFFF0000U)
#define FLEXSPI_SLV_WRITE_COMMAND_COMMANDSET_SHIFT (16U)
/*! COMMANDSET - Write Command 2 Setting */
#define FLEXSPI_SLV_WRITE_COMMAND_COMMANDSET(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_WRITE_COMMAND_COMMANDSET_SHIFT)) & FLEXSPI_SLV_WRITE_COMMAND_COMMANDSET_MASK)
/*! @} */

/* The count of FLEXSPI_SLV_WRITE_COMMAND */
#define FLEXSPI_SLV_WRITE_COMMAND_COUNT          (2U)

/*! @name RW_COMMAND_BASE - Read Write Command Address Base */
/*! @{ */

#define FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE1_MASK (0xFFFFU)
#define FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE1_SHIFT (0U)
/*! ADDRBASE1 - Address Base 1 */
#define FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE1(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE1_SHIFT)) & FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE1_MASK)

#define FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE2_MASK (0xFFFF0000U)
#define FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE2_SHIFT (16U)
/*! ADDRBASE2 - Address Base 2 */
#define FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE2(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE2_SHIFT)) & FLEXSPI_SLV_RW_COMMAND_BASE_ADDRBASE2_MASK)
/*! @} */

/*! @name CMD_RANGE - Command Suit 1 Range..Command Suit 2 Range */
/*! @{ */

#define FLEXSPI_SLV_CMD_RANGE_RANGE_MASK         (0xFFFFFC00U)
#define FLEXSPI_SLV_CMD_RANGE_RANGE_SHIFT        (10U)
/*! RANGE - Memory Range */
#define FLEXSPI_SLV_CMD_RANGE_RANGE(x)           (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_CMD_RANGE_RANGE_SHIFT)) & FLEXSPI_SLV_CMD_RANGE_RANGE_MASK)
/*! @} */

/* The count of FLEXSPI_SLV_CMD_RANGE */
#define FLEXSPI_SLV_CMD_RANGE_COUNT              (2U)

/*! @name MODULE_STATUS - Module Status */
/*! @{ */

#define FLEXSPI_SLV_MODULE_STATUS_WIP_MASK       (0x1U)
#define FLEXSPI_SLV_MODULE_STATUS_WIP_SHIFT      (0U)
/*! WIP - Write in Progress
 *  0b1..Busy
 *  0b0..Not busy
 */
#define FLEXSPI_SLV_MODULE_STATUS_WIP(x)         (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_WIP_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_WIP_MASK)

#define FLEXSPI_SLV_MODULE_STATUS_AXIREADIDLE_MASK (0x2U)
#define FLEXSPI_SLV_MODULE_STATUS_AXIREADIDLE_SHIFT (1U)
/*! AXIREADIDLE - AXI Read Leader Idle
 *  0b1..Idle
 *  0b0..Busy
 */
#define FLEXSPI_SLV_MODULE_STATUS_AXIREADIDLE(x) (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_AXIREADIDLE_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_AXIREADIDLE_MASK)

#define FLEXSPI_SLV_MODULE_STATUS_REGRWIDLE_MASK (0x4U)
#define FLEXSPI_SLV_MODULE_STATUS_REGRWIDLE_SHIFT (2U)
/*! REGRWIDLE - Register Read Write Idle
 *  0b1..Idle
 *  0b0..Busy
 */
#define FLEXSPI_SLV_MODULE_STATUS_REGRWIDLE(x)   (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_REGRWIDLE_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_REGRWIDLE_MASK)

#define FLEXSPI_SLV_MODULE_STATUS_SEQIDLE_MASK   (0x8U)
#define FLEXSPI_SLV_MODULE_STATUS_SEQIDLE_SHIFT  (3U)
/*! SEQIDLE - SEQ Controller Idle
 *  0b1..Idle
 *  0b0..Busy
 */
#define FLEXSPI_SLV_MODULE_STATUS_SEQIDLE(x)     (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_SEQIDLE_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_SEQIDLE_MASK)

#define FLEXSPI_SLV_MODULE_STATUS_RDOFR_MASK     (0xF0U)
#define FLEXSPI_SLV_MODULE_STATUS_RDOFR_SHIFT    (4U)
/*! RDOFR - Read Out-of-Range Counter */
#define FLEXSPI_SLV_MODULE_STATUS_RDOFR(x)       (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_RDOFR_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_RDOFR_MASK)

#define FLEXSPI_SLV_MODULE_STATUS_WROFR_MASK     (0xF00U)
#define FLEXSPI_SLV_MODULE_STATUS_WROFR_SHIFT    (8U)
/*! WROFR - Write Out-of-Range Counter */
#define FLEXSPI_SLV_MODULE_STATUS_WROFR(x)       (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_WROFR_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_WROFR_MASK)

#define FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIRD_MASK (0x1000U)
#define FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIRD_SHIFT (12U)
/*! ALLOWAXIRD - Allow AXI Read Access
 *  0b1..Allowed
 *  0b0..Denied
 */
#define FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIRD(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIRD_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIRD_MASK)

#define FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIWR_MASK (0x2000U)
#define FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIWR_SHIFT (13U)
/*! ALLOWAXIWR - Allow AXI Write Access
 *  0b1..Allowed
 *  0b0..Denied
 */
#define FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIWR(x)  (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIWR_SHIFT)) & FLEXSPI_SLV_MODULE_STATUS_ALLOWAXIWR_MASK)
/*! @} */

/*! @name MODULE_INT - SPI FLR interrupt */
/*! @{ */

#define FLEXSPI_SLV_MODULE_INT_WOF_MASK          (0x1U)
#define FLEXSPI_SLV_MODULE_INT_WOF_SHIFT         (0U)
/*! WOF - Write Overflow Interrupt
 *  0b1..Occurred
 *  0b0..Did not occur
 */
#define FLEXSPI_SLV_MODULE_INT_WOF(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_INT_WOF_SHIFT)) & FLEXSPI_SLV_MODULE_INT_WOF_MASK)

#define FLEXSPI_SLV_MODULE_INT_RUF_MASK          (0x2U)
#define FLEXSPI_SLV_MODULE_INT_RUF_SHIFT         (1U)
/*! RUF - Read Underflow
 *  0b1..Occurred
 *  0b0..Did not occur
 */
#define FLEXSPI_SLV_MODULE_INT_RUF(x)            (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_INT_RUF_SHIFT)) & FLEXSPI_SLV_MODULE_INT_RUF_MASK)

#define FLEXSPI_SLV_MODULE_INT_ERRCMD_MASK       (0x4U)
#define FLEXSPI_SLV_MODULE_INT_ERRCMD_SHIFT      (2U)
/*! ERRCMD - Error Command
 *  0b1..Received
 *  0b0..Not received
 */
#define FLEXSPI_SLV_MODULE_INT_ERRCMD(x)         (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_INT_ERRCMD_SHIFT)) & FLEXSPI_SLV_MODULE_INT_ERRCMD_MASK)
/*! @} */

/*! @name MODULE_INTEN - SPI FLR Interrupt Enable */
/*! @{ */

#define FLEXSPI_SLV_MODULE_INTEN_WOFEN_MASK      (0x1U)
#define FLEXSPI_SLV_MODULE_INTEN_WOFEN_SHIFT     (0U)
/*! WOFEN - Write Overflow Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define FLEXSPI_SLV_MODULE_INTEN_WOFEN(x)        (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_INTEN_WOFEN_SHIFT)) & FLEXSPI_SLV_MODULE_INTEN_WOFEN_MASK)

#define FLEXSPI_SLV_MODULE_INTEN_RUFEN_MASK      (0x2U)
#define FLEXSPI_SLV_MODULE_INTEN_RUFEN_SHIFT     (1U)
/*! RUFEN - Read Underflow Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define FLEXSPI_SLV_MODULE_INTEN_RUFEN(x)        (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_INTEN_RUFEN_SHIFT)) & FLEXSPI_SLV_MODULE_INTEN_RUFEN_MASK)

#define FLEXSPI_SLV_MODULE_INTEN_ERRCMDEN_MASK   (0x4U)
#define FLEXSPI_SLV_MODULE_INTEN_ERRCMDEN_SHIFT  (2U)
/*! ERRCMDEN - Error Command Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define FLEXSPI_SLV_MODULE_INTEN_ERRCMDEN(x)     (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_MODULE_INTEN_ERRCMDEN_SHIFT)) & FLEXSPI_SLV_MODULE_INTEN_ERRCMDEN_MASK)
/*! @} */

/*! @name SPI_MAIL_CTRL - SPI Mailbox control */
/*! @{ */

#define FLEXSPI_SLV_SPI_MAIL_CTRL_CLRINT_MASK    (0x1U)
#define FLEXSPI_SLV_SPI_MAIL_CTRL_CLRINT_SHIFT   (0U)
/*! CLRINT - Clear Interrupt
 *  0b1..Clear
 *  0b0..Do not clear
 */
#define FLEXSPI_SLV_SPI_MAIL_CTRL_CLRINT(x)      (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_SPI_MAIL_CTRL_CLRINT_SHIFT)) & FLEXSPI_SLV_SPI_MAIL_CTRL_CLRINT_MASK)

#define FLEXSPI_SLV_SPI_MAIL_CTRL_SPIINTEN_MASK  (0x2U)
#define FLEXSPI_SLV_SPI_MAIL_CTRL_SPIINTEN_SHIFT (1U)
/*! SPIINTEN - SPI Leader Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define FLEXSPI_SLV_SPI_MAIL_CTRL_SPIINTEN(x)    (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_SPI_MAIL_CTRL_SPIINTEN_SHIFT)) & FLEXSPI_SLV_SPI_MAIL_CTRL_SPIINTEN_MASK)
/*! @} */

/*! @name SPIMAIL - SPI Mail Interrupt */
/*! @{ */

#define FLEXSPI_SLV_SPIMAIL_MAILn_MASK           (0xFFFFFFFFU)
#define FLEXSPI_SLV_SPIMAIL_MAILn_SHIFT          (0U)
#define FLEXSPI_SLV_SPIMAIL_MAILn(x)             (((uint32_t)(((uint32_t)(x)) << FLEXSPI_SLV_SPIMAIL_MAILn_SHIFT)) & FLEXSPI_SLV_SPIMAIL_MAILn_MASK)
/*! @} */

/* The count of FLEXSPI_SLV_SPIMAIL */
#define FLEXSPI_SLV_SPIMAIL_COUNT                (9U)


/*!
 * @}
 */ /* end of group FLEXSPI_SLV_Register_Masks */


/* FLEXSPI_SLV - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral FLEXSPI_SLV base address */
  #define FLEXSPI_SLV_BASE                         (0x52900000u)
  /** Peripheral FLEXSPI_SLV base address */
  #define FLEXSPI_SLV_BASE_NS                      (0x42900000u)
  /** Peripheral FLEXSPI_SLV base pointer */
  #define FLEXSPI_SLV                              ((FLEXSPI_SLV_Type *)FLEXSPI_SLV_BASE)
  /** Peripheral FLEXSPI_SLV base pointer */
  #define FLEXSPI_SLV_NS                           ((FLEXSPI_SLV_Type *)FLEXSPI_SLV_BASE_NS)
  /** Array initializer of FLEXSPI_SLV peripheral base addresses */
  #define FLEXSPI_SLV_BASE_ADDRS                   { FLEXSPI_SLV_BASE }
  /** Array initializer of FLEXSPI_SLV peripheral base pointers */
  #define FLEXSPI_SLV_BASE_PTRS                    { FLEXSPI_SLV }
  /** Array initializer of FLEXSPI_SLV peripheral base addresses */
  #define FLEXSPI_SLV_BASE_ADDRS_NS                { FLEXSPI_SLV_BASE_NS }
  /** Array initializer of FLEXSPI_SLV peripheral base pointers */
  #define FLEXSPI_SLV_BASE_PTRS_NS                 { FLEXSPI_SLV_NS }
#else
  /** Peripheral FLEXSPI_SLV base address */
  #define FLEXSPI_SLV_BASE                         (0x42900000u)
  /** Peripheral FLEXSPI_SLV base pointer */
  #define FLEXSPI_SLV                              ((FLEXSPI_SLV_Type *)FLEXSPI_SLV_BASE)
  /** Array initializer of FLEXSPI_SLV peripheral base addresses */
  #define FLEXSPI_SLV_BASE_ADDRS                   { FLEXSPI_SLV_BASE }
  /** Array initializer of FLEXSPI_SLV peripheral base pointers */
  #define FLEXSPI_SLV_BASE_PTRS                    { FLEXSPI_SLV }
#endif

/*!
 * @}
 */ /* end of group FLEXSPI_SLV_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- GPC_CPU_CTRL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPC_CPU_CTRL_Peripheral_Access_Layer GPC_CPU_CTRL Peripheral Access Layer
 * @{
 */

/** GPC_CPU_CTRL - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x800 */
         uint8_t RESERVED_0[4];
    __IO uint32_t CM_AUTHEN_CTRL;                    /**< CM Authentication Control, array offset: 0x4, array step: 0x800 */
         uint8_t RESERVED_1[4];
    __IO uint32_t CM_MISC;                           /**< Miscellaneous, array offset: 0xC, array step: 0x800 */
    __IO uint32_t CM_MODE_CTRL;                      /**< CPU mode control, array offset: 0x10, array step: 0x800 */
    __I  uint32_t CM_MODE_STAT;                      /**< CM CPU mode Status, array offset: 0x14, array step: 0x800 */
         uint8_t RESERVED_2[232];
    __IO uint32_t CM_IRQ_WAKEUP_MASK[8];             /**< CM IRQ0~31 wakeup mask..CM IRQ224~255 wakeup mask, array offset: 0x100, array step: index*0x800, index2*0x4 */
         uint8_t RESERVED_3[32];
    __IO uint32_t CM_NON_IRQ_WAKEUP_MASK;            /**< CM non-IRQ wakeup mask, array offset: 0x140, array step: 0x800 */
         uint8_t RESERVED_4[12];
    __I  uint32_t CM_IRQ_WAKEUP_STAT[8];             /**< CM IRQ0~31 wakeup status..CM IRQ224~255 wakeup status, array offset: 0x150, array step: index*0x800, index2*0x4 */
         uint8_t RESERVED_5[32];
    __I  uint32_t CM_NON_IRQ_WAKEUP_STAT;            /**< CM non-irq wakeup status, array offset: 0x190, array step: 0x800 */
         uint8_t RESERVED_6[108];
    __IO uint32_t CM_SLEEP_SSAR_CTRL;                /**< CM sleep SSAR control, array offset: 0x200, array step: 0x800 */
         uint8_t RESERVED_7[4];
    __IO uint32_t CM_SLEEP_LPCG_CTRL;                /**< CM sleep LPCG control, array offset: 0x208, array step: 0x800 */
         uint8_t RESERVED_8[4];
    __IO uint32_t CM_SLEEP_PLL_CTRL;                 /**< CM sleep PLL control, array offset: 0x210, array step: 0x800 */
         uint8_t RESERVED_9[4];
    __IO uint32_t CM_SLEEP_ISO_CTRL;                 /**< CM sleep isolation control, array offset: 0x218, array step: 0x800 */
         uint8_t RESERVED_10[4];
    __IO uint32_t CM_SLEEP_RESET_CTRL;               /**< CM sleep reset control, array offset: 0x220, array step: 0x800 */
         uint8_t RESERVED_11[4];
    __IO uint32_t CM_SLEEP_POWER_CTRL;               /**< CM sleep power control, array offset: 0x228, array step: 0x800 */
         uint8_t RESERVED_12[100];
    __IO uint32_t CM_WAKEUP_POWER_CTRL;              /**< CM wakeup power control, array offset: 0x290, array step: 0x800 */
         uint8_t RESERVED_13[4];
    __IO uint32_t CM_WAKEUP_RESET_CTRL;              /**< CM wakeup reset control, array offset: 0x298, array step: 0x800 */
         uint8_t RESERVED_14[4];
    __IO uint32_t CM_WAKEUP_ISO_CTRL;                /**< CM wakeup isolation control, array offset: 0x2A0, array step: 0x800 */
         uint8_t RESERVED_15[4];
    __IO uint32_t CM_WAKEUP_PLL_CTRL;                /**< CM wakeup PLL control, array offset: 0x2A8, array step: 0x800 */
         uint8_t RESERVED_16[4];
    __IO uint32_t CM_WAKEUP_LPCG_CTRL;               /**< CM wakeup LPCG control, array offset: 0x2B0, array step: 0x800 */
         uint8_t RESERVED_17[12];
    __IO uint32_t CM_WAKEUP_SSAR_CTRL;               /**< CM wakeup SSAR control, array offset: 0x2C0, array step: 0x800 */
         uint8_t RESERVED_18[188];
    __IO uint32_t CM_SYS_SLEEP_CTRL;                 /**< CM system sleep control, array offset: 0x380, array step: 0x800 */
         uint8_t RESERVED_19[1148];
  } AUTHEN[2];
} GPC_CPU_CTRL_Type;

/* ----------------------------------------------------------------------------
   -- GPC_CPU_CTRL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPC_CPU_CTRL_Register_Masks GPC_CPU_CTRL Register Masks
 * @{
 */

/*! @name CM_AUTHEN_CTRL - CM Authentication Control */
/*! @{ */

#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_CFG_MASK (0x80U)
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_CFG_SHIFT (7U)
/*! LOCK_CFG - Configuration lock
 *  0b0..The value of low power configuration fields are not locked.
 *  0b1..The value of low power configuration fields are locked. It locks the CPUx_CM registers which are marked
 *       as "Locked by LOCK_CFG field" in the function field.
 */
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_CFG(x)  (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_CFG_SHIFT)) & GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_CFG_MASK)

#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_USER_MASK    (0x100U)
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_USER_SHIFT   (8U)
/*! USER - Allow user mode access
 *  0b0..Allow only privilege mode to access CPU mode control registers
 *  0b1..Allow both privilege and user mode to access CPU mode control registers
 */
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_USER(x)      (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_AUTHEN_CTRL_USER_SHIFT)) & GPC_CPU_CTRL_CM_AUTHEN_CTRL_USER_MASK)

#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_NONSECURE_MASK (0x200U)
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_NONSECURE_SHIFT (9U)
/*! NONSECURE - Allow non-secure mode access
 *  0b0..Allow only secure mode to access CPU mode control
 *  0b1..Allow both secure and non-secure mode to access CPU mode control registers
 */
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_NONSECURE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_AUTHEN_CTRL_NONSECURE_SHIFT)) & GPC_CPU_CTRL_CM_AUTHEN_CTRL_NONSECURE_MASK)

#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_SETTING_MASK (0x800U)
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_SETTING_SHIFT (11U)
/*! LOCK_SETTING - Lock NONSECURE and USER
 *  0b0..NONSECURE and USER fields are not locked
 *  0b1..NONSECURE and USER fields are locked
 */
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_SETTING(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_SETTING_SHIFT)) & GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_SETTING_MASK)

#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_LIST_MASK (0x8000U)
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - White list lock
 *  0b0..WHITE_LIST is not locked
 *  0b1..WHITE_LIST is locked
 */
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_LIST(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_LIST_SHIFT)) & GPC_CPU_CTRL_CM_AUTHEN_CTRL_LOCK_LIST_MASK)

#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_WHITE_LIST_MASK (0xFFFF0000U)
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Domain ID white list */
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_WHITE_LIST(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_AUTHEN_CTRL_WHITE_LIST_SHIFT)) & GPC_CPU_CTRL_CM_AUTHEN_CTRL_WHITE_LIST_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_AUTHEN_CTRL */
#define GPC_CPU_CTRL_CM_AUTHEN_CTRL_COUNT        (2U)

/*! @name CM_MISC - Miscellaneous */
/*! @{ */

#define GPC_CPU_CTRL_CM_MISC_NMI_STAT_MASK       (0x1U)
#define GPC_CPU_CTRL_CM_MISC_NMI_STAT_SHIFT      (0U)
/*! NMI_STAT - Non-masked interrupt status
 *  0b0..NMI is not asserted
 *  0b1..NMI is asserted
 */
#define GPC_CPU_CTRL_CM_MISC_NMI_STAT(x)         (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_MISC_NMI_STAT_SHIFT)) & GPC_CPU_CTRL_CM_MISC_NMI_STAT_MASK)

#define GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_EN_MASK  (0x2U)
#define GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_EN_SHIFT (1U)
/*! SLEEP_HOLD_EN - Allow cpu_sleep_hold_req to assert during CPU low power status
 *  0b0..Disable cpu_sleep_hold_req
 *  0b1..Allow cpu_sleep_hold_req to assert during CPU low power status
 */
#define GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_EN(x)    (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_EN_SHIFT)) & GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_EN_MASK)

#define GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_STAT_MASK (0x4U)
#define GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_STAT_SHIFT (2U)
/*! SLEEP_HOLD_STAT - CPU sleep hold status
 *  0b0..CPU sleep hold is acknowledged
 *  0b1..CPU is not in sleep hold
 */
#define GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_STAT(x)  (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_STAT_SHIFT)) & GPC_CPU_CTRL_CM_MISC_SLEEP_HOLD_STAT_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_MISC */
#define GPC_CPU_CTRL_CM_MISC_COUNT               (2U)

/*! @name CM_MODE_CTRL - CPU mode control */
/*! @{ */

#define GPC_CPU_CTRL_CM_MODE_CTRL_CPU_MODE_TARGET_MASK (0x3U)
#define GPC_CPU_CTRL_CM_MODE_CTRL_CPU_MODE_TARGET_SHIFT (0U)
/*! CPU_MODE_TARGET - The CPU mode the CPU platform should transit to on next sleep event
 *  0b00..Stay in RUN mode
 *  0b01..Transit to WAIT mode
 *  0b10..Transit to STOP mode
 *  0b11..Transit to SUSPEND mode
 */
#define GPC_CPU_CTRL_CM_MODE_CTRL_CPU_MODE_TARGET(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_MODE_CTRL_CPU_MODE_TARGET_SHIFT)) & GPC_CPU_CTRL_CM_MODE_CTRL_CPU_MODE_TARGET_MASK)

#define GPC_CPU_CTRL_CM_MODE_CTRL_WFE_EN_MASK    (0x10U)
#define GPC_CPU_CTRL_CM_MODE_CTRL_WFE_EN_SHIFT   (4U)
/*! WFE_EN - WFE assertion can be sleep event
 *  0b0..WFE assertion can not trigger low power
 *  0b1..WFE assertion can trigger low power
 */
#define GPC_CPU_CTRL_CM_MODE_CTRL_WFE_EN(x)      (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_MODE_CTRL_WFE_EN_SHIFT)) & GPC_CPU_CTRL_CM_MODE_CTRL_WFE_EN_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_MODE_CTRL */
#define GPC_CPU_CTRL_CM_MODE_CTRL_COUNT          (2U)

/*! @name CM_MODE_STAT - CM CPU mode Status */
/*! @{ */

#define GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_CURRENT_MASK (0x3U)
#define GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_CURRENT_SHIFT (0U)
/*! CPU_MODE_CURRENT - Current CPU mode
 *  0b00..CPU is currently in RUN mode
 *  0b01..CPU is currently in WAIT mode
 *  0b10..CPU is currently in STOP mode
 *  0b11..CPU is currently in SUSPEND mode
 */
#define GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_CURRENT(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_CURRENT_SHIFT)) & GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_CURRENT_MASK)

#define GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_PREVIOUS_MASK (0xCU)
#define GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_PREVIOUS_SHIFT (2U)
/*! CPU_MODE_PREVIOUS - Previous CPU mode
 *  0b00..CPU was previously in RUN mode
 *  0b01..CPU was previously in WAIT mode
 *  0b10..CPU was previously in STOP mode
 *  0b11..CPU was previously in SUSPEND mode
 */
#define GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_PREVIOUS(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_PREVIOUS_SHIFT)) & GPC_CPU_CTRL_CM_MODE_STAT_CPU_MODE_PREVIOUS_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_MODE_STAT */
#define GPC_CPU_CTRL_CM_MODE_STAT_COUNT          (2U)

/*! @name CM_IRQ_WAKEUP_MASK - CM IRQ0~31 wakeup mask..CM IRQ224~255 wakeup mask */
/*! @{ */

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_0_31_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_0_31_SHIFT (0U)
/*! MASK_0_31 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_0_31(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_0_31_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_0_31_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_32_63_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_32_63_SHIFT (0U)
/*! MASK_32_63 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_32_63(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_32_63_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_32_63_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_64_95_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_64_95_SHIFT (0U)
/*! MASK_64_95 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_64_95(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_64_95_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_64_95_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_96_127_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_96_127_SHIFT (0U)
/*! MASK_96_127 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_96_127(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_96_127_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_96_127_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_128_159_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_128_159_SHIFT (0U)
/*! MASK_128_159 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_128_159(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_128_159_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_128_159_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_160_191_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_160_191_SHIFT (0U)
/*! MASK_160_191 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_160_191(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_160_191_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_160_191_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_192_223_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_192_223_SHIFT (0U)
/*! MASK_192_223 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_192_223(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_192_223_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_192_223_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_224_255_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_224_255_SHIFT (0U)
/*! MASK_224_255 - "1" means the IRQ cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_224_255(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_224_255_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_MASK_224_255_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_COUNT    (2U)

/* The count of GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_MASK_COUNT2   (8U)

/*! @name CM_NON_IRQ_WAKEUP_MASK - CM non-IRQ wakeup mask */
/*! @{ */

#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_MASK_DEBUG_WAKEUP_MASK_MASK (0x2U)
#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_MASK_DEBUG_WAKEUP_MASK_SHIFT (1U)
/*! DEBUG_WAKEUP_MASK - "1" means the debug_wakeup_request cannot wakeup CPU platform */
#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_MASK_DEBUG_WAKEUP_MASK(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_MASK_DEBUG_WAKEUP_MASK_SHIFT)) & GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_MASK_DEBUG_WAKEUP_MASK_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_MASK */
#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_MASK_COUNT (2U)

/*! @name CM_IRQ_WAKEUP_STAT - CM IRQ0~31 wakeup status..CM IRQ224~255 wakeup status */
/*! @{ */

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_0_31_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_0_31_SHIFT (0U)
/*! STAT_0_31 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_0_31(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_0_31_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_0_31_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_32_63_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_32_63_SHIFT (0U)
/*! STAT_32_63 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_32_63(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_32_63_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_32_63_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_64_95_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_64_95_SHIFT (0U)
/*! STAT_64_95 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_64_95(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_64_95_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_64_95_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_96_127_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_96_127_SHIFT (0U)
/*! STAT_96_127 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_96_127(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_96_127_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_96_127_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_128_159_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_128_159_SHIFT (0U)
/*! STAT_128_159 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_128_159(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_128_159_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_128_159_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_160_191_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_160_191_SHIFT (0U)
/*! STAT_160_191 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_160_191(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_160_191_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_160_191_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_192_223_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_192_223_SHIFT (0U)
/*! STAT_192_223 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_192_223(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_192_223_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_192_223_MASK)

#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_224_255_MASK (0xFFFFFFFFU)
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_224_255_SHIFT (0U)
/*! STAT_224_255 - IRQ status */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_224_255(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_224_255_SHIFT)) & GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_STAT_224_255_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_COUNT    (2U)

/* The count of GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT */
#define GPC_CPU_CTRL_CM_IRQ_WAKEUP_STAT_COUNT2   (8U)

/*! @name CM_NON_IRQ_WAKEUP_STAT - CM non-irq wakeup status */
/*! @{ */

#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_STAT_DEBUG_WAKEUP_STAT_MASK (0x2U)
#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_STAT_DEBUG_WAKEUP_STAT_SHIFT (1U)
/*! DEBUG_WAKEUP_STAT - Debug wakeup status
 *  0b0..No debug wakeup is requested
 *  0b1..Debug wakeup is requested
 */
#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_STAT_DEBUG_WAKEUP_STAT(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_STAT_DEBUG_WAKEUP_STAT_SHIFT)) & GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_STAT_DEBUG_WAKEUP_STAT_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_STAT */
#define GPC_CPU_CTRL_CM_NON_IRQ_WAKEUP_STAT_COUNT (2U)

/*! @name CM_SLEEP_SSAR_CTRL - CM sleep SSAR control */
/*! @{ */

#define GPC_CPU_CTRL_CM_SLEEP_SSAR_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_SLEEP_SSAR_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_SLEEP_SSAR_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SLEEP_SSAR_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_SLEEP_SSAR_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_SLEEP_SSAR_CTRL */
#define GPC_CPU_CTRL_CM_SLEEP_SSAR_CTRL_COUNT    (2U)

/*! @name CM_SLEEP_LPCG_CTRL - CM sleep LPCG control */
/*! @{ */

#define GPC_CPU_CTRL_CM_SLEEP_LPCG_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_SLEEP_LPCG_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_SLEEP_LPCG_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SLEEP_LPCG_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_SLEEP_LPCG_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_SLEEP_LPCG_CTRL */
#define GPC_CPU_CTRL_CM_SLEEP_LPCG_CTRL_COUNT    (2U)

/*! @name CM_SLEEP_PLL_CTRL - CM sleep PLL control */
/*! @{ */

#define GPC_CPU_CTRL_CM_SLEEP_PLL_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_SLEEP_PLL_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_SLEEP_PLL_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SLEEP_PLL_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_SLEEP_PLL_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_SLEEP_PLL_CTRL */
#define GPC_CPU_CTRL_CM_SLEEP_PLL_CTRL_COUNT     (2U)

/*! @name CM_SLEEP_ISO_CTRL - CM sleep isolation control */
/*! @{ */

#define GPC_CPU_CTRL_CM_SLEEP_ISO_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_SLEEP_ISO_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_SLEEP_ISO_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SLEEP_ISO_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_SLEEP_ISO_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_SLEEP_ISO_CTRL */
#define GPC_CPU_CTRL_CM_SLEEP_ISO_CTRL_COUNT     (2U)

/*! @name CM_SLEEP_RESET_CTRL - CM sleep reset control */
/*! @{ */

#define GPC_CPU_CTRL_CM_SLEEP_RESET_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_SLEEP_RESET_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_SLEEP_RESET_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SLEEP_RESET_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_SLEEP_RESET_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_SLEEP_RESET_CTRL */
#define GPC_CPU_CTRL_CM_SLEEP_RESET_CTRL_COUNT   (2U)

/*! @name CM_SLEEP_POWER_CTRL - CM sleep power control */
/*! @{ */

#define GPC_CPU_CTRL_CM_SLEEP_POWER_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_SLEEP_POWER_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_SLEEP_POWER_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SLEEP_POWER_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_SLEEP_POWER_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_SLEEP_POWER_CTRL */
#define GPC_CPU_CTRL_CM_SLEEP_POWER_CTRL_COUNT   (2U)

/*! @name CM_WAKEUP_POWER_CTRL - CM wakeup power control */
/*! @{ */

#define GPC_CPU_CTRL_CM_WAKEUP_POWER_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_WAKEUP_POWER_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_WAKEUP_POWER_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_WAKEUP_POWER_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_WAKEUP_POWER_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_WAKEUP_POWER_CTRL */
#define GPC_CPU_CTRL_CM_WAKEUP_POWER_CTRL_COUNT  (2U)

/*! @name CM_WAKEUP_RESET_CTRL - CM wakeup reset control */
/*! @{ */

#define GPC_CPU_CTRL_CM_WAKEUP_RESET_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_WAKEUP_RESET_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_WAKEUP_RESET_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_WAKEUP_RESET_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_WAKEUP_RESET_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_WAKEUP_RESET_CTRL */
#define GPC_CPU_CTRL_CM_WAKEUP_RESET_CTRL_COUNT  (2U)

/*! @name CM_WAKEUP_ISO_CTRL - CM wakeup isolation control */
/*! @{ */

#define GPC_CPU_CTRL_CM_WAKEUP_ISO_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_WAKEUP_ISO_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_WAKEUP_ISO_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_WAKEUP_ISO_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_WAKEUP_ISO_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_WAKEUP_ISO_CTRL */
#define GPC_CPU_CTRL_CM_WAKEUP_ISO_CTRL_COUNT    (2U)

/*! @name CM_WAKEUP_PLL_CTRL - CM wakeup PLL control */
/*! @{ */

#define GPC_CPU_CTRL_CM_WAKEUP_PLL_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_WAKEUP_PLL_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_WAKEUP_PLL_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_WAKEUP_PLL_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_WAKEUP_PLL_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_WAKEUP_PLL_CTRL */
#define GPC_CPU_CTRL_CM_WAKEUP_PLL_CTRL_COUNT    (2U)

/*! @name CM_WAKEUP_LPCG_CTRL - CM wakeup LPCG control */
/*! @{ */

#define GPC_CPU_CTRL_CM_WAKEUP_LPCG_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_WAKEUP_LPCG_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_WAKEUP_LPCG_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_WAKEUP_LPCG_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_WAKEUP_LPCG_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_WAKEUP_LPCG_CTRL */
#define GPC_CPU_CTRL_CM_WAKEUP_LPCG_CTRL_COUNT   (2U)

/*! @name CM_WAKEUP_SSAR_CTRL - CM wakeup SSAR control */
/*! @{ */

#define GPC_CPU_CTRL_CM_WAKEUP_SSAR_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_CPU_CTRL_CM_WAKEUP_SSAR_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_CPU_CTRL_CM_WAKEUP_SSAR_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_WAKEUP_SSAR_CTRL_DISABLE_SHIFT)) & GPC_CPU_CTRL_CM_WAKEUP_SSAR_CTRL_DISABLE_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_WAKEUP_SSAR_CTRL */
#define GPC_CPU_CTRL_CM_WAKEUP_SSAR_CTRL_COUNT   (2U)

/*! @name CM_SYS_SLEEP_CTRL - CM system sleep control */
/*! @{ */

#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_WAIT_MASK (0x1U)
#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_WAIT_SHIFT (0U)
/*! SS_WAIT - Request system sleep when CPU is in WAIT mode
 *  0b0..Do not request system sleep when CPU is in WAIT mode
 *  0b1..Request system sleep when CPU is in WAIT mode
 */
#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_WAIT(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_WAIT_SHIFT)) & GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_WAIT_MASK)

#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_STOP_MASK (0x2U)
#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_STOP_SHIFT (1U)
/*! SS_STOP - Request system sleep when CPU is in STOP mode
 *  0b0..Do not request system sleep when CPU is in STOP mode
 *  0b1..Request system sleep when CPU is in STOP mode
 */
#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_STOP(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_STOP_SHIFT)) & GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_STOP_MASK)

#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_SUSPEND_MASK (0x4U)
#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_SUSPEND_SHIFT (2U)
/*! SS_SUSPEND - Request system sleep when CPU is in SUSPEND mode
 *  0b0..Do not request system sleep when CPU is in SUSPEND mode
 *  0b1..Request system sleep when CPU is in SUSPEND mode
 */
#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_SUSPEND(x) (((uint32_t)(((uint32_t)(x)) << GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_SUSPEND_SHIFT)) & GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_SS_SUSPEND_MASK)
/*! @} */

/* The count of GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL */
#define GPC_CPU_CTRL_CM_SYS_SLEEP_CTRL_COUNT     (2U)


/*!
 * @}
 */ /* end of group GPC_CPU_CTRL_Register_Masks */


/* GPC_CPU_CTRL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral GPC_CPU_CTRL base address */
  #define GPC_CPU_CTRL_BASE                        (0x54470000u)
  /** Peripheral GPC_CPU_CTRL base address */
  #define GPC_CPU_CTRL_BASE_NS                     (0x44470000u)
  /** Peripheral GPC_CPU_CTRL base pointer */
  #define GPC_CPU_CTRL                             ((GPC_CPU_CTRL_Type *)GPC_CPU_CTRL_BASE)
  /** Peripheral GPC_CPU_CTRL base pointer */
  #define GPC_CPU_CTRL_NS                          ((GPC_CPU_CTRL_Type *)GPC_CPU_CTRL_BASE_NS)
  /** Array initializer of GPC_CPU_CTRL peripheral base addresses */
  #define GPC_CPU_CTRL_BASE_ADDRS                  { GPC_CPU_CTRL_BASE }
  /** Array initializer of GPC_CPU_CTRL peripheral base pointers */
  #define GPC_CPU_CTRL_BASE_PTRS                   { GPC_CPU_CTRL }
  /** Array initializer of GPC_CPU_CTRL peripheral base addresses */
  #define GPC_CPU_CTRL_BASE_ADDRS_NS               { GPC_CPU_CTRL_BASE_NS }
  /** Array initializer of GPC_CPU_CTRL peripheral base pointers */
  #define GPC_CPU_CTRL_BASE_PTRS_NS                { GPC_CPU_CTRL_NS }
#else
  /** Peripheral GPC_CPU_CTRL base address */
  #define GPC_CPU_CTRL_BASE                        (0x44470000u)
  /** Peripheral GPC_CPU_CTRL base pointer */
  #define GPC_CPU_CTRL                             ((GPC_CPU_CTRL_Type *)GPC_CPU_CTRL_BASE)
  /** Array initializer of GPC_CPU_CTRL peripheral base addresses */
  #define GPC_CPU_CTRL_BASE_ADDRS                  { GPC_CPU_CTRL_BASE }
  /** Array initializer of GPC_CPU_CTRL peripheral base pointers */
  #define GPC_CPU_CTRL_BASE_PTRS                   { GPC_CPU_CTRL }
#endif

/*!
 * @}
 */ /* end of group GPC_CPU_CTRL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- GPC_GLOBAL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPC_GLOBAL_Peripheral_Access_Layer GPC_GLOBAL Peripheral Access Layer
 * @{
 */

/** GPC_GLOBAL - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t AUTHEN_CTRL;                       /**< GPC Global Authentication Control, offset: 0x4 */
       uint8_t RESERVED_1[8];
  __IO uint32_t GPC_CPU0_DOMAIN;                   /**< GPC CPU0 domain assignment, offset: 0x10 */
  __IO uint32_t GPC_CPU1_DOMAIN;                   /**< GPC CPU1 domain assignment, offset: 0x14 */
       uint8_t RESERVED_2[12];
  __IO uint32_t GPC_MASTER;                        /**< GPC master CPU configuration, offset: 0x24 */
       uint8_t RESERVED_3[472];
  __IO uint32_t GPC_ROSC_CTRL;                     /**< RCOSC control, offset: 0x200 */
} GPC_GLOBAL_Type;

/* ----------------------------------------------------------------------------
   -- GPC_GLOBAL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPC_GLOBAL_Register_Masks GPC_GLOBAL Register Masks
 * @{
 */

/*! @name AUTHEN_CTRL - GPC Global Authentication Control */
/*! @{ */

#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_CFG_MASK     (0x80U)
#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_CFG_SHIFT    (7U)
/*! LOCK_CFG - Configuration lock
 *  0b0..The value of low power configuration fields are not locked.
 *  0b1..The value of low power configuration fields are locked. Refer to the function field of each gpc_global registers.
 */
#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_CFG(x)       (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_AUTHEN_CTRL_LOCK_CFG_SHIFT)) & GPC_GLOBAL_AUTHEN_CTRL_LOCK_CFG_MASK)

#define GPC_GLOBAL_AUTHEN_CTRL_USER_MASK         (0x100U)
#define GPC_GLOBAL_AUTHEN_CTRL_USER_SHIFT        (8U)
/*! USER - Allow user mode access
 *  0b0..Allow only privilege mode to access CPU mode control registers
 *  0b1..Allow both privilege and user mode to access CPU mode control registers
 */
#define GPC_GLOBAL_AUTHEN_CTRL_USER(x)           (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_AUTHEN_CTRL_USER_SHIFT)) & GPC_GLOBAL_AUTHEN_CTRL_USER_MASK)

#define GPC_GLOBAL_AUTHEN_CTRL_NONSECURE_MASK    (0x200U)
#define GPC_GLOBAL_AUTHEN_CTRL_NONSECURE_SHIFT   (9U)
/*! NONSECURE - Allow non-secure mode access
 *  0b0..Allow only secure mode to access CPU mode registers
 *  0b1..Allow both secure and non-secure mode to access CPU mode control registers.
 */
#define GPC_GLOBAL_AUTHEN_CTRL_NONSECURE(x)      (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_AUTHEN_CTRL_NONSECURE_SHIFT)) & GPC_GLOBAL_AUTHEN_CTRL_NONSECURE_MASK)

#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_SETTING_MASK (0x800U)
#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_SETTING_SHIFT (11U)
/*! LOCK_SETTING - Lock NONSECURE and USER
 *  0b0..NONSECURE and USER fields are not locked
 *  0b1..NONSECURE and USER fields are locked
 */
#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_SETTING(x)   (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_AUTHEN_CTRL_LOCK_SETTING_SHIFT)) & GPC_GLOBAL_AUTHEN_CTRL_LOCK_SETTING_MASK)

#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_LIST_MASK    (0x8000U)
#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_LIST_SHIFT   (15U)
/*! LOCK_LIST - White list lock
 *  0b0..WHITE_LIST is not locked
 *  0b1..WHITE_LIST is locked
 */
#define GPC_GLOBAL_AUTHEN_CTRL_LOCK_LIST(x)      (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_AUTHEN_CTRL_LOCK_LIST_SHIFT)) & GPC_GLOBAL_AUTHEN_CTRL_LOCK_LIST_MASK)

#define GPC_GLOBAL_AUTHEN_CTRL_WHITE_LIST_MASK   (0xFFFF0000U)
#define GPC_GLOBAL_AUTHEN_CTRL_WHITE_LIST_SHIFT  (16U)
/*! WHITE_LIST - Domain ID white list */
#define GPC_GLOBAL_AUTHEN_CTRL_WHITE_LIST(x)     (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_AUTHEN_CTRL_WHITE_LIST_SHIFT)) & GPC_GLOBAL_AUTHEN_CTRL_WHITE_LIST_MASK)
/*! @} */

/*! @name GPC_CPU0_DOMAIN - GPC CPU0 domain assignment */
/*! @{ */

#define GPC_GLOBAL_GPC_CPU0_DOMAIN_CPU0_DOMAIN_MASK (0xFFFFU)
#define GPC_GLOBAL_GPC_CPU0_DOMAIN_CPU0_DOMAIN_SHIFT (0U)
/*! CPU0_DOMAIN - CPU0 domain assignment */
#define GPC_GLOBAL_GPC_CPU0_DOMAIN_CPU0_DOMAIN(x) (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_GPC_CPU0_DOMAIN_CPU0_DOMAIN_SHIFT)) & GPC_GLOBAL_GPC_CPU0_DOMAIN_CPU0_DOMAIN_MASK)
/*! @} */

/*! @name GPC_CPU1_DOMAIN - GPC CPU1 domain assignment */
/*! @{ */

#define GPC_GLOBAL_GPC_CPU1_DOMAIN_CPU1_DOMAIN_MASK (0xFFFFU)
#define GPC_GLOBAL_GPC_CPU1_DOMAIN_CPU1_DOMAIN_SHIFT (0U)
/*! CPU1_DOMAIN - CPU1 domain assignment */
#define GPC_GLOBAL_GPC_CPU1_DOMAIN_CPU1_DOMAIN(x) (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_GPC_CPU1_DOMAIN_CPU1_DOMAIN_SHIFT)) & GPC_GLOBAL_GPC_CPU1_DOMAIN_CPU1_DOMAIN_MASK)
/*! @} */

/*! @name GPC_MASTER - GPC master CPU configuration */
/*! @{ */

#define GPC_GLOBAL_GPC_MASTER_CPU0_MASTER_MASK   (0x1U)
#define GPC_GLOBAL_GPC_MASTER_CPU0_MASTER_SHIFT  (0U)
/*! CPU0_MASTER - Setting to 1 means CPU0 is the master CPU of its domain
 *  0b0..CPU0 is not the master CPU of its domain
 *  0b1..CPU0 is the master CPU of its domain
 */
#define GPC_GLOBAL_GPC_MASTER_CPU0_MASTER(x)     (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_GPC_MASTER_CPU0_MASTER_SHIFT)) & GPC_GLOBAL_GPC_MASTER_CPU0_MASTER_MASK)

#define GPC_GLOBAL_GPC_MASTER_CPU1_MASTER_MASK   (0x2U)
#define GPC_GLOBAL_GPC_MASTER_CPU1_MASTER_SHIFT  (1U)
/*! CPU1_MASTER - Setting to 1 means CPU1 is the master CPU of its domain
 *  0b0..CPU1 is not the master CPU of its domain
 *  0b1..CPU1 is the master CPU of its domain
 */
#define GPC_GLOBAL_GPC_MASTER_CPU1_MASTER(x)     (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_GPC_MASTER_CPU1_MASTER_SHIFT)) & GPC_GLOBAL_GPC_MASTER_CPU1_MASTER_MASK)
/*! @} */

/*! @name GPC_ROSC_CTRL - RCOSC control */
/*! @{ */

#define GPC_GLOBAL_GPC_ROSC_CTRL_ROSC_OFF_EN_MASK (0x1U)
#define GPC_GLOBAL_GPC_ROSC_CTRL_ROSC_OFF_EN_SHIFT (0U)
/*! ROSC_OFF_EN - Shut off the 24 MHz RCOSC clock when system sleep
 *  0b0..Keep 24 MHz ROSC clock running during system sleep
 *  0b1..Shut off 24 MHz ROSC clock during system sleep
 */
#define GPC_GLOBAL_GPC_ROSC_CTRL_ROSC_OFF_EN(x)  (((uint32_t)(((uint32_t)(x)) << GPC_GLOBAL_GPC_ROSC_CTRL_ROSC_OFF_EN_SHIFT)) & GPC_GLOBAL_GPC_ROSC_CTRL_ROSC_OFF_EN_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group GPC_GLOBAL_Register_Masks */


/* GPC_GLOBAL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral GPC_GLOBAL base address */
  #define GPC_GLOBAL_BASE                          (0x54472000u)
  /** Peripheral GPC_GLOBAL base address */
  #define GPC_GLOBAL_BASE_NS                       (0x44472000u)
  /** Peripheral GPC_GLOBAL base pointer */
  #define GPC_GLOBAL                               ((GPC_GLOBAL_Type *)GPC_GLOBAL_BASE)
  /** Peripheral GPC_GLOBAL base pointer */
  #define GPC_GLOBAL_NS                            ((GPC_GLOBAL_Type *)GPC_GLOBAL_BASE_NS)
  /** Array initializer of GPC_GLOBAL peripheral base addresses */
  #define GPC_GLOBAL_BASE_ADDRS                    { GPC_GLOBAL_BASE }
  /** Array initializer of GPC_GLOBAL peripheral base pointers */
  #define GPC_GLOBAL_BASE_PTRS                     { GPC_GLOBAL }
  /** Array initializer of GPC_GLOBAL peripheral base addresses */
  #define GPC_GLOBAL_BASE_ADDRS_NS                 { GPC_GLOBAL_BASE_NS }
  /** Array initializer of GPC_GLOBAL peripheral base pointers */
  #define GPC_GLOBAL_BASE_PTRS_NS                  { GPC_GLOBAL_NS }
#else
  /** Peripheral GPC_GLOBAL base address */
  #define GPC_GLOBAL_BASE                          (0x44472000u)
  /** Peripheral GPC_GLOBAL base pointer */
  #define GPC_GLOBAL                               ((GPC_GLOBAL_Type *)GPC_GLOBAL_BASE)
  /** Array initializer of GPC_GLOBAL peripheral base addresses */
  #define GPC_GLOBAL_BASE_ADDRS                    { GPC_GLOBAL_BASE }
  /** Array initializer of GPC_GLOBAL peripheral base pointers */
  #define GPC_GLOBAL_BASE_PTRS                     { GPC_GLOBAL }
#endif

/*!
 * @}
 */ /* end of group GPC_GLOBAL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- GPC_SYS_SLEEP_CTRL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPC_SYS_SLEEP_CTRL_Peripheral_Access_Layer GPC_SYS_SLEEP_CTRL Peripheral Access Layer
 * @{
 */

/** GPC_SYS_SLEEP_CTRL - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t SS_AUTHEN_CTRL;                    /**< System Sleep Authentication Control, offset: 0x4 */
       uint8_t RESERVED_1[4];
  __IO uint32_t SS_MISC;                           /**< System Sleep Misc, offset: 0xC */
       uint8_t RESERVED_2[48];
  __IO uint32_t PMIC_CTRL;                         /**< PMIC standby control, offset: 0x40 */
       uint8_t RESERVED_3[172];
  __IO uint32_t SS_STEP0_IN_CTRL;                  /**< System Sleep STEP0 (BIAS) in control, offset: 0xF0 */
       uint8_t RESERVED_4[12];
  __IO uint32_t SS_STEP1_IN_CTRL;                  /**< System Sleep STEP1 (PLDO) in control, offset: 0x100 */
       uint8_t RESERVED_5[12];
  __IO uint32_t SS_STEP2_IN_CTRL;                  /**< System Sleep STEP2 (BANDGAP) in control, offset: 0x110 */
       uint8_t RESERVED_6[12];
  __IO uint32_t SS_STEP3_IN_CTRL;                  /**< System Sleep STEP3 (LDO) in control, offset: 0x120 */
       uint8_t RESERVED_7[28];
  __IO uint32_t SS_DCDC_IN_CTRL;                   /**< System Sleep DCDC in control, offset: 0x140 */
       uint8_t RESERVED_8[12];
  __IO uint32_t SS_PMIC_IN_CTRL;                   /**< System Sleep PMIC in control, offset: 0x150 */
       uint8_t RESERVED_9[172];
  __IO uint32_t SS_PMIC_OUT_CTRL;                  /**< System Sleep PMIC out control, offset: 0x200 */
       uint8_t RESERVED_10[12];
  __IO uint32_t SS_DCDC_OUT_CTRL;                  /**< System Sleep DCDC out control, offset: 0x210 */
       uint8_t RESERVED_11[36];
  __IO uint32_t SS_STEP3_OUT_CTRL;                 /**< System Sleep STEP3 (LDO) out control, offset: 0x238 */
       uint8_t RESERVED_12[4];
  __IO uint32_t SS_STEP2_OUT_CTRL;                 /**< System Sleep STEP2 (BANDGAP) out control, offset: 0x240 */
       uint8_t RESERVED_13[12];
  __IO uint32_t SS_STEP1_OUT_CTRL;                 /**< System Sleep STEP1 (PLDO) out control, offset: 0x250 */
       uint8_t RESERVED_14[12];
  __IO uint32_t SS_STEP0_OUT_CTRL;                 /**< System Sleep STEP0 (BIAS) out control, offset: 0x260 */
} GPC_SYS_SLEEP_CTRL_Type;

/* ----------------------------------------------------------------------------
   -- GPC_SYS_SLEEP_CTRL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPC_SYS_SLEEP_CTRL_Register_Masks GPC_SYS_SLEEP_CTRL Register Masks
 * @{
 */

/*! @name SS_AUTHEN_CTRL - System Sleep Authentication Control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_AUTHEN_CTRL_LOCK_CFG_MASK (0x100000U)
#define GPC_SYS_SLEEP_CTRL_SS_AUTHEN_CTRL_LOCK_CFG_SHIFT (20U)
/*! LOCK_CFG - Configuration lock
 *  0b0..The value of low power configuration fields are not locked.
 *  0b1..The value of low power configuration fields are locked. Refer to the function field of each gpc_sys_sleep_ctrl registers.
 */
#define GPC_SYS_SLEEP_CTRL_SS_AUTHEN_CTRL_LOCK_CFG(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_AUTHEN_CTRL_LOCK_CFG_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_AUTHEN_CTRL_LOCK_CFG_MASK)
/*! @} */

/*! @name SS_MISC - System Sleep Misc */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU0_SYS_SLEEP_MASK (0x1U)
#define GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU0_SYS_SLEEP_SHIFT (0U)
/*! FORCE_CPU0_SYS_SLEEP - Force CPU0 to request system sleep mode
 *  0b0..Do not force CPU0 to request system sleep mode
 *  0b1..Force CPU0 to request system sleep mode
 */
#define GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU0_SYS_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU0_SYS_SLEEP_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU0_SYS_SLEEP_MASK)

#define GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU1_SYS_SLEEP_MASK (0x2U)
#define GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU1_SYS_SLEEP_SHIFT (1U)
/*! FORCE_CPU1_SYS_SLEEP - Force CPU1 to request system sleep mode
 *  0b0..Do not force CPU1 to request system sleep mode
 *  0b1..Force CPU1 to request system sleep mode
 */
#define GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU1_SYS_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU1_SYS_SLEEP_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_MISC_FORCE_CPU1_SYS_SLEEP_MASK)
/*! @} */

/*! @name PMIC_CTRL - PMIC standby control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_EN_MASK (0x1U)
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_EN_SHIFT (0U)
/*! PMIC_STBY_EN - Assert the PMIC standby request when system sleep
 *  0b0..Do not assert PMIC_STBY_REQ when system sleep is entered
 *  0b1..Assert PMIC_STBY_REQ when system sleep is entered
 */
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_EN(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_EN_SHIFT)) & GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_EN_MASK)

#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_MASK (0x2U)
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_SHIFT (1U)
/*! PMIC_READY - PMIC_READY pin status
 *  0b0..PMIC_READY not asserted
 *  0b1..PMIC_READY asserted
 */
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_SHIFT)) & GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_MASK)

#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_EXIST_MASK (0x4U)
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_EXIST_SHIFT (2U)
/*! PMIC_READY_EXIST - PMIC_READY is driven from pad
 *  0b0..PMIC_READY is not driven.
 *  0b1..PMIC_READY is driven from pad. If PMIC_READY_EXIST = 1, PMIC_READY = 1 means external PMIC drives PMIC_READY pin.
 */
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_EXIST(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_EXIST_SHIFT)) & GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_READY_EXIST_MASK)

#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_SOFT_MASK (0x10U)
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_SOFT_SHIFT (4U)
/*! PMIC_STBY_SOFT - Software PMIC standby trigger
 *  0b0..Exit PMIC standby
 *  0b1..Trigger PMIC standby
 */
#define GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_SOFT(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_SOFT_SHIFT)) & GPC_SYS_SLEEP_CTRL_PMIC_CTRL_PMIC_STBY_SOFT_MASK)
/*! @} */

/*! @name SS_STEP0_IN_CTRL - System Sleep STEP0 (BIAS) in control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP0_IN_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP0_IN_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP0_IN_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP0_IN_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP0_IN_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_STEP1_IN_CTRL - System Sleep STEP1 (PLDO) in control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP1_IN_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP1_IN_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP1_IN_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP1_IN_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP1_IN_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_STEP2_IN_CTRL - System Sleep STEP2 (BANDGAP) in control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP2_IN_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP2_IN_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP2_IN_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP2_IN_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP2_IN_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_STEP3_IN_CTRL - System Sleep STEP3 (LDO) in control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP3_IN_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP3_IN_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP3_IN_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP3_IN_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP3_IN_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_DCDC_IN_CTRL - System Sleep DCDC in control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_DCDC_IN_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_DCDC_IN_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_DCDC_IN_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_DCDC_IN_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_DCDC_IN_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_PMIC_IN_CTRL - System Sleep PMIC in control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_PMIC_IN_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_PMIC_IN_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_PMIC_IN_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_PMIC_IN_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_PMIC_IN_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_PMIC_OUT_CTRL - System Sleep PMIC out control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_PMIC_OUT_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_PMIC_OUT_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_PMIC_OUT_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_PMIC_OUT_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_PMIC_OUT_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_DCDC_OUT_CTRL - System Sleep DCDC out control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_DCDC_OUT_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_DCDC_OUT_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_DCDC_OUT_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_DCDC_OUT_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_DCDC_OUT_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_STEP3_OUT_CTRL - System Sleep STEP3 (LDO) out control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP3_OUT_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP3_OUT_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP3_OUT_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP3_OUT_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP3_OUT_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_STEP2_OUT_CTRL - System Sleep STEP2 (BANDGAP) out control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP2_OUT_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP2_OUT_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP2_OUT_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP2_OUT_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP2_OUT_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_STEP1_OUT_CTRL - System Sleep STEP1 (PLDO) out control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP1_OUT_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP1_OUT_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP1_OUT_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP1_OUT_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP1_OUT_CTRL_DISABLE_MASK)
/*! @} */

/*! @name SS_STEP0_OUT_CTRL - System Sleep STEP0 (BIAS) out control */
/*! @{ */

#define GPC_SYS_SLEEP_CTRL_SS_STEP0_OUT_CTRL_DISABLE_MASK (0x80000000U)
#define GPC_SYS_SLEEP_CTRL_SS_STEP0_OUT_CTRL_DISABLE_SHIFT (31U)
/*! DISABLE - Disable this step
 *  0b0..This step is enabled.
 *  0b1..This step is disabled. GPC will skip this step and not send any request.
 */
#define GPC_SYS_SLEEP_CTRL_SS_STEP0_OUT_CTRL_DISABLE(x) (((uint32_t)(((uint32_t)(x)) << GPC_SYS_SLEEP_CTRL_SS_STEP0_OUT_CTRL_DISABLE_SHIFT)) & GPC_SYS_SLEEP_CTRL_SS_STEP0_OUT_CTRL_DISABLE_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group GPC_SYS_SLEEP_CTRL_Register_Masks */


/* GPC_SYS_SLEEP_CTRL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral GPC_SYS_SLEEP_CTRL base address */
  #define GPC_SYS_SLEEP_CTRL_BASE                  (0x54472800u)
  /** Peripheral GPC_SYS_SLEEP_CTRL base address */
  #define GPC_SYS_SLEEP_CTRL_BASE_NS               (0x44472800u)
  /** Peripheral GPC_SYS_SLEEP_CTRL base pointer */
  #define GPC_SYS_SLEEP_CTRL                       ((GPC_SYS_SLEEP_CTRL_Type *)GPC_SYS_SLEEP_CTRL_BASE)
  /** Peripheral GPC_SYS_SLEEP_CTRL base pointer */
  #define GPC_SYS_SLEEP_CTRL_NS                    ((GPC_SYS_SLEEP_CTRL_Type *)GPC_SYS_SLEEP_CTRL_BASE_NS)
  /** Array initializer of GPC_SYS_SLEEP_CTRL peripheral base addresses */
  #define GPC_SYS_SLEEP_CTRL_BASE_ADDRS            { GPC_SYS_SLEEP_CTRL_BASE }
  /** Array initializer of GPC_SYS_SLEEP_CTRL peripheral base pointers */
  #define GPC_SYS_SLEEP_CTRL_BASE_PTRS             { GPC_SYS_SLEEP_CTRL }
  /** Array initializer of GPC_SYS_SLEEP_CTRL peripheral base addresses */
  #define GPC_SYS_SLEEP_CTRL_BASE_ADDRS_NS         { GPC_SYS_SLEEP_CTRL_BASE_NS }
  /** Array initializer of GPC_SYS_SLEEP_CTRL peripheral base pointers */
  #define GPC_SYS_SLEEP_CTRL_BASE_PTRS_NS          { GPC_SYS_SLEEP_CTRL_NS }
#else
  /** Peripheral GPC_SYS_SLEEP_CTRL base address */
  #define GPC_SYS_SLEEP_CTRL_BASE                  (0x44472800u)
  /** Peripheral GPC_SYS_SLEEP_CTRL base pointer */
  #define GPC_SYS_SLEEP_CTRL                       ((GPC_SYS_SLEEP_CTRL_Type *)GPC_SYS_SLEEP_CTRL_BASE)
  /** Array initializer of GPC_SYS_SLEEP_CTRL peripheral base addresses */
  #define GPC_SYS_SLEEP_CTRL_BASE_ADDRS            { GPC_SYS_SLEEP_CTRL_BASE }
  /** Array initializer of GPC_SYS_SLEEP_CTRL peripheral base pointers */
  #define GPC_SYS_SLEEP_CTRL_BASE_PTRS             { GPC_SYS_SLEEP_CTRL }
#endif

/*!
 * @}
 */ /* end of group GPC_SYS_SLEEP_CTRL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- GPT Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPT_Peripheral_Access_Layer GPT Peripheral Access Layer
 * @{
 */

/** GPT - Register Layout Typedef */
typedef struct {
  __IO uint32_t CR;                                /**< Control, offset: 0x0 */
  __IO uint32_t PR;                                /**< Prescaler, offset: 0x4 */
  __IO uint32_t SR;                                /**< Status, offset: 0x8 */
  __IO uint32_t IR;                                /**< Interrupt, offset: 0xC */
  __IO uint32_t OCR[3];                            /**< Output Compare, array offset: 0x10, array step: 0x4 */
  __I  uint32_t ICR[2];                            /**< Input Capture, array offset: 0x1C, array step: 0x4 */
  __I  uint32_t CNT;                               /**< Counter, offset: 0x24 */
} GPT_Type;

/* ----------------------------------------------------------------------------
   -- GPT Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup GPT_Register_Masks GPT Register Masks
 * @{
 */

/*! @name CR - Control */
/*! @{ */

#define GPT_CR_EN_MASK                           (0x1U)
#define GPT_CR_EN_SHIFT                          (0U)
/*! EN - GPT Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_CR_EN(x)                             (((uint32_t)(((uint32_t)(x)) << GPT_CR_EN_SHIFT)) & GPT_CR_EN_MASK)

#define GPT_CR_ENMOD_MASK                        (0x2U)
#define GPT_CR_ENMOD_SHIFT                       (1U)
/*! ENMOD - GPT Enable Mode
 *  0b0..Restart counting from frozen values after GPT is enabled (EN=1).
 *  0b1..Reset counting from 0 after GPT is enabled (EN=1).
 */
#define GPT_CR_ENMOD(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_CR_ENMOD_SHIFT)) & GPT_CR_ENMOD_MASK)

#define GPT_CR_DBGEN_MASK                        (0x4U)
#define GPT_CR_DBGEN_SHIFT                       (2U)
/*! DBGEN - GPT Debug Mode Enable
 *  0b0..Disable in Debug mode
 *  0b1..Enable in Debug mode
 */
#define GPT_CR_DBGEN(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_CR_DBGEN_SHIFT)) & GPT_CR_DBGEN_MASK)

#define GPT_CR_WAITEN_MASK                       (0x8U)
#define GPT_CR_WAITEN_SHIFT                      (3U)
/*! WAITEN - GPT Wait Mode Enable
 *  0b0..Disable in Wait mode
 *  0b1..Enable in Wait mode
 */
#define GPT_CR_WAITEN(x)                         (((uint32_t)(((uint32_t)(x)) << GPT_CR_WAITEN_SHIFT)) & GPT_CR_WAITEN_MASK)

#define GPT_CR_DOZEEN_MASK                       (0x10U)
#define GPT_CR_DOZEEN_SHIFT                      (4U)
/*! DOZEEN - GPT Doze Mode Enable
 *  0b0..Disable in Doze mode
 *  0b1..Enable in Doze mode
 */
#define GPT_CR_DOZEEN(x)                         (((uint32_t)(((uint32_t)(x)) << GPT_CR_DOZEEN_SHIFT)) & GPT_CR_DOZEEN_MASK)

#define GPT_CR_STOPEN_MASK                       (0x20U)
#define GPT_CR_STOPEN_SHIFT                      (5U)
/*! STOPEN - GPT Stop Mode Enable
 *  0b0..Disable in Stop mode
 *  0b1..Enable in Stop mode
 */
#define GPT_CR_STOPEN(x)                         (((uint32_t)(((uint32_t)(x)) << GPT_CR_STOPEN_SHIFT)) & GPT_CR_STOPEN_MASK)

#define GPT_CR_CLKSRC_MASK                       (0x1C0U)
#define GPT_CR_CLKSRC_SHIFT                      (6U)
/*! CLKSRC - Clock Source Select
 *  0b000..No clock
 *  0b001..Peripheral clock (MODULE_CLK)
 *  0b010..High-frequency reference clock (ipg_clk_highfreq)
 *  0b011..External clock
 *  0b100..Low-frequency reference clock (ipg_clk_32k)
 */
#define GPT_CR_CLKSRC(x)                         (((uint32_t)(((uint32_t)(x)) << GPT_CR_CLKSRC_SHIFT)) & GPT_CR_CLKSRC_MASK)

#define GPT_CR_FRR_MASK                          (0x200U)
#define GPT_CR_FRR_SHIFT                         (9U)
/*! FRR - Free-Run or Restart Mode
 *  0b0..Restart mode. After a compare event, the counter resets to 0000_0000h and resumes counting.
 *  0b1..Free-Run mode. After a compare event, the counter continues counting until FFFF_FFFFh and then rolls over to 0.
 */
#define GPT_CR_FRR(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_FRR_SHIFT)) & GPT_CR_FRR_MASK)

#define GPT_CR_EN_24M_MASK                       (0x400U)
#define GPT_CR_EN_24M_SHIFT                      (10U)
/*! EN_24M - Enable Oscillator Clock Input
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_CR_EN_24M(x)                         (((uint32_t)(((uint32_t)(x)) << GPT_CR_EN_24M_SHIFT)) & GPT_CR_EN_24M_MASK)

#define GPT_CR_SWR_MASK                          (0x8000U)
#define GPT_CR_SWR_SHIFT                         (15U)
/*! SWR - Software Reset
 *  0b0..GPT is not in software reset state
 *  0b1..GPT is in software reset state
 */
#define GPT_CR_SWR(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_SWR_SHIFT)) & GPT_CR_SWR_MASK)

#define GPT_CR_IM1_MASK                          (0x30000U)
#define GPT_CR_IM1_SHIFT                         (16U)
/*! IM1 - Input Capture Operating Mode for Channel 1
 *  0b00..Capture disabled
 *  0b01..Capture on rising edge only
 *  0b10..Capture on falling edge only
 *  0b11..Capture on both edges
 */
#define GPT_CR_IM1(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_IM1_SHIFT)) & GPT_CR_IM1_MASK)

#define GPT_CR_IM2_MASK                          (0xC0000U)
#define GPT_CR_IM2_SHIFT                         (18U)
/*! IM2 - Input Capture Operating Mode for Channel 2
 *  0b00..Capture disabled
 *  0b01..Capture on rising edge only
 *  0b10..Capture on falling edge only
 *  0b11..Capture on both edges
 */
#define GPT_CR_IM2(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_IM2_SHIFT)) & GPT_CR_IM2_MASK)

#define GPT_CR_OM1_MASK                          (0x700000U)
#define GPT_CR_OM1_SHIFT                         (20U)
/*! OM1 - Output Compare Operating Mode for Channel 1
 *  0b000..Output disabled. No response on the signal.
 *  0b001..Toggle output
 *  0b010..Clear output
 *  0b011..Set output
 *  0b1xx..Generate a low pulse that is one input clock cycle wide on the output signal. When OMn is first
 *         programmed as 1xx, the output is set to one immediately on the next input clock (if it was not already). The
 *         input clock here refers to the clock selected by the CLKSRC field of this register.
 */
#define GPT_CR_OM1(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_OM1_SHIFT)) & GPT_CR_OM1_MASK)

#define GPT_CR_OM2_MASK                          (0x3800000U)
#define GPT_CR_OM2_SHIFT                         (23U)
/*! OM2 - Output Compare Operating Mode for Channel 2
 *  0b000..Output disabled. No response on the signal.
 *  0b001..Toggle output
 *  0b010..Clear output
 *  0b011..Set output
 *  0b1xx..Generate a low pulse that is one input clock cycle wide on the output signal. When OMn is first
 *         programmed as 1xx, the output is set to one immediately on the next input clock (if it was not already). The
 *         input clock here refers to the clock selected by the CLKSRC field of this register.
 */
#define GPT_CR_OM2(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_OM2_SHIFT)) & GPT_CR_OM2_MASK)

#define GPT_CR_OM3_MASK                          (0x1C000000U)
#define GPT_CR_OM3_SHIFT                         (26U)
/*! OM3 - Output Compare Operating Mode for Channel 3
 *  0b000..Output disabled. No response on the signal.
 *  0b001..Toggle output
 *  0b010..Clear output
 *  0b011..Set output
 *  0b1xx..Generate a low pulse that is one input clock cycle wide on the output signal. When OMn is first
 *         programmed as 1xx, the output is set to one immediately on the next input clock (if it was not already). The
 *         input clock here refers to the clock selected by the CLKSRC field of this register.
 */
#define GPT_CR_OM3(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_OM3_SHIFT)) & GPT_CR_OM3_MASK)

#define GPT_CR_FO1_MASK                          (0x20000000U)
#define GPT_CR_FO1_SHIFT                         (29U)
/*! FO1 - Force Output Compare for Channel 1
 *  0b0..No effect
 *  0b1..Trigger the programmed response on the output signal
 */
#define GPT_CR_FO1(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_FO1_SHIFT)) & GPT_CR_FO1_MASK)

#define GPT_CR_FO2_MASK                          (0x40000000U)
#define GPT_CR_FO2_SHIFT                         (30U)
/*! FO2 - Force Output Compare for Channel 2
 *  0b0..No effect
 *  0b1..Trigger the programmed response on the output signal
 */
#define GPT_CR_FO2(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_FO2_SHIFT)) & GPT_CR_FO2_MASK)

#define GPT_CR_FO3_MASK                          (0x80000000U)
#define GPT_CR_FO3_SHIFT                         (31U)
/*! FO3 - Force Output Compare for Channel 3
 *  0b0..No effect
 *  0b1..Trigger the programmed response on the output signal
 */
#define GPT_CR_FO3(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_CR_FO3_SHIFT)) & GPT_CR_FO3_MASK)
/*! @} */

/*! @name PR - Prescaler */
/*! @{ */

#define GPT_PR_PRESCALER_MASK                    (0xFFFU)
#define GPT_PR_PRESCALER_SHIFT                   (0U)
/*! PRESCALER - Prescaler Divide Value
 *  0b000000000000..Divide by 1
 *  0b000000000001..Divide by 2
 *  0b111111111111..Divide by 4096
 */
#define GPT_PR_PRESCALER(x)                      (((uint32_t)(((uint32_t)(x)) << GPT_PR_PRESCALER_SHIFT)) & GPT_PR_PRESCALER_MASK)

#define GPT_PR_PRESCALER24M_MASK                 (0xF000U)
#define GPT_PR_PRESCALER24M_SHIFT                (12U)
/*! PRESCALER24M - Prescaler Divide Value for the Oscillator Clock
 *  0b0000..Divide by 1
 *  0b0001..Divide by 2
 *  0b1111..Divide by 16
 */
#define GPT_PR_PRESCALER24M(x)                   (((uint32_t)(((uint32_t)(x)) << GPT_PR_PRESCALER24M_SHIFT)) & GPT_PR_PRESCALER24M_MASK)
/*! @} */

/*! @name SR - Status */
/*! @{ */

#define GPT_SR_OF1_MASK                          (0x1U)
#define GPT_SR_OF1_SHIFT                         (0U)
/*! OF1 - Output Compare Flag for Channel 1
 *  0b0..Compare event has not occurred.
 *  0b1..Compare event has occurred.
 */
#define GPT_SR_OF1(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_SR_OF1_SHIFT)) & GPT_SR_OF1_MASK)

#define GPT_SR_OF2_MASK                          (0x2U)
#define GPT_SR_OF2_SHIFT                         (1U)
/*! OF2 - Output Compare Flag for Channel 2
 *  0b0..Compare event has not occurred.
 *  0b1..Compare event has occurred.
 */
#define GPT_SR_OF2(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_SR_OF2_SHIFT)) & GPT_SR_OF2_MASK)

#define GPT_SR_OF3_MASK                          (0x4U)
#define GPT_SR_OF3_SHIFT                         (2U)
/*! OF3 - Output Compare Flag for Channel 3
 *  0b0..Compare event has not occurred.
 *  0b1..Compare event has occurred.
 */
#define GPT_SR_OF3(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_SR_OF3_SHIFT)) & GPT_SR_OF3_MASK)

#define GPT_SR_IF1_MASK                          (0x8U)
#define GPT_SR_IF1_SHIFT                         (3U)
/*! IF1 - Input Capture Flag for Channel 1
 *  0b0..Capture event has not occurred.
 *  0b1..Capture event has occurred.
 */
#define GPT_SR_IF1(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_SR_IF1_SHIFT)) & GPT_SR_IF1_MASK)

#define GPT_SR_IF2_MASK                          (0x10U)
#define GPT_SR_IF2_SHIFT                         (4U)
/*! IF2 - Input Capture Flag for Channel 2
 *  0b0..Capture event has not occurred.
 *  0b1..Capture event has occurred.
 */
#define GPT_SR_IF2(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_SR_IF2_SHIFT)) & GPT_SR_IF2_MASK)

#define GPT_SR_ROV_MASK                          (0x20U)
#define GPT_SR_ROV_SHIFT                         (5U)
/*! ROV - Rollover Flag
 *  0b0..Rollover has not occurred.
 *  0b1..Rollover has occurred.
 */
#define GPT_SR_ROV(x)                            (((uint32_t)(((uint32_t)(x)) << GPT_SR_ROV_SHIFT)) & GPT_SR_ROV_MASK)
/*! @} */

/*! @name IR - Interrupt */
/*! @{ */

#define GPT_IR_OF1IE_MASK                        (0x1U)
#define GPT_IR_OF1IE_SHIFT                       (0U)
/*! OF1IE - Output Compare Flag for Channel 1 Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_IR_OF1IE(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_IR_OF1IE_SHIFT)) & GPT_IR_OF1IE_MASK)

#define GPT_IR_OF2IE_MASK                        (0x2U)
#define GPT_IR_OF2IE_SHIFT                       (1U)
/*! OF2IE - Output Compare Flag for Channel 2 Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_IR_OF2IE(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_IR_OF2IE_SHIFT)) & GPT_IR_OF2IE_MASK)

#define GPT_IR_OF3IE_MASK                        (0x4U)
#define GPT_IR_OF3IE_SHIFT                       (2U)
/*! OF3IE - Output Compare Flag for Channel 3 Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_IR_OF3IE(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_IR_OF3IE_SHIFT)) & GPT_IR_OF3IE_MASK)

#define GPT_IR_IF1IE_MASK                        (0x8U)
#define GPT_IR_IF1IE_SHIFT                       (3U)
/*! IF1IE - Input Capture Flag for Channel 1 Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_IR_IF1IE(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_IR_IF1IE_SHIFT)) & GPT_IR_IF1IE_MASK)

#define GPT_IR_IF2IE_MASK                        (0x10U)
#define GPT_IR_IF2IE_SHIFT                       (4U)
/*! IF2IE - Input Capture Flag for Channel 2 Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_IR_IF2IE(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_IR_IF2IE_SHIFT)) & GPT_IR_IF2IE_MASK)

#define GPT_IR_ROVIE_MASK                        (0x20U)
#define GPT_IR_ROVIE_SHIFT                       (5U)
/*! ROVIE - Rollover Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define GPT_IR_ROVIE(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_IR_ROVIE_SHIFT)) & GPT_IR_ROVIE_MASK)
/*! @} */

/*! @name OCR - Output Compare */
/*! @{ */

#define GPT_OCR_COMP_MASK                        (0xFFFFFFFFU)
#define GPT_OCR_COMP_SHIFT                       (0U)
/*! COMP - Compare Value */
#define GPT_OCR_COMP(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_OCR_COMP_SHIFT)) & GPT_OCR_COMP_MASK)
/*! @} */

/* The count of GPT_OCR */
#define GPT_OCR_COUNT                            (3U)

/*! @name ICR - Input Capture */
/*! @{ */

#define GPT_ICR_CAPT_MASK                        (0xFFFFFFFFU)
#define GPT_ICR_CAPT_SHIFT                       (0U)
/*! CAPT - Capture Value */
#define GPT_ICR_CAPT(x)                          (((uint32_t)(((uint32_t)(x)) << GPT_ICR_CAPT_SHIFT)) & GPT_ICR_CAPT_MASK)
/*! @} */

/* The count of GPT_ICR */
#define GPT_ICR_COUNT                            (2U)

/*! @name CNT - Counter */
/*! @{ */

#define GPT_CNT_COUNT_MASK                       (0xFFFFFFFFU)
#define GPT_CNT_COUNT_SHIFT                      (0U)
/*! COUNT - Counter Value */
#define GPT_CNT_COUNT(x)                         (((uint32_t)(((uint32_t)(x)) << GPT_CNT_COUNT_SHIFT)) & GPT_CNT_COUNT_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group GPT_Register_Masks */


/* GPT - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral GPT1 base address */
  #define GPT1_BASE                                (0x546C0000u)
  /** Peripheral GPT1 base address */
  #define GPT1_BASE_NS                             (0x446C0000u)
  /** Peripheral GPT1 base pointer */
  #define GPT1                                     ((GPT_Type *)GPT1_BASE)
  /** Peripheral GPT1 base pointer */
  #define GPT1_NS                                  ((GPT_Type *)GPT1_BASE_NS)
  /** Peripheral GPT2 base address */
  #define GPT2_BASE                                (0x52EC0000u)
  /** Peripheral GPT2 base address */
  #define GPT2_BASE_NS                             (0x42EC0000u)
  /** Peripheral GPT2 base pointer */
  #define GPT2                                     ((GPT_Type *)GPT2_BASE)
  /** Peripheral GPT2 base pointer */
  #define GPT2_NS                                  ((GPT_Type *)GPT2_BASE_NS)
  /** Array initializer of GPT peripheral base addresses */
  #define GPT_BASE_ADDRS                           { 0u, GPT1_BASE, GPT2_BASE }
  /** Array initializer of GPT peripheral base pointers */
  #define GPT_BASE_PTRS                            { (GPT_Type *)0u, GPT1, GPT2 }
  /** Array initializer of GPT peripheral base addresses */
  #define GPT_BASE_ADDRS_NS                        { 0u, GPT1_BASE_NS, GPT2_BASE_NS }
  /** Array initializer of GPT peripheral base pointers */
  #define GPT_BASE_PTRS_NS                         { (GPT_Type *)0u, GPT1_NS, GPT2_NS }
#else
  /** Peripheral GPT1 base address */
  #define GPT1_BASE                                (0x446C0000u)
  /** Peripheral GPT1 base pointer */
  #define GPT1                                     ((GPT_Type *)GPT1_BASE)
  /** Peripheral GPT2 base address */
  #define GPT2_BASE                                (0x42EC0000u)
  /** Peripheral GPT2 base pointer */
  #define GPT2                                     ((GPT_Type *)GPT2_BASE)
  /** Array initializer of GPT peripheral base addresses */
  #define GPT_BASE_ADDRS                           { 0u, GPT1_BASE, GPT2_BASE }
  /** Array initializer of GPT peripheral base pointers */
  #define GPT_BASE_PTRS                            { (GPT_Type *)0u, GPT1, GPT2 }
#endif
/** Interrupt vectors for the GPT peripheral type */
#define GPT_IRQS                                 { NotAvail_IRQn, GPT1_IRQn, GPT2_IRQn }

/*!
 * @}
 */ /* end of group GPT_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- I2S Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup I2S_Peripheral_Access_Layer I2S Peripheral Access Layer
 * @{
 */

/** I2S - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
  __IO uint32_t TCSR;                              /**< Transmit Control, offset: 0x8 */
  __IO uint32_t TCR1;                              /**< Transmit Configuration 1, offset: 0xC */
  __IO uint32_t TCR2;                              /**< Transmit Configuration 2, offset: 0x10 */
  __IO uint32_t TCR3;                              /**< Transmit Configuration 3, offset: 0x14 */
  __IO uint32_t TCR4;                              /**< Transmit Configuration 4, offset: 0x18 */
  __IO uint32_t TCR5;                              /**< Transmit Configuration 5, offset: 0x1C */
  __O  uint32_t TDR[4];                            /**< Transmit Data, array offset: 0x20, array step: 0x4, irregular array, not all indices are valid */
       uint8_t RESERVED_0[16];
  __I  uint32_t TFR[4];                            /**< Transmit FIFO, array offset: 0x40, array step: 0x4, irregular array, not all indices are valid */
       uint8_t RESERVED_1[16];
  __IO uint32_t TMR;                               /**< Transmit Mask, offset: 0x60 */
       uint8_t RESERVED_2[36];
  __IO uint32_t RCSR;                              /**< Receive Control, offset: 0x88 */
  __IO uint32_t RCR1;                              /**< Receive Configuration 1, offset: 0x8C */
  __IO uint32_t RCR2;                              /**< Receive Configuration 2, offset: 0x90 */
  __IO uint32_t RCR3;                              /**< Receive Configuration 3, offset: 0x94 */
  __IO uint32_t RCR4;                              /**< Receive Configuration 4, offset: 0x98 */
  __IO uint32_t RCR5;                              /**< Receive Configuration 5, offset: 0x9C */
  __I  uint32_t RDR[4];                            /**< Receive Data, array offset: 0xA0, array step: 0x4, irregular array, not all indices are valid */
       uint8_t RESERVED_3[16];
  __I  uint32_t RFR[4];                            /**< Receive FIFO, array offset: 0xC0, array step: 0x4, irregular array, not all indices are valid */
       uint8_t RESERVED_4[16];
  __IO uint32_t RMR;                               /**< Receive Mask, offset: 0xE0 */
} I2S_Type;

/* ----------------------------------------------------------------------------
   -- I2S Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup I2S_Register_Masks I2S Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define I2S_VERID_FEATURE_MASK                   (0xFFFFU)
#define I2S_VERID_FEATURE_SHIFT                  (0U)
/*! FEATURE - Feature Specification Number
 *  0b0000000000000000..Standard feature set.
 */
#define I2S_VERID_FEATURE(x)                     (((uint32_t)(((uint32_t)(x)) << I2S_VERID_FEATURE_SHIFT)) & I2S_VERID_FEATURE_MASK)

#define I2S_VERID_MINOR_MASK                     (0xFF0000U)
#define I2S_VERID_MINOR_SHIFT                    (16U)
/*! MINOR - Minor Version Number */
#define I2S_VERID_MINOR(x)                       (((uint32_t)(((uint32_t)(x)) << I2S_VERID_MINOR_SHIFT)) & I2S_VERID_MINOR_MASK)

#define I2S_VERID_MAJOR_MASK                     (0xFF000000U)
#define I2S_VERID_MAJOR_SHIFT                    (24U)
/*! MAJOR - Major Version Number */
#define I2S_VERID_MAJOR(x)                       (((uint32_t)(((uint32_t)(x)) << I2S_VERID_MAJOR_SHIFT)) & I2S_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define I2S_PARAM_DATALINE_MASK                  (0xFU)
#define I2S_PARAM_DATALINE_SHIFT                 (0U)
/*! DATALINE - Number of Datalinks */
#define I2S_PARAM_DATALINE(x)                    (((uint32_t)(((uint32_t)(x)) << I2S_PARAM_DATALINE_SHIFT)) & I2S_PARAM_DATALINE_MASK)

#define I2S_PARAM_FIFO_MASK                      (0xF00U)
#define I2S_PARAM_FIFO_SHIFT                     (8U)
/*! FIFO - FIFO Size */
#define I2S_PARAM_FIFO(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_PARAM_FIFO_SHIFT)) & I2S_PARAM_FIFO_MASK)

#define I2S_PARAM_FRAME_MASK                     (0xF0000U)
#define I2S_PARAM_FRAME_SHIFT                    (16U)
/*! FRAME - Frame Size */
#define I2S_PARAM_FRAME(x)                       (((uint32_t)(((uint32_t)(x)) << I2S_PARAM_FRAME_SHIFT)) & I2S_PARAM_FRAME_MASK)
/*! @} */

/*! @name TCSR - Transmit Control */
/*! @{ */

#define I2S_TCSR_FRDE_MASK                       (0x1U)
#define I2S_TCSR_FRDE_SHIFT                      (0U)
/*! FRDE - FIFO Request DMA Enable
 *  0b0..Disables the DMA request.
 *  0b1..Enables the DMA request.
 */
#define I2S_TCSR_FRDE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FRDE_SHIFT)) & I2S_TCSR_FRDE_MASK)

#define I2S_TCSR_FWDE_MASK                       (0x2U)
#define I2S_TCSR_FWDE_SHIFT                      (1U)
/*! FWDE - FIFO Warning DMA Enable
 *  0b0..Disables the DMA warning.
 *  0b1..Enables the DMA warning.
 */
#define I2S_TCSR_FWDE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FWDE_SHIFT)) & I2S_TCSR_FWDE_MASK)

#define I2S_TCSR_FRIE_MASK                       (0x100U)
#define I2S_TCSR_FRIE_SHIFT                      (8U)
/*! FRIE - FIFO Request Interrupt Enable
 *  0b0..Disables the interrupt.
 *  0b1..Enables the interrupt.
 */
#define I2S_TCSR_FRIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FRIE_SHIFT)) & I2S_TCSR_FRIE_MASK)

#define I2S_TCSR_FWIE_MASK                       (0x200U)
#define I2S_TCSR_FWIE_SHIFT                      (9U)
/*! FWIE - FIFO Warning Interrupt Enable
 *  0b0..Disables the interrupt.
 *  0b1..Enables the interrupt.
 */
#define I2S_TCSR_FWIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FWIE_SHIFT)) & I2S_TCSR_FWIE_MASK)

#define I2S_TCSR_FEIE_MASK                       (0x400U)
#define I2S_TCSR_FEIE_SHIFT                      (10U)
/*! FEIE - FIFO Error Interrupt Enable
 *  0b0..Disables the interrupt.
 *  0b1..Enables the interrupt.
 */
#define I2S_TCSR_FEIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FEIE_SHIFT)) & I2S_TCSR_FEIE_MASK)

#define I2S_TCSR_SEIE_MASK                       (0x800U)
#define I2S_TCSR_SEIE_SHIFT                      (11U)
/*! SEIE - Sync Error Interrupt Enable
 *  0b0..Disables interrupt.
 *  0b1..Enables interrupt.
 */
#define I2S_TCSR_SEIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_SEIE_SHIFT)) & I2S_TCSR_SEIE_MASK)

#define I2S_TCSR_WSIE_MASK                       (0x1000U)
#define I2S_TCSR_WSIE_SHIFT                      (12U)
/*! WSIE - Word Start Interrupt Enable
 *  0b0..Disables interrupt.
 *  0b1..Enables interrupt.
 */
#define I2S_TCSR_WSIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_WSIE_SHIFT)) & I2S_TCSR_WSIE_MASK)

#define I2S_TCSR_FRF_MASK                        (0x10000U)
#define I2S_TCSR_FRF_SHIFT                       (16U)
/*! FRF - FIFO Request Flag
 *  0b0..Transmit FIFO watermark has not been reached.
 *  0b1..Transmit FIFO watermark has been reached.
 */
#define I2S_TCSR_FRF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FRF_SHIFT)) & I2S_TCSR_FRF_MASK)

#define I2S_TCSR_FWF_MASK                        (0x20000U)
#define I2S_TCSR_FWF_SHIFT                       (17U)
/*! FWF - FIFO Warning Flag
 *  0b0..No enabled transmit FIFO is empty.
 *  0b1..Enabled transmit FIFO is empty.
 */
#define I2S_TCSR_FWF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FWF_SHIFT)) & I2S_TCSR_FWF_MASK)

#define I2S_TCSR_FEF_MASK                        (0x40000U)
#define I2S_TCSR_FEF_SHIFT                       (18U)
/*! FEF - FIFO Error Flag
 *  0b0..Transmit underrun not detected.
 *  0b1..Transmit underrun detected.
 */
#define I2S_TCSR_FEF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FEF_SHIFT)) & I2S_TCSR_FEF_MASK)

#define I2S_TCSR_SEF_MASK                        (0x80000U)
#define I2S_TCSR_SEF_SHIFT                       (19U)
/*! SEF - Sync Error Flag
 *  0b0..Sync error not detected.
 *  0b1..Frame sync error detected.
 */
#define I2S_TCSR_SEF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_SEF_SHIFT)) & I2S_TCSR_SEF_MASK)

#define I2S_TCSR_WSF_MASK                        (0x100000U)
#define I2S_TCSR_WSF_SHIFT                       (20U)
/*! WSF - Word Start Flag
 *  0b0..Start of word not detected.
 *  0b1..Start of word detected.
 */
#define I2S_TCSR_WSF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_WSF_SHIFT)) & I2S_TCSR_WSF_MASK)

#define I2S_TCSR_SR_MASK                         (0x1000000U)
#define I2S_TCSR_SR_SHIFT                        (24U)
/*! SR - Software Reset
 *  0b0..No effect
 *  0b1..Software reset
 */
#define I2S_TCSR_SR(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_SR_SHIFT)) & I2S_TCSR_SR_MASK)

#define I2S_TCSR_FR_MASK                         (0x2000000U)
#define I2S_TCSR_FR_SHIFT                        (25U)
/*! FR - FIFO Reset
 *  0b0..No effect.
 *  0b1..FIFO reset.
 */
#define I2S_TCSR_FR(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_FR_SHIFT)) & I2S_TCSR_FR_MASK)

#define I2S_TCSR_BCE_MASK                        (0x10000000U)
#define I2S_TCSR_BCE_SHIFT                       (28U)
/*! BCE - Bit Clock Enable
 *  0b0..Transmit bit clock is disabled.
 *  0b1..Transmit bit clock is enabled.
 */
#define I2S_TCSR_BCE(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_BCE_SHIFT)) & I2S_TCSR_BCE_MASK)

#define I2S_TCSR_DBGE_MASK                       (0x20000000U)
#define I2S_TCSR_DBGE_SHIFT                      (29U)
/*! DBGE - Debug Enable
 *  0b0..Transmitter is disabled in Debug mode, after completing the current frame.
 *  0b1..Transmitter is enabled in Debug mode.
 */
#define I2S_TCSR_DBGE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_DBGE_SHIFT)) & I2S_TCSR_DBGE_MASK)

#define I2S_TCSR_STOPE_MASK                      (0x40000000U)
#define I2S_TCSR_STOPE_SHIFT                     (30U)
/*! STOPE - Stop Enable
 *  0b0..Transmitter disabled in Stop mode.
 *  0b1..Transmitter enabled in Stop mode.
 */
#define I2S_TCSR_STOPE(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_STOPE_SHIFT)) & I2S_TCSR_STOPE_MASK)

#define I2S_TCSR_TE_MASK                         (0x80000000U)
#define I2S_TCSR_TE_SHIFT                        (31U)
/*! TE - Transmitter Enable
 *  0b0..Transmitter is disabled.
 *  0b1..Transmitter is enabled, or transmitter has been disabled and has not yet reached end of frame.
 */
#define I2S_TCSR_TE(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TCSR_TE_SHIFT)) & I2S_TCSR_TE_MASK)
/*! @} */

/*! @name TCR1 - Transmit Configuration 1 */
/*! @{ */

#define I2S_TCR1_TFW_MASK                        (0x1FU)  /* Merged from fields with different position or width, of widths (4, 5), largest definition used */
#define I2S_TCR1_TFW_SHIFT                       (0U)
/*! TFW - Transmit FIFO Watermark
 *  0b00000..1 FIFO word
 *  0b00001..2 FIFO words
 *  0b00010-0b11110..(TFW +1) FIFO words
 *  0b11111..32 FIFO words
 */
#define I2S_TCR1_TFW(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR1_TFW_SHIFT)) & I2S_TCR1_TFW_MASK)  /* Merged from fields with different position or width, of widths (4, 5), largest definition used */
/*! @} */

/*! @name TCR2 - Transmit Configuration 2 */
/*! @{ */

#define I2S_TCR2_DIV_MASK                        (0xFFU)
#define I2S_TCR2_DIV_SHIFT                       (0U)
/*! DIV - Bit Clock Divide */
#define I2S_TCR2_DIV(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_DIV_SHIFT)) & I2S_TCR2_DIV_MASK)

#define I2S_TCR2_BYP_MASK                        (0x800000U)
#define I2S_TCR2_BYP_SHIFT                       (23U)
/*! BYP - Bit Clock Bypass
 *  0b0..Internal bit clock is generated from bit clock divider.
 *  0b1..Internal bit clock is divide-by-one of the audio master clock.
 */
#define I2S_TCR2_BYP(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_BYP_SHIFT)) & I2S_TCR2_BYP_MASK)

#define I2S_TCR2_BCD_MASK                        (0x1000000U)
#define I2S_TCR2_BCD_SHIFT                       (24U)
/*! BCD - Bit Clock Direction
 *  0b0..Bit clock is generated externally in Slave mode.
 *  0b1..Bit clock is generated internally in Master mode.
 */
#define I2S_TCR2_BCD(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_BCD_SHIFT)) & I2S_TCR2_BCD_MASK)

#define I2S_TCR2_BCP_MASK                        (0x2000000U)
#define I2S_TCR2_BCP_SHIFT                       (25U)
/*! BCP - Bit Clock Polarity
 *  0b0..Bit clock is active high with drive outputs on rising edge and sample inputs on falling edge.
 *  0b1..Bit clock is active low with drive outputs on falling edge and sample inputs on rising edge.
 */
#define I2S_TCR2_BCP(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_BCP_SHIFT)) & I2S_TCR2_BCP_MASK)

#define I2S_TCR2_MSEL_MASK                       (0xC000000U)
#define I2S_TCR2_MSEL_SHIFT                      (26U)
/*! MSEL - MCLK Select
 *  0b00..Bus Clock selected.
 *  0b01..Master Clock (MCLK) 1 option selected.
 *  0b10..Master Clock (MCLK) 2 option selected.
 *  0b11..Master Clock (MCLK) 3 option selected.
 */
#define I2S_TCR2_MSEL(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_MSEL_SHIFT)) & I2S_TCR2_MSEL_MASK)

#define I2S_TCR2_BCI_MASK                        (0x10000000U)
#define I2S_TCR2_BCI_SHIFT                       (28U)
/*! BCI - Bit Clock Input
 *  0b0..No effect.
 *  0b1..Internal logic is clocked as if bit clock was externally generated.
 */
#define I2S_TCR2_BCI(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_BCI_SHIFT)) & I2S_TCR2_BCI_MASK)

#define I2S_TCR2_BCS_MASK                        (0x20000000U)
#define I2S_TCR2_BCS_SHIFT                       (29U)
/*! BCS - Bit Clock Swap
 *  0b0..Use the normal bit clock source.
 *  0b1..Swap the bit clock source.
 */
#define I2S_TCR2_BCS(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_BCS_SHIFT)) & I2S_TCR2_BCS_MASK)

#define I2S_TCR2_SYNC_MASK                       (0x40000000U)
#define I2S_TCR2_SYNC_SHIFT                      (30U)
/*! SYNC - Synchronous Mode
 *  0b0..Asynchronous mode
 *  0b1..Synchronous with receiver
 */
#define I2S_TCR2_SYNC(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCR2_SYNC_SHIFT)) & I2S_TCR2_SYNC_MASK)
/*! @} */

/*! @name TCR3 - Transmit Configuration 3 */
/*! @{ */

#define I2S_TCR3_WDFL_MASK                       (0x1FU)
#define I2S_TCR3_WDFL_SHIFT                      (0U)
/*! WDFL - Word Flag Configuration */
#define I2S_TCR3_WDFL(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCR3_WDFL_SHIFT)) & I2S_TCR3_WDFL_MASK)

#define I2S_TCR3_TCE_MASK                        (0xF0000U)  /* Merged from fields with different position or width, of widths (1, 2, 4), largest definition used */
#define I2S_TCR3_TCE_SHIFT                       (16U)
/*! TCE - Transmit Channel Enable */
#define I2S_TCR3_TCE(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR3_TCE_SHIFT)) & I2S_TCR3_TCE_MASK)  /* Merged from fields with different position or width, of widths (1, 2, 4), largest definition used */

#define I2S_TCR3_CFR_MASK                        (0xF000000U)  /* Merged from fields with different position or width, of widths (2, 4), largest definition used */
#define I2S_TCR3_CFR_SHIFT                       (24U)
/*! CFR - Channel FIFO Reset */
#define I2S_TCR3_CFR(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR3_CFR_SHIFT)) & I2S_TCR3_CFR_MASK)  /* Merged from fields with different position or width, of widths (2, 4), largest definition used */
/*! @} */

/*! @name TCR4 - Transmit Configuration 4 */
/*! @{ */

#define I2S_TCR4_FSD_MASK                        (0x1U)
#define I2S_TCR4_FSD_SHIFT                       (0U)
/*! FSD - Frame Sync Direction
 *  0b0..Frame sync is generated externally in Slave mode.
 *  0b1..Frame sync is generated internally in Master mode.
 */
#define I2S_TCR4_FSD(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_FSD_SHIFT)) & I2S_TCR4_FSD_MASK)

#define I2S_TCR4_FSP_MASK                        (0x2U)
#define I2S_TCR4_FSP_SHIFT                       (1U)
/*! FSP - Frame Sync Polarity
 *  0b0..Frame sync is active high.
 *  0b1..Frame sync is active low.
 */
#define I2S_TCR4_FSP(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_FSP_SHIFT)) & I2S_TCR4_FSP_MASK)

#define I2S_TCR4_ONDEM_MASK                      (0x4U)
#define I2S_TCR4_ONDEM_SHIFT                     (2U)
/*! ONDEM - On Demand Mode
 *  0b0..Internal frame sync is generated continuously.
 *  0b1..Internal frame sync is generated when the FIFO warning flag is 0.
 */
#define I2S_TCR4_ONDEM(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_ONDEM_SHIFT)) & I2S_TCR4_ONDEM_MASK)

#define I2S_TCR4_FSE_MASK                        (0x8U)
#define I2S_TCR4_FSE_SHIFT                       (3U)
/*! FSE - Frame Sync Early
 *  0b0..Frame sync asserts with the first bit of the frame.
 *  0b1..Frame sync asserts one bit before the first bit of the frame.
 */
#define I2S_TCR4_FSE(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_FSE_SHIFT)) & I2S_TCR4_FSE_MASK)

#define I2S_TCR4_MF_MASK                         (0x10U)
#define I2S_TCR4_MF_SHIFT                        (4U)
/*! MF - MSB First
 *  0b0..LSB is transmitted first.
 *  0b1..MSB is transmitted first.
 */
#define I2S_TCR4_MF(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_MF_SHIFT)) & I2S_TCR4_MF_MASK)

#define I2S_TCR4_CHMOD_MASK                      (0x20U)
#define I2S_TCR4_CHMOD_SHIFT                     (5U)
/*! CHMOD - Channel Mode
 *  0b0..TDM mode, transmit data pins are 3-stated when slots are masked or channels are disabled.
 *  0b1..Output mode, transmit data pins are never 3-stated and output zero when slots are masked or channels are disabled.
 */
#define I2S_TCR4_CHMOD(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_CHMOD_SHIFT)) & I2S_TCR4_CHMOD_MASK)

#define I2S_TCR4_SYWD_MASK                       (0x1F00U)
#define I2S_TCR4_SYWD_SHIFT                      (8U)
/*! SYWD - Sync Width */
#define I2S_TCR4_SYWD(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_SYWD_SHIFT)) & I2S_TCR4_SYWD_MASK)

#define I2S_TCR4_FRSZ_MASK                       (0x1F0000U)
#define I2S_TCR4_FRSZ_SHIFT                      (16U)
/*! FRSZ - Frame size */
#define I2S_TCR4_FRSZ(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_FRSZ_SHIFT)) & I2S_TCR4_FRSZ_MASK)

#define I2S_TCR4_FPACK_MASK                      (0x3000000U)
#define I2S_TCR4_FPACK_SHIFT                     (24U)
/*! FPACK - FIFO Packing Mode
 *  0b00..FIFO packing is disabled.
 *  0b01..Reserved
 *  0b10..8-bit FIFO packing is enabled.
 *  0b11..16-bit FIFO packing is enabled.
 */
#define I2S_TCR4_FPACK(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_FPACK_SHIFT)) & I2S_TCR4_FPACK_MASK)

#define I2S_TCR4_FCOMB_MASK                      (0xC000000U)
#define I2S_TCR4_FCOMB_SHIFT                     (26U)
/*! FCOMB - FIFO Combine Mode
 *  0b00..FIFO Combine mode disabled.
 *  0b01..FIFO Combine mode enabled on FIFO reads (from transmit shift registers).
 *  0b10..FIFO Combine mode enabled on FIFO writes (by software).
 *  0b11..FIFO Combine mode enabled on FIFO reads (from transmit shift registers) and writes (by software).
 */
#define I2S_TCR4_FCOMB(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_FCOMB_SHIFT)) & I2S_TCR4_FCOMB_MASK)

#define I2S_TCR4_FCONT_MASK                      (0x10000000U)
#define I2S_TCR4_FCONT_SHIFT                     (28U)
/*! FCONT - FIFO Continue on Error
 *  0b0..On FIFO error, SAI continues from the start of the next frame after the FIFO error flag has been cleared.
 *  0b1..On FIFO error, SAI continues from the same word that caused the FIFO error to become 1 after the FIFO warning flag returns to 0.
 */
#define I2S_TCR4_FCONT(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_TCR4_FCONT_SHIFT)) & I2S_TCR4_FCONT_MASK)
/*! @} */

/*! @name TCR5 - Transmit Configuration 5 */
/*! @{ */

#define I2S_TCR5_FBT_MASK                        (0x1F00U)
#define I2S_TCR5_FBT_SHIFT                       (8U)
/*! FBT - First Bit Shifted
 *  0b00000..Bit index is 0.
 *  0b00001-0b11110..Bit index is FBT value.
 *  0b11111..Bit index is 31.
 */
#define I2S_TCR5_FBT(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR5_FBT_SHIFT)) & I2S_TCR5_FBT_MASK)

#define I2S_TCR5_W0W_MASK                        (0x1F0000U)
#define I2S_TCR5_W0W_SHIFT                       (16U)
/*! W0W - Word 0 Width
 *  0b00111..8 bits per word
 *  0b01000..9 bits per word
 *  0b01001-0b11110..(W0W value + 1) bits per word
 *  0b11111..32 bits per word
 */
#define I2S_TCR5_W0W(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR5_W0W_SHIFT)) & I2S_TCR5_W0W_MASK)

#define I2S_TCR5_WNW_MASK                        (0x1F000000U)
#define I2S_TCR5_WNW_SHIFT                       (24U)
/*! WNW - Word N Width
 *  0b00111..8 bits per word
 *  0b01000..9 bits per word
 *  0b01001-0b11110..(WNW value + 1) bits per word
 *  0b11111..32 bits per word
 */
#define I2S_TCR5_WNW(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_TCR5_WNW_SHIFT)) & I2S_TCR5_WNW_MASK)
/*! @} */

/*! @name TDR - Transmit Data */
/*! @{ */

#define I2S_TDR_TDR_MASK                         (0xFFFFFFFFU)
#define I2S_TDR_TDR_SHIFT                        (0U)
/*! TDR - Transmit Data Register */
#define I2S_TDR_TDR(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TDR_TDR_SHIFT)) & I2S_TDR_TDR_MASK)
/*! @} */

/* The count of I2S_TDR */
#define I2S_TDR_COUNT                            (4U)

/*! @name TFR - Transmit FIFO */
/*! @{ */

#define I2S_TFR_RFP_MASK                         (0x3FU)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */
#define I2S_TFR_RFP_SHIFT                        (0U)
/*! RFP - Read FIFO Pointer */
#define I2S_TFR_RFP(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TFR_RFP_SHIFT)) & I2S_TFR_RFP_MASK)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */

#define I2S_TFR_WFP_MASK                         (0x3F0000U)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */
#define I2S_TFR_WFP_SHIFT                        (16U)
/*! WFP - Write FIFO Pointer */
#define I2S_TFR_WFP(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TFR_WFP_SHIFT)) & I2S_TFR_WFP_MASK)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */

#define I2S_TFR_WCP_MASK                         (0x80000000U)
#define I2S_TFR_WCP_SHIFT                        (31U)
/*! WCP - Write Channel Pointer
 *  0b0..No effect
 *  0b1..FIFO Combine mode is enabled for FIFO writes and this FIFO will be written on the next FIFO write.
 */
#define I2S_TFR_WCP(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TFR_WCP_SHIFT)) & I2S_TFR_WCP_MASK)
/*! @} */

/* The count of I2S_TFR */
#define I2S_TFR_COUNT                            (4U)

/*! @name TMR - Transmit Mask */
/*! @{ */

#define I2S_TMR_TWM_MASK                         (0xFFFFFFFFU)
#define I2S_TMR_TWM_SHIFT                        (0U)
/*! TWM - Transmit Word Mask
 *  0b00000000000000000000000000000000..Word N is enabled.
 *  0b00000000000000000000000000000001..Word N is masked. The transmit data pins are 3-stated or drive zero when masked.
 */
#define I2S_TMR_TWM(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_TMR_TWM_SHIFT)) & I2S_TMR_TWM_MASK)
/*! @} */

/*! @name RCSR - Receive Control */
/*! @{ */

#define I2S_RCSR_FRDE_MASK                       (0x1U)
#define I2S_RCSR_FRDE_SHIFT                      (0U)
/*! FRDE - FIFO Request DMA Enable
 *  0b0..Disables the DMA request.
 *  0b1..Enables the DMA request.
 */
#define I2S_RCSR_FRDE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FRDE_SHIFT)) & I2S_RCSR_FRDE_MASK)

#define I2S_RCSR_FWDE_MASK                       (0x2U)
#define I2S_RCSR_FWDE_SHIFT                      (1U)
/*! FWDE - FIFO Warning DMA Enable
 *  0b0..Disables DMA warnings.
 *  0b1..Enables DMA warnings.
 */
#define I2S_RCSR_FWDE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FWDE_SHIFT)) & I2S_RCSR_FWDE_MASK)

#define I2S_RCSR_FRIE_MASK                       (0x100U)
#define I2S_RCSR_FRIE_SHIFT                      (8U)
/*! FRIE - FIFO Request Interrupt Enable
 *  0b0..Disables the interrupt.
 *  0b1..Enables the interrupt.
 */
#define I2S_RCSR_FRIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FRIE_SHIFT)) & I2S_RCSR_FRIE_MASK)

#define I2S_RCSR_FWIE_MASK                       (0x200U)
#define I2S_RCSR_FWIE_SHIFT                      (9U)
/*! FWIE - FIFO Warning Interrupt Enable
 *  0b0..Disables the interrupt.
 *  0b1..Enables the interrupt.
 */
#define I2S_RCSR_FWIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FWIE_SHIFT)) & I2S_RCSR_FWIE_MASK)

#define I2S_RCSR_FEIE_MASK                       (0x400U)
#define I2S_RCSR_FEIE_SHIFT                      (10U)
/*! FEIE - FIFO Error Interrupt Enable
 *  0b0..Disables the interrupt.
 *  0b1..Enables the interrupt.
 */
#define I2S_RCSR_FEIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FEIE_SHIFT)) & I2S_RCSR_FEIE_MASK)

#define I2S_RCSR_SEIE_MASK                       (0x800U)
#define I2S_RCSR_SEIE_SHIFT                      (11U)
/*! SEIE - Sync Error Interrupt Enable
 *  0b0..Disables interrupt.
 *  0b1..Enables interrupt.
 */
#define I2S_RCSR_SEIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_SEIE_SHIFT)) & I2S_RCSR_SEIE_MASK)

#define I2S_RCSR_WSIE_MASK                       (0x1000U)
#define I2S_RCSR_WSIE_SHIFT                      (12U)
/*! WSIE - Word Start Interrupt Enable
 *  0b0..Disables interrupt.
 *  0b1..Enables interrupt.
 */
#define I2S_RCSR_WSIE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_WSIE_SHIFT)) & I2S_RCSR_WSIE_MASK)

#define I2S_RCSR_FRF_MASK                        (0x10000U)
#define I2S_RCSR_FRF_SHIFT                       (16U)
/*! FRF - FIFO Request Flag
 *  0b0..Receive FIFO watermark not reached.
 *  0b1..Receive FIFO watermark has been reached.
 */
#define I2S_RCSR_FRF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FRF_SHIFT)) & I2S_RCSR_FRF_MASK)

#define I2S_RCSR_FWF_MASK                        (0x20000U)
#define I2S_RCSR_FWF_SHIFT                       (17U)
/*! FWF - FIFO Warning Flag
 *  0b0..No enabled receive FIFO is full.
 *  0b1..Enabled receive FIFO is full.
 */
#define I2S_RCSR_FWF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FWF_SHIFT)) & I2S_RCSR_FWF_MASK)

#define I2S_RCSR_FEF_MASK                        (0x40000U)
#define I2S_RCSR_FEF_SHIFT                       (18U)
/*! FEF - FIFO Error Flag
 *  0b0..Receive overflow not detected.
 *  0b1..Receive overflow detected.
 */
#define I2S_RCSR_FEF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FEF_SHIFT)) & I2S_RCSR_FEF_MASK)

#define I2S_RCSR_SEF_MASK                        (0x80000U)
#define I2S_RCSR_SEF_SHIFT                       (19U)
/*! SEF - Sync Error Flag
 *  0b0..Sync error not detected.
 *  0b1..Frame sync error detected.
 */
#define I2S_RCSR_SEF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_SEF_SHIFT)) & I2S_RCSR_SEF_MASK)

#define I2S_RCSR_WSF_MASK                        (0x100000U)
#define I2S_RCSR_WSF_SHIFT                       (20U)
/*! WSF - Word Start Flag
 *  0b0..Start of word not detected.
 *  0b1..Start of word detected.
 */
#define I2S_RCSR_WSF(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_WSF_SHIFT)) & I2S_RCSR_WSF_MASK)

#define I2S_RCSR_SR_MASK                         (0x1000000U)
#define I2S_RCSR_SR_SHIFT                        (24U)
/*! SR - Software Reset
 *  0b0..No effect.
 *  0b1..Software reset.
 */
#define I2S_RCSR_SR(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_SR_SHIFT)) & I2S_RCSR_SR_MASK)

#define I2S_RCSR_FR_MASK                         (0x2000000U)
#define I2S_RCSR_FR_SHIFT                        (25U)
/*! FR - FIFO Reset
 *  0b0..No effect.
 *  0b1..FIFO reset.
 */
#define I2S_RCSR_FR(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_FR_SHIFT)) & I2S_RCSR_FR_MASK)

#define I2S_RCSR_BCE_MASK                        (0x10000000U)
#define I2S_RCSR_BCE_SHIFT                       (28U)
/*! BCE - Bit Clock Enable
 *  0b0..Receive bit clock is disabled.
 *  0b1..Receive bit clock is enabled.
 */
#define I2S_RCSR_BCE(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_BCE_SHIFT)) & I2S_RCSR_BCE_MASK)

#define I2S_RCSR_DBGE_MASK                       (0x20000000U)
#define I2S_RCSR_DBGE_SHIFT                      (29U)
/*! DBGE - Debug Enable
 *  0b0..Receiver is disabled in Debug mode, after completing the current frame.
 *  0b1..Receiver is enabled in Debug mode.
 */
#define I2S_RCSR_DBGE(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_DBGE_SHIFT)) & I2S_RCSR_DBGE_MASK)

#define I2S_RCSR_STOPE_MASK                      (0x40000000U)
#define I2S_RCSR_STOPE_SHIFT                     (30U)
/*! STOPE - Stop Enable
 *  0b0..Receiver disabled in Stop mode.
 *  0b1..Receiver enabled in Stop mode.
 */
#define I2S_RCSR_STOPE(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_STOPE_SHIFT)) & I2S_RCSR_STOPE_MASK)

#define I2S_RCSR_RE_MASK                         (0x80000000U)
#define I2S_RCSR_RE_SHIFT                        (31U)
/*! RE - Receiver Enable
 *  0b0..Receiver is disabled.
 *  0b1..Receiver is enabled, or receiver has been disabled and has not yet reached end of frame.
 */
#define I2S_RCSR_RE(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RCSR_RE_SHIFT)) & I2S_RCSR_RE_MASK)
/*! @} */

/*! @name RCR1 - Receive Configuration 1 */
/*! @{ */

#define I2S_RCR1_RFW_MASK                        (0x1FU)  /* Merged from fields with different position or width, of widths (4, 5), largest definition used */
#define I2S_RCR1_RFW_SHIFT                       (0U)
/*! RFW - Receive FIFO Watermark
 *  0b00000..1 FIFO word
 *  0b00001..2 FIFO words
 *  0b00010-0b11110..(RFW value + 1) FIFO words
 *  0b11111..32 FIFO words
 */
#define I2S_RCR1_RFW(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR1_RFW_SHIFT)) & I2S_RCR1_RFW_MASK)  /* Merged from fields with different position or width, of widths (4, 5), largest definition used */
/*! @} */

/*! @name RCR2 - Receive Configuration 2 */
/*! @{ */

#define I2S_RCR2_DIV_MASK                        (0xFFU)
#define I2S_RCR2_DIV_SHIFT                       (0U)
/*! DIV - Bit Clock Divide */
#define I2S_RCR2_DIV(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_DIV_SHIFT)) & I2S_RCR2_DIV_MASK)

#define I2S_RCR2_BYP_MASK                        (0x800000U)
#define I2S_RCR2_BYP_SHIFT                       (23U)
/*! BYP - Bit Clock Bypass
 *  0b0..Internal bit clock is generated from bit clock divider.
 *  0b1..Internal bit clock is divide-by-one of the audio master clock.
 */
#define I2S_RCR2_BYP(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_BYP_SHIFT)) & I2S_RCR2_BYP_MASK)

#define I2S_RCR2_BCD_MASK                        (0x1000000U)
#define I2S_RCR2_BCD_SHIFT                       (24U)
/*! BCD - Bit Clock Direction
 *  0b0..Bit clock is generated externally in Slave mode.
 *  0b1..Bit clock is generated internally in Master mode.
 */
#define I2S_RCR2_BCD(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_BCD_SHIFT)) & I2S_RCR2_BCD_MASK)

#define I2S_RCR2_BCP_MASK                        (0x2000000U)
#define I2S_RCR2_BCP_SHIFT                       (25U)
/*! BCP - Bit Clock Polarity
 *  0b0..Bit clock is active high with drive outputs on rising edge and sample inputs on falling edge.
 *  0b1..Bit clock is active low with drive outputs on falling edge and sample inputs on rising edge.
 */
#define I2S_RCR2_BCP(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_BCP_SHIFT)) & I2S_RCR2_BCP_MASK)

#define I2S_RCR2_MSEL_MASK                       (0xC000000U)
#define I2S_RCR2_MSEL_SHIFT                      (26U)
/*! MSEL - MCLK Select
 *  0b00..Bus Clock selected.
 *  0b01..Master Clock (MCLK) 1 option selected.
 *  0b10..Master Clock (MCLK) 2 option selected.
 *  0b11..Master Clock (MCLK) 3 option selected.
 */
#define I2S_RCR2_MSEL(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_MSEL_SHIFT)) & I2S_RCR2_MSEL_MASK)

#define I2S_RCR2_BCI_MASK                        (0x10000000U)
#define I2S_RCR2_BCI_SHIFT                       (28U)
/*! BCI - Bit Clock Input
 *  0b0..No effect.
 *  0b1..Internal logic is clocked as if bit clock was externally generated.
 */
#define I2S_RCR2_BCI(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_BCI_SHIFT)) & I2S_RCR2_BCI_MASK)

#define I2S_RCR2_BCS_MASK                        (0x20000000U)
#define I2S_RCR2_BCS_SHIFT                       (29U)
/*! BCS - Bit Clock Swap
 *  0b0..Use the normal bit clock source.
 *  0b1..Swap the bit clock source.
 */
#define I2S_RCR2_BCS(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_BCS_SHIFT)) & I2S_RCR2_BCS_MASK)

#define I2S_RCR2_SYNC_MASK                       (0x40000000U)
#define I2S_RCR2_SYNC_SHIFT                      (30U)
/*! SYNC - Synchronous Mode
 *  0b0..Asynchronous mode
 *  0b1..Synchronous with transmitter
 */
#define I2S_RCR2_SYNC(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCR2_SYNC_SHIFT)) & I2S_RCR2_SYNC_MASK)
/*! @} */

/*! @name RCR3 - Receive Configuration 3 */
/*! @{ */

#define I2S_RCR3_WDFL_MASK                       (0x1FU)
#define I2S_RCR3_WDFL_SHIFT                      (0U)
/*! WDFL - Word Flag Configuration
 *  0b00000..Word 1
 *  0b00001..Word 2
 *  0b00010-0b11110..Word (WDFL value + 1)
 *  0b11111..Word 32
 */
#define I2S_RCR3_WDFL(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCR3_WDFL_SHIFT)) & I2S_RCR3_WDFL_MASK)

#define I2S_RCR3_RCE_MASK                        (0xF0000U)  /* Merged from fields with different position or width, of widths (1, 2, 4), largest definition used */
#define I2S_RCR3_RCE_SHIFT                       (16U)
/*! RCE - Receive Channel Enable */
#define I2S_RCR3_RCE(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR3_RCE_SHIFT)) & I2S_RCR3_RCE_MASK)  /* Merged from fields with different position or width, of widths (1, 2, 4), largest definition used */

#define I2S_RCR3_CFR_MASK                        (0xF000000U)  /* Merged from fields with different position or width, of widths (2, 4), largest definition used */
#define I2S_RCR3_CFR_SHIFT                       (24U)
/*! CFR - Channel FIFO Reset */
#define I2S_RCR3_CFR(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR3_CFR_SHIFT)) & I2S_RCR3_CFR_MASK)  /* Merged from fields with different position or width, of widths (2, 4), largest definition used */
/*! @} */

/*! @name RCR4 - Receive Configuration 4 */
/*! @{ */

#define I2S_RCR4_FSD_MASK                        (0x1U)
#define I2S_RCR4_FSD_SHIFT                       (0U)
/*! FSD - Frame Sync Direction
 *  0b0..Frame Sync is generated externally in Slave mode.
 *  0b1..Frame Sync is generated internally in Master mode.
 */
#define I2S_RCR4_FSD(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_FSD_SHIFT)) & I2S_RCR4_FSD_MASK)

#define I2S_RCR4_FSP_MASK                        (0x2U)
#define I2S_RCR4_FSP_SHIFT                       (1U)
/*! FSP - Frame Sync Polarity
 *  0b0..Frame sync is active high.
 *  0b1..Frame sync is active low.
 */
#define I2S_RCR4_FSP(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_FSP_SHIFT)) & I2S_RCR4_FSP_MASK)

#define I2S_RCR4_ONDEM_MASK                      (0x4U)
#define I2S_RCR4_ONDEM_SHIFT                     (2U)
/*! ONDEM - On Demand Mode
 *  0b0..Internal frame sync is generated continuously.
 *  0b1..Internal frame sync is generated when the FIFO warning flag is 0.
 */
#define I2S_RCR4_ONDEM(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_ONDEM_SHIFT)) & I2S_RCR4_ONDEM_MASK)

#define I2S_RCR4_FSE_MASK                        (0x8U)
#define I2S_RCR4_FSE_SHIFT                       (3U)
/*! FSE - Frame Sync Early
 *  0b0..Frame sync asserts with the first bit of the frame.
 *  0b1..Frame sync asserts one bit before the first bit of the frame.
 */
#define I2S_RCR4_FSE(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_FSE_SHIFT)) & I2S_RCR4_FSE_MASK)

#define I2S_RCR4_MF_MASK                         (0x10U)
#define I2S_RCR4_MF_SHIFT                        (4U)
/*! MF - MSB First
 *  0b0..LSB is received first.
 *  0b1..MSB is received first.
 */
#define I2S_RCR4_MF(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_MF_SHIFT)) & I2S_RCR4_MF_MASK)

#define I2S_RCR4_SYWD_MASK                       (0x1F00U)
#define I2S_RCR4_SYWD_SHIFT                      (8U)
/*! SYWD - Sync Width
 *  0b00000..1 bit-clock cycle
 *  0b00001..2 bit-clock cycle
 *  0b00010-0b11110..(SYWD value + 1) bit-clock cycle
 *  0b11111..32 bit-clock cycle
 */
#define I2S_RCR4_SYWD(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_SYWD_SHIFT)) & I2S_RCR4_SYWD_MASK)

#define I2S_RCR4_FRSZ_MASK                       (0x1F0000U)
#define I2S_RCR4_FRSZ_SHIFT                      (16U)
/*! FRSZ - Frame Size
 *  0b00000..1 word per frame
 *  0b00001..2 words per frame
 *  0b00010-0b11110..(FRSZ value + 1) words per frame
 *  0b11111..32 words per frame
 */
#define I2S_RCR4_FRSZ(x)                         (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_FRSZ_SHIFT)) & I2S_RCR4_FRSZ_MASK)

#define I2S_RCR4_FPACK_MASK                      (0x3000000U)
#define I2S_RCR4_FPACK_SHIFT                     (24U)
/*! FPACK - FIFO Packing Mode
 *  0b00..FIFO packing is disabled
 *  0b01..Reserved
 *  0b10..8-bit FIFO packing is enabled
 *  0b11..16-bit FIFO packing is enabled
 */
#define I2S_RCR4_FPACK(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_FPACK_SHIFT)) & I2S_RCR4_FPACK_MASK)

#define I2S_RCR4_FCOMB_MASK                      (0xC000000U)
#define I2S_RCR4_FCOMB_SHIFT                     (26U)
/*! FCOMB - FIFO Combine Mode
 *  0b00..FIFO Combine mode disabled.
 *  0b01..FIFO Combine mode enabled on FIFO writes (from receive shift registers).
 *  0b10..FIFO Combine mode enabled on FIFO reads (by software).
 *  0b11..FIFO Combine mode enabled on FIFO writes (from receive shift registers) and reads (by software).
 */
#define I2S_RCR4_FCOMB(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_FCOMB_SHIFT)) & I2S_RCR4_FCOMB_MASK)

#define I2S_RCR4_FCONT_MASK                      (0x10000000U)
#define I2S_RCR4_FCONT_SHIFT                     (28U)
/*! FCONT - FIFO Continue on Error
 *  0b0..On FIFO error, SAI continues from the start of the next frame after the FIFO error flag returns to 0.
 *  0b1..On FIFO error, SAI continues from the same word that caused the FIFO error to become 1 after the FIFO warning flag returns to 0.
 */
#define I2S_RCR4_FCONT(x)                        (((uint32_t)(((uint32_t)(x)) << I2S_RCR4_FCONT_SHIFT)) & I2S_RCR4_FCONT_MASK)
/*! @} */

/*! @name RCR5 - Receive Configuration 5 */
/*! @{ */

#define I2S_RCR5_FBT_MASK                        (0x1F00U)
#define I2S_RCR5_FBT_SHIFT                       (8U)
/*! FBT - First Bit Shifted
 *  0b00000..Bit index is 0.
 *  0b00001-0b11110..Bit index is FBT value.
 *  0b11111..Bit index is 31.
 */
#define I2S_RCR5_FBT(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR5_FBT_SHIFT)) & I2S_RCR5_FBT_MASK)

#define I2S_RCR5_W0W_MASK                        (0x1F0000U)
#define I2S_RCR5_W0W_SHIFT                       (16U)
/*! W0W - Word 0 Width
 *  0b00000..1 bit per word
 *  0b00001..2 bits per word
 *  0b00010-0b11110..(W0W value + 1) bits per word
 *  0b11111..32 bits per word
 */
#define I2S_RCR5_W0W(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR5_W0W_SHIFT)) & I2S_RCR5_W0W_MASK)

#define I2S_RCR5_WNW_MASK                        (0x1F000000U)
#define I2S_RCR5_WNW_SHIFT                       (24U)
/*! WNW - Word N Width
 *  0b00111..8 bits per word
 *  0b01000..9 bits per word
 *  0b01001-0b11110..(WNW value + 1) bits per word
 *  0b11111..32 bits per word
 */
#define I2S_RCR5_WNW(x)                          (((uint32_t)(((uint32_t)(x)) << I2S_RCR5_WNW_SHIFT)) & I2S_RCR5_WNW_MASK)
/*! @} */

/*! @name RDR - Receive Data */
/*! @{ */

#define I2S_RDR_RDR_MASK                         (0xFFFFFFFFU)
#define I2S_RDR_RDR_SHIFT                        (0U)
/*! RDR - Receive Data Register */
#define I2S_RDR_RDR(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RDR_RDR_SHIFT)) & I2S_RDR_RDR_MASK)
/*! @} */

/* The count of I2S_RDR */
#define I2S_RDR_COUNT                            (4U)

/*! @name RFR - Receive FIFO */
/*! @{ */

#define I2S_RFR_RFP_MASK                         (0x3FU)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */
#define I2S_RFR_RFP_SHIFT                        (0U)
/*! RFP - Read FIFO Pointer */
#define I2S_RFR_RFP(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RFR_RFP_SHIFT)) & I2S_RFR_RFP_MASK)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */

#define I2S_RFR_RCP_MASK                         (0x8000U)
#define I2S_RFR_RCP_SHIFT                        (15U)
/*! RCP - Receive Channel Pointer
 *  0b0..No effect.
 *  0b1..FIFO Combine mode is enabled for FIFO reads and this FIFO will be read on the next FIFO read.
 */
#define I2S_RFR_RCP(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RFR_RCP_SHIFT)) & I2S_RFR_RCP_MASK)

#define I2S_RFR_WFP_MASK                         (0x3F0000U)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */
#define I2S_RFR_WFP_SHIFT                        (16U)
/*! WFP - Write FIFO Pointer */
#define I2S_RFR_WFP(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RFR_WFP_SHIFT)) & I2S_RFR_WFP_MASK)  /* Merged from fields with different position or width, of widths (5, 6), largest definition used */
/*! @} */

/* The count of I2S_RFR */
#define I2S_RFR_COUNT                            (4U)

/*! @name RMR - Receive Mask */
/*! @{ */

#define I2S_RMR_RWM_MASK                         (0xFFFFFFFFU)
#define I2S_RMR_RWM_SHIFT                        (0U)
/*! RWM - Receive Word Mask
 *  0b00000000000000000000000000000000..Word N is enabled.
 *  0b00000000000000000000000000000001..Word N is masked.
 */
#define I2S_RMR_RWM(x)                           (((uint32_t)(((uint32_t)(x)) << I2S_RMR_RWM_SHIFT)) & I2S_RMR_RWM_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group I2S_Register_Masks */


/* I2S - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SAI1 base address */
  #define SAI1_BASE                                (0x543B0000u)
  /** Peripheral SAI1 base address */
  #define SAI1_BASE_NS                             (0x443B0000u)
  /** Peripheral SAI1 base pointer */
  #define SAI1                                     ((I2S_Type *)SAI1_BASE)
  /** Peripheral SAI1 base pointer */
  #define SAI1_NS                                  ((I2S_Type *)SAI1_BASE_NS)
  /** Peripheral SAI2 base address */
  #define SAI2_BASE                                (0x52BB0000u)
  /** Peripheral SAI2 base address */
  #define SAI2_BASE_NS                             (0x42BB0000u)
  /** Peripheral SAI2 base pointer */
  #define SAI2                                     ((I2S_Type *)SAI2_BASE)
  /** Peripheral SAI2 base pointer */
  #define SAI2_NS                                  ((I2S_Type *)SAI2_BASE_NS)
  /** Peripheral SAI3 base address */
  #define SAI3_BASE                                (0x52BC0000u)
  /** Peripheral SAI3 base address */
  #define SAI3_BASE_NS                             (0x42BC0000u)
  /** Peripheral SAI3 base pointer */
  #define SAI3                                     ((I2S_Type *)SAI3_BASE)
  /** Peripheral SAI3 base pointer */
  #define SAI3_NS                                  ((I2S_Type *)SAI3_BASE_NS)
  /** Peripheral SAI4 base address */
  #define SAI4_BASE                                (0x52BD0000u)
  /** Peripheral SAI4 base address */
  #define SAI4_BASE_NS                             (0x42BD0000u)
  /** Peripheral SAI4 base pointer */
  #define SAI4                                     ((I2S_Type *)SAI4_BASE)
  /** Peripheral SAI4 base pointer */
  #define SAI4_NS                                  ((I2S_Type *)SAI4_BASE_NS)
  /** Array initializer of I2S peripheral base addresses */
  #define I2S_BASE_ADDRS                           { 0u, SAI1_BASE, SAI2_BASE, SAI3_BASE, SAI4_BASE }
  /** Array initializer of I2S peripheral base pointers */
  #define I2S_BASE_PTRS                            { (I2S_Type *)0u, SAI1, SAI2, SAI3, SAI4 }
  /** Array initializer of I2S peripheral base addresses */
  #define I2S_BASE_ADDRS_NS                        { 0u, SAI1_BASE_NS, SAI2_BASE_NS, SAI3_BASE_NS, SAI4_BASE_NS }
  /** Array initializer of I2S peripheral base pointers */
  #define I2S_BASE_PTRS_NS                         { (I2S_Type *)0u, SAI1_NS, SAI2_NS, SAI3_NS, SAI4_NS }
#else
  /** Peripheral SAI1 base address */
  #define SAI1_BASE                                (0x443B0000u)
  /** Peripheral SAI1 base pointer */
  #define SAI1                                     ((I2S_Type *)SAI1_BASE)
  /** Peripheral SAI2 base address */
  #define SAI2_BASE                                (0x42BB0000u)
  /** Peripheral SAI2 base pointer */
  #define SAI2                                     ((I2S_Type *)SAI2_BASE)
  /** Peripheral SAI3 base address */
  #define SAI3_BASE                                (0x42BC0000u)
  /** Peripheral SAI3 base pointer */
  #define SAI3                                     ((I2S_Type *)SAI3_BASE)
  /** Peripheral SAI4 base address */
  #define SAI4_BASE                                (0x42BD0000u)
  /** Peripheral SAI4 base pointer */
  #define SAI4                                     ((I2S_Type *)SAI4_BASE)
  /** Array initializer of I2S peripheral base addresses */
  #define I2S_BASE_ADDRS                           { 0u, SAI1_BASE, SAI2_BASE, SAI3_BASE, SAI4_BASE }
  /** Array initializer of I2S peripheral base pointers */
  #define I2S_BASE_PTRS                            { (I2S_Type *)0u, SAI1, SAI2, SAI3, SAI4 }
#endif
/** Interrupt vectors for the I2S peripheral type */
#define I2S_RX_IRQS                              { NotAvail_IRQn, SAI1_IRQn, SAI2_IRQn, SAI3_IRQn, SAI4_IRQn }
#define I2S_TX_IRQS                              { NotAvail_IRQn, SAI1_IRQn, SAI2_IRQn, SAI3_IRQn, SAI4_IRQn }

/*!
 * @}
 */ /* end of group I2S_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- I3C Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup I3C_Peripheral_Access_Layer I3C Peripheral Access Layer
 * @{
 */

/** I3C - Register Layout Typedef */
typedef struct {
  __IO uint32_t MCONFIG;                           /**< Controller Configuration, offset: 0x0 */
  __IO uint32_t SCONFIG;                           /**< Target Configuration, offset: 0x4 */
  __IO uint32_t SSTATUS;                           /**< Target Status, offset: 0x8 */
  __IO uint32_t SCTRL;                             /**< Target Control, offset: 0xC */
  __IO uint32_t SINTSET;                           /**< Target Interrupt Set, offset: 0x10 */
  __IO uint32_t SINTCLR;                           /**< Target Interrupt Clear, offset: 0x14 */
  __I  uint32_t SINTMASKED;                        /**< Target Interrupt Mask, offset: 0x18 */
  __IO uint32_t SERRWARN;                          /**< Target Errors and Warnings, offset: 0x1C */
  __IO uint32_t SDMACTRL;                          /**< Target DMA Control, offset: 0x20 */
       uint8_t RESERVED_0[8];
  __IO uint32_t SDATACTRL;                         /**< Target Data Control, offset: 0x2C */
  __O  uint32_t SWDATAB;                           /**< Target Write Data Byte, offset: 0x30 */
  __O  uint32_t SWDATABE;                          /**< Target Write Data Byte End, offset: 0x34 */
  __O  uint32_t SWDATAH;                           /**< Target Write Data Halfword, offset: 0x38 */
  __O  uint32_t SWDATAHE;                          /**< Target Write Data Halfword End, offset: 0x3C */
  __I  uint32_t SRDATAB;                           /**< Target Read Data Byte, offset: 0x40 */
       uint8_t RESERVED_1[4];
  __I  uint32_t SRDATAH;                           /**< Target Read Data Halfword, offset: 0x48 */
       uint8_t RESERVED_2[8];
  __O  uint32_t SWDATAB1;                          /**< Target Write Data Byte, offset: 0x54 */
       uint8_t RESERVED_3[4];
  __I  uint32_t SCAPABILITIES2;                    /**< Target Capabilities 2, offset: 0x5C */
  __I  uint32_t SCAPABILITIES;                     /**< Target Capabilities, offset: 0x60 */
       uint8_t RESERVED_4[4];
  __IO uint32_t SMAXLIMITS;                        /**< Target Maximum Limits, offset: 0x68 */
  __IO uint32_t SIDPARTNO;                         /**< Target ID Part Number, offset: 0x6C */
  __IO uint32_t SIDEXT;                            /**< Target ID Extension, offset: 0x70 */
  __IO uint32_t SVENDORID;                         /**< Target Vendor ID, offset: 0x74 */
  __IO uint32_t STCCLOCK;                          /**< Target Time Control Clock, offset: 0x78 */
  __I  uint32_t SMSGMAPADDR;                       /**< Target Message Map Address, offset: 0x7C */
       uint8_t RESERVED_5[4];
  __IO uint32_t MCTRL;                             /**< Controller Control, offset: 0x84 */
  __IO uint32_t MSTATUS;                           /**< Controller Status, offset: 0x88 */
  __IO uint32_t MIBIRULES;                         /**< Controller In-band Interrupt Registry and Rules, offset: 0x8C */
  __IO uint32_t MINTSET;                           /**< Controller Interrupt Set, offset: 0x90 */
  __IO uint32_t MINTCLR;                           /**< Controller Interrupt Clear, offset: 0x94 */
  __I  uint32_t MINTMASKED;                        /**< Controller Interrupt Mask, offset: 0x98 */
  __IO uint32_t MERRWARN;                          /**< Controller Errors and Warnings, offset: 0x9C */
  __IO uint32_t MDMACTRL;                          /**< Controller DMA Control, offset: 0xA0 */
       uint8_t RESERVED_6[8];
  __IO uint32_t MDATACTRL;                         /**< Controller Data Control, offset: 0xAC */
  __O  uint32_t MWDATAB;                           /**< Controller Write Data Byte, offset: 0xB0 */
  __O  uint32_t MWDATABE;                          /**< Controller Write Data Byte End, offset: 0xB4 */
  __O  uint32_t MWDATAH;                           /**< Controller Write Data Halfword, offset: 0xB8 */
  __O  uint32_t MWDATAHE;                          /**< Controller Write Data Halfword End, offset: 0xBC */
  __I  uint32_t MRDATAB;                           /**< Controller Read Data Byte, offset: 0xC0 */
       uint8_t RESERVED_7[4];
  __I  uint32_t MRDATAH;                           /**< Controller Read Data Halfword, offset: 0xC8 */
  __O  uint32_t MWDATAB1;                          /**< Controller Write Byte Data 1 (to Bus), offset: 0xCC */
  union {                                          /* offset: 0xD0 */
    __O  uint32_t MWMSG_SDR_CONTROL;                 /**< Controller Write Message Control in SDR mode, offset: 0xD0 */
    __O  uint32_t MWMSG_SDR_DATA;                    /**< Controller Write Message Data in SDR mode, offset: 0xD0 */
  };
  __I  uint32_t MRMSG_SDR;                         /**< Controller Read Message in SDR mode, offset: 0xD4 */
  union {                                          /* offset: 0xD8 */
    __O  uint32_t MWMSG_DDR_CONTROL;                 /**< Controller Write Message in DDR mode: First Control Word, offset: 0xD8 */
    __O  uint32_t MWMSG_DDR_CONTROL2;                /**< Controller Write Message in DDR Mode Control 2, offset: 0xD8 */
    __O  uint32_t MWMSG_DDR_DATA;                    /**< Controller Write Message Data in DDR mode, offset: 0xD8 */
  };
  __I  uint32_t MRMSG_DDR;                         /**< Controller Read Message in DDR mode, offset: 0xDC */
       uint8_t RESERVED_8[4];
  __IO uint32_t MDYNADDR;                          /**< Controller Dynamic Address, offset: 0xE4 */
       uint8_t RESERVED_9[24];
  __IO uint32_t SRSTACTTIME;                       /**< Timing Rules for Target Reset Recovery, offset: 0x100 */
       uint8_t RESERVED_10[8];
  __IO uint32_t SCCCMASK;                          /**< CCC Mask for Unhandled CCCs, offset: 0x10C */
  __IO uint32_t SERRWARNMASK;                      /**< Target Errors and Warnings Mask, offset: 0x110 */
       uint8_t RESERVED_11[8];
  __I  uint32_t SMAPCTRL0;                         /**< Map Feature Control 0, offset: 0x11C */
  __IO uint32_t SMAPCTRL1;                         /**< Map Feature Control 1, offset: 0x120 */
       uint8_t RESERVED_12[28];
  __IO uint32_t IBIEXT1;                           /**< Extended IBI Data 1, offset: 0x140 */
  __IO uint32_t IBIEXT2;                           /**< Extended IBI Data 2, offset: 0x144 */
} I3C_Type;

/* ----------------------------------------------------------------------------
   -- I3C Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup I3C_Register_Masks I3C Register Masks
 * @{
 */

/*! @name MCONFIG - Controller Configuration */
/*! @{ */

#define I3C_MCONFIG_MSTENA_MASK                  (0x3U)
#define I3C_MCONFIG_MSTENA_SHIFT                 (0U)
/*! MSTENA - Controller Enable
 *  0b00..CONTROLLER_OFF
 *  0b01..CONTROLLER_ON
 *  0b10..CONTROLLER_CAPABLE
 *  0b11..
 */
#define I3C_MCONFIG_MSTENA(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_MSTENA_SHIFT)) & I3C_MCONFIG_MSTENA_MASK)

#define I3C_MCONFIG_DISTO_MASK                   (0x8U)
#define I3C_MCONFIG_DISTO_SHIFT                  (3U)
/*! DISTO - Disable Timeout
 *  0b1..Disabled, if configured
 *  0b0..Enabled
 */
#define I3C_MCONFIG_DISTO(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_DISTO_SHIFT)) & I3C_MCONFIG_DISTO_MASK)

#define I3C_MCONFIG_HKEEP_MASK                   (0x30U)
#define I3C_MCONFIG_HKEEP_SHIFT                  (4U)
/*! HKEEP - High-Keeper
 *  0b00..None
 *  0b01..WIRED_IN
 *  0b10..PASSIVE_SDA
 *  0b11..PASSIVE_ON_SDA_SCL
 */
#define I3C_MCONFIG_HKEEP(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_HKEEP_SHIFT)) & I3C_MCONFIG_HKEEP_MASK)

#define I3C_MCONFIG_ODSTOP_MASK                  (0x40U)
#define I3C_MCONFIG_ODSTOP_SHIFT                 (6U)
/*! ODSTOP - Open Drain Stop
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MCONFIG_ODSTOP(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_ODSTOP_SHIFT)) & I3C_MCONFIG_ODSTOP_MASK)

#define I3C_MCONFIG_PPBAUD_MASK                  (0xF00U)
#define I3C_MCONFIG_PPBAUD_SHIFT                 (8U)
/*! PPBAUD - Push-Pull Baud Rate */
#define I3C_MCONFIG_PPBAUD(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_PPBAUD_SHIFT)) & I3C_MCONFIG_PPBAUD_MASK)

#define I3C_MCONFIG_PPLOW_MASK                   (0xF000U)
#define I3C_MCONFIG_PPLOW_SHIFT                  (12U)
/*! PPLOW - Push-Pull Low */
#define I3C_MCONFIG_PPLOW(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_PPLOW_SHIFT)) & I3C_MCONFIG_PPLOW_MASK)

#define I3C_MCONFIG_ODBAUD_MASK                  (0xFF0000U)
#define I3C_MCONFIG_ODBAUD_SHIFT                 (16U)
/*! ODBAUD - Open Drain Baud Rate */
#define I3C_MCONFIG_ODBAUD(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_ODBAUD_SHIFT)) & I3C_MCONFIG_ODBAUD_MASK)

#define I3C_MCONFIG_ODHPP_MASK                   (0x1000000U)
#define I3C_MCONFIG_ODHPP_SHIFT                  (24U)
/*! ODHPP - Open Drain High Push-Pull
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MCONFIG_ODHPP(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_ODHPP_SHIFT)) & I3C_MCONFIG_ODHPP_MASK)

#define I3C_MCONFIG_SKEW_MASK                    (0xE000000U)
#define I3C_MCONFIG_SKEW_SHIFT                   (25U)
/*! SKEW - Skew */
#define I3C_MCONFIG_SKEW(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_SKEW_SHIFT)) & I3C_MCONFIG_SKEW_MASK)

#define I3C_MCONFIG_I2CBAUD_MASK                 (0xF0000000U)
#define I3C_MCONFIG_I2CBAUD_SHIFT                (28U)
/*! I2CBAUD - I2C Baud Rate */
#define I3C_MCONFIG_I2CBAUD(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MCONFIG_I2CBAUD_SHIFT)) & I3C_MCONFIG_I2CBAUD_MASK)
/*! @} */

/*! @name SCONFIG - Target Configuration */
/*! @{ */

#define I3C_SCONFIG_SLVENA_MASK                  (0x1U)
#define I3C_SCONFIG_SLVENA_SHIFT                 (0U)
/*! SLVENA - Target Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SCONFIG_SLVENA(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SCONFIG_SLVENA_SHIFT)) & I3C_SCONFIG_SLVENA_MASK)

#define I3C_SCONFIG_NACK_MASK                    (0x2U)
#define I3C_SCONFIG_NACK_SHIFT                   (1U)
/*! NACK - Not Acknowledge
 *  0b1..Always enable NACK mode (works normally)
 *  0b0..Always disable NACK mode
 */
#define I3C_SCONFIG_NACK(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_SCONFIG_NACK_SHIFT)) & I3C_SCONFIG_NACK_MASK)

#define I3C_SCONFIG_MATCHSS_MASK                 (0x4U)
#define I3C_SCONFIG_MATCHSS_SHIFT                (2U)
/*! MATCHSS - Match Start or Stop
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SCONFIG_MATCHSS(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SCONFIG_MATCHSS_SHIFT)) & I3C_SCONFIG_MATCHSS_MASK)

#define I3C_SCONFIG_S0IGNORE_MASK                (0x8U)
#define I3C_SCONFIG_S0IGNORE_SHIFT               (3U)
/*! S0IGNORE - Ignore TE0 or TE1 Errors
 *  0b1..Ignore TE0 or TE1 errors
 *  0b0..Do not ignore TE0 or TE1 errors
 */
#define I3C_SCONFIG_S0IGNORE(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SCONFIG_S0IGNORE_SHIFT)) & I3C_SCONFIG_S0IGNORE_MASK)

#define I3C_SCONFIG_DDROK_MASK                   (0x10U)
#define I3C_SCONFIG_DDROK_SHIFT                  (4U)
/*! DDROK - Double Data Rate OK
 *  0b1..Allow HDR-DDR messaging
 *  0b0..Do not allow HDR-DDR messaging
 */
#define I3C_SCONFIG_DDROK(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SCONFIG_DDROK_SHIFT)) & I3C_SCONFIG_DDROK_MASK)

#define I3C_SCONFIG_OFFLINE_MASK                 (0x200U)
#define I3C_SCONFIG_OFFLINE_SHIFT                (9U)
/*! OFFLINE - Offline
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SCONFIG_OFFLINE(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SCONFIG_OFFLINE_SHIFT)) & I3C_SCONFIG_OFFLINE_MASK)

#define I3C_SCONFIG_SADDR_MASK                   (0xFE000000U)
#define I3C_SCONFIG_SADDR_SHIFT                  (25U)
/*! SADDR - Static Address */
#define I3C_SCONFIG_SADDR(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SCONFIG_SADDR_SHIFT)) & I3C_SCONFIG_SADDR_MASK)
/*! @} */

/*! @name SSTATUS - Target Status */
/*! @{ */

#define I3C_SSTATUS_STNOTSTOP_MASK               (0x1U)
#define I3C_SSTATUS_STNOTSTOP_SHIFT              (0U)
/*! STNOTSTOP - Status not Stop
 *  0b1..Busy
 *  0b0..In STOP condition
 */
#define I3C_SSTATUS_STNOTSTOP(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STNOTSTOP_SHIFT)) & I3C_SSTATUS_STNOTSTOP_MASK)

#define I3C_SSTATUS_STMSG_MASK                   (0x2U)
#define I3C_SSTATUS_STMSG_SHIFT                  (1U)
/*! STMSG - Status Message
 *  0b1..Busy
 *  0b0..Idle
 */
#define I3C_SSTATUS_STMSG(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STMSG_SHIFT)) & I3C_SSTATUS_STMSG_MASK)

#define I3C_SSTATUS_STCCCH_MASK                  (0x4U)
#define I3C_SSTATUS_STCCCH_SHIFT                 (2U)
/*! STCCCH - Status Common Command Code Handler
 *  0b1..Handled automatically
 *  0b0..No CCC message handled
 */
#define I3C_SSTATUS_STCCCH(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STCCCH_SHIFT)) & I3C_SSTATUS_STCCCH_MASK)

#define I3C_SSTATUS_STREQRD_MASK                 (0x8U)
#define I3C_SSTATUS_STREQRD_SHIFT                (3U)
/*! STREQRD - Status Request Read
 *  0b1..SDR read from this target or an IBI is being pushed out
 *  0b0..Not an SDR read
 */
#define I3C_SSTATUS_STREQRD(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STREQRD_SHIFT)) & I3C_SSTATUS_STREQRD_MASK)

#define I3C_SSTATUS_STREQWR_MASK                 (0x10U)
#define I3C_SSTATUS_STREQWR_SHIFT                (4U)
/*! STREQWR - Status Request Write
 *  0b1..SDR write data from the controller, but not in ENTDAA mode
 *  0b0..Not an SDR write
 */
#define I3C_SSTATUS_STREQWR(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STREQWR_SHIFT)) & I3C_SSTATUS_STREQWR_MASK)

#define I3C_SSTATUS_STDAA_MASK                   (0x20U)
#define I3C_SSTATUS_STDAA_SHIFT                  (5U)
/*! STDAA - Status Dynamic Address Assignment
 *  0b1..In ENTDAA mode
 *  0b0..Not in ENTDAA mode
 */
#define I3C_SSTATUS_STDAA(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STDAA_SHIFT)) & I3C_SSTATUS_STDAA_MASK)

#define I3C_SSTATUS_STHDR_MASK                   (0x40U)
#define I3C_SSTATUS_STHDR_SHIFT                  (6U)
/*! STHDR - Status High Data Rate
 *  0b1..I3C bus in HDR-DDR mode
 *  0b0..I3C bus not in HDR-DDR mode
 */
#define I3C_SSTATUS_STHDR(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STHDR_SHIFT)) & I3C_SSTATUS_STHDR_MASK)

#define I3C_SSTATUS_START_MASK                   (0x100U)
#define I3C_SSTATUS_START_SHIFT                  (8U)
/*! START - Start Flag
 *  0b1..Detected
 *  0b0..Not detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_START(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_START_SHIFT)) & I3C_SSTATUS_START_MASK)

#define I3C_SSTATUS_MATCHED_MASK                 (0x200U)
#define I3C_SSTATUS_MATCHED_SHIFT                (9U)
/*! MATCHED - Matched Flag
 *  0b1..Header matched
 *  0b0..Header not matched
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_MATCHED(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_MATCHED_SHIFT)) & I3C_SSTATUS_MATCHED_MASK)

#define I3C_SSTATUS_STOP_MASK                    (0x400U)
#define I3C_SSTATUS_STOP_SHIFT                   (10U)
/*! STOP - Stop Flag
 *  0b1..Stopped state detected
 *  0b0..No Stopped state detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_STOP(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_STOP_SHIFT)) & I3C_SSTATUS_STOP_MASK)

#define I3C_SSTATUS_RX_PEND_MASK                 (0x800U)
#define I3C_SSTATUS_RX_PEND_SHIFT                (11U)
/*! RX_PEND - Received Message Pending
 *  0b1..Received message pending
 *  0b0..No received message pending
 */
#define I3C_SSTATUS_RX_PEND(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_RX_PEND_SHIFT)) & I3C_SSTATUS_RX_PEND_MASK)

#define I3C_SSTATUS_TXNOTFULL_MASK               (0x1000U)
#define I3C_SSTATUS_TXNOTFULL_SHIFT              (12U)
/*! TXNOTFULL - Transmit Buffer Not Full
 *  0b1..Transmit buffer not full
 *  0b0..Transmit buffer full
 */
#define I3C_SSTATUS_TXNOTFULL(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_TXNOTFULL_SHIFT)) & I3C_SSTATUS_TXNOTFULL_MASK)

#define I3C_SSTATUS_DACHG_MASK                   (0x2000U)
#define I3C_SSTATUS_DACHG_SHIFT                  (13U)
/*! DACHG - Dynamic Address Change Flag
 *  0b1..DA change detected
 *  0b0..No DA change detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_DACHG(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_DACHG_SHIFT)) & I3C_SSTATUS_DACHG_MASK)

#define I3C_SSTATUS_CCC_MASK                     (0x4000U)
#define I3C_SSTATUS_CCC_SHIFT                    (14U)
/*! CCC - Common Command Code Flag
 *  0b1..CCC received
 *  0b0..CCC not received
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_CCC(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_CCC_SHIFT)) & I3C_SSTATUS_CCC_MASK)

#define I3C_SSTATUS_ERRWARN_MASK                 (0x8000U)
#define I3C_SSTATUS_ERRWARN_SHIFT                (15U)
/*! ERRWARN - Error Warning */
#define I3C_SSTATUS_ERRWARN(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_ERRWARN_SHIFT)) & I3C_SSTATUS_ERRWARN_MASK)

#define I3C_SSTATUS_HDRMATCH_MASK                (0x10000U)
#define I3C_SSTATUS_HDRMATCH_SHIFT               (16U)
/*! HDRMATCH - High Data Rate Command Match Flag
 *  0b1..Matched the I3C dynamic address
 *  0b0..Did not match
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_HDRMATCH(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_HDRMATCH_SHIFT)) & I3C_SSTATUS_HDRMATCH_MASK)

#define I3C_SSTATUS_CHANDLED_MASK                (0x20000U)
#define I3C_SSTATUS_CHANDLED_SHIFT               (17U)
/*! CHANDLED - Common Command Code Handled Flag
 *  0b1..CCC handling in progress
 *  0b0..CCC handling not in progress
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_CHANDLED(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_CHANDLED_SHIFT)) & I3C_SSTATUS_CHANDLED_MASK)

#define I3C_SSTATUS_EVENT_MASK                   (0x40000U)
#define I3C_SSTATUS_EVENT_SHIFT                  (18U)
/*! EVENT - Event Flag
 *  0b1..IBI, CR, or HJ occurred
 *  0b0..No event occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SSTATUS_EVENT(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_EVENT_SHIFT)) & I3C_SSTATUS_EVENT_MASK)

#define I3C_SSTATUS_SLVRST_MASK                  (0x80000U)
#define I3C_SSTATUS_SLVRST_SHIFT                 (19U)
/*! SLVRST - Target Reset Flag */
#define I3C_SSTATUS_SLVRST(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_SLVRST_SHIFT)) & I3C_SSTATUS_SLVRST_MASK)

#define I3C_SSTATUS_EVDET_MASK                   (0x300000U)
#define I3C_SSTATUS_EVDET_SHIFT                  (20U)
/*! EVDET - Event Details
 *  0b00..NONE (no event or no pending event)
 *  0b01..NO_REQUEST (request is not sent yet; either there is no START condition yet, or is waiting for Bus-Available or Bus-Idle (HJ))
 *  0b10..NACKed (not acknowledged, request sent and rejected); I3C tries again
 *  0b11..ACKed (acknowledged; request sent and accepted), so done (unless the time control data is still being sent)
 */
#define I3C_SSTATUS_EVDET(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_EVDET_SHIFT)) & I3C_SSTATUS_EVDET_MASK)

#define I3C_SSTATUS_IBIDIS_MASK                  (0x1000000U)
#define I3C_SSTATUS_IBIDIS_SHIFT                 (24U)
/*! IBIDIS - In-Band Interrupts Disable
 *  0b1..Disabled
 *  0b0..Enabled
 */
#define I3C_SSTATUS_IBIDIS(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_IBIDIS_SHIFT)) & I3C_SSTATUS_IBIDIS_MASK)

#define I3C_SSTATUS_MRDIS_MASK                   (0x2000000U)
#define I3C_SSTATUS_MRDIS_SHIFT                  (25U)
/*! MRDIS - Controller Requests Disable
 *  0b1..Disabled
 *  0b0..Enabled
 */
#define I3C_SSTATUS_MRDIS(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_MRDIS_SHIFT)) & I3C_SSTATUS_MRDIS_MASK)

#define I3C_SSTATUS_HJDIS_MASK                   (0x8000000U)
#define I3C_SSTATUS_HJDIS_SHIFT                  (27U)
/*! HJDIS - Hot-Join Disabled
 *  0b1..Disabled
 *  0b0..Enabled
 */
#define I3C_SSTATUS_HJDIS(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_HJDIS_SHIFT)) & I3C_SSTATUS_HJDIS_MASK)

#define I3C_SSTATUS_ACTSTATE_MASK                (0x30000000U)
#define I3C_SSTATUS_ACTSTATE_SHIFT               (28U)
/*! ACTSTATE - Activity State from Common Command Codes (CCC)
 *  0b00..NO_LATENCY (normal bus operations)
 *  0b01..LATENCY_1MS (1 ms of latency)
 *  0b10..LATENCY_100MS (100 ms of latency)
 *  0b11..LATENCY_10S (10 seconds of latency)
 */
#define I3C_SSTATUS_ACTSTATE(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_ACTSTATE_SHIFT)) & I3C_SSTATUS_ACTSTATE_MASK)

#define I3C_SSTATUS_TIMECTRL_MASK                (0xC0000000U)
#define I3C_SSTATUS_TIMECTRL_SHIFT               (30U)
/*! TIMECTRL - Time Control
 *  0b00..NO_TIME_CONTROL (no time control is enabled)
 *  0b01..SYNC_MODE (Synchronous mode is enabled)
 *  0b10..ASYNC_MODE (Asynchronous standard mode (0 or 1) is enabled)
 *  0b11..BOTHSYNCASYNC (both Synchronous and Asynchronous modes are enabled)
 */
#define I3C_SSTATUS_TIMECTRL(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SSTATUS_TIMECTRL_SHIFT)) & I3C_SSTATUS_TIMECTRL_MASK)
/*! @} */

/*! @name SCTRL - Target Control */
/*! @{ */

#define I3C_SCTRL_EVENT_MASK                     (0x3U)
#define I3C_SCTRL_EVENT_SHIFT                    (0U)
/*! EVENT - Event
 *  0b00..NORMAL_MODE
 *  0b01..IBI
 *  0b10..CONTROLLER_REQUEST
 *  0b11..HOT_JOIN_REQUEST
 */
#define I3C_SCTRL_EVENT(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SCTRL_EVENT_SHIFT)) & I3C_SCTRL_EVENT_MASK)

#define I3C_SCTRL_EXTDATA_MASK                   (0x8U)
#define I3C_SCTRL_EXTDATA_SHIFT                  (3U)
/*! EXTDATA - Extended Data
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SCTRL_EXTDATA(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SCTRL_EXTDATA_SHIFT)) & I3C_SCTRL_EXTDATA_MASK)

#define I3C_SCTRL_MAPIDX_MASK                    (0x10U)
#define I3C_SCTRL_MAPIDX_SHIFT                   (4U)
/*! MAPIDX - Map Index */
#define I3C_SCTRL_MAPIDX(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_SCTRL_MAPIDX_SHIFT)) & I3C_SCTRL_MAPIDX_MASK)

#define I3C_SCTRL_IBIDATA_MASK                   (0xFF00U)
#define I3C_SCTRL_IBIDATA_SHIFT                  (8U)
/*! IBIDATA - In-Band Interrupt Data */
#define I3C_SCTRL_IBIDATA(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SCTRL_IBIDATA_SHIFT)) & I3C_SCTRL_IBIDATA_MASK)

#define I3C_SCTRL_PENDINT_MASK                   (0xF0000U)
#define I3C_SCTRL_PENDINT_SHIFT                  (16U)
/*! PENDINT - Pending Interrupt */
#define I3C_SCTRL_PENDINT(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SCTRL_PENDINT_SHIFT)) & I3C_SCTRL_PENDINT_MASK)

#define I3C_SCTRL_ACTSTATE_MASK                  (0x300000U)
#define I3C_SCTRL_ACTSTATE_SHIFT                 (20U)
/*! ACTSTATE - Activity State of Target */
#define I3C_SCTRL_ACTSTATE(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SCTRL_ACTSTATE_SHIFT)) & I3C_SCTRL_ACTSTATE_MASK)

#define I3C_SCTRL_VENDINFO_MASK                  (0xFF000000U)
#define I3C_SCTRL_VENDINFO_SHIFT                 (24U)
/*! VENDINFO - Vendor Information */
#define I3C_SCTRL_VENDINFO(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SCTRL_VENDINFO_SHIFT)) & I3C_SCTRL_VENDINFO_MASK)
/*! @} */

/*! @name SINTSET - Target Interrupt Set */
/*! @{ */

#define I3C_SINTSET_START_MASK                   (0x100U)
#define I3C_SINTSET_START_SHIFT                  (8U)
/*! START - Start Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_START(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_START_SHIFT)) & I3C_SINTSET_START_MASK)

#define I3C_SINTSET_MATCHED_MASK                 (0x200U)
#define I3C_SINTSET_MATCHED_SHIFT                (9U)
/*! MATCHED - Match Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_MATCHED(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_MATCHED_SHIFT)) & I3C_SINTSET_MATCHED_MASK)

#define I3C_SINTSET_STOP_MASK                    (0x400U)
#define I3C_SINTSET_STOP_SHIFT                   (10U)
/*! STOP - Stop Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_STOP(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_STOP_SHIFT)) & I3C_SINTSET_STOP_MASK)

#define I3C_SINTSET_RXPEND_MASK                  (0x800U)
#define I3C_SINTSET_RXPEND_SHIFT                 (11U)
/*! RXPEND - Receive Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_RXPEND(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_RXPEND_SHIFT)) & I3C_SINTSET_RXPEND_MASK)

#define I3C_SINTSET_TXSEND_MASK                  (0x1000U)
#define I3C_SINTSET_TXSEND_SHIFT                 (12U)
/*! TXSEND - Transmit Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_TXSEND(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_TXSEND_SHIFT)) & I3C_SINTSET_TXSEND_MASK)

#define I3C_SINTSET_DACHG_MASK                   (0x2000U)
#define I3C_SINTSET_DACHG_SHIFT                  (13U)
/*! DACHG - Dynamic Address Change Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_DACHG(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_DACHG_SHIFT)) & I3C_SINTSET_DACHG_MASK)

#define I3C_SINTSET_CCC_MASK                     (0x4000U)
#define I3C_SINTSET_CCC_SHIFT                    (14U)
/*! CCC - Common Command Code (CCC) Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_CCC(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_CCC_SHIFT)) & I3C_SINTSET_CCC_MASK)

#define I3C_SINTSET_ERRWARN_MASK                 (0x8000U)
#define I3C_SINTSET_ERRWARN_SHIFT                (15U)
/*! ERRWARN - Error or Warning Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_ERRWARN(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_ERRWARN_SHIFT)) & I3C_SINTSET_ERRWARN_MASK)

#define I3C_SINTSET_DDRMATCHED_MASK              (0x10000U)
#define I3C_SINTSET_DDRMATCHED_SHIFT             (16U)
/*! DDRMATCHED - Double Data Rate Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_DDRMATCHED(x)                (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_DDRMATCHED_SHIFT)) & I3C_SINTSET_DDRMATCHED_MASK)

#define I3C_SINTSET_CHANDLED_MASK                (0x20000U)
#define I3C_SINTSET_CHANDLED_SHIFT               (17U)
/*! CHANDLED - Common Command Code (CCC) Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_CHANDLED(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_CHANDLED_SHIFT)) & I3C_SINTSET_CHANDLED_MASK)

#define I3C_SINTSET_EVENT_MASK                   (0x40000U)
#define I3C_SINTSET_EVENT_SHIFT                  (18U)
/*! EVENT - Event Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_EVENT(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_EVENT_SHIFT)) & I3C_SINTSET_EVENT_MASK)

#define I3C_SINTSET_SLVRST_MASK                  (0x80000U)
#define I3C_SINTSET_SLVRST_SHIFT                 (19U)
/*! SLVRST - Target Reset
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_SINTSET_SLVRST(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SINTSET_SLVRST_SHIFT)) & I3C_SINTSET_SLVRST_MASK)
/*! @} */

/*! @name SINTCLR - Target Interrupt Clear */
/*! @{ */

#define I3C_SINTCLR_START_MASK                   (0x100U)
#define I3C_SINTCLR_START_SHIFT                  (8U)
/*! START - START Interrupt Enable Clear Flag */
#define I3C_SINTCLR_START(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_START_SHIFT)) & I3C_SINTCLR_START_MASK)

#define I3C_SINTCLR_MATCHED_MASK                 (0x200U)
#define I3C_SINTCLR_MATCHED_SHIFT                (9U)
/*! MATCHED - Matched Interrupt Enable Clear Flag */
#define I3C_SINTCLR_MATCHED(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_MATCHED_SHIFT)) & I3C_SINTCLR_MATCHED_MASK)

#define I3C_SINTCLR_STOP_MASK                    (0x400U)
#define I3C_SINTCLR_STOP_SHIFT                   (10U)
/*! STOP - STOP Interrupt Enable Clear Flag */
#define I3C_SINTCLR_STOP(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_STOP_SHIFT)) & I3C_SINTCLR_STOP_MASK)

#define I3C_SINTCLR_RXPEND_MASK                  (0x800U)
#define I3C_SINTCLR_RXPEND_SHIFT                 (11U)
/*! RXPEND - RXPEND Interrupt Enable Clear Flag */
#define I3C_SINTCLR_RXPEND(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_RXPEND_SHIFT)) & I3C_SINTCLR_RXPEND_MASK)

#define I3C_SINTCLR_TXSEND_MASK                  (0x1000U)
#define I3C_SINTCLR_TXSEND_SHIFT                 (12U)
/*! TXSEND - TXSEND Interrupt Enable Clear Flag */
#define I3C_SINTCLR_TXSEND(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_TXSEND_SHIFT)) & I3C_SINTCLR_TXSEND_MASK)

#define I3C_SINTCLR_DACHG_MASK                   (0x2000U)
#define I3C_SINTCLR_DACHG_SHIFT                  (13U)
/*! DACHG - DACHG Interrupt Enable Clear Flag */
#define I3C_SINTCLR_DACHG(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_DACHG_SHIFT)) & I3C_SINTCLR_DACHG_MASK)

#define I3C_SINTCLR_CCC_MASK                     (0x4000U)
#define I3C_SINTCLR_CCC_SHIFT                    (14U)
/*! CCC - CCC Interrupt Enable Clear Flag */
#define I3C_SINTCLR_CCC(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_CCC_SHIFT)) & I3C_SINTCLR_CCC_MASK)

#define I3C_SINTCLR_ERRWARN_MASK                 (0x8000U)
#define I3C_SINTCLR_ERRWARN_SHIFT                (15U)
/*! ERRWARN - ERRWARN Interrupt Enable Clear Flag */
#define I3C_SINTCLR_ERRWARN(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_ERRWARN_SHIFT)) & I3C_SINTCLR_ERRWARN_MASK)

#define I3C_SINTCLR_DDRMATCHED_MASK              (0x10000U)
#define I3C_SINTCLR_DDRMATCHED_SHIFT             (16U)
/*! DDRMATCHED - DDRMATCHED Interrupt Enable Clear Flag */
#define I3C_SINTCLR_DDRMATCHED(x)                (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_DDRMATCHED_SHIFT)) & I3C_SINTCLR_DDRMATCHED_MASK)

#define I3C_SINTCLR_CHANDLED_MASK                (0x20000U)
#define I3C_SINTCLR_CHANDLED_SHIFT               (17U)
/*! CHANDLED - CHANDLED Interrupt Enable Clear Flag */
#define I3C_SINTCLR_CHANDLED(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_CHANDLED_SHIFT)) & I3C_SINTCLR_CHANDLED_MASK)

#define I3C_SINTCLR_EVENT_MASK                   (0x40000U)
#define I3C_SINTCLR_EVENT_SHIFT                  (18U)
/*! EVENT - EVENT Interrupt Enable Clear Flag */
#define I3C_SINTCLR_EVENT(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_EVENT_SHIFT)) & I3C_SINTCLR_EVENT_MASK)

#define I3C_SINTCLR_SLVRST_MASK                  (0x80000U)
#define I3C_SINTCLR_SLVRST_SHIFT                 (19U)
/*! SLVRST - Target Reset Flag (SLVRST Interrupt Enable Clear) */
#define I3C_SINTCLR_SLVRST(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SINTCLR_SLVRST_SHIFT)) & I3C_SINTCLR_SLVRST_MASK)
/*! @} */

/*! @name SINTMASKED - Target Interrupt Mask */
/*! @{ */

#define I3C_SINTMASKED_START_MASK                (0x100U)
#define I3C_SINTMASKED_START_SHIFT               (8U)
/*! START - START Interrupt Mask */
#define I3C_SINTMASKED_START(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_START_SHIFT)) & I3C_SINTMASKED_START_MASK)

#define I3C_SINTMASKED_MATCHED_MASK              (0x200U)
#define I3C_SINTMASKED_MATCHED_SHIFT             (9U)
/*! MATCHED - MATCHED Interrupt Mask */
#define I3C_SINTMASKED_MATCHED(x)                (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_MATCHED_SHIFT)) & I3C_SINTMASKED_MATCHED_MASK)

#define I3C_SINTMASKED_STOP_MASK                 (0x400U)
#define I3C_SINTMASKED_STOP_SHIFT                (10U)
/*! STOP - STOP Interrupt Mask */
#define I3C_SINTMASKED_STOP(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_STOP_SHIFT)) & I3C_SINTMASKED_STOP_MASK)

#define I3C_SINTMASKED_RXPEND_MASK               (0x800U)
#define I3C_SINTMASKED_RXPEND_SHIFT              (11U)
/*! RXPEND - RXPEND Interrupt Mask */
#define I3C_SINTMASKED_RXPEND(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_RXPEND_SHIFT)) & I3C_SINTMASKED_RXPEND_MASK)

#define I3C_SINTMASKED_TXSEND_MASK               (0x1000U)
#define I3C_SINTMASKED_TXSEND_SHIFT              (12U)
/*! TXSEND - TXSEND Interrupt Mask */
#define I3C_SINTMASKED_TXSEND(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_TXSEND_SHIFT)) & I3C_SINTMASKED_TXSEND_MASK)

#define I3C_SINTMASKED_DACHG_MASK                (0x2000U)
#define I3C_SINTMASKED_DACHG_SHIFT               (13U)
/*! DACHG - DACHG Interrupt Mask */
#define I3C_SINTMASKED_DACHG(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_DACHG_SHIFT)) & I3C_SINTMASKED_DACHG_MASK)

#define I3C_SINTMASKED_CCC_MASK                  (0x4000U)
#define I3C_SINTMASKED_CCC_SHIFT                 (14U)
/*! CCC - CCC Interrupt Mask */
#define I3C_SINTMASKED_CCC(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_CCC_SHIFT)) & I3C_SINTMASKED_CCC_MASK)

#define I3C_SINTMASKED_ERRWARN_MASK              (0x8000U)
#define I3C_SINTMASKED_ERRWARN_SHIFT             (15U)
/*! ERRWARN - ERRWARN Interrupt Mask */
#define I3C_SINTMASKED_ERRWARN(x)                (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_ERRWARN_SHIFT)) & I3C_SINTMASKED_ERRWARN_MASK)

#define I3C_SINTMASKED_DDRMATCHED_MASK           (0x10000U)
#define I3C_SINTMASKED_DDRMATCHED_SHIFT          (16U)
/*! DDRMATCHED - DDRMATCHED Interrupt Mask */
#define I3C_SINTMASKED_DDRMATCHED(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_DDRMATCHED_SHIFT)) & I3C_SINTMASKED_DDRMATCHED_MASK)

#define I3C_SINTMASKED_CHANDLED_MASK             (0x20000U)
#define I3C_SINTMASKED_CHANDLED_SHIFT            (17U)
/*! CHANDLED - CHANDLED Interrupt Mask */
#define I3C_SINTMASKED_CHANDLED(x)               (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_CHANDLED_SHIFT)) & I3C_SINTMASKED_CHANDLED_MASK)

#define I3C_SINTMASKED_EVENT_MASK                (0x40000U)
#define I3C_SINTMASKED_EVENT_SHIFT               (18U)
/*! EVENT - EVENT Interrupt Mask */
#define I3C_SINTMASKED_EVENT(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_EVENT_SHIFT)) & I3C_SINTMASKED_EVENT_MASK)

#define I3C_SINTMASKED_SLVRST_MASK               (0x80000U)
#define I3C_SINTMASKED_SLVRST_SHIFT              (19U)
/*! SLVRST - Target Reset Interrupt Mask */
#define I3C_SINTMASKED_SLVRST(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SINTMASKED_SLVRST_SHIFT)) & I3C_SINTMASKED_SLVRST_MASK)
/*! @} */

/*! @name SERRWARN - Target Errors and Warnings */
/*! @{ */

#define I3C_SERRWARN_ORUN_MASK                   (0x1U)
#define I3C_SERRWARN_ORUN_SHIFT                  (0U)
/*! ORUN - Overrun Error Flag
 *  0b1..Overrun error
 *  0b0..No overrun error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_ORUN(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_ORUN_SHIFT)) & I3C_SERRWARN_ORUN_MASK)

#define I3C_SERRWARN_URUN_MASK                   (0x2U)
#define I3C_SERRWARN_URUN_SHIFT                  (1U)
/*! URUN - Underrun Error Flag
 *  0b1..Underrun error
 *  0b0..No underrun error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_URUN(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_URUN_SHIFT)) & I3C_SERRWARN_URUN_MASK)

#define I3C_SERRWARN_URUNNACK_MASK               (0x4U)
#define I3C_SERRWARN_URUNNACK_SHIFT              (2U)
/*! URUNNACK - Underrun and Not Acknowledged (NACKed) Error Flag
 *  0b1..Underrun; not acknowledged error
 *  0b0..No underrun; not acknowledged error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_URUNNACK(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_URUNNACK_SHIFT)) & I3C_SERRWARN_URUNNACK_MASK)

#define I3C_SERRWARN_TERM_MASK                   (0x8U)
#define I3C_SERRWARN_TERM_SHIFT                  (3U)
/*! TERM - Terminated Error Flag
 *  0b1..Terminated error
 *  0b0..No terminated error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_TERM(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_TERM_SHIFT)) & I3C_SERRWARN_TERM_MASK)

#define I3C_SERRWARN_INVSTART_MASK               (0x10U)
#define I3C_SERRWARN_INVSTART_SHIFT              (4U)
/*! INVSTART - Invalid Start Error Flag
 *  0b1..Invalid start error
 *  0b0..No invalid start error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_INVSTART(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_INVSTART_SHIFT)) & I3C_SERRWARN_INVSTART_MASK)

#define I3C_SERRWARN_SPAR_MASK                   (0x100U)
#define I3C_SERRWARN_SPAR_SHIFT                  (8U)
/*! SPAR - SDR Parity Error Flag
 *  0b1..SDR parity error
 *  0b0..No SDR parity error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_SPAR(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_SPAR_SHIFT)) & I3C_SERRWARN_SPAR_MASK)

#define I3C_SERRWARN_HPAR_MASK                   (0x200U)
#define I3C_SERRWARN_HPAR_SHIFT                  (9U)
/*! HPAR - HDR Parity Error Flag
 *  0b1..HDR parity error
 *  0b0..No HDR parity error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_HPAR(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_HPAR_SHIFT)) & I3C_SERRWARN_HPAR_MASK)

#define I3C_SERRWARN_HCRC_MASK                   (0x400U)
#define I3C_SERRWARN_HCRC_SHIFT                  (10U)
/*! HCRC - HDR-DDR CRC Error Flag
 *  0b1..HDR-DDR CRC error occurred
 *  0b0..No HDR-DDR CRC error occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_HCRC(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_HCRC_SHIFT)) & I3C_SERRWARN_HCRC_MASK)

#define I3C_SERRWARN_S0S1_MASK                   (0x800U)
#define I3C_SERRWARN_S0S1_SHIFT                  (11U)
/*! S0S1 - TE0 or TE1 Error Flag
 *  0b1..TE0 or TE1 error occurred
 *  0b0..No TE0 or TE1 error occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_S0S1(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_S0S1_SHIFT)) & I3C_SERRWARN_S0S1_MASK)

#define I3C_SERRWARN_OREAD_MASK                  (0x10000U)
#define I3C_SERRWARN_OREAD_SHIFT                 (16U)
/*! OREAD - Over-Read Error Flag
 *  0b1..Over-read error
 *  0b0..No over-read error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_OREAD(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_OREAD_SHIFT)) & I3C_SERRWARN_OREAD_MASK)

#define I3C_SERRWARN_OWRITE_MASK                 (0x20000U)
#define I3C_SERRWARN_OWRITE_SHIFT                (17U)
/*! OWRITE - Over-Write Error Flag
 *  0b1..Overwrite error
 *  0b0..No overwrite error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_SERRWARN_OWRITE(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARN_OWRITE_SHIFT)) & I3C_SERRWARN_OWRITE_MASK)
/*! @} */

/*! @name SDMACTRL - Target DMA Control */
/*! @{ */

#define I3C_SDMACTRL_DMAFB_MASK                  (0x3U)
#define I3C_SDMACTRL_DMAFB_SHIFT                 (0U)
/*! DMAFB - DMA Read (From-Bus) Trigger
 *  0b00..DMA not used
 *  0b01..DMA enabled for one frame
 *  0b10..DMA enabled until turned off
 *  0b11..
 */
#define I3C_SDMACTRL_DMAFB(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SDMACTRL_DMAFB_SHIFT)) & I3C_SDMACTRL_DMAFB_MASK)

#define I3C_SDMACTRL_DMATB_MASK                  (0xCU)
#define I3C_SDMACTRL_DMATB_SHIFT                 (2U)
/*! DMATB - DMA Write (To-Bus) Trigger
 *  0b00..DMA not used
 *  0b01..DMA enabled for one frame
 *  0b10..DMA enabled until turned off
 *  0b11..
 */
#define I3C_SDMACTRL_DMATB(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SDMACTRL_DMATB_SHIFT)) & I3C_SDMACTRL_DMATB_MASK)

#define I3C_SDMACTRL_DMAWIDTH_MASK               (0x30U)
#define I3C_SDMACTRL_DMAWIDTH_SHIFT              (4U)
/*! DMAWIDTH - Width of DMA Operations
 *  0b00, 0b01..Byte
 *  0b10..Halfword (16 bits) (this value ensures that two bytes are available in the FIFO)
 *  0b11..
 */
#define I3C_SDMACTRL_DMAWIDTH(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SDMACTRL_DMAWIDTH_SHIFT)) & I3C_SDMACTRL_DMAWIDTH_MASK)
/*! @} */

/*! @name SDATACTRL - Target Data Control */
/*! @{ */

#define I3C_SDATACTRL_FLUSHTB_MASK               (0x1U)
#define I3C_SDATACTRL_FLUSHTB_SHIFT              (0U)
/*! FLUSHTB - Flush To-Bus Buffer or FIFO */
#define I3C_SDATACTRL_FLUSHTB(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_FLUSHTB_SHIFT)) & I3C_SDATACTRL_FLUSHTB_MASK)

#define I3C_SDATACTRL_FLUSHFB_MASK               (0x2U)
#define I3C_SDATACTRL_FLUSHFB_SHIFT              (1U)
/*! FLUSHFB - Flush From-Bus Buffer or FIFO */
#define I3C_SDATACTRL_FLUSHFB(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_FLUSHFB_SHIFT)) & I3C_SDATACTRL_FLUSHFB_MASK)

#define I3C_SDATACTRL_UNLOCK_MASK                (0x8U)
#define I3C_SDATACTRL_UNLOCK_SHIFT               (3U)
/*! UNLOCK - Unlock
 *  0b0..Cannot be changed
 *  0b1..Can be changed
 */
#define I3C_SDATACTRL_UNLOCK(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_UNLOCK_SHIFT)) & I3C_SDATACTRL_UNLOCK_MASK)

#define I3C_SDATACTRL_TXTRIG_MASK                (0x30U)
#define I3C_SDATACTRL_TXTRIG_SHIFT               (4U)
/*! TXTRIG - Transmit Trigger Level
 *  0b00..Trigger when empty
 *  0b01..Trigger when 1/4 full or less
 *  0b10..Trigger when 1/2 full or less
 *  0b11..Default (trigger when 1 less than full or less)
 */
#define I3C_SDATACTRL_TXTRIG(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_TXTRIG_SHIFT)) & I3C_SDATACTRL_TXTRIG_MASK)

#define I3C_SDATACTRL_RXTRIG_MASK                (0xC0U)
#define I3C_SDATACTRL_RXTRIG_SHIFT               (6U)
/*! RXTRIG - Receive Trigger Level
 *  0b00..Trigger when not empty
 *  0b01..Trigger when 1/4 or more full
 *  0b10..Trigger when 1/2 or more full
 *  0b11..Trigger when 3/4 or more full
 */
#define I3C_SDATACTRL_RXTRIG(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_RXTRIG_SHIFT)) & I3C_SDATACTRL_RXTRIG_MASK)

#define I3C_SDATACTRL_TXCOUNT_MASK               (0x1F0000U)
#define I3C_SDATACTRL_TXCOUNT_SHIFT              (16U)
/*! TXCOUNT - Count of Bytes in Transmit */
#define I3C_SDATACTRL_TXCOUNT(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_TXCOUNT_SHIFT)) & I3C_SDATACTRL_TXCOUNT_MASK)

#define I3C_SDATACTRL_RXCOUNT_MASK               (0x1F000000U)
#define I3C_SDATACTRL_RXCOUNT_SHIFT              (24U)
/*! RXCOUNT - Count of Bytes in Receive */
#define I3C_SDATACTRL_RXCOUNT(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_RXCOUNT_SHIFT)) & I3C_SDATACTRL_RXCOUNT_MASK)

#define I3C_SDATACTRL_TXFULL_MASK                (0x40000000U)
#define I3C_SDATACTRL_TXFULL_SHIFT               (30U)
/*! TXFULL - Transmit is Full
 *  0b1..Full
 *  0b0..Not full
 */
#define I3C_SDATACTRL_TXFULL(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_TXFULL_SHIFT)) & I3C_SDATACTRL_TXFULL_MASK)

#define I3C_SDATACTRL_RXEMPTY_MASK               (0x80000000U)
#define I3C_SDATACTRL_RXEMPTY_SHIFT              (31U)
/*! RXEMPTY - Receive is Empty
 *  0b1..Empty
 *  0b0..Not empty
 */
#define I3C_SDATACTRL_RXEMPTY(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SDATACTRL_RXEMPTY_SHIFT)) & I3C_SDATACTRL_RXEMPTY_MASK)
/*! @} */

/*! @name SWDATAB - Target Write Data Byte */
/*! @{ */

#define I3C_SWDATAB_DATA_MASK                    (0xFFU)
#define I3C_SWDATAB_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define I3C_SWDATAB_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAB_DATA_SHIFT)) & I3C_SWDATAB_DATA_MASK)

#define I3C_SWDATAB_END_MASK                     (0x100U)
#define I3C_SWDATAB_END_SHIFT                    (8U)
/*! END - End
 *  0b1..End
 *  0b0..Not the end
 */
#define I3C_SWDATAB_END(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAB_END_SHIFT)) & I3C_SWDATAB_END_MASK)

#define I3C_SWDATAB_END_ALSO_MASK                (0x10000U)
#define I3C_SWDATAB_END_ALSO_SHIFT               (16U)
/*! END_ALSO - End Also
 *  0b1..End
 *  0b0..Not the end
 */
#define I3C_SWDATAB_END_ALSO(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAB_END_ALSO_SHIFT)) & I3C_SWDATAB_END_ALSO_MASK)
/*! @} */

/*! @name SWDATABE - Target Write Data Byte End */
/*! @{ */

#define I3C_SWDATABE_DATA_MASK                   (0xFFU)
#define I3C_SWDATABE_DATA_SHIFT                  (0U)
/*! DATA - Data */
#define I3C_SWDATABE_DATA(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SWDATABE_DATA_SHIFT)) & I3C_SWDATABE_DATA_MASK)
/*! @} */

/*! @name SWDATAH - Target Write Data Halfword */
/*! @{ */

#define I3C_SWDATAH_DATA0_MASK                   (0xFFU)
#define I3C_SWDATAH_DATA0_SHIFT                  (0U)
/*! DATA0 - Data 0 */
#define I3C_SWDATAH_DATA0(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAH_DATA0_SHIFT)) & I3C_SWDATAH_DATA0_MASK)

#define I3C_SWDATAH_DATA1_MASK                   (0xFF00U)
#define I3C_SWDATAH_DATA1_SHIFT                  (8U)
/*! DATA1 - Data 1 */
#define I3C_SWDATAH_DATA1(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAH_DATA1_SHIFT)) & I3C_SWDATAH_DATA1_MASK)

#define I3C_SWDATAH_END_MASK                     (0x10000U)
#define I3C_SWDATAH_END_SHIFT                    (16U)
/*! END - End of Message
 *  0b1..End
 *  0b0..Not the end
 */
#define I3C_SWDATAH_END(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAH_END_SHIFT)) & I3C_SWDATAH_END_MASK)
/*! @} */

/*! @name SWDATAHE - Target Write Data Halfword End */
/*! @{ */

#define I3C_SWDATAHE_DATA0_MASK                  (0xFFU)
#define I3C_SWDATAHE_DATA0_SHIFT                 (0U)
/*! DATA0 - Data 0 */
#define I3C_SWDATAHE_DATA0(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAHE_DATA0_SHIFT)) & I3C_SWDATAHE_DATA0_MASK)

#define I3C_SWDATAHE_DATA1_MASK                  (0xFF00U)
#define I3C_SWDATAHE_DATA1_SHIFT                 (8U)
/*! DATA1 - Data 1 */
#define I3C_SWDATAHE_DATA1(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAHE_DATA1_SHIFT)) & I3C_SWDATAHE_DATA1_MASK)
/*! @} */

/*! @name SRDATAB - Target Read Data Byte */
/*! @{ */

#define I3C_SRDATAB_DATA0_MASK                   (0xFFU)
#define I3C_SRDATAB_DATA0_SHIFT                  (0U)
/*! DATA0 - Data 0 */
#define I3C_SRDATAB_DATA0(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SRDATAB_DATA0_SHIFT)) & I3C_SRDATAB_DATA0_MASK)
/*! @} */

/*! @name SRDATAH - Target Read Data Halfword */
/*! @{ */

#define I3C_SRDATAH_LSB_MASK                     (0xFFU)
#define I3C_SRDATAH_LSB_SHIFT                    (0U)
/*! LSB - Low Byte */
#define I3C_SRDATAH_LSB(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SRDATAH_LSB_SHIFT)) & I3C_SRDATAH_LSB_MASK)

#define I3C_SRDATAH_MSB_MASK                     (0xFF00U)
#define I3C_SRDATAH_MSB_SHIFT                    (8U)
/*! MSB - High Byte */
#define I3C_SRDATAH_MSB(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_SRDATAH_MSB_SHIFT)) & I3C_SRDATAH_MSB_MASK)
/*! @} */

/*! @name SWDATAB1 - Target Write Data Byte */
/*! @{ */

#define I3C_SWDATAB1_DATA_MASK                   (0xFFU)
#define I3C_SWDATAB1_DATA_SHIFT                  (0U)
/*! DATA - Data */
#define I3C_SWDATAB1_DATA(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SWDATAB1_DATA_SHIFT)) & I3C_SWDATAB1_DATA_MASK)
/*! @} */

/*! @name SCAPABILITIES2 - Target Capabilities 2 */
/*! @{ */

#define I3C_SCAPABILITIES2_MAPCNT_MASK           (0xFU)
#define I3C_SCAPABILITIES2_MAPCNT_SHIFT          (0U)
/*! MAPCNT - Map Count */
#define I3C_SCAPABILITIES2_MAPCNT(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_MAPCNT_SHIFT)) & I3C_SCAPABILITIES2_MAPCNT_MASK)

#define I3C_SCAPABILITIES2_I2C10B_MASK           (0x10U)
#define I3C_SCAPABILITIES2_I2C10B_SHIFT          (4U)
/*! I2C10B - I2C 10-bit Address
 *  0b0..Not supported
 *  0b1..Supported
 */
#define I3C_SCAPABILITIES2_I2C10B(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_I2C10B_SHIFT)) & I3C_SCAPABILITIES2_I2C10B_MASK)

#define I3C_SCAPABILITIES2_I2CDEVID_MASK         (0x40U)
#define I3C_SCAPABILITIES2_I2CDEVID_SHIFT        (6U)
/*! I2CDEVID - I2C Device ID
 *  0b0..Not supported
 *  0b1..Supported
 */
#define I3C_SCAPABILITIES2_I2CDEVID(x)           (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_I2CDEVID_SHIFT)) & I3C_SCAPABILITIES2_I2CDEVID_MASK)

#define I3C_SCAPABILITIES2_IBIEXT_MASK           (0x100U)
#define I3C_SCAPABILITIES2_IBIEXT_SHIFT          (8U)
/*! IBIEXT - In-Band Interrupt EXTDATA
 *  0b0..Not supported
 *  0b1..Supported
 */
#define I3C_SCAPABILITIES2_IBIEXT(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_IBIEXT_SHIFT)) & I3C_SCAPABILITIES2_IBIEXT_MASK)

#define I3C_SCAPABILITIES2_IBIXREG_MASK          (0x200U)
#define I3C_SCAPABILITIES2_IBIXREG_SHIFT         (9U)
/*! IBIXREG - In-Band Interrupt Extended Register
 *  0b0..Not supported
 *  0b1..Supported
 */
#define I3C_SCAPABILITIES2_IBIXREG(x)            (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_IBIXREG_SHIFT)) & I3C_SCAPABILITIES2_IBIXREG_MASK)

#define I3C_SCAPABILITIES2_V1_1_MASK             (0x10000U)
#define I3C_SCAPABILITIES2_V1_1_SHIFT            (16U)
/*! V1_1 - Version 1.1
 *  0b0..Not supported
 *  0b1..Supported
 */
#define I3C_SCAPABILITIES2_V1_1(x)               (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_V1_1_SHIFT)) & I3C_SCAPABILITIES2_V1_1_MASK)

#define I3C_SCAPABILITIES2_SLVRST_MASK           (0x20000U)
#define I3C_SCAPABILITIES2_SLVRST_SHIFT          (17U)
/*! SLVRST - Target Reset
 *  0b0..Not supported
 *  0b1..Supported
 */
#define I3C_SCAPABILITIES2_SLVRST(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_SLVRST_SHIFT)) & I3C_SCAPABILITIES2_SLVRST_MASK)

#define I3C_SCAPABILITIES2_GROUP_MASK            (0xC0000U)
#define I3C_SCAPABILITIES2_GROUP_SHIFT           (18U)
/*! GROUP - Group
 *  0b00..v1.1 group addressing not supported
 *  0b01..One group supported
 *  0b10..Two groups supported
 *  0b11..Three groups supported
 */
#define I3C_SCAPABILITIES2_GROUP(x)              (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_GROUP_SHIFT)) & I3C_SCAPABILITIES2_GROUP_MASK)

#define I3C_SCAPABILITIES2_AASA_MASK             (0x200000U)
#define I3C_SCAPABILITIES2_AASA_SHIFT            (21U)
/*! AASA - SETAASA
 *  0b1..SETAASA supported
 *  0b0..SETAASA not supported
 */
#define I3C_SCAPABILITIES2_AASA(x)               (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_AASA_SHIFT)) & I3C_SCAPABILITIES2_AASA_MASK)

#define I3C_SCAPABILITIES2_SSTSUB_MASK           (0x400000U)
#define I3C_SCAPABILITIES2_SSTSUB_SHIFT          (22U)
/*! SSTSUB - Target-Target(s)-Tunnel Subscriber Capable
 *  0b1..Subscriber capable
 *  0b0..Not subscriber capable
 */
#define I3C_SCAPABILITIES2_SSTSUB(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_SSTSUB_SHIFT)) & I3C_SCAPABILITIES2_SSTSUB_MASK)

#define I3C_SCAPABILITIES2_SSTWR_MASK            (0x800000U)
#define I3C_SCAPABILITIES2_SSTWR_SHIFT           (23U)
/*! SSTWR - Target-Target(s)-Tunnel Write Capable
 *  0b1..Write capable
 *  0b0..Not write capable
 */
#define I3C_SCAPABILITIES2_SSTWR(x)              (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES2_SSTWR_SHIFT)) & I3C_SCAPABILITIES2_SSTWR_MASK)
/*! @} */

/*! @name SCAPABILITIES - Target Capabilities */
/*! @{ */

#define I3C_SCAPABILITIES_IDENA_MASK             (0x3U)
#define I3C_SCAPABILITIES_IDENA_SHIFT            (0U)
/*! IDENA - ID 48b Handler
 *  0b00..Application
 *  0b01..Hardware
 *  0b10..Hardware, but the I3C module instance handles ID 48b
 *  0b11..A part number register (PARTNO)
 */
#define I3C_SCAPABILITIES_IDENA(x)               (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_IDENA_SHIFT)) & I3C_SCAPABILITIES_IDENA_MASK)

#define I3C_SCAPABILITIES_IDREG_MASK             (0x3CU)
#define I3C_SCAPABILITIES_IDREG_SHIFT            (2U)
/*! IDREG - ID Register
 *  0b0000..All ID register features disabled
 *  0bxxx1..ID Instance is a register; used if there is no PARTNO register
 *  0bxx1x..An ID Random field is available
 *  0bx1xx..A Device Characteristic Register (DCR) is available
 *  0b1xxx..A Bus Characteristics Register (BCR) is available
 */
#define I3C_SCAPABILITIES_IDREG(x)               (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_IDREG_SHIFT)) & I3C_SCAPABILITIES_IDREG_MASK)

#define I3C_SCAPABILITIES_HDRSUPP_MASK           (0xC0U)
#define I3C_SCAPABILITIES_HDRSUPP_SHIFT          (6U)
/*! HDRSUPP - High Data Rate Support
 *  0b00..No HDR modes supported
 *  0b01..DDR mode supported
 *  *..
 */
#define I3C_SCAPABILITIES_HDRSUPP(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_HDRSUPP_SHIFT)) & I3C_SCAPABILITIES_HDRSUPP_MASK)

#define I3C_SCAPABILITIES_MASTER_MASK            (0x200U)
#define I3C_SCAPABILITIES_MASTER_SHIFT           (9U)
/*! MASTER - Controller
 *  0b0..Not supported
 *  0b1..Supported
 */
#define I3C_SCAPABILITIES_MASTER(x)              (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_MASTER_SHIFT)) & I3C_SCAPABILITIES_MASTER_MASK)

#define I3C_SCAPABILITIES_SADDR_MASK             (0xC00U)
#define I3C_SCAPABILITIES_SADDR_SHIFT            (10U)
/*! SADDR - Static Address
 *  0b00..No static address
 *  0b01..Static address is fixed in hardware
 *  0b10..Hardware controls the static address dynamically (for example, from the pin strap)
 *  0b11..SCONFIG register supplies the static address
 */
#define I3C_SCAPABILITIES_SADDR(x)               (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_SADDR_SHIFT)) & I3C_SCAPABILITIES_SADDR_MASK)

#define I3C_SCAPABILITIES_CCCHANDLE_MASK         (0xF000U)
#define I3C_SCAPABILITIES_CCCHANDLE_SHIFT        (12U)
/*! CCCHANDLE - Common Command Codes Handling
 *  0b0000..All handling features disabled
 *  0bxxx1..The I3C module manages events, activities, status, HDR, and if enabled for it, ID and static-address-related items
 *  0bxx1x..The I3C module manages maximum read and write lengths, and max data speed
 *  0bx1xx..GETSTATUS CCC returns the values of SCTRL[PENDINT] and SCTRL[ACTSTATE]
 *  0b1xxx..GETSTATUS CCC returns the value of SCTRL[VENDINFO]
 */
#define I3C_SCAPABILITIES_CCCHANDLE(x)           (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_CCCHANDLE_SHIFT)) & I3C_SCAPABILITIES_CCCHANDLE_MASK)

#define I3C_SCAPABILITIES_IBI_MR_HJ_MASK         (0x1F0000U)
#define I3C_SCAPABILITIES_IBI_MR_HJ_SHIFT        (16U)
/*! IBI_MR_HJ - In-Band Interrupts, Controller Requests, Hot-Join Events
 *  0b00000..Application cannot generate IBI, CR, or HJ
 *  0bxxxx1..Application can generate an IBI
 *  0bxxx1x..When bit 0 = 1, the IBI has data from the SCTRL register
 *  0bxx1xx..Application can generate a controller request for a secondary controller
 *  0bx1xxx..Application can generate a Hot-Join event
 *  0b1xxxx..Application can use SCONFIG[BAMATCH] for bus-available timing
 */
#define I3C_SCAPABILITIES_IBI_MR_HJ(x)           (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_IBI_MR_HJ_SHIFT)) & I3C_SCAPABILITIES_IBI_MR_HJ_MASK)

#define I3C_SCAPABILITIES_TIMECTRL_MASK          (0x200000U)
#define I3C_SCAPABILITIES_TIMECTRL_SHIFT         (21U)
/*! TIMECTRL - Time Control
 *  0b0..No time control supported
 *  0b1..At least one time-control type supported
 */
#define I3C_SCAPABILITIES_TIMECTRL(x)            (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_TIMECTRL_SHIFT)) & I3C_SCAPABILITIES_TIMECTRL_MASK)

#define I3C_SCAPABILITIES_EXTFIFO_MASK           (0x3800000U)
#define I3C_SCAPABILITIES_EXTFIFO_SHIFT          (23U)
/*! EXTFIFO - External FIFO
 *  0b000..No external FIFO available
 *  0b001..Standard available or free external FIFO
 *  0b010..Request track external FIFO
 *  *..
 */
#define I3C_SCAPABILITIES_EXTFIFO(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_EXTFIFO_SHIFT)) & I3C_SCAPABILITIES_EXTFIFO_MASK)

#define I3C_SCAPABILITIES_FIFOTX_MASK            (0xC000000U)
#define I3C_SCAPABILITIES_FIFOTX_SHIFT           (26U)
/*! FIFOTX - FIFO Transmit
 *  0b00..Two
 *  0b01..Four
 *  0b10..Eight
 *  0b11..16 or larger
 */
#define I3C_SCAPABILITIES_FIFOTX(x)              (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_FIFOTX_SHIFT)) & I3C_SCAPABILITIES_FIFOTX_MASK)

#define I3C_SCAPABILITIES_FIFORX_MASK            (0x30000000U)
#define I3C_SCAPABILITIES_FIFORX_SHIFT           (28U)
/*! FIFORX - FIFO Receive
 *  0b00..Two or three
 *  0b01..Four
 *  0b10..Eight
 *  0b11..16 or larger
 */
#define I3C_SCAPABILITIES_FIFORX(x)              (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_FIFORX_SHIFT)) & I3C_SCAPABILITIES_FIFORX_MASK)

#define I3C_SCAPABILITIES_INT_MASK               (0x40000000U)
#define I3C_SCAPABILITIES_INT_SHIFT              (30U)
/*! INT - Interrupts
 *  0b1..Supported
 *  0b0..Not supported
 */
#define I3C_SCAPABILITIES_INT(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_INT_SHIFT)) & I3C_SCAPABILITIES_INT_MASK)

#define I3C_SCAPABILITIES_DMA_MASK               (0x80000000U)
#define I3C_SCAPABILITIES_DMA_SHIFT              (31U)
/*! DMA - Direct Memory Access
 *  0b1..Supported
 *  0b0..Not supported
 */
#define I3C_SCAPABILITIES_DMA(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SCAPABILITIES_DMA_SHIFT)) & I3C_SCAPABILITIES_DMA_MASK)
/*! @} */

/*! @name SMAXLIMITS - Target Maximum Limits */
/*! @{ */

#define I3C_SMAXLIMITS_MAXRD_MASK                (0xFFFU)
#define I3C_SMAXLIMITS_MAXRD_SHIFT               (0U)
/*! MAXRD - Maximum Read Length */
#define I3C_SMAXLIMITS_MAXRD(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SMAXLIMITS_MAXRD_SHIFT)) & I3C_SMAXLIMITS_MAXRD_MASK)

#define I3C_SMAXLIMITS_MAXWR_MASK                (0xFFF0000U)
#define I3C_SMAXLIMITS_MAXWR_SHIFT               (16U)
/*! MAXWR - Maximum Write Length */
#define I3C_SMAXLIMITS_MAXWR(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SMAXLIMITS_MAXWR_SHIFT)) & I3C_SMAXLIMITS_MAXWR_MASK)
/*! @} */

/*! @name SIDPARTNO - Target ID Part Number */
/*! @{ */

#define I3C_SIDPARTNO_PARTNO_MASK                (0xFFFFFFFFU)
#define I3C_SIDPARTNO_PARTNO_SHIFT               (0U)
/*! PARTNO - Part Number */
#define I3C_SIDPARTNO_PARTNO(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_SIDPARTNO_PARTNO_SHIFT)) & I3C_SIDPARTNO_PARTNO_MASK)
/*! @} */

/*! @name SIDEXT - Target ID Extension */
/*! @{ */

#define I3C_SIDEXT_DCR_MASK                      (0xFF00U)
#define I3C_SIDEXT_DCR_SHIFT                     (8U)
/*! DCR - Device Characteristic Register */
#define I3C_SIDEXT_DCR(x)                        (((uint32_t)(((uint32_t)(x)) << I3C_SIDEXT_DCR_SHIFT)) & I3C_SIDEXT_DCR_MASK)

#define I3C_SIDEXT_BCR_MASK                      (0xFF0000U)
#define I3C_SIDEXT_BCR_SHIFT                     (16U)
/*! BCR - Bus Characteristics Register */
#define I3C_SIDEXT_BCR(x)                        (((uint32_t)(((uint32_t)(x)) << I3C_SIDEXT_BCR_SHIFT)) & I3C_SIDEXT_BCR_MASK)
/*! @} */

/*! @name SVENDORID - Target Vendor ID */
/*! @{ */

#define I3C_SVENDORID_VID_MASK                   (0x7FFFU)
#define I3C_SVENDORID_VID_SHIFT                  (0U)
/*! VID - Vendor ID */
#define I3C_SVENDORID_VID(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SVENDORID_VID_SHIFT)) & I3C_SVENDORID_VID_MASK)
/*! @} */

/*! @name STCCLOCK - Target Time Control Clock */
/*! @{ */

#define I3C_STCCLOCK_ACCURACY_MASK               (0xFFU)
#define I3C_STCCLOCK_ACCURACY_SHIFT              (0U)
/*! ACCURACY - Clock Accuracy */
#define I3C_STCCLOCK_ACCURACY(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_STCCLOCK_ACCURACY_SHIFT)) & I3C_STCCLOCK_ACCURACY_MASK)

#define I3C_STCCLOCK_FREQ_MASK                   (0xFF00U)
#define I3C_STCCLOCK_FREQ_SHIFT                  (8U)
/*! FREQ - Clock Frequency */
#define I3C_STCCLOCK_FREQ(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_STCCLOCK_FREQ_SHIFT)) & I3C_STCCLOCK_FREQ_MASK)
/*! @} */

/*! @name SMSGMAPADDR - Target Message Map Address */
/*! @{ */

#define I3C_SMSGMAPADDR_MAPLAST_MASK             (0xFU)
#define I3C_SMSGMAPADDR_MAPLAST_SHIFT            (0U)
/*! MAPLAST - Matched Address Index */
#define I3C_SMSGMAPADDR_MAPLAST(x)               (((uint32_t)(((uint32_t)(x)) << I3C_SMSGMAPADDR_MAPLAST_SHIFT)) & I3C_SMSGMAPADDR_MAPLAST_MASK)

#define I3C_SMSGMAPADDR_LASTSTATIC_MASK          (0x10U)
#define I3C_SMSGMAPADDR_LASTSTATIC_SHIFT         (4U)
/*! LASTSTATIC - Last Static Address Matched
 *  0b1..I2C static address
 *  0b0..I3C dynamic address
 */
#define I3C_SMSGMAPADDR_LASTSTATIC(x)            (((uint32_t)(((uint32_t)(x)) << I3C_SMSGMAPADDR_LASTSTATIC_SHIFT)) & I3C_SMSGMAPADDR_LASTSTATIC_MASK)

#define I3C_SMSGMAPADDR_MAPLASTM1_MASK           (0xF00U)
#define I3C_SMSGMAPADDR_MAPLASTM1_SHIFT          (8U)
/*! MAPLASTM1 - Matched Previous Address Index 1 */
#define I3C_SMSGMAPADDR_MAPLASTM1(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SMSGMAPADDR_MAPLASTM1_SHIFT)) & I3C_SMSGMAPADDR_MAPLASTM1_MASK)

#define I3C_SMSGMAPADDR_MAPLASTM2_MASK           (0xF0000U)
#define I3C_SMSGMAPADDR_MAPLASTM2_SHIFT          (16U)
/*! MAPLASTM2 - Matched Previous Index 2 */
#define I3C_SMSGMAPADDR_MAPLASTM2(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SMSGMAPADDR_MAPLASTM2_SHIFT)) & I3C_SMSGMAPADDR_MAPLASTM2_MASK)
/*! @} */

/*! @name MCTRL - Controller Control */
/*! @{ */

#define I3C_MCTRL_REQUEST_MASK                   (0x7U)
#define I3C_MCTRL_REQUEST_SHIFT                  (0U)
/*! REQUEST - Request
 *  0b000..NONE
 *  0b001..EMITSTARTADDR
 *  0b010..EMITSTOP
 *  0b011..IBIACKNACK
 *  0b100..PROCESSDAA
 *  0b101..
 *  0b110..Force Exit and Target Reset
 *  0b111..AUTOIBI
 */
#define I3C_MCTRL_REQUEST(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MCTRL_REQUEST_SHIFT)) & I3C_MCTRL_REQUEST_MASK)

#define I3C_MCTRL_TYPE_MASK                      (0x30U)
#define I3C_MCTRL_TYPE_SHIFT                     (4U)
/*! TYPE - Bus Type with EmitStartAddr
 *  0b00..I3C
 *  0b01..I2C
 *  0b10..DDR
 *  0b11..
 */
#define I3C_MCTRL_TYPE(x)                        (((uint32_t)(((uint32_t)(x)) << I3C_MCTRL_TYPE_SHIFT)) & I3C_MCTRL_TYPE_MASK)

#define I3C_MCTRL_IBIRESP_MASK                   (0xC0U)
#define I3C_MCTRL_IBIRESP_SHIFT                  (6U)
/*! IBIRESP - In-Band Interrupt Response
 *  0b00..ACK (acknowledge)
 *  0b01..NACK (reject)
 *  0b10..Acknowledge with mandatory byte
 *  0b11..Manual
 */
#define I3C_MCTRL_IBIRESP(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MCTRL_IBIRESP_SHIFT)) & I3C_MCTRL_IBIRESP_MASK)

#define I3C_MCTRL_DIR_MASK                       (0x100U)
#define I3C_MCTRL_DIR_SHIFT                      (8U)
/*! DIR - Direction
 *  0b0..Write
 *  0b1..Read
 */
#define I3C_MCTRL_DIR(x)                         (((uint32_t)(((uint32_t)(x)) << I3C_MCTRL_DIR_SHIFT)) & I3C_MCTRL_DIR_MASK)

#define I3C_MCTRL_ADDR_MASK                      (0xFE00U)
#define I3C_MCTRL_ADDR_SHIFT                     (9U)
/*! ADDR - Address */
#define I3C_MCTRL_ADDR(x)                        (((uint32_t)(((uint32_t)(x)) << I3C_MCTRL_ADDR_SHIFT)) & I3C_MCTRL_ADDR_MASK)

#define I3C_MCTRL_RDTERM_MASK                    (0xFF0000U)
#define I3C_MCTRL_RDTERM_SHIFT                   (16U)
/*! RDTERM - Read Terminate Counter */
#define I3C_MCTRL_RDTERM(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_MCTRL_RDTERM_SHIFT)) & I3C_MCTRL_RDTERM_MASK)
/*! @} */

/*! @name MSTATUS - Controller Status */
/*! @{ */

#define I3C_MSTATUS_STATE_MASK                   (0x7U)
#define I3C_MSTATUS_STATE_SHIFT                  (0U)
/*! STATE - State of the Controller
 *  0b000..IDLE (bus has stopped)
 *  0b001..SLVREQ (target request)
 *  0b010..MSGSDR
 *  0b011..NORMACT
 *  0b100..MSGDDR
 *  0b101..DAA
 *  0b110..IBIACK
 *  0b111..IBIRCV
 */
#define I3C_MSTATUS_STATE(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_STATE_SHIFT)) & I3C_MSTATUS_STATE_MASK)

#define I3C_MSTATUS_BETWEEN_MASK                 (0x10U)
#define I3C_MSTATUS_BETWEEN_SHIFT                (4U)
/*! BETWEEN - Between
 *  0b0..Inactive (for other cases)
 *  0b1..Active
 */
#define I3C_MSTATUS_BETWEEN(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_BETWEEN_SHIFT)) & I3C_MSTATUS_BETWEEN_MASK)

#define I3C_MSTATUS_NACKED_MASK                  (0x20U)
#define I3C_MSTATUS_NACKED_SHIFT                 (5U)
/*! NACKED - Not Acknowledged
 *  0b1..NACKed (not acknowledged)
 *  0b0..Not NACKed
 */
#define I3C_MSTATUS_NACKED(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_NACKED_SHIFT)) & I3C_MSTATUS_NACKED_MASK)

#define I3C_MSTATUS_IBITYPE_MASK                 (0xC0U)
#define I3C_MSTATUS_IBITYPE_SHIFT                (6U)
/*! IBITYPE - In-Band Interrupt (IBI) Type
 *  0b00..NONE (no IBI: this status occurs when MSTATUS[IBIWON] becomes 0)
 *  0b01..IBI
 *  0b10..CR
 *  0b11..HJ
 */
#define I3C_MSTATUS_IBITYPE(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_IBITYPE_SHIFT)) & I3C_MSTATUS_IBITYPE_MASK)

#define I3C_MSTATUS_SLVSTART_MASK                (0x100U)
#define I3C_MSTATUS_SLVSTART_SHIFT               (8U)
/*! SLVSTART - Target Start Flag
 *  0b1..Target requesting START
 *  0b0..Target not requesting START
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MSTATUS_SLVSTART(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_SLVSTART_SHIFT)) & I3C_MSTATUS_SLVSTART_MASK)

#define I3C_MSTATUS_MCTRLDONE_MASK               (0x200U)
#define I3C_MSTATUS_MCTRLDONE_SHIFT              (9U)
/*! MCTRLDONE - Controller Control Done Flag
 *  0b1..Done
 *  0b0..Not done
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MSTATUS_MCTRLDONE(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_MCTRLDONE_SHIFT)) & I3C_MSTATUS_MCTRLDONE_MASK)

#define I3C_MSTATUS_COMPLETE_MASK                (0x400U)
#define I3C_MSTATUS_COMPLETE_SHIFT               (10U)
/*! COMPLETE - Complete Flag
 *  0b1..Complete
 *  0b0..Not complete
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MSTATUS_COMPLETE(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_COMPLETE_SHIFT)) & I3C_MSTATUS_COMPLETE_MASK)

#define I3C_MSTATUS_RXPEND_MASK                  (0x800U)
#define I3C_MSTATUS_RXPEND_SHIFT                 (11U)
/*! RXPEND - RXPEND
 *  0b1..Receive message pending
 *  0b0..No receive message pending
 */
#define I3C_MSTATUS_RXPEND(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_RXPEND_SHIFT)) & I3C_MSTATUS_RXPEND_MASK)

#define I3C_MSTATUS_TXNOTFULL_MASK               (0x1000U)
#define I3C_MSTATUS_TXNOTFULL_SHIFT              (12U)
/*! TXNOTFULL - TX Buffer or FIFO Not Full
 *  0b1..Receive buffer or FIFO not full
 *  0b0..Receive buffer or FIFO full
 */
#define I3C_MSTATUS_TXNOTFULL(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_TXNOTFULL_SHIFT)) & I3C_MSTATUS_TXNOTFULL_MASK)

#define I3C_MSTATUS_IBIWON_MASK                  (0x2000U)
#define I3C_MSTATUS_IBIWON_SHIFT                 (13U)
/*! IBIWON - In-Band Interrupt (IBI) Won Flag
 *  0b1..IBI arbitration won
 *  0b0..No IBI arbitration won
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MSTATUS_IBIWON(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_IBIWON_SHIFT)) & I3C_MSTATUS_IBIWON_MASK)

#define I3C_MSTATUS_ERRWARN_MASK                 (0x8000U)
#define I3C_MSTATUS_ERRWARN_SHIFT                (15U)
/*! ERRWARN - Error or Warning
 *  0b1..Error or warning
 *  0b0..No error or warning
 */
#define I3C_MSTATUS_ERRWARN(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_ERRWARN_SHIFT)) & I3C_MSTATUS_ERRWARN_MASK)

#define I3C_MSTATUS_NOWMASTER_MASK               (0x80000U)
#define I3C_MSTATUS_NOWMASTER_SHIFT              (19U)
/*! NOWMASTER - Module is now Controller Flag
 *  0b1..Controller
 *  0b0..Not a controller
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MSTATUS_NOWMASTER(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_NOWMASTER_SHIFT)) & I3C_MSTATUS_NOWMASTER_MASK)

#define I3C_MSTATUS_IBIADDR_MASK                 (0x7F000000U)
#define I3C_MSTATUS_IBIADDR_SHIFT                (24U)
/*! IBIADDR - IBI Address */
#define I3C_MSTATUS_IBIADDR(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MSTATUS_IBIADDR_SHIFT)) & I3C_MSTATUS_IBIADDR_MASK)
/*! @} */

/*! @name MIBIRULES - Controller In-band Interrupt Registry and Rules */
/*! @{ */

#define I3C_MIBIRULES_ADDR0_MASK                 (0x3FU)
#define I3C_MIBIRULES_ADDR0_SHIFT                (0U)
/*! ADDR0 - ADDR0 */
#define I3C_MIBIRULES_ADDR0(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MIBIRULES_ADDR0_SHIFT)) & I3C_MIBIRULES_ADDR0_MASK)

#define I3C_MIBIRULES_ADDR1_MASK                 (0xFC0U)
#define I3C_MIBIRULES_ADDR1_SHIFT                (6U)
/*! ADDR1 - ADDR1 */
#define I3C_MIBIRULES_ADDR1(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MIBIRULES_ADDR1_SHIFT)) & I3C_MIBIRULES_ADDR1_MASK)

#define I3C_MIBIRULES_ADDR2_MASK                 (0x3F000U)
#define I3C_MIBIRULES_ADDR2_SHIFT                (12U)
/*! ADDR2 - ADDR2 */
#define I3C_MIBIRULES_ADDR2(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MIBIRULES_ADDR2_SHIFT)) & I3C_MIBIRULES_ADDR2_MASK)

#define I3C_MIBIRULES_ADDR3_MASK                 (0xFC0000U)
#define I3C_MIBIRULES_ADDR3_SHIFT                (18U)
/*! ADDR3 - ADDR3 */
#define I3C_MIBIRULES_ADDR3(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MIBIRULES_ADDR3_SHIFT)) & I3C_MIBIRULES_ADDR3_MASK)

#define I3C_MIBIRULES_ADDR4_MASK                 (0x3F000000U)
#define I3C_MIBIRULES_ADDR4_SHIFT                (24U)
/*! ADDR4 - ADDR4 */
#define I3C_MIBIRULES_ADDR4(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MIBIRULES_ADDR4_SHIFT)) & I3C_MIBIRULES_ADDR4_MASK)

#define I3C_MIBIRULES_MSB0_MASK                  (0x40000000U)
#define I3C_MIBIRULES_MSB0_SHIFT                 (30U)
/*! MSB0 - Most Significant Address Bit is 0
 *  0b1..MSB is 0
 *  0b0..MSB is not 0
 */
#define I3C_MIBIRULES_MSB0(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MIBIRULES_MSB0_SHIFT)) & I3C_MIBIRULES_MSB0_MASK)

#define I3C_MIBIRULES_NOBYTE_MASK                (0x80000000U)
#define I3C_MIBIRULES_NOBYTE_SHIFT               (31U)
/*! NOBYTE - No IBI byte
 *  0b1..Without mandatory IBI byte
 *  0b0..With mandatory IBI byte
 */
#define I3C_MIBIRULES_NOBYTE(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MIBIRULES_NOBYTE_SHIFT)) & I3C_MIBIRULES_NOBYTE_MASK)
/*! @} */

/*! @name MINTSET - Controller Interrupt Set */
/*! @{ */

#define I3C_MINTSET_SLVSTART_MASK                (0x100U)
#define I3C_MINTSET_SLVSTART_SHIFT               (8U)
/*! SLVSTART - Target Start Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MINTSET_SLVSTART(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_SLVSTART_SHIFT)) & I3C_MINTSET_SLVSTART_MASK)

#define I3C_MINTSET_MCTRLDONE_MASK               (0x200U)
#define I3C_MINTSET_MCTRLDONE_SHIFT              (9U)
/*! MCTRLDONE - Controller Control Done Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MINTSET_MCTRLDONE(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_MCTRLDONE_SHIFT)) & I3C_MINTSET_MCTRLDONE_MASK)

#define I3C_MINTSET_COMPLETE_MASK                (0x400U)
#define I3C_MINTSET_COMPLETE_SHIFT               (10U)
/*! COMPLETE - Completed Message Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MINTSET_COMPLETE(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_COMPLETE_SHIFT)) & I3C_MINTSET_COMPLETE_MASK)

#define I3C_MINTSET_RXPEND_MASK                  (0x800U)
#define I3C_MINTSET_RXPEND_SHIFT                 (11U)
/*! RXPEND - Receive Pending Interrupt Enable */
#define I3C_MINTSET_RXPEND(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_RXPEND_SHIFT)) & I3C_MINTSET_RXPEND_MASK)

#define I3C_MINTSET_TXNOTFULL_MASK               (0x1000U)
#define I3C_MINTSET_TXNOTFULL_SHIFT              (12U)
/*! TXNOTFULL - Transmit Buffer/FIFO Not Full Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MINTSET_TXNOTFULL(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_TXNOTFULL_SHIFT)) & I3C_MINTSET_TXNOTFULL_MASK)

#define I3C_MINTSET_IBIWON_MASK                  (0x2000U)
#define I3C_MINTSET_IBIWON_SHIFT                 (13U)
/*! IBIWON - IBI Won Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MINTSET_IBIWON(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_IBIWON_SHIFT)) & I3C_MINTSET_IBIWON_MASK)

#define I3C_MINTSET_ERRWARN_MASK                 (0x8000U)
#define I3C_MINTSET_ERRWARN_SHIFT                (15U)
/*! ERRWARN - Error or Warning (ERRWARN) Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MINTSET_ERRWARN(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_ERRWARN_SHIFT)) & I3C_MINTSET_ERRWARN_MASK)

#define I3C_MINTSET_NOWMASTER_MASK               (0x80000U)
#define I3C_MINTSET_NOWMASTER_SHIFT              (19U)
/*! NOWMASTER - Now Controller Interrupt Enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define I3C_MINTSET_NOWMASTER(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTSET_NOWMASTER_SHIFT)) & I3C_MINTSET_NOWMASTER_MASK)
/*! @} */

/*! @name MINTCLR - Controller Interrupt Clear */
/*! @{ */

#define I3C_MINTCLR_SLVSTART_MASK                (0x100U)
#define I3C_MINTCLR_SLVSTART_SHIFT               (8U)
/*! SLVSTART - SLVSTART Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_SLVSTART(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_SLVSTART_SHIFT)) & I3C_MINTCLR_SLVSTART_MASK)

#define I3C_MINTCLR_MCTRLDONE_MASK               (0x200U)
#define I3C_MINTCLR_MCTRLDONE_SHIFT              (9U)
/*! MCTRLDONE - MCTRLDONE Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_MCTRLDONE(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_MCTRLDONE_SHIFT)) & I3C_MINTCLR_MCTRLDONE_MASK)

#define I3C_MINTCLR_COMPLETE_MASK                (0x400U)
#define I3C_MINTCLR_COMPLETE_SHIFT               (10U)
/*! COMPLETE - COMPLETE Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_COMPLETE(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_COMPLETE_SHIFT)) & I3C_MINTCLR_COMPLETE_MASK)

#define I3C_MINTCLR_RXPEND_MASK                  (0x800U)
#define I3C_MINTCLR_RXPEND_SHIFT                 (11U)
/*! RXPEND - RXPEND Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_RXPEND(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_RXPEND_SHIFT)) & I3C_MINTCLR_RXPEND_MASK)

#define I3C_MINTCLR_TXNOTFULL_MASK               (0x1000U)
#define I3C_MINTCLR_TXNOTFULL_SHIFT              (12U)
/*! TXNOTFULL - TXNOTFULL Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_TXNOTFULL(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_TXNOTFULL_SHIFT)) & I3C_MINTCLR_TXNOTFULL_MASK)

#define I3C_MINTCLR_IBIWON_MASK                  (0x2000U)
#define I3C_MINTCLR_IBIWON_SHIFT                 (13U)
/*! IBIWON - IBIWON Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_IBIWON(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_IBIWON_SHIFT)) & I3C_MINTCLR_IBIWON_MASK)

#define I3C_MINTCLR_ERRWARN_MASK                 (0x8000U)
#define I3C_MINTCLR_ERRWARN_SHIFT                (15U)
/*! ERRWARN - ERRWARN Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_ERRWARN(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_ERRWARN_SHIFT)) & I3C_MINTCLR_ERRWARN_MASK)

#define I3C_MINTCLR_NOWMASTER_MASK               (0x80000U)
#define I3C_MINTCLR_NOWMASTER_SHIFT              (19U)
/*! NOWMASTER - NOWCONTROLLER Interrupt Enable Clear Flag
 *  0b1..Interrupt enable cleared
 *  0b0..No effect
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MINTCLR_NOWMASTER(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTCLR_NOWMASTER_SHIFT)) & I3C_MINTCLR_NOWMASTER_MASK)
/*! @} */

/*! @name MINTMASKED - Controller Interrupt Mask */
/*! @{ */

#define I3C_MINTMASKED_SLVSTART_MASK             (0x100U)
#define I3C_MINTMASKED_SLVSTART_SHIFT            (8U)
/*! SLVSTART - SLVSTART Interrupt Mask
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define I3C_MINTMASKED_SLVSTART(x)               (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_SLVSTART_SHIFT)) & I3C_MINTMASKED_SLVSTART_MASK)

#define I3C_MINTMASKED_MCTRLDONE_MASK            (0x200U)
#define I3C_MINTMASKED_MCTRLDONE_SHIFT           (9U)
/*! MCTRLDONE - MCTRLDONE Interrupt Mask
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define I3C_MINTMASKED_MCTRLDONE(x)              (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_MCTRLDONE_SHIFT)) & I3C_MINTMASKED_MCTRLDONE_MASK)

#define I3C_MINTMASKED_COMPLETE_MASK             (0x400U)
#define I3C_MINTMASKED_COMPLETE_SHIFT            (10U)
/*! COMPLETE - COMPLETE Interrupt Mask
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define I3C_MINTMASKED_COMPLETE(x)               (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_COMPLETE_SHIFT)) & I3C_MINTMASKED_COMPLETE_MASK)

#define I3C_MINTMASKED_RXPEND_MASK               (0x800U)
#define I3C_MINTMASKED_RXPEND_SHIFT              (11U)
/*! RXPEND - RXPEND Interrupt Mask */
#define I3C_MINTMASKED_RXPEND(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_RXPEND_SHIFT)) & I3C_MINTMASKED_RXPEND_MASK)

#define I3C_MINTMASKED_TXNOTFULL_MASK            (0x1000U)
#define I3C_MINTMASKED_TXNOTFULL_SHIFT           (12U)
/*! TXNOTFULL - TXNOTFULL Interrupt Mask
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define I3C_MINTMASKED_TXNOTFULL(x)              (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_TXNOTFULL_SHIFT)) & I3C_MINTMASKED_TXNOTFULL_MASK)

#define I3C_MINTMASKED_IBIWON_MASK               (0x2000U)
#define I3C_MINTMASKED_IBIWON_SHIFT              (13U)
/*! IBIWON - IBIWON Interrupt Mask
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define I3C_MINTMASKED_IBIWON(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_IBIWON_SHIFT)) & I3C_MINTMASKED_IBIWON_MASK)

#define I3C_MINTMASKED_ERRWARN_MASK              (0x8000U)
#define I3C_MINTMASKED_ERRWARN_SHIFT             (15U)
/*! ERRWARN - ERRWARN Interrupt Mask
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define I3C_MINTMASKED_ERRWARN(x)                (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_ERRWARN_SHIFT)) & I3C_MINTMASKED_ERRWARN_MASK)

#define I3C_MINTMASKED_NOWMASTER_MASK            (0x80000U)
#define I3C_MINTMASKED_NOWMASTER_SHIFT           (19U)
/*! NOWMASTER - NOWCONTROLLER Interrupt Mask
 *  0b1..Enabled
 *  0b0..Disabled
 */
#define I3C_MINTMASKED_NOWMASTER(x)              (((uint32_t)(((uint32_t)(x)) << I3C_MINTMASKED_NOWMASTER_SHIFT)) & I3C_MINTMASKED_NOWMASTER_MASK)
/*! @} */

/*! @name MERRWARN - Controller Errors and Warnings */
/*! @{ */

#define I3C_MERRWARN_NACK_MASK                   (0x4U)
#define I3C_MERRWARN_NACK_SHIFT                  (2U)
/*! NACK - Not Acknowledge Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_NACK(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_NACK_SHIFT)) & I3C_MERRWARN_NACK_MASK)

#define I3C_MERRWARN_WRABT_MASK                  (0x8U)
#define I3C_MERRWARN_WRABT_SHIFT                 (3U)
/*! WRABT - Write Abort Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_WRABT(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_WRABT_SHIFT)) & I3C_MERRWARN_WRABT_MASK)

#define I3C_MERRWARN_HPAR_MASK                   (0x200U)
#define I3C_MERRWARN_HPAR_SHIFT                  (9U)
/*! HPAR - High Data Rate Parity Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_HPAR(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_HPAR_SHIFT)) & I3C_MERRWARN_HPAR_MASK)

#define I3C_MERRWARN_HCRC_MASK                   (0x400U)
#define I3C_MERRWARN_HCRC_SHIFT                  (10U)
/*! HCRC - High Data Rate CRC Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_HCRC(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_HCRC_SHIFT)) & I3C_MERRWARN_HCRC_MASK)

#define I3C_MERRWARN_OREAD_MASK                  (0x10000U)
#define I3C_MERRWARN_OREAD_SHIFT                 (16U)
/*! OREAD - Overread Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_OREAD(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_OREAD_SHIFT)) & I3C_MERRWARN_OREAD_MASK)

#define I3C_MERRWARN_OWRITE_MASK                 (0x20000U)
#define I3C_MERRWARN_OWRITE_SHIFT                (17U)
/*! OWRITE - Overwrite Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_OWRITE(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_OWRITE_SHIFT)) & I3C_MERRWARN_OWRITE_MASK)

#define I3C_MERRWARN_MSGERR_MASK                 (0x40000U)
#define I3C_MERRWARN_MSGERR_SHIFT                (18U)
/*! MSGERR - Message Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_MSGERR(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_MSGERR_SHIFT)) & I3C_MERRWARN_MSGERR_MASK)

#define I3C_MERRWARN_INVREQ_MASK                 (0x80000U)
#define I3C_MERRWARN_INVREQ_SHIFT                (19U)
/*! INVREQ - Invalid Request Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_INVREQ(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_INVREQ_SHIFT)) & I3C_MERRWARN_INVREQ_MASK)

#define I3C_MERRWARN_TIMEOUT_MASK                (0x100000U)
#define I3C_MERRWARN_TIMEOUT_SHIFT               (20U)
/*! TIMEOUT - Timeout Error Flag
 *  0b1..Error
 *  0b0..No error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define I3C_MERRWARN_TIMEOUT(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MERRWARN_TIMEOUT_SHIFT)) & I3C_MERRWARN_TIMEOUT_MASK)
/*! @} */

/*! @name MDMACTRL - Controller DMA Control */
/*! @{ */

#define I3C_MDMACTRL_DMAFB_MASK                  (0x3U)
#define I3C_MDMACTRL_DMAFB_SHIFT                 (0U)
/*! DMAFB - DMA from Bus
 *  0b00..DMA not used
 *  0b01..Enable DMA for one frame
 *  0b10..Enable DMA until DMA is turned off
 *  0b11..
 */
#define I3C_MDMACTRL_DMAFB(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MDMACTRL_DMAFB_SHIFT)) & I3C_MDMACTRL_DMAFB_MASK)

#define I3C_MDMACTRL_DMATB_MASK                  (0xCU)
#define I3C_MDMACTRL_DMATB_SHIFT                 (2U)
/*! DMATB - DMA to Bus
 *  0b00..DMA not used
 *  0b01..Enable DMA for one frame (ended by DMA or terminated)
 *  0b10..Enable DMA until DMA is turned off
 *  0b11..
 */
#define I3C_MDMACTRL_DMATB(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MDMACTRL_DMATB_SHIFT)) & I3C_MDMACTRL_DMATB_MASK)

#define I3C_MDMACTRL_DMAWIDTH_MASK               (0x30U)
#define I3C_MDMACTRL_DMAWIDTH_SHIFT              (4U)
/*! DMAWIDTH - DMA Width
 *  0b00, 0b01..Byte
 *  0b10..Halfword (16 bits)
 *  0b11..
 */
#define I3C_MDMACTRL_DMAWIDTH(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MDMACTRL_DMAWIDTH_SHIFT)) & I3C_MDMACTRL_DMAWIDTH_MASK)
/*! @} */

/*! @name MDATACTRL - Controller Data Control */
/*! @{ */

#define I3C_MDATACTRL_FLUSHTB_MASK               (0x1U)
#define I3C_MDATACTRL_FLUSHTB_SHIFT              (0U)
/*! FLUSHTB - Flush To-Bus Buffer or FIFO
 *  0b1..Flush the buffer
 *  0b0..No action
 */
#define I3C_MDATACTRL_FLUSHTB(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_FLUSHTB_SHIFT)) & I3C_MDATACTRL_FLUSHTB_MASK)

#define I3C_MDATACTRL_FLUSHFB_MASK               (0x2U)
#define I3C_MDATACTRL_FLUSHFB_SHIFT              (1U)
/*! FLUSHFB - Flush From-Bus Buffer or FIFO
 *  0b1..Flush the buffer
 *  0b0..No action
 */
#define I3C_MDATACTRL_FLUSHFB(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_FLUSHFB_SHIFT)) & I3C_MDATACTRL_FLUSHFB_MASK)

#define I3C_MDATACTRL_UNLOCK_MASK                (0x8U)
#define I3C_MDATACTRL_UNLOCK_SHIFT               (3U)
/*! UNLOCK - Unlock
 *  0b0..Locked
 *  0b1..Unlocked
 */
#define I3C_MDATACTRL_UNLOCK(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_UNLOCK_SHIFT)) & I3C_MDATACTRL_UNLOCK_MASK)

#define I3C_MDATACTRL_TXTRIG_MASK                (0x30U)
#define I3C_MDATACTRL_TXTRIG_SHIFT               (4U)
/*! TXTRIG - Transmit Trigger Level
 *  0b00..Trigger when empty
 *  0b01..Trigger when 1/4 full or less
 *  0b10..Trigger when 1/2 full or less
 *  0b11..Trigger when 1 less than full or less (default)
 */
#define I3C_MDATACTRL_TXTRIG(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_TXTRIG_SHIFT)) & I3C_MDATACTRL_TXTRIG_MASK)

#define I3C_MDATACTRL_RXTRIG_MASK                (0xC0U)
#define I3C_MDATACTRL_RXTRIG_SHIFT               (6U)
/*! RXTRIG - Receive Trigger Level
 *  0b00..Trigger when not empty
 *  0b01..Trigger when 1/4 full or more
 *  0b10..Trigger when 1/2 full or more
 *  0b11..Trigger when 3/4 full or more
 */
#define I3C_MDATACTRL_RXTRIG(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_RXTRIG_SHIFT)) & I3C_MDATACTRL_RXTRIG_MASK)

#define I3C_MDATACTRL_TXCOUNT_MASK               (0x1F0000U)
#define I3C_MDATACTRL_TXCOUNT_SHIFT              (16U)
/*! TXCOUNT - Transmit Byte Count */
#define I3C_MDATACTRL_TXCOUNT(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_TXCOUNT_SHIFT)) & I3C_MDATACTRL_TXCOUNT_MASK)

#define I3C_MDATACTRL_RXCOUNT_MASK               (0x1F000000U)
#define I3C_MDATACTRL_RXCOUNT_SHIFT              (24U)
/*! RXCOUNT - Receive Byte Count */
#define I3C_MDATACTRL_RXCOUNT(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_RXCOUNT_SHIFT)) & I3C_MDATACTRL_RXCOUNT_MASK)

#define I3C_MDATACTRL_TXFULL_MASK                (0x40000000U)
#define I3C_MDATACTRL_TXFULL_SHIFT               (30U)
/*! TXFULL - Transmit is Full
 *  0b0..Not full
 *  0b1..Full
 */
#define I3C_MDATACTRL_TXFULL(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_TXFULL_SHIFT)) & I3C_MDATACTRL_TXFULL_MASK)

#define I3C_MDATACTRL_RXEMPTY_MASK               (0x80000000U)
#define I3C_MDATACTRL_RXEMPTY_SHIFT              (31U)
/*! RXEMPTY - Receive is Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define I3C_MDATACTRL_RXEMPTY(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_MDATACTRL_RXEMPTY_SHIFT)) & I3C_MDATACTRL_RXEMPTY_MASK)
/*! @} */

/*! @name MWDATAB - Controller Write Data Byte */
/*! @{ */

#define I3C_MWDATAB_VALUE_MASK                   (0xFFU)
#define I3C_MWDATAB_VALUE_SHIFT                  (0U)
/*! VALUE - Data Byte */
#define I3C_MWDATAB_VALUE(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAB_VALUE_SHIFT)) & I3C_MWDATAB_VALUE_MASK)

#define I3C_MWDATAB_END_MASK                     (0x100U)
#define I3C_MWDATAB_END_SHIFT                    (8U)
/*! END - End of Message
 *  0b0..Not the end
 *  0b1..End
 */
#define I3C_MWDATAB_END(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAB_END_SHIFT)) & I3C_MWDATAB_END_MASK)

#define I3C_MWDATAB_END_ALSO_MASK                (0x10000U)
#define I3C_MWDATAB_END_ALSO_SHIFT               (16U)
/*! END_ALSO - End of Message ALSO
 *  0b0..Not the end
 *  0b1..End
 */
#define I3C_MWDATAB_END_ALSO(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAB_END_ALSO_SHIFT)) & I3C_MWDATAB_END_ALSO_MASK)
/*! @} */

/*! @name MWDATABE - Controller Write Data Byte End */
/*! @{ */

#define I3C_MWDATABE_VALUE_MASK                  (0xFFU)
#define I3C_MWDATABE_VALUE_SHIFT                 (0U)
/*! VALUE - Data */
#define I3C_MWDATABE_VALUE(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MWDATABE_VALUE_SHIFT)) & I3C_MWDATABE_VALUE_MASK)
/*! @} */

/*! @name MWDATAH - Controller Write Data Halfword */
/*! @{ */

#define I3C_MWDATAH_DATA0_MASK                   (0xFFU)
#define I3C_MWDATAH_DATA0_SHIFT                  (0U)
/*! DATA0 - Data Byte 0 */
#define I3C_MWDATAH_DATA0(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAH_DATA0_SHIFT)) & I3C_MWDATAH_DATA0_MASK)

#define I3C_MWDATAH_DATA1_MASK                   (0xFF00U)
#define I3C_MWDATAH_DATA1_SHIFT                  (8U)
/*! DATA1 - Data Byte 1 */
#define I3C_MWDATAH_DATA1(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAH_DATA1_SHIFT)) & I3C_MWDATAH_DATA1_MASK)

#define I3C_MWDATAH_END_MASK                     (0x10000U)
#define I3C_MWDATAH_END_SHIFT                    (16U)
/*! END - End of Message
 *  0b0..Not the end
 *  0b1..End
 */
#define I3C_MWDATAH_END(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAH_END_SHIFT)) & I3C_MWDATAH_END_MASK)
/*! @} */

/*! @name MWDATAHE - Controller Write Data Halfword End */
/*! @{ */

#define I3C_MWDATAHE_DATA0_MASK                  (0xFFU)
#define I3C_MWDATAHE_DATA0_SHIFT                 (0U)
/*! DATA0 - Data Byte 0 */
#define I3C_MWDATAHE_DATA0(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAHE_DATA0_SHIFT)) & I3C_MWDATAHE_DATA0_MASK)

#define I3C_MWDATAHE_DATA1_MASK                  (0xFF00U)
#define I3C_MWDATAHE_DATA1_SHIFT                 (8U)
/*! DATA1 - Data Byte 1 */
#define I3C_MWDATAHE_DATA1(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAHE_DATA1_SHIFT)) & I3C_MWDATAHE_DATA1_MASK)
/*! @} */

/*! @name MRDATAB - Controller Read Data Byte */
/*! @{ */

#define I3C_MRDATAB_VALUE_MASK                   (0xFFU)
#define I3C_MRDATAB_VALUE_SHIFT                  (0U)
/*! VALUE - Value */
#define I3C_MRDATAB_VALUE(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_MRDATAB_VALUE_SHIFT)) & I3C_MRDATAB_VALUE_MASK)
/*! @} */

/*! @name MRDATAH - Controller Read Data Halfword */
/*! @{ */

#define I3C_MRDATAH_LSB_MASK                     (0xFFU)
#define I3C_MRDATAH_LSB_SHIFT                    (0U)
/*! LSB - Low Byte */
#define I3C_MRDATAH_LSB(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_MRDATAH_LSB_SHIFT)) & I3C_MRDATAH_LSB_MASK)

#define I3C_MRDATAH_MSB_MASK                     (0xFF00U)
#define I3C_MRDATAH_MSB_SHIFT                    (8U)
/*! MSB - High Byte */
#define I3C_MRDATAH_MSB(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_MRDATAH_MSB_SHIFT)) & I3C_MRDATAH_MSB_MASK)
/*! @} */

/*! @name MWDATAB1 - Controller Write Byte Data 1 (to Bus) */
/*! @{ */

#define I3C_MWDATAB1_VALUE_MASK                  (0xFFU)
#define I3C_MWDATAB1_VALUE_SHIFT                 (0U)
/*! VALUE - Value */
#define I3C_MWDATAB1_VALUE(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MWDATAB1_VALUE_SHIFT)) & I3C_MWDATAB1_VALUE_MASK)
/*! @} */

/*! @name MWMSG_SDR_CONTROL - Controller Write Message Control in SDR mode */
/*! @{ */

#define I3C_MWMSG_SDR_CONTROL_DIR_MASK           (0x1U)
#define I3C_MWMSG_SDR_CONTROL_DIR_SHIFT          (0U)
/*! DIR - Direction
 *  0b0..Write
 *  0b1..Read
 */
#define I3C_MWMSG_SDR_CONTROL_DIR(x)             (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_SDR_CONTROL_DIR_SHIFT)) & I3C_MWMSG_SDR_CONTROL_DIR_MASK)

#define I3C_MWMSG_SDR_CONTROL_ADDR_MASK          (0xFEU)
#define I3C_MWMSG_SDR_CONTROL_ADDR_SHIFT         (1U)
/*! ADDR - Address */
#define I3C_MWMSG_SDR_CONTROL_ADDR(x)            (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_SDR_CONTROL_ADDR_SHIFT)) & I3C_MWMSG_SDR_CONTROL_ADDR_MASK)

#define I3C_MWMSG_SDR_CONTROL_END_MASK           (0x100U)
#define I3C_MWMSG_SDR_CONTROL_END_SHIFT          (8U)
/*! END - End of SDR Message
 *  0b0..Not the end
 *  0b1..End
 */
#define I3C_MWMSG_SDR_CONTROL_END(x)             (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_SDR_CONTROL_END_SHIFT)) & I3C_MWMSG_SDR_CONTROL_END_MASK)

#define I3C_MWMSG_SDR_CONTROL_I2C_MASK           (0x400U)
#define I3C_MWMSG_SDR_CONTROL_I2C_SHIFT          (10U)
/*! I2C - I2C
 *  0b0..I3C message
 *  0b1..I2C message
 */
#define I3C_MWMSG_SDR_CONTROL_I2C(x)             (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_SDR_CONTROL_I2C_SHIFT)) & I3C_MWMSG_SDR_CONTROL_I2C_MASK)

#define I3C_MWMSG_SDR_CONTROL_LEN_MASK           (0xF800U)
#define I3C_MWMSG_SDR_CONTROL_LEN_SHIFT          (11U)
/*! LEN - Length */
#define I3C_MWMSG_SDR_CONTROL_LEN(x)             (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_SDR_CONTROL_LEN_SHIFT)) & I3C_MWMSG_SDR_CONTROL_LEN_MASK)
/*! @} */

/*! @name MWMSG_SDR_DATA - Controller Write Message Data in SDR mode */
/*! @{ */

#define I3C_MWMSG_SDR_DATA_DATA16B_MASK          (0xFFFFU)
#define I3C_MWMSG_SDR_DATA_DATA16B_SHIFT         (0U)
/*! DATA16B - Data */
#define I3C_MWMSG_SDR_DATA_DATA16B(x)            (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_SDR_DATA_DATA16B_SHIFT)) & I3C_MWMSG_SDR_DATA_DATA16B_MASK)
/*! @} */

/*! @name MRMSG_SDR - Controller Read Message in SDR mode */
/*! @{ */

#define I3C_MRMSG_SDR_DATA_MASK                  (0xFFFFU)
#define I3C_MRMSG_SDR_DATA_SHIFT                 (0U)
/*! DATA - Data */
#define I3C_MRMSG_SDR_DATA(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MRMSG_SDR_DATA_SHIFT)) & I3C_MRMSG_SDR_DATA_MASK)
/*! @} */

/*! @name MWMSG_DDR_CONTROL - Controller Write Message in DDR mode: First Control Word */
/*! @{ */

#define I3C_MWMSG_DDR_CONTROL_ADDRCMD_MASK       (0xFFFFU)
#define I3C_MWMSG_DDR_CONTROL_ADDRCMD_SHIFT      (0U)
/*! ADDRCMD - Address Command */
#define I3C_MWMSG_DDR_CONTROL_ADDRCMD(x)         (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_DDR_CONTROL_ADDRCMD_SHIFT)) & I3C_MWMSG_DDR_CONTROL_ADDRCMD_MASK)
/*! @} */

/*! @name MWMSG_DDR_CONTROL2 - Controller Write Message in DDR Mode Control 2 */
/*! @{ */

#define I3C_MWMSG_DDR_CONTROL2_LEN_MASK          (0x3FFU)
#define I3C_MWMSG_DDR_CONTROL2_LEN_SHIFT         (0U)
/*! LEN - Length of Message */
#define I3C_MWMSG_DDR_CONTROL2_LEN(x)            (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_DDR_CONTROL2_LEN_SHIFT)) & I3C_MWMSG_DDR_CONTROL2_LEN_MASK)

#define I3C_MWMSG_DDR_CONTROL2_END_MASK          (0x4000U)
#define I3C_MWMSG_DDR_CONTROL2_END_SHIFT         (14U)
/*! END - End of Message
 *  0b1..End
 *  0b0..Not the end
 */
#define I3C_MWMSG_DDR_CONTROL2_END(x)            (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_DDR_CONTROL2_END_SHIFT)) & I3C_MWMSG_DDR_CONTROL2_END_MASK)
/*! @} */

/*! @name MWMSG_DDR_DATA - Controller Write Message Data in DDR mode */
/*! @{ */

#define I3C_MWMSG_DDR_DATA_DATA16B_MASK          (0xFFFFU)
#define I3C_MWMSG_DDR_DATA_DATA16B_SHIFT         (0U)
/*! DATA16B - Data */
#define I3C_MWMSG_DDR_DATA_DATA16B(x)            (((uint32_t)(((uint32_t)(x)) << I3C_MWMSG_DDR_DATA_DATA16B_SHIFT)) & I3C_MWMSG_DDR_DATA_DATA16B_MASK)
/*! @} */

/*! @name MRMSG_DDR - Controller Read Message in DDR mode */
/*! @{ */

#define I3C_MRMSG_DDR_DATA_MASK                  (0xFFFFU)
#define I3C_MRMSG_DDR_DATA_SHIFT                 (0U)
/*! DATA - Data */
#define I3C_MRMSG_DDR_DATA(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MRMSG_DDR_DATA_SHIFT)) & I3C_MRMSG_DDR_DATA_MASK)
/*! @} */

/*! @name MDYNADDR - Controller Dynamic Address */
/*! @{ */

#define I3C_MDYNADDR_DAVALID_MASK                (0x1U)
#define I3C_MDYNADDR_DAVALID_SHIFT               (0U)
/*! DAVALID - Dynamic Address Valid
 *  0b1..Valid DA assigned
 *  0b0..No valid DA assigned
 */
#define I3C_MDYNADDR_DAVALID(x)                  (((uint32_t)(((uint32_t)(x)) << I3C_MDYNADDR_DAVALID_SHIFT)) & I3C_MDYNADDR_DAVALID_MASK)

#define I3C_MDYNADDR_DADDR_MASK                  (0xFEU)
#define I3C_MDYNADDR_DADDR_SHIFT                 (1U)
/*! DADDR - Dynamic Address */
#define I3C_MDYNADDR_DADDR(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_MDYNADDR_DADDR_SHIFT)) & I3C_MDYNADDR_DADDR_MASK)
/*! @} */

/*! @name SRSTACTTIME - Timing Rules for Target Reset Recovery */
/*! @{ */

#define I3C_SRSTACTTIME_PERRSTTIM_MASK           (0xFFU)
#define I3C_SRSTACTTIME_PERRSTTIM_SHIFT          (0U)
/*! PERRSTTIM - Time to Recover from the I3C Peripheral */
#define I3C_SRSTACTTIME_PERRSTTIM(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SRSTACTTIME_PERRSTTIM_SHIFT)) & I3C_SRSTACTTIME_PERRSTTIM_MASK)

#define I3C_SRSTACTTIME_SYSRSTTIM_MASK           (0xFF00U)
#define I3C_SRSTACTTIME_SYSRSTTIM_SHIFT          (8U)
/*! SYSRSTTIM - Time to Recover from Chip Reset */
#define I3C_SRSTACTTIME_SYSRSTTIM(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SRSTACTTIME_SYSRSTTIM_SHIFT)) & I3C_SRSTACTTIME_SYSRSTTIM_MASK)
/*! @} */

/*! @name SCCCMASK - CCC Mask for Unhandled CCCs */
/*! @{ */

#define I3C_SCCCMASK_BASE_MASK                   (0x1U)
#define I3C_SCCCMASK_BASE_SHIFT                  (0U)
/*! BASE - Base
 *  0b0..Suppressed
 *  0b1..Passed to application
 */
#define I3C_SCCCMASK_BASE(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SCCCMASK_BASE_SHIFT)) & I3C_SCCCMASK_BASE_MASK)

#define I3C_SCCCMASK_BASEBX_MASK                 (0x2U)
#define I3C_SCCCMASK_BASEBX_SHIFT                (1U)
/*! BASEBX - BASEBX
 *  0b0..Suppressed
 *  0b1..Passed to application
 */
#define I3C_SCCCMASK_BASEBX(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SCCCMASK_BASEBX_SHIFT)) & I3C_SCCCMASK_BASEBX_MASK)

#define I3C_SCCCMASK_BASEDX_MASK                 (0x4U)
#define I3C_SCCCMASK_BASEDX_SHIFT                (2U)
/*! BASEDX - BASEDX
 *  0b0..Suppressed
 *  0b1..Passed to application
 */
#define I3C_SCCCMASK_BASEDX(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SCCCMASK_BASEDX_SHIFT)) & I3C_SCCCMASK_BASEDX_MASK)

#define I3C_SCCCMASK_MEXTB_MASK                  (0x8U)
#define I3C_SCCCMASK_MEXTB_SHIFT                 (3U)
/*! MEXTB - MEXTB
 *  0b0..Suppressed
 *  0b1..Passed to application
 */
#define I3C_SCCCMASK_MEXTB(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SCCCMASK_MEXTB_SHIFT)) & I3C_SCCCMASK_MEXTB_MASK)

#define I3C_SCCCMASK_MEXTD_MASK                  (0x10U)
#define I3C_SCCCMASK_MEXTD_SHIFT                 (4U)
/*! MEXTD - MEXTD
 *  0b0..Suppressed
 *  0b1..Passed to application
 */
#define I3C_SCCCMASK_MEXTD(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SCCCMASK_MEXTD_SHIFT)) & I3C_SCCCMASK_MEXTD_MASK)

#define I3C_SCCCMASK_VENDB_MASK                  (0x20U)
#define I3C_SCCCMASK_VENDB_SHIFT                 (5U)
/*! VENDB - VENDB
 *  0b0..Suppressed
 *  0b1..Passed to application
 */
#define I3C_SCCCMASK_VENDB(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SCCCMASK_VENDB_SHIFT)) & I3C_SCCCMASK_VENDB_MASK)

#define I3C_SCCCMASK_VENDD_MASK                  (0x40U)
#define I3C_SCCCMASK_VENDD_SHIFT                 (6U)
/*! VENDD - VENDD
 *  0b0..Suppressed
 *  0b1..Passed to application
 */
#define I3C_SCCCMASK_VENDD(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SCCCMASK_VENDD_SHIFT)) & I3C_SCCCMASK_VENDD_MASK)
/*! @} */

/*! @name SERRWARNMASK - Target Errors and Warnings Mask */
/*! @{ */

#define I3C_SERRWARNMASK_ORUN_MASK               (0x1U)
#define I3C_SERRWARNMASK_ORUN_SHIFT              (0U)
/*! ORUN - ORUN Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_ORUN(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_ORUN_SHIFT)) & I3C_SERRWARNMASK_ORUN_MASK)

#define I3C_SERRWARNMASK_URUN_MASK               (0x2U)
#define I3C_SERRWARNMASK_URUN_SHIFT              (1U)
/*! URUN - URUN Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_URUN(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_URUN_SHIFT)) & I3C_SERRWARNMASK_URUN_MASK)

#define I3C_SERRWARNMASK_URUNNACK_MASK           (0x4U)
#define I3C_SERRWARNMASK_URUNNACK_SHIFT          (2U)
/*! URUNNACK - URUNNACK Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_URUNNACK(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_URUNNACK_SHIFT)) & I3C_SERRWARNMASK_URUNNACK_MASK)

#define I3C_SERRWARNMASK_TERM_MASK               (0x8U)
#define I3C_SERRWARNMASK_TERM_SHIFT              (3U)
/*! TERM - TERM Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_TERM(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_TERM_SHIFT)) & I3C_SERRWARNMASK_TERM_MASK)

#define I3C_SERRWARNMASK_INVSTART_MASK           (0x10U)
#define I3C_SERRWARNMASK_INVSTART_SHIFT          (4U)
/*! INVSTART - INVSTART Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_INVSTART(x)             (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_INVSTART_SHIFT)) & I3C_SERRWARNMASK_INVSTART_MASK)

#define I3C_SERRWARNMASK_SPAR_MASK               (0x100U)
#define I3C_SERRWARNMASK_SPAR_SHIFT              (8U)
/*! SPAR - SPAR Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_SPAR(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_SPAR_SHIFT)) & I3C_SERRWARNMASK_SPAR_MASK)

#define I3C_SERRWARNMASK_HPAR_MASK               (0x200U)
#define I3C_SERRWARNMASK_HPAR_SHIFT              (9U)
/*! HPAR - HPAR Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_HPAR(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_HPAR_SHIFT)) & I3C_SERRWARNMASK_HPAR_MASK)

#define I3C_SERRWARNMASK_HCRC_MASK               (0x400U)
#define I3C_SERRWARNMASK_HCRC_SHIFT              (10U)
/*! HCRC - HCRC Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_HCRC(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_HCRC_SHIFT)) & I3C_SERRWARNMASK_HCRC_MASK)

#define I3C_SERRWARNMASK_S0S1_MASK               (0x800U)
#define I3C_SERRWARNMASK_S0S1_SHIFT              (11U)
/*! S0S1 - S0S1 Mask
 *  0b1..Allow
 *  0b0..Deny
 */
#define I3C_SERRWARNMASK_S0S1(x)                 (((uint32_t)(((uint32_t)(x)) << I3C_SERRWARNMASK_S0S1_SHIFT)) & I3C_SERRWARNMASK_S0S1_MASK)
/*! @} */

/*! @name SMAPCTRL0 - Map Feature Control 0 */
/*! @{ */

#define I3C_SMAPCTRL0_ENA_MASK                   (0x1U)
#define I3C_SMAPCTRL0_ENA_SHIFT                  (0U)
/*! ENA - Enable Primary Dynamic Address
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define I3C_SMAPCTRL0_ENA(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL0_ENA_SHIFT)) & I3C_SMAPCTRL0_ENA_MASK)

#define I3C_SMAPCTRL0_DA_MASK                    (0xFEU)
#define I3C_SMAPCTRL0_DA_SHIFT                   (1U)
/*! DA - Dynamic Address */
#define I3C_SMAPCTRL0_DA(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL0_DA_SHIFT)) & I3C_SMAPCTRL0_DA_MASK)

#define I3C_SMAPCTRL0_CAUSE_MASK                 (0x700U)
#define I3C_SMAPCTRL0_CAUSE_SHIFT                (8U)
/*! CAUSE - Cause
 *  0b000..No information (this value occurs when not configured to write DA)
 *  0b001..Set using ENTDAA
 *  0b010..Set using SETDASA, SETAASA, or SETNEWDA
 *  0b011..Cleared using RSTDAA
 *  0b100..Auto MAP change happened last
 *  *..
 */
#define I3C_SMAPCTRL0_CAUSE(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL0_CAUSE_SHIFT)) & I3C_SMAPCTRL0_CAUSE_MASK)
/*! @} */

/*! @name SMAPCTRL1 - Map Feature Control 1 */
/*! @{ */

#define I3C_SMAPCTRL1_ENA_MASK                   (0x1U)
#define I3C_SMAPCTRL1_ENA_SHIFT                  (0U)
/*! ENA - Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define I3C_SMAPCTRL1_ENA(x)                     (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL1_ENA_SHIFT)) & I3C_SMAPCTRL1_ENA_MASK)

#define I3C_SMAPCTRL1_ADDR_MASK                  (0xFEU)
#define I3C_SMAPCTRL1_ADDR_SHIFT                 (1U)
/*! ADDR - Address */
#define I3C_SMAPCTRL1_ADDR(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL1_ADDR_SHIFT)) & I3C_SMAPCTRL1_ADDR_MASK)

#define I3C_SMAPCTRL1_MAPSA_MASK                 (0x100U)
#define I3C_SMAPCTRL1_MAPSA_SHIFT                (8U)
/*! MAPSA - MAP Static Address
 *  0b0..I3C dynamic address
 *  0b1..Static address (I2C style)
 */
#define I3C_SMAPCTRL1_MAPSA(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL1_MAPSA_SHIFT)) & I3C_SMAPCTRL1_MAPSA_MASK)

#define I3C_SMAPCTRL1_SA10B_MASK                 (0xE00U)
#define I3C_SMAPCTRL1_SA10B_SHIFT                (9U)
/*! SA10B - Static Address 10-Bit Extension */
#define I3C_SMAPCTRL1_SA10B(x)                   (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL1_SA10B_SHIFT)) & I3C_SMAPCTRL1_SA10B_MASK)

#define I3C_SMAPCTRL1_NACK_MASK                  (0x1000U)
#define I3C_SMAPCTRL1_NACK_SHIFT                 (12U)
/*! NACK - Not Acknowledged
 *  0b0..Do not always NACK messages
 *  0b1..Always NACK messages
 */
#define I3C_SMAPCTRL1_NACK(x)                    (((uint32_t)(((uint32_t)(x)) << I3C_SMAPCTRL1_NACK_SHIFT)) & I3C_SMAPCTRL1_NACK_MASK)
/*! @} */

/*! @name IBIEXT1 - Extended IBI Data 1 */
/*! @{ */

#define I3C_IBIEXT1_CNT_MASK                     (0x7U)
#define I3C_IBIEXT1_CNT_SHIFT                    (0U)
/*! CNT - Count */
#define I3C_IBIEXT1_CNT(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT1_CNT_SHIFT)) & I3C_IBIEXT1_CNT_MASK)

#define I3C_IBIEXT1_MAX_MASK                     (0x70U)
#define I3C_IBIEXT1_MAX_SHIFT                    (4U)
/*! MAX - Maximum */
#define I3C_IBIEXT1_MAX(x)                       (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT1_MAX_SHIFT)) & I3C_IBIEXT1_MAX_MASK)

#define I3C_IBIEXT1_EXT1_MASK                    (0xFF00U)
#define I3C_IBIEXT1_EXT1_SHIFT                   (8U)
/*! EXT1 - Extra Byte 1 */
#define I3C_IBIEXT1_EXT1(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT1_EXT1_SHIFT)) & I3C_IBIEXT1_EXT1_MASK)

#define I3C_IBIEXT1_EXT2_MASK                    (0xFF0000U)
#define I3C_IBIEXT1_EXT2_SHIFT                   (16U)
/*! EXT2 - Extra Byte 2 */
#define I3C_IBIEXT1_EXT2(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT1_EXT2_SHIFT)) & I3C_IBIEXT1_EXT2_MASK)

#define I3C_IBIEXT1_EXT3_MASK                    (0xFF000000U)
#define I3C_IBIEXT1_EXT3_SHIFT                   (24U)
/*! EXT3 - Extra Byte 3 */
#define I3C_IBIEXT1_EXT3(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT1_EXT3_SHIFT)) & I3C_IBIEXT1_EXT3_MASK)
/*! @} */

/*! @name IBIEXT2 - Extended IBI Data 2 */
/*! @{ */

#define I3C_IBIEXT2_EXT4_MASK                    (0xFFU)
#define I3C_IBIEXT2_EXT4_SHIFT                   (0U)
/*! EXT4 - Extra Byte 4 */
#define I3C_IBIEXT2_EXT4(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT2_EXT4_SHIFT)) & I3C_IBIEXT2_EXT4_MASK)

#define I3C_IBIEXT2_EXT5_MASK                    (0xFF00U)
#define I3C_IBIEXT2_EXT5_SHIFT                   (8U)
/*! EXT5 - Extra Byte 5 */
#define I3C_IBIEXT2_EXT5(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT2_EXT5_SHIFT)) & I3C_IBIEXT2_EXT5_MASK)

#define I3C_IBIEXT2_EXT6_MASK                    (0xFF0000U)
#define I3C_IBIEXT2_EXT6_SHIFT                   (16U)
/*! EXT6 - Extra Byte 6 */
#define I3C_IBIEXT2_EXT6(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT2_EXT6_SHIFT)) & I3C_IBIEXT2_EXT6_MASK)

#define I3C_IBIEXT2_EXT7_MASK                    (0xFF000000U)
#define I3C_IBIEXT2_EXT7_SHIFT                   (24U)
/*! EXT7 - Extra Byte 7 */
#define I3C_IBIEXT2_EXT7(x)                      (((uint32_t)(((uint32_t)(x)) << I3C_IBIEXT2_EXT7_SHIFT)) & I3C_IBIEXT2_EXT7_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group I3C_Register_Masks */


/* I3C - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral I3C1 base address */
  #define I3C1_BASE                                (0x54330000u)
  /** Peripheral I3C1 base address */
  #define I3C1_BASE_NS                             (0x44330000u)
  /** Peripheral I3C1 base pointer */
  #define I3C1                                     ((I3C_Type *)I3C1_BASE)
  /** Peripheral I3C1 base pointer */
  #define I3C1_NS                                  ((I3C_Type *)I3C1_BASE_NS)
  /** Peripheral I3C2 base address */
  #define I3C2_BASE                                (0x52520000u)
  /** Peripheral I3C2 base address */
  #define I3C2_BASE_NS                             (0x42520000u)
  /** Peripheral I3C2 base pointer */
  #define I3C2                                     ((I3C_Type *)I3C2_BASE)
  /** Peripheral I3C2 base pointer */
  #define I3C2_NS                                  ((I3C_Type *)I3C2_BASE_NS)
  /** Array initializer of I3C peripheral base addresses */
  #define I3C_BASE_ADDRS                           { 0u, I3C1_BASE, I3C2_BASE }
  /** Array initializer of I3C peripheral base pointers */
  #define I3C_BASE_PTRS                            { (I3C_Type *)0u, I3C1, I3C2 }
  /** Array initializer of I3C peripheral base addresses */
  #define I3C_BASE_ADDRS_NS                        { 0u, I3C1_BASE_NS, I3C2_BASE_NS }
  /** Array initializer of I3C peripheral base pointers */
  #define I3C_BASE_PTRS_NS                         { (I3C_Type *)0u, I3C1_NS, I3C2_NS }
#else
  /** Peripheral I3C1 base address */
  #define I3C1_BASE                                (0x44330000u)
  /** Peripheral I3C1 base pointer */
  #define I3C1                                     ((I3C_Type *)I3C1_BASE)
  /** Peripheral I3C2 base address */
  #define I3C2_BASE                                (0x42520000u)
  /** Peripheral I3C2 base pointer */
  #define I3C2                                     ((I3C_Type *)I3C2_BASE)
  /** Array initializer of I3C peripheral base addresses */
  #define I3C_BASE_ADDRS                           { 0u, I3C1_BASE, I3C2_BASE }
  /** Array initializer of I3C peripheral base pointers */
  #define I3C_BASE_PTRS                            { (I3C_Type *)0u, I3C1, I3C2 }
#endif
/** Interrupt vectors for the I3C peripheral type */
#define I3C_IRQS                                 { NotAvail_IRQn, I3C1_IRQn, I3C2_IRQn }

/*!
 * @}
 */ /* end of group I3C_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- IEE Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IEE_Peripheral_Access_Layer IEE Peripheral Access Layer
 * @{
 */

/** IEE - Register Layout Typedef */
typedef struct {
  __IO uint32_t GCFG;                              /**< IEE Global Configuration, offset: 0x0 */
  __I  uint32_t STA;                               /**< IEE Status, offset: 0x4 */
  __IO uint32_t TSTMD;                             /**< IEE Test Mode Register, offset: 0x8 */
  __O  uint32_t DPAMS;                             /**< AES Mask Generation Seed, offset: 0xC */
       uint8_t RESERVED_0[16];
  __IO uint32_t PC_S_LT;                           /**< Performance Counter, AES Slave Latency Threshold Value, offset: 0x20 */
  __IO uint32_t PC_M_LT;                           /**< Performance Counter, AES Master Latency Threshold, offset: 0x24 */
       uint8_t RESERVED_1[24];
  __IO uint32_t PC_BLK_ENC;                        /**< Performance Counter, Number of AES Block Encryptions, offset: 0x40 */
  __IO uint32_t PC_BLK_DEC;                        /**< Performance Counter, Number of AES Block Decryptions, offset: 0x44 */
       uint8_t RESERVED_2[8];
  __IO uint32_t PC_SR_TRANS;                       /**< Performance Counter, Number of AXI Slave Read Transactions, offset: 0x50 */
  __IO uint32_t PC_SW_TRANS;                       /**< Performance Counter, Number of AXI Slave Write Transactions, offset: 0x54 */
  __IO uint32_t PC_MR_TRANS;                       /**< Performance Counter, Number of AXI Master Read Transactions, offset: 0x58 */
  __IO uint32_t PC_MW_TRANS;                       /**< Performance Counter, Number of AXI Master Write Transactions, offset: 0x5C */
       uint8_t RESERVED_3[4];
  __IO uint32_t PC_M_MBR;                          /**< Performance Counter, Number of AXI Master Merge Buffer Read Transactions, offset: 0x64 */
       uint8_t RESERVED_4[8];
  __IO uint32_t PC_SR_TBC_U;                       /**< Performance Counter, Upper Slave Read Transactions Byte Count, offset: 0x70 */
  __IO uint32_t PC_SR_TBC_L;                       /**< Performance Counter, Lower Slave Read Transactions Byte Count, offset: 0x74 */
  __IO uint32_t PC_SW_TBC_U;                       /**< Performance Counter, Upper Slave Write Transactions Byte Count, offset: 0x78 */
  __IO uint32_t PC_SW_TBC_L;                       /**< Performance Counter, Lower Slave Write Transactions Byte Count, offset: 0x7C */
  __IO uint32_t PC_MR_TBC_U;                       /**< Performance Counter, Upper Master Read Transactions Byte Count, offset: 0x80 */
  __IO uint32_t PC_MR_TBC_L;                       /**< Performance Counter, Lower Master Read Transactions Byte Count, offset: 0x84 */
  __IO uint32_t PC_MW_TBC_U;                       /**< Performance Counter, Upper Master Write Transactions Byte Count, offset: 0x88 */
  __IO uint32_t PC_MW_TBC_L;                       /**< Performance Counter, Lower Master Write Transactions Byte Count, offset: 0x8C */
  __IO uint32_t PC_SR_TLGTT;                       /**< Performance Counter, Number of AXI Slave Read Transactions with Latency Greater than the Threshold, offset: 0x90 */
  __IO uint32_t PC_SW_TLGTT;                       /**< Performance Counter, Number of AXI Slave Write Transactions with Latency Greater than the Threshold, offset: 0x94 */
  __IO uint32_t PC_MR_TLGTT;                       /**< Performance Counter, Number of AXI Master Read Transactions with Latency Greater than the Threshold, offset: 0x98 */
  __IO uint32_t PC_MW_TLGTT;                       /**< Performance Counter, Number of AXI Master Write Transactions with Latency Greater than the Threshold, offset: 0x9C */
  __IO uint32_t PC_SR_TLAT_U;                      /**< Performance Counter, Upper Slave Read Latency Count, offset: 0xA0 */
  __IO uint32_t PC_SR_TLAT_L;                      /**< Performance Counter, Lower Slave Read Latency Count, offset: 0xA4 */
  __IO uint32_t PC_SW_TLAT_U;                      /**< Performance Counter, Upper Slave Write Latency Count, offset: 0xA8 */
  __IO uint32_t PC_SW_TLAT_L;                      /**< Performance Counter, Lower Slave Write Latency Count, offset: 0xAC */
  __IO uint32_t PC_MR_TLAT_U;                      /**< Performance Counter, Upper Master Read Latency Count, offset: 0xB0 */
  __IO uint32_t PC_MR_TLAT_L;                      /**< Performance Counter, Lower Master Read Latency Count, offset: 0xB4 */
  __IO uint32_t PC_MW_TLAT_U;                      /**< Performance Counter, Upper Master Write Latency Count, offset: 0xB8 */
  __IO uint32_t PC_MW_TLAT_L;                      /**< Performance Counter, Lower Master Write Latency Count, offset: 0xBC */
  __IO uint32_t PC_SR_TNRT_U;                      /**< Performance Counter, Upper Slave Read Total Non-Responding Time, offset: 0xC0 */
  __IO uint32_t PC_SR_TNRT_L;                      /**< Performance Counter, Lower Slave Read Total Non-Responding Time, offset: 0xC4 */
  __IO uint32_t PC_SW_TNRT_U;                      /**< Performance Counter, Upper Slave Write Total Non-Responding Time, offset: 0xC8 */
  __IO uint32_t PC_SW_TNRT_L;                      /**< Performance Counter, Lower Slave Write Total Non-Responding Time, offset: 0xCC */
       uint8_t RESERVED_5[32];
  __I  uint32_t VIDR1;                             /**< IEE Version ID Register 1, offset: 0xF0 */
       uint8_t RESERVED_6[4];
  __I  uint32_t AESVID;                            /**< IEE AES Version ID Register, offset: 0xF8 */
       uint8_t RESERVED_7[4];
  struct {                                         /* offset: 0x100, array step: 0x100 */
    __IO uint32_t REGATTR;                           /**< IEE Region 0 Attribute Register...IEE Region 7 Attribute Register., array offset: 0x100, array step: 0x100 */
         uint8_t RESERVED_0[4];
    __IO uint32_t REGPO;                             /**< IEE Region 0 Page Offset Register..IEE Region 7 Page Offset Register, array offset: 0x108, array step: 0x100 */
         uint8_t RESERVED_1[52];
    __O  uint32_t REGKEY1[8];                        /**< IEE Region 0 Key 1 Register..IEE Region 7 Key 1 Register, array offset: 0x140, array step: index*0x100, index2*0x4 */
         uint8_t RESERVED_2[32];
    __O  uint32_t REGKEY2[8];                        /**< IEE Region 0 Key 2 Register..IEE Region 7 Key 2 Register, array offset: 0x180, array step: index*0x100, index2*0x4 */
         uint8_t RESERVED_3[96];
  } REGX[8];
       uint8_t RESERVED_8[1536];
  __IO uint32_t AES_TST_DB[32];                    /**< IEE AES Test Mode Data Buffer, array offset: 0xF00, array step: 0x4 */
} IEE_Type;

/* ----------------------------------------------------------------------------
   -- IEE Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IEE_Register_Masks IEE Register Masks
 * @{
 */

/*! @name GCFG - IEE Global Configuration */
/*! @{ */

#define IEE_GCFG_RL0_MASK                        (0x1U)
#define IEE_GCFG_RL0_SHIFT                       (0U)
/*! RL0
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL0(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL0_SHIFT)) & IEE_GCFG_RL0_MASK)

#define IEE_GCFG_RL1_MASK                        (0x2U)
#define IEE_GCFG_RL1_SHIFT                       (1U)
/*! RL1
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL1(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL1_SHIFT)) & IEE_GCFG_RL1_MASK)

#define IEE_GCFG_RL2_MASK                        (0x4U)
#define IEE_GCFG_RL2_SHIFT                       (2U)
/*! RL2
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL2(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL2_SHIFT)) & IEE_GCFG_RL2_MASK)

#define IEE_GCFG_RL3_MASK                        (0x8U)
#define IEE_GCFG_RL3_SHIFT                       (3U)
/*! RL3
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL3(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL3_SHIFT)) & IEE_GCFG_RL3_MASK)

#define IEE_GCFG_RL4_MASK                        (0x10U)
#define IEE_GCFG_RL4_SHIFT                       (4U)
/*! RL4
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL4(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL4_SHIFT)) & IEE_GCFG_RL4_MASK)

#define IEE_GCFG_RL5_MASK                        (0x20U)
#define IEE_GCFG_RL5_SHIFT                       (5U)
/*! RL5
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL5(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL5_SHIFT)) & IEE_GCFG_RL5_MASK)

#define IEE_GCFG_RL6_MASK                        (0x40U)
#define IEE_GCFG_RL6_SHIFT                       (6U)
/*! RL6
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL6(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL6_SHIFT)) & IEE_GCFG_RL6_MASK)

#define IEE_GCFG_RL7_MASK                        (0x80U)
#define IEE_GCFG_RL7_SHIFT                       (7U)
/*! RL7
 *  0b0..Unlocked.
 *  0b1..Key, Offset and Attribute registers are locked.
 */
#define IEE_GCFG_RL7(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RL7_SHIFT)) & IEE_GCFG_RL7_MASK)

#define IEE_GCFG_TME_MASK                        (0x10000U)
#define IEE_GCFG_TME_SHIFT                       (16U)
/*! TME
 *  0b0..Disabled.
 *  0b1..Enabled.
 */
#define IEE_GCFG_TME(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_TME_SHIFT)) & IEE_GCFG_TME_MASK)

#define IEE_GCFG_TMD_MASK                        (0x20000U)
#define IEE_GCFG_TMD_SHIFT                       (17U)
/*! TMD
 *  0b0..Test mode is usable.
 *  0b1..Test mode is disabled.
 */
#define IEE_GCFG_TMD(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_TMD_SHIFT)) & IEE_GCFG_TMD_MASK)

#define IEE_GCFG_KEY_RD_DIS_MASK                 (0x2000000U)
#define IEE_GCFG_KEY_RD_DIS_SHIFT                (25U)
/*! KEY_RD_DIS
 *  0b0..Key read enabled. Reading the key registers is allowed.
 *  0b1..Key read disabled. Reading the key registers is disabled.
 */
#define IEE_GCFG_KEY_RD_DIS(x)                   (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_KEY_RD_DIS_SHIFT)) & IEE_GCFG_KEY_RD_DIS_MASK)

#define IEE_GCFG_MON_EN_MASK                     (0x10000000U)
#define IEE_GCFG_MON_EN_SHIFT                    (28U)
/*! MON_EN
 *  0b0..Performance monitoring disabled. Writing of the performance counter registers is enabled.
 *  0b1..Performance monitoring enabled. Writing of the performance counter registers is disabled.
 */
#define IEE_GCFG_MON_EN(x)                       (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_MON_EN_SHIFT)) & IEE_GCFG_MON_EN_MASK)

#define IEE_GCFG_CLR_MON_MASK                    (0x20000000U)
#define IEE_GCFG_CLR_MON_SHIFT                   (29U)
/*! CLR_MON
 *  0b0..Do not reset.
 *  0b1..Reset performance counters.
 */
#define IEE_GCFG_CLR_MON(x)                      (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_CLR_MON_SHIFT)) & IEE_GCFG_CLR_MON_MASK)

#define IEE_GCFG_RST_MASK                        (0x80000000U)
#define IEE_GCFG_RST_SHIFT                       (31U)
/*! RST
 *  0b0..Do Not Reset.
 *  0b1..Reset IEE.
 */
#define IEE_GCFG_RST(x)                          (((uint32_t)(((uint32_t)(x)) << IEE_GCFG_RST_SHIFT)) & IEE_GCFG_RST_MASK)
/*! @} */

/*! @name STA - IEE Status */
/*! @{ */

#define IEE_STA_DSR_MASK                         (0x1U)
#define IEE_STA_DSR_SHIFT                        (0U)
/*! DSR
 *  0b0..No seed request present
 *  0b1..Seed request present
 */
#define IEE_STA_DSR(x)                           (((uint32_t)(((uint32_t)(x)) << IEE_STA_DSR_SHIFT)) & IEE_STA_DSR_MASK)

#define IEE_STA_AFD_MASK                         (0x10U)
#define IEE_STA_AFD_SHIFT                        (4U)
/*! AFD
 *  0b0..No fault detected
 *  0b1..Fault detected
 */
#define IEE_STA_AFD(x)                           (((uint32_t)(((uint32_t)(x)) << IEE_STA_AFD_SHIFT)) & IEE_STA_AFD_MASK)
/*! @} */

/*! @name TSTMD - IEE Test Mode Register */
/*! @{ */

#define IEE_TSTMD_TMRDY_MASK                     (0x1U)
#define IEE_TSTMD_TMRDY_SHIFT                    (0U)
/*! TMRDY
 *  0b0..Not Ready.
 *  0b1..Ready.
 */
#define IEE_TSTMD_TMRDY(x)                       (((uint32_t)(((uint32_t)(x)) << IEE_TSTMD_TMRDY_SHIFT)) & IEE_TSTMD_TMRDY_MASK)

#define IEE_TSTMD_TMR_MASK                       (0x2U)
#define IEE_TSTMD_TMR_SHIFT                      (1U)
/*! TMR
 *  0b0..Not running. May be written if IEE_GCFG[TME] = 1
 *  0b1..Run AES Test until TMDONE is indicated.
 */
#define IEE_TSTMD_TMR(x)                         (((uint32_t)(((uint32_t)(x)) << IEE_TSTMD_TMR_SHIFT)) & IEE_TSTMD_TMR_MASK)

#define IEE_TSTMD_TMENCR_MASK                    (0x4U)
#define IEE_TSTMD_TMENCR_SHIFT                   (2U)
/*! TMENCR
 *  0b0..AES Test mode will do decryption.
 *  0b1..AES Test mode will do encryption.
 */
#define IEE_TSTMD_TMENCR(x)                      (((uint32_t)(((uint32_t)(x)) << IEE_TSTMD_TMENCR_SHIFT)) & IEE_TSTMD_TMENCR_MASK)

#define IEE_TSTMD_TMCONT_MASK                    (0x8U)
#define IEE_TSTMD_TMCONT_SHIFT                   (3U)
/*! TMCONT
 *  0b0..Do not continue. This is the last block of data for AES.
 *  0b1..Continue. Do not initialize AES after this block.
 */
#define IEE_TSTMD_TMCONT(x)                      (((uint32_t)(((uint32_t)(x)) << IEE_TSTMD_TMCONT_SHIFT)) & IEE_TSTMD_TMCONT_MASK)

#define IEE_TSTMD_TMDONE_MASK                    (0x10U)
#define IEE_TSTMD_TMDONE_SHIFT                   (4U)
/*! TMDONE
 *  0b0..Not Done.
 *  0b1..Test Done.
 */
#define IEE_TSTMD_TMDONE(x)                      (((uint32_t)(((uint32_t)(x)) << IEE_TSTMD_TMDONE_SHIFT)) & IEE_TSTMD_TMDONE_MASK)

#define IEE_TSTMD_TMLEN_MASK                     (0xF00U)
#define IEE_TSTMD_TMLEN_SHIFT                    (8U)
#define IEE_TSTMD_TMLEN(x)                       (((uint32_t)(((uint32_t)(x)) << IEE_TSTMD_TMLEN_SHIFT)) & IEE_TSTMD_TMLEN_MASK)
/*! @} */

/*! @name DPAMS - AES Mask Generation Seed */
/*! @{ */

#define IEE_DPAMS_DPAMS_MASK                     (0xFFFFFFFFU)
#define IEE_DPAMS_DPAMS_SHIFT                    (0U)
#define IEE_DPAMS_DPAMS(x)                       (((uint32_t)(((uint32_t)(x)) << IEE_DPAMS_DPAMS_SHIFT)) & IEE_DPAMS_DPAMS_MASK)
/*! @} */

/*! @name PC_S_LT - Performance Counter, AES Slave Latency Threshold Value */
/*! @{ */

#define IEE_PC_S_LT_SW_LT_MASK                   (0xFFFFU)
#define IEE_PC_S_LT_SW_LT_SHIFT                  (0U)
#define IEE_PC_S_LT_SW_LT(x)                     (((uint32_t)(((uint32_t)(x)) << IEE_PC_S_LT_SW_LT_SHIFT)) & IEE_PC_S_LT_SW_LT_MASK)

#define IEE_PC_S_LT_SR_LT_MASK                   (0xFFFF0000U)
#define IEE_PC_S_LT_SR_LT_SHIFT                  (16U)
#define IEE_PC_S_LT_SR_LT(x)                     (((uint32_t)(((uint32_t)(x)) << IEE_PC_S_LT_SR_LT_SHIFT)) & IEE_PC_S_LT_SR_LT_MASK)
/*! @} */

/*! @name PC_M_LT - Performance Counter, AES Master Latency Threshold */
/*! @{ */

#define IEE_PC_M_LT_MW_LT_MASK                   (0xFFFU)
#define IEE_PC_M_LT_MW_LT_SHIFT                  (0U)
#define IEE_PC_M_LT_MW_LT(x)                     (((uint32_t)(((uint32_t)(x)) << IEE_PC_M_LT_MW_LT_SHIFT)) & IEE_PC_M_LT_MW_LT_MASK)

#define IEE_PC_M_LT_MR_LT_MASK                   (0xFFF0000U)
#define IEE_PC_M_LT_MR_LT_SHIFT                  (16U)
#define IEE_PC_M_LT_MR_LT(x)                     (((uint32_t)(((uint32_t)(x)) << IEE_PC_M_LT_MR_LT_SHIFT)) & IEE_PC_M_LT_MR_LT_MASK)
/*! @} */

/*! @name PC_BLK_ENC - Performance Counter, Number of AES Block Encryptions */
/*! @{ */

#define IEE_PC_BLK_ENC_BLK_ENC_MASK              (0xFFFFFFFFU)
#define IEE_PC_BLK_ENC_BLK_ENC_SHIFT             (0U)
#define IEE_PC_BLK_ENC_BLK_ENC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_BLK_ENC_BLK_ENC_SHIFT)) & IEE_PC_BLK_ENC_BLK_ENC_MASK)
/*! @} */

/*! @name PC_BLK_DEC - Performance Counter, Number of AES Block Decryptions */
/*! @{ */

#define IEE_PC_BLK_DEC_BLK_DEC_MASK              (0xFFFFFFFFU)
#define IEE_PC_BLK_DEC_BLK_DEC_SHIFT             (0U)
#define IEE_PC_BLK_DEC_BLK_DEC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_BLK_DEC_BLK_DEC_SHIFT)) & IEE_PC_BLK_DEC_BLK_DEC_MASK)
/*! @} */

/*! @name PC_SR_TRANS - Performance Counter, Number of AXI Slave Read Transactions */
/*! @{ */

#define IEE_PC_SR_TRANS_SR_TRANS_MASK            (0xFFFFFFFFU)
#define IEE_PC_SR_TRANS_SR_TRANS_SHIFT           (0U)
#define IEE_PC_SR_TRANS_SR_TRANS(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TRANS_SR_TRANS_SHIFT)) & IEE_PC_SR_TRANS_SR_TRANS_MASK)
/*! @} */

/*! @name PC_SW_TRANS - Performance Counter, Number of AXI Slave Write Transactions */
/*! @{ */

#define IEE_PC_SW_TRANS_SW_TRANS_MASK            (0xFFFFFFFFU)
#define IEE_PC_SW_TRANS_SW_TRANS_SHIFT           (0U)
#define IEE_PC_SW_TRANS_SW_TRANS(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TRANS_SW_TRANS_SHIFT)) & IEE_PC_SW_TRANS_SW_TRANS_MASK)
/*! @} */

/*! @name PC_MR_TRANS - Performance Counter, Number of AXI Master Read Transactions */
/*! @{ */

#define IEE_PC_MR_TRANS_MR_TRANS_MASK            (0xFFFFFFFFU)
#define IEE_PC_MR_TRANS_MR_TRANS_SHIFT           (0U)
#define IEE_PC_MR_TRANS_MR_TRANS(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_MR_TRANS_MR_TRANS_SHIFT)) & IEE_PC_MR_TRANS_MR_TRANS_MASK)
/*! @} */

/*! @name PC_MW_TRANS - Performance Counter, Number of AXI Master Write Transactions */
/*! @{ */

#define IEE_PC_MW_TRANS_MW_TRANS_MASK            (0xFFFFFFFFU)
#define IEE_PC_MW_TRANS_MW_TRANS_SHIFT           (0U)
#define IEE_PC_MW_TRANS_MW_TRANS(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_MW_TRANS_MW_TRANS_SHIFT)) & IEE_PC_MW_TRANS_MW_TRANS_MASK)
/*! @} */

/*! @name PC_M_MBR - Performance Counter, Number of AXI Master Merge Buffer Read Transactions */
/*! @{ */

#define IEE_PC_M_MBR_M_MBR_MASK                  (0xFFFFFFFFU)
#define IEE_PC_M_MBR_M_MBR_SHIFT                 (0U)
#define IEE_PC_M_MBR_M_MBR(x)                    (((uint32_t)(((uint32_t)(x)) << IEE_PC_M_MBR_M_MBR_SHIFT)) & IEE_PC_M_MBR_M_MBR_MASK)
/*! @} */

/*! @name PC_SR_TBC_U - Performance Counter, Upper Slave Read Transactions Byte Count */
/*! @{ */

#define IEE_PC_SR_TBC_U_SR_TBC_MASK              (0xFFFFU)
#define IEE_PC_SR_TBC_U_SR_TBC_SHIFT             (0U)
#define IEE_PC_SR_TBC_U_SR_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TBC_U_SR_TBC_SHIFT)) & IEE_PC_SR_TBC_U_SR_TBC_MASK)
/*! @} */

/*! @name PC_SR_TBC_L - Performance Counter, Lower Slave Read Transactions Byte Count */
/*! @{ */

#define IEE_PC_SR_TBC_L_SR_TBC_MASK              (0xFFFFFFFFU)
#define IEE_PC_SR_TBC_L_SR_TBC_SHIFT             (0U)
#define IEE_PC_SR_TBC_L_SR_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TBC_L_SR_TBC_SHIFT)) & IEE_PC_SR_TBC_L_SR_TBC_MASK)
/*! @} */

/*! @name PC_SW_TBC_U - Performance Counter, Upper Slave Write Transactions Byte Count */
/*! @{ */

#define IEE_PC_SW_TBC_U_SW_TBC_MASK              (0xFFFFU)
#define IEE_PC_SW_TBC_U_SW_TBC_SHIFT             (0U)
#define IEE_PC_SW_TBC_U_SW_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TBC_U_SW_TBC_SHIFT)) & IEE_PC_SW_TBC_U_SW_TBC_MASK)
/*! @} */

/*! @name PC_SW_TBC_L - Performance Counter, Lower Slave Write Transactions Byte Count */
/*! @{ */

#define IEE_PC_SW_TBC_L_SW_TBC_MASK              (0xFFFFFFFFU)
#define IEE_PC_SW_TBC_L_SW_TBC_SHIFT             (0U)
#define IEE_PC_SW_TBC_L_SW_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TBC_L_SW_TBC_SHIFT)) & IEE_PC_SW_TBC_L_SW_TBC_MASK)
/*! @} */

/*! @name PC_MR_TBC_U - Performance Counter, Upper Master Read Transactions Byte Count */
/*! @{ */

#define IEE_PC_MR_TBC_U_MR_TBC_MASK              (0xFFFFU)
#define IEE_PC_MR_TBC_U_MR_TBC_SHIFT             (0U)
#define IEE_PC_MR_TBC_U_MR_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_MR_TBC_U_MR_TBC_SHIFT)) & IEE_PC_MR_TBC_U_MR_TBC_MASK)
/*! @} */

/*! @name PC_MR_TBC_L - Performance Counter, Lower Master Read Transactions Byte Count */
/*! @{ */

#define IEE_PC_MR_TBC_L_MR_TBC_LSB_MASK          (0xFU)
#define IEE_PC_MR_TBC_L_MR_TBC_LSB_SHIFT         (0U)
#define IEE_PC_MR_TBC_L_MR_TBC_LSB(x)            (((uint32_t)(((uint32_t)(x)) << IEE_PC_MR_TBC_L_MR_TBC_LSB_SHIFT)) & IEE_PC_MR_TBC_L_MR_TBC_LSB_MASK)

#define IEE_PC_MR_TBC_L_MR_TBC_MASK              (0xFFFFFFF0U)
#define IEE_PC_MR_TBC_L_MR_TBC_SHIFT             (4U)
#define IEE_PC_MR_TBC_L_MR_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_MR_TBC_L_MR_TBC_SHIFT)) & IEE_PC_MR_TBC_L_MR_TBC_MASK)
/*! @} */

/*! @name PC_MW_TBC_U - Performance Counter, Upper Master Write Transactions Byte Count */
/*! @{ */

#define IEE_PC_MW_TBC_U_MW_TBC_MASK              (0xFFFFU)
#define IEE_PC_MW_TBC_U_MW_TBC_SHIFT             (0U)
#define IEE_PC_MW_TBC_U_MW_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_MW_TBC_U_MW_TBC_SHIFT)) & IEE_PC_MW_TBC_U_MW_TBC_MASK)
/*! @} */

/*! @name PC_MW_TBC_L - Performance Counter, Lower Master Write Transactions Byte Count */
/*! @{ */

#define IEE_PC_MW_TBC_L_MW_TBC_LSB_MASK          (0xFU)
#define IEE_PC_MW_TBC_L_MW_TBC_LSB_SHIFT         (0U)
#define IEE_PC_MW_TBC_L_MW_TBC_LSB(x)            (((uint32_t)(((uint32_t)(x)) << IEE_PC_MW_TBC_L_MW_TBC_LSB_SHIFT)) & IEE_PC_MW_TBC_L_MW_TBC_LSB_MASK)

#define IEE_PC_MW_TBC_L_MW_TBC_MASK              (0xFFFFFFF0U)
#define IEE_PC_MW_TBC_L_MW_TBC_SHIFT             (4U)
#define IEE_PC_MW_TBC_L_MW_TBC(x)                (((uint32_t)(((uint32_t)(x)) << IEE_PC_MW_TBC_L_MW_TBC_SHIFT)) & IEE_PC_MW_TBC_L_MW_TBC_MASK)
/*! @} */

/*! @name PC_SR_TLGTT - Performance Counter, Number of AXI Slave Read Transactions with Latency Greater than the Threshold */
/*! @{ */

#define IEE_PC_SR_TLGTT_SR_TLGTT_MASK            (0xFFFFFFFFU)
#define IEE_PC_SR_TLGTT_SR_TLGTT_SHIFT           (0U)
#define IEE_PC_SR_TLGTT_SR_TLGTT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TLGTT_SR_TLGTT_SHIFT)) & IEE_PC_SR_TLGTT_SR_TLGTT_MASK)
/*! @} */

/*! @name PC_SW_TLGTT - Performance Counter, Number of AXI Slave Write Transactions with Latency Greater than the Threshold */
/*! @{ */

#define IEE_PC_SW_TLGTT_SW_TLGTT_MASK            (0xFFFFFFFFU)
#define IEE_PC_SW_TLGTT_SW_TLGTT_SHIFT           (0U)
#define IEE_PC_SW_TLGTT_SW_TLGTT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TLGTT_SW_TLGTT_SHIFT)) & IEE_PC_SW_TLGTT_SW_TLGTT_MASK)
/*! @} */

/*! @name PC_MR_TLGTT - Performance Counter, Number of AXI Master Read Transactions with Latency Greater than the Threshold */
/*! @{ */

#define IEE_PC_MR_TLGTT_MR_TLGTT_MASK            (0xFFFFFFFFU)
#define IEE_PC_MR_TLGTT_MR_TLGTT_SHIFT           (0U)
#define IEE_PC_MR_TLGTT_MR_TLGTT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_MR_TLGTT_MR_TLGTT_SHIFT)) & IEE_PC_MR_TLGTT_MR_TLGTT_MASK)
/*! @} */

/*! @name PC_MW_TLGTT - Performance Counter, Number of AXI Master Write Transactions with Latency Greater than the Threshold */
/*! @{ */

#define IEE_PC_MW_TLGTT_MW_TGTT_MASK             (0xFFFFFFFFU)
#define IEE_PC_MW_TLGTT_MW_TGTT_SHIFT            (0U)
#define IEE_PC_MW_TLGTT_MW_TGTT(x)               (((uint32_t)(((uint32_t)(x)) << IEE_PC_MW_TLGTT_MW_TGTT_SHIFT)) & IEE_PC_MW_TLGTT_MW_TGTT_MASK)
/*! @} */

/*! @name PC_SR_TLAT_U - Performance Counter, Upper Slave Read Latency Count */
/*! @{ */

#define IEE_PC_SR_TLAT_U_SR_TLAT_MASK            (0xFFFFU)
#define IEE_PC_SR_TLAT_U_SR_TLAT_SHIFT           (0U)
#define IEE_PC_SR_TLAT_U_SR_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TLAT_U_SR_TLAT_SHIFT)) & IEE_PC_SR_TLAT_U_SR_TLAT_MASK)
/*! @} */

/*! @name PC_SR_TLAT_L - Performance Counter, Lower Slave Read Latency Count */
/*! @{ */

#define IEE_PC_SR_TLAT_L_SR_TLAT_MASK            (0xFFFFFFFFU)
#define IEE_PC_SR_TLAT_L_SR_TLAT_SHIFT           (0U)
#define IEE_PC_SR_TLAT_L_SR_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TLAT_L_SR_TLAT_SHIFT)) & IEE_PC_SR_TLAT_L_SR_TLAT_MASK)
/*! @} */

/*! @name PC_SW_TLAT_U - Performance Counter, Upper Slave Write Latency Count */
/*! @{ */

#define IEE_PC_SW_TLAT_U_SW_TLAT_MASK            (0xFFFFU)
#define IEE_PC_SW_TLAT_U_SW_TLAT_SHIFT           (0U)
#define IEE_PC_SW_TLAT_U_SW_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TLAT_U_SW_TLAT_SHIFT)) & IEE_PC_SW_TLAT_U_SW_TLAT_MASK)
/*! @} */

/*! @name PC_SW_TLAT_L - Performance Counter, Lower Slave Write Latency Count */
/*! @{ */

#define IEE_PC_SW_TLAT_L_SW_TLAT_MASK            (0xFFFFFFFFU)
#define IEE_PC_SW_TLAT_L_SW_TLAT_SHIFT           (0U)
#define IEE_PC_SW_TLAT_L_SW_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TLAT_L_SW_TLAT_SHIFT)) & IEE_PC_SW_TLAT_L_SW_TLAT_MASK)
/*! @} */

/*! @name PC_MR_TLAT_U - Performance Counter, Upper Master Read Latency Count */
/*! @{ */

#define IEE_PC_MR_TLAT_U_MR_TLAT_MASK            (0xFFFFU)
#define IEE_PC_MR_TLAT_U_MR_TLAT_SHIFT           (0U)
#define IEE_PC_MR_TLAT_U_MR_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_MR_TLAT_U_MR_TLAT_SHIFT)) & IEE_PC_MR_TLAT_U_MR_TLAT_MASK)
/*! @} */

/*! @name PC_MR_TLAT_L - Performance Counter, Lower Master Read Latency Count */
/*! @{ */

#define IEE_PC_MR_TLAT_L_MR_TLAT_MASK            (0xFFFFFFFFU)
#define IEE_PC_MR_TLAT_L_MR_TLAT_SHIFT           (0U)
#define IEE_PC_MR_TLAT_L_MR_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_MR_TLAT_L_MR_TLAT_SHIFT)) & IEE_PC_MR_TLAT_L_MR_TLAT_MASK)
/*! @} */

/*! @name PC_MW_TLAT_U - Performance Counter, Upper Master Write Latency Count */
/*! @{ */

#define IEE_PC_MW_TLAT_U_MW_TLAT_MASK            (0xFFFFU)
#define IEE_PC_MW_TLAT_U_MW_TLAT_SHIFT           (0U)
#define IEE_PC_MW_TLAT_U_MW_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_MW_TLAT_U_MW_TLAT_SHIFT)) & IEE_PC_MW_TLAT_U_MW_TLAT_MASK)
/*! @} */

/*! @name PC_MW_TLAT_L - Performance Counter, Lower Master Write Latency Count */
/*! @{ */

#define IEE_PC_MW_TLAT_L_MW_TLAT_MASK            (0xFFFFFFFFU)
#define IEE_PC_MW_TLAT_L_MW_TLAT_SHIFT           (0U)
#define IEE_PC_MW_TLAT_L_MW_TLAT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_MW_TLAT_L_MW_TLAT_SHIFT)) & IEE_PC_MW_TLAT_L_MW_TLAT_MASK)
/*! @} */

/*! @name PC_SR_TNRT_U - Performance Counter, Upper Slave Read Total Non-Responding Time */
/*! @{ */

#define IEE_PC_SR_TNRT_U_SR_TNRT_MASK            (0xFFFFU)
#define IEE_PC_SR_TNRT_U_SR_TNRT_SHIFT           (0U)
#define IEE_PC_SR_TNRT_U_SR_TNRT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TNRT_U_SR_TNRT_SHIFT)) & IEE_PC_SR_TNRT_U_SR_TNRT_MASK)
/*! @} */

/*! @name PC_SR_TNRT_L - Performance Counter, Lower Slave Read Total Non-Responding Time */
/*! @{ */

#define IEE_PC_SR_TNRT_L_SR_TNRT_MASK            (0xFFFFFFFFU)
#define IEE_PC_SR_TNRT_L_SR_TNRT_SHIFT           (0U)
#define IEE_PC_SR_TNRT_L_SR_TNRT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SR_TNRT_L_SR_TNRT_SHIFT)) & IEE_PC_SR_TNRT_L_SR_TNRT_MASK)
/*! @} */

/*! @name PC_SW_TNRT_U - Performance Counter, Upper Slave Write Total Non-Responding Time */
/*! @{ */

#define IEE_PC_SW_TNRT_U_SW_TNRT_MASK            (0xFFFFU)
#define IEE_PC_SW_TNRT_U_SW_TNRT_SHIFT           (0U)
#define IEE_PC_SW_TNRT_U_SW_TNRT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TNRT_U_SW_TNRT_SHIFT)) & IEE_PC_SW_TNRT_U_SW_TNRT_MASK)
/*! @} */

/*! @name PC_SW_TNRT_L - Performance Counter, Lower Slave Write Total Non-Responding Time */
/*! @{ */

#define IEE_PC_SW_TNRT_L_SW_TNRT_MASK            (0xFFFFFFFFU)
#define IEE_PC_SW_TNRT_L_SW_TNRT_SHIFT           (0U)
#define IEE_PC_SW_TNRT_L_SW_TNRT(x)              (((uint32_t)(((uint32_t)(x)) << IEE_PC_SW_TNRT_L_SW_TNRT_SHIFT)) & IEE_PC_SW_TNRT_L_SW_TNRT_MASK)
/*! @} */

/*! @name VIDR1 - IEE Version ID Register 1 */
/*! @{ */

#define IEE_VIDR1_MIN_REV_MASK                   (0xFFU)
#define IEE_VIDR1_MIN_REV_SHIFT                  (0U)
#define IEE_VIDR1_MIN_REV(x)                     (((uint32_t)(((uint32_t)(x)) << IEE_VIDR1_MIN_REV_SHIFT)) & IEE_VIDR1_MIN_REV_MASK)

#define IEE_VIDR1_MAJ_REV_MASK                   (0xFF00U)
#define IEE_VIDR1_MAJ_REV_SHIFT                  (8U)
#define IEE_VIDR1_MAJ_REV(x)                     (((uint32_t)(((uint32_t)(x)) << IEE_VIDR1_MAJ_REV_SHIFT)) & IEE_VIDR1_MAJ_REV_MASK)

#define IEE_VIDR1_IP_ID_MASK                     (0xFFFF0000U)
#define IEE_VIDR1_IP_ID_SHIFT                    (16U)
#define IEE_VIDR1_IP_ID(x)                       (((uint32_t)(((uint32_t)(x)) << IEE_VIDR1_IP_ID_SHIFT)) & IEE_VIDR1_IP_ID_MASK)
/*! @} */

/*! @name AESVID - IEE AES Version ID Register */
/*! @{ */

#define IEE_AESVID_AESRN_MASK                    (0xFU)
#define IEE_AESVID_AESRN_SHIFT                   (0U)
#define IEE_AESVID_AESRN(x)                      (((uint32_t)(((uint32_t)(x)) << IEE_AESVID_AESRN_SHIFT)) & IEE_AESVID_AESRN_MASK)

#define IEE_AESVID_AESVID_MASK                   (0xF0U)
#define IEE_AESVID_AESVID_SHIFT                  (4U)
#define IEE_AESVID_AESVID(x)                     (((uint32_t)(((uint32_t)(x)) << IEE_AESVID_AESVID_SHIFT)) & IEE_AESVID_AESVID_MASK)
/*! @} */

/*! @name REGATTR - IEE Region 0 Attribute Register...IEE Region 7 Attribute Register. */
/*! @{ */

#define IEE_REGATTR_KS_MASK                      (0x1U)
#define IEE_REGATTR_KS_SHIFT                     (0U)
/*! KS
 *  0b0..128 bits (CTR), 256 bits (XTS).
 *  0b1..256 bits (CTR), 512 bits (XTS).
 */
#define IEE_REGATTR_KS(x)                        (((uint32_t)(((uint32_t)(x)) << IEE_REGATTR_KS_SHIFT)) & IEE_REGATTR_KS_MASK)

#define IEE_REGATTR_MD_MASK                      (0x70U)
#define IEE_REGATTR_MD_SHIFT                     (4U)
/*! MD
 *  0b000..None (AXI error if accessed)
 *  0b001..XTS
 *  0b010..CTR w/ address binding
 *  0b011..CTR w/o address binding
 *  0b100..CTR keystream only
 *  0b101..Undefined, AXI error if used
 *  0b110..Undefined, AXI error if used
 *  0b111..Undefined, AXI error if used
 */
#define IEE_REGATTR_MD(x)                        (((uint32_t)(((uint32_t)(x)) << IEE_REGATTR_MD_SHIFT)) & IEE_REGATTR_MD_MASK)

#define IEE_REGATTR_BYP_MASK                     (0x80U)
#define IEE_REGATTR_BYP_SHIFT                    (7U)
/*! BYP
 *  0b0..use MD field
 *  0b1..Bypass AES, no encrypt/decrypt
 */
#define IEE_REGATTR_BYP(x)                       (((uint32_t)(((uint32_t)(x)) << IEE_REGATTR_BYP_SHIFT)) & IEE_REGATTR_BYP_MASK)
/*! @} */

/* The count of IEE_REGATTR */
#define IEE_REGATTR_COUNT                        (8U)

/*! @name REGPO - IEE Region 0 Page Offset Register..IEE Region 7 Page Offset Register */
/*! @{ */

#define IEE_REGPO_PGOFF_MASK                     (0xFFFFFFU)
#define IEE_REGPO_PGOFF_SHIFT                    (0U)
#define IEE_REGPO_PGOFF(x)                       (((uint32_t)(((uint32_t)(x)) << IEE_REGPO_PGOFF_SHIFT)) & IEE_REGPO_PGOFF_MASK)
/*! @} */

/* The count of IEE_REGPO */
#define IEE_REGPO_COUNT                          (8U)

/*! @name REGKEY1 - IEE Region 0 Key 1 Register..IEE Region 7 Key 1 Register */
/*! @{ */

#define IEE_REGKEY1_KEY1_MASK                    (0xFFFFFFFFU)
#define IEE_REGKEY1_KEY1_SHIFT                   (0U)
#define IEE_REGKEY1_KEY1(x)                      (((uint32_t)(((uint32_t)(x)) << IEE_REGKEY1_KEY1_SHIFT)) & IEE_REGKEY1_KEY1_MASK)
/*! @} */

/* The count of IEE_REGKEY1 */
#define IEE_REGKEY1_COUNT                        (8U)

/* The count of IEE_REGKEY1 */
#define IEE_REGKEY1_COUNT2                       (8U)

/*! @name REGKEY2 - IEE Region 0 Key 2 Register..IEE Region 7 Key 2 Register */
/*! @{ */

#define IEE_REGKEY2_KEY2_MASK                    (0xFFFFFFFFU)
#define IEE_REGKEY2_KEY2_SHIFT                   (0U)
#define IEE_REGKEY2_KEY2(x)                      (((uint32_t)(((uint32_t)(x)) << IEE_REGKEY2_KEY2_SHIFT)) & IEE_REGKEY2_KEY2_MASK)
/*! @} */

/* The count of IEE_REGKEY2 */
#define IEE_REGKEY2_COUNT                        (8U)

/* The count of IEE_REGKEY2 */
#define IEE_REGKEY2_COUNT2                       (8U)

/*! @name AES_TST_DB - IEE AES Test Mode Data Buffer */
/*! @{ */

#define IEE_AES_TST_DB_AES_TST_DB0_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB0_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB0(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB0_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB0_MASK)

#define IEE_AES_TST_DB_AES_TST_DB1_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB1_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB1(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB1_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB1_MASK)

#define IEE_AES_TST_DB_AES_TST_DB2_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB2_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB2(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB2_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB2_MASK)

#define IEE_AES_TST_DB_AES_TST_DB3_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB3_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB3(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB3_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB3_MASK)

#define IEE_AES_TST_DB_AES_TST_DB4_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB4_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB4(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB4_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB4_MASK)

#define IEE_AES_TST_DB_AES_TST_DB5_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB5_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB5(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB5_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB5_MASK)

#define IEE_AES_TST_DB_AES_TST_DB6_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB6_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB6(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB6_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB6_MASK)

#define IEE_AES_TST_DB_AES_TST_DB7_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB7_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB7(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB7_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB7_MASK)

#define IEE_AES_TST_DB_AES_TST_DB8_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB8_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB8(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB8_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB8_MASK)

#define IEE_AES_TST_DB_AES_TST_DB9_MASK          (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB9_SHIFT         (0U)
#define IEE_AES_TST_DB_AES_TST_DB9(x)            (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB9_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB9_MASK)

#define IEE_AES_TST_DB_AES_TST_DB10_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB10_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB10(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB10_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB10_MASK)

#define IEE_AES_TST_DB_AES_TST_DB11_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB11_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB11(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB11_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB11_MASK)

#define IEE_AES_TST_DB_AES_TST_DB12_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB12_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB12(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB12_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB12_MASK)

#define IEE_AES_TST_DB_AES_TST_DB13_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB13_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB13(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB13_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB13_MASK)

#define IEE_AES_TST_DB_AES_TST_DB14_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB14_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB14(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB14_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB14_MASK)

#define IEE_AES_TST_DB_AES_TST_DB15_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB15_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB15(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB15_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB15_MASK)

#define IEE_AES_TST_DB_AES_TST_DB16_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB16_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB16(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB16_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB16_MASK)

#define IEE_AES_TST_DB_AES_TST_DB17_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB17_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB17(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB17_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB17_MASK)

#define IEE_AES_TST_DB_AES_TST_DB18_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB18_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB18(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB18_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB18_MASK)

#define IEE_AES_TST_DB_AES_TST_DB19_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB19_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB19(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB19_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB19_MASK)

#define IEE_AES_TST_DB_AES_TST_DB20_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB20_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB20(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB20_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB20_MASK)

#define IEE_AES_TST_DB_AES_TST_DB21_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB21_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB21(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB21_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB21_MASK)

#define IEE_AES_TST_DB_AES_TST_DB22_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB22_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB22(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB22_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB22_MASK)

#define IEE_AES_TST_DB_AES_TST_DB23_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB23_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB23(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB23_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB23_MASK)

#define IEE_AES_TST_DB_AES_TST_DB24_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB24_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB24(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB24_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB24_MASK)

#define IEE_AES_TST_DB_AES_TST_DB25_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB25_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB25(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB25_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB25_MASK)

#define IEE_AES_TST_DB_AES_TST_DB26_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB26_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB26(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB26_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB26_MASK)

#define IEE_AES_TST_DB_AES_TST_DB27_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB27_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB27(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB27_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB27_MASK)

#define IEE_AES_TST_DB_AES_TST_DB28_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB28_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB28(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB28_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB28_MASK)

#define IEE_AES_TST_DB_AES_TST_DB29_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB29_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB29(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB29_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB29_MASK)

#define IEE_AES_TST_DB_AES_TST_DB30_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB30_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB30(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB30_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB30_MASK)

#define IEE_AES_TST_DB_AES_TST_DB31_MASK         (0xFFFFFFFFU)
#define IEE_AES_TST_DB_AES_TST_DB31_SHIFT        (0U)
#define IEE_AES_TST_DB_AES_TST_DB31(x)           (((uint32_t)(((uint32_t)(x)) << IEE_AES_TST_DB_AES_TST_DB31_SHIFT)) & IEE_AES_TST_DB_AES_TST_DB31_MASK)
/*! @} */

/* The count of IEE_AES_TST_DB */
#define IEE_AES_TST_DB_COUNT                     (32U)


/*!
 * @}
 */ /* end of group IEE_Register_Masks */


/* IEE - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral IEE base address */
  #define IEE_BASE                                 (0x52E40000u)
  /** Peripheral IEE base address */
  #define IEE_BASE_NS                              (0x42E40000u)
  /** Peripheral IEE base pointer */
  #define IEE                                      ((IEE_Type *)IEE_BASE)
  /** Peripheral IEE base pointer */
  #define IEE_NS                                   ((IEE_Type *)IEE_BASE_NS)
  /** Array initializer of IEE peripheral base addresses */
  #define IEE_BASE_ADDRS                           { IEE_BASE }
  /** Array initializer of IEE peripheral base pointers */
  #define IEE_BASE_PTRS                            { IEE }
  /** Array initializer of IEE peripheral base addresses */
  #define IEE_BASE_ADDRS_NS                        { IEE_BASE_NS }
  /** Array initializer of IEE peripheral base pointers */
  #define IEE_BASE_PTRS_NS                         { IEE_NS }
#else
  /** Peripheral IEE base address */
  #define IEE_BASE                                 (0x42E40000u)
  /** Peripheral IEE base pointer */
  #define IEE                                      ((IEE_Type *)IEE_BASE)
  /** Array initializer of IEE peripheral base addresses */
  #define IEE_BASE_ADDRS                           { IEE_BASE }
  /** Array initializer of IEE peripheral base pointers */
  #define IEE_BASE_PTRS                            { IEE }
#endif

/*!
 * @}
 */ /* end of group IEE_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- IEE_APC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IEE_APC_Peripheral_Access_Layer IEE_APC Peripheral Access Layer
 * @{
 */

/** IEE_APC - Register Layout Typedef */
typedef struct {
  __IO uint32_t REGION0_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x0 */
  __IO uint32_t REGION0_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x4 */
  __IO uint32_t REGION0_ENA;                       /**< Region enable for region (n), offset: 0x8 */
  __IO uint32_t REGION0_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0xC */
  __IO uint32_t REGION1_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x10 */
  __IO uint32_t REGION1_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x14 */
  __IO uint32_t REGION1_ENA;                       /**< Region enable for region (n), offset: 0x18 */
  __IO uint32_t REGION1_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0x1C */
  __IO uint32_t REGION2_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x20 */
  __IO uint32_t REGION2_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x24 */
  __IO uint32_t REGION2_ENA;                       /**< Region enable for region (n), offset: 0x28 */
  __IO uint32_t REGION2_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0x2C */
  __IO uint32_t REGION3_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x30 */
  __IO uint32_t REGION3_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x34 */
  __IO uint32_t REGION3_ENA;                       /**< Region enable for region (n), offset: 0x38 */
  __IO uint32_t REGION3_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0x3C */
  __IO uint32_t REGION4_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x40 */
  __IO uint32_t REGION4_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x44 */
  __IO uint32_t REGION4_ENA;                       /**< Region enable for region (n), offset: 0x48 */
  __IO uint32_t REGION4_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0x4C */
  __IO uint32_t REGION5_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x50 */
  __IO uint32_t REGION5_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x54 */
  __IO uint32_t REGION5_ENA;                       /**< Region enable for region (n), offset: 0x58 */
  __IO uint32_t REGION5_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0x5C */
  __IO uint32_t REGION6_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x60 */
  __IO uint32_t REGION6_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x64 */
  __IO uint32_t REGION6_ENA;                       /**< Region enable for region (n), offset: 0x68 */
  __IO uint32_t REGION6_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0x6C */
  __IO uint32_t REGION7_TOP_ADDR;                  /**< End address of IEE region (n), offset: 0x70 */
  __IO uint32_t REGION7_BOT_ADDR;                  /**< Start address of IEE region (n), offset: 0x74 */
  __IO uint32_t REGION7_ENA;                       /**< Region enable for region (n), offset: 0x78 */
  __IO uint32_t REGION7_ACC_CTL;                   /**< Access control for IEE APC registers of region (n), offset: 0x7C */
} IEE_APC_Type;

/* ----------------------------------------------------------------------------
   -- IEE_APC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IEE_APC_Register_Masks IEE_APC Register Masks
 * @{
 */

/*! @name REGION0_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION0_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION0_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION0_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION0_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION0_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION0_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION0_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION0_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION0_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION0_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION0_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION0_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION0_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION0_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION0_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION0_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION0_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION0_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION0_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION0_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION0_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION0_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION0_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION0_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION0_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION0_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION0_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION0_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION0_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION0_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION0_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION0_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION0_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION0_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION0_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION0_ACC_CTL_LOCK_H_MASK)
/*! @} */

/*! @name REGION1_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION1_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION1_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION1_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION1_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION1_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION1_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION1_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION1_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION1_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION1_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION1_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION1_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION1_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION1_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION1_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION1_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION1_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION1_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION1_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION1_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION1_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION1_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION1_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION1_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION1_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION1_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION1_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION1_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION1_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION1_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION1_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION1_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION1_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION1_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION1_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION1_ACC_CTL_LOCK_H_MASK)
/*! @} */

/*! @name REGION2_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION2_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION2_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION2_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION2_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION2_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION2_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION2_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION2_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION2_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION2_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION2_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION2_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION2_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION2_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION2_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION2_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION2_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION2_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION2_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION2_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION2_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION2_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION2_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION2_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION2_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION2_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION2_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION2_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION2_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION2_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION2_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION2_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION2_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION2_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION2_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION2_ACC_CTL_LOCK_H_MASK)
/*! @} */

/*! @name REGION3_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION3_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION3_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION3_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION3_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION3_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION3_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION3_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION3_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION3_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION3_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION3_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION3_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION3_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION3_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION3_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION3_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION3_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION3_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION3_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION3_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION3_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION3_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION3_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION3_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION3_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION3_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION3_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION3_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION3_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION3_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION3_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION3_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION3_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION3_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION3_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION3_ACC_CTL_LOCK_H_MASK)
/*! @} */

/*! @name REGION4_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION4_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION4_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION4_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION4_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION4_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION4_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION4_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION4_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION4_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION4_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION4_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION4_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION4_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION4_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION4_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION4_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION4_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION4_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION4_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION4_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION4_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION4_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION4_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION4_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION4_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION4_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION4_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION4_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION4_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION4_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION4_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION4_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION4_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION4_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION4_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION4_ACC_CTL_LOCK_H_MASK)
/*! @} */

/*! @name REGION5_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION5_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION5_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION5_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION5_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION5_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION5_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION5_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION5_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION5_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION5_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION5_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION5_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION5_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION5_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION5_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION5_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION5_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION5_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION5_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION5_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION5_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION5_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION5_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION5_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION5_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION5_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION5_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION5_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION5_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION5_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION5_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION5_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION5_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION5_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION5_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION5_ACC_CTL_LOCK_H_MASK)
/*! @} */

/*! @name REGION6_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION6_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION6_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION6_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION6_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION6_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION6_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION6_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION6_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION6_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION6_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION6_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION6_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION6_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION6_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION6_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION6_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION6_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION6_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION6_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION6_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION6_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION6_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION6_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION6_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION6_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION6_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION6_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION6_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION6_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION6_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION6_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION6_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION6_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION6_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION6_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION6_ACC_CTL_LOCK_H_MASK)
/*! @} */

/*! @name REGION7_TOP_ADDR - End address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION7_TOP_ADDR_TOP_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION7_TOP_ADDR_TOP_ADDR_SHIFT  (6U)
/*! TOP_ADDR - End address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION7_TOP_ADDR_TOP_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_TOP_ADDR_TOP_ADDR_SHIFT)) & IEE_APC_REGION7_TOP_ADDR_TOP_ADDR_MASK)
/*! @} */

/*! @name REGION7_BOT_ADDR - Start address of IEE region (n) */
/*! @{ */

#define IEE_APC_REGION7_BOT_ADDR_BOT_ADDR_MASK   (0xFFFFFFC0U)
#define IEE_APC_REGION7_BOT_ADDR_BOT_ADDR_SHIFT  (6U)
/*! BOT_ADDR - Start address[31:6] of IEE region. The lower 6 address bits of IEE region is always 0. */
#define IEE_APC_REGION7_BOT_ADDR_BOT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_BOT_ADDR_BOT_ADDR_SHIFT)) & IEE_APC_REGION7_BOT_ADDR_BOT_ADDR_MASK)
/*! @} */

/*! @name REGION7_ENA - Region enable for region (n) */
/*! @{ */

#define IEE_APC_REGION7_ENA_ENCRYPT_ENABLE_MASK  (0x1U)
#define IEE_APC_REGION7_ENA_ENCRYPT_ENABLE_SHIFT (0U)
/*! ENCRYPT_ENABLE - Enable this region
 *  0b1..This region is enabled for IEE routing once hit
 *  0b0..This region is not enabled for IEE routing even hit
 */
#define IEE_APC_REGION7_ENA_ENCRYPT_ENABLE(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_ENA_ENCRYPT_ENABLE_SHIFT)) & IEE_APC_REGION7_ENA_ENCRYPT_ENABLE_MASK)
/*! @} */

/*! @name REGION7_ACC_CTL - Access control for IEE APC registers of region (n) */
/*! @{ */

#define IEE_APC_REGION7_ACC_CTL_ALLOW_DID_MASK   (0xFU)
#define IEE_APC_REGION7_ACC_CTL_ALLOW_DID_SHIFT  (0U)
/*! ALLOW_DID - Allowed domain ID */
#define IEE_APC_REGION7_ACC_CTL_ALLOW_DID(x)     (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_ACC_CTL_ALLOW_DID_SHIFT)) & IEE_APC_REGION7_ACC_CTL_ALLOW_DID_MASK)

#define IEE_APC_REGION7_ACC_CTL_LOCK_L_MASK      (0x8000U)
#define IEE_APC_REGION7_ACC_CTL_LOCK_L_SHIFT     (15U)
/*! LOCK_L - Lock bit for the lower half word
 *  0b0..Lower half word is not locked
 *  0b1..Lower half word is locked
 */
#define IEE_APC_REGION7_ACC_CTL_LOCK_L(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_ACC_CTL_LOCK_L_SHIFT)) & IEE_APC_REGION7_ACC_CTL_LOCK_L_MASK)

#define IEE_APC_REGION7_ACC_CTL_ALLOW_NS_MASK    (0x10000U)
#define IEE_APC_REGION7_ACC_CTL_ALLOW_NS_SHIFT   (16U)
/*! ALLOW_NS - Allow nonsecure mode access
 *  0b1..Secure and nonsecure access to this region's registers is allowed
 *  0b0..Only secure access to this region's registers is allowed
 */
#define IEE_APC_REGION7_ACC_CTL_ALLOW_NS(x)      (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_ACC_CTL_ALLOW_NS_SHIFT)) & IEE_APC_REGION7_ACC_CTL_ALLOW_NS_MASK)

#define IEE_APC_REGION7_ACC_CTL_ALLOW_USER_MASK  (0x20000U)
#define IEE_APC_REGION7_ACC_CTL_ALLOW_USER_SHIFT (17U)
/*! ALLOW_USER - Allow user mode access
 *  0b1..User and privilege access to this region's registers is allowed
 *  0b0..Only privilege access to this region's registers is allowed
 */
#define IEE_APC_REGION7_ACC_CTL_ALLOW_USER(x)    (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_ACC_CTL_ALLOW_USER_SHIFT)) & IEE_APC_REGION7_ACC_CTL_ALLOW_USER_MASK)

#define IEE_APC_REGION7_ACC_CTL_LOCK_H_MASK      (0x80000000U)
#define IEE_APC_REGION7_ACC_CTL_LOCK_H_SHIFT     (31U)
/*! LOCK_H - Lock bit for the higher half word
 *  0b0..Higher half word is not locked
 *  0b1..Higher half word is locked
 */
#define IEE_APC_REGION7_ACC_CTL_LOCK_H(x)        (((uint32_t)(((uint32_t)(x)) << IEE_APC_REGION7_ACC_CTL_LOCK_H_SHIFT)) & IEE_APC_REGION7_ACC_CTL_LOCK_H_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group IEE_APC_Register_Masks */


/* IEE_APC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral IEE_APC base address */
  #define IEE_APC_BASE                             (0x52E44000u)
  /** Peripheral IEE_APC base address */
  #define IEE_APC_BASE_NS                          (0x42E44000u)
  /** Peripheral IEE_APC base pointer */
  #define IEE_APC                                  ((IEE_APC_Type *)IEE_APC_BASE)
  /** Peripheral IEE_APC base pointer */
  #define IEE_APC_NS                               ((IEE_APC_Type *)IEE_APC_BASE_NS)
  /** Array initializer of IEE_APC peripheral base addresses */
  #define IEE_APC_BASE_ADDRS                       { IEE_APC_BASE }
  /** Array initializer of IEE_APC peripheral base pointers */
  #define IEE_APC_BASE_PTRS                        { IEE_APC }
  /** Array initializer of IEE_APC peripheral base addresses */
  #define IEE_APC_BASE_ADDRS_NS                    { IEE_APC_BASE_NS }
  /** Array initializer of IEE_APC peripheral base pointers */
  #define IEE_APC_BASE_PTRS_NS                     { IEE_APC_NS }
#else
  /** Peripheral IEE_APC base address */
  #define IEE_APC_BASE                             (0x42E44000u)
  /** Peripheral IEE_APC base pointer */
  #define IEE_APC                                  ((IEE_APC_Type *)IEE_APC_BASE)
  /** Array initializer of IEE_APC peripheral base addresses */
  #define IEE_APC_BASE_ADDRS                       { IEE_APC_BASE }
  /** Array initializer of IEE_APC peripheral base pointers */
  #define IEE_APC_BASE_PTRS                        { IEE_APC }
#endif

/*!
 * @}
 */ /* end of group IEE_APC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- IERC_IERB Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IERC_IERB_Peripheral_Access_Layer IERC_IERB Peripheral Access Layer
 * @{
 */

/** IERC_IERB - Register Layout Typedef */
typedef struct {
  __IO uint32_t F0_EC_CFH_DIDVID;                  /**< Function 0 EC config header device ID and vendor ID register, offset: 0x0 */
  __IO uint32_t F0_EC_CFH_SIDSVID;                 /**< Function 0 EC config header subsystem ID and subsystem vendor ID register, offset: 0x4 */
} IERC_IERB_Type;

/* ----------------------------------------------------------------------------
   -- IERC_IERB Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IERC_IERB_Register_Masks IERC_IERB Register Masks
 * @{
 */

/*! @name F0_EC_CFH_DIDVID - Function 0 EC config header device ID and vendor ID register */
/*! @{ */

#define IERC_IERB_F0_EC_CFH_DIDVID_VENDOR_ID_MASK (0xFFFFU)
#define IERC_IERB_F0_EC_CFH_DIDVID_VENDOR_ID_SHIFT (0U)
#define IERC_IERB_F0_EC_CFH_DIDVID_VENDOR_ID(x)  (((uint32_t)(((uint32_t)(x)) << IERC_IERB_F0_EC_CFH_DIDVID_VENDOR_ID_SHIFT)) & IERC_IERB_F0_EC_CFH_DIDVID_VENDOR_ID_MASK)

#define IERC_IERB_F0_EC_CFH_DIDVID_DEVICE_ID_MASK (0xFFFF0000U)
#define IERC_IERB_F0_EC_CFH_DIDVID_DEVICE_ID_SHIFT (16U)
#define IERC_IERB_F0_EC_CFH_DIDVID_DEVICE_ID(x)  (((uint32_t)(((uint32_t)(x)) << IERC_IERB_F0_EC_CFH_DIDVID_DEVICE_ID_SHIFT)) & IERC_IERB_F0_EC_CFH_DIDVID_DEVICE_ID_MASK)
/*! @} */

/*! @name F0_EC_CFH_SIDSVID - Function 0 EC config header subsystem ID and subsystem vendor ID register */
/*! @{ */

#define IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT (0U)
#define IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT)) & IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK)

#define IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK (0xFFFF0000U)
#define IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT (16U)
#define IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT)) & IERC_IERB_F0_EC_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group IERC_IERB_Register_Masks */


/* IERC_IERB - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral IERC_IERB base address */
  #define IERC_IERB_BASE                           (0x70810000u)
  /** Peripheral IERC_IERB base address */
  #define IERC_IERB_BASE_NS                        (0x60810000u)
  /** Peripheral IERC_IERB base pointer */
  #define IERC_IERB                                ((IERC_IERB_Type *)IERC_IERB_BASE)
  /** Peripheral IERC_IERB base pointer */
  #define IERC_IERB_NS                             ((IERC_IERB_Type *)IERC_IERB_BASE_NS)
  /** Array initializer of IERC_IERB peripheral base addresses */
  #define IERC_IERB_BASE_ADDRS                     { IERC_IERB_BASE }
  /** Array initializer of IERC_IERB peripheral base pointers */
  #define IERC_IERB_BASE_PTRS                      { IERC_IERB }
  /** Array initializer of IERC_IERB peripheral base addresses */
  #define IERC_IERB_BASE_ADDRS_NS                  { IERC_IERB_BASE_NS }
  /** Array initializer of IERC_IERB peripheral base pointers */
  #define IERC_IERB_BASE_PTRS_NS                   { IERC_IERB_NS }
#else
  /** Peripheral IERC_IERB base address */
  #define IERC_IERB_BASE                           (0x60810000u)
  /** Peripheral IERC_IERB base pointer */
  #define IERC_IERB                                ((IERC_IERB_Type *)IERC_IERB_BASE)
  /** Array initializer of IERC_IERB peripheral base addresses */
  #define IERC_IERB_BASE_ADDRS                     { IERC_IERB_BASE }
  /** Array initializer of IERC_IERB peripheral base pointers */
  #define IERC_IERB_BASE_PTRS                      { IERC_IERB }
#endif

/*!
 * @}
 */ /* end of group IERC_IERB_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- IERC_PCI Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IERC_PCI_Peripheral_Access_Layer IERC_PCI Peripheral Access Layer
 * @{
 */

/** IERC_PCI - Register Layout Typedef */
typedef struct {
  __I  uint32_t PCI_CFH_DID_VID;                   /**< PCI device ID and vendor ID register, offset: 0x0 */
  __IO uint16_t PCI_CFH_CMD;                       /**< PCI command register, offset: 0x4 */
  __I  uint16_t PCI_CFH_STAT;                      /**< PCI status register, offset: 0x6 */
  __I  uint32_t PCI_CFH_REVID_CLASSCODE;           /**< PCI revision ID and classcode register, offset: 0x8 */
  __IO uint8_t PCI_CFH_CL_SIZE;                    /**< PCI cache line size register, offset: 0xC */
       uint8_t RESERVED_0[1];
  __I  uint8_t PCI_CFH_HDR_TYPE;                   /**< PCI header type register, offset: 0xE */
       uint8_t RESERVED_1[29];
  __I  uint16_t PCI_CFH_SUBSYS_VID;                /**< PCI subsystem vendor ID register, offset: 0x2C */
  __I  uint16_t PCI_CFH_SUBSYS_ID;                 /**< PCI subsystem ID register, offset: 0x2E */
       uint8_t RESERVED_2[4];
  __I  uint8_t PCI_CFH_CAP_PTR;                    /**< PCI capabilities pointer register, offset: 0x34 */
       uint8_t RESERVED_3[7];
  __IO uint8_t PCI_CFH_INT_LINE;                   /**< PCI interrupt line register, offset: 0x3C */
  __I  uint8_t PCI_CFH_INT_PIN;                    /**< PCI interrupt pin register, offset: 0x3D */
       uint8_t RESERVED_4[2];
  __I  uint16_t PCI_CFC_PCIE_CAP_LIST;             /**< PCI PCIe capabilities list register, offset: 0x40 */
  __I  uint16_t PCI_CFC_PCIE_CAP;                  /**< PCI PCIe capabilities register, offset: 0x42 */
  __I  uint32_t PCI_CFC_PCIE_DEV_CAP;              /**< PCI PCIe device capabilities register, offset: 0x44 */
       uint8_t RESERVED_5[2];
  __I  uint16_t PCI_CFC_PCIE_DEV_STAT;             /**< PCI PCIe device status register, offset: 0x4A */
       uint8_t RESERVED_6[16];
  __IO uint16_t PCI_CFC_PCIE_ROOT_CTL;             /**< PCI PCIe root control register, offset: 0x5C */
       uint8_t RESERVED_7[2];
  __IO uint32_t PCI_CFC_PCIE_ROOT_STAT;            /**< PCI PCIe root status register, offset: 0x60 */
       uint8_t RESERVED_8[28];
  __I  uint16_t PCI_CFC_PCIPM_CAP_LIST;            /**< PCI PCI-PM capabilities list register, offset: 0x80 */
  __I  uint16_t PCI_CFC_PCIPM_CAP;                 /**< PCI PCI-PM capabilities register, offset: 0x82 */
  __IO uint16_t PCI_CFC_PCIPM_CTL_STAT;            /**< PCI PCI-PM control and status register, offset: 0x84 */
       uint8_t RESERVED_9[1];
       uint8_t PCI_CFC_PCIPM_DATA;                 /**< PCI PCI-PM capabilities data register, offset: 0x87 */
       uint8_t RESERVED_10[120];
  __I  uint32_t PCIE_CFC_AER_EXT_CAP_HDR;          /**< PCIe AER extended capability header, offset: 0x100 */
       uint8_t RESERVED_11[40];
  __IO uint32_t PCIE_CFC_AER_ROOT_ERR_CMD;         /**< PCIe AER root error command register, offset: 0x12C */
  __IO uint32_t PCIE_CFC_AER_ROOT_ERR_STAT;        /**< PCIe AER root error status register, offset: 0x130 */
  __I  uint32_t PCIE_CFC_AER_ERR_SRC_ID;           /**< PCIe AER error source identification register, offset: 0x134 */
  __I  uint32_t PCIE_CFC_RCEC_EPA_EXT_CAP_HDR;     /**< PCIe RCEC Endpoint association extended capability header, offset: 0x138 */
  __I  uint32_t PCIE_CFC_RCEC_EPA_BITMAP;          /**< PCIe RCEC Endpoint association bitmap registerr, offset: 0x13C */
} IERC_PCI_Type;

/* ----------------------------------------------------------------------------
   -- IERC_PCI Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IERC_PCI_Register_Masks IERC_PCI Register Masks
 * @{
 */

/*! @name PCI_CFH_DID_VID - PCI device ID and vendor ID register */
/*! @{ */

#define IERC_PCI_PCI_CFH_DID_VID_VENDOR_ID_MASK  (0xFFFFU)
#define IERC_PCI_PCI_CFH_DID_VID_VENDOR_ID_SHIFT (0U)
#define IERC_PCI_PCI_CFH_DID_VID_VENDOR_ID(x)    (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFH_DID_VID_VENDOR_ID_SHIFT)) & IERC_PCI_PCI_CFH_DID_VID_VENDOR_ID_MASK)

#define IERC_PCI_PCI_CFH_DID_VID_DEVICE_ID_MASK  (0xFFFF0000U)
#define IERC_PCI_PCI_CFH_DID_VID_DEVICE_ID_SHIFT (16U)
#define IERC_PCI_PCI_CFH_DID_VID_DEVICE_ID(x)    (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFH_DID_VID_DEVICE_ID_SHIFT)) & IERC_PCI_PCI_CFH_DID_VID_DEVICE_ID_MASK)
/*! @} */

/*! @name PCI_CFH_CMD - PCI command register */
/*! @{ */

#define IERC_PCI_PCI_CFH_CMD_INTR_DISABLE_MASK   (0x400U)
#define IERC_PCI_PCI_CFH_CMD_INTR_DISABLE_SHIFT  (10U)
#define IERC_PCI_PCI_CFH_CMD_INTR_DISABLE(x)     (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFH_CMD_INTR_DISABLE_SHIFT)) & IERC_PCI_PCI_CFH_CMD_INTR_DISABLE_MASK)
/*! @} */

/*! @name PCI_CFH_STAT - PCI status register */
/*! @{ */

#define IERC_PCI_PCI_CFH_STAT_INTR_STATUS_MASK   (0x8U)
#define IERC_PCI_PCI_CFH_STAT_INTR_STATUS_SHIFT  (3U)
#define IERC_PCI_PCI_CFH_STAT_INTR_STATUS(x)     (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFH_STAT_INTR_STATUS_SHIFT)) & IERC_PCI_PCI_CFH_STAT_INTR_STATUS_MASK)

#define IERC_PCI_PCI_CFH_STAT_CAP_LIST_MASK      (0x10U)
#define IERC_PCI_PCI_CFH_STAT_CAP_LIST_SHIFT     (4U)
#define IERC_PCI_PCI_CFH_STAT_CAP_LIST(x)        (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFH_STAT_CAP_LIST_SHIFT)) & IERC_PCI_PCI_CFH_STAT_CAP_LIST_MASK)
/*! @} */

/*! @name PCI_CFH_REVID_CLASSCODE - PCI revision ID and classcode register */
/*! @{ */

#define IERC_PCI_PCI_CFH_REVID_CLASSCODE_REV_ID_MASK (0xFFU)
#define IERC_PCI_PCI_CFH_REVID_CLASSCODE_REV_ID_SHIFT (0U)
#define IERC_PCI_PCI_CFH_REVID_CLASSCODE_REV_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFH_REVID_CLASSCODE_REV_ID_SHIFT)) & IERC_PCI_PCI_CFH_REVID_CLASSCODE_REV_ID_MASK)

#define IERC_PCI_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_MASK (0xFFFFFF00U)
#define IERC_PCI_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_SHIFT (8U)
#define IERC_PCI_PCI_CFH_REVID_CLASSCODE_CLASS_CODE(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_SHIFT)) & IERC_PCI_PCI_CFH_REVID_CLASSCODE_CLASS_CODE_MASK)
/*! @} */

/*! @name PCI_CFH_CL_SIZE - PCI cache line size register */
/*! @{ */

#define IERC_PCI_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_MASK (0xFFU)
#define IERC_PCI_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_SHIFT (0U)
#define IERC_PCI_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE(x) (((uint8_t)(((uint8_t)(x)) << IERC_PCI_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_SHIFT)) & IERC_PCI_PCI_CFH_CL_SIZE_CACHE_LINE_SIZE_MASK)
/*! @} */

/*! @name PCI_CFH_HDR_TYPE - PCI header type register */
/*! @{ */

#define IERC_PCI_PCI_CFH_HDR_TYPE_HDR_TYPE_MASK  (0x7FU)
#define IERC_PCI_PCI_CFH_HDR_TYPE_HDR_TYPE_SHIFT (0U)
#define IERC_PCI_PCI_CFH_HDR_TYPE_HDR_TYPE(x)    (((uint8_t)(((uint8_t)(x)) << IERC_PCI_PCI_CFH_HDR_TYPE_HDR_TYPE_SHIFT)) & IERC_PCI_PCI_CFH_HDR_TYPE_HDR_TYPE_MASK)

#define IERC_PCI_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_MASK (0x80U)
#define IERC_PCI_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_SHIFT (7U)
#define IERC_PCI_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV(x) (((uint8_t)(((uint8_t)(x)) << IERC_PCI_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_SHIFT)) & IERC_PCI_PCI_CFH_HDR_TYPE_MULT_FUNC_DEV_MASK)
/*! @} */

/*! @name PCI_CFH_SUBSYS_VID - PCI subsystem vendor ID register */
/*! @{ */

#define IERC_PCI_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define IERC_PCI_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_SHIFT (0U)
#define IERC_PCI_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_SHIFT)) & IERC_PCI_PCI_CFH_SUBSYS_VID_SUBSYSTEM_VENDOR_ID_MASK)
/*! @} */

/*! @name PCI_CFH_SUBSYS_ID - PCI subsystem ID register */
/*! @{ */

#define IERC_PCI_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_MASK (0xFFFFU)
#define IERC_PCI_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_SHIFT (0U)
#define IERC_PCI_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_SHIFT)) & IERC_PCI_PCI_CFH_SUBSYS_ID_SUBSYSTEM_ID_MASK)
/*! @} */

/*! @name PCI_CFH_CAP_PTR - PCI capabilities pointer register */
/*! @{ */

#define IERC_PCI_PCI_CFH_CAP_PTR_CAP_PTR_MASK    (0xFFU)
#define IERC_PCI_PCI_CFH_CAP_PTR_CAP_PTR_SHIFT   (0U)
#define IERC_PCI_PCI_CFH_CAP_PTR_CAP_PTR(x)      (((uint8_t)(((uint8_t)(x)) << IERC_PCI_PCI_CFH_CAP_PTR_CAP_PTR_SHIFT)) & IERC_PCI_PCI_CFH_CAP_PTR_CAP_PTR_MASK)
/*! @} */

/*! @name PCI_CFH_INT_LINE - PCI interrupt line register */
/*! @{ */

#define IERC_PCI_PCI_CFH_INT_LINE_INT_LINE_MASK  (0xFFU)
#define IERC_PCI_PCI_CFH_INT_LINE_INT_LINE_SHIFT (0U)
#define IERC_PCI_PCI_CFH_INT_LINE_INT_LINE(x)    (((uint8_t)(((uint8_t)(x)) << IERC_PCI_PCI_CFH_INT_LINE_INT_LINE_SHIFT)) & IERC_PCI_PCI_CFH_INT_LINE_INT_LINE_MASK)
/*! @} */

/*! @name PCI_CFH_INT_PIN - PCI interrupt pin register */
/*! @{ */

#define IERC_PCI_PCI_CFH_INT_PIN_INT_PIN_MASK    (0xFFU)
#define IERC_PCI_PCI_CFH_INT_PIN_INT_PIN_SHIFT   (0U)
#define IERC_PCI_PCI_CFH_INT_PIN_INT_PIN(x)      (((uint8_t)(((uint8_t)(x)) << IERC_PCI_PCI_CFH_INT_PIN_INT_PIN_SHIFT)) & IERC_PCI_PCI_CFH_INT_PIN_INT_PIN_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_CAP_LIST - PCI PCIe capabilities list register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIE_CAP_LIST_CAP_ID_MASK (0xFFU)
#define IERC_PCI_PCI_CFC_PCIE_CAP_LIST_CAP_ID_SHIFT (0U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIE_CAP_LIST_CAP_ID_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_CAP_LIST_CAP_ID_MASK)

#define IERC_PCI_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_ID_MASK (0xFF00U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_ID_SHIFT (8U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_ID_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_CAP_LIST_NEXT_CAP_ID_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_CAP - PCI PCIe capabilities register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIE_CAP_CAP_VER_MASK   (0xFU)
#define IERC_PCI_PCI_CFC_PCIE_CAP_CAP_VER_SHIFT  (0U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_CAP_VER(x)     (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIE_CAP_CAP_VER_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_CAP_CAP_VER_MASK)

#define IERC_PCI_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_MASK (0xF0U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_SHIFT (4U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_CAP_DEV_PORT_TYPE_MASK)

#define IERC_PCI_PCI_CFC_PCIE_CAP_INT_MSG_NUM_MASK (0x3E00U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_INT_MSG_NUM_SHIFT (9U)
#define IERC_PCI_PCI_CFC_PCIE_CAP_INT_MSG_NUM(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIE_CAP_INT_MSG_NUM_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_CAP_INT_MSG_NUM_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_CAP - PCI PCIe device capabilities register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_MASK (0x10000000U)
#define IERC_PCI_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_SHIFT (28U)
#define IERC_PCI_PCI_CFC_PCIE_DEV_CAP_FLR_CAP(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_DEV_CAP_FLR_CAP_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_DEV_STAT - PCI PCIe device status register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_MASK (0x20U)
#define IERC_PCI_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_SHIFT (5U)
#define IERC_PCI_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_DEV_STAT_TRANS_PEND_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_ROOT_CTL - PCI PCIe root control register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIE_ROOT_CTL_PME_INT_EN_MASK (0x8U)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_CTL_PME_INT_EN_SHIFT (3U)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_CTL_PME_INT_EN(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIE_ROOT_CTL_PME_INT_EN_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_ROOT_CTL_PME_INT_EN_MASK)
/*! @} */

/*! @name PCI_CFC_PCIE_ROOT_STAT - PCI PCIe root status register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_REQ_ID_MASK (0xFFFFU)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_REQ_ID_SHIFT (0U)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_REQ_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_REQ_ID_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_REQ_ID_MASK)

#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_STATUS_MASK (0x10000U)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_STATUS_SHIFT (16U)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_STATUS(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_STATUS_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_STATUS_MASK)

#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_PEND_MASK (0x20000U)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_PEND_SHIFT (17U)
#define IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_PEND(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_PEND_SHIFT)) & IERC_PCI_PCI_CFC_PCIE_ROOT_STAT_PME_PEND_MASK)
/*! @} */

/*! @name PCI_CFC_PCIPM_CAP_LIST - PCI PCI-PM capabilities list register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_MASK (0xFFU)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_SHIFT (0U)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_SHIFT)) & IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_CAP_ID_MASK)

#define IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_ID_MASK (0xFF00U)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_ID_SHIFT (8U)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_ID(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_ID_SHIFT)) & IERC_PCI_PCI_CFC_PCIPM_CAP_LIST_NEXT_CAP_ID_MASK)
/*! @} */

/*! @name PCI_CFC_PCIPM_CAP - PCI PCI-PM capabilities register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIPM_CAP_VERSION_MASK  (0x7U)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_VERSION_SHIFT (0U)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_VERSION(x)    (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIPM_CAP_VERSION_SHIFT)) & IERC_PCI_PCI_CFC_PCIPM_CAP_VERSION_MASK)

#define IERC_PCI_PCI_CFC_PCIPM_CAP_PME_SUPPORT_MASK (0xF800U)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_PME_SUPPORT_SHIFT (11U)
#define IERC_PCI_PCI_CFC_PCIPM_CAP_PME_SUPPORT(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIPM_CAP_PME_SUPPORT_SHIFT)) & IERC_PCI_PCI_CFC_PCIPM_CAP_PME_SUPPORT_MASK)
/*! @} */

/*! @name PCI_CFC_PCIPM_CTL_STAT - PCI PCI-PM control and status register */
/*! @{ */

#define IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_MASK (0x3U)
#define IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_SHIFT (0U)
#define IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_SHIFT)) & IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_PWR_STATE_MASK)

#define IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_MASK (0x8U)
#define IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_SHIFT (3U)
#define IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST(x) (((uint16_t)(((uint16_t)(x)) << IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_SHIFT)) & IERC_PCI_PCI_CFC_PCIPM_CTL_STAT_NO_SOFT_RST_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_EXT_CAP_HDR - PCIe AER extended capability header */
/*! @{ */

#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_SHIFT (16U)
#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_SHIFT)) & IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_CAP_VER_MASK)

#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & IERC_PCI_PCIE_CFC_AER_EXT_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_ROOT_ERR_CMD - PCIe AER root error command register */
/*! @{ */

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_CORR_ERR_RPT_EN_MASK (0x1U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_CORR_ERR_RPT_EN_SHIFT (0U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_CORR_ERR_RPT_EN(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_CORR_ERR_RPT_EN_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_CORR_ERR_RPT_EN_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_NON_FATAL_ERR_RPT_EN_MASK (0x2U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_NON_FATAL_ERR_RPT_EN_SHIFT (1U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_NON_FATAL_ERR_RPT_EN(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_NON_FATAL_ERR_RPT_EN_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_NON_FATAL_ERR_RPT_EN_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_FATAL_ERR_RPT_EN_MASK (0x4U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_FATAL_ERR_RPT_EN_SHIFT (2U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_FATAL_ERR_RPT_EN(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_FATAL_ERR_RPT_EN_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_CMD_FATAL_ERR_RPT_EN_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_ROOT_ERR_STAT - PCIe AER root error status register */
/*! @{ */

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_CORR_MASK (0x1U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_CORR_SHIFT (0U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_CORR(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_CORR_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_CORR_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_CORR_MASK (0x2U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_CORR_SHIFT (1U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_CORR(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_CORR_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_CORR_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_NON_FATAL_MASK (0x4U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_NON_FATAL_SHIFT (2U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_NON_FATAL(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_NON_FATAL_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_NON_FATAL_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_FATAL_NON_FATAL_MASK (0x8U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_FATAL_NON_FATAL_SHIFT (3U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_FATAL_NON_FATAL(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_FATAL_NON_FATAL_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_MULT_ERR_FATAL_NON_FATAL_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_FIRST_UCORR_FATAL_MASK (0x10U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_FIRST_UCORR_FATAL_SHIFT (4U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_FIRST_UCORR_FATAL(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_FIRST_UCORR_FATAL_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_FIRST_UCORR_FATAL_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_NON_FATAL_MASK (0x20U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_NON_FATAL_SHIFT (5U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_NON_FATAL(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_NON_FATAL_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_NON_FATAL_MASK)

#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_MASK (0x40U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_SHIFT (6U)
#define IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ROOT_ERR_STAT_ERR_FATAL_MASK)
/*! @} */

/*! @name PCIE_CFC_AER_ERR_SRC_ID - PCIe AER error source identification register */
/*! @{ */

#define IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_CORR_SRC_ID_MASK (0xFFFFU)
#define IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_CORR_SRC_ID_SHIFT (0U)
#define IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_CORR_SRC_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_CORR_SRC_ID_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_CORR_SRC_ID_MASK)

#define IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_FATAL_NON_FATAL_SRC_ID_MASK (0xFFFF0000U)
#define IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_FATAL_NON_FATAL_SRC_ID_SHIFT (16U)
#define IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_FATAL_NON_FATAL_SRC_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_FATAL_NON_FATAL_SRC_ID_SHIFT)) & IERC_PCI_PCIE_CFC_AER_ERR_SRC_ID_ERR_FATAL_NON_FATAL_SRC_ID_MASK)
/*! @} */

/*! @name PCIE_CFC_RCEC_EPA_EXT_CAP_HDR - PCIe RCEC Endpoint association extended capability header */
/*! @{ */

#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK (0xFFFFU)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT (0U)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_PCIE_EXT_CAP_ID(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_PCIE_EXT_CAP_ID_SHIFT)) & IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_PCIE_EXT_CAP_ID_MASK)

#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_CAP_VER_MASK (0xF0000U)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_CAP_VER_SHIFT (16U)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_CAP_VER(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_CAP_VER_SHIFT)) & IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_CAP_VER_MASK)

#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_NEXT_CAP_OFF_MASK (0xFFF00000U)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT (20U)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_NEXT_CAP_OFF(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_NEXT_CAP_OFF_SHIFT)) & IERC_PCI_PCIE_CFC_RCEC_EPA_EXT_CAP_HDR_NEXT_CAP_OFF_MASK)
/*! @} */

/*! @name PCIE_CFC_RCEC_EPA_BITMAP - PCIe RCEC Endpoint association bitmap registerr */
/*! @{ */

#define IERC_PCI_PCIE_CFC_RCEC_EPA_BITMAP_DEV_BITMAP_MASK (0xFFFFFFFFU)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_BITMAP_DEV_BITMAP_SHIFT (0U)
#define IERC_PCI_PCIE_CFC_RCEC_EPA_BITMAP_DEV_BITMAP(x) (((uint32_t)(((uint32_t)(x)) << IERC_PCI_PCIE_CFC_RCEC_EPA_BITMAP_DEV_BITMAP_SHIFT)) & IERC_PCI_PCIE_CFC_RCEC_EPA_BITMAP_DEV_BITMAP_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group IERC_PCI_Register_Masks */


/* IERC_PCI - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral IERC_F0_PCI_HDR_TYPE0 base address */
  #define IERC_F0_PCI_HDR_TYPE0_BASE               (0x700F8000u)
  /** Peripheral IERC_F0_PCI_HDR_TYPE0 base address */
  #define IERC_F0_PCI_HDR_TYPE0_BASE_NS            (0x600F8000u)
  /** Peripheral IERC_F0_PCI_HDR_TYPE0 base pointer */
  #define IERC_F0_PCI_HDR_TYPE0                    ((IERC_PCI_Type *)IERC_F0_PCI_HDR_TYPE0_BASE)
  /** Peripheral IERC_F0_PCI_HDR_TYPE0 base pointer */
  #define IERC_F0_PCI_HDR_TYPE0_NS                 ((IERC_PCI_Type *)IERC_F0_PCI_HDR_TYPE0_BASE_NS)
  /** Array initializer of IERC_PCI peripheral base addresses */
  #define IERC_PCI_BASE_ADDRS                      { IERC_F0_PCI_HDR_TYPE0_BASE }
  /** Array initializer of IERC_PCI peripheral base pointers */
  #define IERC_PCI_BASE_PTRS                       { IERC_F0_PCI_HDR_TYPE0 }
  /** Array initializer of IERC_PCI peripheral base addresses */
  #define IERC_PCI_BASE_ADDRS_NS                   { IERC_F0_PCI_HDR_TYPE0_BASE_NS }
  /** Array initializer of IERC_PCI peripheral base pointers */
  #define IERC_PCI_BASE_PTRS_NS                    { IERC_F0_PCI_HDR_TYPE0_NS }
#else
  /** Peripheral IERC_F0_PCI_HDR_TYPE0 base address */
  #define IERC_F0_PCI_HDR_TYPE0_BASE               (0x600F8000u)
  /** Peripheral IERC_F0_PCI_HDR_TYPE0 base pointer */
  #define IERC_F0_PCI_HDR_TYPE0                    ((IERC_PCI_Type *)IERC_F0_PCI_HDR_TYPE0_BASE)
  /** Array initializer of IERC_PCI peripheral base addresses */
  #define IERC_PCI_BASE_ADDRS                      { IERC_F0_PCI_HDR_TYPE0_BASE }
  /** Array initializer of IERC_PCI peripheral base pointers */
  #define IERC_PCI_BASE_PTRS                       { IERC_F0_PCI_HDR_TYPE0 }
#endif

/*!
 * @}
 */ /* end of group IERC_PCI_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- IOMUXC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IOMUXC_Peripheral_Access_Layer IOMUXC Peripheral Access Layer
 * @{
 */

/** IOMUXC - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[16];
  __IO uint32_t SW_MUX_CTL_PAD[146];               /**< SW_MUX_CTL_PAD_GPIO_EMC_B1_00 SW MUX Control Register..SW_MUX_CTL_PAD_GPIO_B2_13 SW MUX Control Register, array offset: 0x10, array step: 0x4 */
  __IO uint32_t SW_PAD_CTL_PAD[146];               /**< SW_PAD_CTL_PAD_GPIO_EMC_B1_00 SW PAD Control Register..SW_PAD_CTL_PAD_GPIO_B2_13 SW PAD Control Register, array offset: 0x258, array step: 0x4 */
  __IO uint32_t SELECT_INPUT[355];                 /**< CAN1_IPP_IND_CANRX_SELECT_INPUT DAISY Register..XSPI_SLV_IPP_IND_SCK_SELECT_INPUT DAISY Register, array offset: 0x4A0, array step: 0x4, valid indices: [0-143, 190-266, 269-282, 285-315, 344-354] */
} IOMUXC_Type;

/* ----------------------------------------------------------------------------
   -- IOMUXC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IOMUXC_Register_Masks IOMUXC Register Masks
 * @{
 */

/*! @name SW_MUX_CTL_PAD - SW_MUX_CTL_PAD_GPIO_EMC_B1_00 SW MUX Control Register..SW_MUX_CTL_PAD_GPIO_B2_13 SW MUX Control Register */
/*! @{ */

#define IOMUXC_SW_MUX_CTL_PAD_MUX_MODE_MASK      (0xFU)  /* Merged from fields with different position or width, of widths (3, 4), largest definition used */
#define IOMUXC_SW_MUX_CTL_PAD_MUX_MODE_SHIFT     (0U)
/*! MUX_MODE - MUX Mode Select Field.
 *  0b1010..Select mux mode: ALT10 mux port: XSPI_SLV_DATA00 of instance: xspi_slv
 *  0b1011..Select mux mode: ALT11 mux port: NETC_PINMUX_ETH2_RXD00 of instance: netc_pinmux
 *  0b1100..Select mux mode: ALT12 mux port: ECAT_RX_DATA0_1 of instance: ecat
 *  0b0000..Select mux mode: ALT0 mux port: MIC_BITSTREAM00 of instance: mic
 *  0b0001..Select mux mode: ALT1 mux port: SINC2_EMCLK03 of instance: sinc2
 *  0b0010..Select mux mode: ALT2 mux port: CAN3_TX of instance: can3
 *  0b0011..Select mux mode: ALT3 mux port: LPUART8_CTS_B of instance: lpuart8
 *  0b0100..Select mux mode: ALT4 mux port: LPUART6_TX of instance: lpuart6
 *  0b0101..Select mux mode: ALT5 mux port: GPIO6_IO24 of instance: gpio6
 *  0b0110..Select mux mode: ALT6 mux port: LPI2C4_SCL of instance: lpi2c4
 *  0b0111..Select mux mode: ALT7 mux port: FLEXSPI1_BUS2BIT_A_DATA00 of instance: flexspi1_bus2bit
 *  0b1000..Select mux mode: ALT8 mux port: TPM4_CH00 of instance: tpm4
 *  0b1001..Select mux mode: ALT9 mux port: LPSPI4_SCK of instance: lpspi4
 */
#define IOMUXC_SW_MUX_CTL_PAD_MUX_MODE(x)        (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_MUX_CTL_PAD_MUX_MODE_SHIFT)) & IOMUXC_SW_MUX_CTL_PAD_MUX_MODE_MASK)  /* Merged from fields with different position or width, of widths (3, 4), largest definition used */

#define IOMUXC_SW_MUX_CTL_PAD_SION_MASK          (0x10U)
#define IOMUXC_SW_MUX_CTL_PAD_SION_SHIFT         (4U)
/*! SION - Software Input On Field.
 *  0b1..Force input path of pad GPIO_SD_B2_12_DUMMY
 *  0b0..Input Path is determined by functionality
 */
#define IOMUXC_SW_MUX_CTL_PAD_SION(x)            (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_MUX_CTL_PAD_SION_SHIFT)) & IOMUXC_SW_MUX_CTL_PAD_SION_MASK)
/*! @} */

/* The count of IOMUXC_SW_MUX_CTL_PAD */
#define IOMUXC_SW_MUX_CTL_PAD_COUNT              (146U)

/*! @name SW_PAD_CTL_PAD - SW_PAD_CTL_PAD_GPIO_EMC_B1_00 SW PAD Control Register..SW_PAD_CTL_PAD_GPIO_B2_13 SW PAD Control Register */
/*! @{ */

#define IOMUXC_SW_PAD_CTL_PAD_SRE_MASK           (0x1U)
#define IOMUXC_SW_PAD_CTL_PAD_SRE_SHIFT          (0U)
/*! SRE - Slew Rate Field
 *  0b0..Fast Slew Rate
 *  0b1..Slow Slew Rate
 */
#define IOMUXC_SW_PAD_CTL_PAD_SRE(x)             (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_SRE_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_SRE_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_DSE_MASK           (0x2U)
#define IOMUXC_SW_PAD_CTL_PAD_DSE_SHIFT          (1U)
/*! DSE - Drive Strength Field
 *  0b0..normal driver
 *  0b1..high driver
 */
#define IOMUXC_SW_PAD_CTL_PAD_DSE(x)             (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_DSE_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_DSE_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_PDRV_MASK          (0x2U)
#define IOMUXC_SW_PAD_CTL_PAD_PDRV_SHIFT         (1U)
/*! PDRV - PDRV Field
 *  0b0..high driver
 *  0b1..normal driver
 */
#define IOMUXC_SW_PAD_CTL_PAD_PDRV(x)            (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_PDRV_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_PDRV_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_PUE_MASK           (0x4U)
#define IOMUXC_SW_PAD_CTL_PAD_PUE_SHIFT          (2U)
/*! PUE - Pull / Keep Select Field
 *  0b0..Pull Disable, Highz
 *  0b1..Pull Enable
 */
#define IOMUXC_SW_PAD_CTL_PAD_PUE(x)             (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_PUE_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_PUE_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_PULL_MASK          (0xCU)
#define IOMUXC_SW_PAD_CTL_PAD_PULL_SHIFT         (2U)
/*! PULL - Pull Down Pull Up Field
 *  0b00..Forbidden
 *  0b01..PU
 *  0b10..PD
 *  0b11..No Pull
 */
#define IOMUXC_SW_PAD_CTL_PAD_PULL(x)            (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_PULL_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_PULL_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_PUS_MASK           (0x8U)
#define IOMUXC_SW_PAD_CTL_PAD_PUS_SHIFT          (3U)
/*! PUS - Pull Up / Down Config. Field
 *  0b0..Weak pull down
 *  0b1..Weak pull up
 */
#define IOMUXC_SW_PAD_CTL_PAD_PUS(x)             (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_PUS_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_PUS_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_ODE_MASK           (0x10U)
#define IOMUXC_SW_PAD_CTL_PAD_ODE_SHIFT          (4U)
/*! ODE - Open Drain Field
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define IOMUXC_SW_PAD_CTL_PAD_ODE(x)             (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_ODE_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_ODE_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_IBE_OFF_MASK       (0x80U)
#define IOMUXC_SW_PAD_CTL_PAD_IBE_OFF_SHIFT      (7U)
/*! IBE_OFF - Force Input Buffer Enable (IBE) off Field
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define IOMUXC_SW_PAD_CTL_PAD_IBE_OFF(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_IBE_OFF_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_IBE_OFF_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_DWP_MASK           (0xF000000U)
#define IOMUXC_SW_PAD_CTL_PAD_DWP_SHIFT          (24U)
/*! DWP - Domain write protection */
#define IOMUXC_SW_PAD_CTL_PAD_DWP(x)             (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_DWP_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_DWP_MASK)

#define IOMUXC_SW_PAD_CTL_PAD_DWP_LOCK_MASK      (0xF0000000U)
#define IOMUXC_SW_PAD_CTL_PAD_DWP_LOCK_SHIFT     (28U)
/*! DWP_LOCK - Domain write protection lock */
#define IOMUXC_SW_PAD_CTL_PAD_DWP_LOCK(x)        (((uint32_t)(((uint32_t)(x)) << IOMUXC_SW_PAD_CTL_PAD_DWP_LOCK_SHIFT)) & IOMUXC_SW_PAD_CTL_PAD_DWP_LOCK_MASK)
/*! @} */

/* The count of IOMUXC_SW_PAD_CTL_PAD */
#define IOMUXC_SW_PAD_CTL_PAD_COUNT              (146U)

/*! @name SELECT_INPUT - CAN1_IPP_IND_CANRX_SELECT_INPUT DAISY Register..XSPI_SLV_IPP_IND_SCK_SELECT_INPUT DAISY Register */
/*! @{ */

#define IOMUXC_SELECT_INPUT_DAISY_MASK           (0x7U)  /* Merged from fields with different position or width, of widths (1, 2, 3), largest definition used */
#define IOMUXC_SELECT_INPUT_DAISY_SHIFT          (0U)
/*! DAISY - Selecting Pads Involved in Daisy Chain.
 *  0b000..Selecting Pad: GPIO_EMC_B1_19 for Mode: ALT10
 *  0b001..Selecting Pad: GPIO_EMC_B1_41 for Mode: ALT1
 *  0b010..Selecting Pad: GPIO_EMC_B2_01 for Mode: ALT3
 *  0b011..Selecting Pad: GPIO_EMC_B2_20 for Mode: ALT4
 *  0b100..Selecting Pad: GPIO_AD_31 for Mode: ALT7
 *  0b101..Selecting Pad: GPIO_SD_B2_10 for Mode: ALT10
 *  0b110..Selecting Pad: GPIO_B1_12 for Mode: ALT1
 *  0b111..Selecting Pad: GPIO_B2_02 for Mode: ALT3
 */
#define IOMUXC_SELECT_INPUT_DAISY(x)             (((uint32_t)(((uint32_t)(x)) << IOMUXC_SELECT_INPUT_DAISY_SHIFT)) & IOMUXC_SELECT_INPUT_DAISY_MASK)  /* Merged from fields with different position or width, of widths (1, 2, 3), largest definition used */
/*! @} */

/* The count of IOMUXC_SELECT_INPUT */
#define IOMUXC_SELECT_INPUT_COUNT                (355U)


/*!
 * @}
 */ /* end of group IOMUXC_Register_Masks */


/* IOMUXC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral IOMUXC base address */
  #define IOMUXC_BASE                              (0x52A10000u)
  /** Peripheral IOMUXC base address */
  #define IOMUXC_BASE_NS                           (0x42A10000u)
  /** Peripheral IOMUXC base pointer */
  #define IOMUXC                                   ((IOMUXC_Type *)IOMUXC_BASE)
  /** Peripheral IOMUXC base pointer */
  #define IOMUXC_NS                                ((IOMUXC_Type *)IOMUXC_BASE_NS)
  /** Array initializer of IOMUXC peripheral base addresses */
  #define IOMUXC_BASE_ADDRS                        { IOMUXC_BASE }
  /** Array initializer of IOMUXC peripheral base pointers */
  #define IOMUXC_BASE_PTRS                         { IOMUXC }
  /** Array initializer of IOMUXC peripheral base addresses */
  #define IOMUXC_BASE_ADDRS_NS                     { IOMUXC_BASE_NS }
  /** Array initializer of IOMUXC peripheral base pointers */
  #define IOMUXC_BASE_PTRS_NS                      { IOMUXC_NS }
#else
  /** Peripheral IOMUXC base address */
  #define IOMUXC_BASE                              (0x42A10000u)
  /** Peripheral IOMUXC base pointer */
  #define IOMUXC                                   ((IOMUXC_Type *)IOMUXC_BASE)
  /** Array initializer of IOMUXC peripheral base addresses */
  #define IOMUXC_BASE_ADDRS                        { IOMUXC_BASE }
  /** Array initializer of IOMUXC peripheral base pointers */
  #define IOMUXC_BASE_PTRS                         { IOMUXC }
#endif

/*!
 * @}
 */ /* end of group IOMUXC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- IOMUXC_AON Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IOMUXC_AON_Peripheral_Access_Layer IOMUXC_AON Peripheral Access Layer
 * @{
 */

/** IOMUXC_AON - Register Layout Typedef */
typedef struct {
  __IO uint32_t SW_MUX_CTL_PAD[29];                /**< SW_MUX_CTL_PAD_GPIO_AON_00 SW MUX Control Register..SW_MUX_CTL_PAD_GPIO_AON_28_DUMMY SW MUX Control Register, array offset: 0x0, array step: 0x4 */
  __IO uint32_t SW_PAD_CTL_PAD[29];                /**< SW_PAD_CTL_PAD_GPIO_AON_00 SW PAD Control Register..SW_PAD_CTL_PAD_GPIO_AON_28_DUMMY SW PAD Control Register, array offset: 0x74, array step: 0x4 */
  __IO uint32_t SELECT_INPUT[39];                  /**< I3C1_PIN_SCL_IN_SELECT_INPUT DAISY Register..SAI1_IPP_IND_SAI_TXSYNC_SELECT_INPUT DAISY Register, array offset: 0xE8, array step: 0x4 */
} IOMUXC_AON_Type;

/* ----------------------------------------------------------------------------
   -- IOMUXC_AON Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup IOMUXC_AON_Register_Masks IOMUXC_AON Register Masks
 * @{
 */

/*! @name SW_MUX_CTL_PAD - SW_MUX_CTL_PAD_GPIO_AON_00 SW MUX Control Register..SW_MUX_CTL_PAD_GPIO_AON_28_DUMMY SW MUX Control Register */
/*! @{ */

#define IOMUXC_AON_SW_MUX_CTL_PAD_MUX_MODE_MASK  (0xFU)  /* Merged from fields with different position or width, of widths (3, 4), largest definition used */
#define IOMUXC_AON_SW_MUX_CTL_PAD_MUX_MODE_SHIFT (0U)
/*! MUX_MODE - MUX Mode Select Field.
 *  0b0000..Select mux mode: ALT0 mux port: SRC_BOOT_MODE02 of instance: src
 *  0b0001..Select mux mode: ALT1 mux port: CAN3_TX of instance: can3
 *  0b0010..Select mux mode: ALT2 mux port: LPSPI2_PCS3 of instance: lpspi2
 *  0b0011..Select mux mode: ALT3 mux port: LPSPI2_SDO of instance: lpspi2
 *  0b0100..Select mux mode: ALT4 mux port: LPTMR1_ALT3 of instance: lptmr1
 *  0b0101..Select mux mode: ALT5 mux port: GPIO1_IO02 of instance: gpio1
 *  0b0110..Select mux mode: ALT6 mux port: LPUART2_RTS_B of instance: lpuart2
 *  0b1000..Select mux mode: ALT8 mux port: TPM1_CH01 of instance: tpm1
 *  0b1001..Reserved
 *  0b1010..Reserved
 *  0b1011..Reserved
 *  0b1100..Select mux mode: ALT12 mux port: ECAT_CLK_ECAT_CLK25 of instance: ecat
 */
#define IOMUXC_AON_SW_MUX_CTL_PAD_MUX_MODE(x)    (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_MUX_CTL_PAD_MUX_MODE_SHIFT)) & IOMUXC_AON_SW_MUX_CTL_PAD_MUX_MODE_MASK)  /* Merged from fields with different position or width, of widths (3, 4), largest definition used */

#define IOMUXC_AON_SW_MUX_CTL_PAD_SION_MASK      (0x10U)
#define IOMUXC_AON_SW_MUX_CTL_PAD_SION_SHIFT     (4U)
/*! SION - Software Input On Field.
 *  0b1..Force input path of pad GPIO_AON_28_DUMMY
 *  0b0..Input Path is determined by functionality
 */
#define IOMUXC_AON_SW_MUX_CTL_PAD_SION(x)        (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_MUX_CTL_PAD_SION_SHIFT)) & IOMUXC_AON_SW_MUX_CTL_PAD_SION_MASK)
/*! @} */

/* The count of IOMUXC_AON_SW_MUX_CTL_PAD */
#define IOMUXC_AON_SW_MUX_CTL_PAD_COUNT          (29U)

/*! @name SW_PAD_CTL_PAD - SW_PAD_CTL_PAD_GPIO_AON_00 SW PAD Control Register..SW_PAD_CTL_PAD_GPIO_AON_28_DUMMY SW PAD Control Register */
/*! @{ */

#define IOMUXC_AON_SW_PAD_CTL_PAD_SRE_MASK       (0x1U)
#define IOMUXC_AON_SW_PAD_CTL_PAD_SRE_SHIFT      (0U)
/*! SRE - Slew Rate Field
 *  0b0..Fast Slew Rate
 *  0b1..Slow Slew Rate
 */
#define IOMUXC_AON_SW_PAD_CTL_PAD_SRE(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_PAD_CTL_PAD_SRE_SHIFT)) & IOMUXC_AON_SW_PAD_CTL_PAD_SRE_MASK)

#define IOMUXC_AON_SW_PAD_CTL_PAD_DSE_MASK       (0x2U)
#define IOMUXC_AON_SW_PAD_CTL_PAD_DSE_SHIFT      (1U)
/*! DSE - Drive Strength Field
 *  0b0..normal driver
 *  0b1..high driver
 */
#define IOMUXC_AON_SW_PAD_CTL_PAD_DSE(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_PAD_CTL_PAD_DSE_SHIFT)) & IOMUXC_AON_SW_PAD_CTL_PAD_DSE_MASK)

#define IOMUXC_AON_SW_PAD_CTL_PAD_PUE_MASK       (0x4U)
#define IOMUXC_AON_SW_PAD_CTL_PAD_PUE_SHIFT      (2U)
/*! PUE - Pull / Keep Select Field
 *  0b0..Pull Disable, Highz
 *  0b1..Pull Enable
 */
#define IOMUXC_AON_SW_PAD_CTL_PAD_PUE(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_PAD_CTL_PAD_PUE_SHIFT)) & IOMUXC_AON_SW_PAD_CTL_PAD_PUE_MASK)

#define IOMUXC_AON_SW_PAD_CTL_PAD_PUS_MASK       (0x8U)
#define IOMUXC_AON_SW_PAD_CTL_PAD_PUS_SHIFT      (3U)
/*! PUS - Pull Up / Down Config. Field
 *  0b0..Weak pull down
 *  0b1..Weak pull up
 */
#define IOMUXC_AON_SW_PAD_CTL_PAD_PUS(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_PAD_CTL_PAD_PUS_SHIFT)) & IOMUXC_AON_SW_PAD_CTL_PAD_PUS_MASK)

#define IOMUXC_AON_SW_PAD_CTL_PAD_ODE_MASK       (0x10U)
#define IOMUXC_AON_SW_PAD_CTL_PAD_ODE_SHIFT      (4U)
/*! ODE - Open Drain Field
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define IOMUXC_AON_SW_PAD_CTL_PAD_ODE(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_PAD_CTL_PAD_ODE_SHIFT)) & IOMUXC_AON_SW_PAD_CTL_PAD_ODE_MASK)

#define IOMUXC_AON_SW_PAD_CTL_PAD_DWP_MASK       (0xF000000U)
#define IOMUXC_AON_SW_PAD_CTL_PAD_DWP_SHIFT      (24U)
/*! DWP - Domain write protection */
#define IOMUXC_AON_SW_PAD_CTL_PAD_DWP(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_PAD_CTL_PAD_DWP_SHIFT)) & IOMUXC_AON_SW_PAD_CTL_PAD_DWP_MASK)

#define IOMUXC_AON_SW_PAD_CTL_PAD_DWP_LOCK_MASK  (0xF0000000U)
#define IOMUXC_AON_SW_PAD_CTL_PAD_DWP_LOCK_SHIFT (28U)
/*! DWP_LOCK - Domain write protection lock */
#define IOMUXC_AON_SW_PAD_CTL_PAD_DWP_LOCK(x)    (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SW_PAD_CTL_PAD_DWP_LOCK_SHIFT)) & IOMUXC_AON_SW_PAD_CTL_PAD_DWP_LOCK_MASK)
/*! @} */

/* The count of IOMUXC_AON_SW_PAD_CTL_PAD */
#define IOMUXC_AON_SW_PAD_CTL_PAD_COUNT          (29U)

/*! @name SELECT_INPUT - I3C1_PIN_SCL_IN_SELECT_INPUT DAISY Register..SAI1_IPP_IND_SAI_TXSYNC_SELECT_INPUT DAISY Register */
/*! @{ */

#define IOMUXC_AON_SELECT_INPUT_DAISY_MASK       (0x3U)  /* Merged from fields with different position or width, of widths (1, 2), largest definition used */
#define IOMUXC_AON_SELECT_INPUT_DAISY_SHIFT      (0U)
/*! DAISY - Selecting Pads Involved in Daisy Chain.
 *  0b00..Selecting Pad: GPIO_AON_06 for Mode: ALT3
 *  0b01..Selecting Pad: GPIO_AON_08 for Mode: ALT6
 *  0b10..Selecting Pad: GPIO_AON_20 for Mode: ALT2
 *  0b11..Selecting Pad: GPIO_AON_24 for Mode: ALT1
 */
#define IOMUXC_AON_SELECT_INPUT_DAISY(x)         (((uint32_t)(((uint32_t)(x)) << IOMUXC_AON_SELECT_INPUT_DAISY_SHIFT)) & IOMUXC_AON_SELECT_INPUT_DAISY_MASK)  /* Merged from fields with different position or width, of widths (1, 2), largest definition used */
/*! @} */

/* The count of IOMUXC_AON_SELECT_INPUT */
#define IOMUXC_AON_SELECT_INPUT_COUNT            (39U)


/*!
 * @}
 */ /* end of group IOMUXC_AON_Register_Masks */


/* IOMUXC_AON - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral IOMUXC_AON base address */
  #define IOMUXC_AON_BASE                          (0x543C0000u)
  /** Peripheral IOMUXC_AON base address */
  #define IOMUXC_AON_BASE_NS                       (0x443C0000u)
  /** Peripheral IOMUXC_AON base pointer */
  #define IOMUXC_AON                               ((IOMUXC_AON_Type *)IOMUXC_AON_BASE)
  /** Peripheral IOMUXC_AON base pointer */
  #define IOMUXC_AON_NS                            ((IOMUXC_AON_Type *)IOMUXC_AON_BASE_NS)
  /** Array initializer of IOMUXC_AON peripheral base addresses */
  #define IOMUXC_AON_BASE_ADDRS                    { IOMUXC_AON_BASE }
  /** Array initializer of IOMUXC_AON peripheral base pointers */
  #define IOMUXC_AON_BASE_PTRS                     { IOMUXC_AON }
  /** Array initializer of IOMUXC_AON peripheral base addresses */
  #define IOMUXC_AON_BASE_ADDRS_NS                 { IOMUXC_AON_BASE_NS }
  /** Array initializer of IOMUXC_AON peripheral base pointers */
  #define IOMUXC_AON_BASE_PTRS_NS                  { IOMUXC_AON_NS }
#else
  /** Peripheral IOMUXC_AON base address */
  #define IOMUXC_AON_BASE                          (0x443C0000u)
  /** Peripheral IOMUXC_AON base pointer */
  #define IOMUXC_AON                               ((IOMUXC_AON_Type *)IOMUXC_AON_BASE)
  /** Array initializer of IOMUXC_AON peripheral base addresses */
  #define IOMUXC_AON_BASE_ADDRS                    { IOMUXC_AON_BASE }
  /** Array initializer of IOMUXC_AON peripheral base pointers */
  #define IOMUXC_AON_BASE_PTRS                     { IOMUXC_AON }
#endif

/*!
 * @}
 */ /* end of group IOMUXC_AON_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- KPP Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup KPP_Peripheral_Access_Layer KPP Peripheral Access Layer
 * @{
 */

/** KPP - Register Layout Typedef */
typedef struct {
  __IO uint16_t KPCR;                              /**< Keypad Control Register, offset: 0x0 */
  __IO uint16_t KPSR;                              /**< Keypad Status Register, offset: 0x2 */
  __IO uint16_t KDDR;                              /**< Keypad Data Direction Register, offset: 0x4 */
  __IO uint16_t KPDR;                              /**< Keypad Data Register, offset: 0x6 */
} KPP_Type;

/* ----------------------------------------------------------------------------
   -- KPP Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup KPP_Register_Masks KPP Register Masks
 * @{
 */

/*! @name KPCR - Keypad Control Register */
/*! @{ */

#define KPP_KPCR_KRE_MASK                        (0xFFU)
#define KPP_KPCR_KRE_SHIFT                       (0U)
/*! KRE - KRE
 *  0b00000000..Row is not included in the keypad key press detect.
 *  0b00000001..Row is included in the keypad key press detect.
 */
#define KPP_KPCR_KRE(x)                          (((uint16_t)(((uint16_t)(x)) << KPP_KPCR_KRE_SHIFT)) & KPP_KPCR_KRE_MASK)

#define KPP_KPCR_KCO_MASK                        (0xFF00U)
#define KPP_KPCR_KCO_SHIFT                       (8U)
/*! KCO - KCO
 *  0b00000000..Column strobe output is totem pole drive.
 *  0b00000001..Column strobe output is open drain.
 */
#define KPP_KPCR_KCO(x)                          (((uint16_t)(((uint16_t)(x)) << KPP_KPCR_KCO_SHIFT)) & KPP_KPCR_KCO_MASK)
/*! @} */

/*! @name KPSR - Keypad Status Register */
/*! @{ */

#define KPP_KPSR_KPKD_MASK                       (0x1U)
#define KPP_KPSR_KPKD_SHIFT                      (0U)
/*! KPKD - KPKD
 *  0b0..No key presses detected
 *  0b1..A key has been depressed
 */
#define KPP_KPSR_KPKD(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KPSR_KPKD_SHIFT)) & KPP_KPSR_KPKD_MASK)

#define KPP_KPSR_KPKR_MASK                       (0x2U)
#define KPP_KPSR_KPKR_SHIFT                      (1U)
/*! KPKR - KPKR
 *  0b0..No key release detected
 *  0b1..All keys have been released
 */
#define KPP_KPSR_KPKR(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KPSR_KPKR_SHIFT)) & KPP_KPSR_KPKR_MASK)

#define KPP_KPSR_KDSC_MASK                       (0x4U)
#define KPP_KPSR_KDSC_SHIFT                      (2U)
/*! KDSC - KDSC
 *  0b0..No effect
 *  0b1..Set bits that clear the keypad depress synchronizer chain
 */
#define KPP_KPSR_KDSC(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KPSR_KDSC_SHIFT)) & KPP_KPSR_KDSC_MASK)

#define KPP_KPSR_KRSS_MASK                       (0x8U)
#define KPP_KPSR_KRSS_SHIFT                      (3U)
/*! KRSS - KRSS
 *  0b0..No effect
 *  0b1..Set bits which sets keypad release synchronizer chain
 */
#define KPP_KPSR_KRSS(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KPSR_KRSS_SHIFT)) & KPP_KPSR_KRSS_MASK)

#define KPP_KPSR_KDIE_MASK                       (0x100U)
#define KPP_KPSR_KDIE_SHIFT                      (8U)
/*! KDIE - KDIE
 *  0b0..No interrupt request is generated when KPKD is set.
 *  0b1..An interrupt request is generated when KPKD is set.
 */
#define KPP_KPSR_KDIE(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KPSR_KDIE_SHIFT)) & KPP_KPSR_KDIE_MASK)

#define KPP_KPSR_KRIE_MASK                       (0x200U)
#define KPP_KPSR_KRIE_SHIFT                      (9U)
/*! KRIE - KRIE
 *  0b0..No interrupt request is generated when KPKR is set.
 *  0b1..An interrupt request is generated when KPKR is set.
 */
#define KPP_KPSR_KRIE(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KPSR_KRIE_SHIFT)) & KPP_KPSR_KRIE_MASK)
/*! @} */

/*! @name KDDR - Keypad Data Direction Register */
/*! @{ */

#define KPP_KDDR_KRDD_MASK                       (0xFFU)
#define KPP_KDDR_KRDD_SHIFT                      (0U)
/*! KRDD - KRDD
 *  0b00000000..ROWn pin configured as an input.
 *  0b00000001..ROWn pin configured as an output.
 */
#define KPP_KDDR_KRDD(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KDDR_KRDD_SHIFT)) & KPP_KDDR_KRDD_MASK)

#define KPP_KDDR_KCDD_MASK                       (0xFF00U)
#define KPP_KDDR_KCDD_SHIFT                      (8U)
/*! KCDD - KCDD
 *  0b00000000..COLn pin is configured as an input.
 *  0b00000001..COLn pin is configured as an output.
 */
#define KPP_KDDR_KCDD(x)                         (((uint16_t)(((uint16_t)(x)) << KPP_KDDR_KCDD_SHIFT)) & KPP_KDDR_KCDD_MASK)
/*! @} */

/*! @name KPDR - Keypad Data Register */
/*! @{ */

#define KPP_KPDR_KRD_MASK                        (0xFFU)
#define KPP_KPDR_KRD_SHIFT                       (0U)
/*! KRD - KRD */
#define KPP_KPDR_KRD(x)                          (((uint16_t)(((uint16_t)(x)) << KPP_KPDR_KRD_SHIFT)) & KPP_KPDR_KRD_MASK)

#define KPP_KPDR_KCD_MASK                        (0xFF00U)
#define KPP_KPDR_KCD_SHIFT                       (8U)
/*! KCD - KCD */
#define KPP_KPDR_KCD(x)                          (((uint16_t)(((uint16_t)(x)) << KPP_KPDR_KCD_SHIFT)) & KPP_KPDR_KCD_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group KPP_Register_Masks */


/* KPP - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral KPP base address */
  #define KPP_BASE                                 (0x52A00000u)
  /** Peripheral KPP base address */
  #define KPP_BASE_NS                              (0x42A00000u)
  /** Peripheral KPP base pointer */
  #define KPP                                      ((KPP_Type *)KPP_BASE)
  /** Peripheral KPP base pointer */
  #define KPP_NS                                   ((KPP_Type *)KPP_BASE_NS)
  /** Array initializer of KPP peripheral base addresses */
  #define KPP_BASE_ADDRS                           { KPP_BASE }
  /** Array initializer of KPP peripheral base pointers */
  #define KPP_BASE_PTRS                            { KPP }
  /** Array initializer of KPP peripheral base addresses */
  #define KPP_BASE_ADDRS_NS                        { KPP_BASE_NS }
  /** Array initializer of KPP peripheral base pointers */
  #define KPP_BASE_PTRS_NS                         { KPP_NS }
#else
  /** Peripheral KPP base address */
  #define KPP_BASE                                 (0x42A00000u)
  /** Peripheral KPP base pointer */
  #define KPP                                      ((KPP_Type *)KPP_BASE)
  /** Array initializer of KPP peripheral base addresses */
  #define KPP_BASE_ADDRS                           { KPP_BASE }
  /** Array initializer of KPP peripheral base pointers */
  #define KPP_BASE_PTRS                            { KPP }
#endif
/** Interrupt vectors for the KPP peripheral type */
#define KPP_IRQS                                 { KPP_IRQn }

/*!
 * @}
 */ /* end of group KPP_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- LPI2C Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPI2C_Peripheral_Access_Layer LPI2C Peripheral Access Layer
 * @{
 */

/** LPI2C - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
       uint8_t RESERVED_0[8];
  __IO uint32_t MCR;                               /**< Controller Control, offset: 0x10 */
  __IO uint32_t MSR;                               /**< Controller Status, offset: 0x14 */
  __IO uint32_t MIER;                              /**< Controller Interrupt Enable, offset: 0x18 */
  __IO uint32_t MDER;                              /**< Controller DMA Enable, offset: 0x1C */
  __IO uint32_t MCFGR0;                            /**< Controller Configuration 0, offset: 0x20 */
  __IO uint32_t MCFGR1;                            /**< Controller Configuration 1, offset: 0x24 */
  __IO uint32_t MCFGR2;                            /**< Controller Configuration 2, offset: 0x28 */
  __IO uint32_t MCFGR3;                            /**< Controller Configuration 3, offset: 0x2C */
       uint8_t RESERVED_1[16];
  __IO uint32_t MDMR;                              /**< Controller Data Match, offset: 0x40 */
       uint8_t RESERVED_2[4];
  __IO uint32_t MCCR0;                             /**< Controller Clock Configuration 0, offset: 0x48 */
       uint8_t RESERVED_3[4];
  __IO uint32_t MCCR1;                             /**< Controller Clock Configuration 1, offset: 0x50 */
       uint8_t RESERVED_4[4];
  __IO uint32_t MFCR;                              /**< Controller FIFO Control, offset: 0x58 */
  __I  uint32_t MFSR;                              /**< Controller FIFO Status, offset: 0x5C */
  __O  uint32_t MTDR;                              /**< Controller Transmit Data, offset: 0x60 */
       uint8_t RESERVED_5[12];
  __I  uint32_t MRDR;                              /**< Controller Receive Data, offset: 0x70 */
       uint8_t RESERVED_6[4];
  __I  uint32_t MRDROR;                            /**< Controller Receive Data Read Only, offset: 0x78 */
       uint8_t RESERVED_7[148];
  __IO uint32_t SCR;                               /**< Target Control, offset: 0x110 */
  __IO uint32_t SSR;                               /**< Target Status, offset: 0x114 */
  __IO uint32_t SIER;                              /**< Target Interrupt Enable, offset: 0x118 */
  __IO uint32_t SDER;                              /**< Target DMA Enable, offset: 0x11C */
  __IO uint32_t SCFGR0;                            /**< Target Configuration 0, offset: 0x120 */
  __IO uint32_t SCFGR1;                            /**< Target Configuration 1, offset: 0x124 */
  __IO uint32_t SCFGR2;                            /**< Target Configuration 2, offset: 0x128 */
       uint8_t RESERVED_8[20];
  __IO uint32_t SAMR;                              /**< Target Address Match, offset: 0x140 */
       uint8_t RESERVED_9[12];
  __I  uint32_t SASR;                              /**< Target Address Status, offset: 0x150 */
  __IO uint32_t STAR;                              /**< Target Transmit ACK, offset: 0x154 */
       uint8_t RESERVED_10[8];
  __O  uint32_t STDR;                              /**< Target Transmit Data, offset: 0x160 */
       uint8_t RESERVED_11[12];
  __I  uint32_t SRDR;                              /**< Target Receive Data, offset: 0x170 */
       uint8_t RESERVED_12[4];
  __I  uint32_t SRDROR;                            /**< Target Receive Data Read Only, offset: 0x178 */
       uint8_t RESERVED_13[132];
  __O  uint32_t MTCBR[128];                        /**< Controller Transmit Command Burst, array offset: 0x200, array step: 0x4 */
  __O  uint32_t MTDBR[256];                        /**< Transmit Data Burst, array offset: 0x400, array step: 0x4 */
} LPI2C_Type;

/* ----------------------------------------------------------------------------
   -- LPI2C Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPI2C_Register_Masks LPI2C Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define LPI2C_VERID_FEATURE_MASK                 (0xFFFFU)
#define LPI2C_VERID_FEATURE_SHIFT                (0U)
/*! FEATURE - Feature Specification Number
 *  0b0000000000000010..Controller only, with standard feature set
 *  0b0000000000000011..Controller and target, with standard feature set
 */
#define LPI2C_VERID_FEATURE(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_VERID_FEATURE_SHIFT)) & LPI2C_VERID_FEATURE_MASK)

#define LPI2C_VERID_MINOR_MASK                   (0xFF0000U)
#define LPI2C_VERID_MINOR_SHIFT                  (16U)
/*! MINOR - Minor Version Number */
#define LPI2C_VERID_MINOR(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_VERID_MINOR_SHIFT)) & LPI2C_VERID_MINOR_MASK)

#define LPI2C_VERID_MAJOR_MASK                   (0xFF000000U)
#define LPI2C_VERID_MAJOR_SHIFT                  (24U)
/*! MAJOR - Major Version Number */
#define LPI2C_VERID_MAJOR(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_VERID_MAJOR_SHIFT)) & LPI2C_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define LPI2C_PARAM_MTXFIFO_MASK                 (0xFU)
#define LPI2C_PARAM_MTXFIFO_SHIFT                (0U)
/*! MTXFIFO - Controller Transmit FIFO Size */
#define LPI2C_PARAM_MTXFIFO(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_PARAM_MTXFIFO_SHIFT)) & LPI2C_PARAM_MTXFIFO_MASK)

#define LPI2C_PARAM_MRXFIFO_MASK                 (0xF00U)
#define LPI2C_PARAM_MRXFIFO_SHIFT                (8U)
/*! MRXFIFO - Controller Receive FIFO Size */
#define LPI2C_PARAM_MRXFIFO(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_PARAM_MRXFIFO_SHIFT)) & LPI2C_PARAM_MRXFIFO_MASK)
/*! @} */

/*! @name MCR - Controller Control */
/*! @{ */

#define LPI2C_MCR_MEN_MASK                       (0x1U)
#define LPI2C_MCR_MEN_SHIFT                      (0U)
/*! MEN - Controller Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MCR_MEN(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MCR_MEN_SHIFT)) & LPI2C_MCR_MEN_MASK)

#define LPI2C_MCR_RST_MASK                       (0x2U)
#define LPI2C_MCR_RST_SHIFT                      (1U)
/*! RST - Software Reset
 *  0b0..No effect
 *  0b1..Reset
 */
#define LPI2C_MCR_RST(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MCR_RST_SHIFT)) & LPI2C_MCR_RST_MASK)

#define LPI2C_MCR_DOZEN_MASK                     (0x4U)
#define LPI2C_MCR_DOZEN_SHIFT                    (2U)
/*! DOZEN - Doze Mode Enable
 *  0b0..Enable
 *  0b1..Disable
 */
#define LPI2C_MCR_DOZEN(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MCR_DOZEN_SHIFT)) & LPI2C_MCR_DOZEN_MASK)

#define LPI2C_MCR_DBGEN_MASK                     (0x8U)
#define LPI2C_MCR_DBGEN_SHIFT                    (3U)
/*! DBGEN - Debug Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MCR_DBGEN(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MCR_DBGEN_SHIFT)) & LPI2C_MCR_DBGEN_MASK)

#define LPI2C_MCR_RTF_MASK                       (0x100U)
#define LPI2C_MCR_RTF_SHIFT                      (8U)
/*! RTF - Reset Transmit FIFO
 *  0b0..No effect
 *  0b1..Reset transmit FIFO
 */
#define LPI2C_MCR_RTF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MCR_RTF_SHIFT)) & LPI2C_MCR_RTF_MASK)

#define LPI2C_MCR_RRF_MASK                       (0x200U)
#define LPI2C_MCR_RRF_SHIFT                      (9U)
/*! RRF - Reset Receive FIFO
 *  0b0..No effect
 *  0b1..Reset receive FIFO
 */
#define LPI2C_MCR_RRF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MCR_RRF_SHIFT)) & LPI2C_MCR_RRF_MASK)
/*! @} */

/*! @name MSR - Controller Status */
/*! @{ */

#define LPI2C_MSR_TDF_MASK                       (0x1U)
#define LPI2C_MSR_TDF_SHIFT                      (0U)
/*! TDF - Transmit Data Flag
 *  0b0..Transmit data not requested
 *  0b1..Transmit data requested
 */
#define LPI2C_MSR_TDF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_TDF_SHIFT)) & LPI2C_MSR_TDF_MASK)

#define LPI2C_MSR_RDF_MASK                       (0x2U)
#define LPI2C_MSR_RDF_SHIFT                      (1U)
/*! RDF - Receive Data Flag
 *  0b0..Receive data not ready
 *  0b1..Receive data ready
 */
#define LPI2C_MSR_RDF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_RDF_SHIFT)) & LPI2C_MSR_RDF_MASK)

#define LPI2C_MSR_EPF_MASK                       (0x100U)
#define LPI2C_MSR_EPF_SHIFT                      (8U)
/*! EPF - End Packet Flag
 *  0b0..No Stop or repeated Start generated
 *  0b1..Stop or repeated Start generated
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_EPF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_EPF_SHIFT)) & LPI2C_MSR_EPF_MASK)

#define LPI2C_MSR_SDF_MASK                       (0x200U)
#define LPI2C_MSR_SDF_SHIFT                      (9U)
/*! SDF - Stop Detect Flag
 *  0b0..No Stop condition generated
 *  0b1..Stop condition generated
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_SDF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_SDF_SHIFT)) & LPI2C_MSR_SDF_MASK)

#define LPI2C_MSR_NDF_MASK                       (0x400U)
#define LPI2C_MSR_NDF_SHIFT                      (10U)
/*! NDF - NACK Detect Flag
 *  0b0..No unexpected NACK detected
 *  0b1..Unexpected NACK detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_NDF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_NDF_SHIFT)) & LPI2C_MSR_NDF_MASK)

#define LPI2C_MSR_ALF_MASK                       (0x800U)
#define LPI2C_MSR_ALF_SHIFT                      (11U)
/*! ALF - Arbitration Lost Flag
 *  0b0..Controller did not lose arbitration
 *  0b1..Controller lost arbitration
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_ALF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_ALF_SHIFT)) & LPI2C_MSR_ALF_MASK)

#define LPI2C_MSR_FEF_MASK                       (0x1000U)
#define LPI2C_MSR_FEF_SHIFT                      (12U)
/*! FEF - FIFO Error Flag
 *  0b0..No FIFO error
 *  0b1..FIFO error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_FEF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_FEF_SHIFT)) & LPI2C_MSR_FEF_MASK)

#define LPI2C_MSR_PLTF_MASK                      (0x2000U)
#define LPI2C_MSR_PLTF_SHIFT                     (13U)
/*! PLTF - Pin Low Timeout Flag
 *  0b0..Pin low timeout did not occur
 *  0b1..Pin low timeout occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_PLTF(x)                        (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_PLTF_SHIFT)) & LPI2C_MSR_PLTF_MASK)

#define LPI2C_MSR_DMF_MASK                       (0x4000U)
#define LPI2C_MSR_DMF_SHIFT                      (14U)
/*! DMF - Data Match Flag
 *  0b0..Matching data not received
 *  0b1..Matching data received
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_DMF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_DMF_SHIFT)) & LPI2C_MSR_DMF_MASK)

#define LPI2C_MSR_STF_MASK                       (0x8000U)
#define LPI2C_MSR_STF_SHIFT                      (15U)
/*! STF - Start Flag
 *  0b0..Start condition not detected
 *  0b1..Start condition detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_MSR_STF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_STF_SHIFT)) & LPI2C_MSR_STF_MASK)

#define LPI2C_MSR_MBF_MASK                       (0x1000000U)
#define LPI2C_MSR_MBF_SHIFT                      (24U)
/*! MBF - Controller Busy Flag
 *  0b0..Idle
 *  0b1..Busy
 */
#define LPI2C_MSR_MBF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_MBF_SHIFT)) & LPI2C_MSR_MBF_MASK)

#define LPI2C_MSR_BBF_MASK                       (0x2000000U)
#define LPI2C_MSR_BBF_SHIFT                      (25U)
/*! BBF - Bus Busy Flag
 *  0b0..Idle
 *  0b1..Busy
 */
#define LPI2C_MSR_BBF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_MSR_BBF_SHIFT)) & LPI2C_MSR_BBF_MASK)
/*! @} */

/*! @name MIER - Controller Interrupt Enable */
/*! @{ */

#define LPI2C_MIER_TDIE_MASK                     (0x1U)
#define LPI2C_MIER_TDIE_SHIFT                    (0U)
/*! TDIE - Transmit Data Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_TDIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_TDIE_SHIFT)) & LPI2C_MIER_TDIE_MASK)

#define LPI2C_MIER_RDIE_MASK                     (0x2U)
#define LPI2C_MIER_RDIE_SHIFT                    (1U)
/*! RDIE - Receive Data Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_RDIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_RDIE_SHIFT)) & LPI2C_MIER_RDIE_MASK)

#define LPI2C_MIER_EPIE_MASK                     (0x100U)
#define LPI2C_MIER_EPIE_SHIFT                    (8U)
/*! EPIE - End Packet Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_EPIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_EPIE_SHIFT)) & LPI2C_MIER_EPIE_MASK)

#define LPI2C_MIER_SDIE_MASK                     (0x200U)
#define LPI2C_MIER_SDIE_SHIFT                    (9U)
/*! SDIE - Stop Detect Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_SDIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_SDIE_SHIFT)) & LPI2C_MIER_SDIE_MASK)

#define LPI2C_MIER_NDIE_MASK                     (0x400U)
#define LPI2C_MIER_NDIE_SHIFT                    (10U)
/*! NDIE - NACK Detect Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_NDIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_NDIE_SHIFT)) & LPI2C_MIER_NDIE_MASK)

#define LPI2C_MIER_ALIE_MASK                     (0x800U)
#define LPI2C_MIER_ALIE_SHIFT                    (11U)
/*! ALIE - Arbitration Lost Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_ALIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_ALIE_SHIFT)) & LPI2C_MIER_ALIE_MASK)

#define LPI2C_MIER_FEIE_MASK                     (0x1000U)
#define LPI2C_MIER_FEIE_SHIFT                    (12U)
/*! FEIE - FIFO Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_FEIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_FEIE_SHIFT)) & LPI2C_MIER_FEIE_MASK)

#define LPI2C_MIER_PLTIE_MASK                    (0x2000U)
#define LPI2C_MIER_PLTIE_SHIFT                   (13U)
/*! PLTIE - Pin Low Timeout Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_PLTIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_PLTIE_SHIFT)) & LPI2C_MIER_PLTIE_MASK)

#define LPI2C_MIER_DMIE_MASK                     (0x4000U)
#define LPI2C_MIER_DMIE_SHIFT                    (14U)
/*! DMIE - Data Match Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_DMIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_DMIE_SHIFT)) & LPI2C_MIER_DMIE_MASK)

#define LPI2C_MIER_STIE_MASK                     (0x8000U)
#define LPI2C_MIER_STIE_SHIFT                    (15U)
/*! STIE - Start Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MIER_STIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MIER_STIE_SHIFT)) & LPI2C_MIER_STIE_MASK)
/*! @} */

/*! @name MDER - Controller DMA Enable */
/*! @{ */

#define LPI2C_MDER_TDDE_MASK                     (0x1U)
#define LPI2C_MDER_TDDE_SHIFT                    (0U)
/*! TDDE - Transmit Data DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MDER_TDDE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MDER_TDDE_SHIFT)) & LPI2C_MDER_TDDE_MASK)

#define LPI2C_MDER_RDDE_MASK                     (0x2U)
#define LPI2C_MDER_RDDE_SHIFT                    (1U)
/*! RDDE - Receive Data DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MDER_RDDE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MDER_RDDE_SHIFT)) & LPI2C_MDER_RDDE_MASK)
/*! @} */

/*! @name MCFGR0 - Controller Configuration 0 */
/*! @{ */

#define LPI2C_MCFGR0_HREN_MASK                   (0x1U)
#define LPI2C_MCFGR0_HREN_SHIFT                  (0U)
/*! HREN - Host Request Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MCFGR0_HREN(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_HREN_SHIFT)) & LPI2C_MCFGR0_HREN_MASK)

#define LPI2C_MCFGR0_HRPOL_MASK                  (0x2U)
#define LPI2C_MCFGR0_HRPOL_SHIFT                 (1U)
/*! HRPOL - Host Request Polarity
 *  0b0..Active low
 *  0b1..Active high
 */
#define LPI2C_MCFGR0_HRPOL(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_HRPOL_SHIFT)) & LPI2C_MCFGR0_HRPOL_MASK)

#define LPI2C_MCFGR0_HRSEL_MASK                  (0x4U)
#define LPI2C_MCFGR0_HRSEL_SHIFT                 (2U)
/*! HRSEL - Host Request Select
 *  0b0..Host request input is pin HREQ
 *  0b1..Host request input is input trigger
 */
#define LPI2C_MCFGR0_HRSEL(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_HRSEL_SHIFT)) & LPI2C_MCFGR0_HRSEL_MASK)

#define LPI2C_MCFGR0_HRDIR_MASK                  (0x8U)
#define LPI2C_MCFGR0_HRDIR_SHIFT                 (3U)
/*! HRDIR - Host Request Direction
 *  0b0..HREQ pin is input (for LPI2C controller)
 *  0b1..HREQ pin is output (for LPI2C target)
 */
#define LPI2C_MCFGR0_HRDIR(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_HRDIR_SHIFT)) & LPI2C_MCFGR0_HRDIR_MASK)

#define LPI2C_MCFGR0_CIRFIFO_MASK                (0x100U)
#define LPI2C_MCFGR0_CIRFIFO_SHIFT               (8U)
/*! CIRFIFO - Circular FIFO Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_MCFGR0_CIRFIFO(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_CIRFIFO_SHIFT)) & LPI2C_MCFGR0_CIRFIFO_MASK)

#define LPI2C_MCFGR0_RDMO_MASK                   (0x200U)
#define LPI2C_MCFGR0_RDMO_SHIFT                  (9U)
/*! RDMO - Receive Data Match Only
 *  0b0..Received data is stored in the receive FIFO
 *  0b1..Received data is discarded unless MSR[DMF] is set
 */
#define LPI2C_MCFGR0_RDMO(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_RDMO_SHIFT)) & LPI2C_MCFGR0_RDMO_MASK)

#define LPI2C_MCFGR0_RELAX_MASK                  (0x10000U)
#define LPI2C_MCFGR0_RELAX_SHIFT                 (16U)
/*! RELAX - Relaxed Mode
 *  0b0..Normal transfer
 *  0b1..Relaxed transfer
 */
#define LPI2C_MCFGR0_RELAX(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_RELAX_SHIFT)) & LPI2C_MCFGR0_RELAX_MASK)

#define LPI2C_MCFGR0_ABORT_MASK                  (0x20000U)
#define LPI2C_MCFGR0_ABORT_SHIFT                 (17U)
/*! ABORT - Abort Transfer
 *  0b0..Normal transfer
 *  0b1..Abort existing transfer and do not start a new one
 */
#define LPI2C_MCFGR0_ABORT(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR0_ABORT_SHIFT)) & LPI2C_MCFGR0_ABORT_MASK)
/*! @} */

/*! @name MCFGR1 - Controller Configuration 1 */
/*! @{ */

#define LPI2C_MCFGR1_PRESCALE_MASK               (0x7U)
#define LPI2C_MCFGR1_PRESCALE_SHIFT              (0U)
/*! PRESCALE - Prescaler
 *  0b000..Divide by 1
 *  0b001..Divide by 2
 *  0b010..Divide by 4
 *  0b011..Divide by 8
 *  0b100..Divide by 16
 *  0b101..Divide by 32
 *  0b110..Divide by 64
 *  0b111..Divide by 128
 */
#define LPI2C_MCFGR1_PRESCALE(x)                 (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_PRESCALE_SHIFT)) & LPI2C_MCFGR1_PRESCALE_MASK)

#define LPI2C_MCFGR1_AUTOSTOP_MASK               (0x100U)
#define LPI2C_MCFGR1_AUTOSTOP_SHIFT              (8U)
/*! AUTOSTOP - Automatic Stop Generation
 *  0b0..No effect
 *  0b1..Stop automatically generated
 */
#define LPI2C_MCFGR1_AUTOSTOP(x)                 (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_AUTOSTOP_SHIFT)) & LPI2C_MCFGR1_AUTOSTOP_MASK)

#define LPI2C_MCFGR1_IGNACK_MASK                 (0x200U)
#define LPI2C_MCFGR1_IGNACK_SHIFT                (9U)
/*! IGNACK - Ignore NACK
 *  0b0..No effect
 *  0b1..Treat a received NACK as an ACK
 */
#define LPI2C_MCFGR1_IGNACK(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_IGNACK_SHIFT)) & LPI2C_MCFGR1_IGNACK_MASK)

#define LPI2C_MCFGR1_TIMECFG_MASK                (0x400U)
#define LPI2C_MCFGR1_TIMECFG_SHIFT               (10U)
/*! TIMECFG - Timeout Configuration
 *  0b0..SCL
 *  0b1..SCL or SDA
 */
#define LPI2C_MCFGR1_TIMECFG(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_TIMECFG_SHIFT)) & LPI2C_MCFGR1_TIMECFG_MASK)

#define LPI2C_MCFGR1_STOPCFG_MASK                (0x800U)
#define LPI2C_MCFGR1_STOPCFG_SHIFT               (11U)
/*! STOPCFG - Stop Configuration
 *  0b0..Any Stop condition
 *  0b1..Last Stop condition
 */
#define LPI2C_MCFGR1_STOPCFG(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_STOPCFG_SHIFT)) & LPI2C_MCFGR1_STOPCFG_MASK)

#define LPI2C_MCFGR1_STARTCFG_MASK               (0x1000U)
#define LPI2C_MCFGR1_STARTCFG_SHIFT              (12U)
/*! STARTCFG - Start Configuration
 *  0b0..Sets when both I2C bus and LPI2C controller are idle
 *  0b1..Sets when I2C bus is idle
 */
#define LPI2C_MCFGR1_STARTCFG(x)                 (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_STARTCFG_SHIFT)) & LPI2C_MCFGR1_STARTCFG_MASK)

#define LPI2C_MCFGR1_MATCFG_MASK                 (0x70000U)
#define LPI2C_MCFGR1_MATCFG_SHIFT                (16U)
/*! MATCFG - Match Configuration
 *  0b000..Match is disabled
 *  0b001..Reserved
 *  0b010..Match is enabled: first data word equals MDMR[MATCH0] OR MDMR[MATCH1]
 *  0b011..Match is enabled: any data word equals MDMR[MATCH0] OR MDMR[MATCH1]
 *  0b100..Match is enabled: (first data word equals MDMR[MATCH0]) AND (second data word equals MDMR[MATCH1)
 *  0b101..Match is enabled: (any data word equals MDMR[MATCH0]) AND (next data word equals MDMR[MATCH1)
 *  0b110..Match is enabled: (first data word AND MDMR[MATCH1]) equals (MDMR[MATCH0] AND MDMR[MATCH1])
 *  0b111..Match is enabled: (any data word AND MDMR[MATCH1]) equals (MDMR[MATCH0] AND MDMR[MATCH1])
 */
#define LPI2C_MCFGR1_MATCFG(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_MATCFG_SHIFT)) & LPI2C_MCFGR1_MATCFG_MASK)

#define LPI2C_MCFGR1_PINCFG_MASK                 (0x7000000U)
#define LPI2C_MCFGR1_PINCFG_SHIFT                (24U)
/*! PINCFG - Pin Configuration
 *  0b000..Two-pin open drain mode
 *  0b001..Two-pin output only mode (Ultra-Fast mode)
 *  0b010..Two-pin push-pull mode
 *  0b011..Four-pin push-pull mode
 *  0b100..Two-pin open-drain mode with separate LPI2C target
 *  0b101..Two-pin output only mode (Ultra-Fast mode) with separate LPI2C target
 *  0b110..Two-pin push-pull mode with separate LPI2C target
 *  0b111..Four-pin push-pull mode (inverted outputs)
 */
#define LPI2C_MCFGR1_PINCFG(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR1_PINCFG_SHIFT)) & LPI2C_MCFGR1_PINCFG_MASK)
/*! @} */

/*! @name MCFGR2 - Controller Configuration 2 */
/*! @{ */

#define LPI2C_MCFGR2_BUSIDLE_MASK                (0xFFFU)
#define LPI2C_MCFGR2_BUSIDLE_SHIFT               (0U)
/*! BUSIDLE - Bus Idle Timeout */
#define LPI2C_MCFGR2_BUSIDLE(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR2_BUSIDLE_SHIFT)) & LPI2C_MCFGR2_BUSIDLE_MASK)

#define LPI2C_MCFGR2_FILTSCL_MASK                (0xF0000U)
#define LPI2C_MCFGR2_FILTSCL_SHIFT               (16U)
/*! FILTSCL - Glitch Filter SCL */
#define LPI2C_MCFGR2_FILTSCL(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR2_FILTSCL_SHIFT)) & LPI2C_MCFGR2_FILTSCL_MASK)

#define LPI2C_MCFGR2_FILTSDA_MASK                (0xF000000U)
#define LPI2C_MCFGR2_FILTSDA_SHIFT               (24U)
/*! FILTSDA - Glitch Filter SDA */
#define LPI2C_MCFGR2_FILTSDA(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR2_FILTSDA_SHIFT)) & LPI2C_MCFGR2_FILTSDA_MASK)
/*! @} */

/*! @name MCFGR3 - Controller Configuration 3 */
/*! @{ */

#define LPI2C_MCFGR3_PINLOW_MASK                 (0xFFF00U)
#define LPI2C_MCFGR3_PINLOW_SHIFT                (8U)
/*! PINLOW - Pin Low Timeout */
#define LPI2C_MCFGR3_PINLOW(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_MCFGR3_PINLOW_SHIFT)) & LPI2C_MCFGR3_PINLOW_MASK)
/*! @} */

/*! @name MDMR - Controller Data Match */
/*! @{ */

#define LPI2C_MDMR_MATCH0_MASK                   (0xFFU)
#define LPI2C_MDMR_MATCH0_SHIFT                  (0U)
/*! MATCH0 - Match 0 Value */
#define LPI2C_MDMR_MATCH0(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MDMR_MATCH0_SHIFT)) & LPI2C_MDMR_MATCH0_MASK)

#define LPI2C_MDMR_MATCH1_MASK                   (0xFF0000U)
#define LPI2C_MDMR_MATCH1_SHIFT                  (16U)
/*! MATCH1 - Match 1 Value */
#define LPI2C_MDMR_MATCH1(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MDMR_MATCH1_SHIFT)) & LPI2C_MDMR_MATCH1_MASK)
/*! @} */

/*! @name MCCR0 - Controller Clock Configuration 0 */
/*! @{ */

#define LPI2C_MCCR0_CLKLO_MASK                   (0x3FU)
#define LPI2C_MCCR0_CLKLO_SHIFT                  (0U)
/*! CLKLO - Clock Low Period */
#define LPI2C_MCCR0_CLKLO(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR0_CLKLO_SHIFT)) & LPI2C_MCCR0_CLKLO_MASK)

#define LPI2C_MCCR0_CLKHI_MASK                   (0x3F00U)
#define LPI2C_MCCR0_CLKHI_SHIFT                  (8U)
/*! CLKHI - Clock High Period */
#define LPI2C_MCCR0_CLKHI(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR0_CLKHI_SHIFT)) & LPI2C_MCCR0_CLKHI_MASK)

#define LPI2C_MCCR0_SETHOLD_MASK                 (0x3F0000U)
#define LPI2C_MCCR0_SETHOLD_SHIFT                (16U)
/*! SETHOLD - Setup Hold Delay */
#define LPI2C_MCCR0_SETHOLD(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR0_SETHOLD_SHIFT)) & LPI2C_MCCR0_SETHOLD_MASK)

#define LPI2C_MCCR0_DATAVD_MASK                  (0x3F000000U)
#define LPI2C_MCCR0_DATAVD_SHIFT                 (24U)
/*! DATAVD - Data Valid Delay */
#define LPI2C_MCCR0_DATAVD(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR0_DATAVD_SHIFT)) & LPI2C_MCCR0_DATAVD_MASK)
/*! @} */

/*! @name MCCR1 - Controller Clock Configuration 1 */
/*! @{ */

#define LPI2C_MCCR1_CLKLO_MASK                   (0x3FU)
#define LPI2C_MCCR1_CLKLO_SHIFT                  (0U)
/*! CLKLO - Clock Low Period */
#define LPI2C_MCCR1_CLKLO(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR1_CLKLO_SHIFT)) & LPI2C_MCCR1_CLKLO_MASK)

#define LPI2C_MCCR1_CLKHI_MASK                   (0x3F00U)
#define LPI2C_MCCR1_CLKHI_SHIFT                  (8U)
/*! CLKHI - Clock High Period */
#define LPI2C_MCCR1_CLKHI(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR1_CLKHI_SHIFT)) & LPI2C_MCCR1_CLKHI_MASK)

#define LPI2C_MCCR1_SETHOLD_MASK                 (0x3F0000U)
#define LPI2C_MCCR1_SETHOLD_SHIFT                (16U)
/*! SETHOLD - Setup Hold Delay */
#define LPI2C_MCCR1_SETHOLD(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR1_SETHOLD_SHIFT)) & LPI2C_MCCR1_SETHOLD_MASK)

#define LPI2C_MCCR1_DATAVD_MASK                  (0x3F000000U)
#define LPI2C_MCCR1_DATAVD_SHIFT                 (24U)
/*! DATAVD - Data Valid Delay */
#define LPI2C_MCCR1_DATAVD(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MCCR1_DATAVD_SHIFT)) & LPI2C_MCCR1_DATAVD_MASK)
/*! @} */

/*! @name MFCR - Controller FIFO Control */
/*! @{ */

#define LPI2C_MFCR_TXWATER_MASK                  (0x7U)
#define LPI2C_MFCR_TXWATER_SHIFT                 (0U)
/*! TXWATER - Transmit FIFO Watermark */
#define LPI2C_MFCR_TXWATER(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MFCR_TXWATER_SHIFT)) & LPI2C_MFCR_TXWATER_MASK)

#define LPI2C_MFCR_RXWATER_MASK                  (0x70000U)
#define LPI2C_MFCR_RXWATER_SHIFT                 (16U)
/*! RXWATER - Receive FIFO Watermark */
#define LPI2C_MFCR_RXWATER(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MFCR_RXWATER_SHIFT)) & LPI2C_MFCR_RXWATER_MASK)
/*! @} */

/*! @name MFSR - Controller FIFO Status */
/*! @{ */

#define LPI2C_MFSR_TXCOUNT_MASK                  (0xFU)
#define LPI2C_MFSR_TXCOUNT_SHIFT                 (0U)
/*! TXCOUNT - Transmit FIFO Count */
#define LPI2C_MFSR_TXCOUNT(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MFSR_TXCOUNT_SHIFT)) & LPI2C_MFSR_TXCOUNT_MASK)

#define LPI2C_MFSR_RXCOUNT_MASK                  (0xF0000U)
#define LPI2C_MFSR_RXCOUNT_SHIFT                 (16U)
/*! RXCOUNT - Receive FIFO Count */
#define LPI2C_MFSR_RXCOUNT(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MFSR_RXCOUNT_SHIFT)) & LPI2C_MFSR_RXCOUNT_MASK)
/*! @} */

/*! @name MTDR - Controller Transmit Data */
/*! @{ */

#define LPI2C_MTDR_DATA_MASK                     (0xFFU)
#define LPI2C_MTDR_DATA_SHIFT                    (0U)
/*! DATA - Transmit Data */
#define LPI2C_MTDR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MTDR_DATA_SHIFT)) & LPI2C_MTDR_DATA_MASK)

#define LPI2C_MTDR_CMD_MASK                      (0x700U)
#define LPI2C_MTDR_CMD_SHIFT                     (8U)
/*! CMD - Command Data
 *  0b000..Transmit the value in DATA[7:0]
 *  0b001..Receive (DATA[7:0] + 1) bytes
 *  0b010..Generate Stop condition on I2C bus
 *  0b011..Receive and discard (DATA[7:0] + 1) bytes
 *  0b100..Generate (repeated) Start on the I2C bus and transmit the address in DATA[7:0]
 *  0b101..Generate (repeated) Start on the I2C bus and transmit the address in DATA[7:0] (this transfer expects a NACK to be returned)
 *  0b110..Generate (repeated) Start on the I2C bus and transmit the address in DATA[7:0] using HS mode
 *  0b111..Generate (repeated) Start on the I2C bus and transmit the address in DATA[7:0] using HS mode (this transfer expects a NACK to be returned)
 */
#define LPI2C_MTDR_CMD(x)                        (((uint32_t)(((uint32_t)(x)) << LPI2C_MTDR_CMD_SHIFT)) & LPI2C_MTDR_CMD_MASK)
/*! @} */

/*! @name MRDR - Controller Receive Data */
/*! @{ */

#define LPI2C_MRDR_DATA_MASK                     (0xFFU)
#define LPI2C_MRDR_DATA_SHIFT                    (0U)
/*! DATA - Receive Data */
#define LPI2C_MRDR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MRDR_DATA_SHIFT)) & LPI2C_MRDR_DATA_MASK)

#define LPI2C_MRDR_RXEMPTY_MASK                  (0x4000U)
#define LPI2C_MRDR_RXEMPTY_SHIFT                 (14U)
/*! RXEMPTY - Receive Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define LPI2C_MRDR_RXEMPTY(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_MRDR_RXEMPTY_SHIFT)) & LPI2C_MRDR_RXEMPTY_MASK)
/*! @} */

/*! @name MRDROR - Controller Receive Data Read Only */
/*! @{ */

#define LPI2C_MRDROR_DATA_MASK                   (0xFFU)
#define LPI2C_MRDROR_DATA_SHIFT                  (0U)
/*! DATA - Receive Data */
#define LPI2C_MRDROR_DATA(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MRDROR_DATA_SHIFT)) & LPI2C_MRDROR_DATA_MASK)

#define LPI2C_MRDROR_RXEMPTY_MASK                (0x4000U)
#define LPI2C_MRDROR_RXEMPTY_SHIFT               (14U)
/*! RXEMPTY - RX Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define LPI2C_MRDROR_RXEMPTY(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_MRDROR_RXEMPTY_SHIFT)) & LPI2C_MRDROR_RXEMPTY_MASK)
/*! @} */

/*! @name SCR - Target Control */
/*! @{ */

#define LPI2C_SCR_SEN_MASK                       (0x1U)
#define LPI2C_SCR_SEN_SHIFT                      (0U)
/*! SEN - Target Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCR_SEN(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SCR_SEN_SHIFT)) & LPI2C_SCR_SEN_MASK)

#define LPI2C_SCR_RST_MASK                       (0x2U)
#define LPI2C_SCR_RST_SHIFT                      (1U)
/*! RST - Software Reset
 *  0b0..Not reset
 *  0b1..Reset
 */
#define LPI2C_SCR_RST(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SCR_RST_SHIFT)) & LPI2C_SCR_RST_MASK)

#define LPI2C_SCR_FILTEN_MASK                    (0x10U)
#define LPI2C_SCR_FILTEN_SHIFT                   (4U)
/*! FILTEN - Filter Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCR_FILTEN(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SCR_FILTEN_SHIFT)) & LPI2C_SCR_FILTEN_MASK)

#define LPI2C_SCR_FILTDZ_MASK                    (0x20U)
#define LPI2C_SCR_FILTDZ_SHIFT                   (5U)
/*! FILTDZ - Filter Doze Enable
 *  0b0..Enable
 *  0b1..Disable
 */
#define LPI2C_SCR_FILTDZ(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SCR_FILTDZ_SHIFT)) & LPI2C_SCR_FILTDZ_MASK)

#define LPI2C_SCR_RTF_MASK                       (0x100U)
#define LPI2C_SCR_RTF_SHIFT                      (8U)
/*! RTF - Reset Transmit FIFO
 *  0b0..No effect
 *  0b1..STDR is now empty
 */
#define LPI2C_SCR_RTF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SCR_RTF_SHIFT)) & LPI2C_SCR_RTF_MASK)

#define LPI2C_SCR_RRF_MASK                       (0x200U)
#define LPI2C_SCR_RRF_SHIFT                      (9U)
/*! RRF - Reset Receive FIFO
 *  0b0..No effect
 *  0b1..SRDR is now empty
 */
#define LPI2C_SCR_RRF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SCR_RRF_SHIFT)) & LPI2C_SCR_RRF_MASK)
/*! @} */

/*! @name SSR - Target Status */
/*! @{ */

#define LPI2C_SSR_TDF_MASK                       (0x1U)
#define LPI2C_SSR_TDF_SHIFT                      (0U)
/*! TDF - Transmit Data Flag
 *  0b0..Transmit data not requested
 *  0b1..Transmit data is requested
 */
#define LPI2C_SSR_TDF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_TDF_SHIFT)) & LPI2C_SSR_TDF_MASK)

#define LPI2C_SSR_RDF_MASK                       (0x2U)
#define LPI2C_SSR_RDF_SHIFT                      (1U)
/*! RDF - Receive Data Flag
 *  0b0..Not ready
 *  0b1..Ready
 */
#define LPI2C_SSR_RDF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_RDF_SHIFT)) & LPI2C_SSR_RDF_MASK)

#define LPI2C_SSR_AVF_MASK                       (0x4U)
#define LPI2C_SSR_AVF_SHIFT                      (2U)
/*! AVF - Address Valid Flag
 *  0b0..Not valid
 *  0b1..Valid
 */
#define LPI2C_SSR_AVF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_AVF_SHIFT)) & LPI2C_SSR_AVF_MASK)

#define LPI2C_SSR_TAF_MASK                       (0x8U)
#define LPI2C_SSR_TAF_SHIFT                      (3U)
/*! TAF - Transmit ACK Flag
 *  0b0..Not required
 *  0b1..Required
 */
#define LPI2C_SSR_TAF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_TAF_SHIFT)) & LPI2C_SSR_TAF_MASK)

#define LPI2C_SSR_RSF_MASK                       (0x100U)
#define LPI2C_SSR_RSF_SHIFT                      (8U)
/*! RSF - Repeated Start Flag
 *  0b0..No repeated Start detected
 *  0b1..Repeated Start detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_SSR_RSF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_RSF_SHIFT)) & LPI2C_SSR_RSF_MASK)

#define LPI2C_SSR_SDF_MASK                       (0x200U)
#define LPI2C_SSR_SDF_SHIFT                      (9U)
/*! SDF - Stop Detect Flag
 *  0b0..No Stop detected
 *  0b1..Stop detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_SSR_SDF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_SDF_SHIFT)) & LPI2C_SSR_SDF_MASK)

#define LPI2C_SSR_BEF_MASK                       (0x400U)
#define LPI2C_SSR_BEF_SHIFT                      (10U)
/*! BEF - Bit Error Flag
 *  0b0..No bit error occurred
 *  0b1..Bit error occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_SSR_BEF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_BEF_SHIFT)) & LPI2C_SSR_BEF_MASK)

#define LPI2C_SSR_FEF_MASK                       (0x800U)
#define LPI2C_SSR_FEF_SHIFT                      (11U)
/*! FEF - FIFO Error Flag
 *  0b0..No FIFO error
 *  0b1..FIFO error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPI2C_SSR_FEF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_FEF_SHIFT)) & LPI2C_SSR_FEF_MASK)

#define LPI2C_SSR_AM0F_MASK                      (0x1000U)
#define LPI2C_SSR_AM0F_SHIFT                     (12U)
/*! AM0F - Address Match 0 Flag
 *  0b0..ADDR0 matching address not received
 *  0b1..ADDR0 matching address received
 */
#define LPI2C_SSR_AM0F(x)                        (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_AM0F_SHIFT)) & LPI2C_SSR_AM0F_MASK)

#define LPI2C_SSR_AM1F_MASK                      (0x2000U)
#define LPI2C_SSR_AM1F_SHIFT                     (13U)
/*! AM1F - Address Match 1 Flag
 *  0b0..Matching address not received
 *  0b1..Matching address received
 */
#define LPI2C_SSR_AM1F(x)                        (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_AM1F_SHIFT)) & LPI2C_SSR_AM1F_MASK)

#define LPI2C_SSR_GCF_MASK                       (0x4000U)
#define LPI2C_SSR_GCF_SHIFT                      (14U)
/*! GCF - General Call Flag
 *  0b0..General call address disabled or not detected
 *  0b1..General call address detected
 */
#define LPI2C_SSR_GCF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_GCF_SHIFT)) & LPI2C_SSR_GCF_MASK)

#define LPI2C_SSR_SARF_MASK                      (0x8000U)
#define LPI2C_SSR_SARF_SHIFT                     (15U)
/*! SARF - SMBus Alert Response Flag
 *  0b0..Disabled or not detected
 *  0b1..Enabled and detected
 */
#define LPI2C_SSR_SARF(x)                        (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_SARF_SHIFT)) & LPI2C_SSR_SARF_MASK)

#define LPI2C_SSR_SBF_MASK                       (0x1000000U)
#define LPI2C_SSR_SBF_SHIFT                      (24U)
/*! SBF - Target Busy Flag
 *  0b0..Idle
 *  0b1..Busy
 */
#define LPI2C_SSR_SBF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_SBF_SHIFT)) & LPI2C_SSR_SBF_MASK)

#define LPI2C_SSR_BBF_MASK                       (0x2000000U)
#define LPI2C_SSR_BBF_SHIFT                      (25U)
/*! BBF - Bus Busy Flag
 *  0b0..Idle
 *  0b1..Busy
 */
#define LPI2C_SSR_BBF(x)                         (((uint32_t)(((uint32_t)(x)) << LPI2C_SSR_BBF_SHIFT)) & LPI2C_SSR_BBF_MASK)
/*! @} */

/*! @name SIER - Target Interrupt Enable */
/*! @{ */

#define LPI2C_SIER_TDIE_MASK                     (0x1U)
#define LPI2C_SIER_TDIE_SHIFT                    (0U)
/*! TDIE - Transmit Data Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_TDIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_TDIE_SHIFT)) & LPI2C_SIER_TDIE_MASK)

#define LPI2C_SIER_RDIE_MASK                     (0x2U)
#define LPI2C_SIER_RDIE_SHIFT                    (1U)
/*! RDIE - Receive Data Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_RDIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_RDIE_SHIFT)) & LPI2C_SIER_RDIE_MASK)

#define LPI2C_SIER_AVIE_MASK                     (0x4U)
#define LPI2C_SIER_AVIE_SHIFT                    (2U)
/*! AVIE - Address Valid Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_AVIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_AVIE_SHIFT)) & LPI2C_SIER_AVIE_MASK)

#define LPI2C_SIER_TAIE_MASK                     (0x8U)
#define LPI2C_SIER_TAIE_SHIFT                    (3U)
/*! TAIE - Transmit ACK Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_TAIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_TAIE_SHIFT)) & LPI2C_SIER_TAIE_MASK)

#define LPI2C_SIER_RSIE_MASK                     (0x100U)
#define LPI2C_SIER_RSIE_SHIFT                    (8U)
/*! RSIE - Repeated Start Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_RSIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_RSIE_SHIFT)) & LPI2C_SIER_RSIE_MASK)

#define LPI2C_SIER_SDIE_MASK                     (0x200U)
#define LPI2C_SIER_SDIE_SHIFT                    (9U)
/*! SDIE - Stop Detect Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_SDIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_SDIE_SHIFT)) & LPI2C_SIER_SDIE_MASK)

#define LPI2C_SIER_BEIE_MASK                     (0x400U)
#define LPI2C_SIER_BEIE_SHIFT                    (10U)
/*! BEIE - Bit Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_BEIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_BEIE_SHIFT)) & LPI2C_SIER_BEIE_MASK)

#define LPI2C_SIER_FEIE_MASK                     (0x800U)
#define LPI2C_SIER_FEIE_SHIFT                    (11U)
/*! FEIE - FIFO Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_FEIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_FEIE_SHIFT)) & LPI2C_SIER_FEIE_MASK)

#define LPI2C_SIER_AM0IE_MASK                    (0x1000U)
#define LPI2C_SIER_AM0IE_SHIFT                   (12U)
/*! AM0IE - Address Match 0 Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_AM0IE(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_AM0IE_SHIFT)) & LPI2C_SIER_AM0IE_MASK)

#define LPI2C_SIER_AM1IE_MASK                    (0x2000U)
#define LPI2C_SIER_AM1IE_SHIFT                   (13U)
/*! AM1IE - Address Match 1 Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_AM1IE(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_AM1IE_SHIFT)) & LPI2C_SIER_AM1IE_MASK)

#define LPI2C_SIER_GCIE_MASK                     (0x4000U)
#define LPI2C_SIER_GCIE_SHIFT                    (14U)
/*! GCIE - General Call Interrupt Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define LPI2C_SIER_GCIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_GCIE_SHIFT)) & LPI2C_SIER_GCIE_MASK)

#define LPI2C_SIER_SARIE_MASK                    (0x8000U)
#define LPI2C_SIER_SARIE_SHIFT                   (15U)
/*! SARIE - SMBus Alert Response Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SIER_SARIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SIER_SARIE_SHIFT)) & LPI2C_SIER_SARIE_MASK)
/*! @} */

/*! @name SDER - Target DMA Enable */
/*! @{ */

#define LPI2C_SDER_TDDE_MASK                     (0x1U)
#define LPI2C_SDER_TDDE_SHIFT                    (0U)
/*! TDDE - Transmit Data DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SDER_TDDE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SDER_TDDE_SHIFT)) & LPI2C_SDER_TDDE_MASK)

#define LPI2C_SDER_RDDE_MASK                     (0x2U)
#define LPI2C_SDER_RDDE_SHIFT                    (1U)
/*! RDDE - Receive Data DMA Enable
 *  0b0..Disable DMA request
 *  0b1..Enable DMA request
 */
#define LPI2C_SDER_RDDE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SDER_RDDE_SHIFT)) & LPI2C_SDER_RDDE_MASK)

#define LPI2C_SDER_AVDE_MASK                     (0x4U)
#define LPI2C_SDER_AVDE_SHIFT                    (2U)
/*! AVDE - Address Valid DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SDER_AVDE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SDER_AVDE_SHIFT)) & LPI2C_SDER_AVDE_MASK)

#define LPI2C_SDER_RSDE_MASK                     (0x100U)
#define LPI2C_SDER_RSDE_SHIFT                    (8U)
/*! RSDE - Repeated Start DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SDER_RSDE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SDER_RSDE_SHIFT)) & LPI2C_SDER_RSDE_MASK)

#define LPI2C_SDER_SDDE_MASK                     (0x200U)
#define LPI2C_SDER_SDDE_SHIFT                    (9U)
/*! SDDE - Stop Detect DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SDER_SDDE(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SDER_SDDE_SHIFT)) & LPI2C_SDER_SDDE_MASK)
/*! @} */

/*! @name SCFGR0 - Target Configuration 0 */
/*! @{ */

#define LPI2C_SCFGR0_RDREQ_MASK                  (0x1U)
#define LPI2C_SCFGR0_RDREQ_SHIFT                 (0U)
/*! RDREQ - Read Request
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR0_RDREQ(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR0_RDREQ_SHIFT)) & LPI2C_SCFGR0_RDREQ_MASK)

#define LPI2C_SCFGR0_RDACK_MASK                  (0x2U)
#define LPI2C_SCFGR0_RDACK_SHIFT                 (1U)
/*! RDACK - Read Acknowledge Flag
 *  0b0..Read Request not acknowledged
 *  0b1..Read Request acknowledged
 */
#define LPI2C_SCFGR0_RDACK(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR0_RDACK_SHIFT)) & LPI2C_SCFGR0_RDACK_MASK)
/*! @} */

/*! @name SCFGR1 - Target Configuration 1 */
/*! @{ */

#define LPI2C_SCFGR1_ADRSTALL_MASK               (0x1U)
#define LPI2C_SCFGR1_ADRSTALL_SHIFT              (0U)
/*! ADRSTALL - Address SCL Stall
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_ADRSTALL(x)                 (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_ADRSTALL_SHIFT)) & LPI2C_SCFGR1_ADRSTALL_MASK)

#define LPI2C_SCFGR1_RXSTALL_MASK                (0x2U)
#define LPI2C_SCFGR1_RXSTALL_SHIFT               (1U)
/*! RXSTALL - RX SCL Stall
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_RXSTALL(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_RXSTALL_SHIFT)) & LPI2C_SCFGR1_RXSTALL_MASK)

#define LPI2C_SCFGR1_TXDSTALL_MASK               (0x4U)
#define LPI2C_SCFGR1_TXDSTALL_SHIFT              (2U)
/*! TXDSTALL - Transmit Data SCL Stall
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_TXDSTALL(x)                 (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_TXDSTALL_SHIFT)) & LPI2C_SCFGR1_TXDSTALL_MASK)

#define LPI2C_SCFGR1_ACKSTALL_MASK               (0x8U)
#define LPI2C_SCFGR1_ACKSTALL_SHIFT              (3U)
/*! ACKSTALL - ACK SCL Stall
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_ACKSTALL(x)                 (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_ACKSTALL_SHIFT)) & LPI2C_SCFGR1_ACKSTALL_MASK)

#define LPI2C_SCFGR1_RXNACK_MASK                 (0x10U)
#define LPI2C_SCFGR1_RXNACK_SHIFT                (4U)
/*! RXNACK - Receive NACK
 *  0b0..ACK or NACK always determined by STAR[TXNACK]
 *  0b1..NACK always generated on address overrun or receive data overrun, otherwise ACK or NACK is determined by STAR[TXNACK]
 */
#define LPI2C_SCFGR1_RXNACK(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_RXNACK_SHIFT)) & LPI2C_SCFGR1_RXNACK_MASK)

#define LPI2C_SCFGR1_GCEN_MASK                   (0x100U)
#define LPI2C_SCFGR1_GCEN_SHIFT                  (8U)
/*! GCEN - General Call Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_GCEN(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_GCEN_SHIFT)) & LPI2C_SCFGR1_GCEN_MASK)

#define LPI2C_SCFGR1_SAEN_MASK                   (0x200U)
#define LPI2C_SCFGR1_SAEN_SHIFT                  (9U)
/*! SAEN - SMBus Alert Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_SAEN(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_SAEN_SHIFT)) & LPI2C_SCFGR1_SAEN_MASK)

#define LPI2C_SCFGR1_TXCFG_MASK                  (0x400U)
#define LPI2C_SCFGR1_TXCFG_SHIFT                 (10U)
/*! TXCFG - Transmit Flag Configuration
 *  0b0..MSR[TDF] is set only during a target-transmit transfer when STDR is empty
 *  0b1..MSR[TDF] is set whenever STDR is empty
 */
#define LPI2C_SCFGR1_TXCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_TXCFG_SHIFT)) & LPI2C_SCFGR1_TXCFG_MASK)

#define LPI2C_SCFGR1_RXCFG_MASK                  (0x800U)
#define LPI2C_SCFGR1_RXCFG_SHIFT                 (11U)
/*! RXCFG - Receive Data Configuration
 *  0b0..Return received data, clear MSR[RDF]
 *  0b1..Return SASR and clear SSR[AVF] when SSR[AVF] is set, return received data and clear MSR[RDF] when SSR[AFV] is not set
 */
#define LPI2C_SCFGR1_RXCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_RXCFG_SHIFT)) & LPI2C_SCFGR1_RXCFG_MASK)

#define LPI2C_SCFGR1_IGNACK_MASK                 (0x1000U)
#define LPI2C_SCFGR1_IGNACK_SHIFT                (12U)
/*! IGNACK - Ignore NACK
 *  0b0..End transfer on NACK
 *  0b1..Do not end transfer on NACK
 */
#define LPI2C_SCFGR1_IGNACK(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_IGNACK_SHIFT)) & LPI2C_SCFGR1_IGNACK_MASK)

#define LPI2C_SCFGR1_HSMEN_MASK                  (0x2000U)
#define LPI2C_SCFGR1_HSMEN_SHIFT                 (13U)
/*! HSMEN - HS Mode Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_HSMEN(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_HSMEN_SHIFT)) & LPI2C_SCFGR1_HSMEN_MASK)

#define LPI2C_SCFGR1_ADDRCFG_MASK                (0x70000U)
#define LPI2C_SCFGR1_ADDRCFG_SHIFT               (16U)
/*! ADDRCFG - Address Configuration
 *  0b000..Address match 0 (7-bit)
 *  0b001..Address match 0 (10-bit)
 *  0b010..Address match 0 (7-bit) or address match 1 (7-bit)
 *  0b011..Address match 0 (10-bit) or address match 1 (10-bit)
 *  0b100..Address match 0 (7-bit) or address match 1 (10-bit)
 *  0b101..Address match 0 (10-bit) or address match 1 (7-bit)
 *  0b110..From address match 0 (7-bit) to address match 1 (7-bit)
 *  0b111..From address match 0 (10-bit) to address match 1 (10-bit)
 */
#define LPI2C_SCFGR1_ADDRCFG(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_ADDRCFG_SHIFT)) & LPI2C_SCFGR1_ADDRCFG_MASK)

#define LPI2C_SCFGR1_RXALL_MASK                  (0x1000000U)
#define LPI2C_SCFGR1_RXALL_SHIFT                 (24U)
/*! RXALL - Receive All
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPI2C_SCFGR1_RXALL(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_RXALL_SHIFT)) & LPI2C_SCFGR1_RXALL_MASK)

#define LPI2C_SCFGR1_RSCFG_MASK                  (0x2000000U)
#define LPI2C_SCFGR1_RSCFG_SHIFT                 (25U)
/*! RSCFG - Repeated Start Configuration
 *  0b0..Any repeated Start condition following an address match
 *  0b1..Any repeated Start condition
 */
#define LPI2C_SCFGR1_RSCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_RSCFG_SHIFT)) & LPI2C_SCFGR1_RSCFG_MASK)

#define LPI2C_SCFGR1_SDCFG_MASK                  (0x4000000U)
#define LPI2C_SCFGR1_SDCFG_SHIFT                 (26U)
/*! SDCFG - Stop Detect Configuration
 *  0b0..Any Stop condition following an address match
 *  0b1..Any Stop condition
 */
#define LPI2C_SCFGR1_SDCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR1_SDCFG_SHIFT)) & LPI2C_SCFGR1_SDCFG_MASK)
/*! @} */

/*! @name SCFGR2 - Target Configuration 2 */
/*! @{ */

#define LPI2C_SCFGR2_CLKHOLD_MASK                (0xFU)
#define LPI2C_SCFGR2_CLKHOLD_SHIFT               (0U)
/*! CLKHOLD - Clock Hold Time */
#define LPI2C_SCFGR2_CLKHOLD(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR2_CLKHOLD_SHIFT)) & LPI2C_SCFGR2_CLKHOLD_MASK)

#define LPI2C_SCFGR2_DATAVD_MASK                 (0x3F00U)
#define LPI2C_SCFGR2_DATAVD_SHIFT                (8U)
/*! DATAVD - Data Valid Delay */
#define LPI2C_SCFGR2_DATAVD(x)                   (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR2_DATAVD_SHIFT)) & LPI2C_SCFGR2_DATAVD_MASK)

#define LPI2C_SCFGR2_FILTSCL_MASK                (0xF0000U)
#define LPI2C_SCFGR2_FILTSCL_SHIFT               (16U)
/*! FILTSCL - Glitch Filter SCL */
#define LPI2C_SCFGR2_FILTSCL(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR2_FILTSCL_SHIFT)) & LPI2C_SCFGR2_FILTSCL_MASK)

#define LPI2C_SCFGR2_FILTSDA_MASK                (0xF000000U)
#define LPI2C_SCFGR2_FILTSDA_SHIFT               (24U)
/*! FILTSDA - Glitch Filter SDA */
#define LPI2C_SCFGR2_FILTSDA(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_SCFGR2_FILTSDA_SHIFT)) & LPI2C_SCFGR2_FILTSDA_MASK)
/*! @} */

/*! @name SAMR - Target Address Match */
/*! @{ */

#define LPI2C_SAMR_ADDR0_MASK                    (0x7FEU)
#define LPI2C_SAMR_ADDR0_SHIFT                   (1U)
/*! ADDR0 - Address 0 Value */
#define LPI2C_SAMR_ADDR0(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SAMR_ADDR0_SHIFT)) & LPI2C_SAMR_ADDR0_MASK)

#define LPI2C_SAMR_ADDR1_MASK                    (0x7FE0000U)
#define LPI2C_SAMR_ADDR1_SHIFT                   (17U)
/*! ADDR1 - Address 1 Value */
#define LPI2C_SAMR_ADDR1(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SAMR_ADDR1_SHIFT)) & LPI2C_SAMR_ADDR1_MASK)
/*! @} */

/*! @name SASR - Target Address Status */
/*! @{ */

#define LPI2C_SASR_RADDR_MASK                    (0x7FFU)
#define LPI2C_SASR_RADDR_SHIFT                   (0U)
/*! RADDR - Received Address */
#define LPI2C_SASR_RADDR(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SASR_RADDR_SHIFT)) & LPI2C_SASR_RADDR_MASK)

#define LPI2C_SASR_ANV_MASK                      (0x4000U)
#define LPI2C_SASR_ANV_SHIFT                     (14U)
/*! ANV - Address Not Valid
 *  0b0..Valid
 *  0b1..Not valid
 */
#define LPI2C_SASR_ANV(x)                        (((uint32_t)(((uint32_t)(x)) << LPI2C_SASR_ANV_SHIFT)) & LPI2C_SASR_ANV_MASK)
/*! @} */

/*! @name STAR - Target Transmit ACK */
/*! @{ */

#define LPI2C_STAR_TXNACK_MASK                   (0x1U)
#define LPI2C_STAR_TXNACK_SHIFT                  (0U)
/*! TXNACK - Transmit NACK
 *  0b0..Transmit ACK
 *  0b1..Transmit NACK
 */
#define LPI2C_STAR_TXNACK(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_STAR_TXNACK_SHIFT)) & LPI2C_STAR_TXNACK_MASK)
/*! @} */

/*! @name STDR - Target Transmit Data */
/*! @{ */

#define LPI2C_STDR_DATA_MASK                     (0xFFU)
#define LPI2C_STDR_DATA_SHIFT                    (0U)
/*! DATA - Transmit Data */
#define LPI2C_STDR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_STDR_DATA_SHIFT)) & LPI2C_STDR_DATA_MASK)
/*! @} */

/*! @name SRDR - Target Receive Data */
/*! @{ */

#define LPI2C_SRDR_DATA_MASK                     (0xFFU)
#define LPI2C_SRDR_DATA_SHIFT                    (0U)
/*! DATA - Received Data */
#define LPI2C_SRDR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDR_DATA_SHIFT)) & LPI2C_SRDR_DATA_MASK)

#define LPI2C_SRDR_RADDR_MASK                    (0x700U)
#define LPI2C_SRDR_RADDR_SHIFT                   (8U)
/*! RADDR - Received Address */
#define LPI2C_SRDR_RADDR(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDR_RADDR_SHIFT)) & LPI2C_SRDR_RADDR_MASK)

#define LPI2C_SRDR_RXEMPTY_MASK                  (0x4000U)
#define LPI2C_SRDR_RXEMPTY_SHIFT                 (14U)
/*! RXEMPTY - Receive Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define LPI2C_SRDR_RXEMPTY(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDR_RXEMPTY_SHIFT)) & LPI2C_SRDR_RXEMPTY_MASK)

#define LPI2C_SRDR_SOF_MASK                      (0x8000U)
#define LPI2C_SRDR_SOF_SHIFT                     (15U)
/*! SOF - Start of Frame
 *  0b0..Not first
 *  0b1..First
 */
#define LPI2C_SRDR_SOF(x)                        (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDR_SOF_SHIFT)) & LPI2C_SRDR_SOF_MASK)
/*! @} */

/*! @name SRDROR - Target Receive Data Read Only */
/*! @{ */

#define LPI2C_SRDROR_DATA_MASK                   (0xFFU)
#define LPI2C_SRDROR_DATA_SHIFT                  (0U)
/*! DATA - Receive Data */
#define LPI2C_SRDROR_DATA(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDROR_DATA_SHIFT)) & LPI2C_SRDROR_DATA_MASK)

#define LPI2C_SRDROR_RADDR_MASK                  (0x700U)
#define LPI2C_SRDROR_RADDR_SHIFT                 (8U)
/*! RADDR - Received Address */
#define LPI2C_SRDROR_RADDR(x)                    (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDROR_RADDR_SHIFT)) & LPI2C_SRDROR_RADDR_MASK)

#define LPI2C_SRDROR_RXEMPTY_MASK                (0x4000U)
#define LPI2C_SRDROR_RXEMPTY_SHIFT               (14U)
/*! RXEMPTY - Receive Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define LPI2C_SRDROR_RXEMPTY(x)                  (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDROR_RXEMPTY_SHIFT)) & LPI2C_SRDROR_RXEMPTY_MASK)

#define LPI2C_SRDROR_SOF_MASK                    (0x8000U)
#define LPI2C_SRDROR_SOF_SHIFT                   (15U)
/*! SOF - Start of Frame
 *  0b0..Not the first
 *  0b1..First
 */
#define LPI2C_SRDROR_SOF(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_SRDROR_SOF_SHIFT)) & LPI2C_SRDROR_SOF_MASK)
/*! @} */

/*! @name MTCBR - Controller Transmit Command Burst */
/*! @{ */

#define LPI2C_MTCBR_DATA_MASK                    (0xFFU)
#define LPI2C_MTCBR_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define LPI2C_MTCBR_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << LPI2C_MTCBR_DATA_SHIFT)) & LPI2C_MTCBR_DATA_MASK)

#define LPI2C_MTCBR_CMD_MASK                     (0x700U)
#define LPI2C_MTCBR_CMD_SHIFT                    (8U)
/*! CMD - Command */
#define LPI2C_MTCBR_CMD(x)                       (((uint32_t)(((uint32_t)(x)) << LPI2C_MTCBR_CMD_SHIFT)) & LPI2C_MTCBR_CMD_MASK)
/*! @} */

/* The count of LPI2C_MTCBR */
#define LPI2C_MTCBR_COUNT                        (128U)

/*! @name MTDBR - Transmit Data Burst */
/*! @{ */

#define LPI2C_MTDBR_DATA0_MASK                   (0xFFU)
#define LPI2C_MTDBR_DATA0_SHIFT                  (0U)
/*! DATA0 - Data */
#define LPI2C_MTDBR_DATA0(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MTDBR_DATA0_SHIFT)) & LPI2C_MTDBR_DATA0_MASK)

#define LPI2C_MTDBR_DATA1_MASK                   (0xFF00U)
#define LPI2C_MTDBR_DATA1_SHIFT                  (8U)
/*! DATA1 - Data */
#define LPI2C_MTDBR_DATA1(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MTDBR_DATA1_SHIFT)) & LPI2C_MTDBR_DATA1_MASK)

#define LPI2C_MTDBR_DATA2_MASK                   (0xFF0000U)
#define LPI2C_MTDBR_DATA2_SHIFT                  (16U)
/*! DATA2 - Data */
#define LPI2C_MTDBR_DATA2(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MTDBR_DATA2_SHIFT)) & LPI2C_MTDBR_DATA2_MASK)

#define LPI2C_MTDBR_DATA3_MASK                   (0xFF000000U)
#define LPI2C_MTDBR_DATA3_SHIFT                  (24U)
/*! DATA3 - Data */
#define LPI2C_MTDBR_DATA3(x)                     (((uint32_t)(((uint32_t)(x)) << LPI2C_MTDBR_DATA3_SHIFT)) & LPI2C_MTDBR_DATA3_MASK)
/*! @} */

/* The count of LPI2C_MTDBR */
#define LPI2C_MTDBR_COUNT                        (256U)


/*!
 * @}
 */ /* end of group LPI2C_Register_Masks */


/* LPI2C - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral LPI2C1 base address */
  #define LPI2C1_BASE                              (0x54340000u)
  /** Peripheral LPI2C1 base address */
  #define LPI2C1_BASE_NS                           (0x44340000u)
  /** Peripheral LPI2C1 base pointer */
  #define LPI2C1                                   ((LPI2C_Type *)LPI2C1_BASE)
  /** Peripheral LPI2C1 base pointer */
  #define LPI2C1_NS                                ((LPI2C_Type *)LPI2C1_BASE_NS)
  /** Peripheral LPI2C2 base address */
  #define LPI2C2_BASE                              (0x54350000u)
  /** Peripheral LPI2C2 base address */
  #define LPI2C2_BASE_NS                           (0x44350000u)
  /** Peripheral LPI2C2 base pointer */
  #define LPI2C2                                   ((LPI2C_Type *)LPI2C2_BASE)
  /** Peripheral LPI2C2 base pointer */
  #define LPI2C2_NS                                ((LPI2C_Type *)LPI2C2_BASE_NS)
  /** Peripheral LPI2C3 base address */
  #define LPI2C3_BASE                              (0x52530000u)
  /** Peripheral LPI2C3 base address */
  #define LPI2C3_BASE_NS                           (0x42530000u)
  /** Peripheral LPI2C3 base pointer */
  #define LPI2C3                                   ((LPI2C_Type *)LPI2C3_BASE)
  /** Peripheral LPI2C3 base pointer */
  #define LPI2C3_NS                                ((LPI2C_Type *)LPI2C3_BASE_NS)
  /** Array initializer of LPI2C peripheral base addresses */
  #define LPI2C_BASE_ADDRS                         { 0u, LPI2C1_BASE, LPI2C2_BASE, LPI2C3_BASE }
  /** Array initializer of LPI2C peripheral base pointers */
  #define LPI2C_BASE_PTRS                          { (LPI2C_Type *)0u, LPI2C1, LPI2C2, LPI2C3 }
  /** Array initializer of LPI2C peripheral base addresses */
  #define LPI2C_BASE_ADDRS_NS                      { 0u, LPI2C1_BASE_NS, LPI2C2_BASE_NS, LPI2C3_BASE_NS }
  /** Array initializer of LPI2C peripheral base pointers */
  #define LPI2C_BASE_PTRS_NS                       { (LPI2C_Type *)0u, LPI2C1_NS, LPI2C2_NS, LPI2C3_NS }
#else
  /** Peripheral LPI2C1 base address */
  #define LPI2C1_BASE                              (0x44340000u)
  /** Peripheral LPI2C1 base pointer */
  #define LPI2C1                                   ((LPI2C_Type *)LPI2C1_BASE)
  /** Peripheral LPI2C2 base address */
  #define LPI2C2_BASE                              (0x44350000u)
  /** Peripheral LPI2C2 base pointer */
  #define LPI2C2                                   ((LPI2C_Type *)LPI2C2_BASE)
  /** Peripheral LPI2C3 base address */
  #define LPI2C3_BASE                              (0x42530000u)
  /** Peripheral LPI2C3 base pointer */
  #define LPI2C3                                   ((LPI2C_Type *)LPI2C3_BASE)
  /** Array initializer of LPI2C peripheral base addresses */
  #define LPI2C_BASE_ADDRS                         { 0u, LPI2C1_BASE, LPI2C2_BASE, LPI2C3_BASE }
  /** Array initializer of LPI2C peripheral base pointers */
  #define LPI2C_BASE_PTRS                          { (LPI2C_Type *)0u, LPI2C1, LPI2C2, LPI2C3 }
#endif
/** Interrupt vectors for the LPI2C peripheral type */
#define LPI2C_IRQS                               { NotAvail_IRQn, LPI2C1_IRQn, LPI2C2_IRQn, LPI2C3_IRQn }

/*!
 * @}
 */ /* end of group LPI2C_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- LPIT Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPIT_Peripheral_Access_Layer LPIT Peripheral Access Layer
 * @{
 */

/** LPIT - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
  __IO uint32_t MCR;                               /**< Module Control, offset: 0x8 */
  __IO uint32_t MSR;                               /**< Module Status, offset: 0xC */
  __IO uint32_t MIER;                              /**< Module Interrupt Enable, offset: 0x10 */
  __IO uint32_t SETTEN;                            /**< Set Timer Enable, offset: 0x14 */
  __O  uint32_t CLRTEN;                            /**< Clear Timer Enable, offset: 0x18 */
       uint8_t RESERVED_0[4];
  struct {                                         /* offset: 0x20, array step: 0x10 */
    __IO uint32_t TVAL;                              /**< Timer Value, array offset: 0x20, array step: 0x10 */
    __I  uint32_t CVAL;                              /**< Current Timer Value, array offset: 0x24, array step: 0x10 */
    __IO uint32_t TCTRL;                             /**< Timer Control, array offset: 0x28, array step: 0x10 */
         uint8_t RESERVED_0[4];
  } CHANNEL[4];
} LPIT_Type;

/* ----------------------------------------------------------------------------
   -- LPIT Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPIT_Register_Masks LPIT Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define LPIT_VERID_FEATURE_MASK                  (0xFFFFU)
#define LPIT_VERID_FEATURE_SHIFT                 (0U)
/*! FEATURE - Feature Number */
#define LPIT_VERID_FEATURE(x)                    (((uint32_t)(((uint32_t)(x)) << LPIT_VERID_FEATURE_SHIFT)) & LPIT_VERID_FEATURE_MASK)

#define LPIT_VERID_MINOR_MASK                    (0xFF0000U)
#define LPIT_VERID_MINOR_SHIFT                   (16U)
/*! MINOR - Minor Version Number */
#define LPIT_VERID_MINOR(x)                      (((uint32_t)(((uint32_t)(x)) << LPIT_VERID_MINOR_SHIFT)) & LPIT_VERID_MINOR_MASK)

#define LPIT_VERID_MAJOR_MASK                    (0xFF000000U)
#define LPIT_VERID_MAJOR_SHIFT                   (24U)
/*! MAJOR - Major Version Number */
#define LPIT_VERID_MAJOR(x)                      (((uint32_t)(((uint32_t)(x)) << LPIT_VERID_MAJOR_SHIFT)) & LPIT_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define LPIT_PARAM_CHANNEL_MASK                  (0xFFU)
#define LPIT_PARAM_CHANNEL_SHIFT                 (0U)
/*! CHANNEL - Number of Timer Channels */
#define LPIT_PARAM_CHANNEL(x)                    (((uint32_t)(((uint32_t)(x)) << LPIT_PARAM_CHANNEL_SHIFT)) & LPIT_PARAM_CHANNEL_MASK)

#define LPIT_PARAM_EXT_TRIG_MASK                 (0xFF00U)
#define LPIT_PARAM_EXT_TRIG_SHIFT                (8U)
/*! EXT_TRIG - Number of External Trigger Inputs */
#define LPIT_PARAM_EXT_TRIG(x)                   (((uint32_t)(((uint32_t)(x)) << LPIT_PARAM_EXT_TRIG_SHIFT)) & LPIT_PARAM_EXT_TRIG_MASK)
/*! @} */

/*! @name MCR - Module Control */
/*! @{ */

#define LPIT_MCR_M_CEN_MASK                      (0x1U)
#define LPIT_MCR_M_CEN_SHIFT                     (0U)
/*! M_CEN - Module Clock Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPIT_MCR_M_CEN(x)                        (((uint32_t)(((uint32_t)(x)) << LPIT_MCR_M_CEN_SHIFT)) & LPIT_MCR_M_CEN_MASK)

#define LPIT_MCR_SW_RST_MASK                     (0x2U)
#define LPIT_MCR_SW_RST_SHIFT                    (1U)
/*! SW_RST - Software Reset
 *  0b0..Does not reset
 *  0b1..Resets
 */
#define LPIT_MCR_SW_RST(x)                       (((uint32_t)(((uint32_t)(x)) << LPIT_MCR_SW_RST_SHIFT)) & LPIT_MCR_SW_RST_MASK)

#define LPIT_MCR_DOZE_EN_MASK                    (0x4U)
#define LPIT_MCR_DOZE_EN_SHIFT                   (2U)
/*! DOZE_EN - DOZE Mode Enable
 *  0b0..Stops timer channels
 *  0b1..Allows timer channels to continue running
 */
#define LPIT_MCR_DOZE_EN(x)                      (((uint32_t)(((uint32_t)(x)) << LPIT_MCR_DOZE_EN_SHIFT)) & LPIT_MCR_DOZE_EN_MASK)

#define LPIT_MCR_DBG_EN_MASK                     (0x8U)
#define LPIT_MCR_DBG_EN_SHIFT                    (3U)
/*! DBG_EN - Debug Mode Enable
 *  0b0..Stops timer channels
 *  0b1..Allows timer channels to continue running
 */
#define LPIT_MCR_DBG_EN(x)                       (((uint32_t)(((uint32_t)(x)) << LPIT_MCR_DBG_EN_SHIFT)) & LPIT_MCR_DBG_EN_MASK)
/*! @} */

/*! @name MSR - Module Status */
/*! @{ */

#define LPIT_MSR_TIF0_MASK                       (0x1U)
#define LPIT_MSR_TIF0_SHIFT                      (0U)
/*! TIF0 - Channel 0 Timer Interrupt Flag
 *  0b0..Not timed out
 *  0b1..Timed out
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPIT_MSR_TIF0(x)                         (((uint32_t)(((uint32_t)(x)) << LPIT_MSR_TIF0_SHIFT)) & LPIT_MSR_TIF0_MASK)

#define LPIT_MSR_TIF1_MASK                       (0x2U)
#define LPIT_MSR_TIF1_SHIFT                      (1U)
/*! TIF1 - Channel 1 Timer Interrupt Flag
 *  0b0..Not timed out
 *  0b1..Timed out
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPIT_MSR_TIF1(x)                         (((uint32_t)(((uint32_t)(x)) << LPIT_MSR_TIF1_SHIFT)) & LPIT_MSR_TIF1_MASK)

#define LPIT_MSR_TIF2_MASK                       (0x4U)
#define LPIT_MSR_TIF2_SHIFT                      (2U)
/*! TIF2 - Channel 2 Timer Interrupt Flag
 *  0b0..Not timed out
 *  0b1..Timed out
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPIT_MSR_TIF2(x)                         (((uint32_t)(((uint32_t)(x)) << LPIT_MSR_TIF2_SHIFT)) & LPIT_MSR_TIF2_MASK)

#define LPIT_MSR_TIF3_MASK                       (0x8U)
#define LPIT_MSR_TIF3_SHIFT                      (3U)
/*! TIF3 - Channel 3 Timer Interrupt Flag
 *  0b0..Not timed out
 *  0b1..Timed out
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPIT_MSR_TIF3(x)                         (((uint32_t)(((uint32_t)(x)) << LPIT_MSR_TIF3_SHIFT)) & LPIT_MSR_TIF3_MASK)
/*! @} */

/*! @name MIER - Module Interrupt Enable */
/*! @{ */

#define LPIT_MIER_TIE0_MASK                      (0x1U)
#define LPIT_MIER_TIE0_SHIFT                     (0U)
/*! TIE0 - Channel 0 Timer Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPIT_MIER_TIE0(x)                        (((uint32_t)(((uint32_t)(x)) << LPIT_MIER_TIE0_SHIFT)) & LPIT_MIER_TIE0_MASK)

#define LPIT_MIER_TIE1_MASK                      (0x2U)
#define LPIT_MIER_TIE1_SHIFT                     (1U)
/*! TIE1 - Channel 1 Timer Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPIT_MIER_TIE1(x)                        (((uint32_t)(((uint32_t)(x)) << LPIT_MIER_TIE1_SHIFT)) & LPIT_MIER_TIE1_MASK)

#define LPIT_MIER_TIE2_MASK                      (0x4U)
#define LPIT_MIER_TIE2_SHIFT                     (2U)
/*! TIE2 - Channel 2 Timer Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPIT_MIER_TIE2(x)                        (((uint32_t)(((uint32_t)(x)) << LPIT_MIER_TIE2_SHIFT)) & LPIT_MIER_TIE2_MASK)

#define LPIT_MIER_TIE3_MASK                      (0x8U)
#define LPIT_MIER_TIE3_SHIFT                     (3U)
/*! TIE3 - Channel 3 Timer Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPIT_MIER_TIE3(x)                        (((uint32_t)(((uint32_t)(x)) << LPIT_MIER_TIE3_SHIFT)) & LPIT_MIER_TIE3_MASK)
/*! @} */

/*! @name SETTEN - Set Timer Enable */
/*! @{ */

#define LPIT_SETTEN_SET_T_EN_0_MASK              (0x1U)
#define LPIT_SETTEN_SET_T_EN_0_SHIFT             (0U)
/*! SET_T_EN_0 - Set Timer 0 Enable
 *  0b0..No effect
 *  0b1..Enables timer channel 0
 */
#define LPIT_SETTEN_SET_T_EN_0(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_SETTEN_SET_T_EN_0_SHIFT)) & LPIT_SETTEN_SET_T_EN_0_MASK)

#define LPIT_SETTEN_SET_T_EN_1_MASK              (0x2U)
#define LPIT_SETTEN_SET_T_EN_1_SHIFT             (1U)
/*! SET_T_EN_1 - Set Timer 1 Enable
 *  0b0..No Effect
 *  0b1..Enables timer channel 1
 */
#define LPIT_SETTEN_SET_T_EN_1(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_SETTEN_SET_T_EN_1_SHIFT)) & LPIT_SETTEN_SET_T_EN_1_MASK)

#define LPIT_SETTEN_SET_T_EN_2_MASK              (0x4U)
#define LPIT_SETTEN_SET_T_EN_2_SHIFT             (2U)
/*! SET_T_EN_2 - Set Timer 2 Enable
 *  0b0..No Effect
 *  0b1..Enables timer channel 2
 */
#define LPIT_SETTEN_SET_T_EN_2(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_SETTEN_SET_T_EN_2_SHIFT)) & LPIT_SETTEN_SET_T_EN_2_MASK)

#define LPIT_SETTEN_SET_T_EN_3_MASK              (0x8U)
#define LPIT_SETTEN_SET_T_EN_3_SHIFT             (3U)
/*! SET_T_EN_3 - Set Timer 3 Enable
 *  0b0..No effect
 *  0b1..Enables timer channel 3
 */
#define LPIT_SETTEN_SET_T_EN_3(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_SETTEN_SET_T_EN_3_SHIFT)) & LPIT_SETTEN_SET_T_EN_3_MASK)
/*! @} */

/*! @name CLRTEN - Clear Timer Enable */
/*! @{ */

#define LPIT_CLRTEN_CLR_T_EN_0_MASK              (0x1U)
#define LPIT_CLRTEN_CLR_T_EN_0_SHIFT             (0U)
/*! CLR_T_EN_0 - Clear Timer 0 Enable
 *  0b0..No action
 *  0b1..Turns TCTRL0[T_EN] = 0 for timer channel 0
 */
#define LPIT_CLRTEN_CLR_T_EN_0(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_CLRTEN_CLR_T_EN_0_SHIFT)) & LPIT_CLRTEN_CLR_T_EN_0_MASK)

#define LPIT_CLRTEN_CLR_T_EN_1_MASK              (0x2U)
#define LPIT_CLRTEN_CLR_T_EN_1_SHIFT             (1U)
/*! CLR_T_EN_1 - Clear Timer 1 Enable
 *  0b0..No action
 *  0b1..Turns TCTRL1[T_EN] = 0 for timer channel 1
 */
#define LPIT_CLRTEN_CLR_T_EN_1(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_CLRTEN_CLR_T_EN_1_SHIFT)) & LPIT_CLRTEN_CLR_T_EN_1_MASK)

#define LPIT_CLRTEN_CLR_T_EN_2_MASK              (0x4U)
#define LPIT_CLRTEN_CLR_T_EN_2_SHIFT             (2U)
/*! CLR_T_EN_2 - Clear Timer 2 Enable
 *  0b0..No action
 *  0b1..Turns TCTRL2[T_EN] = 0 for timer channel 2
 */
#define LPIT_CLRTEN_CLR_T_EN_2(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_CLRTEN_CLR_T_EN_2_SHIFT)) & LPIT_CLRTEN_CLR_T_EN_2_MASK)

#define LPIT_CLRTEN_CLR_T_EN_3_MASK              (0x8U)
#define LPIT_CLRTEN_CLR_T_EN_3_SHIFT             (3U)
/*! CLR_T_EN_3 - Clear Timer 3 Enable
 *  0b0..No action
 *  0b1..Turns TCTRL3[T_EN] = 0 for timer channel 3
 */
#define LPIT_CLRTEN_CLR_T_EN_3(x)                (((uint32_t)(((uint32_t)(x)) << LPIT_CLRTEN_CLR_T_EN_3_SHIFT)) & LPIT_CLRTEN_CLR_T_EN_3_MASK)
/*! @} */

/*! @name TVAL - Timer Value */
/*! @{ */

#define LPIT_TVAL_TMR_VAL_MASK                   (0xFFFFFFFFU)
#define LPIT_TVAL_TMR_VAL_SHIFT                  (0U)
/*! TMR_VAL - Timer Value
 *  0b00000000000000000000000000000000, 0b00000000000000000000000000000001..Invalid load value in Compare mode
 *  0b00000000000000000000000000000010-0b11111111111111111111111111111111..In Compare mode: the value to be loaded; in Capture mode, the value of the timer
 */
#define LPIT_TVAL_TMR_VAL(x)                     (((uint32_t)(((uint32_t)(x)) << LPIT_TVAL_TMR_VAL_SHIFT)) & LPIT_TVAL_TMR_VAL_MASK)
/*! @} */

/* The count of LPIT_TVAL */
#define LPIT_TVAL_COUNT                          (4U)

/*! @name CVAL - Current Timer Value */
/*! @{ */

#define LPIT_CVAL_TMR_CUR_VAL_MASK               (0xFFFFFFFFU)
#define LPIT_CVAL_TMR_CUR_VAL_SHIFT              (0U)
/*! TMR_CUR_VAL - Current Timer Value */
#define LPIT_CVAL_TMR_CUR_VAL(x)                 (((uint32_t)(((uint32_t)(x)) << LPIT_CVAL_TMR_CUR_VAL_SHIFT)) & LPIT_CVAL_TMR_CUR_VAL_MASK)
/*! @} */

/* The count of LPIT_CVAL */
#define LPIT_CVAL_COUNT                          (4U)

/*! @name TCTRL - Timer Control */
/*! @{ */

#define LPIT_TCTRL_T_EN_MASK                     (0x1U)
#define LPIT_TCTRL_T_EN_SHIFT                    (0U)
/*! T_EN - Timer Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPIT_TCTRL_T_EN(x)                       (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_T_EN_SHIFT)) & LPIT_TCTRL_T_EN_MASK)

#define LPIT_TCTRL_CHAIN_MASK                    (0x2U)
#define LPIT_TCTRL_CHAIN_SHIFT                   (1U)
/*! CHAIN - Chain Channel
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define LPIT_TCTRL_CHAIN(x)                      (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_CHAIN_SHIFT)) & LPIT_TCTRL_CHAIN_MASK)

#define LPIT_TCTRL_MODE_MASK                     (0xCU)
#define LPIT_TCTRL_MODE_SHIFT                    (2U)
/*! MODE - Timer Operation Mode
 *  0b00..32-bit periodic counter
 *  0b01..Dual 16-bit periodic counter
 *  0b10..32-bit trigger accumulator
 *  0b11..32-bit trigger input capture
 */
#define LPIT_TCTRL_MODE(x)                       (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_MODE_SHIFT)) & LPIT_TCTRL_MODE_MASK)

#define LPIT_TCTRL_TSOT_MASK                     (0x10000U)
#define LPIT_TCTRL_TSOT_SHIFT                    (16U)
/*! TSOT - Timer Start on Trigger
 *  0b0..Immediately
 *  0b1..When a rising edge is detected
 */
#define LPIT_TCTRL_TSOT(x)                       (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_TSOT_SHIFT)) & LPIT_TCTRL_TSOT_MASK)

#define LPIT_TCTRL_TSOI_MASK                     (0x20000U)
#define LPIT_TCTRL_TSOI_SHIFT                    (17U)
/*! TSOI - Timer Stop on Interrupt
 *  0b0..Does not stop
 *  0b1..Stops
 */
#define LPIT_TCTRL_TSOI(x)                       (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_TSOI_SHIFT)) & LPIT_TCTRL_TSOI_MASK)

#define LPIT_TCTRL_TROT_MASK                     (0x40000U)
#define LPIT_TCTRL_TROT_SHIFT                    (18U)
/*! TROT - Timer Reload on Trigger
 *  0b0..Does not reload
 *  0b1..Reloads
 */
#define LPIT_TCTRL_TROT(x)                       (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_TROT_SHIFT)) & LPIT_TCTRL_TROT_MASK)

#define LPIT_TCTRL_TRG_SRC_MASK                  (0x800000U)
#define LPIT_TCTRL_TRG_SRC_SHIFT                 (23U)
/*! TRG_SRC - Trigger Source
 *  0b0..External
 *  0b1..Internal
 */
#define LPIT_TCTRL_TRG_SRC(x)                    (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_TRG_SRC_SHIFT)) & LPIT_TCTRL_TRG_SRC_MASK)

#define LPIT_TCTRL_TRG_SEL_MASK                  (0xF000000U)
#define LPIT_TCTRL_TRG_SEL_SHIFT                 (24U)
/*! TRG_SEL - Trigger Select
 *  0b0000-0b0011..Timer channel 0-3 trigger source
 *  0b0100-0b1111..Reserved
 */
#define LPIT_TCTRL_TRG_SEL(x)                    (((uint32_t)(((uint32_t)(x)) << LPIT_TCTRL_TRG_SEL_SHIFT)) & LPIT_TCTRL_TRG_SEL_MASK)
/*! @} */

/* The count of LPIT_TCTRL */
#define LPIT_TCTRL_COUNT                         (4U)


/*!
 * @}
 */ /* end of group LPIT_Register_Masks */


/* LPIT - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral LPIT1 base address */
  #define LPIT1_BASE                               (0x542F0000u)
  /** Peripheral LPIT1 base address */
  #define LPIT1_BASE_NS                            (0x442F0000u)
  /** Peripheral LPIT1 base pointer */
  #define LPIT1                                    ((LPIT_Type *)LPIT1_BASE)
  /** Peripheral LPIT1 base pointer */
  #define LPIT1_NS                                 ((LPIT_Type *)LPIT1_BASE_NS)
  /** Peripheral LPIT2 base address */
  #define LPIT2_BASE                               (0x524C0000u)
  /** Peripheral LPIT2 base address */
  #define LPIT2_BASE_NS                            (0x424C0000u)
  /** Peripheral LPIT2 base pointer */
  #define LPIT2                                    ((LPIT_Type *)LPIT2_BASE)
  /** Peripheral LPIT2 base pointer */
  #define LPIT2_NS                                 ((LPIT_Type *)LPIT2_BASE_NS)
  /** Peripheral LPIT3 base address */
  #define LPIT3_BASE                               (0x52CC0000u)
  /** Peripheral LPIT3 base address */
  #define LPIT3_BASE_NS                            (0x42CC0000u)
  /** Peripheral LPIT3 base pointer */
  #define LPIT3                                    ((LPIT_Type *)LPIT3_BASE)
  /** Peripheral LPIT3 base pointer */
  #define LPIT3_NS                                 ((LPIT_Type *)LPIT3_BASE_NS)
  /** Array initializer of LPIT peripheral base addresses */
  #define LPIT_BASE_ADDRS                          { 0u, LPIT1_BASE, LPIT2_BASE, LPIT3_BASE }
  /** Array initializer of LPIT peripheral base pointers */
  #define LPIT_BASE_PTRS                           { (LPIT_Type *)0u, LPIT1, LPIT2, LPIT3 }
  /** Array initializer of LPIT peripheral base addresses */
  #define LPIT_BASE_ADDRS_NS                       { 0u, LPIT1_BASE_NS, LPIT2_BASE_NS, LPIT3_BASE_NS }
  /** Array initializer of LPIT peripheral base pointers */
  #define LPIT_BASE_PTRS_NS                        { (LPIT_Type *)0u, LPIT1_NS, LPIT2_NS, LPIT3_NS }
#else
  /** Peripheral LPIT1 base address */
  #define LPIT1_BASE                               (0x442F0000u)
  /** Peripheral LPIT1 base pointer */
  #define LPIT1                                    ((LPIT_Type *)LPIT1_BASE)
  /** Peripheral LPIT2 base address */
  #define LPIT2_BASE                               (0x424C0000u)
  /** Peripheral LPIT2 base pointer */
  #define LPIT2                                    ((LPIT_Type *)LPIT2_BASE)
  /** Peripheral LPIT3 base address */
  #define LPIT3_BASE                               (0x42CC0000u)
  /** Peripheral LPIT3 base pointer */
  #define LPIT3                                    ((LPIT_Type *)LPIT3_BASE)
  /** Array initializer of LPIT peripheral base addresses */
  #define LPIT_BASE_ADDRS                          { 0u, LPIT1_BASE, LPIT2_BASE, LPIT3_BASE }
  /** Array initializer of LPIT peripheral base pointers */
  #define LPIT_BASE_PTRS                           { (LPIT_Type *)0u, LPIT1, LPIT2, LPIT3 }
#endif
/** Interrupt vectors for the LPIT peripheral type */
#define LPIT_IRQS                                { { NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn }, { LPIT1_IRQn, LPIT1_IRQn, LPIT1_IRQn, LPIT1_IRQn }, { LPIT2_IRQn, LPIT2_IRQn, LPIT2_IRQn, LPIT2_IRQn }, { LPIT3_IRQn, LPIT3_IRQn, LPIT3_IRQn, LPIT3_IRQn } }

/*!
 * @}
 */ /* end of group LPIT_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- LPSPI Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPSPI_Peripheral_Access_Layer LPSPI Peripheral Access Layer
 * @{
 */

/** LPSPI - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
       uint8_t RESERVED_0[8];
  __IO uint32_t CR;                                /**< Control, offset: 0x10 */
  __IO uint32_t SR;                                /**< Status, offset: 0x14 */
  __IO uint32_t IER;                               /**< Interrupt Enable, offset: 0x18 */
  __IO uint32_t DER;                               /**< DMA Enable, offset: 0x1C */
  __IO uint32_t CFGR0;                             /**< Configuration 0, offset: 0x20 */
  __IO uint32_t CFGR1;                             /**< Configuration 1, offset: 0x24 */
       uint8_t RESERVED_1[8];
  __IO uint32_t DMR0;                              /**< Data Match 0, offset: 0x30 */
  __IO uint32_t DMR1;                              /**< Data Match 1, offset: 0x34 */
       uint8_t RESERVED_2[8];
  __IO uint32_t CCR;                               /**< Clock Configuration, offset: 0x40 */
  __IO uint32_t CCR1;                              /**< Clock Configuration 1, offset: 0x44 */
       uint8_t RESERVED_3[16];
  __IO uint32_t FCR;                               /**< FIFO Control, offset: 0x58 */
  __I  uint32_t FSR;                               /**< FIFO Status, offset: 0x5C */
  __IO uint32_t TCR;                               /**< Transmit Command, offset: 0x60 */
  __O  uint32_t TDR;                               /**< Transmit Data, offset: 0x64 */
       uint8_t RESERVED_4[8];
  __I  uint32_t RSR;                               /**< Receive Status, offset: 0x70 */
  __I  uint32_t RDR;                               /**< Receive Data, offset: 0x74 */
  __I  uint32_t RDROR;                             /**< Receive Data Read Only, offset: 0x78 */
       uint8_t RESERVED_5[896];
  __O  uint32_t TCBR;                              /**< Transmit Command Burst, offset: 0x3FC */
  __O  uint32_t TDBR[128];                         /**< Transmit Data Burst, array offset: 0x400, array step: 0x4 */
  __I  uint32_t RDBR[128];                         /**< Receive Data Burst, array offset: 0x600, array step: 0x4 */
} LPSPI_Type;

/* ----------------------------------------------------------------------------
   -- LPSPI Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPSPI_Register_Masks LPSPI Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define LPSPI_VERID_FEATURE_MASK                 (0xFFFFU)
#define LPSPI_VERID_FEATURE_SHIFT                (0U)
/*! FEATURE - Module Identification Number
 *  0b0000000000000100..Standard feature set supporting a 32-bit shift register.
 *  *..
 */
#define LPSPI_VERID_FEATURE(x)                   (((uint32_t)(((uint32_t)(x)) << LPSPI_VERID_FEATURE_SHIFT)) & LPSPI_VERID_FEATURE_MASK)

#define LPSPI_VERID_MINOR_MASK                   (0xFF0000U)
#define LPSPI_VERID_MINOR_SHIFT                  (16U)
/*! MINOR - Minor Version Number */
#define LPSPI_VERID_MINOR(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_VERID_MINOR_SHIFT)) & LPSPI_VERID_MINOR_MASK)

#define LPSPI_VERID_MAJOR_MASK                   (0xFF000000U)
#define LPSPI_VERID_MAJOR_SHIFT                  (24U)
/*! MAJOR - Major Version Number */
#define LPSPI_VERID_MAJOR(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_VERID_MAJOR_SHIFT)) & LPSPI_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define LPSPI_PARAM_TXFIFO_MASK                  (0xFFU)
#define LPSPI_PARAM_TXFIFO_SHIFT                 (0U)
/*! TXFIFO - Transmit FIFO Size */
#define LPSPI_PARAM_TXFIFO(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_PARAM_TXFIFO_SHIFT)) & LPSPI_PARAM_TXFIFO_MASK)

#define LPSPI_PARAM_RXFIFO_MASK                  (0xFF00U)
#define LPSPI_PARAM_RXFIFO_SHIFT                 (8U)
/*! RXFIFO - Receive FIFO Size */
#define LPSPI_PARAM_RXFIFO(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_PARAM_RXFIFO_SHIFT)) & LPSPI_PARAM_RXFIFO_MASK)

#define LPSPI_PARAM_PCSNUM_MASK                  (0xFF0000U)
#define LPSPI_PARAM_PCSNUM_SHIFT                 (16U)
/*! PCSNUM - PCS Number */
#define LPSPI_PARAM_PCSNUM(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_PARAM_PCSNUM_SHIFT)) & LPSPI_PARAM_PCSNUM_MASK)
/*! @} */

/*! @name CR - Control */
/*! @{ */

#define LPSPI_CR_MEN_MASK                        (0x1U)
#define LPSPI_CR_MEN_SHIFT                       (0U)
/*! MEN - Module Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_CR_MEN(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_CR_MEN_SHIFT)) & LPSPI_CR_MEN_MASK)

#define LPSPI_CR_RST_MASK                        (0x2U)
#define LPSPI_CR_RST_SHIFT                       (1U)
/*! RST - Software Reset
 *  0b0..Not reset
 *  0b1..Reset
 */
#define LPSPI_CR_RST(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_CR_RST_SHIFT)) & LPSPI_CR_RST_MASK)

#define LPSPI_CR_DOZEN_MASK                      (0x4U)
#define LPSPI_CR_DOZEN_SHIFT                     (2U)
/*! DOZEN - Doze Mode Enable
 *  0b0..Enable
 *  0b1..Disable
 */
#define LPSPI_CR_DOZEN(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_CR_DOZEN_SHIFT)) & LPSPI_CR_DOZEN_MASK)

#define LPSPI_CR_DBGEN_MASK                      (0x8U)
#define LPSPI_CR_DBGEN_SHIFT                     (3U)
/*! DBGEN - Debug Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_CR_DBGEN(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_CR_DBGEN_SHIFT)) & LPSPI_CR_DBGEN_MASK)

#define LPSPI_CR_RTF_MASK                        (0x100U)
#define LPSPI_CR_RTF_SHIFT                       (8U)
/*! RTF - Reset Transmit FIFO
 *  0b0..No effect
 *  0b1..Reset
 */
#define LPSPI_CR_RTF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_CR_RTF_SHIFT)) & LPSPI_CR_RTF_MASK)

#define LPSPI_CR_RRF_MASK                        (0x200U)
#define LPSPI_CR_RRF_SHIFT                       (9U)
/*! RRF - Reset Receive FIFO
 *  0b0..No effect
 *  0b1..Reset
 */
#define LPSPI_CR_RRF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_CR_RRF_SHIFT)) & LPSPI_CR_RRF_MASK)
/*! @} */

/*! @name SR - Status */
/*! @{ */

#define LPSPI_SR_TDF_MASK                        (0x1U)
#define LPSPI_SR_TDF_SHIFT                       (0U)
/*! TDF - Transmit Data Flag
 *  0b0..Transmit data not requested
 *  0b1..Transmit data is requested
 */
#define LPSPI_SR_TDF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_TDF_SHIFT)) & LPSPI_SR_TDF_MASK)

#define LPSPI_SR_RDF_MASK                        (0x2U)
#define LPSPI_SR_RDF_SHIFT                       (1U)
/*! RDF - Receive Data Flag
 *  0b0..Receive data not ready
 *  0b1..Receive data is ready
 */
#define LPSPI_SR_RDF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_RDF_SHIFT)) & LPSPI_SR_RDF_MASK)

#define LPSPI_SR_WCF_MASK                        (0x100U)
#define LPSPI_SR_WCF_SHIFT                       (8U)
/*! WCF - Word Complete Flag
 *  0b0..Not complete
 *  0b1..Complete
 */
#define LPSPI_SR_WCF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_WCF_SHIFT)) & LPSPI_SR_WCF_MASK)

#define LPSPI_SR_FCF_MASK                        (0x200U)
#define LPSPI_SR_FCF_SHIFT                       (9U)
/*! FCF - Frame Complete Flag
 *  0b0..Not complete
 *  0b1..Complete
 */
#define LPSPI_SR_FCF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_FCF_SHIFT)) & LPSPI_SR_FCF_MASK)

#define LPSPI_SR_TCF_MASK                        (0x400U)
#define LPSPI_SR_TCF_SHIFT                       (10U)
/*! TCF - Transfer Complete Flag
 *  0b0..Not complete
 *  0b1..Complete
 */
#define LPSPI_SR_TCF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_TCF_SHIFT)) & LPSPI_SR_TCF_MASK)

#define LPSPI_SR_TEF_MASK                        (0x800U)
#define LPSPI_SR_TEF_SHIFT                       (11U)
/*! TEF - Transmit Error Flag
 *  0b0..No underrun
 *  0b1..Underrun
 */
#define LPSPI_SR_TEF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_TEF_SHIFT)) & LPSPI_SR_TEF_MASK)

#define LPSPI_SR_REF_MASK                        (0x1000U)
#define LPSPI_SR_REF_SHIFT                       (12U)
/*! REF - Receive Error Flag
 *  0b0..No overflow
 *  0b1..Overflow
 */
#define LPSPI_SR_REF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_REF_SHIFT)) & LPSPI_SR_REF_MASK)

#define LPSPI_SR_DMF_MASK                        (0x2000U)
#define LPSPI_SR_DMF_SHIFT                       (13U)
/*! DMF - Data Match Flag
 *  0b0..No match
 *  0b1..Match
 */
#define LPSPI_SR_DMF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_DMF_SHIFT)) & LPSPI_SR_DMF_MASK)

#define LPSPI_SR_MBF_MASK                        (0x1000000U)
#define LPSPI_SR_MBF_SHIFT                       (24U)
/*! MBF - Module Busy Flag
 *  0b0..LPSPI is idle
 *  0b1..LPSPI is busy
 */
#define LPSPI_SR_MBF(x)                          (((uint32_t)(((uint32_t)(x)) << LPSPI_SR_MBF_SHIFT)) & LPSPI_SR_MBF_MASK)
/*! @} */

/*! @name IER - Interrupt Enable */
/*! @{ */

#define LPSPI_IER_TDIE_MASK                      (0x1U)
#define LPSPI_IER_TDIE_SHIFT                     (0U)
/*! TDIE - Transmit Data Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_TDIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_TDIE_SHIFT)) & LPSPI_IER_TDIE_MASK)

#define LPSPI_IER_RDIE_MASK                      (0x2U)
#define LPSPI_IER_RDIE_SHIFT                     (1U)
/*! RDIE - Receive Data Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_RDIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_RDIE_SHIFT)) & LPSPI_IER_RDIE_MASK)

#define LPSPI_IER_WCIE_MASK                      (0x100U)
#define LPSPI_IER_WCIE_SHIFT                     (8U)
/*! WCIE - Word Complete Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_WCIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_WCIE_SHIFT)) & LPSPI_IER_WCIE_MASK)

#define LPSPI_IER_FCIE_MASK                      (0x200U)
#define LPSPI_IER_FCIE_SHIFT                     (9U)
/*! FCIE - Frame Complete Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_FCIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_FCIE_SHIFT)) & LPSPI_IER_FCIE_MASK)

#define LPSPI_IER_TCIE_MASK                      (0x400U)
#define LPSPI_IER_TCIE_SHIFT                     (10U)
/*! TCIE - Transfer Complete Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_TCIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_TCIE_SHIFT)) & LPSPI_IER_TCIE_MASK)

#define LPSPI_IER_TEIE_MASK                      (0x800U)
#define LPSPI_IER_TEIE_SHIFT                     (11U)
/*! TEIE - Transmit Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_TEIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_TEIE_SHIFT)) & LPSPI_IER_TEIE_MASK)

#define LPSPI_IER_REIE_MASK                      (0x1000U)
#define LPSPI_IER_REIE_SHIFT                     (12U)
/*! REIE - Receive Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_REIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_REIE_SHIFT)) & LPSPI_IER_REIE_MASK)

#define LPSPI_IER_DMIE_MASK                      (0x2000U)
#define LPSPI_IER_DMIE_SHIFT                     (13U)
/*! DMIE - Data Match Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_IER_DMIE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_IER_DMIE_SHIFT)) & LPSPI_IER_DMIE_MASK)
/*! @} */

/*! @name DER - DMA Enable */
/*! @{ */

#define LPSPI_DER_TDDE_MASK                      (0x1U)
#define LPSPI_DER_TDDE_SHIFT                     (0U)
/*! TDDE - Transmit Data DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_DER_TDDE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_DER_TDDE_SHIFT)) & LPSPI_DER_TDDE_MASK)

#define LPSPI_DER_RDDE_MASK                      (0x2U)
#define LPSPI_DER_RDDE_SHIFT                     (1U)
/*! RDDE - Receive Data DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_DER_RDDE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_DER_RDDE_SHIFT)) & LPSPI_DER_RDDE_MASK)

#define LPSPI_DER_FCDE_MASK                      (0x200U)
#define LPSPI_DER_FCDE_SHIFT                     (9U)
/*! FCDE - Frame Complete DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_DER_FCDE(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_DER_FCDE_SHIFT)) & LPSPI_DER_FCDE_MASK)
/*! @} */

/*! @name CFGR0 - Configuration 0 */
/*! @{ */

#define LPSPI_CFGR0_HREN_MASK                    (0x1U)
#define LPSPI_CFGR0_HREN_SHIFT                   (0U)
/*! HREN - Host Request Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_CFGR0_HREN(x)                      (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR0_HREN_SHIFT)) & LPSPI_CFGR0_HREN_MASK)

#define LPSPI_CFGR0_HRPOL_MASK                   (0x2U)
#define LPSPI_CFGR0_HRPOL_SHIFT                  (1U)
/*! HRPOL - Host Request Polarity
 *  0b0..Active high
 *  0b1..Active low
 */
#define LPSPI_CFGR0_HRPOL(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR0_HRPOL_SHIFT)) & LPSPI_CFGR0_HRPOL_MASK)

#define LPSPI_CFGR0_HRSEL_MASK                   (0x4U)
#define LPSPI_CFGR0_HRSEL_SHIFT                  (2U)
/*! HRSEL - Host Request Select
 *  0b0..HREQ pin
 *  0b1..Input trigger
 */
#define LPSPI_CFGR0_HRSEL(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR0_HRSEL_SHIFT)) & LPSPI_CFGR0_HRSEL_MASK)

#define LPSPI_CFGR0_HRDIR_MASK                   (0x8U)
#define LPSPI_CFGR0_HRDIR_SHIFT                  (3U)
/*! HRDIR - Host Request Direction
 *  0b0..Input
 *  0b1..Output
 */
#define LPSPI_CFGR0_HRDIR(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR0_HRDIR_SHIFT)) & LPSPI_CFGR0_HRDIR_MASK)

#define LPSPI_CFGR0_CIRFIFO_MASK                 (0x100U)
#define LPSPI_CFGR0_CIRFIFO_SHIFT                (8U)
/*! CIRFIFO - Circular FIFO Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_CFGR0_CIRFIFO(x)                   (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR0_CIRFIFO_SHIFT)) & LPSPI_CFGR0_CIRFIFO_MASK)

#define LPSPI_CFGR0_RDMO_MASK                    (0x200U)
#define LPSPI_CFGR0_RDMO_SHIFT                   (9U)
/*! RDMO - Receive Data Match Only
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_CFGR0_RDMO(x)                      (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR0_RDMO_SHIFT)) & LPSPI_CFGR0_RDMO_MASK)
/*! @} */

/*! @name CFGR1 - Configuration 1 */
/*! @{ */

#define LPSPI_CFGR1_MASTER_MASK                  (0x1U)
#define LPSPI_CFGR1_MASTER_SHIFT                 (0U)
/*! MASTER - Master Mode
 *  0b0..Slave mode
 *  0b1..Master mode
 */
#define LPSPI_CFGR1_MASTER(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_MASTER_SHIFT)) & LPSPI_CFGR1_MASTER_MASK)

#define LPSPI_CFGR1_SAMPLE_MASK                  (0x2U)
#define LPSPI_CFGR1_SAMPLE_SHIFT                 (1U)
/*! SAMPLE - Sample Point
 *  0b0..SCK edge
 *  0b1..Delayed SCK edge
 */
#define LPSPI_CFGR1_SAMPLE(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_SAMPLE_SHIFT)) & LPSPI_CFGR1_SAMPLE_MASK)

#define LPSPI_CFGR1_AUTOPCS_MASK                 (0x4U)
#define LPSPI_CFGR1_AUTOPCS_SHIFT                (2U)
/*! AUTOPCS - Automatic PCS
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_CFGR1_AUTOPCS(x)                   (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_AUTOPCS_SHIFT)) & LPSPI_CFGR1_AUTOPCS_MASK)

#define LPSPI_CFGR1_NOSTALL_MASK                 (0x8U)
#define LPSPI_CFGR1_NOSTALL_SHIFT                (3U)
/*! NOSTALL - No Stall
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPSPI_CFGR1_NOSTALL(x)                   (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_NOSTALL_SHIFT)) & LPSPI_CFGR1_NOSTALL_MASK)

#define LPSPI_CFGR1_PARTIAL_MASK                 (0x10U)
#define LPSPI_CFGR1_PARTIAL_SHIFT                (4U)
/*! PARTIAL - Partial Enable
 *  0b0..Discard
 *  0b1..Store
 */
#define LPSPI_CFGR1_PARTIAL(x)                   (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_PARTIAL_SHIFT)) & LPSPI_CFGR1_PARTIAL_MASK)

#define LPSPI_CFGR1_PCSPOL_MASK                  (0xF00U)
#define LPSPI_CFGR1_PCSPOL_SHIFT                 (8U)
/*! PCSPOL - Peripheral Chip Select Polarity */
#define LPSPI_CFGR1_PCSPOL(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_PCSPOL_SHIFT)) & LPSPI_CFGR1_PCSPOL_MASK)

#define LPSPI_CFGR1_MATCFG_MASK                  (0x70000U)
#define LPSPI_CFGR1_MATCFG_SHIFT                 (16U)
/*! MATCFG - Match Configuration
 *  0b000..Match is disabled
 *  0b001..
 *  0b010..Match first data word with compare word
 *  0b011..Match any data word with compare word
 *  0b100..Sequential match, first data word
 *  0b101..Sequential match, any data word
 *  0b110..Match first data word (masked) with compare word (masked)
 *  0b111..Match any data word (masked) with compare word (masked)
 */
#define LPSPI_CFGR1_MATCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_MATCFG_SHIFT)) & LPSPI_CFGR1_MATCFG_MASK)

#define LPSPI_CFGR1_PINCFG_MASK                  (0x3000000U)
#define LPSPI_CFGR1_PINCFG_SHIFT                 (24U)
/*! PINCFG - Pin Configuration
 *  0b00..SIN is used for input data; SOUT is used for output data.
 *  0b01..SIN is used for both input and output data. Only half-duplex serial transfers are supported.
 *  0b10..SOUT is used for both input and output data. Only half-duplex serial transfers are supported.
 *  0b11..SOUT is used for input data; SIN is used for output data.
 */
#define LPSPI_CFGR1_PINCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_PINCFG_SHIFT)) & LPSPI_CFGR1_PINCFG_MASK)

#define LPSPI_CFGR1_OUTCFG_MASK                  (0x4000000U)
#define LPSPI_CFGR1_OUTCFG_SHIFT                 (26U)
/*! OUTCFG - Output Configuration
 *  0b0..Output data retains last value.
 *  0b1..Output data is 3-stated.
 */
#define LPSPI_CFGR1_OUTCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_OUTCFG_SHIFT)) & LPSPI_CFGR1_OUTCFG_MASK)

#define LPSPI_CFGR1_PCSCFG_MASK                  (0x8000000U)
#define LPSPI_CFGR1_PCSCFG_SHIFT                 (27U)
/*! PCSCFG - Peripheral Chip Select Configuration
 *  0b0..PCS[3:2] are configured for chip select function
 *  0b1..PCS[3:2] are configured for half-duplex 4-bit transfers (PCS[3:2] = DATA[3:2])
 */
#define LPSPI_CFGR1_PCSCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_CFGR1_PCSCFG_SHIFT)) & LPSPI_CFGR1_PCSCFG_MASK)
/*! @} */

/*! @name DMR0 - Data Match 0 */
/*! @{ */

#define LPSPI_DMR0_MATCH0_MASK                   (0xFFFFFFFFU)
#define LPSPI_DMR0_MATCH0_SHIFT                  (0U)
/*! MATCH0 - Match 0 Value */
#define LPSPI_DMR0_MATCH0(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_DMR0_MATCH0_SHIFT)) & LPSPI_DMR0_MATCH0_MASK)
/*! @} */

/*! @name DMR1 - Data Match 1 */
/*! @{ */

#define LPSPI_DMR1_MATCH1_MASK                   (0xFFFFFFFFU)
#define LPSPI_DMR1_MATCH1_SHIFT                  (0U)
/*! MATCH1 - Match 1 Value */
#define LPSPI_DMR1_MATCH1(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_DMR1_MATCH1_SHIFT)) & LPSPI_DMR1_MATCH1_MASK)
/*! @} */

/*! @name CCR - Clock Configuration */
/*! @{ */

#define LPSPI_CCR_SCKDIV_MASK                    (0xFFU)
#define LPSPI_CCR_SCKDIV_SHIFT                   (0U)
/*! SCKDIV - SCK Divider */
#define LPSPI_CCR_SCKDIV(x)                      (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR_SCKDIV_SHIFT)) & LPSPI_CCR_SCKDIV_MASK)

#define LPSPI_CCR_DBT_MASK                       (0xFF00U)
#define LPSPI_CCR_DBT_SHIFT                      (8U)
/*! DBT - Delay Between Transfers */
#define LPSPI_CCR_DBT(x)                         (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR_DBT_SHIFT)) & LPSPI_CCR_DBT_MASK)

#define LPSPI_CCR_PCSSCK_MASK                    (0xFF0000U)
#define LPSPI_CCR_PCSSCK_SHIFT                   (16U)
/*! PCSSCK - PCS-to-SCK Delay */
#define LPSPI_CCR_PCSSCK(x)                      (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR_PCSSCK_SHIFT)) & LPSPI_CCR_PCSSCK_MASK)

#define LPSPI_CCR_SCKPCS_MASK                    (0xFF000000U)
#define LPSPI_CCR_SCKPCS_SHIFT                   (24U)
/*! SCKPCS - SCK-to-PCS Delay */
#define LPSPI_CCR_SCKPCS(x)                      (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR_SCKPCS_SHIFT)) & LPSPI_CCR_SCKPCS_MASK)
/*! @} */

/*! @name CCR1 - Clock Configuration 1 */
/*! @{ */

#define LPSPI_CCR1_SCKSET_MASK                   (0xFFU)
#define LPSPI_CCR1_SCKSET_SHIFT                  (0U)
/*! SCKSET - SCK Setup */
#define LPSPI_CCR1_SCKSET(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR1_SCKSET_SHIFT)) & LPSPI_CCR1_SCKSET_MASK)

#define LPSPI_CCR1_SCKHLD_MASK                   (0xFF00U)
#define LPSPI_CCR1_SCKHLD_SHIFT                  (8U)
/*! SCKHLD - SCK Hold */
#define LPSPI_CCR1_SCKHLD(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR1_SCKHLD_SHIFT)) & LPSPI_CCR1_SCKHLD_MASK)

#define LPSPI_CCR1_PCSPCS_MASK                   (0xFF0000U)
#define LPSPI_CCR1_PCSPCS_SHIFT                  (16U)
/*! PCSPCS - PCS to PCS delay */
#define LPSPI_CCR1_PCSPCS(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR1_PCSPCS_SHIFT)) & LPSPI_CCR1_PCSPCS_MASK)

#define LPSPI_CCR1_SCKSCK_MASK                   (0xFF000000U)
#define LPSPI_CCR1_SCKSCK_SHIFT                  (24U)
/*! SCKSCK - SCK Inter-Frame Delay */
#define LPSPI_CCR1_SCKSCK(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_CCR1_SCKSCK_SHIFT)) & LPSPI_CCR1_SCKSCK_MASK)
/*! @} */

/*! @name FCR - FIFO Control */
/*! @{ */

#define LPSPI_FCR_TXWATER_MASK                   (0xFU)
#define LPSPI_FCR_TXWATER_SHIFT                  (0U)
/*! TXWATER - Transmit FIFO Watermark */
#define LPSPI_FCR_TXWATER(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_FCR_TXWATER_SHIFT)) & LPSPI_FCR_TXWATER_MASK)

#define LPSPI_FCR_RXWATER_MASK                   (0xF0000U)
#define LPSPI_FCR_RXWATER_SHIFT                  (16U)
/*! RXWATER - Receive FIFO Watermark */
#define LPSPI_FCR_RXWATER(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_FCR_RXWATER_SHIFT)) & LPSPI_FCR_RXWATER_MASK)
/*! @} */

/*! @name FSR - FIFO Status */
/*! @{ */

#define LPSPI_FSR_TXCOUNT_MASK                   (0x1FU)
#define LPSPI_FSR_TXCOUNT_SHIFT                  (0U)
/*! TXCOUNT - Transmit FIFO Count */
#define LPSPI_FSR_TXCOUNT(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_FSR_TXCOUNT_SHIFT)) & LPSPI_FSR_TXCOUNT_MASK)

#define LPSPI_FSR_RXCOUNT_MASK                   (0x1F0000U)
#define LPSPI_FSR_RXCOUNT_SHIFT                  (16U)
/*! RXCOUNT - Receive FIFO Count */
#define LPSPI_FSR_RXCOUNT(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_FSR_RXCOUNT_SHIFT)) & LPSPI_FSR_RXCOUNT_MASK)
/*! @} */

/*! @name TCR - Transmit Command */
/*! @{ */

#define LPSPI_TCR_FRAMESZ_MASK                   (0xFFFU)
#define LPSPI_TCR_FRAMESZ_SHIFT                  (0U)
/*! FRAMESZ - Frame Size */
#define LPSPI_TCR_FRAMESZ(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_FRAMESZ_SHIFT)) & LPSPI_TCR_FRAMESZ_MASK)

#define LPSPI_TCR_WIDTH_MASK                     (0x30000U)
#define LPSPI_TCR_WIDTH_SHIFT                    (16U)
/*! WIDTH - Transfer Width
 *  0b00..1-bit transfer
 *  0b01..2-bit transfer
 *  0b10..4-bit transfer
 *  0b11..Reserved
 */
#define LPSPI_TCR_WIDTH(x)                       (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_WIDTH_SHIFT)) & LPSPI_TCR_WIDTH_MASK)

#define LPSPI_TCR_TXMSK_MASK                     (0x40000U)
#define LPSPI_TCR_TXMSK_SHIFT                    (18U)
/*! TXMSK - Transmit Data Mask
 *  0b0..Normal transfer
 *  0b1..Mask transmit data
 */
#define LPSPI_TCR_TXMSK(x)                       (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_TXMSK_SHIFT)) & LPSPI_TCR_TXMSK_MASK)

#define LPSPI_TCR_RXMSK_MASK                     (0x80000U)
#define LPSPI_TCR_RXMSK_SHIFT                    (19U)
/*! RXMSK - Receive Data Mask
 *  0b0..Normal transfer
 *  0b1..Receive data is masked
 */
#define LPSPI_TCR_RXMSK(x)                       (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_RXMSK_SHIFT)) & LPSPI_TCR_RXMSK_MASK)

#define LPSPI_TCR_CONTC_MASK                     (0x100000U)
#define LPSPI_TCR_CONTC_SHIFT                    (20U)
/*! CONTC - Continuing Command
 *  0b0..Command word for start of new transfer
 *  0b1..Command word for continuing transfer
 */
#define LPSPI_TCR_CONTC(x)                       (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_CONTC_SHIFT)) & LPSPI_TCR_CONTC_MASK)

#define LPSPI_TCR_CONT_MASK                      (0x200000U)
#define LPSPI_TCR_CONT_SHIFT                     (21U)
/*! CONT - Continuous Transfer
 *  0b0..Continuous transfer is disabled
 *  0b1..Continuous transfer is enabled
 */
#define LPSPI_TCR_CONT(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_CONT_SHIFT)) & LPSPI_TCR_CONT_MASK)

#define LPSPI_TCR_BYSW_MASK                      (0x400000U)
#define LPSPI_TCR_BYSW_SHIFT                     (22U)
/*! BYSW - Byte Swap
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define LPSPI_TCR_BYSW(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_BYSW_SHIFT)) & LPSPI_TCR_BYSW_MASK)

#define LPSPI_TCR_LSBF_MASK                      (0x800000U)
#define LPSPI_TCR_LSBF_SHIFT                     (23U)
/*! LSBF - LSB First
 *  0b0..Data is transferred MSB first
 *  0b1..Data is transferred LSB first
 */
#define LPSPI_TCR_LSBF(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_LSBF_SHIFT)) & LPSPI_TCR_LSBF_MASK)

#define LPSPI_TCR_PCS_MASK                       (0x3000000U)
#define LPSPI_TCR_PCS_SHIFT                      (24U)
/*! PCS - Peripheral Chip Select
 *  0b00..Transfer using PCS[0]
 *  0b01..Transfer using PCS[1]
 *  0b10..Transfer using PCS[2]
 *  0b11..Transfer using PCS[3]
 */
#define LPSPI_TCR_PCS(x)                         (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_PCS_SHIFT)) & LPSPI_TCR_PCS_MASK)

#define LPSPI_TCR_PRESCALE_MASK                  (0x38000000U)
#define LPSPI_TCR_PRESCALE_SHIFT                 (27U)
/*! PRESCALE - Prescaler Value
 *  0b000..Divide by 1
 *  0b001..Divide by 2
 *  0b010..Divide by 4
 *  0b011..Divide by 8
 *  0b100..Divide by 16
 *  0b101..Divide by 32
 *  0b110..Divide by 64
 *  0b111..Divide by 128
 */
#define LPSPI_TCR_PRESCALE(x)                    (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_PRESCALE_SHIFT)) & LPSPI_TCR_PRESCALE_MASK)

#define LPSPI_TCR_CPHA_MASK                      (0x40000000U)
#define LPSPI_TCR_CPHA_SHIFT                     (30U)
/*! CPHA - Clock Phase
 *  0b0..Captured
 *  0b1..Changed
 */
#define LPSPI_TCR_CPHA(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_CPHA_SHIFT)) & LPSPI_TCR_CPHA_MASK)

#define LPSPI_TCR_CPOL_MASK                      (0x80000000U)
#define LPSPI_TCR_CPOL_SHIFT                     (31U)
/*! CPOL - Clock Polarity
 *  0b0..Inactive low
 *  0b1..Inactive high
 */
#define LPSPI_TCR_CPOL(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_TCR_CPOL_SHIFT)) & LPSPI_TCR_CPOL_MASK)
/*! @} */

/*! @name TDR - Transmit Data */
/*! @{ */

#define LPSPI_TDR_DATA_MASK                      (0xFFFFFFFFU)
#define LPSPI_TDR_DATA_SHIFT                     (0U)
/*! DATA - Transmit Data */
#define LPSPI_TDR_DATA(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_TDR_DATA_SHIFT)) & LPSPI_TDR_DATA_MASK)
/*! @} */

/*! @name RSR - Receive Status */
/*! @{ */

#define LPSPI_RSR_SOF_MASK                       (0x1U)
#define LPSPI_RSR_SOF_SHIFT                      (0U)
/*! SOF - Start Of Frame
 *  0b0..Subsequent data word
 *  0b1..First data word
 */
#define LPSPI_RSR_SOF(x)                         (((uint32_t)(((uint32_t)(x)) << LPSPI_RSR_SOF_SHIFT)) & LPSPI_RSR_SOF_MASK)

#define LPSPI_RSR_RXEMPTY_MASK                   (0x2U)
#define LPSPI_RSR_RXEMPTY_SHIFT                  (1U)
/*! RXEMPTY - RX FIFO Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define LPSPI_RSR_RXEMPTY(x)                     (((uint32_t)(((uint32_t)(x)) << LPSPI_RSR_RXEMPTY_SHIFT)) & LPSPI_RSR_RXEMPTY_MASK)
/*! @} */

/*! @name RDR - Receive Data */
/*! @{ */

#define LPSPI_RDR_DATA_MASK                      (0xFFFFFFFFU)
#define LPSPI_RDR_DATA_SHIFT                     (0U)
/*! DATA - Receive Data */
#define LPSPI_RDR_DATA(x)                        (((uint32_t)(((uint32_t)(x)) << LPSPI_RDR_DATA_SHIFT)) & LPSPI_RDR_DATA_MASK)
/*! @} */

/*! @name RDROR - Receive Data Read Only */
/*! @{ */

#define LPSPI_RDROR_DATA_MASK                    (0xFFFFFFFFU)
#define LPSPI_RDROR_DATA_SHIFT                   (0U)
/*! DATA - Receive Data */
#define LPSPI_RDROR_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << LPSPI_RDROR_DATA_SHIFT)) & LPSPI_RDROR_DATA_MASK)
/*! @} */

/*! @name TCBR - Transmit Command Burst */
/*! @{ */

#define LPSPI_TCBR_DATA_MASK                     (0xFFFFFFFFU)
#define LPSPI_TCBR_DATA_SHIFT                    (0U)
/*! DATA - Command Data */
#define LPSPI_TCBR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << LPSPI_TCBR_DATA_SHIFT)) & LPSPI_TCBR_DATA_MASK)
/*! @} */

/*! @name TDBR - Transmit Data Burst */
/*! @{ */

#define LPSPI_TDBR_DATA_MASK                     (0xFFFFFFFFU)
#define LPSPI_TDBR_DATA_SHIFT                    (0U)
/*! DATA - Data */
#define LPSPI_TDBR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << LPSPI_TDBR_DATA_SHIFT)) & LPSPI_TDBR_DATA_MASK)
/*! @} */

/* The count of LPSPI_TDBR */
#define LPSPI_TDBR_COUNT                         (128U)

/*! @name RDBR - Receive Data Burst */
/*! @{ */

#define LPSPI_RDBR_DATA_MASK                     (0xFFFFFFFFU)
#define LPSPI_RDBR_DATA_SHIFT                    (0U)
/*! DATA - Data */
#define LPSPI_RDBR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << LPSPI_RDBR_DATA_SHIFT)) & LPSPI_RDBR_DATA_MASK)
/*! @} */

/* The count of LPSPI_RDBR */
#define LPSPI_RDBR_COUNT                         (128U)


/*!
 * @}
 */ /* end of group LPSPI_Register_Masks */


/* LPSPI - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral LPSPI1 base address */
  #define LPSPI1_BASE                              (0x54360000u)
  /** Peripheral LPSPI1 base address */
  #define LPSPI1_BASE_NS                           (0x44360000u)
  /** Peripheral LPSPI1 base pointer */
  #define LPSPI1                                   ((LPSPI_Type *)LPSPI1_BASE)
  /** Peripheral LPSPI1 base pointer */
  #define LPSPI1_NS                                ((LPSPI_Type *)LPSPI1_BASE_NS)
  /** Peripheral LPSPI2 base address */
  #define LPSPI2_BASE                              (0x54370000u)
  /** Peripheral LPSPI2 base address */
  #define LPSPI2_BASE_NS                           (0x44370000u)
  /** Peripheral LPSPI2 base pointer */
  #define LPSPI2                                   ((LPSPI_Type *)LPSPI2_BASE)
  /** Peripheral LPSPI2 base pointer */
  #define LPSPI2_NS                                ((LPSPI_Type *)LPSPI2_BASE_NS)
  /** Peripheral LPSPI3 base address */
  #define LPSPI3_BASE                              (0x52550000u)
  /** Peripheral LPSPI3 base address */
  #define LPSPI3_BASE_NS                           (0x42550000u)
  /** Peripheral LPSPI3 base pointer */
  #define LPSPI3                                   ((LPSPI_Type *)LPSPI3_BASE)
  /** Peripheral LPSPI3 base pointer */
  #define LPSPI3_NS                                ((LPSPI_Type *)LPSPI3_BASE_NS)
  /** Peripheral LPSPI4 base address */
  #define LPSPI4_BASE                              (0x52560000u)
  /** Peripheral LPSPI4 base address */
  #define LPSPI4_BASE_NS                           (0x42560000u)
  /** Peripheral LPSPI4 base pointer */
  #define LPSPI4                                   ((LPSPI_Type *)LPSPI4_BASE)
  /** Peripheral LPSPI4 base pointer */
  #define LPSPI4_NS                                ((LPSPI_Type *)LPSPI4_BASE_NS)
  /** Array initializer of LPSPI peripheral base addresses */
  #define LPSPI_BASE_ADDRS                         { 0u, LPSPI1_BASE, LPSPI2_BASE, LPSPI3_BASE, LPSPI4_BASE }
  /** Array initializer of LPSPI peripheral base pointers */
  #define LPSPI_BASE_PTRS                          { (LPSPI_Type *)0u, LPSPI1, LPSPI2, LPSPI3, LPSPI4 }
  /** Array initializer of LPSPI peripheral base addresses */
  #define LPSPI_BASE_ADDRS_NS                      { 0u, LPSPI1_BASE_NS, LPSPI2_BASE_NS, LPSPI3_BASE_NS, LPSPI4_BASE_NS }
  /** Array initializer of LPSPI peripheral base pointers */
  #define LPSPI_BASE_PTRS_NS                       { (LPSPI_Type *)0u, LPSPI1_NS, LPSPI2_NS, LPSPI3_NS, LPSPI4_NS }
#else
  /** Peripheral LPSPI1 base address */
  #define LPSPI1_BASE                              (0x44360000u)
  /** Peripheral LPSPI1 base pointer */
  #define LPSPI1                                   ((LPSPI_Type *)LPSPI1_BASE)
  /** Peripheral LPSPI2 base address */
  #define LPSPI2_BASE                              (0x44370000u)
  /** Peripheral LPSPI2 base pointer */
  #define LPSPI2                                   ((LPSPI_Type *)LPSPI2_BASE)
  /** Peripheral LPSPI3 base address */
  #define LPSPI3_BASE                              (0x42550000u)
  /** Peripheral LPSPI3 base pointer */
  #define LPSPI3                                   ((LPSPI_Type *)LPSPI3_BASE)
  /** Peripheral LPSPI4 base address */
  #define LPSPI4_BASE                              (0x42560000u)
  /** Peripheral LPSPI4 base pointer */
  #define LPSPI4                                   ((LPSPI_Type *)LPSPI4_BASE)
  /** Array initializer of LPSPI peripheral base addresses */
  #define LPSPI_BASE_ADDRS                         { 0u, LPSPI1_BASE, LPSPI2_BASE, LPSPI3_BASE, LPSPI4_BASE }
  /** Array initializer of LPSPI peripheral base pointers */
  #define LPSPI_BASE_PTRS                          { (LPSPI_Type *)0u, LPSPI1, LPSPI2, LPSPI3, LPSPI4 }
#endif
/** Interrupt vectors for the LPSPI peripheral type */
#define LPSPI_IRQS                               { NotAvail_IRQn, LPSPI1_IRQn, LPSPI2_IRQn, LPSPI3_IRQn, LPSPI4_IRQn }

/*!
 * @}
 */ /* end of group LPSPI_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- LPTMR Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPTMR_Peripheral_Access_Layer LPTMR Peripheral Access Layer
 * @{
 */

/** LPTMR - Register Layout Typedef */
typedef struct {
  __IO uint32_t CSR;                               /**< Control Status, offset: 0x0 */
  __IO uint32_t PSR;                               /**< Prescaler and Glitch Filter, offset: 0x4 */
  __IO uint32_t CMR;                               /**< Compare, offset: 0x8 */
  __IO uint32_t CNR;                               /**< Counter, offset: 0xC */
} LPTMR_Type;

/* ----------------------------------------------------------------------------
   -- LPTMR Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPTMR_Register_Masks LPTMR Register Masks
 * @{
 */

/*! @name CSR - Control Status */
/*! @{ */

#define LPTMR_CSR_TEN_MASK                       (0x1U)
#define LPTMR_CSR_TEN_SHIFT                      (0U)
/*! TEN - Timer Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPTMR_CSR_TEN(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TEN_SHIFT)) & LPTMR_CSR_TEN_MASK)

#define LPTMR_CSR_TMS_MASK                       (0x2U)
#define LPTMR_CSR_TMS_SHIFT                      (1U)
/*! TMS - Timer Mode Select
 *  0b0..Time Counter
 *  0b1..Pulse Counter
 */
#define LPTMR_CSR_TMS(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TMS_SHIFT)) & LPTMR_CSR_TMS_MASK)

#define LPTMR_CSR_TFC_MASK                       (0x4U)
#define LPTMR_CSR_TFC_SHIFT                      (2U)
/*! TFC - Timer Free-Running Counter
 *  0b0..Reset when TCF asserts
 *  0b1..Reset on overflow
 */
#define LPTMR_CSR_TFC(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TFC_SHIFT)) & LPTMR_CSR_TFC_MASK)

#define LPTMR_CSR_TPP_MASK                       (0x8U)
#define LPTMR_CSR_TPP_SHIFT                      (3U)
/*! TPP - Timer Pin Polarity
 *  0b0..Active-high
 *  0b1..Active-low
 */
#define LPTMR_CSR_TPP(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TPP_SHIFT)) & LPTMR_CSR_TPP_MASK)

#define LPTMR_CSR_TPS_MASK                       (0x30U)
#define LPTMR_CSR_TPS_SHIFT                      (4U)
/*! TPS - Timer Pin Select
 *  0b00..Input 0
 *  0b01..Input 1
 *  0b10..Input 2
 *  0b11..Input 3
 */
#define LPTMR_CSR_TPS(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TPS_SHIFT)) & LPTMR_CSR_TPS_MASK)

#define LPTMR_CSR_TIE_MASK                       (0x40U)
#define LPTMR_CSR_TIE_SHIFT                      (6U)
/*! TIE - Timer Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPTMR_CSR_TIE(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TIE_SHIFT)) & LPTMR_CSR_TIE_MASK)

#define LPTMR_CSR_TCF_MASK                       (0x80U)
#define LPTMR_CSR_TCF_SHIFT                      (7U)
/*! TCF - Timer Compare Flag
 *  0b0..CNR != (CMR + 1)
 *  0b1..CNR = (CMR + 1)
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPTMR_CSR_TCF(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TCF_SHIFT)) & LPTMR_CSR_TCF_MASK)

#define LPTMR_CSR_TDRE_MASK                      (0x100U)
#define LPTMR_CSR_TDRE_SHIFT                     (8U)
/*! TDRE - Timer DMA Request Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPTMR_CSR_TDRE(x)                        (((uint32_t)(((uint32_t)(x)) << LPTMR_CSR_TDRE_SHIFT)) & LPTMR_CSR_TDRE_MASK)
/*! @} */

/*! @name PSR - Prescaler and Glitch Filter */
/*! @{ */

#define LPTMR_PSR_PCS_MASK                       (0x3U)
#define LPTMR_PSR_PCS_SHIFT                      (0U)
/*! PCS - Prescaler and Glitch Filter Clock Select
 *  0b00..Clock 0
 *  0b01..Clock 1
 *  0b10..Clock 2
 *  0b11..Clock 3
 */
#define LPTMR_PSR_PCS(x)                         (((uint32_t)(((uint32_t)(x)) << LPTMR_PSR_PCS_SHIFT)) & LPTMR_PSR_PCS_MASK)

#define LPTMR_PSR_PBYP_MASK                      (0x4U)
#define LPTMR_PSR_PBYP_SHIFT                     (2U)
/*! PBYP - Prescaler and Glitch Filter Bypass
 *  0b0..Prescaler and glitch filter enable
 *  0b1..Prescaler and glitch filter bypass
 */
#define LPTMR_PSR_PBYP(x)                        (((uint32_t)(((uint32_t)(x)) << LPTMR_PSR_PBYP_SHIFT)) & LPTMR_PSR_PBYP_MASK)

#define LPTMR_PSR_PRESCALE_MASK                  (0x78U)
#define LPTMR_PSR_PRESCALE_SHIFT                 (3U)
/*! PRESCALE - Prescaler and Glitch Filter Value
 *  0b0000..Prescaler divides the prescaler clock by 2; glitch filter does not support this configuration
 *  0b0001..Prescaler divides the prescaler clock by 4; glitch filter recognizes change on input pin after two rising clock edges
 *  0b0010..Prescaler divides the prescaler clock by 8; glitch filter recognizes change on input pin after four rising clock edges
 *  0b0011..Prescaler divides the prescaler clock by 16; glitch filter recognizes change on input pin after eight rising clock edges
 *  0b0100..Prescaler divides the prescaler clock by 32; glitch filter recognizes change on input pin after 16 rising clock edges
 *  0b0101..Prescaler divides the prescaler clock by 64; glitch filter recognizes change on input pin after 32 rising clock edges
 *  0b0110..Prescaler divides the prescaler clock by 128; glitch filter recognizes change on input pin after 64 rising clock edges
 *  0b0111..Prescaler divides the prescaler clock by 256; glitch filter recognizes change on input pin after 128 rising clock edges
 *  0b1000..Prescaler divides the prescaler clock by 512; glitch filter recognizes change on input pin after 256 rising clock edges
 *  0b1001..Prescaler divides the prescaler clock by 1024; glitch filter recognizes change on input pin after 512 rising clock edges
 *  0b1010..Prescaler divides the prescaler clock by 2048; glitch filter recognizes change on input pin after 1024 rising clock edges
 *  0b1011..Prescaler divides the prescaler clock by 4096; glitch filter recognizes change on input pin after 2048 rising clock edges
 *  0b1100..Prescaler divides the prescaler clock by 8192; glitch filter recognizes change on input pin after 4096 rising clock edges
 *  0b1101..Prescaler divides the prescaler clock by 16,384; glitch filter recognizes change on input pin after 8192 rising clock edges
 *  0b1110..Prescaler divides the prescaler clock by 32,768; glitch filter recognizes change on input pin after 16,384 rising clock edges
 *  0b1111..Prescaler divides the prescaler clock by 65,536; glitch filter recognizes change on input pin after 32,768 rising clock edges
 */
#define LPTMR_PSR_PRESCALE(x)                    (((uint32_t)(((uint32_t)(x)) << LPTMR_PSR_PRESCALE_SHIFT)) & LPTMR_PSR_PRESCALE_MASK)
/*! @} */

/*! @name CMR - Compare */
/*! @{ */

#define LPTMR_CMR_COMPARE_MASK                   (0xFFFFFFFFU)
#define LPTMR_CMR_COMPARE_SHIFT                  (0U)
/*! COMPARE - Compare Value */
#define LPTMR_CMR_COMPARE(x)                     (((uint32_t)(((uint32_t)(x)) << LPTMR_CMR_COMPARE_SHIFT)) & LPTMR_CMR_COMPARE_MASK)
/*! @} */

/*! @name CNR - Counter */
/*! @{ */

#define LPTMR_CNR_COUNTER_MASK                   (0xFFFFFFFFU)
#define LPTMR_CNR_COUNTER_SHIFT                  (0U)
/*! COUNTER - Counter Value */
#define LPTMR_CNR_COUNTER(x)                     (((uint32_t)(((uint32_t)(x)) << LPTMR_CNR_COUNTER_SHIFT)) & LPTMR_CNR_COUNTER_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group LPTMR_Register_Masks */


/* LPTMR - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral LPTMR1 base address */
  #define LPTMR1_BASE                              (0x54300000u)
  /** Peripheral LPTMR1 base address */
  #define LPTMR1_BASE_NS                           (0x44300000u)
  /** Peripheral LPTMR1 base pointer */
  #define LPTMR1                                   ((LPTMR_Type *)LPTMR1_BASE)
  /** Peripheral LPTMR1 base pointer */
  #define LPTMR1_NS                                ((LPTMR_Type *)LPTMR1_BASE_NS)
  /** Array initializer of LPTMR peripheral base addresses */
  #define LPTMR_BASE_ADDRS                         { 0u, LPTMR1_BASE }
  /** Array initializer of LPTMR peripheral base pointers */
  #define LPTMR_BASE_PTRS                          { (LPTMR_Type *)0u, LPTMR1 }
  /** Array initializer of LPTMR peripheral base addresses */
  #define LPTMR_BASE_ADDRS_NS                      { 0u, LPTMR1_BASE_NS }
  /** Array initializer of LPTMR peripheral base pointers */
  #define LPTMR_BASE_PTRS_NS                       { (LPTMR_Type *)0u, LPTMR1_NS }
#else
  /** Peripheral LPTMR1 base address */
  #define LPTMR1_BASE                              (0x44300000u)
  /** Peripheral LPTMR1 base pointer */
  #define LPTMR1                                   ((LPTMR_Type *)LPTMR1_BASE)
  /** Array initializer of LPTMR peripheral base addresses */
  #define LPTMR_BASE_ADDRS                         { 0u, LPTMR1_BASE }
  /** Array initializer of LPTMR peripheral base pointers */
  #define LPTMR_BASE_PTRS                          { (LPTMR_Type *)0u, LPTMR1 }
#endif
/** Interrupt vectors for the LPTMR peripheral type */
#define LPTMR_IRQS                               { NotAvail_IRQn, LPTMR1_IRQn }

/*!
 * @}
 */ /* end of group LPTMR_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- LPUART Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPUART_Peripheral_Access_Layer LPUART Peripheral Access Layer
 * @{
 */

/** LPUART - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
  __IO uint32_t GLOBAL;                            /**< Global, offset: 0x8 */
  __IO uint32_t PINCFG;                            /**< Pin Configuration, offset: 0xC */
  __IO uint32_t BAUD;                              /**< Baud Rate, offset: 0x10 */
  __IO uint32_t STAT;                              /**< Status, offset: 0x14 */
  __IO uint32_t CTRL;                              /**< Control, offset: 0x18 */
  __IO uint32_t DATA;                              /**< Data, offset: 0x1C */
  __IO uint32_t MATCH;                             /**< Match Address, offset: 0x20 */
  __IO uint32_t MODIR;                             /**< MODEM IrDA, offset: 0x24 */
  __IO uint32_t FIFO;                              /**< FIFO, offset: 0x28 */
  __IO uint32_t WATER;                             /**< Watermark, offset: 0x2C */
  __I  uint32_t DATARO;                            /**< Data Read-Only, offset: 0x30 */
       uint8_t RESERVED_0[12];
  __IO uint32_t MCR;                               /**< MODEM Control, offset: 0x40 */
  __IO uint32_t MSR;                               /**< MODEM Status, offset: 0x44 */
  __IO uint32_t REIR;                              /**< Receiver Extended Idle, offset: 0x48 */
  __IO uint32_t TEIR;                              /**< Transmitter Extended Idle, offset: 0x4C */
  __IO uint32_t HDCR;                              /**< Half Duplex Control, offset: 0x50 */
       uint8_t RESERVED_1[4];
  __IO uint32_t TOCR;                              /**< Timeout Control, offset: 0x58 */
  __IO uint32_t TOSR;                              /**< Timeout Status, offset: 0x5C */
  __IO uint32_t TIMEOUT[4];                        /**< Timeout N, array offset: 0x60, array step: 0x4 */
       uint8_t RESERVED_2[400];
  __O  uint32_t TCBR[128];                         /**< Transmit Command Burst, array offset: 0x200, array step: 0x4 */
  __O  uint32_t TDBR[256];                         /**< Transmit Data Burst, array offset: 0x400, array step: 0x4 */
} LPUART_Type;

/* ----------------------------------------------------------------------------
   -- LPUART Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup LPUART_Register_Masks LPUART Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define LPUART_VERID_FEATURE_MASK                (0xFFFFU)
#define LPUART_VERID_FEATURE_SHIFT               (0U)
/*! FEATURE - Feature Identification Number
 *  0b0000000000000001..Standard feature set
 *  0b0000000000000011..Standard feature set with MODEM and IrDA support
 *  0b0000000000000111..Enhanced feature set with full MODEM, IrDA, and enhanced idle detection
 */
#define LPUART_VERID_FEATURE(x)                  (((uint32_t)(((uint32_t)(x)) << LPUART_VERID_FEATURE_SHIFT)) & LPUART_VERID_FEATURE_MASK)

#define LPUART_VERID_MINOR_MASK                  (0xFF0000U)
#define LPUART_VERID_MINOR_SHIFT                 (16U)
/*! MINOR - Minor Version Number */
#define LPUART_VERID_MINOR(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_VERID_MINOR_SHIFT)) & LPUART_VERID_MINOR_MASK)

#define LPUART_VERID_MAJOR_MASK                  (0xFF000000U)
#define LPUART_VERID_MAJOR_SHIFT                 (24U)
/*! MAJOR - Major Version Number */
#define LPUART_VERID_MAJOR(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_VERID_MAJOR_SHIFT)) & LPUART_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define LPUART_PARAM_TXFIFO_MASK                 (0xFFU)
#define LPUART_PARAM_TXFIFO_SHIFT                (0U)
/*! TXFIFO - Transmit FIFO Size */
#define LPUART_PARAM_TXFIFO(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_PARAM_TXFIFO_SHIFT)) & LPUART_PARAM_TXFIFO_MASK)

#define LPUART_PARAM_RXFIFO_MASK                 (0xFF00U)
#define LPUART_PARAM_RXFIFO_SHIFT                (8U)
/*! RXFIFO - Receive FIFO Size */
#define LPUART_PARAM_RXFIFO(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_PARAM_RXFIFO_SHIFT)) & LPUART_PARAM_RXFIFO_MASK)
/*! @} */

/*! @name GLOBAL - Global */
/*! @{ */

#define LPUART_GLOBAL_RST_MASK                   (0x2U)
#define LPUART_GLOBAL_RST_SHIFT                  (1U)
/*! RST - Software Reset
 *  0b0..Not reset
 *  0b1..Reset
 */
#define LPUART_GLOBAL_RST(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_GLOBAL_RST_SHIFT)) & LPUART_GLOBAL_RST_MASK)
/*! @} */

/*! @name PINCFG - Pin Configuration */
/*! @{ */

#define LPUART_PINCFG_TRGSEL_MASK                (0x3U)
#define LPUART_PINCFG_TRGSEL_SHIFT               (0U)
/*! TRGSEL - Trigger Select
 *  0b00..Input trigger disabled
 *  0b01..Input trigger used instead of the RXD pin input
 *  0b10..Input trigger used instead of the CTS_B pin input
 *  0b11..Input trigger used to modulate the TXD pin output, which (after TXINV configuration) is internally ANDed with the input trigger
 */
#define LPUART_PINCFG_TRGSEL(x)                  (((uint32_t)(((uint32_t)(x)) << LPUART_PINCFG_TRGSEL_SHIFT)) & LPUART_PINCFG_TRGSEL_MASK)
/*! @} */

/*! @name BAUD - Baud Rate */
/*! @{ */

#define LPUART_BAUD_SBR_MASK                     (0x1FFFU)
#define LPUART_BAUD_SBR_SHIFT                    (0U)
/*! SBR - Baud Rate Modulo Divisor */
#define LPUART_BAUD_SBR(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_SBR_SHIFT)) & LPUART_BAUD_SBR_MASK)

#define LPUART_BAUD_SBNS_MASK                    (0x2000U)
#define LPUART_BAUD_SBNS_SHIFT                   (13U)
/*! SBNS - Stop Bit Number Select
 *  0b0..One stop bit
 *  0b1..Two stop bits
 */
#define LPUART_BAUD_SBNS(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_SBNS_SHIFT)) & LPUART_BAUD_SBNS_MASK)

#define LPUART_BAUD_RXEDGIE_MASK                 (0x4000U)
#define LPUART_BAUD_RXEDGIE_SHIFT                (14U)
/*! RXEDGIE - RX Input Active Edge Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_BAUD_RXEDGIE(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_RXEDGIE_SHIFT)) & LPUART_BAUD_RXEDGIE_MASK)

#define LPUART_BAUD_LBKDIE_MASK                  (0x8000U)
#define LPUART_BAUD_LBKDIE_SHIFT                 (15U)
/*! LBKDIE - LIN Break Detect Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_BAUD_LBKDIE(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_LBKDIE_SHIFT)) & LPUART_BAUD_LBKDIE_MASK)

#define LPUART_BAUD_RESYNCDIS_MASK               (0x10000U)
#define LPUART_BAUD_RESYNCDIS_SHIFT              (16U)
/*! RESYNCDIS - Resynchronization Disable
 *  0b0..Enable
 *  0b1..Disable
 */
#define LPUART_BAUD_RESYNCDIS(x)                 (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_RESYNCDIS_SHIFT)) & LPUART_BAUD_RESYNCDIS_MASK)

#define LPUART_BAUD_BOTHEDGE_MASK                (0x20000U)
#define LPUART_BAUD_BOTHEDGE_SHIFT               (17U)
/*! BOTHEDGE - Both Edge Sampling
 *  0b0..Rising edge
 *  0b1..Both rising and falling edges
 */
#define LPUART_BAUD_BOTHEDGE(x)                  (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_BOTHEDGE_SHIFT)) & LPUART_BAUD_BOTHEDGE_MASK)

#define LPUART_BAUD_MATCFG_MASK                  (0xC0000U)
#define LPUART_BAUD_MATCFG_SHIFT                 (18U)
/*! MATCFG - Match Configuration
 *  0b00..Address match wake-up
 *  0b01..Idle match wake-up
 *  0b10..Match on and match off
 *  0b11..Enables RWU on data match and match on or off for the transmitter CTS input
 */
#define LPUART_BAUD_MATCFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_MATCFG_SHIFT)) & LPUART_BAUD_MATCFG_MASK)

#define LPUART_BAUD_RIDMAE_MASK                  (0x100000U)
#define LPUART_BAUD_RIDMAE_SHIFT                 (20U)
/*! RIDMAE - Receiver Idle DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_BAUD_RIDMAE(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_RIDMAE_SHIFT)) & LPUART_BAUD_RIDMAE_MASK)

#define LPUART_BAUD_RDMAE_MASK                   (0x200000U)
#define LPUART_BAUD_RDMAE_SHIFT                  (21U)
/*! RDMAE - Receiver Full DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_BAUD_RDMAE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_RDMAE_SHIFT)) & LPUART_BAUD_RDMAE_MASK)

#define LPUART_BAUD_TDMAE_MASK                   (0x800000U)
#define LPUART_BAUD_TDMAE_SHIFT                  (23U)
/*! TDMAE - Transmitter DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_BAUD_TDMAE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_TDMAE_SHIFT)) & LPUART_BAUD_TDMAE_MASK)

#define LPUART_BAUD_OSR_MASK                     (0x1F000000U)
#define LPUART_BAUD_OSR_SHIFT                    (24U)
/*! OSR - Oversampling Ratio
 *  0b00000..Results in an OSR of 16
 *  0b00001..Reserved
 *  0b00010..Reserved
 *  0b00011..Results in an OSR of 4 (requires BAUD[BOTHEDGE] to be 1)
 *  0b00100..Results in an OSR of 5 (requires BAUD[BOTHEDGE] to be 1)
 *  0b00101..Results in an OSR of 6 (requires BAUD[BOTHEDGE] to be 1)
 *  0b00110..Results in an OSR of 7 (requires BAUD[BOTHEDGE] to be 1)
 *  0b00111..Results in an OSR of 8
 *  0b01000..Results in an OSR of 9
 *  0b01001..Results in an OSR of 10
 *  0b01010..Results in an OSR of 11
 *  0b01011..Results in an OSR of 12
 *  0b01100..Results in an OSR of 13
 *  0b01101..Results in an OSR of 14
 *  0b01110..Results in an OSR of 15
 *  0b01111..Results in an OSR of 16
 *  0b10000..Results in an OSR of 17
 *  0b10001..Results in an OSR of 18
 *  0b10010..Results in an OSR of 19
 *  0b10011..Results in an OSR of 20
 *  0b10100..Results in an OSR of 21
 *  0b10101..Results in an OSR of 22
 *  0b10110..Results in an OSR of 23
 *  0b10111..Results in an OSR of 24
 *  0b11000..Results in an OSR of 25
 *  0b11001..Results in an OSR of 26
 *  0b11010..Results in an OSR of 27
 *  0b11011..Results in an OSR of 28
 *  0b11100..Results in an OSR of 29
 *  0b11101..Results in an OSR of 30
 *  0b11110..Results in an OSR of 31
 *  0b11111..Results in an OSR of 32
 */
#define LPUART_BAUD_OSR(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_OSR_SHIFT)) & LPUART_BAUD_OSR_MASK)

#define LPUART_BAUD_M10_MASK                     (0x20000000U)
#define LPUART_BAUD_M10_SHIFT                    (29U)
/*! M10 - 10-Bit Mode Select
 *  0b0..Receiver and transmitter use 7-bit to 9-bit data characters
 *  0b1..Receiver and transmitter use 10-bit data characters
 */
#define LPUART_BAUD_M10(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_M10_SHIFT)) & LPUART_BAUD_M10_MASK)

#define LPUART_BAUD_MAEN2_MASK                   (0x40000000U)
#define LPUART_BAUD_MAEN2_SHIFT                  (30U)
/*! MAEN2 - Match Address Mode Enable 2
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_BAUD_MAEN2(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_MAEN2_SHIFT)) & LPUART_BAUD_MAEN2_MASK)

#define LPUART_BAUD_MAEN1_MASK                   (0x80000000U)
#define LPUART_BAUD_MAEN1_SHIFT                  (31U)
/*! MAEN1 - Match Address Mode Enable 1
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_BAUD_MAEN1(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_BAUD_MAEN1_SHIFT)) & LPUART_BAUD_MAEN1_MASK)
/*! @} */

/*! @name STAT - Status */
/*! @{ */

#define LPUART_STAT_LBKFE_MASK                   (0x1U)
#define LPUART_STAT_LBKFE_SHIFT                  (0U)
/*! LBKFE - LIN Break Flag Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_STAT_LBKFE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_LBKFE_SHIFT)) & LPUART_STAT_LBKFE_MASK)

#define LPUART_STAT_AME_MASK                     (0x2U)
#define LPUART_STAT_AME_SHIFT                    (1U)
/*! AME - Address Mark Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_STAT_AME(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_AME_SHIFT)) & LPUART_STAT_AME_MASK)

#define LPUART_STAT_MSF_MASK                     (0x100U)
#define LPUART_STAT_MSF_SHIFT                    (8U)
/*! MSF - MODEM Status Flag
 *  0b0..Field is 0
 *  0b1..Field is 1
 */
#define LPUART_STAT_MSF(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_MSF_SHIFT)) & LPUART_STAT_MSF_MASK)

#define LPUART_STAT_TSF_MASK                     (0x200U)
#define LPUART_STAT_TSF_SHIFT                    (9U)
/*! TSF - Timeout Status Flag
 *  0b0..Field is 0
 *  0b1..Field is 1
 */
#define LPUART_STAT_TSF(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_TSF_SHIFT)) & LPUART_STAT_TSF_MASK)

#define LPUART_STAT_MA2F_MASK                    (0x4000U)
#define LPUART_STAT_MA2F_SHIFT                   (14U)
/*! MA2F - Match 2 Flag
 *  0b0..Not equal to MA2
 *  0b1..Equal to MA2
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_MA2F(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_MA2F_SHIFT)) & LPUART_STAT_MA2F_MASK)

#define LPUART_STAT_MA1F_MASK                    (0x8000U)
#define LPUART_STAT_MA1F_SHIFT                   (15U)
/*! MA1F - Match 1 Flag
 *  0b0..Not equal to MA1
 *  0b1..Equal to MA1
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_MA1F(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_MA1F_SHIFT)) & LPUART_STAT_MA1F_MASK)

#define LPUART_STAT_PF_MASK                      (0x10000U)
#define LPUART_STAT_PF_SHIFT                     (16U)
/*! PF - Parity Error Flag
 *  0b0..No parity error detected
 *  0b1..Parity error detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_PF(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_PF_SHIFT)) & LPUART_STAT_PF_MASK)

#define LPUART_STAT_FE_MASK                      (0x20000U)
#define LPUART_STAT_FE_SHIFT                     (17U)
/*! FE - Framing Error Flag
 *  0b0..No framing error detected (this does not guarantee that the framing is correct)
 *  0b1..Framing error detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_FE(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_FE_SHIFT)) & LPUART_STAT_FE_MASK)

#define LPUART_STAT_NF_MASK                      (0x40000U)
#define LPUART_STAT_NF_SHIFT                     (18U)
/*! NF - Noise Flag
 *  0b0..No noise detected
 *  0b1..Noise detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_NF(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_NF_SHIFT)) & LPUART_STAT_NF_MASK)

#define LPUART_STAT_OR_MASK                      (0x80000U)
#define LPUART_STAT_OR_SHIFT                     (19U)
/*! OR - Receiver Overrun Flag
 *  0b0..No overrun
 *  0b1..Receive overrun (new LPUART data is lost)
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_OR(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_OR_SHIFT)) & LPUART_STAT_OR_MASK)

#define LPUART_STAT_IDLE_MASK                    (0x100000U)
#define LPUART_STAT_IDLE_SHIFT                   (20U)
/*! IDLE - Idle Line Flag
 *  0b0..Idle line detected
 *  0b1..Idle line not detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_IDLE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_IDLE_SHIFT)) & LPUART_STAT_IDLE_MASK)

#define LPUART_STAT_RDRF_MASK                    (0x200000U)
#define LPUART_STAT_RDRF_SHIFT                   (21U)
/*! RDRF - Receive Data Register Full Flag
 *  0b0..Equal to or less than watermark
 *  0b1..Greater than watermark
 */
#define LPUART_STAT_RDRF(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_RDRF_SHIFT)) & LPUART_STAT_RDRF_MASK)

#define LPUART_STAT_TC_MASK                      (0x400000U)
#define LPUART_STAT_TC_SHIFT                     (22U)
/*! TC - Transmission Complete Flag
 *  0b0..Transmitter active
 *  0b1..Transmitter idle
 */
#define LPUART_STAT_TC(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_TC_SHIFT)) & LPUART_STAT_TC_MASK)

#define LPUART_STAT_TDRE_MASK                    (0x800000U)
#define LPUART_STAT_TDRE_SHIFT                   (23U)
/*! TDRE - Transmit Data Register Empty Flag
 *  0b0..Greater than watermark
 *  0b1..Equal to or less than watermark
 */
#define LPUART_STAT_TDRE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_TDRE_SHIFT)) & LPUART_STAT_TDRE_MASK)

#define LPUART_STAT_RAF_MASK                     (0x1000000U)
#define LPUART_STAT_RAF_SHIFT                    (24U)
/*! RAF - Receiver Active Flag
 *  0b0..Idle, waiting for a start bit
 *  0b1..Receiver active (RXD pin input not idle)
 */
#define LPUART_STAT_RAF(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_RAF_SHIFT)) & LPUART_STAT_RAF_MASK)

#define LPUART_STAT_LBKDE_MASK                   (0x2000000U)
#define LPUART_STAT_LBKDE_SHIFT                  (25U)
/*! LBKDE - LIN Break Detection Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_STAT_LBKDE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_LBKDE_SHIFT)) & LPUART_STAT_LBKDE_MASK)

#define LPUART_STAT_BRK13_MASK                   (0x4000000U)
#define LPUART_STAT_BRK13_SHIFT                  (26U)
/*! BRK13 - Break Character Generation Length
 *  0b0..9 to 13 bit times
 *  0b1..12 to 15 bit times
 */
#define LPUART_STAT_BRK13(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_BRK13_SHIFT)) & LPUART_STAT_BRK13_MASK)

#define LPUART_STAT_RWUID_MASK                   (0x8000000U)
#define LPUART_STAT_RWUID_SHIFT                  (27U)
/*! RWUID - Receive Wake Up Idle Detect
 *  0b0..STAT[IDLE] does not become 1
 *  0b1..STAT[IDLE] becomes 1
 */
#define LPUART_STAT_RWUID(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_RWUID_SHIFT)) & LPUART_STAT_RWUID_MASK)

#define LPUART_STAT_RXINV_MASK                   (0x10000000U)
#define LPUART_STAT_RXINV_SHIFT                  (28U)
/*! RXINV - Receive Data Inversion
 *  0b0..Inverted
 *  0b1..Not inverted
 */
#define LPUART_STAT_RXINV(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_RXINV_SHIFT)) & LPUART_STAT_RXINV_MASK)

#define LPUART_STAT_MSBF_MASK                    (0x20000000U)
#define LPUART_STAT_MSBF_SHIFT                   (29U)
/*! MSBF - MSB First
 *  0b0..LSB
 *  0b1..MSB
 */
#define LPUART_STAT_MSBF(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_MSBF_SHIFT)) & LPUART_STAT_MSBF_MASK)

#define LPUART_STAT_RXEDGIF_MASK                 (0x40000000U)
#define LPUART_STAT_RXEDGIF_SHIFT                (30U)
/*! RXEDGIF - RXD Pin Active Edge Interrupt Flag
 *  0b0..Not occurred
 *  0b1..Occurred
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_RXEDGIF(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_RXEDGIF_SHIFT)) & LPUART_STAT_RXEDGIF_MASK)

#define LPUART_STAT_LBKDIF_MASK                  (0x80000000U)
#define LPUART_STAT_LBKDIF_SHIFT                 (31U)
/*! LBKDIF - LIN Break Detect Interrupt Flag
 *  0b0..Not detected
 *  0b1..Detected
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_STAT_LBKDIF(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_STAT_LBKDIF_SHIFT)) & LPUART_STAT_LBKDIF_MASK)
/*! @} */

/*! @name CTRL - Control */
/*! @{ */

#define LPUART_CTRL_PT_MASK                      (0x1U)
#define LPUART_CTRL_PT_SHIFT                     (0U)
/*! PT - Parity Type
 *  0b0..Even parity
 *  0b1..Odd parity
 */
#define LPUART_CTRL_PT(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_PT_SHIFT)) & LPUART_CTRL_PT_MASK)

#define LPUART_CTRL_PE_MASK                      (0x2U)
#define LPUART_CTRL_PE_SHIFT                     (1U)
/*! PE - Parity Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_PE(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_PE_SHIFT)) & LPUART_CTRL_PE_MASK)

#define LPUART_CTRL_ILT_MASK                     (0x4U)
#define LPUART_CTRL_ILT_SHIFT                    (2U)
/*! ILT - Idle Line Type Select
 *  0b0..After the start bit
 *  0b1..After the stop bit
 */
#define LPUART_CTRL_ILT(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_ILT_SHIFT)) & LPUART_CTRL_ILT_MASK)

#define LPUART_CTRL_WAKE_MASK                    (0x8U)
#define LPUART_CTRL_WAKE_SHIFT                   (3U)
/*! WAKE - Receiver Wake-Up Method Select
 *  0b0..Idle
 *  0b1..Mark
 */
#define LPUART_CTRL_WAKE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_WAKE_SHIFT)) & LPUART_CTRL_WAKE_MASK)

#define LPUART_CTRL_M_MASK                       (0x10U)
#define LPUART_CTRL_M_SHIFT                      (4U)
/*! M - 9-Bit Or 8-Bit Mode Select
 *  0b0..8-bit
 *  0b1..9-bit
 */
#define LPUART_CTRL_M(x)                         (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_M_SHIFT)) & LPUART_CTRL_M_MASK)

#define LPUART_CTRL_RSRC_MASK                    (0x20U)
#define LPUART_CTRL_RSRC_SHIFT                   (5U)
/*! RSRC - Receiver Source Select
 *  0b0..Internal Loopback mode
 *  0b1..Single-wire mode
 */
#define LPUART_CTRL_RSRC(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_RSRC_SHIFT)) & LPUART_CTRL_RSRC_MASK)

#define LPUART_CTRL_DOZEEN_MASK                  (0x40U)
#define LPUART_CTRL_DOZEEN_SHIFT                 (6U)
/*! DOZEEN - Doze Mode
 *  0b0..Enable
 *  0b1..Disable
 */
#define LPUART_CTRL_DOZEEN(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_DOZEEN_SHIFT)) & LPUART_CTRL_DOZEEN_MASK)

#define LPUART_CTRL_LOOPS_MASK                   (0x80U)
#define LPUART_CTRL_LOOPS_SHIFT                  (7U)
/*! LOOPS - Loop Mode Select
 *  0b0..Normal operation: RXD and TXD use separate pins
 *  0b1..Loop mode or Single-Wire mode
 */
#define LPUART_CTRL_LOOPS(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_LOOPS_SHIFT)) & LPUART_CTRL_LOOPS_MASK)

#define LPUART_CTRL_IDLECFG_MASK                 (0x700U)
#define LPUART_CTRL_IDLECFG_SHIFT                (8U)
/*! IDLECFG - Idle Configuration
 *  0b000..1
 *  0b001..2
 *  0b010..4
 *  0b011..8
 *  0b100..16
 *  0b101..32
 *  0b110..64
 *  0b111..128
 */
#define LPUART_CTRL_IDLECFG(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_IDLECFG_SHIFT)) & LPUART_CTRL_IDLECFG_MASK)

#define LPUART_CTRL_M7_MASK                      (0x800U)
#define LPUART_CTRL_M7_SHIFT                     (11U)
/*! M7 - 7-Bit Mode Select
 *  0b0..8-bit to 10-bit
 *  0b1..7-bit
 */
#define LPUART_CTRL_M7(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_M7_SHIFT)) & LPUART_CTRL_M7_MASK)

#define LPUART_CTRL_MA2IE_MASK                   (0x4000U)
#define LPUART_CTRL_MA2IE_SHIFT                  (14U)
/*! MA2IE - Match 2 (MA2F) Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_MA2IE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_MA2IE_SHIFT)) & LPUART_CTRL_MA2IE_MASK)

#define LPUART_CTRL_MA1IE_MASK                   (0x8000U)
#define LPUART_CTRL_MA1IE_SHIFT                  (15U)
/*! MA1IE - Match 1 (MA1F) Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_MA1IE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_MA1IE_SHIFT)) & LPUART_CTRL_MA1IE_MASK)

#define LPUART_CTRL_SBK_MASK                     (0x10000U)
#define LPUART_CTRL_SBK_SHIFT                    (16U)
/*! SBK - Send Break
 *  0b0..Normal transmitter operation
 *  0b1..Queue break character(s) to be sent
 */
#define LPUART_CTRL_SBK(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_SBK_SHIFT)) & LPUART_CTRL_SBK_MASK)

#define LPUART_CTRL_RWU_MASK                     (0x20000U)
#define LPUART_CTRL_RWU_SHIFT                    (17U)
/*! RWU - Receiver Wake-Up Control
 *  0b0..Normal receiver operation
 *  0b1..LPUART receiver in standby, waiting for a wake-up condition
 */
#define LPUART_CTRL_RWU(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_RWU_SHIFT)) & LPUART_CTRL_RWU_MASK)

#define LPUART_CTRL_RE_MASK                      (0x40000U)
#define LPUART_CTRL_RE_SHIFT                     (18U)
/*! RE - Receiver Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_RE(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_RE_SHIFT)) & LPUART_CTRL_RE_MASK)

#define LPUART_CTRL_TE_MASK                      (0x80000U)
#define LPUART_CTRL_TE_SHIFT                     (19U)
/*! TE - Transmitter Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_TE(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_TE_SHIFT)) & LPUART_CTRL_TE_MASK)

#define LPUART_CTRL_ILIE_MASK                    (0x100000U)
#define LPUART_CTRL_ILIE_SHIFT                   (20U)
/*! ILIE - Idle Line Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_ILIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_ILIE_SHIFT)) & LPUART_CTRL_ILIE_MASK)

#define LPUART_CTRL_RIE_MASK                     (0x200000U)
#define LPUART_CTRL_RIE_SHIFT                    (21U)
/*! RIE - Receiver Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_RIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_RIE_SHIFT)) & LPUART_CTRL_RIE_MASK)

#define LPUART_CTRL_TCIE_MASK                    (0x400000U)
#define LPUART_CTRL_TCIE_SHIFT                   (22U)
/*! TCIE - Transmission Complete Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_TCIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_TCIE_SHIFT)) & LPUART_CTRL_TCIE_MASK)

#define LPUART_CTRL_TIE_MASK                     (0x800000U)
#define LPUART_CTRL_TIE_SHIFT                    (23U)
/*! TIE - Transmit Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_TIE(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_TIE_SHIFT)) & LPUART_CTRL_TIE_MASK)

#define LPUART_CTRL_PEIE_MASK                    (0x1000000U)
#define LPUART_CTRL_PEIE_SHIFT                   (24U)
/*! PEIE - Parity Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_PEIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_PEIE_SHIFT)) & LPUART_CTRL_PEIE_MASK)

#define LPUART_CTRL_FEIE_MASK                    (0x2000000U)
#define LPUART_CTRL_FEIE_SHIFT                   (25U)
/*! FEIE - Framing Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_FEIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_FEIE_SHIFT)) & LPUART_CTRL_FEIE_MASK)

#define LPUART_CTRL_NEIE_MASK                    (0x4000000U)
#define LPUART_CTRL_NEIE_SHIFT                   (26U)
/*! NEIE - Noise Error Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_NEIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_NEIE_SHIFT)) & LPUART_CTRL_NEIE_MASK)

#define LPUART_CTRL_ORIE_MASK                    (0x8000000U)
#define LPUART_CTRL_ORIE_SHIFT                   (27U)
/*! ORIE - Overrun Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_CTRL_ORIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_ORIE_SHIFT)) & LPUART_CTRL_ORIE_MASK)

#define LPUART_CTRL_TXINV_MASK                   (0x10000000U)
#define LPUART_CTRL_TXINV_SHIFT                  (28U)
/*! TXINV - Transmit Data Inversion
 *  0b0..Not inverted
 *  0b1..Inverted
 */
#define LPUART_CTRL_TXINV(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_TXINV_SHIFT)) & LPUART_CTRL_TXINV_MASK)

#define LPUART_CTRL_TXDIR_MASK                   (0x20000000U)
#define LPUART_CTRL_TXDIR_SHIFT                  (29U)
/*! TXDIR - TXD Pin Direction in Single-Wire Mode
 *  0b0..Input
 *  0b1..Output
 */
#define LPUART_CTRL_TXDIR(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_TXDIR_SHIFT)) & LPUART_CTRL_TXDIR_MASK)

#define LPUART_CTRL_R9T8_MASK                    (0x40000000U)
#define LPUART_CTRL_R9T8_SHIFT                   (30U)
/*! R9T8 - Receive Bit 9 Transmit Bit 8 */
#define LPUART_CTRL_R9T8(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_R9T8_SHIFT)) & LPUART_CTRL_R9T8_MASK)

#define LPUART_CTRL_R8T9_MASK                    (0x80000000U)
#define LPUART_CTRL_R8T9_SHIFT                   (31U)
/*! R8T9 - Receive Bit 8 Transmit Bit 9 */
#define LPUART_CTRL_R8T9(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_CTRL_R8T9_SHIFT)) & LPUART_CTRL_R8T9_MASK)
/*! @} */

/*! @name DATA - Data */
/*! @{ */

#define LPUART_DATA_R0T0_MASK                    (0x1U)
#define LPUART_DATA_R0T0_SHIFT                   (0U)
/*! R0T0 - Read receive FIFO bit 0 or write transmit FIFO bit 0 */
#define LPUART_DATA_R0T0(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R0T0_SHIFT)) & LPUART_DATA_R0T0_MASK)

#define LPUART_DATA_R1T1_MASK                    (0x2U)
#define LPUART_DATA_R1T1_SHIFT                   (1U)
/*! R1T1 - Read receive FIFO bit 1 or write transmit FIFO bit 1 */
#define LPUART_DATA_R1T1(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R1T1_SHIFT)) & LPUART_DATA_R1T1_MASK)

#define LPUART_DATA_R2T2_MASK                    (0x4U)
#define LPUART_DATA_R2T2_SHIFT                   (2U)
/*! R2T2 - Read receive FIFO bit 2 or write transmit FIFO bit 2 */
#define LPUART_DATA_R2T2(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R2T2_SHIFT)) & LPUART_DATA_R2T2_MASK)

#define LPUART_DATA_R3T3_MASK                    (0x8U)
#define LPUART_DATA_R3T3_SHIFT                   (3U)
/*! R3T3 - Read receive FIFO bit 3 or write transmit FIFO bit 3 */
#define LPUART_DATA_R3T3(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R3T3_SHIFT)) & LPUART_DATA_R3T3_MASK)

#define LPUART_DATA_R4T4_MASK                    (0x10U)
#define LPUART_DATA_R4T4_SHIFT                   (4U)
/*! R4T4 - Read receive FIFO bit 4 or write transmit FIFO bit 4 */
#define LPUART_DATA_R4T4(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R4T4_SHIFT)) & LPUART_DATA_R4T4_MASK)

#define LPUART_DATA_R5T5_MASK                    (0x20U)
#define LPUART_DATA_R5T5_SHIFT                   (5U)
/*! R5T5 - Read receive FIFO bit 5 or write transmit FIFO bit 5 */
#define LPUART_DATA_R5T5(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R5T5_SHIFT)) & LPUART_DATA_R5T5_MASK)

#define LPUART_DATA_R6T6_MASK                    (0x40U)
#define LPUART_DATA_R6T6_SHIFT                   (6U)
/*! R6T6 - Read receive FIFO bit 6 or write transmit FIFO bit 6 */
#define LPUART_DATA_R6T6(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R6T6_SHIFT)) & LPUART_DATA_R6T6_MASK)

#define LPUART_DATA_R7T7_MASK                    (0x80U)
#define LPUART_DATA_R7T7_SHIFT                   (7U)
/*! R7T7 - Read receive FIFO bit 7 or write transmit FIFO bit 7 */
#define LPUART_DATA_R7T7(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R7T7_SHIFT)) & LPUART_DATA_R7T7_MASK)

#define LPUART_DATA_R8T8_MASK                    (0x100U)
#define LPUART_DATA_R8T8_SHIFT                   (8U)
/*! R8T8 - Read receive FIFO bit 8 or write transmit FIFO bit 8 */
#define LPUART_DATA_R8T8(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R8T8_SHIFT)) & LPUART_DATA_R8T8_MASK)

#define LPUART_DATA_R9T9_MASK                    (0x200U)
#define LPUART_DATA_R9T9_SHIFT                   (9U)
/*! R9T9 - Read receive FIFO bit 9 or write transmit FIFO bit 9 */
#define LPUART_DATA_R9T9(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_R9T9_SHIFT)) & LPUART_DATA_R9T9_MASK)

#define LPUART_DATA_LINBRK_MASK                  (0x400U)
#define LPUART_DATA_LINBRK_SHIFT                 (10U)
/*! LINBRK - LIN Break
 *  0b0..Not detected
 *  0b1..Detected
 */
#define LPUART_DATA_LINBRK(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_LINBRK_SHIFT)) & LPUART_DATA_LINBRK_MASK)

#define LPUART_DATA_IDLINE_MASK                  (0x800U)
#define LPUART_DATA_IDLINE_SHIFT                 (11U)
/*! IDLINE - Idle Line
 *  0b0..Not idle
 *  0b1..Idle
 */
#define LPUART_DATA_IDLINE(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_IDLINE_SHIFT)) & LPUART_DATA_IDLINE_MASK)

#define LPUART_DATA_RXEMPT_MASK                  (0x1000U)
#define LPUART_DATA_RXEMPT_SHIFT                 (12U)
/*! RXEMPT - Receive Buffer Empty
 *  0b0..Valid data
 *  0b1..Invalid data and empty
 */
#define LPUART_DATA_RXEMPT(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_RXEMPT_SHIFT)) & LPUART_DATA_RXEMPT_MASK)

#define LPUART_DATA_FRETSC_MASK                  (0x2000U)
#define LPUART_DATA_FRETSC_SHIFT                 (13U)
/*! FRETSC - Frame Error Transmit Special Character
 *  0b0..Received without a frame error on reads or transmits a normal character on writes
 *  0b1..Received with a frame error on reads or transmits an idle or break character on writes
 */
#define LPUART_DATA_FRETSC(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_FRETSC_SHIFT)) & LPUART_DATA_FRETSC_MASK)

#define LPUART_DATA_PARITYE_MASK                 (0x4000U)
#define LPUART_DATA_PARITYE_SHIFT                (14U)
/*! PARITYE - Parity Error
 *  0b0..Received without a parity error
 *  0b1..Received with a parity error
 */
#define LPUART_DATA_PARITYE(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_PARITYE_SHIFT)) & LPUART_DATA_PARITYE_MASK)

#define LPUART_DATA_NOISY_MASK                   (0x8000U)
#define LPUART_DATA_NOISY_SHIFT                  (15U)
/*! NOISY - Noisy Data Received
 *  0b0..Received without noise
 *  0b1..Received with noise
 */
#define LPUART_DATA_NOISY(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_DATA_NOISY_SHIFT)) & LPUART_DATA_NOISY_MASK)
/*! @} */

/*! @name MATCH - Match Address */
/*! @{ */

#define LPUART_MATCH_MA1_MASK                    (0x3FFU)
#define LPUART_MATCH_MA1_SHIFT                   (0U)
/*! MA1 - Match Address 1 */
#define LPUART_MATCH_MA1(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_MATCH_MA1_SHIFT)) & LPUART_MATCH_MA1_MASK)

#define LPUART_MATCH_MA2_MASK                    (0x3FF0000U)
#define LPUART_MATCH_MA2_SHIFT                   (16U)
/*! MA2 - Match Address 2 */
#define LPUART_MATCH_MA2(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_MATCH_MA2_SHIFT)) & LPUART_MATCH_MA2_MASK)
/*! @} */

/*! @name MODIR - MODEM IrDA */
/*! @{ */

#define LPUART_MODIR_TXCTSE_MASK                 (0x1U)
#define LPUART_MODIR_TXCTSE_SHIFT                (0U)
/*! TXCTSE - Transmitter CTS Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_MODIR_TXCTSE(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_TXCTSE_SHIFT)) & LPUART_MODIR_TXCTSE_MASK)

#define LPUART_MODIR_TXRTSE_MASK                 (0x2U)
#define LPUART_MODIR_TXRTSE_SHIFT                (1U)
/*! TXRTSE - Transmitter RTS Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_MODIR_TXRTSE(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_TXRTSE_SHIFT)) & LPUART_MODIR_TXRTSE_MASK)

#define LPUART_MODIR_TXRTSPOL_MASK               (0x4U)
#define LPUART_MODIR_TXRTSPOL_SHIFT              (2U)
/*! TXRTSPOL - Transmitter RTS Polarity
 *  0b0..Active low
 *  0b1..Active high
 */
#define LPUART_MODIR_TXRTSPOL(x)                 (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_TXRTSPOL_SHIFT)) & LPUART_MODIR_TXRTSPOL_MASK)

#define LPUART_MODIR_RXRTSE_MASK                 (0x8U)
#define LPUART_MODIR_RXRTSE_SHIFT                (3U)
/*! RXRTSE - Receiver RTS Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_MODIR_RXRTSE(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_RXRTSE_SHIFT)) & LPUART_MODIR_RXRTSE_MASK)

#define LPUART_MODIR_TXCTSC_MASK                 (0x10U)
#define LPUART_MODIR_TXCTSC_SHIFT                (4U)
/*! TXCTSC - Transmit CTS Configuration
 *  0b0..Sampled at the start of each character
 *  0b1..Sampled when the transmitter is idle
 */
#define LPUART_MODIR_TXCTSC(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_TXCTSC_SHIFT)) & LPUART_MODIR_TXCTSC_MASK)

#define LPUART_MODIR_TXCTSSRC_MASK               (0x20U)
#define LPUART_MODIR_TXCTSSRC_SHIFT              (5U)
/*! TXCTSSRC - Transmit CTS Source
 *  0b0..The CTS_B pin
 *  0b1..An internal connection to the receiver address match result
 */
#define LPUART_MODIR_TXCTSSRC(x)                 (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_TXCTSSRC_SHIFT)) & LPUART_MODIR_TXCTSSRC_MASK)

#define LPUART_MODIR_RTSWATER_MASK               (0xF00U)
#define LPUART_MODIR_RTSWATER_SHIFT              (8U)
/*! RTSWATER - Receive RTS Configuration */
#define LPUART_MODIR_RTSWATER(x)                 (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_RTSWATER_SHIFT)) & LPUART_MODIR_RTSWATER_MASK)

#define LPUART_MODIR_TNP_MASK                    (0x30000U)
#define LPUART_MODIR_TNP_SHIFT                   (16U)
/*! TNP - Transmitter Narrow Pulse
 *  0b00..1 / OSR
 *  0b01..2 / OSR
 *  0b10..3 / OSR
 *  0b11..4 / OSR
 */
#define LPUART_MODIR_TNP(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_TNP_SHIFT)) & LPUART_MODIR_TNP_MASK)

#define LPUART_MODIR_IREN_MASK                   (0x40000U)
#define LPUART_MODIR_IREN_SHIFT                  (18U)
/*! IREN - IR Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_MODIR_IREN(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_MODIR_IREN_SHIFT)) & LPUART_MODIR_IREN_MASK)
/*! @} */

/*! @name FIFO - FIFO */
/*! @{ */

#define LPUART_FIFO_RXFIFOSIZE_MASK              (0x7U)
#define LPUART_FIFO_RXFIFOSIZE_SHIFT             (0U)
/*! RXFIFOSIZE - Receive FIFO Buffer Depth
 *  0b000..1
 *  0b001..4
 *  0b010..8
 *  0b011..16
 *  0b100..32
 *  0b101..64
 *  0b110..128
 *  0b111..256
 */
#define LPUART_FIFO_RXFIFOSIZE(x)                (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_RXFIFOSIZE_SHIFT)) & LPUART_FIFO_RXFIFOSIZE_MASK)

#define LPUART_FIFO_RXFE_MASK                    (0x8U)
#define LPUART_FIFO_RXFE_SHIFT                   (3U)
/*! RXFE - Receive FIFO Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_FIFO_RXFE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_RXFE_SHIFT)) & LPUART_FIFO_RXFE_MASK)

#define LPUART_FIFO_TXFIFOSIZE_MASK              (0x70U)
#define LPUART_FIFO_TXFIFOSIZE_SHIFT             (4U)
/*! TXFIFOSIZE - Transmit FIFO Buffer Depth
 *  0b000..1
 *  0b001..4
 *  0b010..8
 *  0b011..16
 *  0b100..32
 *  0b101..64
 *  0b110..128
 *  0b111..256
 */
#define LPUART_FIFO_TXFIFOSIZE(x)                (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_TXFIFOSIZE_SHIFT)) & LPUART_FIFO_TXFIFOSIZE_MASK)

#define LPUART_FIFO_TXFE_MASK                    (0x80U)
#define LPUART_FIFO_TXFE_SHIFT                   (7U)
/*! TXFE - Transmit FIFO Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_FIFO_TXFE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_TXFE_SHIFT)) & LPUART_FIFO_TXFE_MASK)

#define LPUART_FIFO_RXUFE_MASK                   (0x100U)
#define LPUART_FIFO_RXUFE_SHIFT                  (8U)
/*! RXUFE - Receive FIFO Underflow Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_FIFO_RXUFE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_RXUFE_SHIFT)) & LPUART_FIFO_RXUFE_MASK)

#define LPUART_FIFO_TXOFE_MASK                   (0x200U)
#define LPUART_FIFO_TXOFE_SHIFT                  (9U)
/*! TXOFE - Transmit FIFO Overflow Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define LPUART_FIFO_TXOFE(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_TXOFE_SHIFT)) & LPUART_FIFO_TXOFE_MASK)

#define LPUART_FIFO_RXIDEN_MASK                  (0x1C00U)
#define LPUART_FIFO_RXIDEN_SHIFT                 (10U)
/*! RXIDEN - Receiver Idle Empty Enable
 *  0b000..Disable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle
 *  0b001..Enable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle for one character
 *  0b010..Enable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle for two characters
 *  0b011..Enable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle for four characters
 *  0b100..Enable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle for eight characters
 *  0b101..Enable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle for 16 characters
 *  0b110..Enable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle for 32 characters
 *  0b111..Enable STAT[RDRF] to become 1 because of partially filled FIFO when the receiver is idle for 64 characters
 */
#define LPUART_FIFO_RXIDEN(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_RXIDEN_SHIFT)) & LPUART_FIFO_RXIDEN_MASK)

#define LPUART_FIFO_RXFLUSH_MASK                 (0x4000U)
#define LPUART_FIFO_RXFLUSH_SHIFT                (14U)
/*! RXFLUSH - Receive FIFO Flush
 *  0b0..No effect
 *  0b1..All data flushed out
 */
#define LPUART_FIFO_RXFLUSH(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_RXFLUSH_SHIFT)) & LPUART_FIFO_RXFLUSH_MASK)

#define LPUART_FIFO_TXFLUSH_MASK                 (0x8000U)
#define LPUART_FIFO_TXFLUSH_SHIFT                (15U)
/*! TXFLUSH - Transmit FIFO Flush
 *  0b0..No effect
 *  0b1..All data flushed out
 */
#define LPUART_FIFO_TXFLUSH(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_TXFLUSH_SHIFT)) & LPUART_FIFO_TXFLUSH_MASK)

#define LPUART_FIFO_RXUF_MASK                    (0x10000U)
#define LPUART_FIFO_RXUF_SHIFT                   (16U)
/*! RXUF - Receiver FIFO Underflow Flag
 *  0b0..No underflow
 *  0b1..Underflow
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_FIFO_RXUF(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_RXUF_SHIFT)) & LPUART_FIFO_RXUF_MASK)

#define LPUART_FIFO_TXOF_MASK                    (0x20000U)
#define LPUART_FIFO_TXOF_SHIFT                   (17U)
/*! TXOF - Transmitter FIFO Overflow Flag
 *  0b0..No overflow
 *  0b1..Overflow
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_FIFO_TXOF(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_TXOF_SHIFT)) & LPUART_FIFO_TXOF_MASK)

#define LPUART_FIFO_RXEMPT_MASK                  (0x400000U)
#define LPUART_FIFO_RXEMPT_SHIFT                 (22U)
/*! RXEMPT - Receive FIFO Or Buffer Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define LPUART_FIFO_RXEMPT(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_RXEMPT_SHIFT)) & LPUART_FIFO_RXEMPT_MASK)

#define LPUART_FIFO_TXEMPT_MASK                  (0x800000U)
#define LPUART_FIFO_TXEMPT_SHIFT                 (23U)
/*! TXEMPT - Transmit FIFO Or Buffer Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define LPUART_FIFO_TXEMPT(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_FIFO_TXEMPT_SHIFT)) & LPUART_FIFO_TXEMPT_MASK)
/*! @} */

/*! @name WATER - Watermark */
/*! @{ */

#define LPUART_WATER_TXWATER_MASK                (0xFU)
#define LPUART_WATER_TXWATER_SHIFT               (0U)
/*! TXWATER - Transmit Watermark */
#define LPUART_WATER_TXWATER(x)                  (((uint32_t)(((uint32_t)(x)) << LPUART_WATER_TXWATER_SHIFT)) & LPUART_WATER_TXWATER_MASK)

#define LPUART_WATER_TXCOUNT_MASK                (0x1F00U)
#define LPUART_WATER_TXCOUNT_SHIFT               (8U)
/*! TXCOUNT - Transmit Counter */
#define LPUART_WATER_TXCOUNT(x)                  (((uint32_t)(((uint32_t)(x)) << LPUART_WATER_TXCOUNT_SHIFT)) & LPUART_WATER_TXCOUNT_MASK)

#define LPUART_WATER_RXWATER_MASK                (0xF0000U)
#define LPUART_WATER_RXWATER_SHIFT               (16U)
/*! RXWATER - Receive Watermark */
#define LPUART_WATER_RXWATER(x)                  (((uint32_t)(((uint32_t)(x)) << LPUART_WATER_RXWATER_SHIFT)) & LPUART_WATER_RXWATER_MASK)

#define LPUART_WATER_RXCOUNT_MASK                (0x1F000000U)
#define LPUART_WATER_RXCOUNT_SHIFT               (24U)
/*! RXCOUNT - Receive Counter */
#define LPUART_WATER_RXCOUNT(x)                  (((uint32_t)(((uint32_t)(x)) << LPUART_WATER_RXCOUNT_SHIFT)) & LPUART_WATER_RXCOUNT_MASK)
/*! @} */

/*! @name DATARO - Data Read-Only */
/*! @{ */

#define LPUART_DATARO_DATA_MASK                  (0xFFFFU)
#define LPUART_DATARO_DATA_SHIFT                 (0U)
/*! DATA - Receive Data */
#define LPUART_DATARO_DATA(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_DATARO_DATA_SHIFT)) & LPUART_DATARO_DATA_MASK)
/*! @} */

/*! @name MCR - MODEM Control */
/*! @{ */

#define LPUART_MCR_CTS_MASK                      (0x1U)
#define LPUART_MCR_CTS_SHIFT                     (0U)
/*! CTS - Clear To Send
 *  0b0..Disable interrupt
 *  0b1..Enable interrupt
 */
#define LPUART_MCR_CTS(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MCR_CTS_SHIFT)) & LPUART_MCR_CTS_MASK)

#define LPUART_MCR_DSR_MASK                      (0x2U)
#define LPUART_MCR_DSR_SHIFT                     (1U)
/*! DSR - Data Set Ready
 *  0b0..Disable interrupt
 *  0b1..Enable interrupt
 */
#define LPUART_MCR_DSR(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MCR_DSR_SHIFT)) & LPUART_MCR_DSR_MASK)

#define LPUART_MCR_RIN_MASK                      (0x4U)
#define LPUART_MCR_RIN_SHIFT                     (2U)
/*! RIN - Ring Indicator
 *  0b0..Disable interrupt
 *  0b1..Enable interrupt
 */
#define LPUART_MCR_RIN(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MCR_RIN_SHIFT)) & LPUART_MCR_RIN_MASK)

#define LPUART_MCR_DCD_MASK                      (0x8U)
#define LPUART_MCR_DCD_SHIFT                     (3U)
/*! DCD - Data Carrier Detect
 *  0b0..Disable interrupt
 *  0b1..Enable interrupt
 */
#define LPUART_MCR_DCD(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MCR_DCD_SHIFT)) & LPUART_MCR_DCD_MASK)

#define LPUART_MCR_DTR_MASK                      (0x100U)
#define LPUART_MCR_DTR_SHIFT                     (8U)
/*! DTR - Data Terminal Ready
 *  0b0..Logic one
 *  0b1..Logic zero
 */
#define LPUART_MCR_DTR(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MCR_DTR_SHIFT)) & LPUART_MCR_DTR_MASK)

#define LPUART_MCR_RTS_MASK                      (0x200U)
#define LPUART_MCR_RTS_SHIFT                     (9U)
/*! RTS - Request To Send
 *  0b0..Logic one
 *  0b1..Logic zero
 */
#define LPUART_MCR_RTS(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MCR_RTS_SHIFT)) & LPUART_MCR_RTS_MASK)
/*! @} */

/*! @name MSR - MODEM Status */
/*! @{ */

#define LPUART_MSR_DCTS_MASK                     (0x1U)
#define LPUART_MSR_DCTS_SHIFT                    (0U)
/*! DCTS - Delta Clear To Send
 *  0b0..Did not change state
 *  0b1..Changed state
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_MSR_DCTS(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_DCTS_SHIFT)) & LPUART_MSR_DCTS_MASK)

#define LPUART_MSR_DDSR_MASK                     (0x2U)
#define LPUART_MSR_DDSR_SHIFT                    (1U)
/*! DDSR - Delta Data Set Ready
 *  0b0..Did not change state
 *  0b1..Changed state
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_MSR_DDSR(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_DDSR_SHIFT)) & LPUART_MSR_DDSR_MASK)

#define LPUART_MSR_DRI_MASK                      (0x4U)
#define LPUART_MSR_DRI_SHIFT                     (2U)
/*! DRI - Delta Ring Indicator
 *  0b0..Did not change state
 *  0b1..Changed state
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_MSR_DRI(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_DRI_SHIFT)) & LPUART_MSR_DRI_MASK)

#define LPUART_MSR_DDCD_MASK                     (0x8U)
#define LPUART_MSR_DDCD_SHIFT                    (3U)
/*! DDCD - Delta Data Carrier Detect
 *  0b0..Did not change state
 *  0b1..Changed state
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define LPUART_MSR_DDCD(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_DDCD_SHIFT)) & LPUART_MSR_DDCD_MASK)

#define LPUART_MSR_CTS_MASK                      (0x10U)
#define LPUART_MSR_CTS_SHIFT                     (4U)
/*! CTS - Clear To Send
 *  0b0..Logic one
 *  0b1..Logic zero
 */
#define LPUART_MSR_CTS(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_CTS_SHIFT)) & LPUART_MSR_CTS_MASK)

#define LPUART_MSR_DSR_MASK                      (0x20U)
#define LPUART_MSR_DSR_SHIFT                     (5U)
/*! DSR - Data Set Ready
 *  0b0..Logic one
 *  0b1..Logic zero
 */
#define LPUART_MSR_DSR(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_DSR_SHIFT)) & LPUART_MSR_DSR_MASK)

#define LPUART_MSR_RIN_MASK                      (0x40U)
#define LPUART_MSR_RIN_SHIFT                     (6U)
/*! RIN - Ring Indicator
 *  0b0..Logic one
 *  0b1..Logic zero
 */
#define LPUART_MSR_RIN(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_RIN_SHIFT)) & LPUART_MSR_RIN_MASK)

#define LPUART_MSR_DCD_MASK                      (0x80U)
#define LPUART_MSR_DCD_SHIFT                     (7U)
/*! DCD - Data Carrier Detect
 *  0b0..Logic one
 *  0b1..Logic zero
 */
#define LPUART_MSR_DCD(x)                        (((uint32_t)(((uint32_t)(x)) << LPUART_MSR_DCD_SHIFT)) & LPUART_MSR_DCD_MASK)
/*! @} */

/*! @name REIR - Receiver Extended Idle */
/*! @{ */

#define LPUART_REIR_IDTIME_MASK                  (0x3FFFU)
#define LPUART_REIR_IDTIME_SHIFT                 (0U)
/*! IDTIME - Idle Time */
#define LPUART_REIR_IDTIME(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_REIR_IDTIME_SHIFT)) & LPUART_REIR_IDTIME_MASK)
/*! @} */

/*! @name TEIR - Transmitter Extended Idle */
/*! @{ */

#define LPUART_TEIR_IDTIME_MASK                  (0x3FFFU)
#define LPUART_TEIR_IDTIME_SHIFT                 (0U)
/*! IDTIME - Idle Time */
#define LPUART_TEIR_IDTIME(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_TEIR_IDTIME_SHIFT)) & LPUART_TEIR_IDTIME_MASK)
/*! @} */

/*! @name HDCR - Half Duplex Control */
/*! @{ */

#define LPUART_HDCR_TXSTALL_MASK                 (0x1U)
#define LPUART_HDCR_TXSTALL_SHIFT                (0U)
/*! TXSTALL - Transmit Stall
 *  0b0..No effect
 *  0b1..Does not become busy
 */
#define LPUART_HDCR_TXSTALL(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_HDCR_TXSTALL_SHIFT)) & LPUART_HDCR_TXSTALL_MASK)

#define LPUART_HDCR_RXSEL_MASK                   (0x2U)
#define LPUART_HDCR_RXSEL_SHIFT                  (1U)
/*! RXSEL - Receive Select
 *  0b0..RXD
 *  0b1..TXD
 */
#define LPUART_HDCR_RXSEL(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_HDCR_RXSEL_SHIFT)) & LPUART_HDCR_RXSEL_MASK)

#define LPUART_HDCR_RXWRMSK_MASK                 (0x4U)
#define LPUART_HDCR_RXWRMSK_SHIFT                (2U)
/*! RXWRMSK - Receive FIFO Write Mask
 *  0b0..Do not mask
 *  0b1..Mask
 */
#define LPUART_HDCR_RXWRMSK(x)                   (((uint32_t)(((uint32_t)(x)) << LPUART_HDCR_RXWRMSK_SHIFT)) & LPUART_HDCR_RXWRMSK_MASK)

#define LPUART_HDCR_RXMSK_MASK                   (0x8U)
#define LPUART_HDCR_RXMSK_SHIFT                  (3U)
/*! RXMSK - Receive Mask
 *  0b0..Do not mask
 *  0b1..Mask
 */
#define LPUART_HDCR_RXMSK(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_HDCR_RXMSK_SHIFT)) & LPUART_HDCR_RXMSK_MASK)

#define LPUART_HDCR_RTSEXT_MASK                  (0xFF00U)
#define LPUART_HDCR_RTSEXT_SHIFT                 (8U)
/*! RTSEXT - RTS Extended */
#define LPUART_HDCR_RTSEXT(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_HDCR_RTSEXT_SHIFT)) & LPUART_HDCR_RTSEXT_MASK)
/*! @} */

/*! @name TOCR - Timeout Control */
/*! @{ */

#define LPUART_TOCR_TOEN_MASK                    (0xFU)
#define LPUART_TOCR_TOEN_SHIFT                   (0U)
/*! TOEN - Timeout Enable */
#define LPUART_TOCR_TOEN(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_TOCR_TOEN_SHIFT)) & LPUART_TOCR_TOEN_MASK)

#define LPUART_TOCR_TOIE_MASK                    (0xF00U)
#define LPUART_TOCR_TOIE_SHIFT                   (8U)
/*! TOIE - Timeout Interrupt Enable */
#define LPUART_TOCR_TOIE(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_TOCR_TOIE_SHIFT)) & LPUART_TOCR_TOIE_MASK)
/*! @} */

/*! @name TOSR - Timeout Status */
/*! @{ */

#define LPUART_TOSR_TOZ_MASK                     (0xFU)
#define LPUART_TOSR_TOZ_SHIFT                    (0U)
/*! TOZ - Timeout Zero */
#define LPUART_TOSR_TOZ(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_TOSR_TOZ_SHIFT)) & LPUART_TOSR_TOZ_MASK)

#define LPUART_TOSR_TOF_MASK                     (0xF00U)
#define LPUART_TOSR_TOF_SHIFT                    (8U)
/*! TOF - Timeout Flag
 *  0b0000..Not occurred
 *  0b0001..Occurred
 *  0b0000..No effect
 *  0b0001..Clear the flag
 */
#define LPUART_TOSR_TOF(x)                       (((uint32_t)(((uint32_t)(x)) << LPUART_TOSR_TOF_SHIFT)) & LPUART_TOSR_TOF_MASK)
/*! @} */

/*! @name TIMEOUT - Timeout N */
/*! @{ */

#define LPUART_TIMEOUT_TIMEOUT_MASK              (0x3FFFU)
#define LPUART_TIMEOUT_TIMEOUT_SHIFT             (0U)
/*! TIMEOUT - Timeout Value */
#define LPUART_TIMEOUT_TIMEOUT(x)                (((uint32_t)(((uint32_t)(x)) << LPUART_TIMEOUT_TIMEOUT_SHIFT)) & LPUART_TIMEOUT_TIMEOUT_MASK)

#define LPUART_TIMEOUT_CFG_MASK                  (0xC0000000U)
#define LPUART_TIMEOUT_CFG_SHIFT                 (30U)
/*! CFG - Idle Configuration
 *  0b00..Becomes 1 after timeout characters are received
 *  0b01..Becomes 1 when idle for timeout bit clocks
 *  0b10..Becomes 1 when idle for timeout bit clocks following the next character
 *  0b11..Becomes 1 when idle for at least timeout bit clocks, but a new character is detected before the extended idle timeout is reached
 */
#define LPUART_TIMEOUT_CFG(x)                    (((uint32_t)(((uint32_t)(x)) << LPUART_TIMEOUT_CFG_SHIFT)) & LPUART_TIMEOUT_CFG_MASK)
/*! @} */

/* The count of LPUART_TIMEOUT */
#define LPUART_TIMEOUT_COUNT                     (4U)

/*! @name TCBR - Transmit Command Burst */
/*! @{ */

#define LPUART_TCBR_DATA_MASK                    (0xFFFFU)
#define LPUART_TCBR_DATA_SHIFT                   (0U)
/*! DATA - Data */
#define LPUART_TCBR_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << LPUART_TCBR_DATA_SHIFT)) & LPUART_TCBR_DATA_MASK)
/*! @} */

/* The count of LPUART_TCBR */
#define LPUART_TCBR_COUNT                        (128U)

/*! @name TDBR - Transmit Data Burst */
/*! @{ */

#define LPUART_TDBR_DATA0_MASK                   (0xFFU)
#define LPUART_TDBR_DATA0_SHIFT                  (0U)
/*! DATA0 - Data0 */
#define LPUART_TDBR_DATA0(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_TDBR_DATA0_SHIFT)) & LPUART_TDBR_DATA0_MASK)

#define LPUART_TDBR_DATA1_MASK                   (0xFF00U)
#define LPUART_TDBR_DATA1_SHIFT                  (8U)
/*! DATA1 - Data1 */
#define LPUART_TDBR_DATA1(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_TDBR_DATA1_SHIFT)) & LPUART_TDBR_DATA1_MASK)

#define LPUART_TDBR_DATA2_MASK                   (0xFF0000U)
#define LPUART_TDBR_DATA2_SHIFT                  (16U)
/*! DATA2 - Data2 */
#define LPUART_TDBR_DATA2(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_TDBR_DATA2_SHIFT)) & LPUART_TDBR_DATA2_MASK)

#define LPUART_TDBR_DATA3_MASK                   (0xFF000000U)
#define LPUART_TDBR_DATA3_SHIFT                  (24U)
/*! DATA3 - Data3 */
#define LPUART_TDBR_DATA3(x)                     (((uint32_t)(((uint32_t)(x)) << LPUART_TDBR_DATA3_SHIFT)) & LPUART_TDBR_DATA3_MASK)
/*! @} */

/* The count of LPUART_TDBR */
#define LPUART_TDBR_COUNT                        (256U)


/*!
 * @}
 */ /* end of group LPUART_Register_Masks */


/* LPUART - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral LPUART1 base address */
  #define LPUART1_BASE                             (0x54380000u)
  /** Peripheral LPUART1 base address */
  #define LPUART1_BASE_NS                          (0x44380000u)
  /** Peripheral LPUART1 base pointer */
  #define LPUART1                                  ((LPUART_Type *)LPUART1_BASE)
  /** Peripheral LPUART1 base pointer */
  #define LPUART1_NS                               ((LPUART_Type *)LPUART1_BASE_NS)
  /** Peripheral LPUART2 base address */
  #define LPUART2_BASE                             (0x54390000u)
  /** Peripheral LPUART2 base address */
  #define LPUART2_BASE_NS                          (0x44390000u)
  /** Peripheral LPUART2 base pointer */
  #define LPUART2                                  ((LPUART_Type *)LPUART2_BASE)
  /** Peripheral LPUART2 base pointer */
  #define LPUART2_NS                               ((LPUART_Type *)LPUART2_BASE_NS)
  /** Peripheral LPUART3 base address */
  #define LPUART3_BASE                             (0x52570000u)
  /** Peripheral LPUART3 base address */
  #define LPUART3_BASE_NS                          (0x42570000u)
  /** Peripheral LPUART3 base pointer */
  #define LPUART3                                  ((LPUART_Type *)LPUART3_BASE)
  /** Peripheral LPUART3 base pointer */
  #define LPUART3_NS                               ((LPUART_Type *)LPUART3_BASE_NS)
  /** Peripheral LPUART4 base address */
  #define LPUART4_BASE                             (0x52580000u)
  /** Peripheral LPUART4 base address */
  #define LPUART4_BASE_NS                          (0x42580000u)
  /** Peripheral LPUART4 base pointer */
  #define LPUART4                                  ((LPUART_Type *)LPUART4_BASE)
  /** Peripheral LPUART4 base pointer */
  #define LPUART4_NS                               ((LPUART_Type *)LPUART4_BASE_NS)
  /** Peripheral LPUART5 base address */
  #define LPUART5_BASE                             (0x52590000u)
  /** Peripheral LPUART5 base address */
  #define LPUART5_BASE_NS                          (0x42590000u)
  /** Peripheral LPUART5 base pointer */
  #define LPUART5                                  ((LPUART_Type *)LPUART5_BASE)
  /** Peripheral LPUART5 base pointer */
  #define LPUART5_NS                               ((LPUART_Type *)LPUART5_BASE_NS)
  /** Peripheral LPUART6 base address */
  #define LPUART6_BASE                             (0x525A0000u)
  /** Peripheral LPUART6 base address */
  #define LPUART6_BASE_NS                          (0x425A0000u)
  /** Peripheral LPUART6 base pointer */
  #define LPUART6                                  ((LPUART_Type *)LPUART6_BASE)
  /** Peripheral LPUART6 base pointer */
  #define LPUART6_NS                               ((LPUART_Type *)LPUART6_BASE_NS)
  /** Peripheral LPUART7 base address */
  #define LPUART7_BASE                             (0x54570000u)
  /** Peripheral LPUART7 base address */
  #define LPUART7_BASE_NS                          (0x44570000u)
  /** Peripheral LPUART7 base pointer */
  #define LPUART7                                  ((LPUART_Type *)LPUART7_BASE)
  /** Peripheral LPUART7 base pointer */
  #define LPUART7_NS                               ((LPUART_Type *)LPUART7_BASE_NS)
  /** Peripheral LPUART8 base address */
  #define LPUART8_BASE                             (0x52DA0000u)
  /** Peripheral LPUART8 base address */
  #define LPUART8_BASE_NS                          (0x42DA0000u)
  /** Peripheral LPUART8 base pointer */
  #define LPUART8                                  ((LPUART_Type *)LPUART8_BASE)
  /** Peripheral LPUART8 base pointer */
  #define LPUART8_NS                               ((LPUART_Type *)LPUART8_BASE_NS)
  /** Array initializer of LPUART peripheral base addresses */
  #define LPUART_BASE_ADDRS                        { 0u, LPUART1_BASE, LPUART2_BASE, LPUART3_BASE, LPUART4_BASE, LPUART5_BASE, LPUART6_BASE, LPUART7_BASE, LPUART8_BASE }
  /** Array initializer of LPUART peripheral base pointers */
  #define LPUART_BASE_PTRS                         { (LPUART_Type *)0u, LPUART1, LPUART2, LPUART3, LPUART4, LPUART5, LPUART6, LPUART7, LPUART8 }
  /** Array initializer of LPUART peripheral base addresses */
  #define LPUART_BASE_ADDRS_NS                     { 0u, LPUART1_BASE_NS, LPUART2_BASE_NS, LPUART3_BASE_NS, LPUART4_BASE_NS, LPUART5_BASE_NS, LPUART6_BASE_NS, LPUART7_BASE_NS, LPUART8_BASE_NS }
  /** Array initializer of LPUART peripheral base pointers */
  #define LPUART_BASE_PTRS_NS                      { (LPUART_Type *)0u, LPUART1_NS, LPUART2_NS, LPUART3_NS, LPUART4_NS, LPUART5_NS, LPUART6_NS, LPUART7_NS, LPUART8_NS }
#else
  /** Peripheral LPUART1 base address */
  #define LPUART1_BASE                             (0x44380000u)
  /** Peripheral LPUART1 base pointer */
  #define LPUART1                                  ((LPUART_Type *)LPUART1_BASE)
  /** Peripheral LPUART2 base address */
  #define LPUART2_BASE                             (0x44390000u)
  /** Peripheral LPUART2 base pointer */
  #define LPUART2                                  ((LPUART_Type *)LPUART2_BASE)
  /** Peripheral LPUART3 base address */
  #define LPUART3_BASE                             (0x42570000u)
  /** Peripheral LPUART3 base pointer */
  #define LPUART3                                  ((LPUART_Type *)LPUART3_BASE)
  /** Peripheral LPUART4 base address */
  #define LPUART4_BASE                             (0x42580000u)
  /** Peripheral LPUART4 base pointer */
  #define LPUART4                                  ((LPUART_Type *)LPUART4_BASE)
  /** Peripheral LPUART5 base address */
  #define LPUART5_BASE                             (0x42590000u)
  /** Peripheral LPUART5 base pointer */
  #define LPUART5                                  ((LPUART_Type *)LPUART5_BASE)
  /** Peripheral LPUART6 base address */
  #define LPUART6_BASE                             (0x425A0000u)
  /** Peripheral LPUART6 base pointer */
  #define LPUART6                                  ((LPUART_Type *)LPUART6_BASE)
  /** Peripheral LPUART7 base address */
  #define LPUART7_BASE                             (0x44570000u)
  /** Peripheral LPUART7 base pointer */
  #define LPUART7                                  ((LPUART_Type *)LPUART7_BASE)
  /** Peripheral LPUART8 base address */
  #define LPUART8_BASE                             (0x42DA0000u)
  /** Peripheral LPUART8 base pointer */
  #define LPUART8                                  ((LPUART_Type *)LPUART8_BASE)
  /** Array initializer of LPUART peripheral base addresses */
  #define LPUART_BASE_ADDRS                        { 0u, LPUART1_BASE, LPUART2_BASE, LPUART3_BASE, LPUART4_BASE, LPUART5_BASE, LPUART6_BASE, LPUART7_BASE, LPUART8_BASE }
  /** Array initializer of LPUART peripheral base pointers */
  #define LPUART_BASE_PTRS                         { (LPUART_Type *)0u, LPUART1, LPUART2, LPUART3, LPUART4, LPUART5, LPUART6, LPUART7, LPUART8 }
#endif
/** Interrupt vectors for the LPUART peripheral type */
#define LPUART_RX_TX_IRQS                        { NotAvail_IRQn, LPUART1_IRQn, LPUART2_IRQn, LPUART3_IRQn, LPUART4_IRQn, LPUART5_IRQn, LPUART6_IRQn, LPUART7_IRQn, LPUART8_IRQn }

/*!
 * @}
 */ /* end of group LPUART_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- MECC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup MECC_Peripheral_Access_Layer MECC Peripheral Access Layer
 * @{
 */

/** MECC - Register Layout Typedef */
typedef struct {
  __IO uint32_t ERR_STATUS;                        /**< Error Interrupt Status Register, offset: 0x0 */
  __IO uint32_t ERR_STAT_EN;                       /**< Error Interrupt Status Enable Register, offset: 0x4 */
  __IO uint32_t ERR_SIG_EN;                        /**< Error Interrupt Enable Register, offset: 0x8 */
  __IO uint32_t ERR_DATA_INJ_LOW0;                 /**< Error Injection On LOW 32 bits Of OCRAM Bank0 Write Data, offset: 0xC */
  __IO uint32_t ERR_DATA_INJ_HIGH0;                /**< Error Injection On HIGH 32 bits Of OCRAM Bank0 Write Data, offset: 0x10 */
  __IO uint32_t ERR_ECC_INJ0;                      /**< Error Injection On 8 bits ECC code Of OCRAM Bank0 Write Data, offset: 0x14 */
  __IO uint32_t ERR_DATA_INJ_LOW1;                 /**< Error Injection On LOW 32 bits Of OCRAM Bank1 Write Data, offset: 0x18 */
  __IO uint32_t ERR_DATA_INJ_HIGH1;                /**< Error Injection On HIGH 32 bits Of OCRAM Bank1 Write Data, offset: 0x1C */
  __IO uint32_t ERR_ECC_INJ1;                      /**< Error Injection On 8 bits ECC code Of OCRAM Bank1 Write Data, offset: 0x20 */
  __IO uint32_t ERR_DATA_INJ_LOW2;                 /**< Error Injection On LOW 32 bits Of OCRAM Bank2 Write Data, offset: 0x24 */
  __IO uint32_t ERR_DATA_INJ_HIGH2;                /**< Error Injection On HIGH 32 bits Of OCRAM Bank2 Write Data, offset: 0x28 */
  __IO uint32_t ERR_ECC_INJ2;                      /**< Error Injection On 8 bits ECC code Of OCRAM Bank2 Write Data, offset: 0x2C */
  __IO uint32_t ERR_DATA_INJ_LOW3;                 /**< Error Injection On LOW 32 bits Of OCRAM Bank3 Write Data, offset: 0x30 */
  __IO uint32_t ERR_DATA_INJ_HIGH3;                /**< Error Injection On HIGH 32 bits Of OCRAM Bank3 Write Data, offset: 0x34 */
  __IO uint32_t ERR_ECC_INJ3;                      /**< Error Injection On 8 bits ECC code Of OCRAM Bank3 Write Data, offset: 0x38 */
  __I  uint32_t SINGLE_ERR_ADDR_ECC0;              /**< Single Error Address And ECC code On OCRAM Bank0, offset: 0x3C */
  __I  uint32_t SINGLE_ERR_DATA_LOW0;              /**< LOW 32 Bits Single Error Read Data On OCRAM Bank0, offset: 0x40 */
  __I  uint32_t SINGLE_ERR_DATA_HIGH0;             /**< HIGH 32 Bits Single Error Read Data On OCRAM Bank0, offset: 0x44 */
  __I  uint32_t SINGLE_ERR_POS_LOW0;               /**< LOW Single Error Bit Position On OCRAM Bank0, offset: 0x48 */
  __I  uint32_t SINGLE_ERR_POS_HIGH0;              /**< HIGH Single Error Bit Position On OCRAM Bank0, offset: 0x4C */
  __I  uint32_t SINGLE_ERR_ADDR_ECC1;              /**< Single Error Address And ECC code On OCRAM Bank1, offset: 0x50 */
  __I  uint32_t SINGLE_ERR_DATA_LOW1;              /**< LOW 32 Bits Single Error Read Data On OCRAM Bank1, offset: 0x54 */
  __I  uint32_t SINGLE_ERR_DATA_HIGH1;             /**< HIGH 32 Bits Single Error Read Data On OCRAM Bank1, offset: 0x58 */
  __I  uint32_t SINGLE_ERR_POS_LOW1;               /**< LOW Single Error Bit Position On OCRAM Bank1, offset: 0x5C */
  __I  uint32_t SINGLE_ERR_POS_HIGH1;              /**< HIGH Single Error Bit Position On OCRAM Bank1, offset: 0x60 */
  __I  uint32_t SINGLE_ERR_ADDR_ECC2;              /**< Single Error Address And ECC code On OCRAM Bank2, offset: 0x64 */
  __I  uint32_t SINGLE_ERR_DATA_LOW2;              /**< LOW 32 Bits Single Error Read Data On OCRAM Bank2, offset: 0x68 */
  __I  uint32_t SINGLE_ERR_DATA_HIGH2;             /**< HIGH 32 Bits Single Error Read Data On OCRAM Bank2, offset: 0x6C */
  __I  uint32_t SINGLE_ERR_POS_LOW2;               /**< LOW Single Error Bit Position On OCRAM Bank2, offset: 0x70 */
  __I  uint32_t SINGLE_ERR_POS_HIGH2;              /**< HIGH Single Error Bit Position On OCRAM Bank2, offset: 0x74 */
  __I  uint32_t SINGLE_ERR_ADDR_ECC3;              /**< Single Error Address And ECC code On OCRAM Bank3, offset: 0x78 */
  __I  uint32_t SINGLE_ERR_DATA_LOW3;              /**< LOW 32 Bits Single Error Read Data On OCRAM Bank3, offset: 0x7C */
  __I  uint32_t SINGLE_ERR_DATA_HIGH3;             /**< HIGH 32 Bits Single Error Read Data On OCRAM Bank3, offset: 0x80 */
  __I  uint32_t SINGLE_ERR_POS_LOW3;               /**< LOW Single Error Bit Position On OCRAM Bank3, offset: 0x84 */
  __I  uint32_t SINGLE_ERR_POS_HIGH3;              /**< HIGH Single Error Bit Position On OCRAM Bank3, offset: 0x88 */
  __I  uint32_t MULTI_ERR_ADDR_ECC0;               /**< Multiple Error Address And ECC code On OCRAM Bank0, offset: 0x8C */
  __I  uint32_t MULTI_ERR_DATA_LOW0;               /**< LOW 32 Bits Multiple Error Read Data On OCRAM Bank0, offset: 0x90 */
  __I  uint32_t MULTI_ERR_DATA_HIGH0;              /**< HIGH 32 Bits Multiple Error Read Data On OCRAM Bank0, offset: 0x94 */
  __I  uint32_t MULTI_ERR_ADDR_ECC1;               /**< Multiple Error Address And ECC code On OCRAM Bank1, offset: 0x98 */
  __I  uint32_t MULTI_ERR_DATA_LOW1;               /**< LOW 32 Bits Multiple Error Read Data On OCRAM Bank1, offset: 0x9C */
  __I  uint32_t MULTI_ERR_DATA_HIGH1;              /**< HIGH 32 Bits Multiple Error Read Data On OCRAM Bank1, offset: 0xA0 */
  __I  uint32_t MULTI_ERR_ADDR_ECC2;               /**< Multiple Error Address And ECC code On OCRAM Bank2, offset: 0xA4 */
  __I  uint32_t MULTI_ERR_DATA_LOW2;               /**< LOW 32 Bits Multiple Error Read Data On OCRAM Bank2, offset: 0xA8 */
  __I  uint32_t MULTI_ERR_DATA_HIGH2;              /**< HIGH 32 Bits Multiple Error Read Data On OCRAM Bank2, offset: 0xAC */
  __I  uint32_t MULTI_ERR_ADDR_ECC3;               /**< Multiple Error Address And ECC code On OCRAM Bank3, offset: 0xB0 */
  __I  uint32_t MULTI_ERR_DATA_LOW3;               /**< LOW 32 Bits Multiple Error Read Data On OCRAM Bank3, offset: 0xB4 */
  __I  uint32_t MULTI_ERR_DATA_HIGH3;              /**< HIGH 32 Bits Multiple Error Read Data On OCRAM Bank3, offset: 0xB8 */
       uint8_t RESERVED_0[68];
  __IO uint32_t PIPE_ECC_EN;                       /**< OCRAM Pipeline And ECC Enable, offset: 0x100 */
  __I  uint32_t PENDING_STAT;                      /**< Pending Status, offset: 0x104 */
} MECC_Type;

/* ----------------------------------------------------------------------------
   -- MECC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup MECC_Register_Masks MECC Register Masks
 * @{
 */

/*! @name ERR_STATUS - Error Interrupt Status Register */
/*! @{ */

#define MECC_ERR_STATUS_SINGLE_ERR0_MASK         (0x1U)
#define MECC_ERR_STATUS_SINGLE_ERR0_SHIFT        (0U)
/*! SINGLE_ERR0 - Single Bit Error On OCRAM Bank0
 *  0b0..Single bit error does not happen on OCRAM bank0.
 *  0b1..Single bit error happens on OCRAM bank0.
 */
#define MECC_ERR_STATUS_SINGLE_ERR0(x)           (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_SINGLE_ERR0_SHIFT)) & MECC_ERR_STATUS_SINGLE_ERR0_MASK)

#define MECC_ERR_STATUS_SINGLE_ERR1_MASK         (0x2U)
#define MECC_ERR_STATUS_SINGLE_ERR1_SHIFT        (1U)
/*! SINGLE_ERR1 - Single Bit Error On OCRAM Bank1
 *  0b0..Single bit error does not happen on OCRAM bank1.
 *  0b1..Single bit error happens on OCRAM bank1.
 */
#define MECC_ERR_STATUS_SINGLE_ERR1(x)           (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_SINGLE_ERR1_SHIFT)) & MECC_ERR_STATUS_SINGLE_ERR1_MASK)

#define MECC_ERR_STATUS_SINGLE_ERR2_MASK         (0x4U)
#define MECC_ERR_STATUS_SINGLE_ERR2_SHIFT        (2U)
/*! SINGLE_ERR2 - Single Bit Error On OCRAM Bank2
 *  0b0..Single bit error does not happen on OCRAM bank2.
 *  0b1..Single bit error happens on OCRAM bank2.
 */
#define MECC_ERR_STATUS_SINGLE_ERR2(x)           (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_SINGLE_ERR2_SHIFT)) & MECC_ERR_STATUS_SINGLE_ERR2_MASK)

#define MECC_ERR_STATUS_SINGLE_ERR3_MASK         (0x8U)
#define MECC_ERR_STATUS_SINGLE_ERR3_SHIFT        (3U)
/*! SINGLE_ERR3 - Single Bit Error On OCRAM Bank3
 *  0b0..Single bit error does not happen on OCRAM bank3.
 *  0b1..Single bit error happens on OCRAM bank3.
 */
#define MECC_ERR_STATUS_SINGLE_ERR3(x)           (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_SINGLE_ERR3_SHIFT)) & MECC_ERR_STATUS_SINGLE_ERR3_MASK)

#define MECC_ERR_STATUS_MULTI_ERR0_MASK          (0x10U)
#define MECC_ERR_STATUS_MULTI_ERR0_SHIFT         (4U)
/*! MULTI_ERR0 - Multiple Bits Error On OCRAM Bank0
 *  0b0..Multiple bits error does not happen on OCRAM bank0.
 *  0b1..Multiple bits error happens on OCRAM bank0.
 */
#define MECC_ERR_STATUS_MULTI_ERR0(x)            (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_MULTI_ERR0_SHIFT)) & MECC_ERR_STATUS_MULTI_ERR0_MASK)

#define MECC_ERR_STATUS_MULTI_ERR1_MASK          (0x20U)
#define MECC_ERR_STATUS_MULTI_ERR1_SHIFT         (5U)
/*! MULTI_ERR1 - Multiple Bits Error On OCRAM Bank1
 *  0b0..Multiple bits error does not happen on OCRAM bank1.
 *  0b1..Multiple bits error happens on OCRAM bank1.
 */
#define MECC_ERR_STATUS_MULTI_ERR1(x)            (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_MULTI_ERR1_SHIFT)) & MECC_ERR_STATUS_MULTI_ERR1_MASK)

#define MECC_ERR_STATUS_MULTI_ERR2_MASK          (0x40U)
#define MECC_ERR_STATUS_MULTI_ERR2_SHIFT         (6U)
/*! MULTI_ERR2 - Multiple Bits Error On OCRAM Bank2
 *  0b0..Multiple bits error does not happen on OCRAM bank2.
 *  0b1..Multiple bits error happens on OCRAM bank2.
 */
#define MECC_ERR_STATUS_MULTI_ERR2(x)            (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_MULTI_ERR2_SHIFT)) & MECC_ERR_STATUS_MULTI_ERR2_MASK)

#define MECC_ERR_STATUS_MULTI_ERR3_MASK          (0x80U)
#define MECC_ERR_STATUS_MULTI_ERR3_SHIFT         (7U)
/*! MULTI_ERR3 - Multiple Bits Error On OCRAM Bank3
 *  0b0..Multiple bits error does not happen on OCRAM bank3.
 *  0b1..Multiple bits error happens on OCRAM bank3.
 */
#define MECC_ERR_STATUS_MULTI_ERR3(x)            (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_MULTI_ERR3_SHIFT)) & MECC_ERR_STATUS_MULTI_ERR3_MASK)

#define MECC_ERR_STATUS_STRB_ERR0_MASK           (0x100U)
#define MECC_ERR_STATUS_STRB_ERR0_SHIFT          (8U)
/*! STRB_ERR0 - AXI Strobe Error On OCRAM Bank0
 *  0b0..AXI strobe error does not happen on OCRAM bank0.
 *  0b1..AXI strobe error happens on OCRAM bank0.
 */
#define MECC_ERR_STATUS_STRB_ERR0(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_STRB_ERR0_SHIFT)) & MECC_ERR_STATUS_STRB_ERR0_MASK)

#define MECC_ERR_STATUS_STRB_ERR1_MASK           (0x200U)
#define MECC_ERR_STATUS_STRB_ERR1_SHIFT          (9U)
/*! STRB_ERR1 - AXI Strobe Error On OCRAM Bank1
 *  0b0..AXI strobe error does not happen on OCRAM bank1.
 *  0b1..AXI strobe error happens on OCRAM bank1.
 */
#define MECC_ERR_STATUS_STRB_ERR1(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_STRB_ERR1_SHIFT)) & MECC_ERR_STATUS_STRB_ERR1_MASK)

#define MECC_ERR_STATUS_STRB_ERR2_MASK           (0x400U)
#define MECC_ERR_STATUS_STRB_ERR2_SHIFT          (10U)
/*! STRB_ERR2 - AXI Strobe Error On OCRAM Bank2
 *  0b0..AXI strobe error does not happen on OCRAM bank2.
 *  0b1..AXI strobe error happens on OCRAM bank2.
 */
#define MECC_ERR_STATUS_STRB_ERR2(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_STRB_ERR2_SHIFT)) & MECC_ERR_STATUS_STRB_ERR2_MASK)

#define MECC_ERR_STATUS_STRB_ERR3_MASK           (0x800U)
#define MECC_ERR_STATUS_STRB_ERR3_SHIFT          (11U)
/*! STRB_ERR3 - AXI Strobe Error On OCRAM Bank3
 *  0b0..AXI strobe error does not happen on OCRAM bank3.
 *  0b1..AXI strobe error happens on OCRAM bank3.
 */
#define MECC_ERR_STATUS_STRB_ERR3(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_STRB_ERR3_SHIFT)) & MECC_ERR_STATUS_STRB_ERR3_MASK)

#define MECC_ERR_STATUS_ADDR_ERR0_MASK           (0x1000U)
#define MECC_ERR_STATUS_ADDR_ERR0_SHIFT          (12U)
/*! ADDR_ERR0 - OCRAM Access Error On Bank0
 *  0b0..OCRAM access error does not happen on OCRAM bank0.
 *  0b1..OCRAM access error happens on OCRAM bank0.
 */
#define MECC_ERR_STATUS_ADDR_ERR0(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_ADDR_ERR0_SHIFT)) & MECC_ERR_STATUS_ADDR_ERR0_MASK)

#define MECC_ERR_STATUS_ADDR_ERR1_MASK           (0x2000U)
#define MECC_ERR_STATUS_ADDR_ERR1_SHIFT          (13U)
/*! ADDR_ERR1 - OCRAM Access Error On Bank1
 *  0b0..OCRAM access error does not happen on OCRAM bank1.
 *  0b1..OCRAM access error happens on OCRAM bank1.
 */
#define MECC_ERR_STATUS_ADDR_ERR1(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_ADDR_ERR1_SHIFT)) & MECC_ERR_STATUS_ADDR_ERR1_MASK)

#define MECC_ERR_STATUS_ADDR_ERR2_MASK           (0x4000U)
#define MECC_ERR_STATUS_ADDR_ERR2_SHIFT          (14U)
/*! ADDR_ERR2 - OCRAM Access Error On Bank2
 *  0b0..OCRAM access error does not happen on OCRAM bank2.
 *  0b1..OCRAM access error happens on OCRAM bank2.
 */
#define MECC_ERR_STATUS_ADDR_ERR2(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_ADDR_ERR2_SHIFT)) & MECC_ERR_STATUS_ADDR_ERR2_MASK)

#define MECC_ERR_STATUS_ADDR_ERR3_MASK           (0x8000U)
#define MECC_ERR_STATUS_ADDR_ERR3_SHIFT          (15U)
/*! ADDR_ERR3 - OCRAM Access Error On Bank3
 *  0b0..OCRAM access error does not happen on OCRAM bank3.
 *  0b1..OCRAM access error happens on OCRAM bank3.
 */
#define MECC_ERR_STATUS_ADDR_ERR3(x)             (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STATUS_ADDR_ERR3_SHIFT)) & MECC_ERR_STATUS_ADDR_ERR3_MASK)
/*! @} */

/*! @name ERR_STAT_EN - Error Interrupt Status Enable Register */
/*! @{ */

#define MECC_ERR_STAT_EN_SINGLE_ERR0_STAT_EN_MASK (0x1U)
#define MECC_ERR_STAT_EN_SINGLE_ERR0_STAT_EN_SHIFT (0U)
/*! SINGLE_ERR0_STAT_EN - Single Bit Error Status Enable On OCRAM Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_SINGLE_ERR0_STAT_EN(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_SINGLE_ERR0_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_SINGLE_ERR0_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_SINGLE_ERR1_STAT_EN_MASK (0x2U)
#define MECC_ERR_STAT_EN_SINGLE_ERR1_STAT_EN_SHIFT (1U)
/*! SINGLE_ERR1_STAT_EN - Single Bit Error Status Enable On OCRAM Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_SINGLE_ERR1_STAT_EN(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_SINGLE_ERR1_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_SINGLE_ERR1_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_SINGLE_ERR2_STAT_EN_MASK (0x4U)
#define MECC_ERR_STAT_EN_SINGLE_ERR2_STAT_EN_SHIFT (2U)
/*! SINGLE_ERR2_STAT_EN - Single Bit Error Status Enable On OCRAM Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_SINGLE_ERR2_STAT_EN(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_SINGLE_ERR2_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_SINGLE_ERR2_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_SINGLE_ERR3_STAT_EN_MASK (0x8U)
#define MECC_ERR_STAT_EN_SINGLE_ERR3_STAT_EN_SHIFT (3U)
/*! SINGLE_ERR3_STAT_EN - Single Bit Error Status Enable On OCRAM Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_SINGLE_ERR3_STAT_EN(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_SINGLE_ERR3_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_SINGLE_ERR3_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_MULTI_ERR0_STAT_EN_MASK (0x10U)
#define MECC_ERR_STAT_EN_MULTI_ERR0_STAT_EN_SHIFT (4U)
/*! MULTI_ERR0_STAT_EN - Multiple Bits Error Status Enable On OCRAM Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_MULTI_ERR0_STAT_EN(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_MULTI_ERR0_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_MULTI_ERR0_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_MULTI_ERR1_STAT_EN_MASK (0x20U)
#define MECC_ERR_STAT_EN_MULTI_ERR1_STAT_EN_SHIFT (5U)
/*! MULTI_ERR1_STAT_EN - Multiple Bits Error Status Enable On OCRAM Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_MULTI_ERR1_STAT_EN(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_MULTI_ERR1_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_MULTI_ERR1_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_MULTI_ERR2_STAT_EN_MASK (0x40U)
#define MECC_ERR_STAT_EN_MULTI_ERR2_STAT_EN_SHIFT (6U)
/*! MULTI_ERR2_STAT_EN - Multiple Bits Error Status Enable On OCRAM Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_MULTI_ERR2_STAT_EN(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_MULTI_ERR2_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_MULTI_ERR2_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_MULTI_ERR3_STAT_EN_MASK (0x80U)
#define MECC_ERR_STAT_EN_MULTI_ERR3_STAT_EN_SHIFT (7U)
/*! MULTI_ERR3_STAT_EN - Multiple Bits Error Status Enable On OCRAM Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_MULTI_ERR3_STAT_EN(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_MULTI_ERR3_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_MULTI_ERR3_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_STRB_ERR0_STAT_EN_MASK  (0x100U)
#define MECC_ERR_STAT_EN_STRB_ERR0_STAT_EN_SHIFT (8U)
/*! STRB_ERR0_STAT_EN - AXI Strobe Error Status Enable On OCRAM Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_STRB_ERR0_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_STRB_ERR0_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_STRB_ERR0_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_STRB_ERR1_STAT_EN_MASK  (0x200U)
#define MECC_ERR_STAT_EN_STRB_ERR1_STAT_EN_SHIFT (9U)
/*! STRB_ERR1_STAT_EN - AXI Strobe Error Status Enable On OCRAM Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_STRB_ERR1_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_STRB_ERR1_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_STRB_ERR1_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_STRB_ERR2_STAT_EN_MASK  (0x400U)
#define MECC_ERR_STAT_EN_STRB_ERR2_STAT_EN_SHIFT (10U)
/*! STRB_ERR2_STAT_EN - AXI Strobe Error Status Enable On OCRAM Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_STRB_ERR2_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_STRB_ERR2_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_STRB_ERR2_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_STRB_ERR3_STAT_EN_MASK  (0x800U)
#define MECC_ERR_STAT_EN_STRB_ERR3_STAT_EN_SHIFT (11U)
/*! STRB_ERR3_STAT_EN - AXI Strobe Error Status Enable On OCRAM Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_STRB_ERR3_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_STRB_ERR3_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_STRB_ERR3_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_ADDR_ERR0_STAT_EN_MASK  (0x1000U)
#define MECC_ERR_STAT_EN_ADDR_ERR0_STAT_EN_SHIFT (12U)
/*! ADDR_ERR0_STAT_EN - OCRAM Access Error Status Enable On Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_ADDR_ERR0_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_ADDR_ERR0_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_ADDR_ERR0_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_ADDR_ERR1_STAT_EN_MASK  (0x2000U)
#define MECC_ERR_STAT_EN_ADDR_ERR1_STAT_EN_SHIFT (13U)
/*! ADDR_ERR1_STAT_EN - OCRAM Access Error Status Enable On Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_ADDR_ERR1_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_ADDR_ERR1_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_ADDR_ERR1_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_ADDR_ERR2_STAT_EN_MASK  (0x4000U)
#define MECC_ERR_STAT_EN_ADDR_ERR2_STAT_EN_SHIFT (14U)
/*! ADDR_ERR2_STAT_EN - OCRAM Access Error Status Enable On Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_ADDR_ERR2_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_ADDR_ERR2_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_ADDR_ERR2_STAT_EN_MASK)

#define MECC_ERR_STAT_EN_ADDR_ERR3_STAT_EN_MASK  (0x8000U)
#define MECC_ERR_STAT_EN_ADDR_ERR3_STAT_EN_SHIFT (15U)
/*! ADDR_ERR3_STAT_EN - OCRAM Access Error Status Enable On Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_STAT_EN_ADDR_ERR3_STAT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_STAT_EN_ADDR_ERR3_STAT_EN_SHIFT)) & MECC_ERR_STAT_EN_ADDR_ERR3_STAT_EN_MASK)
/*! @} */

/*! @name ERR_SIG_EN - Error Interrupt Enable Register */
/*! @{ */

#define MECC_ERR_SIG_EN_SINGLE_ERR0_SIG_EN_MASK  (0x1U)
#define MECC_ERR_SIG_EN_SINGLE_ERR0_SIG_EN_SHIFT (0U)
/*! SINGLE_ERR0_SIG_EN - Single Bit Error Interrupt Enable On OCRAM Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_SINGLE_ERR0_SIG_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_SINGLE_ERR0_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_SINGLE_ERR0_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_SINGLE_ERR1_SIG_EN_MASK  (0x2U)
#define MECC_ERR_SIG_EN_SINGLE_ERR1_SIG_EN_SHIFT (1U)
/*! SINGLE_ERR1_SIG_EN - Single Bit Error Interrupt Enable On OCRAM Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_SINGLE_ERR1_SIG_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_SINGLE_ERR1_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_SINGLE_ERR1_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_SINGLE_ERR2_SIG_EN_MASK  (0x4U)
#define MECC_ERR_SIG_EN_SINGLE_ERR2_SIG_EN_SHIFT (2U)
/*! SINGLE_ERR2_SIG_EN - Single Bit Error Interrupt Enable On OCRAM Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_SINGLE_ERR2_SIG_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_SINGLE_ERR2_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_SINGLE_ERR2_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_SINGLE_ERR3_SIG_EN_MASK  (0x8U)
#define MECC_ERR_SIG_EN_SINGLE_ERR3_SIG_EN_SHIFT (3U)
/*! SINGLE_ERR3_SIG_EN - Single Bit Error Interrupt Enable On OCRAM Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_SINGLE_ERR3_SIG_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_SINGLE_ERR3_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_SINGLE_ERR3_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_MULTI_ERR0_SIG_EN_MASK   (0x10U)
#define MECC_ERR_SIG_EN_MULTI_ERR0_SIG_EN_SHIFT  (4U)
/*! MULTI_ERR0_SIG_EN - Multiple Bits Error Interrupt Enable On OCRAM Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_MULTI_ERR0_SIG_EN(x)     (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_MULTI_ERR0_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_MULTI_ERR0_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_MULTI_ERR1_SIG_EN_MASK   (0x20U)
#define MECC_ERR_SIG_EN_MULTI_ERR1_SIG_EN_SHIFT  (5U)
/*! MULTI_ERR1_SIG_EN - Multiple Bits Error Interrupt Enable On OCRAM Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_MULTI_ERR1_SIG_EN(x)     (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_MULTI_ERR1_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_MULTI_ERR1_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_MULTI_ERR2_SIG_EN_MASK   (0x40U)
#define MECC_ERR_SIG_EN_MULTI_ERR2_SIG_EN_SHIFT  (6U)
/*! MULTI_ERR2_SIG_EN - Multiple Bits Error Interrupt Enable On OCRAM Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_MULTI_ERR2_SIG_EN(x)     (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_MULTI_ERR2_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_MULTI_ERR2_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_MULTI_ERR3_SIG_EN_MASK   (0x80U)
#define MECC_ERR_SIG_EN_MULTI_ERR3_SIG_EN_SHIFT  (7U)
/*! MULTI_ERR3_SIG_EN - Multiple Bits Error Interrupt Enable On OCRAM Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_MULTI_ERR3_SIG_EN(x)     (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_MULTI_ERR3_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_MULTI_ERR3_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_STRB_ERR0_SIG_EN_MASK    (0x100U)
#define MECC_ERR_SIG_EN_STRB_ERR0_SIG_EN_SHIFT   (8U)
/*! STRB_ERR0_SIG_EN - AXI Strobe Error Interrupt Enable On OCRAM Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_STRB_ERR0_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_STRB_ERR0_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_STRB_ERR0_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_STRB_ERR1_SIG_EN_MASK    (0x200U)
#define MECC_ERR_SIG_EN_STRB_ERR1_SIG_EN_SHIFT   (9U)
/*! STRB_ERR1_SIG_EN - AXI Strobe Error Interrupt Enable On OCRAM Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_STRB_ERR1_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_STRB_ERR1_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_STRB_ERR1_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_STRB_ERR2_SIG_EN_MASK    (0x400U)
#define MECC_ERR_SIG_EN_STRB_ERR2_SIG_EN_SHIFT   (10U)
/*! STRB_ERR2_SIG_EN - AXI Strobe Error Interrupt Enable On OCRAM Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_STRB_ERR2_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_STRB_ERR2_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_STRB_ERR2_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_STRB_ERR3_SIG_EN_MASK    (0x800U)
#define MECC_ERR_SIG_EN_STRB_ERR3_SIG_EN_SHIFT   (11U)
/*! STRB_ERR3_SIG_EN - AXI Strobe Error Interrupt Enable On OCRAM Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_STRB_ERR3_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_STRB_ERR3_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_STRB_ERR3_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_ADDR_ERR0_SIG_EN_MASK    (0x1000U)
#define MECC_ERR_SIG_EN_ADDR_ERR0_SIG_EN_SHIFT   (12U)
/*! ADDR_ERR0_SIG_EN - OCRAM Access Error Interrupt Enable On Bank0
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_ADDR_ERR0_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_ADDR_ERR0_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_ADDR_ERR0_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_ADDR_ERR1_SIG_EN_MASK    (0x2000U)
#define MECC_ERR_SIG_EN_ADDR_ERR1_SIG_EN_SHIFT   (13U)
/*! ADDR_ERR1_SIG_EN - OCRAM Access Error Interrupt Enable On Bank1
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_ADDR_ERR1_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_ADDR_ERR1_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_ADDR_ERR1_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_ADDR_ERR2_SIG_EN_MASK    (0x4000U)
#define MECC_ERR_SIG_EN_ADDR_ERR2_SIG_EN_SHIFT   (14U)
/*! ADDR_ERR2_SIG_EN - OCRAM Access Error Interrupt Enable On Bank2
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_ADDR_ERR2_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_ADDR_ERR2_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_ADDR_ERR2_SIG_EN_MASK)

#define MECC_ERR_SIG_EN_ADDR_ERR3_SIG_EN_MASK    (0x8000U)
#define MECC_ERR_SIG_EN_ADDR_ERR3_SIG_EN_SHIFT   (15U)
/*! ADDR_ERR3_SIG_EN - OCRAM Access Error Interrupt Enable On Bank3
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define MECC_ERR_SIG_EN_ADDR_ERR3_SIG_EN(x)      (((uint32_t)(((uint32_t)(x)) << MECC_ERR_SIG_EN_ADDR_ERR3_SIG_EN_SHIFT)) & MECC_ERR_SIG_EN_ADDR_ERR3_SIG_EN_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_LOW0 - Error Injection On LOW 32 bits Of OCRAM Bank0 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_LOW0_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_LOW0_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On LOW 32 bits Of OCRAM Bank0 Write Data */
#define MECC_ERR_DATA_INJ_LOW0_ERR_DATA_INJ(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_LOW0_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_LOW0_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_HIGH0 - Error Injection On HIGH 32 bits Of OCRAM Bank0 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_HIGH0_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_HIGH0_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On HIGH 32 bits Of OCRAM Bank0 Write Data */
#define MECC_ERR_DATA_INJ_HIGH0_ERR_DATA_INJ(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_HIGH0_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_HIGH0_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_ECC_INJ0 - Error Injection On 8 bits ECC code Of OCRAM Bank0 Write Data */
/*! @{ */

#define MECC_ERR_ECC_INJ0_ERR_ECC_INJ_MASK       (0xFFU)
#define MECC_ERR_ECC_INJ0_ERR_ECC_INJ_SHIFT      (0U)
/*! ERR_ECC_INJ - Error Injection On 8 bits ECC code Of OCRAM Bank0 Write Data */
#define MECC_ERR_ECC_INJ0_ERR_ECC_INJ(x)         (((uint32_t)(((uint32_t)(x)) << MECC_ERR_ECC_INJ0_ERR_ECC_INJ_SHIFT)) & MECC_ERR_ECC_INJ0_ERR_ECC_INJ_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_LOW1 - Error Injection On LOW 32 bits Of OCRAM Bank1 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_LOW1_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_LOW1_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On LOW 32 bits Of OCRAM Bank1 Write Data */
#define MECC_ERR_DATA_INJ_LOW1_ERR_DATA_INJ(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_LOW1_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_LOW1_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_HIGH1 - Error Injection On HIGH 32 bits Of OCRAM Bank1 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_HIGH1_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_HIGH1_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On HIGH 32 bits Of OCRAM Bank1 Write Data */
#define MECC_ERR_DATA_INJ_HIGH1_ERR_DATA_INJ(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_HIGH1_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_HIGH1_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_ECC_INJ1 - Error Injection On 8 bits ECC code Of OCRAM Bank1 Write Data */
/*! @{ */

#define MECC_ERR_ECC_INJ1_ERR_ECC_INJ_MASK       (0xFFU)
#define MECC_ERR_ECC_INJ1_ERR_ECC_INJ_SHIFT      (0U)
/*! ERR_ECC_INJ - Error Injection On 8 bits ECC code Of OCRAM Bank1 Write Data */
#define MECC_ERR_ECC_INJ1_ERR_ECC_INJ(x)         (((uint32_t)(((uint32_t)(x)) << MECC_ERR_ECC_INJ1_ERR_ECC_INJ_SHIFT)) & MECC_ERR_ECC_INJ1_ERR_ECC_INJ_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_LOW2 - Error Injection On LOW 32 bits Of OCRAM Bank2 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_LOW2_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_LOW2_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On LOW 32 bits Of OCRAM Bank2 Write Data */
#define MECC_ERR_DATA_INJ_LOW2_ERR_DATA_INJ(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_LOW2_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_LOW2_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_HIGH2 - Error Injection On HIGH 32 bits Of OCRAM Bank2 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_HIGH2_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_HIGH2_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On HIGH 32 bits Of OCRAM Bank2 Write Data */
#define MECC_ERR_DATA_INJ_HIGH2_ERR_DATA_INJ(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_HIGH2_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_HIGH2_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_ECC_INJ2 - Error Injection On 8 bits ECC code Of OCRAM Bank2 Write Data */
/*! @{ */

#define MECC_ERR_ECC_INJ2_ERR_ECC_INJ_MASK       (0xFFU)
#define MECC_ERR_ECC_INJ2_ERR_ECC_INJ_SHIFT      (0U)
/*! ERR_ECC_INJ - Error Injection On 8 bits ECC code Of OCRAM Bank2 Write Data */
#define MECC_ERR_ECC_INJ2_ERR_ECC_INJ(x)         (((uint32_t)(((uint32_t)(x)) << MECC_ERR_ECC_INJ2_ERR_ECC_INJ_SHIFT)) & MECC_ERR_ECC_INJ2_ERR_ECC_INJ_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_LOW3 - Error Injection On LOW 32 bits Of OCRAM Bank3 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_LOW3_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_LOW3_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On LOW 32 bits Of OCRAM Bank3 Write Data */
#define MECC_ERR_DATA_INJ_LOW3_ERR_DATA_INJ(x)   (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_LOW3_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_LOW3_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_DATA_INJ_HIGH3 - Error Injection On HIGH 32 bits Of OCRAM Bank3 Write Data */
/*! @{ */

#define MECC_ERR_DATA_INJ_HIGH3_ERR_DATA_INJ_MASK (0xFFFFFFFFU)
#define MECC_ERR_DATA_INJ_HIGH3_ERR_DATA_INJ_SHIFT (0U)
/*! ERR_DATA_INJ - Error Injection On HIGH 32 bits Of OCRAM Bank3 Write Data */
#define MECC_ERR_DATA_INJ_HIGH3_ERR_DATA_INJ(x)  (((uint32_t)(((uint32_t)(x)) << MECC_ERR_DATA_INJ_HIGH3_ERR_DATA_INJ_SHIFT)) & MECC_ERR_DATA_INJ_HIGH3_ERR_DATA_INJ_MASK)
/*! @} */

/*! @name ERR_ECC_INJ3 - Error Injection On 8 bits ECC code Of OCRAM Bank3 Write Data */
/*! @{ */

#define MECC_ERR_ECC_INJ3_ERR_ECC_INJ_MASK       (0xFFU)
#define MECC_ERR_ECC_INJ3_ERR_ECC_INJ_SHIFT      (0U)
/*! ERR_ECC_INJ - Error Injection On 8 bits ECC code Of OCRAM Bank3 Write Data */
#define MECC_ERR_ECC_INJ3_ERR_ECC_INJ(x)         (((uint32_t)(((uint32_t)(x)) << MECC_ERR_ECC_INJ3_ERR_ECC_INJ_SHIFT)) & MECC_ERR_ECC_INJ3_ERR_ECC_INJ_MASK)
/*! @} */

/*! @name SINGLE_ERR_ADDR_ECC0 - Single Error Address And ECC code On OCRAM Bank0 */
/*! @{ */

#define MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ECC_MASK (0xFFU)
#define MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ECC_SHIFT (0U)
/*! SINGLE_ERR_ECC - Single Error ECC code On OCRAM Bank0 */
#define MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ECC_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ECC_MASK)

#define MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ADDR_SHIFT (8U)
/*! SINGLE_ERR_ADDR - Single Error Address On OCRAM Bank0 */
#define MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ADDR_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC0_SINGLE_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name SINGLE_ERR_DATA_LOW0 - LOW 32 Bits Single Error Read Data On OCRAM Bank0 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_LOW0_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_LOW0_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - LOW 32 Bits Single Error Read Data On OCRAM Bank0 */
#define MECC_SINGLE_ERR_DATA_LOW0_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_LOW0_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_LOW0_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_DATA_HIGH0 - HIGH 32 Bits Single Error Read Data On OCRAM Bank0 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_HIGH0_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_HIGH0_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - HIGH 32 Bits Single Error Read Data On OCRAM Bank0 */
#define MECC_SINGLE_ERR_DATA_HIGH0_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_HIGH0_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_HIGH0_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_LOW0 - LOW Single Error Bit Position On OCRAM Bank0 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_LOW0_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_LOW0_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - LOW Single Error Bit Position On OCRAM Bank0 */
#define MECC_SINGLE_ERR_POS_LOW0_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_LOW0_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_LOW0_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_HIGH0 - HIGH Single Error Bit Position On OCRAM Bank0 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_HIGH0_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_HIGH0_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - HIGH Single Error Bit Position On OCRAM Bank0 */
#define MECC_SINGLE_ERR_POS_HIGH0_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_HIGH0_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_HIGH0_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name SINGLE_ERR_ADDR_ECC1 - Single Error Address And ECC code On OCRAM Bank1 */
/*! @{ */

#define MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ECC_MASK (0xFFU)
#define MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ECC_SHIFT (0U)
/*! SINGLE_ERR_ECC - Single Error ECC code On OCRAM Bank1 */
#define MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ECC_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ECC_MASK)

#define MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ADDR_SHIFT (8U)
/*! SINGLE_ERR_ADDR - Single Error Address On OCRAM Bank1 */
#define MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ADDR_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC1_SINGLE_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name SINGLE_ERR_DATA_LOW1 - LOW 32 Bits Single Error Read Data On OCRAM Bank1 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_LOW1_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_LOW1_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - LOW 32 Bits Single Error Read Data On OCRAM Bank1 */
#define MECC_SINGLE_ERR_DATA_LOW1_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_LOW1_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_LOW1_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_DATA_HIGH1 - HIGH 32 Bits Single Error Read Data On OCRAM Bank1 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_HIGH1_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_HIGH1_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - HIGH 32 Bits Single Error Read Data On OCRAM Bank1 */
#define MECC_SINGLE_ERR_DATA_HIGH1_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_HIGH1_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_HIGH1_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_LOW1 - LOW Single Error Bit Position On OCRAM Bank1 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_LOW1_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_LOW1_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - LOW Single Error Bit Position On OCRAM Bank1 */
#define MECC_SINGLE_ERR_POS_LOW1_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_LOW1_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_LOW1_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_HIGH1 - HIGH Single Error Bit Position On OCRAM Bank1 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_HIGH1_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_HIGH1_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - HIGH Single Error Bit Position On OCRAM Bank1 */
#define MECC_SINGLE_ERR_POS_HIGH1_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_HIGH1_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_HIGH1_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name SINGLE_ERR_ADDR_ECC2 - Single Error Address And ECC code On OCRAM Bank2 */
/*! @{ */

#define MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ECC_MASK (0xFFU)
#define MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ECC_SHIFT (0U)
/*! SINGLE_ERR_ECC - Single Error ECC code On OCRAM Bank2 */
#define MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ECC_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ECC_MASK)

#define MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ADDR_SHIFT (8U)
/*! SINGLE_ERR_ADDR - Single Error Address On OCRAM Bank2 */
#define MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ADDR_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC2_SINGLE_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name SINGLE_ERR_DATA_LOW2 - LOW 32 Bits Single Error Read Data On OCRAM Bank2 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_LOW2_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_LOW2_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - LOW 32 Bits Single Error Read Data On OCRAM Bank2 */
#define MECC_SINGLE_ERR_DATA_LOW2_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_LOW2_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_LOW2_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_DATA_HIGH2 - HIGH 32 Bits Single Error Read Data On OCRAM Bank2 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_HIGH2_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_HIGH2_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - HIGH 32 Bits Single Error Read Data On OCRAM Bank2 */
#define MECC_SINGLE_ERR_DATA_HIGH2_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_HIGH2_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_HIGH2_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_LOW2 - LOW Single Error Bit Position On OCRAM Bank2 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_LOW2_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_LOW2_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - LOW Single Error Bit Position On OCRAM Bank2 */
#define MECC_SINGLE_ERR_POS_LOW2_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_LOW2_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_LOW2_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_HIGH2 - HIGH Single Error Bit Position On OCRAM Bank2 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_HIGH2_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_HIGH2_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - HIGH Single Error Bit Position On OCRAM Bank2 */
#define MECC_SINGLE_ERR_POS_HIGH2_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_HIGH2_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_HIGH2_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name SINGLE_ERR_ADDR_ECC3 - Single Error Address And ECC code On OCRAM Bank3 */
/*! @{ */

#define MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ECC_MASK (0xFFU)
#define MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ECC_SHIFT (0U)
/*! SINGLE_ERR_ECC - Single Error ECC code On OCRAM Bank3 */
#define MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ECC_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ECC_MASK)

#define MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ADDR_SHIFT (8U)
/*! SINGLE_ERR_ADDR - Single Error Address On OCRAM Bank3 */
#define MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ADDR_SHIFT)) & MECC_SINGLE_ERR_ADDR_ECC3_SINGLE_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name SINGLE_ERR_DATA_LOW3 - LOW 32 Bits Single Error Read Data On OCRAM Bank3 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_LOW3_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_LOW3_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - LOW 32 Bits Single Error Read Data On OCRAM Bank3 */
#define MECC_SINGLE_ERR_DATA_LOW3_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_LOW3_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_LOW3_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_DATA_HIGH3 - HIGH 32 Bits Single Error Read Data On OCRAM Bank3 */
/*! @{ */

#define MECC_SINGLE_ERR_DATA_HIGH3_SINGLE_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_DATA_HIGH3_SINGLE_ERR_DATA_SHIFT (0U)
/*! SINGLE_ERR_DATA - HIGH 32 Bits Single Error Read Data On OCRAM Bank3 */
#define MECC_SINGLE_ERR_DATA_HIGH3_SINGLE_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_DATA_HIGH3_SINGLE_ERR_DATA_SHIFT)) & MECC_SINGLE_ERR_DATA_HIGH3_SINGLE_ERR_DATA_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_LOW3 - LOW Single Error Bit Position On OCRAM Bank3 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_LOW3_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_LOW3_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - LOW Single Error Bit Position On OCRAM Bank3 */
#define MECC_SINGLE_ERR_POS_LOW3_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_LOW3_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_LOW3_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name SINGLE_ERR_POS_HIGH3 - HIGH Single Error Bit Position On OCRAM Bank3 */
/*! @{ */

#define MECC_SINGLE_ERR_POS_HIGH3_SINGLE_ERR_POS_MASK (0xFFFFFFFFU)
#define MECC_SINGLE_ERR_POS_HIGH3_SINGLE_ERR_POS_SHIFT (0U)
/*! SINGLE_ERR_POS - HIGH Single Error Bit Position On OCRAM Bank3 */
#define MECC_SINGLE_ERR_POS_HIGH3_SINGLE_ERR_POS(x) (((uint32_t)(((uint32_t)(x)) << MECC_SINGLE_ERR_POS_HIGH3_SINGLE_ERR_POS_SHIFT)) & MECC_SINGLE_ERR_POS_HIGH3_SINGLE_ERR_POS_MASK)
/*! @} */

/*! @name MULTI_ERR_ADDR_ECC0 - Multiple Error Address And ECC code On OCRAM Bank0 */
/*! @{ */

#define MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ECC_MASK (0xFFU)
#define MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ECC_SHIFT (0U)
/*! MULTI_ERR_ECC - Multiple Error ECC code On OCRAM Bank0 */
#define MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ECC_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ECC_MASK)

#define MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ADDR_SHIFT (8U)
/*! MULTI_ERR_ADDR - Multiple Error Address On OCRAM Bank0 */
#define MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ADDR_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC0_MULTI_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name MULTI_ERR_DATA_LOW0 - LOW 32 Bits Multiple Error Read Data On OCRAM Bank0 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_LOW0_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_LOW0_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - LOW 32 Bits Multiple Error Read Data On OCRAM Bank0 */
#define MECC_MULTI_ERR_DATA_LOW0_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_LOW0_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_LOW0_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name MULTI_ERR_DATA_HIGH0 - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank0 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_HIGH0_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_HIGH0_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank0 */
#define MECC_MULTI_ERR_DATA_HIGH0_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_HIGH0_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_HIGH0_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name MULTI_ERR_ADDR_ECC1 - Multiple Error Address And ECC code On OCRAM Bank1 */
/*! @{ */

#define MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ECC_MASK (0xFFU)
#define MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ECC_SHIFT (0U)
/*! MULTI_ERR_ECC - Multiple Error ECC code On OCRAM Bank1 */
#define MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ECC_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ECC_MASK)

#define MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ADDR_SHIFT (8U)
/*! MULTI_ERR_ADDR - Multiple Error Address On OCRAM Bank1 */
#define MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ADDR_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC1_MULTI_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name MULTI_ERR_DATA_LOW1 - LOW 32 Bits Multiple Error Read Data On OCRAM Bank1 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_LOW1_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_LOW1_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - LOW 32 Bits Multiple Error Read Data On OCRAM Bank1 */
#define MECC_MULTI_ERR_DATA_LOW1_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_LOW1_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_LOW1_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name MULTI_ERR_DATA_HIGH1 - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank1 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_HIGH1_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_HIGH1_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank1 */
#define MECC_MULTI_ERR_DATA_HIGH1_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_HIGH1_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_HIGH1_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name MULTI_ERR_ADDR_ECC2 - Multiple Error Address And ECC code On OCRAM Bank2 */
/*! @{ */

#define MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ECC_MASK (0xFFU)
#define MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ECC_SHIFT (0U)
/*! MULTI_ERR_ECC - Multiple Error ECC code On OCRAM Bank2 */
#define MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ECC_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ECC_MASK)

#define MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ADDR_SHIFT (8U)
/*! MULTI_ERR_ADDR - Multiple Error Address On OCRAM Bank2 */
#define MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ADDR_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC2_MULTI_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name MULTI_ERR_DATA_LOW2 - LOW 32 Bits Multiple Error Read Data On OCRAM Bank2 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_LOW2_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_LOW2_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - LOW 32 Bits Multiple Error Read Data On OCRAM Bank2 */
#define MECC_MULTI_ERR_DATA_LOW2_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_LOW2_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_LOW2_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name MULTI_ERR_DATA_HIGH2 - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank2 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_HIGH2_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_HIGH2_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank2 */
#define MECC_MULTI_ERR_DATA_HIGH2_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_HIGH2_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_HIGH2_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name MULTI_ERR_ADDR_ECC3 - Multiple Error Address And ECC code On OCRAM Bank3 */
/*! @{ */

#define MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ECC_MASK (0xFFU)
#define MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ECC_SHIFT (0U)
/*! MULTI_ERR_ECC - Multiple Error ECC code On OCRAM Bank3 */
#define MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ECC(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ECC_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ECC_MASK)

#define MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ADDR_MASK (0x7FFFF00U)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
#define MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ADDR_SHIFT (8U)
/*! MULTI_ERR_ADDR - Multiple Error Address On OCRAM Bank3 */
#define MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ADDR(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ADDR_SHIFT)) & MECC_MULTI_ERR_ADDR_ECC3_MULTI_ERR_ADDR_MASK)  /* Merged from fields with different position or width, of widths (18, 19), largest definition used */
/*! @} */

/*! @name MULTI_ERR_DATA_LOW3 - LOW 32 Bits Multiple Error Read Data On OCRAM Bank3 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_LOW3_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_LOW3_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - LOW 32 Bits Multiple Error Read Data On OCRAM Bank3 */
#define MECC_MULTI_ERR_DATA_LOW3_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_LOW3_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_LOW3_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name MULTI_ERR_DATA_HIGH3 - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank3 */
/*! @{ */

#define MECC_MULTI_ERR_DATA_HIGH3_MULTI_ERR_DATA_MASK (0xFFFFFFFFU)
#define MECC_MULTI_ERR_DATA_HIGH3_MULTI_ERR_DATA_SHIFT (0U)
/*! MULTI_ERR_DATA - HIGH 32 Bits Multiple Error Read Data On OCRAM Bank3 */
#define MECC_MULTI_ERR_DATA_HIGH3_MULTI_ERR_DATA(x) (((uint32_t)(((uint32_t)(x)) << MECC_MULTI_ERR_DATA_HIGH3_MULTI_ERR_DATA_SHIFT)) & MECC_MULTI_ERR_DATA_HIGH3_MULTI_ERR_DATA_MASK)
/*! @} */

/*! @name PIPE_ECC_EN - OCRAM Pipeline And ECC Enable */
/*! @{ */

#define MECC_PIPE_ECC_EN_READ_DATA_WAIT_EN_MASK  (0x1U)
#define MECC_PIPE_ECC_EN_READ_DATA_WAIT_EN_SHIFT (0U)
/*! READ_DATA_WAIT_EN - Read Data Wait Enable
 *  0b0..Disable.
 *  0b1..Enable.
 */
#define MECC_PIPE_ECC_EN_READ_DATA_WAIT_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_PIPE_ECC_EN_READ_DATA_WAIT_EN_SHIFT)) & MECC_PIPE_ECC_EN_READ_DATA_WAIT_EN_MASK)

#define MECC_PIPE_ECC_EN_READ_ADDR_PIPE_EN_MASK  (0x2U)
#define MECC_PIPE_ECC_EN_READ_ADDR_PIPE_EN_SHIFT (1U)
/*! READ_ADDR_PIPE_EN - Read Address Pipeline Enable
 *  0b0..Disable.
 *  0b1..Enable.
 */
#define MECC_PIPE_ECC_EN_READ_ADDR_PIPE_EN(x)    (((uint32_t)(((uint32_t)(x)) << MECC_PIPE_ECC_EN_READ_ADDR_PIPE_EN_SHIFT)) & MECC_PIPE_ECC_EN_READ_ADDR_PIPE_EN_MASK)

#define MECC_PIPE_ECC_EN_WRITE_DATA_PIPE_EN_MASK (0x4U)
#define MECC_PIPE_ECC_EN_WRITE_DATA_PIPE_EN_SHIFT (2U)
/*! WRITE_DATA_PIPE_EN - Write Data Pipeline Enable
 *  0b0..Disable.
 *  0b1..Enable.
 */
#define MECC_PIPE_ECC_EN_WRITE_DATA_PIPE_EN(x)   (((uint32_t)(((uint32_t)(x)) << MECC_PIPE_ECC_EN_WRITE_DATA_PIPE_EN_SHIFT)) & MECC_PIPE_ECC_EN_WRITE_DATA_PIPE_EN_MASK)

#define MECC_PIPE_ECC_EN_WRITE_ADDR_PIPE_EN_MASK (0x8U)
#define MECC_PIPE_ECC_EN_WRITE_ADDR_PIPE_EN_SHIFT (3U)
/*! WRITE_ADDR_PIPE_EN - Write Address Pipeline Enable
 *  0b0..Disable.
 *  0b1..Enable.
 */
#define MECC_PIPE_ECC_EN_WRITE_ADDR_PIPE_EN(x)   (((uint32_t)(((uint32_t)(x)) << MECC_PIPE_ECC_EN_WRITE_ADDR_PIPE_EN_SHIFT)) & MECC_PIPE_ECC_EN_WRITE_ADDR_PIPE_EN_MASK)

#define MECC_PIPE_ECC_EN_ECC_EN_MASK             (0x10U)
#define MECC_PIPE_ECC_EN_ECC_EN_SHIFT            (4U)
/*! ECC_EN - ECC Function Enable
 *  0b0..Disable.
 *  0b1..Enable.
 */
#define MECC_PIPE_ECC_EN_ECC_EN(x)               (((uint32_t)(((uint32_t)(x)) << MECC_PIPE_ECC_EN_ECC_EN_SHIFT)) & MECC_PIPE_ECC_EN_ECC_EN_MASK)
/*! @} */

/*! @name PENDING_STAT - Pending Status */
/*! @{ */

#define MECC_PENDING_STAT_READ_DATA_WAIT_PENDING_MASK (0x1U)
#define MECC_PENDING_STAT_READ_DATA_WAIT_PENDING_SHIFT (0U)
/*! READ_DATA_WAIT_PENDING - Read Data Wait Pending
 *  0b0..No update pending status for READ_DATA_WAIT_EN.
 *  0b1..When READ_DATA_WAIT_EN register bit is changed, this register bit will be set until the new setup becomes valid in the controller.
 */
#define MECC_PENDING_STAT_READ_DATA_WAIT_PENDING(x) (((uint32_t)(((uint32_t)(x)) << MECC_PENDING_STAT_READ_DATA_WAIT_PENDING_SHIFT)) & MECC_PENDING_STAT_READ_DATA_WAIT_PENDING_MASK)

#define MECC_PENDING_STAT_READ_ADDR_PIPE_PENDING_MASK (0x2U)
#define MECC_PENDING_STAT_READ_ADDR_PIPE_PENDING_SHIFT (1U)
/*! READ_ADDR_PIPE_PENDING - Read Address Pipeline Pending
 *  0b0..No update pending status for READ_ADDR_PIPE_EN.
 *  0b1..When READ_ADDR_PIPE_EN register bit is changed, this register bit will be set until the new setup becomes valid in the controller.
 */
#define MECC_PENDING_STAT_READ_ADDR_PIPE_PENDING(x) (((uint32_t)(((uint32_t)(x)) << MECC_PENDING_STAT_READ_ADDR_PIPE_PENDING_SHIFT)) & MECC_PENDING_STAT_READ_ADDR_PIPE_PENDING_MASK)

#define MECC_PENDING_STAT_WRITE_DATA_PIPE_PENDING_MASK (0x4U)
#define MECC_PENDING_STAT_WRITE_DATA_PIPE_PENDING_SHIFT (2U)
/*! WRITE_DATA_PIPE_PENDING - Write Data Pipeline Pending
 *  0b0..No update pending status for WRITE_DATA_PIPE_EN.
 *  0b1..When WRITE_DATA_PIPE_EN register bit is changed, this register bit will be set until the new setup becomes valid in the controller.
 */
#define MECC_PENDING_STAT_WRITE_DATA_PIPE_PENDING(x) (((uint32_t)(((uint32_t)(x)) << MECC_PENDING_STAT_WRITE_DATA_PIPE_PENDING_SHIFT)) & MECC_PENDING_STAT_WRITE_DATA_PIPE_PENDING_MASK)

#define MECC_PENDING_STAT_WRITE_ADDR_PIPE_PENDING_MASK (0x8U)
#define MECC_PENDING_STAT_WRITE_ADDR_PIPE_PENDING_SHIFT (3U)
/*! WRITE_ADDR_PIPE_PENDING - Write Address Pipeline Pending
 *  0b0..No update pending status for WRITE_ADDR_PIPE_EN.
 *  0b1..When WRITE_ADDR_PIPE_EN register bit is changed, this register bit will be set until the new setup becomes valid in the controller.
 */
#define MECC_PENDING_STAT_WRITE_ADDR_PIPE_PENDING(x) (((uint32_t)(((uint32_t)(x)) << MECC_PENDING_STAT_WRITE_ADDR_PIPE_PENDING_SHIFT)) & MECC_PENDING_STAT_WRITE_ADDR_PIPE_PENDING_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group MECC_Register_Masks */


/* MECC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral MECC1 base address */
  #define MECC1_BASE                               (0x52920000u)
  /** Peripheral MECC1 base address */
  #define MECC1_BASE_NS                            (0x42920000u)
  /** Peripheral MECC1 base pointer */
  #define MECC1                                    ((MECC_Type *)MECC1_BASE)
  /** Peripheral MECC1 base pointer */
  #define MECC1_NS                                 ((MECC_Type *)MECC1_BASE_NS)
  /** Peripheral MECC2 base address */
  #define MECC2_BASE                               (0x52930000u)
  /** Peripheral MECC2 base address */
  #define MECC2_BASE_NS                            (0x42930000u)
  /** Peripheral MECC2 base pointer */
  #define MECC2                                    ((MECC_Type *)MECC2_BASE)
  /** Peripheral MECC2 base pointer */
  #define MECC2_NS                                 ((MECC_Type *)MECC2_BASE_NS)
  /** Array initializer of MECC peripheral base addresses */
  #define MECC_BASE_ADDRS                          { 0u, MECC1_BASE, MECC2_BASE }
  /** Array initializer of MECC peripheral base pointers */
  #define MECC_BASE_PTRS                           { (MECC_Type *)0u, MECC1, MECC2 }
  /** Array initializer of MECC peripheral base addresses */
  #define MECC_BASE_ADDRS_NS                       { 0u, MECC1_BASE_NS, MECC2_BASE_NS }
  /** Array initializer of MECC peripheral base pointers */
  #define MECC_BASE_PTRS_NS                        { (MECC_Type *)0u, MECC1_NS, MECC2_NS }
#else
  /** Peripheral MECC1 base address */
  #define MECC1_BASE                               (0x42920000u)
  /** Peripheral MECC1 base pointer */
  #define MECC1                                    ((MECC_Type *)MECC1_BASE)
  /** Peripheral MECC2 base address */
  #define MECC2_BASE                               (0x42930000u)
  /** Peripheral MECC2 base pointer */
  #define MECC2                                    ((MECC_Type *)MECC2_BASE)
  /** Array initializer of MECC peripheral base addresses */
  #define MECC_BASE_ADDRS                          { 0u, MECC1_BASE, MECC2_BASE }
  /** Array initializer of MECC peripheral base pointers */
  #define MECC_BASE_PTRS                           { (MECC_Type *)0u, MECC1, MECC2 }
#endif

/*!
 * @}
 */ /* end of group MECC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- MSGINTR Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup MSGINTR_Peripheral_Access_Layer MSGINTR Peripheral Access Layer
 * @{
 */

/** MSGINTR - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x8 */
    __O  uint32_t MSIIR;                             /**< Message Signaled Interrupt Index Register 0..Message Signaled Interrupt Index Register 2, array offset: 0x0, array step: 0x8 */
    __I  uint32_t MSIR;                              /**< Message Signaled Interrupt Register 0..Message Signaled Interrupt Register 2, array offset: 0x4, array step: 0x8 */
  } MSI[3];
} MSGINTR_Type;

/* ----------------------------------------------------------------------------
   -- MSGINTR Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup MSGINTR_Register_Masks MSGINTR Register Masks
 * @{
 */

/*! @name MSIIR - Message Signaled Interrupt Index Register 0..Message Signaled Interrupt Index Register 2 */
/*! @{ */

#define MSGINTR_MSIIR_IBS_MASK                   (0x1FU)
#define MSGINTR_MSIIR_IBS_SHIFT                  (0U)
#define MSGINTR_MSIIR_IBS(x)                     (((uint32_t)(((uint32_t)(x)) << MSGINTR_MSIIR_IBS_SHIFT)) & MSGINTR_MSIIR_IBS_MASK)
/*! @} */

/* The count of MSGINTR_MSIIR */
#define MSGINTR_MSIIR_COUNT                      (3U)

/*! @name MSIR - Message Signaled Interrupt Register 0..Message Signaled Interrupt Register 2 */
/*! @{ */

#define MSGINTR_MSIR_SHn_MASK                    (0xFFFFFFFFU)
#define MSGINTR_MSIR_SHn_SHIFT                   (0U)
#define MSGINTR_MSIR_SHn(x)                      (((uint32_t)(((uint32_t)(x)) << MSGINTR_MSIR_SHn_SHIFT)) & MSGINTR_MSIR_SHn_MASK)
/*! @} */

/* The count of MSGINTR_MSIR */
#define MSGINTR_MSIR_COUNT                       (3U)


/*!
 * @}
 */ /* end of group MSGINTR_Register_Masks */


/* MSGINTR - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral MSGINTR1 base address */
  #define MSGINTR1_BASE                            (0x528A0000u)
  /** Peripheral MSGINTR1 base address */
  #define MSGINTR1_BASE_NS                         (0x428A0000u)
  /** Peripheral MSGINTR1 base pointer */
  #define MSGINTR1                                 ((MSGINTR_Type *)MSGINTR1_BASE)
  /** Peripheral MSGINTR1 base pointer */
  #define MSGINTR1_NS                              ((MSGINTR_Type *)MSGINTR1_BASE_NS)
  /** Peripheral MSGINTR2 base address */
  #define MSGINTR2_BASE                            (0x528B0000u)
  /** Peripheral MSGINTR2 base address */
  #define MSGINTR2_BASE_NS                         (0x428B0000u)
  /** Peripheral MSGINTR2 base pointer */
  #define MSGINTR2                                 ((MSGINTR_Type *)MSGINTR2_BASE)
  /** Peripheral MSGINTR2 base pointer */
  #define MSGINTR2_NS                              ((MSGINTR_Type *)MSGINTR2_BASE_NS)
  /** Peripheral MSGINTR3 base address */
  #define MSGINTR3_BASE                            (0x528C0000u)
  /** Peripheral MSGINTR3 base address */
  #define MSGINTR3_BASE_NS                         (0x428C0000u)
  /** Peripheral MSGINTR3 base pointer */
  #define MSGINTR3                                 ((MSGINTR_Type *)MSGINTR3_BASE)
  /** Peripheral MSGINTR3 base pointer */
  #define MSGINTR3_NS                              ((MSGINTR_Type *)MSGINTR3_BASE_NS)
  /** Peripheral MSGINTR4 base address */
  #define MSGINTR4_BASE                            (0x528D0000u)
  /** Peripheral MSGINTR4 base address */
  #define MSGINTR4_BASE_NS                         (0x428D0000u)
  /** Peripheral MSGINTR4 base pointer */
  #define MSGINTR4                                 ((MSGINTR_Type *)MSGINTR4_BASE)
  /** Peripheral MSGINTR4 base pointer */
  #define MSGINTR4_NS                              ((MSGINTR_Type *)MSGINTR4_BASE_NS)
  /** Peripheral MSGINTR5 base address */
  #define MSGINTR5_BASE                            (0x528E0000u)
  /** Peripheral MSGINTR5 base address */
  #define MSGINTR5_BASE_NS                         (0x428E0000u)
  /** Peripheral MSGINTR5 base pointer */
  #define MSGINTR5                                 ((MSGINTR_Type *)MSGINTR5_BASE)
  /** Peripheral MSGINTR5 base pointer */
  #define MSGINTR5_NS                              ((MSGINTR_Type *)MSGINTR5_BASE_NS)
  /** Peripheral MSGINTR6 base address */
  #define MSGINTR6_BASE                            (0x528F0000u)
  /** Peripheral MSGINTR6 base address */
  #define MSGINTR6_BASE_NS                         (0x428F0000u)
  /** Peripheral MSGINTR6 base pointer */
  #define MSGINTR6                                 ((MSGINTR_Type *)MSGINTR6_BASE)
  /** Peripheral MSGINTR6 base pointer */
  #define MSGINTR6_NS                              ((MSGINTR_Type *)MSGINTR6_BASE_NS)
  /** Array initializer of MSGINTR peripheral base addresses */
  #define MSGINTR_BASE_ADDRS                       { 0u, MSGINTR1_BASE, MSGINTR2_BASE, MSGINTR3_BASE, MSGINTR4_BASE, MSGINTR5_BASE, MSGINTR6_BASE }
  /** Array initializer of MSGINTR peripheral base pointers */
  #define MSGINTR_BASE_PTRS                        { (MSGINTR_Type *)0u, MSGINTR1, MSGINTR2, MSGINTR3, MSGINTR4, MSGINTR5, MSGINTR6 }
  /** Array initializer of MSGINTR peripheral base addresses */
  #define MSGINTR_BASE_ADDRS_NS                    { 0u, MSGINTR1_BASE_NS, MSGINTR2_BASE_NS, MSGINTR3_BASE_NS, MSGINTR4_BASE_NS, MSGINTR5_BASE_NS, MSGINTR6_BASE_NS }
  /** Array initializer of MSGINTR peripheral base pointers */
  #define MSGINTR_BASE_PTRS_NS                     { (MSGINTR_Type *)0u, MSGINTR1_NS, MSGINTR2_NS, MSGINTR3_NS, MSGINTR4_NS, MSGINTR5_NS, MSGINTR6_NS }
#else
  /** Peripheral MSGINTR1 base address */
  #define MSGINTR1_BASE                            (0x428A0000u)
  /** Peripheral MSGINTR1 base pointer */
  #define MSGINTR1                                 ((MSGINTR_Type *)MSGINTR1_BASE)
  /** Peripheral MSGINTR2 base address */
  #define MSGINTR2_BASE                            (0x428B0000u)
  /** Peripheral MSGINTR2 base pointer */
  #define MSGINTR2                                 ((MSGINTR_Type *)MSGINTR2_BASE)
  /** Peripheral MSGINTR3 base address */
  #define MSGINTR3_BASE                            (0x428C0000u)
  /** Peripheral MSGINTR3 base pointer */
  #define MSGINTR3                                 ((MSGINTR_Type *)MSGINTR3_BASE)
  /** Peripheral MSGINTR4 base address */
  #define MSGINTR4_BASE                            (0x428D0000u)
  /** Peripheral MSGINTR4 base pointer */
  #define MSGINTR4                                 ((MSGINTR_Type *)MSGINTR4_BASE)
  /** Peripheral MSGINTR5 base address */
  #define MSGINTR5_BASE                            (0x428E0000u)
  /** Peripheral MSGINTR5 base pointer */
  #define MSGINTR5                                 ((MSGINTR_Type *)MSGINTR5_BASE)
  /** Peripheral MSGINTR6 base address */
  #define MSGINTR6_BASE                            (0x428F0000u)
  /** Peripheral MSGINTR6 base pointer */
  #define MSGINTR6                                 ((MSGINTR_Type *)MSGINTR6_BASE)
  /** Array initializer of MSGINTR peripheral base addresses */
  #define MSGINTR_BASE_ADDRS                       { 0u, MSGINTR1_BASE, MSGINTR2_BASE, MSGINTR3_BASE, MSGINTR4_BASE, MSGINTR5_BASE, MSGINTR6_BASE }
  /** Array initializer of MSGINTR peripheral base pointers */
  #define MSGINTR_BASE_PTRS                        { (MSGINTR_Type *)0u, MSGINTR1, MSGINTR2, MSGINTR3, MSGINTR4, MSGINTR5, MSGINTR6 }
#endif

/*!
 * @}
 */ /* end of group MSGINTR_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- MU Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup MU_Peripheral_Access_Layer MU Peripheral Access Layer
 * @{
 */

/** MU - Register Layout Typedef */
typedef struct {
  __I  uint32_t VER;                               /**< Version ID, offset: 0x0 */
  __I  uint32_t PAR;                               /**< Parameter, offset: 0x4 */
  __IO uint32_t CR;                                /**< Control, offset: 0x8 */
  __IO uint32_t SR;                                /**< Status, offset: 0xC */
       uint32_t CCR0;                              /**< Core Control 0, offset: 0x10 */
       uint32_t CIER0;                             /**< Core Interrupt Enable 0, offset: 0x14 */
       uint8_t RESERVED_0[232];
  __IO uint32_t FCR;                               /**< Flag Control, offset: 0x100 */
  __I  uint32_t FSR;                               /**< Flag Status, offset: 0x104 */
       uint8_t RESERVED_1[8];
  __IO uint32_t GIER;                              /**< General-Purpose Interrupt Enable, offset: 0x110 */
  __IO uint32_t GCR;                               /**< General-Purpose Control, offset: 0x114 */
  __IO uint32_t GSR;                               /**< General-purpose Status, offset: 0x118 */
       uint8_t RESERVED_2[4];
  __IO uint32_t TCR;                               /**< Transmit Control, offset: 0x120 */
  __I  uint32_t TSR;                               /**< Transmit Status, offset: 0x124 */
  __IO uint32_t RCR;                               /**< Receive Control, offset: 0x128 */
  __I  uint32_t RSR;                               /**< Receive Status, offset: 0x12C */
       uint8_t RESERVED_3[208];
  __O  uint32_t TR[4];                             /**< Transmit, array offset: 0x200, array step: 0x4 */
       uint8_t RESERVED_4[112];
  __I  uint32_t RR[4];                             /**< Receive, array offset: 0x280, array step: 0x4 */
} MU_Type;

/* ----------------------------------------------------------------------------
   -- MU Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup MU_Register_Masks MU Register Masks
 * @{
 */

/*! @name VER - Version ID */
/*! @{ */

#define MU_VER_FEATURE_MASK                      (0xFFFFU)
#define MU_VER_FEATURE_SHIFT                     (0U)
/*! FEATURE - Feature Set Number */
#define MU_VER_FEATURE(x)                        (((uint32_t)(((uint32_t)(x)) << MU_VER_FEATURE_SHIFT)) & MU_VER_FEATURE_MASK)

#define MU_VER_MINOR_MASK                        (0xFF0000U)
#define MU_VER_MINOR_SHIFT                       (16U)
/*! MINOR - Minor Version Number */
#define MU_VER_MINOR(x)                          (((uint32_t)(((uint32_t)(x)) << MU_VER_MINOR_SHIFT)) & MU_VER_MINOR_MASK)

#define MU_VER_MAJOR_MASK                        (0xFF000000U)
#define MU_VER_MAJOR_SHIFT                       (24U)
/*! MAJOR - Major Version Number */
#define MU_VER_MAJOR(x)                          (((uint32_t)(((uint32_t)(x)) << MU_VER_MAJOR_SHIFT)) & MU_VER_MAJOR_MASK)
/*! @} */

/*! @name PAR - Parameter */
/*! @{ */

#define MU_PAR_TR_NUM_MASK                       (0xFFU)
#define MU_PAR_TR_NUM_SHIFT                      (0U)
/*! TR_NUM - Transmit Register Number */
#define MU_PAR_TR_NUM(x)                         (((uint32_t)(((uint32_t)(x)) << MU_PAR_TR_NUM_SHIFT)) & MU_PAR_TR_NUM_MASK)

#define MU_PAR_RR_NUM_MASK                       (0xFF00U)
#define MU_PAR_RR_NUM_SHIFT                      (8U)
/*! RR_NUM - Receive Register Number */
#define MU_PAR_RR_NUM(x)                         (((uint32_t)(((uint32_t)(x)) << MU_PAR_RR_NUM_SHIFT)) & MU_PAR_RR_NUM_MASK)

#define MU_PAR_GIR_NUM_MASK                      (0xFF0000U)
#define MU_PAR_GIR_NUM_SHIFT                     (16U)
/*! GIR_NUM - General-Purpose Interrupt Request Number */
#define MU_PAR_GIR_NUM(x)                        (((uint32_t)(((uint32_t)(x)) << MU_PAR_GIR_NUM_SHIFT)) & MU_PAR_GIR_NUM_MASK)

#define MU_PAR_FLAG_WIDTH_MASK                   (0xFF000000U)
#define MU_PAR_FLAG_WIDTH_SHIFT                  (24U)
/*! FLAG_WIDTH - Flag Width */
#define MU_PAR_FLAG_WIDTH(x)                     (((uint32_t)(((uint32_t)(x)) << MU_PAR_FLAG_WIDTH_SHIFT)) & MU_PAR_FLAG_WIDTH_MASK)
/*! @} */

/*! @name CR - Control */
/*! @{ */

#define MU_CR_MUR_MASK                           (0x1U)
#define MU_CR_MUR_SHIFT                          (0U)
/*! MUR - MU Reset
 *  0b0..Idle
 *  0b1..Reset
 */
#define MU_CR_MUR(x)                             (((uint32_t)(((uint32_t)(x)) << MU_CR_MUR_SHIFT)) & MU_CR_MUR_MASK)

#define MU_CR_MURIE_MASK                         (0x2U)
#define MU_CR_MURIE_SHIFT                        (1U)
/*! MURIE - MUA Reset Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_CR_MURIE(x)                           (((uint32_t)(((uint32_t)(x)) << MU_CR_MURIE_SHIFT)) & MU_CR_MURIE_MASK)
/*! @} */

/*! @name SR - Status */
/*! @{ */

#define MU_SR_MURS_MASK                          (0x1U)
#define MU_SR_MURS_SHIFT                         (0U)
/*! MURS - MUA and MUB Reset State
 *  0b0..Out of reset
 *  0b1..In reset
 */
#define MU_SR_MURS(x)                            (((uint32_t)(((uint32_t)(x)) << MU_SR_MURS_SHIFT)) & MU_SR_MURS_MASK)

#define MU_SR_MURIP_MASK                         (0x2U)
#define MU_SR_MURIP_SHIFT                        (1U)
/*! MURIP - MU Reset Interrupt Pending Flag
 *  0b0..Reset not issued
 *  0b1..Reset issued
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define MU_SR_MURIP(x)                           (((uint32_t)(((uint32_t)(x)) << MU_SR_MURIP_SHIFT)) & MU_SR_MURIP_MASK)

#define MU_SR_EP_MASK                            (0x4U)
#define MU_SR_EP_SHIFT                           (2U)
/*! EP - MUA Side Event Pending
 *  0b0..Not pending
 *  0b1..Pending
 */
#define MU_SR_EP(x)                              (((uint32_t)(((uint32_t)(x)) << MU_SR_EP_SHIFT)) & MU_SR_EP_MASK)

#define MU_SR_FUP_MASK                           (0x8U)
#define MU_SR_FUP_SHIFT                          (3U)
/*! FUP - MUA Flag Update Pending
 *  0b0..No pending update flags (initiated by MUA)
 *  0b1..Pending update flags (initiated by MUA)
 */
#define MU_SR_FUP(x)                             (((uint32_t)(((uint32_t)(x)) << MU_SR_FUP_SHIFT)) & MU_SR_FUP_MASK)

#define MU_SR_GIRP_MASK                          (0x10U)
#define MU_SR_GIRP_SHIFT                         (4U)
/*! GIRP - MUA General-Purpose Interrupt Pending
 *  0b0..No request sent
 *  0b1..Request sent
 */
#define MU_SR_GIRP(x)                            (((uint32_t)(((uint32_t)(x)) << MU_SR_GIRP_SHIFT)) & MU_SR_GIRP_MASK)

#define MU_SR_TEP_MASK                           (0x20U)
#define MU_SR_TEP_SHIFT                          (5U)
/*! TEP - MUA Transmit Empty Pending
 *  0b0..Not pending; MUB is reading no Receive (RRn) register
 *  0b1..Pending; MUB is reading a Receive (RRn) register
 */
#define MU_SR_TEP(x)                             (((uint32_t)(((uint32_t)(x)) << MU_SR_TEP_SHIFT)) & MU_SR_TEP_MASK)

#define MU_SR_RFP_MASK                           (0x40U)
#define MU_SR_RFP_SHIFT                          (6U)
/*! RFP - MUA Receive Full Pending
 *  0b0..Not pending; MUB is not writing to a Transmit register
 *  0b1..Pending; MUB is writing to a Transmit register
 */
#define MU_SR_RFP(x)                             (((uint32_t)(((uint32_t)(x)) << MU_SR_RFP_SHIFT)) & MU_SR_RFP_MASK)
/*! @} */

/*! @name FCR - Flag Control */
/*! @{ */

#define MU_FCR_F0_MASK                           (0x1U)
#define MU_FCR_F0_SHIFT                          (0U)
/*! F0 - MUA to MUB Flag
 *  0b0..Clear MUB_FSR[Fn]
 *  0b1..Set MUB_FSR[Fn]
 */
#define MU_FCR_F0(x)                             (((uint32_t)(((uint32_t)(x)) << MU_FCR_F0_SHIFT)) & MU_FCR_F0_MASK)

#define MU_FCR_F1_MASK                           (0x2U)
#define MU_FCR_F1_SHIFT                          (1U)
/*! F1 - MUA to MUB Flag
 *  0b0..Clear MUB_FSR[Fn]
 *  0b1..Set MUB_FSR[Fn]
 */
#define MU_FCR_F1(x)                             (((uint32_t)(((uint32_t)(x)) << MU_FCR_F1_SHIFT)) & MU_FCR_F1_MASK)

#define MU_FCR_F2_MASK                           (0x4U)
#define MU_FCR_F2_SHIFT                          (2U)
/*! F2 - MUA to MUB Flag
 *  0b0..Clear MUB_FSR[Fn]
 *  0b1..Set MUB_FSR[Fn]
 */
#define MU_FCR_F2(x)                             (((uint32_t)(((uint32_t)(x)) << MU_FCR_F2_SHIFT)) & MU_FCR_F2_MASK)
/*! @} */

/*! @name FSR - Flag Status */
/*! @{ */

#define MU_FSR_F0_MASK                           (0x1U)
#define MU_FSR_F0_SHIFT                          (0U)
/*! F0 - MUB to MUA-Side Flag
 *  0b0..MUB_FCR[Fn] = 0
 *  0b1..MUB_FCR[Fn] = 1
 */
#define MU_FSR_F0(x)                             (((uint32_t)(((uint32_t)(x)) << MU_FSR_F0_SHIFT)) & MU_FSR_F0_MASK)

#define MU_FSR_F1_MASK                           (0x2U)
#define MU_FSR_F1_SHIFT                          (1U)
/*! F1 - MUB to MUA-Side Flag
 *  0b0..MUB_FCR[Fn] = 0
 *  0b1..MUB_FCR[Fn] = 1
 */
#define MU_FSR_F1(x)                             (((uint32_t)(((uint32_t)(x)) << MU_FSR_F1_SHIFT)) & MU_FSR_F1_MASK)

#define MU_FSR_F2_MASK                           (0x4U)
#define MU_FSR_F2_SHIFT                          (2U)
/*! F2 - MUB to MUA-Side Flag
 *  0b0..MUB_FCR[Fn] = 0
 *  0b1..MUB_FCR[Fn] = 1
 */
#define MU_FSR_F2(x)                             (((uint32_t)(((uint32_t)(x)) << MU_FSR_F2_SHIFT)) & MU_FSR_F2_MASK)
/*! @} */

/*! @name GIER - General-Purpose Interrupt Enable */
/*! @{ */

#define MU_GIER_GIE0_MASK                        (0x1U)
#define MU_GIER_GIE0_SHIFT                       (0U)
/*! GIE0 - MUA General-purpose Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_GIER_GIE0(x)                          (((uint32_t)(((uint32_t)(x)) << MU_GIER_GIE0_SHIFT)) & MU_GIER_GIE0_MASK)

#define MU_GIER_GIE1_MASK                        (0x2U)
#define MU_GIER_GIE1_SHIFT                       (1U)
/*! GIE1 - MUA General-purpose Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_GIER_GIE1(x)                          (((uint32_t)(((uint32_t)(x)) << MU_GIER_GIE1_SHIFT)) & MU_GIER_GIE1_MASK)

#define MU_GIER_GIE2_MASK                        (0x4U)
#define MU_GIER_GIE2_SHIFT                       (2U)
/*! GIE2 - MUA General-purpose Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_GIER_GIE2(x)                          (((uint32_t)(((uint32_t)(x)) << MU_GIER_GIE2_SHIFT)) & MU_GIER_GIE2_MASK)

#define MU_GIER_GIE3_MASK                        (0x8U)
#define MU_GIER_GIE3_SHIFT                       (3U)
/*! GIE3 - MUA General-purpose Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_GIER_GIE3(x)                          (((uint32_t)(((uint32_t)(x)) << MU_GIER_GIE3_SHIFT)) & MU_GIER_GIE3_MASK)
/*! @} */

/*! @name GCR - General-Purpose Control */
/*! @{ */

#define MU_GCR_GIR0_MASK                         (0x1U)
#define MU_GCR_GIR0_SHIFT                        (0U)
/*! GIR0 - MUA General-Purpose Interrupt Request
 *  0b0..Not requested
 *  0b1..Requested
 */
#define MU_GCR_GIR0(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GCR_GIR0_SHIFT)) & MU_GCR_GIR0_MASK)

#define MU_GCR_GIR1_MASK                         (0x2U)
#define MU_GCR_GIR1_SHIFT                        (1U)
/*! GIR1 - MUA General-Purpose Interrupt Request
 *  0b0..Not requested
 *  0b1..Requested
 */
#define MU_GCR_GIR1(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GCR_GIR1_SHIFT)) & MU_GCR_GIR1_MASK)

#define MU_GCR_GIR2_MASK                         (0x4U)
#define MU_GCR_GIR2_SHIFT                        (2U)
/*! GIR2 - MUA General-Purpose Interrupt Request
 *  0b0..Not requested
 *  0b1..Requested
 */
#define MU_GCR_GIR2(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GCR_GIR2_SHIFT)) & MU_GCR_GIR2_MASK)

#define MU_GCR_GIR3_MASK                         (0x8U)
#define MU_GCR_GIR3_SHIFT                        (3U)
/*! GIR3 - MUA General-Purpose Interrupt Request
 *  0b0..Not requested
 *  0b1..Requested
 */
#define MU_GCR_GIR3(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GCR_GIR3_SHIFT)) & MU_GCR_GIR3_MASK)
/*! @} */

/*! @name GSR - General-purpose Status */
/*! @{ */

#define MU_GSR_GIP0_MASK                         (0x1U)
#define MU_GSR_GIP0_SHIFT                        (0U)
/*! GIP0 - MUA General-Purpose Interrupt Request Pending
 *  0b0..Not pending
 *  0b1..Pending
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define MU_GSR_GIP0(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GSR_GIP0_SHIFT)) & MU_GSR_GIP0_MASK)

#define MU_GSR_GIP1_MASK                         (0x2U)
#define MU_GSR_GIP1_SHIFT                        (1U)
/*! GIP1 - MUA General-Purpose Interrupt Request Pending
 *  0b0..Not pending
 *  0b1..Pending
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define MU_GSR_GIP1(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GSR_GIP1_SHIFT)) & MU_GSR_GIP1_MASK)

#define MU_GSR_GIP2_MASK                         (0x4U)
#define MU_GSR_GIP2_SHIFT                        (2U)
/*! GIP2 - MUA General-Purpose Interrupt Request Pending
 *  0b0..Not pending
 *  0b1..Pending
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define MU_GSR_GIP2(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GSR_GIP2_SHIFT)) & MU_GSR_GIP2_MASK)

#define MU_GSR_GIP3_MASK                         (0x8U)
#define MU_GSR_GIP3_SHIFT                        (3U)
/*! GIP3 - MUA General-Purpose Interrupt Request Pending
 *  0b0..Not pending
 *  0b1..Pending
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define MU_GSR_GIP3(x)                           (((uint32_t)(((uint32_t)(x)) << MU_GSR_GIP3_SHIFT)) & MU_GSR_GIP3_MASK)
/*! @} */

/*! @name TCR - Transmit Control */
/*! @{ */

#define MU_TCR_TIE0_MASK                         (0x1U)
#define MU_TCR_TIE0_SHIFT                        (0U)
/*! TIE0 - MUA Transmit Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_TCR_TIE0(x)                           (((uint32_t)(((uint32_t)(x)) << MU_TCR_TIE0_SHIFT)) & MU_TCR_TIE0_MASK)

#define MU_TCR_TIE1_MASK                         (0x2U)
#define MU_TCR_TIE1_SHIFT                        (1U)
/*! TIE1 - MUA Transmit Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_TCR_TIE1(x)                           (((uint32_t)(((uint32_t)(x)) << MU_TCR_TIE1_SHIFT)) & MU_TCR_TIE1_MASK)

#define MU_TCR_TIE2_MASK                         (0x4U)
#define MU_TCR_TIE2_SHIFT                        (2U)
/*! TIE2 - MUA Transmit Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_TCR_TIE2(x)                           (((uint32_t)(((uint32_t)(x)) << MU_TCR_TIE2_SHIFT)) & MU_TCR_TIE2_MASK)

#define MU_TCR_TIE3_MASK                         (0x8U)
#define MU_TCR_TIE3_SHIFT                        (3U)
/*! TIE3 - MUA Transmit Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_TCR_TIE3(x)                           (((uint32_t)(((uint32_t)(x)) << MU_TCR_TIE3_SHIFT)) & MU_TCR_TIE3_MASK)
/*! @} */

/*! @name TSR - Transmit Status */
/*! @{ */

#define MU_TSR_TE0_MASK                          (0x1U)
#define MU_TSR_TE0_SHIFT                         (0U)
/*! TE0 - MUA Transmit Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define MU_TSR_TE0(x)                            (((uint32_t)(((uint32_t)(x)) << MU_TSR_TE0_SHIFT)) & MU_TSR_TE0_MASK)

#define MU_TSR_TE1_MASK                          (0x2U)
#define MU_TSR_TE1_SHIFT                         (1U)
/*! TE1 - MUA Transmit Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define MU_TSR_TE1(x)                            (((uint32_t)(((uint32_t)(x)) << MU_TSR_TE1_SHIFT)) & MU_TSR_TE1_MASK)

#define MU_TSR_TE2_MASK                          (0x4U)
#define MU_TSR_TE2_SHIFT                         (2U)
/*! TE2 - MUA Transmit Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define MU_TSR_TE2(x)                            (((uint32_t)(((uint32_t)(x)) << MU_TSR_TE2_SHIFT)) & MU_TSR_TE2_MASK)

#define MU_TSR_TE3_MASK                          (0x8U)
#define MU_TSR_TE3_SHIFT                         (3U)
/*! TE3 - MUA Transmit Empty
 *  0b0..Not empty
 *  0b1..Empty
 */
#define MU_TSR_TE3(x)                            (((uint32_t)(((uint32_t)(x)) << MU_TSR_TE3_SHIFT)) & MU_TSR_TE3_MASK)
/*! @} */

/*! @name RCR - Receive Control */
/*! @{ */

#define MU_RCR_RIE0_MASK                         (0x1U)
#define MU_RCR_RIE0_SHIFT                        (0U)
/*! RIE0 - MUA Receive Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_RCR_RIE0(x)                           (((uint32_t)(((uint32_t)(x)) << MU_RCR_RIE0_SHIFT)) & MU_RCR_RIE0_MASK)

#define MU_RCR_RIE1_MASK                         (0x2U)
#define MU_RCR_RIE1_SHIFT                        (1U)
/*! RIE1 - MUA Receive Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_RCR_RIE1(x)                           (((uint32_t)(((uint32_t)(x)) << MU_RCR_RIE1_SHIFT)) & MU_RCR_RIE1_MASK)

#define MU_RCR_RIE2_MASK                         (0x4U)
#define MU_RCR_RIE2_SHIFT                        (2U)
/*! RIE2 - MUA Receive Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_RCR_RIE2(x)                           (((uint32_t)(((uint32_t)(x)) << MU_RCR_RIE2_SHIFT)) & MU_RCR_RIE2_MASK)

#define MU_RCR_RIE3_MASK                         (0x8U)
#define MU_RCR_RIE3_SHIFT                        (3U)
/*! RIE3 - MUA Receive Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define MU_RCR_RIE3(x)                           (((uint32_t)(((uint32_t)(x)) << MU_RCR_RIE3_SHIFT)) & MU_RCR_RIE3_MASK)
/*! @} */

/*! @name RSR - Receive Status */
/*! @{ */

#define MU_RSR_RF0_MASK                          (0x1U)
#define MU_RSR_RF0_SHIFT                         (0U)
/*! RF0 - MUA Receive Register Full
 *  0b0..Not full
 *  0b1..Full
 */
#define MU_RSR_RF0(x)                            (((uint32_t)(((uint32_t)(x)) << MU_RSR_RF0_SHIFT)) & MU_RSR_RF0_MASK)

#define MU_RSR_RF1_MASK                          (0x2U)
#define MU_RSR_RF1_SHIFT                         (1U)
/*! RF1 - MUA Receive Register Full
 *  0b0..Not full
 *  0b1..Full
 */
#define MU_RSR_RF1(x)                            (((uint32_t)(((uint32_t)(x)) << MU_RSR_RF1_SHIFT)) & MU_RSR_RF1_MASK)

#define MU_RSR_RF2_MASK                          (0x4U)
#define MU_RSR_RF2_SHIFT                         (2U)
/*! RF2 - MUA Receive Register Full
 *  0b0..Not full
 *  0b1..Full
 */
#define MU_RSR_RF2(x)                            (((uint32_t)(((uint32_t)(x)) << MU_RSR_RF2_SHIFT)) & MU_RSR_RF2_MASK)

#define MU_RSR_RF3_MASK                          (0x8U)
#define MU_RSR_RF3_SHIFT                         (3U)
/*! RF3 - MUA Receive Register Full
 *  0b0..Not full
 *  0b1..Full
 */
#define MU_RSR_RF3(x)                            (((uint32_t)(((uint32_t)(x)) << MU_RSR_RF3_SHIFT)) & MU_RSR_RF3_MASK)
/*! @} */

/*! @name TR - Transmit */
/*! @{ */

#define MU_TR_TR_DATA_MASK                       (0xFFFFFFFFU)
#define MU_TR_TR_DATA_SHIFT                      (0U)
/*! TR_DATA - MUA Transmit Data */
#define MU_TR_TR_DATA(x)                         (((uint32_t)(((uint32_t)(x)) << MU_TR_TR_DATA_SHIFT)) & MU_TR_TR_DATA_MASK)
/*! @} */

/* The count of MU_TR */
#define MU_TR_COUNT                              (4U)

/*! @name RR - Receive */
/*! @{ */

#define MU_RR_RR_DATA_MASK                       (0xFFFFFFFFU)
#define MU_RR_RR_DATA_SHIFT                      (0U)
/*! RR_DATA - MUA Receive Data */
#define MU_RR_RR_DATA(x)                         (((uint32_t)(((uint32_t)(x)) << MU_RR_RR_DATA_SHIFT)) & MU_RR_RR_DATA_MASK)
/*! @} */

/* The count of MU_RR */
#define MU_RR_COUNT                              (4U)


/*!
 * @}
 */ /* end of group MU_Register_Masks */


/* MU - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral MU1_MUA base address */
  #define MU1_MUA_BASE                             (0x54220000u)
  /** Peripheral MU1_MUA base address */
  #define MU1_MUA_BASE_NS                          (0x44220000u)
  /** Peripheral MU1_MUA base pointer */
  #define MU1_MUA                                  ((MU_Type *)MU1_MUA_BASE)
  /** Peripheral MU1_MUA base pointer */
  #define MU1_MUA_NS                               ((MU_Type *)MU1_MUA_BASE_NS)
  /** Peripheral MU2_MUA base address */
  #define MU2_MUA_BASE                             (0x52430000u)
  /** Peripheral MU2_MUA base address */
  #define MU2_MUA_BASE_NS                          (0x42430000u)
  /** Peripheral MU2_MUA base pointer */
  #define MU2_MUA                                  ((MU_Type *)MU2_MUA_BASE)
  /** Peripheral MU2_MUA base pointer */
  #define MU2_MUA_NS                               ((MU_Type *)MU2_MUA_BASE_NS)
  /** Array initializer of MU peripheral base addresses */
  #define MU_BASE_ADDRS                            { MU1_MUA_BASE, MU2_MUA_BASE }
  /** Array initializer of MU peripheral base pointers */
  #define MU_BASE_PTRS                             { MU1_MUA, MU2_MUA }
  /** Array initializer of MU peripheral base addresses */
  #define MU_BASE_ADDRS_NS                         { MU1_MUA_BASE_NS, MU2_MUA_BASE_NS }
  /** Array initializer of MU peripheral base pointers */
  #define MU_BASE_PTRS_NS                          { MU1_MUA_NS, MU2_MUA_NS }
#else
  /** Peripheral MU1_MUA base address */
  #define MU1_MUA_BASE                             (0x44220000u)
  /** Peripheral MU1_MUA base pointer */
  #define MU1_MUA                                  ((MU_Type *)MU1_MUA_BASE)
  /** Peripheral MU2_MUA base address */
  #define MU2_MUA_BASE                             (0x42430000u)
  /** Peripheral MU2_MUA base pointer */
  #define MU2_MUA                                  ((MU_Type *)MU2_MUA_BASE)
  /** Array initializer of MU peripheral base addresses */
  #define MU_BASE_ADDRS                            { MU1_MUA_BASE, MU2_MUA_BASE }
  /** Array initializer of MU peripheral base pointers */
  #define MU_BASE_PTRS                             { MU1_MUA, MU2_MUA }
#endif
/** Interrupt vectors for the MU peripheral type */
#define MU_IRQS                                  { MU1_IRQn, MU2_IRQn }

/*!
 * @}
 */ /* end of group MU_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_ENETC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_ENETC_Peripheral_Access_Layer NETC_ENETC Peripheral Access Layer
 * @{
 */

/** NETC_ENETC - Register Layout Typedef */
typedef struct {
  __I  uint32_t ECAPR0;                            /**< ENETC capability register 0, offset: 0x0 */
  __I  uint32_t ECAPR1;                            /**< ENETC capability register 1, offset: 0x4 */
  __I  uint32_t ECAPR2;                            /**< ENETC capability register 2, offset: 0x8 */
       uint8_t RESERVED_0[4];
  __IO uint32_t PMR;                               /**< Port mode register, offset: 0x10 */
       uint8_t RESERVED_1[108];
  __IO uint32_t PONVLANR;                          /**< Port outer native VLAN register, offset: 0x80 */
  __IO uint32_t PINVLANR;                          /**< Port inner native VLAN register, offset: 0x84 */
  __IO uint32_t PVCLCTR;                           /**< Port VLAN classification control register, offset: 0x88 */
       uint8_t RESERVED_2[16];
  __IO uint32_t PARCSCR;                           /**< Parser checksum configuration register, offset: 0x9C */
  __IO uint32_t PARCECR[4];                        /**< Parser custom Ethertype 0 configuration register..Parser custom Ethertype 3 configuration register, array offset: 0xA0, array step: 0x4 */
       uint8_t RESERVED_3[88];
  __IO uint32_t PPAUONTR;                          /**< Port pause ON threshold register, offset: 0x108 */
  __IO uint32_t PPAUOFFTR;                         /**< Port pause OFF threshold register, offset: 0x10C */
       uint8_t RESERVED_4[16];
  __I  uint32_t PRXMBER;                           /**< Port receive memory buffer entitlement register, offset: 0x120 */
  __I  uint32_t PRXMBLR;                           /**< Port receive memory buffer limit register, offset: 0x124 */
  __I  uint32_t PRXBCR;                            /**< Port receive buffer count register, offset: 0x128 */
  __I  uint32_t PRXBCHWMR;                         /**< Port receive buffer count high watermark register, offset: 0x12C */
       uint8_t RESERVED_5[16];
  struct {                                         /* offset: 0x140, array step: 0x10 */
    __I  uint32_t PICDRDCR;                          /**< Port ingress congestion DR0 discard count register..Port ingress congestion DR3 discard count register, array offset: 0x140, array step: 0x10 */
         uint8_t RESERVED_0[4];
    __I  uint32_t PICDRDCRRR;                        /**< Port ingress congestion DR0 discard count read-reset register..Port ingress congestion DR3 discard count read-reset register, array offset: 0x148, array step: 0x10 */
         uint8_t RESERVED_1[4];
  } PICDRADCR[4];
  __IO uint32_t PICPDSR;                           /**< Port ingress congestion priority discard status register, offset: 0x180 */
       uint8_t RESERVED_6[124];
  __IO uint32_t PSIPMMR;                           /**< Port station interface promiscuous MAC mode register, offset: 0x200 */
  __IO uint32_t PSIPVMR;                           /**< Port station interface promiscuous VLAN mode register, offset: 0x204 */
  __I  uint32_t PBFDSIR;                           /**< Port broadcast frames dropped due to MAC filtering register, offset: 0x208 */
  __I  uint32_t PFDMSAPR;                          /**< Port frame drop MAC source address pruning register, offset: 0x20C */
       uint8_t RESERVED_7[112];
  __I  uint32_t PSIMAFCAPR;                        /**< Port station interface MAC address filtering capability register, offset: 0x280 */
  __I  uint32_t PUFDMFR;                           /**< Port unicast frames dropped due to MAC filtering register, offset: 0x284 */
  __I  uint32_t PMFDMFR;                           /**< Port multicast frames dropped due to MAC filtering register, offset: 0x288 */
       uint8_t RESERVED_8[52];
  __I  uint32_t PSIVLANFCAPR;                      /**< Port station interface VLAN filtering capability register, offset: 0x2C0 */
  __IO uint32_t PSIVLANFMR;                        /**< Port station interface VLAN filtering mode register, offset: 0x2C4 */
       uint8_t RESERVED_9[8];
  __I  uint32_t PUFDVFR;                           /**< Port unicast frames dropped VLAN filtering register, offset: 0x2D0 */
  __I  uint32_t PMFDVFR;                           /**< Port multicast frames dropped VLAN filtering register, offset: 0x2D4 */
  __I  uint32_t PBFDVFR;                           /**< Port broadcast frames dropped VLAN filtering register, offset: 0x2D8 */
       uint8_t RESERVED_10[100];
  __IO uint32_t PLPMR;                             /**< Port low power mode register, offset: 0x340, available only on: ENETC0_BASE (missing on ENETC1_BASE) */
  __I  uint32_t PWOSR;                             /**< Port wake-on status register, offset: 0x344, available only on: ENETC0_BASE (missing on ENETC1_BASE) */
       uint8_t RESERVED_11[40];
  __IO uint32_t IPV2ICMPMR0;                       /**< Receive IPV to ICM priority mapping register 0, offset: 0x370 */
       uint8_t RESERVED_12[12];
  __IO uint32_t PRIO2TCMR0;                        /**< Transmit priority to traffic class mapping register 0, offset: 0x380 */
       uint8_t RESERVED_13[12];
  __IO uint32_t PTCTSDR[8];                        /**< Port traffic class 0 time specific departure register..Port traffic class 7 time specific departure register, array offset: 0x390, array step: 0x4 */
       uint8_t RESERVED_14[1104];
  __I  uint32_t SMCAPR;                            /**< Switch management capability register, offset: 0x800 */
       uint8_t RESERVED_15[124];
  __IO uint32_t SMHRBDRMR[15];                     /**< Switch management host reason 1 receive BD ring mapping register..Switch management host reason 15 receive BD ring mapping register, array offset: 0x880, array step: 0x4, available only on: ENETC1_BASE (missing on ENETC0_BASE) */
       uint8_t RESERVED_16[5956];
  struct {                                         /* offset: 0x2000, array step: 0x80 */
    __IO uint32_t PSIPMAR0;                          /**< Port station interface 0 primary MAC address register 0..Port station interface 1 primary MAC address register 0, array offset: 0x2000, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIPMAR1;                          /**< Port station interface 0 primary MAC address register 1..Port station interface 1 primary MAC address register 1, array offset: 0x2004, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIVLANR;                          /**< Port station interface 0 VLAN register..Port station interface 1 VLAN register, array offset: 0x2008, array step: 0x80, irregular array, not all indices are valid */
         uint8_t RESERVED_0[4];
    __IO uint32_t PSICFGR0;                          /**< Port station interface 0 configuration register 0..Port station interface 1 configuration register 0, array offset: 0x2010, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSICFGR1;                          /**< Port station interface 1 configuration register 1, array offset: 0x2014, array step: 0x80, available only on: ENETC1_BASE (missing on ENETC0_BASE), valid indices: [1] */
    __IO uint32_t PSICFGR2;                          /**< Port station interface 0 configuration register 2..Port station interface 1 configuration register 2, array offset: 0x2018, array step: 0x80, irregular array, not all indices are valid */
         uint8_t RESERVED_1[20];
    __IO uint32_t PSIVMAFCFGR;                       /**< Port station interface 0 VSI MAC address filtering configuration register..Port station interface 1 VSI MAC address filtering configuration register, array offset: 0x2030, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIVLANFCFGR;                      /**< Port station interface 0 VLAN filtering configuration register..Port station interface 1 VLAN filtering configuration register, array offset: 0x2034, array step: 0x80, irregular array, not all indices are valid */
         uint8_t RESERVED_2[24];
    __IO uint32_t PSIUMHFR0;                         /**< Port station interface 0 unicast MAC hash filter register 0..Port station interface 1 unicast MAC hash filter register 0, array offset: 0x2050, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIUMHFR1;                         /**< Port station interface 0 unicast MAC hash filter register 1..Port station interface 1 unicast MAC hash filter register 1, array offset: 0x2054, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIMMHFR0;                         /**< Port station interface 0 multicast MAC hash filter register 0..Port station interface 1 multicast MAC hash filter register 0, array offset: 0x2058, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIMMHFR1;                         /**< Port station interface 0 multicast MAC hash filter register 1..Port station interface 1 multicast MAC hash filter register 1, array offset: 0x205C, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIVHFR0;                          /**< Port station interface 0 VLAN hash filter register 0..Port station interface 1 VLAN hash filter register 0, array offset: 0x2060, array step: 0x80, irregular array, not all indices are valid */
    __IO uint32_t PSIVHFR1;                          /**< Port station interface 0 VLAN hash filter register 1..Port station interface 1 VLAN hash filter register 1, array offset: 0x2064, array step: 0x80, irregular array, not all indices are valid */
         uint8_t RESERVED_3[24];
  } NUM_SI[2];
} NETC_ENETC_Type;

/* ----------------------------------------------------------------------------
   -- NETC_ENETC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_ENETC_Register_Masks NETC_ENETC Register Masks
 * @{
 */

/*! @name ECAPR0 - ENETC capability register 0 */
/*! @{ */

#define NETC_ENETC_ECAPR0_RFS_MASK               (0x4U)
#define NETC_ENETC_ECAPR0_RFS_SHIFT              (2U)
#define NETC_ENETC_ECAPR0_RFS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR0_RFS_SHIFT)) & NETC_ENETC_ECAPR0_RFS_MASK)

#define NETC_ENETC_ECAPR0_TSD_MASK               (0x20U)
#define NETC_ENETC_ECAPR0_TSD_SHIFT              (5U)
#define NETC_ENETC_ECAPR0_TSD(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR0_TSD_SHIFT)) & NETC_ENETC_ECAPR0_TSD_MASK)

#define NETC_ENETC_ECAPR0_RSS_MASK               (0x100U)
#define NETC_ENETC_ECAPR0_RSS_SHIFT              (8U)
#define NETC_ENETC_ECAPR0_RSS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR0_RSS_SHIFT)) & NETC_ENETC_ECAPR0_RSS_MASK)

#define NETC_ENETC_ECAPR0_WO_MASK                (0x2000U)
#define NETC_ENETC_ECAPR0_WO_SHIFT               (13U)
#define NETC_ENETC_ECAPR0_WO(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR0_WO_SHIFT)) & NETC_ENETC_ECAPR0_WO_MASK)

#define NETC_ENETC_ECAPR0_FS_MASK                (0x10000U)
#define NETC_ENETC_ECAPR0_FS_SHIFT               (16U)
/*! FS - Functional safety capability supported. */
#define NETC_ENETC_ECAPR0_FS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR0_FS_SHIFT)) & NETC_ENETC_ECAPR0_FS_MASK)
/*! @} */

/*! @name ECAPR1 - ENETC capability register 1 */
/*! @{ */

#define NETC_ENETC_ECAPR1_NUM_TCS_MASK           (0x70U)
#define NETC_ENETC_ECAPR1_NUM_TCS_SHIFT          (4U)
#define NETC_ENETC_ECAPR1_NUM_TCS(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR1_NUM_TCS_SHIFT)) & NETC_ENETC_ECAPR1_NUM_TCS_MASK)

#define NETC_ENETC_ECAPR1_NUM_MCH_MASK           (0x300U)
#define NETC_ENETC_ECAPR1_NUM_MCH_SHIFT          (8U)
#define NETC_ENETC_ECAPR1_NUM_MCH(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR1_NUM_MCH_SHIFT)) & NETC_ENETC_ECAPR1_NUM_MCH_MASK)

#define NETC_ENETC_ECAPR1_NUM_UCH_MASK           (0xC00U)
#define NETC_ENETC_ECAPR1_NUM_UCH_SHIFT          (10U)
#define NETC_ENETC_ECAPR1_NUM_UCH(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR1_NUM_UCH_SHIFT)) & NETC_ENETC_ECAPR1_NUM_UCH_MASK)

#define NETC_ENETC_ECAPR1_NUM_MSIX_MASK          (0x7FF000U)
#define NETC_ENETC_ECAPR1_NUM_MSIX_SHIFT         (12U)
/*! NUM_MSIX - Number of MSI-X */
#define NETC_ENETC_ECAPR1_NUM_MSIX(x)            (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR1_NUM_MSIX_SHIFT)) & NETC_ENETC_ECAPR1_NUM_MSIX_MASK)

#define NETC_ENETC_ECAPR1_NUM_VSI_MASK           (0xF000000U)
#define NETC_ENETC_ECAPR1_NUM_VSI_SHIFT          (24U)
#define NETC_ENETC_ECAPR1_NUM_VSI(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR1_NUM_VSI_SHIFT)) & NETC_ENETC_ECAPR1_NUM_VSI_MASK)

#define NETC_ENETC_ECAPR1_NUM_IPV_MASK           (0x80000000U)
#define NETC_ENETC_ECAPR1_NUM_IPV_SHIFT          (31U)
#define NETC_ENETC_ECAPR1_NUM_IPV(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR1_NUM_IPV_SHIFT)) & NETC_ENETC_ECAPR1_NUM_IPV_MASK)
/*! @} */

/*! @name ECAPR2 - ENETC capability register 2 */
/*! @{ */

#define NETC_ENETC_ECAPR2_NUM_TX_BDR_MASK        (0x3FFU)
#define NETC_ENETC_ECAPR2_NUM_TX_BDR_SHIFT       (0U)
#define NETC_ENETC_ECAPR2_NUM_TX_BDR(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR2_NUM_TX_BDR_SHIFT)) & NETC_ENETC_ECAPR2_NUM_TX_BDR_MASK)

#define NETC_ENETC_ECAPR2_NUM_RX_BDR_MASK        (0x3FF0000U)
#define NETC_ENETC_ECAPR2_NUM_RX_BDR_SHIFT       (16U)
#define NETC_ENETC_ECAPR2_NUM_RX_BDR(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_ECAPR2_NUM_RX_BDR_SHIFT)) & NETC_ENETC_ECAPR2_NUM_RX_BDR_MASK)
/*! @} */

/*! @name PMR - Port mode register */
/*! @{ */

#define NETC_ENETC_PMR_SI0EN_MASK                (0x10000U)
#define NETC_ENETC_PMR_SI0EN_SHIFT               (16U)
/*! SI0EN
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PMR_SI0EN(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PMR_SI0EN_SHIFT)) & NETC_ENETC_PMR_SI0EN_MASK)

#define NETC_ENETC_PMR_SI1EN_MASK                (0x20000U)
#define NETC_ENETC_PMR_SI1EN_SHIFT               (17U)
/*! SI1EN
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PMR_SI1EN(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PMR_SI1EN_SHIFT)) & NETC_ENETC_PMR_SI1EN_MASK)
/*! @} */

/*! @name PONVLANR - Port outer native VLAN register */
/*! @{ */

#define NETC_ENETC_PONVLANR_VID_MASK             (0xFFFU)
#define NETC_ENETC_PONVLANR_VID_SHIFT            (0U)
#define NETC_ENETC_PONVLANR_VID(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PONVLANR_VID_SHIFT)) & NETC_ENETC_PONVLANR_VID_MASK)

#define NETC_ENETC_PONVLANR_DEI_MASK             (0x1000U)
#define NETC_ENETC_PONVLANR_DEI_SHIFT            (12U)
#define NETC_ENETC_PONVLANR_DEI(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PONVLANR_DEI_SHIFT)) & NETC_ENETC_PONVLANR_DEI_MASK)

#define NETC_ENETC_PONVLANR_PCP_MASK             (0xE000U)
#define NETC_ENETC_PONVLANR_PCP_SHIFT            (13U)
#define NETC_ENETC_PONVLANR_PCP(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PONVLANR_PCP_SHIFT)) & NETC_ENETC_PONVLANR_PCP_MASK)

#define NETC_ENETC_PONVLANR_TPID_MASK            (0x30000U)
#define NETC_ENETC_PONVLANR_TPID_SHIFT           (16U)
/*! TPID
 *  0b00..Standard C-VLAN 0x8100
 *  0b01..Standard S-VLAN 0x88A8
 *  0b10..Custom VLAN as defined by CVLANR1[ETYPE]
 *  0b11..Custom VLAN as defined by CVLANR2[ETYPE]
 */
#define NETC_ENETC_PONVLANR_TPID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PONVLANR_TPID_SHIFT)) & NETC_ENETC_PONVLANR_TPID_MASK)

#define NETC_ENETC_PONVLANR_PNE_MASK             (0x40000U)
#define NETC_ENETC_PONVLANR_PNE_SHIFT            (18U)
/*! PNE - Port Native VLAN Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PONVLANR_PNE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PONVLANR_PNE_SHIFT)) & NETC_ENETC_PONVLANR_PNE_MASK)

#define NETC_ENETC_PONVLANR_VZE_MASK             (0x80000U)
#define NETC_ENETC_PONVLANR_VZE_SHIFT            (19U)
/*! VZE - VID 0 Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PONVLANR_VZE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PONVLANR_VZE_SHIFT)) & NETC_ENETC_PONVLANR_VZE_MASK)
/*! @} */

/*! @name PINVLANR - Port inner native VLAN register */
/*! @{ */

#define NETC_ENETC_PINVLANR_VID_MASK             (0xFFFU)
#define NETC_ENETC_PINVLANR_VID_SHIFT            (0U)
#define NETC_ENETC_PINVLANR_VID(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PINVLANR_VID_SHIFT)) & NETC_ENETC_PINVLANR_VID_MASK)

#define NETC_ENETC_PINVLANR_DEI_MASK             (0x1000U)
#define NETC_ENETC_PINVLANR_DEI_SHIFT            (12U)
#define NETC_ENETC_PINVLANR_DEI(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PINVLANR_DEI_SHIFT)) & NETC_ENETC_PINVLANR_DEI_MASK)

#define NETC_ENETC_PINVLANR_PCP_MASK             (0xE000U)
#define NETC_ENETC_PINVLANR_PCP_SHIFT            (13U)
#define NETC_ENETC_PINVLANR_PCP(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PINVLANR_PCP_SHIFT)) & NETC_ENETC_PINVLANR_PCP_MASK)

#define NETC_ENETC_PINVLANR_TPID_MASK            (0x30000U)
#define NETC_ENETC_PINVLANR_TPID_SHIFT           (16U)
/*! TPID
 *  0b00..Standard C-VLAN 0x8100
 *  0b01..Standard S-VLAN 0x88A8
 *  0b10..Custom VLAN as defined by CVLANR1[ETYPE]
 *  0b11..Custom VLAN as defined by CVLANR2[ETYPE]
 */
#define NETC_ENETC_PINVLANR_TPID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PINVLANR_TPID_SHIFT)) & NETC_ENETC_PINVLANR_TPID_MASK)

#define NETC_ENETC_PINVLANR_PNE_MASK             (0x40000U)
#define NETC_ENETC_PINVLANR_PNE_SHIFT            (18U)
/*! PNE - Port Native VLAN Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PINVLANR_PNE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PINVLANR_PNE_SHIFT)) & NETC_ENETC_PINVLANR_PNE_MASK)

#define NETC_ENETC_PINVLANR_VZE_MASK             (0x80000U)
#define NETC_ENETC_PINVLANR_VZE_SHIFT            (19U)
/*! VZE - VID 0 Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PINVLANR_VZE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PINVLANR_VZE_SHIFT)) & NETC_ENETC_PINVLANR_VZE_MASK)
/*! @} */

/*! @name PVCLCTR - Port VLAN classification control register */
/*! @{ */

#define NETC_ENETC_PVCLCTR_OAI_MASK              (0x200U)
#define NETC_ENETC_PVCLCTR_OAI_SHIFT             (9U)
/*! OAI - Outer as Inner
 *  0b0..Indicates that the Inner is not valid if only one tag is found
 *  0b1..Indicates that the outer should be used as the Inner if only one tag is found
 */
#define NETC_ENETC_PVCLCTR_OAI(x)                (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PVCLCTR_OAI_SHIFT)) & NETC_ENETC_PVCLCTR_OAI_MASK)
/*! @} */

/*! @name PARCSCR - Parser checksum configuration register */
/*! @{ */

#define NETC_ENETC_PARCSCR_L4CD_MASK             (0x1U)
#define NETC_ENETC_PARCSCR_L4CD_SHIFT            (0U)
/*! L4CD
 *  0b0..Enabled
 *  0b1..Disabled
 */
#define NETC_ENETC_PARCSCR_L4CD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PARCSCR_L4CD_SHIFT)) & NETC_ENETC_PARCSCR_L4CD_MASK)

#define NETC_ENETC_PARCSCR_L3CD_MASK             (0x2U)
#define NETC_ENETC_PARCSCR_L3CD_SHIFT            (1U)
/*! L3CD
 *  0b0..Enabled
 *  0b1..Disabled
 */
#define NETC_ENETC_PARCSCR_L3CD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PARCSCR_L3CD_SHIFT)) & NETC_ENETC_PARCSCR_L3CD_MASK)
/*! @} */

/*! @name PARCECR - Parser custom Ethertype 0 configuration register..Parser custom Ethertype 3 configuration register */
/*! @{ */

#define NETC_ENETC_PARCECR_CP_MASK               (0xFU)
#define NETC_ENETC_PARCECR_CP_SHIFT              (0U)
/*! CP - Code Point */
#define NETC_ENETC_PARCECR_CP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PARCECR_CP_SHIFT)) & NETC_ENETC_PARCECR_CP_MASK)

#define NETC_ENETC_PARCECR_EN_MASK               (0x20U)
#define NETC_ENETC_PARCECR_EN_SHIFT              (5U)
/*! EN - Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PARCECR_EN(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PARCECR_EN_SHIFT)) & NETC_ENETC_PARCECR_EN_MASK)

#define NETC_ENETC_PARCECR_ETYPE_MASK            (0xFFFF0000U)
#define NETC_ENETC_PARCECR_ETYPE_SHIFT           (16U)
/*! ETYPE - ETYPE */
#define NETC_ENETC_PARCECR_ETYPE(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PARCECR_ETYPE_SHIFT)) & NETC_ENETC_PARCECR_ETYPE_MASK)
/*! @} */

/* The count of NETC_ENETC_PARCECR */
#define NETC_ENETC_PARCECR_COUNT                 (4U)

/*! @name PPAUONTR - Port pause ON threshold register */
/*! @{ */

#define NETC_ENETC_PPAUONTR_THRESH_MASK          (0xFFFFFFU)
#define NETC_ENETC_PPAUONTR_THRESH_SHIFT         (0U)
#define NETC_ENETC_PPAUONTR_THRESH(x)            (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PPAUONTR_THRESH_SHIFT)) & NETC_ENETC_PPAUONTR_THRESH_MASK)
/*! @} */

/*! @name PPAUOFFTR - Port pause OFF threshold register */
/*! @{ */

#define NETC_ENETC_PPAUOFFTR_THRESH_MASK         (0xFFFFFFU)
#define NETC_ENETC_PPAUOFFTR_THRESH_SHIFT        (0U)
#define NETC_ENETC_PPAUOFFTR_THRESH(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PPAUOFFTR_THRESH_SHIFT)) & NETC_ENETC_PPAUOFFTR_THRESH_MASK)
/*! @} */

/*! @name PRXMBER - Port receive memory buffer entitlement register */
/*! @{ */

#define NETC_ENETC_PRXMBER_AMOUNT_MASK           (0xFFFFFFU)
#define NETC_ENETC_PRXMBER_AMOUNT_SHIFT          (0U)
#define NETC_ENETC_PRXMBER_AMOUNT(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRXMBER_AMOUNT_SHIFT)) & NETC_ENETC_PRXMBER_AMOUNT_MASK)
/*! @} */

/*! @name PRXMBLR - Port receive memory buffer limit register */
/*! @{ */

#define NETC_ENETC_PRXMBLR_LIMIT_MASK            (0xFFFFFFU)
#define NETC_ENETC_PRXMBLR_LIMIT_SHIFT           (0U)
#define NETC_ENETC_PRXMBLR_LIMIT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRXMBLR_LIMIT_SHIFT)) & NETC_ENETC_PRXMBLR_LIMIT_MASK)
/*! @} */

/*! @name PRXBCR - Port receive buffer count register */
/*! @{ */

#define NETC_ENETC_PRXBCR_COUNT_MASK             (0xFFFFFFU)
#define NETC_ENETC_PRXBCR_COUNT_SHIFT            (0U)
#define NETC_ENETC_PRXBCR_COUNT(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRXBCR_COUNT_SHIFT)) & NETC_ENETC_PRXBCR_COUNT_MASK)
/*! @} */

/*! @name PRXBCHWMR - Port receive buffer count high watermark register */
/*! @{ */

#define NETC_ENETC_PRXBCHWMR_WATERMARK_MASK      (0xFFFFFFU)
#define NETC_ENETC_PRXBCHWMR_WATERMARK_SHIFT     (0U)
#define NETC_ENETC_PRXBCHWMR_WATERMARK(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRXBCHWMR_WATERMARK_SHIFT)) & NETC_ENETC_PRXBCHWMR_WATERMARK_MASK)
/*! @} */

/*! @name PICDRDCR - Port ingress congestion DR0 discard count register..Port ingress congestion DR3 discard count register */
/*! @{ */

#define NETC_ENETC_PICDRDCR_COUNT_MASK           (0xFFFFFFFFU)
#define NETC_ENETC_PICDRDCR_COUNT_SHIFT          (0U)
#define NETC_ENETC_PICDRDCR_COUNT(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICDRDCR_COUNT_SHIFT)) & NETC_ENETC_PICDRDCR_COUNT_MASK)
/*! @} */

/* The count of NETC_ENETC_PICDRDCR */
#define NETC_ENETC_PICDRADCR_PICDRDCR_COUNT      (4U)

/*! @name PICDRDCRRR - Port ingress congestion DR0 discard count read-reset register..Port ingress congestion DR3 discard count read-reset register */
/*! @{ */

#define NETC_ENETC_PICDRDCRRR_COUNT_MASK         (0xFFFFFFFFU)
#define NETC_ENETC_PICDRDCRRR_COUNT_SHIFT        (0U)
#define NETC_ENETC_PICDRDCRRR_COUNT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICDRDCRRR_COUNT_SHIFT)) & NETC_ENETC_PICDRDCRRR_COUNT_MASK)
/*! @} */

/* The count of NETC_ENETC_PICDRDCRRR */
#define NETC_ENETC_PICDRADCR_PICDRDCRRR_COUNT    (4U)

/*! @name PICPDSR - Port ingress congestion priority discard status register */
/*! @{ */

#define NETC_ENETC_PICPDSR_DR0_P0DS_MASK         (0x1U)
#define NETC_ENETC_PICPDSR_DR0_P0DS_SHIFT        (0U)
#define NETC_ENETC_PICPDSR_DR0_P0DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR0_P0DS_SHIFT)) & NETC_ENETC_PICPDSR_DR0_P0DS_MASK)

#define NETC_ENETC_PICPDSR_DR0_P1DS_MASK         (0x10U)
#define NETC_ENETC_PICPDSR_DR0_P1DS_SHIFT        (4U)
#define NETC_ENETC_PICPDSR_DR0_P1DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR0_P1DS_SHIFT)) & NETC_ENETC_PICPDSR_DR0_P1DS_MASK)

#define NETC_ENETC_PICPDSR_DR1_P0DS_MASK         (0x100U)
#define NETC_ENETC_PICPDSR_DR1_P0DS_SHIFT        (8U)
#define NETC_ENETC_PICPDSR_DR1_P0DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR1_P0DS_SHIFT)) & NETC_ENETC_PICPDSR_DR1_P0DS_MASK)

#define NETC_ENETC_PICPDSR_DR1_P1DS_MASK         (0x1000U)
#define NETC_ENETC_PICPDSR_DR1_P1DS_SHIFT        (12U)
#define NETC_ENETC_PICPDSR_DR1_P1DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR1_P1DS_SHIFT)) & NETC_ENETC_PICPDSR_DR1_P1DS_MASK)

#define NETC_ENETC_PICPDSR_DR2_P0DS_MASK         (0x10000U)
#define NETC_ENETC_PICPDSR_DR2_P0DS_SHIFT        (16U)
#define NETC_ENETC_PICPDSR_DR2_P0DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR2_P0DS_SHIFT)) & NETC_ENETC_PICPDSR_DR2_P0DS_MASK)

#define NETC_ENETC_PICPDSR_DR2_P1DS_MASK         (0x100000U)
#define NETC_ENETC_PICPDSR_DR2_P1DS_SHIFT        (20U)
#define NETC_ENETC_PICPDSR_DR2_P1DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR2_P1DS_SHIFT)) & NETC_ENETC_PICPDSR_DR2_P1DS_MASK)

#define NETC_ENETC_PICPDSR_DR3_P0DS_MASK         (0x1000000U)
#define NETC_ENETC_PICPDSR_DR3_P0DS_SHIFT        (24U)
#define NETC_ENETC_PICPDSR_DR3_P0DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR3_P0DS_SHIFT)) & NETC_ENETC_PICPDSR_DR3_P0DS_MASK)

#define NETC_ENETC_PICPDSR_DR3_P1DS_MASK         (0x10000000U)
#define NETC_ENETC_PICPDSR_DR3_P1DS_SHIFT        (28U)
#define NETC_ENETC_PICPDSR_DR3_P1DS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PICPDSR_DR3_P1DS_SHIFT)) & NETC_ENETC_PICPDSR_DR3_P1DS_MASK)
/*! @} */

/*! @name PSIPMMR - Port station interface promiscuous MAC mode register */
/*! @{ */

#define NETC_ENETC_PSIPMMR_SI0_MAC_UP_MASK       (0x1U)
#define NETC_ENETC_PSIPMMR_SI0_MAC_UP_SHIFT      (0U)
/*! SI0_MAC_UP
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PSIPMMR_SI0_MAC_UP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPMMR_SI0_MAC_UP_SHIFT)) & NETC_ENETC_PSIPMMR_SI0_MAC_UP_MASK)

#define NETC_ENETC_PSIPMMR_SI1_MAC_UP_MASK       (0x2U)
#define NETC_ENETC_PSIPMMR_SI1_MAC_UP_SHIFT      (1U)
/*! SI1_MAC_UP
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PSIPMMR_SI1_MAC_UP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPMMR_SI1_MAC_UP_SHIFT)) & NETC_ENETC_PSIPMMR_SI1_MAC_UP_MASK)

#define NETC_ENETC_PSIPMMR_SI0_MAC_MP_MASK       (0x10000U)
#define NETC_ENETC_PSIPMMR_SI0_MAC_MP_SHIFT      (16U)
/*! SI0_MAC_MP
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PSIPMMR_SI0_MAC_MP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPMMR_SI0_MAC_MP_SHIFT)) & NETC_ENETC_PSIPMMR_SI0_MAC_MP_MASK)

#define NETC_ENETC_PSIPMMR_SI1_MAC_MP_MASK       (0x20000U)
#define NETC_ENETC_PSIPMMR_SI1_MAC_MP_SHIFT      (17U)
/*! SI1_MAC_MP
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PSIPMMR_SI1_MAC_MP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPMMR_SI1_MAC_MP_SHIFT)) & NETC_ENETC_PSIPMMR_SI1_MAC_MP_MASK)
/*! @} */

/*! @name PSIPVMR - Port station interface promiscuous VLAN mode register */
/*! @{ */

#define NETC_ENETC_PSIPVMR_SI0_VLAN_P_MASK       (0x1U)
#define NETC_ENETC_PSIPVMR_SI0_VLAN_P_SHIFT      (0U)
/*! SI0_VLAN_P
 *  0b0..SI 0 does not qualify for the reception of all VLAN tags
 *  0b1..SI 0 does qualify for the reception of all VLAN tags
 */
#define NETC_ENETC_PSIPVMR_SI0_VLAN_P(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPVMR_SI0_VLAN_P_SHIFT)) & NETC_ENETC_PSIPVMR_SI0_VLAN_P_MASK)

#define NETC_ENETC_PSIPVMR_SI1_VLAN_P_MASK       (0x2U)
#define NETC_ENETC_PSIPVMR_SI1_VLAN_P_SHIFT      (1U)
/*! SI1_VLAN_P
 *  0b0..SI 1 does not qualify for the reception of all VLAN tags
 *  0b1..SI 1 does qualify for the reception of all VLAN tags
 */
#define NETC_ENETC_PSIPVMR_SI1_VLAN_P(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPVMR_SI1_VLAN_P_SHIFT)) & NETC_ENETC_PSIPVMR_SI1_VLAN_P_MASK)

#define NETC_ENETC_PSIPVMR_SI0_VLAN_UTA_MASK     (0x10000U)
#define NETC_ENETC_PSIPVMR_SI0_VLAN_UTA_SHIFT    (16U)
/*! SI0_VLAN_UTA
 *  0b0..SI 0 does not qualify for reception of untagged frames
 *  0b1..SI 0 does qualify for reception of untagged frames
 */
#define NETC_ENETC_PSIPVMR_SI0_VLAN_UTA(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPVMR_SI0_VLAN_UTA_SHIFT)) & NETC_ENETC_PSIPVMR_SI0_VLAN_UTA_MASK)

#define NETC_ENETC_PSIPVMR_SI1_VLAN_UTA_MASK     (0x20000U)
#define NETC_ENETC_PSIPVMR_SI1_VLAN_UTA_SHIFT    (17U)
/*! SI1_VLAN_UTA
 *  0b0..SI 1 does not qualify for reception of untagged frames
 *  0b1..SI 1 does qualify for reception of untagged frames
 */
#define NETC_ENETC_PSIPVMR_SI1_VLAN_UTA(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPVMR_SI1_VLAN_UTA_SHIFT)) & NETC_ENETC_PSIPVMR_SI1_VLAN_UTA_MASK)
/*! @} */

/*! @name PBFDSIR - Port broadcast frames dropped due to MAC filtering register */
/*! @{ */

#define NETC_ENETC_PBFDSIR_FRAME_DROP_MASK       (0xFFFFFFFFU)
#define NETC_ENETC_PBFDSIR_FRAME_DROP_SHIFT      (0U)
#define NETC_ENETC_PBFDSIR_FRAME_DROP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PBFDSIR_FRAME_DROP_SHIFT)) & NETC_ENETC_PBFDSIR_FRAME_DROP_MASK)
/*! @} */

/*! @name PFDMSAPR - Port frame drop MAC source address pruning register */
/*! @{ */

#define NETC_ENETC_PFDMSAPR_FRAME_DROP_MASK      (0xFFFFFFFFU)
#define NETC_ENETC_PFDMSAPR_FRAME_DROP_SHIFT     (0U)
#define NETC_ENETC_PFDMSAPR_FRAME_DROP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PFDMSAPR_FRAME_DROP_SHIFT)) & NETC_ENETC_PFDMSAPR_FRAME_DROP_MASK)
/*! @} */

/*! @name PSIMAFCAPR - Port station interface MAC address filtering capability register */
/*! @{ */

#define NETC_ENETC_PSIMAFCAPR_NUM_MAC_AFTE_MASK  (0xFFFU)
#define NETC_ENETC_PSIMAFCAPR_NUM_MAC_AFTE_SHIFT (0U)
#define NETC_ENETC_PSIMAFCAPR_NUM_MAC_AFTE(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIMAFCAPR_NUM_MAC_AFTE_SHIFT)) & NETC_ENETC_PSIMAFCAPR_NUM_MAC_AFTE_MASK)
/*! @} */

/*! @name PUFDMFR - Port unicast frames dropped due to MAC filtering register */
/*! @{ */

#define NETC_ENETC_PUFDMFR_FRAME_DROP_MASK       (0xFFFFFFFFU)
#define NETC_ENETC_PUFDMFR_FRAME_DROP_SHIFT      (0U)
#define NETC_ENETC_PUFDMFR_FRAME_DROP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PUFDMFR_FRAME_DROP_SHIFT)) & NETC_ENETC_PUFDMFR_FRAME_DROP_MASK)
/*! @} */

/*! @name PMFDMFR - Port multicast frames dropped due to MAC filtering register */
/*! @{ */

#define NETC_ENETC_PMFDMFR_FRAME_DROP_MASK       (0xFFFFFFFFU)
#define NETC_ENETC_PMFDMFR_FRAME_DROP_SHIFT      (0U)
#define NETC_ENETC_PMFDMFR_FRAME_DROP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PMFDMFR_FRAME_DROP_SHIFT)) & NETC_ENETC_PMFDMFR_FRAME_DROP_MASK)
/*! @} */

/*! @name PSIVLANFCAPR - Port station interface VLAN filtering capability register */
/*! @{ */

#define NETC_ENETC_PSIVLANFCAPR_NUM_VLAN_FTE_MASK (0xFFFU)
#define NETC_ENETC_PSIVLANFCAPR_NUM_VLAN_FTE_SHIFT (0U)
#define NETC_ENETC_PSIVLANFCAPR_NUM_VLAN_FTE(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANFCAPR_NUM_VLAN_FTE_SHIFT)) & NETC_ENETC_PSIVLANFCAPR_NUM_VLAN_FTE_MASK)
/*! @} */

/*! @name PSIVLANFMR - Port station interface VLAN filtering mode register */
/*! @{ */

#define NETC_ENETC_PSIVLANFMR_VS_MASK            (0x1U)
#define NETC_ENETC_PSIVLANFMR_VS_SHIFT           (0U)
/*! VS
 *  0b0..Inner VLAN tag will be used for VLAN filtering
 *  0b1..Outer VLAN tag will be used for VLAN filtering
 */
#define NETC_ENETC_PSIVLANFMR_VS(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANFMR_VS_SHIFT)) & NETC_ENETC_PSIVLANFMR_VS_MASK)
/*! @} */

/*! @name PUFDVFR - Port unicast frames dropped VLAN filtering register */
/*! @{ */

#define NETC_ENETC_PUFDVFR_FRAME_DROP_MASK       (0xFFFFFFFFU)
#define NETC_ENETC_PUFDVFR_FRAME_DROP_SHIFT      (0U)
#define NETC_ENETC_PUFDVFR_FRAME_DROP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PUFDVFR_FRAME_DROP_SHIFT)) & NETC_ENETC_PUFDVFR_FRAME_DROP_MASK)
/*! @} */

/*! @name PMFDVFR - Port multicast frames dropped VLAN filtering register */
/*! @{ */

#define NETC_ENETC_PMFDVFR_FRAME_DROP_MASK       (0xFFFFFFFFU)
#define NETC_ENETC_PMFDVFR_FRAME_DROP_SHIFT      (0U)
#define NETC_ENETC_PMFDVFR_FRAME_DROP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PMFDVFR_FRAME_DROP_SHIFT)) & NETC_ENETC_PMFDVFR_FRAME_DROP_MASK)
/*! @} */

/*! @name PBFDVFR - Port broadcast frames dropped VLAN filtering register */
/*! @{ */

#define NETC_ENETC_PBFDVFR_FRAME_DROP_MASK       (0xFFFFFFFFU)
#define NETC_ENETC_PBFDVFR_FRAME_DROP_SHIFT      (0U)
#define NETC_ENETC_PBFDVFR_FRAME_DROP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PBFDVFR_FRAME_DROP_SHIFT)) & NETC_ENETC_PBFDVFR_FRAME_DROP_MASK)
/*! @} */

/*! @name PLPMR - Port low power mode register */
/*! @{ */

#define NETC_ENETC_PLPMR_WME_MASK                (0x1U)
#define NETC_ENETC_PLPMR_WME_SHIFT               (0U)
/*! WME
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PLPMR_WME(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PLPMR_WME_SHIFT)) & NETC_ENETC_PLPMR_WME_MASK)
/*! @} */

/*! @name PWOSR - Port wake-on status register */
/*! @{ */

#define NETC_ENETC_PWOSR_WOLA_MASK               (0x1U)
#define NETC_ENETC_PWOSR_WOLA_SHIFT              (0U)
/*! WOLA
 *  0b0..Inactive
 *  0b1..Active. ENETC is actively searching for frames matching the Wake-on-LAN event criteria.
 */
#define NETC_ENETC_PWOSR_WOLA(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PWOSR_WOLA_SHIFT)) & NETC_ENETC_PWOSR_WOLA_MASK)

#define NETC_ENETC_PWOSR_ICMB_MASK               (0x2U)
#define NETC_ENETC_PWOSR_ICMB_SHIFT              (1U)
/*! ICMB
 *  0b0..Not blocked
 *  0b1..Blocked.
 */
#define NETC_ENETC_PWOSR_ICMB(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PWOSR_ICMB_SHIFT)) & NETC_ENETC_PWOSR_ICMB_MASK)
/*! @} */

/*! @name IPV2ICMPMR0 - Receive IPV to ICM priority mapping register 0 */
/*! @{ */

#define NETC_ENETC_IPV2ICMPMR0_IPV0ICM_MASK      (0x1U)
#define NETC_ENETC_IPV2ICMPMR0_IPV0ICM_SHIFT     (0U)
#define NETC_ENETC_IPV2ICMPMR0_IPV0ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV0ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV0ICM_MASK)

#define NETC_ENETC_IPV2ICMPMR0_IPV1ICM_MASK      (0x10U)
#define NETC_ENETC_IPV2ICMPMR0_IPV1ICM_SHIFT     (4U)
#define NETC_ENETC_IPV2ICMPMR0_IPV1ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV1ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV1ICM_MASK)

#define NETC_ENETC_IPV2ICMPMR0_IPV2ICM_MASK      (0x100U)
#define NETC_ENETC_IPV2ICMPMR0_IPV2ICM_SHIFT     (8U)
#define NETC_ENETC_IPV2ICMPMR0_IPV2ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV2ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV2ICM_MASK)

#define NETC_ENETC_IPV2ICMPMR0_IPV3ICM_MASK      (0x1000U)
#define NETC_ENETC_IPV2ICMPMR0_IPV3ICM_SHIFT     (12U)
#define NETC_ENETC_IPV2ICMPMR0_IPV3ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV3ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV3ICM_MASK)

#define NETC_ENETC_IPV2ICMPMR0_IPV4ICM_MASK      (0x10000U)
#define NETC_ENETC_IPV2ICMPMR0_IPV4ICM_SHIFT     (16U)
#define NETC_ENETC_IPV2ICMPMR0_IPV4ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV4ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV4ICM_MASK)

#define NETC_ENETC_IPV2ICMPMR0_IPV5ICM_MASK      (0x100000U)
#define NETC_ENETC_IPV2ICMPMR0_IPV5ICM_SHIFT     (20U)
#define NETC_ENETC_IPV2ICMPMR0_IPV5ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV5ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV5ICM_MASK)

#define NETC_ENETC_IPV2ICMPMR0_IPV6ICM_MASK      (0x1000000U)
#define NETC_ENETC_IPV2ICMPMR0_IPV6ICM_SHIFT     (24U)
#define NETC_ENETC_IPV2ICMPMR0_IPV6ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV6ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV6ICM_MASK)

#define NETC_ENETC_IPV2ICMPMR0_IPV7ICM_MASK      (0x10000000U)
#define NETC_ENETC_IPV2ICMPMR0_IPV7ICM_SHIFT     (28U)
#define NETC_ENETC_IPV2ICMPMR0_IPV7ICM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_IPV2ICMPMR0_IPV7ICM_SHIFT)) & NETC_ENETC_IPV2ICMPMR0_IPV7ICM_MASK)
/*! @} */

/*! @name PRIO2TCMR0 - Transmit priority to traffic class mapping register 0 */
/*! @{ */

#define NETC_ENETC_PRIO2TCMR0_PRIO0TC_MASK       (0x7U)
#define NETC_ENETC_PRIO2TCMR0_PRIO0TC_SHIFT      (0U)
#define NETC_ENETC_PRIO2TCMR0_PRIO0TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO0TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO0TC_MASK)

#define NETC_ENETC_PRIO2TCMR0_PRIO1TC_MASK       (0x70U)
#define NETC_ENETC_PRIO2TCMR0_PRIO1TC_SHIFT      (4U)
#define NETC_ENETC_PRIO2TCMR0_PRIO1TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO1TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO1TC_MASK)

#define NETC_ENETC_PRIO2TCMR0_PRIO2TC_MASK       (0x700U)
#define NETC_ENETC_PRIO2TCMR0_PRIO2TC_SHIFT      (8U)
#define NETC_ENETC_PRIO2TCMR0_PRIO2TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO2TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO2TC_MASK)

#define NETC_ENETC_PRIO2TCMR0_PRIO3TC_MASK       (0x7000U)
#define NETC_ENETC_PRIO2TCMR0_PRIO3TC_SHIFT      (12U)
#define NETC_ENETC_PRIO2TCMR0_PRIO3TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO3TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO3TC_MASK)

#define NETC_ENETC_PRIO2TCMR0_PRIO4TC_MASK       (0x70000U)
#define NETC_ENETC_PRIO2TCMR0_PRIO4TC_SHIFT      (16U)
#define NETC_ENETC_PRIO2TCMR0_PRIO4TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO4TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO4TC_MASK)

#define NETC_ENETC_PRIO2TCMR0_PRIO5TC_MASK       (0x700000U)
#define NETC_ENETC_PRIO2TCMR0_PRIO5TC_SHIFT      (20U)
#define NETC_ENETC_PRIO2TCMR0_PRIO5TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO5TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO5TC_MASK)

#define NETC_ENETC_PRIO2TCMR0_PRIO6TC_MASK       (0x7000000U)
#define NETC_ENETC_PRIO2TCMR0_PRIO6TC_SHIFT      (24U)
#define NETC_ENETC_PRIO2TCMR0_PRIO6TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO6TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO6TC_MASK)

#define NETC_ENETC_PRIO2TCMR0_PRIO7TC_MASK       (0x70000000U)
#define NETC_ENETC_PRIO2TCMR0_PRIO7TC_SHIFT      (28U)
#define NETC_ENETC_PRIO2TCMR0_PRIO7TC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PRIO2TCMR0_PRIO7TC_SHIFT)) & NETC_ENETC_PRIO2TCMR0_PRIO7TC_MASK)
/*! @} */

/*! @name PTCTSDR - Port traffic class 0 time specific departure register..Port traffic class 7 time specific departure register */
/*! @{ */

#define NETC_ENETC_PTCTSDR_TSDE_MASK             (0x80000000U)
#define NETC_ENETC_PTCTSDR_TSDE_SHIFT            (31U)
/*! TSDE
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PTCTSDR_TSDE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PTCTSDR_TSDE_SHIFT)) & NETC_ENETC_PTCTSDR_TSDE_MASK)
/*! @} */

/* The count of NETC_ENETC_PTCTSDR */
#define NETC_ENETC_PTCTSDR_COUNT                 (8U)

/*! @name SMCAPR - Switch management capability register */
/*! @{ */

#define NETC_ENETC_SMCAPR_SM_MASK                (0x1U)
#define NETC_ENETC_SMCAPR_SM_SHIFT               (0U)
/*! SM - Switch Management
 *  0b0..ENETC instance has no switch management capability
 *  0b1..ENETC instance has switch management capability
 */
#define NETC_ENETC_SMCAPR_SM(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_SMCAPR_SM_SHIFT)) & NETC_ENETC_SMCAPR_SM_MASK)
/*! @} */

/*! @name SMHRBDRMR - Switch management host reason 1 receive BD ring mapping register..Switch management host reason 15 receive BD ring mapping register */
/*! @{ */

#define NETC_ENETC_SMHRBDRMR_RXBDR_MASK          (0xFFU)
#define NETC_ENETC_SMHRBDRMR_RXBDR_SHIFT         (0U)
#define NETC_ENETC_SMHRBDRMR_RXBDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_SMHRBDRMR_RXBDR_SHIFT)) & NETC_ENETC_SMHRBDRMR_RXBDR_MASK)
/*! @} */

/* The count of NETC_ENETC_SMHRBDRMR */
#define NETC_ENETC_SMHRBDRMR_COUNT               (15U)

/*! @name PSIPMAR0 - Port station interface 0 primary MAC address register 0..Port station interface 1 primary MAC address register 0 */
/*! @{ */

#define NETC_ENETC_PSIPMAR0_PRIM_MAC_ADDR_MASK   (0xFFFFFFFFU)
#define NETC_ENETC_PSIPMAR0_PRIM_MAC_ADDR_SHIFT  (0U)
#define NETC_ENETC_PSIPMAR0_PRIM_MAC_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPMAR0_PRIM_MAC_ADDR_SHIFT)) & NETC_ENETC_PSIPMAR0_PRIM_MAC_ADDR_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIPMAR0 */
#define NETC_ENETC_PSIPMAR0_COUNT                (2U)

/*! @name PSIPMAR1 - Port station interface 0 primary MAC address register 1..Port station interface 1 primary MAC address register 1 */
/*! @{ */

#define NETC_ENETC_PSIPMAR1_PRIM_MAC_ADDR_MASK   (0xFFFFU)
#define NETC_ENETC_PSIPMAR1_PRIM_MAC_ADDR_SHIFT  (0U)
#define NETC_ENETC_PSIPMAR1_PRIM_MAC_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIPMAR1_PRIM_MAC_ADDR_SHIFT)) & NETC_ENETC_PSIPMAR1_PRIM_MAC_ADDR_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIPMAR1 */
#define NETC_ENETC_PSIPMAR1_COUNT                (2U)

/*! @name PSIVLANR - Port station interface 0 VLAN register..Port station interface 1 VLAN register */
/*! @{ */

#define NETC_ENETC_PSIVLANR_VID_MASK             (0xFFFU)
#define NETC_ENETC_PSIVLANR_VID_SHIFT            (0U)
#define NETC_ENETC_PSIVLANR_VID(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANR_VID_SHIFT)) & NETC_ENETC_PSIVLANR_VID_MASK)

#define NETC_ENETC_PSIVLANR_DEI_MASK             (0x1000U)
#define NETC_ENETC_PSIVLANR_DEI_SHIFT            (12U)
#define NETC_ENETC_PSIVLANR_DEI(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANR_DEI_SHIFT)) & NETC_ENETC_PSIVLANR_DEI_MASK)

#define NETC_ENETC_PSIVLANR_PCP_MASK             (0xE000U)
#define NETC_ENETC_PSIVLANR_PCP_SHIFT            (13U)
#define NETC_ENETC_PSIVLANR_PCP(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANR_PCP_SHIFT)) & NETC_ENETC_PSIVLANR_PCP_MASK)

#define NETC_ENETC_PSIVLANR_TPID_MASK            (0x30000U)
#define NETC_ENETC_PSIVLANR_TPID_SHIFT           (16U)
/*! TPID
 *  0b00..Standard C-VLAN 0x8100
 *  0b01..Standard S-VLAN 0x88A8
 *  0b10..Custom VLAN as defined by CVLANR1[ETYPE]. Note that CVLANR1[V] is not checked for SI-based VLAN
 *        insertion; TPID value specified in CVLANR1[ETYPE] will be used to construct the VLAN header regardless of the
 *        value specified in CVLANR1[V].
 *  0b11..Custom VLAN as defined by CVLANR2[ETYPE]. Note that CVLANR2[V] is not checked for SI-based VLAN
 *        insertion; TPID value specified in CVLANR2[ETYPE] will be used to construct the VLAN header regardless of the
 *        value specified in CVLANR2[V].
 */
#define NETC_ENETC_PSIVLANR_TPID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANR_TPID_SHIFT)) & NETC_ENETC_PSIVLANR_TPID_MASK)

#define NETC_ENETC_PSIVLANR_E_MASK               (0x80000000U)
#define NETC_ENETC_PSIVLANR_E_SHIFT              (31U)
/*! E - Enable
 *  0b0..Disabled
 *  0b1..Enabled; SI-based VLAN information is added on transmit and removed on receive.
 */
#define NETC_ENETC_PSIVLANR_E(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANR_E_SHIFT)) & NETC_ENETC_PSIVLANR_E_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIVLANR */
#define NETC_ENETC_PSIVLANR_COUNT                (2U)

/*! @name PSICFGR0 - Port station interface 0 configuration register 0..Port station interface 1 configuration register 0 */
/*! @{ */

#define NETC_ENETC_PSICFGR0_NUM_TX_BDR_MASK      (0x7FU)
#define NETC_ENETC_PSICFGR0_NUM_TX_BDR_SHIFT     (0U)
#define NETC_ENETC_PSICFGR0_NUM_TX_BDR(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_NUM_TX_BDR_SHIFT)) & NETC_ENETC_PSICFGR0_NUM_TX_BDR_MASK)

#define NETC_ENETC_PSICFGR0_SPE_MASK             (0x800U)
#define NETC_ENETC_PSICFGR0_SPE_SHIFT            (11U)
/*! SPE - Source Pruning Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_ENETC_PSICFGR0_SPE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_SPE_SHIFT)) & NETC_ENETC_PSICFGR0_SPE_MASK)

#define NETC_ENETC_PSICFGR0_VTE_MASK             (0x1000U)
#define NETC_ENETC_PSICFGR0_VTE_SHIFT            (12U)
/*! VTE - VLAN Tag Extract
 *  0b0..SI-based VLAN removal disabled
 *  0b1..SI-based VLAN removal enabled
 */
#define NETC_ENETC_PSICFGR0_VTE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_VTE_SHIFT)) & NETC_ENETC_PSICFGR0_VTE_MASK)

#define NETC_ENETC_PSICFGR0_SIVIE_MASK           (0x4000U)
#define NETC_ENETC_PSICFGR0_SIVIE_SHIFT          (14U)
/*! SIVIE - SI-based VLAN Insertion Enable
 *  0b0..SI-based VLAN insertion disabled
 *  0b1..SI-based VLAN insertion enabled
 */
#define NETC_ENETC_PSICFGR0_SIVIE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_SIVIE_SHIFT)) & NETC_ENETC_PSICFGR0_SIVIE_MASK)

#define NETC_ENETC_PSICFGR0_ASE_MASK             (0x8000U)
#define NETC_ENETC_PSICFGR0_ASE_SHIFT            (15U)
/*! ASE - Anti-spoofing enable */
#define NETC_ENETC_PSICFGR0_ASE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_ASE_SHIFT)) & NETC_ENETC_PSICFGR0_ASE_MASK)

#define NETC_ENETC_PSICFGR0_NUM_RX_BDR_MASK      (0x7F0000U)
#define NETC_ENETC_PSICFGR0_NUM_RX_BDR_SHIFT     (16U)
#define NETC_ENETC_PSICFGR0_NUM_RX_BDR(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_NUM_RX_BDR_SHIFT)) & NETC_ENETC_PSICFGR0_NUM_RX_BDR_MASK)

#define NETC_ENETC_PSICFGR0_SIVC_MASK            (0xF000000U)
#define NETC_ENETC_PSICFGR0_SIVC_SHIFT           (24U)
#define NETC_ENETC_PSICFGR0_SIVC(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_SIVC_SHIFT)) & NETC_ENETC_PSICFGR0_SIVC_MASK)

#define NETC_ENETC_PSICFGR0_SIBW_MASK            (0xF0000000U)
#define NETC_ENETC_PSICFGR0_SIBW_SHIFT           (28U)
#define NETC_ENETC_PSICFGR0_SIBW(x)              (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR0_SIBW_SHIFT)) & NETC_ENETC_PSICFGR0_SIBW_MASK)
/*! @} */

/* The count of NETC_ENETC_PSICFGR0 */
#define NETC_ENETC_PSICFGR0_COUNT                (2U)

/*! @name PSICFGR1 - Port station interface 1 configuration register 1 */
/*! @{ */

#define NETC_ENETC_PSICFGR1_TC0_MAP_MASK         (0x7U)
#define NETC_ENETC_PSICFGR1_TC0_MAP_SHIFT        (0U)
#define NETC_ENETC_PSICFGR1_TC0_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC0_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC0_MAP_MASK)

#define NETC_ENETC_PSICFGR1_TC1_MAP_MASK         (0x70U)
#define NETC_ENETC_PSICFGR1_TC1_MAP_SHIFT        (4U)
#define NETC_ENETC_PSICFGR1_TC1_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC1_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC1_MAP_MASK)

#define NETC_ENETC_PSICFGR1_TC2_MAP_MASK         (0x700U)
#define NETC_ENETC_PSICFGR1_TC2_MAP_SHIFT        (8U)
#define NETC_ENETC_PSICFGR1_TC2_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC2_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC2_MAP_MASK)

#define NETC_ENETC_PSICFGR1_TC3_MAP_MASK         (0x7000U)
#define NETC_ENETC_PSICFGR1_TC3_MAP_SHIFT        (12U)
#define NETC_ENETC_PSICFGR1_TC3_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC3_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC3_MAP_MASK)

#define NETC_ENETC_PSICFGR1_TC4_MAP_MASK         (0x70000U)
#define NETC_ENETC_PSICFGR1_TC4_MAP_SHIFT        (16U)
#define NETC_ENETC_PSICFGR1_TC4_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC4_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC4_MAP_MASK)

#define NETC_ENETC_PSICFGR1_TC5_MAP_MASK         (0x700000U)
#define NETC_ENETC_PSICFGR1_TC5_MAP_SHIFT        (20U)
#define NETC_ENETC_PSICFGR1_TC5_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC5_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC5_MAP_MASK)

#define NETC_ENETC_PSICFGR1_TC6_MAP_MASK         (0x7000000U)
#define NETC_ENETC_PSICFGR1_TC6_MAP_SHIFT        (24U)
#define NETC_ENETC_PSICFGR1_TC6_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC6_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC6_MAP_MASK)

#define NETC_ENETC_PSICFGR1_TC7_MAP_MASK         (0x70000000U)
#define NETC_ENETC_PSICFGR1_TC7_MAP_SHIFT        (28U)
#define NETC_ENETC_PSICFGR1_TC7_MAP(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR1_TC7_MAP_SHIFT)) & NETC_ENETC_PSICFGR1_TC7_MAP_MASK)
/*! @} */

/* The count of NETC_ENETC_PSICFGR1 */
#define NETC_ENETC_PSICFGR1_COUNT                (2U)

/*! @name PSICFGR2 - Port station interface 0 configuration register 2..Port station interface 1 configuration register 2 */
/*! @{ */

#define NETC_ENETC_PSICFGR2_NUM_MSIX_MASK        (0x3FU)
#define NETC_ENETC_PSICFGR2_NUM_MSIX_SHIFT       (0U)
/*! NUM_MSIX - Number of MSI-X */
#define NETC_ENETC_PSICFGR2_NUM_MSIX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSICFGR2_NUM_MSIX_SHIFT)) & NETC_ENETC_PSICFGR2_NUM_MSIX_MASK)
/*! @} */

/* The count of NETC_ENETC_PSICFGR2 */
#define NETC_ENETC_PSICFGR2_COUNT                (2U)

/*! @name PSIVMAFCFGR - Port station interface 0 VSI MAC address filtering configuration register..Port station interface 1 VSI MAC address filtering configuration register */
/*! @{ */

#define NETC_ENETC_PSIVMAFCFGR_NUM_MAC_AFTE_MASK (0xFFU)
#define NETC_ENETC_PSIVMAFCFGR_NUM_MAC_AFTE_SHIFT (0U)
#define NETC_ENETC_PSIVMAFCFGR_NUM_MAC_AFTE(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVMAFCFGR_NUM_MAC_AFTE_SHIFT)) & NETC_ENETC_PSIVMAFCFGR_NUM_MAC_AFTE_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIVMAFCFGR */
#define NETC_ENETC_PSIVMAFCFGR_COUNT             (2U)

/*! @name PSIVLANFCFGR - Port station interface 0 VLAN filtering configuration register..Port station interface 1 VLAN filtering configuration register */
/*! @{ */

#define NETC_ENETC_PSIVLANFCFGR_NUM_VLAN_FTE_MASK (0xFFU)
#define NETC_ENETC_PSIVLANFCFGR_NUM_VLAN_FTE_SHIFT (0U)
#define NETC_ENETC_PSIVLANFCFGR_NUM_VLAN_FTE(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVLANFCFGR_NUM_VLAN_FTE_SHIFT)) & NETC_ENETC_PSIVLANFCFGR_NUM_VLAN_FTE_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIVLANFCFGR */
#define NETC_ENETC_PSIVLANFCFGR_COUNT            (2U)

/*! @name PSIUMHFR0 - Port station interface 0 unicast MAC hash filter register 0..Port station interface 1 unicast MAC hash filter register 0 */
/*! @{ */

#define NETC_ENETC_PSIUMHFR0_MAC_HASH_FLT_LOW_MASK (0xFFFFFFFFU)
#define NETC_ENETC_PSIUMHFR0_MAC_HASH_FLT_LOW_SHIFT (0U)
#define NETC_ENETC_PSIUMHFR0_MAC_HASH_FLT_LOW(x) (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIUMHFR0_MAC_HASH_FLT_LOW_SHIFT)) & NETC_ENETC_PSIUMHFR0_MAC_HASH_FLT_LOW_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIUMHFR0 */
#define NETC_ENETC_PSIUMHFR0_COUNT               (2U)

/*! @name PSIUMHFR1 - Port station interface 0 unicast MAC hash filter register 1..Port station interface 1 unicast MAC hash filter register 1 */
/*! @{ */

#define NETC_ENETC_PSIUMHFR1_MAC_HASH_FLT_HIGH_MASK (0xFFFFFFFFU)
#define NETC_ENETC_PSIUMHFR1_MAC_HASH_FLT_HIGH_SHIFT (0U)
#define NETC_ENETC_PSIUMHFR1_MAC_HASH_FLT_HIGH(x) (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIUMHFR1_MAC_HASH_FLT_HIGH_SHIFT)) & NETC_ENETC_PSIUMHFR1_MAC_HASH_FLT_HIGH_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIUMHFR1 */
#define NETC_ENETC_PSIUMHFR1_COUNT               (2U)

/*! @name PSIMMHFR0 - Port station interface 0 multicast MAC hash filter register 0..Port station interface 1 multicast MAC hash filter register 0 */
/*! @{ */

#define NETC_ENETC_PSIMMHFR0_MAC_HASH_FLT_LOW_MASK (0xFFFFFFFFU)
#define NETC_ENETC_PSIMMHFR0_MAC_HASH_FLT_LOW_SHIFT (0U)
#define NETC_ENETC_PSIMMHFR0_MAC_HASH_FLT_LOW(x) (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIMMHFR0_MAC_HASH_FLT_LOW_SHIFT)) & NETC_ENETC_PSIMMHFR0_MAC_HASH_FLT_LOW_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIMMHFR0 */
#define NETC_ENETC_PSIMMHFR0_COUNT               (2U)

/*! @name PSIMMHFR1 - Port station interface 0 multicast MAC hash filter register 1..Port station interface 1 multicast MAC hash filter register 1 */
/*! @{ */

#define NETC_ENETC_PSIMMHFR1_MAC_HASH_FLT_HIGH_MASK (0xFFFFFFFFU)
#define NETC_ENETC_PSIMMHFR1_MAC_HASH_FLT_HIGH_SHIFT (0U)
#define NETC_ENETC_PSIMMHFR1_MAC_HASH_FLT_HIGH(x) (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIMMHFR1_MAC_HASH_FLT_HIGH_SHIFT)) & NETC_ENETC_PSIMMHFR1_MAC_HASH_FLT_HIGH_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIMMHFR1 */
#define NETC_ENETC_PSIMMHFR1_COUNT               (2U)

/*! @name PSIVHFR0 - Port station interface 0 VLAN hash filter register 0..Port station interface 1 VLAN hash filter register 0 */
/*! @{ */

#define NETC_ENETC_PSIVHFR0_VLAN_HASH_FLT_LOW_MASK (0xFFFFFFFFU)
#define NETC_ENETC_PSIVHFR0_VLAN_HASH_FLT_LOW_SHIFT (0U)
#define NETC_ENETC_PSIVHFR0_VLAN_HASH_FLT_LOW(x) (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVHFR0_VLAN_HASH_FLT_LOW_SHIFT)) & NETC_ENETC_PSIVHFR0_VLAN_HASH_FLT_LOW_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIVHFR0 */
#define NETC_ENETC_PSIVHFR0_COUNT                (2U)

/*! @name PSIVHFR1 - Port station interface 0 VLAN hash filter register 1..Port station interface 1 VLAN hash filter register 1 */
/*! @{ */

#define NETC_ENETC_PSIVHFR1_VLAN_HASH_FLT_HIGH_MASK (0xFFFFFFFFU)
#define NETC_ENETC_PSIVHFR1_VLAN_HASH_FLT_HIGH_SHIFT (0U)
#define NETC_ENETC_PSIVHFR1_VLAN_HASH_FLT_HIGH(x) (((uint32_t)(((uint32_t)(x)) << NETC_ENETC_PSIVHFR1_VLAN_HASH_FLT_HIGH_SHIFT)) & NETC_ENETC_PSIVHFR1_VLAN_HASH_FLT_HIGH_MASK)
/*! @} */

/* The count of NETC_ENETC_PSIVHFR1 */
#define NETC_ENETC_PSIVHFR1_COUNT                (2U)


/*!
 * @}
 */ /* end of group NETC_ENETC_Register_Masks */


/* NETC_ENETC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ENETC0_BASE base address */
  #define ENETC0_BASE_BASE                         (0x70B10000u)
  /** Peripheral ENETC0_BASE base address */
  #define ENETC0_BASE_BASE_NS                      (0x60B10000u)
  /** Peripheral ENETC0_BASE base pointer */
  #define ENETC0_BASE                              ((NETC_ENETC_Type *)ENETC0_BASE_BASE)
  /** Peripheral ENETC0_BASE base pointer */
  #define ENETC0_BASE_NS                           ((NETC_ENETC_Type *)ENETC0_BASE_BASE_NS)
  /** Peripheral ENETC1_BASE base address */
  #define ENETC1_BASE_BASE                         (0x70B50000u)
  /** Peripheral ENETC1_BASE base address */
  #define ENETC1_BASE_BASE_NS                      (0x60B50000u)
  /** Peripheral ENETC1_BASE base pointer */
  #define ENETC1_BASE                              ((NETC_ENETC_Type *)ENETC1_BASE_BASE)
  /** Peripheral ENETC1_BASE base pointer */
  #define ENETC1_BASE_NS                           ((NETC_ENETC_Type *)ENETC1_BASE_BASE_NS)
  /** Array initializer of NETC_ENETC peripheral base addresses */
  #define NETC_ENETC_BASE_ADDRS                    { ENETC0_BASE_BASE, ENETC1_BASE_BASE }
  /** Array initializer of NETC_ENETC peripheral base pointers */
  #define NETC_ENETC_BASE_PTRS                     { ENETC0_BASE, ENETC1_BASE }
  /** Array initializer of NETC_ENETC peripheral base addresses */
  #define NETC_ENETC_BASE_ADDRS_NS                 { ENETC0_BASE_BASE_NS, ENETC1_BASE_BASE_NS }
  /** Array initializer of NETC_ENETC peripheral base pointers */
  #define NETC_ENETC_BASE_PTRS_NS                  { ENETC0_BASE_NS, ENETC1_BASE_NS }
#else
  /** Peripheral ENETC0_BASE base address */
  #define ENETC0_BASE_BASE                         (0x60B10000u)
  /** Peripheral ENETC0_BASE base pointer */
  #define ENETC0_BASE                              ((NETC_ENETC_Type *)ENETC0_BASE_BASE)
  /** Peripheral ENETC1_BASE base address */
  #define ENETC1_BASE_BASE                         (0x60B50000u)
  /** Peripheral ENETC1_BASE base pointer */
  #define ENETC1_BASE                              ((NETC_ENETC_Type *)ENETC1_BASE_BASE)
  /** Array initializer of NETC_ENETC peripheral base addresses */
  #define NETC_ENETC_BASE_ADDRS                    { ENETC0_BASE_BASE, ENETC1_BASE_BASE }
  /** Array initializer of NETC_ENETC peripheral base pointers */
  #define NETC_ENETC_BASE_PTRS                     { ENETC0_BASE, ENETC1_BASE }
#endif

/*!
 * @}
 */ /* end of group NETC_ENETC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_ETH_LINK Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_ETH_LINK_Peripheral_Access_Layer NETC_ETH_LINK Peripheral Access Layer
 * @{
 */

/** NETC_ETH_LINK - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[8];
  __IO uint32_t PM0_COMMAND_CONFIG;                /**< Port MAC 0 Command and Configuration Register, offset: 0x8 */
  __I  uint32_t PM0_MAC_ADDR_0;                    /**< Port MAC 0 MAC Address Register 0, offset: 0xC */
  __I  uint32_t PM0_MAC_ADDR_1;                    /**< Port MAC 0 MAC Address Register 1, offset: 0x10 */
  __IO uint32_t PM0_MAXFRM;                        /**< Port MAC 0 Maximum Frame Length Register, offset: 0x14 */
  __IO uint32_t PM0_MINFRM;                        /**< Port MAC 0 Minimum Frame Length Register, offset: 0x18 */
       uint8_t RESERVED_1[20];
  __IO uint32_t PM0_MDIO_CFG;                      /**< Port MAC 0 Internal MDIO Configuration Register, offset: 0x30, not available in all instances (available on 6 out of 10) */
  __IO uint32_t PM0_MDIO_CTL;                      /**< Port MAC 0 Internal MDIO Interface Control Register, offset: 0x34, not available in all instances (available on 6 out of 10) */
  __IO uint32_t PM0_MDIO_DATA;                     /**< Port MAC 0 Internal MDIO Interface Data Register, offset: 0x38, not available in all instances (available on 6 out of 10) */
       uint8_t RESERVED_2[4];
  __IO uint32_t PM0_IEVENT;                        /**< Port MAC 0 Interrupt Event Register, offset: 0x40 */
  __IO uint32_t PM0_TX_IPG_PREAMBLE;               /**< Port MAC 0 Transmit Inter-Packet Gap Length and Flexible Preamble length Register, offset: 0x44 */
       uint8_t RESERVED_3[4];
  __IO uint32_t PM0_IMASK;                         /**< Port MAC 0 Interrupt Mask Register(INT_MASK), offset: 0x4C */
       uint8_t RESERVED_4[4];
  __IO uint32_t PM0_PAUSE_QUANTA;                  /**< Port MAC 0 Pause Quanta Register, offset: 0x54 */
       uint8_t RESERVED_5[12];
  __IO uint32_t PM0_PAUSE_THRESH;                  /**< Port MAC 0 Pause Quanta Threshold Register, offset: 0x64 */
       uint8_t RESERVED_6[12];
  __I  uint32_t PM0_RX_PAUSE_STATUS;               /**< Port MAC 0 Receive Pause Status Register, offset: 0x74 */
       uint8_t RESERVED_7[64];
  __IO uint32_t PM0_LPWAKE_TIMER;                  /**< Port MAC 0 EEE Low Power Wakeup Timer Register, offset: 0xB8 */
  __IO uint32_t PM0_SLEEP_TIMER;                   /**< Port MAC 0 Transmit EEE Low Power Timer Register, offset: 0xBC */
  __IO uint32_t PM0_SINGLE_STEP;                   /**< Port MAC 0 IEEE1588 Single-Step Control Register, offset: 0xC0 */
       uint8_t RESERVED_8[12];
  __IO uint32_t PM0_HD_BACKOFF_ENTROPY;            /**< Port MAC 0 half-duplex backoff entropy register, offset: 0xD0 */
  __IO uint32_t PM0_HD_FLOW_CTRL;                  /**< Port MAC 0 Half-Duplex Flow Control Register, offset: 0xD4 */
       uint8_t RESERVED_9[8];
  __IO uint32_t PM0_STATN_CONFIG;                  /**< Port MAC 0 Statistics Configuration Register, offset: 0xE0 */
       uint8_t RESERVED_10[28];
  __I  uint64_t PM0_REOCTN;                        /**< Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn), offset: 0x100 */
  __I  uint64_t PM0_ROCTN;                         /**< Port MAC 0 Receive Octets Counter(iflnOctetsn), offset: 0x108 */
       uint8_t RESERVED_11[8];
  __I  uint64_t PM0_RXPFN;                         /**< Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn), offset: 0x118 */
  __I  uint64_t PM0_RFRMN;                         /**< Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn), offset: 0x120 */
  __I  uint64_t PM0_RFCSN;                         /**< Port MAC 0 Receive Frame Check Sequence Error Counter Register(), offset: 0x128 */
  __I  uint64_t PM0_RVLANN;                        /**< Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn), offset: 0x130 */
  __I  uint64_t PM0_RERRN;                         /**< Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn), offset: 0x138 */
  __I  uint64_t PM0_RUCAN;                         /**< Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn), offset: 0x140 */
  __I  uint64_t PM0_RMCAN;                         /**< Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn), offset: 0x148 */
  __I  uint64_t PM0_RBCAN;                         /**< Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn), offset: 0x150 */
  __I  uint64_t PM0_RDRPN;                         /**< Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn), offset: 0x158 */
  __I  uint64_t PM0_RPKTN;                         /**< Port MAC 0 Receive Packets Counter Register(etherStatsPktsn), offset: 0x160 */
  __I  uint64_t PM0_RUNDN;                         /**< Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn), offset: 0x168 */
  __I  uint64_t PM0_R64N;                          /**< Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN), offset: 0x170 */
  __I  uint64_t PM0_R127N;                         /**< Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN), offset: 0x178 */
  __I  uint64_t PM0_R255N;                         /**< Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN), offset: 0x180 */
  __I  uint64_t PM0_R511N;                         /**< Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN), offset: 0x188 */
  __I  uint64_t PM0_R1023N;                        /**< Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN), offset: 0x190 */
  __I  uint64_t PM0_R1522N;                        /**< Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN), offset: 0x198 */
  __I  uint64_t PM0_R1523XN;                       /**< Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN), offset: 0x1A0 */
  __I  uint64_t PM0_ROVRN;                         /**< Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn), offset: 0x1A8 */
  __I  uint64_t PM0_RJBRN;                         /**< Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn), offset: 0x1B0 */
  __I  uint64_t PM0_RFRGN;                         /**< Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn, offset: 0x1B8 */
  __I  uint64_t PM0_RCNPN;                         /**< Port MAC 0 Receive Control Packet Counter Register, offset: 0x1C0 */
  __I  uint64_t PM0_RDRNTPN;                       /**< Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn), offset: 0x1C8 */
  __I  uint64_t PM0_RMIN63N;                       /**< Port MAC 0 Receive Valid Small Packet Counter Register, offset: 0x1D0 */
       uint8_t RESERVED_12[40];
  __I  uint64_t PM0_TEOCTN;                        /**< Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn), offset: 0x200 */
  __I  uint64_t PM0_TOCTN;                         /**< Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn), offset: 0x208 */
       uint8_t RESERVED_13[8];
  __I  uint64_t PM0_TXPFN;                         /**< Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn), offset: 0x218 */
  __I  uint64_t PM0_TFRMN;                         /**< Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn), offset: 0x220 */
  __I  uint64_t PM0_TFCSN;                         /**< Port MAC 0 Transmit Frame Check Sequence Error Counter Register(), offset: 0x228 */
  __I  uint64_t PM0_TVLANN;                        /**< Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn), offset: 0x230 */
  __I  uint64_t PM0_TERRN;                         /**< Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn), offset: 0x238 */
  __I  uint64_t PM0_TUCAN;                         /**< Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn), offset: 0x240 */
  __I  uint64_t PM0_TMCAN;                         /**< Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn), offset: 0x248 */
  __I  uint64_t PM0_TBCAN;                         /**< Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn), offset: 0x250 */
       uint8_t RESERVED_14[8];
  __I  uint64_t PM0_TPKTN;                         /**< Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn), offset: 0x260 */
  __I  uint64_t PM0_TUNDN;                         /**< Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn), offset: 0x268 */
  __I  uint64_t PM0_T64N;                          /**< Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN), offset: 0x270 */
  __I  uint64_t PM0_T127N;                         /**< Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN), offset: 0x278 */
  __I  uint64_t PM0_T255N;                         /**< Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN), offset: 0x280 */
  __I  uint64_t PM0_T511N;                         /**< Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN), offset: 0x288 */
  __I  uint64_t PM0_T1023N;                        /**< Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN), offset: 0x290 */
  __I  uint64_t PM0_T1522N;                        /**< Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN), offset: 0x298 */
  __I  uint64_t PM0_T1523XN;                       /**< Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN), offset: 0x2A0 */
       uint8_t RESERVED_15[24];
  __I  uint64_t PM0_TCNPN;                         /**< Port MAC 0 Transmit Control Packet Counter Register, offset: 0x2C0 */
       uint8_t RESERVED_16[8];
  __I  uint64_t PM0_TDFRN;                         /**< Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions), offset: 0x2D0 */
  __I  uint64_t PM0_TMCOLN;                        /**< Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames), offset: 0x2D8 */
  __I  uint64_t PM0_TSCOLN;                        /**< Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register, offset: 0x2E0 */
  __I  uint64_t PM0_TLCOLN;                        /**< Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register, offset: 0x2E8 */
  __I  uint64_t PM0_TECOLN;                        /**< Port MAC 0 Transmit Excessive Collisions Counter Register, offset: 0x2F0 */
       uint8_t RESERVED_17[8];
  __IO uint32_t PM0_IF_MODE;                       /**< Port MAC 0 Interface Mode Control Register, offset: 0x300 */
       uint8_t RESERVED_18[260];
  __IO uint32_t PM1_COMMAND_CONFIG;                /**< Port MAC 1 Command and Configuration Register, offset: 0x408 */
  __I  uint32_t PM1_MAC_ADDR_0;                    /**< Port MAC 1 MAC Address Register 0, offset: 0x40C */
  __I  uint32_t PM1_MAC_ADDR_1;                    /**< Port MAC 1 MAC Address Register 1, offset: 0x410 */
  __IO uint32_t PM1_MAXFRM;                        /**< Port MAC 1 Maximum Frame Length Register, offset: 0x414 */
  __IO uint32_t PM1_MINFRM;                        /**< Port MAC 1 Minimum Frame Length Register, offset: 0x418 */
       uint8_t RESERVED_19[36];
  __IO uint32_t PM1_IEVENT;                        /**< Port MAC 1 Interrupt Event Register, offset: 0x440 */
  __IO uint32_t PM1_TX_IPG_PREAMBLE;               /**< Port MAC 1 Transmit Inter-Packet Gap Length and Flexible Preamble length Register, offset: 0x444 */
       uint8_t RESERVED_20[4];
  __IO uint32_t PM1_IMASK;                         /**< Port MAC 1 Interrupt Mask Register(INT_MASK), offset: 0x44C */
       uint8_t RESERVED_21[4];
  __IO uint32_t PM1_PAUSE_QUANTA;                  /**< Port MAC 1 Pause Quanta Register, offset: 0x454 */
       uint8_t RESERVED_22[12];
  __IO uint32_t PM1_PAUSE_THRESH;                  /**< Port MAC 1 Pause Quanta Threshold Register, offset: 0x464 */
       uint8_t RESERVED_23[12];
  __I  uint32_t PM1_RX_PAUSE_STATUS;               /**< Port MAC 1 Receive Pause Status Register, offset: 0x474 */
       uint8_t RESERVED_24[64];
  __IO uint32_t PM1_LPWAKE_TIMER;                  /**< Port MAC 1 EEE Low Power Wakeup Timer Register, offset: 0x4B8 */
  __IO uint32_t PM1_SLEEP_TIMER;                   /**< Port MAC 1 Transmit EEE Low Power Timer Register, offset: 0x4BC */
  __IO uint32_t PM1_SINGLE_STEP;                   /**< Port MAC 1 IEEE1588 Single-Step Control Register, offset: 0x4C0 */
       uint8_t RESERVED_25[12];
  __IO uint32_t PM1_HD_BACKOFF_ENTROPY;            /**< Port MAC 1 half-duplex backoff entropy register, offset: 0x4D0 */
  __IO uint32_t PM1_HD_FLOW_CTRL;                  /**< Port MAC 1 Half-Duplex Flow Control Register, offset: 0x4D4 */
       uint8_t RESERVED_26[8];
  __IO uint32_t PM1_STATN_CONFIG;                  /**< Port MAC 1 Statistics Configuration Register, offset: 0x4E0 */
       uint8_t RESERVED_27[28];
  __I  uint64_t PM1_REOCTN;                        /**< Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn), offset: 0x500 */
  __I  uint64_t PM1_ROCTN;                         /**< Port MAC 1 Receive Octets Counter(iflnOctetsn), offset: 0x508 */
       uint8_t RESERVED_28[8];
  __I  uint64_t PM1_RXPFN;                         /**< Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn), offset: 0x518 */
  __I  uint64_t PM1_RFRMN;                         /**< Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn), offset: 0x520 */
  __I  uint64_t PM1_RFCSN;                         /**< Port MAC 1 Receive Frame Check Sequence Error Counter Register(), offset: 0x528 */
  __I  uint64_t PM1_RVLANN;                        /**< Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn), offset: 0x530 */
  __I  uint64_t PM1_RERRN;                         /**< Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn), offset: 0x538 */
  __I  uint64_t PM1_RUCAN;                         /**< Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn), offset: 0x540 */
  __I  uint64_t PM1_RMCAN;                         /**< Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn), offset: 0x548 */
  __I  uint64_t PM1_RBCAN;                         /**< Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn), offset: 0x550 */
  __I  uint64_t PM1_RDRPN;                         /**< Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn), offset: 0x558 */
  __I  uint64_t PM1_RPKTN;                         /**< Port MAC 1 Receive Packets Counter Register(etherStatsPktsn), offset: 0x560 */
  __I  uint64_t PM1_RUNDN;                         /**< Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn), offset: 0x568 */
  __I  uint64_t PM1_R64N;                          /**< Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN), offset: 0x570 */
  __I  uint64_t PM1_R127N;                         /**< Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN), offset: 0x578 */
  __I  uint64_t PM1_R255N;                         /**< Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN), offset: 0x580 */
  __I  uint64_t PM1_R511N;                         /**< Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN), offset: 0x588 */
  __I  uint64_t PM1_R1023N;                        /**< Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN), offset: 0x590 */
  __I  uint64_t PM1_R1522N;                        /**< Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN), offset: 0x598 */
  __I  uint64_t PM1_R1523XN;                       /**< Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN), offset: 0x5A0 */
  __I  uint64_t PM1_ROVRN;                         /**< Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn), offset: 0x5A8 */
  __I  uint64_t PM1_RJBRN;                         /**< Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn), offset: 0x5B0 */
  __I  uint64_t PM1_RFRGN;                         /**< Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn, offset: 0x5B8 */
  __I  uint64_t PM1_RCNPN;                         /**< Port MAC 1 Receive Control Packet Counter Register, offset: 0x5C0 */
  __I  uint64_t PM1_RDRNTPN;                       /**< Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn), offset: 0x5C8 */
  __I  uint64_t PM1_RMIN63N;                       /**< Port MAC 1 Receive Valid Small Packet Counter Register, offset: 0x5D0 */
       uint8_t RESERVED_29[40];
  __I  uint64_t PM1_TEOCTN;                        /**< Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn), offset: 0x600 */
  __I  uint64_t PM1_TOCTN;                         /**< Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn), offset: 0x608 */
       uint8_t RESERVED_30[8];
  __I  uint64_t PM1_TXPFN;                         /**< Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn), offset: 0x618 */
  __I  uint64_t PM1_TFRMN;                         /**< Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn), offset: 0x620 */
  __I  uint64_t PM1_TFCSN;                         /**< Port MAC 1 Transmit Frame Check Sequence Error Counter Register(), offset: 0x628 */
  __I  uint64_t PM1_TVLANN;                        /**< Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn), offset: 0x630 */
  __I  uint64_t PM1_TERRN;                         /**< Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn), offset: 0x638 */
  __I  uint64_t PM1_TUCAN;                         /**< Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn), offset: 0x640 */
  __I  uint64_t PM1_TMCAN;                         /**< Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn), offset: 0x648 */
  __I  uint64_t PM1_TBCAN;                         /**< Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn), offset: 0x650 */
       uint8_t RESERVED_31[8];
  __I  uint64_t PM1_TPKTN;                         /**< Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn), offset: 0x660 */
  __I  uint64_t PM1_TUNDN;                         /**< Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn), offset: 0x668 */
  __I  uint64_t PM1_T64N;                          /**< Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN), offset: 0x670 */
  __I  uint64_t PM1_T127N;                         /**< Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN), offset: 0x678 */
  __I  uint64_t PM1_T255N;                         /**< Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN), offset: 0x680 */
  __I  uint64_t PM1_T511N;                         /**< Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN), offset: 0x688 */
  __I  uint64_t PM1_T1023N;                        /**< Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN), offset: 0x690 */
  __I  uint64_t PM1_T1522N;                        /**< Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN), offset: 0x698 */
  __I  uint64_t PM1_T1523XN;                       /**< Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN), offset: 0x6A0 */
       uint8_t RESERVED_32[24];
  __I  uint64_t PM1_TCNPN;                         /**< Port MAC 1 Transmit Control Packet Counter Register, offset: 0x6C0 */
       uint8_t RESERVED_33[8];
  __I  uint64_t PM1_TDFRN;                         /**< Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions), offset: 0x6D0 */
  __I  uint64_t PM1_TMCOLN;                        /**< Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames), offset: 0x6D8 */
  __I  uint64_t PM1_TSCOLN;                        /**< Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register, offset: 0x6E0 */
  __I  uint64_t PM1_TLCOLN;                        /**< Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register, offset: 0x6E8 */
  __I  uint64_t PM1_TECOLN;                        /**< Port MAC 1 Transmit Excessive Collisions Counter Register, offset: 0x6F0 */
       uint8_t RESERVED_34[8];
  __IO uint32_t PM1_IF_MODE;                       /**< Port MAC 1 Interface Mode Control Register, offset: 0x700 */
       uint8_t RESERVED_35[252];
  __IO uint32_t MAC_MERGE_MMCSR;                   /**< Port MAC Merge Control and Status Register, offset: 0x800 */
       uint8_t RESERVED_36[4];
  __IO uint32_t MAC_MERGE_MMFAECR;                 /**< Port MAC Merge Frame Assembly Error Count Register, offset: 0x808 */
  __IO uint32_t MAC_MERGE_MMFSECR;                 /**< Port MAC Merge Frame SMD Error Count Register, offset: 0x80C */
  __IO uint32_t MAC_MERGE_MMFAOCR;                 /**< Port MAC Merge Frame Assembly OK Count Register, offset: 0x810 */
  __IO uint32_t MAC_MERGE_MMFCRXR;                 /**< Port MAC Merge Fragment Count RX Register, offset: 0x814 */
  __IO uint32_t MAC_MERGE_MMFCTXR;                 /**< Port MAC Merge Fragment Count TX Register, offset: 0x818 */
  __IO uint32_t MAC_MERGE_MMHCR;                   /**< Port MAC Merge Hold Count Register, offset: 0x81C */
       uint8_t RESERVED_37[992];
  __IO uint32_t PEMDIOCR;                          /**< Port external MDIO configuration register, offset: 0xC00 */
  __IO uint32_t PEMDIOICR;                         /**< Port external MDIO interface control register, offset: 0xC04 */
  __IO uint32_t PEMDIOIDR;                         /**< Port external MDIO interface data register, offset: 0xC08 */
  __IO uint32_t PEMDIORAR;                         /**< Port external MDIO register address register, offset: 0xC0C */
  __I  uint32_t PEMDIOSR;                          /**< Port external MDIO status register, offset: 0xC10 */
       uint8_t RESERVED_38[12];
  __IO uint32_t PPSCR;                             /**< PHY status configuration register, offset: 0xC20 */
  __IO uint32_t PPSCTRLR;                          /**< Port PHY status control register, offset: 0xC24 */
  __I  uint32_t PPSDR;                             /**< Port PHY status data register, offset: 0xC28 */
  __IO uint32_t PPSRAR;                            /**< Port PHY status register address register, offset: 0xC2C */
  __IO uint32_t PPSER;                             /**< Port PHY status event register, offset: 0xC30 */
  __IO uint32_t PPSMR;                             /**< Port PHY status mask register, offset: 0xC34 */
} NETC_ETH_LINK_Type;

/* ----------------------------------------------------------------------------
   -- NETC_ETH_LINK Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_ETH_LINK_Register_Masks NETC_ETH_LINK Register Masks
 * @{
 */

/*! @name PM0_COMMAND_CONFIG - Port MAC 0 Command and Configuration Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_EN_MASK (0x1U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_EN_SHIFT (0U)
/*! TX_EN - MAC transmit path enable */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_EN_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_EN_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_EN_MASK (0x2U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_EN_SHIFT (1U)
/*! RX_EN - MAC receive path enable */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_EN_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_EN_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_FWD_MASK (0x80U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_FWD_SHIFT (7U)
/*! PAUSE_FWD - Terminate/forward received PAUSE frames */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_FWD(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_FWD_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_FWD_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_IGN_MASK (0x100U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_IGN_SHIFT (8U)
/*! PAUSE_IGN - Ignore PAUSE frame quanta */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_IGN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_IGN_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_PAUSE_IGN_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_ADDR_INS_MASK (0x200U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_ADDR_INS_SHIFT (9U)
/*! TX_ADDR_INS - Transmit source MAC address insertion */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_ADDR_INS(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_ADDR_INS_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_ADDR_INS_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_LOOP_ENA_MASK (0x400U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_LOOP_ENA_SHIFT (10U)
/*! LOOP_ENA - Loopback enable */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_LOOP_ENA(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_LOOP_ENA_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_LOOP_ENA_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_LPBK_MODE_MASK (0x1800U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_LPBK_MODE_SHIFT (11U)
/*! LPBK_MODE - Loopback mode */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_LPBK_MODE(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_LPBK_MODE_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_LPBK_MODE_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_CNT_FRM_EN_MASK (0x2000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_CNT_FRM_EN_SHIFT (13U)
/*! CNT_FRM_EN - Control frame reception enable */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_CNT_FRM_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_CNT_FRM_EN_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_CNT_FRM_EN_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_PNT_MASK (0x4000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_PNT_SHIFT (14U)
/*! TS_PNT - Timestamp Point */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_PNT(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_PNT_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_PNT_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TXP_MASK (0x8000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TXP_SHIFT (15U)
/*! TXP - Enable padding of frames in transmit direction (1, default). */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TXP(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_TXP_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_TXP_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_HD_FCEN_MASK (0x40000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_HD_FCEN_SHIFT (18U)
/*! HD_FCEN - Half Duplex Flow Control Enable */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_HD_FCEN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_HD_FCEN_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_HD_FCEN_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_FLUSH_MASK (0x400000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_FLUSH_SHIFT (22U)
/*! TX_FLUSH - Tx flush */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_FLUSH(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_FLUSH_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_FLUSH_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_LOWP_ENA_MASK (0x800000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_LOWP_ENA_SHIFT (23U)
/*! TX_LOWP_ENA - Transmit Low Power Idle Enable.
 *  0b0..(default), the MAC operates in normal mode.
 *  0b1..The MAC completes the transmission of the current Frame and generates Low Power Idle Sequences to the
 *       line. It is advised to inspect IEVENT[TX_EMPTY] is set before enabling the LPI.
 */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_LOWP_ENA(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_LOWP_ENA_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_TX_LOWP_ENA_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_SWR_MASK (0x4000000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_SWR_SHIFT (26U)
/*! SWR - Software Reset. Self clearing bit. */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_SWR(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_SWR_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_SWR_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_FLUSH_MASK (0x10000000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_FLUSH_SHIFT (28U)
/*! RX_FLUSH - Ingress flush enable */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_FLUSH(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_FLUSH_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_RX_FLUSH_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_MODE_MASK (0x40000000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_MODE_SHIFT (30U)
/*! TS_MODE - Transmit timestamp mode */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_MODE(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_MODE_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_TS_MODE_MASK)

#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_MG_MASK (0x80000000U)
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_MG_SHIFT (31U)
/*! MG - Magic Packet detection enable. */
#define NETC_ETH_LINK_PM0_COMMAND_CONFIG_MG(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_COMMAND_CONFIG_MG_SHIFT)) & NETC_ETH_LINK_PM0_COMMAND_CONFIG_MG_MASK)
/*! @} */

/*! @name PM0_MAC_ADDR_0 - Port MAC 0 MAC Address Register 0 */
/*! @{ */

#define NETC_ETH_LINK_PM0_MAC_ADDR_0_MAC_ADDR_0_MASK (0xFFFFFFFFU)
#define NETC_ETH_LINK_PM0_MAC_ADDR_0_MAC_ADDR_0_SHIFT (0U)
/*! MAC_ADDR_0 - MAC address 0 */
#define NETC_ETH_LINK_PM0_MAC_ADDR_0_MAC_ADDR_0(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MAC_ADDR_0_MAC_ADDR_0_SHIFT)) & NETC_ETH_LINK_PM0_MAC_ADDR_0_MAC_ADDR_0_MASK)
/*! @} */

/*! @name PM0_MAC_ADDR_1 - Port MAC 0 MAC Address Register 1 */
/*! @{ */

#define NETC_ETH_LINK_PM0_MAC_ADDR_1_MAC_ADDR_1_MASK (0xFFFFU)
#define NETC_ETH_LINK_PM0_MAC_ADDR_1_MAC_ADDR_1_SHIFT (0U)
/*! MAC_ADDR_1 - MAC address 1 */
#define NETC_ETH_LINK_PM0_MAC_ADDR_1_MAC_ADDR_1(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MAC_ADDR_1_MAC_ADDR_1_SHIFT)) & NETC_ETH_LINK_PM0_MAC_ADDR_1_MAC_ADDR_1_MASK)
/*! @} */

/*! @name PM0_MAXFRM - Port MAC 0 Maximum Frame Length Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_MAXFRM_MAXFRM_MASK     (0xFFFFU)
#define NETC_ETH_LINK_PM0_MAXFRM_MAXFRM_SHIFT    (0U)
/*! MAXFRM - Maximum supported received frame length. */
#define NETC_ETH_LINK_PM0_MAXFRM_MAXFRM(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MAXFRM_MAXFRM_SHIFT)) & NETC_ETH_LINK_PM0_MAXFRM_MAXFRM_MASK)

#define NETC_ETH_LINK_PM0_MAXFRM_TX_MTU_MASK     (0xFFFF0000U)
#define NETC_ETH_LINK_PM0_MAXFRM_TX_MTU_SHIFT    (16U)
/*! TX_MTU - Maximum transmit frame length */
#define NETC_ETH_LINK_PM0_MAXFRM_TX_MTU(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MAXFRM_TX_MTU_SHIFT)) & NETC_ETH_LINK_PM0_MAXFRM_TX_MTU_MASK)
/*! @} */

/*! @name PM0_MINFRM - Port MAC 0 Minimum Frame Length Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_MINFRM_NUM_BYTES_MASK  (0x7FU)
#define NETC_ETH_LINK_PM0_MINFRM_NUM_BYTES_SHIFT (0U)
/*! NUM_BYTES - Receive Minimum Frame Length size in bytes. */
#define NETC_ETH_LINK_PM0_MINFRM_NUM_BYTES(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MINFRM_NUM_BYTES_SHIFT)) & NETC_ETH_LINK_PM0_MINFRM_NUM_BYTES_MASK)
/*! @} */

/*! @name PM0_MDIO_CFG - Port MAC 0 Internal MDIO Configuration Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_MDIO_CFG_BSY2_MASK     (0x1U)
#define NETC_ETH_LINK_PM0_MDIO_CFG_BSY2_SHIFT    (0U)
/*! BSY2 - MDIO busy (same as bit 31) */
#define NETC_ETH_LINK_PM0_MDIO_CFG_BSY2(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_BSY2_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_BSY2_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_HOLD_MASK (0x1CU)
#define NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_HOLD_SHIFT (2U)
/*! MDIO_HOLD - MDIO hold time */
#define NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_HOLD(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_HOLD_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_HOLD_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CFG_PRE_DIS_MASK  (0x20U)
#define NETC_ETH_LINK_PM0_MDIO_CFG_PRE_DIS_SHIFT (5U)
/*! PRE_DIS - MDIO preamble disable. */
#define NETC_ETH_LINK_PM0_MDIO_CFG_PRE_DIS(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_PRE_DIS_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_PRE_DIS_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CFG_ENC45_MASK    (0x40U)
#define NETC_ETH_LINK_PM0_MDIO_CFG_ENC45_SHIFT   (6U)
/*! ENC45 - Enable Clause 45 support. */
#define NETC_ETH_LINK_PM0_MDIO_CFG_ENC45(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_ENC45_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_ENC45_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_CLK_DIV_MASK (0xFF80U)
#define NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_CLK_DIV_SHIFT (7U)
/*! MDIO_CLK_DIV - MDIO clock divisor. */
#define NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_CLK_DIV(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_CLK_DIV_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_MDIO_CLK_DIV_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CFG_CIM_MASK      (0x20000000U)
#define NETC_ETH_LINK_PM0_MDIO_CFG_CIM_SHIFT     (29U)
/*! CIM - MDIO command completion interrupt mask. */
#define NETC_ETH_LINK_PM0_MDIO_CFG_CIM(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_CIM_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_CIM_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CFG_CMP_MASK      (0x40000000U)
#define NETC_ETH_LINK_PM0_MDIO_CFG_CMP_SHIFT     (30U)
/*! CMP - MDIO command completion event. Bit is cleared by writing `1'. */
#define NETC_ETH_LINK_PM0_MDIO_CFG_CMP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_CMP_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_CMP_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CFG_BSY1_MASK     (0x80000000U)
#define NETC_ETH_LINK_PM0_MDIO_CFG_BSY1_SHIFT    (31U)
/*! BSY1 - MDIO busy */
#define NETC_ETH_LINK_PM0_MDIO_CFG_BSY1(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CFG_BSY1_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CFG_BSY1_MASK)
/*! @} */

/*! @name PM0_MDIO_CTL - Port MAC 0 Internal MDIO Interface Control Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_MDIO_CTL_DEV_ADDR_MASK (0x1FU)
#define NETC_ETH_LINK_PM0_MDIO_CTL_DEV_ADDR_SHIFT (0U)
/*! DEV_ADDR - MDIO register address (Clause 22) */
#define NETC_ETH_LINK_PM0_MDIO_CTL_DEV_ADDR(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CTL_DEV_ADDR_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CTL_DEV_ADDR_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CTL_PORT_ADDR_MASK (0x3E0U)
#define NETC_ETH_LINK_PM0_MDIO_CTL_PORT_ADDR_SHIFT (5U)
/*! PORT_ADDR - MDIO PHY address (Clause 22) */
#define NETC_ETH_LINK_PM0_MDIO_CTL_PORT_ADDR(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CTL_PORT_ADDR_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CTL_PORT_ADDR_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CTL_READ_MASK     (0x8000U)
#define NETC_ETH_LINK_PM0_MDIO_CTL_READ_SHIFT    (15U)
/*! READ - MDIO read initiation. */
#define NETC_ETH_LINK_PM0_MDIO_CTL_READ(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CTL_READ_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CTL_READ_MASK)

#define NETC_ETH_LINK_PM0_MDIO_CTL_BSY_MASK      (0x80000000U)
#define NETC_ETH_LINK_PM0_MDIO_CTL_BSY_SHIFT     (31U)
/*! BSY - MDIO busy */
#define NETC_ETH_LINK_PM0_MDIO_CTL_BSY(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_CTL_BSY_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_CTL_BSY_MASK)
/*! @} */

/*! @name PM0_MDIO_DATA - Port MAC 0 Internal MDIO Interface Data Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_MDIO_DATA_MDIO_DATA_MASK (0xFFFFU)
#define NETC_ETH_LINK_PM0_MDIO_DATA_MDIO_DATA_SHIFT (0U)
/*! MDIO_DATA - 16-bit MDIO data. */
#define NETC_ETH_LINK_PM0_MDIO_DATA_MDIO_DATA(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_DATA_MDIO_DATA_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_DATA_MDIO_DATA_MASK)

#define NETC_ETH_LINK_PM0_MDIO_DATA_BUSY_MASK    (0x80000000U)
#define NETC_ETH_LINK_PM0_MDIO_DATA_BUSY_SHIFT   (31U)
/*! BUSY - MDIO busy bit. The state of this bit is also reflected in MDIO_CFG[BSY]. */
#define NETC_ETH_LINK_PM0_MDIO_DATA_BUSY(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_MDIO_DATA_BUSY_SHIFT)) & NETC_ETH_LINK_PM0_MDIO_DATA_BUSY_MASK)
/*! @} */

/*! @name PM0_IEVENT - Port MAC 0 Interrupt Event Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_IEVENT_TX_EMPTY_MASK   (0x20U)
#define NETC_ETH_LINK_PM0_IEVENT_TX_EMPTY_SHIFT  (5U)
/*! TX_EMPTY - Transmit fifo empty event */
#define NETC_ETH_LINK_PM0_IEVENT_TX_EMPTY(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_TX_EMPTY_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_TX_EMPTY_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_RX_EMPTY_MASK   (0x40U)
#define NETC_ETH_LINK_PM0_IEVENT_RX_EMPTY_SHIFT  (6U)
/*! RX_EMPTY - Receive idle event */
#define NETC_ETH_LINK_PM0_IEVENT_RX_EMPTY(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_RX_EMPTY_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_RX_EMPTY_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_TX_OVFL_MASK    (0x400U)
#define NETC_ETH_LINK_PM0_IEVENT_TX_OVFL_SHIFT   (10U)
/*! TX_OVFL - Transmit FIFO overflow event. */
#define NETC_ETH_LINK_PM0_IEVENT_TX_OVFL(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_TX_OVFL_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_TX_OVFL_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_TX_UNFL_MASK    (0x800U)
#define NETC_ETH_LINK_PM0_IEVENT_TX_UNFL_SHIFT   (11U)
/*! TX_UNFL - Transmit FIFO underflow event. */
#define NETC_ETH_LINK_PM0_IEVENT_TX_UNFL(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_TX_UNFL_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_TX_UNFL_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_RX_OVFL_MASK    (0x1000U)
#define NETC_ETH_LINK_PM0_IEVENT_RX_OVFL_SHIFT   (12U)
/*! RX_OVFL - Receive FIFO overflow event. */
#define NETC_ETH_LINK_PM0_IEVENT_RX_OVFL(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_RX_OVFL_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_RX_OVFL_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_MGI_MASK        (0x4000U)
#define NETC_ETH_LINK_PM0_IEVENT_MGI_SHIFT       (14U)
/*! MGI - Magic packet detection indication event */
#define NETC_ETH_LINK_PM0_IEVENT_MGI(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_MGI_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_MGI_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_TX_CSD_MASK     (0x200000U)
#define NETC_ETH_LINK_PM0_IEVENT_TX_CSD_SHIFT    (21U)
/*! TX_CSD - Tx Clock Stop Detection */
#define NETC_ETH_LINK_PM0_IEVENT_TX_CSD(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_TX_CSD_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_TX_CSD_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_RX_CSD_MASK     (0x400000U)
#define NETC_ETH_LINK_PM0_IEVENT_RX_CSD_SHIFT    (22U)
/*! RX_CSD - Rx Clock Stop Detection */
#define NETC_ETH_LINK_PM0_IEVENT_RX_CSD(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_RX_CSD_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_RX_CSD_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_SPD_DUP_MASK    (0x800000U)
#define NETC_ETH_LINK_PM0_IEVENT_SPD_DUP_SHIFT   (23U)
/*! SPD_DUP - Speed/Duplex Change */
#define NETC_ETH_LINK_PM0_IEVENT_SPD_DUP(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_SPD_DUP_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_SPD_DUP_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_MRG_SERR_MASK   (0x8000000U)
#define NETC_ETH_LINK_PM0_IEVENT_MRG_SERR_SHIFT  (27U)
/*! MRG_SERR - MAC merge frame SMD error received event */
#define NETC_ETH_LINK_PM0_IEVENT_MRG_SERR(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_MRG_SERR_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_MRG_SERR_MASK)

#define NETC_ETH_LINK_PM0_IEVENT_MRG_AERR_MASK   (0x10000000U)
#define NETC_ETH_LINK_PM0_IEVENT_MRG_AERR_SHIFT  (28U)
/*! MRG_AERR - MAC merge frame assembly error event */
#define NETC_ETH_LINK_PM0_IEVENT_MRG_AERR(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IEVENT_MRG_AERR_SHIFT)) & NETC_ETH_LINK_PM0_IEVENT_MRG_AERR_MASK)
/*! @} */

/*! @name PM0_TX_IPG_PREAMBLE - Port MAC 0 Transmit Inter-Packet Gap Length and Flexible Preamble length Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_IPG_LEN_MASK (0x7FU)
#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_IPG_LEN_SHIFT (0U)
/*! IPG_LEN - Transmit inter-packet gap value. */
#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_IPG_LEN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_IPG_LEN_SHIFT)) & NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_IPG_LEN_MASK)

#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_MASK (0x700U)
#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_SHIFT (8U)
/*! FLEX_PREAMBLE_CNT - Flexible Preamble Count */
#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_SHIFT)) & NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_MASK)

#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_MASK (0x80000000U)
#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_SHIFT (31U)
/*! FLEX_PREAMBLE_EN - Enable Flexible Preamble Count */
#define NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_SHIFT)) & NETC_ETH_LINK_PM0_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_MASK)
/*! @} */

/*! @name PM0_IMASK - Port MAC 0 Interrupt Mask Register(INT_MASK) */
/*! @{ */

#define NETC_ETH_LINK_PM0_IMASK_MGI_MASK         (0x4000U)
#define NETC_ETH_LINK_PM0_IMASK_MGI_SHIFT        (14U)
/*! MGI - Magic packet detection indication event mask. */
#define NETC_ETH_LINK_PM0_IMASK_MGI(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IMASK_MGI_SHIFT)) & NETC_ETH_LINK_PM0_IMASK_MGI_MASK)

#define NETC_ETH_LINK_PM0_IMASK_TX_CSD_MASK      (0x200000U)
#define NETC_ETH_LINK_PM0_IMASK_TX_CSD_SHIFT     (21U)
/*! TX_CSD - Tx Clock Stop Detection */
#define NETC_ETH_LINK_PM0_IMASK_TX_CSD(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IMASK_TX_CSD_SHIFT)) & NETC_ETH_LINK_PM0_IMASK_TX_CSD_MASK)

#define NETC_ETH_LINK_PM0_IMASK_RX_CSD_MASK      (0x400000U)
#define NETC_ETH_LINK_PM0_IMASK_RX_CSD_SHIFT     (22U)
/*! RX_CSD - Rx Clock Stop Detection */
#define NETC_ETH_LINK_PM0_IMASK_RX_CSD(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IMASK_RX_CSD_SHIFT)) & NETC_ETH_LINK_PM0_IMASK_RX_CSD_MASK)

#define NETC_ETH_LINK_PM0_IMASK_SPD_DUP_MASK     (0x800000U)
#define NETC_ETH_LINK_PM0_IMASK_SPD_DUP_SHIFT    (23U)
/*! SPD_DUP - Speed/Duplex change event mask. */
#define NETC_ETH_LINK_PM0_IMASK_SPD_DUP(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IMASK_SPD_DUP_SHIFT)) & NETC_ETH_LINK_PM0_IMASK_SPD_DUP_MASK)

#define NETC_ETH_LINK_PM0_IMASK_MRG_SERR_MASK    (0x8000000U)
#define NETC_ETH_LINK_PM0_IMASK_MRG_SERR_SHIFT   (27U)
/*! MRG_SERR - MAC merge frame SMD error received event interrupt mask */
#define NETC_ETH_LINK_PM0_IMASK_MRG_SERR(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IMASK_MRG_SERR_SHIFT)) & NETC_ETH_LINK_PM0_IMASK_MRG_SERR_MASK)

#define NETC_ETH_LINK_PM0_IMASK_MRG_AERR_MASK    (0x10000000U)
#define NETC_ETH_LINK_PM0_IMASK_MRG_AERR_SHIFT   (28U)
/*! MRG_AERR - MAC merge frame assembly error event interrupt mask */
#define NETC_ETH_LINK_PM0_IMASK_MRG_AERR(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IMASK_MRG_AERR_SHIFT)) & NETC_ETH_LINK_PM0_IMASK_MRG_AERR_MASK)
/*! @} */

/*! @name PM0_PAUSE_QUANTA - Port MAC 0 Pause Quanta Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_PAUSE_QUANTA_PQNT_MASK (0xFFFFU)
#define NETC_ETH_LINK_PM0_PAUSE_QUANTA_PQNT_SHIFT (0U)
/*! PQNT - Value to be used for the quanta value when XOFF is triggered. */
#define NETC_ETH_LINK_PM0_PAUSE_QUANTA_PQNT(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_PAUSE_QUANTA_PQNT_SHIFT)) & NETC_ETH_LINK_PM0_PAUSE_QUANTA_PQNT_MASK)
/*! @} */

/*! @name PM0_PAUSE_THRESH - Port MAC 0 Pause Quanta Threshold Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_PAUSE_THRESH_QTH_MASK  (0xFFFFU)
#define NETC_ETH_LINK_PM0_PAUSE_THRESH_QTH_SHIFT (0U)
/*! QTH - Quanta threshold. */
#define NETC_ETH_LINK_PM0_PAUSE_THRESH_QTH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_PAUSE_THRESH_QTH_SHIFT)) & NETC_ETH_LINK_PM0_PAUSE_THRESH_QTH_MASK)
/*! @} */

/*! @name PM0_RX_PAUSE_STATUS - Port MAC 0 Receive Pause Status Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_RX_PAUSE_STATUS_PSTAT_MASK (0x1U)
#define NETC_ETH_LINK_PM0_RX_PAUSE_STATUS_PSTAT_SHIFT (0U)
/*! PSTAT - Pause status. */
#define NETC_ETH_LINK_PM0_RX_PAUSE_STATUS_PSTAT(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_RX_PAUSE_STATUS_PSTAT_SHIFT)) & NETC_ETH_LINK_PM0_RX_PAUSE_STATUS_PSTAT_MASK)
/*! @} */

/*! @name PM0_LPWAKE_TIMER - Port MAC 0 EEE Low Power Wakeup Timer Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_LPWAKE_TIMER_TW_SYS_TX_MASK (0xFFFFFFU)
#define NETC_ETH_LINK_PM0_LPWAKE_TIMER_TW_SYS_TX_SHIFT (0U)
/*! TW_SYS_TX - EEE System transmit wait time */
#define NETC_ETH_LINK_PM0_LPWAKE_TIMER_TW_SYS_TX(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_LPWAKE_TIMER_TW_SYS_TX_SHIFT)) & NETC_ETH_LINK_PM0_LPWAKE_TIMER_TW_SYS_TX_MASK)
/*! @} */

/*! @name PM0_SLEEP_TIMER - Port MAC 0 Transmit EEE Low Power Timer Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_SLEEP_TIMER_SLEEPT_MASK (0xFFFFFFU)
#define NETC_ETH_LINK_PM0_SLEEP_TIMER_SLEEPT_SHIFT (0U)
#define NETC_ETH_LINK_PM0_SLEEP_TIMER_SLEEPT(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_SLEEP_TIMER_SLEEPT_SHIFT)) & NETC_ETH_LINK_PM0_SLEEP_TIMER_SLEEPT_MASK)
/*! @} */

/*! @name PM0_SINGLE_STEP - Port MAC 0 IEEE1588 Single-Step Control Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_SINGLE_STEP_CH_MASK    (0x40U)
#define NETC_ETH_LINK_PM0_SINGLE_STEP_CH_SHIFT   (6U)
/*! CH - Checksum update */
#define NETC_ETH_LINK_PM0_SINGLE_STEP_CH(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_SINGLE_STEP_CH_SHIFT)) & NETC_ETH_LINK_PM0_SINGLE_STEP_CH_MASK)

#define NETC_ETH_LINK_PM0_SINGLE_STEP_OFFSET_MASK (0xFF80U)
#define NETC_ETH_LINK_PM0_SINGLE_STEP_OFFSET_SHIFT (7U)
#define NETC_ETH_LINK_PM0_SINGLE_STEP_OFFSET(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_SINGLE_STEP_OFFSET_SHIFT)) & NETC_ETH_LINK_PM0_SINGLE_STEP_OFFSET_MASK)

#define NETC_ETH_LINK_PM0_SINGLE_STEP_EN_MASK    (0x80000000U)
#define NETC_ETH_LINK_PM0_SINGLE_STEP_EN_SHIFT   (31U)
/*! EN - IEEE-1588 Single-Step enable. */
#define NETC_ETH_LINK_PM0_SINGLE_STEP_EN(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_SINGLE_STEP_EN_SHIFT)) & NETC_ETH_LINK_PM0_SINGLE_STEP_EN_MASK)
/*! @} */

/*! @name PM0_HD_BACKOFF_ENTROPY - Port MAC 0 half-duplex backoff entropy register */
/*! @{ */

#define NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_MASK (0x3FFU)
#define NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_SHIFT (0U)
/*! HD_BACKOFF_ENTROPY - Half duplex backoff entropy */
#define NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_SHIFT)) & NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_MASK)

#define NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_MASK (0x80000000U)
#define NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_SHIFT (31U)
/*! SW_ENTROPY_VALID - SW programmable entropy valid */
#define NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_SHIFT)) & NETC_ETH_LINK_PM0_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_MASK)
/*! @} */

/*! @name PM0_HD_FLOW_CTRL - Port MAC 0 Half-Duplex Flow Control Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_OFF_MIN_MASK (0xFFFU)
#define NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_OFF_MIN_SHIFT (0U)
/*! HD_BP_OFF_MIN - Half-Duplex Back-Pressure Off Minimum */
#define NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_OFF_MIN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_OFF_MIN_SHIFT)) & NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_OFF_MIN_MASK)

#define NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_ON_MAX_MASK (0xFFF0000U)
#define NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_ON_MAX_SHIFT (16U)
/*! HD_BP_ON_MAX - Half-Duplex Back-Pressure On Maximum */
#define NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_ON_MAX(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_ON_MAX_SHIFT)) & NETC_ETH_LINK_PM0_HD_FLOW_CTRL_HD_BP_ON_MAX_MASK)
/*! @} */

/*! @name PM0_STATN_CONFIG - Port MAC 0 Statistics Configuration Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_STATN_CONFIG_SAT_MASK  (0x1U)
#define NETC_ETH_LINK_PM0_STATN_CONFIG_SAT_SHIFT (0U)
#define NETC_ETH_LINK_PM0_STATN_CONFIG_SAT(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_STATN_CONFIG_SAT_SHIFT)) & NETC_ETH_LINK_PM0_STATN_CONFIG_SAT_MASK)

#define NETC_ETH_LINK_PM0_STATN_CONFIG_COD_MASK  (0x2U)
#define NETC_ETH_LINK_PM0_STATN_CONFIG_COD_SHIFT (1U)
#define NETC_ETH_LINK_PM0_STATN_CONFIG_COD(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_STATN_CONFIG_COD_SHIFT)) & NETC_ETH_LINK_PM0_STATN_CONFIG_COD_MASK)

#define NETC_ETH_LINK_PM0_STATN_CONFIG_CLR_MASK  (0x4U)
#define NETC_ETH_LINK_PM0_STATN_CONFIG_CLR_SHIFT (2U)
#define NETC_ETH_LINK_PM0_STATN_CONFIG_CLR(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_STATN_CONFIG_CLR_SHIFT)) & NETC_ETH_LINK_PM0_STATN_CONFIG_CLR_MASK)

#define NETC_ETH_LINK_PM0_STATN_CONFIG_WEN_MASK  (0x8U)
#define NETC_ETH_LINK_PM0_STATN_CONFIG_WEN_SHIFT (3U)
/*! WEN - Write enable for Tx/Rx stats registers */
#define NETC_ETH_LINK_PM0_STATN_CONFIG_WEN(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_STATN_CONFIG_WEN_SHIFT)) & NETC_ETH_LINK_PM0_STATN_CONFIG_WEN_MASK)
/*! @} */

/*! @name PM0_REOCTN - Port MAC 0 Receive Ethernet Octets Counter(etherStatsOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_REOCTN_REOCTn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_REOCTN_REOCTn_SHIFT    (0U)
/*! REOCTn - Incremented for each octet received in both good and bad packets. */
#define NETC_ETH_LINK_PM0_REOCTN_REOCTn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_REOCTN_REOCTn_SHIFT)) & NETC_ETH_LINK_PM0_REOCTN_REOCTn_MASK)
/*! @} */

/*! @name PM0_ROCTN - Port MAC 0 Receive Octets Counter(iflnOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_ROCTN_ROCTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_ROCTN_ROCTn_SHIFT      (0U)
/*! ROCTn - Incremented for each octet received except preamble (that is, Header, Payload, Pad and
 *    FCS) for all valid frames and valid PAUSE frames received.
 */
#define NETC_ETH_LINK_PM0_ROCTN_ROCTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_ROCTN_ROCTn_SHIFT)) & NETC_ETH_LINK_PM0_ROCTN_ROCTn_MASK)
/*! @} */

/*! @name PM0_RXPFN - Port MAC 0 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RXPFN_RXPFn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RXPFN_RXPFn_SHIFT      (0U)
/*! RXPFn - Incremented for each valid PAUSE frame received . */
#define NETC_ETH_LINK_PM0_RXPFN_RXPFn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RXPFN_RXPFn_SHIFT)) & NETC_ETH_LINK_PM0_RXPFN_RXPFn_MASK)
/*! @} */

/*! @name PM0_RFRMN - Port MAC 0 Receive Frame Counter Register(aFramesReceivedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RFRMN_RFRMn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RFRMN_RFRMn_SHIFT      (0U)
/*! RFRMn - Incremented for each frame received without error, including PAUSE frames. */
#define NETC_ETH_LINK_PM0_RFRMN_RFRMn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RFRMN_RFRMn_SHIFT)) & NETC_ETH_LINK_PM0_RFRMN_RFRMn_MASK)
/*! @} */

/*! @name PM0_RFCSN - Port MAC 0 Receive Frame Check Sequence Error Counter Register() */
/*! @{ */

#define NETC_ETH_LINK_PM0_RFCSN_RFCSn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RFCSN_RFCSn_SHIFT      (0U)
/*! RFCSn - Incremented for each frame received with a CRC-32 error but the frame is otherwise of correct length. */
#define NETC_ETH_LINK_PM0_RFCSN_RFCSn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RFCSN_RFCSn_SHIFT)) & NETC_ETH_LINK_PM0_RFCSN_RFCSn_MASK)
/*! @} */

/*! @name PM0_RVLANN - Port MAC 0 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RVLANN_RVLANn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RVLANN_RVLANn_SHIFT    (0U)
/*! RVLANn - Incremented for each valid VLAN tagged frame received with ethertype 0x8100 */
#define NETC_ETH_LINK_PM0_RVLANN_RVLANn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RVLANN_RVLANn_SHIFT)) & NETC_ETH_LINK_PM0_RVLANN_RVLANn_MASK)
/*! @} */

/*! @name PM0_RERRN - Port MAC 0 Receive Frame Error Counter Register(ifInErrorsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RERRN_RERRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RERRN_RERRn_SHIFT      (0U)
/*! RERRn - Incremented for each frame received with an error (except for undersized/fragment frame): */
#define NETC_ETH_LINK_PM0_RERRN_RERRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RERRN_RERRn_SHIFT)) & NETC_ETH_LINK_PM0_RERRN_RERRn_MASK)
/*! @} */

/*! @name PM0_RUCAN - Port MAC 0 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RUCAN_RUCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RUCAN_RUCAn_SHIFT      (0U)
/*! RUCAn - Incremented for each valid frame received (on the receive FIFO interface) in which bit 0
 *    of the destination address was 0 .
 */
#define NETC_ETH_LINK_PM0_RUCAN_RUCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RUCAN_RUCAn_SHIFT)) & NETC_ETH_LINK_PM0_RUCAN_RUCAn_MASK)
/*! @} */

/*! @name PM0_RMCAN - Port MAC 0 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RMCAN_RMCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RMCAN_RMCAn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_RMCAN_RMCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RMCAN_RMCAn_SHIFT)) & NETC_ETH_LINK_PM0_RMCAN_RMCAn_MASK)
/*! @} */

/*! @name PM0_RBCAN - Port MAC 0 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RBCAN_RBCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RBCAN_RBCAn_SHIFT      (0U)
/*! RBCAn - Incremented for each valid frame received (on the receive FIFO interface) in which all
 *    bits of the destination address were 1 .
 */
#define NETC_ETH_LINK_PM0_RBCAN_RBCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RBCAN_RBCAn_SHIFT)) & NETC_ETH_LINK_PM0_RBCAN_RBCAn_MASK)
/*! @} */

/*! @name PM0_RDRPN - Port MAC 0 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RDRPN_RDRPn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RDRPN_RDRPn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_RDRPN_RDRPn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RDRPN_RDRPn_SHIFT)) & NETC_ETH_LINK_PM0_RDRPN_RDRPn_MASK)
/*! @} */

/*! @name PM0_RPKTN - Port MAC 0 Receive Packets Counter Register(etherStatsPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RPKTN_RPKTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RPKTN_RPKTn_SHIFT      (0U)
/*! RPKTn - Incremented for each good or bad packet received. */
#define NETC_ETH_LINK_PM0_RPKTN_RPKTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RPKTN_RPKTn_SHIFT)) & NETC_ETH_LINK_PM0_RPKTN_RPKTn_MASK)
/*! @} */

/*! @name PM0_RUNDN - Port MAC 0 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RUNDN_RUNDn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RUNDN_RUNDn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_RUNDN_RUNDn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RUNDN_RUNDn_SHIFT)) & NETC_ETH_LINK_PM0_RUNDN_RUNDn_MASK)
/*! @} */

/*! @name PM0_R64N - Port MAC 0 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_R64N_R64n_MASK         (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_R64N_R64n_SHIFT        (0U)
/*! R64n - Incremented for each 64-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM0_R64N_R64n(x)           (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_R64N_R64n_SHIFT)) & NETC_ETH_LINK_PM0_R64N_R64n_MASK)
/*! @} */

/*! @name PM0_R127N - Port MAC 0 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_R127N_R127n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_R127N_R127n_SHIFT      (0U)
/*! R127n - Incremented for each 65- to 127-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM0_R127N_R127n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_R127N_R127n_SHIFT)) & NETC_ETH_LINK_PM0_R127N_R127n_MASK)
/*! @} */

/*! @name PM0_R255N - Port MAC 0 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_R255N_R255n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_R255N_R255n_SHIFT      (0U)
/*! R255n - Incremented for each 128- to 255-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM0_R255N_R255n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_R255N_R255n_SHIFT)) & NETC_ETH_LINK_PM0_R255N_R255n_MASK)
/*! @} */

/*! @name PM0_R511N - Port MAC 0 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_R511N_R511n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_R511N_R511n_SHIFT      (0U)
/*! R511n - Incremented for each 256- to 511-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM0_R511N_R511n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_R511N_R511n_SHIFT)) & NETC_ETH_LINK_PM0_R511N_R511n_MASK)
/*! @} */

/*! @name PM0_R1023N - Port MAC 0 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_R1023N_R1023n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_R1023N_R1023n_SHIFT    (0U)
/*! R1023n - Incremented for each 512- to 1023-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM0_R1023N_R1023n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_R1023N_R1023n_SHIFT)) & NETC_ETH_LINK_PM0_R1023N_R1023n_MASK)
/*! @} */

/*! @name PM0_R1522N - Port MAC 0 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_R1522N_R1522n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_R1522N_R1522n_SHIFT    (0U)
/*! R1522n - Incremented for each 1024- to 1522-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM0_R1522N_R1522n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_R1522N_R1522n_SHIFT)) & NETC_ETH_LINK_PM0_R1522N_R1522n_MASK)
/*! @} */

/*! @name PM0_R1523XN - Port MAC 0 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_R1523XN_R1523Xn_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_R1523XN_R1523Xn_SHIFT  (0U)
#define NETC_ETH_LINK_PM0_R1523XN_R1523Xn(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_R1523XN_R1523Xn_SHIFT)) & NETC_ETH_LINK_PM0_R1523XN_R1523Xn_MASK)
/*! @} */

/*! @name PM0_ROVRN - Port MAC 0 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_ROVRN_ROVRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_ROVRN_ROVRn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_ROVRN_ROVRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_ROVRN_ROVRn_SHIFT)) & NETC_ETH_LINK_PM0_ROVRN_ROVRn_MASK)
/*! @} */

/*! @name PM0_RJBRN - Port MAC 0 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RJBRN_RJBRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RJBRN_RJBRn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_RJBRN_RJBRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RJBRN_RJBRn_SHIFT)) & NETC_ETH_LINK_PM0_RJBRN_RJBRn_MASK)
/*! @} */

/*! @name PM0_RFRGN - Port MAC 0 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
/*! @{ */

#define NETC_ETH_LINK_PM0_RFRGN_RFRGn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RFRGN_RFRGn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_RFRGN_RFRGn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RFRGN_RFRGn_SHIFT)) & NETC_ETH_LINK_PM0_RFRGN_RFRGn_MASK)
/*! @} */

/*! @name PM0_RCNPN - Port MAC 0 Receive Control Packet Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_RCNPN_RCNPn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RCNPN_RCNPn_SHIFT      (0U)
/*! RCNPn - Incremented for each valid control packet (type 0x8808) but not for PAUSE packets */
#define NETC_ETH_LINK_PM0_RCNPN_RCNPn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RCNPN_RCNPn_SHIFT)) & NETC_ETH_LINK_PM0_RCNPN_RCNPn_MASK)
/*! @} */

/*! @name PM0_RDRNTPN - Port MAC 0 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_RDRNTPN_RDRNTPn_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RDRNTPN_RDRNTPn_SHIFT  (0U)
/*! RDRNTPn - Incremented for each fully dropped packet (not truncated) due to internal errors of
 *    the MAC client. Occurs when a receive FIFO overflows.
 */
#define NETC_ETH_LINK_PM0_RDRNTPN_RDRNTPn(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RDRNTPN_RDRNTPn_SHIFT)) & NETC_ETH_LINK_PM0_RDRNTPN_RDRNTPn_MASK)
/*! @} */

/*! @name PM0_RMIN63N - Port MAC 0 Receive Valid Small Packet Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_RMIN63N_RMIN63n_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_RMIN63N_RMIN63n_SHIFT  (0U)
/*! RMIN63n - Incremented for each valid small packet less than 64B but greater or equal to the
 *    length programmed in PMa_MINFRM register
 */
#define NETC_ETH_LINK_PM0_RMIN63N_RMIN63n(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_RMIN63N_RMIN63n_SHIFT)) & NETC_ETH_LINK_PM0_RMIN63N_RMIN63n_MASK)
/*! @} */

/*! @name PM0_TEOCTN - Port MAC 0 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TEOCTN_TEOCTn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TEOCTN_TEOCTn_SHIFT    (0U)
/*! TEOCTn - Incremented for each octet transmitted in both good and bad packets. */
#define NETC_ETH_LINK_PM0_TEOCTN_TEOCTn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TEOCTN_TEOCTn_SHIFT)) & NETC_ETH_LINK_PM0_TEOCTN_TEOCTn_MASK)
/*! @} */

/*! @name PM0_TOCTN - Port MAC 0 Transmit Octets Counter Register(ifOutOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TOCTN_TOCTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TOCTN_TOCTn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_TOCTN_TOCTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TOCTN_TOCTn_SHIFT)) & NETC_ETH_LINK_PM0_TOCTN_TOCTn_MASK)
/*! @} */

/*! @name PM0_TXPFN - Port MAC 0 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TXPFN_TXPFn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TXPFN_TXPFn_SHIFT      (0U)
/*! TXPFn - Incremented for each valid PAUSE frame transmitted . Note: Pause frames forwarded to the
 *    MAC from MAC Client are not counted by TXPFn.
 */
#define NETC_ETH_LINK_PM0_TXPFN_TXPFn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TXPFN_TXPFn_SHIFT)) & NETC_ETH_LINK_PM0_TXPFN_TXPFn_MASK)
/*! @} */

/*! @name PM0_TFRMN - Port MAC 0 Transmit Frame Counter Register(aFramesTransmittedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TFRMN_TFRMn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TFRMN_TFRMn_SHIFT      (0U)
/*! TFRMn - Incremented for each frame transmitted without error, including PAUSE frames. */
#define NETC_ETH_LINK_PM0_TFRMN_TFRMn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TFRMN_TFRMn_SHIFT)) & NETC_ETH_LINK_PM0_TFRMN_TFRMn_MASK)
/*! @} */

/*! @name PM0_TFCSN - Port MAC 0 Transmit Frame Check Sequence Error Counter Register() */
/*! @{ */

#define NETC_ETH_LINK_PM0_TFCSN_TFCSn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TFCSN_TFCSn_SHIFT      (0U)
/*! TFCSn - Incremented for each frame transmitted with a CRC-32 error except for underflows. */
#define NETC_ETH_LINK_PM0_TFCSN_TFCSn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TFCSN_TFCSn_SHIFT)) & NETC_ETH_LINK_PM0_TFCSN_TFCSn_MASK)
/*! @} */

/*! @name PM0_TVLANN - Port MAC 0 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TVLANN_TVLANn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TVLANN_TVLANn_SHIFT    (0U)
/*! TVLANn - Incremented for each valid VLAN tagged frame transmitted with ethertype 0x8100. */
#define NETC_ETH_LINK_PM0_TVLANN_TVLANn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TVLANN_TVLANn_SHIFT)) & NETC_ETH_LINK_PM0_TVLANN_TVLANn_MASK)
/*! @} */

/*! @name PM0_TERRN - Port MAC 0 Transmit Frame Error Counter Register(ifOutErrorsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TERRN_TERRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TERRN_TERRn_SHIFT      (0U)
/*! TERRn - Transmit frame error count */
#define NETC_ETH_LINK_PM0_TERRN_TERRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TERRN_TERRn_SHIFT)) & NETC_ETH_LINK_PM0_TERRN_TERRn_MASK)
/*! @} */

/*! @name PM0_TUCAN - Port MAC 0 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TUCAN_TUCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TUCAN_TUCAn_SHIFT      (0U)
/*! TUCAn - Incremented for each valid frame transmitted (to the FIFO interface) in which bit 0 of the destination address was 0. */
#define NETC_ETH_LINK_PM0_TUCAN_TUCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TUCAN_TUCAn_SHIFT)) & NETC_ETH_LINK_PM0_TUCAN_TUCAn_MASK)
/*! @} */

/*! @name PM0_TMCAN - Port MAC 0 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TMCAN_TMCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TMCAN_TMCAn_SHIFT      (0U)
#define NETC_ETH_LINK_PM0_TMCAN_TMCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TMCAN_TMCAn_SHIFT)) & NETC_ETH_LINK_PM0_TMCAN_TMCAn_MASK)
/*! @} */

/*! @name PM0_TBCAN - Port MAC 0 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TBCAN_TBCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TBCAN_TBCAn_SHIFT      (0U)
/*! TBCAn - Incremented for each valid frame transmitted (to the FIFO interface) in which all bits
 *    of the destination address were 1 .
 */
#define NETC_ETH_LINK_PM0_TBCAN_TBCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TBCAN_TBCAn_SHIFT)) & NETC_ETH_LINK_PM0_TBCAN_TBCAn_MASK)
/*! @} */

/*! @name PM0_TPKTN - Port MAC 0 Transmit Packets Counter Register(etherStatsPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TPKTN_TPKTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TPKTN_TPKTn_SHIFT      (0U)
/*! TPKTn - Incremented for each good or bad packet transmitted. */
#define NETC_ETH_LINK_PM0_TPKTN_TPKTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TPKTN_TPKTn_SHIFT)) & NETC_ETH_LINK_PM0_TPKTN_TPKTn_MASK)
/*! @} */

/*! @name PM0_TUNDN - Port MAC 0 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TUNDN_TUNDn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TUNDN_TUNDn_SHIFT      (0U)
/*! TUNDn - Incremented for each packet transmitted that was less than 64 octets long with a good CRC. */
#define NETC_ETH_LINK_PM0_TUNDN_TUNDn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TUNDN_TUNDn_SHIFT)) & NETC_ETH_LINK_PM0_TUNDN_TUNDn_MASK)
/*! @} */

/*! @name PM0_T64N - Port MAC 0 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_T64N_T64n_MASK         (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_T64N_T64n_SHIFT        (0U)
/*! T64n - Incremented for each 64-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM0_T64N_T64n(x)           (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_T64N_T64n_SHIFT)) & NETC_ETH_LINK_PM0_T64N_T64n_MASK)
/*! @} */

/*! @name PM0_T127N - Port MAC 0 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_T127N_T127n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_T127N_T127n_SHIFT      (0U)
/*! T127n - Incremented for each 65 to 127-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM0_T127N_T127n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_T127N_T127n_SHIFT)) & NETC_ETH_LINK_PM0_T127N_T127n_MASK)
/*! @} */

/*! @name PM0_T255N - Port MAC 0 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_T255N_T255n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_T255N_T255n_SHIFT      (0U)
/*! T255n - Incremented for each 128 to 255-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM0_T255N_T255n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_T255N_T255n_SHIFT)) & NETC_ETH_LINK_PM0_T255N_T255n_MASK)
/*! @} */

/*! @name PM0_T511N - Port MAC 0 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_T511N_T511n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_T511N_T511n_SHIFT      (0U)
/*! T511n - Incremented for each 256 to 511-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM0_T511N_T511n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_T511N_T511n_SHIFT)) & NETC_ETH_LINK_PM0_T511N_T511n_MASK)
/*! @} */

/*! @name PM0_T1023N - Port MAC 0 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_T1023N_T1023n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_T1023N_T1023n_SHIFT    (0U)
/*! T1023n - Incremented for each 512 to 1023-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM0_T1023N_T1023n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_T1023N_T1023n_SHIFT)) & NETC_ETH_LINK_PM0_T1023N_T1023n_MASK)
/*! @} */

/*! @name PM0_T1522N - Port MAC 0 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_T1522N_T1522n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_T1522N_T1522n_SHIFT    (0U)
/*! T1522n - Incremented for each 1024- to 1522-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM0_T1522N_T1522n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_T1522N_T1522n_SHIFT)) & NETC_ETH_LINK_PM0_T1522N_T1522n_MASK)
/*! @} */

/*! @name PM0_T1523XN - Port MAC 0 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM0_T1523XN_T1523Xn_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_T1523XN_T1523Xn_SHIFT  (0U)
#define NETC_ETH_LINK_PM0_T1523XN_T1523Xn(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_T1523XN_T1523Xn_SHIFT)) & NETC_ETH_LINK_PM0_T1523XN_T1523Xn_MASK)
/*! @} */

/*! @name PM0_TCNPN - Port MAC 0 Transmit Control Packet Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_TCNPN_TCNPn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TCNPN_TCNPn_SHIFT      (0U)
/*! TCNPn - Incremented for each valid control packet transmitted (type 0x8808) but not for PAUSE packets */
#define NETC_ETH_LINK_PM0_TCNPN_TCNPn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TCNPN_TCNPn_SHIFT)) & NETC_ETH_LINK_PM0_TCNPN_TCNPn_MASK)
/*! @} */

/*! @name PM0_TDFRN - Port MAC 0 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TDFRN_TDFRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TDFRN_TDFRn_SHIFT      (0U)
/*! TDFRn - Increments for successful transmissions, without retransmits, that were deferred (half-duplex only). */
#define NETC_ETH_LINK_PM0_TDFRN_TDFRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TDFRN_TDFRn_SHIFT)) & NETC_ETH_LINK_PM0_TDFRN_TDFRn_MASK)
/*! @} */

/*! @name PM0_TMCOLN - Port MAC 0 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
/*! @{ */

#define NETC_ETH_LINK_PM0_TMCOLN_TMCOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TMCOLN_TMCOLn_SHIFT    (0U)
/*! TMCOLn - Increments for successful transmission after more than one retransmission (half-duplex only). */
#define NETC_ETH_LINK_PM0_TMCOLN_TMCOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TMCOLN_TMCOLn_SHIFT)) & NETC_ETH_LINK_PM0_TMCOLN_TMCOLn_MASK)
/*! @} */

/*! @name PM0_TSCOLN - Port MAC 0 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_TSCOLN_TSCOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TSCOLN_TSCOLn_SHIFT    (0U)
/*! TSCOLn - Increments for successful transmission after one retransmission (half-duplex only). */
#define NETC_ETH_LINK_PM0_TSCOLN_TSCOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TSCOLN_TSCOLn_SHIFT)) & NETC_ETH_LINK_PM0_TSCOLN_TSCOLn_MASK)
/*! @} */

/*! @name PM0_TLCOLN - Port MAC 0 Transmit Late Collision Counter(aLateCollisions) Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_TLCOLN_TLCOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TLCOLN_TLCOLn_SHIFT    (0U)
/*! TLCOLn - Late collision occurred. Frame corrupted / discarded (half-duplex only) */
#define NETC_ETH_LINK_PM0_TLCOLN_TLCOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TLCOLN_TLCOLn_SHIFT)) & NETC_ETH_LINK_PM0_TLCOLN_TLCOLn_MASK)
/*! @} */

/*! @name PM0_TECOLN - Port MAC 0 Transmit Excessive Collisions Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_TECOLN_TECOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM0_TECOLN_TECOLn_SHIFT    (0U)
/*! TECOLn - Excessive collisions occurred. Frame was discarded (half-duplex only) */
#define NETC_ETH_LINK_PM0_TECOLN_TECOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM0_TECOLN_TECOLn_SHIFT)) & NETC_ETH_LINK_PM0_TECOLN_TECOLn_MASK)
/*! @} */

/*! @name PM0_IF_MODE - Port MAC 0 Interface Mode Control Register */
/*! @{ */

#define NETC_ETH_LINK_PM0_IF_MODE_IFMODE_MASK    (0x7U)
#define NETC_ETH_LINK_PM0_IF_MODE_IFMODE_SHIFT   (0U)
/*! IFMODE - Interface mode */
#define NETC_ETH_LINK_PM0_IF_MODE_IFMODE(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IF_MODE_IFMODE_SHIFT)) & NETC_ETH_LINK_PM0_IF_MODE_IFMODE_MASK)

#define NETC_ETH_LINK_PM0_IF_MODE_REVMII_MASK    (0x8U)
#define NETC_ETH_LINK_PM0_IF_MODE_REVMII_SHIFT   (3U)
/*! REVMII - Reverse Mode
 *  0b0..Reverse mode disabled - port is in MAC mode
 *  0b1..Reverse mode enabled - port is in PHY mode
 */
#define NETC_ETH_LINK_PM0_IF_MODE_REVMII(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IF_MODE_REVMII_SHIFT)) & NETC_ETH_LINK_PM0_IF_MODE_REVMII_MASK)

#define NETC_ETH_LINK_PM0_IF_MODE_M10_MASK       (0x10U)
#define NETC_ETH_LINK_PM0_IF_MODE_M10_SHIFT      (4U)
/*! M10
 *  0b0..100 Mbps
 *  0b1..10 Mbps
 */
#define NETC_ETH_LINK_PM0_IF_MODE_M10(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IF_MODE_M10_SHIFT)) & NETC_ETH_LINK_PM0_IF_MODE_M10_MASK)

#define NETC_ETH_LINK_PM0_IF_MODE_HD_MASK        (0x40U)
#define NETC_ETH_LINK_PM0_IF_MODE_HD_SHIFT       (6U)
/*! HD - Half-duplex
 *  0b0..full duplex
 *  0b1..half duplex
 */
#define NETC_ETH_LINK_PM0_IF_MODE_HD(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IF_MODE_HD_SHIFT)) & NETC_ETH_LINK_PM0_IF_MODE_HD_MASK)

#define NETC_ETH_LINK_PM0_IF_MODE_CLK_STOP_MASK  (0x1000U)
#define NETC_ETH_LINK_PM0_IF_MODE_CLK_STOP_SHIFT (12U)
/*! CLK_STOP - Clock Stop
 *  0b0..Not stoppable
 *  0b1..Stoppable
 */
#define NETC_ETH_LINK_PM0_IF_MODE_CLK_STOP(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IF_MODE_CLK_STOP_SHIFT)) & NETC_ETH_LINK_PM0_IF_MODE_CLK_STOP_MASK)

#define NETC_ETH_LINK_PM0_IF_MODE_SSP_MASK       (0x6000U)
#define NETC_ETH_LINK_PM0_IF_MODE_SSP_SHIFT      (13U)
/*! SSP - Set Speed
 *  0b00..100 Mbps
 *  0b01..10 Mbps
 *  0b10..1 Gbps
 *  0b11..reserved
 */
#define NETC_ETH_LINK_PM0_IF_MODE_SSP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM0_IF_MODE_SSP_SHIFT)) & NETC_ETH_LINK_PM0_IF_MODE_SSP_MASK)
/*! @} */

/*! @name PM1_COMMAND_CONFIG - Port MAC 1 Command and Configuration Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_EN_MASK (0x1U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_EN_SHIFT (0U)
/*! TX_EN - MAC transmit path enable */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_EN_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_EN_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_EN_MASK (0x2U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_EN_SHIFT (1U)
/*! RX_EN - MAC receive path enable */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_EN_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_EN_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_FWD_MASK (0x80U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_FWD_SHIFT (7U)
/*! PAUSE_FWD - Terminate/forward received PAUSE frames */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_FWD(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_FWD_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_FWD_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_IGN_MASK (0x100U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_IGN_SHIFT (8U)
/*! PAUSE_IGN - Ignore PAUSE frame quanta */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_IGN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_IGN_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_PAUSE_IGN_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_ADDR_INS_MASK (0x200U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_ADDR_INS_SHIFT (9U)
/*! TX_ADDR_INS - Transmit source MAC address insertion */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_ADDR_INS(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_ADDR_INS_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_ADDR_INS_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_LOOP_ENA_MASK (0x400U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_LOOP_ENA_SHIFT (10U)
/*! LOOP_ENA - Loopback enable */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_LOOP_ENA(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_LOOP_ENA_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_LOOP_ENA_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_LPBK_MODE_MASK (0x1800U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_LPBK_MODE_SHIFT (11U)
/*! LPBK_MODE - Loopback mode */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_LPBK_MODE(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_LPBK_MODE_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_LPBK_MODE_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_CNT_FRM_EN_MASK (0x2000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_CNT_FRM_EN_SHIFT (13U)
/*! CNT_FRM_EN - Control frame reception enable */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_CNT_FRM_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_CNT_FRM_EN_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_CNT_FRM_EN_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_PNT_MASK (0x4000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_PNT_SHIFT (14U)
/*! TS_PNT - Timestamp Point */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_PNT(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_PNT_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_PNT_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TXP_MASK (0x8000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TXP_SHIFT (15U)
/*! TXP - Enable padding of frames in transmit direction (1, default). */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TXP(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_TXP_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_TXP_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_HD_FCEN_MASK (0x40000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_HD_FCEN_SHIFT (18U)
/*! HD_FCEN - Half Duplex Flow Control Enable */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_HD_FCEN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_HD_FCEN_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_HD_FCEN_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_FLUSH_MASK (0x400000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_FLUSH_SHIFT (22U)
/*! TX_FLUSH - Tx flush */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_FLUSH(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_FLUSH_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_FLUSH_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_LOWP_ENA_MASK (0x800000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_LOWP_ENA_SHIFT (23U)
/*! TX_LOWP_ENA - Transmit Low Power Idle Enable.
 *  0b0..(default), the MAC operates in normal mode.
 *  0b1..The MAC completes the transmission of the current Frame and generates Low Power Idle Sequences to the
 *       line. It is advised to inspect IEVENT[TX_EMPTY] is set before enabling the LPI.
 */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_LOWP_ENA(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_LOWP_ENA_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_TX_LOWP_ENA_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_SWR_MASK (0x4000000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_SWR_SHIFT (26U)
/*! SWR - Software Reset. Self clearing bit. */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_SWR(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_SWR_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_SWR_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_FLUSH_MASK (0x10000000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_FLUSH_SHIFT (28U)
/*! RX_FLUSH - Ingress flush enable */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_FLUSH(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_FLUSH_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_RX_FLUSH_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_MODE_MASK (0x40000000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_MODE_SHIFT (30U)
/*! TS_MODE - Transmit timestamp mode */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_MODE(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_MODE_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_TS_MODE_MASK)

#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_MG_MASK (0x80000000U)
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_MG_SHIFT (31U)
/*! MG - Magic Packet detection enable. */
#define NETC_ETH_LINK_PM1_COMMAND_CONFIG_MG(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_COMMAND_CONFIG_MG_SHIFT)) & NETC_ETH_LINK_PM1_COMMAND_CONFIG_MG_MASK)
/*! @} */

/*! @name PM1_MAC_ADDR_0 - Port MAC 1 MAC Address Register 0 */
/*! @{ */

#define NETC_ETH_LINK_PM1_MAC_ADDR_0_MAC_ADDR_0_MASK (0xFFFFFFFFU)
#define NETC_ETH_LINK_PM1_MAC_ADDR_0_MAC_ADDR_0_SHIFT (0U)
/*! MAC_ADDR_0 - MAC address 0 */
#define NETC_ETH_LINK_PM1_MAC_ADDR_0_MAC_ADDR_0(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_MAC_ADDR_0_MAC_ADDR_0_SHIFT)) & NETC_ETH_LINK_PM1_MAC_ADDR_0_MAC_ADDR_0_MASK)
/*! @} */

/*! @name PM1_MAC_ADDR_1 - Port MAC 1 MAC Address Register 1 */
/*! @{ */

#define NETC_ETH_LINK_PM1_MAC_ADDR_1_MAC_ADDR_1_MASK (0xFFFFU)
#define NETC_ETH_LINK_PM1_MAC_ADDR_1_MAC_ADDR_1_SHIFT (0U)
/*! MAC_ADDR_1 - MAC address 1 */
#define NETC_ETH_LINK_PM1_MAC_ADDR_1_MAC_ADDR_1(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_MAC_ADDR_1_MAC_ADDR_1_SHIFT)) & NETC_ETH_LINK_PM1_MAC_ADDR_1_MAC_ADDR_1_MASK)
/*! @} */

/*! @name PM1_MAXFRM - Port MAC 1 Maximum Frame Length Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_MAXFRM_MAXFRM_MASK     (0xFFFFU)
#define NETC_ETH_LINK_PM1_MAXFRM_MAXFRM_SHIFT    (0U)
/*! MAXFRM - Maximum supported received frame length. */
#define NETC_ETH_LINK_PM1_MAXFRM_MAXFRM(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_MAXFRM_MAXFRM_SHIFT)) & NETC_ETH_LINK_PM1_MAXFRM_MAXFRM_MASK)

#define NETC_ETH_LINK_PM1_MAXFRM_TX_MTU_MASK     (0xFFFF0000U)
#define NETC_ETH_LINK_PM1_MAXFRM_TX_MTU_SHIFT    (16U)
/*! TX_MTU - Maximum transmit frame length */
#define NETC_ETH_LINK_PM1_MAXFRM_TX_MTU(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_MAXFRM_TX_MTU_SHIFT)) & NETC_ETH_LINK_PM1_MAXFRM_TX_MTU_MASK)
/*! @} */

/*! @name PM1_MINFRM - Port MAC 1 Minimum Frame Length Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_MINFRM_NUM_BYTES_MASK  (0x7FU)
#define NETC_ETH_LINK_PM1_MINFRM_NUM_BYTES_SHIFT (0U)
/*! NUM_BYTES - Receive Minimum Frame Length size in bytes. */
#define NETC_ETH_LINK_PM1_MINFRM_NUM_BYTES(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_MINFRM_NUM_BYTES_SHIFT)) & NETC_ETH_LINK_PM1_MINFRM_NUM_BYTES_MASK)
/*! @} */

/*! @name PM1_IEVENT - Port MAC 1 Interrupt Event Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_IEVENT_TX_EMPTY_MASK   (0x20U)
#define NETC_ETH_LINK_PM1_IEVENT_TX_EMPTY_SHIFT  (5U)
/*! TX_EMPTY - Transmit fifo empty event */
#define NETC_ETH_LINK_PM1_IEVENT_TX_EMPTY(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_TX_EMPTY_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_TX_EMPTY_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_RX_EMPTY_MASK   (0x40U)
#define NETC_ETH_LINK_PM1_IEVENT_RX_EMPTY_SHIFT  (6U)
/*! RX_EMPTY - Receive idle event */
#define NETC_ETH_LINK_PM1_IEVENT_RX_EMPTY(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_RX_EMPTY_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_RX_EMPTY_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_TX_OVFL_MASK    (0x400U)
#define NETC_ETH_LINK_PM1_IEVENT_TX_OVFL_SHIFT   (10U)
/*! TX_OVFL - Transmit FIFO overflow event. */
#define NETC_ETH_LINK_PM1_IEVENT_TX_OVFL(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_TX_OVFL_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_TX_OVFL_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_TX_UNFL_MASK    (0x800U)
#define NETC_ETH_LINK_PM1_IEVENT_TX_UNFL_SHIFT   (11U)
/*! TX_UNFL - Transmit FIFO underflow event. */
#define NETC_ETH_LINK_PM1_IEVENT_TX_UNFL(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_TX_UNFL_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_TX_UNFL_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_RX_OVFL_MASK    (0x1000U)
#define NETC_ETH_LINK_PM1_IEVENT_RX_OVFL_SHIFT   (12U)
/*! RX_OVFL - Receive FIFO overflow event. */
#define NETC_ETH_LINK_PM1_IEVENT_RX_OVFL(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_RX_OVFL_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_RX_OVFL_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_MGI_MASK        (0x4000U)
#define NETC_ETH_LINK_PM1_IEVENT_MGI_SHIFT       (14U)
/*! MGI - Magic packet detection indication event */
#define NETC_ETH_LINK_PM1_IEVENT_MGI(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_MGI_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_MGI_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_TX_CSD_MASK     (0x200000U)
#define NETC_ETH_LINK_PM1_IEVENT_TX_CSD_SHIFT    (21U)
/*! TX_CSD - Tx Clock Stop Detection */
#define NETC_ETH_LINK_PM1_IEVENT_TX_CSD(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_TX_CSD_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_TX_CSD_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_RX_CSD_MASK     (0x400000U)
#define NETC_ETH_LINK_PM1_IEVENT_RX_CSD_SHIFT    (22U)
/*! RX_CSD - Rx Clock Stop Detection */
#define NETC_ETH_LINK_PM1_IEVENT_RX_CSD(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_RX_CSD_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_RX_CSD_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_SPD_DUP_MASK    (0x800000U)
#define NETC_ETH_LINK_PM1_IEVENT_SPD_DUP_SHIFT   (23U)
/*! SPD_DUP - Speed/Duplex Change */
#define NETC_ETH_LINK_PM1_IEVENT_SPD_DUP(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_SPD_DUP_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_SPD_DUP_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_MRG_SERR_MASK   (0x8000000U)
#define NETC_ETH_LINK_PM1_IEVENT_MRG_SERR_SHIFT  (27U)
/*! MRG_SERR - MAC merge frame SMD error received event */
#define NETC_ETH_LINK_PM1_IEVENT_MRG_SERR(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_MRG_SERR_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_MRG_SERR_MASK)

#define NETC_ETH_LINK_PM1_IEVENT_MRG_AERR_MASK   (0x10000000U)
#define NETC_ETH_LINK_PM1_IEVENT_MRG_AERR_SHIFT  (28U)
/*! MRG_AERR - MAC merge frame assembly error event */
#define NETC_ETH_LINK_PM1_IEVENT_MRG_AERR(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IEVENT_MRG_AERR_SHIFT)) & NETC_ETH_LINK_PM1_IEVENT_MRG_AERR_MASK)
/*! @} */

/*! @name PM1_TX_IPG_PREAMBLE - Port MAC 1 Transmit Inter-Packet Gap Length and Flexible Preamble length Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_IPG_LEN_MASK (0x7FU)
#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_IPG_LEN_SHIFT (0U)
/*! IPG_LEN - Transmit inter-packet gap value. */
#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_IPG_LEN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_IPG_LEN_SHIFT)) & NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_IPG_LEN_MASK)

#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_MASK (0x700U)
#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_SHIFT (8U)
/*! FLEX_PREAMBLE_CNT - Flexible Preamble Count */
#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_SHIFT)) & NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_CNT_MASK)

#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_MASK (0x80000000U)
#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_SHIFT (31U)
/*! FLEX_PREAMBLE_EN - Enable Flexible Preamble Count */
#define NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_SHIFT)) & NETC_ETH_LINK_PM1_TX_IPG_PREAMBLE_FLEX_PREAMBLE_EN_MASK)
/*! @} */

/*! @name PM1_IMASK - Port MAC 1 Interrupt Mask Register(INT_MASK) */
/*! @{ */

#define NETC_ETH_LINK_PM1_IMASK_MGI_MASK         (0x4000U)
#define NETC_ETH_LINK_PM1_IMASK_MGI_SHIFT        (14U)
/*! MGI - Magic packet detection indication event mask. */
#define NETC_ETH_LINK_PM1_IMASK_MGI(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IMASK_MGI_SHIFT)) & NETC_ETH_LINK_PM1_IMASK_MGI_MASK)

#define NETC_ETH_LINK_PM1_IMASK_TX_CSD_MASK      (0x200000U)
#define NETC_ETH_LINK_PM1_IMASK_TX_CSD_SHIFT     (21U)
/*! TX_CSD - Tx Clock Stop Detection */
#define NETC_ETH_LINK_PM1_IMASK_TX_CSD(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IMASK_TX_CSD_SHIFT)) & NETC_ETH_LINK_PM1_IMASK_TX_CSD_MASK)

#define NETC_ETH_LINK_PM1_IMASK_RX_CSD_MASK      (0x400000U)
#define NETC_ETH_LINK_PM1_IMASK_RX_CSD_SHIFT     (22U)
/*! RX_CSD - Rx Clock Stop Detection */
#define NETC_ETH_LINK_PM1_IMASK_RX_CSD(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IMASK_RX_CSD_SHIFT)) & NETC_ETH_LINK_PM1_IMASK_RX_CSD_MASK)

#define NETC_ETH_LINK_PM1_IMASK_SPD_DUP_MASK     (0x800000U)
#define NETC_ETH_LINK_PM1_IMASK_SPD_DUP_SHIFT    (23U)
/*! SPD_DUP - Speed/Duplex change event mask. */
#define NETC_ETH_LINK_PM1_IMASK_SPD_DUP(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IMASK_SPD_DUP_SHIFT)) & NETC_ETH_LINK_PM1_IMASK_SPD_DUP_MASK)

#define NETC_ETH_LINK_PM1_IMASK_MRG_SERR_MASK    (0x8000000U)
#define NETC_ETH_LINK_PM1_IMASK_MRG_SERR_SHIFT   (27U)
/*! MRG_SERR - MAC merge frame SMD error received event interrupt mask */
#define NETC_ETH_LINK_PM1_IMASK_MRG_SERR(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IMASK_MRG_SERR_SHIFT)) & NETC_ETH_LINK_PM1_IMASK_MRG_SERR_MASK)

#define NETC_ETH_LINK_PM1_IMASK_MRG_AERR_MASK    (0x10000000U)
#define NETC_ETH_LINK_PM1_IMASK_MRG_AERR_SHIFT   (28U)
/*! MRG_AERR - MAC merge frame assembly error event interrupt mask */
#define NETC_ETH_LINK_PM1_IMASK_MRG_AERR(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IMASK_MRG_AERR_SHIFT)) & NETC_ETH_LINK_PM1_IMASK_MRG_AERR_MASK)
/*! @} */

/*! @name PM1_PAUSE_QUANTA - Port MAC 1 Pause Quanta Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_PAUSE_QUANTA_PQNT_MASK (0xFFFFU)
#define NETC_ETH_LINK_PM1_PAUSE_QUANTA_PQNT_SHIFT (0U)
/*! PQNT - Value to be used for the quanta value when XOFF is triggered. */
#define NETC_ETH_LINK_PM1_PAUSE_QUANTA_PQNT(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_PAUSE_QUANTA_PQNT_SHIFT)) & NETC_ETH_LINK_PM1_PAUSE_QUANTA_PQNT_MASK)
/*! @} */

/*! @name PM1_PAUSE_THRESH - Port MAC 1 Pause Quanta Threshold Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_PAUSE_THRESH_QTH_MASK  (0xFFFFU)
#define NETC_ETH_LINK_PM1_PAUSE_THRESH_QTH_SHIFT (0U)
/*! QTH - Quanta threshold. */
#define NETC_ETH_LINK_PM1_PAUSE_THRESH_QTH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_PAUSE_THRESH_QTH_SHIFT)) & NETC_ETH_LINK_PM1_PAUSE_THRESH_QTH_MASK)
/*! @} */

/*! @name PM1_RX_PAUSE_STATUS - Port MAC 1 Receive Pause Status Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_RX_PAUSE_STATUS_PSTAT_MASK (0x1U)
#define NETC_ETH_LINK_PM1_RX_PAUSE_STATUS_PSTAT_SHIFT (0U)
/*! PSTAT - Pause status. */
#define NETC_ETH_LINK_PM1_RX_PAUSE_STATUS_PSTAT(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_RX_PAUSE_STATUS_PSTAT_SHIFT)) & NETC_ETH_LINK_PM1_RX_PAUSE_STATUS_PSTAT_MASK)
/*! @} */

/*! @name PM1_LPWAKE_TIMER - Port MAC 1 EEE Low Power Wakeup Timer Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_LPWAKE_TIMER_TW_SYS_TX_MASK (0xFFFFFFU)
#define NETC_ETH_LINK_PM1_LPWAKE_TIMER_TW_SYS_TX_SHIFT (0U)
/*! TW_SYS_TX - EEE System transmit wait time */
#define NETC_ETH_LINK_PM1_LPWAKE_TIMER_TW_SYS_TX(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_LPWAKE_TIMER_TW_SYS_TX_SHIFT)) & NETC_ETH_LINK_PM1_LPWAKE_TIMER_TW_SYS_TX_MASK)
/*! @} */

/*! @name PM1_SLEEP_TIMER - Port MAC 1 Transmit EEE Low Power Timer Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_SLEEP_TIMER_SLEEPT_MASK (0xFFFFFFU)
#define NETC_ETH_LINK_PM1_SLEEP_TIMER_SLEEPT_SHIFT (0U)
#define NETC_ETH_LINK_PM1_SLEEP_TIMER_SLEEPT(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_SLEEP_TIMER_SLEEPT_SHIFT)) & NETC_ETH_LINK_PM1_SLEEP_TIMER_SLEEPT_MASK)
/*! @} */

/*! @name PM1_SINGLE_STEP - Port MAC 1 IEEE1588 Single-Step Control Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_SINGLE_STEP_CH_MASK    (0x40U)
#define NETC_ETH_LINK_PM1_SINGLE_STEP_CH_SHIFT   (6U)
/*! CH - Checksum update */
#define NETC_ETH_LINK_PM1_SINGLE_STEP_CH(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_SINGLE_STEP_CH_SHIFT)) & NETC_ETH_LINK_PM1_SINGLE_STEP_CH_MASK)

#define NETC_ETH_LINK_PM1_SINGLE_STEP_OFFSET_MASK (0xFF80U)
#define NETC_ETH_LINK_PM1_SINGLE_STEP_OFFSET_SHIFT (7U)
#define NETC_ETH_LINK_PM1_SINGLE_STEP_OFFSET(x)  (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_SINGLE_STEP_OFFSET_SHIFT)) & NETC_ETH_LINK_PM1_SINGLE_STEP_OFFSET_MASK)

#define NETC_ETH_LINK_PM1_SINGLE_STEP_EN_MASK    (0x80000000U)
#define NETC_ETH_LINK_PM1_SINGLE_STEP_EN_SHIFT   (31U)
/*! EN - IEEE-1588 Single-Step enable. */
#define NETC_ETH_LINK_PM1_SINGLE_STEP_EN(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_SINGLE_STEP_EN_SHIFT)) & NETC_ETH_LINK_PM1_SINGLE_STEP_EN_MASK)
/*! @} */

/*! @name PM1_HD_BACKOFF_ENTROPY - Port MAC 1 half-duplex backoff entropy register */
/*! @{ */

#define NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_MASK (0x3FFU)
#define NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_SHIFT (0U)
/*! HD_BACKOFF_ENTROPY - Half duplex backoff entropy */
#define NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_SHIFT)) & NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_HD_BACKOFF_ENTROPY_MASK)

#define NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_MASK (0x80000000U)
#define NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_SHIFT (31U)
/*! SW_ENTROPY_VALID - SW programmable entropy valid */
#define NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_SHIFT)) & NETC_ETH_LINK_PM1_HD_BACKOFF_ENTROPY_SW_ENTROPY_VALID_MASK)
/*! @} */

/*! @name PM1_HD_FLOW_CTRL - Port MAC 1 Half-Duplex Flow Control Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_OFF_MIN_MASK (0xFFFU)
#define NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_OFF_MIN_SHIFT (0U)
/*! HD_BP_OFF_MIN - Half-Duplex Back-Pressure Off Minimum */
#define NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_OFF_MIN(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_OFF_MIN_SHIFT)) & NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_OFF_MIN_MASK)

#define NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_ON_MAX_MASK (0xFFF0000U)
#define NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_ON_MAX_SHIFT (16U)
/*! HD_BP_ON_MAX - Half-Duplex Back-Pressure On Maximum */
#define NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_ON_MAX(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_ON_MAX_SHIFT)) & NETC_ETH_LINK_PM1_HD_FLOW_CTRL_HD_BP_ON_MAX_MASK)
/*! @} */

/*! @name PM1_STATN_CONFIG - Port MAC 1 Statistics Configuration Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_STATN_CONFIG_SAT_MASK  (0x1U)
#define NETC_ETH_LINK_PM1_STATN_CONFIG_SAT_SHIFT (0U)
#define NETC_ETH_LINK_PM1_STATN_CONFIG_SAT(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_STATN_CONFIG_SAT_SHIFT)) & NETC_ETH_LINK_PM1_STATN_CONFIG_SAT_MASK)

#define NETC_ETH_LINK_PM1_STATN_CONFIG_COD_MASK  (0x2U)
#define NETC_ETH_LINK_PM1_STATN_CONFIG_COD_SHIFT (1U)
#define NETC_ETH_LINK_PM1_STATN_CONFIG_COD(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_STATN_CONFIG_COD_SHIFT)) & NETC_ETH_LINK_PM1_STATN_CONFIG_COD_MASK)

#define NETC_ETH_LINK_PM1_STATN_CONFIG_CLR_MASK  (0x4U)
#define NETC_ETH_LINK_PM1_STATN_CONFIG_CLR_SHIFT (2U)
#define NETC_ETH_LINK_PM1_STATN_CONFIG_CLR(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_STATN_CONFIG_CLR_SHIFT)) & NETC_ETH_LINK_PM1_STATN_CONFIG_CLR_MASK)

#define NETC_ETH_LINK_PM1_STATN_CONFIG_WEN_MASK  (0x8U)
#define NETC_ETH_LINK_PM1_STATN_CONFIG_WEN_SHIFT (3U)
/*! WEN - Write enable for Tx/Rx stats registers */
#define NETC_ETH_LINK_PM1_STATN_CONFIG_WEN(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_STATN_CONFIG_WEN_SHIFT)) & NETC_ETH_LINK_PM1_STATN_CONFIG_WEN_MASK)
/*! @} */

/*! @name PM1_REOCTN - Port MAC 1 Receive Ethernet Octets Counter(etherStatsOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_REOCTN_REOCTn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_REOCTN_REOCTn_SHIFT    (0U)
/*! REOCTn - Incremented for each octet received in both good and bad packets. */
#define NETC_ETH_LINK_PM1_REOCTN_REOCTn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_REOCTN_REOCTn_SHIFT)) & NETC_ETH_LINK_PM1_REOCTN_REOCTn_MASK)
/*! @} */

/*! @name PM1_ROCTN - Port MAC 1 Receive Octets Counter(iflnOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_ROCTN_ROCTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_ROCTN_ROCTn_SHIFT      (0U)
/*! ROCTn - Incremented for each octet received except preamble (that is, Header, Payload, Pad and
 *    FCS) for all valid frames and valid PAUSE frames received.
 */
#define NETC_ETH_LINK_PM1_ROCTN_ROCTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_ROCTN_ROCTn_SHIFT)) & NETC_ETH_LINK_PM1_ROCTN_ROCTn_MASK)
/*! @} */

/*! @name PM1_RXPFN - Port MAC 1 Receive Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RXPFN_RXPFn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RXPFN_RXPFn_SHIFT      (0U)
/*! RXPFn - Incremented for each valid PAUSE frame received . */
#define NETC_ETH_LINK_PM1_RXPFN_RXPFn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RXPFN_RXPFn_SHIFT)) & NETC_ETH_LINK_PM1_RXPFN_RXPFn_MASK)
/*! @} */

/*! @name PM1_RFRMN - Port MAC 1 Receive Frame Counter Register(aFramesReceivedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RFRMN_RFRMn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RFRMN_RFRMn_SHIFT      (0U)
/*! RFRMn - Incremented for each frame received without error, including PAUSE frames. */
#define NETC_ETH_LINK_PM1_RFRMN_RFRMn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RFRMN_RFRMn_SHIFT)) & NETC_ETH_LINK_PM1_RFRMN_RFRMn_MASK)
/*! @} */

/*! @name PM1_RFCSN - Port MAC 1 Receive Frame Check Sequence Error Counter Register() */
/*! @{ */

#define NETC_ETH_LINK_PM1_RFCSN_RFCSn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RFCSN_RFCSn_SHIFT      (0U)
/*! RFCSn - Incremented for each frame received with a CRC-32 error but the frame is otherwise of correct length. */
#define NETC_ETH_LINK_PM1_RFCSN_RFCSn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RFCSN_RFCSn_SHIFT)) & NETC_ETH_LINK_PM1_RFCSN_RFCSn_MASK)
/*! @} */

/*! @name PM1_RVLANN - Port MAC 1 Receive VLAN Frame Counter Register(VLANReceivedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RVLANN_RVLANn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RVLANN_RVLANn_SHIFT    (0U)
/*! RVLANn - Incremented for each valid VLAN tagged frame received with ethertype 0x8100 */
#define NETC_ETH_LINK_PM1_RVLANN_RVLANn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RVLANN_RVLANn_SHIFT)) & NETC_ETH_LINK_PM1_RVLANN_RVLANn_MASK)
/*! @} */

/*! @name PM1_RERRN - Port MAC 1 Receive Frame Error Counter Register(ifInErrorsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RERRN_RERRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RERRN_RERRn_SHIFT      (0U)
/*! RERRn - Incremented for each frame received with an error (except for undersized/fragment frame): */
#define NETC_ETH_LINK_PM1_RERRN_RERRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RERRN_RERRn_SHIFT)) & NETC_ETH_LINK_PM1_RERRN_RERRn_MASK)
/*! @} */

/*! @name PM1_RUCAN - Port MAC 1 Receive Unicast Frame Counter Register(ifInUcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RUCAN_RUCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RUCAN_RUCAn_SHIFT      (0U)
/*! RUCAn - Incremented for each valid frame received (on the receive FIFO interface) in which bit 0
 *    of the destination address was 0 .
 */
#define NETC_ETH_LINK_PM1_RUCAN_RUCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RUCAN_RUCAn_SHIFT)) & NETC_ETH_LINK_PM1_RUCAN_RUCAn_MASK)
/*! @} */

/*! @name PM1_RMCAN - Port MAC 1 Receive Multicast Frame Counter Register(ifInMulticastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RMCAN_RMCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RMCAN_RMCAn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_RMCAN_RMCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RMCAN_RMCAn_SHIFT)) & NETC_ETH_LINK_PM1_RMCAN_RMCAn_MASK)
/*! @} */

/*! @name PM1_RBCAN - Port MAC 1 Receive Broadcast Frame Counter Register(ifInBroadcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RBCAN_RBCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RBCAN_RBCAn_SHIFT      (0U)
/*! RBCAn - Incremented for each valid frame received (on the receive FIFO interface) in which all
 *    bits of the destination address were 1 .
 */
#define NETC_ETH_LINK_PM1_RBCAN_RBCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RBCAN_RBCAn_SHIFT)) & NETC_ETH_LINK_PM1_RBCAN_RBCAn_MASK)
/*! @} */

/*! @name PM1_RDRPN - Port MAC 1 Receive Dropped Packets Counter Register(etherStatsDropEventsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RDRPN_RDRPn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RDRPN_RDRPn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_RDRPN_RDRPn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RDRPN_RDRPn_SHIFT)) & NETC_ETH_LINK_PM1_RDRPN_RDRPn_MASK)
/*! @} */

/*! @name PM1_RPKTN - Port MAC 1 Receive Packets Counter Register(etherStatsPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RPKTN_RPKTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RPKTN_RPKTn_SHIFT      (0U)
/*! RPKTn - Incremented for each good or bad packet received. */
#define NETC_ETH_LINK_PM1_RPKTN_RPKTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RPKTN_RPKTn_SHIFT)) & NETC_ETH_LINK_PM1_RPKTN_RPKTn_MASK)
/*! @} */

/*! @name PM1_RUNDN - Port MAC 1 Receive Undersized Packet Counter Register(etherStatsUndersizePktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RUNDN_RUNDn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RUNDN_RUNDn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_RUNDN_RUNDn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RUNDN_RUNDn_SHIFT)) & NETC_ETH_LINK_PM1_RUNDN_RUNDn_MASK)
/*! @} */

/*! @name PM1_R64N - Port MAC 1 Receive 64-Octet Packet Counter Register(etherStatsPkts64OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_R64N_R64n_MASK         (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_R64N_R64n_SHIFT        (0U)
/*! R64n - Incremented for each 64-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM1_R64N_R64n(x)           (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_R64N_R64n_SHIFT)) & NETC_ETH_LINK_PM1_R64N_R64n_MASK)
/*! @} */

/*! @name PM1_R127N - Port MAC 1 Receive 65 to 127-Octet Packet Counter Register(etherStatsPkts65to127OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_R127N_R127n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_R127N_R127n_SHIFT      (0U)
/*! R127n - Incremented for each 65- to 127-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM1_R127N_R127n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_R127N_R127n_SHIFT)) & NETC_ETH_LINK_PM1_R127N_R127n_MASK)
/*! @} */

/*! @name PM1_R255N - Port MAC 1 Receive 128 to 255-Octet Packet Counter Register(etherStatsPkts128to255OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_R255N_R255n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_R255N_R255n_SHIFT      (0U)
/*! R255n - Incremented for each 128- to 255-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM1_R255N_R255n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_R255N_R255n_SHIFT)) & NETC_ETH_LINK_PM1_R255N_R255n_MASK)
/*! @} */

/*! @name PM1_R511N - Port MAC 1 Receive 256 to 511-Octet Packet Counter Register(etherStatsPkts256to511OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_R511N_R511n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_R511N_R511n_SHIFT      (0U)
/*! R511n - Incremented for each 256- to 511-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM1_R511N_R511n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_R511N_R511n_SHIFT)) & NETC_ETH_LINK_PM1_R511N_R511n_MASK)
/*! @} */

/*! @name PM1_R1023N - Port MAC 1 Receive 512 to 1023-Octet Packet Counter Register(etherStatsPkts512to1023OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_R1023N_R1023n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_R1023N_R1023n_SHIFT    (0U)
/*! R1023n - Incremented for each 512- to 1023-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM1_R1023N_R1023n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_R1023N_R1023n_SHIFT)) & NETC_ETH_LINK_PM1_R1023N_R1023n_MASK)
/*! @} */

/*! @name PM1_R1522N - Port MAC 1 Receive 1024 to 1522-Octet Packet Counter Register(etherStatsPkts1024to1522OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_R1522N_R1522n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_R1522N_R1522n_SHIFT    (0U)
/*! R1522n - Incremented for each 1024- to 1522-octet frame received, good or bad. */
#define NETC_ETH_LINK_PM1_R1522N_R1522n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_R1522N_R1522n_SHIFT)) & NETC_ETH_LINK_PM1_R1522N_R1522n_MASK)
/*! @} */

/*! @name PM1_R1523XN - Port MAC 1 Receive 1523 to Max-Octet Packet Counter Register(etherStatsPkts1523toMaxOctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_R1523XN_R1523Xn_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_R1523XN_R1523Xn_SHIFT  (0U)
#define NETC_ETH_LINK_PM1_R1523XN_R1523Xn(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_R1523XN_R1523Xn_SHIFT)) & NETC_ETH_LINK_PM1_R1523XN_R1523Xn_MASK)
/*! @} */

/*! @name PM1_ROVRN - Port MAC 1 Receive Oversized Packet Counter Register(etherStatsOversizePktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_ROVRN_ROVRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_ROVRN_ROVRn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_ROVRN_ROVRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_ROVRN_ROVRn_SHIFT)) & NETC_ETH_LINK_PM1_ROVRN_ROVRn_MASK)
/*! @} */

/*! @name PM1_RJBRN - Port MAC 1 Receive Jabber Packet Counter Register(etherStatsJabbersn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RJBRN_RJBRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RJBRN_RJBRn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_RJBRN_RJBRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RJBRN_RJBRn_SHIFT)) & NETC_ETH_LINK_PM1_RJBRN_RJBRn_MASK)
/*! @} */

/*! @name PM1_RFRGN - Port MAC 1 Receive Fragment Packet Counter Register(etherStatsFragmentsn */
/*! @{ */

#define NETC_ETH_LINK_PM1_RFRGN_RFRGn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RFRGN_RFRGn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_RFRGN_RFRGn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RFRGN_RFRGn_SHIFT)) & NETC_ETH_LINK_PM1_RFRGN_RFRGn_MASK)
/*! @} */

/*! @name PM1_RCNPN - Port MAC 1 Receive Control Packet Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_RCNPN_RCNPn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RCNPN_RCNPn_SHIFT      (0U)
/*! RCNPn - Incremented for each valid control packet (type 0x8808) but not for PAUSE packets */
#define NETC_ETH_LINK_PM1_RCNPN_RCNPn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RCNPN_RCNPn_SHIFT)) & NETC_ETH_LINK_PM1_RCNPN_RCNPn_MASK)
/*! @} */

/*! @name PM1_RDRNTPN - Port MAC 1 Receive Dropped Not Truncated Packets Counter Register(etherStatsDropEventsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_RDRNTPN_RDRNTPn_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RDRNTPN_RDRNTPn_SHIFT  (0U)
/*! RDRNTPn - Incremented for each fully dropped packet (not truncated) due to internal errors of
 *    the MAC client. Occurs when a receive FIFO overflows.
 */
#define NETC_ETH_LINK_PM1_RDRNTPN_RDRNTPn(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RDRNTPN_RDRNTPn_SHIFT)) & NETC_ETH_LINK_PM1_RDRNTPN_RDRNTPn_MASK)
/*! @} */

/*! @name PM1_RMIN63N - Port MAC 1 Receive Valid Small Packet Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_RMIN63N_RMIN63n_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_RMIN63N_RMIN63n_SHIFT  (0U)
/*! RMIN63n - Incremented for each valid small packet less than 64B but greater or equal to the
 *    length programmed in PMa_MINFRM register
 */
#define NETC_ETH_LINK_PM1_RMIN63N_RMIN63n(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_RMIN63N_RMIN63n_SHIFT)) & NETC_ETH_LINK_PM1_RMIN63N_RMIN63n_MASK)
/*! @} */

/*! @name PM1_TEOCTN - Port MAC 1 Transmit Ethernet Octets Counter(etherStatsOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TEOCTN_TEOCTn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TEOCTN_TEOCTn_SHIFT    (0U)
/*! TEOCTn - Incremented for each octet transmitted in both good and bad packets. */
#define NETC_ETH_LINK_PM1_TEOCTN_TEOCTn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TEOCTN_TEOCTn_SHIFT)) & NETC_ETH_LINK_PM1_TEOCTN_TEOCTn_MASK)
/*! @} */

/*! @name PM1_TOCTN - Port MAC 1 Transmit Octets Counter Register(ifOutOctetsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TOCTN_TOCTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TOCTN_TOCTn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_TOCTN_TOCTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TOCTN_TOCTn_SHIFT)) & NETC_ETH_LINK_PM1_TOCTN_TOCTn_MASK)
/*! @} */

/*! @name PM1_TXPFN - Port MAC 1 Transmit Valid Pause Frame Counter Register(aPAUSEMACCtrlFramesReceivedn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TXPFN_TXPFn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TXPFN_TXPFn_SHIFT      (0U)
/*! TXPFn - Incremented for each valid PAUSE frame transmitted . Note: Pause frames forwarded to the
 *    MAC from MAC Client are not counted by TXPFn.
 */
#define NETC_ETH_LINK_PM1_TXPFN_TXPFn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TXPFN_TXPFn_SHIFT)) & NETC_ETH_LINK_PM1_TXPFN_TXPFn_MASK)
/*! @} */

/*! @name PM1_TFRMN - Port MAC 1 Transmit Frame Counter Register(aFramesTransmittedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TFRMN_TFRMn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TFRMN_TFRMn_SHIFT      (0U)
/*! TFRMn - Incremented for each frame transmitted without error, including PAUSE frames. */
#define NETC_ETH_LINK_PM1_TFRMN_TFRMn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TFRMN_TFRMn_SHIFT)) & NETC_ETH_LINK_PM1_TFRMN_TFRMn_MASK)
/*! @} */

/*! @name PM1_TFCSN - Port MAC 1 Transmit Frame Check Sequence Error Counter Register() */
/*! @{ */

#define NETC_ETH_LINK_PM1_TFCSN_TFCSn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TFCSN_TFCSn_SHIFT      (0U)
/*! TFCSn - Incremented for each frame transmitted with a CRC-32 error except for underflows. */
#define NETC_ETH_LINK_PM1_TFCSN_TFCSn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TFCSN_TFCSn_SHIFT)) & NETC_ETH_LINK_PM1_TFCSN_TFCSn_MASK)
/*! @} */

/*! @name PM1_TVLANN - Port MAC 1 Transmit VLAN Frame Counter Register(VLANTransmittedOKn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TVLANN_TVLANn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TVLANN_TVLANn_SHIFT    (0U)
/*! TVLANn - Incremented for each valid VLAN tagged frame transmitted with ethertype 0x8100. */
#define NETC_ETH_LINK_PM1_TVLANN_TVLANn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TVLANN_TVLANn_SHIFT)) & NETC_ETH_LINK_PM1_TVLANN_TVLANn_MASK)
/*! @} */

/*! @name PM1_TERRN - Port MAC 1 Transmit Frame Error Counter Register(ifOutErrorsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TERRN_TERRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TERRN_TERRn_SHIFT      (0U)
/*! TERRn - Transmit frame error count */
#define NETC_ETH_LINK_PM1_TERRN_TERRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TERRN_TERRn_SHIFT)) & NETC_ETH_LINK_PM1_TERRN_TERRn_MASK)
/*! @} */

/*! @name PM1_TUCAN - Port MAC 1 Transmit Unicast Frame Counter Register(ifOutUcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TUCAN_TUCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TUCAN_TUCAn_SHIFT      (0U)
/*! TUCAn - Incremented for each valid frame transmitted (to the FIFO interface) in which bit 0 of the destination address was 0. */
#define NETC_ETH_LINK_PM1_TUCAN_TUCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TUCAN_TUCAn_SHIFT)) & NETC_ETH_LINK_PM1_TUCAN_TUCAn_MASK)
/*! @} */

/*! @name PM1_TMCAN - Port MAC 1 Transmit Multicast Frame Counter Register(ifOutMulticastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TMCAN_TMCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TMCAN_TMCAn_SHIFT      (0U)
#define NETC_ETH_LINK_PM1_TMCAN_TMCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TMCAN_TMCAn_SHIFT)) & NETC_ETH_LINK_PM1_TMCAN_TMCAn_MASK)
/*! @} */

/*! @name PM1_TBCAN - Port MAC 1 Transmit Broadcast Frame Counter Register(ifOutBroadcastPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TBCAN_TBCAn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TBCAN_TBCAn_SHIFT      (0U)
/*! TBCAn - Incremented for each valid frame transmitted (to the FIFO interface) in which all bits
 *    of the destination address were 1 .
 */
#define NETC_ETH_LINK_PM1_TBCAN_TBCAn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TBCAN_TBCAn_SHIFT)) & NETC_ETH_LINK_PM1_TBCAN_TBCAn_MASK)
/*! @} */

/*! @name PM1_TPKTN - Port MAC 1 Transmit Packets Counter Register(etherStatsPktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TPKTN_TPKTn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TPKTN_TPKTn_SHIFT      (0U)
/*! TPKTn - Incremented for each good or bad packet transmitted. */
#define NETC_ETH_LINK_PM1_TPKTN_TPKTn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TPKTN_TPKTn_SHIFT)) & NETC_ETH_LINK_PM1_TPKTN_TPKTn_MASK)
/*! @} */

/*! @name PM1_TUNDN - Port MAC 1 Transmit Undersized Packet Counter Register(etherStatsUndersizePktsn) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TUNDN_TUNDn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TUNDN_TUNDn_SHIFT      (0U)
/*! TUNDn - Incremented for each packet transmitted that was less than 64 octets long with a good CRC. */
#define NETC_ETH_LINK_PM1_TUNDN_TUNDn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TUNDN_TUNDn_SHIFT)) & NETC_ETH_LINK_PM1_TUNDN_TUNDn_MASK)
/*! @} */

/*! @name PM1_T64N - Port MAC 1 Transmit 64-Octet Packet Counter Register (etherStatsPkts64OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_T64N_T64n_MASK         (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_T64N_T64n_SHIFT        (0U)
/*! T64n - Incremented for each 64-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM1_T64N_T64n(x)           (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_T64N_T64n_SHIFT)) & NETC_ETH_LINK_PM1_T64N_T64n_MASK)
/*! @} */

/*! @name PM1_T127N - Port MAC 1 Transmit 65 to 127-Octet Packet Counter Register (etherStatsPkts65to127OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_T127N_T127n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_T127N_T127n_SHIFT      (0U)
/*! T127n - Incremented for each 65 to 127-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM1_T127N_T127n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_T127N_T127n_SHIFT)) & NETC_ETH_LINK_PM1_T127N_T127n_MASK)
/*! @} */

/*! @name PM1_T255N - Port MAC 1 Transmit 128 to 255-Octet Packet Counter Register (etherStatsPkts128to255OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_T255N_T255n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_T255N_T255n_SHIFT      (0U)
/*! T255n - Incremented for each 128 to 255-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM1_T255N_T255n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_T255N_T255n_SHIFT)) & NETC_ETH_LINK_PM1_T255N_T255n_MASK)
/*! @} */

/*! @name PM1_T511N - Port MAC 1 Transmit 256 to 511-Octet Packet Counter Register (etherStatsPkts256to511OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_T511N_T511n_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_T511N_T511n_SHIFT      (0U)
/*! T511n - Incremented for each 256 to 511-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM1_T511N_T511n(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_T511N_T511n_SHIFT)) & NETC_ETH_LINK_PM1_T511N_T511n_MASK)
/*! @} */

/*! @name PM1_T1023N - Port MAC 1 Transmit 512 to 1023-Octet Packet Counter Register (etherStatsPkts512to1023OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_T1023N_T1023n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_T1023N_T1023n_SHIFT    (0U)
/*! T1023n - Incremented for each 512 to 1023-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM1_T1023N_T1023n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_T1023N_T1023n_SHIFT)) & NETC_ETH_LINK_PM1_T1023N_T1023n_MASK)
/*! @} */

/*! @name PM1_T1522N - Port MAC 1 Transmit 1024 to 1522-Octet Packet Counter Register (etherStatsPkts1024to1522OctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_T1522N_T1522n_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_T1522N_T1522n_SHIFT    (0U)
/*! T1522n - Incremented for each 1024- to 1522-octet frame transmitted, good or bad. */
#define NETC_ETH_LINK_PM1_T1522N_T1522n(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_T1522N_T1522n_SHIFT)) & NETC_ETH_LINK_PM1_T1522N_T1522n_MASK)
/*! @} */

/*! @name PM1_T1523XN - Port MAC 1 Transmit 1523 to TX_MTU-Octet Packet Counter Register (etherStatsPkts1523toMaxOctetsN) */
/*! @{ */

#define NETC_ETH_LINK_PM1_T1523XN_T1523Xn_MASK   (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_T1523XN_T1523Xn_SHIFT  (0U)
#define NETC_ETH_LINK_PM1_T1523XN_T1523Xn(x)     (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_T1523XN_T1523Xn_SHIFT)) & NETC_ETH_LINK_PM1_T1523XN_T1523Xn_MASK)
/*! @} */

/*! @name PM1_TCNPN - Port MAC 1 Transmit Control Packet Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_TCNPN_TCNPn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TCNPN_TCNPn_SHIFT      (0U)
/*! TCNPn - Incremented for each valid control packet transmitted (type 0x8808) but not for PAUSE packets */
#define NETC_ETH_LINK_PM1_TCNPN_TCNPn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TCNPN_TCNPn_SHIFT)) & NETC_ETH_LINK_PM1_TCNPN_TCNPn_MASK)
/*! @} */

/*! @name PM1_TDFRN - Port MAC 1 Transmit Deferred Packet Counter Register(aFramesWithDeferredXmissions) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TDFRN_TDFRn_MASK       (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TDFRN_TDFRn_SHIFT      (0U)
/*! TDFRn - Increments for successful transmissions, without retransmits, that were deferred (half-duplex only). */
#define NETC_ETH_LINK_PM1_TDFRN_TDFRn(x)         (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TDFRN_TDFRn_SHIFT)) & NETC_ETH_LINK_PM1_TDFRN_TDFRn_MASK)
/*! @} */

/*! @name PM1_TMCOLN - Port MAC 1 Transmit Multiple Collisions Counter Register(aMultipleCollisionFrames) */
/*! @{ */

#define NETC_ETH_LINK_PM1_TMCOLN_TMCOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TMCOLN_TMCOLn_SHIFT    (0U)
/*! TMCOLn - Increments for successful transmission after more than one retransmission (half-duplex only). */
#define NETC_ETH_LINK_PM1_TMCOLN_TMCOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TMCOLN_TMCOLn_SHIFT)) & NETC_ETH_LINK_PM1_TMCOLN_TMCOLn_MASK)
/*! @} */

/*! @name PM1_TSCOLN - Port MAC 1 Transmit Single Collision Counter(aSingleCollisionFrames) Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_TSCOLN_TSCOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TSCOLN_TSCOLn_SHIFT    (0U)
/*! TSCOLn - Increments for successful transmission after one retransmission (half-duplex only). */
#define NETC_ETH_LINK_PM1_TSCOLN_TSCOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TSCOLN_TSCOLn_SHIFT)) & NETC_ETH_LINK_PM1_TSCOLN_TSCOLn_MASK)
/*! @} */

/*! @name PM1_TLCOLN - Port MAC 1 Transmit Late Collision Counter(aLateCollisions) Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_TLCOLN_TLCOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TLCOLN_TLCOLn_SHIFT    (0U)
/*! TLCOLn - Late collision occurred. Frame corrupted / discarded (half-duplex only) */
#define NETC_ETH_LINK_PM1_TLCOLN_TLCOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TLCOLN_TLCOLn_SHIFT)) & NETC_ETH_LINK_PM1_TLCOLN_TLCOLn_MASK)
/*! @} */

/*! @name PM1_TECOLN - Port MAC 1 Transmit Excessive Collisions Counter Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_TECOLN_TECOLn_MASK     (0xFFFFFFFFFFFFFFFFU)
#define NETC_ETH_LINK_PM1_TECOLN_TECOLn_SHIFT    (0U)
/*! TECOLn - Excessive collisions occurred. Frame was discarded (half-duplex only) */
#define NETC_ETH_LINK_PM1_TECOLN_TECOLn(x)       (((uint64_t)(((uint64_t)(x)) << NETC_ETH_LINK_PM1_TECOLN_TECOLn_SHIFT)) & NETC_ETH_LINK_PM1_TECOLN_TECOLn_MASK)
/*! @} */

/*! @name PM1_IF_MODE - Port MAC 1 Interface Mode Control Register */
/*! @{ */

#define NETC_ETH_LINK_PM1_IF_MODE_IFMODE_MASK    (0x7U)
#define NETC_ETH_LINK_PM1_IF_MODE_IFMODE_SHIFT   (0U)
/*! IFMODE - Interface mode */
#define NETC_ETH_LINK_PM1_IF_MODE_IFMODE(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IF_MODE_IFMODE_SHIFT)) & NETC_ETH_LINK_PM1_IF_MODE_IFMODE_MASK)

#define NETC_ETH_LINK_PM1_IF_MODE_REVMII_MASK    (0x8U)
#define NETC_ETH_LINK_PM1_IF_MODE_REVMII_SHIFT   (3U)
/*! REVMII - Reverse Mode
 *  0b0..Reverse mode disabled - port is in MAC mode
 *  0b1..Reverse mode enabled - port is in PHY mode
 */
#define NETC_ETH_LINK_PM1_IF_MODE_REVMII(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IF_MODE_REVMII_SHIFT)) & NETC_ETH_LINK_PM1_IF_MODE_REVMII_MASK)

#define NETC_ETH_LINK_PM1_IF_MODE_M10_MASK       (0x10U)
#define NETC_ETH_LINK_PM1_IF_MODE_M10_SHIFT      (4U)
/*! M10
 *  0b0..100 Mbps
 *  0b1..10 Mbps
 */
#define NETC_ETH_LINK_PM1_IF_MODE_M10(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IF_MODE_M10_SHIFT)) & NETC_ETH_LINK_PM1_IF_MODE_M10_MASK)

#define NETC_ETH_LINK_PM1_IF_MODE_HD_MASK        (0x40U)
#define NETC_ETH_LINK_PM1_IF_MODE_HD_SHIFT       (6U)
/*! HD - Half-duplex
 *  0b0..full duplex
 *  0b1..half duplex
 */
#define NETC_ETH_LINK_PM1_IF_MODE_HD(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IF_MODE_HD_SHIFT)) & NETC_ETH_LINK_PM1_IF_MODE_HD_MASK)

#define NETC_ETH_LINK_PM1_IF_MODE_CLK_STOP_MASK  (0x1000U)
#define NETC_ETH_LINK_PM1_IF_MODE_CLK_STOP_SHIFT (12U)
/*! CLK_STOP - Clock Stop
 *  0b0..Not stoppable
 *  0b1..Stoppable
 */
#define NETC_ETH_LINK_PM1_IF_MODE_CLK_STOP(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IF_MODE_CLK_STOP_SHIFT)) & NETC_ETH_LINK_PM1_IF_MODE_CLK_STOP_MASK)

#define NETC_ETH_LINK_PM1_IF_MODE_SSP_MASK       (0x6000U)
#define NETC_ETH_LINK_PM1_IF_MODE_SSP_SHIFT      (13U)
/*! SSP - Set Speed
 *  0b00..100 Mbps
 *  0b01..10 Mbps
 *  0b10..1 Gbps
 *  0b11..reserved
 */
#define NETC_ETH_LINK_PM1_IF_MODE_SSP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PM1_IF_MODE_SSP_SHIFT)) & NETC_ETH_LINK_PM1_IF_MODE_SSP_MASK)
/*! @} */

/*! @name MAC_MERGE_MMCSR - Port MAC Merge Control and Status Register */
/*! @{ */

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPS_MASK   (0x1U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPS_SHIFT  (0U)
/*! LPS - Local preemption supported */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPS(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_LPS_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_LPS_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPE_MASK   (0x2U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPE_SHIFT  (1U)
/*! LPE - Local preemption enabled */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPE(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_LPE_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_LPE_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPA_MASK   (0x4U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPA_SHIFT  (2U)
/*! LPA - Local preemption active */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LPA(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_LPA_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_LPA_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LAFS_MASK  (0x18U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LAFS_SHIFT (3U)
/*! LAFS - Local additional fragment size */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LAFS(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_LAFS_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_LAFS_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPS_MASK   (0x20U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPS_SHIFT  (5U)
/*! RPS - Remote preemption supported */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPS(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_RPS_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_RPS_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPE_MASK   (0x40U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPE_SHIFT  (6U)
/*! RPE - Remote preemption enabled */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPE(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_RPE_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_RPE_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPA_MASK   (0x80U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPA_SHIFT  (7U)
/*! RPA - Remote preemption active */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RPA(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_RPA_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_RPA_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RAFS_MASK  (0x300U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RAFS_SHIFT (8U)
/*! RAFS - Remote additional fragment size */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_RAFS(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_RAFS_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_RAFS_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_ME_MASK    (0x18000U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_ME_SHIFT   (15U)
/*! ME - Merge enabled */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_ME(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_ME_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_ME_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VDIS_MASK  (0x20000U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VDIS_SHIFT (17U)
/*! VDIS - Verify disabled */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VDIS(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_VDIS_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_VDIS_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VSTS_MASK  (0x1C0000U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VSTS_SHIFT (18U)
/*! VSTS - Verify status */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VSTS(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_VSTS_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_VSTS_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_TXSTS_MASK (0x600000U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_TXSTS_SHIFT (21U)
/*! TXSTS - Merge status */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_TXSTS(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_TXSTS_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_TXSTS_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VT_MASK    (0x3F800000U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VT_SHIFT   (23U)
/*! VT - Verify Time */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_VT(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_VT_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_VT_MASK)

#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LINK_FAIL_MASK (0x80000000U)
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LINK_FAIL_SHIFT (31U)
/*! LINK_FAIL - Link Fail */
#define NETC_ETH_LINK_MAC_MERGE_MMCSR_LINK_FAIL(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMCSR_LINK_FAIL_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMCSR_LINK_FAIL_MASK)
/*! @} */

/*! @name MAC_MERGE_MMFAECR - Port MAC Merge Frame Assembly Error Count Register */
/*! @{ */

#define NETC_ETH_LINK_MAC_MERGE_MMFAECR_MMFAEC_MASK (0xFFFFFFFFU)
#define NETC_ETH_LINK_MAC_MERGE_MMFAECR_MMFAEC_SHIFT (0U)
#define NETC_ETH_LINK_MAC_MERGE_MMFAECR_MMFAEC(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMFAECR_MMFAEC_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMFAECR_MMFAEC_MASK)
/*! @} */

/*! @name MAC_MERGE_MMFSECR - Port MAC Merge Frame SMD Error Count Register */
/*! @{ */

#define NETC_ETH_LINK_MAC_MERGE_MMFSECR_MMFSEC_MASK (0xFFFFFFFFU)
#define NETC_ETH_LINK_MAC_MERGE_MMFSECR_MMFSEC_SHIFT (0U)
#define NETC_ETH_LINK_MAC_MERGE_MMFSECR_MMFSEC(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMFSECR_MMFSEC_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMFSECR_MMFSEC_MASK)
/*! @} */

/*! @name MAC_MERGE_MMFAOCR - Port MAC Merge Frame Assembly OK Count Register */
/*! @{ */

#define NETC_ETH_LINK_MAC_MERGE_MMFAOCR_MMFAOC_MASK (0xFFFFFFFFU)
#define NETC_ETH_LINK_MAC_MERGE_MMFAOCR_MMFAOC_SHIFT (0U)
#define NETC_ETH_LINK_MAC_MERGE_MMFAOCR_MMFAOC(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMFAOCR_MMFAOC_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMFAOCR_MMFAOC_MASK)
/*! @} */

/*! @name MAC_MERGE_MMFCRXR - Port MAC Merge Fragment Count RX Register */
/*! @{ */

#define NETC_ETH_LINK_MAC_MERGE_MMFCRXR_MMFCRX_MASK (0xFFFFFFFFU)
#define NETC_ETH_LINK_MAC_MERGE_MMFCRXR_MMFCRX_SHIFT (0U)
#define NETC_ETH_LINK_MAC_MERGE_MMFCRXR_MMFCRX(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMFCRXR_MMFCRX_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMFCRXR_MMFCRX_MASK)
/*! @} */

/*! @name MAC_MERGE_MMFCTXR - Port MAC Merge Fragment Count TX Register */
/*! @{ */

#define NETC_ETH_LINK_MAC_MERGE_MMFCTXR_MMFCTX_MASK (0xFFFFFFFFU)
#define NETC_ETH_LINK_MAC_MERGE_MMFCTXR_MMFCTX_SHIFT (0U)
#define NETC_ETH_LINK_MAC_MERGE_MMFCTXR_MMFCTX(x) (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMFCTXR_MMFCTX_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMFCTXR_MMFCTX_MASK)
/*! @} */

/*! @name MAC_MERGE_MMHCR - Port MAC Merge Hold Count Register */
/*! @{ */

#define NETC_ETH_LINK_MAC_MERGE_MMHCR_MMHC_MASK  (0xFFFFFFFFU)
#define NETC_ETH_LINK_MAC_MERGE_MMHCR_MMHC_SHIFT (0U)
#define NETC_ETH_LINK_MAC_MERGE_MMHCR_MMHC(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_MAC_MERGE_MMHCR_MMHC_SHIFT)) & NETC_ETH_LINK_MAC_MERGE_MMHCR_MMHC_MASK)
/*! @} */

/*! @name PEMDIOCR - Port external MDIO configuration register */
/*! @{ */

#define NETC_ETH_LINK_PEMDIOCR_BSY2_MASK         (0x1U)
#define NETC_ETH_LINK_PEMDIOCR_BSY2_SHIFT        (0U)
/*! BSY2 - Busy 2 (same as bit 31)
 *  0b0..An MDIO transaction is not occurring; software may access other MDIO registers.
 *  0b1..An MDIO transaction is occurring.
 */
#define NETC_ETH_LINK_PEMDIOCR_BSY2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_BSY2_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_BSY2_MASK)

#define NETC_ETH_LINK_PEMDIOCR_MDIO_RD_ER_MASK   (0x2U)
#define NETC_ETH_LINK_PEMDIOCR_MDIO_RD_ER_SHIFT  (1U)
/*! MDIO_RD_ER
 *  0b0..No error on last MDIO transaction (read or write).
 *  0b1..An error was detected on the last MDIO transaction (read or write). Errors on internal MDIO accesses can
 *       be triggered by an access to an invalid device, or by a write to a shared on-die PHY device that has not
 *       been locked.
 */
#define NETC_ETH_LINK_PEMDIOCR_MDIO_RD_ER(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_MDIO_RD_ER_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_MDIO_RD_ER_MASK)

#define NETC_ETH_LINK_PEMDIOCR_MDIO_HOLD_MASK    (0x1CU)
#define NETC_ETH_LINK_PEMDIOCR_MDIO_HOLD_SHIFT   (2U)
/*! MDIO_HOLD - MDIO Hold Time
 *  0b000..1 NETC cycle
 *  0b001..3 NETC cycles
 *  0b010..5 NETC cycles (default - recommended value)
 *  0b011..7 NETC cycles
 *  0b100..9 NETC cycles
 *  0b101..11 NETC cycles
 *  0b110..13 NETC cycles
 *  0b111..15 NETC cycles
 */
#define NETC_ETH_LINK_PEMDIOCR_MDIO_HOLD(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_MDIO_HOLD_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_MDIO_HOLD_MASK)

#define NETC_ETH_LINK_PEMDIOCR_PRE_DIS_MASK      (0x20U)
#define NETC_ETH_LINK_PEMDIOCR_PRE_DIS_SHIFT     (5U)
/*! PRE_DIS - MDIO Preamble Disable
 *  0b0..Generation of MDIO preamble is enabled (default operation).
 *  0b1..Generation of MDIO preamble is disabled.
 */
#define NETC_ETH_LINK_PEMDIOCR_PRE_DIS(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_PRE_DIS_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_PRE_DIS_MASK)

#define NETC_ETH_LINK_PEMDIOCR_ENC45_MASK        (0x40U)
#define NETC_ETH_LINK_PEMDIOCR_ENC45_SHIFT       (6U)
/*! ENC45 - Enable Clause 45 Support
 *  0b0..Clause 22 transactions are used.
 *  0b1..Clause 45 transactions are used.
 */
#define NETC_ETH_LINK_PEMDIOCR_ENC45(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_ENC45_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_ENC45_MASK)

#define NETC_ETH_LINK_PEMDIOCR_MDIO_CLK_DIV_MASK (0xFF80U)
#define NETC_ETH_LINK_PEMDIOCR_MDIO_CLK_DIV_SHIFT (7U)
/*! MDIO_CLK_DIV - MDIO Clock Divisor */
#define NETC_ETH_LINK_PEMDIOCR_MDIO_CLK_DIV(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_MDIO_CLK_DIV_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_MDIO_CLK_DIV_MASK)

#define NETC_ETH_LINK_PEMDIOCR_WHOAMI_MASK       (0x70000U)
#define NETC_ETH_LINK_PEMDIOCR_WHOAMI_SHIFT      (16U)
/*! WHOAMI - Returns the link ID */
#define NETC_ETH_LINK_PEMDIOCR_WHOAMI(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_WHOAMI_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_WHOAMI_MASK)

#define NETC_ETH_LINK_PEMDIOCR_EHOLD_MASK        (0x400000U)
#define NETC_ETH_LINK_PEMDIOCR_EHOLD_SHIFT       (22U)
/*! EHOLD - Extended HOLD
 *  0b0..Normal operation, MDIO hold time is as specified in PEMDIOCR[MDIO_HOLD].
 *  0b1..Extended operation
 */
#define NETC_ETH_LINK_PEMDIOCR_EHOLD(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_EHOLD_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_EHOLD_MASK)

#define NETC_ETH_LINK_PEMDIOCR_NEG_MASK          (0x800000U)
#define NETC_ETH_LINK_PEMDIOCR_NEG_SHIFT         (23U)
/*! NEG
 *  0b0..normal operation - positive edge
 *  0b1..MDIO is driven by master on MDC negative edge (default for external MDIOs)
 */
#define NETC_ETH_LINK_PEMDIOCR_NEG(x)            (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_NEG_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_NEG_MASK)

#define NETC_ETH_LINK_PEMDIOCR_ADDR_ERR_MASK     (0x10000000U)
#define NETC_ETH_LINK_PEMDIOCR_ADDR_ERR_SHIFT    (28U)
/*! ADDR_ERR - Address Error
 *  0b0..Normal
 *  0b1..Error. An access control violation has occurred. The request address used does not match the MDIO PHY's
 *       address (clause 22) or MDIO port address (clause 45) assigned.
 */
#define NETC_ETH_LINK_PEMDIOCR_ADDR_ERR(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_ADDR_ERR_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_ADDR_ERR_MASK)

#define NETC_ETH_LINK_PEMDIOCR_CIM_MASK          (0x20000000U)
#define NETC_ETH_LINK_PEMDIOCR_CIM_SHIFT         (29U)
/*! CIM
 *  0b0..Masked
 *  0b1..Enabled
 */
#define NETC_ETH_LINK_PEMDIOCR_CIM(x)            (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_CIM_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_CIM_MASK)

#define NETC_ETH_LINK_PEMDIOCR_CMP_MASK          (0x40000000U)
#define NETC_ETH_LINK_PEMDIOCR_CMP_SHIFT         (30U)
/*! CMP - MDIO Command Completion
 *  0b0..An MDIO command completion did not occur.
 *  0b1..An MDIO command completion occurred.
 */
#define NETC_ETH_LINK_PEMDIOCR_CMP(x)            (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_CMP_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_CMP_MASK)

#define NETC_ETH_LINK_PEMDIOCR_BSY1_MASK         (0x80000000U)
#define NETC_ETH_LINK_PEMDIOCR_BSY1_SHIFT        (31U)
/*! BSY1 - Busy 1
 *  0b0..An MDIO transaction is not occurring; software may access other MDIO registers.
 *  0b1..An MDIO transaction is occurring.
 */
#define NETC_ETH_LINK_PEMDIOCR_BSY1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOCR_BSY1_SHIFT)) & NETC_ETH_LINK_PEMDIOCR_BSY1_MASK)
/*! @} */

/*! @name PEMDIOICR - Port external MDIO interface control register */
/*! @{ */

#define NETC_ETH_LINK_PEMDIOICR_DEV_ADDR_MASK    (0x1FU)
#define NETC_ETH_LINK_PEMDIOICR_DEV_ADDR_SHIFT   (0U)
/*! DEV_ADDR - 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
#define NETC_ETH_LINK_PEMDIOICR_DEV_ADDR(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOICR_DEV_ADDR_SHIFT)) & NETC_ETH_LINK_PEMDIOICR_DEV_ADDR_MASK)

#define NETC_ETH_LINK_PEMDIOICR_PORT_ADDR_MASK   (0x3E0U)
#define NETC_ETH_LINK_PEMDIOICR_PORT_ADDR_SHIFT  (5U)
/*! PORT_ADDR - 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
#define NETC_ETH_LINK_PEMDIOICR_PORT_ADDR(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOICR_PORT_ADDR_SHIFT)) & NETC_ETH_LINK_PEMDIOICR_PORT_ADDR_MASK)

#define NETC_ETH_LINK_PEMDIOICR_POST_INC_MASK    (0x4000U)
#define NETC_ETH_LINK_PEMDIOICR_POST_INC_SHIFT   (14U)
/*! POST_INC - MDIO read with address post-increment initiation. Self-clearing once transaction is complete. */
#define NETC_ETH_LINK_PEMDIOICR_POST_INC(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOICR_POST_INC_SHIFT)) & NETC_ETH_LINK_PEMDIOICR_POST_INC_MASK)

#define NETC_ETH_LINK_PEMDIOICR_READ_MASK        (0x8000U)
#define NETC_ETH_LINK_PEMDIOICR_READ_SHIFT       (15U)
/*! READ - MDIO read initiation. */
#define NETC_ETH_LINK_PEMDIOICR_READ(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOICR_READ_SHIFT)) & NETC_ETH_LINK_PEMDIOICR_READ_MASK)

#define NETC_ETH_LINK_PEMDIOICR_BSY_MASK         (0x80000000U)
#define NETC_ETH_LINK_PEMDIOICR_BSY_SHIFT        (31U)
/*! BSY - MDIO busy */
#define NETC_ETH_LINK_PEMDIOICR_BSY(x)           (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOICR_BSY_SHIFT)) & NETC_ETH_LINK_PEMDIOICR_BSY_MASK)
/*! @} */

/*! @name PEMDIOIDR - Port external MDIO interface data register */
/*! @{ */

#define NETC_ETH_LINK_PEMDIOIDR_MDIO_DATA_MASK   (0xFFFFU)
#define NETC_ETH_LINK_PEMDIOIDR_MDIO_DATA_SHIFT  (0U)
/*! MDIO_DATA - 16-bit MDIO data. */
#define NETC_ETH_LINK_PEMDIOIDR_MDIO_DATA(x)     (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOIDR_MDIO_DATA_SHIFT)) & NETC_ETH_LINK_PEMDIOIDR_MDIO_DATA_MASK)
/*! @} */

/*! @name PEMDIORAR - Port external MDIO register address register */
/*! @{ */

#define NETC_ETH_LINK_PEMDIORAR_REGADDR_MASK     (0xFFFFU)
#define NETC_ETH_LINK_PEMDIORAR_REGADDR_SHIFT    (0U)
/*! REGADDR - MDIO PHY register address. */
#define NETC_ETH_LINK_PEMDIORAR_REGADDR(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIORAR_REGADDR_SHIFT)) & NETC_ETH_LINK_PEMDIORAR_REGADDR_MASK)
/*! @} */

/*! @name PEMDIOSR - Port external MDIO status register */
/*! @{ */

#define NETC_ETH_LINK_PEMDIOSR_BSY_MASK          (0x1U)
#define NETC_ETH_LINK_PEMDIOSR_BSY_SHIFT         (0U)
/*! BSY - Global MDIO busy */
#define NETC_ETH_LINK_PEMDIOSR_BSY(x)            (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOSR_BSY_SHIFT)) & NETC_ETH_LINK_PEMDIOSR_BSY_MASK)

#define NETC_ETH_LINK_PEMDIOSR_WHT_LIST_MASK     (0x1F00U)
#define NETC_ETH_LINK_PEMDIOSR_WHT_LIST_SHIFT    (8U)
/*! WHT_LIST - PHY white list */
#define NETC_ETH_LINK_PEMDIOSR_WHT_LIST(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOSR_WHT_LIST_SHIFT)) & NETC_ETH_LINK_PEMDIOSR_WHT_LIST_MASK)

#define NETC_ETH_LINK_PEMDIOSR_WHT_LIST_ENA_MASK (0x8000U)
#define NETC_ETH_LINK_PEMDIOSR_WHT_LIST_ENA_SHIFT (15U)
/*! WHT_LIST_ENA - PHY white list enable */
#define NETC_ETH_LINK_PEMDIOSR_WHT_LIST_ENA(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOSR_WHT_LIST_ENA_SHIFT)) & NETC_ETH_LINK_PEMDIOSR_WHT_LIST_ENA_MASK)

#define NETC_ETH_LINK_PEMDIOSR_PORT_ID_MASK      (0x70000U)
#define NETC_ETH_LINK_PEMDIOSR_PORT_ID_SHIFT     (16U)
/*! PORT_ID - Port ID */
#define NETC_ETH_LINK_PEMDIOSR_PORT_ID(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOSR_PORT_ID_SHIFT)) & NETC_ETH_LINK_PEMDIOSR_PORT_ID_MASK)

#define NETC_ETH_LINK_PEMDIOSR_REQ_TYPE_MASK     (0x80000U)
#define NETC_ETH_LINK_PEMDIOSR_REQ_TYPE_SHIFT    (19U)
/*! REQ_TYPE - Port ID */
#define NETC_ETH_LINK_PEMDIOSR_REQ_TYPE(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PEMDIOSR_REQ_TYPE_SHIFT)) & NETC_ETH_LINK_PEMDIOSR_REQ_TYPE_MASK)
/*! @} */

/*! @name PPSCR - PHY status configuration register */
/*! @{ */

#define NETC_ETH_LINK_PPSCR_BSY_MASK             (0x1U)
#define NETC_ETH_LINK_PPSCR_BSY_SHIFT            (0U)
/*! BSY - MDIO busy */
#define NETC_ETH_LINK_PPSCR_BSY(x)               (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSCR_BSY_SHIFT)) & NETC_ETH_LINK_PPSCR_BSY_MASK)

#define NETC_ETH_LINK_PPSCR_MDIO_RD_ER_MASK      (0x2U)
#define NETC_ETH_LINK_PPSCR_MDIO_RD_ER_SHIFT     (1U)
/*! MDIO_RD_ER - MDIO read error */
#define NETC_ETH_LINK_PPSCR_MDIO_RD_ER(x)        (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSCR_MDIO_RD_ER_SHIFT)) & NETC_ETH_LINK_PPSCR_MDIO_RD_ER_MASK)

#define NETC_ETH_LINK_PPSCR_STATUS_INTERVAL_MASK (0xFFFF0000U)
#define NETC_ETH_LINK_PPSCR_STATUS_INTERVAL_SHIFT (16U)
/*! STATUS_INTERVAL - PHY status read interval */
#define NETC_ETH_LINK_PPSCR_STATUS_INTERVAL(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSCR_STATUS_INTERVAL_SHIFT)) & NETC_ETH_LINK_PPSCR_STATUS_INTERVAL_MASK)
/*! @} */

/*! @name PPSCTRLR - Port PHY status control register */
/*! @{ */

#define NETC_ETH_LINK_PPSCTRLR_DEV_ADDR_MASK     (0x1FU)
#define NETC_ETH_LINK_PPSCTRLR_DEV_ADDR_SHIFT    (0U)
/*! DEV_ADDR - 5-bit MDIO device address (Clause 45) / register address (Clause 22) */
#define NETC_ETH_LINK_PPSCTRLR_DEV_ADDR(x)       (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSCTRLR_DEV_ADDR_SHIFT)) & NETC_ETH_LINK_PPSCTRLR_DEV_ADDR_MASK)

#define NETC_ETH_LINK_PPSCTRLR_PORT_ADDR_MASK    (0x3E0U)
#define NETC_ETH_LINK_PPSCTRLR_PORT_ADDR_SHIFT   (5U)
/*! PORT_ADDR - 5-bit MDIO port address (Clause 45) / PHY address (Clause 22) */
#define NETC_ETH_LINK_PPSCTRLR_PORT_ADDR(x)      (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSCTRLR_PORT_ADDR_SHIFT)) & NETC_ETH_LINK_PPSCTRLR_PORT_ADDR_MASK)
/*! @} */

/*! @name PPSDR - Port PHY status data register */
/*! @{ */

#define NETC_ETH_LINK_PPSDR_MDIO_DATA_MASK       (0xFFFFU)
#define NETC_ETH_LINK_PPSDR_MDIO_DATA_SHIFT      (0U)
/*! MDIO_DATA - 16-bit MDIO data */
#define NETC_ETH_LINK_PPSDR_MDIO_DATA(x)         (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSDR_MDIO_DATA_SHIFT)) & NETC_ETH_LINK_PPSDR_MDIO_DATA_MASK)

#define NETC_ETH_LINK_PPSDR_CURR_CNT_MASK        (0xFFFF0000U)
#define NETC_ETH_LINK_PPSDR_CURR_CNT_SHIFT       (16U)
/*! CURR_CNT - Current count */
#define NETC_ETH_LINK_PPSDR_CURR_CNT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSDR_CURR_CNT_SHIFT)) & NETC_ETH_LINK_PPSDR_CURR_CNT_MASK)
/*! @} */

/*! @name PPSRAR - Port PHY status register address register */
/*! @{ */

#define NETC_ETH_LINK_PPSRAR_REGADDR_MASK        (0xFFFFU)
#define NETC_ETH_LINK_PPSRAR_REGADDR_SHIFT       (0U)
/*! REGADDR - MDIO PHY register address. Address of the register within the Clause 45 PHY device from which data is to be read. */
#define NETC_ETH_LINK_PPSRAR_REGADDR(x)          (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSRAR_REGADDR_SHIFT)) & NETC_ETH_LINK_PPSRAR_REGADDR_MASK)
/*! @} */

/*! @name PPSER - Port PHY status event register */
/*! @{ */

#define NETC_ETH_LINK_PPSER_STATUS_EVENT_HL_MASK (0xFFFFU)
#define NETC_ETH_LINK_PPSER_STATUS_EVENT_HL_SHIFT (0U)
/*! STATUS_EVENT_HL - Status event high-to-low. Set to 1 if a 1->0 transition on a corresponding data bit has occurred. Write 1 to clear. */
#define NETC_ETH_LINK_PPSER_STATUS_EVENT_HL(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSER_STATUS_EVENT_HL_SHIFT)) & NETC_ETH_LINK_PPSER_STATUS_EVENT_HL_MASK)

#define NETC_ETH_LINK_PPSER_STATUS_EVENT_LH_MASK (0xFFFF0000U)
#define NETC_ETH_LINK_PPSER_STATUS_EVENT_LH_SHIFT (16U)
/*! STATUS_EVENT_LH - Status event low-to-high. Set to 1 if a 0->1 transition on a corresponding data bit has occurred. Write 1 to clear. */
#define NETC_ETH_LINK_PPSER_STATUS_EVENT_LH(x)   (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSER_STATUS_EVENT_LH_SHIFT)) & NETC_ETH_LINK_PPSER_STATUS_EVENT_LH_MASK)
/*! @} */

/*! @name PPSMR - Port PHY status mask register */
/*! @{ */

#define NETC_ETH_LINK_PPSMR_STATUS_MASK_HL_MASK  (0xFFFFU)
#define NETC_ETH_LINK_PPSMR_STATUS_MASK_HL_SHIFT (0U)
/*! STATUS_MASK_HL - Status high-to-low mask. If set to 1, assert an interrupt if the corresponding event bit is set. */
#define NETC_ETH_LINK_PPSMR_STATUS_MASK_HL(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSMR_STATUS_MASK_HL_SHIFT)) & NETC_ETH_LINK_PPSMR_STATUS_MASK_HL_MASK)

#define NETC_ETH_LINK_PPSMR_STATUS_MASK_LH_MASK  (0xFFFF0000U)
#define NETC_ETH_LINK_PPSMR_STATUS_MASK_LH_SHIFT (16U)
/*! STATUS_MASK_LH - Status mask low-to-high. If set to 1, assert an interrupt if the corresponding event bit is set. */
#define NETC_ETH_LINK_PPSMR_STATUS_MASK_LH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_ETH_LINK_PPSMR_STATUS_MASK_LH_SHIFT)) & NETC_ETH_LINK_PPSMR_STATUS_MASK_LH_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group NETC_ETH_LINK_Register_Masks */


/* NETC_ETH_LINK - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ENETC0_ETH_MAC_PORT base address */
  #define ENETC0_ETH_MAC_PORT_BASE                 (0x70B15000u)
  /** Peripheral ENETC0_ETH_MAC_PORT base address */
  #define ENETC0_ETH_MAC_PORT_BASE_NS              (0x60B15000u)
  /** Peripheral ENETC0_ETH_MAC_PORT base pointer */
  #define ENETC0_ETH_MAC_PORT                      ((NETC_ETH_LINK_Type *)ENETC0_ETH_MAC_PORT_BASE)
  /** Peripheral ENETC0_ETH_MAC_PORT base pointer */
  #define ENETC0_ETH_MAC_PORT_NS                   ((NETC_ETH_LINK_Type *)ENETC0_ETH_MAC_PORT_BASE_NS)
  /** Peripheral SW0_ETH_MAC_PORT0 base address */
  #define SW0_ETH_MAC_PORT0_BASE                   (0x70A05000u)
  /** Peripheral SW0_ETH_MAC_PORT0 base address */
  #define SW0_ETH_MAC_PORT0_BASE_NS                (0x60A05000u)
  /** Peripheral SW0_ETH_MAC_PORT0 base pointer */
  #define SW0_ETH_MAC_PORT0                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT0_BASE)
  /** Peripheral SW0_ETH_MAC_PORT0 base pointer */
  #define SW0_ETH_MAC_PORT0_NS                     ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT0_BASE_NS)
  /** Peripheral SW0_ETH_MAC_PORT1 base address */
  #define SW0_ETH_MAC_PORT1_BASE                   (0x70A09000u)
  /** Peripheral SW0_ETH_MAC_PORT1 base address */
  #define SW0_ETH_MAC_PORT1_BASE_NS                (0x60A09000u)
  /** Peripheral SW0_ETH_MAC_PORT1 base pointer */
  #define SW0_ETH_MAC_PORT1                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT1_BASE)
  /** Peripheral SW0_ETH_MAC_PORT1 base pointer */
  #define SW0_ETH_MAC_PORT1_NS                     ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT1_BASE_NS)
  /** Peripheral SW0_ETH_MAC_PORT2 base address */
  #define SW0_ETH_MAC_PORT2_BASE                   (0x70A0D000u)
  /** Peripheral SW0_ETH_MAC_PORT2 base address */
  #define SW0_ETH_MAC_PORT2_BASE_NS                (0x60A0D000u)
  /** Peripheral SW0_ETH_MAC_PORT2 base pointer */
  #define SW0_ETH_MAC_PORT2                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT2_BASE)
  /** Peripheral SW0_ETH_MAC_PORT2 base pointer */
  #define SW0_ETH_MAC_PORT2_NS                     ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT2_BASE_NS)
  /** Peripheral SW0_ETH_MAC_PORT3 base address */
  #define SW0_ETH_MAC_PORT3_BASE                   (0x70A11000u)
  /** Peripheral SW0_ETH_MAC_PORT3 base address */
  #define SW0_ETH_MAC_PORT3_BASE_NS                (0x60A11000u)
  /** Peripheral SW0_ETH_MAC_PORT3 base pointer */
  #define SW0_ETH_MAC_PORT3                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT3_BASE)
  /** Peripheral SW0_ETH_MAC_PORT3 base pointer */
  #define SW0_ETH_MAC_PORT3_NS                     ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT3_BASE_NS)
  /** Array initializer of NETC_ETH_LINK peripheral base addresses */
  #define NETC_ETH_LINK_BASE_ADDRS                 { ENETC0_ETH_MAC_PORT_BASE, SW0_ETH_MAC_PORT0_BASE, SW0_ETH_MAC_PORT1_BASE, SW0_ETH_MAC_PORT2_BASE, SW0_ETH_MAC_PORT3_BASE }
  /** Array initializer of NETC_ETH_LINK peripheral base pointers */
  #define NETC_ETH_LINK_BASE_PTRS                  { ENETC0_ETH_MAC_PORT, SW0_ETH_MAC_PORT0, SW0_ETH_MAC_PORT1, SW0_ETH_MAC_PORT2, SW0_ETH_MAC_PORT3 }
  /** Array initializer of NETC_ETH_LINK peripheral base addresses */
  #define NETC_ETH_LINK_BASE_ADDRS_NS              { ENETC0_ETH_MAC_PORT_BASE_NS, SW0_ETH_MAC_PORT0_BASE_NS, SW0_ETH_MAC_PORT1_BASE_NS, SW0_ETH_MAC_PORT2_BASE_NS, SW0_ETH_MAC_PORT3_BASE_NS }
  /** Array initializer of NETC_ETH_LINK peripheral base pointers */
  #define NETC_ETH_LINK_BASE_PTRS_NS               { ENETC0_ETH_MAC_PORT_NS, SW0_ETH_MAC_PORT0_NS, SW0_ETH_MAC_PORT1_NS, SW0_ETH_MAC_PORT2_NS, SW0_ETH_MAC_PORT3_NS }
#else
  /** Peripheral ENETC0_ETH_MAC_PORT base address */
  #define ENETC0_ETH_MAC_PORT_BASE                 (0x60B15000u)
  /** Peripheral ENETC0_ETH_MAC_PORT base pointer */
  #define ENETC0_ETH_MAC_PORT                      ((NETC_ETH_LINK_Type *)ENETC0_ETH_MAC_PORT_BASE)
  /** Peripheral SW0_ETH_MAC_PORT0 base address */
  #define SW0_ETH_MAC_PORT0_BASE                   (0x60A05000u)
  /** Peripheral SW0_ETH_MAC_PORT0 base pointer */
  #define SW0_ETH_MAC_PORT0                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT0_BASE)
  /** Peripheral SW0_ETH_MAC_PORT1 base address */
  #define SW0_ETH_MAC_PORT1_BASE                   (0x60A09000u)
  /** Peripheral SW0_ETH_MAC_PORT1 base pointer */
  #define SW0_ETH_MAC_PORT1                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT1_BASE)
  /** Peripheral SW0_ETH_MAC_PORT2 base address */
  #define SW0_ETH_MAC_PORT2_BASE                   (0x60A0D000u)
  /** Peripheral SW0_ETH_MAC_PORT2 base pointer */
  #define SW0_ETH_MAC_PORT2                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT2_BASE)
  /** Peripheral SW0_ETH_MAC_PORT3 base address */
  #define SW0_ETH_MAC_PORT3_BASE                   (0x60A11000u)
  /** Peripheral SW0_ETH_MAC_PORT3 base pointer */
  #define SW0_ETH_MAC_PORT3                        ((NETC_ETH_LINK_Type *)SW0_ETH_MAC_PORT3_BASE)
  /** Array initializer of NETC_ETH_LINK peripheral base addresses */
  #define NETC_ETH_LINK_BASE_ADDRS                 { ENETC0_ETH_MAC_PORT_BASE, SW0_ETH_MAC_PORT0_BASE, SW0_ETH_MAC_PORT1_BASE, SW0_ETH_MAC_PORT2_BASE, SW0_ETH_MAC_PORT3_BASE }
  /** Array initializer of NETC_ETH_LINK peripheral base pointers */
  #define NETC_ETH_LINK_BASE_PTRS                  { ENETC0_ETH_MAC_PORT, SW0_ETH_MAC_PORT0, SW0_ETH_MAC_PORT1, SW0_ETH_MAC_PORT2, SW0_ETH_MAC_PORT3 }
#endif

/*!
 * @}
 */ /* end of group NETC_ETH_LINK_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_IERB Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_IERB_Peripheral_Access_Layer NETC_IERB Peripheral Access Layer
 * @{
 */

/** NETC_IERB - Register Layout Typedef */
typedef struct {
  __I  uint32_t CAPR0;                             /**< Capability register 0, offset: 0x0 */
  __I  uint32_t CAPR1;                             /**< Capability register 1, offset: 0x4 */
  __I  uint32_t CAPR2;                             /**< Capability register 2, offset: 0x8 */
  __I  uint32_t CAPR3;                             /**< Capability register 3, offset: 0xC */
       uint8_t RESERVED_0[16];
  __I  uint32_t CMCAPR;                            /**< Common memory capability register, offset: 0x20 */
       uint8_t RESERVED_1[12];
  __I  uint32_t IPFTMCAPR;                         /**< Ingress port filter ternary memory capability register, offset: 0x30 */
       uint8_t RESERVED_2[16];
  __I  uint32_t TGSMCAPR;                          /**< Time gate scheduling memory capability register, offset: 0x44 */
       uint8_t RESERVED_3[56];
  __IO uint32_t SMDTR;                             /**< Shared memory depletion threshold register, offset: 0x80 */
  __IO uint32_t ERSMBAR;                           /**< ENETC receive shared memory buffer allotment register, offset: 0x84 */
       uint8_t RESERVED_4[56];
  struct {                                         /* offset: 0xC0, array step: 0x8 */
    __IO uint32_t HTAHPCR;                           /**< HTA 0 HP configuration register, array offset: 0xC0, array step: 0x8 */
    __IO uint32_t HTALPCR;                           /**< HTA 0 LP configuration register, array offset: 0xC4, array step: 0x8 */
  } HTA_NUM[1];
       uint8_t RESERVED_5[56];
  __IO uint32_t HBTMAR;                            /**< Hash bucket table memory allocation register, offset: 0x100 */
  __IO uint32_t HBTCR;                             /**< Hash bucket table configuration register, offset: 0x104 */
  __I  uint32_t GHTEMCAPR;                         /**< Guaranteed hash table entry memory capability register, offset: 0x108 */
       uint8_t RESERVED_6[100];
  __IO uint32_t NETCFLRCR;                         /**< NETC FLR configuration register, offset: 0x170 */
       uint8_t RESERVED_7[4];
  __IO uint32_t NETCCLKFR;                         /**< NETC clock period fractional register, offset: 0x178 */
  __IO uint32_t NETCCLKCR;                         /**< NETC clock configuration register, offset: 0x17C */
  __IO uint32_t SBCR;                              /**< System bus configuration register, offset: 0x180 */
  __IO uint32_t SBOTCR;                            /**< System bus outstanding transaction control register, offset: 0x184 */
       uint8_t RESERVED_8[8];
  __IO uint32_t SGLTTR;                            /**< Stream gating lag time for refresh register, offset: 0x190 */
       uint8_t RESERVED_9[108];
  struct {                                         /* offset: 0x200, array step: 0x10 */
    __I  uint32_t RBCR;                              /**< Root complex 0 binding configuration register, array offset: 0x200, array step: 0x10 */
         uint8_t RESERVED_0[4];
    __IO uint32_t RCMSICAR;                          /**< Root complex 0 MSI-X cache attribute register, array offset: 0x208, array step: 0x10 */
    __IO uint32_t RCMSIAMQR;                         /**< Root complex 0 MSI access management qualifier register, array offset: 0x20C, array step: 0x10 */
  } ARRAY_NUM_RC[1];
       uint8_t RESERVED_10[240];
  __I  uint32_t EMDIOBCR;                          /**< EMDIO binding configuration register, offset: 0x300 */
       uint8_t RESERVED_11[16];
  __I  uint32_t EMDIOMCR;                          /**< EMDIO MSI-X configuration register, offset: 0x314 */
       uint8_t RESERVED_12[8];
  __IO uint32_t EMDIO_CFH_DIDVID;                  /**< EMDIO config header device ID and vendor ID register, offset: 0x320 */
  __IO uint32_t EMDIO_CFH_SIDSVID;                 /**< EMDIO config header subsystem ID and subsystem vendor ID register, offset: 0x324 */
       uint8_t RESERVED_13[32];
  __IO uint32_t EMDIOBLPR[2];                      /**< EMDIO boot loader parameter register 0..EMDIO boot loader parameter register 1, array offset: 0x348, array step: 0x4 */
  __IO uint32_t EMDIO_CFG;                         /**< EMDIO configuration register, offset: 0x350 */
       uint8_t RESERVED_14[172];
  struct {                                         /* offset: 0x400, array step: 0x50 */
    __I  uint32_t TBCR;                              /**< Timer 0 binding configuration register, array offset: 0x400, array step: 0x50 */
         uint8_t RESERVED_0[16];
    __IO uint32_t TMCR;                              /**< Timer 0 MSI-X configuration register, array offset: 0x414, array step: 0x50 */
         uint8_t RESERVED_1[8];
    __IO uint32_t T_CFH_DIDVID;                      /**< Timer 0 config header device ID and vendor ID register, array offset: 0x420, array step: 0x50 */
    __IO uint32_t T_CFH_SIDSVID;                     /**< Timer 0 config header subsystem ID and subsystem vendor ID register, array offset: 0x424, array step: 0x50 */
         uint8_t RESERVED_2[32];
    __IO uint32_t TBLPR[2];                          /**< Timer 0 boot loader parameter register 0..Timer 0 boot loader parameter register 1, array offset: 0x448, array step: index*0x50, index2*0x4 */
  } NUM_TMR_ARRAY[1];
       uint8_t RESERVED_15[2992];
  __I  uint32_t L0CAPR;                            /**< Link 0 capability register, offset: 0x1000 */
  __I  uint32_t L0MCAPR;                           /**< Link 0 MAC capability register, offset: 0x1004 */
  __I  uint32_t L0IOCAPR;                          /**< Link 0 I/O capability register, offset: 0x1008 */
       uint8_t RESERVED_16[4];
  __IO uint32_t L0BCR;                             /**< Link 0 binding configuration register, offset: 0x1010 */
  __IO uint32_t L0TXBCCTR;                         /**< Link 0 transmit byte credit comfort threshold register, offset: 0x1014 */
       uint8_t RESERVED_17[8];
  __IO uint32_t L0E0MAR0;                          /**< Link 0 end 0 MAC address register 0, offset: 0x1020 */
  __IO uint32_t L0E0MAR1;                          /**< Link 0 end 0 MAC address register 1, offset: 0x1024 */
       uint8_t RESERVED_18[24];
  __I  uint32_t L1CAPR;                            /**< Link 1 capability register, offset: 0x1040 */
  __I  uint32_t L1MCAPR;                           /**< Link 1 MAC capability register, offset: 0x1044 */
  __I  uint32_t L1IOCAPR;                          /**< Link 1 I/O capability register, offset: 0x1048 */
       uint8_t RESERVED_19[4];
  __IO uint32_t L1BCR;                             /**< Link 1 binding configuration register, offset: 0x1050 */
  __IO uint32_t L1TXBCCTR;                         /**< Link 1 transmit byte credit comfort threshold register, offset: 0x1054 */
       uint8_t RESERVED_20[8];
  __IO uint32_t L1E0MAR0;                          /**< Link 1 end 0 MAC address register 0, offset: 0x1060 */
  __IO uint32_t L1E0MAR1;                          /**< Link 1 end 0 MAC address register 1, offset: 0x1064 */
       uint8_t RESERVED_21[24];
  __I  uint32_t L2CAPR;                            /**< Link 2 capability register, offset: 0x1080 */
  __I  uint32_t L2MCAPR;                           /**< Link 2 MAC capability register, offset: 0x1084 */
  __I  uint32_t L2IOCAPR;                          /**< Link 2 I/O capability register, offset: 0x1088 */
       uint8_t RESERVED_22[4];
  __IO uint32_t L2BCR;                             /**< Link 2 binding configuration register, offset: 0x1090 */
  __IO uint32_t L2TXBCCTR;                         /**< Link 2 transmit byte credit comfort threshold register, offset: 0x1094 */
       uint8_t RESERVED_23[8];
  __IO uint32_t L2E0MAR0;                          /**< Link 2 end 0 MAC address register 0, offset: 0x10A0 */
  __IO uint32_t L2E0MAR1;                          /**< Link 2 end 0 MAC address register 1, offset: 0x10A4 */
       uint8_t RESERVED_24[24];
  __I  uint32_t L3CAPR;                            /**< Link 3 capability register, offset: 0x10C0 */
  __I  uint32_t L3MCAPR;                           /**< Link 3 MAC capability register, offset: 0x10C4 */
  __I  uint32_t L3IOCAPR;                          /**< Link 3 I/O capability register, offset: 0x10C8 */
       uint8_t RESERVED_25[4];
  __IO uint32_t L3BCR;                             /**< Link 3 binding configuration register, offset: 0x10D0 */
  __IO uint32_t L3TXBCCTR;                         /**< Link 3 transmit byte credit comfort threshold register, offset: 0x10D4 */
       uint8_t RESERVED_26[8];
  __IO uint32_t L3E0MAR0;                          /**< Link 3 end 0 MAC address register 0, offset: 0x10E0 */
  __IO uint32_t L3E0MAR1;                          /**< Link 3 end 0 MAC address register 1, offset: 0x10E4 */
       uint8_t RESERVED_27[24];
  __I  uint32_t L4CAPR;                            /**< Link 4 capability register, offset: 0x1100 */
  __I  uint32_t L4MCAPR;                           /**< Link 4 MAC capability register, offset: 0x1104 */
  __I  uint32_t L4IOCAPR;                          /**< Link 4 I/O capability register, offset: 0x1108 */
       uint8_t RESERVED_28[4];
  __IO uint32_t L4BCR;                             /**< Link 4 binding configuration register, offset: 0x1110 */
  __IO uint32_t L4TXBCCTR;                         /**< Link 4 transmit byte credit comfort threshold register, offset: 0x1114 */
       uint8_t RESERVED_29[8];
  __IO uint32_t L4E0MAR0;                          /**< Link 4 end 0 MAC address register 0, offset: 0x1120 */
  __IO uint32_t L4E0MAR1;                          /**< Link 4 end 0 MAC address register 1, offset: 0x1124 */
       uint8_t RESERVED_30[24];
  __I  uint32_t L5CAPR;                            /**< Link 5 capability register, offset: 0x1140 */
  __I  uint32_t L5MCAPR;                           /**< Link 5 MAC capability register, offset: 0x1144 */
       uint8_t RESERVED_31[8];
  __I  uint32_t L5BCR;                             /**< Link 5 binding configuration register, offset: 0x1150 */
  __IO uint32_t L5TXBCCTR;                         /**< Link 5 transmit byte credit comfort threshold register, offset: 0x1154 */
       uint8_t RESERVED_32[8];
  __IO uint32_t L5E0MAR0;                          /**< Link 5 end 0 MAC address register 0, offset: 0x1160 */
  __IO uint32_t L5E0MAR1;                          /**< Link 5 end 0 MAC address register 1, offset: 0x1164 */
  __IO uint32_t L5E1MAR0;                          /**< Link 5 end 1 MAC address register 0, offset: 0x1168 */
  __IO uint32_t L5E1MAR1;                          /**< Link 5 end 1 MAC address register 1, offset: 0x116C */
       uint8_t RESERVED_33[3728];
  struct {                                         /* offset: 0x2000, array step: 0x21C */
    __I  uint32_t SBCR;                              /**< Switch 0 binding configuration register, array offset: 0x2000, array step: 0x21C */
         uint8_t RESERVED_0[16];
    __IO uint32_t SMCR;                              /**< Switch 0 MSI-X configuration register, array offset: 0x2014, array step: 0x21C */
         uint8_t RESERVED_1[8];
    __IO uint32_t S_CFH_DIDVID;                      /**< Switch 0 config header device ID and vendor ID register, array offset: 0x2020, array step: 0x21C */
    __IO uint32_t S_CFH_SIDSVID;                     /**< Switch 0 config header subsystem ID and subsystem vendor ID register, array offset: 0x2024, array step: 0x21C */
         uint8_t RESERVED_2[16];
    __IO uint32_t SCCAR;                             /**< Switch 0 command cache attribute register, array offset: 0x2038, array step: 0x21C */
         uint8_t RESERVED_3[4];
    __IO uint32_t SAMQR;                             /**< Switch 0 access management qualifier register, array offset: 0x2040, array step: 0x21C */
         uint8_t RESERVED_4[4];
    __IO uint32_t SBLPR[2];                          /**< Switch 0 boot loader parameter register 0..Switch 0 boot loader parameter register 1, array offset: 0x2048, array step: index*0x21C, index2*0x4 */
         uint8_t RESERVED_5[16];
    __IO uint32_t SSMBAR;                            /**< Switch 0 shared memory buffer allotment register, array offset: 0x2060, array step: 0x21C */
         uint8_t RESERVED_6[28];
    __IO uint32_t SHTMAR;                            /**< Switch 0 hash table memory allotment register, array offset: 0x2080, array step: 0x21C */
    __IO uint32_t SITMAR;                            /**< Switch 0 index table memory allocation register, array offset: 0x2084, array step: 0x21C */
    __IO uint32_t SIPFTMAR;                          /**< Switch 0 ingress port filter table memory allocation register, array offset: 0x2088, array step: 0x21C */
         uint8_t RESERVED_7[20];
    __IO uint32_t SRPITMAR;                          /**< Switch 0 rate policer index table memory allocation register, array offset: 0x20A0, array step: 0x21C */
    __IO uint32_t SISCITMAR;                         /**< Switch 0 ingress stream counter index table memory allocation register, array offset: 0x20A4, array step: 0x21C */
    __IO uint32_t SISITMAR;                          /**< Switch 0 ingress stream index table memory allocation register, array offset: 0x20A8, array step: 0x21C */
    __IO uint32_t SISQGITMAR;                        /**< Switch 0 ingress sequence generation index table memory allocation register, array offset: 0x20AC, array step: 0x21C */
         uint8_t RESERVED_8[4];
    __IO uint32_t SSGIITMAR;                         /**< Switch 0 stream gate instance index table memory allocation register, array offset: 0x20B4, array step: 0x21C */
    __IO uint32_t SSGCLITMAR;                        /**< Switch 0 stream gate control list index table memory allocation register, array offset: 0x20B8, array step: 0x21C */
    __IO uint32_t SFMITMAR;                          /**< Switch 0 frame modification index table memory allocation register, array offset: 0x20BC, array step: 0x21C */
    __IO uint32_t SFMDITMAR;                         /**< Switch 0 frame modification data index table memory allocation register, array offset: 0x20C0, array step: 0x21C */
         uint8_t RESERVED_9[44];
    __IO uint32_t STGSTAR;                           /**< Switch 0 time gate scheduling table allocation register, array offset: 0x20F0, array step: 0x21C */
    __IO uint32_t STGSLR;                            /**< Switch 0 time gate scheduling lookahead register, array offset: 0x20F4, array step: 0x21C */
         uint8_t RESERVED_10[268];
    __I  uint32_t SMPCR;                             /**< Switch 0 management port configuration register, array offset: 0x2204, array step: 0x21C */
         uint8_t RESERVED_11[8];
    __IO uint32_t SVFHTDECR0;                        /**< Switch 0 VLAN Filter (hash) table default entry configuration registers 0, array offset: 0x2210, array step: 0x21C */
    __IO uint32_t SVFHTDECR1;                        /**< Switch 0 VLAN filter hash table default entry configuration registers 1, array offset: 0x2214, array step: 0x21C */
    __IO uint32_t SVFHTDECR2;                        /**< Switch 0 VLAN filter hash table default entry configuration registers 2, array offset: 0x2218, array step: 0x21C */
  } CFG_SW_INST[1];
       uint8_t RESERVED_34[3556];
  struct {                                         /* offset: 0x3000, array step: 0x100 */
    __I  uint32_t EBCR0;                             /**< ENETC 0 binding configuration register 0..ENETC 1 binding configuration register 0, array offset: 0x3000, array step: 0x100 */
    __I  uint32_t EBCR1;                             /**< ENETC 0 binding configuration register 1..ENETC 1 binding configuration register 1, array offset: 0x3004, array step: 0x100 */
    __I  uint32_t EBCR2;                             /**< ENETC 0 binding configuration register 2..ENETC 1 binding configuration register 2, array offset: 0x3008, array step: 0x100 */
         uint8_t RESERVED_0[4];
    __I  uint32_t EVBCR;                             /**< ENETC 0 VSI binding configuration register..ENETC 1 VSI binding configuration register, array offset: 0x3010, array step: 0x100 */
    __IO uint32_t EMCR;                              /**< ENETC 0 MSI-X configuration register..ENETC 1 MSI-X configuration register, array offset: 0x3014, array step: 0x100 */
         uint8_t RESERVED_1[8];
    __IO uint32_t E_CFH_DIDVID;                      /**< ENETC 0 config header device ID and vendor ID register..ENETC 1 config header device ID and vendor ID register, array offset: 0x3020, array step: 0x100 */
    __IO uint32_t E_CFH_SIDSVID;                     /**< ENETC 0 config header subsystem ID and subsystem vendor ID register..ENETC 1 config header subsystem ID and subsystem vendor ID register, array offset: 0x3024, array step: 0x100 */
    __IO uint32_t E_CFC_VFDID;                       /**< ENETC 0 config capability VF device ID register..ENETC 1 config capability VF device ID register, array offset: 0x3028, array step: 0x100 */
         uint8_t RESERVED_2[4];
    __IO uint32_t EBCAR;                             /**< ENETC 0 buffer cache attribute register 0..ENETC 1 buffer cache attribute register 0, array offset: 0x3030, array step: 0x100 */
    __IO uint32_t EMCAR;                             /**< ENETC 0 message cache attribute register..ENETC 1 message cache attribute register, array offset: 0x3034, array step: 0x100 */
    __IO uint32_t ECAR;                              /**< ENETC 0 command cache attribute register..ENETC 1 command cache attribute register, array offset: 0x3038, array step: 0x100 */
         uint8_t RESERVED_3[4];
    __IO uint32_t EAMQR;                             /**< ENETC 0 access management qualifier register..ENETC 1 access management qualifier register, array offset: 0x3040, array step: 0x100 */
         uint8_t RESERVED_4[4];
    __IO uint32_t EBLPR[2];                          /**< ENETC 0 boot loader parameter register 0..ENETC 1 boot loader parameter register 1, array offset: 0x3048, array step: index*0x100, index2*0x4 */
    __IO uint32_t ERXMBER;                           /**< ENETC 0 receive memory buffer entitlement register..ENETC 1 receive memory buffer entitlement register, array offset: 0x3050, array step: 0x100 */
    __IO uint32_t ERXMBLR;                           /**< ENETC 0 receive memory buffer limit register..ENETC 1 receive memory buffer limit register, array offset: 0x3054, array step: 0x100 */
         uint8_t RESERVED_5[24];
    __IO uint32_t ETXHPTBCR;                         /**< ENETC 0 transmit high priority tier byte credit register..ENETC 1 transmit high priority tier byte credit register, array offset: 0x3070, array step: 0x100 */
    __IO uint32_t ETXLPTBCR;                         /**< ENETC 0 transmit low priority tier byte credit register..ENETC 1 transmit low priority tier byte credit register, array offset: 0x3074, array step: 0x100 */
         uint8_t RESERVED_6[8];
    __IO uint32_t EHTMAR;                            /**< ENETC 0 hash table memory allotment register..ENETC 1 hash table memory allotment register, array offset: 0x3080, array step: 0x100 */
    __IO uint32_t EITMAR;                            /**< ENETC 0 index table memory allocation register..ENETC 1 index table memory allocation register, array offset: 0x3084, array step: 0x100 */
    __IO uint32_t EIPFTMAR;                          /**< ENETC 0 ingress port filter table memory allocation register..ENETC 1 ingress port filter table memory allocation register, array offset: 0x3088, array step: 0x100 */
         uint8_t RESERVED_7[20];
    __IO uint32_t ERPITMAR;                          /**< ENETC 0 rate policer index table memory allocation register..ENETC 1 rate policer index table memory allocation register, array offset: 0x30A0, array step: 0x100 */
    __IO uint32_t EISCITMAR;                         /**< ENETC 0 ingress stream counter index table memory allocation register..ENETC 1 ingress stream counter index table memory allocation register, array offset: 0x30A4, array step: 0x100 */
    __IO uint32_t EISITMAR;                          /**< ENETC 0 ingress stream index table memory allocation register..ENETC 1 ingress stream index table memory allocation register, array offset: 0x30A8, array step: 0x100 */
         uint8_t RESERVED_8[8];
    __IO uint32_t ESGIITMAR;                         /**< ENETC 0 stream gate instance index table memory allocation register..ENETC 1 stream gate instance index table memory allocation register, array offset: 0x30B4, array step: 0x100 */
    __IO uint32_t ESGCLITMAR;                        /**< ENETC 0 stream gate control list index table memory allocation register..ENETC 1 stream gate control list index table memory allocation register, array offset: 0x30B8, array step: 0x100 */
         uint8_t RESERVED_9[52];
    __IO uint32_t ETGSTAR;                           /**< ENETC 0 time gate scheduling table allocation register..ENETC 1 time gate scheduling table allocation register, array offset: 0x30F0, array step: 0x100 */
    __IO uint32_t ETGSLR;                            /**< ENETC 0 time gate scheduling lookahead register..ENETC 1 time gate scheduling lookahead register, array offset: 0x30F4, array step: 0x100 */
         uint8_t RESERVED_10[8];
  } CFG_ENETC_INST[2];
       uint8_t RESERVED_35[3584];
  struct {                                         /* offset: 0x4000, array step: 0x18 */
    __IO uint32_t VAMQR;                             /**< VSI 0 access management qualifier register, array offset: 0x4000, array step: 0x18 */
         uint8_t RESERVED_0[4];
    __IO uint32_t VBLPR[2];                          /**< VSI 0 boot loader parameter register 0..VSI 0 boot loader parameter register 1, array offset: 0x4008, array step: index*0x18, index2*0x4 */
    __IO uint32_t VPMAR0;                            /**< VSI 0 primary MAC address register 0, array offset: 0x4010, array step: 0x18 */
    __IO uint32_t VPMAR1;                            /**< VSI 0 primary MAC address register 1, array offset: 0x4014, array step: 0x18 */
  } CFG_VSI_INST[1];
} NETC_IERB_Type;

/* ----------------------------------------------------------------------------
   -- NETC_IERB Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_IERB_Register_Masks NETC_IERB Register Masks
 * @{
 */

/*! @name CAPR0 - Capability register 0 */
/*! @{ */

#define NETC_IERB_CAPR0_NUM_RC_MASK              (0xFU)
#define NETC_IERB_CAPR0_NUM_RC_SHIFT             (0U)
#define NETC_IERB_CAPR0_NUM_RC(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR0_NUM_RC_SHIFT)) & NETC_IERB_CAPR0_NUM_RC_MASK)

#define NETC_IERB_CAPR0_NUM_EMDIO_MASK           (0x10U)
#define NETC_IERB_CAPR0_NUM_EMDIO_SHIFT          (4U)
#define NETC_IERB_CAPR0_NUM_EMDIO(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR0_NUM_EMDIO_SHIFT)) & NETC_IERB_CAPR0_NUM_EMDIO_MASK)

#define NETC_IERB_CAPR0_NUM_TMR_MASK             (0xC0U)
#define NETC_IERB_CAPR0_NUM_TMR_SHIFT            (6U)
#define NETC_IERB_CAPR0_NUM_TMR(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR0_NUM_TMR_SHIFT)) & NETC_IERB_CAPR0_NUM_TMR_MASK)

#define NETC_IERB_CAPR0_NUM_LINKS_MASK           (0x1F00U)
#define NETC_IERB_CAPR0_NUM_LINKS_SHIFT          (8U)
#define NETC_IERB_CAPR0_NUM_LINKS(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR0_NUM_LINKS_SHIFT)) & NETC_IERB_CAPR0_NUM_LINKS_MASK)

#define NETC_IERB_CAPR0_NUM_SW_MASK              (0x30000U)
#define NETC_IERB_CAPR0_NUM_SW_SHIFT             (16U)
#define NETC_IERB_CAPR0_NUM_SW(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR0_NUM_SW_SHIFT)) & NETC_IERB_CAPR0_NUM_SW_MASK)

#define NETC_IERB_CAPR0_NUM_ENETC_MASK           (0xF80000U)
#define NETC_IERB_CAPR0_NUM_ENETC_SHIFT          (19U)
#define NETC_IERB_CAPR0_NUM_ENETC(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR0_NUM_ENETC_SHIFT)) & NETC_IERB_CAPR0_NUM_ENETC_MASK)

#define NETC_IERB_CAPR0_NUM_VSI_MASK             (0x7F000000U)
#define NETC_IERB_CAPR0_NUM_VSI_SHIFT            (24U)
#define NETC_IERB_CAPR0_NUM_VSI(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR0_NUM_VSI_SHIFT)) & NETC_IERB_CAPR0_NUM_VSI_MASK)
/*! @} */

/*! @name CAPR1 - Capability register 1 */
/*! @{ */

#define NETC_IERB_CAPR1_NUM_RX_BDR_MASK          (0x3FFU)
#define NETC_IERB_CAPR1_NUM_RX_BDR_SHIFT         (0U)
#define NETC_IERB_CAPR1_NUM_RX_BDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR1_NUM_RX_BDR_SHIFT)) & NETC_IERB_CAPR1_NUM_RX_BDR_MASK)

#define NETC_IERB_CAPR1_NUM_TX_BDR_MASK          (0x3FF0000U)
#define NETC_IERB_CAPR1_NUM_TX_BDR_SHIFT         (16U)
#define NETC_IERB_CAPR1_NUM_TX_BDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR1_NUM_TX_BDR_SHIFT)) & NETC_IERB_CAPR1_NUM_TX_BDR_MASK)
/*! @} */

/*! @name CAPR2 - Capability register 2 */
/*! @{ */

#define NETC_IERB_CAPR2_NUM_MSIX_MASK            (0x7FFU)
#define NETC_IERB_CAPR2_NUM_MSIX_SHIFT           (0U)
#define NETC_IERB_CAPR2_NUM_MSIX(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR2_NUM_MSIX_SHIFT)) & NETC_IERB_CAPR2_NUM_MSIX_MASK)
/*! @} */

/*! @name CAPR3 - Capability register 3 */
/*! @{ */

#define NETC_IERB_CAPR3_NUM_MAC_AFTE_MASK        (0xFFFU)
#define NETC_IERB_CAPR3_NUM_MAC_AFTE_SHIFT       (0U)
#define NETC_IERB_CAPR3_NUM_MAC_AFTE(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR3_NUM_MAC_AFTE_SHIFT)) & NETC_IERB_CAPR3_NUM_MAC_AFTE_MASK)

#define NETC_IERB_CAPR3_NUM_VLAN_FTE_MASK        (0xFFF0000U)
#define NETC_IERB_CAPR3_NUM_VLAN_FTE_SHIFT       (16U)
#define NETC_IERB_CAPR3_NUM_VLAN_FTE(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CAPR3_NUM_VLAN_FTE_SHIFT)) & NETC_IERB_CAPR3_NUM_VLAN_FTE_MASK)
/*! @} */

/*! @name CMCAPR - Common memory capability register */
/*! @{ */

#define NETC_IERB_CMCAPR_NUM_WORDS_MASK          (0xFFFFFFU)
#define NETC_IERB_CMCAPR_NUM_WORDS_SHIFT         (0U)
#define NETC_IERB_CMCAPR_NUM_WORDS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CMCAPR_NUM_WORDS_SHIFT)) & NETC_IERB_CMCAPR_NUM_WORDS_MASK)

#define NETC_IERB_CMCAPR_WORD_SIZE_MASK          (0x30000000U)
#define NETC_IERB_CMCAPR_WORD_SIZE_SHIFT         (28U)
#define NETC_IERB_CMCAPR_WORD_SIZE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_CMCAPR_WORD_SIZE_SHIFT)) & NETC_IERB_CMCAPR_WORD_SIZE_MASK)
/*! @} */

/*! @name IPFTMCAPR - Ingress port filter ternary memory capability register */
/*! @{ */

#define NETC_IERB_IPFTMCAPR_NUM_WORDS_MASK       (0xFFFFU)
#define NETC_IERB_IPFTMCAPR_NUM_WORDS_SHIFT      (0U)
#define NETC_IERB_IPFTMCAPR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_IPFTMCAPR_NUM_WORDS_SHIFT)) & NETC_IERB_IPFTMCAPR_NUM_WORDS_MASK)

#define NETC_IERB_IPFTMCAPR_WORD_SIZE_MASK       (0x30000000U)
#define NETC_IERB_IPFTMCAPR_WORD_SIZE_SHIFT      (28U)
#define NETC_IERB_IPFTMCAPR_WORD_SIZE(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_IPFTMCAPR_WORD_SIZE_SHIFT)) & NETC_IERB_IPFTMCAPR_WORD_SIZE_MASK)
/*! @} */

/*! @name TGSMCAPR - Time gate scheduling memory capability register */
/*! @{ */

#define NETC_IERB_TGSMCAPR_NUM_WORDS_MASK        (0xFFFFU)
#define NETC_IERB_TGSMCAPR_NUM_WORDS_SHIFT       (0U)
#define NETC_IERB_TGSMCAPR_NUM_WORDS(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_TGSMCAPR_NUM_WORDS_SHIFT)) & NETC_IERB_TGSMCAPR_NUM_WORDS_MASK)
/*! @} */

/*! @name SMDTR - Shared memory depletion threshold register */
/*! @{ */

#define NETC_IERB_SMDTR_THRESH_MASK              (0xFFFFFFU)
#define NETC_IERB_SMDTR_THRESH_SHIFT             (0U)
#define NETC_IERB_SMDTR_THRESH(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SMDTR_THRESH_SHIFT)) & NETC_IERB_SMDTR_THRESH_MASK)
/*! @} */

/*! @name ERSMBAR - ENETC receive shared memory buffer allotment register */
/*! @{ */

#define NETC_IERB_ERSMBAR_THRESH_MASK            (0xFFFFFFU)
#define NETC_IERB_ERSMBAR_THRESH_SHIFT           (0U)
#define NETC_IERB_ERSMBAR_THRESH(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ERSMBAR_THRESH_SHIFT)) & NETC_IERB_ERSMBAR_THRESH_MASK)
/*! @} */

/*! @name HTAHPCR - HTA 0 HP configuration register */
/*! @{ */

#define NETC_IERB_HTAHPCR_BLIMIT_MASK            (0xFFFFU)
#define NETC_IERB_HTAHPCR_BLIMIT_SHIFT           (0U)
#define NETC_IERB_HTAHPCR_BLIMIT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HTAHPCR_BLIMIT_SHIFT)) & NETC_IERB_HTAHPCR_BLIMIT_MASK)

#define NETC_IERB_HTAHPCR_FLIMIT_MASK            (0xFF000000U)
#define NETC_IERB_HTAHPCR_FLIMIT_SHIFT           (24U)
#define NETC_IERB_HTAHPCR_FLIMIT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HTAHPCR_FLIMIT_SHIFT)) & NETC_IERB_HTAHPCR_FLIMIT_MASK)
/*! @} */

/* The count of NETC_IERB_HTAHPCR */
#define NETC_IERB_HTAHPCR_COUNT                  (1U)

/*! @name HTALPCR - HTA 0 LP configuration register */
/*! @{ */

#define NETC_IERB_HTALPCR_BLIMIT_MASK            (0xFFFFU)
#define NETC_IERB_HTALPCR_BLIMIT_SHIFT           (0U)
#define NETC_IERB_HTALPCR_BLIMIT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HTALPCR_BLIMIT_SHIFT)) & NETC_IERB_HTALPCR_BLIMIT_MASK)

#define NETC_IERB_HTALPCR_FLIMIT_MASK            (0xFF000000U)
#define NETC_IERB_HTALPCR_FLIMIT_SHIFT           (24U)
#define NETC_IERB_HTALPCR_FLIMIT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HTALPCR_FLIMIT_SHIFT)) & NETC_IERB_HTALPCR_FLIMIT_MASK)
/*! @} */

/* The count of NETC_IERB_HTALPCR */
#define NETC_IERB_HTALPCR_COUNT                  (1U)

/*! @name HBTMAR - Hash bucket table memory allocation register */
/*! @{ */

#define NETC_IERB_HBTMAR_NUM_WORDS_MASK          (0x3FFFU)
#define NETC_IERB_HBTMAR_NUM_WORDS_SHIFT         (0U)
#define NETC_IERB_HBTMAR_NUM_WORDS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HBTMAR_NUM_WORDS_SHIFT)) & NETC_IERB_HBTMAR_NUM_WORDS_MASK)

#define NETC_IERB_HBTMAR_MIN_WORDS_MASK          (0xFF0000U)
#define NETC_IERB_HBTMAR_MIN_WORDS_SHIFT         (16U)
#define NETC_IERB_HBTMAR_MIN_WORDS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HBTMAR_MIN_WORDS_SHIFT)) & NETC_IERB_HBTMAR_MIN_WORDS_MASK)

#define NETC_IERB_HBTMAR_NEPW_MASK               (0x7000000U)
#define NETC_IERB_HBTMAR_NEPW_SHIFT              (24U)
#define NETC_IERB_HBTMAR_NEPW(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HBTMAR_NEPW_SHIFT)) & NETC_IERB_HBTMAR_NEPW_MASK)

#define NETC_IERB_HBTMAR_MLOC_MASK               (0xC0000000U)
#define NETC_IERB_HBTMAR_MLOC_SHIFT              (30U)
#define NETC_IERB_HBTMAR_MLOC(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HBTMAR_MLOC_SHIFT)) & NETC_IERB_HBTMAR_MLOC_MASK)
/*! @} */

/*! @name HBTCR - Hash bucket table configuration register */
/*! @{ */

#define NETC_IERB_HBTCR_MAX_COL_MASK             (0x7U)
#define NETC_IERB_HBTCR_MAX_COL_SHIFT            (0U)
#define NETC_IERB_HBTCR_MAX_COL(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HBTCR_MAX_COL_SHIFT)) & NETC_IERB_HBTCR_MAX_COL_MASK)

#define NETC_IERB_HBTCR_MAX_VISITS_MASK          (0xF0U)
#define NETC_IERB_HBTCR_MAX_VISITS_SHIFT         (4U)
#define NETC_IERB_HBTCR_MAX_VISITS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_HBTCR_MAX_VISITS_SHIFT)) & NETC_IERB_HBTCR_MAX_VISITS_MASK)
/*! @} */

/*! @name GHTEMCAPR - Guaranteed hash table entry memory capability register */
/*! @{ */

#define NETC_IERB_GHTEMCAPR_NUM_WORDS_MASK       (0x1FFU)
#define NETC_IERB_GHTEMCAPR_NUM_WORDS_SHIFT      (0U)
#define NETC_IERB_GHTEMCAPR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_GHTEMCAPR_NUM_WORDS_SHIFT)) & NETC_IERB_GHTEMCAPR_NUM_WORDS_MASK)

#define NETC_IERB_GHTEMCAPR_MLOC_MASK            (0xC0000000U)
#define NETC_IERB_GHTEMCAPR_MLOC_SHIFT           (30U)
#define NETC_IERB_GHTEMCAPR_MLOC(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_GHTEMCAPR_MLOC_SHIFT)) & NETC_IERB_GHTEMCAPR_MLOC_MASK)
/*! @} */

/*! @name NETCFLRCR - NETC FLR configuration register */
/*! @{ */

#define NETC_IERB_NETCFLRCR_VALUE_MASK           (0x1FFU)
#define NETC_IERB_NETCFLRCR_VALUE_SHIFT          (0U)
#define NETC_IERB_NETCFLRCR_VALUE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_NETCFLRCR_VALUE_SHIFT)) & NETC_IERB_NETCFLRCR_VALUE_MASK)

#define NETC_IERB_NETCFLRCR_SCALE_MASK           (0xE00U)
#define NETC_IERB_NETCFLRCR_SCALE_SHIFT          (9U)
/*! SCALE - Scale */
#define NETC_IERB_NETCFLRCR_SCALE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_NETCFLRCR_SCALE_SHIFT)) & NETC_IERB_NETCFLRCR_SCALE_MASK)
/*! @} */

/*! @name NETCCLKFR - NETC clock period fractional register */
/*! @{ */

#define NETC_IERB_NETCCLKFR_FRAC_MASK            (0xFFFFFFFFU)
#define NETC_IERB_NETCCLKFR_FRAC_SHIFT           (0U)
#define NETC_IERB_NETCCLKFR_FRAC(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_NETCCLKFR_FRAC_SHIFT)) & NETC_IERB_NETCCLKFR_FRAC_MASK)
/*! @} */

/*! @name NETCCLKCR - NETC clock configuration register */
/*! @{ */

#define NETC_IERB_NETCCLKCR_FREQ_MASK            (0x7FFU)
#define NETC_IERB_NETCCLKCR_FREQ_SHIFT           (0U)
/*! FREQ - Frequency */
#define NETC_IERB_NETCCLKCR_FREQ(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_NETCCLKCR_FREQ_SHIFT)) & NETC_IERB_NETCCLKCR_FREQ_MASK)

#define NETC_IERB_NETCCLKCR_PERIOD_MASK          (0x3FF0000U)
#define NETC_IERB_NETCCLKCR_PERIOD_SHIFT         (16U)
/*! PERIOD - Period */
#define NETC_IERB_NETCCLKCR_PERIOD(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_NETCCLKCR_PERIOD_SHIFT)) & NETC_IERB_NETCCLKCR_PERIOD_MASK)
/*! @} */

/*! @name SBCR - System bus configuration register */
/*! @{ */

#define NETC_IERB_SBCR_WBS_MASK                  (0x3U)
#define NETC_IERB_SBCR_WBS_SHIFT                 (0U)
#define NETC_IERB_SBCR_WBS(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SBCR_WBS_SHIFT)) & NETC_IERB_SBCR_WBS_MASK)

#define NETC_IERB_SBCR_RBS_MASK                  (0xCU)
#define NETC_IERB_SBCR_RBS_SHIFT                 (2U)
#define NETC_IERB_SBCR_RBS(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SBCR_RBS_SHIFT)) & NETC_IERB_SBCR_RBS_MASK)
/*! @} */

/*! @name SBOTCR - System bus outstanding transaction control register */
/*! @{ */

#define NETC_IERB_SBOTCR_OT_LIMIT_MASK           (0xFFFFFFFFU)
#define NETC_IERB_SBOTCR_OT_LIMIT_SHIFT          (0U)
#define NETC_IERB_SBOTCR_OT_LIMIT(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SBOTCR_OT_LIMIT_SHIFT)) & NETC_IERB_SBOTCR_OT_LIMIT_MASK)
/*! @} */

/*! @name SGLTTR - Stream gating lag time for refresh register */
/*! @{ */

#define NETC_IERB_SGLTTR_LAG_TIME_MASK           (0x1FU)
#define NETC_IERB_SGLTTR_LAG_TIME_SHIFT          (0U)
#define NETC_IERB_SGLTTR_LAG_TIME(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SGLTTR_LAG_TIME_SHIFT)) & NETC_IERB_SGLTTR_LAG_TIME_MASK)
/*! @} */

/*! @name RBCR - Root complex 0 binding configuration register */
/*! @{ */

#define NETC_IERB_RBCR_TYPE_MASK                 (0x1U)
#define NETC_IERB_RBCR_TYPE_SHIFT                (0U)
#define NETC_IERB_RBCR_TYPE(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_IERB_RBCR_TYPE_SHIFT)) & NETC_IERB_RBCR_TYPE_MASK)

#define NETC_IERB_RBCR_PORT_MASK                 (0xF0U)
#define NETC_IERB_RBCR_PORT_SHIFT                (4U)
#define NETC_IERB_RBCR_PORT(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_IERB_RBCR_PORT_SHIFT)) & NETC_IERB_RBCR_PORT_MASK)
/*! @} */

/* The count of NETC_IERB_RBCR */
#define NETC_IERB_RBCR_COUNT                     (1U)

/*! @name RCMSICAR - Root complex 0 MSI-X cache attribute register */
/*! @{ */

#define NETC_IERB_RCMSICAR_MSI_WRCACHE_MASK      (0xFU)
#define NETC_IERB_RCMSICAR_MSI_WRCACHE_SHIFT     (0U)
#define NETC_IERB_RCMSICAR_MSI_WRCACHE(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_RCMSICAR_MSI_WRCACHE_SHIFT)) & NETC_IERB_RCMSICAR_MSI_WRCACHE_MASK)

#define NETC_IERB_RCMSICAR_MSI_WRDOMAIN_MASK     (0x30U)
#define NETC_IERB_RCMSICAR_MSI_WRDOMAIN_SHIFT    (4U)
#define NETC_IERB_RCMSICAR_MSI_WRDOMAIN(x)       (((uint32_t)(((uint32_t)(x)) << NETC_IERB_RCMSICAR_MSI_WRDOMAIN_SHIFT)) & NETC_IERB_RCMSICAR_MSI_WRDOMAIN_MASK)

#define NETC_IERB_RCMSICAR_MSI_WRSNP_MASK        (0x40U)
#define NETC_IERB_RCMSICAR_MSI_WRSNP_SHIFT       (6U)
#define NETC_IERB_RCMSICAR_MSI_WRSNP(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_RCMSICAR_MSI_WRSNP_SHIFT)) & NETC_IERB_RCMSICAR_MSI_WRSNP_MASK)
/*! @} */

/* The count of NETC_IERB_RCMSICAR */
#define NETC_IERB_RCMSICAR_COUNT                 (1U)

/*! @name RCMSIAMQR - Root complex 0 MSI access management qualifier register */
/*! @{ */

#define NETC_IERB_RCMSIAMQR_AWQOS_MASK           (0xF00000U)
#define NETC_IERB_RCMSIAMQR_AWQOS_SHIFT          (20U)
#define NETC_IERB_RCMSIAMQR_AWQOS(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_RCMSIAMQR_AWQOS_SHIFT)) & NETC_IERB_RCMSIAMQR_AWQOS_MASK)

#define NETC_IERB_RCMSIAMQR_BMT_MASK             (0x80000000U)
#define NETC_IERB_RCMSIAMQR_BMT_SHIFT            (31U)
#define NETC_IERB_RCMSIAMQR_BMT(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_RCMSIAMQR_BMT_SHIFT)) & NETC_IERB_RCMSIAMQR_BMT_MASK)
/*! @} */

/* The count of NETC_IERB_RCMSIAMQR */
#define NETC_IERB_RCMSIAMQR_COUNT                (1U)

/*! @name EMDIOBCR - EMDIO binding configuration register */
/*! @{ */

#define NETC_IERB_EMDIOBCR_RC_INST_MASK          (0xFU)
#define NETC_IERB_EMDIOBCR_RC_INST_SHIFT         (0U)
#define NETC_IERB_EMDIOBCR_RC_INST(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIOBCR_RC_INST_SHIFT)) & NETC_IERB_EMDIOBCR_RC_INST_MASK)

#define NETC_IERB_EMDIOBCR_FN_MASK               (0xF00U)
#define NETC_IERB_EMDIOBCR_FN_SHIFT              (8U)
#define NETC_IERB_EMDIOBCR_FN(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIOBCR_FN_SHIFT)) & NETC_IERB_EMDIOBCR_FN_MASK)

#define NETC_IERB_EMDIOBCR_VALID_MASK            (0x80000000U)
#define NETC_IERB_EMDIOBCR_VALID_SHIFT           (31U)
#define NETC_IERB_EMDIOBCR_VALID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIOBCR_VALID_SHIFT)) & NETC_IERB_EMDIOBCR_VALID_MASK)
/*! @} */

/*! @name EMDIOMCR - EMDIO MSI-X configuration register */
/*! @{ */

#define NETC_IERB_EMDIOMCR_NUM_MSIX_MASK         (0x1U)
#define NETC_IERB_EMDIOMCR_NUM_MSIX_SHIFT        (0U)
#define NETC_IERB_EMDIOMCR_NUM_MSIX(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIOMCR_NUM_MSIX_SHIFT)) & NETC_IERB_EMDIOMCR_NUM_MSIX_MASK)
/*! @} */

/*! @name EMDIO_CFH_DIDVID - EMDIO config header device ID and vendor ID register */
/*! @{ */

#define NETC_IERB_EMDIO_CFH_DIDVID_VENDOR_ID_MASK (0xFFFFU)
#define NETC_IERB_EMDIO_CFH_DIDVID_VENDOR_ID_SHIFT (0U)
#define NETC_IERB_EMDIO_CFH_DIDVID_VENDOR_ID(x)  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIO_CFH_DIDVID_VENDOR_ID_SHIFT)) & NETC_IERB_EMDIO_CFH_DIDVID_VENDOR_ID_MASK)

#define NETC_IERB_EMDIO_CFH_DIDVID_DEVICE_ID_MASK (0xFFFF0000U)
#define NETC_IERB_EMDIO_CFH_DIDVID_DEVICE_ID_SHIFT (16U)
#define NETC_IERB_EMDIO_CFH_DIDVID_DEVICE_ID(x)  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIO_CFH_DIDVID_DEVICE_ID_SHIFT)) & NETC_IERB_EMDIO_CFH_DIDVID_DEVICE_ID_MASK)
/*! @} */

/*! @name EMDIO_CFH_SIDSVID - EMDIO config header subsystem ID and subsystem vendor ID register */
/*! @{ */

#define NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT (0U)
#define NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT)) & NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK)

#define NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK (0xFFFF0000U)
#define NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT (16U)
#define NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT)) & NETC_IERB_EMDIO_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK)
/*! @} */

/*! @name EMDIOBLPR - EMDIO boot loader parameter register 0..EMDIO boot loader parameter register 1 */
/*! @{ */

#define NETC_IERB_EMDIOBLPR_PARAM_VAL_MASK       (0xFFFFFFFFU)
#define NETC_IERB_EMDIOBLPR_PARAM_VAL_SHIFT      (0U)
#define NETC_IERB_EMDIOBLPR_PARAM_VAL(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIOBLPR_PARAM_VAL_SHIFT)) & NETC_IERB_EMDIOBLPR_PARAM_VAL_MASK)
/*! @} */

/* The count of NETC_IERB_EMDIOBLPR */
#define NETC_IERB_EMDIOBLPR_COUNT                (2U)

/*! @name EMDIO_CFG - EMDIO configuration register */
/*! @{ */

#define NETC_IERB_EMDIO_CFG_MDIO_MODE_MASK       (0x10U)
#define NETC_IERB_EMDIO_CFG_MDIO_MODE_SHIFT      (4U)
#define NETC_IERB_EMDIO_CFG_MDIO_MODE(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIO_CFG_MDIO_MODE_SHIFT)) & NETC_IERB_EMDIO_CFG_MDIO_MODE_MASK)

#define NETC_IERB_EMDIO_CFG_MDC_MODE_MASK        (0x20U)
#define NETC_IERB_EMDIO_CFG_MDC_MODE_SHIFT       (5U)
#define NETC_IERB_EMDIO_CFG_MDC_MODE(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMDIO_CFG_MDC_MODE_SHIFT)) & NETC_IERB_EMDIO_CFG_MDC_MODE_MASK)
/*! @} */

/*! @name TBCR - Timer 0 binding configuration register */
/*! @{ */

#define NETC_IERB_TBCR_RC_INST_MASK              (0xFU)
#define NETC_IERB_TBCR_RC_INST_SHIFT             (0U)
#define NETC_IERB_TBCR_RC_INST(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_TBCR_RC_INST_SHIFT)) & NETC_IERB_TBCR_RC_INST_MASK)

#define NETC_IERB_TBCR_FN_MASK                   (0xF00U)
#define NETC_IERB_TBCR_FN_SHIFT                  (8U)
#define NETC_IERB_TBCR_FN(x)                     (((uint32_t)(((uint32_t)(x)) << NETC_IERB_TBCR_FN_SHIFT)) & NETC_IERB_TBCR_FN_MASK)

#define NETC_IERB_TBCR_VALID_MASK                (0x80000000U)
#define NETC_IERB_TBCR_VALID_SHIFT               (31U)
#define NETC_IERB_TBCR_VALID(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_TBCR_VALID_SHIFT)) & NETC_IERB_TBCR_VALID_MASK)
/*! @} */

/* The count of NETC_IERB_TBCR */
#define NETC_IERB_TBCR_COUNT                     (1U)

/*! @name TMCR - Timer 0 MSI-X configuration register */
/*! @{ */

#define NETC_IERB_TMCR_NUM_MSIX_MASK             (0x1U)
#define NETC_IERB_TMCR_NUM_MSIX_SHIFT            (0U)
#define NETC_IERB_TMCR_NUM_MSIX(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_TMCR_NUM_MSIX_SHIFT)) & NETC_IERB_TMCR_NUM_MSIX_MASK)
/*! @} */

/* The count of NETC_IERB_TMCR */
#define NETC_IERB_TMCR_COUNT                     (1U)

/*! @name T_CFH_DIDVID - Timer 0 config header device ID and vendor ID register */
/*! @{ */

#define NETC_IERB_T_CFH_DIDVID_VENDOR_ID_MASK    (0xFFFFU)
#define NETC_IERB_T_CFH_DIDVID_VENDOR_ID_SHIFT   (0U)
#define NETC_IERB_T_CFH_DIDVID_VENDOR_ID(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_T_CFH_DIDVID_VENDOR_ID_SHIFT)) & NETC_IERB_T_CFH_DIDVID_VENDOR_ID_MASK)

#define NETC_IERB_T_CFH_DIDVID_DEVICE_ID_MASK    (0xFFFF0000U)
#define NETC_IERB_T_CFH_DIDVID_DEVICE_ID_SHIFT   (16U)
#define NETC_IERB_T_CFH_DIDVID_DEVICE_ID(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_T_CFH_DIDVID_DEVICE_ID_SHIFT)) & NETC_IERB_T_CFH_DIDVID_DEVICE_ID_MASK)
/*! @} */

/* The count of NETC_IERB_T_CFH_DIDVID */
#define NETC_IERB_T_CFH_DIDVID_COUNT             (1U)

/*! @name T_CFH_SIDSVID - Timer 0 config header subsystem ID and subsystem vendor ID register */
/*! @{ */

#define NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT (0U)
#define NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT)) & NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK)

#define NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK (0xFFFF0000U)
#define NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT (16U)
#define NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT)) & NETC_IERB_T_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK)
/*! @} */

/* The count of NETC_IERB_T_CFH_SIDSVID */
#define NETC_IERB_T_CFH_SIDSVID_COUNT            (1U)

/*! @name TBLPR - Timer 0 boot loader parameter register 0..Timer 0 boot loader parameter register 1 */
/*! @{ */

#define NETC_IERB_TBLPR_PARAM_VAL_MASK           (0xFFFFFFFFU)
#define NETC_IERB_TBLPR_PARAM_VAL_SHIFT          (0U)
#define NETC_IERB_TBLPR_PARAM_VAL(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_TBLPR_PARAM_VAL_SHIFT)) & NETC_IERB_TBLPR_PARAM_VAL_MASK)
/*! @} */

/* The count of NETC_IERB_TBLPR */
#define NETC_IERB_TBLPR_COUNT                    (1U)

/* The count of NETC_IERB_TBLPR */
#define NETC_IERB_TBLPR_COUNT2                   (2U)

/*! @name L0CAPR - Link 0 capability register */
/*! @{ */

#define NETC_IERB_L0CAPR_LINK_TYPE_MASK          (0x10U)
#define NETC_IERB_L0CAPR_LINK_TYPE_SHIFT         (4U)
#define NETC_IERB_L0CAPR_LINK_TYPE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0CAPR_LINK_TYPE_SHIFT)) & NETC_IERB_L0CAPR_LINK_TYPE_MASK)

#define NETC_IERB_L0CAPR_NUM_TC_MASK             (0xF000U)
#define NETC_IERB_L0CAPR_NUM_TC_SHIFT            (12U)
#define NETC_IERB_L0CAPR_NUM_TC(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0CAPR_NUM_TC_SHIFT)) & NETC_IERB_L0CAPR_NUM_TC_MASK)

#define NETC_IERB_L0CAPR_NUM_Q_MASK              (0xF0000U)
#define NETC_IERB_L0CAPR_NUM_Q_SHIFT             (16U)
/*! NUM_Q - Number of Egress Traffic Management (ETM) class queues supported */
#define NETC_IERB_L0CAPR_NUM_Q(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0CAPR_NUM_Q_SHIFT)) & NETC_IERB_L0CAPR_NUM_Q_MASK)

#define NETC_IERB_L0CAPR_NUM_CG_MASK             (0xF000000U)
#define NETC_IERB_L0CAPR_NUM_CG_SHIFT            (24U)
/*! NUM_CG - Number of congestion groups supported */
#define NETC_IERB_L0CAPR_NUM_CG(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0CAPR_NUM_CG_SHIFT)) & NETC_IERB_L0CAPR_NUM_CG_MASK)

#define NETC_IERB_L0CAPR_TGS_MASK                (0x10000000U)
#define NETC_IERB_L0CAPR_TGS_SHIFT               (28U)
/*! TGS - Time Gate Scheduling */
#define NETC_IERB_L0CAPR_TGS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0CAPR_TGS_SHIFT)) & NETC_IERB_L0CAPR_TGS_MASK)

#define NETC_IERB_L0CAPR_CBS_MASK                (0x20000000U)
#define NETC_IERB_L0CAPR_CBS_SHIFT               (29U)
/*! CBS - Credit Based Shaping */
#define NETC_IERB_L0CAPR_CBS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0CAPR_CBS_SHIFT)) & NETC_IERB_L0CAPR_CBS_MASK)
/*! @} */

/*! @name L0MCAPR - Link 0 MAC capability register */
/*! @{ */

#define NETC_IERB_L0MCAPR_MAC_VAR_MASK           (0x7U)
#define NETC_IERB_L0MCAPR_MAC_VAR_SHIFT          (0U)
/*! MAC_VAR - MAC Variant */
#define NETC_IERB_L0MCAPR_MAC_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0MCAPR_MAC_VAR_SHIFT)) & NETC_IERB_L0MCAPR_MAC_VAR_MASK)

#define NETC_IERB_L0MCAPR_EFPAD_MASK             (0x30U)
#define NETC_IERB_L0MCAPR_EFPAD_SHIFT            (4U)
/*! EFPAD - Egress frame padding capability */
#define NETC_IERB_L0MCAPR_EFPAD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0MCAPR_EFPAD_SHIFT)) & NETC_IERB_L0MCAPR_EFPAD_MASK)

#define NETC_IERB_L0MCAPR_PIPG_MASK              (0x40U)
#define NETC_IERB_L0MCAPR_PIPG_SHIFT             (6U)
/*! PIPG - Configurable preamble/IPG capability */
#define NETC_IERB_L0MCAPR_PIPG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0MCAPR_PIPG_SHIFT)) & NETC_IERB_L0MCAPR_PIPG_MASK)

#define NETC_IERB_L0MCAPR_HD_MASK                (0x100U)
#define NETC_IERB_L0MCAPR_HD_SHIFT               (8U)
/*! HD - Half Duplex capability */
#define NETC_IERB_L0MCAPR_HD(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0MCAPR_HD_SHIFT)) & NETC_IERB_L0MCAPR_HD_MASK)

#define NETC_IERB_L0MCAPR_FP_MASK                (0x600U)
#define NETC_IERB_L0MCAPR_FP_SHIFT               (9U)
/*! FP - Indicates if frame preemption is supported */
#define NETC_IERB_L0MCAPR_FP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0MCAPR_FP_SHIFT)) & NETC_IERB_L0MCAPR_FP_MASK)

#define NETC_IERB_L0MCAPR_MIN_MPDU_MASK          (0x1000U)
#define NETC_IERB_L0MCAPR_MIN_MPDU_SHIFT         (12U)
/*! MIN_MPDU - Minimum MAC Protocol Data Unit (PDU) size check */
#define NETC_IERB_L0MCAPR_MIN_MPDU(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0MCAPR_MIN_MPDU_SHIFT)) & NETC_IERB_L0MCAPR_MIN_MPDU_MASK)

#define NETC_IERB_L0MCAPR_MII_PROT_MASK          (0xF000000U)
#define NETC_IERB_L0MCAPR_MII_PROT_SHIFT         (24U)
/*! MII_PROT - Indicates the MII protocol supported */
#define NETC_IERB_L0MCAPR_MII_PROT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0MCAPR_MII_PROT_SHIFT)) & NETC_IERB_L0MCAPR_MII_PROT_MASK)
/*! @} */

/*! @name L0IOCAPR - Link 0 I/O capability register */
/*! @{ */

#define NETC_IERB_L0IOCAPR_PCS_PROT_MASK         (0xFFFFU)
#define NETC_IERB_L0IOCAPR_PCS_PROT_SHIFT        (0U)
/*! PCS_PROT - PCS protocols supported */
#define NETC_IERB_L0IOCAPR_PCS_PROT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0IOCAPR_PCS_PROT_SHIFT)) & NETC_IERB_L0IOCAPR_PCS_PROT_MASK)

#define NETC_IERB_L0IOCAPR_IO_VAR_MASK           (0xF000000U)
#define NETC_IERB_L0IOCAPR_IO_VAR_SHIFT          (24U)
/*! IO_VAR - IO Variants supported */
#define NETC_IERB_L0IOCAPR_IO_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0IOCAPR_IO_VAR_SHIFT)) & NETC_IERB_L0IOCAPR_IO_VAR_MASK)

#define NETC_IERB_L0IOCAPR_EMDIO_MASK            (0x10000000U)
#define NETC_IERB_L0IOCAPR_EMDIO_SHIFT           (28U)
/*! EMDIO - External MDIO supported. */
#define NETC_IERB_L0IOCAPR_EMDIO(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0IOCAPR_EMDIO_SHIFT)) & NETC_IERB_L0IOCAPR_EMDIO_MASK)

#define NETC_IERB_L0IOCAPR_REVMII_RATE_MASK      (0x40000000U)
#define NETC_IERB_L0IOCAPR_REVMII_RATE_SHIFT     (30U)
/*! REVMII_RATE - RevMII MII rate */
#define NETC_IERB_L0IOCAPR_REVMII_RATE(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0IOCAPR_REVMII_RATE_SHIFT)) & NETC_IERB_L0IOCAPR_REVMII_RATE_MASK)

#define NETC_IERB_L0IOCAPR_REVMII_MASK           (0x80000000U)
#define NETC_IERB_L0IOCAPR_REVMII_SHIFT          (31U)
/*! REVMII - Reverse Mode Device Configuration */
#define NETC_IERB_L0IOCAPR_REVMII(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0IOCAPR_REVMII_SHIFT)) & NETC_IERB_L0IOCAPR_REVMII_MASK)
/*! @} */

/*! @name L0BCR - Link 0 binding configuration register */
/*! @{ */

#define NETC_IERB_L0BCR_SW_PORT_ENETC_INST_MASK  (0x1FU)
#define NETC_IERB_L0BCR_SW_PORT_ENETC_INST_SHIFT (0U)
#define NETC_IERB_L0BCR_SW_PORT_ENETC_INST(x)    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0BCR_SW_PORT_ENETC_INST_SHIFT)) & NETC_IERB_L0BCR_SW_PORT_ENETC_INST_MASK)

#define NETC_IERB_L0BCR_NETC_FUNC_MASK           (0x40U)
#define NETC_IERB_L0BCR_NETC_FUNC_SHIFT          (6U)
#define NETC_IERB_L0BCR_NETC_FUNC(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0BCR_NETC_FUNC_SHIFT)) & NETC_IERB_L0BCR_NETC_FUNC_MASK)

#define NETC_IERB_L0BCR_MDIO_PHYAD_PRTAD_MASK    (0x1F00U)
#define NETC_IERB_L0BCR_MDIO_PHYAD_PRTAD_SHIFT   (8U)
#define NETC_IERB_L0BCR_MDIO_PHYAD_PRTAD(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0BCR_MDIO_PHYAD_PRTAD_SHIFT)) & NETC_IERB_L0BCR_MDIO_PHYAD_PRTAD_MASK)

#define NETC_IERB_L0BCR_SPL_SW_PORT_MASK         (0x1F0000U)
#define NETC_IERB_L0BCR_SPL_SW_PORT_SHIFT        (16U)
#define NETC_IERB_L0BCR_SPL_SW_PORT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0BCR_SPL_SW_PORT_SHIFT)) & NETC_IERB_L0BCR_SPL_SW_PORT_MASK)
/*! @} */

/*! @name L0TXBCCTR - Link 0 transmit byte credit comfort threshold register */
/*! @{ */

#define NETC_IERB_L0TXBCCTR_THRESH_MASK          (0xFFFFU)
#define NETC_IERB_L0TXBCCTR_THRESH_SHIFT         (0U)
#define NETC_IERB_L0TXBCCTR_THRESH(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0TXBCCTR_THRESH_SHIFT)) & NETC_IERB_L0TXBCCTR_THRESH_MASK)
/*! @} */

/*! @name L0E0MAR0 - Link 0 end 0 MAC address register 0 */
/*! @{ */

#define NETC_IERB_L0E0MAR0_MAC_ADDR_MASK         (0xFFFFFFFFU)
#define NETC_IERB_L0E0MAR0_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L0E0MAR0_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0E0MAR0_MAC_ADDR_SHIFT)) & NETC_IERB_L0E0MAR0_MAC_ADDR_MASK)
/*! @} */

/*! @name L0E0MAR1 - Link 0 end 0 MAC address register 1 */
/*! @{ */

#define NETC_IERB_L0E0MAR1_MAC_ADDR_MASK         (0xFFFFU)
#define NETC_IERB_L0E0MAR1_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L0E0MAR1_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L0E0MAR1_MAC_ADDR_SHIFT)) & NETC_IERB_L0E0MAR1_MAC_ADDR_MASK)
/*! @} */

/*! @name L1CAPR - Link 1 capability register */
/*! @{ */

#define NETC_IERB_L1CAPR_LINK_TYPE_MASK          (0x10U)
#define NETC_IERB_L1CAPR_LINK_TYPE_SHIFT         (4U)
#define NETC_IERB_L1CAPR_LINK_TYPE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1CAPR_LINK_TYPE_SHIFT)) & NETC_IERB_L1CAPR_LINK_TYPE_MASK)

#define NETC_IERB_L1CAPR_NUM_TC_MASK             (0xF000U)
#define NETC_IERB_L1CAPR_NUM_TC_SHIFT            (12U)
#define NETC_IERB_L1CAPR_NUM_TC(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1CAPR_NUM_TC_SHIFT)) & NETC_IERB_L1CAPR_NUM_TC_MASK)

#define NETC_IERB_L1CAPR_NUM_Q_MASK              (0xF0000U)
#define NETC_IERB_L1CAPR_NUM_Q_SHIFT             (16U)
/*! NUM_Q - Number of Egress Traffic Management (ETM) class queues supported */
#define NETC_IERB_L1CAPR_NUM_Q(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1CAPR_NUM_Q_SHIFT)) & NETC_IERB_L1CAPR_NUM_Q_MASK)

#define NETC_IERB_L1CAPR_NUM_CG_MASK             (0xF000000U)
#define NETC_IERB_L1CAPR_NUM_CG_SHIFT            (24U)
/*! NUM_CG - Number of congestion groups supported */
#define NETC_IERB_L1CAPR_NUM_CG(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1CAPR_NUM_CG_SHIFT)) & NETC_IERB_L1CAPR_NUM_CG_MASK)

#define NETC_IERB_L1CAPR_TGS_MASK                (0x10000000U)
#define NETC_IERB_L1CAPR_TGS_SHIFT               (28U)
/*! TGS - Time Gate Scheduling */
#define NETC_IERB_L1CAPR_TGS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1CAPR_TGS_SHIFT)) & NETC_IERB_L1CAPR_TGS_MASK)

#define NETC_IERB_L1CAPR_CBS_MASK                (0x20000000U)
#define NETC_IERB_L1CAPR_CBS_SHIFT               (29U)
/*! CBS - Credit Based Shaping */
#define NETC_IERB_L1CAPR_CBS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1CAPR_CBS_SHIFT)) & NETC_IERB_L1CAPR_CBS_MASK)
/*! @} */

/*! @name L1MCAPR - Link 1 MAC capability register */
/*! @{ */

#define NETC_IERB_L1MCAPR_MAC_VAR_MASK           (0x7U)
#define NETC_IERB_L1MCAPR_MAC_VAR_SHIFT          (0U)
/*! MAC_VAR - MAC Variant */
#define NETC_IERB_L1MCAPR_MAC_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1MCAPR_MAC_VAR_SHIFT)) & NETC_IERB_L1MCAPR_MAC_VAR_MASK)

#define NETC_IERB_L1MCAPR_EFPAD_MASK             (0x30U)
#define NETC_IERB_L1MCAPR_EFPAD_SHIFT            (4U)
/*! EFPAD - Egress frame padding capability */
#define NETC_IERB_L1MCAPR_EFPAD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1MCAPR_EFPAD_SHIFT)) & NETC_IERB_L1MCAPR_EFPAD_MASK)

#define NETC_IERB_L1MCAPR_PIPG_MASK              (0x40U)
#define NETC_IERB_L1MCAPR_PIPG_SHIFT             (6U)
/*! PIPG - Configurable preamble/IPG capability */
#define NETC_IERB_L1MCAPR_PIPG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1MCAPR_PIPG_SHIFT)) & NETC_IERB_L1MCAPR_PIPG_MASK)

#define NETC_IERB_L1MCAPR_HD_MASK                (0x100U)
#define NETC_IERB_L1MCAPR_HD_SHIFT               (8U)
/*! HD - Half Duplex capability */
#define NETC_IERB_L1MCAPR_HD(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1MCAPR_HD_SHIFT)) & NETC_IERB_L1MCAPR_HD_MASK)

#define NETC_IERB_L1MCAPR_FP_MASK                (0x600U)
#define NETC_IERB_L1MCAPR_FP_SHIFT               (9U)
/*! FP - Indicates if frame preemption is supported */
#define NETC_IERB_L1MCAPR_FP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1MCAPR_FP_SHIFT)) & NETC_IERB_L1MCAPR_FP_MASK)

#define NETC_IERB_L1MCAPR_MIN_MPDU_MASK          (0x1000U)
#define NETC_IERB_L1MCAPR_MIN_MPDU_SHIFT         (12U)
/*! MIN_MPDU - Minimum MAC Protocol Data Unit (PDU) size check */
#define NETC_IERB_L1MCAPR_MIN_MPDU(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1MCAPR_MIN_MPDU_SHIFT)) & NETC_IERB_L1MCAPR_MIN_MPDU_MASK)

#define NETC_IERB_L1MCAPR_MII_PROT_MASK          (0xF000000U)
#define NETC_IERB_L1MCAPR_MII_PROT_SHIFT         (24U)
/*! MII_PROT - Indicates the MII protocol supported */
#define NETC_IERB_L1MCAPR_MII_PROT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1MCAPR_MII_PROT_SHIFT)) & NETC_IERB_L1MCAPR_MII_PROT_MASK)
/*! @} */

/*! @name L1IOCAPR - Link 1 I/O capability register */
/*! @{ */

#define NETC_IERB_L1IOCAPR_PCS_PROT_MASK         (0xFFFFU)
#define NETC_IERB_L1IOCAPR_PCS_PROT_SHIFT        (0U)
/*! PCS_PROT - PCS protocols supported */
#define NETC_IERB_L1IOCAPR_PCS_PROT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1IOCAPR_PCS_PROT_SHIFT)) & NETC_IERB_L1IOCAPR_PCS_PROT_MASK)

#define NETC_IERB_L1IOCAPR_IO_VAR_MASK           (0xF000000U)
#define NETC_IERB_L1IOCAPR_IO_VAR_SHIFT          (24U)
/*! IO_VAR - IO Variants supported */
#define NETC_IERB_L1IOCAPR_IO_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1IOCAPR_IO_VAR_SHIFT)) & NETC_IERB_L1IOCAPR_IO_VAR_MASK)

#define NETC_IERB_L1IOCAPR_EMDIO_MASK            (0x10000000U)
#define NETC_IERB_L1IOCAPR_EMDIO_SHIFT           (28U)
/*! EMDIO - External MDIO supported. */
#define NETC_IERB_L1IOCAPR_EMDIO(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1IOCAPR_EMDIO_SHIFT)) & NETC_IERB_L1IOCAPR_EMDIO_MASK)

#define NETC_IERB_L1IOCAPR_REVMII_RATE_MASK      (0x40000000U)
#define NETC_IERB_L1IOCAPR_REVMII_RATE_SHIFT     (30U)
/*! REVMII_RATE - RevMII MII rate */
#define NETC_IERB_L1IOCAPR_REVMII_RATE(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1IOCAPR_REVMII_RATE_SHIFT)) & NETC_IERB_L1IOCAPR_REVMII_RATE_MASK)

#define NETC_IERB_L1IOCAPR_REVMII_MASK           (0x80000000U)
#define NETC_IERB_L1IOCAPR_REVMII_SHIFT          (31U)
/*! REVMII - Reverse Mode Device Configuration */
#define NETC_IERB_L1IOCAPR_REVMII(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1IOCAPR_REVMII_SHIFT)) & NETC_IERB_L1IOCAPR_REVMII_MASK)
/*! @} */

/*! @name L1BCR - Link 1 binding configuration register */
/*! @{ */

#define NETC_IERB_L1BCR_SW_PORT_ENETC_INST_MASK  (0x1FU)
#define NETC_IERB_L1BCR_SW_PORT_ENETC_INST_SHIFT (0U)
#define NETC_IERB_L1BCR_SW_PORT_ENETC_INST(x)    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1BCR_SW_PORT_ENETC_INST_SHIFT)) & NETC_IERB_L1BCR_SW_PORT_ENETC_INST_MASK)

#define NETC_IERB_L1BCR_NETC_FUNC_MASK           (0x40U)
#define NETC_IERB_L1BCR_NETC_FUNC_SHIFT          (6U)
#define NETC_IERB_L1BCR_NETC_FUNC(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1BCR_NETC_FUNC_SHIFT)) & NETC_IERB_L1BCR_NETC_FUNC_MASK)

#define NETC_IERB_L1BCR_MDIO_PHYAD_PRTAD_MASK    (0x1F00U)
#define NETC_IERB_L1BCR_MDIO_PHYAD_PRTAD_SHIFT   (8U)
#define NETC_IERB_L1BCR_MDIO_PHYAD_PRTAD(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1BCR_MDIO_PHYAD_PRTAD_SHIFT)) & NETC_IERB_L1BCR_MDIO_PHYAD_PRTAD_MASK)

#define NETC_IERB_L1BCR_SPL_SW_PORT_MASK         (0x1F0000U)
#define NETC_IERB_L1BCR_SPL_SW_PORT_SHIFT        (16U)
#define NETC_IERB_L1BCR_SPL_SW_PORT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1BCR_SPL_SW_PORT_SHIFT)) & NETC_IERB_L1BCR_SPL_SW_PORT_MASK)
/*! @} */

/*! @name L1TXBCCTR - Link 1 transmit byte credit comfort threshold register */
/*! @{ */

#define NETC_IERB_L1TXBCCTR_THRESH_MASK          (0xFFFFU)
#define NETC_IERB_L1TXBCCTR_THRESH_SHIFT         (0U)
#define NETC_IERB_L1TXBCCTR_THRESH(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1TXBCCTR_THRESH_SHIFT)) & NETC_IERB_L1TXBCCTR_THRESH_MASK)
/*! @} */

/*! @name L1E0MAR0 - Link 1 end 0 MAC address register 0 */
/*! @{ */

#define NETC_IERB_L1E0MAR0_MAC_ADDR_MASK         (0xFFFFFFFFU)
#define NETC_IERB_L1E0MAR0_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L1E0MAR0_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1E0MAR0_MAC_ADDR_SHIFT)) & NETC_IERB_L1E0MAR0_MAC_ADDR_MASK)
/*! @} */

/*! @name L1E0MAR1 - Link 1 end 0 MAC address register 1 */
/*! @{ */

#define NETC_IERB_L1E0MAR1_MAC_ADDR_MASK         (0xFFFFU)
#define NETC_IERB_L1E0MAR1_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L1E0MAR1_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L1E0MAR1_MAC_ADDR_SHIFT)) & NETC_IERB_L1E0MAR1_MAC_ADDR_MASK)
/*! @} */

/*! @name L2CAPR - Link 2 capability register */
/*! @{ */

#define NETC_IERB_L2CAPR_LINK_TYPE_MASK          (0x10U)
#define NETC_IERB_L2CAPR_LINK_TYPE_SHIFT         (4U)
#define NETC_IERB_L2CAPR_LINK_TYPE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2CAPR_LINK_TYPE_SHIFT)) & NETC_IERB_L2CAPR_LINK_TYPE_MASK)

#define NETC_IERB_L2CAPR_NUM_TC_MASK             (0xF000U)
#define NETC_IERB_L2CAPR_NUM_TC_SHIFT            (12U)
#define NETC_IERB_L2CAPR_NUM_TC(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2CAPR_NUM_TC_SHIFT)) & NETC_IERB_L2CAPR_NUM_TC_MASK)

#define NETC_IERB_L2CAPR_NUM_Q_MASK              (0xF0000U)
#define NETC_IERB_L2CAPR_NUM_Q_SHIFT             (16U)
/*! NUM_Q - Number of Egress Traffic Management (ETM) class queues supported */
#define NETC_IERB_L2CAPR_NUM_Q(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2CAPR_NUM_Q_SHIFT)) & NETC_IERB_L2CAPR_NUM_Q_MASK)

#define NETC_IERB_L2CAPR_NUM_CG_MASK             (0xF000000U)
#define NETC_IERB_L2CAPR_NUM_CG_SHIFT            (24U)
/*! NUM_CG - Number of congestion groups supported */
#define NETC_IERB_L2CAPR_NUM_CG(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2CAPR_NUM_CG_SHIFT)) & NETC_IERB_L2CAPR_NUM_CG_MASK)

#define NETC_IERB_L2CAPR_TGS_MASK                (0x10000000U)
#define NETC_IERB_L2CAPR_TGS_SHIFT               (28U)
/*! TGS - Time Gate Scheduling */
#define NETC_IERB_L2CAPR_TGS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2CAPR_TGS_SHIFT)) & NETC_IERB_L2CAPR_TGS_MASK)

#define NETC_IERB_L2CAPR_CBS_MASK                (0x20000000U)
#define NETC_IERB_L2CAPR_CBS_SHIFT               (29U)
/*! CBS - Credit Based Shaping */
#define NETC_IERB_L2CAPR_CBS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2CAPR_CBS_SHIFT)) & NETC_IERB_L2CAPR_CBS_MASK)
/*! @} */

/*! @name L2MCAPR - Link 2 MAC capability register */
/*! @{ */

#define NETC_IERB_L2MCAPR_MAC_VAR_MASK           (0x7U)
#define NETC_IERB_L2MCAPR_MAC_VAR_SHIFT          (0U)
/*! MAC_VAR - MAC Variant */
#define NETC_IERB_L2MCAPR_MAC_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2MCAPR_MAC_VAR_SHIFT)) & NETC_IERB_L2MCAPR_MAC_VAR_MASK)

#define NETC_IERB_L2MCAPR_EFPAD_MASK             (0x30U)
#define NETC_IERB_L2MCAPR_EFPAD_SHIFT            (4U)
/*! EFPAD - Egress frame padding capability */
#define NETC_IERB_L2MCAPR_EFPAD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2MCAPR_EFPAD_SHIFT)) & NETC_IERB_L2MCAPR_EFPAD_MASK)

#define NETC_IERB_L2MCAPR_PIPG_MASK              (0x40U)
#define NETC_IERB_L2MCAPR_PIPG_SHIFT             (6U)
/*! PIPG - Configurable preamble/IPG capability */
#define NETC_IERB_L2MCAPR_PIPG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2MCAPR_PIPG_SHIFT)) & NETC_IERB_L2MCAPR_PIPG_MASK)

#define NETC_IERB_L2MCAPR_HD_MASK                (0x100U)
#define NETC_IERB_L2MCAPR_HD_SHIFT               (8U)
/*! HD - Half Duplex capability */
#define NETC_IERB_L2MCAPR_HD(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2MCAPR_HD_SHIFT)) & NETC_IERB_L2MCAPR_HD_MASK)

#define NETC_IERB_L2MCAPR_FP_MASK                (0x600U)
#define NETC_IERB_L2MCAPR_FP_SHIFT               (9U)
/*! FP - Indicates if frame preemption is supported */
#define NETC_IERB_L2MCAPR_FP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2MCAPR_FP_SHIFT)) & NETC_IERB_L2MCAPR_FP_MASK)

#define NETC_IERB_L2MCAPR_MIN_MPDU_MASK          (0x1000U)
#define NETC_IERB_L2MCAPR_MIN_MPDU_SHIFT         (12U)
/*! MIN_MPDU - Minimum MAC Protocol Data Unit (PDU) size check */
#define NETC_IERB_L2MCAPR_MIN_MPDU(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2MCAPR_MIN_MPDU_SHIFT)) & NETC_IERB_L2MCAPR_MIN_MPDU_MASK)

#define NETC_IERB_L2MCAPR_MII_PROT_MASK          (0xF000000U)
#define NETC_IERB_L2MCAPR_MII_PROT_SHIFT         (24U)
/*! MII_PROT - Indicates the MII protocol supported */
#define NETC_IERB_L2MCAPR_MII_PROT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2MCAPR_MII_PROT_SHIFT)) & NETC_IERB_L2MCAPR_MII_PROT_MASK)
/*! @} */

/*! @name L2IOCAPR - Link 2 I/O capability register */
/*! @{ */

#define NETC_IERB_L2IOCAPR_PCS_PROT_MASK         (0xFFFFU)
#define NETC_IERB_L2IOCAPR_PCS_PROT_SHIFT        (0U)
/*! PCS_PROT - PCS protocols supported */
#define NETC_IERB_L2IOCAPR_PCS_PROT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2IOCAPR_PCS_PROT_SHIFT)) & NETC_IERB_L2IOCAPR_PCS_PROT_MASK)

#define NETC_IERB_L2IOCAPR_IO_VAR_MASK           (0xF000000U)
#define NETC_IERB_L2IOCAPR_IO_VAR_SHIFT          (24U)
/*! IO_VAR - IO Variants supported */
#define NETC_IERB_L2IOCAPR_IO_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2IOCAPR_IO_VAR_SHIFT)) & NETC_IERB_L2IOCAPR_IO_VAR_MASK)

#define NETC_IERB_L2IOCAPR_EMDIO_MASK            (0x10000000U)
#define NETC_IERB_L2IOCAPR_EMDIO_SHIFT           (28U)
/*! EMDIO - External MDIO supported. */
#define NETC_IERB_L2IOCAPR_EMDIO(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2IOCAPR_EMDIO_SHIFT)) & NETC_IERB_L2IOCAPR_EMDIO_MASK)

#define NETC_IERB_L2IOCAPR_REVMII_RATE_MASK      (0x40000000U)
#define NETC_IERB_L2IOCAPR_REVMII_RATE_SHIFT     (30U)
/*! REVMII_RATE - RevMII MII rate */
#define NETC_IERB_L2IOCAPR_REVMII_RATE(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2IOCAPR_REVMII_RATE_SHIFT)) & NETC_IERB_L2IOCAPR_REVMII_RATE_MASK)

#define NETC_IERB_L2IOCAPR_REVMII_MASK           (0x80000000U)
#define NETC_IERB_L2IOCAPR_REVMII_SHIFT          (31U)
/*! REVMII - Reverse Mode Device Configuration */
#define NETC_IERB_L2IOCAPR_REVMII(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2IOCAPR_REVMII_SHIFT)) & NETC_IERB_L2IOCAPR_REVMII_MASK)
/*! @} */

/*! @name L2BCR - Link 2 binding configuration register */
/*! @{ */

#define NETC_IERB_L2BCR_SW_PORT_ENETC_INST_MASK  (0x1FU)
#define NETC_IERB_L2BCR_SW_PORT_ENETC_INST_SHIFT (0U)
#define NETC_IERB_L2BCR_SW_PORT_ENETC_INST(x)    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2BCR_SW_PORT_ENETC_INST_SHIFT)) & NETC_IERB_L2BCR_SW_PORT_ENETC_INST_MASK)

#define NETC_IERB_L2BCR_NETC_FUNC_MASK           (0x40U)
#define NETC_IERB_L2BCR_NETC_FUNC_SHIFT          (6U)
#define NETC_IERB_L2BCR_NETC_FUNC(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2BCR_NETC_FUNC_SHIFT)) & NETC_IERB_L2BCR_NETC_FUNC_MASK)

#define NETC_IERB_L2BCR_MDIO_PHYAD_PRTAD_MASK    (0x1F00U)
#define NETC_IERB_L2BCR_MDIO_PHYAD_PRTAD_SHIFT   (8U)
#define NETC_IERB_L2BCR_MDIO_PHYAD_PRTAD(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2BCR_MDIO_PHYAD_PRTAD_SHIFT)) & NETC_IERB_L2BCR_MDIO_PHYAD_PRTAD_MASK)

#define NETC_IERB_L2BCR_SPL_SW_PORT_MASK         (0x1F0000U)
#define NETC_IERB_L2BCR_SPL_SW_PORT_SHIFT        (16U)
#define NETC_IERB_L2BCR_SPL_SW_PORT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2BCR_SPL_SW_PORT_SHIFT)) & NETC_IERB_L2BCR_SPL_SW_PORT_MASK)
/*! @} */

/*! @name L2TXBCCTR - Link 2 transmit byte credit comfort threshold register */
/*! @{ */

#define NETC_IERB_L2TXBCCTR_THRESH_MASK          (0xFFFFU)
#define NETC_IERB_L2TXBCCTR_THRESH_SHIFT         (0U)
#define NETC_IERB_L2TXBCCTR_THRESH(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2TXBCCTR_THRESH_SHIFT)) & NETC_IERB_L2TXBCCTR_THRESH_MASK)
/*! @} */

/*! @name L2E0MAR0 - Link 2 end 0 MAC address register 0 */
/*! @{ */

#define NETC_IERB_L2E0MAR0_MAC_ADDR_MASK         (0xFFFFFFFFU)
#define NETC_IERB_L2E0MAR0_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L2E0MAR0_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2E0MAR0_MAC_ADDR_SHIFT)) & NETC_IERB_L2E0MAR0_MAC_ADDR_MASK)
/*! @} */

/*! @name L2E0MAR1 - Link 2 end 0 MAC address register 1 */
/*! @{ */

#define NETC_IERB_L2E0MAR1_MAC_ADDR_MASK         (0xFFFFU)
#define NETC_IERB_L2E0MAR1_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L2E0MAR1_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L2E0MAR1_MAC_ADDR_SHIFT)) & NETC_IERB_L2E0MAR1_MAC_ADDR_MASK)
/*! @} */

/*! @name L3CAPR - Link 3 capability register */
/*! @{ */

#define NETC_IERB_L3CAPR_LINK_TYPE_MASK          (0x10U)
#define NETC_IERB_L3CAPR_LINK_TYPE_SHIFT         (4U)
#define NETC_IERB_L3CAPR_LINK_TYPE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3CAPR_LINK_TYPE_SHIFT)) & NETC_IERB_L3CAPR_LINK_TYPE_MASK)

#define NETC_IERB_L3CAPR_NUM_TC_MASK             (0xF000U)
#define NETC_IERB_L3CAPR_NUM_TC_SHIFT            (12U)
#define NETC_IERB_L3CAPR_NUM_TC(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3CAPR_NUM_TC_SHIFT)) & NETC_IERB_L3CAPR_NUM_TC_MASK)

#define NETC_IERB_L3CAPR_NUM_Q_MASK              (0xF0000U)
#define NETC_IERB_L3CAPR_NUM_Q_SHIFT             (16U)
/*! NUM_Q - Number of Egress Traffic Management (ETM) class queues supported */
#define NETC_IERB_L3CAPR_NUM_Q(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3CAPR_NUM_Q_SHIFT)) & NETC_IERB_L3CAPR_NUM_Q_MASK)

#define NETC_IERB_L3CAPR_NUM_CG_MASK             (0xF000000U)
#define NETC_IERB_L3CAPR_NUM_CG_SHIFT            (24U)
/*! NUM_CG - Number of congestion groups supported */
#define NETC_IERB_L3CAPR_NUM_CG(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3CAPR_NUM_CG_SHIFT)) & NETC_IERB_L3CAPR_NUM_CG_MASK)

#define NETC_IERB_L3CAPR_TGS_MASK                (0x10000000U)
#define NETC_IERB_L3CAPR_TGS_SHIFT               (28U)
/*! TGS - Time Gate Scheduling */
#define NETC_IERB_L3CAPR_TGS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3CAPR_TGS_SHIFT)) & NETC_IERB_L3CAPR_TGS_MASK)

#define NETC_IERB_L3CAPR_CBS_MASK                (0x20000000U)
#define NETC_IERB_L3CAPR_CBS_SHIFT               (29U)
/*! CBS - Credit Based Shaping */
#define NETC_IERB_L3CAPR_CBS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3CAPR_CBS_SHIFT)) & NETC_IERB_L3CAPR_CBS_MASK)
/*! @} */

/*! @name L3MCAPR - Link 3 MAC capability register */
/*! @{ */

#define NETC_IERB_L3MCAPR_MAC_VAR_MASK           (0x7U)
#define NETC_IERB_L3MCAPR_MAC_VAR_SHIFT          (0U)
/*! MAC_VAR - MAC Variant */
#define NETC_IERB_L3MCAPR_MAC_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3MCAPR_MAC_VAR_SHIFT)) & NETC_IERB_L3MCAPR_MAC_VAR_MASK)

#define NETC_IERB_L3MCAPR_EFPAD_MASK             (0x30U)
#define NETC_IERB_L3MCAPR_EFPAD_SHIFT            (4U)
/*! EFPAD - Egress frame padding capability */
#define NETC_IERB_L3MCAPR_EFPAD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3MCAPR_EFPAD_SHIFT)) & NETC_IERB_L3MCAPR_EFPAD_MASK)

#define NETC_IERB_L3MCAPR_PIPG_MASK              (0x40U)
#define NETC_IERB_L3MCAPR_PIPG_SHIFT             (6U)
/*! PIPG - Configurable preamble/IPG capability */
#define NETC_IERB_L3MCAPR_PIPG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3MCAPR_PIPG_SHIFT)) & NETC_IERB_L3MCAPR_PIPG_MASK)

#define NETC_IERB_L3MCAPR_HD_MASK                (0x100U)
#define NETC_IERB_L3MCAPR_HD_SHIFT               (8U)
/*! HD - Half Duplex capability */
#define NETC_IERB_L3MCAPR_HD(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3MCAPR_HD_SHIFT)) & NETC_IERB_L3MCAPR_HD_MASK)

#define NETC_IERB_L3MCAPR_FP_MASK                (0x600U)
#define NETC_IERB_L3MCAPR_FP_SHIFT               (9U)
/*! FP - Indicates if frame preemption is supported */
#define NETC_IERB_L3MCAPR_FP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3MCAPR_FP_SHIFT)) & NETC_IERB_L3MCAPR_FP_MASK)

#define NETC_IERB_L3MCAPR_MIN_MPDU_MASK          (0x1000U)
#define NETC_IERB_L3MCAPR_MIN_MPDU_SHIFT         (12U)
/*! MIN_MPDU - Minimum MAC Protocol Data Unit (PDU) size check */
#define NETC_IERB_L3MCAPR_MIN_MPDU(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3MCAPR_MIN_MPDU_SHIFT)) & NETC_IERB_L3MCAPR_MIN_MPDU_MASK)

#define NETC_IERB_L3MCAPR_MII_PROT_MASK          (0xF000000U)
#define NETC_IERB_L3MCAPR_MII_PROT_SHIFT         (24U)
/*! MII_PROT - Indicates the MII protocol supported */
#define NETC_IERB_L3MCAPR_MII_PROT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3MCAPR_MII_PROT_SHIFT)) & NETC_IERB_L3MCAPR_MII_PROT_MASK)
/*! @} */

/*! @name L3IOCAPR - Link 3 I/O capability register */
/*! @{ */

#define NETC_IERB_L3IOCAPR_PCS_PROT_MASK         (0xFFFFU)
#define NETC_IERB_L3IOCAPR_PCS_PROT_SHIFT        (0U)
/*! PCS_PROT - PCS protocols supported */
#define NETC_IERB_L3IOCAPR_PCS_PROT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3IOCAPR_PCS_PROT_SHIFT)) & NETC_IERB_L3IOCAPR_PCS_PROT_MASK)

#define NETC_IERB_L3IOCAPR_IO_VAR_MASK           (0xF000000U)
#define NETC_IERB_L3IOCAPR_IO_VAR_SHIFT          (24U)
/*! IO_VAR - IO Variants supported */
#define NETC_IERB_L3IOCAPR_IO_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3IOCAPR_IO_VAR_SHIFT)) & NETC_IERB_L3IOCAPR_IO_VAR_MASK)

#define NETC_IERB_L3IOCAPR_EMDIO_MASK            (0x10000000U)
#define NETC_IERB_L3IOCAPR_EMDIO_SHIFT           (28U)
/*! EMDIO - External MDIO supported. */
#define NETC_IERB_L3IOCAPR_EMDIO(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3IOCAPR_EMDIO_SHIFT)) & NETC_IERB_L3IOCAPR_EMDIO_MASK)

#define NETC_IERB_L3IOCAPR_REVMII_RATE_MASK      (0x40000000U)
#define NETC_IERB_L3IOCAPR_REVMII_RATE_SHIFT     (30U)
/*! REVMII_RATE - RevMII MII rate */
#define NETC_IERB_L3IOCAPR_REVMII_RATE(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3IOCAPR_REVMII_RATE_SHIFT)) & NETC_IERB_L3IOCAPR_REVMII_RATE_MASK)

#define NETC_IERB_L3IOCAPR_REVMII_MASK           (0x80000000U)
#define NETC_IERB_L3IOCAPR_REVMII_SHIFT          (31U)
/*! REVMII - Reverse Mode Device Configuration */
#define NETC_IERB_L3IOCAPR_REVMII(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3IOCAPR_REVMII_SHIFT)) & NETC_IERB_L3IOCAPR_REVMII_MASK)
/*! @} */

/*! @name L3BCR - Link 3 binding configuration register */
/*! @{ */

#define NETC_IERB_L3BCR_SW_PORT_ENETC_INST_MASK  (0x1FU)
#define NETC_IERB_L3BCR_SW_PORT_ENETC_INST_SHIFT (0U)
#define NETC_IERB_L3BCR_SW_PORT_ENETC_INST(x)    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3BCR_SW_PORT_ENETC_INST_SHIFT)) & NETC_IERB_L3BCR_SW_PORT_ENETC_INST_MASK)

#define NETC_IERB_L3BCR_NETC_FUNC_MASK           (0x40U)
#define NETC_IERB_L3BCR_NETC_FUNC_SHIFT          (6U)
#define NETC_IERB_L3BCR_NETC_FUNC(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3BCR_NETC_FUNC_SHIFT)) & NETC_IERB_L3BCR_NETC_FUNC_MASK)

#define NETC_IERB_L3BCR_MDIO_PHYAD_PRTAD_MASK    (0x1F00U)
#define NETC_IERB_L3BCR_MDIO_PHYAD_PRTAD_SHIFT   (8U)
#define NETC_IERB_L3BCR_MDIO_PHYAD_PRTAD(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3BCR_MDIO_PHYAD_PRTAD_SHIFT)) & NETC_IERB_L3BCR_MDIO_PHYAD_PRTAD_MASK)

#define NETC_IERB_L3BCR_SPL_SW_PORT_MASK         (0x1F0000U)
#define NETC_IERB_L3BCR_SPL_SW_PORT_SHIFT        (16U)
#define NETC_IERB_L3BCR_SPL_SW_PORT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3BCR_SPL_SW_PORT_SHIFT)) & NETC_IERB_L3BCR_SPL_SW_PORT_MASK)
/*! @} */

/*! @name L3TXBCCTR - Link 3 transmit byte credit comfort threshold register */
/*! @{ */

#define NETC_IERB_L3TXBCCTR_THRESH_MASK          (0xFFFFU)
#define NETC_IERB_L3TXBCCTR_THRESH_SHIFT         (0U)
#define NETC_IERB_L3TXBCCTR_THRESH(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3TXBCCTR_THRESH_SHIFT)) & NETC_IERB_L3TXBCCTR_THRESH_MASK)
/*! @} */

/*! @name L3E0MAR0 - Link 3 end 0 MAC address register 0 */
/*! @{ */

#define NETC_IERB_L3E0MAR0_MAC_ADDR_MASK         (0xFFFFFFFFU)
#define NETC_IERB_L3E0MAR0_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L3E0MAR0_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3E0MAR0_MAC_ADDR_SHIFT)) & NETC_IERB_L3E0MAR0_MAC_ADDR_MASK)
/*! @} */

/*! @name L3E0MAR1 - Link 3 end 0 MAC address register 1 */
/*! @{ */

#define NETC_IERB_L3E0MAR1_MAC_ADDR_MASK         (0xFFFFU)
#define NETC_IERB_L3E0MAR1_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L3E0MAR1_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L3E0MAR1_MAC_ADDR_SHIFT)) & NETC_IERB_L3E0MAR1_MAC_ADDR_MASK)
/*! @} */

/*! @name L4CAPR - Link 4 capability register */
/*! @{ */

#define NETC_IERB_L4CAPR_LINK_TYPE_MASK          (0x10U)
#define NETC_IERB_L4CAPR_LINK_TYPE_SHIFT         (4U)
#define NETC_IERB_L4CAPR_LINK_TYPE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4CAPR_LINK_TYPE_SHIFT)) & NETC_IERB_L4CAPR_LINK_TYPE_MASK)

#define NETC_IERB_L4CAPR_NUM_TC_MASK             (0xF000U)
#define NETC_IERB_L4CAPR_NUM_TC_SHIFT            (12U)
#define NETC_IERB_L4CAPR_NUM_TC(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4CAPR_NUM_TC_SHIFT)) & NETC_IERB_L4CAPR_NUM_TC_MASK)

#define NETC_IERB_L4CAPR_NUM_Q_MASK              (0xF0000U)
#define NETC_IERB_L4CAPR_NUM_Q_SHIFT             (16U)
/*! NUM_Q - Number of Egress Traffic Management (ETM) class queues supported */
#define NETC_IERB_L4CAPR_NUM_Q(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4CAPR_NUM_Q_SHIFT)) & NETC_IERB_L4CAPR_NUM_Q_MASK)

#define NETC_IERB_L4CAPR_NUM_CG_MASK             (0xF000000U)
#define NETC_IERB_L4CAPR_NUM_CG_SHIFT            (24U)
/*! NUM_CG - Number of congestion groups supported */
#define NETC_IERB_L4CAPR_NUM_CG(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4CAPR_NUM_CG_SHIFT)) & NETC_IERB_L4CAPR_NUM_CG_MASK)

#define NETC_IERB_L4CAPR_TGS_MASK                (0x10000000U)
#define NETC_IERB_L4CAPR_TGS_SHIFT               (28U)
/*! TGS - Time Gate Scheduling */
#define NETC_IERB_L4CAPR_TGS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4CAPR_TGS_SHIFT)) & NETC_IERB_L4CAPR_TGS_MASK)

#define NETC_IERB_L4CAPR_CBS_MASK                (0x20000000U)
#define NETC_IERB_L4CAPR_CBS_SHIFT               (29U)
/*! CBS - Credit Based Shaping */
#define NETC_IERB_L4CAPR_CBS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4CAPR_CBS_SHIFT)) & NETC_IERB_L4CAPR_CBS_MASK)
/*! @} */

/*! @name L4MCAPR - Link 4 MAC capability register */
/*! @{ */

#define NETC_IERB_L4MCAPR_MAC_VAR_MASK           (0x7U)
#define NETC_IERB_L4MCAPR_MAC_VAR_SHIFT          (0U)
/*! MAC_VAR - MAC Variant */
#define NETC_IERB_L4MCAPR_MAC_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4MCAPR_MAC_VAR_SHIFT)) & NETC_IERB_L4MCAPR_MAC_VAR_MASK)

#define NETC_IERB_L4MCAPR_EFPAD_MASK             (0x30U)
#define NETC_IERB_L4MCAPR_EFPAD_SHIFT            (4U)
/*! EFPAD - Egress frame padding capability */
#define NETC_IERB_L4MCAPR_EFPAD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4MCAPR_EFPAD_SHIFT)) & NETC_IERB_L4MCAPR_EFPAD_MASK)

#define NETC_IERB_L4MCAPR_PIPG_MASK              (0x40U)
#define NETC_IERB_L4MCAPR_PIPG_SHIFT             (6U)
/*! PIPG - Configurable preamble/IPG capability */
#define NETC_IERB_L4MCAPR_PIPG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4MCAPR_PIPG_SHIFT)) & NETC_IERB_L4MCAPR_PIPG_MASK)

#define NETC_IERB_L4MCAPR_HD_MASK                (0x100U)
#define NETC_IERB_L4MCAPR_HD_SHIFT               (8U)
/*! HD - Half Duplex capability */
#define NETC_IERB_L4MCAPR_HD(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4MCAPR_HD_SHIFT)) & NETC_IERB_L4MCAPR_HD_MASK)

#define NETC_IERB_L4MCAPR_FP_MASK                (0x600U)
#define NETC_IERB_L4MCAPR_FP_SHIFT               (9U)
/*! FP - Indicates if frame preemption is supported */
#define NETC_IERB_L4MCAPR_FP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4MCAPR_FP_SHIFT)) & NETC_IERB_L4MCAPR_FP_MASK)

#define NETC_IERB_L4MCAPR_MIN_MPDU_MASK          (0x1000U)
#define NETC_IERB_L4MCAPR_MIN_MPDU_SHIFT         (12U)
/*! MIN_MPDU - Minimum MAC Protocol Data Unit (PDU) size check */
#define NETC_IERB_L4MCAPR_MIN_MPDU(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4MCAPR_MIN_MPDU_SHIFT)) & NETC_IERB_L4MCAPR_MIN_MPDU_MASK)

#define NETC_IERB_L4MCAPR_MII_PROT_MASK          (0xF000000U)
#define NETC_IERB_L4MCAPR_MII_PROT_SHIFT         (24U)
/*! MII_PROT - Indicates the MII protocol supported */
#define NETC_IERB_L4MCAPR_MII_PROT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4MCAPR_MII_PROT_SHIFT)) & NETC_IERB_L4MCAPR_MII_PROT_MASK)
/*! @} */

/*! @name L4IOCAPR - Link 4 I/O capability register */
/*! @{ */

#define NETC_IERB_L4IOCAPR_PCS_PROT_MASK         (0xFFFFU)
#define NETC_IERB_L4IOCAPR_PCS_PROT_SHIFT        (0U)
/*! PCS_PROT - PCS protocols supported */
#define NETC_IERB_L4IOCAPR_PCS_PROT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4IOCAPR_PCS_PROT_SHIFT)) & NETC_IERB_L4IOCAPR_PCS_PROT_MASK)

#define NETC_IERB_L4IOCAPR_IO_VAR_MASK           (0xF000000U)
#define NETC_IERB_L4IOCAPR_IO_VAR_SHIFT          (24U)
/*! IO_VAR - IO Variants supported */
#define NETC_IERB_L4IOCAPR_IO_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4IOCAPR_IO_VAR_SHIFT)) & NETC_IERB_L4IOCAPR_IO_VAR_MASK)

#define NETC_IERB_L4IOCAPR_EMDIO_MASK            (0x10000000U)
#define NETC_IERB_L4IOCAPR_EMDIO_SHIFT           (28U)
/*! EMDIO - External MDIO supported. */
#define NETC_IERB_L4IOCAPR_EMDIO(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4IOCAPR_EMDIO_SHIFT)) & NETC_IERB_L4IOCAPR_EMDIO_MASK)

#define NETC_IERB_L4IOCAPR_REVMII_RATE_MASK      (0x40000000U)
#define NETC_IERB_L4IOCAPR_REVMII_RATE_SHIFT     (30U)
/*! REVMII_RATE - RevMII MII rate */
#define NETC_IERB_L4IOCAPR_REVMII_RATE(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4IOCAPR_REVMII_RATE_SHIFT)) & NETC_IERB_L4IOCAPR_REVMII_RATE_MASK)

#define NETC_IERB_L4IOCAPR_REVMII_MASK           (0x80000000U)
#define NETC_IERB_L4IOCAPR_REVMII_SHIFT          (31U)
/*! REVMII - Reverse Mode Device Configuration */
#define NETC_IERB_L4IOCAPR_REVMII(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4IOCAPR_REVMII_SHIFT)) & NETC_IERB_L4IOCAPR_REVMII_MASK)
/*! @} */

/*! @name L4BCR - Link 4 binding configuration register */
/*! @{ */

#define NETC_IERB_L4BCR_SW_PORT_ENETC_INST_MASK  (0x1FU)
#define NETC_IERB_L4BCR_SW_PORT_ENETC_INST_SHIFT (0U)
#define NETC_IERB_L4BCR_SW_PORT_ENETC_INST(x)    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4BCR_SW_PORT_ENETC_INST_SHIFT)) & NETC_IERB_L4BCR_SW_PORT_ENETC_INST_MASK)

#define NETC_IERB_L4BCR_NETC_FUNC_MASK           (0x40U)
#define NETC_IERB_L4BCR_NETC_FUNC_SHIFT          (6U)
#define NETC_IERB_L4BCR_NETC_FUNC(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4BCR_NETC_FUNC_SHIFT)) & NETC_IERB_L4BCR_NETC_FUNC_MASK)

#define NETC_IERB_L4BCR_MDIO_PHYAD_PRTAD_MASK    (0x1F00U)
#define NETC_IERB_L4BCR_MDIO_PHYAD_PRTAD_SHIFT   (8U)
#define NETC_IERB_L4BCR_MDIO_PHYAD_PRTAD(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4BCR_MDIO_PHYAD_PRTAD_SHIFT)) & NETC_IERB_L4BCR_MDIO_PHYAD_PRTAD_MASK)

#define NETC_IERB_L4BCR_SPL_SW_PORT_MASK         (0x1F0000U)
#define NETC_IERB_L4BCR_SPL_SW_PORT_SHIFT        (16U)
#define NETC_IERB_L4BCR_SPL_SW_PORT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4BCR_SPL_SW_PORT_SHIFT)) & NETC_IERB_L4BCR_SPL_SW_PORT_MASK)
/*! @} */

/*! @name L4TXBCCTR - Link 4 transmit byte credit comfort threshold register */
/*! @{ */

#define NETC_IERB_L4TXBCCTR_THRESH_MASK          (0xFFFFU)
#define NETC_IERB_L4TXBCCTR_THRESH_SHIFT         (0U)
#define NETC_IERB_L4TXBCCTR_THRESH(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4TXBCCTR_THRESH_SHIFT)) & NETC_IERB_L4TXBCCTR_THRESH_MASK)
/*! @} */

/*! @name L4E0MAR0 - Link 4 end 0 MAC address register 0 */
/*! @{ */

#define NETC_IERB_L4E0MAR0_MAC_ADDR_MASK         (0xFFFFFFFFU)
#define NETC_IERB_L4E0MAR0_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L4E0MAR0_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4E0MAR0_MAC_ADDR_SHIFT)) & NETC_IERB_L4E0MAR0_MAC_ADDR_MASK)
/*! @} */

/*! @name L4E0MAR1 - Link 4 end 0 MAC address register 1 */
/*! @{ */

#define NETC_IERB_L4E0MAR1_MAC_ADDR_MASK         (0xFFFFU)
#define NETC_IERB_L4E0MAR1_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L4E0MAR1_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L4E0MAR1_MAC_ADDR_SHIFT)) & NETC_IERB_L4E0MAR1_MAC_ADDR_MASK)
/*! @} */

/*! @name L5CAPR - Link 5 capability register */
/*! @{ */

#define NETC_IERB_L5CAPR_LINK_TYPE_MASK          (0x10U)
#define NETC_IERB_L5CAPR_LINK_TYPE_SHIFT         (4U)
#define NETC_IERB_L5CAPR_LINK_TYPE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5CAPR_LINK_TYPE_SHIFT)) & NETC_IERB_L5CAPR_LINK_TYPE_MASK)

#define NETC_IERB_L5CAPR_NUM_TC_MASK             (0xF000U)
#define NETC_IERB_L5CAPR_NUM_TC_SHIFT            (12U)
#define NETC_IERB_L5CAPR_NUM_TC(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5CAPR_NUM_TC_SHIFT)) & NETC_IERB_L5CAPR_NUM_TC_MASK)

#define NETC_IERB_L5CAPR_NUM_Q_MASK              (0xF0000U)
#define NETC_IERB_L5CAPR_NUM_Q_SHIFT             (16U)
/*! NUM_Q - Number of Egress Traffic Management (ETM) class queues supported */
#define NETC_IERB_L5CAPR_NUM_Q(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5CAPR_NUM_Q_SHIFT)) & NETC_IERB_L5CAPR_NUM_Q_MASK)

#define NETC_IERB_L5CAPR_NUM_CG_MASK             (0xF000000U)
#define NETC_IERB_L5CAPR_NUM_CG_SHIFT            (24U)
/*! NUM_CG - Number of congestion groups supported */
#define NETC_IERB_L5CAPR_NUM_CG(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5CAPR_NUM_CG_SHIFT)) & NETC_IERB_L5CAPR_NUM_CG_MASK)

#define NETC_IERB_L5CAPR_TGS_MASK                (0x10000000U)
#define NETC_IERB_L5CAPR_TGS_SHIFT               (28U)
/*! TGS - Time Gate Scheduling */
#define NETC_IERB_L5CAPR_TGS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5CAPR_TGS_SHIFT)) & NETC_IERB_L5CAPR_TGS_MASK)

#define NETC_IERB_L5CAPR_CBS_MASK                (0x20000000U)
#define NETC_IERB_L5CAPR_CBS_SHIFT               (29U)
/*! CBS - Credit Based Shaping */
#define NETC_IERB_L5CAPR_CBS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5CAPR_CBS_SHIFT)) & NETC_IERB_L5CAPR_CBS_MASK)
/*! @} */

/*! @name L5MCAPR - Link 5 MAC capability register */
/*! @{ */

#define NETC_IERB_L5MCAPR_MAC_VAR_MASK           (0x7U)
#define NETC_IERB_L5MCAPR_MAC_VAR_SHIFT          (0U)
/*! MAC_VAR - MAC Variant */
#define NETC_IERB_L5MCAPR_MAC_VAR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5MCAPR_MAC_VAR_SHIFT)) & NETC_IERB_L5MCAPR_MAC_VAR_MASK)

#define NETC_IERB_L5MCAPR_EFPAD_MASK             (0x30U)
#define NETC_IERB_L5MCAPR_EFPAD_SHIFT            (4U)
/*! EFPAD - Egress frame padding capability */
#define NETC_IERB_L5MCAPR_EFPAD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5MCAPR_EFPAD_SHIFT)) & NETC_IERB_L5MCAPR_EFPAD_MASK)

#define NETC_IERB_L5MCAPR_PIPG_MASK              (0x40U)
#define NETC_IERB_L5MCAPR_PIPG_SHIFT             (6U)
/*! PIPG - Configurable preamble/IPG capability */
#define NETC_IERB_L5MCAPR_PIPG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5MCAPR_PIPG_SHIFT)) & NETC_IERB_L5MCAPR_PIPG_MASK)

#define NETC_IERB_L5MCAPR_HD_MASK                (0x100U)
#define NETC_IERB_L5MCAPR_HD_SHIFT               (8U)
/*! HD - Half Duplex capability */
#define NETC_IERB_L5MCAPR_HD(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5MCAPR_HD_SHIFT)) & NETC_IERB_L5MCAPR_HD_MASK)

#define NETC_IERB_L5MCAPR_FP_MASK                (0x600U)
#define NETC_IERB_L5MCAPR_FP_SHIFT               (9U)
/*! FP - Indicates if frame preemption is supported */
#define NETC_IERB_L5MCAPR_FP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5MCAPR_FP_SHIFT)) & NETC_IERB_L5MCAPR_FP_MASK)

#define NETC_IERB_L5MCAPR_MIN_MPDU_MASK          (0x1000U)
#define NETC_IERB_L5MCAPR_MIN_MPDU_SHIFT         (12U)
/*! MIN_MPDU - Minimum MAC Protocol Data Unit (PDU) size check */
#define NETC_IERB_L5MCAPR_MIN_MPDU(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5MCAPR_MIN_MPDU_SHIFT)) & NETC_IERB_L5MCAPR_MIN_MPDU_MASK)

#define NETC_IERB_L5MCAPR_MII_PROT_MASK          (0xF000000U)
#define NETC_IERB_L5MCAPR_MII_PROT_SHIFT         (24U)
/*! MII_PROT - Indicates the MII protocol supported */
#define NETC_IERB_L5MCAPR_MII_PROT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5MCAPR_MII_PROT_SHIFT)) & NETC_IERB_L5MCAPR_MII_PROT_MASK)
/*! @} */

/*! @name L5BCR - Link 5 binding configuration register */
/*! @{ */

#define NETC_IERB_L5BCR_SW_PORT_ENETC_INST_MASK  (0x1FU)
#define NETC_IERB_L5BCR_SW_PORT_ENETC_INST_SHIFT (0U)
#define NETC_IERB_L5BCR_SW_PORT_ENETC_INST(x)    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5BCR_SW_PORT_ENETC_INST_SHIFT)) & NETC_IERB_L5BCR_SW_PORT_ENETC_INST_MASK)

#define NETC_IERB_L5BCR_NETC_FUNC_MASK           (0x40U)
#define NETC_IERB_L5BCR_NETC_FUNC_SHIFT          (6U)
#define NETC_IERB_L5BCR_NETC_FUNC(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5BCR_NETC_FUNC_SHIFT)) & NETC_IERB_L5BCR_NETC_FUNC_MASK)

#define NETC_IERB_L5BCR_SPL_SW_PORT_MASK         (0x1F0000U)
#define NETC_IERB_L5BCR_SPL_SW_PORT_SHIFT        (16U)
#define NETC_IERB_L5BCR_SPL_SW_PORT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5BCR_SPL_SW_PORT_SHIFT)) & NETC_IERB_L5BCR_SPL_SW_PORT_MASK)
/*! @} */

/*! @name L5TXBCCTR - Link 5 transmit byte credit comfort threshold register */
/*! @{ */

#define NETC_IERB_L5TXBCCTR_THRESH_MASK          (0xFFFFU)
#define NETC_IERB_L5TXBCCTR_THRESH_SHIFT         (0U)
#define NETC_IERB_L5TXBCCTR_THRESH(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5TXBCCTR_THRESH_SHIFT)) & NETC_IERB_L5TXBCCTR_THRESH_MASK)
/*! @} */

/*! @name L5E0MAR0 - Link 5 end 0 MAC address register 0 */
/*! @{ */

#define NETC_IERB_L5E0MAR0_MAC_ADDR_MASK         (0xFFFFFFFFU)
#define NETC_IERB_L5E0MAR0_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L5E0MAR0_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5E0MAR0_MAC_ADDR_SHIFT)) & NETC_IERB_L5E0MAR0_MAC_ADDR_MASK)
/*! @} */

/*! @name L5E0MAR1 - Link 5 end 0 MAC address register 1 */
/*! @{ */

#define NETC_IERB_L5E0MAR1_MAC_ADDR_MASK         (0xFFFFU)
#define NETC_IERB_L5E0MAR1_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L5E0MAR1_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5E0MAR1_MAC_ADDR_SHIFT)) & NETC_IERB_L5E0MAR1_MAC_ADDR_MASK)
/*! @} */

/*! @name L5E1MAR0 - Link 5 end 1 MAC address register 0 */
/*! @{ */

#define NETC_IERB_L5E1MAR0_MAC_ADDR_MASK         (0xFFFFFFFFU)
#define NETC_IERB_L5E1MAR0_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L5E1MAR0_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5E1MAR0_MAC_ADDR_SHIFT)) & NETC_IERB_L5E1MAR0_MAC_ADDR_MASK)
/*! @} */

/*! @name L5E1MAR1 - Link 5 end 1 MAC address register 1 */
/*! @{ */

#define NETC_IERB_L5E1MAR1_MAC_ADDR_MASK         (0xFFFFU)
#define NETC_IERB_L5E1MAR1_MAC_ADDR_SHIFT        (0U)
#define NETC_IERB_L5E1MAR1_MAC_ADDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_L5E1MAR1_MAC_ADDR_SHIFT)) & NETC_IERB_L5E1MAR1_MAC_ADDR_MASK)
/*! @} */

/*! @name SBCR - Switch 0 binding configuration register */
/*! @{ */

#define NETC_IERB_SBCR_RC_INST_MASK              (0xFU)
#define NETC_IERB_SBCR_RC_INST_SHIFT             (0U)
#define NETC_IERB_SBCR_RC_INST(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SBCR_RC_INST_SHIFT)) & NETC_IERB_SBCR_RC_INST_MASK)

#define NETC_IERB_SBCR_FN_MASK                   (0xF00U)
#define NETC_IERB_SBCR_FN_SHIFT                  (8U)
#define NETC_IERB_SBCR_FN(x)                     (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SBCR_FN_SHIFT)) & NETC_IERB_SBCR_FN_MASK)

#define NETC_IERB_SBCR_VALID_MASK                (0x80000000U)
#define NETC_IERB_SBCR_VALID_SHIFT               (31U)
#define NETC_IERB_SBCR_VALID(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SBCR_VALID_SHIFT)) & NETC_IERB_SBCR_VALID_MASK)
/*! @} */

/* The count of NETC_IERB_SBCR */
#define NETC_IERB_SBCR_COUNT                     (1U)

/*! @name SMCR - Switch 0 MSI-X configuration register */
/*! @{ */

#define NETC_IERB_SMCR_NUM_MSIX_MASK             (0xFU)
#define NETC_IERB_SMCR_NUM_MSIX_SHIFT            (0U)
#define NETC_IERB_SMCR_NUM_MSIX(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SMCR_NUM_MSIX_SHIFT)) & NETC_IERB_SMCR_NUM_MSIX_MASK)
/*! @} */

/* The count of NETC_IERB_SMCR */
#define NETC_IERB_SMCR_COUNT                     (1U)

/*! @name S_CFH_DIDVID - Switch 0 config header device ID and vendor ID register */
/*! @{ */

#define NETC_IERB_S_CFH_DIDVID_VENDOR_ID_MASK    (0xFFFFU)
#define NETC_IERB_S_CFH_DIDVID_VENDOR_ID_SHIFT   (0U)
#define NETC_IERB_S_CFH_DIDVID_VENDOR_ID(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_S_CFH_DIDVID_VENDOR_ID_SHIFT)) & NETC_IERB_S_CFH_DIDVID_VENDOR_ID_MASK)

#define NETC_IERB_S_CFH_DIDVID_DEVICE_ID_MASK    (0xFFFF0000U)
#define NETC_IERB_S_CFH_DIDVID_DEVICE_ID_SHIFT   (16U)
#define NETC_IERB_S_CFH_DIDVID_DEVICE_ID(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_S_CFH_DIDVID_DEVICE_ID_SHIFT)) & NETC_IERB_S_CFH_DIDVID_DEVICE_ID_MASK)
/*! @} */

/* The count of NETC_IERB_S_CFH_DIDVID */
#define NETC_IERB_S_CFH_DIDVID_COUNT             (1U)

/*! @name S_CFH_SIDSVID - Switch 0 config header subsystem ID and subsystem vendor ID register */
/*! @{ */

#define NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT (0U)
#define NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT)) & NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK)

#define NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK (0xFFFF0000U)
#define NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT (16U)
#define NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT)) & NETC_IERB_S_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK)
/*! @} */

/* The count of NETC_IERB_S_CFH_SIDSVID */
#define NETC_IERB_S_CFH_SIDSVID_COUNT            (1U)

/*! @name SCCAR - Switch 0 command cache attribute register */
/*! @{ */

#define NETC_IERB_SCCAR_CBD_WRCACHE_MASK         (0xFU)
#define NETC_IERB_SCCAR_CBD_WRCACHE_SHIFT        (0U)
#define NETC_IERB_SCCAR_CBD_WRCACHE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CBD_WRCACHE_SHIFT)) & NETC_IERB_SCCAR_CBD_WRCACHE_MASK)

#define NETC_IERB_SCCAR_CBD_WRDOMAIN_MASK        (0x30U)
#define NETC_IERB_SCCAR_CBD_WRDOMAIN_SHIFT       (4U)
#define NETC_IERB_SCCAR_CBD_WRDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CBD_WRDOMAIN_SHIFT)) & NETC_IERB_SCCAR_CBD_WRDOMAIN_MASK)

#define NETC_IERB_SCCAR_CBD_WRSNP_MASK           (0x40U)
#define NETC_IERB_SCCAR_CBD_WRSNP_SHIFT          (6U)
#define NETC_IERB_SCCAR_CBD_WRSNP(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CBD_WRSNP_SHIFT)) & NETC_IERB_SCCAR_CBD_WRSNP_MASK)

#define NETC_IERB_SCCAR_CWRCACHE_MASK            (0xF00U)
#define NETC_IERB_SCCAR_CWRCACHE_SHIFT           (8U)
#define NETC_IERB_SCCAR_CWRCACHE(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CWRCACHE_SHIFT)) & NETC_IERB_SCCAR_CWRCACHE_MASK)

#define NETC_IERB_SCCAR_CWRDOMAIN_MASK           (0x3000U)
#define NETC_IERB_SCCAR_CWRDOMAIN_SHIFT          (12U)
#define NETC_IERB_SCCAR_CWRDOMAIN(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CWRDOMAIN_SHIFT)) & NETC_IERB_SCCAR_CWRDOMAIN_MASK)

#define NETC_IERB_SCCAR_CWRSNP_MASK              (0x4000U)
#define NETC_IERB_SCCAR_CWRSNP_SHIFT             (14U)
#define NETC_IERB_SCCAR_CWRSNP(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CWRSNP_SHIFT)) & NETC_IERB_SCCAR_CWRSNP_MASK)

#define NETC_IERB_SCCAR_CBD_RDCACHE_MASK         (0xF0000U)
#define NETC_IERB_SCCAR_CBD_RDCACHE_SHIFT        (16U)
#define NETC_IERB_SCCAR_CBD_RDCACHE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CBD_RDCACHE_SHIFT)) & NETC_IERB_SCCAR_CBD_RDCACHE_MASK)

#define NETC_IERB_SCCAR_CBD_RDDOMAIN_MASK        (0x300000U)
#define NETC_IERB_SCCAR_CBD_RDDOMAIN_SHIFT       (20U)
#define NETC_IERB_SCCAR_CBD_RDDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CBD_RDDOMAIN_SHIFT)) & NETC_IERB_SCCAR_CBD_RDDOMAIN_MASK)

#define NETC_IERB_SCCAR_CBD_RDSNP_MASK           (0x400000U)
#define NETC_IERB_SCCAR_CBD_RDSNP_SHIFT          (22U)
#define NETC_IERB_SCCAR_CBD_RDSNP(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CBD_RDSNP_SHIFT)) & NETC_IERB_SCCAR_CBD_RDSNP_MASK)

#define NETC_IERB_SCCAR_CRDCACHE_MASK            (0xF000000U)
#define NETC_IERB_SCCAR_CRDCACHE_SHIFT           (24U)
#define NETC_IERB_SCCAR_CRDCACHE(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CRDCACHE_SHIFT)) & NETC_IERB_SCCAR_CRDCACHE_MASK)

#define NETC_IERB_SCCAR_CRDDOMAIN_MASK           (0x30000000U)
#define NETC_IERB_SCCAR_CRDDOMAIN_SHIFT          (28U)
#define NETC_IERB_SCCAR_CRDDOMAIN(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CRDDOMAIN_SHIFT)) & NETC_IERB_SCCAR_CRDDOMAIN_MASK)

#define NETC_IERB_SCCAR_CRDSNP_MASK              (0x40000000U)
#define NETC_IERB_SCCAR_CRDSNP_SHIFT             (30U)
#define NETC_IERB_SCCAR_CRDSNP(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SCCAR_CRDSNP_SHIFT)) & NETC_IERB_SCCAR_CRDSNP_MASK)
/*! @} */

/* The count of NETC_IERB_SCCAR */
#define NETC_IERB_SCCAR_COUNT                    (1U)

/*! @name SAMQR - Switch 0 access management qualifier register */
/*! @{ */

#define NETC_IERB_SAMQR_ARQOS_MASK               (0xF0000U)
#define NETC_IERB_SAMQR_ARQOS_SHIFT              (16U)
#define NETC_IERB_SAMQR_ARQOS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SAMQR_ARQOS_SHIFT)) & NETC_IERB_SAMQR_ARQOS_MASK)

#define NETC_IERB_SAMQR_AWQOS_MASK               (0xF00000U)
#define NETC_IERB_SAMQR_AWQOS_SHIFT              (20U)
#define NETC_IERB_SAMQR_AWQOS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SAMQR_AWQOS_SHIFT)) & NETC_IERB_SAMQR_AWQOS_MASK)

#define NETC_IERB_SAMQR_BMT_MASK                 (0x80000000U)
#define NETC_IERB_SAMQR_BMT_SHIFT                (31U)
#define NETC_IERB_SAMQR_BMT(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SAMQR_BMT_SHIFT)) & NETC_IERB_SAMQR_BMT_MASK)
/*! @} */

/* The count of NETC_IERB_SAMQR */
#define NETC_IERB_SAMQR_COUNT                    (1U)

/*! @name SBLPR - Switch 0 boot loader parameter register 0..Switch 0 boot loader parameter register 1 */
/*! @{ */

#define NETC_IERB_SBLPR_PARAM_VAL_MASK           (0xFFFFFFFFU)
#define NETC_IERB_SBLPR_PARAM_VAL_SHIFT          (0U)
#define NETC_IERB_SBLPR_PARAM_VAL(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SBLPR_PARAM_VAL_SHIFT)) & NETC_IERB_SBLPR_PARAM_VAL_MASK)
/*! @} */

/* The count of NETC_IERB_SBLPR */
#define NETC_IERB_SBLPR_COUNT                    (1U)

/* The count of NETC_IERB_SBLPR */
#define NETC_IERB_SBLPR_COUNT2                   (2U)

/*! @name SSMBAR - Switch 0 shared memory buffer allotment register */
/*! @{ */

#define NETC_IERB_SSMBAR_ALLOC_MASK              (0xFFFFFFU)
#define NETC_IERB_SSMBAR_ALLOC_SHIFT             (0U)
#define NETC_IERB_SSMBAR_ALLOC(x)                (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SSMBAR_ALLOC_SHIFT)) & NETC_IERB_SSMBAR_ALLOC_MASK)

#define NETC_IERB_SSMBAR_MLOC_MASK               (0xC0000000U)
#define NETC_IERB_SSMBAR_MLOC_SHIFT              (30U)
#define NETC_IERB_SSMBAR_MLOC(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SSMBAR_MLOC_SHIFT)) & NETC_IERB_SSMBAR_MLOC_MASK)
/*! @} */

/* The count of NETC_IERB_SSMBAR */
#define NETC_IERB_SSMBAR_COUNT                   (1U)

/*! @name SHTMAR - Switch 0 hash table memory allotment register */
/*! @{ */

#define NETC_IERB_SHTMAR_NUM_WORDS_MASK          (0xFFFFU)
#define NETC_IERB_SHTMAR_NUM_WORDS_SHIFT         (0U)
#define NETC_IERB_SHTMAR_NUM_WORDS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SHTMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SHTMAR_NUM_WORDS_MASK)

#define NETC_IERB_SHTMAR_MLOC_MASK               (0xC0000000U)
#define NETC_IERB_SHTMAR_MLOC_SHIFT              (30U)
#define NETC_IERB_SHTMAR_MLOC(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SHTMAR_MLOC_SHIFT)) & NETC_IERB_SHTMAR_MLOC_MASK)
/*! @} */

/* The count of NETC_IERB_SHTMAR */
#define NETC_IERB_SHTMAR_COUNT                   (1U)

/*! @name SITMAR - Switch 0 index table memory allocation register */
/*! @{ */

#define NETC_IERB_SITMAR_NUM_WORDS_MASK          (0xFFFFU)
#define NETC_IERB_SITMAR_NUM_WORDS_SHIFT         (0U)
#define NETC_IERB_SITMAR_NUM_WORDS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SITMAR_NUM_WORDS_MASK)

#define NETC_IERB_SITMAR_MLOC_MASK               (0xC0000000U)
#define NETC_IERB_SITMAR_MLOC_SHIFT              (30U)
#define NETC_IERB_SITMAR_MLOC(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SITMAR_MLOC_SHIFT)) & NETC_IERB_SITMAR_MLOC_MASK)
/*! @} */

/* The count of NETC_IERB_SITMAR */
#define NETC_IERB_SITMAR_COUNT                   (1U)

/*! @name SIPFTMAR - Switch 0 ingress port filter table memory allocation register */
/*! @{ */

#define NETC_IERB_SIPFTMAR_ALLOC_MASK            (0xFFFFU)
#define NETC_IERB_SIPFTMAR_ALLOC_SHIFT           (0U)
#define NETC_IERB_SIPFTMAR_ALLOC(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SIPFTMAR_ALLOC_SHIFT)) & NETC_IERB_SIPFTMAR_ALLOC_MASK)
/*! @} */

/* The count of NETC_IERB_SIPFTMAR */
#define NETC_IERB_SIPFTMAR_COUNT                 (1U)

/*! @name SRPITMAR - Switch 0 rate policer index table memory allocation register */
/*! @{ */

#define NETC_IERB_SRPITMAR_NUM_WORDS_MASK        (0xFFFFU)
#define NETC_IERB_SRPITMAR_NUM_WORDS_SHIFT       (0U)
#define NETC_IERB_SRPITMAR_NUM_WORDS(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SRPITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SRPITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SRPITMAR */
#define NETC_IERB_SRPITMAR_COUNT                 (1U)

/*! @name SISCITMAR - Switch 0 ingress stream counter index table memory allocation register */
/*! @{ */

#define NETC_IERB_SISCITMAR_NUM_WORDS_MASK       (0xFFFFU)
#define NETC_IERB_SISCITMAR_NUM_WORDS_SHIFT      (0U)
#define NETC_IERB_SISCITMAR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SISCITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SISCITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SISCITMAR */
#define NETC_IERB_SISCITMAR_COUNT                (1U)

/*! @name SISITMAR - Switch 0 ingress stream index table memory allocation register */
/*! @{ */

#define NETC_IERB_SISITMAR_NUM_WORDS_MASK        (0xFFFFU)
#define NETC_IERB_SISITMAR_NUM_WORDS_SHIFT       (0U)
#define NETC_IERB_SISITMAR_NUM_WORDS(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SISITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SISITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SISITMAR */
#define NETC_IERB_SISITMAR_COUNT                 (1U)

/*! @name SISQGITMAR - Switch 0 ingress sequence generation index table memory allocation register */
/*! @{ */

#define NETC_IERB_SISQGITMAR_NUM_WORDS_MASK      (0x1FFFU)
#define NETC_IERB_SISQGITMAR_NUM_WORDS_SHIFT     (0U)
#define NETC_IERB_SISQGITMAR_NUM_WORDS(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SISQGITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SISQGITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SISQGITMAR */
#define NETC_IERB_SISQGITMAR_COUNT               (1U)

/*! @name SSGIITMAR - Switch 0 stream gate instance index table memory allocation register */
/*! @{ */

#define NETC_IERB_SSGIITMAR_NUM_WORDS_MASK       (0xFFFFU)
#define NETC_IERB_SSGIITMAR_NUM_WORDS_SHIFT      (0U)
#define NETC_IERB_SSGIITMAR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SSGIITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SSGIITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SSGIITMAR */
#define NETC_IERB_SSGIITMAR_COUNT                (1U)

/*! @name SSGCLITMAR - Switch 0 stream gate control list index table memory allocation register */
/*! @{ */

#define NETC_IERB_SSGCLITMAR_NUM_WORDS_MASK      (0xFFFFU)
#define NETC_IERB_SSGCLITMAR_NUM_WORDS_SHIFT     (0U)
#define NETC_IERB_SSGCLITMAR_NUM_WORDS(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SSGCLITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SSGCLITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SSGCLITMAR */
#define NETC_IERB_SSGCLITMAR_COUNT               (1U)

/*! @name SFMITMAR - Switch 0 frame modification index table memory allocation register */
/*! @{ */

#define NETC_IERB_SFMITMAR_NUM_WORDS_MASK        (0x1FFFU)
#define NETC_IERB_SFMITMAR_NUM_WORDS_SHIFT       (0U)
#define NETC_IERB_SFMITMAR_NUM_WORDS(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SFMITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SFMITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SFMITMAR */
#define NETC_IERB_SFMITMAR_COUNT                 (1U)

/*! @name SFMDITMAR - Switch 0 frame modification data index table memory allocation register */
/*! @{ */

#define NETC_IERB_SFMDITMAR_NUM_WORDS_MASK       (0xFFFFU)
#define NETC_IERB_SFMDITMAR_NUM_WORDS_SHIFT      (0U)
#define NETC_IERB_SFMDITMAR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SFMDITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_SFMDITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_SFMDITMAR */
#define NETC_IERB_SFMDITMAR_COUNT                (1U)

/*! @name STGSTAR - Switch 0 time gate scheduling table allocation register */
/*! @{ */

#define NETC_IERB_STGSTAR_NUM_WORDS_MASK         (0xFFFU)
#define NETC_IERB_STGSTAR_NUM_WORDS_SHIFT        (0U)
#define NETC_IERB_STGSTAR_NUM_WORDS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_STGSTAR_NUM_WORDS_SHIFT)) & NETC_IERB_STGSTAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_STGSTAR */
#define NETC_IERB_STGSTAR_COUNT                  (1U)

/*! @name STGSLR - Switch 0 time gate scheduling lookahead register */
/*! @{ */

#define NETC_IERB_STGSLR_MIN_LOOKAHEAD_MASK      (0xFFFFFU)
#define NETC_IERB_STGSLR_MIN_LOOKAHEAD_SHIFT     (0U)
#define NETC_IERB_STGSLR_MIN_LOOKAHEAD(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_STGSLR_MIN_LOOKAHEAD_SHIFT)) & NETC_IERB_STGSLR_MIN_LOOKAHEAD_MASK)
/*! @} */

/* The count of NETC_IERB_STGSLR */
#define NETC_IERB_STGSLR_COUNT                   (1U)

/*! @name SMPCR - Switch 0 management port configuration register */
/*! @{ */

#define NETC_IERB_SMPCR_PORT_MASK                (0x1FU)
#define NETC_IERB_SMPCR_PORT_SHIFT               (0U)
#define NETC_IERB_SMPCR_PORT(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SMPCR_PORT_SHIFT)) & NETC_IERB_SMPCR_PORT_MASK)
/*! @} */

/* The count of NETC_IERB_SMPCR */
#define NETC_IERB_SMPCR_COUNT                    (1U)

/*! @name SVFHTDECR0 - Switch 0 VLAN Filter (hash) table default entry configuration registers 0 */
/*! @{ */

#define NETC_IERB_SVFHTDECR0_PORT0_MASK          (0x1U)
#define NETC_IERB_SVFHTDECR0_PORT0_SHIFT         (0U)
/*! PORT0 - Port n.
 *  0b0..Port n is not a member of this VLAN.
 *  0b1..Port n is a member of this VLAN.
 */
#define NETC_IERB_SVFHTDECR0_PORT0(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_PORT0_SHIFT)) & NETC_IERB_SVFHTDECR0_PORT0_MASK)

#define NETC_IERB_SVFHTDECR0_PORT1_MASK          (0x2U)
#define NETC_IERB_SVFHTDECR0_PORT1_SHIFT         (1U)
/*! PORT1 - Port n.
 *  0b0..Port n is not a member of this VLAN.
 *  0b1..Port n is a member of this VLAN.
 */
#define NETC_IERB_SVFHTDECR0_PORT1(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_PORT1_SHIFT)) & NETC_IERB_SVFHTDECR0_PORT1_MASK)

#define NETC_IERB_SVFHTDECR0_PORT2_MASK          (0x4U)
#define NETC_IERB_SVFHTDECR0_PORT2_SHIFT         (2U)
/*! PORT2 - Port n.
 *  0b0..Port n is not a member of this VLAN.
 *  0b1..Port n is a member of this VLAN.
 */
#define NETC_IERB_SVFHTDECR0_PORT2(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_PORT2_SHIFT)) & NETC_IERB_SVFHTDECR0_PORT2_MASK)

#define NETC_IERB_SVFHTDECR0_PORT3_MASK          (0x8U)
#define NETC_IERB_SVFHTDECR0_PORT3_SHIFT         (3U)
/*! PORT3 - Port n.
 *  0b0..Port n is not a member of this VLAN.
 *  0b1..Port n is a member of this VLAN.
 */
#define NETC_IERB_SVFHTDECR0_PORT3(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_PORT3_SHIFT)) & NETC_IERB_SVFHTDECR0_PORT3_MASK)

#define NETC_IERB_SVFHTDECR0_PORT4_MASK          (0x10U)
#define NETC_IERB_SVFHTDECR0_PORT4_SHIFT         (4U)
/*! PORT4 - Port n.
 *  0b0..Port n is not a member of this VLAN.
 *  0b1..Port n is a member of this VLAN.
 */
#define NETC_IERB_SVFHTDECR0_PORT4(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_PORT4_SHIFT)) & NETC_IERB_SVFHTDECR0_PORT4_MASK)

#define NETC_IERB_SVFHTDECR0_STG_ID_MASK         (0xF000000U)
#define NETC_IERB_SVFHTDECR0_STG_ID_SHIFT        (24U)
#define NETC_IERB_SVFHTDECR0_STG_ID(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_STG_ID_SHIFT)) & NETC_IERB_SVFHTDECR0_STG_ID_MASK)

#define NETC_IERB_SVFHTDECR0_IPMFE_MASK          (0x20000000U)
#define NETC_IERB_SVFHTDECR0_IPMFE_SHIFT         (29U)
/*! IPMFE
 *  0b0..No IP multicast filtering is performed.
 *  0b1..If the frame is identified as a multicast IP packet, then IP multicast filtering is performed. If the
 *       frame is not identified as an IP multicast packet, the IP multicast filtering is not performed.
 */
#define NETC_IERB_SVFHTDECR0_IPMFE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_IPMFE_SHIFT)) & NETC_IERB_SVFHTDECR0_IPMFE_MASK)

#define NETC_IERB_SVFHTDECR0_IPMFLE_MASK         (0x40000000U)
#define NETC_IERB_SVFHTDECR0_IPMFLE_SHIFT        (30U)
/*! IPMFLE
 *  0b0..IP Multicast Flooding disabled, the frame is discarded.
 *  0b1..IP Multicast Flooding enabled, the frame is flooded.
 */
#define NETC_IERB_SVFHTDECR0_IPMFLE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR0_IPMFLE_SHIFT)) & NETC_IERB_SVFHTDECR0_IPMFLE_MASK)
/*! @} */

/* The count of NETC_IERB_SVFHTDECR0 */
#define NETC_IERB_SVFHTDECR0_COUNT               (1U)

/*! @name SVFHTDECR1 - Switch 0 VLAN filter hash table default entry configuration registers 1 */
/*! @{ */

#define NETC_IERB_SVFHTDECR1_FID_MASK            (0xFFFU)
#define NETC_IERB_SVFHTDECR1_FID_SHIFT           (0U)
#define NETC_IERB_SVFHTDECR1_FID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR1_FID_SHIFT)) & NETC_IERB_SVFHTDECR1_FID_MASK)

#define NETC_IERB_SVFHTDECR1_VL_MODE_MASK        (0x1000U)
#define NETC_IERB_SVFHTDECR1_VL_MODE_SHIFT       (12U)
#define NETC_IERB_SVFHTDECR1_VL_MODE(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR1_VL_MODE_SHIFT)) & NETC_IERB_SVFHTDECR1_VL_MODE_MASK)

#define NETC_IERB_SVFHTDECR1_BASE_ETEID_MASK     (0xFFFF0000U)
#define NETC_IERB_SVFHTDECR1_BASE_ETEID_SHIFT    (16U)
#define NETC_IERB_SVFHTDECR1_BASE_ETEID(x)       (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR1_BASE_ETEID_SHIFT)) & NETC_IERB_SVFHTDECR1_BASE_ETEID_MASK)
/*! @} */

/* The count of NETC_IERB_SVFHTDECR1 */
#define NETC_IERB_SVFHTDECR1_COUNT               (1U)

/*! @name SVFHTDECR2 - Switch 0 VLAN filter hash table default entry configuration registers 2 */
/*! @{ */

#define NETC_IERB_SVFHTDECR2_ES_PORT0_MASK       (0x1U)
#define NETC_IERB_SVFHTDECR2_ES_PORT0_SHIFT      (0U)
#define NETC_IERB_SVFHTDECR2_ES_PORT0(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR2_ES_PORT0_SHIFT)) & NETC_IERB_SVFHTDECR2_ES_PORT0_MASK)

#define NETC_IERB_SVFHTDECR2_ES_PORT1_MASK       (0x2U)
#define NETC_IERB_SVFHTDECR2_ES_PORT1_SHIFT      (1U)
#define NETC_IERB_SVFHTDECR2_ES_PORT1(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR2_ES_PORT1_SHIFT)) & NETC_IERB_SVFHTDECR2_ES_PORT1_MASK)

#define NETC_IERB_SVFHTDECR2_ES_PORT2_MASK       (0x4U)
#define NETC_IERB_SVFHTDECR2_ES_PORT2_SHIFT      (2U)
#define NETC_IERB_SVFHTDECR2_ES_PORT2(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR2_ES_PORT2_SHIFT)) & NETC_IERB_SVFHTDECR2_ES_PORT2_MASK)

#define NETC_IERB_SVFHTDECR2_ES_PORT3_MASK       (0x8U)
#define NETC_IERB_SVFHTDECR2_ES_PORT3_SHIFT      (3U)
#define NETC_IERB_SVFHTDECR2_ES_PORT3(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR2_ES_PORT3_SHIFT)) & NETC_IERB_SVFHTDECR2_ES_PORT3_MASK)

#define NETC_IERB_SVFHTDECR2_ES_PORT4_MASK       (0x10U)
#define NETC_IERB_SVFHTDECR2_ES_PORT4_SHIFT      (4U)
#define NETC_IERB_SVFHTDECR2_ES_PORT4(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR2_ES_PORT4_SHIFT)) & NETC_IERB_SVFHTDECR2_ES_PORT4_MASK)

#define NETC_IERB_SVFHTDECR2_MLO_MASK            (0x7000000U)
#define NETC_IERB_SVFHTDECR2_MLO_SHIFT           (24U)
#define NETC_IERB_SVFHTDECR2_MLO(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR2_MLO_SHIFT)) & NETC_IERB_SVFHTDECR2_MLO_MASK)

#define NETC_IERB_SVFHTDECR2_MFO_MASK            (0x18000000U)
#define NETC_IERB_SVFHTDECR2_MFO_SHIFT           (27U)
#define NETC_IERB_SVFHTDECR2_MFO(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_SVFHTDECR2_MFO_SHIFT)) & NETC_IERB_SVFHTDECR2_MFO_MASK)
/*! @} */

/* The count of NETC_IERB_SVFHTDECR2 */
#define NETC_IERB_SVFHTDECR2_COUNT               (1U)

/*! @name EBCR0 - ENETC 0 binding configuration register 0..ENETC 1 binding configuration register 0 */
/*! @{ */

#define NETC_IERB_EBCR0_RC_INST_MASK             (0xFU)
#define NETC_IERB_EBCR0_RC_INST_SHIFT            (0U)
#define NETC_IERB_EBCR0_RC_INST(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCR0_RC_INST_SHIFT)) & NETC_IERB_EBCR0_RC_INST_MASK)

#define NETC_IERB_EBCR0_FN_MASK                  (0xF00U)
#define NETC_IERB_EBCR0_FN_SHIFT                 (8U)
#define NETC_IERB_EBCR0_FN(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCR0_FN_SHIFT)) & NETC_IERB_EBCR0_FN_MASK)

#define NETC_IERB_EBCR0_VALID_MASK               (0x80000000U)
#define NETC_IERB_EBCR0_VALID_SHIFT              (31U)
#define NETC_IERB_EBCR0_VALID(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCR0_VALID_SHIFT)) & NETC_IERB_EBCR0_VALID_MASK)
/*! @} */

/* The count of NETC_IERB_EBCR0 */
#define NETC_IERB_EBCR0_COUNT                    (2U)

/*! @name EBCR1 - ENETC 0 binding configuration register 1..ENETC 1 binding configuration register 1 */
/*! @{ */

#define NETC_IERB_EBCR1_NUM_RX_BDR_MASK          (0x3FFU)
#define NETC_IERB_EBCR1_NUM_RX_BDR_SHIFT         (0U)
#define NETC_IERB_EBCR1_NUM_RX_BDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCR1_NUM_RX_BDR_SHIFT)) & NETC_IERB_EBCR1_NUM_RX_BDR_MASK)

#define NETC_IERB_EBCR1_NUM_TX_BDR_MASK          (0x3FF0000U)
#define NETC_IERB_EBCR1_NUM_TX_BDR_SHIFT         (16U)
#define NETC_IERB_EBCR1_NUM_TX_BDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCR1_NUM_TX_BDR_SHIFT)) & NETC_IERB_EBCR1_NUM_TX_BDR_MASK)
/*! @} */

/* The count of NETC_IERB_EBCR1 */
#define NETC_IERB_EBCR1_COUNT                    (2U)

/*! @name EBCR2 - ENETC 0 binding configuration register 2..ENETC 1 binding configuration register 2 */
/*! @{ */

#define NETC_IERB_EBCR2_NUM_MAC_AFTE_MASK        (0xFFFU)
#define NETC_IERB_EBCR2_NUM_MAC_AFTE_SHIFT       (0U)
#define NETC_IERB_EBCR2_NUM_MAC_AFTE(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCR2_NUM_MAC_AFTE_SHIFT)) & NETC_IERB_EBCR2_NUM_MAC_AFTE_MASK)

#define NETC_IERB_EBCR2_NUM_VLAN_FTE_MASK        (0xFFF0000U)
#define NETC_IERB_EBCR2_NUM_VLAN_FTE_SHIFT       (16U)
#define NETC_IERB_EBCR2_NUM_VLAN_FTE(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCR2_NUM_VLAN_FTE_SHIFT)) & NETC_IERB_EBCR2_NUM_VLAN_FTE_MASK)
/*! @} */

/* The count of NETC_IERB_EBCR2 */
#define NETC_IERB_EBCR2_COUNT                    (2U)

/*! @name EVBCR - ENETC 0 VSI binding configuration register..ENETC 1 VSI binding configuration register */
/*! @{ */

#define NETC_IERB_EVBCR_NUM_VSI_MASK             (0xFU)
#define NETC_IERB_EVBCR_NUM_VSI_SHIFT            (0U)
#define NETC_IERB_EVBCR_NUM_VSI(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EVBCR_NUM_VSI_SHIFT)) & NETC_IERB_EVBCR_NUM_VSI_MASK)
/*! @} */

/* The count of NETC_IERB_EVBCR */
#define NETC_IERB_EVBCR_COUNT                    (2U)

/*! @name EMCR - ENETC 0 MSI-X configuration register..ENETC 1 MSI-X configuration register */
/*! @{ */

#define NETC_IERB_EMCR_NUM_MSIX_MASK             (0x7FFU)
#define NETC_IERB_EMCR_NUM_MSIX_SHIFT            (0U)
#define NETC_IERB_EMCR_NUM_MSIX(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMCR_NUM_MSIX_SHIFT)) & NETC_IERB_EMCR_NUM_MSIX_MASK)
/*! @} */

/* The count of NETC_IERB_EMCR */
#define NETC_IERB_EMCR_COUNT                     (2U)

/*! @name E_CFH_DIDVID - ENETC 0 config header device ID and vendor ID register..ENETC 1 config header device ID and vendor ID register */
/*! @{ */

#define NETC_IERB_E_CFH_DIDVID_VENDOR_ID_MASK    (0xFFFFU)
#define NETC_IERB_E_CFH_DIDVID_VENDOR_ID_SHIFT   (0U)
#define NETC_IERB_E_CFH_DIDVID_VENDOR_ID(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_E_CFH_DIDVID_VENDOR_ID_SHIFT)) & NETC_IERB_E_CFH_DIDVID_VENDOR_ID_MASK)

#define NETC_IERB_E_CFH_DIDVID_DEVICE_ID_MASK    (0xFFFF0000U)
#define NETC_IERB_E_CFH_DIDVID_DEVICE_ID_SHIFT   (16U)
#define NETC_IERB_E_CFH_DIDVID_DEVICE_ID(x)      (((uint32_t)(((uint32_t)(x)) << NETC_IERB_E_CFH_DIDVID_DEVICE_ID_SHIFT)) & NETC_IERB_E_CFH_DIDVID_DEVICE_ID_MASK)
/*! @} */

/* The count of NETC_IERB_E_CFH_DIDVID */
#define NETC_IERB_E_CFH_DIDVID_COUNT             (2U)

/*! @name E_CFH_SIDSVID - ENETC 0 config header subsystem ID and subsystem vendor ID register..ENETC 1 config header subsystem ID and subsystem vendor ID register */
/*! @{ */

#define NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK (0xFFFFU)
#define NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT (0U)
#define NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_SHIFT)) & NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_VENDOR_ID_MASK)

#define NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK (0xFFFF0000U)
#define NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT (16U)
#define NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID(x) (((uint32_t)(((uint32_t)(x)) << NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_SHIFT)) & NETC_IERB_E_CFH_SIDSVID_SUBSYSTEM_DEVICE_ID_MASK)
/*! @} */

/* The count of NETC_IERB_E_CFH_SIDSVID */
#define NETC_IERB_E_CFH_SIDSVID_COUNT            (2U)

/*! @name E_CFC_VFDID - ENETC 0 config capability VF device ID register..ENETC 1 config capability VF device ID register */
/*! @{ */

#define NETC_IERB_E_CFC_VFDID_VF_DEVICE_ID_MASK  (0xFFFF0000U)
#define NETC_IERB_E_CFC_VFDID_VF_DEVICE_ID_SHIFT (16U)
#define NETC_IERB_E_CFC_VFDID_VF_DEVICE_ID(x)    (((uint32_t)(((uint32_t)(x)) << NETC_IERB_E_CFC_VFDID_VF_DEVICE_ID_SHIFT)) & NETC_IERB_E_CFC_VFDID_VF_DEVICE_ID_MASK)
/*! @} */

/* The count of NETC_IERB_E_CFC_VFDID */
#define NETC_IERB_E_CFC_VFDID_COUNT              (2U)

/*! @name EBCAR - ENETC 0 buffer cache attribute register 0..ENETC 1 buffer cache attribute register 0 */
/*! @{ */

#define NETC_IERB_EBCAR_BD_WRCACHE_MASK          (0xFU)
#define NETC_IERB_EBCAR_BD_WRCACHE_SHIFT         (0U)
#define NETC_IERB_EBCAR_BD_WRCACHE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_BD_WRCACHE_SHIFT)) & NETC_IERB_EBCAR_BD_WRCACHE_MASK)

#define NETC_IERB_EBCAR_BD_WRDOMAIN_MASK         (0x30U)
#define NETC_IERB_EBCAR_BD_WRDOMAIN_SHIFT        (4U)
#define NETC_IERB_EBCAR_BD_WRDOMAIN(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_BD_WRDOMAIN_SHIFT)) & NETC_IERB_EBCAR_BD_WRDOMAIN_MASK)

#define NETC_IERB_EBCAR_BD_WRSNP_MASK            (0x40U)
#define NETC_IERB_EBCAR_BD_WRSNP_SHIFT           (6U)
#define NETC_IERB_EBCAR_BD_WRSNP(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_BD_WRSNP_SHIFT)) & NETC_IERB_EBCAR_BD_WRSNP_MASK)

#define NETC_IERB_EBCAR_WRCACHE_MASK             (0xF00U)
#define NETC_IERB_EBCAR_WRCACHE_SHIFT            (8U)
#define NETC_IERB_EBCAR_WRCACHE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_WRCACHE_SHIFT)) & NETC_IERB_EBCAR_WRCACHE_MASK)

#define NETC_IERB_EBCAR_WRDOMAIN_MASK            (0x3000U)
#define NETC_IERB_EBCAR_WRDOMAIN_SHIFT           (12U)
#define NETC_IERB_EBCAR_WRDOMAIN(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_WRDOMAIN_SHIFT)) & NETC_IERB_EBCAR_WRDOMAIN_MASK)

#define NETC_IERB_EBCAR_WRSNP_MASK               (0x4000U)
#define NETC_IERB_EBCAR_WRSNP_SHIFT              (14U)
#define NETC_IERB_EBCAR_WRSNP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_WRSNP_SHIFT)) & NETC_IERB_EBCAR_WRSNP_MASK)

#define NETC_IERB_EBCAR_BD_RDCACHE_MASK          (0xF0000U)
#define NETC_IERB_EBCAR_BD_RDCACHE_SHIFT         (16U)
#define NETC_IERB_EBCAR_BD_RDCACHE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_BD_RDCACHE_SHIFT)) & NETC_IERB_EBCAR_BD_RDCACHE_MASK)

#define NETC_IERB_EBCAR_BD_RDDOMAIN_MASK         (0x300000U)
#define NETC_IERB_EBCAR_BD_RDDOMAIN_SHIFT        (20U)
#define NETC_IERB_EBCAR_BD_RDDOMAIN(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_BD_RDDOMAIN_SHIFT)) & NETC_IERB_EBCAR_BD_RDDOMAIN_MASK)

#define NETC_IERB_EBCAR_BD_RDSNP_MASK            (0x400000U)
#define NETC_IERB_EBCAR_BD_RDSNP_SHIFT           (22U)
#define NETC_IERB_EBCAR_BD_RDSNP(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_BD_RDSNP_SHIFT)) & NETC_IERB_EBCAR_BD_RDSNP_MASK)

#define NETC_IERB_EBCAR_RDCACHE_MASK             (0xF000000U)
#define NETC_IERB_EBCAR_RDCACHE_SHIFT            (24U)
#define NETC_IERB_EBCAR_RDCACHE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_RDCACHE_SHIFT)) & NETC_IERB_EBCAR_RDCACHE_MASK)

#define NETC_IERB_EBCAR_RDDOMAIN_MASK            (0x30000000U)
#define NETC_IERB_EBCAR_RDDOMAIN_SHIFT           (28U)
#define NETC_IERB_EBCAR_RDDOMAIN(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_RDDOMAIN_SHIFT)) & NETC_IERB_EBCAR_RDDOMAIN_MASK)

#define NETC_IERB_EBCAR_RDSNP_MASK               (0x40000000U)
#define NETC_IERB_EBCAR_RDSNP_SHIFT              (30U)
#define NETC_IERB_EBCAR_RDSNP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBCAR_RDSNP_SHIFT)) & NETC_IERB_EBCAR_RDSNP_MASK)
/*! @} */

/* The count of NETC_IERB_EBCAR */
#define NETC_IERB_EBCAR_COUNT                    (2U)

/*! @name EMCAR - ENETC 0 message cache attribute register..ENETC 1 message cache attribute register */
/*! @{ */

#define NETC_IERB_EMCAR_MSG_WRCACHE_MASK         (0xFU)
#define NETC_IERB_EMCAR_MSG_WRCACHE_SHIFT        (0U)
#define NETC_IERB_EMCAR_MSG_WRCACHE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMCAR_MSG_WRCACHE_SHIFT)) & NETC_IERB_EMCAR_MSG_WRCACHE_MASK)

#define NETC_IERB_EMCAR_MSG_WRDOMAIN_MASK        (0x30U)
#define NETC_IERB_EMCAR_MSG_WRDOMAIN_SHIFT       (4U)
#define NETC_IERB_EMCAR_MSG_WRDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMCAR_MSG_WRDOMAIN_SHIFT)) & NETC_IERB_EMCAR_MSG_WRDOMAIN_MASK)

#define NETC_IERB_EMCAR_MSG_WRSNP_MASK           (0x40U)
#define NETC_IERB_EMCAR_MSG_WRSNP_SHIFT          (6U)
#define NETC_IERB_EMCAR_MSG_WRSNP(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMCAR_MSG_WRSNP_SHIFT)) & NETC_IERB_EMCAR_MSG_WRSNP_MASK)

#define NETC_IERB_EMCAR_MSG_RDCACHE_MASK         (0xF0000U)
#define NETC_IERB_EMCAR_MSG_RDCACHE_SHIFT        (16U)
#define NETC_IERB_EMCAR_MSG_RDCACHE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMCAR_MSG_RDCACHE_SHIFT)) & NETC_IERB_EMCAR_MSG_RDCACHE_MASK)

#define NETC_IERB_EMCAR_MSG_RDDOMAIN_MASK        (0x300000U)
#define NETC_IERB_EMCAR_MSG_RDDOMAIN_SHIFT       (20U)
#define NETC_IERB_EMCAR_MSG_RDDOMAIN(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMCAR_MSG_RDDOMAIN_SHIFT)) & NETC_IERB_EMCAR_MSG_RDDOMAIN_MASK)

#define NETC_IERB_EMCAR_MSG_RDSNP_MASK           (0x400000U)
#define NETC_IERB_EMCAR_MSG_RDSNP_SHIFT          (22U)
#define NETC_IERB_EMCAR_MSG_RDSNP(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EMCAR_MSG_RDSNP_SHIFT)) & NETC_IERB_EMCAR_MSG_RDSNP_MASK)
/*! @} */

/* The count of NETC_IERB_EMCAR */
#define NETC_IERB_EMCAR_COUNT                    (2U)

/*! @name ECAR - ENETC 0 command cache attribute register..ENETC 1 command cache attribute register */
/*! @{ */

#define NETC_IERB_ECAR_CBD_WRCACHE_MASK          (0xFU)
#define NETC_IERB_ECAR_CBD_WRCACHE_SHIFT         (0U)
#define NETC_IERB_ECAR_CBD_WRCACHE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CBD_WRCACHE_SHIFT)) & NETC_IERB_ECAR_CBD_WRCACHE_MASK)

#define NETC_IERB_ECAR_CBD_WRDOMAIN_MASK         (0x30U)
#define NETC_IERB_ECAR_CBD_WRDOMAIN_SHIFT        (4U)
#define NETC_IERB_ECAR_CBD_WRDOMAIN(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CBD_WRDOMAIN_SHIFT)) & NETC_IERB_ECAR_CBD_WRDOMAIN_MASK)

#define NETC_IERB_ECAR_CBD_WRSNP_MASK            (0x40U)
#define NETC_IERB_ECAR_CBD_WRSNP_SHIFT           (6U)
#define NETC_IERB_ECAR_CBD_WRSNP(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CBD_WRSNP_SHIFT)) & NETC_IERB_ECAR_CBD_WRSNP_MASK)

#define NETC_IERB_ECAR_CWRCACHE_MASK             (0xF00U)
#define NETC_IERB_ECAR_CWRCACHE_SHIFT            (8U)
#define NETC_IERB_ECAR_CWRCACHE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CWRCACHE_SHIFT)) & NETC_IERB_ECAR_CWRCACHE_MASK)

#define NETC_IERB_ECAR_CWRDOMAIN_MASK            (0x3000U)
#define NETC_IERB_ECAR_CWRDOMAIN_SHIFT           (12U)
#define NETC_IERB_ECAR_CWRDOMAIN(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CWRDOMAIN_SHIFT)) & NETC_IERB_ECAR_CWRDOMAIN_MASK)

#define NETC_IERB_ECAR_CWRSNP_MASK               (0x4000U)
#define NETC_IERB_ECAR_CWRSNP_SHIFT              (14U)
#define NETC_IERB_ECAR_CWRSNP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CWRSNP_SHIFT)) & NETC_IERB_ECAR_CWRSNP_MASK)

#define NETC_IERB_ECAR_CBD_RDCACHE_MASK          (0xF0000U)
#define NETC_IERB_ECAR_CBD_RDCACHE_SHIFT         (16U)
#define NETC_IERB_ECAR_CBD_RDCACHE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CBD_RDCACHE_SHIFT)) & NETC_IERB_ECAR_CBD_RDCACHE_MASK)

#define NETC_IERB_ECAR_CBD_RDDOMAIN_MASK         (0x300000U)
#define NETC_IERB_ECAR_CBD_RDDOMAIN_SHIFT        (20U)
#define NETC_IERB_ECAR_CBD_RDDOMAIN(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CBD_RDDOMAIN_SHIFT)) & NETC_IERB_ECAR_CBD_RDDOMAIN_MASK)

#define NETC_IERB_ECAR_CBD_RDSNP_MASK            (0x400000U)
#define NETC_IERB_ECAR_CBD_RDSNP_SHIFT           (22U)
#define NETC_IERB_ECAR_CBD_RDSNP(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CBD_RDSNP_SHIFT)) & NETC_IERB_ECAR_CBD_RDSNP_MASK)

#define NETC_IERB_ECAR_CRDCACHE_MASK             (0xF000000U)
#define NETC_IERB_ECAR_CRDCACHE_SHIFT            (24U)
#define NETC_IERB_ECAR_CRDCACHE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CRDCACHE_SHIFT)) & NETC_IERB_ECAR_CRDCACHE_MASK)

#define NETC_IERB_ECAR_CRDDOMAIN_MASK            (0x30000000U)
#define NETC_IERB_ECAR_CRDDOMAIN_SHIFT           (28U)
#define NETC_IERB_ECAR_CRDDOMAIN(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CRDDOMAIN_SHIFT)) & NETC_IERB_ECAR_CRDDOMAIN_MASK)

#define NETC_IERB_ECAR_CRDSNP_MASK               (0x40000000U)
#define NETC_IERB_ECAR_CRDSNP_SHIFT              (30U)
#define NETC_IERB_ECAR_CRDSNP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ECAR_CRDSNP_SHIFT)) & NETC_IERB_ECAR_CRDSNP_MASK)
/*! @} */

/* The count of NETC_IERB_ECAR */
#define NETC_IERB_ECAR_COUNT                     (2U)

/*! @name EAMQR - ENETC 0 access management qualifier register..ENETC 1 access management qualifier register */
/*! @{ */

#define NETC_IERB_EAMQR_ARQOS_MASK               (0xF0000U)
#define NETC_IERB_EAMQR_ARQOS_SHIFT              (16U)
#define NETC_IERB_EAMQR_ARQOS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EAMQR_ARQOS_SHIFT)) & NETC_IERB_EAMQR_ARQOS_MASK)

#define NETC_IERB_EAMQR_AWQOS_MASK               (0xF00000U)
#define NETC_IERB_EAMQR_AWQOS_SHIFT              (20U)
#define NETC_IERB_EAMQR_AWQOS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EAMQR_AWQOS_SHIFT)) & NETC_IERB_EAMQR_AWQOS_MASK)

#define NETC_IERB_EAMQR_BMT_MASK                 (0x80000000U)
#define NETC_IERB_EAMQR_BMT_SHIFT                (31U)
#define NETC_IERB_EAMQR_BMT(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EAMQR_BMT_SHIFT)) & NETC_IERB_EAMQR_BMT_MASK)
/*! @} */

/* The count of NETC_IERB_EAMQR */
#define NETC_IERB_EAMQR_COUNT                    (2U)

/*! @name EBLPR - ENETC 0 boot loader parameter register 0..ENETC 1 boot loader parameter register 1 */
/*! @{ */

#define NETC_IERB_EBLPR_PARAM_VAL_MASK           (0xFFFFFFFFU)
#define NETC_IERB_EBLPR_PARAM_VAL_SHIFT          (0U)
#define NETC_IERB_EBLPR_PARAM_VAL(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EBLPR_PARAM_VAL_SHIFT)) & NETC_IERB_EBLPR_PARAM_VAL_MASK)
/*! @} */

/* The count of NETC_IERB_EBLPR */
#define NETC_IERB_EBLPR_COUNT                    (2U)

/* The count of NETC_IERB_EBLPR */
#define NETC_IERB_EBLPR_COUNT2                   (2U)

/*! @name ERXMBER - ENETC 0 receive memory buffer entitlement register..ENETC 1 receive memory buffer entitlement register */
/*! @{ */

#define NETC_IERB_ERXMBER_AMOUNT_MASK            (0xFFFFFFU)
#define NETC_IERB_ERXMBER_AMOUNT_SHIFT           (0U)
#define NETC_IERB_ERXMBER_AMOUNT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ERXMBER_AMOUNT_SHIFT)) & NETC_IERB_ERXMBER_AMOUNT_MASK)
/*! @} */

/* The count of NETC_IERB_ERXMBER */
#define NETC_IERB_ERXMBER_COUNT                  (2U)

/*! @name ERXMBLR - ENETC 0 receive memory buffer limit register..ENETC 1 receive memory buffer limit register */
/*! @{ */

#define NETC_IERB_ERXMBLR_LIMIT_MASK             (0xFFFFFFU)
#define NETC_IERB_ERXMBLR_LIMIT_SHIFT            (0U)
#define NETC_IERB_ERXMBLR_LIMIT(x)               (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ERXMBLR_LIMIT_SHIFT)) & NETC_IERB_ERXMBLR_LIMIT_MASK)
/*! @} */

/* The count of NETC_IERB_ERXMBLR */
#define NETC_IERB_ERXMBLR_COUNT                  (2U)

/*! @name ETXHPTBCR - ENETC 0 transmit high priority tier byte credit register..ENETC 1 transmit high priority tier byte credit register */
/*! @{ */

#define NETC_IERB_ETXHPTBCR_BYTE_CREDIT_MASK     (0xFFFFU)
#define NETC_IERB_ETXHPTBCR_BYTE_CREDIT_SHIFT    (0U)
#define NETC_IERB_ETXHPTBCR_BYTE_CREDIT(x)       (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ETXHPTBCR_BYTE_CREDIT_SHIFT)) & NETC_IERB_ETXHPTBCR_BYTE_CREDIT_MASK)
/*! @} */

/* The count of NETC_IERB_ETXHPTBCR */
#define NETC_IERB_ETXHPTBCR_COUNT                (2U)

/*! @name ETXLPTBCR - ENETC 0 transmit low priority tier byte credit register..ENETC 1 transmit low priority tier byte credit register */
/*! @{ */

#define NETC_IERB_ETXLPTBCR_BYTE_CREDIT_MASK     (0xFFFFU)
#define NETC_IERB_ETXLPTBCR_BYTE_CREDIT_SHIFT    (0U)
#define NETC_IERB_ETXLPTBCR_BYTE_CREDIT(x)       (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ETXLPTBCR_BYTE_CREDIT_SHIFT)) & NETC_IERB_ETXLPTBCR_BYTE_CREDIT_MASK)
/*! @} */

/* The count of NETC_IERB_ETXLPTBCR */
#define NETC_IERB_ETXLPTBCR_COUNT                (2U)

/*! @name EHTMAR - ENETC 0 hash table memory allotment register..ENETC 1 hash table memory allotment register */
/*! @{ */

#define NETC_IERB_EHTMAR_NUM_WORDS_MASK          (0xFFFFU)
#define NETC_IERB_EHTMAR_NUM_WORDS_SHIFT         (0U)
#define NETC_IERB_EHTMAR_NUM_WORDS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EHTMAR_NUM_WORDS_SHIFT)) & NETC_IERB_EHTMAR_NUM_WORDS_MASK)

#define NETC_IERB_EHTMAR_MLOC_MASK               (0xC0000000U)
#define NETC_IERB_EHTMAR_MLOC_SHIFT              (30U)
#define NETC_IERB_EHTMAR_MLOC(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EHTMAR_MLOC_SHIFT)) & NETC_IERB_EHTMAR_MLOC_MASK)
/*! @} */

/* The count of NETC_IERB_EHTMAR */
#define NETC_IERB_EHTMAR_COUNT                   (2U)

/*! @name EITMAR - ENETC 0 index table memory allocation register..ENETC 1 index table memory allocation register */
/*! @{ */

#define NETC_IERB_EITMAR_NUM_WORDS_MASK          (0xFFFFU)
#define NETC_IERB_EITMAR_NUM_WORDS_SHIFT         (0U)
#define NETC_IERB_EITMAR_NUM_WORDS(x)            (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_EITMAR_NUM_WORDS_MASK)

#define NETC_IERB_EITMAR_MLOC_MASK               (0xC0000000U)
#define NETC_IERB_EITMAR_MLOC_SHIFT              (30U)
#define NETC_IERB_EITMAR_MLOC(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EITMAR_MLOC_SHIFT)) & NETC_IERB_EITMAR_MLOC_MASK)
/*! @} */

/* The count of NETC_IERB_EITMAR */
#define NETC_IERB_EITMAR_COUNT                   (2U)

/*! @name EIPFTMAR - ENETC 0 ingress port filter table memory allocation register..ENETC 1 ingress port filter table memory allocation register */
/*! @{ */

#define NETC_IERB_EIPFTMAR_ALLOC_MASK            (0xFFFFU)
#define NETC_IERB_EIPFTMAR_ALLOC_SHIFT           (0U)
#define NETC_IERB_EIPFTMAR_ALLOC(x)              (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EIPFTMAR_ALLOC_SHIFT)) & NETC_IERB_EIPFTMAR_ALLOC_MASK)
/*! @} */

/* The count of NETC_IERB_EIPFTMAR */
#define NETC_IERB_EIPFTMAR_COUNT                 (2U)

/*! @name ERPITMAR - ENETC 0 rate policer index table memory allocation register..ENETC 1 rate policer index table memory allocation register */
/*! @{ */

#define NETC_IERB_ERPITMAR_NUM_WORDS_MASK        (0xFFFFU)
#define NETC_IERB_ERPITMAR_NUM_WORDS_SHIFT       (0U)
#define NETC_IERB_ERPITMAR_NUM_WORDS(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ERPITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_ERPITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_ERPITMAR */
#define NETC_IERB_ERPITMAR_COUNT                 (2U)

/*! @name EISCITMAR - ENETC 0 ingress stream counter index table memory allocation register..ENETC 1 ingress stream counter index table memory allocation register */
/*! @{ */

#define NETC_IERB_EISCITMAR_NUM_WORDS_MASK       (0xFFFFU)
#define NETC_IERB_EISCITMAR_NUM_WORDS_SHIFT      (0U)
#define NETC_IERB_EISCITMAR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EISCITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_EISCITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_EISCITMAR */
#define NETC_IERB_EISCITMAR_COUNT                (2U)

/*! @name EISITMAR - ENETC 0 ingress stream index table memory allocation register..ENETC 1 ingress stream index table memory allocation register */
/*! @{ */

#define NETC_IERB_EISITMAR_NUM_WORDS_MASK        (0xFFFFU)
#define NETC_IERB_EISITMAR_NUM_WORDS_SHIFT       (0U)
#define NETC_IERB_EISITMAR_NUM_WORDS(x)          (((uint32_t)(((uint32_t)(x)) << NETC_IERB_EISITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_EISITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_EISITMAR */
#define NETC_IERB_EISITMAR_COUNT                 (2U)

/*! @name ESGIITMAR - ENETC 0 stream gate instance index table memory allocation register..ENETC 1 stream gate instance index table memory allocation register */
/*! @{ */

#define NETC_IERB_ESGIITMAR_NUM_WORDS_MASK       (0xFFFFU)
#define NETC_IERB_ESGIITMAR_NUM_WORDS_SHIFT      (0U)
#define NETC_IERB_ESGIITMAR_NUM_WORDS(x)         (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ESGIITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_ESGIITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_ESGIITMAR */
#define NETC_IERB_ESGIITMAR_COUNT                (2U)

/*! @name ESGCLITMAR - ENETC 0 stream gate control list index table memory allocation register..ENETC 1 stream gate control list index table memory allocation register */
/*! @{ */

#define NETC_IERB_ESGCLITMAR_NUM_WORDS_MASK      (0xFFFFU)
#define NETC_IERB_ESGCLITMAR_NUM_WORDS_SHIFT     (0U)
#define NETC_IERB_ESGCLITMAR_NUM_WORDS(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ESGCLITMAR_NUM_WORDS_SHIFT)) & NETC_IERB_ESGCLITMAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_ESGCLITMAR */
#define NETC_IERB_ESGCLITMAR_COUNT               (2U)

/*! @name ETGSTAR - ENETC 0 time gate scheduling table allocation register..ENETC 1 time gate scheduling table allocation register */
/*! @{ */

#define NETC_IERB_ETGSTAR_NUM_WORDS_MASK         (0xFFFU)
#define NETC_IERB_ETGSTAR_NUM_WORDS_SHIFT        (0U)
#define NETC_IERB_ETGSTAR_NUM_WORDS(x)           (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ETGSTAR_NUM_WORDS_SHIFT)) & NETC_IERB_ETGSTAR_NUM_WORDS_MASK)
/*! @} */

/* The count of NETC_IERB_ETGSTAR */
#define NETC_IERB_ETGSTAR_COUNT                  (2U)

/*! @name ETGSLR - ENETC 0 time gate scheduling lookahead register..ENETC 1 time gate scheduling lookahead register */
/*! @{ */

#define NETC_IERB_ETGSLR_MIN_LOOKAHEAD_MASK      (0xFFFFFU)
#define NETC_IERB_ETGSLR_MIN_LOOKAHEAD_SHIFT     (0U)
#define NETC_IERB_ETGSLR_MIN_LOOKAHEAD(x)        (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ETGSLR_MIN_LOOKAHEAD_SHIFT)) & NETC_IERB_ETGSLR_MIN_LOOKAHEAD_MASK)

#define NETC_IERB_ETGSLR_ZERO_LOOKAHEAD_MASK     (0x80000000U)
#define NETC_IERB_ETGSLR_ZERO_LOOKAHEAD_SHIFT    (31U)
/*! ZERO_LOOKAHEAD - Zero Lookahead
 *  0b0..Use MIN_LOOKAHEAD value
 *  0b1..If a gate control list is configured or when time specific departure is enabled on any traffic class
 *       (PTCaTSDR[TSDE] set to 1, where a corresponds to the traffic class number), use MIN_LOOKAHEAD value, otherwise
 *       use value of zero
 */
#define NETC_IERB_ETGSLR_ZERO_LOOKAHEAD(x)       (((uint32_t)(((uint32_t)(x)) << NETC_IERB_ETGSLR_ZERO_LOOKAHEAD_SHIFT)) & NETC_IERB_ETGSLR_ZERO_LOOKAHEAD_MASK)
/*! @} */

/* The count of NETC_IERB_ETGSLR */
#define NETC_IERB_ETGSLR_COUNT                   (2U)

/*! @name VAMQR - VSI 0 access management qualifier register */
/*! @{ */

#define NETC_IERB_VAMQR_ARQOS_MASK               (0xF0000U)
#define NETC_IERB_VAMQR_ARQOS_SHIFT              (16U)
#define NETC_IERB_VAMQR_ARQOS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_VAMQR_ARQOS_SHIFT)) & NETC_IERB_VAMQR_ARQOS_MASK)

#define NETC_IERB_VAMQR_AWQOS_MASK               (0xF00000U)
#define NETC_IERB_VAMQR_AWQOS_SHIFT              (20U)
#define NETC_IERB_VAMQR_AWQOS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_IERB_VAMQR_AWQOS_SHIFT)) & NETC_IERB_VAMQR_AWQOS_MASK)

#define NETC_IERB_VAMQR_BMT_MASK                 (0x80000000U)
#define NETC_IERB_VAMQR_BMT_SHIFT                (31U)
#define NETC_IERB_VAMQR_BMT(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_IERB_VAMQR_BMT_SHIFT)) & NETC_IERB_VAMQR_BMT_MASK)
/*! @} */

/* The count of NETC_IERB_VAMQR */
#define NETC_IERB_VAMQR_COUNT                    (1U)

/*! @name VBLPR - VSI 0 boot loader parameter register 0..VSI 0 boot loader parameter register 1 */
/*! @{ */

#define NETC_IERB_VBLPR_PARAM_VAL_MASK           (0xFFFFFFFFU)
#define NETC_IERB_VBLPR_PARAM_VAL_SHIFT          (0U)
#define NETC_IERB_VBLPR_PARAM_VAL(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_VBLPR_PARAM_VAL_SHIFT)) & NETC_IERB_VBLPR_PARAM_VAL_MASK)
/*! @} */

/* The count of NETC_IERB_VBLPR */
#define NETC_IERB_VBLPR_COUNT                    (1U)

/* The count of NETC_IERB_VBLPR */
#define NETC_IERB_VBLPR_COUNT2                   (2U)

/*! @name VPMAR0 - VSI 0 primary MAC address register 0 */
/*! @{ */

#define NETC_IERB_VPMAR0_MAC_ADDR_MASK           (0xFFFFFFFFU)
#define NETC_IERB_VPMAR0_MAC_ADDR_SHIFT          (0U)
#define NETC_IERB_VPMAR0_MAC_ADDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_VPMAR0_MAC_ADDR_SHIFT)) & NETC_IERB_VPMAR0_MAC_ADDR_MASK)
/*! @} */

/* The count of NETC_IERB_VPMAR0 */
#define NETC_IERB_VPMAR0_COUNT                   (1U)

/*! @name VPMAR1 - VSI 0 primary MAC address register 1 */
/*! @{ */

#define NETC_IERB_VPMAR1_MAC_ADDR_MASK           (0xFFFFU)
#define NETC_IERB_VPMAR1_MAC_ADDR_SHIFT          (0U)
#define NETC_IERB_VPMAR1_MAC_ADDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_IERB_VPMAR1_MAC_ADDR_SHIFT)) & NETC_IERB_VPMAR1_MAC_ADDR_MASK)
/*! @} */

/* The count of NETC_IERB_VPMAR1 */
#define NETC_IERB_VPMAR1_COUNT                   (1U)


/*!
 * @}
 */ /* end of group NETC_IERB_Register_Masks */


/* NETC_IERB - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral NETC_IERB base address */
  #define NETC_IERB_BASE                           (0x70800000u)
  /** Peripheral NETC_IERB base address */
  #define NETC_IERB_BASE_NS                        (0x60800000u)
  /** Peripheral NETC_IERB base pointer */
  #define NETC_IERB                                ((NETC_IERB_Type *)NETC_IERB_BASE)
  /** Peripheral NETC_IERB base pointer */
  #define NETC_IERB_NS                             ((NETC_IERB_Type *)NETC_IERB_BASE_NS)
  /** Array initializer of NETC_IERB peripheral base addresses */
  #define NETC_IERB_BASE_ADDRS                     { NETC_IERB_BASE }
  /** Array initializer of NETC_IERB peripheral base pointers */
  #define NETC_IERB_BASE_PTRS                      { NETC_IERB }
  /** Array initializer of NETC_IERB peripheral base addresses */
  #define NETC_IERB_BASE_ADDRS_NS                  { NETC_IERB_BASE_NS }
  /** Array initializer of NETC_IERB peripheral base pointers */
  #define NETC_IERB_BASE_PTRS_NS                   { NETC_IERB_NS }
#else
  /** Peripheral NETC_IERB base address */
  #define NETC_IERB_BASE                           (0x60800000u)
  /** Peripheral NETC_IERB base pointer */
  #define NETC_IERB                                ((NETC_IERB_Type *)NETC_IERB_BASE)
  /** Array initializer of NETC_IERB peripheral base addresses */
  #define NETC_IERB_BASE_ADDRS                     { NETC_IERB_BASE }
  /** Array initializer of NETC_IERB peripheral base pointers */
  #define NETC_IERB_BASE_PTRS                      { NETC_IERB }
#endif

/*!
 * @}
 */ /* end of group NETC_IERB_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_PORT Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_PORT_Peripheral_Access_Layer NETC_PORT Peripheral Access Layer
 * @{
 */

/** NETC_PORT - Register Layout Typedef */
typedef struct {
  __I  uint32_t PCAPR;                             /**< Port capability register, offset: 0x0 */
  __I  uint32_t PMCAPR;                            /**< Port MAC capability register, offset: 0x4 */
  __I  uint32_t PIOCAPR;                           /**< Port I/O capability register, offset: 0x8 */
       uint8_t RESERVED_0[4];
  __IO uint32_t PCR;                               /**< Port configuration register, offset: 0x10 */
       uint8_t RESERVED_1[12];
  __IO uint32_t PMAR0;                             /**< Port MAC address register 0, offset: 0x20 */
  __IO uint32_t PMAR1;                             /**< Port MAC address register 1, offset: 0x24 */
       uint8_t RESERVED_2[40];
  __IO uint32_t PTAR;                              /**< Port TPID acceptance register, offset: 0x50 */
  __IO uint32_t PQOSMR;                            /**< Port QoS mode register, offset: 0x54 */
       uint8_t RESERVED_3[8];
  __I  uint32_t PQOR;                              /**< Port Queue Operational register, offset: 0x60, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_4[28];
  __IO uint32_t PPCR;                              /**< Port parser configuration register, offset: 0x80 */
  __IO uint32_t PIPFCR;                            /**< Port ingress port filter configuration register, offset: 0x84 */
       uint8_t RESERVED_5[24];
  __IO uint32_t PSGCR;                             /**< Port stream gate configuration register, offset: 0xA0 */
       uint8_t RESERVED_6[92];
  __IO uint32_t POR;                               /**< Port operational register, offset: 0x100 */
  __I  uint32_t PSR;                               /**< Port status register, offset: 0x104 */
  __IO uint32_t PRXSDUOR;                          /**< Port receive SDU overhead register, offset: 0x108 */
  __IO uint32_t PTXSDUOR;                          /**< Port transmit SDU overhead register, offset: 0x10C */
  __IO uint32_t PTGSCR;                            /**< Port time gate scheduling control register, offset: 0x110 */
  __I  uint32_t PTGAGLSR;                          /**< Port time gate scheduling admin gate list status register, offset: 0x114 */
  __I  uint32_t PTGAGLLR;                          /**< Port time gate scheduling admin gate list length register, offset: 0x118 */
  __I  uint32_t PTGOGLLR;                          /**< Port time gating operational gate list length register, offset: 0x11C */
  __IO uint32_t PTGSATOR;                          /**< Port time gate scheduling advance time offset register, offset: 0x120, available only on: ENETC0_PORT, ENETC1_PORT (missing on SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4) */
  __I  uint32_t PTGSHAR;                           /**< Port time gate scheduling hold advance register, offset: 0x124, available only on: ENETC0_PORT, SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3 (missing on ENETC1_PORT, SW0_PORT4) */
  __I  uint32_t PTGSRAR;                           /**< Port time gate scheduling release advance register, offset: 0x128, available only on: ENETC0_PORT, SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3 (missing on ENETC1_PORT, SW0_PORT4) */
  __IO uint32_t PTGSHCR;                           /**< Port time gate scheduling hold configuration register, offset: 0x12C, available only on: ENETC0_PORT, SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3 (missing on ENETC1_PORT, SW0_PORT4) */
       uint8_t RESERVED_7[4];
  __IO uint32_t PFPCR;                             /**< Port frame preemption configuration register, offset: 0x134, available only on: ENETC0_PORT, SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3 (missing on ENETC1_PORT, SW0_PORT4) */
  __IO uint32_t PDGSR;                             /**< Port default gate state register, offset: 0x138 */
       uint8_t RESERVED_8[132];
  __I  uint32_t PRXDCR;                            /**< Port Rx discard count register, offset: 0x1C0 */
       uint8_t RESERVED_9[4];
  __IO uint32_t PRXDCRR0;                          /**< Port Rx discard count reason register 0, offset: 0x1C8 */
  __IO uint32_t PRXDCRR1;                          /**< Port Rx discard count reason register 1, offset: 0x1CC */
       uint8_t RESERVED_10[16];
  __I  uint32_t PTXDCR;                            /**< Port Tx discard count register, offset: 0x1E0, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_11[4];
  __IO uint32_t PTXDCRR0;                          /**< Port Tx discard count reason register 0, offset: 0x1E8, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
  __IO uint32_t PTXDCRR1;                          /**< Port Tx discard count reason register 1, offset: 0x1EC, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_12[16];
  struct {                                         /* offset: 0x200, array step: 0x20 */
    __I  uint32_t PTGSTCSR;                          /**< Port time gate scheduling traffic class 0 status register..Port time gate scheduling traffic class 7 status register, array offset: 0x200, array step: 0x20 */
         uint8_t RESERVED_0[4];
    __IO uint32_t PTCTMSDUR;                         /**< Port traffic class 0 transmit maximum SDU register..Port traffic class 7 transmit maximum SDU register, array offset: 0x208, array step: 0x20 */
         uint8_t RESERVED_1[4];
    __IO uint32_t PTCCBSR0;                          /**< Port transmit traffic class 0 credit based shaper register 0..Port transmit traffic class 7 credit based shaper register 0, array offset: 0x210, array step: 0x20 */
    __IO uint32_t PTCCBSR1;                          /**< Port traffic class 0 credit based shaper register 1..Port traffic class 7 credit based shaper register 1, array offset: 0x214, array step: 0x20 */
         uint8_t RESERVED_2[8];
  } TCT_NUM[8];
       uint8_t RESERVED_13[256];
  __IO uint32_t PBPMCR0;                           /**< Port buffer pool mapping configuration register 0, offset: 0x400, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
  __IO uint32_t PBPMCR1;                           /**< Port buffer pool mapping configuration register 1, offset: 0x404, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_14[48];
  __IO uint32_t PPCPDEIMR;                         /**< Port PCP DEI mapping register, offset: 0x438 */
       uint8_t RESERVED_15[4];
  __IO uint32_t PMCR;                              /**< Port mirror configuration register, offset: 0x440, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_16[12];
  __IO uint32_t PCTFCR;                            /**< Port cut through forwarding configuration register, offset: 0x450, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3 (missing on ENETC0_PORT, ENETC1_PORT, SW0_PORT4) */
       uint8_t RESERVED_17[4];
  __IO uint32_t PLANIDCR;                          /**< Port LANID configuration register, offset: 0x458, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_18[4];
  __IO uint32_t PISIDCR;                           /**< Port ingress stream identification configuration register, offset: 0x460 */
  __IO uint32_t PFMCR;                             /**< Port frame modification configuration register, offset: 0x464, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_19[8];
  __IO uint32_t PIPV2QMR0;                         /**< Port IPV to queue mapping register 0, offset: 0x470, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_20[60];
  __I  uint32_t PTCMINLR;                          /**< Port time capture minimum latency register, offset: 0x4B0, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
  __I  uint32_t PTCMAXLR;                          /**< Port time capture maximum latency register, offset: 0x4B4, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_21[72];
  __IO uint32_t BPCR;                              /**< Bridge port configuration register, offset: 0x500, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_22[12];
  __IO uint32_t BPDVR;                             /**< Bridge port default VLAN register, offset: 0x510, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_23[12];
  __IO uint32_t BPSTGSR;                           /**< Bridge port spanning tree group state register, offset: 0x520, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_24[4];
  __IO uint32_t BPSCR0;                            /**< Bridge port storm control register 0, offset: 0x528, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
  __IO uint32_t BPSCR1;                            /**< Bridge port storm control register 1, offset: 0x52C, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
  __I  uint32_t BPOR;                              /**< Bridge port operational register, offset: 0x530, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_25[76];
  __I  uint32_t BPDCR;                             /**< Bridge port discard count register, offset: 0x580, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
       uint8_t RESERVED_26[4];
  __IO uint32_t BPDCRR0;                           /**< Bridge port discard count reason register 0, offset: 0x588, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
  __IO uint32_t BPDCRR1;                           /**< Bridge port discard count reason register 1, offset: 0x58C, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
  __IO uint32_t BPMLFSR;                           /**< Bridge port MAC learning failure status register, offset: 0x590, available only on: SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 (missing on ENETC0_PORT, ENETC1_PORT) */
} NETC_PORT_Type;

/* ----------------------------------------------------------------------------
   -- NETC_PORT Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_PORT_Register_Masks NETC_PORT Register Masks
 * @{
 */

/*! @name PCAPR - Port capability register */
/*! @{ */

#define NETC_PORT_PCAPR_LINK_TYPE_MASK           (0x10U)
#define NETC_PORT_PCAPR_LINK_TYPE_SHIFT          (4U)
#define NETC_PORT_PCAPR_LINK_TYPE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCAPR_LINK_TYPE_SHIFT)) & NETC_PORT_PCAPR_LINK_TYPE_MASK)

#define NETC_PORT_PCAPR_NUM_TC_MASK              (0xF000U)
#define NETC_PORT_PCAPR_NUM_TC_SHIFT             (12U)
#define NETC_PORT_PCAPR_NUM_TC(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCAPR_NUM_TC_SHIFT)) & NETC_PORT_PCAPR_NUM_TC_MASK)

#define NETC_PORT_PCAPR_NUM_Q_MASK               (0xF0000U)
#define NETC_PORT_PCAPR_NUM_Q_SHIFT              (16U)
#define NETC_PORT_PCAPR_NUM_Q(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCAPR_NUM_Q_SHIFT)) & NETC_PORT_PCAPR_NUM_Q_MASK)

#define NETC_PORT_PCAPR_NUM_CG_MASK              (0xF000000U)
#define NETC_PORT_PCAPR_NUM_CG_SHIFT             (24U)
#define NETC_PORT_PCAPR_NUM_CG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCAPR_NUM_CG_SHIFT)) & NETC_PORT_PCAPR_NUM_CG_MASK)

#define NETC_PORT_PCAPR_TGS_MASK                 (0x10000000U)
#define NETC_PORT_PCAPR_TGS_SHIFT                (28U)
/*! TGS - Time Gate Scheduling */
#define NETC_PORT_PCAPR_TGS(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCAPR_TGS_SHIFT)) & NETC_PORT_PCAPR_TGS_MASK)

#define NETC_PORT_PCAPR_CBS_MASK                 (0x20000000U)
#define NETC_PORT_PCAPR_CBS_SHIFT                (29U)
/*! CBS - Credit Based Shaping */
#define NETC_PORT_PCAPR_CBS(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCAPR_CBS_SHIFT)) & NETC_PORT_PCAPR_CBS_MASK)
/*! @} */

/*! @name PMCAPR - Port MAC capability register */
/*! @{ */

#define NETC_PORT_PMCAPR_MAC_VAR_MASK            (0x7U)
#define NETC_PORT_PMCAPR_MAC_VAR_SHIFT           (0U)
/*! MAC_VAR - MAC Variant */
#define NETC_PORT_PMCAPR_MAC_VAR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCAPR_MAC_VAR_SHIFT)) & NETC_PORT_PMCAPR_MAC_VAR_MASK)

#define NETC_PORT_PMCAPR_EFPAD_MASK              (0x30U)
#define NETC_PORT_PMCAPR_EFPAD_SHIFT             (4U)
#define NETC_PORT_PMCAPR_EFPAD(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCAPR_EFPAD_SHIFT)) & NETC_PORT_PMCAPR_EFPAD_MASK)

#define NETC_PORT_PMCAPR_PIPG_MASK               (0x40U)
#define NETC_PORT_PMCAPR_PIPG_SHIFT              (6U)
#define NETC_PORT_PMCAPR_PIPG(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCAPR_PIPG_SHIFT)) & NETC_PORT_PMCAPR_PIPG_MASK)

#define NETC_PORT_PMCAPR_HD_MASK                 (0x100U)
#define NETC_PORT_PMCAPR_HD_SHIFT                (8U)
#define NETC_PORT_PMCAPR_HD(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCAPR_HD_SHIFT)) & NETC_PORT_PMCAPR_HD_MASK)

#define NETC_PORT_PMCAPR_FP_MASK                 (0x600U)
#define NETC_PORT_PMCAPR_FP_SHIFT                (9U)
/*! FP - Indicates if frame preemption is supported */
#define NETC_PORT_PMCAPR_FP(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCAPR_FP_SHIFT)) & NETC_PORT_PMCAPR_FP_MASK)

#define NETC_PORT_PMCAPR_MIN_MPDU_MASK           (0x1000U)
#define NETC_PORT_PMCAPR_MIN_MPDU_SHIFT          (12U)
/*! MIN_MPDU - Minimum MAC Protocol Data Unit (PDU) size check */
#define NETC_PORT_PMCAPR_MIN_MPDU(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCAPR_MIN_MPDU_SHIFT)) & NETC_PORT_PMCAPR_MIN_MPDU_MASK)

#define NETC_PORT_PMCAPR_MII_PROT_MASK           (0xF000000U)
#define NETC_PORT_PMCAPR_MII_PROT_SHIFT          (24U)
/*! MII_PROT - Indicates the MII protocol supported */
#define NETC_PORT_PMCAPR_MII_PROT(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCAPR_MII_PROT_SHIFT)) & NETC_PORT_PMCAPR_MII_PROT_MASK)
/*! @} */

/*! @name PIOCAPR - Port I/O capability register */
/*! @{ */

#define NETC_PORT_PIOCAPR_PCS_PROT_MASK          (0xFFFFU)
#define NETC_PORT_PIOCAPR_PCS_PROT_SHIFT         (0U)
/*! PCS_PROT - PCS protocols supported */
#define NETC_PORT_PIOCAPR_PCS_PROT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIOCAPR_PCS_PROT_SHIFT)) & NETC_PORT_PIOCAPR_PCS_PROT_MASK)

#define NETC_PORT_PIOCAPR_IO_VAR_MASK            (0xF000000U)
#define NETC_PORT_PIOCAPR_IO_VAR_SHIFT           (24U)
/*! IO_VAR - IO Variants supported */
#define NETC_PORT_PIOCAPR_IO_VAR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIOCAPR_IO_VAR_SHIFT)) & NETC_PORT_PIOCAPR_IO_VAR_MASK)

#define NETC_PORT_PIOCAPR_EMDIO_MASK             (0x10000000U)
#define NETC_PORT_PIOCAPR_EMDIO_SHIFT            (28U)
#define NETC_PORT_PIOCAPR_EMDIO(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIOCAPR_EMDIO_SHIFT)) & NETC_PORT_PIOCAPR_EMDIO_MASK)

#define NETC_PORT_PIOCAPR_REVMII_RATE_MASK       (0x40000000U)
#define NETC_PORT_PIOCAPR_REVMII_RATE_SHIFT      (30U)
/*! REVMII_RATE - RevMII MII rate */
#define NETC_PORT_PIOCAPR_REVMII_RATE(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIOCAPR_REVMII_RATE_SHIFT)) & NETC_PORT_PIOCAPR_REVMII_RATE_MASK)

#define NETC_PORT_PIOCAPR_REVMII_MASK            (0x80000000U)
#define NETC_PORT_PIOCAPR_REVMII_SHIFT           (31U)
/*! REVMII - Reverse Mode Device Configuration */
#define NETC_PORT_PIOCAPR_REVMII(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIOCAPR_REVMII_SHIFT)) & NETC_PORT_PIOCAPR_REVMII_MASK)
/*! @} */

/*! @name PCR - Port configuration register */
/*! @{ */

#define NETC_PORT_PCR_HDR_FMT_MASK               (0x1U)
#define NETC_PORT_PCR_HDR_FMT_SHIFT              (0U)
#define NETC_PORT_PCR_HDR_FMT(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCR_HDR_FMT_SHIFT)) & NETC_PORT_PCR_HDR_FMT_MASK)

#define NETC_PORT_PCR_L2DOSE_MASK                (0x10U)
#define NETC_PORT_PCR_L2DOSE_SHIFT               (4U)
#define NETC_PORT_PCR_L2DOSE(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCR_L2DOSE_SHIFT)) & NETC_PORT_PCR_L2DOSE_MASK)

#define NETC_PORT_PCR_TIMER_CS_MASK              (0x100U)
#define NETC_PORT_PCR_TIMER_CS_SHIFT             (8U)
#define NETC_PORT_PCR_TIMER_CS(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCR_TIMER_CS_SHIFT)) & NETC_PORT_PCR_TIMER_CS_MASK)

#define NETC_PORT_PCR_PSPEED_MASK                (0x3FFF0000U)
#define NETC_PORT_PCR_PSPEED_SHIFT               (16U)
#define NETC_PORT_PCR_PSPEED(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCR_PSPEED_SHIFT)) & NETC_PORT_PCR_PSPEED_MASK)
/*! @} */

/*! @name PMAR0 - Port MAC address register 0 */
/*! @{ */

#define NETC_PORT_PMAR0_PRIM_MAC_ADDR_MASK       (0xFFFFFFFFU)
#define NETC_PORT_PMAR0_PRIM_MAC_ADDR_SHIFT      (0U)
#define NETC_PORT_PMAR0_PRIM_MAC_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMAR0_PRIM_MAC_ADDR_SHIFT)) & NETC_PORT_PMAR0_PRIM_MAC_ADDR_MASK)
/*! @} */

/*! @name PMAR1 - Port MAC address register 1 */
/*! @{ */

#define NETC_PORT_PMAR1_PRIM_MAC_ADDR_MASK       (0xFFFFU)
#define NETC_PORT_PMAR1_PRIM_MAC_ADDR_SHIFT      (0U)
#define NETC_PORT_PMAR1_PRIM_MAC_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMAR1_PRIM_MAC_ADDR_SHIFT)) & NETC_PORT_PMAR1_PRIM_MAC_ADDR_MASK)
/*! @} */

/*! @name PTAR - Port TPID acceptance register */
/*! @{ */

#define NETC_PORT_PTAR_OVTPIDL_MASK              (0xFU)
#define NETC_PORT_PTAR_OVTPIDL_SHIFT             (0U)
#define NETC_PORT_PTAR_OVTPIDL(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTAR_OVTPIDL_SHIFT)) & NETC_PORT_PTAR_OVTPIDL_MASK)

#define NETC_PORT_PTAR_IVTPIDL_MASK              (0xF0U)
#define NETC_PORT_PTAR_IVTPIDL_SHIFT             (4U)
#define NETC_PORT_PTAR_IVTPIDL(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTAR_IVTPIDL_SHIFT)) & NETC_PORT_PTAR_IVTPIDL_MASK)
/*! @} */

/*! @name PQOSMR - Port QoS mode register */
/*! @{ */

#define NETC_PORT_PQOSMR_VS_MASK                 (0x1U)
#define NETC_PORT_PQOSMR_VS_SHIFT                (0U)
#define NETC_PORT_PQOSMR_VS(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOSMR_VS_SHIFT)) & NETC_PORT_PQOSMR_VS_MASK)

#define NETC_PORT_PQOSMR_VE_MASK                 (0x2U)
#define NETC_PORT_PQOSMR_VE_SHIFT                (1U)
#define NETC_PORT_PQOSMR_VE(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOSMR_VE_SHIFT)) & NETC_PORT_PQOSMR_VE_MASK)

#define NETC_PORT_PQOSMR_DDR_MASK                (0xCU)
#define NETC_PORT_PQOSMR_DDR_SHIFT               (2U)
#define NETC_PORT_PQOSMR_DDR(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOSMR_DDR_SHIFT)) & NETC_PORT_PQOSMR_DDR_MASK)

#define NETC_PORT_PQOSMR_DIPV_MASK               (0x70U)
#define NETC_PORT_PQOSMR_DIPV_SHIFT              (4U)
#define NETC_PORT_PQOSMR_DIPV(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOSMR_DIPV_SHIFT)) & NETC_PORT_PQOSMR_DIPV_MASK)

#define NETC_PORT_PQOSMR_VQMP_MASK               (0xF0000U)
#define NETC_PORT_PQOSMR_VQMP_SHIFT              (16U)
/*! VQMP - Mapping profile index */
#define NETC_PORT_PQOSMR_VQMP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOSMR_VQMP_SHIFT)) & NETC_PORT_PQOSMR_VQMP_MASK)

#define NETC_PORT_PQOSMR_QVMP_MASK               (0xF00000U)
#define NETC_PORT_PQOSMR_QVMP_SHIFT              (20U)
/*! QVMP - Mapping profile index */
#define NETC_PORT_PQOSMR_QVMP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOSMR_QVMP_SHIFT)) & NETC_PORT_PQOSMR_QVMP_MASK)
/*! @} */

/*! @name PQOR - Port Queue Operational register */
/*! @{ */

#define NETC_PORT_PQOR_Q0S_MASK                  (0x1U)
#define NETC_PORT_PQOR_Q0S_SHIFT                 (0U)
#define NETC_PORT_PQOR_Q0S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q0S_SHIFT)) & NETC_PORT_PQOR_Q0S_MASK)

#define NETC_PORT_PQOR_Q1S_MASK                  (0x2U)
#define NETC_PORT_PQOR_Q1S_SHIFT                 (1U)
#define NETC_PORT_PQOR_Q1S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q1S_SHIFT)) & NETC_PORT_PQOR_Q1S_MASK)

#define NETC_PORT_PQOR_Q2S_MASK                  (0x4U)
#define NETC_PORT_PQOR_Q2S_SHIFT                 (2U)
#define NETC_PORT_PQOR_Q2S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q2S_SHIFT)) & NETC_PORT_PQOR_Q2S_MASK)

#define NETC_PORT_PQOR_Q3S_MASK                  (0x8U)
#define NETC_PORT_PQOR_Q3S_SHIFT                 (3U)
#define NETC_PORT_PQOR_Q3S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q3S_SHIFT)) & NETC_PORT_PQOR_Q3S_MASK)

#define NETC_PORT_PQOR_Q4S_MASK                  (0x10U)
#define NETC_PORT_PQOR_Q4S_SHIFT                 (4U)
#define NETC_PORT_PQOR_Q4S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q4S_SHIFT)) & NETC_PORT_PQOR_Q4S_MASK)

#define NETC_PORT_PQOR_Q5S_MASK                  (0x20U)
#define NETC_PORT_PQOR_Q5S_SHIFT                 (5U)
#define NETC_PORT_PQOR_Q5S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q5S_SHIFT)) & NETC_PORT_PQOR_Q5S_MASK)

#define NETC_PORT_PQOR_Q6S_MASK                  (0x40U)
#define NETC_PORT_PQOR_Q6S_SHIFT                 (6U)
#define NETC_PORT_PQOR_Q6S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q6S_SHIFT)) & NETC_PORT_PQOR_Q6S_MASK)

#define NETC_PORT_PQOR_Q7S_MASK                  (0x80U)
#define NETC_PORT_PQOR_Q7S_SHIFT                 (7U)
#define NETC_PORT_PQOR_Q7S(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PQOR_Q7S_SHIFT)) & NETC_PORT_PQOR_Q7S_MASK)
/*! @} */

/*! @name PPCR - Port parser configuration register */
/*! @{ */

#define NETC_PORT_PPCR_L1PFS_MASK                (0x3EU)
#define NETC_PORT_PPCR_L1PFS_SHIFT               (1U)
#define NETC_PORT_PPCR_L1PFS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCR_L1PFS_SHIFT)) & NETC_PORT_PPCR_L1PFS_MASK)

#define NETC_PORT_PPCR_L2PFS_MASK                (0x3E00U)
#define NETC_PORT_PPCR_L2PFS_SHIFT               (9U)
#define NETC_PORT_PPCR_L2PFS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCR_L2PFS_SHIFT)) & NETC_PORT_PPCR_L2PFS_MASK)

#define NETC_PORT_PPCR_L3HFP_MASK                (0x10000U)
#define NETC_PORT_PPCR_L3HFP_SHIFT               (16U)
/*! L3HFP - L3 header fields present
 *  0b0..No L3 header present. Indicates to the parser of not parsing the L3 header. Parsing in this case would go
 *       as far as the L2 header regardless of whether or not there is an L3 header in the frame. This option
 *       should not be used if there are any table lookup entries that contain L3/L4 key fields that could be matched
 *       against a frame.
 *  0b1..Parse L3 header if present in the frame.
 */
#define NETC_PORT_PPCR_L3HFP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCR_L3HFP_SHIFT)) & NETC_PORT_PPCR_L3HFP_MASK)

#define NETC_PORT_PPCR_L3PFS_MASK                (0x3E0000U)
#define NETC_PORT_PPCR_L3PFS_SHIFT               (17U)
/*! L3PFS - L3 payload fields size in bytes */
#define NETC_PORT_PPCR_L3PFS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCR_L3PFS_SHIFT)) & NETC_PORT_PPCR_L3PFS_MASK)

#define NETC_PORT_PPCR_L4HFP_MASK                (0x1000000U)
#define NETC_PORT_PPCR_L4HFP_SHIFT               (24U)
/*! L4HFP - L4 Header fields present
 *  0b0..No L4 header present. Indicates to the parser of not parsing the L4 header. Parsing in this case would go
 *       as far as the L3 header if configured to parse it (L3HFP=1) regardless of whether or not there is an L4
 *       header in the frame. This option should not be used if there are any table lookup entries that contain L4
 *       key fields that could be matched against a frame.
 *  0b1..Parse L4 header if present in the frame
 */
#define NETC_PORT_PPCR_L4HFP(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCR_L4HFP_SHIFT)) & NETC_PORT_PPCR_L4HFP_MASK)

#define NETC_PORT_PPCR_L4PFS_MASK                (0x3E000000U)
#define NETC_PORT_PPCR_L4PFS_SHIFT               (25U)
/*! L4PFS - L4 payload fields size in bytes */
#define NETC_PORT_PPCR_L4PFS(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCR_L4PFS_SHIFT)) & NETC_PORT_PPCR_L4PFS_MASK)
/*! @} */

/*! @name PIPFCR - Port ingress port filter configuration register */
/*! @{ */

#define NETC_PORT_PIPFCR_EN_MASK                 (0x1U)
#define NETC_PORT_PIPFCR_EN_SHIFT                (0U)
#define NETC_PORT_PIPFCR_EN(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPFCR_EN_SHIFT)) & NETC_PORT_PIPFCR_EN_MASK)
/*! @} */

/*! @name PSGCR - Port stream gate configuration register */
/*! @{ */

#define NETC_PORT_PSGCR_PDELAY_MASK              (0xFFFFFFU)
#define NETC_PORT_PSGCR_PDELAY_SHIFT             (0U)
#define NETC_PORT_PSGCR_PDELAY(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PSGCR_PDELAY_SHIFT)) & NETC_PORT_PSGCR_PDELAY_MASK)

#define NETC_PORT_PSGCR_OGC_MASK                 (0x80000000U)
#define NETC_PORT_PSGCR_OGC_SHIFT                (31U)
/*! OGC - Stream Gate Open Gate Check mode */
#define NETC_PORT_PSGCR_OGC(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PSGCR_OGC_SHIFT)) & NETC_PORT_PSGCR_OGC_MASK)
/*! @} */

/*! @name POR - Port operational register */
/*! @{ */

#define NETC_PORT_POR_TXDIS_MASK                 (0x1U)
#define NETC_PORT_POR_TXDIS_SHIFT                (0U)
/*! TXDIS - Tx Disable.
 *  0b0..Tx path is enabled
 *  0b1..Tx path is disabled.
 */
#define NETC_PORT_POR_TXDIS(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_POR_TXDIS_SHIFT)) & NETC_PORT_POR_TXDIS_MASK)

#define NETC_PORT_POR_RXDIS_MASK                 (0x2U)
#define NETC_PORT_POR_RXDIS_SHIFT                (1U)
/*! RXDIS - Rx Disable.
 *  0b0..Rx path is enabled.
 *  0b1..Rx path is disabled.
 */
#define NETC_PORT_POR_RXDIS(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_POR_RXDIS_SHIFT)) & NETC_PORT_POR_RXDIS_MASK)
/*! @} */

/*! @name PSR - Port status register */
/*! @{ */

#define NETC_PORT_PSR_TX_BUSY_MASK               (0x1U)
#define NETC_PORT_PSR_TX_BUSY_SHIFT              (0U)
/*! TX_BUSY - Transmit busy.
 *  0b0..Idle
 *  0b1..Busy
 */
#define NETC_PORT_PSR_TX_BUSY(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PSR_TX_BUSY_SHIFT)) & NETC_PORT_PSR_TX_BUSY_MASK)

#define NETC_PORT_PSR_RX_BUSY_MASK               (0x2U)
#define NETC_PORT_PSR_RX_BUSY_SHIFT              (1U)
/*! RX_BUSY - Receive busy.
 *  0b0..Idle
 *  0b1..Busy
 */
#define NETC_PORT_PSR_RX_BUSY(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PSR_RX_BUSY_SHIFT)) & NETC_PORT_PSR_RX_BUSY_MASK)
/*! @} */

/*! @name PRXSDUOR - Port receive SDU overhead register */
/*! @{ */

#define NETC_PORT_PRXSDUOR_PPDU_BCO_MASK         (0x1FU)
#define NETC_PORT_PRXSDUOR_PPDU_BCO_SHIFT        (0U)
/*! PPDU_BCO - PPDU Byte count overhead */
#define NETC_PORT_PRXSDUOR_PPDU_BCO(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXSDUOR_PPDU_BCO_SHIFT)) & NETC_PORT_PRXSDUOR_PPDU_BCO_MASK)

#define NETC_PORT_PRXSDUOR_MACSEC_BCO_MASK       (0x1F00U)
#define NETC_PORT_PRXSDUOR_MACSEC_BCO_SHIFT      (8U)
/*! MACSEC_BCO - MACSec byte count overhead */
#define NETC_PORT_PRXSDUOR_MACSEC_BCO(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXSDUOR_MACSEC_BCO_SHIFT)) & NETC_PORT_PRXSDUOR_MACSEC_BCO_MASK)
/*! @} */

/*! @name PTXSDUOR - Port transmit SDU overhead register */
/*! @{ */

#define NETC_PORT_PTXSDUOR_PPDU_BCO_MASK         (0x1FU)
#define NETC_PORT_PTXSDUOR_PPDU_BCO_SHIFT        (0U)
/*! PPDU_BCO - PPDU Byte count overhead */
#define NETC_PORT_PTXSDUOR_PPDU_BCO(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXSDUOR_PPDU_BCO_SHIFT)) & NETC_PORT_PTXSDUOR_PPDU_BCO_MASK)

#define NETC_PORT_PTXSDUOR_MACSEC_BCO_MASK       (0x1F00U)
#define NETC_PORT_PTXSDUOR_MACSEC_BCO_SHIFT      (8U)
/*! MACSEC_BCO - MACSec byte count overhead */
#define NETC_PORT_PTXSDUOR_MACSEC_BCO(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXSDUOR_MACSEC_BCO_SHIFT)) & NETC_PORT_PTXSDUOR_MACSEC_BCO_MASK)
/*! @} */

/*! @name PTGSCR - Port time gate scheduling control register */
/*! @{ */

#define NETC_PORT_PTGSCR_TGE_MASK                (0x80000000U)
#define NETC_PORT_PTGSCR_TGE_SHIFT               (31U)
/*! TGE - Time Gating Enable
 *  0b0..Time gating disabled.
 *  0b1..Time gating enabled.
 */
#define NETC_PORT_PTGSCR_TGE(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGSCR_TGE_SHIFT)) & NETC_PORT_PTGSCR_TGE_MASK)
/*! @} */

/*! @name PTGAGLSR - Port time gate scheduling admin gate list status register */
/*! @{ */

#define NETC_PORT_PTGAGLSR_TG_MASK               (0x1U)
#define NETC_PORT_PTGAGLSR_TG_SHIFT              (0U)
#define NETC_PORT_PTGAGLSR_TG(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGAGLSR_TG_SHIFT)) & NETC_PORT_PTGAGLSR_TG_MASK)

#define NETC_PORT_PTGAGLSR_CFG_PEND_MASK         (0x2U)
#define NETC_PORT_PTGAGLSR_CFG_PEND_SHIFT        (1U)
#define NETC_PORT_PTGAGLSR_CFG_PEND(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGAGLSR_CFG_PEND_SHIFT)) & NETC_PORT_PTGAGLSR_CFG_PEND_MASK)
/*! @} */

/*! @name PTGAGLLR - Port time gate scheduling admin gate list length register */
/*! @{ */

#define NETC_PORT_PTGAGLLR_ADMIN_GATE_LIST_LENGTH_MASK (0xFFFFU)
#define NETC_PORT_PTGAGLLR_ADMIN_GATE_LIST_LENGTH_SHIFT (0U)
#define NETC_PORT_PTGAGLLR_ADMIN_GATE_LIST_LENGTH(x) (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGAGLLR_ADMIN_GATE_LIST_LENGTH_SHIFT)) & NETC_PORT_PTGAGLLR_ADMIN_GATE_LIST_LENGTH_MASK)
/*! @} */

/*! @name PTGOGLLR - Port time gating operational gate list length register */
/*! @{ */

#define NETC_PORT_PTGOGLLR_OPER_GATE_LIST_LENGTH_MASK (0xFFFFU)
#define NETC_PORT_PTGOGLLR_OPER_GATE_LIST_LENGTH_SHIFT (0U)
#define NETC_PORT_PTGOGLLR_OPER_GATE_LIST_LENGTH(x) (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGOGLLR_OPER_GATE_LIST_LENGTH_SHIFT)) & NETC_PORT_PTGOGLLR_OPER_GATE_LIST_LENGTH_MASK)
/*! @} */

/*! @name PTGSATOR - Port time gate scheduling advance time offset register */
/*! @{ */

#define NETC_PORT_PTGSATOR_ADV_TIME_OFFSET_MASK  (0xFFFFU)
#define NETC_PORT_PTGSATOR_ADV_TIME_OFFSET_SHIFT (0U)
#define NETC_PORT_PTGSATOR_ADV_TIME_OFFSET(x)    (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGSATOR_ADV_TIME_OFFSET_SHIFT)) & NETC_PORT_PTGSATOR_ADV_TIME_OFFSET_MASK)
/*! @} */

/*! @name PTGSHAR - Port time gate scheduling hold advance register */
/*! @{ */

#define NETC_PORT_PTGSHAR_HOLDADVANCE_MASK       (0xFFFFU)
#define NETC_PORT_PTGSHAR_HOLDADVANCE_SHIFT      (0U)
#define NETC_PORT_PTGSHAR_HOLDADVANCE(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGSHAR_HOLDADVANCE_SHIFT)) & NETC_PORT_PTGSHAR_HOLDADVANCE_MASK)
/*! @} */

/*! @name PTGSRAR - Port time gate scheduling release advance register */
/*! @{ */

#define NETC_PORT_PTGSRAR_RELEASEADVANCE_MASK    (0xFFFFU)
#define NETC_PORT_PTGSRAR_RELEASEADVANCE_SHIFT   (0U)
#define NETC_PORT_PTGSRAR_RELEASEADVANCE(x)      (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGSRAR_RELEASEADVANCE_SHIFT)) & NETC_PORT_PTGSRAR_RELEASEADVANCE_MASK)
/*! @} */

/*! @name PTGSHCR - Port time gate scheduling hold configuration register */
/*! @{ */

#define NETC_PORT_PTGSHCR_HOLD_SKEW_MASK         (0xFFFFFU)
#define NETC_PORT_PTGSHCR_HOLD_SKEW_SHIFT        (0U)
#define NETC_PORT_PTGSHCR_HOLD_SKEW(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGSHCR_HOLD_SKEW_SHIFT)) & NETC_PORT_PTGSHCR_HOLD_SKEW_MASK)
/*! @} */

/*! @name PFPCR - Port frame preemption configuration register */
/*! @{ */

#define NETC_PORT_PFPCR_FPE_TC0_MASK             (0x1U)
#define NETC_PORT_PFPCR_FPE_TC0_SHIFT            (0U)
#define NETC_PORT_PFPCR_FPE_TC0(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC0_SHIFT)) & NETC_PORT_PFPCR_FPE_TC0_MASK)

#define NETC_PORT_PFPCR_FPE_TC1_MASK             (0x2U)
#define NETC_PORT_PFPCR_FPE_TC1_SHIFT            (1U)
#define NETC_PORT_PFPCR_FPE_TC1(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC1_SHIFT)) & NETC_PORT_PFPCR_FPE_TC1_MASK)

#define NETC_PORT_PFPCR_FPE_TC2_MASK             (0x4U)
#define NETC_PORT_PFPCR_FPE_TC2_SHIFT            (2U)
#define NETC_PORT_PFPCR_FPE_TC2(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC2_SHIFT)) & NETC_PORT_PFPCR_FPE_TC2_MASK)

#define NETC_PORT_PFPCR_FPE_TC3_MASK             (0x8U)
#define NETC_PORT_PFPCR_FPE_TC3_SHIFT            (3U)
#define NETC_PORT_PFPCR_FPE_TC3(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC3_SHIFT)) & NETC_PORT_PFPCR_FPE_TC3_MASK)

#define NETC_PORT_PFPCR_FPE_TC4_MASK             (0x10U)
#define NETC_PORT_PFPCR_FPE_TC4_SHIFT            (4U)
#define NETC_PORT_PFPCR_FPE_TC4(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC4_SHIFT)) & NETC_PORT_PFPCR_FPE_TC4_MASK)

#define NETC_PORT_PFPCR_FPE_TC5_MASK             (0x20U)
#define NETC_PORT_PFPCR_FPE_TC5_SHIFT            (5U)
#define NETC_PORT_PFPCR_FPE_TC5(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC5_SHIFT)) & NETC_PORT_PFPCR_FPE_TC5_MASK)

#define NETC_PORT_PFPCR_FPE_TC6_MASK             (0x40U)
#define NETC_PORT_PFPCR_FPE_TC6_SHIFT            (6U)
#define NETC_PORT_PFPCR_FPE_TC6(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC6_SHIFT)) & NETC_PORT_PFPCR_FPE_TC6_MASK)

#define NETC_PORT_PFPCR_FPE_TC7_MASK             (0x80U)
#define NETC_PORT_PFPCR_FPE_TC7_SHIFT            (7U)
#define NETC_PORT_PFPCR_FPE_TC7(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFPCR_FPE_TC7_SHIFT)) & NETC_PORT_PFPCR_FPE_TC7_MASK)
/*! @} */

/*! @name PDGSR - Port default gate state register */
/*! @{ */

#define NETC_PORT_PDGSR_DGS_TC0_MASK             (0x1U)
#define NETC_PORT_PDGSR_DGS_TC0_SHIFT            (0U)
/*! DGS_TC0
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC0(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC0_SHIFT)) & NETC_PORT_PDGSR_DGS_TC0_MASK)

#define NETC_PORT_PDGSR_DGS_TC1_MASK             (0x2U)
#define NETC_PORT_PDGSR_DGS_TC1_SHIFT            (1U)
/*! DGS_TC1
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC1(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC1_SHIFT)) & NETC_PORT_PDGSR_DGS_TC1_MASK)

#define NETC_PORT_PDGSR_DGS_TC2_MASK             (0x4U)
#define NETC_PORT_PDGSR_DGS_TC2_SHIFT            (2U)
/*! DGS_TC2
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC2(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC2_SHIFT)) & NETC_PORT_PDGSR_DGS_TC2_MASK)

#define NETC_PORT_PDGSR_DGS_TC3_MASK             (0x8U)
#define NETC_PORT_PDGSR_DGS_TC3_SHIFT            (3U)
/*! DGS_TC3
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC3(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC3_SHIFT)) & NETC_PORT_PDGSR_DGS_TC3_MASK)

#define NETC_PORT_PDGSR_DGS_TC4_MASK             (0x10U)
#define NETC_PORT_PDGSR_DGS_TC4_SHIFT            (4U)
/*! DGS_TC4
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC4(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC4_SHIFT)) & NETC_PORT_PDGSR_DGS_TC4_MASK)

#define NETC_PORT_PDGSR_DGS_TC5_MASK             (0x20U)
#define NETC_PORT_PDGSR_DGS_TC5_SHIFT            (5U)
/*! DGS_TC5
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC5(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC5_SHIFT)) & NETC_PORT_PDGSR_DGS_TC5_MASK)

#define NETC_PORT_PDGSR_DGS_TC6_MASK             (0x40U)
#define NETC_PORT_PDGSR_DGS_TC6_SHIFT            (6U)
/*! DGS_TC6
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC6(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC6_SHIFT)) & NETC_PORT_PDGSR_DGS_TC6_MASK)

#define NETC_PORT_PDGSR_DGS_TC7_MASK             (0x80U)
#define NETC_PORT_PDGSR_DGS_TC7_SHIFT            (7U)
/*! DGS_TC7
 *  0b0..Closed
 *  0b1..Open
 */
#define NETC_PORT_PDGSR_DGS_TC7(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PDGSR_DGS_TC7_SHIFT)) & NETC_PORT_PDGSR_DGS_TC7_MASK)
/*! @} */

/*! @name PRXDCR - Port Rx discard count register */
/*! @{ */

#define NETC_PORT_PRXDCR_COUNT_MASK              (0xFFFFFFFFU)
#define NETC_PORT_PRXDCR_COUNT_SHIFT             (0U)
#define NETC_PORT_PRXDCR_COUNT(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCR_COUNT_SHIFT)) & NETC_PORT_PRXDCR_COUNT_MASK)
/*! @} */

/*! @name PRXDCRR0 - Port Rx discard count reason register 0 */
/*! @{ */

#define NETC_PORT_PRXDCRR0_PCDR_MASK             (0x1U)
#define NETC_PORT_PRXDCRR0_PCDR_SHIFT            (0U)
#define NETC_PORT_PRXDCRR0_PCDR(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_PCDR_SHIFT)) & NETC_PORT_PRXDCRR0_PCDR_MASK)

#define NETC_PORT_PRXDCRR0_SMREDR_MASK           (0x2U)
#define NETC_PORT_PRXDCRR0_SMREDR_SHIFT          (1U)
#define NETC_PORT_PRXDCRR0_SMREDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_SMREDR_SHIFT)) & NETC_PORT_PRXDCRR0_SMREDR_MASK)

#define NETC_PORT_PRXDCRR0_RXDISDR_MASK          (0x4U)
#define NETC_PORT_PRXDCRR0_RXDISDR_SHIFT         (2U)
#define NETC_PORT_PRXDCRR0_RXDISDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_RXDISDR_SHIFT)) & NETC_PORT_PRXDCRR0_RXDISDR_MASK)

#define NETC_PORT_PRXDCRR0_IPFDR_MASK            (0x8U)
#define NETC_PORT_PRXDCRR0_IPFDR_SHIFT           (3U)
#define NETC_PORT_PRXDCRR0_IPFDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_IPFDR_SHIFT)) & NETC_PORT_PRXDCRR0_IPFDR_MASK)

#define NETC_PORT_PRXDCRR0_RPDR_MASK             (0x10U)
#define NETC_PORT_PRXDCRR0_RPDR_SHIFT            (4U)
#define NETC_PORT_PRXDCRR0_RPDR(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_RPDR_SHIFT)) & NETC_PORT_PRXDCRR0_RPDR_MASK)

#define NETC_PORT_PRXDCRR0_ISFDR_MASK            (0x20U)
#define NETC_PORT_PRXDCRR0_ISFDR_SHIFT           (5U)
#define NETC_PORT_PRXDCRR0_ISFDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_ISFDR_SHIFT)) & NETC_PORT_PRXDCRR0_ISFDR_MASK)

#define NETC_PORT_PRXDCRR0_SGCDR_MASK            (0x40U)
#define NETC_PORT_PRXDCRR0_SGCDR_SHIFT           (6U)
#define NETC_PORT_PRXDCRR0_SGCDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_SGCDR_SHIFT)) & NETC_PORT_PRXDCRR0_SGCDR_MASK)

#define NETC_PORT_PRXDCRR0_SGOEDR_MASK           (0x80U)
#define NETC_PORT_PRXDCRR0_SGOEDR_SHIFT          (7U)
#define NETC_PORT_PRXDCRR0_SGOEDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_SGOEDR_SHIFT)) & NETC_PORT_PRXDCRR0_SGOEDR_MASK)

#define NETC_PORT_PRXDCRR0_MSDUEDR_MASK          (0x100U)
#define NETC_PORT_PRXDCRR0_MSDUEDR_SHIFT         (8U)
#define NETC_PORT_PRXDCRR0_MSDUEDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_MSDUEDR_SHIFT)) & NETC_PORT_PRXDCRR0_MSDUEDR_MASK)

#define NETC_PORT_PRXDCRR0_FMMEDR_MASK           (0x200U)
#define NETC_PORT_PRXDCRR0_FMMEDR_SHIFT          (9U)
#define NETC_PORT_PRXDCRR0_FMMEDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_FMMEDR_SHIFT)) & NETC_PORT_PRXDCRR0_FMMEDR_MASK)

#define NETC_PORT_PRXDCRR0_CMDR_MASK             (0x400U)
#define NETC_PORT_PRXDCRR0_CMDR_SHIFT            (10U)
#define NETC_PORT_PRXDCRR0_CMDR(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_CMDR_SHIFT)) & NETC_PORT_PRXDCRR0_CMDR_MASK)

#define NETC_PORT_PRXDCRR0_ITEDR_MASK            (0x800U)
#define NETC_PORT_PRXDCRR0_ITEDR_SHIFT           (11U)
#define NETC_PORT_PRXDCRR0_ITEDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_ITEDR_SHIFT)) & NETC_PORT_PRXDCRR0_ITEDR_MASK)

#define NETC_PORT_PRXDCRR0_ECCEDR_MASK           (0x1000U)
#define NETC_PORT_PRXDCRR0_ECCEDR_SHIFT          (12U)
#define NETC_PORT_PRXDCRR0_ECCEDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_ECCEDR_SHIFT)) & NETC_PORT_PRXDCRR0_ECCEDR_MASK)

#define NETC_PORT_PRXDCRR0_SIFDR_MASK            (0x2000U)
#define NETC_PORT_PRXDCRR0_SIFDR_SHIFT           (13U)
#define NETC_PORT_PRXDCRR0_SIFDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_SIFDR_SHIFT)) & NETC_PORT_PRXDCRR0_SIFDR_MASK)

#define NETC_PORT_PRXDCRR0_L2DOSDR_MASK          (0x4000U)
#define NETC_PORT_PRXDCRR0_L2DOSDR_SHIFT         (14U)
#define NETC_PORT_PRXDCRR0_L2DOSDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_L2DOSDR_SHIFT)) & NETC_PORT_PRXDCRR0_L2DOSDR_MASK)

#define NETC_PORT_PRXDCRR0_NODESTDR_MASK         (0x20000U)
#define NETC_PORT_PRXDCRR0_NODESTDR_SHIFT        (17U)
/*! NODESTDR - No Destination Discard Reason */
#define NETC_PORT_PRXDCRR0_NODESTDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR0_NODESTDR_SHIFT)) & NETC_PORT_PRXDCRR0_NODESTDR_MASK)
/*! @} */

/*! @name PRXDCRR1 - Port Rx discard count reason register 1 */
/*! @{ */

#define NETC_PORT_PRXDCRR1_ENTRYID_MASK          (0xFFFFU)
#define NETC_PORT_PRXDCRR1_ENTRYID_SHIFT         (0U)
#define NETC_PORT_PRXDCRR1_ENTRYID(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR1_ENTRYID_SHIFT)) & NETC_PORT_PRXDCRR1_ENTRYID_MASK)

#define NETC_PORT_PRXDCRR1_TT_MASK               (0xF0000000U)
#define NETC_PORT_PRXDCRR1_TT_SHIFT              (28U)
#define NETC_PORT_PRXDCRR1_TT(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PRXDCRR1_TT_SHIFT)) & NETC_PORT_PRXDCRR1_TT_MASK)
/*! @} */

/*! @name PTXDCR - Port Tx discard count register */
/*! @{ */

#define NETC_PORT_PTXDCR_COUNT_MASK              (0xFFFFFFFFU)
#define NETC_PORT_PTXDCR_COUNT_SHIFT             (0U)
#define NETC_PORT_PTXDCR_COUNT(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCR_COUNT_SHIFT)) & NETC_PORT_PTXDCR_COUNT_MASK)
/*! @} */

/*! @name PTXDCRR0 - Port Tx discard count reason register 0 */
/*! @{ */

#define NETC_PORT_PTXDCRR0_TXDISDR_MASK          (0x1U)
#define NETC_PORT_PTXDCRR0_TXDISDR_SHIFT         (0U)
#define NETC_PORT_PTXDCRR0_TXDISDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_TXDISDR_SHIFT)) & NETC_PORT_PTXDCRR0_TXDISDR_MASK)

#define NETC_PORT_PTXDCRR0_ECCEDR_MASK           (0x2U)
#define NETC_PORT_PTXDCRR0_ECCEDR_SHIFT          (1U)
/*! ECCEDR - ECC Error Discard Reason */
#define NETC_PORT_PTXDCRR0_ECCEDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_ECCEDR_SHIFT)) & NETC_PORT_PTXDCRR0_ECCEDR_MASK)

#define NETC_PORT_PTXDCRR0_PEDR_MASK             (0x4U)
#define NETC_PORT_PTXDCRR0_PEDR_SHIFT            (2U)
/*! PEDR - Parity Error Discard Reason */
#define NETC_PORT_PTXDCRR0_PEDR(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_PEDR_SHIFT)) & NETC_PORT_PTXDCRR0_PEDR_MASK)

#define NETC_PORT_PTXDCRR0_TGSFTLDR_MASK         (0x10U)
#define NETC_PORT_PTXDCRR0_TGSFTLDR_SHIFT        (4U)
#define NETC_PORT_PTXDCRR0_TGSFTLDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_TGSFTLDR_SHIFT)) & NETC_PORT_PTXDCRR0_TGSFTLDR_MASK)

#define NETC_PORT_PTXDCRR0_FMMDR_MASK            (0x20U)
#define NETC_PORT_PTXDCRR0_FMMDR_SHIFT           (5U)
#define NETC_PORT_PTXDCRR0_FMMDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_FMMDR_SHIFT)) & NETC_PORT_PTXDCRR0_FMMDR_MASK)

#define NETC_PORT_PTXDCRR0_TXDISEDR_MASK         (0x40U)
#define NETC_PORT_PTXDCRR0_TXDISEDR_SHIFT        (6U)
#define NETC_PORT_PTXDCRR0_TXDISEDR(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_TXDISEDR_SHIFT)) & NETC_PORT_PTXDCRR0_TXDISEDR_MASK)

#define NETC_PORT_PTXDCRR0_MSDUEDR_MASK          (0x80U)
#define NETC_PORT_PTXDCRR0_MSDUEDR_SHIFT         (7U)
#define NETC_PORT_PTXDCRR0_MSDUEDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_MSDUEDR_SHIFT)) & NETC_PORT_PTXDCRR0_MSDUEDR_MASK)

#define NETC_PORT_PTXDCRR0_QCONGDR_MASK          (0x100U)
#define NETC_PORT_PTXDCRR0_QCONGDR_SHIFT         (8U)
#define NETC_PORT_PTXDCRR0_QCONGDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_QCONGDR_SHIFT)) & NETC_PORT_PTXDCRR0_QCONGDR_MASK)

#define NETC_PORT_PTXDCRR0_ITEDR_MASK            (0x200U)
#define NETC_PORT_PTXDCRR0_ITEDR_SHIFT           (9U)
#define NETC_PORT_PTXDCRR0_ITEDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_ITEDR_SHIFT)) & NETC_PORT_PTXDCRR0_ITEDR_MASK)

#define NETC_PORT_PTXDCRR0_INVEQDR_MASK          (0x400U)
#define NETC_PORT_PTXDCRR0_INVEQDR_SHIFT         (10U)
#define NETC_PORT_PTXDCRR0_INVEQDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_INVEQDR_SHIFT)) & NETC_PORT_PTXDCRR0_INVEQDR_MASK)

#define NETC_PORT_PTXDCRR0_SQRTNSQDR_MASK        (0x800U)
#define NETC_PORT_PTXDCRR0_SQRTNSQDR_SHIFT       (11U)
#define NETC_PORT_PTXDCRR0_SQRTNSQDR(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_SQRTNSQDR_SHIFT)) & NETC_PORT_PTXDCRR0_SQRTNSQDR_MASK)

#define NETC_PORT_PTXDCRR0_SQRRDR_MASK           (0x2000U)
#define NETC_PORT_PTXDCRR0_SQRRDR_SHIFT          (13U)
#define NETC_PORT_PTXDCRR0_SQRRDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_SQRRDR_SHIFT)) & NETC_PORT_PTXDCRR0_SQRRDR_MASK)

#define NETC_PORT_PTXDCRR0_SQRDDR_MASK           (0x4000U)
#define NETC_PORT_PTXDCRR0_SQRDDR_SHIFT          (14U)
#define NETC_PORT_PTXDCRR0_SQRDDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_SQRDDR_SHIFT)) & NETC_PORT_PTXDCRR0_SQRDDR_MASK)

#define NETC_PORT_PTXDCRR0_SMREDR_MASK           (0x8000U)
#define NETC_PORT_PTXDCRR0_SMREDR_SHIFT          (15U)
#define NETC_PORT_PTXDCRR0_SMREDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR0_SMREDR_SHIFT)) & NETC_PORT_PTXDCRR0_SMREDR_MASK)
/*! @} */

/*! @name PTXDCRR1 - Port Tx discard count reason register 1 */
/*! @{ */

#define NETC_PORT_PTXDCRR1_ENTRYID_MASK          (0xFFFFU)
#define NETC_PORT_PTXDCRR1_ENTRYID_SHIFT         (0U)
#define NETC_PORT_PTXDCRR1_ENTRYID(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR1_ENTRYID_SHIFT)) & NETC_PORT_PTXDCRR1_ENTRYID_MASK)

#define NETC_PORT_PTXDCRR1_TT_MASK               (0xF0000000U)
#define NETC_PORT_PTXDCRR1_TT_SHIFT              (28U)
#define NETC_PORT_PTXDCRR1_TT(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTXDCRR1_TT_SHIFT)) & NETC_PORT_PTXDCRR1_TT_MASK)
/*! @} */

/*! @name PTGSTCSR - Port time gate scheduling traffic class 0 status register..Port time gate scheduling traffic class 7 status register */
/*! @{ */

#define NETC_PORT_PTGSTCSR_LH_STATE_MASK         (0x10000U)
#define NETC_PORT_PTGSTCSR_LH_STATE_SHIFT        (16U)
#define NETC_PORT_PTGSTCSR_LH_STATE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTGSTCSR_LH_STATE_SHIFT)) & NETC_PORT_PTGSTCSR_LH_STATE_MASK)
/*! @} */

/* The count of NETC_PORT_PTGSTCSR */
#define NETC_PORT_PTGSTCSR_COUNT                 (8U)

/*! @name PTCTMSDUR - Port traffic class 0 transmit maximum SDU register..Port traffic class 7 transmit maximum SDU register */
/*! @{ */

#define NETC_PORT_PTCTMSDUR_MAXSDU_MASK          (0xFFFFU)
#define NETC_PORT_PTCTMSDUR_MAXSDU_SHIFT         (0U)
#define NETC_PORT_PTCTMSDUR_MAXSDU(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCTMSDUR_MAXSDU_SHIFT)) & NETC_PORT_PTCTMSDUR_MAXSDU_MASK)

#define NETC_PORT_PTCTMSDUR_SDU_TYPE_MASK        (0x30000U)
#define NETC_PORT_PTCTMSDUR_SDU_TYPE_SHIFT       (16U)
#define NETC_PORT_PTCTMSDUR_SDU_TYPE(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCTMSDUR_SDU_TYPE_SHIFT)) & NETC_PORT_PTCTMSDUR_SDU_TYPE_MASK)

#define NETC_PORT_PTCTMSDUR_SF_MAXSDU_DIS_MASK   (0x1000000U)
#define NETC_PORT_PTCTMSDUR_SF_MAXSDU_DIS_SHIFT  (24U)
#define NETC_PORT_PTCTMSDUR_SF_MAXSDU_DIS(x)     (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCTMSDUR_SF_MAXSDU_DIS_SHIFT)) & NETC_PORT_PTCTMSDUR_SF_MAXSDU_DIS_MASK)
/*! @} */

/* The count of NETC_PORT_PTCTMSDUR */
#define NETC_PORT_PTCTMSDUR_COUNT                (8U)

/*! @name PTCCBSR0 - Port transmit traffic class 0 credit based shaper register 0..Port transmit traffic class 7 credit based shaper register 0 */
/*! @{ */

#define NETC_PORT_PTCCBSR0_BW_MASK               (0x7FU)
#define NETC_PORT_PTCCBSR0_BW_SHIFT              (0U)
/*! BW - Bandwidth */
#define NETC_PORT_PTCCBSR0_BW(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCCBSR0_BW_SHIFT)) & NETC_PORT_PTCCBSR0_BW_MASK)

#define NETC_PORT_PTCCBSR0_CBSE_MASK             (0x80000000U)
#define NETC_PORT_PTCCBSR0_CBSE_SHIFT            (31U)
/*! CBSE - Credit Based Shaper Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_PORT_PTCCBSR0_CBSE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCCBSR0_CBSE_SHIFT)) & NETC_PORT_PTCCBSR0_CBSE_MASK)
/*! @} */

/* The count of NETC_PORT_PTCCBSR0 */
#define NETC_PORT_PTCCBSR0_COUNT                 (8U)

/*! @name PTCCBSR1 - Port traffic class 0 credit based shaper register 1..Port traffic class 7 credit based shaper register 1 */
/*! @{ */

#define NETC_PORT_PTCCBSR1_HI_CREDIT_MASK        (0xFFFFFFFFU)
#define NETC_PORT_PTCCBSR1_HI_CREDIT_SHIFT       (0U)
/*! HI_CREDIT - hicredit (in credit units) */
#define NETC_PORT_PTCCBSR1_HI_CREDIT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCCBSR1_HI_CREDIT_SHIFT)) & NETC_PORT_PTCCBSR1_HI_CREDIT_MASK)
/*! @} */

/* The count of NETC_PORT_PTCCBSR1 */
#define NETC_PORT_PTCCBSR1_COUNT                 (8U)

/*! @name PBPMCR0 - Port buffer pool mapping configuration register 0 */
/*! @{ */

#define NETC_PORT_PBPMCR0_IPV0_INDEX_MASK        (0xFFU)
#define NETC_PORT_PBPMCR0_IPV0_INDEX_SHIFT       (0U)
#define NETC_PORT_PBPMCR0_IPV0_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR0_IPV0_INDEX_SHIFT)) & NETC_PORT_PBPMCR0_IPV0_INDEX_MASK)

#define NETC_PORT_PBPMCR0_IPV1_INDEX_MASK        (0xFF00U)
#define NETC_PORT_PBPMCR0_IPV1_INDEX_SHIFT       (8U)
#define NETC_PORT_PBPMCR0_IPV1_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR0_IPV1_INDEX_SHIFT)) & NETC_PORT_PBPMCR0_IPV1_INDEX_MASK)

#define NETC_PORT_PBPMCR0_IPV2_INDEX_MASK        (0xFF0000U)
#define NETC_PORT_PBPMCR0_IPV2_INDEX_SHIFT       (16U)
#define NETC_PORT_PBPMCR0_IPV2_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR0_IPV2_INDEX_SHIFT)) & NETC_PORT_PBPMCR0_IPV2_INDEX_MASK)

#define NETC_PORT_PBPMCR0_IPV3_INDEX_MASK        (0xFF000000U)
#define NETC_PORT_PBPMCR0_IPV3_INDEX_SHIFT       (24U)
#define NETC_PORT_PBPMCR0_IPV3_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR0_IPV3_INDEX_SHIFT)) & NETC_PORT_PBPMCR0_IPV3_INDEX_MASK)
/*! @} */

/*! @name PBPMCR1 - Port buffer pool mapping configuration register 1 */
/*! @{ */

#define NETC_PORT_PBPMCR1_IPV4_INDEX_MASK        (0xFFU)
#define NETC_PORT_PBPMCR1_IPV4_INDEX_SHIFT       (0U)
#define NETC_PORT_PBPMCR1_IPV4_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR1_IPV4_INDEX_SHIFT)) & NETC_PORT_PBPMCR1_IPV4_INDEX_MASK)

#define NETC_PORT_PBPMCR1_IPV5_INDEX_MASK        (0xFF00U)
#define NETC_PORT_PBPMCR1_IPV5_INDEX_SHIFT       (8U)
#define NETC_PORT_PBPMCR1_IPV5_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR1_IPV5_INDEX_SHIFT)) & NETC_PORT_PBPMCR1_IPV5_INDEX_MASK)

#define NETC_PORT_PBPMCR1_IPV6_INDEX_MASK        (0xFF0000U)
#define NETC_PORT_PBPMCR1_IPV6_INDEX_SHIFT       (16U)
#define NETC_PORT_PBPMCR1_IPV6_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR1_IPV6_INDEX_SHIFT)) & NETC_PORT_PBPMCR1_IPV6_INDEX_MASK)

#define NETC_PORT_PBPMCR1_IPV7_INDEX_MASK        (0xFF000000U)
#define NETC_PORT_PBPMCR1_IPV7_INDEX_SHIFT       (24U)
#define NETC_PORT_PBPMCR1_IPV7_INDEX(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PBPMCR1_IPV7_INDEX_SHIFT)) & NETC_PORT_PBPMCR1_IPV7_INDEX_MASK)
/*! @} */

/*! @name PPCPDEIMR - Port PCP DEI mapping register */
/*! @{ */

#define NETC_PORT_PPCPDEIMR_IPCPMP_MASK          (0xFU)
#define NETC_PORT_PPCPDEIMR_IPCPMP_SHIFT         (0U)
#define NETC_PORT_PPCPDEIMR_IPCPMP(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_IPCPMP_SHIFT)) & NETC_PORT_PPCPDEIMR_IPCPMP_MASK)

#define NETC_PORT_PPCPDEIMR_IPCPMPV_MASK         (0x80U)
#define NETC_PORT_PPCPDEIMR_IPCPMPV_SHIFT        (7U)
/*! IPCPMPV
 *  0b0..Ingress PCP to PCP Mapping Profile is not valid.
 *  0b1..Ingress frame modification of outer VLAN tag's PCP value is mapped to a new value based on IPCPMP instance
 */
#define NETC_PORT_PPCPDEIMR_IPCPMPV(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_IPCPMPV_SHIFT)) & NETC_PORT_PPCPDEIMR_IPCPMPV_MASK)

#define NETC_PORT_PPCPDEIMR_EPCPMP_MASK          (0xF00U)
#define NETC_PORT_PPCPDEIMR_EPCPMP_SHIFT         (8U)
#define NETC_PORT_PPCPDEIMR_EPCPMP(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_EPCPMP_SHIFT)) & NETC_PORT_PPCPDEIMR_EPCPMP_MASK)

#define NETC_PORT_PPCPDEIMR_EPCPMPV_MASK         (0x8000U)
#define NETC_PORT_PPCPDEIMR_EPCPMPV_SHIFT        (15U)
/*! EPCPMPV
 *  0b0..Egress PCP to PCP Mapping Profile is not valid.
 *  0b1..Egress frame modification of outer VLAN tag's PCP value is mapped to a new value based on EPCPMP instance.
 */
#define NETC_PORT_PPCPDEIMR_EPCPMPV(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_EPCPMPV_SHIFT)) & NETC_PORT_PPCPDEIMR_EPCPMPV_MASK)

#define NETC_PORT_PPCPDEIMR_DR0DEI_MASK          (0x10000U)
#define NETC_PORT_PPCPDEIMR_DR0DEI_SHIFT         (16U)
#define NETC_PORT_PPCPDEIMR_DR0DEI(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_DR0DEI_SHIFT)) & NETC_PORT_PPCPDEIMR_DR0DEI_MASK)

#define NETC_PORT_PPCPDEIMR_DR1DEI_MASK          (0x20000U)
#define NETC_PORT_PPCPDEIMR_DR1DEI_SHIFT         (17U)
#define NETC_PORT_PPCPDEIMR_DR1DEI(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_DR1DEI_SHIFT)) & NETC_PORT_PPCPDEIMR_DR1DEI_MASK)

#define NETC_PORT_PPCPDEIMR_DR2DEI_MASK          (0x40000U)
#define NETC_PORT_PPCPDEIMR_DR2DEI_SHIFT         (18U)
#define NETC_PORT_PPCPDEIMR_DR2DEI(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_DR2DEI_SHIFT)) & NETC_PORT_PPCPDEIMR_DR2DEI_MASK)

#define NETC_PORT_PPCPDEIMR_DR3DEI_MASK          (0x80000U)
#define NETC_PORT_PPCPDEIMR_DR3DEI_SHIFT         (19U)
#define NETC_PORT_PPCPDEIMR_DR3DEI(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_DR3DEI_SHIFT)) & NETC_PORT_PPCPDEIMR_DR3DEI_MASK)

#define NETC_PORT_PPCPDEIMR_DRME_MASK            (0x100000U)
#define NETC_PORT_PPCPDEIMR_DRME_SHIFT           (20U)
/*! DRME
 *  0b0..Preserve the DR value in the outer VLAN.
 *  0b1..Update DR value in the outer VLAN based on DEnDEI field.
 */
#define NETC_PORT_PPCPDEIMR_DRME(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PPCPDEIMR_DRME_SHIFT)) & NETC_PORT_PPCPDEIMR_DRME_MASK)
/*! @} */

/*! @name PMCR - Port mirror configuration register */
/*! @{ */

#define NETC_PORT_PMCR_IMIRE_MASK                (0x1U)
#define NETC_PORT_PMCR_IMIRE_SHIFT               (0U)
#define NETC_PORT_PMCR_IMIRE(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PMCR_IMIRE_SHIFT)) & NETC_PORT_PMCR_IMIRE_MASK)
/*! @} */

/*! @name PCTFCR - Port cut through forwarding configuration register */
/*! @{ */

#define NETC_PORT_PCTFCR_ICTS_MASK               (0x1U)
#define NETC_PORT_PCTFCR_ICTS_SHIFT              (0U)
#define NETC_PORT_PCTFCR_ICTS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCTFCR_ICTS_SHIFT)) & NETC_PORT_PCTFCR_ICTS_MASK)

#define NETC_PORT_PCTFCR_ECTS_MASK               (0x2U)
#define NETC_PORT_PCTFCR_ECTS_SHIFT              (1U)
#define NETC_PORT_PCTFCR_ECTS(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PCTFCR_ECTS_SHIFT)) & NETC_PORT_PCTFCR_ECTS_MASK)
/*! @} */

/*! @name PLANIDCR - Port LANID configuration register */
/*! @{ */

#define NETC_PORT_PLANIDCR_LANID_MASK            (0xFU)
#define NETC_PORT_PLANIDCR_LANID_SHIFT           (0U)
/*! LANID - LAN Identifier */
#define NETC_PORT_PLANIDCR_LANID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PLANIDCR_LANID_SHIFT)) & NETC_PORT_PLANIDCR_LANID_MASK)
/*! @} */

/*! @name PISIDCR - Port ingress stream identification configuration register */
/*! @{ */

#define NETC_PORT_PISIDCR_KCPAIR_MASK            (0x1U)
#define NETC_PORT_PISIDCR_KCPAIR_SHIFT           (0U)
#define NETC_PORT_PISIDCR_KCPAIR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PISIDCR_KCPAIR_SHIFT)) & NETC_PORT_PISIDCR_KCPAIR_MASK)

#define NETC_PORT_PISIDCR_KC0EN_MASK             (0x2U)
#define NETC_PORT_PISIDCR_KC0EN_SHIFT            (1U)
/*! KC0EN - Key Construction 0 Enable */
#define NETC_PORT_PISIDCR_KC0EN(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PISIDCR_KC0EN_SHIFT)) & NETC_PORT_PISIDCR_KC0EN_MASK)

#define NETC_PORT_PISIDCR_KC1EN_MASK             (0x4U)
#define NETC_PORT_PISIDCR_KC1EN_SHIFT            (2U)
/*! KC1EN - Key Construction 1 Enable */
#define NETC_PORT_PISIDCR_KC1EN(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PISIDCR_KC1EN_SHIFT)) & NETC_PORT_PISIDCR_KC1EN_MASK)

#define NETC_PORT_PISIDCR_ISEID_MASK             (0xFFFF0000U)
#define NETC_PORT_PISIDCR_ISEID_SHIFT            (16U)
#define NETC_PORT_PISIDCR_ISEID(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PISIDCR_ISEID_SHIFT)) & NETC_PORT_PISIDCR_ISEID_MASK)
/*! @} */

/*! @name PFMCR - Port frame modification configuration register */
/*! @{ */

#define NETC_PORT_PFMCR_FMMA_MASK                (0x1U)
#define NETC_PORT_PFMCR_FMMA_SHIFT               (0U)
/*! FMMA - Frame Modification Misconfiguration Action
 *  0b0..Discard the frame and counted against the port's Tx discard count register (PTXDCR) along with the
 *       setting of the Frame Modification Misconfiguration Discard Reason (FMMDR) flag to 1 in the port's Tx discard
 *       count reason register 0 (PTXDCRR0).
 *  0b1..Transmit the frame without performing any of the ingress and egress frame modification actions specified.
 */
#define NETC_PORT_PFMCR_FMMA(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PFMCR_FMMA_SHIFT)) & NETC_PORT_PFMCR_FMMA_MASK)
/*! @} */

/*! @name PIPV2QMR0 - Port IPV to queue mapping register 0 */
/*! @{ */

#define NETC_PORT_PIPV2QMR0_IPV0_Q_MASK          (0xFU)
#define NETC_PORT_PIPV2QMR0_IPV0_Q_SHIFT         (0U)
#define NETC_PORT_PIPV2QMR0_IPV0_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV0_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV0_Q_MASK)

#define NETC_PORT_PIPV2QMR0_IPV1_Q_MASK          (0xF0U)
#define NETC_PORT_PIPV2QMR0_IPV1_Q_SHIFT         (4U)
#define NETC_PORT_PIPV2QMR0_IPV1_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV1_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV1_Q_MASK)

#define NETC_PORT_PIPV2QMR0_IPV2_Q_MASK          (0xF00U)
#define NETC_PORT_PIPV2QMR0_IPV2_Q_SHIFT         (8U)
#define NETC_PORT_PIPV2QMR0_IPV2_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV2_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV2_Q_MASK)

#define NETC_PORT_PIPV2QMR0_IPV3_Q_MASK          (0xF000U)
#define NETC_PORT_PIPV2QMR0_IPV3_Q_SHIFT         (12U)
#define NETC_PORT_PIPV2QMR0_IPV3_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV3_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV3_Q_MASK)

#define NETC_PORT_PIPV2QMR0_IPV4_Q_MASK          (0xF0000U)
#define NETC_PORT_PIPV2QMR0_IPV4_Q_SHIFT         (16U)
#define NETC_PORT_PIPV2QMR0_IPV4_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV4_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV4_Q_MASK)

#define NETC_PORT_PIPV2QMR0_IPV5_Q_MASK          (0xF00000U)
#define NETC_PORT_PIPV2QMR0_IPV5_Q_SHIFT         (20U)
#define NETC_PORT_PIPV2QMR0_IPV5_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV5_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV5_Q_MASK)

#define NETC_PORT_PIPV2QMR0_IPV6_Q_MASK          (0xF000000U)
#define NETC_PORT_PIPV2QMR0_IPV6_Q_SHIFT         (24U)
#define NETC_PORT_PIPV2QMR0_IPV6_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV6_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV6_Q_MASK)

#define NETC_PORT_PIPV2QMR0_IPV7_Q_MASK          (0xF0000000U)
#define NETC_PORT_PIPV2QMR0_IPV7_Q_SHIFT         (28U)
#define NETC_PORT_PIPV2QMR0_IPV7_Q(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PIPV2QMR0_IPV7_Q_SHIFT)) & NETC_PORT_PIPV2QMR0_IPV7_Q_MASK)
/*! @} */

/*! @name PTCMINLR - Port time capture minimum latency register */
/*! @{ */

#define NETC_PORT_PTCMINLR_LATENCY_MASK          (0x3FFFFFFFU)
#define NETC_PORT_PTCMINLR_LATENCY_SHIFT         (0U)
#define NETC_PORT_PTCMINLR_LATENCY(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCMINLR_LATENCY_SHIFT)) & NETC_PORT_PTCMINLR_LATENCY_MASK)

#define NETC_PORT_PTCMINLR_COUNT_MASK            (0xC0000000U)
#define NETC_PORT_PTCMINLR_COUNT_SHIFT           (30U)
#define NETC_PORT_PTCMINLR_COUNT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCMINLR_COUNT_SHIFT)) & NETC_PORT_PTCMINLR_COUNT_MASK)
/*! @} */

/*! @name PTCMAXLR - Port time capture maximum latency register */
/*! @{ */

#define NETC_PORT_PTCMAXLR_LATENCY_MASK          (0x3FFFFFFFU)
#define NETC_PORT_PTCMAXLR_LATENCY_SHIFT         (0U)
#define NETC_PORT_PTCMAXLR_LATENCY(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_PTCMAXLR_LATENCY_SHIFT)) & NETC_PORT_PTCMAXLR_LATENCY_MASK)
/*! @} */

/*! @name BPCR - Bridge port configuration register */
/*! @{ */

#define NETC_PORT_BPCR_DYN_LIMIT_MASK            (0xFFFFU)
#define NETC_PORT_BPCR_DYN_LIMIT_SHIFT           (0U)
/*! DYN_LIMIT - Dynamic Limit */
#define NETC_PORT_BPCR_DYN_LIMIT(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPCR_DYN_LIMIT_SHIFT)) & NETC_PORT_BPCR_DYN_LIMIT_MASK)

#define NETC_PORT_BPCR_UUCASTE_MASK              (0x1000000U)
#define NETC_PORT_BPCR_UUCASTE_SHIFT             (24U)
/*! UUCASTE - Unknown Unicast Storm Control Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_PORT_BPCR_UUCASTE(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPCR_UUCASTE_SHIFT)) & NETC_PORT_BPCR_UUCASTE_MASK)

#define NETC_PORT_BPCR_UMCASTE_MASK              (0x2000000U)
#define NETC_PORT_BPCR_UMCASTE_SHIFT             (25U)
/*! UMCASTE - Unknown Multicast Storm Control Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_PORT_BPCR_UMCASTE(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPCR_UMCASTE_SHIFT)) & NETC_PORT_BPCR_UMCASTE_MASK)

#define NETC_PORT_BPCR_MCASTE_MASK               (0x4000000U)
#define NETC_PORT_BPCR_MCASTE_SHIFT              (26U)
/*! MCASTE - Multicast Storm Control Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_PORT_BPCR_MCASTE(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPCR_MCASTE_SHIFT)) & NETC_PORT_BPCR_MCASTE_MASK)

#define NETC_PORT_BPCR_BCASTE_MASK               (0x8000000U)
#define NETC_PORT_BPCR_BCASTE_SHIFT              (27U)
/*! BCASTE - Broadcast Storm Control Enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define NETC_PORT_BPCR_BCASTE(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPCR_BCASTE_SHIFT)) & NETC_PORT_BPCR_BCASTE_MASK)

#define NETC_PORT_BPCR_STAMVD_MASK               (0x10000000U)
#define NETC_PORT_BPCR_STAMVD_SHIFT              (28U)
/*! STAMVD - Station Move Disallow
 *  0b0..Allowed
 *  0b1..Disallowed. A received frame for which a MAC station move is detected, will be discarded.
 */
#define NETC_PORT_BPCR_STAMVD(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPCR_STAMVD_SHIFT)) & NETC_PORT_BPCR_STAMVD_MASK)

#define NETC_PORT_BPCR_SRCPRND_MASK              (0x20000000U)
#define NETC_PORT_BPCR_SRCPRND_SHIFT             (29U)
/*! SRCPRND - Source Port Pruning Disable
 *  0b0..Enabled
 *  0b1..Disabled
 */
#define NETC_PORT_BPCR_SRCPRND(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPCR_SRCPRND_SHIFT)) & NETC_PORT_BPCR_SRCPRND_MASK)
/*! @} */

/*! @name BPDVR - Bridge port default VLAN register */
/*! @{ */

#define NETC_PORT_BPDVR_VID_MASK                 (0xFFFU)
#define NETC_PORT_BPDVR_VID_SHIFT                (0U)
#define NETC_PORT_BPDVR_VID(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDVR_VID_SHIFT)) & NETC_PORT_BPDVR_VID_MASK)

#define NETC_PORT_BPDVR_DEI_MASK                 (0x1000U)
#define NETC_PORT_BPDVR_DEI_SHIFT                (12U)
#define NETC_PORT_BPDVR_DEI(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDVR_DEI_SHIFT)) & NETC_PORT_BPDVR_DEI_MASK)

#define NETC_PORT_BPDVR_PCP_MASK                 (0xE000U)
#define NETC_PORT_BPDVR_PCP_SHIFT                (13U)
#define NETC_PORT_BPDVR_PCP(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDVR_PCP_SHIFT)) & NETC_PORT_BPDVR_PCP_MASK)

#define NETC_PORT_BPDVR_TPID_MASK                (0x10000U)
#define NETC_PORT_BPDVR_TPID_SHIFT               (16U)
#define NETC_PORT_BPDVR_TPID(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDVR_TPID_SHIFT)) & NETC_PORT_BPDVR_TPID_MASK)

#define NETC_PORT_BPDVR_RXTAGA_MASK              (0xF00000U)
#define NETC_PORT_BPDVR_RXTAGA_SHIFT             (20U)
#define NETC_PORT_BPDVR_RXTAGA(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDVR_RXTAGA_SHIFT)) & NETC_PORT_BPDVR_RXTAGA_MASK)

#define NETC_PORT_BPDVR_RXVAM_MASK               (0x1000000U)
#define NETC_PORT_BPDVR_RXVAM_SHIFT              (24U)
#define NETC_PORT_BPDVR_RXVAM(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDVR_RXVAM_SHIFT)) & NETC_PORT_BPDVR_RXVAM_MASK)

#define NETC_PORT_BPDVR_TXTAGA_MASK              (0x6000000U)
#define NETC_PORT_BPDVR_TXTAGA_SHIFT             (25U)
#define NETC_PORT_BPDVR_TXTAGA(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDVR_TXTAGA_SHIFT)) & NETC_PORT_BPDVR_TXTAGA_MASK)
/*! @} */

/*! @name BPSTGSR - Bridge port spanning tree group state register */
/*! @{ */

#define NETC_PORT_BPSTGSR_STG_STATE0_MASK        (0x3U)
#define NETC_PORT_BPSTGSR_STG_STATE0_SHIFT       (0U)
#define NETC_PORT_BPSTGSR_STG_STATE0(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE0_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE0_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE1_MASK        (0xCU)
#define NETC_PORT_BPSTGSR_STG_STATE1_SHIFT       (2U)
#define NETC_PORT_BPSTGSR_STG_STATE1(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE1_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE1_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE2_MASK        (0x30U)
#define NETC_PORT_BPSTGSR_STG_STATE2_SHIFT       (4U)
#define NETC_PORT_BPSTGSR_STG_STATE2(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE2_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE2_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE3_MASK        (0xC0U)
#define NETC_PORT_BPSTGSR_STG_STATE3_SHIFT       (6U)
#define NETC_PORT_BPSTGSR_STG_STATE3(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE3_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE3_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE4_MASK        (0x300U)
#define NETC_PORT_BPSTGSR_STG_STATE4_SHIFT       (8U)
#define NETC_PORT_BPSTGSR_STG_STATE4(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE4_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE4_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE5_MASK        (0xC00U)
#define NETC_PORT_BPSTGSR_STG_STATE5_SHIFT       (10U)
#define NETC_PORT_BPSTGSR_STG_STATE5(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE5_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE5_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE6_MASK        (0x3000U)
#define NETC_PORT_BPSTGSR_STG_STATE6_SHIFT       (12U)
#define NETC_PORT_BPSTGSR_STG_STATE6(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE6_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE6_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE7_MASK        (0xC000U)
#define NETC_PORT_BPSTGSR_STG_STATE7_SHIFT       (14U)
#define NETC_PORT_BPSTGSR_STG_STATE7(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE7_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE7_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE8_MASK        (0x30000U)
#define NETC_PORT_BPSTGSR_STG_STATE8_SHIFT       (16U)
#define NETC_PORT_BPSTGSR_STG_STATE8(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE8_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE8_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE9_MASK        (0xC0000U)
#define NETC_PORT_BPSTGSR_STG_STATE9_SHIFT       (18U)
#define NETC_PORT_BPSTGSR_STG_STATE9(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE9_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE9_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE10_MASK       (0x300000U)
#define NETC_PORT_BPSTGSR_STG_STATE10_SHIFT      (20U)
#define NETC_PORT_BPSTGSR_STG_STATE10(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE10_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE10_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE11_MASK       (0xC00000U)
#define NETC_PORT_BPSTGSR_STG_STATE11_SHIFT      (22U)
#define NETC_PORT_BPSTGSR_STG_STATE11(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE11_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE11_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE12_MASK       (0x3000000U)
#define NETC_PORT_BPSTGSR_STG_STATE12_SHIFT      (24U)
#define NETC_PORT_BPSTGSR_STG_STATE12(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE12_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE12_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE13_MASK       (0xC000000U)
#define NETC_PORT_BPSTGSR_STG_STATE13_SHIFT      (26U)
#define NETC_PORT_BPSTGSR_STG_STATE13(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE13_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE13_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE14_MASK       (0x30000000U)
#define NETC_PORT_BPSTGSR_STG_STATE14_SHIFT      (28U)
#define NETC_PORT_BPSTGSR_STG_STATE14(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE14_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE14_MASK)

#define NETC_PORT_BPSTGSR_STG_STATE15_MASK       (0xC0000000U)
#define NETC_PORT_BPSTGSR_STG_STATE15_SHIFT      (30U)
#define NETC_PORT_BPSTGSR_STG_STATE15(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSTGSR_STG_STATE15_SHIFT)) & NETC_PORT_BPSTGSR_STG_STATE15_MASK)
/*! @} */

/*! @name BPSCR0 - Bridge port storm control register 0 */
/*! @{ */

#define NETC_PORT_BPSCR0_UUCASTRPEID_MASK        (0xFFFU)
#define NETC_PORT_BPSCR0_UUCASTRPEID_SHIFT       (0U)
#define NETC_PORT_BPSCR0_UUCASTRPEID(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSCR0_UUCASTRPEID_SHIFT)) & NETC_PORT_BPSCR0_UUCASTRPEID_MASK)

#define NETC_PORT_BPSCR0_BCASTRPEID_MASK         (0xFFF0000U)
#define NETC_PORT_BPSCR0_BCASTRPEID_SHIFT        (16U)
#define NETC_PORT_BPSCR0_BCASTRPEID(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSCR0_BCASTRPEID_SHIFT)) & NETC_PORT_BPSCR0_BCASTRPEID_MASK)
/*! @} */

/*! @name BPSCR1 - Bridge port storm control register 1 */
/*! @{ */

#define NETC_PORT_BPSCR1_MCASTRPEID_MASK         (0xFFFU)
#define NETC_PORT_BPSCR1_MCASTRPEID_SHIFT        (0U)
#define NETC_PORT_BPSCR1_MCASTRPEID(x)           (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSCR1_MCASTRPEID_SHIFT)) & NETC_PORT_BPSCR1_MCASTRPEID_MASK)

#define NETC_PORT_BPSCR1_UMCASTRPEID_MASK        (0xFFF0000U)
#define NETC_PORT_BPSCR1_UMCASTRPEID_SHIFT       (16U)
#define NETC_PORT_BPSCR1_UMCASTRPEID(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPSCR1_UMCASTRPEID_SHIFT)) & NETC_PORT_BPSCR1_UMCASTRPEID_MASK)
/*! @} */

/*! @name BPOR - Bridge port operational register */
/*! @{ */

#define NETC_PORT_BPOR_NUM_DYN_MASK              (0xFFFFU)
#define NETC_PORT_BPOR_NUM_DYN_SHIFT             (0U)
#define NETC_PORT_BPOR_NUM_DYN(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPOR_NUM_DYN_SHIFT)) & NETC_PORT_BPOR_NUM_DYN_MASK)
/*! @} */

/*! @name BPDCR - Bridge port discard count register */
/*! @{ */

#define NETC_PORT_BPDCR_COUNT_MASK               (0xFFFFFFFFU)
#define NETC_PORT_BPDCR_COUNT_SHIFT              (0U)
#define NETC_PORT_BPDCR_COUNT(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCR_COUNT_SHIFT)) & NETC_PORT_BPDCR_COUNT_MASK)
/*! @} */

/*! @name BPDCRR0 - Bridge port discard count reason register 0 */
/*! @{ */

#define NETC_PORT_BPDCRR0_BPACDR_MASK            (0x1U)
#define NETC_PORT_BPDCRR0_BPACDR_SHIFT           (0U)
#define NETC_PORT_BPDCRR0_BPACDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_BPACDR_SHIFT)) & NETC_PORT_BPDCRR0_BPACDR_MASK)

#define NETC_PORT_BPDCRR0_ISTGSDR_MASK           (0x2U)
#define NETC_PORT_BPDCRR0_ISTGSDR_SHIFT          (1U)
#define NETC_PORT_BPDCRR0_ISTGSDR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_ISTGSDR_SHIFT)) & NETC_PORT_BPDCRR0_ISTGSDR_MASK)

#define NETC_PORT_BPDCRR0_BPVFLTDR_MASK          (0x4U)
#define NETC_PORT_BPDCRR0_BPVFLTDR_SHIFT         (2U)
#define NETC_PORT_BPDCRR0_BPVFLTDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_BPVFLTDR_SHIFT)) & NETC_PORT_BPDCRR0_BPVFLTDR_MASK)

#define NETC_PORT_BPDCRR0_MACLNFDR_MASK          (0x8U)
#define NETC_PORT_BPDCRR0_MACLNFDR_SHIFT         (3U)
#define NETC_PORT_BPDCRR0_MACLNFDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_MACLNFDR_SHIFT)) & NETC_PORT_BPDCRR0_MACLNFDR_MASK)

#define NETC_PORT_BPDCRR0_STAMVDDR_MASK          (0x80U)
#define NETC_PORT_BPDCRR0_STAMVDDR_SHIFT         (7U)
#define NETC_PORT_BPDCRR0_STAMVDDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_STAMVDDR_SHIFT)) & NETC_PORT_BPDCRR0_STAMVDDR_MASK)

#define NETC_PORT_BPDCRR0_MACFDDDR_MASK          (0x100U)
#define NETC_PORT_BPDCRR0_MACFDDDR_SHIFT         (8U)
#define NETC_PORT_BPDCRR0_MACFDDDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_MACFDDDR_SHIFT)) & NETC_PORT_BPDCRR0_MACFDDDR_MASK)

#define NETC_PORT_BPDCRR0_NODESTDR_MASK          (0x200U)
#define NETC_PORT_BPDCRR0_NODESTDR_SHIFT         (9U)
#define NETC_PORT_BPDCRR0_NODESTDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_NODESTDR_SHIFT)) & NETC_PORT_BPDCRR0_NODESTDR_MASK)

#define NETC_PORT_BPDCRR0_IPMFDR_MASK            (0x400U)
#define NETC_PORT_BPDCRR0_IPMFDR_SHIFT           (10U)
#define NETC_PORT_BPDCRR0_IPMFDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_IPMFDR_SHIFT)) & NETC_PORT_BPDCRR0_IPMFDR_MASK)

#define NETC_PORT_BPDCRR0_UFMMDR_MASK            (0x800U)
#define NETC_PORT_BPDCRR0_UFMMDR_SHIFT           (11U)
#define NETC_PORT_BPDCRR0_UFMMDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_UFMMDR_SHIFT)) & NETC_PORT_BPDCRR0_UFMMDR_MASK)

#define NETC_PORT_BPDCRR0_MISCDR_MASK            (0x1000U)
#define NETC_PORT_BPDCRR0_MISCDR_SHIFT           (12U)
#define NETC_PORT_BPDCRR0_MISCDR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_MISCDR_SHIFT)) & NETC_PORT_BPDCRR0_MISCDR_MASK)

#define NETC_PORT_BPDCRR0_STRMCTRLDR_MASK        (0x2000U)
#define NETC_PORT_BPDCRR0_STRMCTRLDR_SHIFT       (13U)
/*! STRMCTRLDR - Discard due to Storm Control Policer Discard Reason */
#define NETC_PORT_BPDCRR0_STRMCTRLDR(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR0_STRMCTRLDR_SHIFT)) & NETC_PORT_BPDCRR0_STRMCTRLDR_MASK)
/*! @} */

/*! @name BPDCRR1 - Bridge port discard count reason register 1 */
/*! @{ */

#define NETC_PORT_BPDCRR1_ENTRYID_MASK           (0x7FFFFFFU)
#define NETC_PORT_BPDCRR1_ENTRYID_SHIFT          (0U)
#define NETC_PORT_BPDCRR1_ENTRYID(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR1_ENTRYID_SHIFT)) & NETC_PORT_BPDCRR1_ENTRYID_MASK)

#define NETC_PORT_BPDCRR1_TT_MASK                (0xF0000000U)
#define NETC_PORT_BPDCRR1_TT_SHIFT               (28U)
#define NETC_PORT_BPDCRR1_TT(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPDCRR1_TT_SHIFT)) & NETC_PORT_BPDCRR1_TT_MASK)
/*! @} */

/*! @name BPMLFSR - Bridge port MAC learning failure status register */
/*! @{ */

#define NETC_PORT_BPMLFSR_BPMLLRFR_MASK          (0x1U)
#define NETC_PORT_BPMLFSR_BPMLLRFR_SHIFT         (0U)
/*! BPMLLRFR - Bridge Port MAC Learn Limit Reached Failure Reason */
#define NETC_PORT_BPMLFSR_BPMLLRFR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPMLFSR_BPMLLRFR_SHIFT)) & NETC_PORT_BPMLFSR_BPMLLRFR_MASK)

#define NETC_PORT_BPMLFSR_FFDBTRFR_MASK          (0x2U)
#define NETC_PORT_BPMLFSR_FFDBTRFR_SHIFT         (1U)
/*! FFDBTRFR - Full FDB Table Reached Failure Reason */
#define NETC_PORT_BPMLFSR_FFDBTRFR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPMLFSR_FFDBTRFR_SHIFT)) & NETC_PORT_BPMLFSR_FFDBTRFR_MASK)

#define NETC_PORT_BPMLFSR_HCCLRFR_MASK           (0x4U)
#define NETC_PORT_BPMLFSR_HCCLRFR_SHIFT          (2U)
/*! HCCLRFR - Hash Collision chain limit Reached Failure Reason */
#define NETC_PORT_BPMLFSR_HCCLRFR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PORT_BPMLFSR_HCCLRFR_SHIFT)) & NETC_PORT_BPMLFSR_HCCLRFR_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group NETC_PORT_Register_Masks */


/* NETC_PORT - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ENETC0_PORT base address */
  #define ENETC0_PORT_BASE                         (0x70B14000u)
  /** Peripheral ENETC0_PORT base address */
  #define ENETC0_PORT_BASE_NS                      (0x60B14000u)
  /** Peripheral ENETC0_PORT base pointer */
  #define ENETC0_PORT                              ((NETC_PORT_Type *)ENETC0_PORT_BASE)
  /** Peripheral ENETC0_PORT base pointer */
  #define ENETC0_PORT_NS                           ((NETC_PORT_Type *)ENETC0_PORT_BASE_NS)
  /** Peripheral ENETC1_PORT base address */
  #define ENETC1_PORT_BASE                         (0x70B54000u)
  /** Peripheral ENETC1_PORT base address */
  #define ENETC1_PORT_BASE_NS                      (0x60B54000u)
  /** Peripheral ENETC1_PORT base pointer */
  #define ENETC1_PORT                              ((NETC_PORT_Type *)ENETC1_PORT_BASE)
  /** Peripheral ENETC1_PORT base pointer */
  #define ENETC1_PORT_NS                           ((NETC_PORT_Type *)ENETC1_PORT_BASE_NS)
  /** Peripheral SW0_PORT0 base address */
  #define SW0_PORT0_BASE                           (0x70A04000u)
  /** Peripheral SW0_PORT0 base address */
  #define SW0_PORT0_BASE_NS                        (0x60A04000u)
  /** Peripheral SW0_PORT0 base pointer */
  #define SW0_PORT0                                ((NETC_PORT_Type *)SW0_PORT0_BASE)
  /** Peripheral SW0_PORT0 base pointer */
  #define SW0_PORT0_NS                             ((NETC_PORT_Type *)SW0_PORT0_BASE_NS)
  /** Peripheral SW0_PORT1 base address */
  #define SW0_PORT1_BASE                           (0x70A08000u)
  /** Peripheral SW0_PORT1 base address */
  #define SW0_PORT1_BASE_NS                        (0x60A08000u)
  /** Peripheral SW0_PORT1 base pointer */
  #define SW0_PORT1                                ((NETC_PORT_Type *)SW0_PORT1_BASE)
  /** Peripheral SW0_PORT1 base pointer */
  #define SW0_PORT1_NS                             ((NETC_PORT_Type *)SW0_PORT1_BASE_NS)
  /** Peripheral SW0_PORT2 base address */
  #define SW0_PORT2_BASE                           (0x70A0C000u)
  /** Peripheral SW0_PORT2 base address */
  #define SW0_PORT2_BASE_NS                        (0x60A0C000u)
  /** Peripheral SW0_PORT2 base pointer */
  #define SW0_PORT2                                ((NETC_PORT_Type *)SW0_PORT2_BASE)
  /** Peripheral SW0_PORT2 base pointer */
  #define SW0_PORT2_NS                             ((NETC_PORT_Type *)SW0_PORT2_BASE_NS)
  /** Peripheral SW0_PORT3 base address */
  #define SW0_PORT3_BASE                           (0x70A10000u)
  /** Peripheral SW0_PORT3 base address */
  #define SW0_PORT3_BASE_NS                        (0x60A10000u)
  /** Peripheral SW0_PORT3 base pointer */
  #define SW0_PORT3                                ((NETC_PORT_Type *)SW0_PORT3_BASE)
  /** Peripheral SW0_PORT3 base pointer */
  #define SW0_PORT3_NS                             ((NETC_PORT_Type *)SW0_PORT3_BASE_NS)
  /** Peripheral SW0_PORT4 base address */
  #define SW0_PORT4_BASE                           (0x70A14000u)
  /** Peripheral SW0_PORT4 base address */
  #define SW0_PORT4_BASE_NS                        (0x60A14000u)
  /** Peripheral SW0_PORT4 base pointer */
  #define SW0_PORT4                                ((NETC_PORT_Type *)SW0_PORT4_BASE)
  /** Peripheral SW0_PORT4 base pointer */
  #define SW0_PORT4_NS                             ((NETC_PORT_Type *)SW0_PORT4_BASE_NS)
  /** Array initializer of NETC_PORT peripheral base addresses */
  #define NETC_PORT_BASE_ADDRS                     { ENETC0_PORT_BASE, ENETC1_PORT_BASE, SW0_PORT0_BASE, SW0_PORT1_BASE, SW0_PORT2_BASE, SW0_PORT3_BASE, SW0_PORT4_BASE }
  /** Array initializer of NETC_PORT peripheral base pointers */
  #define NETC_PORT_BASE_PTRS                      { ENETC0_PORT, ENETC1_PORT, SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 }
  /** Array initializer of NETC_PORT peripheral base addresses */
  #define NETC_PORT_BASE_ADDRS_NS                  { ENETC0_PORT_BASE_NS, ENETC1_PORT_BASE_NS, SW0_PORT0_BASE_NS, SW0_PORT1_BASE_NS, SW0_PORT2_BASE_NS, SW0_PORT3_BASE_NS, SW0_PORT4_BASE_NS }
  /** Array initializer of NETC_PORT peripheral base pointers */
  #define NETC_PORT_BASE_PTRS_NS                   { ENETC0_PORT_NS, ENETC1_PORT_NS, SW0_PORT0_NS, SW0_PORT1_NS, SW0_PORT2_NS, SW0_PORT3_NS, SW0_PORT4_NS }
#else
  /** Peripheral ENETC0_PORT base address */
  #define ENETC0_PORT_BASE                         (0x60B14000u)
  /** Peripheral ENETC0_PORT base pointer */
  #define ENETC0_PORT                              ((NETC_PORT_Type *)ENETC0_PORT_BASE)
  /** Peripheral ENETC1_PORT base address */
  #define ENETC1_PORT_BASE                         (0x60B54000u)
  /** Peripheral ENETC1_PORT base pointer */
  #define ENETC1_PORT                              ((NETC_PORT_Type *)ENETC1_PORT_BASE)
  /** Peripheral SW0_PORT0 base address */
  #define SW0_PORT0_BASE                           (0x60A04000u)
  /** Peripheral SW0_PORT0 base pointer */
  #define SW0_PORT0                                ((NETC_PORT_Type *)SW0_PORT0_BASE)
  /** Peripheral SW0_PORT1 base address */
  #define SW0_PORT1_BASE                           (0x60A08000u)
  /** Peripheral SW0_PORT1 base pointer */
  #define SW0_PORT1                                ((NETC_PORT_Type *)SW0_PORT1_BASE)
  /** Peripheral SW0_PORT2 base address */
  #define SW0_PORT2_BASE                           (0x60A0C000u)
  /** Peripheral SW0_PORT2 base pointer */
  #define SW0_PORT2                                ((NETC_PORT_Type *)SW0_PORT2_BASE)
  /** Peripheral SW0_PORT3 base address */
  #define SW0_PORT3_BASE                           (0x60A10000u)
  /** Peripheral SW0_PORT3 base pointer */
  #define SW0_PORT3                                ((NETC_PORT_Type *)SW0_PORT3_BASE)
  /** Peripheral SW0_PORT4 base address */
  #define SW0_PORT4_BASE                           (0x60A14000u)
  /** Peripheral SW0_PORT4 base pointer */
  #define SW0_PORT4                                ((NETC_PORT_Type *)SW0_PORT4_BASE)
  /** Array initializer of NETC_PORT peripheral base addresses */
  #define NETC_PORT_BASE_ADDRS                     { ENETC0_PORT_BASE, ENETC1_PORT_BASE, SW0_PORT0_BASE, SW0_PORT1_BASE, SW0_PORT2_BASE, SW0_PORT3_BASE, SW0_PORT4_BASE }
  /** Array initializer of NETC_PORT peripheral base pointers */
  #define NETC_PORT_BASE_PTRS                      { ENETC0_PORT, ENETC1_PORT, SW0_PORT0, SW0_PORT1, SW0_PORT2, SW0_PORT3, SW0_PORT4 }
#endif

/*!
 * @}
 */ /* end of group NETC_PORT_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_PRIV Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_PRIV_Peripheral_Access_Layer NETC_PRIV Peripheral Access Layer
 * @{
 */

/** NETC_PRIV - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[256];
  __IO uint32_t NETCRR;                            /**< NETC reset register, offset: 0x100 */
  __I  uint32_t NETCSR;                            /**< NETC status register, offset: 0x104 */
       uint8_t RESERVED_1[256];
  __IO uint32_t MEICR;                             /**< Memory Error Injection Config Register, offset: 0x208 */
       uint8_t RESERVED_2[3060];
  __IO uint32_t CMECR;                             /**< Correctable memory error configuration register, offset: 0xE00 */
  __IO uint32_t CMESR;                             /**< Correctable memory error status register, offset: 0xE04 */
       uint8_t RESERVED_3[4];
  __I  uint32_t CMECTR;                            /**< Correctable memory error count register, offset: 0xE0C */
       uint8_t RESERVED_4[32];
  __IO uint32_t UNMECR;                            /**< Uncorrectable non-fatal memory error configuration register, offset: 0xE30 */
  __IO uint32_t UNMESR0;                           /**< Uncorrectable non-fatal memory error status register 0, offset: 0xE34 */
  __I  uint32_t UNMESR1;                           /**< Uncorrectable non-fatal memory error status register 1, offset: 0xE38 */
  __I  uint32_t UNMECTR;                           /**< Uncorrectable non-fatal memory error count register, offset: 0xE3C */
  __IO uint32_t UFMECR;                            /**< Uncorrectable fatal memory error configuration register, offset: 0xE40 */
  __IO uint32_t UFMESR0;                           /**< Uncorrectable fatal memory error status register 0, offset: 0xE44 */
  __I  uint32_t UFMESR1;                           /**< Uncorrectable fatal memory error status register 1, offset: 0xE48 */
} NETC_PRIV_Type;

/* ----------------------------------------------------------------------------
   -- NETC_PRIV Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_PRIV_Register_Masks NETC_PRIV Register Masks
 * @{
 */

/*! @name NETCRR - NETC reset register */
/*! @{ */

#define NETC_PRIV_NETCRR_SR_MASK                 (0x1U)
#define NETC_PRIV_NETCRR_SR_SHIFT                (0U)
/*! SR - Soft reset */
#define NETC_PRIV_NETCRR_SR(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_NETCRR_SR_SHIFT)) & NETC_PRIV_NETCRR_SR_MASK)

#define NETC_PRIV_NETCRR_LOCK_MASK               (0x2U)
#define NETC_PRIV_NETCRR_LOCK_SHIFT              (1U)
/*! LOCK - Lock */
#define NETC_PRIV_NETCRR_LOCK(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_NETCRR_LOCK_SHIFT)) & NETC_PRIV_NETCRR_LOCK_MASK)
/*! @} */

/*! @name NETCSR - NETC status register */
/*! @{ */

#define NETC_PRIV_NETCSR_ERROR_MASK              (0x1U)
#define NETC_PRIV_NETCSR_ERROR_SHIFT             (0U)
/*! ERROR - Error */
#define NETC_PRIV_NETCSR_ERROR(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_NETCSR_ERROR_SHIFT)) & NETC_PRIV_NETCSR_ERROR_MASK)

#define NETC_PRIV_NETCSR_STATE_MASK              (0x2U)
#define NETC_PRIV_NETCSR_STATE_SHIFT             (1U)
/*! STATE - Indicates NETC's global operational state */
#define NETC_PRIV_NETCSR_STATE(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_NETCSR_STATE_SHIFT)) & NETC_PRIV_NETCSR_STATE_MASK)
/*! @} */

/*! @name MEICR - Memory Error Injection Config Register */
/*! @{ */

#define NETC_PRIV_MEICR_MEM_ID_MASK              (0x1FU)
#define NETC_PRIV_MEICR_MEM_ID_SHIFT             (0U)
/*! MEM_ID - Memory ID */
#define NETC_PRIV_MEICR_MEM_ID(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_MEICR_MEM_ID_SHIFT)) & NETC_PRIV_MEICR_MEM_ID_MASK)

#define NETC_PRIV_MEICR_ARM_MASK                 (0xC00000U)
#define NETC_PRIV_MEICR_ARM_SHIFT                (22U)
/*! ARM - Armed */
#define NETC_PRIV_MEICR_ARM(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_MEICR_ARM_SHIFT)) & NETC_PRIV_MEICR_ARM_MASK)

#define NETC_PRIV_MEICR_EN_MASK                  (0xFF000000U)
#define NETC_PRIV_MEICR_EN_SHIFT                 (24U)
/*! EN - Enable */
#define NETC_PRIV_MEICR_EN(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_MEICR_EN_SHIFT)) & NETC_PRIV_MEICR_EN_MASK)
/*! @} */

/*! @name CMECR - Correctable memory error configuration register */
/*! @{ */

#define NETC_PRIV_CMECR_THRESHOLD_MASK           (0xFFU)
#define NETC_PRIV_CMECR_THRESHOLD_SHIFT          (0U)
/*! THRESHOLD - Threshold */
#define NETC_PRIV_CMECR_THRESHOLD(x)             (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_CMECR_THRESHOLD_SHIFT)) & NETC_PRIV_CMECR_THRESHOLD_MASK)
/*! @} */

/*! @name CMESR - Correctable memory error status register */
/*! @{ */

#define NETC_PRIV_CMESR_MEM_ID_MASK              (0x1F0000U)
#define NETC_PRIV_CMESR_MEM_ID_SHIFT             (16U)
/*! MEM_ID - Memory ID */
#define NETC_PRIV_CMESR_MEM_ID(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_CMESR_MEM_ID_SHIFT)) & NETC_PRIV_CMESR_MEM_ID_MASK)

#define NETC_PRIV_CMESR_SBEE_MASK                (0x80000000U)
#define NETC_PRIV_CMESR_SBEE_SHIFT               (31U)
/*! SBEE - Single-bit ECC error */
#define NETC_PRIV_CMESR_SBEE(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_CMESR_SBEE_SHIFT)) & NETC_PRIV_CMESR_SBEE_MASK)
/*! @} */

/*! @name CMECTR - Correctable memory error count register */
/*! @{ */

#define NETC_PRIV_CMECTR_COUNT_MASK              (0xFFU)
#define NETC_PRIV_CMECTR_COUNT_SHIFT             (0U)
/*! COUNT - Count */
#define NETC_PRIV_CMECTR_COUNT(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_CMECTR_COUNT_SHIFT)) & NETC_PRIV_CMECTR_COUNT_MASK)
/*! @} */

/*! @name UNMECR - Uncorrectable non-fatal memory error configuration register */
/*! @{ */

#define NETC_PRIV_UNMECR_THRESHOLD_MASK          (0xFFU)
#define NETC_PRIV_UNMECR_THRESHOLD_SHIFT         (0U)
/*! THRESHOLD - Threshold */
#define NETC_PRIV_UNMECR_THRESHOLD(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UNMECR_THRESHOLD_SHIFT)) & NETC_PRIV_UNMECR_THRESHOLD_MASK)

#define NETC_PRIV_UNMECR_RD_MASK                 (0x80000000U)
#define NETC_PRIV_UNMECR_RD_SHIFT                (31U)
/*! RD - Report disable */
#define NETC_PRIV_UNMECR_RD(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UNMECR_RD_SHIFT)) & NETC_PRIV_UNMECR_RD_MASK)
/*! @} */

/*! @name UNMESR0 - Uncorrectable non-fatal memory error status register 0 */
/*! @{ */

#define NETC_PRIV_UNMESR0_SYNDROME_MASK          (0x7FFU)
#define NETC_PRIV_UNMESR0_SYNDROME_SHIFT         (0U)
/*! SYNDROME - Syndrome */
#define NETC_PRIV_UNMESR0_SYNDROME(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UNMESR0_SYNDROME_SHIFT)) & NETC_PRIV_UNMESR0_SYNDROME_MASK)

#define NETC_PRIV_UNMESR0_MEM_ID_MASK            (0x1F0000U)
#define NETC_PRIV_UNMESR0_MEM_ID_SHIFT           (16U)
/*! MEM_ID - Memory ID */
#define NETC_PRIV_UNMESR0_MEM_ID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UNMESR0_MEM_ID_SHIFT)) & NETC_PRIV_UNMESR0_MEM_ID_MASK)

#define NETC_PRIV_UNMESR0_MBEE_MASK              (0x80000000U)
#define NETC_PRIV_UNMESR0_MBEE_SHIFT             (31U)
/*! MBEE - Multi-bit ECC error */
#define NETC_PRIV_UNMESR0_MBEE(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UNMESR0_MBEE_SHIFT)) & NETC_PRIV_UNMESR0_MBEE_MASK)
/*! @} */

/*! @name UNMESR1 - Uncorrectable non-fatal memory error status register 1 */
/*! @{ */

#define NETC_PRIV_UNMESR1_ADDR_MASK              (0xFFFFFFFFU)
#define NETC_PRIV_UNMESR1_ADDR_SHIFT             (0U)
/*! ADDR - Address */
#define NETC_PRIV_UNMESR1_ADDR(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UNMESR1_ADDR_SHIFT)) & NETC_PRIV_UNMESR1_ADDR_MASK)
/*! @} */

/*! @name UNMECTR - Uncorrectable non-fatal memory error count register */
/*! @{ */

#define NETC_PRIV_UNMECTR_COUNT_MASK             (0xFFU)
#define NETC_PRIV_UNMECTR_COUNT_SHIFT            (0U)
/*! COUNT - Count */
#define NETC_PRIV_UNMECTR_COUNT(x)               (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UNMECTR_COUNT_SHIFT)) & NETC_PRIV_UNMECTR_COUNT_MASK)
/*! @} */

/*! @name UFMECR - Uncorrectable fatal memory error configuration register */
/*! @{ */

#define NETC_PRIV_UFMECR_RD_MASK                 (0x80000000U)
#define NETC_PRIV_UFMECR_RD_SHIFT                (31U)
/*! RD - Report disable */
#define NETC_PRIV_UFMECR_RD(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UFMECR_RD_SHIFT)) & NETC_PRIV_UFMECR_RD_MASK)
/*! @} */

/*! @name UFMESR0 - Uncorrectable fatal memory error status register 0 */
/*! @{ */

#define NETC_PRIV_UFMESR0_SYNDROME_MASK          (0x7FFU)
#define NETC_PRIV_UFMESR0_SYNDROME_SHIFT         (0U)
/*! SYNDROME - Syndrome */
#define NETC_PRIV_UFMESR0_SYNDROME(x)            (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UFMESR0_SYNDROME_SHIFT)) & NETC_PRIV_UFMESR0_SYNDROME_MASK)

#define NETC_PRIV_UFMESR0_MEM_ID_MASK            (0x1F0000U)
#define NETC_PRIV_UFMESR0_MEM_ID_SHIFT           (16U)
/*! MEM_ID - Memory ID */
#define NETC_PRIV_UFMESR0_MEM_ID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UFMESR0_MEM_ID_SHIFT)) & NETC_PRIV_UFMESR0_MEM_ID_MASK)

#define NETC_PRIV_UFMESR0_M_MASK                 (0x40000000U)
#define NETC_PRIV_UFMESR0_M_SHIFT                (30U)
/*! M - Multiple */
#define NETC_PRIV_UFMESR0_M(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UFMESR0_M_SHIFT)) & NETC_PRIV_UFMESR0_M_MASK)

#define NETC_PRIV_UFMESR0_MBEE_MASK              (0x80000000U)
#define NETC_PRIV_UFMESR0_MBEE_SHIFT             (31U)
/*! MBEE - Multi-bit ECC error */
#define NETC_PRIV_UFMESR0_MBEE(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UFMESR0_MBEE_SHIFT)) & NETC_PRIV_UFMESR0_MBEE_MASK)
/*! @} */

/*! @name UFMESR1 - Uncorrectable fatal memory error status register 1 */
/*! @{ */

#define NETC_PRIV_UFMESR1_ADDR_MASK              (0xFFFFFFFFU)
#define NETC_PRIV_UFMESR1_ADDR_SHIFT             (0U)
/*! ADDR - Address */
#define NETC_PRIV_UFMESR1_ADDR(x)                (((uint32_t)(((uint32_t)(x)) << NETC_PRIV_UFMESR1_ADDR_SHIFT)) & NETC_PRIV_UFMESR1_ADDR_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group NETC_PRIV_Register_Masks */


/* NETC_PRIV - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral NETC_PRIV base address */
  #define NETC_PRIV_BASE                           (0x70900000u)
  /** Peripheral NETC_PRIV base address */
  #define NETC_PRIV_BASE_NS                        (0x60900000u)
  /** Peripheral NETC_PRIV base pointer */
  #define NETC_PRIV                                ((NETC_PRIV_Type *)NETC_PRIV_BASE)
  /** Peripheral NETC_PRIV base pointer */
  #define NETC_PRIV_NS                             ((NETC_PRIV_Type *)NETC_PRIV_BASE_NS)
  /** Array initializer of NETC_PRIV peripheral base addresses */
  #define NETC_PRIV_BASE_ADDRS                     { NETC_PRIV_BASE }
  /** Array initializer of NETC_PRIV peripheral base pointers */
  #define NETC_PRIV_BASE_PTRS                      { NETC_PRIV }
  /** Array initializer of NETC_PRIV peripheral base addresses */
  #define NETC_PRIV_BASE_ADDRS_NS                  { NETC_PRIV_BASE_NS }
  /** Array initializer of NETC_PRIV peripheral base pointers */
  #define NETC_PRIV_BASE_PTRS_NS                   { NETC_PRIV_NS }
#else
  /** Peripheral NETC_PRIV base address */
  #define NETC_PRIV_BASE                           (0x60900000u)
  /** Peripheral NETC_PRIV base pointer */
  #define NETC_PRIV                                ((NETC_PRIV_Type *)NETC_PRIV_BASE)
  /** Array initializer of NETC_PRIV peripheral base addresses */
  #define NETC_PRIV_BASE_ADDRS                     { NETC_PRIV_BASE }
  /** Array initializer of NETC_PRIV peripheral base pointers */
  #define NETC_PRIV_BASE_PTRS                      { NETC_PRIV }
#endif

/*!
 * @}
 */ /* end of group NETC_PRIV_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_PSEUDO_LINK Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_PSEUDO_LINK_Peripheral_Access_Layer NETC_PSEUDO_LINK Peripheral Access Layer
 * @{
 */

/** NETC_PSEUDO_LINK - Register Layout Typedef */
typedef struct {
  __I  uint32_t PPMSR;                             /**< Port pseudo MAC status register, offset: 0x0 */
       uint8_t RESERVED_0[12];
  __IO uint32_t PPMCR;                             /**< Port pseudo MAC configuration register, offset: 0x10, available only on: ENETC1_PSEUDO_MAC_PORT (missing on SW0_PSEUDO_MAC_PORT4) */
       uint8_t RESERVED_1[108];
  __I  uint32_t PPMROCR[2];                        /**< Port pseudo MAC receive octets counter, array offset: 0x80, array step: 0x4 */
  __I  uint32_t PPMRUFCR[2];                       /**< Port pseudo MAC receive unicast frame counter register, array offset: 0x88, array step: 0x4 */
  __I  uint32_t PPMRMFCR[2];                       /**< Port pseudo MAC receive multicast frame counter register, array offset: 0x90, array step: 0x4 */
  __I  uint32_t PPMRBFCR[2];                       /**< Port pseudo MAC receive broadcast frame counter register, array offset: 0x98, array step: 0x4 */
       uint8_t RESERVED_2[32];
  __I  uint32_t PPMTOCR[2];                        /**< Port pseudo MAC transmit octets counter, array offset: 0xC0, array step: 0x4 */
  __I  uint32_t PPMTUFCR[2];                       /**< Port pseudo MAC transmit unicast frame counter register, array offset: 0xC8, array step: 0x4 */
  __I  uint32_t PPMTMFCR[2];                       /**< Port pseudo MAC transmit multicast frame counter register, array offset: 0xD0, array step: 0x4 */
  __I  uint32_t PPMTBFCR[2];                       /**< Port pseudo MAC transmit broadcast frame counter register, array offset: 0xD8, array step: 0x4 */
} NETC_PSEUDO_LINK_Type;

/* ----------------------------------------------------------------------------
   -- NETC_PSEUDO_LINK Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_PSEUDO_LINK_Register_Masks NETC_PSEUDO_LINK Register Masks
 * @{
 */

/*! @name PPMSR - Port pseudo MAC status register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMSR_LSTATE_MASK       (0x1U)
#define NETC_PSEUDO_LINK_PPMSR_LSTATE_SHIFT      (0U)
#define NETC_PSEUDO_LINK_PPMSR_LSTATE(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMSR_LSTATE_SHIFT)) & NETC_PSEUDO_LINK_PPMSR_LSTATE_MASK)

#define NETC_PSEUDO_LINK_PPMSR_RSTATE_MASK       (0x100U)
#define NETC_PSEUDO_LINK_PPMSR_RSTATE_SHIFT      (8U)
#define NETC_PSEUDO_LINK_PPMSR_RSTATE(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMSR_RSTATE_SHIFT)) & NETC_PSEUDO_LINK_PPMSR_RSTATE_MASK)
/*! @} */

/*! @name PPMCR - Port pseudo MAC configuration register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMCR_TXPAD_MASK        (0x10000U)
#define NETC_PSEUDO_LINK_PPMCR_TXPAD_SHIFT       (16U)
/*! TXPAD - Transmit Padding */
#define NETC_PSEUDO_LINK_PPMCR_TXPAD(x)          (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMCR_TXPAD_SHIFT)) & NETC_PSEUDO_LINK_PPMCR_TXPAD_MASK)
/*! @} */

/*! @name PPMROCR - Port pseudo MAC receive octets counter */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMROCR_ROCT_MASK       (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMROCR_ROCT_SHIFT      (0U)
#define NETC_PSEUDO_LINK_PPMROCR_ROCT(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMROCR_ROCT_SHIFT)) & NETC_PSEUDO_LINK_PPMROCR_ROCT_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMROCR */
#define NETC_PSEUDO_LINK_PPMROCR_COUNT           (2U)

/*! @name PPMRUFCR - Port pseudo MAC receive unicast frame counter register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMRUFCR_RUCA_MASK      (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMRUFCR_RUCA_SHIFT     (0U)
#define NETC_PSEUDO_LINK_PPMRUFCR_RUCA(x)        (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMRUFCR_RUCA_SHIFT)) & NETC_PSEUDO_LINK_PPMRUFCR_RUCA_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMRUFCR */
#define NETC_PSEUDO_LINK_PPMRUFCR_COUNT          (2U)

/*! @name PPMRMFCR - Port pseudo MAC receive multicast frame counter register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMRMFCR_RMCA_MASK      (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMRMFCR_RMCA_SHIFT     (0U)
#define NETC_PSEUDO_LINK_PPMRMFCR_RMCA(x)        (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMRMFCR_RMCA_SHIFT)) & NETC_PSEUDO_LINK_PPMRMFCR_RMCA_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMRMFCR */
#define NETC_PSEUDO_LINK_PPMRMFCR_COUNT          (2U)

/*! @name PPMRBFCR - Port pseudo MAC receive broadcast frame counter register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMRBFCR_RBCA_MASK      (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMRBFCR_RBCA_SHIFT     (0U)
#define NETC_PSEUDO_LINK_PPMRBFCR_RBCA(x)        (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMRBFCR_RBCA_SHIFT)) & NETC_PSEUDO_LINK_PPMRBFCR_RBCA_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMRBFCR */
#define NETC_PSEUDO_LINK_PPMRBFCR_COUNT          (2U)

/*! @name PPMTOCR - Port pseudo MAC transmit octets counter */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMTOCR_TOCT_MASK       (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMTOCR_TOCT_SHIFT      (0U)
#define NETC_PSEUDO_LINK_PPMTOCR_TOCT(x)         (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMTOCR_TOCT_SHIFT)) & NETC_PSEUDO_LINK_PPMTOCR_TOCT_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMTOCR */
#define NETC_PSEUDO_LINK_PPMTOCR_COUNT           (2U)

/*! @name PPMTUFCR - Port pseudo MAC transmit unicast frame counter register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMTUFCR_TUCA_MASK      (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMTUFCR_TUCA_SHIFT     (0U)
#define NETC_PSEUDO_LINK_PPMTUFCR_TUCA(x)        (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMTUFCR_TUCA_SHIFT)) & NETC_PSEUDO_LINK_PPMTUFCR_TUCA_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMTUFCR */
#define NETC_PSEUDO_LINK_PPMTUFCR_COUNT          (2U)

/*! @name PPMTMFCR - Port pseudo MAC transmit multicast frame counter register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMTMFCR_TMCA_MASK      (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMTMFCR_TMCA_SHIFT     (0U)
#define NETC_PSEUDO_LINK_PPMTMFCR_TMCA(x)        (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMTMFCR_TMCA_SHIFT)) & NETC_PSEUDO_LINK_PPMTMFCR_TMCA_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMTMFCR */
#define NETC_PSEUDO_LINK_PPMTMFCR_COUNT          (2U)

/*! @name PPMTBFCR - Port pseudo MAC transmit broadcast frame counter register */
/*! @{ */

#define NETC_PSEUDO_LINK_PPMTBFCR_TBCA_MASK      (0xFFFFFFFFU)
#define NETC_PSEUDO_LINK_PPMTBFCR_TBCA_SHIFT     (0U)
#define NETC_PSEUDO_LINK_PPMTBFCR_TBCA(x)        (((uint32_t)(((uint32_t)(x)) << NETC_PSEUDO_LINK_PPMTBFCR_TBCA_SHIFT)) & NETC_PSEUDO_LINK_PPMTBFCR_TBCA_MASK)
/*! @} */

/* The count of NETC_PSEUDO_LINK_PPMTBFCR */
#define NETC_PSEUDO_LINK_PPMTBFCR_COUNT          (2U)


/*!
 * @}
 */ /* end of group NETC_PSEUDO_LINK_Register_Masks */


/* NETC_PSEUDO_LINK - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ENETC1_PSEUDO_MAC_PORT base address */
  #define ENETC1_PSEUDO_MAC_PORT_BASE              (0x70B55000u)
  /** Peripheral ENETC1_PSEUDO_MAC_PORT base address */
  #define ENETC1_PSEUDO_MAC_PORT_BASE_NS           (0x60B55000u)
  /** Peripheral ENETC1_PSEUDO_MAC_PORT base pointer */
  #define ENETC1_PSEUDO_MAC_PORT                   ((NETC_PSEUDO_LINK_Type *)ENETC1_PSEUDO_MAC_PORT_BASE)
  /** Peripheral ENETC1_PSEUDO_MAC_PORT base pointer */
  #define ENETC1_PSEUDO_MAC_PORT_NS                ((NETC_PSEUDO_LINK_Type *)ENETC1_PSEUDO_MAC_PORT_BASE_NS)
  /** Peripheral SW0_PSEUDO_MAC_PORT4 base address */
  #define SW0_PSEUDO_MAC_PORT4_BASE                (0x70A15000u)
  /** Peripheral SW0_PSEUDO_MAC_PORT4 base address */
  #define SW0_PSEUDO_MAC_PORT4_BASE_NS             (0x60A15000u)
  /** Peripheral SW0_PSEUDO_MAC_PORT4 base pointer */
  #define SW0_PSEUDO_MAC_PORT4                     ((NETC_PSEUDO_LINK_Type *)SW0_PSEUDO_MAC_PORT4_BASE)
  /** Peripheral SW0_PSEUDO_MAC_PORT4 base pointer */
  #define SW0_PSEUDO_MAC_PORT4_NS                  ((NETC_PSEUDO_LINK_Type *)SW0_PSEUDO_MAC_PORT4_BASE_NS)
  /** Array initializer of NETC_PSEUDO_LINK peripheral base addresses */
  #define NETC_PSEUDO_LINK_BASE_ADDRS              { ENETC1_PSEUDO_MAC_PORT_BASE, SW0_PSEUDO_MAC_PORT4_BASE }
  /** Array initializer of NETC_PSEUDO_LINK peripheral base pointers */
  #define NETC_PSEUDO_LINK_BASE_PTRS               { ENETC1_PSEUDO_MAC_PORT, SW0_PSEUDO_MAC_PORT4 }
  /** Array initializer of NETC_PSEUDO_LINK peripheral base addresses */
  #define NETC_PSEUDO_LINK_BASE_ADDRS_NS           { ENETC1_PSEUDO_MAC_PORT_BASE_NS, SW0_PSEUDO_MAC_PORT4_BASE_NS }
  /** Array initializer of NETC_PSEUDO_LINK peripheral base pointers */
  #define NETC_PSEUDO_LINK_BASE_PTRS_NS            { ENETC1_PSEUDO_MAC_PORT_NS, SW0_PSEUDO_MAC_PORT4_NS }
#else
  /** Peripheral ENETC1_PSEUDO_MAC_PORT base address */
  #define ENETC1_PSEUDO_MAC_PORT_BASE              (0x60B55000u)
  /** Peripheral ENETC1_PSEUDO_MAC_PORT base pointer */
  #define ENETC1_PSEUDO_MAC_PORT                   ((NETC_PSEUDO_LINK_Type *)ENETC1_PSEUDO_MAC_PORT_BASE)
  /** Peripheral SW0_PSEUDO_MAC_PORT4 base address */
  #define SW0_PSEUDO_MAC_PORT4_BASE                (0x60A15000u)
  /** Peripheral SW0_PSEUDO_MAC_PORT4 base pointer */
  #define SW0_PSEUDO_MAC_PORT4                     ((NETC_PSEUDO_LINK_Type *)SW0_PSEUDO_MAC_PORT4_BASE)
  /** Array initializer of NETC_PSEUDO_LINK peripheral base addresses */
  #define NETC_PSEUDO_LINK_BASE_ADDRS              { ENETC1_PSEUDO_MAC_PORT_BASE, SW0_PSEUDO_MAC_PORT4_BASE }
  /** Array initializer of NETC_PSEUDO_LINK peripheral base pointers */
  #define NETC_PSEUDO_LINK_BASE_PTRS               { ENETC1_PSEUDO_MAC_PORT, SW0_PSEUDO_MAC_PORT4 }
#endif

/*!
 * @}
 */ /* end of group NETC_PSEUDO_LINK_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_SW Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_SW_Peripheral_Access_Layer NETC_SW Peripheral Access Layer
 * @{
 */

/** NETC_SW - Register Layout Typedef */
typedef struct {
  __I  uint32_t SCAPR0;                            /**< Switch capability register 0, offset: 0x0 */
  __I  uint32_t SCAPR1;                            /**< Switch capability register 1, offset: 0x4 */
  __I  uint32_t BPCAPR;                            /**< Buffer pool capability register, offset: 0x8 */
       uint8_t RESERVED_0[12];
  __I  uint32_t FCAPR;                             /**< Forwarding capability register, offset: 0x18 */
       uint8_t RESERVED_1[36];
  __I  uint32_t SMBCAPR;                           /**< Shared memory buffer capability register, offset: 0x40 */
  __I  uint32_t SMBOR0;                            /**< Shared memory buffer operational register 0, offset: 0x44 */
  __I  uint32_t SMBOR1;                            /**< Shared memory buffer operational register 1, offset: 0x48 */
       uint8_t RESERVED_2[52];
  __IO uint32_t CCAR;                              /**< Command cache attribute register, offset: 0x80 */
       uint8_t RESERVED_3[892];
  __I  uint32_t MPCR;                              /**< Management port configuration register, offset: 0x400 */
       uint8_t RESERVED_4[28];
  __IO uint32_t IMDCR0;                            /**< Ingress mirror destination configuration register 0, offset: 0x420 */
  __IO uint32_t IMDCR1;                            /**< Ingress mirror destination configuration register 1, offset: 0x424 */
       uint8_t RESERVED_5[24];
  __IO uint32_t CTFCR;                             /**< Cut-through forwarding count register, offset: 0x440 */
       uint8_t RESERVED_6[956];
  struct {                                         /* offset: 0x800, array step: 0x30 */
    __IO uint32_t CBDRMR;                            /**< Command BDR 0 mode register..Command BDR 1 mode register, array offset: 0x800, array step: 0x30 */
    __I  uint32_t CBDRSR;                            /**< Command BDR 0 status register..Command BDR 1 status register, array offset: 0x804, array step: 0x30 */
         uint8_t RESERVED_0[8];
    __IO uint32_t CBDRBAR0;                          /**< Command BDR base address register 0, array offset: 0x810, array step: 0x30 */
    __IO uint32_t CBDRBAR1;                          /**< Command BDR 0 base address register 1..Command BDR 1 base address register 1, array offset: 0x814, array step: 0x30 */
    __IO uint32_t CBDRPIR;                           /**< Command BDR 0 producer index register..Command BDR 1 producer index register, array offset: 0x818, array step: 0x30 */
    __IO uint32_t CBDRCIR;                           /**< Command BDR 0 consumer index register..Command BDR 1 consumer index register, array offset: 0x81C, array step: 0x30 */
    __IO uint32_t CBDRLENR;                          /**< Command BDR 0 length register..Command BDR 1 length register, array offset: 0x820, array step: 0x30 */
         uint8_t RESERVED_1[12];
  } NUM_CBDR[2];
       uint8_t RESERVED_7[64];
  struct {                                         /* offset: 0x8A0, array step: 0x10 */
    __IO uint32_t CBDRIER;                           /**< Command BDR 0 interrupt enable register..Command BDR 1 interrupt enable register, array offset: 0x8A0, array step: 0x10 */
    __IO uint32_t CBDRIDR;                           /**< Command BDR 0 interrupt detect register..Command BDR 1 interrupt detect register, array offset: 0x8A4, array step: 0x10 */
    __IO uint32_t CBDRMSIVR;                         /**< Command BDR 0 MSI-X vector register..Command BDR 1 MSI-X vector register, array offset: 0x8A8, array step: 0x10 */
         uint8_t RESERVED_0[4];
  } NUM_CBDR_INT[2];
       uint8_t RESERVED_8[64];
  struct {                                         /* offset: 0x900, array step: 0x20 */
    __IO uint32_t QOSVLANMPR[4];                     /**< QoS to VLAN mapping profile 0 register 0..QoS to VLAN mapping profile 1 register 3, array offset: 0x900, array step: index*0x20, index2*0x4 */
         uint8_t RESERVED_0[16];
  } MAP_PCP[2];
       uint8_t RESERVED_9[448];
  __IO uint32_t PCP2PCPMPR[2];                     /**< PCP to PCP mapping profile 0 register..PCP to PCP mapping profile 1 register, array offset: 0xB00, array step: 0x4 */
       uint8_t RESERVED_10[5368];
  __I  uint32_t BRCAPR;                            /**< Bridge capability register, offset: 0x2000 */
       uint8_t RESERVED_11[4];
  __I  uint32_t VFHTCAPR;                          /**< VLAN filter hash table capability register, offset: 0x2008 */
  __I  uint32_t VFHTOR;                            /**< VLAN filter hash table operational register, offset: 0x200C */
  __IO uint32_t VFHTDECR0;                         /**< VLAN Filter (hash) table default entry configuration registers 0, offset: 0x2010 */
  __IO uint32_t VFHTDECR1;                         /**< VLAN filter hash table default entry configuration registers 1, offset: 0x2014 */
  __IO uint32_t VFHTDECR2;                         /**< VLAN filter hash table default entry configuration registers 2, offset: 0x2018 */
       uint8_t RESERVED_12[4];
  __I  uint32_t FDBHTCAPR;                         /**< FDB hash table capability register, offset: 0x2020 */
  __IO uint32_t FDBHTMCR;                          /**< FDB hash table memory configuration register, offset: 0x2024 */
  __I  uint32_t FDBHTOR0;                          /**< FDB hash table operational register 0, offset: 0x2028 */
  __I  uint32_t FDBHTOR1;                          /**< FDB hash table operational register 1, offset: 0x202C */
       uint8_t RESERVED_13[16];
  __I  uint32_t IPMFHTCAPR;                        /**< IP multicast filter hash table capability register, offset: 0x2040 */
  __I  uint32_t IPV4MFHTOR;                        /**< IPv4 multicast filter hash table operation register, offset: 0x2044 */
} NETC_SW_Type;

/* ----------------------------------------------------------------------------
   -- NETC_SW Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_SW_Register_Masks NETC_SW Register Masks
 * @{
 */

/*! @name SCAPR0 - Switch capability register 0 */
/*! @{ */

#define NETC_SW_SCAPR0_NUM_PORT_MASK             (0x1FU)
#define NETC_SW_SCAPR0_NUM_PORT_SHIFT            (0U)
#define NETC_SW_SCAPR0_NUM_PORT(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR0_NUM_PORT_SHIFT)) & NETC_SW_SCAPR0_NUM_PORT_MASK)

#define NETC_SW_SCAPR0_NUM_IPV_MASK              (0x100U)
#define NETC_SW_SCAPR0_NUM_IPV_SHIFT             (8U)
#define NETC_SW_SCAPR0_NUM_IPV(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR0_NUM_IPV_SHIFT)) & NETC_SW_SCAPR0_NUM_IPV_MASK)

#define NETC_SW_SCAPR0_NUM_MSIX_MASK             (0xF0000U)
#define NETC_SW_SCAPR0_NUM_MSIX_SHIFT            (16U)
#define NETC_SW_SCAPR0_NUM_MSIX(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR0_NUM_MSIX_SHIFT)) & NETC_SW_SCAPR0_NUM_MSIX_MASK)

#define NETC_SW_SCAPR0_NUM_PCPMP_MASK            (0xF00000U)
#define NETC_SW_SCAPR0_NUM_PCPMP_SHIFT           (20U)
#define NETC_SW_SCAPR0_NUM_PCPMP(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR0_NUM_PCPMP_SHIFT)) & NETC_SW_SCAPR0_NUM_PCPMP_MASK)

#define NETC_SW_SCAPR0_NUM_QVMP_MASK             (0xF000000U)
#define NETC_SW_SCAPR0_NUM_QVMP_SHIFT            (24U)
#define NETC_SW_SCAPR0_NUM_QVMP(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR0_NUM_QVMP_SHIFT)) & NETC_SW_SCAPR0_NUM_QVMP_MASK)
/*! @} */

/*! @name SCAPR1 - Switch capability register 1 */
/*! @{ */

#define NETC_SW_SCAPR1_FS_MASK                   (0x1U)
#define NETC_SW_SCAPR1_FS_SHIFT                  (0U)
/*! FS - Functional safety capability supported. */
#define NETC_SW_SCAPR1_FS(x)                     (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR1_FS_SHIFT)) & NETC_SW_SCAPR1_FS_MASK)

#define NETC_SW_SCAPR1_CTF_MASK                  (0x4U)
#define NETC_SW_SCAPR1_CTF_SHIFT                 (2U)
#define NETC_SW_SCAPR1_CTF(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR1_CTF_SHIFT)) & NETC_SW_SCAPR1_CTF_MASK)

#define NETC_SW_SCAPR1_TIMCAP_MASK               (0x8U)
#define NETC_SW_SCAPR1_TIMCAP_SHIFT              (3U)
#define NETC_SW_SCAPR1_TIMCAP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR1_TIMCAP_SHIFT)) & NETC_SW_SCAPR1_TIMCAP_MASK)

#define NETC_SW_SCAPR1_IMIR_MASK                 (0x10U)
#define NETC_SW_SCAPR1_IMIR_SHIFT                (4U)
#define NETC_SW_SCAPR1_IMIR(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR1_IMIR_SHIFT)) & NETC_SW_SCAPR1_IMIR_MASK)

#define NETC_SW_SCAPR1_SQ_TAGS_MASK              (0x1F0000U)
#define NETC_SW_SCAPR1_SQ_TAGS_SHIFT             (16U)
#define NETC_SW_SCAPR1_SQ_TAGS(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_SCAPR1_SQ_TAGS_SHIFT)) & NETC_SW_SCAPR1_SQ_TAGS_MASK)
/*! @} */

/*! @name BPCAPR - Buffer pool capability register */
/*! @{ */

#define NETC_SW_BPCAPR_NUM_BP_MASK               (0xFFU)
#define NETC_SW_BPCAPR_NUM_BP_SHIFT              (0U)
#define NETC_SW_BPCAPR_NUM_BP(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_BPCAPR_NUM_BP_SHIFT)) & NETC_SW_BPCAPR_NUM_BP_MASK)

#define NETC_SW_BPCAPR_NUM_SPB_MASK              (0x1F0000U)
#define NETC_SW_BPCAPR_NUM_SPB_SHIFT             (16U)
#define NETC_SW_BPCAPR_NUM_SPB(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_BPCAPR_NUM_SPB_SHIFT)) & NETC_SW_BPCAPR_NUM_SPB_MASK)
/*! @} */

/*! @name FCAPR - Forwarding capability register */
/*! @{ */

#define NETC_SW_FCAPR_BR_MASK                    (0x1U)
#define NETC_SW_FCAPR_BR_SHIFT                   (0U)
/*! BR - 802.1Q bridge forwarding support.
 *  0b0..Not supported
 *  0b1..Supported
 */
#define NETC_SW_FCAPR_BR(x)                      (((uint32_t)(((uint32_t)(x)) << NETC_SW_FCAPR_BR_SHIFT)) & NETC_SW_FCAPR_BR_MASK)

#define NETC_SW_FCAPR_SF_MASK                    (0x2U)
#define NETC_SW_FCAPR_SF_SHIFT                   (1U)
/*! SF - Stream forwarding supported
 *  0b0..Not supported
 *  0b1..Supported
 */
#define NETC_SW_FCAPR_SF(x)                      (((uint32_t)(((uint32_t)(x)) << NETC_SW_FCAPR_SF_SHIFT)) & NETC_SW_FCAPR_SF_MASK)
/*! @} */

/*! @name SMBCAPR - Shared memory buffer capability register */
/*! @{ */

#define NETC_SW_SMBCAPR_NUM_WORDS_MASK           (0xFFFFFFU)
#define NETC_SW_SMBCAPR_NUM_WORDS_SHIFT          (0U)
#define NETC_SW_SMBCAPR_NUM_WORDS(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_SMBCAPR_NUM_WORDS_SHIFT)) & NETC_SW_SMBCAPR_NUM_WORDS_MASK)

#define NETC_SW_SMBCAPR_WORD_SIZE_MASK           (0x30000000U)
#define NETC_SW_SMBCAPR_WORD_SIZE_SHIFT          (28U)
/*! WORD_SIZE
 *  0b00..24 bytes
 *  0b01-0b11..
 */
#define NETC_SW_SMBCAPR_WORD_SIZE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_SMBCAPR_WORD_SIZE_SHIFT)) & NETC_SW_SMBCAPR_WORD_SIZE_MASK)

#define NETC_SW_SMBCAPR_MLOC_MASK                (0xC0000000U)
#define NETC_SW_SMBCAPR_MLOC_SHIFT               (30U)
/*! MLOC - Indicates memory location
 *  0b00..Common memory
 *  0b01-0b11..
 */
#define NETC_SW_SMBCAPR_MLOC(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_SW_SMBCAPR_MLOC_SHIFT)) & NETC_SW_SMBCAPR_MLOC_MASK)
/*! @} */

/*! @name SMBOR0 - Shared memory buffer operational register 0 */
/*! @{ */

#define NETC_SW_SMBOR0_COUNT_MASK                (0xFFFFFFU)
#define NETC_SW_SMBOR0_COUNT_SHIFT               (0U)
#define NETC_SW_SMBOR0_COUNT(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_SW_SMBOR0_COUNT_SHIFT)) & NETC_SW_SMBOR0_COUNT_MASK)
/*! @} */

/*! @name SMBOR1 - Shared memory buffer operational register 1 */
/*! @{ */

#define NETC_SW_SMBOR1_WATERMARK_MASK            (0xFFFFFFU)
#define NETC_SW_SMBOR1_WATERMARK_SHIFT           (0U)
#define NETC_SW_SMBOR1_WATERMARK(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_SMBOR1_WATERMARK_SHIFT)) & NETC_SW_SMBOR1_WATERMARK_MASK)
/*! @} */

/*! @name CCAR - Command cache attribute register */
/*! @{ */

#define NETC_SW_CCAR_CBD_WRCACHE_MASK            (0xFU)
#define NETC_SW_CCAR_CBD_WRCACHE_SHIFT           (0U)
#define NETC_SW_CCAR_CBD_WRCACHE(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CBD_WRCACHE_SHIFT)) & NETC_SW_CCAR_CBD_WRCACHE_MASK)

#define NETC_SW_CCAR_CBD_WRDOMAIN_MASK           (0x30U)
#define NETC_SW_CCAR_CBD_WRDOMAIN_SHIFT          (4U)
#define NETC_SW_CCAR_CBD_WRDOMAIN(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CBD_WRDOMAIN_SHIFT)) & NETC_SW_CCAR_CBD_WRDOMAIN_MASK)

#define NETC_SW_CCAR_CBD_WRSNP_MASK              (0x40U)
#define NETC_SW_CCAR_CBD_WRSNP_SHIFT             (6U)
#define NETC_SW_CCAR_CBD_WRSNP(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CBD_WRSNP_SHIFT)) & NETC_SW_CCAR_CBD_WRSNP_MASK)

#define NETC_SW_CCAR_CWRCACHE_MASK               (0xF00U)
#define NETC_SW_CCAR_CWRCACHE_SHIFT              (8U)
#define NETC_SW_CCAR_CWRCACHE(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CWRCACHE_SHIFT)) & NETC_SW_CCAR_CWRCACHE_MASK)

#define NETC_SW_CCAR_CWRDOMAIN_MASK              (0x3000U)
#define NETC_SW_CCAR_CWRDOMAIN_SHIFT             (12U)
#define NETC_SW_CCAR_CWRDOMAIN(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CWRDOMAIN_SHIFT)) & NETC_SW_CCAR_CWRDOMAIN_MASK)

#define NETC_SW_CCAR_CWRSNP_MASK                 (0x4000U)
#define NETC_SW_CCAR_CWRSNP_SHIFT                (14U)
#define NETC_SW_CCAR_CWRSNP(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CWRSNP_SHIFT)) & NETC_SW_CCAR_CWRSNP_MASK)

#define NETC_SW_CCAR_CBD_RDCACHE_MASK            (0xF0000U)
#define NETC_SW_CCAR_CBD_RDCACHE_SHIFT           (16U)
#define NETC_SW_CCAR_CBD_RDCACHE(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CBD_RDCACHE_SHIFT)) & NETC_SW_CCAR_CBD_RDCACHE_MASK)

#define NETC_SW_CCAR_CBD_RDDOMAIN_MASK           (0x300000U)
#define NETC_SW_CCAR_CBD_RDDOMAIN_SHIFT          (20U)
#define NETC_SW_CCAR_CBD_RDDOMAIN(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CBD_RDDOMAIN_SHIFT)) & NETC_SW_CCAR_CBD_RDDOMAIN_MASK)

#define NETC_SW_CCAR_CBD_RDSNP_MASK              (0x400000U)
#define NETC_SW_CCAR_CBD_RDSNP_SHIFT             (22U)
#define NETC_SW_CCAR_CBD_RDSNP(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CBD_RDSNP_SHIFT)) & NETC_SW_CCAR_CBD_RDSNP_MASK)

#define NETC_SW_CCAR_CRDCACHE_MASK               (0xF000000U)
#define NETC_SW_CCAR_CRDCACHE_SHIFT              (24U)
#define NETC_SW_CCAR_CRDCACHE(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CRDCACHE_SHIFT)) & NETC_SW_CCAR_CRDCACHE_MASK)

#define NETC_SW_CCAR_CRDDOMAIN_MASK              (0x30000000U)
#define NETC_SW_CCAR_CRDDOMAIN_SHIFT             (28U)
#define NETC_SW_CCAR_CRDDOMAIN(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CRDDOMAIN_SHIFT)) & NETC_SW_CCAR_CRDDOMAIN_MASK)

#define NETC_SW_CCAR_CRDSNP_MASK                 (0x40000000U)
#define NETC_SW_CCAR_CRDSNP_SHIFT                (30U)
#define NETC_SW_CCAR_CRDSNP(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_SW_CCAR_CRDSNP_SHIFT)) & NETC_SW_CCAR_CRDSNP_MASK)
/*! @} */

/*! @name MPCR - Management port configuration register */
/*! @{ */

#define NETC_SW_MPCR_PORT_MASK                   (0x1FU)
#define NETC_SW_MPCR_PORT_SHIFT                  (0U)
/*! PORT - Switch Management Port */
#define NETC_SW_MPCR_PORT(x)                     (((uint32_t)(((uint32_t)(x)) << NETC_SW_MPCR_PORT_SHIFT)) & NETC_SW_MPCR_PORT_MASK)
/*! @} */

/*! @name IMDCR0 - Ingress mirror destination configuration register 0 */
/*! @{ */

#define NETC_SW_IMDCR0_MIREN_MASK                (0x1U)
#define NETC_SW_IMDCR0_MIREN_SHIFT               (0U)
/*! MIREN - Mirror enable.
 *  0b0..Ingress mirroring disabled
 *  0b1..Ingress mirroring enabled
 */
#define NETC_SW_IMDCR0_MIREN(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_SW_IMDCR0_MIREN_SHIFT)) & NETC_SW_IMDCR0_MIREN_MASK)

#define NETC_SW_IMDCR0_MIRDEST_MASK              (0x2U)
#define NETC_SW_IMDCR0_MIRDEST_SHIFT             (1U)
/*! MIRDEST - Indicates the mirror destination */
#define NETC_SW_IMDCR0_MIRDEST(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_IMDCR0_MIRDEST_SHIFT)) & NETC_SW_IMDCR0_MIRDEST_MASK)

#define NETC_SW_IMDCR0_IPV_MASK                  (0x1CU)
#define NETC_SW_IMDCR0_IPV_SHIFT                 (2U)
#define NETC_SW_IMDCR0_IPV(x)                    (((uint32_t)(((uint32_t)(x)) << NETC_SW_IMDCR0_IPV_SHIFT)) & NETC_SW_IMDCR0_IPV_MASK)

#define NETC_SW_IMDCR0_DR_MASK                   (0xC0U)
#define NETC_SW_IMDCR0_DR_SHIFT                  (6U)
#define NETC_SW_IMDCR0_DR(x)                     (((uint32_t)(((uint32_t)(x)) << NETC_SW_IMDCR0_DR_SHIFT)) & NETC_SW_IMDCR0_DR_MASK)

#define NETC_SW_IMDCR0_PORT_MASK                 (0x1F00U)
#define NETC_SW_IMDCR0_PORT_SHIFT                (8U)
#define NETC_SW_IMDCR0_PORT(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_SW_IMDCR0_PORT_SHIFT)) & NETC_SW_IMDCR0_PORT_MASK)
/*! @} */

/*! @name IMDCR1 - Ingress mirror destination configuration register 1 */
/*! @{ */

#define NETC_SW_IMDCR1_EFMEID_MASK               (0xFFFFU)
#define NETC_SW_IMDCR1_EFMEID_SHIFT              (0U)
#define NETC_SW_IMDCR1_EFMEID(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_IMDCR1_EFMEID_SHIFT)) & NETC_SW_IMDCR1_EFMEID_MASK)

#define NETC_SW_IMDCR1_EFM_LEN_CHANGE_MASK       (0x7F0000U)
#define NETC_SW_IMDCR1_EFM_LEN_CHANGE_SHIFT      (16U)
#define NETC_SW_IMDCR1_EFM_LEN_CHANGE(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_IMDCR1_EFM_LEN_CHANGE_SHIFT)) & NETC_SW_IMDCR1_EFM_LEN_CHANGE_MASK)
/*! @} */

/*! @name CTFCR - Cut-through forwarding count register */
/*! @{ */

#define NETC_SW_CTFCR_COUNT_MASK                 (0xFFFFFFFFU)
#define NETC_SW_CTFCR_COUNT_SHIFT                (0U)
#define NETC_SW_CTFCR_COUNT(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_SW_CTFCR_COUNT_SHIFT)) & NETC_SW_CTFCR_COUNT_MASK)
/*! @} */

/*! @name CBDRMR - Command BDR 0 mode register..Command BDR 1 mode register */
/*! @{ */

#define NETC_SW_CBDRMR_EN_MASK                   (0x80000000U)
#define NETC_SW_CBDRMR_EN_SHIFT                  (31U)
/*! EN - Enable command buffer descriptor ring
 *  0b0..Disabled
 *  0b1..Enabled. When the ring is non-empty, command buffer descriptors will be processed
 */
#define NETC_SW_CBDRMR_EN(x)                     (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRMR_EN_SHIFT)) & NETC_SW_CBDRMR_EN_MASK)
/*! @} */

/* The count of NETC_SW_CBDRMR */
#define NETC_SW_CBDRMR_COUNT                     (2U)

/*! @name CBDRSR - Command BDR 0 status register..Command BDR 1 status register */
/*! @{ */

#define NETC_SW_CBDRSR_BUSY_MASK                 (0x1U)
#define NETC_SW_CBDRSR_BUSY_SHIFT                (0U)
/*! BUSY - Busy.
 *  0b0..Idle
 *  0b1..Busy
 */
#define NETC_SW_CBDRSR_BUSY(x)                   (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRSR_BUSY_SHIFT)) & NETC_SW_CBDRSR_BUSY_MASK)
/*! @} */

/* The count of NETC_SW_CBDRSR */
#define NETC_SW_CBDRSR_COUNT                     (2U)

/*! @name CBDRBAR0 - Command BDR base address register 0 */
/*! @{ */

#define NETC_SW_CBDRBAR0_ADDRL_MASK              (0xFFFFFF80U)
#define NETC_SW_CBDRBAR0_ADDRL_SHIFT             (7U)
#define NETC_SW_CBDRBAR0_ADDRL(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRBAR0_ADDRL_SHIFT)) & NETC_SW_CBDRBAR0_ADDRL_MASK)
/*! @} */

/* The count of NETC_SW_CBDRBAR0 */
#define NETC_SW_CBDRBAR0_COUNT                   (2U)

/*! @name CBDRBAR1 - Command BDR 0 base address register 1..Command BDR 1 base address register 1 */
/*! @{ */

#define NETC_SW_CBDRBAR1_ADDRH_MASK              (0xFFFFFFFFU)
#define NETC_SW_CBDRBAR1_ADDRH_SHIFT             (0U)
#define NETC_SW_CBDRBAR1_ADDRH(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRBAR1_ADDRH_SHIFT)) & NETC_SW_CBDRBAR1_ADDRH_MASK)
/*! @} */

/* The count of NETC_SW_CBDRBAR1 */
#define NETC_SW_CBDRBAR1_COUNT                   (2U)

/*! @name CBDRPIR - Command BDR 0 producer index register..Command BDR 1 producer index register */
/*! @{ */

#define NETC_SW_CBDRPIR_BDR_INDEX_MASK           (0x3FFU)
#define NETC_SW_CBDRPIR_BDR_INDEX_SHIFT          (0U)
#define NETC_SW_CBDRPIR_BDR_INDEX(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRPIR_BDR_INDEX_SHIFT)) & NETC_SW_CBDRPIR_BDR_INDEX_MASK)
/*! @} */

/* The count of NETC_SW_CBDRPIR */
#define NETC_SW_CBDRPIR_COUNT                    (2U)

/*! @name CBDRCIR - Command BDR 0 consumer index register..Command BDR 1 consumer index register */
/*! @{ */

#define NETC_SW_CBDRCIR_BDR_INDEX_MASK           (0x3FFU)
#define NETC_SW_CBDRCIR_BDR_INDEX_SHIFT          (0U)
#define NETC_SW_CBDRCIR_BDR_INDEX(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRCIR_BDR_INDEX_SHIFT)) & NETC_SW_CBDRCIR_BDR_INDEX_MASK)
/*! @} */

/* The count of NETC_SW_CBDRCIR */
#define NETC_SW_CBDRCIR_COUNT                    (2U)

/*! @name CBDRLENR - Command BDR 0 length register..Command BDR 1 length register */
/*! @{ */

#define NETC_SW_CBDRLENR_LENGTH_MASK             (0x7F8U)
#define NETC_SW_CBDRLENR_LENGTH_SHIFT            (3U)
#define NETC_SW_CBDRLENR_LENGTH(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRLENR_LENGTH_SHIFT)) & NETC_SW_CBDRLENR_LENGTH_MASK)
/*! @} */

/* The count of NETC_SW_CBDRLENR */
#define NETC_SW_CBDRLENR_COUNT                   (2U)

/*! @name CBDRIER - Command BDR 0 interrupt enable register..Command BDR 1 interrupt enable register */
/*! @{ */

#define NETC_SW_CBDRIER_CBDCIE_MASK              (0x1U)
#define NETC_SW_CBDRIER_CBDCIE_SHIFT             (0U)
#define NETC_SW_CBDRIER_CBDCIE(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRIER_CBDCIE_SHIFT)) & NETC_SW_CBDRIER_CBDCIE_MASK)
/*! @} */

/* The count of NETC_SW_CBDRIER */
#define NETC_SW_CBDRIER_COUNT                    (2U)

/*! @name CBDRIDR - Command BDR 0 interrupt detect register..Command BDR 1 interrupt detect register */
/*! @{ */

#define NETC_SW_CBDRIDR_CBDC_MASK                (0x1U)
#define NETC_SW_CBDRIDR_CBDC_SHIFT               (0U)
/*! CBDC
 *  0b0..No BD with CI=1 completed
 *  0b1..Processed BD with CI=1
 */
#define NETC_SW_CBDRIDR_CBDC(x)                  (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRIDR_CBDC_SHIFT)) & NETC_SW_CBDRIDR_CBDC_MASK)
/*! @} */

/* The count of NETC_SW_CBDRIDR */
#define NETC_SW_CBDRIDR_COUNT                    (2U)

/*! @name CBDRMSIVR - Command BDR 0 MSI-X vector register..Command BDR 1 MSI-X vector register */
/*! @{ */

#define NETC_SW_CBDRMSIVR_VECTOR_MASK            (0xFU)
#define NETC_SW_CBDRMSIVR_VECTOR_SHIFT           (0U)
#define NETC_SW_CBDRMSIVR_VECTOR(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_CBDRMSIVR_VECTOR_SHIFT)) & NETC_SW_CBDRMSIVR_VECTOR_MASK)
/*! @} */

/* The count of NETC_SW_CBDRMSIVR */
#define NETC_SW_CBDRMSIVR_COUNT                  (2U)

/*! @name QOSVLANMPR - QoS to VLAN mapping profile 0 register 0..QoS to VLAN mapping profile 1 register 3 */
/*! @{ */

#define NETC_SW_QOSVLANMPR_IPV0_DR0_MASK         (0xFU)
#define NETC_SW_QOSVLANMPR_IPV0_DR0_SHIFT        (0U)
#define NETC_SW_QOSVLANMPR_IPV0_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV0_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV0_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV2_DR0_MASK         (0xFU)
#define NETC_SW_QOSVLANMPR_IPV2_DR0_SHIFT        (0U)
#define NETC_SW_QOSVLANMPR_IPV2_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV2_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV2_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV4_DR0_MASK         (0xFU)
#define NETC_SW_QOSVLANMPR_IPV4_DR0_SHIFT        (0U)
#define NETC_SW_QOSVLANMPR_IPV4_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV4_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV4_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV6_DR0_MASK         (0xFU)
#define NETC_SW_QOSVLANMPR_IPV6_DR0_SHIFT        (0U)
#define NETC_SW_QOSVLANMPR_IPV6_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV6_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV6_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV0_DR1_MASK         (0xF0U)
#define NETC_SW_QOSVLANMPR_IPV0_DR1_SHIFT        (4U)
#define NETC_SW_QOSVLANMPR_IPV0_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV0_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV0_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV2_DR1_MASK         (0xF0U)
#define NETC_SW_QOSVLANMPR_IPV2_DR1_SHIFT        (4U)
#define NETC_SW_QOSVLANMPR_IPV2_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV2_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV2_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV4_DR1_MASK         (0xF0U)
#define NETC_SW_QOSVLANMPR_IPV4_DR1_SHIFT        (4U)
#define NETC_SW_QOSVLANMPR_IPV4_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV4_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV4_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV6_DR1_MASK         (0xF0U)
#define NETC_SW_QOSVLANMPR_IPV6_DR1_SHIFT        (4U)
#define NETC_SW_QOSVLANMPR_IPV6_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV6_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV6_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV0_DR2_MASK         (0xF00U)
#define NETC_SW_QOSVLANMPR_IPV0_DR2_SHIFT        (8U)
#define NETC_SW_QOSVLANMPR_IPV0_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV0_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV0_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV2_DR2_MASK         (0xF00U)
#define NETC_SW_QOSVLANMPR_IPV2_DR2_SHIFT        (8U)
#define NETC_SW_QOSVLANMPR_IPV2_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV2_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV2_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV4_DR2_MASK         (0xF00U)
#define NETC_SW_QOSVLANMPR_IPV4_DR2_SHIFT        (8U)
#define NETC_SW_QOSVLANMPR_IPV4_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV4_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV4_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV6_DR2_MASK         (0xF00U)
#define NETC_SW_QOSVLANMPR_IPV6_DR2_SHIFT        (8U)
#define NETC_SW_QOSVLANMPR_IPV6_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV6_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV6_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV0_DR3_MASK         (0xF000U)
#define NETC_SW_QOSVLANMPR_IPV0_DR3_SHIFT        (12U)
#define NETC_SW_QOSVLANMPR_IPV0_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV0_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV0_DR3_MASK)

#define NETC_SW_QOSVLANMPR_IPV2_DR3_MASK         (0xF000U)
#define NETC_SW_QOSVLANMPR_IPV2_DR3_SHIFT        (12U)
#define NETC_SW_QOSVLANMPR_IPV2_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV2_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV2_DR3_MASK)

#define NETC_SW_QOSVLANMPR_IPV4_DR3_MASK         (0xF000U)
#define NETC_SW_QOSVLANMPR_IPV4_DR3_SHIFT        (12U)
#define NETC_SW_QOSVLANMPR_IPV4_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV4_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV4_DR3_MASK)

#define NETC_SW_QOSVLANMPR_IPV6_DR3_MASK         (0xF000U)
#define NETC_SW_QOSVLANMPR_IPV6_DR3_SHIFT        (12U)
#define NETC_SW_QOSVLANMPR_IPV6_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV6_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV6_DR3_MASK)

#define NETC_SW_QOSVLANMPR_IPV1_DR0_MASK         (0xF0000U)
#define NETC_SW_QOSVLANMPR_IPV1_DR0_SHIFT        (16U)
#define NETC_SW_QOSVLANMPR_IPV1_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV1_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV1_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV3_DR0_MASK         (0xF0000U)
#define NETC_SW_QOSVLANMPR_IPV3_DR0_SHIFT        (16U)
#define NETC_SW_QOSVLANMPR_IPV3_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV3_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV3_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV5_DR0_MASK         (0xF0000U)
#define NETC_SW_QOSVLANMPR_IPV5_DR0_SHIFT        (16U)
#define NETC_SW_QOSVLANMPR_IPV5_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV5_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV5_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV7_DR0_MASK         (0xF0000U)
#define NETC_SW_QOSVLANMPR_IPV7_DR0_SHIFT        (16U)
#define NETC_SW_QOSVLANMPR_IPV7_DR0(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV7_DR0_SHIFT)) & NETC_SW_QOSVLANMPR_IPV7_DR0_MASK)

#define NETC_SW_QOSVLANMPR_IPV1_DR1_MASK         (0xF00000U)
#define NETC_SW_QOSVLANMPR_IPV1_DR1_SHIFT        (20U)
#define NETC_SW_QOSVLANMPR_IPV1_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV1_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV1_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV3_DR1_MASK         (0xF00000U)
#define NETC_SW_QOSVLANMPR_IPV3_DR1_SHIFT        (20U)
#define NETC_SW_QOSVLANMPR_IPV3_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV3_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV3_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV5_DR1_MASK         (0xF00000U)
#define NETC_SW_QOSVLANMPR_IPV5_DR1_SHIFT        (20U)
#define NETC_SW_QOSVLANMPR_IPV5_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV5_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV5_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV7_DR1_MASK         (0xF00000U)
#define NETC_SW_QOSVLANMPR_IPV7_DR1_SHIFT        (20U)
#define NETC_SW_QOSVLANMPR_IPV7_DR1(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV7_DR1_SHIFT)) & NETC_SW_QOSVLANMPR_IPV7_DR1_MASK)

#define NETC_SW_QOSVLANMPR_IPV1_DR2_MASK         (0xF000000U)
#define NETC_SW_QOSVLANMPR_IPV1_DR2_SHIFT        (24U)
#define NETC_SW_QOSVLANMPR_IPV1_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV1_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV1_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV3_DR2_MASK         (0xF000000U)
#define NETC_SW_QOSVLANMPR_IPV3_DR2_SHIFT        (24U)
#define NETC_SW_QOSVLANMPR_IPV3_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV3_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV3_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV5_DR2_MASK         (0xF000000U)
#define NETC_SW_QOSVLANMPR_IPV5_DR2_SHIFT        (24U)
#define NETC_SW_QOSVLANMPR_IPV5_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV5_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV5_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV7_DR2_MASK         (0xF000000U)
#define NETC_SW_QOSVLANMPR_IPV7_DR2_SHIFT        (24U)
#define NETC_SW_QOSVLANMPR_IPV7_DR2(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV7_DR2_SHIFT)) & NETC_SW_QOSVLANMPR_IPV7_DR2_MASK)

#define NETC_SW_QOSVLANMPR_IPV1_DR3_MASK         (0xF0000000U)
#define NETC_SW_QOSVLANMPR_IPV1_DR3_SHIFT        (28U)
#define NETC_SW_QOSVLANMPR_IPV1_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV1_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV1_DR3_MASK)

#define NETC_SW_QOSVLANMPR_IPV3_DR3_MASK         (0xF0000000U)
#define NETC_SW_QOSVLANMPR_IPV3_DR3_SHIFT        (28U)
#define NETC_SW_QOSVLANMPR_IPV3_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV3_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV3_DR3_MASK)

#define NETC_SW_QOSVLANMPR_IPV5_DR3_MASK         (0xF0000000U)
#define NETC_SW_QOSVLANMPR_IPV5_DR3_SHIFT        (28U)
#define NETC_SW_QOSVLANMPR_IPV5_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV5_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV5_DR3_MASK)

#define NETC_SW_QOSVLANMPR_IPV7_DR3_MASK         (0xF0000000U)
#define NETC_SW_QOSVLANMPR_IPV7_DR3_SHIFT        (28U)
#define NETC_SW_QOSVLANMPR_IPV7_DR3(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_QOSVLANMPR_IPV7_DR3_SHIFT)) & NETC_SW_QOSVLANMPR_IPV7_DR3_MASK)
/*! @} */

/* The count of NETC_SW_QOSVLANMPR */
#define NETC_SW_QOSVLANMPR_COUNT                 (2U)

/* The count of NETC_SW_QOSVLANMPR */
#define NETC_SW_QOSVLANMPR_COUNT2                (4U)

/*! @name PCP2PCPMPR - PCP to PCP mapping profile 0 register..PCP to PCP mapping profile 1 register */
/*! @{ */

#define NETC_SW_PCP2PCPMPR_PCP0_MASK             (0x7U)
#define NETC_SW_PCP2PCPMPR_PCP0_SHIFT            (0U)
#define NETC_SW_PCP2PCPMPR_PCP0(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP0_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP0_MASK)

#define NETC_SW_PCP2PCPMPR_PCP1_MASK             (0x70U)
#define NETC_SW_PCP2PCPMPR_PCP1_SHIFT            (4U)
#define NETC_SW_PCP2PCPMPR_PCP1(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP1_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP1_MASK)

#define NETC_SW_PCP2PCPMPR_PCP2_MASK             (0x700U)
#define NETC_SW_PCP2PCPMPR_PCP2_SHIFT            (8U)
#define NETC_SW_PCP2PCPMPR_PCP2(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP2_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP2_MASK)

#define NETC_SW_PCP2PCPMPR_PCP3_MASK             (0x7000U)
#define NETC_SW_PCP2PCPMPR_PCP3_SHIFT            (12U)
#define NETC_SW_PCP2PCPMPR_PCP3(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP3_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP3_MASK)

#define NETC_SW_PCP2PCPMPR_PCP4_MASK             (0x70000U)
#define NETC_SW_PCP2PCPMPR_PCP4_SHIFT            (16U)
#define NETC_SW_PCP2PCPMPR_PCP4(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP4_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP4_MASK)

#define NETC_SW_PCP2PCPMPR_PCP5_MASK             (0x700000U)
#define NETC_SW_PCP2PCPMPR_PCP5_SHIFT            (20U)
#define NETC_SW_PCP2PCPMPR_PCP5(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP5_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP5_MASK)

#define NETC_SW_PCP2PCPMPR_PCP6_MASK             (0x7000000U)
#define NETC_SW_PCP2PCPMPR_PCP6_SHIFT            (24U)
#define NETC_SW_PCP2PCPMPR_PCP6(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP6_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP6_MASK)

#define NETC_SW_PCP2PCPMPR_PCP7_MASK             (0x70000000U)
#define NETC_SW_PCP2PCPMPR_PCP7_SHIFT            (28U)
#define NETC_SW_PCP2PCPMPR_PCP7(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_PCP2PCPMPR_PCP7_SHIFT)) & NETC_SW_PCP2PCPMPR_PCP7_MASK)
/*! @} */

/* The count of NETC_SW_PCP2PCPMPR */
#define NETC_SW_PCP2PCPMPR_COUNT                 (2U)

/*! @name BRCAPR - Bridge capability register */
/*! @{ */

#define NETC_SW_BRCAPR_IPV4MFLT_MASK             (0x1U)
#define NETC_SW_BRCAPR_IPV4MFLT_SHIFT            (0U)
/*! IPV4MFLT - L2 IPv4 multicast filtering supported.
 *  0b0..Not supported
 *  0b1..Supported
 */
#define NETC_SW_BRCAPR_IPV4MFLT(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_BRCAPR_IPV4MFLT_SHIFT)) & NETC_SW_BRCAPR_IPV4MFLT_MASK)

#define NETC_SW_BRCAPR_STAMVD_MASK               (0x4U)
#define NETC_SW_BRCAPR_STAMVD_SHIFT              (2U)
/*! STAMVD - Station Move Disable supported
 *  0b0..Not supported
 *  0b1..Supported
 */
#define NETC_SW_BRCAPR_STAMVD(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_BRCAPR_STAMVD_SHIFT)) & NETC_SW_BRCAPR_STAMVD_MASK)

#define NETC_SW_BRCAPR_STRMCTRL_MASK             (0x8U)
#define NETC_SW_BRCAPR_STRMCTRL_SHIFT            (3U)
/*! STRMCTRL - Storm Control supported.
 *  0b0..Not supported
 *  0b1..Supported
 */
#define NETC_SW_BRCAPR_STRMCTRL(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_BRCAPR_STRMCTRL_SHIFT)) & NETC_SW_BRCAPR_STRMCTRL_MASK)

#define NETC_SW_BRCAPR_SRCPPRND_MASK             (0x10U)
#define NETC_SW_BRCAPR_SRCPPRND_SHIFT            (4U)
/*! SRCPPRND - Source port pruning disable supported
 *  0b0..Source port pruning disable not supported
 *  0b1..Source port pruning disable supported
 */
#define NETC_SW_BRCAPR_SRCPPRND(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_BRCAPR_SRCPPRND_SHIFT)) & NETC_SW_BRCAPR_SRCPPRND_MASK)

#define NETC_SW_BRCAPR_EVLANXLATE_MASK           (0x20U)
#define NETC_SW_BRCAPR_EVLANXLATE_SHIFT          (5U)
/*! EVLANXLATE - Egress VLAN translation supported
 *  0b0..Egress VLAN translation not supported
 *  0b1..Egress VLAN translation supported
 */
#define NETC_SW_BRCAPR_EVLANXLATE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_BRCAPR_EVLANXLATE_SHIFT)) & NETC_SW_BRCAPR_EVLANXLATE_MASK)

#define NETC_SW_BRCAPR_NUM_STG_MASK              (0x3000U)
#define NETC_SW_BRCAPR_NUM_STG_SHIFT             (12U)
#define NETC_SW_BRCAPR_NUM_STG(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_BRCAPR_NUM_STG_SHIFT)) & NETC_SW_BRCAPR_NUM_STG_MASK)
/*! @} */

/*! @name VFHTCAPR - VLAN filter hash table capability register */
/*! @{ */

#define NETC_SW_VFHTCAPR_ACCESS_METH_MASK        (0xF00000U)
#define NETC_SW_VFHTCAPR_ACCESS_METH_SHIFT       (20U)
#define NETC_SW_VFHTCAPR_ACCESS_METH(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_VFHTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name VFHTOR - VLAN filter hash table operational register */
/*! @{ */

#define NETC_SW_VFHTOR_NUM_ENTRIES_MASK          (0xFFFU)
#define NETC_SW_VFHTOR_NUM_ENTRIES_SHIFT         (0U)
#define NETC_SW_VFHTOR_NUM_ENTRIES(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTOR_NUM_ENTRIES_SHIFT)) & NETC_SW_VFHTOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name VFHTDECR0 - VLAN Filter (hash) table default entry configuration registers 0 */
/*! @{ */

#define NETC_SW_VFHTDECR0_PORT0_MASK             (0x1U)
#define NETC_SW_VFHTDECR0_PORT0_SHIFT            (0U)
/*! PORT0 - Port n
 *  0b0..Port is not a member of this VLAN.
 *  0b1..Port is a member of this VLAN.
 */
#define NETC_SW_VFHTDECR0_PORT0(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_PORT0_SHIFT)) & NETC_SW_VFHTDECR0_PORT0_MASK)

#define NETC_SW_VFHTDECR0_PORT1_MASK             (0x2U)
#define NETC_SW_VFHTDECR0_PORT1_SHIFT            (1U)
/*! PORT1 - Port n
 *  0b0..Port is not a member of this VLAN.
 *  0b1..Port is a member of this VLAN.
 */
#define NETC_SW_VFHTDECR0_PORT1(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_PORT1_SHIFT)) & NETC_SW_VFHTDECR0_PORT1_MASK)

#define NETC_SW_VFHTDECR0_PORT2_MASK             (0x4U)
#define NETC_SW_VFHTDECR0_PORT2_SHIFT            (2U)
/*! PORT2 - Port n
 *  0b0..Port is not a member of this VLAN.
 *  0b1..Port is a member of this VLAN.
 */
#define NETC_SW_VFHTDECR0_PORT2(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_PORT2_SHIFT)) & NETC_SW_VFHTDECR0_PORT2_MASK)

#define NETC_SW_VFHTDECR0_PORT3_MASK             (0x8U)
#define NETC_SW_VFHTDECR0_PORT3_SHIFT            (3U)
/*! PORT3 - Port n
 *  0b0..Port is not a member of this VLAN.
 *  0b1..Port is a member of this VLAN.
 */
#define NETC_SW_VFHTDECR0_PORT3(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_PORT3_SHIFT)) & NETC_SW_VFHTDECR0_PORT3_MASK)

#define NETC_SW_VFHTDECR0_PORT4_MASK             (0x10U)
#define NETC_SW_VFHTDECR0_PORT4_SHIFT            (4U)
/*! PORT4 - Port n
 *  0b0..Port is not a member of this VLAN.
 *  0b1..Port is a member of this VLAN.
 */
#define NETC_SW_VFHTDECR0_PORT4(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_PORT4_SHIFT)) & NETC_SW_VFHTDECR0_PORT4_MASK)

#define NETC_SW_VFHTDECR0_STG_ID_MASK            (0xF000000U)
#define NETC_SW_VFHTDECR0_STG_ID_SHIFT           (24U)
#define NETC_SW_VFHTDECR0_STG_ID(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_STG_ID_SHIFT)) & NETC_SW_VFHTDECR0_STG_ID_MASK)

#define NETC_SW_VFHTDECR0_IPMFE_MASK             (0x20000000U)
#define NETC_SW_VFHTDECR0_IPMFE_SHIFT            (29U)
/*! IPMFE - IP Multicast Filtering Enable
 *  0b0..No IP multicast filtering is performed.
 *  0b1..If the frame is identified as a multicast IP packet, then IP multicast filtering is performed. If the
 *       frame is not identified as an IP multicast packet, the IP multicast filtering is not performed.
 */
#define NETC_SW_VFHTDECR0_IPMFE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_IPMFE_SHIFT)) & NETC_SW_VFHTDECR0_IPMFE_MASK)

#define NETC_SW_VFHTDECR0_IPMFLE_MASK            (0x40000000U)
#define NETC_SW_VFHTDECR0_IPMFLE_SHIFT           (30U)
/*! IPMFLE - IP Multicast Flooding Enable
 *  0b0..IP Multicast Flooding disabled, the frame is discarded.
 *  0b1..IP Multicast Flooding enabled, the frame is flooded.
 */
#define NETC_SW_VFHTDECR0_IPMFLE(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR0_IPMFLE_SHIFT)) & NETC_SW_VFHTDECR0_IPMFLE_MASK)
/*! @} */

/*! @name VFHTDECR1 - VLAN filter hash table default entry configuration registers 1 */
/*! @{ */

#define NETC_SW_VFHTDECR1_FID_MASK               (0xFFFU)
#define NETC_SW_VFHTDECR1_FID_SHIFT              (0U)
#define NETC_SW_VFHTDECR1_FID(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR1_FID_SHIFT)) & NETC_SW_VFHTDECR1_FID_MASK)

#define NETC_SW_VFHTDECR1_VL_MODE_MASK           (0x1000U)
#define NETC_SW_VFHTDECR1_VL_MODE_SHIFT          (12U)
#define NETC_SW_VFHTDECR1_VL_MODE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR1_VL_MODE_SHIFT)) & NETC_SW_VFHTDECR1_VL_MODE_MASK)

#define NETC_SW_VFHTDECR1_BASE_ETEID_MASK        (0xFFFF0000U)
#define NETC_SW_VFHTDECR1_BASE_ETEID_SHIFT       (16U)
#define NETC_SW_VFHTDECR1_BASE_ETEID(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR1_BASE_ETEID_SHIFT)) & NETC_SW_VFHTDECR1_BASE_ETEID_MASK)
/*! @} */

/*! @name VFHTDECR2 - VLAN filter hash table default entry configuration registers 2 */
/*! @{ */

#define NETC_SW_VFHTDECR2_ET_PORT0_MASK          (0x1U)
#define NETC_SW_VFHTDECR2_ET_PORT0_SHIFT         (0U)
/*! ET_PORT0 - Egress Treatment Applicability Port n.
 *  0b0..Port has no entry in the Egress Treatment table
 *  0b1..Port has an entry in the Egress Treatment table
 */
#define NETC_SW_VFHTDECR2_ET_PORT0(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR2_ET_PORT0_SHIFT)) & NETC_SW_VFHTDECR2_ET_PORT0_MASK)

#define NETC_SW_VFHTDECR2_ET_PORT1_MASK          (0x2U)
#define NETC_SW_VFHTDECR2_ET_PORT1_SHIFT         (1U)
/*! ET_PORT1 - Egress Treatment Applicability Port n.
 *  0b0..Port has no entry in the Egress Treatment table
 *  0b1..Port has an entry in the Egress Treatment table
 */
#define NETC_SW_VFHTDECR2_ET_PORT1(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR2_ET_PORT1_SHIFT)) & NETC_SW_VFHTDECR2_ET_PORT1_MASK)

#define NETC_SW_VFHTDECR2_ET_PORT2_MASK          (0x4U)
#define NETC_SW_VFHTDECR2_ET_PORT2_SHIFT         (2U)
/*! ET_PORT2 - Egress Treatment Applicability Port n.
 *  0b0..Port has no entry in the Egress Treatment table
 *  0b1..Port has an entry in the Egress Treatment table
 */
#define NETC_SW_VFHTDECR2_ET_PORT2(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR2_ET_PORT2_SHIFT)) & NETC_SW_VFHTDECR2_ET_PORT2_MASK)

#define NETC_SW_VFHTDECR2_ET_PORT3_MASK          (0x8U)
#define NETC_SW_VFHTDECR2_ET_PORT3_SHIFT         (3U)
/*! ET_PORT3 - Egress Treatment Applicability Port n.
 *  0b0..Port has no entry in the Egress Treatment table
 *  0b1..Port has an entry in the Egress Treatment table
 */
#define NETC_SW_VFHTDECR2_ET_PORT3(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR2_ET_PORT3_SHIFT)) & NETC_SW_VFHTDECR2_ET_PORT3_MASK)

#define NETC_SW_VFHTDECR2_ET_PORT4_MASK          (0x10U)
#define NETC_SW_VFHTDECR2_ET_PORT4_SHIFT         (4U)
/*! ET_PORT4 - Egress Treatment Applicability Port n.
 *  0b0..Port has no entry in the Egress Treatment table
 *  0b1..Port has an entry in the Egress Treatment table
 */
#define NETC_SW_VFHTDECR2_ET_PORT4(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR2_ET_PORT4_SHIFT)) & NETC_SW_VFHTDECR2_ET_PORT4_MASK)

#define NETC_SW_VFHTDECR2_MLO_MASK               (0x7000000U)
#define NETC_SW_VFHTDECR2_MLO_SHIFT              (24U)
#define NETC_SW_VFHTDECR2_MLO(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR2_MLO_SHIFT)) & NETC_SW_VFHTDECR2_MLO_MASK)

#define NETC_SW_VFHTDECR2_MFO_MASK               (0x18000000U)
#define NETC_SW_VFHTDECR2_MFO_SHIFT              (27U)
#define NETC_SW_VFHTDECR2_MFO(x)                 (((uint32_t)(((uint32_t)(x)) << NETC_SW_VFHTDECR2_MFO_SHIFT)) & NETC_SW_VFHTDECR2_MFO_MASK)
/*! @} */

/*! @name FDBHTCAPR - FDB hash table capability register */
/*! @{ */

#define NETC_SW_FDBHTCAPR_NUM_GMAC_MASK          (0x1FFU)
#define NETC_SW_FDBHTCAPR_NUM_GMAC_SHIFT         (0U)
#define NETC_SW_FDBHTCAPR_NUM_GMAC(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_FDBHTCAPR_NUM_GMAC_SHIFT)) & NETC_SW_FDBHTCAPR_NUM_GMAC_MASK)

#define NETC_SW_FDBHTCAPR_ACCESS_METH_MASK       (0xF00000U)
#define NETC_SW_FDBHTCAPR_ACCESS_METH_SHIFT      (20U)
#define NETC_SW_FDBHTCAPR_ACCESS_METH(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_FDBHTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_FDBHTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name FDBHTMCR - FDB hash table memory configuration register */
/*! @{ */

#define NETC_SW_FDBHTMCR_DYN_LIMIT_MASK          (0xFFFFU)
#define NETC_SW_FDBHTMCR_DYN_LIMIT_SHIFT         (0U)
#define NETC_SW_FDBHTMCR_DYN_LIMIT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_FDBHTMCR_DYN_LIMIT_SHIFT)) & NETC_SW_FDBHTMCR_DYN_LIMIT_MASK)
/*! @} */

/*! @name FDBHTOR0 - FDB hash table operational register 0 */
/*! @{ */

#define NETC_SW_FDBHTOR0_STATIC_ENTRIES_MASK     (0xFFFFU)
#define NETC_SW_FDBHTOR0_STATIC_ENTRIES_SHIFT    (0U)
#define NETC_SW_FDBHTOR0_STATIC_ENTRIES(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_FDBHTOR0_STATIC_ENTRIES_SHIFT)) & NETC_SW_FDBHTOR0_STATIC_ENTRIES_MASK)

#define NETC_SW_FDBHTOR0_NUM_GENTRIES_MASK       (0x1FF0000U)
#define NETC_SW_FDBHTOR0_NUM_GENTRIES_SHIFT      (16U)
#define NETC_SW_FDBHTOR0_NUM_GENTRIES(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_FDBHTOR0_NUM_GENTRIES_SHIFT)) & NETC_SW_FDBHTOR0_NUM_GENTRIES_MASK)
/*! @} */

/*! @name FDBHTOR1 - FDB hash table operational register 1 */
/*! @{ */

#define NETC_SW_FDBHTOR1_DYN_ENTRIES_MASK        (0xFFFFU)
#define NETC_SW_FDBHTOR1_DYN_ENTRIES_SHIFT       (0U)
#define NETC_SW_FDBHTOR1_DYN_ENTRIES(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_FDBHTOR1_DYN_ENTRIES_SHIFT)) & NETC_SW_FDBHTOR1_DYN_ENTRIES_MASK)

#define NETC_SW_FDBHTOR1_HWM_DYN_ENTRIES_MASK    (0xFFFF0000U)
#define NETC_SW_FDBHTOR1_HWM_DYN_ENTRIES_SHIFT   (16U)
#define NETC_SW_FDBHTOR1_HWM_DYN_ENTRIES(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_FDBHTOR1_HWM_DYN_ENTRIES_SHIFT)) & NETC_SW_FDBHTOR1_HWM_DYN_ENTRIES_MASK)
/*! @} */

/*! @name IPMFHTCAPR - IP multicast filter hash table capability register */
/*! @{ */

#define NETC_SW_IPMFHTCAPR_ACCESS_METH_MASK      (0xF00000U)
#define NETC_SW_IPMFHTCAPR_ACCESS_METH_SHIFT     (20U)
#define NETC_SW_IPMFHTCAPR_ACCESS_METH(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_IPMFHTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_IPMFHTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name IPV4MFHTOR - IPv4 multicast filter hash table operation register */
/*! @{ */

#define NETC_SW_IPV4MFHTOR_ASM_ENTRIES_MASK      (0xFFFFU)
#define NETC_SW_IPV4MFHTOR_ASM_ENTRIES_SHIFT     (0U)
#define NETC_SW_IPV4MFHTOR_ASM_ENTRIES(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_IPV4MFHTOR_ASM_ENTRIES_SHIFT)) & NETC_SW_IPV4MFHTOR_ASM_ENTRIES_MASK)

#define NETC_SW_IPV4MFHTOR_SSM_ENTRIES_MASK      (0xFFFF0000U)
#define NETC_SW_IPV4MFHTOR_SSM_ENTRIES_SHIFT     (16U)
#define NETC_SW_IPV4MFHTOR_SSM_ENTRIES(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_IPV4MFHTOR_SSM_ENTRIES_SHIFT)) & NETC_SW_IPV4MFHTOR_SSM_ENTRIES_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group NETC_SW_Register_Masks */


/* NETC_SW - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SW0_BASE base address */
  #define SW0_BASE_BASE                            (0x70A00000u)
  /** Peripheral SW0_BASE base address */
  #define SW0_BASE_BASE_NS                         (0x60A00000u)
  /** Peripheral SW0_BASE base pointer */
  #define SW0_BASE                                 ((NETC_SW_Type *)SW0_BASE_BASE)
  /** Peripheral SW0_BASE base pointer */
  #define SW0_BASE_NS                              ((NETC_SW_Type *)SW0_BASE_BASE_NS)
  /** Array initializer of NETC_SW peripheral base addresses */
  #define NETC_SW_BASE_ADDRS                       { SW0_BASE_BASE }
  /** Array initializer of NETC_SW peripheral base pointers */
  #define NETC_SW_BASE_PTRS                        { SW0_BASE }
  /** Array initializer of NETC_SW peripheral base addresses */
  #define NETC_SW_BASE_ADDRS_NS                    { SW0_BASE_BASE_NS }
  /** Array initializer of NETC_SW peripheral base pointers */
  #define NETC_SW_BASE_PTRS_NS                     { SW0_BASE_NS }
#else
  /** Peripheral SW0_BASE base address */
  #define SW0_BASE_BASE                            (0x60A00000u)
  /** Peripheral SW0_BASE base pointer */
  #define SW0_BASE                                 ((NETC_SW_Type *)SW0_BASE_BASE)
  /** Array initializer of NETC_SW peripheral base addresses */
  #define NETC_SW_BASE_ADDRS                       { SW0_BASE_BASE }
  /** Array initializer of NETC_SW peripheral base pointers */
  #define NETC_SW_BASE_PTRS                        { SW0_BASE }
#endif

/*!
 * @}
 */ /* end of group NETC_SW_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- NETC_SW_ENETC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_SW_ENETC_Peripheral_Access_Layer NETC_SW_ENETC Peripheral Access Layer
 * @{
 */

/** NETC_SW_ENETC - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4096];
  __I  uint32_t IPCAPR;                            /**< Ingress port capability register, offset: 0x1000 */
  __I  uint32_t EPCAPR;                            /**< Egress port capability register, offset: 0x1004 */
       uint8_t RESERVED_1[8];
  __I  uint32_t OSR;                               /**< Operational state register, offset: 0x1010 */
       uint8_t RESERVED_2[44];
  __IO uint32_t CMECR;                             /**< Correctable memory error configuration register, offset: 0x1040 */
  __IO uint32_t CMESR;                             /**< Correctable memory error status register, offset: 0x1044 */
       uint8_t RESERVED_3[4];
  __I  uint32_t CMECTR;                            /**< Correctable memory error count register, offset: 0x104C */
       uint8_t RESERVED_4[16];
  __IO uint32_t UNMACECR;                          /**< Uncorrectable non-fatal MAC error configuration register, offset: 0x1060 */
  __I  uint32_t UNMACESR;                          /**< Uncorrectable non-fatal MAC error status register, offset: 0x1064 */
       uint8_t RESERVED_5[8];
  __IO uint32_t UNSBECR;                           /**< Uncorrectable non-fatal system bus error configuration register, offset: 0x1070, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t UNSBESR;                           /**< Uncorrectable non-fatal system bus error status register, offset: 0x1074, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_6[4];
  __I  uint32_t UNSBECTR;                          /**< Uncorrectable non-fatal system bus error count register, offset: 0x107C, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t UFSBECR;                           /**< Uncorrectable fatal system bus error configuration register, offset: 0x1080, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t UFSBESR;                           /**< Uncorrectable fatal system bus error status register, offset: 0x1084, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_7[8];
  __IO uint32_t UNMECR;                            /**< Uncorrectable non-fatal memory error configuration register, offset: 0x1090 */
  __IO uint32_t UNMESR0;                           /**< Uncorrectable non-fatal memory error status register 0, offset: 0x1094 */
  __I  uint32_t UNMESR1;                           /**< Uncorrectable non-fatal memory error status register 1, offset: 0x1098 */
  __I  uint32_t UNMECTR;                           /**< Uncorrectable non-fatal memory error count register, offset: 0x109C */
  __IO uint32_t UFMECR;                            /**< Uncorrectable fatal memory error configuration register, offset: 0x10A0 */
  __IO uint32_t UFMESR0;                           /**< Uncorrectable fatal memory error status register 0, offset: 0x10A4 */
  __I  uint32_t UFMESR1;                           /**< Uncorrectable fatal memory error status register 1, offset: 0x10A8 */
       uint8_t RESERVED_8[52];
  __I  uint32_t IMDIOIRR;                          /**< Internal MDIO interrupt reason register, offset: 0x10E0, available only on: ENETC0_COMMON, SW0_COMMON (missing on ENETC1_COMMON) */
  __IO uint32_t IMDIOMSIVR;                        /**< Internal MDIO MSI-X vector register, offset: 0x10E4, available only on: ENETC0_COMMON, SW0_COMMON (missing on ENETC1_COMMON) */
  __I  uint32_t EMDIOIRR;                          /**< External MDIO interrupt reason register, offset: 0x10E8, available only on: ENETC0_COMMON, SW0_COMMON (missing on ENETC1_COMMON) */
  __IO uint32_t EMDIOMSIVR;                        /**< External MDIO MSI-X vector register, offset: 0x10EC, available only on: ENETC0_COMMON, SW0_COMMON (missing on ENETC1_COMMON) */
       uint8_t RESERVED_9[16];
  __IO uint32_t TCCR;                              /**< Time capture configuration register, offset: 0x1100, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t TCIER;                             /**< Time capture interrupt enable register, offset: 0x1104, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t TCRPIDR;                           /**< Time capture receive port interrupt detect register, offset: 0x1108, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t TCRPSR;                            /**< Time capture receive port status register, offset: 0x110C, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_10[4];
  __I  uint32_t TCRPTSR;                           /**< Time capture receive port timestamp register, offset: 0x1114, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t TCMSIVR;                           /**< Time capture MSI-X vector register, offset: 0x1118, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_11[228];
  __IO uint32_t CVLANR1;                           /**< Custom VLAN Ethertype register 1, offset: 0x1200 */
  __IO uint32_t CVLANR2;                           /**< Custom VLAN Ethertype register 2, offset: 0x1204 */
  __IO uint32_t PSRTAGETR;                         /**< Pre-Standard RTAG Ethertype register, offset: 0x1208, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_12[20];
  __IO uint32_t DOSL2CR;                           /**< DoS L2 configuration register, offset: 0x1220 */
       uint8_t RESERVED_13[220];
  struct {                                         /* offset: 0x1300, array step: 0x10 */
    __IO uint32_t VLANIPVMPR0;                       /**< VLAN to IPV mapping profile 0 register 0..VLAN to IPV mapping profile 1 register 0, array offset: 0x1300, array step: 0x10, irregular array, not all indices are valid */
    __IO uint32_t VLANIPVMPR1;                       /**< VLAN to IPV mapping profile 0 register 1..VLAN to IPV mapping profile 1 register 1, array offset: 0x1304, array step: 0x10, irregular array, not all indices are valid */
    __IO uint32_t VLANDRMPR;                         /**< VLAN to DR mapping profile 0 register..VLAN to DR mapping profile 1 register, array offset: 0x1308, array step: 0x10, irregular array, not all indices are valid */
         uint8_t RESERVED_0[4];
  } NUM_PROFILE[2];
       uint8_t RESERVED_14[800];
  __I  uint32_t IPFCAPR;                           /**< Ingress port filter capability register, offset: 0x1640 */
  __I  uint32_t IPFTCAPR;                          /**< Ingress port filter table capability register, offset: 0x1644 */
  __I  uint32_t IPFTMOR;                           /**< Ingress port filter table memory operational register, offset: 0x1648 */
       uint8_t RESERVED_15[436];
  __I  uint32_t ITMCAPR;                           /**< Index table memory capability register, offset: 0x1800 */
       uint8_t RESERVED_16[12];
  __I  uint32_t RPCAPR;                            /**< Rate policer capability register, offset: 0x1810 */
  __I  uint32_t RPITCAPR;                          /**< Rate policer index table capability register, offset: 0x1814 */
  __IO uint32_t RPITMAR;                           /**< Rate policer index table memory allocation register, offset: 0x1818 */
  __I  uint32_t RPITOR;                            /**< Rate policer index table operational register, offset: 0x181C */
       uint8_t RESERVED_17[4];
  __I  uint32_t ISCITCAPR;                         /**< Ingress stream counter index table capability register, offset: 0x1824 */
  __IO uint32_t ISCITMAR;                          /**< Ingress stream counter index table memory allocation register, offset: 0x1828 */
  __I  uint32_t ISCITOR;                           /**< Ingress stream counter index table operational register, offset: 0x182C */
  __I  uint32_t ISCAPR;                            /**< Ingress stream capability register, offset: 0x1830 */
  __I  uint32_t ISITCAPR;                          /**< Ingress stream index table capability register, offset: 0x1834 */
  __IO uint32_t ISITMAR;                           /**< Ingress stream index table memory allocation register, offset: 0x1838 */
  __I  uint32_t ISITOR;                            /**< Ingress stream index table operational register, offset: 0x183C */
       uint8_t RESERVED_18[4];
  __I  uint32_t ISQGITCAPR;                        /**< Ingress sequence generation index table capability register, offset: 0x1844, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISQGITMAR;                         /**< Ingress sequence generation index table memory allocation register, offset: 0x1848, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t ISQGITOR;                          /**< Ingress sequence generation index table operational register, offset: 0x184C, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_19[16];
  __I  uint32_t SGCAPR;                            /**< Stream gate capability register, offset: 0x1860 */
  __I  uint32_t SGIITCAPR;                         /**< Stream gate instance index table capability register, offset: 0x1864 */
  __IO uint32_t SGIITMAR;                          /**< Stream gate instance index table memory allocation register, offset: 0x1868 */
  __I  uint32_t SGIITOR;                           /**< Stream gate instance index table operational register, offset: 0x186C */
       uint8_t RESERVED_20[4];
  __I  uint32_t SGCLITCAPR;                        /**< Stream gate control list index table capability register, offset: 0x1874 */
  __IO uint32_t SGCLITMAR;                         /**< Stream gate control list index table memory allocation register, offset: 0x1878 */
  __I  uint32_t SGCLTMOR;                          /**< Stream gate control list table memory operational register, offset: 0x187C */
  __I  uint32_t FMICAPR;                           /**< Frame modification ingress capability register, offset: 0x1880, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t FMECAPR;                           /**< Frame modification egress capability register, offset: 0x1884, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t FMITCAPR;                          /**< Frame modification index table capability register, offset: 0x1888, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t FMITMAR;                           /**< Frame modification index table memory allocation register, offset: 0x188C, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t FMITOR;                            /**< Frame modification index table operational register, offset: 0x1890, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t FMDITCAPR;                         /**< Frame modification data index table capability register, offset: 0x1894, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t FMDITMAR;                          /**< Frame modification data index table memory allocation register, offset: 0x1898, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_21[36];
  __I  uint32_t ETCAPR;                            /**< Egress treatment capability register, offset: 0x18C0, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t ETTCAPR;                           /**< Egress treatment table capability register, offset: 0x18C4, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_22[4];
  __I  uint32_t ETTOR;                             /**< Egress treatment table operational register, offset: 0x18CC, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_23[4];
  __I  uint32_t TGSTCAPR;                          /**< Time gate scheduling table capability register, offset: 0x18D4 */
       uint8_t RESERVED_24[4];
  __I  uint32_t TGSTMOR;                           /**< Time gate scheduling table memory operation register, offset: 0x18DC */
  __I  uint32_t ESQRCAPR;                          /**< Egress sequence recovery capability register, offset: 0x18E0, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t ESQRTCAPR;                         /**< Egress sequence recovery table capability register, offset: 0x18E4, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_25[4];
  __I  uint32_t ECTCAPR;                           /**< Egress counter table capability register, offset: 0x18EC, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_26[16];
  __I  uint32_t HTMCAPR;                           /**< Hash table memory capability register, offset: 0x1900 */
  __I  uint32_t HTMOR;                             /**< Hash table memory operational register, offset: 0x1904 */
       uint8_t RESERVED_27[8];
  __I  uint32_t ISIDCAPR;                          /**< Ingress stream identification capability register, offset: 0x1910 */
  __I  uint32_t ISIDHTCAPR;                        /**< Ingress stream identification hash table capability register, offset: 0x1914 */
       uint8_t RESERVED_28[8];
  __I  uint32_t ISIDKC0OR;                         /**< Ingress stream identification key construction 0 operational register, offset: 0x1920 */
  __IO uint32_t ISIDKC0CR0;                        /**< Ingress stream identification key construction 0 configuration register 0, offset: 0x1924 */
       uint8_t RESERVED_29[8];
  __IO uint32_t ISIDKC0PF0CR;                      /**< Ingress stream identification key construction 0 payload field 0 configuration register, offset: 0x1930 */
  __IO uint32_t ISIDKC0PF1CR;                      /**< Ingress stream identification key construction 0 payload field 1 configuration register, offset: 0x1934 */
  __IO uint32_t ISIDKC0PF2CR;                      /**< Ingress stream identification key construction 0 payload field 2 configuration register, offset: 0x1938 */
  __IO uint32_t ISIDKC0PF3CR;                      /**< Ingress stream identification key construction 0 payload field 3 configuration register, offset: 0x193C */
  __I  uint32_t ISIDKC1OR;                         /**< Ingress stream identification key construction 1 operational register, offset: 0x1940 */
  __IO uint32_t ISIDKC1CR0;                        /**< Ingress stream identification key construction 1 configuration register 0, offset: 0x1944 */
       uint8_t RESERVED_30[8];
  __IO uint32_t ISIDKC1PF0CR;                      /**< Ingress stream identification key construction 1 payload field 0 configuration register, offset: 0x1950 */
  __IO uint32_t ISIDKC1PF1CR;                      /**< Ingress stream identification key construction 1 payload field 1 configuration register, offset: 0x1954 */
  __IO uint32_t ISIDKC1PF2CR;                      /**< Ingress stream identification key construction 1 payload field 2 configuration register, offset: 0x1958 */
  __IO uint32_t ISIDKC1PF3CR;                      /**< Ingress stream identification key construction 1 payload field 3 configuration register, offset: 0x195C */
  __I  uint32_t ISIDKC2OR;                         /**< Ingress stream identification key construction 2 operational register, offset: 0x1960, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC2CR0;                        /**< Ingress stream identification key construction 2 configuration register 0, offset: 0x1964, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_31[8];
  __IO uint32_t ISIDKC2PF0CR;                      /**< Ingress stream identification key construction 2 payload field 0 configuration register, offset: 0x1970, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC2PF1CR;                      /**< Ingress stream identification key construction 2 payload field 1 configuration register, offset: 0x1974, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC2PF2CR;                      /**< Ingress stream identification key construction 2 payload field 2 configuration register, offset: 0x1978, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC2PF3CR;                      /**< Ingress stream identification key construction 2 payload field 3 configuration register, offset: 0x197C, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __I  uint32_t ISIDKC3OR;                         /**< Ingress stream identification key construction 3 operational register, offset: 0x1980, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC3CR0;                        /**< Ingress stream identification key construction 3 configuration register 0, offset: 0x1984, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_32[8];
  __IO uint32_t ISIDKC3PF0CR;                      /**< Ingress stream identification key construction 3 payload field 0 configuration register, offset: 0x1990, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC3PF1CR;                      /**< Ingress stream identification key construction 3 payload field 1 configuration register, offset: 0x1994, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC3PF2CR;                      /**< Ingress stream identification key construction 3 payload field 2 configuration register, offset: 0x1998, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
  __IO uint32_t ISIDKC3PF3CR;                      /**< Ingress stream identification key construction 3 payload field 3 configuration register, offset: 0x199C, available only on: SW0_COMMON (missing on ENETC0_COMMON, ENETC1_COMMON) */
       uint8_t RESERVED_33[96];
  __I  uint32_t ISFHTCAPR;                         /**< Ingress stream filter hash table capability register, offset: 0x1A00 */
  __I  uint32_t ISFHTOR;                           /**< Ingress stream filter hash table operational register, offset: 0x1A04 */
} NETC_SW_ENETC_Type;

/* ----------------------------------------------------------------------------
   -- NETC_SW_ENETC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup NETC_SW_ENETC_Register_Masks NETC_SW_ENETC Register Masks
 * @{
 */

/*! @name IPCAPR - Ingress port capability register */
/*! @{ */

#define NETC_SW_ENETC_IPCAPR_RP_MASK             (0x1U)
#define NETC_SW_ENETC_IPCAPR_RP_SHIFT            (0U)
#define NETC_SW_ENETC_IPCAPR_RP(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPCAPR_RP_SHIFT)) & NETC_SW_ENETC_IPCAPR_RP_MASK)

#define NETC_SW_ENETC_IPCAPR_IPFLT_MASK          (0x2U)
#define NETC_SW_ENETC_IPCAPR_IPFLT_SHIFT         (1U)
#define NETC_SW_ENETC_IPCAPR_IPFLT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPCAPR_IPFLT_SHIFT)) & NETC_SW_ENETC_IPCAPR_IPFLT_MASK)

#define NETC_SW_ENETC_IPCAPR_ISID_MASK           (0x4U)
#define NETC_SW_ENETC_IPCAPR_ISID_SHIFT          (2U)
#define NETC_SW_ENETC_IPCAPR_ISID(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPCAPR_ISID_SHIFT)) & NETC_SW_ENETC_IPCAPR_ISID_MASK)

#define NETC_SW_ENETC_IPCAPR_SDU_MASK            (0x1F00U)
#define NETC_SW_ENETC_IPCAPR_SDU_SHIFT           (8U)
/*! SDU - Indicates support for various PDU/SDUs (Protocol/Service Data Unit) definitions. */
#define NETC_SW_ENETC_IPCAPR_SDU(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPCAPR_SDU_SHIFT)) & NETC_SW_ENETC_IPCAPR_SDU_MASK)

#define NETC_SW_ENETC_IPCAPR_NUM_VQMP_MASK       (0xF0000U)
#define NETC_SW_ENETC_IPCAPR_NUM_VQMP_SHIFT      (16U)
#define NETC_SW_ENETC_IPCAPR_NUM_VQMP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPCAPR_NUM_VQMP_SHIFT)) & NETC_SW_ENETC_IPCAPR_NUM_VQMP_MASK)
/*! @} */

/*! @name EPCAPR - Egress port capability register */
/*! @{ */

#define NETC_SW_ENETC_EPCAPR_ET_MASK             (0x1U)
#define NETC_SW_ENETC_EPCAPR_ET_SHIFT            (0U)
#define NETC_SW_ENETC_EPCAPR_ET(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EPCAPR_ET_SHIFT)) & NETC_SW_ENETC_EPCAPR_ET_MASK)

#define NETC_SW_ENETC_EPCAPR_SDU_MASK            (0x1F00U)
#define NETC_SW_ENETC_EPCAPR_SDU_SHIFT           (8U)
/*! SDU - Indicates support for various PDU/SDUs (Protocol/Service Data Unit) definitions. */
#define NETC_SW_ENETC_EPCAPR_SDU(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EPCAPR_SDU_SHIFT)) & NETC_SW_ENETC_EPCAPR_SDU_MASK)

#define NETC_SW_ENETC_EPCAPR_NUM_QVMP_MASK       (0xF0000U)
#define NETC_SW_ENETC_EPCAPR_NUM_QVMP_SHIFT      (16U)
#define NETC_SW_ENETC_EPCAPR_NUM_QVMP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EPCAPR_NUM_QVMP_SHIFT)) & NETC_SW_ENETC_EPCAPR_NUM_QVMP_MASK)
/*! @} */

/*! @name OSR - Operational state register */
/*! @{ */

#define NETC_SW_ENETC_OSR_STATE_MASK             (0x1U)
#define NETC_SW_ENETC_OSR_STATE_SHIFT            (0U)
#define NETC_SW_ENETC_OSR_STATE(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_OSR_STATE_SHIFT)) & NETC_SW_ENETC_OSR_STATE_MASK)

#define NETC_SW_ENETC_OSR_ITM_STATE_MASK         (0x2U)
#define NETC_SW_ENETC_OSR_ITM_STATE_SHIFT        (1U)
#define NETC_SW_ENETC_OSR_ITM_STATE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_OSR_ITM_STATE_SHIFT)) & NETC_SW_ENETC_OSR_ITM_STATE_MASK)
/*! @} */

/*! @name CMECR - Correctable memory error configuration register */
/*! @{ */

#define NETC_SW_ENETC_CMECR_THRESHOLD_MASK       (0xFFU)
#define NETC_SW_ENETC_CMECR_THRESHOLD_SHIFT      (0U)
/*! THRESHOLD - Threshold */
#define NETC_SW_ENETC_CMECR_THRESHOLD(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CMECR_THRESHOLD_SHIFT)) & NETC_SW_ENETC_CMECR_THRESHOLD_MASK)
/*! @} */

/*! @name CMESR - Correctable memory error status register */
/*! @{ */

#define NETC_SW_ENETC_CMESR_MEM_ID_MASK          (0x1F0000U)
#define NETC_SW_ENETC_CMESR_MEM_ID_SHIFT         (16U)
/*! MEM_ID - Memory ID */
#define NETC_SW_ENETC_CMESR_MEM_ID(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CMESR_MEM_ID_SHIFT)) & NETC_SW_ENETC_CMESR_MEM_ID_MASK)

#define NETC_SW_ENETC_CMESR_SBEE_MASK            (0x80000000U)
#define NETC_SW_ENETC_CMESR_SBEE_SHIFT           (31U)
/*! SBEE - Single-bit ECC error */
#define NETC_SW_ENETC_CMESR_SBEE(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CMESR_SBEE_SHIFT)) & NETC_SW_ENETC_CMESR_SBEE_MASK)
/*! @} */

/*! @name CMECTR - Correctable memory error count register */
/*! @{ */

#define NETC_SW_ENETC_CMECTR_COUNT_MASK          (0xFFU)
#define NETC_SW_ENETC_CMECTR_COUNT_SHIFT         (0U)
/*! COUNT - Count */
#define NETC_SW_ENETC_CMECTR_COUNT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CMECTR_COUNT_SHIFT)) & NETC_SW_ENETC_CMECTR_COUNT_MASK)
/*! @} */

/*! @name UNMACECR - Uncorrectable non-fatal MAC error configuration register */
/*! @{ */

#define NETC_SW_ENETC_UNMACECR_PORT0_MASK        (0x1U)
#define NETC_SW_ENETC_UNMACECR_PORT0_SHIFT       (0U)
/*! PORT0 - Report disable port */
#define NETC_SW_ENETC_UNMACECR_PORT0(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACECR_PORT0_SHIFT)) & NETC_SW_ENETC_UNMACECR_PORT0_MASK)

#define NETC_SW_ENETC_UNMACECR_PORT1_MASK        (0x2U)
#define NETC_SW_ENETC_UNMACECR_PORT1_SHIFT       (1U)
/*! PORT1 - Report disable port */
#define NETC_SW_ENETC_UNMACECR_PORT1(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACECR_PORT1_SHIFT)) & NETC_SW_ENETC_UNMACECR_PORT1_MASK)

#define NETC_SW_ENETC_UNMACECR_PORT2_MASK        (0x4U)
#define NETC_SW_ENETC_UNMACECR_PORT2_SHIFT       (2U)
/*! PORT2 - Report disable port */
#define NETC_SW_ENETC_UNMACECR_PORT2(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACECR_PORT2_SHIFT)) & NETC_SW_ENETC_UNMACECR_PORT2_MASK)

#define NETC_SW_ENETC_UNMACECR_PORT3_MASK        (0x8U)
#define NETC_SW_ENETC_UNMACECR_PORT3_SHIFT       (3U)
/*! PORT3 - Report disable port */
#define NETC_SW_ENETC_UNMACECR_PORT3(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACECR_PORT3_SHIFT)) & NETC_SW_ENETC_UNMACECR_PORT3_MASK)

#define NETC_SW_ENETC_UNMACECR_PORT4_MASK        (0x10U)
#define NETC_SW_ENETC_UNMACECR_PORT4_SHIFT       (4U)
/*! PORT4 - Report disable port */
#define NETC_SW_ENETC_UNMACECR_PORT4(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACECR_PORT4_SHIFT)) & NETC_SW_ENETC_UNMACECR_PORT4_MASK)
/*! @} */

/*! @name UNMACESR - Uncorrectable non-fatal MAC error status register */
/*! @{ */

#define NETC_SW_ENETC_UNMACESR_PORT0_MASK        (0x1U)
#define NETC_SW_ENETC_UNMACESR_PORT0_SHIFT       (0U)
/*! PORT0 - Port 0 MAC error */
#define NETC_SW_ENETC_UNMACESR_PORT0(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACESR_PORT0_SHIFT)) & NETC_SW_ENETC_UNMACESR_PORT0_MASK)

#define NETC_SW_ENETC_UNMACESR_PORT1_MASK        (0x2U)
#define NETC_SW_ENETC_UNMACESR_PORT1_SHIFT       (1U)
/*! PORT1 - Port 1 MAC error */
#define NETC_SW_ENETC_UNMACESR_PORT1(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACESR_PORT1_SHIFT)) & NETC_SW_ENETC_UNMACESR_PORT1_MASK)

#define NETC_SW_ENETC_UNMACESR_PORT2_MASK        (0x4U)
#define NETC_SW_ENETC_UNMACESR_PORT2_SHIFT       (2U)
/*! PORT2 - Port 2 MAC error */
#define NETC_SW_ENETC_UNMACESR_PORT2(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACESR_PORT2_SHIFT)) & NETC_SW_ENETC_UNMACESR_PORT2_MASK)

#define NETC_SW_ENETC_UNMACESR_PORT3_MASK        (0x8U)
#define NETC_SW_ENETC_UNMACESR_PORT3_SHIFT       (3U)
/*! PORT3 - Port 3 MAC error */
#define NETC_SW_ENETC_UNMACESR_PORT3(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACESR_PORT3_SHIFT)) & NETC_SW_ENETC_UNMACESR_PORT3_MASK)

#define NETC_SW_ENETC_UNMACESR_PORT4_MASK        (0x10U)
#define NETC_SW_ENETC_UNMACESR_PORT4_SHIFT       (4U)
/*! PORT4 - Port 4 MAC error */
#define NETC_SW_ENETC_UNMACESR_PORT4(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMACESR_PORT4_SHIFT)) & NETC_SW_ENETC_UNMACESR_PORT4_MASK)
/*! @} */

/*! @name UNSBECR - Uncorrectable non-fatal system bus error configuration register */
/*! @{ */

#define NETC_SW_ENETC_UNSBECR_THRESHOLD_MASK     (0xFFU)
#define NETC_SW_ENETC_UNSBECR_THRESHOLD_SHIFT    (0U)
/*! THRESHOLD - Threshold */
#define NETC_SW_ENETC_UNSBECR_THRESHOLD(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNSBECR_THRESHOLD_SHIFT)) & NETC_SW_ENETC_UNSBECR_THRESHOLD_MASK)
/*! @} */

/*! @name UNSBESR - Uncorrectable non-fatal system bus error status register */
/*! @{ */

#define NETC_SW_ENETC_UNSBESR_SB_ID_MASK         (0xFU)
#define NETC_SW_ENETC_UNSBESR_SB_ID_SHIFT        (0U)
/*! SB_ID - System Bus ID */
#define NETC_SW_ENETC_UNSBESR_SB_ID(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNSBESR_SB_ID_SHIFT)) & NETC_SW_ENETC_UNSBESR_SB_ID_MASK)

#define NETC_SW_ENETC_UNSBESR_SBE_MASK           (0x80000000U)
#define NETC_SW_ENETC_UNSBESR_SBE_SHIFT          (31U)
/*! SBE - System bus error */
#define NETC_SW_ENETC_UNSBESR_SBE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNSBESR_SBE_SHIFT)) & NETC_SW_ENETC_UNSBESR_SBE_MASK)
/*! @} */

/*! @name UNSBECTR - Uncorrectable non-fatal system bus error count register */
/*! @{ */

#define NETC_SW_ENETC_UNSBECTR_COUNT_MASK        (0xFFU)
#define NETC_SW_ENETC_UNSBECTR_COUNT_SHIFT       (0U)
/*! COUNT - Count */
#define NETC_SW_ENETC_UNSBECTR_COUNT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNSBECTR_COUNT_SHIFT)) & NETC_SW_ENETC_UNSBECTR_COUNT_MASK)
/*! @} */

/*! @name UFSBECR - Uncorrectable fatal system bus error configuration register */
/*! @{ */

#define NETC_SW_ENETC_UFSBECR_RD_MASK            (0x80000000U)
#define NETC_SW_ENETC_UFSBECR_RD_SHIFT           (31U)
/*! RD - Report disable
 *  0b0..Enabled
 *  0b1..Disabled
 */
#define NETC_SW_ENETC_UFSBECR_RD(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFSBECR_RD_SHIFT)) & NETC_SW_ENETC_UFSBECR_RD_MASK)
/*! @} */

/*! @name UFSBESR - Uncorrectable fatal system bus error status register */
/*! @{ */

#define NETC_SW_ENETC_UFSBESR_SB_ID_MASK         (0xFU)
#define NETC_SW_ENETC_UFSBESR_SB_ID_SHIFT        (0U)
/*! SB_ID - System Bus ID */
#define NETC_SW_ENETC_UFSBESR_SB_ID(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFSBESR_SB_ID_SHIFT)) & NETC_SW_ENETC_UFSBESR_SB_ID_MASK)

#define NETC_SW_ENETC_UFSBESR_M_MASK             (0x40000000U)
#define NETC_SW_ENETC_UFSBESR_M_SHIFT            (30U)
/*! M - Multiple */
#define NETC_SW_ENETC_UFSBESR_M(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFSBESR_M_SHIFT)) & NETC_SW_ENETC_UFSBESR_M_MASK)

#define NETC_SW_ENETC_UFSBESR_SBE_MASK           (0x80000000U)
#define NETC_SW_ENETC_UFSBESR_SBE_SHIFT          (31U)
/*! SBE - System bus error */
#define NETC_SW_ENETC_UFSBESR_SBE(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFSBESR_SBE_SHIFT)) & NETC_SW_ENETC_UFSBESR_SBE_MASK)
/*! @} */

/*! @name UNMECR - Uncorrectable non-fatal memory error configuration register */
/*! @{ */

#define NETC_SW_ENETC_UNMECR_THRESHOLD_MASK      (0xFFU)
#define NETC_SW_ENETC_UNMECR_THRESHOLD_SHIFT     (0U)
/*! THRESHOLD - Threshold */
#define NETC_SW_ENETC_UNMECR_THRESHOLD(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMECR_THRESHOLD_SHIFT)) & NETC_SW_ENETC_UNMECR_THRESHOLD_MASK)

#define NETC_SW_ENETC_UNMECR_RD_MASK             (0x80000000U)
#define NETC_SW_ENETC_UNMECR_RD_SHIFT            (31U)
/*! RD - Report disable */
#define NETC_SW_ENETC_UNMECR_RD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMECR_RD_SHIFT)) & NETC_SW_ENETC_UNMECR_RD_MASK)
/*! @} */

/*! @name UNMESR0 - Uncorrectable non-fatal memory error status register 0 */
/*! @{ */

#define NETC_SW_ENETC_UNMESR0_SYNDROME_MASK      (0x7FFU)
#define NETC_SW_ENETC_UNMESR0_SYNDROME_SHIFT     (0U)
/*! SYNDROME - Syndrome */
#define NETC_SW_ENETC_UNMESR0_SYNDROME(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMESR0_SYNDROME_SHIFT)) & NETC_SW_ENETC_UNMESR0_SYNDROME_MASK)

#define NETC_SW_ENETC_UNMESR0_MEM_ID_MASK        (0x1F0000U)
#define NETC_SW_ENETC_UNMESR0_MEM_ID_SHIFT       (16U)
/*! MEM_ID - Memory ID */
#define NETC_SW_ENETC_UNMESR0_MEM_ID(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMESR0_MEM_ID_SHIFT)) & NETC_SW_ENETC_UNMESR0_MEM_ID_MASK)

#define NETC_SW_ENETC_UNMESR0_MBEE_MASK          (0x80000000U)
#define NETC_SW_ENETC_UNMESR0_MBEE_SHIFT         (31U)
/*! MBEE - Multi-bit ECC error */
#define NETC_SW_ENETC_UNMESR0_MBEE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMESR0_MBEE_SHIFT)) & NETC_SW_ENETC_UNMESR0_MBEE_MASK)
/*! @} */

/*! @name UNMESR1 - Uncorrectable non-fatal memory error status register 1 */
/*! @{ */

#define NETC_SW_ENETC_UNMESR1_ADDR_MASK          (0xFFFFFFFFU)
#define NETC_SW_ENETC_UNMESR1_ADDR_SHIFT         (0U)
/*! ADDR - Address */
#define NETC_SW_ENETC_UNMESR1_ADDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMESR1_ADDR_SHIFT)) & NETC_SW_ENETC_UNMESR1_ADDR_MASK)
/*! @} */

/*! @name UNMECTR - Uncorrectable non-fatal memory error count register */
/*! @{ */

#define NETC_SW_ENETC_UNMECTR_COUNT_MASK         (0xFFU)
#define NETC_SW_ENETC_UNMECTR_COUNT_SHIFT        (0U)
/*! COUNT - Count */
#define NETC_SW_ENETC_UNMECTR_COUNT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UNMECTR_COUNT_SHIFT)) & NETC_SW_ENETC_UNMECTR_COUNT_MASK)
/*! @} */

/*! @name UFMECR - Uncorrectable fatal memory error configuration register */
/*! @{ */

#define NETC_SW_ENETC_UFMECR_RD_MASK             (0x80000000U)
#define NETC_SW_ENETC_UFMECR_RD_SHIFT            (31U)
/*! RD - Report disable */
#define NETC_SW_ENETC_UFMECR_RD(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFMECR_RD_SHIFT)) & NETC_SW_ENETC_UFMECR_RD_MASK)
/*! @} */

/*! @name UFMESR0 - Uncorrectable fatal memory error status register 0 */
/*! @{ */

#define NETC_SW_ENETC_UFMESR0_SYNDROME_MASK      (0x7FFU)
#define NETC_SW_ENETC_UFMESR0_SYNDROME_SHIFT     (0U)
/*! SYNDROME - Syndrome */
#define NETC_SW_ENETC_UFMESR0_SYNDROME(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFMESR0_SYNDROME_SHIFT)) & NETC_SW_ENETC_UFMESR0_SYNDROME_MASK)

#define NETC_SW_ENETC_UFMESR0_MEM_ID_MASK        (0x1F0000U)
#define NETC_SW_ENETC_UFMESR0_MEM_ID_SHIFT       (16U)
/*! MEM_ID - Memory ID */
#define NETC_SW_ENETC_UFMESR0_MEM_ID(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFMESR0_MEM_ID_SHIFT)) & NETC_SW_ENETC_UFMESR0_MEM_ID_MASK)

#define NETC_SW_ENETC_UFMESR0_M_MASK             (0x40000000U)
#define NETC_SW_ENETC_UFMESR0_M_SHIFT            (30U)
/*! M - Multiple */
#define NETC_SW_ENETC_UFMESR0_M(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFMESR0_M_SHIFT)) & NETC_SW_ENETC_UFMESR0_M_MASK)

#define NETC_SW_ENETC_UFMESR0_MBEE_MASK          (0x80000000U)
#define NETC_SW_ENETC_UFMESR0_MBEE_SHIFT         (31U)
/*! MBEE - Multi-bit ECC error */
#define NETC_SW_ENETC_UFMESR0_MBEE(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFMESR0_MBEE_SHIFT)) & NETC_SW_ENETC_UFMESR0_MBEE_MASK)
/*! @} */

/*! @name UFMESR1 - Uncorrectable fatal memory error status register 1 */
/*! @{ */

#define NETC_SW_ENETC_UFMESR1_ADDR_MASK          (0xFFFFFFFFU)
#define NETC_SW_ENETC_UFMESR1_ADDR_SHIFT         (0U)
/*! ADDR - Address */
#define NETC_SW_ENETC_UFMESR1_ADDR(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_UFMESR1_ADDR_SHIFT)) & NETC_SW_ENETC_UFMESR1_ADDR_MASK)
/*! @} */

/*! @name IMDIOIRR - Internal MDIO interrupt reason register */
/*! @{ */

#define NETC_SW_ENETC_IMDIOIRR_PORT0_MASK        (0x1U)
#define NETC_SW_ENETC_IMDIOIRR_PORT0_SHIFT       (0U)
#define NETC_SW_ENETC_IMDIOIRR_PORT0(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IMDIOIRR_PORT0_SHIFT)) & NETC_SW_ENETC_IMDIOIRR_PORT0_MASK)

#define NETC_SW_ENETC_IMDIOIRR_PORT1_MASK        (0x2U)
#define NETC_SW_ENETC_IMDIOIRR_PORT1_SHIFT       (1U)
#define NETC_SW_ENETC_IMDIOIRR_PORT1(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IMDIOIRR_PORT1_SHIFT)) & NETC_SW_ENETC_IMDIOIRR_PORT1_MASK)

#define NETC_SW_ENETC_IMDIOIRR_PORT2_MASK        (0x4U)
#define NETC_SW_ENETC_IMDIOIRR_PORT2_SHIFT       (2U)
#define NETC_SW_ENETC_IMDIOIRR_PORT2(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IMDIOIRR_PORT2_SHIFT)) & NETC_SW_ENETC_IMDIOIRR_PORT2_MASK)

#define NETC_SW_ENETC_IMDIOIRR_PORT3_MASK        (0x8U)
#define NETC_SW_ENETC_IMDIOIRR_PORT3_SHIFT       (3U)
#define NETC_SW_ENETC_IMDIOIRR_PORT3(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IMDIOIRR_PORT3_SHIFT)) & NETC_SW_ENETC_IMDIOIRR_PORT3_MASK)

#define NETC_SW_ENETC_IMDIOIRR_PORT4_MASK        (0x10U)
#define NETC_SW_ENETC_IMDIOIRR_PORT4_SHIFT       (4U)
#define NETC_SW_ENETC_IMDIOIRR_PORT4(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IMDIOIRR_PORT4_SHIFT)) & NETC_SW_ENETC_IMDIOIRR_PORT4_MASK)
/*! @} */

/*! @name IMDIOMSIVR - Internal MDIO MSI-X vector register */
/*! @{ */

#define NETC_SW_ENETC_IMDIOMSIVR_VECTOR_MASK     (0x3FU)  /* Merged from fields with different position or width, of widths (4, 6), largest definition used */
#define NETC_SW_ENETC_IMDIOMSIVR_VECTOR_SHIFT    (0U)
#define NETC_SW_ENETC_IMDIOMSIVR_VECTOR(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IMDIOMSIVR_VECTOR_SHIFT)) & NETC_SW_ENETC_IMDIOMSIVR_VECTOR_MASK)  /* Merged from fields with different position or width, of widths (4, 6), largest definition used */
/*! @} */

/*! @name EMDIOIRR - External MDIO interrupt reason register */
/*! @{ */

#define NETC_SW_ENETC_EMDIOIRR_PORT0_MASK        (0x1U)
#define NETC_SW_ENETC_EMDIOIRR_PORT0_SHIFT       (0U)
#define NETC_SW_ENETC_EMDIOIRR_PORT0(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EMDIOIRR_PORT0_SHIFT)) & NETC_SW_ENETC_EMDIOIRR_PORT0_MASK)

#define NETC_SW_ENETC_EMDIOIRR_PORT1_MASK        (0x2U)
#define NETC_SW_ENETC_EMDIOIRR_PORT1_SHIFT       (1U)
#define NETC_SW_ENETC_EMDIOIRR_PORT1(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EMDIOIRR_PORT1_SHIFT)) & NETC_SW_ENETC_EMDIOIRR_PORT1_MASK)

#define NETC_SW_ENETC_EMDIOIRR_PORT2_MASK        (0x4U)
#define NETC_SW_ENETC_EMDIOIRR_PORT2_SHIFT       (2U)
#define NETC_SW_ENETC_EMDIOIRR_PORT2(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EMDIOIRR_PORT2_SHIFT)) & NETC_SW_ENETC_EMDIOIRR_PORT2_MASK)

#define NETC_SW_ENETC_EMDIOIRR_PORT3_MASK        (0x8U)
#define NETC_SW_ENETC_EMDIOIRR_PORT3_SHIFT       (3U)
#define NETC_SW_ENETC_EMDIOIRR_PORT3(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EMDIOIRR_PORT3_SHIFT)) & NETC_SW_ENETC_EMDIOIRR_PORT3_MASK)

#define NETC_SW_ENETC_EMDIOIRR_PORT4_MASK        (0x10U)
#define NETC_SW_ENETC_EMDIOIRR_PORT4_SHIFT       (4U)
#define NETC_SW_ENETC_EMDIOIRR_PORT4(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EMDIOIRR_PORT4_SHIFT)) & NETC_SW_ENETC_EMDIOIRR_PORT4_MASK)
/*! @} */

/*! @name EMDIOMSIVR - External MDIO MSI-X vector register */
/*! @{ */

#define NETC_SW_ENETC_EMDIOMSIVR_VECTOR_MASK     (0x3FU)  /* Merged from fields with different position or width, of widths (4, 6), largest definition used */
#define NETC_SW_ENETC_EMDIOMSIVR_VECTOR_SHIFT    (0U)
#define NETC_SW_ENETC_EMDIOMSIVR_VECTOR(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_EMDIOMSIVR_VECTOR_SHIFT)) & NETC_SW_ENETC_EMDIOMSIVR_VECTOR_MASK)  /* Merged from fields with different position or width, of widths (4, 6), largest definition used */
/*! @} */

/*! @name TCCR - Time capture configuration register */
/*! @{ */

#define NETC_SW_ENETC_TCCR_TIMEOUT_MASK          (0xFFFFFFU)
#define NETC_SW_ENETC_TCCR_TIMEOUT_SHIFT         (0U)
#define NETC_SW_ENETC_TCCR_TIMEOUT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCCR_TIMEOUT_SHIFT)) & NETC_SW_ENETC_TCCR_TIMEOUT_MASK)

#define NETC_SW_ENETC_TCCR_ARM_MASK              (0xC0000000U)
#define NETC_SW_ENETC_TCCR_ARM_SHIFT             (30U)
#define NETC_SW_ENETC_TCCR_ARM(x)                (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCCR_ARM_SHIFT)) & NETC_SW_ENETC_TCCR_ARM_MASK)
/*! @} */

/*! @name TCIER - Time capture interrupt enable register */
/*! @{ */

#define NETC_SW_ENETC_TCIER_TRANSMIT_MASK        (0x40000000U)
#define NETC_SW_ENETC_TCIER_TRANSMIT_SHIFT       (30U)
/*! TRANSMIT - Transmit interrupt */
#define NETC_SW_ENETC_TCIER_TRANSMIT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCIER_TRANSMIT_SHIFT)) & NETC_SW_ENETC_TCIER_TRANSMIT_MASK)

#define NETC_SW_ENETC_TCIER_TIMEOUT_MASK         (0x80000000U)
#define NETC_SW_ENETC_TCIER_TIMEOUT_SHIFT        (31U)
/*! TIMEOUT - Timeout interrupt */
#define NETC_SW_ENETC_TCIER_TIMEOUT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCIER_TIMEOUT_SHIFT)) & NETC_SW_ENETC_TCIER_TIMEOUT_MASK)
/*! @} */

/*! @name TCRPIDR - Time capture receive port interrupt detect register */
/*! @{ */

#define NETC_SW_ENETC_TCRPIDR_TX_PORT0_MASK      (0x1U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT0_SHIFT     (0U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT0(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPIDR_TX_PORT0_SHIFT)) & NETC_SW_ENETC_TCRPIDR_TX_PORT0_MASK)

#define NETC_SW_ENETC_TCRPIDR_TX_PORT1_MASK      (0x2U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT1_SHIFT     (1U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT1(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPIDR_TX_PORT1_SHIFT)) & NETC_SW_ENETC_TCRPIDR_TX_PORT1_MASK)

#define NETC_SW_ENETC_TCRPIDR_TX_PORT2_MASK      (0x4U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT2_SHIFT     (2U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT2(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPIDR_TX_PORT2_SHIFT)) & NETC_SW_ENETC_TCRPIDR_TX_PORT2_MASK)

#define NETC_SW_ENETC_TCRPIDR_TX_PORT3_MASK      (0x8U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT3_SHIFT     (3U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT3(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPIDR_TX_PORT3_SHIFT)) & NETC_SW_ENETC_TCRPIDR_TX_PORT3_MASK)

#define NETC_SW_ENETC_TCRPIDR_TX_PORT4_MASK      (0x10U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT4_SHIFT     (4U)
#define NETC_SW_ENETC_TCRPIDR_TX_PORT4(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPIDR_TX_PORT4_SHIFT)) & NETC_SW_ENETC_TCRPIDR_TX_PORT4_MASK)

#define NETC_SW_ENETC_TCRPIDR_TRANSMIT_MASK      (0x40000000U)
#define NETC_SW_ENETC_TCRPIDR_TRANSMIT_SHIFT     (30U)
/*! TRANSMIT - Transmit interrupt */
#define NETC_SW_ENETC_TCRPIDR_TRANSMIT(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPIDR_TRANSMIT_SHIFT)) & NETC_SW_ENETC_TCRPIDR_TRANSMIT_MASK)

#define NETC_SW_ENETC_TCRPIDR_TIMEOUT_MASK       (0x80000000U)
#define NETC_SW_ENETC_TCRPIDR_TIMEOUT_SHIFT      (31U)
/*! TIMEOUT - Timeout interrupt */
#define NETC_SW_ENETC_TCRPIDR_TIMEOUT(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPIDR_TIMEOUT_SHIFT)) & NETC_SW_ENETC_TCRPIDR_TIMEOUT_MASK)
/*! @} */

/*! @name TCRPSR - Time capture receive port status register */
/*! @{ */

#define NETC_SW_ENETC_TCRPSR_RX_PORT_MASK        (0x1FU)
#define NETC_SW_ENETC_TCRPSR_RX_PORT_SHIFT       (0U)
#define NETC_SW_ENETC_TCRPSR_RX_PORT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPSR_RX_PORT_SHIFT)) & NETC_SW_ENETC_TCRPSR_RX_PORT_MASK)

#define NETC_SW_ENETC_TCRPSR_RX_CNT_MASK         (0x300U)
#define NETC_SW_ENETC_TCRPSR_RX_CNT_SHIFT        (8U)
#define NETC_SW_ENETC_TCRPSR_RX_CNT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPSR_RX_CNT_SHIFT)) & NETC_SW_ENETC_TCRPSR_RX_CNT_MASK)
/*! @} */

/*! @name TCRPTSR - Time capture receive port timestamp register */
/*! @{ */

#define NETC_SW_ENETC_TCRPTSR_TIMESTAMP_MASK     (0xFFFFFFFFU)
#define NETC_SW_ENETC_TCRPTSR_TIMESTAMP_SHIFT    (0U)
#define NETC_SW_ENETC_TCRPTSR_TIMESTAMP(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCRPTSR_TIMESTAMP_SHIFT)) & NETC_SW_ENETC_TCRPTSR_TIMESTAMP_MASK)
/*! @} */

/*! @name TCMSIVR - Time capture MSI-X vector register */
/*! @{ */

#define NETC_SW_ENETC_TCMSIVR_VECTOR_MASK        (0xFU)
#define NETC_SW_ENETC_TCMSIVR_VECTOR_SHIFT       (0U)
#define NETC_SW_ENETC_TCMSIVR_VECTOR(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TCMSIVR_VECTOR_SHIFT)) & NETC_SW_ENETC_TCMSIVR_VECTOR_MASK)
/*! @} */

/*! @name CVLANR1 - Custom VLAN Ethertype register 1 */
/*! @{ */

#define NETC_SW_ENETC_CVLANR1_ETYPE_MASK         (0xFFFFU)
#define NETC_SW_ENETC_CVLANR1_ETYPE_SHIFT        (0U)
#define NETC_SW_ENETC_CVLANR1_ETYPE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CVLANR1_ETYPE_SHIFT)) & NETC_SW_ENETC_CVLANR1_ETYPE_MASK)

#define NETC_SW_ENETC_CVLANR1_V_MASK             (0x80000000U)
#define NETC_SW_ENETC_CVLANR1_V_SHIFT            (31U)
/*! V - Valid */
#define NETC_SW_ENETC_CVLANR1_V(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CVLANR1_V_SHIFT)) & NETC_SW_ENETC_CVLANR1_V_MASK)
/*! @} */

/*! @name CVLANR2 - Custom VLAN Ethertype register 2 */
/*! @{ */

#define NETC_SW_ENETC_CVLANR2_ETYPE_MASK         (0xFFFFU)
#define NETC_SW_ENETC_CVLANR2_ETYPE_SHIFT        (0U)
#define NETC_SW_ENETC_CVLANR2_ETYPE(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CVLANR2_ETYPE_SHIFT)) & NETC_SW_ENETC_CVLANR2_ETYPE_MASK)

#define NETC_SW_ENETC_CVLANR2_V_MASK             (0x80000000U)
#define NETC_SW_ENETC_CVLANR2_V_SHIFT            (31U)
/*! V - Valid */
#define NETC_SW_ENETC_CVLANR2_V(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_CVLANR2_V_SHIFT)) & NETC_SW_ENETC_CVLANR2_V_MASK)
/*! @} */

/*! @name PSRTAGETR - Pre-Standard RTAG Ethertype register */
/*! @{ */

#define NETC_SW_ENETC_PSRTAGETR_ETHERTYPE_MASK   (0xFFFFU)
#define NETC_SW_ENETC_PSRTAGETR_ETHERTYPE_SHIFT  (0U)
#define NETC_SW_ENETC_PSRTAGETR_ETHERTYPE(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_PSRTAGETR_ETHERTYPE_SHIFT)) & NETC_SW_ENETC_PSRTAGETR_ETHERTYPE_MASK)
/*! @} */

/*! @name DOSL2CR - DoS L2 configuration register */
/*! @{ */

#define NETC_SW_ENETC_DOSL2CR_SAMEADDR_MASK      (0x1U)
#define NETC_SW_ENETC_DOSL2CR_SAMEADDR_SHIFT     (0U)
#define NETC_SW_ENETC_DOSL2CR_SAMEADDR(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_DOSL2CR_SAMEADDR_SHIFT)) & NETC_SW_ENETC_DOSL2CR_SAMEADDR_MASK)

#define NETC_SW_ENETC_DOSL2CR_MSAMCC_MASK        (0x2U)
#define NETC_SW_ENETC_DOSL2CR_MSAMCC_SHIFT       (1U)
#define NETC_SW_ENETC_DOSL2CR_MSAMCC(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_DOSL2CR_MSAMCC_SHIFT)) & NETC_SW_ENETC_DOSL2CR_MSAMCC_MASK)
/*! @} */

/*! @name VLANIPVMPR0 - VLAN to IPV mapping profile 0 register 0..VLAN to IPV mapping profile 1 register 0 */
/*! @{ */

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_0_MASK (0x7U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_0_SHIFT (0U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_0(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_0_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_0_MASK)

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_1_MASK (0x70U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_1_SHIFT (4U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_1(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_1_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_1_MASK)

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_2_MASK (0x700U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_2_SHIFT (8U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_2(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_2_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_2_MASK)

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_3_MASK (0x7000U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_3_SHIFT (12U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_3(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_3_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_3_MASK)

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_4_MASK (0x70000U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_4_SHIFT (16U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_4(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_4_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_4_MASK)

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_5_MASK (0x700000U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_5_SHIFT (20U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_5(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_5_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_5_MASK)

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_6_MASK (0x7000000U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_6_SHIFT (24U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_6(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_6_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_6_MASK)

#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_7_MASK (0x70000000U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_7_SHIFT (28U)
#define NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_7(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_7_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR0_PCP_DEI_7_MASK)
/*! @} */

/* The count of NETC_SW_ENETC_VLANIPVMPR0 */
#define NETC_SW_ENETC_VLANIPVMPR0_COUNT          (2U)

/*! @name VLANIPVMPR1 - VLAN to IPV mapping profile 0 register 1..VLAN to IPV mapping profile 1 register 1 */
/*! @{ */

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_8_MASK (0x7U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_8_SHIFT (0U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_8(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_8_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_8_MASK)

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_9_MASK (0x70U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_9_SHIFT (4U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_9(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_9_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_9_MASK)

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_10_MASK (0x700U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_10_SHIFT (8U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_10(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_10_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_10_MASK)

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_11_MASK (0x7000U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_11_SHIFT (12U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_11(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_11_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_11_MASK)

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_12_MASK (0x70000U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_12_SHIFT (16U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_12(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_12_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_12_MASK)

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_13_MASK (0x700000U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_13_SHIFT (20U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_13(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_13_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_13_MASK)

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_14_MASK (0x7000000U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_14_SHIFT (24U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_14(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_14_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_14_MASK)

#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_15_MASK (0x70000000U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_15_SHIFT (28U)
#define NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_15(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_15_SHIFT)) & NETC_SW_ENETC_VLANIPVMPR1_PCP_DEI_15_MASK)
/*! @} */

/* The count of NETC_SW_ENETC_VLANIPVMPR1 */
#define NETC_SW_ENETC_VLANIPVMPR1_COUNT          (2U)

/*! @name VLANDRMPR - VLAN to DR mapping profile 0 register..VLAN to DR mapping profile 1 register */
/*! @{ */

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_0_MASK   (0x3U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_0_SHIFT  (0U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_0(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_0_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_0_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_1_MASK   (0xCU)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_1_SHIFT  (2U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_1(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_1_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_1_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_2_MASK   (0x30U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_2_SHIFT  (4U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_2(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_2_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_2_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_3_MASK   (0xC0U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_3_SHIFT  (6U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_3(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_3_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_3_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_4_MASK   (0x300U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_4_SHIFT  (8U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_4(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_4_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_4_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_5_MASK   (0xC00U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_5_SHIFT  (10U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_5(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_5_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_5_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_6_MASK   (0x3000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_6_SHIFT  (12U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_6(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_6_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_6_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_7_MASK   (0xC000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_7_SHIFT  (14U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_7(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_7_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_7_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_8_MASK   (0x30000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_8_SHIFT  (16U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_8(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_8_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_8_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_9_MASK   (0xC0000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_9_SHIFT  (18U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_9(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_9_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_9_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_10_MASK  (0x300000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_10_SHIFT (20U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_10(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_10_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_10_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_11_MASK  (0xC00000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_11_SHIFT (22U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_11(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_11_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_11_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_12_MASK  (0x3000000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_12_SHIFT (24U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_12(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_12_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_12_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_13_MASK  (0xC000000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_13_SHIFT (26U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_13(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_13_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_13_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_14_MASK  (0x30000000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_14_SHIFT (28U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_14(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_14_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_14_MASK)

#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_15_MASK  (0xC0000000U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_15_SHIFT (30U)
#define NETC_SW_ENETC_VLANDRMPR_PCP_DEI_15(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_VLANDRMPR_PCP_DEI_15_SHIFT)) & NETC_SW_ENETC_VLANDRMPR_PCP_DEI_15_MASK)
/*! @} */

/* The count of NETC_SW_ENETC_VLANDRMPR */
#define NETC_SW_ENETC_VLANDRMPR_COUNT            (2U)

/*! @name IPFCAPR - Ingress port filter capability register */
/*! @{ */

#define NETC_SW_ENETC_IPFCAPR_RP_MASK            (0x1U)
#define NETC_SW_ENETC_IPFCAPR_RP_SHIFT           (0U)
/*! RP - Rate Policer function supported */
#define NETC_SW_ENETC_IPFCAPR_RP(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFCAPR_RP_SHIFT)) & NETC_SW_ENETC_IPFCAPR_RP_MASK)

#define NETC_SW_ENETC_IPFCAPR_ISID_MASK          (0x2U)
#define NETC_SW_ENETC_IPFCAPR_ISID_SHIFT         (1U)
/*! ISID - Ingress Stream Identification supported. */
#define NETC_SW_ENETC_IPFCAPR_ISID(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFCAPR_ISID_SHIFT)) & NETC_SW_ENETC_IPFCAPR_ISID_MASK)

#define NETC_SW_ENETC_IPFCAPR_FWD_SI_MASK        (0x4U)
#define NETC_SW_ENETC_IPFCAPR_FWD_SI_SHIFT       (2U)
#define NETC_SW_ENETC_IPFCAPR_FWD_SI(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFCAPR_FWD_SI_SHIFT)) & NETC_SW_ENETC_IPFCAPR_FWD_SI_MASK)

#define NETC_SW_ENETC_IPFCAPR_WOL_MASK           (0x8U)
#define NETC_SW_ENETC_IPFCAPR_WOL_SHIFT          (3U)
#define NETC_SW_ENETC_IPFCAPR_WOL(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFCAPR_WOL_SHIFT)) & NETC_SW_ENETC_IPFCAPR_WOL_MASK)
/*! @} */

/*! @name IPFTCAPR - Ingress port filter table capability register */
/*! @{ */

#define NETC_SW_ENETC_IPFTCAPR_NUM_WORDS_MASK    (0xFFFFU)
#define NETC_SW_ENETC_IPFTCAPR_NUM_WORDS_SHIFT   (0U)
#define NETC_SW_ENETC_IPFTCAPR_NUM_WORDS(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFTCAPR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_IPFTCAPR_NUM_WORDS_MASK)

#define NETC_SW_ENETC_IPFTCAPR_MGMT_MASK         (0x10000U)
#define NETC_SW_ENETC_IPFTCAPR_MGMT_SHIFT        (16U)
#define NETC_SW_ENETC_IPFTCAPR_MGMT(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFTCAPR_MGMT_SHIFT)) & NETC_SW_ENETC_IPFTCAPR_MGMT_MASK)

#define NETC_SW_ENETC_IPFTCAPR_ACCESS_METH_MASK  (0xF00000U)
#define NETC_SW_ENETC_IPFTCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_IPFTCAPR_ACCESS_METH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_IPFTCAPR_ACCESS_METH_MASK)

#define NETC_SW_ENETC_IPFTCAPR_ENTRY_MAX_WORDS_MASK (0xF000000U)
#define NETC_SW_ENETC_IPFTCAPR_ENTRY_MAX_WORDS_SHIFT (24U)
#define NETC_SW_ENETC_IPFTCAPR_ENTRY_MAX_WORDS(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFTCAPR_ENTRY_MAX_WORDS_SHIFT)) & NETC_SW_ENETC_IPFTCAPR_ENTRY_MAX_WORDS_MASK)

#define NETC_SW_ENETC_IPFTCAPR_WORD_SIZE_MASK    (0x30000000U)
#define NETC_SW_ENETC_IPFTCAPR_WORD_SIZE_SHIFT   (28U)
#define NETC_SW_ENETC_IPFTCAPR_WORD_SIZE(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFTCAPR_WORD_SIZE_SHIFT)) & NETC_SW_ENETC_IPFTCAPR_WORD_SIZE_MASK)
/*! @} */

/*! @name IPFTMOR - Ingress port filter table memory operational register */
/*! @{ */

#define NETC_SW_ENETC_IPFTMOR_NUM_WORDS_MASK     (0xFFFFU)
#define NETC_SW_ENETC_IPFTMOR_NUM_WORDS_SHIFT    (0U)
#define NETC_SW_ENETC_IPFTMOR_NUM_WORDS(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_IPFTMOR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_IPFTMOR_NUM_WORDS_MASK)
/*! @} */

/*! @name ITMCAPR - Index table memory capability register */
/*! @{ */

#define NETC_SW_ENETC_ITMCAPR_NUM_WORDS_MASK     (0xFFFFU)
#define NETC_SW_ENETC_ITMCAPR_NUM_WORDS_SHIFT    (0U)
#define NETC_SW_ENETC_ITMCAPR_NUM_WORDS(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ITMCAPR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_ITMCAPR_NUM_WORDS_MASK)

#define NETC_SW_ENETC_ITMCAPR_WORD_SIZE_MASK     (0x30000000U)
#define NETC_SW_ENETC_ITMCAPR_WORD_SIZE_SHIFT    (28U)
#define NETC_SW_ENETC_ITMCAPR_WORD_SIZE(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ITMCAPR_WORD_SIZE_SHIFT)) & NETC_SW_ENETC_ITMCAPR_WORD_SIZE_MASK)

#define NETC_SW_ENETC_ITMCAPR_MLOC_MASK          (0xC0000000U)
#define NETC_SW_ENETC_ITMCAPR_MLOC_SHIFT         (30U)
#define NETC_SW_ENETC_ITMCAPR_MLOC(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ITMCAPR_MLOC_SHIFT)) & NETC_SW_ENETC_ITMCAPR_MLOC_MASK)
/*! @} */

/*! @name RPCAPR - Rate policer capability register */
/*! @{ */

#define NETC_SW_ENETC_RPCAPR_TRTCM_MASK          (0x1U)
#define NETC_SW_ENETC_RPCAPR_TRTCM_SHIFT         (0U)
/*! TRTCM - Two-Rate Three-Color Marker supported per MEF 10.3 standard. */
#define NETC_SW_ENETC_RPCAPR_TRTCM(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_RPCAPR_TRTCM_SHIFT)) & NETC_SW_ENETC_RPCAPR_TRTCM_MASK)

#define NETC_SW_ENETC_RPCAPR_CM_MASK             (0x2U)
#define NETC_SW_ENETC_RPCAPR_CM_SHIFT            (1U)
#define NETC_SW_ENETC_RPCAPR_CM(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_RPCAPR_CM_SHIFT)) & NETC_SW_ENETC_RPCAPR_CM_MASK)
/*! @} */

/*! @name RPITCAPR - Rate policer index table capability register */
/*! @{ */

#define NETC_SW_ENETC_RPITCAPR_NUM_ENTRIES_MASK  (0x3FFFU)
#define NETC_SW_ENETC_RPITCAPR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_RPITCAPR_NUM_ENTRIES(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_RPITCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_RPITCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_RPITCAPR_ACCESS_METH_MASK  (0xF00000U)
#define NETC_SW_ENETC_RPITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_RPITCAPR_ACCESS_METH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_RPITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_RPITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name RPITMAR - Rate policer index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_RPITMAR_NUM_WORDS_MASK     (0xFFFFU)
#define NETC_SW_ENETC_RPITMAR_NUM_WORDS_SHIFT    (0U)
#define NETC_SW_ENETC_RPITMAR_NUM_WORDS(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_RPITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_RPITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name RPITOR - Rate policer index table operational register */
/*! @{ */

#define NETC_SW_ENETC_RPITOR_NUM_ENTRIES_MASK    (0x3FFFU)
#define NETC_SW_ENETC_RPITOR_NUM_ENTRIES_SHIFT   (0U)
#define NETC_SW_ENETC_RPITOR_NUM_ENTRIES(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_RPITOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_RPITOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name ISCITCAPR - Ingress stream counter index table capability register */
/*! @{ */

#define NETC_SW_ENETC_ISCITCAPR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_ISCITCAPR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISCITCAPR_NUM_ENTRIES(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCITCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISCITCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ISCITCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_ISCITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_ISCITCAPR_ACCESS_METH(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ISCITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name ISCITMAR - Ingress stream counter index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_ISCITMAR_NUM_WORDS_MASK    (0xFFFFU)
#define NETC_SW_ENETC_ISCITMAR_NUM_WORDS_SHIFT   (0U)
#define NETC_SW_ENETC_ISCITMAR_NUM_WORDS(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_ISCITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name ISCITOR - Ingress stream counter index table operational register */
/*! @{ */

#define NETC_SW_ENETC_ISCITOR_NUM_ENTRIES_MASK   (0xFFFFU)
#define NETC_SW_ENETC_ISCITOR_NUM_ENTRIES_SHIFT  (0U)
#define NETC_SW_ENETC_ISCITOR_NUM_ENTRIES(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCITOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISCITOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name ISCAPR - Ingress stream capability register */
/*! @{ */

#define NETC_SW_ENETC_ISCAPR_ISQG_MASK           (0x2U)
#define NETC_SW_ENETC_ISCAPR_ISQG_SHIFT          (1U)
#define NETC_SW_ENETC_ISCAPR_ISQG(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCAPR_ISQG_SHIFT)) & NETC_SW_ENETC_ISCAPR_ISQG_MASK)

#define NETC_SW_ENETC_ISCAPR_SG_MASK             (0x8U)
#define NETC_SW_ENETC_ISCAPR_SG_SHIFT            (3U)
#define NETC_SW_ENETC_ISCAPR_SG(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCAPR_SG_SHIFT)) & NETC_SW_ENETC_ISCAPR_SG_MASK)

#define NETC_SW_ENETC_ISCAPR_RP_MASK             (0x10U)
#define NETC_SW_ENETC_ISCAPR_RP_SHIFT            (4U)
#define NETC_SW_ENETC_ISCAPR_RP(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCAPR_RP_SHIFT)) & NETC_SW_ENETC_ISCAPR_RP_MASK)

#define NETC_SW_ENETC_ISCAPR_MAXSDU_MASK         (0x20U)
#define NETC_SW_ENETC_ISCAPR_MAXSDU_SHIFT        (5U)
#define NETC_SW_ENETC_ISCAPR_MAXSDU(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCAPR_MAXSDU_SHIFT)) & NETC_SW_ENETC_ISCAPR_MAXSDU_MASK)

#define NETC_SW_ENETC_ISCAPR_FWD_MASK            (0x200U)
#define NETC_SW_ENETC_ISCAPR_FWD_SHIFT           (9U)
#define NETC_SW_ENETC_ISCAPR_FWD(x)              (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCAPR_FWD_SHIFT)) & NETC_SW_ENETC_ISCAPR_FWD_MASK)

#define NETC_SW_ENETC_ISCAPR_ET_MASK             (0x400U)
#define NETC_SW_ENETC_ISCAPR_ET_SHIFT            (10U)
#define NETC_SW_ENETC_ISCAPR_ET(x)               (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISCAPR_ET_SHIFT)) & NETC_SW_ENETC_ISCAPR_ET_MASK)
/*! @} */

/*! @name ISITCAPR - Ingress stream index table capability register */
/*! @{ */

#define NETC_SW_ENETC_ISITCAPR_NUM_ENTRIES_MASK  (0xFFFFU)
#define NETC_SW_ENETC_ISITCAPR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISITCAPR_NUM_ENTRIES(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISITCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISITCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ISITCAPR_ACCESS_METH_MASK  (0xF00000U)
#define NETC_SW_ENETC_ISITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_ISITCAPR_ACCESS_METH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ISITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name ISITMAR - Ingress stream index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_ISITMAR_NUM_WORDS_MASK     (0xFFFFU)
#define NETC_SW_ENETC_ISITMAR_NUM_WORDS_SHIFT    (0U)
#define NETC_SW_ENETC_ISITMAR_NUM_WORDS(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_ISITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name ISITOR - Ingress stream index table operational register */
/*! @{ */

#define NETC_SW_ENETC_ISITOR_NUM_ENTRIES_MASK    (0xFFFFU)
#define NETC_SW_ENETC_ISITOR_NUM_ENTRIES_SHIFT   (0U)
#define NETC_SW_ENETC_ISITOR_NUM_ENTRIES(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISITOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISITOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name ISQGITCAPR - Ingress sequence generation index table capability register */
/*! @{ */

#define NETC_SW_ENETC_ISQGITCAPR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_ISQGITCAPR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISQGITCAPR_NUM_ENTRIES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISQGITCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISQGITCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ISQGITCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_ISQGITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_ISQGITCAPR_ACCESS_METH(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISQGITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ISQGITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name ISQGITMAR - Ingress sequence generation index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_ISQGITMAR_NUM_WORDS_MASK   (0x1FFFU)
#define NETC_SW_ENETC_ISQGITMAR_NUM_WORDS_SHIFT  (0U)
#define NETC_SW_ENETC_ISQGITMAR_NUM_WORDS(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISQGITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_ISQGITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name ISQGITOR - Ingress sequence generation index table operational register */
/*! @{ */

#define NETC_SW_ENETC_ISQGITOR_NUM_ENTRIES_MASK  (0xFFFFU)
#define NETC_SW_ENETC_ISQGITOR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISQGITOR_NUM_ENTRIES(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISQGITOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISQGITOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name SGCAPR - Stream gate capability register */
/*! @{ */

#define NETC_SW_ENETC_SGCAPR_GLC_AO_MASK         (0x1U)
#define NETC_SW_ENETC_SGCAPR_GLC_AO_SHIFT        (0U)
#define NETC_SW_ENETC_SGCAPR_GLC_AO(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCAPR_GLC_AO_SHIFT)) & NETC_SW_ENETC_SGCAPR_GLC_AO_MASK)

#define NETC_SW_ENETC_SGCAPR_GLC_GC_MASK         (0x2U)
#define NETC_SW_ENETC_SGCAPR_GLC_GC_SHIFT        (1U)
#define NETC_SW_ENETC_SGCAPR_GLC_GC(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCAPR_GLC_GC_SHIFT)) & NETC_SW_ENETC_SGCAPR_GLC_GC_MASK)

#define NETC_SW_ENETC_SGCAPR_GLC_IO_MASK         (0x4U)
#define NETC_SW_ENETC_SGCAPR_GLC_IO_SHIFT        (2U)
#define NETC_SW_ENETC_SGCAPR_GLC_IO(x)           (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCAPR_GLC_IO_SHIFT)) & NETC_SW_ENETC_SGCAPR_GLC_IO_MASK)

#define NETC_SW_ENETC_SGCAPR_GLC_IPV_MASK        (0x8U)
#define NETC_SW_ENETC_SGCAPR_GLC_IPV_SHIFT       (3U)
#define NETC_SW_ENETC_SGCAPR_GLC_IPV(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCAPR_GLC_IPV_SHIFT)) & NETC_SW_ENETC_SGCAPR_GLC_IPV_MASK)

#define NETC_SW_ENETC_SGCAPR_GLC_CTD_MASK        (0x10U)
#define NETC_SW_ENETC_SGCAPR_GLC_CTD_SHIFT       (4U)
#define NETC_SW_ENETC_SGCAPR_GLC_CTD(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCAPR_GLC_CTD_SHIFT)) & NETC_SW_ENETC_SGCAPR_GLC_CTD_MASK)
/*! @} */

/*! @name SGIITCAPR - Stream gate instance index table capability register */
/*! @{ */

#define NETC_SW_ENETC_SGIITCAPR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_SGIITCAPR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_SGIITCAPR_NUM_ENTRIES(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGIITCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_SGIITCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_SGIITCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_SGIITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_SGIITCAPR_ACCESS_METH(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGIITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_SGIITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name SGIITMAR - Stream gate instance index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_SGIITMAR_NUM_WORDS_MASK    (0xFFFFU)
#define NETC_SW_ENETC_SGIITMAR_NUM_WORDS_SHIFT   (0U)
#define NETC_SW_ENETC_SGIITMAR_NUM_WORDS(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGIITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_SGIITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name SGIITOR - Stream gate instance index table operational register */
/*! @{ */

#define NETC_SW_ENETC_SGIITOR_NUM_ENTRIES_MASK   (0xFFFFU)
#define NETC_SW_ENETC_SGIITOR_NUM_ENTRIES_SHIFT  (0U)
#define NETC_SW_ENETC_SGIITOR_NUM_ENTRIES(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGIITOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_SGIITOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name SGCLITCAPR - Stream gate control list index table capability register */
/*! @{ */

#define NETC_SW_ENETC_SGCLITCAPR_NUM_WORDS_MASK  (0xFFFFU)
#define NETC_SW_ENETC_SGCLITCAPR_NUM_WORDS_SHIFT (0U)
#define NETC_SW_ENETC_SGCLITCAPR_NUM_WORDS(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCLITCAPR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_SGCLITCAPR_NUM_WORDS_MASK)

#define NETC_SW_ENETC_SGCLITCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_SGCLITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_SGCLITCAPR_ACCESS_METH(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCLITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_SGCLITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name SGCLITMAR - Stream gate control list index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_SGCLITMAR_NUM_WORDS_MASK   (0xFFFFU)
#define NETC_SW_ENETC_SGCLITMAR_NUM_WORDS_SHIFT  (0U)
#define NETC_SW_ENETC_SGCLITMAR_NUM_WORDS(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCLITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_SGCLITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name SGCLTMOR - Stream gate control list table memory operational register */
/*! @{ */

#define NETC_SW_ENETC_SGCLTMOR_NUM_WORDS_MASK    (0xFFFFU)
#define NETC_SW_ENETC_SGCLTMOR_NUM_WORDS_SHIFT   (0U)
#define NETC_SW_ENETC_SGCLTMOR_NUM_WORDS(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_SGCLTMOR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_SGCLTMOR_NUM_WORDS_MASK)
/*! @} */

/*! @name FMICAPR - Frame modification ingress capability register */
/*! @{ */

#define NETC_SW_ENETC_FMICAPR_L2_ACT_MASK        (0xFFU)
#define NETC_SW_ENETC_FMICAPR_L2_ACT_SHIFT       (0U)
/*! L2_ACT - Layer 2 frame modification actions supported */
#define NETC_SW_ENETC_FMICAPR_L2_ACT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMICAPR_L2_ACT_SHIFT)) & NETC_SW_ENETC_FMICAPR_L2_ACT_MASK)
/*! @} */

/*! @name FMECAPR - Frame modification egress capability register */
/*! @{ */

#define NETC_SW_ENETC_FMECAPR_L2_ACT_MASK        (0xFFU)
#define NETC_SW_ENETC_FMECAPR_L2_ACT_SHIFT       (0U)
/*! L2_ACT - Layer 2 frame modification actions */
#define NETC_SW_ENETC_FMECAPR_L2_ACT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMECAPR_L2_ACT_SHIFT)) & NETC_SW_ENETC_FMECAPR_L2_ACT_MASK)

#define NETC_SW_ENETC_FMECAPR_L3_ACT_MASK        (0xFF0000U)
#define NETC_SW_ENETC_FMECAPR_L3_ACT_SHIFT       (16U)
/*! L3_ACT - Layer 3 frame modification actions */
#define NETC_SW_ENETC_FMECAPR_L3_ACT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMECAPR_L3_ACT_SHIFT)) & NETC_SW_ENETC_FMECAPR_L3_ACT_MASK)
/*! @} */

/*! @name FMITCAPR - Frame modification index table capability register */
/*! @{ */

#define NETC_SW_ENETC_FMITCAPR_NUM_ENTRIES_MASK  (0x1FFFU)
#define NETC_SW_ENETC_FMITCAPR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_FMITCAPR_NUM_ENTRIES(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMITCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_FMITCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_FMITCAPR_ACCESS_METH_MASK  (0xF00000U)
#define NETC_SW_ENETC_FMITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_FMITCAPR_ACCESS_METH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_FMITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name FMITMAR - Frame modification index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_FMITMAR_NUM_WORDS_MASK     (0x1FFFU)
#define NETC_SW_ENETC_FMITMAR_NUM_WORDS_SHIFT    (0U)
#define NETC_SW_ENETC_FMITMAR_NUM_WORDS(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_FMITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name FMITOR - Frame modification index table operational register */
/*! @{ */

#define NETC_SW_ENETC_FMITOR_NUM_ENTRIES_MASK    (0x1FFFU)
#define NETC_SW_ENETC_FMITOR_NUM_ENTRIES_SHIFT   (0U)
#define NETC_SW_ENETC_FMITOR_NUM_ENTRIES(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMITOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_FMITOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name FMDITCAPR - Frame modification data index table capability register */
/*! @{ */

#define NETC_SW_ENETC_FMDITCAPR_NUM_WORDS_MASK   (0xFFFFU)
#define NETC_SW_ENETC_FMDITCAPR_NUM_WORDS_SHIFT  (0U)
#define NETC_SW_ENETC_FMDITCAPR_NUM_WORDS(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMDITCAPR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_FMDITCAPR_NUM_WORDS_MASK)

#define NETC_SW_ENETC_FMDITCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_FMDITCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_FMDITCAPR_ACCESS_METH(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMDITCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_FMDITCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name FMDITMAR - Frame modification data index table memory allocation register */
/*! @{ */

#define NETC_SW_ENETC_FMDITMAR_NUM_WORDS_MASK    (0xFFFFU)
#define NETC_SW_ENETC_FMDITMAR_NUM_WORDS_SHIFT   (0U)
#define NETC_SW_ENETC_FMDITMAR_NUM_WORDS(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_FMDITMAR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_FMDITMAR_NUM_WORDS_MASK)
/*! @} */

/*! @name ETCAPR - Egress treatment capability register */
/*! @{ */

#define NETC_SW_ENETC_ETCAPR_ESQR_MASK           (0x1U)
#define NETC_SW_ENETC_ETCAPR_ESQR_SHIFT          (0U)
/*! ESQR - Egress Sequence Recovery supported */
#define NETC_SW_ENETC_ETCAPR_ESQR(x)             (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ETCAPR_ESQR_SHIFT)) & NETC_SW_ENETC_ETCAPR_ESQR_MASK)
/*! @} */

/*! @name ETTCAPR - Egress treatment table capability register */
/*! @{ */

#define NETC_SW_ENETC_ETTCAPR_NUM_ENTRIES_MASK   (0xFFFFU)
#define NETC_SW_ENETC_ETTCAPR_NUM_ENTRIES_SHIFT  (0U)
#define NETC_SW_ENETC_ETTCAPR_NUM_ENTRIES(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ETTCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ETTCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ETTCAPR_ACCESS_METH_MASK   (0xF00000U)
#define NETC_SW_ENETC_ETTCAPR_ACCESS_METH_SHIFT  (20U)
#define NETC_SW_ENETC_ETTCAPR_ACCESS_METH(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ETTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ETTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name ETTOR - Egress treatment table operational register */
/*! @{ */

#define NETC_SW_ENETC_ETTOR_NUM_ENTRIES_MASK     (0xFFFFU)
#define NETC_SW_ENETC_ETTOR_NUM_ENTRIES_SHIFT    (0U)
#define NETC_SW_ENETC_ETTOR_NUM_ENTRIES(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ETTOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ETTOR_NUM_ENTRIES_MASK)
/*! @} */

/*! @name TGSTCAPR - Time gate scheduling table capability register */
/*! @{ */

#define NETC_SW_ENETC_TGSTCAPR_NUM_WORDS_MASK    (0xFFFFU)
#define NETC_SW_ENETC_TGSTCAPR_NUM_WORDS_SHIFT   (0U)
/*! NUM_WORDS - Number of Words */
#define NETC_SW_ENETC_TGSTCAPR_NUM_WORDS(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TGSTCAPR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_TGSTCAPR_NUM_WORDS_MASK)

#define NETC_SW_ENETC_TGSTCAPR_ACCESS_METH_MASK  (0xF00000U)
#define NETC_SW_ENETC_TGSTCAPR_ACCESS_METH_SHIFT (20U)
/*! ACCESS_METH - Access Method
 *  0bxxx1..Index
 *  0bxx1x..EntryId
 *  0bx1xx..Search
 *  0b1xxx..
 */
#define NETC_SW_ENETC_TGSTCAPR_ACCESS_METH(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TGSTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_TGSTCAPR_ACCESS_METH_MASK)

#define NETC_SW_ENETC_TGSTCAPR_MAX_GCL_LEN_MASK  (0x3000000U)
#define NETC_SW_ENETC_TGSTCAPR_MAX_GCL_LEN_SHIFT (24U)
/*! MAX_GCL_LEN - Maximum Gate Control List Length
 *  0b00..64
 *  0b01..128
 *  0b10..256
 *  0b11..
 */
#define NETC_SW_ENETC_TGSTCAPR_MAX_GCL_LEN(x)    (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TGSTCAPR_MAX_GCL_LEN_SHIFT)) & NETC_SW_ENETC_TGSTCAPR_MAX_GCL_LEN_MASK)
/*! @} */

/*! @name TGSTMOR - Time gate scheduling table memory operation register */
/*! @{ */

#define NETC_SW_ENETC_TGSTMOR_NUM_WORDS_MASK     (0xFFFFU)
#define NETC_SW_ENETC_TGSTMOR_NUM_WORDS_SHIFT    (0U)
#define NETC_SW_ENETC_TGSTMOR_NUM_WORDS(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_TGSTMOR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_TGSTMOR_NUM_WORDS_MASK)
/*! @} */

/*! @name ESQRCAPR - Egress sequence recovery capability register */
/*! @{ */

#define NETC_SW_ENETC_ESQRCAPR_SQR_TYPE_MASK     (0x3U)
#define NETC_SW_ENETC_ESQRCAPR_SQR_TYPE_SHIFT    (0U)
#define NETC_SW_ENETC_ESQRCAPR_SQR_TYPE(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ESQRCAPR_SQR_TYPE_SHIFT)) & NETC_SW_ENETC_ESQRCAPR_SQR_TYPE_MASK)

#define NETC_SW_ENETC_ESQRCAPR_SQR_ALG_MASK      (0xCU)
#define NETC_SW_ENETC_ESQRCAPR_SQR_ALG_SHIFT     (2U)
#define NETC_SW_ENETC_ESQRCAPR_SQR_ALG(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ESQRCAPR_SQR_ALG_SHIFT)) & NETC_SW_ENETC_ESQRCAPR_SQR_ALG_MASK)

#define NETC_SW_ENETC_ESQRCAPR_SQR_MAX_HL_MASK   (0x700U)
#define NETC_SW_ENETC_ESQRCAPR_SQR_MAX_HL_SHIFT  (8U)
#define NETC_SW_ENETC_ESQRCAPR_SQR_MAX_HL(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ESQRCAPR_SQR_MAX_HL_SHIFT)) & NETC_SW_ENETC_ESQRCAPR_SQR_MAX_HL_MASK)
/*! @} */

/*! @name ESQRTCAPR - Egress sequence recovery table capability register */
/*! @{ */

#define NETC_SW_ENETC_ESQRTCAPR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_ESQRTCAPR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ESQRTCAPR_NUM_ENTRIES(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ESQRTCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ESQRTCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ESQRTCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_ESQRTCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_ESQRTCAPR_ACCESS_METH(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ESQRTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ESQRTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name ECTCAPR - Egress counter table capability register */
/*! @{ */

#define NETC_SW_ENETC_ECTCAPR_NUM_ENTRIES_MASK   (0xFFFFU)
#define NETC_SW_ENETC_ECTCAPR_NUM_ENTRIES_SHIFT  (0U)
#define NETC_SW_ENETC_ECTCAPR_NUM_ENTRIES(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ECTCAPR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ECTCAPR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ECTCAPR_ACCESS_METH_MASK   (0xF00000U)
#define NETC_SW_ENETC_ECTCAPR_ACCESS_METH_SHIFT  (20U)
#define NETC_SW_ENETC_ECTCAPR_ACCESS_METH(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ECTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ECTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name HTMCAPR - Hash table memory capability register */
/*! @{ */

#define NETC_SW_ENETC_HTMCAPR_NUM_WORDS_MASK     (0xFFFFU)
#define NETC_SW_ENETC_HTMCAPR_NUM_WORDS_SHIFT    (0U)
#define NETC_SW_ENETC_HTMCAPR_NUM_WORDS(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_HTMCAPR_NUM_WORDS_SHIFT)) & NETC_SW_ENETC_HTMCAPR_NUM_WORDS_MASK)

#define NETC_SW_ENETC_HTMCAPR_WORD_SIZE_MASK     (0x30000000U)
#define NETC_SW_ENETC_HTMCAPR_WORD_SIZE_SHIFT    (28U)
#define NETC_SW_ENETC_HTMCAPR_WORD_SIZE(x)       (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_HTMCAPR_WORD_SIZE_SHIFT)) & NETC_SW_ENETC_HTMCAPR_WORD_SIZE_MASK)

#define NETC_SW_ENETC_HTMCAPR_MLOC_MASK          (0xC0000000U)
#define NETC_SW_ENETC_HTMCAPR_MLOC_SHIFT         (30U)
#define NETC_SW_ENETC_HTMCAPR_MLOC(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_HTMCAPR_MLOC_SHIFT)) & NETC_SW_ENETC_HTMCAPR_MLOC_MASK)
/*! @} */

/*! @name HTMOR - Hash table memory operational register */
/*! @{ */

#define NETC_SW_ENETC_HTMOR_AMOUNT_MASK          (0xFFFFU)
#define NETC_SW_ENETC_HTMOR_AMOUNT_SHIFT         (0U)
#define NETC_SW_ENETC_HTMOR_AMOUNT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_HTMOR_AMOUNT_SHIFT)) & NETC_SW_ENETC_HTMOR_AMOUNT_MASK)

#define NETC_SW_ENETC_HTMOR_WATERMARK_MASK       (0xFFFF0000U)
#define NETC_SW_ENETC_HTMOR_WATERMARK_SHIFT      (16U)
#define NETC_SW_ENETC_HTMOR_WATERMARK(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_HTMOR_WATERMARK_SHIFT)) & NETC_SW_ENETC_HTMOR_WATERMARK_MASK)
/*! @} */

/*! @name ISIDCAPR - Ingress stream identification capability register */
/*! @{ */

#define NETC_SW_ENETC_ISIDCAPR_NUM_KC_MASK       (0x3U)
#define NETC_SW_ENETC_ISIDCAPR_NUM_KC_SHIFT      (0U)
#define NETC_SW_ENETC_ISIDCAPR_NUM_KC(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDCAPR_NUM_KC_SHIFT)) & NETC_SW_ENETC_ISIDCAPR_NUM_KC_MASK)

#define NETC_SW_ENETC_ISIDCAPR_NUM_PF_MASK       (0x1CU)
#define NETC_SW_ENETC_ISIDCAPR_NUM_PF_SHIFT      (2U)
#define NETC_SW_ENETC_ISIDCAPR_NUM_PF(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDCAPR_NUM_PF_SHIFT)) & NETC_SW_ENETC_ISIDCAPR_NUM_PF_MASK)

#define NETC_SW_ENETC_ISIDCAPR_MAX_KSIZE_MASK    (0x1F00U)
#define NETC_SW_ENETC_ISIDCAPR_MAX_KSIZE_SHIFT   (8U)
#define NETC_SW_ENETC_ISIDCAPR_MAX_KSIZE(x)      (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDCAPR_MAX_KSIZE_SHIFT)) & NETC_SW_ENETC_ISIDCAPR_MAX_KSIZE_MASK)

#define NETC_SW_ENETC_ISIDCAPR_UFT_MASK          (0x10000U)
#define NETC_SW_ENETC_ISIDCAPR_UFT_SHIFT         (16U)
#define NETC_SW_ENETC_ISIDCAPR_UFT(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDCAPR_UFT_SHIFT)) & NETC_SW_ENETC_ISIDCAPR_UFT_MASK)

#define NETC_SW_ENETC_ISIDCAPR_ETHFT_MASK        (0x20000U)
#define NETC_SW_ENETC_ISIDCAPR_ETHFT_SHIFT       (17U)
#define NETC_SW_ENETC_ISIDCAPR_ETHFT(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDCAPR_ETHFT_SHIFT)) & NETC_SW_ENETC_ISIDCAPR_ETHFT_MASK)
/*! @} */

/*! @name ISIDHTCAPR - Ingress stream identification hash table capability register */
/*! @{ */

#define NETC_SW_ENETC_ISIDHTCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_ISIDHTCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_ISIDHTCAPR_ACCESS_METH(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDHTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ISIDHTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name ISIDKC0OR - Ingress stream identification key construction 0 operational register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC0OR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_ISIDKC0OR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISIDKC0OR_NUM_ENTRIES(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0OR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISIDKC0OR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ISIDKC0OR_EN_MASK          (0x80000000U)
#define NETC_SW_ENETC_ISIDKC0OR_EN_SHIFT         (31U)
#define NETC_SW_ENETC_ISIDKC0OR_EN(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0OR_EN_SHIFT)) & NETC_SW_ENETC_ISIDKC0OR_EN_MASK)
/*! @} */

/*! @name ISIDKC0CR0 - Ingress stream identification key construction 0 configuration register 0 */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC0CR0_VALID_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC0CR0_VALID_SHIFT     (0U)
/*! VALID - Valid
 *  0b0..The entire key construction rule is not valid including any configuration payload key fields defined.
 *  0b1..The key construction rule is valid.
 */
#define NETC_SW_ENETC_ISIDKC0CR0_VALID(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_VALID_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_VALID_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_PORTP_MASK      (0x2U)
#define NETC_SW_ENETC_ISIDKC0CR0_PORTP_SHIFT     (1U)
/*! PORTP
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_PORTP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_PORTP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_PORTP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_SPMP_MASK       (0x4U)
#define NETC_SW_ENETC_ISIDKC0CR0_SPMP_SHIFT      (2U)
/*! SPMP - Switch Port Masquerading (flag) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_SPMP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_SPMP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_SPMP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_DMACP_MASK      (0x8U)
#define NETC_SW_ENETC_ISIDKC0CR0_DMACP_SHIFT     (3U)
/*! DMACP - Destination MAC (address) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_DMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_DMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_DMACP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_SMACP_MASK      (0x10U)
#define NETC_SW_ENETC_ISIDKC0CR0_SMACP_SHIFT     (4U)
/*! SMACP - Source MAC (address) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_SMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_SMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_SMACP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_OVIDP_MASK      (0x20U)
#define NETC_SW_ENETC_ISIDKC0CR0_OVIDP_SHIFT     (5U)
/*! OVIDP - Outer VID Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_OVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_OVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_OVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_OPCPP_MASK      (0x40U)
#define NETC_SW_ENETC_ISIDKC0CR0_OPCPP_SHIFT     (6U)
/*! OPCPP - Outer PCP Present
 *  0b0..Outer VLAN header's PCP field not present in the key
 *  0b1..Outer VLAN header's PCP field present in the key
 */
#define NETC_SW_ENETC_ISIDKC0CR0_OPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_OPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_OPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_IVIDP_MASK      (0x80U)
#define NETC_SW_ENETC_ISIDKC0CR0_IVIDP_SHIFT     (7U)
/*! IVIDP - Inner VID Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_IVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_IVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_IVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_IPCPP_MASK      (0x100U)
#define NETC_SW_ENETC_ISIDKC0CR0_IPCPP_SHIFT     (8U)
/*! IPCPP - Inner PCP Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_IPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_IPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_IPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_SQTP_MASK       (0x200U)
#define NETC_SW_ENETC_ISIDKC0CR0_SQTP_SHIFT      (9U)
/*! SQTP - Sequence Tag (code point) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_SQTP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_SQTP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_SQTP_MASK)

#define NETC_SW_ENETC_ISIDKC0CR0_ETP_MASK        (0x400U)
#define NETC_SW_ENETC_ISIDKC0CR0_ETP_SHIFT       (10U)
/*! ETP - EtherType Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC0CR0_ETP(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0CR0_ETP_SHIFT)) & NETC_SW_ENETC_ISIDKC0CR0_ETP_MASK)
/*! @} */

/*! @name ISIDKC0PF0CR - Ingress stream identification key construction 0 payload field 0 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC0PF0CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF0CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF0CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC0PF0CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC0PF0CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF0CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF0CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC0PF0CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF0CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF0CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC0PF0CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF0CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF0CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC0PF0CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC0PF0CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF0CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF0CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC0PF1CR - Ingress stream identification key construction 0 payload field 1 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC0PF1CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF1CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF1CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC0PF1CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC0PF1CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF1CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF1CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC0PF1CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF1CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF1CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC0PF1CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF1CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF1CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC0PF1CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC0PF1CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF1CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF1CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC0PF2CR - Ingress stream identification key construction 0 payload field 2 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC0PF2CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF2CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF2CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC0PF2CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC0PF2CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF2CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF2CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC0PF2CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF2CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF2CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC0PF2CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF2CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF2CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC0PF2CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC0PF2CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF2CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF2CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC0PF3CR - Ingress stream identification key construction 0 payload field 3 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC0PF3CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF3CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF3CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC0PF3CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC0PF3CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF3CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF3CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC0PF3CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF3CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF3CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC0PF3CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF3CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF3CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC0PF3CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC0PF3CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC0PF3CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC0PF3CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC1OR - Ingress stream identification key construction 1 operational register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC1OR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_ISIDKC1OR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISIDKC1OR_NUM_ENTRIES(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1OR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISIDKC1OR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ISIDKC1OR_EN_MASK          (0x80000000U)
#define NETC_SW_ENETC_ISIDKC1OR_EN_SHIFT         (31U)
#define NETC_SW_ENETC_ISIDKC1OR_EN(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1OR_EN_SHIFT)) & NETC_SW_ENETC_ISIDKC1OR_EN_MASK)
/*! @} */

/*! @name ISIDKC1CR0 - Ingress stream identification key construction 1 configuration register 0 */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC1CR0_VALID_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC1CR0_VALID_SHIFT     (0U)
/*! VALID - Valid
 *  0b0..The entire key construction rule is not valid including any configuration payload key fields defined.
 *  0b1..The key construction rule is valid.
 */
#define NETC_SW_ENETC_ISIDKC1CR0_VALID(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_VALID_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_VALID_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_PORTP_MASK      (0x2U)
#define NETC_SW_ENETC_ISIDKC1CR0_PORTP_SHIFT     (1U)
/*! PORTP
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_PORTP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_PORTP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_PORTP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_SPMP_MASK       (0x4U)
#define NETC_SW_ENETC_ISIDKC1CR0_SPMP_SHIFT      (2U)
/*! SPMP - Switch Port Masquerading (flag) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_SPMP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_SPMP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_SPMP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_DMACP_MASK      (0x8U)
#define NETC_SW_ENETC_ISIDKC1CR0_DMACP_SHIFT     (3U)
/*! DMACP - Destination MAC (address) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_DMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_DMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_DMACP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_SMACP_MASK      (0x10U)
#define NETC_SW_ENETC_ISIDKC1CR0_SMACP_SHIFT     (4U)
/*! SMACP - Source MAC (address) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_SMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_SMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_SMACP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_OVIDP_MASK      (0x20U)
#define NETC_SW_ENETC_ISIDKC1CR0_OVIDP_SHIFT     (5U)
/*! OVIDP - Outer VID Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_OVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_OVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_OVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_OPCPP_MASK      (0x40U)
#define NETC_SW_ENETC_ISIDKC1CR0_OPCPP_SHIFT     (6U)
/*! OPCPP - Outer PCP Present
 *  0b0..Outer VLAN header's PCP field not present in the key
 *  0b1..Outer VLAN header's PCP field present in the key
 */
#define NETC_SW_ENETC_ISIDKC1CR0_OPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_OPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_OPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_IVIDP_MASK      (0x80U)
#define NETC_SW_ENETC_ISIDKC1CR0_IVIDP_SHIFT     (7U)
/*! IVIDP - Inner VID Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_IVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_IVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_IVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_IPCPP_MASK      (0x100U)
#define NETC_SW_ENETC_ISIDKC1CR0_IPCPP_SHIFT     (8U)
/*! IPCPP - Inner PCP Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_IPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_IPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_IPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_SQTP_MASK       (0x200U)
#define NETC_SW_ENETC_ISIDKC1CR0_SQTP_SHIFT      (9U)
/*! SQTP - Sequence Tag (code point) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_SQTP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_SQTP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_SQTP_MASK)

#define NETC_SW_ENETC_ISIDKC1CR0_ETP_MASK        (0x400U)
#define NETC_SW_ENETC_ISIDKC1CR0_ETP_SHIFT       (10U)
/*! ETP - EtherType Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC1CR0_ETP(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1CR0_ETP_SHIFT)) & NETC_SW_ENETC_ISIDKC1CR0_ETP_MASK)
/*! @} */

/*! @name ISIDKC1PF0CR - Ingress stream identification key construction 1 payload field 0 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC1PF0CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF0CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF0CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC1PF0CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC1PF0CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF0CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF0CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC1PF0CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF0CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF0CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC1PF0CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF0CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF0CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC1PF0CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC1PF0CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF0CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF0CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC1PF1CR - Ingress stream identification key construction 1 payload field 1 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC1PF1CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF1CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF1CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC1PF1CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC1PF1CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF1CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF1CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC1PF1CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF1CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF1CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC1PF1CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF1CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF1CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC1PF1CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC1PF1CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF1CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF1CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC1PF2CR - Ingress stream identification key construction 1 payload field 2 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC1PF2CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF2CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF2CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC1PF2CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC1PF2CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF2CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF2CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC1PF2CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF2CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF2CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC1PF2CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF2CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF2CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC1PF2CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC1PF2CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF2CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF2CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC1PF3CR - Ingress stream identification key construction 1 payload field 3 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC1PF3CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF3CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF3CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC1PF3CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC1PF3CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF3CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF3CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC1PF3CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF3CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF3CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC1PF3CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF3CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF3CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC1PF3CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC1PF3CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC1PF3CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC1PF3CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC2OR - Ingress stream identification key construction 2 operational register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC2OR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_ISIDKC2OR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISIDKC2OR_NUM_ENTRIES(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2OR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISIDKC2OR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ISIDKC2OR_EN_MASK          (0x80000000U)
#define NETC_SW_ENETC_ISIDKC2OR_EN_SHIFT         (31U)
#define NETC_SW_ENETC_ISIDKC2OR_EN(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2OR_EN_SHIFT)) & NETC_SW_ENETC_ISIDKC2OR_EN_MASK)
/*! @} */

/*! @name ISIDKC2CR0 - Ingress stream identification key construction 2 configuration register 0 */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC2CR0_VALID_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC2CR0_VALID_SHIFT     (0U)
/*! VALID - Valid
 *  0b0..The entire key construction rule is not valid including any configuration payload key fields defined.
 *  0b1..The key construction rule is valid.
 */
#define NETC_SW_ENETC_ISIDKC2CR0_VALID(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_VALID_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_VALID_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_PORTP_MASK      (0x2U)
#define NETC_SW_ENETC_ISIDKC2CR0_PORTP_SHIFT     (1U)
/*! PORTP
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_PORTP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_PORTP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_PORTP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_SPMP_MASK       (0x4U)
#define NETC_SW_ENETC_ISIDKC2CR0_SPMP_SHIFT      (2U)
/*! SPMP - Switch Port Masquerading (flag) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_SPMP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_SPMP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_SPMP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_DMACP_MASK      (0x8U)
#define NETC_SW_ENETC_ISIDKC2CR0_DMACP_SHIFT     (3U)
/*! DMACP - Destination MAC (address) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_DMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_DMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_DMACP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_SMACP_MASK      (0x10U)
#define NETC_SW_ENETC_ISIDKC2CR0_SMACP_SHIFT     (4U)
/*! SMACP - Source MAC (address) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_SMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_SMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_SMACP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_OVIDP_MASK      (0x20U)
#define NETC_SW_ENETC_ISIDKC2CR0_OVIDP_SHIFT     (5U)
/*! OVIDP - Outer VID Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_OVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_OVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_OVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_OPCPP_MASK      (0x40U)
#define NETC_SW_ENETC_ISIDKC2CR0_OPCPP_SHIFT     (6U)
/*! OPCPP - Outer PCP Present
 *  0b0..Outer VLAN header's PCP field not present in the key
 *  0b1..Outer VLAN header's PCP field present in the key
 */
#define NETC_SW_ENETC_ISIDKC2CR0_OPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_OPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_OPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_IVIDP_MASK      (0x80U)
#define NETC_SW_ENETC_ISIDKC2CR0_IVIDP_SHIFT     (7U)
/*! IVIDP - Inner VID Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_IVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_IVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_IVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_IPCPP_MASK      (0x100U)
#define NETC_SW_ENETC_ISIDKC2CR0_IPCPP_SHIFT     (8U)
/*! IPCPP - Inner PCP Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_IPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_IPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_IPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_SQTP_MASK       (0x200U)
#define NETC_SW_ENETC_ISIDKC2CR0_SQTP_SHIFT      (9U)
/*! SQTP - Sequence Tag (code point) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_SQTP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_SQTP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_SQTP_MASK)

#define NETC_SW_ENETC_ISIDKC2CR0_ETP_MASK        (0x400U)
#define NETC_SW_ENETC_ISIDKC2CR0_ETP_SHIFT       (10U)
/*! ETP - EtherType Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC2CR0_ETP(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2CR0_ETP_SHIFT)) & NETC_SW_ENETC_ISIDKC2CR0_ETP_MASK)
/*! @} */

/*! @name ISIDKC2PF0CR - Ingress stream identification key construction 2 payload field 0 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC2PF0CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF0CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF0CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC2PF0CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC2PF0CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF0CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF0CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC2PF0CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF0CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF0CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC2PF0CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF0CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF0CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC2PF0CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC2PF0CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF0CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF0CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC2PF1CR - Ingress stream identification key construction 2 payload field 1 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC2PF1CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF1CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF1CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC2PF1CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC2PF1CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF1CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF1CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC2PF1CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF1CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF1CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC2PF1CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF1CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF1CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC2PF1CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC2PF1CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF1CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF1CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC2PF2CR - Ingress stream identification key construction 2 payload field 2 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC2PF2CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF2CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF2CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC2PF2CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC2PF2CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF2CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF2CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC2PF2CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF2CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF2CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC2PF2CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF2CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF2CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC2PF2CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC2PF2CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF2CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF2CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC2PF3CR - Ingress stream identification key construction 2 payload field 3 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC2PF3CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF3CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF3CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC2PF3CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC2PF3CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF3CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF3CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC2PF3CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF3CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF3CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC2PF3CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF3CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF3CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC2PF3CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC2PF3CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC2PF3CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC2PF3CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC3OR - Ingress stream identification key construction 3 operational register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC3OR_NUM_ENTRIES_MASK (0xFFFFU)
#define NETC_SW_ENETC_ISIDKC3OR_NUM_ENTRIES_SHIFT (0U)
#define NETC_SW_ENETC_ISIDKC3OR_NUM_ENTRIES(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3OR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISIDKC3OR_NUM_ENTRIES_MASK)

#define NETC_SW_ENETC_ISIDKC3OR_EN_MASK          (0x80000000U)
#define NETC_SW_ENETC_ISIDKC3OR_EN_SHIFT         (31U)
#define NETC_SW_ENETC_ISIDKC3OR_EN(x)            (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3OR_EN_SHIFT)) & NETC_SW_ENETC_ISIDKC3OR_EN_MASK)
/*! @} */

/*! @name ISIDKC3CR0 - Ingress stream identification key construction 3 configuration register 0 */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC3CR0_VALID_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC3CR0_VALID_SHIFT     (0U)
/*! VALID - Valid
 *  0b0..The entire key construction rule is not valid including any configuration payload key fields defined.
 *  0b1..The key construction rule is valid.
 */
#define NETC_SW_ENETC_ISIDKC3CR0_VALID(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_VALID_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_VALID_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_PORTP_MASK      (0x2U)
#define NETC_SW_ENETC_ISIDKC3CR0_PORTP_SHIFT     (1U)
/*! PORTP
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_PORTP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_PORTP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_PORTP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_SPMP_MASK       (0x4U)
#define NETC_SW_ENETC_ISIDKC3CR0_SPMP_SHIFT      (2U)
/*! SPMP - Switch Port Masquerading (flag) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_SPMP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_SPMP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_SPMP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_DMACP_MASK      (0x8U)
#define NETC_SW_ENETC_ISIDKC3CR0_DMACP_SHIFT     (3U)
/*! DMACP - Destination MAC (address) Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_DMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_DMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_DMACP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_SMACP_MASK      (0x10U)
#define NETC_SW_ENETC_ISIDKC3CR0_SMACP_SHIFT     (4U)
/*! SMACP - Source MAC (address) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_SMACP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_SMACP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_SMACP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_OVIDP_MASK      (0x20U)
#define NETC_SW_ENETC_ISIDKC3CR0_OVIDP_SHIFT     (5U)
/*! OVIDP - Outer VID Present
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_OVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_OVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_OVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_OPCPP_MASK      (0x40U)
#define NETC_SW_ENETC_ISIDKC3CR0_OPCPP_SHIFT     (6U)
/*! OPCPP - Outer PCP Present
 *  0b0..Outer VLAN header's PCP field not present in the key
 *  0b1..Outer VLAN header's PCP field present in the key
 */
#define NETC_SW_ENETC_ISIDKC3CR0_OPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_OPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_OPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_IVIDP_MASK      (0x80U)
#define NETC_SW_ENETC_ISIDKC3CR0_IVIDP_SHIFT     (7U)
/*! IVIDP - Inner VID Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_IVIDP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_IVIDP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_IVIDP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_IPCPP_MASK      (0x100U)
#define NETC_SW_ENETC_ISIDKC3CR0_IPCPP_SHIFT     (8U)
/*! IPCPP - Inner PCP Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_IPCPP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_IPCPP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_IPCPP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_SQTP_MASK       (0x200U)
#define NETC_SW_ENETC_ISIDKC3CR0_SQTP_SHIFT      (9U)
/*! SQTP - Sequence Tag (code point) Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_SQTP(x)         (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_SQTP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_SQTP_MASK)

#define NETC_SW_ENETC_ISIDKC3CR0_ETP_MASK        (0x400U)
#define NETC_SW_ENETC_ISIDKC3CR0_ETP_SHIFT       (10U)
/*! ETP - EtherType Present.
 *  0b0..Not present
 *  0b1..Present
 */
#define NETC_SW_ENETC_ISIDKC3CR0_ETP(x)          (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3CR0_ETP_SHIFT)) & NETC_SW_ENETC_ISIDKC3CR0_ETP_MASK)
/*! @} */

/*! @name ISIDKC3PF0CR - Ingress stream identification key construction 3 payload field 0 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC3PF0CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF0CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF0CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC3PF0CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC3PF0CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF0CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF0CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC3PF0CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF0CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF0CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC3PF0CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF0CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF0CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC3PF0CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC3PF0CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF0CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF0CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC3PF1CR - Ingress stream identification key construction 3 payload field 1 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC3PF1CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF1CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF1CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC3PF1CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC3PF1CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF1CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF1CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC3PF1CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF1CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF1CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC3PF1CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF1CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF1CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC3PF1CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC3PF1CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF1CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF1CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC3PF2CR - Ingress stream identification key construction 3 payload field 2 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC3PF2CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF2CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF2CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC3PF2CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC3PF2CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF2CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF2CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC3PF2CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF2CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF2CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC3PF2CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF2CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF2CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC3PF2CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC3PF2CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF2CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF2CR_LBMASK_MASK)
/*! @} */

/*! @name ISIDKC3PF3CR - Ingress stream identification key construction 3 payload field 3 configuration register */
/*! @{ */

#define NETC_SW_ENETC_ISIDKC3PF3CR_PFP_MASK      (0x1U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_PFP_SHIFT     (0U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_PFP(x)        (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF3CR_PFP_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF3CR_PFP_MASK)

#define NETC_SW_ENETC_ISIDKC3PF3CR_NUM_BYTES_MASK (0x1EU)
#define NETC_SW_ENETC_ISIDKC3PF3CR_NUM_BYTES_SHIFT (1U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_NUM_BYTES(x)  (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF3CR_NUM_BYTES_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF3CR_NUM_BYTES_MASK)

#define NETC_SW_ENETC_ISIDKC3PF3CR_BYTE_OFFSET_MASK (0x7F00U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_BYTE_OFFSET_SHIFT (8U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_BYTE_OFFSET(x) (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF3CR_BYTE_OFFSET_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF3CR_BYTE_OFFSET_MASK)

#define NETC_SW_ENETC_ISIDKC3PF3CR_FBMASK_MASK   (0x70000U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_FBMASK_SHIFT  (16U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_FBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF3CR_FBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF3CR_FBMASK_MASK)

#define NETC_SW_ENETC_ISIDKC3PF3CR_LBMASK_MASK   (0x700000U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_LBMASK_SHIFT  (20U)
#define NETC_SW_ENETC_ISIDKC3PF3CR_LBMASK(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISIDKC3PF3CR_LBMASK_SHIFT)) & NETC_SW_ENETC_ISIDKC3PF3CR_LBMASK_MASK)
/*! @} */

/*! @name ISFHTCAPR - Ingress stream filter hash table capability register */
/*! @{ */

#define NETC_SW_ENETC_ISFHTCAPR_ACCESS_METH_MASK (0xF00000U)
#define NETC_SW_ENETC_ISFHTCAPR_ACCESS_METH_SHIFT (20U)
#define NETC_SW_ENETC_ISFHTCAPR_ACCESS_METH(x)   (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISFHTCAPR_ACCESS_METH_SHIFT)) & NETC_SW_ENETC_ISFHTCAPR_ACCESS_METH_MASK)
/*! @} */

/*! @name ISFHTOR - Ingress stream filter hash table operational register */
/*! @{ */

#define NETC_SW_ENETC_ISFHTOR_NUM_ENTRIES_MASK   (0xFFFFU)
#define NETC_SW_ENETC_ISFHTOR_NUM_ENTRIES_SHIFT  (0U)
#define NETC_SW_ENETC_ISFHTOR_NUM_ENTRIES(x)     (((uint32_t)(((uint32_t)(x)) << NETC_SW_ENETC_ISFHTOR_NUM_ENTRIES_SHIFT)) & NETC_SW_ENETC_ISFHTOR_NUM_ENTRIES_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group NETC_SW_ENETC_Register_Masks */


/* NETC_SW_ENETC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral ENETC0_COMMON base address */
  #define ENETC0_COMMON_BASE                       (0x70B10000u)
  /** Peripheral ENETC0_COMMON base address */
  #define ENETC0_COMMON_BASE_NS                    (0x60B10000u)
  /** Peripheral ENETC0_COMMON base pointer */
  #define ENETC0_COMMON                            ((NETC_SW_ENETC_Type *)ENETC0_COMMON_BASE)
  /** Peripheral ENETC0_COMMON base pointer */
  #define ENETC0_COMMON_NS                         ((NETC_SW_ENETC_Type *)ENETC0_COMMON_BASE_NS)
  /** Peripheral ENETC1_COMMON base address */
  #define ENETC1_COMMON_BASE                       (0x70B50000u)
  /** Peripheral ENETC1_COMMON base address */
  #define ENETC1_COMMON_BASE_NS                    (0x60B50000u)
  /** Peripheral ENETC1_COMMON base pointer */
  #define ENETC1_COMMON                            ((NETC_SW_ENETC_Type *)ENETC1_COMMON_BASE)
  /** Peripheral ENETC1_COMMON base pointer */
  #define ENETC1_COMMON_NS                         ((NETC_SW_ENETC_Type *)ENETC1_COMMON_BASE_NS)
  /** Peripheral SW0_COMMON base address */
  #define SW0_COMMON_BASE                          (0x70A00000u)
  /** Peripheral SW0_COMMON base address */
  #define SW0_COMMON_BASE_NS                       (0x60A00000u)
  /** Peripheral SW0_COMMON base pointer */
  #define SW0_COMMON                               ((NETC_SW_ENETC_Type *)SW0_COMMON_BASE)
  /** Peripheral SW0_COMMON base pointer */
  #define SW0_COMMON_NS                            ((NETC_SW_ENETC_Type *)SW0_COMMON_BASE_NS)
  /** Array initializer of NETC_SW_ENETC peripheral base addresses */
  #define NETC_SW_ENETC_BASE_ADDRS                 { ENETC0_COMMON_BASE, ENETC1_COMMON_BASE, SW0_COMMON_BASE }
  /** Array initializer of NETC_SW_ENETC peripheral base pointers */
  #define NETC_SW_ENETC_BASE_PTRS                  { ENETC0_COMMON, ENETC1_COMMON, SW0_COMMON }
  /** Array initializer of NETC_SW_ENETC peripheral base addresses */
  #define NETC_SW_ENETC_BASE_ADDRS_NS              { ENETC0_COMMON_BASE_NS, ENETC1_COMMON_BASE_NS, SW0_COMMON_BASE_NS }
  /** Array initializer of NETC_SW_ENETC peripheral base pointers */
  #define NETC_SW_ENETC_BASE_PTRS_NS               { ENETC0_COMMON_NS, ENETC1_COMMON_NS, SW0_COMMON_NS }
#else
  /** Peripheral ENETC0_COMMON base address */
  #define ENETC0_COMMON_BASE                       (0x60B10000u)
  /** Peripheral ENETC0_COMMON base pointer */
  #define ENETC0_COMMON                            ((NETC_SW_ENETC_Type *)ENETC0_COMMON_BASE)
  /** Peripheral ENETC1_COMMON base address */
  #define ENETC1_COMMON_BASE                       (0x60B50000u)
  /** Peripheral ENETC1_COMMON base pointer */
  #define ENETC1_COMMON                            ((NETC_SW_ENETC_Type *)ENETC1_COMMON_BASE)
  /** Peripheral SW0_COMMON base address */
  #define SW0_COMMON_BASE                          (0x60A00000u)
  /** Peripheral SW0_COMMON base pointer */
  #define SW0_COMMON                               ((NETC_SW_ENETC_Type *)SW0_COMMON_BASE)
  /** Array initializer of NETC_SW_ENETC peripheral base addresses */
  #define NETC_SW_ENETC_BASE_ADDRS                 { ENETC0_COMMON_BASE, ENETC1_COMMON_BASE, SW0_COMMON_BASE }
  /** Array initializer of NETC_SW_ENETC peripheral base pointers */
  #define NETC_SW_ENETC_BASE_PTRS                  { ENETC0_COMMON, ENETC1_COMMON, SW0_COMMON }
#endif

/*!
 * @}
 */ /* end of group NETC_SW_ENETC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- OCOTP_FSB Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup OCOTP_FSB_Peripheral_Access_Layer OCOTP_FSB Peripheral Access Layer
 * @{
 */

/** OCOTP_FSB - Register Layout Typedef */
typedef struct {
  __IO uint32_t OTP_SHADOW_PARTA[52];              /**< OTP shadow register, array offset: 0x0, array step: 0x4 */
       uint8_t RESERVED_0[1040];
  __IO uint32_t OTP_SHADOW_PARTB[200];             /**< OTP shadow register, array offset: 0x4E0, array step: 0x4 */
} OCOTP_FSB_Type;

/* ----------------------------------------------------------------------------
   -- OCOTP_FSB Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup OCOTP_FSB_Register_Masks OCOTP_FSB Register Masks
 * @{
 */

/*! @name OTP_SHADOW_PARTA - OTP shadow register */
/*! @{ */

#define OCOTP_FSB_OTP_SHADOW_PARTA_SHADOW_MASK   (0xFFFFFFFFU)
#define OCOTP_FSB_OTP_SHADOW_PARTA_SHADOW_SHIFT  (0U)
/*! SHADOW - OTP shadow register, fsb have read access of shadow 0-51 (offset should be 0*4-51*4) */
#define OCOTP_FSB_OTP_SHADOW_PARTA_SHADOW(x)     (((uint32_t)(((uint32_t)(x)) << OCOTP_FSB_OTP_SHADOW_PARTA_SHADOW_SHIFT)) & OCOTP_FSB_OTP_SHADOW_PARTA_SHADOW_MASK)
/*! @} */

/* The count of OCOTP_FSB_OTP_SHADOW_PARTA */
#define OCOTP_FSB_OTP_SHADOW_PARTA_COUNT         (52U)

/*! @name OTP_SHADOW_PARTB - OTP shadow register */
/*! @{ */

#define OCOTP_FSB_OTP_SHADOW_PARTB_SHADOW_MASK   (0xFFFFFFFFU)
#define OCOTP_FSB_OTP_SHADOW_PARTB_SHADOW_SHIFT  (0U)
/*! SHADOW - OTP shadow register, fsb have read access of shadow 312-511 (offset should be 312*4-511*4) */
#define OCOTP_FSB_OTP_SHADOW_PARTB_SHADOW(x)     (((uint32_t)(((uint32_t)(x)) << OCOTP_FSB_OTP_SHADOW_PARTB_SHADOW_SHIFT)) & OCOTP_FSB_OTP_SHADOW_PARTB_SHADOW_MASK)
/*! @} */

/* The count of OCOTP_FSB_OTP_SHADOW_PARTB */
#define OCOTP_FSB_OTP_SHADOW_PARTB_COUNT         (200U)


/*!
 * @}
 */ /* end of group OCOTP_FSB_Register_Masks */


/* OCOTP_FSB - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral OCOTP_FSB base address */
  #define OCOTP_FSB_BASE                           (0x57518000u)
  /** Peripheral OCOTP_FSB base address */
  #define OCOTP_FSB_BASE_NS                        (0x47518000u)
  /** Peripheral OCOTP_FSB base pointer */
  #define OCOTP_FSB                                ((OCOTP_FSB_Type *)OCOTP_FSB_BASE)
  /** Peripheral OCOTP_FSB base pointer */
  #define OCOTP_FSB_NS                             ((OCOTP_FSB_Type *)OCOTP_FSB_BASE_NS)
  /** Array initializer of OCOTP_FSB peripheral base addresses */
  #define OCOTP_FSB_BASE_ADDRS                     { OCOTP_FSB_BASE }
  /** Array initializer of OCOTP_FSB peripheral base pointers */
  #define OCOTP_FSB_BASE_PTRS                      { OCOTP_FSB }
  /** Array initializer of OCOTP_FSB peripheral base addresses */
  #define OCOTP_FSB_BASE_ADDRS_NS                  { OCOTP_FSB_BASE_NS }
  /** Array initializer of OCOTP_FSB peripheral base pointers */
  #define OCOTP_FSB_BASE_PTRS_NS                   { OCOTP_FSB_NS }
#else
  /** Peripheral OCOTP_FSB base address */
  #define OCOTP_FSB_BASE                           (0x47518000u)
  /** Peripheral OCOTP_FSB base pointer */
  #define OCOTP_FSB                                ((OCOTP_FSB_Type *)OCOTP_FSB_BASE)
  /** Array initializer of OCOTP_FSB peripheral base addresses */
  #define OCOTP_FSB_BASE_ADDRS                     { OCOTP_FSB_BASE }
  /** Array initializer of OCOTP_FSB peripheral base pointers */
  #define OCOTP_FSB_BASE_PTRS                      { OCOTP_FSB }
#endif

/*!
 * @}
 */ /* end of group OCOTP_FSB_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- OSC_RC_400M Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup OSC_RC_400M_Peripheral_Access_Layer OSC_RC_400M Peripheral Access Layer
 * @{
 */

/** OSC_RC_400M - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0 */
    __IO uint32_t RW;                                /**< Analog Control Register CTRL0, offset: 0x0 */
    __IO uint32_t SET;                               /**< Analog Control Register CTRL0, offset: 0x4 */
    __IO uint32_t CLR;                               /**< Analog Control Register CTRL0, offset: 0x8 */
    __IO uint32_t TOG;                               /**< Analog Control Register CTRL0, offset: 0xC */
  } CTRL0;
  struct {                                         /* offset: 0x10 */
    __IO uint32_t RW;                                /**< Analog Control Register CTRL1, offset: 0x10 */
    __IO uint32_t SET;                               /**< Analog Control Register CTRL1, offset: 0x14 */
    __IO uint32_t CLR;                               /**< Analog Control Register CTRL1, offset: 0x18 */
    __IO uint32_t TOG;                               /**< Analog Control Register CTRL1, offset: 0x1C */
  } CTRL1;
  struct {                                         /* offset: 0x20 */
    __IO uint32_t RW;                                /**< Analog Control Register CTRL2, offset: 0x20 */
    __IO uint32_t SET;                               /**< Analog Control Register CTRL2, offset: 0x24 */
    __IO uint32_t CLR;                               /**< Analog Control Register CTRL2, offset: 0x28 */
    __IO uint32_t TOG;                               /**< Analog Control Register CTRL2, offset: 0x2C */
  } CTRL2;
  struct {                                         /* offset: 0x30 */
    __IO uint32_t RW;                                /**< Analog Control Register CTRL3, offset: 0x30 */
    __IO uint32_t SET;                               /**< Analog Control Register CTRL3, offset: 0x34 */
    __IO uint32_t CLR;                               /**< Analog Control Register CTRL3, offset: 0x38 */
    __IO uint32_t TOG;                               /**< Analog Control Register CTRL3, offset: 0x3C */
  } CTRL3;
       uint8_t RESERVED_0[16];
  __I  uint32_t STAT0;                             /**< Analog Status Register STAT0, offset: 0x50 */
       uint8_t RESERVED_1[12];
  __I  uint32_t STAT1;                             /**< Analog Status Register STAT1, offset: 0x60 */
       uint8_t RESERVED_2[12];
  __I  uint32_t STAT2;                             /**< Analog Status Register STAT2, offset: 0x70 */
} OSC_RC_400M_Type;

/* ----------------------------------------------------------------------------
   -- OSC_RC_400M Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup OSC_RC_400M_Register_Masks OSC_RC_400M Register Masks
 * @{
 */

/*! @name CTRL0 - Analog Control Register CTRL0 */
/*! @{ */

#define OSC_RC_400M_CTRL0_REF_CLK_DIV_MASK       (0x3F000000U)
#define OSC_RC_400M_CTRL0_REF_CLK_DIV_SHIFT      (24U)
/*! REF_CLK_DIV - Divide value for ref_clk to generate slow_clk. */
#define OSC_RC_400M_CTRL0_REF_CLK_DIV(x)         (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL0_REF_CLK_DIV_SHIFT)) & OSC_RC_400M_CTRL0_REF_CLK_DIV_MASK)
/*! @} */

/*! @name CTRL1 - Analog Control Register CTRL1 */
/*! @{ */

#define OSC_RC_400M_CTRL1_HYST_MINUS_MASK        (0xFU)
#define OSC_RC_400M_CTRL1_HYST_MINUS_SHIFT       (0U)
/*! HYST_MINUS - Negative hysteresis value for the tuned clock. */
#define OSC_RC_400M_CTRL1_HYST_MINUS(x)          (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL1_HYST_MINUS_SHIFT)) & OSC_RC_400M_CTRL1_HYST_MINUS_MASK)

#define OSC_RC_400M_CTRL1_HYST_PLUS_MASK         (0xF00U)
#define OSC_RC_400M_CTRL1_HYST_PLUS_SHIFT        (8U)
/*! HYST_PLUS - Positive hysteresis value for the tuned clock. */
#define OSC_RC_400M_CTRL1_HYST_PLUS(x)           (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL1_HYST_PLUS_SHIFT)) & OSC_RC_400M_CTRL1_HYST_PLUS_MASK)

#define OSC_RC_400M_CTRL1_TARGET_COUNT_MASK      (0xFFFF0000U)
#define OSC_RC_400M_CTRL1_TARGET_COUNT_SHIFT     (16U)
/*! TARGET_COUNT - Target count for the fast clock. */
#define OSC_RC_400M_CTRL1_TARGET_COUNT(x)        (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL1_TARGET_COUNT_SHIFT)) & OSC_RC_400M_CTRL1_TARGET_COUNT_MASK)
/*! @} */

/*! @name CTRL2 - Analog Control Register CTRL2 */
/*! @{ */

#define OSC_RC_400M_CTRL2_TUNE_INV_MASK          (0x100U)
#define OSC_RC_400M_CTRL2_TUNE_INV_SHIFT         (8U)
/*! TUNE_INV - Inverse tuning direction. */
#define OSC_RC_400M_CTRL2_TUNE_INV(x)            (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL2_TUNE_INV_SHIFT)) & OSC_RC_400M_CTRL2_TUNE_INV_MASK)

#define OSC_RC_400M_CTRL2_TUNE_BYP_MASK          (0x400U)
#define OSC_RC_400M_CTRL2_TUNE_BYP_SHIFT         (10U)
/*! TUNE_BYP - Bypass the tuning logic */
#define OSC_RC_400M_CTRL2_TUNE_BYP(x)            (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL2_TUNE_BYP_SHIFT)) & OSC_RC_400M_CTRL2_TUNE_BYP_MASK)

#define OSC_RC_400M_CTRL2_TUNE_EN_MASK           (0x1000U)
#define OSC_RC_400M_CTRL2_TUNE_EN_SHIFT          (12U)
/*! TUNE_EN - Freeze/Unfreeze the tuning value. */
#define OSC_RC_400M_CTRL2_TUNE_EN(x)             (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL2_TUNE_EN_SHIFT)) & OSC_RC_400M_CTRL2_TUNE_EN_MASK)

#define OSC_RC_400M_CTRL2_TUNE_START_MASK        (0x4000U)
#define OSC_RC_400M_CTRL2_TUNE_START_SHIFT       (14U)
/*! TUNE_START - Start/Stop tuning. */
#define OSC_RC_400M_CTRL2_TUNE_START(x)          (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL2_TUNE_START_SHIFT)) & OSC_RC_400M_CTRL2_TUNE_START_MASK)

#define OSC_RC_400M_CTRL2_OSC_TUNE_VAL_MASK      (0xFF000000U)
#define OSC_RC_400M_CTRL2_OSC_TUNE_VAL_SHIFT     (24U)
/*! OSC_TUNE_VAL - Program the oscillator frequency. */
#define OSC_RC_400M_CTRL2_OSC_TUNE_VAL(x)        (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL2_OSC_TUNE_VAL_SHIFT)) & OSC_RC_400M_CTRL2_OSC_TUNE_VAL_MASK)
/*! @} */

/*! @name CTRL3 - Analog Control Register CTRL3 */
/*! @{ */

#define OSC_RC_400M_CTRL3_CLR_ERR_MASK           (0x1U)
#define OSC_RC_400M_CTRL3_CLR_ERR_SHIFT          (0U)
/*! CLR_ERR - Clear the error flag CLK1M_ERR
 *  0b0..Normal operation
 *  0b1..Clears the error flag CLK1M_ERR in status register STAT0.
 */
#define OSC_RC_400M_CTRL3_CLR_ERR(x)             (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL3_CLR_ERR_SHIFT)) & OSC_RC_400M_CTRL3_CLR_ERR_MASK)

#define OSC_RC_400M_CTRL3_EN_1M_CLK_MASK         (0x100U)
#define OSC_RC_400M_CTRL3_EN_1M_CLK_SHIFT        (8U)
/*! EN_1M_CLK - 1: Disable clk_1m_out. */
#define OSC_RC_400M_CTRL3_EN_1M_CLK(x)           (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL3_EN_1M_CLK_SHIFT)) & OSC_RC_400M_CTRL3_EN_1M_CLK_MASK)

#define OSC_RC_400M_CTRL3_MUX_1M_CLK_MASK        (0x400U)
#define OSC_RC_400M_CTRL3_MUX_1M_CLK_SHIFT       (10U)
/*! MUX_1M_CLK - Select free/locked 1MHz output */
#define OSC_RC_400M_CTRL3_MUX_1M_CLK(x)          (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL3_MUX_1M_CLK_SHIFT)) & OSC_RC_400M_CTRL3_MUX_1M_CLK_MASK)

#define OSC_RC_400M_CTRL3_COUNT_1M_CLK_MASK      (0xFFFF0000U)
#define OSC_RC_400M_CTRL3_COUNT_1M_CLK_SHIFT     (16U)
/*! COUNT_1M_CLK - Count for the locked clk_1m_out. */
#define OSC_RC_400M_CTRL3_COUNT_1M_CLK(x)        (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_CTRL3_COUNT_1M_CLK_SHIFT)) & OSC_RC_400M_CTRL3_COUNT_1M_CLK_MASK)
/*! @} */

/*! @name STAT0 - Analog Status Register STAT0 */
/*! @{ */

#define OSC_RC_400M_STAT0_CLK1M_ERR_MASK         (0x1U)
#define OSC_RC_400M_STAT0_CLK1M_ERR_SHIFT        (0U)
/*! CLK1M_ERR - Error flag for clk_1m_locked.
 *  0b0..No error.
 *  0b1..Indicates error.
 */
#define OSC_RC_400M_STAT0_CLK1M_ERR(x)           (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_STAT0_CLK1M_ERR_SHIFT)) & OSC_RC_400M_STAT0_CLK1M_ERR_MASK)
/*! @} */

/*! @name STAT1 - Analog Status Register STAT1 */
/*! @{ */

#define OSC_RC_400M_STAT1_CURR_COUNT_VAL_MASK    (0xFFFF0000U)
#define OSC_RC_400M_STAT1_CURR_COUNT_VAL_SHIFT   (16U)
/*! CURR_COUNT_VAL - Current count for the fast clock. */
#define OSC_RC_400M_STAT1_CURR_COUNT_VAL(x)      (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_STAT1_CURR_COUNT_VAL_SHIFT)) & OSC_RC_400M_STAT1_CURR_COUNT_VAL_MASK)
/*! @} */

/*! @name STAT2 - Analog Status Register STAT2 */
/*! @{ */

#define OSC_RC_400M_STAT2_CURR_OSC_TUNE_VAL_MASK (0xFF000000U)
#define OSC_RC_400M_STAT2_CURR_OSC_TUNE_VAL_SHIFT (24U)
/*! CURR_OSC_TUNE_VAL - Current tuning value used by oscillator. */
#define OSC_RC_400M_STAT2_CURR_OSC_TUNE_VAL(x)   (((uint32_t)(((uint32_t)(x)) << OSC_RC_400M_STAT2_CURR_OSC_TUNE_VAL_SHIFT)) & OSC_RC_400M_STAT2_CURR_OSC_TUNE_VAL_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group OSC_RC_400M_Register_Masks */


/* OSC_RC_400M - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral OSC_RC_400M base address */
  #define OSC_RC_400M_BASE                         (0x54484380u)
  /** Peripheral OSC_RC_400M base address */
  #define OSC_RC_400M_BASE_NS                      (0x44484380u)
  /** Peripheral OSC_RC_400M base pointer */
  #define OSC_RC_400M                              ((OSC_RC_400M_Type *)OSC_RC_400M_BASE)
  /** Peripheral OSC_RC_400M base pointer */
  #define OSC_RC_400M_NS                           ((OSC_RC_400M_Type *)OSC_RC_400M_BASE_NS)
  /** Array initializer of OSC_RC_400M peripheral base addresses */
  #define OSC_RC_400M_BASE_ADDRS                   { OSC_RC_400M_BASE }
  /** Array initializer of OSC_RC_400M peripheral base pointers */
  #define OSC_RC_400M_BASE_PTRS                    { OSC_RC_400M }
  /** Array initializer of OSC_RC_400M peripheral base addresses */
  #define OSC_RC_400M_BASE_ADDRS_NS                { OSC_RC_400M_BASE_NS }
  /** Array initializer of OSC_RC_400M peripheral base pointers */
  #define OSC_RC_400M_BASE_PTRS_NS                 { OSC_RC_400M_NS }
#else
  /** Peripheral OSC_RC_400M base address */
  #define OSC_RC_400M_BASE                         (0x44484380u)
  /** Peripheral OSC_RC_400M base pointer */
  #define OSC_RC_400M                              ((OSC_RC_400M_Type *)OSC_RC_400M_BASE)
  /** Array initializer of OSC_RC_400M peripheral base addresses */
  #define OSC_RC_400M_BASE_ADDRS                   { OSC_RC_400M_BASE }
  /** Array initializer of OSC_RC_400M peripheral base pointers */
  #define OSC_RC_400M_BASE_PTRS                    { OSC_RC_400M }
#endif

/*!
 * @}
 */ /* end of group OSC_RC_400M_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- OTFAD Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup OTFAD_Peripheral_Access_Layer OTFAD Peripheral Access Layer
 * @{
 */

/** OTFAD - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[3072];
  __IO uint32_t CR;                                /**< Control Register, offset: 0xC00 */
  __IO uint32_t SR;                                /**< Status Register, offset: 0xC04 */
       uint8_t RESERVED_1[248];
  struct {                                         /* offset: 0xD00, array step: 0x40 */
    __IO uint32_t KEY[4];                            /**< AES Key Word, array offset: 0xD00, array step: index*0x40, index2*0x4 */
    __IO uint32_t CTR[2];                            /**< AES Counter Word, array offset: 0xD10, array step: index*0x40, index2*0x4 */
    __IO uint32_t RGD_W0;                            /**< AES Region Descriptor Word0, array offset: 0xD18, array step: 0x40 */
    __IO uint32_t RGD_W1;                            /**< AES Region Descriptor Word1, array offset: 0xD1C, array step: 0x40 */
         uint8_t RESERVED_0[32];
  } CTX[4];
} OTFAD_Type;

/* ----------------------------------------------------------------------------
   -- OTFAD Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup OTFAD_Register_Masks OTFAD Register Masks
 * @{
 */

/*! @name CR - Control Register */
/*! @{ */

#define OTFAD_CR_FERR_MASK                       (0x2U)
#define OTFAD_CR_FERR_SHIFT                      (1U)
/*! FERR - Force Error
 *  0b0..No effect on the SR[KBERE] indicator.
 *  0b1..SR[KBERR] is immediately set after a write with this data bit set.
 */
#define OTFAD_CR_FERR(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_CR_FERR_SHIFT)) & OTFAD_CR_FERR_MASK)

#define OTFAD_CR_FLDM_MASK                       (0x8U)
#define OTFAD_CR_FLDM_SHIFT                      (3U)
/*! FLDM - Force Logically Disabled Mode
 *  0b0..No effect on the operating mode.
 *  0b1..Force entry into LDM after a write with this data bit set. SR[MODE] signals the operating mode.
 */
#define OTFAD_CR_FLDM(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_CR_FLDM_SHIFT)) & OTFAD_CR_FLDM_MASK)

#define OTFAD_CR_KBSE_MASK                       (0x10U)
#define OTFAD_CR_KBSE_SHIFT                      (4U)
/*! KBSE - Key Blob Scramble Enable
 *  0b0..Key blob KEK scrambling is disabled.
 *  0b1..Key blob KEK scrambling is enabled.
 */
#define OTFAD_CR_KBSE(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_CR_KBSE_SHIFT)) & OTFAD_CR_KBSE_MASK)

#define OTFAD_CR_KBPE_MASK                       (0x20U)
#define OTFAD_CR_KBPE_SHIFT                      (5U)
/*! KBPE - Key Blob Processing Enable
 *  0b0..Key blob processing is disabled.
 *  0b1..Key blob processing is enabled.
 */
#define OTFAD_CR_KBPE(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_CR_KBPE_SHIFT)) & OTFAD_CR_KBPE_MASK)

#define OTFAD_CR_RRAE_MASK                       (0x80U)
#define OTFAD_CR_RRAE_SHIFT                      (7U)
/*! RRAE - Restricted Register Access Enable
 *  0b0..Register access is fully enabled. The OTFAD programming model registers can be accessed "normally".
 *  0b1..Register access is restricted and only the CR, SR and optional MDPC registers can be accessed; others are treated as RAZ/WI.
 */
#define OTFAD_CR_RRAE(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_CR_RRAE_SHIFT)) & OTFAD_CR_RRAE_MASK)

#define OTFAD_CR_SKBP_MASK                       (0x40000000U)
#define OTFAD_CR_SKBP_SHIFT                      (30U)
/*! SKBP - Start key blob processing
 *  0b0..Key blob processing is not initiated.
 *  0b1..Properly-enabled key blob processing is initiated.
 */
#define OTFAD_CR_SKBP(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_CR_SKBP_SHIFT)) & OTFAD_CR_SKBP_MASK)

#define OTFAD_CR_GE_MASK                         (0x80000000U)
#define OTFAD_CR_GE_SHIFT                        (31U)
/*! GE - Global OTFAD Enable
 *  0b0..OTFAD has decryption disabled. All data fetched by the QuadSPI bypasses OTFAD processing.
 *  0b1..OTFAD has decryption enabled, and processes data fetched by the QuadSPI as defined by the hardware configuration.
 */
#define OTFAD_CR_GE(x)                           (((uint32_t)(((uint32_t)(x)) << OTFAD_CR_GE_SHIFT)) & OTFAD_CR_GE_MASK)
/*! @} */

/*! @name SR - Status Register */
/*! @{ */

#define OTFAD_SR_KBERR_MASK                      (0x1U)
#define OTFAD_SR_KBERR_SHIFT                     (0U)
/*! KBERR - Key Blob Error
 *  0b0..No key blob error detected.
 *  0b1..One or more key blob errors has been detected.
 */
#define OTFAD_SR_KBERR(x)                        (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_KBERR_SHIFT)) & OTFAD_SR_KBERR_MASK)

#define OTFAD_SR_MDPCP_MASK                      (0x2U)
#define OTFAD_SR_MDPCP_SHIFT                     (1U)
/*! MDPCP - MDPC Present */
#define OTFAD_SR_MDPCP(x)                        (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_MDPCP_SHIFT)) & OTFAD_SR_MDPCP_MASK)

#define OTFAD_SR_MODE_MASK                       (0xCU)
#define OTFAD_SR_MODE_SHIFT                      (2U)
/*! MODE - Operating Mode
 *  0b00..Operating in Normal mode (NRM)
 *  0b01..Unused (reserved)
 *  0b10..Unused (reserved)
 *  0b11..Operating in Logically Disabled Mode (LDM)
 */
#define OTFAD_SR_MODE(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_MODE_SHIFT)) & OTFAD_SR_MODE_MASK)

#define OTFAD_SR_NCTX_MASK                       (0xF0U)
#define OTFAD_SR_NCTX_SHIFT                      (4U)
/*! NCTX - Number of Contexts */
#define OTFAD_SR_NCTX(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_NCTX_SHIFT)) & OTFAD_SR_NCTX_MASK)

#define OTFAD_SR_CTXER0_MASK                     (0x100U)
#define OTFAD_SR_CTXER0_SHIFT                    (8U)
/*! CTXER0 - Context Error
 *  0b0..No key blob error was detected for context "n".
 *  0b1..A key blob integrity error might have been detected in context "n".
 */
#define OTFAD_SR_CTXER0(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXER0_SHIFT)) & OTFAD_SR_CTXER0_MASK)

#define OTFAD_SR_CTXER1_MASK                     (0x200U)
#define OTFAD_SR_CTXER1_SHIFT                    (9U)
/*! CTXER1 - Context Error
 *  0b0..No key blob error was detected for context "n".
 *  0b1..A key blob integrity error might have been detected in context "n".
 */
#define OTFAD_SR_CTXER1(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXER1_SHIFT)) & OTFAD_SR_CTXER1_MASK)

#define OTFAD_SR_CTXER2_MASK                     (0x400U)
#define OTFAD_SR_CTXER2_SHIFT                    (10U)
/*! CTXER2 - Context Error
 *  0b0..No key blob error was detected for context "n".
 *  0b1..A key blob integrity error might have been detected in context "n".
 */
#define OTFAD_SR_CTXER2(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXER2_SHIFT)) & OTFAD_SR_CTXER2_MASK)

#define OTFAD_SR_CTXER3_MASK                     (0x800U)
#define OTFAD_SR_CTXER3_SHIFT                    (11U)
/*! CTXER3 - Context Error
 *  0b0..No key blob error was detected for context "n".
 *  0b1..A key blob integrity error might have been detected in context "n".
 */
#define OTFAD_SR_CTXER3(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXER3_SHIFT)) & OTFAD_SR_CTXER3_MASK)

#define OTFAD_SR_CTXIE0_MASK                     (0x10000U)
#define OTFAD_SR_CTXIE0_SHIFT                    (16U)
/*! CTXIE0 - Context Integrity Error
 *  0b0..No key blob integrity error was detected for context "n".
 *  0b1..A key blob integrity error was detected in context "n".
 */
#define OTFAD_SR_CTXIE0(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXIE0_SHIFT)) & OTFAD_SR_CTXIE0_MASK)

#define OTFAD_SR_CTXIE1_MASK                     (0x20000U)
#define OTFAD_SR_CTXIE1_SHIFT                    (17U)
/*! CTXIE1 - Context Integrity Error
 *  0b0..No key blob integrity error was detected for context "n".
 *  0b1..A key blob integrity error was detected in context "n".
 */
#define OTFAD_SR_CTXIE1(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXIE1_SHIFT)) & OTFAD_SR_CTXIE1_MASK)

#define OTFAD_SR_CTXIE2_MASK                     (0x40000U)
#define OTFAD_SR_CTXIE2_SHIFT                    (18U)
/*! CTXIE2 - Context Integrity Error
 *  0b0..No key blob integrity error was detected for context "n".
 *  0b1..A key blob integrity error was detected in context "n".
 */
#define OTFAD_SR_CTXIE2(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXIE2_SHIFT)) & OTFAD_SR_CTXIE2_MASK)

#define OTFAD_SR_CTXIE3_MASK                     (0x80000U)
#define OTFAD_SR_CTXIE3_SHIFT                    (19U)
/*! CTXIE3 - Context Integrity Error
 *  0b0..No key blob integrity error was detected for context "n".
 *  0b1..A key blob integrity error was detected in context "n".
 */
#define OTFAD_SR_CTXIE3(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_CTXIE3_SHIFT)) & OTFAD_SR_CTXIE3_MASK)

#define OTFAD_SR_HRL_MASK                        (0xF000000U)
#define OTFAD_SR_HRL_SHIFT                       (24U)
/*! HRL - Hardware Revision Level */
#define OTFAD_SR_HRL(x)                          (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_HRL_SHIFT)) & OTFAD_SR_HRL_MASK)

#define OTFAD_SR_RRAM_MASK                       (0x10000000U)
#define OTFAD_SR_RRAM_SHIFT                      (28U)
/*! RRAM - Restricted Register Access Mode
 *  0b0..Register access is fully enabled. The OTFAD programming model registers can be accessed "normally".
 *  0b1..Register access is restricted and only the CR, SR and optional MDPC registers can be accessed; others are treated as RAZ/WI.
 */
#define OTFAD_SR_RRAM(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_RRAM_SHIFT)) & OTFAD_SR_RRAM_MASK)

#define OTFAD_SR_GEM_MASK                        (0x20000000U)
#define OTFAD_SR_GEM_SHIFT                       (29U)
/*! GEM - Global Enable Mode
 *  0b0..OTFAD is disabled. All data fetched by the QuadSPI bypasses OTFAD processing.
 *  0b1..OTFAD is enabled, and processes data fetched by the QuadSPI as defined by the hardware configuration.
 */
#define OTFAD_SR_GEM(x)                          (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_GEM_SHIFT)) & OTFAD_SR_GEM_MASK)

#define OTFAD_SR_KBPE_MASK                       (0x40000000U)
#define OTFAD_SR_KBPE_SHIFT                      (30U)
/*! KBPE - Key Blob Processing Enable
 *  0b0..Key blob processing is not enabled.
 *  0b1..Key blob processing is enabled.
 */
#define OTFAD_SR_KBPE(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_KBPE_SHIFT)) & OTFAD_SR_KBPE_MASK)

#define OTFAD_SR_KBD_MASK                        (0x80000000U)
#define OTFAD_SR_KBD_SHIFT                       (31U)
/*! KBD - Key Blob Processing Done
 *  0b0..Key blob processing was not enabled, or is not complete.
 *  0b1..Key blob processing was enabled and is complete.
 */
#define OTFAD_SR_KBD(x)                          (((uint32_t)(((uint32_t)(x)) << OTFAD_SR_KBD_SHIFT)) & OTFAD_SR_KBD_MASK)
/*! @} */

/*! @name KEY - AES Key Word */
/*! @{ */

#define OTFAD_KEY_KEY_MASK                       (0xFFFFFFFFU)
#define OTFAD_KEY_KEY_SHIFT                      (0U)
/*! KEY - AES Key */
#define OTFAD_KEY_KEY(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_KEY_KEY_SHIFT)) & OTFAD_KEY_KEY_MASK)
/*! @} */

/* The count of OTFAD_KEY */
#define OTFAD_KEY_COUNT                          (4U)

/* The count of OTFAD_KEY */
#define OTFAD_KEY_COUNT2                         (4U)

/*! @name CTR - AES Counter Word */
/*! @{ */

#define OTFAD_CTR_CTR_MASK                       (0xFFFFFFFFU)
#define OTFAD_CTR_CTR_SHIFT                      (0U)
/*! CTR - AES Counter */
#define OTFAD_CTR_CTR(x)                         (((uint32_t)(((uint32_t)(x)) << OTFAD_CTR_CTR_SHIFT)) & OTFAD_CTR_CTR_MASK)
/*! @} */

/* The count of OTFAD_CTR */
#define OTFAD_CTR_COUNT                          (4U)

/* The count of OTFAD_CTR */
#define OTFAD_CTR_COUNT2                         (2U)

/*! @name RGD_W0 - AES Region Descriptor Word0 */
/*! @{ */

#define OTFAD_RGD_W0_SRTADDR_MASK                (0xFFFFFC00U)
#define OTFAD_RGD_W0_SRTADDR_SHIFT               (10U)
/*! SRTADDR - Start Address */
#define OTFAD_RGD_W0_SRTADDR(x)                  (((uint32_t)(((uint32_t)(x)) << OTFAD_RGD_W0_SRTADDR_SHIFT)) & OTFAD_RGD_W0_SRTADDR_MASK)
/*! @} */

/* The count of OTFAD_RGD_W0 */
#define OTFAD_RGD_W0_COUNT                       (4U)

/*! @name RGD_W1 - AES Region Descriptor Word1 */
/*! @{ */

#define OTFAD_RGD_W1_VLD_MASK                    (0x1U)
#define OTFAD_RGD_W1_VLD_SHIFT                   (0U)
/*! VLD - Valid
 *  0b0..Context is invalid.
 *  0b1..Context is valid.
 */
#define OTFAD_RGD_W1_VLD(x)                      (((uint32_t)(((uint32_t)(x)) << OTFAD_RGD_W1_VLD_SHIFT)) & OTFAD_RGD_W1_VLD_MASK)

#define OTFAD_RGD_W1_ADE_MASK                    (0x2U)
#define OTFAD_RGD_W1_ADE_SHIFT                   (1U)
/*! ADE - AES Decryption Enable.
 *  0b0..Bypass the fetched data.
 *  0b1..Perform the CTR-AES128 mode decryption on the fetched data.
 */
#define OTFAD_RGD_W1_ADE(x)                      (((uint32_t)(((uint32_t)(x)) << OTFAD_RGD_W1_ADE_SHIFT)) & OTFAD_RGD_W1_ADE_MASK)

#define OTFAD_RGD_W1_RO_MASK                     (0x4U)
#define OTFAD_RGD_W1_RO_SHIFT                    (2U)
/*! RO - Read-Only
 *  0b0..The context registers can be accessed normally (as defined by SR[RRAM]).
 *  0b1..The context registers are read-only and accesses may be further restricted based on SR[RRAM].
 */
#define OTFAD_RGD_W1_RO(x)                       (((uint32_t)(((uint32_t)(x)) << OTFAD_RGD_W1_RO_SHIFT)) & OTFAD_RGD_W1_RO_MASK)

#define OTFAD_RGD_W1_ENDADDR_MASK                (0xFFFFFC00U)
#define OTFAD_RGD_W1_ENDADDR_SHIFT               (10U)
/*! ENDADDR - End Address */
#define OTFAD_RGD_W1_ENDADDR(x)                  (((uint32_t)(((uint32_t)(x)) << OTFAD_RGD_W1_ENDADDR_SHIFT)) & OTFAD_RGD_W1_ENDADDR_MASK)
/*! @} */

/* The count of OTFAD_RGD_W1 */
#define OTFAD_RGD_W1_COUNT                       (4U)


/*!
 * @}
 */ /* end of group OTFAD_Register_Masks */


/* OTFAD - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral OTFAD1 base address */
  #define OTFAD1_BASE                              (0x525E0000u)
  /** Peripheral OTFAD1 base address */
  #define OTFAD1_BASE_NS                           (0x425E0000u)
  /** Peripheral OTFAD1 base pointer */
  #define OTFAD1                                   ((OTFAD_Type *)OTFAD1_BASE)
  /** Peripheral OTFAD1 base pointer */
  #define OTFAD1_NS                                ((OTFAD_Type *)OTFAD1_BASE_NS)
  /** Peripheral OTFAD2 base address */
  #define OTFAD2_BASE                              (0x545E0000u)
  /** Peripheral OTFAD2 base address */
  #define OTFAD2_BASE_NS                           (0x445E0000u)
  /** Peripheral OTFAD2 base pointer */
  #define OTFAD2                                   ((OTFAD_Type *)OTFAD2_BASE)
  /** Peripheral OTFAD2 base pointer */
  #define OTFAD2_NS                                ((OTFAD_Type *)OTFAD2_BASE_NS)
  /** Array initializer of OTFAD peripheral base addresses */
  #define OTFAD_BASE_ADDRS                         { 0u, OTFAD1_BASE, OTFAD2_BASE }
  /** Array initializer of OTFAD peripheral base pointers */
  #define OTFAD_BASE_PTRS                          { (OTFAD_Type *)0u, OTFAD1, OTFAD2 }
  /** Array initializer of OTFAD peripheral base addresses */
  #define OTFAD_BASE_ADDRS_NS                      { 0u, OTFAD1_BASE_NS, OTFAD2_BASE_NS }
  /** Array initializer of OTFAD peripheral base pointers */
  #define OTFAD_BASE_PTRS_NS                       { (OTFAD_Type *)0u, OTFAD1_NS, OTFAD2_NS }
#else
  /** Peripheral OTFAD1 base address */
  #define OTFAD1_BASE                              (0x425E0000u)
  /** Peripheral OTFAD1 base pointer */
  #define OTFAD1                                   ((OTFAD_Type *)OTFAD1_BASE)
  /** Peripheral OTFAD2 base address */
  #define OTFAD2_BASE                              (0x445E0000u)
  /** Peripheral OTFAD2 base pointer */
  #define OTFAD2                                   ((OTFAD_Type *)OTFAD2_BASE)
  /** Array initializer of OTFAD peripheral base addresses */
  #define OTFAD_BASE_ADDRS                         { 0u, OTFAD1_BASE, OTFAD2_BASE }
  /** Array initializer of OTFAD peripheral base pointers */
  #define OTFAD_BASE_PTRS                          { (OTFAD_Type *)0u, OTFAD1, OTFAD2 }
#endif

/*!
 * @}
 */ /* end of group OTFAD_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- PDM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PDM_Peripheral_Access_Layer PDM Peripheral Access Layer
 * @{
 */

/** PDM - Register Layout Typedef */
typedef struct {
  __IO uint32_t CTRL_1;                            /**< MICFIL Control 1, offset: 0x0 */
  __IO uint32_t CTRL_2;                            /**< MICFIL Control 2, offset: 0x4 */
  __IO uint32_t STAT;                              /**< MICFIL Status, offset: 0x8 */
       uint8_t RESERVED_0[4];
  __IO uint32_t FIFO_CTRL;                         /**< MICFIL FIFO Control, offset: 0x10 */
  __IO uint32_t FIFO_STAT;                         /**< MICFIL FIFO Status, offset: 0x14 */
       uint8_t RESERVED_1[12];
  __I  uint32_t DATACH[8];                         /**< MICFIL Output Result, array offset: 0x24, array step: 0x4 */
       uint8_t RESERVED_2[32];
  __IO uint32_t DC_CTRL;                           /**< MICFIL DC Remover Control, offset: 0x64 */
       uint8_t RESERVED_3[12];
  __IO uint32_t RANGE_CTRL;                        /**< MICFIL Range Control, offset: 0x74 */
       uint8_t RESERVED_4[4];
  __IO uint32_t RANGE_STAT;                        /**< MICFIL Range Status, offset: 0x7C */
       uint8_t RESERVED_5[16];
  __IO uint32_t VAD0_CTRL_1;                       /**< Voice Activity Detector 0 Control, offset: 0x90 */
  __IO uint32_t VAD0_CTRL_2;                       /**< Voice Activity Detector 0 Control, offset: 0x94 */
  __IO uint32_t VAD0_STAT;                         /**< Voice Activity Detector 0 Status, offset: 0x98 */
  __IO uint32_t VAD0_SCONFIG;                      /**< Voice Activity Detector 0 Signal Configuration, offset: 0x9C */
  __IO uint32_t VAD0_NCONFIG;                      /**< Voice Activity Detector 0 Noise Configuration, offset: 0xA0 */
  __I  uint32_t VAD0_NDATA;                        /**< Voice Activity Detector 0 Noise Data, offset: 0xA4 */
  __IO uint32_t VAD0_ZCD;                          /**< Voice Activity Detector 0 Zero-Crossing Detector, offset: 0xA8 */
} PDM_Type;

/* ----------------------------------------------------------------------------
   -- PDM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PDM_Register_Masks PDM Register Masks
 * @{
 */

/*! @name CTRL_1 - MICFIL Control 1 */
/*! @{ */

#define PDM_CTRL_1_CH0EN_MASK                    (0x1U)
#define PDM_CTRL_1_CH0EN_SHIFT                   (0U)
/*! CH0EN - Channel 0 Enable */
#define PDM_CTRL_1_CH0EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH0EN_SHIFT)) & PDM_CTRL_1_CH0EN_MASK)

#define PDM_CTRL_1_CH1EN_MASK                    (0x2U)
#define PDM_CTRL_1_CH1EN_SHIFT                   (1U)
/*! CH1EN - Channel 1 Enable */
#define PDM_CTRL_1_CH1EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH1EN_SHIFT)) & PDM_CTRL_1_CH1EN_MASK)

#define PDM_CTRL_1_CH2EN_MASK                    (0x4U)
#define PDM_CTRL_1_CH2EN_SHIFT                   (2U)
/*! CH2EN - Channel 2 Enable */
#define PDM_CTRL_1_CH2EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH2EN_SHIFT)) & PDM_CTRL_1_CH2EN_MASK)

#define PDM_CTRL_1_CH3EN_MASK                    (0x8U)
#define PDM_CTRL_1_CH3EN_SHIFT                   (3U)
/*! CH3EN - Channel 3 Enable */
#define PDM_CTRL_1_CH3EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH3EN_SHIFT)) & PDM_CTRL_1_CH3EN_MASK)

#define PDM_CTRL_1_CH4EN_MASK                    (0x10U)
#define PDM_CTRL_1_CH4EN_SHIFT                   (4U)
/*! CH4EN - Channel 4 Enable */
#define PDM_CTRL_1_CH4EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH4EN_SHIFT)) & PDM_CTRL_1_CH4EN_MASK)

#define PDM_CTRL_1_CH5EN_MASK                    (0x20U)
#define PDM_CTRL_1_CH5EN_SHIFT                   (5U)
/*! CH5EN - Channel 5 Enable */
#define PDM_CTRL_1_CH5EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH5EN_SHIFT)) & PDM_CTRL_1_CH5EN_MASK)

#define PDM_CTRL_1_CH6EN_MASK                    (0x40U)
#define PDM_CTRL_1_CH6EN_SHIFT                   (6U)
/*! CH6EN - Channel 6 Enable */
#define PDM_CTRL_1_CH6EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH6EN_SHIFT)) & PDM_CTRL_1_CH6EN_MASK)

#define PDM_CTRL_1_CH7EN_MASK                    (0x80U)
#define PDM_CTRL_1_CH7EN_SHIFT                   (7U)
/*! CH7EN - Channel 7 Enable */
#define PDM_CTRL_1_CH7EN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_CH7EN_SHIFT)) & PDM_CTRL_1_CH7EN_MASK)

#define PDM_CTRL_1_ERREN_MASK                    (0x800000U)
#define PDM_CTRL_1_ERREN_SHIFT                   (23U)
/*! ERREN - Error Interruption Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_CTRL_1_ERREN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_ERREN_SHIFT)) & PDM_CTRL_1_ERREN_MASK)

#define PDM_CTRL_1_DISEL_MASK                    (0x3000000U)
#define PDM_CTRL_1_DISEL_SHIFT                   (24U)
/*! DISEL - DMA Interrupt Selection
 *  0b00..Disables DMA and interrupt requests
 *  0b01..Enables DMA requests
 *  0b10..Enables interrupt requests
 *  0b11..Reserved
 */
#define PDM_CTRL_1_DISEL(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_DISEL_SHIFT)) & PDM_CTRL_1_DISEL_MASK)

#define PDM_CTRL_1_DBGE_MASK                     (0x4000000U)
#define PDM_CTRL_1_DBGE_SHIFT                    (26U)
/*! DBGE - Module Enable in Debug
 *  0b0..Disables after completing the current frame
 *  0b1..Enables operation
 */
#define PDM_CTRL_1_DBGE(x)                       (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_DBGE_SHIFT)) & PDM_CTRL_1_DBGE_MASK)

#define PDM_CTRL_1_SRES_MASK                     (0x8000000U)
#define PDM_CTRL_1_SRES_SHIFT                    (27U)
/*! SRES - Software Reset
 *  0b0..No action
 *  0b1..Software reset
 */
#define PDM_CTRL_1_SRES(x)                       (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_SRES_SHIFT)) & PDM_CTRL_1_SRES_MASK)

#define PDM_CTRL_1_DBG_MASK                      (0x10000000U)
#define PDM_CTRL_1_DBG_SHIFT                     (28U)
/*! DBG - Debug Mode
 *  0b0..Normal
 *  0b1..Debug
 */
#define PDM_CTRL_1_DBG(x)                        (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_DBG_SHIFT)) & PDM_CTRL_1_DBG_MASK)

#define PDM_CTRL_1_PDMIEN_MASK                   (0x20000000U)
#define PDM_CTRL_1_PDMIEN_SHIFT                  (29U)
/*! PDMIEN - MICFIL Enable
 *  0b0..Stops MICFIL operation
 *  0b1..Starts MICFIL operation
 */
#define PDM_CTRL_1_PDMIEN(x)                     (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_PDMIEN_SHIFT)) & PDM_CTRL_1_PDMIEN_MASK)

#define PDM_CTRL_1_DOZEN_MASK                    (0x40000000U)
#define PDM_CTRL_1_DOZEN_SHIFT                   (30U)
/*! DOZEN - Doze Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_CTRL_1_DOZEN(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_DOZEN_SHIFT)) & PDM_CTRL_1_DOZEN_MASK)

#define PDM_CTRL_1_MDIS_MASK                     (0x80000000U)
#define PDM_CTRL_1_MDIS_SHIFT                    (31U)
/*! MDIS - Module Disable
 *  0b0..Normal mode
 *  0b1..DLL mode
 */
#define PDM_CTRL_1_MDIS(x)                       (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_1_MDIS_SHIFT)) & PDM_CTRL_1_MDIS_MASK)
/*! @} */

/*! @name CTRL_2 - MICFIL Control 2 */
/*! @{ */

#define PDM_CTRL_2_CLKDIV_MASK                   (0xFFU)
#define PDM_CTRL_2_CLKDIV_SHIFT                  (0U)
/*! CLKDIV - Clock Divider
 *  0b00000000..Internal clock divider value = 0
 *  0b00000001..Internal clock divider value = 1
 *  0b00000010-0b11111110.....
 *  0b11111111..Internal clock divider value = 255
 */
#define PDM_CTRL_2_CLKDIV(x)                     (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_2_CLKDIV_SHIFT)) & PDM_CTRL_2_CLKDIV_MASK)

#define PDM_CTRL_2_CICOSR_MASK                   (0xF0000U)
#define PDM_CTRL_2_CICOSR_SHIFT                  (16U)
/*! CICOSR - CIC Decimation Rate
 *  0b0000..CIC oversampling rate = 0
 *  0b0001..CIC oversampling rate = 1
 *  0b0010-0b1110.....
 *  0b1111..CIC oversampling rate = 15
 */
#define PDM_CTRL_2_CICOSR(x)                     (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_2_CICOSR_SHIFT)) & PDM_CTRL_2_CICOSR_MASK)

#define PDM_CTRL_2_QSEL_MASK                     (0xE000000U)
#define PDM_CTRL_2_QSEL_SHIFT                    (25U)
/*! QSEL - Quality Mode
 *  0b001..High-Quality mode
 *  0b000..Medium-Quality mode
 *  0b111..Low-Quality mode
 *  0b110..Very-Low-Quality 0 mode
 *  0b101..Very-Low-Quality 1 mode
 *  0b100..Very-Low-Quality 2 mode
 */
#define PDM_CTRL_2_QSEL(x)                       (((uint32_t)(((uint32_t)(x)) << PDM_CTRL_2_QSEL_SHIFT)) & PDM_CTRL_2_QSEL_MASK)
/*! @} */

/*! @name STAT - MICFIL Status */
/*! @{ */

#define PDM_STAT_CH0F_MASK                       (0x1U)
#define PDM_STAT_CH0F_SHIFT                      (0U)
/*! CH0F - Channel 0 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH0F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH0F_SHIFT)) & PDM_STAT_CH0F_MASK)

#define PDM_STAT_CH1F_MASK                       (0x2U)
#define PDM_STAT_CH1F_SHIFT                      (1U)
/*! CH1F - Channel 1 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH1F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH1F_SHIFT)) & PDM_STAT_CH1F_MASK)

#define PDM_STAT_CH2F_MASK                       (0x4U)
#define PDM_STAT_CH2F_SHIFT                      (2U)
/*! CH2F - Channel 2 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH2F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH2F_SHIFT)) & PDM_STAT_CH2F_MASK)

#define PDM_STAT_CH3F_MASK                       (0x8U)
#define PDM_STAT_CH3F_SHIFT                      (3U)
/*! CH3F - Channel 3 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH3F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH3F_SHIFT)) & PDM_STAT_CH3F_MASK)

#define PDM_STAT_CH4F_MASK                       (0x10U)
#define PDM_STAT_CH4F_SHIFT                      (4U)
/*! CH4F - Channel 4 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH4F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH4F_SHIFT)) & PDM_STAT_CH4F_MASK)

#define PDM_STAT_CH5F_MASK                       (0x20U)
#define PDM_STAT_CH5F_SHIFT                      (5U)
/*! CH5F - Channel 5 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH5F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH5F_SHIFT)) & PDM_STAT_CH5F_MASK)

#define PDM_STAT_CH6F_MASK                       (0x40U)
#define PDM_STAT_CH6F_SHIFT                      (6U)
/*! CH6F - Channel 6 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH6F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH6F_SHIFT)) & PDM_STAT_CH6F_MASK)

#define PDM_STAT_CH7F_MASK                       (0x80U)
#define PDM_STAT_CH7F_SHIFT                      (7U)
/*! CH7F - Channel 7 Output Data Flag
 *  0b0..Not surpassed
 *  0b1..Surpassed
 */
#define PDM_STAT_CH7F(x)                         (((uint32_t)(((uint32_t)(x)) << PDM_STAT_CH7F_SHIFT)) & PDM_STAT_CH7F_MASK)

#define PDM_STAT_LOWFREQF_MASK                   (0x20000000U)
#define PDM_STAT_LOWFREQF_SHIFT                  (29U)
/*! LOWFREQF - Low Frequency Flag
 *  0b0..CLKDIV value is OK
 *  0b1..CLKDIV value is too low
 */
#define PDM_STAT_LOWFREQF(x)                     (((uint32_t)(((uint32_t)(x)) << PDM_STAT_LOWFREQF_SHIFT)) & PDM_STAT_LOWFREQF_MASK)

#define PDM_STAT_FIR_RDY_MASK                    (0x40000000U)
#define PDM_STAT_FIR_RDY_SHIFT                   (30U)
/*! FIR_RDY - Filter Data Ready
 *  0b0..Not reliable
 *  0b1..Reliable
 */
#define PDM_STAT_FIR_RDY(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_STAT_FIR_RDY_SHIFT)) & PDM_STAT_FIR_RDY_MASK)

#define PDM_STAT_BSY_FIL_MASK                    (0x80000000U)
#define PDM_STAT_BSY_FIL_SHIFT                   (31U)
/*! BSY_FIL - Busy Flag
 *  0b1..MICFIL is running
 *  0b0..MICFIL is stopped
 */
#define PDM_STAT_BSY_FIL(x)                      (((uint32_t)(((uint32_t)(x)) << PDM_STAT_BSY_FIL_SHIFT)) & PDM_STAT_BSY_FIL_MASK)
/*! @} */

/*! @name FIFO_CTRL - MICFIL FIFO Control */
/*! @{ */

#define PDM_FIFO_CTRL_FIFOWMK_MASK               (0x7U)
#define PDM_FIFO_CTRL_FIFOWMK_SHIFT              (0U)
/*! FIFOWMK - FIFO Watermark Control */
#define PDM_FIFO_CTRL_FIFOWMK(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_CTRL_FIFOWMK_SHIFT)) & PDM_FIFO_CTRL_FIFOWMK_MASK)
/*! @} */

/*! @name FIFO_STAT - MICFIL FIFO Status */
/*! @{ */

#define PDM_FIFO_STAT_FIFOOVF0_MASK              (0x1U)
#define PDM_FIFO_STAT_FIFOOVF0_SHIFT             (0U)
/*! FIFOOVF0 - FIFO Overflow Exception Flag for Channel 0
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF0(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF0_SHIFT)) & PDM_FIFO_STAT_FIFOOVF0_MASK)

#define PDM_FIFO_STAT_FIFOOVF1_MASK              (0x2U)
#define PDM_FIFO_STAT_FIFOOVF1_SHIFT             (1U)
/*! FIFOOVF1 - FIFO Overflow Exception Flag for Channel 1
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF1(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF1_SHIFT)) & PDM_FIFO_STAT_FIFOOVF1_MASK)

#define PDM_FIFO_STAT_FIFOOVF2_MASK              (0x4U)
#define PDM_FIFO_STAT_FIFOOVF2_SHIFT             (2U)
/*! FIFOOVF2 - FIFO Overflow Exception Flag for Channel 2
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF2(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF2_SHIFT)) & PDM_FIFO_STAT_FIFOOVF2_MASK)

#define PDM_FIFO_STAT_FIFOOVF3_MASK              (0x8U)
#define PDM_FIFO_STAT_FIFOOVF3_SHIFT             (3U)
/*! FIFOOVF3 - FIFO Overflow Exception Flag for Channel 3
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF3(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF3_SHIFT)) & PDM_FIFO_STAT_FIFOOVF3_MASK)

#define PDM_FIFO_STAT_FIFOOVF4_MASK              (0x10U)
#define PDM_FIFO_STAT_FIFOOVF4_SHIFT             (4U)
/*! FIFOOVF4 - FIFO Overflow Exception Flag for Channel 4
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF4(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF4_SHIFT)) & PDM_FIFO_STAT_FIFOOVF4_MASK)

#define PDM_FIFO_STAT_FIFOOVF5_MASK              (0x20U)
#define PDM_FIFO_STAT_FIFOOVF5_SHIFT             (5U)
/*! FIFOOVF5 - FIFO Overflow Exception Flag for Channel 5
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF5(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF5_SHIFT)) & PDM_FIFO_STAT_FIFOOVF5_MASK)

#define PDM_FIFO_STAT_FIFOOVF6_MASK              (0x40U)
#define PDM_FIFO_STAT_FIFOOVF6_SHIFT             (6U)
/*! FIFOOVF6 - FIFO Overflow Exception Flag for Channel 6
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF6(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF6_SHIFT)) & PDM_FIFO_STAT_FIFOOVF6_MASK)

#define PDM_FIFO_STAT_FIFOOVF7_MASK              (0x80U)
#define PDM_FIFO_STAT_FIFOOVF7_SHIFT             (7U)
/*! FIFOOVF7 - FIFO Overflow Exception Flag for Channel 7
 *  0b0..No exception by FIFO overflow
 *  0b1..Exception by FIFO overflow
 */
#define PDM_FIFO_STAT_FIFOOVF7(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOOVF7_SHIFT)) & PDM_FIFO_STAT_FIFOOVF7_MASK)

#define PDM_FIFO_STAT_FIFOUND0_MASK              (0x100U)
#define PDM_FIFO_STAT_FIFOUND0_SHIFT             (8U)
/*! FIFOUND0 - FIFO Underflow Exception Flag for Channel 0
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND0(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND0_SHIFT)) & PDM_FIFO_STAT_FIFOUND0_MASK)

#define PDM_FIFO_STAT_FIFOUND1_MASK              (0x200U)
#define PDM_FIFO_STAT_FIFOUND1_SHIFT             (9U)
/*! FIFOUND1 - FIFO Underflow Exception Flag for Channel 1
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND1(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND1_SHIFT)) & PDM_FIFO_STAT_FIFOUND1_MASK)

#define PDM_FIFO_STAT_FIFOUND2_MASK              (0x400U)
#define PDM_FIFO_STAT_FIFOUND2_SHIFT             (10U)
/*! FIFOUND2 - FIFO Underflow Exception Flag for Channel 2
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND2(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND2_SHIFT)) & PDM_FIFO_STAT_FIFOUND2_MASK)

#define PDM_FIFO_STAT_FIFOUND3_MASK              (0x800U)
#define PDM_FIFO_STAT_FIFOUND3_SHIFT             (11U)
/*! FIFOUND3 - FIFO Underflow Exception Flag for Channel 3
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND3(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND3_SHIFT)) & PDM_FIFO_STAT_FIFOUND3_MASK)

#define PDM_FIFO_STAT_FIFOUND4_MASK              (0x1000U)
#define PDM_FIFO_STAT_FIFOUND4_SHIFT             (12U)
/*! FIFOUND4 - FIFO Underflow Exception Flag for Channel 4
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND4(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND4_SHIFT)) & PDM_FIFO_STAT_FIFOUND4_MASK)

#define PDM_FIFO_STAT_FIFOUND5_MASK              (0x2000U)
#define PDM_FIFO_STAT_FIFOUND5_SHIFT             (13U)
/*! FIFOUND5 - FIFO Underflow Exception Flag for Channel 5
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND5(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND5_SHIFT)) & PDM_FIFO_STAT_FIFOUND5_MASK)

#define PDM_FIFO_STAT_FIFOUND6_MASK              (0x4000U)
#define PDM_FIFO_STAT_FIFOUND6_SHIFT             (14U)
/*! FIFOUND6 - FIFO Underflow Exception Flag for Channel 6
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND6(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND6_SHIFT)) & PDM_FIFO_STAT_FIFOUND6_MASK)

#define PDM_FIFO_STAT_FIFOUND7_MASK              (0x8000U)
#define PDM_FIFO_STAT_FIFOUND7_SHIFT             (15U)
/*! FIFOUND7 - FIFO Underflow Exception Flag for Channel 7
 *  0b0..No exception by FIFO underflow
 *  0b1..Exception by FIFO underflow
 */
#define PDM_FIFO_STAT_FIFOUND7(x)                (((uint32_t)(((uint32_t)(x)) << PDM_FIFO_STAT_FIFOUND7_SHIFT)) & PDM_FIFO_STAT_FIFOUND7_MASK)
/*! @} */

/*! @name DATACH - MICFIL Output Result */
/*! @{ */

#define PDM_DATACH_DATA_MASK                     (0xFFFFFFFFU)
#define PDM_DATACH_DATA_SHIFT                    (0U)
/*! DATA - Channel n Data */
#define PDM_DATACH_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << PDM_DATACH_DATA_SHIFT)) & PDM_DATACH_DATA_MASK)
/*! @} */

/* The count of PDM_DATACH */
#define PDM_DATACH_COUNT                         (8U)

/*! @name DC_CTRL - MICFIL DC Remover Control */
/*! @{ */

#define PDM_DC_CTRL_DCCONFIG0_MASK               (0x3U)
#define PDM_DC_CTRL_DCCONFIG0_SHIFT              (0U)
/*! DCCONFIG0 - Channel 0 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG0(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG0_SHIFT)) & PDM_DC_CTRL_DCCONFIG0_MASK)

#define PDM_DC_CTRL_DCCONFIG1_MASK               (0xCU)
#define PDM_DC_CTRL_DCCONFIG1_SHIFT              (2U)
/*! DCCONFIG1 - Channel 1 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG1(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG1_SHIFT)) & PDM_DC_CTRL_DCCONFIG1_MASK)

#define PDM_DC_CTRL_DCCONFIG2_MASK               (0x30U)
#define PDM_DC_CTRL_DCCONFIG2_SHIFT              (4U)
/*! DCCONFIG2 - Channel 2 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG2(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG2_SHIFT)) & PDM_DC_CTRL_DCCONFIG2_MASK)

#define PDM_DC_CTRL_DCCONFIG3_MASK               (0xC0U)
#define PDM_DC_CTRL_DCCONFIG3_SHIFT              (6U)
/*! DCCONFIG3 - Channel 3 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG3(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG3_SHIFT)) & PDM_DC_CTRL_DCCONFIG3_MASK)

#define PDM_DC_CTRL_DCCONFIG4_MASK               (0x300U)
#define PDM_DC_CTRL_DCCONFIG4_SHIFT              (8U)
/*! DCCONFIG4 - Channel 4 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG4(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG4_SHIFT)) & PDM_DC_CTRL_DCCONFIG4_MASK)

#define PDM_DC_CTRL_DCCONFIG5_MASK               (0xC00U)
#define PDM_DC_CTRL_DCCONFIG5_SHIFT              (10U)
/*! DCCONFIG5 - Channel 5 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG5(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG5_SHIFT)) & PDM_DC_CTRL_DCCONFIG5_MASK)

#define PDM_DC_CTRL_DCCONFIG6_MASK               (0x3000U)
#define PDM_DC_CTRL_DCCONFIG6_SHIFT              (12U)
/*! DCCONFIG6 - Channel 6 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG6(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG6_SHIFT)) & PDM_DC_CTRL_DCCONFIG6_MASK)

#define PDM_DC_CTRL_DCCONFIG7_MASK               (0xC000U)
#define PDM_DC_CTRL_DCCONFIG7_SHIFT              (14U)
/*! DCCONFIG7 - Channel 7 DC Remover Configuration
 *  0b11..DC remover is bypassed
 *  0b00..21 Hz
 *  0b01..83 Hz
 *  0b10..152 Hz
 */
#define PDM_DC_CTRL_DCCONFIG7(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_DC_CTRL_DCCONFIG7_SHIFT)) & PDM_DC_CTRL_DCCONFIG7_MASK)
/*! @} */

/*! @name RANGE_CTRL - MICFIL Range Control */
/*! @{ */

#define PDM_RANGE_CTRL_RANGEADJ0_MASK            (0xFU)
#define PDM_RANGE_CTRL_RANGEADJ0_SHIFT           (0U)
/*! RANGEADJ0 - Channel 0 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ0(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ0_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ0_MASK)

#define PDM_RANGE_CTRL_RANGEADJ1_MASK            (0xF0U)
#define PDM_RANGE_CTRL_RANGEADJ1_SHIFT           (4U)
/*! RANGEADJ1 - Channel 1 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ1(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ1_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ1_MASK)

#define PDM_RANGE_CTRL_RANGEADJ2_MASK            (0xF00U)
#define PDM_RANGE_CTRL_RANGEADJ2_SHIFT           (8U)
/*! RANGEADJ2 - Channel 2 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ2(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ2_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ2_MASK)

#define PDM_RANGE_CTRL_RANGEADJ3_MASK            (0xF000U)
#define PDM_RANGE_CTRL_RANGEADJ3_SHIFT           (12U)
/*! RANGEADJ3 - Channel 3 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ3(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ3_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ3_MASK)

#define PDM_RANGE_CTRL_RANGEADJ4_MASK            (0xF0000U)
#define PDM_RANGE_CTRL_RANGEADJ4_SHIFT           (16U)
/*! RANGEADJ4 - Channel 4 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ4(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ4_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ4_MASK)

#define PDM_RANGE_CTRL_RANGEADJ5_MASK            (0xF00000U)
#define PDM_RANGE_CTRL_RANGEADJ5_SHIFT           (20U)
/*! RANGEADJ5 - Channel 5 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ5(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ5_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ5_MASK)

#define PDM_RANGE_CTRL_RANGEADJ6_MASK            (0xF000000U)
#define PDM_RANGE_CTRL_RANGEADJ6_SHIFT           (24U)
/*! RANGEADJ6 - Channel 6 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ6(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ6_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ6_MASK)

#define PDM_RANGE_CTRL_RANGEADJ7_MASK            (0xF0000000U)
#define PDM_RANGE_CTRL_RANGEADJ7_SHIFT           (28U)
/*! RANGEADJ7 - Channel 7 Range Adjustment */
#define PDM_RANGE_CTRL_RANGEADJ7(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_CTRL_RANGEADJ7_SHIFT)) & PDM_RANGE_CTRL_RANGEADJ7_MASK)
/*! @} */

/*! @name RANGE_STAT - MICFIL Range Status */
/*! @{ */

#define PDM_RANGE_STAT_RANGEOVF0_MASK            (0x1U)
#define PDM_RANGE_STAT_RANGEOVF0_SHIFT           (0U)
/*! RANGEOVF0 - Channel 0 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF0(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF0_SHIFT)) & PDM_RANGE_STAT_RANGEOVF0_MASK)

#define PDM_RANGE_STAT_RANGEOVF1_MASK            (0x2U)
#define PDM_RANGE_STAT_RANGEOVF1_SHIFT           (1U)
/*! RANGEOVF1 - Channel 1 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF1(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF1_SHIFT)) & PDM_RANGE_STAT_RANGEOVF1_MASK)

#define PDM_RANGE_STAT_RANGEOVF2_MASK            (0x4U)
#define PDM_RANGE_STAT_RANGEOVF2_SHIFT           (2U)
/*! RANGEOVF2 - Channel 2 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF2(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF2_SHIFT)) & PDM_RANGE_STAT_RANGEOVF2_MASK)

#define PDM_RANGE_STAT_RANGEOVF3_MASK            (0x8U)
#define PDM_RANGE_STAT_RANGEOVF3_SHIFT           (3U)
/*! RANGEOVF3 - Channel 3 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF3(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF3_SHIFT)) & PDM_RANGE_STAT_RANGEOVF3_MASK)

#define PDM_RANGE_STAT_RANGEOVF4_MASK            (0x10U)
#define PDM_RANGE_STAT_RANGEOVF4_SHIFT           (4U)
/*! RANGEOVF4 - Channel 4 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF4(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF4_SHIFT)) & PDM_RANGE_STAT_RANGEOVF4_MASK)

#define PDM_RANGE_STAT_RANGEOVF5_MASK            (0x20U)
#define PDM_RANGE_STAT_RANGEOVF5_SHIFT           (5U)
/*! RANGEOVF5 - Channel 5 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF5(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF5_SHIFT)) & PDM_RANGE_STAT_RANGEOVF5_MASK)

#define PDM_RANGE_STAT_RANGEOVF6_MASK            (0x40U)
#define PDM_RANGE_STAT_RANGEOVF6_SHIFT           (6U)
/*! RANGEOVF6 - Channel 6 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF6(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF6_SHIFT)) & PDM_RANGE_STAT_RANGEOVF6_MASK)

#define PDM_RANGE_STAT_RANGEOVF7_MASK            (0x80U)
#define PDM_RANGE_STAT_RANGEOVF7_SHIFT           (7U)
/*! RANGEOVF7 - Channel 7 Range Overflow Error Flag
 *  0b0..No exception by range overflow
 *  0b1..Exception by range overflow
 */
#define PDM_RANGE_STAT_RANGEOVF7(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEOVF7_SHIFT)) & PDM_RANGE_STAT_RANGEOVF7_MASK)

#define PDM_RANGE_STAT_RANGEUNF0_MASK            (0x10000U)
#define PDM_RANGE_STAT_RANGEUNF0_SHIFT           (16U)
/*! RANGEUNF0 - Channel 0 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF0(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF0_SHIFT)) & PDM_RANGE_STAT_RANGEUNF0_MASK)

#define PDM_RANGE_STAT_RANGEUNF1_MASK            (0x20000U)
#define PDM_RANGE_STAT_RANGEUNF1_SHIFT           (17U)
/*! RANGEUNF1 - Channel 1 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF1(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF1_SHIFT)) & PDM_RANGE_STAT_RANGEUNF1_MASK)

#define PDM_RANGE_STAT_RANGEUNF2_MASK            (0x40000U)
#define PDM_RANGE_STAT_RANGEUNF2_SHIFT           (18U)
/*! RANGEUNF2 - Channel 2 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF2(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF2_SHIFT)) & PDM_RANGE_STAT_RANGEUNF2_MASK)

#define PDM_RANGE_STAT_RANGEUNF3_MASK            (0x80000U)
#define PDM_RANGE_STAT_RANGEUNF3_SHIFT           (19U)
/*! RANGEUNF3 - Channel 3 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF3(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF3_SHIFT)) & PDM_RANGE_STAT_RANGEUNF3_MASK)

#define PDM_RANGE_STAT_RANGEUNF4_MASK            (0x100000U)
#define PDM_RANGE_STAT_RANGEUNF4_SHIFT           (20U)
/*! RANGEUNF4 - Channel 4 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF4(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF4_SHIFT)) & PDM_RANGE_STAT_RANGEUNF4_MASK)

#define PDM_RANGE_STAT_RANGEUNF5_MASK            (0x200000U)
#define PDM_RANGE_STAT_RANGEUNF5_SHIFT           (21U)
/*! RANGEUNF5 - Channel 5 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF5(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF5_SHIFT)) & PDM_RANGE_STAT_RANGEUNF5_MASK)

#define PDM_RANGE_STAT_RANGEUNF6_MASK            (0x400000U)
#define PDM_RANGE_STAT_RANGEUNF6_SHIFT           (22U)
/*! RANGEUNF6 - Channel 6 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF6(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF6_SHIFT)) & PDM_RANGE_STAT_RANGEUNF6_MASK)

#define PDM_RANGE_STAT_RANGEUNF7_MASK            (0x800000U)
#define PDM_RANGE_STAT_RANGEUNF7_SHIFT           (23U)
/*! RANGEUNF7 - Channel 7 Range Underflow Error Flag
 *  0b0..No exception by range underflow
 *  0b1..Exception by range underflow
 */
#define PDM_RANGE_STAT_RANGEUNF7(x)              (((uint32_t)(((uint32_t)(x)) << PDM_RANGE_STAT_RANGEUNF7_SHIFT)) & PDM_RANGE_STAT_RANGEUNF7_MASK)
/*! @} */

/*! @name VAD0_CTRL_1 - Voice Activity Detector 0 Control */
/*! @{ */

#define PDM_VAD0_CTRL_1_VADEN_MASK               (0x1U)
#define PDM_VAD0_CTRL_1_VADEN_SHIFT              (0U)
/*! VADEN - Voice Activity Detector Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_VAD0_CTRL_1_VADEN(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADEN_SHIFT)) & PDM_VAD0_CTRL_1_VADEN_MASK)

#define PDM_VAD0_CTRL_1_VADRST_MASK              (0x2U)
#define PDM_VAD0_CTRL_1_VADRST_SHIFT             (1U)
/*! VADRST - Voice Activity Detector Reset */
#define PDM_VAD0_CTRL_1_VADRST(x)                (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADRST_SHIFT)) & PDM_VAD0_CTRL_1_VADRST_MASK)

#define PDM_VAD0_CTRL_1_VADIE_MASK               (0x4U)
#define PDM_VAD0_CTRL_1_VADIE_SHIFT              (2U)
/*! VADIE - Voice Activity Detector Interruption Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_VAD0_CTRL_1_VADIE(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADIE_SHIFT)) & PDM_VAD0_CTRL_1_VADIE_MASK)

#define PDM_VAD0_CTRL_1_VADERIE_MASK             (0x8U)
#define PDM_VAD0_CTRL_1_VADERIE_SHIFT            (3U)
/*! VADERIE - Voice Activity Detector Error Interruption Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_VAD0_CTRL_1_VADERIE(x)               (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADERIE_SHIFT)) & PDM_VAD0_CTRL_1_VADERIE_MASK)

#define PDM_VAD0_CTRL_1_VADST10_MASK             (0x10U)
#define PDM_VAD0_CTRL_1_VADST10_SHIFT            (4U)
/*! VADST10 - Voice Activity Detector Internal Filters Initialization
 *  0b0..Normal operation
 *  0b1..Filters initialized
 */
#define PDM_VAD0_CTRL_1_VADST10(x)               (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADST10_SHIFT)) & PDM_VAD0_CTRL_1_VADST10_MASK)

#define PDM_VAD0_CTRL_1_VADINITT_MASK            (0x1F00U)
#define PDM_VAD0_CTRL_1_VADINITT_SHIFT           (8U)
/*! VADINITT - Voice Activity Detector Initialization Time
 *  0b00000..VADINITT = 0
 *  0b00001..VADINITT = 1
 *  0b00010-0b11110.....
 *  0b11111..VADINITT = 31
 */
#define PDM_VAD0_CTRL_1_VADINITT(x)              (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADINITT_SHIFT)) & PDM_VAD0_CTRL_1_VADINITT_MASK)

#define PDM_VAD0_CTRL_1_VADCICOSR_MASK           (0xF0000U)
#define PDM_VAD0_CTRL_1_VADCICOSR_SHIFT          (16U)
/*! VADCICOSR - Voice Activity Detector CIC Oversampling Rate */
#define PDM_VAD0_CTRL_1_VADCICOSR(x)             (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADCICOSR_SHIFT)) & PDM_VAD0_CTRL_1_VADCICOSR_MASK)

#define PDM_VAD0_CTRL_1_VADCHSEL_MASK            (0x7000000U)
#define PDM_VAD0_CTRL_1_VADCHSEL_SHIFT           (24U)
/*! VADCHSEL - Voice Activity Detector Channel Selector
 *  0b000..PDM Microphone 0 Left
 *  0b001..PDM Microphone 0 Right
 *  0b010..PDM Microphone 1 Left
 *  0b011-0b101.....
 *  0b110..PDM Microphone 3 Left
 *  0b111..PDM Microphone 3 Right
 */
#define PDM_VAD0_CTRL_1_VADCHSEL(x)              (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_1_VADCHSEL_SHIFT)) & PDM_VAD0_CTRL_1_VADCHSEL_MASK)
/*! @} */

/*! @name VAD0_CTRL_2 - Voice Activity Detector 0 Control */
/*! @{ */

#define PDM_VAD0_CTRL_2_VADHPF_MASK              (0x3U)
#define PDM_VAD0_CTRL_2_VADHPF_SHIFT             (0U)
/*! VADHPF - Voice Activity Detector High-Pass Filter
 *  0b00..Filter bypassed
 *  0b01..Cut-off frequency at 1750 Hz
 *  0b10..Cut-off frequency at 215 Hz
 *  0b11..Cut-off frequency at 102 Hz
 */
#define PDM_VAD0_CTRL_2_VADHPF(x)                (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_2_VADHPF_SHIFT)) & PDM_VAD0_CTRL_2_VADHPF_MASK)

#define PDM_VAD0_CTRL_2_VADINPGAIN_MASK          (0xF00U)
#define PDM_VAD0_CTRL_2_VADINPGAIN_SHIFT         (8U)
/*! VADINPGAIN - Voice Activity Detector Input Gain
 *  0b0000..No shift
 *  0b0001..Shift 1 bit to the left
 *  0b0010..Shift 2 bits to the left
 *  0b0011-0b0110.....
 *  0b0111..Shift 7 bits to the left
 *  0b1000..Shift 8 bits to the right
 *  0b1001..Shift 7 bits to the right
 *  0b1010-0b1110.....
 *  0b1111..Shift 1 bits to the right
 */
#define PDM_VAD0_CTRL_2_VADINPGAIN(x)            (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_2_VADINPGAIN_SHIFT)) & PDM_VAD0_CTRL_2_VADINPGAIN_MASK)

#define PDM_VAD0_CTRL_2_VADFRAMET_MASK           (0x3F0000U)
#define PDM_VAD0_CTRL_2_VADFRAMET_SHIFT          (16U)
/*! VADFRAMET - Voice Activity Detector Frame Time
 *  0b000000..VADFRAMET = 1
 *  0b000001..VADFRAMET = 2
 *  0b000010-0b111110.....
 *  0b111111..VADFRAMET = 63
 */
#define PDM_VAD0_CTRL_2_VADFRAMET(x)             (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_2_VADFRAMET_SHIFT)) & PDM_VAD0_CTRL_2_VADFRAMET_MASK)

#define PDM_VAD0_CTRL_2_VADFOUTDIS_MASK          (0x10000000U)
#define PDM_VAD0_CTRL_2_VADFOUTDIS_SHIFT         (28U)
/*! VADFOUTDIS - Voice Activity Detector Force Output Disable
 *  0b0..Enables
 *  0b1..Disables
 */
#define PDM_VAD0_CTRL_2_VADFOUTDIS(x)            (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_2_VADFOUTDIS_SHIFT)) & PDM_VAD0_CTRL_2_VADFOUTDIS_MASK)

#define PDM_VAD0_CTRL_2_VADPREFEN_MASK           (0x40000000U)
#define PDM_VAD0_CTRL_2_VADPREFEN_SHIFT          (30U)
/*! VADPREFEN - Voice Activity Detector Pre Filter Enable
 *  0b0..Pre-filter bypassed
 *  0b1..Pre-filter enabled
 */
#define PDM_VAD0_CTRL_2_VADPREFEN(x)             (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_2_VADPREFEN_SHIFT)) & PDM_VAD0_CTRL_2_VADPREFEN_MASK)

#define PDM_VAD0_CTRL_2_VADFRENDIS_MASK          (0x80000000U)
#define PDM_VAD0_CTRL_2_VADFRENDIS_SHIFT         (31U)
/*! VADFRENDIS - Voice Activity Detector Frame Energy Disable
 *  0b1..Disables
 *  0b0..Enables
 */
#define PDM_VAD0_CTRL_2_VADFRENDIS(x)            (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_CTRL_2_VADFRENDIS_SHIFT)) & PDM_VAD0_CTRL_2_VADFRENDIS_MASK)
/*! @} */

/*! @name VAD0_STAT - Voice Activity Detector 0 Status */
/*! @{ */

#define PDM_VAD0_STAT_VADIF_MASK                 (0x1U)
#define PDM_VAD0_STAT_VADIF_SHIFT                (0U)
/*! VADIF - Voice Activity Detector Interrupt Flag
 *  0b0..Not detected
 *  0b1..Detected
 */
#define PDM_VAD0_STAT_VADIF(x)                   (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_STAT_VADIF_SHIFT)) & PDM_VAD0_STAT_VADIF_MASK)

#define PDM_VAD0_STAT_VADINSATF_MASK             (0x10000U)
#define PDM_VAD0_STAT_VADINSATF_SHIFT            (16U)
/*! VADINSATF - Voice Activity Detector Input Saturation Flag
 *  0b0..No exception
 *  0b1..Exception
 */
#define PDM_VAD0_STAT_VADINSATF(x)               (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_STAT_VADINSATF_SHIFT)) & PDM_VAD0_STAT_VADINSATF_MASK)

#define PDM_VAD0_STAT_VADINITF_MASK              (0x80000000U)
#define PDM_VAD0_STAT_VADINITF_SHIFT             (31U)
/*! VADINITF - Voice Activity Detector Initialization Flag
 *  0b0..Not being initialized
 *  0b1..Being initialized
 */
#define PDM_VAD0_STAT_VADINITF(x)                (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_STAT_VADINITF_SHIFT)) & PDM_VAD0_STAT_VADINITF_MASK)
/*! @} */

/*! @name VAD0_SCONFIG - Voice Activity Detector 0 Signal Configuration */
/*! @{ */

#define PDM_VAD0_SCONFIG_VADSGAIN_MASK           (0xFU)
#define PDM_VAD0_SCONFIG_VADSGAIN_SHIFT          (0U)
/*! VADSGAIN - Voice Activity Detector Signal Gain
 *  0b0000, 0b0001..Multiplier = 1
 *  0b0010..Multiplier = 2
 *  0b0011-0b1110.....
 *  0b1111..Multiplier = 15
 */
#define PDM_VAD0_SCONFIG_VADSGAIN(x)             (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_SCONFIG_VADSGAIN_SHIFT)) & PDM_VAD0_SCONFIG_VADSGAIN_MASK)

#define PDM_VAD0_SCONFIG_VADSMAXEN_MASK          (0x40000000U)
#define PDM_VAD0_SCONFIG_VADSMAXEN_SHIFT         (30U)
/*! VADSMAXEN - Voice Activity Detector Signal Maximum Enable
 *  0b0..Maximum block bypassed
 *  0b1..Maximum block enabled
 */
#define PDM_VAD0_SCONFIG_VADSMAXEN(x)            (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_SCONFIG_VADSMAXEN_SHIFT)) & PDM_VAD0_SCONFIG_VADSMAXEN_MASK)

#define PDM_VAD0_SCONFIG_VADSFILEN_MASK          (0x80000000U)
#define PDM_VAD0_SCONFIG_VADSFILEN_SHIFT         (31U)
/*! VADSFILEN - Voice Activity Detector Signal Filter Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_VAD0_SCONFIG_VADSFILEN(x)            (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_SCONFIG_VADSFILEN_SHIFT)) & PDM_VAD0_SCONFIG_VADSFILEN_MASK)
/*! @} */

/*! @name VAD0_NCONFIG - Voice Activity Detector 0 Noise Configuration */
/*! @{ */

#define PDM_VAD0_NCONFIG_VADNGAIN_MASK           (0xFU)
#define PDM_VAD0_NCONFIG_VADNGAIN_SHIFT          (0U)
/*! VADNGAIN - Voice Activity Detector Noise Gain
 *  0b0000, 0b0001..Multiplier = 1
 *  0b0010..Multiplier = 2
 *  0b0011-0b1110.....
 *  0b1111..Multiplier = 15
 */
#define PDM_VAD0_NCONFIG_VADNGAIN(x)             (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_NCONFIG_VADNGAIN_SHIFT)) & PDM_VAD0_NCONFIG_VADNGAIN_MASK)

#define PDM_VAD0_NCONFIG_VADNFILADJ_MASK         (0x1F00U)
#define PDM_VAD0_NCONFIG_VADNFILADJ_SHIFT        (8U)
/*! VADNFILADJ - Voice Activity Detector Noise Filter Adjustment
 *  0b00000..Adjustment value = 0
 *  0b00001..Adjustment value = 1
 *  0b00010-0b11110.....
 *  0b11111..Adjustment value = 31
 */
#define PDM_VAD0_NCONFIG_VADNFILADJ(x)           (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_NCONFIG_VADNFILADJ_SHIFT)) & PDM_VAD0_NCONFIG_VADNFILADJ_MASK)

#define PDM_VAD0_NCONFIG_VADNOREN_MASK           (0x10000000U)
#define PDM_VAD0_NCONFIG_VADNOREN_SHIFT          (28U)
/*! VADNOREN - Voice Activity Detector Noise OR Enable
 *  0b0..Not decimated
 *  0b1..Decimated
 */
#define PDM_VAD0_NCONFIG_VADNOREN(x)             (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_NCONFIG_VADNOREN_SHIFT)) & PDM_VAD0_NCONFIG_VADNOREN_MASK)

#define PDM_VAD0_NCONFIG_VADNDECEN_MASK          (0x20000000U)
#define PDM_VAD0_NCONFIG_VADNDECEN_SHIFT         (29U)
/*! VADNDECEN - Voice Activity Detector Noise Decimation Enable
 *  0b0..Not decimated
 *  0b1..Decimated
 */
#define PDM_VAD0_NCONFIG_VADNDECEN(x)            (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_NCONFIG_VADNDECEN_SHIFT)) & PDM_VAD0_NCONFIG_VADNDECEN_MASK)

#define PDM_VAD0_NCONFIG_VADNMINEN_MASK          (0x40000000U)
#define PDM_VAD0_NCONFIG_VADNMINEN_SHIFT         (30U)
/*! VADNMINEN - Voice Activity Detector Noise Minimum Enable
 *  0b0..Minimum block bypassed
 *  0b1..Minimum block enabled
 */
#define PDM_VAD0_NCONFIG_VADNMINEN(x)            (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_NCONFIG_VADNMINEN_SHIFT)) & PDM_VAD0_NCONFIG_VADNMINEN_MASK)

#define PDM_VAD0_NCONFIG_VADNFILAUTO_MASK        (0x80000000U)
#define PDM_VAD0_NCONFIG_VADNFILAUTO_SHIFT       (31U)
/*! VADNFILAUTO - Voice Activity Detector Noise Filter Auto
 *  0b0..Noise filter always enabled
 *  0b1..Noise filter enabled/disabled based on voice activity information
 */
#define PDM_VAD0_NCONFIG_VADNFILAUTO(x)          (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_NCONFIG_VADNFILAUTO_SHIFT)) & PDM_VAD0_NCONFIG_VADNFILAUTO_MASK)
/*! @} */

/*! @name VAD0_NDATA - Voice Activity Detector 0 Noise Data */
/*! @{ */

#define PDM_VAD0_NDATA_VADNDATA_MASK             (0xFFFFU)
#define PDM_VAD0_NDATA_VADNDATA_SHIFT            (0U)
/*! VADNDATA - Voice Activity Detector Noise Data */
#define PDM_VAD0_NDATA_VADNDATA(x)               (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_NDATA_VADNDATA_SHIFT)) & PDM_VAD0_NDATA_VADNDATA_MASK)
/*! @} */

/*! @name VAD0_ZCD - Voice Activity Detector 0 Zero-Crossing Detector */
/*! @{ */

#define PDM_VAD0_ZCD_VADZCDEN_MASK               (0x1U)
#define PDM_VAD0_ZCD_VADZCDEN_SHIFT              (0U)
/*! VADZCDEN - Zero-Crossing Detector Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_VAD0_ZCD_VADZCDEN(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_ZCD_VADZCDEN_SHIFT)) & PDM_VAD0_ZCD_VADZCDEN_MASK)

#define PDM_VAD0_ZCD_VADZCDAUTO_MASK             (0x4U)
#define PDM_VAD0_ZCD_VADZCDAUTO_SHIFT            (2U)
/*! VADZCDAUTO - Zero-Crossing Detector Automatic Threshold
 *  0b0..Disables
 *  0b1..Enables
 */
#define PDM_VAD0_ZCD_VADZCDAUTO(x)               (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_ZCD_VADZCDAUTO_SHIFT)) & PDM_VAD0_ZCD_VADZCDAUTO_MASK)

#define PDM_VAD0_ZCD_VADZCDAND_MASK              (0x10U)
#define PDM_VAD0_ZCD_VADZCDAND_SHIFT             (4U)
/*! VADZCDAND - Zero-Crossing Detector AND Behavior
 *  0b0..OR
 *  0b1..AND
 */
#define PDM_VAD0_ZCD_VADZCDAND(x)                (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_ZCD_VADZCDAND_SHIFT)) & PDM_VAD0_ZCD_VADZCDAND_MASK)

#define PDM_VAD0_ZCD_VADZCDADJ_MASK              (0xF00U)
#define PDM_VAD0_ZCD_VADZCDADJ_SHIFT             (8U)
/*! VADZCDADJ - Zero-Crossing Detector Adjustment
 *  0b0000..Adjustment value = 0
 *  0b0001..Adjustment value = 1
 *  0b0010-0b1110.....
 *  0b1111..Adjustment value = 15
 */
#define PDM_VAD0_ZCD_VADZCDADJ(x)                (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_ZCD_VADZCDADJ_SHIFT)) & PDM_VAD0_ZCD_VADZCDADJ_MASK)

#define PDM_VAD0_ZCD_VADZCDTH_MASK               (0x3FF0000U)
#define PDM_VAD0_ZCD_VADZCDTH_SHIFT              (16U)
/*! VADZCDTH - Zero-Crossing Detector Threshold */
#define PDM_VAD0_ZCD_VADZCDTH(x)                 (((uint32_t)(((uint32_t)(x)) << PDM_VAD0_ZCD_VADZCDTH_SHIFT)) & PDM_VAD0_ZCD_VADZCDTH_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group PDM_Register_Masks */


/* PDM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral PDM base address */
  #define PDM_BASE                                 (0x52BE0000u)
  /** Peripheral PDM base address */
  #define PDM_BASE_NS                              (0x42BE0000u)
  /** Peripheral PDM base pointer */
  #define PDM                                      ((PDM_Type *)PDM_BASE)
  /** Peripheral PDM base pointer */
  #define PDM_NS                                   ((PDM_Type *)PDM_BASE_NS)
  /** Array initializer of PDM peripheral base addresses */
  #define PDM_BASE_ADDRS                           { PDM_BASE }
  /** Array initializer of PDM peripheral base pointers */
  #define PDM_BASE_PTRS                            { PDM }
  /** Array initializer of PDM peripheral base addresses */
  #define PDM_BASE_ADDRS_NS                        { PDM_BASE_NS }
  /** Array initializer of PDM peripheral base pointers */
  #define PDM_BASE_PTRS_NS                         { PDM_NS }
#else
  /** Peripheral PDM base address */
  #define PDM_BASE                                 (0x42BE0000u)
  /** Peripheral PDM base pointer */
  #define PDM                                      ((PDM_Type *)PDM_BASE)
  /** Array initializer of PDM peripheral base addresses */
  #define PDM_BASE_ADDRS                           { PDM_BASE }
  /** Array initializer of PDM peripheral base pointers */
  #define PDM_BASE_PTRS                            { PDM }
#endif
/** Interrupt vectors for the PDM peripheral type */
#define PDM_HWVAD_Event_IRQS                     { PDM_HWVAD_EVENT_IRQn }
#define PDM_HWVAD_Error_IRQS                     { PDM_HWVAD_ERROR_IRQn }
#define PDM_Event_IRQS                           { PDM_EVENT_IRQn }
#define PDM_Error_IRQS                           { PDM_ERROR_IRQn }

/*!
 * @}
 */ /* end of group PDM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- PHY_LDO Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PHY_LDO_Peripheral_Access_Layer PHY_LDO Peripheral Access Layer
 * @{
 */

/** PHY_LDO - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0 */
    __IO uint32_t RW;                                /**< Analog Control Register CTRL0, offset: 0x0 */
    __IO uint32_t SET;                               /**< Analog Control Register CTRL0, offset: 0x4 */
    __IO uint32_t CLR;                               /**< Analog Control Register CTRL0, offset: 0x8 */
    __IO uint32_t TOG;                               /**< Analog Control Register CTRL0, offset: 0xC */
  } CTRL0;
       uint8_t RESERVED_0[64];
  struct {                                         /* offset: 0x50 */
    __I  uint32_t RW;                                /**< Analog Status Register STAT0, offset: 0x50 */
    __I  uint32_t SET;                               /**< Analog Status Register STAT0, offset: 0x54 */
    __I  uint32_t CLR;                               /**< Analog Status Register STAT0, offset: 0x58 */
    __I  uint32_t TOG;                               /**< Analog Status Register STAT0, offset: 0x5C */
  } STAT0;
} PHY_LDO_Type;

/* ----------------------------------------------------------------------------
   -- PHY_LDO Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PHY_LDO_Register_Masks PHY_LDO Register Masks
 * @{
 */

/*! @name CTRL0 - Analog Control Register CTRL0 */
/*! @{ */

#define PHY_LDO_CTRL0_LINREG_EN_MASK             (0x1U)
#define PHY_LDO_CTRL0_LINREG_EN_SHIFT            (0U)
/*! LINREG_EN - LinrReg master enable */
#define PHY_LDO_CTRL0_LINREG_EN(x)               (((uint32_t)(((uint32_t)(x)) << PHY_LDO_CTRL0_LINREG_EN_SHIFT)) & PHY_LDO_CTRL0_LINREG_EN_MASK)

#define PHY_LDO_CTRL0_LINREG_PWRUPLOAD_DIS_MASK  (0x2U)
#define PHY_LDO_CTRL0_LINREG_PWRUPLOAD_DIS_SHIFT (1U)
/*! LINREG_PWRUPLOAD_DIS - LinReg power-up load disable
 *  0b0..Internal pull-down enabled
 *  0b1..Internal pull-down disabled
 */
#define PHY_LDO_CTRL0_LINREG_PWRUPLOAD_DIS(x)    (((uint32_t)(((uint32_t)(x)) << PHY_LDO_CTRL0_LINREG_PWRUPLOAD_DIS_SHIFT)) & PHY_LDO_CTRL0_LINREG_PWRUPLOAD_DIS_MASK)

#define PHY_LDO_CTRL0_LINREG_ILIMIT_EN_MASK      (0x4U)
#define PHY_LDO_CTRL0_LINREG_ILIMIT_EN_SHIFT     (2U)
/*! LINREG_ILIMIT_EN - LinReg current-limit enable */
#define PHY_LDO_CTRL0_LINREG_ILIMIT_EN(x)        (((uint32_t)(((uint32_t)(x)) << PHY_LDO_CTRL0_LINREG_ILIMIT_EN_SHIFT)) & PHY_LDO_CTRL0_LINREG_ILIMIT_EN_MASK)

#define PHY_LDO_CTRL0_LINREG_OUTPUT_TRG_MASK     (0x1F0U)
#define PHY_LDO_CTRL0_LINREG_OUTPUT_TRG_SHIFT    (4U)
/*! LINREG_OUTPUT_TRG - LinReg output voltage target setting */
#define PHY_LDO_CTRL0_LINREG_OUTPUT_TRG(x)       (((uint32_t)(((uint32_t)(x)) << PHY_LDO_CTRL0_LINREG_OUTPUT_TRG_SHIFT)) & PHY_LDO_CTRL0_LINREG_OUTPUT_TRG_MASK)

#define PHY_LDO_CTRL0_LINREG_PHY_ISO_B_MASK      (0x8000U)
#define PHY_LDO_CTRL0_LINREG_PHY_ISO_B_SHIFT     (15U)
/*! LINREG_PHY_ISO_B - Isolation control for attached PHY load */
#define PHY_LDO_CTRL0_LINREG_PHY_ISO_B(x)        (((uint32_t)(((uint32_t)(x)) << PHY_LDO_CTRL0_LINREG_PHY_ISO_B_SHIFT)) & PHY_LDO_CTRL0_LINREG_PHY_ISO_B_MASK)
/*! @} */

/*! @name STAT0 - Analog Status Register STAT0 */
/*! @{ */

#define PHY_LDO_STAT0_LINREG_STAT_MASK           (0xFU)
#define PHY_LDO_STAT0_LINREG_STAT_SHIFT          (0U)
/*! LINREG_STAT - LinReg Status Bits */
#define PHY_LDO_STAT0_LINREG_STAT(x)             (((uint32_t)(((uint32_t)(x)) << PHY_LDO_STAT0_LINREG_STAT_SHIFT)) & PHY_LDO_STAT0_LINREG_STAT_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group PHY_LDO_Register_Masks */


/* PHY_LDO - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral PHY_LDO base address */
  #define PHY_LDO_BASE                             (0x54484680u)
  /** Peripheral PHY_LDO base address */
  #define PHY_LDO_BASE_NS                          (0x44484680u)
  /** Peripheral PHY_LDO base pointer */
  #define PHY_LDO                                  ((PHY_LDO_Type *)PHY_LDO_BASE)
  /** Peripheral PHY_LDO base pointer */
  #define PHY_LDO_NS                               ((PHY_LDO_Type *)PHY_LDO_BASE_NS)
  /** Array initializer of PHY_LDO peripheral base addresses */
  #define PHY_LDO_BASE_ADDRS                       { PHY_LDO_BASE }
  /** Array initializer of PHY_LDO peripheral base pointers */
  #define PHY_LDO_BASE_PTRS                        { PHY_LDO }
  /** Array initializer of PHY_LDO peripheral base addresses */
  #define PHY_LDO_BASE_ADDRS_NS                    { PHY_LDO_BASE_NS }
  /** Array initializer of PHY_LDO peripheral base pointers */
  #define PHY_LDO_BASE_PTRS_NS                     { PHY_LDO_NS }
#else
  /** Peripheral PHY_LDO base address */
  #define PHY_LDO_BASE                             (0x44484680u)
  /** Peripheral PHY_LDO base pointer */
  #define PHY_LDO                                  ((PHY_LDO_Type *)PHY_LDO_BASE)
  /** Array initializer of PHY_LDO peripheral base addresses */
  #define PHY_LDO_BASE_ADDRS                       { PHY_LDO_BASE }
  /** Array initializer of PHY_LDO peripheral base pointers */
  #define PHY_LDO_BASE_PTRS                        { PHY_LDO }
#endif

/*!
 * @}
 */ /* end of group PHY_LDO_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- PLL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PLL_Peripheral_Access_Layer PLL Peripheral Access Layer
 * @{
 */

/** PLL - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0 */
    __IO uint32_t RW;                                /**< Fractional PLL Control Register, offset: 0x0 */
    __IO uint32_t SET;                               /**< Fractional PLL Control Register, offset: 0x4 */
    __IO uint32_t CLR;                               /**< Fractional PLL Control Register, offset: 0x8 */
    __IO uint32_t TOG;                               /**< Fractional PLL Control Register, offset: 0xC */
  } CTRL0;
  struct {                                         /* offset: 0x10 */
    __IO uint32_t RW;                                /**< Fractional PLL Spread Spectrum Control Register, offset: 0x10 */
    __IO uint32_t SET;                               /**< Fractional PLL Spread Spectrum Control Register, offset: 0x14 */
    __IO uint32_t CLR;                               /**< Fractional PLL Spread Spectrum Control Register, offset: 0x18 */
    __IO uint32_t TOG;                               /**< Fractional PLL Spread Spectrum Control Register, offset: 0x1C */
  } SPREAD_SPECTRUM;
  struct {                                         /* offset: 0x20 */
    __IO uint32_t RW;                                /**< Fractional PLL Numerator Control Register, offset: 0x20 */
    __IO uint32_t SET;                               /**< Fractional PLL Numerator Control Register, offset: 0x24 */
    __IO uint32_t CLR;                               /**< Fractional PLL Numerator Control Register, offset: 0x28 */
    __IO uint32_t TOG;                               /**< Fractional PLL Numerator Control Register, offset: 0x2C */
  } NUMERATOR;
  struct {                                         /* offset: 0x30 */
    __IO uint32_t RW;                                /**< Fractional PLL Denominator Control Register, offset: 0x30 */
    __IO uint32_t SET;                               /**< Fractional PLL Denominator Control Register, offset: 0x34 */
    __IO uint32_t CLR;                               /**< Fractional PLL Denominator Control Register, offset: 0x38 */
    __IO uint32_t TOG;                               /**< Fractional PLL Denominator Control Register, offset: 0x3C */
  } DENOMINATOR;
} PLL_Type;

/* ----------------------------------------------------------------------------
   -- PLL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PLL_Register_Masks PLL Register Masks
 * @{
 */

/*! @name CTRL0 - Fractional PLL Control Register */
/*! @{ */

#define PLL_CTRL0_DIV_SELECT_MASK                (0x7FU)
#define PLL_CTRL0_DIV_SELECT_SHIFT               (0U)
/*! DIV_SELECT - DIV_SELECT */
#define PLL_CTRL0_DIV_SELECT(x)                  (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_DIV_SELECT_SHIFT)) & PLL_CTRL0_DIV_SELECT_MASK)

#define PLL_CTRL0_ENABLE_ALT_MASK                (0x100U)
#define PLL_CTRL0_ENABLE_ALT_SHIFT               (8U)
/*! ENABLE_ALT - ENABLE_ALT
 *  0b0..Disable the alternate clock output
 *  0b1..Enable the alternate clock output which is the output of the post_divider, and cannot be bypassed
 */
#define PLL_CTRL0_ENABLE_ALT(x)                  (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_ENABLE_ALT_SHIFT)) & PLL_CTRL0_ENABLE_ALT_MASK)

#define PLL_CTRL0_HOLD_RING_OFF_MASK             (0x2000U)
#define PLL_CTRL0_HOLD_RING_OFF_SHIFT            (13U)
/*! HOLD_RING_OFF - PLL Start up initialization
 *  0b0..Normal operation
 *  0b1..Initialize PLL start up
 */
#define PLL_CTRL0_HOLD_RING_OFF(x)               (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_HOLD_RING_OFF_SHIFT)) & PLL_CTRL0_HOLD_RING_OFF_MASK)

#define PLL_CTRL0_POWERUP_MASK                   (0x4000U)
#define PLL_CTRL0_POWERUP_SHIFT                  (14U)
/*! POWERUP - POWERUP
 *  0b1..Power Up the PLL
 *  0b0..Power down the PLL
 */
#define PLL_CTRL0_POWERUP(x)                     (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_POWERUP_SHIFT)) & PLL_CTRL0_POWERUP_MASK)

#define PLL_CTRL0_ENABLE_MASK                    (0x8000U)
#define PLL_CTRL0_ENABLE_SHIFT                   (15U)
/*! ENABLE - ENABLE
 *  0b1..Enable the clock output
 *  0b0..Disable the clock output
 */
#define PLL_CTRL0_ENABLE(x)                      (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_ENABLE_SHIFT)) & PLL_CTRL0_ENABLE_MASK)

#define PLL_CTRL0_BYPASS_MASK                    (0x10000U)
#define PLL_CTRL0_BYPASS_SHIFT                   (16U)
/*! BYPASS - BYPASS
 *  0b1..Bypass the PLL
 *  0b0..No Bypass
 */
#define PLL_CTRL0_BYPASS(x)                      (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_BYPASS_SHIFT)) & PLL_CTRL0_BYPASS_MASK)

#define PLL_CTRL0_DITHER_EN_MASK                 (0x20000U)
#define PLL_CTRL0_DITHER_EN_SHIFT                (17U)
/*! DITHER_EN - DITHER_EN
 *  0b0..Disable Dither
 *  0b1..Enable Dither
 */
#define PLL_CTRL0_DITHER_EN(x)                   (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_DITHER_EN_SHIFT)) & PLL_CTRL0_DITHER_EN_MASK)

#define PLL_CTRL0_PLL_REG_EN_MASK                (0x400000U)
#define PLL_CTRL0_PLL_REG_EN_SHIFT               (22U)
/*! PLL_REG_EN - PLL_REG_EN
 *  0b0..Disable
 *  0b1..Enable
 */
#define PLL_CTRL0_PLL_REG_EN(x)                  (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_PLL_REG_EN_SHIFT)) & PLL_CTRL0_PLL_REG_EN_MASK)

#define PLL_CTRL0_POST_DIV_SEL_MASK              (0xE000000U)
#define PLL_CTRL0_POST_DIV_SEL_SHIFT             (25U)
/*! POST_DIV_SEL - Post Divide Select
 *  0b000..Divide by 1
 *  0b001..Divide by 2
 *  0b010..Divide by 4
 *  0b011..Divide by 8
 *  0b100..Divide by 16
 *  0b101..Divide by 32
 */
#define PLL_CTRL0_POST_DIV_SEL(x)                (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_POST_DIV_SEL_SHIFT)) & PLL_CTRL0_POST_DIV_SEL_MASK)

#define PLL_CTRL0_BIAS_SELECT_MASK               (0x20000000U)
#define PLL_CTRL0_BIAS_SELECT_SHIFT              (29U)
/*! BIAS_SELECT - BIAS_SELECT
 *  0b0..Used in SoCs with a bias current of 10uA
 *  0b1..Used in SoCs with a bias current of 2uA
 */
#define PLL_CTRL0_BIAS_SELECT(x)                 (((uint32_t)(((uint32_t)(x)) << PLL_CTRL0_BIAS_SELECT_SHIFT)) & PLL_CTRL0_BIAS_SELECT_MASK)
/*! @} */

/*! @name SPREAD_SPECTRUM - Fractional PLL Spread Spectrum Control Register */
/*! @{ */

#define PLL_SPREAD_SPECTRUM_STEP_MASK            (0x7FFFU)
#define PLL_SPREAD_SPECTRUM_STEP_SHIFT           (0U)
/*! STEP - Step */
#define PLL_SPREAD_SPECTRUM_STEP(x)              (((uint32_t)(((uint32_t)(x)) << PLL_SPREAD_SPECTRUM_STEP_SHIFT)) & PLL_SPREAD_SPECTRUM_STEP_MASK)

#define PLL_SPREAD_SPECTRUM_ENABLE_MASK          (0x8000U)
#define PLL_SPREAD_SPECTRUM_ENABLE_SHIFT         (15U)
/*! ENABLE - Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define PLL_SPREAD_SPECTRUM_ENABLE(x)            (((uint32_t)(((uint32_t)(x)) << PLL_SPREAD_SPECTRUM_ENABLE_SHIFT)) & PLL_SPREAD_SPECTRUM_ENABLE_MASK)

#define PLL_SPREAD_SPECTRUM_STOP_MASK            (0xFFFF0000U)
#define PLL_SPREAD_SPECTRUM_STOP_SHIFT           (16U)
/*! STOP - Stop */
#define PLL_SPREAD_SPECTRUM_STOP(x)              (((uint32_t)(((uint32_t)(x)) << PLL_SPREAD_SPECTRUM_STOP_SHIFT)) & PLL_SPREAD_SPECTRUM_STOP_MASK)
/*! @} */

/*! @name NUMERATOR - Fractional PLL Numerator Control Register */
/*! @{ */

#define PLL_NUMERATOR_NUM_MASK                   (0x3FFFFFFFU)
#define PLL_NUMERATOR_NUM_SHIFT                  (0U)
/*! NUM - Numerator */
#define PLL_NUMERATOR_NUM(x)                     (((uint32_t)(((uint32_t)(x)) << PLL_NUMERATOR_NUM_SHIFT)) & PLL_NUMERATOR_NUM_MASK)
/*! @} */

/*! @name DENOMINATOR - Fractional PLL Denominator Control Register */
/*! @{ */

#define PLL_DENOMINATOR_DENOM_MASK               (0x3FFFFFFFU)
#define PLL_DENOMINATOR_DENOM_SHIFT              (0U)
/*! DENOM - Denominator */
#define PLL_DENOMINATOR_DENOM(x)                 (((uint32_t)(((uint32_t)(x)) << PLL_DENOMINATOR_DENOM_SHIFT)) & PLL_DENOMINATOR_DENOM_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group PLL_Register_Masks */


/* PLL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral AUDIO_PLL base address */
  #define AUDIO_PLL_BASE                           (0x54484280u)
  /** Peripheral AUDIO_PLL base address */
  #define AUDIO_PLL_BASE_NS                        (0x44484280u)
  /** Peripheral AUDIO_PLL base pointer */
  #define AUDIO_PLL                                ((PLL_Type *)AUDIO_PLL_BASE)
  /** Peripheral AUDIO_PLL base pointer */
  #define AUDIO_PLL_NS                             ((PLL_Type *)AUDIO_PLL_BASE_NS)
  /** Peripheral ETHERNET_PLL base address */
  #define ETHERNET_PLL_BASE                        (0x54484180u)
  /** Peripheral ETHERNET_PLL base address */
  #define ETHERNET_PLL_BASE_NS                     (0x44484180u)
  /** Peripheral ETHERNET_PLL base pointer */
  #define ETHERNET_PLL                             ((PLL_Type *)ETHERNET_PLL_BASE)
  /** Peripheral ETHERNET_PLL base pointer */
  #define ETHERNET_PLL_NS                          ((PLL_Type *)ETHERNET_PLL_BASE_NS)
  /** Array initializer of PLL peripheral base addresses */
  #define PLL_BASE_ADDRS                           { AUDIO_PLL_BASE, ETHERNET_PLL_BASE }
  /** Array initializer of PLL peripheral base pointers */
  #define PLL_BASE_PTRS                            { AUDIO_PLL, ETHERNET_PLL }
  /** Array initializer of PLL peripheral base addresses */
  #define PLL_BASE_ADDRS_NS                        { AUDIO_PLL_BASE_NS, ETHERNET_PLL_BASE_NS }
  /** Array initializer of PLL peripheral base pointers */
  #define PLL_BASE_PTRS_NS                         { AUDIO_PLL_NS, ETHERNET_PLL_NS }
#else
  /** Peripheral AUDIO_PLL base address */
  #define AUDIO_PLL_BASE                           (0x44484280u)
  /** Peripheral AUDIO_PLL base pointer */
  #define AUDIO_PLL                                ((PLL_Type *)AUDIO_PLL_BASE)
  /** Peripheral ETHERNET_PLL base address */
  #define ETHERNET_PLL_BASE                        (0x44484180u)
  /** Peripheral ETHERNET_PLL base pointer */
  #define ETHERNET_PLL                             ((PLL_Type *)ETHERNET_PLL_BASE)
  /** Array initializer of PLL peripheral base addresses */
  #define PLL_BASE_ADDRS                           { AUDIO_PLL_BASE, ETHERNET_PLL_BASE }
  /** Array initializer of PLL peripheral base pointers */
  #define PLL_BASE_PTRS                            { AUDIO_PLL, ETHERNET_PLL }
#endif

/*!
 * @}
 */ /* end of group PLL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- PWM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PWM_Peripheral_Access_Layer PWM Peripheral Access Layer
 * @{
 */

/** PWM - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x60 */
    __I  uint16_t CNT;                               /**< Counter Register, array offset: 0x0, array step: 0x60 */
    __IO uint16_t INIT;                              /**< Initial Count Register, array offset: 0x2, array step: 0x60 */
    __IO uint16_t CTRL2;                             /**< Control 2 Register, array offset: 0x4, array step: 0x60 */
    __IO uint16_t CTRL;                              /**< Control Register, array offset: 0x6, array step: 0x60 */
         uint8_t RESERVED_0[2];
    __IO uint16_t VAL0;                              /**< Value Register 0, array offset: 0xA, array step: 0x60 */
    __IO uint16_t FRACVAL1;                          /**< Fractional Value Register 1, array offset: 0xC, array step: 0x60 */
    __IO uint16_t VAL1;                              /**< Value Register 1, array offset: 0xE, array step: 0x60 */
    __IO uint16_t FRACVAL2;                          /**< Fractional Value Register 2, array offset: 0x10, array step: 0x60 */
    __IO uint16_t VAL2;                              /**< Value Register 2, array offset: 0x12, array step: 0x60 */
    __IO uint16_t FRACVAL3;                          /**< Fractional Value Register 3, array offset: 0x14, array step: 0x60 */
    __IO uint16_t VAL3;                              /**< Value Register 3, array offset: 0x16, array step: 0x60 */
    __IO uint16_t FRACVAL4;                          /**< Fractional Value Register 4, array offset: 0x18, array step: 0x60 */
    __IO uint16_t VAL4;                              /**< Value Register 4, array offset: 0x1A, array step: 0x60 */
    __IO uint16_t FRACVAL5;                          /**< Fractional Value Register 5, array offset: 0x1C, array step: 0x60 */
    __IO uint16_t VAL5;                              /**< Value Register 5, array offset: 0x1E, array step: 0x60 */
    __IO uint16_t FRCTRL;                            /**< Fractional Control Register, array offset: 0x20, array step: 0x60 */
    __IO uint16_t OCTRL;                             /**< Output Control Register, array offset: 0x22, array step: 0x60 */
    __IO uint16_t STS;                               /**< Status Register, array offset: 0x24, array step: 0x60 */
    __IO uint16_t INTEN;                             /**< Interrupt Enable Register, array offset: 0x26, array step: 0x60 */
    __IO uint16_t DMAEN;                             /**< DMA Enable Register, array offset: 0x28, array step: 0x60 */
    __IO uint16_t TCTRL;                             /**< Output Trigger Control Register, array offset: 0x2A, array step: 0x60 */
    __IO uint16_t DISMAP[1];                         /**< Fault Disable Mapping Register 0, array offset: 0x2C, array step: index*0x60, index2*0x2 */
         uint8_t RESERVED_1[2];
    __IO uint16_t DTCNT0;                            /**< Deadtime Count Register 0, array offset: 0x30, array step: 0x60 */
    __IO uint16_t DTCNT1;                            /**< Deadtime Count Register 1, array offset: 0x32, array step: 0x60 */
    __IO uint16_t CAPTCTRLA;                         /**< Capture Control A Register, array offset: 0x34, array step: 0x60 */
    __IO uint16_t CAPTCOMPA;                         /**< Capture Compare A Register, array offset: 0x36, array step: 0x60 */
    __IO uint16_t CAPTCTRLB;                         /**< Capture Control B Register, array offset: 0x38, array step: 0x60 */
    __IO uint16_t CAPTCOMPB;                         /**< Capture Compare B Register, array offset: 0x3A, array step: 0x60 */
    __IO uint16_t CAPTCTRLX;                         /**< Capture Control X Register, array offset: 0x3C, array step: 0x60 */
    __IO uint16_t CAPTCOMPX;                         /**< Capture Compare X Register, array offset: 0x3E, array step: 0x60 */
    __I  uint16_t CVAL0;                             /**< Capture Value 0 Register, array offset: 0x40, array step: 0x60 */
    __I  uint16_t CVAL0CYC;                          /**< Capture Value 0 Cycle Register, array offset: 0x42, array step: 0x60 */
    __I  uint16_t CVAL1;                             /**< Capture Value 1 Register, array offset: 0x44, array step: 0x60 */
    __I  uint16_t CVAL1CYC;                          /**< Capture Value 1 Cycle Register, array offset: 0x46, array step: 0x60 */
    __I  uint16_t CVAL2;                             /**< Capture Value 2 Register, array offset: 0x48, array step: 0x60 */
    __I  uint16_t CVAL2CYC;                          /**< Capture Value 2 Cycle Register, array offset: 0x4A, array step: 0x60 */
    __I  uint16_t CVAL3;                             /**< Capture Value 3 Register, array offset: 0x4C, array step: 0x60 */
    __I  uint16_t CVAL3CYC;                          /**< Capture Value 3 Cycle Register, array offset: 0x4E, array step: 0x60 */
    __I  uint16_t CVAL4;                             /**< Capture Value 4 Register, array offset: 0x50, array step: 0x60 */
    __I  uint16_t CVAL4CYC;                          /**< Capture Value 4 Cycle Register, array offset: 0x52, array step: 0x60 */
    __I  uint16_t CVAL5;                             /**< Capture Value 5 Register, array offset: 0x54, array step: 0x60 */
    __I  uint16_t CVAL5CYC;                          /**< Capture Value 5 Cycle Register, array offset: 0x56, array step: 0x60 */
    __IO uint16_t PHASEDLY;                          /**< Phase Delay Register, array offset: 0x58, array step: 0x60, valid indices: [1-3] */
    __IO uint16_t CAPTFILTA;                         /**< Capture PWM_A Input Filter Register, array offset: 0x5A, array step: 0x60 */
    __IO uint16_t CAPTFILTB;                         /**< Capture PWM_B Input Filter Register, array offset: 0x5C, array step: 0x60 */
    __IO uint16_t CAPTFILTX;                         /**< Capture PWM_X Input Filter Register, array offset: 0x5E, array step: 0x60 */
  } SM[4];
  __IO uint16_t OUTEN;                             /**< Output Enable Register, offset: 0x180 */
  __IO uint16_t MASK;                              /**< Mask Register, offset: 0x182 */
  __IO uint16_t SWCOUT;                            /**< Software Controlled Output Register, offset: 0x184 */
  __IO uint16_t DTSRCSEL;                          /**< PWM Source Select Register, offset: 0x186 */
  __IO uint16_t MCTRL;                             /**< Master Control Register, offset: 0x188 */
  __IO uint16_t MCTRL2;                            /**< Master Control 2 Register, offset: 0x18A */
  __IO uint16_t FCTRL;                             /**< Fault Control Register, offset: 0x18C */
  __IO uint16_t FSTS;                              /**< Fault Status Register, offset: 0x18E */
  __IO uint16_t FFILT;                             /**< Fault Filter Register, offset: 0x190 */
  __IO uint16_t FTST;                              /**< Fault Test Register, offset: 0x192 */
  __IO uint16_t FCTRL2;                            /**< Fault Control 2 Register, offset: 0x194 */
} PWM_Type;

/* ----------------------------------------------------------------------------
   -- PWM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup PWM_Register_Masks PWM Register Masks
 * @{
 */

/*! @name CNT - Counter Register */
/*! @{ */

#define PWM_CNT_CNT_MASK                         (0xFFFFU)
#define PWM_CNT_CNT_SHIFT                        (0U)
/*! CNT - Counter Register Bits */
#define PWM_CNT_CNT(x)                           (((uint16_t)(((uint16_t)(x)) << PWM_CNT_CNT_SHIFT)) & PWM_CNT_CNT_MASK)
/*! @} */

/* The count of PWM_CNT */
#define PWM_CNT_COUNT                            (4U)

/*! @name INIT - Initial Count Register */
/*! @{ */

#define PWM_INIT_INIT_MASK                       (0xFFFFU)
#define PWM_INIT_INIT_SHIFT                      (0U)
/*! INIT - Initial Count Register Bits */
#define PWM_INIT_INIT(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_INIT_INIT_SHIFT)) & PWM_INIT_INIT_MASK)
/*! @} */

/* The count of PWM_INIT */
#define PWM_INIT_COUNT                           (4U)

/*! @name CTRL2 - Control 2 Register */
/*! @{ */

#define PWM_CTRL2_CLK_SEL_MASK                   (0x3U)
#define PWM_CTRL2_CLK_SEL_SHIFT                  (0U)
/*! CLK_SEL - Clock Source Select
 *  0b00..The IPBus clock is used as the clock for the local prescaler and counter.
 *  0b01..EXT_CLK is used as the clock for the local prescaler and counter.
 *  0b10..Submodule 0's clock (AUX_CLK) is used as the source clock for the local prescaler and counter. This
 *        setting should not be used in submodule 0 as it forces the clock to logic 0.
 *  0b11..Reserved
 */
#define PWM_CTRL2_CLK_SEL(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_CLK_SEL_SHIFT)) & PWM_CTRL2_CLK_SEL_MASK)

#define PWM_CTRL2_RELOAD_SEL_MASK                (0x4U)
#define PWM_CTRL2_RELOAD_SEL_SHIFT               (2U)
/*! RELOAD_SEL - Reload Source Select
 *  0b0..The local RELOAD signal is used to reload registers.
 *  0b1..The master RELOAD signal (from submodule 0) is used to reload registers. This setting should not be used
 *       in submodule 0 as it forces the RELOAD signal to logic 0.
 */
#define PWM_CTRL2_RELOAD_SEL(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_RELOAD_SEL_SHIFT)) & PWM_CTRL2_RELOAD_SEL_MASK)

#define PWM_CTRL2_FORCE_SEL_MASK                 (0x38U)
#define PWM_CTRL2_FORCE_SEL_SHIFT                (3U)
/*! FORCE_SEL - Force Select
 *  0b000..The local force signal, CTRL2[FORCE], from this submodule is used to force updates.
 *  0b001..The master force signal from submodule 0 is used to force updates. This setting should not be used in
 *         submodule 0 as it holds the FORCE OUTPUT signal to logic 0.
 *  0b010..The local reload signal from this submodule is used to force updates without regard to the state of LDOK.
 *  0b011..The master reload signal from submodule0 is used to force updates if LDOK is set. This setting should
 *         not be used in submodule0 as it holds the FORCE OUTPUT signal to logic 0.
 *  0b100..The local sync signal from this submodule is used to force updates.
 *  0b101..The master sync signal from submodule0 is used to force updates. This setting should not be used in
 *         submodule0 as it holds the FORCE OUTPUT signal to logic 0.
 *  0b110..The external force signal, EXT_FORCE, from outside the PWM module causes updates.
 *  0b111..The external sync signal, EXT_SYNC, from outside the PWM module causes updates.
 */
#define PWM_CTRL2_FORCE_SEL(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_FORCE_SEL_SHIFT)) & PWM_CTRL2_FORCE_SEL_MASK)

#define PWM_CTRL2_FORCE_MASK                     (0x40U)
#define PWM_CTRL2_FORCE_SHIFT                    (6U)
/*! FORCE - Force Initialization */
#define PWM_CTRL2_FORCE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_FORCE_SHIFT)) & PWM_CTRL2_FORCE_MASK)

#define PWM_CTRL2_FRCEN_MASK                     (0x80U)
#define PWM_CTRL2_FRCEN_SHIFT                    (7U)
/*! FRCEN - Force Enable
 *  0b0..Initialization from a FORCE_OUT is disabled.
 *  0b1..Initialization from a FORCE_OUT is enabled.
 */
#define PWM_CTRL2_FRCEN(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_FRCEN_SHIFT)) & PWM_CTRL2_FRCEN_MASK)

#define PWM_CTRL2_INIT_SEL_MASK                  (0x300U)
#define PWM_CTRL2_INIT_SEL_SHIFT                 (8U)
/*! INIT_SEL - Initialization Control Select
 *  0b00..Local sync (PWM_X) causes initialization.
 *  0b01..Master reload from submodule 0 causes initialization. This setting should not be used in submodule 0 as
 *        it forces the INIT signal to logic 0. The submodule counter will only re-initialize when a master reload
 *        occurs.
 *  0b10..Master sync from submodule 0 causes initialization. This setting should not be used in submodule 0 as it forces the INIT signal to logic 0.
 *  0b11..EXT_SYNC causes initialization.
 */
#define PWM_CTRL2_INIT_SEL(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_INIT_SEL_SHIFT)) & PWM_CTRL2_INIT_SEL_MASK)

#define PWM_CTRL2_PWMX_INIT_MASK                 (0x400U)
#define PWM_CTRL2_PWMX_INIT_SHIFT                (10U)
/*! PWMX_INIT - PWM_X Initial Value */
#define PWM_CTRL2_PWMX_INIT(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_PWMX_INIT_SHIFT)) & PWM_CTRL2_PWMX_INIT_MASK)

#define PWM_CTRL2_PWM45_INIT_MASK                (0x800U)
#define PWM_CTRL2_PWM45_INIT_SHIFT               (11U)
/*! PWM45_INIT - PWM45 Initial Value */
#define PWM_CTRL2_PWM45_INIT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_PWM45_INIT_SHIFT)) & PWM_CTRL2_PWM45_INIT_MASK)

#define PWM_CTRL2_PWM23_INIT_MASK                (0x1000U)
#define PWM_CTRL2_PWM23_INIT_SHIFT               (12U)
/*! PWM23_INIT - PWM23 Initial Value */
#define PWM_CTRL2_PWM23_INIT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_PWM23_INIT_SHIFT)) & PWM_CTRL2_PWM23_INIT_MASK)

#define PWM_CTRL2_INDEP_MASK                     (0x2000U)
#define PWM_CTRL2_INDEP_SHIFT                    (13U)
/*! INDEP - Independent or Complementary Pair Operation
 *  0b0..PWM_A and PWM_B form a complementary PWM pair.
 *  0b1..PWM_A and PWM_B outputs are independent PWMs.
 */
#define PWM_CTRL2_INDEP(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_INDEP_SHIFT)) & PWM_CTRL2_INDEP_MASK)

#define PWM_CTRL2_WAITEN_MASK                    (0x4000U)
#define PWM_CTRL2_WAITEN_SHIFT                   (14U)
/*! WAITEN - Wait Enable */
#define PWM_CTRL2_WAITEN(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_WAITEN_SHIFT)) & PWM_CTRL2_WAITEN_MASK)

#define PWM_CTRL2_DBGEN_MASK                     (0x8000U)
#define PWM_CTRL2_DBGEN_SHIFT                    (15U)
/*! DBGEN - Debug Enable */
#define PWM_CTRL2_DBGEN(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_CTRL2_DBGEN_SHIFT)) & PWM_CTRL2_DBGEN_MASK)
/*! @} */

/* The count of PWM_CTRL2 */
#define PWM_CTRL2_COUNT                          (4U)

/*! @name CTRL - Control Register */
/*! @{ */

#define PWM_CTRL_DBLEN_MASK                      (0x1U)
#define PWM_CTRL_DBLEN_SHIFT                     (0U)
/*! DBLEN - Double Switching Enable
 *  0b0..Double switching disabled.
 *  0b1..Double switching enabled.
 */
#define PWM_CTRL_DBLEN(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_DBLEN_SHIFT)) & PWM_CTRL_DBLEN_MASK)

#define PWM_CTRL_DBLX_MASK                       (0x2U)
#define PWM_CTRL_DBLX_SHIFT                      (1U)
/*! DBLX - PWM_X Double Switching Enable
 *  0b0..PWM_X double pulse disabled.
 *  0b1..PWM_X double pulse enabled.
 */
#define PWM_CTRL_DBLX(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_DBLX_SHIFT)) & PWM_CTRL_DBLX_MASK)

#define PWM_CTRL_LDMOD_MASK                      (0x4U)
#define PWM_CTRL_LDMOD_SHIFT                     (2U)
/*! LDMOD - Load Mode Select
 *  0b0..Buffered registers of this submodule are loaded and take effect at the next PWM reload if MCTRL[LDOK] is set.
 *  0b1..Buffered registers of this submodule are loaded and take effect immediately upon MCTRL[LDOK] being set.
 *       In this case, it is not necessary to set CTRL[FULL] or CTRL[HALF].
 */
#define PWM_CTRL_LDMOD(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_LDMOD_SHIFT)) & PWM_CTRL_LDMOD_MASK)

#define PWM_CTRL_SPLIT_MASK                      (0x8U)
#define PWM_CTRL_SPLIT_SHIFT                     (3U)
/*! SPLIT - Split the DBLPWM signal to PWM_A and PWM_B
 *  0b0..DBLPWM is not split. PWM_A and PWM_B each have double pulses.
 *  0b1..DBLPWM is split to PWM_A and PWM_B.
 */
#define PWM_CTRL_SPLIT(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_SPLIT_SHIFT)) & PWM_CTRL_SPLIT_MASK)

#define PWM_CTRL_PRSC_MASK                       (0x70U)
#define PWM_CTRL_PRSC_SHIFT                      (4U)
/*! PRSC - Prescaler
 *  0b000..Prescaler 1
 *  0b001..Prescaler 2
 *  0b010..Prescaler 4
 *  0b011..Prescaler 8
 *  0b100..Prescaler 16
 *  0b101..Prescaler 32
 *  0b110..Prescaler 64
 *  0b111..Prescaler 128
 */
#define PWM_CTRL_PRSC(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_PRSC_SHIFT)) & PWM_CTRL_PRSC_MASK)

#define PWM_CTRL_COMPMODE_MASK                   (0x80U)
#define PWM_CTRL_COMPMODE_SHIFT                  (7U)
/*! COMPMODE - Compare Mode
 *  0b0..The VAL* registers and the PWM counter are compared using an "equal to" method. This means that PWM edges
 *       are only produced when the counter is equal to one of the VAL* register values. This implies that a PWM_A
 *       output that is high at the end of a period maintains this state until a match with VAL3 clears the output
 *       in the following period.
 *  0b1..The VAL* registers and the PWM counter are compared using an "equal to or greater than" method. This
 *       means that PWM edges are produced when the counter is equal to or greater than one of the VAL* register
 *       values. This implies that a PWM_A output that is high at the end of a period could go low at the start of the
 *       next period if the starting counter value is greater than (but not necessarily equal to) the new VAL3 value.
 */
#define PWM_CTRL_COMPMODE(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_COMPMODE_SHIFT)) & PWM_CTRL_COMPMODE_MASK)

#define PWM_CTRL_DT_MASK                         (0x300U)
#define PWM_CTRL_DT_SHIFT                        (8U)
/*! DT - Deadtime */
#define PWM_CTRL_DT(x)                           (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_DT_SHIFT)) & PWM_CTRL_DT_MASK)

#define PWM_CTRL_FULL_MASK                       (0x400U)
#define PWM_CTRL_FULL_SHIFT                      (10U)
/*! FULL - Full Cycle Reload
 *  0b0..Full-cycle reloads disabled.
 *  0b1..Full-cycle reloads enabled.
 */
#define PWM_CTRL_FULL(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_FULL_SHIFT)) & PWM_CTRL_FULL_MASK)

#define PWM_CTRL_HALF_MASK                       (0x800U)
#define PWM_CTRL_HALF_SHIFT                      (11U)
/*! HALF - Half Cycle Reload
 *  0b0..Half-cycle reloads disabled.
 *  0b1..Half-cycle reloads enabled.
 */
#define PWM_CTRL_HALF(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_HALF_SHIFT)) & PWM_CTRL_HALF_MASK)

#define PWM_CTRL_LDFQ_MASK                       (0xF000U)
#define PWM_CTRL_LDFQ_SHIFT                      (12U)
/*! LDFQ - Load Frequency
 *  0b0000..Every PWM opportunity
 *  0b0001..Every 2 PWM opportunities
 *  0b0010..Every 3 PWM opportunities
 *  0b0011..Every 4 PWM opportunities
 *  0b0100..Every 5 PWM opportunities
 *  0b0101..Every 6 PWM opportunities
 *  0b0110..Every 7 PWM opportunities
 *  0b0111..Every 8 PWM opportunities
 *  0b1000..Every 9 PWM opportunities
 *  0b1001..Every 10 PWM opportunities
 *  0b1010..Every 11 PWM opportunities
 *  0b1011..Every 12 PWM opportunities
 *  0b1100..Every 13 PWM opportunities
 *  0b1101..Every 14 PWM opportunities
 *  0b1110..Every 15 PWM opportunities
 *  0b1111..Every 16 PWM opportunities
 */
#define PWM_CTRL_LDFQ(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_CTRL_LDFQ_SHIFT)) & PWM_CTRL_LDFQ_MASK)
/*! @} */

/* The count of PWM_CTRL */
#define PWM_CTRL_COUNT                           (4U)

/*! @name VAL0 - Value Register 0 */
/*! @{ */

#define PWM_VAL0_VAL0_MASK                       (0xFFFFU)
#define PWM_VAL0_VAL0_SHIFT                      (0U)
/*! VAL0 - Value 0 */
#define PWM_VAL0_VAL0(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_VAL0_VAL0_SHIFT)) & PWM_VAL0_VAL0_MASK)
/*! @} */

/* The count of PWM_VAL0 */
#define PWM_VAL0_COUNT                           (4U)

/*! @name FRACVAL1 - Fractional Value Register 1 */
/*! @{ */

#define PWM_FRACVAL1_FRACVAL1_MASK               (0xF800U)
#define PWM_FRACVAL1_FRACVAL1_SHIFT              (11U)
/*! FRACVAL1 - Fractional Value 1 */
#define PWM_FRACVAL1_FRACVAL1(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_FRACVAL1_FRACVAL1_SHIFT)) & PWM_FRACVAL1_FRACVAL1_MASK)
/*! @} */

/* The count of PWM_FRACVAL1 */
#define PWM_FRACVAL1_COUNT                       (4U)

/*! @name VAL1 - Value Register 1 */
/*! @{ */

#define PWM_VAL1_VAL1_MASK                       (0xFFFFU)
#define PWM_VAL1_VAL1_SHIFT                      (0U)
/*! VAL1 - Value 1 */
#define PWM_VAL1_VAL1(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_VAL1_VAL1_SHIFT)) & PWM_VAL1_VAL1_MASK)
/*! @} */

/* The count of PWM_VAL1 */
#define PWM_VAL1_COUNT                           (4U)

/*! @name FRACVAL2 - Fractional Value Register 2 */
/*! @{ */

#define PWM_FRACVAL2_FRACVAL2_MASK               (0xF800U)
#define PWM_FRACVAL2_FRACVAL2_SHIFT              (11U)
/*! FRACVAL2 - Fractional Value 2 */
#define PWM_FRACVAL2_FRACVAL2(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_FRACVAL2_FRACVAL2_SHIFT)) & PWM_FRACVAL2_FRACVAL2_MASK)
/*! @} */

/* The count of PWM_FRACVAL2 */
#define PWM_FRACVAL2_COUNT                       (4U)

/*! @name VAL2 - Value Register 2 */
/*! @{ */

#define PWM_VAL2_VAL2_MASK                       (0xFFFFU)
#define PWM_VAL2_VAL2_SHIFT                      (0U)
/*! VAL2 - Value 2 */
#define PWM_VAL2_VAL2(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_VAL2_VAL2_SHIFT)) & PWM_VAL2_VAL2_MASK)
/*! @} */

/* The count of PWM_VAL2 */
#define PWM_VAL2_COUNT                           (4U)

/*! @name FRACVAL3 - Fractional Value Register 3 */
/*! @{ */

#define PWM_FRACVAL3_FRACVAL3_MASK               (0xF800U)
#define PWM_FRACVAL3_FRACVAL3_SHIFT              (11U)
/*! FRACVAL3 - Fractional Value 3 */
#define PWM_FRACVAL3_FRACVAL3(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_FRACVAL3_FRACVAL3_SHIFT)) & PWM_FRACVAL3_FRACVAL3_MASK)
/*! @} */

/* The count of PWM_FRACVAL3 */
#define PWM_FRACVAL3_COUNT                       (4U)

/*! @name VAL3 - Value Register 3 */
/*! @{ */

#define PWM_VAL3_VAL3_MASK                       (0xFFFFU)
#define PWM_VAL3_VAL3_SHIFT                      (0U)
/*! VAL3 - Value 3 */
#define PWM_VAL3_VAL3(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_VAL3_VAL3_SHIFT)) & PWM_VAL3_VAL3_MASK)
/*! @} */

/* The count of PWM_VAL3 */
#define PWM_VAL3_COUNT                           (4U)

/*! @name FRACVAL4 - Fractional Value Register 4 */
/*! @{ */

#define PWM_FRACVAL4_FRACVAL4_MASK               (0xF800U)
#define PWM_FRACVAL4_FRACVAL4_SHIFT              (11U)
/*! FRACVAL4 - Fractional Value 4 */
#define PWM_FRACVAL4_FRACVAL4(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_FRACVAL4_FRACVAL4_SHIFT)) & PWM_FRACVAL4_FRACVAL4_MASK)
/*! @} */

/* The count of PWM_FRACVAL4 */
#define PWM_FRACVAL4_COUNT                       (4U)

/*! @name VAL4 - Value Register 4 */
/*! @{ */

#define PWM_VAL4_VAL4_MASK                       (0xFFFFU)
#define PWM_VAL4_VAL4_SHIFT                      (0U)
/*! VAL4 - Value 4 */
#define PWM_VAL4_VAL4(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_VAL4_VAL4_SHIFT)) & PWM_VAL4_VAL4_MASK)
/*! @} */

/* The count of PWM_VAL4 */
#define PWM_VAL4_COUNT                           (4U)

/*! @name FRACVAL5 - Fractional Value Register 5 */
/*! @{ */

#define PWM_FRACVAL5_FRACVAL5_MASK               (0xF800U)
#define PWM_FRACVAL5_FRACVAL5_SHIFT              (11U)
/*! FRACVAL5 - Fractional Value 5 */
#define PWM_FRACVAL5_FRACVAL5(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_FRACVAL5_FRACVAL5_SHIFT)) & PWM_FRACVAL5_FRACVAL5_MASK)
/*! @} */

/* The count of PWM_FRACVAL5 */
#define PWM_FRACVAL5_COUNT                       (4U)

/*! @name VAL5 - Value Register 5 */
/*! @{ */

#define PWM_VAL5_VAL5_MASK                       (0xFFFFU)
#define PWM_VAL5_VAL5_SHIFT                      (0U)
/*! VAL5 - Value 5 */
#define PWM_VAL5_VAL5(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_VAL5_VAL5_SHIFT)) & PWM_VAL5_VAL5_MASK)
/*! @} */

/* The count of PWM_VAL5 */
#define PWM_VAL5_COUNT                           (4U)

/*! @name FRCTRL - Fractional Control Register */
/*! @{ */

#define PWM_FRCTRL_FRAC1_EN_MASK                 (0x2U)
#define PWM_FRCTRL_FRAC1_EN_SHIFT                (1U)
/*! FRAC1_EN - Fractional Cycle PWM Period Enable
 *  0b0..Disable fractional cycle length for the PWM period.
 *  0b1..Enable fractional cycle length for the PWM period.
 */
#define PWM_FRCTRL_FRAC1_EN(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_FRCTRL_FRAC1_EN_SHIFT)) & PWM_FRCTRL_FRAC1_EN_MASK)

#define PWM_FRCTRL_FRAC23_EN_MASK                (0x4U)
#define PWM_FRCTRL_FRAC23_EN_SHIFT               (2U)
/*! FRAC23_EN - Fractional Cycle Placement Enable for PWM_A
 *  0b0..Disable fractional cycle placement for PWM_A.
 *  0b1..Enable fractional cycle placement for PWM_A.
 */
#define PWM_FRCTRL_FRAC23_EN(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_FRCTRL_FRAC23_EN_SHIFT)) & PWM_FRCTRL_FRAC23_EN_MASK)

#define PWM_FRCTRL_FRAC45_EN_MASK                (0x10U)
#define PWM_FRCTRL_FRAC45_EN_SHIFT               (4U)
/*! FRAC45_EN - Fractional Cycle Placement Enable for PWM_B
 *  0b0..Disable fractional cycle placement for PWM_B.
 *  0b1..Enable fractional cycle placement for PWM_B.
 */
#define PWM_FRCTRL_FRAC45_EN(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_FRCTRL_FRAC45_EN_SHIFT)) & PWM_FRCTRL_FRAC45_EN_MASK)

#define PWM_FRCTRL_TEST_MASK                     (0x8000U)
#define PWM_FRCTRL_TEST_SHIFT                    (15U)
/*! TEST - Test Status Bit */
#define PWM_FRCTRL_TEST(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_FRCTRL_TEST_SHIFT)) & PWM_FRCTRL_TEST_MASK)
/*! @} */

/* The count of PWM_FRCTRL */
#define PWM_FRCTRL_COUNT                         (4U)

/*! @name OCTRL - Output Control Register */
/*! @{ */

#define PWM_OCTRL_PWMXFS_MASK                    (0x3U)
#define PWM_OCTRL_PWMXFS_SHIFT                   (0U)
/*! PWMXFS - PWM_X Fault State
 *  0b00..Output is forced to logic 0 state prior to consideration of output polarity control.
 *  0b01..Output is forced to logic 1 state prior to consideration of output polarity control.
 *  0b10, 0b11..Output is put in a high-impedance state.
 */
#define PWM_OCTRL_PWMXFS(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_PWMXFS_SHIFT)) & PWM_OCTRL_PWMXFS_MASK)

#define PWM_OCTRL_PWMBFS_MASK                    (0xCU)
#define PWM_OCTRL_PWMBFS_SHIFT                   (2U)
/*! PWMBFS - PWM_B Fault State
 *  0b00..Output is forced to logic 0 state prior to consideration of output polarity control.
 *  0b01..Output is forced to logic 1 state prior to consideration of output polarity control.
 *  0b10, 0b11..Output is put in a high-impedance state.
 */
#define PWM_OCTRL_PWMBFS(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_PWMBFS_SHIFT)) & PWM_OCTRL_PWMBFS_MASK)

#define PWM_OCTRL_PWMAFS_MASK                    (0x30U)
#define PWM_OCTRL_PWMAFS_SHIFT                   (4U)
/*! PWMAFS - PWM_A Fault State
 *  0b00..Output is forced to logic 0 state prior to consideration of output polarity control.
 *  0b01..Output is forced to logic 1 state prior to consideration of output polarity control.
 *  0b10, 0b11..Output is put in a high-impedance state.
 */
#define PWM_OCTRL_PWMAFS(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_PWMAFS_SHIFT)) & PWM_OCTRL_PWMAFS_MASK)

#define PWM_OCTRL_POLX_MASK                      (0x100U)
#define PWM_OCTRL_POLX_SHIFT                     (8U)
/*! POLX - PWM_X Output Polarity
 *  0b0..PWM_X output not inverted. A high level on the PWM_X pin represents the "on" or "active" state.
 *  0b1..PWM_X output inverted. A low level on the PWM_X pin represents the "on" or "active" state.
 */
#define PWM_OCTRL_POLX(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_POLX_SHIFT)) & PWM_OCTRL_POLX_MASK)

#define PWM_OCTRL_POLB_MASK                      (0x200U)
#define PWM_OCTRL_POLB_SHIFT                     (9U)
/*! POLB - PWM_B Output Polarity
 *  0b0..PWM_B output not inverted. A high level on the PWM_B pin represents the "on" or "active" state.
 *  0b1..PWM_B output inverted. A low level on the PWM_B pin represents the "on" or "active" state.
 */
#define PWM_OCTRL_POLB(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_POLB_SHIFT)) & PWM_OCTRL_POLB_MASK)

#define PWM_OCTRL_POLA_MASK                      (0x400U)
#define PWM_OCTRL_POLA_SHIFT                     (10U)
/*! POLA - PWM_A Output Polarity
 *  0b0..PWM_A output not inverted. A high level on the PWM_A pin represents the "on" or "active" state.
 *  0b1..PWM_A output inverted. A low level on the PWM_A pin represents the "on" or "active" state.
 */
#define PWM_OCTRL_POLA(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_POLA_SHIFT)) & PWM_OCTRL_POLA_MASK)

#define PWM_OCTRL_PWMX_IN_MASK                   (0x2000U)
#define PWM_OCTRL_PWMX_IN_SHIFT                  (13U)
/*! PWMX_IN - PWM_X Input */
#define PWM_OCTRL_PWMX_IN(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_PWMX_IN_SHIFT)) & PWM_OCTRL_PWMX_IN_MASK)

#define PWM_OCTRL_PWMB_IN_MASK                   (0x4000U)
#define PWM_OCTRL_PWMB_IN_SHIFT                  (14U)
/*! PWMB_IN - PWM_B Input */
#define PWM_OCTRL_PWMB_IN(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_PWMB_IN_SHIFT)) & PWM_OCTRL_PWMB_IN_MASK)

#define PWM_OCTRL_PWMA_IN_MASK                   (0x8000U)
#define PWM_OCTRL_PWMA_IN_SHIFT                  (15U)
/*! PWMA_IN - PWM_A Input */
#define PWM_OCTRL_PWMA_IN(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_OCTRL_PWMA_IN_SHIFT)) & PWM_OCTRL_PWMA_IN_MASK)
/*! @} */

/* The count of PWM_OCTRL */
#define PWM_OCTRL_COUNT                          (4U)

/*! @name STS - Status Register */
/*! @{ */

#define PWM_STS_CMPF_MASK                        (0x3FU)
#define PWM_STS_CMPF_SHIFT                       (0U)
/*! CMPF - Compare Flags
 *  0b000000..No compare event has occurred for a particular VALx value.
 *  0b000001..A compare event has occurred for a particular VALx value.
 */
#define PWM_STS_CMPF(x)                          (((uint16_t)(((uint16_t)(x)) << PWM_STS_CMPF_SHIFT)) & PWM_STS_CMPF_MASK)

#define PWM_STS_CFX0_MASK                        (0x40U)
#define PWM_STS_CFX0_SHIFT                       (6U)
/*! CFX0 - Capture Flag X0 */
#define PWM_STS_CFX0(x)                          (((uint16_t)(((uint16_t)(x)) << PWM_STS_CFX0_SHIFT)) & PWM_STS_CFX0_MASK)

#define PWM_STS_CFX1_MASK                        (0x80U)
#define PWM_STS_CFX1_SHIFT                       (7U)
/*! CFX1 - Capture Flag X1 */
#define PWM_STS_CFX1(x)                          (((uint16_t)(((uint16_t)(x)) << PWM_STS_CFX1_SHIFT)) & PWM_STS_CFX1_MASK)

#define PWM_STS_CFB0_MASK                        (0x100U)
#define PWM_STS_CFB0_SHIFT                       (8U)
/*! CFB0 - Capture Flag B0 */
#define PWM_STS_CFB0(x)                          (((uint16_t)(((uint16_t)(x)) << PWM_STS_CFB0_SHIFT)) & PWM_STS_CFB0_MASK)

#define PWM_STS_CFB1_MASK                        (0x200U)
#define PWM_STS_CFB1_SHIFT                       (9U)
/*! CFB1 - Capture Flag B1 */
#define PWM_STS_CFB1(x)                          (((uint16_t)(((uint16_t)(x)) << PWM_STS_CFB1_SHIFT)) & PWM_STS_CFB1_MASK)

#define PWM_STS_CFA0_MASK                        (0x400U)
#define PWM_STS_CFA0_SHIFT                       (10U)
/*! CFA0 - Capture Flag A0 */
#define PWM_STS_CFA0(x)                          (((uint16_t)(((uint16_t)(x)) << PWM_STS_CFA0_SHIFT)) & PWM_STS_CFA0_MASK)

#define PWM_STS_CFA1_MASK                        (0x800U)
#define PWM_STS_CFA1_SHIFT                       (11U)
/*! CFA1 - Capture Flag A1 */
#define PWM_STS_CFA1(x)                          (((uint16_t)(((uint16_t)(x)) << PWM_STS_CFA1_SHIFT)) & PWM_STS_CFA1_MASK)

#define PWM_STS_RF_MASK                          (0x1000U)
#define PWM_STS_RF_SHIFT                         (12U)
/*! RF - Reload Flag
 *  0b0..No new reload cycle since last STS[RF] clearing
 *  0b1..New reload cycle since last STS[RF] clearing
 */
#define PWM_STS_RF(x)                            (((uint16_t)(((uint16_t)(x)) << PWM_STS_RF_SHIFT)) & PWM_STS_RF_MASK)

#define PWM_STS_REF_MASK                         (0x2000U)
#define PWM_STS_REF_SHIFT                        (13U)
/*! REF - Reload Error Flag
 *  0b0..No reload error occurred.
 *  0b1..Reload signal occurred with non-coherent data and MCTRL[LDOK] = 0.
 */
#define PWM_STS_REF(x)                           (((uint16_t)(((uint16_t)(x)) << PWM_STS_REF_SHIFT)) & PWM_STS_REF_MASK)

#define PWM_STS_RUF_MASK                         (0x4000U)
#define PWM_STS_RUF_SHIFT                        (14U)
/*! RUF - Registers Updated Flag
 *  0b0..No register update has occurred since last reload.
 *  0b1..At least one of the double buffered registers has been updated since the last reload.
 */
#define PWM_STS_RUF(x)                           (((uint16_t)(((uint16_t)(x)) << PWM_STS_RUF_SHIFT)) & PWM_STS_RUF_MASK)
/*! @} */

/* The count of PWM_STS */
#define PWM_STS_COUNT                            (4U)

/*! @name INTEN - Interrupt Enable Register */
/*! @{ */

#define PWM_INTEN_CMPIE_MASK                     (0x3FU)
#define PWM_INTEN_CMPIE_SHIFT                    (0U)
/*! CMPIE - Compare Interrupt Enables
 *  0b000000..The corresponding STS[CMPF] bit will not cause an interrupt request.
 *  0b000001..The corresponding STS[CMPF] bit will cause an interrupt request.
 */
#define PWM_INTEN_CMPIE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_CMPIE_SHIFT)) & PWM_INTEN_CMPIE_MASK)

#define PWM_INTEN_CX0IE_MASK                     (0x40U)
#define PWM_INTEN_CX0IE_SHIFT                    (6U)
/*! CX0IE - Capture X 0 Interrupt Enable
 *  0b0..Interrupt request disabled for STS[CFX0].
 *  0b1..Interrupt request enabled for STS[CFX0].
 */
#define PWM_INTEN_CX0IE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_CX0IE_SHIFT)) & PWM_INTEN_CX0IE_MASK)

#define PWM_INTEN_CX1IE_MASK                     (0x80U)
#define PWM_INTEN_CX1IE_SHIFT                    (7U)
/*! CX1IE - Capture X 1 Interrupt Enable
 *  0b0..Interrupt request disabled for STS[CFX1].
 *  0b1..Interrupt request enabled for STS[CFX1].
 */
#define PWM_INTEN_CX1IE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_CX1IE_SHIFT)) & PWM_INTEN_CX1IE_MASK)

#define PWM_INTEN_CB0IE_MASK                     (0x100U)
#define PWM_INTEN_CB0IE_SHIFT                    (8U)
/*! CB0IE - Capture B 0 Interrupt Enable
 *  0b0..Interrupt request disabled for STS[CFB0].
 *  0b1..Interrupt request enabled for STS[CFB0].
 */
#define PWM_INTEN_CB0IE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_CB0IE_SHIFT)) & PWM_INTEN_CB0IE_MASK)

#define PWM_INTEN_CB1IE_MASK                     (0x200U)
#define PWM_INTEN_CB1IE_SHIFT                    (9U)
/*! CB1IE - Capture B 1 Interrupt Enable
 *  0b0..Interrupt request disabled for STS[CFB1].
 *  0b1..Interrupt request enabled for STS[CFB1].
 */
#define PWM_INTEN_CB1IE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_CB1IE_SHIFT)) & PWM_INTEN_CB1IE_MASK)

#define PWM_INTEN_CA0IE_MASK                     (0x400U)
#define PWM_INTEN_CA0IE_SHIFT                    (10U)
/*! CA0IE - Capture A 0 Interrupt Enable
 *  0b0..Interrupt request disabled for STS[CFA0].
 *  0b1..Interrupt request enabled for STS[CFA0].
 */
#define PWM_INTEN_CA0IE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_CA0IE_SHIFT)) & PWM_INTEN_CA0IE_MASK)

#define PWM_INTEN_CA1IE_MASK                     (0x800U)
#define PWM_INTEN_CA1IE_SHIFT                    (11U)
/*! CA1IE - Capture A 1 Interrupt Enable
 *  0b0..Interrupt request disabled for STS[CFA1]
 *  0b1..Interrupt request enabled for STS[CFA1]
 */
#define PWM_INTEN_CA1IE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_CA1IE_SHIFT)) & PWM_INTEN_CA1IE_MASK)

#define PWM_INTEN_RIE_MASK                       (0x1000U)
#define PWM_INTEN_RIE_SHIFT                      (12U)
/*! RIE - Reload Interrupt Enable
 *  0b0..STS[RF] CPU interrupt requests disabled
 *  0b1..STS[RF] CPU interrupt requests enabled
 */
#define PWM_INTEN_RIE(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_RIE_SHIFT)) & PWM_INTEN_RIE_MASK)

#define PWM_INTEN_REIE_MASK                      (0x2000U)
#define PWM_INTEN_REIE_SHIFT                     (13U)
/*! REIE - Reload Error Interrupt Enable
 *  0b0..STS[REF] CPU interrupt requests disabled
 *  0b1..STS[REF] CPU interrupt requests enabled
 */
#define PWM_INTEN_REIE(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_INTEN_REIE_SHIFT)) & PWM_INTEN_REIE_MASK)
/*! @} */

/* The count of PWM_INTEN */
#define PWM_INTEN_COUNT                          (4U)

/*! @name DMAEN - DMA Enable Register */
/*! @{ */

#define PWM_DMAEN_CX0DE_MASK                     (0x1U)
#define PWM_DMAEN_CX0DE_SHIFT                    (0U)
/*! CX0DE - Capture X0 FIFO DMA Enable */
#define PWM_DMAEN_CX0DE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_CX0DE_SHIFT)) & PWM_DMAEN_CX0DE_MASK)

#define PWM_DMAEN_CX1DE_MASK                     (0x2U)
#define PWM_DMAEN_CX1DE_SHIFT                    (1U)
/*! CX1DE - Capture X1 FIFO DMA Enable */
#define PWM_DMAEN_CX1DE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_CX1DE_SHIFT)) & PWM_DMAEN_CX1DE_MASK)

#define PWM_DMAEN_CB0DE_MASK                     (0x4U)
#define PWM_DMAEN_CB0DE_SHIFT                    (2U)
/*! CB0DE - Capture B0 FIFO DMA Enable */
#define PWM_DMAEN_CB0DE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_CB0DE_SHIFT)) & PWM_DMAEN_CB0DE_MASK)

#define PWM_DMAEN_CB1DE_MASK                     (0x8U)
#define PWM_DMAEN_CB1DE_SHIFT                    (3U)
/*! CB1DE - Capture B1 FIFO DMA Enable */
#define PWM_DMAEN_CB1DE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_CB1DE_SHIFT)) & PWM_DMAEN_CB1DE_MASK)

#define PWM_DMAEN_CA0DE_MASK                     (0x10U)
#define PWM_DMAEN_CA0DE_SHIFT                    (4U)
/*! CA0DE - Capture A0 FIFO DMA Enable */
#define PWM_DMAEN_CA0DE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_CA0DE_SHIFT)) & PWM_DMAEN_CA0DE_MASK)

#define PWM_DMAEN_CA1DE_MASK                     (0x20U)
#define PWM_DMAEN_CA1DE_SHIFT                    (5U)
/*! CA1DE - Capture A1 FIFO DMA Enable */
#define PWM_DMAEN_CA1DE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_CA1DE_SHIFT)) & PWM_DMAEN_CA1DE_MASK)

#define PWM_DMAEN_CAPTDE_MASK                    (0xC0U)
#define PWM_DMAEN_CAPTDE_SHIFT                   (6U)
/*! CAPTDE - Capture DMA Enable Source Select
 *  0b00..Read DMA requests disabled.
 *  0b01..Exceeding a FIFO watermark sets the DMA read request. This requires at least one of DMAEN[CA1DE],
 *        DMAEN[CA0DE], DMAEN[CB1DE], DMAEN[CB0DE], DMAEN[CX1DE], or DMAEN[CX0DE] to be set to determine which
 *        watermark(s) the DMA request is sensitive.
 *  0b10..A local synchronization (VAL1 matches counter) sets the read DMA request.
 *  0b11..A local reload (STS[RF] being set) sets the read DMA request.
 */
#define PWM_DMAEN_CAPTDE(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_CAPTDE_SHIFT)) & PWM_DMAEN_CAPTDE_MASK)

#define PWM_DMAEN_FAND_MASK                      (0x100U)
#define PWM_DMAEN_FAND_SHIFT                     (8U)
/*! FAND - FIFO Watermark AND Control
 *  0b0..Selected FIFO watermarks are OR'ed together.
 *  0b1..Selected FIFO watermarks are AND'ed together.
 */
#define PWM_DMAEN_FAND(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_FAND_SHIFT)) & PWM_DMAEN_FAND_MASK)

#define PWM_DMAEN_VALDE_MASK                     (0x200U)
#define PWM_DMAEN_VALDE_SHIFT                    (9U)
/*! VALDE - Value Registers DMA Enable
 *  0b0..DMA write requests disabled
 *  0b1..Enabled
 */
#define PWM_DMAEN_VALDE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_DMAEN_VALDE_SHIFT)) & PWM_DMAEN_VALDE_MASK)
/*! @} */

/* The count of PWM_DMAEN */
#define PWM_DMAEN_COUNT                          (4U)

/*! @name TCTRL - Output Trigger Control Register */
/*! @{ */

#define PWM_TCTRL_OUT_TRIG_EN_MASK               (0x3FU)
#define PWM_TCTRL_OUT_TRIG_EN_SHIFT              (0U)
/*! OUT_TRIG_EN - Output Trigger Enables
 *  0bxxxxx1..PWM_OUT_TRIG0 will set when the counter value matches the VAL0 value.
 *  0bxxxx1x..PWM_OUT_TRIG1 will set when the counter value matches the VAL1 value.
 *  0bxxx1xx..PWM_OUT_TRIG0 will set when the counter value matches the VAL2 value.
 *  0bxx1xxx..PWM_OUT_TRIG1 will set when the counter value matches the VAL3 value.
 *  0bx1xxxx..PWM_OUT_TRIG0 will set when the counter value matches the VAL4 value.
 *  0b1xxxxx..PWM_OUT_TRIG1 will set when the counter value matches the VAL5 value.
 */
#define PWM_TCTRL_OUT_TRIG_EN(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_TCTRL_OUT_TRIG_EN_SHIFT)) & PWM_TCTRL_OUT_TRIG_EN_MASK)

#define PWM_TCTRL_TRGFRQ_MASK                    (0x1000U)
#define PWM_TCTRL_TRGFRQ_SHIFT                   (12U)
/*! TRGFRQ - Trigger Frequency
 *  0b0..Trigger outputs are generated during every PWM period even if the PWM is not reloaded every period due to CTRL[LDFQ] being non-zero.
 *  0b1..Trigger outputs are generated only during the final PWM period prior to a reload opportunity when the PWM
 *       is not reloaded every period due to CTRL[LDFQ] being non-zero.
 */
#define PWM_TCTRL_TRGFRQ(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_TCTRL_TRGFRQ_SHIFT)) & PWM_TCTRL_TRGFRQ_MASK)

#define PWM_TCTRL_PWBOT1_MASK                    (0x4000U)
#define PWM_TCTRL_PWBOT1_SHIFT                   (14U)
/*! PWBOT1 - Mux Output Trigger 1 Source Select
 *  0b0..Route the PWM_OUT_TRIG1 signal to PWM_MUX_TRIG1 port.
 *  0b1..Route the PWM_B output to the PWM_MUX_TRIG1 port.
 */
#define PWM_TCTRL_PWBOT1(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_TCTRL_PWBOT1_SHIFT)) & PWM_TCTRL_PWBOT1_MASK)

#define PWM_TCTRL_PWAOT0_MASK                    (0x8000U)
#define PWM_TCTRL_PWAOT0_SHIFT                   (15U)
/*! PWAOT0 - Mux Output Trigger 0 Source Select
 *  0b0..Route the PWM_OUT_TRIG0 signal to PWM_MUX_TRIG0 port.
 *  0b1..Route the PWM_A output to the PWM_MUX_TRIG0 port.
 */
#define PWM_TCTRL_PWAOT0(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_TCTRL_PWAOT0_SHIFT)) & PWM_TCTRL_PWAOT0_MASK)
/*! @} */

/* The count of PWM_TCTRL */
#define PWM_TCTRL_COUNT                          (4U)

/*! @name DISMAP - Fault Disable Mapping Register 0 */
/*! @{ */

#define PWM_DISMAP_DIS0A_MASK                    (0xFU)
#define PWM_DISMAP_DIS0A_SHIFT                   (0U)
/*! DIS0A - PWM_A Fault Disable Mask 0 */
#define PWM_DISMAP_DIS0A(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_DISMAP_DIS0A_SHIFT)) & PWM_DISMAP_DIS0A_MASK)

#define PWM_DISMAP_DIS0B_MASK                    (0xF0U)
#define PWM_DISMAP_DIS0B_SHIFT                   (4U)
/*! DIS0B - PWM_B Fault Disable Mask 0 */
#define PWM_DISMAP_DIS0B(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_DISMAP_DIS0B_SHIFT)) & PWM_DISMAP_DIS0B_MASK)

#define PWM_DISMAP_DIS0X_MASK                    (0xF00U)
#define PWM_DISMAP_DIS0X_SHIFT                   (8U)
/*! DIS0X - PWM_X Fault Disable Mask 0 */
#define PWM_DISMAP_DIS0X(x)                      (((uint16_t)(((uint16_t)(x)) << PWM_DISMAP_DIS0X_SHIFT)) & PWM_DISMAP_DIS0X_MASK)
/*! @} */

/* The count of PWM_DISMAP */
#define PWM_DISMAP_COUNT                         (4U)

/* The count of PWM_DISMAP */
#define PWM_DISMAP_COUNT2                        (1U)

/*! @name DTCNT0 - Deadtime Count Register 0 */
/*! @{ */

#define PWM_DTCNT0_DTCNT0_MASK                   (0xFFFFU)
#define PWM_DTCNT0_DTCNT0_SHIFT                  (0U)
/*! DTCNT0 - DTCNT0 */
#define PWM_DTCNT0_DTCNT0(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_DTCNT0_DTCNT0_SHIFT)) & PWM_DTCNT0_DTCNT0_MASK)
/*! @} */

/* The count of PWM_DTCNT0 */
#define PWM_DTCNT0_COUNT                         (4U)

/*! @name DTCNT1 - Deadtime Count Register 1 */
/*! @{ */

#define PWM_DTCNT1_DTCNT1_MASK                   (0xFFFFU)
#define PWM_DTCNT1_DTCNT1_SHIFT                  (0U)
/*! DTCNT1 - DTCNT1 */
#define PWM_DTCNT1_DTCNT1(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_DTCNT1_DTCNT1_SHIFT)) & PWM_DTCNT1_DTCNT1_MASK)
/*! @} */

/* The count of PWM_DTCNT1 */
#define PWM_DTCNT1_COUNT                         (4U)

/*! @name CAPTCTRLA - Capture Control A Register */
/*! @{ */

#define PWM_CAPTCTRLA_ARMA_MASK                  (0x1U)
#define PWM_CAPTCTRLA_ARMA_SHIFT                 (0U)
/*! ARMA - Arm A
 *  0b0..Input capture operation is disabled.
 *  0b1..Input capture operation as specified by CAPTCTRLA[EDGAx] is enabled.
 */
#define PWM_CAPTCTRLA_ARMA(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_ARMA_SHIFT)) & PWM_CAPTCTRLA_ARMA_MASK)

#define PWM_CAPTCTRLA_ONESHOTA_MASK              (0x2U)
#define PWM_CAPTCTRLA_ONESHOTA_SHIFT             (1U)
/*! ONESHOTA - One Shot Mode A
 *  0b0..Free Running
 *  0b1..One Shot
 */
#define PWM_CAPTCTRLA_ONESHOTA(x)                (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_ONESHOTA_SHIFT)) & PWM_CAPTCTRLA_ONESHOTA_MASK)

#define PWM_CAPTCTRLA_EDGA0_MASK                 (0xCU)
#define PWM_CAPTCTRLA_EDGA0_SHIFT                (2U)
/*! EDGA0 - Edge A 0
 *  0b00..Disabled
 *  0b01..Capture falling edges
 *  0b10..Capture rising edges
 *  0b11..Capture any edge
 */
#define PWM_CAPTCTRLA_EDGA0(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_EDGA0_SHIFT)) & PWM_CAPTCTRLA_EDGA0_MASK)

#define PWM_CAPTCTRLA_EDGA1_MASK                 (0x30U)
#define PWM_CAPTCTRLA_EDGA1_SHIFT                (4U)
/*! EDGA1 - Edge A 1
 *  0b00..Disabled
 *  0b01..Capture falling edges
 *  0b10..Capture rising edges
 *  0b11..Capture any edge
 */
#define PWM_CAPTCTRLA_EDGA1(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_EDGA1_SHIFT)) & PWM_CAPTCTRLA_EDGA1_MASK)

#define PWM_CAPTCTRLA_INP_SELA_MASK              (0x40U)
#define PWM_CAPTCTRLA_INP_SELA_SHIFT             (6U)
/*! INP_SELA - Input Select A
 *  0b0..Raw PWM_A input signal selected as source.
 *  0b1..Edge Counter
 */
#define PWM_CAPTCTRLA_INP_SELA(x)                (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_INP_SELA_SHIFT)) & PWM_CAPTCTRLA_INP_SELA_MASK)

#define PWM_CAPTCTRLA_EDGCNTA_EN_MASK            (0x80U)
#define PWM_CAPTCTRLA_EDGCNTA_EN_SHIFT           (7U)
/*! EDGCNTA_EN - Edge Counter A Enable
 *  0b0..Edge counter disabled and held in reset
 *  0b1..Edge counter enabled
 */
#define PWM_CAPTCTRLA_EDGCNTA_EN(x)              (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_EDGCNTA_EN_SHIFT)) & PWM_CAPTCTRLA_EDGCNTA_EN_MASK)

#define PWM_CAPTCTRLA_CFAWM_MASK                 (0x300U)
#define PWM_CAPTCTRLA_CFAWM_SHIFT                (8U)
/*! CFAWM - Capture A FIFOs Water Mark */
#define PWM_CAPTCTRLA_CFAWM(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_CFAWM_SHIFT)) & PWM_CAPTCTRLA_CFAWM_MASK)

#define PWM_CAPTCTRLA_CA0CNT_MASK                (0x1C00U)
#define PWM_CAPTCTRLA_CA0CNT_SHIFT               (10U)
/*! CA0CNT - Capture A0 FIFO Word Count */
#define PWM_CAPTCTRLA_CA0CNT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_CA0CNT_SHIFT)) & PWM_CAPTCTRLA_CA0CNT_MASK)

#define PWM_CAPTCTRLA_CA1CNT_MASK                (0xE000U)
#define PWM_CAPTCTRLA_CA1CNT_SHIFT               (13U)
/*! CA1CNT - Capture A1 FIFO Word Count */
#define PWM_CAPTCTRLA_CA1CNT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLA_CA1CNT_SHIFT)) & PWM_CAPTCTRLA_CA1CNT_MASK)
/*! @} */

/* The count of PWM_CAPTCTRLA */
#define PWM_CAPTCTRLA_COUNT                      (4U)

/*! @name CAPTCOMPA - Capture Compare A Register */
/*! @{ */

#define PWM_CAPTCOMPA_EDGCMPA_MASK               (0xFFU)
#define PWM_CAPTCOMPA_EDGCMPA_SHIFT              (0U)
/*! EDGCMPA - Edge Compare A */
#define PWM_CAPTCOMPA_EDGCMPA(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCOMPA_EDGCMPA_SHIFT)) & PWM_CAPTCOMPA_EDGCMPA_MASK)

#define PWM_CAPTCOMPA_EDGCNTA_MASK               (0xFF00U)
#define PWM_CAPTCOMPA_EDGCNTA_SHIFT              (8U)
/*! EDGCNTA - Edge Counter A */
#define PWM_CAPTCOMPA_EDGCNTA(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCOMPA_EDGCNTA_SHIFT)) & PWM_CAPTCOMPA_EDGCNTA_MASK)
/*! @} */

/* The count of PWM_CAPTCOMPA */
#define PWM_CAPTCOMPA_COUNT                      (4U)

/*! @name CAPTCTRLB - Capture Control B Register */
/*! @{ */

#define PWM_CAPTCTRLB_ARMB_MASK                  (0x1U)
#define PWM_CAPTCTRLB_ARMB_SHIFT                 (0U)
/*! ARMB - Arm B
 *  0b0..Input capture operation is disabled.
 *  0b1..Input capture operation as specified by CAPTCTRLB[EDGBx] is enabled.
 */
#define PWM_CAPTCTRLB_ARMB(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_ARMB_SHIFT)) & PWM_CAPTCTRLB_ARMB_MASK)

#define PWM_CAPTCTRLB_ONESHOTB_MASK              (0x2U)
#define PWM_CAPTCTRLB_ONESHOTB_SHIFT             (1U)
/*! ONESHOTB - One Shot Mode B
 *  0b0..Free Running
 *  0b1..One Shot
 */
#define PWM_CAPTCTRLB_ONESHOTB(x)                (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_ONESHOTB_SHIFT)) & PWM_CAPTCTRLB_ONESHOTB_MASK)

#define PWM_CAPTCTRLB_EDGB0_MASK                 (0xCU)
#define PWM_CAPTCTRLB_EDGB0_SHIFT                (2U)
/*! EDGB0 - Edge B 0
 *  0b00..Disabled
 *  0b01..Capture falling edges
 *  0b10..Capture rising edges
 *  0b11..Capture any edge
 */
#define PWM_CAPTCTRLB_EDGB0(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_EDGB0_SHIFT)) & PWM_CAPTCTRLB_EDGB0_MASK)

#define PWM_CAPTCTRLB_EDGB1_MASK                 (0x30U)
#define PWM_CAPTCTRLB_EDGB1_SHIFT                (4U)
/*! EDGB1 - Edge B 1
 *  0b00..Disabled
 *  0b01..Capture falling edges
 *  0b10..Capture rising edges
 *  0b11..Capture any edge
 */
#define PWM_CAPTCTRLB_EDGB1(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_EDGB1_SHIFT)) & PWM_CAPTCTRLB_EDGB1_MASK)

#define PWM_CAPTCTRLB_INP_SELB_MASK              (0x40U)
#define PWM_CAPTCTRLB_INP_SELB_SHIFT             (6U)
/*! INP_SELB - Input Select B
 *  0b0..Raw PWM_B input signal selected as source.
 *  0b1..Edge Counter
 */
#define PWM_CAPTCTRLB_INP_SELB(x)                (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_INP_SELB_SHIFT)) & PWM_CAPTCTRLB_INP_SELB_MASK)

#define PWM_CAPTCTRLB_EDGCNTB_EN_MASK            (0x80U)
#define PWM_CAPTCTRLB_EDGCNTB_EN_SHIFT           (7U)
/*! EDGCNTB_EN - Edge Counter B Enable
 *  0b0..Edge counter disabled and held in reset
 *  0b1..Edge counter enabled
 */
#define PWM_CAPTCTRLB_EDGCNTB_EN(x)              (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_EDGCNTB_EN_SHIFT)) & PWM_CAPTCTRLB_EDGCNTB_EN_MASK)

#define PWM_CAPTCTRLB_CFBWM_MASK                 (0x300U)
#define PWM_CAPTCTRLB_CFBWM_SHIFT                (8U)
/*! CFBWM - Capture B FIFOs Water Mark */
#define PWM_CAPTCTRLB_CFBWM(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_CFBWM_SHIFT)) & PWM_CAPTCTRLB_CFBWM_MASK)

#define PWM_CAPTCTRLB_CB0CNT_MASK                (0x1C00U)
#define PWM_CAPTCTRLB_CB0CNT_SHIFT               (10U)
/*! CB0CNT - Capture B0 FIFO Word Count */
#define PWM_CAPTCTRLB_CB0CNT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_CB0CNT_SHIFT)) & PWM_CAPTCTRLB_CB0CNT_MASK)

#define PWM_CAPTCTRLB_CB1CNT_MASK                (0xE000U)
#define PWM_CAPTCTRLB_CB1CNT_SHIFT               (13U)
/*! CB1CNT - Capture B1 FIFO Word Count */
#define PWM_CAPTCTRLB_CB1CNT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLB_CB1CNT_SHIFT)) & PWM_CAPTCTRLB_CB1CNT_MASK)
/*! @} */

/* The count of PWM_CAPTCTRLB */
#define PWM_CAPTCTRLB_COUNT                      (4U)

/*! @name CAPTCOMPB - Capture Compare B Register */
/*! @{ */

#define PWM_CAPTCOMPB_EDGCMPB_MASK               (0xFFU)
#define PWM_CAPTCOMPB_EDGCMPB_SHIFT              (0U)
/*! EDGCMPB - Edge Compare B */
#define PWM_CAPTCOMPB_EDGCMPB(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCOMPB_EDGCMPB_SHIFT)) & PWM_CAPTCOMPB_EDGCMPB_MASK)

#define PWM_CAPTCOMPB_EDGCNTB_MASK               (0xFF00U)
#define PWM_CAPTCOMPB_EDGCNTB_SHIFT              (8U)
/*! EDGCNTB - Edge Counter B */
#define PWM_CAPTCOMPB_EDGCNTB(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCOMPB_EDGCNTB_SHIFT)) & PWM_CAPTCOMPB_EDGCNTB_MASK)
/*! @} */

/* The count of PWM_CAPTCOMPB */
#define PWM_CAPTCOMPB_COUNT                      (4U)

/*! @name CAPTCTRLX - Capture Control X Register */
/*! @{ */

#define PWM_CAPTCTRLX_ARMX_MASK                  (0x1U)
#define PWM_CAPTCTRLX_ARMX_SHIFT                 (0U)
/*! ARMX - Arm X
 *  0b0..Input capture operation is disabled.
 *  0b1..Input capture operation as specified by CAPTCTRLX[EDGXx] is enabled.
 */
#define PWM_CAPTCTRLX_ARMX(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_ARMX_SHIFT)) & PWM_CAPTCTRLX_ARMX_MASK)

#define PWM_CAPTCTRLX_ONESHOTX_MASK              (0x2U)
#define PWM_CAPTCTRLX_ONESHOTX_SHIFT             (1U)
/*! ONESHOTX - One Shot Mode Aux
 *  0b0..Free Running
 *  0b1..One Shot
 */
#define PWM_CAPTCTRLX_ONESHOTX(x)                (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_ONESHOTX_SHIFT)) & PWM_CAPTCTRLX_ONESHOTX_MASK)

#define PWM_CAPTCTRLX_EDGX0_MASK                 (0xCU)
#define PWM_CAPTCTRLX_EDGX0_SHIFT                (2U)
/*! EDGX0 - Edge X 0
 *  0b00..Disabled
 *  0b01..Capture falling edges
 *  0b10..Capture rising edges
 *  0b11..Capture any edge
 */
#define PWM_CAPTCTRLX_EDGX0(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_EDGX0_SHIFT)) & PWM_CAPTCTRLX_EDGX0_MASK)

#define PWM_CAPTCTRLX_EDGX1_MASK                 (0x30U)
#define PWM_CAPTCTRLX_EDGX1_SHIFT                (4U)
/*! EDGX1 - Edge X 1
 *  0b00..Disabled
 *  0b01..Capture falling edges
 *  0b10..Capture rising edges
 *  0b11..Capture any edge
 */
#define PWM_CAPTCTRLX_EDGX1(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_EDGX1_SHIFT)) & PWM_CAPTCTRLX_EDGX1_MASK)

#define PWM_CAPTCTRLX_INP_SELX_MASK              (0x40U)
#define PWM_CAPTCTRLX_INP_SELX_SHIFT             (6U)
/*! INP_SELX - Input Select X
 *  0b0..Raw PWM_X input signal selected as source.
 *  0b1..Edge Counter
 */
#define PWM_CAPTCTRLX_INP_SELX(x)                (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_INP_SELX_SHIFT)) & PWM_CAPTCTRLX_INP_SELX_MASK)

#define PWM_CAPTCTRLX_EDGCNTX_EN_MASK            (0x80U)
#define PWM_CAPTCTRLX_EDGCNTX_EN_SHIFT           (7U)
/*! EDGCNTX_EN - Edge Counter X Enable
 *  0b0..Edge counter disabled and held in reset
 *  0b1..Edge counter enabled
 */
#define PWM_CAPTCTRLX_EDGCNTX_EN(x)              (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_EDGCNTX_EN_SHIFT)) & PWM_CAPTCTRLX_EDGCNTX_EN_MASK)

#define PWM_CAPTCTRLX_CFXWM_MASK                 (0x300U)
#define PWM_CAPTCTRLX_CFXWM_SHIFT                (8U)
/*! CFXWM - Capture X FIFOs Water Mark */
#define PWM_CAPTCTRLX_CFXWM(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_CFXWM_SHIFT)) & PWM_CAPTCTRLX_CFXWM_MASK)

#define PWM_CAPTCTRLX_CX0CNT_MASK                (0x1C00U)
#define PWM_CAPTCTRLX_CX0CNT_SHIFT               (10U)
/*! CX0CNT - Capture X0 FIFO Word Count */
#define PWM_CAPTCTRLX_CX0CNT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_CX0CNT_SHIFT)) & PWM_CAPTCTRLX_CX0CNT_MASK)

#define PWM_CAPTCTRLX_CX1CNT_MASK                (0xE000U)
#define PWM_CAPTCTRLX_CX1CNT_SHIFT               (13U)
/*! CX1CNT - Capture X1 FIFO Word Count */
#define PWM_CAPTCTRLX_CX1CNT(x)                  (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCTRLX_CX1CNT_SHIFT)) & PWM_CAPTCTRLX_CX1CNT_MASK)
/*! @} */

/* The count of PWM_CAPTCTRLX */
#define PWM_CAPTCTRLX_COUNT                      (4U)

/*! @name CAPTCOMPX - Capture Compare X Register */
/*! @{ */

#define PWM_CAPTCOMPX_EDGCMPX_MASK               (0xFFU)
#define PWM_CAPTCOMPX_EDGCMPX_SHIFT              (0U)
/*! EDGCMPX - Edge Compare X */
#define PWM_CAPTCOMPX_EDGCMPX(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCOMPX_EDGCMPX_SHIFT)) & PWM_CAPTCOMPX_EDGCMPX_MASK)

#define PWM_CAPTCOMPX_EDGCNTX_MASK               (0xFF00U)
#define PWM_CAPTCOMPX_EDGCNTX_SHIFT              (8U)
/*! EDGCNTX - Edge Counter X */
#define PWM_CAPTCOMPX_EDGCNTX(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CAPTCOMPX_EDGCNTX_SHIFT)) & PWM_CAPTCOMPX_EDGCNTX_MASK)
/*! @} */

/* The count of PWM_CAPTCOMPX */
#define PWM_CAPTCOMPX_COUNT                      (4U)

/*! @name CVAL0 - Capture Value 0 Register */
/*! @{ */

#define PWM_CVAL0_CAPTVAL0_MASK                  (0xFFFFU)
#define PWM_CVAL0_CAPTVAL0_SHIFT                 (0U)
/*! CAPTVAL0 - Capture Value 0 */
#define PWM_CVAL0_CAPTVAL0(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CVAL0_CAPTVAL0_SHIFT)) & PWM_CVAL0_CAPTVAL0_MASK)
/*! @} */

/* The count of PWM_CVAL0 */
#define PWM_CVAL0_COUNT                          (4U)

/*! @name CVAL0CYC - Capture Value 0 Cycle Register */
/*! @{ */

#define PWM_CVAL0CYC_CVAL0CYC_MASK               (0xFU)
#define PWM_CVAL0CYC_CVAL0CYC_SHIFT              (0U)
/*! CVAL0CYC - Capture Value 0 Cycle */
#define PWM_CVAL0CYC_CVAL0CYC(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CVAL0CYC_CVAL0CYC_SHIFT)) & PWM_CVAL0CYC_CVAL0CYC_MASK)
/*! @} */

/* The count of PWM_CVAL0CYC */
#define PWM_CVAL0CYC_COUNT                       (4U)

/*! @name CVAL1 - Capture Value 1 Register */
/*! @{ */

#define PWM_CVAL1_CAPTVAL1_MASK                  (0xFFFFU)
#define PWM_CVAL1_CAPTVAL1_SHIFT                 (0U)
/*! CAPTVAL1 - Capture Value 1 */
#define PWM_CVAL1_CAPTVAL1(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CVAL1_CAPTVAL1_SHIFT)) & PWM_CVAL1_CAPTVAL1_MASK)
/*! @} */

/* The count of PWM_CVAL1 */
#define PWM_CVAL1_COUNT                          (4U)

/*! @name CVAL1CYC - Capture Value 1 Cycle Register */
/*! @{ */

#define PWM_CVAL1CYC_CVAL1CYC_MASK               (0xFU)
#define PWM_CVAL1CYC_CVAL1CYC_SHIFT              (0U)
/*! CVAL1CYC - Capture Value 1 Cycle */
#define PWM_CVAL1CYC_CVAL1CYC(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CVAL1CYC_CVAL1CYC_SHIFT)) & PWM_CVAL1CYC_CVAL1CYC_MASK)
/*! @} */

/* The count of PWM_CVAL1CYC */
#define PWM_CVAL1CYC_COUNT                       (4U)

/*! @name CVAL2 - Capture Value 2 Register */
/*! @{ */

#define PWM_CVAL2_CAPTVAL2_MASK                  (0xFFFFU)
#define PWM_CVAL2_CAPTVAL2_SHIFT                 (0U)
/*! CAPTVAL2 - Capture Value 2 */
#define PWM_CVAL2_CAPTVAL2(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CVAL2_CAPTVAL2_SHIFT)) & PWM_CVAL2_CAPTVAL2_MASK)
/*! @} */

/* The count of PWM_CVAL2 */
#define PWM_CVAL2_COUNT                          (4U)

/*! @name CVAL2CYC - Capture Value 2 Cycle Register */
/*! @{ */

#define PWM_CVAL2CYC_CVAL2CYC_MASK               (0xFU)
#define PWM_CVAL2CYC_CVAL2CYC_SHIFT              (0U)
/*! CVAL2CYC - Capture Value 2 Cycle */
#define PWM_CVAL2CYC_CVAL2CYC(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CVAL2CYC_CVAL2CYC_SHIFT)) & PWM_CVAL2CYC_CVAL2CYC_MASK)
/*! @} */

/* The count of PWM_CVAL2CYC */
#define PWM_CVAL2CYC_COUNT                       (4U)

/*! @name CVAL3 - Capture Value 3 Register */
/*! @{ */

#define PWM_CVAL3_CAPTVAL3_MASK                  (0xFFFFU)
#define PWM_CVAL3_CAPTVAL3_SHIFT                 (0U)
/*! CAPTVAL3 - Capture Value 3 */
#define PWM_CVAL3_CAPTVAL3(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CVAL3_CAPTVAL3_SHIFT)) & PWM_CVAL3_CAPTVAL3_MASK)
/*! @} */

/* The count of PWM_CVAL3 */
#define PWM_CVAL3_COUNT                          (4U)

/*! @name CVAL3CYC - Capture Value 3 Cycle Register */
/*! @{ */

#define PWM_CVAL3CYC_CVAL3CYC_MASK               (0xFU)
#define PWM_CVAL3CYC_CVAL3CYC_SHIFT              (0U)
/*! CVAL3CYC - Capture Value 3 Cycle */
#define PWM_CVAL3CYC_CVAL3CYC(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CVAL3CYC_CVAL3CYC_SHIFT)) & PWM_CVAL3CYC_CVAL3CYC_MASK)
/*! @} */

/* The count of PWM_CVAL3CYC */
#define PWM_CVAL3CYC_COUNT                       (4U)

/*! @name CVAL4 - Capture Value 4 Register */
/*! @{ */

#define PWM_CVAL4_CAPTVAL4_MASK                  (0xFFFFU)
#define PWM_CVAL4_CAPTVAL4_SHIFT                 (0U)
/*! CAPTVAL4 - Capture Value 4 */
#define PWM_CVAL4_CAPTVAL4(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CVAL4_CAPTVAL4_SHIFT)) & PWM_CVAL4_CAPTVAL4_MASK)
/*! @} */

/* The count of PWM_CVAL4 */
#define PWM_CVAL4_COUNT                          (4U)

/*! @name CVAL4CYC - Capture Value 4 Cycle Register */
/*! @{ */

#define PWM_CVAL4CYC_CVAL4CYC_MASK               (0xFU)
#define PWM_CVAL4CYC_CVAL4CYC_SHIFT              (0U)
/*! CVAL4CYC - Capture Value 4 Cycle */
#define PWM_CVAL4CYC_CVAL4CYC(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CVAL4CYC_CVAL4CYC_SHIFT)) & PWM_CVAL4CYC_CVAL4CYC_MASK)
/*! @} */

/* The count of PWM_CVAL4CYC */
#define PWM_CVAL4CYC_COUNT                       (4U)

/*! @name CVAL5 - Capture Value 5 Register */
/*! @{ */

#define PWM_CVAL5_CAPTVAL5_MASK                  (0xFFFFU)
#define PWM_CVAL5_CAPTVAL5_SHIFT                 (0U)
/*! CAPTVAL5 - Capture Value 5 */
#define PWM_CVAL5_CAPTVAL5(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_CVAL5_CAPTVAL5_SHIFT)) & PWM_CVAL5_CAPTVAL5_MASK)
/*! @} */

/* The count of PWM_CVAL5 */
#define PWM_CVAL5_COUNT                          (4U)

/*! @name CVAL5CYC - Capture Value 5 Cycle Register */
/*! @{ */

#define PWM_CVAL5CYC_CVAL5CYC_MASK               (0xFU)
#define PWM_CVAL5CYC_CVAL5CYC_SHIFT              (0U)
/*! CVAL5CYC - Capture Value 5 Cycle */
#define PWM_CVAL5CYC_CVAL5CYC(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_CVAL5CYC_CVAL5CYC_SHIFT)) & PWM_CVAL5CYC_CVAL5CYC_MASK)
/*! @} */

/* The count of PWM_CVAL5CYC */
#define PWM_CVAL5CYC_COUNT                       (4U)

/*! @name PHASEDLY - Phase Delay Register */
/*! @{ */

#define PWM_PHASEDLY_PHASEDLY_MASK               (0xFFFFU)
#define PWM_PHASEDLY_PHASEDLY_SHIFT              (0U)
/*! PHASEDLY - Initial Count Register Bits */
#define PWM_PHASEDLY_PHASEDLY(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_PHASEDLY_PHASEDLY_SHIFT)) & PWM_PHASEDLY_PHASEDLY_MASK)
/*! @} */

/* The count of PWM_PHASEDLY */
#define PWM_PHASEDLY_COUNT                       (4U)

/*! @name CAPTFILTA - Capture PWM_A Input Filter Register */
/*! @{ */

#define PWM_CAPTFILTA_CAPTA_FILT_PER_MASK        (0xFFU)
#define PWM_CAPTFILTA_CAPTA_FILT_PER_SHIFT       (0U)
/*! CAPTA_FILT_PER - Input Capture Filter Period */
#define PWM_CAPTFILTA_CAPTA_FILT_PER(x)          (((uint16_t)(((uint16_t)(x)) << PWM_CAPTFILTA_CAPTA_FILT_PER_SHIFT)) & PWM_CAPTFILTA_CAPTA_FILT_PER_MASK)

#define PWM_CAPTFILTA_CAPTA_FILT_CNT_MASK        (0x700U)
#define PWM_CAPTFILTA_CAPTA_FILT_CNT_SHIFT       (8U)
/*! CAPTA_FILT_CNT - Input Capture Filter Count */
#define PWM_CAPTFILTA_CAPTA_FILT_CNT(x)          (((uint16_t)(((uint16_t)(x)) << PWM_CAPTFILTA_CAPTA_FILT_CNT_SHIFT)) & PWM_CAPTFILTA_CAPTA_FILT_CNT_MASK)
/*! @} */

/* The count of PWM_CAPTFILTA */
#define PWM_CAPTFILTA_COUNT                      (4U)

/*! @name CAPTFILTB - Capture PWM_B Input Filter Register */
/*! @{ */

#define PWM_CAPTFILTB_CAPTB_FILT_PER_MASK        (0xFFU)
#define PWM_CAPTFILTB_CAPTB_FILT_PER_SHIFT       (0U)
/*! CAPTB_FILT_PER - Input Capture Filter Period */
#define PWM_CAPTFILTB_CAPTB_FILT_PER(x)          (((uint16_t)(((uint16_t)(x)) << PWM_CAPTFILTB_CAPTB_FILT_PER_SHIFT)) & PWM_CAPTFILTB_CAPTB_FILT_PER_MASK)

#define PWM_CAPTFILTB_CAPTB_FILT_CNT_MASK        (0x700U)
#define PWM_CAPTFILTB_CAPTB_FILT_CNT_SHIFT       (8U)
/*! CAPTB_FILT_CNT - Input Capture Filter Count */
#define PWM_CAPTFILTB_CAPTB_FILT_CNT(x)          (((uint16_t)(((uint16_t)(x)) << PWM_CAPTFILTB_CAPTB_FILT_CNT_SHIFT)) & PWM_CAPTFILTB_CAPTB_FILT_CNT_MASK)
/*! @} */

/* The count of PWM_CAPTFILTB */
#define PWM_CAPTFILTB_COUNT                      (4U)

/*! @name CAPTFILTX - Capture PWM_X Input Filter Register */
/*! @{ */

#define PWM_CAPTFILTX_CAPTX_FILT_PER_MASK        (0xFFU)
#define PWM_CAPTFILTX_CAPTX_FILT_PER_SHIFT       (0U)
/*! CAPTX_FILT_PER - Input Capture Filter Period */
#define PWM_CAPTFILTX_CAPTX_FILT_PER(x)          (((uint16_t)(((uint16_t)(x)) << PWM_CAPTFILTX_CAPTX_FILT_PER_SHIFT)) & PWM_CAPTFILTX_CAPTX_FILT_PER_MASK)

#define PWM_CAPTFILTX_CAPTX_FILT_CNT_MASK        (0x700U)
#define PWM_CAPTFILTX_CAPTX_FILT_CNT_SHIFT       (8U)
/*! CAPTX_FILT_CNT - Input Capture Filter Count */
#define PWM_CAPTFILTX_CAPTX_FILT_CNT(x)          (((uint16_t)(((uint16_t)(x)) << PWM_CAPTFILTX_CAPTX_FILT_CNT_SHIFT)) & PWM_CAPTFILTX_CAPTX_FILT_CNT_MASK)
/*! @} */

/* The count of PWM_CAPTFILTX */
#define PWM_CAPTFILTX_COUNT                      (4U)

/*! @name OUTEN - Output Enable Register */
/*! @{ */

#define PWM_OUTEN_PWMX_EN_MASK                   (0xFU)
#define PWM_OUTEN_PWMX_EN_SHIFT                  (0U)
/*! PWMX_EN - PWM_X Output Enables */
#define PWM_OUTEN_PWMX_EN(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_OUTEN_PWMX_EN_SHIFT)) & PWM_OUTEN_PWMX_EN_MASK)

#define PWM_OUTEN_PWMB_EN_MASK                   (0xF0U)
#define PWM_OUTEN_PWMB_EN_SHIFT                  (4U)
/*! PWMB_EN - PWM_B Output Enables */
#define PWM_OUTEN_PWMB_EN(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_OUTEN_PWMB_EN_SHIFT)) & PWM_OUTEN_PWMB_EN_MASK)

#define PWM_OUTEN_PWMA_EN_MASK                   (0xF00U)
#define PWM_OUTEN_PWMA_EN_SHIFT                  (8U)
/*! PWMA_EN - PWM_A Output Enables */
#define PWM_OUTEN_PWMA_EN(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_OUTEN_PWMA_EN_SHIFT)) & PWM_OUTEN_PWMA_EN_MASK)
/*! @} */

/*! @name MASK - Mask Register */
/*! @{ */

#define PWM_MASK_MASKX_MASK                      (0xFU)
#define PWM_MASK_MASKX_SHIFT                     (0U)
/*! MASKX - PWM_X Masks */
#define PWM_MASK_MASKX(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_MASK_MASKX_SHIFT)) & PWM_MASK_MASKX_MASK)

#define PWM_MASK_MASKB_MASK                      (0xF0U)
#define PWM_MASK_MASKB_SHIFT                     (4U)
/*! MASKB - PWM_B Masks */
#define PWM_MASK_MASKB(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_MASK_MASKB_SHIFT)) & PWM_MASK_MASKB_MASK)

#define PWM_MASK_MASKA_MASK                      (0xF00U)
#define PWM_MASK_MASKA_SHIFT                     (8U)
/*! MASKA - PWM_A Masks */
#define PWM_MASK_MASKA(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_MASK_MASKA_SHIFT)) & PWM_MASK_MASKA_MASK)
/*! @} */

/*! @name SWCOUT - Software Controlled Output Register */
/*! @{ */

#define PWM_SWCOUT_SM0OUT45_MASK                 (0x1U)
#define PWM_SWCOUT_SM0OUT45_SHIFT                (0U)
/*! SM0OUT45 - Submodule 0 Software Controlled Output 45
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 0 instead of PWM45.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 0 instead of PWM45.
 */
#define PWM_SWCOUT_SM0OUT45(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM0OUT45_SHIFT)) & PWM_SWCOUT_SM0OUT45_MASK)

#define PWM_SWCOUT_SM0OUT23_MASK                 (0x2U)
#define PWM_SWCOUT_SM0OUT23_SHIFT                (1U)
/*! SM0OUT23 - Submodule 0 Software Controlled Output 23
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 0 instead of PWM23.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 0 instead of PWM23.
 */
#define PWM_SWCOUT_SM0OUT23(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM0OUT23_SHIFT)) & PWM_SWCOUT_SM0OUT23_MASK)

#define PWM_SWCOUT_SM1OUT45_MASK                 (0x4U)
#define PWM_SWCOUT_SM1OUT45_SHIFT                (2U)
/*! SM1OUT45 - Submodule 1 Software Controlled Output 45
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 1 instead of PWM45.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 1 instead of PWM45.
 */
#define PWM_SWCOUT_SM1OUT45(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM1OUT45_SHIFT)) & PWM_SWCOUT_SM1OUT45_MASK)

#define PWM_SWCOUT_SM1OUT23_MASK                 (0x8U)
#define PWM_SWCOUT_SM1OUT23_SHIFT                (3U)
/*! SM1OUT23 - Submodule 1 Software Controlled Output 23
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 1 instead of PWM23.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 1 instead of PWM23.
 */
#define PWM_SWCOUT_SM1OUT23(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM1OUT23_SHIFT)) & PWM_SWCOUT_SM1OUT23_MASK)

#define PWM_SWCOUT_SM2OUT45_MASK                 (0x10U)
#define PWM_SWCOUT_SM2OUT45_SHIFT                (4U)
/*! SM2OUT45 - Submodule 2 Software Controlled Output 45
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 2 instead of PWM45.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 2 instead of PWM45.
 */
#define PWM_SWCOUT_SM2OUT45(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM2OUT45_SHIFT)) & PWM_SWCOUT_SM2OUT45_MASK)

#define PWM_SWCOUT_SM2OUT23_MASK                 (0x20U)
#define PWM_SWCOUT_SM2OUT23_SHIFT                (5U)
/*! SM2OUT23 - Submodule 2 Software Controlled Output 23
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 2 instead of PWM23.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 2 instead of PWM23.
 */
#define PWM_SWCOUT_SM2OUT23(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM2OUT23_SHIFT)) & PWM_SWCOUT_SM2OUT23_MASK)

#define PWM_SWCOUT_SM3OUT45_MASK                 (0x40U)
#define PWM_SWCOUT_SM3OUT45_SHIFT                (6U)
/*! SM3OUT45 - Submodule 3 Software Controlled Output 45
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 3 instead of PWM45.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 3 instead of PWM45.
 */
#define PWM_SWCOUT_SM3OUT45(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM3OUT45_SHIFT)) & PWM_SWCOUT_SM3OUT45_MASK)

#define PWM_SWCOUT_SM3OUT23_MASK                 (0x80U)
#define PWM_SWCOUT_SM3OUT23_SHIFT                (7U)
/*! SM3OUT23 - Submodule 3 Software Controlled Output 23
 *  0b0..A logic 0 is supplied to the deadtime generator of submodule 3 instead of PWM23.
 *  0b1..A logic 1 is supplied to the deadtime generator of submodule 3 instead of PWM23.
 */
#define PWM_SWCOUT_SM3OUT23(x)                   (((uint16_t)(((uint16_t)(x)) << PWM_SWCOUT_SM3OUT23_SHIFT)) & PWM_SWCOUT_SM3OUT23_MASK)
/*! @} */

/*! @name DTSRCSEL - PWM Source Select Register */
/*! @{ */

#define PWM_DTSRCSEL_SM0SEL45_MASK               (0x3U)
#define PWM_DTSRCSEL_SM0SEL45_SHIFT              (0U)
/*! SM0SEL45 - Submodule 0 PWM45 Control Select
 *  0b00..Generated SM0PWM45 signal used by the deadtime logic.
 *  0b01..Inverted generated SM0PWM45 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM0OUT45] used by the deadtime logic.
 *  0b11..Reserved
 */
#define PWM_DTSRCSEL_SM0SEL45(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM0SEL45_SHIFT)) & PWM_DTSRCSEL_SM0SEL45_MASK)

#define PWM_DTSRCSEL_SM0SEL23_MASK               (0xCU)
#define PWM_DTSRCSEL_SM0SEL23_SHIFT              (2U)
/*! SM0SEL23 - Submodule 0 PWM23 Control Select
 *  0b00..Generated SM0PWM23 signal used by the deadtime logic.
 *  0b01..Inverted generated SM0PWM23 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM0OUT23] used by the deadtime logic.
 *  0b11..PWM0_EXTA signal used by the deadtime logic.
 */
#define PWM_DTSRCSEL_SM0SEL23(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM0SEL23_SHIFT)) & PWM_DTSRCSEL_SM0SEL23_MASK)

#define PWM_DTSRCSEL_SM1SEL45_MASK               (0x30U)
#define PWM_DTSRCSEL_SM1SEL45_SHIFT              (4U)
/*! SM1SEL45 - Submodule 1 PWM45 Control Select
 *  0b00..Generated SM1PWM45 signal used by the deadtime logic.
 *  0b01..Inverted generated SM1PWM45 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM1OUT45] used by the deadtime logic.
 *  0b11..Reserved
 */
#define PWM_DTSRCSEL_SM1SEL45(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM1SEL45_SHIFT)) & PWM_DTSRCSEL_SM1SEL45_MASK)

#define PWM_DTSRCSEL_SM1SEL23_MASK               (0xC0U)
#define PWM_DTSRCSEL_SM1SEL23_SHIFT              (6U)
/*! SM1SEL23 - Submodule 1 PWM23 Control Select
 *  0b00..Generated SM1PWM23 signal used by the deadtime logic.
 *  0b01..Inverted generated SM1PWM23 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM1OUT23] used by the deadtime logic.
 *  0b11..PWM1_EXTA signal used by the deadtime logic.
 */
#define PWM_DTSRCSEL_SM1SEL23(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM1SEL23_SHIFT)) & PWM_DTSRCSEL_SM1SEL23_MASK)

#define PWM_DTSRCSEL_SM2SEL45_MASK               (0x300U)
#define PWM_DTSRCSEL_SM2SEL45_SHIFT              (8U)
/*! SM2SEL45 - Submodule 2 PWM45 Control Select
 *  0b00..Generated SM2PWM45 signal used by the deadtime logic.
 *  0b01..Inverted generated SM2PWM45 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM2OUT45] used by the deadtime logic.
 *  0b11..Reserved
 */
#define PWM_DTSRCSEL_SM2SEL45(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM2SEL45_SHIFT)) & PWM_DTSRCSEL_SM2SEL45_MASK)

#define PWM_DTSRCSEL_SM2SEL23_MASK               (0xC00U)
#define PWM_DTSRCSEL_SM2SEL23_SHIFT              (10U)
/*! SM2SEL23 - Submodule 2 PWM23 Control Select
 *  0b00..Generated SM2PWM23 signal used by the deadtime logic.
 *  0b01..Inverted generated SM2PWM23 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM2OUT23] used by the deadtime logic.
 *  0b11..PWM2_EXTA signal used by the deadtime logic.
 */
#define PWM_DTSRCSEL_SM2SEL23(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM2SEL23_SHIFT)) & PWM_DTSRCSEL_SM2SEL23_MASK)

#define PWM_DTSRCSEL_SM3SEL45_MASK               (0x3000U)
#define PWM_DTSRCSEL_SM3SEL45_SHIFT              (12U)
/*! SM3SEL45 - Submodule 3 PWM45 Control Select
 *  0b00..Generated SM3PWM45 signal used by the deadtime logic.
 *  0b01..Inverted generated SM3PWM45 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM3OUT45] used by the deadtime logic.
 *  0b11..Reserved
 */
#define PWM_DTSRCSEL_SM3SEL45(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM3SEL45_SHIFT)) & PWM_DTSRCSEL_SM3SEL45_MASK)

#define PWM_DTSRCSEL_SM3SEL23_MASK               (0xC000U)
#define PWM_DTSRCSEL_SM3SEL23_SHIFT              (14U)
/*! SM3SEL23 - Submodule 3 PWM23 Control Select
 *  0b00..Generated SM3PWM23 signal used by the deadtime logic.
 *  0b01..Inverted generated SM3PWM23 signal used by the deadtime logic.
 *  0b10..SWCOUT[SM3OUT23] used by the deadtime logic.
 *  0b11..PWM3_EXTA signal used by the deadtime logic.
 */
#define PWM_DTSRCSEL_SM3SEL23(x)                 (((uint16_t)(((uint16_t)(x)) << PWM_DTSRCSEL_SM3SEL23_SHIFT)) & PWM_DTSRCSEL_SM3SEL23_MASK)
/*! @} */

/*! @name MCTRL - Master Control Register */
/*! @{ */

#define PWM_MCTRL_LDOK_MASK                      (0xFU)
#define PWM_MCTRL_LDOK_SHIFT                     (0U)
/*! LDOK - Load Okay
 *  0b0000..Do not load new values.
 *  0b0001..Load prescaler, modulus, and PWM values of the corresponding submodule.
 */
#define PWM_MCTRL_LDOK(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_MCTRL_LDOK_SHIFT)) & PWM_MCTRL_LDOK_MASK)

#define PWM_MCTRL_CLDOK_MASK                     (0xF0U)
#define PWM_MCTRL_CLDOK_SHIFT                    (4U)
/*! CLDOK - Clear Load Okay */
#define PWM_MCTRL_CLDOK(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_MCTRL_CLDOK_SHIFT)) & PWM_MCTRL_CLDOK_MASK)

#define PWM_MCTRL_RUN_MASK                       (0xF00U)
#define PWM_MCTRL_RUN_SHIFT                      (8U)
/*! RUN - Run
 *  0b0000..PWM counter is stopped, but PWM outputs hold the current state.
 *  0b0001..PWM counter is started in the corresponding submodule.
 */
#define PWM_MCTRL_RUN(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_MCTRL_RUN_SHIFT)) & PWM_MCTRL_RUN_MASK)

#define PWM_MCTRL_IPOL_MASK                      (0xF000U)
#define PWM_MCTRL_IPOL_SHIFT                     (12U)
/*! IPOL - Current Polarity
 *  0b0000..PWM23 is used to generate complementary PWM pair in the corresponding submodule.
 *  0b0001..PWM45 is used to generate complementary PWM pair in the corresponding submodule.
 */
#define PWM_MCTRL_IPOL(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_MCTRL_IPOL_SHIFT)) & PWM_MCTRL_IPOL_MASK)
/*! @} */

/*! @name MCTRL2 - Master Control 2 Register */
/*! @{ */

#define PWM_MCTRL2_WRPROT_MASK                   (0xCU)
#define PWM_MCTRL2_WRPROT_SHIFT                  (2U)
/*! WRPROT - Write protect
 *  0b00..Write protection off (default).
 *  0b01..Write protection on.
 *  0b10..Write protection off and locked until chip reset.
 *  0b11..Write protection on and locked until chip reset.
 */
#define PWM_MCTRL2_WRPROT(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_MCTRL2_WRPROT_SHIFT)) & PWM_MCTRL2_WRPROT_MASK)
/*! @} */

/*! @name FCTRL - Fault Control Register */
/*! @{ */

#define PWM_FCTRL_FIE_MASK                       (0xFU)
#define PWM_FCTRL_FIE_SHIFT                      (0U)
/*! FIE - Fault Interrupt Enables
 *  0b0000..FAULTx CPU interrupt requests disabled.
 *  0b0001..FAULTx CPU interrupt requests enabled.
 */
#define PWM_FCTRL_FIE(x)                         (((uint16_t)(((uint16_t)(x)) << PWM_FCTRL_FIE_SHIFT)) & PWM_FCTRL_FIE_MASK)

#define PWM_FCTRL_FSAFE_MASK                     (0xF0U)
#define PWM_FCTRL_FSAFE_SHIFT                    (4U)
/*! FSAFE - Fault Safety Mode
 *  0b0000..Normal mode. PWM outputs disabled by this fault are not enabled until FSTS[FFLAGx] is clear at the
 *          start of a half cycle or full cycle depending on the states of FSTS[FHALF] and FSTS[FFULL] without regard
 *          to the state of FSTS[FFPINx]. If neither FHALF nor FFULL is set, then the fault condition cannot be
 *          cleared. The PWM outputs disabled by this fault input will not be re-enabled until the actual FAULTx input
 *          signal de-asserts since the fault input will combinationally disable the PWM outputs (as programmed in
 *          DISMAPn).
 *  0b0001..Safe mode. PWM outputs disabled by this fault are not enabled until FSTS[FFLAGx] is clear and
 *          FSTS[FFPINx] is clear at the start of a half cycle or full cycle depending on the states of FSTS[FHALF] and
 *          FSTS[FFULL]. If neither FHLAF nor FFULL is set, then the fault condition cannot be cleared.
 */
#define PWM_FCTRL_FSAFE(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_FCTRL_FSAFE_SHIFT)) & PWM_FCTRL_FSAFE_MASK)

#define PWM_FCTRL_FAUTO_MASK                     (0xF00U)
#define PWM_FCTRL_FAUTO_SHIFT                    (8U)
/*! FAUTO - Automatic Fault Clearing
 *  0b0000..Manual fault clearing. PWM outputs disabled by this fault are not enabled until FSTS[FFLAGx] is clear
 *          at the start of a half cycle or full cycle depending on the states of FSTS[FHALF] and FSTS[FFULL]. If
 *          neither FFULL nor FHALF is set, then the fault condition cannot be cleared. This is further controlled
 *          by FCTRL[FSAFE].
 *  0b0001..Automatic fault clearing. PWM outputs disabled by this fault are enabled when FSTS[FFPINx] is clear at
 *          the start of a half cycle or full cycle depending on the states of FSTS[FHALF] and FSTS[FFULL] without
 *          regard to the state of FSTS[FFLAGx]. If neither FFULL nor FHALF is set, then the fault condition
 *          cannot be cleared.
 */
#define PWM_FCTRL_FAUTO(x)                       (((uint16_t)(((uint16_t)(x)) << PWM_FCTRL_FAUTO_SHIFT)) & PWM_FCTRL_FAUTO_MASK)

#define PWM_FCTRL_FLVL_MASK                      (0xF000U)
#define PWM_FCTRL_FLVL_SHIFT                     (12U)
/*! FLVL - Fault Level
 *  0b0000..A logic 0 on the fault input indicates a fault condition.
 *  0b0001..A logic 1 on the fault input indicates a fault condition.
 */
#define PWM_FCTRL_FLVL(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_FCTRL_FLVL_SHIFT)) & PWM_FCTRL_FLVL_MASK)
/*! @} */

/*! @name FSTS - Fault Status Register */
/*! @{ */

#define PWM_FSTS_FFLAG_MASK                      (0xFU)
#define PWM_FSTS_FFLAG_SHIFT                     (0U)
/*! FFLAG - Fault Flags
 *  0b0000..No fault on the FAULTx pin.
 *  0b0001..Fault on the FAULTx pin.
 */
#define PWM_FSTS_FFLAG(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_FSTS_FFLAG_SHIFT)) & PWM_FSTS_FFLAG_MASK)

#define PWM_FSTS_FFULL_MASK                      (0xF0U)
#define PWM_FSTS_FFULL_SHIFT                     (4U)
/*! FFULL - Full Cycle
 *  0b0000..PWM outputs are not re-enabled at the start of a full cycle
 *  0b0001..PWM outputs are re-enabled at the start of a full cycle
 */
#define PWM_FSTS_FFULL(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_FSTS_FFULL_SHIFT)) & PWM_FSTS_FFULL_MASK)

#define PWM_FSTS_FFPIN_MASK                      (0xF00U)
#define PWM_FSTS_FFPIN_SHIFT                     (8U)
/*! FFPIN - Filtered Fault Pins */
#define PWM_FSTS_FFPIN(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_FSTS_FFPIN_SHIFT)) & PWM_FSTS_FFPIN_MASK)

#define PWM_FSTS_FHALF_MASK                      (0xF000U)
#define PWM_FSTS_FHALF_SHIFT                     (12U)
/*! FHALF - Half Cycle Fault Recovery
 *  0b0000..PWM outputs are not re-enabled at the start of a half cycle.
 *  0b0001..PWM outputs are re-enabled at the start of a half cycle (as defined by VAL0).
 */
#define PWM_FSTS_FHALF(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_FSTS_FHALF_SHIFT)) & PWM_FSTS_FHALF_MASK)
/*! @} */

/*! @name FFILT - Fault Filter Register */
/*! @{ */

#define PWM_FFILT_FILT_PER_MASK                  (0xFFU)
#define PWM_FFILT_FILT_PER_SHIFT                 (0U)
/*! FILT_PER - Fault Filter Period */
#define PWM_FFILT_FILT_PER(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_FFILT_FILT_PER_SHIFT)) & PWM_FFILT_FILT_PER_MASK)

#define PWM_FFILT_FILT_CNT_MASK                  (0x700U)
#define PWM_FFILT_FILT_CNT_SHIFT                 (8U)
/*! FILT_CNT - Fault Filter Count */
#define PWM_FFILT_FILT_CNT(x)                    (((uint16_t)(((uint16_t)(x)) << PWM_FFILT_FILT_CNT_SHIFT)) & PWM_FFILT_FILT_CNT_MASK)

#define PWM_FFILT_GSTR_MASK                      (0x8000U)
#define PWM_FFILT_GSTR_SHIFT                     (15U)
/*! GSTR - Fault Glitch Stretch Enable
 *  0b0..Fault input glitch stretching is disabled.
 *  0b1..Input fault signals are stretched to at least 2 IPBus clock cycles.
 */
#define PWM_FFILT_GSTR(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_FFILT_GSTR_SHIFT)) & PWM_FFILT_GSTR_MASK)
/*! @} */

/*! @name FTST - Fault Test Register */
/*! @{ */

#define PWM_FTST_FTEST_MASK                      (0x1U)
#define PWM_FTST_FTEST_SHIFT                     (0U)
/*! FTEST - Fault Test
 *  0b0..No fault
 *  0b1..Cause a simulated fault
 */
#define PWM_FTST_FTEST(x)                        (((uint16_t)(((uint16_t)(x)) << PWM_FTST_FTEST_SHIFT)) & PWM_FTST_FTEST_MASK)
/*! @} */

/*! @name FCTRL2 - Fault Control 2 Register */
/*! @{ */

#define PWM_FCTRL2_NOCOMB_MASK                   (0xFU)
#define PWM_FCTRL2_NOCOMB_SHIFT                  (0U)
/*! NOCOMB - No Combinational Path From Fault Input To PWM Output
 *  0b0000..There is a combinational link from the fault inputs to the PWM outputs. The fault inputs are combined
 *          with the filtered and latched fault signals to disable the PWM outputs.
 *  0b0001..The direct combinational path from the fault inputs to the PWM outputs is disabled and the filtered
 *          and latched fault signals are used to disable the PWM outputs.
 */
#define PWM_FCTRL2_NOCOMB(x)                     (((uint16_t)(((uint16_t)(x)) << PWM_FCTRL2_NOCOMB_SHIFT)) & PWM_FCTRL2_NOCOMB_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group PWM_Register_Masks */


/* PWM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral PWM1 base address */
  #define PWM1_BASE                                (0x52650000u)
  /** Peripheral PWM1 base address */
  #define PWM1_BASE_NS                             (0x42650000u)
  /** Peripheral PWM1 base pointer */
  #define PWM1                                     ((PWM_Type *)PWM1_BASE)
  /** Peripheral PWM1 base pointer */
  #define PWM1_NS                                  ((PWM_Type *)PWM1_BASE_NS)
  /** Peripheral PWM2 base address */
  #define PWM2_BASE                                (0x52660000u)
  /** Peripheral PWM2 base address */
  #define PWM2_BASE_NS                             (0x42660000u)
  /** Peripheral PWM2 base pointer */
  #define PWM2                                     ((PWM_Type *)PWM2_BASE)
  /** Peripheral PWM2 base pointer */
  #define PWM2_NS                                  ((PWM_Type *)PWM2_BASE_NS)
  /** Array initializer of PWM peripheral base addresses */
  #define PWM_BASE_ADDRS                           { 0u, PWM1_BASE, PWM2_BASE }
  /** Array initializer of PWM peripheral base pointers */
  #define PWM_BASE_PTRS                            { (PWM_Type *)0u, PWM1, PWM2 }
  /** Array initializer of PWM peripheral base addresses */
  #define PWM_BASE_ADDRS_NS                        { 0u, PWM1_BASE_NS, PWM2_BASE_NS }
  /** Array initializer of PWM peripheral base pointers */
  #define PWM_BASE_PTRS_NS                         { (PWM_Type *)0u, PWM1_NS, PWM2_NS }
#else
  /** Peripheral PWM1 base address */
  #define PWM1_BASE                                (0x42650000u)
  /** Peripheral PWM1 base pointer */
  #define PWM1                                     ((PWM_Type *)PWM1_BASE)
  /** Peripheral PWM2 base address */
  #define PWM2_BASE                                (0x42660000u)
  /** Peripheral PWM2 base pointer */
  #define PWM2                                     ((PWM_Type *)PWM2_BASE)
  /** Array initializer of PWM peripheral base addresses */
  #define PWM_BASE_ADDRS                           { 0u, PWM1_BASE, PWM2_BASE }
  /** Array initializer of PWM peripheral base pointers */
  #define PWM_BASE_PTRS                            { (PWM_Type *)0u, PWM1, PWM2 }
#endif
/** Interrupt vectors for the PWM peripheral type */
#define PWM_CMP_IRQS                             { { NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn }, { PWM1_0_IRQn, PWM1_1_IRQn, PWM1_2_IRQn, PWM1_3_IRQn }, { PWM2_0_IRQn, PWM2_1_IRQn, PWM2_2_IRQn, PWM2_3_IRQn } }
#define PWM_RELOAD_IRQS                          { { NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn }, { PWM1_0_IRQn, PWM1_1_IRQn, PWM1_2_IRQn, PWM1_3_IRQn }, { PWM2_0_IRQn, PWM2_1_IRQn, PWM2_2_IRQn, PWM2_3_IRQn } }
#define PWM_CAPTURE_IRQS                         { { NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn, NotAvail_IRQn }, { PWM1_0_IRQn, PWM1_1_IRQn, PWM1_2_IRQn, PWM1_3_IRQn }, { PWM2_0_IRQn, PWM2_1_IRQn, PWM2_2_IRQn, PWM2_3_IRQn } }
#define PWM_FAULT_IRQS                           { NotAvail_IRQn, PWM1_FAULT_IRQn, PWM2_FAULT_IRQn }
#define PWM_RELOAD_ERROR_IRQS                    { NotAvail_IRQn, PWM1_FAULT_IRQn, PWM2_FAULT_IRQn }

/*!
 * @}
 */ /* end of group PWM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- RGPIO Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup RGPIO_Peripheral_Access_Layer RGPIO Peripheral Access Layer
 * @{
 */

/** RGPIO - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
       uint8_t RESERVED_0[4];
  __IO uint32_t LOCK;                              /**< Lock, offset: 0xC */
  __IO uint32_t PCNS;                              /**< Pin Control Non-Secure, offset: 0x10 */
  __IO uint32_t ICNS;                              /**< Interrupt Control Non-Secure, offset: 0x14 */
  __IO uint32_t PCNP;                              /**< Pin Control Non-Privilege, offset: 0x18 */
  __IO uint32_t ICNP;                              /**< Interrupt Control Non-Privilege, offset: 0x1C */
       uint8_t RESERVED_1[32];
  __IO uint32_t PDOR;                              /**< Port Data Output Register, offset: 0x40 */
  __O  uint32_t PSOR;                              /**< Port Set Output Register, offset: 0x44 */
  __O  uint32_t PCOR;                              /**< Port Clear Output Register, offset: 0x48 */
  __O  uint32_t PTOR;                              /**< Port Toggle Output Register, offset: 0x4C */
  __I  uint32_t PDIR;                              /**< Port Data Input Register, offset: 0x50 */
  __IO uint32_t PDDR;                              /**< Port Data Direction Register, offset: 0x54 */
  __IO uint32_t PIDR;                              /**< Port Input Disable Register, offset: 0x58 */
       uint8_t RESERVED_2[4];
  __IO uint8_t PDR[32];                            /**< Pin Data Register a, array offset: 0x60, array step: 0x1 */
  __IO uint32_t ICR[32];                           /**< Interrupt Control Register 0..Interrupt Control Register 31, array offset: 0x80, array step: 0x4 */
  __O  uint32_t GICLR;                             /**< Global Interrupt Control Low Register, offset: 0x100 */
  __O  uint32_t GICHR;                             /**< Global Interrupt Control High Register, offset: 0x104 */
       uint8_t RESERVED_3[24];
  __IO uint32_t ISFR[2];                           /**< Interrupt Status Flag Register, array offset: 0x120, array step: 0x4 */
} RGPIO_Type;

/* ----------------------------------------------------------------------------
   -- RGPIO Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup RGPIO_Register_Masks RGPIO Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define RGPIO_VERID_FEATURE_MASK                 (0xFFFFU)
#define RGPIO_VERID_FEATURE_SHIFT                (0U)
/*! FEATURE - Feature Specification Number
 *  0b0000000000000000..Basic implementation.
 *  0b0000000000000001..Protection registers implemented.
 */
#define RGPIO_VERID_FEATURE(x)                   (((uint32_t)(((uint32_t)(x)) << RGPIO_VERID_FEATURE_SHIFT)) & RGPIO_VERID_FEATURE_MASK)

#define RGPIO_VERID_MINOR_MASK                   (0xFF0000U)
#define RGPIO_VERID_MINOR_SHIFT                  (16U)
/*! MINOR - Minor Version Number */
#define RGPIO_VERID_MINOR(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_VERID_MINOR_SHIFT)) & RGPIO_VERID_MINOR_MASK)

#define RGPIO_VERID_MAJOR_MASK                   (0xFF000000U)
#define RGPIO_VERID_MAJOR_SHIFT                  (24U)
/*! MAJOR - Major Version Number */
#define RGPIO_VERID_MAJOR(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_VERID_MAJOR_SHIFT)) & RGPIO_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define RGPIO_PARAM_IRQNUM_MASK                  (0xFU)
#define RGPIO_PARAM_IRQNUM_SHIFT                 (0U)
/*! IRQNUM - Interrupt Number */
#define RGPIO_PARAM_IRQNUM(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_PARAM_IRQNUM_SHIFT)) & RGPIO_PARAM_IRQNUM_MASK)
/*! @} */

/*! @name LOCK - Lock */
/*! @{ */

#define RGPIO_LOCK_PCNS_MASK                     (0x1U)
#define RGPIO_LOCK_PCNS_SHIFT                    (0U)
/*! PCNS - Lock PCNS
 *  0b0..PCNS register is writable by software in Secure-Privilege state.
 *  0b1..PCNS register is not writable until the next reset.
 */
#define RGPIO_LOCK_PCNS(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_LOCK_PCNS_SHIFT)) & RGPIO_LOCK_PCNS_MASK)

#define RGPIO_LOCK_ICNS_MASK                     (0x2U)
#define RGPIO_LOCK_ICNS_SHIFT                    (1U)
/*! ICNS - Lock ICNS
 *  0b0..ICNS register is writable by software in Secure-Privilege state.
 *  0b1..ICNS register is not writable until the next reset.
 */
#define RGPIO_LOCK_ICNS(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_LOCK_ICNS_SHIFT)) & RGPIO_LOCK_ICNS_MASK)

#define RGPIO_LOCK_PCNP_MASK                     (0x4U)
#define RGPIO_LOCK_PCNP_SHIFT                    (2U)
/*! PCNP - Lock PCNP
 *  0b0..PCNP register is writable by software in Secure-Privilege state.
 *  0b1..PCNP register is not writable until the next reset.
 */
#define RGPIO_LOCK_PCNP(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_LOCK_PCNP_SHIFT)) & RGPIO_LOCK_PCNP_MASK)

#define RGPIO_LOCK_ICNP_MASK                     (0x8U)
#define RGPIO_LOCK_ICNP_SHIFT                    (3U)
/*! ICNP - Lock ICNP
 *  0b0..ICNP register is writable by software in Secure-Privilege state.
 *  0b1..ICNP register is not writable until the next reset.
 */
#define RGPIO_LOCK_ICNP(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_LOCK_ICNP_SHIFT)) & RGPIO_LOCK_ICNP_MASK)
/*! @} */

/*! @name PCNS - Pin Control Non-Secure */
/*! @{ */

#define RGPIO_PCNS_NSE0_MASK                     (0x1U)
#define RGPIO_PCNS_NSE0_SHIFT                    (0U)
/*! NSE0 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE0_SHIFT)) & RGPIO_PCNS_NSE0_MASK)

#define RGPIO_PCNS_NSE1_MASK                     (0x2U)
#define RGPIO_PCNS_NSE1_SHIFT                    (1U)
/*! NSE1 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE1_SHIFT)) & RGPIO_PCNS_NSE1_MASK)

#define RGPIO_PCNS_NSE2_MASK                     (0x4U)
#define RGPIO_PCNS_NSE2_SHIFT                    (2U)
/*! NSE2 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE2(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE2_SHIFT)) & RGPIO_PCNS_NSE2_MASK)

#define RGPIO_PCNS_NSE3_MASK                     (0x8U)
#define RGPIO_PCNS_NSE3_SHIFT                    (3U)
/*! NSE3 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE3(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE3_SHIFT)) & RGPIO_PCNS_NSE3_MASK)

#define RGPIO_PCNS_NSE4_MASK                     (0x10U)
#define RGPIO_PCNS_NSE4_SHIFT                    (4U)
/*! NSE4 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE4(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE4_SHIFT)) & RGPIO_PCNS_NSE4_MASK)

#define RGPIO_PCNS_NSE5_MASK                     (0x20U)
#define RGPIO_PCNS_NSE5_SHIFT                    (5U)
/*! NSE5 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE5(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE5_SHIFT)) & RGPIO_PCNS_NSE5_MASK)

#define RGPIO_PCNS_NSE6_MASK                     (0x40U)
#define RGPIO_PCNS_NSE6_SHIFT                    (6U)
/*! NSE6 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE6(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE6_SHIFT)) & RGPIO_PCNS_NSE6_MASK)

#define RGPIO_PCNS_NSE7_MASK                     (0x80U)
#define RGPIO_PCNS_NSE7_SHIFT                    (7U)
/*! NSE7 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE7(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE7_SHIFT)) & RGPIO_PCNS_NSE7_MASK)

#define RGPIO_PCNS_NSE8_MASK                     (0x100U)
#define RGPIO_PCNS_NSE8_SHIFT                    (8U)
/*! NSE8 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE8(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE8_SHIFT)) & RGPIO_PCNS_NSE8_MASK)

#define RGPIO_PCNS_NSE9_MASK                     (0x200U)
#define RGPIO_PCNS_NSE9_SHIFT                    (9U)
/*! NSE9 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE9(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE9_SHIFT)) & RGPIO_PCNS_NSE9_MASK)

#define RGPIO_PCNS_NSE10_MASK                    (0x400U)
#define RGPIO_PCNS_NSE10_SHIFT                   (10U)
/*! NSE10 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE10(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE10_SHIFT)) & RGPIO_PCNS_NSE10_MASK)

#define RGPIO_PCNS_NSE11_MASK                    (0x800U)
#define RGPIO_PCNS_NSE11_SHIFT                   (11U)
/*! NSE11 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE11(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE11_SHIFT)) & RGPIO_PCNS_NSE11_MASK)

#define RGPIO_PCNS_NSE12_MASK                    (0x1000U)
#define RGPIO_PCNS_NSE12_SHIFT                   (12U)
/*! NSE12 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE12(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE12_SHIFT)) & RGPIO_PCNS_NSE12_MASK)

#define RGPIO_PCNS_NSE13_MASK                    (0x2000U)
#define RGPIO_PCNS_NSE13_SHIFT                   (13U)
/*! NSE13 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE13(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE13_SHIFT)) & RGPIO_PCNS_NSE13_MASK)

#define RGPIO_PCNS_NSE14_MASK                    (0x4000U)
#define RGPIO_PCNS_NSE14_SHIFT                   (14U)
/*! NSE14 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE14(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE14_SHIFT)) & RGPIO_PCNS_NSE14_MASK)

#define RGPIO_PCNS_NSE15_MASK                    (0x8000U)
#define RGPIO_PCNS_NSE15_SHIFT                   (15U)
/*! NSE15 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE15(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE15_SHIFT)) & RGPIO_PCNS_NSE15_MASK)

#define RGPIO_PCNS_NSE16_MASK                    (0x10000U)
#define RGPIO_PCNS_NSE16_SHIFT                   (16U)
/*! NSE16 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE16(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE16_SHIFT)) & RGPIO_PCNS_NSE16_MASK)

#define RGPIO_PCNS_NSE17_MASK                    (0x20000U)
#define RGPIO_PCNS_NSE17_SHIFT                   (17U)
/*! NSE17 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE17(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE17_SHIFT)) & RGPIO_PCNS_NSE17_MASK)

#define RGPIO_PCNS_NSE18_MASK                    (0x40000U)
#define RGPIO_PCNS_NSE18_SHIFT                   (18U)
/*! NSE18 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE18(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE18_SHIFT)) & RGPIO_PCNS_NSE18_MASK)

#define RGPIO_PCNS_NSE19_MASK                    (0x80000U)
#define RGPIO_PCNS_NSE19_SHIFT                   (19U)
/*! NSE19 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE19(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE19_SHIFT)) & RGPIO_PCNS_NSE19_MASK)

#define RGPIO_PCNS_NSE20_MASK                    (0x100000U)
#define RGPIO_PCNS_NSE20_SHIFT                   (20U)
/*! NSE20 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE20(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE20_SHIFT)) & RGPIO_PCNS_NSE20_MASK)

#define RGPIO_PCNS_NSE21_MASK                    (0x200000U)
#define RGPIO_PCNS_NSE21_SHIFT                   (21U)
/*! NSE21 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE21(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE21_SHIFT)) & RGPIO_PCNS_NSE21_MASK)

#define RGPIO_PCNS_NSE22_MASK                    (0x400000U)
#define RGPIO_PCNS_NSE22_SHIFT                   (22U)
/*! NSE22 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE22(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE22_SHIFT)) & RGPIO_PCNS_NSE22_MASK)

#define RGPIO_PCNS_NSE23_MASK                    (0x800000U)
#define RGPIO_PCNS_NSE23_SHIFT                   (23U)
/*! NSE23 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE23(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE23_SHIFT)) & RGPIO_PCNS_NSE23_MASK)

#define RGPIO_PCNS_NSE24_MASK                    (0x1000000U)
#define RGPIO_PCNS_NSE24_SHIFT                   (24U)
/*! NSE24 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE24(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE24_SHIFT)) & RGPIO_PCNS_NSE24_MASK)

#define RGPIO_PCNS_NSE25_MASK                    (0x2000000U)
#define RGPIO_PCNS_NSE25_SHIFT                   (25U)
/*! NSE25 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE25(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE25_SHIFT)) & RGPIO_PCNS_NSE25_MASK)

#define RGPIO_PCNS_NSE26_MASK                    (0x4000000U)
#define RGPIO_PCNS_NSE26_SHIFT                   (26U)
/*! NSE26 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE26(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE26_SHIFT)) & RGPIO_PCNS_NSE26_MASK)

#define RGPIO_PCNS_NSE27_MASK                    (0x8000000U)
#define RGPIO_PCNS_NSE27_SHIFT                   (27U)
/*! NSE27 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE27(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE27_SHIFT)) & RGPIO_PCNS_NSE27_MASK)

#define RGPIO_PCNS_NSE28_MASK                    (0x10000000U)
#define RGPIO_PCNS_NSE28_SHIFT                   (28U)
/*! NSE28 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE28(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE28_SHIFT)) & RGPIO_PCNS_NSE28_MASK)

#define RGPIO_PCNS_NSE29_MASK                    (0x20000000U)
#define RGPIO_PCNS_NSE29_SHIFT                   (29U)
/*! NSE29 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE29(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE29_SHIFT)) & RGPIO_PCNS_NSE29_MASK)

#define RGPIO_PCNS_NSE30_MASK                    (0x40000000U)
#define RGPIO_PCNS_NSE30_SHIFT                   (30U)
/*! NSE30 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE30(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE30_SHIFT)) & RGPIO_PCNS_NSE30_MASK)

#define RGPIO_PCNS_NSE31_MASK                    (0x80000000U)
#define RGPIO_PCNS_NSE31_SHIFT                   (31U)
/*! NSE31 - Non-Secure Enable
 *  0b0..The pin is configured for Secure access. Read or write access to the corresponding pin's registers and
 *       bit fields is only allowed by software in Secure state. When the corresponding pin's registers are accessed
 *       by software in Non-Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 *  0b1..The pin is configured for Non-Secure access. Read or write access to the corresponding pin's registers
 *       and bit fields is only allowed by software in Non-Secure state. When the corresponding pin's registers are
 *       accessed by software in Secure state, all bits in the registers related to that pin are read zero and write
 *       ignored.
 */
#define RGPIO_PCNS_NSE31(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNS_NSE31_SHIFT)) & RGPIO_PCNS_NSE31_MASK)
/*! @} */

/*! @name ICNS - Interrupt Control Non-Secure */
/*! @{ */

#define RGPIO_ICNS_NSE0_MASK                     (0x1U)
#define RGPIO_ICNS_NSE0_SHIFT                    (0U)
/*! NSE0 - Non-Secure Enable
 *  0b0..The interrupt or DMA request is configured for Secure access. Only software in Secure state can configure
 *       a pin to use the corresponding interrupt or DMA request or reconfigure a pin that is already configured
 *       to use the corresponding interrupt or DMA request.
 *  0b1..The interrupt or DMA request is configured for Non-Secure access. Only software in Non-Secure state can
 *       configure a pin to use the corresponding interrupt or DMA request or reconfigure a pin that is already
 *       configured to use the corresponding interrupt or DMA request.
 */
#define RGPIO_ICNS_NSE0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ICNS_NSE0_SHIFT)) & RGPIO_ICNS_NSE0_MASK)

#define RGPIO_ICNS_NSE1_MASK                     (0x2U)
#define RGPIO_ICNS_NSE1_SHIFT                    (1U)
/*! NSE1 - Non-Secure Enable
 *  0b0..The interrupt or DMA request is configured for Secure access. Only software in Secure state can configure
 *       a pin to use the corresponding interrupt or DMA request or reconfigure a pin that is already configured
 *       to use the corresponding interrupt or DMA request.
 *  0b1..The interrupt or DMA request is configured for Non-Secure access. Only software in Non-Secure state can
 *       configure a pin to use the corresponding interrupt or DMA request or reconfigure a pin that is already
 *       configured to use the corresponding interrupt or DMA request.
 */
#define RGPIO_ICNS_NSE1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ICNS_NSE1_SHIFT)) & RGPIO_ICNS_NSE1_MASK)
/*! @} */

/*! @name PCNP - Pin Control Non-Privilege */
/*! @{ */

#define RGPIO_PCNP_NPE0_MASK                     (0x1U)
#define RGPIO_PCNP_NPE0_SHIFT                    (0U)
/*! NPE0 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE0_SHIFT)) & RGPIO_PCNP_NPE0_MASK)

#define RGPIO_PCNP_NPE1_MASK                     (0x2U)
#define RGPIO_PCNP_NPE1_SHIFT                    (1U)
/*! NPE1 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE1_SHIFT)) & RGPIO_PCNP_NPE1_MASK)

#define RGPIO_PCNP_NPE2_MASK                     (0x4U)
#define RGPIO_PCNP_NPE2_SHIFT                    (2U)
/*! NPE2 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE2(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE2_SHIFT)) & RGPIO_PCNP_NPE2_MASK)

#define RGPIO_PCNP_NPE3_MASK                     (0x8U)
#define RGPIO_PCNP_NPE3_SHIFT                    (3U)
/*! NPE3 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE3(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE3_SHIFT)) & RGPIO_PCNP_NPE3_MASK)

#define RGPIO_PCNP_NPE4_MASK                     (0x10U)
#define RGPIO_PCNP_NPE4_SHIFT                    (4U)
/*! NPE4 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE4(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE4_SHIFT)) & RGPIO_PCNP_NPE4_MASK)

#define RGPIO_PCNP_NPE5_MASK                     (0x20U)
#define RGPIO_PCNP_NPE5_SHIFT                    (5U)
/*! NPE5 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE5(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE5_SHIFT)) & RGPIO_PCNP_NPE5_MASK)

#define RGPIO_PCNP_NPE6_MASK                     (0x40U)
#define RGPIO_PCNP_NPE6_SHIFT                    (6U)
/*! NPE6 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE6(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE6_SHIFT)) & RGPIO_PCNP_NPE6_MASK)

#define RGPIO_PCNP_NPE7_MASK                     (0x80U)
#define RGPIO_PCNP_NPE7_SHIFT                    (7U)
/*! NPE7 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE7(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE7_SHIFT)) & RGPIO_PCNP_NPE7_MASK)

#define RGPIO_PCNP_NPE8_MASK                     (0x100U)
#define RGPIO_PCNP_NPE8_SHIFT                    (8U)
/*! NPE8 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE8(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE8_SHIFT)) & RGPIO_PCNP_NPE8_MASK)

#define RGPIO_PCNP_NPE9_MASK                     (0x200U)
#define RGPIO_PCNP_NPE9_SHIFT                    (9U)
/*! NPE9 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE9(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE9_SHIFT)) & RGPIO_PCNP_NPE9_MASK)

#define RGPIO_PCNP_NPE10_MASK                    (0x400U)
#define RGPIO_PCNP_NPE10_SHIFT                   (10U)
/*! NPE10 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE10(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE10_SHIFT)) & RGPIO_PCNP_NPE10_MASK)

#define RGPIO_PCNP_NPE11_MASK                    (0x800U)
#define RGPIO_PCNP_NPE11_SHIFT                   (11U)
/*! NPE11 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE11(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE11_SHIFT)) & RGPIO_PCNP_NPE11_MASK)

#define RGPIO_PCNP_NPE12_MASK                    (0x1000U)
#define RGPIO_PCNP_NPE12_SHIFT                   (12U)
/*! NPE12 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE12(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE12_SHIFT)) & RGPIO_PCNP_NPE12_MASK)

#define RGPIO_PCNP_NPE13_MASK                    (0x2000U)
#define RGPIO_PCNP_NPE13_SHIFT                   (13U)
/*! NPE13 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE13(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE13_SHIFT)) & RGPIO_PCNP_NPE13_MASK)

#define RGPIO_PCNP_NPE14_MASK                    (0x4000U)
#define RGPIO_PCNP_NPE14_SHIFT                   (14U)
/*! NPE14 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE14(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE14_SHIFT)) & RGPIO_PCNP_NPE14_MASK)

#define RGPIO_PCNP_NPE15_MASK                    (0x8000U)
#define RGPIO_PCNP_NPE15_SHIFT                   (15U)
/*! NPE15 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE15(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE15_SHIFT)) & RGPIO_PCNP_NPE15_MASK)

#define RGPIO_PCNP_NPE16_MASK                    (0x10000U)
#define RGPIO_PCNP_NPE16_SHIFT                   (16U)
/*! NPE16 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE16(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE16_SHIFT)) & RGPIO_PCNP_NPE16_MASK)

#define RGPIO_PCNP_NPE17_MASK                    (0x20000U)
#define RGPIO_PCNP_NPE17_SHIFT                   (17U)
/*! NPE17 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE17(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE17_SHIFT)) & RGPIO_PCNP_NPE17_MASK)

#define RGPIO_PCNP_NPE18_MASK                    (0x40000U)
#define RGPIO_PCNP_NPE18_SHIFT                   (18U)
/*! NPE18 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE18(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE18_SHIFT)) & RGPIO_PCNP_NPE18_MASK)

#define RGPIO_PCNP_NPE19_MASK                    (0x80000U)
#define RGPIO_PCNP_NPE19_SHIFT                   (19U)
/*! NPE19 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE19(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE19_SHIFT)) & RGPIO_PCNP_NPE19_MASK)

#define RGPIO_PCNP_NPE20_MASK                    (0x100000U)
#define RGPIO_PCNP_NPE20_SHIFT                   (20U)
/*! NPE20 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE20(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE20_SHIFT)) & RGPIO_PCNP_NPE20_MASK)

#define RGPIO_PCNP_NPE21_MASK                    (0x200000U)
#define RGPIO_PCNP_NPE21_SHIFT                   (21U)
/*! NPE21 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE21(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE21_SHIFT)) & RGPIO_PCNP_NPE21_MASK)

#define RGPIO_PCNP_NPE22_MASK                    (0x400000U)
#define RGPIO_PCNP_NPE22_SHIFT                   (22U)
/*! NPE22 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE22(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE22_SHIFT)) & RGPIO_PCNP_NPE22_MASK)

#define RGPIO_PCNP_NPE23_MASK                    (0x800000U)
#define RGPIO_PCNP_NPE23_SHIFT                   (23U)
/*! NPE23 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE23(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE23_SHIFT)) & RGPIO_PCNP_NPE23_MASK)

#define RGPIO_PCNP_NPE24_MASK                    (0x1000000U)
#define RGPIO_PCNP_NPE24_SHIFT                   (24U)
/*! NPE24 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE24(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE24_SHIFT)) & RGPIO_PCNP_NPE24_MASK)

#define RGPIO_PCNP_NPE25_MASK                    (0x2000000U)
#define RGPIO_PCNP_NPE25_SHIFT                   (25U)
/*! NPE25 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE25(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE25_SHIFT)) & RGPIO_PCNP_NPE25_MASK)

#define RGPIO_PCNP_NPE26_MASK                    (0x4000000U)
#define RGPIO_PCNP_NPE26_SHIFT                   (26U)
/*! NPE26 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE26(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE26_SHIFT)) & RGPIO_PCNP_NPE26_MASK)

#define RGPIO_PCNP_NPE27_MASK                    (0x8000000U)
#define RGPIO_PCNP_NPE27_SHIFT                   (27U)
/*! NPE27 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE27(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE27_SHIFT)) & RGPIO_PCNP_NPE27_MASK)

#define RGPIO_PCNP_NPE28_MASK                    (0x10000000U)
#define RGPIO_PCNP_NPE28_SHIFT                   (28U)
/*! NPE28 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE28(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE28_SHIFT)) & RGPIO_PCNP_NPE28_MASK)

#define RGPIO_PCNP_NPE29_MASK                    (0x20000000U)
#define RGPIO_PCNP_NPE29_SHIFT                   (29U)
/*! NPE29 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE29(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE29_SHIFT)) & RGPIO_PCNP_NPE29_MASK)

#define RGPIO_PCNP_NPE30_MASK                    (0x40000000U)
#define RGPIO_PCNP_NPE30_SHIFT                   (30U)
/*! NPE30 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE30(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE30_SHIFT)) & RGPIO_PCNP_NPE30_MASK)

#define RGPIO_PCNP_NPE31_MASK                    (0x80000000U)
#define RGPIO_PCNP_NPE31_SHIFT                   (31U)
/*! NPE31 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Write access to the corresponding pin's registers and bit
 *       fields is allowed only by software in Privilege state. When the corresponding pin's registers and bit fields
 *       are accessed by software in Non-Privilege state, all bits related to that pin in this GPIO are readable
 *       but write ignored.
 *  0b1..The pin is configured for Non-Privilege access, Read or write access to the corresponding pin's registers
 *       is allowed by software in both Privilege and Non-Privilege state.
 */
#define RGPIO_PCNP_NPE31(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCNP_NPE31_SHIFT)) & RGPIO_PCNP_NPE31_MASK)
/*! @} */

/*! @name ICNP - Interrupt Control Non-Privilege */
/*! @{ */

#define RGPIO_ICNP_NPE0_MASK                     (0x1U)
#define RGPIO_ICNP_NPE0_SHIFT                    (0U)
/*! NPE0 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Only software in Privilege state can configure a pin to use
 *       the corresponding interrupt/DMA request or reconfigure a pin that is already configured to use the
 *       corresponding interrupt/DMA request.
 *  0b1..The pin is configured for Non-Privilege access. Software in either Privilege or Non-Privilege state can
 *       configure a pin to use the corresponding interrupt/DMA request or reconfigure a pin that is already
 *       configured to use the corresponding interrupt/DMA request.
 */
#define RGPIO_ICNP_NPE0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ICNP_NPE0_SHIFT)) & RGPIO_ICNP_NPE0_MASK)

#define RGPIO_ICNP_NPE1_MASK                     (0x2U)
#define RGPIO_ICNP_NPE1_SHIFT                    (1U)
/*! NPE1 - Non-Privilege Enable
 *  0b0..The pin is configured for Privilege access. Only software in Privilege state can configure a pin to use
 *       the corresponding interrupt/DMA request or reconfigure a pin that is already configured to use the
 *       corresponding interrupt/DMA request.
 *  0b1..The pin is configured for Non-Privilege access. Software in either Privilege or Non-Privilege state can
 *       configure a pin to use the corresponding interrupt/DMA request or reconfigure a pin that is already
 *       configured to use the corresponding interrupt/DMA request.
 */
#define RGPIO_ICNP_NPE1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ICNP_NPE1_SHIFT)) & RGPIO_ICNP_NPE1_MASK)
/*! @} */

/*! @name PDOR - Port Data Output Register */
/*! @{ */

#define RGPIO_PDOR_PDO0_MASK                     (0x1U)
#define RGPIO_PDOR_PDO0_SHIFT                    (0U)
/*! PDO0 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO0_SHIFT)) & RGPIO_PDOR_PDO0_MASK)

#define RGPIO_PDOR_PDO1_MASK                     (0x2U)
#define RGPIO_PDOR_PDO1_SHIFT                    (1U)
/*! PDO1 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO1_SHIFT)) & RGPIO_PDOR_PDO1_MASK)

#define RGPIO_PDOR_PDO2_MASK                     (0x4U)
#define RGPIO_PDOR_PDO2_SHIFT                    (2U)
/*! PDO2 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO2(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO2_SHIFT)) & RGPIO_PDOR_PDO2_MASK)

#define RGPIO_PDOR_PDO3_MASK                     (0x8U)
#define RGPIO_PDOR_PDO3_SHIFT                    (3U)
/*! PDO3 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO3(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO3_SHIFT)) & RGPIO_PDOR_PDO3_MASK)

#define RGPIO_PDOR_PDO4_MASK                     (0x10U)
#define RGPIO_PDOR_PDO4_SHIFT                    (4U)
/*! PDO4 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO4(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO4_SHIFT)) & RGPIO_PDOR_PDO4_MASK)

#define RGPIO_PDOR_PDO5_MASK                     (0x20U)
#define RGPIO_PDOR_PDO5_SHIFT                    (5U)
/*! PDO5 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO5(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO5_SHIFT)) & RGPIO_PDOR_PDO5_MASK)

#define RGPIO_PDOR_PDO6_MASK                     (0x40U)
#define RGPIO_PDOR_PDO6_SHIFT                    (6U)
/*! PDO6 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO6(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO6_SHIFT)) & RGPIO_PDOR_PDO6_MASK)

#define RGPIO_PDOR_PDO7_MASK                     (0x80U)
#define RGPIO_PDOR_PDO7_SHIFT                    (7U)
/*! PDO7 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO7(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO7_SHIFT)) & RGPIO_PDOR_PDO7_MASK)

#define RGPIO_PDOR_PDO8_MASK                     (0x100U)
#define RGPIO_PDOR_PDO8_SHIFT                    (8U)
/*! PDO8 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO8(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO8_SHIFT)) & RGPIO_PDOR_PDO8_MASK)

#define RGPIO_PDOR_PDO9_MASK                     (0x200U)
#define RGPIO_PDOR_PDO9_SHIFT                    (9U)
/*! PDO9 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO9(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO9_SHIFT)) & RGPIO_PDOR_PDO9_MASK)

#define RGPIO_PDOR_PDO10_MASK                    (0x400U)
#define RGPIO_PDOR_PDO10_SHIFT                   (10U)
/*! PDO10 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO10(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO10_SHIFT)) & RGPIO_PDOR_PDO10_MASK)

#define RGPIO_PDOR_PDO11_MASK                    (0x800U)
#define RGPIO_PDOR_PDO11_SHIFT                   (11U)
/*! PDO11 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO11(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO11_SHIFT)) & RGPIO_PDOR_PDO11_MASK)

#define RGPIO_PDOR_PDO12_MASK                    (0x1000U)
#define RGPIO_PDOR_PDO12_SHIFT                   (12U)
/*! PDO12 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO12(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO12_SHIFT)) & RGPIO_PDOR_PDO12_MASK)

#define RGPIO_PDOR_PDO13_MASK                    (0x2000U)
#define RGPIO_PDOR_PDO13_SHIFT                   (13U)
/*! PDO13 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO13(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO13_SHIFT)) & RGPIO_PDOR_PDO13_MASK)

#define RGPIO_PDOR_PDO14_MASK                    (0x4000U)
#define RGPIO_PDOR_PDO14_SHIFT                   (14U)
/*! PDO14 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO14(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO14_SHIFT)) & RGPIO_PDOR_PDO14_MASK)

#define RGPIO_PDOR_PDO15_MASK                    (0x8000U)
#define RGPIO_PDOR_PDO15_SHIFT                   (15U)
/*! PDO15 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO15(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO15_SHIFT)) & RGPIO_PDOR_PDO15_MASK)

#define RGPIO_PDOR_PDO16_MASK                    (0x10000U)
#define RGPIO_PDOR_PDO16_SHIFT                   (16U)
/*! PDO16 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO16(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO16_SHIFT)) & RGPIO_PDOR_PDO16_MASK)

#define RGPIO_PDOR_PDO17_MASK                    (0x20000U)
#define RGPIO_PDOR_PDO17_SHIFT                   (17U)
/*! PDO17 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO17(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO17_SHIFT)) & RGPIO_PDOR_PDO17_MASK)

#define RGPIO_PDOR_PDO18_MASK                    (0x40000U)
#define RGPIO_PDOR_PDO18_SHIFT                   (18U)
/*! PDO18 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO18(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO18_SHIFT)) & RGPIO_PDOR_PDO18_MASK)

#define RGPIO_PDOR_PDO19_MASK                    (0x80000U)
#define RGPIO_PDOR_PDO19_SHIFT                   (19U)
/*! PDO19 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO19(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO19_SHIFT)) & RGPIO_PDOR_PDO19_MASK)

#define RGPIO_PDOR_PDO20_MASK                    (0x100000U)
#define RGPIO_PDOR_PDO20_SHIFT                   (20U)
/*! PDO20 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO20(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO20_SHIFT)) & RGPIO_PDOR_PDO20_MASK)

#define RGPIO_PDOR_PDO21_MASK                    (0x200000U)
#define RGPIO_PDOR_PDO21_SHIFT                   (21U)
/*! PDO21 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO21(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO21_SHIFT)) & RGPIO_PDOR_PDO21_MASK)

#define RGPIO_PDOR_PDO22_MASK                    (0x400000U)
#define RGPIO_PDOR_PDO22_SHIFT                   (22U)
/*! PDO22 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO22(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO22_SHIFT)) & RGPIO_PDOR_PDO22_MASK)

#define RGPIO_PDOR_PDO23_MASK                    (0x800000U)
#define RGPIO_PDOR_PDO23_SHIFT                   (23U)
/*! PDO23 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO23(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO23_SHIFT)) & RGPIO_PDOR_PDO23_MASK)

#define RGPIO_PDOR_PDO24_MASK                    (0x1000000U)
#define RGPIO_PDOR_PDO24_SHIFT                   (24U)
/*! PDO24 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO24(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO24_SHIFT)) & RGPIO_PDOR_PDO24_MASK)

#define RGPIO_PDOR_PDO25_MASK                    (0x2000000U)
#define RGPIO_PDOR_PDO25_SHIFT                   (25U)
/*! PDO25 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO25(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO25_SHIFT)) & RGPIO_PDOR_PDO25_MASK)

#define RGPIO_PDOR_PDO26_MASK                    (0x4000000U)
#define RGPIO_PDOR_PDO26_SHIFT                   (26U)
/*! PDO26 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO26(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO26_SHIFT)) & RGPIO_PDOR_PDO26_MASK)

#define RGPIO_PDOR_PDO27_MASK                    (0x8000000U)
#define RGPIO_PDOR_PDO27_SHIFT                   (27U)
/*! PDO27 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO27(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO27_SHIFT)) & RGPIO_PDOR_PDO27_MASK)

#define RGPIO_PDOR_PDO28_MASK                    (0x10000000U)
#define RGPIO_PDOR_PDO28_SHIFT                   (28U)
/*! PDO28 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO28(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO28_SHIFT)) & RGPIO_PDOR_PDO28_MASK)

#define RGPIO_PDOR_PDO29_MASK                    (0x20000000U)
#define RGPIO_PDOR_PDO29_SHIFT                   (29U)
/*! PDO29 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO29(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO29_SHIFT)) & RGPIO_PDOR_PDO29_MASK)

#define RGPIO_PDOR_PDO30_MASK                    (0x40000000U)
#define RGPIO_PDOR_PDO30_SHIFT                   (30U)
/*! PDO30 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO30(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO30_SHIFT)) & RGPIO_PDOR_PDO30_MASK)

#define RGPIO_PDOR_PDO31_MASK                    (0x80000000U)
#define RGPIO_PDOR_PDO31_SHIFT                   (31U)
/*! PDO31 - Port Data Output
 *  0b0..Logic level 0 is driven on pin, if the pin is configured for general-purpose output.
 *  0b1..Logic level 1 is driven on pin, if the pin is configured for general-purpose output.
 */
#define RGPIO_PDOR_PDO31(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDOR_PDO31_SHIFT)) & RGPIO_PDOR_PDO31_MASK)
/*! @} */

/*! @name PSOR - Port Set Output Register */
/*! @{ */

#define RGPIO_PSOR_PTSO0_MASK                    (0x1U)
#define RGPIO_PSOR_PTSO0_SHIFT                   (0U)
/*! PTSO0 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO0(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO0_SHIFT)) & RGPIO_PSOR_PTSO0_MASK)

#define RGPIO_PSOR_PTSO1_MASK                    (0x2U)
#define RGPIO_PSOR_PTSO1_SHIFT                   (1U)
/*! PTSO1 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO1(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO1_SHIFT)) & RGPIO_PSOR_PTSO1_MASK)

#define RGPIO_PSOR_PTSO2_MASK                    (0x4U)
#define RGPIO_PSOR_PTSO2_SHIFT                   (2U)
/*! PTSO2 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO2(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO2_SHIFT)) & RGPIO_PSOR_PTSO2_MASK)

#define RGPIO_PSOR_PTSO3_MASK                    (0x8U)
#define RGPIO_PSOR_PTSO3_SHIFT                   (3U)
/*! PTSO3 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO3(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO3_SHIFT)) & RGPIO_PSOR_PTSO3_MASK)

#define RGPIO_PSOR_PTSO4_MASK                    (0x10U)
#define RGPIO_PSOR_PTSO4_SHIFT                   (4U)
/*! PTSO4 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO4(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO4_SHIFT)) & RGPIO_PSOR_PTSO4_MASK)

#define RGPIO_PSOR_PTSO5_MASK                    (0x20U)
#define RGPIO_PSOR_PTSO5_SHIFT                   (5U)
/*! PTSO5 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO5(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO5_SHIFT)) & RGPIO_PSOR_PTSO5_MASK)

#define RGPIO_PSOR_PTSO6_MASK                    (0x40U)
#define RGPIO_PSOR_PTSO6_SHIFT                   (6U)
/*! PTSO6 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO6(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO6_SHIFT)) & RGPIO_PSOR_PTSO6_MASK)

#define RGPIO_PSOR_PTSO7_MASK                    (0x80U)
#define RGPIO_PSOR_PTSO7_SHIFT                   (7U)
/*! PTSO7 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO7(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO7_SHIFT)) & RGPIO_PSOR_PTSO7_MASK)

#define RGPIO_PSOR_PTSO8_MASK                    (0x100U)
#define RGPIO_PSOR_PTSO8_SHIFT                   (8U)
/*! PTSO8 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO8(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO8_SHIFT)) & RGPIO_PSOR_PTSO8_MASK)

#define RGPIO_PSOR_PTSO9_MASK                    (0x200U)
#define RGPIO_PSOR_PTSO9_SHIFT                   (9U)
/*! PTSO9 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO9(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO9_SHIFT)) & RGPIO_PSOR_PTSO9_MASK)

#define RGPIO_PSOR_PTSO10_MASK                   (0x400U)
#define RGPIO_PSOR_PTSO10_SHIFT                  (10U)
/*! PTSO10 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO10(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO10_SHIFT)) & RGPIO_PSOR_PTSO10_MASK)

#define RGPIO_PSOR_PTSO11_MASK                   (0x800U)
#define RGPIO_PSOR_PTSO11_SHIFT                  (11U)
/*! PTSO11 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO11(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO11_SHIFT)) & RGPIO_PSOR_PTSO11_MASK)

#define RGPIO_PSOR_PTSO12_MASK                   (0x1000U)
#define RGPIO_PSOR_PTSO12_SHIFT                  (12U)
/*! PTSO12 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO12(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO12_SHIFT)) & RGPIO_PSOR_PTSO12_MASK)

#define RGPIO_PSOR_PTSO13_MASK                   (0x2000U)
#define RGPIO_PSOR_PTSO13_SHIFT                  (13U)
/*! PTSO13 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO13(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO13_SHIFT)) & RGPIO_PSOR_PTSO13_MASK)

#define RGPIO_PSOR_PTSO14_MASK                   (0x4000U)
#define RGPIO_PSOR_PTSO14_SHIFT                  (14U)
/*! PTSO14 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO14(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO14_SHIFT)) & RGPIO_PSOR_PTSO14_MASK)

#define RGPIO_PSOR_PTSO15_MASK                   (0x8000U)
#define RGPIO_PSOR_PTSO15_SHIFT                  (15U)
/*! PTSO15 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO15(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO15_SHIFT)) & RGPIO_PSOR_PTSO15_MASK)

#define RGPIO_PSOR_PTSO16_MASK                   (0x10000U)
#define RGPIO_PSOR_PTSO16_SHIFT                  (16U)
/*! PTSO16 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO16(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO16_SHIFT)) & RGPIO_PSOR_PTSO16_MASK)

#define RGPIO_PSOR_PTSO17_MASK                   (0x20000U)
#define RGPIO_PSOR_PTSO17_SHIFT                  (17U)
/*! PTSO17 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO17(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO17_SHIFT)) & RGPIO_PSOR_PTSO17_MASK)

#define RGPIO_PSOR_PTSO18_MASK                   (0x40000U)
#define RGPIO_PSOR_PTSO18_SHIFT                  (18U)
/*! PTSO18 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO18(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO18_SHIFT)) & RGPIO_PSOR_PTSO18_MASK)

#define RGPIO_PSOR_PTSO19_MASK                   (0x80000U)
#define RGPIO_PSOR_PTSO19_SHIFT                  (19U)
/*! PTSO19 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO19(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO19_SHIFT)) & RGPIO_PSOR_PTSO19_MASK)

#define RGPIO_PSOR_PTSO20_MASK                   (0x100000U)
#define RGPIO_PSOR_PTSO20_SHIFT                  (20U)
/*! PTSO20 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO20(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO20_SHIFT)) & RGPIO_PSOR_PTSO20_MASK)

#define RGPIO_PSOR_PTSO21_MASK                   (0x200000U)
#define RGPIO_PSOR_PTSO21_SHIFT                  (21U)
/*! PTSO21 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO21(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO21_SHIFT)) & RGPIO_PSOR_PTSO21_MASK)

#define RGPIO_PSOR_PTSO22_MASK                   (0x400000U)
#define RGPIO_PSOR_PTSO22_SHIFT                  (22U)
/*! PTSO22 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO22(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO22_SHIFT)) & RGPIO_PSOR_PTSO22_MASK)

#define RGPIO_PSOR_PTSO23_MASK                   (0x800000U)
#define RGPIO_PSOR_PTSO23_SHIFT                  (23U)
/*! PTSO23 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO23(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO23_SHIFT)) & RGPIO_PSOR_PTSO23_MASK)

#define RGPIO_PSOR_PTSO24_MASK                   (0x1000000U)
#define RGPIO_PSOR_PTSO24_SHIFT                  (24U)
/*! PTSO24 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO24(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO24_SHIFT)) & RGPIO_PSOR_PTSO24_MASK)

#define RGPIO_PSOR_PTSO25_MASK                   (0x2000000U)
#define RGPIO_PSOR_PTSO25_SHIFT                  (25U)
/*! PTSO25 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO25(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO25_SHIFT)) & RGPIO_PSOR_PTSO25_MASK)

#define RGPIO_PSOR_PTSO26_MASK                   (0x4000000U)
#define RGPIO_PSOR_PTSO26_SHIFT                  (26U)
/*! PTSO26 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO26(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO26_SHIFT)) & RGPIO_PSOR_PTSO26_MASK)

#define RGPIO_PSOR_PTSO27_MASK                   (0x8000000U)
#define RGPIO_PSOR_PTSO27_SHIFT                  (27U)
/*! PTSO27 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO27(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO27_SHIFT)) & RGPIO_PSOR_PTSO27_MASK)

#define RGPIO_PSOR_PTSO28_MASK                   (0x10000000U)
#define RGPIO_PSOR_PTSO28_SHIFT                  (28U)
/*! PTSO28 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO28(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO28_SHIFT)) & RGPIO_PSOR_PTSO28_MASK)

#define RGPIO_PSOR_PTSO29_MASK                   (0x20000000U)
#define RGPIO_PSOR_PTSO29_SHIFT                  (29U)
/*! PTSO29 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO29(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO29_SHIFT)) & RGPIO_PSOR_PTSO29_MASK)

#define RGPIO_PSOR_PTSO30_MASK                   (0x40000000U)
#define RGPIO_PSOR_PTSO30_SHIFT                  (30U)
/*! PTSO30 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO30(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO30_SHIFT)) & RGPIO_PSOR_PTSO30_MASK)

#define RGPIO_PSOR_PTSO31_MASK                   (0x80000000U)
#define RGPIO_PSOR_PTSO31_SHIFT                  (31U)
/*! PTSO31 - Port Set Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to logic 1.
 */
#define RGPIO_PSOR_PTSO31(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PSOR_PTSO31_SHIFT)) & RGPIO_PSOR_PTSO31_MASK)
/*! @} */

/*! @name PCOR - Port Clear Output Register */
/*! @{ */

#define RGPIO_PCOR_PTCO0_MASK                    (0x1U)
#define RGPIO_PCOR_PTCO0_SHIFT                   (0U)
/*! PTCO0 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO0(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO0_SHIFT)) & RGPIO_PCOR_PTCO0_MASK)

#define RGPIO_PCOR_PTCO1_MASK                    (0x2U)
#define RGPIO_PCOR_PTCO1_SHIFT                   (1U)
/*! PTCO1 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO1(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO1_SHIFT)) & RGPIO_PCOR_PTCO1_MASK)

#define RGPIO_PCOR_PTCO2_MASK                    (0x4U)
#define RGPIO_PCOR_PTCO2_SHIFT                   (2U)
/*! PTCO2 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO2(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO2_SHIFT)) & RGPIO_PCOR_PTCO2_MASK)

#define RGPIO_PCOR_PTCO3_MASK                    (0x8U)
#define RGPIO_PCOR_PTCO3_SHIFT                   (3U)
/*! PTCO3 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO3(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO3_SHIFT)) & RGPIO_PCOR_PTCO3_MASK)

#define RGPIO_PCOR_PTCO4_MASK                    (0x10U)
#define RGPIO_PCOR_PTCO4_SHIFT                   (4U)
/*! PTCO4 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO4(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO4_SHIFT)) & RGPIO_PCOR_PTCO4_MASK)

#define RGPIO_PCOR_PTCO5_MASK                    (0x20U)
#define RGPIO_PCOR_PTCO5_SHIFT                   (5U)
/*! PTCO5 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO5(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO5_SHIFT)) & RGPIO_PCOR_PTCO5_MASK)

#define RGPIO_PCOR_PTCO6_MASK                    (0x40U)
#define RGPIO_PCOR_PTCO6_SHIFT                   (6U)
/*! PTCO6 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO6(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO6_SHIFT)) & RGPIO_PCOR_PTCO6_MASK)

#define RGPIO_PCOR_PTCO7_MASK                    (0x80U)
#define RGPIO_PCOR_PTCO7_SHIFT                   (7U)
/*! PTCO7 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO7(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO7_SHIFT)) & RGPIO_PCOR_PTCO7_MASK)

#define RGPIO_PCOR_PTCO8_MASK                    (0x100U)
#define RGPIO_PCOR_PTCO8_SHIFT                   (8U)
/*! PTCO8 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO8(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO8_SHIFT)) & RGPIO_PCOR_PTCO8_MASK)

#define RGPIO_PCOR_PTCO9_MASK                    (0x200U)
#define RGPIO_PCOR_PTCO9_SHIFT                   (9U)
/*! PTCO9 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO9(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO9_SHIFT)) & RGPIO_PCOR_PTCO9_MASK)

#define RGPIO_PCOR_PTCO10_MASK                   (0x400U)
#define RGPIO_PCOR_PTCO10_SHIFT                  (10U)
/*! PTCO10 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO10(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO10_SHIFT)) & RGPIO_PCOR_PTCO10_MASK)

#define RGPIO_PCOR_PTCO11_MASK                   (0x800U)
#define RGPIO_PCOR_PTCO11_SHIFT                  (11U)
/*! PTCO11 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO11(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO11_SHIFT)) & RGPIO_PCOR_PTCO11_MASK)

#define RGPIO_PCOR_PTCO12_MASK                   (0x1000U)
#define RGPIO_PCOR_PTCO12_SHIFT                  (12U)
/*! PTCO12 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO12(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO12_SHIFT)) & RGPIO_PCOR_PTCO12_MASK)

#define RGPIO_PCOR_PTCO13_MASK                   (0x2000U)
#define RGPIO_PCOR_PTCO13_SHIFT                  (13U)
/*! PTCO13 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO13(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO13_SHIFT)) & RGPIO_PCOR_PTCO13_MASK)

#define RGPIO_PCOR_PTCO14_MASK                   (0x4000U)
#define RGPIO_PCOR_PTCO14_SHIFT                  (14U)
/*! PTCO14 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO14(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO14_SHIFT)) & RGPIO_PCOR_PTCO14_MASK)

#define RGPIO_PCOR_PTCO15_MASK                   (0x8000U)
#define RGPIO_PCOR_PTCO15_SHIFT                  (15U)
/*! PTCO15 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO15(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO15_SHIFT)) & RGPIO_PCOR_PTCO15_MASK)

#define RGPIO_PCOR_PTCO16_MASK                   (0x10000U)
#define RGPIO_PCOR_PTCO16_SHIFT                  (16U)
/*! PTCO16 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO16(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO16_SHIFT)) & RGPIO_PCOR_PTCO16_MASK)

#define RGPIO_PCOR_PTCO17_MASK                   (0x20000U)
#define RGPIO_PCOR_PTCO17_SHIFT                  (17U)
/*! PTCO17 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO17(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO17_SHIFT)) & RGPIO_PCOR_PTCO17_MASK)

#define RGPIO_PCOR_PTCO18_MASK                   (0x40000U)
#define RGPIO_PCOR_PTCO18_SHIFT                  (18U)
/*! PTCO18 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO18(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO18_SHIFT)) & RGPIO_PCOR_PTCO18_MASK)

#define RGPIO_PCOR_PTCO19_MASK                   (0x80000U)
#define RGPIO_PCOR_PTCO19_SHIFT                  (19U)
/*! PTCO19 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO19(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO19_SHIFT)) & RGPIO_PCOR_PTCO19_MASK)

#define RGPIO_PCOR_PTCO20_MASK                   (0x100000U)
#define RGPIO_PCOR_PTCO20_SHIFT                  (20U)
/*! PTCO20 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO20(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO20_SHIFT)) & RGPIO_PCOR_PTCO20_MASK)

#define RGPIO_PCOR_PTCO21_MASK                   (0x200000U)
#define RGPIO_PCOR_PTCO21_SHIFT                  (21U)
/*! PTCO21 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO21(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO21_SHIFT)) & RGPIO_PCOR_PTCO21_MASK)

#define RGPIO_PCOR_PTCO22_MASK                   (0x400000U)
#define RGPIO_PCOR_PTCO22_SHIFT                  (22U)
/*! PTCO22 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO22(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO22_SHIFT)) & RGPIO_PCOR_PTCO22_MASK)

#define RGPIO_PCOR_PTCO23_MASK                   (0x800000U)
#define RGPIO_PCOR_PTCO23_SHIFT                  (23U)
/*! PTCO23 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO23(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO23_SHIFT)) & RGPIO_PCOR_PTCO23_MASK)

#define RGPIO_PCOR_PTCO24_MASK                   (0x1000000U)
#define RGPIO_PCOR_PTCO24_SHIFT                  (24U)
/*! PTCO24 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO24(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO24_SHIFT)) & RGPIO_PCOR_PTCO24_MASK)

#define RGPIO_PCOR_PTCO25_MASK                   (0x2000000U)
#define RGPIO_PCOR_PTCO25_SHIFT                  (25U)
/*! PTCO25 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO25(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO25_SHIFT)) & RGPIO_PCOR_PTCO25_MASK)

#define RGPIO_PCOR_PTCO26_MASK                   (0x4000000U)
#define RGPIO_PCOR_PTCO26_SHIFT                  (26U)
/*! PTCO26 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO26(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO26_SHIFT)) & RGPIO_PCOR_PTCO26_MASK)

#define RGPIO_PCOR_PTCO27_MASK                   (0x8000000U)
#define RGPIO_PCOR_PTCO27_SHIFT                  (27U)
/*! PTCO27 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO27(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO27_SHIFT)) & RGPIO_PCOR_PTCO27_MASK)

#define RGPIO_PCOR_PTCO28_MASK                   (0x10000000U)
#define RGPIO_PCOR_PTCO28_SHIFT                  (28U)
/*! PTCO28 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO28(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO28_SHIFT)) & RGPIO_PCOR_PTCO28_MASK)

#define RGPIO_PCOR_PTCO29_MASK                   (0x20000000U)
#define RGPIO_PCOR_PTCO29_SHIFT                  (29U)
/*! PTCO29 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO29(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO29_SHIFT)) & RGPIO_PCOR_PTCO29_MASK)

#define RGPIO_PCOR_PTCO30_MASK                   (0x40000000U)
#define RGPIO_PCOR_PTCO30_SHIFT                  (30U)
/*! PTCO30 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO30(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO30_SHIFT)) & RGPIO_PCOR_PTCO30_MASK)

#define RGPIO_PCOR_PTCO31_MASK                   (0x80000000U)
#define RGPIO_PCOR_PTCO31_SHIFT                  (31U)
/*! PTCO31 - Port Clear Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is cleared to logic 0.
 */
#define RGPIO_PCOR_PTCO31(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PCOR_PTCO31_SHIFT)) & RGPIO_PCOR_PTCO31_MASK)
/*! @} */

/*! @name PTOR - Port Toggle Output Register */
/*! @{ */

#define RGPIO_PTOR_PTTO0_MASK                    (0x1U)
#define RGPIO_PTOR_PTTO0_SHIFT                   (0U)
/*! PTTO0 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO0(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO0_SHIFT)) & RGPIO_PTOR_PTTO0_MASK)

#define RGPIO_PTOR_PTTO1_MASK                    (0x2U)
#define RGPIO_PTOR_PTTO1_SHIFT                   (1U)
/*! PTTO1 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO1(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO1_SHIFT)) & RGPIO_PTOR_PTTO1_MASK)

#define RGPIO_PTOR_PTTO2_MASK                    (0x4U)
#define RGPIO_PTOR_PTTO2_SHIFT                   (2U)
/*! PTTO2 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO2(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO2_SHIFT)) & RGPIO_PTOR_PTTO2_MASK)

#define RGPIO_PTOR_PTTO3_MASK                    (0x8U)
#define RGPIO_PTOR_PTTO3_SHIFT                   (3U)
/*! PTTO3 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO3(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO3_SHIFT)) & RGPIO_PTOR_PTTO3_MASK)

#define RGPIO_PTOR_PTTO4_MASK                    (0x10U)
#define RGPIO_PTOR_PTTO4_SHIFT                   (4U)
/*! PTTO4 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO4(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO4_SHIFT)) & RGPIO_PTOR_PTTO4_MASK)

#define RGPIO_PTOR_PTTO5_MASK                    (0x20U)
#define RGPIO_PTOR_PTTO5_SHIFT                   (5U)
/*! PTTO5 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO5(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO5_SHIFT)) & RGPIO_PTOR_PTTO5_MASK)

#define RGPIO_PTOR_PTTO6_MASK                    (0x40U)
#define RGPIO_PTOR_PTTO6_SHIFT                   (6U)
/*! PTTO6 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO6(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO6_SHIFT)) & RGPIO_PTOR_PTTO6_MASK)

#define RGPIO_PTOR_PTTO7_MASK                    (0x80U)
#define RGPIO_PTOR_PTTO7_SHIFT                   (7U)
/*! PTTO7 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO7(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO7_SHIFT)) & RGPIO_PTOR_PTTO7_MASK)

#define RGPIO_PTOR_PTTO8_MASK                    (0x100U)
#define RGPIO_PTOR_PTTO8_SHIFT                   (8U)
/*! PTTO8 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO8(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO8_SHIFT)) & RGPIO_PTOR_PTTO8_MASK)

#define RGPIO_PTOR_PTTO9_MASK                    (0x200U)
#define RGPIO_PTOR_PTTO9_SHIFT                   (9U)
/*! PTTO9 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO9(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO9_SHIFT)) & RGPIO_PTOR_PTTO9_MASK)

#define RGPIO_PTOR_PTTO10_MASK                   (0x400U)
#define RGPIO_PTOR_PTTO10_SHIFT                  (10U)
/*! PTTO10 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO10(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO10_SHIFT)) & RGPIO_PTOR_PTTO10_MASK)

#define RGPIO_PTOR_PTTO11_MASK                   (0x800U)
#define RGPIO_PTOR_PTTO11_SHIFT                  (11U)
/*! PTTO11 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO11(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO11_SHIFT)) & RGPIO_PTOR_PTTO11_MASK)

#define RGPIO_PTOR_PTTO12_MASK                   (0x1000U)
#define RGPIO_PTOR_PTTO12_SHIFT                  (12U)
/*! PTTO12 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO12(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO12_SHIFT)) & RGPIO_PTOR_PTTO12_MASK)

#define RGPIO_PTOR_PTTO13_MASK                   (0x2000U)
#define RGPIO_PTOR_PTTO13_SHIFT                  (13U)
/*! PTTO13 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO13(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO13_SHIFT)) & RGPIO_PTOR_PTTO13_MASK)

#define RGPIO_PTOR_PTTO14_MASK                   (0x4000U)
#define RGPIO_PTOR_PTTO14_SHIFT                  (14U)
/*! PTTO14 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO14(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO14_SHIFT)) & RGPIO_PTOR_PTTO14_MASK)

#define RGPIO_PTOR_PTTO15_MASK                   (0x8000U)
#define RGPIO_PTOR_PTTO15_SHIFT                  (15U)
/*! PTTO15 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO15(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO15_SHIFT)) & RGPIO_PTOR_PTTO15_MASK)

#define RGPIO_PTOR_PTTO16_MASK                   (0x10000U)
#define RGPIO_PTOR_PTTO16_SHIFT                  (16U)
/*! PTTO16 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO16(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO16_SHIFT)) & RGPIO_PTOR_PTTO16_MASK)

#define RGPIO_PTOR_PTTO17_MASK                   (0x20000U)
#define RGPIO_PTOR_PTTO17_SHIFT                  (17U)
/*! PTTO17 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO17(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO17_SHIFT)) & RGPIO_PTOR_PTTO17_MASK)

#define RGPIO_PTOR_PTTO18_MASK                   (0x40000U)
#define RGPIO_PTOR_PTTO18_SHIFT                  (18U)
/*! PTTO18 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO18(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO18_SHIFT)) & RGPIO_PTOR_PTTO18_MASK)

#define RGPIO_PTOR_PTTO19_MASK                   (0x80000U)
#define RGPIO_PTOR_PTTO19_SHIFT                  (19U)
/*! PTTO19 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO19(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO19_SHIFT)) & RGPIO_PTOR_PTTO19_MASK)

#define RGPIO_PTOR_PTTO20_MASK                   (0x100000U)
#define RGPIO_PTOR_PTTO20_SHIFT                  (20U)
/*! PTTO20 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO20(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO20_SHIFT)) & RGPIO_PTOR_PTTO20_MASK)

#define RGPIO_PTOR_PTTO21_MASK                   (0x200000U)
#define RGPIO_PTOR_PTTO21_SHIFT                  (21U)
/*! PTTO21 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO21(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO21_SHIFT)) & RGPIO_PTOR_PTTO21_MASK)

#define RGPIO_PTOR_PTTO22_MASK                   (0x400000U)
#define RGPIO_PTOR_PTTO22_SHIFT                  (22U)
/*! PTTO22 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO22(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO22_SHIFT)) & RGPIO_PTOR_PTTO22_MASK)

#define RGPIO_PTOR_PTTO23_MASK                   (0x800000U)
#define RGPIO_PTOR_PTTO23_SHIFT                  (23U)
/*! PTTO23 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO23(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO23_SHIFT)) & RGPIO_PTOR_PTTO23_MASK)

#define RGPIO_PTOR_PTTO24_MASK                   (0x1000000U)
#define RGPIO_PTOR_PTTO24_SHIFT                  (24U)
/*! PTTO24 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO24(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO24_SHIFT)) & RGPIO_PTOR_PTTO24_MASK)

#define RGPIO_PTOR_PTTO25_MASK                   (0x2000000U)
#define RGPIO_PTOR_PTTO25_SHIFT                  (25U)
/*! PTTO25 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO25(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO25_SHIFT)) & RGPIO_PTOR_PTTO25_MASK)

#define RGPIO_PTOR_PTTO26_MASK                   (0x4000000U)
#define RGPIO_PTOR_PTTO26_SHIFT                  (26U)
/*! PTTO26 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO26(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO26_SHIFT)) & RGPIO_PTOR_PTTO26_MASK)

#define RGPIO_PTOR_PTTO27_MASK                   (0x8000000U)
#define RGPIO_PTOR_PTTO27_SHIFT                  (27U)
/*! PTTO27 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO27(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO27_SHIFT)) & RGPIO_PTOR_PTTO27_MASK)

#define RGPIO_PTOR_PTTO28_MASK                   (0x10000000U)
#define RGPIO_PTOR_PTTO28_SHIFT                  (28U)
/*! PTTO28 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO28(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO28_SHIFT)) & RGPIO_PTOR_PTTO28_MASK)

#define RGPIO_PTOR_PTTO29_MASK                   (0x20000000U)
#define RGPIO_PTOR_PTTO29_SHIFT                  (29U)
/*! PTTO29 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO29(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO29_SHIFT)) & RGPIO_PTOR_PTTO29_MASK)

#define RGPIO_PTOR_PTTO30_MASK                   (0x40000000U)
#define RGPIO_PTOR_PTTO30_SHIFT                  (30U)
/*! PTTO30 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO30(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO30_SHIFT)) & RGPIO_PTOR_PTTO30_MASK)

#define RGPIO_PTOR_PTTO31_MASK                   (0x80000000U)
#define RGPIO_PTOR_PTTO31_SHIFT                  (31U)
/*! PTTO31 - Port Toggle Output
 *  0b0..Corresponding field of PDOR[PDOn] does not change.
 *  0b1..Corresponding field of PDOR[PDOn] is set to the inverse of its current logic state.
 */
#define RGPIO_PTOR_PTTO31(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_PTOR_PTTO31_SHIFT)) & RGPIO_PTOR_PTTO31_MASK)
/*! @} */

/*! @name PDIR - Port Data Input Register */
/*! @{ */

#define RGPIO_PDIR_PDI0_MASK                     (0x1U)
#define RGPIO_PDIR_PDI0_SHIFT                    (0U)
/*! PDI0 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI0_SHIFT)) & RGPIO_PDIR_PDI0_MASK)

#define RGPIO_PDIR_PDI1_MASK                     (0x2U)
#define RGPIO_PDIR_PDI1_SHIFT                    (1U)
/*! PDI1 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI1_SHIFT)) & RGPIO_PDIR_PDI1_MASK)

#define RGPIO_PDIR_PDI2_MASK                     (0x4U)
#define RGPIO_PDIR_PDI2_SHIFT                    (2U)
/*! PDI2 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI2(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI2_SHIFT)) & RGPIO_PDIR_PDI2_MASK)

#define RGPIO_PDIR_PDI3_MASK                     (0x8U)
#define RGPIO_PDIR_PDI3_SHIFT                    (3U)
/*! PDI3 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI3(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI3_SHIFT)) & RGPIO_PDIR_PDI3_MASK)

#define RGPIO_PDIR_PDI4_MASK                     (0x10U)
#define RGPIO_PDIR_PDI4_SHIFT                    (4U)
/*! PDI4 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI4(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI4_SHIFT)) & RGPIO_PDIR_PDI4_MASK)

#define RGPIO_PDIR_PDI5_MASK                     (0x20U)
#define RGPIO_PDIR_PDI5_SHIFT                    (5U)
/*! PDI5 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI5(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI5_SHIFT)) & RGPIO_PDIR_PDI5_MASK)

#define RGPIO_PDIR_PDI6_MASK                     (0x40U)
#define RGPIO_PDIR_PDI6_SHIFT                    (6U)
/*! PDI6 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI6(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI6_SHIFT)) & RGPIO_PDIR_PDI6_MASK)

#define RGPIO_PDIR_PDI7_MASK                     (0x80U)
#define RGPIO_PDIR_PDI7_SHIFT                    (7U)
/*! PDI7 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI7(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI7_SHIFT)) & RGPIO_PDIR_PDI7_MASK)

#define RGPIO_PDIR_PDI8_MASK                     (0x100U)
#define RGPIO_PDIR_PDI8_SHIFT                    (8U)
/*! PDI8 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI8(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI8_SHIFT)) & RGPIO_PDIR_PDI8_MASK)

#define RGPIO_PDIR_PDI9_MASK                     (0x200U)
#define RGPIO_PDIR_PDI9_SHIFT                    (9U)
/*! PDI9 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI9(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI9_SHIFT)) & RGPIO_PDIR_PDI9_MASK)

#define RGPIO_PDIR_PDI10_MASK                    (0x400U)
#define RGPIO_PDIR_PDI10_SHIFT                   (10U)
/*! PDI10 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI10(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI10_SHIFT)) & RGPIO_PDIR_PDI10_MASK)

#define RGPIO_PDIR_PDI11_MASK                    (0x800U)
#define RGPIO_PDIR_PDI11_SHIFT                   (11U)
/*! PDI11 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI11(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI11_SHIFT)) & RGPIO_PDIR_PDI11_MASK)

#define RGPIO_PDIR_PDI12_MASK                    (0x1000U)
#define RGPIO_PDIR_PDI12_SHIFT                   (12U)
/*! PDI12 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI12(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI12_SHIFT)) & RGPIO_PDIR_PDI12_MASK)

#define RGPIO_PDIR_PDI13_MASK                    (0x2000U)
#define RGPIO_PDIR_PDI13_SHIFT                   (13U)
/*! PDI13 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI13(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI13_SHIFT)) & RGPIO_PDIR_PDI13_MASK)

#define RGPIO_PDIR_PDI14_MASK                    (0x4000U)
#define RGPIO_PDIR_PDI14_SHIFT                   (14U)
/*! PDI14 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI14(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI14_SHIFT)) & RGPIO_PDIR_PDI14_MASK)

#define RGPIO_PDIR_PDI15_MASK                    (0x8000U)
#define RGPIO_PDIR_PDI15_SHIFT                   (15U)
/*! PDI15 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI15(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI15_SHIFT)) & RGPIO_PDIR_PDI15_MASK)

#define RGPIO_PDIR_PDI16_MASK                    (0x10000U)
#define RGPIO_PDIR_PDI16_SHIFT                   (16U)
/*! PDI16 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI16(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI16_SHIFT)) & RGPIO_PDIR_PDI16_MASK)

#define RGPIO_PDIR_PDI17_MASK                    (0x20000U)
#define RGPIO_PDIR_PDI17_SHIFT                   (17U)
/*! PDI17 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI17(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI17_SHIFT)) & RGPIO_PDIR_PDI17_MASK)

#define RGPIO_PDIR_PDI18_MASK                    (0x40000U)
#define RGPIO_PDIR_PDI18_SHIFT                   (18U)
/*! PDI18 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI18(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI18_SHIFT)) & RGPIO_PDIR_PDI18_MASK)

#define RGPIO_PDIR_PDI19_MASK                    (0x80000U)
#define RGPIO_PDIR_PDI19_SHIFT                   (19U)
/*! PDI19 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI19(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI19_SHIFT)) & RGPIO_PDIR_PDI19_MASK)

#define RGPIO_PDIR_PDI20_MASK                    (0x100000U)
#define RGPIO_PDIR_PDI20_SHIFT                   (20U)
/*! PDI20 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI20(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI20_SHIFT)) & RGPIO_PDIR_PDI20_MASK)

#define RGPIO_PDIR_PDI21_MASK                    (0x200000U)
#define RGPIO_PDIR_PDI21_SHIFT                   (21U)
/*! PDI21 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI21(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI21_SHIFT)) & RGPIO_PDIR_PDI21_MASK)

#define RGPIO_PDIR_PDI22_MASK                    (0x400000U)
#define RGPIO_PDIR_PDI22_SHIFT                   (22U)
/*! PDI22 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI22(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI22_SHIFT)) & RGPIO_PDIR_PDI22_MASK)

#define RGPIO_PDIR_PDI23_MASK                    (0x800000U)
#define RGPIO_PDIR_PDI23_SHIFT                   (23U)
/*! PDI23 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI23(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI23_SHIFT)) & RGPIO_PDIR_PDI23_MASK)

#define RGPIO_PDIR_PDI24_MASK                    (0x1000000U)
#define RGPIO_PDIR_PDI24_SHIFT                   (24U)
/*! PDI24 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI24(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI24_SHIFT)) & RGPIO_PDIR_PDI24_MASK)

#define RGPIO_PDIR_PDI25_MASK                    (0x2000000U)
#define RGPIO_PDIR_PDI25_SHIFT                   (25U)
/*! PDI25 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI25(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI25_SHIFT)) & RGPIO_PDIR_PDI25_MASK)

#define RGPIO_PDIR_PDI26_MASK                    (0x4000000U)
#define RGPIO_PDIR_PDI26_SHIFT                   (26U)
/*! PDI26 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI26(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI26_SHIFT)) & RGPIO_PDIR_PDI26_MASK)

#define RGPIO_PDIR_PDI27_MASK                    (0x8000000U)
#define RGPIO_PDIR_PDI27_SHIFT                   (27U)
/*! PDI27 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI27(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI27_SHIFT)) & RGPIO_PDIR_PDI27_MASK)

#define RGPIO_PDIR_PDI28_MASK                    (0x10000000U)
#define RGPIO_PDIR_PDI28_SHIFT                   (28U)
/*! PDI28 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI28(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI28_SHIFT)) & RGPIO_PDIR_PDI28_MASK)

#define RGPIO_PDIR_PDI29_MASK                    (0x20000000U)
#define RGPIO_PDIR_PDI29_SHIFT                   (29U)
/*! PDI29 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI29(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI29_SHIFT)) & RGPIO_PDIR_PDI29_MASK)

#define RGPIO_PDIR_PDI30_MASK                    (0x40000000U)
#define RGPIO_PDIR_PDI30_SHIFT                   (30U)
/*! PDI30 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI30(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI30_SHIFT)) & RGPIO_PDIR_PDI30_MASK)

#define RGPIO_PDIR_PDI31_MASK                    (0x80000000U)
#define RGPIO_PDIR_PDI31_SHIFT                   (31U)
/*! PDI31 - Port Data Input
 *  0b0..Pin logic level is logic 0 or is not configured for use by digital function.
 *  0b1..Pin logic level is logic 1.
 */
#define RGPIO_PDIR_PDI31(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDIR_PDI31_SHIFT)) & RGPIO_PDIR_PDI31_MASK)
/*! @} */

/*! @name PDDR - Port Data Direction Register */
/*! @{ */

#define RGPIO_PDDR_PDD0_MASK                     (0x1U)
#define RGPIO_PDDR_PDD0_SHIFT                    (0U)
/*! PDD0 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD0_SHIFT)) & RGPIO_PDDR_PDD0_MASK)

#define RGPIO_PDDR_PDD1_MASK                     (0x2U)
#define RGPIO_PDDR_PDD1_SHIFT                    (1U)
/*! PDD1 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD1_SHIFT)) & RGPIO_PDDR_PDD1_MASK)

#define RGPIO_PDDR_PDD2_MASK                     (0x4U)
#define RGPIO_PDDR_PDD2_SHIFT                    (2U)
/*! PDD2 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD2(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD2_SHIFT)) & RGPIO_PDDR_PDD2_MASK)

#define RGPIO_PDDR_PDD3_MASK                     (0x8U)
#define RGPIO_PDDR_PDD3_SHIFT                    (3U)
/*! PDD3 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD3(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD3_SHIFT)) & RGPIO_PDDR_PDD3_MASK)

#define RGPIO_PDDR_PDD4_MASK                     (0x10U)
#define RGPIO_PDDR_PDD4_SHIFT                    (4U)
/*! PDD4 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD4(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD4_SHIFT)) & RGPIO_PDDR_PDD4_MASK)

#define RGPIO_PDDR_PDD5_MASK                     (0x20U)
#define RGPIO_PDDR_PDD5_SHIFT                    (5U)
/*! PDD5 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD5(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD5_SHIFT)) & RGPIO_PDDR_PDD5_MASK)

#define RGPIO_PDDR_PDD6_MASK                     (0x40U)
#define RGPIO_PDDR_PDD6_SHIFT                    (6U)
/*! PDD6 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD6(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD6_SHIFT)) & RGPIO_PDDR_PDD6_MASK)

#define RGPIO_PDDR_PDD7_MASK                     (0x80U)
#define RGPIO_PDDR_PDD7_SHIFT                    (7U)
/*! PDD7 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD7(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD7_SHIFT)) & RGPIO_PDDR_PDD7_MASK)

#define RGPIO_PDDR_PDD8_MASK                     (0x100U)
#define RGPIO_PDDR_PDD8_SHIFT                    (8U)
/*! PDD8 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD8(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD8_SHIFT)) & RGPIO_PDDR_PDD8_MASK)

#define RGPIO_PDDR_PDD9_MASK                     (0x200U)
#define RGPIO_PDDR_PDD9_SHIFT                    (9U)
/*! PDD9 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD9(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD9_SHIFT)) & RGPIO_PDDR_PDD9_MASK)

#define RGPIO_PDDR_PDD10_MASK                    (0x400U)
#define RGPIO_PDDR_PDD10_SHIFT                   (10U)
/*! PDD10 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD10(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD10_SHIFT)) & RGPIO_PDDR_PDD10_MASK)

#define RGPIO_PDDR_PDD11_MASK                    (0x800U)
#define RGPIO_PDDR_PDD11_SHIFT                   (11U)
/*! PDD11 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD11(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD11_SHIFT)) & RGPIO_PDDR_PDD11_MASK)

#define RGPIO_PDDR_PDD12_MASK                    (0x1000U)
#define RGPIO_PDDR_PDD12_SHIFT                   (12U)
/*! PDD12 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD12(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD12_SHIFT)) & RGPIO_PDDR_PDD12_MASK)

#define RGPIO_PDDR_PDD13_MASK                    (0x2000U)
#define RGPIO_PDDR_PDD13_SHIFT                   (13U)
/*! PDD13 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD13(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD13_SHIFT)) & RGPIO_PDDR_PDD13_MASK)

#define RGPIO_PDDR_PDD14_MASK                    (0x4000U)
#define RGPIO_PDDR_PDD14_SHIFT                   (14U)
/*! PDD14 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD14(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD14_SHIFT)) & RGPIO_PDDR_PDD14_MASK)

#define RGPIO_PDDR_PDD15_MASK                    (0x8000U)
#define RGPIO_PDDR_PDD15_SHIFT                   (15U)
/*! PDD15 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD15(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD15_SHIFT)) & RGPIO_PDDR_PDD15_MASK)

#define RGPIO_PDDR_PDD16_MASK                    (0x10000U)
#define RGPIO_PDDR_PDD16_SHIFT                   (16U)
/*! PDD16 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD16(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD16_SHIFT)) & RGPIO_PDDR_PDD16_MASK)

#define RGPIO_PDDR_PDD17_MASK                    (0x20000U)
#define RGPIO_PDDR_PDD17_SHIFT                   (17U)
/*! PDD17 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD17(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD17_SHIFT)) & RGPIO_PDDR_PDD17_MASK)

#define RGPIO_PDDR_PDD18_MASK                    (0x40000U)
#define RGPIO_PDDR_PDD18_SHIFT                   (18U)
/*! PDD18 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD18(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD18_SHIFT)) & RGPIO_PDDR_PDD18_MASK)

#define RGPIO_PDDR_PDD19_MASK                    (0x80000U)
#define RGPIO_PDDR_PDD19_SHIFT                   (19U)
/*! PDD19 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD19(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD19_SHIFT)) & RGPIO_PDDR_PDD19_MASK)

#define RGPIO_PDDR_PDD20_MASK                    (0x100000U)
#define RGPIO_PDDR_PDD20_SHIFT                   (20U)
/*! PDD20 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD20(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD20_SHIFT)) & RGPIO_PDDR_PDD20_MASK)

#define RGPIO_PDDR_PDD21_MASK                    (0x200000U)
#define RGPIO_PDDR_PDD21_SHIFT                   (21U)
/*! PDD21 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD21(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD21_SHIFT)) & RGPIO_PDDR_PDD21_MASK)

#define RGPIO_PDDR_PDD22_MASK                    (0x400000U)
#define RGPIO_PDDR_PDD22_SHIFT                   (22U)
/*! PDD22 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD22(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD22_SHIFT)) & RGPIO_PDDR_PDD22_MASK)

#define RGPIO_PDDR_PDD23_MASK                    (0x800000U)
#define RGPIO_PDDR_PDD23_SHIFT                   (23U)
/*! PDD23 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD23(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD23_SHIFT)) & RGPIO_PDDR_PDD23_MASK)

#define RGPIO_PDDR_PDD24_MASK                    (0x1000000U)
#define RGPIO_PDDR_PDD24_SHIFT                   (24U)
/*! PDD24 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD24(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD24_SHIFT)) & RGPIO_PDDR_PDD24_MASK)

#define RGPIO_PDDR_PDD25_MASK                    (0x2000000U)
#define RGPIO_PDDR_PDD25_SHIFT                   (25U)
/*! PDD25 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD25(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD25_SHIFT)) & RGPIO_PDDR_PDD25_MASK)

#define RGPIO_PDDR_PDD26_MASK                    (0x4000000U)
#define RGPIO_PDDR_PDD26_SHIFT                   (26U)
/*! PDD26 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD26(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD26_SHIFT)) & RGPIO_PDDR_PDD26_MASK)

#define RGPIO_PDDR_PDD27_MASK                    (0x8000000U)
#define RGPIO_PDDR_PDD27_SHIFT                   (27U)
/*! PDD27 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD27(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD27_SHIFT)) & RGPIO_PDDR_PDD27_MASK)

#define RGPIO_PDDR_PDD28_MASK                    (0x10000000U)
#define RGPIO_PDDR_PDD28_SHIFT                   (28U)
/*! PDD28 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD28(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD28_SHIFT)) & RGPIO_PDDR_PDD28_MASK)

#define RGPIO_PDDR_PDD29_MASK                    (0x20000000U)
#define RGPIO_PDDR_PDD29_SHIFT                   (29U)
/*! PDD29 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD29(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD29_SHIFT)) & RGPIO_PDDR_PDD29_MASK)

#define RGPIO_PDDR_PDD30_MASK                    (0x40000000U)
#define RGPIO_PDDR_PDD30_SHIFT                   (30U)
/*! PDD30 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD30(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD30_SHIFT)) & RGPIO_PDDR_PDD30_MASK)

#define RGPIO_PDDR_PDD31_MASK                    (0x80000000U)
#define RGPIO_PDDR_PDD31_SHIFT                   (31U)
/*! PDD31 - Port Data Direction
 *  0b0..Pin is configured as general-purpose input for the GPIO function.
 *  0b1..Pin is configured as general-purpose output for the GPIO function.
 */
#define RGPIO_PDDR_PDD31(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PDDR_PDD31_SHIFT)) & RGPIO_PDDR_PDD31_MASK)
/*! @} */

/*! @name PIDR - Port Input Disable Register */
/*! @{ */

#define RGPIO_PIDR_PID0_MASK                     (0x1U)
#define RGPIO_PIDR_PID0_SHIFT                    (0U)
/*! PID0 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID0_SHIFT)) & RGPIO_PIDR_PID0_MASK)

#define RGPIO_PIDR_PID1_MASK                     (0x2U)
#define RGPIO_PIDR_PID1_SHIFT                    (1U)
/*! PID1 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID1_SHIFT)) & RGPIO_PIDR_PID1_MASK)

#define RGPIO_PIDR_PID2_MASK                     (0x4U)
#define RGPIO_PIDR_PID2_SHIFT                    (2U)
/*! PID2 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID2(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID2_SHIFT)) & RGPIO_PIDR_PID2_MASK)

#define RGPIO_PIDR_PID3_MASK                     (0x8U)
#define RGPIO_PIDR_PID3_SHIFT                    (3U)
/*! PID3 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID3(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID3_SHIFT)) & RGPIO_PIDR_PID3_MASK)

#define RGPIO_PIDR_PID4_MASK                     (0x10U)
#define RGPIO_PIDR_PID4_SHIFT                    (4U)
/*! PID4 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID4(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID4_SHIFT)) & RGPIO_PIDR_PID4_MASK)

#define RGPIO_PIDR_PID5_MASK                     (0x20U)
#define RGPIO_PIDR_PID5_SHIFT                    (5U)
/*! PID5 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID5(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID5_SHIFT)) & RGPIO_PIDR_PID5_MASK)

#define RGPIO_PIDR_PID6_MASK                     (0x40U)
#define RGPIO_PIDR_PID6_SHIFT                    (6U)
/*! PID6 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID6(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID6_SHIFT)) & RGPIO_PIDR_PID6_MASK)

#define RGPIO_PIDR_PID7_MASK                     (0x80U)
#define RGPIO_PIDR_PID7_SHIFT                    (7U)
/*! PID7 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID7(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID7_SHIFT)) & RGPIO_PIDR_PID7_MASK)

#define RGPIO_PIDR_PID8_MASK                     (0x100U)
#define RGPIO_PIDR_PID8_SHIFT                    (8U)
/*! PID8 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID8(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID8_SHIFT)) & RGPIO_PIDR_PID8_MASK)

#define RGPIO_PIDR_PID9_MASK                     (0x200U)
#define RGPIO_PIDR_PID9_SHIFT                    (9U)
/*! PID9 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID9(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID9_SHIFT)) & RGPIO_PIDR_PID9_MASK)

#define RGPIO_PIDR_PID10_MASK                    (0x400U)
#define RGPIO_PIDR_PID10_SHIFT                   (10U)
/*! PID10 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID10(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID10_SHIFT)) & RGPIO_PIDR_PID10_MASK)

#define RGPIO_PIDR_PID11_MASK                    (0x800U)
#define RGPIO_PIDR_PID11_SHIFT                   (11U)
/*! PID11 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID11(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID11_SHIFT)) & RGPIO_PIDR_PID11_MASK)

#define RGPIO_PIDR_PID12_MASK                    (0x1000U)
#define RGPIO_PIDR_PID12_SHIFT                   (12U)
/*! PID12 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID12(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID12_SHIFT)) & RGPIO_PIDR_PID12_MASK)

#define RGPIO_PIDR_PID13_MASK                    (0x2000U)
#define RGPIO_PIDR_PID13_SHIFT                   (13U)
/*! PID13 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID13(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID13_SHIFT)) & RGPIO_PIDR_PID13_MASK)

#define RGPIO_PIDR_PID14_MASK                    (0x4000U)
#define RGPIO_PIDR_PID14_SHIFT                   (14U)
/*! PID14 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID14(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID14_SHIFT)) & RGPIO_PIDR_PID14_MASK)

#define RGPIO_PIDR_PID15_MASK                    (0x8000U)
#define RGPIO_PIDR_PID15_SHIFT                   (15U)
/*! PID15 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID15(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID15_SHIFT)) & RGPIO_PIDR_PID15_MASK)

#define RGPIO_PIDR_PID16_MASK                    (0x10000U)
#define RGPIO_PIDR_PID16_SHIFT                   (16U)
/*! PID16 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID16(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID16_SHIFT)) & RGPIO_PIDR_PID16_MASK)

#define RGPIO_PIDR_PID17_MASK                    (0x20000U)
#define RGPIO_PIDR_PID17_SHIFT                   (17U)
/*! PID17 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID17(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID17_SHIFT)) & RGPIO_PIDR_PID17_MASK)

#define RGPIO_PIDR_PID18_MASK                    (0x40000U)
#define RGPIO_PIDR_PID18_SHIFT                   (18U)
/*! PID18 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID18(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID18_SHIFT)) & RGPIO_PIDR_PID18_MASK)

#define RGPIO_PIDR_PID19_MASK                    (0x80000U)
#define RGPIO_PIDR_PID19_SHIFT                   (19U)
/*! PID19 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID19(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID19_SHIFT)) & RGPIO_PIDR_PID19_MASK)

#define RGPIO_PIDR_PID20_MASK                    (0x100000U)
#define RGPIO_PIDR_PID20_SHIFT                   (20U)
/*! PID20 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID20(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID20_SHIFT)) & RGPIO_PIDR_PID20_MASK)

#define RGPIO_PIDR_PID21_MASK                    (0x200000U)
#define RGPIO_PIDR_PID21_SHIFT                   (21U)
/*! PID21 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID21(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID21_SHIFT)) & RGPIO_PIDR_PID21_MASK)

#define RGPIO_PIDR_PID22_MASK                    (0x400000U)
#define RGPIO_PIDR_PID22_SHIFT                   (22U)
/*! PID22 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID22(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID22_SHIFT)) & RGPIO_PIDR_PID22_MASK)

#define RGPIO_PIDR_PID23_MASK                    (0x800000U)
#define RGPIO_PIDR_PID23_SHIFT                   (23U)
/*! PID23 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID23(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID23_SHIFT)) & RGPIO_PIDR_PID23_MASK)

#define RGPIO_PIDR_PID24_MASK                    (0x1000000U)
#define RGPIO_PIDR_PID24_SHIFT                   (24U)
/*! PID24 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID24(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID24_SHIFT)) & RGPIO_PIDR_PID24_MASK)

#define RGPIO_PIDR_PID25_MASK                    (0x2000000U)
#define RGPIO_PIDR_PID25_SHIFT                   (25U)
/*! PID25 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID25(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID25_SHIFT)) & RGPIO_PIDR_PID25_MASK)

#define RGPIO_PIDR_PID26_MASK                    (0x4000000U)
#define RGPIO_PIDR_PID26_SHIFT                   (26U)
/*! PID26 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID26(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID26_SHIFT)) & RGPIO_PIDR_PID26_MASK)

#define RGPIO_PIDR_PID27_MASK                    (0x8000000U)
#define RGPIO_PIDR_PID27_SHIFT                   (27U)
/*! PID27 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID27(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID27_SHIFT)) & RGPIO_PIDR_PID27_MASK)

#define RGPIO_PIDR_PID28_MASK                    (0x10000000U)
#define RGPIO_PIDR_PID28_SHIFT                   (28U)
/*! PID28 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID28(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID28_SHIFT)) & RGPIO_PIDR_PID28_MASK)

#define RGPIO_PIDR_PID29_MASK                    (0x20000000U)
#define RGPIO_PIDR_PID29_SHIFT                   (29U)
/*! PID29 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID29(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID29_SHIFT)) & RGPIO_PIDR_PID29_MASK)

#define RGPIO_PIDR_PID30_MASK                    (0x40000000U)
#define RGPIO_PIDR_PID30_SHIFT                   (30U)
/*! PID30 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID30(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID30_SHIFT)) & RGPIO_PIDR_PID30_MASK)

#define RGPIO_PIDR_PID31_MASK                    (0x80000000U)
#define RGPIO_PIDR_PID31_SHIFT                   (31U)
/*! PID31 - Port Input Disable
 *  0b0..Pin is configured for general-purpose input provided, the pin is configured for any digital function.
 *  0b1..Pin is disabled for general-purpose input.
 */
#define RGPIO_PIDR_PID31(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_PIDR_PID31_SHIFT)) & RGPIO_PIDR_PID31_MASK)
/*! @} */

/*! @name PDR - Pin Data Register a */
/*! @{ */

#define RGPIO_PDR_PD_MASK                        (0x1U)
#define RGPIO_PDR_PD_SHIFT                       (0U)
/*! PD - Pin Data (input and output)
 *  0b0..Pin logic level is logic zero or not configured for use by digital function.
 *  0b1..Pin logic level is logic one.
 */
#define RGPIO_PDR_PD(x)                          (((uint8_t)(((uint8_t)(x)) << RGPIO_PDR_PD_SHIFT)) & RGPIO_PDR_PD_MASK)
/*! @} */

/* The count of RGPIO_PDR */
#define RGPIO_PDR_COUNT                          (32U)

/*! @name ICR - Interrupt Control Register 0..Interrupt Control Register 31 */
/*! @{ */

#define RGPIO_ICR_IRQC_MASK                      (0xF0000U)
#define RGPIO_ICR_IRQC_SHIFT                     (16U)
/*! IRQC - Interrupt Configuration
 *  0b0000..Interrupt Status Flag (ISF) is disabled.
 *  0b0001..ISF flag and DMA request on rising edge.
 *  0b0010..ISF flag and DMA request on falling edge.
 *  0b0011..ISF flag and DMA request on either edge.
 *  0b0100..Reserved.
 *  0b0101..ISF flag sets on rising edge.
 *  0b0110..ISF flag sets on falling edge.
 *  0b0111..ISF flag sets on either edge.
 *  0b1000..ISF flag and Interrupt when logic 0.
 *  0b1001..ISF flag and Interrupt on rising-edge.
 *  0b1010..ISF flag and Interrupt on falling-edge.
 *  0b1011..ISF flag and Interrupt on either edge.
 *  0b1100..ISF flag and Interrupt when logic 1.
 *  0b1101..Reserved
 *  0b1110..Reserved
 *  0b1111..Reserved.
 */
#define RGPIO_ICR_IRQC(x)                        (((uint32_t)(((uint32_t)(x)) << RGPIO_ICR_IRQC_SHIFT)) & RGPIO_ICR_IRQC_MASK)

#define RGPIO_ICR_IRQS_MASK                      (0x100000U)
#define RGPIO_ICR_IRQS_SHIFT                     (20U)
/*! IRQS - Interrupt Select
 *  0b0..Interrupt/DMA request 0.
 *  0b1..Interrupt/DMA request 1.
 */
#define RGPIO_ICR_IRQS(x)                        (((uint32_t)(((uint32_t)(x)) << RGPIO_ICR_IRQS_SHIFT)) & RGPIO_ICR_IRQS_MASK)

#define RGPIO_ICR_LK_MASK                        (0x800000U)
#define RGPIO_ICR_LK_SHIFT                       (23U)
/*! LK - Lock Register
 *  0b0..Interrupt configuration by ICR[23:0] is not locked and can be updated.
 *  0b1..Interrupt configuration by ICR[23:0] is locked and cannot be updated until next system reset.
 */
#define RGPIO_ICR_LK(x)                          (((uint32_t)(((uint32_t)(x)) << RGPIO_ICR_LK_SHIFT)) & RGPIO_ICR_LK_MASK)

#define RGPIO_ICR_ISF_MASK                       (0x1000000U)
#define RGPIO_ICR_ISF_SHIFT                      (24U)
/*! ISF - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected.
 *  0b1..Configured interrupt is detected. If the pin is configured to generate a DMA request, then the
 *       corresponding flag will be cleared automatically at the completion of the requested DMA transfer. Otherwise, the
 *       flag remains set until a logic 1 is written to the flag. If the pin is configured for a level sensitive
 *       interrupt and the pin remains asserted, then the flag is set again immediately after it is cleared.
 */
#define RGPIO_ICR_ISF(x)                         (((uint32_t)(((uint32_t)(x)) << RGPIO_ICR_ISF_SHIFT)) & RGPIO_ICR_ISF_MASK)
/*! @} */

/* The count of RGPIO_ICR */
#define RGPIO_ICR_COUNT                          (32U)

/*! @name GICLR - Global Interrupt Control Low Register */
/*! @{ */

#define RGPIO_GICLR_GIWE0_MASK                   (0x1U)
#define RGPIO_GICLR_GIWE0_SHIFT                  (0U)
/*! GIWE0 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE0(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE0_SHIFT)) & RGPIO_GICLR_GIWE0_MASK)

#define RGPIO_GICLR_GIWE1_MASK                   (0x2U)
#define RGPIO_GICLR_GIWE1_SHIFT                  (1U)
/*! GIWE1 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE1(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE1_SHIFT)) & RGPIO_GICLR_GIWE1_MASK)

#define RGPIO_GICLR_GIWE2_MASK                   (0x4U)
#define RGPIO_GICLR_GIWE2_SHIFT                  (2U)
/*! GIWE2 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE2(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE2_SHIFT)) & RGPIO_GICLR_GIWE2_MASK)

#define RGPIO_GICLR_GIWE3_MASK                   (0x8U)
#define RGPIO_GICLR_GIWE3_SHIFT                  (3U)
/*! GIWE3 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE3(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE3_SHIFT)) & RGPIO_GICLR_GIWE3_MASK)

#define RGPIO_GICLR_GIWE4_MASK                   (0x10U)
#define RGPIO_GICLR_GIWE4_SHIFT                  (4U)
/*! GIWE4 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE4(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE4_SHIFT)) & RGPIO_GICLR_GIWE4_MASK)

#define RGPIO_GICLR_GIWE5_MASK                   (0x20U)
#define RGPIO_GICLR_GIWE5_SHIFT                  (5U)
/*! GIWE5 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE5(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE5_SHIFT)) & RGPIO_GICLR_GIWE5_MASK)

#define RGPIO_GICLR_GIWE6_MASK                   (0x40U)
#define RGPIO_GICLR_GIWE6_SHIFT                  (6U)
/*! GIWE6 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE6(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE6_SHIFT)) & RGPIO_GICLR_GIWE6_MASK)

#define RGPIO_GICLR_GIWE7_MASK                   (0x80U)
#define RGPIO_GICLR_GIWE7_SHIFT                  (7U)
/*! GIWE7 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE7(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE7_SHIFT)) & RGPIO_GICLR_GIWE7_MASK)

#define RGPIO_GICLR_GIWE8_MASK                   (0x100U)
#define RGPIO_GICLR_GIWE8_SHIFT                  (8U)
/*! GIWE8 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE8(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE8_SHIFT)) & RGPIO_GICLR_GIWE8_MASK)

#define RGPIO_GICLR_GIWE9_MASK                   (0x200U)
#define RGPIO_GICLR_GIWE9_SHIFT                  (9U)
/*! GIWE9 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE9(x)                     (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE9_SHIFT)) & RGPIO_GICLR_GIWE9_MASK)

#define RGPIO_GICLR_GIWE10_MASK                  (0x400U)
#define RGPIO_GICLR_GIWE10_SHIFT                 (10U)
/*! GIWE10 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE10(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE10_SHIFT)) & RGPIO_GICLR_GIWE10_MASK)

#define RGPIO_GICLR_GIWE11_MASK                  (0x800U)
#define RGPIO_GICLR_GIWE11_SHIFT                 (11U)
/*! GIWE11 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE11(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE11_SHIFT)) & RGPIO_GICLR_GIWE11_MASK)

#define RGPIO_GICLR_GIWE12_MASK                  (0x1000U)
#define RGPIO_GICLR_GIWE12_SHIFT                 (12U)
/*! GIWE12 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE12(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE12_SHIFT)) & RGPIO_GICLR_GIWE12_MASK)

#define RGPIO_GICLR_GIWE13_MASK                  (0x2000U)
#define RGPIO_GICLR_GIWE13_SHIFT                 (13U)
/*! GIWE13 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE13(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE13_SHIFT)) & RGPIO_GICLR_GIWE13_MASK)

#define RGPIO_GICLR_GIWE14_MASK                  (0x4000U)
#define RGPIO_GICLR_GIWE14_SHIFT                 (14U)
/*! GIWE14 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE14(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE14_SHIFT)) & RGPIO_GICLR_GIWE14_MASK)

#define RGPIO_GICLR_GIWE15_MASK                  (0x8000U)
#define RGPIO_GICLR_GIWE15_SHIFT                 (15U)
/*! GIWE15 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICLR_GIWE15(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWE15_SHIFT)) & RGPIO_GICLR_GIWE15_MASK)

#define RGPIO_GICLR_GIWD_MASK                    (0xFFFF0000U)
#define RGPIO_GICLR_GIWD_SHIFT                   (16U)
/*! GIWD - Global Interrupt Write Data */
#define RGPIO_GICLR_GIWD(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_GICLR_GIWD_SHIFT)) & RGPIO_GICLR_GIWD_MASK)
/*! @} */

/*! @name GICHR - Global Interrupt Control High Register */
/*! @{ */

#define RGPIO_GICHR_GIWE16_MASK                  (0x1U)
#define RGPIO_GICHR_GIWE16_SHIFT                 (0U)
/*! GIWE16 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE16(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE16_SHIFT)) & RGPIO_GICHR_GIWE16_MASK)

#define RGPIO_GICHR_GIWE17_MASK                  (0x2U)
#define RGPIO_GICHR_GIWE17_SHIFT                 (1U)
/*! GIWE17 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE17(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE17_SHIFT)) & RGPIO_GICHR_GIWE17_MASK)

#define RGPIO_GICHR_GIWE18_MASK                  (0x4U)
#define RGPIO_GICHR_GIWE18_SHIFT                 (2U)
/*! GIWE18 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE18(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE18_SHIFT)) & RGPIO_GICHR_GIWE18_MASK)

#define RGPIO_GICHR_GIWE19_MASK                  (0x8U)
#define RGPIO_GICHR_GIWE19_SHIFT                 (3U)
/*! GIWE19 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE19(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE19_SHIFT)) & RGPIO_GICHR_GIWE19_MASK)

#define RGPIO_GICHR_GIWE20_MASK                  (0x10U)
#define RGPIO_GICHR_GIWE20_SHIFT                 (4U)
/*! GIWE20 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE20(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE20_SHIFT)) & RGPIO_GICHR_GIWE20_MASK)

#define RGPIO_GICHR_GIWE21_MASK                  (0x20U)
#define RGPIO_GICHR_GIWE21_SHIFT                 (5U)
/*! GIWE21 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE21(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE21_SHIFT)) & RGPIO_GICHR_GIWE21_MASK)

#define RGPIO_GICHR_GIWE22_MASK                  (0x40U)
#define RGPIO_GICHR_GIWE22_SHIFT                 (6U)
/*! GIWE22 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE22(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE22_SHIFT)) & RGPIO_GICHR_GIWE22_MASK)

#define RGPIO_GICHR_GIWE23_MASK                  (0x80U)
#define RGPIO_GICHR_GIWE23_SHIFT                 (7U)
/*! GIWE23 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE23(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE23_SHIFT)) & RGPIO_GICHR_GIWE23_MASK)

#define RGPIO_GICHR_GIWE24_MASK                  (0x100U)
#define RGPIO_GICHR_GIWE24_SHIFT                 (8U)
/*! GIWE24 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE24(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE24_SHIFT)) & RGPIO_GICHR_GIWE24_MASK)

#define RGPIO_GICHR_GIWE25_MASK                  (0x200U)
#define RGPIO_GICHR_GIWE25_SHIFT                 (9U)
/*! GIWE25 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE25(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE25_SHIFT)) & RGPIO_GICHR_GIWE25_MASK)

#define RGPIO_GICHR_GIWE26_MASK                  (0x400U)
#define RGPIO_GICHR_GIWE26_SHIFT                 (10U)
/*! GIWE26 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE26(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE26_SHIFT)) & RGPIO_GICHR_GIWE26_MASK)

#define RGPIO_GICHR_GIWE27_MASK                  (0x800U)
#define RGPIO_GICHR_GIWE27_SHIFT                 (11U)
/*! GIWE27 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE27(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE27_SHIFT)) & RGPIO_GICHR_GIWE27_MASK)

#define RGPIO_GICHR_GIWE28_MASK                  (0x1000U)
#define RGPIO_GICHR_GIWE28_SHIFT                 (12U)
/*! GIWE28 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE28(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE28_SHIFT)) & RGPIO_GICHR_GIWE28_MASK)

#define RGPIO_GICHR_GIWE29_MASK                  (0x2000U)
#define RGPIO_GICHR_GIWE29_SHIFT                 (13U)
/*! GIWE29 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE29(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE29_SHIFT)) & RGPIO_GICHR_GIWE29_MASK)

#define RGPIO_GICHR_GIWE30_MASK                  (0x4000U)
#define RGPIO_GICHR_GIWE30_SHIFT                 (14U)
/*! GIWE30 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE30(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE30_SHIFT)) & RGPIO_GICHR_GIWE30_MASK)

#define RGPIO_GICHR_GIWE31_MASK                  (0x8000U)
#define RGPIO_GICHR_GIWE31_SHIFT                 (15U)
/*! GIWE31 - Global Interrupt Write Enable
 *  0b0..Upper 16-bit of corresponding Interrupt Control Register is not updated with the value in GIWD.
 *  0b1..Upper 16-bit of corresponding Interrupt Control Register is updated with the value in GIWD.
 */
#define RGPIO_GICHR_GIWE31(x)                    (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWE31_SHIFT)) & RGPIO_GICHR_GIWE31_MASK)

#define RGPIO_GICHR_GIWD_MASK                    (0xFFFF0000U)
#define RGPIO_GICHR_GIWD_SHIFT                   (16U)
/*! GIWD - Global Interrupt Write Data */
#define RGPIO_GICHR_GIWD(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_GICHR_GIWD_SHIFT)) & RGPIO_GICHR_GIWD_MASK)
/*! @} */

/*! @name ISFR - Interrupt Status Flag Register */
/*! @{ */

#define RGPIO_ISFR_ISF0_MASK                     (0x1U)
#define RGPIO_ISFR_ISF0_SHIFT                    (0U)
/*! ISF0 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF0(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF0_SHIFT)) & RGPIO_ISFR_ISF0_MASK)

#define RGPIO_ISFR_ISF1_MASK                     (0x2U)
#define RGPIO_ISFR_ISF1_SHIFT                    (1U)
/*! ISF1 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF1(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF1_SHIFT)) & RGPIO_ISFR_ISF1_MASK)

#define RGPIO_ISFR_ISF2_MASK                     (0x4U)
#define RGPIO_ISFR_ISF2_SHIFT                    (2U)
/*! ISF2 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF2(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF2_SHIFT)) & RGPIO_ISFR_ISF2_MASK)

#define RGPIO_ISFR_ISF3_MASK                     (0x8U)
#define RGPIO_ISFR_ISF3_SHIFT                    (3U)
/*! ISF3 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF3(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF3_SHIFT)) & RGPIO_ISFR_ISF3_MASK)

#define RGPIO_ISFR_ISF4_MASK                     (0x10U)
#define RGPIO_ISFR_ISF4_SHIFT                    (4U)
/*! ISF4 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF4(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF4_SHIFT)) & RGPIO_ISFR_ISF4_MASK)

#define RGPIO_ISFR_ISF5_MASK                     (0x20U)
#define RGPIO_ISFR_ISF5_SHIFT                    (5U)
/*! ISF5 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF5(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF5_SHIFT)) & RGPIO_ISFR_ISF5_MASK)

#define RGPIO_ISFR_ISF6_MASK                     (0x40U)
#define RGPIO_ISFR_ISF6_SHIFT                    (6U)
/*! ISF6 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF6(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF6_SHIFT)) & RGPIO_ISFR_ISF6_MASK)

#define RGPIO_ISFR_ISF7_MASK                     (0x80U)
#define RGPIO_ISFR_ISF7_SHIFT                    (7U)
/*! ISF7 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF7(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF7_SHIFT)) & RGPIO_ISFR_ISF7_MASK)

#define RGPIO_ISFR_ISF8_MASK                     (0x100U)
#define RGPIO_ISFR_ISF8_SHIFT                    (8U)
/*! ISF8 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF8(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF8_SHIFT)) & RGPIO_ISFR_ISF8_MASK)

#define RGPIO_ISFR_ISF9_MASK                     (0x200U)
#define RGPIO_ISFR_ISF9_SHIFT                    (9U)
/*! ISF9 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF9(x)                       (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF9_SHIFT)) & RGPIO_ISFR_ISF9_MASK)

#define RGPIO_ISFR_ISF10_MASK                    (0x400U)
#define RGPIO_ISFR_ISF10_SHIFT                   (10U)
/*! ISF10 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF10(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF10_SHIFT)) & RGPIO_ISFR_ISF10_MASK)

#define RGPIO_ISFR_ISF11_MASK                    (0x800U)
#define RGPIO_ISFR_ISF11_SHIFT                   (11U)
/*! ISF11 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF11(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF11_SHIFT)) & RGPIO_ISFR_ISF11_MASK)

#define RGPIO_ISFR_ISF12_MASK                    (0x1000U)
#define RGPIO_ISFR_ISF12_SHIFT                   (12U)
/*! ISF12 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF12(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF12_SHIFT)) & RGPIO_ISFR_ISF12_MASK)

#define RGPIO_ISFR_ISF13_MASK                    (0x2000U)
#define RGPIO_ISFR_ISF13_SHIFT                   (13U)
/*! ISF13 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF13(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF13_SHIFT)) & RGPIO_ISFR_ISF13_MASK)

#define RGPIO_ISFR_ISF14_MASK                    (0x4000U)
#define RGPIO_ISFR_ISF14_SHIFT                   (14U)
/*! ISF14 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF14(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF14_SHIFT)) & RGPIO_ISFR_ISF14_MASK)

#define RGPIO_ISFR_ISF15_MASK                    (0x8000U)
#define RGPIO_ISFR_ISF15_SHIFT                   (15U)
/*! ISF15 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF15(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF15_SHIFT)) & RGPIO_ISFR_ISF15_MASK)

#define RGPIO_ISFR_ISF16_MASK                    (0x10000U)
#define RGPIO_ISFR_ISF16_SHIFT                   (16U)
/*! ISF16 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF16(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF16_SHIFT)) & RGPIO_ISFR_ISF16_MASK)

#define RGPIO_ISFR_ISF17_MASK                    (0x20000U)
#define RGPIO_ISFR_ISF17_SHIFT                   (17U)
/*! ISF17 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF17(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF17_SHIFT)) & RGPIO_ISFR_ISF17_MASK)

#define RGPIO_ISFR_ISF18_MASK                    (0x40000U)
#define RGPIO_ISFR_ISF18_SHIFT                   (18U)
/*! ISF18 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF18(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF18_SHIFT)) & RGPIO_ISFR_ISF18_MASK)

#define RGPIO_ISFR_ISF19_MASK                    (0x80000U)
#define RGPIO_ISFR_ISF19_SHIFT                   (19U)
/*! ISF19 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF19(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF19_SHIFT)) & RGPIO_ISFR_ISF19_MASK)

#define RGPIO_ISFR_ISF20_MASK                    (0x100000U)
#define RGPIO_ISFR_ISF20_SHIFT                   (20U)
/*! ISF20 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF20(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF20_SHIFT)) & RGPIO_ISFR_ISF20_MASK)

#define RGPIO_ISFR_ISF21_MASK                    (0x200000U)
#define RGPIO_ISFR_ISF21_SHIFT                   (21U)
/*! ISF21 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF21(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF21_SHIFT)) & RGPIO_ISFR_ISF21_MASK)

#define RGPIO_ISFR_ISF22_MASK                    (0x400000U)
#define RGPIO_ISFR_ISF22_SHIFT                   (22U)
/*! ISF22 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF22(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF22_SHIFT)) & RGPIO_ISFR_ISF22_MASK)

#define RGPIO_ISFR_ISF23_MASK                    (0x800000U)
#define RGPIO_ISFR_ISF23_SHIFT                   (23U)
/*! ISF23 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF23(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF23_SHIFT)) & RGPIO_ISFR_ISF23_MASK)

#define RGPIO_ISFR_ISF24_MASK                    (0x1000000U)
#define RGPIO_ISFR_ISF24_SHIFT                   (24U)
/*! ISF24 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF24(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF24_SHIFT)) & RGPIO_ISFR_ISF24_MASK)

#define RGPIO_ISFR_ISF25_MASK                    (0x2000000U)
#define RGPIO_ISFR_ISF25_SHIFT                   (25U)
/*! ISF25 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF25(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF25_SHIFT)) & RGPIO_ISFR_ISF25_MASK)

#define RGPIO_ISFR_ISF26_MASK                    (0x4000000U)
#define RGPIO_ISFR_ISF26_SHIFT                   (26U)
/*! ISF26 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF26(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF26_SHIFT)) & RGPIO_ISFR_ISF26_MASK)

#define RGPIO_ISFR_ISF27_MASK                    (0x8000000U)
#define RGPIO_ISFR_ISF27_SHIFT                   (27U)
/*! ISF27 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF27(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF27_SHIFT)) & RGPIO_ISFR_ISF27_MASK)

#define RGPIO_ISFR_ISF28_MASK                    (0x10000000U)
#define RGPIO_ISFR_ISF28_SHIFT                   (28U)
/*! ISF28 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF28(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF28_SHIFT)) & RGPIO_ISFR_ISF28_MASK)

#define RGPIO_ISFR_ISF29_MASK                    (0x20000000U)
#define RGPIO_ISFR_ISF29_SHIFT                   (29U)
/*! ISF29 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF29(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF29_SHIFT)) & RGPIO_ISFR_ISF29_MASK)

#define RGPIO_ISFR_ISF30_MASK                    (0x40000000U)
#define RGPIO_ISFR_ISF30_SHIFT                   (30U)
/*! ISF30 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF30(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF30_SHIFT)) & RGPIO_ISFR_ISF30_MASK)

#define RGPIO_ISFR_ISF31_MASK                    (0x80000000U)
#define RGPIO_ISFR_ISF31_SHIFT                   (31U)
/*! ISF31 - Interrupt Status Flag
 *  0b0..Configured interrupt is not detected on the pin of the same number.
 *  0b1..Configured interrupt is detected on the pin of the same number. If the pin is configured to generate a
 *       DMA request, then the corresponding flag will be cleared automatically at the completion of the requested
 *       DMA transfer. Otherwise, the flag remains set until a logic 1 is written to the flag. If the pin is
 *       configured for a level sensitive interrupt and the pin remains asserted, then the flag is set again immediately
 *       after it is cleared.
 */
#define RGPIO_ISFR_ISF31(x)                      (((uint32_t)(((uint32_t)(x)) << RGPIO_ISFR_ISF31_SHIFT)) & RGPIO_ISFR_ISF31_MASK)
/*! @} */

/* The count of RGPIO_ISFR */
#define RGPIO_ISFR_COUNT                         (2U)


/*!
 * @}
 */ /* end of group RGPIO_Register_Masks */


/* RGPIO - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral RGPIO1 base address */
  #define RGPIO1_BASE                              (0x57400000u)
  /** Peripheral RGPIO1 base address */
  #define RGPIO1_BASE_NS                           (0x47400000u)
  /** Peripheral RGPIO1 base pointer */
  #define RGPIO1                                   ((RGPIO_Type *)RGPIO1_BASE)
  /** Peripheral RGPIO1 base pointer */
  #define RGPIO1_NS                                ((RGPIO_Type *)RGPIO1_BASE_NS)
  /** Peripheral RGPIO2 base address */
  #define RGPIO2_BASE                              (0x53810000u)
  /** Peripheral RGPIO2 base address */
  #define RGPIO2_BASE_NS                           (0x43810000u)
  /** Peripheral RGPIO2 base pointer */
  #define RGPIO2                                   ((RGPIO_Type *)RGPIO2_BASE)
  /** Peripheral RGPIO2 base pointer */
  #define RGPIO2_NS                                ((RGPIO_Type *)RGPIO2_BASE_NS)
  /** Peripheral RGPIO3 base address */
  #define RGPIO3_BASE                              (0x53820000u)
  /** Peripheral RGPIO3 base address */
  #define RGPIO3_BASE_NS                           (0x43820000u)
  /** Peripheral RGPIO3 base pointer */
  #define RGPIO3                                   ((RGPIO_Type *)RGPIO3_BASE)
  /** Peripheral RGPIO3 base pointer */
  #define RGPIO3_NS                                ((RGPIO_Type *)RGPIO3_BASE_NS)
  /** Peripheral RGPIO4 base address */
  #define RGPIO4_BASE                              (0x53830000u)
  /** Peripheral RGPIO4 base address */
  #define RGPIO4_BASE_NS                           (0x43830000u)
  /** Peripheral RGPIO4 base pointer */
  #define RGPIO4                                   ((RGPIO_Type *)RGPIO4_BASE)
  /** Peripheral RGPIO4 base pointer */
  #define RGPIO4_NS                                ((RGPIO_Type *)RGPIO4_BASE_NS)
  /** Peripheral RGPIO5 base address */
  #define RGPIO5_BASE                              (0x53840000u)
  /** Peripheral RGPIO5 base address */
  #define RGPIO5_BASE_NS                           (0x43840000u)
  /** Peripheral RGPIO5 base pointer */
  #define RGPIO5                                   ((RGPIO_Type *)RGPIO5_BASE)
  /** Peripheral RGPIO5 base pointer */
  #define RGPIO5_NS                                ((RGPIO_Type *)RGPIO5_BASE_NS)
  /** Peripheral RGPIO6 base address */
  #define RGPIO6_BASE                              (0x53850000u)
  /** Peripheral RGPIO6 base address */
  #define RGPIO6_BASE_NS                           (0x43850000u)
  /** Peripheral RGPIO6 base pointer */
  #define RGPIO6                                   ((RGPIO_Type *)RGPIO6_BASE)
  /** Peripheral RGPIO6 base pointer */
  #define RGPIO6_NS                                ((RGPIO_Type *)RGPIO6_BASE_NS)
  /** Array initializer of RGPIO peripheral base addresses */
  #define RGPIO_BASE_ADDRS                         { 0u, RGPIO1_BASE, RGPIO2_BASE, RGPIO3_BASE, RGPIO4_BASE, RGPIO5_BASE, RGPIO6_BASE }
  /** Array initializer of RGPIO peripheral base pointers */
  #define RGPIO_BASE_PTRS                          { (RGPIO_Type *)0u, RGPIO1, RGPIO2, RGPIO3, RGPIO4, RGPIO5, RGPIO6 }
  /** Array initializer of RGPIO peripheral base addresses */
  #define RGPIO_BASE_ADDRS_NS                      { 0u, RGPIO1_BASE_NS, RGPIO2_BASE_NS, RGPIO3_BASE_NS, RGPIO4_BASE_NS, RGPIO5_BASE_NS, RGPIO6_BASE_NS }
  /** Array initializer of RGPIO peripheral base pointers */
  #define RGPIO_BASE_PTRS_NS                       { (RGPIO_Type *)0u, RGPIO1_NS, RGPIO2_NS, RGPIO3_NS, RGPIO4_NS, RGPIO5_NS, RGPIO6_NS }
#else
  /** Peripheral RGPIO1 base address */
  #define RGPIO1_BASE                              (0x47400000u)
  /** Peripheral RGPIO1 base pointer */
  #define RGPIO1                                   ((RGPIO_Type *)RGPIO1_BASE)
  /** Peripheral RGPIO2 base address */
  #define RGPIO2_BASE                              (0x43810000u)
  /** Peripheral RGPIO2 base pointer */
  #define RGPIO2                                   ((RGPIO_Type *)RGPIO2_BASE)
  /** Peripheral RGPIO3 base address */
  #define RGPIO3_BASE                              (0x43820000u)
  /** Peripheral RGPIO3 base pointer */
  #define RGPIO3                                   ((RGPIO_Type *)RGPIO3_BASE)
  /** Peripheral RGPIO4 base address */
  #define RGPIO4_BASE                              (0x43830000u)
  /** Peripheral RGPIO4 base pointer */
  #define RGPIO4                                   ((RGPIO_Type *)RGPIO4_BASE)
  /** Peripheral RGPIO5 base address */
  #define RGPIO5_BASE                              (0x43840000u)
  /** Peripheral RGPIO5 base pointer */
  #define RGPIO5                                   ((RGPIO_Type *)RGPIO5_BASE)
  /** Peripheral RGPIO6 base address */
  #define RGPIO6_BASE                              (0x43850000u)
  /** Peripheral RGPIO6 base pointer */
  #define RGPIO6                                   ((RGPIO_Type *)RGPIO6_BASE)
  /** Array initializer of RGPIO peripheral base addresses */
  #define RGPIO_BASE_ADDRS                         { 0u, RGPIO1_BASE, RGPIO2_BASE, RGPIO3_BASE, RGPIO4_BASE, RGPIO5_BASE, RGPIO6_BASE }
  /** Array initializer of RGPIO peripheral base pointers */
  #define RGPIO_BASE_PTRS                          { (RGPIO_Type *)0u, RGPIO1, RGPIO2, RGPIO3, RGPIO4, RGPIO5, RGPIO6 }
#endif
/** Interrupt vectors for the RGPIO peripheral type */
#define RGPIO_CH0_IRQS                           { NotAvail_IRQn, GPIO1_0_IRQn, GPIO2_0_IRQn, GPIO3_0_IRQn, GPIO4_IRQn, GPIO5_IRQn, GPIO6_IRQn }
#define RGPIO_CH1_IRQS                           { NotAvail_IRQn, GPIO1_1_IRQn, GPIO2_1_IRQn, GPIO3_1_IRQn, GPIO4_IRQn, GPIO5_IRQn, GPIO6_IRQn }

/*!
 * @}
 */ /* end of group RGPIO_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- RTWDOG Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup RTWDOG_Peripheral_Access_Layer RTWDOG Peripheral Access Layer
 * @{
 */

/** RTWDOG - Register Layout Typedef */
typedef struct {
  __IO uint32_t CS;                                /**< WDOG Control and Status, offset: 0x0 */
  __IO uint32_t CNT;                               /**< WDOG Counter, offset: 0x4 */
  __IO uint32_t TOVAL;                             /**< WDOG Timeout Value, offset: 0x8 */
  __IO uint32_t WIN;                               /**< Watchdog Window, offset: 0xC */
} RTWDOG_Type;

/* ----------------------------------------------------------------------------
   -- RTWDOG Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup RTWDOG_Register_Masks RTWDOG Register Masks
 * @{
 */

/*! @name CS - WDOG Control and Status */
/*! @{ */

#define RTWDOG_CS_STOP_MASK                      (0x1U)
#define RTWDOG_CS_STOP_SHIFT                     (0U)
/*! STOP - Stop Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_STOP(x)                        (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_STOP_SHIFT)) & RTWDOG_CS_STOP_MASK)

#define RTWDOG_CS_WAIT_MASK                      (0x2U)
#define RTWDOG_CS_WAIT_SHIFT                     (1U)
/*! WAIT - Wait Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_WAIT(x)                        (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_WAIT_SHIFT)) & RTWDOG_CS_WAIT_MASK)

#define RTWDOG_CS_DBG_MASK                       (0x4U)
#define RTWDOG_CS_DBG_SHIFT                      (2U)
/*! DBG - Debug Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_DBG(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_DBG_SHIFT)) & RTWDOG_CS_DBG_MASK)

#define RTWDOG_CS_TST_MASK                       (0x18U)
#define RTWDOG_CS_TST_SHIFT                      (3U)
/*! TST - WDOG Test
 *  0b00..Disable WDOG Test mode
 *  0b01..Enable WDOG User mode
 *  0b10-0b11..Enable WDOG Test mode
 */
#define RTWDOG_CS_TST(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_TST_SHIFT)) & RTWDOG_CS_TST_MASK)

#define RTWDOG_CS_UPDATE_MASK                    (0x20U)
#define RTWDOG_CS_UPDATE_SHIFT                   (5U)
/*! UPDATE - Updates Allowed
 *  0b0..Updates not allowed
 *  0b1..Updates allowed
 */
#define RTWDOG_CS_UPDATE(x)                      (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_UPDATE_SHIFT)) & RTWDOG_CS_UPDATE_MASK)

#define RTWDOG_CS_INT_MASK                       (0x40U)
#define RTWDOG_CS_INT_SHIFT                      (6U)
/*! INT - WDOG Interrupt
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_INT(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_INT_SHIFT)) & RTWDOG_CS_INT_MASK)

#define RTWDOG_CS_EN_MASK                        (0x80U)
#define RTWDOG_CS_EN_SHIFT                       (7U)
/*! EN - WDOG Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_EN(x)                          (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_EN_SHIFT)) & RTWDOG_CS_EN_MASK)

#define RTWDOG_CS_CLK_MASK                       (0x300U)
#define RTWDOG_CS_CLK_SHIFT                      (8U)
/*! CLK - WDOG Clock */
#define RTWDOG_CS_CLK(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_CLK_SHIFT)) & RTWDOG_CS_CLK_MASK)

#define RTWDOG_CS_RCS_MASK                       (0x400U)
#define RTWDOG_CS_RCS_SHIFT                      (10U)
/*! RCS - Reconfiguration Success
 *  0b0..Unsuccessful
 *  0b1..Successful
 */
#define RTWDOG_CS_RCS(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_RCS_SHIFT)) & RTWDOG_CS_RCS_MASK)

#define RTWDOG_CS_ULK_MASK                       (0x800U)
#define RTWDOG_CS_ULK_SHIFT                      (11U)
/*! ULK - Unlock Status
 *  0b0..Locked
 *  0b1..Unlocked
 */
#define RTWDOG_CS_ULK(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_ULK_SHIFT)) & RTWDOG_CS_ULK_MASK)

#define RTWDOG_CS_PRES_MASK                      (0x1000U)
#define RTWDOG_CS_PRES_SHIFT                     (12U)
/*! PRES - WDOG Prescaler
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_PRES(x)                        (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_PRES_SHIFT)) & RTWDOG_CS_PRES_MASK)

#define RTWDOG_CS_CMD32EN_MASK                   (0x2000U)
#define RTWDOG_CS_CMD32EN_SHIFT                  (13U)
/*! CMD32EN - Command 32 Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_CMD32EN(x)                     (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_CMD32EN_SHIFT)) & RTWDOG_CS_CMD32EN_MASK)

#define RTWDOG_CS_FLG_MASK                       (0x4000U)
#define RTWDOG_CS_FLG_SHIFT                      (14U)
/*! FLG - WDOG Interrupt Flag
 *  0b0..No interrupt occurred
 *  0b1..An interrupt occurred
 */
#define RTWDOG_CS_FLG(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_FLG_SHIFT)) & RTWDOG_CS_FLG_MASK)

#define RTWDOG_CS_WIN_MASK                       (0x8000U)
#define RTWDOG_CS_WIN_SHIFT                      (15U)
/*! WIN - WDOG Window
 *  0b0..Disable
 *  0b1..Enable
 */
#define RTWDOG_CS_WIN(x)                         (((uint32_t)(((uint32_t)(x)) << RTWDOG_CS_WIN_SHIFT)) & RTWDOG_CS_WIN_MASK)
/*! @} */

/*! @name CNT - WDOG Counter */
/*! @{ */

#define RTWDOG_CNT_CNTLOW_MASK                   (0xFFU)
#define RTWDOG_CNT_CNTLOW_SHIFT                  (0U)
/*! CNTLOW - Counter High Byte */
#define RTWDOG_CNT_CNTLOW(x)                     (((uint32_t)(((uint32_t)(x)) << RTWDOG_CNT_CNTLOW_SHIFT)) & RTWDOG_CNT_CNTLOW_MASK)

#define RTWDOG_CNT_CNTHIGH_MASK                  (0xFF00U)
#define RTWDOG_CNT_CNTHIGH_SHIFT                 (8U)
/*! CNTHIGH - Counter Low Byte */
#define RTWDOG_CNT_CNTHIGH(x)                    (((uint32_t)(((uint32_t)(x)) << RTWDOG_CNT_CNTHIGH_SHIFT)) & RTWDOG_CNT_CNTHIGH_MASK)
/*! @} */

/*! @name TOVAL - WDOG Timeout Value */
/*! @{ */

#define RTWDOG_TOVAL_TOVALLOW_MASK               (0xFFU)
#define RTWDOG_TOVAL_TOVALLOW_SHIFT              (0U)
/*! TOVALLOW - Timeout Value Low */
#define RTWDOG_TOVAL_TOVALLOW(x)                 (((uint32_t)(((uint32_t)(x)) << RTWDOG_TOVAL_TOVALLOW_SHIFT)) & RTWDOG_TOVAL_TOVALLOW_MASK)

#define RTWDOG_TOVAL_TOVALHIGH_MASK              (0xFF00U)
#define RTWDOG_TOVAL_TOVALHIGH_SHIFT             (8U)
/*! TOVALHIGH - Timeout Value High */
#define RTWDOG_TOVAL_TOVALHIGH(x)                (((uint32_t)(((uint32_t)(x)) << RTWDOG_TOVAL_TOVALHIGH_SHIFT)) & RTWDOG_TOVAL_TOVALHIGH_MASK)
/*! @} */

/*! @name WIN - Watchdog Window */
/*! @{ */

#define RTWDOG_WIN_WINLOW_MASK                   (0xFFU)
#define RTWDOG_WIN_WINLOW_SHIFT                  (0U)
/*! WINLOW - Low Byte */
#define RTWDOG_WIN_WINLOW(x)                     (((uint32_t)(((uint32_t)(x)) << RTWDOG_WIN_WINLOW_SHIFT)) & RTWDOG_WIN_WINLOW_MASK)

#define RTWDOG_WIN_WINHIGH_MASK                  (0xFF00U)
#define RTWDOG_WIN_WINHIGH_SHIFT                 (8U)
/*! WINHIGH - High Byte */
#define RTWDOG_WIN_WINHIGH(x)                    (((uint32_t)(((uint32_t)(x)) << RTWDOG_WIN_WINHIGH_SHIFT)) & RTWDOG_WIN_WINHIGH_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group RTWDOG_Register_Masks */


/* RTWDOG - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral RTWDOG1 base address */
  #define RTWDOG1_BASE                             (0x542D0000u)
  /** Peripheral RTWDOG1 base address */
  #define RTWDOG1_BASE_NS                          (0x442D0000u)
  /** Peripheral RTWDOG1 base pointer */
  #define RTWDOG1                                  ((RTWDOG_Type *)RTWDOG1_BASE)
  /** Peripheral RTWDOG1 base pointer */
  #define RTWDOG1_NS                               ((RTWDOG_Type *)RTWDOG1_BASE_NS)
  /** Peripheral RTWDOG2 base address */
  #define RTWDOG2_BASE                             (0x542E0000u)
  /** Peripheral RTWDOG2 base address */
  #define RTWDOG2_BASE_NS                          (0x442E0000u)
  /** Peripheral RTWDOG2 base pointer */
  #define RTWDOG2                                  ((RTWDOG_Type *)RTWDOG2_BASE)
  /** Peripheral RTWDOG2 base pointer */
  #define RTWDOG2_NS                               ((RTWDOG_Type *)RTWDOG2_BASE_NS)
  /** Peripheral RTWDOG3 base address */
  #define RTWDOG3_BASE                             (0x52490000u)
  /** Peripheral RTWDOG3 base address */
  #define RTWDOG3_BASE_NS                          (0x42490000u)
  /** Peripheral RTWDOG3 base pointer */
  #define RTWDOG3                                  ((RTWDOG_Type *)RTWDOG3_BASE)
  /** Peripheral RTWDOG3 base pointer */
  #define RTWDOG3_NS                               ((RTWDOG_Type *)RTWDOG3_BASE_NS)
  /** Peripheral RTWDOG4 base address */
  #define RTWDOG4_BASE                             (0x524A0000u)
  /** Peripheral RTWDOG4 base address */
  #define RTWDOG4_BASE_NS                          (0x424A0000u)
  /** Peripheral RTWDOG4 base pointer */
  #define RTWDOG4                                  ((RTWDOG_Type *)RTWDOG4_BASE)
  /** Peripheral RTWDOG4 base pointer */
  #define RTWDOG4_NS                               ((RTWDOG_Type *)RTWDOG4_BASE_NS)
  /** Peripheral RTWDOG5 base address */
  #define RTWDOG5_BASE                             (0x524B0000u)
  /** Peripheral RTWDOG5 base address */
  #define RTWDOG5_BASE_NS                          (0x424B0000u)
  /** Peripheral RTWDOG5 base pointer */
  #define RTWDOG5                                  ((RTWDOG_Type *)RTWDOG5_BASE)
  /** Peripheral RTWDOG5 base pointer */
  #define RTWDOG5_NS                               ((RTWDOG_Type *)RTWDOG5_BASE_NS)
  /** Array initializer of RTWDOG peripheral base addresses */
  #define RTWDOG_BASE_ADDRS                        { 0u, RTWDOG1_BASE, RTWDOG2_BASE, RTWDOG3_BASE, RTWDOG4_BASE, RTWDOG5_BASE }
  /** Array initializer of RTWDOG peripheral base pointers */
  #define RTWDOG_BASE_PTRS                         { (RTWDOG_Type *)0u, RTWDOG1, RTWDOG2, RTWDOG3, RTWDOG4, RTWDOG5 }
  /** Array initializer of RTWDOG peripheral base addresses */
  #define RTWDOG_BASE_ADDRS_NS                     { 0u, RTWDOG1_BASE_NS, RTWDOG2_BASE_NS, RTWDOG3_BASE_NS, RTWDOG4_BASE_NS, RTWDOG5_BASE_NS }
  /** Array initializer of RTWDOG peripheral base pointers */
  #define RTWDOG_BASE_PTRS_NS                      { (RTWDOG_Type *)0u, RTWDOG1_NS, RTWDOG2_NS, RTWDOG3_NS, RTWDOG4_NS, RTWDOG5_NS }
#else
  /** Peripheral RTWDOG1 base address */
  #define RTWDOG1_BASE                             (0x442D0000u)
  /** Peripheral RTWDOG1 base pointer */
  #define RTWDOG1                                  ((RTWDOG_Type *)RTWDOG1_BASE)
  /** Peripheral RTWDOG2 base address */
  #define RTWDOG2_BASE                             (0x442E0000u)
  /** Peripheral RTWDOG2 base pointer */
  #define RTWDOG2                                  ((RTWDOG_Type *)RTWDOG2_BASE)
  /** Peripheral RTWDOG3 base address */
  #define RTWDOG3_BASE                             (0x42490000u)
  /** Peripheral RTWDOG3 base pointer */
  #define RTWDOG3                                  ((RTWDOG_Type *)RTWDOG3_BASE)
  /** Peripheral RTWDOG4 base address */
  #define RTWDOG4_BASE                             (0x424A0000u)
  /** Peripheral RTWDOG4 base pointer */
  #define RTWDOG4                                  ((RTWDOG_Type *)RTWDOG4_BASE)
  /** Peripheral RTWDOG5 base address */
  #define RTWDOG5_BASE                             (0x424B0000u)
  /** Peripheral RTWDOG5 base pointer */
  #define RTWDOG5                                  ((RTWDOG_Type *)RTWDOG5_BASE)
  /** Array initializer of RTWDOG peripheral base addresses */
  #define RTWDOG_BASE_ADDRS                        { 0u, RTWDOG1_BASE, RTWDOG2_BASE, RTWDOG3_BASE, RTWDOG4_BASE, RTWDOG5_BASE }
  /** Array initializer of RTWDOG peripheral base pointers */
  #define RTWDOG_BASE_PTRS                         { (RTWDOG_Type *)0u, RTWDOG1, RTWDOG2, RTWDOG3, RTWDOG4, RTWDOG5 }
#endif
/** Interrupt vectors for the RTWDOG peripheral type */
#define RTWDOG_IRQS                              { NotAvail_IRQn, RTWDOG1_IRQn, RTWDOG2_IRQn, RTWDOG3_IRQn, RTWDOG4_IRQn, RTWDOG5_IRQn }
/* Extra definition */
#define RTWDOG_UPDATE_KEY                        (0xD928C520U)
#define RTWDOG_REFRESH_KEY                       (0xB480A602U)


/*!
 * @}
 */ /* end of group RTWDOG_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- S3MU Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup S3MU_Peripheral_Access_Layer S3MU Peripheral Access Layer
 * @{
 */

/** S3MU - Register Layout Typedef */
typedef struct {
  __I  uint32_t VER;                               /**< Version ID Register, offset: 0x0 */
  __I  uint32_t PAR;                               /**< Parameter Register, offset: 0x4 */
       uint32_t UNUSED0;                           /**< Unused Register 0, offset: 0x8 */
  __I  uint32_t SR;                                /**< Status Register, offset: 0xC */
       uint8_t RESERVED_0[272];
  __IO uint32_t TCR;                               /**< Transmit Control Register, offset: 0x120 */
  __I  uint32_t TSR;                               /**< Transmit Status Register, offset: 0x124 */
  __IO uint32_t RCR;                               /**< Receive Control Register, offset: 0x128 */
  __I  uint32_t RSR;                               /**< Receive Status Register, offset: 0x12C */
       uint8_t RESERVED_1[204];
  __IO uint32_t UNUSED1;                           /**< Unused Register 1, offset: 0x1FC */
  __O  uint32_t TR[8];                             /**< Transmit Register, array offset: 0x200, array step: 0x4 */
       uint8_t RESERVED_2[96];
  __I  uint32_t RR[4];                             /**< Receive Register, array offset: 0x280, array step: 0x4 */
} S3MU_Type;

/* ----------------------------------------------------------------------------
   -- S3MU Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup S3MU_Register_Masks S3MU Register Masks
 * @{
 */

/*! @name VER - Version ID Register */
/*! @{ */

#define S3MU_VER_FEATURE_MASK                    (0xFFFFU)
#define S3MU_VER_FEATURE_SHIFT                   (0U)
/*! FEATURE - Feature Set Number
 *  0b0000000000000000..Standard features are implemented.
 */
#define S3MU_VER_FEATURE(x)                      (((uint32_t)(((uint32_t)(x)) << S3MU_VER_FEATURE_SHIFT)) & S3MU_VER_FEATURE_MASK)

#define S3MU_VER_MINOR_MASK                      (0xFF0000U)
#define S3MU_VER_MINOR_SHIFT                     (16U)
/*! MINOR - Minor Version Number (0x00 ) */
#define S3MU_VER_MINOR(x)                        (((uint32_t)(((uint32_t)(x)) << S3MU_VER_MINOR_SHIFT)) & S3MU_VER_MINOR_MASK)

#define S3MU_VER_MAJOR_MASK                      (0xFF000000U)
#define S3MU_VER_MAJOR_SHIFT                     (24U)
/*! MAJOR - Major Version Number (0x01 ) */
#define S3MU_VER_MAJOR(x)                        (((uint32_t)(((uint32_t)(x)) << S3MU_VER_MAJOR_SHIFT)) & S3MU_VER_MAJOR_MASK)
/*! @} */

/*! @name PAR - Parameter Register */
/*! @{ */

#define S3MU_PAR_TR_NUM_MASK                     (0xFFU)
#define S3MU_PAR_TR_NUM_SHIFT                    (0U)
/*! TR_NUM - Number of Transmit (TRn) registers (8) */
#define S3MU_PAR_TR_NUM(x)                       (((uint32_t)(((uint32_t)(x)) << S3MU_PAR_TR_NUM_SHIFT)) & S3MU_PAR_TR_NUM_MASK)

#define S3MU_PAR_RR_NUM_MASK                     (0xFF00U)
#define S3MU_PAR_RR_NUM_SHIFT                    (8U)
/*! RR_NUM - Number of Receive (RRn) registers (4) */
#define S3MU_PAR_RR_NUM(x)                       (((uint32_t)(((uint32_t)(x)) << S3MU_PAR_RR_NUM_SHIFT)) & S3MU_PAR_RR_NUM_MASK)
/*! @} */

/*! @name SR - Status Register */
/*! @{ */

#define S3MU_SR_TEP_MASK                         (0x20U)
#define S3MU_SR_TEP_SHIFT                        (5U)
/*! TEP - Transmit Empty Pending */
#define S3MU_SR_TEP(x)                           (((uint32_t)(((uint32_t)(x)) << S3MU_SR_TEP_SHIFT)) & S3MU_SR_TEP_MASK)

#define S3MU_SR_RFP_MASK                         (0x40U)
#define S3MU_SR_RFP_SHIFT                        (6U)
/*! RFP - Receive Full Pending Flag
 *  0b0..No data is ready to be read. All RSR[RFn] bits are clear.
 *  0b1..Data is ready to be read. One or more RSR[RFn] bits are set.
 */
#define S3MU_SR_RFP(x)                           (((uint32_t)(((uint32_t)(x)) << S3MU_SR_RFP_SHIFT)) & S3MU_SR_RFP_MASK)
/*! @} */

/*! @name TCR - Transmit Control Register */
/*! @{ */

#define S3MU_TCR_TEIEn_MASK                      (0xFFU)
#define S3MU_TCR_TEIEn_SHIFT                     (0U)
/*! TEIEn - Transmit Register n Empty Interrupt Enable */
#define S3MU_TCR_TEIEn(x)                        (((uint32_t)(((uint32_t)(x)) << S3MU_TCR_TEIEn_SHIFT)) & S3MU_TCR_TEIEn_MASK)
/*! @} */

/*! @name TSR - Transmit Status Register */
/*! @{ */

#define S3MU_TSR_TEn_MASK                        (0xFFU)
#define S3MU_TSR_TEn_SHIFT                       (0U)
/*! TEn - Transmit Register n Empty */
#define S3MU_TSR_TEn(x)                          (((uint32_t)(((uint32_t)(x)) << S3MU_TSR_TEn_SHIFT)) & S3MU_TSR_TEn_MASK)
/*! @} */

/*! @name RCR - Receive Control Register */
/*! @{ */

#define S3MU_RCR_RFIEn_MASK                      (0xFU)
#define S3MU_RCR_RFIEn_SHIFT                     (0U)
/*! RFIEn - Receive Register n Full Interrupt Enable */
#define S3MU_RCR_RFIEn(x)                        (((uint32_t)(((uint32_t)(x)) << S3MU_RCR_RFIEn_SHIFT)) & S3MU_RCR_RFIEn_MASK)
/*! @} */

/*! @name RSR - Receive Status Register */
/*! @{ */

#define S3MU_RSR_RFn_MASK                        (0xFU)
#define S3MU_RSR_RFn_SHIFT                       (0U)
/*! RFn - Receive Register n Full */
#define S3MU_RSR_RFn(x)                          (((uint32_t)(((uint32_t)(x)) << S3MU_RSR_RFn_SHIFT)) & S3MU_RSR_RFn_MASK)
/*! @} */

/*! @name UNUSED1 - Unused Register 1 */
/*! @{ */

#define S3MU_UNUSED1_DATA16_MASK                 (0xFFFFU)
#define S3MU_UNUSED1_DATA16_SHIFT                (0U)
/*! DATA16 - Unused 16-bit Register */
#define S3MU_UNUSED1_DATA16(x)                   (((uint32_t)(((uint32_t)(x)) << S3MU_UNUSED1_DATA16_SHIFT)) & S3MU_UNUSED1_DATA16_MASK)
/*! @} */

/*! @name TR - Transmit Register */
/*! @{ */

#define S3MU_TR_TR_DATA_MASK                     (0xFFFFFFFFU)
#define S3MU_TR_TR_DATA_SHIFT                    (0U)
/*! TR_DATA - Transmit Data */
#define S3MU_TR_TR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << S3MU_TR_TR_DATA_SHIFT)) & S3MU_TR_TR_DATA_MASK)
/*! @} */

/* The count of S3MU_TR */
#define S3MU_TR_COUNT                            (8U)

/*! @name RR - Receive Register */
/*! @{ */

#define S3MU_RR_RR_DATA_MASK                     (0xFFFFFFFFU)
#define S3MU_RR_RR_DATA_SHIFT                    (0U)
/*! RR_DATA - Receive Data */
#define S3MU_RR_RR_DATA(x)                       (((uint32_t)(((uint32_t)(x)) << S3MU_RR_RR_DATA_SHIFT)) & S3MU_RR_RR_DATA_MASK)
/*! @} */

/* The count of S3MU_RR */
#define S3MU_RR_COUNT                            (4U)


/*!
 * @}
 */ /* end of group S3MU_Register_Masks */


/* S3MU - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral MU_APPS_S3MUA base address */
  #define MU_APPS_S3MUA_BASE                       (0x57520000u)
  /** Peripheral MU_APPS_S3MUA base address */
  #define MU_APPS_S3MUA_BASE_NS                    (0x47520000u)
  /** Peripheral MU_APPS_S3MUA base pointer */
  #define MU_APPS_S3MUA                            ((S3MU_Type *)MU_APPS_S3MUA_BASE)
  /** Peripheral MU_APPS_S3MUA base pointer */
  #define MU_APPS_S3MUA_NS                         ((S3MU_Type *)MU_APPS_S3MUA_BASE_NS)
  /** Peripheral MU_RT_S3MUA base address */
  #define MU_RT_S3MUA_BASE                         (0x57540000u)
  /** Peripheral MU_RT_S3MUA base address */
  #define MU_RT_S3MUA_BASE_NS                      (0x47540000u)
  /** Peripheral MU_RT_S3MUA base pointer */
  #define MU_RT_S3MUA                              ((S3MU_Type *)MU_RT_S3MUA_BASE)
  /** Peripheral MU_RT_S3MUA base pointer */
  #define MU_RT_S3MUA_NS                           ((S3MU_Type *)MU_RT_S3MUA_BASE_NS)
  /** Array initializer of S3MU peripheral base addresses */
  #define S3MU_BASE_ADDRS                          { MU_APPS_S3MUA_BASE, MU_RT_S3MUA_BASE }
  /** Array initializer of S3MU peripheral base pointers */
  #define S3MU_BASE_PTRS                           { MU_APPS_S3MUA, MU_RT_S3MUA }
  /** Array initializer of S3MU peripheral base addresses */
  #define S3MU_BASE_ADDRS_NS                       { MU_APPS_S3MUA_BASE_NS, MU_RT_S3MUA_BASE_NS }
  /** Array initializer of S3MU peripheral base pointers */
  #define S3MU_BASE_PTRS_NS                        { MU_APPS_S3MUA_NS, MU_RT_S3MUA_NS }
#else
  /** Peripheral MU_APPS_S3MUA base address */
  #define MU_APPS_S3MUA_BASE                       (0x47520000u)
  /** Peripheral MU_APPS_S3MUA base pointer */
  #define MU_APPS_S3MUA                            ((S3MU_Type *)MU_APPS_S3MUA_BASE)
  /** Peripheral MU_RT_S3MUA base address */
  #define MU_RT_S3MUA_BASE                         (0x47540000u)
  /** Peripheral MU_RT_S3MUA base pointer */
  #define MU_RT_S3MUA                              ((S3MU_Type *)MU_RT_S3MUA_BASE)
  /** Array initializer of S3MU peripheral base addresses */
  #define S3MU_BASE_ADDRS                          { MU_APPS_S3MUA_BASE, MU_RT_S3MUA_BASE }
  /** Array initializer of S3MU peripheral base pointers */
  #define S3MU_BASE_PTRS                           { MU_APPS_S3MUA, MU_RT_S3MUA }
#endif

/*!
 * @}
 */ /* end of group S3MU_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SEMA42 Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SEMA42_Peripheral_Access_Layer SEMA42 Peripheral Access Layer
 * @{
 */

/** SEMA42 - Register Layout Typedef */
typedef struct {
  __IO uint8_t GATE3;                              /**< Gate, offset: 0x0 */
  __IO uint8_t GATE2;                              /**< Gate, offset: 0x1 */
  __IO uint8_t GATE1;                              /**< Gate, offset: 0x2 */
  __IO uint8_t GATE0;                              /**< Gate, offset: 0x3 */
  __IO uint8_t GATE7;                              /**< Gate, offset: 0x4 */
  __IO uint8_t GATE6;                              /**< Gate, offset: 0x5 */
  __IO uint8_t GATE5;                              /**< Gate, offset: 0x6 */
  __IO uint8_t GATE4;                              /**< Gate, offset: 0x7 */
  __IO uint8_t GATE11;                             /**< Gate, offset: 0x8 */
  __IO uint8_t GATE10;                             /**< Gate, offset: 0x9 */
  __IO uint8_t GATE9;                              /**< Gate, offset: 0xA */
  __IO uint8_t GATE8;                              /**< Gate, offset: 0xB */
  __IO uint8_t GATE15;                             /**< Gate, offset: 0xC */
  __IO uint8_t GATE14;                             /**< Gate, offset: 0xD */
  __IO uint8_t GATE13;                             /**< Gate, offset: 0xE */
  __IO uint8_t GATE12;                             /**< Gate, offset: 0xF */
  __IO uint8_t GATE19;                             /**< Gate, offset: 0x10 */
  __IO uint8_t GATE18;                             /**< Gate, offset: 0x11 */
  __IO uint8_t GATE17;                             /**< Gate, offset: 0x12 */
  __IO uint8_t GATE16;                             /**< Gate, offset: 0x13 */
  __IO uint8_t GATE23;                             /**< Gate, offset: 0x14 */
  __IO uint8_t GATE22;                             /**< Gate, offset: 0x15 */
  __IO uint8_t GATE21;                             /**< Gate, offset: 0x16 */
  __IO uint8_t GATE20;                             /**< Gate, offset: 0x17 */
  __IO uint8_t GATE27;                             /**< Gate, offset: 0x18 */
  __IO uint8_t GATE26;                             /**< Gate, offset: 0x19 */
  __IO uint8_t GATE25;                             /**< Gate, offset: 0x1A */
  __IO uint8_t GATE24;                             /**< Gate, offset: 0x1B */
  __IO uint8_t GATE31;                             /**< Gate, offset: 0x1C */
  __IO uint8_t GATE30;                             /**< Gate, offset: 0x1D */
  __IO uint8_t GATE29;                             /**< Gate, offset: 0x1E */
  __IO uint8_t GATE28;                             /**< Gate, offset: 0x1F */
  __IO uint8_t GATE35;                             /**< Gate, offset: 0x20 */
  __IO uint8_t GATE34;                             /**< Gate, offset: 0x21 */
  __IO uint8_t GATE33;                             /**< Gate, offset: 0x22 */
  __IO uint8_t GATE32;                             /**< Gate, offset: 0x23 */
  __IO uint8_t GATE39;                             /**< Gate, offset: 0x24 */
  __IO uint8_t GATE38;                             /**< Gate, offset: 0x25 */
  __IO uint8_t GATE37;                             /**< Gate, offset: 0x26 */
  __IO uint8_t GATE36;                             /**< Gate, offset: 0x27 */
  __IO uint8_t GATE43;                             /**< Gate, offset: 0x28 */
  __IO uint8_t GATE42;                             /**< Gate, offset: 0x29 */
  __IO uint8_t GATE41;                             /**< Gate, offset: 0x2A */
  __IO uint8_t GATE40;                             /**< Gate, offset: 0x2B */
  __IO uint8_t GATE47;                             /**< Gate, offset: 0x2C */
  __IO uint8_t GATE46;                             /**< Gate, offset: 0x2D */
  __IO uint8_t GATE45;                             /**< Gate, offset: 0x2E */
  __IO uint8_t GATE44;                             /**< Gate, offset: 0x2F */
  __IO uint8_t GATE51;                             /**< Gate, offset: 0x30 */
  __IO uint8_t GATE50;                             /**< Gate, offset: 0x31 */
  __IO uint8_t GATE49;                             /**< Gate, offset: 0x32 */
  __IO uint8_t GATE48;                             /**< Gate, offset: 0x33 */
  __IO uint8_t GATE55;                             /**< Gate, offset: 0x34 */
  __IO uint8_t GATE54;                             /**< Gate, offset: 0x35 */
  __IO uint8_t GATE53;                             /**< Gate, offset: 0x36 */
  __IO uint8_t GATE52;                             /**< Gate, offset: 0x37 */
  __IO uint8_t GATE59;                             /**< Gate, offset: 0x38 */
  __IO uint8_t GATE58;                             /**< Gate, offset: 0x39 */
  __IO uint8_t GATE57;                             /**< Gate, offset: 0x3A */
  __IO uint8_t GATE56;                             /**< Gate, offset: 0x3B */
  __IO uint8_t GATE63;                             /**< Gate, offset: 0x3C */
  __IO uint8_t GATE62;                             /**< Gate, offset: 0x3D */
  __IO uint8_t GATE61;                             /**< Gate, offset: 0x3E */
  __IO uint8_t GATE60;                             /**< Gate, offset: 0x3F */
       uint8_t RESERVED_0[2];
  union {                                          /* offset: 0x42 */
    __I  uint16_t RSTGT_R;                           /**< Reset Gate Read, offset: 0x42 */
    __O  uint16_t RSTGT_W;                           /**< Reset Gate Write, offset: 0x42 */
  };
} SEMA42_Type;

/* ----------------------------------------------------------------------------
   -- SEMA42 Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SEMA42_Register_Masks SEMA42 Register Masks
 * @{
 */

/*! @name GATE3 - Gate */
/*! @{ */

#define SEMA42_GATE3_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE3_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE3_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE3_GTFSM_SHIFT)) & SEMA42_GATE3_GTFSM_MASK)
/*! @} */

/*! @name GATE2 - Gate */
/*! @{ */

#define SEMA42_GATE2_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE2_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE2_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE2_GTFSM_SHIFT)) & SEMA42_GATE2_GTFSM_MASK)
/*! @} */

/*! @name GATE1 - Gate */
/*! @{ */

#define SEMA42_GATE1_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE1_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE1_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE1_GTFSM_SHIFT)) & SEMA42_GATE1_GTFSM_MASK)
/*! @} */

/*! @name GATE0 - Gate */
/*! @{ */

#define SEMA42_GATE0_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE0_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE0_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE0_GTFSM_SHIFT)) & SEMA42_GATE0_GTFSM_MASK)
/*! @} */

/*! @name GATE7 - Gate */
/*! @{ */

#define SEMA42_GATE7_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE7_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE7_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE7_GTFSM_SHIFT)) & SEMA42_GATE7_GTFSM_MASK)
/*! @} */

/*! @name GATE6 - Gate */
/*! @{ */

#define SEMA42_GATE6_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE6_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE6_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE6_GTFSM_SHIFT)) & SEMA42_GATE6_GTFSM_MASK)
/*! @} */

/*! @name GATE5 - Gate */
/*! @{ */

#define SEMA42_GATE5_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE5_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE5_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE5_GTFSM_SHIFT)) & SEMA42_GATE5_GTFSM_MASK)
/*! @} */

/*! @name GATE4 - Gate */
/*! @{ */

#define SEMA42_GATE4_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE4_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE4_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE4_GTFSM_SHIFT)) & SEMA42_GATE4_GTFSM_MASK)
/*! @} */

/*! @name GATE11 - Gate */
/*! @{ */

#define SEMA42_GATE11_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE11_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE11_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE11_GTFSM_SHIFT)) & SEMA42_GATE11_GTFSM_MASK)
/*! @} */

/*! @name GATE10 - Gate */
/*! @{ */

#define SEMA42_GATE10_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE10_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE10_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE10_GTFSM_SHIFT)) & SEMA42_GATE10_GTFSM_MASK)
/*! @} */

/*! @name GATE9 - Gate */
/*! @{ */

#define SEMA42_GATE9_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE9_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE9_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE9_GTFSM_SHIFT)) & SEMA42_GATE9_GTFSM_MASK)
/*! @} */

/*! @name GATE8 - Gate */
/*! @{ */

#define SEMA42_GATE8_GTFSM_MASK                  (0xFU)
#define SEMA42_GATE8_GTFSM_SHIFT                 (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE8_GTFSM(x)                    (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE8_GTFSM_SHIFT)) & SEMA42_GATE8_GTFSM_MASK)
/*! @} */

/*! @name GATE15 - Gate */
/*! @{ */

#define SEMA42_GATE15_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE15_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE15_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE15_GTFSM_SHIFT)) & SEMA42_GATE15_GTFSM_MASK)
/*! @} */

/*! @name GATE14 - Gate */
/*! @{ */

#define SEMA42_GATE14_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE14_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE14_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE14_GTFSM_SHIFT)) & SEMA42_GATE14_GTFSM_MASK)
/*! @} */

/*! @name GATE13 - Gate */
/*! @{ */

#define SEMA42_GATE13_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE13_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE13_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE13_GTFSM_SHIFT)) & SEMA42_GATE13_GTFSM_MASK)
/*! @} */

/*! @name GATE12 - Gate */
/*! @{ */

#define SEMA42_GATE12_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE12_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE12_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE12_GTFSM_SHIFT)) & SEMA42_GATE12_GTFSM_MASK)
/*! @} */

/*! @name GATE19 - Gate */
/*! @{ */

#define SEMA42_GATE19_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE19_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE19_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE19_GTFSM_SHIFT)) & SEMA42_GATE19_GTFSM_MASK)
/*! @} */

/*! @name GATE18 - Gate */
/*! @{ */

#define SEMA42_GATE18_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE18_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE18_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE18_GTFSM_SHIFT)) & SEMA42_GATE18_GTFSM_MASK)
/*! @} */

/*! @name GATE17 - Gate */
/*! @{ */

#define SEMA42_GATE17_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE17_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE17_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE17_GTFSM_SHIFT)) & SEMA42_GATE17_GTFSM_MASK)
/*! @} */

/*! @name GATE16 - Gate */
/*! @{ */

#define SEMA42_GATE16_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE16_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE16_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE16_GTFSM_SHIFT)) & SEMA42_GATE16_GTFSM_MASK)
/*! @} */

/*! @name GATE23 - Gate */
/*! @{ */

#define SEMA42_GATE23_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE23_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE23_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE23_GTFSM_SHIFT)) & SEMA42_GATE23_GTFSM_MASK)
/*! @} */

/*! @name GATE22 - Gate */
/*! @{ */

#define SEMA42_GATE22_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE22_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE22_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE22_GTFSM_SHIFT)) & SEMA42_GATE22_GTFSM_MASK)
/*! @} */

/*! @name GATE21 - Gate */
/*! @{ */

#define SEMA42_GATE21_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE21_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE21_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE21_GTFSM_SHIFT)) & SEMA42_GATE21_GTFSM_MASK)
/*! @} */

/*! @name GATE20 - Gate */
/*! @{ */

#define SEMA42_GATE20_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE20_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE20_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE20_GTFSM_SHIFT)) & SEMA42_GATE20_GTFSM_MASK)
/*! @} */

/*! @name GATE27 - Gate */
/*! @{ */

#define SEMA42_GATE27_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE27_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE27_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE27_GTFSM_SHIFT)) & SEMA42_GATE27_GTFSM_MASK)
/*! @} */

/*! @name GATE26 - Gate */
/*! @{ */

#define SEMA42_GATE26_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE26_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE26_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE26_GTFSM_SHIFT)) & SEMA42_GATE26_GTFSM_MASK)
/*! @} */

/*! @name GATE25 - Gate */
/*! @{ */

#define SEMA42_GATE25_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE25_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE25_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE25_GTFSM_SHIFT)) & SEMA42_GATE25_GTFSM_MASK)
/*! @} */

/*! @name GATE24 - Gate */
/*! @{ */

#define SEMA42_GATE24_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE24_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE24_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE24_GTFSM_SHIFT)) & SEMA42_GATE24_GTFSM_MASK)
/*! @} */

/*! @name GATE31 - Gate */
/*! @{ */

#define SEMA42_GATE31_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE31_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE31_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE31_GTFSM_SHIFT)) & SEMA42_GATE31_GTFSM_MASK)
/*! @} */

/*! @name GATE30 - Gate */
/*! @{ */

#define SEMA42_GATE30_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE30_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE30_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE30_GTFSM_SHIFT)) & SEMA42_GATE30_GTFSM_MASK)
/*! @} */

/*! @name GATE29 - Gate */
/*! @{ */

#define SEMA42_GATE29_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE29_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE29_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE29_GTFSM_SHIFT)) & SEMA42_GATE29_GTFSM_MASK)
/*! @} */

/*! @name GATE28 - Gate */
/*! @{ */

#define SEMA42_GATE28_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE28_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE28_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE28_GTFSM_SHIFT)) & SEMA42_GATE28_GTFSM_MASK)
/*! @} */

/*! @name GATE35 - Gate */
/*! @{ */

#define SEMA42_GATE35_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE35_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE35_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE35_GTFSM_SHIFT)) & SEMA42_GATE35_GTFSM_MASK)
/*! @} */

/*! @name GATE34 - Gate */
/*! @{ */

#define SEMA42_GATE34_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE34_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE34_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE34_GTFSM_SHIFT)) & SEMA42_GATE34_GTFSM_MASK)
/*! @} */

/*! @name GATE33 - Gate */
/*! @{ */

#define SEMA42_GATE33_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE33_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE33_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE33_GTFSM_SHIFT)) & SEMA42_GATE33_GTFSM_MASK)
/*! @} */

/*! @name GATE32 - Gate */
/*! @{ */

#define SEMA42_GATE32_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE32_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE32_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE32_GTFSM_SHIFT)) & SEMA42_GATE32_GTFSM_MASK)
/*! @} */

/*! @name GATE39 - Gate */
/*! @{ */

#define SEMA42_GATE39_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE39_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE39_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE39_GTFSM_SHIFT)) & SEMA42_GATE39_GTFSM_MASK)
/*! @} */

/*! @name GATE38 - Gate */
/*! @{ */

#define SEMA42_GATE38_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE38_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE38_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE38_GTFSM_SHIFT)) & SEMA42_GATE38_GTFSM_MASK)
/*! @} */

/*! @name GATE37 - Gate */
/*! @{ */

#define SEMA42_GATE37_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE37_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE37_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE37_GTFSM_SHIFT)) & SEMA42_GATE37_GTFSM_MASK)
/*! @} */

/*! @name GATE36 - Gate */
/*! @{ */

#define SEMA42_GATE36_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE36_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE36_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE36_GTFSM_SHIFT)) & SEMA42_GATE36_GTFSM_MASK)
/*! @} */

/*! @name GATE43 - Gate */
/*! @{ */

#define SEMA42_GATE43_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE43_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE43_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE43_GTFSM_SHIFT)) & SEMA42_GATE43_GTFSM_MASK)
/*! @} */

/*! @name GATE42 - Gate */
/*! @{ */

#define SEMA42_GATE42_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE42_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE42_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE42_GTFSM_SHIFT)) & SEMA42_GATE42_GTFSM_MASK)
/*! @} */

/*! @name GATE41 - Gate */
/*! @{ */

#define SEMA42_GATE41_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE41_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE41_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE41_GTFSM_SHIFT)) & SEMA42_GATE41_GTFSM_MASK)
/*! @} */

/*! @name GATE40 - Gate */
/*! @{ */

#define SEMA42_GATE40_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE40_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE40_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE40_GTFSM_SHIFT)) & SEMA42_GATE40_GTFSM_MASK)
/*! @} */

/*! @name GATE47 - Gate */
/*! @{ */

#define SEMA42_GATE47_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE47_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE47_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE47_GTFSM_SHIFT)) & SEMA42_GATE47_GTFSM_MASK)
/*! @} */

/*! @name GATE46 - Gate */
/*! @{ */

#define SEMA42_GATE46_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE46_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE46_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE46_GTFSM_SHIFT)) & SEMA42_GATE46_GTFSM_MASK)
/*! @} */

/*! @name GATE45 - Gate */
/*! @{ */

#define SEMA42_GATE45_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE45_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE45_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE45_GTFSM_SHIFT)) & SEMA42_GATE45_GTFSM_MASK)
/*! @} */

/*! @name GATE44 - Gate */
/*! @{ */

#define SEMA42_GATE44_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE44_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE44_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE44_GTFSM_SHIFT)) & SEMA42_GATE44_GTFSM_MASK)
/*! @} */

/*! @name GATE51 - Gate */
/*! @{ */

#define SEMA42_GATE51_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE51_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE51_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE51_GTFSM_SHIFT)) & SEMA42_GATE51_GTFSM_MASK)
/*! @} */

/*! @name GATE50 - Gate */
/*! @{ */

#define SEMA42_GATE50_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE50_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE50_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE50_GTFSM_SHIFT)) & SEMA42_GATE50_GTFSM_MASK)
/*! @} */

/*! @name GATE49 - Gate */
/*! @{ */

#define SEMA42_GATE49_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE49_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE49_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE49_GTFSM_SHIFT)) & SEMA42_GATE49_GTFSM_MASK)
/*! @} */

/*! @name GATE48 - Gate */
/*! @{ */

#define SEMA42_GATE48_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE48_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE48_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE48_GTFSM_SHIFT)) & SEMA42_GATE48_GTFSM_MASK)
/*! @} */

/*! @name GATE55 - Gate */
/*! @{ */

#define SEMA42_GATE55_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE55_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE55_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE55_GTFSM_SHIFT)) & SEMA42_GATE55_GTFSM_MASK)
/*! @} */

/*! @name GATE54 - Gate */
/*! @{ */

#define SEMA42_GATE54_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE54_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE54_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE54_GTFSM_SHIFT)) & SEMA42_GATE54_GTFSM_MASK)
/*! @} */

/*! @name GATE53 - Gate */
/*! @{ */

#define SEMA42_GATE53_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE53_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE53_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE53_GTFSM_SHIFT)) & SEMA42_GATE53_GTFSM_MASK)
/*! @} */

/*! @name GATE52 - Gate */
/*! @{ */

#define SEMA42_GATE52_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE52_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE52_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE52_GTFSM_SHIFT)) & SEMA42_GATE52_GTFSM_MASK)
/*! @} */

/*! @name GATE59 - Gate */
/*! @{ */

#define SEMA42_GATE59_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE59_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE59_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE59_GTFSM_SHIFT)) & SEMA42_GATE59_GTFSM_MASK)
/*! @} */

/*! @name GATE58 - Gate */
/*! @{ */

#define SEMA42_GATE58_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE58_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE58_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE58_GTFSM_SHIFT)) & SEMA42_GATE58_GTFSM_MASK)
/*! @} */

/*! @name GATE57 - Gate */
/*! @{ */

#define SEMA42_GATE57_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE57_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE57_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE57_GTFSM_SHIFT)) & SEMA42_GATE57_GTFSM_MASK)
/*! @} */

/*! @name GATE56 - Gate */
/*! @{ */

#define SEMA42_GATE56_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE56_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE56_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE56_GTFSM_SHIFT)) & SEMA42_GATE56_GTFSM_MASK)
/*! @} */

/*! @name GATE63 - Gate */
/*! @{ */

#define SEMA42_GATE63_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE63_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE63_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE63_GTFSM_SHIFT)) & SEMA42_GATE63_GTFSM_MASK)
/*! @} */

/*! @name GATE62 - Gate */
/*! @{ */

#define SEMA42_GATE62_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE62_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE62_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE62_GTFSM_SHIFT)) & SEMA42_GATE62_GTFSM_MASK)
/*! @} */

/*! @name GATE61 - Gate */
/*! @{ */

#define SEMA42_GATE61_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE61_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE61_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE61_GTFSM_SHIFT)) & SEMA42_GATE61_GTFSM_MASK)
/*! @} */

/*! @name GATE60 - Gate */
/*! @{ */

#define SEMA42_GATE60_GTFSM_MASK                 (0xFU)
#define SEMA42_GATE60_GTFSM_SHIFT                (0U)
/*! GTFSM - Gate Finite State Machine
 *  0b0000..The gate is unlocked (free).
 *  0b0001..Domain 0 locked the gate.
 *  0b0010..Domain 1 locked the gate.
 *  0b0011..Domain 2 locked the gate.
 *  0b0100..Domain 3 locked the gate.
 *  0b0101..Domain 4 locked the gate.
 *  0b0110..Domain 5 locked the gate.
 *  0b0111..Domain 6 locked the gate.
 *  0b1000..Domain 7 locked the gate.
 *  0b1001..Domain 8 locked the gate.
 *  0b1010..Domain 9 locked the gate.
 *  0b1011..Domain 10 locked the gate.
 *  0b1100..Domain 11 locked the gate.
 *  0b1101..Domain 12 locked the gate.
 *  0b1110..Domain 13 locked the gate.
 *  0b1111..Domain 14 locked the gate.
 */
#define SEMA42_GATE60_GTFSM(x)                   (((uint8_t)(((uint8_t)(x)) << SEMA42_GATE60_GTFSM_SHIFT)) & SEMA42_GATE60_GTFSM_MASK)
/*! @} */

/*! @name RSTGT_R - Reset Gate Read */
/*! @{ */

#define SEMA42_RSTGT_R_RSTGTN_MASK               (0xFFU)
#define SEMA42_RSTGT_R_RSTGTN_SHIFT              (0U)
/*! RSTGTN - Reset Gate Number */
#define SEMA42_RSTGT_R_RSTGTN(x)                 (((uint16_t)(((uint16_t)(x)) << SEMA42_RSTGT_R_RSTGTN_SHIFT)) & SEMA42_RSTGT_R_RSTGTN_MASK)

#define SEMA42_RSTGT_R_RSTGMS_MASK               (0xF00U)
#define SEMA42_RSTGT_R_RSTGMS_SHIFT              (8U)
/*! RSTGMS - Reset Gate Domain */
#define SEMA42_RSTGT_R_RSTGMS(x)                 (((uint16_t)(((uint16_t)(x)) << SEMA42_RSTGT_R_RSTGMS_SHIFT)) & SEMA42_RSTGT_R_RSTGMS_MASK)

#define SEMA42_RSTGT_R_RSTGSM_MASK               (0x3000U)
#define SEMA42_RSTGT_R_RSTGSM_SHIFT              (12U)
/*! RSTGSM - Reset Gate Finite State Machine
 *  0b00..Idle, waiting for the first data pattern write.
 *  0b01..Waiting for the second data pattern write
 *  0b10..The 2-write sequence has completed. Generate the specified gate reset(s). After the reset is performed,
 *        this machine returns to the idle (waiting for first data pattern write) state.
 *  0b11..This state encoding is never used and therefore reserved.
 */
#define SEMA42_RSTGT_R_RSTGSM(x)                 (((uint16_t)(((uint16_t)(x)) << SEMA42_RSTGT_R_RSTGSM_SHIFT)) & SEMA42_RSTGT_R_RSTGSM_MASK)
/*! @} */

/*! @name RSTGT_W - Reset Gate Write */
/*! @{ */

#define SEMA42_RSTGT_W_RSTGTN_MASK               (0xFFU)
#define SEMA42_RSTGT_W_RSTGTN_SHIFT              (0U)
/*! RSTGTN - Reset Gate Number */
#define SEMA42_RSTGT_W_RSTGTN(x)                 (((uint16_t)(((uint16_t)(x)) << SEMA42_RSTGT_W_RSTGTN_SHIFT)) & SEMA42_RSTGT_W_RSTGTN_MASK)

#define SEMA42_RSTGT_W_RSTGDP_MASK               (0xFF00U)
#define SEMA42_RSTGT_W_RSTGDP_SHIFT              (8U)
/*! RSTGDP - Reset Gate Data Pattern */
#define SEMA42_RSTGT_W_RSTGDP(x)                 (((uint16_t)(((uint16_t)(x)) << SEMA42_RSTGT_W_RSTGDP_SHIFT)) & SEMA42_RSTGT_W_RSTGDP_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SEMA42_Register_Masks */


/* SEMA42 - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SEMA1 base address */
  #define SEMA1_BASE                               (0x54260000u)
  /** Peripheral SEMA1 base address */
  #define SEMA1_BASE_NS                            (0x44260000u)
  /** Peripheral SEMA1 base pointer */
  #define SEMA1                                    ((SEMA42_Type *)SEMA1_BASE)
  /** Peripheral SEMA1 base pointer */
  #define SEMA1_NS                                 ((SEMA42_Type *)SEMA1_BASE_NS)
  /** Peripheral SEMA2 base address */
  #define SEMA2_BASE                               (0x52450000u)
  /** Peripheral SEMA2 base address */
  #define SEMA2_BASE_NS                            (0x42450000u)
  /** Peripheral SEMA2 base pointer */
  #define SEMA2                                    ((SEMA42_Type *)SEMA2_BASE)
  /** Peripheral SEMA2 base pointer */
  #define SEMA2_NS                                 ((SEMA42_Type *)SEMA2_BASE_NS)
  /** Array initializer of SEMA42 peripheral base addresses */
  #define SEMA42_BASE_ADDRS                        { SEMA1_BASE, SEMA2_BASE }
  /** Array initializer of SEMA42 peripheral base pointers */
  #define SEMA42_BASE_PTRS                         { SEMA1, SEMA2 }
  /** Array initializer of SEMA42 peripheral base addresses */
  #define SEMA42_BASE_ADDRS_NS                     { SEMA1_BASE_NS, SEMA2_BASE_NS }
  /** Array initializer of SEMA42 peripheral base pointers */
  #define SEMA42_BASE_PTRS_NS                      { SEMA1_NS, SEMA2_NS }
#else
  /** Peripheral SEMA1 base address */
  #define SEMA1_BASE                               (0x44260000u)
  /** Peripheral SEMA1 base pointer */
  #define SEMA1                                    ((SEMA42_Type *)SEMA1_BASE)
  /** Peripheral SEMA2 base address */
  #define SEMA2_BASE                               (0x42450000u)
  /** Peripheral SEMA2 base pointer */
  #define SEMA2                                    ((SEMA42_Type *)SEMA2_BASE)
  /** Array initializer of SEMA42 peripheral base addresses */
  #define SEMA42_BASE_ADDRS                        { SEMA1_BASE, SEMA2_BASE }
  /** Array initializer of SEMA42 peripheral base pointers */
  #define SEMA42_BASE_PTRS                         { SEMA1, SEMA2 }
#endif

/*!
 * @}
 */ /* end of group SEMA42_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SPDIF Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SPDIF_Peripheral_Access_Layer SPDIF Peripheral Access Layer
 * @{
 */

/** SPDIF - Register Layout Typedef */
typedef struct {
  __IO uint32_t SCR;                               /**< SPDIF Configuration Register, offset: 0x0 */
  __IO uint32_t SRCD;                              /**< CDText Control Register, offset: 0x4 */
  __IO uint32_t SRPC;                              /**< PhaseConfig Register, offset: 0x8 */
  __IO uint32_t SIE;                               /**< InterruptEn Register, offset: 0xC */
  __IO uint32_t SIS;                               /**< InterruptStat Register, offset: 0x10 */
  __I  uint32_t SRL;                               /**< SPDIFRxLeft Register, offset: 0x14 */
  __I  uint32_t SRR;                               /**< SPDIFRxRight Register, offset: 0x18 */
  __I  uint32_t SRCSH;                             /**< SPDIFRxCChannel_h Register, offset: 0x1C */
  __I  uint32_t SRCSL;                             /**< SPDIFRxCChannel_l Register, offset: 0x20 */
  __I  uint32_t SRU;                               /**< UchannelRx Register, offset: 0x24 */
  __I  uint32_t SRQ;                               /**< QchannelRx Register, offset: 0x28 */
  __O  uint32_t STL;                               /**< SPDIFTxLeft Register, offset: 0x2C */
  __O  uint32_t STR;                               /**< SPDIFTxRight Register, offset: 0x30 */
  __IO uint32_t STCSCH;                            /**< SPDIFTxCChannelCons_h Register, offset: 0x34 */
  __IO uint32_t STCSCL;                            /**< SPDIFTxCChannelCons_l Register, offset: 0x38 */
       uint8_t RESERVED_0[8];
  __I  uint32_t SRFM;                              /**< FreqMeas Register, offset: 0x44 */
       uint8_t RESERVED_1[8];
  __IO uint32_t STC;                               /**< SPDIFTxClk Register, offset: 0x50 */
       uint8_t RESERVED_2[12];
  __I  uint32_t SPDIFRXCCHANNEL_ADDR_31_0;         /**< SPDIF receive C channel register, bits 31-0, offset: 0x60 */
  __I  uint32_t SPDIFRXCCHANNEL_ADDR_63_32;        /**< SPDIF receive C channel register, bits 63-32, offset: 0x64 */
  __I  uint32_t SPDIFRXCCHANNEL_ADDR_95_64;        /**< SPDIF receive C channel register, bits 95-64, offset: 0x68 */
  __I  uint32_t SPDIFRXCCHANNEL_ADDR_127_96;       /**< SPDIF receive C channel register, bits 127-96, offset: 0x6C */
  __I  uint32_t SPDIFRXCCHANNEL_ADDR_159_128;      /**< SPDIF receive C channel register, bits 159-128, offset: 0x70 */
  __I  uint32_t SPDIFRXCCHANNEL_ADDR_191_160;      /**< SPDIF receive C channel register, bits 191-160, offset: 0x74 */
  __IO uint32_t SPDIFTXCCHANNEL_ADDR_31_0;         /**< SPDIF transmit C channel register, bits 31-0, offset: 0x78 */
  __IO uint32_t SPDIFTXCCHANNEL_ADDR_63_32;        /**< SPDIF transmit C channel register, bits 63-32, offset: 0x7C */
  __IO uint32_t SPDIFTXCCHANNEL_ADDR_95_64;        /**< SPDIF transmit C channel register, bits 95-64, offset: 0x80 */
  __IO uint32_t SPDIFTXCCHANNEL_ADDR_127_96;       /**< SPDIF transmit C channel register, bits 127-96, offset: 0x84 */
  __IO uint32_t SPDIFTXCCHANNEL_ADDR_159_128;      /**< SPDIF transmit C channel register, bits 159-128, offset: 0x88 */
  __IO uint32_t SPDIFTXCCHANNEL_ADDR_191_160;      /**< SPDIF transmit C channel register, bits 191-160, offset: 0x8C */
} SPDIF_Type;

/* ----------------------------------------------------------------------------
   -- SPDIF Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SPDIF_Register_Masks SPDIF Register Masks
 * @{
 */

/*! @name SCR - SPDIF Configuration Register */
/*! @{ */

#define SPDIF_SCR_USRC_SEL_MASK                  (0x3U)
#define SPDIF_SCR_USRC_SEL_SHIFT                 (0U)
/*! USrc_Sel - USrc_Sel
 *  0b00..No embedded U channel
 *  0b01..U channel from SPDIF receive block (CD mode)
 *  0b10..Reserved
 *  0b11..U channel from on chip transmitter
 */
#define SPDIF_SCR_USRC_SEL(x)                    (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_USRC_SEL_SHIFT)) & SPDIF_SCR_USRC_SEL_MASK)

#define SPDIF_SCR_TXSEL_MASK                     (0x1CU)
#define SPDIF_SCR_TXSEL_SHIFT                    (2U)
/*! TxSel - TxSel
 *  0b000..Off and output 0
 *  0b001..Feed-through SPDIF_IN
 *  0b010-0b100..Reserved
 *  0b101..Tx Normal operation - From SPDIF Tx Block
 *  0b110, 0b111..Reserved
 */
#define SPDIF_SCR_TXSEL(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_TXSEL_SHIFT)) & SPDIF_SCR_TXSEL_MASK)

#define SPDIF_SCR_VALCTRL_MASK                   (0x20U)
#define SPDIF_SCR_VALCTRL_SHIFT                  (5U)
/*! ValCtrl - ValCtrl
 *  0b0..Outgoing Validity always set
 *  0b1..Outgoing Validity always clear
 */
#define SPDIF_SCR_VALCTRL(x)                     (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_VALCTRL_SHIFT)) & SPDIF_SCR_VALCTRL_MASK)

#define SPDIF_SCR_INPUTSRCSEL_MASK               (0xC0U)
#define SPDIF_SCR_INPUTSRCSEL_SHIFT              (6U)
/*! InputSrcSel - InputSrcSel
 *  0b00..SPDIF_IN
 *  0b01-0b11..None
 */
#define SPDIF_SCR_INPUTSRCSEL(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_INPUTSRCSEL_SHIFT)) & SPDIF_SCR_INPUTSRCSEL_MASK)

#define SPDIF_SCR_DMA_TX_EN_MASK                 (0x100U)
#define SPDIF_SCR_DMA_TX_EN_SHIFT                (8U)
/*! DMA_TX_En - DMA_TX_En */
#define SPDIF_SCR_DMA_TX_EN(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_DMA_TX_EN_SHIFT)) & SPDIF_SCR_DMA_TX_EN_MASK)

#define SPDIF_SCR_DMA_RX_EN_MASK                 (0x200U)
#define SPDIF_SCR_DMA_RX_EN_SHIFT                (9U)
/*! DMA_Rx_En - DMA_Rx_En */
#define SPDIF_SCR_DMA_RX_EN(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_DMA_RX_EN_SHIFT)) & SPDIF_SCR_DMA_RX_EN_MASK)

#define SPDIF_SCR_TXFIFO_CTRL_MASK               (0xC00U)
#define SPDIF_SCR_TXFIFO_CTRL_SHIFT              (10U)
/*! TxFIFO_Ctrl - TxFIFO_Ctrl
 *  0b00..Send out digital zero on SPDIF Tx
 *  0b01..Tx Normal operation
 *  0b10..Reset to 1 sample remaining
 *  0b11..Reserved
 */
#define SPDIF_SCR_TXFIFO_CTRL(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_TXFIFO_CTRL_SHIFT)) & SPDIF_SCR_TXFIFO_CTRL_MASK)

#define SPDIF_SCR_SOFT_RESET_MASK                (0x1000U)
#define SPDIF_SCR_SOFT_RESET_SHIFT               (12U)
/*! soft_reset - soft_reset */
#define SPDIF_SCR_SOFT_RESET(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_SOFT_RESET_SHIFT)) & SPDIF_SCR_SOFT_RESET_MASK)

#define SPDIF_SCR_LOW_POWER_MASK                 (0x2000U)
#define SPDIF_SCR_LOW_POWER_SHIFT                (13U)
/*! LOW_POWER - LOW_POWER */
#define SPDIF_SCR_LOW_POWER(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_LOW_POWER_SHIFT)) & SPDIF_SCR_LOW_POWER_MASK)

#define SPDIF_SCR_TXFIFOEMPTY_SEL_MASK           (0x18000U)
#define SPDIF_SCR_TXFIFOEMPTY_SEL_SHIFT          (15U)
/*! TxFIFOEmpty_Sel - TxFIFOEmpty_Sel
 *  0b00..Empty interrupt if 0 sample in Tx left and right FIFOs
 *  0b01..Empty interrupt if at most 4 sample in Tx left and right FIFOs
 *  0b10..Empty interrupt if at most 8 sample in Tx left and right FIFOs
 *  0b11..Empty interrupt if at most 12 sample in Tx left and right FIFOs
 */
#define SPDIF_SCR_TXFIFOEMPTY_SEL(x)             (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_TXFIFOEMPTY_SEL_SHIFT)) & SPDIF_SCR_TXFIFOEMPTY_SEL_MASK)

#define SPDIF_SCR_TXAUTOSYNC_MASK                (0x20000U)
#define SPDIF_SCR_TXAUTOSYNC_SHIFT               (17U)
/*! TxAutoSync - TxAutoSync
 *  0b0..Tx FIFO auto sync off
 *  0b1..Tx FIFO auto sync on
 */
#define SPDIF_SCR_TXAUTOSYNC(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_TXAUTOSYNC_SHIFT)) & SPDIF_SCR_TXAUTOSYNC_MASK)

#define SPDIF_SCR_RXAUTOSYNC_MASK                (0x40000U)
#define SPDIF_SCR_RXAUTOSYNC_SHIFT               (18U)
/*! RxAutoSync - RxAutoSync
 *  0b0..Rx FIFO auto sync off
 *  0b1..RxFIFO auto sync on
 */
#define SPDIF_SCR_RXAUTOSYNC(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_RXAUTOSYNC_SHIFT)) & SPDIF_SCR_RXAUTOSYNC_MASK)

#define SPDIF_SCR_RXFIFOFULL_SEL_MASK            (0x180000U)
#define SPDIF_SCR_RXFIFOFULL_SEL_SHIFT           (19U)
/*! RxFIFOFull_Sel - RxFIFOFull_Sel
 *  0b00..Full interrupt if at least 1 sample in Rx left and right FIFOs
 *  0b01..Full interrupt if at least 4 sample in Rx left and right FIFOs
 *  0b10..Full interrupt if at least 8 sample in Rx left and right FIFOs
 *  0b11..Full interrupt if at least 16 sample in Rx left and right FIFO
 */
#define SPDIF_SCR_RXFIFOFULL_SEL(x)              (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_RXFIFOFULL_SEL_SHIFT)) & SPDIF_SCR_RXFIFOFULL_SEL_MASK)

#define SPDIF_SCR_RXFIFO_RST_MASK                (0x200000U)
#define SPDIF_SCR_RXFIFO_RST_SHIFT               (21U)
/*! RxFIFO_Rst - RxFIFO_Rst
 *  0b0..Normal operation
 *  0b1..Reset register to 1 sample remaining
 */
#define SPDIF_SCR_RXFIFO_RST(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_RXFIFO_RST_SHIFT)) & SPDIF_SCR_RXFIFO_RST_MASK)

#define SPDIF_SCR_RXFIFO_OFF_ON_MASK             (0x400000U)
#define SPDIF_SCR_RXFIFO_OFF_ON_SHIFT            (22U)
/*! RxFIFO_Off_On - RxFIFO_Off_On
 *  0b0..SPDIF Rx FIFO is on
 *  0b1..SPDIF Rx FIFO is off. Does not accept data from interface
 */
#define SPDIF_SCR_RXFIFO_OFF_ON(x)               (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_RXFIFO_OFF_ON_SHIFT)) & SPDIF_SCR_RXFIFO_OFF_ON_MASK)

#define SPDIF_SCR_RXFIFO_CTRL_MASK               (0x800000U)
#define SPDIF_SCR_RXFIFO_CTRL_SHIFT              (23U)
/*! RxFIFO_Ctrl - RxFIFO_Ctrl
 *  0b0..Normal operation
 *  0b1..Always read zero from Rx data register
 */
#define SPDIF_SCR_RXFIFO_CTRL(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_RXFIFO_CTRL_SHIFT)) & SPDIF_SCR_RXFIFO_CTRL_MASK)

#define SPDIF_SCR_TXCCHANNEL_192B_EN_MASK        (0x1000000U)
#define SPDIF_SCR_TXCCHANNEL_192B_EN_SHIFT       (24U)
/*! TXCChannel_192b_en - TXCChannel_192b_en
 *  0b0..SPDIF transmits 48 bits of C in audio stream. Other C bits in 49 to 192 frames are 0
 *  0b1..SPDIF transmits 192 bits of C in audio stream
 */
#define SPDIF_SCR_TXCCHANNEL_192B_EN(x)          (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_TXCCHANNEL_192B_EN_SHIFT)) & SPDIF_SCR_TXCCHANNEL_192B_EN_MASK)

#define SPDIF_SCR_RXCCHANNEL_192B_EN_MASK        (0x2000000U)
#define SPDIF_SCR_RXCCHANNEL_192B_EN_SHIFT       (25U)
/*! RXCChannel_192b_en - RXCChannel_192b_en
 *  0b0..SPDIF receives only 48 bits of 192 C bits from input audio stream
 *  0b1..SPDIF receives 192 bits of C in audio stream
 */
#define SPDIF_SCR_RXCCHANNEL_192B_EN(x)          (((uint32_t)(((uint32_t)(x)) << SPDIF_SCR_RXCCHANNEL_192B_EN_SHIFT)) & SPDIF_SCR_RXCCHANNEL_192B_EN_MASK)
/*! @} */

/*! @name SRCD - CDText Control Register */
/*! @{ */

#define SPDIF_SRCD_USYNCMODE_MASK                (0x2U)
#define SPDIF_SRCD_USYNCMODE_SHIFT               (1U)
/*! USyncMode - USyncMode
 *  0b0..Non-CD data
 *  0b1..CD user channel subcode
 */
#define SPDIF_SRCD_USYNCMODE(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SRCD_USYNCMODE_SHIFT)) & SPDIF_SRCD_USYNCMODE_MASK)
/*! @} */

/*! @name SRPC - PhaseConfig Register */
/*! @{ */

#define SPDIF_SRPC_GAINSEL_MASK                  (0x38U)
#define SPDIF_SRPC_GAINSEL_SHIFT                 (3U)
/*! GainSel - GainSel
 *  0b000..24*(2**10)
 *  0b001..16*(2**10)
 *  0b010..12*(2**10)
 *  0b011..8*(2**10)
 *  0b100..6*(2**10)
 *  0b101..4*(2**10)
 *  0b110..3*(2**10)
 *  0b111..Reserved
 */
#define SPDIF_SRPC_GAINSEL(x)                    (((uint32_t)(((uint32_t)(x)) << SPDIF_SRPC_GAINSEL_SHIFT)) & SPDIF_SRPC_GAINSEL_MASK)

#define SPDIF_SRPC_LOCK_MASK                     (0x40U)
#define SPDIF_SRPC_LOCK_SHIFT                    (6U)
/*! LOCK - LOCK */
#define SPDIF_SRPC_LOCK(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SRPC_LOCK_SHIFT)) & SPDIF_SRPC_LOCK_MASK)

#define SPDIF_SRPC_CLKSRC_SEL_MASK               (0x780U)
#define SPDIF_SRPC_CLKSRC_SEL_SHIFT              (7U)
/*! ClkSrc_Sel - ClkSrc_Sel
 *  0b0000..if (DPLL Locked) SPDIF_RxClk else REF_CLK_32K (XTALOSC)
 *  0b0001..if (DPLL Locked) SPDIF_RxClk else tx_clk (SPDIF0_CLK_ROOT)
 *  0b0011..if (DPLL Locked) SPDIF_RxClk else SPDIF_EXT_CLK
 *  0b0101..REF_CLK_32K (XTALOSC)
 *  0b0110..tx_clk (SPDIF0_CLK_ROOT)
 *  0b1000..SPDIF_EXT_CLK
 */
#define SPDIF_SRPC_CLKSRC_SEL(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_SRPC_CLKSRC_SEL_SHIFT)) & SPDIF_SRPC_CLKSRC_SEL_MASK)
/*! @} */

/*! @name SIE - InterruptEn Register */
/*! @{ */

#define SPDIF_SIE_RXFIFOFUL_MASK                 (0x1U)
#define SPDIF_SIE_RXFIFOFUL_SHIFT                (0U)
/*! RxFIFOFul - RxFIFOFul */
#define SPDIF_SIE_RXFIFOFUL(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_RXFIFOFUL_SHIFT)) & SPDIF_SIE_RXFIFOFUL_MASK)

#define SPDIF_SIE_TXEM_MASK                      (0x2U)
#define SPDIF_SIE_TXEM_SHIFT                     (1U)
/*! TxEm - TxEm */
#define SPDIF_SIE_TXEM(x)                        (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_TXEM_SHIFT)) & SPDIF_SIE_TXEM_MASK)

#define SPDIF_SIE_LOCKLOSS_MASK                  (0x4U)
#define SPDIF_SIE_LOCKLOSS_SHIFT                 (2U)
/*! LockLoss - LockLoss */
#define SPDIF_SIE_LOCKLOSS(x)                    (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_LOCKLOSS_SHIFT)) & SPDIF_SIE_LOCKLOSS_MASK)

#define SPDIF_SIE_RXFIFORESYN_MASK               (0x8U)
#define SPDIF_SIE_RXFIFORESYN_SHIFT              (3U)
/*! RxFIFOResyn - RxFIFOResyn */
#define SPDIF_SIE_RXFIFORESYN(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_RXFIFORESYN_SHIFT)) & SPDIF_SIE_RXFIFORESYN_MASK)

#define SPDIF_SIE_RXFIFOUNOV_MASK                (0x10U)
#define SPDIF_SIE_RXFIFOUNOV_SHIFT               (4U)
/*! RxFIFOUnOv - RxFIFOUnOv */
#define SPDIF_SIE_RXFIFOUNOV(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_RXFIFOUNOV_SHIFT)) & SPDIF_SIE_RXFIFOUNOV_MASK)

#define SPDIF_SIE_UQERR_MASK                     (0x20U)
#define SPDIF_SIE_UQERR_SHIFT                    (5U)
/*! UQErr - UQErr */
#define SPDIF_SIE_UQERR(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_UQERR_SHIFT)) & SPDIF_SIE_UQERR_MASK)

#define SPDIF_SIE_UQSYNC_MASK                    (0x40U)
#define SPDIF_SIE_UQSYNC_SHIFT                   (6U)
/*! UQSync - UQSync */
#define SPDIF_SIE_UQSYNC(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_UQSYNC_SHIFT)) & SPDIF_SIE_UQSYNC_MASK)

#define SPDIF_SIE_QRXOV_MASK                     (0x80U)
#define SPDIF_SIE_QRXOV_SHIFT                    (7U)
/*! QRxOv - QRxOv */
#define SPDIF_SIE_QRXOV(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_QRXOV_SHIFT)) & SPDIF_SIE_QRXOV_MASK)

#define SPDIF_SIE_QRXFUL_MASK                    (0x100U)
#define SPDIF_SIE_QRXFUL_SHIFT                   (8U)
/*! QRxFul - QRxFul */
#define SPDIF_SIE_QRXFUL(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_QRXFUL_SHIFT)) & SPDIF_SIE_QRXFUL_MASK)

#define SPDIF_SIE_URXOV_MASK                     (0x200U)
#define SPDIF_SIE_URXOV_SHIFT                    (9U)
/*! URxOv - URxOv */
#define SPDIF_SIE_URXOV(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_URXOV_SHIFT)) & SPDIF_SIE_URXOV_MASK)

#define SPDIF_SIE_URXFUL_MASK                    (0x400U)
#define SPDIF_SIE_URXFUL_SHIFT                   (10U)
/*! URxFul - URxFul */
#define SPDIF_SIE_URXFUL(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_URXFUL_SHIFT)) & SPDIF_SIE_URXFUL_MASK)

#define SPDIF_SIE_BITERR_MASK                    (0x4000U)
#define SPDIF_SIE_BITERR_SHIFT                   (14U)
/*! BitErr - BitErr */
#define SPDIF_SIE_BITERR(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_BITERR_SHIFT)) & SPDIF_SIE_BITERR_MASK)

#define SPDIF_SIE_SYMERR_MASK                    (0x8000U)
#define SPDIF_SIE_SYMERR_SHIFT                   (15U)
/*! SymErr - SymErr */
#define SPDIF_SIE_SYMERR(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_SYMERR_SHIFT)) & SPDIF_SIE_SYMERR_MASK)

#define SPDIF_SIE_VALNOGOOD_MASK                 (0x10000U)
#define SPDIF_SIE_VALNOGOOD_SHIFT                (16U)
/*! ValNoGood - ValNoGood */
#define SPDIF_SIE_VALNOGOOD(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_VALNOGOOD_SHIFT)) & SPDIF_SIE_VALNOGOOD_MASK)

#define SPDIF_SIE_CNEW_MASK                      (0x20000U)
#define SPDIF_SIE_CNEW_SHIFT                     (17U)
/*! CNew - CNew */
#define SPDIF_SIE_CNEW(x)                        (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_CNEW_SHIFT)) & SPDIF_SIE_CNEW_MASK)

#define SPDIF_SIE_TXRESYN_MASK                   (0x40000U)
#define SPDIF_SIE_TXRESYN_SHIFT                  (18U)
/*! TxResyn - TxResyn */
#define SPDIF_SIE_TXRESYN(x)                     (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_TXRESYN_SHIFT)) & SPDIF_SIE_TXRESYN_MASK)

#define SPDIF_SIE_TXUNOV_MASK                    (0x80000U)
#define SPDIF_SIE_TXUNOV_SHIFT                   (19U)
/*! TxUnOv - TxUnOv */
#define SPDIF_SIE_TXUNOV(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_TXUNOV_SHIFT)) & SPDIF_SIE_TXUNOV_MASK)

#define SPDIF_SIE_LOCK_MASK                      (0x100000U)
#define SPDIF_SIE_LOCK_SHIFT                     (20U)
/*! Lock - Lock */
#define SPDIF_SIE_LOCK(x)                        (((uint32_t)(((uint32_t)(x)) << SPDIF_SIE_LOCK_SHIFT)) & SPDIF_SIE_LOCK_MASK)
/*! @} */

/*! @name SIS - InterruptStat Register */
/*! @{ */

#define SPDIF_SIS_RXFIFOFUL_MASK                 (0x1U)
#define SPDIF_SIS_RXFIFOFUL_SHIFT                (0U)
/*! RxFIFOFul - RxFIFOFul */
#define SPDIF_SIS_RXFIFOFUL(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_RXFIFOFUL_SHIFT)) & SPDIF_SIS_RXFIFOFUL_MASK)

#define SPDIF_SIS_TXEM_MASK                      (0x2U)
#define SPDIF_SIS_TXEM_SHIFT                     (1U)
/*! TxEm - TxEm */
#define SPDIF_SIS_TXEM(x)                        (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_TXEM_SHIFT)) & SPDIF_SIS_TXEM_MASK)

#define SPDIF_SIS_LOCKLOSS_MASK                  (0x4U)
#define SPDIF_SIS_LOCKLOSS_SHIFT                 (2U)
/*! LockLoss - LockLoss */
#define SPDIF_SIS_LOCKLOSS(x)                    (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_LOCKLOSS_SHIFT)) & SPDIF_SIS_LOCKLOSS_MASK)

#define SPDIF_SIS_RXFIFORESYN_MASK               (0x8U)
#define SPDIF_SIS_RXFIFORESYN_SHIFT              (3U)
/*! RxFIFOResyn - RxFIFOResyn */
#define SPDIF_SIS_RXFIFORESYN(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_RXFIFORESYN_SHIFT)) & SPDIF_SIS_RXFIFORESYN_MASK)

#define SPDIF_SIS_RXFIFOUNOV_MASK                (0x10U)
#define SPDIF_SIS_RXFIFOUNOV_SHIFT               (4U)
/*! RxFIFOUnOv - RxFIFOUnOv */
#define SPDIF_SIS_RXFIFOUNOV(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_RXFIFOUNOV_SHIFT)) & SPDIF_SIS_RXFIFOUNOV_MASK)

#define SPDIF_SIS_UQERR_MASK                     (0x20U)
#define SPDIF_SIS_UQERR_SHIFT                    (5U)
/*! UQErr - UQErr */
#define SPDIF_SIS_UQERR(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_UQERR_SHIFT)) & SPDIF_SIS_UQERR_MASK)

#define SPDIF_SIS_UQSYNC_MASK                    (0x40U)
#define SPDIF_SIS_UQSYNC_SHIFT                   (6U)
/*! UQSync - UQSync */
#define SPDIF_SIS_UQSYNC(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_UQSYNC_SHIFT)) & SPDIF_SIS_UQSYNC_MASK)

#define SPDIF_SIS_QRXOV_MASK                     (0x80U)
#define SPDIF_SIS_QRXOV_SHIFT                    (7U)
/*! QRxOv - QRxOv */
#define SPDIF_SIS_QRXOV(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_QRXOV_SHIFT)) & SPDIF_SIS_QRXOV_MASK)

#define SPDIF_SIS_QRXFUL_MASK                    (0x100U)
#define SPDIF_SIS_QRXFUL_SHIFT                   (8U)
/*! QRxFul - QRxFul */
#define SPDIF_SIS_QRXFUL(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_QRXFUL_SHIFT)) & SPDIF_SIS_QRXFUL_MASK)

#define SPDIF_SIS_URXOV_MASK                     (0x200U)
#define SPDIF_SIS_URXOV_SHIFT                    (9U)
/*! URxOv - URxOv */
#define SPDIF_SIS_URXOV(x)                       (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_URXOV_SHIFT)) & SPDIF_SIS_URXOV_MASK)

#define SPDIF_SIS_URXFUL_MASK                    (0x400U)
#define SPDIF_SIS_URXFUL_SHIFT                   (10U)
/*! URxFul - URxFul */
#define SPDIF_SIS_URXFUL(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_URXFUL_SHIFT)) & SPDIF_SIS_URXFUL_MASK)

#define SPDIF_SIS_BITERR_MASK                    (0x4000U)
#define SPDIF_SIS_BITERR_SHIFT                   (14U)
/*! BitErr - BitErr */
#define SPDIF_SIS_BITERR(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_BITERR_SHIFT)) & SPDIF_SIS_BITERR_MASK)

#define SPDIF_SIS_SYMERR_MASK                    (0x8000U)
#define SPDIF_SIS_SYMERR_SHIFT                   (15U)
/*! SymErr - SymErr */
#define SPDIF_SIS_SYMERR(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_SYMERR_SHIFT)) & SPDIF_SIS_SYMERR_MASK)

#define SPDIF_SIS_VALNOGOOD_MASK                 (0x10000U)
#define SPDIF_SIS_VALNOGOOD_SHIFT                (16U)
/*! ValNoGood - ValNoGood */
#define SPDIF_SIS_VALNOGOOD(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_VALNOGOOD_SHIFT)) & SPDIF_SIS_VALNOGOOD_MASK)

#define SPDIF_SIS_CNEW_MASK                      (0x20000U)
#define SPDIF_SIS_CNEW_SHIFT                     (17U)
/*! CNew - CNew */
#define SPDIF_SIS_CNEW(x)                        (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_CNEW_SHIFT)) & SPDIF_SIS_CNEW_MASK)

#define SPDIF_SIS_TXRESYN_MASK                   (0x40000U)
#define SPDIF_SIS_TXRESYN_SHIFT                  (18U)
/*! TxResyn - TxResyn */
#define SPDIF_SIS_TXRESYN(x)                     (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_TXRESYN_SHIFT)) & SPDIF_SIS_TXRESYN_MASK)

#define SPDIF_SIS_TXUNOV_MASK                    (0x80000U)
#define SPDIF_SIS_TXUNOV_SHIFT                   (19U)
/*! TxUnOv - TxUnOv */
#define SPDIF_SIS_TXUNOV(x)                      (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_TXUNOV_SHIFT)) & SPDIF_SIS_TXUNOV_MASK)

#define SPDIF_SIS_LOCK_MASK                      (0x100000U)
#define SPDIF_SIS_LOCK_SHIFT                     (20U)
/*! Lock - Lock */
#define SPDIF_SIS_LOCK(x)                        (((uint32_t)(((uint32_t)(x)) << SPDIF_SIS_LOCK_SHIFT)) & SPDIF_SIS_LOCK_MASK)
/*! @} */

/*! @name SRL - SPDIFRxLeft Register */
/*! @{ */

#define SPDIF_SRL_RXDATALEFT_MASK                (0xFFFFFFU)
#define SPDIF_SRL_RXDATALEFT_SHIFT               (0U)
/*! RxDataLeft - RxDataLeft */
#define SPDIF_SRL_RXDATALEFT(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SRL_RXDATALEFT_SHIFT)) & SPDIF_SRL_RXDATALEFT_MASK)
/*! @} */

/*! @name SRR - SPDIFRxRight Register */
/*! @{ */

#define SPDIF_SRR_RXDATARIGHT_MASK               (0xFFFFFFU)
#define SPDIF_SRR_RXDATARIGHT_SHIFT              (0U)
/*! RxDataRight - RxDataRight */
#define SPDIF_SRR_RXDATARIGHT(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_SRR_RXDATARIGHT_SHIFT)) & SPDIF_SRR_RXDATARIGHT_MASK)
/*! @} */

/*! @name SRCSH - SPDIFRxCChannel_h Register */
/*! @{ */

#define SPDIF_SRCSH_RXCCHANNEL_H_MASK            (0xFFFFFFU)
#define SPDIF_SRCSH_RXCCHANNEL_H_SHIFT           (0U)
/*! RxCChannel_h - RxCChannel_h */
#define SPDIF_SRCSH_RXCCHANNEL_H(x)              (((uint32_t)(((uint32_t)(x)) << SPDIF_SRCSH_RXCCHANNEL_H_SHIFT)) & SPDIF_SRCSH_RXCCHANNEL_H_MASK)
/*! @} */

/*! @name SRCSL - SPDIFRxCChannel_l Register */
/*! @{ */

#define SPDIF_SRCSL_RXCCHANNEL_L_MASK            (0xFFFFFFU)
#define SPDIF_SRCSL_RXCCHANNEL_L_SHIFT           (0U)
/*! RxCChannel_l - RxCChannel_l */
#define SPDIF_SRCSL_RXCCHANNEL_L(x)              (((uint32_t)(((uint32_t)(x)) << SPDIF_SRCSL_RXCCHANNEL_L_SHIFT)) & SPDIF_SRCSL_RXCCHANNEL_L_MASK)
/*! @} */

/*! @name SRU - UchannelRx Register */
/*! @{ */

#define SPDIF_SRU_RXUCHANNEL_MASK                (0xFFFFFFU)
#define SPDIF_SRU_RXUCHANNEL_SHIFT               (0U)
/*! RxUChannel - RxUChannel */
#define SPDIF_SRU_RXUCHANNEL(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SRU_RXUCHANNEL_SHIFT)) & SPDIF_SRU_RXUCHANNEL_MASK)
/*! @} */

/*! @name SRQ - QchannelRx Register */
/*! @{ */

#define SPDIF_SRQ_RXQCHANNEL_MASK                (0xFFFFFFU)
#define SPDIF_SRQ_RXQCHANNEL_SHIFT               (0U)
/*! RxQChannel - RxQChannel */
#define SPDIF_SRQ_RXQCHANNEL(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_SRQ_RXQCHANNEL_SHIFT)) & SPDIF_SRQ_RXQCHANNEL_MASK)
/*! @} */

/*! @name STL - SPDIFTxLeft Register */
/*! @{ */

#define SPDIF_STL_TXDATALEFT_MASK                (0xFFFFFFU)
#define SPDIF_STL_TXDATALEFT_SHIFT               (0U)
/*! TxDataLeft - TxDataLeft */
#define SPDIF_STL_TXDATALEFT(x)                  (((uint32_t)(((uint32_t)(x)) << SPDIF_STL_TXDATALEFT_SHIFT)) & SPDIF_STL_TXDATALEFT_MASK)
/*! @} */

/*! @name STR - SPDIFTxRight Register */
/*! @{ */

#define SPDIF_STR_TXDATARIGHT_MASK               (0xFFFFFFU)
#define SPDIF_STR_TXDATARIGHT_SHIFT              (0U)
/*! TxDataRight - TxDataRight */
#define SPDIF_STR_TXDATARIGHT(x)                 (((uint32_t)(((uint32_t)(x)) << SPDIF_STR_TXDATARIGHT_SHIFT)) & SPDIF_STR_TXDATARIGHT_MASK)
/*! @} */

/*! @name STCSCH - SPDIFTxCChannelCons_h Register */
/*! @{ */

#define SPDIF_STCSCH_TXCCHANNELCONS_H_MASK       (0xFFFFFFU)
#define SPDIF_STCSCH_TXCCHANNELCONS_H_SHIFT      (0U)
/*! TxCChannelCons_h - TxCChannelCons_h */
#define SPDIF_STCSCH_TXCCHANNELCONS_H(x)         (((uint32_t)(((uint32_t)(x)) << SPDIF_STCSCH_TXCCHANNELCONS_H_SHIFT)) & SPDIF_STCSCH_TXCCHANNELCONS_H_MASK)
/*! @} */

/*! @name STCSCL - SPDIFTxCChannelCons_l Register */
/*! @{ */

#define SPDIF_STCSCL_TXCCHANNELCONS_L_MASK       (0xFFFFFFU)
#define SPDIF_STCSCL_TXCCHANNELCONS_L_SHIFT      (0U)
/*! TxCChannelCons_l - TxCChannelCons_l */
#define SPDIF_STCSCL_TXCCHANNELCONS_L(x)         (((uint32_t)(((uint32_t)(x)) << SPDIF_STCSCL_TXCCHANNELCONS_L_SHIFT)) & SPDIF_STCSCL_TXCCHANNELCONS_L_MASK)
/*! @} */

/*! @name SRFM - FreqMeas Register */
/*! @{ */

#define SPDIF_SRFM_FREQMEAS_MASK                 (0xFFFFFFU)
#define SPDIF_SRFM_FREQMEAS_SHIFT                (0U)
/*! FreqMeas - FreqMeas */
#define SPDIF_SRFM_FREQMEAS(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_SRFM_FREQMEAS_SHIFT)) & SPDIF_SRFM_FREQMEAS_MASK)
/*! @} */

/*! @name STC - SPDIFTxClk Register */
/*! @{ */

#define SPDIF_STC_TXCLK_DF_MASK                  (0x7FU)
#define SPDIF_STC_TXCLK_DF_SHIFT                 (0U)
/*! TxClk_DF - TxClk_DF
 *  0b0000000..divider factor is 1
 *  0b0000001..divider factor is 2
 *  0b1111111..divider factor is 128
 */
#define SPDIF_STC_TXCLK_DF(x)                    (((uint32_t)(((uint32_t)(x)) << SPDIF_STC_TXCLK_DF_SHIFT)) & SPDIF_STC_TXCLK_DF_MASK)

#define SPDIF_STC_TX_ALL_CLK_EN_MASK             (0x80U)
#define SPDIF_STC_TX_ALL_CLK_EN_SHIFT            (7U)
/*! tx_all_clk_en - tx_all_clk_en
 *  0b0..disable transfer clock.
 *  0b1..enable transfer clock.
 */
#define SPDIF_STC_TX_ALL_CLK_EN(x)               (((uint32_t)(((uint32_t)(x)) << SPDIF_STC_TX_ALL_CLK_EN_SHIFT)) & SPDIF_STC_TX_ALL_CLK_EN_MASK)

#define SPDIF_STC_TXCLK_SOURCE_MASK              (0x700U)
#define SPDIF_STC_TXCLK_SOURCE_SHIFT             (8U)
/*! TxClk_Source - TxClk_Source */
#define SPDIF_STC_TXCLK_SOURCE(x)                (((uint32_t)(((uint32_t)(x)) << SPDIF_STC_TXCLK_SOURCE_SHIFT)) & SPDIF_STC_TXCLK_SOURCE_MASK)

#define SPDIF_STC_SYSCLK_DF_MASK                 (0xFF800U)
#define SPDIF_STC_SYSCLK_DF_SHIFT                (11U)
/*! SYSCLK_DF - SYSCLK_DF
 *  0b000000000..no clock signal
 *  0b000000001..divider factor is 2
 *  0b111111111..divider factor is 512
 */
#define SPDIF_STC_SYSCLK_DF(x)                   (((uint32_t)(((uint32_t)(x)) << SPDIF_STC_SYSCLK_DF_SHIFT)) & SPDIF_STC_SYSCLK_DF_MASK)
/*! @} */

/*! @name SPDIFRXCCHANNEL_ADDR_31_0 - SPDIF receive C channel register, bits 31-0 */
/*! @{ */

#define SPDIF_SPDIFRXCCHANNEL_ADDR_31_0_RXCCHANNEL_ADDR_31_0_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFRXCCHANNEL_ADDR_31_0_RXCCHANNEL_ADDR_31_0_SHIFT (0U)
/*! RxCChannel_Addr_31_0 - RxCChannel_Addr_31_0 */
#define SPDIF_SPDIFRXCCHANNEL_ADDR_31_0_RXCCHANNEL_ADDR_31_0(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFRXCCHANNEL_ADDR_31_0_RXCCHANNEL_ADDR_31_0_SHIFT)) & SPDIF_SPDIFRXCCHANNEL_ADDR_31_0_RXCCHANNEL_ADDR_31_0_MASK)
/*! @} */

/*! @name SPDIFRXCCHANNEL_ADDR_63_32 - SPDIF receive C channel register, bits 63-32 */
/*! @{ */

#define SPDIF_SPDIFRXCCHANNEL_ADDR_63_32_RXCCHANNEL_ADDR_63_32_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFRXCCHANNEL_ADDR_63_32_RXCCHANNEL_ADDR_63_32_SHIFT (0U)
/*! RxCChannel_Addr_63_32 - RxCChannel_Addr_63_32 */
#define SPDIF_SPDIFRXCCHANNEL_ADDR_63_32_RXCCHANNEL_ADDR_63_32(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFRXCCHANNEL_ADDR_63_32_RXCCHANNEL_ADDR_63_32_SHIFT)) & SPDIF_SPDIFRXCCHANNEL_ADDR_63_32_RXCCHANNEL_ADDR_63_32_MASK)
/*! @} */

/*! @name SPDIFRXCCHANNEL_ADDR_95_64 - SPDIF receive C channel register, bits 95-64 */
/*! @{ */

#define SPDIF_SPDIFRXCCHANNEL_ADDR_95_64_RXCCHANNEL_ADDR_95_64_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFRXCCHANNEL_ADDR_95_64_RXCCHANNEL_ADDR_95_64_SHIFT (0U)
/*! RxCChannel_Addr_95_64 - RxCChannel_Addr_95_64 */
#define SPDIF_SPDIFRXCCHANNEL_ADDR_95_64_RXCCHANNEL_ADDR_95_64(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFRXCCHANNEL_ADDR_95_64_RXCCHANNEL_ADDR_95_64_SHIFT)) & SPDIF_SPDIFRXCCHANNEL_ADDR_95_64_RXCCHANNEL_ADDR_95_64_MASK)
/*! @} */

/*! @name SPDIFRXCCHANNEL_ADDR_127_96 - SPDIF receive C channel register, bits 127-96 */
/*! @{ */

#define SPDIF_SPDIFRXCCHANNEL_ADDR_127_96_RXCCHANNEL_ADDR_127_96_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFRXCCHANNEL_ADDR_127_96_RXCCHANNEL_ADDR_127_96_SHIFT (0U)
/*! RxCChannel_Addr_127_96 - RxCChannel_Addr_127_96 */
#define SPDIF_SPDIFRXCCHANNEL_ADDR_127_96_RXCCHANNEL_ADDR_127_96(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFRXCCHANNEL_ADDR_127_96_RXCCHANNEL_ADDR_127_96_SHIFT)) & SPDIF_SPDIFRXCCHANNEL_ADDR_127_96_RXCCHANNEL_ADDR_127_96_MASK)
/*! @} */

/*! @name SPDIFRXCCHANNEL_ADDR_159_128 - SPDIF receive C channel register, bits 159-128 */
/*! @{ */

#define SPDIF_SPDIFRXCCHANNEL_ADDR_159_128_RXCCHANNEL_ADDR_159_128_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFRXCCHANNEL_ADDR_159_128_RXCCHANNEL_ADDR_159_128_SHIFT (0U)
/*! RxCChannel_Addr_159_128 - RxCChannel_Addr_159_128 */
#define SPDIF_SPDIFRXCCHANNEL_ADDR_159_128_RXCCHANNEL_ADDR_159_128(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFRXCCHANNEL_ADDR_159_128_RXCCHANNEL_ADDR_159_128_SHIFT)) & SPDIF_SPDIFRXCCHANNEL_ADDR_159_128_RXCCHANNEL_ADDR_159_128_MASK)
/*! @} */

/*! @name SPDIFRXCCHANNEL_ADDR_191_160 - SPDIF receive C channel register, bits 191-160 */
/*! @{ */

#define SPDIF_SPDIFRXCCHANNEL_ADDR_191_160_RXCCHANNEL_ADDR_191_160_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFRXCCHANNEL_ADDR_191_160_RXCCHANNEL_ADDR_191_160_SHIFT (0U)
/*! RxCChannel_Addr_191_160 - RxCChannel_Addr_191_160 */
#define SPDIF_SPDIFRXCCHANNEL_ADDR_191_160_RXCCHANNEL_ADDR_191_160(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFRXCCHANNEL_ADDR_191_160_RXCCHANNEL_ADDR_191_160_SHIFT)) & SPDIF_SPDIFRXCCHANNEL_ADDR_191_160_RXCCHANNEL_ADDR_191_160_MASK)
/*! @} */

/*! @name SPDIFTXCCHANNEL_ADDR_31_0 - SPDIF transmit C channel register, bits 31-0 */
/*! @{ */

#define SPDIF_SPDIFTXCCHANNEL_ADDR_31_0_TXCCHANNEL_ADDR_31_0_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFTXCCHANNEL_ADDR_31_0_TXCCHANNEL_ADDR_31_0_SHIFT (0U)
/*! TxCChannel_Addr_31_0 - TxCChannel_Addr_31_0 */
#define SPDIF_SPDIFTXCCHANNEL_ADDR_31_0_TXCCHANNEL_ADDR_31_0(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFTXCCHANNEL_ADDR_31_0_TXCCHANNEL_ADDR_31_0_SHIFT)) & SPDIF_SPDIFTXCCHANNEL_ADDR_31_0_TXCCHANNEL_ADDR_31_0_MASK)
/*! @} */

/*! @name SPDIFTXCCHANNEL_ADDR_63_32 - SPDIF transmit C channel register, bits 63-32 */
/*! @{ */

#define SPDIF_SPDIFTXCCHANNEL_ADDR_63_32_TXCCHANNEL_ADDR_63_32_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFTXCCHANNEL_ADDR_63_32_TXCCHANNEL_ADDR_63_32_SHIFT (0U)
/*! TxCChannel_Addr_63_32 - TxCChannel_Addr_63_32 */
#define SPDIF_SPDIFTXCCHANNEL_ADDR_63_32_TXCCHANNEL_ADDR_63_32(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFTXCCHANNEL_ADDR_63_32_TXCCHANNEL_ADDR_63_32_SHIFT)) & SPDIF_SPDIFTXCCHANNEL_ADDR_63_32_TXCCHANNEL_ADDR_63_32_MASK)
/*! @} */

/*! @name SPDIFTXCCHANNEL_ADDR_95_64 - SPDIF transmit C channel register, bits 95-64 */
/*! @{ */

#define SPDIF_SPDIFTXCCHANNEL_ADDR_95_64_TXCCHANNEL_ADDR_95_64_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFTXCCHANNEL_ADDR_95_64_TXCCHANNEL_ADDR_95_64_SHIFT (0U)
/*! TxCChannel_Addr_95_64 - TxCChannel_Addr_95_64 */
#define SPDIF_SPDIFTXCCHANNEL_ADDR_95_64_TXCCHANNEL_ADDR_95_64(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFTXCCHANNEL_ADDR_95_64_TXCCHANNEL_ADDR_95_64_SHIFT)) & SPDIF_SPDIFTXCCHANNEL_ADDR_95_64_TXCCHANNEL_ADDR_95_64_MASK)
/*! @} */

/*! @name SPDIFTXCCHANNEL_ADDR_127_96 - SPDIF transmit C channel register, bits 127-96 */
/*! @{ */

#define SPDIF_SPDIFTXCCHANNEL_ADDR_127_96_TXCCHANNEL_ADDR_127_96_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFTXCCHANNEL_ADDR_127_96_TXCCHANNEL_ADDR_127_96_SHIFT (0U)
/*! TxCChannel_Addr_127_96 - TxCChannel_Addr_127_96 */
#define SPDIF_SPDIFTXCCHANNEL_ADDR_127_96_TXCCHANNEL_ADDR_127_96(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFTXCCHANNEL_ADDR_127_96_TXCCHANNEL_ADDR_127_96_SHIFT)) & SPDIF_SPDIFTXCCHANNEL_ADDR_127_96_TXCCHANNEL_ADDR_127_96_MASK)
/*! @} */

/*! @name SPDIFTXCCHANNEL_ADDR_159_128 - SPDIF transmit C channel register, bits 159-128 */
/*! @{ */

#define SPDIF_SPDIFTXCCHANNEL_ADDR_159_128_TXCCHANNEL_ADDR_159_128_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFTXCCHANNEL_ADDR_159_128_TXCCHANNEL_ADDR_159_128_SHIFT (0U)
/*! TxCChannel_Addr_159_128 - TxCChannel_Addr_159_128 */
#define SPDIF_SPDIFTXCCHANNEL_ADDR_159_128_TXCCHANNEL_ADDR_159_128(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFTXCCHANNEL_ADDR_159_128_TXCCHANNEL_ADDR_159_128_SHIFT)) & SPDIF_SPDIFTXCCHANNEL_ADDR_159_128_TXCCHANNEL_ADDR_159_128_MASK)
/*! @} */

/*! @name SPDIFTXCCHANNEL_ADDR_191_160 - SPDIF transmit C channel register, bits 191-160 */
/*! @{ */

#define SPDIF_SPDIFTXCCHANNEL_ADDR_191_160_TXCCHANNEL_ADDR_191_160_MASK (0xFFFFFFFFU)
#define SPDIF_SPDIFTXCCHANNEL_ADDR_191_160_TXCCHANNEL_ADDR_191_160_SHIFT (0U)
/*! TxCChannel_Addr_191_160 - TxCChannel_Addr_191_160 */
#define SPDIF_SPDIFTXCCHANNEL_ADDR_191_160_TXCCHANNEL_ADDR_191_160(x) (((uint32_t)(((uint32_t)(x)) << SPDIF_SPDIFTXCCHANNEL_ADDR_191_160_TXCCHANNEL_ADDR_191_160_SHIFT)) & SPDIF_SPDIFTXCCHANNEL_ADDR_191_160_TXCCHANNEL_ADDR_191_160_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SPDIF_Register_Masks */


/* SPDIF - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SPDIF base address */
  #define SPDIF_BASE                               (0x52BA0000u)
  /** Peripheral SPDIF base address */
  #define SPDIF_BASE_NS                            (0x42BA0000u)
  /** Peripheral SPDIF base pointer */
  #define SPDIF                                    ((SPDIF_Type *)SPDIF_BASE)
  /** Peripheral SPDIF base pointer */
  #define SPDIF_NS                                 ((SPDIF_Type *)SPDIF_BASE_NS)
  /** Array initializer of SPDIF peripheral base addresses */
  #define SPDIF_BASE_ADDRS                         { SPDIF_BASE }
  /** Array initializer of SPDIF peripheral base pointers */
  #define SPDIF_BASE_PTRS                          { SPDIF }
  /** Array initializer of SPDIF peripheral base addresses */
  #define SPDIF_BASE_ADDRS_NS                      { SPDIF_BASE_NS }
  /** Array initializer of SPDIF peripheral base pointers */
  #define SPDIF_BASE_PTRS_NS                       { SPDIF_NS }
#else
  /** Peripheral SPDIF base address */
  #define SPDIF_BASE                               (0x42BA0000u)
  /** Peripheral SPDIF base pointer */
  #define SPDIF                                    ((SPDIF_Type *)SPDIF_BASE)
  /** Array initializer of SPDIF peripheral base addresses */
  #define SPDIF_BASE_ADDRS                         { SPDIF_BASE }
  /** Array initializer of SPDIF peripheral base pointers */
  #define SPDIF_BASE_PTRS                          { SPDIF }
#endif
/** Interrupt vectors for the SPDIF peripheral type */
#define SPDIF_IRQS                               { SPDIF_IRQn }

/*!
 * @}
 */ /* end of group SPDIF_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SRC_GENERAL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_GENERAL_Peripheral_Access_Layer SRC_GENERAL Peripheral Access Layer
 * @{
 */

/** SRC_GENERAL - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t AUTHEN_CTRL;                       /**< Authentication Control, offset: 0x4 */
       uint8_t RESERVED_1[8];
  __IO uint32_t SCR;                               /**< SRC Control Register, offset: 0x10 */
  __IO uint32_t SRTMR;                             /**< SRC Reset Trigger Mode Register, offset: 0x14 */
  __IO uint32_t SRMASK;                            /**< SRC Reset Mask Register, offset: 0x18 */
       uint8_t RESERVED_2[36];
  __I  uint32_t SBMR1;                             /**< SRC Boot Mode Register 1, offset: 0x40 */
  __I  uint32_t SBMR2;                             /**< SRC Boot Mode Register 2, offset: 0x44 */
       uint8_t RESERVED_3[4];
  __IO uint32_t SRSR_BBSM;                         /**< SRC Reset Status Register backup in BBSM domain, offset: 0x4C */
  __IO uint32_t SRSR;                              /**< SRC Reset Status Register, offset: 0x50 */
  __IO uint32_t GPR[20];                           /**< SRC General Purpose Register, array offset: 0x54, array step: 0x4 */
} SRC_GENERAL_Type;

/* ----------------------------------------------------------------------------
   -- SRC_GENERAL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_GENERAL_Register_Masks SRC_GENERAL Register Masks
 * @{
 */

/*! @name AUTHEN_CTRL - Authentication Control */
/*! @{ */

#define SRC_GENERAL_AUTHEN_CTRL_LOCK_CFG_MASK    (0x80U)
#define SRC_GENERAL_AUTHEN_CTRL_LOCK_CFG_SHIFT   (7U)
/*! LOCK_CFG - Configuration lock
 *  0b0..General registers are not locked.
 *  0b1..LOCK_CFG and registers in the list are locked.
 */
#define SRC_GENERAL_AUTHEN_CTRL_LOCK_CFG(x)      (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_AUTHEN_CTRL_LOCK_CFG_SHIFT)) & SRC_GENERAL_AUTHEN_CTRL_LOCK_CFG_MASK)

#define SRC_GENERAL_AUTHEN_CTRL_TZ_USER_MASK     (0x100U)
#define SRC_GENERAL_AUTHEN_CTRL_TZ_USER_SHIFT    (8U)
/*! TZ_USER - Allow user mode write
 *  0b0..General registers can only be written in privilege mode.
 *  0b1..General registers can be written either in privilege mode or user mode.
 */
#define SRC_GENERAL_AUTHEN_CTRL_TZ_USER(x)       (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_AUTHEN_CTRL_TZ_USER_SHIFT)) & SRC_GENERAL_AUTHEN_CTRL_TZ_USER_MASK)

#define SRC_GENERAL_AUTHEN_CTRL_TZ_NS_MASK       (0x200U)
#define SRC_GENERAL_AUTHEN_CTRL_TZ_NS_SHIFT      (9U)
/*! TZ_NS - Allow non-secure mode access
 *  0b0..General registers can only be written in secure mode.
 *  0b1..General registers can be written either in secure mode or non-secure mode.
 */
#define SRC_GENERAL_AUTHEN_CTRL_TZ_NS(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_AUTHEN_CTRL_TZ_NS_SHIFT)) & SRC_GENERAL_AUTHEN_CTRL_TZ_NS_MASK)

#define SRC_GENERAL_AUTHEN_CTRL_LOCK_TZ_MASK     (0x800U)
#define SRC_GENERAL_AUTHEN_CTRL_LOCK_TZ_SHIFT    (11U)
/*! LOCK_TZ - Lock Trust Zone Non Secure(TZ_NS) and Trust Zone User(TZ_USER) bits
 *  0b0..TZ_NS and TZ_USER values can be changed.
 *  0b1..LOCK_TZ, TZ_NS and TZ_USER values cannot be changed.
 */
#define SRC_GENERAL_AUTHEN_CTRL_LOCK_TZ(x)       (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_AUTHEN_CTRL_LOCK_TZ_SHIFT)) & SRC_GENERAL_AUTHEN_CTRL_LOCK_TZ_MASK)

#define SRC_GENERAL_AUTHEN_CTRL_LOCK_LIST_MASK   (0x8000U)
#define SRC_GENERAL_AUTHEN_CTRL_LOCK_LIST_SHIFT  (15U)
/*! LOCK_LIST - White list lock
 *  0b0..WHITE_LIST value can be changed.
 *  0b1..LOCK_LIST and WHITE_LIST values cannot be changed.
 */
#define SRC_GENERAL_AUTHEN_CTRL_LOCK_LIST(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_AUTHEN_CTRL_LOCK_LIST_SHIFT)) & SRC_GENERAL_AUTHEN_CTRL_LOCK_LIST_MASK)

#define SRC_GENERAL_AUTHEN_CTRL_WHITE_LIST_MASK  (0xFFFF0000U)
#define SRC_GENERAL_AUTHEN_CTRL_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Domain ID white list
 *  0b0000000000000001..Core with domain ID=0 can write General registers.
 *  0b0000000000000010..Core with domain ID=1 can write General registers.
 *  0b0000000000000100..Core with domain ID=2 can write General registers.
 *  0b0000000000001000..Core with domain ID=3 can write General registers.
 *  0b0000000000010000..Core with domain ID=4 can write General registers.
 *  0b0000000000100000..Core with domain ID=5 can write General registers.
 *  0b0000000001000000..Core with domain ID=6 can write General registers.
 *  0b0000000010000000..Core with domain ID=7 can write General registers.
 *  0b0000000100000000..Core with domain ID=8 can write General registers.
 *  0b0000001000000000..Core with domain ID=9 can write General registers.
 *  0b0000010000000000..Core with domain ID=10 can write General registers.
 *  0b0000100000000000..Core with domain ID=11 can write General registers.
 *  0b0001000000000000..Core with domain ID=12 can write General registers.
 *  0b0010000000000000..Core with domain ID=13 can write General registers.
 *  0b0100000000000000..Core with domain ID=14 can write General registers.
 *  0b1000000000000000..Core with domain ID=15 can write General registers.
 */
#define SRC_GENERAL_AUTHEN_CTRL_WHITE_LIST(x)    (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_AUTHEN_CTRL_WHITE_LIST_SHIFT)) & SRC_GENERAL_AUTHEN_CTRL_WHITE_LIST_MASK)
/*! @} */

/*! @name SCR - SRC Control Register */
/*! @{ */

#define SRC_GENERAL_SCR_BT_RELEASE_M7_MASK       (0x1U)
#define SRC_GENERAL_SCR_BT_RELEASE_M7_SHIFT      (0U)
/*! BT_RELEASE_M7 - Boot release M7
 *  0b0..Holds M7 Core reset.
 *  0b1..Releases M7 Core reset and let it run. After this bit is set, it cannot be cleared by SW write.
 */
#define SRC_GENERAL_SCR_BT_RELEASE_M7(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SCR_BT_RELEASE_M7_SHIFT)) & SRC_GENERAL_SCR_BT_RELEASE_M7_MASK)
/*! @} */

/*! @name SRTMR - SRC Reset Trigger Mode Register */
/*! @{ */

#define SRC_GENERAL_SRTMR_WDOG1_TRIG_MODE_MASK   (0x1U)
#define SRC_GENERAL_SRTMR_WDOG1_TRIG_MODE_SHIFT  (0U)
/*! WDOG1_TRIG_MODE - Wdog1 reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_WDOG1_TRIG_MODE(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_WDOG1_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_WDOG1_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_WDOG2_TRIG_MODE_MASK   (0x2U)
#define SRC_GENERAL_SRTMR_WDOG2_TRIG_MODE_SHIFT  (1U)
/*! WDOG2_TRIG_MODE - Wdog2 reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_WDOG2_TRIG_MODE(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_WDOG2_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_WDOG2_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_WDOG3_TRIG_MODE_MASK   (0x4U)
#define SRC_GENERAL_SRTMR_WDOG3_TRIG_MODE_SHIFT  (2U)
/*! WDOG3_TRIG_MODE - Wdog3 reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_WDOG3_TRIG_MODE(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_WDOG3_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_WDOG3_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_WDOG4_TRIG_MODE_MASK   (0x8U)
#define SRC_GENERAL_SRTMR_WDOG4_TRIG_MODE_SHIFT  (3U)
/*! WDOG4_TRIG_MODE - Wdog4 reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_WDOG4_TRIG_MODE(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_WDOG4_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_WDOG4_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_WDOG5_TRIG_MODE_MASK   (0x10U)
#define SRC_GENERAL_SRTMR_WDOG5_TRIG_MODE_SHIFT  (4U)
/*! WDOG5_TRIG_MODE - Wdog5 reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_WDOG5_TRIG_MODE(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_WDOG5_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_WDOG5_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_TEMPSENSE_TRIG_MODE_MASK (0x20U)
#define SRC_GENERAL_SRTMR_TEMPSENSE_TRIG_MODE_SHIFT (5U)
/*! TEMPSENSE_TRIG_MODE - TempSense reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_TEMPSENSE_TRIG_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_TEMPSENSE_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_TEMPSENSE_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_EDGELOCK_TRIG_MODE_MASK (0x40U)
#define SRC_GENERAL_SRTMR_EDGELOCK_TRIG_MODE_SHIFT (6U)
/*! EDGELOCK_TRIG_MODE - Edgelock reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_EDGELOCK_TRIG_MODE(x)  (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_EDGELOCK_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_EDGELOCK_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_JTAGSW_TRIG_MODE_MASK  (0x80U)
#define SRC_GENERAL_SRTMR_JTAGSW_TRIG_MODE_SHIFT (7U)
/*! JTAGSW_TRIG_MODE - Jtagsw reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_JTAGSW_TRIG_MODE(x)    (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_JTAGSW_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_JTAGSW_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_CM33_RESET_TRIG_MODE_MASK (0x100U)
#define SRC_GENERAL_SRTMR_CM33_RESET_TRIG_MODE_SHIFT (8U)
/*! CM33_RESET_TRIG_MODE - CM33 reset trigger mode configuration, locked by LOCK_CFG field.
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_CM33_RESET_TRIG_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_CM33_RESET_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_CM33_RESET_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_CM33_LOCKUP_TRIG_MODE_MASK (0x200U)
#define SRC_GENERAL_SRTMR_CM33_LOCKUP_TRIG_MODE_SHIFT (9U)
/*! CM33_LOCKUP_TRIG_MODE - CM33 lockup trigger mode configuration, locked by LOCK_CFG field.
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_CM33_LOCKUP_TRIG_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_CM33_LOCKUP_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_CM33_LOCKUP_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_CM7_RESET_TRIG_MODE_MASK (0x400U)
#define SRC_GENERAL_SRTMR_CM7_RESET_TRIG_MODE_SHIFT (10U)
/*! CM7_RESET_TRIG_MODE - CM7 reset trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_CM7_RESET_TRIG_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_CM7_RESET_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_CM7_RESET_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_CM7_LOCKUP_TRIG_MODE_MASK (0x800U)
#define SRC_GENERAL_SRTMR_CM7_LOCKUP_TRIG_MODE_SHIFT (11U)
/*! CM7_LOCKUP_TRIG_MODE - CM7 lockup trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_CM7_LOCKUP_TRIG_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_CM7_LOCKUP_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_CM7_LOCKUP_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_DCDC_OVVT_TRIG_MODE_MASK (0x1000U)
#define SRC_GENERAL_SRTMR_DCDC_OVVT_TRIG_MODE_SHIFT (12U)
/*! DCDC_OVVT_TRIG_MODE - DCDC over voltage trigger mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_DCDC_OVVT_TRIG_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_DCDC_OVVT_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_DCDC_OVVT_TRIG_MODE_MASK)

#define SRC_GENERAL_SRTMR_ECAT_RSTO_TRIG_MODE_MASK (0x2000U)
#define SRC_GENERAL_SRTMR_ECAT_RSTO_TRIG_MODE_SHIFT (13U)
/*! ECAT_RSTO_TRIG_MODE - ECAT reset output mode configuration, locked by LOCK_CFG field
 *  0b0..Level-sensitive: System stays in reset until the reset source deasserts.
 *  0b1..Edge-sensitive: System resets once, even if the reset source remains asserted.
 */
#define SRC_GENERAL_SRTMR_ECAT_RSTO_TRIG_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRTMR_ECAT_RSTO_TRIG_MODE_SHIFT)) & SRC_GENERAL_SRTMR_ECAT_RSTO_TRIG_MODE_MASK)
/*! @} */

/*! @name SRMASK - SRC Reset Mask Register */
/*! @{ */

#define SRC_GENERAL_SRMASK_WDOG1_MASK_MASK       (0x1U)
#define SRC_GENERAL_SRMASK_WDOG1_MASK_SHIFT      (0U)
/*! WDOG1_MASK - WDOG1 reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_WDOG1_MASK(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG1_MASK_SHIFT)) & SRC_GENERAL_SRMASK_WDOG1_MASK_MASK)

#define SRC_GENERAL_SRMASK_WDOG2_MASK_MASK       (0x2U)
#define SRC_GENERAL_SRMASK_WDOG2_MASK_SHIFT      (1U)
/*! WDOG2_MASK - WDOG2 reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_WDOG2_MASK(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG2_MASK_SHIFT)) & SRC_GENERAL_SRMASK_WDOG2_MASK_MASK)

#define SRC_GENERAL_SRMASK_WDOG3_MASK_MASK       (0x4U)
#define SRC_GENERAL_SRMASK_WDOG3_MASK_SHIFT      (2U)
/*! WDOG3_MASK - WDOG3 reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_WDOG3_MASK(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG3_MASK_SHIFT)) & SRC_GENERAL_SRMASK_WDOG3_MASK_MASK)

#define SRC_GENERAL_SRMASK_WDOG4_MASK_MASK       (0x8U)
#define SRC_GENERAL_SRMASK_WDOG4_MASK_SHIFT      (3U)
/*! WDOG4_MASK - WDOG4 reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_WDOG4_MASK(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG4_MASK_SHIFT)) & SRC_GENERAL_SRMASK_WDOG4_MASK_MASK)

#define SRC_GENERAL_SRMASK_WDOG5_MASK_MASK       (0x10U)
#define SRC_GENERAL_SRMASK_WDOG5_MASK_SHIFT      (4U)
/*! WDOG5_MASK - WDOG5 reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_WDOG5_MASK(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG5_MASK_SHIFT)) & SRC_GENERAL_SRMASK_WDOG5_MASK_MASK)

#define SRC_GENERAL_SRMASK_TEMPSENSE_MASK_MASK   (0x20U)
#define SRC_GENERAL_SRMASK_TEMPSENSE_MASK_SHIFT  (5U)
/*! TEMPSENSE_MASK - TempSense reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_TEMPSENSE_MASK(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_TEMPSENSE_MASK_SHIFT)) & SRC_GENERAL_SRMASK_TEMPSENSE_MASK_MASK)

#define SRC_GENERAL_SRMASK_EDGELOCK_MASK_MASK    (0x40U)
#define SRC_GENERAL_SRMASK_EDGELOCK_MASK_SHIFT   (6U)
/*! EDGELOCK_MASK - Edgelock reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_EDGELOCK_MASK(x)      (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_EDGELOCK_MASK_SHIFT)) & SRC_GENERAL_SRMASK_EDGELOCK_MASK_MASK)

#define SRC_GENERAL_SRMASK_JTAGSW_MASK_MASK      (0x80U)
#define SRC_GENERAL_SRMASK_JTAGSW_MASK_SHIFT     (7U)
/*! JTAGSW_MASK - JTAGSW reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_JTAGSW_MASK(x)        (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_JTAGSW_MASK_SHIFT)) & SRC_GENERAL_SRMASK_JTAGSW_MASK_MASK)

#define SRC_GENERAL_SRMASK_CM33_RESET_MASK_MASK  (0x100U)
#define SRC_GENERAL_SRMASK_CM33_RESET_MASK_SHIFT (8U)
/*! CM33_RESET_MASK - CM33 reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_CM33_RESET_MASK(x)    (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM33_RESET_MASK_SHIFT)) & SRC_GENERAL_SRMASK_CM33_RESET_MASK_MASK)

#define SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_MASK (0x200U)
#define SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_SHIFT (9U)
/*! CM33_LOCKUP_MASK - CM33 lockup mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK(x)   (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_SHIFT)) & SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_MASK)

#define SRC_GENERAL_SRMASK_CM7_RESET_MASK_MASK   (0x400U)
#define SRC_GENERAL_SRMASK_CM7_RESET_MASK_SHIFT  (10U)
/*! CM7_RESET_MASK - CM7 reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_CM7_RESET_MASK(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM7_RESET_MASK_SHIFT)) & SRC_GENERAL_SRMASK_CM7_RESET_MASK_MASK)

#define SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_MASK  (0x800U)
#define SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_SHIFT (11U)
/*! CM7_LOCKUP_MASK - CM7 lockup reset mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK(x)    (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_SHIFT)) & SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_MASK)

#define SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_MASK   (0x1000U)
#define SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_SHIFT  (12U)
/*! DCDC_OVVT_MASK - DCDC over voltage mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_DCDC_OVVT_MASK(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_SHIFT)) & SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_MASK)

#define SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_MASK   (0x2000U)
#define SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_SHIFT  (13U)
/*! ECAT_RSTO_MASK - ECAT reset output mask
 *  0b0..The reset source can work
 *  0b1..The reset source is masked, cannot work
 */
#define SRC_GENERAL_SRMASK_ECAT_RSTO_MASK(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_SHIFT)) & SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_MASK)

#define SRC_GENERAL_SRMASK_WDOG1_MASK_LOCKED_MASK (0x10000U)
#define SRC_GENERAL_SRMASK_WDOG1_MASK_LOCKED_SHIFT (16U)
/*! WDOG1_MASK_LOCKED - Lock WDOG1_MASK
 *  0b0..This bit and WDOG1_MASK's value can be changed.
 *  0b1..This bit and WDOG1_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_WDOG1_MASK_LOCKED(x)  (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG1_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_WDOG1_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_WDOG2_MASK_LOCKED_MASK (0x20000U)
#define SRC_GENERAL_SRMASK_WDOG2_MASK_LOCKED_SHIFT (17U)
/*! WDOG2_MASK_LOCKED - Lock WDOG2_MASK
 *  0b0..This bit and WDOG2_MASK's value can be changed.
 *  0b1..This bit and WDOG2_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_WDOG2_MASK_LOCKED(x)  (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG2_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_WDOG2_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_WDOG3_MASK_LOCKED_MASK (0x40000U)
#define SRC_GENERAL_SRMASK_WDOG3_MASK_LOCKED_SHIFT (18U)
/*! WDOG3_MASK_LOCKED - Lock WDOG3_MASK
 *  0b0..This bit and WDOG3_MASK's value can be changed.
 *  0b1..This bit and WDOG3_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_WDOG3_MASK_LOCKED(x)  (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG3_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_WDOG3_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_WDOG4_MASK_LOCKED_MASK (0x80000U)
#define SRC_GENERAL_SRMASK_WDOG4_MASK_LOCKED_SHIFT (19U)
/*! WDOG4_MASK_LOCKED - Lock WDOG4_MASK
 *  0b0..This bit and WDOG4_MASK's value can be changed.
 *  0b1..This bit and WDOG4_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_WDOG4_MASK_LOCKED(x)  (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG4_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_WDOG4_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_WDOG5_MASK_LOCKED_MASK (0x100000U)
#define SRC_GENERAL_SRMASK_WDOG5_MASK_LOCKED_SHIFT (20U)
/*! WDOG5_MASK_LOCKED - Lock WDOG5_MASK
 *  0b0..This bit and WDOG5_MASK's value can be changed.
 *  0b1..This bit and WDOG5_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_WDOG5_MASK_LOCKED(x)  (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_WDOG5_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_WDOG5_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_TEMPSENSE_MASK_LOCKED_MASK (0x200000U)
#define SRC_GENERAL_SRMASK_TEMPSENSE_MASK_LOCKED_SHIFT (21U)
/*! TEMPSENSE_MASK_LOCKED - Lock TEMPSENSE_MASK
 *  0b0..TEMPSENSE_MASK's value can be changed.
 *  0b1..This bit and TEMPSENSE_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_TEMPSENSE_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_TEMPSENSE_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_TEMPSENSE_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_EDGELOCK_MASK_LOCKED_MASK (0x400000U)
#define SRC_GENERAL_SRMASK_EDGELOCK_MASK_LOCKED_SHIFT (22U)
/*! EDGELOCK_MASK_LOCKED - Lock EDGELOCK_MASK
 *  0b0..EDGELOCK_MASK's value can be changed.
 *  0b1..This bit and EDGELOCK_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_EDGELOCK_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_EDGELOCK_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_EDGELOCK_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_JTAGSW_MASK_LOCKED_MASK (0x800000U)
#define SRC_GENERAL_SRMASK_JTAGSW_MASK_LOCKED_SHIFT (23U)
/*! JTAGSW_MASK_LOCKED - Lock JTAGSW_MASK
 *  0b0..JTAGSW_MASK's value can be changed.
 *  0b1..This bit and JTAGSW_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_JTAGSW_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_JTAGSW_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_JTAGSW_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_CM33_RESET_MASK_LOCKED_MASK (0x1000000U)
#define SRC_GENERAL_SRMASK_CM33_RESET_MASK_LOCKED_SHIFT (24U)
/*! CM33_RESET_MASK_LOCKED - Lock CM33_RESET_MASK
 *  0b0..CM33_RESET_MASK's value can be changed.
 *  0b1..This bit and CM33_RESET_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_CM33_RESET_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM33_RESET_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_CM33_RESET_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_LOCKED_MASK (0x2000000U)
#define SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_LOCKED_SHIFT (25U)
/*! CM33_LOCKUP_MASK_LOCKED - Lock CM33_LOCKUP_MASK
 *  0b0..CM33_LOCKUP_MASK's value can be changed.
 *  0b1..This bit and CM33_LOCKUP_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_CM33_LOCKUP_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_CM7_RESET_MASK_LOCKED_MASK (0x4000000U)
#define SRC_GENERAL_SRMASK_CM7_RESET_MASK_LOCKED_SHIFT (26U)
/*! CM7_RESET_MASK_LOCKED - Lock CM7 reset mask bit
 *  0b0..CM7_RESET_MASK's value can be changed.
 *  0b1..This bit and CM7_RESET_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_CM7_RESET_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM7_RESET_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_CM7_RESET_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_LOCKED_MASK (0x8000000U)
#define SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_LOCKED_SHIFT (27U)
/*! CM7_LOCKUP_MASK_LOCKED - Lock CM7_LOCKUP_MASK
 *  0b0..CM7_LOCKUP_MASK's value can be changed.
 *  0b1..This bit and CM7_LOCKUP_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_CM7_LOCKUP_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_LOCKED_MASK (0x10000000U)
#define SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_LOCKED_SHIFT (28U)
/*! DCDC_OVVT_MASK_LOCKED - Lock DCDC_OVVT_MASK
 *  0b0..DCDC_OVVT_MASK's value can be changed.
 *  0b1..This bit and DCDC_OVVT_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_DCDC_OVVT_MASK_LOCKED_MASK)

#define SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_LOCKED_MASK (0x20000000U)
#define SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_LOCKED_SHIFT (29U)
/*! ECAT_RSTO_MASK_LOCKED - Lock ECAT_RSTO_MASK
 *  0b0..ECAT_RSTO_MASK's value can be changed.
 *  0b1..This bit and ECAT_RSTO_MASK's value cannot be changed.
 */
#define SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_LOCKED(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_LOCKED_SHIFT)) & SRC_GENERAL_SRMASK_ECAT_RSTO_MASK_LOCKED_MASK)
/*! @} */

/*! @name SBMR1 - SRC Boot Mode Register 1 */
/*! @{ */

#define SRC_GENERAL_SBMR1_BOOT_CFG1_MASK         (0xFFU)
#define SRC_GENERAL_SBMR1_BOOT_CFG1_SHIFT        (0U)
/*! BOOT_CFG1 - Reserved. */
#define SRC_GENERAL_SBMR1_BOOT_CFG1(x)           (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SBMR1_BOOT_CFG1_SHIFT)) & SRC_GENERAL_SBMR1_BOOT_CFG1_MASK)

#define SRC_GENERAL_SBMR1_BOOT_CFG2_MASK         (0xFF00U)
#define SRC_GENERAL_SBMR1_BOOT_CFG2_SHIFT        (8U)
/*! BOOT_CFG2 - Reserved. */
#define SRC_GENERAL_SBMR1_BOOT_CFG2(x)           (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SBMR1_BOOT_CFG2_SHIFT)) & SRC_GENERAL_SBMR1_BOOT_CFG2_MASK)

#define SRC_GENERAL_SBMR1_BOOT_CFG3_MASK         (0xFF0000U)
#define SRC_GENERAL_SBMR1_BOOT_CFG3_SHIFT        (16U)
/*! BOOT_CFG3 - Reserved. */
#define SRC_GENERAL_SBMR1_BOOT_CFG3(x)           (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SBMR1_BOOT_CFG3_SHIFT)) & SRC_GENERAL_SBMR1_BOOT_CFG3_MASK)

#define SRC_GENERAL_SBMR1_BOOT_CFG4_MASK         (0xFF000000U)
#define SRC_GENERAL_SBMR1_BOOT_CFG4_SHIFT        (24U)
/*! BOOT_CFG4 - Reserved. */
#define SRC_GENERAL_SBMR1_BOOT_CFG4(x)           (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SBMR1_BOOT_CFG4_SHIFT)) & SRC_GENERAL_SBMR1_BOOT_CFG4_MASK)
/*! @} */

/*! @name SBMR2 - SRC Boot Mode Register 2 */
/*! @{ */

#define SRC_GENERAL_SBMR2_IPP_BOOT_MODE_MASK     (0x3F000000U)
#define SRC_GENERAL_SBMR2_IPP_BOOT_MODE_SHIFT    (24U)
/*! IPP_BOOT_MODE
 *  0b000000..Boot from internal Fuses
 *  0b000001..Serial Downloader: USB1 or LPUART1
 *  0b000010..USDHC1 8-bit eMMC 5.1
 *  0b000011..USDHC2 4-bit SD 3.0
 *  0b000100..FlexSPI Serial NOR with SFDP (JESD-216) discoverable parameters
 *  0b000101..FlexSPI Serial NAND 2k page
 *  0b000110..FlexSPI Serial NAND 4k page
 *  0b000111..Test mode/Infinite loop mode
 */
#define SRC_GENERAL_SBMR2_IPP_BOOT_MODE(x)       (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SBMR2_IPP_BOOT_MODE_SHIFT)) & SRC_GENERAL_SBMR2_IPP_BOOT_MODE_MASK)
/*! @} */

/*! @name SRSR_BBSM - SRC Reset Status Register backup in BBSM domain */
/*! @{ */

#define SRC_GENERAL_SRSR_BBSM_POR_RST_MASK       (0x1U)
#define SRC_GENERAL_SRSR_BBSM_POR_RST_SHIFT      (0U)
/*! POR_RST - Indicates whether the reset was the result of power up or chip PAD POR_B.
 *  0b0..Reset is not a result of power up or chip PAD POR_B.
 *  0b1..Reset is a result of power up or chip PAD POR_B.
 */
#define SRC_GENERAL_SRSR_BBSM_POR_RST(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_POR_RST_SHIFT)) & SRC_GENERAL_SRSR_BBSM_POR_RST_MASK)

#define SRC_GENERAL_SRSR_BBSM_WDOG1_RST_B_MASK   (0x2U)
#define SRC_GENERAL_SRSR_BBSM_WDOG1_RST_B_SHIFT  (1U)
/*! WDOG1_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog1 time-out event.
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_BBSM_WDOG1_RST_B(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_WDOG1_RST_B_SHIFT)) & SRC_GENERAL_SRSR_BBSM_WDOG1_RST_B_MASK)

#define SRC_GENERAL_SRSR_BBSM_WDOG2_RST_B_MASK   (0x4U)
#define SRC_GENERAL_SRSR_BBSM_WDOG2_RST_B_SHIFT  (2U)
/*! WDOG2_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog2 time-out event.
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_BBSM_WDOG2_RST_B(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_WDOG2_RST_B_SHIFT)) & SRC_GENERAL_SRSR_BBSM_WDOG2_RST_B_MASK)

#define SRC_GENERAL_SRSR_BBSM_WDOG3_RST_B_MASK   (0x8U)
#define SRC_GENERAL_SRSR_BBSM_WDOG3_RST_B_SHIFT  (3U)
/*! WDOG3_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog3 time-out
 *  0b0..Reset is not a result of the watchdog3 time-out event.
 *  0b1..Reset is a result of the watchdog3 time-out event.
 */
#define SRC_GENERAL_SRSR_BBSM_WDOG3_RST_B(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_WDOG3_RST_B_SHIFT)) & SRC_GENERAL_SRSR_BBSM_WDOG3_RST_B_MASK)

#define SRC_GENERAL_SRSR_BBSM_WDOG4_RST_B_MASK   (0x10U)
#define SRC_GENERAL_SRSR_BBSM_WDOG4_RST_B_SHIFT  (4U)
/*! WDOG4_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog4 time-out
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_BBSM_WDOG4_RST_B(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_WDOG4_RST_B_SHIFT)) & SRC_GENERAL_SRSR_BBSM_WDOG4_RST_B_MASK)

#define SRC_GENERAL_SRSR_BBSM_WDOG5_RST_B_MASK   (0x20U)
#define SRC_GENERAL_SRSR_BBSM_WDOG5_RST_B_SHIFT  (5U)
/*! WDOG5_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog5 time-out
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_BBSM_WDOG5_RST_B(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_WDOG5_RST_B_SHIFT)) & SRC_GENERAL_SRSR_BBSM_WDOG5_RST_B_MASK)

#define SRC_GENERAL_SRSR_BBSM_TEMPSENSE_RST_B_MASK (0x40U)
#define SRC_GENERAL_SRSR_BBSM_TEMPSENSE_RST_B_SHIFT (6U)
/*! TEMPSENSE_RST_B - TempSensor software reset. Indicates whether the reset was the result of software reset from on-chip Temperature Sensor.
 *  0b0..Reset is not a result of software reset from Temperature Sensor.
 *  0b1..Reset is a result of software reset from Temperature Sensor.
 */
#define SRC_GENERAL_SRSR_BBSM_TEMPSENSE_RST_B(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_TEMPSENSE_RST_B_SHIFT)) & SRC_GENERAL_SRSR_BBSM_TEMPSENSE_RST_B_MASK)

#define SRC_GENERAL_SRSR_BBSM_EDGELOCK_RESET_B_MASK (0x80U)
#define SRC_GENERAL_SRSR_BBSM_EDGELOCK_RESET_B_SHIFT (7U)
/*! EDGELOCK_RESET_B - Indicates whether the reset was the result of the Edgelock's reset input.
 *  0b0..Reset is not a result of the Edgelock's reset event.
 *  0b1..Reset is a result of the Edgelock's reset event.
 */
#define SRC_GENERAL_SRSR_BBSM_EDGELOCK_RESET_B(x) (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_EDGELOCK_RESET_B_SHIFT)) & SRC_GENERAL_SRSR_BBSM_EDGELOCK_RESET_B_MASK)

#define SRC_GENERAL_SRSR_BBSM_JTAG_SW_RST_MASK   (0x100U)
#define SRC_GENERAL_SRSR_BBSM_JTAG_SW_RST_SHIFT  (8U)
/*! JTAG_SW_RST - JTAG software reset. Indicates whether the reset was the result of software reset from JTAG.
 *  0b0..Reset is not a result of software reset from JTAG.
 *  0b1..Reset is a result of software reset from JTAG.
 */
#define SRC_GENERAL_SRSR_BBSM_JTAG_SW_RST(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_JTAG_SW_RST_SHIFT)) & SRC_GENERAL_SRSR_BBSM_JTAG_SW_RST_MASK)

#define SRC_GENERAL_SRSR_BBSM_CM33_REQUEST_MASK  (0x200U)
#define SRC_GENERAL_SRSR_BBSM_CM33_REQUEST_SHIFT (9U)
/*! CM33_REQUEST - Indicates whether reset was the result of cm33 reset request
 *  0b0..Reset is not a result of cm33 reset request.
 *  0b1..Reset is a result of cm33 reset request.
 */
#define SRC_GENERAL_SRSR_BBSM_CM33_REQUEST(x)    (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_CM33_REQUEST_SHIFT)) & SRC_GENERAL_SRSR_BBSM_CM33_REQUEST_MASK)

#define SRC_GENERAL_SRSR_BBSM_CM33_LOCKUP_MASK   (0x400U)
#define SRC_GENERAL_SRSR_BBSM_CM33_LOCKUP_SHIFT  (10U)
/*! CM33_LOCKUP - Indicates a reset has been caused by cm33 CPU lockup
 *  0b0..Reset is not a result of the cm33 lockup.
 *  0b1..Reset is a result of the cm33 lockup.
 */
#define SRC_GENERAL_SRSR_BBSM_CM33_LOCKUP(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_CM33_LOCKUP_SHIFT)) & SRC_GENERAL_SRSR_BBSM_CM33_LOCKUP_MASK)

#define SRC_GENERAL_SRSR_BBSM_CM7_REQUEST_MASK   (0x800U)
#define SRC_GENERAL_SRSR_BBSM_CM7_REQUEST_SHIFT  (11U)
/*! CM7_REQUEST - Indicates whether reset was the result of cm7 reset request
 *  0b0..Reset is not a result of cm7 reset request.
 *  0b1..Reset is a result of cm7 reset request.
 */
#define SRC_GENERAL_SRSR_BBSM_CM7_REQUEST(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_CM7_REQUEST_SHIFT)) & SRC_GENERAL_SRSR_BBSM_CM7_REQUEST_MASK)

#define SRC_GENERAL_SRSR_BBSM_CM7_LOCKUP_MASK    (0x1000U)
#define SRC_GENERAL_SRSR_BBSM_CM7_LOCKUP_SHIFT   (12U)
/*! CM7_LOCKUP - Indicates a reset has been caused by CM7 CPU
 *  0b0..Reset is not a result of the cm7 lockup.
 *  0b1..Reset is a result of the cm7 lockup.
 */
#define SRC_GENERAL_SRSR_BBSM_CM7_LOCKUP(x)      (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_CM7_LOCKUP_SHIFT)) & SRC_GENERAL_SRSR_BBSM_CM7_LOCKUP_MASK)

#define SRC_GENERAL_SRSR_BBSM_DCDC_OVVT_MASK     (0x2000U)
#define SRC_GENERAL_SRSR_BBSM_DCDC_OVVT_SHIFT    (13U)
/*! DCDC_OVVT - Indicates a reset has been caused by DCDC over voltage
 *  0b0..Reset is not a result of the DCDC over voltage.
 *  0b1..Reset is a result of the DCDC over voltage.
 */
#define SRC_GENERAL_SRSR_BBSM_DCDC_OVVT(x)       (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_DCDC_OVVT_SHIFT)) & SRC_GENERAL_SRSR_BBSM_DCDC_OVVT_MASK)

#define SRC_GENERAL_SRSR_BBSM_ECAT_RSTO_MASK     (0x4000U)
#define SRC_GENERAL_SRSR_BBSM_ECAT_RSTO_SHIFT    (14U)
/*! ECAT_RSTO - Indicates a reset has been caused by ECAT reset output
 *  0b0..Reset is not a result of the ECAT reset output.
 *  0b1..Reset is a result of the ECAT reset output.
 */
#define SRC_GENERAL_SRSR_BBSM_ECAT_RSTO(x)       (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_BBSM_ECAT_RSTO_SHIFT)) & SRC_GENERAL_SRSR_BBSM_ECAT_RSTO_MASK)
/*! @} */

/*! @name SRSR - SRC Reset Status Register */
/*! @{ */

#define SRC_GENERAL_SRSR_POR_RST_MASK            (0x1U)
#define SRC_GENERAL_SRSR_POR_RST_SHIFT           (0U)
/*! POR_RST - Indicates whether the reset was the result of POR.
 *  0b0..Reset is not a result of POR.
 *  0b1..Reset is a result of POR.
 */
#define SRC_GENERAL_SRSR_POR_RST(x)              (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_POR_RST_SHIFT)) & SRC_GENERAL_SRSR_POR_RST_MASK)

#define SRC_GENERAL_SRSR_WDOG1_RST_B_MASK        (0x2U)
#define SRC_GENERAL_SRSR_WDOG1_RST_B_SHIFT       (1U)
/*! WDOG1_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog1 time-out event.
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_WDOG1_RST_B(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_WDOG1_RST_B_SHIFT)) & SRC_GENERAL_SRSR_WDOG1_RST_B_MASK)

#define SRC_GENERAL_SRSR_WDOG2_RST_B_MASK        (0x4U)
#define SRC_GENERAL_SRSR_WDOG2_RST_B_SHIFT       (2U)
/*! WDOG2_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog2 time-out event.
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_WDOG2_RST_B(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_WDOG2_RST_B_SHIFT)) & SRC_GENERAL_SRSR_WDOG2_RST_B_MASK)

#define SRC_GENERAL_SRSR_WDOG3_RST_B_MASK        (0x8U)
#define SRC_GENERAL_SRSR_WDOG3_RST_B_SHIFT       (3U)
/*! WDOG3_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog3 time-out
 *  0b0..Reset is not a result of the watchdog3 time-out event.
 *  0b1..Reset is a result of the watchdog3 time-out event.
 */
#define SRC_GENERAL_SRSR_WDOG3_RST_B(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_WDOG3_RST_B_SHIFT)) & SRC_GENERAL_SRSR_WDOG3_RST_B_MASK)

#define SRC_GENERAL_SRSR_WDOG4_RST_B_MASK        (0x10U)
#define SRC_GENERAL_SRSR_WDOG4_RST_B_SHIFT       (4U)
/*! WDOG4_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog4 time-out
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_WDOG4_RST_B(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_WDOG4_RST_B_SHIFT)) & SRC_GENERAL_SRSR_WDOG4_RST_B_MASK)

#define SRC_GENERAL_SRSR_WDOG5_RST_B_MASK        (0x20U)
#define SRC_GENERAL_SRSR_WDOG5_RST_B_SHIFT       (5U)
/*! WDOG5_RST_B - Time-out reset. Indicates whether the reset was the result of the watchdog5 time-out
 *  0b0..Reset is not a result of the watchdog time-out event.
 *  0b1..Reset is a result of the watchdog time-out event.
 */
#define SRC_GENERAL_SRSR_WDOG5_RST_B(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_WDOG5_RST_B_SHIFT)) & SRC_GENERAL_SRSR_WDOG5_RST_B_MASK)

#define SRC_GENERAL_SRSR_TEMPSENSE_RST_B_MASK    (0x40U)
#define SRC_GENERAL_SRSR_TEMPSENSE_RST_B_SHIFT   (6U)
/*! TEMPSENSE_RST_B - Temper Sensor software reset. Indicates whether the reset was the result of
 *    software reset from on-chip Temperature Sensor.
 *  0b0..Reset is not a result of software reset from Temperature Sensor.
 *  0b1..Reset is a result of software reset from Temperature Sensor.
 */
#define SRC_GENERAL_SRSR_TEMPSENSE_RST_B(x)      (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_TEMPSENSE_RST_B_SHIFT)) & SRC_GENERAL_SRSR_TEMPSENSE_RST_B_MASK)

#define SRC_GENERAL_SRSR_EDGELOCK_RESET_B_MASK   (0x80U)
#define SRC_GENERAL_SRSR_EDGELOCK_RESET_B_SHIFT  (7U)
/*! EDGELOCK_RESET_B - Indicates whether the reset was the result of the Edgelock's reset input.
 *  0b0..Reset is not a result of the Edgelock's reset event.
 *  0b1..Reset is a result of the Edgelock's reset event.
 */
#define SRC_GENERAL_SRSR_EDGELOCK_RESET_B(x)     (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_EDGELOCK_RESET_B_SHIFT)) & SRC_GENERAL_SRSR_EDGELOCK_RESET_B_MASK)

#define SRC_GENERAL_SRSR_JTAG_SW_RST_MASK        (0x100U)
#define SRC_GENERAL_SRSR_JTAG_SW_RST_SHIFT       (8U)
/*! JTAG_SW_RST - JTAG software reset. Indicates whether the reset was the result of software reset from JTAG.
 *  0b0..Reset is not a result of software reset from JTAG.
 *  0b1..Reset is a result of software reset from JTAG.
 */
#define SRC_GENERAL_SRSR_JTAG_SW_RST(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_JTAG_SW_RST_SHIFT)) & SRC_GENERAL_SRSR_JTAG_SW_RST_MASK)

#define SRC_GENERAL_SRSR_CM33_REQUEST_MASK       (0x200U)
#define SRC_GENERAL_SRSR_CM33_REQUEST_SHIFT      (9U)
/*! CM33_REQUEST - Indicates whether reset was the result of cm33 reset request
 *  0b0..Reset is not a result of cm33 reset request.
 *  0b1..Reset is a result of cm33 reset request.
 */
#define SRC_GENERAL_SRSR_CM33_REQUEST(x)         (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_CM33_REQUEST_SHIFT)) & SRC_GENERAL_SRSR_CM33_REQUEST_MASK)

#define SRC_GENERAL_SRSR_CM33_LOCKUP_MASK        (0x400U)
#define SRC_GENERAL_SRSR_CM33_LOCKUP_SHIFT       (10U)
/*! CM33_LOCKUP - Indicates a reset has been caused by cm33 CPU lockup
 *  0b0..Reset is not a result of the cm33 lockup.
 *  0b1..Reset is a result of the cm33 lockup.
 */
#define SRC_GENERAL_SRSR_CM33_LOCKUP(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_CM33_LOCKUP_SHIFT)) & SRC_GENERAL_SRSR_CM33_LOCKUP_MASK)

#define SRC_GENERAL_SRSR_CM7_REQUEST_MASK        (0x800U)
#define SRC_GENERAL_SRSR_CM7_REQUEST_SHIFT       (11U)
/*! CM7_REQUEST - Indicates whether reset was the result of cm7 reset request
 *  0b0..Reset is not a result of cm7 reset request.
 *  0b1..Reset is a result of cm7 reset request.
 */
#define SRC_GENERAL_SRSR_CM7_REQUEST(x)          (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_CM7_REQUEST_SHIFT)) & SRC_GENERAL_SRSR_CM7_REQUEST_MASK)

#define SRC_GENERAL_SRSR_CM7_LOCKUP_MASK         (0x1000U)
#define SRC_GENERAL_SRSR_CM7_LOCKUP_SHIFT        (12U)
/*! CM7_LOCKUP - Indicates a reset has been caused by CM7 CPU
 *  0b0..Reset is not a result of the cm7 lockup.
 *  0b1..Reset is a result of the cm7 lockup.
 */
#define SRC_GENERAL_SRSR_CM7_LOCKUP(x)           (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_CM7_LOCKUP_SHIFT)) & SRC_GENERAL_SRSR_CM7_LOCKUP_MASK)

#define SRC_GENERAL_SRSR_DCDC_OVVT_MASK          (0x2000U)
#define SRC_GENERAL_SRSR_DCDC_OVVT_SHIFT         (13U)
/*! DCDC_OVVT - Indicates a reset has been caused by DCDC over voltage
 *  0b0..Reset is not a result of the DCDC over voltage.
 *  0b1..Reset is a result of the DCDC over voltage.
 */
#define SRC_GENERAL_SRSR_DCDC_OVVT(x)            (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_DCDC_OVVT_SHIFT)) & SRC_GENERAL_SRSR_DCDC_OVVT_MASK)

#define SRC_GENERAL_SRSR_ECAT_RSTO_MASK          (0x4000U)
#define SRC_GENERAL_SRSR_ECAT_RSTO_SHIFT         (14U)
/*! ECAT_RSTO - Indicates a reset has been caused by ECAT reset output
 *  0b0..Reset is not a result of the ECAT reset output.
 *  0b1..Reset is a result of the ECAT reset output.
 */
#define SRC_GENERAL_SRSR_ECAT_RSTO(x)            (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_ECAT_RSTO_SHIFT)) & SRC_GENERAL_SRSR_ECAT_RSTO_MASK)

#define SRC_GENERAL_SRSR_IPP_POR_B_MASK          (0x10000U)
#define SRC_GENERAL_SRSR_IPP_POR_B_SHIFT         (16U)
/*! IPP_POR_B - Indicates whether the reset was the result of chip PAD POR_B.
 *  0b0..Reset is not a result of chip PAD POR_B.
 *  0b1..Reset is a result of chip PAD POR_B.
 */
#define SRC_GENERAL_SRSR_IPP_POR_B(x)            (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_SRSR_IPP_POR_B_SHIFT)) & SRC_GENERAL_SRSR_IPP_POR_B_MASK)
/*! @} */

/*! @name GPR - SRC General Purpose Register */
/*! @{ */

#define SRC_GENERAL_GPR_GPR_MASK                 (0xFFFFFFFFU)
#define SRC_GENERAL_GPR_GPR_SHIFT                (0U)
/*! GPR - General Purpose Register. */
#define SRC_GENERAL_GPR_GPR(x)                   (((uint32_t)(((uint32_t)(x)) << SRC_GENERAL_GPR_GPR_SHIFT)) & SRC_GENERAL_GPR_GPR_MASK)
/*! @} */

/* The count of SRC_GENERAL_GPR */
#define SRC_GENERAL_GPR_COUNT                    (20U)


/*!
 * @}
 */ /* end of group SRC_GENERAL_Register_Masks */


/* SRC_GENERAL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SRC_GENERAL_REG base address */
  #define SRC_GENERAL_REG_BASE                     (0x54460000u)
  /** Peripheral SRC_GENERAL_REG base address */
  #define SRC_GENERAL_REG_BASE_NS                  (0x44460000u)
  /** Peripheral SRC_GENERAL_REG base pointer */
  #define SRC_GENERAL_REG                          ((SRC_GENERAL_Type *)SRC_GENERAL_REG_BASE)
  /** Peripheral SRC_GENERAL_REG base pointer */
  #define SRC_GENERAL_REG_NS                       ((SRC_GENERAL_Type *)SRC_GENERAL_REG_BASE_NS)
  /** Array initializer of SRC_GENERAL peripheral base addresses */
  #define SRC_GENERAL_BASE_ADDRS                   { SRC_GENERAL_REG_BASE }
  /** Array initializer of SRC_GENERAL peripheral base pointers */
  #define SRC_GENERAL_BASE_PTRS                    { SRC_GENERAL_REG }
  /** Array initializer of SRC_GENERAL peripheral base addresses */
  #define SRC_GENERAL_BASE_ADDRS_NS                { SRC_GENERAL_REG_BASE_NS }
  /** Array initializer of SRC_GENERAL peripheral base pointers */
  #define SRC_GENERAL_BASE_PTRS_NS                 { SRC_GENERAL_REG_NS }
#else
  /** Peripheral SRC_GENERAL_REG base address */
  #define SRC_GENERAL_REG_BASE                     (0x44460000u)
  /** Peripheral SRC_GENERAL_REG base pointer */
  #define SRC_GENERAL_REG                          ((SRC_GENERAL_Type *)SRC_GENERAL_REG_BASE)
  /** Array initializer of SRC_GENERAL peripheral base addresses */
  #define SRC_GENERAL_BASE_ADDRS                   { SRC_GENERAL_REG_BASE }
  /** Array initializer of SRC_GENERAL peripheral base pointers */
  #define SRC_GENERAL_BASE_PTRS                    { SRC_GENERAL_REG }
#endif

/*!
 * @}
 */ /* end of group SRC_GENERAL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SRC_MIF_LN28FDSOI_SPLLRAM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIF_LN28FDSOI_SPLLRAM_Peripheral_Access_Layer SRC_MIF_LN28FDSOI_SPLLRAM Peripheral Access Layer
 * @{
 */

/** SRC_MIF_LN28FDSOI_SPLLRAM - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t MIF_CTRL;                          /**< MPC Control, offset: 0x4 */
  __I  uint32_t MIF_STAT;                          /**< MIF Status, offset: 0x8 */
       uint8_t RESERVED_1[20];
  __IO uint32_t MIF_MLPL_IG;                       /**< MIF MLPL control of IG, offset: 0x20 */
  __IO uint32_t MIF_DLY_IG;                        /**< MIF Delay of IG, offset: 0x24 */
       uint8_t RESERVED_2[8];
  __IO uint32_t MIF_MLPL_WLPD;                     /**< MIF MLPL control of WLPD, offset: 0x30 */
  __IO uint32_t MIF_DLY_WLPD;                      /**< MIF Delay of WLPD, offset: 0x34 */
       uint8_t RESERVED_3[8];
  __IO uint32_t MIF_MLPL_PD_B;                     /**< MIF MLPL control of PD_B, offset: 0x40 */
  __IO uint32_t MIF_DLY_PD_B;                      /**< MIF Delay of PD_B, offset: 0x44 */
} SRC_MIF_LN28FDSOI_SPLLRAM_Type;

/* ----------------------------------------------------------------------------
   -- SRC_MIF_LN28FDSOI_SPLLRAM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIF_LN28FDSOI_SPLLRAM_Register_Masks SRC_MIF_LN28FDSOI_SPLLRAM Register Masks
 * @{
 */

/*! @name MIF_CTRL - MPC Control */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_SW_CTRL_PIN_MASK (0x1U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_SW_CTRL_PIN_SHIFT (0U)
/*! SW_CTRL_PIN - Memory low power pins controlled by SW
 *  0b0..Use CURRENT_MLPL field to select MLPL_CTRL to control low power signal.
 *  0b1..Use SW_* field to control low power signal directly.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_SW_CTRL_PIN(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_SW_CTRL_PIN_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_SW_CTRL_PIN_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_MEM_PWR_ST_EN_MASK (0x2U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_MEM_PWR_ST_EN_SHIFT (1U)
/*! MEM_PWR_ST_EN - Memory power status will be considered when determining slice power status.
 *  0b0..Memory power status will not be considered when determining slice power status.
 *  0b1..Memory power status will be considered when determining slice power status.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_MEM_PWR_ST_EN(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_MEM_PWR_ST_EN_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_MEM_PWR_ST_EN_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_LOCK_CFG_MASK (0x100000U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_LOCK_CFG_SHIFT (20U)
/*! LOCK_CFG - Configuration lock
 *  0b0..The fields are not locked.
 *  0b1..The fields are locked.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_LOCK_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_LOCK_CFG_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_CTRL_LOCK_CFG_MASK)
/*! @} */

/*! @name MIF_STAT - MIF Status */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_MLPL_STATE_MASK (0x7U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_MLPL_STATE_SHIFT (0U)
/*! MLPL_STATE - Current state of CURRENT_MLPL */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_MLPL_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_MLPL_STATE_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_MLPL_STATE_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_IG_STATE_MASK (0x8U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_IG_STATE_SHIFT (3U)
/*! IG_STATE - Current state of IG
 *  0b0..IG is 0.
 *  0b1..IG is 1.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_IG_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_IG_STATE_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_IG_STATE_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_WLPD_STATE_MASK (0x10U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_WLPD_STATE_SHIFT (4U)
/*! WLPD_STATE - Current state of WLPD
 *  0b0..WLPD is 0.
 *  0b1..WLPD is 1.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_WLPD_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_WLPD_STATE_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_WLPD_STATE_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_PD_B_STATE_MASK (0x20U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_PD_B_STATE_SHIFT (5U)
/*! PD_B_STATE - Current state of PD_B
 *  0b0..PD_B is 0.
 *  0b1..PD_B is 1.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_PD_B_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_PD_B_STATE_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_STAT_PD_B_STATE_MASK)
/*! @} */

/*! @name MIF_MLPL_IG - MIF MLPL control of IG */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_MLPL_CTRL_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_MLPL_CTRL_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_SW_IG_MASK (0x10000U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_SW_IG_SHIFT (16U)
/*! SW_IG - Software control IG
 *  0b0..IG is 0.
 *  0b1..IG is 1.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_SW_IG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_SW_IG_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_IG_SW_IG_MASK)
/*! @} */

/*! @name MIF_DLY_IG - MIF Delay of IG */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_HI_DLY_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_HI_DLY_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_LO_DLY_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_IG_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_WLPD - MIF MLPL control of WLPD */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_MLPL_CTRL_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_MLPL_CTRL_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_SW_WLPD_MASK (0x10000U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_SW_WLPD_SHIFT (16U)
/*! SW_WLPD - Software control WLPD
 *  0b1..WLPD is 1.
 *  0b0..WLPD is 0.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_SW_WLPD(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_SW_WLPD_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_WLPD_SW_WLPD_MASK)
/*! @} */

/*! @name MIF_DLY_WLPD - MIF Delay of WLPD */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_HI_DLY_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_HI_DLY_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_LO_DLY_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_WLPD_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_PD_B - MIF MLPL control of PD_B */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_MLPL_CTRL_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_MLPL_CTRL_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_SW_PD_B_MASK (0x10000U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_SW_PD_B_SHIFT (16U)
/*! SW_PD_B - software control PD_B
 *  0b1..PD_B is 1.
 *  0b0..PD_B is 0.
 */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_SW_PD_B(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_SW_PD_B_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_MLPL_PD_B_SW_PD_B_MASK)
/*! @} */

/*! @name MIF_DLY_PD_B - MIF Delay of PD_B */
/*! @{ */

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_HI_DLY_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_HI_DLY_MASK)

#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_LO_DLY_SHIFT)) & SRC_MIF_LN28FDSOI_SPLLRAM_MIF_DLY_PD_B_PRE_LO_DLY_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SRC_MIF_LN28FDSOI_SPLLRAM_Register_Masks */


/* SRC_MIF_LN28FDSOI_SPLLRAM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral AON_MIF_LN28FDSOI_SPLLRAM base address */
  #define AON_MIF_LN28FDSOI_SPLLRAM_BASE           (0x54462400u)
  /** Peripheral AON_MIF_LN28FDSOI_SPLLRAM base address */
  #define AON_MIF_LN28FDSOI_SPLLRAM_BASE_NS        (0x44462400u)
  /** Peripheral AON_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define AON_MIF_LN28FDSOI_SPLLRAM                ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)AON_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Peripheral AON_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define AON_MIF_LN28FDSOI_SPLLRAM_NS             ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)AON_MIF_LN28FDSOI_SPLLRAM_BASE_NS)
  /** Peripheral WAKEUP_MIF_LN28FDSOI_SPLLRAM base address */
  #define WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE        (0x54462C00u)
  /** Peripheral WAKEUP_MIF_LN28FDSOI_SPLLRAM base address */
  #define WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE_NS     (0x44462C00u)
  /** Peripheral WAKEUP_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define WAKEUP_MIF_LN28FDSOI_SPLLRAM             ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Peripheral WAKEUP_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define WAKEUP_MIF_LN28FDSOI_SPLLRAM_NS          ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE_NS)
  /** Peripheral MEGA_MIF_LN28FDSOI_SPLLRAM base address */
  #define MEGA_MIF_LN28FDSOI_SPLLRAM_BASE          (0x54463400u)
  /** Peripheral MEGA_MIF_LN28FDSOI_SPLLRAM base address */
  #define MEGA_MIF_LN28FDSOI_SPLLRAM_BASE_NS       (0x44463400u)
  /** Peripheral MEGA_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define MEGA_MIF_LN28FDSOI_SPLLRAM               ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)MEGA_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Peripheral MEGA_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define MEGA_MIF_LN28FDSOI_SPLLRAM_NS            ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)MEGA_MIF_LN28FDSOI_SPLLRAM_BASE_NS)
  /** Peripheral NETC_MIF_LN28FDSOI_SPLLRAM base address */
  #define NETC_MIF_LN28FDSOI_SPLLRAM_BASE          (0x54463C00u)
  /** Peripheral NETC_MIF_LN28FDSOI_SPLLRAM base address */
  #define NETC_MIF_LN28FDSOI_SPLLRAM_BASE_NS       (0x44463C00u)
  /** Peripheral NETC_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define NETC_MIF_LN28FDSOI_SPLLRAM               ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)NETC_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Peripheral NETC_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define NETC_MIF_LN28FDSOI_SPLLRAM_NS            ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)NETC_MIF_LN28FDSOI_SPLLRAM_BASE_NS)
  /** Array initializer of SRC_MIF_LN28FDSOI_SPLLRAM peripheral base addresses */
  #define SRC_MIF_LN28FDSOI_SPLLRAM_BASE_ADDRS     { AON_MIF_LN28FDSOI_SPLLRAM_BASE, WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE, MEGA_MIF_LN28FDSOI_SPLLRAM_BASE, NETC_MIF_LN28FDSOI_SPLLRAM_BASE }
  /** Array initializer of SRC_MIF_LN28FDSOI_SPLLRAM peripheral base pointers */
  #define SRC_MIF_LN28FDSOI_SPLLRAM_BASE_PTRS      { AON_MIF_LN28FDSOI_SPLLRAM, WAKEUP_MIF_LN28FDSOI_SPLLRAM, MEGA_MIF_LN28FDSOI_SPLLRAM, NETC_MIF_LN28FDSOI_SPLLRAM }
  /** Array initializer of SRC_MIF_LN28FDSOI_SPLLRAM peripheral base addresses */
  #define SRC_MIF_LN28FDSOI_SPLLRAM_BASE_ADDRS_NS  { AON_MIF_LN28FDSOI_SPLLRAM_BASE_NS, WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE_NS, MEGA_MIF_LN28FDSOI_SPLLRAM_BASE_NS, NETC_MIF_LN28FDSOI_SPLLRAM_BASE_NS }
  /** Array initializer of SRC_MIF_LN28FDSOI_SPLLRAM peripheral base pointers */
  #define SRC_MIF_LN28FDSOI_SPLLRAM_BASE_PTRS_NS   { AON_MIF_LN28FDSOI_SPLLRAM_NS, WAKEUP_MIF_LN28FDSOI_SPLLRAM_NS, MEGA_MIF_LN28FDSOI_SPLLRAM_NS, NETC_MIF_LN28FDSOI_SPLLRAM_NS }
#else
  /** Peripheral AON_MIF_LN28FDSOI_SPLLRAM base address */
  #define AON_MIF_LN28FDSOI_SPLLRAM_BASE           (0x44462400u)
  /** Peripheral AON_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define AON_MIF_LN28FDSOI_SPLLRAM                ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)AON_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Peripheral WAKEUP_MIF_LN28FDSOI_SPLLRAM base address */
  #define WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE        (0x44462C00u)
  /** Peripheral WAKEUP_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define WAKEUP_MIF_LN28FDSOI_SPLLRAM             ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Peripheral MEGA_MIF_LN28FDSOI_SPLLRAM base address */
  #define MEGA_MIF_LN28FDSOI_SPLLRAM_BASE          (0x44463400u)
  /** Peripheral MEGA_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define MEGA_MIF_LN28FDSOI_SPLLRAM               ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)MEGA_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Peripheral NETC_MIF_LN28FDSOI_SPLLRAM base address */
  #define NETC_MIF_LN28FDSOI_SPLLRAM_BASE          (0x44463C00u)
  /** Peripheral NETC_MIF_LN28FDSOI_SPLLRAM base pointer */
  #define NETC_MIF_LN28FDSOI_SPLLRAM               ((SRC_MIF_LN28FDSOI_SPLLRAM_Type *)NETC_MIF_LN28FDSOI_SPLLRAM_BASE)
  /** Array initializer of SRC_MIF_LN28FDSOI_SPLLRAM peripheral base addresses */
  #define SRC_MIF_LN28FDSOI_SPLLRAM_BASE_ADDRS     { AON_MIF_LN28FDSOI_SPLLRAM_BASE, WAKEUP_MIF_LN28FDSOI_SPLLRAM_BASE, MEGA_MIF_LN28FDSOI_SPLLRAM_BASE, NETC_MIF_LN28FDSOI_SPLLRAM_BASE }
  /** Array initializer of SRC_MIF_LN28FDSOI_SPLLRAM peripheral base pointers */
  #define SRC_MIF_LN28FDSOI_SPLLRAM_BASE_PTRS      { AON_MIF_LN28FDSOI_SPLLRAM, WAKEUP_MIF_LN28FDSOI_SPLLRAM, MEGA_MIF_LN28FDSOI_SPLLRAM, NETC_MIF_LN28FDSOI_SPLLRAM }
#endif

/*!
 * @}
 */ /* end of group SRC_MIF_LN28FDSOI_SPLLRAM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SRC_MIF_S28SPREGH Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIF_S28SPREGH_Peripheral_Access_Layer SRC_MIF_S28SPREGH Peripheral Access Layer
 * @{
 */

/** SRC_MIF_S28SPREGH - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t MIF_CTRL;                          /**< MPC Control, offset: 0x4 */
  __I  uint32_t MIF_STAT;                          /**< MIF Status, offset: 0x8 */
       uint8_t RESERVED_1[4];
  __IO uint32_t MIF_MLPL_LS;                       /**< MIF MLPL control of LS, offset: 0x10 */
  __IO uint32_t MIF_DLY_LS;                        /**< MIF Delay of LS, offset: 0x14 */
       uint8_t RESERVED_2[8];
  __IO uint32_t MIF_MLPL_HS;                       /**< MIF MLPL control of HS, offset: 0x20 */
  __IO uint32_t MIF_DLY_HS;                        /**< MIF Delay of HS, offset: 0x24 */
       uint8_t RESERVED_3[8];
  __IO uint32_t MIF_MLPL_IG;                       /**< MIF MLPL control of Input Gating (IG), offset: 0x30 */
  __IO uint32_t MIF_DLY_IG;                        /**< MIF Delay of IG, offset: 0x34 */
       uint8_t RESERVED_4[8];
  __IO uint32_t MIF_MLPL_STDBY;                    /**< MIF MLPL control of STDBY, offset: 0x40 */
  __IO uint32_t MIF_DLY_STDBY;                     /**< MIF Delay of STDBY, offset: 0x44 */
} SRC_MIF_S28SPREGH_Type;

/* ----------------------------------------------------------------------------
   -- SRC_MIF_S28SPREGH Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIF_S28SPREGH_Register_Masks SRC_MIF_S28SPREGH Register Masks
 * @{
 */

/*! @name MIF_CTRL - MPC Control */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_CTRL_SW_CTRL_PIN_MASK (0x1U)
#define SRC_MIF_S28SPREGH_MIF_CTRL_SW_CTRL_PIN_SHIFT (0U)
/*! SW_CTRL_PIN - Memory low power pins controlled by software (SW)
 *  0b0..Uses CURRENT_MLPL field to select MLPL_CTRL to control low power signal.
 *  0b1..Uses SW_* field to control low power signal directly.
 */
#define SRC_MIF_S28SPREGH_MIF_CTRL_SW_CTRL_PIN(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_CTRL_SW_CTRL_PIN_SHIFT)) & SRC_MIF_S28SPREGH_MIF_CTRL_SW_CTRL_PIN_MASK)

#define SRC_MIF_S28SPREGH_MIF_CTRL_MEM_PWR_ST_EN_MASK (0x2U)
#define SRC_MIF_S28SPREGH_MIF_CTRL_MEM_PWR_ST_EN_SHIFT (1U)
/*! MEM_PWR_ST_EN - Memory power status will be considered when determining slice power status.
 *  0b1..Memory power status will be considered when determining slice power status.
 *  0b0..Memory power status will not be considered when determining slice power status.
 */
#define SRC_MIF_S28SPREGH_MIF_CTRL_MEM_PWR_ST_EN(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_CTRL_MEM_PWR_ST_EN_SHIFT)) & SRC_MIF_S28SPREGH_MIF_CTRL_MEM_PWR_ST_EN_MASK)

#define SRC_MIF_S28SPREGH_MIF_CTRL_LOCK_CFG_MASK (0x100000U)
#define SRC_MIF_S28SPREGH_MIF_CTRL_LOCK_CFG_SHIFT (20U)
/*! LOCK_CFG - Configuration lock
 *  0b0..The fields are not locked.
 *  0b1..The fields are locked.
 */
#define SRC_MIF_S28SPREGH_MIF_CTRL_LOCK_CFG(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_CTRL_LOCK_CFG_SHIFT)) & SRC_MIF_S28SPREGH_MIF_CTRL_LOCK_CFG_MASK)
/*! @} */

/*! @name MIF_STAT - MIF Status */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_STAT_MLPL_STATE_MASK (0x7U)
#define SRC_MIF_S28SPREGH_MIF_STAT_MLPL_STATE_SHIFT (0U)
/*! MLPL_STATE - Current state of CURRENT_MLPL */
#define SRC_MIF_S28SPREGH_MIF_STAT_MLPL_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_STAT_MLPL_STATE_SHIFT)) & SRC_MIF_S28SPREGH_MIF_STAT_MLPL_STATE_MASK)

#define SRC_MIF_S28SPREGH_MIF_STAT_LS_STATE_MASK (0x8U)
#define SRC_MIF_S28SPREGH_MIF_STAT_LS_STATE_SHIFT (3U)
/*! LS_STATE - Current state of LS
 *  0b0..LS is 0.
 *  0b1..LS is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_STAT_LS_STATE(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_STAT_LS_STATE_SHIFT)) & SRC_MIF_S28SPREGH_MIF_STAT_LS_STATE_MASK)

#define SRC_MIF_S28SPREGH_MIF_STAT_HS_STATE_MASK (0x10U)
#define SRC_MIF_S28SPREGH_MIF_STAT_HS_STATE_SHIFT (4U)
/*! HS_STATE - Current state of HS
 *  0b0..HS is 0.
 *  0b1..HS is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_STAT_HS_STATE(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_STAT_HS_STATE_SHIFT)) & SRC_MIF_S28SPREGH_MIF_STAT_HS_STATE_MASK)

#define SRC_MIF_S28SPREGH_MIF_STAT_IG_STATE_MASK (0x20U)
#define SRC_MIF_S28SPREGH_MIF_STAT_IG_STATE_SHIFT (5U)
/*! IG_STATE - Current state of IG
 *  0b0..IG is 0.
 *  0b1..IG is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_STAT_IG_STATE(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_STAT_IG_STATE_SHIFT)) & SRC_MIF_S28SPREGH_MIF_STAT_IG_STATE_MASK)

#define SRC_MIF_S28SPREGH_MIF_STAT_STDBY_STATE_MASK (0x40U)
#define SRC_MIF_S28SPREGH_MIF_STAT_STDBY_STATE_SHIFT (6U)
/*! STDBY_STATE - Current state of STDBY
 *  0b0..STDBY is 0.
 *  0b1..STDBY is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_STAT_STDBY_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_STAT_STDBY_STATE_SHIFT)) & SRC_MIF_S28SPREGH_MIF_STAT_STDBY_STATE_MASK)
/*! @} */

/*! @name MIF_MLPL_LS - MIF MLPL control of LS */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_MLPL_LS_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_MIF_MLPL_LS_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_MIF_MLPL_LS_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_LS_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_LS_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_MIF_MLPL_LS_SW_LS_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_MIF_MLPL_LS_SW_LS_SHIFT (16U)
/*! SW_LS - Software control LS
 *  0b0..LS is 0.
 *  0b1..LS is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_MLPL_LS_SW_LS(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_LS_SW_LS_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_LS_SW_LS_MASK)
/*! @} */

/*! @name MIF_DLY_LS - MIF Delay of LS */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_LS_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_HS - MIF MLPL control of HS */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_MLPL_HS_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_MIF_MLPL_HS_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_MIF_MLPL_HS_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_HS_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_HS_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_MIF_MLPL_HS_SW_HS_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_MIF_MLPL_HS_SW_HS_SHIFT (16U)
/*! SW_HS - software control HS
 *  0b0..HS is 0.
 *  0b1..HS is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_MLPL_HS_SW_HS(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_HS_SW_HS_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_HS_SW_HS_MASK)
/*! @} */

/*! @name MIF_DLY_HS - MIF Delay of HS */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_HS_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_IG - MIF MLPL control of Input Gating (IG) */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_MLPL_IG_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_MIF_MLPL_IG_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_MIF_MLPL_IG_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_IG_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_IG_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_MIF_MLPL_IG_SW_IG_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_MIF_MLPL_IG_SW_IG_SHIFT (16U)
/*! SW_IG - Software control IG
 *  0b0..IG is 0.
 *  0b1..IG is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_MLPL_IG_SW_IG(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_IG_SW_IG_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_IG_SW_IG_MASK)
/*! @} */

/*! @name MIF_DLY_IG - MIF Delay of IG */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_IG_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_STDBY - MIF MLPL control of STDBY */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_SW_STDBY_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_SW_STDBY_SHIFT (16U)
/*! SW_STDBY - Software control STDBY
 *  0b0..STDBY is 0.
 *  0b1..STDBY is 1.
 */
#define SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_SW_STDBY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_SW_STDBY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_MLPL_STDBY_SW_STDBY_MASK)
/*! @} */

/*! @name MIF_DLY_STDBY - MIF Delay of STDBY */
/*! @{ */

#define SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_MIF_DLY_STDBY_PRE_LO_DLY_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SRC_MIF_S28SPREGH_Register_Masks */


/* SRC_MIF_S28SPREGH - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral AON_MIF_S28SPREGH base address */
  #define AON_MIF_S28SPREGH_BASE                   (0x54462000u)
  /** Peripheral AON_MIF_S28SPREGH base address */
  #define AON_MIF_S28SPREGH_BASE_NS                (0x44462000u)
  /** Peripheral AON_MIF_S28SPREGH base pointer */
  #define AON_MIF_S28SPREGH                        ((SRC_MIF_S28SPREGH_Type *)AON_MIF_S28SPREGH_BASE)
  /** Peripheral AON_MIF_S28SPREGH base pointer */
  #define AON_MIF_S28SPREGH_NS                     ((SRC_MIF_S28SPREGH_Type *)AON_MIF_S28SPREGH_BASE_NS)
  /** Peripheral WAKEUP_MIF_S28SPREGH base address */
  #define WAKEUP_MIF_S28SPREGH_BASE                (0x54462800u)
  /** Peripheral WAKEUP_MIF_S28SPREGH base address */
  #define WAKEUP_MIF_S28SPREGH_BASE_NS             (0x44462800u)
  /** Peripheral WAKEUP_MIF_S28SPREGH base pointer */
  #define WAKEUP_MIF_S28SPREGH                     ((SRC_MIF_S28SPREGH_Type *)WAKEUP_MIF_S28SPREGH_BASE)
  /** Peripheral WAKEUP_MIF_S28SPREGH base pointer */
  #define WAKEUP_MIF_S28SPREGH_NS                  ((SRC_MIF_S28SPREGH_Type *)WAKEUP_MIF_S28SPREGH_BASE_NS)
  /** Peripheral MEGA_MIF_S28SPREGH base address */
  #define MEGA_MIF_S28SPREGH_BASE                  (0x54463000u)
  /** Peripheral MEGA_MIF_S28SPREGH base address */
  #define MEGA_MIF_S28SPREGH_BASE_NS               (0x44463000u)
  /** Peripheral MEGA_MIF_S28SPREGH base pointer */
  #define MEGA_MIF_S28SPREGH                       ((SRC_MIF_S28SPREGH_Type *)MEGA_MIF_S28SPREGH_BASE)
  /** Peripheral MEGA_MIF_S28SPREGH base pointer */
  #define MEGA_MIF_S28SPREGH_NS                    ((SRC_MIF_S28SPREGH_Type *)MEGA_MIF_S28SPREGH_BASE_NS)
  /** Peripheral NETC_MIF_S28SPREGH base address */
  #define NETC_MIF_S28SPREGH_BASE                  (0x54463800u)
  /** Peripheral NETC_MIF_S28SPREGH base address */
  #define NETC_MIF_S28SPREGH_BASE_NS               (0x44463800u)
  /** Peripheral NETC_MIF_S28SPREGH base pointer */
  #define NETC_MIF_S28SPREGH                       ((SRC_MIF_S28SPREGH_Type *)NETC_MIF_S28SPREGH_BASE)
  /** Peripheral NETC_MIF_S28SPREGH base pointer */
  #define NETC_MIF_S28SPREGH_NS                    ((SRC_MIF_S28SPREGH_Type *)NETC_MIF_S28SPREGH_BASE_NS)
  /** Peripheral CM33PLATFORM_CACHE base address */
  #define CM33PLATFORM_CACHE_BASE                  (0x54464000u)
  /** Peripheral CM33PLATFORM_CACHE base address */
  #define CM33PLATFORM_CACHE_BASE_NS               (0x44464000u)
  /** Peripheral CM33PLATFORM_CACHE base pointer */
  #define CM33PLATFORM_CACHE                       ((SRC_MIF_S28SPREGH_Type *)CM33PLATFORM_CACHE_BASE)
  /** Peripheral CM33PLATFORM_CACHE base pointer */
  #define CM33PLATFORM_CACHE_NS                    ((SRC_MIF_S28SPREGH_Type *)CM33PLATFORM_CACHE_BASE_NS)
  /** Peripheral CM7PLATFORM_CACHE base address */
  #define CM7PLATFORM_CACHE_BASE                   (0x54464800u)
  /** Peripheral CM7PLATFORM_CACHE base address */
  #define CM7PLATFORM_CACHE_BASE_NS                (0x44464800u)
  /** Peripheral CM7PLATFORM_CACHE base pointer */
  #define CM7PLATFORM_CACHE                        ((SRC_MIF_S28SPREGH_Type *)CM7PLATFORM_CACHE_BASE)
  /** Peripheral CM7PLATFORM_CACHE base pointer */
  #define CM7PLATFORM_CACHE_NS                     ((SRC_MIF_S28SPREGH_Type *)CM7PLATFORM_CACHE_BASE_NS)
  /** Peripheral CM33PLATFORM_TCM base address */
  #define CM33PLATFORM_TCM_BASE                    (0x54464400u)
  /** Peripheral CM33PLATFORM_TCM base address */
  #define CM33PLATFORM_TCM_BASE_NS                 (0x44464400u)
  /** Peripheral CM33PLATFORM_TCM base pointer */
  #define CM33PLATFORM_TCM                         ((SRC_MIF_S28SPREGH_Type *)CM33PLATFORM_TCM_BASE)
  /** Peripheral CM33PLATFORM_TCM base pointer */
  #define CM33PLATFORM_TCM_NS                      ((SRC_MIF_S28SPREGH_Type *)CM33PLATFORM_TCM_BASE_NS)
  /** Array initializer of SRC_MIF_S28SPREGH peripheral base addresses */
  #define SRC_MIF_S28SPREGH_BASE_ADDRS             { AON_MIF_S28SPREGH_BASE, WAKEUP_MIF_S28SPREGH_BASE, MEGA_MIF_S28SPREGH_BASE, NETC_MIF_S28SPREGH_BASE, CM33PLATFORM_CACHE_BASE, CM7PLATFORM_CACHE_BASE, CM33PLATFORM_TCM_BASE }
  /** Array initializer of SRC_MIF_S28SPREGH peripheral base pointers */
  #define SRC_MIF_S28SPREGH_BASE_PTRS              { AON_MIF_S28SPREGH, WAKEUP_MIF_S28SPREGH, MEGA_MIF_S28SPREGH, NETC_MIF_S28SPREGH, CM33PLATFORM_CACHE, CM7PLATFORM_CACHE, CM33PLATFORM_TCM }
  /** Array initializer of SRC_MIF_S28SPREGH peripheral base addresses */
  #define SRC_MIF_S28SPREGH_BASE_ADDRS_NS          { AON_MIF_S28SPREGH_BASE_NS, WAKEUP_MIF_S28SPREGH_BASE_NS, MEGA_MIF_S28SPREGH_BASE_NS, NETC_MIF_S28SPREGH_BASE_NS, CM33PLATFORM_CACHE_BASE_NS, CM7PLATFORM_CACHE_BASE_NS, CM33PLATFORM_TCM_BASE_NS }
  /** Array initializer of SRC_MIF_S28SPREGH peripheral base pointers */
  #define SRC_MIF_S28SPREGH_BASE_PTRS_NS           { AON_MIF_S28SPREGH_NS, WAKEUP_MIF_S28SPREGH_NS, MEGA_MIF_S28SPREGH_NS, NETC_MIF_S28SPREGH_NS, CM33PLATFORM_CACHE_NS, CM7PLATFORM_CACHE_NS, CM33PLATFORM_TCM_NS }
#else
  /** Peripheral AON_MIF_S28SPREGH base address */
  #define AON_MIF_S28SPREGH_BASE                   (0x44462000u)
  /** Peripheral AON_MIF_S28SPREGH base pointer */
  #define AON_MIF_S28SPREGH                        ((SRC_MIF_S28SPREGH_Type *)AON_MIF_S28SPREGH_BASE)
  /** Peripheral WAKEUP_MIF_S28SPREGH base address */
  #define WAKEUP_MIF_S28SPREGH_BASE                (0x44462800u)
  /** Peripheral WAKEUP_MIF_S28SPREGH base pointer */
  #define WAKEUP_MIF_S28SPREGH                     ((SRC_MIF_S28SPREGH_Type *)WAKEUP_MIF_S28SPREGH_BASE)
  /** Peripheral MEGA_MIF_S28SPREGH base address */
  #define MEGA_MIF_S28SPREGH_BASE                  (0x44463000u)
  /** Peripheral MEGA_MIF_S28SPREGH base pointer */
  #define MEGA_MIF_S28SPREGH                       ((SRC_MIF_S28SPREGH_Type *)MEGA_MIF_S28SPREGH_BASE)
  /** Peripheral NETC_MIF_S28SPREGH base address */
  #define NETC_MIF_S28SPREGH_BASE                  (0x44463800u)
  /** Peripheral NETC_MIF_S28SPREGH base pointer */
  #define NETC_MIF_S28SPREGH                       ((SRC_MIF_S28SPREGH_Type *)NETC_MIF_S28SPREGH_BASE)
  /** Peripheral CM33PLATFORM_CACHE base address */
  #define CM33PLATFORM_CACHE_BASE                  (0x44464000u)
  /** Peripheral CM33PLATFORM_CACHE base pointer */
  #define CM33PLATFORM_CACHE                       ((SRC_MIF_S28SPREGH_Type *)CM33PLATFORM_CACHE_BASE)
  /** Peripheral CM7PLATFORM_CACHE base address */
  #define CM7PLATFORM_CACHE_BASE                   (0x44464800u)
  /** Peripheral CM7PLATFORM_CACHE base pointer */
  #define CM7PLATFORM_CACHE                        ((SRC_MIF_S28SPREGH_Type *)CM7PLATFORM_CACHE_BASE)
  /** Peripheral CM33PLATFORM_TCM base address */
  #define CM33PLATFORM_TCM_BASE                    (0x44464400u)
  /** Peripheral CM33PLATFORM_TCM base pointer */
  #define CM33PLATFORM_TCM                         ((SRC_MIF_S28SPREGH_Type *)CM33PLATFORM_TCM_BASE)
  /** Array initializer of SRC_MIF_S28SPREGH peripheral base addresses */
  #define SRC_MIF_S28SPREGH_BASE_ADDRS             { AON_MIF_S28SPREGH_BASE, WAKEUP_MIF_S28SPREGH_BASE, MEGA_MIF_S28SPREGH_BASE, NETC_MIF_S28SPREGH_BASE, CM33PLATFORM_CACHE_BASE, CM7PLATFORM_CACHE_BASE, CM33PLATFORM_TCM_BASE }
  /** Array initializer of SRC_MIF_S28SPREGH peripheral base pointers */
  #define SRC_MIF_S28SPREGH_BASE_PTRS              { AON_MIF_S28SPREGH, WAKEUP_MIF_S28SPREGH, MEGA_MIF_S28SPREGH, NETC_MIF_S28SPREGH, CM33PLATFORM_CACHE, CM7PLATFORM_CACHE, CM33PLATFORM_TCM }
#endif

/*!
 * @}
 */ /* end of group SRC_MIF_S28SPREGH_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SRC_MIF_S28SPREGH_PSWA Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIF_S28SPREGH_PSWA_Peripheral_Access_Layer SRC_MIF_S28SPREGH_PSWA Peripheral Access Layer
 * @{
 */

/** SRC_MIF_S28SPREGH_PSWA - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t MIF_CTRL;                          /**< MPC Control, offset: 0x4 */
  __I  uint32_t MIF_STAT;                          /**< MIF Status, offset: 0x8 */
       uint8_t RESERVED_1[4];
  __IO uint32_t MIF_MLPL_LS;                       /**< MIF MLPL control of LS, offset: 0x10 */
  __IO uint32_t MIF_DLY_LS;                        /**< MIF Delay of LS, offset: 0x14 */
       uint8_t RESERVED_2[8];
  __IO uint32_t MIF_MLPL_HS;                       /**< MIF MLPL control of HS, offset: 0x20 */
  __IO uint32_t MIF_DLY_HS;                        /**< MIF Delay of HS, offset: 0x24 */
       uint8_t RESERVED_3[8];
  __IO uint32_t MIF_MLPL_IG;                       /**< MIF MLPL control of Input Gating (IG), offset: 0x30 */
  __IO uint32_t MIF_DLY_IG;                        /**< MIF Delay of IG, offset: 0x34 */
       uint8_t RESERVED_4[8];
  __IO uint32_t MIF_MLPL_STDBY;                    /**< MIF MLPL control of STDBY, offset: 0x40 */
  __IO uint32_t MIF_DLY_STDBY;                     /**< MIF Delay of STDBY, offset: 0x44 */
       uint8_t RESERVED_5[8];
  __IO uint32_t MIF_MLPL_SLEEP;                    /**< MIF MLPL control of SLEEP, offset: 0x50 */
  __IO uint32_t MIF_DLY_SLEEP;                     /**< MIF Delay of SLEEP, offset: 0x54 */
       uint8_t RESERVED_6[8];
  __IO uint32_t MIF_MLPL_ARR_PDN;                  /**< MIF MLPL control of array power down, offset: 0x60 */
  __IO uint32_t MIF_DLY_ARR_HF;                    /**< MIF Delay of array high-fanout power switch, offset: 0x64 */
} SRC_MIF_S28SPREGH_PSWA_Type;

/* ----------------------------------------------------------------------------
   -- SRC_MIF_S28SPREGH_PSWA Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIF_S28SPREGH_PSWA_Register_Masks SRC_MIF_S28SPREGH_PSWA Register Masks
 * @{
 */

/*! @name MIF_CTRL - MPC Control */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_SW_CTRL_PIN_MASK (0x1U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_SW_CTRL_PIN_SHIFT (0U)
/*! SW_CTRL_PIN - Memory low power pins controlled by SW
 *  0b0..Use CURRENT_MLPL field to select MLPL_CTRL to control low power signal.
 *  0b1..Use SW_* field to control low power signal directly.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_SW_CTRL_PIN(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_SW_CTRL_PIN_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_SW_CTRL_PIN_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_MEM_PWR_ST_EN_MASK (0x2U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_MEM_PWR_ST_EN_SHIFT (1U)
/*! MEM_PWR_ST_EN - Memory power status will be considered when determining slice power status.
 *  0b1..Memory power status will be considered when determining slice power status.
 *  0b0..Memory power status will not be considered when determining slice power status.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_MEM_PWR_ST_EN(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_MEM_PWR_ST_EN_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_MEM_PWR_ST_EN_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_LOCK_CFG_MASK (0x100000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_LOCK_CFG_SHIFT (20U)
/*! LOCK_CFG - Configuration lock
 *  0b0..The fields are not locked.
 *  0b1..The fields are locked.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_LOCK_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_LOCK_CFG_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_CTRL_LOCK_CFG_MASK)
/*! @} */

/*! @name MIF_STAT - MIF Status */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_MLPL_STATE_MASK (0x7U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_MLPL_STATE_SHIFT (0U)
/*! MLPL_STATE - Current state of CURRENT_MLPL */
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_MLPL_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_STAT_MLPL_STATE_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_STAT_MLPL_STATE_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_LS_STATE_MASK (0x8U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_LS_STATE_SHIFT (3U)
/*! LS_STATE - Current state of LS
 *  0b0..LS is 0.
 *  0b1..LS is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_LS_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_STAT_LS_STATE_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_STAT_LS_STATE_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_HS_STATE_MASK (0x10U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_HS_STATE_SHIFT (4U)
/*! HS_STATE - Current state of HS
 *  0b0..HS is 0.
 *  0b1..HS is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_HS_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_STAT_HS_STATE_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_STAT_HS_STATE_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_IG_STATE_MASK (0x20U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_IG_STATE_SHIFT (5U)
/*! IG_STATE - Current state of IG
 *  0b0..IG is 0.
 *  0b1..IG is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_IG_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_STAT_IG_STATE_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_STAT_IG_STATE_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_STDBY_STATE_MASK (0x40U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_STDBY_STATE_SHIFT (6U)
/*! STDBY_STATE - Current state of STDBY
 *  0b0..STDBY is 0.
 *  0b1..STDBY is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_STDBY_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_STAT_STDBY_STATE_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_STAT_STDBY_STATE_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_SLEEP_STATE_MASK (0x80U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_SLEEP_STATE_SHIFT (7U)
/*! SLEEP_STATE - SLEEP status
 *  0b0..SLEEP is 0.
 *  0b1..SLEEP is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_SLEEP_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_STAT_SLEEP_STATE_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_STAT_SLEEP_STATE_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_ARR_HF_STATE_MASK (0x100U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_ARR_HF_STATE_SHIFT (8U)
/*! ARR_HF_STATE - ARR_HF_OFF status
 *  0b0..ARR_HS is 0.
 *  0b1..ARR_HS is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_STAT_ARR_HF_STATE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_STAT_ARR_HF_STATE_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_STAT_ARR_HF_STATE_MASK)
/*! @} */

/*! @name MIF_MLPL_LS - MIF MLPL control of LS */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_SW_LS_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_SW_LS_SHIFT (16U)
/*! SW_LS - Software control LS
 *  0b0..LS is 0.
 *  0b1..LS is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_SW_LS(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_SW_LS_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_LS_SW_LS_MASK)
/*! @} */

/*! @name MIF_DLY_LS - MIF Delay of LS */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_LS_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_HS - MIF MLPL control of HS */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_SW_HS_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_SW_HS_SHIFT (16U)
/*! SW_HS - software control HS
 *  0b0..HS is 0.
 *  0b1..HS is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_SW_HS(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_SW_HS_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_HS_SW_HS_MASK)
/*! @} */

/*! @name MIF_DLY_HS - MIF Delay of HS */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_HS_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_IG - MIF MLPL control of Input Gating (IG) */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_SW_IG_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_SW_IG_SHIFT (16U)
/*! SW_IG - Software control IG
 *  0b0..IG is 0.
 *  0b1..IG is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_SW_IG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_SW_IG_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_IG_SW_IG_MASK)
/*! @} */

/*! @name MIF_DLY_IG - MIF Delay of IG */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_IG_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_STDBY - MIF MLPL control of STDBY */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_SW_STDBY_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_SW_STDBY_SHIFT (16U)
/*! SW_STDBY - Software control STDBY
 *  0b0..STDBY is 0.
 *  0b1..STDBY is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_SW_STDBY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_SW_STDBY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_STDBY_SW_STDBY_MASK)
/*! @} */

/*! @name MIF_DLY_STDBY - MIF Delay of STDBY */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_STDBY_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_SLEEP - MIF MLPL control of SLEEP */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_SW_SLEEP_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_SW_SLEEP_SHIFT (16U)
/*! SW_SLEEP - Software control SLEEP
 *  0b0..SLEEP is 0.
 *  0b1..SLEEP is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_SW_SLEEP(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_SW_SLEEP_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_SLEEP_SW_SLEEP_MASK)
/*! @} */

/*! @name MIF_DLY_SLEEP - MIF Delay of SLEEP */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before asserting signal to high, locked by LOCK_CFG field. */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before de-asserting signal to low, locked by LOCK_CFG field. */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_SLEEP_PRE_LO_DLY_MASK)
/*! @} */

/*! @name MIF_MLPL_ARR_PDN - MIF MLPL control of array power down */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_MLPL_CTRL_MASK (0xFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_MLPL_CTRL_SHIFT (0U)
/*! MLPL_CTRL - Signal behavior at 8 different MLPL settings */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_MLPL_CTRL(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_MLPL_CTRL_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_MLPL_CTRL_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_SW_ARR_PDN_MASK (0x10000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_SW_ARR_PDN_SHIFT (16U)
/*! SW_ARR_PDN - Software control arr pdn
 *  0b0..ARR_PDN is 0.
 *  0b1..ARR_PDN is 1.
 */
#define SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_SW_ARR_PDN(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_SW_ARR_PDN_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_MLPL_ARR_PDN_SW_ARR_PDN_MASK)
/*! @} */

/*! @name MIF_DLY_ARR_HF - MIF Delay of array high-fanout power switch */
/*! @{ */

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_HI_DLY_MASK (0xFFFFU)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_HI_DLY_SHIFT (0U)
/*! PRE_HI_DLY - Delay before turn off the high-fanout power switch, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_HI_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_HI_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_HI_DLY_MASK)

#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_LO_DLY_MASK (0xFFFF0000U)
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_LO_DLY_SHIFT (16U)
/*! PRE_LO_DLY - Delay before turn on the high-fanout power switch, locked by LOCK_CFG field */
#define SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_LO_DLY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_LO_DLY_SHIFT)) & SRC_MIF_S28SPREGH_PSWA_MIF_DLY_ARR_HF_PRE_LO_DLY_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SRC_MIF_S28SPREGH_PSWA_Register_Masks */


/* SRC_MIF_S28SPREGH_PSWA - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral CM7PLATFORM_TCM base address */
  #define CM7PLATFORM_TCM_BASE                     (0x54464C00u)
  /** Peripheral CM7PLATFORM_TCM base address */
  #define CM7PLATFORM_TCM_BASE_NS                  (0x44464C00u)
  /** Peripheral CM7PLATFORM_TCM base pointer */
  #define CM7PLATFORM_TCM                          ((SRC_MIF_S28SPREGH_PSWA_Type *)CM7PLATFORM_TCM_BASE)
  /** Peripheral CM7PLATFORM_TCM base pointer */
  #define CM7PLATFORM_TCM_NS                       ((SRC_MIF_S28SPREGH_PSWA_Type *)CM7PLATFORM_TCM_BASE_NS)
  /** Array initializer of SRC_MIF_S28SPREGH_PSWA peripheral base addresses */
  #define SRC_MIF_S28SPREGH_PSWA_BASE_ADDRS        { CM7PLATFORM_TCM_BASE }
  /** Array initializer of SRC_MIF_S28SPREGH_PSWA peripheral base pointers */
  #define SRC_MIF_S28SPREGH_PSWA_BASE_PTRS         { CM7PLATFORM_TCM }
  /** Array initializer of SRC_MIF_S28SPREGH_PSWA peripheral base addresses */
  #define SRC_MIF_S28SPREGH_PSWA_BASE_ADDRS_NS     { CM7PLATFORM_TCM_BASE_NS }
  /** Array initializer of SRC_MIF_S28SPREGH_PSWA peripheral base pointers */
  #define SRC_MIF_S28SPREGH_PSWA_BASE_PTRS_NS      { CM7PLATFORM_TCM_NS }
#else
  /** Peripheral CM7PLATFORM_TCM base address */
  #define CM7PLATFORM_TCM_BASE                     (0x44464C00u)
  /** Peripheral CM7PLATFORM_TCM base pointer */
  #define CM7PLATFORM_TCM                          ((SRC_MIF_S28SPREGH_PSWA_Type *)CM7PLATFORM_TCM_BASE)
  /** Array initializer of SRC_MIF_S28SPREGH_PSWA peripheral base addresses */
  #define SRC_MIF_S28SPREGH_PSWA_BASE_ADDRS        { CM7PLATFORM_TCM_BASE }
  /** Array initializer of SRC_MIF_S28SPREGH_PSWA peripheral base pointers */
  #define SRC_MIF_S28SPREGH_PSWA_BASE_PTRS         { CM7PLATFORM_TCM }
#endif

/*!
 * @}
 */ /* end of group SRC_MIF_S28SPREGH_PSWA_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SRC_MIX_SLICE Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIX_SLICE_Peripheral_Access_Layer SRC_MIX_SLICE Peripheral Access Layer
 * @{
 */

/** SRC_MIX_SLICE - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t AUTHEN_CTRL;                       /**< Authentication Control, offset: 0x4 */
       uint8_t RESERVED_1[8];
  __IO uint32_t SLICE_SW_CTRL;                     /**< SLICE Software Control, offset: 0x10 */
  __I  uint32_t FUNC_STAT;                         /**< Function Status, offset: 0x14 */
       uint8_t RESERVED_2[8];
  __I  uint32_t UPI_STAT_0;                        /**< UPI Status 0, offset: 0x20 */
  __I  uint32_t UPI_STAT_1;                        /**< UPI Status 1, offset: 0x24 */
       uint8_t RESERVED_3[8];
  __IO uint32_t LPM_SETTING_0;                     /**< Low Power Mode Setting 0, offset: 0x30 */
  __IO uint32_t LPM_SETTING_1;                     /**< Low Power Mode Setting 1, offset: 0x34 */
  __IO uint32_t LPM_SETTING_2;                     /**< Low Power Mode Setting 2, offset: 0x38 */
       uint8_t RESERVED_4[4];
  __IO uint32_t EDGELOCK_HDSK_CTRL;                /**< Edgelock Handshake Control, offset: 0x40 */
  __I  uint32_t EDGELOCK_HDSK_STAT;                /**< Edgelock Handshake Status, offset: 0x44 */
  __IO uint32_t EDGELOCK_HDSK_CNT_CFG;             /**< Edgelock Handshake Counter Config, offset: 0x48 */
  __I  uint32_t EDGELOCK_HDSK_CNT_STAT;            /**< Edgelock Handshake Counter Status, offset: 0x4C */
  __IO uint32_t ISO_DLY_PRE;                       /**< ISO Delay Pre control, offset: 0x50 */
  __IO uint32_t PSW_DLY_PRE_HF;                    /**< PSW Delay Pre for HF, offset: 0x54 */
  __IO uint32_t PSW_DLY_PRE_LF;                    /**< PSW Delay Pre for LF, offset: 0x58 */
  __IO uint32_t PSW_CTRL;                          /**< PSW Control, offset: 0x5C */
  __I  uint32_t PSW_STAT;                          /**< PSW Status, offset: 0x60 */
  __IO uint32_t PSW_CNT_CFG_HF;                    /**< PSW Counter Config for HF, offset: 0x64 */
  __I  uint32_t PSW_CNT_STAT_HF;                   /**< PSW Counter Status for HF, offset: 0x68 */
  __IO uint32_t PSW_CNT_CFG_LF;                    /**< PSW Counter Config for LF, offset: 0x6C */
  __I  uint32_t PSW_CNT_STAT_LF;                   /**< PSW Counter Status for LF, offset: 0x70 */
       uint8_t RESERVED_5[12];
  __IO uint32_t MLPL_TRIG_CTRL;                    /**< Memory Low Power Level Trigger Control, offset: 0x80 */
  __IO uint32_t MLPL_CFG;                          /**< Memory Low Power Level Config, offset: 0x84 */
  __I  uint32_t MLPL_STAT;                         /**< Memory Low Power Level Status, offset: 0x88 */
} SRC_MIX_SLICE_Type;

/* ----------------------------------------------------------------------------
   -- SRC_MIX_SLICE Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SRC_MIX_SLICE_Register_Masks SRC_MIX_SLICE Register Masks
 * @{
 */

/*! @name AUTHEN_CTRL - Authentication Control */
/*! @{ */

#define SRC_MIX_SLICE_AUTHEN_CTRL_LPM_MODE_MASK  (0x4U)
#define SRC_MIX_SLICE_AUTHEN_CTRL_LPM_MODE_SHIFT (2U)
/*! LPM_MODE - HW low power mode
 *  0b0..Low power mode controlled by software. SLICE_SW_CTRL register can be updated. LPM_SETTING_0 register cannot be updated.
 *  0b1..Low power mode controlled by GPC. SLICE_SW_CTRL register cannot be updated. LPM_SETTING_0 register can be updated.
 */
#define SRC_MIX_SLICE_AUTHEN_CTRL_LPM_MODE(x)    (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_AUTHEN_CTRL_LPM_MODE_SHIFT)) & SRC_MIX_SLICE_AUTHEN_CTRL_LPM_MODE_MASK)

#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_CFG_MASK  (0x80U)
#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_CFG_SHIFT (7U)
/*! LOCK_CFG - Configuration lock
 *  0b0..Registers are not locked.
 *  0b1..Fields and registers in the list are locked.
 */
#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_CFG(x)    (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_CFG_SHIFT)) & SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_CFG_MASK)

#define SRC_MIX_SLICE_AUTHEN_CTRL_TZ_USER_MASK   (0x100U)
#define SRC_MIX_SLICE_AUTHEN_CTRL_TZ_USER_SHIFT  (8U)
/*! TZ_USER - Allow user mode write
 *  0b0..The MIX SLICE registers and its MEM Type I/II registers can only be written in privilege mode.
 *  0b1..The MIX SLICE registers and its MEM Type I/II registers can be written either in privilege mode or user mode.
 */
#define SRC_MIX_SLICE_AUTHEN_CTRL_TZ_USER(x)     (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_AUTHEN_CTRL_TZ_USER_SHIFT)) & SRC_MIX_SLICE_AUTHEN_CTRL_TZ_USER_MASK)

#define SRC_MIX_SLICE_AUTHEN_CTRL_TZ_NS_MASK     (0x200U)
#define SRC_MIX_SLICE_AUTHEN_CTRL_TZ_NS_SHIFT    (9U)
/*! TZ_NS - Allow non-secure mode access
 *  0b0..The MIX SLICE registers and its MEM Type I/II registers can only be written in secure mode.
 *  0b1..The MIX SLICE registers and its MEM Type I/II registers can be written either in secure mode or non-secure mode.
 */
#define SRC_MIX_SLICE_AUTHEN_CTRL_TZ_NS(x)       (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_AUTHEN_CTRL_TZ_NS_SHIFT)) & SRC_MIX_SLICE_AUTHEN_CTRL_TZ_NS_MASK)

#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_TZ_MASK   (0x800U)
#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_TZ_SHIFT  (11U)
/*! LOCK_TZ - Lock TZ_NS and TZ_USER
 *  0b0..TZ_NS and TZ_USER value can be changed.
 *  0b1..LOCK_TZ, TZ_NS and TZ_USER value cannot be changed.
 */
#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_TZ(x)     (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_TZ_SHIFT)) & SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_TZ_MASK)

#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_LIST_MASK (0x8000U)
#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_LIST_SHIFT (15U)
/*! LOCK_LIST - White list lock
 *  0b0..WHITE_LIST value can be changed.
 *  0b1..LOCK_LIST and WHITE_LIST value cannot be changed.
 */
#define SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_LIST(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_LIST_SHIFT)) & SRC_MIX_SLICE_AUTHEN_CTRL_LOCK_LIST_MASK)

#define SRC_MIX_SLICE_AUTHEN_CTRL_WHITE_LIST_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_AUTHEN_CTRL_WHITE_LIST_SHIFT (16U)
/*! WHITE_LIST - Domain ID white list
 *  0b0000000000000001..Core with domain ID=0 can write SRC MIX SLICE registers.
 *  0b0000000000000010..Core with domain ID=1 can write SRC MIX SLICE registers.
 *  0b0000000000000100..Core with domain ID=2 can write SRC MIX SLICE registers.
 *  0b0000000000001000..Core with domain ID=3 can write SRC MIX SLICE registers.
 *  0b0000000000010000..Core with domain ID=4 can write SRC MIX SLICE registers.
 *  0b0000000000100000..Core with domain ID=5 can write SRC MIX SLICE registers.
 *  0b0000000001000000..Core with domain ID=6 can write SRC MIX SLICE registers.
 *  0b0000000010000000..Core with domain ID=7 can write SRC MIX SLICE registers.
 *  0b0000000100000000..Core with domain ID=8 can write SRC MIX SLICE registers.
 *  0b0000001000000000..Core with domain ID=9 can write SRC MIX SLICE registers.
 *  0b0000010000000000..Core with domain ID=10 can write SRC MIX SLICE registers.
 *  0b0000100000000000..Core with domain ID=11 can write SRC MIX SLICE registers.
 *  0b0001000000000000..Core with domain ID=12 can write SRC MIX SLICE registers.
 *  0b0010000000000000..Core with domain ID=13 can write SRC MIX SLICE registers.
 *  0b0100000000000000..Core with domain ID=14 can write SRC MIX SLICE registers.
 *  0b1000000000000000..Core with domain ID=15 can write SRC MIX SLICE registers.
 */
#define SRC_MIX_SLICE_AUTHEN_CTRL_WHITE_LIST(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_AUTHEN_CTRL_WHITE_LIST_SHIFT)) & SRC_MIX_SLICE_AUTHEN_CTRL_WHITE_LIST_MASK)
/*! @} */

/*! @name SLICE_SW_CTRL - SLICE Software Control */
/*! @{ */

#define SRC_MIX_SLICE_SLICE_SW_CTRL_PSW_OFF_SOFT_MASK (0x1U)
#define SRC_MIX_SLICE_SLICE_SW_CTRL_PSW_OFF_SOFT_SHIFT (0U)
/*! PSW_OFF_SOFT - Software power off control. This field can only be updated while LPM_MODE=0.
 *  0b0..Clear to 0 to trigger power switch on
 *  0b1..Write 1 to trigger power switch off
 */
#define SRC_MIX_SLICE_SLICE_SW_CTRL_PSW_OFF_SOFT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_SLICE_SW_CTRL_PSW_OFF_SOFT_SHIFT)) & SRC_MIX_SLICE_SLICE_SW_CTRL_PSW_OFF_SOFT_MASK)

#define SRC_MIX_SLICE_SLICE_SW_CTRL_RST_CTRL_SOFT_MASK (0x4U)
#define SRC_MIX_SLICE_SLICE_SW_CTRL_RST_CTRL_SOFT_SHIFT (2U)
/*! RST_CTRL_SOFT - Software reset control. This field can only be updated while LPM_MODE=0.
 *  0b0..Clear to 0 to trigger reset deassert
 *  0b1..Write 1 to trigger reset assert
 */
#define SRC_MIX_SLICE_SLICE_SW_CTRL_RST_CTRL_SOFT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_SLICE_SW_CTRL_RST_CTRL_SOFT_SHIFT)) & SRC_MIX_SLICE_SLICE_SW_CTRL_RST_CTRL_SOFT_MASK)

#define SRC_MIX_SLICE_SLICE_SW_CTRL_ISO_ON_SOFT_MASK (0x10U)
#define SRC_MIX_SLICE_SLICE_SW_CTRL_ISO_ON_SOFT_SHIFT (4U)
/*! ISO_ON_SOFT - Software isolation on control. This field can only be updated while LPM_MODE=0.
 *  0b0..Clear to 0 to trigger isolation off
 *  0b1..Write 1 to trigger isolation on
 */
#define SRC_MIX_SLICE_SLICE_SW_CTRL_ISO_ON_SOFT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_SLICE_SW_CTRL_ISO_ON_SOFT_SHIFT)) & SRC_MIX_SLICE_SLICE_SW_CTRL_ISO_ON_SOFT_MASK)

#define SRC_MIX_SLICE_SLICE_SW_CTRL_EDGELOCK_HDSK_SOFT_MASK (0x40U)
#define SRC_MIX_SLICE_SLICE_SW_CTRL_EDGELOCK_HDSK_SOFT_SHIFT (6U)
/*! EDGELOCK_HDSK_SOFT - Software Edgelock handshake control. This field can only be updated while LPM_MODE=0.
 *  0b0..Clear to 0 to trigger Edgelock handshake for power on.
 *  0b1..Write to 1 to trigger Edgelock handshake for power off.
 */
#define SRC_MIX_SLICE_SLICE_SW_CTRL_EDGELOCK_HDSK_SOFT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_SLICE_SW_CTRL_EDGELOCK_HDSK_SOFT_SHIFT)) & SRC_MIX_SLICE_SLICE_SW_CTRL_EDGELOCK_HDSK_SOFT_MASK)

#define SRC_MIX_SLICE_SLICE_SW_CTRL_PDN_SOFT_MASK (0x80000000U)
#define SRC_MIX_SLICE_SLICE_SW_CTRL_PDN_SOFT_SHIFT (31U)
/*! PDN_SOFT - By flow sequence, Software power down sequence includes Edgelock handshake for power
 *    down, isolation on, reset assert, power off; Software power up sequence includes power up,
 *    reset deassert, isolation off, Edgelock handshake for power up. This bit can only be updated
 *    while LPM_MODE=0.
 *  0b0..Clear to 0 to trigger a power up sequence.
 *  0b1..Write 1 to trigger a power down sequence.
 */
#define SRC_MIX_SLICE_SLICE_SW_CTRL_PDN_SOFT(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_SLICE_SW_CTRL_PDN_SOFT_SHIFT)) & SRC_MIX_SLICE_SLICE_SW_CTRL_PDN_SOFT_MASK)
/*! @} */

/*! @name FUNC_STAT - Function Status */
/*! @{ */

#define SRC_MIX_SLICE_FUNC_STAT_PSW_STAT_MASK    (0x1U)
#define SRC_MIX_SLICE_FUNC_STAT_PSW_STAT_SHIFT   (0U)
/*! PSW_STAT - power switch status
 *  0b0..power switch on
 *  0b1..power switch off
 */
#define SRC_MIX_SLICE_FUNC_STAT_PSW_STAT(x)      (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_FUNC_STAT_PSW_STAT_SHIFT)) & SRC_MIX_SLICE_FUNC_STAT_PSW_STAT_MASK)

#define SRC_MIX_SLICE_FUNC_STAT_RST_STAT_MASK    (0x4U)
#define SRC_MIX_SLICE_FUNC_STAT_RST_STAT_SHIFT   (2U)
/*! RST_STAT - reset status
 *  0b0..reset asserted
 *  0b1..reset released
 */
#define SRC_MIX_SLICE_FUNC_STAT_RST_STAT(x)      (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_FUNC_STAT_RST_STAT_SHIFT)) & SRC_MIX_SLICE_FUNC_STAT_RST_STAT_MASK)

#define SRC_MIX_SLICE_FUNC_STAT_ISO_STAT_MASK    (0x10U)
#define SRC_MIX_SLICE_FUNC_STAT_ISO_STAT_SHIFT   (4U)
/*! ISO_STAT - isolation status
 *  0b0..isolation off
 *  0b1..isolation on
 */
#define SRC_MIX_SLICE_FUNC_STAT_ISO_STAT(x)      (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_FUNC_STAT_ISO_STAT_SHIFT)) & SRC_MIX_SLICE_FUNC_STAT_ISO_STAT_MASK)

#define SRC_MIX_SLICE_FUNC_STAT_EDGELOCK_HDSK_STAT_MASK (0x40U)
#define SRC_MIX_SLICE_FUNC_STAT_EDGELOCK_HDSK_STAT_SHIFT (6U)
/*! EDGELOCK_HDSK_STAT - Edgelock handshake status
 *  0b0..no effect or power up handshake with Edgelock done
 *  0b1..power down handshake with Edgelock done
 */
#define SRC_MIX_SLICE_FUNC_STAT_EDGELOCK_HDSK_STAT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_FUNC_STAT_EDGELOCK_HDSK_STAT_SHIFT)) & SRC_MIX_SLICE_FUNC_STAT_EDGELOCK_HDSK_STAT_MASK)
/*! @} */

/*! @name UPI_STAT_0 - UPI Status 0 */
/*! @{ */

#define SRC_MIX_SLICE_UPI_STAT_0_UPI_ISO_REQUEST_MASK (0xFFFFU)
#define SRC_MIX_SLICE_UPI_STAT_0_UPI_ISO_REQUEST_SHIFT (0U)
/*! UPI_ISO_REQUEST - CPU mode transfer to trigger isolation change request of 16 domains */
#define SRC_MIX_SLICE_UPI_STAT_0_UPI_ISO_REQUEST(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_UPI_STAT_0_UPI_ISO_REQUEST_SHIFT)) & SRC_MIX_SLICE_UPI_STAT_0_UPI_ISO_REQUEST_MASK)

#define SRC_MIX_SLICE_UPI_STAT_0_UPI_EDGELOCK_HDSK_REQUEST_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_UPI_STAT_0_UPI_EDGELOCK_HDSK_REQUEST_SHIFT (16U)
/*! UPI_EDGELOCK_HDSK_REQUEST - CPU mode transfer to trigger Edgelock handshake request of 16 domains */
#define SRC_MIX_SLICE_UPI_STAT_0_UPI_EDGELOCK_HDSK_REQUEST(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_UPI_STAT_0_UPI_EDGELOCK_HDSK_REQUEST_SHIFT)) & SRC_MIX_SLICE_UPI_STAT_0_UPI_EDGELOCK_HDSK_REQUEST_MASK)
/*! @} */

/*! @name UPI_STAT_1 - UPI Status 1 */
/*! @{ */

#define SRC_MIX_SLICE_UPI_STAT_1_UPI_POWER_REQUEST_MASK (0xFFFFU)
#define SRC_MIX_SLICE_UPI_STAT_1_UPI_POWER_REQUEST_SHIFT (0U)
/*! UPI_POWER_REQUEST - CPU mode transfer to trigger power switch request of 16 domains */
#define SRC_MIX_SLICE_UPI_STAT_1_UPI_POWER_REQUEST(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_UPI_STAT_1_UPI_POWER_REQUEST_SHIFT)) & SRC_MIX_SLICE_UPI_STAT_1_UPI_POWER_REQUEST_MASK)

#define SRC_MIX_SLICE_UPI_STAT_1_UPI_RESET_REQUEST_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_UPI_STAT_1_UPI_RESET_REQUEST_SHIFT (16U)
/*! UPI_RESET_REQUEST - CPU mode transfer to trigger reset change request of 16 domains */
#define SRC_MIX_SLICE_UPI_STAT_1_UPI_RESET_REQUEST(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_UPI_STAT_1_UPI_RESET_REQUEST_SHIFT)) & SRC_MIX_SLICE_UPI_STAT_1_UPI_RESET_REQUEST_MASK)
/*! @} */

/*! @name LPM_SETTING_0 - Low Power Mode Setting 0 */
/*! @{ */

#define SRC_MIX_SLICE_LPM_SETTING_0_LPM_SETTING_CD_MASK (0x7U)
#define SRC_MIX_SLICE_LPM_SETTING_0_LPM_SETTING_CD_SHIFT (0U)
/*! LPM_SETTING_CD - LPM setting of current domain */
#define SRC_MIX_SLICE_LPM_SETTING_0_LPM_SETTING_CD(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_0_LPM_SETTING_CD_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_0_LPM_SETTING_CD_MASK)
/*! @} */

/*! @name LPM_SETTING_1 - Low Power Mode Setting 1 */
/*! @{ */

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D0_MASK (0x7U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D0_SHIFT (0U)
/*! LPM_SETTING_D0 - LPM setting of domain 0
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D0(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D0_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D0_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D1_MASK (0x70U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D1_SHIFT (4U)
/*! LPM_SETTING_D1 - LPM setting of domain 1
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D1(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D1_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D1_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D2_MASK (0x700U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D2_SHIFT (8U)
/*! LPM_SETTING_D2 - LPM setting of domain 2
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D2(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D2_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D2_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D3_MASK (0x7000U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D3_SHIFT (12U)
/*! LPM_SETTING_D3 - LPM setting of domain 3
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D3(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D3_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D3_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D4_MASK (0x70000U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D4_SHIFT (16U)
/*! LPM_SETTING_D4 - LPM setting of domain 4
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D4(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D4_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D4_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D5_MASK (0x700000U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D5_SHIFT (20U)
/*! LPM_SETTING_D5 - LPM setting of domain 5
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D5(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D5_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D5_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D6_MASK (0x7000000U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D6_SHIFT (24U)
/*! LPM_SETTING_D6 - LPM setting of domain 6
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D6(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D6_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D6_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D7_MASK (0x70000000U)
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D7_SHIFT (28U)
/*! LPM_SETTING_D7 - LPM setting of domain 7
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D7(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D7_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_1_LPM_SETTING_D7_MASK)
/*! @} */

/*! @name LPM_SETTING_2 - Low Power Mode Setting 2 */
/*! @{ */

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D8_MASK (0x7U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D8_SHIFT (0U)
/*! LPM_SETTING_D8 - LPM setting of domain 8
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D8(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D8_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D8_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D9_MASK (0x70U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D9_SHIFT (4U)
/*! LPM_SETTING_D9 - LPM setting of domain 9
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D9(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D9_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D9_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D10_MASK (0x700U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D10_SHIFT (8U)
/*! LPM_SETTING_D10 - LPM setting of domain 10
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D10(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D10_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D10_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D11_MASK (0x7000U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D11_SHIFT (12U)
/*! LPM_SETTING_D11 - LPM setting of domain 11
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D11(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D11_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D11_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D12_MASK (0x70000U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D12_SHIFT (16U)
/*! LPM_SETTING_D12 - LPM setting of domain 12
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D12(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D12_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D12_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D13_MASK (0x700000U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D13_SHIFT (20U)
/*! LPM_SETTING_D13 - LPM setting of domain 13
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D13(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D13_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D13_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D14_MASK (0x7000000U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D14_SHIFT (24U)
/*! LPM_SETTING_D14 - LPM setting of domain 14
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D14(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D14_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D14_MASK)

#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D15_MASK (0x70000000U)
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D15_SHIFT (28U)
/*! LPM_SETTING_D15 - LPM setting of domain 15
 *  0b000..Not used. Do not write this value.
 *  0b001..power on when domain n is in RUN, off in WAIT/STOP/SUSPEND
 *  0b010..power on when domain n is in RUN/WAIT, off in STOP/SUSPEND
 *  0b011..power on when domain n is in RUN/WAIT/STOP, off in SUSPEND
 *  0b100-0b111..power always on
 */
#define SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D15(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D15_SHIFT)) & SRC_MIX_SLICE_LPM_SETTING_2_LPM_SETTING_D15_MASK)
/*! @} */

/*! @name EDGELOCK_HDSK_CTRL - Edgelock Handshake Control */
/*! @{ */

#define SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_PON_CNT_MODE_MASK (0xC000U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_PON_CNT_MODE_SHIFT (14U)
/*! PON_CNT_MODE - Configures the acknowledge counter for power up working mode, locked by LOCK_CFG field
 *  0b00..Not use counter, raise Edgelock handshake done to GPC once get Edgelock ack
 *  0b01..Delay after receiving Edgelock ack, delay cycle number is PON_CNT_CFG
 *  0b10..Ignore Edgelock ack, raise Edgelock handshake done to GPC when counting to PON_CNT_CFG value
 *  0b11..Time out mode, raise Edgelock handshake done to GPC when either Edgelock ack received or counting to PON_CNT_CFG value
 */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_PON_CNT_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_PON_CNT_MODE_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_PON_CNT_MODE_MASK)

#define SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_POFF_CNT_MODE_MASK (0xC0000000U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_POFF_CNT_MODE_SHIFT (30U)
/*! POFF_CNT_MODE - Configures the acknowledge counter for power down working mode, locked by LOCK_CFG field
 *  0b00..Not use counter, raise Edgelock handshake done to GPC once get Edgelock ack
 *  0b01..Delay after receiving Edgelock ack, delay cycle number is POFF_CNT_CFG
 *  0b10..Ignore Edgelock ack, raise Edgelock handshake done to GPC when counting to POFF_CNT_CFG value
 *  0b11..Time out mode, raise Edgelock handshake done to GPC when either Edgelock ack received or counting to POFF_CNT_CFG value
 */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_POFF_CNT_MODE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_POFF_CNT_MODE_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_CTRL_POFF_CNT_MODE_MASK)
/*! @} */

/*! @name EDGELOCK_HDSK_STAT - Edgelock Handshake Status */
/*! @{ */

#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_ACK_MASK (0x1U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_ACK_SHIFT (0U)
/*! PON_ACK - Indicates this mix power up sequence has accepted Edgelock ack
 *  0b0..Does not get Edgelock ack for power up.
 *  0b1..Gets Edgelock ack for power up.
 */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_ACK(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_ACK_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_ACK_MASK)

#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_BUSY_MASK (0x2U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_BUSY_SHIFT (1U)
/*! PON_BUSY - Busy requesting Edgelock handshake for power up
 *  0b0..Does not send Edgelock handshake for power up.
 *  0b1..Sends Edgelock handshake for power up.
 */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_BUSY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_BUSY_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_PON_BUSY_MASK)

#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_ACK_MASK (0x10000U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_ACK_SHIFT (16U)
/*! POFF_ACK - Indicates this mix power down sequence has accepted Edgelock ack
 *  0b0..Does not get Edgelock ack for power down.
 *  0b1..Gets Edgelock ack for power down.
 */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_ACK(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_ACK_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_ACK_MASK)

#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_BUSY_MASK (0x20000U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_BUSY_SHIFT (17U)
/*! POFF_BUSY - Busy requesting Edgelock handshake for power down
 *  0b1..Sends Edgelock handshake for power down.
 *  0b0..Does not send Edgelock handshake for power down.
 */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_BUSY(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_BUSY_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_STAT_POFF_BUSY_MASK)
/*! @} */

/*! @name EDGELOCK_HDSK_CNT_CFG - Edgelock Handshake Counter Config */
/*! @{ */

#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_PON_CNT_CFG_MASK (0xFFFFU)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_PON_CNT_CFG_SHIFT (0U)
/*! PON_CNT_CFG - Edgelock handshake ack count for power up config: usage depends on CNT_MODE, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_PON_CNT_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_PON_CNT_CFG_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_PON_CNT_CFG_MASK)

#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_POFF_CNT_CFG_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_POFF_CNT_CFG_SHIFT (16U)
/*! POFF_CNT_CFG - Edgelock handshake ack count for power down config: usage depends on CNT_MODE, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_POFF_CNT_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_POFF_CNT_CFG_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_CFG_POFF_CNT_CFG_MASK)
/*! @} */

/*! @name EDGELOCK_HDSK_CNT_STAT - Edgelock Handshake Counter Status */
/*! @{ */

#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_PON_CNT_MASK (0xFFFFU)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_PON_CNT_SHIFT (0U)
/*! PON_CNT - Edgelock handshake for power up acknowledge count, record the delay from stat change to acknowledge received */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_PON_CNT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_PON_CNT_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_PON_CNT_MASK)

#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_POFF_CNT_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_POFF_CNT_SHIFT (16U)
/*! POFF_CNT - Edgelock handshake for power down acknowledge count, record the delay from stat change to acknowledge received */
#define SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_POFF_CNT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_POFF_CNT_SHIFT)) & SRC_MIX_SLICE_EDGELOCK_HDSK_CNT_STAT_POFF_CNT_MASK)
/*! @} */

/*! @name ISO_DLY_PRE - ISO Delay Pre control */
/*! @{ */

#define SRC_MIX_SLICE_ISO_DLY_PRE_ISO_OFF_CNT_MASK (0xFFFFU)
#define SRC_MIX_SLICE_ISO_DLY_PRE_ISO_OFF_CNT_SHIFT (0U)
/*! ISO_OFF_CNT - Delay from receiving iso off request to isolation disable, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_ISO_DLY_PRE_ISO_OFF_CNT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_ISO_DLY_PRE_ISO_OFF_CNT_SHIFT)) & SRC_MIX_SLICE_ISO_DLY_PRE_ISO_OFF_CNT_MASK)

#define SRC_MIX_SLICE_ISO_DLY_PRE_ISO_ON_CNT_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_ISO_DLY_PRE_ISO_ON_CNT_SHIFT (16U)
/*! ISO_ON_CNT - Delay from receiving iso_on request to isolation enable, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_ISO_DLY_PRE_ISO_ON_CNT(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_ISO_DLY_PRE_ISO_ON_CNT_SHIFT)) & SRC_MIX_SLICE_ISO_DLY_PRE_ISO_ON_CNT_MASK)
/*! @} */

/*! @name PSW_DLY_PRE_HF - PSW Delay Pre for HF */
/*! @{ */

#define SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_ON_MASK (0xFFFFU)
#define SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_ON_SHIFT (0U)
/*! PSW_ON - Delay from receiving power on hf request to power switch turn on, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_ON(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_ON_SHIFT)) & SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_ON_MASK)

#define SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_OFF_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_OFF_SHIFT (16U)
/*! PSW_OFF - Delay from receiving power off hf request to power switch shut off, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_OFF(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_OFF_SHIFT)) & SRC_MIX_SLICE_PSW_DLY_PRE_HF_PSW_OFF_MASK)
/*! @} */

/*! @name PSW_DLY_PRE_LF - PSW Delay Pre for LF */
/*! @{ */

#define SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_ON_MASK (0xFFFFU)
#define SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_ON_SHIFT (0U)
/*! PSW_ON - Delay from receiving power on lf request to power switch turn on, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_ON(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_ON_SHIFT)) & SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_ON_MASK)

#define SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_OFF_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_OFF_SHIFT (16U)
/*! PSW_OFF - Delay from receiving power off lf request to power switch shut off, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_OFF(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_OFF_SHIFT)) & SRC_MIX_SLICE_PSW_DLY_PRE_LF_PSW_OFF_MASK)
/*! @} */

/*! @name PSW_CTRL - PSW Control */
/*! @{ */

#define SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_LF_MASK (0x1U)
#define SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_LF_SHIFT (0U)
/*! ACK_INVERT_LF - Acknowledge for low fanout value is inverted from power switch control, locked by LOCK_CFG field
 *  0b0..Use original signal as ack
 *  0b1..Use inverted original signal as ack
 */
#define SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_LF(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_LF_SHIFT)) & SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_LF_MASK)

#define SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_HF_MASK (0x2U)
#define SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_HF_SHIFT (1U)
/*! ACK_INVERT_HF - Acknowledge for high fanout value is inverted from power switch control, locked by LOCK_CFG field
 *  0b0..Use original signal as ack
 *  0b1..Use inverted original signal as ack
 */
#define SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_HF(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_HF_SHIFT)) & SRC_MIX_SLICE_PSW_CTRL_ACK_INVERT_HF_MASK)

#define SRC_MIX_SLICE_PSW_CTRL_PON_CNT_MODE_MASK (0xC000U)
#define SRC_MIX_SLICE_PSW_CTRL_PON_CNT_MODE_SHIFT (14U)
/*! PON_CNT_MODE - Configure the acknowledge counter for power up working mode, locked by LOCK_CFG field
 *  0b00..Not use counter, raise power_on/off done to GPC once get psw ack
 *  0b01..Delay after receiving psw ack, delay cycle number is PON_CNT_CFG
 *  0b10..Ignore psw ack, raise power_on/off done to GPC when counting to PON_CNT_CFG value
 *  0b11..Time out mode, raise power_on/off done to GPC when either psw ack received or counting to PON_CNT_CFG value
 */
#define SRC_MIX_SLICE_PSW_CTRL_PON_CNT_MODE(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CTRL_PON_CNT_MODE_SHIFT)) & SRC_MIX_SLICE_PSW_CTRL_PON_CNT_MODE_MASK)

#define SRC_MIX_SLICE_PSW_CTRL_POFF_CNT_MODE_MASK (0xC0000000U)
#define SRC_MIX_SLICE_PSW_CTRL_POFF_CNT_MODE_SHIFT (30U)
/*! POFF_CNT_MODE - Configures the acknowledge counter for power down working mode, locked by LOCK_CFG field
 *  0b00..Not use counter, raise power_on/off done to GPC once get psw ack
 *  0b01..Delay after receiving psw ack, delay cycle number is POFF_CNT_CFG. When POFF_CNT_CFG value in HF/LF is different, bigger value will be used.
 *  0b10..Ignore psw ack, raise power_on/off done to GPC when counting to POFF_CNT_CFG value. When POFF_CNT_CFG
 *        value in HF/LF is different, bigger value will be used.
 *  0b11..Time out mode, raise power_on/off done to GPC when either psw ack received or counting to POFF_CNT_CFG
 *        value. When POFF_CNT_CFG value in HF/LF is different, bigger value will be used.
 */
#define SRC_MIX_SLICE_PSW_CTRL_POFF_CNT_MODE(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CTRL_POFF_CNT_MODE_SHIFT)) & SRC_MIX_SLICE_PSW_CTRL_POFF_CNT_MODE_MASK)
/*! @} */

/*! @name PSW_STAT - PSW Status */
/*! @{ */

#define SRC_MIX_SLICE_PSW_STAT_ACK_STAT_LF_MASK  (0x1U)
#define SRC_MIX_SLICE_PSW_STAT_ACK_STAT_LF_SHIFT (0U)
/*! ACK_STAT_LF - Power switch acknowledge status for low fanout
 *  0b0..Does not get ack for power switch
 *  0b1..Gets ack for power switch
 */
#define SRC_MIX_SLICE_PSW_STAT_ACK_STAT_LF(x)    (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_STAT_ACK_STAT_LF_SHIFT)) & SRC_MIX_SLICE_PSW_STAT_ACK_STAT_LF_MASK)

#define SRC_MIX_SLICE_PSW_STAT_ACK_STAT_HF_MASK  (0x2U)
#define SRC_MIX_SLICE_PSW_STAT_ACK_STAT_HF_SHIFT (1U)
/*! ACK_STAT_HF - Power switch acknowledge status for high fanout
 *  0b0..Does not get ack for power switch
 *  0b1..Gets ack for power switch
 */
#define SRC_MIX_SLICE_PSW_STAT_ACK_STAT_HF(x)    (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_STAT_ACK_STAT_HF_SHIFT)) & SRC_MIX_SLICE_PSW_STAT_ACK_STAT_HF_MASK)
/*! @} */

/*! @name PSW_CNT_CFG_HF - PSW Counter Config for HF */
/*! @{ */

#define SRC_MIX_SLICE_PSW_CNT_CFG_HF_PON_CNT_CFG_MASK (0xFFFFU)
#define SRC_MIX_SLICE_PSW_CNT_CFG_HF_PON_CNT_CFG_SHIFT (0U)
/*! PON_CNT_CFG - PUP HF Count config: usage depends on CNT_MODE, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_CNT_CFG_HF_PON_CNT_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_CFG_HF_PON_CNT_CFG_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_CFG_HF_PON_CNT_CFG_MASK)

#define SRC_MIX_SLICE_PSW_CNT_CFG_HF_POFF_CNT_CFG_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_PSW_CNT_CFG_HF_POFF_CNT_CFG_SHIFT (16U)
/*! POFF_CNT_CFG - PDN HF Count config: usage depends on CNT_MODE, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_CNT_CFG_HF_POFF_CNT_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_CFG_HF_POFF_CNT_CFG_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_CFG_HF_POFF_CNT_CFG_MASK)
/*! @} */

/*! @name PSW_CNT_STAT_HF - PSW Counter Status for HF */
/*! @{ */

#define SRC_MIX_SLICE_PSW_CNT_STAT_HF_PON_CNT_MASK (0xFFFFU)
#define SRC_MIX_SLICE_PSW_CNT_STAT_HF_PON_CNT_SHIFT (0U)
/*! PON_CNT - HF PSW acknowledge count for power up, record the delay from power switch change to acknowledge received */
#define SRC_MIX_SLICE_PSW_CNT_STAT_HF_PON_CNT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_STAT_HF_PON_CNT_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_STAT_HF_PON_CNT_MASK)

#define SRC_MIX_SLICE_PSW_CNT_STAT_HF_POFF_CNT_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_PSW_CNT_STAT_HF_POFF_CNT_SHIFT (16U)
/*! POFF_CNT - HF PSW acknowledge count for power down, record the delay from power switch change to acknowledge received */
#define SRC_MIX_SLICE_PSW_CNT_STAT_HF_POFF_CNT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_STAT_HF_POFF_CNT_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_STAT_HF_POFF_CNT_MASK)
/*! @} */

/*! @name PSW_CNT_CFG_LF - PSW Counter Config for LF */
/*! @{ */

#define SRC_MIX_SLICE_PSW_CNT_CFG_LF_PON_CNT_CFG_MASK (0xFFFFU)
#define SRC_MIX_SLICE_PSW_CNT_CFG_LF_PON_CNT_CFG_SHIFT (0U)
/*! PON_CNT_CFG - PUP LF Count config: usage depends on CNT_MODE, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_CNT_CFG_LF_PON_CNT_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_CFG_LF_PON_CNT_CFG_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_CFG_LF_PON_CNT_CFG_MASK)

#define SRC_MIX_SLICE_PSW_CNT_CFG_LF_POFF_CNT_CFG_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_PSW_CNT_CFG_LF_POFF_CNT_CFG_SHIFT (16U)
/*! POFF_CNT_CFG - PDN LF Count config: usage depends on CNT_MODE, locked by LOCK_CFG field */
#define SRC_MIX_SLICE_PSW_CNT_CFG_LF_POFF_CNT_CFG(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_CFG_LF_POFF_CNT_CFG_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_CFG_LF_POFF_CNT_CFG_MASK)
/*! @} */

/*! @name PSW_CNT_STAT_LF - PSW Counter Status for LF */
/*! @{ */

#define SRC_MIX_SLICE_PSW_CNT_STAT_LF_PON_CNT_MASK (0xFFFFU)
#define SRC_MIX_SLICE_PSW_CNT_STAT_LF_PON_CNT_SHIFT (0U)
/*! PON_CNT - LF PSW acknowledge count for power up, record the delay from power switch change to acknowledge received */
#define SRC_MIX_SLICE_PSW_CNT_STAT_LF_PON_CNT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_STAT_LF_PON_CNT_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_STAT_LF_PON_CNT_MASK)

#define SRC_MIX_SLICE_PSW_CNT_STAT_LF_POFF_CNT_MASK (0xFFFF0000U)
#define SRC_MIX_SLICE_PSW_CNT_STAT_LF_POFF_CNT_SHIFT (16U)
/*! POFF_CNT - LF PSW acknowledge count for power down, record the delay from power switch change to acknowledge received */
#define SRC_MIX_SLICE_PSW_CNT_STAT_LF_POFF_CNT(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_PSW_CNT_STAT_LF_POFF_CNT_SHIFT)) & SRC_MIX_SLICE_PSW_CNT_STAT_LF_POFF_CNT_MASK)
/*! @} */

/*! @name MLPL_TRIG_CTRL - Memory Low Power Level Trigger Control */
/*! @{ */

#define SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_LMEM_MLPL_TRANS_MASK (0x10000U)
#define SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_LMEM_MLPL_TRANS_SHIFT (16U)
/*! SW_TRIG_LMEM_MLPL_TRANS - Software trigger local memory (MEM Type I with PSW or MEM Type I
 *    except for CM33PLATFORM_CACHE or CM7PLATFORM_CACHE, depending on which MIX of the register) update
 *    low power level
 *  0b0..No trigger
 *  0b1..Software trigger local memory update low power level
 */
#define SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_LMEM_MLPL_TRANS(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_LMEM_MLPL_TRANS_SHIFT)) & SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_LMEM_MLPL_TRANS_MASK)

#define SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_CACHE_MLPL_TRANS_MASK (0x20000U)
#define SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_CACHE_MLPL_TRANS_SHIFT (17U)
/*! SW_TRIG_CACHE_MLPL_TRANS - Software trigger CACHE memory (MEM Type II or MEM Type I's
 *    CM33PLATFORM_CACHE or CM7PLATFORM_CACHE, depending on which MIX of the register) update low power level
 *  0b0..No trigger
 *  0b1..Software trigger CACHE memory update low power level
 */
#define SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_CACHE_MLPL_TRANS(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_CACHE_MLPL_TRANS_SHIFT)) & SRC_MIX_SLICE_MLPL_TRIG_CTRL_SW_TRIG_CACHE_MLPL_TRANS_MASK)
/*! @} */

/*! @name MLPL_CFG - Memory Low Power Level Config */
/*! @{ */

#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_LMEM_MASK (0x7U)
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_LMEM_SHIFT (0U)
/*! MLPL_HW_PDN_LMEM - Memory(MEM Type I with PSW or MEM Type I except for CM33PLATFORM_CACHE or
 *    CM7PLATFORM_CACHE, depending on which MIX of the register) low power level when hw power down
 */
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_LMEM(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_LMEM_SHIFT)) & SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_LMEM_MASK)

#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_LMEM_MASK (0x70U)
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_LMEM_SHIFT (4U)
/*! MLPL_HW_PUP_LMEM - Memory(MEM Type I with PSW or MEM Type I except for CM33PLATFORM_CACHE or
 *    CM7PLATFORM_CACHE, depending on which MIX of the register) low power level when hw power up
 */
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_LMEM(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_LMEM_SHIFT)) & SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_LMEM_MASK)

#define SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_LMEM_MASK (0x7000U)
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_LMEM_SHIFT (12U)
/*! MLPL_SW_LMEM - Memory(MEM Type I with PSW or MEM Type I except for CM33PLATFORM_CACHE or
 *    CM7PLATFORM_CACHE, depending on which MIX of the register) low power level when sw trigger memory
 *    update power level
 */
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_LMEM(x)   (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_LMEM_SHIFT)) & SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_LMEM_MASK)

#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_CACHE_MASK (0x70000U)
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_CACHE_SHIFT (16U)
/*! MLPL_HW_PDN_CACHE - Memory(MEM Type II or MEM Type I's CM33PLATFORM_CACHE or CM7PLATFORM_CACHE,
 *    depending on which MIX of the register) low power level when hw power down
 */
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_CACHE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_CACHE_SHIFT)) & SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PDN_CACHE_MASK)

#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_CACHE_MASK (0x700000U)
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_CACHE_SHIFT (20U)
/*! MLPL_HW_PUP_CACHE - Memory(MEM Type II or MEM Type I's CM33PLATFORM_CACHE or CM7PLATFORM_CACHE,
 *    depending on which MIX of the register) low power level when hw power up
 */
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_CACHE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_CACHE_SHIFT)) & SRC_MIX_SLICE_MLPL_CFG_MLPL_HW_PUP_CACHE_MASK)

#define SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_CACHE_MASK (0x70000000U)
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_CACHE_SHIFT (28U)
/*! MLPL_SW_CACHE - Memory(MEM Type II or MEM Type I's CM33PLATFORM_CACHE or CM7PLATFORM_CACHE,
 *    depending on which MIX of the register) low power level when sw trigger memory update power level
 */
#define SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_CACHE(x)  (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_CACHE_SHIFT)) & SRC_MIX_SLICE_MLPL_CFG_MLPL_SW_CACHE_MASK)
/*! @} */

/*! @name MLPL_STAT - Memory Low Power Level Status */
/*! @{ */

#define SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_LMEM_MASK (0x7U)
#define SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_LMEM_SHIFT (0U)
/*! CURRENT_MLPL_LMEM - Current memory (MEM Type I with PSW or MEM Type I except for
 *    CM33PLATFORM_CACHE or CM7PLATFORM_CACHE, depending on which MIX of the register) low power level
 */
#define SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_LMEM(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_LMEM_SHIFT)) & SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_LMEM_MASK)

#define SRC_MIX_SLICE_MLPL_STAT_LMEM_BUSY_MASK   (0x8U)
#define SRC_MIX_SLICE_MLPL_STAT_LMEM_BUSY_SHIFT  (3U)
/*! LMEM_BUSY - Busy requesting low power level of memory (MEM Type I with PSW or MEM Type I except
 *    for CM33PLATFORM_CACHE or CM7PLATFORM_CACHE, depending on which MIX of the register)
 */
#define SRC_MIX_SLICE_MLPL_STAT_LMEM_BUSY(x)     (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_STAT_LMEM_BUSY_SHIFT)) & SRC_MIX_SLICE_MLPL_STAT_LMEM_BUSY_MASK)

#define SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_CACHE_MASK (0x70U)
#define SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_CACHE_SHIFT (4U)
/*! CURRENT_MLPL_CACHE - Current memory (MEM Type II or MEM Type I's CM33PLATFORM_CACHE or
 *    CM7PLATFORM_CACHE, depending on which MIX of the register) low power level
 */
#define SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_CACHE(x) (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_CACHE_SHIFT)) & SRC_MIX_SLICE_MLPL_STAT_CURRENT_MLPL_CACHE_MASK)

#define SRC_MIX_SLICE_MLPL_STAT_CACHE_BUSY_MASK  (0x80U)
#define SRC_MIX_SLICE_MLPL_STAT_CACHE_BUSY_SHIFT (7U)
/*! CACHE_BUSY - Busy requesting low power level of memory (MEM Type II or MEM Type I's
 *    CM33PLATFORM_CACHE or CM7PLATFORM_CACHE, depending on which MIX of the register)
 */
#define SRC_MIX_SLICE_MLPL_STAT_CACHE_BUSY(x)    (((uint32_t)(((uint32_t)(x)) << SRC_MIX_SLICE_MLPL_STAT_CACHE_BUSY_SHIFT)) & SRC_MIX_SLICE_MLPL_STAT_CACHE_BUSY_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SRC_MIX_SLICE_Register_Masks */


/* SRC_MIX_SLICE - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral AON_MIX_SLICE base address */
  #define AON_MIX_SLICE_BASE                       (0x54460800u)
  /** Peripheral AON_MIX_SLICE base address */
  #define AON_MIX_SLICE_BASE_NS                    (0x44460800u)
  /** Peripheral AON_MIX_SLICE base pointer */
  #define AON_MIX_SLICE                            ((SRC_MIX_SLICE_Type *)AON_MIX_SLICE_BASE)
  /** Peripheral AON_MIX_SLICE base pointer */
  #define AON_MIX_SLICE_NS                         ((SRC_MIX_SLICE_Type *)AON_MIX_SLICE_BASE_NS)
  /** Peripheral WAKEUP_MIX_SLICE base address */
  #define WAKEUP_MIX_SLICE_BASE                    (0x54460C00u)
  /** Peripheral WAKEUP_MIX_SLICE base address */
  #define WAKEUP_MIX_SLICE_BASE_NS                 (0x44460C00u)
  /** Peripheral WAKEUP_MIX_SLICE base pointer */
  #define WAKEUP_MIX_SLICE                         ((SRC_MIX_SLICE_Type *)WAKEUP_MIX_SLICE_BASE)
  /** Peripheral WAKEUP_MIX_SLICE base pointer */
  #define WAKEUP_MIX_SLICE_NS                      ((SRC_MIX_SLICE_Type *)WAKEUP_MIX_SLICE_BASE_NS)
  /** Peripheral MEGA_MIX_SLICE base address */
  #define MEGA_MIX_SLICE_BASE                      (0x54461000u)
  /** Peripheral MEGA_MIX_SLICE base address */
  #define MEGA_MIX_SLICE_BASE_NS                   (0x44461000u)
  /** Peripheral MEGA_MIX_SLICE base pointer */
  #define MEGA_MIX_SLICE                           ((SRC_MIX_SLICE_Type *)MEGA_MIX_SLICE_BASE)
  /** Peripheral MEGA_MIX_SLICE base pointer */
  #define MEGA_MIX_SLICE_NS                        ((SRC_MIX_SLICE_Type *)MEGA_MIX_SLICE_BASE_NS)
  /** Peripheral NETC_MIX_SLICE base address */
  #define NETC_MIX_SLICE_BASE                      (0x54461400u)
  /** Peripheral NETC_MIX_SLICE base address */
  #define NETC_MIX_SLICE_BASE_NS                   (0x44461400u)
  /** Peripheral NETC_MIX_SLICE base pointer */
  #define NETC_MIX_SLICE                           ((SRC_MIX_SLICE_Type *)NETC_MIX_SLICE_BASE)
  /** Peripheral NETC_MIX_SLICE base pointer */
  #define NETC_MIX_SLICE_NS                        ((SRC_MIX_SLICE_Type *)NETC_MIX_SLICE_BASE_NS)
  /** Peripheral CM33PLATFORM_MIX_SLICE base address */
  #define CM33PLATFORM_MIX_SLICE_BASE              (0x54461800u)
  /** Peripheral CM33PLATFORM_MIX_SLICE base address */
  #define CM33PLATFORM_MIX_SLICE_BASE_NS           (0x44461800u)
  /** Peripheral CM33PLATFORM_MIX_SLICE base pointer */
  #define CM33PLATFORM_MIX_SLICE                   ((SRC_MIX_SLICE_Type *)CM33PLATFORM_MIX_SLICE_BASE)
  /** Peripheral CM33PLATFORM_MIX_SLICE base pointer */
  #define CM33PLATFORM_MIX_SLICE_NS                ((SRC_MIX_SLICE_Type *)CM33PLATFORM_MIX_SLICE_BASE_NS)
  /** Peripheral CM7PLATFORM_MIX_SLICE base address */
  #define CM7PLATFORM_MIX_SLICE_BASE               (0x54461C00u)
  /** Peripheral CM7PLATFORM_MIX_SLICE base address */
  #define CM7PLATFORM_MIX_SLICE_BASE_NS            (0x44461C00u)
  /** Peripheral CM7PLATFORM_MIX_SLICE base pointer */
  #define CM7PLATFORM_MIX_SLICE                    ((SRC_MIX_SLICE_Type *)CM7PLATFORM_MIX_SLICE_BASE)
  /** Peripheral CM7PLATFORM_MIX_SLICE base pointer */
  #define CM7PLATFORM_MIX_SLICE_NS                 ((SRC_MIX_SLICE_Type *)CM7PLATFORM_MIX_SLICE_BASE_NS)
  /** Array initializer of SRC_MIX_SLICE peripheral base addresses */
  #define SRC_MIX_SLICE_BASE_ADDRS                 { AON_MIX_SLICE_BASE, WAKEUP_MIX_SLICE_BASE, MEGA_MIX_SLICE_BASE, NETC_MIX_SLICE_BASE, CM33PLATFORM_MIX_SLICE_BASE, CM7PLATFORM_MIX_SLICE_BASE }
  /** Array initializer of SRC_MIX_SLICE peripheral base pointers */
  #define SRC_MIX_SLICE_BASE_PTRS                  { AON_MIX_SLICE, WAKEUP_MIX_SLICE, MEGA_MIX_SLICE, NETC_MIX_SLICE, CM33PLATFORM_MIX_SLICE, CM7PLATFORM_MIX_SLICE }
  /** Array initializer of SRC_MIX_SLICE peripheral base addresses */
  #define SRC_MIX_SLICE_BASE_ADDRS_NS              { AON_MIX_SLICE_BASE_NS, WAKEUP_MIX_SLICE_BASE_NS, MEGA_MIX_SLICE_BASE_NS, NETC_MIX_SLICE_BASE_NS, CM33PLATFORM_MIX_SLICE_BASE_NS, CM7PLATFORM_MIX_SLICE_BASE_NS }
  /** Array initializer of SRC_MIX_SLICE peripheral base pointers */
  #define SRC_MIX_SLICE_BASE_PTRS_NS               { AON_MIX_SLICE_NS, WAKEUP_MIX_SLICE_NS, MEGA_MIX_SLICE_NS, NETC_MIX_SLICE_NS, CM33PLATFORM_MIX_SLICE_NS, CM7PLATFORM_MIX_SLICE_NS }
#else
  /** Peripheral AON_MIX_SLICE base address */
  #define AON_MIX_SLICE_BASE                       (0x44460800u)
  /** Peripheral AON_MIX_SLICE base pointer */
  #define AON_MIX_SLICE                            ((SRC_MIX_SLICE_Type *)AON_MIX_SLICE_BASE)
  /** Peripheral WAKEUP_MIX_SLICE base address */
  #define WAKEUP_MIX_SLICE_BASE                    (0x44460C00u)
  /** Peripheral WAKEUP_MIX_SLICE base pointer */
  #define WAKEUP_MIX_SLICE                         ((SRC_MIX_SLICE_Type *)WAKEUP_MIX_SLICE_BASE)
  /** Peripheral MEGA_MIX_SLICE base address */
  #define MEGA_MIX_SLICE_BASE                      (0x44461000u)
  /** Peripheral MEGA_MIX_SLICE base pointer */
  #define MEGA_MIX_SLICE                           ((SRC_MIX_SLICE_Type *)MEGA_MIX_SLICE_BASE)
  /** Peripheral NETC_MIX_SLICE base address */
  #define NETC_MIX_SLICE_BASE                      (0x44461400u)
  /** Peripheral NETC_MIX_SLICE base pointer */
  #define NETC_MIX_SLICE                           ((SRC_MIX_SLICE_Type *)NETC_MIX_SLICE_BASE)
  /** Peripheral CM33PLATFORM_MIX_SLICE base address */
  #define CM33PLATFORM_MIX_SLICE_BASE              (0x44461800u)
  /** Peripheral CM33PLATFORM_MIX_SLICE base pointer */
  #define CM33PLATFORM_MIX_SLICE                   ((SRC_MIX_SLICE_Type *)CM33PLATFORM_MIX_SLICE_BASE)
  /** Peripheral CM7PLATFORM_MIX_SLICE base address */
  #define CM7PLATFORM_MIX_SLICE_BASE               (0x44461C00u)
  /** Peripheral CM7PLATFORM_MIX_SLICE base pointer */
  #define CM7PLATFORM_MIX_SLICE                    ((SRC_MIX_SLICE_Type *)CM7PLATFORM_MIX_SLICE_BASE)
  /** Array initializer of SRC_MIX_SLICE peripheral base addresses */
  #define SRC_MIX_SLICE_BASE_ADDRS                 { AON_MIX_SLICE_BASE, WAKEUP_MIX_SLICE_BASE, MEGA_MIX_SLICE_BASE, NETC_MIX_SLICE_BASE, CM33PLATFORM_MIX_SLICE_BASE, CM7PLATFORM_MIX_SLICE_BASE }
  /** Array initializer of SRC_MIX_SLICE peripheral base pointers */
  #define SRC_MIX_SLICE_BASE_PTRS                  { AON_MIX_SLICE, WAKEUP_MIX_SLICE, MEGA_MIX_SLICE, NETC_MIX_SLICE, CM33PLATFORM_MIX_SLICE, CM7PLATFORM_MIX_SLICE }
#endif

/*!
 * @}
 */ /* end of group SRC_MIX_SLICE_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SYSPM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYSPM_Peripheral_Access_Layer SYSPM Peripheral Access Layer
 * @{
 */

/** SYSPM - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x30 */
    __IO uint32_t PMCR;                              /**< Performance Monitor Control, array offset: 0x0, array step: 0x30 */
         uint8_t RESERVED_0[12];
    __I  uint8_t PMICTR_HI;                          /**< Performance Monitor Instruction Counter, array offset: 0x10, array step: 0x30 */
         uint8_t RESERVED_1[3];
    __I  uint32_t PMICTR_LO;                         /**< Performance Monitor Instruction Counter, array offset: 0x14, array step: 0x30 */
    struct {                                         /* offset: 0x18, array step: index*0x30, index2*0x8 */
      __I  uint8_t HI;                                 /**< Performance Monitor Event Counter, array offset: 0x18, array step: index*0x30, index2*0x8 */
           uint8_t RESERVED_0[3];
      __I  uint32_t LO;                                /**< Performance Monitor Event Counter, array offset: 0x1C, array step: index*0x30, index2*0x8 */
    } PMECTR[3];
  } PMCR[1];
} SYSPM_Type;

/* ----------------------------------------------------------------------------
   -- SYSPM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYSPM_Register_Masks SYSPM Register Masks
 * @{
 */

/*! @name PMCR - Performance Monitor Control */
/*! @{ */

#define SYSPM_PMCR_MENB_MASK                     (0x1U)
#define SYSPM_PMCR_MENB_SHIFT                    (0U)
/*! MENB - Module Is Enabled
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define SYSPM_PMCR_MENB(x)                       (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_MENB_SHIFT)) & SYSPM_PMCR_MENB_MASK)

#define SYSPM_PMCR_SSC_MASK                      (0xEU)
#define SYSPM_PMCR_SSC_SHIFT                     (1U)
/*! SSC - Start and Stop Control
 *  0b000..Idle or no-op
 *  0b001..Local stop
 *  0b010, 0b011..Local start
 *  0b100..
 *  0b101..
 *  0b110, 0b111..
 */
#define SYSPM_PMCR_SSC(x)                        (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_SSC_SHIFT)) & SYSPM_PMCR_SSC_MASK)

#define SYSPM_PMCR_CMODE_MASK                    (0x30U)
#define SYSPM_PMCR_CMODE_SHIFT                   (4U)
/*! CMODE - Count Mode
 *  0b00..Counted in both User and Privileged modes
 *  0b01..
 *  0b10..Counted only in User mode
 *  0b11..Counted only in Privileged mode
 */
#define SYSPM_PMCR_CMODE(x)                      (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_CMODE_SHIFT)) & SYSPM_PMCR_CMODE_MASK)

#define SYSPM_PMCR_DCIFSH_MASK                   (0x40U)
#define SYSPM_PMCR_DCIFSH_SHIFT                  (6U)
/*! DCIFSH - Disable Counters if Stopped or Halted
 *  0b0..Continue
 *  0b1..Stop
 */
#define SYSPM_PMCR_DCIFSH(x)                     (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_DCIFSH_SHIFT)) & SYSPM_PMCR_DCIFSH_MASK)

#define SYSPM_PMCR_RICTR_MASK                    (0x80U)
#define SYSPM_PMCR_RICTR_SHIFT                   (7U)
/*! RICTR - Reset Instruction Counter
 *  0b0..Do not reset
 *  0b1..Clear
 */
#define SYSPM_PMCR_RICTR(x)                      (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_RICTR_SHIFT)) & SYSPM_PMCR_RICTR_MASK)

#define SYSPM_PMCR_RECTR1_MASK                   (0x100U)
#define SYSPM_PMCR_RECTR1_SHIFT                  (8U)
/*! RECTR1 - Reset Event Counter 1
 *  0b0..Run normally
 *  0b1..Reset
 */
#define SYSPM_PMCR_RECTR1(x)                     (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_RECTR1_SHIFT)) & SYSPM_PMCR_RECTR1_MASK)

#define SYSPM_PMCR_RECTR2_MASK                   (0x200U)
#define SYSPM_PMCR_RECTR2_SHIFT                  (9U)
/*! RECTR2 - Reset Event Counter 2
 *  0b0..Run normally
 *  0b1..Reset
 */
#define SYSPM_PMCR_RECTR2(x)                     (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_RECTR2_SHIFT)) & SYSPM_PMCR_RECTR2_MASK)

#define SYSPM_PMCR_RECTR3_MASK                   (0x400U)
#define SYSPM_PMCR_RECTR3_SHIFT                  (10U)
/*! RECTR3 - Reset Event Counter 3
 *  0b0..Run normally
 *  0b1..Reset
 */
#define SYSPM_PMCR_RECTR3(x)                     (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_RECTR3_SHIFT)) & SYSPM_PMCR_RECTR3_MASK)

#define SYSPM_PMCR_SELEVT1_MASK                  (0x3F800U)
#define SYSPM_PMCR_SELEVT1_SHIFT                 (11U)
/*! SELEVT1 - Select Event 1 */
#define SYSPM_PMCR_SELEVT1(x)                    (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_SELEVT1_SHIFT)) & SYSPM_PMCR_SELEVT1_MASK)

#define SYSPM_PMCR_SELEVT2_MASK                  (0x1FC0000U)
#define SYSPM_PMCR_SELEVT2_SHIFT                 (18U)
/*! SELEVT2 - Select Event 2 */
#define SYSPM_PMCR_SELEVT2(x)                    (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_SELEVT2_SHIFT)) & SYSPM_PMCR_SELEVT2_MASK)

#define SYSPM_PMCR_SELEVT3_MASK                  (0xFE000000U)
#define SYSPM_PMCR_SELEVT3_SHIFT                 (25U)
/*! SELEVT3 - Select Event 3 */
#define SYSPM_PMCR_SELEVT3(x)                    (((uint32_t)(((uint32_t)(x)) << SYSPM_PMCR_SELEVT3_SHIFT)) & SYSPM_PMCR_SELEVT3_MASK)
/*! @} */

/* The count of SYSPM_PMCR */
#define SYSPM_PMCR_COUNT                         (1U)

/*! @name PMICTR_HI - Performance Monitor Instruction Counter */
/*! @{ */

#define SYSPM_PMICTR_HI_ICTR_MASK                (0xFFU)
#define SYSPM_PMICTR_HI_ICTR_SHIFT               (0U)
/*! ICTR - Instruction Counter */
#define SYSPM_PMICTR_HI_ICTR(x)                  (((uint8_t)(((uint8_t)(x)) << SYSPM_PMICTR_HI_ICTR_SHIFT)) & SYSPM_PMICTR_HI_ICTR_MASK)
/*! @} */

/* The count of SYSPM_PMICTR_HI */
#define SYSPM_PMICTR_HI_COUNT                    (1U)

/*! @name PMICTR_LO - Performance Monitor Instruction Counter */
/*! @{ */

#define SYSPM_PMICTR_LO_ICTR_MASK                (0xFFFFFFFFU)
#define SYSPM_PMICTR_LO_ICTR_SHIFT               (0U)
/*! ICTR - Instruction Counter */
#define SYSPM_PMICTR_LO_ICTR(x)                  (((uint32_t)(((uint32_t)(x)) << SYSPM_PMICTR_LO_ICTR_SHIFT)) & SYSPM_PMICTR_LO_ICTR_MASK)
/*! @} */

/* The count of SYSPM_PMICTR_LO */
#define SYSPM_PMICTR_LO_COUNT                    (1U)

/*! @name HI - Performance Monitor Event Counter */
/*! @{ */

#define SYSPM_HI_ECTR_MASK                       (0xFFU)
#define SYSPM_HI_ECTR_SHIFT                      (0U)
/*! ECTR - Event Counter */
#define SYSPM_HI_ECTR(x)                         (((uint8_t)(((uint8_t)(x)) << SYSPM_HI_ECTR_SHIFT)) & SYSPM_HI_ECTR_MASK)
/*! @} */

/* The count of SYSPM_HI */
#define SYSPM_HI_COUNT                           (1U)

/* The count of SYSPM_HI */
#define SYSPM_HI_COUNT2                          (3U)

/*! @name LO - Performance Monitor Event Counter */
/*! @{ */

#define SYSPM_LO_ECTR_MASK                       (0xFFFFFFFFU)
#define SYSPM_LO_ECTR_SHIFT                      (0U)
/*! ECTR - Event Counter */
#define SYSPM_LO_ECTR(x)                         (((uint32_t)(((uint32_t)(x)) << SYSPM_LO_ECTR_SHIFT)) & SYSPM_LO_ECTR_MASK)
/*! @} */

/* The count of SYSPM_LO */
#define SYSPM_LO_COUNT                           (1U)

/* The count of SYSPM_LO */
#define SYSPM_LO_COUNT2                          (3U)


/*!
 * @}
 */ /* end of group SYSPM_Register_Masks */


/* SYSPM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral M33_PCF1 base address */
  #define M33_PCF1_BASE                            (0x543E0000u)
  /** Peripheral M33_PCF1 base address */
  #define M33_PCF1_BASE_NS                         (0x443E0000u)
  /** Peripheral M33_PCF1 base pointer */
  #define M33_PCF1                                 ((SYSPM_Type *)M33_PCF1_BASE)
  /** Peripheral M33_PCF1 base pointer */
  #define M33_PCF1_NS                              ((SYSPM_Type *)M33_PCF1_BASE_NS)
  /** Peripheral M33_PSF1 base address */
  #define M33_PSF1_BASE                            (0x543F0000u)
  /** Peripheral M33_PSF1 base address */
  #define M33_PSF1_BASE_NS                         (0x443F0000u)
  /** Peripheral M33_PSF1 base pointer */
  #define M33_PSF1                                 ((SYSPM_Type *)M33_PSF1_BASE)
  /** Peripheral M33_PSF1 base pointer */
  #define M33_PSF1_NS                              ((SYSPM_Type *)M33_PSF1_BASE_NS)
  /** Array initializer of SYSPM peripheral base addresses */
  #define SYSPM_BASE_ADDRS                         { M33_PCF1_BASE, M33_PSF1_BASE }
  /** Array initializer of SYSPM peripheral base pointers */
  #define SYSPM_BASE_PTRS                          { M33_PCF1, M33_PSF1 }
  /** Array initializer of SYSPM peripheral base addresses */
  #define SYSPM_BASE_ADDRS_NS                      { M33_PCF1_BASE_NS, M33_PSF1_BASE_NS }
  /** Array initializer of SYSPM peripheral base pointers */
  #define SYSPM_BASE_PTRS_NS                       { M33_PCF1_NS, M33_PSF1_NS }
#else
  /** Peripheral M33_PCF1 base address */
  #define M33_PCF1_BASE                            (0x443E0000u)
  /** Peripheral M33_PCF1 base pointer */
  #define M33_PCF1                                 ((SYSPM_Type *)M33_PCF1_BASE)
  /** Peripheral M33_PSF1 base address */
  #define M33_PSF1_BASE                            (0x443F0000u)
  /** Peripheral M33_PSF1 base pointer */
  #define M33_PSF1                                 ((SYSPM_Type *)M33_PSF1_BASE)
  /** Array initializer of SYSPM peripheral base addresses */
  #define SYSPM_BASE_ADDRS                         { M33_PCF1_BASE, M33_PSF1_BASE }
  /** Array initializer of SYSPM peripheral base pointers */
  #define SYSPM_BASE_PTRS                          { M33_PCF1, M33_PSF1 }
#endif

/*!
 * @}
 */ /* end of group SYSPM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SYS_CTR_COMPARE Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYS_CTR_COMPARE_Peripheral_Access_Layer SYS_CTR_COMPARE Peripheral Access Layer
 * @{
 */

/** SYS_CTR_COMPARE - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[32];
  __IO uint32_t CMPCVL0;                           /**< Compare Count Value Low, offset: 0x20 */
  __IO uint32_t CMPCVH0;                           /**< Compare Count Value High, offset: 0x24 */
       uint8_t RESERVED_1[4];
  __IO uint32_t CMPCR0;                            /**< Compare Control, offset: 0x2C */
       uint8_t RESERVED_2[240];
  __IO uint32_t CMPCVL1;                           /**< Compare Count Value Low, offset: 0x120 */
  __IO uint32_t CMPCVH1;                           /**< Compare Count Value High, offset: 0x124 */
       uint8_t RESERVED_3[4];
  __IO uint32_t CMPCR1;                            /**< Compare Control, offset: 0x12C */
       uint8_t RESERVED_4[3744];
  __I  uint32_t CNTID0;                            /**< Counter ID, offset: 0xFD0 */
} SYS_CTR_COMPARE_Type;

/* ----------------------------------------------------------------------------
   -- SYS_CTR_COMPARE Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYS_CTR_COMPARE_Register_Masks SYS_CTR_COMPARE Register Masks
 * @{
 */

/*! @name CMPCVL0 - Compare Count Value Low */
/*! @{ */

#define SYS_CTR_COMPARE_CMPCVL0_CMPCV0_MASK      (0xFFFFFFFFU)
#define SYS_CTR_COMPARE_CMPCVL0_CMPCV0_SHIFT     (0U)
/*! CMPCV0 - Compare Count Value Bits [31:0] */
#define SYS_CTR_COMPARE_CMPCVL0_CMPCV0(x)        (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCVL0_CMPCV0_SHIFT)) & SYS_CTR_COMPARE_CMPCVL0_CMPCV0_MASK)
/*! @} */

/*! @name CMPCVH0 - Compare Count Value High */
/*! @{ */

#define SYS_CTR_COMPARE_CMPCVH0_CMPCV1_MASK      (0xFFFFFFU)
#define SYS_CTR_COMPARE_CMPCVH0_CMPCV1_SHIFT     (0U)
/*! CMPCV1 - Compare Count Value Bits [55:32] */
#define SYS_CTR_COMPARE_CMPCVH0_CMPCV1(x)        (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCVH0_CMPCV1_SHIFT)) & SYS_CTR_COMPARE_CMPCVH0_CMPCV1_MASK)
/*! @} */

/*! @name CMPCR0 - Compare Control */
/*! @{ */

#define SYS_CTR_COMPARE_CMPCR0_EN_MASK           (0x1U)
#define SYS_CTR_COMPARE_CMPCR0_EN_SHIFT          (0U)
/*! EN - Compare Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define SYS_CTR_COMPARE_CMPCR0_EN(x)             (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCR0_EN_SHIFT)) & SYS_CTR_COMPARE_CMPCR0_EN_MASK)

#define SYS_CTR_COMPARE_CMPCR0_IMASK_MASK        (0x2U)
#define SYS_CTR_COMPARE_CMPCR0_IMASK_SHIFT       (1U)
/*! IMASK - Interrupt Request Mask
 *  0b0..Not masked
 *  0b1..Masked
 */
#define SYS_CTR_COMPARE_CMPCR0_IMASK(x)          (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCR0_IMASK_SHIFT)) & SYS_CTR_COMPARE_CMPCR0_IMASK_MASK)

#define SYS_CTR_COMPARE_CMPCR0_ISTAT_MASK        (0x4U)
#define SYS_CTR_COMPARE_CMPCR0_ISTAT_SHIFT       (2U)
/*! ISTAT - Compare Interrupt Status
 *  0b0..Either less than the compare value or compare is disabled
 *  0b1..Greater than or equal to the compare value and compare is enabled
 */
#define SYS_CTR_COMPARE_CMPCR0_ISTAT(x)          (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCR0_ISTAT_SHIFT)) & SYS_CTR_COMPARE_CMPCR0_ISTAT_MASK)
/*! @} */

/*! @name CMPCVL1 - Compare Count Value Low */
/*! @{ */

#define SYS_CTR_COMPARE_CMPCVL1_CMPCV0_MASK      (0xFFFFFFFFU)
#define SYS_CTR_COMPARE_CMPCVL1_CMPCV0_SHIFT     (0U)
/*! CMPCV0 - Compare Count Value Bits [31:0] */
#define SYS_CTR_COMPARE_CMPCVL1_CMPCV0(x)        (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCVL1_CMPCV0_SHIFT)) & SYS_CTR_COMPARE_CMPCVL1_CMPCV0_MASK)
/*! @} */

/*! @name CMPCVH1 - Compare Count Value High */
/*! @{ */

#define SYS_CTR_COMPARE_CMPCVH1_CMPCV1_MASK      (0xFFFFFFU)
#define SYS_CTR_COMPARE_CMPCVH1_CMPCV1_SHIFT     (0U)
/*! CMPCV1 - Compare Count Value Bits [55:32] */
#define SYS_CTR_COMPARE_CMPCVH1_CMPCV1(x)        (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCVH1_CMPCV1_SHIFT)) & SYS_CTR_COMPARE_CMPCVH1_CMPCV1_MASK)
/*! @} */

/*! @name CMPCR1 - Compare Control */
/*! @{ */

#define SYS_CTR_COMPARE_CMPCR1_EN_MASK           (0x1U)
#define SYS_CTR_COMPARE_CMPCR1_EN_SHIFT          (0U)
/*! EN - Compare Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define SYS_CTR_COMPARE_CMPCR1_EN(x)             (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCR1_EN_SHIFT)) & SYS_CTR_COMPARE_CMPCR1_EN_MASK)

#define SYS_CTR_COMPARE_CMPCR1_IMASK_MASK        (0x2U)
#define SYS_CTR_COMPARE_CMPCR1_IMASK_SHIFT       (1U)
/*! IMASK - Interrupt Request Mask
 *  0b0..Not masked
 *  0b1..Masked
 */
#define SYS_CTR_COMPARE_CMPCR1_IMASK(x)          (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCR1_IMASK_SHIFT)) & SYS_CTR_COMPARE_CMPCR1_IMASK_MASK)

#define SYS_CTR_COMPARE_CMPCR1_ISTAT_MASK        (0x4U)
#define SYS_CTR_COMPARE_CMPCR1_ISTAT_SHIFT       (2U)
/*! ISTAT - Compare Interrupt Status
 *  0b0..Either less than the compare value or compare is disabled
 *  0b1..Greater than or equal to the compare value and compare is enabled
 */
#define SYS_CTR_COMPARE_CMPCR1_ISTAT(x)          (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CMPCR1_ISTAT_SHIFT)) & SYS_CTR_COMPARE_CMPCR1_ISTAT_MASK)
/*! @} */

/*! @name CNTID0 - Counter ID */
/*! @{ */

#define SYS_CTR_COMPARE_CNTID0_CNTID_MASK        (0xFFFFFFFFU)
#define SYS_CTR_COMPARE_CNTID0_CNTID_SHIFT       (0U)
/*! CNTID - Counter Identification */
#define SYS_CTR_COMPARE_CNTID0_CNTID(x)          (((uint32_t)(((uint32_t)(x)) << SYS_CTR_COMPARE_CNTID0_CNTID_SHIFT)) & SYS_CTR_COMPARE_CNTID0_CNTID_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SYS_CTR_COMPARE_Register_Masks */


/* SYS_CTR_COMPARE - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SYS_CTR_COMPARE base address */
  #define SYS_CTR_COMPARE_BASE                     (0x542A0000u)
  /** Peripheral SYS_CTR_COMPARE base address */
  #define SYS_CTR_COMPARE_BASE_NS                  (0x442A0000u)
  /** Peripheral SYS_CTR_COMPARE base pointer */
  #define SYS_CTR_COMPARE                          ((SYS_CTR_COMPARE_Type *)SYS_CTR_COMPARE_BASE)
  /** Peripheral SYS_CTR_COMPARE base pointer */
  #define SYS_CTR_COMPARE_NS                       ((SYS_CTR_COMPARE_Type *)SYS_CTR_COMPARE_BASE_NS)
  /** Array initializer of SYS_CTR_COMPARE peripheral base addresses */
  #define SYS_CTR_COMPARE_BASE_ADDRS               { SYS_CTR_COMPARE_BASE }
  /** Array initializer of SYS_CTR_COMPARE peripheral base pointers */
  #define SYS_CTR_COMPARE_BASE_PTRS                { SYS_CTR_COMPARE }
  /** Array initializer of SYS_CTR_COMPARE peripheral base addresses */
  #define SYS_CTR_COMPARE_BASE_ADDRS_NS            { SYS_CTR_COMPARE_BASE_NS }
  /** Array initializer of SYS_CTR_COMPARE peripheral base pointers */
  #define SYS_CTR_COMPARE_BASE_PTRS_NS             { SYS_CTR_COMPARE_NS }
#else
  /** Peripheral SYS_CTR_COMPARE base address */
  #define SYS_CTR_COMPARE_BASE                     (0x442A0000u)
  /** Peripheral SYS_CTR_COMPARE base pointer */
  #define SYS_CTR_COMPARE                          ((SYS_CTR_COMPARE_Type *)SYS_CTR_COMPARE_BASE)
  /** Array initializer of SYS_CTR_COMPARE peripheral base addresses */
  #define SYS_CTR_COMPARE_BASE_ADDRS               { SYS_CTR_COMPARE_BASE }
  /** Array initializer of SYS_CTR_COMPARE peripheral base pointers */
  #define SYS_CTR_COMPARE_BASE_PTRS                { SYS_CTR_COMPARE }
#endif

/*!
 * @}
 */ /* end of group SYS_CTR_COMPARE_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SYS_CTR_CONTROL Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYS_CTR_CONTROL_Peripheral_Access_Layer SYS_CTR_CONTROL Peripheral Access Layer
 * @{
 */

/** SYS_CTR_CONTROL - Register Layout Typedef */
typedef struct {
  __IO uint32_t CNTCR;                             /**< Counter Control, offset: 0x0 */
  __I  uint32_t CNTSR;                             /**< Counter Status, offset: 0x4 */
  __IO uint32_t CNTCV0;                            /**< Counter Count Value Low, offset: 0x8 */
  __IO uint32_t CNTCV1;                            /**< Counter Count Value High, offset: 0xC */
       uint8_t RESERVED_0[16];
  __I  uint32_t CNTFID0;                           /**< Frequency-Modes Table 0, offset: 0x20 */
  __I  uint32_t CNTFID1;                           /**< Frequency-Modes Table 1, offset: 0x24 */
  __I  uint32_t CNTFID2;                           /**< Frequency-Modes Table 2, offset: 0x28 */
       uint8_t RESERVED_1[148];
  __IO uint32_t CNTCR2;                            /**< Counter Control 2, offset: 0xC0 */
       uint8_t RESERVED_2[3852];
  __I  uint32_t CNTID0;                            /**< Counter ID, offset: 0xFD0 */
} SYS_CTR_CONTROL_Type;

/* ----------------------------------------------------------------------------
   -- SYS_CTR_CONTROL Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYS_CTR_CONTROL_Register_Masks SYS_CTR_CONTROL Register Masks
 * @{
 */

/*! @name CNTCR - Counter Control */
/*! @{ */

#define SYS_CTR_CONTROL_CNTCR_EN_MASK            (0x1U)
#define SYS_CTR_CONTROL_CNTCR_EN_SHIFT           (0U)
/*! EN - Enable Counting
 *  0b0..Disable
 *  0b1..Enable
 */
#define SYS_CTR_CONTROL_CNTCR_EN(x)              (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTCR_EN_SHIFT)) & SYS_CTR_CONTROL_CNTCR_EN_MASK)

#define SYS_CTR_CONTROL_CNTCR_HDBG_MASK          (0x2U)
#define SYS_CTR_CONTROL_CNTCR_HDBG_SHIFT         (1U)
/*! HDBG - Enable Debug Halt
 *  0b0..Ignored
 *  0b1..Causes SYS_CTR to halt
 */
#define SYS_CTR_CONTROL_CNTCR_HDBG(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTCR_HDBG_SHIFT)) & SYS_CTR_CONTROL_CNTCR_HDBG_MASK)

#define SYS_CTR_CONTROL_CNTCR_FCR0_MASK          (0x100U)
#define SYS_CTR_CONTROL_CNTCR_FCR0_SHIFT         (8U)
/*! FCR0 - Frequency Change Request, ID 0
 *  0b0..No change
 *  0b1..Base frequency
 */
#define SYS_CTR_CONTROL_CNTCR_FCR0(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTCR_FCR0_SHIFT)) & SYS_CTR_CONTROL_CNTCR_FCR0_MASK)

#define SYS_CTR_CONTROL_CNTCR_FCR1_MASK          (0x200U)
#define SYS_CTR_CONTROL_CNTCR_FCR1_SHIFT         (9U)
/*! FCR1 - Frequency Change Request, ID 1
 *  0b0..No change
 *  0b1..Base frequency
 */
#define SYS_CTR_CONTROL_CNTCR_FCR1(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTCR_FCR1_SHIFT)) & SYS_CTR_CONTROL_CNTCR_FCR1_MASK)
/*! @} */

/*! @name CNTSR - Counter Status */
/*! @{ */

#define SYS_CTR_CONTROL_CNTSR_DBGH_MASK          (0x1U)
#define SYS_CTR_CONTROL_CNTSR_DBGH_SHIFT         (0U)
/*! DBGH - Debug Halt
 *  0b0..Did not halt
 *  0b1..Halted
 */
#define SYS_CTR_CONTROL_CNTSR_DBGH(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTSR_DBGH_SHIFT)) & SYS_CTR_CONTROL_CNTSR_DBGH_MASK)

#define SYS_CTR_CONTROL_CNTSR_FCA0_MASK          (0x100U)
#define SYS_CTR_CONTROL_CNTSR_FCA0_SHIFT         (8U)
/*! FCA0 - Frequency Change Acknowledge, ID 0
 *  0b0..Not selected
 *  0b1..Selected
 */
#define SYS_CTR_CONTROL_CNTSR_FCA0(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTSR_FCA0_SHIFT)) & SYS_CTR_CONTROL_CNTSR_FCA0_MASK)

#define SYS_CTR_CONTROL_CNTSR_FCA1_MASK          (0x200U)
#define SYS_CTR_CONTROL_CNTSR_FCA1_SHIFT         (9U)
/*! FCA1 - Frequency Change Acknowledge, ID 1
 *  0b0..Not selected
 *  0b1..Selected
 */
#define SYS_CTR_CONTROL_CNTSR_FCA1(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTSR_FCA1_SHIFT)) & SYS_CTR_CONTROL_CNTSR_FCA1_MASK)
/*! @} */

/*! @name CNTCV0 - Counter Count Value Low */
/*! @{ */

#define SYS_CTR_CONTROL_CNTCV0_CNTCV0_MASK       (0xFFFFFFFFU)
#define SYS_CTR_CONTROL_CNTCV0_CNTCV0_SHIFT      (0U)
/*! CNTCV0 - Counter Count Value Bits [31:0] */
#define SYS_CTR_CONTROL_CNTCV0_CNTCV0(x)         (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTCV0_CNTCV0_SHIFT)) & SYS_CTR_CONTROL_CNTCV0_CNTCV0_MASK)
/*! @} */

/*! @name CNTCV1 - Counter Count Value High */
/*! @{ */

#define SYS_CTR_CONTROL_CNTCV1_CNTCV1_MASK       (0xFFFFFFU)
#define SYS_CTR_CONTROL_CNTCV1_CNTCV1_SHIFT      (0U)
/*! CNTCV1 - Counter Count Value Bits [55:32] */
#define SYS_CTR_CONTROL_CNTCV1_CNTCV1(x)         (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTCV1_CNTCV1_SHIFT)) & SYS_CTR_CONTROL_CNTCV1_CNTCV1_MASK)
/*! @} */

/*! @name CNTFID0 - Frequency-Modes Table 0 */
/*! @{ */

#define SYS_CTR_CONTROL_CNTFID0_CNTFID0_MASK     (0xFFFFFFFFU)
#define SYS_CTR_CONTROL_CNTFID0_CNTFID0_SHIFT    (0U)
/*! CNTFID0 - Counter Frequency ID 0 */
#define SYS_CTR_CONTROL_CNTFID0_CNTFID0(x)       (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTFID0_CNTFID0_SHIFT)) & SYS_CTR_CONTROL_CNTFID0_CNTFID0_MASK)
/*! @} */

/*! @name CNTFID1 - Frequency-Modes Table 1 */
/*! @{ */

#define SYS_CTR_CONTROL_CNTFID1_CNTFID1_MASK     (0xFFFFFFFFU)
#define SYS_CTR_CONTROL_CNTFID1_CNTFID1_SHIFT    (0U)
/*! CNTFID1 - Counter Frequency ID 1 */
#define SYS_CTR_CONTROL_CNTFID1_CNTFID1(x)       (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTFID1_CNTFID1_SHIFT)) & SYS_CTR_CONTROL_CNTFID1_CNTFID1_MASK)
/*! @} */

/*! @name CNTFID2 - Frequency-Modes Table 2 */
/*! @{ */

#define SYS_CTR_CONTROL_CNTFID2_CNTFID2_MASK     (0xFFFFFFFFU)
#define SYS_CTR_CONTROL_CNTFID2_CNTFID2_SHIFT    (0U)
/*! CNTFID2 - Counter Frequency ID 2 */
#define SYS_CTR_CONTROL_CNTFID2_CNTFID2(x)       (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTFID2_CNTFID2_SHIFT)) & SYS_CTR_CONTROL_CNTFID2_CNTFID2_MASK)
/*! @} */

/*! @name CNTCR2 - Counter Control 2 */
/*! @{ */

#define SYS_CTR_CONTROL_CNTCR2_HWFC_EN_MASK      (0x1U)
#define SYS_CTR_CONTROL_CNTCR2_HWFC_EN_SHIFT     (0U)
/*! HWFC_EN - Hardware Frequency Change Enable
 *  0b0..No effect
 *  0b1..Same as performed via software
 */
#define SYS_CTR_CONTROL_CNTCR2_HWFC_EN(x)        (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTCR2_HWFC_EN_SHIFT)) & SYS_CTR_CONTROL_CNTCR2_HWFC_EN_MASK)
/*! @} */

/*! @name CNTID0 - Counter ID */
/*! @{ */

#define SYS_CTR_CONTROL_CNTID0_CNTID_MASK        (0xFFFFFFFFU)
#define SYS_CTR_CONTROL_CNTID0_CNTID_SHIFT       (0U)
/*! CNTID - Counter Identification */
#define SYS_CTR_CONTROL_CNTID0_CNTID(x)          (((uint32_t)(((uint32_t)(x)) << SYS_CTR_CONTROL_CNTID0_CNTID_SHIFT)) & SYS_CTR_CONTROL_CNTID0_CNTID_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SYS_CTR_CONTROL_Register_Masks */


/* SYS_CTR_CONTROL - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SYS_CTR_CONTROL base address */
  #define SYS_CTR_CONTROL_BASE                     (0x54290000u)
  /** Peripheral SYS_CTR_CONTROL base address */
  #define SYS_CTR_CONTROL_BASE_NS                  (0x44290000u)
  /** Peripheral SYS_CTR_CONTROL base pointer */
  #define SYS_CTR_CONTROL                          ((SYS_CTR_CONTROL_Type *)SYS_CTR_CONTROL_BASE)
  /** Peripheral SYS_CTR_CONTROL base pointer */
  #define SYS_CTR_CONTROL_NS                       ((SYS_CTR_CONTROL_Type *)SYS_CTR_CONTROL_BASE_NS)
  /** Array initializer of SYS_CTR_CONTROL peripheral base addresses */
  #define SYS_CTR_CONTROL_BASE_ADDRS               { SYS_CTR_CONTROL_BASE }
  /** Array initializer of SYS_CTR_CONTROL peripheral base pointers */
  #define SYS_CTR_CONTROL_BASE_PTRS                { SYS_CTR_CONTROL }
  /** Array initializer of SYS_CTR_CONTROL peripheral base addresses */
  #define SYS_CTR_CONTROL_BASE_ADDRS_NS            { SYS_CTR_CONTROL_BASE_NS }
  /** Array initializer of SYS_CTR_CONTROL peripheral base pointers */
  #define SYS_CTR_CONTROL_BASE_PTRS_NS             { SYS_CTR_CONTROL_NS }
#else
  /** Peripheral SYS_CTR_CONTROL base address */
  #define SYS_CTR_CONTROL_BASE                     (0x44290000u)
  /** Peripheral SYS_CTR_CONTROL base pointer */
  #define SYS_CTR_CONTROL                          ((SYS_CTR_CONTROL_Type *)SYS_CTR_CONTROL_BASE)
  /** Array initializer of SYS_CTR_CONTROL peripheral base addresses */
  #define SYS_CTR_CONTROL_BASE_ADDRS               { SYS_CTR_CONTROL_BASE }
  /** Array initializer of SYS_CTR_CONTROL peripheral base pointers */
  #define SYS_CTR_CONTROL_BASE_PTRS                { SYS_CTR_CONTROL }
#endif

/*!
 * @}
 */ /* end of group SYS_CTR_CONTROL_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- SYS_CTR_READ Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYS_CTR_READ_Peripheral_Access_Layer SYS_CTR_READ Peripheral Access Layer
 * @{
 */

/** SYS_CTR_READ - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[8];
  __I  uint32_t CNTCV0;                            /**< Counter Count Value Low, offset: 0x8 */
  __I  uint32_t CNTCV1;                            /**< Counter Count Value High, offset: 0xC */
       uint8_t RESERVED_1[4032];
  __I  uint32_t CNTID0;                            /**< Counter ID, offset: 0xFD0 */
} SYS_CTR_READ_Type;

/* ----------------------------------------------------------------------------
   -- SYS_CTR_READ Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SYS_CTR_READ_Register_Masks SYS_CTR_READ Register Masks
 * @{
 */

/*! @name CNTCV0 - Counter Count Value Low */
/*! @{ */

#define SYS_CTR_READ_CNTCV0_CNTCV0_MASK          (0xFFFFFFFFU)
#define SYS_CTR_READ_CNTCV0_CNTCV0_SHIFT         (0U)
/*! CNTCV0 - Counter Count Value Bits [31:0] */
#define SYS_CTR_READ_CNTCV0_CNTCV0(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_READ_CNTCV0_CNTCV0_SHIFT)) & SYS_CTR_READ_CNTCV0_CNTCV0_MASK)
/*! @} */

/*! @name CNTCV1 - Counter Count Value High */
/*! @{ */

#define SYS_CTR_READ_CNTCV1_CNTCV1_MASK          (0xFFFFFFU)
#define SYS_CTR_READ_CNTCV1_CNTCV1_SHIFT         (0U)
/*! CNTCV1 - Counter Count Value Bits [55:32] */
#define SYS_CTR_READ_CNTCV1_CNTCV1(x)            (((uint32_t)(((uint32_t)(x)) << SYS_CTR_READ_CNTCV1_CNTCV1_SHIFT)) & SYS_CTR_READ_CNTCV1_CNTCV1_MASK)
/*! @} */

/*! @name CNTID0 - Counter ID */
/*! @{ */

#define SYS_CTR_READ_CNTID0_CNTID_MASK           (0xFFFFFFFFU)
#define SYS_CTR_READ_CNTID0_CNTID_SHIFT          (0U)
/*! CNTID - Counter Identification */
#define SYS_CTR_READ_CNTID0_CNTID(x)             (((uint32_t)(((uint32_t)(x)) << SYS_CTR_READ_CNTID0_CNTID_SHIFT)) & SYS_CTR_READ_CNTID0_CNTID_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group SYS_CTR_READ_Register_Masks */


/* SYS_CTR_READ - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral SYS_CTR_READ base address */
  #define SYS_CTR_READ_BASE                        (0x542B0000u)
  /** Peripheral SYS_CTR_READ base address */
  #define SYS_CTR_READ_BASE_NS                     (0x442B0000u)
  /** Peripheral SYS_CTR_READ base pointer */
  #define SYS_CTR_READ                             ((SYS_CTR_READ_Type *)SYS_CTR_READ_BASE)
  /** Peripheral SYS_CTR_READ base pointer */
  #define SYS_CTR_READ_NS                          ((SYS_CTR_READ_Type *)SYS_CTR_READ_BASE_NS)
  /** Array initializer of SYS_CTR_READ peripheral base addresses */
  #define SYS_CTR_READ_BASE_ADDRS                  { SYS_CTR_READ_BASE }
  /** Array initializer of SYS_CTR_READ peripheral base pointers */
  #define SYS_CTR_READ_BASE_PTRS                   { SYS_CTR_READ }
  /** Array initializer of SYS_CTR_READ peripheral base addresses */
  #define SYS_CTR_READ_BASE_ADDRS_NS               { SYS_CTR_READ_BASE_NS }
  /** Array initializer of SYS_CTR_READ peripheral base pointers */
  #define SYS_CTR_READ_BASE_PTRS_NS                { SYS_CTR_READ_NS }
#else
  /** Peripheral SYS_CTR_READ base address */
  #define SYS_CTR_READ_BASE                        (0x442B0000u)
  /** Peripheral SYS_CTR_READ base pointer */
  #define SYS_CTR_READ                             ((SYS_CTR_READ_Type *)SYS_CTR_READ_BASE)
  /** Array initializer of SYS_CTR_READ peripheral base addresses */
  #define SYS_CTR_READ_BASE_ADDRS                  { SYS_CTR_READ_BASE }
  /** Array initializer of SYS_CTR_READ peripheral base pointers */
  #define SYS_CTR_READ_BASE_PTRS                   { SYS_CTR_READ }
#endif

/*!
 * @}
 */ /* end of group SYS_CTR_READ_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- TCM_ECC_MCM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TCM_ECC_MCM_Peripheral_Access_Layer TCM_ECC_MCM Peripheral Access Layer
 * @{
 */

/** TCM_ECC_MCM - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[4];
  __IO uint32_t TCMECCR;                           /**< TCM ECC Control, offset: 0x4 */
       uint8_t RESERVED_1[24];
  __IO uint32_t INT_STATUS;                        /**< Interrupt Status, offset: 0x20 */
  __IO uint32_t INT_STAT_EN;                       /**< Interrupt Status Enable, offset: 0x24 */
  __IO uint32_t INT_SIG_EN;                        /**< Interrupt Enable, offset: 0x28 */
       uint8_t RESERVED_2[48];
  __I  uint32_t CODE_TCM_ECC_SINGLE_ERROR_INFO;    /**< Code TCM Single-Bit ECC Error Information, offset: 0x5C */
  __I  uint32_t CODE_TCM_ECC_SINGLE_ERROR_ADDR;    /**< Code TCM Single-Bit ECC Error Address, offset: 0x60 */
       uint8_t RESERVED_3[4];
  __I  uint32_t CODE_TCM_ECC_MULTI_ERROR_INFO;     /**< Code TCM Multibit ECC Error Information, offset: 0x68 */
  __I  uint32_t CODE_TCM_ECC_MULTI_ERROR_ADDR;     /**< Code TCM Multibit ECC Error Address, offset: 0x6C */
       uint8_t RESERVED_4[4];
  __I  uint32_t SYS_TCM_ECC_SINGLE_ERROR_INFO;     /**< System TCM Single-Bit ECC Error Information, offset: 0x74 */
  __I  uint32_t SYS_TCM_ECC_SINGLE_ERROR_ADDR;     /**< System TCM Single-Bit ECC Error Address, offset: 0x78 */
       uint8_t RESERVED_5[4];
  __I  uint32_t SYS_TCM_ECC_MULTI_ERROR_INFO;      /**< System TCM Multibit ECC Error Information, offset: 0x80 */
  __I  uint32_t SYS_TCM_ECC_MULTI_ERROR_ADDR;      /**< System TCM Multibit ECC Error Address, offset: 0x84 */
       uint8_t RESERVED_6[12];
  __IO uint32_t CODE_TCM_ECC_ERROR_INJEC;          /**< Code TCM ECC Error Injection, offset: 0x94 */
  __IO uint32_t SYS_TCM_ECC_ERROR_INJEC;           /**< System TCM ECC Error Injection, offset: 0x98 */
} TCM_ECC_MCM_Type;

/* ----------------------------------------------------------------------------
   -- TCM_ECC_MCM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TCM_ECC_MCM_Register_Masks TCM_ECC_MCM Register Masks
 * @{
 */

/*! @name TCMECCR - TCM ECC Control */
/*! @{ */

#define TCM_ECC_MCM_TCMECCR_WECC_DIS_MASK        (0x1U)
#define TCM_ECC_MCM_TCMECCR_WECC_DIS_SHIFT       (0U)
/*! WECC_DIS - TCM ECC Write Generation Disable
 *  0b1..Disable
 *  0b0..Enable
 */
#define TCM_ECC_MCM_TCMECCR_WECC_DIS(x)          (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_TCMECCR_WECC_DIS_SHIFT)) & TCM_ECC_MCM_TCMECCR_WECC_DIS_MASK)

#define TCM_ECC_MCM_TCMECCR_RECC_DIS_MASK        (0x2U)
#define TCM_ECC_MCM_TCMECCR_RECC_DIS_SHIFT       (1U)
/*! RECC_DIS - TCM ECC Read Check Disable
 *  0b1..Disable
 *  0b0..Enable
 */
#define TCM_ECC_MCM_TCMECCR_RECC_DIS(x)          (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_TCMECCR_RECC_DIS_SHIFT)) & TCM_ECC_MCM_TCMECCR_RECC_DIS_MASK)
/*! @} */

/*! @name INT_STATUS - Interrupt Status */
/*! @{ */

#define TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRM_INT_MASK (0x400U)
#define TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRM_INT_SHIFT (10U)
/*! CODE_TCM_ECC_ERRM_INT - Code TCM Access Multibit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRM_INT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRM_INT_SHIFT)) & TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRM_INT_MASK)

#define TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRS_INT_MASK (0x800U)
#define TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRS_INT_SHIFT (11U)
/*! CODE_TCM_ECC_ERRS_INT - Code TCM Access Single-Bit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRS_INT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRS_INT_SHIFT)) & TCM_ECC_MCM_INT_STATUS_CODE_TCM_ECC_ERRS_INT_MASK)

#define TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRM_INT_MASK (0x1000U)
#define TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRM_INT_SHIFT (12U)
/*! SYS_TCM_ECC_ERRM_INT - System TCM Access Multibit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRM_INT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRM_INT_SHIFT)) & TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRM_INT_MASK)

#define TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRS_INT_MASK (0x2000U)
#define TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRS_INT_SHIFT (13U)
/*! SYS_TCM_ECC_ERRS_INT - System TCM Access Single-Bit ECC Error Interrupt Status
 *  0b0..No error
 *  0b1..Error
 *  0b0..No effect
 *  0b1..Clear the flag
 */
#define TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRS_INT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRS_INT_SHIFT)) & TCM_ECC_MCM_INT_STATUS_SYS_TCM_ECC_ERRS_INT_MASK)
/*! @} */

/*! @name INT_STAT_EN - Interrupt Status Enable */
/*! @{ */

#define TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRM_INT_EN_MASK (0x400U)
#define TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRM_INT_EN_SHIFT (10U)
/*! CODE_TCM_ERRM_INT_EN - Code TCM Access Multibit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRM_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRM_INT_EN_SHIFT)) & TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRM_INT_EN_MASK)

#define TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRS_INT_EN_MASK (0x800U)
#define TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRS_INT_EN_SHIFT (11U)
/*! CODE_TCM_ERRS_INT_EN - Code TCM Access Single-Bit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRS_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRS_INT_EN_SHIFT)) & TCM_ECC_MCM_INT_STAT_EN_CODE_TCM_ERRS_INT_EN_MASK)

#define TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRM_INT_EN_MASK (0x1000U)
#define TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRM_INT_EN_SHIFT (12U)
/*! SYS_TCM_ERRM_INT_EN - System TCM Access Multibit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRM_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRM_INT_EN_SHIFT)) & TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRM_INT_EN_MASK)

#define TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRS_INT_EN_MASK (0x2000U)
#define TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRS_INT_EN_SHIFT (13U)
/*! SYS_TCM_ERRS_INT_EN - System TCM Access Single-Bit ECC Error Interrupt Status Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRS_INT_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRS_INT_EN_SHIFT)) & TCM_ECC_MCM_INT_STAT_EN_SYS_TCM_ERRS_INT_EN_MASK)
/*! @} */

/*! @name INT_SIG_EN - Interrupt Enable */
/*! @{ */

#define TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRM_INT_SIG_EN_MASK (0x400U)
#define TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRM_INT_SIG_EN_SHIFT (10U)
/*! CODE_TCM_ERRM_INT_SIG_EN - Code TCM Access Multibit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRM_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRM_INT_SIG_EN_SHIFT)) & TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRM_INT_SIG_EN_MASK)

#define TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRS_INT_SIG_EN_MASK (0x800U)
#define TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRS_INT_SIG_EN_SHIFT (11U)
/*! CODE_TCM_ERRS_INT_SIG_EN - Code TCM Access Single-Bit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRS_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRS_INT_SIG_EN_SHIFT)) & TCM_ECC_MCM_INT_SIG_EN_CODE_TCM_ERRS_INT_SIG_EN_MASK)

#define TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRM_INT_SIG_EN_MASK (0x1000U)
#define TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRM_INT_SIG_EN_SHIFT (12U)
/*! SYS_TCM_ERRM_INT_SIG_EN - System TCM Access Multibit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRM_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRM_INT_SIG_EN_SHIFT)) & TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRM_INT_SIG_EN_MASK)

#define TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRS_INT_SIG_EN_MASK (0x2000U)
#define TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRS_INT_SIG_EN_SHIFT (13U)
/*! SYS_TCM_ERRS_INT_SIG_EN - System TCM Access Single-Bit ECC Error Interrupt Signal Enable
 *  0b0..Mask
 *  0b1..Enable
 */
#define TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRS_INT_SIG_EN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRS_INT_SIG_EN_SHIFT)) & TCM_ECC_MCM_INT_SIG_EN_SYS_TCM_ERRS_INT_SIG_EN_MASK)
/*! @} */

/*! @name CODE_TCM_ECC_SINGLE_ERROR_INFO - Code TCM Single-Bit ECC Error Information */
/*! @{ */

#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSIZ_MASK (0xEU)
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSIZ_SHIFT (1U)
/*! CODE_TCM_ECCS_EFSIZ - Code TCM Single-Bit ECC Error for Corresponding TCM Access Size */
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSIZ(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSIZ_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSIZ_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFMST_MASK (0xF0U)
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFMST_SHIFT (4U)
/*! CODE_TCM_ECCS_EFMST - Code TCM Single-Bit ECC Error for Corresponding TCM Master */
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFMST(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFMST_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFMST_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFPRT_MASK (0xF00U)
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFPRT_SHIFT (8U)
/*! CODE_TCM_ECCS_EFPRT - Code TCM Single-Bit ECC Error for Corresponding TCM Access Protection */
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFPRT_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFPRT_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSYN_MASK (0x7F000U)
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSYN_SHIFT (12U)
/*! CODE_TCM_ECCS_EFSYN - Code TCM Single-Bit ECC Error Corresponding Syndrome */
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSYN_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_INFO_CODE_TCM_ECCS_EFSYN_MASK)
/*! @} */

/*! @name CODE_TCM_ECC_SINGLE_ERROR_ADDR - Code TCM Single-Bit ECC Error Address */
/*! @{ */

#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_ADDR_CODE_TCM_ECCS_ERRED_ADDR_MASK (0xFFFFFFFFU)
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_ADDR_CODE_TCM_ECCS_ERRED_ADDR_SHIFT (0U)
/*! CODE_TCM_ECCS_ERRED_ADDR - Code TCM Single-Bit ECC Error Address */
#define TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_ADDR_CODE_TCM_ECCS_ERRED_ADDR(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_ADDR_CODE_TCM_ECCS_ERRED_ADDR_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_SINGLE_ERROR_ADDR_CODE_TCM_ECCS_ERRED_ADDR_MASK)
/*! @} */

/*! @name CODE_TCM_ECC_MULTI_ERROR_INFO - Code TCM Multibit ECC Error Information */
/*! @{ */

#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSIZ_MASK (0xEU)
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSIZ_SHIFT (1U)
/*! CODE_TCM_ECCM_EFSIZ - Code TCM Multibit ECC Error for Corresponding TCM Access Size */
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSIZ(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSIZ_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSIZ_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFMST_MASK (0xF0U)
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFMST_SHIFT (4U)
/*! CODE_TCM_ECCM_EFMST - Code TCM Multibit ECC Error for Corresponding TCM Master */
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFMST(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFMST_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFMST_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFPRT_MASK (0xF00U)
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFPRT_SHIFT (8U)
/*! CODE_TCM_ECCM_EFPRT - CODE_TCM Multibit ECC Error for Corresponding Access Protection Attribute */
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFPRT_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFPRT_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSYN_MASK (0x7F000U)
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSYN_SHIFT (12U)
/*! CODE_TCM_ECCM_EFSYN - Code TCM Multibit ECC Error for Corresponding Syndrome */
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSYN_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_INFO_CODE_TCM_ECCM_EFSYN_MASK)
/*! @} */

/*! @name CODE_TCM_ECC_MULTI_ERROR_ADDR - Code TCM Multibit ECC Error Address */
/*! @{ */

#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_ADDR_CODE_TCM_ECCM_ERRED_ADDR_MASK (0xFFFFFFFFU)
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_ADDR_CODE_TCM_ECCM_ERRED_ADDR_SHIFT (0U)
/*! CODE_TCM_ECCM_ERRED_ADDR - Code TCM Multibit ECC Error Address */
#define TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_ADDR_CODE_TCM_ECCM_ERRED_ADDR(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_ADDR_CODE_TCM_ECCM_ERRED_ADDR_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_MULTI_ERROR_ADDR_CODE_TCM_ECCM_ERRED_ADDR_MASK)
/*! @} */

/*! @name SYS_TCM_ECC_SINGLE_ERROR_INFO - System TCM Single-Bit ECC Error Information */
/*! @{ */

#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSIZ_MASK (0xEU)
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSIZ_SHIFT (1U)
/*! SYS_TCM_ECCS_EFSIZ - System TCM Single-Bit ECC Error for Corresponding TCM Access Size */
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSIZ(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSIZ_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSIZ_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFMST_MASK (0xF0U)
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFMST_SHIFT (4U)
/*! SYS_TCM_ECCS_EFMST - System TCM Single-Bit ECC Error for Corresponding TCM Master */
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFMST(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFMST_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFMST_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFPRT_MASK (0xF00U)
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFPRT_SHIFT (8U)
/*! SYS_TCM_ECCS_EFPRT - System TCM Single-Bit ECC Error for Corresponding Access Protection Attribute */
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFPRT_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFPRT_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSYN_MASK (0x7F000U)
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSYN_SHIFT (12U)
/*! SYS_TCM_ECCS_EFSYN - System TCM Single-Bit ECC Error for Corresponding Syndrome */
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSYN_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_INFO_SYS_TCM_ECCS_EFSYN_MASK)
/*! @} */

/*! @name SYS_TCM_ECC_SINGLE_ERROR_ADDR - System TCM Single-Bit ECC Error Address */
/*! @{ */

#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_ADDR_SYS_TCM_ECCS_ERRED_ADDR_MASK (0xFFFFFFFFU)
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_ADDR_SYS_TCM_ECCS_ERRED_ADDR_SHIFT (0U)
/*! SYS_TCM_ECCS_ERRED_ADDR - System TCM Single-Bit ECC Error Address */
#define TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_ADDR_SYS_TCM_ECCS_ERRED_ADDR(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_ADDR_SYS_TCM_ECCS_ERRED_ADDR_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_SINGLE_ERROR_ADDR_SYS_TCM_ECCS_ERRED_ADDR_MASK)
/*! @} */

/*! @name SYS_TCM_ECC_MULTI_ERROR_INFO - System TCM Multibit ECC Error Information */
/*! @{ */

#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSIZ_MASK (0xEU)
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSIZ_SHIFT (1U)
/*! SYS_TCM_ECCM_EFSIZ - System TCM Multibit ECC Error for Corresponding TCM Access Size */
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSIZ(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSIZ_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSIZ_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFMST_MASK (0xF0U)
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFMST_SHIFT (4U)
/*! SYS_TCM_ECCM_EFMST - System TCM Multibit ECC Error for Corresponding TCM Master */
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFMST(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFMST_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFMST_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFPRT_MASK (0xF00U)
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFPRT_SHIFT (8U)
/*! SYS_TCM_ECCM_EFPRT - System TCM Multibit ECC Error for Corresponding Access Protection Attribute */
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFPRT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFPRT_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFPRT_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSYN_MASK (0x7F000U)
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSYN_SHIFT (12U)
/*! SYS_TCM_ECCM_EFSYN - System TCM Multibit ECC Error for Corresponding Syndrome */
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSYN(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSYN_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_INFO_SYS_TCM_ECCM_EFSYN_MASK)
/*! @} */

/*! @name SYS_TCM_ECC_MULTI_ERROR_ADDR - System TCM Multibit ECC Error Address */
/*! @{ */

#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_ADDR_SYS_TCM_ECCM_ERRED_ADDR_MASK (0xFFFFFFFFU)
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_ADDR_SYS_TCM_ECCM_ERRED_ADDR_SHIFT (0U)
/*! SYS_TCM_ECCM_ERRED_ADDR - System TCM Multibit ECC Error Address */
#define TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_ADDR_SYS_TCM_ECCM_ERRED_ADDR(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_ADDR_SYS_TCM_ECCM_ERRED_ADDR_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_MULTI_ERROR_ADDR_SYS_TCM_ECCM_ERRED_ADDR_MASK)
/*! @} */

/*! @name CODE_TCM_ECC_ERROR_INJEC - Code TCM ECC Error Injection */
/*! @{ */

#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR1BIT_MASK (0xFFU)
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR1BIT_SHIFT (0U)
/*! CODE_TCM_ERR1BIT - Position of First Bit to Inject ECC Error */
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR1BIT_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR1BIT_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR2BIT_MASK (0xFF00U)
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR2BIT_SHIFT (8U)
/*! CODE_TCM_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR2BIT_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_ERR2BIT_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR11BI_MASK (0x10000U)
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR11BI_SHIFT (16U)
/*! CODE_TCM_FR11BI - Force One 1-Bit Data Inversion on Code TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR11BI_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR11BI_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR1NCI_MASK (0x20000U)
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR1NCI_SHIFT (17U)
/*! CODE_TCM_FR1NCI - Force One Noncorrectable Data Inversion on Code TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR1NCI_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FR1NCI_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRC1BI_MASK (0x40000U)
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRC1BI_SHIFT (18U)
/*! CODE_TCM_FRC1BI - Force Continuous 1-Bit Data Inversions on Code TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRC1BI_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRC1BI_MASK)

#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRCNCI_MASK (0x80000U)
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRCNCI_SHIFT (19U)
/*! CODE_TCM_FRCNCI - Force Continuous Noncorrectable Data Inversions on Code TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRCNCI_SHIFT)) & TCM_ECC_MCM_CODE_TCM_ECC_ERROR_INJEC_CODE_TCM_FRCNCI_MASK)
/*! @} */

/*! @name SYS_TCM_ECC_ERROR_INJEC - System TCM ECC Error Injection */
/*! @{ */

#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR1BIT_MASK (0xFFU)
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR1BIT_SHIFT (0U)
/*! SYS_TCM_ERR1BIT - Position of First Bit to Inject ECC Error */
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR1BIT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR1BIT_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR1BIT_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR2BIT_MASK (0xFF00U)
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR2BIT_SHIFT (8U)
/*! SYS_TCM_ERR2BIT - Position of Second Bit to Inject ECC Error */
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR2BIT(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR2BIT_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_ERR2BIT_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR11BI_MASK (0x10000U)
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR11BI_SHIFT (16U)
/*! SYS_TCM_FR11BI - Force One 1-Bit Data Inversion on System TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR11BI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR11BI_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR11BI_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR1NCI_MASK (0x20000U)
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR1NCI_SHIFT (17U)
/*! SYS_TCM_FR1NCI - Force One Noncorrectable Data Inversion on System TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR1NCI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR1NCI_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FR1NCI_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRC1BI_MASK (0x40000U)
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRC1BI_SHIFT (18U)
/*! SYS_TCM_FRC1BI - Force Continuous 1-Bit Data Inversions on System TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRC1BI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRC1BI_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRC1BI_MASK)

#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRCNCI_MASK (0x80000U)
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRCNCI_SHIFT (19U)
/*! SYS_TCM_FRCNCI - Force Continuous Noncorrectable Data Inversions on System TCM Write Access
 *  0b0..Disable injection
 *  0b1..Enable injection
 */
#define TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRCNCI(x) (((uint32_t)(((uint32_t)(x)) << TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRCNCI_SHIFT)) & TCM_ECC_MCM_SYS_TCM_ECC_ERROR_INJEC_SYS_TCM_FRCNCI_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group TCM_ECC_MCM_Register_Masks */


/* TCM_ECC_MCM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral CP_CM33_IMX9RTC__CM33_TCM_MCM base address */
  #define CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE       (0x54420000u)
  /** Peripheral CP_CM33_IMX9RTC__CM33_TCM_MCM base address */
  #define CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE_NS    (0x44420000u)
  /** Peripheral CP_CM33_IMX9RTC__CM33_TCM_MCM base pointer */
  #define CP_CM33_IMX9RTC__CM33_TCM_MCM            ((TCM_ECC_MCM_Type *)CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE)
  /** Peripheral CP_CM33_IMX9RTC__CM33_TCM_MCM base pointer */
  #define CP_CM33_IMX9RTC__CM33_TCM_MCM_NS         ((TCM_ECC_MCM_Type *)CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE_NS)
  /** Array initializer of TCM_ECC_MCM peripheral base addresses */
  #define TCM_ECC_MCM_BASE_ADDRS                   { CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE }
  /** Array initializer of TCM_ECC_MCM peripheral base pointers */
  #define TCM_ECC_MCM_BASE_PTRS                    { CP_CM33_IMX9RTC__CM33_TCM_MCM }
  /** Array initializer of TCM_ECC_MCM peripheral base addresses */
  #define TCM_ECC_MCM_BASE_ADDRS_NS                { CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE_NS }
  /** Array initializer of TCM_ECC_MCM peripheral base pointers */
  #define TCM_ECC_MCM_BASE_PTRS_NS                 { CP_CM33_IMX9RTC__CM33_TCM_MCM_NS }
#else
  /** Peripheral CP_CM33_IMX9RTC__CM33_TCM_MCM base address */
  #define CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE       (0x44420000u)
  /** Peripheral CP_CM33_IMX9RTC__CM33_TCM_MCM base pointer */
  #define CP_CM33_IMX9RTC__CM33_TCM_MCM            ((TCM_ECC_MCM_Type *)CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE)
  /** Array initializer of TCM_ECC_MCM peripheral base addresses */
  #define TCM_ECC_MCM_BASE_ADDRS                   { CP_CM33_IMX9RTC__CM33_TCM_MCM_BASE }
  /** Array initializer of TCM_ECC_MCM peripheral base pointers */
  #define TCM_ECC_MCM_BASE_PTRS                    { CP_CM33_IMX9RTC__CM33_TCM_MCM }
#endif

/*!
 * @}
 */ /* end of group TCM_ECC_MCM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- TMPSNS Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TMPSNS_Peripheral_Access_Layer TMPSNS Peripheral Access Layer
 * @{
 */

/** TMPSNS - Register Layout Typedef */
typedef struct {
       uint8_t RESERVED_0[16];
  __IO uint32_t CTRL1;                             /**< Control 1, offset: 0x10 */
  __IO uint32_t CTRL1_SET;                         /**< Control 1, offset: 0x14 */
  __IO uint32_t CTRL1_CLR;                         /**< Control 1, offset: 0x18 */
  __IO uint32_t CTRL1_TOG;                         /**< Control 1, offset: 0x1C */
  __IO uint32_t RANGE0;                            /**< Range 0, offset: 0x20 */
  __IO uint32_t RANGE0_SET;                        /**< Range 0, offset: 0x24 */
  __IO uint32_t RANGE0_CLR;                        /**< Range 0, offset: 0x28 */
  __IO uint32_t RANGE0_TOG;                        /**< Range 0, offset: 0x2C */
  __IO uint32_t RANGE1;                            /**< Range 1, offset: 0x30 */
  __IO uint32_t RANGE1_SET;                        /**< Range 1, offset: 0x34 */
  __IO uint32_t RANGE1_CLR;                        /**< Range 1, offset: 0x38 */
  __IO uint32_t RANGE1_TOG;                        /**< Range 1, offset: 0x3C */
       uint8_t RESERVED_1[16];
  __IO uint32_t STATUS0;                           /**< Status 0, offset: 0x50 */
} TMPSNS_Type;

/* ----------------------------------------------------------------------------
   -- TMPSNS Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TMPSNS_Register_Masks TMPSNS Register Masks
 * @{
 */

/*! @name CTRL1 - Control 1 */
/*! @{ */

#define TMPSNS_CTRL1_FREQ_MASK                   (0xFFFFU)
#define TMPSNS_CTRL1_FREQ_SHIFT                  (0U)
/*! FREQ - Temperature Measurement Frequency
 *  0b0000000000000000..Single Reading Mode. A new reading is available every time you change START from 0 to 1.
 *  0b0000000000000001-0b1111111111111111..Continuous Reading Mode. TMPSNS takes the next temperature reading when
 *                                         it completes the programmed number of cycles after the current reading.
 */
#define TMPSNS_CTRL1_FREQ(x)                     (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_FREQ_SHIFT)) & TMPSNS_CTRL1_FREQ_MASK)

#define TMPSNS_CTRL1_FINISH_IE_MASK              (0x10000U)
#define TMPSNS_CTRL1_FINISH_IE_SHIFT             (16U)
/*! FINISH_IE - Measurement Finished Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TMPSNS_CTRL1_FINISH_IE(x)                (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_FINISH_IE_SHIFT)) & TMPSNS_CTRL1_FINISH_IE_MASK)

#define TMPSNS_CTRL1_LOW_TEMP_IE_MASK            (0x20000U)
#define TMPSNS_CTRL1_LOW_TEMP_IE_SHIFT           (17U)
/*! LOW_TEMP_IE - Low Temperature Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TMPSNS_CTRL1_LOW_TEMP_IE(x)              (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_LOW_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_LOW_TEMP_IE_MASK)

#define TMPSNS_CTRL1_HIGH_TEMP_IE_MASK           (0x40000U)
#define TMPSNS_CTRL1_HIGH_TEMP_IE_SHIFT          (18U)
/*! HIGH_TEMP_IE - High Temperature Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TMPSNS_CTRL1_HIGH_TEMP_IE(x)             (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_HIGH_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_HIGH_TEMP_IE_MASK)

#define TMPSNS_CTRL1_PANIC_TEMP_IE_MASK          (0x80000U)
#define TMPSNS_CTRL1_PANIC_TEMP_IE_SHIFT         (19U)
/*! PANIC_TEMP_IE - Panic Temperature Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TMPSNS_CTRL1_PANIC_TEMP_IE(x)            (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_PANIC_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_PANIC_TEMP_IE_MASK)

#define TMPSNS_CTRL1_START_MASK                  (0x400000U)
#define TMPSNS_CTRL1_START_SHIFT                 (22U)
/*! START - Start Temperature Measurement
 *  0b0..No read
 *  0b1..New read
 */
#define TMPSNS_CTRL1_START(x)                    (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_START_SHIFT)) & TMPSNS_CTRL1_START_MASK)

#define TMPSNS_CTRL1_PWD_MASK                    (0x800000U)
#define TMPSNS_CTRL1_PWD_SHIFT                   (23U)
/*! PWD - Power Down Except Bias Current
 *  0b0..Active
 *  0b1..Inactive
 */
#define TMPSNS_CTRL1_PWD(x)                      (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_PWD_SHIFT)) & TMPSNS_CTRL1_PWD_MASK)

#define TMPSNS_CTRL1_PWD_FULL_MASK               (0x80000000U)
#define TMPSNS_CTRL1_PWD_FULL_SHIFT              (31U)
/*! PWD_FULL - Power Down
 *  0b0..Active
 *  0b1..Inactive
 */
#define TMPSNS_CTRL1_PWD_FULL(x)                 (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_PWD_FULL_SHIFT)) & TMPSNS_CTRL1_PWD_FULL_MASK)
/*! @} */

/*! @name CTRL1_SET - Control 1 */
/*! @{ */

#define TMPSNS_CTRL1_SET_FREQ_MASK               (0xFFFFU)
#define TMPSNS_CTRL1_SET_FREQ_SHIFT              (0U)
/*! FREQ - Temperature Measurement Frequency */
#define TMPSNS_CTRL1_SET_FREQ(x)                 (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_FREQ_SHIFT)) & TMPSNS_CTRL1_SET_FREQ_MASK)

#define TMPSNS_CTRL1_SET_FINISH_IE_MASK          (0x10000U)
#define TMPSNS_CTRL1_SET_FINISH_IE_SHIFT         (16U)
/*! FINISH_IE - Measurement Finished Interrupt Enable */
#define TMPSNS_CTRL1_SET_FINISH_IE(x)            (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_FINISH_IE_SHIFT)) & TMPSNS_CTRL1_SET_FINISH_IE_MASK)

#define TMPSNS_CTRL1_SET_LOW_TEMP_IE_MASK        (0x20000U)
#define TMPSNS_CTRL1_SET_LOW_TEMP_IE_SHIFT       (17U)
/*! LOW_TEMP_IE - Low Temperature Interrupt Enable */
#define TMPSNS_CTRL1_SET_LOW_TEMP_IE(x)          (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_LOW_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_SET_LOW_TEMP_IE_MASK)

#define TMPSNS_CTRL1_SET_HIGH_TEMP_IE_MASK       (0x40000U)
#define TMPSNS_CTRL1_SET_HIGH_TEMP_IE_SHIFT      (18U)
/*! HIGH_TEMP_IE - High Temperature Interrupt Enable */
#define TMPSNS_CTRL1_SET_HIGH_TEMP_IE(x)         (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_HIGH_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_SET_HIGH_TEMP_IE_MASK)

#define TMPSNS_CTRL1_SET_PANIC_TEMP_IE_MASK      (0x80000U)
#define TMPSNS_CTRL1_SET_PANIC_TEMP_IE_SHIFT     (19U)
/*! PANIC_TEMP_IE - Panic Temperature Interrupt Enable */
#define TMPSNS_CTRL1_SET_PANIC_TEMP_IE(x)        (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_PANIC_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_SET_PANIC_TEMP_IE_MASK)

#define TMPSNS_CTRL1_SET_START_MASK              (0x400000U)
#define TMPSNS_CTRL1_SET_START_SHIFT             (22U)
/*! START - Start Temperature Measurement */
#define TMPSNS_CTRL1_SET_START(x)                (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_START_SHIFT)) & TMPSNS_CTRL1_SET_START_MASK)

#define TMPSNS_CTRL1_SET_PWD_MASK                (0x800000U)
#define TMPSNS_CTRL1_SET_PWD_SHIFT               (23U)
/*! PWD - Power Down Except Bias Current */
#define TMPSNS_CTRL1_SET_PWD(x)                  (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_PWD_SHIFT)) & TMPSNS_CTRL1_SET_PWD_MASK)

#define TMPSNS_CTRL1_SET_PWD_FULL_MASK           (0x80000000U)
#define TMPSNS_CTRL1_SET_PWD_FULL_SHIFT          (31U)
/*! PWD_FULL - Power Down */
#define TMPSNS_CTRL1_SET_PWD_FULL(x)             (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_SET_PWD_FULL_SHIFT)) & TMPSNS_CTRL1_SET_PWD_FULL_MASK)
/*! @} */

/*! @name CTRL1_CLR - Control 1 */
/*! @{ */

#define TMPSNS_CTRL1_CLR_FREQ_MASK               (0xFFFFU)
#define TMPSNS_CTRL1_CLR_FREQ_SHIFT              (0U)
/*! FREQ - Temperature Measurement Frequency */
#define TMPSNS_CTRL1_CLR_FREQ(x)                 (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_FREQ_SHIFT)) & TMPSNS_CTRL1_CLR_FREQ_MASK)

#define TMPSNS_CTRL1_CLR_FINISH_IE_MASK          (0x10000U)
#define TMPSNS_CTRL1_CLR_FINISH_IE_SHIFT         (16U)
/*! FINISH_IE - Measurement Finished Interrupt Enable */
#define TMPSNS_CTRL1_CLR_FINISH_IE(x)            (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_FINISH_IE_SHIFT)) & TMPSNS_CTRL1_CLR_FINISH_IE_MASK)

#define TMPSNS_CTRL1_CLR_LOW_TEMP_IE_MASK        (0x20000U)
#define TMPSNS_CTRL1_CLR_LOW_TEMP_IE_SHIFT       (17U)
/*! LOW_TEMP_IE - Low Temperature Interrupt Enable */
#define TMPSNS_CTRL1_CLR_LOW_TEMP_IE(x)          (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_LOW_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_CLR_LOW_TEMP_IE_MASK)

#define TMPSNS_CTRL1_CLR_HIGH_TEMP_IE_MASK       (0x40000U)
#define TMPSNS_CTRL1_CLR_HIGH_TEMP_IE_SHIFT      (18U)
/*! HIGH_TEMP_IE - High Temperature Interrupt Enable */
#define TMPSNS_CTRL1_CLR_HIGH_TEMP_IE(x)         (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_HIGH_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_CLR_HIGH_TEMP_IE_MASK)

#define TMPSNS_CTRL1_CLR_PANIC_TEMP_IE_MASK      (0x80000U)
#define TMPSNS_CTRL1_CLR_PANIC_TEMP_IE_SHIFT     (19U)
/*! PANIC_TEMP_IE - Panic Temperature Interrupt Enable */
#define TMPSNS_CTRL1_CLR_PANIC_TEMP_IE(x)        (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_PANIC_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_CLR_PANIC_TEMP_IE_MASK)

#define TMPSNS_CTRL1_CLR_START_MASK              (0x400000U)
#define TMPSNS_CTRL1_CLR_START_SHIFT             (22U)
/*! START - Start Temperature Measurement */
#define TMPSNS_CTRL1_CLR_START(x)                (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_START_SHIFT)) & TMPSNS_CTRL1_CLR_START_MASK)

#define TMPSNS_CTRL1_CLR_PWD_MASK                (0x800000U)
#define TMPSNS_CTRL1_CLR_PWD_SHIFT               (23U)
/*! PWD - Power Down Except Bias Current */
#define TMPSNS_CTRL1_CLR_PWD(x)                  (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_PWD_SHIFT)) & TMPSNS_CTRL1_CLR_PWD_MASK)

#define TMPSNS_CTRL1_CLR_PWD_FULL_MASK           (0x80000000U)
#define TMPSNS_CTRL1_CLR_PWD_FULL_SHIFT          (31U)
/*! PWD_FULL - Power Down */
#define TMPSNS_CTRL1_CLR_PWD_FULL(x)             (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_CLR_PWD_FULL_SHIFT)) & TMPSNS_CTRL1_CLR_PWD_FULL_MASK)
/*! @} */

/*! @name CTRL1_TOG - Control 1 */
/*! @{ */

#define TMPSNS_CTRL1_TOG_FREQ_MASK               (0xFFFFU)
#define TMPSNS_CTRL1_TOG_FREQ_SHIFT              (0U)
/*! FREQ - Temperature Measurement Frequency */
#define TMPSNS_CTRL1_TOG_FREQ(x)                 (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_FREQ_SHIFT)) & TMPSNS_CTRL1_TOG_FREQ_MASK)

#define TMPSNS_CTRL1_TOG_FINISH_IE_MASK          (0x10000U)
#define TMPSNS_CTRL1_TOG_FINISH_IE_SHIFT         (16U)
/*! FINISH_IE - Measurement Finished Interrupt Enable */
#define TMPSNS_CTRL1_TOG_FINISH_IE(x)            (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_FINISH_IE_SHIFT)) & TMPSNS_CTRL1_TOG_FINISH_IE_MASK)

#define TMPSNS_CTRL1_TOG_LOW_TEMP_IE_MASK        (0x20000U)
#define TMPSNS_CTRL1_TOG_LOW_TEMP_IE_SHIFT       (17U)
/*! LOW_TEMP_IE - Low Temperature Interrupt Enable */
#define TMPSNS_CTRL1_TOG_LOW_TEMP_IE(x)          (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_LOW_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_TOG_LOW_TEMP_IE_MASK)

#define TMPSNS_CTRL1_TOG_HIGH_TEMP_IE_MASK       (0x40000U)
#define TMPSNS_CTRL1_TOG_HIGH_TEMP_IE_SHIFT      (18U)
/*! HIGH_TEMP_IE - High Temperature Interrupt Enable */
#define TMPSNS_CTRL1_TOG_HIGH_TEMP_IE(x)         (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_HIGH_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_TOG_HIGH_TEMP_IE_MASK)

#define TMPSNS_CTRL1_TOG_PANIC_TEMP_IE_MASK      (0x80000U)
#define TMPSNS_CTRL1_TOG_PANIC_TEMP_IE_SHIFT     (19U)
/*! PANIC_TEMP_IE - Panic Temperature Interrupt Enable */
#define TMPSNS_CTRL1_TOG_PANIC_TEMP_IE(x)        (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_PANIC_TEMP_IE_SHIFT)) & TMPSNS_CTRL1_TOG_PANIC_TEMP_IE_MASK)

#define TMPSNS_CTRL1_TOG_START_MASK              (0x400000U)
#define TMPSNS_CTRL1_TOG_START_SHIFT             (22U)
/*! START - Start Temperature Measurement */
#define TMPSNS_CTRL1_TOG_START(x)                (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_START_SHIFT)) & TMPSNS_CTRL1_TOG_START_MASK)

#define TMPSNS_CTRL1_TOG_PWD_MASK                (0x800000U)
#define TMPSNS_CTRL1_TOG_PWD_SHIFT               (23U)
/*! PWD - Power Down Except Bias Current */
#define TMPSNS_CTRL1_TOG_PWD(x)                  (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_PWD_SHIFT)) & TMPSNS_CTRL1_TOG_PWD_MASK)

#define TMPSNS_CTRL1_TOG_PWD_FULL_MASK           (0x80000000U)
#define TMPSNS_CTRL1_TOG_PWD_FULL_SHIFT          (31U)
/*! PWD_FULL - Power Down */
#define TMPSNS_CTRL1_TOG_PWD_FULL(x)             (((uint32_t)(((uint32_t)(x)) << TMPSNS_CTRL1_TOG_PWD_FULL_SHIFT)) & TMPSNS_CTRL1_TOG_PWD_FULL_MASK)
/*! @} */

/*! @name RANGE0 - Range 0 */
/*! @{ */

#define TMPSNS_RANGE0_LOW_TEMP_VAL_MASK          (0xFFFU)
#define TMPSNS_RANGE0_LOW_TEMP_VAL_SHIFT         (0U)
/*! LOW_TEMP_VAL - Low Temperature Threshold Value */
#define TMPSNS_RANGE0_LOW_TEMP_VAL(x)            (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_LOW_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_LOW_TEMP_VAL_MASK)

#define TMPSNS_RANGE0_HIGH_TEMP_VAL_MASK         (0xFFF0000U)
#define TMPSNS_RANGE0_HIGH_TEMP_VAL_SHIFT        (16U)
/*! HIGH_TEMP_VAL - High Temperature Threshold Value */
#define TMPSNS_RANGE0_HIGH_TEMP_VAL(x)           (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_HIGH_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_HIGH_TEMP_VAL_MASK)
/*! @} */

/*! @name RANGE0_SET - Range 0 */
/*! @{ */

#define TMPSNS_RANGE0_SET_LOW_TEMP_VAL_MASK      (0xFFFU)
#define TMPSNS_RANGE0_SET_LOW_TEMP_VAL_SHIFT     (0U)
/*! LOW_TEMP_VAL - Low Temperature Threshold Value */
#define TMPSNS_RANGE0_SET_LOW_TEMP_VAL(x)        (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_SET_LOW_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_SET_LOW_TEMP_VAL_MASK)

#define TMPSNS_RANGE0_SET_HIGH_TEMP_VAL_MASK     (0xFFF0000U)
#define TMPSNS_RANGE0_SET_HIGH_TEMP_VAL_SHIFT    (16U)
/*! HIGH_TEMP_VAL - High Temperature Threshold Value */
#define TMPSNS_RANGE0_SET_HIGH_TEMP_VAL(x)       (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_SET_HIGH_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_SET_HIGH_TEMP_VAL_MASK)
/*! @} */

/*! @name RANGE0_CLR - Range 0 */
/*! @{ */

#define TMPSNS_RANGE0_CLR_LOW_TEMP_VAL_MASK      (0xFFFU)
#define TMPSNS_RANGE0_CLR_LOW_TEMP_VAL_SHIFT     (0U)
/*! LOW_TEMP_VAL - Low Temperature Threshold Value */
#define TMPSNS_RANGE0_CLR_LOW_TEMP_VAL(x)        (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_CLR_LOW_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_CLR_LOW_TEMP_VAL_MASK)

#define TMPSNS_RANGE0_CLR_HIGH_TEMP_VAL_MASK     (0xFFF0000U)
#define TMPSNS_RANGE0_CLR_HIGH_TEMP_VAL_SHIFT    (16U)
/*! HIGH_TEMP_VAL - High Temperature Threshold Value */
#define TMPSNS_RANGE0_CLR_HIGH_TEMP_VAL(x)       (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_CLR_HIGH_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_CLR_HIGH_TEMP_VAL_MASK)
/*! @} */

/*! @name RANGE0_TOG - Range 0 */
/*! @{ */

#define TMPSNS_RANGE0_TOG_LOW_TEMP_VAL_MASK      (0xFFFU)
#define TMPSNS_RANGE0_TOG_LOW_TEMP_VAL_SHIFT     (0U)
/*! LOW_TEMP_VAL - Low Temperature Threshold Value */
#define TMPSNS_RANGE0_TOG_LOW_TEMP_VAL(x)        (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_TOG_LOW_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_TOG_LOW_TEMP_VAL_MASK)

#define TMPSNS_RANGE0_TOG_HIGH_TEMP_VAL_MASK     (0xFFF0000U)
#define TMPSNS_RANGE0_TOG_HIGH_TEMP_VAL_SHIFT    (16U)
/*! HIGH_TEMP_VAL - High Temperature Threshold Value */
#define TMPSNS_RANGE0_TOG_HIGH_TEMP_VAL(x)       (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE0_TOG_HIGH_TEMP_VAL_SHIFT)) & TMPSNS_RANGE0_TOG_HIGH_TEMP_VAL_MASK)
/*! @} */

/*! @name RANGE1 - Range 1 */
/*! @{ */

#define TMPSNS_RANGE1_PANIC_TEMP_VAL_MASK        (0xFFFU)
#define TMPSNS_RANGE1_PANIC_TEMP_VAL_SHIFT       (0U)
/*! PANIC_TEMP_VAL - Panic Temperature Threshold Value */
#define TMPSNS_RANGE1_PANIC_TEMP_VAL(x)          (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE1_PANIC_TEMP_VAL_SHIFT)) & TMPSNS_RANGE1_PANIC_TEMP_VAL_MASK)
/*! @} */

/*! @name RANGE1_SET - Range 1 */
/*! @{ */

#define TMPSNS_RANGE1_SET_PANIC_TEMP_VAL_MASK    (0xFFFU)
#define TMPSNS_RANGE1_SET_PANIC_TEMP_VAL_SHIFT   (0U)
/*! PANIC_TEMP_VAL - Panic Temperature Threshold Value */
#define TMPSNS_RANGE1_SET_PANIC_TEMP_VAL(x)      (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE1_SET_PANIC_TEMP_VAL_SHIFT)) & TMPSNS_RANGE1_SET_PANIC_TEMP_VAL_MASK)
/*! @} */

/*! @name RANGE1_CLR - Range 1 */
/*! @{ */

#define TMPSNS_RANGE1_CLR_PANIC_TEMP_VAL_MASK    (0xFFFU)
#define TMPSNS_RANGE1_CLR_PANIC_TEMP_VAL_SHIFT   (0U)
/*! PANIC_TEMP_VAL - Panic Temperature Threshold Value */
#define TMPSNS_RANGE1_CLR_PANIC_TEMP_VAL(x)      (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE1_CLR_PANIC_TEMP_VAL_SHIFT)) & TMPSNS_RANGE1_CLR_PANIC_TEMP_VAL_MASK)
/*! @} */

/*! @name RANGE1_TOG - Range 1 */
/*! @{ */

#define TMPSNS_RANGE1_TOG_PANIC_TEMP_VAL_MASK    (0xFFFU)
#define TMPSNS_RANGE1_TOG_PANIC_TEMP_VAL_SHIFT   (0U)
/*! PANIC_TEMP_VAL - Panic Temperature Threshold Value */
#define TMPSNS_RANGE1_TOG_PANIC_TEMP_VAL(x)      (((uint32_t)(((uint32_t)(x)) << TMPSNS_RANGE1_TOG_PANIC_TEMP_VAL_SHIFT)) & TMPSNS_RANGE1_TOG_PANIC_TEMP_VAL_MASK)
/*! @} */

/*! @name STATUS0 - Status 0 */
/*! @{ */

#define TMPSNS_STATUS0_TEMP_VAL_MASK             (0xFFFU)
#define TMPSNS_STATUS0_TEMP_VAL_SHIFT            (0U)
/*! TEMP_VAL - Measured Temperature Value */
#define TMPSNS_STATUS0_TEMP_VAL(x)               (((uint32_t)(((uint32_t)(x)) << TMPSNS_STATUS0_TEMP_VAL_SHIFT)) & TMPSNS_STATUS0_TEMP_VAL_MASK)

#define TMPSNS_STATUS0_FINISH_MASK               (0x10000U)
#define TMPSNS_STATUS0_FINISH_SHIFT              (16U)
/*! FINISH - Temperature Measurement Complete
 *  0b0..No read
 *  0b1..New read
 */
#define TMPSNS_STATUS0_FINISH(x)                 (((uint32_t)(((uint32_t)(x)) << TMPSNS_STATUS0_FINISH_SHIFT)) & TMPSNS_STATUS0_FINISH_MASK)

#define TMPSNS_STATUS0_LOW_TEMP_MASK             (0x20000U)
#define TMPSNS_STATUS0_LOW_TEMP_SHIFT            (17U)
/*! LOW_TEMP - Low Temperature Alarm
 *  0b0..No alert
 *  0b1..Alert
 */
#define TMPSNS_STATUS0_LOW_TEMP(x)               (((uint32_t)(((uint32_t)(x)) << TMPSNS_STATUS0_LOW_TEMP_SHIFT)) & TMPSNS_STATUS0_LOW_TEMP_MASK)

#define TMPSNS_STATUS0_HIGH_TEMP_MASK            (0x40000U)
#define TMPSNS_STATUS0_HIGH_TEMP_SHIFT           (18U)
/*! HIGH_TEMP - High Temperature Alarm
 *  0b0..No alert
 *  0b1..Alert
 */
#define TMPSNS_STATUS0_HIGH_TEMP(x)              (((uint32_t)(((uint32_t)(x)) << TMPSNS_STATUS0_HIGH_TEMP_SHIFT)) & TMPSNS_STATUS0_HIGH_TEMP_MASK)

#define TMPSNS_STATUS0_PANIC_TEMP_MASK           (0x80000U)
#define TMPSNS_STATUS0_PANIC_TEMP_SHIFT          (19U)
/*! PANIC_TEMP - Panic Temperature Alarm
 *  0b0..No alert
 *  0b1..Alert
 */
#define TMPSNS_STATUS0_PANIC_TEMP(x)             (((uint32_t)(((uint32_t)(x)) << TMPSNS_STATUS0_PANIC_TEMP_SHIFT)) & TMPSNS_STATUS0_PANIC_TEMP_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group TMPSNS_Register_Masks */


/* TMPSNS - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral TMPSNS base address */
  #define TMPSNS_BASE                              (0x54484580u)
  /** Peripheral TMPSNS base address */
  #define TMPSNS_BASE_NS                           (0x44484580u)
  /** Peripheral TMPSNS base pointer */
  #define TMPSNS                                   ((TMPSNS_Type *)TMPSNS_BASE)
  /** Peripheral TMPSNS base pointer */
  #define TMPSNS_NS                                ((TMPSNS_Type *)TMPSNS_BASE_NS)
  /** Array initializer of TMPSNS peripheral base addresses */
  #define TMPSNS_BASE_ADDRS                        { TMPSNS_BASE }
  /** Array initializer of TMPSNS peripheral base pointers */
  #define TMPSNS_BASE_PTRS                         { TMPSNS }
  /** Array initializer of TMPSNS peripheral base addresses */
  #define TMPSNS_BASE_ADDRS_NS                     { TMPSNS_BASE_NS }
  /** Array initializer of TMPSNS peripheral base pointers */
  #define TMPSNS_BASE_PTRS_NS                      { TMPSNS_NS }
#else
  /** Peripheral TMPSNS base address */
  #define TMPSNS_BASE                              (0x44484580u)
  /** Peripheral TMPSNS base pointer */
  #define TMPSNS                                   ((TMPSNS_Type *)TMPSNS_BASE)
  /** Array initializer of TMPSNS peripheral base addresses */
  #define TMPSNS_BASE_ADDRS                        { TMPSNS_BASE }
  /** Array initializer of TMPSNS peripheral base pointers */
  #define TMPSNS_BASE_PTRS                         { TMPSNS }
#endif

/*!
 * @}
 */ /* end of group TMPSNS_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- TMR Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TMR_Peripheral_Access_Layer TMR Peripheral Access Layer
 * @{
 */

/** TMR - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0, array step: 0x20 */
    __IO uint16_t COMP1;                             /**< Timer Channel Compare Register 1, array offset: 0x0, array step: 0x20 */
    __IO uint16_t COMP2;                             /**< Timer Channel Compare Register 2, array offset: 0x2, array step: 0x20 */
    __IO uint16_t CAPT;                              /**< Timer Channel Capture Register, array offset: 0x4, array step: 0x20 */
    __IO uint16_t LOAD;                              /**< Timer Channel Load Register, array offset: 0x6, array step: 0x20 */
    __IO uint16_t HOLD;                              /**< Timer Channel Hold Register, array offset: 0x8, array step: 0x20 */
    __IO uint16_t CNTR;                              /**< Timer Channel Counter Register, array offset: 0xA, array step: 0x20 */
    __IO uint16_t CTRL;                              /**< Timer Channel Control Register, array offset: 0xC, array step: 0x20 */
    __IO uint16_t SCTRL;                             /**< Timer Channel Status and Control Register, array offset: 0xE, array step: 0x20 */
    __IO uint16_t CMPLD1;                            /**< Timer Channel Comparator Load Register 1, array offset: 0x10, array step: 0x20 */
    __IO uint16_t CMPLD2;                            /**< Timer Channel Comparator Load Register 2, array offset: 0x12, array step: 0x20 */
    __IO uint16_t CSCTRL;                            /**< Timer Channel Comparator Status and Control Register, array offset: 0x14, array step: 0x20 */
    __IO uint16_t FILT;                              /**< Timer Channel Input Filter Register, array offset: 0x16, array step: 0x20 */
    __IO uint16_t DMA;                               /**< Timer Channel DMA Enable Register, array offset: 0x18, array step: 0x20 */
         uint8_t RESERVED_0[4];
    __IO uint16_t ENBL;                              /**< Timer Channel Enable Register, array offset: 0x1E, array step: 0x20, valid indices: [0] */
  } CHANNEL[4];
} TMR_Type;

/* ----------------------------------------------------------------------------
   -- TMR Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TMR_Register_Masks TMR Register Masks
 * @{
 */

/*! @name COMP1 - Timer Channel Compare Register 1 */
/*! @{ */

#define TMR_COMP1_COMPARISON_1_MASK              (0xFFFFU)
#define TMR_COMP1_COMPARISON_1_SHIFT             (0U)
/*! COMPARISON_1 - Comparison Value 1 */
#define TMR_COMP1_COMPARISON_1(x)                (((uint16_t)(((uint16_t)(x)) << TMR_COMP1_COMPARISON_1_SHIFT)) & TMR_COMP1_COMPARISON_1_MASK)
/*! @} */

/* The count of TMR_COMP1 */
#define TMR_COMP1_COUNT                          (4U)

/*! @name COMP2 - Timer Channel Compare Register 2 */
/*! @{ */

#define TMR_COMP2_COMPARISON_2_MASK              (0xFFFFU)
#define TMR_COMP2_COMPARISON_2_SHIFT             (0U)
/*! COMPARISON_2 - Comparison Value 2 */
#define TMR_COMP2_COMPARISON_2(x)                (((uint16_t)(((uint16_t)(x)) << TMR_COMP2_COMPARISON_2_SHIFT)) & TMR_COMP2_COMPARISON_2_MASK)
/*! @} */

/* The count of TMR_COMP2 */
#define TMR_COMP2_COUNT                          (4U)

/*! @name CAPT - Timer Channel Capture Register */
/*! @{ */

#define TMR_CAPT_CAPTURE_MASK                    (0xFFFFU)
#define TMR_CAPT_CAPTURE_SHIFT                   (0U)
/*! CAPTURE - Capture Value */
#define TMR_CAPT_CAPTURE(x)                      (((uint16_t)(((uint16_t)(x)) << TMR_CAPT_CAPTURE_SHIFT)) & TMR_CAPT_CAPTURE_MASK)
/*! @} */

/* The count of TMR_CAPT */
#define TMR_CAPT_COUNT                           (4U)

/*! @name LOAD - Timer Channel Load Register */
/*! @{ */

#define TMR_LOAD_LOAD_MASK                       (0xFFFFU)
#define TMR_LOAD_LOAD_SHIFT                      (0U)
/*! LOAD - Timer Load Register */
#define TMR_LOAD_LOAD(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_LOAD_LOAD_SHIFT)) & TMR_LOAD_LOAD_MASK)
/*! @} */

/* The count of TMR_LOAD */
#define TMR_LOAD_COUNT                           (4U)

/*! @name HOLD - Timer Channel Hold Register */
/*! @{ */

#define TMR_HOLD_HOLD_MASK                       (0xFFFFU)
#define TMR_HOLD_HOLD_SHIFT                      (0U)
/*! HOLD - HOLD */
#define TMR_HOLD_HOLD(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_HOLD_HOLD_SHIFT)) & TMR_HOLD_HOLD_MASK)
/*! @} */

/* The count of TMR_HOLD */
#define TMR_HOLD_COUNT                           (4U)

/*! @name CNTR - Timer Channel Counter Register */
/*! @{ */

#define TMR_CNTR_COUNTER_MASK                    (0xFFFFU)
#define TMR_CNTR_COUNTER_SHIFT                   (0U)
/*! COUNTER - COUNTER */
#define TMR_CNTR_COUNTER(x)                      (((uint16_t)(((uint16_t)(x)) << TMR_CNTR_COUNTER_SHIFT)) & TMR_CNTR_COUNTER_MASK)
/*! @} */

/* The count of TMR_CNTR */
#define TMR_CNTR_COUNT                           (4U)

/*! @name CTRL - Timer Channel Control Register */
/*! @{ */

#define TMR_CTRL_OUTMODE_MASK                    (0x7U)
#define TMR_CTRL_OUTMODE_SHIFT                   (0U)
/*! OUTMODE - Output Mode
 *  0b000..Asserted while counter is active
 *  0b001..Clear OFLAG output on successful compare
 *  0b010..Set OFLAG output on successful compare
 *  0b011..Toggle OFLAG output on successful compare
 *  0b100..Toggle OFLAG output using alternating compare registers
 *  0b101..Set on compare, cleared on secondary source input edge
 *  0b110..Set on compare, cleared on counter rollover
 *  0b111..Enable gated clock output while counter is active
 */
#define TMR_CTRL_OUTMODE(x)                      (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_OUTMODE_SHIFT)) & TMR_CTRL_OUTMODE_MASK)

#define TMR_CTRL_COINIT_MASK                     (0x8U)
#define TMR_CTRL_COINIT_SHIFT                    (3U)
/*! COINIT - Co-Channel Initialization
 *  0b0..Co-channel counter/timers cannot force a re-initialization of this counter/timer
 *  0b1..Co-channel counter/timers may force a re-initialization of this counter/timer
 */
#define TMR_CTRL_COINIT(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_COINIT_SHIFT)) & TMR_CTRL_COINIT_MASK)

#define TMR_CTRL_DIR_MASK                        (0x10U)
#define TMR_CTRL_DIR_SHIFT                       (4U)
/*! DIR - Count Direction
 *  0b0..Count up.
 *  0b1..Count down.
 */
#define TMR_CTRL_DIR(x)                          (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_DIR_SHIFT)) & TMR_CTRL_DIR_MASK)

#define TMR_CTRL_LENGTH_MASK                     (0x20U)
#define TMR_CTRL_LENGTH_SHIFT                    (5U)
/*! LENGTH - Count Length
 *  0b0..Count until roll over at $FFFF and then continue by re-initializing the counter from the LOAD register.
 *  0b1..Count until compare, then re-initialize using the LOAD register. If counting up, a successful compare
 *       occurs when the counter reaches a COMP1 value. If counting down, a successful compare occurs when the counter
 *       reaches a COMP2 value. When output mode $4 is used, alternating values of COMP1 and COMP2 are used to
 *       generate successful comparisons. For example, the counter counts until a COMP1 value is reached,
 *       re-initializes, counts until COMP2 value is reached, re-initializes, counts until COMP1 value is reached, and so on.
 */
#define TMR_CTRL_LENGTH(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_LENGTH_SHIFT)) & TMR_CTRL_LENGTH_MASK)

#define TMR_CTRL_ONCE_MASK                       (0x40U)
#define TMR_CTRL_ONCE_SHIFT                      (6U)
/*! ONCE - Count Once
 *  0b0..Count repeatedly.
 *  0b1..Count until compare and then stop. If counting up, a successful compare occurs when the counter reaches a
 *       COMP1 value. If counting down, a successful compare occurs when the counter reaches a COMP2 value. When
 *       output mode $4 is used, the counter re-initializes after reaching the COMP1 value, continues to count to
 *       the COMP2 value, and then stops.
 */
#define TMR_CTRL_ONCE(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_ONCE_SHIFT)) & TMR_CTRL_ONCE_MASK)

#define TMR_CTRL_SCS_MASK                        (0x180U)
#define TMR_CTRL_SCS_SHIFT                       (7U)
/*! SCS - Secondary Count Source
 *  0b00..Counter 0 input pin
 *  0b01..Counter 1 input pin
 *  0b10..Counter 2 input pin
 *  0b11..Counter 3 input pin
 */
#define TMR_CTRL_SCS(x)                          (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_SCS_SHIFT)) & TMR_CTRL_SCS_MASK)

#define TMR_CTRL_PCS_MASK                        (0x1E00U)
#define TMR_CTRL_PCS_SHIFT                       (9U)
/*! PCS - Primary Count Source
 *  0b0000..Counter 0 input pin
 *  0b0001..Counter 1 input pin
 *  0b0010..Counter 2 input pin
 *  0b0011..Counter 3 input pin
 *  0b0100..Counter 0 output
 *  0b0101..Counter 1 output
 *  0b0110..Counter 2 output
 *  0b0111..Counter 3 output
 *  0b1000..IP bus clock divide by 1 prescaler
 *  0b1001..IP bus clock divide by 2 prescaler
 *  0b1010..IP bus clock divide by 4 prescaler
 *  0b1011..IP bus clock divide by 8 prescaler
 *  0b1100..IP bus clock divide by 16 prescaler
 *  0b1101..IP bus clock divide by 32 prescaler
 *  0b1110..IP bus clock divide by 64 prescaler
 *  0b1111..IP bus clock divide by 128 prescaler
 */
#define TMR_CTRL_PCS(x)                          (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_PCS_SHIFT)) & TMR_CTRL_PCS_MASK)

#define TMR_CTRL_CM_MASK                         (0xE000U)
#define TMR_CTRL_CM_SHIFT                        (13U)
/*! CM - Count Mode
 *  0b000..No operation
 *  0b001..Count rising edges of primary sourceRising edges are counted only when SCTRL[IPS] = 0. Falling edges
 *         are counted when SCTRL[IPS] = 1. If the primary count source is IP bus clock divide by 1, only rising
 *         edges are counted regardless of the value of SCTRL[IPS].
 *  0b010..Count rising and falling edges of primary sourceIP bus clock divide by 1 cannot be used as a primary count source in edge count mode.
 *  0b011..Count rising edges of primary source while secondary input high active
 *  0b100..Quadrature count mode, uses primary and secondary sources
 *  0b101..Count rising edges of primary source; secondary source specifies directionRising edges are counted only
 *         when SCTRL[IPS] = 0. Falling edges are counted when SCTRL[IPS] = 1.
 *  0b110..Edge of secondary source triggers primary count until compare
 *  0b111..Cascaded counter mode (up/down)The primary count source must be set to one of the counter outputs.
 */
#define TMR_CTRL_CM(x)                           (((uint16_t)(((uint16_t)(x)) << TMR_CTRL_CM_SHIFT)) & TMR_CTRL_CM_MASK)
/*! @} */

/* The count of TMR_CTRL */
#define TMR_CTRL_COUNT                           (4U)

/*! @name SCTRL - Timer Channel Status and Control Register */
/*! @{ */

#define TMR_SCTRL_OEN_MASK                       (0x1U)
#define TMR_SCTRL_OEN_SHIFT                      (0U)
/*! OEN - Output Enable
 *  0b0..The external pin is configured as an input.
 *  0b1..The OFLAG output signal is driven on the external pin. Other timer groups using this external pin as
 *       their input see the driven value. The polarity of the signal is determined by OPS.
 */
#define TMR_SCTRL_OEN(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_OEN_SHIFT)) & TMR_SCTRL_OEN_MASK)

#define TMR_SCTRL_OPS_MASK                       (0x2U)
#define TMR_SCTRL_OPS_SHIFT                      (1U)
/*! OPS - Output Polarity Select
 *  0b0..True polarity.
 *  0b1..Inverted polarity.
 */
#define TMR_SCTRL_OPS(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_OPS_SHIFT)) & TMR_SCTRL_OPS_MASK)

#define TMR_SCTRL_FORCE_MASK                     (0x4U)
#define TMR_SCTRL_FORCE_SHIFT                    (2U)
/*! FORCE - Force OFLAG Output */
#define TMR_SCTRL_FORCE(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_FORCE_SHIFT)) & TMR_SCTRL_FORCE_MASK)

#define TMR_SCTRL_VAL_MASK                       (0x8U)
#define TMR_SCTRL_VAL_SHIFT                      (3U)
/*! VAL - Forced OFLAG Value */
#define TMR_SCTRL_VAL(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_VAL_SHIFT)) & TMR_SCTRL_VAL_MASK)

#define TMR_SCTRL_EEOF_MASK                      (0x10U)
#define TMR_SCTRL_EEOF_SHIFT                     (4U)
/*! EEOF - Enable External OFLAG Force */
#define TMR_SCTRL_EEOF(x)                        (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_EEOF_SHIFT)) & TMR_SCTRL_EEOF_MASK)

#define TMR_SCTRL_MSTR_MASK                      (0x20U)
#define TMR_SCTRL_MSTR_SHIFT                     (5U)
/*! MSTR - Master Mode */
#define TMR_SCTRL_MSTR(x)                        (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_MSTR_SHIFT)) & TMR_SCTRL_MSTR_MASK)

#define TMR_SCTRL_CAPTURE_MODE_MASK              (0xC0U)
#define TMR_SCTRL_CAPTURE_MODE_SHIFT             (6U)
/*! CAPTURE_MODE - Input Capture Mode
 *  0b00..Capture function is disabled
 *  0b01..Load capture register on rising edge (when IPS=0) or falling edge (when IPS=1) of input
 *  0b10..Load capture register on falling edge (when IPS=0) or rising edge (when IPS=1) of input
 *  0b11..Load capture register on both edges of input
 */
#define TMR_SCTRL_CAPTURE_MODE(x)                (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_CAPTURE_MODE_SHIFT)) & TMR_SCTRL_CAPTURE_MODE_MASK)

#define TMR_SCTRL_INPUT_MASK                     (0x100U)
#define TMR_SCTRL_INPUT_SHIFT                    (8U)
/*! INPUT - External Input Signal */
#define TMR_SCTRL_INPUT(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_INPUT_SHIFT)) & TMR_SCTRL_INPUT_MASK)

#define TMR_SCTRL_IPS_MASK                       (0x200U)
#define TMR_SCTRL_IPS_SHIFT                      (9U)
/*! IPS - Input Polarity Select */
#define TMR_SCTRL_IPS(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_IPS_SHIFT)) & TMR_SCTRL_IPS_MASK)

#define TMR_SCTRL_IEFIE_MASK                     (0x400U)
#define TMR_SCTRL_IEFIE_SHIFT                    (10U)
/*! IEFIE - Input Edge Flag Interrupt Enable */
#define TMR_SCTRL_IEFIE(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_IEFIE_SHIFT)) & TMR_SCTRL_IEFIE_MASK)

#define TMR_SCTRL_IEF_MASK                       (0x800U)
#define TMR_SCTRL_IEF_SHIFT                      (11U)
/*! IEF - Input Edge Flag */
#define TMR_SCTRL_IEF(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_IEF_SHIFT)) & TMR_SCTRL_IEF_MASK)

#define TMR_SCTRL_TOFIE_MASK                     (0x1000U)
#define TMR_SCTRL_TOFIE_SHIFT                    (12U)
/*! TOFIE - Timer Overflow Flag Interrupt Enable */
#define TMR_SCTRL_TOFIE(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_TOFIE_SHIFT)) & TMR_SCTRL_TOFIE_MASK)

#define TMR_SCTRL_TOF_MASK                       (0x2000U)
#define TMR_SCTRL_TOF_SHIFT                      (13U)
/*! TOF - Timer Overflow Flag */
#define TMR_SCTRL_TOF(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_TOF_SHIFT)) & TMR_SCTRL_TOF_MASK)

#define TMR_SCTRL_TCFIE_MASK                     (0x4000U)
#define TMR_SCTRL_TCFIE_SHIFT                    (14U)
/*! TCFIE - Timer Compare Flag Interrupt Enable */
#define TMR_SCTRL_TCFIE(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_TCFIE_SHIFT)) & TMR_SCTRL_TCFIE_MASK)

#define TMR_SCTRL_TCF_MASK                       (0x8000U)
#define TMR_SCTRL_TCF_SHIFT                      (15U)
/*! TCF - Timer Compare Flag */
#define TMR_SCTRL_TCF(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_SCTRL_TCF_SHIFT)) & TMR_SCTRL_TCF_MASK)
/*! @} */

/* The count of TMR_SCTRL */
#define TMR_SCTRL_COUNT                          (4U)

/*! @name CMPLD1 - Timer Channel Comparator Load Register 1 */
/*! @{ */

#define TMR_CMPLD1_COMPARATOR_LOAD_1_MASK        (0xFFFFU)
#define TMR_CMPLD1_COMPARATOR_LOAD_1_SHIFT       (0U)
/*! COMPARATOR_LOAD_1 - COMPARATOR_LOAD_1 */
#define TMR_CMPLD1_COMPARATOR_LOAD_1(x)          (((uint16_t)(((uint16_t)(x)) << TMR_CMPLD1_COMPARATOR_LOAD_1_SHIFT)) & TMR_CMPLD1_COMPARATOR_LOAD_1_MASK)
/*! @} */

/* The count of TMR_CMPLD1 */
#define TMR_CMPLD1_COUNT                         (4U)

/*! @name CMPLD2 - Timer Channel Comparator Load Register 2 */
/*! @{ */

#define TMR_CMPLD2_COMPARATOR_LOAD_2_MASK        (0xFFFFU)
#define TMR_CMPLD2_COMPARATOR_LOAD_2_SHIFT       (0U)
/*! COMPARATOR_LOAD_2 - COMPARATOR_LOAD_2 */
#define TMR_CMPLD2_COMPARATOR_LOAD_2(x)          (((uint16_t)(((uint16_t)(x)) << TMR_CMPLD2_COMPARATOR_LOAD_2_SHIFT)) & TMR_CMPLD2_COMPARATOR_LOAD_2_MASK)
/*! @} */

/* The count of TMR_CMPLD2 */
#define TMR_CMPLD2_COUNT                         (4U)

/*! @name CSCTRL - Timer Channel Comparator Status and Control Register */
/*! @{ */

#define TMR_CSCTRL_CL1_MASK                      (0x3U)
#define TMR_CSCTRL_CL1_SHIFT                     (0U)
/*! CL1 - Compare Load Control 1
 *  0b00..Never preload
 *  0b01..Load upon successful compare with the value in COMP1
 *  0b10..Load upon successful compare with the value in COMP2
 *  0b11..Reserved
 */
#define TMR_CSCTRL_CL1(x)                        (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_CL1_SHIFT)) & TMR_CSCTRL_CL1_MASK)

#define TMR_CSCTRL_CL2_MASK                      (0xCU)
#define TMR_CSCTRL_CL2_SHIFT                     (2U)
/*! CL2 - Compare Load Control 2
 *  0b00..Never preload
 *  0b01..Load upon successful compare with the value in COMP1
 *  0b10..Load upon successful compare with the value in COMP2
 *  0b11..Reserved
 */
#define TMR_CSCTRL_CL2(x)                        (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_CL2_SHIFT)) & TMR_CSCTRL_CL2_MASK)

#define TMR_CSCTRL_TCF1_MASK                     (0x10U)
#define TMR_CSCTRL_TCF1_SHIFT                    (4U)
/*! TCF1 - Timer Compare 1 Interrupt Flag */
#define TMR_CSCTRL_TCF1(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_TCF1_SHIFT)) & TMR_CSCTRL_TCF1_MASK)

#define TMR_CSCTRL_TCF2_MASK                     (0x20U)
#define TMR_CSCTRL_TCF2_SHIFT                    (5U)
/*! TCF2 - Timer Compare 2 Interrupt Flag */
#define TMR_CSCTRL_TCF2(x)                       (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_TCF2_SHIFT)) & TMR_CSCTRL_TCF2_MASK)

#define TMR_CSCTRL_TCF1EN_MASK                   (0x40U)
#define TMR_CSCTRL_TCF1EN_SHIFT                  (6U)
/*! TCF1EN - Timer Compare 1 Interrupt Enable */
#define TMR_CSCTRL_TCF1EN(x)                     (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_TCF1EN_SHIFT)) & TMR_CSCTRL_TCF1EN_MASK)

#define TMR_CSCTRL_TCF2EN_MASK                   (0x80U)
#define TMR_CSCTRL_TCF2EN_SHIFT                  (7U)
/*! TCF2EN - Timer Compare 2 Interrupt Enable */
#define TMR_CSCTRL_TCF2EN(x)                     (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_TCF2EN_SHIFT)) & TMR_CSCTRL_TCF2EN_MASK)

#define TMR_CSCTRL_UP_MASK                       (0x200U)
#define TMR_CSCTRL_UP_SHIFT                      (9U)
/*! UP - Counting Direction Indicator
 *  0b0..The last count was in the DOWN direction.
 *  0b1..The last count was in the UP direction.
 */
#define TMR_CSCTRL_UP(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_UP_SHIFT)) & TMR_CSCTRL_UP_MASK)

#define TMR_CSCTRL_TCI_MASK                      (0x400U)
#define TMR_CSCTRL_TCI_SHIFT                     (10U)
/*! TCI - Triggered Count Initialization Control
 *  0b0..Stop the counter upon receiving a second trigger event while still counting from the first trigger event.
 *  0b1..Reload the counter upon receiving a second trigger event while still counting from the first trigger event.
 */
#define TMR_CSCTRL_TCI(x)                        (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_TCI_SHIFT)) & TMR_CSCTRL_TCI_MASK)

#define TMR_CSCTRL_ROC_MASK                      (0x800U)
#define TMR_CSCTRL_ROC_SHIFT                     (11U)
/*! ROC - Reload on Capture
 *  0b0..Disables
 *  0b1..Enables
 */
#define TMR_CSCTRL_ROC(x)                        (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_ROC_SHIFT)) & TMR_CSCTRL_ROC_MASK)

#define TMR_CSCTRL_ALT_LOAD_MASK                 (0x1000U)
#define TMR_CSCTRL_ALT_LOAD_SHIFT                (12U)
/*! ALT_LOAD - Alternative Load Enable
 *  0b0..Counter can be re-initialized only with the LOAD register.
 *  0b1..Counter can be re-initialized with the LOAD or CMPLD2 registers depending on count direction.
 */
#define TMR_CSCTRL_ALT_LOAD(x)                   (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_ALT_LOAD_SHIFT)) & TMR_CSCTRL_ALT_LOAD_MASK)

#define TMR_CSCTRL_FAULT_MASK                    (0x2000U)
#define TMR_CSCTRL_FAULT_SHIFT                   (13U)
/*! FAULT - Fault Enable
 *  0b0..Disables
 *  0b1..Enables
 */
#define TMR_CSCTRL_FAULT(x)                      (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_FAULT_SHIFT)) & TMR_CSCTRL_FAULT_MASK)

#define TMR_CSCTRL_DBG_EN_MASK                   (0xC000U)
#define TMR_CSCTRL_DBG_EN_SHIFT                  (14U)
/*! DBG_EN - Debug Actions Enable
 *  0b00..Continue with normal operation during debug mode. (default)
 *  0b01..Halt TMR counter during debug mode.
 *  0b10..Force TMR output to logic 0 (prior to consideration of SCTRL[OPS]).
 *  0b11..Both halt counter and force output to 0 during debug mode.
 */
#define TMR_CSCTRL_DBG_EN(x)                     (((uint16_t)(((uint16_t)(x)) << TMR_CSCTRL_DBG_EN_SHIFT)) & TMR_CSCTRL_DBG_EN_MASK)
/*! @} */

/* The count of TMR_CSCTRL */
#define TMR_CSCTRL_COUNT                         (4U)

/*! @name FILT - Timer Channel Input Filter Register */
/*! @{ */

#define TMR_FILT_FILT_PER_MASK                   (0xFFU)
#define TMR_FILT_FILT_PER_SHIFT                  (0U)
/*! FILT_PER - Input Filter Sample Period */
#define TMR_FILT_FILT_PER(x)                     (((uint16_t)(((uint16_t)(x)) << TMR_FILT_FILT_PER_SHIFT)) & TMR_FILT_FILT_PER_MASK)

#define TMR_FILT_FILT_CNT_MASK                   (0x700U)
#define TMR_FILT_FILT_CNT_SHIFT                  (8U)
/*! FILT_CNT - Input Filter Sample Count */
#define TMR_FILT_FILT_CNT(x)                     (((uint16_t)(((uint16_t)(x)) << TMR_FILT_FILT_CNT_SHIFT)) & TMR_FILT_FILT_CNT_MASK)
/*! @} */

/* The count of TMR_FILT */
#define TMR_FILT_COUNT                           (4U)

/*! @name DMA - Timer Channel DMA Enable Register */
/*! @{ */

#define TMR_DMA_IEFDE_MASK                       (0x1U)
#define TMR_DMA_IEFDE_SHIFT                      (0U)
/*! IEFDE - Input Edge Flag DMA Enable */
#define TMR_DMA_IEFDE(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_DMA_IEFDE_SHIFT)) & TMR_DMA_IEFDE_MASK)

#define TMR_DMA_CMPLD1DE_MASK                    (0x2U)
#define TMR_DMA_CMPLD1DE_SHIFT                   (1U)
/*! CMPLD1DE - Comparator Preload Register 1 DMA Enable */
#define TMR_DMA_CMPLD1DE(x)                      (((uint16_t)(((uint16_t)(x)) << TMR_DMA_CMPLD1DE_SHIFT)) & TMR_DMA_CMPLD1DE_MASK)

#define TMR_DMA_CMPLD2DE_MASK                    (0x4U)
#define TMR_DMA_CMPLD2DE_SHIFT                   (2U)
/*! CMPLD2DE - Comparator Preload Register 2 DMA Enable */
#define TMR_DMA_CMPLD2DE(x)                      (((uint16_t)(((uint16_t)(x)) << TMR_DMA_CMPLD2DE_SHIFT)) & TMR_DMA_CMPLD2DE_MASK)
/*! @} */

/* The count of TMR_DMA */
#define TMR_DMA_COUNT                            (4U)

/*! @name ENBL - Timer Channel Enable Register */
/*! @{ */

#define TMR_ENBL_ENBL_MASK                       (0xFU)
#define TMR_ENBL_ENBL_SHIFT                      (0U)
/*! ENBL - Timer Channel Enable
 *  0b0000..Disables the timer channel.
 *  0b0001..Enables the timer channel. (default)
 */
#define TMR_ENBL_ENBL(x)                         (((uint16_t)(((uint16_t)(x)) << TMR_ENBL_ENBL_SHIFT)) & TMR_ENBL_ENBL_MASK)
/*! @} */

/* The count of TMR_ENBL */
#define TMR_ENBL_COUNT                           (4U)


/*!
 * @}
 */ /* end of group TMR_Register_Masks */


/* TMR - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral TMR1 base address */
  #define TMR1_BASE                                (0x52690000u)
  /** Peripheral TMR1 base address */
  #define TMR1_BASE_NS                             (0x42690000u)
  /** Peripheral TMR1 base pointer */
  #define TMR1                                     ((TMR_Type *)TMR1_BASE)
  /** Peripheral TMR1 base pointer */
  #define TMR1_NS                                  ((TMR_Type *)TMR1_BASE_NS)
  /** Peripheral TMR2 base address */
  #define TMR2_BASE                                (0x526A0000u)
  /** Peripheral TMR2 base address */
  #define TMR2_BASE_NS                             (0x426A0000u)
  /** Peripheral TMR2 base pointer */
  #define TMR2                                     ((TMR_Type *)TMR2_BASE)
  /** Peripheral TMR2 base pointer */
  #define TMR2_NS                                  ((TMR_Type *)TMR2_BASE_NS)
  /** Peripheral TMR3 base address */
  #define TMR3_BASE                                (0x526B0000u)
  /** Peripheral TMR3 base address */
  #define TMR3_BASE_NS                             (0x426B0000u)
  /** Peripheral TMR3 base pointer */
  #define TMR3                                     ((TMR_Type *)TMR3_BASE)
  /** Peripheral TMR3 base pointer */
  #define TMR3_NS                                  ((TMR_Type *)TMR3_BASE_NS)
  /** Peripheral TMR4 base address */
  #define TMR4_BASE                                (0x526C0000u)
  /** Peripheral TMR4 base address */
  #define TMR4_BASE_NS                             (0x426C0000u)
  /** Peripheral TMR4 base pointer */
  #define TMR4                                     ((TMR_Type *)TMR4_BASE)
  /** Peripheral TMR4 base pointer */
  #define TMR4_NS                                  ((TMR_Type *)TMR4_BASE_NS)
  /** Array initializer of TMR peripheral base addresses */
  #define TMR_BASE_ADDRS                           { 0u, TMR1_BASE, TMR2_BASE, TMR3_BASE, TMR4_BASE }
  /** Array initializer of TMR peripheral base pointers */
  #define TMR_BASE_PTRS                            { (TMR_Type *)0u, TMR1, TMR2, TMR3, TMR4 }
  /** Array initializer of TMR peripheral base addresses */
  #define TMR_BASE_ADDRS_NS                        { 0u, TMR1_BASE_NS, TMR2_BASE_NS, TMR3_BASE_NS, TMR4_BASE_NS }
  /** Array initializer of TMR peripheral base pointers */
  #define TMR_BASE_PTRS_NS                         { (TMR_Type *)0u, TMR1_NS, TMR2_NS, TMR3_NS, TMR4_NS }
#else
  /** Peripheral TMR1 base address */
  #define TMR1_BASE                                (0x42690000u)
  /** Peripheral TMR1 base pointer */
  #define TMR1                                     ((TMR_Type *)TMR1_BASE)
  /** Peripheral TMR2 base address */
  #define TMR2_BASE                                (0x426A0000u)
  /** Peripheral TMR2 base pointer */
  #define TMR2                                     ((TMR_Type *)TMR2_BASE)
  /** Peripheral TMR3 base address */
  #define TMR3_BASE                                (0x426B0000u)
  /** Peripheral TMR3 base pointer */
  #define TMR3                                     ((TMR_Type *)TMR3_BASE)
  /** Peripheral TMR4 base address */
  #define TMR4_BASE                                (0x426C0000u)
  /** Peripheral TMR4 base pointer */
  #define TMR4                                     ((TMR_Type *)TMR4_BASE)
  /** Array initializer of TMR peripheral base addresses */
  #define TMR_BASE_ADDRS                           { 0u, TMR1_BASE, TMR2_BASE, TMR3_BASE, TMR4_BASE }
  /** Array initializer of TMR peripheral base pointers */
  #define TMR_BASE_PTRS                            { (TMR_Type *)0u, TMR1, TMR2, TMR3, TMR4 }
#endif
/** Interrupt vectors for the TMR peripheral type */
#define TMR_IRQS                                 { NotAvail_IRQn, TMR1_IRQn, TMR2_IRQn, TMR3_IRQn, TMR4_IRQn }

/*!
 * @}
 */ /* end of group TMR_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- TPM Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TPM_Peripheral_Access_Layer TPM Peripheral Access Layer
 * @{
 */

/** TPM - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< Version ID, offset: 0x0 */
  __I  uint32_t PARAM;                             /**< Parameter, offset: 0x4 */
  __IO uint32_t GLOBAL;                            /**< TPM Global, offset: 0x8 */
       uint8_t RESERVED_0[4];
  __IO uint32_t SC;                                /**< Status and Control, offset: 0x10 */
  __IO uint32_t CNT;                               /**< Counter, offset: 0x14 */
  __IO uint32_t MOD;                               /**< Modulo, offset: 0x18 */
  __IO uint32_t STATUS;                            /**< Capture and Compare Status, offset: 0x1C */
  struct {                                         /* offset: 0x20, array step: 0x8 */
    __IO uint32_t CnSC;                              /**< Channel n Status and Control, array offset: 0x20, array step: 0x8 */
    __IO uint32_t CnV;                               /**< Channel n Value, array offset: 0x24, array step: 0x8 */
  } CONTROLS[4];
       uint8_t RESERVED_1[36];
  __IO uint32_t COMBINE;                           /**< Combine Channel, offset: 0x64 */
       uint8_t RESERVED_2[4];
  __IO uint32_t TRIG;                              /**< Channel Trigger, offset: 0x6C */
  __IO uint32_t POL;                               /**< Channel Polarity, offset: 0x70 */
       uint8_t RESERVED_3[4];
  __IO uint32_t FILTER;                            /**< Filter Control, offset: 0x78 */
       uint8_t RESERVED_4[4];
  __IO uint32_t QDCTRL;                            /**< Quadrature Decoder Control and Status, offset: 0x80 */
  __IO uint32_t CONF;                              /**< Configuration, offset: 0x84 */
} TPM_Type;

/* ----------------------------------------------------------------------------
   -- TPM Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TPM_Register_Masks TPM Register Masks
 * @{
 */

/*! @name VERID - Version ID */
/*! @{ */

#define TPM_VERID_FEATURE_MASK                   (0xFFFFU)
#define TPM_VERID_FEATURE_SHIFT                  (0U)
/*! FEATURE - Feature Identification Number
 *  0b0000000000000001..Standard feature set
 *  0b0000000000000011..Standard feature set with the filter and combine registers implemented
 *  0b0000000000000101..Standard feature set with the quadrature register implemented
 *  0b0000000000000111..Standard feature set with the filter, combine, and quadrature registers implemented
 */
#define TPM_VERID_FEATURE(x)                     (((uint32_t)(((uint32_t)(x)) << TPM_VERID_FEATURE_SHIFT)) & TPM_VERID_FEATURE_MASK)

#define TPM_VERID_MINOR_MASK                     (0xFF0000U)
#define TPM_VERID_MINOR_SHIFT                    (16U)
/*! MINOR - Minor Version Number */
#define TPM_VERID_MINOR(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_VERID_MINOR_SHIFT)) & TPM_VERID_MINOR_MASK)

#define TPM_VERID_MAJOR_MASK                     (0xFF000000U)
#define TPM_VERID_MAJOR_SHIFT                    (24U)
/*! MAJOR - Major Version Number */
#define TPM_VERID_MAJOR(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_VERID_MAJOR_SHIFT)) & TPM_VERID_MAJOR_MASK)
/*! @} */

/*! @name PARAM - Parameter */
/*! @{ */

#define TPM_PARAM_CHAN_MASK                      (0xFFU)
#define TPM_PARAM_CHAN_SHIFT                     (0U)
/*! CHAN - Channel Count */
#define TPM_PARAM_CHAN(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_PARAM_CHAN_SHIFT)) & TPM_PARAM_CHAN_MASK)

#define TPM_PARAM_TRIG_MASK                      (0xFF00U)
#define TPM_PARAM_TRIG_SHIFT                     (8U)
/*! TRIG - Trigger Count */
#define TPM_PARAM_TRIG(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_PARAM_TRIG_SHIFT)) & TPM_PARAM_TRIG_MASK)

#define TPM_PARAM_WIDTH_MASK                     (0xFF0000U)
#define TPM_PARAM_WIDTH_SHIFT                    (16U)
/*! WIDTH - Counter Width */
#define TPM_PARAM_WIDTH(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_PARAM_WIDTH_SHIFT)) & TPM_PARAM_WIDTH_MASK)
/*! @} */

/*! @name GLOBAL - TPM Global */
/*! @{ */

#define TPM_GLOBAL_NOUPDATE_MASK                 (0x1U)
#define TPM_GLOBAL_NOUPDATE_SHIFT                (0U)
/*! NOUPDATE - No Update
 *  0b0..Internal double-buffered registers update as normal
 *  0b1..Internal double-buffered registers do not update
 */
#define TPM_GLOBAL_NOUPDATE(x)                   (((uint32_t)(((uint32_t)(x)) << TPM_GLOBAL_NOUPDATE_SHIFT)) & TPM_GLOBAL_NOUPDATE_MASK)

#define TPM_GLOBAL_RST_MASK                      (0x2U)
#define TPM_GLOBAL_RST_SHIFT                     (1U)
/*! RST - Software Reset
 *  0b0..Module is not reset
 *  0b1..Module is reset
 */
#define TPM_GLOBAL_RST(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_GLOBAL_RST_SHIFT)) & TPM_GLOBAL_RST_MASK)
/*! @} */

/*! @name SC - Status and Control */
/*! @{ */

#define TPM_SC_PS_MASK                           (0x7U)
#define TPM_SC_PS_SHIFT                          (0U)
/*! PS - Prescale Factor Selection
 *  0b000..Divide by 1
 *  0b001..Divide by 2
 *  0b010..Divide by 4
 *  0b011..Divide by 8
 *  0b100..Divide by 16
 *  0b101..Divide by 32
 *  0b110..Divide by 64
 *  0b111..Divide by 128
 */
#define TPM_SC_PS(x)                             (((uint32_t)(((uint32_t)(x)) << TPM_SC_PS_SHIFT)) & TPM_SC_PS_MASK)

#define TPM_SC_CMOD_MASK                         (0x18U)
#define TPM_SC_CMOD_SHIFT                        (3U)
/*! CMOD - Clock Mode Selection
 *  0b00..TPM counter is disabled
 *  0b01..TPM counter increments on every TPM counter clock
 *  0b10..TPM counter increments on the rising edge of EXTCLK synchronized to the TPM counter clock
 *  0b11..TPM counter increments on the rising edge of the selected external input trigger
 */
#define TPM_SC_CMOD(x)                           (((uint32_t)(((uint32_t)(x)) << TPM_SC_CMOD_SHIFT)) & TPM_SC_CMOD_MASK)

#define TPM_SC_CPWMS_MASK                        (0x20U)
#define TPM_SC_CPWMS_SHIFT                       (5U)
/*! CPWMS - Center-Aligned PWM Select
 *  0b0..Up counting mode
 *  0b1..Up-down counting mode
 */
#define TPM_SC_CPWMS(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_SC_CPWMS_SHIFT)) & TPM_SC_CPWMS_MASK)

#define TPM_SC_TOIE_MASK                         (0x40U)
#define TPM_SC_TOIE_SHIFT                        (6U)
/*! TOIE - Timer Overflow Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TPM_SC_TOIE(x)                           (((uint32_t)(((uint32_t)(x)) << TPM_SC_TOIE_SHIFT)) & TPM_SC_TOIE_MASK)

#define TPM_SC_TOF_MASK                          (0x80U)
#define TPM_SC_TOF_SHIFT                         (7U)
/*! TOF - Timer Overflow Flag
 *  0b0..No overflow
 *  0b1..Overflow
 */
#define TPM_SC_TOF(x)                            (((uint32_t)(((uint32_t)(x)) << TPM_SC_TOF_SHIFT)) & TPM_SC_TOF_MASK)

#define TPM_SC_DMA_MASK                          (0x100U)
#define TPM_SC_DMA_SHIFT                         (8U)
/*! DMA - DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TPM_SC_DMA(x)                            (((uint32_t)(((uint32_t)(x)) << TPM_SC_DMA_SHIFT)) & TPM_SC_DMA_MASK)
/*! @} */

/*! @name CNT - Counter */
/*! @{ */

#define TPM_CNT_COUNT_MASK                       (0xFFFFFFFFU)
#define TPM_CNT_COUNT_SHIFT                      (0U)
/*! COUNT - Counter Value */
#define TPM_CNT_COUNT(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CNT_COUNT_SHIFT)) & TPM_CNT_COUNT_MASK)
/*! @} */

/*! @name MOD - Modulo */
/*! @{ */

#define TPM_MOD_MOD_MASK                         (0xFFFFFFFFU)
#define TPM_MOD_MOD_SHIFT                        (0U)
/*! MOD - Modulo Value */
#define TPM_MOD_MOD(x)                           (((uint32_t)(((uint32_t)(x)) << TPM_MOD_MOD_SHIFT)) & TPM_MOD_MOD_MASK)
/*! @} */

/*! @name STATUS - Capture and Compare Status */
/*! @{ */

#define TPM_STATUS_CH0F_MASK                     (0x1U)
#define TPM_STATUS_CH0F_SHIFT                    (0U)
/*! CH0F - Channel 0 Flag
 *  0b0..Event not occurred
 *  0b1..Event occurred
 */
#define TPM_STATUS_CH0F(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_STATUS_CH0F_SHIFT)) & TPM_STATUS_CH0F_MASK)

#define TPM_STATUS_CH1F_MASK                     (0x2U)
#define TPM_STATUS_CH1F_SHIFT                    (1U)
/*! CH1F - Channel 1 Flag
 *  0b0..Event not occurred
 *  0b1..Event occurred
 */
#define TPM_STATUS_CH1F(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_STATUS_CH1F_SHIFT)) & TPM_STATUS_CH1F_MASK)

#define TPM_STATUS_CH2F_MASK                     (0x4U)
#define TPM_STATUS_CH2F_SHIFT                    (2U)
/*! CH2F - Channel 2 Flag
 *  0b0..Event not occurred
 *  0b1..Event occurred
 */
#define TPM_STATUS_CH2F(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_STATUS_CH2F_SHIFT)) & TPM_STATUS_CH2F_MASK)

#define TPM_STATUS_CH3F_MASK                     (0x8U)
#define TPM_STATUS_CH3F_SHIFT                    (3U)
/*! CH3F - Channel 3 Flag
 *  0b0..Event not occurred
 *  0b1..Event occurred
 */
#define TPM_STATUS_CH3F(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_STATUS_CH3F_SHIFT)) & TPM_STATUS_CH3F_MASK)

#define TPM_STATUS_TOF_MASK                      (0x100U)
#define TPM_STATUS_TOF_SHIFT                     (8U)
/*! TOF - Timer Overflow Flag
 *  0b0..No overflow
 *  0b1..Overflow
 */
#define TPM_STATUS_TOF(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_STATUS_TOF_SHIFT)) & TPM_STATUS_TOF_MASK)
/*! @} */

/*! @name CnSC - Channel n Status and Control */
/*! @{ */

#define TPM_CnSC_DMA_MASK                        (0x1U)
#define TPM_CnSC_DMA_SHIFT                       (0U)
/*! DMA - DMA Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TPM_CnSC_DMA(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_CnSC_DMA_SHIFT)) & TPM_CnSC_DMA_MASK)

#define TPM_CnSC_ELSA_MASK                       (0x4U)
#define TPM_CnSC_ELSA_SHIFT                      (2U)
/*! ELSA - Edge or Level Select A */
#define TPM_CnSC_ELSA(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CnSC_ELSA_SHIFT)) & TPM_CnSC_ELSA_MASK)

#define TPM_CnSC_ELSB_MASK                       (0x8U)
#define TPM_CnSC_ELSB_SHIFT                      (3U)
/*! ELSB - Edge or Level Select B */
#define TPM_CnSC_ELSB(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CnSC_ELSB_SHIFT)) & TPM_CnSC_ELSB_MASK)

#define TPM_CnSC_MSA_MASK                        (0x10U)
#define TPM_CnSC_MSA_SHIFT                       (4U)
/*! MSA - Channel Mode Select A */
#define TPM_CnSC_MSA(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_CnSC_MSA_SHIFT)) & TPM_CnSC_MSA_MASK)

#define TPM_CnSC_MSB_MASK                        (0x20U)
#define TPM_CnSC_MSB_SHIFT                       (5U)
/*! MSB - Channel Mode Select B */
#define TPM_CnSC_MSB(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_CnSC_MSB_SHIFT)) & TPM_CnSC_MSB_MASK)

#define TPM_CnSC_CHIE_MASK                       (0x40U)
#define TPM_CnSC_CHIE_SHIFT                      (6U)
/*! CHIE - Channel Interrupt Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TPM_CnSC_CHIE(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CnSC_CHIE_SHIFT)) & TPM_CnSC_CHIE_MASK)

#define TPM_CnSC_CHF_MASK                        (0x80U)
#define TPM_CnSC_CHF_SHIFT                       (7U)
/*! CHF - Channel Flag
 *  0b0..Event not occurred
 *  0b1..Event occurred
 */
#define TPM_CnSC_CHF(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_CnSC_CHF_SHIFT)) & TPM_CnSC_CHF_MASK)
/*! @} */

/* The count of TPM_CnSC */
#define TPM_CnSC_COUNT                           (4U)

/*! @name CnV - Channel n Value */
/*! @{ */

#define TPM_CnV_VAL_MASK                         (0xFFFFFFFFU)
#define TPM_CnV_VAL_SHIFT                        (0U)
/*! VAL - Channel Value */
#define TPM_CnV_VAL(x)                           (((uint32_t)(((uint32_t)(x)) << TPM_CnV_VAL_SHIFT)) & TPM_CnV_VAL_MASK)
/*! @} */

/* The count of TPM_CnV */
#define TPM_CnV_COUNT                            (4U)

/*! @name COMBINE - Combine Channel */
/*! @{ */

#define TPM_COMBINE_COMBINE0_MASK                (0x1U)
#define TPM_COMBINE_COMBINE0_SHIFT               (0U)
/*! COMBINE0 - Combine Channels 0 and 1
 *  0b0..Independent
 *  0b1..Combined
 */
#define TPM_COMBINE_COMBINE0(x)                  (((uint32_t)(((uint32_t)(x)) << TPM_COMBINE_COMBINE0_SHIFT)) & TPM_COMBINE_COMBINE0_MASK)

#define TPM_COMBINE_COMSWAP0_MASK                (0x2U)
#define TPM_COMBINE_COMSWAP0_SHIFT               (1U)
/*! COMSWAP0 - Combine Channel 0 and 1 Swap
 *  0b0..Even channel
 *  0b1..Odd channel
 */
#define TPM_COMBINE_COMSWAP0(x)                  (((uint32_t)(((uint32_t)(x)) << TPM_COMBINE_COMSWAP0_SHIFT)) & TPM_COMBINE_COMSWAP0_MASK)

#define TPM_COMBINE_COMBINE1_MASK                (0x100U)
#define TPM_COMBINE_COMBINE1_SHIFT               (8U)
/*! COMBINE1 - Combine Channels 2 and 3
 *  0b0..Independent
 *  0b1..Combined
 */
#define TPM_COMBINE_COMBINE1(x)                  (((uint32_t)(((uint32_t)(x)) << TPM_COMBINE_COMBINE1_SHIFT)) & TPM_COMBINE_COMBINE1_MASK)

#define TPM_COMBINE_COMSWAP1_MASK                (0x200U)
#define TPM_COMBINE_COMSWAP1_SHIFT               (9U)
/*! COMSWAP1 - Combine Channels 2 and 3 Swap
 *  0b0..Even channel
 *  0b1..Odd channel
 */
#define TPM_COMBINE_COMSWAP1(x)                  (((uint32_t)(((uint32_t)(x)) << TPM_COMBINE_COMSWAP1_SHIFT)) & TPM_COMBINE_COMSWAP1_MASK)
/*! @} */

/*! @name TRIG - Channel Trigger */
/*! @{ */

#define TPM_TRIG_TRIG0_MASK                      (0x1U)
#define TPM_TRIG_TRIG0_SHIFT                     (0U)
/*! TRIG0 - Channel 0 Trigger
 *  0b0..No effect
 *  0b1..Configures trigger input 0 to be used by channel 0
 */
#define TPM_TRIG_TRIG0(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_TRIG_TRIG0_SHIFT)) & TPM_TRIG_TRIG0_MASK)

#define TPM_TRIG_TRIG1_MASK                      (0x2U)
#define TPM_TRIG_TRIG1_SHIFT                     (1U)
/*! TRIG1 - Channel 1 Trigger
 *  0b0..No effect
 *  0b1..Configures trigger input 1 to be used by channel 1
 */
#define TPM_TRIG_TRIG1(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_TRIG_TRIG1_SHIFT)) & TPM_TRIG_TRIG1_MASK)

#define TPM_TRIG_TRIG2_MASK                      (0x4U)
#define TPM_TRIG_TRIG2_SHIFT                     (2U)
/*! TRIG2 - Channel 2 Trigger
 *  0b0..No effect
 *  0b1..Configures trigger input 0 to be used by channel 2
 */
#define TPM_TRIG_TRIG2(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_TRIG_TRIG2_SHIFT)) & TPM_TRIG_TRIG2_MASK)

#define TPM_TRIG_TRIG3_MASK                      (0x8U)
#define TPM_TRIG_TRIG3_SHIFT                     (3U)
/*! TRIG3 - Channel 3 Trigger
 *  0b0..No effect
 *  0b1..Configures trigger input 1 to be used by channel 3
 */
#define TPM_TRIG_TRIG3(x)                        (((uint32_t)(((uint32_t)(x)) << TPM_TRIG_TRIG3_SHIFT)) & TPM_TRIG_TRIG3_MASK)
/*! @} */

/*! @name POL - Channel Polarity */
/*! @{ */

#define TPM_POL_POL0_MASK                        (0x1U)
#define TPM_POL_POL0_SHIFT                       (0U)
/*! POL0 - Channel 0 Polarity
 *  0b0..Active high
 *  0b1..Active low
 */
#define TPM_POL_POL0(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_POL_POL0_SHIFT)) & TPM_POL_POL0_MASK)

#define TPM_POL_POL1_MASK                        (0x2U)
#define TPM_POL_POL1_SHIFT                       (1U)
/*! POL1 - Channel 1 Polarity
 *  0b0..Active high
 *  0b1..Active low
 */
#define TPM_POL_POL1(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_POL_POL1_SHIFT)) & TPM_POL_POL1_MASK)

#define TPM_POL_POL2_MASK                        (0x4U)
#define TPM_POL_POL2_SHIFT                       (2U)
/*! POL2 - Channel 2 Polarity
 *  0b0..Active high
 *  0b1..Active low
 */
#define TPM_POL_POL2(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_POL_POL2_SHIFT)) & TPM_POL_POL2_MASK)

#define TPM_POL_POL3_MASK                        (0x8U)
#define TPM_POL_POL3_SHIFT                       (3U)
/*! POL3 - Channel 3 Polarity
 *  0b0..Active high
 *  0b1..Active low
 */
#define TPM_POL_POL3(x)                          (((uint32_t)(((uint32_t)(x)) << TPM_POL_POL3_SHIFT)) & TPM_POL_POL3_MASK)
/*! @} */

/*! @name FILTER - Filter Control */
/*! @{ */

#define TPM_FILTER_CH0FVAL_MASK                  (0xFU)
#define TPM_FILTER_CH0FVAL_SHIFT                 (0U)
/*! CH0FVAL - Channel 0 Filter Value */
#define TPM_FILTER_CH0FVAL(x)                    (((uint32_t)(((uint32_t)(x)) << TPM_FILTER_CH0FVAL_SHIFT)) & TPM_FILTER_CH0FVAL_MASK)

#define TPM_FILTER_CH1FVAL_MASK                  (0xF0U)
#define TPM_FILTER_CH1FVAL_SHIFT                 (4U)
/*! CH1FVAL - Channel 1 Filter Value */
#define TPM_FILTER_CH1FVAL(x)                    (((uint32_t)(((uint32_t)(x)) << TPM_FILTER_CH1FVAL_SHIFT)) & TPM_FILTER_CH1FVAL_MASK)

#define TPM_FILTER_CH2FVAL_MASK                  (0xF00U)
#define TPM_FILTER_CH2FVAL_SHIFT                 (8U)
/*! CH2FVAL - Channel 2 Filter Value */
#define TPM_FILTER_CH2FVAL(x)                    (((uint32_t)(((uint32_t)(x)) << TPM_FILTER_CH2FVAL_SHIFT)) & TPM_FILTER_CH2FVAL_MASK)

#define TPM_FILTER_CH3FVAL_MASK                  (0xF000U)
#define TPM_FILTER_CH3FVAL_SHIFT                 (12U)
/*! CH3FVAL - Channel 3 Filter Value */
#define TPM_FILTER_CH3FVAL(x)                    (((uint32_t)(((uint32_t)(x)) << TPM_FILTER_CH3FVAL_SHIFT)) & TPM_FILTER_CH3FVAL_MASK)
/*! @} */

/*! @name QDCTRL - Quadrature Decoder Control and Status */
/*! @{ */

#define TPM_QDCTRL_QUADEN_MASK                   (0x1U)
#define TPM_QDCTRL_QUADEN_SHIFT                  (0U)
/*! QUADEN - Quadrature Decoder Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define TPM_QDCTRL_QUADEN(x)                     (((uint32_t)(((uint32_t)(x)) << TPM_QDCTRL_QUADEN_SHIFT)) & TPM_QDCTRL_QUADEN_MASK)

#define TPM_QDCTRL_TOFDIR_MASK                   (0x2U)
#define TPM_QDCTRL_TOFDIR_SHIFT                  (1U)
/*! TOFDIR - Timer Overflow Direction
 *  0b0..Bottom of counting
 *  0b1..Top of counting
 */
#define TPM_QDCTRL_TOFDIR(x)                     (((uint32_t)(((uint32_t)(x)) << TPM_QDCTRL_TOFDIR_SHIFT)) & TPM_QDCTRL_TOFDIR_MASK)

#define TPM_QDCTRL_QUADIR_MASK                   (0x4U)
#define TPM_QDCTRL_QUADIR_SHIFT                  (2U)
/*! QUADIR - Counter Direction in Quadrature Decode Mode
 *  0b0..Decreasing (counter decrement)
 *  0b1..Increasing (counter increment)
 */
#define TPM_QDCTRL_QUADIR(x)                     (((uint32_t)(((uint32_t)(x)) << TPM_QDCTRL_QUADIR_SHIFT)) & TPM_QDCTRL_QUADIR_MASK)

#define TPM_QDCTRL_QUADMODE_MASK                 (0x8U)
#define TPM_QDCTRL_QUADMODE_SHIFT                (3U)
/*! QUADMODE - Quadrature Decoder Mode
 *  0b0..Phase encoding mode
 *  0b1..Count and direction encoding mode
 */
#define TPM_QDCTRL_QUADMODE(x)                   (((uint32_t)(((uint32_t)(x)) << TPM_QDCTRL_QUADMODE_SHIFT)) & TPM_QDCTRL_QUADMODE_MASK)
/*! @} */

/*! @name CONF - Configuration */
/*! @{ */

#define TPM_CONF_DOZEEN_MASK                     (0x20U)
#define TPM_CONF_DOZEEN_SHIFT                    (5U)
/*! DOZEEN - Doze Enable
 *  0b0..TPM counter continues
 *  0b1..TPM counter pauses
 */
#define TPM_CONF_DOZEEN(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_CONF_DOZEEN_SHIFT)) & TPM_CONF_DOZEEN_MASK)

#define TPM_CONF_DBGMODE_MASK                    (0xC0U)
#define TPM_CONF_DBGMODE_SHIFT                   (6U)
/*! DBGMODE - Debug Mode
 *  0b00..TPM counter pauses
 *  0b11..TPM counter continues
 */
#define TPM_CONF_DBGMODE(x)                      (((uint32_t)(((uint32_t)(x)) << TPM_CONF_DBGMODE_SHIFT)) & TPM_CONF_DBGMODE_MASK)

#define TPM_CONF_GTBSYNC_MASK                    (0x100U)
#define TPM_CONF_GTBSYNC_SHIFT                   (8U)
/*! GTBSYNC - GTB Synchronization
 *  0b0..Disable
 *  0b1..Enable
 */
#define TPM_CONF_GTBSYNC(x)                      (((uint32_t)(((uint32_t)(x)) << TPM_CONF_GTBSYNC_SHIFT)) & TPM_CONF_GTBSYNC_MASK)

#define TPM_CONF_GTBEEN_MASK                     (0x200U)
#define TPM_CONF_GTBEEN_SHIFT                    (9U)
/*! GTBEEN - GTB Enable
 *  0b0..Internally generated TPM counter
 *  0b1..Externally generated GTB counter
 */
#define TPM_CONF_GTBEEN(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_CONF_GTBEEN_SHIFT)) & TPM_CONF_GTBEEN_MASK)

#define TPM_CONF_CSOT_MASK                       (0x10000U)
#define TPM_CONF_CSOT_SHIFT                      (16U)
/*! CSOT - Counter Start on Trigger
 *  0b0..Counter starts immediately
 *  0b1..Counter starts after detection of a rising edge on the selected input trigger
 */
#define TPM_CONF_CSOT(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CONF_CSOT_SHIFT)) & TPM_CONF_CSOT_MASK)

#define TPM_CONF_CSOO_MASK                       (0x20000U)
#define TPM_CONF_CSOO_SHIFT                      (17U)
/*! CSOO - Counter Stop on Overflow
 *  0b0..TPM counter continues
 *  0b1..TPM counter stops
 */
#define TPM_CONF_CSOO(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CONF_CSOO_SHIFT)) & TPM_CONF_CSOO_MASK)

#define TPM_CONF_CROT_MASK                       (0x40000U)
#define TPM_CONF_CROT_SHIFT                      (18U)
/*! CROT - Counter Reload on Trigger
 *  0b0..No reload
 *  0b1..Reload
 */
#define TPM_CONF_CROT(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CONF_CROT_SHIFT)) & TPM_CONF_CROT_MASK)

#define TPM_CONF_CPOT_MASK                       (0x80000U)
#define TPM_CONF_CPOT_SHIFT                      (19U)
/*! CPOT - Counter Pause on Trigger
 *  0b0..TPM counter continues
 *  0b1..TPM counter pauses
 */
#define TPM_CONF_CPOT(x)                         (((uint32_t)(((uint32_t)(x)) << TPM_CONF_CPOT_SHIFT)) & TPM_CONF_CPOT_MASK)

#define TPM_CONF_TRGPOL_MASK                     (0x400000U)
#define TPM_CONF_TRGPOL_SHIFT                    (22U)
/*! TRGPOL - Trigger Polarity
 *  0b0..Active high
 *  0b1..Active low
 */
#define TPM_CONF_TRGPOL(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_CONF_TRGPOL_SHIFT)) & TPM_CONF_TRGPOL_MASK)

#define TPM_CONF_TRGSRC_MASK                     (0x800000U)
#define TPM_CONF_TRGSRC_SHIFT                    (23U)
/*! TRGSRC - Trigger Source
 *  0b0..External
 *  0b1..Internal (channel pin input capture)
 */
#define TPM_CONF_TRGSRC(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_CONF_TRGSRC_SHIFT)) & TPM_CONF_TRGSRC_MASK)

#define TPM_CONF_TRGSEL_MASK                     (0x3000000U)
#define TPM_CONF_TRGSEL_SHIFT                    (24U)
/*! TRGSEL - Trigger Select
 *  0b01..Channel 0 pin input capture
 *  0b10..Channel 1 pin input capture
 *  0b11..Channel 0 or channel 1 pin input capture
 */
#define TPM_CONF_TRGSEL(x)                       (((uint32_t)(((uint32_t)(x)) << TPM_CONF_TRGSEL_SHIFT)) & TPM_CONF_TRGSEL_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group TPM_Register_Masks */


/* TPM - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral TPM1 base address */
  #define TPM1_BASE                                (0x54310000u)
  /** Peripheral TPM1 base address */
  #define TPM1_BASE_NS                             (0x44310000u)
  /** Peripheral TPM1 base pointer */
  #define TPM1                                     ((TPM_Type *)TPM1_BASE)
  /** Peripheral TPM1 base pointer */
  #define TPM1_NS                                  ((TPM_Type *)TPM1_BASE_NS)
  /** Peripheral TPM2 base address */
  #define TPM2_BASE                                (0x54320000u)
  /** Peripheral TPM2 base address */
  #define TPM2_BASE_NS                             (0x44320000u)
  /** Peripheral TPM2 base pointer */
  #define TPM2                                     ((TPM_Type *)TPM2_BASE)
  /** Peripheral TPM2 base pointer */
  #define TPM2_NS                                  ((TPM_Type *)TPM2_BASE_NS)
  /** Peripheral TPM3 base address */
  #define TPM3_BASE                                (0x524E0000u)
  /** Peripheral TPM3 base address */
  #define TPM3_BASE_NS                             (0x424E0000u)
  /** Peripheral TPM3 base pointer */
  #define TPM3                                     ((TPM_Type *)TPM3_BASE)
  /** Peripheral TPM3 base pointer */
  #define TPM3_NS                                  ((TPM_Type *)TPM3_BASE_NS)
  /** Array initializer of TPM peripheral base addresses */
  #define TPM_BASE_ADDRS                           { 0u, TPM1_BASE, TPM2_BASE, TPM3_BASE }
  /** Array initializer of TPM peripheral base pointers */
  #define TPM_BASE_PTRS                            { (TPM_Type *)0u, TPM1, TPM2, TPM3 }
  /** Array initializer of TPM peripheral base addresses */
  #define TPM_BASE_ADDRS_NS                        { 0u, TPM1_BASE_NS, TPM2_BASE_NS, TPM3_BASE_NS }
  /** Array initializer of TPM peripheral base pointers */
  #define TPM_BASE_PTRS_NS                         { (TPM_Type *)0u, TPM1_NS, TPM2_NS, TPM3_NS }
#else
  /** Peripheral TPM1 base address */
  #define TPM1_BASE                                (0x44310000u)
  /** Peripheral TPM1 base pointer */
  #define TPM1                                     ((TPM_Type *)TPM1_BASE)
  /** Peripheral TPM2 base address */
  #define TPM2_BASE                                (0x44320000u)
  /** Peripheral TPM2 base pointer */
  #define TPM2                                     ((TPM_Type *)TPM2_BASE)
  /** Peripheral TPM3 base address */
  #define TPM3_BASE                                (0x424E0000u)
  /** Peripheral TPM3 base pointer */
  #define TPM3                                     ((TPM_Type *)TPM3_BASE)
  /** Array initializer of TPM peripheral base addresses */
  #define TPM_BASE_ADDRS                           { 0u, TPM1_BASE, TPM2_BASE, TPM3_BASE }
  /** Array initializer of TPM peripheral base pointers */
  #define TPM_BASE_PTRS                            { (TPM_Type *)0u, TPM1, TPM2, TPM3 }
#endif
/** Interrupt vectors for the TPM peripheral type */
#define TPM_IRQS                                 { NotAvail_IRQn, TPM1_IRQn, TPM2_IRQn, TPM3_IRQn }

/*!
 * @}
 */ /* end of group TPM_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- TRDC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TRDC_Peripheral_Access_Layer TRDC Peripheral Access Layer
 * @{
 */

/** TRDC - Register Layout Typedef */
typedef struct {
  __IO uint32_t TRDC_CR;                           /**< TRDC Register, offset: 0x0 */
       uint8_t RESERVED_0[236];
  __I  uint32_t TRDC_HWCFG0;                       /**< TRDC Hardware Configuration Register 0, offset: 0xF0 */
  __I  uint32_t TRDC_HWCFG1;                       /**< TRDC Hardware Configuration Register 1, offset: 0xF4 */
  __I  uint32_t TRDC_HWCFG2;                       /**< TRDC Hardware Configuration Register 2, offset: 0xF8 */
  __I  uint32_t TRDC_HWCFG3;                       /**< TRDC Hardware Configuration Register 3, offset: 0xFC */
  __I  uint8_t DACFG[6];                           /**< Domain Assignment Configuration Register, array offset: 0x100, array step: 0x1, irregular array, not all indices are valid */
       uint8_t RESERVED_1[186];
  __IO uint32_t TRDC_IDAU_CR;                      /**< TRDC IDAU Control Register, offset: 0x1C0 */
       uint8_t RESERVED_2[28];
  __IO uint32_t TRDC_FLW_CTL;                      /**< TRDC FLW Control, offset: 0x1E0 */
  __I  uint32_t TRDC_FLW_PBASE;                    /**< TRDC FLW Physical Base, offset: 0x1E4 */
  __IO uint32_t TRDC_FLW_ABASE;                    /**< TRDC FLW Array Base, offset: 0x1E8 */
  __IO uint32_t TRDC_FLW_BCNT;                     /**< TRDC FLW Block Count, offset: 0x1EC */
       uint8_t RESERVED_3[12];
  __IO uint32_t TRDC_FDID;                         /**< TRDC Fault Domain ID, offset: 0x1FC */
  __I  uint32_t TRDC_DERRLOC[16];                  /**< TRDC Domain Error Location Register, array offset: 0x200, array step: 0x4 */
       uint8_t RESERVED_4[448];
  struct {                                         /* offset: 0x400, array step: 0x10 */
    __I  uint32_t W0;                                /**< MBC Domain Error Word0 Register, array offset: 0x400, array step: 0x10, available only on: TRDC1, TRDC2 (missing on TRDC3) */
    __I  uint32_t W1;                                /**< MBC Domain Error Word1 Register, array offset: 0x404, array step: 0x10, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_0[4];
    __O  uint32_t W3;                                /**< MBC Domain Error Word3 Register, array offset: 0x40C, array step: 0x10, available only on: TRDC1, TRDC2 (missing on TRDC3) */
  } MBC_DERR[2];
       uint8_t RESERVED_5[96];
  struct {                                         /* offset: 0x480, array step: 0x10 */
    __I  uint32_t W0;                                /**< MRC Domain Error Word0 Register, array offset: 0x480, array step: 0x10, irregular array, not all indices are valid */
    __I  uint32_t W1;                                /**< MRC Domain Error Word1 Register, array offset: 0x484, array step: 0x10, irregular array, not all indices are valid */
         uint8_t RESERVED_0[4];
    __O  uint32_t W3;                                /**< MRC Domain Error Word3 Register, array offset: 0x48C, array step: 0x10, irregular array, not all indices are valid */
  } MRC_DERR[7];
       uint8_t RESERVED_6[528];
  __IO uint32_t PID[2];                            /**< Process Identifier, array offset: 0x700, array step: 0x4, available only on: TRDC1 (missing on TRDC2, TRDC3) */
       uint8_t RESERVED_7[248];
  union {                                          /* offset: 0x800 */
    struct {                                         /* offset: 0x800, array step: 0x20 */
      __IO uint32_t MDA_W_DFMT0[7];                    /**< DAC Master Domain Assignment Register, array offset: 0x800, array step: index*0x20, index2*0x4, irregular array, not all indices are valid */
           uint8_t RESERVED_0[4];
    } MDA_DFMT0[6];
    struct {                                         /* offset: 0x800, array step: 0x20 */
      __IO uint32_t MDA_W_DFMT1[1];                    /**< DAC Master Domain Assignment Register, array offset: 0x800, array step: index*0x20, index2*0x4, irregular array, not all indices are valid */
           uint8_t RESERVED_0[28];
    } MDA_DFMT1[5];
  };
       uint8_t RESERVED_8[63296];
  struct {                                         /* offset: 0x10000, array step: 0x2000 */
    __I  uint32_t MBC_MEM_GLBCFG[4];                 /**< MBC Global Configuration Register, array offset: 0x10000, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
    __IO uint32_t MBC_NSE_BLK_INDEX;                 /**< MBC NonSecure Enable Block Index, array offset: 0x10010, array step: 0x2000, available only on: TRDC1, TRDC2 (missing on TRDC3) */
    __O  uint32_t MBC_NSE_BLK_SET;                   /**< MBC NonSecure Enable Block Set, array offset: 0x10014, array step: 0x2000, available only on: TRDC1, TRDC2 (missing on TRDC3) */
    __O  uint32_t MBC_NSE_BLK_CLR;                   /**< MBC NonSecure Enable Block Clear, array offset: 0x10018, array step: 0x2000, available only on: TRDC1, TRDC2 (missing on TRDC3) */
    __O  uint32_t MBC_NSE_BLK_CLR_ALL;               /**< MBC NonSecure Enable Block Clear All, array offset: 0x1001C, array step: 0x2000, available only on: TRDC1, TRDC2 (missing on TRDC3) */
    __IO uint32_t MBC_MEMN_GLBAC[8];                 /**< MBC Global Access Control, array offset: 0x10020, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
    __IO uint32_t MBC_DOM0_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10040, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_0[192];
    __IO uint32_t MBC_DOM0_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10140, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_1[48];
    __IO uint32_t MBC_DOM0_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10180, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_2[16];
    __IO uint32_t MBC_DOM0_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x101A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_3[4];
    __IO uint32_t MBC_DOM0_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x101A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_4[28];
    __IO uint32_t MBC_DOM0_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x101C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_5[4];
    __IO uint32_t MBC_DOM0_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x101D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_6[20];
    __IO uint32_t MBC_DOM0_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x101F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_7[76];
    __IO uint32_t MBC_DOM1_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10240, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_8[192];
    __IO uint32_t MBC_DOM1_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10340, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_9[48];
    __IO uint32_t MBC_DOM1_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10380, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_10[16];
    __IO uint32_t MBC_DOM1_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x103A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_11[4];
    __IO uint32_t MBC_DOM1_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x103A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_12[28];
    __IO uint32_t MBC_DOM1_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x103C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_13[4];
    __IO uint32_t MBC_DOM1_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x103D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_14[20];
    __IO uint32_t MBC_DOM1_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x103F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_15[76];
    __IO uint32_t MBC_DOM2_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10440, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_16[192];
    __IO uint32_t MBC_DOM2_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10540, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_17[48];
    __IO uint32_t MBC_DOM2_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10580, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_18[16];
    __IO uint32_t MBC_DOM2_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x105A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_19[4];
    __IO uint32_t MBC_DOM2_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x105A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_20[28];
    __IO uint32_t MBC_DOM2_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x105C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_21[4];
    __IO uint32_t MBC_DOM2_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x105D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_22[20];
    __IO uint32_t MBC_DOM2_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x105F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_23[76];
    __IO uint32_t MBC_DOM3_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10640, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_24[192];
    __IO uint32_t MBC_DOM3_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10740, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_25[48];
    __IO uint32_t MBC_DOM3_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10780, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_26[16];
    __IO uint32_t MBC_DOM3_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x107A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_27[4];
    __IO uint32_t MBC_DOM3_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x107A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_28[28];
    __IO uint32_t MBC_DOM3_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x107C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_29[4];
    __IO uint32_t MBC_DOM3_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x107D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_30[20];
    __IO uint32_t MBC_DOM3_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x107F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_31[76];
    __IO uint32_t MBC_DOM4_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10840, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_32[192];
    __IO uint32_t MBC_DOM4_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10940, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_33[48];
    __IO uint32_t MBC_DOM4_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10980, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_34[16];
    __IO uint32_t MBC_DOM4_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x109A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_35[4];
    __IO uint32_t MBC_DOM4_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x109A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_36[28];
    __IO uint32_t MBC_DOM4_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x109C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_37[4];
    __IO uint32_t MBC_DOM4_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x109D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_38[20];
    __IO uint32_t MBC_DOM4_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x109F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_39[76];
    __IO uint32_t MBC_DOM5_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10A40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_40[192];
    __IO uint32_t MBC_DOM5_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10B40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_41[48];
    __IO uint32_t MBC_DOM5_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10B80, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_42[16];
    __IO uint32_t MBC_DOM5_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10BA0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_43[4];
    __IO uint32_t MBC_DOM5_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x10BA8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_44[28];
    __IO uint32_t MBC_DOM5_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10BC8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_45[4];
    __IO uint32_t MBC_DOM5_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x10BD0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_46[20];
    __IO uint32_t MBC_DOM5_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10BF0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_47[76];
    __IO uint32_t MBC_DOM6_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10C40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_48[192];
    __IO uint32_t MBC_DOM6_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10D40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_49[48];
    __IO uint32_t MBC_DOM6_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10D80, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_50[16];
    __IO uint32_t MBC_DOM6_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10DA0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_51[4];
    __IO uint32_t MBC_DOM6_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x10DA8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_52[28];
    __IO uint32_t MBC_DOM6_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10DC8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_53[4];
    __IO uint32_t MBC_DOM6_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x10DD0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_54[20];
    __IO uint32_t MBC_DOM6_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10DF0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_55[76];
    __IO uint32_t MBC_DOM7_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x10E40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_56[192];
    __IO uint32_t MBC_DOM7_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10F40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_57[48];
    __IO uint32_t MBC_DOM7_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x10F80, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_58[16];
    __IO uint32_t MBC_DOM7_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10FA0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_59[4];
    __IO uint32_t MBC_DOM7_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x10FA8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_60[28];
    __IO uint32_t MBC_DOM7_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10FC8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_61[4];
    __IO uint32_t MBC_DOM7_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x10FD0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_62[20];
    __IO uint32_t MBC_DOM7_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x10FF0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_63[76];
    __IO uint32_t MBC_DOM8_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x11040, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_64[192];
    __IO uint32_t MBC_DOM8_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11140, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_65[48];
    __IO uint32_t MBC_DOM8_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x11180, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_66[16];
    __IO uint32_t MBC_DOM8_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x111A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_67[4];
    __IO uint32_t MBC_DOM8_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x111A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_68[28];
    __IO uint32_t MBC_DOM8_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x111C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_69[4];
    __IO uint32_t MBC_DOM8_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x111D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_70[20];
    __IO uint32_t MBC_DOM8_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x111F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_71[76];
    __IO uint32_t MBC_DOM9_MEM0_BLK_CFG_W[16];       /**< MBC Memory Block Configuration Word, array offset: 0x11240, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_72[192];
    __IO uint32_t MBC_DOM9_MEM0_BLK_NSE_W[4];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11340, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_73[48];
    __IO uint32_t MBC_DOM9_MEM1_BLK_CFG_W[4];        /**< MBC Memory Block Configuration Word, array offset: 0x11380, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_74[16];
    __IO uint32_t MBC_DOM9_MEM1_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x113A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_75[4];
    __IO uint32_t MBC_DOM9_MEM2_BLK_CFG_W[1];        /**< MBC Memory Block Configuration Word, array offset: 0x113A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_76[28];
    __IO uint32_t MBC_DOM9_MEM2_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x113C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_77[4];
    __IO uint32_t MBC_DOM9_MEM3_BLK_CFG_W[3];        /**< MBC Memory Block Configuration Word, array offset: 0x113D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_78[20];
    __IO uint32_t MBC_DOM9_MEM3_BLK_NSE_W[1];        /**< MBC Memory Block NonSecure Enable Word, array offset: 0x113F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_79[76];
    __IO uint32_t MBC_DOM10_MEM0_BLK_CFG_W[16];      /**< MBC Memory Block Configuration Word, array offset: 0x11440, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_80[192];
    __IO uint32_t MBC_DOM10_MEM0_BLK_NSE_W[4];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11540, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_81[48];
    __IO uint32_t MBC_DOM10_MEM1_BLK_CFG_W[4];       /**< MBC Memory Block Configuration Word, array offset: 0x11580, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_82[16];
    __IO uint32_t MBC_DOM10_MEM1_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x115A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_83[4];
    __IO uint32_t MBC_DOM10_MEM2_BLK_CFG_W[1];       /**< MBC Memory Block Configuration Word, array offset: 0x115A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_84[28];
    __IO uint32_t MBC_DOM10_MEM2_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x115C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_85[4];
    __IO uint32_t MBC_DOM10_MEM3_BLK_CFG_W[3];       /**< MBC Memory Block Configuration Word, array offset: 0x115D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_86[20];
    __IO uint32_t MBC_DOM10_MEM3_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x115F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_87[76];
    __IO uint32_t MBC_DOM11_MEM0_BLK_CFG_W[16];      /**< MBC Memory Block Configuration Word, array offset: 0x11640, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_88[192];
    __IO uint32_t MBC_DOM11_MEM0_BLK_NSE_W[4];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11740, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_89[48];
    __IO uint32_t MBC_DOM11_MEM1_BLK_CFG_W[4];       /**< MBC Memory Block Configuration Word, array offset: 0x11780, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_90[16];
    __IO uint32_t MBC_DOM11_MEM1_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x117A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_91[4];
    __IO uint32_t MBC_DOM11_MEM2_BLK_CFG_W[1];       /**< MBC Memory Block Configuration Word, array offset: 0x117A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_92[28];
    __IO uint32_t MBC_DOM11_MEM2_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x117C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_93[4];
    __IO uint32_t MBC_DOM11_MEM3_BLK_CFG_W[3];       /**< MBC Memory Block Configuration Word, array offset: 0x117D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_94[20];
    __IO uint32_t MBC_DOM11_MEM3_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x117F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_95[76];
    __IO uint32_t MBC_DOM12_MEM0_BLK_CFG_W[16];      /**< MBC Memory Block Configuration Word, array offset: 0x11840, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_96[192];
    __IO uint32_t MBC_DOM12_MEM0_BLK_NSE_W[4];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11940, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_97[48];
    __IO uint32_t MBC_DOM12_MEM1_BLK_CFG_W[4];       /**< MBC Memory Block Configuration Word, array offset: 0x11980, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_98[16];
    __IO uint32_t MBC_DOM12_MEM1_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x119A0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_99[4];
    __IO uint32_t MBC_DOM12_MEM2_BLK_CFG_W[1];       /**< MBC Memory Block Configuration Word, array offset: 0x119A8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_100[28];
    __IO uint32_t MBC_DOM12_MEM2_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x119C8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_101[4];
    __IO uint32_t MBC_DOM12_MEM3_BLK_CFG_W[3];       /**< MBC Memory Block Configuration Word, array offset: 0x119D0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_102[20];
    __IO uint32_t MBC_DOM12_MEM3_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x119F0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_103[76];
    __IO uint32_t MBC_DOM13_MEM0_BLK_CFG_W[16];      /**< MBC Memory Block Configuration Word, array offset: 0x11A40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_104[192];
    __IO uint32_t MBC_DOM13_MEM0_BLK_NSE_W[4];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11B40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_105[48];
    __IO uint32_t MBC_DOM13_MEM1_BLK_CFG_W[4];       /**< MBC Memory Block Configuration Word, array offset: 0x11B80, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_106[16];
    __IO uint32_t MBC_DOM13_MEM1_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11BA0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_107[4];
    __IO uint32_t MBC_DOM13_MEM2_BLK_CFG_W[1];       /**< MBC Memory Block Configuration Word, array offset: 0x11BA8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_108[28];
    __IO uint32_t MBC_DOM13_MEM2_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11BC8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_109[4];
    __IO uint32_t MBC_DOM13_MEM3_BLK_CFG_W[3];       /**< MBC Memory Block Configuration Word, array offset: 0x11BD0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_110[20];
    __IO uint32_t MBC_DOM13_MEM3_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11BF0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_111[76];
    __IO uint32_t MBC_DOM14_MEM0_BLK_CFG_W[16];      /**< MBC Memory Block Configuration Word, array offset: 0x11C40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_112[192];
    __IO uint32_t MBC_DOM14_MEM0_BLK_NSE_W[4];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11D40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_113[48];
    __IO uint32_t MBC_DOM14_MEM1_BLK_CFG_W[4];       /**< MBC Memory Block Configuration Word, array offset: 0x11D80, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_114[16];
    __IO uint32_t MBC_DOM14_MEM1_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11DA0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_115[4];
    __IO uint32_t MBC_DOM14_MEM2_BLK_CFG_W[1];       /**< MBC Memory Block Configuration Word, array offset: 0x11DA8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_116[28];
    __IO uint32_t MBC_DOM14_MEM2_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11DC8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_117[4];
    __IO uint32_t MBC_DOM14_MEM3_BLK_CFG_W[3];       /**< MBC Memory Block Configuration Word, array offset: 0x11DD0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_118[20];
    __IO uint32_t MBC_DOM14_MEM3_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11DF0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_119[76];
    __IO uint32_t MBC_DOM15_MEM0_BLK_CFG_W[16];      /**< MBC Memory Block Configuration Word, array offset: 0x11E40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_120[192];
    __IO uint32_t MBC_DOM15_MEM0_BLK_NSE_W[4];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11F40, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_121[48];
    __IO uint32_t MBC_DOM15_MEM1_BLK_CFG_W[4];       /**< MBC Memory Block Configuration Word, array offset: 0x11F80, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_122[16];
    __IO uint32_t MBC_DOM15_MEM1_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11FA0, array step: index*0x2000, index2*0x4, available only on: TRDC1, TRDC2 (missing on TRDC3) */
         uint8_t RESERVED_123[4];
    __IO uint32_t MBC_DOM15_MEM2_BLK_CFG_W[1];       /**< MBC Memory Block Configuration Word, array offset: 0x11FA8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_124[28];
    __IO uint32_t MBC_DOM15_MEM2_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11FC8, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_125[4];
    __IO uint32_t MBC_DOM15_MEM3_BLK_CFG_W[3];       /**< MBC Memory Block Configuration Word, array offset: 0x11FD0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_126[20];
    __IO uint32_t MBC_DOM15_MEM3_BLK_NSE_W[1];       /**< MBC Memory Block NonSecure Enable Word, array offset: 0x11FF0, array step: index*0x2000, index2*0x4, irregular array, not all indices are valid */
         uint8_t RESERVED_127[12];
  } MBC_INDEX[2];
  struct {                                         /* offset: 0x14000, array step: 0x1000 */
    __I  uint32_t MRC_GLBCFG;                        /**< MRC Global Configuration Register, array offset: 0x14000, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_0[12];
    __IO uint32_t MRC_NSE_RGN_INDIRECT;              /**< MRC NonSecure Enable Region Indirect, array offset: 0x14010, array step: 0x1000, irregular array, not all indices are valid */
    __O  uint32_t MRC_NSE_RGN_SET;                   /**< MRC NonSecure Enable Region Set, array offset: 0x14014, array step: 0x1000, irregular array, not all indices are valid */
    __O  uint32_t MRC_NSE_RGN_CLR;                   /**< MRC NonSecure Enable Region Clear, array offset: 0x14018, array step: 0x1000, irregular array, not all indices are valid */
    __O  uint32_t MRC_NSE_RGN_CLR_ALL;               /**< MRC NonSecure Enable Region Clear All, array offset: 0x1401C, array step: 0x1000, irregular array, not all indices are valid */
    __IO uint32_t MRC_GLBAC[8];                      /**< MRC Global Access Control, array offset: 0x14020, array step: index*0x1000, index2*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM0_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14040, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM0_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x140C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_1[124];
    __IO uint32_t MRC_DOM1_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14140, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM1_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x141C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_2[124];
    __IO uint32_t MRC_DOM2_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14240, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM2_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x142C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_3[124];
    __IO uint32_t MRC_DOM3_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14340, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM3_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x143C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_4[124];
    __IO uint32_t MRC_DOM4_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14440, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM4_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x144C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_5[124];
    __IO uint32_t MRC_DOM5_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14540, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM5_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x145C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_6[124];
    __IO uint32_t MRC_DOM6_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14640, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM6_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x146C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_7[124];
    __IO uint32_t MRC_DOM7_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14740, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM7_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x147C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_8[124];
    __IO uint32_t MRC_DOM8_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14840, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM8_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x148C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_9[124];
    __IO uint32_t MRC_DOM9_RGD_W[16][2];             /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14940, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM9_RGD_NSE;                  /**< MRC Region Descriptor NonSecure Enable, array offset: 0x149C0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_10[124];
    __IO uint32_t MRC_DOM10_RGD_W[16][2];            /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14A40, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM10_RGD_NSE;                 /**< MRC Region Descriptor NonSecure Enable, array offset: 0x14AC0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_11[124];
    __IO uint32_t MRC_DOM11_RGD_W[16][2];            /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14B40, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM11_RGD_NSE;                 /**< MRC Region Descriptor NonSecure Enable, array offset: 0x14BC0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_12[124];
    __IO uint32_t MRC_DOM12_RGD_W[16][2];            /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14C40, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM12_RGD_NSE;                 /**< MRC Region Descriptor NonSecure Enable, array offset: 0x14CC0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_13[124];
    __IO uint32_t MRC_DOM13_RGD_W[16][2];            /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14D40, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM13_RGD_NSE;                 /**< MRC Region Descriptor NonSecure Enable, array offset: 0x14DC0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_14[124];
    __IO uint32_t MRC_DOM14_RGD_W[16][2];            /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14E40, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM14_RGD_NSE;                 /**< MRC Region Descriptor NonSecure Enable, array offset: 0x14EC0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_15[124];
    __IO uint32_t MRC_DOM15_RGD_W[16][2];            /**< MRC Region Descriptor Word 0..MRC Region Descriptor Word 1, array offset: 0x14F40, array step: index*0x1000, index2*0x8, index3*0x4, irregular array, not all indices are valid */
    __IO uint32_t MRC_DOM15_RGD_NSE;                 /**< MRC Region Descriptor NonSecure Enable, array offset: 0x14FC0, array step: 0x1000, irregular array, not all indices are valid */
         uint8_t RESERVED_16[60];
  } MRC_INDEX[7];
} TRDC_Type;

/* ----------------------------------------------------------------------------
   -- TRDC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TRDC_Register_Masks TRDC Register Masks
 * @{
 */

/*! @name TRDC_CR - TRDC Register */
/*! @{ */

#define TRDC_TRDC_CR_GVLDM_MASK                  (0x1U)
#define TRDC_TRDC_CR_GVLDM_SHIFT                 (0U)
/*! GVLDM - Global Valid for Domain Assignment Controllers
 *  0b0..TRDC DACs are disabled.
 *  0b1..TRDC DACs are enabled.
 */
#define TRDC_TRDC_CR_GVLDM(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_CR_GVLDM_SHIFT)) & TRDC_TRDC_CR_GVLDM_MASK)

#define TRDC_TRDC_CR_HRL_MASK                    (0x1EU)
#define TRDC_TRDC_CR_HRL_SHIFT                   (1U)
/*! HRL - Hardware Revision Level */
#define TRDC_TRDC_CR_HRL(x)                      (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_CR_HRL_SHIFT)) & TRDC_TRDC_CR_HRL_MASK)

#define TRDC_TRDC_CR_GVLDB_MASK                  (0x4000U)
#define TRDC_TRDC_CR_GVLDB_SHIFT                 (14U)
/*! GVLDB - Global Valid for Memory Block Checkers
 *  0b0..TRDC MBCs are disabled.
 *  0b1..TRDC MBCs are enabled.
 */
#define TRDC_TRDC_CR_GVLDB(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_CR_GVLDB_SHIFT)) & TRDC_TRDC_CR_GVLDB_MASK)

#define TRDC_TRDC_CR_GVLDR_MASK                  (0x8000U)
#define TRDC_TRDC_CR_GVLDR_SHIFT                 (15U)
/*! GVLDR - Global Valid for Memory Region Checkers
 *  0b0..TRDC MRCs are disabled.
 *  0b1..TRDC MRCs are enabled.
 */
#define TRDC_TRDC_CR_GVLDR(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_CR_GVLDR_SHIFT)) & TRDC_TRDC_CR_GVLDR_MASK)

#define TRDC_TRDC_CR_LK1_MASK                    (0x40000000U)
#define TRDC_TRDC_CR_LK1_SHIFT                   (30U)
/*! LK1 - Lock Status
 *  0b0..The CR can be written by any secure privileged write.
 *  0b1..The CR is locked (read-only) until the next reset.
 */
#define TRDC_TRDC_CR_LK1(x)                      (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_CR_LK1_SHIFT)) & TRDC_TRDC_CR_LK1_MASK)
/*! @} */

/*! @name TRDC_HWCFG0 - TRDC Hardware Configuration Register 0 */
/*! @{ */

#define TRDC_TRDC_HWCFG0_NDID_MASK               (0x1FU)
#define TRDC_TRDC_HWCFG0_NDID_SHIFT              (0U)
/*! NDID - Number of domains */
#define TRDC_TRDC_HWCFG0_NDID(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG0_NDID_SHIFT)) & TRDC_TRDC_HWCFG0_NDID_MASK)

#define TRDC_TRDC_HWCFG0_NMSTR_MASK              (0xFF00U)
#define TRDC_TRDC_HWCFG0_NMSTR_SHIFT             (8U)
/*! NMSTR - Number of bus masters */
#define TRDC_TRDC_HWCFG0_NMSTR(x)                (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG0_NMSTR_SHIFT)) & TRDC_TRDC_HWCFG0_NMSTR_MASK)

#define TRDC_TRDC_HWCFG0_NMBC_MASK               (0xF0000U)
#define TRDC_TRDC_HWCFG0_NMBC_SHIFT              (16U)
/*! NMBC - Number of MBCs */
#define TRDC_TRDC_HWCFG0_NMBC(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG0_NMBC_SHIFT)) & TRDC_TRDC_HWCFG0_NMBC_MASK)

#define TRDC_TRDC_HWCFG0_NMRC_MASK               (0x1F000000U)
#define TRDC_TRDC_HWCFG0_NMRC_SHIFT              (24U)
/*! NMRC - Number of MRCs */
#define TRDC_TRDC_HWCFG0_NMRC(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG0_NMRC_SHIFT)) & TRDC_TRDC_HWCFG0_NMRC_MASK)

#define TRDC_TRDC_HWCFG0_MID_MASK                (0xE0000000U)
#define TRDC_TRDC_HWCFG0_MID_SHIFT               (29U)
/*! MID - Module ID */
#define TRDC_TRDC_HWCFG0_MID(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG0_MID_SHIFT)) & TRDC_TRDC_HWCFG0_MID_MASK)
/*! @} */

/*! @name TRDC_HWCFG1 - TRDC Hardware Configuration Register 1 */
/*! @{ */

#define TRDC_TRDC_HWCFG1_DID_MASK                (0xFU)
#define TRDC_TRDC_HWCFG1_DID_SHIFT               (0U)
/*! DID - Domain identifier number */
#define TRDC_TRDC_HWCFG1_DID(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG1_DID_SHIFT)) & TRDC_TRDC_HWCFG1_DID_MASK)
/*! @} */

/*! @name TRDC_HWCFG2 - TRDC Hardware Configuration Register 2 */
/*! @{ */

#define TRDC_TRDC_HWCFG2_PIDPn_MASK              (0xFFFFFFFFU)
#define TRDC_TRDC_HWCFG2_PIDPn_SHIFT             (0U)
/*! PIDPn - Process identifier present */
#define TRDC_TRDC_HWCFG2_PIDPn(x)                (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG2_PIDPn_SHIFT)) & TRDC_TRDC_HWCFG2_PIDPn_MASK)
/*! @} */

/*! @name TRDC_HWCFG3 - TRDC Hardware Configuration Register 3 */
/*! @{ */

#define TRDC_TRDC_HWCFG3_PIDPn_MASK              (0xFFFFFFFFU)
#define TRDC_TRDC_HWCFG3_PIDPn_SHIFT             (0U)
/*! PIDPn - Process identifier present */
#define TRDC_TRDC_HWCFG3_PIDPn(x)                (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_HWCFG3_PIDPn_SHIFT)) & TRDC_TRDC_HWCFG3_PIDPn_MASK)
/*! @} */

/*! @name DACFG - Domain Assignment Configuration Register */
/*! @{ */

#define TRDC_DACFG_NMDAR_MASK                    (0xFU)
#define TRDC_DACFG_NMDAR_SHIFT                   (0U)
/*! NMDAR - Number of master domain assignment registers for bus master m */
#define TRDC_DACFG_NMDAR(x)                      (((uint8_t)(((uint8_t)(x)) << TRDC_DACFG_NMDAR_SHIFT)) & TRDC_DACFG_NMDAR_MASK)

#define TRDC_DACFG_NCM_MASK                      (0x80U)
#define TRDC_DACFG_NCM_SHIFT                     (7U)
/*! NCM - Non-CPU Master
 *  0b0..Bus master is a processor.
 *  0b1..Bus master is a non-processor.
 */
#define TRDC_DACFG_NCM(x)                        (((uint8_t)(((uint8_t)(x)) << TRDC_DACFG_NCM_SHIFT)) & TRDC_DACFG_NCM_MASK)
/*! @} */

/* The count of TRDC_DACFG */
#define TRDC_DACFG_COUNT                         (6U)

/*! @name TRDC_IDAU_CR - TRDC IDAU Control Register */
/*! @{ */

#define TRDC_TRDC_IDAU_CR_VLD_MASK               (0x1U)
#define TRDC_TRDC_IDAU_CR_VLD_SHIFT              (0U)
/*! VLD - Valid */
#define TRDC_TRDC_IDAU_CR_VLD(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_VLD_SHIFT)) & TRDC_TRDC_IDAU_CR_VLD_MASK)

#define TRDC_TRDC_IDAU_CR_CFGSECEXT_MASK         (0x8U)
#define TRDC_TRDC_IDAU_CR_CFGSECEXT_SHIFT        (3U)
/*! CFGSECEXT - Configure Security Extension
 *  0b0..Armv8M Security Extension is disabled
 *  0b1..Armv8-M Security Extension is enabled
 */
#define TRDC_TRDC_IDAU_CR_CFGSECEXT(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_CFGSECEXT_SHIFT)) & TRDC_TRDC_IDAU_CR_CFGSECEXT_MASK)

#define TRDC_TRDC_IDAU_CR_MPUSDIS_MASK           (0x10U)
#define TRDC_TRDC_IDAU_CR_MPUSDIS_SHIFT          (4U)
/*! MPUSDIS - Secure Memory Protection Unit Disabled
 *  0b0..Secure MPU is enabled
 *  0b1..Secure MPU is disabled
 */
#define TRDC_TRDC_IDAU_CR_MPUSDIS(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_MPUSDIS_SHIFT)) & TRDC_TRDC_IDAU_CR_MPUSDIS_MASK)

#define TRDC_TRDC_IDAU_CR_MPUNSDIS_MASK          (0x20U)
#define TRDC_TRDC_IDAU_CR_MPUNSDIS_SHIFT         (5U)
/*! MPUNSDIS - NonSecure Memory Protection Unit Disabled
 *  0b0..Nonsecure MPU is enabled
 *  0b1..Nonsecure MPU is disabled
 */
#define TRDC_TRDC_IDAU_CR_MPUNSDIS(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_MPUNSDIS_SHIFT)) & TRDC_TRDC_IDAU_CR_MPUNSDIS_MASK)

#define TRDC_TRDC_IDAU_CR_SAUDIS_MASK            (0x40U)
#define TRDC_TRDC_IDAU_CR_SAUDIS_SHIFT           (6U)
/*! SAUDIS - Security Attribution Unit Disable
 *  0b0..SAU is enabled
 *  0b1..SAU is disabled
 */
#define TRDC_TRDC_IDAU_CR_SAUDIS(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_SAUDIS_SHIFT)) & TRDC_TRDC_IDAU_CR_SAUDIS_MASK)

#define TRDC_TRDC_IDAU_CR_LKSVTAIRCR_MASK        (0x100U)
#define TRDC_TRDC_IDAU_CR_LKSVTAIRCR_SHIFT       (8U)
/*! LKSVTAIRCR - Lock Secure VTOR, Application interrupt and Reset Control Registers
 *  0b0..Unlock these registers
 *  0b1..Disable writes to the VTOR_S, AIRCR[PRIS], and AIRCR[BFHFNMINS] registers
 */
#define TRDC_TRDC_IDAU_CR_LKSVTAIRCR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_LKSVTAIRCR_SHIFT)) & TRDC_TRDC_IDAU_CR_LKSVTAIRCR_MASK)

#define TRDC_TRDC_IDAU_CR_LKNSVTOR_MASK          (0x200U)
#define TRDC_TRDC_IDAU_CR_LKNSVTOR_SHIFT         (9U)
/*! LKNSVTOR - Lock Nonsecure Vector Table Offset Register
 *  0b0..Unlock this register
 *  0b1..Disable writes to the VTOR_NS register
 */
#define TRDC_TRDC_IDAU_CR_LKNSVTOR(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_LKNSVTOR_SHIFT)) & TRDC_TRDC_IDAU_CR_LKNSVTOR_MASK)

#define TRDC_TRDC_IDAU_CR_LKSMPU_MASK            (0x400U)
#define TRDC_TRDC_IDAU_CR_LKSMPU_SHIFT           (10U)
/*! LKSMPU - Lock Secure MPU
 *  0b0..Unlock these registers
 *  0b1..Disable writes to the MPU_CTRL, MPU_RNR, MPU_RBAR, MPU_RLAR, MPU_RBAR_An and MPU_RLAR_An from software or
 *       from a debug agent connected to the processor in Secure state
 */
#define TRDC_TRDC_IDAU_CR_LKSMPU(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_LKSMPU_SHIFT)) & TRDC_TRDC_IDAU_CR_LKSMPU_MASK)

#define TRDC_TRDC_IDAU_CR_LKNSMPU_MASK           (0x800U)
#define TRDC_TRDC_IDAU_CR_LKNSMPU_SHIFT          (11U)
/*! LKNSMPU - Lock Nonsecure MPU
 *  0b0..Unlock these registers
 *  0b1..Disable writes to the MPU_CTRL_NS, MPU_RNR_NS, MPU_RBAR_NS, MPU_RLAR_NS, MPU_RBAR_A_NSn and
 *       MPU_RLAR_A_NSn from software or from a debug agent connected to the processor
 */
#define TRDC_TRDC_IDAU_CR_LKNSMPU(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_LKNSMPU_SHIFT)) & TRDC_TRDC_IDAU_CR_LKNSMPU_MASK)

#define TRDC_TRDC_IDAU_CR_LKSAU_MASK             (0x1000U)
#define TRDC_TRDC_IDAU_CR_LKSAU_SHIFT            (12U)
/*! LKSAU - Lock SAU
 *  0b0..Unlock these registers
 *  0b1..Disable writes to the SAU_CTRL, SAU_RNR, SAU_RBAR and SAU_RLAR registers from software or from a debug agent connected to the processor
 */
#define TRDC_TRDC_IDAU_CR_LKSAU(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_LKSAU_SHIFT)) & TRDC_TRDC_IDAU_CR_LKSAU_MASK)

#define TRDC_TRDC_IDAU_CR_PCURRNS_MASK           (0x80000000U)
#define TRDC_TRDC_IDAU_CR_PCURRNS_SHIFT          (31U)
/*! PCURRNS - Processor current security
 *  0b0..Processor is in Secure state
 *  0b1..Processor is in Nonsecure state
 */
#define TRDC_TRDC_IDAU_CR_PCURRNS(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_IDAU_CR_PCURRNS_SHIFT)) & TRDC_TRDC_IDAU_CR_PCURRNS_MASK)
/*! @} */

/*! @name TRDC_FLW_CTL - TRDC FLW Control */
/*! @{ */

#define TRDC_TRDC_FLW_CTL_LK_MASK                (0x40000000U)
#define TRDC_TRDC_FLW_CTL_LK_SHIFT               (30U)
/*! LK - Lock bit
 *  0b0..FLW registers may be modified.
 *  0b1..FLW registers are locked until the next reset.
 */
#define TRDC_TRDC_FLW_CTL_LK(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_FLW_CTL_LK_SHIFT)) & TRDC_TRDC_FLW_CTL_LK_MASK)

#define TRDC_TRDC_FLW_CTL_V_MASK                 (0x80000000U)
#define TRDC_TRDC_FLW_CTL_V_SHIFT                (31U)
/*! V - Valid bit
 *  0b0..FLW function is disabled.
 *  0b1..FLW function is enabled.
 */
#define TRDC_TRDC_FLW_CTL_V(x)                   (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_FLW_CTL_V_SHIFT)) & TRDC_TRDC_FLW_CTL_V_MASK)
/*! @} */

/*! @name TRDC_FLW_PBASE - TRDC FLW Physical Base */
/*! @{ */

#define TRDC_TRDC_FLW_PBASE_PBASE_MASK           (0xFFFFFFFFU)
#define TRDC_TRDC_FLW_PBASE_PBASE_SHIFT          (0U)
/*! PBASE - Physical base address */
#define TRDC_TRDC_FLW_PBASE_PBASE(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_FLW_PBASE_PBASE_SHIFT)) & TRDC_TRDC_FLW_PBASE_PBASE_MASK)
/*! @} */

/*! @name TRDC_FLW_ABASE - TRDC FLW Array Base */
/*! @{ */

#define TRDC_TRDC_FLW_ABASE_ABASE_L_MASK         (0x3F8000U)
#define TRDC_TRDC_FLW_ABASE_ABASE_L_SHIFT        (15U)
/*! ABASE_L - Array base address low */
#define TRDC_TRDC_FLW_ABASE_ABASE_L(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_FLW_ABASE_ABASE_L_SHIFT)) & TRDC_TRDC_FLW_ABASE_ABASE_L_MASK)

#define TRDC_TRDC_FLW_ABASE_ABASE_H_MASK         (0xFFC00000U)
#define TRDC_TRDC_FLW_ABASE_ABASE_H_SHIFT        (22U)
/*! ABASE_H - Array base address high */
#define TRDC_TRDC_FLW_ABASE_ABASE_H(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_FLW_ABASE_ABASE_H_SHIFT)) & TRDC_TRDC_FLW_ABASE_ABASE_H_MASK)
/*! @} */

/*! @name TRDC_FLW_BCNT - TRDC FLW Block Count */
/*! @{ */

#define TRDC_TRDC_FLW_BCNT_BCNT_MASK             (0x7FFFU)
#define TRDC_TRDC_FLW_BCNT_BCNT_SHIFT            (0U)
/*! BCNT - Block Count */
#define TRDC_TRDC_FLW_BCNT_BCNT(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_FLW_BCNT_BCNT_SHIFT)) & TRDC_TRDC_FLW_BCNT_BCNT_MASK)
/*! @} */

/*! @name TRDC_FDID - TRDC Fault Domain ID */
/*! @{ */

#define TRDC_TRDC_FDID_FDID_MASK                 (0xFU)
#define TRDC_TRDC_FDID_FDID_SHIFT                (0U)
/*! FDID - Domain ID of Faulted Access */
#define TRDC_TRDC_FDID_FDID(x)                   (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_FDID_FDID_SHIFT)) & TRDC_TRDC_FDID_FDID_MASK)
/*! @} */

/*! @name TRDC_DERRLOC - TRDC Domain Error Location Register */
/*! @{ */

#define TRDC_TRDC_DERRLOC_MBCINST_MASK           (0xFFU)
#define TRDC_TRDC_DERRLOC_MBCINST_SHIFT          (0U)
/*! MBCINST - MBC instance */
#define TRDC_TRDC_DERRLOC_MBCINST(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_DERRLOC_MBCINST_SHIFT)) & TRDC_TRDC_DERRLOC_MBCINST_MASK)

#define TRDC_TRDC_DERRLOC_MRCINST_MASK           (0xFFFF0000U)
#define TRDC_TRDC_DERRLOC_MRCINST_SHIFT          (16U)
/*! MRCINST - MRC instance */
#define TRDC_TRDC_DERRLOC_MRCINST(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_TRDC_DERRLOC_MRCINST_SHIFT)) & TRDC_TRDC_DERRLOC_MRCINST_MASK)
/*! @} */

/* The count of TRDC_TRDC_DERRLOC */
#define TRDC_TRDC_DERRLOC_COUNT                  (16U)

/*! @name W0 - MBC Domain Error Word0 Register */
/*! @{ */

#define TRDC_W0_EADDR_MASK                       (0xFFFFFFFFU)
#define TRDC_W0_EADDR_SHIFT                      (0U)
/*! EADDR - Error address */
#define TRDC_W0_EADDR(x)                         (((uint32_t)(((uint32_t)(x)) << TRDC_W0_EADDR_SHIFT)) & TRDC_W0_EADDR_MASK)
/*! @} */

/* The count of TRDC_W0 */
#define TRDC_W0_COUNT                            (2U)

/*! @name W1 - MBC Domain Error Word1 Register */
/*! @{ */

#define TRDC_W1_EDID_MASK                        (0xFU)
#define TRDC_W1_EDID_SHIFT                       (0U)
/*! EDID - Error domain identifier */
#define TRDC_W1_EDID(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EDID_SHIFT)) & TRDC_W1_EDID_MASK)

#define TRDC_W1_EATR_MASK                        (0x700U)
#define TRDC_W1_EATR_SHIFT                       (8U)
/*! EATR - Error attributes
 *  0b000..Secure user mode, instruction fetch access.
 *  0b001..Secure user mode, data access.
 *  0b010..Secure privileged mode, instruction fetch access.
 *  0b011..Secure privileged mode, data access.
 *  0b100..Nonsecure user mode, instruction fetch access.
 *  0b101..Nonsecure user mode, data access.
 *  0b110..Nonsecure privileged mode, instruction fetch access.
 *  0b111..Nonsecure privileged mode, data access.
 */
#define TRDC_W1_EATR(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EATR_SHIFT)) & TRDC_W1_EATR_MASK)

#define TRDC_W1_ERW_MASK                         (0x800U)
#define TRDC_W1_ERW_SHIFT                        (11U)
/*! ERW - Error read/write
 *  0b0..Read access
 *  0b1..Write access
 */
#define TRDC_W1_ERW(x)                           (((uint32_t)(((uint32_t)(x)) << TRDC_W1_ERW_SHIFT)) & TRDC_W1_ERW_MASK)

#define TRDC_W1_EPORT_MASK                       (0x7000000U)
#define TRDC_W1_EPORT_SHIFT                      (24U)
/*! EPORT - Error port
 *  0b000..mbcxslv0
 *  0b001..mbcxslv1
 *  0b010..mbcxslv2
 *  0b011..mbcxslv3
 */
#define TRDC_W1_EPORT(x)                         (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EPORT_SHIFT)) & TRDC_W1_EPORT_MASK)

#define TRDC_W1_EST_MASK                         (0xC0000000U)
#define TRDC_W1_EST_SHIFT                        (30U)
/*! EST - Error state
 *  0b00..No access violation has been detected.
 *  0b01..No access violation has been detected.
 *  0b10..A single access violation has been detected.
 *  0b11..Multiple access violations for this domain have been detected by this submodule instance. Only the
 *        address and attribute information for the first error have been captured in DERR_W0_i and DERR_W1_i.
 */
#define TRDC_W1_EST(x)                           (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EST_SHIFT)) & TRDC_W1_EST_MASK)
/*! @} */

/* The count of TRDC_W1 */
#define TRDC_W1_COUNT                            (2U)

/*! @name W3 - MBC Domain Error Word3 Register */
/*! @{ */

#define TRDC_W3_RECR_MASK                        (0xC0000000U)
#define TRDC_W3_RECR_SHIFT                       (30U)
/*! RECR - Rearm Error Capture Registers */
#define TRDC_W3_RECR(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_W3_RECR_SHIFT)) & TRDC_W3_RECR_MASK)
/*! @} */

/* The count of TRDC_W3 */
#define TRDC_W3_COUNT                            (2U)

/*! @name W0 - MRC Domain Error Word0 Register */
/*! @{ */

#define TRDC_W0_EADDR_MASK                       (0xFFFFFFFFU)
#define TRDC_W0_EADDR_SHIFT                      (0U)
/*! EADDR - Error address */
#define TRDC_W0_EADDR(x)                         (((uint32_t)(((uint32_t)(x)) << TRDC_W0_EADDR_SHIFT)) & TRDC_W0_EADDR_MASK)
/*! @} */

/* The count of TRDC_W0 */
#define TRDC_MRC_DERR_W0_COUNT                   (7U)

/*! @name W1 - MRC Domain Error Word1 Register */
/*! @{ */

#define TRDC_W1_EDID_MASK                        (0xFU)
#define TRDC_W1_EDID_SHIFT                       (0U)
/*! EDID - Error domain identifier */
#define TRDC_W1_EDID(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EDID_SHIFT)) & TRDC_W1_EDID_MASK)

#define TRDC_W1_EATR_MASK                        (0x700U)
#define TRDC_W1_EATR_SHIFT                       (8U)
/*! EATR - Error attributes
 *  0b000..Secure user mode, instruction fetch access.
 *  0b001..Secure user mode, data access.
 *  0b010..Secure privileged mode, instruction fetch access.
 *  0b011..Secure privileged mode, data access.
 *  0b100..Nonsecure user mode, instruction fetch access.
 *  0b101..Nonsecure user mode, data access.
 *  0b110..Nonsecure privileged mode, instruction fetch access.
 *  0b111..Nonsecure privileged mode, data access.
 */
#define TRDC_W1_EATR(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EATR_SHIFT)) & TRDC_W1_EATR_MASK)

#define TRDC_W1_ERW_MASK                         (0x800U)
#define TRDC_W1_ERW_SHIFT                        (11U)
/*! ERW - Error read/write
 *  0b0..Read access
 *  0b1..Write access
 */
#define TRDC_W1_ERW(x)                           (((uint32_t)(((uint32_t)(x)) << TRDC_W1_ERW_SHIFT)) & TRDC_W1_ERW_MASK)

#define TRDC_W1_EPORT_MASK                       (0x7000000U)
#define TRDC_W1_EPORT_SHIFT                      (24U)
/*! EPORT - Error port */
#define TRDC_W1_EPORT(x)                         (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EPORT_SHIFT)) & TRDC_W1_EPORT_MASK)

#define TRDC_W1_EST_MASK                         (0xC0000000U)
#define TRDC_W1_EST_SHIFT                        (30U)
/*! EST - Error state
 *  0b00..No access violation has been detected.
 *  0b01..No access violation has been detected.
 *  0b10..A single access violation has been detected.
 *  0b11..Multiple access violations for this domain have been detected by this submodule instance. Only the
 *        address and attribute information for the first error have been captured in DERR_W0_i and DERR_W1_i.
 */
#define TRDC_W1_EST(x)                           (((uint32_t)(((uint32_t)(x)) << TRDC_W1_EST_SHIFT)) & TRDC_W1_EST_MASK)
/*! @} */

/* The count of TRDC_W1 */
#define TRDC_MRC_DERR_W1_COUNT                   (7U)

/*! @name W3 - MRC Domain Error Word3 Register */
/*! @{ */

#define TRDC_W3_RECR_MASK                        (0xC0000000U)
#define TRDC_W3_RECR_SHIFT                       (30U)
/*! RECR - Rearm Error Capture Registers */
#define TRDC_W3_RECR(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_W3_RECR_SHIFT)) & TRDC_W3_RECR_MASK)
/*! @} */

/* The count of TRDC_W3 */
#define TRDC_MRC_DERR_W3_COUNT                   (7U)

/*! @name PID - Process Identifier */
/*! @{ */

#define TRDC_PID_PID_MASK                        (0x3FU)
#define TRDC_PID_PID_SHIFT                       (0U)
/*! PID - Process identifier */
#define TRDC_PID_PID(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_PID_PID_SHIFT)) & TRDC_PID_PID_MASK)

#define TRDC_PID_LK2_MASK                        (0x60000000U)
#define TRDC_PID_LK2_SHIFT                       (29U)
/*! LK2 - Lock
 *  0b00..Register can be written by any secure privileged write.
 *  0b01..Register can be written by any secure privileged write.
 *  0b10..Register can only be written by a secure privileged write from the bus master that locked the register.
 *  0b11..Register is locked (read-only) until the next reset.
 */
#define TRDC_PID_LK2(x)                          (((uint32_t)(((uint32_t)(x)) << TRDC_PID_LK2_SHIFT)) & TRDC_PID_LK2_MASK)
/*! @} */

/* The count of TRDC_PID */
#define TRDC_PID_COUNT                           (2U)

/*! @name MDA_W_DFMT0 - DAC Master Domain Assignment Register */
/*! @{ */

#define TRDC_MDA_W_DFMT0_DID_MASK                (0xFU)
#define TRDC_MDA_W_DFMT0_DID_SHIFT               (0U)
/*! DID - Domain identifier */
#define TRDC_MDA_W_DFMT0_DID(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_DID_SHIFT)) & TRDC_MDA_W_DFMT0_DID_MASK)

#define TRDC_MDA_W_DFMT0_DIDS_MASK               (0x30U)
#define TRDC_MDA_W_DFMT0_DIDS_SHIFT              (4U)
/*! DIDS - DID Select
 *  0b00..Use MDAm[3:0] as the domain identifier.
 *  0b01..Use the input DID as the domain identifier.
 *  0b10..Use MDAm[3:2] concatenated with the low-order 2 bits of the input DID (DID_in[1:0]) as the domain identifier.
 *  0b11..Reserved for future use.
 */
#define TRDC_MDA_W_DFMT0_DIDS(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_DIDS_SHIFT)) & TRDC_MDA_W_DFMT0_DIDS_MASK)

#define TRDC_MDA_W_DFMT0_PE_MASK                 (0xC0U)
#define TRDC_MDA_W_DFMT0_PE_SHIFT                (6U)
/*! PE - Process identifier enable
 *  0b00..No process identifier is included in the domain hit evaluation.
 *  0b01..No process identifier is included in the domain hit evaluation.
 *  0b10..PE = 2
 *  0b11..PE = 3
 */
#define TRDC_MDA_W_DFMT0_PE(x)                   (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_PE_SHIFT)) & TRDC_MDA_W_DFMT0_PE_MASK)

#define TRDC_MDA_W_DFMT0_PIDM_MASK               (0x3F00U)
#define TRDC_MDA_W_DFMT0_PIDM_SHIFT              (8U)
/*! PIDM - Process Identifier Mask */
#define TRDC_MDA_W_DFMT0_PIDM(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_PIDM_SHIFT)) & TRDC_MDA_W_DFMT0_PIDM_MASK)

#define TRDC_MDA_W_DFMT0_SA_MASK                 (0xC000U)
#define TRDC_MDA_W_DFMT0_SA_SHIFT                (14U)
/*! SA - Secure attribute
 *  0b00..Force the bus attribute for this master to secure.
 *  0b01..Force the bus attribute for this master to nonsecure.
 *  0b10..Use the bus master's secure/nonsecure attribute directly.
 *  0b11..Use the bus master's secure/nonsecure attribute directly.
 */
#define TRDC_MDA_W_DFMT0_SA(x)                   (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_SA_SHIFT)) & TRDC_MDA_W_DFMT0_SA_MASK)

#define TRDC_MDA_W_DFMT0_PID_MASK                (0x3F0000U)
#define TRDC_MDA_W_DFMT0_PID_SHIFT               (16U)
/*! PID - Process Identifier */
#define TRDC_MDA_W_DFMT0_PID(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_PID_SHIFT)) & TRDC_MDA_W_DFMT0_PID_MASK)

#define TRDC_MDA_W_DFMT0_DFMT_MASK               (0x20000000U)
#define TRDC_MDA_W_DFMT0_DFMT_SHIFT              (29U)
/*! DFMT - Domain format
 *  0b0..Processor-core domain assignment
 *  0b1..Non-processor domain assignment
 */
#define TRDC_MDA_W_DFMT0_DFMT(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_DFMT_SHIFT)) & TRDC_MDA_W_DFMT0_DFMT_MASK)

#define TRDC_MDA_W_DFMT0_LK1_MASK                (0x40000000U)
#define TRDC_MDA_W_DFMT0_LK1_SHIFT               (30U)
/*! LK1 - 1-bit Lock
 *  0b0..Register can be written by any secure privileged write.
 *  0b1..Register is locked (read-only) until the next reset.
 */
#define TRDC_MDA_W_DFMT0_LK1(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_LK1_SHIFT)) & TRDC_MDA_W_DFMT0_LK1_MASK)

#define TRDC_MDA_W_DFMT0_VLD_MASK                (0x80000000U)
#define TRDC_MDA_W_DFMT0_VLD_SHIFT               (31U)
/*! VLD - Valid
 *  0b0..The Wr domain assignment is invalid.
 *  0b1..The Wr domain assignment is valid.
 */
#define TRDC_MDA_W_DFMT0_VLD(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT0_VLD_SHIFT)) & TRDC_MDA_W_DFMT0_VLD_MASK)
/*! @} */

/* The count of TRDC_MDA_W_DFMT0 */
#define TRDC_MDA_W_DFMT0_COUNT                   (6U)

/* The count of TRDC_MDA_W_DFMT0 */
#define TRDC_MDA_W_DFMT0_COUNT2                  (7U)

/*! @name MDA_W_DFMT1 - DAC Master Domain Assignment Register */
/*! @{ */

#define TRDC_MDA_W_DFMT1_DID_MASK                (0xFU)
#define TRDC_MDA_W_DFMT1_DID_SHIFT               (0U)
/*! DID - Domain identifier */
#define TRDC_MDA_W_DFMT1_DID(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT1_DID_SHIFT)) & TRDC_MDA_W_DFMT1_DID_MASK)

#define TRDC_MDA_W_DFMT1_PA_MASK                 (0x30U)
#define TRDC_MDA_W_DFMT1_PA_SHIFT                (4U)
/*! PA - Privileged attribute
 *  0b00..Force the bus attribute for this master to user.
 *  0b01..Force the bus attribute for this master to privileged.
 *  0b10..Use the bus master's privileged/user attribute directly.
 *  0b11..Use the bus master's privileged/user attribute directly.
 */
#define TRDC_MDA_W_DFMT1_PA(x)                   (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT1_PA_SHIFT)) & TRDC_MDA_W_DFMT1_PA_MASK)

#define TRDC_MDA_W_DFMT1_SA_MASK                 (0xC0U)
#define TRDC_MDA_W_DFMT1_SA_SHIFT                (6U)
/*! SA - Secure attribute
 *  0b00..Force the bus attribute for this master to secure.
 *  0b01..Force the bus attribute for this master to nonsecure.
 *  0b10..Use the bus master's secure/nonsecure attribute directly.
 *  0b11..Use the bus master's secure/nonsecure attribute directly.
 */
#define TRDC_MDA_W_DFMT1_SA(x)                   (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT1_SA_SHIFT)) & TRDC_MDA_W_DFMT1_SA_MASK)

#define TRDC_MDA_W_DFMT1_DIDB_MASK               (0x100U)
#define TRDC_MDA_W_DFMT1_DIDB_SHIFT              (8U)
/*! DIDB - DID Bypass
 *  0b0..Use MDAn[3:0] as the domain identifier.
 *  0b1..Use the DID input as the domain identifier.
 */
#define TRDC_MDA_W_DFMT1_DIDB(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT1_DIDB_SHIFT)) & TRDC_MDA_W_DFMT1_DIDB_MASK)

#define TRDC_MDA_W_DFMT1_DFMT_MASK               (0x20000000U)
#define TRDC_MDA_W_DFMT1_DFMT_SHIFT              (29U)
/*! DFMT - Domain format
 *  0b0..Processor-core domain assignment
 *  0b1..Non-processor domain assignment
 */
#define TRDC_MDA_W_DFMT1_DFMT(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT1_DFMT_SHIFT)) & TRDC_MDA_W_DFMT1_DFMT_MASK)

#define TRDC_MDA_W_DFMT1_LK1_MASK                (0x40000000U)
#define TRDC_MDA_W_DFMT1_LK1_SHIFT               (30U)
/*! LK1 - 1-bit Lock
 *  0b0..Register can be written by any secure privileged write.
 *  0b1..Register is locked (read-only) until the next reset.
 */
#define TRDC_MDA_W_DFMT1_LK1(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT1_LK1_SHIFT)) & TRDC_MDA_W_DFMT1_LK1_MASK)

#define TRDC_MDA_W_DFMT1_VLD_MASK                (0x80000000U)
#define TRDC_MDA_W_DFMT1_VLD_SHIFT               (31U)
/*! VLD - Valid
 *  0b0..The Wr domain assignment is invalid.
 *  0b1..The Wr domain assignment is valid.
 */
#define TRDC_MDA_W_DFMT1_VLD(x)                  (((uint32_t)(((uint32_t)(x)) << TRDC_MDA_W_DFMT1_VLD_SHIFT)) & TRDC_MDA_W_DFMT1_VLD_MASK)
/*! @} */

/* The count of TRDC_MDA_W_DFMT1 */
#define TRDC_MDA_W_DFMT1_COUNT                   (5U)

/* The count of TRDC_MDA_W_DFMT1 */
#define TRDC_MDA_W_DFMT1_COUNT2                  (1U)

/*! @name MBC_MEM_GLBCFG - MBC Global Configuration Register */
/*! @{ */

#define TRDC_MBC_MEM_GLBCFG_NBLKS_MASK           (0x3FFU)
#define TRDC_MBC_MEM_GLBCFG_NBLKS_SHIFT          (0U)
/*! NBLKS - Number of blocks in this memory */
#define TRDC_MBC_MEM_GLBCFG_NBLKS(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEM_GLBCFG_NBLKS_SHIFT)) & TRDC_MBC_MEM_GLBCFG_NBLKS_MASK)

#define TRDC_MBC_MEM_GLBCFG_SIZE_LOG2_MASK       (0x1F0000U)
#define TRDC_MBC_MEM_GLBCFG_SIZE_LOG2_SHIFT      (16U)
/*! SIZE_LOG2 - Log2 size per block */
#define TRDC_MBC_MEM_GLBCFG_SIZE_LOG2(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEM_GLBCFG_SIZE_LOG2_SHIFT)) & TRDC_MBC_MEM_GLBCFG_SIZE_LOG2_MASK)
/*! @} */

/* The count of TRDC_MBC_MEM_GLBCFG */
#define TRDC_MBC_MEM_GLBCFG_COUNT                (2U)

/* The count of TRDC_MBC_MEM_GLBCFG */
#define TRDC_MBC_MEM_GLBCFG_COUNT2               (4U)

/*! @name MBC_NSE_BLK_INDEX - MBC NonSecure Enable Block Index */
/*! @{ */

#define TRDC_MBC_NSE_BLK_INDEX_AI_MASK           (0x1U)
#define TRDC_MBC_NSE_BLK_INDEX_AI_SHIFT          (0U)
/*! AI - Auto Increment
 *  0b0..No effect.
 *  0b1..Add 1 to the WNDX field after the register write.
 */
#define TRDC_MBC_NSE_BLK_INDEX_AI(x)             (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_AI_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_AI_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_WNDX_MASK         (0x3CU)
#define TRDC_MBC_NSE_BLK_INDEX_WNDX_SHIFT        (2U)
/*! WNDX - Word index into the block NSE bitmap. It selects the BLK_NSE_Wn register, where WNDX determines the value of n. */
#define TRDC_MBC_NSE_BLK_INDEX_WNDX(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_WNDX_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_WNDX_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_MEM_SEL_MASK      (0xF00U)
#define TRDC_MBC_NSE_BLK_INDEX_MEM_SEL_SHIFT     (8U)
/*! MEM_SEL - Memory Select */
#define TRDC_MBC_NSE_BLK_INDEX_MEM_SEL(x)        (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_MEM_SEL_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_MEM_SEL_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL0_MASK     (0x10000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL0_SHIFT    (16U)
/*! DID_SEL0 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL0(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL0_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL0_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL1_MASK     (0x20000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL1_SHIFT    (17U)
/*! DID_SEL1 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL1(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL1_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL1_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL2_MASK     (0x40000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL2_SHIFT    (18U)
/*! DID_SEL2 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL2(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL2_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL2_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL3_MASK     (0x80000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL3_SHIFT    (19U)
/*! DID_SEL3 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL3(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL3_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL3_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL4_MASK     (0x100000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL4_SHIFT    (20U)
/*! DID_SEL4 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL4(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL4_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL4_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL5_MASK     (0x200000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL5_SHIFT    (21U)
/*! DID_SEL5 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL5(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL5_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL5_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL6_MASK     (0x400000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL6_SHIFT    (22U)
/*! DID_SEL6 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL6(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL6_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL6_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL7_MASK     (0x800000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL7_SHIFT    (23U)
/*! DID_SEL7 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL7(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL7_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL7_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL8_MASK     (0x1000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL8_SHIFT    (24U)
/*! DID_SEL8 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL8(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL8_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL8_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL9_MASK     (0x2000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL9_SHIFT    (25U)
/*! DID_SEL9 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL9(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL9_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL9_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL10_MASK    (0x4000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL10_SHIFT   (26U)
/*! DID_SEL10 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL10(x)      (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL10_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL10_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL11_MASK    (0x8000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL11_SHIFT   (27U)
/*! DID_SEL11 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL11(x)      (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL11_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL11_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL12_MASK    (0x10000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL12_SHIFT   (28U)
/*! DID_SEL12 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL12(x)      (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL12_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL12_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL13_MASK    (0x20000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL13_SHIFT   (29U)
/*! DID_SEL13 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL13(x)      (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL13_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL13_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL14_MASK    (0x40000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL14_SHIFT   (30U)
/*! DID_SEL14 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL14(x)      (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL14_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL14_MASK)

#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL15_MASK    (0x80000000U)
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL15_SHIFT   (31U)
/*! DID_SEL15 - DID Select
 *  0b0..No effect.
 *  0b1..Selects NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_INDEX_DID_SEL15(x)      (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_INDEX_DID_SEL15_SHIFT)) & TRDC_MBC_NSE_BLK_INDEX_DID_SEL15_MASK)
/*! @} */

/* The count of TRDC_MBC_NSE_BLK_INDEX */
#define TRDC_MBC_NSE_BLK_INDEX_COUNT             (2U)

/*! @name MBC_NSE_BLK_SET - MBC NonSecure Enable Block Set */
/*! @{ */

#define TRDC_MBC_NSE_BLK_SET_W1SET_MASK          (0xFFFFFFFFU)
#define TRDC_MBC_NSE_BLK_SET_W1SET_SHIFT         (0U)
/*! W1SET - Write-1 Set */
#define TRDC_MBC_NSE_BLK_SET_W1SET(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_SET_W1SET_SHIFT)) & TRDC_MBC_NSE_BLK_SET_W1SET_MASK)
/*! @} */

/* The count of TRDC_MBC_NSE_BLK_SET */
#define TRDC_MBC_NSE_BLK_SET_COUNT               (2U)

/*! @name MBC_NSE_BLK_CLR - MBC NonSecure Enable Block Clear */
/*! @{ */

#define TRDC_MBC_NSE_BLK_CLR_W1CLR_MASK          (0xFFFFFFFFU)
#define TRDC_MBC_NSE_BLK_CLR_W1CLR_SHIFT         (0U)
/*! W1CLR - Write-1 Clear */
#define TRDC_MBC_NSE_BLK_CLR_W1CLR(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_W1CLR_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_W1CLR_MASK)
/*! @} */

/* The count of TRDC_MBC_NSE_BLK_CLR */
#define TRDC_MBC_NSE_BLK_CLR_COUNT               (2U)

/*! @name MBC_NSE_BLK_CLR_ALL - MBC NonSecure Enable Block Clear All */
/*! @{ */

#define TRDC_MBC_NSE_BLK_CLR_ALL_MEMSEL_MASK     (0xF00U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_MEMSEL_SHIFT    (8U)
/*! MEMSEL - Memory Select */
#define TRDC_MBC_NSE_BLK_CLR_ALL_MEMSEL(x)       (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_MEMSEL_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_MEMSEL_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL0_MASK   (0x10000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL0_SHIFT  (16U)
/*! DID_SEL0 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL0_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL0_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL1_MASK   (0x20000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL1_SHIFT  (17U)
/*! DID_SEL1 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL1_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL1_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL2_MASK   (0x40000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL2_SHIFT  (18U)
/*! DID_SEL2 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL2_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL2_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL3_MASK   (0x80000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL3_SHIFT  (19U)
/*! DID_SEL3 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL3_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL3_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL4_MASK   (0x100000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL4_SHIFT  (20U)
/*! DID_SEL4 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL4_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL4_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL5_MASK   (0x200000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL5_SHIFT  (21U)
/*! DID_SEL5 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL5_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL5_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL6_MASK   (0x400000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL6_SHIFT  (22U)
/*! DID_SEL6 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL6_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL6_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL7_MASK   (0x800000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL7_SHIFT  (23U)
/*! DID_SEL7 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL7_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL7_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL8_MASK   (0x1000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL8_SHIFT  (24U)
/*! DID_SEL8 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL8_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL8_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL9_MASK   (0x2000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL9_SHIFT  (25U)
/*! DID_SEL9 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL9_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL9_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL10_MASK  (0x4000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL10_SHIFT (26U)
/*! DID_SEL10 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL10_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL10_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL11_MASK  (0x8000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL11_SHIFT (27U)
/*! DID_SEL11 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL11_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL11_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL12_MASK  (0x10000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL12_SHIFT (28U)
/*! DID_SEL12 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL12_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL12_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL13_MASK  (0x20000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL13_SHIFT (29U)
/*! DID_SEL13 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL13_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL13_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL14_MASK  (0x40000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL14_SHIFT (30U)
/*! DID_SEL14 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL14_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL14_MASK)

#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL15_MASK  (0x80000000U)
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL15_SHIFT (31U)
/*! DID_SEL15 - DID Select
 *  0b0..No effect.
 *  0b1..Clear all NSE bits for this domain.
 */
#define TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL15_SHIFT)) & TRDC_MBC_NSE_BLK_CLR_ALL_DID_SEL15_MASK)
/*! @} */

/* The count of TRDC_MBC_NSE_BLK_CLR_ALL */
#define TRDC_MBC_NSE_BLK_CLR_ALL_COUNT           (2U)

/*! @name MBC_MEMN_GLBAC - MBC Global Access Control */
/*! @{ */

#define TRDC_MBC_MEMN_GLBAC_NUX_MASK             (0x1U)
#define TRDC_MBC_MEMN_GLBAC_NUX_SHIFT            (0U)
/*! NUX - NonsecureUser Execute
 *  0b0..Execute access is not allowed in Nonsecure User mode.
 *  0b1..Execute access is allowed in Nonsecure User mode.
 */
#define TRDC_MBC_MEMN_GLBAC_NUX(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_NUX_SHIFT)) & TRDC_MBC_MEMN_GLBAC_NUX_MASK)

#define TRDC_MBC_MEMN_GLBAC_NUW_MASK             (0x2U)
#define TRDC_MBC_MEMN_GLBAC_NUW_SHIFT            (1U)
/*! NUW - NonsecureUser Write
 *  0b0..Write access is not allowed in Nonsecure User mode.
 *  0b1..Write access is allowed in Nonsecure User mode.
 */
#define TRDC_MBC_MEMN_GLBAC_NUW(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_NUW_SHIFT)) & TRDC_MBC_MEMN_GLBAC_NUW_MASK)

#define TRDC_MBC_MEMN_GLBAC_NUR_MASK             (0x4U)
#define TRDC_MBC_MEMN_GLBAC_NUR_SHIFT            (2U)
/*! NUR - NonsecureUser Read
 *  0b0..Read access is not allowed in Nonsecure User mode.
 *  0b1..Read access is allowed in Nonsecure User mode.
 */
#define TRDC_MBC_MEMN_GLBAC_NUR(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_NUR_SHIFT)) & TRDC_MBC_MEMN_GLBAC_NUR_MASK)

#define TRDC_MBC_MEMN_GLBAC_NPX_MASK             (0x10U)
#define TRDC_MBC_MEMN_GLBAC_NPX_SHIFT            (4U)
/*! NPX - NonsecurePriv Execute
 *  0b0..Execute access is not allowed in Nonsecure Privilege mode.
 *  0b1..Execute access is allowed in Nonsecure Privilege mode.
 */
#define TRDC_MBC_MEMN_GLBAC_NPX(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_NPX_SHIFT)) & TRDC_MBC_MEMN_GLBAC_NPX_MASK)

#define TRDC_MBC_MEMN_GLBAC_NPW_MASK             (0x20U)
#define TRDC_MBC_MEMN_GLBAC_NPW_SHIFT            (5U)
/*! NPW - NonsecurePriv Write
 *  0b0..Write access is not allowed in Nonsecure Privilege mode.
 *  0b1..Write access is allowed in Nonsecure Privilege mode.
 */
#define TRDC_MBC_MEMN_GLBAC_NPW(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_NPW_SHIFT)) & TRDC_MBC_MEMN_GLBAC_NPW_MASK)

#define TRDC_MBC_MEMN_GLBAC_NPR_MASK             (0x40U)
#define TRDC_MBC_MEMN_GLBAC_NPR_SHIFT            (6U)
/*! NPR - NonsecurePriv Read
 *  0b0..Read access is not allowed in Nonsecure Privilege mode.
 *  0b1..Read access is allowed in Nonsecure Privilege mode.
 */
#define TRDC_MBC_MEMN_GLBAC_NPR(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_NPR_SHIFT)) & TRDC_MBC_MEMN_GLBAC_NPR_MASK)

#define TRDC_MBC_MEMN_GLBAC_SUX_MASK             (0x100U)
#define TRDC_MBC_MEMN_GLBAC_SUX_SHIFT            (8U)
/*! SUX - SecureUser Execute
 *  0b0..Execute access is not allowed in Secure User mode.
 *  0b1..Execute access is allowed in Secure User mode.
 */
#define TRDC_MBC_MEMN_GLBAC_SUX(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_SUX_SHIFT)) & TRDC_MBC_MEMN_GLBAC_SUX_MASK)

#define TRDC_MBC_MEMN_GLBAC_SUW_MASK             (0x200U)
#define TRDC_MBC_MEMN_GLBAC_SUW_SHIFT            (9U)
/*! SUW - SecureUser Write
 *  0b0..Write access is not allowed in Secure User mode.
 *  0b1..Write access is allowed in Secure User mode.
 */
#define TRDC_MBC_MEMN_GLBAC_SUW(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_SUW_SHIFT)) & TRDC_MBC_MEMN_GLBAC_SUW_MASK)

#define TRDC_MBC_MEMN_GLBAC_SUR_MASK             (0x400U)
#define TRDC_MBC_MEMN_GLBAC_SUR_SHIFT            (10U)
/*! SUR - SecureUser Read
 *  0b0..Read access is not allowed in Secure User mode.
 *  0b1..Read access is allowed in Secure User mode.
 */
#define TRDC_MBC_MEMN_GLBAC_SUR(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_SUR_SHIFT)) & TRDC_MBC_MEMN_GLBAC_SUR_MASK)

#define TRDC_MBC_MEMN_GLBAC_SPX_MASK             (0x1000U)
#define TRDC_MBC_MEMN_GLBAC_SPX_SHIFT            (12U)
/*! SPX - SecurePriv Execute
 *  0b0..Execute access is not allowed in Secure Privilege mode.
 *  0b1..Execute access is allowed in Secure Privilege mode.
 */
#define TRDC_MBC_MEMN_GLBAC_SPX(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_SPX_SHIFT)) & TRDC_MBC_MEMN_GLBAC_SPX_MASK)

#define TRDC_MBC_MEMN_GLBAC_SPW_MASK             (0x2000U)
#define TRDC_MBC_MEMN_GLBAC_SPW_SHIFT            (13U)
/*! SPW - SecurePriv Write
 *  0b0..Write access is not allowed in Secure Privilege mode.
 *  0b1..Write access is allowed in Secure Privilege mode.
 */
#define TRDC_MBC_MEMN_GLBAC_SPW(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_SPW_SHIFT)) & TRDC_MBC_MEMN_GLBAC_SPW_MASK)

#define TRDC_MBC_MEMN_GLBAC_SPR_MASK             (0x4000U)
#define TRDC_MBC_MEMN_GLBAC_SPR_SHIFT            (14U)
/*! SPR - SecurePriv Read
 *  0b0..Read access is not allowed in Secure Privilege mode.
 *  0b1..Read access is allowed in Secure Privilege mode.
 */
#define TRDC_MBC_MEMN_GLBAC_SPR(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_SPR_SHIFT)) & TRDC_MBC_MEMN_GLBAC_SPR_MASK)

#define TRDC_MBC_MEMN_GLBAC_LK_MASK              (0x80000000U)
#define TRDC_MBC_MEMN_GLBAC_LK_SHIFT             (31U)
/*! LK - LOCK
 *  0b0..This register is not locked and can be altered.
 *  0b1..This register is locked and cannot be altered.
 */
#define TRDC_MBC_MEMN_GLBAC_LK(x)                (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_MEMN_GLBAC_LK_SHIFT)) & TRDC_MBC_MEMN_GLBAC_LK_MASK)
/*! @} */

/* The count of TRDC_MBC_MEMN_GLBAC */
#define TRDC_MBC_MEMN_GLBAC_COUNT                (2U)

/* The count of TRDC_MBC_MEMN_GLBAC */
#define TRDC_MBC_MEMN_GLBAC_COUNT2               (8U)

/*! @name MBC_DOM0_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM0_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM0_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM0_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM0_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM0_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM0_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM0_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM0_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM0_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM0_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM0_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM0_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM0_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM0_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM0_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM1_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM1_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM1_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM1_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM1_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM1_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM1_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM1_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM1_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM1_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM1_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM1_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM1_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM1_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM1_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM1_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM2_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM2_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM2_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM2_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM2_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM2_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM2_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM2_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM2_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM2_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM2_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM2_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM2_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM2_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM2_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM2_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM3_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM3_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM3_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM3_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM3_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM3_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM3_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM3_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM3_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM3_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM3_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM3_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM3_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM3_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM3_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM3_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM4_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM4_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM4_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM4_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM4_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM4_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM4_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM4_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM4_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM4_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM4_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM4_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM4_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM4_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM4_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM4_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM5_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM5_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM5_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM5_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM5_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM5_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM5_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM5_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM5_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM5_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM5_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM5_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM5_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM5_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM5_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM5_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM6_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM6_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM6_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM6_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM6_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM6_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM6_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM6_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM6_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM6_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM6_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM6_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM6_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM6_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM6_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM6_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM7_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM7_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM7_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM7_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM7_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM7_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM7_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM7_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM7_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM7_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM7_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM7_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM7_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM7_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM7_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM7_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM8_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM8_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM8_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM8_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM8_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM8_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM8_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM8_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM8_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM8_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM8_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM8_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM8_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM8_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM8_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM8_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM9_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM0_BLK_CFG_W_COUNT2      (16U)

/*! @name MBC_DOM9_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM9_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM0_BLK_NSE_W_COUNT2      (4U)

/*! @name MBC_DOM9_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM1_BLK_CFG_W_COUNT2      (4U)

/*! @name MBC_DOM9_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM9_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM1_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM9_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM2_BLK_CFG_W_COUNT2      (1U)

/*! @name MBC_DOM9_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM9_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM2_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM9_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE0_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE0_SHIFT  (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE1_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE1_SHIFT  (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE2_MASK   (0x800U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE2_SHIFT  (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE3_MASK   (0x8000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE3_SHIFT  (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE4_MASK   (0x80000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE4_SHIFT  (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE5_MASK   (0x800000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE5_SHIFT  (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE6_MASK   (0x8000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE6_SHIFT  (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE7_MASK   (0x80000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE7_SHIFT  (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM9_MEM3_BLK_CFG_W_COUNT2      (3U)

/*! @name MBC_DOM9_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT0_MASK   (0x1U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT0_SHIFT  (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT0(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT1_MASK   (0x2U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT1_SHIFT  (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT1(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT2_MASK   (0x4U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT2_SHIFT  (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT2(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT3_MASK   (0x8U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT3_SHIFT  (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT3(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT4_MASK   (0x10U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT4_SHIFT  (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT4(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT5_MASK   (0x20U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT5_SHIFT  (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT5(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT6_MASK   (0x40U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT6_SHIFT  (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT6(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT7_MASK   (0x80U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT7_SHIFT  (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT7(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT8_MASK   (0x100U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT8_SHIFT  (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT8(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT9_MASK   (0x200U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT9_SHIFT  (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT9(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT10_MASK  (0x400U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT10(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT11_MASK  (0x800U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT11(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT12_MASK  (0x1000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT12(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT13_MASK  (0x2000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT13(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT14_MASK  (0x4000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT14(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT15_MASK  (0x8000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT15(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT16_MASK  (0x10000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT16(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT17_MASK  (0x20000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT17(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT18_MASK  (0x40000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT18(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT19_MASK  (0x80000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT19(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT20_MASK  (0x100000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT20(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT21_MASK  (0x200000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT21(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT22_MASK  (0x400000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT22(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT23_MASK  (0x800000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT23(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT24_MASK  (0x1000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT24(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT25_MASK  (0x2000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT25(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT26_MASK  (0x4000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT26(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT27_MASK  (0x8000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT27(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT28_MASK  (0x10000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT28(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT29_MASK  (0x20000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT29(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT30_MASK  (0x40000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT30(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT31_MASK  (0x80000000U)
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT31(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM9_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM9_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_COUNT       (2U)

/* The count of TRDC_MBC_DOM9_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM9_MEM3_BLK_NSE_W_COUNT2      (1U)

/*! @name MBC_DOM10_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM0_BLK_CFG_W_COUNT2     (16U)

/*! @name MBC_DOM10_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM10_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM0_BLK_NSE_W_COUNT2     (4U)

/*! @name MBC_DOM10_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM1_BLK_CFG_W_COUNT2     (4U)

/*! @name MBC_DOM10_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM10_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM1_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM10_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM2_BLK_CFG_W_COUNT2     (1U)

/*! @name MBC_DOM10_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM10_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM2_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM10_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM10_MEM3_BLK_CFG_W_COUNT2     (3U)

/*! @name MBC_DOM10_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM10_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM10_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM10_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM10_MEM3_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM11_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM0_BLK_CFG_W_COUNT2     (16U)

/*! @name MBC_DOM11_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM11_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM0_BLK_NSE_W_COUNT2     (4U)

/*! @name MBC_DOM11_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM1_BLK_CFG_W_COUNT2     (4U)

/*! @name MBC_DOM11_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM11_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM1_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM11_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM2_BLK_CFG_W_COUNT2     (1U)

/*! @name MBC_DOM11_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM11_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM2_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM11_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM11_MEM3_BLK_CFG_W_COUNT2     (3U)

/*! @name MBC_DOM11_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM11_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM11_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM11_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM11_MEM3_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM12_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM0_BLK_CFG_W_COUNT2     (16U)

/*! @name MBC_DOM12_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM12_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM0_BLK_NSE_W_COUNT2     (4U)

/*! @name MBC_DOM12_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM1_BLK_CFG_W_COUNT2     (4U)

/*! @name MBC_DOM12_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM12_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM1_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM12_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM2_BLK_CFG_W_COUNT2     (1U)

/*! @name MBC_DOM12_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM12_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM2_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM12_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM12_MEM3_BLK_CFG_W_COUNT2     (3U)

/*! @name MBC_DOM12_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM12_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM12_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM12_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM12_MEM3_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM13_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM0_BLK_CFG_W_COUNT2     (16U)

/*! @name MBC_DOM13_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM13_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM0_BLK_NSE_W_COUNT2     (4U)

/*! @name MBC_DOM13_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM1_BLK_CFG_W_COUNT2     (4U)

/*! @name MBC_DOM13_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM13_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM1_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM13_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM2_BLK_CFG_W_COUNT2     (1U)

/*! @name MBC_DOM13_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM13_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM2_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM13_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM13_MEM3_BLK_CFG_W_COUNT2     (3U)

/*! @name MBC_DOM13_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM13_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM13_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM13_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM13_MEM3_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM14_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM0_BLK_CFG_W_COUNT2     (16U)

/*! @name MBC_DOM14_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM14_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM0_BLK_NSE_W_COUNT2     (4U)

/*! @name MBC_DOM14_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM1_BLK_CFG_W_COUNT2     (4U)

/*! @name MBC_DOM14_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM14_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM1_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM14_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM2_BLK_CFG_W_COUNT2     (1U)

/*! @name MBC_DOM14_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM14_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM2_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM14_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM14_MEM3_BLK_CFG_W_COUNT2     (3U)

/*! @name MBC_DOM14_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM14_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM14_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM14_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM14_MEM3_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM15_MEM0_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM0_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM0_BLK_CFG_W_COUNT2     (16U)

/*! @name MBC_DOM15_MEM0_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM15_MEM0_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM0_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM0_BLK_NSE_W_COUNT2     (4U)

/*! @name MBC_DOM15_MEM1_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM1_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM1_BLK_CFG_W_COUNT2     (4U)

/*! @name MBC_DOM15_MEM1_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM15_MEM1_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM1_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM1_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM15_MEM2_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM2_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM2_BLK_CFG_W_COUNT2     (1U)

/*! @name MBC_DOM15_MEM2_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM15_MEM2_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM2_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM2_BLK_NSE_W_COUNT2     (1U)

/*! @name MBC_DOM15_MEM3_BLK_CFG_W - MBC Memory Block Configuration Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL0_MASK (0x7U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL0_SHIFT (0U)
/*! MBACSEL0 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL0(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL0_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL0_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE0_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE0_SHIFT (3U)
/*! NSE0 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE0_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE0_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL1_MASK (0x70U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL1_SHIFT (4U)
/*! MBACSEL1 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL1(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL1_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL1_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE1_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE1_SHIFT (7U)
/*! NSE1 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE1_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE1_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL2_MASK (0x700U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL2_SHIFT (8U)
/*! MBACSEL2 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL2(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL2_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL2_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE2_MASK  (0x800U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE2_SHIFT (11U)
/*! NSE2 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE2_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE2_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL3_MASK (0x7000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL3_SHIFT (12U)
/*! MBACSEL3 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL3(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL3_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL3_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE3_MASK  (0x8000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE3_SHIFT (15U)
/*! NSE3 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE3_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE3_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL4_MASK (0x70000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL4_SHIFT (16U)
/*! MBACSEL4 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL4(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL4_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL4_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE4_MASK  (0x80000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE4_SHIFT (19U)
/*! NSE4 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE4_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE4_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL5_MASK (0x700000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL5_SHIFT (20U)
/*! MBACSEL5 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL5(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL5_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL5_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE5_MASK  (0x800000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE5_SHIFT (23U)
/*! NSE5 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE5_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE5_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL6_MASK (0x7000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL6_SHIFT (24U)
/*! MBACSEL6 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL6(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL6_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL6_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE6_MASK  (0x8000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE6_SHIFT (27U)
/*! NSE6 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE6_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE6_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL7_MASK (0x70000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL7_SHIFT (28U)
/*! MBACSEL7 - Memory Block Access Control Select for block B
 *  0b000..select MBC_MEMN_GLBAC0 access control policy for block B
 *  0b001..select MBC_MEMN_GLBAC1 access control policy for block B
 *  0b010..select MBC_MEMN_GLBAC2 access control policy for block B
 *  0b011..select MBC_MEMN_GLBAC3 access control policy for block B
 *  0b100..select MBC_MEMN_GLBAC4 access control policy for block B
 *  0b101..select MBC_MEMN_GLBAC5 access control policy for block B
 *  0b110..select MBC_MEMN_GLBAC6 access control policy for block B
 *  0b111..select MBC_MEMN_GLBAC7 access control policy for block B
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL7(x) (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL7_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_MBACSEL7_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE7_MASK  (0x80000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE7_SHIFT (31U)
/*! NSE7 - NonSecure Enable for block B
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in this register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in this register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE7_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_CFG_W_NSE7_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM3_BLK_CFG_W */
#define TRDC_MBC_DOM15_MEM3_BLK_CFG_W_COUNT2     (3U)

/*! @name MBC_DOM15_MEM3_BLK_NSE_W - MBC Memory Block NonSecure Enable Word */
/*! @{ */

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT0_MASK  (0x1U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT0_SHIFT (0U)
/*! BIT0 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT0(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT0_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT0_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT1_MASK  (0x2U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT1_SHIFT (1U)
/*! BIT1 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT1(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT1_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT1_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT2_MASK  (0x4U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT2_SHIFT (2U)
/*! BIT2 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT2(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT2_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT2_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT3_MASK  (0x8U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT3_SHIFT (3U)
/*! BIT3 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT3(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT3_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT3_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT4_MASK  (0x10U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT4_SHIFT (4U)
/*! BIT4 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT4(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT4_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT4_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT5_MASK  (0x20U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT5_SHIFT (5U)
/*! BIT5 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT5(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT5_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT5_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT6_MASK  (0x40U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT6_SHIFT (6U)
/*! BIT6 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT6(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT6_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT6_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT7_MASK  (0x80U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT7_SHIFT (7U)
/*! BIT7 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT7(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT7_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT7_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT8_MASK  (0x100U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT8_SHIFT (8U)
/*! BIT8 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT8(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT8_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT8_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT9_MASK  (0x200U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT9_SHIFT (9U)
/*! BIT9 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT9(x)    (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT9_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT9_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT10_MASK (0x400U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT10_SHIFT (10U)
/*! BIT10 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT10(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT10_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT10_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT11_MASK (0x800U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT11_SHIFT (11U)
/*! BIT11 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT11(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT11_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT11_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT12_MASK (0x1000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT12_SHIFT (12U)
/*! BIT12 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT12(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT12_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT12_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT13_MASK (0x2000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT13_SHIFT (13U)
/*! BIT13 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT13(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT13_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT13_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT14_MASK (0x4000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT14_SHIFT (14U)
/*! BIT14 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT14(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT14_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT14_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT15_MASK (0x8000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT15_SHIFT (15U)
/*! BIT15 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT15(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT15_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT15_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT16_MASK (0x10000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT16_SHIFT (16U)
/*! BIT16 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT16(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT16_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT16_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT17_MASK (0x20000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT17_SHIFT (17U)
/*! BIT17 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT17(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT17_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT17_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT18_MASK (0x40000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT18_SHIFT (18U)
/*! BIT18 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT18(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT18_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT18_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT19_MASK (0x80000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT19_SHIFT (19U)
/*! BIT19 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT19(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT19_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT19_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT20_MASK (0x100000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT20_SHIFT (20U)
/*! BIT20 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT20(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT20_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT20_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT21_MASK (0x200000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT21_SHIFT (21U)
/*! BIT21 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT21(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT21_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT21_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT22_MASK (0x400000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT22_SHIFT (22U)
/*! BIT22 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT22(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT22_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT22_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT23_MASK (0x800000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT23_SHIFT (23U)
/*! BIT23 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT23(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT23_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT23_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT24_MASK (0x1000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT24_SHIFT (24U)
/*! BIT24 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT24(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT24_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT24_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT25_MASK (0x2000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT25_SHIFT (25U)
/*! BIT25 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT25(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT25_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT25_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT26_MASK (0x4000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT26_SHIFT (26U)
/*! BIT26 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT26(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT26_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT26_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT27_MASK (0x8000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT27_SHIFT (27U)
/*! BIT27 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT27(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT27_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT27_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT28_MASK (0x10000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT28_SHIFT (28U)
/*! BIT28 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT28(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT28_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT28_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT29_MASK (0x20000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT29_SHIFT (29U)
/*! BIT29 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT29(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT29_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT29_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT30_MASK (0x40000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT30_SHIFT (30U)
/*! BIT30 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT30(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT30_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT30_MASK)

#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT31_MASK (0x80000000U)
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT31_SHIFT (31U)
/*! BIT31 - Bit b NonSecure Enable [b = 0 - 31]
 *  0b0..Secure accesses to block B are based on corresponding MBACSEL field in register
 *       (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]), nonsecure accesses to block B are not allowed.
 *  0b1..Secure accesses to block B are not allowed, nonsecure accesses to block B are based on corresponding
 *       MBACSEL field in register (MBCm_DOMd_MEMs_BLK_CFG_Ww[MBACSEL]).
 */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT31(x)   (((uint32_t)(((uint32_t)(x)) << TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT31_SHIFT)) & TRDC_MBC_DOM15_MEM3_BLK_NSE_W_BIT31_MASK)
/*! @} */

/* The count of TRDC_MBC_DOM15_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_COUNT      (2U)

/* The count of TRDC_MBC_DOM15_MEM3_BLK_NSE_W */
#define TRDC_MBC_DOM15_MEM3_BLK_NSE_W_COUNT2     (1U)

/*! @name MRC_GLBCFG - MRC Global Configuration Register */
/*! @{ */

#define TRDC_MRC_GLBCFG_NRGNS_MASK               (0x1FU)
#define TRDC_MRC_GLBCFG_NRGNS_SHIFT              (0U)
/*! NRGNS - Number of regions [1-16] */
#define TRDC_MRC_GLBCFG_NRGNS(x)                 (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBCFG_NRGNS_SHIFT)) & TRDC_MRC_GLBCFG_NRGNS_MASK)
/*! @} */

/* The count of TRDC_MRC_GLBCFG */
#define TRDC_MRC_GLBCFG_COUNT                    (7U)

/*! @name MRC_NSE_RGN_INDIRECT - MRC NonSecure Enable Region Indirect */
/*! @{ */

#define TRDC_MRC_NSE_RGN_INDIRECT_DID_SEL_MASK   (0xFFFF0000U)
#define TRDC_MRC_NSE_RGN_INDIRECT_DID_SEL_SHIFT  (16U)
/*! DID_SEL - DID Select */
#define TRDC_MRC_NSE_RGN_INDIRECT_DID_SEL(x)     (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_NSE_RGN_INDIRECT_DID_SEL_SHIFT)) & TRDC_MRC_NSE_RGN_INDIRECT_DID_SEL_MASK)
/*! @} */

/* The count of TRDC_MRC_NSE_RGN_INDIRECT */
#define TRDC_MRC_NSE_RGN_INDIRECT_COUNT          (7U)

/*! @name MRC_NSE_RGN_SET - MRC NonSecure Enable Region Set */
/*! @{ */

#define TRDC_MRC_NSE_RGN_SET_W1SET_MASK          (0xFFFFU)
#define TRDC_MRC_NSE_RGN_SET_W1SET_SHIFT         (0U)
/*! W1SET - Write-1 Set */
#define TRDC_MRC_NSE_RGN_SET_W1SET(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_NSE_RGN_SET_W1SET_SHIFT)) & TRDC_MRC_NSE_RGN_SET_W1SET_MASK)
/*! @} */

/* The count of TRDC_MRC_NSE_RGN_SET */
#define TRDC_MRC_NSE_RGN_SET_COUNT               (7U)

/*! @name MRC_NSE_RGN_CLR - MRC NonSecure Enable Region Clear */
/*! @{ */

#define TRDC_MRC_NSE_RGN_CLR_W1CLR_MASK          (0xFFFFU)
#define TRDC_MRC_NSE_RGN_CLR_W1CLR_SHIFT         (0U)
/*! W1CLR - Write-1 Clear */
#define TRDC_MRC_NSE_RGN_CLR_W1CLR(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_NSE_RGN_CLR_W1CLR_SHIFT)) & TRDC_MRC_NSE_RGN_CLR_W1CLR_MASK)
/*! @} */

/* The count of TRDC_MRC_NSE_RGN_CLR */
#define TRDC_MRC_NSE_RGN_CLR_COUNT               (7U)

/*! @name MRC_NSE_RGN_CLR_ALL - MRC NonSecure Enable Region Clear All */
/*! @{ */

#define TRDC_MRC_NSE_RGN_CLR_ALL_DID_SEL_MASK    (0xFFFF0000U)
#define TRDC_MRC_NSE_RGN_CLR_ALL_DID_SEL_SHIFT   (16U)
/*! DID_SEL - DID Select */
#define TRDC_MRC_NSE_RGN_CLR_ALL_DID_SEL(x)      (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_NSE_RGN_CLR_ALL_DID_SEL_SHIFT)) & TRDC_MRC_NSE_RGN_CLR_ALL_DID_SEL_MASK)
/*! @} */

/* The count of TRDC_MRC_NSE_RGN_CLR_ALL */
#define TRDC_MRC_NSE_RGN_CLR_ALL_COUNT           (7U)

/*! @name MRC_GLBAC - MRC Global Access Control */
/*! @{ */

#define TRDC_MRC_GLBAC_NUX_MASK                  (0x1U)
#define TRDC_MRC_GLBAC_NUX_SHIFT                 (0U)
/*! NUX - NonsecureUser Execute
 *  0b0..Execute access is not allowed in Nonsecure User mode.
 *  0b1..Execute access is allowed in Nonsecure User mode.
 */
#define TRDC_MRC_GLBAC_NUX(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_NUX_SHIFT)) & TRDC_MRC_GLBAC_NUX_MASK)

#define TRDC_MRC_GLBAC_NUW_MASK                  (0x2U)
#define TRDC_MRC_GLBAC_NUW_SHIFT                 (1U)
/*! NUW - NonsecureUser Write
 *  0b0..Write access is not allowed in Nonsecure User mode.
 *  0b1..Write access is allowed in Nonsecure User mode.
 */
#define TRDC_MRC_GLBAC_NUW(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_NUW_SHIFT)) & TRDC_MRC_GLBAC_NUW_MASK)

#define TRDC_MRC_GLBAC_NUR_MASK                  (0x4U)
#define TRDC_MRC_GLBAC_NUR_SHIFT                 (2U)
/*! NUR - NonsecureUser Read
 *  0b0..Read access is not allowed in Nonsecure User mode.
 *  0b1..Read access is allowed in Nonsecure User mode.
 */
#define TRDC_MRC_GLBAC_NUR(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_NUR_SHIFT)) & TRDC_MRC_GLBAC_NUR_MASK)

#define TRDC_MRC_GLBAC_NPX_MASK                  (0x10U)
#define TRDC_MRC_GLBAC_NPX_SHIFT                 (4U)
/*! NPX - NonsecurePriv Execute
 *  0b0..Execute access is not allowed in Nonsecure Privilege mode.
 *  0b1..Execute access is allowed in Nonsecure Privilege mode.
 */
#define TRDC_MRC_GLBAC_NPX(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_NPX_SHIFT)) & TRDC_MRC_GLBAC_NPX_MASK)

#define TRDC_MRC_GLBAC_NPW_MASK                  (0x20U)
#define TRDC_MRC_GLBAC_NPW_SHIFT                 (5U)
/*! NPW - NonsecurePriv Write
 *  0b0..Write access is not allowed in Nonsecure Privilege mode.
 *  0b1..Write access is allowed in Nonsecure Privilege mode.
 */
#define TRDC_MRC_GLBAC_NPW(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_NPW_SHIFT)) & TRDC_MRC_GLBAC_NPW_MASK)

#define TRDC_MRC_GLBAC_NPR_MASK                  (0x40U)
#define TRDC_MRC_GLBAC_NPR_SHIFT                 (6U)
/*! NPR - NonsecurePriv Read
 *  0b0..Read access is not allowed in Nonsecure Privilege mode.
 *  0b1..Read access is allowed in Nonsecure Privilege mode.
 */
#define TRDC_MRC_GLBAC_NPR(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_NPR_SHIFT)) & TRDC_MRC_GLBAC_NPR_MASK)

#define TRDC_MRC_GLBAC_SUX_MASK                  (0x100U)
#define TRDC_MRC_GLBAC_SUX_SHIFT                 (8U)
/*! SUX - SecureUser Execute
 *  0b0..Execute access is not allowed in Secure User mode.
 *  0b1..Execute access is allowed in Secure User mode.
 */
#define TRDC_MRC_GLBAC_SUX(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_SUX_SHIFT)) & TRDC_MRC_GLBAC_SUX_MASK)

#define TRDC_MRC_GLBAC_SUW_MASK                  (0x200U)
#define TRDC_MRC_GLBAC_SUW_SHIFT                 (9U)
/*! SUW - SecureUser Write
 *  0b0..Write access is not allowed in Secure User mode.
 *  0b1..Write access is allowed in Secure User mode.
 */
#define TRDC_MRC_GLBAC_SUW(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_SUW_SHIFT)) & TRDC_MRC_GLBAC_SUW_MASK)

#define TRDC_MRC_GLBAC_SUR_MASK                  (0x400U)
#define TRDC_MRC_GLBAC_SUR_SHIFT                 (10U)
/*! SUR - SecureUser Read
 *  0b0..Read access is not allowed in Secure User mode.
 *  0b1..Read access is allowed in Secure User mode.
 */
#define TRDC_MRC_GLBAC_SUR(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_SUR_SHIFT)) & TRDC_MRC_GLBAC_SUR_MASK)

#define TRDC_MRC_GLBAC_SPX_MASK                  (0x1000U)
#define TRDC_MRC_GLBAC_SPX_SHIFT                 (12U)
/*! SPX - SecurePriv Execute
 *  0b0..Execute access is not allowed in Secure Privilege mode.
 *  0b1..Execute access is allowed in Secure Privilege mode.
 */
#define TRDC_MRC_GLBAC_SPX(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_SPX_SHIFT)) & TRDC_MRC_GLBAC_SPX_MASK)

#define TRDC_MRC_GLBAC_SPW_MASK                  (0x2000U)
#define TRDC_MRC_GLBAC_SPW_SHIFT                 (13U)
/*! SPW - SecurePriv Write
 *  0b0..Write access is not allowed in Secure Privilege mode.
 *  0b1..Write access is allowed in Secure Privilege mode.
 */
#define TRDC_MRC_GLBAC_SPW(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_SPW_SHIFT)) & TRDC_MRC_GLBAC_SPW_MASK)

#define TRDC_MRC_GLBAC_SPR_MASK                  (0x4000U)
#define TRDC_MRC_GLBAC_SPR_SHIFT                 (14U)
/*! SPR - SecurePriv Read
 *  0b0..Read access is not allowed in Secure Privilege mode.
 *  0b1..Read access is allowed in Secure Privilege mode.
 */
#define TRDC_MRC_GLBAC_SPR(x)                    (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_SPR_SHIFT)) & TRDC_MRC_GLBAC_SPR_MASK)

#define TRDC_MRC_GLBAC_LK_MASK                   (0x80000000U)
#define TRDC_MRC_GLBAC_LK_SHIFT                  (31U)
/*! LK - LOCK
 *  0b0..This register is not locked and can be altered.
 *  0b1..This register is locked (read-only) and cannot be altered.
 */
#define TRDC_MRC_GLBAC_LK(x)                     (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_GLBAC_LK_SHIFT)) & TRDC_MRC_GLBAC_LK_MASK)
/*! @} */

/* The count of TRDC_MRC_GLBAC */
#define TRDC_MRC_GLBAC_COUNT                     (7U)

/* The count of TRDC_MRC_GLBAC */
#define TRDC_MRC_GLBAC_COUNT2                    (8U)

/*! @name MRC_DOM0_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM0_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM0_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM0_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM0_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM0_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM0_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM0_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM0_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM0_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM0_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM0_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM0_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM0_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM0_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM0_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM0_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM0_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM0_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM0_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM0_RGD_W */
#define TRDC_MRC_DOM0_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM0_RGD_W */
#define TRDC_MRC_DOM0_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM0_RGD_W */
#define TRDC_MRC_DOM0_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM0_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM0_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM0_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM0_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM0_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM0_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM0_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM0_RGD_NSE */
#define TRDC_MRC_DOM0_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM1_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM1_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM1_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM1_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM1_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM1_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM1_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM1_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM1_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM1_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM1_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM1_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM1_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM1_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM1_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM1_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM1_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM1_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM1_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM1_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM1_RGD_W */
#define TRDC_MRC_DOM1_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM1_RGD_W */
#define TRDC_MRC_DOM1_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM1_RGD_W */
#define TRDC_MRC_DOM1_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM1_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM1_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM1_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM1_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM1_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM1_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM1_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM1_RGD_NSE */
#define TRDC_MRC_DOM1_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM2_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM2_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM2_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM2_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM2_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM2_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM2_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM2_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM2_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM2_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM2_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM2_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM2_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM2_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM2_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM2_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM2_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM2_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM2_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM2_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM2_RGD_W */
#define TRDC_MRC_DOM2_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM2_RGD_W */
#define TRDC_MRC_DOM2_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM2_RGD_W */
#define TRDC_MRC_DOM2_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM2_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM2_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM2_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM2_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM2_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM2_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM2_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM2_RGD_NSE */
#define TRDC_MRC_DOM2_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM3_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM3_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM3_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM3_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM3_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM3_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM3_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM3_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM3_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM3_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM3_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM3_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM3_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM3_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM3_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM3_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM3_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM3_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM3_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM3_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM3_RGD_W */
#define TRDC_MRC_DOM3_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM3_RGD_W */
#define TRDC_MRC_DOM3_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM3_RGD_W */
#define TRDC_MRC_DOM3_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM3_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM3_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM3_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM3_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM3_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM3_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM3_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM3_RGD_NSE */
#define TRDC_MRC_DOM3_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM4_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM4_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM4_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM4_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM4_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM4_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM4_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM4_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM4_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM4_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM4_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM4_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM4_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM4_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM4_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM4_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM4_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM4_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM4_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM4_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM4_RGD_W */
#define TRDC_MRC_DOM4_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM4_RGD_W */
#define TRDC_MRC_DOM4_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM4_RGD_W */
#define TRDC_MRC_DOM4_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM4_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM4_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM4_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM4_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM4_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM4_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM4_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM4_RGD_NSE */
#define TRDC_MRC_DOM4_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM5_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM5_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM5_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM5_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM5_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM5_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM5_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM5_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM5_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM5_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM5_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM5_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM5_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM5_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM5_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM5_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM5_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM5_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM5_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM5_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM5_RGD_W */
#define TRDC_MRC_DOM5_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM5_RGD_W */
#define TRDC_MRC_DOM5_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM5_RGD_W */
#define TRDC_MRC_DOM5_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM5_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM5_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM5_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM5_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM5_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM5_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM5_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM5_RGD_NSE */
#define TRDC_MRC_DOM5_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM6_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM6_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM6_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM6_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM6_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM6_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM6_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM6_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM6_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM6_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM6_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM6_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM6_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM6_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM6_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM6_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM6_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM6_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM6_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM6_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM6_RGD_W */
#define TRDC_MRC_DOM6_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM6_RGD_W */
#define TRDC_MRC_DOM6_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM6_RGD_W */
#define TRDC_MRC_DOM6_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM6_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM6_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM6_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM6_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM6_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM6_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM6_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM6_RGD_NSE */
#define TRDC_MRC_DOM6_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM7_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM7_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM7_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM7_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM7_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM7_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM7_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM7_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM7_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM7_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM7_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM7_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM7_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM7_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM7_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM7_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM7_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM7_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM7_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM7_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM7_RGD_W */
#define TRDC_MRC_DOM7_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM7_RGD_W */
#define TRDC_MRC_DOM7_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM7_RGD_W */
#define TRDC_MRC_DOM7_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM7_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM7_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM7_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM7_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM7_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM7_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM7_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM7_RGD_NSE */
#define TRDC_MRC_DOM7_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM8_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM8_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM8_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM8_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM8_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM8_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM8_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM8_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM8_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM8_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM8_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM8_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM8_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM8_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM8_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM8_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM8_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM8_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM8_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM8_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM8_RGD_W */
#define TRDC_MRC_DOM8_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM8_RGD_W */
#define TRDC_MRC_DOM8_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM8_RGD_W */
#define TRDC_MRC_DOM8_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM8_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM8_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM8_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM8_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM8_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM8_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM8_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM8_RGD_NSE */
#define TRDC_MRC_DOM8_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM9_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM9_RGD_W_MRACSEL_MASK         (0x7U)
#define TRDC_MRC_DOM9_RGD_W_MRACSEL_SHIFT        (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM9_RGD_W_MRACSEL(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM9_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM9_RGD_W_VLD_MASK             (0x1U)
#define TRDC_MRC_DOM9_RGD_W_VLD_SHIFT            (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM9_RGD_W_VLD(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM9_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM9_RGD_W_NSE_MASK             (0x10U)
#define TRDC_MRC_DOM9_RGD_W_NSE_SHIFT            (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_W_NSE(x)               (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM9_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM9_RGD_W_END_ADDR_MASK        (0xFFFFC000U)
#define TRDC_MRC_DOM9_RGD_W_END_ADDR_SHIFT       (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM9_RGD_W_END_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM9_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM9_RGD_W_STRT_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM9_RGD_W_STRT_ADDR_SHIFT      (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM9_RGD_W_STRT_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM9_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM9_RGD_W */
#define TRDC_MRC_DOM9_RGD_W_COUNT                (7U)

/* The count of TRDC_MRC_DOM9_RGD_W */
#define TRDC_MRC_DOM9_RGD_W_COUNT2               (16U)

/* The count of TRDC_MRC_DOM9_RGD_W */
#define TRDC_MRC_DOM9_RGD_W_COUNT3               (2U)

/*! @name MRC_DOM9_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM9_RGD_NSE_BIT0_MASK          (0x1U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT0_SHIFT         (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT0(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT1_MASK          (0x2U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT1_SHIFT         (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT1(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT2_MASK          (0x4U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT2_SHIFT         (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT2(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT3_MASK          (0x8U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT3_SHIFT         (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT3(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT4_MASK          (0x10U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT4_SHIFT         (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT4(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT5_MASK          (0x20U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT5_SHIFT         (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT5(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT6_MASK          (0x40U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT6_SHIFT         (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT6(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT7_MASK          (0x80U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT7_SHIFT         (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT7(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT8_MASK          (0x100U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT8_SHIFT         (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT8(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT9_MASK          (0x200U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT9_SHIFT         (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT9(x)            (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT10_MASK         (0x400U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT10_SHIFT        (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT10(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT11_MASK         (0x800U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT11_SHIFT        (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT11(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT12_MASK         (0x1000U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT12_SHIFT        (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT12(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT13_MASK         (0x2000U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT13_SHIFT        (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT13(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT14_MASK         (0x4000U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT14_SHIFT        (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT14(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM9_RGD_NSE_BIT15_MASK         (0x8000U)
#define TRDC_MRC_DOM9_RGD_NSE_BIT15_SHIFT        (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM9_RGD_NSE_BIT15(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM9_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM9_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM9_RGD_NSE */
#define TRDC_MRC_DOM9_RGD_NSE_COUNT              (7U)

/*! @name MRC_DOM10_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM10_RGD_W_MRACSEL_MASK        (0x7U)
#define TRDC_MRC_DOM10_RGD_W_MRACSEL_SHIFT       (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM10_RGD_W_MRACSEL(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM10_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM10_RGD_W_VLD_MASK            (0x1U)
#define TRDC_MRC_DOM10_RGD_W_VLD_SHIFT           (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM10_RGD_W_VLD(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM10_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM10_RGD_W_NSE_MASK            (0x10U)
#define TRDC_MRC_DOM10_RGD_W_NSE_SHIFT           (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_W_NSE(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM10_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM10_RGD_W_END_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM10_RGD_W_END_ADDR_SHIFT      (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM10_RGD_W_END_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM10_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM10_RGD_W_STRT_ADDR_MASK      (0xFFFFC000U)
#define TRDC_MRC_DOM10_RGD_W_STRT_ADDR_SHIFT     (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM10_RGD_W_STRT_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM10_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM10_RGD_W */
#define TRDC_MRC_DOM10_RGD_W_COUNT               (7U)

/* The count of TRDC_MRC_DOM10_RGD_W */
#define TRDC_MRC_DOM10_RGD_W_COUNT2              (16U)

/* The count of TRDC_MRC_DOM10_RGD_W */
#define TRDC_MRC_DOM10_RGD_W_COUNT3              (2U)

/*! @name MRC_DOM10_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM10_RGD_NSE_BIT0_MASK         (0x1U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT0_SHIFT        (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT0(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT1_MASK         (0x2U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT1_SHIFT        (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT1(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT2_MASK         (0x4U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT2_SHIFT        (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT2(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT3_MASK         (0x8U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT3_SHIFT        (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT3(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT4_MASK         (0x10U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT4_SHIFT        (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT4(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT5_MASK         (0x20U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT5_SHIFT        (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT5(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT6_MASK         (0x40U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT6_SHIFT        (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT6(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT7_MASK         (0x80U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT7_SHIFT        (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT7(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT8_MASK         (0x100U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT8_SHIFT        (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT8(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT9_MASK         (0x200U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT9_SHIFT        (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT9(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT10_MASK        (0x400U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT10_SHIFT       (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT10(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT11_MASK        (0x800U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT11_SHIFT       (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT11(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT12_MASK        (0x1000U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT12_SHIFT       (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT12(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT13_MASK        (0x2000U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT13_SHIFT       (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT13(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT14_MASK        (0x4000U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT14_SHIFT       (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT14(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM10_RGD_NSE_BIT15_MASK        (0x8000U)
#define TRDC_MRC_DOM10_RGD_NSE_BIT15_SHIFT       (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM10_RGD_NSE_BIT15(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM10_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM10_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM10_RGD_NSE */
#define TRDC_MRC_DOM10_RGD_NSE_COUNT             (7U)

/*! @name MRC_DOM11_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM11_RGD_W_MRACSEL_MASK        (0x7U)
#define TRDC_MRC_DOM11_RGD_W_MRACSEL_SHIFT       (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM11_RGD_W_MRACSEL(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM11_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM11_RGD_W_VLD_MASK            (0x1U)
#define TRDC_MRC_DOM11_RGD_W_VLD_SHIFT           (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM11_RGD_W_VLD(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM11_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM11_RGD_W_NSE_MASK            (0x10U)
#define TRDC_MRC_DOM11_RGD_W_NSE_SHIFT           (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_W_NSE(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM11_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM11_RGD_W_END_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM11_RGD_W_END_ADDR_SHIFT      (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM11_RGD_W_END_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM11_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM11_RGD_W_STRT_ADDR_MASK      (0xFFFFC000U)
#define TRDC_MRC_DOM11_RGD_W_STRT_ADDR_SHIFT     (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM11_RGD_W_STRT_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM11_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM11_RGD_W */
#define TRDC_MRC_DOM11_RGD_W_COUNT               (7U)

/* The count of TRDC_MRC_DOM11_RGD_W */
#define TRDC_MRC_DOM11_RGD_W_COUNT2              (16U)

/* The count of TRDC_MRC_DOM11_RGD_W */
#define TRDC_MRC_DOM11_RGD_W_COUNT3              (2U)

/*! @name MRC_DOM11_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM11_RGD_NSE_BIT0_MASK         (0x1U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT0_SHIFT        (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT0(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT1_MASK         (0x2U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT1_SHIFT        (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT1(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT2_MASK         (0x4U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT2_SHIFT        (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT2(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT3_MASK         (0x8U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT3_SHIFT        (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT3(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT4_MASK         (0x10U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT4_SHIFT        (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT4(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT5_MASK         (0x20U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT5_SHIFT        (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT5(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT6_MASK         (0x40U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT6_SHIFT        (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT6(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT7_MASK         (0x80U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT7_SHIFT        (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT7(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT8_MASK         (0x100U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT8_SHIFT        (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT8(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT9_MASK         (0x200U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT9_SHIFT        (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT9(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT10_MASK        (0x400U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT10_SHIFT       (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT10(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT11_MASK        (0x800U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT11_SHIFT       (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT11(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT12_MASK        (0x1000U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT12_SHIFT       (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT12(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT13_MASK        (0x2000U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT13_SHIFT       (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT13(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT14_MASK        (0x4000U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT14_SHIFT       (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT14(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM11_RGD_NSE_BIT15_MASK        (0x8000U)
#define TRDC_MRC_DOM11_RGD_NSE_BIT15_SHIFT       (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM11_RGD_NSE_BIT15(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM11_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM11_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM11_RGD_NSE */
#define TRDC_MRC_DOM11_RGD_NSE_COUNT             (7U)

/*! @name MRC_DOM12_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM12_RGD_W_MRACSEL_MASK        (0x7U)
#define TRDC_MRC_DOM12_RGD_W_MRACSEL_SHIFT       (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM12_RGD_W_MRACSEL(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM12_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM12_RGD_W_VLD_MASK            (0x1U)
#define TRDC_MRC_DOM12_RGD_W_VLD_SHIFT           (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM12_RGD_W_VLD(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM12_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM12_RGD_W_NSE_MASK            (0x10U)
#define TRDC_MRC_DOM12_RGD_W_NSE_SHIFT           (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_W_NSE(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM12_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM12_RGD_W_END_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM12_RGD_W_END_ADDR_SHIFT      (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM12_RGD_W_END_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM12_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM12_RGD_W_STRT_ADDR_MASK      (0xFFFFC000U)
#define TRDC_MRC_DOM12_RGD_W_STRT_ADDR_SHIFT     (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM12_RGD_W_STRT_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM12_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM12_RGD_W */
#define TRDC_MRC_DOM12_RGD_W_COUNT               (7U)

/* The count of TRDC_MRC_DOM12_RGD_W */
#define TRDC_MRC_DOM12_RGD_W_COUNT2              (16U)

/* The count of TRDC_MRC_DOM12_RGD_W */
#define TRDC_MRC_DOM12_RGD_W_COUNT3              (2U)

/*! @name MRC_DOM12_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM12_RGD_NSE_BIT0_MASK         (0x1U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT0_SHIFT        (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT0(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT1_MASK         (0x2U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT1_SHIFT        (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT1(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT2_MASK         (0x4U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT2_SHIFT        (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT2(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT3_MASK         (0x8U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT3_SHIFT        (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT3(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT4_MASK         (0x10U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT4_SHIFT        (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT4(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT5_MASK         (0x20U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT5_SHIFT        (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT5(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT6_MASK         (0x40U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT6_SHIFT        (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT6(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT7_MASK         (0x80U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT7_SHIFT        (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT7(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT8_MASK         (0x100U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT8_SHIFT        (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT8(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT9_MASK         (0x200U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT9_SHIFT        (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT9(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT10_MASK        (0x400U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT10_SHIFT       (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT10(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT11_MASK        (0x800U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT11_SHIFT       (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT11(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT12_MASK        (0x1000U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT12_SHIFT       (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT12(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT13_MASK        (0x2000U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT13_SHIFT       (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT13(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT14_MASK        (0x4000U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT14_SHIFT       (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT14(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM12_RGD_NSE_BIT15_MASK        (0x8000U)
#define TRDC_MRC_DOM12_RGD_NSE_BIT15_SHIFT       (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM12_RGD_NSE_BIT15(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM12_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM12_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM12_RGD_NSE */
#define TRDC_MRC_DOM12_RGD_NSE_COUNT             (7U)

/*! @name MRC_DOM13_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM13_RGD_W_MRACSEL_MASK        (0x7U)
#define TRDC_MRC_DOM13_RGD_W_MRACSEL_SHIFT       (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM13_RGD_W_MRACSEL(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM13_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM13_RGD_W_VLD_MASK            (0x1U)
#define TRDC_MRC_DOM13_RGD_W_VLD_SHIFT           (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM13_RGD_W_VLD(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM13_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM13_RGD_W_NSE_MASK            (0x10U)
#define TRDC_MRC_DOM13_RGD_W_NSE_SHIFT           (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_W_NSE(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM13_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM13_RGD_W_END_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM13_RGD_W_END_ADDR_SHIFT      (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM13_RGD_W_END_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM13_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM13_RGD_W_STRT_ADDR_MASK      (0xFFFFC000U)
#define TRDC_MRC_DOM13_RGD_W_STRT_ADDR_SHIFT     (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM13_RGD_W_STRT_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM13_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM13_RGD_W */
#define TRDC_MRC_DOM13_RGD_W_COUNT               (7U)

/* The count of TRDC_MRC_DOM13_RGD_W */
#define TRDC_MRC_DOM13_RGD_W_COUNT2              (16U)

/* The count of TRDC_MRC_DOM13_RGD_W */
#define TRDC_MRC_DOM13_RGD_W_COUNT3              (2U)

/*! @name MRC_DOM13_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM13_RGD_NSE_BIT0_MASK         (0x1U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT0_SHIFT        (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT0(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT1_MASK         (0x2U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT1_SHIFT        (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT1(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT2_MASK         (0x4U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT2_SHIFT        (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT2(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT3_MASK         (0x8U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT3_SHIFT        (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT3(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT4_MASK         (0x10U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT4_SHIFT        (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT4(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT5_MASK         (0x20U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT5_SHIFT        (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT5(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT6_MASK         (0x40U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT6_SHIFT        (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT6(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT7_MASK         (0x80U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT7_SHIFT        (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT7(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT8_MASK         (0x100U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT8_SHIFT        (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT8(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT9_MASK         (0x200U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT9_SHIFT        (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT9(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT10_MASK        (0x400U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT10_SHIFT       (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT10(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT11_MASK        (0x800U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT11_SHIFT       (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT11(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT12_MASK        (0x1000U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT12_SHIFT       (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT12(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT13_MASK        (0x2000U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT13_SHIFT       (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT13(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT14_MASK        (0x4000U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT14_SHIFT       (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT14(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM13_RGD_NSE_BIT15_MASK        (0x8000U)
#define TRDC_MRC_DOM13_RGD_NSE_BIT15_SHIFT       (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM13_RGD_NSE_BIT15(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM13_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM13_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM13_RGD_NSE */
#define TRDC_MRC_DOM13_RGD_NSE_COUNT             (7U)

/*! @name MRC_DOM14_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM14_RGD_W_MRACSEL_MASK        (0x7U)
#define TRDC_MRC_DOM14_RGD_W_MRACSEL_SHIFT       (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM14_RGD_W_MRACSEL(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM14_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM14_RGD_W_VLD_MASK            (0x1U)
#define TRDC_MRC_DOM14_RGD_W_VLD_SHIFT           (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM14_RGD_W_VLD(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM14_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM14_RGD_W_NSE_MASK            (0x10U)
#define TRDC_MRC_DOM14_RGD_W_NSE_SHIFT           (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_W_NSE(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM14_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM14_RGD_W_END_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM14_RGD_W_END_ADDR_SHIFT      (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM14_RGD_W_END_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM14_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM14_RGD_W_STRT_ADDR_MASK      (0xFFFFC000U)
#define TRDC_MRC_DOM14_RGD_W_STRT_ADDR_SHIFT     (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM14_RGD_W_STRT_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM14_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM14_RGD_W */
#define TRDC_MRC_DOM14_RGD_W_COUNT               (7U)

/* The count of TRDC_MRC_DOM14_RGD_W */
#define TRDC_MRC_DOM14_RGD_W_COUNT2              (16U)

/* The count of TRDC_MRC_DOM14_RGD_W */
#define TRDC_MRC_DOM14_RGD_W_COUNT3              (2U)

/*! @name MRC_DOM14_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM14_RGD_NSE_BIT0_MASK         (0x1U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT0_SHIFT        (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT0(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT1_MASK         (0x2U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT1_SHIFT        (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT1(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT2_MASK         (0x4U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT2_SHIFT        (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT2(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT3_MASK         (0x8U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT3_SHIFT        (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT3(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT4_MASK         (0x10U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT4_SHIFT        (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT4(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT5_MASK         (0x20U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT5_SHIFT        (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT5(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT6_MASK         (0x40U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT6_SHIFT        (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT6(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT7_MASK         (0x80U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT7_SHIFT        (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT7(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT8_MASK         (0x100U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT8_SHIFT        (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT8(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT9_MASK         (0x200U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT9_SHIFT        (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT9(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT10_MASK        (0x400U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT10_SHIFT       (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT10(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT11_MASK        (0x800U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT11_SHIFT       (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT11(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT12_MASK        (0x1000U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT12_SHIFT       (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT12(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT13_MASK        (0x2000U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT13_SHIFT       (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT13(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT14_MASK        (0x4000U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT14_SHIFT       (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT14(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM14_RGD_NSE_BIT15_MASK        (0x8000U)
#define TRDC_MRC_DOM14_RGD_NSE_BIT15_SHIFT       (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM14_RGD_NSE_BIT15(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM14_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM14_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM14_RGD_NSE */
#define TRDC_MRC_DOM14_RGD_NSE_COUNT             (7U)

/*! @name MRC_DOM15_RGD_W - MRC Region Descriptor Word 0..MRC Region Descriptor Word 1 */
/*! @{ */

#define TRDC_MRC_DOM15_RGD_W_MRACSEL_MASK        (0x7U)
#define TRDC_MRC_DOM15_RGD_W_MRACSEL_SHIFT       (0U)
/*! MRACSEL - Memory Region Access Control Select
 *  0b000..Select MRC_GLBAC0 access control policy
 *  0b001..Select MRC_GLBAC1 access control policy
 *  0b010..Select MRC_GLBAC2 access control policy
 *  0b011..Select MRC_GLBAC3 access control policy
 *  0b100..Select MRC_GLBAC4 access control policy
 *  0b101..Select MRC_GLBAC5 access control policy
 *  0b110..Select MRC_GLBAC6 access control policy
 *  0b111..Select MRC_GLBAC7 access control policy
 */
#define TRDC_MRC_DOM15_RGD_W_MRACSEL(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_W_MRACSEL_SHIFT)) & TRDC_MRC_DOM15_RGD_W_MRACSEL_MASK)

#define TRDC_MRC_DOM15_RGD_W_VLD_MASK            (0x1U)
#define TRDC_MRC_DOM15_RGD_W_VLD_SHIFT           (0U)
/*! VLD - Valid */
#define TRDC_MRC_DOM15_RGD_W_VLD(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_W_VLD_SHIFT)) & TRDC_MRC_DOM15_RGD_W_VLD_MASK)

#define TRDC_MRC_DOM15_RGD_W_NSE_MASK            (0x10U)
#define TRDC_MRC_DOM15_RGD_W_NSE_SHIFT           (4U)
/*! NSE - NonSecure Enable
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in this register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_W_NSE(x)              (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_W_NSE_SHIFT)) & TRDC_MRC_DOM15_RGD_W_NSE_MASK)

#define TRDC_MRC_DOM15_RGD_W_END_ADDR_MASK       (0xFFFFC000U)
#define TRDC_MRC_DOM15_RGD_W_END_ADDR_SHIFT      (14U)
/*! END_ADDR - End Address */
#define TRDC_MRC_DOM15_RGD_W_END_ADDR(x)         (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_W_END_ADDR_SHIFT)) & TRDC_MRC_DOM15_RGD_W_END_ADDR_MASK)

#define TRDC_MRC_DOM15_RGD_W_STRT_ADDR_MASK      (0xFFFFC000U)
#define TRDC_MRC_DOM15_RGD_W_STRT_ADDR_SHIFT     (14U)
/*! STRT_ADDR - Start Address */
#define TRDC_MRC_DOM15_RGD_W_STRT_ADDR(x)        (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_W_STRT_ADDR_SHIFT)) & TRDC_MRC_DOM15_RGD_W_STRT_ADDR_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM15_RGD_W */
#define TRDC_MRC_DOM15_RGD_W_COUNT               (7U)

/* The count of TRDC_MRC_DOM15_RGD_W */
#define TRDC_MRC_DOM15_RGD_W_COUNT2              (16U)

/* The count of TRDC_MRC_DOM15_RGD_W */
#define TRDC_MRC_DOM15_RGD_W_COUNT3              (2U)

/*! @name MRC_DOM15_RGD_NSE - MRC Region Descriptor NonSecure Enable */
/*! @{ */

#define TRDC_MRC_DOM15_RGD_NSE_BIT0_MASK         (0x1U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT0_SHIFT        (0U)
/*! BIT0 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT0(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT0_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT0_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT1_MASK         (0x2U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT1_SHIFT        (1U)
/*! BIT1 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT1(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT1_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT1_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT2_MASK         (0x4U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT2_SHIFT        (2U)
/*! BIT2 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT2(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT2_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT2_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT3_MASK         (0x8U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT3_SHIFT        (3U)
/*! BIT3 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT3(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT3_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT3_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT4_MASK         (0x10U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT4_SHIFT        (4U)
/*! BIT4 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT4(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT4_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT4_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT5_MASK         (0x20U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT5_SHIFT        (5U)
/*! BIT5 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT5(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT5_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT5_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT6_MASK         (0x40U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT6_SHIFT        (6U)
/*! BIT6 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT6(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT6_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT6_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT7_MASK         (0x80U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT7_SHIFT        (7U)
/*! BIT7 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT7(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT7_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT7_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT8_MASK         (0x100U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT8_SHIFT        (8U)
/*! BIT8 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT8(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT8_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT8_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT9_MASK         (0x200U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT9_SHIFT        (9U)
/*! BIT9 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT9(x)           (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT9_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT9_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT10_MASK        (0x400U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT10_SHIFT       (10U)
/*! BIT10 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT10(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT10_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT10_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT11_MASK        (0x800U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT11_SHIFT       (11U)
/*! BIT11 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT11(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT11_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT11_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT12_MASK        (0x1000U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT12_SHIFT       (12U)
/*! BIT12 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT12(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT12_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT12_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT13_MASK        (0x2000U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT13_SHIFT       (13U)
/*! BIT13 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT13(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT13_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT13_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT14_MASK        (0x4000U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT14_SHIFT       (14U)
/*! BIT14 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT14(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT14_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT14_MASK)

#define TRDC_MRC_DOM15_RGD_NSE_BIT15_MASK        (0x8000U)
#define TRDC_MRC_DOM15_RGD_NSE_BIT15_SHIFT       (15U)
/*! BIT15 - Bit n NonSecure Enable [n = 0 - 15]
 *  0b0..Secure accesses to region r are based on corresponding MRACSEL field in this register
 *       (MRCm_DOMd_RGDr_Ww[MRACSEL]), nonsecure accesses to region r are not allowed.
 *  0b1..Secure accesses to region r are not allowed, nonsecure accesses to region r are based on corresponding
 *       MRACSEL field in register (MRCm_DOMd_RGDr_Ww[MRACSEL]).
 */
#define TRDC_MRC_DOM15_RGD_NSE_BIT15(x)          (((uint32_t)(((uint32_t)(x)) << TRDC_MRC_DOM15_RGD_NSE_BIT15_SHIFT)) & TRDC_MRC_DOM15_RGD_NSE_BIT15_MASK)
/*! @} */

/* The count of TRDC_MRC_DOM15_RGD_NSE */
#define TRDC_MRC_DOM15_RGD_NSE_COUNT             (7U)


/*!
 * @}
 */ /* end of group TRDC_Register_Masks */


/* TRDC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral TRDC1 base address */
  #define TRDC1_BASE                               (0x54270000u)
  /** Peripheral TRDC1 base address */
  #define TRDC1_BASE_NS                            (0x44270000u)
  /** Peripheral TRDC1 base pointer */
  #define TRDC1                                    ((TRDC_Type *)TRDC1_BASE)
  /** Peripheral TRDC1 base pointer */
  #define TRDC1_NS                                 ((TRDC_Type *)TRDC1_BASE_NS)
  /** Peripheral TRDC2 base address */
  #define TRDC2_BASE                               (0x52460000u)
  /** Peripheral TRDC2 base address */
  #define TRDC2_BASE_NS                            (0x42460000u)
  /** Peripheral TRDC2 base pointer */
  #define TRDC2                                    ((TRDC_Type *)TRDC2_BASE)
  /** Peripheral TRDC2 base pointer */
  #define TRDC2_NS                                 ((TRDC_Type *)TRDC2_BASE_NS)
  /** Peripheral TRDC3 base address */
  #define TRDC3_BASE                               (0x52810000u)
  /** Peripheral TRDC3 base address */
  #define TRDC3_BASE_NS                            (0x42810000u)
  /** Peripheral TRDC3 base pointer */
  #define TRDC3                                    ((TRDC_Type *)TRDC3_BASE)
  /** Peripheral TRDC3 base pointer */
  #define TRDC3_NS                                 ((TRDC_Type *)TRDC3_BASE_NS)
  /** Array initializer of TRDC peripheral base addresses */
  #define TRDC_BASE_ADDRS                          { 0u, TRDC1_BASE, TRDC2_BASE, TRDC3_BASE }
  /** Array initializer of TRDC peripheral base pointers */
  #define TRDC_BASE_PTRS                           { (TRDC_Type *)0u, TRDC1, TRDC2, TRDC3 }
  /** Array initializer of TRDC peripheral base addresses */
  #define TRDC_BASE_ADDRS_NS                       { 0u, TRDC1_BASE_NS, TRDC2_BASE_NS, TRDC3_BASE_NS }
  /** Array initializer of TRDC peripheral base pointers */
  #define TRDC_BASE_PTRS_NS                        { (TRDC_Type *)0u, TRDC1_NS, TRDC2_NS, TRDC3_NS }
#else
  /** Peripheral TRDC1 base address */
  #define TRDC1_BASE                               (0x44270000u)
  /** Peripheral TRDC1 base pointer */
  #define TRDC1                                    ((TRDC_Type *)TRDC1_BASE)
  /** Peripheral TRDC2 base address */
  #define TRDC2_BASE                               (0x42460000u)
  /** Peripheral TRDC2 base pointer */
  #define TRDC2                                    ((TRDC_Type *)TRDC2_BASE)
  /** Peripheral TRDC3 base address */
  #define TRDC3_BASE                               (0x42810000u)
  /** Peripheral TRDC3 base pointer */
  #define TRDC3                                    ((TRDC_Type *)TRDC3_BASE)
  /** Array initializer of TRDC peripheral base addresses */
  #define TRDC_BASE_ADDRS                          { 0u, TRDC1_BASE, TRDC2_BASE, TRDC3_BASE }
  /** Array initializer of TRDC peripheral base pointers */
  #define TRDC_BASE_PTRS                           { (TRDC_Type *)0u, TRDC1, TRDC2, TRDC3 }
#endif
/** Interrupt vectors for the TRDC peripheral type */
#define TRDC_IRQS                                { NotAvail_IRQn, TRDC_MGR_AON_IRQn, TRDC_MGR_WKUP_IRQn, TRDC_MGR_MEGA_IRQn }
#define MBC0_MEMORY_CFG_WORD_COUNT {1,2,4,1}
#define MBC1_MEMORY_CFG_WORD_COUNT {1,1,1,1}
#define MBC2_MEMORY_CFG_WORD_COUNT {9,6,1,1}
#define MBC3_MEMORY_CFG_WORD_COUNT {3,0,0,0}
#define MBC_MEMORY_CFG_WORD_COUNT {MBC0_MEMORY_CFG_WORD_COUNT , MBC1_MEMORY_CFG_WORD_COUNT, MBC2_MEMORY_CFG_WORD_COUNT, MBC3_MEMORY_CFG_WORD_COUNT}
#define MBC0_MEMORY_NSE_WORD_COUNT {1,1,1,1}
#define MBC1_MEMORY_NSE_WORD_COUNT {1,1,1,1}
#define MBC2_MEMORY_NSE_WORD_COUNT {3,2,1,1}
#define MBC3_MEMORY_NSE_WORD_COUNT {1,0,0,0}
#define MBC_MEMORY_NSE_WORD_COUNT {MBC0_MEMORY_NSE_WORD_COUNT , MBC1_MEMORY_NSE_WORD_COUNT, MBC2_MEMORY_NSE_WORD_COUNT, MBC3_MEMORY_NSE_WORD_COUNT}


/*!
 * @}
 */ /* end of group TRDC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- TSTMR Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TSTMR_Peripheral_Access_Layer TSTMR Peripheral Access Layer
 * @{
 */

/** TSTMR - Register Layout Typedef */
typedef struct {
  __I  uint32_t L;                                 /**< Timestamp Timer Low, offset: 0x0 */
  __I  uint32_t H;                                 /**< Timestamp Timer High, offset: 0x4 */
} TSTMR_Type;

/* ----------------------------------------------------------------------------
   -- TSTMR Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup TSTMR_Register_Masks TSTMR Register Masks
 * @{
 */

/*! @name L - Timestamp Timer Low */
/*! @{ */

#define TSTMR_L_VALUE_MASK                       (0xFFFFFFFFU)
#define TSTMR_L_VALUE_SHIFT                      (0U)
/*! VALUE - Timestamp Timer Low */
#define TSTMR_L_VALUE(x)                         (((uint32_t)(((uint32_t)(x)) << TSTMR_L_VALUE_SHIFT)) & TSTMR_L_VALUE_MASK)
/*! @} */

/*! @name H - Timestamp Timer High */
/*! @{ */

#define TSTMR_H_VALUE_MASK                       (0xFFFFFFU)
#define TSTMR_H_VALUE_SHIFT                      (0U)
/*! VALUE - Timestamp Timer High */
#define TSTMR_H_VALUE(x)                         (((uint32_t)(((uint32_t)(x)) << TSTMR_H_VALUE_SHIFT)) & TSTMR_H_VALUE_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group TSTMR_Register_Masks */


/* TSTMR - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral TSTMR1_TSTMRA base address */
  #define TSTMR1_TSTMRA_BASE                       (0x542C0000u)
  /** Peripheral TSTMR1_TSTMRA base address */
  #define TSTMR1_TSTMRA_BASE_NS                    (0x442C0000u)
  /** Peripheral TSTMR1_TSTMRA base pointer */
  #define TSTMR1_TSTMRA                            ((TSTMR_Type *)TSTMR1_TSTMRA_BASE)
  /** Peripheral TSTMR1_TSTMRA base pointer */
  #define TSTMR1_TSTMRA_NS                         ((TSTMR_Type *)TSTMR1_TSTMRA_BASE_NS)
  /** Peripheral TSTMR2_TSTMRA base address */
  #define TSTMR2_TSTMRA_BASE                       (0x52480000u)
  /** Peripheral TSTMR2_TSTMRA base address */
  #define TSTMR2_TSTMRA_BASE_NS                    (0x42480000u)
  /** Peripheral TSTMR2_TSTMRA base pointer */
  #define TSTMR2_TSTMRA                            ((TSTMR_Type *)TSTMR2_TSTMRA_BASE)
  /** Peripheral TSTMR2_TSTMRA base pointer */
  #define TSTMR2_TSTMRA_NS                         ((TSTMR_Type *)TSTMR2_TSTMRA_BASE_NS)
  /** Array initializer of TSTMR peripheral base addresses */
  #define TSTMR_BASE_ADDRS                         { TSTMR1_TSTMRA_BASE, TSTMR2_TSTMRA_BASE }
  /** Array initializer of TSTMR peripheral base pointers */
  #define TSTMR_BASE_PTRS                          { TSTMR1_TSTMRA, TSTMR2_TSTMRA }
  /** Array initializer of TSTMR peripheral base addresses */
  #define TSTMR_BASE_ADDRS_NS                      { TSTMR1_TSTMRA_BASE_NS, TSTMR2_TSTMRA_BASE_NS }
  /** Array initializer of TSTMR peripheral base pointers */
  #define TSTMR_BASE_PTRS_NS                       { TSTMR1_TSTMRA_NS, TSTMR2_TSTMRA_NS }
#else
  /** Peripheral TSTMR1_TSTMRA base address */
  #define TSTMR1_TSTMRA_BASE                       (0x442C0000u)
  /** Peripheral TSTMR1_TSTMRA base pointer */
  #define TSTMR1_TSTMRA                            ((TSTMR_Type *)TSTMR1_TSTMRA_BASE)
  /** Peripheral TSTMR2_TSTMRA base address */
  #define TSTMR2_TSTMRA_BASE                       (0x42480000u)
  /** Peripheral TSTMR2_TSTMRA base pointer */
  #define TSTMR2_TSTMRA                            ((TSTMR_Type *)TSTMR2_TSTMRA_BASE)
  /** Array initializer of TSTMR peripheral base addresses */
  #define TSTMR_BASE_ADDRS                         { TSTMR1_TSTMRA_BASE, TSTMR2_TSTMRA_BASE }
  /** Array initializer of TSTMR peripheral base pointers */
  #define TSTMR_BASE_PTRS                          { TSTMR1_TSTMRA, TSTMR2_TSTMRA }
#endif
/* Extra definition */
#define TSTMR_CLOCK_FREQUENCY_MHZ                (24U)


/*!
 * @}
 */ /* end of group TSTMR_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- USBHSDCD Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup USBHSDCD_Peripheral_Access_Layer USBHSDCD Peripheral Access Layer
 * @{
 */

/** USBHSDCD - Register Layout Typedef */
typedef struct {
  __IO uint32_t CONTROL;                           /**< Control, offset: 0x0 */
  __IO uint32_t CLOCK;                             /**< Clock, offset: 0x4 */
  __I  uint32_t STATUS;                            /**< Status, offset: 0x8 */
  __IO uint32_t SIGNAL_OVERRIDE;                   /**< Signal Override, offset: 0xC */
  __IO uint32_t TIMER0;                            /**< TIMER0, offset: 0x10 */
  __IO uint32_t TIMER1;                            /**< TIMER1, offset: 0x14 */
  union {                                          /* offset: 0x18 */
    __IO uint32_t TIMER2_BC11;                       /**< TIMER2_BC11, offset: 0x18 */
    __IO uint32_t TIMER2_BC12;                       /**< TIMER2_BC12, offset: 0x18 */
  };
} USBHSDCD_Type;

/* ----------------------------------------------------------------------------
   -- USBHSDCD Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup USBHSDCD_Register_Masks USBHSDCD Register Masks
 * @{
 */

/*! @name CONTROL - Control */
/*! @{ */

#define USBHSDCD_CONTROL_IACK_MASK               (0x1U)
#define USBHSDCD_CONTROL_IACK_SHIFT              (0U)
/*! IACK - Interrupt Acknowledge
 *  0b0..Do not clear the interrupt.
 *  0b1..Clear the IF field (interrupt flag).
 */
#define USBHSDCD_CONTROL_IACK(x)                 (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CONTROL_IACK_SHIFT)) & USBHSDCD_CONTROL_IACK_MASK)

#define USBHSDCD_CONTROL_IF_MASK                 (0x100U)
#define USBHSDCD_CONTROL_IF_SHIFT                (8U)
/*! IF - Interrupt Flag
 *  0b0..No interrupt is pending.
 *  0b1..An interrupt is pending.
 */
#define USBHSDCD_CONTROL_IF(x)                   (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CONTROL_IF_SHIFT)) & USBHSDCD_CONTROL_IF_MASK)

#define USBHSDCD_CONTROL_IE_MASK                 (0x10000U)
#define USBHSDCD_CONTROL_IE_SHIFT                (16U)
/*! IE - Interrupt Enable
 *  0b0..Disable interrupts to the system.
 *  0b1..Enable interrupts to the system.
 */
#define USBHSDCD_CONTROL_IE(x)                   (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CONTROL_IE_SHIFT)) & USBHSDCD_CONTROL_IE_MASK)

#define USBHSDCD_CONTROL_BC12_MASK               (0x20000U)
#define USBHSDCD_CONTROL_BC12_SHIFT              (17U)
/*! BC12 - Battery Charging Revision 1.2 Compatibility
 *  0b0..Compatible with BC1.1 (default)
 *  0b1..Compatible with BC1.2
 */
#define USBHSDCD_CONTROL_BC12(x)                 (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CONTROL_BC12_SHIFT)) & USBHSDCD_CONTROL_BC12_MASK)

#define USBHSDCD_CONTROL_START_MASK              (0x1000000U)
#define USBHSDCD_CONTROL_START_SHIFT             (24U)
/*! START - Start Change Detection Sequence
 *  0b0..Do not start the sequence. Writes of this value have no effect.
 *  0b1..Initiate the charger detection sequence. If the sequence is already running, writes of this value have no effect.
 */
#define USBHSDCD_CONTROL_START(x)                (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CONTROL_START_SHIFT)) & USBHSDCD_CONTROL_START_MASK)

#define USBHSDCD_CONTROL_SR_MASK                 (0x2000000U)
#define USBHSDCD_CONTROL_SR_SHIFT                (25U)
/*! SR - Software Reset
 *  0b0..Do not perform a software reset.
 *  0b1..Perform a software reset.
 */
#define USBHSDCD_CONTROL_SR(x)                   (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CONTROL_SR_SHIFT)) & USBHSDCD_CONTROL_SR_MASK)
/*! @} */

/*! @name CLOCK - Clock */
/*! @{ */

#define USBHSDCD_CLOCK_CLOCK_UNIT_MASK           (0x1U)
#define USBHSDCD_CLOCK_CLOCK_UNIT_SHIFT          (0U)
/*! CLOCK_UNIT - Unit of Measurement Encoding for Clock Speed
 *  0b0..kHz Speed (between 4 kHz and 1023 kHz)
 *  0b1..MHz Speed (between 1 MHz and 1023 MHz)
 */
#define USBHSDCD_CLOCK_CLOCK_UNIT(x)             (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CLOCK_CLOCK_UNIT_SHIFT)) & USBHSDCD_CLOCK_CLOCK_UNIT_MASK)

#define USBHSDCD_CLOCK_CLOCK_SPEED_MASK          (0xFFCU)
#define USBHSDCD_CLOCK_CLOCK_SPEED_SHIFT         (2U)
/*! CLOCK_SPEED - Numerical Value of Clock Speed in Binary */
#define USBHSDCD_CLOCK_CLOCK_SPEED(x)            (((uint32_t)(((uint32_t)(x)) << USBHSDCD_CLOCK_CLOCK_SPEED_SHIFT)) & USBHSDCD_CLOCK_CLOCK_SPEED_MASK)
/*! @} */

/*! @name STATUS - Status */
/*! @{ */

#define USBHSDCD_STATUS_SEQ_RES_MASK             (0x30000U)
#define USBHSDCD_STATUS_SEQ_RES_SHIFT            (16U)
/*! SEQ_RES - Charger Detection Sequence Results
 *  0b00..No results to report.
 *  0b01..Attached to an SDP. Must comply with USB 2.0 by drawing only 2.5 mA (max) until connected.
 *  0b10..Attached to a charging port. The exact meaning depends on the STATUS[SEQ_STAT] field (value 0: Attached
 *        to either a CDP or a DCP. The charger type detection has not completed. value 1: Attached to a CDP. The
 *        charger type detection has completed.)
 *  0b11..Attached to a DCP.
 */
#define USBHSDCD_STATUS_SEQ_RES(x)               (((uint32_t)(((uint32_t)(x)) << USBHSDCD_STATUS_SEQ_RES_SHIFT)) & USBHSDCD_STATUS_SEQ_RES_MASK)

#define USBHSDCD_STATUS_SEQ_STAT_MASK            (0xC0000U)
#define USBHSDCD_STATUS_SEQ_STAT_SHIFT           (18U)
/*! SEQ_STAT - Charger Detection Sequence Status
 *  0b00..The module is either not enabled, or the module is enabled but the data pins have not yet been detected.
 *  0b01..Data pin contact detection is complete.
 *  0b10..Charging port detection is complete.
 *  0b11..Charger type detection is complete.
 */
#define USBHSDCD_STATUS_SEQ_STAT(x)              (((uint32_t)(((uint32_t)(x)) << USBHSDCD_STATUS_SEQ_STAT_SHIFT)) & USBHSDCD_STATUS_SEQ_STAT_MASK)

#define USBHSDCD_STATUS_ERR_MASK                 (0x100000U)
#define USBHSDCD_STATUS_ERR_SHIFT                (20U)
/*! ERR - Error Flag
 *  0b0..No sequence errors.
 *  0b1..Error in the detection sequence.
 */
#define USBHSDCD_STATUS_ERR(x)                   (((uint32_t)(((uint32_t)(x)) << USBHSDCD_STATUS_ERR_SHIFT)) & USBHSDCD_STATUS_ERR_MASK)

#define USBHSDCD_STATUS_TO_MASK                  (0x200000U)
#define USBHSDCD_STATUS_TO_SHIFT                 (21U)
/*! TO - Timeout Flag
 *  0b0..The detection sequence is not running for over 1 s.
 *  0b1..It is over 1 s since the data pin contact was detected and debounced.
 */
#define USBHSDCD_STATUS_TO(x)                    (((uint32_t)(((uint32_t)(x)) << USBHSDCD_STATUS_TO_SHIFT)) & USBHSDCD_STATUS_TO_MASK)

#define USBHSDCD_STATUS_ACTIVE_MASK              (0x400000U)
#define USBHSDCD_STATUS_ACTIVE_SHIFT             (22U)
/*! ACTIVE - Active Status Indicator
 *  0b0..The sequence is not running.
 *  0b1..The sequence is running.
 */
#define USBHSDCD_STATUS_ACTIVE(x)                (((uint32_t)(((uint32_t)(x)) << USBHSDCD_STATUS_ACTIVE_SHIFT)) & USBHSDCD_STATUS_ACTIVE_MASK)
/*! @} */

/*! @name SIGNAL_OVERRIDE - Signal Override */
/*! @{ */

#define USBHSDCD_SIGNAL_OVERRIDE_PS_MASK         (0x3U)
#define USBHSDCD_SIGNAL_OVERRIDE_PS_SHIFT        (0U)
/*! PS - Phase Selection
 *  0b00..No overrides. Field must remain at this value during normal USB data communication to prevent unexpected
 *        conditions on USB_DP and USB_DM pins. (Default)
 *  0b01..Reserved, not for customer use.
 *  0b10..Enables VDP_SRC voltage source for the USB_DP pin and IDM_SINK current source for the USB_DM pin.
 *  0b11..Reserved, not for customer use.
 */
#define USBHSDCD_SIGNAL_OVERRIDE_PS(x)           (((uint32_t)(((uint32_t)(x)) << USBHSDCD_SIGNAL_OVERRIDE_PS_SHIFT)) & USBHSDCD_SIGNAL_OVERRIDE_PS_MASK)
/*! @} */

/*! @name TIMER0 - TIMER0 */
/*! @{ */

#define USBHSDCD_TIMER0_TUNITCON_MASK            (0xFFFU)
#define USBHSDCD_TIMER0_TUNITCON_SHIFT           (0U)
/*! TUNITCON - Unit Connection Timer Elapse (in ms) */
#define USBHSDCD_TIMER0_TUNITCON(x)              (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER0_TUNITCON_SHIFT)) & USBHSDCD_TIMER0_TUNITCON_MASK)

#define USBHSDCD_TIMER0_TSEQ_INIT_MASK           (0x3FF0000U)
#define USBHSDCD_TIMER0_TSEQ_INIT_SHIFT          (16U)
/*! TSEQ_INIT - Sequence Initiation Time
 *  0b0000000000-0b1111111111..0 ms - 1023 ms
 */
#define USBHSDCD_TIMER0_TSEQ_INIT(x)             (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER0_TSEQ_INIT_SHIFT)) & USBHSDCD_TIMER0_TSEQ_INIT_MASK)
/*! @} */

/*! @name TIMER1 - TIMER1 */
/*! @{ */

#define USBHSDCD_TIMER1_TVDPSRC_ON_MASK          (0x3FFU)
#define USBHSDCD_TIMER1_TVDPSRC_ON_SHIFT         (0U)
/*! TVDPSRC_ON - Time Period Comparator Enabled
 *  0b0000000001-0b1111111111..1 ms - 1023 ms
 */
#define USBHSDCD_TIMER1_TVDPSRC_ON(x)            (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER1_TVDPSRC_ON_SHIFT)) & USBHSDCD_TIMER1_TVDPSRC_ON_MASK)

#define USBHSDCD_TIMER1_TDCD_DBNC_MASK           (0x3FF0000U)
#define USBHSDCD_TIMER1_TDCD_DBNC_SHIFT          (16U)
/*! TDCD_DBNC - Time Period to Debounce D+ Signal
 *  0b0000000001-0b1111111111..1 ms - 1023 ms
 */
#define USBHSDCD_TIMER1_TDCD_DBNC(x)             (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER1_TDCD_DBNC_SHIFT)) & USBHSDCD_TIMER1_TDCD_DBNC_MASK)
/*! @} */

/*! @name TIMER2_BC11 - TIMER2_BC11 */
/*! @{ */

#define USBHSDCD_TIMER2_BC11_CHECK_DM_MASK       (0xFU)
#define USBHSDCD_TIMER2_BC11_CHECK_DM_SHIFT      (0U)
/*! CHECK_DM - Time Before Check of D- Line
 *  0b0001-0b1111..1 ms - 15 ms
 */
#define USBHSDCD_TIMER2_BC11_CHECK_DM(x)         (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER2_BC11_CHECK_DM_SHIFT)) & USBHSDCD_TIMER2_BC11_CHECK_DM_MASK)

#define USBHSDCD_TIMER2_BC11_TVDPSRC_CON_MASK    (0x3FF0000U)
#define USBHSDCD_TIMER2_BC11_TVDPSRC_CON_SHIFT   (16U)
/*! TVDPSRC_CON - Time Period Before Enabling D+ Pullup
 *  0b0000000001-0b1111111111..1 ms - 1023 ms
 */
#define USBHSDCD_TIMER2_BC11_TVDPSRC_CON(x)      (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER2_BC11_TVDPSRC_CON_SHIFT)) & USBHSDCD_TIMER2_BC11_TVDPSRC_CON_MASK)
/*! @} */

/*! @name TIMER2_BC12 - TIMER2_BC12 */
/*! @{ */

#define USBHSDCD_TIMER2_BC12_TVDMSRC_ON_MASK     (0x3FFU)
#define USBHSDCD_TIMER2_BC12_TVDMSRC_ON_SHIFT    (0U)
/*! TVDMSRC_ON - TVDMSRC_ON
 *  0b0000000000-0b0000101000..0 ms - 40 ms
 */
#define USBHSDCD_TIMER2_BC12_TVDMSRC_ON(x)       (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER2_BC12_TVDMSRC_ON_SHIFT)) & USBHSDCD_TIMER2_BC12_TVDMSRC_ON_MASK)

#define USBHSDCD_TIMER2_BC12_TWAIT_AFTER_PRD_MASK (0x3FF0000U)
#define USBHSDCD_TIMER2_BC12_TWAIT_AFTER_PRD_SHIFT (16U)
/*! TWAIT_AFTER_PRD - TWAIT_AFTER_PRD
 *  0b0000000001-0b1111111111..1 ms - 1023 ms
 */
#define USBHSDCD_TIMER2_BC12_TWAIT_AFTER_PRD(x)  (((uint32_t)(((uint32_t)(x)) << USBHSDCD_TIMER2_BC12_TWAIT_AFTER_PRD_SHIFT)) & USBHSDCD_TIMER2_BC12_TWAIT_AFTER_PRD_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group USBHSDCD_Register_Masks */


/* USBHSDCD - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral USBHSDCD1 base address */
  #define USBHSDCD1_BASE                           (0x52CA0800u)
  /** Peripheral USBHSDCD1 base address */
  #define USBHSDCD1_BASE_NS                        (0x42CA0800u)
  /** Peripheral USBHSDCD1 base pointer */
  #define USBHSDCD1                                ((USBHSDCD_Type *)USBHSDCD1_BASE)
  /** Peripheral USBHSDCD1 base pointer */
  #define USBHSDCD1_NS                             ((USBHSDCD_Type *)USBHSDCD1_BASE_NS)
  /** Peripheral USBHSDCD2 base address */
  #define USBHSDCD2_BASE                           (0x52CB0800u)
  /** Peripheral USBHSDCD2 base address */
  #define USBHSDCD2_BASE_NS                        (0x42CB0800u)
  /** Peripheral USBHSDCD2 base pointer */
  #define USBHSDCD2                                ((USBHSDCD_Type *)USBHSDCD2_BASE)
  /** Peripheral USBHSDCD2 base pointer */
  #define USBHSDCD2_NS                             ((USBHSDCD_Type *)USBHSDCD2_BASE_NS)
  /** Array initializer of USBHSDCD peripheral base addresses */
  #define USBHSDCD_BASE_ADDRS                      { 0u, USBHSDCD1_BASE, USBHSDCD2_BASE }
  /** Array initializer of USBHSDCD peripheral base pointers */
  #define USBHSDCD_BASE_PTRS                       { (USBHSDCD_Type *)0u, USBHSDCD1, USBHSDCD2 }
  /** Array initializer of USBHSDCD peripheral base addresses */
  #define USBHSDCD_BASE_ADDRS_NS                   { 0u, USBHSDCD1_BASE_NS, USBHSDCD2_BASE_NS }
  /** Array initializer of USBHSDCD peripheral base pointers */
  #define USBHSDCD_BASE_PTRS_NS                    { (USBHSDCD_Type *)0u, USBHSDCD1_NS, USBHSDCD2_NS }
#else
  /** Peripheral USBHSDCD1 base address */
  #define USBHSDCD1_BASE                           (0x42CA0800u)
  /** Peripheral USBHSDCD1 base pointer */
  #define USBHSDCD1                                ((USBHSDCD_Type *)USBHSDCD1_BASE)
  /** Peripheral USBHSDCD2 base address */
  #define USBHSDCD2_BASE                           (0x42CB0800u)
  /** Peripheral USBHSDCD2 base pointer */
  #define USBHSDCD2                                ((USBHSDCD_Type *)USBHSDCD2_BASE)
  /** Array initializer of USBHSDCD peripheral base addresses */
  #define USBHSDCD_BASE_ADDRS                      { 0u, USBHSDCD1_BASE, USBHSDCD2_BASE }
  /** Array initializer of USBHSDCD peripheral base pointers */
  #define USBHSDCD_BASE_PTRS                       { (USBHSDCD_Type *)0u, USBHSDCD1, USBHSDCD2 }
#endif
/* Backward compatibility */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
#define USBHSDCD_STACK_BASE_ADDRS               { USBHSDCD1_BASE, USBHSDCD2_BASE }
#define USBHSDCD_STACK_BASE_ADDRS_NS            { USBHSDCD1_BASE_NS, USBHSDCD2_BASE_NS }
#else
#define USBHSDCD_STACK_BASE_ADDRS                { USBHSDCD1_BASE, USBHSDCD2_BASE }
#endif


/*!
 * @}
 */ /* end of group USBHSDCD_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- USDHC Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup USDHC_Peripheral_Access_Layer USDHC Peripheral Access Layer
 * @{
 */

/** USDHC - Register Layout Typedef */
typedef struct {
  __IO uint32_t DS_ADDR;                           /**< DMA System Address, offset: 0x0 */
  __IO uint32_t BLK_ATT;                           /**< Block Attributes, offset: 0x4 */
  __IO uint32_t CMD_ARG;                           /**< Command Argument, offset: 0x8 */
  __IO uint32_t CMD_XFR_TYP;                       /**< Command Transfer Type, offset: 0xC */
  __I  uint32_t CMD_RSP0;                          /**< Command Response0, offset: 0x10 */
  __I  uint32_t CMD_RSP1;                          /**< Command Response1, offset: 0x14 */
  __I  uint32_t CMD_RSP2;                          /**< Command Response2, offset: 0x18 */
  __I  uint32_t CMD_RSP3;                          /**< Command Response3, offset: 0x1C */
  __IO uint32_t DATA_BUFF_ACC_PORT;                /**< Data Buffer Access Port, offset: 0x20 */
  __I  uint32_t PRES_STATE;                        /**< Present State, offset: 0x24 */
  __IO uint32_t PROT_CTRL;                         /**< Protocol Control, offset: 0x28 */
  __IO uint32_t SYS_CTRL;                          /**< System Control, offset: 0x2C */
  __IO uint32_t INT_STATUS;                        /**< Interrupt Status, offset: 0x30 */
  __IO uint32_t INT_STATUS_EN;                     /**< Interrupt Status Enable, offset: 0x34 */
  __IO uint32_t INT_SIGNAL_EN;                     /**< Interrupt Signal Enable, offset: 0x38 */
  __IO uint32_t AUTOCMD12_ERR_STATUS;              /**< Auto CMD12 Error Status, offset: 0x3C */
  __IO uint32_t HOST_CTRL_CAP;                     /**< Host Controller Capabilities, offset: 0x40 */
  __IO uint32_t WTMK_LVL;                          /**< Watermark Level, offset: 0x44 */
  __IO uint32_t MIX_CTRL;                          /**< Mixer Control, offset: 0x48 */
       uint8_t RESERVED_0[4];
  __O  uint32_t FORCE_EVENT;                       /**< Force Event, offset: 0x50 */
  __I  uint32_t ADMA_ERR_STATUS;                   /**< ADMA Error Status, offset: 0x54 */
  __IO uint32_t ADMA_SYS_ADDR;                     /**< ADMA System Address, offset: 0x58 */
       uint8_t RESERVED_1[4];
  __IO uint32_t DLL_CTRL;                          /**< DLL (Delay Line) Control, offset: 0x60 */
  __I  uint32_t DLL_STATUS;                        /**< DLL Status, offset: 0x64 */
  __IO uint32_t CLK_TUNE_CTRL_STATUS;              /**< CLK Tuning Control and Status, offset: 0x68 */
       uint8_t RESERVED_2[4];
  __IO uint32_t STROBE_DLL_CTRL;                   /**< Strobe DLL control, offset: 0x70 */
  __I  uint32_t STROBE_DLL_STATUS;                 /**< Strobe DLL status, offset: 0x74 */
       uint8_t RESERVED_3[72];
  __IO uint32_t VEND_SPEC;                         /**< Vendor Specific Register, offset: 0xC0 */
  __IO uint32_t MMC_BOOT;                          /**< eMMC Boot, offset: 0xC4 */
  __IO uint32_t VEND_SPEC2;                        /**< Vendor Specific 2 Register, offset: 0xC8 */
  __IO uint32_t TUNING_CTRL;                       /**< Tuning Control, offset: 0xCC */
       uint8_t RESERVED_4[48];
  __I  uint32_t CQVER;                             /**< Command Queuing Version, offset: 0x100 */
  __IO uint32_t CQCAP;                             /**< Command Queuing Capabilities, offset: 0x104 */
  __IO uint32_t CQCFG;                             /**< Command Queuing Configuration, offset: 0x108 */
  __IO uint32_t CQCTL;                             /**< Command Queuing Control, offset: 0x10C */
  __IO uint32_t CQIS;                              /**< Command Queuing Interrupt Status, offset: 0x110 */
  __IO uint32_t CQISTE;                            /**< Command Queuing Interrupt Status Enable, offset: 0x114 */
  __IO uint32_t CQISGE;                            /**< Command Queuing Interrupt Signal Enable, offset: 0x118 */
  __IO uint32_t CQIC;                              /**< Command Queuing Interrupt Coalescing, offset: 0x11C */
  __IO uint32_t CQTDLBA;                           /**< Command Queuing Task Descriptor List Base Address, offset: 0x120 */
  __IO uint32_t CQTDLBAU;                          /**< Command Queuing Task Descriptor List Base Address Upper 32 Bits, offset: 0x124 */
  __IO uint32_t CQTDBR;                            /**< Command Queuing Task Doorbell, offset: 0x128 */
  __IO uint32_t CQTCN;                             /**< Command Queuing Task Completion Notification, offset: 0x12C */
  __I  uint32_t CQDQS;                             /**< Command Queuing Device Queue Status, offset: 0x130 */
  __I  uint32_t CQDPT;                             /**< Command Queuing Device Pending Tasks, offset: 0x134 */
  __IO uint32_t CQTCLR;                            /**< Command Queuing Task Clear, offset: 0x138 */
       uint8_t RESERVED_5[4];
  __IO uint32_t CQSSC1;                            /**< Command Queuing Send Status Configuration 1, offset: 0x140 */
  __IO uint32_t CQSSC2;                            /**< Command Queuing Send Status Configuration 2, offset: 0x144 */
  __I  uint32_t CQCRDCT;                           /**< Command Queuing Command Response for Direct-Command Task, offset: 0x148 */
       uint8_t RESERVED_6[4];
  __IO uint32_t CQRMEM;                            /**< Command Queuing Response Mode Error Mask, offset: 0x150 */
  __I  uint32_t CQTERRI;                           /**< Command Queuing Task Error Information, offset: 0x154 */
  __I  uint32_t CQCRI;                             /**< Command Queuing Command Response Index, offset: 0x158 */
  __I  uint32_t CQCRA;                             /**< Command Queuing Command Response Argument, offset: 0x15C */
} USDHC_Type;

/* ----------------------------------------------------------------------------
   -- USDHC Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup USDHC_Register_Masks USDHC Register Masks
 * @{
 */

/*! @name DS_ADDR - DMA System Address */
/*! @{ */

#define USDHC_DS_ADDR_DS_ADDR_MASK               (0xFFFFFFFFU)
#define USDHC_DS_ADDR_DS_ADDR_SHIFT              (0U)
/*! DS_ADDR - System address */
#define USDHC_DS_ADDR_DS_ADDR(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_DS_ADDR_DS_ADDR_SHIFT)) & USDHC_DS_ADDR_DS_ADDR_MASK)
/*! @} */

/*! @name BLK_ATT - Block Attributes */
/*! @{ */

#define USDHC_BLK_ATT_BLKSIZE_MASK               (0x1FFFU)
#define USDHC_BLK_ATT_BLKSIZE_SHIFT              (0U)
/*! BLKSIZE - Transfer block size
 *  0b1000000000000..4096 bytes
 *  0b0100000000000..2048 bytes
 *  0b0001000000000..512 bytes
 *  0b0000111111111..511 bytes
 *  0b0000000000100..4 bytes
 *  0b0000000000011..3 bytes
 *  0b0000000000010..2 bytes
 *  0b0000000000001..1 byte
 *  0b0000000000000..No data transfer
 */
#define USDHC_BLK_ATT_BLKSIZE(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_BLK_ATT_BLKSIZE_SHIFT)) & USDHC_BLK_ATT_BLKSIZE_MASK)

#define USDHC_BLK_ATT_BLKCNT_MASK                (0xFFFF0000U)
#define USDHC_BLK_ATT_BLKCNT_SHIFT               (16U)
/*! BLKCNT - Blocks count for current transfer
 *  0b1111111111111111..65535 blocks
 *  0b0000000000000010..2 blocks
 *  0b0000000000000001..1 block
 *  0b0000000000000000..Stop count
 */
#define USDHC_BLK_ATT_BLKCNT(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_BLK_ATT_BLKCNT_SHIFT)) & USDHC_BLK_ATT_BLKCNT_MASK)
/*! @} */

/*! @name CMD_ARG - Command Argument */
/*! @{ */

#define USDHC_CMD_ARG_CMDARG_MASK                (0xFFFFFFFFU)
#define USDHC_CMD_ARG_CMDARG_SHIFT               (0U)
/*! CMDARG - Command argument */
#define USDHC_CMD_ARG_CMDARG(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_ARG_CMDARG_SHIFT)) & USDHC_CMD_ARG_CMDARG_MASK)
/*! @} */

/*! @name CMD_XFR_TYP - Command Transfer Type */
/*! @{ */

#define USDHC_CMD_XFR_TYP_DMAEN_MASK             (0x1U)
#define USDHC_CMD_XFR_TYP_DMAEN_SHIFT            (0U)
/*! DMAEN - DMAEN
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_DMAEN(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_DMAEN_SHIFT)) & USDHC_CMD_XFR_TYP_DMAEN_MASK)

#define USDHC_CMD_XFR_TYP_BCEN_MASK              (0x2U)
#define USDHC_CMD_XFR_TYP_BCEN_SHIFT             (1U)
/*! BCEN - BCEN
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_BCEN(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_BCEN_SHIFT)) & USDHC_CMD_XFR_TYP_BCEN_MASK)

#define USDHC_CMD_XFR_TYP_AC12EN_MASK            (0x4U)
#define USDHC_CMD_XFR_TYP_AC12EN_SHIFT           (2U)
/*! AC12EN - AC12EN
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_AC12EN(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_AC12EN_SHIFT)) & USDHC_CMD_XFR_TYP_AC12EN_MASK)

#define USDHC_CMD_XFR_TYP_DDR_EN_MASK            (0x8U)
#define USDHC_CMD_XFR_TYP_DDR_EN_SHIFT           (3U)
/*! DDR_EN - DDR_EN
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_DDR_EN(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_DDR_EN_SHIFT)) & USDHC_CMD_XFR_TYP_DDR_EN_MASK)

#define USDHC_CMD_XFR_TYP_DTDSEL_MASK            (0x10U)
#define USDHC_CMD_XFR_TYP_DTDSEL_SHIFT           (4U)
/*! DTDSEL - DTDSEL
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_DTDSEL(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_DTDSEL_SHIFT)) & USDHC_CMD_XFR_TYP_DTDSEL_MASK)

#define USDHC_CMD_XFR_TYP_MSBSEL_MASK            (0x20U)
#define USDHC_CMD_XFR_TYP_MSBSEL_SHIFT           (5U)
/*! MSBSEL - MSBSEL
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_MSBSEL(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_MSBSEL_SHIFT)) & USDHC_CMD_XFR_TYP_MSBSEL_MASK)

#define USDHC_CMD_XFR_TYP_NIBBLE_POS_MASK        (0x40U)
#define USDHC_CMD_XFR_TYP_NIBBLE_POS_SHIFT       (6U)
/*! NIBBLE_POS - NIBBLE_POS
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_NIBBLE_POS(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_NIBBLE_POS_SHIFT)) & USDHC_CMD_XFR_TYP_NIBBLE_POS_MASK)

#define USDHC_CMD_XFR_TYP_AC23EN_MASK            (0x80U)
#define USDHC_CMD_XFR_TYP_AC23EN_SHIFT           (7U)
/*! AC23EN - AC23EN
 *  0b0..Disable
 *  0b1..Enable
 */
#define USDHC_CMD_XFR_TYP_AC23EN(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_AC23EN_SHIFT)) & USDHC_CMD_XFR_TYP_AC23EN_MASK)

#define USDHC_CMD_XFR_TYP_RSPTYP_MASK            (0x30000U)
#define USDHC_CMD_XFR_TYP_RSPTYP_SHIFT           (16U)
/*! RSPTYP - Response type select
 *  0b00..No response
 *  0b01..Response length 136
 *  0b10..Response length 48
 *  0b11..Response length 48, check busy after response
 */
#define USDHC_CMD_XFR_TYP_RSPTYP(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_RSPTYP_SHIFT)) & USDHC_CMD_XFR_TYP_RSPTYP_MASK)

#define USDHC_CMD_XFR_TYP_CCCEN_MASK             (0x80000U)
#define USDHC_CMD_XFR_TYP_CCCEN_SHIFT            (19U)
/*! CCCEN - Command CRC check enable
 *  0b1..Enables command CRC check
 *  0b0..Disables command CRC check
 */
#define USDHC_CMD_XFR_TYP_CCCEN(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_CCCEN_SHIFT)) & USDHC_CMD_XFR_TYP_CCCEN_MASK)

#define USDHC_CMD_XFR_TYP_CICEN_MASK             (0x100000U)
#define USDHC_CMD_XFR_TYP_CICEN_SHIFT            (20U)
/*! CICEN - Command index check enable
 *  0b1..Enables command index check
 *  0b0..Disable command index check
 */
#define USDHC_CMD_XFR_TYP_CICEN(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_CICEN_SHIFT)) & USDHC_CMD_XFR_TYP_CICEN_MASK)

#define USDHC_CMD_XFR_TYP_DPSEL_MASK             (0x200000U)
#define USDHC_CMD_XFR_TYP_DPSEL_SHIFT            (21U)
/*! DPSEL - Data present select
 *  0b1..Data present
 *  0b0..No data present
 */
#define USDHC_CMD_XFR_TYP_DPSEL(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_DPSEL_SHIFT)) & USDHC_CMD_XFR_TYP_DPSEL_MASK)

#define USDHC_CMD_XFR_TYP_CMDTYP_MASK            (0xC00000U)
#define USDHC_CMD_XFR_TYP_CMDTYP_SHIFT           (22U)
/*! CMDTYP - Command type
 *  0b11..Abort CMD12, CMD52 for writing I/O Abort in CCCR
 *  0b10..Resume CMD52 for writing function select in CCCR
 *  0b01..Suspend CMD52 for writing bus suspend in CCCR
 *  0b00..Normal other commands
 */
#define USDHC_CMD_XFR_TYP_CMDTYP(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_CMDTYP_SHIFT)) & USDHC_CMD_XFR_TYP_CMDTYP_MASK)

#define USDHC_CMD_XFR_TYP_CMDINX_MASK            (0x3F000000U)
#define USDHC_CMD_XFR_TYP_CMDINX_SHIFT           (24U)
/*! CMDINX - Command index */
#define USDHC_CMD_XFR_TYP_CMDINX(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_XFR_TYP_CMDINX_SHIFT)) & USDHC_CMD_XFR_TYP_CMDINX_MASK)
/*! @} */

/*! @name CMD_RSP0 - Command Response0 */
/*! @{ */

#define USDHC_CMD_RSP0_CMDRSP0_MASK              (0xFFFFFFFFU)
#define USDHC_CMD_RSP0_CMDRSP0_SHIFT             (0U)
/*! CMDRSP0 - Command response 0 */
#define USDHC_CMD_RSP0_CMDRSP0(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_RSP0_CMDRSP0_SHIFT)) & USDHC_CMD_RSP0_CMDRSP0_MASK)
/*! @} */

/*! @name CMD_RSP1 - Command Response1 */
/*! @{ */

#define USDHC_CMD_RSP1_CMDRSP1_MASK              (0xFFFFFFFFU)
#define USDHC_CMD_RSP1_CMDRSP1_SHIFT             (0U)
/*! CMDRSP1 - Command response 1 */
#define USDHC_CMD_RSP1_CMDRSP1(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_RSP1_CMDRSP1_SHIFT)) & USDHC_CMD_RSP1_CMDRSP1_MASK)
/*! @} */

/*! @name CMD_RSP2 - Command Response2 */
/*! @{ */

#define USDHC_CMD_RSP2_CMDRSP2_MASK              (0xFFFFFFFFU)
#define USDHC_CMD_RSP2_CMDRSP2_SHIFT             (0U)
/*! CMDRSP2 - Command response 2 */
#define USDHC_CMD_RSP2_CMDRSP2(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_RSP2_CMDRSP2_SHIFT)) & USDHC_CMD_RSP2_CMDRSP2_MASK)
/*! @} */

/*! @name CMD_RSP3 - Command Response3 */
/*! @{ */

#define USDHC_CMD_RSP3_CMDRSP3_MASK              (0xFFFFFFFFU)
#define USDHC_CMD_RSP3_CMDRSP3_SHIFT             (0U)
/*! CMDRSP3 - Command response 3 */
#define USDHC_CMD_RSP3_CMDRSP3(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_CMD_RSP3_CMDRSP3_SHIFT)) & USDHC_CMD_RSP3_CMDRSP3_MASK)
/*! @} */

/*! @name DATA_BUFF_ACC_PORT - Data Buffer Access Port */
/*! @{ */

#define USDHC_DATA_BUFF_ACC_PORT_DATCONT_MASK    (0xFFFFFFFFU)
#define USDHC_DATA_BUFF_ACC_PORT_DATCONT_SHIFT   (0U)
/*! DATCONT - Data content */
#define USDHC_DATA_BUFF_ACC_PORT_DATCONT(x)      (((uint32_t)(((uint32_t)(x)) << USDHC_DATA_BUFF_ACC_PORT_DATCONT_SHIFT)) & USDHC_DATA_BUFF_ACC_PORT_DATCONT_MASK)
/*! @} */

/*! @name PRES_STATE - Present State */
/*! @{ */

#define USDHC_PRES_STATE_CIHB_MASK               (0x1U)
#define USDHC_PRES_STATE_CIHB_SHIFT              (0U)
/*! CIHB - Command inhibit (CMD)
 *  0b1..Cannot issue command
 *  0b0..Can issue command using only CMD line
 */
#define USDHC_PRES_STATE_CIHB(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_CIHB_SHIFT)) & USDHC_PRES_STATE_CIHB_MASK)

#define USDHC_PRES_STATE_CDIHB_MASK              (0x2U)
#define USDHC_PRES_STATE_CDIHB_SHIFT             (1U)
/*! CDIHB - Command Inhibit Data (DATA)
 *  0b1..Cannot issue command that uses the DATA line
 *  0b0..Can issue command that uses the DATA line
 */
#define USDHC_PRES_STATE_CDIHB(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_CDIHB_SHIFT)) & USDHC_PRES_STATE_CDIHB_MASK)

#define USDHC_PRES_STATE_DLA_MASK                (0x4U)
#define USDHC_PRES_STATE_DLA_SHIFT               (2U)
/*! DLA - Data line active
 *  0b1..DATA line active
 *  0b0..DATA line inactive
 */
#define USDHC_PRES_STATE_DLA(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_DLA_SHIFT)) & USDHC_PRES_STATE_DLA_MASK)

#define USDHC_PRES_STATE_SDSTB_MASK              (0x8U)
#define USDHC_PRES_STATE_SDSTB_SHIFT             (3U)
/*! SDSTB - SD clock stable
 *  0b1..Clock is stable.
 *  0b0..Clock is changing frequency and not stable.
 */
#define USDHC_PRES_STATE_SDSTB(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_SDSTB_SHIFT)) & USDHC_PRES_STATE_SDSTB_MASK)

#define USDHC_PRES_STATE_WTA_MASK                (0x100U)
#define USDHC_PRES_STATE_WTA_SHIFT               (8U)
/*! WTA - Write transfer active
 *  0b1..Transferring data
 *  0b0..No valid data
 */
#define USDHC_PRES_STATE_WTA(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_WTA_SHIFT)) & USDHC_PRES_STATE_WTA_MASK)

#define USDHC_PRES_STATE_RTA_MASK                (0x200U)
#define USDHC_PRES_STATE_RTA_SHIFT               (9U)
/*! RTA - Read transfer active
 *  0b1..Transferring data
 *  0b0..No valid data
 */
#define USDHC_PRES_STATE_RTA(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_RTA_SHIFT)) & USDHC_PRES_STATE_RTA_MASK)

#define USDHC_PRES_STATE_BWEN_MASK               (0x400U)
#define USDHC_PRES_STATE_BWEN_SHIFT              (10U)
/*! BWEN - Buffer write enable
 *  0b1..Write enable
 *  0b0..Write disable
 */
#define USDHC_PRES_STATE_BWEN(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_BWEN_SHIFT)) & USDHC_PRES_STATE_BWEN_MASK)

#define USDHC_PRES_STATE_BREN_MASK               (0x800U)
#define USDHC_PRES_STATE_BREN_SHIFT              (11U)
/*! BREN - Buffer read enable
 *  0b1..Read enable
 *  0b0..Read disable
 */
#define USDHC_PRES_STATE_BREN(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_BREN_SHIFT)) & USDHC_PRES_STATE_BREN_MASK)

#define USDHC_PRES_STATE_RTR_MASK                (0x1000U)
#define USDHC_PRES_STATE_RTR_SHIFT               (12U)
/*! RTR - Re-Tuning Request (only for SD3.0 SDR104 mode, and eMMC HS200 mode)
 *  0b1..Sampling clock needs re-tuning
 *  0b0..Fixed or well tuned sampling clock
 */
#define USDHC_PRES_STATE_RTR(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_RTR_SHIFT)) & USDHC_PRES_STATE_RTR_MASK)

#define USDHC_PRES_STATE_TSCD_MASK               (0x8000U)
#define USDHC_PRES_STATE_TSCD_SHIFT              (15U)
/*! TSCD - Tap select change done
 *  0b1..Delay cell select change is finished.
 *  0b0..Delay cell select change is not finished.
 */
#define USDHC_PRES_STATE_TSCD(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_TSCD_SHIFT)) & USDHC_PRES_STATE_TSCD_MASK)

#define USDHC_PRES_STATE_CINST_MASK              (0x10000U)
#define USDHC_PRES_STATE_CINST_SHIFT             (16U)
/*! CINST - Card inserted
 *  0b1..Card inserted
 *  0b0..Power on reset or no card
 */
#define USDHC_PRES_STATE_CINST(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_CINST_SHIFT)) & USDHC_PRES_STATE_CINST_MASK)

#define USDHC_PRES_STATE_CDPL_MASK               (0x40000U)
#define USDHC_PRES_STATE_CDPL_SHIFT              (18U)
/*! CDPL - Card detect pin level
 *  0b1..Card present (CD_B = 0)
 *  0b0..No card present (CD_B = 1)
 */
#define USDHC_PRES_STATE_CDPL(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_CDPL_SHIFT)) & USDHC_PRES_STATE_CDPL_MASK)

#define USDHC_PRES_STATE_WPSPL_MASK              (0x80000U)
#define USDHC_PRES_STATE_WPSPL_SHIFT             (19U)
/*! WPSPL - Write protect switch pin level
 *  0b1..Write enabled (WP = 0)
 *  0b0..Write protected (WP = 1)
 */
#define USDHC_PRES_STATE_WPSPL(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_WPSPL_SHIFT)) & USDHC_PRES_STATE_WPSPL_MASK)

#define USDHC_PRES_STATE_CLSL_MASK               (0x800000U)
#define USDHC_PRES_STATE_CLSL_SHIFT              (23U)
/*! CLSL - CMD line signal level */
#define USDHC_PRES_STATE_CLSL(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_CLSL_SHIFT)) & USDHC_PRES_STATE_CLSL_MASK)

#define USDHC_PRES_STATE_DLSL_MASK               (0xFF000000U)
#define USDHC_PRES_STATE_DLSL_SHIFT              (24U)
/*! DLSL - DATA[7:0] line signal level
 *  0b00000111..Data 7 line signal level
 *  0b00000110..Data 6 line signal level
 *  0b00000101..Data 5 line signal level
 *  0b00000100..Data 4 line signal level
 *  0b00000011..Data 3 line signal level
 *  0b00000010..Data 2 line signal level
 *  0b00000001..Data 1 line signal level
 *  0b00000000..Data 0 line signal level
 */
#define USDHC_PRES_STATE_DLSL(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PRES_STATE_DLSL_SHIFT)) & USDHC_PRES_STATE_DLSL_MASK)
/*! @} */

/*! @name PROT_CTRL - Protocol Control */
/*! @{ */

#define USDHC_PROT_CTRL_DTW_MASK                 (0x6U)
#define USDHC_PROT_CTRL_DTW_SHIFT                (1U)
/*! DTW - Data transfer width
 *  0b10..8-bit mode
 *  0b01..4-bit mode
 *  0b00..1-bit mode
 *  0b11..Reserved
 */
#define USDHC_PROT_CTRL_DTW(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_DTW_SHIFT)) & USDHC_PROT_CTRL_DTW_MASK)

#define USDHC_PROT_CTRL_D3CD_MASK                (0x8U)
#define USDHC_PROT_CTRL_D3CD_SHIFT               (3U)
/*! D3CD - DATA3 as card detection pin
 *  0b1..DATA3 as card detection pin
 *  0b0..DATA3 does not monitor card insertion
 */
#define USDHC_PROT_CTRL_D3CD(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_D3CD_SHIFT)) & USDHC_PROT_CTRL_D3CD_MASK)

#define USDHC_PROT_CTRL_EMODE_MASK               (0x30U)
#define USDHC_PROT_CTRL_EMODE_SHIFT              (4U)
/*! EMODE - Endian mode
 *  0b00..Big endian mode
 *  0b01..Half word big endian mode
 *  0b10..Little endian mode
 *  0b11..Reserved
 */
#define USDHC_PROT_CTRL_EMODE(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_EMODE_SHIFT)) & USDHC_PROT_CTRL_EMODE_MASK)

#define USDHC_PROT_CTRL_DMASEL_MASK              (0x300U)
#define USDHC_PROT_CTRL_DMASEL_SHIFT             (8U)
/*! DMASEL - DMA select
 *  0b00..No DMA or simple DMA is selected.
 *  0b01..ADMA1 is selected.
 *  0b10..ADMA2 is selected.
 *  0b11..Reserved
 */
#define USDHC_PROT_CTRL_DMASEL(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_DMASEL_SHIFT)) & USDHC_PROT_CTRL_DMASEL_MASK)

#define USDHC_PROT_CTRL_SABGREQ_MASK             (0x10000U)
#define USDHC_PROT_CTRL_SABGREQ_SHIFT            (16U)
/*! SABGREQ - Stop at block gap request
 *  0b1..Stop
 *  0b0..Transfer
 */
#define USDHC_PROT_CTRL_SABGREQ(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_SABGREQ_SHIFT)) & USDHC_PROT_CTRL_SABGREQ_MASK)

#define USDHC_PROT_CTRL_CREQ_MASK                (0x20000U)
#define USDHC_PROT_CTRL_CREQ_SHIFT               (17U)
/*! CREQ - Continue request
 *  0b1..Restart
 *  0b0..No effect
 */
#define USDHC_PROT_CTRL_CREQ(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_CREQ_SHIFT)) & USDHC_PROT_CTRL_CREQ_MASK)

#define USDHC_PROT_CTRL_RWCTL_MASK               (0x40000U)
#define USDHC_PROT_CTRL_RWCTL_SHIFT              (18U)
/*! RWCTL - Read wait control
 *  0b1..Enables read wait control and assert read wait without stopping SD clock at block gap when SABGREQ field is set
 *  0b0..Disables read wait control and stop SD clock at block gap when SABGREQ field is set
 */
#define USDHC_PROT_CTRL_RWCTL(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_RWCTL_SHIFT)) & USDHC_PROT_CTRL_RWCTL_MASK)

#define USDHC_PROT_CTRL_IABG_MASK                (0x80000U)
#define USDHC_PROT_CTRL_IABG_SHIFT               (19U)
/*! IABG - Interrupt at block gap
 *  0b1..Enables interrupt at block gap
 *  0b0..Disables interrupt at block gap
 */
#define USDHC_PROT_CTRL_IABG(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_IABG_SHIFT)) & USDHC_PROT_CTRL_IABG_MASK)

#define USDHC_PROT_CTRL_RD_DONE_NO_8CLK_MASK     (0x100000U)
#define USDHC_PROT_CTRL_RD_DONE_NO_8CLK_SHIFT    (20U)
/*! RD_DONE_NO_8CLK - Read performed number 8 clock */
#define USDHC_PROT_CTRL_RD_DONE_NO_8CLK(x)       (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_RD_DONE_NO_8CLK_SHIFT)) & USDHC_PROT_CTRL_RD_DONE_NO_8CLK_MASK)

#define USDHC_PROT_CTRL_WECINT_MASK              (0x1000000U)
#define USDHC_PROT_CTRL_WECINT_SHIFT             (24U)
/*! WECINT - Wakeup event enable on card interrupt
 *  0b1..Enables wakeup event enable on card interrupt
 *  0b0..Disables wakeup event enable on card interrupt
 */
#define USDHC_PROT_CTRL_WECINT(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_WECINT_SHIFT)) & USDHC_PROT_CTRL_WECINT_MASK)

#define USDHC_PROT_CTRL_WECINS_MASK              (0x2000000U)
#define USDHC_PROT_CTRL_WECINS_SHIFT             (25U)
/*! WECINS - Wakeup event enable on SD card insertion
 *  0b1..Enable wakeup event enable on SD card insertion
 *  0b0..Disable wakeup event enable on SD card insertion
 */
#define USDHC_PROT_CTRL_WECINS(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_WECINS_SHIFT)) & USDHC_PROT_CTRL_WECINS_MASK)

#define USDHC_PROT_CTRL_WECRM_MASK               (0x4000000U)
#define USDHC_PROT_CTRL_WECRM_SHIFT              (26U)
/*! WECRM - Wakeup event enable on SD card removal
 *  0b1..Enables wakeup event enable on SD card removal
 *  0b0..Disables wakeup event enable on SD card removal
 */
#define USDHC_PROT_CTRL_WECRM(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_WECRM_SHIFT)) & USDHC_PROT_CTRL_WECRM_MASK)

#define USDHC_PROT_CTRL_NON_EXACT_BLK_RD_MASK    (0x40000000U)
#define USDHC_PROT_CTRL_NON_EXACT_BLK_RD_SHIFT   (30U)
/*! NON_EXACT_BLK_RD - Non-exact block read
 *  0b1..The block read is non-exact block read. Host driver needs to issue abort command to terminate this multi-block read.
 *  0b0..The block read is exact block read. Host driver does not need to issue abort command to terminate this multi-block read.
 */
#define USDHC_PROT_CTRL_NON_EXACT_BLK_RD(x)      (((uint32_t)(((uint32_t)(x)) << USDHC_PROT_CTRL_NON_EXACT_BLK_RD_SHIFT)) & USDHC_PROT_CTRL_NON_EXACT_BLK_RD_MASK)
/*! @} */

/*! @name SYS_CTRL - System Control */
/*! @{ */

#define USDHC_SYS_CTRL_DVS_MASK                  (0xF0U)
#define USDHC_SYS_CTRL_DVS_SHIFT                 (4U)
/*! DVS - Divisor
 *  0b0000..Divide-by-1
 *  0b0001..Divide-by-2
 *  0b1110..Divide-by-15
 *  0b1111..Divide-by-16
 */
#define USDHC_SYS_CTRL_DVS(x)                    (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_DVS_SHIFT)) & USDHC_SYS_CTRL_DVS_MASK)

#define USDHC_SYS_CTRL_SDCLKFS_MASK              (0xFF00U)
#define USDHC_SYS_CTRL_SDCLKFS_SHIFT             (8U)
/*! SDCLKFS - SDCLK frequency select */
#define USDHC_SYS_CTRL_SDCLKFS(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_SDCLKFS_SHIFT)) & USDHC_SYS_CTRL_SDCLKFS_MASK)

#define USDHC_SYS_CTRL_DTOCV_MASK                (0xF0000U)
#define USDHC_SYS_CTRL_DTOCV_SHIFT               (16U)
/*! DTOCV - Data timeout counter value
 *  0b1111..SDCLK x 2 31, recommend to use for HS400 mode
 *  0b1110..SDCLK x 2 30, recommend to use for HS200 and SDR104 mode
 *  0b1101..SDCLK x 2 29, recommend to use for supported speed modes except HS200, HS400, SDR104 mode
 *  0b0011..SDCLK x 2 19
 *  0b0010..SDCLK x 2 18
 *  0b0001..SDCLK x 2 33
 *  0b0000..SDCLK x 2 32
 */
#define USDHC_SYS_CTRL_DTOCV(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_DTOCV_SHIFT)) & USDHC_SYS_CTRL_DTOCV_MASK)

#define USDHC_SYS_CTRL_RST_FIFO_MASK             (0x400000U)
#define USDHC_SYS_CTRL_RST_FIFO_SHIFT            (22U)
/*! RST_FIFO - Reset the async FIFO */
#define USDHC_SYS_CTRL_RST_FIFO(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_RST_FIFO_SHIFT)) & USDHC_SYS_CTRL_RST_FIFO_MASK)

#define USDHC_SYS_CTRL_IPP_RST_N_MASK            (0x800000U)
#define USDHC_SYS_CTRL_IPP_RST_N_SHIFT           (23U)
/*! IPP_RST_N - Hardware reset */
#define USDHC_SYS_CTRL_IPP_RST_N(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_IPP_RST_N_SHIFT)) & USDHC_SYS_CTRL_IPP_RST_N_MASK)

#define USDHC_SYS_CTRL_RSTA_MASK                 (0x1000000U)
#define USDHC_SYS_CTRL_RSTA_SHIFT                (24U)
/*! RSTA - Software reset for all
 *  0b1..Reset
 *  0b0..No reset
 */
#define USDHC_SYS_CTRL_RSTA(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_RSTA_SHIFT)) & USDHC_SYS_CTRL_RSTA_MASK)

#define USDHC_SYS_CTRL_RSTC_MASK                 (0x2000000U)
#define USDHC_SYS_CTRL_RSTC_SHIFT                (25U)
/*! RSTC - Software reset for CMD line
 *  0b1..Reset
 *  0b0..No reset
 */
#define USDHC_SYS_CTRL_RSTC(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_RSTC_SHIFT)) & USDHC_SYS_CTRL_RSTC_MASK)

#define USDHC_SYS_CTRL_RSTD_MASK                 (0x4000000U)
#define USDHC_SYS_CTRL_RSTD_SHIFT                (26U)
/*! RSTD - Software reset for data line
 *  0b1..Reset
 *  0b0..No reset
 */
#define USDHC_SYS_CTRL_RSTD(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_RSTD_SHIFT)) & USDHC_SYS_CTRL_RSTD_MASK)

#define USDHC_SYS_CTRL_INITA_MASK                (0x8000000U)
#define USDHC_SYS_CTRL_INITA_SHIFT               (27U)
/*! INITA - Initialization active */
#define USDHC_SYS_CTRL_INITA(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_INITA_SHIFT)) & USDHC_SYS_CTRL_INITA_MASK)

#define USDHC_SYS_CTRL_RSTT_MASK                 (0x10000000U)
#define USDHC_SYS_CTRL_RSTT_SHIFT                (28U)
/*! RSTT - Reset tuning */
#define USDHC_SYS_CTRL_RSTT(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_SYS_CTRL_RSTT_SHIFT)) & USDHC_SYS_CTRL_RSTT_MASK)
/*! @} */

/*! @name INT_STATUS - Interrupt Status */
/*! @{ */

#define USDHC_INT_STATUS_CC_MASK                 (0x1U)
#define USDHC_INT_STATUS_CC_SHIFT                (0U)
/*! CC - Command complete
 *  0b1..Command complete
 *  0b0..Command not complete
 */
#define USDHC_INT_STATUS_CC(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CC_SHIFT)) & USDHC_INT_STATUS_CC_MASK)

#define USDHC_INT_STATUS_TC_MASK                 (0x2U)
#define USDHC_INT_STATUS_TC_SHIFT                (1U)
/*! TC - Transfer complete
 *  0b1..Transfer complete
 *  0b0..Transfer does not complete
 */
#define USDHC_INT_STATUS_TC(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_TC_SHIFT)) & USDHC_INT_STATUS_TC_MASK)

#define USDHC_INT_STATUS_BGE_MASK                (0x4U)
#define USDHC_INT_STATUS_BGE_SHIFT               (2U)
/*! BGE - Block gap event
 *  0b1..Transaction stopped at block gap
 *  0b0..No block gap event
 */
#define USDHC_INT_STATUS_BGE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_BGE_SHIFT)) & USDHC_INT_STATUS_BGE_MASK)

#define USDHC_INT_STATUS_DINT_MASK               (0x8U)
#define USDHC_INT_STATUS_DINT_SHIFT              (3U)
/*! DINT - DMA interrupt
 *  0b1..DMA interrupt is generated.
 *  0b0..No DMA interrupt
 */
#define USDHC_INT_STATUS_DINT(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_DINT_SHIFT)) & USDHC_INT_STATUS_DINT_MASK)

#define USDHC_INT_STATUS_BWR_MASK                (0x10U)
#define USDHC_INT_STATUS_BWR_SHIFT               (4U)
/*! BWR - Buffer write ready
 *  0b1..Ready to write buffer
 *  0b0..Not ready to write buffer
 */
#define USDHC_INT_STATUS_BWR(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_BWR_SHIFT)) & USDHC_INT_STATUS_BWR_MASK)

#define USDHC_INT_STATUS_BRR_MASK                (0x20U)
#define USDHC_INT_STATUS_BRR_SHIFT               (5U)
/*! BRR - Buffer read ready
 *  0b1..Ready to read buffer
 *  0b0..Not ready to read buffer
 */
#define USDHC_INT_STATUS_BRR(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_BRR_SHIFT)) & USDHC_INT_STATUS_BRR_MASK)

#define USDHC_INT_STATUS_CINS_MASK               (0x40U)
#define USDHC_INT_STATUS_CINS_SHIFT              (6U)
/*! CINS - Card insertion
 *  0b1..Card inserted
 *  0b0..Card state unstable or removed
 */
#define USDHC_INT_STATUS_CINS(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CINS_SHIFT)) & USDHC_INT_STATUS_CINS_MASK)

#define USDHC_INT_STATUS_CRM_MASK                (0x80U)
#define USDHC_INT_STATUS_CRM_SHIFT               (7U)
/*! CRM - Card removal
 *  0b1..Card removed
 *  0b0..Card state unstable or inserted
 */
#define USDHC_INT_STATUS_CRM(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CRM_SHIFT)) & USDHC_INT_STATUS_CRM_MASK)

#define USDHC_INT_STATUS_CINT_MASK               (0x100U)
#define USDHC_INT_STATUS_CINT_SHIFT              (8U)
/*! CINT - Card interrupt
 *  0b1..Generate card interrupt
 *  0b0..No card interrupt
 */
#define USDHC_INT_STATUS_CINT(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CINT_SHIFT)) & USDHC_INT_STATUS_CINT_MASK)

#define USDHC_INT_STATUS_RTE_MASK                (0x1000U)
#define USDHC_INT_STATUS_RTE_SHIFT               (12U)
/*! RTE - Re-tuning event: (only for SD3.0 SDR104 mode and eMMC HS200 mode)
 *  0b1..Re-tuning should be performed.
 *  0b0..Re-tuning is not required.
 */
#define USDHC_INT_STATUS_RTE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_RTE_SHIFT)) & USDHC_INT_STATUS_RTE_MASK)

#define USDHC_INT_STATUS_TP_MASK                 (0x2000U)
#define USDHC_INT_STATUS_TP_SHIFT                (13U)
/*! TP - Tuning pass:(only for SD3.0 SDR104 mode and eMMC HS200 mode) */
#define USDHC_INT_STATUS_TP(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_TP_SHIFT)) & USDHC_INT_STATUS_TP_MASK)

#define USDHC_INT_STATUS_CQI_MASK                (0x4000U)
#define USDHC_INT_STATUS_CQI_SHIFT               (14U)
/*! CQI - Command queuing interrupt */
#define USDHC_INT_STATUS_CQI(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CQI_SHIFT)) & USDHC_INT_STATUS_CQI_MASK)

#define USDHC_INT_STATUS_ERR_INT_STATUS_MASK     (0x8000U)
#define USDHC_INT_STATUS_ERR_INT_STATUS_SHIFT    (15U)
/*! ERR_INT_STATUS - Error Interrupt Status */
#define USDHC_INT_STATUS_ERR_INT_STATUS(x)       (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_ERR_INT_STATUS_SHIFT)) & USDHC_INT_STATUS_ERR_INT_STATUS_MASK)

#define USDHC_INT_STATUS_CTOE_MASK               (0x10000U)
#define USDHC_INT_STATUS_CTOE_SHIFT              (16U)
/*! CTOE - Command timeout error
 *  0b1..Time out
 *  0b0..No error
 */
#define USDHC_INT_STATUS_CTOE(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CTOE_SHIFT)) & USDHC_INT_STATUS_CTOE_MASK)

#define USDHC_INT_STATUS_CCE_MASK                (0x20000U)
#define USDHC_INT_STATUS_CCE_SHIFT               (17U)
/*! CCE - Command CRC error
 *  0b1..CRC error generated
 *  0b0..No error
 */
#define USDHC_INT_STATUS_CCE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CCE_SHIFT)) & USDHC_INT_STATUS_CCE_MASK)

#define USDHC_INT_STATUS_CEBE_MASK               (0x40000U)
#define USDHC_INT_STATUS_CEBE_SHIFT              (18U)
/*! CEBE - Command end bit error
 *  0b1..End bit error generated
 *  0b0..No error
 */
#define USDHC_INT_STATUS_CEBE(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CEBE_SHIFT)) & USDHC_INT_STATUS_CEBE_MASK)

#define USDHC_INT_STATUS_CIE_MASK                (0x80000U)
#define USDHC_INT_STATUS_CIE_SHIFT               (19U)
/*! CIE - Command index error
 *  0b1..Error
 *  0b0..No error
 */
#define USDHC_INT_STATUS_CIE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_CIE_SHIFT)) & USDHC_INT_STATUS_CIE_MASK)

#define USDHC_INT_STATUS_DTOE_MASK               (0x100000U)
#define USDHC_INT_STATUS_DTOE_SHIFT              (20U)
/*! DTOE - Data timeout error
 *  0b1..Time out
 *  0b0..No error
 */
#define USDHC_INT_STATUS_DTOE(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_DTOE_SHIFT)) & USDHC_INT_STATUS_DTOE_MASK)

#define USDHC_INT_STATUS_DCE_MASK                (0x200000U)
#define USDHC_INT_STATUS_DCE_SHIFT               (21U)
/*! DCE - Data CRC error
 *  0b1..Error
 *  0b0..No error
 */
#define USDHC_INT_STATUS_DCE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_DCE_SHIFT)) & USDHC_INT_STATUS_DCE_MASK)

#define USDHC_INT_STATUS_DEBE_MASK               (0x400000U)
#define USDHC_INT_STATUS_DEBE_SHIFT              (22U)
/*! DEBE - Data end bit error
 *  0b1..Error
 *  0b0..No error
 */
#define USDHC_INT_STATUS_DEBE(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_DEBE_SHIFT)) & USDHC_INT_STATUS_DEBE_MASK)

#define USDHC_INT_STATUS_AC12E_MASK              (0x1000000U)
#define USDHC_INT_STATUS_AC12E_SHIFT             (24U)
/*! AC12E - Auto CMD12 error
 *  0b1..Error
 *  0b0..No error
 */
#define USDHC_INT_STATUS_AC12E(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_AC12E_SHIFT)) & USDHC_INT_STATUS_AC12E_MASK)

#define USDHC_INT_STATUS_TNE_MASK                (0x4000000U)
#define USDHC_INT_STATUS_TNE_SHIFT               (26U)
/*! TNE - Tuning error: (only for SD3.0 SDR104 mode and eMMC HS200 mode) */
#define USDHC_INT_STATUS_TNE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_TNE_SHIFT)) & USDHC_INT_STATUS_TNE_MASK)

#define USDHC_INT_STATUS_DMAE_MASK               (0x10000000U)
#define USDHC_INT_STATUS_DMAE_SHIFT              (28U)
/*! DMAE - DMA error
 *  0b1..Error
 *  0b0..No error
 */
#define USDHC_INT_STATUS_DMAE(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_DMAE_SHIFT)) & USDHC_INT_STATUS_DMAE_MASK)
/*! @} */

/*! @name INT_STATUS_EN - Interrupt Status Enable */
/*! @{ */

#define USDHC_INT_STATUS_EN_CCSEN_MASK           (0x1U)
#define USDHC_INT_STATUS_EN_CCSEN_SHIFT          (0U)
/*! CCSEN - Command complete status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CCSEN(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CCSEN_SHIFT)) & USDHC_INT_STATUS_EN_CCSEN_MASK)

#define USDHC_INT_STATUS_EN_TCSEN_MASK           (0x2U)
#define USDHC_INT_STATUS_EN_TCSEN_SHIFT          (1U)
/*! TCSEN - Transfer complete status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_TCSEN(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_TCSEN_SHIFT)) & USDHC_INT_STATUS_EN_TCSEN_MASK)

#define USDHC_INT_STATUS_EN_BGESEN_MASK          (0x4U)
#define USDHC_INT_STATUS_EN_BGESEN_SHIFT         (2U)
/*! BGESEN - Block gap event status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_BGESEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_BGESEN_SHIFT)) & USDHC_INT_STATUS_EN_BGESEN_MASK)

#define USDHC_INT_STATUS_EN_DINTSEN_MASK         (0x8U)
#define USDHC_INT_STATUS_EN_DINTSEN_SHIFT        (3U)
/*! DINTSEN - DMA interrupt status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_DINTSEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_DINTSEN_SHIFT)) & USDHC_INT_STATUS_EN_DINTSEN_MASK)

#define USDHC_INT_STATUS_EN_BWRSEN_MASK          (0x10U)
#define USDHC_INT_STATUS_EN_BWRSEN_SHIFT         (4U)
/*! BWRSEN - Buffer write ready status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_BWRSEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_BWRSEN_SHIFT)) & USDHC_INT_STATUS_EN_BWRSEN_MASK)

#define USDHC_INT_STATUS_EN_BRRSEN_MASK          (0x20U)
#define USDHC_INT_STATUS_EN_BRRSEN_SHIFT         (5U)
/*! BRRSEN - Buffer read ready status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_BRRSEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_BRRSEN_SHIFT)) & USDHC_INT_STATUS_EN_BRRSEN_MASK)

#define USDHC_INT_STATUS_EN_CINSSEN_MASK         (0x40U)
#define USDHC_INT_STATUS_EN_CINSSEN_SHIFT        (6U)
/*! CINSSEN - Card insertion status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CINSSEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CINSSEN_SHIFT)) & USDHC_INT_STATUS_EN_CINSSEN_MASK)

#define USDHC_INT_STATUS_EN_CRMSEN_MASK          (0x80U)
#define USDHC_INT_STATUS_EN_CRMSEN_SHIFT         (7U)
/*! CRMSEN - Card removal status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CRMSEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CRMSEN_SHIFT)) & USDHC_INT_STATUS_EN_CRMSEN_MASK)

#define USDHC_INT_STATUS_EN_CINTSEN_MASK         (0x100U)
#define USDHC_INT_STATUS_EN_CINTSEN_SHIFT        (8U)
/*! CINTSEN - Card interrupt status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CINTSEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CINTSEN_SHIFT)) & USDHC_INT_STATUS_EN_CINTSEN_MASK)

#define USDHC_INT_STATUS_EN_RTESEN_MASK          (0x1000U)
#define USDHC_INT_STATUS_EN_RTESEN_SHIFT         (12U)
/*! RTESEN - Re-tuning event status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_RTESEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_RTESEN_SHIFT)) & USDHC_INT_STATUS_EN_RTESEN_MASK)

#define USDHC_INT_STATUS_EN_TPSEN_MASK           (0x2000U)
#define USDHC_INT_STATUS_EN_TPSEN_SHIFT          (13U)
/*! TPSEN - Tuning pass status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_TPSEN(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_TPSEN_SHIFT)) & USDHC_INT_STATUS_EN_TPSEN_MASK)

#define USDHC_INT_STATUS_EN_CQISEN_MASK          (0x4000U)
#define USDHC_INT_STATUS_EN_CQISEN_SHIFT         (14U)
/*! CQISEN - Command queuing status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CQISEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CQISEN_SHIFT)) & USDHC_INT_STATUS_EN_CQISEN_MASK)

#define USDHC_INT_STATUS_EN_CTOESEN_MASK         (0x10000U)
#define USDHC_INT_STATUS_EN_CTOESEN_SHIFT        (16U)
/*! CTOESEN - Command timeout error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CTOESEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CTOESEN_SHIFT)) & USDHC_INT_STATUS_EN_CTOESEN_MASK)

#define USDHC_INT_STATUS_EN_CCESEN_MASK          (0x20000U)
#define USDHC_INT_STATUS_EN_CCESEN_SHIFT         (17U)
/*! CCESEN - Command CRC error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CCESEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CCESEN_SHIFT)) & USDHC_INT_STATUS_EN_CCESEN_MASK)

#define USDHC_INT_STATUS_EN_CEBESEN_MASK         (0x40000U)
#define USDHC_INT_STATUS_EN_CEBESEN_SHIFT        (18U)
/*! CEBESEN - Command end bit error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CEBESEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CEBESEN_SHIFT)) & USDHC_INT_STATUS_EN_CEBESEN_MASK)

#define USDHC_INT_STATUS_EN_CIESEN_MASK          (0x80000U)
#define USDHC_INT_STATUS_EN_CIESEN_SHIFT         (19U)
/*! CIESEN - Command index error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_CIESEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_CIESEN_SHIFT)) & USDHC_INT_STATUS_EN_CIESEN_MASK)

#define USDHC_INT_STATUS_EN_DTOESEN_MASK         (0x100000U)
#define USDHC_INT_STATUS_EN_DTOESEN_SHIFT        (20U)
/*! DTOESEN - Data timeout error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_DTOESEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_DTOESEN_SHIFT)) & USDHC_INT_STATUS_EN_DTOESEN_MASK)

#define USDHC_INT_STATUS_EN_DCESEN_MASK          (0x200000U)
#define USDHC_INT_STATUS_EN_DCESEN_SHIFT         (21U)
/*! DCESEN - Data CRC error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_DCESEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_DCESEN_SHIFT)) & USDHC_INT_STATUS_EN_DCESEN_MASK)

#define USDHC_INT_STATUS_EN_DEBESEN_MASK         (0x400000U)
#define USDHC_INT_STATUS_EN_DEBESEN_SHIFT        (22U)
/*! DEBESEN - Data end bit error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_DEBESEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_DEBESEN_SHIFT)) & USDHC_INT_STATUS_EN_DEBESEN_MASK)

#define USDHC_INT_STATUS_EN_AC12ESEN_MASK        (0x1000000U)
#define USDHC_INT_STATUS_EN_AC12ESEN_SHIFT       (24U)
/*! AC12ESEN - Auto CMD12 error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_AC12ESEN(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_AC12ESEN_SHIFT)) & USDHC_INT_STATUS_EN_AC12ESEN_MASK)

#define USDHC_INT_STATUS_EN_TNESEN_MASK          (0x4000000U)
#define USDHC_INT_STATUS_EN_TNESEN_SHIFT         (26U)
/*! TNESEN - Tuning error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_TNESEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_TNESEN_SHIFT)) & USDHC_INT_STATUS_EN_TNESEN_MASK)

#define USDHC_INT_STATUS_EN_DMAESEN_MASK         (0x10000000U)
#define USDHC_INT_STATUS_EN_DMAESEN_SHIFT        (28U)
/*! DMAESEN - DMA error status enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_STATUS_EN_DMAESEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_STATUS_EN_DMAESEN_SHIFT)) & USDHC_INT_STATUS_EN_DMAESEN_MASK)
/*! @} */

/*! @name INT_SIGNAL_EN - Interrupt Signal Enable */
/*! @{ */

#define USDHC_INT_SIGNAL_EN_CCIEN_MASK           (0x1U)
#define USDHC_INT_SIGNAL_EN_CCIEN_SHIFT          (0U)
/*! CCIEN - Command complete interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CCIEN(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CCIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CCIEN_MASK)

#define USDHC_INT_SIGNAL_EN_TCIEN_MASK           (0x2U)
#define USDHC_INT_SIGNAL_EN_TCIEN_SHIFT          (1U)
/*! TCIEN - Transfer complete interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_TCIEN(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_TCIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_TCIEN_MASK)

#define USDHC_INT_SIGNAL_EN_BGEIEN_MASK          (0x4U)
#define USDHC_INT_SIGNAL_EN_BGEIEN_SHIFT         (2U)
/*! BGEIEN - Block gap event interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_BGEIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_BGEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_BGEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_DINTIEN_MASK         (0x8U)
#define USDHC_INT_SIGNAL_EN_DINTIEN_SHIFT        (3U)
/*! DINTIEN - DMA interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_DINTIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_DINTIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_DINTIEN_MASK)

#define USDHC_INT_SIGNAL_EN_BWRIEN_MASK          (0x10U)
#define USDHC_INT_SIGNAL_EN_BWRIEN_SHIFT         (4U)
/*! BWRIEN - Buffer write ready interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_BWRIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_BWRIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_BWRIEN_MASK)

#define USDHC_INT_SIGNAL_EN_BRRIEN_MASK          (0x20U)
#define USDHC_INT_SIGNAL_EN_BRRIEN_SHIFT         (5U)
/*! BRRIEN - Buffer read ready interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_BRRIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_BRRIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_BRRIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CINSIEN_MASK         (0x40U)
#define USDHC_INT_SIGNAL_EN_CINSIEN_SHIFT        (6U)
/*! CINSIEN - Card insertion interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CINSIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CINSIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CINSIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CRMIEN_MASK          (0x80U)
#define USDHC_INT_SIGNAL_EN_CRMIEN_SHIFT         (7U)
/*! CRMIEN - Card removal interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CRMIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CRMIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CRMIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CINTIEN_MASK         (0x100U)
#define USDHC_INT_SIGNAL_EN_CINTIEN_SHIFT        (8U)
/*! CINTIEN - Card interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CINTIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CINTIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CINTIEN_MASK)

#define USDHC_INT_SIGNAL_EN_RTEIEN_MASK          (0x1000U)
#define USDHC_INT_SIGNAL_EN_RTEIEN_SHIFT         (12U)
/*! RTEIEN - Re-tuning event interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_RTEIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_RTEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_RTEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_TPIEN_MASK           (0x2000U)
#define USDHC_INT_SIGNAL_EN_TPIEN_SHIFT          (13U)
/*! TPIEN - Tuning pass interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_TPIEN(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_TPIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_TPIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CQIIEN_MASK          (0x4000U)
#define USDHC_INT_SIGNAL_EN_CQIIEN_SHIFT         (14U)
/*! CQIIEN - Command queuing signal enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CQIIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CQIIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CQIIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CTOEIEN_MASK         (0x10000U)
#define USDHC_INT_SIGNAL_EN_CTOEIEN_SHIFT        (16U)
/*! CTOEIEN - Command timeout error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CTOEIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CTOEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CTOEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CCEIEN_MASK          (0x20000U)
#define USDHC_INT_SIGNAL_EN_CCEIEN_SHIFT         (17U)
/*! CCEIEN - Command CRC error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CCEIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CCEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CCEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CEBEIEN_MASK         (0x40000U)
#define USDHC_INT_SIGNAL_EN_CEBEIEN_SHIFT        (18U)
/*! CEBEIEN - Command end bit error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CEBEIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CEBEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CEBEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_CIEIEN_MASK          (0x80000U)
#define USDHC_INT_SIGNAL_EN_CIEIEN_SHIFT         (19U)
/*! CIEIEN - Command index error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_CIEIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_CIEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_CIEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_DTOEIEN_MASK         (0x100000U)
#define USDHC_INT_SIGNAL_EN_DTOEIEN_SHIFT        (20U)
/*! DTOEIEN - Data timeout error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_DTOEIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_DTOEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_DTOEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_DCEIEN_MASK          (0x200000U)
#define USDHC_INT_SIGNAL_EN_DCEIEN_SHIFT         (21U)
/*! DCEIEN - Data CRC error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_DCEIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_DCEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_DCEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_DEBEIEN_MASK         (0x400000U)
#define USDHC_INT_SIGNAL_EN_DEBEIEN_SHIFT        (22U)
/*! DEBEIEN - Data end bit error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_DEBEIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_DEBEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_DEBEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_AC12EIEN_MASK        (0x1000000U)
#define USDHC_INT_SIGNAL_EN_AC12EIEN_SHIFT       (24U)
/*! AC12EIEN - Auto CMD12 error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_AC12EIEN(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_AC12EIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_AC12EIEN_MASK)

#define USDHC_INT_SIGNAL_EN_TNEIEN_MASK          (0x4000000U)
#define USDHC_INT_SIGNAL_EN_TNEIEN_SHIFT         (26U)
/*! TNEIEN - Tuning error interrupt enable
 *  0b1..Enabled
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_TNEIEN(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_TNEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_TNEIEN_MASK)

#define USDHC_INT_SIGNAL_EN_DMAEIEN_MASK         (0x10000000U)
#define USDHC_INT_SIGNAL_EN_DMAEIEN_SHIFT        (28U)
/*! DMAEIEN - DMA error interrupt enable
 *  0b1..Enable
 *  0b0..Masked
 */
#define USDHC_INT_SIGNAL_EN_DMAEIEN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_INT_SIGNAL_EN_DMAEIEN_SHIFT)) & USDHC_INT_SIGNAL_EN_DMAEIEN_MASK)
/*! @} */

/*! @name AUTOCMD12_ERR_STATUS - Auto CMD12 Error Status */
/*! @{ */

#define USDHC_AUTOCMD12_ERR_STATUS_AC12NE_MASK   (0x1U)
#define USDHC_AUTOCMD12_ERR_STATUS_AC12NE_SHIFT  (0U)
/*! AC12NE - Auto CMD12 not executed
 *  0b1..Not executed
 *  0b0..Executed
 */
#define USDHC_AUTOCMD12_ERR_STATUS_AC12NE(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_AC12NE_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_AC12NE_MASK)

#define USDHC_AUTOCMD12_ERR_STATUS_AC12TOE_MASK  (0x2U)
#define USDHC_AUTOCMD12_ERR_STATUS_AC12TOE_SHIFT (1U)
/*! AC12TOE - Auto CMD12 / 23 timeout error
 *  0b1..Time out
 *  0b0..No error
 */
#define USDHC_AUTOCMD12_ERR_STATUS_AC12TOE(x)    (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_AC12TOE_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_AC12TOE_MASK)

#define USDHC_AUTOCMD12_ERR_STATUS_AC12CE_MASK   (0x4U)
#define USDHC_AUTOCMD12_ERR_STATUS_AC12CE_SHIFT  (2U)
/*! AC12CE - Auto CMD12 / 23 CRC error
 *  0b1..CRC error met in Auto CMD12/23 response
 *  0b0..No CRC error
 */
#define USDHC_AUTOCMD12_ERR_STATUS_AC12CE(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_AC12CE_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_AC12CE_MASK)

#define USDHC_AUTOCMD12_ERR_STATUS_AC12EBE_MASK  (0x8U)
#define USDHC_AUTOCMD12_ERR_STATUS_AC12EBE_SHIFT (3U)
/*! AC12EBE - Auto CMD12 / 23 end bit error
 *  0b1..End bit error generated
 *  0b0..No error
 */
#define USDHC_AUTOCMD12_ERR_STATUS_AC12EBE(x)    (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_AC12EBE_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_AC12EBE_MASK)

#define USDHC_AUTOCMD12_ERR_STATUS_AC12IE_MASK   (0x10U)
#define USDHC_AUTOCMD12_ERR_STATUS_AC12IE_SHIFT  (4U)
/*! AC12IE - Auto CMD12 / 23 index error
 *  0b1..Error, the CMD index in response is not CMD12/23
 *  0b0..No error
 */
#define USDHC_AUTOCMD12_ERR_STATUS_AC12IE(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_AC12IE_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_AC12IE_MASK)

#define USDHC_AUTOCMD12_ERR_STATUS_CNIBAC12E_MASK (0x80U)
#define USDHC_AUTOCMD12_ERR_STATUS_CNIBAC12E_SHIFT (7U)
/*! CNIBAC12E - Command not issued by Auto CMD12 error
 *  0b1..Not issued
 *  0b0..No error
 */
#define USDHC_AUTOCMD12_ERR_STATUS_CNIBAC12E(x)  (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_CNIBAC12E_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_CNIBAC12E_MASK)

#define USDHC_AUTOCMD12_ERR_STATUS_EXECUTE_TUNING_MASK (0x400000U)
#define USDHC_AUTOCMD12_ERR_STATUS_EXECUTE_TUNING_SHIFT (22U)
/*! EXECUTE_TUNING - Execute tuning
 *  0b1..Start tuning procedure
 *  0b0..Tuning procedure is aborted
 */
#define USDHC_AUTOCMD12_ERR_STATUS_EXECUTE_TUNING(x) (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_EXECUTE_TUNING_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_EXECUTE_TUNING_MASK)

#define USDHC_AUTOCMD12_ERR_STATUS_SMP_CLK_SEL_MASK (0x800000U)
#define USDHC_AUTOCMD12_ERR_STATUS_SMP_CLK_SEL_SHIFT (23U)
/*! SMP_CLK_SEL - Sample clock select
 *  0b1..Tuned clock is used to sample data
 *  0b0..Fixed clock is used to sample data
 */
#define USDHC_AUTOCMD12_ERR_STATUS_SMP_CLK_SEL(x) (((uint32_t)(((uint32_t)(x)) << USDHC_AUTOCMD12_ERR_STATUS_SMP_CLK_SEL_SHIFT)) & USDHC_AUTOCMD12_ERR_STATUS_SMP_CLK_SEL_MASK)
/*! @} */

/*! @name HOST_CTRL_CAP - Host Controller Capabilities */
/*! @{ */

#define USDHC_HOST_CTRL_CAP_SDR50_SUPPORT_MASK   (0x1U)
#define USDHC_HOST_CTRL_CAP_SDR50_SUPPORT_SHIFT  (0U)
/*! SDR50_SUPPORT - SDR50 support */
#define USDHC_HOST_CTRL_CAP_SDR50_SUPPORT(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_SDR50_SUPPORT_SHIFT)) & USDHC_HOST_CTRL_CAP_SDR50_SUPPORT_MASK)

#define USDHC_HOST_CTRL_CAP_SDR104_SUPPORT_MASK  (0x2U)
#define USDHC_HOST_CTRL_CAP_SDR104_SUPPORT_SHIFT (1U)
/*! SDR104_SUPPORT - SDR104 support */
#define USDHC_HOST_CTRL_CAP_SDR104_SUPPORT(x)    (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_SDR104_SUPPORT_SHIFT)) & USDHC_HOST_CTRL_CAP_SDR104_SUPPORT_MASK)

#define USDHC_HOST_CTRL_CAP_DDR50_SUPPORT_MASK   (0x4U)
#define USDHC_HOST_CTRL_CAP_DDR50_SUPPORT_SHIFT  (2U)
/*! DDR50_SUPPORT - DDR50 support */
#define USDHC_HOST_CTRL_CAP_DDR50_SUPPORT(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_DDR50_SUPPORT_SHIFT)) & USDHC_HOST_CTRL_CAP_DDR50_SUPPORT_MASK)

#define USDHC_HOST_CTRL_CAP_USE_TUNING_SDR50_MASK (0x2000U)
#define USDHC_HOST_CTRL_CAP_USE_TUNING_SDR50_SHIFT (13U)
/*! USE_TUNING_SDR50 - Use Tuning for SDR50
 *  0b1..SDR50 supports tuning
 *  0b0..SDR50 does not support tuning
 */
#define USDHC_HOST_CTRL_CAP_USE_TUNING_SDR50(x)  (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_USE_TUNING_SDR50_SHIFT)) & USDHC_HOST_CTRL_CAP_USE_TUNING_SDR50_MASK)

#define USDHC_HOST_CTRL_CAP_MBL_MASK             (0x70000U)
#define USDHC_HOST_CTRL_CAP_MBL_SHIFT            (16U)
/*! MBL - Max block length
 *  0b000..512 bytes
 *  0b001..1024 bytes
 *  0b010..2048 bytes
 *  0b011..4096 bytes
 */
#define USDHC_HOST_CTRL_CAP_MBL(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_MBL_SHIFT)) & USDHC_HOST_CTRL_CAP_MBL_MASK)

#define USDHC_HOST_CTRL_CAP_ADMAS_MASK           (0x100000U)
#define USDHC_HOST_CTRL_CAP_ADMAS_SHIFT          (20U)
/*! ADMAS - ADMA support
 *  0b1..Advanced DMA supported
 *  0b0..Advanced DMA not supported
 */
#define USDHC_HOST_CTRL_CAP_ADMAS(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_ADMAS_SHIFT)) & USDHC_HOST_CTRL_CAP_ADMAS_MASK)

#define USDHC_HOST_CTRL_CAP_HSS_MASK             (0x200000U)
#define USDHC_HOST_CTRL_CAP_HSS_SHIFT            (21U)
/*! HSS - High speed support
 *  0b1..High speed supported
 *  0b0..High speed not supported
 */
#define USDHC_HOST_CTRL_CAP_HSS(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_HSS_SHIFT)) & USDHC_HOST_CTRL_CAP_HSS_MASK)

#define USDHC_HOST_CTRL_CAP_DMAS_MASK            (0x400000U)
#define USDHC_HOST_CTRL_CAP_DMAS_SHIFT           (22U)
/*! DMAS - DMA support
 *  0b1..DMA supported
 *  0b0..DMA not supported
 */
#define USDHC_HOST_CTRL_CAP_DMAS(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_DMAS_SHIFT)) & USDHC_HOST_CTRL_CAP_DMAS_MASK)

#define USDHC_HOST_CTRL_CAP_SRS_MASK             (0x800000U)
#define USDHC_HOST_CTRL_CAP_SRS_SHIFT            (23U)
/*! SRS - Suspend / resume support
 *  0b1..Supported
 *  0b0..Not supported
 */
#define USDHC_HOST_CTRL_CAP_SRS(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_SRS_SHIFT)) & USDHC_HOST_CTRL_CAP_SRS_MASK)

#define USDHC_HOST_CTRL_CAP_VS33_MASK            (0x1000000U)
#define USDHC_HOST_CTRL_CAP_VS33_SHIFT           (24U)
/*! VS33 - Voltage support 3.3 V
 *  0b1..3.3 V supported
 *  0b0..3.3 V not supported
 */
#define USDHC_HOST_CTRL_CAP_VS33(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_VS33_SHIFT)) & USDHC_HOST_CTRL_CAP_VS33_MASK)

#define USDHC_HOST_CTRL_CAP_VS30_MASK            (0x2000000U)
#define USDHC_HOST_CTRL_CAP_VS30_SHIFT           (25U)
/*! VS30 - Voltage support 3.0 V
 *  0b1..3.0 V supported
 *  0b0..3.0 V not supported
 */
#define USDHC_HOST_CTRL_CAP_VS30(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_VS30_SHIFT)) & USDHC_HOST_CTRL_CAP_VS30_MASK)

#define USDHC_HOST_CTRL_CAP_VS18_MASK            (0x4000000U)
#define USDHC_HOST_CTRL_CAP_VS18_SHIFT           (26U)
/*! VS18 - Voltage support 1.8 V
 *  0b1..1.8 V supported
 *  0b0..1.8 V not supported
 */
#define USDHC_HOST_CTRL_CAP_VS18(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_HOST_CTRL_CAP_VS18_SHIFT)) & USDHC_HOST_CTRL_CAP_VS18_MASK)
/*! @} */

/*! @name WTMK_LVL - Watermark Level */
/*! @{ */

#define USDHC_WTMK_LVL_RD_WML_MASK               (0xFFU)
#define USDHC_WTMK_LVL_RD_WML_SHIFT              (0U)
/*! RD_WML - Read watermark level */
#define USDHC_WTMK_LVL_RD_WML(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_WTMK_LVL_RD_WML_SHIFT)) & USDHC_WTMK_LVL_RD_WML_MASK)

#define USDHC_WTMK_LVL_WR_WML_MASK               (0xFF0000U)
#define USDHC_WTMK_LVL_WR_WML_SHIFT              (16U)
/*! WR_WML - Write watermark level */
#define USDHC_WTMK_LVL_WR_WML(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_WTMK_LVL_WR_WML_SHIFT)) & USDHC_WTMK_LVL_WR_WML_MASK)
/*! @} */

/*! @name MIX_CTRL - Mixer Control */
/*! @{ */

#define USDHC_MIX_CTRL_DMAEN_MASK                (0x1U)
#define USDHC_MIX_CTRL_DMAEN_SHIFT               (0U)
/*! DMAEN - DMA enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define USDHC_MIX_CTRL_DMAEN(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_DMAEN_SHIFT)) & USDHC_MIX_CTRL_DMAEN_MASK)

#define USDHC_MIX_CTRL_BCEN_MASK                 (0x2U)
#define USDHC_MIX_CTRL_BCEN_SHIFT                (1U)
/*! BCEN - Block count enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define USDHC_MIX_CTRL_BCEN(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_BCEN_SHIFT)) & USDHC_MIX_CTRL_BCEN_MASK)

#define USDHC_MIX_CTRL_AC12EN_MASK               (0x4U)
#define USDHC_MIX_CTRL_AC12EN_SHIFT              (2U)
/*! AC12EN - Auto CMD12 enable
 *  0b1..Enable
 *  0b0..Disable
 */
#define USDHC_MIX_CTRL_AC12EN(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_AC12EN_SHIFT)) & USDHC_MIX_CTRL_AC12EN_MASK)

#define USDHC_MIX_CTRL_DDR_EN_MASK               (0x8U)
#define USDHC_MIX_CTRL_DDR_EN_SHIFT              (3U)
/*! DDR_EN - Dual data rate mode selection */
#define USDHC_MIX_CTRL_DDR_EN(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_DDR_EN_SHIFT)) & USDHC_MIX_CTRL_DDR_EN_MASK)

#define USDHC_MIX_CTRL_DTDSEL_MASK               (0x10U)
#define USDHC_MIX_CTRL_DTDSEL_SHIFT              (4U)
/*! DTDSEL - Data transfer direction select
 *  0b1..Read (Card to host)
 *  0b0..Write (Host to card)
 */
#define USDHC_MIX_CTRL_DTDSEL(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_DTDSEL_SHIFT)) & USDHC_MIX_CTRL_DTDSEL_MASK)

#define USDHC_MIX_CTRL_MSBSEL_MASK               (0x20U)
#define USDHC_MIX_CTRL_MSBSEL_SHIFT              (5U)
/*! MSBSEL - Multi / Single block select
 *  0b1..Multiple blocks
 *  0b0..Single block
 */
#define USDHC_MIX_CTRL_MSBSEL(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_MSBSEL_SHIFT)) & USDHC_MIX_CTRL_MSBSEL_MASK)

#define USDHC_MIX_CTRL_NIBBLE_POS_MASK           (0x40U)
#define USDHC_MIX_CTRL_NIBBLE_POS_SHIFT          (6U)
/*! NIBBLE_POS - Nibble position indication */
#define USDHC_MIX_CTRL_NIBBLE_POS(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_NIBBLE_POS_SHIFT)) & USDHC_MIX_CTRL_NIBBLE_POS_MASK)

#define USDHC_MIX_CTRL_AC23EN_MASK               (0x80U)
#define USDHC_MIX_CTRL_AC23EN_SHIFT              (7U)
/*! AC23EN - Auto CMD23 enable */
#define USDHC_MIX_CTRL_AC23EN(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_AC23EN_SHIFT)) & USDHC_MIX_CTRL_AC23EN_MASK)

#define USDHC_MIX_CTRL_EXE_TUNE_MASK             (0x400000U)
#define USDHC_MIX_CTRL_EXE_TUNE_SHIFT            (22U)
/*! EXE_TUNE - Execute tuning: (Only used for SD3.0, SDR104 mode and eMMC HS200 mode)
 *  0b1..Execute tuning
 *  0b0..Not tuned or tuning completed
 */
#define USDHC_MIX_CTRL_EXE_TUNE(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_EXE_TUNE_SHIFT)) & USDHC_MIX_CTRL_EXE_TUNE_MASK)

#define USDHC_MIX_CTRL_SMP_CLK_SEL_MASK          (0x800000U)
#define USDHC_MIX_CTRL_SMP_CLK_SEL_SHIFT         (23U)
/*! SMP_CLK_SEL - Clock selection
 *  0b1..Tuned clock is used to sample data / cmd
 *  0b0..Fixed clock is used to sample data / cmd
 */
#define USDHC_MIX_CTRL_SMP_CLK_SEL(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_SMP_CLK_SEL_SHIFT)) & USDHC_MIX_CTRL_SMP_CLK_SEL_MASK)

#define USDHC_MIX_CTRL_AUTO_TUNE_EN_MASK         (0x1000000U)
#define USDHC_MIX_CTRL_AUTO_TUNE_EN_SHIFT        (24U)
/*! AUTO_TUNE_EN - Auto tuning enable (Only used for SD3.0, SDR104 mode and eMMC HS200 mode)
 *  0b1..Enable auto tuning
 *  0b0..Disable auto tuning
 */
#define USDHC_MIX_CTRL_AUTO_TUNE_EN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_AUTO_TUNE_EN_SHIFT)) & USDHC_MIX_CTRL_AUTO_TUNE_EN_MASK)

#define USDHC_MIX_CTRL_FBCLK_SEL_MASK            (0x2000000U)
#define USDHC_MIX_CTRL_FBCLK_SEL_SHIFT           (25U)
/*! FBCLK_SEL - Feedback clock source selection (Only used for SD3.0, SDR104 mode and eMMC HS200 mode)
 *  0b1..Feedback clock comes from the ipp_card_clk_out
 *  0b0..Feedback clock comes from the loopback CLK
 */
#define USDHC_MIX_CTRL_FBCLK_SEL(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_FBCLK_SEL_SHIFT)) & USDHC_MIX_CTRL_FBCLK_SEL_MASK)

#define USDHC_MIX_CTRL_HS400_MODE_MASK           (0x4000000U)
#define USDHC_MIX_CTRL_HS400_MODE_SHIFT          (26U)
/*! HS400_MODE - Enable HS400 mode */
#define USDHC_MIX_CTRL_HS400_MODE(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_HS400_MODE_SHIFT)) & USDHC_MIX_CTRL_HS400_MODE_MASK)

#define USDHC_MIX_CTRL_EN_HS400_MODE_MASK        (0x8000000U)
#define USDHC_MIX_CTRL_EN_HS400_MODE_SHIFT       (27U)
/*! EN_HS400_MODE - Enable enhance HS400 mode */
#define USDHC_MIX_CTRL_EN_HS400_MODE(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_MIX_CTRL_EN_HS400_MODE_SHIFT)) & USDHC_MIX_CTRL_EN_HS400_MODE_MASK)
/*! @} */

/*! @name FORCE_EVENT - Force Event */
/*! @{ */

#define USDHC_FORCE_EVENT_FEVTAC12NE_MASK        (0x1U)
#define USDHC_FORCE_EVENT_FEVTAC12NE_SHIFT       (0U)
/*! FEVTAC12NE - Force event auto command 12 not executed */
#define USDHC_FORCE_EVENT_FEVTAC12NE(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTAC12NE_SHIFT)) & USDHC_FORCE_EVENT_FEVTAC12NE_MASK)

#define USDHC_FORCE_EVENT_FEVTAC12TOE_MASK       (0x2U)
#define USDHC_FORCE_EVENT_FEVTAC12TOE_SHIFT      (1U)
/*! FEVTAC12TOE - Force event auto command 12 time out error */
#define USDHC_FORCE_EVENT_FEVTAC12TOE(x)         (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTAC12TOE_SHIFT)) & USDHC_FORCE_EVENT_FEVTAC12TOE_MASK)

#define USDHC_FORCE_EVENT_FEVTAC12CE_MASK        (0x4U)
#define USDHC_FORCE_EVENT_FEVTAC12CE_SHIFT       (2U)
/*! FEVTAC12CE - Force event auto command 12 CRC error */
#define USDHC_FORCE_EVENT_FEVTAC12CE(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTAC12CE_SHIFT)) & USDHC_FORCE_EVENT_FEVTAC12CE_MASK)

#define USDHC_FORCE_EVENT_FEVTAC12EBE_MASK       (0x8U)
#define USDHC_FORCE_EVENT_FEVTAC12EBE_SHIFT      (3U)
/*! FEVTAC12EBE - Force event Auto Command 12 end bit error */
#define USDHC_FORCE_EVENT_FEVTAC12EBE(x)         (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTAC12EBE_SHIFT)) & USDHC_FORCE_EVENT_FEVTAC12EBE_MASK)

#define USDHC_FORCE_EVENT_FEVTAC12IE_MASK        (0x10U)
#define USDHC_FORCE_EVENT_FEVTAC12IE_SHIFT       (4U)
/*! FEVTAC12IE - Force event Auto Command 12 index error */
#define USDHC_FORCE_EVENT_FEVTAC12IE(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTAC12IE_SHIFT)) & USDHC_FORCE_EVENT_FEVTAC12IE_MASK)

#define USDHC_FORCE_EVENT_FEVTCNIBAC12E_MASK     (0x80U)
#define USDHC_FORCE_EVENT_FEVTCNIBAC12E_SHIFT    (7U)
/*! FEVTCNIBAC12E - Force event command not executed by Auto Command 12 error */
#define USDHC_FORCE_EVENT_FEVTCNIBAC12E(x)       (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTCNIBAC12E_SHIFT)) & USDHC_FORCE_EVENT_FEVTCNIBAC12E_MASK)

#define USDHC_FORCE_EVENT_FEVTCTOE_MASK          (0x10000U)
#define USDHC_FORCE_EVENT_FEVTCTOE_SHIFT         (16U)
/*! FEVTCTOE - Force event command time out error */
#define USDHC_FORCE_EVENT_FEVTCTOE(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTCTOE_SHIFT)) & USDHC_FORCE_EVENT_FEVTCTOE_MASK)

#define USDHC_FORCE_EVENT_FEVTCCE_MASK           (0x20000U)
#define USDHC_FORCE_EVENT_FEVTCCE_SHIFT          (17U)
/*! FEVTCCE - Force event command CRC error */
#define USDHC_FORCE_EVENT_FEVTCCE(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTCCE_SHIFT)) & USDHC_FORCE_EVENT_FEVTCCE_MASK)

#define USDHC_FORCE_EVENT_FEVTCEBE_MASK          (0x40000U)
#define USDHC_FORCE_EVENT_FEVTCEBE_SHIFT         (18U)
/*! FEVTCEBE - Force event command end bit error */
#define USDHC_FORCE_EVENT_FEVTCEBE(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTCEBE_SHIFT)) & USDHC_FORCE_EVENT_FEVTCEBE_MASK)

#define USDHC_FORCE_EVENT_FEVTCIE_MASK           (0x80000U)
#define USDHC_FORCE_EVENT_FEVTCIE_SHIFT          (19U)
/*! FEVTCIE - Force event command index error */
#define USDHC_FORCE_EVENT_FEVTCIE(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTCIE_SHIFT)) & USDHC_FORCE_EVENT_FEVTCIE_MASK)

#define USDHC_FORCE_EVENT_FEVTDTOE_MASK          (0x100000U)
#define USDHC_FORCE_EVENT_FEVTDTOE_SHIFT         (20U)
/*! FEVTDTOE - Force event data time out error */
#define USDHC_FORCE_EVENT_FEVTDTOE(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTDTOE_SHIFT)) & USDHC_FORCE_EVENT_FEVTDTOE_MASK)

#define USDHC_FORCE_EVENT_FEVTDCE_MASK           (0x200000U)
#define USDHC_FORCE_EVENT_FEVTDCE_SHIFT          (21U)
/*! FEVTDCE - Force event data CRC error */
#define USDHC_FORCE_EVENT_FEVTDCE(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTDCE_SHIFT)) & USDHC_FORCE_EVENT_FEVTDCE_MASK)

#define USDHC_FORCE_EVENT_FEVTDEBE_MASK          (0x400000U)
#define USDHC_FORCE_EVENT_FEVTDEBE_SHIFT         (22U)
/*! FEVTDEBE - Force event data end bit error */
#define USDHC_FORCE_EVENT_FEVTDEBE(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTDEBE_SHIFT)) & USDHC_FORCE_EVENT_FEVTDEBE_MASK)

#define USDHC_FORCE_EVENT_FEVTAC12E_MASK         (0x1000000U)
#define USDHC_FORCE_EVENT_FEVTAC12E_SHIFT        (24U)
/*! FEVTAC12E - Force event Auto Command 12 error */
#define USDHC_FORCE_EVENT_FEVTAC12E(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTAC12E_SHIFT)) & USDHC_FORCE_EVENT_FEVTAC12E_MASK)

#define USDHC_FORCE_EVENT_FEVTTNE_MASK           (0x4000000U)
#define USDHC_FORCE_EVENT_FEVTTNE_SHIFT          (26U)
/*! FEVTTNE - Force tuning error */
#define USDHC_FORCE_EVENT_FEVTTNE(x)             (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTTNE_SHIFT)) & USDHC_FORCE_EVENT_FEVTTNE_MASK)

#define USDHC_FORCE_EVENT_FEVTDMAE_MASK          (0x10000000U)
#define USDHC_FORCE_EVENT_FEVTDMAE_SHIFT         (28U)
/*! FEVTDMAE - Force event DMA error */
#define USDHC_FORCE_EVENT_FEVTDMAE(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTDMAE_SHIFT)) & USDHC_FORCE_EVENT_FEVTDMAE_MASK)

#define USDHC_FORCE_EVENT_FEVTCINT_MASK          (0x80000000U)
#define USDHC_FORCE_EVENT_FEVTCINT_SHIFT         (31U)
/*! FEVTCINT - Force event card interrupt */
#define USDHC_FORCE_EVENT_FEVTCINT(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_FORCE_EVENT_FEVTCINT_SHIFT)) & USDHC_FORCE_EVENT_FEVTCINT_MASK)
/*! @} */

/*! @name ADMA_ERR_STATUS - ADMA Error Status */
/*! @{ */

#define USDHC_ADMA_ERR_STATUS_ADMAES_MASK        (0x3U)
#define USDHC_ADMA_ERR_STATUS_ADMAES_SHIFT       (0U)
/*! ADMAES - ADMA error state (when ADMA error is occurred) */
#define USDHC_ADMA_ERR_STATUS_ADMAES(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_ADMA_ERR_STATUS_ADMAES_SHIFT)) & USDHC_ADMA_ERR_STATUS_ADMAES_MASK)

#define USDHC_ADMA_ERR_STATUS_ADMALME_MASK       (0x4U)
#define USDHC_ADMA_ERR_STATUS_ADMALME_SHIFT      (2U)
/*! ADMALME - ADMA length mismatch error
 *  0b1..Error
 *  0b0..No error
 */
#define USDHC_ADMA_ERR_STATUS_ADMALME(x)         (((uint32_t)(((uint32_t)(x)) << USDHC_ADMA_ERR_STATUS_ADMALME_SHIFT)) & USDHC_ADMA_ERR_STATUS_ADMALME_MASK)

#define USDHC_ADMA_ERR_STATUS_ADMADCE_MASK       (0x8U)
#define USDHC_ADMA_ERR_STATUS_ADMADCE_SHIFT      (3U)
/*! ADMADCE - ADMA descriptor error
 *  0b1..Error
 *  0b0..No error
 */
#define USDHC_ADMA_ERR_STATUS_ADMADCE(x)         (((uint32_t)(((uint32_t)(x)) << USDHC_ADMA_ERR_STATUS_ADMADCE_SHIFT)) & USDHC_ADMA_ERR_STATUS_ADMADCE_MASK)
/*! @} */

/*! @name ADMA_SYS_ADDR - ADMA System Address */
/*! @{ */

#define USDHC_ADMA_SYS_ADDR_ADS_ADDR_MASK        (0xFFFFFFFCU)
#define USDHC_ADMA_SYS_ADDR_ADS_ADDR_SHIFT       (2U)
/*! ADS_ADDR - ADMA system address */
#define USDHC_ADMA_SYS_ADDR_ADS_ADDR(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_ADMA_SYS_ADDR_ADS_ADDR_SHIFT)) & USDHC_ADMA_SYS_ADDR_ADS_ADDR_MASK)
/*! @} */

/*! @name DLL_CTRL - DLL (Delay Line) Control */
/*! @{ */

#define USDHC_DLL_CTRL_DLL_CTRL_ENABLE_MASK      (0x1U)
#define USDHC_DLL_CTRL_DLL_CTRL_ENABLE_SHIFT     (0U)
/*! DLL_CTRL_ENABLE - DLL and delay chain */
#define USDHC_DLL_CTRL_DLL_CTRL_ENABLE(x)        (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_ENABLE_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_ENABLE_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_RESET_MASK       (0x2U)
#define USDHC_DLL_CTRL_DLL_CTRL_RESET_SHIFT      (1U)
/*! DLL_CTRL_RESET - DLL reset */
#define USDHC_DLL_CTRL_DLL_CTRL_RESET(x)         (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_RESET_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_RESET_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_SLV_FORCE_UPD_MASK (0x4U)
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_FORCE_UPD_SHIFT (2U)
/*! DLL_CTRL_SLV_FORCE_UPD - DLL slave delay line */
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_FORCE_UPD(x) (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_SLV_FORCE_UPD_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_SLV_FORCE_UPD_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET0_MASK (0x78U)
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET0_SHIFT (3U)
/*! DLL_CTRL_SLV_DLY_TARGET0 - DLL slave delay target0 */
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET0(x) (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET0_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET0_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_GATE_UPDATE_MASK (0x80U)
#define USDHC_DLL_CTRL_DLL_CTRL_GATE_UPDATE_SHIFT (7U)
/*! DLL_CTRL_GATE_UPDATE - DLL gate update */
#define USDHC_DLL_CTRL_DLL_CTRL_GATE_UPDATE(x)   (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_GATE_UPDATE_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_GATE_UPDATE_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_MASK (0x100U)
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_SHIFT (8U)
/*! DLL_CTRL_SLV_OVERRIDE - DLL slave override */
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE(x)  (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_VAL_MASK (0xFE00U)
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_VAL_SHIFT (9U)
/*! DLL_CTRL_SLV_OVERRIDE_VAL - DLL slave override val */
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_VAL(x) (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_VAL_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_SLV_OVERRIDE_VAL_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET1_MASK (0x70000U)
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET1_SHIFT (16U)
/*! DLL_CTRL_SLV_DLY_TARGET1 - DLL slave delay target1 */
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET1(x) (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET1_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_SLV_DLY_TARGET1_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_SLV_UPDATE_INT_MASK (0xFF00000U)
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_UPDATE_INT_SHIFT (20U)
/*! DLL_CTRL_SLV_UPDATE_INT - Slave delay line update interval */
#define USDHC_DLL_CTRL_DLL_CTRL_SLV_UPDATE_INT(x) (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_SLV_UPDATE_INT_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_SLV_UPDATE_INT_MASK)

#define USDHC_DLL_CTRL_DLL_CTRL_REF_UPDATE_INT_MASK (0xF0000000U)
#define USDHC_DLL_CTRL_DLL_CTRL_REF_UPDATE_INT_SHIFT (28U)
/*! DLL_CTRL_REF_UPDATE_INT - DLL control loop update interval */
#define USDHC_DLL_CTRL_DLL_CTRL_REF_UPDATE_INT(x) (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_CTRL_DLL_CTRL_REF_UPDATE_INT_SHIFT)) & USDHC_DLL_CTRL_DLL_CTRL_REF_UPDATE_INT_MASK)
/*! @} */

/*! @name DLL_STATUS - DLL Status */
/*! @{ */

#define USDHC_DLL_STATUS_DLL_STS_SLV_LOCK_MASK   (0x1U)
#define USDHC_DLL_STATUS_DLL_STS_SLV_LOCK_SHIFT  (0U)
/*! DLL_STS_SLV_LOCK - Slave delay-line lock status */
#define USDHC_DLL_STATUS_DLL_STS_SLV_LOCK(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_STATUS_DLL_STS_SLV_LOCK_SHIFT)) & USDHC_DLL_STATUS_DLL_STS_SLV_LOCK_MASK)

#define USDHC_DLL_STATUS_DLL_STS_REF_LOCK_MASK   (0x2U)
#define USDHC_DLL_STATUS_DLL_STS_REF_LOCK_SHIFT  (1U)
/*! DLL_STS_REF_LOCK - Reference DLL lock status */
#define USDHC_DLL_STATUS_DLL_STS_REF_LOCK(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_STATUS_DLL_STS_REF_LOCK_SHIFT)) & USDHC_DLL_STATUS_DLL_STS_REF_LOCK_MASK)

#define USDHC_DLL_STATUS_DLL_STS_SLV_SEL_MASK    (0x1FCU)
#define USDHC_DLL_STATUS_DLL_STS_SLV_SEL_SHIFT   (2U)
/*! DLL_STS_SLV_SEL - Slave delay line select status */
#define USDHC_DLL_STATUS_DLL_STS_SLV_SEL(x)      (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_STATUS_DLL_STS_SLV_SEL_SHIFT)) & USDHC_DLL_STATUS_DLL_STS_SLV_SEL_MASK)

#define USDHC_DLL_STATUS_DLL_STS_REF_SEL_MASK    (0xFE00U)
#define USDHC_DLL_STATUS_DLL_STS_REF_SEL_SHIFT   (9U)
/*! DLL_STS_REF_SEL - Reference delay line select taps */
#define USDHC_DLL_STATUS_DLL_STS_REF_SEL(x)      (((uint32_t)(((uint32_t)(x)) << USDHC_DLL_STATUS_DLL_STS_REF_SEL_SHIFT)) & USDHC_DLL_STATUS_DLL_STS_REF_SEL_MASK)
/*! @} */

/*! @name CLK_TUNE_CTRL_STATUS - CLK Tuning Control and Status */
/*! @{ */

#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_POST_MASK (0xFU)
#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_POST_SHIFT (0U)
/*! DLY_CELL_SET_POST - Delay cells on the feedback clock between CLK_OUT and CLK_POST */
#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_POST(x) (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_POST_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_POST_MASK)

#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_OUT_MASK (0xF0U)
#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_OUT_SHIFT (4U)
/*! DLY_CELL_SET_OUT - Delay cells on the feedback clock between CLK_PRE and CLK_OUT */
#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_OUT(x) (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_OUT_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_OUT_MASK)

#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_PRE_MASK (0x7F00U)
#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_PRE_SHIFT (8U)
/*! DLY_CELL_SET_PRE - delay cells on the feedback clock between the feedback clock and CLK_PRE */
#define USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_PRE(x) (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_PRE_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_DLY_CELL_SET_PRE_MASK)

#define USDHC_CLK_TUNE_CTRL_STATUS_NXT_ERR_MASK  (0x8000U)
#define USDHC_CLK_TUNE_CTRL_STATUS_NXT_ERR_SHIFT (15U)
/*! NXT_ERR - NXT error */
#define USDHC_CLK_TUNE_CTRL_STATUS_NXT_ERR(x)    (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_NXT_ERR_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_NXT_ERR_MASK)

#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_POST_MASK (0xF0000U)
#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_POST_SHIFT (16U)
/*! TAP_SEL_POST - Delay cells added on the feedback clock between CLK_OUT and CLK_POST */
#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_POST(x) (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_POST_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_POST_MASK)

#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_OUT_MASK (0xF00000U)
#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_OUT_SHIFT (20U)
/*! TAP_SEL_OUT - Delay cells added on the feedback clock between CLK_PRE and CLK_OUT */
#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_OUT(x) (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_OUT_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_OUT_MASK)

#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_PRE_MASK (0x7F000000U)
#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_PRE_SHIFT (24U)
/*! TAP_SEL_PRE - TAP_SEL_PRE */
#define USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_PRE(x) (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_PRE_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_TAP_SEL_PRE_MASK)

#define USDHC_CLK_TUNE_CTRL_STATUS_PRE_ERR_MASK  (0x80000000U)
#define USDHC_CLK_TUNE_CTRL_STATUS_PRE_ERR_SHIFT (31U)
/*! PRE_ERR - PRE error */
#define USDHC_CLK_TUNE_CTRL_STATUS_PRE_ERR(x)    (((uint32_t)(((uint32_t)(x)) << USDHC_CLK_TUNE_CTRL_STATUS_PRE_ERR_SHIFT)) & USDHC_CLK_TUNE_CTRL_STATUS_PRE_ERR_MASK)
/*! @} */

/*! @name STROBE_DLL_CTRL - Strobe DLL control */
/*! @{ */

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_ENABLE_MASK (0x1U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_ENABLE_SHIFT (0U)
/*! STROBE_DLL_CTRL_ENABLE - Strobe DLL control enable */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_ENABLE(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_ENABLE_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_ENABLE_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_RESET_MASK (0x2U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_RESET_SHIFT (1U)
/*! STROBE_DLL_CTRL_RESET - Strobe DLL control reset */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_RESET(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_RESET_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_RESET_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_FORCE_UPD_MASK (0x4U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_FORCE_UPD_SHIFT (2U)
/*! STROBE_DLL_CTRL_SLV_FORCE_UPD - Strobe DLL control slave force updated */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_FORCE_UPD(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_FORCE_UPD_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_FORCE_UPD_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_DLY_TARGET_MASK (0x78U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_DLY_TARGET_SHIFT (3U)
/*! STROBE_DLL_CTRL_SLV_DLY_TARGET - Strobe DLL Control Slave Delay Target */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_DLY_TARGET(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_DLY_TARGET_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_DLY_TARGET_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_GATE_UPDATE_MASK (0x80U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_GATE_UPDATE_SHIFT (7U)
/*! STROBE_DLL_CTRL_GATE_UPDATE - Strobe DLL control gate update */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_GATE_UPDATE(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_GATE_UPDATE_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_GATE_UPDATE_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_MASK (0x100U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_SHIFT (8U)
/*! STROBE_DLL_CTRL_SLV_OVERRIDE - Strobe DLL control slave override */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_VAL_MASK (0xFE00U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_VAL_SHIFT (9U)
/*! STROBE_DLL_CTRL_SLV_OVERRIDE_VAL - Strobe DLL control slave Override value */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_VAL(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_VAL_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_OVERRIDE_VAL_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_UPDATE_INT_MASK (0xFF00000U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_UPDATE_INT_SHIFT (20U)
/*! STROBE_DLL_CTRL_SLV_UPDATE_INT - Strobe DLL control slave update interval */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_UPDATE_INT(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_UPDATE_INT_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_SLV_UPDATE_INT_MASK)

#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_REF_UPDATE_INT_MASK (0xF0000000U)
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_REF_UPDATE_INT_SHIFT (28U)
/*! STROBE_DLL_CTRL_REF_UPDATE_INT - Strobe DLL control reference update interval */
#define USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_REF_UPDATE_INT(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_REF_UPDATE_INT_SHIFT)) & USDHC_STROBE_DLL_CTRL_STROBE_DLL_CTRL_REF_UPDATE_INT_MASK)
/*! @} */

/*! @name STROBE_DLL_STATUS - Strobe DLL status */
/*! @{ */

#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_LOCK_MASK (0x1U)
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_LOCK_SHIFT (0U)
/*! STROBE_DLL_STS_SLV_LOCK - Strobe DLL status slave lock */
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_LOCK(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_LOCK_SHIFT)) & USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_LOCK_MASK)

#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_LOCK_MASK (0x2U)
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_LOCK_SHIFT (1U)
/*! STROBE_DLL_STS_REF_LOCK - Strobe DLL status reference lock */
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_LOCK(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_LOCK_SHIFT)) & USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_LOCK_MASK)

#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_SEL_MASK (0x1FCU)
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_SEL_SHIFT (2U)
/*! STROBE_DLL_STS_SLV_SEL - Strobe DLL status slave select */
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_SEL(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_SEL_SHIFT)) & USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_SLV_SEL_MASK)

#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_SEL_MASK (0xFE00U)
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_SEL_SHIFT (9U)
/*! STROBE_DLL_STS_REF_SEL - Strobe DLL status reference select */
#define USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_SEL(x) (((uint32_t)(((uint32_t)(x)) << USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_SEL_SHIFT)) & USDHC_STROBE_DLL_STATUS_STROBE_DLL_STS_REF_SEL_MASK)
/*! @} */

/*! @name VEND_SPEC - Vendor Specific Register */
/*! @{ */

#define USDHC_VEND_SPEC_VSELECT_MASK             (0x2U)
#define USDHC_VEND_SPEC_VSELECT_SHIFT            (1U)
/*! VSELECT - Voltage selection
 *  0b1..Change the voltage to low voltage range , around 1.8 V
 *  0b0..Change the voltage to high voltage range, around 3.0 V
 */
#define USDHC_VEND_SPEC_VSELECT(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC_VSELECT_SHIFT)) & USDHC_VEND_SPEC_VSELECT_MASK)

#define USDHC_VEND_SPEC_AC12_WR_CHKBUSY_EN_MASK  (0x8U)
#define USDHC_VEND_SPEC_AC12_WR_CHKBUSY_EN_SHIFT (3U)
/*! AC12_WR_CHKBUSY_EN - Check busy enable
 *  0b0..Do not check busy after auto CMD12 for write data packet
 *  0b1..Check busy after auto CMD12 for write data packet
 */
#define USDHC_VEND_SPEC_AC12_WR_CHKBUSY_EN(x)    (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC_AC12_WR_CHKBUSY_EN_SHIFT)) & USDHC_VEND_SPEC_AC12_WR_CHKBUSY_EN_MASK)

#define USDHC_VEND_SPEC_FRC_SDCLK_ON_MASK        (0x100U)
#define USDHC_VEND_SPEC_FRC_SDCLK_ON_SHIFT       (8U)
/*! FRC_SDCLK_ON - Force CLK
 *  0b0..CLK active or inactive is fully controlled by the hardware.
 *  0b1..Force CLK active
 */
#define USDHC_VEND_SPEC_FRC_SDCLK_ON(x)          (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC_FRC_SDCLK_ON_SHIFT)) & USDHC_VEND_SPEC_FRC_SDCLK_ON_MASK)

#define USDHC_VEND_SPEC_CRC_CHK_DIS_MASK         (0x8000U)
#define USDHC_VEND_SPEC_CRC_CHK_DIS_SHIFT        (15U)
/*! CRC_CHK_DIS - CRC Check Disable
 *  0b0..Check CRC16 for every read data packet and check CRC fields for every write data packet
 *  0b1..Ignore CRC16 check for every read data packet and ignore CRC fields check for every write data packet
 */
#define USDHC_VEND_SPEC_CRC_CHK_DIS(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC_CRC_CHK_DIS_SHIFT)) & USDHC_VEND_SPEC_CRC_CHK_DIS_MASK)

#define USDHC_VEND_SPEC_CMD_BYTE_EN_MASK         (0x80000000U)
#define USDHC_VEND_SPEC_CMD_BYTE_EN_SHIFT        (31U)
/*! CMD_BYTE_EN - Register byte access for CMD_XFR_TYP
 *  0b0..Disable. MIX_CTRL[7:0] is read/write and CMD_XFR_TYP[7:0] is read-only.
 *  0b1..Enable. MIX_CTRL[7:0] is read-only and CMD_XFR_TYP[7:0] is read/write.
 */
#define USDHC_VEND_SPEC_CMD_BYTE_EN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC_CMD_BYTE_EN_SHIFT)) & USDHC_VEND_SPEC_CMD_BYTE_EN_MASK)
/*! @} */

/*! @name MMC_BOOT - eMMC Boot */
/*! @{ */

#define USDHC_MMC_BOOT_DTOCV_ACK_MASK            (0xFU)
#define USDHC_MMC_BOOT_DTOCV_ACK_SHIFT           (0U)
/*! DTOCV_ACK - DTOCV_ACK
 *  0b0000..SDCLK x 2^32
 *  0b0001..SDCLK x 2^33
 *  0b0010..SDCLK x 2^18
 *  0b0011..SDCLK x 2^19
 *  0b0100..SDCLK x 2^20
 *  0b0101..SDCLK x 2^21
 *  0b0110..SDCLK x 2^22
 *  0b0111..SDCLK x 2^23
 *  0b1110..SDCLK x 2^30
 *  0b1111..SDCLK x 2^31
 */
#define USDHC_MMC_BOOT_DTOCV_ACK(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_MMC_BOOT_DTOCV_ACK_SHIFT)) & USDHC_MMC_BOOT_DTOCV_ACK_MASK)

#define USDHC_MMC_BOOT_BOOT_ACK_MASK             (0x10U)
#define USDHC_MMC_BOOT_BOOT_ACK_SHIFT            (4U)
/*! BOOT_ACK - BOOT ACK
 *  0b0..No ack
 *  0b1..Ack
 */
#define USDHC_MMC_BOOT_BOOT_ACK(x)               (((uint32_t)(((uint32_t)(x)) << USDHC_MMC_BOOT_BOOT_ACK_SHIFT)) & USDHC_MMC_BOOT_BOOT_ACK_MASK)

#define USDHC_MMC_BOOT_BOOT_MODE_MASK            (0x20U)
#define USDHC_MMC_BOOT_BOOT_MODE_SHIFT           (5U)
/*! BOOT_MODE - Boot mode
 *  0b0..Normal boot
 *  0b1..Alternative boot
 */
#define USDHC_MMC_BOOT_BOOT_MODE(x)              (((uint32_t)(((uint32_t)(x)) << USDHC_MMC_BOOT_BOOT_MODE_SHIFT)) & USDHC_MMC_BOOT_BOOT_MODE_MASK)

#define USDHC_MMC_BOOT_BOOT_EN_MASK              (0x40U)
#define USDHC_MMC_BOOT_BOOT_EN_SHIFT             (6U)
/*! BOOT_EN - Boot enable
 *  0b0..Fast boot disable
 *  0b1..Fast boot enable
 */
#define USDHC_MMC_BOOT_BOOT_EN(x)                (((uint32_t)(((uint32_t)(x)) << USDHC_MMC_BOOT_BOOT_EN_SHIFT)) & USDHC_MMC_BOOT_BOOT_EN_MASK)

#define USDHC_MMC_BOOT_AUTO_SABG_EN_MASK         (0x80U)
#define USDHC_MMC_BOOT_AUTO_SABG_EN_SHIFT        (7U)
/*! AUTO_SABG_EN - Auto stop at block gap */
#define USDHC_MMC_BOOT_AUTO_SABG_EN(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_MMC_BOOT_AUTO_SABG_EN_SHIFT)) & USDHC_MMC_BOOT_AUTO_SABG_EN_MASK)

#define USDHC_MMC_BOOT_DISABLE_TIME_OUT_MASK     (0x100U)
#define USDHC_MMC_BOOT_DISABLE_TIME_OUT_SHIFT    (8U)
/*! DISABLE_TIME_OUT - Time out
 *  0b0..Enable time out
 *  0b1..Disable time out
 */
#define USDHC_MMC_BOOT_DISABLE_TIME_OUT(x)       (((uint32_t)(((uint32_t)(x)) << USDHC_MMC_BOOT_DISABLE_TIME_OUT_SHIFT)) & USDHC_MMC_BOOT_DISABLE_TIME_OUT_MASK)

#define USDHC_MMC_BOOT_BOOT_BLK_CNT_MASK         (0xFFFF0000U)
#define USDHC_MMC_BOOT_BOOT_BLK_CNT_SHIFT        (16U)
/*! BOOT_BLK_CNT - Stop At Block Gap value of automatic mode */
#define USDHC_MMC_BOOT_BOOT_BLK_CNT(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_MMC_BOOT_BOOT_BLK_CNT_SHIFT)) & USDHC_MMC_BOOT_BOOT_BLK_CNT_MASK)
/*! @} */

/*! @name VEND_SPEC2 - Vendor Specific 2 Register */
/*! @{ */

#define USDHC_VEND_SPEC2_CARD_INT_D3_TEST_MASK   (0x8U)
#define USDHC_VEND_SPEC2_CARD_INT_D3_TEST_SHIFT  (3U)
/*! CARD_INT_D3_TEST - Card interrupt detection test
 *  0b0..Check the card interrupt only when DATA3 is high.
 *  0b1..Check the card interrupt by ignoring the status of DATA3.
 */
#define USDHC_VEND_SPEC2_CARD_INT_D3_TEST(x)     (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_CARD_INT_D3_TEST_SHIFT)) & USDHC_VEND_SPEC2_CARD_INT_D3_TEST_MASK)

#define USDHC_VEND_SPEC2_TUNING_BIT_EN_MASK      (0x30U)
#define USDHC_VEND_SPEC2_TUNING_BIT_EN_SHIFT     (4U)
/*! TUNING_BIT_EN - Tuning bit enable
 *  0b00..Enable Tuning circuit for DATA[3:0]
 *  0b01..Enable Tuning circuit for DATA[7:0]
 *  0b10..Enable Tuning circuit for DATA[0]
 *  0b11..Invalid
 */
#define USDHC_VEND_SPEC2_TUNING_BIT_EN(x)        (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_TUNING_BIT_EN_SHIFT)) & USDHC_VEND_SPEC2_TUNING_BIT_EN_MASK)

#define USDHC_VEND_SPEC2_TUNING_CMD_EN_MASK      (0x40U)
#define USDHC_VEND_SPEC2_TUNING_CMD_EN_SHIFT     (6U)
/*! TUNING_CMD_EN - Tuning command enable
 *  0b0..Auto tuning circuit does not check the CMD line.
 *  0b1..Auto tuning circuit checks the CMD line.
 */
#define USDHC_VEND_SPEC2_TUNING_CMD_EN(x)        (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_TUNING_CMD_EN_SHIFT)) & USDHC_VEND_SPEC2_TUNING_CMD_EN_MASK)

#define USDHC_VEND_SPEC2_HS400_WR_CLK_STOP_EN_MASK (0x400U)
#define USDHC_VEND_SPEC2_HS400_WR_CLK_STOP_EN_SHIFT (10U)
/*! HS400_WR_CLK_STOP_EN - HS400 write clock stop enable */
#define USDHC_VEND_SPEC2_HS400_WR_CLK_STOP_EN(x) (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_HS400_WR_CLK_STOP_EN_SHIFT)) & USDHC_VEND_SPEC2_HS400_WR_CLK_STOP_EN_MASK)

#define USDHC_VEND_SPEC2_HS400_RD_CLK_STOP_EN_MASK (0x800U)
#define USDHC_VEND_SPEC2_HS400_RD_CLK_STOP_EN_SHIFT (11U)
/*! HS400_RD_CLK_STOP_EN - HS400 read clock stop enable */
#define USDHC_VEND_SPEC2_HS400_RD_CLK_STOP_EN(x) (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_HS400_RD_CLK_STOP_EN_SHIFT)) & USDHC_VEND_SPEC2_HS400_RD_CLK_STOP_EN_MASK)

#define USDHC_VEND_SPEC2_ACMD23_ARGU2_EN_MASK    (0x1000U)
#define USDHC_VEND_SPEC2_ACMD23_ARGU2_EN_SHIFT   (12U)
/*! ACMD23_ARGU2_EN - Argument2 register enable for ACMD23
 *  0b1..Argument2 register enable for ACMD23 sharing with SDMA system address register. Default is enabled.
 *  0b0..Disable
 */
#define USDHC_VEND_SPEC2_ACMD23_ARGU2_EN(x)      (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_ACMD23_ARGU2_EN_SHIFT)) & USDHC_VEND_SPEC2_ACMD23_ARGU2_EN_MASK)

#define USDHC_VEND_SPEC2_EN_32K_CLK_MASK         (0x8000U)
#define USDHC_VEND_SPEC2_EN_32K_CLK_SHIFT        (15U)
/*! EN_32K_CLK - Select the clock source for host card detection.
 *  0b0..Use the peripheral clock (ipg_clk) for card detection.
 *  0b1..Use the low power clock (ipg_clk_lp) for card detection.
 */
#define USDHC_VEND_SPEC2_EN_32K_CLK(x)           (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_EN_32K_CLK_SHIFT)) & USDHC_VEND_SPEC2_EN_32K_CLK_MASK)

#define USDHC_VEND_SPEC2_FBCLK_TAP_SEL_MASK      (0xFFFF0000U)
#define USDHC_VEND_SPEC2_FBCLK_TAP_SEL_SHIFT     (16U)
/*! FBCLK_TAP_SEL - Enable extra delay on internal feedback clock */
#define USDHC_VEND_SPEC2_FBCLK_TAP_SEL(x)        (((uint32_t)(((uint32_t)(x)) << USDHC_VEND_SPEC2_FBCLK_TAP_SEL_SHIFT)) & USDHC_VEND_SPEC2_FBCLK_TAP_SEL_MASK)
/*! @} */

/*! @name TUNING_CTRL - Tuning Control */
/*! @{ */

#define USDHC_TUNING_CTRL_TUNING_START_TAP_MASK  (0x7FU)
#define USDHC_TUNING_CTRL_TUNING_START_TAP_SHIFT (0U)
/*! TUNING_START_TAP - Tuning start */
#define USDHC_TUNING_CTRL_TUNING_START_TAP(x)    (((uint32_t)(((uint32_t)(x)) << USDHC_TUNING_CTRL_TUNING_START_TAP_SHIFT)) & USDHC_TUNING_CTRL_TUNING_START_TAP_MASK)

#define USDHC_TUNING_CTRL_DIS_CMD_CHK_FOR_STD_TUNING_MASK (0x80U)
#define USDHC_TUNING_CTRL_DIS_CMD_CHK_FOR_STD_TUNING_SHIFT (7U)
/*! DIS_CMD_CHK_FOR_STD_TUNING - Disable command check for standard tuning */
#define USDHC_TUNING_CTRL_DIS_CMD_CHK_FOR_STD_TUNING(x) (((uint32_t)(((uint32_t)(x)) << USDHC_TUNING_CTRL_DIS_CMD_CHK_FOR_STD_TUNING_SHIFT)) & USDHC_TUNING_CTRL_DIS_CMD_CHK_FOR_STD_TUNING_MASK)

#define USDHC_TUNING_CTRL_TUNING_COUNTER_MASK    (0xFF00U)
#define USDHC_TUNING_CTRL_TUNING_COUNTER_SHIFT   (8U)
/*! TUNING_COUNTER - Tuning counter */
#define USDHC_TUNING_CTRL_TUNING_COUNTER(x)      (((uint32_t)(((uint32_t)(x)) << USDHC_TUNING_CTRL_TUNING_COUNTER_SHIFT)) & USDHC_TUNING_CTRL_TUNING_COUNTER_MASK)

#define USDHC_TUNING_CTRL_TUNING_STEP_MASK       (0x70000U)
#define USDHC_TUNING_CTRL_TUNING_STEP_SHIFT      (16U)
/*! TUNING_STEP - TUNING_STEP */
#define USDHC_TUNING_CTRL_TUNING_STEP(x)         (((uint32_t)(((uint32_t)(x)) << USDHC_TUNING_CTRL_TUNING_STEP_SHIFT)) & USDHC_TUNING_CTRL_TUNING_STEP_MASK)

#define USDHC_TUNING_CTRL_TUNING_WINDOW_MASK     (0x700000U)
#define USDHC_TUNING_CTRL_TUNING_WINDOW_SHIFT    (20U)
/*! TUNING_WINDOW - Data window */
#define USDHC_TUNING_CTRL_TUNING_WINDOW(x)       (((uint32_t)(((uint32_t)(x)) << USDHC_TUNING_CTRL_TUNING_WINDOW_SHIFT)) & USDHC_TUNING_CTRL_TUNING_WINDOW_MASK)

#define USDHC_TUNING_CTRL_STD_TUNING_EN_MASK     (0x1000000U)
#define USDHC_TUNING_CTRL_STD_TUNING_EN_SHIFT    (24U)
/*! STD_TUNING_EN - Standard tuning circuit and procedure enable */
#define USDHC_TUNING_CTRL_STD_TUNING_EN(x)       (((uint32_t)(((uint32_t)(x)) << USDHC_TUNING_CTRL_STD_TUNING_EN_SHIFT)) & USDHC_TUNING_CTRL_STD_TUNING_EN_MASK)
/*! @} */

/*! @name CQVER - Command Queuing Version */
/*! @{ */

#define USDHC_CQVER_VERSION_SUFFIX_MASK          (0xFU)
#define USDHC_CQVER_VERSION_SUFFIX_SHIFT         (0U)
/*! VERSION_SUFFIX - eMMC version suffix */
#define USDHC_CQVER_VERSION_SUFFIX(x)            (((uint32_t)(((uint32_t)(x)) << USDHC_CQVER_VERSION_SUFFIX_SHIFT)) & USDHC_CQVER_VERSION_SUFFIX_MASK)

#define USDHC_CQVER_MINOR_VN_MASK                (0xF0U)
#define USDHC_CQVER_MINOR_VN_SHIFT               (4U)
/*! MINOR_VN - eMMC minor version number */
#define USDHC_CQVER_MINOR_VN(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQVER_MINOR_VN_SHIFT)) & USDHC_CQVER_MINOR_VN_MASK)

#define USDHC_CQVER_MAJOR_VN_MASK                (0xF00U)
#define USDHC_CQVER_MAJOR_VN_SHIFT               (8U)
/*! MAJOR_VN - eMMC major version number */
#define USDHC_CQVER_MAJOR_VN(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQVER_MAJOR_VN_SHIFT)) & USDHC_CQVER_MAJOR_VN_MASK)
/*! @} */

/*! @name CQCAP - Command Queuing Capabilities */
/*! @{ */

#define USDHC_CQCAP_ITCFVAL_MASK                 (0x3FFU)
#define USDHC_CQCAP_ITCFVAL_SHIFT                (0U)
/*! ITCFVAL - Internal timer clock frequency value */
#define USDHC_CQCAP_ITCFVAL(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQCAP_ITCFVAL_SHIFT)) & USDHC_CQCAP_ITCFVAL_MASK)

#define USDHC_CQCAP_ITCFMUL_MASK                 (0xF000U)
#define USDHC_CQCAP_ITCFMUL_SHIFT                (12U)
/*! ITCFMUL - Internal timer clock frequency multiplier
 *  0b0001..0.001 MHz
 *  0b0010..0.01 MHz
 *  0b0011..0.1 MHz
 *  0b0100..1 MHz
 *  0b0101..10 MHz
 *  0b0110-0b1001..Reserved
 */
#define USDHC_CQCAP_ITCFMUL(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQCAP_ITCFMUL_SHIFT)) & USDHC_CQCAP_ITCFMUL_MASK)
/*! @} */

/*! @name CQCFG - Command Queuing Configuration */
/*! @{ */

#define USDHC_CQCFG_CQUE_MASK                    (0x1U)
#define USDHC_CQCFG_CQUE_SHIFT                   (0U)
/*! CQUE - Command queuing enable */
#define USDHC_CQCFG_CQUE(x)                      (((uint32_t)(((uint32_t)(x)) << USDHC_CQCFG_CQUE_SHIFT)) & USDHC_CQCFG_CQUE_MASK)

#define USDHC_CQCFG_TDS_MASK                     (0x100U)
#define USDHC_CQCFG_TDS_SHIFT                    (8U)
/*! TDS - Task descriptor size
 *  0b0..Task descriptor size is 64 bits
 *  0b1..Task descriptor size is 128 bits
 */
#define USDHC_CQCFG_TDS(x)                       (((uint32_t)(((uint32_t)(x)) << USDHC_CQCFG_TDS_SHIFT)) & USDHC_CQCFG_TDS_MASK)

#define USDHC_CQCFG_DCMDE_MASK                   (0x1000U)
#define USDHC_CQCFG_DCMDE_SHIFT                  (12U)
/*! DCMDE - Direct command (DCMD) enable
 *  0b0..Task descriptor in slot #31 is a Data Transfer Task Descriptor
 *  0b1..Task descriptor in slot #31 is a DCMD Task Descriptor
 */
#define USDHC_CQCFG_DCMDE(x)                     (((uint32_t)(((uint32_t)(x)) << USDHC_CQCFG_DCMDE_SHIFT)) & USDHC_CQCFG_DCMDE_MASK)
/*! @} */

/*! @name CQCTL - Command Queuing Control */
/*! @{ */

#define USDHC_CQCTL_HALT_MASK                    (0x1U)
#define USDHC_CQCTL_HALT_SHIFT                   (0U)
/*! HALT - Halt */
#define USDHC_CQCTL_HALT(x)                      (((uint32_t)(((uint32_t)(x)) << USDHC_CQCTL_HALT_SHIFT)) & USDHC_CQCTL_HALT_MASK)

#define USDHC_CQCTL_CLEAR_MASK                   (0x100U)
#define USDHC_CQCTL_CLEAR_SHIFT                  (8U)
/*! CLEAR - Clear all tasks */
#define USDHC_CQCTL_CLEAR(x)                     (((uint32_t)(((uint32_t)(x)) << USDHC_CQCTL_CLEAR_SHIFT)) & USDHC_CQCTL_CLEAR_MASK)
/*! @} */

/*! @name CQIS - Command Queuing Interrupt Status */
/*! @{ */

#define USDHC_CQIS_HAC_MASK                      (0x1U)
#define USDHC_CQIS_HAC_SHIFT                     (0U)
/*! HAC - Halt complete interrupt */
#define USDHC_CQIS_HAC(x)                        (((uint32_t)(((uint32_t)(x)) << USDHC_CQIS_HAC_SHIFT)) & USDHC_CQIS_HAC_MASK)

#define USDHC_CQIS_TCC_MASK                      (0x2U)
#define USDHC_CQIS_TCC_SHIFT                     (1U)
/*! TCC - Task complete interrupt */
#define USDHC_CQIS_TCC(x)                        (((uint32_t)(((uint32_t)(x)) << USDHC_CQIS_TCC_SHIFT)) & USDHC_CQIS_TCC_MASK)

#define USDHC_CQIS_RED_MASK                      (0x4U)
#define USDHC_CQIS_RED_SHIFT                     (2U)
/*! RED - Response error detected interrupt */
#define USDHC_CQIS_RED(x)                        (((uint32_t)(((uint32_t)(x)) << USDHC_CQIS_RED_SHIFT)) & USDHC_CQIS_RED_MASK)

#define USDHC_CQIS_TCL_MASK                      (0x8U)
#define USDHC_CQIS_TCL_SHIFT                     (3U)
/*! TCL - Task cleared */
#define USDHC_CQIS_TCL(x)                        (((uint32_t)(((uint32_t)(x)) << USDHC_CQIS_TCL_SHIFT)) & USDHC_CQIS_TCL_MASK)
/*! @} */

/*! @name CQISTE - Command Queuing Interrupt Status Enable */
/*! @{ */

#define USDHC_CQISTE_HAC_STE_MASK                (0x1U)
#define USDHC_CQISTE_HAC_STE_SHIFT               (0U)
/*! HAC_STE - Halt complete status enable
 *  0b0..CQIS[HAC] is disabled
 *  0b1..CQIS[HAC] is set when its interrupt condition is active
 */
#define USDHC_CQISTE_HAC_STE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISTE_HAC_STE_SHIFT)) & USDHC_CQISTE_HAC_STE_MASK)

#define USDHC_CQISTE_TCC_STE_MASK                (0x2U)
#define USDHC_CQISTE_TCC_STE_SHIFT               (1U)
/*! TCC_STE - Task complete status enable
 *  0b0..CQIS[TCC] is disabled
 *  0b1..CQIS[TCC] is set when its interrupt condition is active
 */
#define USDHC_CQISTE_TCC_STE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISTE_TCC_STE_SHIFT)) & USDHC_CQISTE_TCC_STE_MASK)

#define USDHC_CQISTE_RED_STE_MASK                (0x4U)
#define USDHC_CQISTE_RED_STE_SHIFT               (2U)
/*! RED_STE - Response error detected status enable
 *  0b0..CQIS[RED]is disabled
 *  0b1..CQIS[RED] is set when its interrupt condition is active
 */
#define USDHC_CQISTE_RED_STE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISTE_RED_STE_SHIFT)) & USDHC_CQISTE_RED_STE_MASK)

#define USDHC_CQISTE_TCL_STE_MASK                (0x8U)
#define USDHC_CQISTE_TCL_STE_SHIFT               (3U)
/*! TCL_STE - Task cleared status enable
 *  0b0..CQIS[TCL] is disabled
 *  0b1..CQIS[TCL] is set when its interrupt condition is active
 */
#define USDHC_CQISTE_TCL_STE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISTE_TCL_STE_SHIFT)) & USDHC_CQISTE_TCL_STE_MASK)
/*! @} */

/*! @name CQISGE - Command Queuing Interrupt Signal Enable */
/*! @{ */

#define USDHC_CQISGE_HAC_SGE_MASK                (0x1U)
#define USDHC_CQISGE_HAC_SGE_SHIFT               (0U)
/*! HAC_SGE - Halt complete signal enable */
#define USDHC_CQISGE_HAC_SGE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISGE_HAC_SGE_SHIFT)) & USDHC_CQISGE_HAC_SGE_MASK)

#define USDHC_CQISGE_TCC_SGE_MASK                (0x2U)
#define USDHC_CQISGE_TCC_SGE_SHIFT               (1U)
/*! TCC_SGE - Task complete signal enable */
#define USDHC_CQISGE_TCC_SGE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISGE_TCC_SGE_SHIFT)) & USDHC_CQISGE_TCC_SGE_MASK)

#define USDHC_CQISGE_RED_SGE_MASK                (0x4U)
#define USDHC_CQISGE_RED_SGE_SHIFT               (2U)
/*! RED_SGE - Response error detected signal enable */
#define USDHC_CQISGE_RED_SGE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISGE_RED_SGE_SHIFT)) & USDHC_CQISGE_RED_SGE_MASK)

#define USDHC_CQISGE_TCL_SGE_MASK                (0x8U)
#define USDHC_CQISGE_TCL_SGE_SHIFT               (3U)
/*! TCL_SGE - Task cleared signal enable */
#define USDHC_CQISGE_TCL_SGE(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQISGE_TCL_SGE_SHIFT)) & USDHC_CQISGE_TCL_SGE_MASK)
/*! @} */

/*! @name CQIC - Command Queuing Interrupt Coalescing */
/*! @{ */

#define USDHC_CQIC_ICTOVAL_MASK                  (0x7FU)
#define USDHC_CQIC_ICTOVAL_SHIFT                 (0U)
/*! ICTOVAL - Interrupt coalescing timeout value */
#define USDHC_CQIC_ICTOVAL(x)                    (((uint32_t)(((uint32_t)(x)) << USDHC_CQIC_ICTOVAL_SHIFT)) & USDHC_CQIC_ICTOVAL_MASK)

#define USDHC_CQIC_ICTOVALWEN_MASK               (0x80U)
#define USDHC_CQIC_ICTOVALWEN_SHIFT              (7U)
/*! ICTOVALWEN - Interrupt coalescing timeout value write enable */
#define USDHC_CQIC_ICTOVALWEN(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_CQIC_ICTOVALWEN_SHIFT)) & USDHC_CQIC_ICTOVALWEN_MASK)

#define USDHC_CQIC_ICCTH_MASK                    (0x1F00U)
#define USDHC_CQIC_ICCTH_SHIFT                   (8U)
/*! ICCTH - Interrupt coalescing counter threshold */
#define USDHC_CQIC_ICCTH(x)                      (((uint32_t)(((uint32_t)(x)) << USDHC_CQIC_ICCTH_SHIFT)) & USDHC_CQIC_ICCTH_MASK)

#define USDHC_CQIC_ICCTHWEN_MASK                 (0x8000U)
#define USDHC_CQIC_ICCTHWEN_SHIFT                (15U)
/*! ICCTHWEN - Interrupt coalescing counter threshold write enable */
#define USDHC_CQIC_ICCTHWEN(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQIC_ICCTHWEN_SHIFT)) & USDHC_CQIC_ICCTHWEN_MASK)

#define USDHC_CQIC_ICCTR_MASK                    (0x10000U)
#define USDHC_CQIC_ICCTR_SHIFT                   (16U)
/*! ICCTR - Counter and timer reset */
#define USDHC_CQIC_ICCTR(x)                      (((uint32_t)(((uint32_t)(x)) << USDHC_CQIC_ICCTR_SHIFT)) & USDHC_CQIC_ICCTR_MASK)

#define USDHC_CQIC_ICSB_MASK                     (0x100000U)
#define USDHC_CQIC_ICSB_SHIFT                    (20U)
/*! ICSB - Interrupt coalescing status
 *  0b0..No task completions have occurred since last counter reset (IC counter =0)
 *  0b1..At least one task completion has been counted (IC counter >0)
 */
#define USDHC_CQIC_ICSB(x)                       (((uint32_t)(((uint32_t)(x)) << USDHC_CQIC_ICSB_SHIFT)) & USDHC_CQIC_ICSB_MASK)

#define USDHC_CQIC_ICENDIS_MASK                  (0x80000000U)
#define USDHC_CQIC_ICENDIS_SHIFT                 (31U)
/*! ICENDIS - Interrupt coalescing enable/disable */
#define USDHC_CQIC_ICENDIS(x)                    (((uint32_t)(((uint32_t)(x)) << USDHC_CQIC_ICENDIS_SHIFT)) & USDHC_CQIC_ICENDIS_MASK)
/*! @} */

/*! @name CQTDLBA - Command Queuing Task Descriptor List Base Address */
/*! @{ */

#define USDHC_CQTDLBA_TDLBA_MASK                 (0xFFFFFFFFU)
#define USDHC_CQTDLBA_TDLBA_SHIFT                (0U)
/*! TDLBA - Task descriptor list base address */
#define USDHC_CQTDLBA_TDLBA(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQTDLBA_TDLBA_SHIFT)) & USDHC_CQTDLBA_TDLBA_MASK)
/*! @} */

/*! @name CQTDLBAU - Command Queuing Task Descriptor List Base Address Upper 32 Bits */
/*! @{ */

#define USDHC_CQTDLBAU_TDLBAU_MASK               (0xFFFFFFFFU)
#define USDHC_CQTDLBAU_TDLBAU_SHIFT              (0U)
/*! TDLBAU - Task descriptor list base address */
#define USDHC_CQTDLBAU_TDLBAU(x)                 (((uint32_t)(((uint32_t)(x)) << USDHC_CQTDLBAU_TDLBAU_SHIFT)) & USDHC_CQTDLBAU_TDLBAU_MASK)
/*! @} */

/*! @name CQTDBR - Command Queuing Task Doorbell */
/*! @{ */

#define USDHC_CQTDBR_TDBR_MASK                   (0xFFFFFFFFU)
#define USDHC_CQTDBR_TDBR_SHIFT                  (0U)
/*! TDBR - Task doorbell */
#define USDHC_CQTDBR_TDBR(x)                     (((uint32_t)(((uint32_t)(x)) << USDHC_CQTDBR_TDBR_SHIFT)) & USDHC_CQTDBR_TDBR_MASK)
/*! @} */

/*! @name CQTCN - Command Queuing Task Completion Notification */
/*! @{ */

#define USDHC_CQTCN_TCN_MASK                     (0xFFFFFFFFU)
#define USDHC_CQTCN_TCN_SHIFT                    (0U)
/*! TCN - Task complete notification */
#define USDHC_CQTCN_TCN(x)                       (((uint32_t)(((uint32_t)(x)) << USDHC_CQTCN_TCN_SHIFT)) & USDHC_CQTCN_TCN_MASK)
/*! @} */

/*! @name CQDQS - Command Queuing Device Queue Status */
/*! @{ */

#define USDHC_CQDQS_DQS_MASK                     (0xFFFFFFFFU)
#define USDHC_CQDQS_DQS_SHIFT                    (0U)
/*! DQS - Device queue status */
#define USDHC_CQDQS_DQS(x)                       (((uint32_t)(((uint32_t)(x)) << USDHC_CQDQS_DQS_SHIFT)) & USDHC_CQDQS_DQS_MASK)
/*! @} */

/*! @name CQDPT - Command Queuing Device Pending Tasks */
/*! @{ */

#define USDHC_CQDPT_DPT_MASK                     (0xFFFFFFFFU)
#define USDHC_CQDPT_DPT_SHIFT                    (0U)
/*! DPT - Device pending tasks */
#define USDHC_CQDPT_DPT(x)                       (((uint32_t)(((uint32_t)(x)) << USDHC_CQDPT_DPT_SHIFT)) & USDHC_CQDPT_DPT_MASK)
/*! @} */

/*! @name CQTCLR - Command Queuing Task Clear */
/*! @{ */

#define USDHC_CQTCLR_TCLR_MASK                   (0xFFFFFFFFU)
#define USDHC_CQTCLR_TCLR_SHIFT                  (0U)
/*! TCLR - Task clear */
#define USDHC_CQTCLR_TCLR(x)                     (((uint32_t)(((uint32_t)(x)) << USDHC_CQTCLR_TCLR_SHIFT)) & USDHC_CQTCLR_TCLR_MASK)
/*! @} */

/*! @name CQSSC1 - Command Queuing Send Status Configuration 1 */
/*! @{ */

#define USDHC_CQSSC1_CIT_MASK                    (0xFFFFU)
#define USDHC_CQSSC1_CIT_SHIFT                   (0U)
/*! CIT - Send status command idle timer */
#define USDHC_CQSSC1_CIT(x)                      (((uint32_t)(((uint32_t)(x)) << USDHC_CQSSC1_CIT_SHIFT)) & USDHC_CQSSC1_CIT_MASK)

#define USDHC_CQSSC1_CBC_MASK                    (0xF0000U)
#define USDHC_CQSSC1_CBC_SHIFT                   (16U)
/*! CBC - Send status command block counter */
#define USDHC_CQSSC1_CBC(x)                      (((uint32_t)(((uint32_t)(x)) << USDHC_CQSSC1_CBC_SHIFT)) & USDHC_CQSSC1_CBC_MASK)
/*! @} */

/*! @name CQSSC2 - Command Queuing Send Status Configuration 2 */
/*! @{ */

#define USDHC_CQSSC2_SSC2_MASK                   (0xFFFFU)
#define USDHC_CQSSC2_SSC2_SHIFT                  (0U)
/*! SSC2 - Send queue status RCA */
#define USDHC_CQSSC2_SSC2(x)                     (((uint32_t)(((uint32_t)(x)) << USDHC_CQSSC2_SSC2_SHIFT)) & USDHC_CQSSC2_SSC2_MASK)
/*! @} */

/*! @name CQCRDCT - Command Queuing Command Response for Direct-Command Task */
/*! @{ */

#define USDHC_CQCRDCT_CRDCT_MASK                 (0xFFFFFFFFU)
#define USDHC_CQCRDCT_CRDCT_SHIFT                (0U)
/*! CRDCT - Direct command last response */
#define USDHC_CQCRDCT_CRDCT(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQCRDCT_CRDCT_SHIFT)) & USDHC_CQCRDCT_CRDCT_MASK)
/*! @} */

/*! @name CQRMEM - Command Queuing Response Mode Error Mask */
/*! @{ */

#define USDHC_CQRMEM_RMEM_MASK                   (0xFFFFFFFFU)
#define USDHC_CQRMEM_RMEM_SHIFT                  (0U)
/*! RMEM - Response mode error mask
 *  0b00000000000000000000000000000000..When a R1/R1b response is received, bit i in the device status is ignored
 *  0b00000000000000000000000000000001..When a R1/R1b response is received, with bit i in the device status set, a RED interrupt is generated
 */
#define USDHC_CQRMEM_RMEM(x)                     (((uint32_t)(((uint32_t)(x)) << USDHC_CQRMEM_RMEM_SHIFT)) & USDHC_CQRMEM_RMEM_MASK)
/*! @} */

/*! @name CQTERRI - Command Queuing Task Error Information */
/*! @{ */

#define USDHC_CQTERRI_RMECI_MASK                 (0x3FU)
#define USDHC_CQTERRI_RMECI_SHIFT                (0U)
/*! RMECI - Response mode error command index */
#define USDHC_CQTERRI_RMECI(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQTERRI_RMECI_SHIFT)) & USDHC_CQTERRI_RMECI_MASK)

#define USDHC_CQTERRI_RMETID_MASK                (0x1F00U)
#define USDHC_CQTERRI_RMETID_SHIFT               (8U)
/*! RMETID - Response mode error task ID */
#define USDHC_CQTERRI_RMETID(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQTERRI_RMETID_SHIFT)) & USDHC_CQTERRI_RMETID_MASK)

#define USDHC_CQTERRI_RMEFV_MASK                 (0x8000U)
#define USDHC_CQTERRI_RMEFV_SHIFT                (15U)
/*! RMEFV - Response mode error fields valid */
#define USDHC_CQTERRI_RMEFV(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQTERRI_RMEFV_SHIFT)) & USDHC_CQTERRI_RMEFV_MASK)

#define USDHC_CQTERRI_DTECI_MASK                 (0x3F0000U)
#define USDHC_CQTERRI_DTECI_SHIFT                (16U)
/*! DTECI - Data transfer error command index */
#define USDHC_CQTERRI_DTECI(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQTERRI_DTECI_SHIFT)) & USDHC_CQTERRI_DTECI_MASK)

#define USDHC_CQTERRI_DTETID_MASK                (0x1F000000U)
#define USDHC_CQTERRI_DTETID_SHIFT               (24U)
/*! DTETID - Data transfer error task ID */
#define USDHC_CQTERRI_DTETID(x)                  (((uint32_t)(((uint32_t)(x)) << USDHC_CQTERRI_DTETID_SHIFT)) & USDHC_CQTERRI_DTETID_MASK)

#define USDHC_CQTERRI_DTEFV_MASK                 (0x80000000U)
#define USDHC_CQTERRI_DTEFV_SHIFT                (31U)
/*! DTEFV - Data transfer error fields valid */
#define USDHC_CQTERRI_DTEFV(x)                   (((uint32_t)(((uint32_t)(x)) << USDHC_CQTERRI_DTEFV_SHIFT)) & USDHC_CQTERRI_DTEFV_MASK)
/*! @} */

/*! @name CQCRI - Command Queuing Command Response Index */
/*! @{ */

#define USDHC_CQCRI_LCMDRI_MASK                  (0x3FU)
#define USDHC_CQCRI_LCMDRI_SHIFT                 (0U)
/*! LCMDRI - Last command response index */
#define USDHC_CQCRI_LCMDRI(x)                    (((uint32_t)(((uint32_t)(x)) << USDHC_CQCRI_LCMDRI_SHIFT)) & USDHC_CQCRI_LCMDRI_MASK)
/*! @} */

/*! @name CQCRA - Command Queuing Command Response Argument */
/*! @{ */

#define USDHC_CQCRA_LCMDRA_MASK                  (0xFFFFFFFFU)
#define USDHC_CQCRA_LCMDRA_SHIFT                 (0U)
/*! LCMDRA - Last command response argument */
#define USDHC_CQCRA_LCMDRA(x)                    (((uint32_t)(((uint32_t)(x)) << USDHC_CQCRA_LCMDRA_SHIFT)) & USDHC_CQCRA_LCMDRA_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group USDHC_Register_Masks */


/* USDHC - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral USDHC1 base address */
  #define USDHC1_BASE                              (0x52850000u)
  /** Peripheral USDHC1 base address */
  #define USDHC1_BASE_NS                           (0x42850000u)
  /** Peripheral USDHC1 base pointer */
  #define USDHC1                                   ((USDHC_Type *)USDHC1_BASE)
  /** Peripheral USDHC1 base pointer */
  #define USDHC1_NS                                ((USDHC_Type *)USDHC1_BASE_NS)
  /** Array initializer of USDHC peripheral base addresses */
  #define USDHC_BASE_ADDRS                         { 0u, USDHC1_BASE }
  /** Array initializer of USDHC peripheral base pointers */
  #define USDHC_BASE_PTRS                          { (USDHC_Type *)0u, USDHC1 }
  /** Array initializer of USDHC peripheral base addresses */
  #define USDHC_BASE_ADDRS_NS                      { 0u, USDHC1_BASE_NS }
  /** Array initializer of USDHC peripheral base pointers */
  #define USDHC_BASE_PTRS_NS                       { (USDHC_Type *)0u, USDHC1_NS }
#else
  /** Peripheral USDHC1 base address */
  #define USDHC1_BASE                              (0x42850000u)
  /** Peripheral USDHC1 base pointer */
  #define USDHC1                                   ((USDHC_Type *)USDHC1_BASE)
  /** Array initializer of USDHC peripheral base addresses */
  #define USDHC_BASE_ADDRS                         { 0u, USDHC1_BASE }
  /** Array initializer of USDHC peripheral base pointers */
  #define USDHC_BASE_PTRS                          { (USDHC_Type *)0u, USDHC1 }
#endif
/** Interrupt vectors for the USDHC peripheral type */
#define USDHC_IRQS                               { NotAvail_IRQn, USDHC1_IRQn }

/*!
 * @}
 */ /* end of group USDHC_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- VMBANDGAP Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup VMBANDGAP_Peripheral_Access_Layer VMBANDGAP Peripheral Access Layer
 * @{
 */

/** VMBANDGAP - Register Layout Typedef */
typedef struct {
  struct {                                         /* offset: 0x0 */
    __IO uint32_t RW;                                /**< Analog Control Register CTRL0, offset: 0x0 */
    __IO uint32_t SET;                               /**< Analog Control Register CTRL0, offset: 0x4 */
    __IO uint32_t CLR;                               /**< Analog Control Register CTRL0, offset: 0x8 */
    __IO uint32_t TOG;                               /**< Analog Control Register CTRL0, offset: 0xC */
  } CTRL0;
       uint8_t RESERVED_0[64];
  struct {                                         /* offset: 0x50 */
    __I  uint32_t RW;                                /**< Analog Status Register STAT0, offset: 0x50 */
    __I  uint32_t SET;                               /**< Analog Status Register STAT0, offset: 0x54 */
    __I  uint32_t CLR;                               /**< Analog Status Register STAT0, offset: 0x58 */
    __I  uint32_t TOG;                               /**< Analog Status Register STAT0, offset: 0x5C */
  } STAT0;
} VMBANDGAP_Type;

/* ----------------------------------------------------------------------------
   -- VMBANDGAP Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup VMBANDGAP_Register_Masks VMBANDGAP Register Masks
 * @{
 */

/*! @name CTRL0 - Analog Control Register CTRL0 */
/*! @{ */

#define VMBANDGAP_CTRL0_REFTOP_PWD_MASK          (0x1U)
#define VMBANDGAP_CTRL0_REFTOP_PWD_SHIFT         (0U)
/*! REFTOP_PWD - Master power-down for bandgap module */
#define VMBANDGAP_CTRL0_REFTOP_PWD(x)            (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_CTRL0_REFTOP_PWD_SHIFT)) & VMBANDGAP_CTRL0_REFTOP_PWD_MASK)

#define VMBANDGAP_CTRL0_REFTOP_LINREGREF_PWD_MASK (0x2U)
#define VMBANDGAP_CTRL0_REFTOP_LINREGREF_PWD_SHIFT (1U)
/*! REFTOP_LINREGREF_PWD - Power-down for bandgap voltage-reference buffer */
#define VMBANDGAP_CTRL0_REFTOP_LINREGREF_PWD(x)  (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_CTRL0_REFTOP_LINREGREF_PWD_SHIFT)) & VMBANDGAP_CTRL0_REFTOP_LINREGREF_PWD_MASK)

#define VMBANDGAP_CTRL0_REFTOP_PWDVBGUP_MASK     (0x4U)
#define VMBANDGAP_CTRL0_REFTOP_PWDVBGUP_SHIFT    (2U)
/*! REFTOP_PWDVBGUP - Power-down VBGUP detector in bandgap */
#define VMBANDGAP_CTRL0_REFTOP_PWDVBGUP(x)       (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_CTRL0_REFTOP_PWDVBGUP_SHIFT)) & VMBANDGAP_CTRL0_REFTOP_PWDVBGUP_MASK)

#define VMBANDGAP_CTRL0_REFTOP_LOWPOWER_MASK     (0x8U)
#define VMBANDGAP_CTRL0_REFTOP_LOWPOWER_SHIFT    (3U)
/*! REFTOP_LOWPOWER - Low-power control bit */
#define VMBANDGAP_CTRL0_REFTOP_LOWPOWER(x)       (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_CTRL0_REFTOP_LOWPOWER_SHIFT)) & VMBANDGAP_CTRL0_REFTOP_LOWPOWER_MASK)

#define VMBANDGAP_CTRL0_REFTOP_SELFBIASOFF_MASK  (0x10U)
#define VMBANDGAP_CTRL0_REFTOP_SELFBIASOFF_SHIFT (4U)
/*! REFTOP_SELFBIASOFF - bandgap self-bias control bit */
#define VMBANDGAP_CTRL0_REFTOP_SELFBIASOFF(x)    (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_CTRL0_REFTOP_SELFBIASOFF_SHIFT)) & VMBANDGAP_CTRL0_REFTOP_SELFBIASOFF_MASK)
/*! @} */

/*! @name STAT0 - Analog Status Register STAT0 */
/*! @{ */

#define VMBANDGAP_STAT0_REFTOP_VBGUP_MASK        (0x1U)
#define VMBANDGAP_STAT0_REFTOP_VBGUP_SHIFT       (0U)
/*! REFTOP_VBGUP - Brief description here */
#define VMBANDGAP_STAT0_REFTOP_VBGUP(x)          (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_STAT0_REFTOP_VBGUP_SHIFT)) & VMBANDGAP_STAT0_REFTOP_VBGUP_MASK)

#define VMBANDGAP_STAT0_VDD1_PORB_MASK           (0x2U)
#define VMBANDGAP_STAT0_VDD1_PORB_SHIFT          (1U)
/*! VDD1_PORB - Brief description here */
#define VMBANDGAP_STAT0_VDD1_PORB(x)             (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_STAT0_VDD1_PORB_SHIFT)) & VMBANDGAP_STAT0_VDD1_PORB_MASK)

#define VMBANDGAP_STAT0_VDD2_PORB_MASK           (0x4U)
#define VMBANDGAP_STAT0_VDD2_PORB_SHIFT          (2U)
/*! VDD2_PORB - Brief description here */
#define VMBANDGAP_STAT0_VDD2_PORB(x)             (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_STAT0_VDD2_PORB_SHIFT)) & VMBANDGAP_STAT0_VDD2_PORB_MASK)

#define VMBANDGAP_STAT0_VDD3_PORB_MASK           (0x8U)
#define VMBANDGAP_STAT0_VDD3_PORB_SHIFT          (3U)
/*! VDD3_PORB - Brief description here */
#define VMBANDGAP_STAT0_VDD3_PORB(x)             (((uint32_t)(((uint32_t)(x)) << VMBANDGAP_STAT0_VDD3_PORB_SHIFT)) & VMBANDGAP_STAT0_VDD3_PORB_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group VMBANDGAP_Register_Masks */


/* VMBANDGAP - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral VMBANDGAP base address */
  #define VMBANDGAP_BASE                           (0x54484780u)
  /** Peripheral VMBANDGAP base address */
  #define VMBANDGAP_BASE_NS                        (0x44484780u)
  /** Peripheral VMBANDGAP base pointer */
  #define VMBANDGAP                                ((VMBANDGAP_Type *)VMBANDGAP_BASE)
  /** Peripheral VMBANDGAP base pointer */
  #define VMBANDGAP_NS                             ((VMBANDGAP_Type *)VMBANDGAP_BASE_NS)
  /** Array initializer of VMBANDGAP peripheral base addresses */
  #define VMBANDGAP_BASE_ADDRS                     { VMBANDGAP_BASE }
  /** Array initializer of VMBANDGAP peripheral base pointers */
  #define VMBANDGAP_BASE_PTRS                      { VMBANDGAP }
  /** Array initializer of VMBANDGAP peripheral base addresses */
  #define VMBANDGAP_BASE_ADDRS_NS                  { VMBANDGAP_BASE_NS }
  /** Array initializer of VMBANDGAP peripheral base pointers */
  #define VMBANDGAP_BASE_PTRS_NS                   { VMBANDGAP_NS }
#else
  /** Peripheral VMBANDGAP base address */
  #define VMBANDGAP_BASE                           (0x44484780u)
  /** Peripheral VMBANDGAP base pointer */
  #define VMBANDGAP                                ((VMBANDGAP_Type *)VMBANDGAP_BASE)
  /** Array initializer of VMBANDGAP peripheral base addresses */
  #define VMBANDGAP_BASE_ADDRS                     { VMBANDGAP_BASE }
  /** Array initializer of VMBANDGAP peripheral base pointers */
  #define VMBANDGAP_BASE_PTRS                      { VMBANDGAP }
#endif

/*!
 * @}
 */ /* end of group VMBANDGAP_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- VREF Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup VREF_Peripheral_Access_Layer VREF Peripheral Access Layer
 * @{
 */

/** VREF - Register Layout Typedef */
typedef struct {
  __I  uint32_t VERID;                             /**< VREF Version ID, offset: 0x0 */
       uint8_t RESERVED_0[4];
  __IO uint32_t CSR;                               /**< VREF Control and Status Register, offset: 0x8 */
       uint8_t RESERVED_1[4];
  __IO uint32_t UTRIM;                             /**< VREF User Trim, offset: 0x10 */
} VREF_Type;

/* ----------------------------------------------------------------------------
   -- VREF Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup VREF_Register_Masks VREF Register Masks
 * @{
 */

/*! @name VERID - VREF Version ID */
/*! @{ */

#define VREF_VERID_FEATURE_MASK                  (0xFFFFU)
#define VREF_VERID_FEATURE_SHIFT                 (0U)
/*! FEATURE - FEATURE */
#define VREF_VERID_FEATURE(x)                    (((uint32_t)(((uint32_t)(x)) << VREF_VERID_FEATURE_SHIFT)) & VREF_VERID_FEATURE_MASK)

#define VREF_VERID_MINOR_MASK                    (0xFF0000U)
#define VREF_VERID_MINOR_SHIFT                   (16U)
/*! MINOR - MINOR */
#define VREF_VERID_MINOR(x)                      (((uint32_t)(((uint32_t)(x)) << VREF_VERID_MINOR_SHIFT)) & VREF_VERID_MINOR_MASK)

#define VREF_VERID_MAJOR_MASK                    (0xFF000000U)
#define VREF_VERID_MAJOR_SHIFT                   (24U)
/*! MAJOR - MAJOR */
#define VREF_VERID_MAJOR(x)                      (((uint32_t)(((uint32_t)(x)) << VREF_VERID_MAJOR_SHIFT)) & VREF_VERID_MAJOR_MASK)
/*! @} */

/*! @name CSR - VREF Control and Status Register */
/*! @{ */

#define VREF_CSR_HCBGEN_MASK                     (0x1U)
#define VREF_CSR_HCBGEN_SHIFT                    (0U)
/*! HCBGEN - High Accuracy Bandgap enabled
 *  0b0..HC Bandgap is disabled
 *  0b1..HC Bandgap is enabled
 */
#define VREF_CSR_HCBGEN(x)                       (((uint32_t)(((uint32_t)(x)) << VREF_CSR_HCBGEN_SHIFT)) & VREF_CSR_HCBGEN_MASK)

#define VREF_CSR_LPBGEN_MASK                     (0x2U)
#define VREF_CSR_LPBGEN_SHIFT                    (1U)
/*! LPBGEN - Low Power Bandgap enable
 *  0b0..LP Bandgap is disabled
 *  0b1..LP Bandgap is enabled
 */
#define VREF_CSR_LPBGEN(x)                       (((uint32_t)(((uint32_t)(x)) << VREF_CSR_LPBGEN_SHIFT)) & VREF_CSR_LPBGEN_MASK)

#define VREF_CSR_CHOPEN_MASK                     (0x8U)
#define VREF_CSR_CHOPEN_SHIFT                    (3U)
/*! CHOPEN - Chop oscillator enable. When set, the internal chopping operation is enabled and the
 *    internal analog offset will be minimized.
 *  0b0..Chop oscillator is disabled.
 *  0b1..Chop oscillator is enabled.
 */
#define VREF_CSR_CHOPEN(x)                       (((uint32_t)(((uint32_t)(x)) << VREF_CSR_CHOPEN_SHIFT)) & VREF_CSR_CHOPEN_MASK)

#define VREF_CSR_ICOMPEN_MASK                    (0x10U)
#define VREF_CSR_ICOMPEN_SHIFT                   (4U)
/*! ICOMPEN - Second order curvature compensation enable
 *  0b0..Disabled
 *  0b1..Enabled
 */
#define VREF_CSR_ICOMPEN(x)                      (((uint32_t)(((uint32_t)(x)) << VREF_CSR_ICOMPEN_SHIFT)) & VREF_CSR_ICOMPEN_MASK)

#define VREF_CSR_REGEN_MASK                      (0x20U)
#define VREF_CSR_REGEN_SHIFT                     (5U)
/*! REGEN - Regulator enable
 *  0b0..Internal 1.75 V regulator is disabled.
 *  0b1..Internal 1.75 V regulator is enabled.
 */
#define VREF_CSR_REGEN(x)                        (((uint32_t)(((uint32_t)(x)) << VREF_CSR_REGEN_SHIFT)) & VREF_CSR_REGEN_MASK)

#define VREF_CSR_HI_PWR_LV_MASK                  (0x800U)
#define VREF_CSR_HI_PWR_LV_SHIFT                 (11U)
/*! HI_PWR_LV
 *  0b0..buffer is in low power mode
 *  0b1..buffer is in high power mode
 */
#define VREF_CSR_HI_PWR_LV(x)                    (((uint32_t)(((uint32_t)(x)) << VREF_CSR_HI_PWR_LV_SHIFT)) & VREF_CSR_HI_PWR_LV_MASK)

#define VREF_CSR_Buf21EN_MASK                    (0x10000U)
#define VREF_CSR_Buf21EN_SHIFT                   (16U)
/*! Buf21EN - Internal buffer enable
 *  0b0..buffer is disabled
 *  0b1..buffer is enabled
 */
#define VREF_CSR_Buf21EN(x)                      (((uint32_t)(((uint32_t)(x)) << VREF_CSR_Buf21EN_SHIFT)) & VREF_CSR_Buf21EN_MASK)

#define VREF_CSR_VREFST_MASK                     (0x80000000U)
#define VREF_CSR_VREFST_SHIFT                    (31U)
/*! VREFST - Internal High Accuracy Voltage Reference stable
 *  0b0..The module is disabled or not stable.
 *  0b1..The module is stable.
 */
#define VREF_CSR_VREFST(x)                       (((uint32_t)(((uint32_t)(x)) << VREF_CSR_VREFST_SHIFT)) & VREF_CSR_VREFST_MASK)
/*! @} */

/*! @name UTRIM - VREF User Trim */
/*! @{ */

#define VREF_UTRIM_VREFTRIM_MASK                 (0x3F00U)
#define VREF_UTRIM_VREFTRIM_SHIFT                (8U)
/*! VREFTRIM - VREF Trim bits
 *  0b000000..Min
 *  0b000001..Max-31*(4/3) mV
 *  0b111111..Max
 */
#define VREF_UTRIM_VREFTRIM(x)                   (((uint32_t)(((uint32_t)(x)) << VREF_UTRIM_VREFTRIM_SHIFT)) & VREF_UTRIM_VREFTRIM_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group VREF_Register_Masks */


/* VREF - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral VREF base address */
  #define VREF_BASE                                (0x52E30000u)
  /** Peripheral VREF base address */
  #define VREF_BASE_NS                             (0x42E30000u)
  /** Peripheral VREF base pointer */
  #define VREF                                     ((VREF_Type *)VREF_BASE)
  /** Peripheral VREF base pointer */
  #define VREF_NS                                  ((VREF_Type *)VREF_BASE_NS)
  /** Array initializer of VREF peripheral base addresses */
  #define VREF_BASE_ADDRS                          { VREF_BASE }
  /** Array initializer of VREF peripheral base pointers */
  #define VREF_BASE_PTRS                           { VREF }
  /** Array initializer of VREF peripheral base addresses */
  #define VREF_BASE_ADDRS_NS                       { VREF_BASE_NS }
  /** Array initializer of VREF peripheral base pointers */
  #define VREF_BASE_PTRS_NS                        { VREF_NS }
#else
  /** Peripheral VREF base address */
  #define VREF_BASE                                (0x42E30000u)
  /** Peripheral VREF base pointer */
  #define VREF                                     ((VREF_Type *)VREF_BASE)
  /** Array initializer of VREF peripheral base addresses */
  #define VREF_BASE_ADDRS                          { VREF_BASE }
  /** Array initializer of VREF peripheral base pointers */
  #define VREF_BASE_PTRS                           { VREF }
#endif

/*!
 * @}
 */ /* end of group VREF_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- XBAR_NUM_OUT221 Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup XBAR_NUM_OUT221_Peripheral_Access_Layer XBAR_NUM_OUT221 Peripheral Access Layer
 * @{
 */

/** XBAR_NUM_OUT221 - Register Layout Typedef */
typedef struct {
  __IO uint16_t SEL[111];                          /**< Crossbar Select Register, array offset: 0x0, array step: 0x2 */
  __IO uint16_t CTRL[2];                           /**< Crossbar Control Register, array offset: 0xDE, array step: 0x2 */
} XBAR_NUM_OUT221_Type;

/* ----------------------------------------------------------------------------
   -- XBAR_NUM_OUT221 Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup XBAR_NUM_OUT221_Register_Masks XBAR_NUM_OUT221 Register Masks
 * @{
 */

/*! @name SEL - Crossbar Select Register */
/*! @{ */

#define XBAR_NUM_OUT221_SEL_SEL0_MASK            (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL0_SHIFT           (0U)
/*! SEL0 - SEL0 */
#define XBAR_NUM_OUT221_SEL_SEL0(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL0_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL0_MASK)

#define XBAR_NUM_OUT221_SEL_SEL2_MASK            (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL2_SHIFT           (0U)
/*! SEL2 - SEL2 */
#define XBAR_NUM_OUT221_SEL_SEL2(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL2_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL2_MASK)

#define XBAR_NUM_OUT221_SEL_SEL4_MASK            (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL4_SHIFT           (0U)
/*! SEL4 - SEL4 */
#define XBAR_NUM_OUT221_SEL_SEL4(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL4_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL4_MASK)

#define XBAR_NUM_OUT221_SEL_SEL6_MASK            (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL6_SHIFT           (0U)
/*! SEL6 - SEL6 */
#define XBAR_NUM_OUT221_SEL_SEL6(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL6_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL6_MASK)

#define XBAR_NUM_OUT221_SEL_SEL8_MASK            (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL8_SHIFT           (0U)
/*! SEL8 - SEL8 */
#define XBAR_NUM_OUT221_SEL_SEL8(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL8_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL8_MASK)

#define XBAR_NUM_OUT221_SEL_SEL10_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL10_SHIFT          (0U)
/*! SEL10 - SEL10 */
#define XBAR_NUM_OUT221_SEL_SEL10(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL10_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL10_MASK)

#define XBAR_NUM_OUT221_SEL_SEL12_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL12_SHIFT          (0U)
/*! SEL12 - SEL12 */
#define XBAR_NUM_OUT221_SEL_SEL12(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL12_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL12_MASK)

#define XBAR_NUM_OUT221_SEL_SEL14_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL14_SHIFT          (0U)
/*! SEL14 - SEL14 */
#define XBAR_NUM_OUT221_SEL_SEL14(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL14_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL14_MASK)

#define XBAR_NUM_OUT221_SEL_SEL16_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL16_SHIFT          (0U)
/*! SEL16 - SEL16 */
#define XBAR_NUM_OUT221_SEL_SEL16(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL16_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL16_MASK)

#define XBAR_NUM_OUT221_SEL_SEL18_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL18_SHIFT          (0U)
/*! SEL18 - SEL18 */
#define XBAR_NUM_OUT221_SEL_SEL18(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL18_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL18_MASK)

#define XBAR_NUM_OUT221_SEL_SEL20_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL20_SHIFT          (0U)
/*! SEL20 - SEL20 */
#define XBAR_NUM_OUT221_SEL_SEL20(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL20_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL20_MASK)

#define XBAR_NUM_OUT221_SEL_SEL22_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL22_SHIFT          (0U)
/*! SEL22 - SEL22 */
#define XBAR_NUM_OUT221_SEL_SEL22(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL22_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL22_MASK)

#define XBAR_NUM_OUT221_SEL_SEL24_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL24_SHIFT          (0U)
/*! SEL24 - SEL24 */
#define XBAR_NUM_OUT221_SEL_SEL24(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL24_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL24_MASK)

#define XBAR_NUM_OUT221_SEL_SEL26_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL26_SHIFT          (0U)
/*! SEL26 - SEL26 */
#define XBAR_NUM_OUT221_SEL_SEL26(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL26_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL26_MASK)

#define XBAR_NUM_OUT221_SEL_SEL28_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL28_SHIFT          (0U)
/*! SEL28 - SEL28 */
#define XBAR_NUM_OUT221_SEL_SEL28(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL28_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL28_MASK)

#define XBAR_NUM_OUT221_SEL_SEL30_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL30_SHIFT          (0U)
/*! SEL30 - SEL30 */
#define XBAR_NUM_OUT221_SEL_SEL30(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL30_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL30_MASK)

#define XBAR_NUM_OUT221_SEL_SEL32_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL32_SHIFT          (0U)
/*! SEL32 - SEL32 */
#define XBAR_NUM_OUT221_SEL_SEL32(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL32_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL32_MASK)

#define XBAR_NUM_OUT221_SEL_SEL34_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL34_SHIFT          (0U)
/*! SEL34 - SEL34 */
#define XBAR_NUM_OUT221_SEL_SEL34(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL34_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL34_MASK)

#define XBAR_NUM_OUT221_SEL_SEL36_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL36_SHIFT          (0U)
/*! SEL36 - SEL36 */
#define XBAR_NUM_OUT221_SEL_SEL36(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL36_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL36_MASK)

#define XBAR_NUM_OUT221_SEL_SEL38_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL38_SHIFT          (0U)
/*! SEL38 - SEL38 */
#define XBAR_NUM_OUT221_SEL_SEL38(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL38_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL38_MASK)

#define XBAR_NUM_OUT221_SEL_SEL40_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL40_SHIFT          (0U)
/*! SEL40 - SEL40 */
#define XBAR_NUM_OUT221_SEL_SEL40(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL40_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL40_MASK)

#define XBAR_NUM_OUT221_SEL_SEL42_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL42_SHIFT          (0U)
/*! SEL42 - SEL42 */
#define XBAR_NUM_OUT221_SEL_SEL42(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL42_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL42_MASK)

#define XBAR_NUM_OUT221_SEL_SEL44_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL44_SHIFT          (0U)
/*! SEL44 - SEL44 */
#define XBAR_NUM_OUT221_SEL_SEL44(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL44_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL44_MASK)

#define XBAR_NUM_OUT221_SEL_SEL46_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL46_SHIFT          (0U)
/*! SEL46 - SEL46 */
#define XBAR_NUM_OUT221_SEL_SEL46(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL46_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL46_MASK)

#define XBAR_NUM_OUT221_SEL_SEL48_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL48_SHIFT          (0U)
/*! SEL48 - SEL48 */
#define XBAR_NUM_OUT221_SEL_SEL48(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL48_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL48_MASK)

#define XBAR_NUM_OUT221_SEL_SEL50_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL50_SHIFT          (0U)
/*! SEL50 - SEL50 */
#define XBAR_NUM_OUT221_SEL_SEL50(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL50_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL50_MASK)

#define XBAR_NUM_OUT221_SEL_SEL52_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL52_SHIFT          (0U)
/*! SEL52 - SEL52 */
#define XBAR_NUM_OUT221_SEL_SEL52(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL52_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL52_MASK)

#define XBAR_NUM_OUT221_SEL_SEL54_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL54_SHIFT          (0U)
/*! SEL54 - SEL54 */
#define XBAR_NUM_OUT221_SEL_SEL54(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL54_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL54_MASK)

#define XBAR_NUM_OUT221_SEL_SEL56_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL56_SHIFT          (0U)
/*! SEL56 - SEL56 */
#define XBAR_NUM_OUT221_SEL_SEL56(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL56_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL56_MASK)

#define XBAR_NUM_OUT221_SEL_SEL58_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL58_SHIFT          (0U)
/*! SEL58 - SEL58 */
#define XBAR_NUM_OUT221_SEL_SEL58(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL58_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL58_MASK)

#define XBAR_NUM_OUT221_SEL_SEL60_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL60_SHIFT          (0U)
/*! SEL60 - SEL60 */
#define XBAR_NUM_OUT221_SEL_SEL60(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL60_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL60_MASK)

#define XBAR_NUM_OUT221_SEL_SEL62_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL62_SHIFT          (0U)
/*! SEL62 - SEL62 */
#define XBAR_NUM_OUT221_SEL_SEL62(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL62_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL62_MASK)

#define XBAR_NUM_OUT221_SEL_SEL64_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL64_SHIFT          (0U)
/*! SEL64 - SEL64 */
#define XBAR_NUM_OUT221_SEL_SEL64(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL64_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL64_MASK)

#define XBAR_NUM_OUT221_SEL_SEL66_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL66_SHIFT          (0U)
/*! SEL66 - SEL66 */
#define XBAR_NUM_OUT221_SEL_SEL66(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL66_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL66_MASK)

#define XBAR_NUM_OUT221_SEL_SEL68_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL68_SHIFT          (0U)
/*! SEL68 - SEL68 */
#define XBAR_NUM_OUT221_SEL_SEL68(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL68_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL68_MASK)

#define XBAR_NUM_OUT221_SEL_SEL70_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL70_SHIFT          (0U)
/*! SEL70 - SEL70 */
#define XBAR_NUM_OUT221_SEL_SEL70(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL70_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL70_MASK)

#define XBAR_NUM_OUT221_SEL_SEL72_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL72_SHIFT          (0U)
/*! SEL72 - SEL72 */
#define XBAR_NUM_OUT221_SEL_SEL72(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL72_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL72_MASK)

#define XBAR_NUM_OUT221_SEL_SEL74_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL74_SHIFT          (0U)
/*! SEL74 - SEL74 */
#define XBAR_NUM_OUT221_SEL_SEL74(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL74_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL74_MASK)

#define XBAR_NUM_OUT221_SEL_SEL76_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL76_SHIFT          (0U)
/*! SEL76 - SEL76 */
#define XBAR_NUM_OUT221_SEL_SEL76(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL76_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL76_MASK)

#define XBAR_NUM_OUT221_SEL_SEL78_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL78_SHIFT          (0U)
/*! SEL78 - SEL78 */
#define XBAR_NUM_OUT221_SEL_SEL78(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL78_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL78_MASK)

#define XBAR_NUM_OUT221_SEL_SEL80_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL80_SHIFT          (0U)
/*! SEL80 - SEL80 */
#define XBAR_NUM_OUT221_SEL_SEL80(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL80_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL80_MASK)

#define XBAR_NUM_OUT221_SEL_SEL82_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL82_SHIFT          (0U)
/*! SEL82 - SEL82 */
#define XBAR_NUM_OUT221_SEL_SEL82(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL82_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL82_MASK)

#define XBAR_NUM_OUT221_SEL_SEL84_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL84_SHIFT          (0U)
/*! SEL84 - SEL84 */
#define XBAR_NUM_OUT221_SEL_SEL84(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL84_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL84_MASK)

#define XBAR_NUM_OUT221_SEL_SEL86_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL86_SHIFT          (0U)
/*! SEL86 - SEL86 */
#define XBAR_NUM_OUT221_SEL_SEL86(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL86_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL86_MASK)

#define XBAR_NUM_OUT221_SEL_SEL88_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL88_SHIFT          (0U)
/*! SEL88 - SEL88 */
#define XBAR_NUM_OUT221_SEL_SEL88(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL88_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL88_MASK)

#define XBAR_NUM_OUT221_SEL_SEL90_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL90_SHIFT          (0U)
/*! SEL90 - SEL90 */
#define XBAR_NUM_OUT221_SEL_SEL90(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL90_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL90_MASK)

#define XBAR_NUM_OUT221_SEL_SEL92_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL92_SHIFT          (0U)
/*! SEL92 - SEL92 */
#define XBAR_NUM_OUT221_SEL_SEL92(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL92_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL92_MASK)

#define XBAR_NUM_OUT221_SEL_SEL94_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL94_SHIFT          (0U)
/*! SEL94 - SEL94 */
#define XBAR_NUM_OUT221_SEL_SEL94(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL94_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL94_MASK)

#define XBAR_NUM_OUT221_SEL_SEL96_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL96_SHIFT          (0U)
/*! SEL96 - SEL96 */
#define XBAR_NUM_OUT221_SEL_SEL96(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL96_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL96_MASK)

#define XBAR_NUM_OUT221_SEL_SEL98_MASK           (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL98_SHIFT          (0U)
/*! SEL98 - SEL98 */
#define XBAR_NUM_OUT221_SEL_SEL98(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL98_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL98_MASK)

#define XBAR_NUM_OUT221_SEL_SEL100_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL100_SHIFT         (0U)
/*! SEL100 - SEL100 */
#define XBAR_NUM_OUT221_SEL_SEL100(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL100_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL100_MASK)

#define XBAR_NUM_OUT221_SEL_SEL102_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL102_SHIFT         (0U)
/*! SEL102 - SEL102 */
#define XBAR_NUM_OUT221_SEL_SEL102(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL102_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL102_MASK)

#define XBAR_NUM_OUT221_SEL_SEL104_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL104_SHIFT         (0U)
/*! SEL104 - SEL104 */
#define XBAR_NUM_OUT221_SEL_SEL104(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL104_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL104_MASK)

#define XBAR_NUM_OUT221_SEL_SEL106_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL106_SHIFT         (0U)
/*! SEL106 - SEL106 */
#define XBAR_NUM_OUT221_SEL_SEL106(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL106_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL106_MASK)

#define XBAR_NUM_OUT221_SEL_SEL108_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL108_SHIFT         (0U)
/*! SEL108 - SEL108 */
#define XBAR_NUM_OUT221_SEL_SEL108(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL108_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL108_MASK)

#define XBAR_NUM_OUT221_SEL_SEL110_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL110_SHIFT         (0U)
/*! SEL110 - SEL110 */
#define XBAR_NUM_OUT221_SEL_SEL110(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL110_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL110_MASK)

#define XBAR_NUM_OUT221_SEL_SEL112_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL112_SHIFT         (0U)
/*! SEL112 - SEL112 */
#define XBAR_NUM_OUT221_SEL_SEL112(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL112_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL112_MASK)

#define XBAR_NUM_OUT221_SEL_SEL114_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL114_SHIFT         (0U)
/*! SEL114 - SEL114 */
#define XBAR_NUM_OUT221_SEL_SEL114(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL114_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL114_MASK)

#define XBAR_NUM_OUT221_SEL_SEL116_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL116_SHIFT         (0U)
/*! SEL116 - SEL116 */
#define XBAR_NUM_OUT221_SEL_SEL116(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL116_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL116_MASK)

#define XBAR_NUM_OUT221_SEL_SEL118_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL118_SHIFT         (0U)
/*! SEL118 - SEL118 */
#define XBAR_NUM_OUT221_SEL_SEL118(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL118_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL118_MASK)

#define XBAR_NUM_OUT221_SEL_SEL120_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL120_SHIFT         (0U)
/*! SEL120 - SEL120 */
#define XBAR_NUM_OUT221_SEL_SEL120(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL120_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL120_MASK)

#define XBAR_NUM_OUT221_SEL_SEL122_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL122_SHIFT         (0U)
/*! SEL122 - SEL122 */
#define XBAR_NUM_OUT221_SEL_SEL122(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL122_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL122_MASK)

#define XBAR_NUM_OUT221_SEL_SEL124_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL124_SHIFT         (0U)
/*! SEL124 - SEL124 */
#define XBAR_NUM_OUT221_SEL_SEL124(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL124_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL124_MASK)

#define XBAR_NUM_OUT221_SEL_SEL126_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL126_SHIFT         (0U)
/*! SEL126 - SEL126 */
#define XBAR_NUM_OUT221_SEL_SEL126(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL126_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL126_MASK)

#define XBAR_NUM_OUT221_SEL_SEL128_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL128_SHIFT         (0U)
/*! SEL128 - SEL128 */
#define XBAR_NUM_OUT221_SEL_SEL128(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL128_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL128_MASK)

#define XBAR_NUM_OUT221_SEL_SEL130_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL130_SHIFT         (0U)
/*! SEL130 - SEL130 */
#define XBAR_NUM_OUT221_SEL_SEL130(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL130_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL130_MASK)

#define XBAR_NUM_OUT221_SEL_SEL132_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL132_SHIFT         (0U)
/*! SEL132 - SEL132 */
#define XBAR_NUM_OUT221_SEL_SEL132(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL132_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL132_MASK)

#define XBAR_NUM_OUT221_SEL_SEL134_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL134_SHIFT         (0U)
/*! SEL134 - SEL134 */
#define XBAR_NUM_OUT221_SEL_SEL134(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL134_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL134_MASK)

#define XBAR_NUM_OUT221_SEL_SEL136_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL136_SHIFT         (0U)
/*! SEL136 - SEL136 */
#define XBAR_NUM_OUT221_SEL_SEL136(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL136_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL136_MASK)

#define XBAR_NUM_OUT221_SEL_SEL138_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL138_SHIFT         (0U)
/*! SEL138 - SEL138 */
#define XBAR_NUM_OUT221_SEL_SEL138(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL138_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL138_MASK)

#define XBAR_NUM_OUT221_SEL_SEL140_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL140_SHIFT         (0U)
/*! SEL140 - SEL140 */
#define XBAR_NUM_OUT221_SEL_SEL140(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL140_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL140_MASK)

#define XBAR_NUM_OUT221_SEL_SEL142_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL142_SHIFT         (0U)
/*! SEL142 - SEL142 */
#define XBAR_NUM_OUT221_SEL_SEL142(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL142_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL142_MASK)

#define XBAR_NUM_OUT221_SEL_SEL144_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL144_SHIFT         (0U)
/*! SEL144 - SEL144 */
#define XBAR_NUM_OUT221_SEL_SEL144(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL144_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL144_MASK)

#define XBAR_NUM_OUT221_SEL_SEL146_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL146_SHIFT         (0U)
/*! SEL146 - SEL146 */
#define XBAR_NUM_OUT221_SEL_SEL146(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL146_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL146_MASK)

#define XBAR_NUM_OUT221_SEL_SEL148_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL148_SHIFT         (0U)
/*! SEL148 - SEL148 */
#define XBAR_NUM_OUT221_SEL_SEL148(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL148_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL148_MASK)

#define XBAR_NUM_OUT221_SEL_SEL150_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL150_SHIFT         (0U)
/*! SEL150 - SEL150 */
#define XBAR_NUM_OUT221_SEL_SEL150(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL150_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL150_MASK)

#define XBAR_NUM_OUT221_SEL_SEL152_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL152_SHIFT         (0U)
/*! SEL152 - SEL152 */
#define XBAR_NUM_OUT221_SEL_SEL152(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL152_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL152_MASK)

#define XBAR_NUM_OUT221_SEL_SEL154_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL154_SHIFT         (0U)
/*! SEL154 - SEL154 */
#define XBAR_NUM_OUT221_SEL_SEL154(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL154_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL154_MASK)

#define XBAR_NUM_OUT221_SEL_SEL156_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL156_SHIFT         (0U)
/*! SEL156 - SEL156 */
#define XBAR_NUM_OUT221_SEL_SEL156(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL156_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL156_MASK)

#define XBAR_NUM_OUT221_SEL_SEL158_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL158_SHIFT         (0U)
/*! SEL158 - SEL158 */
#define XBAR_NUM_OUT221_SEL_SEL158(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL158_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL158_MASK)

#define XBAR_NUM_OUT221_SEL_SEL160_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL160_SHIFT         (0U)
/*! SEL160 - SEL160 */
#define XBAR_NUM_OUT221_SEL_SEL160(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL160_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL160_MASK)

#define XBAR_NUM_OUT221_SEL_SEL162_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL162_SHIFT         (0U)
/*! SEL162 - SEL162 */
#define XBAR_NUM_OUT221_SEL_SEL162(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL162_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL162_MASK)

#define XBAR_NUM_OUT221_SEL_SEL164_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL164_SHIFT         (0U)
/*! SEL164 - SEL164 */
#define XBAR_NUM_OUT221_SEL_SEL164(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL164_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL164_MASK)

#define XBAR_NUM_OUT221_SEL_SEL166_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL166_SHIFT         (0U)
/*! SEL166 - SEL166 */
#define XBAR_NUM_OUT221_SEL_SEL166(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL166_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL166_MASK)

#define XBAR_NUM_OUT221_SEL_SEL168_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL168_SHIFT         (0U)
/*! SEL168 - SEL168 */
#define XBAR_NUM_OUT221_SEL_SEL168(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL168_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL168_MASK)

#define XBAR_NUM_OUT221_SEL_SEL170_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL170_SHIFT         (0U)
/*! SEL170 - SEL170 */
#define XBAR_NUM_OUT221_SEL_SEL170(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL170_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL170_MASK)

#define XBAR_NUM_OUT221_SEL_SEL172_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL172_SHIFT         (0U)
/*! SEL172 - SEL172 */
#define XBAR_NUM_OUT221_SEL_SEL172(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL172_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL172_MASK)

#define XBAR_NUM_OUT221_SEL_SEL174_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL174_SHIFT         (0U)
/*! SEL174 - SEL174 */
#define XBAR_NUM_OUT221_SEL_SEL174(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL174_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL174_MASK)

#define XBAR_NUM_OUT221_SEL_SEL176_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL176_SHIFT         (0U)
/*! SEL176 - SEL176 */
#define XBAR_NUM_OUT221_SEL_SEL176(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL176_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL176_MASK)

#define XBAR_NUM_OUT221_SEL_SEL178_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL178_SHIFT         (0U)
/*! SEL178 - SEL178 */
#define XBAR_NUM_OUT221_SEL_SEL178(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL178_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL178_MASK)

#define XBAR_NUM_OUT221_SEL_SEL180_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL180_SHIFT         (0U)
/*! SEL180 - SEL180 */
#define XBAR_NUM_OUT221_SEL_SEL180(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL180_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL180_MASK)

#define XBAR_NUM_OUT221_SEL_SEL182_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL182_SHIFT         (0U)
/*! SEL182 - SEL182 */
#define XBAR_NUM_OUT221_SEL_SEL182(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL182_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL182_MASK)

#define XBAR_NUM_OUT221_SEL_SEL184_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL184_SHIFT         (0U)
/*! SEL184 - SEL184 */
#define XBAR_NUM_OUT221_SEL_SEL184(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL184_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL184_MASK)

#define XBAR_NUM_OUT221_SEL_SEL186_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL186_SHIFT         (0U)
/*! SEL186 - SEL186 */
#define XBAR_NUM_OUT221_SEL_SEL186(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL186_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL186_MASK)

#define XBAR_NUM_OUT221_SEL_SEL188_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL188_SHIFT         (0U)
/*! SEL188 - SEL188 */
#define XBAR_NUM_OUT221_SEL_SEL188(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL188_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL188_MASK)

#define XBAR_NUM_OUT221_SEL_SEL190_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL190_SHIFT         (0U)
/*! SEL190 - SEL190 */
#define XBAR_NUM_OUT221_SEL_SEL190(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL190_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL190_MASK)

#define XBAR_NUM_OUT221_SEL_SEL192_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL192_SHIFT         (0U)
/*! SEL192 - SEL192 */
#define XBAR_NUM_OUT221_SEL_SEL192(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL192_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL192_MASK)

#define XBAR_NUM_OUT221_SEL_SEL194_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL194_SHIFT         (0U)
/*! SEL194 - SEL194 */
#define XBAR_NUM_OUT221_SEL_SEL194(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL194_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL194_MASK)

#define XBAR_NUM_OUT221_SEL_SEL196_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL196_SHIFT         (0U)
/*! SEL196 - SEL196 */
#define XBAR_NUM_OUT221_SEL_SEL196(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL196_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL196_MASK)

#define XBAR_NUM_OUT221_SEL_SEL198_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL198_SHIFT         (0U)
/*! SEL198 - SEL198 */
#define XBAR_NUM_OUT221_SEL_SEL198(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL198_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL198_MASK)

#define XBAR_NUM_OUT221_SEL_SEL200_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL200_SHIFT         (0U)
/*! SEL200 - SEL200 */
#define XBAR_NUM_OUT221_SEL_SEL200(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL200_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL200_MASK)

#define XBAR_NUM_OUT221_SEL_SEL202_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL202_SHIFT         (0U)
/*! SEL202 - SEL202 */
#define XBAR_NUM_OUT221_SEL_SEL202(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL202_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL202_MASK)

#define XBAR_NUM_OUT221_SEL_SEL204_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL204_SHIFT         (0U)
/*! SEL204 - SEL204 */
#define XBAR_NUM_OUT221_SEL_SEL204(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL204_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL204_MASK)

#define XBAR_NUM_OUT221_SEL_SEL206_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL206_SHIFT         (0U)
/*! SEL206 - SEL206 */
#define XBAR_NUM_OUT221_SEL_SEL206(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL206_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL206_MASK)

#define XBAR_NUM_OUT221_SEL_SEL208_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL208_SHIFT         (0U)
/*! SEL208 - SEL208 */
#define XBAR_NUM_OUT221_SEL_SEL208(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL208_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL208_MASK)

#define XBAR_NUM_OUT221_SEL_SEL210_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL210_SHIFT         (0U)
/*! SEL210 - SEL210 */
#define XBAR_NUM_OUT221_SEL_SEL210(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL210_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL210_MASK)

#define XBAR_NUM_OUT221_SEL_SEL212_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL212_SHIFT         (0U)
/*! SEL212 - SEL212 */
#define XBAR_NUM_OUT221_SEL_SEL212(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL212_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL212_MASK)

#define XBAR_NUM_OUT221_SEL_SEL214_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL214_SHIFT         (0U)
/*! SEL214 - SEL214 */
#define XBAR_NUM_OUT221_SEL_SEL214(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL214_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL214_MASK)

#define XBAR_NUM_OUT221_SEL_SEL216_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL216_SHIFT         (0U)
/*! SEL216 - SEL216 */
#define XBAR_NUM_OUT221_SEL_SEL216(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL216_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL216_MASK)

#define XBAR_NUM_OUT221_SEL_SEL218_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL218_SHIFT         (0U)
/*! SEL218 - SEL218 */
#define XBAR_NUM_OUT221_SEL_SEL218(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL218_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL218_MASK)

#define XBAR_NUM_OUT221_SEL_SEL220_MASK          (0xFFU)
#define XBAR_NUM_OUT221_SEL_SEL220_SHIFT         (0U)
/*! SEL220 - SEL220 */
#define XBAR_NUM_OUT221_SEL_SEL220(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL220_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL220_MASK)

#define XBAR_NUM_OUT221_SEL_SEL1_MASK            (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL1_SHIFT           (8U)
/*! SEL1 - SEL1 */
#define XBAR_NUM_OUT221_SEL_SEL1(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL1_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL1_MASK)

#define XBAR_NUM_OUT221_SEL_SEL3_MASK            (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL3_SHIFT           (8U)
/*! SEL3 - SEL3 */
#define XBAR_NUM_OUT221_SEL_SEL3(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL3_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL3_MASK)

#define XBAR_NUM_OUT221_SEL_SEL5_MASK            (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL5_SHIFT           (8U)
/*! SEL5 - SEL5 */
#define XBAR_NUM_OUT221_SEL_SEL5(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL5_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL5_MASK)

#define XBAR_NUM_OUT221_SEL_SEL7_MASK            (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL7_SHIFT           (8U)
/*! SEL7 - SEL7 */
#define XBAR_NUM_OUT221_SEL_SEL7(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL7_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL7_MASK)

#define XBAR_NUM_OUT221_SEL_SEL9_MASK            (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL9_SHIFT           (8U)
/*! SEL9 - SEL9 */
#define XBAR_NUM_OUT221_SEL_SEL9(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL9_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL9_MASK)

#define XBAR_NUM_OUT221_SEL_SEL11_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL11_SHIFT          (8U)
/*! SEL11 - SEL11 */
#define XBAR_NUM_OUT221_SEL_SEL11(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL11_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL11_MASK)

#define XBAR_NUM_OUT221_SEL_SEL13_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL13_SHIFT          (8U)
/*! SEL13 - SEL13 */
#define XBAR_NUM_OUT221_SEL_SEL13(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL13_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL13_MASK)

#define XBAR_NUM_OUT221_SEL_SEL15_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL15_SHIFT          (8U)
/*! SEL15 - SEL15 */
#define XBAR_NUM_OUT221_SEL_SEL15(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL15_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL15_MASK)

#define XBAR_NUM_OUT221_SEL_SEL17_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL17_SHIFT          (8U)
/*! SEL17 - SEL17 */
#define XBAR_NUM_OUT221_SEL_SEL17(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL17_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL17_MASK)

#define XBAR_NUM_OUT221_SEL_SEL19_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL19_SHIFT          (8U)
/*! SEL19 - SEL19 */
#define XBAR_NUM_OUT221_SEL_SEL19(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL19_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL19_MASK)

#define XBAR_NUM_OUT221_SEL_SEL21_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL21_SHIFT          (8U)
/*! SEL21 - SEL21 */
#define XBAR_NUM_OUT221_SEL_SEL21(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL21_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL21_MASK)

#define XBAR_NUM_OUT221_SEL_SEL23_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL23_SHIFT          (8U)
/*! SEL23 - SEL23 */
#define XBAR_NUM_OUT221_SEL_SEL23(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL23_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL23_MASK)

#define XBAR_NUM_OUT221_SEL_SEL25_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL25_SHIFT          (8U)
/*! SEL25 - SEL25 */
#define XBAR_NUM_OUT221_SEL_SEL25(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL25_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL25_MASK)

#define XBAR_NUM_OUT221_SEL_SEL27_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL27_SHIFT          (8U)
/*! SEL27 - SEL27 */
#define XBAR_NUM_OUT221_SEL_SEL27(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL27_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL27_MASK)

#define XBAR_NUM_OUT221_SEL_SEL29_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL29_SHIFT          (8U)
/*! SEL29 - SEL29 */
#define XBAR_NUM_OUT221_SEL_SEL29(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL29_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL29_MASK)

#define XBAR_NUM_OUT221_SEL_SEL31_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL31_SHIFT          (8U)
/*! SEL31 - SEL31 */
#define XBAR_NUM_OUT221_SEL_SEL31(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL31_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL31_MASK)

#define XBAR_NUM_OUT221_SEL_SEL33_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL33_SHIFT          (8U)
/*! SEL33 - SEL33 */
#define XBAR_NUM_OUT221_SEL_SEL33(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL33_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL33_MASK)

#define XBAR_NUM_OUT221_SEL_SEL35_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL35_SHIFT          (8U)
/*! SEL35 - SEL35 */
#define XBAR_NUM_OUT221_SEL_SEL35(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL35_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL35_MASK)

#define XBAR_NUM_OUT221_SEL_SEL37_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL37_SHIFT          (8U)
/*! SEL37 - SEL37 */
#define XBAR_NUM_OUT221_SEL_SEL37(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL37_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL37_MASK)

#define XBAR_NUM_OUT221_SEL_SEL39_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL39_SHIFT          (8U)
/*! SEL39 - SEL39 */
#define XBAR_NUM_OUT221_SEL_SEL39(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL39_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL39_MASK)

#define XBAR_NUM_OUT221_SEL_SEL41_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL41_SHIFT          (8U)
/*! SEL41 - SEL41 */
#define XBAR_NUM_OUT221_SEL_SEL41(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL41_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL41_MASK)

#define XBAR_NUM_OUT221_SEL_SEL43_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL43_SHIFT          (8U)
/*! SEL43 - SEL43 */
#define XBAR_NUM_OUT221_SEL_SEL43(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL43_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL43_MASK)

#define XBAR_NUM_OUT221_SEL_SEL45_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL45_SHIFT          (8U)
/*! SEL45 - SEL45 */
#define XBAR_NUM_OUT221_SEL_SEL45(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL45_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL45_MASK)

#define XBAR_NUM_OUT221_SEL_SEL47_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL47_SHIFT          (8U)
/*! SEL47 - SEL47 */
#define XBAR_NUM_OUT221_SEL_SEL47(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL47_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL47_MASK)

#define XBAR_NUM_OUT221_SEL_SEL49_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL49_SHIFT          (8U)
/*! SEL49 - SEL49 */
#define XBAR_NUM_OUT221_SEL_SEL49(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL49_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL49_MASK)

#define XBAR_NUM_OUT221_SEL_SEL51_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL51_SHIFT          (8U)
/*! SEL51 - SEL51 */
#define XBAR_NUM_OUT221_SEL_SEL51(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL51_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL51_MASK)

#define XBAR_NUM_OUT221_SEL_SEL53_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL53_SHIFT          (8U)
/*! SEL53 - SEL53 */
#define XBAR_NUM_OUT221_SEL_SEL53(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL53_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL53_MASK)

#define XBAR_NUM_OUT221_SEL_SEL55_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL55_SHIFT          (8U)
/*! SEL55 - SEL55 */
#define XBAR_NUM_OUT221_SEL_SEL55(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL55_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL55_MASK)

#define XBAR_NUM_OUT221_SEL_SEL57_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL57_SHIFT          (8U)
/*! SEL57 - SEL57 */
#define XBAR_NUM_OUT221_SEL_SEL57(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL57_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL57_MASK)

#define XBAR_NUM_OUT221_SEL_SEL59_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL59_SHIFT          (8U)
/*! SEL59 - SEL59 */
#define XBAR_NUM_OUT221_SEL_SEL59(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL59_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL59_MASK)

#define XBAR_NUM_OUT221_SEL_SEL61_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL61_SHIFT          (8U)
/*! SEL61 - SEL61 */
#define XBAR_NUM_OUT221_SEL_SEL61(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL61_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL61_MASK)

#define XBAR_NUM_OUT221_SEL_SEL63_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL63_SHIFT          (8U)
/*! SEL63 - SEL63 */
#define XBAR_NUM_OUT221_SEL_SEL63(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL63_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL63_MASK)

#define XBAR_NUM_OUT221_SEL_SEL65_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL65_SHIFT          (8U)
/*! SEL65 - SEL65 */
#define XBAR_NUM_OUT221_SEL_SEL65(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL65_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL65_MASK)

#define XBAR_NUM_OUT221_SEL_SEL67_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL67_SHIFT          (8U)
/*! SEL67 - SEL67 */
#define XBAR_NUM_OUT221_SEL_SEL67(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL67_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL67_MASK)

#define XBAR_NUM_OUT221_SEL_SEL69_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL69_SHIFT          (8U)
/*! SEL69 - SEL69 */
#define XBAR_NUM_OUT221_SEL_SEL69(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL69_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL69_MASK)

#define XBAR_NUM_OUT221_SEL_SEL71_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL71_SHIFT          (8U)
/*! SEL71 - SEL71 */
#define XBAR_NUM_OUT221_SEL_SEL71(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL71_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL71_MASK)

#define XBAR_NUM_OUT221_SEL_SEL73_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL73_SHIFT          (8U)
/*! SEL73 - SEL73 */
#define XBAR_NUM_OUT221_SEL_SEL73(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL73_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL73_MASK)

#define XBAR_NUM_OUT221_SEL_SEL75_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL75_SHIFT          (8U)
/*! SEL75 - SEL75 */
#define XBAR_NUM_OUT221_SEL_SEL75(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL75_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL75_MASK)

#define XBAR_NUM_OUT221_SEL_SEL77_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL77_SHIFT          (8U)
/*! SEL77 - SEL77 */
#define XBAR_NUM_OUT221_SEL_SEL77(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL77_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL77_MASK)

#define XBAR_NUM_OUT221_SEL_SEL79_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL79_SHIFT          (8U)
/*! SEL79 - SEL79 */
#define XBAR_NUM_OUT221_SEL_SEL79(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL79_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL79_MASK)

#define XBAR_NUM_OUT221_SEL_SEL81_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL81_SHIFT          (8U)
/*! SEL81 - SEL81 */
#define XBAR_NUM_OUT221_SEL_SEL81(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL81_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL81_MASK)

#define XBAR_NUM_OUT221_SEL_SEL83_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL83_SHIFT          (8U)
/*! SEL83 - SEL83 */
#define XBAR_NUM_OUT221_SEL_SEL83(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL83_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL83_MASK)

#define XBAR_NUM_OUT221_SEL_SEL85_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL85_SHIFT          (8U)
/*! SEL85 - SEL85 */
#define XBAR_NUM_OUT221_SEL_SEL85(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL85_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL85_MASK)

#define XBAR_NUM_OUT221_SEL_SEL87_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL87_SHIFT          (8U)
/*! SEL87 - SEL87 */
#define XBAR_NUM_OUT221_SEL_SEL87(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL87_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL87_MASK)

#define XBAR_NUM_OUT221_SEL_SEL89_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL89_SHIFT          (8U)
/*! SEL89 - SEL89 */
#define XBAR_NUM_OUT221_SEL_SEL89(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL89_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL89_MASK)

#define XBAR_NUM_OUT221_SEL_SEL91_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL91_SHIFT          (8U)
/*! SEL91 - SEL91 */
#define XBAR_NUM_OUT221_SEL_SEL91(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL91_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL91_MASK)

#define XBAR_NUM_OUT221_SEL_SEL93_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL93_SHIFT          (8U)
/*! SEL93 - SEL93 */
#define XBAR_NUM_OUT221_SEL_SEL93(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL93_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL93_MASK)

#define XBAR_NUM_OUT221_SEL_SEL95_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL95_SHIFT          (8U)
/*! SEL95 - SEL95 */
#define XBAR_NUM_OUT221_SEL_SEL95(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL95_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL95_MASK)

#define XBAR_NUM_OUT221_SEL_SEL97_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL97_SHIFT          (8U)
/*! SEL97 - SEL97 */
#define XBAR_NUM_OUT221_SEL_SEL97(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL97_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL97_MASK)

#define XBAR_NUM_OUT221_SEL_SEL99_MASK           (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL99_SHIFT          (8U)
/*! SEL99 - SEL99 */
#define XBAR_NUM_OUT221_SEL_SEL99(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL99_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL99_MASK)

#define XBAR_NUM_OUT221_SEL_SEL101_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL101_SHIFT         (8U)
/*! SEL101 - SEL101 */
#define XBAR_NUM_OUT221_SEL_SEL101(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL101_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL101_MASK)

#define XBAR_NUM_OUT221_SEL_SEL103_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL103_SHIFT         (8U)
/*! SEL103 - SEL103 */
#define XBAR_NUM_OUT221_SEL_SEL103(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL103_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL103_MASK)

#define XBAR_NUM_OUT221_SEL_SEL105_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL105_SHIFT         (8U)
/*! SEL105 - SEL105 */
#define XBAR_NUM_OUT221_SEL_SEL105(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL105_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL105_MASK)

#define XBAR_NUM_OUT221_SEL_SEL107_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL107_SHIFT         (8U)
/*! SEL107 - SEL107 */
#define XBAR_NUM_OUT221_SEL_SEL107(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL107_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL107_MASK)

#define XBAR_NUM_OUT221_SEL_SEL109_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL109_SHIFT         (8U)
/*! SEL109 - SEL109 */
#define XBAR_NUM_OUT221_SEL_SEL109(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL109_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL109_MASK)

#define XBAR_NUM_OUT221_SEL_SEL111_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL111_SHIFT         (8U)
/*! SEL111 - SEL111 */
#define XBAR_NUM_OUT221_SEL_SEL111(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL111_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL111_MASK)

#define XBAR_NUM_OUT221_SEL_SEL113_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL113_SHIFT         (8U)
/*! SEL113 - SEL113 */
#define XBAR_NUM_OUT221_SEL_SEL113(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL113_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL113_MASK)

#define XBAR_NUM_OUT221_SEL_SEL115_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL115_SHIFT         (8U)
/*! SEL115 - SEL115 */
#define XBAR_NUM_OUT221_SEL_SEL115(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL115_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL115_MASK)

#define XBAR_NUM_OUT221_SEL_SEL117_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL117_SHIFT         (8U)
/*! SEL117 - SEL117 */
#define XBAR_NUM_OUT221_SEL_SEL117(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL117_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL117_MASK)

#define XBAR_NUM_OUT221_SEL_SEL119_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL119_SHIFT         (8U)
/*! SEL119 - SEL119 */
#define XBAR_NUM_OUT221_SEL_SEL119(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL119_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL119_MASK)

#define XBAR_NUM_OUT221_SEL_SEL121_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL121_SHIFT         (8U)
/*! SEL121 - SEL121 */
#define XBAR_NUM_OUT221_SEL_SEL121(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL121_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL121_MASK)

#define XBAR_NUM_OUT221_SEL_SEL123_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL123_SHIFT         (8U)
/*! SEL123 - SEL123 */
#define XBAR_NUM_OUT221_SEL_SEL123(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL123_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL123_MASK)

#define XBAR_NUM_OUT221_SEL_SEL125_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL125_SHIFT         (8U)
/*! SEL125 - SEL125 */
#define XBAR_NUM_OUT221_SEL_SEL125(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL125_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL125_MASK)

#define XBAR_NUM_OUT221_SEL_SEL127_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL127_SHIFT         (8U)
/*! SEL127 - SEL127 */
#define XBAR_NUM_OUT221_SEL_SEL127(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL127_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL127_MASK)

#define XBAR_NUM_OUT221_SEL_SEL129_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL129_SHIFT         (8U)
/*! SEL129 - SEL129 */
#define XBAR_NUM_OUT221_SEL_SEL129(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL129_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL129_MASK)

#define XBAR_NUM_OUT221_SEL_SEL131_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL131_SHIFT         (8U)
/*! SEL131 - SEL131 */
#define XBAR_NUM_OUT221_SEL_SEL131(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL131_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL131_MASK)

#define XBAR_NUM_OUT221_SEL_SEL133_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL133_SHIFT         (8U)
/*! SEL133 - SEL133 */
#define XBAR_NUM_OUT221_SEL_SEL133(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL133_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL133_MASK)

#define XBAR_NUM_OUT221_SEL_SEL135_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL135_SHIFT         (8U)
/*! SEL135 - SEL135 */
#define XBAR_NUM_OUT221_SEL_SEL135(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL135_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL135_MASK)

#define XBAR_NUM_OUT221_SEL_SEL137_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL137_SHIFT         (8U)
/*! SEL137 - SEL137 */
#define XBAR_NUM_OUT221_SEL_SEL137(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL137_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL137_MASK)

#define XBAR_NUM_OUT221_SEL_SEL139_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL139_SHIFT         (8U)
/*! SEL139 - SEL139 */
#define XBAR_NUM_OUT221_SEL_SEL139(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL139_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL139_MASK)

#define XBAR_NUM_OUT221_SEL_SEL141_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL141_SHIFT         (8U)
/*! SEL141 - SEL141 */
#define XBAR_NUM_OUT221_SEL_SEL141(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL141_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL141_MASK)

#define XBAR_NUM_OUT221_SEL_SEL143_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL143_SHIFT         (8U)
/*! SEL143 - SEL143 */
#define XBAR_NUM_OUT221_SEL_SEL143(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL143_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL143_MASK)

#define XBAR_NUM_OUT221_SEL_SEL145_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL145_SHIFT         (8U)
/*! SEL145 - SEL145 */
#define XBAR_NUM_OUT221_SEL_SEL145(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL145_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL145_MASK)

#define XBAR_NUM_OUT221_SEL_SEL147_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL147_SHIFT         (8U)
/*! SEL147 - SEL147 */
#define XBAR_NUM_OUT221_SEL_SEL147(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL147_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL147_MASK)

#define XBAR_NUM_OUT221_SEL_SEL149_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL149_SHIFT         (8U)
/*! SEL149 - SEL149 */
#define XBAR_NUM_OUT221_SEL_SEL149(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL149_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL149_MASK)

#define XBAR_NUM_OUT221_SEL_SEL151_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL151_SHIFT         (8U)
/*! SEL151 - SEL151 */
#define XBAR_NUM_OUT221_SEL_SEL151(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL151_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL151_MASK)

#define XBAR_NUM_OUT221_SEL_SEL153_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL153_SHIFT         (8U)
/*! SEL153 - SEL153 */
#define XBAR_NUM_OUT221_SEL_SEL153(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL153_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL153_MASK)

#define XBAR_NUM_OUT221_SEL_SEL155_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL155_SHIFT         (8U)
/*! SEL155 - SEL155 */
#define XBAR_NUM_OUT221_SEL_SEL155(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL155_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL155_MASK)

#define XBAR_NUM_OUT221_SEL_SEL157_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL157_SHIFT         (8U)
/*! SEL157 - SEL157 */
#define XBAR_NUM_OUT221_SEL_SEL157(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL157_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL157_MASK)

#define XBAR_NUM_OUT221_SEL_SEL159_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL159_SHIFT         (8U)
/*! SEL159 - SEL159 */
#define XBAR_NUM_OUT221_SEL_SEL159(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL159_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL159_MASK)

#define XBAR_NUM_OUT221_SEL_SEL161_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL161_SHIFT         (8U)
/*! SEL161 - SEL161 */
#define XBAR_NUM_OUT221_SEL_SEL161(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL161_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL161_MASK)

#define XBAR_NUM_OUT221_SEL_SEL163_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL163_SHIFT         (8U)
/*! SEL163 - SEL163 */
#define XBAR_NUM_OUT221_SEL_SEL163(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL163_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL163_MASK)

#define XBAR_NUM_OUT221_SEL_SEL165_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL165_SHIFT         (8U)
/*! SEL165 - SEL165 */
#define XBAR_NUM_OUT221_SEL_SEL165(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL165_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL165_MASK)

#define XBAR_NUM_OUT221_SEL_SEL167_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL167_SHIFT         (8U)
/*! SEL167 - SEL167 */
#define XBAR_NUM_OUT221_SEL_SEL167(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL167_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL167_MASK)

#define XBAR_NUM_OUT221_SEL_SEL169_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL169_SHIFT         (8U)
/*! SEL169 - SEL169 */
#define XBAR_NUM_OUT221_SEL_SEL169(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL169_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL169_MASK)

#define XBAR_NUM_OUT221_SEL_SEL171_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL171_SHIFT         (8U)
/*! SEL171 - SEL171 */
#define XBAR_NUM_OUT221_SEL_SEL171(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL171_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL171_MASK)

#define XBAR_NUM_OUT221_SEL_SEL173_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL173_SHIFT         (8U)
/*! SEL173 - SEL173 */
#define XBAR_NUM_OUT221_SEL_SEL173(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL173_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL173_MASK)

#define XBAR_NUM_OUT221_SEL_SEL175_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL175_SHIFT         (8U)
/*! SEL175 - SEL175 */
#define XBAR_NUM_OUT221_SEL_SEL175(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL175_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL175_MASK)

#define XBAR_NUM_OUT221_SEL_SEL177_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL177_SHIFT         (8U)
/*! SEL177 - SEL177 */
#define XBAR_NUM_OUT221_SEL_SEL177(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL177_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL177_MASK)

#define XBAR_NUM_OUT221_SEL_SEL179_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL179_SHIFT         (8U)
/*! SEL179 - SEL179 */
#define XBAR_NUM_OUT221_SEL_SEL179(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL179_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL179_MASK)

#define XBAR_NUM_OUT221_SEL_SEL181_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL181_SHIFT         (8U)
/*! SEL181 - SEL181 */
#define XBAR_NUM_OUT221_SEL_SEL181(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL181_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL181_MASK)

#define XBAR_NUM_OUT221_SEL_SEL183_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL183_SHIFT         (8U)
/*! SEL183 - SEL183 */
#define XBAR_NUM_OUT221_SEL_SEL183(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL183_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL183_MASK)

#define XBAR_NUM_OUT221_SEL_SEL185_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL185_SHIFT         (8U)
/*! SEL185 - SEL185 */
#define XBAR_NUM_OUT221_SEL_SEL185(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL185_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL185_MASK)

#define XBAR_NUM_OUT221_SEL_SEL187_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL187_SHIFT         (8U)
/*! SEL187 - SEL187 */
#define XBAR_NUM_OUT221_SEL_SEL187(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL187_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL187_MASK)

#define XBAR_NUM_OUT221_SEL_SEL189_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL189_SHIFT         (8U)
/*! SEL189 - SEL189 */
#define XBAR_NUM_OUT221_SEL_SEL189(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL189_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL189_MASK)

#define XBAR_NUM_OUT221_SEL_SEL191_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL191_SHIFT         (8U)
/*! SEL191 - SEL191 */
#define XBAR_NUM_OUT221_SEL_SEL191(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL191_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL191_MASK)

#define XBAR_NUM_OUT221_SEL_SEL193_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL193_SHIFT         (8U)
/*! SEL193 - SEL193 */
#define XBAR_NUM_OUT221_SEL_SEL193(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL193_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL193_MASK)

#define XBAR_NUM_OUT221_SEL_SEL195_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL195_SHIFT         (8U)
/*! SEL195 - SEL195 */
#define XBAR_NUM_OUT221_SEL_SEL195(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL195_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL195_MASK)

#define XBAR_NUM_OUT221_SEL_SEL197_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL197_SHIFT         (8U)
/*! SEL197 - SEL197 */
#define XBAR_NUM_OUT221_SEL_SEL197(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL197_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL197_MASK)

#define XBAR_NUM_OUT221_SEL_SEL199_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL199_SHIFT         (8U)
/*! SEL199 - SEL199 */
#define XBAR_NUM_OUT221_SEL_SEL199(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL199_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL199_MASK)

#define XBAR_NUM_OUT221_SEL_SEL201_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL201_SHIFT         (8U)
/*! SEL201 - SEL201 */
#define XBAR_NUM_OUT221_SEL_SEL201(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL201_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL201_MASK)

#define XBAR_NUM_OUT221_SEL_SEL203_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL203_SHIFT         (8U)
/*! SEL203 - SEL203 */
#define XBAR_NUM_OUT221_SEL_SEL203(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL203_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL203_MASK)

#define XBAR_NUM_OUT221_SEL_SEL205_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL205_SHIFT         (8U)
/*! SEL205 - SEL205 */
#define XBAR_NUM_OUT221_SEL_SEL205(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL205_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL205_MASK)

#define XBAR_NUM_OUT221_SEL_SEL207_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL207_SHIFT         (8U)
/*! SEL207 - SEL207 */
#define XBAR_NUM_OUT221_SEL_SEL207(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL207_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL207_MASK)

#define XBAR_NUM_OUT221_SEL_SEL209_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL209_SHIFT         (8U)
/*! SEL209 - SEL209 */
#define XBAR_NUM_OUT221_SEL_SEL209(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL209_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL209_MASK)

#define XBAR_NUM_OUT221_SEL_SEL211_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL211_SHIFT         (8U)
/*! SEL211 - SEL211 */
#define XBAR_NUM_OUT221_SEL_SEL211(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL211_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL211_MASK)

#define XBAR_NUM_OUT221_SEL_SEL213_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL213_SHIFT         (8U)
/*! SEL213 - SEL213 */
#define XBAR_NUM_OUT221_SEL_SEL213(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL213_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL213_MASK)

#define XBAR_NUM_OUT221_SEL_SEL215_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL215_SHIFT         (8U)
/*! SEL215 - SEL215 */
#define XBAR_NUM_OUT221_SEL_SEL215(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL215_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL215_MASK)

#define XBAR_NUM_OUT221_SEL_SEL217_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL217_SHIFT         (8U)
/*! SEL217 - SEL217 */
#define XBAR_NUM_OUT221_SEL_SEL217(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL217_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL217_MASK)

#define XBAR_NUM_OUT221_SEL_SEL219_MASK          (0xFF00U)
#define XBAR_NUM_OUT221_SEL_SEL219_SHIFT         (8U)
/*! SEL219 - SEL219 */
#define XBAR_NUM_OUT221_SEL_SEL219(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_SEL_SEL219_SHIFT)) & XBAR_NUM_OUT221_SEL_SEL219_MASK)
/*! @} */

/* The count of XBAR_NUM_OUT221_SEL */
#define XBAR_NUM_OUT221_SEL_COUNT                (111U)

/*! @name CTRL - Crossbar Control Register */
/*! @{ */

#define XBAR_NUM_OUT221_CTRL_DEN0_MASK           (0x1U)
#define XBAR_NUM_OUT221_CTRL_DEN0_SHIFT          (0U)
/*! DEN0 - DMA Enable for XBAR_OUT0
 *  0b0..DMA disabled
 *  0b1..DMA enabled
 */
#define XBAR_NUM_OUT221_CTRL_DEN0(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_DEN0_SHIFT)) & XBAR_NUM_OUT221_CTRL_DEN0_MASK)

#define XBAR_NUM_OUT221_CTRL_DEN2_MASK           (0x1U)
#define XBAR_NUM_OUT221_CTRL_DEN2_SHIFT          (0U)
/*! DEN2 - DMA Enable for XBAR_OUT2
 *  0b0..DMA disabled
 *  0b1..DMA enabled
 */
#define XBAR_NUM_OUT221_CTRL_DEN2(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_DEN2_SHIFT)) & XBAR_NUM_OUT221_CTRL_DEN2_MASK)

#define XBAR_NUM_OUT221_CTRL_IEN0_MASK           (0x2U)
#define XBAR_NUM_OUT221_CTRL_IEN0_SHIFT          (1U)
/*! IEN0 - Interrupt Enable for XBAR_OUT0
 *  0b0..Interrupt disabled
 *  0b1..Interrupt enabled
 */
#define XBAR_NUM_OUT221_CTRL_IEN0(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_IEN0_SHIFT)) & XBAR_NUM_OUT221_CTRL_IEN0_MASK)

#define XBAR_NUM_OUT221_CTRL_IEN2_MASK           (0x2U)
#define XBAR_NUM_OUT221_CTRL_IEN2_SHIFT          (1U)
/*! IEN2 - Interrupt Enable for XBAR_OUT2
 *  0b0..Interrupt disabled
 *  0b1..Interrupt enabled
 */
#define XBAR_NUM_OUT221_CTRL_IEN2(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_IEN2_SHIFT)) & XBAR_NUM_OUT221_CTRL_IEN2_MASK)

#define XBAR_NUM_OUT221_CTRL_EDGE0_MASK          (0xCU)
#define XBAR_NUM_OUT221_CTRL_EDGE0_SHIFT         (2U)
/*! EDGE0 - Active edge for edge detection on XBAR_OUT0
 *  0b00..STS0 never asserts
 *  0b01..STS0 asserts on rising edges of XBAR_OUT0
 *  0b10..STS0 asserts on falling edges of XBAR_OUT0
 *  0b11..STS0 asserts on rising and falling edges of XBAR_OUT0
 */
#define XBAR_NUM_OUT221_CTRL_EDGE0(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_EDGE0_SHIFT)) & XBAR_NUM_OUT221_CTRL_EDGE0_MASK)

#define XBAR_NUM_OUT221_CTRL_EDGE2_MASK          (0xCU)
#define XBAR_NUM_OUT221_CTRL_EDGE2_SHIFT         (2U)
/*! EDGE2 - Active edge for edge detection on XBAR_OUT2
 *  0b00..STS2 never asserts
 *  0b01..STS2 asserts on rising edges of XBAR_OUT2
 *  0b10..STS2 asserts on falling edges of XBAR_OUT2
 *  0b11..STS2 asserts on rising and falling edges of XBAR_OUT2
 */
#define XBAR_NUM_OUT221_CTRL_EDGE2(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_EDGE2_SHIFT)) & XBAR_NUM_OUT221_CTRL_EDGE2_MASK)

#define XBAR_NUM_OUT221_CTRL_STS0_MASK           (0x10U)
#define XBAR_NUM_OUT221_CTRL_STS0_SHIFT          (4U)
/*! STS0 - Edge detection status for XBAR_OUT0
 *  0b0..Active edge not yet detected on XBAR_OUT0
 *  0b1..Active edge detected on XBAR_OUT0
 */
#define XBAR_NUM_OUT221_CTRL_STS0(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_STS0_SHIFT)) & XBAR_NUM_OUT221_CTRL_STS0_MASK)

#define XBAR_NUM_OUT221_CTRL_STS2_MASK           (0x10U)
#define XBAR_NUM_OUT221_CTRL_STS2_SHIFT          (4U)
/*! STS2 - Edge detection status for XBAR_OUT2
 *  0b0..Active edge not yet detected on XBAR_OUT2
 *  0b1..Active edge detected on XBAR_OUT2
 */
#define XBAR_NUM_OUT221_CTRL_STS2(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_STS2_SHIFT)) & XBAR_NUM_OUT221_CTRL_STS2_MASK)

#define XBAR_NUM_OUT221_CTRL_DEN1_MASK           (0x100U)
#define XBAR_NUM_OUT221_CTRL_DEN1_SHIFT          (8U)
/*! DEN1 - DMA Enable for XBAR_OUT1
 *  0b0..DMA disabled
 *  0b1..DMA enabled
 */
#define XBAR_NUM_OUT221_CTRL_DEN1(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_DEN1_SHIFT)) & XBAR_NUM_OUT221_CTRL_DEN1_MASK)

#define XBAR_NUM_OUT221_CTRL_DEN3_MASK           (0x100U)
#define XBAR_NUM_OUT221_CTRL_DEN3_SHIFT          (8U)
/*! DEN3 - DMA Enable for XBAR_OUT3
 *  0b0..DMA disabled
 *  0b1..DMA enabled
 */
#define XBAR_NUM_OUT221_CTRL_DEN3(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_DEN3_SHIFT)) & XBAR_NUM_OUT221_CTRL_DEN3_MASK)

#define XBAR_NUM_OUT221_CTRL_IEN1_MASK           (0x200U)
#define XBAR_NUM_OUT221_CTRL_IEN1_SHIFT          (9U)
/*! IEN1 - Interrupt Enable for XBAR_OUT1
 *  0b0..Interrupt disabled
 *  0b1..Interrupt enabled
 */
#define XBAR_NUM_OUT221_CTRL_IEN1(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_IEN1_SHIFT)) & XBAR_NUM_OUT221_CTRL_IEN1_MASK)

#define XBAR_NUM_OUT221_CTRL_IEN3_MASK           (0x200U)
#define XBAR_NUM_OUT221_CTRL_IEN3_SHIFT          (9U)
/*! IEN3 - Interrupt Enable for XBAR_OUT3
 *  0b0..Interrupt disabled
 *  0b1..Interrupt enabled
 */
#define XBAR_NUM_OUT221_CTRL_IEN3(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_IEN3_SHIFT)) & XBAR_NUM_OUT221_CTRL_IEN3_MASK)

#define XBAR_NUM_OUT221_CTRL_EDGE1_MASK          (0xC00U)
#define XBAR_NUM_OUT221_CTRL_EDGE1_SHIFT         (10U)
/*! EDGE1 - Active edge for edge detection on XBAR_OUT1
 *  0b00..STS1 never asserts
 *  0b01..STS1 asserts on rising edges of XBAR_OUT1
 *  0b10..STS1 asserts on falling edges of XBAR_OUT1
 *  0b11..STS1 asserts on rising and falling edges of XBAR_OUT1
 */
#define XBAR_NUM_OUT221_CTRL_EDGE1(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_EDGE1_SHIFT)) & XBAR_NUM_OUT221_CTRL_EDGE1_MASK)

#define XBAR_NUM_OUT221_CTRL_EDGE3_MASK          (0xC00U)
#define XBAR_NUM_OUT221_CTRL_EDGE3_SHIFT         (10U)
/*! EDGE3 - Active edge for edge detection on XBAR_OUT3
 *  0b00..STS3 never asserts
 *  0b01..STS3 asserts on rising edges of XBAR_OUT3
 *  0b10..STS3 asserts on falling edges of XBAR_OUT3
 *  0b11..STS3 asserts on rising and falling edges of XBAR_OUT3
 */
#define XBAR_NUM_OUT221_CTRL_EDGE3(x)            (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_EDGE3_SHIFT)) & XBAR_NUM_OUT221_CTRL_EDGE3_MASK)

#define XBAR_NUM_OUT221_CTRL_STS1_MASK           (0x1000U)
#define XBAR_NUM_OUT221_CTRL_STS1_SHIFT          (12U)
/*! STS1 - Edge detection status for XBAR_OUT1
 *  0b0..Active edge not yet detected on XBAR_OUT1
 *  0b1..Active edge detected on XBAR_OUT1
 */
#define XBAR_NUM_OUT221_CTRL_STS1(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_STS1_SHIFT)) & XBAR_NUM_OUT221_CTRL_STS1_MASK)

#define XBAR_NUM_OUT221_CTRL_STS3_MASK           (0x1000U)
#define XBAR_NUM_OUT221_CTRL_STS3_SHIFT          (12U)
/*! STS3 - Edge detection status for XBAR_OUT3
 *  0b0..Active edge not yet detected on XBAR_OUT3
 *  0b1..Active edge detected on XBAR_OUT3
 */
#define XBAR_NUM_OUT221_CTRL_STS3(x)             (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT221_CTRL_STS3_SHIFT)) & XBAR_NUM_OUT221_CTRL_STS3_MASK)
/*! @} */

/* The count of XBAR_NUM_OUT221_CTRL */
#define XBAR_NUM_OUT221_CTRL_COUNT               (2U)


/*!
 * @}
 */ /* end of group XBAR_NUM_OUT221_Register_Masks */


/* XBAR_NUM_OUT221 - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral XBAR1 base address */
  #define XBAR1_BASE                               (0x52750000u)
  /** Peripheral XBAR1 base address */
  #define XBAR1_BASE_NS                            (0x42750000u)
  /** Peripheral XBAR1 base pointer */
  #define XBAR1                                    ((XBAR_NUM_OUT221_Type *)XBAR1_BASE)
  /** Peripheral XBAR1 base pointer */
  #define XBAR1_NS                                 ((XBAR_NUM_OUT221_Type *)XBAR1_BASE_NS)
  /** Array initializer of XBAR_NUM_OUT221 peripheral base addresses */
  #define XBAR_NUM_OUT221_BASE_ADDRS               { XBAR1_BASE }
  /** Array initializer of XBAR_NUM_OUT221 peripheral base pointers */
  #define XBAR_NUM_OUT221_BASE_PTRS                { XBAR1 }
  /** Array initializer of XBAR_NUM_OUT221 peripheral base addresses */
  #define XBAR_NUM_OUT221_BASE_ADDRS_NS            { XBAR1_BASE_NS }
  /** Array initializer of XBAR_NUM_OUT221 peripheral base pointers */
  #define XBAR_NUM_OUT221_BASE_PTRS_NS             { XBAR1_NS }
#else
  /** Peripheral XBAR1 base address */
  #define XBAR1_BASE                               (0x42750000u)
  /** Peripheral XBAR1 base pointer */
  #define XBAR1                                    ((XBAR_NUM_OUT221_Type *)XBAR1_BASE)
  /** Array initializer of XBAR_NUM_OUT221 peripheral base addresses */
  #define XBAR_NUM_OUT221_BASE_ADDRS               { XBAR1_BASE }
  /** Array initializer of XBAR_NUM_OUT221 peripheral base pointers */
  #define XBAR_NUM_OUT221_BASE_PTRS                { XBAR1 }
#endif

/*!
 * @}
 */ /* end of group XBAR_NUM_OUT221_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- XBAR_NUM_OUT32 Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup XBAR_NUM_OUT32_Peripheral_Access_Layer XBAR_NUM_OUT32 Peripheral Access Layer
 * @{
 */

/** XBAR_NUM_OUT32 - Register Layout Typedef */
typedef struct {
  __IO uint16_t SEL[16];                           /**< Crossbar Select Register, array offset: 0x0, array step: 0x2 */
} XBAR_NUM_OUT32_Type;

/* ----------------------------------------------------------------------------
   -- XBAR_NUM_OUT32 Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup XBAR_NUM_OUT32_Register_Masks XBAR_NUM_OUT32 Register Masks
 * @{
 */

/*! @name SEL - Crossbar Select Register */
/*! @{ */

#define XBAR_NUM_OUT32_SEL_SEL0_MASK             (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL0_SHIFT            (0U)
/*! SEL0 - SEL0 */
#define XBAR_NUM_OUT32_SEL_SEL0(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL0_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL0_MASK)

#define XBAR_NUM_OUT32_SEL_SEL2_MASK             (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL2_SHIFT            (0U)
/*! SEL2 - SEL2 */
#define XBAR_NUM_OUT32_SEL_SEL2(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL2_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL2_MASK)

#define XBAR_NUM_OUT32_SEL_SEL4_MASK             (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL4_SHIFT            (0U)
/*! SEL4 - SEL4 */
#define XBAR_NUM_OUT32_SEL_SEL4(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL4_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL4_MASK)

#define XBAR_NUM_OUT32_SEL_SEL6_MASK             (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL6_SHIFT            (0U)
/*! SEL6 - SEL6 */
#define XBAR_NUM_OUT32_SEL_SEL6(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL6_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL6_MASK)

#define XBAR_NUM_OUT32_SEL_SEL8_MASK             (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL8_SHIFT            (0U)
/*! SEL8 - SEL8 */
#define XBAR_NUM_OUT32_SEL_SEL8(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL8_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL8_MASK)

#define XBAR_NUM_OUT32_SEL_SEL10_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL10_SHIFT           (0U)
/*! SEL10 - SEL10 */
#define XBAR_NUM_OUT32_SEL_SEL10(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL10_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL10_MASK)

#define XBAR_NUM_OUT32_SEL_SEL12_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL12_SHIFT           (0U)
/*! SEL12 - SEL12 */
#define XBAR_NUM_OUT32_SEL_SEL12(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL12_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL12_MASK)

#define XBAR_NUM_OUT32_SEL_SEL14_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL14_SHIFT           (0U)
/*! SEL14 - SEL14 */
#define XBAR_NUM_OUT32_SEL_SEL14(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL14_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL14_MASK)

#define XBAR_NUM_OUT32_SEL_SEL16_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL16_SHIFT           (0U)
/*! SEL16 - SEL16 */
#define XBAR_NUM_OUT32_SEL_SEL16(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL16_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL16_MASK)

#define XBAR_NUM_OUT32_SEL_SEL18_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL18_SHIFT           (0U)
/*! SEL18 - SEL18 */
#define XBAR_NUM_OUT32_SEL_SEL18(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL18_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL18_MASK)

#define XBAR_NUM_OUT32_SEL_SEL20_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL20_SHIFT           (0U)
/*! SEL20 - SEL20 */
#define XBAR_NUM_OUT32_SEL_SEL20(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL20_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL20_MASK)

#define XBAR_NUM_OUT32_SEL_SEL22_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL22_SHIFT           (0U)
/*! SEL22 - SEL22 */
#define XBAR_NUM_OUT32_SEL_SEL22(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL22_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL22_MASK)

#define XBAR_NUM_OUT32_SEL_SEL24_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL24_SHIFT           (0U)
/*! SEL24 - SEL24 */
#define XBAR_NUM_OUT32_SEL_SEL24(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL24_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL24_MASK)

#define XBAR_NUM_OUT32_SEL_SEL26_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL26_SHIFT           (0U)
/*! SEL26 - SEL26 */
#define XBAR_NUM_OUT32_SEL_SEL26(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL26_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL26_MASK)

#define XBAR_NUM_OUT32_SEL_SEL28_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL28_SHIFT           (0U)
/*! SEL28 - SEL28 */
#define XBAR_NUM_OUT32_SEL_SEL28(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL28_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL28_MASK)

#define XBAR_NUM_OUT32_SEL_SEL30_MASK            (0xFFU)
#define XBAR_NUM_OUT32_SEL_SEL30_SHIFT           (0U)
/*! SEL30 - SEL30 */
#define XBAR_NUM_OUT32_SEL_SEL30(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL30_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL30_MASK)

#define XBAR_NUM_OUT32_SEL_SEL1_MASK             (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL1_SHIFT            (8U)
/*! SEL1 - SEL1 */
#define XBAR_NUM_OUT32_SEL_SEL1(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL1_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL1_MASK)

#define XBAR_NUM_OUT32_SEL_SEL3_MASK             (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL3_SHIFT            (8U)
/*! SEL3 - SEL3 */
#define XBAR_NUM_OUT32_SEL_SEL3(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL3_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL3_MASK)

#define XBAR_NUM_OUT32_SEL_SEL5_MASK             (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL5_SHIFT            (8U)
/*! SEL5 - SEL5 */
#define XBAR_NUM_OUT32_SEL_SEL5(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL5_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL5_MASK)

#define XBAR_NUM_OUT32_SEL_SEL7_MASK             (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL7_SHIFT            (8U)
/*! SEL7 - SEL7 */
#define XBAR_NUM_OUT32_SEL_SEL7(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL7_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL7_MASK)

#define XBAR_NUM_OUT32_SEL_SEL9_MASK             (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL9_SHIFT            (8U)
/*! SEL9 - SEL9 */
#define XBAR_NUM_OUT32_SEL_SEL9(x)               (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL9_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL9_MASK)

#define XBAR_NUM_OUT32_SEL_SEL11_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL11_SHIFT           (8U)
/*! SEL11 - SEL11 */
#define XBAR_NUM_OUT32_SEL_SEL11(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL11_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL11_MASK)

#define XBAR_NUM_OUT32_SEL_SEL13_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL13_SHIFT           (8U)
/*! SEL13 - SEL13 */
#define XBAR_NUM_OUT32_SEL_SEL13(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL13_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL13_MASK)

#define XBAR_NUM_OUT32_SEL_SEL15_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL15_SHIFT           (8U)
/*! SEL15 - SEL15 */
#define XBAR_NUM_OUT32_SEL_SEL15(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL15_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL15_MASK)

#define XBAR_NUM_OUT32_SEL_SEL17_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL17_SHIFT           (8U)
/*! SEL17 - SEL17 */
#define XBAR_NUM_OUT32_SEL_SEL17(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL17_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL17_MASK)

#define XBAR_NUM_OUT32_SEL_SEL19_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL19_SHIFT           (8U)
/*! SEL19 - SEL19 */
#define XBAR_NUM_OUT32_SEL_SEL19(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL19_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL19_MASK)

#define XBAR_NUM_OUT32_SEL_SEL21_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL21_SHIFT           (8U)
/*! SEL21 - SEL21 */
#define XBAR_NUM_OUT32_SEL_SEL21(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL21_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL21_MASK)

#define XBAR_NUM_OUT32_SEL_SEL23_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL23_SHIFT           (8U)
/*! SEL23 - SEL23 */
#define XBAR_NUM_OUT32_SEL_SEL23(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL23_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL23_MASK)

#define XBAR_NUM_OUT32_SEL_SEL25_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL25_SHIFT           (8U)
/*! SEL25 - SEL25 */
#define XBAR_NUM_OUT32_SEL_SEL25(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL25_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL25_MASK)

#define XBAR_NUM_OUT32_SEL_SEL27_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL27_SHIFT           (8U)
/*! SEL27 - SEL27 */
#define XBAR_NUM_OUT32_SEL_SEL27(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL27_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL27_MASK)

#define XBAR_NUM_OUT32_SEL_SEL29_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL29_SHIFT           (8U)
/*! SEL29 - SEL29 */
#define XBAR_NUM_OUT32_SEL_SEL29(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL29_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL29_MASK)

#define XBAR_NUM_OUT32_SEL_SEL31_MASK            (0xFF00U)
#define XBAR_NUM_OUT32_SEL_SEL31_SHIFT           (8U)
/*! SEL31 - SEL31 */
#define XBAR_NUM_OUT32_SEL_SEL31(x)              (((uint16_t)(((uint16_t)(x)) << XBAR_NUM_OUT32_SEL_SEL31_SHIFT)) & XBAR_NUM_OUT32_SEL_SEL31_MASK)
/*! @} */

/* The count of XBAR_NUM_OUT32_SEL */
#define XBAR_NUM_OUT32_SEL_COUNT                 (16U)


/*!
 * @}
 */ /* end of group XBAR_NUM_OUT32_Register_Masks */


/* XBAR_NUM_OUT32 - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral XBAR2 base address */
  #define XBAR2_BASE                               (0x52760000u)
  /** Peripheral XBAR2 base address */
  #define XBAR2_BASE_NS                            (0x42760000u)
  /** Peripheral XBAR2 base pointer */
  #define XBAR2                                    ((XBAR_NUM_OUT32_Type *)XBAR2_BASE)
  /** Peripheral XBAR2 base pointer */
  #define XBAR2_NS                                 ((XBAR_NUM_OUT32_Type *)XBAR2_BASE_NS)
  /** Peripheral XBAR3 base address */
  #define XBAR3_BASE                               (0x52770000u)
  /** Peripheral XBAR3 base address */
  #define XBAR3_BASE_NS                            (0x42770000u)
  /** Peripheral XBAR3 base pointer */
  #define XBAR3                                    ((XBAR_NUM_OUT32_Type *)XBAR3_BASE)
  /** Peripheral XBAR3 base pointer */
  #define XBAR3_NS                                 ((XBAR_NUM_OUT32_Type *)XBAR3_BASE_NS)
  /** Array initializer of XBAR_NUM_OUT32 peripheral base addresses */
  #define XBAR_NUM_OUT32_BASE_ADDRS                { XBAR2_BASE, XBAR3_BASE }
  /** Array initializer of XBAR_NUM_OUT32 peripheral base pointers */
  #define XBAR_NUM_OUT32_BASE_PTRS                 { XBAR2, XBAR3 }
  /** Array initializer of XBAR_NUM_OUT32 peripheral base addresses */
  #define XBAR_NUM_OUT32_BASE_ADDRS_NS             { XBAR2_BASE_NS, XBAR3_BASE_NS }
  /** Array initializer of XBAR_NUM_OUT32 peripheral base pointers */
  #define XBAR_NUM_OUT32_BASE_PTRS_NS              { XBAR2_NS, XBAR3_NS }
#else
  /** Peripheral XBAR2 base address */
  #define XBAR2_BASE                               (0x42760000u)
  /** Peripheral XBAR2 base pointer */
  #define XBAR2                                    ((XBAR_NUM_OUT32_Type *)XBAR2_BASE)
  /** Peripheral XBAR3 base address */
  #define XBAR3_BASE                               (0x42770000u)
  /** Peripheral XBAR3 base pointer */
  #define XBAR3                                    ((XBAR_NUM_OUT32_Type *)XBAR3_BASE)
  /** Array initializer of XBAR_NUM_OUT32 peripheral base addresses */
  #define XBAR_NUM_OUT32_BASE_ADDRS                { XBAR2_BASE, XBAR3_BASE }
  /** Array initializer of XBAR_NUM_OUT32 peripheral base pointers */
  #define XBAR_NUM_OUT32_BASE_PTRS                 { XBAR2, XBAR3 }
#endif

/*!
 * @}
 */ /* end of group XBAR_NUM_OUT32_Peripheral_Access_Layer */


/* ----------------------------------------------------------------------------
   -- XCACHE Peripheral Access Layer
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup XCACHE_Peripheral_Access_Layer XCACHE Peripheral Access Layer
 * @{
 */

/** XCACHE - Register Layout Typedef */
typedef struct {
  __IO uint32_t CCR;                               /**< Cache Control, offset: 0x0 */
  __IO uint32_t CLCR;                              /**< Cache Line Control, offset: 0x4 */
  __IO uint32_t CSAR;                              /**< Cache Search Address, offset: 0x8 */
  __IO uint32_t CCVR;                              /**< Cache Read/Write Value, offset: 0xC */
} XCACHE_Type;

/* ----------------------------------------------------------------------------
   -- XCACHE Register Masks
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup XCACHE_Register_Masks XCACHE Register Masks
 * @{
 */

/*! @name CCR - Cache Control */
/*! @{ */

#define XCACHE_CCR_ENCACHE_MASK                  (0x1U)
#define XCACHE_CCR_ENCACHE_SHIFT                 (0U)
/*! ENCACHE - Cache Enable
 *  0b0..Disable
 *  0b1..Enable
 */
#define XCACHE_CCR_ENCACHE(x)                    (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_ENCACHE_SHIFT)) & XCACHE_CCR_ENCACHE_MASK)

#define XCACHE_CCR_FRCWT_MASK                    (0x4U)
#define XCACHE_CCR_FRCWT_SHIFT                   (2U)
/*! FRCWT - Force Write Through Mode
 *  0b0..Does not force
 *  0b1..Force
 */
#define XCACHE_CCR_FRCWT(x)                      (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_FRCWT_SHIFT)) & XCACHE_CCR_FRCWT_MASK)

#define XCACHE_CCR_FRCNOALLC_MASK                (0x8U)
#define XCACHE_CCR_FRCNOALLC_SHIFT               (3U)
/*! FRCNOALLC - Forces No Allocation on Cache Misses
 *  0b0..Allocation on cache misses
 *  0b1..Forces no allocation on cache misses (must also have FRCWT asserted)
 */
#define XCACHE_CCR_FRCNOALLC(x)                  (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_FRCNOALLC_SHIFT)) & XCACHE_CCR_FRCNOALLC_MASK)

#define XCACHE_CCR_INVW0_MASK                    (0x1000000U)
#define XCACHE_CCR_INVW0_SHIFT                   (24U)
/*! INVW0 - Invalidate Way 0
 *  0b0..No operation
 *  0b1..When you write 1 to GO, invalidates all lines in way 0.
 */
#define XCACHE_CCR_INVW0(x)                      (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_INVW0_SHIFT)) & XCACHE_CCR_INVW0_MASK)

#define XCACHE_CCR_PUSHW0_MASK                   (0x2000000U)
#define XCACHE_CCR_PUSHW0_SHIFT                  (25U)
/*! PUSHW0 - Push Way 0
 *  0b0..No operation
 *  0b1..When you write 1 to GO, push all modified lines in way 0
 */
#define XCACHE_CCR_PUSHW0(x)                     (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_PUSHW0_SHIFT)) & XCACHE_CCR_PUSHW0_MASK)

#define XCACHE_CCR_INVW1_MASK                    (0x4000000U)
#define XCACHE_CCR_INVW1_SHIFT                   (26U)
/*! INVW1 - Invalidate Way 1
 *  0b0..No operation
 *  0b1..When you write 1 to GO, invalidates all lines in way 1
 */
#define XCACHE_CCR_INVW1(x)                      (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_INVW1_SHIFT)) & XCACHE_CCR_INVW1_MASK)

#define XCACHE_CCR_PUSHW1_MASK                   (0x8000000U)
#define XCACHE_CCR_PUSHW1_SHIFT                  (27U)
/*! PUSHW1 - Push Way 1
 *  0b0..No operation
 *  0b1..When you write 1 to GO, push all modified lines in way 1
 */
#define XCACHE_CCR_PUSHW1(x)                     (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_PUSHW1_SHIFT)) & XCACHE_CCR_PUSHW1_MASK)

#define XCACHE_CCR_GO_MASK                       (0x80000000U)
#define XCACHE_CCR_GO_SHIFT                      (31U)
/*! GO - Initiate Cache Command
 *  0b0..Write: no effect. Read: no cache command active
 *  0b1..Write: initiates command; Read: cache command active
 */
#define XCACHE_CCR_GO(x)                         (((uint32_t)(((uint32_t)(x)) << XCACHE_CCR_GO_SHIFT)) & XCACHE_CCR_GO_MASK)
/*! @} */

/*! @name CLCR - Cache Line Control */
/*! @{ */

#define XCACHE_CLCR_LGO_MASK                     (0x1U)
#define XCACHE_CLCR_LGO_SHIFT                    (0U)
/*! LGO - Initiate Cache Line Command
 *  0b0..Write: no effect. Read: no line command active.
 *  0b1..Write: initiate line command. Read: line command active.
 */
#define XCACHE_CLCR_LGO(x)                       (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_LGO_SHIFT)) & XCACHE_CLCR_LGO_MASK)

#define XCACHE_CLCR_CACHEADDR_MASK               (0x3FFCU)
#define XCACHE_CLCR_CACHEADDR_SHIFT              (2U)
/*! CACHEADDR - Cache Address */
#define XCACHE_CLCR_CACHEADDR(x)                 (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_CACHEADDR_SHIFT)) & XCACHE_CLCR_CACHEADDR_MASK)

#define XCACHE_CLCR_WSEL_MASK                    (0x4000U)
#define XCACHE_CLCR_WSEL_SHIFT                   (14U)
/*! WSEL - Way Select
 *  0b0..Way 0
 *  0b1..Way 1
 */
#define XCACHE_CLCR_WSEL(x)                      (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_WSEL_SHIFT)) & XCACHE_CLCR_WSEL_MASK)

#define XCACHE_CLCR_TDSEL_MASK                   (0x10000U)
#define XCACHE_CLCR_TDSEL_SHIFT                  (16U)
/*! TDSEL - Tag or Data Select
 *  0b0..Data
 *  0b1..Tag
 */
#define XCACHE_CLCR_TDSEL(x)                     (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_TDSEL_SHIFT)) & XCACHE_CLCR_TDSEL_MASK)

#define XCACHE_CLCR_LCIVB_MASK                   (0x100000U)
#define XCACHE_CLCR_LCIVB_SHIFT                  (20U)
/*! LCIVB - Line Command Initial Valid */
#define XCACHE_CLCR_LCIVB(x)                     (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_LCIVB_SHIFT)) & XCACHE_CLCR_LCIVB_MASK)

#define XCACHE_CLCR_LCIMB_MASK                   (0x200000U)
#define XCACHE_CLCR_LCIMB_SHIFT                  (21U)
/*! LCIMB - Line Command Initial Modified */
#define XCACHE_CLCR_LCIMB(x)                     (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_LCIMB_SHIFT)) & XCACHE_CLCR_LCIMB_MASK)

#define XCACHE_CLCR_LCWAY_MASK                   (0x400000U)
#define XCACHE_CLCR_LCWAY_SHIFT                  (22U)
/*! LCWAY - Line Command Way */
#define XCACHE_CLCR_LCWAY(x)                     (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_LCWAY_SHIFT)) & XCACHE_CLCR_LCWAY_MASK)

#define XCACHE_CLCR_LCMD_MASK                    (0x3000000U)
#define XCACHE_CLCR_LCMD_SHIFT                   (24U)
/*! LCMD - Line Command
 *  0b00..Search and read or write
 *  0b01..Invalidate
 *  0b10..Push
 *  0b11..Clear
 */
#define XCACHE_CLCR_LCMD(x)                      (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_LCMD_SHIFT)) & XCACHE_CLCR_LCMD_MASK)

#define XCACHE_CLCR_LADSEL_MASK                  (0x4000000U)
#define XCACHE_CLCR_LADSEL_SHIFT                 (26U)
/*! LADSEL - Line Address Select
 *  0b0..Cache address
 *  0b1..Physical address
 */
#define XCACHE_CLCR_LADSEL(x)                    (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_LADSEL_SHIFT)) & XCACHE_CLCR_LADSEL_MASK)

#define XCACHE_CLCR_LACC_MASK                    (0x8000000U)
#define XCACHE_CLCR_LACC_SHIFT                   (27U)
/*! LACC - Line Access Type
 *  0b0..Read
 *  0b1..Write
 */
#define XCACHE_CLCR_LACC(x)                      (((uint32_t)(((uint32_t)(x)) << XCACHE_CLCR_LACC_SHIFT)) & XCACHE_CLCR_LACC_MASK)
/*! @} */

/*! @name CSAR - Cache Search Address */
/*! @{ */

#define XCACHE_CSAR_LGO_MASK                     (0x1U)
#define XCACHE_CSAR_LGO_SHIFT                    (0U)
/*! LGO - Initiate Cache Line Command
 *  0b0..Write: no effect. Read: no line command active.
 *  0b1..Write: initiate line command. Read: line command active.
 */
#define XCACHE_CSAR_LGO(x)                       (((uint32_t)(((uint32_t)(x)) << XCACHE_CSAR_LGO_SHIFT)) & XCACHE_CSAR_LGO_MASK)

#define XCACHE_CSAR_PHYADDR_MASK                 (0xFFFFFFFCU)
#define XCACHE_CSAR_PHYADDR_SHIFT                (2U)
/*! PHYADDR - Physical Address */
#define XCACHE_CSAR_PHYADDR(x)                   (((uint32_t)(((uint32_t)(x)) << XCACHE_CSAR_PHYADDR_SHIFT)) & XCACHE_CSAR_PHYADDR_MASK)
/*! @} */

/*! @name CCVR - Cache Read/Write Value */
/*! @{ */

#define XCACHE_CCVR_DATA_MASK                    (0xFFFFFFFFU)
#define XCACHE_CCVR_DATA_SHIFT                   (0U)
/*! DATA - Cache Read/Write Data */
#define XCACHE_CCVR_DATA(x)                      (((uint32_t)(((uint32_t)(x)) << XCACHE_CCVR_DATA_SHIFT)) & XCACHE_CCVR_DATA_MASK)
/*! @} */


/*!
 * @}
 */ /* end of group XCACHE_Register_Masks */


/* XCACHE - Peripheral instance base addresses */
#if (defined(__ARM_FEATURE_CMSE) && (__ARM_FEATURE_CMSE & 0x2))
  /** Peripheral XCACHE_PC base address */
  #define XCACHE_PC_BASE                           (0x54400000u)
  /** Peripheral XCACHE_PC base address */
  #define XCACHE_PC_BASE_NS                        (0x44400000u)
  /** Peripheral XCACHE_PC base pointer */
  #define XCACHE_PC                                ((XCACHE_Type *)XCACHE_PC_BASE)
  /** Peripheral XCACHE_PC base pointer */
  #define XCACHE_PC_NS                             ((XCACHE_Type *)XCACHE_PC_BASE_NS)
  /** Peripheral XCACHE_PS base address */
  #define XCACHE_PS_BASE                           (0x54400800u)
  /** Peripheral XCACHE_PS base address */
  #define XCACHE_PS_BASE_NS                        (0x44400800u)
  /** Peripheral XCACHE_PS base pointer */
  #define XCACHE_PS                                ((XCACHE_Type *)XCACHE_PS_BASE)
  /** Peripheral XCACHE_PS base pointer */
  #define XCACHE_PS_NS                             ((XCACHE_Type *)XCACHE_PS_BASE_NS)
  /** Array initializer of XCACHE peripheral base addresses */
  #define XCACHE_BASE_ADDRS                        { XCACHE_PC_BASE, XCACHE_PS_BASE }
  /** Array initializer of XCACHE peripheral base pointers */
  #define XCACHE_BASE_PTRS                         { XCACHE_PC, XCACHE_PS }
  /** Array initializer of XCACHE peripheral base addresses */
  #define XCACHE_BASE_ADDRS_NS                     { XCACHE_PC_BASE_NS, XCACHE_PS_BASE_NS }
  /** Array initializer of XCACHE peripheral base pointers */
  #define XCACHE_BASE_PTRS_NS                      { XCACHE_PC_NS, XCACHE_PS_NS }
#else
  /** Peripheral XCACHE_PC base address */
  #define XCACHE_PC_BASE                           (0x44400000u)
  /** Peripheral XCACHE_PC base pointer */
  #define XCACHE_PC                                ((XCACHE_Type *)XCACHE_PC_BASE)
  /** Peripheral XCACHE_PS base address */
  #define XCACHE_PS_BASE                           (0x44400800u)
  /** Peripheral XCACHE_PS base pointer */
  #define XCACHE_PS                                ((XCACHE_Type *)XCACHE_PS_BASE)
  /** Array initializer of XCACHE peripheral base addresses */
  #define XCACHE_BASE_ADDRS                        { XCACHE_PC_BASE, XCACHE_PS_BASE }
  /** Array initializer of XCACHE peripheral base pointers */
  #define XCACHE_BASE_PTRS                         { XCACHE_PC, XCACHE_PS }
#endif
/** XCACHE physical memory base address */
#define XCACHE_PHYMEM_BASES                        { 0x00000000u, 0x20000000u }
/** XCACHE physical memory size */
#define XCACHE_PHYMEM_SIZES                        { 0x20000000u, 0xE0000000u }


/*!
 * @}
 */ /* end of group XCACHE_Peripheral_Access_Layer */


/*
** End of section using anonymous unions
*/

#if defined(__ARMCC_VERSION)
  #if (__ARMCC_VERSION >= 6010050)
    #pragma clang diagnostic pop
  #else
    #pragma pop
  #endif
#elif defined(__GNUC__)
  /* leave anonymous unions enabled */
#elif defined(__IAR_SYSTEMS_ICC__)
  #pragma language=default
#else
  #error Not supported compiler type
#endif

/*!
 * @}
 */ /* end of group Peripheral_access_layer */


/* ----------------------------------------------------------------------------
   -- Macros for use with bit field definitions (xxx_SHIFT, xxx_MASK).
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup Bit_Field_Generic_Macros Macros for use with bit field definitions (xxx_SHIFT, xxx_MASK).
 * @{
 */

#if defined(__ARMCC_VERSION)
  #if (__ARMCC_VERSION >= 6010050)
    #pragma clang system_header
  #endif
#elif defined(__IAR_SYSTEMS_ICC__)
  #pragma system_include
#endif

/**
 * @brief Mask and left-shift a bit field value for use in a register bit range.
 * @param field Name of the register bit field.
 * @param value Value of the bit field.
 * @return Masked and shifted value.
 */
#define NXP_VAL2FLD(field, value)    (((value) << (field ## _SHIFT)) & (field ## _MASK))
/**
 * @brief Mask and right-shift a register value to extract a bit field value.
 * @param field Name of the register bit field.
 * @param value Value of the register.
 * @return Masked and shifted bit field value.
 */
#define NXP_FLD2VAL(field, value)    (((value) & (field ## _MASK)) >> (field ## _SHIFT))

/*!
 * @}
 */ /* end of group Bit_Field_Generic_Macros */


/* ----------------------------------------------------------------------------
   -- SDK Compatibility
   ---------------------------------------------------------------------------- */

/*!
 * @addtogroup SDK_Compatibility_Symbols SDK Compatibility
 * @{
 */

/* No SDK compatibility issues. */

/*!
 * @}
 */ /* end of group SDK_Compatibility_Symbols */


#endif  /* MIMXRT1182_H_ */

